#!/usr/local/bin/perl
#--------------------------------------------------------------------------
# $Id: //depot/prod/test/Rsierranevada/ATI/atifbgen.pl#1 $
#
# Copyright (c) 1992-2005 Network Appliance, Inc.
# All rights reserved.
#--------------------------------------------------------------------------
#
# ATI Fallback Cache Generator
#
# AUTHOR: Will McGovern (will@netapp.com)
#
# This code uses the reflective APIs of Data ONTAP to build the fallback
# cache file used by the ATI libraries in the case where the target filer
# does not support the reflective APIs upon which ATI depends. This cache
# file contains all the information that would normally be requested from
# the ATI libraries via the reflective interfaces. This is required for
# filers running ONTAPI versions 1.4 and earlier (ONTAP 7.0.x and earlier).
#
# In actuality, this fallback cache file is just a fully populated cache
# file as normally used by ATI for persistent API/type caching. Generating
# this cache file requires fetching the API/type information from a filer
# that _does_ support the reflective APIs. Technically, the generated
# cache file represents a superset of the APIs supported by V1.4 of ONTAPI
# but is not an  issue because of the backward compatibility of ONTAPI.
#
#--------------------------------------------------------------------------
# NOTES:
#
# Running: perl atifbgen.pl [-s] [-u <user] [-p <password>] <filer>\n" .
#
#          -s            =  use secure HTTPS connection
#          -u <user>     =  filer login (default=\"root\")
#          -p <password> =  filer password (default=\"\")
#          <filer>       =  filer from which to download API information
#
# The cache file fallback_cache.ati is generated in the current directory.
#--------------------------------------------------------------------------
#
# Always require hard-assed checking
#
use strict;
use warnings;
#
# Add our parent directory to the module search path
#
use FindBin;
use lib "$FindBin::Bin/..";
#
# Import system modules
#
use Getopt::Std;
#
# Import ATI base module
#
use ATI::ati_ontapi qw(ATI_ONTAPI_TRANSPORT_HTTP
		       ATI_ONTAPI_TRANSPORT_HTTPS
		       ATI_ONTAPI_USER_DEFAULT
		       ATI_ONTAPI_PASSWORD_DEFAULT
		       ATI_ONTAPI_DEBUG_TERSE
		       ati_ontapi_create_context);

#--------------------------------------------------------------------------
#
# Start of main code
#
#--------------------------------------------------------------------------
#
# Parse the options and arguments
#
my %opts;
my $gotopts   = getopts('su:p:', \%opts);
my $transport = defined($opts{'s'}) ? ATI_ONTAPI_TRANSPORT_HTTPS : ATI_ONTAPI_TRANSPORT_HTTP;
my $user      = $opts{'u'} || ATI_ONTAPI_USER_DEFAULT;
my $password  = $opts{'p'} || ATI_ONTAPI_PASSWORD_DEFAULT;
#
# Sanity check the arguments
#
($gotopts && (@ARGV == 1)) ||
    die "Usage: atifbgen.pl [-s] [-u <user] [-p <password>] [-f <file>] <filer>\n" .
        "       -s            =  use secure HTTPS connection\n" .
        "       -u <user>     =  filer login (default=\"root\")\n" .
        "       -p <password> =  filer password (default=\"\")\n" .
        "       <filer>       =  filer from which to download API information\n" .
        "\n" .
        "The cache file fallback_cache.ati is generated in the current directory.\n";
#
# Create an ATI context for the filer connection with the appropriate
# parameters to force generation of the fallback cache file.
#
my $context;
eval { $context = ati_ontapi_create_context(filer      => $ARGV[0],
					    transport  => $transport,
					    user       => $user,
					    password   => $password,
					    cache_dir  => ".",
					    cache_file => "fallback_cache.ati",
					    cache_all  => 1,
					    debug      => ATI_ONTAPI_DEBUG_TERSE) };
($@) && die "Failed to create fallback cache file:\n\n$@\n";
#
# Cache file was generated successfully
#
print "Fallback cache file generated successfully.\n";
exit 0;
