#!/usr/local/bin/perl
#--------------------------------------------------------------------------
# $Id: //depot/prod/test/Rsierranevada/ATI/demo.pl#1 $
#
# Copyright (c) 1992-2004 Network Appliance, Inc.
# All rights reserved.
#--------------------------------------------------------------------------
#
# DRIVER: --==<< ATI MODULE DEMO CODE >>==--
#
# AUTHOR: Will McGovern (will@netapp.com)
#
# This code demonstrates the functionality of the ONTAPI API libraries.
#
#--------------------------------------------------------------------------
# NOTES:
#
# Running: perl demo.pl <filer>
#
# Currently assumes filer has 'root' user login with null password to
# make the code less verbose.
#
#--------------------------------------------------------------------------
#
# Always require hard-assed checking
#
use strict;
use warnings;
#
# Add our parent directory to the module search path
#
use FindBin;
use lib "$FindBin::Bin/..";
#
# System packages
#
use Data::Dumper;
#
# ATI base modules
#
use ATI::ati_ontapi qw(ATI_SUCCESS
		       ATI_ONTAPI_DEBUG_NONE
		       ATI_ONTAPI_DEBUG_VERBOSE
		       ati_ontapi_create_context
		       ati_ontapi_call_api);

use ATI::ati_util   qw(ATI_UTIL_AGGR_STATE_OFFLINE
		       ATI_UTIL_VOLUME_STATE_OFFLINE
		       ati_util_aggr_wait_state
		       ati_util_volume_wait_state);
#
# Demo parameters
#
if (@ARGV != 1) {
	die "Usage: perl demo.pl <filer>\n"
}
my $filer = $ARGV[0];
my @out;
#
# Create an ATI context for the filer connection with explicitly
# specified parameters (rather than taking the defaults)
#
my $context = ati_ontapi_create_context(filer   => $filer,
					timeout => 30);
#
# Get the Data ONTAP version
#
print "Getting Data ONTAP version...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'system-get-version');
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
print "Data ONTAP version = " . $out[1]->{'version'} . "\n";
#
# Get the ONTAPI SDK version
#
print "Getting API version...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'system-get-ontapi-version');
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
print "ONTAPI SDK version = $out[1]->{'major-version'}.$out[1]->{'minor-version'}\n";
#
# Get the system information
#
print "Getting system information...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'system-get-info');
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
print "System name = " . $out[1]->{'system-info'}{'system-name'} . "\n";
print "System model = " . $out[1]->{'system-info'}{'system-model'} . "\n";
print "System serial number = " . $out[1]->{'system-info'}{'system-serial-number'} . "\n";
#
# Get the list of available licenses
#
print "Getting list of available licenses...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'license-list-info');
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
foreach my $license (@{$out[1]->{licenses}}) {
	print "$license->{'service'} = $license->{'is-licensed'}\n";
}
#
# Write a test file "/vol/vol0/testfile"
#
print "Writing test file /vol/vol0/testfile ...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'file-write-file',
			   args    => ['path'   => '/vol/vol0/testfile',
				       'offset' => 0,
				       'data'   => '41424344']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Delete the test file "/vol/vol0/testfile" with tracing enabled.
#
print "Deleting test file /vol/vol0/testfile ... TRACING ENABLED\n";
$context->{debug} = ATI_ONTAPI_DEBUG_VERBOSE;
@out = ati_ontapi_call_api(context => $context,
			   api     => 'file-delete-file',
			   args    => ['path' => '/vol/vol0/testfile']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
$context->{debug} = ATI_ONTAPI_DEBUG_NONE;
#
# Create a snapshot
#
print "Creating snapshot snappy_666 on volume vol0...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'snapshot-create',
			   args    => ['volume'   => 'vol0',
				       'snapshot' => 'snappy_666']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Delete the snapshot
#
print "Deleting snapshot snappy_666 on volume vol0...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'snapshot-delete',
			   args    => ['volume'   => 'vol0',
				       'snapshot' => 'snappy_666']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Asynchronously create a new 3-disk aggregate
#
print "Creating aggregate aggy_666 ...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'aggr-create',
			   args    => ['aggregate'  => 'aggy_666',
				       'disk-count' => '3']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Wait for aggregate to come online
#
print "Waiting for aggregate to come online...\n";
@out = ati_util_aggr_wait_state(context   => $context,
				aggregate => 'aggy_666');
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Asynchronously create a new 20MB flexible volume
#
print "Creating 20MB flexible volume volly_666 on aggregate aggy_666...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'volume-create',
			   args    => ['volume'               => 'volly_666',
				       'containing-aggr-name' => 'aggy_666',
				       'size'                 => '20m']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Wait for volume to come online
#
print "Waiting for flexible volume volly_666 to come online...\n";
@out = ati_util_volume_wait_state(context => $context,
				  volume  => 'volly_666');
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Get flexible volume info and dump to stdout
#
print "Getting volume info for flexible volume...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'volume-list-info',
			   args    => ['volume'  => 'volly_666',
				       'verbose' => 'true']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
print Dumper($out[1]);
#
# ati_volume.pm: Offline the flexible volume
#
print "Offlining flexible volume volly_666...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'volume-offline',
			   args    => ['name' => 'volly_666']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Wait for the flexible volume to go offline with 30 second timeout
#
print "Waiting for flexible volume volly_666 to go offline...\n";
@out =  ati_util_volume_wait_state(context => $context,
				   volume  => 'volly_666',
				   state   => ATI_UTIL_VOLUME_STATE_OFFLINE,
				   timeout => 30);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Destroy the flexible volume
#
print "Destroying flexible volume volly_666...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'volume-destroy',
			   args    => ['name' => 'volly_666']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Offline the aggregate
#
print "Offlining aggregate aggy_666...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'aggr-offline',
			   args    => ['aggregate' => 'aggy_666']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Wait for aggregate to go offline with 30 second timeout
#
print "Waiting for aggregate aggy_666 to go offline...\n";
@out = ati_util_aggr_wait_state(context   => $context,
				aggregate => 'aggy_666',
				state     => ATI_UTIL_AGGR_STATE_OFFLINE,
				timeout   => 30);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
#
# Destroy the aggregate
#
print "Destroying aggregate aggy_666...\n";
@out = ati_ontapi_call_api(context => $context,
			   api     => 'aggr-destroy',
			   args    => ['aggregate' => 'aggy_666']);
($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";

print "All done!!\n";
exit 0;
