# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/FSR/DR/DRQTutils.pm#1 $

## @summary Dump/Restore QTree utility library
##
## @description
##	This package provide a common set of library routines
##	useful for the Dump/Restore QTree tests. These are not
##	general purpose routines, they exist to serve DRQT tests.
##
## @dependencies
##	Tharn,
##	QtreeCmds
##
## @bugs
##
## @author marlon@netapp.com
## @status public

package FSR::DR::DRQTutils;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(hasThreeQtrees hasNoQtrees
		 isMixedQtree isNtfsQtree isUnixQtree);

use Tharn;
use QtreeCmds;
use strict;


## @name ::hasThreeQtrees
## @usage hasThreeQtrees($qt_list, $vol_name, $unix_qt, $mixed_qt, $ntfs_qt)
## @synopsis
##  Does $vol_name have the 3 qtrees and are they the correct security style.
##  Return success (1) if it does, die() otherwise.
sub hasThreeQtrees ($$$$$) {
    my ($qtlist, $vol_name, $unix_qt, $mixed_qt, $ntfs_qt) = @_ ;

    my $errstring = "";

    return eval {
        if (! ($qtlist =~ /$vol_name\s+$mixed_qt\s+mixed/))  {
            $errstring = "$vol_name $mixed_qt is not a mixed qtree ";
        }
        if (! ($qtlist =~ /$vol_name\s+$ntfs_qt\s+ntfs/))  {
            $errstring = "$errstring" . "$vol_name $ntfs_qt is not a ntfs qtree ";
        }
        if (! ($qtlist =~ /$vol_name\s+$unix_qt\s+unix/))  {
            $errstring = "$errstring" . "$vol_name $unix_qt is not a unix qtree";
        }

        if ($errstring ne "") { die "$errstring"; }
	return 1;
    }
}

## @name ::hasNoQtrees
## @usage hasNoQtrees($qt_list, $vol_name, $unix_qt, $mixed_qt, $ntfs_qt)
## @synopsis
##  If none of the 3 "qtrees" are actually qtrees then return success (1). If
##  any are qtrees then die().
sub hasNoQtrees ($$$$$) {
    my ($qtlist, $vol_name, $unix_qt, $mixed_qt, $ntfs_qt) = @_ ;

    my $errstring = "";

    return eval {
        if ($qtlist =~ /$vol_name\s+$mixed_qt\s+/)  {
            $errstring = ":$vol_name $mixed_qt is a qtree ";
        }
        if ($qtlist =~ /$vol_name\s+$ntfs_qt\s+ntfs/)  {
            $errstring = "$errstring" . ":$vol_name $ntfs_qt is a qtree ";
        }
        if ($qtlist =~ /$vol_name\s+$unix_qt\s+unix/)  {
            $errstring = "$errstring" . ":$vol_name $unix_qt is a qtree";
        }

        if ($errstring ne "") { die "$errstring"; }
	return 1;
    }
}

## @name ::isMixedQtree
## @usage isMixedQtree($connetion, $qtree_path)
## @synopsis
##    Returns whether or not the given qtree has a mixed security style.
##    On failure, returns undef with an error message in $@.
sub isMixedQtree ($$) {
  my ($cf, $qtpath) = @_;

  return eval {
    my $secstyle = qtreeSecurity($cf, $qtpath);
    die $@ if ($@);
    return ($secstyle eq "mixed");
  };
}

## @name ::isNtfsQtree
## @usage isNtfsQtree($connetion, $qtree_path)
## @synopsis
##    Returns whether or not the given qtree has a NTFS security style.
##    On failure, returns undef with an error message in $@.
sub isNtfsQtree ($$) {
  my ($cf, $qtpath) = @_;

  return eval {
    my $secstyle = qtreeSecurity($cf, $qtpath);
    die $@ if ($@);
    return ($secstyle eq "ntfs");
  };
}

## @name ::isUnixQtree
## @usage isUnixQtree($connetion, $qtree_path)
## @synopsis
##    Returns whether or not the given qtree has a unix security style.
##    On failure, returns undef with an error message in $@.
sub isUnixQtree ($$) {
  my ($cf, $qtpath) = @_;

  return eval {
    my $secstyle = qtreeSecurity($cf, $qtpath);
    die $@ if ($@);
    return ($secstyle eq "unix");
  };
}
