#-*-perl-*-
##
## @author erickson@netapp.com
##
# $Id: //depot/prod/test/Rsierranevada/SharedTestBed/NicConfigTypes.pm#1 $
use strict;
use warnings;


package SharedTestBed::NicConfigTypes;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw($NIC_INFO init_nic_data filer_eth_configs);

our $NIC_INFO;

sub init_nic_data ( $$$ ) {
    my ($NIC_NAME, $NIC_ID, $NIC_TYPE) = @_;

    while (<DATA>) {
	chomp;
	my ($id, $name, $type) = split(/:/, $_);
	$$NIC_NAME{$id} = $name;
	$$NIC_ID{$name} = $id;
	$$NIC_TYPE{$name} = $type;
	$$NIC_TYPE{$id} = $type;
    }

    return ($NIC_NAME, $NIC_ID, $NIC_TYPE);
}

sub filer_eth_configs ( @ ) {
    chomp( my @sysconfig_out = split('\n',"$_[0]"));

    # Use keys of this hash to only report unique configs, even if the same NIC is present multiple times
    my %ethernet_configs;
    my $found_config_type = '';

    foreach my $sysline (@sysconfig_out) {

        ## Find the slot line and look for ethernet NICs
	if( $sysline =~ /slot \d+: (.*[^\s])\s*$/ ) {
	    my $nic_name = $1;
            $found_config_type = ''; # We are starting a new slot

	    if ( my $id = $$NIC_INFO{NIC_ID}{$nic_name} ) {
		if ( $$NIC_INFO{NIC_TYPE}{$id} =~ /^10/ ) {
                    $found_config_type = $id;
                    if ($found_config_type !~ m/\$/) {
                        $ethernet_configs{$found_config_type} = 1;
                    }
		}
	    } 
	} elsif ($sysline =~/^\s*Remote LAN Module/) {
            $found_config_type = ''; # We are starting a new slot
        }
        if ($found_config_type) {
            # Look for additional config type information for this NIC
            #>                 Device Type:        CT-B-1    Version: T2-SRAM1F-BR00-DR2
            if( $sysline =~ /Device Type:\s*(\S+)/ ) {
                my $devicetype = $1;
                # Make sure invalid config type characters are excluded
                $devicetype =~ s![_\/]!!g;
                if ($found_config_type =~ s/\$devicetype/$devicetype/) {
                    $found_config_type = lc($found_config_type);
                    $ethernet_configs{$found_config_type} = 1;
                } else {
                    # Add a key that includes the device type information
                    $ethernet_configs{"$found_config_type-dt-".lc($devicetype)} = 1;
                }
                
                if( $sysline =~ /Device Type:\s*\S+\s+Version:\s*(\S+)/ ) {
                    my $version = $1;
                    # Make sure invalid config type characters are excluded
                    $version     =~ s![_\/]!!g;

                    if ($found_config_type !~ m/\$/) {
                        # Add a key that includes the version information
                        $ethernet_configs{"$found_config_type-v-".lc($version)} = 1;
                    }
                }
            }
        }
    }

    if (wantarray) {
	return (keys %ethernet_configs);
    } else {
	return join ("_", (keys %ethernet_configs));
    }
}

sub filer_atm_configs ( @ ) {
    chomp( my @sysconfig_out = split('\n',"$_[0]"));

    my @atm_configs;

    foreach my $sysline (@sysconfig_out) {

        my %seen_configs;  # so that we only report unique configs, even if the same NIC is present multiple times

        ## Find the slot line and look for atm NICs
	if( $sysline =~ /slot \d+: (.*[^\s])\s*$/ ) {
	    my $nic_name = $1;

	    if ( my $id = $$NIC_INFO{NIC_ID}{$nic_name} ) {
		if ( $$NIC_INFO{NIC_TYPE}{$id} =~ /^ATM/ ) {
		    push (@atm_configs, "$id") unless $seen_configs{$id}++;
		}
	    } 
	}
    }

    if (wantarray) {
	return @atm_configs;
    } else {
	return join ("_", @atm_configs);
    }
}

(@$NIC_INFO{'NIC_NAME', 'NIC_ID', 'NIC_TYPE'}) = init_nic_data({},{},{});

# config type: Slot description : NIC Type
__DATA__
f-eth-x1001c:FastEnet-10/100 Ethernet Controller:10/100Base-T:
f-eth-x1005a:TOE-10G Ethernet Controller:10000Base-SX:RoHS
f-eth-x1006a:Quad TOE-1G Ethernet Controller (T204E):10/100/1000Base-T:RoHS
f-eth-x1007a:Quad TOE-1G Ethernet Controller (T204V):10/100/1000Base-T:RoHS
f-eth-x1008a:Dual TOE-10G Ethernet Controller (T320E-XFP):10000Base-SX:RoHS
f-eth-x1010a:Dual TOE-10G Ethernet Controller (T320E-CX4):10000Base-SX:RoHS
f-eth-x1012b:10BaseT/100BaseTX Quad-Ethernet Controller:10/100Base-T:
f-eth-x1012c:Quad FastEnet-10/100 Ethernet Controller:10/100Base-T:
f-eth-x1025a:Gigabit Ethernet Controller:1000Base-SX:
f-eth-x1025d:Gigabit Ethernet Controller IV:1000Base-SX:
f-eth-x1025e:Gigabit Ethernet Controller VI:1000Base-SX:
f-eth-x1026a:Gigabit Ethernet Controller II:1000Base-SX:
f-eth-x1026b:Gigabit Ethernet Controller III:1000Base-SX:
f-eth-x1027:100/1000 Ethernet Controller III:100/1000Base-T:
f-eth-x1027b:10/100/1000 Ethernet Controller IV:10/100/1000Base-T:
f-eth-x1027c:10/100/1000 Ethernet Controller VI:10/100/1000Base-T:
f-eth-x1029a:iSCSI Target Host Adapter:10/100/1000Base-T:
f-eth-x1035:Dual Gigabit Ethernet Controller V:1000Base-SX:
f-eth-x1035b:Dual Gigabit Ethernet Controller VI:1000Base-SX:RoHS
f-eth-x1037:Dual 10/100/1000 Ethernet Controller V:10/100/1000Base-T:
f-eth-x1037b:Dual 10/100/1000 Ethernet Controller VI:10/100/1000Base-T:
f-eth-x1037c:Dual 10/100/1000 Ethernet Controller VII:10/100/1000Base-T:RoHS
f-eth-x1038a:Dual Gigabit Ethernet Controller G20:1000Base-SX:RoHS
f-eth-x1039a:Dual 10/100/1000 Ethernet Controller G20:10/100/1000Base-T:RoHS
f-eth-x1047:Quad 10/100/1000 Ethernet Controller V:10/100/1000Base-T:
f-eth-x1047b:Quad 10/100/1000 Ethernet Controller VII:10/100/1000Base-T:RoHS
f-eth-x1049a:Quad 10/100/1000 Ethernet Controller G20:10/100/1000Base-T:RoHS
f-eth-x1201b:ATM55 Mb UTP:ATM 155 Mb UTP:
f-eth-x1202:ATM Network Adapter UTP:ATM 155 Mb UTP:
f-eth-x1203b:ATM HE Network Adapter:ATM OC-12:
f-eth-mb-dec21143:Ethernet Controller e0:10/100Base-T:
f-eth-mb-bcm1250:Dual SB1250-Gigabit Ethernet Controller:10/100/1000Base-T:
f-eth-mb-intel82544gc:10/100 Ethernet Controller IV:10/100Base-T:
f-eth-mb-$devicetype:Internal BGE 10/100 Ethernet Controller:10/100Base-T:RoHS
f-eth-mb-bcm5714:BGE 10/100/1000 Ethernet Controller:10/100/1000Base-T:RoHS
