#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/SharedTestBed/StorConfigTypes.pm#1 $
#
#Copyright (c) 2006 Network Appliance, Inc.
#All rights reserved.
#
## @summary Definition of storage configuration types

## @description
## This package is meant to define all storage-specific shared test bed configuration types 
## This package is used by NATE scripts and by web CGI scripts
## as found in the current ontap main branch.
## This package does not need to include the Tharn package.
## To keep compatibility with all recent ONTAP releases,
## it is best to only introduce new configuration types.
## Removal of existing types may impact test plans that
## include those values.
## @usage
##   use StorConfigTypes ;
##
## @author Charlie Hedstrom (hedstrom@netapp.com),erickson@netapp.com
## @keywords configuration type 
## @status testme
## @change 2006-04-04 erickson: create from ConfigTypes.pm
 
use strict;

package SharedTestBed::StorConfigTypes;

use vars qw(@ISA @EXPORT
   );

require Exporter;
@ISA = ('Exporter');

@EXPORT = qw(
   get_adapter_config
   get_loop_config
   get_shelf_config
   get_drive_config
   add_inferred_configs
   config_match
   );

use English;
use Carp;
use SharedTestBed::ConfigTypes qw(testbed_config_types);

our $debug = 1;

sub get_adapter_config (@) {
    my @fc_hba_descr = @_;

    if (! grep(/(FC|Fibre Channel)\s+Host\s+Adapter\s+.*(UP|ONLINE)/,@fc_hba_descr)) {
        warn ("A request has been made to get_adapter_config() with an unknown or offline FC adapter: @fc_hba_descr\n");
        return "";
    }

    foreach my $fc_hba_line (@fc_hba_descr){
        if ($fc_hba_line =~ m/FC Host Adapter \S+ \(Dual-channel, QLogic 2342 |FC Host Adapter \S+ \(Dual-channel, QLogic 2312 \(2342|FC Host Adapter \S+ \(Dual-channel, QLogic 2312 rev. / ) {
            return "fcha-ql2342";
        } elsif ($fc_hba_line =~ m/ FC Host Adapter \S+ \(Dual-channel, QLogic 2322 / ) {
            return "fcha-ql2322";
        } elsif ($fc_hba_line =~ m/ Host Adapter \S+ \(Dual-channel, QLogic 2212 / ) {
            return "fcha-ql2212";
        } elsif ($fc_hba_line =~ m/ Host Adapter \S+ \(Dual-channel, QLogic 2200A | Host Adapter \S+ \(QLogic 2200 / ) {
            # Lines may start with 'Fibre Channel' or 'FC'
            return "fcha-ql2200";
        } elsif ($fc_hba_line =~ m/FC Host Adapter \S+ \(QLogic 2100 / ) {
            return "fcha-ql2100";
        } elsif ($fc_hba_line =~ m/FC Host Adapter \S+ \(\S+, LSI Logic 929X,|FC Host Adapter \S+ \(LSI Logic 929X / ) {
            # Second text string is the newer version as of 2004-02-04
            return "fcha-lsi929x";
        } elsif ($fc_hba_line =~ m/Board name:\s+(\S+)/i ) {
            return "fcha-".lc($1);
        }
    }
    warn ("A request has been made to get_adapter_config() with an unknown FC adapter: @fc_hba_descr\n");
    return "";
}


sub get_loop_config (@) {
    shift; shift; # skip the two header lines
    my @storage_show_disk_path_lines= @_;
    my $storage_show_disk_path = join ("\n",@storage_show_disk_path_lines);

    my  $loop_config = "";
    foreach my $storage_show (@storage_show_disk_path_lines){
        if ($storage_show =~ /[\w:\.]+\s+[\w]+\s+[\w:\.]+\s+[\w]+\s+[\d]+\s+[\d]+/){
            if ("$loop_config" eq "dl-single"){
                warn "CONFIG error: filer has a mix of dual and single pathed drives:\n$storage_show_disk_path\n";
            } else {
                $loop_config = "dl-dual";
            }
        } elsif ($storage_show =~ /[\w:\.]+\s+[\w]+\s+[\d]+\s+[\d]+/){
            if ("$loop_config" eq "dl-dual"){
                warn "CONFIG error: filer has a mix of dual and single pathed drives:\n$storage_show_disk_path\n";
            } else {
                $loop_config = "dl-single";
            }
        } else {
            warn "CONFIG error: storage show disk -p is missing information:\n$storage_show_disk_path\n";
        }
    }
    return $loop_config;
}

sub get_shelf_config ($@) {
    # Determine drive shelf type(s)
    #                Shelf 1: LRC  Firmware rev. LRC A: 11  LRC B: 11
    # Shelf 4: ESH4  Firmware rev. ESH A: 03  ESH B: 03
    my $shelf_id = shift;
    my $link_rate = shift;
    my @environ_shelf = @_;

    my $shelf_type = '';
    my $module_type = '';
    foreach my $environ (@environ_shelf){
        if ($environ =~ m/product identification/){
            (undef,$shelf_type,undef) = split('=',"$environ");
        } elsif ($environ =~ m/active/){
            # Rscrimshaw_030919_2300
            # Module type: LRC; monitoring is active:
            $environ =~ s/Module type: //; # drop field name if it is there
            $environ =~ s/;//;             # drop semicolon at end of module type value
            ($module_type,undef) = split(' ',"$environ");
        }
        if ($module_type ne '' and $shelf_type ne '') {
            if ($shelf_type=~/DS14-Mk2-FC/){
                $shelf_type="ds14mk2-fc";
            } elsif ($shelf_type=~/DiskShelf14/){
                $shelf_type="ds14";
            } elsif ($shelf_type=~/DS14-Mk2-AT/){
                $shelf_type="ds14mk2-at";
            } elsif ($shelf_type=~/XL500/){
                return "ds-vem";
            } elsif ($shelf_type =~/Eurologic EDM/){
                return "ds-edm";
            } elsif ( $shelf_type=~/R1XXShelf'/){
                return "ds-r1xx";
            } else {
                warn "unknown module type: $module_type";
                return "";
            }

            if ($module_type=~/LRC|EFH/){
                $module_type="lrc";
            } elsif ($module_type=~/ESH(\d*)/){
                $module_type="esh$1";
            } elsif ($module_type=~/AT-FCX/){
                $module_type="fcx";
            } elsif ($module_type=~/AT-FC2/){
                $module_type="scm2";
            } elsif ($module_type=~/AT-FC/){
                $module_type="scm";
            } else {
                warn "unknown module type: $module_type";
                return "";
            }

            return "ds-$shelf_type-$module_type-${link_rate}g";
        }
    }
    warn "module type or shelf type not defined:\n", join("\n",@environ_shelf), "\n";
    return "";

}

sub get_drive_config ($) {
    my $sysconfig_drive_line = shift;
    my $X_config_type;
    if ($sysconfig_drive_line =~ m/\d+\s*:\s+\S+\s+(\S+)\s+(\S+)\s+(\S+)/) {
        my $pn = $1;
        my $drive_fw = lc($2);
        my $drive_size = $3;
        # Convert part number and drive name to configuration type
        $pn =~ s/_/-/g;

        if ($pn =~ m/^(X\d\d\d)/) {
            $X_config_type = $1;
        }

        # Drop insignificant decimal digits
        $drive_size =~ s/(\.0)?GB//;
        # Convert fractional TB digits to GB units
        $drive_size =~ s/\.(\d\d\d)TB/$1/;
        $drive_size =~ s/\.(\d\d)TB/0/;
        $drive_size =~ s/\.(\d)TB/00/;
        $drive_size =~ s/\.?TB/000/;

        if ($pn =~ m/^X\d+-S10K7/) {
            return "dd-cheetah7-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-S15K3/) {
            return "dd-cheetah15k3-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-S15K4/) {
            return "dd-cheetah15k4-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-SCHT6/) {
            return "dd-cheetah6-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-SCHT5/) {
            return "dd-cheetah5-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-ST\d+04FC/) {
            return "dd-cheetah4-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/ST39103FC|ST318203FC|ST136403FC/) {
            return "dd-cheetah3-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-HJURD/) {
            return "dd-jura-d-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-HJURE/) {
            return "dd-jura-e-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-HPYTA/) {
            return "dd-hpyta-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-MCALY/) {
            return "dd-maxcaly-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-MSABR/) {
            return "dd-maxsabr-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-MTOM/) {
            return "dd-maxtom-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-MGRIZ/) {
            return "dd-maxgriz-$drive_size-$drive_fw $X_config_type";
	} elsif ($pn =~ m/^X\d+-HKURO/) {
	    return "dd-hkuro-$drive_size-$drive_fw $X_config_type";
        } elsif ($pn =~ m/^X\d+-STONK/) {
	    return "dd-stonk-$drive_size-$drive_fw $X_config_type";
        }
    }
    warn "CONFIG: drive type not defined in StorConfigTypes.pm: $sysconfig_drive_line";
    return "";

}

sub add_inferred_configs($){
    my $act_configs= shift;
    if ($act_configs){
        $act_configs =  join (" ", testbed_config_types(split (" ",$act_configs)));
        print "DEBUG: inferred_configs in add_inferred_configs: $act_configs\n" if $debug;
    }
    return $act_configs;
}


sub config_match ($$) {
    my $host_configs = shift;
    my $plan_configs = shift;

    print "DEBUG: in config_match host_configs = $host_configs\n" if $debug;
    print "DEBUG: in config_match plan_configs = $plan_configs\n" if $debug;
    if (! $plan_configs) {
        return 1;
    } elsif (! $host_configs){
        return 0;
    } else {
        my @host_plan_array = split (/\s+/, $plan_configs);

        foreach my $plan_conf (@host_plan_array){
            if ($host_configs =~ /$plan_conf/){
                print "DEBUG: in config_match host_configs matches $plan_conf\n" if $debug;
                return 1;
            } else {
                print "DEBUG: in config_match host_configs does not match $plan_conf\n" if $debug;
            }
        }
        return 0;
    }
}

1;
