
use Utilities;
use TestObject;
use NACL::Transit;

my @node_transit_objects;

########### SWITCHOVER ############

sub pre_op_validation_SO {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];
    my $node_a2 = $testObject->{args}->{nodes}->[1];

    my $current_DRC_state = &get_DRC_FSM_state($node_a1);

    if (($current_DRC_state != 'normal') &&
        ($current_DRC_state != 'switchover-partial')) {
        die "Invalid DRC state: ($current_DRC_state)\n";
    }

    my $current_DRSOM_state_a1 = &get_DRSOM_FSM_state($node_a1);
    my $current_DRSOM_state_a2 = &get_DRSOM_FSM_state($node_a2);

    if (($current_DRSOM_state_a1->{dr_pri} != 'Normal') ||
        ($current_DRSOM_state_a2->{dr_pri} != 'Normal')) {
        die "Invalid DRSOM state (A1, A2) => ($current_DRSOM_state_a1->{dr_pri}, $current_DRSOM_state_a2->{dr_pri})\n";
    }
}


sub pre_op_setup_SO {
    my $testObject = shift;

    my $node_name_b1 = $testObject->{args}->{nodes}->[2]->name;
    my $node_name_b2 = $testObject->{args}->{nodes}->[3]->name;

    my $transit_obj_b1 = NACL::Transit->new(name => $node_name_b1);
    my $transit_obj_b2 = NACL::Transit->new(name => $node_name_b2);

    push(@node_transit_objects, $transit_obj_b1);
    push(@node_transit_objects, $transit_obj_b2);

    #parallelize this
    $transit_obj_b1->change_state(to => 'FIRMWARE', timeout => 2000);
    $transit_obj_b2->change_state(to => 'FIRMWARE', timeout => 2000);

    sleep(60);
}

sub run_SO {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];

    $node_a1->apiset(set => 'CMode')->execute_raw_command(command => 'metrocluster switchover -forced-on-disaster true',
                                                          timeout => 120);

    sleep(60);
}

sub verify_results_SO {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];
    my $node_a2 = $testObject->{args}->{nodes}->[1];

    my $current_DRC_state = &get_DRC_FSM_state($node_a1);

    if ($current_DRC_state ne 'switchover-completed') {
        die "Invalid DRC state ($current_DRC_state)\n";
    }

    my $current_DRSOM_state_a1 = get_DRSOM_FSM_state($node_a1);
    my $current_DRSOM_state_a2 = get_DRSOM_FSM_state($node_a2);

    if (($current_DRSOM_state_a1->{dr_pri} ne 'Switchover completed') ||
        ($current_DRSOM_state_a2->{dr_pri} ne 'Switchover completed')) {
        die "Invalid DRSOM state (A1, A2) => ($current_DRSOM_state_a1->{dr_pri}, $current_DRSOM_state_a2->{dr_pri})\n";
    }
}

########### HEAL AGGRS ############


sub pre_op_validation_HA {
    my $testObject = shift;
    &verify_results_SO($testObject);
}

sub run_HA {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];

    $node_a1->apiset(set => 'CMode')->execute_raw_command(command => 'metrocluster heal -phase aggregates');

    sleep(60);
}

sub verify_results_HA {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];
    my $node_a2 = $testObject->{args}->{nodes}->[1];

    my $current_DRC_state = &get_DRC_FSM_state($node_a1);

    if ($current_DRC_state ne 'healing-aggrs-completed') {
        die "Invalid DRC state ($current_DRC_state)\n";
    }

    my $current_DRSOM_state_a1 = get_DRSOM_FSM_state($node_a1);
    my $current_DRSOM_state_a2 = get_DRSOM_FSM_state($node_a2);

    if (($current_DRSOM_state_a1->{dr_pri} ne 'Aggr healing completed') ||
        ($current_DRSOM_state_a2->{dr_pri} ne 'Aggr healing completed')) {
        die "Invalid DRSOM state (A1, A2) => ($current_DRSOM_state_a1->{dr_pri}, $current_DRSOM_state_a2->{dr_pri})\n";
    }
}

########### HEAL ROOTS ############


sub pre_op_validation_HR {
    my $testObject = shift;
    &verify_results_HA($testObject);
}

sub run_HR {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];

    $node_a1->apiset(set => 'CMode')->execute_raw_command(command => 'metrocluster heal -phase root-aggregates');

    sleep(60);
}

sub verify_results_HR {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];
    my $node_a2 = $testObject->{args}->{nodes}->[1];

    my $current_DRC_state = &get_DRC_FSM_state($node_a1);

    if ($current_DRC_state ne 'healing-roots-completed') {
        die "Invalid DRC state ($current_DRC_state)\n";
    }

    my $current_DRSOM_state_a1 = get_DRSOM_FSM_state($node_a1);
    my $current_DRSOM_state_a2 = get_DRSOM_FSM_state($node_a2);

    if (($current_DRSOM_state_a1->{dr_pri} ne 'Root aggr switchback completed') ||
        ($current_DRSOM_state_a2->{dr_pri} ne 'Root aggr switchback completed')) {
        die "Invalid DRSOM state (A1, A2) => ($current_DRSOM_state_a1->{dr_pri}, $current_DRSOM_state_a2->{dr_pri})\n";
    }
}

########### SWITCHBACK ############

sub pre_op_validation_SB {
    my $testObject = shift;
    &verify_results_HR($testObject);
}

sub pre_op_setup_SB {
    my $testObject = shift;

    my $node_name_b1 = $testObject->{args}->{nodes}->[2]->name;
    my $node_name_b2 = $testObject->{args}->{nodes}->[3]->name;

    my $transit_obj_b1 = @node_transit_objects[0];
    my $transit_obj_b2 = @node_transit_objects[1];

    #parallelize this
    $transit_obj_b1->change_state(to => 'CLI', timeout => 1500);
    $transit_obj_b2->change_state(to => 'CLI', timeout => 1500);

    sleep(60);
}

sub run_SB {
    my $testObject = shift;

    my $node_a1 = $testObject->{args}->{nodes}->[0];

    $node_a1->apiset(set => 'CMode')->execute_raw_command(command => 'metrocluster switchback',
                                                          timeout => 120);

    sleep(60);

}

sub verify_results_SB {
    my $testObject = shift;
    &pre_op_validation_SO($testObject);
}
1;
