#  $Id: //depot/prod/test/Rsierranevada/dp/snapdiff/zapi/lib/SDutils.pm#1 $

package SDutils;

use strict;
use Encode qw(decode);
use Carp;
use Params::Validate qw(:all);
use Tharn;
use dp::snapdiff::zapi::lib::ZapiSD;
use Data::Dumper;
use NACL::C::Node;
use NACL::C::Client;
use NACL::STask::Client::Mount;
use NACL::STask::Volume;
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

BEGIN {
    use Exporter;
    use vars qw(@ISA @EXPORT_OK);
    @ISA = 'Exporter';
    @EXPORT_OK = qw(check datagen do_mount);

}

my $verbose=0;
my $Node;
my ($Mount_base, $Mount_diff);
my ($Client, $client_name);


# Check snapdiff output and actual filesystem check
# Returns 0 Pass
# Returns 1 Fail
sub check {
    my ($s) = @_;

    # Flag to ignore checking snapdiff output with and actal file system.
    # Default is false. This flag is set for those tests 
    # with huget dataset like stress/scalability/performance etc.

    if (defined($::SNAPDIFF_VERIFY) 
         and ($::SNAPDIFF_VERIFY==0)) {
        $Log->comment("Not comparing snapdiff output and filesystem diff");
        return 0;
    }

    # Final result of the snapdiff and actal file system diff
    my $result = 0;

    $Log->comment("In Filesystem check\n");
    $Log->comment(' Filer: ' . $s->{filer} .
                  ' Volume: ' . $s->{volume} .
                  ' Base Snapshot: ' . $s->{base_snapshot} .
                  ' Diff Snapshot:  ' . $s->{diff_snapshot});

    $verbose=$::DBG if ($::DBG ne '');

    my ($sd_deleted_files, $sd_new_updated_files);
    my ($fs_deleted_files, $fs_new_updated_files);

    $Log->comment('Before calling get all diffs') if ($verbose);
    my $h = $s->get_diffs_hash();
    $Log->comment('Get all diff finished with error') if ($verbose);
    #print Dumper ($h) if ($verbose);

    # Get snapdiff ouput into two lists (deleted, new/updated)
    ($sd_deleted_files, $sd_new_updated_files) = add_to_file_list($s, $h);

    print "\nSnapdiff output:\n";
    print 'Number of Deleted file : ', scalar(@$sd_deleted_files), "\n";
    print 'Number of New/Updated file list: ',
                scalar(@$sd_new_updated_files), "\n\n";

    if ($verbose) {
        print "Deleted file list: \n";
        print join "\n", @$sd_deleted_files, "\n"
                if (defined($sd_deleted_files));
        print "New/Modified file list: \n";
        print join "\n", @$sd_new_updated_files, "\n"
                if (defined($sd_new_updated_files));
    }


    # Get file system diff into two lists (deleted, new/updated)

    ($fs_deleted_files, $fs_new_updated_files) =
                        get_filesystem_changes($s, $h);

    # As snapdiff output will be unix path style,
    # For comparing with Windows Client convert from 
    # unix-path to windows-path 
    if ( ($s->{file_access_protocol} =~ /cifs/) 
         || ($::FILE_ACCESS_PROTOCOL =~ /cifs/) ) {
        if ($Client->hostrec()->isa('Hostrec::Windows')) {
            _convert_path($sd_deleted_files);
            _convert_path($sd_new_updated_files);
        }
    }

    # Decode file name to UTF-8 for comparing with snapdiff output.
    if ( ($s->{file_access_protocol} =~ /cifs|nfsv4/)
         || ($::FILE_ACCESS_PROTOCOL =~ /cifs|nfsv4/) ) {
        $Log->comment("decode fs output to utf8");
        _decode_utf($fs_deleted_files);
        _decode_utf($fs_new_updated_files);
    }

    print "\nFilesystem compare output :\n";
    print 'Number of Deleted file : ',
                scalar(@$fs_deleted_files), "\n";
    print 'Number of New/Updated file list: ',
                scalar(@$fs_new_updated_files), "\n\n";

    if ($verbose) {
        print "Deleted file list: \n";
        print join "\n", @$fs_deleted_files, "\n"
                    if (defined($fs_deleted_files));

        print "New/Modified file list: \n";
        print join "\n", @$fs_new_updated_files, "\n"
                    if (defined(@$fs_new_updated_files));
    }

    # Handle dir rename or dir delete :
    #
    # As when dir deleted snapdiff output will
    # will have only dirname as deleted, not the files in that dir
    # but Filesystem compare returns all files under that dir,
    # and in case of rename all files under new dir as new files.
    # To handle this difference, if same dir is listed as deleted and created
    # with same inode number, (i.e dir rename)
    # exclude files/dirs under this dir from filesystem output

    my $dir_del_hash = get_dirs_deleted($h);

    while ( my ($dir_deleted, $dir_renamed_as) = each (%$dir_del_hash) ) {

        print "Dir renamed : $dir_deleted to $dir_renamed_as \n"
                                                if ($dir_renamed_as);
        print "Dir deleted : $dir_deleted \n" if (!$dir_renamed_as);

        print 'Excluding files/dirs under this dirs from '.
                                "Filesystem diff output\n";

        # exclude all files under deleted dir

        @$fs_deleted_files  = grep {$_ !~ /^$dir_deleted\// }
                                        @$fs_deleted_files;
        # exclude files or subdirs under deleted dirs 
        @$sd_deleted_files  = grep {$_ !~ /^$dir_deleted\// }
                                        @$sd_deleted_files;

        print 'Size of FSystem deleted_files :',
                scalar(@$fs_deleted_files) if ($verbose);

        # exclude files as created under new file list for rename
        @$fs_new_updated_files =
                grep {$_ !~ /^$dir_renamed_as\// } @$fs_new_updated_files
                                if ($dir_renamed_as);
        # exclude files or subdirs under renamed dir
        @$sd_new_updated_files =
                grep {$_ !~ /^$dir_renamed_as\// } @$sd_new_updated_files
                                if ($dir_renamed_as);

        print "\nSize of File System new_updated_files after update :",
                scalar(@$fs_new_updated_files) if ($verbose);
    }

    if (defined($dir_del_hash)) {
        print "\nFilesystem compare output:\n";
        print "After excluding files under deleted/renamed dirs:\n";
        print 'Number of Deleted file : ', scalar(@$fs_deleted_files), "\n";
        print 'Number of New/Updated file list: ',
                    scalar(@$fs_new_updated_files), "\n\n";
    }

    print("\nCompare snapdiff output and actual filesystem diff\n");

    # Compare both snapdiff files/dirs diffs with
    # filesystem compare diff output.

    #
    # Check deleted files/dirs between snapdiff and filesystem diff
    #
    my ($d_err1, $d_err2) =
            compare_files($sd_deleted_files, $fs_deleted_files);

    if (defined($d_err1) && scalar(@$d_err1)){
        print 'Snapdiff output has this files as deleted, ' .
                 "but not found in Filesystem diff:\n";
        print join ("\n", @$d_err1), "\n";
    }
    if (defined ($d_err2) && scalar(@$d_err2)){
        print 'Filesystem has this deleted file list, ' .
                 "but not found in Snapdiff output :\n";
        print join ("\n", @$d_err2), "\n";
    }

    #
    # Check new/updated files/dirs between snapdiff and filesystem diff
    #
    my ($nd_err1, $nd_err2) =
            compare_files($sd_new_updated_files, $fs_new_updated_files);

    if (defined($nd_err1) && scalar(@$nd_err1)){
        print 'Snapdiff output has this new/updated files, ' .
                 "but not found in Filesystem diff:\n";
        print join ("\n", @$nd_err1), "\n";
    }
    if (defined($nd_err2) && scalar(@$nd_err2)){
        print 'Filesystem has this new/updated file list, ' .
                 "but not found in Snapdiff output :\n";
        print join ("\n", @$nd_err2), "\n";
    }

    #
    # Print success if no mismatch between snapdiff and filesystem outputs.
    #
    print "\n", '='x6 , "\n";
    if (( defined($d_err1) && scalar(@$d_err1)) ||
        ( defined($d_err2) && scalar(@$d_err2)) ||
        ( defined($nd_err1) && scalar(@$nd_err1)) ||
        ( defined($nd_err2) && scalar(@$nd_err2)) )
    {
        print 'FAIL : Mismatch found between snapdiff ' .
                    'and filesystem diff !!!'. "\n";
        $result=1;
    } else {
        print 'PASS : Snapdiff output and Actual ' .
                'filesystem diff are Same!!!' . "\n";
        $result=0;
    }
    print '='x6 , "\n";

    return $result;
}

# As Snapdiff output always be in unix path separator,
# For windows call this to convert path to windows format.
# i.e replace all '/' with '\'
sub _convert_path {
    my ($list) = @_;
    $Log->debug("convert unix path separator to windows"); 
    if (ref($list) eq "ARRAY") {
        $_ =~ s/\//\\/g  foreach (@$list);
    } elsif (ref($list) eq "SCALAR") {
        ${$list} =~ s/\//\\/g;
    } else {
        $Log->error("_convert_path is not array ref or scalar ref");
    }
}

# Decode to utf-8
sub _decode_utf {
    my ($list) = @_;
    # $Log->debug("In decode data to utf-8 function");
    $_ = decode ('UTF-8', $_) foreach (@$list);
}

# Depreciated only for backport
sub datagen {
    create_dataset(@_);
}
#
# Create dataset with multiple tools 
#
sub create_dataset {
    my ($Client, $Mount, %opts) = @_;
    my $Dataset_info;

    my $pop_info = populate_create(@_);
    $Dataset_info->{$pop_info->{name}} = $pop_info;

    my $datagen_info = datagen_create(@_);
    $Dataset_info->{$datagen_info->{name}} = $datagen_info;

    my $copy_dataset_info = copy_dataset_create(@_);
    $Dataset_info->{$copy_dataset_info->{name}} = $copy_dataset_info;

    return $Dataset_info;
}

#
# Modify dataset with same 
# tools which dataset created.
#
sub modify_dataset {
    my ($Client, $Mount, $Dataset_info, %opts) = @_;

    my $pop_info = populate_modify(@_);
    $Dataset_info->{$pop_info->{name}} = $pop_info;

    my $datagen_info = datagen_modify(@_);
    $Dataset_info->{$datagen_info->{name}} = $datagen_info;

    my $copy_dataset_info = copy_dataset_modify(@_);
    $Dataset_info->{$copy_dataset_info->{name}} = $copy_dataset_info;

    return $Dataset_info;
}

#
# Using datagen tool create dataset 
#
sub datagen_create {
    my $dataset_info;
    $dataset_info->{name} = 'datagen_create';
    return $dataset_info;
}

#
# Modify dataset created using datagen tool 
#
sub datagen_modify {
    my $dataset_info;
    $dataset_info->{name} = 'datagen_modify';
    return $dataset_info;
}

sub _map_share {
    my ($Client, $path, $user, $password) = @_;

    my %opts = ( nacltask_if_exists => 'reuse');
    
    # If UNC path specified split into server and share 
    if ($path =~ /^\\\\(.*?)\\(.*?)$/){
        $opts{server} = $1;
        $opts{share} = $2;
        $opts{driveless_mapping} = 1;
    }

    #
    # If user not passed, then use the 
    # global params to map the share

    if ($user) {
        $opts{user} = $user; 
        $opts{password} = $password; 
    } else {
        $opts{user} = $::CIFS_DOMAIN .'\\'. $::CIFS_USER;
        $opts{password} = $::CIFS_PASSWD;
    }

    $Log->comment("Map share %opts");

    my $Mount; 
    try {
	    $Mount = NACL::STask::Client::Mount->create(
	               command_interface => $Client,
                   %opts,
             );
    } 
    otherwise {
        my $eobj = shift;
        croak "Failed to Map share $path : ". $eobj->text() . "\n";
    };

    $Log->comment("Map path: " . $Mount->mount_point());
    return $Mount->mount_point();
}

#
# Using copy dataset 
#
sub copy_dataset_create {
    my ($Client, $Mount, %opts) = @_;
    my $dataset_info;
    $dataset_info->{name} = 'copy_dataset_create';

    my $dataset_dir = $::DATASET_DIR;

    # For windows we need to map the dataset dir
    if ( $Client->hostrec()->isa('Hostrec::Windows') ) {
        $dataset_dir = _map_share($Client, $::DATASET_DIR_WINDOWS,
                            $::DATASET_CIFS_USER, $::DATASET_CIFS_PASSWORD);
    }

    # overide if passed as parameter (windows ?)
    $dataset_dir = $opts{dataset_dir} if ( $opts{dataset_dir} );

    my $data_dir = $Mount->{mount_point} . 
                "/copy_dataset_$$" . int(rand(99999));

    my $api_set_obj_Client = $Client->apiset();

    $api_set_obj_Client->{session}->
        set_timeout( timeout => $::DATASET_COPY_TIMEOUT );

    $Log->comment("Dataset copy timeout set to " . 
                $api_set_obj_Client->{session}->get_timeout());

    $Log->comment('Copy dataset using client ' .  $Client->client());

    if ( $Client->hostrec()->isa('Hostrec::Windows') ) {
        _convert_path(\$data_dir);
        $api_set_obj_Client->mkdir(path => $data_dir);
    } else {
        $api_set_obj_Client->mkdir(
                                   paths => $data_dir,
                          'make-parents' => 'true',
                                    mode => '777',
        );
    }

    # archive => 'true', 
    my %args = ( 
               recursive => 'true',
                  source => $dataset_dir,
                    dest => $data_dir,
                   force => 'true',);

    $Log->debug("Args passed to copy dataset: " . Dumper \%args);

    # copy dataset to volume
    eval {
        # warning if robocoly not found.
        if ( $Client->hostrec()->isa('Hostrec::Windows') ) {
            my $cmd = "robocopy $dataset_dir $data_dir /MIR";
            $api_set_obj_Client->execute_raw_command(command=>$cmd);
        } else {
            $api_set_obj_Client->cp( %args );
        }
    };

    $Log->warn("copy dataset has some isues!!! need to be resolved $@")
        if ($@);

    $dataset_info->{source} = $dataset_dir;
    $dataset_info->{dest} = $data_dir;

    return $dataset_info;
}

#
# Modify dataset created by copying dataset 
#
sub copy_dataset_modify {
    my ($Client, $Mount, $Dataset_info, %opts) = @_;
    my $dataset_info;
    my $dataset_dir;

    $dataset_info->{name} = 'copy_dataset_modify';

    # For windows we need to map the dataset dir
    if ( $Client->hostrec()->isa('Hostrec::Windows') ) {
        $dataset_dir = $dataset_info->{source};
        # Map if not already maped.
        if (not $dataset_dir) {
            $dataset_dir = _map_share($Client, $::DATASET_DIR_WINDOWS,
                            $::DATASET_CIFS_USER, $::DATASET_CIFS_PASSWORD);
        }
    }

    my $data_dir = $Dataset_info->{'copy_dataset_create'}->{dest};

    my $api_set_obj_Client = $Client->apiset();

    if (not $Client->hostrec()->isa('Hostrec::Windows')) {
        $api_set_obj_Client->set_privilege('privilege-level' => 'root');
    }

    $api_set_obj_Client->{session}->
        set_timeout( timeout => ($::DATASET_COPY_TIMEOUT || 7200) );

    $Log->comment("copy dataset modify timeout set to " . 
                $api_set_obj_Client->{session}->get_timeout());

    $Log->comment('Copy dataset using client ' .  $Client->client());

    my %args = ( recursive => 'true',
                      mode => '777',
                     paths => $data_dir,
    );

    $Log->comment("Args passed to modify dataset: " . Dumper \%args);

    # copy dataset to volume
    eval {
        if ( $Client->hostrec()->isa('Hostrec::Windows') ) {
            my $cmd = "robocopy $dataset_dir $data_dir /MIR";
            $api_set_obj_Client->execute_raw_command(command => $cmd);
        } else {
            $api_set_obj_Client->chmod( %args );
        }
    };
    $Log->comment("modify dataset has some isues!!!" . $@) if ($@);

    $dataset_info->{dest} = $data_dir;

    return $dataset_info;
}

#
# Modify dataset created using populate tool 
#
sub populate_modify {
    my ($Client, $Mount, $Dataset_info, %opts) = @_;
    my $dataset_info;
    $dataset_info->{name} = 'populate_modify';

    # TOTAL_SUBTREE_DEL=0.01 MOD_AVG=0.10 GROW_AVG=0.10 SHRINK_AVG=0.10
    use vars qw($POP_TOTAL_MOD $POP_TOTAL_DEL 
                $POP_TOTAL_GROW $POP_TOTAL_SHRINK);

    param ('POP_TOTAL_MOD','-default','0.5');
    param ('POP_TOTAL_DEL','-default','0.1');
    param ('POP_TOTAL_GROW','-default','0.1');
    param ('POP_TOTAL_SHRINK','-default','0.1');

    # Get params used in populate command
    my $data_dir = $Dataset_info->{populate_create}->{tree};
    my %args = @{$Dataset_info->{populate_create}->{options}};

    $args{SEED} = $Dataset_info->{populate_create}->{seed};
    $args{TOTAL_MOD} = $::POP_TOTAL_MOD;
    $args{TOTAL_DEL} = $::POP_TOTAL_DEL;
    $args{TOTAL_GROW} = $::POP_TOTAL_GROW;
    $args{TOTAL_SHRINK} = $::POP_TOTAL_SHRINK;
    $args{VERBOSE} = "3";

    # merge params of populate and modify
    %args = (%args, %opts);

    print "Dir tree for Modify: $data_dir\n";
    print "Args passed to Modify: ", Dumper \%args;
    my @rc = eval { run_modify_tree($Client->client(), $data_dir, %args); };
    print "Modify return values: ".  Dumper \@rc if ($verbose);

    $dataset_info->{tree} = $data_dir;
    $dataset_info->{options} = [%args];

    return $dataset_info;
}

#
# Using populate tool to create dataset
#
sub populate_create {

    my ($Client, $Mount, %opts) = @_;
    my $dataset_info;
    $dataset_info->{name} = 'populate_create';

    use vars qw($POP_DIR_AVG $POP_DEPTH_AVG
                $POP_FILE_AVG $POP_BREADTH_AVG);

    param ('POP_DIR_AVG','-default','3');
    param ('POP_DEPTH_AVG','-default','3');
    param ('POP_FILE_AVG','-default',20+int(rand(15)));
    param ('POP_BREADTH_AVG','-default','3');

    # Overwrite Populate params if given
    $::POP_DIR_AVG = $opts{pop_dir_avg} if ($opts{pop_dir_avg});
    $::POP_DEPTH_AVG = $opts{pop_depth_avg} if ($opts{pop_depth_avg});
    $::POP_FILE_AVG = $opts{pop_file_avg} if ($opts{pop_file_avg});
    $::POP_BREADTH_AVG = $opts{pop_breadth_avg} if ($opts{pop_breadth_avg});

    my $data_dir = $Mount->{mount_point} . "/data$$" . int(rand(99999));
    my $api_set_obj_Client = $Client->apiset();

    $Log->debug('populate data using client' . 
                                $Client->client());

    if ($Client->hostrec()->isa('Hostrec::Windows')) {
        _convert_path(\$data_dir);
        $api_set_obj_Client->mkdir(path => $data_dir);
    } else {
        $api_set_obj_Client->mkdir(
                                paths => $data_dir,
                        'make-parents'=> 'true',
                                 mode => '777',
                            );
    }
    my %args = ( 
                DIR_AVG=>$::POP_DIR_AVG, 
                DEPTH_AVG=>$::POP_DEPTH_AVG, 
                FILE_AVG=>$::POP_FILE_AVG, 
                BREADTH_AVG=>$::POP_BREADTH_AVG, 
                );

    print "Args passed to Populate: ". Dumper \%args if ($verbose);

    # run populate on given client
    my @rc = eval { run_populate_tree($Client->client(), $data_dir, %args); };

    print "Populate return values: ".  Dumper \@rc if ($verbose);

    if ($@ or (scalar(@rc) < 3)) {
        croak("\npopulate_tree failed for $data_dir $@: $!\n");
    }

    # Save options given to populate, later used for modify

    $dataset_info->{tree} = $data_dir;
    $dataset_info->{options} = [%args];
    $dataset_info->{seed} = $rc[0];

    return $dataset_info;
}

#
# split snapdiff output into deleted or new/updated
# deleted if base-snapshot present in filename
# new/updated if diff-snapshot present in filename
#
sub add_to_file_list {
    my ($s, $h) = @_;
    my (@deleted_files, @add_updated_files);
    my $volume = $s->{volume};

    foreach my $e (@$h) {

        $e->{filename} =~ s/\/vol\/${volume}\/\.snapshot\/(.*?)\///;   # 7-mode /vol/vol1/.snapshot/snap0/file1
        next if ($e->{filename} eq "\/vol\/${volume}");                # 7-mode Exclude top level change [ i.e /vol/volume ]

        if (!$1) {
            $e->{filename} =~ s/^\///;                                 # C-mode remove initial '/' from [ i.e /dir1/file1 to dir1/file1 ]
            next if ($e->{filename} eq "");                            # C-mode exclude top level change [ i.e / ]
        }

        # unescap filename: As Snapdiff escapes all non printable chars and 
        # all unicode chars for NFS. 

        # $Log->debug('Before unescape: ' . $e->{filename});
        $e->{filename} =~ s/\\([0-9A-Fa-f]{2})/chr(hex($1))/eg;
        # $Log->debug('After unescape: ' .  $e->{filename});

        # Possible change-type values are:
        #   "file_creation",   "file_deletion"
        #   "inode_creation",  "inode_deletion"
        #   "inode_modification"

        if ($e->{'change-type'} =~ /file_deletion|inode_deletion/i) {
            push @deleted_files, $e->{filename};
        } elsif ($e->{'change-type'} =~ /file_creation|inode_creation|inode_modification/i) {
            push @add_updated_files, $e->{filename};
        } else {
            croak 'unknown change-type ' . $e->{'change-type'} ."\n" . $e->{filename};
        }
    }

    return (\@deleted_files, \@add_updated_files);
}

#
# This routine return list deleted/new/updated
# (includes file/inode/attribute changes)
# for given base & diff snapshots, by mounting the
# snapshots and comparing dir trees.
#
sub get_filesystem_changes {
    my ($s, $h ) = @_;

    my ($dir1, $dir2) = _new_mount_vols($s);

    croak "Failed to get Clien object\n" unless ($Client) ;
    $client_name = $Client->hostrec->hostname();
    croak "Failed to get client host name\n" unless ($client_name) ;
    print ("Client name : $client_name \n") if ($verbose);

    my @rc = eval {
                    run_compare_trees($client_name, $dir1, $dir2,
                        IGNORE_LIST => '(q(.snapshot))',
                        EXTRA => 1,
                        ATTRIBUTES => 1);
             };
    if ($@ or (scalar(@rc) < 3)) {
        croak("\nCompare_tree failed for $dir1 and $dir2 $@: $!\n");
    }
    my ($deleted, $added, $differ, $subpaths) = @rc;

    my (@deleted_files, @new_updated_files);
    if (scalar(@$deleted)) {
        if ($verbose) {
            print "----Deleted File list----\n";
            print $_,"\n" foreach @$deleted;
        }
        push @deleted_files, @$deleted;
    }
    if (scalar(@$added)) {
        if ($verbose) {
            print "----New File list----\n";
            print $_,"\n" foreach @$added;
        }
        push @new_updated_files, @$added;
    }
    if (scalar(@$differ)) {
        if ($verbose) {
            print "----Modified File list----\n";
            print $_,"\n" foreach @$differ;
        }
        # exclude '.' for modified list,
        # as we exclude top level dir fron snapdiff also
        print "exclude . from filesystem output if exists\n" if ($verbose);
        @$differ = grep {$_ ne '.' } @$differ;
        push @new_updated_files, @$differ;
    }

    my $recreated = get_files_recreated ($h, $differ);
    if (defined($recreated) && scalar(@$recreated)) {
        push @deleted_files, @$recreated ;
        if ($verbose) {
            print "----Files recreated adding to deleted list----\n";
            print $_,"\n" foreach @$recreated;
        }
    }

    un_mount_dirs($dir1, $dir2);

    if ($verbose){
        print 'Filesystem compare returned ',
            scalar(@deleted_files), " files/dirs as deleted\n";
        print 'and ', scalar(@new_updated_files),
                " new/updated files/dirs\n";
    }

    return (\@deleted_files, \@new_updated_files);
}


sub run_populate_tree {

    my $client = shift;
    my $tree = shift;
    my %opts = @_;

    my $code = <<'CODE';
{
    use Tharn qw($Parent);
    use Data::Dumper;
    use DataManip::local qw(populate_local_tree);
    my ($seed, $subpaths, $subseeds) = eval { &populate_local_tree(@_) };
    $subpaths=[] unless($subpaths);
    $subseeds=[] unless($subseeds);
    logresult("FATAL", "EVAL_ERROR=$@") unless ($seed);

    $Parent->message_put(message=>$seed,
                            type=>"Populate/seed");
    $Parent->message_put(message=>$subpaths,
                            type=>"Populate/subpaths");
    $Parent->message_put(message=>$subseeds,
                            type=>"Populate/subseeds");

    my $out = $Parent->message_get(type=>"Populate/DONE");
    print "OUT = $out\n";
}
CODE

    # Forces the options used to get printed in the logs
    $opts{SHOW_OPTS} = 1;

    $Log->comment('Before calling populate subtest') if ($verbose);

    my $obj = subtest($code,
                '-runid','populate',
                '-bg',
                '-client', $client,
                '-onexit', sub {},
                '--', $tree, %opts);

    my $seed = $obj->message_get(type=>'Populate/seed');
    my $subpaths = $obj->message_get(type=>'Populate/subpaths');
    my $subseeds = $obj->message_get(type=>'Populate/subseeds');

    my $done="DONE";
    $obj->message_put(message=>$done, type=>"Populate/DONE");

    if ($verbose) {
        print ("SEED :=", $seed, "\n");
        print ("SUBPATHS :=", join("\n", @$subpaths),"\n");
        print ("SUBSEEDS :=", join("\n", @$subseeds),"\n");
    }

    $obj->wait_finish();

    return ($seed, $subpaths, $subseeds);
}

sub run_modify_tree {

    my $client = shift;
    my $tree = shift;
    my %opts = @_;

    my $code = <<'CODE';
{
    use Tharn qw($Parent);
    use Data::Dumper;
    use DataManip::local qw(modify_local_tree);
    my ($seed, $subpaths, $subseeds) = eval { &modify_local_tree(@_) };
    $subpaths=[] unless($subpaths);
    $subseeds=[] unless($subseeds);
    logresult("FATAL", "EVAL_ERROR=$@") unless ($seed);

    $Parent->message_put(message=>$seed,
                            type=>"Modify/seed");
    $Parent->message_put(message=>$subpaths,
                            type=>"Modify/subpaths");
    $Parent->message_put(message=>$subseeds,
                            type=>"Modify/subseeds");

    my $out = $Parent->message_get(type=>"Modify/DONE");
    print "OUT = $out\n";
}
CODE

    # Forces the options used to get printed in the logs
    $opts{SHOW_OPTS} = 1;

    $Log->comment('Before calling modify subtest') if ($verbose);

    my $obj = subtest($code,
                '-runid','modify',
                '-bg',
                '-client', $client,
                '-onexit', sub {},
                '--', $tree, %opts);

    my $seed = $obj->message_get(type=>'Modify/seed');
    my $subpaths = $obj->message_get(type=>'Modify/subpaths');
    my $subseeds = $obj->message_get(type=>'Modify/subseeds');

    my $done="DONE";
    $obj->message_put(message=>$done, type=>"Modify/DONE");

    if ($verbose) {
        print ("SEED :=", $seed,"\n");
        print ("SUBPATHS :=", join("\n", @$subpaths),"\n");
        print ("SUBSEEDS :=", join("\n", @$subseeds),"\n");
    }

    $obj->wait_finish();

    return ($seed, $subpaths, $subseeds);
}

sub run_compare_trees {

    my $client = shift;
    my $tree1 = shift;
    my $tree2 = shift;
    my %opts = @_;

    my $code = <<'CODE';
{
    use Tharn qw($Parent);
    use Data::Dumper;
    use DataManip::local qw(compare_local_trees);
    require File::Temp; 
    print "Tree1 : $_[0] \n";
    # hack for Windows temp dir if base-snapshot is none
    if ($_[0] eq "TEMPDIR") {
        $_[0] = File::Temp->newdir();
    }
    print "Tree1 : $_[0] \n";
    print "Tree2 : $_[1] \n";

    my ($in1, $in2, $differ, $subpaths) = eval { &compare_local_trees(@_) };
    $Log->debug( Dumper $in1);
    $Log->debug( Dumper $in2);
    $Log->debug( Dumper $differ);
    $Log->debug( Dumper $subpaths);
    logresult("FATAL", "EVAL_ERROR=$@") if ($@);

    $Parent->message_put(message=>$in1,
                            type=>"Compare/in1");
    $Parent->message_put(message=>$in2,
                            type=>"Compare/in2");
    $Parent->message_put(message=>$differ,
                            type=>"Compare/differ");
    $Parent->message_put(message=>$subpaths,
                            type=>"Compare/subpaths");

    my $out = $Parent->message_get(type=>"Compare/DONE");
    print "OUT = $out\n";
}
CODE

    # Forces the options used to get printed in the logs
    $opts{SHOW_OPTS} = 1;

    $Log->comment('Before calling compare subtest') if ($verbose);

    my @subtest_params = (
                '-runid','compare',
                '-bg',
                '-client', $client,
                '-onexit', sub {});

    if ($Client->hostrec()->isa('Hostrec::Windows')) {
        $Log->comment("Don't run as root for Windows");
    } else {
        push @subtest_params,('-username','root') 
    }
    my $obj = subtest($code,
                @subtest_params, 
                '--', $tree1, $tree2, %opts);

    my $in1 = $obj->message_get(type=>'Compare/in1');
    my $in2 = $obj->message_get(type=>'Compare/in2');
    my $differ = $obj->message_get(type=>'Compare/differ');
    my $subpaths = $obj->message_get(type=>'Compare/subpaths');

    my $done="DONE";
    $obj->message_put(message=>$done, type=>"Compare/DONE");

    if ($verbose) {
        print ("IN1 :=", join("\n", @$in1),"\n");
        print ("IN2 :=", join("\n", @$in2),"\n");
        print ("DIFF :=", join("\n", @$differ),"\n");
        print ("DIFF :=", join("\n", @$subpaths),"\n");
    }

    $obj->wait_finish();

    return ($in1, $in2, $differ, $subpaths);
}


sub _new_mount_vols {
    my ($s) = @_;
    my ($DIR1, $DIR2);

    my ($filer, $volume, $base_snapshot, $diff_snapshot, $vserver) =
            ($s->{filer}, $s->{volume}, $s->{base_snapshot},
             $s->{diff_snapshot}, $s->{vserver},);

    my $Volume;
    if ($s->{node} && ref($s->{node}) =~ /Node/){
        $Node = $s->{node};
    } else {
        try {
            $Node = NACL::C::Node->new(node=>$filer);
        }
        catch NATE::BaseException with {
            my $eobj = shift;
            croak 'Failed to find Node Error: '. $eobj->text() . "\n";
        };
    }

    print 'Node : ' . $Node->hostrec->hostname,"\n";

    try {
        my @params=(volume => $volume);
        push @params, ('vserver' => $vserver) if ($vserver);
        $Volume = NACL::STask::Volume->find(
                        command_interface => $Node,
                                   filter => {
                                        vserver => $vserver,
                                         volume => $volume,
                                    },
        );
    }
    catch NATE::BaseException with {
        my $eobj = shift;
        croak "Failed to get Volume Object Error: ", $eobj->text(), "\n";
    };

    print "Get Client objece \n" if ($verbose);

    $Client = NACL::C::Client->find();
    my $vol_path;
    if ($Node->mode() eq "CMode") {
        $vol_path = $Volume->state()->junction_path();
    } else {
        $vol_path = "/vol/" . $Volume->volume;
    }

    if ($verbose) {
        print "Junction path : ", $vol_path, "\n";
        print "Volume :", $Volume->volume, "\n";
        print "Client :", $Client->hostrec->hostname, "\n";
    }

    if (not $Client->hostrec()->isa('Hostrec::Windows')) {
        $DIR1 = "/t/${filer}${vol_path}_${base_snapshot}_$$";
        $DIR2 = "/t/${filer}${vol_path}_${diff_snapshot}_$$";
    }

    # unmount if path already exists
    #un_mount_dirs($DIR1, $DIR2);

    if (!$base_snapshot or ($base_snapshot eq 'none')) {
        $Mount_base = undef;
        my ($cmd, $out);
        if ($Client->hostrec()->isa('Hostrec::Windows')) {
            $DIR1 = "TEMPDIR";
        } else {
            if (! -d $DIR1) {
                $cmd = "$::SUDO_PATH /bin/mkdir -p $DIR1 \; ".
                " $::SUDO_PATH /bin/chmod 777 $DIR1" ;
                $out = $Client->apiset()->execute_raw_command(command=>$cmd);
                print $out;
            }
        }
    } else {
        $Mount_base = mount_snap($Node, $Client, $base_snapshot, $vol_path, $DIR1, $vserver, $Volume);
        croak "Mount snapshot $base_snapshot failed!" unless ($Mount_base);
    }

    $Mount_diff = mount_snap($Node, $Client, $diff_snapshot, $vol_path, $DIR2, $vserver, $Volume);
    croak "Mount snapshot $diff_snapshot failed!" unless ($Mount_diff);

    # May be Mount objs itself can be returned, for unmount
    if (($::FILE_ACCESS_PROTOCOL eq "cifs") 
         || $Client->hostrec()->isa('Hostrec::Windows')) {
        # no Mount obj will be created when if only diff snapshot
        # is passed, and base_snapshot is none
        # In this case DIR1 will have TEMPDIR string.
        # compare() will create temp empty dir for comparing.
        $DIR1 = $Mount_base->mount_point()
                . '\\~snapshot'. '\\' . $base_snapshot 
                    unless($DIR1 eq "TEMPDIR");
        $DIR2 = $Mount_diff->mount_point() 
                . '\\~snapshot'. '\\' . $diff_snapshot;
        $Log->debug("Windows DIR1 $DIR1 DIR2 $DIR2");
    }

    return ($DIR1, $DIR2);
}

sub un_mount_dirs {
    foreach my $Mount ($Mount_base, $Mount_diff) {
        if (ref($Mount) =~ /Mount/i) {
            eval { $Mount->purge(); };
            if ($verbose) {
                print "Unmount returned :$@\n" if ($@);
            }
        }
    };
}

#
# Returns symmetric difference for given two file lists.
# (i.e elements which are in either of the sets and not in their intersection)
# Files which are missing in other list.
#
sub compare_files {

    my ($list1, $list2) = @_;
    my ($only_in_list1, $only_in_list2);
    my (%list1_hash, %list2_hash);

    $list1_hash{$_}++ foreach (@$list1);
    $list2_hash{$_}++ foreach (@$list2);

    my ($count1, $count2) = (0,0);
    foreach (keys %list1_hash){
        $count1++;
        push @$only_in_list1, $_
            if (not exists $list2_hash{$_});
    }
    foreach (keys %list2_hash){
        $count2++;
        push @$only_in_list2, $_
            if (not exists $list1_hash{$_});
    }

    return ($only_in_list1, $only_in_list2);
}

# If a file/dir deleted and created with same name
# filesystem compare gives as modified whereas
# snapdiff gives as file/dir deleted & file/dir created
# To handle this case
# We add this to fs deleted list, and
# No need to delete from modify and add to create list
# as both lists (created & modified) are combined for checking.
sub get_files_recreated {
    my ($h, $fs_modified_list) = @_;
    my @recreated;
    my %found;
    foreach my $fname (@$fs_modified_list){
        foreach my $e (@$h){
            if ( ($e->{filename} eq $fname) &&
                 ( ($e->{'change-type'} eq 'file_deletion') ||
                   ($e->{'change-type'} eq 'file_creation')
                 )
               )
            {
                if (exists $found{$fname}) {
                    push @recreated, $fname;
                    last;
                } else {
                    $found{$fname}=1;
                }
            }
        }
    }
    return \@recreated;
}

# Handle dir delete/rename
# 1) Dir Delete
#     Snapdiff returns only dir (i.e won't return all files
#           inside the deleted dir)
#     Filesystem returns all files and dirs under the deleted dir
#     example: rm -rf dir864  snapdiff output will have only one entry dir864
#            where as filesystem output will have all the files or dirs
#                   under dir dir864.
# 2) Dir Rename
#         Snapdiff returns only three entries for dir rename
#           (i.e inode_modified, old dir as deleted, new as added )
#         filesystem compare will return all files under old as deleted and
#         also added under new dir.
#
# Generate hash for handling above
#
# Returns hash with (old_dir_name => new_dir_name, ...  # if dir renamed
#                    dir_deleted => undef)              # if dir deleted
#

# Based on inode and crtime we can figureout dir/files renamed.
#
sub get_dirs_deleted {
    my ($h) = @_;

    my (%dir_d_hash, %dir_c_hash);
    foreach my $e (@$h){
        if ($e->{ftype} == 2) {
            my $dir_name = $e->{filename};
            my $crtime = $e->{crtime};
            if ($e->{'change-type'} eq 'file_deletion') {
                $dir_d_hash{$e->{inode}} = [$dir_name,$crtime];
            }elsif ($e->{'change-type'} eq 'file_creation') {
                $dir_c_hash{$e->{inode}} = [$dir_name,$crtime];
            }
        }
    }

    my %dir_del_hash;
    while ( my ($inode, $spec) = each (%dir_d_hash) ) {
        my ($dir_d,$crtime_d) = @{$spec};
        if (!defined ($dir_c_hash{$inode})) {
            $dir_del_hash{$dir_d}= undef;     # dir-deleted
            next;
        }
        my ($dir_c,$crtime_c) = @{$dir_c_hash{$inode}};
        if ($dir_c) {
            if ($crtime_d == $crtime_c){
                $dir_del_hash{$dir_d} = $dir_c;    # dir-renamed
            } else {
                $dir_del_hash{$dir_d} = undef;     # dir-deleted
                print "$dir_d recreated as crtime differ $crtime_d & " .
                      "$crtime_c and $dir_c \n" if ($verbose) ;
                ## hopfully this is handled by get_recreated_dirs
            }
        }
    }

    return (%dir_del_hash) ? \%dir_del_hash : undef;
}

# Finds a client from CLIENTS/CLIENT param and
# Mounts the volume on the Client.
#
# Input:
#    Volume NACL Volume object.
#
# Returns:
#    Client NACL component Object
#    Mount NACL MTask Object

sub do_mount {
    my $Volume = shift;

    my ($Client, $Mount);
    my $Node = NACL::STask::Node->find();

    # todo purge before and after
    try
    {
        $Client = NACL::C::Client->find();

        $Log->comment('Mount volume ' . $Volume->volume .
                        ' on client ' . $Client->hostrec->hostname);

        my %mount_param = (
            nacltask_if_exists=> 'reuse',
            nacltask_address_family => $::IPV || 'ipv4',
        );

        if($Client->hostrec()->isa('Hostrec::Windows')) {
            $Log->comment("Windows client");
        } else {
            $mount_param{chmod} = '777';
        }

        print Dumper \%mount_param;

        if ( ((!defined($::FILE_ACCESS_PROTOCOL)) 
                || ($::FILE_ACCESS_PROTOCOL eq "nfs")) 
             || 
             (($::FILE_ACCESS_PROTOCOL eq "cifs") 
                || ($Client->hostrec()->isa('Hostrec::Windows')))
           ) {

            if($Client->hostrec()->isa('Hostrec::Windows')) {
                $Log->comment("Add user/password for maping cifs share");
                $mount_param{user} = $::CIFS_DOMAIN .'\\'. $::CIFS_USER;
                $mount_param{password} = $::CIFS_PASSWD;
                $Log->comment("Sleep for 1 min.");
                sleep(60);
            }
	        $Mount = NACL::STask::Client::Mount->create(
	                            command_interface => $Client,
                                volume => $Volume,
                                %mount_param,
                 );
            $Log->comment("Mount point: " . $Mount->mount_point());
            $Log->comment("Sleep for 20 sec.");
            sleep(20);
        } else {

            $Log->comment("get cifs share");
            my $share_name; 
            try {
                my $Share = NACL::CS::VserverCifsShare->fetch (
                                command_interface => 
                                    $Volume->command_interface(),
                                filter => {
                                    path => 
                                        $Volume->get_one_state_attribute('junction-path'),
                                }
                            );
                $share_name = $Share->share_name();
            } otherwise {
                my $eobj = shift;
                $Log->comment("Unable to map Cifs Share " 
                 . $Volume->volume() . " Error: " . $eobj->text());
            };

            $Log->comment("SMB mount on Linux client");
            # Assumptions 
            # . cifs setup done 
            # . volume level qtree security mixed
            # . Share name same as volume name 
            # . 7mode: create_ucode and convert_ucode are enabled
            my $ip;
            if ($Node->mode() eq "CMode") {
                my $interface = NACL::CS::Interface->fetch(
                    command_interface => $Node,
                    filter            => {
                        vserver         => $Volume->get_one_state_attribute('vserver'),
                        'data-protocol' => ['cifs'],
                    },
                    requested_fields => [qw(address)],
                    allow_empty      => 1,
                );
                $ip = $interface->address();
            } else {
                $ip = $Node->hostrec()->default_ip();
            }
  
            if ($ip)  { 
                $Log->comment("Using Net Interface addr: $ip");
            } else {
                $Log->error('Unable to get network interface address');
                return $TCD::FAIL;
            }

            $Mount = NACL::STask::Client::Mount->create(
                        command_interface => $Client,
                                   server => $ip,
                                     path => "//$ip/$share_name",
                                 'fstype' => 'cifs',
                                   'type' => 'device',
                                mountopts => "domain=$::CIFS_DOMAIN,user=$::CIFS_USER,password=$::CIFS_PASSWD",
                        nacltask_if_exists=> 'purge',
                  nacltask_address_family => $::IPV || 'ipv4', 
            );
        }
    }
    catch NATE::Exceptions::Argument with {
        my $eobj = shift;
        my $error = $eobj->text();
        croak "Mount failed : $error \n";
    }
    catch NATE::BaseException with {
        my $eobj = shift;
        my $error = $eobj->text();
        $Log->error('Error while mounting : Volume '.
                    $Volume->volume .  ' failure : ' . $error);
    };
    if (!$Mount || !$Client) {
        croak 'Failed to mount volume ' . $Volume->volume .": $@\n";
            'on Client ' . $Client->hostrec->hostname ."\n" if ($Client);
    }

    $Log->comment('Mount point '.  $Mount->mount_point() );

    return  ($Client, $Mount);
}

sub mount_snap {
    my ($Node, $Client, $snap, $jpath, $mpoint, $vserver, $Volume) = @_;

    my $Mount;

    try {
        print "Mounting snap $snap \n";

        if ((!defined($::FILE_ACCESS_PROTOCOL)) 
             || ($::FILE_ACCESS_PROTOCOL eq "nfs")) 
        {
            if ($Client->hostrec()->isa('Hostrec::Windows')) {
                $Log->error("Snapdiff option File access protocol nfs " .
                            " and Client used is Windows client !.");
                return $TCD::FAIL;
            }

            $Mount = NACL::STask::Client::Mount->create(
                                command_interface => $Client,
                                             path => "$jpath/\.snapshot/$snap",
                                          vserver => $vserver,
                         server_command_interface => $Node,
                                        mountopts => "ro",
                                      mount_point => $mpoint,
                               nacltask_if_exists => 'purge',
                          nacltask_address_family => $::IPV || 'ipv4', 
             );
        } elsif (($::FILE_ACCESS_PROTOCOL eq "cifs") 
                  || ($Client->hostrec()->isa('Hostrec::Windows'))) {
        
                $Log->comment("Add user/password for maping cifs share");
                my $cifs_user = $::CIFS_DOMAIN .'\\'. $::CIFS_USER;
                my $cifs_password = $::CIFS_PASSWD;

                $Mount = NACL::STask::Client::Mount->create(
                                command_interface => $Client,
                                           volume => $Volume,
                                             user => $cifs_user,
                                         password => $cifs_password,
                               nacltask_if_exists => 'reuse',
                         );
        } else {
            my $ip;
            if ($Node->mode() eq "CMode") {
                my $interface = NACL::CS::Interface->fetch(
                    command_interface => $Node,
                    filter            => {
                        vserver => $vserver,
                        'data-protocol' => ['cifs'],
                       'address-family' => $::IPV || 'ipv4', 
                    },
                    requested_fields => [qw(address)],
                    allow_empty      => 1,
                );
                $ip = $interface->address();
            } else {
                $ip = $Node->hostrec()->default_ip();
            }
  
            if ($ip)  { 
                $Log->comment("Using Net Interface addr: $ip");
            } else {
                $Log->error('Unable to get network interface address');
                return $TCD::FAIL;
            }

            $Log->comment("SMB mount on Linux client");
            $jpath =~ s/^\/vol\//\//;
            $Mount = NACL::STask::Client::Mount->create(
                        command_interface => $Client,
                                   server => $ip,
                                     path => "//${ip}${jpath}/\.snapshot/$snap",
                                 'fstype' => 'cifs',
                                   'type' => 'device',
                                mountopts => "domain=$::CIFS_DOMAIN,user=$::CIFS_USER,password=$::CIFS_PASSWD",
                              mount_point => $mpoint,
                        nacltask_if_exists=> 'purge',
                  nacltask_address_family => $::IPV || 'ipv4', 
            );
        }
    }
    catch NATE::Exceptions::Argument with {
        my $eobj = shift;
        my $error = $eobj->text();
        croak "Nate exception argument : $error \n";
    };

    if ($Mount) {
        print "Mount point : ", $Mount->mount_point(), "\n";
    } else {
        print "Mount failed\n";
    }

    return $Mount;
}

1;
__END__

TODO: Script to handle following cases:


    - XMP's encoding of &, <, >, ',", i.e (remove &amp;, &qot;, ...)
    - Non-Ascii char 
    - Unicode char and Lang etc..

    a)  Need check sudo supported on the client script is running,
        if not bailout.

    c)  If snapdiff fails with error print error and bail out.
        Example:
          <results reason='Parsing error in results: detected
                    an error in element content
              ' status='failed' errno='13001'/>
    d)  if a dir/file was deleted and recreated with same name,
        new dir/file created may get the same inode,
            (as inodes get recycled)
        script should not include as dir rename.
        Since sd gives ad dir deleted and dir created with same inode.
    e)
        Special case once u access from cifs
        all files treated as deleted and recreated. ?

Done:
 Snapdiff gives dir deleted with same inode for new dir created and
 Snapdiff gives dir deleted with same inode new dir created and
 also gives files inside some times. ex: between snapshots
 dir renamed and new files added or modified.

    We can't exclude all new files in renamed dir
    as same snap shot after dir rename new files would have
    been created or some files modified OR
    files would have been deleted really.
    so only exclude deleted files which are in new
    and only exclude new files which are in deleted.

    b)  If a file is deleted and recreated with same name
        Filesystem diff only gives file as modified, where as
        snapdiff output will have same file as deleted and created.
        note: Check has to be based on different crtime,
        some time new file may get same inode.
        Done: i.e already handled by gunction get_recreated_files);

  FIX: /x/eng/rtp-testbeds/dpg/snapdiff/burra/logs/20120309_171435  24k diffs in 2 hours.


  NEED to fix:
    check($Zapi) is called without running snapdiff start/end with $Zapi.
        i.e $Zapi->new() imediatly check($Zapi);...
