#  $Id: //depot/prod/test/Rsierranevada/dp/snapdiff/zapi/lib/ZapiSD.pm#2 $

package ZapiSD;

use	Zapi;
use	strict;
use	NaElement;
use Params::Validate qw(:all);
use Tharn;
use XML::Simple;
use Carp;
use Data::Dumper;
use NACL::C::Node;
use NACL::STask::NetworkInterface;
use dp::snapdiff::zapi::lib::CommonParams;

BEGIN {
    use Exporter;
    use	vars ('@ISA', '@EXPORT');
    @ISA = qw(Exporter);
    @EXPORT = qw(get_all_diffs iter_start iter_next
                iter_status iter_end get_total_changes);
    use vars qw($MAX_WAIT_TIME_FOR_PENDING 
                $WARN_ITER_NEXT_TIMEOUT);
    param ('MAX_WAIT_TIME_FOR_PENDING','-default',1800); # 1/2 hour.
};

@ZapiSD::ISA = qw(Zapi);

# Returns new ZapiSD object
sub new {
    my ($class)  = shift;

    my %args = validate_with (
                params => \@_,
                spec => {
                            filer => {type=>SCALAR|HASHREF},
                          vserver => {type=>SCALAR|HASHREF|UNDEF,
                                       optional =>1, default=>''},
                       sd_zapi_ip => {type=>SCALAR, optional =>1},
                         username => {type=>SCALAR, optional =>1},
                         password => {type=>SCALAR, optional =>1},
                          timeout => {type=>SCALAR, optional =>1},
                          verbose => {type=>SCALAR, 
                                        optional =>1, default=> 1},
                           vfiler => {type=>SCALAR, optional =>1},
                     print_xml_op => {type=>SCALAR, optional =>1,
                                            default=>1},
                },
               );

    my $default_ip;

    if (ref($args{filer}) =~ /Node/) {
        $default_ip = $args{filer}->hostrec->default_ip;
    } else {
        $default_ip = host($args{filer})->default_ip;
    }

    # Use ip if provided
    if ($args{sd_zapi_ip}){
        $default_ip = $args{sd_zapi_ip};
    } elsif ($::USE_DATA_LIF_IP) {
        $default_ip = $::USE_DATA_LIF_IP;
        $Log->comment("Using data lif ip $::USE_DATA_LIF_IP");
    } elsif($::USE_DATA_LIF){ 
	## Fetch the data lif IP from filer, if USE_DATA_LIF is set to 1 in stest file
	$default_ip = Fetch_data_lif($args{vserver}, $args{filer});
	$Log->comment("Using Fetched Data Lif IP $default_ip");
    }
    my ($user)   = $args{username};
    my ($password) = $args{password};
    my ($timeout) = $args{timeout};

    $Log->comment('Filer: ', $args{filer},' default IP: ' . $default_ip,
                "user: $user, passwd: $password");
	my $self = new Zapi($default_ip, $user, $password, $timeout);

    if (ref($args{filer}) =~ /Node/) {
        print "Node Object passed\n";
        $self->{node} = $args{filer};
        $self->{filer} = $args{filer}->hostrec()->hostname;
    } else {
        $self->{filer} = $args{filer};
        # Get Node object if filer name passed.
        $self->{node} = NACL::C::Node->new( name => $args{'filer'} );
    }

	$self->set_fail_routine(\&ZapiSD::nate_fail);
	$self->set_log_routine(\&ZapiSD::nate_log);

    $self->{verbose} = $args{verbose};
    $self->{verbose} = $::DBG if ($::DBG ne ''); 

    $self->{print_xml_op} = $args{print_xml_op} 
        if ($args{print_xml_op});

    $self->{print_xml_op} = $::PRINT_XML_OUTPUT 
        if (defined $::PRINT_XML_OUTPUT);

	bless $self, $class;

    $self->set_vfiler_server($args{vfiler}) if ($args{vfiler});

    $self->set_vserver($args{vserver});

    ## init values
    $self->reset_values();

    return $self;
}

##Function to fetch the Vserver's data LIF if USE_DATA_LIF parameter is set to 1 
## Parameter 1. Vserver whose Data lif Ip to be fetched
## 2. Node in which the vserver resides.

sub Fetch_data_lif{
    my $vserver_temp = shift;
    my $node_temp = shift;
    if (ref($node_temp) =~ /Node/) {
	print "Node Object passed\n";
    } else {
	# Get Node object if filer name passed.
	$node_temp = NACL::C::Node->new( name => $node_temp );
    }

    my @interface = NACL::STask::NetworkInterface->find(
    			command_interface => $node_temp,
			filter => {
			vserver => $vserver_temp
			    },
			);
    ## Fetch the first IP of the Vserver
    my $data_ip = $interface[0]->get_one_state_attribute('address');

    return $data_ip;
}
sub set_vserver {
    my $self = shift;
    my $vserver = shift;

    # input can be Vserver NACL object or vserver name
    if (ref($vserver) =~ /Vserver/i){
        $self->{vserver} = $vserver->vserver;
    } elsif ($vserver) {
        $self->{vserver} = $vserver;
    } else {
        $self->{vserver} = undef;
    }
    if ($vserver) {
        $self->set_vfiler_server($self->{vserver});
        # Workaround for cmode
        $self->set_legacy_http_conn(0);
    }
}

sub get_vserver {
    my $self = shift;
    return $self->{vserver};
}

#
# init/reset values
#
sub reset_values {
    my $self = shift;

    # our params
    $self->{volume} = undef;
    $self->{base_snapshot} = 'none';
    $self->{diff_snapshot} = undef;
    $self->{session_id} = undef;
    $self->{total_changes} = undef;
    $self->{total_duration} = undef;
	$self->{last_diff_duration} = 0;
    $self->{last_obj} = undef;
    $self->{xml_fh} = undef;
    $self->{objs} = undef;

}

sub wait_for_active_state {
    my $self = shift;
    my %args = validate_with (
                params => \@_,
                spec => {
                         session_id   => {type=>SCALAR, optional=>1},
                },);

    # Default use the session_id stored in the object,
    # If session_id is passed use it
    # Other wise croak
    my $session_id = ($args{session_id}) ? $args{session_id} :
                                ($self->{session_id}) ? $self->{session_id} :
                                        croak("No session id \n");

    my @params = ("snapdiff-iter-status", "session-id", $session_id);

    $Log->comment("snapdiff zapi params: @params") if ($self->{verbose});
    my $obj;
    my $sleep_time=1;
    while (1) {
        $obj = $self->invoke_api(@params);
        my $session_status = $obj->child_get_string("session-status");

        if ($session_status eq 'snapdiff_status_aborting') {
            if ($obj && $obj->results_ok()){
                $Log->comment('Session status aborting. Failure reason : ' .
                            $obj->child_get_string('failure-reason'));
            }
            return $obj;
        } elsif ($session_status ne 'snapdiff_status_pending') {
            return $obj;
        }
        $Log->comment("Snapdiff status still $session_status, " .
                      "Waiting for $sleep_time sec.");
        sleep($sleep_time++);
        # Total time waited cumulative sleep time (n*(n+1))/2
        if ((($sleep_time*($sleep_time + 1))/2) > $::MAX_WAIT_TIME_FOR_PENDING){
            croak('Max wait time for pending exceeded, waited for ' .
                        $::MAX_WAIT_TIME_FOR_PENDING);
        }
    }
    return $obj;
}

# This function returns file handle for writing 
# snapdiff zapi xml output.
# Try to create xml file in same place run log file located.
# by replacing .log file with $seq_no.xml 
# other wise create in TOP_LOGDIR 

sub get_xml_file {
    my $self = shift;
    # To get unique sequence no for each .xml file
    our $seq_no++;  

    my ($fh, $filename, $fno_file);

    # Use NATE Log filehandle to get full path.
    my $fno = fileno $NATE::Log::FH;

    if ($^O=~/linux/) {
        $fno_file = "/proc/$$/fd/$fno";
    } elsif ($^O =~ /solaris/) {
        $fno_file = "/proc/$$/path/$fno";
    }

    if (-l $fno_file) {
        $filename = readlink ($fno_file);
        $filename =~ s/\.log/_${seq_no}\.xml/;
    } else {
        $filename = "$::TOP_LOGDIR/snapdiff_$seq_no.xml";
    }

    $Log->comment("Snapdiff zapi xml output in file: $filename\n") 
        if ($self->{verbose});

    open $fh, ">>$filename" 
        or print "Failed open file: $filename $!\n";
    binmode $fh, ":utf8";
    return $fh;
}

#
# Calls three steps fo snapdiff zapi to fetch all diffs.
# . start session
# . fetch all changes using next
# . ends the session.
#
# Returns NaElement object of last zapi.
# If zapi call fails, reason and errno can be
# accessed with NaElement object methods
# . $obj->results_ok()
# . $obj->results_reson()
# . $obj->results_errno()
#
# Change records can get accessed using class methods
# get_diffs_hash() / get_diffs()
#
sub get_all_diffs {
    my $self = shift;
    my @params = @_;

    # see iter_start params
    # Params are same as iter_start.

    $Log->comment("In get_all_diffs") if ($self->{verbose});

    my $obj = $self->iter_start (@params);
    return $obj if (!$obj or !$obj->results_ok());

    $obj = $self->wait_for_active_state();
    return $obj if (!$obj or !$obj->results_ok());

    my $num_changes;
    my ($t1, $t2, $t3, $t4);
    do {
        $obj = $self->iter_next();
        $t1 = time();

        # check if iter_next taking more than 30 sec. log warning
        if ($self->{last_diff_duration} > ($::WARN_ITER_NEXT_TIMEOUT || 30)){
            $Log->warn("Time taken for iter next more than " .
                        "$::WARN_ITER_NEXT_TIMEOUT sec : " . 
                        $self->{last_diff_duration} );
        }

        return $obj if (!$obj or !$obj->results_ok());

        $num_changes = $obj->child_get_int("num-changes");
        $t2 = time();

        # Log error if num changes 0 even without dc switch 
        my $restart_cookie = $obj->child_get_string('restart-cookie');
        if ($self->{verbose}) {
            $Log->comment("Restart cookie : $restart_cookie") if ($restart_cookie);
        }

        # For pre SN.0 may not return end-of-diff
        if ($num_changes == 0) {
            my $end_of_diff = $obj->child_get_string('end-of-diff');

            # Release check needed, as Pre SN end-of-diff is not returned.
            if ($end_of_diff eq 'false' && !$restart_cookie) {
                $Log->warn("Zero diffs returned and ".
                    "no DC switch (i.e no restart cookie) or end-of-diffs ");
            } 
        }

        # $t3 = time();
        # $Log->debug(Dumper($obj));
        $t4 = time();
        if (($t4 - $t1) > 600) {
            $Log->warn("Client parsing took more than 10 min" .
                     " t1:$t1 t2:$t2 t3:$t3 t4:$t4");
        }
        $Log->comment("Num changes: $num_changes") if ($self->{verbose});

    } while (! $self->is_end_of_diff());

    $obj = $self->iter_end();


    return $obj;
}

sub is_end_of_diff {
    my $self = shift;
    my $obj = shift;

    # If no NaElement obj passed return session id
    if (!$obj) {
        if ($self->{last_obj}) {
            $obj = $self->{last_obj};
        } else {
            croak("No NaElement object passed or found in object\n");
        }
    }

    if (ref($obj) ne 'NaElement' ) {
        croak('Unexpected reference found,expected NaElement');
    }

    if ( !$obj->results_ok() ) {
        $Log->comment( 'The  Reason of  failure is :' .
                        $obj->results_reason() );
        return 0;
    } else {
        # From SN flag end-of-diff 'true' will be
        # returned for both cmode & 7mode, 
        # for backword compatibility check num-changes for 7Mode
        if ($self->{node}->mode() eq '7Mode'){
            $Log->comment('Using num-changes tag for end-of-diff')
                if ($self->{verbose} > 1);
            return ($obj->child_get_int('num-changes') == 0);
        } else {
            $Log->comment('Cmode check for end-of-diff true') if ($self->{verbose} > 1);
            return ($obj->child_get_string('end-of-diff') eq 'true');
        }
    }
}

#
# Snapdiff zapi start session call
# Returns NaElement object, and saves session id for future
# next, status and end calls
# new c-mode mandetary params
#    application-type can have any string(like: index)
#    application-name can have any string(like: snap-protect)
#
sub iter_start {
    my $self = shift;
    my %args = validate_with (
                params => \@_,
                spec => {
                         volume => {type=>SCALAR|HASHREF},
                         base_snapshot => {type=>SCALAR, optional =>1},
                         diff_snapshot => {type=>SCALAR},
                         max_diffs => {type=>SCALAR, default=> 256},
                         atime => {type=>SCALAR, default=> 'false'},
                         file_access_protocol => {type=>SCALAR, optional=>1},
                         'application-type' => {type=>SCALAR,
                                    optional=>1, default=> 'other'},
                         'application-name' => {type=>SCALAR,
                                    optional=>1, default=> 'other'},

                },
                allow_extra => 1,
                );

    # Reset attributes
    $self->reset_values();

    #print Dumper \%args;

    # snapdiff zapi output to xml file.
    if ($self->{print_xml_op}){
        $self->{xml_fh} = $self->get_xml_file();
    }

    # Volume can be NACL object / just volume  name
    if (ref($args{volume}) =~ /Volume/) {
        $self->{volume} = $args{volume}->volume;
    } else {
        $self->{volume} = $args{volume};
    }
    $self->{base_snapshot} = $args{base_snapshot} if ($args{base_snapshot});
    $self->{diff_snapshot} = $args{diff_snapshot};

    my @params = ('snapdiff-iter-start',
                   'volume', $args{volume},
                   'diff-snapshot', $args{diff_snapshot},
                   'max-diffs', $args{max_diffs},
                   'atime', $args{atime},);
    # Cmode only mandatory params
    if ($self->{node}->mode() eq '7Mode'){
        # Only for C-mode
        delete $args{'application-type'};
        delete $args{'application-name'};
    } else {
        use vars qw($INCL_APPLICATION_PARAM);
        param('INCL_APPLICATION_PARAM','-default', 1);
        if ($::INCL_APPLICATION_PARAM) {
            push @params, (
                    'application-type' => $args{'application-type'},
                    'application-name' => $args{'application-name'},
            );
        }
    }

    if (defined($args{base_snapshot})) {
        push @params, 'base-snapshot', $args{base_snapshot};
    }

    if (defined($args{file_access_protocol})) {
        $Log->debug("Using param file_access_protocol " . $args{file_access_protocol})
            if ($self->{verbose});
        push @params, 'file-access-protocol', $args{file_access_protocol};
    } elsif ($::FILE_ACCESS_PROTOCOL) {
        $Log->debug("Using FILE_ACCESS_PROTOCOL $::FILE_ACCESS_PROTOCOL")
            if ($self->{verbose});
        push @params, 'file-access-protocol', $::FILE_ACCESS_PROTOCOL;
    }

    if (defined($args{'include-file-attrs'})) {
        push @params, 'include-file-attrs', $args{'include-file-attrs'};
    }
    if (defined($args{'restart-cookie'})) {
        push @params, 'restart-cookie', $args{'restart-cookie'};
        $Log->comment( 'Restart cookie passed' . $args{'restart-cookie'})
            if ($self->{verbose});
    }

    $Log->comment("snapdiff zapi params: @params"); # if ($self->{verbose});
    my $xml_fh = $self->{xml_fh};
    print $xml_fh "@params" . "\n" if ($xml_fh);

    # Invoke snapdiff-iter-start zapi
    my $obj = $self->invoke_api(@params);
    if (!$obj->results_ok()) {
        $Log->comment('Failed to start snapdiff session '.
                                  $obj->results_reason() .
                           ' : ' . $obj->results_errno());
    }

    my $session_id = $self->get_session_id($obj);
    print $xml_fh $obj->sprint_pretty() if ($xml_fh);
    $Log->comment("snapdiff session id: $session_id") if ($self->{verbose});
    $self->{session_id} = $session_id;
    $self->{last_obj} = $obj;
    $self->{objs} = undef;

    return $obj;
}

#
# Return session id from given NaElement obj
# or previous saved session id of snapdiff start call
#
sub get_session_id {
    my $self = shift;
    my $obj = shift;

    # If no NaElement obj passed return session id
    if (!$obj) {
        if ($self->{session_id}) {
            return $self->{session_id};
        } else {
            croak("No NaElement object passed and No session id\n");
        }
    }

    if (ref($obj) ne 'NaElement' ) {
        croak("Unexpected reference found, expected NaElement not ref($obj).\n");
    }

    if ( !$obj->results_ok() ) {
        $Log->comment( 'The  Reason of  failure is :' .
                        $obj->results_reason() );
        return 0;
    } else {
        return $obj->child_get_string('session-id');
    }
}

#
# Snapdiff next iteration, to fetch next bunch of changes
#
sub iter_next {
    my $self = shift;
    my %args = validate_with (
                params => \@_,
                spec => {
                         session_id   => {type=>SCALAR, optional=>1},
                },);

    # Default use the session_id stored in the object,
    # If session_id is passed use it
    # Other wise croak
    my $session_id = ($args{session_id}) ? $args{session_id} :
                                ($self->{session_id}) ? $self->{session_id} :
                                        croak("No session id \n");

    my @params = ('snapdiff-iter-next', 'session-id', $session_id);

    $Log->comment("snapdiff zapi params: @params") if ($self->{verbose});
    my $xml_fh = $self->{xml_fh};
    print $xml_fh "@params" . "\n" if ($xml_fh);

    my $t1 = time();

	my $obj = $self->invoke_api(@params);

    my $t2 = time();

    print $xml_fh $obj->sprint_pretty() if ($xml_fh);

	$self->{last_diff_duration} = ($t2 - $t1);

    $self->{last_obj} = $obj;
    if ( !$obj->results_ok() ) {
        $Log->comment( 'The  Reason of  failure is :' .
                        $obj->results_reason() );
        return $obj;
    }

    push @{$self->{objs}}, $obj  
        unless ( defined($::SNAPDIFF_VERIFY) 
                 and ($::SNAPDIFF_VERIFY== 0));

	my $num_changes = $obj->child_get_int('num-changes');
	$Log->comment("snapdiff-iter-next returned $num_changes changes") 
        if ($self->{verbose});

	$self->{total_changes} += $num_changes;
	$self->{total_duration} += $self->{last_diff_duration}; 

    return $obj;
}


#
# Get Snapdiff session status
#
sub iter_status {
    my $self = shift;
    my %args = validate_with (
                params => \@_,
                spec => {
                         session_id   => {type=>SCALAR, optional=>1},
                },);

    # Default use the session_id stored in the object,
    # If session_id is passed use it
    # Other wise croak
    my $session_id = ($args{session_id}) ? $args{session_id} :
                                ($self->{session_id}) ? $self->{session_id} :
                                        croak("No session id \n");

    my @params = ('snapdiff-iter-status', 'session-id', $session_id);

    $Log->comment("snapdiff zapi params: @params") if ($self->{verbose});
    my $obj = $self->invoke_api(@params);

    my $xml_fh = $self->{xml_fh};
    print $xml_fh $obj->sprint_pretty() if ($xml_fh);

    if ($obj && !$obj->results_ok()) {
        $Log->comment('The  Reason of  failure is :' . $obj->results_reason());
    }

    return $obj;
}

#
# End Snapdiff Session
#
sub iter_end {
    my $self = shift;
    my %args = validate_with (
                params => \@_,
                spec => {
                         session_id   => {type=>SCALAR,optional=>1},
                },);

    # Default use the session_id stored in the object,
    # If session_id is passed use it
    # Other wise croak
    my $session_id = ($args{session_id}) ? $args{session_id} :
                                ($self->{session_id}) ? $self->{session_id} :
                                        croak("No session id \n");

    my @params = ('snapdiff-iter-end', 'session-id', $session_id);

    $Log->comment("snapdiff zapi params: @params") if ($self->{verbose});
    my $obj = $self->invoke_api(@params);

    my $xml_fh = $self->{xml_fh};
    if ($xml_fh){
        print $xml_fh $obj->sprint_pretty();
        # close xml output file
        close $xml_fh if ($xml_fh);
    }

    if ($obj && !$obj->results_ok()) {
        $Log->comment('The  Reason of  failure is :' . $obj->results_reason());
    }

    return $obj;
}

#
# Returns total num changes
#
sub get_total_changes {
    my $self = shift;
    return $self->{total_changes};
}

#
# Returns total time taken to get all diffs 
#
sub get_total_duration {
    my $self = shift;
    return $self->{total_duration};
}

#
# Return array of NaElement array
# which has all the change records.
#
sub get_objs {
    my $self = shift;
    return $self->{objs};
}

#
# Returns XML data,
#
sub get_hash {
    my $self = shift;
    my $XML_obj = new XML::Simple;
    my @c;
    foreach (@{$self->{objs}}) {
        push @c, $XML_obj->XMLin($_->toEncodedString());
    }
    return \@c;
}

#
# Returns array of change records hash
#
sub get_diffs_hash {
    my $self = shift;
    my @arr;
    my $arr_diff_hash = $self->get_hash();
    foreach my $diff_hash (@$arr_diff_hash) {

        # output can be array if more than one
        # or simple hash if only one element

        my $ch_rec_ref = $diff_hash->{'snapshot-changes'}->{'change-rec'};
        if ($ch_rec_ref){
            if (ref($ch_rec_ref) eq 'ARRAY') {
                push @arr, @$ch_rec_ref;
            } else {
                push @arr, $ch_rec_ref;
            }
        }
    }
    return \@arr;
}

# Prototops
sub nate_fail {
        my $msg = shift;
	Tharn::tstlog('-fail',$msg);
}

sub nate_log {
        my $msg = shift;
	Tharn::tstlog('-commline',$msg);
}

1;

__END__


SYNOPSYS:

use ZapiSD;

my $server = new ZapiSD(
                    filer=>$FILER,
                    username=>$USERNAME,
                    password=>$PASSWORD,
                    print_xml_op=>1,
            );

my $objs = $server->get_all_diffs(
            volume          => $VOLUME,
            base_snapshot   => $BS,
            diff_snapshot   => $DS,
            max_diffs       => $MAX_DIFFS,
         );

# To get array of all all change records hash
my $arr = $server->get_diffs_hash();
foreach $h (@$arr) {
    print $h->{filename} . "\n";
    print $h->{inode} . "\n";
}

## To get all NaElement objects of all iterations
foreach my $obj (@$objs) {
    if ($obj && $obj->results_ok()){
	    my $num_changes = $obj->child_get_int('num-changes');
        print "Num changes: $num_changes\n";
    }
}

# To get hash including num-changes of each iteration.
my $diff_hash = $server->get_hash();
foreach my $d_hash (@$diff_hash){
    print "--------\n";
    print Dumper $d_hash;
    print "--------\n";
}

print 'Total CHanges : '. $server->get_total_changes() ."\n";

# Notes
. All five calls return NeElement object.
   so that error can be checked.
. next, end, status should accept NeElement object as param.


# Todo
# flexibility in choosind ip & vserver at start time
# instead of new.
