#  $Id: //depot/prod/test/Rsierranevada/dp/snapdiff/zapi/lib/commonSetup.pm#1 $

#
# Common setup needed for most of the Snapdiff zapi test cases.
#

use Tharn;
use NACL::C::Vserver;
use NACL::STask::Node;
use NACL::STask::Aggregate;
use NACL::C::StorageDisk;
use NACL::C::VserverCifs;
use NACL::MTask::CIFS;
use NACL::STask::VolumeSnapshot;
use NACL::STask::Volume;
use NATE::BaseException qw{:try};
use NACL::Exceptions::NoElementsFound qw{:try};
use NACL::C::Exceptions::AlreadyExists qw{:try};
use Data::Dumper;
use Digest::SHA1 qw(sha1_hex);
use dp::snapdiff::zapi::lib::CommonParams;

# Add options
# Add vsadmin setup
use vars qw (
        $Test
        $Node
        $Volume
        $Aggregate
        $Vserver
        $is_repository
        $volume_created
        $aggregate_created
    );

#
# This routing initializes global variables with NACL objects, 
# which will be used by most of the test cases. 
# handles 7mode/cmode/infinivol
# . Find Vserver for cmode 
# . Create/reuse aggregate 
# . Create/reuse volume
# Todo: 
# . enable http service for vserver user cmode. 
# . enable options httpd.admin for 7mode 
# 

sub useCommonSetup {

    $Node = NACL::STask::Node->find();
    if (!$Node) {
        $Test->nlog("Unable to find Node");
        return $TCD::FAIL;
    }

    my $Mode = $Node->mode();

    # On Cmode Disable apache load balance till bug 585674 is fixed.
    apache_loadbalance(0) if ($Mode ne '7Mode');

    my $volume_name = $VOLUME || 'SD_' . int(rand(9999)) . '_volume';
    my $aggr_name   = $AGGR_NAME   || 'SD_' . int(rand(9999)) . '_aggregate';
    my $vol_path = "/vol/$volume_name";

    $vol_path = "/$volume_name" if ($Mode ne '7Mode');

    # Check if vserver is repository 
    if($Mode ne "7Mode")  {
        if ($VSERVER) {
            $Vserver       = NACL::C::Vserver->find(
                command_interface => $Node,
                canned_filters => [ qw(data_vservers) ],
                filter            => {  vserver => $VSERVER,
                                     },
                allow_empty       => 1,
            );
        } else {
            # If VSERVER not specified, pick a non repository vserver
            $Vserver       = NACL::C::Vserver->find(
                command_interface => $Node,
                canned_filters => [ qw(data_vservers) ],
                filter            => { 
                                        'is-repository' => 
                                            (($REPOSITORY) ? 'true' : 'false'),
                                     },
                allow_empty       => 1,
            );
        }
        if (!$Vserver) {
            $Test->nlog('Vserver not specified, Unable to find vserver!');
            return $TCD::FAIL;
        }

        if ($Vserver->get_one_state_attribute('is-repository') eq 'true'){
            $Test->nlog('Vserver is repository');
            $is_repository = 1;  
        }

        $Test->nlog('Using VServer : ' . $Vserver->vserver);

        # Get the infini volume exclude aggr lookup etc 
    }

    $Test->step('Create/fetch volume');

    # If volume specified then use it 
    if ($VOLUME) {
        try {
            $Volume = NACL::STask::Volume->find(
                    command_interface => $Node,
                                filter=> {
                                    volume => $volume_name,
                                    vserver => $Vserver,
                                },
                  );
        }
        catch NATE::BaseException with {
            my $eobj = shift;
            $Test->nlog("Volume $volume_name not found: " .
                        $eobj->text());
        };

        if ($Volume) {
            $Test->nlog('Using Volume : ' . $Volume->volume);
            return check_share();

        } elsif ($is_repository){
            $Test->nlog("Infinite volume $volume_name not found");
            return $TCD::FAIL;
        }
    } 

    # If infinite volume, fail if volume not found
    if ($is_repository){
        if (!$Volume) {
            $Volume = 
                NACL::STask::Volume->find(
                    command_interface => $Node,
                    filter => {
                                vserver => $Vserver,
                                'volume-style' => 'infinitevol',
                    },
                );
            if (!$Volume) {
                $Test->nlog("Unable to find infinite volume ".
                        "and no value passed for VOLUME param ");
                return $TCD::FAIL;
            }
        }
        $Test->nlog('Using Infinite Volume : ' . 
                    $Volume->volume());

        # Verify enable-snapdiff flag is set to 'true' other wise set it.
        if ($Volume->get_one_state_attribute('enable-snapdiff') ne 'true') {
            $Test->nlog('Volume flag enable-snapdiff to true');
            $Volume->modify('enable-snapdiff' => 'true');
        }

        return check_share();
    }

    # As volume not found and we need flex volume, create one.
     
    # First check and create aggr if not found 
    # todo pick if aggr not specified based on size
    try 
    {
        $Aggregate = 
            NACL::STask::Aggregate->find( 
                    command_interface => $Node,
                                filter=> {
                                    aggregate => $aggr_name,
                                },
                );
        $Log->comment('Using Aggregate ' . $Aggregate->aggregate);
    }
    catch NACL::Exceptions::NoElementsFound with {

        try {
            $Test->step('Create/fetch Aggregate');
            $Aggregate = NACL::STask::Aggregate->create(
                                aggregate => $aggr_name,
                        command_interface => $Node,
                                if_exists => 'reuse',
                             _was_created => \$aggr_created,
                            nacltask_wait => 1,
                                diskcount => 3,
                                 );

            if ($aggr_created){
                $Test->nlog('Aggregate ' . $Aggregate->aggregate .' created');
            } else {
                $Test->nlog('Using Aggregate ' . $Aggregate->aggregate);
            }
        }
        catch NATE::BaseException with {
            my $eobj = shift;
            $Test->nlog("Failed while creating Aggregate Error: " .
                        $eobj->text());
        };
    }
    catch NATE::BaseException with {
            my $eobj = shift;
            $Test->nlog("Failed while creating Aggregate Error: " .
                        $eobj->text());
    };

    die "Unable to find/create Aggregate : $aggr_name #@" 
        if (!$Aggregate);

    # Create flex volume

    try {
        $Volume  = NACL::STask::Volume->create(
            command_interface   => $Node,
            volume              => $volume_name,
            'junction-path'     => $vol_path,
            aggregate           => $Aggregate,
            vserver             => $Vserver,
            size                => $VOL_SIZE,
            'space-guarantee'   => 'none',
            'unix-permissions'  => '777',
            'security-style'    => 'mixed',
            if_exists           => 'reuse',
            _was_created        => \$volume_created,
        );

        if ($volume_created){
            $Test->nlog('Volume ' . $Volume->volume . ' created');
        } else {
            $Test->nlog('Using Volume ' . $Volume->volume . ' created');
        }
    }
    catch NATE::BaseException with {
        my $eobj = shift;
        $Test->nlog("Failed to create/fetch volume. Error: " .
                    $eobj->text());
        # Additional fail reason for bug# 571519 
        if ($eobj->text() =~ /File exists/) {
            my $apiSet = $Node->get_7m_or_nodescope_apiset();
            my $cmd1 = "priv set -q diag; vol_db lookup name $volume_name";
            my $out  = $apiSet->execute_command(command=>$cmd1);
            print Dumper $out;
            chomp $out;
            if ($out =~ /\[0x/) {
                my $cmd2 = "priv set -q diag; vol_db print $out";
                my $out1  = $apiSet->execute_command(command=>$cmd2);
                print Dumper $out1;
            }
        }
        return $TCD::FAIL;
    };

    die "Unable to create/fetch Volume : $volume_name #@" if (!$Volume);

    # Run exportfs for 7mode 

    if ( $Mode eq '7Mode' ) {
        # 7-mode set export flag anon=0
        my $apiset = $Node->get_7m_or_nodescope_apiset();
        $apiset->exportfs(
                          persistent => 1,
                'persistent-options' => 'sec=sys,rw,anon=0',
                                path => $vol_path,
        );
        # Enable unicode flag for 7mode 
        my $vol_cmd = "vol options " . $Volume->volume();
        $apiset->execute_command( command => 
                    "$vol_cmd create_ucode on; " . 
                    "$vol_cmd convert_ucode on"
                );
        # Set qtree security style to mixed
        my $qtree_cmd = "qtree security $vol_path mixed ";
        $apiset->execute_command( command => $qtree_cmd );
    }

    return check_share();
}

#
# check share exists for the volume
#
sub check_share {

    # Need to create share only for file-access-protocol is cifs/nfsv4
    # XXX if the client is windows, we need the share
    return $TCD::PASS 
        if ($::FILE_ACCESS_PROTOCOL eq "nfs");

    my $vserver = $Vserver->vserver() 
            if ($Node->mode() ne "7Mode");

    my $Share;

    my $junction_path = 
        $Volume->get_one_state_attribute('junction-path');

    $Test->step('Check and create CIFS share ' . $share );
    try {
        $Share = NACL::C::VserverCifsShare->find(
            'command_interface' => $Node,
            filter => {
                'vserver' => $vserver,
                path      => $junction_path,
            }
        );
    }
    catch NACL::Exceptions::NoElementsFound with {

        $Test->nlog("Check Cifs configured ");
        check_cifs_server();

        $Test->nlog("Creating Cifs share " . $share);
        try {
            $Share = NACL::C::VserverCifsShare->create(
                'command_interface' => $Node,
                   'share-name' => $Volume->volume(),
                         'path' => $junction_path,
                      'vserver' => $vserver,
            );
        } catch NACL::C::Exceptions::AlreadyExists with {
            $Log->comment("Cifs share already exists " . $share);
        };
    };

    if (!$Share) {
        $Test->nlog("Failed to create share $share" . $@);
        return $TCD::FAIL;
    }

    return $TCD::PASS;
}

#
# Check and create Cifs server
#
sub check_cifs_server {

    if ($Node->mode() eq "7Mode") {
        $Log->comment("Only for C-mode");
        return $TCD::NA;
    }

    my $vserver = $Vserver->vserver();
    if (!$vserver){
        $Test->nlog("Unable to get vserver");
        return $TCD::FAIL;
    }

    $Log->comment('Check and create Cifs server');
    my $Cifs_server;
    $Cifs_server = NACL::C::VserverCifs->find(
        command_interface => $Node,
        filter=> { vserver => $vserver, },
        allow_empty       => 1,
    );

    if ($Cifs_server){
        return $TCD::PASS;
    }

    $Test->nlog("Create cifs server for vserver "
            . $vserver);

    # Join dns 
    check_and_add_dns();

    my $nm = [
        {
            'position'    => 1,
            'direction'   => 'win-unix',
            'pattern'     => $::CIFS_DOMAIN . '\\\\'. $::CIFS_USER,
            'replacement' => "root",
        },
        {
            'position'    => 2,
            'direction'   => 'win-unix',
            'pattern'     => 'netapp\\\\(.+)',
            'replacement' => '\\1',
        },
        {
            'position'    => 1,
            'direction'   => 'unix-win',
            'pattern'     => "root",
            'replacement' => $::CIFS_DOMAIN . '\\\\'.  $::CIFS_USER
        },
        {
            'position'    => 2,
            'direction'   => 'unix-win',
            'pattern'     => '(.+)',
            'replacement' => 'netapp\\\\1',
        },
    ];

    my $cifs_server_name = 
        _generate_cifs_server_name(vserver_name => $vserver);

    $Cifs_server = NACL::MTask::CIFS->setup(
           'command_interface' => $Node,
           'vserver'           => $Vserver->vserver(),
           'if_exists'         => 'reuse',
           'cifs-server'       => $cifs_server_name, 
           'cifs_domain'       => $::CIFS_DOMAIN_LONG,
           'username'          => $::AD_SERVER_USER,
           'password'          => $::AD_SERVER_PASSWORD,
           'vserver_name_mapping' => $nm,
    );

    if ($Cifs_server) {
        return $TCD::PASS;
    } else {
        $Log->error("Failed to create cifs server");
        return $TCD::FAIL 
    }
}

sub get_sinfo {

    our %Siteinfo = ();
 
    my $file = $::SITE_FILE;
    my $code = NATE::Inc::get_file($file);
    if (!defined $code) {
        $Log->error("Could not find SITE_FILE $file");
    }
    eval $code;
    if ($@) {
        $Log->error("Could not eval SITE_FILE '$file': $@\n");
    }
    if (! keys %Siteinfo ) {
        $Log->error("SITE_FILE $file did not have a \%Siteinfo hash");
    }
    return %Siteinfo;
}

# Join the dns domain
sub check_and_add_dns {
    $Test->description("Adding $Site DNS to vserver");

    my $Dns = NACL::C::VserverServicesDns->find(
                    command_interface => $Node,
                    filter => { vserver => $VSERVER, },
                    allow_empty =>1,
              );
    return $TCD::PASS if ($Dns);

    $Log->comment("Configuring Dns service for vserver" );

    my @dns_servers;
    my @dns_domains;
    my %SiteData = get_sinfo();

    my $Site = ($::SITE) ? $::SITE : 
        ($Node->hostrec()->SITE()) ? $Node->hostrec()->SITE() : 'svl';

    if ($::DNS_SERVER_IP && $::DNS_DOMAIN) {
        $Test->nlog(
            'Overriding DNS Servers IP and DNS' .
                ' Domain Name provided by site data file'
        );
        @dns_servers = split(/,/, $::DNS_SERVER_IP);
        @dns_domains = split(/,/, $::DNS_DOMAIN);
    }
    else {
        @dns_servers = $SiteData{$Site}->{mgmt_dns1};
        $Log->comment("@dns_servers");
        if ( $SiteData{$Site}->{mgmt_dns2} ) {
            push( @dns_servers, $SiteData{$Site}->{mgmt_dns2} );
        }
        @dns_domains = split( /\s+/, $SiteData{$Site}->{dns_domains} );
        $Log->comment("@dns_domains");
    }
    NACL::C::VserverServicesDns->create(
        command_interface => $Node,
        'domains'         => \@dns_domains,
        'name-servers'    => \@dns_servers,
        'vserver'         => $Vserver,
    );
    return $TCD::PASS;
}

#
# Create snapshot and return VolumeSnapshot object
#
sub create_snapshot {
    my ($snap_name, $fetch) = @_;

    my $si;
    my %opts;
    $opts{vserver} = $Vserver->vserver() if ($Node->mode() ne '7Mode');
    $opts{volume} = $Volume->volume();
    
    if ($fetch) {
        $opts{snapshot} = $snap_name;
        $si = NACL::STask::VolumeSnapshot->find(
                  command_interface     => $Node,
                  filter => {
                        %opts,
                  }
              );
        if ($si) {
            $Test->nlog('Using Snapshot: ' . $si->snapshot());
        } else {
            $Log->error('Failed to fetch Snapshot: ' . $snap_name);
            return $si;
        }
    } else {
        $opts{foreground} = 'true';
        $opts{snapshot} = $snap_name || 
            'SD_'. $$ . int(rand(99999)) . '_snapshot';
        $si = NACL::STask::VolumeSnapshot->create(
                  command_interface     => $Node,
                  nacltask_if_exists    => 'reuse',
                  nacltask_wait         => 'true',
                  %opts,
              );
        $Test->nlog('Snapshot: '.$si->snapshot().' created');
    }


    return $si;
}

#
# As the burt 585674 fixed,
# this function will not be used. 
# Except one or two specific test 
# which needs apache load balance disabled.
#
sub apache_loadbalance {
    my ($enable) = @_;

    use vars qw($DISABLE_APACHE_LOADBALANCE);
    param('DISABLE_APACHE_LOADBALANCE', '-default' , 0);
    return unless($DISABLE_APACHE_LOADBALANCE);

    my @Nodes = NACL::C::Node->find();

    foreach my $Node (@Nodes)  {
        $Log->debug("Disable apache load balance on Node: " 
                            . $Node->node());
        my $apiset = NACL::APISet->new(
            hostobj     => $Node->hostrec(),
            category    => 'Node',
            interface   => 'CLI',
            set         => "Systemshell",
        );
        if (!$apiset){
            $Log->debug("Unable to get apiset for FreeBSD shell.");
            return;
        }
        # Command to query status
        # my $query_cmd = 'wget -qO- ' . 
        #     '" http://localhost/sys/config/cluster_proxy';
        my $cmd = 'wget -O- --post-data="' . "enabled=$enable" . 
                        '" http://localhost/sys/config/cluster_proxy';
        my $response = $apiset->execute_command(command => $cmd);
        if ($response =~ /Error:/){
            $Log->comment("Fail to disable appache loadbalance on Node: "
                           . $Node->node() .  ": " . $response); 
            $Log->debug(Dumper $response);
            return;
        } else {
            $Log->comment("Apache load balance disabled");
        }
    }
}

#
# This function sets zapi validation flags to false
# debug smdb table zapi_schema_validator modify 
# -validate-input-schema false 
# -validate-output-schema false 
# -validate-values false
#
# Set zapi validation flags to false,
# By default debug build will have this flag true. 
# while running 16 snapdiff zapi sessions.
# other wise we may get into this zapi related error:
#  :ZSM - Can't read reply header (line 1) (60)
# bug# 559704
    
sub set_zapi_validation_flags {

    # Only required for cluster mode 
    return if ($Node->mode() ne "CMode");

    # Return without error if not able to set the flags
    # due to no-debug build etc.
    my $apiset = NACL::APISet->new(
        hostobj     => $Node->hostrec(),
        category    => 'Node',
        interface   => 'CLI',
        set         => 'CMode',
    );
    if (!$apiset){
        $Log->debug("Unable to get apiset for checking zapi validation flags.");
        return;
    }

    my $response = $apiset->execute_command(command=>
                        'debug smdb table zapi_schema_validator show'
                   );
    if ($response =~ /Error:/){
        $Log->comment("Fail to access table: " . $response); 
        return;
    }

    my $delim = "X;X";
    if ($response =~ /true${delim}true${delim}true/){
        $Log->comment("Set zapi schema validation flags to false"); 
        $response = $apiset->execute_command(command=>
                        'debug smdb table zapi_schema_validator modify ' .
                        ' -validate-output-schema false ' .
                        ' -validate-values false',
                    );

        if ($response =~ /Error:/){
            $Log->comment("Fail to set zapi schema validation to false");
            return;
        }

        $Log->comment("Zapi schema validation set to false");
    }
}

#
# This function dumps the snapdiff zapi mhost and dblade counters 
# Returns both dblade and mhost counters
#
sub dump_sd_zapi_counters {
    my ($dump) = @_;

    # Only required for cluster mode 
    return if ($Node->mode() ne "CMode");

    # Return without error if not able to set the flags
    # due to no-debug build etc.
    my $apiset = NACL::APISet->new(
        hostobj     => $Node->hostrec(),
        category    => 'Node',
        interface   => 'CLI',
        set         => 'CMode',
    );
    if (!$apiset){
        $Log->debug("Unable to get apiset for checking zapi validation flags.");
        return;
    }

    my $resp = $apiset->statistics_v1_show(object=> 'snapdiff_zapi');
    my $dblade_counters = $resp->get_parsed_output();
    $resp = $apiset->statistics_v1_show(object=> 'snapdiff_zapi_mhost');
    my $mhost_counters = $resp->get_parsed_output();

    if ($dump) {
        $Log->comment("Dumping snapdiff zapi Dblade Counters : $cmd1");
        print Dumper $dblade_counters;
        $Log->comment("Dumping snapdiff zapi Mhost Counters : $cmd2");
        print Dumper $mhost_counters;
    }

    return ($dblade_counters, $mhost_counters);
}

#
# This function dumps the snapdiff zapi dblade mem stats 
# Returns mem_stats output.
#
sub dump_mem_stats {
    my ($dump) = @_;

    # Only required for cluster mode 
    return if ($Node->mode() ne "CMode");

    my $apiset = $Node->get_7m_or_nodescope_apiset();
    if (!$apiset){
        $Log->debug("Unable to get apiset for mem_stats dump");
        return;
    }
    $Test->step('dump mem stats');
    my $response = $apiset->execute_command(
                    command => 'priv set -q diag; mem_stats'
                );
    if ($dump) {
        print Dumper $response;
    }

    return $response;
}

#
# This function starts collecting packet trace 
# Input:
#   Flag to dump the output or not (default not)
#   Maximum size with unit (ex: 1g) to be collected by pktt 
# Currently this function is used by only SnapDiff-stress1.thpl 
# when Node not healthy error happens. 
#
sub pktt_start {
    my ($dump, $pktt_size) = @_;

    # Max pktt trace file size 
    $pktt_size = '1g' unless($pktt_size);

    # Only required for cluster mode 
    return if ($Node->mode() ne "CMode");

    my $apiset = $Node->get_7m_or_nodescope_apiset();
    if (!$apiset){
        $Log->debug("Unable to get apiset for packet trace i.e pktt start");
        return;
    }
    my $response = $apiset->execute_command(command=>
                                "priv set -q diag; pktt status e0a; pktt status e0b"
                   );
    if ($response =~ /Error:/){
        $Log->comment("Fail to get pktt status e0a/e0b : " . $response); 
        return;
    }

    my $delim = "X;X";
    if ($response =~ /Stopped writing trace/){
        $Log->comment("pktt is currently stopped writing trace." .
                     "will run pktt stop all, then pktt start"); 
        $response = $apiset->execute_command(command=>
                                "priv set -q diag; pktt stop all;");
    }elsif ($response =~ /Currently tracing/){
        $Log->comment("pktt is alreay tracing, returning!" . $response); 
        return $response;
    }
    $Test->step('start collecing cluster port pktt trace');
    $response = $apiset->execute_command(
                    command => "priv set -q diag; pktt status ;" .
                               "pktt start e0a -d / -m 128 -s $pktt_size ;" . 
                               "pktt start e0b -d / -m 128 -s $pktt_size ;" .
                               "pktt status ;"
                );
    if ($dump) {
        print Dumper $response;
    }

    return $response;
}

sub _generate_cifs_server_name {
    my %params = @_;
    my $cifsvsname;

    if (!$::CIFS_SERVER) {
        $cifsvsname = $Node->get_one_state_attribute('node');
        $cifsvsname .= $params{'vserver_name'};
        $cifsvsname =~ s/-|_//g;
        $cifsvsname = substr( sha1_hex("$cifsvsname"), 0, 15 )
            if (length($cifsvsname) > 15);
    } else {
        $cifsvsname = $::CIFS_SERVER;
    }

    $cifsvsname = uc $cifsvsname;
}
1;
