## @summary GNOTE::Jumpahead library
##
## @description
##		This library will provide jump ahead support for Paloma testing. 
## @dependencies
## 		use strict
##		use Carp
##		use Tharn
##		use base
##
## @author corina@netapp.com, dl-dpgqa-automation@netapp.com
## 

package dp::snapmirror::paloma::Jumpahead;

use strict;
use Carp;
use Tharn;
use GNOTE;
use GNOTE::FileSystem;
use Filesys::Tree qw/tree/;

use base qw( Exporter );

our @EXPORT_OK = qw(
			vol_read_access
			vol_write_access
			read_snapshot
			open_files
			read_inexistent_files
			try_read_no_r_permission
);

 
## @notes read ($vol); input : GNOTE volume object ; returns 0 when successful
# jard_1   Read access to destinations
sub vol_read_access {
	my ($vol) = @_;
	my $root = ""; 
	
	logcomment (" *** Vol read access");
	$root = $vol->nfs;
	$root->ls( '-laiR' );
	
	return 0;
}

## @notes write ($vol) ; input : GNOTE volume object ; returns 0 when successful
# jard-2  	Write access to destination is denied
sub vol_write_access {
	my ($vol) = @_;
	my $root = "";	
	my $file = "";

	logcomment (" *** Vol write access");
	eval {
        	$root = $vol->nfs;
	};
	if ($@) {
		my $error = $@;
		my $path = "";
		my $ip = "";
		my $volume = "";

		if ($error =~ m/(\/\w+\/(\d+.\d+.\d+.\d+)\/(\w+)\/\w+\/)/) {
			$path = $1;
			$ip = $2;
			$volume = $3;
			eval {
				system ("/usr/local/bin/sudo mkdir -p $path");
				my $mount_result = system ("/usr/local/bin/sudo mount $ip:/$volume $path");
				logcomment ("Successful mount ") if (0 == $mount_result);
			};
			warn $@ if $@;
			$file = $path . 'somefile_to_be_written';
			my $returned_error = system ("/usr/local/bin/sudo touch $file");
			if ( 256 == $returned_error) {
				logcomment (" PASS : Read-only file system ");
				return 0;
			} else { 
				return 1;
			}
		} else {
			warn "\n Can not find path in error ";
			return 1;
		}
		return 0;
	} else {
        	# try to "write" to destination through NFS
        	eval {
                	$file = $root->create( "somefile_to_be_written" );
        	};
        	logcomment (" $@ ") if $@;
	}

	return 0;
}

## @notes read_snapshot ($vol) ; input : GNOTE volume object ; returns 0 when successful
# jard-3  	Read access to destination snapshots
sub read_snapshot {
	my ($vol) = @_;
	my $root = "";	
	my $path = "";
	my @snapshots = ();
	
	logcomment (" *** Vol read snapshot");
	eval {
		# try and get the path by using Volume.pm, else 
		# mount the volume using the return error info 
		# and system call
		$root = $vol->nfs;
	};
	if ($@) {
		my $error = $@;
		my $ip = "";
                my $volume = "";

		# parse the error message and get the
		# 	1) path
		# 	2) IP address 
		# 	3) the volume name
		if ($error =~ m/(\/\w+\/(\d+.\d+.\d+.\d+)\/(\w+)\/\w+\/)/) {
			$path = $1;
			$ip = $2;
                        $volume = $3;
		} else {
                	warn "\n Can not find path in error ";
                	return 1;
        	}
		# mount the volume using system call
		# and log the successful result
		eval {
			system ("/usr/local/bin/sudo mkdir -p $path");
                        my $mount_result = system ("/usr/local/bin/sudo mount $ip:/$volume $path");
                        logcomment ("Successful mount ") if (0 == $mount_result);
                };
		warn $@ if $@;
	} else {
		# if Volume.pm mount succedded, then 
		# use NFS.pm for retrieveing the full path
		$path = $root->fullpath;
	} 
	$path = $path . '.snapshot';
	eval {
		system ("ls -la $path");
	};
	warn $@ if ($@); 
	# attempt to "write" to those snapshots should be denied with proper messages
	# get the list of snapshots
	@snapshots = `ls $path`;
	chomp (@snapshots);
	if ( 0 < scalar ( @snapshots) ) {
		eval {
			# use the first snapshot in the list to rename
			my $file_src = $path . '/' . $snapshots[0];
			my $file_dst = $path . '/' . $snapshots[0]. '_renamed';
			my $returned = rename $file_src, $file_dst;
			if (0 == $returned ) {
				logcomment ("Snapshot name could NOT be changed");	
				system ("ls -l $path");
			} else {
				# exit if the snapshot name was changed
				warn "Snapshot name WAS changed ! Terminating...";
				return 1;
			}
		};
		warn $@ if ($@);
	} else {
		logcomment ("There are no snapshots in the volume !!! we can not test jard-3");
		return 1;
	} 
	my $file = $path . '/' . $snapshots[0] . '/some_file_to_snapshot';
	# try to write to snapshot
	open(FILE, "+>> $file") || logcomment ("Couldn't open file: $file \n Could NOT write to .snapshot ==> PASS");
	close(FILE);
		
	return 0;
}

## @notes open_files ($src,$dst,$sm) ; input : GNOTE src and dst volume objects, GNOTE sm object ; returns 0 when successful, and 1 when not successfull.
# jard-4  	Client access to files that are not removed
# call this before Utils:: renaming_datasets - FIXED this
# since snapmirror->monitor doesn;t work properly because of the snapmirror show command (idle state not displayed)
# do not call this method right after a previous snapmirror->update, since this transfer will fail
# XXX to be fixed soon !
sub open_files {
	my ($src,$dst,$sm,%options) = @_;
	my $root_src = "";
	my $root_dst = "";
		
	my $path_src = "";
	my $path_dst = "";
	
	my $file_src = "";
	my $file_dst = "";
		
	my @files = ();
	my $file  = "";
		
	logcomment (" *** Vol open files");
	eval {
		$root_src = $src->nfs;
	};
	if ($@) {
		my $error = $@;
                my $path = "";
                my $ip = "";
                my $volume = "";

                if ($error =~ m/(\/\w+\/(\d+.\d+.\d+.\d+)\/(\w+)\/\w+\/)/) {
                        $path = $1;
                        $ip = $2;
                        $volume = $3;
			eval {
                                system ("/usr/local/bin/sudo mkdir -p $path");
                                my $mount_result = system ("/usr/local/bin/sudo mount $ip:/$volume $path");
                                logcomment ("Successful mount ") if (0 == $mount_result);
                        };
                        warn $@ if $@;
			$path_src = $path;
		} else {
			warn "\n Can not find path in error ";
                        return 1;
		}
	} else {
		$path_src = $root_src->fullpath . 'random_nfs';
	}

	my $tree = "";
	my $previous = "";
	my @dirs = ();
	my @files = ();

	do {
 		$tree = tree( { 'full' => 1 } , $path_src);
 		@dirs = keys %$tree; 
 		@files = values %$tree;
 		my @next = keys %{${$files[0]}{'contents'}};
 		
       		$previous = $path_src if ( -d $path_src ); 
		$file_src = $next[0] if ( -f $next[0] );
        	$path_src = $next[0];
	}while ( ${$files[0]}{'type'} =~ /d/ );
	$path_src = $previous;

	eval {
                $root_dst = $dst->nfs;
        };
        if ($@) {
		my $error = $@;
                my $path = "";
                my $ip = "";
                my $volume = "";

                if ($error =~ m/(\/\w+\/(\d+.\d+.\d+.\d+)\/(\w+)\/\w+\/)/) {
                        $path = $1;
                        $ip = $2;
                        $volume = $3;
                        eval {
                                system ("/usr/local/bin/sudo mkdir -p $path");
                                my $mount_result = system ("/usr/local/bin/sudo mount $ip:/$volume $path");
                                logcomment ("Successful mount ") if (0 == $mount_result);
                        };
                        warn $@ if $@;
			$path_dst = $path;
		 } else {
                        warn "\n Can not find path in error, $@ ";
                        return 1;
                 }
        } else {
                $path_dst = $root_dst->fullpath ;
        }
	my $iter = 0;
	chop($path_dst);
	$file_dst = $path_dst;
	foreach my $d ( split (/\//, $file_src) ) {
		$iter = $iter + 1;
		$file_dst = $file_dst . '/' . $d if ($iter > 6);
	}
	
	logcomment (" Using : $file_src \n $file_dst");
	if ( (!(-e $file_dst))) {
		warn "\n file 2 not on destination: $file_dst non-existent \n";
		return 1;
	}
	open(SRC_FILE, "+>> $file_src") || warn "Couldn't open src file: $file_src";
	# on the destination, open at least one file from NFS client for read
	open(DST_FILE, "$file_dst") || warn "Couldn't open dst file: $file_dst";
	# on the source, change some contents of those files being 
	# opened for read on destination
	my $file_position = sysseek (SRC_FILE, 0, 0);
	syswrite SRC_FILE, "zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz";

	$sm->update ( %options );
	$sm->update ( %options );

	close (SRC_FILE); 
	close (DST_FILE);
	# you need to close the application used for read and 
	# reopen and should see new content in those files
	open(DST_FILE, "$file_dst") || warn "Couldn't open file: $file_dst";
	my $found = 0;
	while ( defined (my $line  = <DST_FILE>)) {
		chomp $line;
		if ($line =~ /zzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzzz/){
			logcomment ("Src modification found on dst after sm update. Continuing...");
			$found = 1;
			last;
		}
	}
	warn "\n Did not find modification of src data on dst - Did UPDATE work?" if (!($found));
	close(DST_FILE);
	
	return 0;
}

## @notes read_inexistent_files ($src,$dst,$sm) ; input : GNOTE src and dst volume objects, GNOTE sm object ; returns 0 when successful
# jard-5   Client access to files that are removed

sub read_inexistent_files {
	my ($src,$dst,$sm,%options) = @_;
	my $root_src = "";
	my $root_dst = "";
		
	my $path_src = "";
	my $path_dst = "";
	
	my $file_src = "";
	my $file_dst = "";
		
	my @files = ();
	my $file  = "";
		
	logcomment (" *** Vol read inexistent files");
	$root_src = $src->nfs;
	$path_src = $root_src->fullpath . 'random_nfs';

	# getting one file for deletion from sourse
	my $tree = "";
        my $previous = "";
        my @dirs = ();
        my @files = ();

	do {
                $tree = tree( { 'full' => 1 } , $path_src);
                @dirs = keys %$tree;
                @files = values %$tree;
                my @next = keys %{${$files[0]}{'contents'}};

                $previous = $path_src if ( -d $path_src );
                $file_src = $next[0] if ( -f $next[0] );
                $path_src = $next[0];
        }while ( ${$files[0]}{'type'} =~ /d/ );
        $path_src = $previous;


	# make NFS clients access destination volume for reads
	eval {
		$root_dst = $dst->nfs;
	};
	if ($@) {
                my $error = $@;
                my $path = "";
                my $ip = "";
                my $volume = "";

                if ($error =~ m/(\/\w+\/(\d+.\d+.\d+.\d+)\/(\w+)\/\w+\/)/) {
                        $path = $1;
                        $ip = $2;
                        $volume = $3;
                        eval {
                                system ("/usr/local/bin/sudo mkdir -p $path");
                                my $mount_result = system ("/usr/local/bin/sudo mount $ip:/$volume $path");
                                logcomment ("Successful mount ") if (0 == $mount_result);
                        };
                        warn $@ if $@;
                        $path_dst = $path;
                 } else {
                        warn "\n Can not find path in error, $@ ";
                        return 1;
                 }
        } else {	
		$path_dst = $root_dst->fullpath . 'random_nfs';
	}

	my $iter = 0;
        chop($path_dst);
        $file_dst = $path_dst;
	
        foreach my $d ( split (/\//, $file_src) ) {
                $iter = $iter + 1;
                $file_dst = $file_dst . '/' . $d if ($iter > 6);
        }

	logcomment ("Using $file_src and $file_dst ");
	system (" ls -l $file_src ");
        system (" ls -l $file_dst ");


	if ( (!(-e $file_dst))) {
		warn "\n file 2 not on destination: $file_dst non-existent \n";
		return 1;
	}
	# on the destination, open at least one file from NFS client for read
	open(DST_FILE, "$file_dst") || warn "Couldn't open file: $file_dst";
	# one source, remove those files being opened for read on destination
	unlink ($file_src);
	# run update while the files remain opened for both CIFS and NFS clients

	$sm->update ( %options );

	# read something on the destination after the update and 
	# before the close and you should get stale file handle ???. 
	# you should handle this error properly.
	# the file may be cached in client's memory
	eval {
		while (<DST_FILE>) {
			my $line = $_;
		}
	};
	warn $@ if ($@);
	close (DST_FILE);
	open (DST_FILE, "$file_dst") || logcomment ("jard-5 : Couldn't open file: $file_dst on destination. 'snapmirror update' worked ==> PASS");
	close (DST_FILE);
	
	return 0;
}

## @notes try_read_no_r_permission ($src,$dst,$sm) ; input : GNOTE src and dst volume objects, GNOTE sm object ; returns 0 when successful
# jard-6  	(S) Client access to files whose permission get changed
sub try_read_no_r_permission {
	my ($src,$dst,$sm,%options) = @_;
	my $root_src = "";
	my $root_dst = "";
		
	my $path_src = "";
	my $path_dst = "";
	
	my $file_src = "";
	my $file_dst = "";
		
	my @files = ();
	my $file  = "";
		
	logcomment (" *** Vol read with no r parmission files");
	$root_src = $src->nfs;
	$path_src = $root_src->fullpath . 'random_nfs';

	# getting one file for deletion from sourse
        my $tree = "";
        my $previous = "";
        my @dirs = ();
        my @files = ();

        do {
                $tree = tree( { 'full' => 1 } , $path_src);
                @dirs = keys %$tree;
                @files = values %$tree;
                my @next = keys %{${$files[0]}{'contents'}};

                $previous = $path_src if ( -d $path_src );
                $file_src = $next[0] if ( -f $next[0] );
                $path_src = $next[0];
        }while ( ${$files[0]}{'type'} =~ /d/ );
        $path_src = $previous;

	# make NFS clients access destination volume for reads
	eval {
		$root_dst = $dst->nfs;
	};
	if ($@) {
                my $error = $@;
                my $path = "";
                my $ip = "";
                my $volume = "";

                if ($error =~ m/(\/\w+\/(\d+.\d+.\d+.\d+)\/(\w+)\/\w+\/)/) {
                        $path = $1;
                        $ip = $2;
                        $volume = $3;
                        eval {
                                system ("/usr/local/bin/sudo mkdir -p $path");
                                my $mount_result = system ("/usr/local/bin/sudo mount $ip:/$volume $path");
                                logcomment ("Successful mount ") if (0 == $mount_result);
                        };
                        warn $@ if $@;
                        $path_dst = $path;
                 } else {
                        warn "\n Can not find path in error, $@ ";
                        return 1;
                 }
        } else {
                $path_dst = $root_dst->fullpath . 'random_nfs';
        }
	
	my $iter = 0;
        chop($path_dst);
        $file_dst = $path_dst;

        foreach my $d ( split (/\//, $file_src) ) {
                $iter = $iter + 1;
                $file_dst = $file_dst . '/' . $d if ($iter > 6);
        }

        logcomment ("Using $file_src and $file_dst ");
        system (" ls -l $file_src ");
        system (" ls -l $file_dst ");

	if ( (!(-e $file_dst))) {
		warn "\n file 2 not on destination: $file_dst non-existent \n";
		return 1;
	}
	# on the destination, open at least one file from NFS client for read
	open(DST_FILE, "<$file_dst") || warn "Couldn't open file: $file_dst";
	# one source, remove read permissions on those files
	my $mode = 0222;   
	chmod $mode, $file_src;   
	# run update while the files remain opened for both CIFS and NFS clients

	$sm->update ( %options );

	# try to read it now
	eval {
		while (<DST_FILE>) {
			my $line = $_;
		}
	};
	warn $@ if ($@);
	close (DST_FILE);
	open (DST_FILE, "$file_dst") || logcomment ("jard-6 : Couldn't open file: $file_dst for reading on destination. 'snapmirror update' worked ==> PASS");
	close (DST_FILE);
	
	return 0;
}

1;
