# $Id: //depot/prod/test/Rsierranevada/dp/snapmirror/paloma/NACLwrapper.pm#1 $

# WARNING!  This is not a library, this is script extension!
#	    Call with 'require dp::snapmirror::paloma::NACLwrapper;'
#	    not with 'use dp::snapmirror::paloma::NACLwrapper;' !!!

#
## @summary  Not OOP NACL wrapper for rapid script development. Makes writing somewhat GNOTE like.
##
## @author   cpeter@netapp.com
##
## @status   private
##
## @pod      here

=head1 NAME

dp::snapmirror::paloma::NACLwrapper

=head1 DESCRIPTION

Rapid script development wrapper around set of NACL libraries.

=head2 Not a package!

This is not a library or package. This is truly a wrapper, an aggregation of
frequently used functions and methods in one place. This file must be called
with B<C<require>> statement instead of B<C<use>> statement.

=head2 Loaded libraries

Following libraries are loaded first. It means you do not have to re-declare them inside
your script thanks to missing B<C<package>> statement.

B<NACL::STask::Node>, B<NACL::STask::Cluster>, B<NACL::STask::Vserver>, B<NACL::STask::Aggregate>,
B<NACL::STask::Volume>, B<NACL::STask::VolumeEfficiency>, B<NACL::STask::VolumeSnapshot>,
B<NACL::CS::VolumeSnapshot>, B<NACL::STask::Snapmirror>,
B<NACL::C::SnapmirrorPolicy>, B<NACL::C::Client>, B<NACL::STask::Client::Mount>,
B<DataManip>, B<Data::Dumper>, B<TCD>

=head1 Functions

=cut

use Carp qw( confess );

use NACL::STask::Node;
use NACL::STask::Cluster;
use NACL::STask::Vserver;
use NACL::STask::Aggregate;

use NACL::STask::Volume;
use NACL::STask::VolumeEfficiency;
use NACL::STask::VolumeSnapshot;
use NACL::CS::VolumeSnapshot;
use NACL::STask::Snapmirror;
use NACL::C::SnapmirrorPolicy;

use NACL::C::Client;
use NACL::STask::Client::Mount;
use DataManip; # using Remi's DataManip library directly instead of 'NACL::APISet::Host::CLI::Linux::Redhat::populate()' DataManip wrapper

use Data::Dumper;   # these entries are

use TCD;

#-------------------------------------------------------------------------------------------------------------------------------------------
my @ObjStack;	# used for TearDown as LIFO stack
my $ObjStackPID = $$;
my $LogTimeStamp = time();
#-------------------------------------------------------------------------------------------------------------------------------------------

=head2 walk_param

Fancy wrapper around Tharn::param(). Search environment for most appropriate CLI parameters based on suffix, prefix and array of core values.

As for arguments, if first element is not an array reference then Tharn::param() is called as is. And if first element is an array reference then
elements of referenced array are used as parameter permutations. Permutations are combined with prefix and suffix if they are present.

Function returns found value as scalar or returns hash if array is wanted.

=over

=item Examples
Forward to Tharn::param() example:

 $filer = walk_param('FILER', '-mesg', 'Filer name is required'); # works exactly as Tharn::param();

Search for multiple possibilities with scalar return example:

 $filer = walk_param([qw(FILER NODE)], clisuffix=>[qw(SRC A)], clirequired=>1);	# this will cause to try multiple variables names in following order
										# FILER_SRC, FILER_A, FILERSRC, FILERA, NODE_SRC, NODE_A, NODESRC, NODEA, FILER, NODE
										# if all attempts have failed, then error message will be returned displaying all
										# permutations used.

Search for multiple possibilities with hash as returned value example:

 %filer = walk_param([qw(FILER NODE)], clisuffix=>[qw(SRC A)], clirequired=>1);	# this will cause to try multiple variables names in following order
										# FILER_SRC, FILER_A, FILERSRC, FILERA, NODE_SRC, NODE_A, NODESRC, NODEA, FILER, NODE
										# if all attempts have failed, then error message will be returned displaying all
										# permutations used.
Returned hash has following keys:

	cliprefix => "prefix part used when value was found",
	clicore => "core part, such as 'FILER' was used when value was found",
	clisuffix => "suffix part was used when value was found",
	param => "actual compiled parameter",
	value => "value returned by Tharn::param()"

Example of using parameters in parent/child form. Volume size of all volumes will fall to 50g instead of default 20g, accept for FILER_SRC_VOL2 it will be 100g

	foo.thpl FILER_SRC=box3070a NODEB=box3070b FILER_SRC_VOL1=hello FILER_SRC_VOL2=world NODEB_VOLUME1=saturn NODEB_VOLUME2=jupiter VOL_SIZE=50g FILER_SRC_VOL2_SIZE=100g

	#
	%filera = walk_params([qw(FILER NODE)], clisuffix=>[qw(SRC A)], clirequired=>1); # fails if nothing found
	%filerb = walk_params([qw(FILER NODE)], clisuffix=>[qw(DST B)], clirequired=>1);
	#
	%srcvol1 = walk_params(qw(VOL VOLUME)], cliprefix=>$filera{param}, clisuffix=>'1'); # use prefix from the parent! if parent is FILER_SRC
	%srcvol2 = walk_params(qw(VOL VOLUME)], cliprefix=>$filera{param}, clisuffix=>'2'); # then volume is FILER_SRC_VOL + its own suffix
	#
	%srcvol1size = walk_params(qw(VOL_SIZE VOLUME_SIZE SIZE)], cliprefix=>[$srcvol1{param},$filera{param}], clidefault=>'20g'); # use prefix from the parents! if parent is FILER_SRC_VOL1
	%srcvol2size = walk_params(qw(VOL_SIZE VOLUME_SIZE SIZE)], cliprefix=>[$srcvol2{param},$filera{param}], clidefault=>'20g'); # then parameter search is FILER_SRC_VOL1_SIZE, FILER_SRC_VOL_SIZE,
																    # and then VOL_SIZE
	#
	%dstvol1 = walk_params(qw(VOL VOLUME)], cliprefix=>$filerb{param}, clisuffix=>'1'); # cliprefix=>$filerb{param} - taking NODEB part for NODEB_VOLUME
	%dstvol2 = walk_params(qw(VOL VOLUME)], cliprefix=>$filerb{param}, clisuffix=>'2'); # from parent and the suffix part is our own
	#
	%srcvol1size = walk_params(qw(VOL_SIZE VOLUME_SIZE SIZE)], cliprefix=>[$dstvol1{param},$filerb{param}], clidefault=>'20g');
	%srcvol2size = walk_params(qw(VOL_SIZE VOLUME_SIZE SIZE)], cliprefix=>[$dstvol2{param},$filerb{param}], clidefault=>'20g');

=item Arguments

=over

=item B<first positional parameter> => I<array reference>

B<Required.> Array reference of core values, for example C<[qw/FILER NODE/]> or C<[VOL VOLUME]>. Note it must be a reference, not an array,
not a scalar, it must be a reference to an array! Values are case insensitive and converted to upper case internally. If first element is not a reference, then B<Tharn::param()> will be
called as is with all the arguments just passed through.

=item B<clisuffix> => I<array ref or scalar>

B<Optional.> String or array ref of suffixes to append to core value. For example cliprefix => [qw/SRC B 2/], in combination with core value
it will become C<FILER_SRC> or C<FILERB> etc.

=item B<cliprefix> => I<array ref or scalar>

B<Optional.> String or array ref of prefixes to prepend to core value. For example cliprefix => 'SRC', in combination with core value
it will become C<SRC_VOLUME>

=item B<clidebug> => I<boolean>

B<Optional.> Dumps all permutations into log file. This options can also be set by cli parameter CLIDEBUG=1

=item B<clirequired> => I<boolean>

B<Optional.> Fail if no parameters found. Also dumps all attempted permutations

=item B<clidefault> => I<string>

B<Optional.> Assign default value if nothing found. Function returns undef by default if nothing found for scalar or value => undef if array is wanted

=item B<clitag> => I<scalar>

B<Optional.> Case sensetive key to predefined array of suffixes and prefixes. Used to ease parameter search in bots and smokes testbeds across multiple
scripts. Some script writers may prefer to use FILER_SRC for source and other may prefer to use NODEA. Tag allows to aggregate such permutations in a
group so script will understand that FILER_SRS and NODEA is a same thing. These are predefined keys:

  'src' => [qw( SRC A  1 PRI PRIMARY    SOURCE )],
  'pri' => [qw( SRC A  1 PRI PRIMARY    SOURCE )],
  'dst' => [qw( DST B  2 SEC SECONDARY  DESTINATION )],
  'sec' => [qw( DST B  2 SEC SECONDARY  DESTINATION )],
  'ter' => [qw( CAS C  3 TER TERTIARY   CASCADE )],
  'cas' => [qw( CAS C  3 TER TERTIARY   CASCADE )],
  'qua' => [qw( CHN D  4 QUA QUATERNARY CHAIN )],
  'qui' => [qw(     E  5 QUI QUINARY )],
  'sen' => [qw(     F  6 SEN SENARY )],
  'sep' => [qw(     G  7 SEP SEPTENARY )],
  'oct' => [qw(     H  8 OCT OCTONARY )],
  'non' => [qw(     I  9 NON NONARY )],
  'den' => [qw(     J 10 DEN DENARY )]

clitag allows you to make simple call like

 walk_params([qw(FILER NODE)], clitag=>'src');

and function will search for FILER_SRC FILERA FILER1 FILER_PRI etc.

=back

=back

=cut

sub walk_param {
    return Tharn::param(@_) if ref( $_[0] ) ne 'ARRAY';

    my %cli_types = (   # order must be in a likelihood of usage
		    'src' => [qw( SRC A  1 PRI PRIMARY    SOURCE )],
		    'pri' => [qw( SRC A  1 PRI PRIMARY    SOURCE )],
		    'dst' => [qw( DST B  2 SEC SECONDARY  DESTINATION )],
		    'sec' => [qw( DST B  2 SEC SECONDARY  DESTINATION )],
		    'ter' => [qw( CAS C  3 TER TERTIARY   CASCADE )],
		    'cas' => [qw( CAS C  3 TER TERTIARY   CASCADE )],
		    'qua' => [qw( CHN D  4 QUA QUATERNARY CHAIN )],
		    'qui' => [qw(     E  5 QUI QUINARY )],
		    'sen' => [qw(     F  6 SEN SENARY )],
		    'sep' => [qw(     G  7 SEP SEPTENARY )],
		    'oct' => [qw(     H  8 OCT OCTONARY )],
		    'non' => [qw(     I  9 NON NONARY )],
		    'den' => [qw(     J 10 DEN DENARY )],
		);

    my @attempted;
    my @clicorenames = @{$_[0]};
    shift;
    my %args = @_;

    $args{clidebug} ||= param( 'CLIDEBUG', '-default', undef);

    # kind of "zippified", but it means try cliperfix first, if its array then use it if not then convert scalar to an array, if cliperfix is not given
	# then try to check clitag and run it against %cli_types predefined sequence, and if all fail then use empty string, so concatenation won't fail
	my @cliprefix = $args{cliprefix} ? ref($args{cliprefix}) eq 'ARRAY' ? @{$args{cliprefix}} : ( $args{cliprefix} ) : $cli_types{$args{clitag}} ? @{$cli_types{$args{clitag}}} : ( '' );
    my @clisuffix = $args{clisuffix} ? ref($args{clisuffix}) eq 'ARRAY' ? @{$args{clisuffix}} : ( $args{clisuffix} ) : $cli_types{$args{clitag}} ? @{$cli_types{$args{clitag}}} : ( '' );

    print "walk_cli_sequence: " . Dumper(\%args) . Dumper(\@clicorenames) if $args{clidebug};

    foreach my $clicore ( grep /\w+/, @clicorenames ) {
	$clicore = uc($clicore);
	foreach my $clisuffix ( @clisuffix ) {
	    foreach my $cliprefix ( @cliprefix ) {
		$clisuffix = uc($clisuffix);
		$cliprefix = uc($cliprefix);

		push @attempted, { cliprefix=>$cliprefix, clicore=>$clicore, clisuffix=>$clisuffix, param=>$cliprefix . '_' . $clicore . '_' . $clisuffix }; # PreFix + Core + Suffix
		push @attempted, { cliprefix=>$cliprefix, clicore=>$clicore, param=>$cliprefix . '_' . $clicore };					     # PreFix + Core
		push @attempted, { clicore=>$clicore, clisuffix=>$clisuffix, param=>$clicore . '_' . $clisuffix };					     # Core + Suffix

		push @attempted, { cliprefix=>$cliprefix, clicore=>$clicore, clisuffix=>$clisuffix, param=>$cliprefix . '_' . $clicore . $clisuffix };	     # PreFix Core Suffix, Suffix is without '_'

		push @attempted, { cliprefix=>$cliprefix, clicore=>$clicore, clisuffix=>$clisuffix, param=>$cliprefix . $clicore . $clisuffix };	     # PreFix Core Suffix without '_'
		push @attempted, { cliprefix=>$cliprefix, clicore=>$clicore, param=>$cliprefix . $clicore };						     # PreFix Core without '_'
		push @attempted, { clicore=>$clicore, clisuffix=>$clisuffix, param=>$clicore . $clisuffix };						     # Core Suffix without '_'

	    }
	}
    }
    foreach my $clicore ( grep /\w+/, @clicorenames ) { push @attempted, { clicore=>$clicore, param=>$clicore }; }					     # Only the core, must be strictly last!
    print Dumper(\@attempted) if $args{clidebug};
    # -----------------------------------------------------------------------------------------------------------------------------------

    foreach my $prmt (grep { defined } @attempted) { # loop through all permutations
	return wantarray ? %{$prmt} : $prmt->{value} if( $prmt->{value} = Tharn::param($prmt->{param}) );
    }

    if( $args{clirequired} ) {
		confess "Didn't find any of these CLI cliparameters: " . join(', ', map{ $_->{cliparam} } grep{ defined } @attempted );
    }

    return wantarray ? ( value => $args{clidefault} ) : $args{clidefault};
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 get_node

Connects to node and returns NACL::STask::Node object. There are no required arguments

Because function has no required parameters it tries to make sense from environment. The B<I<name>> argument is required by B<NACL::STask::Node>,
therefore function checks for B<I<name>> was passed in a first place and if B<I<name>>is not present then tries to check CLI for any useful node
or filer related parameter. It does so by calling B<walk_param( [qw/NODE FILER/], %args )>. All the I<clisuffix>, I<cliprefix>, I<clitag>,
I<clidefault> arguments are passed through as is with exception of I<clirequired>. That argument is set to B<1> unless I<clidefault> exists. So
function ether finds I<name> argument, finds it via CLI search, uses default value provided by script or fails.

Ones name is acquired its time to call B<NACL::STask::Node> constructor. Returned object will be passed as function's result.

Object enhancements:

If CLI was used to acquire node name, then hash from B<walk_param> is stored within node object itself under $node_obj->{_cli_ref} key. It can be later
used in order to find out what CLI parameter was used to retrieve the node name: $node_obj->{_cli_ref}->{param}.

Connects to node and retrieves cluster name. Cluster name is then stored under $node_obj->{_cluster}->{name}.

Connects to cluster and retrieves all member node names and stores them as array @{$node_obj->{_cluster}->{nodes}}.

Connect to cluster and retrieves all public vservers and stores them as list @{$node_obj->{_vserver_list}}.

=over

=item Arguments

See NACL::STask::Node->new for arguments

=item Examples

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $src_node = get_node( clitag => 'src' );
  4 my $dst_node = get_node( clitag => 'dst' );

=back

=cut

sub get_node {
    my %args = @_;
    my %cliout;

    unless( $args{name} ) {	# passed name has priority over CLI
	$args{clirequired} = 1 if $args{clidefault} !~ /\w/;
	%cliout = walk_param( [qw(NODE FILER)], %args );
	$args{name} = $cliout{value};
    }
    delete $args{$_} foreach( grep(/^cli/, keys %args) );   # remove all CLI related parameters

    $args{name} ||= $cliout{value};

    my $node_obj = NACL::STask::Node->new( %args ) || die "Unable to connect to filer\n";  # connect to node, dont like this error handling though
    $node_obj->{_cli_ref} = \%cliout;
    $node_obj->{_cluster}->{name} = NACL::STask::Cluster->identity( command_interface => $node_obj )->{name};
    @{$node_obj->{_cluster}->{nodes}} = NACL::STask::Cluster->nodes( command_interface => $node_obj );

    my @skipme = @{$node_obj->{_cluster}->{nodes}};
    push @skipme,$node_obj->{_cluster}->{name};
    my $skipRegEx = join('|', @skipme);
    @{$node_obj->{_vserver_list}} = grep( !/^($skipRegEx)$/, map{ $_->{vserver} } NACL::STask::Vserver->find( command_interface => $node_obj ));

    push @ObjStack, $node_obj;

    return $node_obj;
}

#-------------------------------------------------------------------------------------------------------------------------------------------
# saves logs from filer into $LOGRIR/$RUNSEQ_$TESTNAME/
sub _grab_logs {
    my $node = shift;
    eval {  # we must have eval so script won't fail if filer is down to to panic or whatever
        my $end_time = int( ( $LogTimeStamp - time() ) / 60 ) + 1;
        my $out  = $node->apiset(interface => 'CLI', set => 'Systemshell' )->execute_raw_command( command=>"find /mroot/etc/log -type f -cmin $end_time" );
        if( $ENV{LOGDIR_UNIX} && -d "$ENV{LOGDIR_UNIX}" ) { 
            my $logpath = $ENV{LOGDIR_UNIX} . '/' . $ENV{RUNSEQ} ;
            if($ENV{TESTNAME} && $ENV{TESTNAME} =~ /([^\/]+).thpl$/ ) { 
                $logpath .= '_' . $1; 
            } else {
                $logpath .= '_filer_logs';
            }   
            if( ! -d "$logpath" ) { 
                mkdir($logpath) || confess "Unable to create $logpath: $!";
            }   
            foreach(grep( /\w+/, split(/[\n\r]/, $out))) {
                (my $destname = $_ ) =~ s/\//_/g;
                $destname = "$logpath/".$node->name."$destname";
                my $data = $node->apiset(interface => 'CLI', set => 'Systemshell' )->execute_raw_command( command=>"cat $_" );
                open(FO,">$destname");
                print FO $data;
                close(FO);
            }   
        }   
    };  
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 get_aggregate

Creates new, find or create existing aggregate, recycles previously used aggregate. Returns NACL::STask::Aggregate object returned
by NACL::STask::Aggregate->find_or_create or NACL::STask::Aggregate->create.

Function uses only one mandatory argument, others are optional. Required argument is a NACL::STask::Node object created by B<get_node>
call. Its ok to use any NACL::STask::Node object, however CLI inheritance will be lost if object is not created by B<get_node>. Node
object is a first positional parameter. This is done with OOP transition to NACL::App layer in mind. OOP places first argument as
object all the time.

Aggregate is assumed to be recyclable if it starts with name AG_. To reuse recyclable aggregate function renames first two letters
from AG_ to ag_. This makes aggregate locked by script. When script exist it renames aggregate back to start with AG_ instead of
deleting aggregate itself.

If function is called with aggregate name ether by providing I<aggregate> parameter or via CLI search then aggregate is create if
its not present on the filer or its simply taken if already exists. No locking or renaming is performed for I<named> aggregate.
Aggregate is never deleted automatically. User must delete aggregate by force if required.

CLI search relies on node object as parent. It uses CLI parameter used to pull node name as its prefix unless I<cliprefix> argument
is given. If CLI parameter for parent was C<'NODEA'>, then search permutation will look like C<'NODEA_AGGR'>, C<'NODEA_AGGREGATE'>.
Actual search call looks like this: C<walk_param( [qw/AGGR AGGREGATE/], %args );>. See B<walk_param> description for more
info.

CLI search is also applied to disk count parameter. Actual call is

 walk_param( [qw(DISKS DISKCOUNT DISKCNT DSKCNT)], cliprefix=>[$cliout{param}, qw(AGGR AGGREGATE)], clidefault=>4 );

I<$cliout{param}> is actually parameter used to find aggregate name, such as 'NODEA_AGGR' to produce something like
'NODEA_AGGR_DSKCNT'.

All the walk_param's cli related arguments are stripped off before passing control over to NACL::STask::Aggregate.

There are few important things happens before NACL::STask::Aggregate object is returned. First - node object is stored as part of
aggregate object as $aggr_obj->{_node}. This allow future access for inherited field such as CLI of node instead of aggregate and
also opens access to cluster name, vserver list and nodes members of the cluster without actually calling of filer.

Next function adds vserver to aggregate. It checks I<nacltask_add_to_aggr_list_of_vserver> parameter and also I<vserver> parameter
first and if vserver is not given then it turns to parent node object to pull list of found public vservers and takes first
available one. It works in most cases when cluster has only one vserver. If cluster has two or more vserver then its job of the
script to provide one. Its probably good to check AGGR_VSERVER parameter, but its a thing of the future if needed. Vserver name
is stored internally as $aggr_obj->{_vserver}

CLI used to find aggregate name is also stored as $aggr_obj->{_cli_ref} hash reference.

Last step is to add newly created object to global LIFO object stack so it will get cleaned up at the end

=over

=item Arguments

See NACL::STask::Aggregate::find_or_create() and walk_param() for arguments

=item Examples

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $src_node = get_node( clitag => 'src' );
  4 my $src_aggr = get_aggregate( $src_node );

or

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $src_aggr = get_aggregate( get_node( clitag => 'src' ));

=back

=cut

sub get_aggregate {
    my ($node_obj,%args) = @_;

    $args{cliprefix} ||= $node_obj->{_cli_ref}->{param};    # use prefix from parent, like FILER or FILER_SRC
    my %cliout = walk_param( [qw(AGGR AGGREGATE)], %args );
    my $disks = delete $args{diskcount} || walk_param( [qw(DISKS DISKCOUNT DISKCNT DSKCNT)], cliprefix=>[$cliout{param}, qw(AGGR AGGREGATE)], clidefault=>4 );

    my $vserver = $args{nacltask_add_to_aggr_list_of_vserver} || $args{vserver} || $node_obj->{_vserver_list}->[0];
    delete $args{$_} foreach( grep(/^cli/, keys %args) );
    delete $args{$_} foreach( qw( nacltask_add_to_aggr_list_of_vserver vserver diskcount ) );

    my $aggr_obj = undef;
    $args{command_interface} = $node_obj;
    if( $args{aggregate} ) {										    # if $args{aggregate} then try it
	($aggr_obj) =  NACL::STask::Aggregate->find_or_create( %args, diskcount => $disks, nodes=> [ $node_obj->name ] );
    } else {												    # else check CLI and try CLI name
	$args{aggregate} = $cliout{value};
	if( $args{aggregate} ) {
	    ($aggr_obj) =  NACL::STask::Aggregate->find_or_create( %args, diskcount => $disks, nodes=> [ $node_obj->name ] );
	} else {											    # if CLI is not given then try recycled
	    delete $args{aggregate};
	    delete $args{allow_empty};

	    ( $aggr_obj ) = grep( $_->{aggregate} =~ /AG_/, NACL::STask::Aggregate->find( %args, filter=>{ nodes=> [ $node_obj->name ] }, allow_empty => 1 ));
	    unless( $aggr_obj ) {
		$args{aggregate} = $args{command_interface}->random_name_generator( prefix => 'ag' );
		($aggr_obj) = NACL::STask::Aggregate->create( %args, diskcount => $disks, nodes=> [ $node_obj->name ] );
	    }
	}
    }
    sleep(5) && logcomment( "Waiting for online state...") while( $aggr_obj->state()->{'state'} !~ /online/ );  # this one is ugly
    if( $aggr_obj->{aggregate} =~ /^AG_/ ) {
	( my $aggr_name = $aggr_obj->{aggregate} ) =~ s/^AG_/ag_/;
	$aggr_obj->rename( newname=>$aggr_name );
	sleep(5) && logcomment( "Waiting for online state...") while( $aggr_obj->state()->{'state'} !~ /online/ );  # this one is ugly
    }
    $aggr_obj->{_cli_ref} = \%cliout;
    $aggr_obj->{_node} = $node_obj;
    if( $vserver ) {	# add vserver here, because find() will fail if you give "nacl_stask_add_to_aggr_list_of_vserver => $vserver"
	$aggr_obj->add_to_aggr_list_of_vserver( vserver => $vserver );
	$aggr_obj->{_vserver} = $vserver;
    }

    push @ObjStack, $aggr_obj;
    return $aggr_obj;
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 delete_aggregate

Delete / Return to recycle pool.

=over

=item Arguments

Single positional parameter - NACL::STask::Aggregate object.

=back

=cut

sub delete_aggregate {
    my $aggr_obj = shift;
    while( pop @{$aggr_obj->{_children}} ) {
	$_->purge if ref($_);
    }

    if( $aggr_obj->{aggregate} =~ /^ag_/ ) {
	( my $aggr_name = $aggr_obj->{aggregate} ) =~ s/^ag_/AG_/;
	$aggr_obj->rename( newname=>$aggr_name );
    } else {
	$aggr_obj->offline;
	$aggr_obj->delete;
    }
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 get_volume

Creates and returns NACL::STask::Volume object by calling NACL::STask::Volume->find_or_create.

Function has only one required argument - NACL::STask::Aggregate object, others are optional. Optional arguments are acquired by inheritance
and from CLI by calling walk_param( [qw(VOL VOLUME)],...);

Beside creation of the object function stores extra info is stored inside the object and object itself is referenced in the global LIFO stack
for proper clean up.

Parent aggregate object is stored as $vol_obj->{_aggregate}. This is NACL::STask::Aggregate object.

Parent node object is stored as $vol_obj->{_node}. This is NACL::STask::Node object.

Name of the vserver is stored as $vol_obj->{_vserver}. Note, this is just a name.

Efficiency object is stored as $vol_obj->{_efficiency} if efficiency is enabled. Its NACL::STask::VolumeEfficiency object and all methods
can be used with it. For examle its possible to call resed as $vol_obj->{_efficiency}->reset();

CLI hash used to fetch volume name is stored as $vol_obj->{_cli_ref}.

B<NOTE:> Volume is always get destroyed when script exits. This is subject to change.

=over

=item Arguments

=over

=item B<volume> => I<scalar>

B<Optional.> Volume name to search for or create. If argument is not given then CLI search is performed. Prefix for CLI parameter is taken from both
aggregate and node. Node object is pulled from aggregate object. All parameter used by walk_param function can be used to pull CLI.

=item B<size> => I<scalar>

B<Optional.> Size of the volume. If size is not given then CLI is searched. CLI prefix is constructed using volume name parameter as parent and also
node name parameter. Aggregate is ignored. It means FILER_SRC_VOLUME may be taken, but FILER_SRC_AGGR_VOLUME will be ignored. We can change this
order upon request. If search is not successful then default value of 20g is used. Setting CLI like VOL_SIZE=50g will effectively change force
defaults to 50g for all volumes.

=item B<efficiency> => I<boolean>

B<Optional.> Enable storage efficiency if provided. If parameter is not given then CLI search is performed just in case. The core array for
variable construction is 'SE', 'VOLUME_SE'. Volume and node CLI is used as parent making it as FILER_SRC_VOLUME_SE. Because node is used as
parent its possible to set something like FILER_SRC_SE=1 to enable efficiency on all source volumes when used within same filer.

B<NOTE:> All the arguments used by NACL::STask::Volume->find_or_create are permitted.

=item Examples

Somewhat minimalistic:

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $node = get_node( clitag => 'src' ); # FILER_SRC FILERA NODEA NODE_SRC etc
  4 my $aggr = get_aggregate( $node );      # FILER_SRC_AGGR FILER_AGGR AGGR etc
  5 my $volm = get_volume( $aggr );         # FILER_SRC_VOLm FILER_SRC_VOL_SIZE FILER_SRC_VOL_SE etc

Smaller:

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $aggr = get_aggregate( get_node( clitag => 'src' ) );
  4 my $volm = get_volume( $aggr );

Even smaller:

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $volm = get_volume( get_aggregate( get_node( clitag => 'src' ) ) );

Working with parents (aggregate, node) and child (efficiency) objects:

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $volm = get_volume( get_aggregate( get_node( clitag => 'src' ) ), efficiency=>1 );
  4
  5 print "Nodes on current cluster are: " . join(' ', @{$volm->{_aggregate}->{_node}->{_cluster}->{nodes}}) . "\n";    # pull node from parents!
  6 $volm->{_efficiency}->start();                                                                                      # start efficiency

=back

=back

=cut

sub get_volume {
    my( $aggr_obj, %args ) = @_;
    my $node_obj = $aggr_obj->{_node};
    my %cliout = walk_param( [qw(VOL VOLUME)], cliprefix => $args{cliprefix} || [ $aggr_obj->{_cli_ref}->{param}, $node_obj->{_cli_ref}->{param} ], %args );

    $args{volume} ||= $cliout{value} || $aggr_obj->random_name_generator( prefix => 'vl' );
    $args{size} ||= walk_param(
				[ qw( SIZE VOL_SIZE VOLUME_SIZE ) ],
				cliprefix => [ $cliout{param}, $node_obj->{_cli_ref}->{param}, qw(VOL VOLUME) ],
				clisuffix => $args{clisuffix},
				clidefault => '20g'
			    );

    my $efficiency = delete $args{efficiency} || walk_param(
							    [ qw( SE VOLUME_SE ) ],
							    cliprefix => [ $cliout{param}, $node_obj->{_cli_ref}->{param}, qw(VOL VOLUME) ],
							    clisuffix => $args{clisuffix},
							);

    $args{command_interface} ||= $node_obj;
    $args{aggregate} ||= $aggr_obj->{aggregate};
    $args{vserver} ||= $aggr_obj->{_vserver};

    delete $args{$_} foreach( grep(/^cli/, keys %args) );
    my $vol_obj = NACL::STask::Volume->find_or_create( %args );

    $vol_obj->{_aggregate} = $aggr_obj;
    $vol_obj->{_vserver} = $args{vserver};
    $vol_obj->{_node} = $node_obj;
    $vol_obj->{_cli_ref} = \%cliout;

    if( $efficiency ) {
	$vol_obj->{_efficiency} = NACL::STask::VolumeEfficiency->on(
					    command_interface => $node_obj,
					    vserver => $args{vserver},
					    volume => $vol_obj->{volume},
					    'method-timeout' => 180
					);
    }

    push @ObjStack, $vol_obj;
    return $vol_obj;
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2	snapmirror_create

Creates and returns NACL::STask::Snapmirror object using two NACL::STask::Volume objects. Most of the required information
is taken from these objects.

Function can optionally create new policy and add new rule if required

Extended functionality:

Source volume object can be also accessed as  $sm_obj->{_src}

Destination volume object can be also accessed as  $sm_obj->{_dst}

Snapshot label for created policy can be accessed as $sm_obj->{_policy}->{snapshot_label}

Policy object of type NACL::C::SnapmirrorPolicy is accessible via $sm_obj->{_policy}->{policy_obj}

Generated policy name is accessible via $sm_obj->{_policy}->{name}.

Newly create NACL::STask::Snapmirror object is referenced in global LIFO stack for proper teardown.

=over

=item Arguments

=over

=item B<src> => I<NACL::STask::Volume>

B<Required.>	Source volume object


=item B<dst> => I<NACL::STask::Volume>

B<Required.>	Destination volume object

=item B<type> => I<scalar>

B<Optional.> Mirror type, 'dp' or 'xdp'. If type is not given then CLI search is performed as walk_param( [qw(MIRROR_TYPE)], %args ). All walk_param arguments
are ok to be used together with parameters for NACL::STask::Snapmirror.

=item B<do_initialize> => I<boolean>

B<Optional.> Initialize after create. Default is B<1>.

=item B<generate_policy> => I<boolean>

B<Optional.> Create new policy and add new rule. Uses random name for policy name unless I<policy> argument is given together with I<generate_policyi=1>.
Policy snapshot label is also generated and stored internally.

=back

=item Examples

Minimalistic example:

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $smobj = snapmirror_create(
  4                     src => get_volume( get_aggregate( get_node( clitag => 'src' ) ), type=>'rw' ),
  5                     dst => get_volume( get_aggregate( get_node( clitag => 'dst' ) ), type=>'dp' ),
  6                 );

=back

=cut

sub snapmirror_create() {
    my (%args) = @_;
    my (
	$src_volume_obj, $dst_volume_obj, $sm_obj,
	$policy_name, $policy_snapshot_label, $policy_obj,
       );

    $src_volume_obj = delete $args{src};
    $dst_volume_obj = delete $args{dst};

    $args{clidefault} = 'xdp';
    $args{type} ||= walk_param( [qw(MIRROR_TYPE)], %args );
    delete $args{$_} foreach( grep(/^cli/, keys %args) );

    if( delete $args{generate_policy} ) {
	$policy_name = $dst_volume_obj->random_name_generator( prefix => 'plcy' );
	$policy_snapshot_label = substr( $dst_volume_obj->random_name_generator( prefix => 'sn_lbl' ), 0, 30 );	# size is 1 .. 31

	$policy_obj = NACL::C::SnapmirrorPolicy->create(
			command_interface   => $dst_volume_obj->{command_interface},
			vserver		    => $dst_volume_obj->state->{vserver},
			policy		    => $policy_name,
		    );
	push @ObjStack, $policy_obj;

	$policy_obj->add_rule(
			    'snapmirror-label'	=> $policy_snapshot_label,
			    'keep'		=> 251			    # 254 doesn't work anymore!, 253 does ether...
			);
	$args{policy} = $policy_name;
    }

    $args{type} ||= param('MIRROR_TYPE', '-default', 'XDP');
    $args{do_initialize} = 1 unless exists($args{do_initialize});

    $sm_obj = NACL::STask::Snapmirror->create(
        command_interface  => $dst_volume_obj->{command_interface},
        'source-path'      => $src_volume_obj,
        'destination-path' => $dst_volume_obj,
        nacltask_wait      => 1,
	%args
    );

    $sm_obj->{_src} = $src_volume_obj;
    $sm_obj->{_dst} = $dst_volume_obj;

    push @ObjStack, $sm_obj;

    if( ref($policy_obj) ) {
	$sm_obj->{_policy}->{snapshot_label} = $policy_snapshot_label;
	$sm_obj->{_policy}->{policy_obj}     = $policy_obj;
	$sm_obj->{_policy}->{name}	     = $policy_name;
    }

    return $sm_obj;
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 data_works

Generate data on volume from client. At the present time a DataManip is supported.

Also enhances volume properties in following way:

Ones client connection is established $vol_obj->{clients} hash is created and can be used later.

Each client will have its own record as $vol_obj->{clients}->{$client_name}

Mount object and client object are stored accordingly as $vol_obj->{clients}->{$client_name}->{_mount_obj}

Client object is stored as $vol_obj->{clients}->{$client_name}->{_client_obj} and can be used later to perform
operations. See example section for details.

=over

=item Arguments

=over

=item B<volume> => I<NACL::STask::Volume>

B<Required>

=item B<client> => I<scalar>

B<Optional> Host name of client. If not defined then is taken from CLI with core name CLIENT and prefix is taken from volume. Both parent object including
volume and node provide their CLI variables as prefix. As example if node had variable NODEA and volume inherited name as NODEA_VOL then search sequence
will look like NODEA_VOL_CLIENT -> NODEA_CLIENT -> CLIENT. If all attempts fail then default value for hostname is 'localhost'.

=item B<dataset> => I<scalar>

B<Optional> Dataset name key. There are currently only two datasets are stored within a library itself. One of them is named I<'default'> and another is named
I<'simulator'>. Dataset uses CLIENT's CLI variable as parent during CLI search. So if variable for client found to be FILER_SRC_VOL1_CLIENT then
function will attempt to search for FILER_SRC_VOL1_CLIENT_DATASET before rolling to FILER_SRC_DATASET and finally to DATASET. If no dataset argument was given
and CLI search returned nothing then value is set to I<'default'>.

=item B<dataset_file> => I<scalar>

B<Optional> Name of the dataset file. Used to expand stored dataset definition from the file. If argument is not provided then CLI search is attempted. Search
logic is similar to I<dataset> with a difference that its not a DATASET used as a core but DATASET_FILE.

File structure is similar to structure of NATE I<.ntest> file accept its
enhanced to include sections. Section from section is divided by dataset name in square brackets. Here is example of content of dataset file:

 # well, I guess some comments are ok...
 # and leading spaces also!
    [apple]	# Dataset for MacOS 10
	BREADTH_AVG="60416" # have not idea what is this
	BREADTH_DEV="2048"
	CREATE_DIR_AVG="0"
	CREATE_DIR_DEV="0"
	...
    [dpg_4tb]
	BREADTH_AVG="146243"
	BREADTH_DEV="48747"
	CREATE_DIR_AVG="0"
	CREATE_DIR_DEV="0.30303"
	CREATE_FILE_AVG="3"
	CREATE_FILE_SIZE_AVG="146243"
	...
    [simulator]
	BREADTH_AVG="16384"
	BREADTH_DEV="4096"
	CREATE_DIR_AVG="0.3"

Its permitted to have no dataset names at all, in which case all of the found values are stored in default section. Please refer to lib/DataManip/defaults.pm for information
how to construct your own dataset.

=back

=item Examples

Basics:

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $aggr = get_aggregate( get_node( clitag => 'src' ) );
  4 my $volm = get_volume( $aggr );
  5 data_works( volume=>$volm, dataset=>'apple' );

Using enhancements:

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $aggr = get_aggregate( get_node( clitag => 'src' ) );
  4 my $volm = get_volume( $aggr );
  5 data_works( volume=>$volm, dataset=>'apple', client => 'dpg-35h' );
  6 #
  7 $volm->{_clients}->{'dpg-35h'}->{_client_obj}->apiset->execute_raw_command(command => 'sudo reboot');

=back


=cut

sub data_works {
    my %args = @_;

    my $vol_obj = $args{volume};
    my %cliout = walk_param(
				[ qw( CLIENT ) ],
				cliprefix => [ $vol_obj->{_cli_ref}->{param}, $vol_obj->{_node}->{_cli_ref}->{param} ],
				clisuffix => [qw(UNIX LINUX NFS)],
				clidefault => 'localhost'
			    );

    my $client_name = $args{client} || $cliout{value};

    unless( defined($vol_obj->{_clients}->{$client_name}->{_client_obj}) && ref( $vol_obj->{_clients}->{$client_name}->{_client_obj} ) eq 'NACL::C::Client' ) {
	$vol_obj->{_clients}->{$client_name}->{_client_obj} = NACL::C::Client->new(name =>$client_name);
    }

    unless( defined($vol_obj->{_clients}->{$client_name}->{_mount_obj}) && ref( $vol_obj->{_clients}->{$client_name}->{_mount_obj} ) eq 'NACL::STask::Client::Mount' ) {
	$vol_obj->{_clients}->{$client_name}->{_mount_obj} =  NACL::STask::Client::Mount->create( command_interface => $vol_obj->{_clients}->{$client_name}->{_client_obj}, volume => $vol_obj );
	push @ObjStack,$vol_obj->{_clients}->{$client_name}->{_mount_obj};
	$vol_obj->{_clients}->{$client_name}->{_client_obj}->apiset->execute_raw_command(command =>'/usr/software/bin/sudo chmod 777 ' . $vol_obj->{_clients}->{$client_name}->{_mount_obj}->mount_point());

	$vol_obj->{_clients}->{$client_name}->{_datasets} = {
	    'default' => {
		'TOTAL_MOD' => '0.5',
		'GROW_RAW' => '0',
		'SCRAMBLE' => '0',
		'MOD_RAW' => '0',
		'SUBTREE_DEPTH_AVG' => '3.5',
		'CREATE_FILE_SIZE_DEV' => 4096,
		'SHRINK_AVG' => '0.05',
		'DEPTH_BASE' => '0',
		'CREATE_DIR_AVG' => '0.3',
		'CREATE_FILE_AVG' => '0.5',
		'TOTAL_SUBTREE_DEL' => '0.075',
		'TOTAL_SHRINK' => '0.1',
		'FRAGMENT_DEV' => '0',
		'DEPTH_DEV' => 1,
		'FLOOD' => '0',
		'VERBOSE' => 4,
		'BREADTH_DEV' => 4096,
		'FORK_COUNT' => '0',
		'GROW_AVG' => '0.05',
		'DIR_AVG' => '3.2',
		'MOD_AVG' => '0.25',
		'FRAGMENT_RAW' => '0',
		'FILE_DEV' => 2,
		'TOTAL_GROW' => '0.1',
		'CREATE_FILE_SIZE_AVG' => 8192,
		'SUBTREE_DEPTH_DEV' => '0.25',
		'SHRINK_DEV' => '0.0066666',
		'CREATE_DIR_DEV' => '0.0833333',
		'CREATE_FILE_DEV' => '0.15',
		'FRAGMENT_AVG' => 1,
		'PERSEVERE' => '0',
		'DEPTH_AVG' => '4.3',
		'BREADTH_AVG' => 16384,
		'SHRINK_RAW' => '0',
		'TOTAL_DEL' => '0.125',
		'MIN_DIR_SIZE' => '0',
		'FILE_AVG' => 4,
		'DIR_DEV' => 1,
		'GROW_DEV' => '0.0066666',
		'MOD_DEV' => '0'
	    },
	'simulator' => {
		'BREADTH_AVG' => '16384',
		'BREADTH_DEV' => '4096',
		'CREATE_DIR_AVG' => '0.3',
		'CREATE_DIR_DEV' => '0.0833333',
		'CREATE_FILE_AVG' => '0.5',
		'CREATE_FILE_DEV' => '0.15',
		'CREATE_FILE_SIZE_AVG' => '8192',
		'CREATE_FILE_SIZE_DEV' => '4096',
		'DEPTH_AVG' => '4.3',
		'DEPTH_BASE' => '0',
		'DEPTH_DEV' => '1',
		'DIR_AVG' => '3.2',
		'DIR_DEV' => '1',
		'FILE_AVG' => '4',
		'FILE_DEV' => '2',
		'FLOOD' => '0',
		'FORK_COUNT' => '0',
		'FRAGMENT_AVG' => '1',
		'FRAGMENT_DEV' => '0',
		'FRAGMENT_RAW' => '0',
		'GROW_AVG' => '0.05',
		'GROW_DEV' => '0.0066666',
		'GROW_RAW' => '0',
		'MIN_DIR_SIZE' => '0',
		'MOD_AVG' => '0.25',
		'MOD_DEV' => '0',
		'MOD_RAW' => '0',
		'PERSEVERE' => '0',
		'SCRAMBLE' => '0',
		'SHRINK_AVG' => '0.05',
		'SHRINK_DEV' => '0.0066666',
		'SHRINK_RAW' => '0',
		'SUBTREE_DEPTH_AVG' => '3.5',
		'SUBTREE_DEPTH_DEV' => '0.25',
		'TOTAL_DEL' => '0.125',
		'TOTAL_GROW' => '0.1',
		'TOTAL_MOD' => '0.5',
		'TOTAL_SHRINK' => '0.1',
		'TOTAL_SUBTREE_DEL' => '0.075',
		'VERBOSE' => '4'
	    }
	};

	my $datasetfile = $args{datasetfile} || walk_param(
				[ qw( DATASET_FILE CLIENT_DATASET_FILE ) ],
				cliprefix => $cliout{param}
			    );

	if( $datasetfile ) {
	    my $raw_dataset_config = $vol_obj->{_clients}->{$client_name}->{_client_obj}->apiset->execute_raw_command(command =>"test -f $datasetfile && cat $datasetfile");
	    if( $raw_dataset_config =~ /\w+/si ) {
		$raw_dataset_config =~ s/#.*$//mg;
		if( my %sets = ( $raw_dataset_config =~ /\s*\[([^\[\]]+)\](?:[^\n\r]*)\s*([^\[\]]+)/gs) ) {
		    %{$vol_obj->{_clients}->{$client_name}->{_datasets}{$_}} = $sets{$_} =~ /^\s*([^\s=#]+)=[\042\047]?([^=\s\042\047#]+)/gm  foreach keys %sets;
		} else {
		    $vol_obj->{_clients}->{$client_name}->{_datasets}->{'default'} = $raw_dataset_config =~ /^\s*([^\s=#]+)=[\042\047]?([^=\s\042\047#]+)/gm;
		}
	    } else {
		confess "Cannot read dataset config file: $datasetfile";
	    }
	}

	my $dataset = $args{dataset} || walk_param(
				[ qw( DATASET CLIENT_DATASET ) ],
				cliprefix => [ $cliout{param}, $vol_obj->{_cli_ref}->{param} ],
				clidefault => 'default'
			    );
	$vol_obj->{_clients}->{$client_name}->{_dataset_use_as_default} = $dataset;
    }

    my $dataset = $args{dataset} || $vol_obj->{_clients}->{$client_name}->{_dataset_use_as_default};
    confess "Didn't find required dataset: [$dataset] ".Dumper($vol_obj->{_clients}->{$client_name}) unless $vol_obj->{_clients}->{$client_name}->{_datasets}->{$dataset};

    my $dir_tree = $vol_obj->{_clients}->{$client_name}->{_mount_obj}->mount_point() . '/' . $vol_obj->random_name_generator( prefix => 'dataset.dir' );
    $vol_obj->{_clients}->{$client_name}->{_client_obj}->apiset->execute_raw_command(command =>"/bin/mkdir -p $dir_tree && chmod 777 $dir_tree");

    return populate_tree(
		    $vol_obj->{_clients}->{$client_name}->{_client_obj}->hostrec,
		    $dir_tree,
		    %{$vol_obj->{_clients}->{$client_name}->{_datasets}->{$dataset}},
		    BACKGROUND => $args{background}
		);
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 snapshot_list

Returns list of snapshots on given volume. Wrapper around NACL::CS::VolumeSnapshot to reduce parameter. Uses volume object as first argument and rest of the
arguments are passed to NACL::CS::VolumeSnapshot as is. Required parameters such as I<command_interface>, I<vserver>, I<volume> are all taken
from volume object itself.

=over

=item Arguments

See NACL::CS::VolumeSnapshot->fetch for additional arguments.

=item Example

  1 use strict;
  2 require dp::snapmirror::paloma::NACLwrapper;
  3 my $aggr = get_aggregate( get_node( clitag => 'src' ) );
  4 my $volm = get_volume( $aggr );
  5 my @snapshots = snapshot_list($volm);


=back

=cut

sub snapshot_list {
    my ($vol_obj, %args) = @_;
    my @snapshots = map{ $_->{snapshot} } NACL::CS::VolumeSnapshot->fetch(
	    command_interface => $vol_obj->{_node},
	    allow_empty	      => 1,
	    filter	      => {
				    vserver => $vol_obj->{_vserver},
				    volume => $vol_obj->{volume}
				 },
	    %args
	);
    return \@snapshots;
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 snapshot_delete

Deletes snapshot from volume. Wrapper around NACL::STask::VolumeSnapshot::delete to reduce number of parameters. Uses volume object as first argument
and rest of the arguments are passed to NACL::STask::VolumeSnapshot::delete as is.

=over

=item Arguments

See NACL::CS::VolumeSnapshot->fetch for additional arguments.

=back

=cut

sub snapshot_delete {
    my ($vol_obj, %args) = @_;
    return NACL::STask::VolumeSnapshot->delete(
					    command_interface => $vol_obj->{_node},
					    volume => $vol_obj->{volume},
					    vserver => $vol_obj->{_vserver},
					    %args
					);
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head2 snapshot_create

Creates new snapshot. Wrapper around NACL::STask::VolumeSnapshot::create() to reduce number of parameters. Uses volume object as first
argument and rest of the arguments are passed to NACL::STask::VolumeSnapshot::create() as is. If name of snapshot to create is not given
then it is auto generated with $vol_obj->{_node}->random_name_generator( prefix => 'snp' ).

Returns whatever NACL::STask::VolumeSnapshot->create returns as is.

=cut

sub snapshot_create {
    my ($vol_obj, %args) = @_;
    return NACL::STask::VolumeSnapshot->create(
					    command_interface => $vol_obj->{_node},
					    volume => $vol_obj->{volume},
					    vserver => $vol_obj->{_vserver},
					    snapshot => $args{snapshot} || $vol_obj->{_node}->random_name_generator( prefix => 'snp' ),
					    %args
					);
}

=head2 ObjTearDown

Cleans up objects. Should be called from TCD's uninit. This function will get called automatically even
without TCD because of B<'END'> block. There is no danger to call this function multiple times because
ones object is cleaned its removed from stack.

=cut

#-------------------------------------------------------------------------------------------------------------------------------------------
sub ObjTearDown {
    while( my $o = pop @ObjStack ) {					# process our LIFO stack
	if( ref($o) =~ /NACL/ ) {					# if object is NACL
	    # $Test->nlog("Cleaning up: " . ref($o));			# tell what object we're cleaning up
	    if( ref($o) eq 'NACL::STask::Aggregate' ) {			# aggregates need to be recycled
		if( (my $aggr_name = $o->{aggregate}) =~ /^ag_/ ) {	# by renaming prefix from
		    $aggr_name =~ s/^ag_/AG_/;				# ag to AG
		    $o->rename( newname=>$aggr_name );
		}
	    } elsif ( ref($o) eq 'NACL::STask::Node' ) {		# if object is the NODE, then we need to
		_grab_logs($o);						# save all the logs from filer using Best Effort approach
	    } else {							# and for all other objects we check if
		if( $o->can('purge') ) {				# object supports 'purge' method,
		    eval { $o->purge() };				# run 'purge' as best effort
		} else {
		    $o->can('delete') && eval { $o->delete };		# try 'delete' if 'purge' is not supported
		}
	    }
	}
    }
}

#-------------------------------------------------------------------------------------------------------------------------------------------
END {
    if( $ObjStackPID eq $$ ) {
	logcomment("ObjTearDown started\n");
	ObjTearDown();
    } else {
	logcomment("No an owner, cannot allow ObjTearDown!");
    }
}

#-------------------------------------------------------------------------------------------------------------------------------------------
=head1 Examples

=head2 Simple init_update with TCD

Log file for this run: C</x/eng/natelogs/cpeter/20120822_163649>

      1 # $Id: //depot/prod/test/Rsierranevada/dp/snapmirror/paloma/NACLwrapper.pm#1 $
      2
      3 ## @summary Example for dp::snapmirror::paloma::NACLwrapper
      4 ## @author cpeter@netapp.com
      5
      6 use strict;
      7 use vars qw($Test);
      8 require dp::snapmirror::paloma::NACLwrapper;
      9
     10 my ($src_volm, $dst_volm, $sm_xdp);
     11 my @Testcases=(
     12     dataset_prep => "Preparing datasets",
     13     sm_init_update => "Trimmed snapmirror life cycle",
     14 );
     15
     16 $Test = new TCD(-testcases=>\@Testcases);
     17 if ($Test->error) {
     18     $Test->nlog($Test->errmsg());
     19     return $TCD::FAIL;
     20 }
     21
     22 $Test->run_test();
     23 if ($Test->error){
     24     $Test->nlog($Test->errmsg());
     25     return $TCD::FAIL;
     26 }
     27
     28 sub init {
     29     $Test->description("Create Aggrs, Volumes");
     30     $src_volm = get_volume(get_aggregate(get_node( clitag => 'src' )));             ## @step Create a RW src volume
     31     $dst_volm = get_volume(get_aggregate(get_node( clitag => 'dst')), type=>'dp');  ## @step Create a DP dst volume
     32     return $TCD::PASS;
     33 }
     34
     35 sub setup {
     36     $Test->description("Snapmirror create, no initialize");
     37     ## @step Snapmirror create
     38     $sm_xdp = snapmirror_create( src=>$src_volm, dst=>$dst_volm, type=>'xdp', do_initialize => 0 );
     39     return $TCD::PASS;
     40 }
     41
     42 sub dataset_prep {
     43     data_works( volume=>$src_volm );                            ## @step Populate source with data
     44     snapshot_create($src_volm, 'snapmirror-label'=>'weekly');   ## @step Create single snapshot
     45     return $TCD::PASS;
     46 }
     47
     48 sub sm_init_update {
     49     ## @step Snapmirror initialize
     50     $sm_xdp->initialize();
     51     ## @step Create single snapshot again
     52     snapshot_create($src_volm, 'snapmirror-label'=>'weekly');
     53     ## @step Populate source with more data
     54     data_works( volume=>$src_volm );
     55     ## @step Snapmirro update
     56     $sm_xdp->update;
     57     return $TCD::PASS;
     58 }
     59
     60 sub uninit {
     61     $Test->description("...teardown");
     62     ObjTearDown();
     63     return $TCD::PASS;
     64 }

=head2 Simple init_update without TCD

Log file for this run: C</x/eng/natelogs/cpeter/20120822_164754>

      1 # $Id: //depot/prod/test/Rsierranevada/dp/snapmirror/paloma/NACLwrapper.pm#1 $
      2
      3 ## @summary Example for dp::snapmirror::paloma::NACLwrapper
      4 ## @author cpeter@netapp.com
      5
      6 use strict;
      7 use vars qw($Test);
      8 require dp::snapmirror::paloma::NACLwrapper;
      9
     10 my ($src_volm, $dst_volm, $sm_xdp);
     11
     12 $src_volm = get_volume(get_aggregate(get_node( clitag => 'src' )));             ## @step Create a RW src volume
     13 $dst_volm = get_volume(get_aggregate(get_node( clitag => 'dst')), type=>'dp');  ## @step Create a DP dst volume
     14
     15 ## @step Snapmirror create
     16 $sm_xdp = snapmirror_create( src=>$src_volm, dst=>$dst_volm, type=>'xdp', do_initialize => 0 );
     17
     18 data_works( volume=>$src_volm );                            ## @step Populate source with data
     19 snapshot_create($src_volm, 'snapmirror-label'=>'weekly');   ## @step Create single snapshot
     20
     21 ## @step Snapmirror initialize
     22 $sm_xdp->initialize();
     23
     24 ## @step Create single snapshot again
     25 snapshot_create($src_volm, 'snapmirror-label'=>'weekly');
     26
     27 ## @step Populate source with more data
     28 data_works( volume=>$src_volm );
     29
     30 ## @step Snapmirror update
     31 $sm_xdp->update;
     32
     33 ## @step Clean up
     34 # clean up is done by dp::snapmirror::paloma::NACLwrapper, called automatically

=cut

1;
