# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/dp/snapmirror/paloma/Utils.pm#1 $ //depot/prod/test/main/dp/snapmirror/paloma/Utils.pm#13 $
##
## @summary GNOTE::Utils library
##	Utils methods for Paloma scripts
##
## @type library
##
## @pod here
##
## @author neha@netapp.com, dl-dpgqa-automation@netapp.com
## @status work-in-progress

package dp::snapmirror::paloma::Utils;

use strict;
use Carp;
use Tharn;

use Data::Dumper;
use DataManip;
use PDF::Create;
use File::chmod;
use Filesys::Tree qw/tree/;
use GNOTE;
#use nfs::ACL::AclNFSV4 qw( :ACE4_TYPES :ACE4_FLAGS :ACE4_MASK );

use base qw( Exporter );

our @EXPORT_OK = qw(
    populate_datasets
    modify_datsets
    add_files
    rename_datasets
    inode_recycle
    rearrange_trees
    get_existing_aggregate
    fscreate
    allow_mixed_geometry
);

sub rearrange_trees {
	## Input: vol obj
	## Returns: 0 on success
	my ($vol) = @_;
	my $root = "";
	my $path = "";
	my $tree = "";
	
	$root = $vol->nfs;
	$path = $root->fullpath;
	chomp($path);
	$tree = tree( { 'directories-only' => 1 } , 
		      "$path" );
	## we have the dir structure in a tree-like format
	## Algorithm:
	##	1> pick 2 paths
	##	2> interchange them
	my $move_1 = $path;
	my $move_2 = $move_1;
	
	my $levels = _find_value ($tree);
	my @array = split (' ',$levels);
	@array = grep ( !/(contents|type|tcp_v3)/ , @array);
	foreach my $dir ( @array ) {		
		if ( -d $move_1.$dir ) {
			$move_1 = $move_1 . $dir . '/';
		}
		elsif ( -d $move_2.$dir ) {
			$move_2 = $move_2 . $dir . '/';
		}
		else {}
	}
	logcomment("\n $move_1 \n is moved into \n $move_2 \n");
	eval {
		system("mv $move_1 $move_2/.; ls -l $move_2");
	};	
	warn $@ if $@;
	return 0;
}

sub _find_value {
    my $hash_ref = shift;
    my $level;

    foreach my $key (keys %$hash_ref) {
        $level .= "$key\n";
        my $value = $hash_ref->{$key};
        if (ref($value) eq 'HASH') {
            $level .= _find_value($value);
        } else {
            $level .= "\t$value\n";
        }
    }
    return $level;
}

sub rename_datasets {
	## rename directories and files
			
	## Input: vol obj
	## Returns: 0 on success
	my ($vol) = @_;
	my @datasets = ();
	my @files = ();
	my $root = "";
	my $chosen_dataset = "";
	my $path = "";
	
	## assuming all datasets have their respective 
	## directory at the base of the "root" directory,
	## we will pick one of them for renaming
	$root = $vol->nfs;
	$path = $root->fullpath;
	@datasets = $root->ls;
	$chosen_dataset = $datasets [int (rand (scalar(@datasets)))];
        logcomment("Chose $chosen_dataset to rename");
	$path = $path . $chosen_dataset;

	## renaming dataset directory	
	if ( -d chomp($path) ){
		my $newname = $path."_renamed".time();
		if ( !(rename $path, $newname) ) {
			croak "\n Could not rename $path to $newname";
		}
		$path = $newname;
	}
	@files = `ls $path`;
	chomp(@files);
	## rename all files from the dataset (directory)
	foreach my $file (@files) {		
		my $file_length = length ($file);
		my $new_filename = "";
		if ( $file_length > 190 ) {
			$new_filename = substr($file, 0, -50);
		}
		$file = $path . '/' . $file;
		sleep (3);
		$new_filename = $path . '/' . $new_filename . _time_id(). '_renamed_@%$#%$#';
		if (!(rename $file, $new_filename) ){
			croak "\n Could not rename \n $file \n to \n $new_filename";
		}
		else {}
		$file = $new_filename;
	}
	return 0;
}

sub add_files {
	## this subroutine should add the following files to the dataset:
	## 1> hardlinks to special files (block, char and pipes)
	## 2> HTML, binary, PDF files
	
	## Input: vol obj
	## Returns: 0 on success
	my ($vol) = @_;
	
	## add special files
	my $dataset = "";
	my $root = $vol->nfs;
	my $dataset = $root->mkdir( '/added_special_files' );
	_create_special_files ( dataset	=>	$dataset );
	_create_pipes ( dataset	=>	$dataset );
	_add_pdf($vol);
	_create_symlinks( dataset	=>	$dataset );
	_create_hardlinks( dataset	=>	$dataset );
		
	return 0;
}

sub inode_recycle {
	## Input: vol obj
	
	## volume max dir size must be modifyed from 10 MB default to 
	## max vol size in order to be able to create all the files in the
	## same dir.
	
	## for very large volumes > 100GB, we need to have multple directories 
	## for the data, and each dir max size should be 3999MB
	 
	my ($vol) = @_;	
	my $dir = "";	
	my $path = "";
	my $vol_status = "";
	my $total_files = "";
	my $used_files = "";
	my $files_to_be_created = 0;
	my @files_to_delete = ();
	my @files_to_populate = ();
		
	$dir = $vol->nfs->mkdir( '/max_number_of_files' );
	$path = $dir->fullpath;
	$vol_status = $vol->status;
	$total_files = $$vol_status{'files'};
	$used_files = $$vol_status{'files-used'};
	$files_to_be_created = $total_files - $used_files - 1000;
	my %mod_options = ( ENVIRONMENT_TYPE => 'nil',
						SEED => '10',
						CREATE_FILE_SIZE_AVG => '0',
						CREATE_FILE_AVG => $files_to_be_created,
						CREATE_DIR_AVG => '1',
						SUBTREE_DEPTH_AVG => '1',
						TOTAL_DEL => '10',
						VERBOSE          => '1'
						);
	$vol->modify(%mod_options);
	$vol_status = $vol->status;
	$total_files = $$vol_status{'files'};
	$used_files = $$vol_status{'files-used'};
	$files_to_be_created = $total_files - $used_files;
	%mod_options = (    ENVIRONMENT_TYPE => 'nil',
						SEED => '10',
						CREATE_FILE_SIZE_AVG => '0',
						CREATE_FILE_AVG => $files_to_be_created,
						CREATE_DIR_AVG => '1',
						SUBTREE_DEPTH_AVG => '1',
						VERBOSE          => '1'
					   );
	$vol->modify(%mod_options);
	$vol_status = $vol->status;
	$total_files = $$vol_status{'files'};
	$used_files = $$vol_status{'files-used'};
	$files_to_be_created = $total_files - $used_files;
	
	return 0;
}

sub _modify_hard_links {
	## Input: vol obj
	## will change the permission to 0777 to all hard links
	my ($vol) = @_;	
	my $dir = "";	
	my $path = "";
	my @files = ();

	$dir = $vol->nfs->mkdir( '/hard_links' );
	$path = $dir->fullpath;
	@files = `ls $path`;
	chomp(@files);
	foreach my $file (@files) {		
		$file = $path.$file;
	}
	# change permissions
	chmod (0777, @files);
	# remove some links
	my $half = int ( scalar (@files) / 2);
	_remove_links(@files[1..$half]);
	# rename links
	_rename_links(@files);
	system("ls -lR $path");	
		
	return 0;
}

sub _modify_sym_links {
	## Input: vol obj
	my ($vol) = @_;	
	my $dir = "";	
	my $path = "";
	my @files = ();
	
	$dir = $vol->nfs->mkdir( '/sym_links' );
	$path = $dir->fullpath;
	@files = `ls $path`;
	chomp(@files);
	foreach my $file (@files) {		
		$file = $path.$file;
	}
	_change_uid_gid ( @files );
	system("ls -lR $path");
	return 0;
}

sub _change_uid_gid {
	## bash-3.00$ ypcat passwd | grep eit | grep DUMMY
	## eit:Bvh9yjcEa4s5o:12859:30:System account for Todd Shoenfelt (DUMMY):/u/eit:/bin/csh
	## bash-3.00$
	## input: array of files that will have the uid/gid changed to the one above
	
	my @files = @_;
	logcomment ("Changing UID/GID of \n @files");
	foreach my $file ( @files ){
		eval {
			system ("sudo chown 12859:30 $file");
		};
		warn $@ if $@; 
	}
	warn $@ if $@;		
	return 0; 	
}

sub _remove_links {
	my @files = @_;
	
	logcomment("Removing links");
	foreach my $file (@files){
		eval{
			system ("rm $file");
		};
		warn $@ if $@;
	}
	
	return 0;
}

sub _rename_links {
	my @files = @_;
	
	logcomment("Renaming links");
	foreach my $file (@files) {
		my $old_file = $file;
		my $newname = $file."_renamed";
		rename $file, $newname;
		if ( -l $old_file ) {
			if ( -l $file ){
				next;
			}
			else {
				warn "$file is not a sym link as the original file before renaming";
			}
		}
	}
	return 0;
}

sub _add_pdf {
	my ($vol) = @_;	
	my $dir = "";	
	my $path = "";
	my @files = ();
	
	$dir = $vol->nfs->mkdir( '/added_special_files' );
	$path = $dir->fullpath;
	# generating 250 file names ending in numbers and @
	for (my $i = 0 ; $i < 125; $i++) {
		my $tmp_name = $path . "pdf_${i}".'.pdf';
		push @files,$tmp_name;
	}
	for (my $i = 125; $i < 250 ; $i++){
		my $tmp_name = $path . "pdf_${i}_@";
		push @files,$tmp_name;
	}
	# populate PDFs using these names
	logcomment ("Creating PDF files");
	foreach my $file (@files){
		my $pdf = new PDF::Create(
					'filename'     => $file,
				  	'Version'      => 1,
			      	'PageMode'     => 'UseOutlines',
			     	'Author'       => 'DPG QA',
			      	'Title'        => "title_${file}",
			      	'CreationDate' => [ localtime ],
			 );
		my $r = $pdf->new_page('MediaBox' => [ 0, 0, 612, 792 ]);
		my $page = $r->new_page;
		my $f1 = $pdf->font('Subtype'  => 'Type1',
 	   	        			'Encoding' => 'WinAnsiEncoding',
 		        			'BaseFont' => 'Helvetica');
    	$page->string($f1, 20, 306, 396, "some text");
		$pdf->close;
	}
}

sub modify_datsets {
	
	## TEMPLATE  !!!
	 
	my $DEBUG = 1;
	my ($vol, %args) = @_;
	my @dataset_types = ();
	
	while (my ($key, $value) = each %args) {
		if ($value eq '1') {
			push (@dataset_types,$key);
		}
	}
	
	foreach my $dataset_type (@dataset_types) {
		if ($dataset_type =~ /random_nfs/) {
			logcomment ("Modifying random_nfs");
			_modify_random_nfs($vol);
		}
		elsif ($dataset_type =~ /small_files/) {
			logcomment ("Modifying small files");
			_modify_small_files($vol);
		}
		elsif ($dataset_type =~ /large_files/) {
			logcomment ("Modifying large files");
			_modify_large_files($vol);
		}
		elsif ($dataset_type =~ /hard_links/) {
			logcomment ("Modifying hard links");
			_modify_hard_links($vol);
		}
		elsif ($dataset_type =~ /files_with_maxlen/) {
			logcomment ("Modifying files with maxlen");
			_modify_files_with_maxlen($vol);
		}
		elsif ($dataset_type =~ /file_dir_with_same_name/) {
			logcomment ("Modifying file dir with same name");
		}
		elsif ($dataset_type =~ /sym_links/) {
			logcomment ("Modifying sym links");
			_modify_sym_links($vol);
		}
		elsif ($dataset_type =~ /random_cifs/) {
			logcomment ("Modifying random cifs");
		}
		elsif ($dataset_type =~ /fifo/) {
			logcomment ("Modifying fifos");
		}
		elsif ($dataset_type =~ /sockets/) {
			logcomment ("Modifying sockets");
		}
		elsif ($dataset_type =~ /sparse_files/) {
			logcomment ("Modifying sparse files");
		}
		elsif ($dataset_type =~ /special_files/) {
			logcomment ("Modifying special files");
		}
		elsif ($dataset_type =~ /targeted_cifs/) {
			logcomment ("Modifying targeted cifs");
		}
		elsif ($dataset_type =~ /junctions/) {
			logcomment ("Modifying junctions ");
		} 
		else {
			die "Incorrect dataset type: $dataset_type.Pl choose one of these:nfs_datamanip|small_files|large_files|hard_links|files_with_maxlen|file_dir_with_same_name|cifs_data|";
		}
	}
}

sub populate_datasets {
    my ($vol, %args) = @_;

    ## XXX 'all' is not supported yet

    my $win_client = $args{win_client};
    delete $args{win_client};
    my $dataset_type = $args{dataset_type};
    delete $args{dataset_type};
    my $nodes = $args{nodes};
    delete $args{nodes};
    my $vol_type = $args{vol_type};
    delete $args{vol_type};
    my $vol_size = $args{vol_size};
    delete $args{vol_size};
    my $count = $args{count};
    delete $args{count};
     
    my ($root, $dataset);
    # In case of 'random_nfs' the script should create the directory
    # and pass it because the volume object needs to keep track of delete
    unless ($dataset_type =~ /random_nfs|junctions/) {
	$root = $vol->nfs;
        $dataset = $root->mkdir( '/'.$dataset_type );
    }

    if ($dataset_type =~ /random_nfs/) {
        # uses DataManip.pm
        logcomment("Now populating random_nfs");
        $vol->populate(%args);

    } elsif ($dataset_type =~ /small_files/) {
        logcomment("Now populating small_files");
        _create_small_files(dataset =>$dataset);

    } elsif ($dataset_type =~ /large_files/) {
        logcomment("Now populating large_files");
        $dataset->create( 'file-2g' )->fill( max => '2g' );
        $dataset->create( 'file-4g' )->fill( max => '4g' );
        $dataset->create( 'file-5g' )->fill( max => '5g' );
        $dataset->create( 'file-64g' )->fill( max => '64g' );
        $dataset->create( 'file-65g' )->fill( max => '65g' );
    } elsif ($dataset_type =~ /hard_links/) {
        logcomment("Now populating hard_links");
        _create_hardlinks(dataset=>$dataset);

    } elsif ($dataset_type =~ /files_with_maxlen/) {
        logcomment("Now populating files_with_maxlen");
        _create_files_with_maxlen(dataset=>$dataset); 

    } elsif ($dataset_type =~ /file_dir_with_same_name/) {
        logcomment("Now populating file_dir_with_same_name");
        _create_file_dir_with_same_name(vol=>$vol, root=>$root, dataset=>$dataset);

    } elsif ($dataset_type =~ /sym_links/) {
        logcomment("Now populating sym_links");
        _create_symlinks(dataset=>$dataset);

    } elsif ($dataset_type =~ /random_cifs/) {
       logcomment("Now populating random_cifs");
       # uses RichData.pm
       my $cifs = NOTE::Client->new(storage=>$vol, name=>$win_client);
	eval {
       		RichData::populate(	path=>$cifs->mountpoint."\\\\", 
       					client=>$win_client, 
       					fill_vol=>1,
       					stream_avg => 100,
       					stream_size_avg => 100,
       					file_stream_num => 100,
					file_avg => 100,
					file_size_avg => 10,
					file_acl_num => 10,
					slink_num => 100,
					sparse_num => 10);
	};
	warn ($@) if ($@);

    } elsif ($dataset_type =~ /fifo/) {
        logcomment("Now populating fifos");
        _create_fifos(dataset=>$dataset);
     
    } elsif ($dataset_type =~ /sockets/) {
        logcomment("Now populating sockets");
        _create_sockets(dataset=>$dataset);
     
    } elsif ($dataset_type =~ /sparse_files/) {
        logcomment("Now populating sparse files");
        _create_sparse_files(dataset=>$dataset);

    } elsif ($dataset_type =~ /large_sparse_files/) {
         logcomment("Now populating large sparse files");
         _create_sparse_files(dataset=>$dataset, large_sparse_files=>1);	
     
    } elsif ($dataset_type =~ /special_files/) {
        logcomment("Now populating special files");
        _create_special_files(dataset=>$dataset);
     
    } elsif ($dataset_type =~ /targeted_cifs/) {
        # uses Client.pm  XXX Need to know the interesting cases before coding them here
        logcomment("Now populating targeted_cifs");

    } elsif ($dataset_type =~ /junctions/) {
        logcomment("Now populating junctions");
	_create_junctions(dataset=>$dataset, vol=>$vol, nodes=>$nodes, vol_type=>$vol_type, vol_size=>$vol_size, count=>$count);

    } else {
        # do nothing
	die "Incorrect dataset type: $dataset_type.Pl choose one of these:nfs_datamanip|small_files|large_files|hard_links|files_with_maxlen|file_dir_with_same_name|cifs_data|";
    }
}

sub _modify_random_nfs() {
	## Use DataManip.pm to modify files
	my ($vol) = @_;
	my $dir = "";
	my $path = "";
	my $root = "";
	my %opts = {};
		
	$root = $vol->nfs;
    $dir = $vol->nfs->mkdir("random_nfs");
    $path = $dir->fullpath;
    
    %opts = {CREATE_FILE_SIZE_AVG			=>	'100k',
    		 CREATE_FILE_AVG			=>	100,
    		 CREATE_DIR_AVG				=>	3,
    		 TOTAL_DEL				=>	20,
    		 TOTAL_MOD				=>	20,
    		 MOD_AVG				=>	10,
    		 TOTAL_GROW				=>	5,
    		 GROW_AVG				=>	3,
    		 TOTAL_SHRINK				=>	5,
    		 SHRINK_AVG				=>	1};
	modify_tree('localhost',$path,%opts);
	    
    return 0;
}

sub _modify_small_files() {
	## This sub will delete all the small 
	## files, and recreate them ==> modify 100%
	my ($vol) = @_;
	my $root = "";
	my $dir = "";
	my $dataset = "";
		
	$root = $vol->nfs;
	$dir = $vol->nfs->mkdir("small_files");
	my $tmp = $dir->fullpath;
	## because rmdir from NFS.pm doesn't work
	system("rm -r $tmp");
	croak $dir->path." still exists! *** " if ( $dir->stat );
	$dataset = $root->mkdir( '/small_files' );
	_create_small_files(dataset	=>	$dataset);
	
	return 0;
}

sub _modify_files_with_maxlen {
	## This sub will shrink special files
	## Input: vol obj
	my ($vol) = @_;
	my $root = ();
	my $dir = "";
	my $path = "";
	my @files = ();
	
	$root = $vol->nfs;
	$dir = $vol->nfs->mkdir("files_with_maxlen");
	$path = $dir->fullpath;
	@files = `ls $path`;
	chomp(@files);
	foreach my $file (@files) {		
		$file = $path.$file;
	}
	NOTE::logcomment ("Shrinking/changing atime and mtime for files_with_maxlen");
	foreach my $file (@files) {
		my  ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,$ctime,$blksize,$blocks) = stat($file);
        	my $new_size = int ( $size / 2 );
        	my $trunc_return = truncate $file,$new_size;
        	die "Trunc failed Utils.pm" if ( 1 != $trunc_return );
    		my $name = (split ('/',$file))[-1];
	}	
	return 0;
}

sub _modify_large_files {
	## This subroutine will 
	## * change file content for large files
	## * rename files
	## * grow files
	my ($vol) = @_;
	my $root = ();
	my $dir = "";
	my $path = "";
	my @files = ();
		
	$root = $vol->nfs;
	## because chdir from NFS.pm doesn't work'
	$dir = $vol->nfs->mkdir("large_files");
	$path = $dir->fullpath;
	@files = `ls $path`;
	chomp(@files);
	foreach my $file (@files) {		
		$file = $path.$file;
	}
	logcomment("grow and change file content for large files");
	_grow_change_content_files(@files);
	logcomment("rename all large files");
	## sound we put this in another sub in case we want 
	## to use it for other files too?
	foreach my $file (@files) {	
		my $new_file = $file . "_renamed";
		rename $file,$new_file;
		$file = $new_file;
	}
	
	return 0;
}

sub _grow_change_content_files() {
	## Param: array of filenames
	my @local_files = @_;
	my @alphabet = ('a'..'z','A'..'Z','!','@','#','$','%','^','&','*','(',')','-','_','`','~','1'..'9',' ');
	my $one_k_string = 'fgdsjfgsdmfghdfkjghdfjkhgkyrwiueryuewitriq3wy48372563287reyw78fdtgewfgcewut328765r47rtf7werfgyweuryi7df6ri7w34htfr74ruieyhfdw4u6tcgreuyhdwi7ydh836r74thfehjrfyudr6gt4dhusairtgfdkhkk64ftg764trgyesdrtgrdrfg  eg6oritgutu34987th8dsjwalkhsfhcnskvh kvku   thyfgygfuoiern kktykeygvh iuetueroitu45ytggf d riehyrvce brtewkjtyvireyvceiutncer erkj kuerytgiureyt45674843yt8vn kdjfsdfj lw;eic[2p34iorco][qw-roc-[oeq][wep]qw\pe we[ekvkl;rev ]fpr\3tp54bu67n 6g5h687gjh5b4gy5j7nm5h7jm fyhrdt7rf86tgrdhryfenbfdsmfnbsdmnbfmsdnbfmsdnbfs dhgkjdfhgkjfdhgkjfdhgyrwiueryuewitriq3wy48372563287reyw78fdtgewfgcewut328765r47rtf7werfgyweuryi7df6ri7w34htfr74ruieyhfdw4u6tcgreuyhdwi7ydh836r74thfehjrfyudr6gt4dhusairtgfdkhkk64ftg764trgyesdrtgrdrfg  eg6oritgutu34987th8dsjwalkhsfhcnskvh kvku   thyfgygfuoiern kktykeygvh iuetueroitu45ytggf d riehyrvce brtewkjtyvireyvceiutncer erkj kuerytgiureyt45674843yt8vn kdjfsdfj lw;eic[2p34iorco][qw-roc-[oeq][wep]qw\pe we[ekvkl;rev ]fpr\3tp54bu67n 6g5h687gjh5b4gy5j7nm5h7jm fyhrdt7rf86tgrdhryfenbfdsmfnbsdmnbfmsdnbfmsdnbfs';
		
	foreach my $file (@local_files) {
		croak "nonexistent file" if (! (-e $file) );
		open(FILE, "+>> $file") || warn "Couldn't open file: $file";
		my $file_position = sysseek (FILE, 0, 0);
		croak "\n File sysseek failed in Utils.pm (1)" if ('0 but true' ne $file_position);
		for (my $i = 0; $i < 4 ; $i++) {
			syswrite FILE, "$one_k_string";
		}
		## grow the original file and 
		## create holey files from the original files
		my  ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
             $atime,$mtime,$ctime,$blksize,$blocks) = stat($file);
		for (my $i = 0 ; $i < 2 ; $i++) {
			$file_position = sysseek (FILE, 4096, 2);
			syswrite FILE, "$one_k_string";
			my $tmp_count = int($size / $blksize);
			`dd if=$file of=${file}_hole bs=$blksize oseek=2 count=$tmp_count`;
		}		
		close(FILE);
		# suffle the 1k string with 10 characters
		for(my $k = 0 ; $k < 10 ; $k++) {
			my $i = int(rand($#alphabet));
			chop($one_k_string);
			$one_k_string = $alphabet[$i] . $one_k_string;
		}		
	}
	return 0;	
}
# Create junctions
sub _create_junctions {
    my (%args) = @_;

    $args{count} = 1 unless (exists $args{count} && $args{count});
    my $aggrname = $args{vol}->Aggregate->name;
    for (my $i=0; $i<$args{count}; $i++) {
        my %vol_create_opts = (
	    Aggregate => { name => $aggrname },
            size      => $args{vol_size},
	    name      => "jn_vol".$i,
            tag       => 'junction-volume',
	    path       => "/".$args{vol}->name."/jn_vol".$i,
            #path       => "/".$vol."/junctions",
        );
        my $vol_src = NOTE::Volume->create( %vol_create_opts);
    }

}

# Create small files
sub _create_small_files {
    my (%args) = @_;

    $args{dataset}->create( 'file-0' );
    $args{dataset}->create( 'file-4b' )->fill( max => '4' );
    $args{dataset}->create( 'file-8b' )->fill( max => '8' );
    $args{dataset}->create( 'file-12b' )->fill( max => '12' );
    $args{dataset}->create( 'file-64b' )->fill( max => '64' );
    $args{dataset}->create( 'file-1028b' )->fill( max => '1028' );
    $args{dataset}->create( 'file-1kb' )->fill( max => '1k' );
    $args{dataset}->create( 'file-4kb' )->fill( max => '4k' );
    $args{dataset}->create( 'file-5kb' )->fill( max => '5k' );
    $args{dataset}->create( 'file-64kb' )->fill( max => '64k' );
    $args{dataset}->create( 'file-100kb' )->fill( max => '100k' );
    $args{dataset}->create( 'file-64mb' )->fill( max => '64m' );
    $args{dataset}->create( 'file-128mb' )->fill( max => '128m' );
}

sub _create_files_with_maxlen {
    my (%args) = @_;

    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789a' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789b' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789c' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789aa' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789bb' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789cc' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789abc' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789bca' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789cab' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789aaa' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789bbb' )->fill( max => '1k' );
    $args{dataset}->create( '0123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789ccc' )->fill( max => '1k' );
    $args{dataset}->create( '012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234567890123456789012345678901234' )->fill( max => '1k' );
}

sub _create_symlinks {
    my (%args) = @_; 

    # XXX Need to vary the size of symlinks. See testplan of burt 255807
    # Creates 5 files each with a symlink
    for my $i (1..5) {
        my $timeid = _time_id();
        my $filename = "file-1kb-with-symlink_".$timeid;
        my $symlinkname = "symlink_to_file-1kb-with-symlink_" . $timeid . "_$i";

        my $file = $args{dataset}->create( $filename )->fill( max => '1k' );
        $file->symlink( $symlinkname );
    }

    # Creates 1 file with 5 symlinks
    my $filename = "file-1kb-with-symlink_"._time_id();
    my $file_with_mul_links = $args{dataset}->create( $filename )->fill( max => '1k' );

    for my $i (1..5) {
        my $symlinkname = "symlink_to_file-1kb-with-symlink_". _time_id() . $i;
        $file_with_mul_links->symlink( $symlinkname );
    }
}

sub _create_hardlinks {
    my (%args) = @_; 

    # Creates 5 files each with a hardlink
    for my $i (1..5) {
        my $timeid = _time_id();
        my $filename = "file-1kb-with-hardlink_".$timeid;
        my $hardlinkname = "hardlink_to_file-1kb-with-hardlink_".$timeid.$i;

        my $file = $args{dataset}->create( $filename )->fill( max => '1k' );
        $file->link( $hardlinkname );
    }

    # Creates 1 file with 5 hardlinks
    my $filename = "file-1kb-with-hardlink_"._time_id();
    my $file_with_mul_links = $args{dataset}->create( $filename )->fill( max => '1k' );

    for my $i (1..5) {
        my $hardlinkname = "hardlink_to_file-1kb-with-hardlink_" . _time_id() . $i;
        $file_with_mul_links->link( $hardlinkname );
    }
}

sub _create_sockets {
    my (%args) = @_; 

    # Creates 5 sockets
    for my $i (1..5) {
	my $pathname = $args{dataset}->path . "socket_" . _time_id() . $i;
        $args{dataset}->create_socket($pathname);
   }
}

sub _create_sparse_files {
    my (%args) = @_;
   
    ## removing the small holy files since we need at least 3 blocks for them
    my @small_file_size = ('file-64m','file-128m','file-150m','file-180m',
    					   'file-220m','file-280m','file-314m','file-412m',
    					   'file-512m','file-670m','file-800m','file-912m');
    my @large_file_size = ('file-2g','file-4g','file-8g','file-16g','file-32g','file-64g');

    my @ary = ($args{large_sparse_files})? @large_file_size : @small_file_size;

    for (my $i = 0; $i < scalar(@ary); $i++) {
        $ary[$i] =~ /^file-(.*?)$/;
        my $size = $1;
        $args{dataset}->create( $ary[$i] )->fill( type => 'sparse', max => $size );        
    }
}


sub _create_fifos {
    my (%args) = @_; 

    # XXX Find out the interesting modes for fifos
    my @modes = ('0777','0770','0077','0666');

    # Creates 5 fifos
    for my $i (1..5) {
	my $pathname = $args{dataset}->path . "fifo_". _time_id(). "_$i";
        my $mode = shift @modes; 
        push @modes, $mode; # To make @modes a circular list so we can keep iterating
	my $client = $args{client}? $args{client} : 'localhost';
        $args{dataset}->create_fifo(pathname=>$pathname, client=>$client, mode=>$mode);
   }
}

sub _create_special_files {
    my (%args) = @_; 

    #my @types = ('c','u','b','f');
    my @types = ('c','b','p');
    for my $i (1..10) {
        my $type = shift @types; 
        push @types, $type; # To make @modes a circular list so we can keep iterating
        my $pathname = $args{dataset}->path . "block_". _time_id() . "_$i";
#        my $pathname = $args{dataset}->path . "block_". _time_id(). "$i";
        my $major = 10; # XXX Add variation in this parameter (0 - 255)
        my $minor = 2; # XXX Add variation in this parameter (0 - 255)

        $args{dataset}->mknod(name=>$pathname, type=>$type, major=>$major, minor=>$minor);
    }
}

sub _create_file_dir_with_same_name { 
    my (%args) = @_; 

    # Creates 10 level directories each having a file of the same name as its parent
    my $dir = $args{dataset};
    $args{vol}->cd( $dir->path );
    # Make sure $vol->delete gets executed before NOTE tries to
    # remove $dataset1, otherwise it'll fail because $dataset1
    # isn't empty
#    NOTE::reference( sub { $args{vol}->delete } );

    for my $i (1..10) {
        my $common_name = "paloma_" . _time_id(). "_$i";
        $dir = $dir->mkdir('/'.$common_name);
        my $file_path = $dir->path.$common_name;
        my $file = $dir->create($file_path);
        $args{vol}->cd( $dir->path );
    }
}


sub _create_pipes {
    my (%args) = @_; 
}

## @synopsis
##    Returns current time as one big human-readable number.
sub _time_id () {
  my ($sec,$min,$hour,$mday,$mon,$year) = localtime;
  $sec .= int (rand 50) + 1;
  sleep(1);
  return sprintf("%04d%02d%02d%02d%02d%02d",$year+1900,$mon+1,$mday,$hour,$min,$sec);
}

## @synopsis
##    Returns an aggregate that matches the criteria that has passsed.
## @arg
##    node      Where the aggregate is needed to reside
##    voltype   flex or Coral
##    volsize   What is the minimum size that should be availabe 
##
##    This routine will return an aggregate name if found other wise undefined value
##
sub get_existing_aggregate {
    my $node = shift;
    my $voltype = shift;
    my $volsize = shift;
    my $aggr;
    my $aggregate;
    my $server_obj = Server->new( nodes=>$node );
    my $original_node; 
 
    logcomment("Aggr selection criteria: Node: $node, Vol Type: $voltype, Vol Size: $volsize"); 
    $original_node = $server_obj->set_execution_server(-Server=>$node); 
    my @aggrlist = $server_obj->pick_aggregates(
                                                state=>"online",
                                                voltype=>$voltype,
                                                avail_size=>$volsize,
                                                );

    foreach $aggr (@aggrlist) {
        logcomment("Considering ". $aggr->{aggregate} );
        if ( $node eq $aggr->{nodes}) {
           logcomment("Aggr:" . $aggr->{aggregate}." is on " . $aggr->{nodes});
           $aggregate = $aggr;
           last;
        }
    }

    $server_obj->set_execution_server(-Server=>$original_node);
    return $aggregate;
}

##############################################################
# FScreator
#
## @summary Creates a subtree
##
## @description
##    opens an FScreator process and "pipes" the argument to it
##
## @synopsis
##    fscreate( $subtree_root_dir );
##
## @arg
##    scalar - path to the root of a test subtree
##
##############################################################

sub fscreate( $ ) {

    my ($fullpath) = @_;
    my $utility = _findtest('dp/snapmirror/tools/FScreator'); 

    eval {         
        # sudo FScreator allows creation of char and block devices and
        # also ensures repeatable subtree ownership (i.e. uid/gid == 0/0)
        open( FSC, "|/usr/software/bin/sudo $utility" );
    };
    croak( "open FScreator failed: $@" ) if $@;

    eval {
        print( FSC $fullpath );   # This print is the "cmd line" interaction
    };
    croak( "Failed to pipe path to FScreator: $@" ) if $@;

    eval {
        close( FSC );
    };
    croak( "Failed to close FScreator pipe: $@" ) if $@;

    return 0;
}

sub _findtest {
    my ($test) = @_;
    foreach my $dir (@INC) {
    my $full = "$dir/$test";
    	return $full if (-r $full);
    }
    logresult('CONFIG',"Could not find \"$test\" in \@INC (" . join(" ",@INC) . ")"); 
}


##############################################################
# allow_mixed_geometry
#
## @summary When called, this sub sets the current server into
## test mode and turns on paloma_allow_unlike_geometry in each 
## node in the cluster.
##
## @synopsis
##    allow_mixed_geometry( $server, $src_node);
##
## @arg
##    ref - reference to server object
##
sub allow_mixed_geometry {
    my ($server) = @_;
    my @nodes;
    my $original_node = $server->get_execution_server;
    my $cmd = "priv set test; setflag paloma_allow_unlike_geometry 1";

    @nodes = $server->servers;
    foreach my $node ( @nodes ) {
        eval {
            $server->set_execution_server( -Server => $node );
            $server->dblade( cmd => $cmd );
        };
        croak "$@" if $@;
    }
    eval {
        $server->set_execution_server( -Server => $original_node );
    };
    croak "$@" if $@;

    eval {
        $server->set( privilege => "test", confirmations => "off" );
    };
    croak "$@" if $@;
}

       
1;
