# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/dp/snapmirror/paloma/dpmirror/hardlinks/Hardlinks.pm#1 $
##
## @summary    Common functions used by Hardlinks test scripts
##
## @status     public
##
## @author     tyu@netapp.com dl-dpgqa-automation@netapp.com
##
## @pod        here
##

=head1 NAME

Hardlinks

=head1 DESCRIPTION

The purpose of this module is to collect the commonly used functions related to testing hardlinks.

=cut

package Hardlinks;

use strict;
use Carp qw(croak);

=head2 show_file_stat_by_file 

Method to show the stat information of a file.
Information will be log by means of the NOTE::logcomment () method.

=over

=item Arguments

B<msg> => I<string>

Required: The prefix message for the log comment.

B<file> => I<NOTE::File>

Required. The File handle.

=item Examples

=back

    # Show the stat information of a file :
	my $nfs = $vol->nfs ();
	my $testfile = $nfs->create ("testfile");
    Hardlinks::show_file_stat_by_file ("(Hardlinks08::do_setup"): Debug msg", $testfile);

=cut

sub show_file_stat_by_file {
    my ($msg, $file) = @_;

    my ($my_dev, $my_ino, $my_mode, $my_nlink,
        $my_uid, $my_gid, $my_rdev, $my_size,
        $my_atime, $my_mtime, $my_ctime, $my_blksize,
        $my_blocks) = $file->stat;
    NOTE::logcomment("".$msg."\n".
                     "\t dev is ".$my_dev."\n".
                     "\t ino is ".$my_ino."\n".
                     "\t mode is ".$my_mode."\n".
                     "\t nlink is ".$my_nlink."\n".
                     "\t uid is ".$my_uid."\n".
                     "\t gid is ".$my_gid."\n".
                     "\t rdev is ".$my_rdev."\n".
                     "\t size is ".$my_size."\n".
                     "\t atime is ".$my_atime."\n".
                     "\t ctime is ".$my_ctime."\n".
                     "\t blksize is ".$my_blksize."\n".
                     "\t blocks is ".$my_blocks."\n");
}

=head2 verify_stat_by_files 

Method to compare the nlink, and size stats of 2 files.

=over

=item Arguments

B<msg> => I<string>

Required: The prefix message for the log comment.

B<src_file> => I<NOTE::File>

Required. Handle to the source file for comparison.

B<dst_file> => I<NOTE::File>

Required. Handle to the destination file for comparison.

=item Examples

=back

    # Verify that the 2 files had some nlink, and size :
	my $nfs = $vol->nfs ();
	my $testfile = $nfs->create ("testfile");
	my $hardlink = $testfile->link;
    Hardlinks::verify_stat_by_files ("(Hardlinks08::do_setup"): Debug msg", $testfile, $hardlink);

=cut

sub verify_stat_by_files {
    my ($msg, $src_file, $dst_file) = @_;

    my $src_nlink = $src_file->nlink;
    my $src_size = $src_file->size;

    my $dst_nlink = $dst_file->nlink;
    my $dst_size = $dst_file->size;

    croak "".$msg.", file ".$src_file->path.
          " in DST volume is different from one in the SRC volume. ".
          " SRC's nlink = ".$src_nlink." size = ".$src_size.
          " DST's nlink = ".$dst_nlink." size = ".$dst_size."\n"
          unless ($dst_nlink == $src_nlink && $dst_size == $src_size);
}

1;
