# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/dp/snapmirror/qsm/initialize/QSMInitSpecialDirs.pm#1 $
#
# Copyright 2005 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##  Common code for testing SnapMirror initialize to special directories.
##
## @description
##  Verify that a QSM initialize fails if the given destination qtree
##  is a special directory. (I.e. the etc directory of the root volume,
##  the home directory of same, "-".)
##
## @status private Only for use by tests in dp/snapmirror/qsm/initialize.
##
## @author orosz@netapp.com

package dp::snapmirror::qsm::initialize::QSMInitSpecialDirs;

use strict;
use Tharn qw(logresult);
use SnapMirror qw(snapmirror_initialize);
use NOTE;

sub setup {
    my ($FILER_SRC, $FILER_DST) = @_;

    my $filer_src = NOTE::Filer->connect(host => $FILER_SRC);
    my $filer_dst = NOTE::Filer->connect(host => $FILER_DST);

    my $vol_src  = NOTE::Volume->create(
        Filer => $filer_src,
        exportfs => "anon=0",
        size => "25m"
    );
    my $qtree_src = NOTE::Qtree->create(
        Volume => $vol_src,
    );
    # Leaving the source qtree unpopulated since only testing initialize
    my $root_vol = $filer_dst->vol_root;
    my $vol_dst = NOTE::Volume->attach(
        Filer => $filer_dst,
        name => $root_vol
    );
    return ($qtree_src, $vol_dst);
}

sub test {
    my ($qtree_src, $vol_dst, $qtree_dst, $case_msg, $fail_msg) = @_;

    # Hokey hackery follows. To wit: NOTE::SnapMirror->initialize ultimately
    # calls SnapMirror::Helper::sm_init, which does friendly things like set
    # up snapmirror.conf and sometimes friendly things like checking that the
    # destination doesn't exist.  When trying to test the ONTAP error when the
    # destination does exist, one can't use sm_init, because the error will be
    # returned from it, not from "snapmirror initialize".  To get the ONTAP
    # error one must call SnapMirror::snapmirror_initialize, which is what
    # sm_init eventually calls.  The code commented out is what should work.
    #   eval {
    #       NOTE::SnapMirror->initialize(
    #           src => $qtree_src,
    #           dst => $vol_dst,
    #           name => $qtree_dst
    #       )
    #   };
    #   unless ($@) {
    #       die $case_msg."succeeded when it should have failed.";
    #   }
    eval {
        my $f = $vol_dst->Filer;
        my $dst = $f->host .":"."/vol/". $vol_dst->name ."/". $qtree_dst;
        snapmirror_initialize(
            host => $f->hostrec,
            dst => $dst,
            src => $qtree_src->hostpath
        )
    };
    unless ($@) {
        die $case_msg."succeeded when it should have failed.";
    }
    if ($@ =~ /$fail_msg/i ) {
        logresult('PASS', $case_msg."failed as expected.");
    } else {
        die("snapmirror_initialize returned:\n$@\nExpected: $fail_msg");
    }
}

1;
