
# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/dp/snapmirror/qsm/initialize/initCifs.pm#1 $
# Copyright 2005 Network Appliance, Inc.
# All rights reserved.

## @summary 
##	Shared code for CIFS setup and teardown for QSM initialize scripts
## @status private
## @author ram@netapp.com

package dp::snapmirror::qsm::initialize::initCifs;

use strict;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw(common_cifs_setup common_cifs_teardown 
					create_and_map_cifs_share unmap_and_delete_cifs_share);

use Tharn;
use CIFS::Setup qw(is_cifs_setup);
use SPARSE::Util::Filer_Utils qw( setup_cifs );
use ExportFS qw(share_add share_delete);
use MapShare qw(map_cifs_share unmap_cifs_share);
use QtreeCmds qw(qtreeSecurity);

my $DST_CIFS_FLG = 0; # assume CIFS isn't setup
my $SRC_CIFS_FLG = 0;
my $SRC_SHARE = "src";
my $CIFS_USER = "administrator";
my $CIFS_PASSWD = "sundance";
my $DST_DRIVE = "*";
my $SRC_DRIVE = "*";
my $CURR_DRIVE;

sub _setup_filer_cifs
{
	my ($FILER) = @_;

	setup_cifs($FILER);
}

sub _terminate_filer_cifs
{
	my ($dstvol, $srcvol) = @_;

	sayEOF($dstvol->Filer()->conn(), "cifs terminate");
	sayEOF($srcvol->Filer()->conn(), "cifs terminate");
}

sub create_and_map_cifs_share 
{
	my ($vol, $sharename, $client) = @_;

	share_add($vol->Filer()->conn(), $sharename, $vol->mount_path(), 
			  verify => 1);

	return map_cifs_share( 
					server 		=> $vol->Filer()->conn()->connhost(),
					client		=> $client,
					drive		=> $SRC_DRIVE,
					share		=> $sharename,
					username 	=> $CIFS_USER, 
					password 	=> $CIFS_PASSWD,
					) or croak $@;
}

sub unmap_and_delete_cifs_share
{
	my ($vol, $sharename, $client, $drive) = @_;

	unmap_cifs_share(
		drive	=> $drive,
		client	=> $client
		) or croak $@;

	share_delete($vol->Filer()->conn(), $sharename);
}

sub common_cifs_setup
{
	my ($dstvol, $srcvol, $client) = @_;
		
	$DST_CIFS_FLG = is_cifs_setup(host => $dstvol->Filer()->hostrec());
	$SRC_CIFS_FLG = is_cifs_setup(host => $srcvol->Filer()->hostrec());
	
	_setup_filer_cifs($dstvol->Filer()->hostrec())
		unless ($DST_CIFS_FLG == 1);
	_setup_filer_cifs($srcvol->Filer()->hostrec())
		unless ($SRC_CIFS_FLG == 1 || 
				$dstvol->Filer()->hostrec() eq $srcvol->Filer()->hostrec());

	qtreeSecurity($srcvol->Filer()->conn(), $srcvol->mount_path(), "mixed")
		or croak $@;
	qtreeSecurity($dstvol->Filer()->conn(), $dstvol->mount_path(), "mixed")
		or croak $@;

	logcomment("create and map cifs share $srcvol $SRC_SHARE $client \n");
	$CURR_DRIVE = create_and_map_cifs_share($srcvol, $SRC_SHARE, $client);
	return $CURR_DRIVE;
}

sub common_cifs_teardown
{
	my ($dstvol, $srcvol, $client) = @_;

	unmap_and_delete_cifs_share($srcvol, $SRC_SHARE, $client, $CURR_DRIVE);

	_terminate_filer_cifs($dstvol->Filer()->hostrec())
		unless ($DST_CIFS_FLG == 1);
	_terminate_filer_cifs($srcvol->Filer()->hostrec())
		unless ($SRC_CIFS_FLG == 1 || 
				$dstvol->Filer()->hostrec() eq $srcvol->Filer()->hostrec());

	unlicense($dstvol->Filer()->conn(), "CIFS") or croak $@
		unless ($DST_CIFS_FLG == 1);
	unlicense($srcvol->Filer()->conn(), "CIFS") or croak $@
		unless ($SRC_CIFS_FLG == 1 || 
				$dstvol->Filer()->hostrec() eq $srcvol->Filer()->hostrec());
}

1;

