# this is basically -*- perl -*-
# $Id: //depot/prod/test/main/dp/snapvault/vif/util.pm#1 $
#
## @summary Utilities for vif functions
## @author      indra@netapp.com
## @status      private
## @pod here


package dp::snapvault::vif::util;

use strict;
use warnings;
use Network::Vif qw(vif_create vif_destroy verify_vif_status);
use Network::Utils qw(setup_interfaces do_ifconfig get_ifconfig_info verify_ifconfig_info);
use Params::Validate qw(validate validate_pos BOOLEAN SCALAR OBJECT HASHREF);
use Data::Dumper;
use Tharn;

#---------------------------------------------------------------------
# Function prototypes
#---------------------------------------------------------------------
sub vif_setup;
sub vif_cleanup ;
sub sv_lifecycle;
sub ping;

sub ping{
    my ($fc,$ip) = @_;
    my (undef, undef, $out) = say($fc, "priv set test \; ping $ip");
    return 1 if($out =~ /alive/);
}

sub vif_setup{

    my (%args) = validate( @_, { fc => { type => HASHREF,
                                         required => 1 },
                                 vif_type => { type => SCALAR,
                                              required => 1,
                                              regex => qr/single|multi|lacp/ },
                                 load_balance => { type => SCALAR,
                                                  required => 0,
                                                  optional => 1,
                                                  regex => qr/rr|mac|ip|port/ },
                                 vif_name => { type => SCALAR,
                                              required => 1},
                                 vif_ip =>   { type => SCALAR,
                                              required => 0,
                                              optional => 1,},
                                 vif_int1 => { type => SCALAR,
                                              required => 1,},
                                 vif_int2 => { type => SCALAR,
                                              required => 1,},
                               } );


    my ($fc, $vif_type, $vif_name, $vif_ip, $vif_int1, $vif_int2, $load_balance);
    $fc = $args{'fc'};
    $vif_type = $args{'vif_type'};
    $vif_name = $args{'vif_name'};
    $vif_ip = $args{'vif_ip'};
    $vif_int1 = $args{'vif_int1'};
    $vif_int2 = $args{'vif_int2'};
    $load_balance = $args{'load_balance'};
    
    my $if1_state = get_ifconfig_info($fc, $vif_int1);
    my $if2_state = get_ifconfig_info($fc, $vif_int2);

    my $ifc_list = $vif_int1." ".$vif_int2;
    
    NOTE::logcomment("Bringing down the interfaces for VIF configuration");
    do_ifconfig($fc, $vif_int1, ip_addr => "0.0.0.0") or
        die $@;
    do_ifconfig($fc, $vif_int2, ip_addr => "0.0.0.0") or
        die $@;
    NOTE::logcomment("All interfaces are down");

    ## Create a multi mode vif using the specified interfaces and assign
    ## a unique IP address.  

    NOTE::logcomment("Creating vif using specified interfaces");
    vif_create( $fc,  
        name => $vif_name, 
        type => $vif_type,
        ifc_list => $ifc_list, 
        ip_addr => $vif_ip,
        load_balance => $load_balance
        ) or die(fail_result("vif creation failed - $@"));

    verify_vif_status($fc, $vif_name, type => $vif_type) or
        die(fail_result("vif type is not ".$vif_type));

    NOTE::logcomment("vif $vif_name created as ".$vif_type." vif");    

    ## Invoke ping from the client and verify the response
    ## Verify the vif status.  
    sleep(10);

    NOTE::logcomment("client and filer can communicate using vif $vif_name");
    my %filer_if_state;
    $filer_if_state{$vif_int1} = $if1_state->{$vif_int1};
    $filer_if_state{$vif_int2} = $if2_state->{$vif_int2};
    return \%filer_if_state;
}

sub vif_cleanup {
    my (%args) = validate( @_, { fc => { type => HASHREF,
                                         required => 1 },
                                 if_state => { type => HASHREF,
                                              required => 1 },
                                 vif_name => { type => SCALAR,
                                              required => 1},
                               } );
    my ($fc, $if_state , $vif_name);
    $fc = $args{'fc'};
    $if_state = $args{'if_state'};
    $vif_name = $args{'vif_name'};

    NOTE::logcomment("Destroying vif on specified interfaces");
    vif_destroy( $fc,  $vif_name ) 
        or die(fail_result("vif destroy failed - $@"));

    foreach my $if (keys %{$if_state}){
        $if_state->{$if}{'state'} = $if_state->{$if}{'up'}? 'up':'down';
        NOTE::logcomment("Configuring $if with IP: ".$if_state->{$if}{'ip_addr'});
        do_ifconfig($fc, $if, 
                   ip_addr => $if_state->{$if}{'ip_addr'},
                   state => $if_state->{$if}{'state'},
                   netmask => $if_state->{$if}{'netmask'});

        NOTE::logcomment("Interface state after reset: ".Dumper(get_ifconfig_info($fc, $if)));        

        verify_ifconfig_info($fc, $if, 
                   ip_addr => $if_state->{$if}{'ip_addr'},
                   state => $if_state->{$if}{'state'},
                   netmask => $if_state->{$if}{'netmask'});
        
    }
}

sub sv_lifecycle{
    my (%args) = validate( @_, { src_fc => { type => HASHREF,
                                              required => 1 },
                                 dst_fc => { type => HASHREF,
                                              required => 1},
                                 src_ip => { type => SCALAR,
                                              required => 1 },
                                 dst_ip => { type => SCALAR,
                                              required => 1},
                                 src_qt => { type => SCALAR,
                                              required => 1,},
                                 dst_qt => { type => SCALAR,
                                              required => 1,},
                               } );
    my ($src_fc, $dst_fc, $src_qt, $dst_qt, $src_ip, $dst_ip);
    $src_qt = $args{'src_qt'};
    $dst_qt = $args{'dst_qt'};
    $src_ip = $args{'src_ip'};
    $dst_ip = $args{'dst_ip'};    

    logcomment("Inside Lifecycle");

    $src_fc = $args{'src_fc'};
    $src_fc->{'connhost'} = $src_ip;  
    die "Unable to ping to $dst_ip from Source filer" if(!ping($src_fc,$dst_ip));
    
    $dst_fc = $args{'dst_fc'};    
    $dst_fc->{'connhost'} = $dst_ip; 
    die "Unable to ping to $src_ip from Destination filer" if(!ping($dst_fc,$src_ip));
    
    SnapVault::sv_start(
        $dst_fc,
        $dst_qt,
        SOURCE => $src_ip.":".$src_qt,
        COMPLETE => 1
    );
    # Check for errors
    #die "Snapvault start from $src_ip:$src_qt to $dst_ip:$dst_qt Failed: ".$@ if $@;
    SnapVault::sv_start(
        $dst_fc,
        $dst_qt,
        SOURCE => $src_ip.":".$src_qt,
        COMPLETE => 1
    ) if $@;

    sleep 1600;

    SnapVault::sv_update(
        $dst_fc,
        $dst_qt,
        COMPLETE => 1
    );
    # Check for errors
    die "Snapvault update from $src_ip:$src_qt to $dst_ip:$dst_qt Failed: ".$@ if $@;

    SnapVault::sv_restore(
        $src_fc,
        $src_qt,
        SOURCE => $dst_ip.":".$dst_qt,
        COMPLETE => 1
    );
    # Check for errors
    die "Snapvault restore from $src_ip:$src_qt to $dst_ip:$dst_qt Failed: ".$@ if $@;

    $src_fc = $args{'src_fc'};
    #$src_fc->{'connhost'} = $src_ip;
    die "Unable to ping to $dst_ip from Source filer" if(!ping($src_fc,$dst_ip));

    $dst_fc = $args{'dst_fc'};
    #$dst_fc->{'connhost'} = $dst_ip;
    die "Unable to ping to $src_ip from Destination filer" if(!ping($dst_fc,$src_ip));

    sleep 300;

    $src_fc = $args{'src_fc'};
    #$src_fc->{'connhost'} = $src_ip;
    die "Unable to ping to $dst_ip from Source filer" if(!ping($src_fc,$dst_ip));

    $dst_fc = $args{'dst_fc'};
    #$dst_fc->{'connhost'} = $dst_ip;
    die "Unable to ping to $src_ip from Destination filer" if(!ping($dst_fc,$src_ip));
     
    SnapVault::sv_resync(
        $dst_fc,
        $dst_qt,
        SOURCE => $src_ip.":".$src_qt,
        COMPLETE => 1
    );
   # Check for errors
    die "Snapvault resync from $src_ip:$src_qt to $dst_ip:$dst_qt Failed: ".$@ if $@;

}

1;
