#!/usr/local/bin/perl

package NEWSMParser;

use strict;
use warnings;

use Date::Manip;
use Date::Parse;

require Exporter;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw(get_transfer_duration get_diff_in_sec print_transfer_duration to_dblade_time_format parse_sysstat get_stats_log_start_time);
use strict;

#get the start time from stats log 
sub get_stats_log_start_time($)
{
	my ($log_record) = @_;
	my ($stats_start_time,$interval)=(-1,-1);
	if ($log_record=~/^LOG>\s*Start:/ ) {
		($stats_start_time, $interval)
			=  $log_record
			=~ /^\s*LOG>\s*Start:(\w+\s+\w+\s+\d+\s+\d+:\d+:\d+\s+\w+\s+\d+)\s+@\s*(\d+).*$/;
	}
	return($stats_start_time,$interval);
}

sub parse_sysstat($$)
{
    my ($type, $record) = @_;
    my @data=();
    $record =~ s/^\s+//;
    $record =~ s/[()]/ /g;

    if ( $type =~ /SYSSTAT_X/ ) {
	    @data = split (/\s+/,$record);
    } elsif ($type =~ /SYSSTAT_M/ ) { 
	    @data = $record =~ /[0-9]+%*[ ]?/g;
    }

    return @data;
}

sub oldget_transfer_duration($) {
    my ($mgwd_log) = @_;
    my ($transfers_started, $transfers_ended, $transfers_failed,
        $data_started, $data_ended, 
	$first_transfer_start_time , $last_transfer_start_time,
	$first_transfer_end_time, $last_transfer_end_time,
	$first_data_transfer_start_time, $last_data_transfer_start_time,
	$first_data_transfer_end_time, $last_data_transfer_end_time,
	$data_transfer_duration, $total_transfer_duration) =();

    open (SMLOGSRC,"<$mgwd_log") or die "Cannot open $mgwd_log";

    my ($REG_START, $REG_SN_START, $REG2, $REG3, $REG_END, $REG_SN_END, $REG_SN_END_ERR, $REG44, $REG5) = ();

    # Compile our regular expressions 

    # $REG1 = qr/INFO:SnapMirror::Update:(\d+) action=Start source=(\S+) destination=(\S+)\s*$/;
    # $REG2 = qr/INFO:SnapMirror::Update:(\d+) action=Start_snapshot base=(\S+) target=(\S+) source=(\S+) destination=(\S+)\s*$/;
    # $REG3 = qr/INFO:SnapMirror::Update:(\d+) action=End_snapshot base=(\S+) target=(\S+) size=(\S+) source=(\S+) destination=(\S+)\s*$/;
    # $REG_END = qr/INFO:SnapMirror::Update:(\d+) action=End status=(\w+) source=(\S+) destination=(\S+)\s*$/;
    # $REG5 = qr/.*Defer.*/;

    $REG_START = qr/.*?:(\d+) action=Start source=(\S+) destination=(\S+)\s*$/;
    $REG_SN_START = qr/.*?:(\S+) action=Start source=(\S+) destination=(\S+)\s*$/;
    $REG2 = qr/.*?:(\d+) action=Start_snapshot base=(\S+) target=(\S+) source=(\S+) destination=(\S+)\s*$/;
    $REG3 = qr/.*?(\d+) action=End_snapshot base=(\S+) target=(\S+) size=(\S+) source=(\S+) destination=(\S+)\s*$/;
    $REG_END = qr/.*?:(\d+) action=End status=(\w+) source=(\S+) destination=(\S+)\s*$/;
    $REG_SN_END = qr/.*?:(\S+) action=End source=(\S+) destination=(\S+) status=(\w+) bytes_transferred=(\S+)/;
    $REG_SN_END_ERR = qr/.*?:(\S+) action=End source=(\S+) destination=(\S+) status=(.+\)) bytes_transferred=(\S+)$/;
    $REG44 = qr/.*?:(\d+) action=End status=(\w+) .*? size=(\S+) source=(\S+) destination=(\S+)\s*$/;
    $REG5 = qr/.*Defer.*/;

    while (<SMLOGSRC>)
    {
       # Strip the line into its basic components.  time and location seem
       #  to be added by the generic logging mechanism, the rest is what
       #  the application asks to print
       my ($time, $loc, $msg) = $_ =~ m/(.*?) (\[.*?\]) (.*)/;

       if (!$time || !$loc || !$msg) {
              next; # couldn't recognize it, so let's skip it
       }

       # There's some bizzare cruft at the beginning of each line,
       #  before the timestamp.  Junk it.
       $time =~ s/(.*?) (.*?) //;

       # Check to see if it looks like a message from SnapMirror
       if ($msg =~ $REG_START) {
	  $transfers_started++;
	  if (!$first_transfer_start_time) {
	  	$first_transfer_start_time = $time;
	  }
	  $last_transfer_start_time = $time ;
       } elsif ($msg =~ $REG2) {
	  $data_started++;
	  if (!$first_data_transfer_start_time) {
	  	$first_data_transfer_start_time = $time;
	  }
	  $last_data_transfer_start_time = $time;
       } elsif ($msg =~ $REG3) {
	  $data_ended++;
	  if (!$first_data_transfer_end_time) {
	  	$first_data_transfer_end_time = $time;
	  }
	  $last_data_transfer_end_time = $time;
       } elsif ($msg =~ $REG44) {
	  $transfers_ended++;
	  if (!$first_transfer_end_time) {
	  	$first_transfer_end_time = $time;
	  }
	  $last_transfer_end_time = $time;

       } elsif ($msg =~ $REG5) {
	   $transfers_failed++;
	   print $msg. "\n";
       } 

    }

    close(SMLOGSRC);

    if ($transfers_started != $transfers_ended ) {
	    printf ("Missing Transfer log entries\n");
    }

    if ($data_started != $data_ended ) {
	    printf ("Missing Snapshot log entries\n");
    }

    # $data_transfer_duration = get_diff_in_sec(to_dblade_time_format($last_data_transfer_start_time),
    #                                          to_dblade_time_format($first_data_transfer_end_time) ) ;

    $data_transfer_duration = get_diff_in_sec($first_data_transfer_end_time, $last_data_transfer_start_time);

    print "Xfers Started: $transfers_started \n";
    print "Xfers Ended: $transfers_ended \n";
    if (defined $transfers_failed) { print "**** Xfers Failed  . $transfers_failed *****\n"; }
    print "Last data Xfer start: $last_data_transfer_start_time (Epoch seconds),  " . localtime($last_data_transfer_start_time) . " \n";
    print "First data Xfer start: $first_data_transfer_end_time (Epoch seconds),  " . localtime($first_data_transfer_end_time) . " \n";
    print "Parallel data Xfer duration: $data_transfer_duration \n";


    if ($data_transfer_duration =~ /Invalid/)
    {
        $last_data_transfer_start_time = -1;

    }

    # We decrement the duration time by 60 seconds to remove any
    # calculation errors that may arise due to skew in mgwd.log
    # timings. This skew can be caused due to snapmirror status
    # being checked atleast a minute after the transfer ends.
    if ( $data_transfer_duration > 120 )
    {
        $data_transfer_duration -= 60;

    }

    return ($last_data_transfer_start_time,$data_transfer_duration);
}

# Arguments: snapmirror log file, start time to begin with, number of transfers
#
# Returns: $first_data_transfer_start_time,$data_transfer_duration, $transfers_ended, $totalxsize
#
sub get_transfer_duration($$$) {

        # starttime will be in epoch seconds
        # my ($mgwd_log, $starttime, $numtransfers) = @_;
        my ($mgwd_log, $starttime, $numtransfers) = @_;

        my ($transfers_started, $transfers_ended, $transfers_failed,
                $data_started, $data_ended, 
                $first_transfer_start_time, $first_transfer_start_time_string, 
                $last_transfer_start_time, $last_transfer_start_time_string,
                $last_start_line, $last_end_line,
                $first_transfer_end_time, $first_transfer_end_time_string, 
                $last_transfer_end_time, $last_transfer_end_time_string, 
                $first_data_transfer_start_time, $last_data_transfer_start_time,
                $first_data_transfer_end_time, $last_data_transfer_end_time,
                $data_transfer_duration, $total_transfer_duration) =();

        my $totalxsize = 0;
        my $xsize = 0;

        open (SMLOGSRC,"<$mgwd_log") or die "Cannot open $mgwd_log";

        my ($REG_START, $REG_SN_START, $REG2, $REG3, $REG_END, $REG_SN_END, $REG_SN_END_ERR, $REG44, $REG5) = ();
        my ($REG7MODE_START, $REG7MODE_END) = ();
        my ($REG_CREATESNAPDST_START, $REG_CREATESNAPDST_END) = ();
        my $Is7Mode = 0;
        my $IsSN = 0;
        my $firsttime = 1;
        my $linenum;
        my $xfernumstart = 0;
        my $xfernumend = 0;
        # my @TransferTab = ();
        my %TransferTab = ();


        # Compile our regular expressions 

        $REG_START = qr/.*?:(\d+) action=Start source=(\S+) destination=(\S+)\s*$/;
    	$REG_SN_START = qr/.*?:(\S+) action=Start source=(\S+) destination=(\S+)\s*$/;
    	$REG_CREATESNAPDST_START = qr/.*? CreateSnapshotdst:(\S+) action=Start source=(\S+) destination=(\S+)\s*$/;
        # These are ABSENT CURRENTLY in the SnapMirror logs ...
        # $REG2 = qr/.*?:(\d+) action=Start_snapshot base=(\S+) target=(\S+) source=(\S+) destination=(\S+)\s*$/;
        # $REG3 = qr/.*?(\d+) action=End_snapshot base=(\S+) target=(\S+) size=(\S+) source=(\S+) destination=(\S+)\s*$/;
        $REG_END = qr/.*?:(\d+) action=End status=Success size=(\S+) source=(\S+) destination=(\S+)\s*$/;
    	$REG_SN_END = qr/.*?:(\S+) action=End source=(\S+) destination=(\S+) status=(\w+) bytes_transferred=(\S+)/;
    	$REG_SN_END_ERR = qr/.*?:(\S+) action=End source=(\S+) destination=(\S+) status=(.+\)) bytes_transferred=(\S+)/;
    	$REG_CREATESNAPDST_END = qr/.*? CreateSnapshotDest:(\S+) action=End source=(\S+) destination=(\S+) status=(\w+)/;
    	$REG44 = qr/.*?:(\d+) action=End status=(\w+) .*? size=(\S+) source=(\S+) destination=(\S+)\s*$/;
        $REG5 = qr/.*Defer.*/;

        # looks like "... :/vol/large_no_d_dst_8/large_qtree_ds Start"
        $REG7MODE_START = qr/.*?\sStart$/; 

        # looks like "... :/vol/large_no_d_dst_1/large_qtree_ds End (136847380 KB)"
        $REG7MODE_END = qr/.*?\sEnd \((\d+) (\w+)\)$/;

        $transfers_started = 0;
        $Is7Mode = 0;
        $IsSN = 0;
        $firsttime = 1;
        $transfers_ended = 0;
        $last_transfer_start_time = 0;
        $linenum = 0;
        READLOGFILE:while (<SMLOGSRC>) {
                my $thistime;
                my ($time, $loc, $msg) = ();

                $linenum++;
                if ($firsttime) {
                        ($time, $loc, $msg) = $_ =~ m/(.*?) (\[.*?\]) (.*)/;
                        if (!$time || !$loc || !$msg) {
                                # let's try 7Mode
                                # ($time, $msg) = $_ =~ m/\w+ (.*?) (\d+\.\d+\.\d+\.\d+.*)/;
                                ($time, $msg) = $_ =~ m/^\S+ (.*?\s\d+:\d+:\d+\s\S+) (.*)/;
                                if (!$time || !$msg) {
					($time, $msg) = $_ =~ m/(.*?2012) (.*)/;
					if (!$time || !$msg) {
						next;
					} else {
						$firsttime = 0;
						$IsSN = 1;
					}
                                } else {
                                	$firsttime = 0;
                                	$Is7Mode = 1;
				}
                        }
                        else {
                                $firsttime = 0;
                        }
                }

                # Strip the line into its basic components.  time and location seem
                #  to be added by the generic logging mechanism, the rest is what
                #  the application asks to print
                if ($Is7Mode) {
                        # ($time, $msg) = $_ =~ m/\w+ (.*?) (\d+\.\d+\.\d+\.\d+.*)/;
                        ($time, $msg) = $_ =~ m/^\S+ (.*?\s\d+:\d+:\d+\s\S+) (.*)/;
                        if (!$time || !$msg) {
                                next; # couldn't recognize it, so let's skip it
                        }
		} elsif ($IsSN) {
			
			($time, $msg) = $_ =~ m/(.*?2012) (.*)/;
			if (!$time || !$msg) {
				next;
			} 
		} else {
                        ($time, $loc, $msg) = $_ =~ m/(.*?) (\[.*?\]) (.*)/;
                        if (!$time || !$loc || !$msg) {
                                next; # couldn't recognize it, so let's skip it
                        }
                        # There's some bizzare cruft at the beginning of each line,
                        #  before the timestamp.  Junk it.
                        $time =~ s/(.*?) (.*?) //;
                }

		# Check to see if it looks like a message from SnapMirror
		if ( ($msg =~ $REG_CREATESNAPDST_START) || 
				($msg =~ $REG_CREATESNAPDST_END)) {
			next;
		} elsif ( ($msg =~ $REG_START) || ($msg =~ $REG_SN_START) || ($msg =~ $REG7MODE_START) ) {
                        $thistime = str2time($time);
                        if ($thistime >= $starttime) {
                                $transfers_started++;
                                if (!$first_transfer_start_time) {
                                        $first_transfer_start_time = $thistime;
                                        $first_transfer_start_time_string = $time;
                                }
                                $last_start_line = $linenum;
                                # $TransferTab[$xfernumstart++] =  [$last_start_line, 0]; # the 2nd entry will get filled later
                                $TransferTab{$1} =  [$last_start_line, 0]; # the 2nd entry will get filled later
                        }
                        $last_start_line = $linenum;
                        $last_transfer_start_time = $thistime ;
                        $last_transfer_start_time_string = $time ;

                # } elsif ($msg =~ $REG2) {
                #	  $data_started++;
                #        if (!$first_data_transfer_start_time) {
                #                $first_data_transfer_start_time = $time;
                #        }
                #        $last_data_transfer_start_time = $time;
                # } elsif ($msg =~ $REG3) {
                #         $data_ended++;
                #         if (!$first_data_transfer_end_time) {
                #                 $first_data_transfer_end_time = $time;
                #         }
                #         $last_data_transfer_end_time = $time;
                } elsif (($msg =~ $REG_END) && ($first_transfer_start_time)) {
                        if (!defined $TransferTab{$1}) { # let's ignore those whose "Starts" we haven't seen 
                                next;
                        }
                        $thistime = str2time($time);
                        $xsize = $2;
                        $totalxsize += getvalue($xsize);
                        $transfers_ended++;
                        if (!$first_transfer_end_time) {
                                $first_transfer_end_time = $thistime;
                                $first_transfer_end_time_string = $time;
                        }
                        $last_transfer_end_time = $thistime;
                        $last_transfer_end_time_string = $time;
                        $last_end_line = $linenum;
                        # $TransferTab[$xfernumend++][1] =  $last_end_line;
                        $TransferTab{$1}[1] =  $last_end_line;
                        if ($transfers_ended == $numtransfers) {
                                last READLOGFILE;
                        }
                
		} elsif ((($msg =~ $REG_SN_END) || ($msg =~ $REG_SN_END_ERR)) 
				&& ($first_transfer_start_time)) {
                        if (!defined $TransferTab{$1}) { # let's ignore those whose "Starts" we haven't seen 
                                next;
                        }
                        $thistime = str2time($time);
                        $xsize = $5;
                        $totalxsize += getvalue($xsize);
                        $transfers_ended++;
                        if (!$first_transfer_end_time) {
                                $first_transfer_end_time = $thistime;
                                $first_transfer_end_time_string = $time;
                        }
                        $last_transfer_end_time = $thistime;
                        $last_transfer_end_time_string = $time;
                        $last_end_line = $linenum;
                        # $TransferTab[$xfernumend++][1] =  $last_end_line;
                        $TransferTab{$1}[1] =  $last_end_line;
                        if ($transfers_ended == $numtransfers) {
                                last READLOGFILE;
                        }

		} elsif ($msg =~ $REG44 && ($first_transfer_start_time)) {
                        if (!defined $TransferTab{$1}) { # let's ignore those whose "Starts" we haven't seen 
                                next;
                        }
                        $thistime = str2time($time);
                        $xsize = $3;
                        $totalxsize += getvalue($xsize);
                        $transfers_ended++;
                        if (!$first_transfer_end_time) {
                                $first_transfer_end_time = $thistime;
                                $first_transfer_end_time_string = $time;
                        }
                        $last_transfer_end_time = $thistime;
                        $last_transfer_end_time_string = $time;
                        $last_end_line = $linenum;
                        # $TransferTab[$xfernumend++][1] =  $last_end_line;
                        $TransferTab{$1}[1] =  $last_end_line;
                        if ($transfers_ended == $numtransfers) {
                                last READLOGFILE;
                        }
                } elsif ($msg =~ $REG7MODE_END  && ($first_transfer_start_time)) {
                        if (!defined $TransferTab{$1}) { # let's ignore those whose "Starts" we haven't seen 
                                next;
                        }
                        $thistime = str2time($time);
                        $xsize = $1 . $2;
                        $totalxsize += getvalue($xsize);
                        $transfers_ended++;
                        if (!$first_transfer_end_time) {
                                $first_transfer_end_time = $thistime;
                                $first_transfer_end_time_string = $time;
                        }
                        $last_transfer_end_time = $thistime;
                        $last_transfer_end_time_string = $time;
                        $last_end_line = $linenum;
                        # $TransferTab[$xfernumend++][1] =  $last_end_line;
                        $TransferTab{$1}[1] =  $last_end_line;
                        if ($transfers_ended == $numtransfers) {
                                last READLOGFILE;
                        }
                } elsif ($msg =~ $REG5 && ($first_transfer_start_time)) {
                        $transfers_failed++;
                        # print $msg. "\n";
                } 
        }

        close(SMLOGSRC);

        # $TransferTab[$xfernumend++][1] =  $last_end_line;

        if ($transfers_started != $transfers_ended ) {
                printf ("Missing Transfer log entries, $transfers_started and $transfers_ended ended\n");
        }

        my $tmp;
        $tmp = localtime($starttime);
        printf ("Starttime requested at Epoch %d secs (%s)\n", $starttime, $tmp);

        if ($first_transfer_start_time) {
                $tmp = localtime($first_transfer_start_time);
                printf ("Starttime found at Epoch %d secs (%s)\n", $first_transfer_start_time, $tmp);
        }

        $tmp = localtime($last_transfer_start_time);
        printf ("Last Start seen in SnapMirror log %s is on line %d, at Epoch %d secs (%s)\n",  $mgwd_log, $last_start_line, $last_transfer_start_time, $tmp);

        $tmp = localtime($last_transfer_end_time);
        printf ("Last End seen in SnapMirror log %s is on line %d, at Epoch %d secs (%s)\n",  $mgwd_log, $last_end_line, $last_transfer_end_time, $tmp);


        # Print the transfer start and end line numbers (used mainly for debugging)
        #
        # for (my $i = 0; $i < $transfers_ended; $i++) {
        #         printf ("\tTransfer number %d, start and end lines = %d, %d\n", $i + 1, $TransferTab[$i][0], $TransferTab[$i][1]);
        # }

        printf ("\n\nPrinting transfer start and end line numbers in file %s\n\n", $mgwd_log);
        printf ("REGEXPs used for these searches are:\n");
        printf ("\t\t $REG_START\n");
        printf ("\t\t $REG_SN_START\n");
        printf ("\t\t $REG_END\n");
        printf ("\t\t $REG_SN_END\n");
        printf ("\t\t $REG_SN_END_ERR\n");
        printf ("\t\t $REG44\n");
        printf ("\t\t $REG7MODE_END\n");
        printf ("\n\n");

        for (keys %TransferTab ) {
                if ($TransferTab{$_}[0] > 0  && $TransferTab{$_}[1] > 0 ) {
                        printf ("\tTransfer number %d, start and end lines = %d, %d\n", $_, $TransferTab{$_}[0], $TransferTab{$_}[1]);
                }
                else {
                        printf ("\t**ERROR ** Transfer number %d, has one or more undefined values for start and end lines = %d, %d\n", $_, $TransferTab{$_}[0], $TransferTab{$_}[1]);
                }
        }

        printf ("\n\n");
        if ($transfers_ended != $numtransfers ) {
                printf ("ERROR: transfers requested is $numtransfers AND not equal to tranfers ended which is $transfers_ended\n");
                return (0, 0, 0, 0); 
        }

        # $data_transfer_duration = $first_data_transfer_end_time - $last_data_transfer_start_time;
        $data_transfer_duration = $last_transfer_end_time - $first_transfer_start_time;

        print "Xfers Started: $transfers_started \n";
        print "Xfers Ended: $transfers_ended \n";
        if (defined $transfers_failed) { 
                print "**** Xfers Failed  . $transfers_failed *****\n"; 
        }
        # print "First data Xfer start: $first_transfer_start_time (Epoch seconds),  " . localtime($first_transfer_start_time) . " \n";
        # print "Last data Xfer start: $last_transfer_start_time (Epoch seconds),  " . localtime($last_transfer_start_time) . " \n";
        # print "First data Xfer end: $first_transfer_end_time (Epoch seconds),  " . localtime($first_transfer_end_time) . " \n";
        # print "Last data Xfer end: $last_transfer_end_time (Epoch seconds),  " . localtime($last_transfer_end_time) . " \n";

        print "First data Xfer start: $first_transfer_start_time (Epoch seconds),  " . $first_transfer_start_time_string . " \n";
        print "Last data Xfer start: $last_transfer_start_time (Epoch seconds),  " . $last_transfer_start_time_string . " \n";
        print "First data Xfer end: $first_transfer_end_time (Epoch seconds),  " . $first_transfer_end_time_string . " \n";
        print "Last data Xfer end: $last_transfer_end_time (Epoch seconds),  " . $last_transfer_end_time_string . " \n";

        print "Parallel data Xfer duration: $data_transfer_duration \n";


        if ($data_transfer_duration =~ /Invalid/) {
                $last_data_transfer_start_time = -1;
        }

        # We decrement the duration time by 60 seconds to remove any
        # calculation errors that may arise due to skew in mgwd.log
        # timings. This skew can be caused due to snapmirror status
        # being checked atleast a minute after the transfer ends.
        # 4/18/2011 No longer subtracting 60 seconds
        # if ( $data_transfer_duration > 120 ) {
        #         $data_transfer_duration -= 60;
        # }

        return ($first_transfer_start_time, $data_transfer_duration, $transfers_ended, $totalxsize, 
                $first_transfer_start_time_string, $last_transfer_end_time_string); 
}


sub print_transfer_duration($$$$) {
    my ($mgwd_log, $datafile, $sumfile, $transfers) = @_;
    my %SnapMirror =();
    my ($transfers_started, $transfers_ended, $transfers_failed,
        $data_started, $data_ended, 
	$first_transfer_start_time , $last_transfer_start_time,
	$first_transfer_end_time, $last_transfer_end_time,
	$first_data_transfer_start_time, $last_data_transfer_start_time,
	$first_data_transfer_end_time, $last_data_transfer_end_time,
	$data_transfer_duration, $parallel_data_transfer_duration, $total_transfer_duration) =();

    print "#Reading $mgwd_log \n";
    print "#logging to $datafile \n";
    open (SMLOGSRC,"<$mgwd_log");

    while (<SMLOGSRC>)
    {
       # Strip the line into its basic components.  time and location seem
       #  to be added by the generic logging mechanism, the rest is what
       #  the application asks to print
       my ($time, $loc, $msg) = $_=~ m/(.*?) (\[.*?\]) (.*)/;

       if (!$time || !$loc || !$msg) {
              next; # couldn't recognize it, so let's skip it
       }

       # There's some bizzare cruft at the beginning of each line,
       #  before the timestamp.  Junk it.
       $time =~ s/(.*?) (.*?) //;

       # Check to see if it looks like a message from SnapMirror
       if ($msg =~ m/INFO:SnapMirror::Update:(\d+) action=Start source=(\S+) destination=(\S+)\s*$/) {
	  $transfers_started ++;

	  my ($job, $src, $dst) = ($1, $2, $3);
	  $SnapMirror{"${job}_$dst"}->{'Id'} = $job;
	  $SnapMirror{"${job}_$dst"}->{'Src'} = $src;
	  $SnapMirror{"${job}_$dst"}->{'Dst'} = $dst;
	  $SnapMirror{"${job}_$dst"}->{'Start'} = $time;

	  if (!$first_transfer_start_time) {
	  	$first_transfer_start_time = $time;
	  }

	  $SnapMirror{"${job}_$dst"}->{'QueueTime'} = 
                               # get_diff_in_sec(to_dblade_time_format($first_transfer_start_time),
                               #                to_dblade_time_format($time)) ;
	                       get_diff_in_sec($first_transfer_start_time, $time) ;
	  $last_transfer_start_time = $time ;

       } elsif ($msg =~ m/INFO:SnapMirror::Update:(\d+) action=Start_snapshot base=(\S+) target=(\S+) source=(\S+) destination=(\S+)\s*$/) {
	  $data_started ++;
	  if (!$first_data_transfer_start_time) {
	  	$first_data_transfer_start_time = $time;
	  }
	  my ($job, $base, $target, $src, $dst) = ($1, $2, $3, $4, $5);
	  $SnapMirror{"${job}_$dst"}->{'Base'} = $base;
	  $SnapMirror{"${job}_$dst"}->{'Target'} = $target;
	  $SnapMirror{"${job}_$dst"}->{'StartSnapshot'} = $time;
	  $SnapMirror{"${job}_$dst"}->{'SetupTime'} = 
                                # get_diff_in_sec(to_dblade_time_format($SnapMirror{"${job}_$dst"}->{'Start'}),
                                #                to_dblade_time_format($time)) ;
	  			get_diff_in_sec(SnapMirror{"${job}_$dst"}->{'Start'}, $time) ;

	  $last_data_transfer_start_time = $time;

       } elsif ($msg =~ m/INFO:SnapMirror::Update:(\d+) action=End_snapshot base=(\S+) target=(\S+) size=(\S+) source=(\S+) destination=(\S+)\s*$/) {
	  $data_ended ++;
	  if (!$first_data_transfer_end_time) {
	  	$first_data_transfer_end_time = $time;
	  }
	  my ($job, $base, $target, $size, $src, $dst) = ($1, $2, $3, $4, $5, $6);
	  $SnapMirror{"${job}_$dst"}->{'EndSnapshot'} = $time;
	  $SnapMirror{"${job}_$dst"}->{'Size'} = $size;
	  $SnapMirror{"${job}_$dst"}->{'XferTime'} = 
	  			get_diff_in_sec($SnapMirror{"${job}_$dst"}->{'StartSnapshot'}, $time) ;
                                # get_diff_in_sec(to_dblade_time_format($SnapMirror{"${job}_$dst"}->{'StartSnapshot'}),
                                #                 to_dblade_time_format($time)) ;

	  $last_data_transfer_end_time = $time;


       } elsif ($msg =~ m/INFO:SnapMirror::Update:(\d+) action=End status=Success source=(\S+) destination=(\S+)\s*$/) {
	  $transfers_ended ++;
	  if (!$first_transfer_end_time) {
	  	$first_transfer_end_time = $time;
	  }
	  my ($job, $src, $dst) = ($1, $2, $3);
	  $SnapMirror{"${job}_$dst"}->{'Result'} = "Success";
	  $SnapMirror{"${job}_$dst"}->{'End'} = $time;
	  $SnapMirror{"${job}_$dst"}->{'NumSnaps'} += 1;
	  $SnapMirror{"${job}_$dst"}->{'CleanupTime'} = 
	  			get_diff_in_sec($SnapMirror{"${job}_$dst"}->{'EndSnapshot'}, $time) ;
                                # get_diff_in_sec(to_dblade_time_format($SnapMirror{"${job}_$dst"}->{'EndSnapshot'}),
                                #                to_dblade_time_format($time)) ;

	  $SnapMirror{"${job}_$dst"}->{'TotalXferTime'} = 
	   			get_diff_in_sec($SnapMirror{"${job}_$dst"}->{'Start'}, $time) ;
                                # get_diff_in_sec(to_dblade_time_format($SnapMirror{"${job}_$dst"}->{'Start'}),
                                #                to_dblade_time_format($time)) ;
	  $last_transfer_end_time = $time;

       } elsif ($msg =~ m/INFO:SnapMirrro::Update:(\d+) action=Defer status=(.*) source=(\S+) destination=(\S+)\s*$/ ) {
	   my ($job, $status, $src, $dst) = ($1, $2, $3, $4);
	   $SnapMirror{"${job}_$dst"}->{'End'} = $time;
	   $SnapMirror{"${job}_$dst"}->{'Defer'} = $time;
	   $SnapMirror{"${job}_$dst"}->{'Failed'} += 1;
	   $SnapMirror{"${job}_$dst"}->{'Result'} = "$status";
	   $SnapMirror{"${job}_$dst"}->{'Size'} = 0;
	   $SnapMirror{"${job}_$dst"}->{'TotalXferTime'} = 
	   			get_diff_in_sec($SnapMirror{"${job}_$dst"}->{'Start'}, $time) ;
                                # get_diff_in_sec(to_dblade_time_format($SnapMirror{"${job}_$dst"}->{'Start'}),
                                #                to_dblade_time_format($time)) ;
	   $transfers_failed ++;
	   print " *** ". $msg. " ***\n";

       } 
    }

    if ($transfers_started != $transfers_ended ) {
	    printf ("Missing Transfer log entries\n");
    }

    if ($data_started != $data_ended ) {
	    printf ("Missing Snapshot log entries\n");
    }

    # $data_transfer_duration = get_diff_in_sec(to_dblade_time_format($first_data_transfer_start_time),
    #                                           to_dblade_time_format($last_data_transfer_end_time) ) ;
    $data_transfer_duration = get_diff_in_sec($first_data_transfer_start_time, $last_data_transfer_end_time) ;

    # $parallel_data_transfer_duration = get_diff_in_sec(to_dblade_time_format($last_data_transfer_start_time),
    #                                          to_dblade_time_format($first_data_transfer_end_time) ) ;
    $parallel_data_transfer_duration = get_diff_in_sec($last_data_transfer_start_time, $first_data_transfer_end_time) ;

    # $total_transfer_duration = get_diff_in_sec(to_dblade_time_format($first_transfer_start_time),
    #                                          to_dblade_time_format($last_transfer_end_time) ) ;
    $total_transfer_duration = get_diff_in_sec($first_transfer_start_time, $last_transfer_end_time) ;

    print "Xfers Started: $transfers_started \n";
    print "Xfers Ended: $transfers_ended \n";
    if (defined $transfers_failed) { print "**** Xfers Failed  . $transfers_failed *****\n"; }
    print "Last data Xfer start: $last_data_transfer_start_time \n";
    print "First data Xfer end: $first_data_transfer_end_time \n";
    print "Parallel data Xfer duration: $parallel_data_transfer_duration secs\n";
    print "Data Xfer duration: $data_transfer_duration secs\n";
    print "End to End Xfer duration: $total_transfer_duration secs\n";


    if ($parallel_data_transfer_duration =~ /Invalid/)
    {
        $last_data_transfer_start_time = -1;
    	return ($last_data_transfer_start_time, $parallel_data_transfer_duration);

    }

    print_per_xfer_stats(\%SnapMirror, $total_transfer_duration, $parallel_data_transfer_duration, $data_transfer_duration, $datafile, $sumfile);

    return ($last_data_transfer_start_time, $parallel_data_transfer_duration);
}

sub convertToMB($) {
   my ($val) = @_ ;
   my($size, $units) = (0, 0);
   
   ($size, $units) = $val =~ /([\d]+\.?[\d]*)\s*(\S+)/;

   $size = int($size + 0.5);
	
   if ($units =~ /KB/ ) {
	   return ( $size / 1024);
   } elsif ($units =~ /MB/ ) {
	   return ( $size );
   } elsif ($units =~ /GB/ ) {
	   return ( $size * 1024 );
   } elsif ($units =~ /TB/ ) {
	   return ( $size * 1024 * 1024);
   } else {
	   return ( $size / (1024 * 1024));
   }
}

sub print_per_xfer_stats($$$) {

 my ($reftable, $total_transfer_duration, $parallel_transfer_duration, $data_transfer_duration, $datafile , $sumfile) = @_;

 # check if summary file already exists
 if ( -e $sumfile ) {
     print "#Appending $sumfile for summary data \n";
     open (SUMFILE,">>$sumfile") or die "Cannot open $sumfile for appending";
 } else {
     print "#Creating $sumfile for summary data \n";
     open (SUMFILE,">$sumfile") or die "Cannot open $sumfile for writing";
     print SUMFILE "NumVolumes Result PASSED FAILED TotalKBXferd(MB) TotalTransferTime(sec) ParallelTransferTime(sec) DataTransferTime(sec)  AVGTPUT(MB/sec) ENDTOEND_TPUT(MB/sec) \n";
 }

 print "#Creating $datafile for data \n";
 open (DATAFILE,">$datafile") or die "Cannot open $datafile for writing";
 print DATAFILE "Id\t Sourcevolume\t\tDestVolume\t\tQueueTime(sec)  SetupTime(sec)  DataXferTime(sec) CleanupTime(sec) TotalXferTime(sec)  NumSnaps  Result  TotalKBXferd(MB)  AVGTPUT(MB/sec) \n";

# print per batch stats
 my($total_vols,$run_result, $num_passed, $num_failed, $total_data_size, $avg_tput) = (0, "PASS", 0, 0, 0, 0);
 foreach my $key ( sort keys %$reftable ) {
	my $datasize = convertToMB($reftable->{$key}{'Size'});
 	my $average =  $datasize / $reftable->{$key}{'XferTime'};
	print DATAFILE "$reftable->{$key}->{'Id'}\t";
	print DATAFILE "$reftable->{$key}->{'Src'}\t\t";
	print DATAFILE "$reftable->{$key}->{'Dst'}\t\t";
	print DATAFILE "$reftable->{$key}->{'QueueTime'}\t";
	print DATAFILE "$reftable->{$key}->{'SetupTime'}\t";
	print DATAFILE "$reftable->{$key}->{'XferTime'}\t";
	print DATAFILE "$reftable->{$key}->{'CleanupTime'}\t";
	print DATAFILE "$reftable->{$key}->{'TotalXferTime'}\t";
	print DATAFILE "$reftable->{$key}->{'NumSnaps'}\t";
	print DATAFILE "$reftable->{$key}->{'Result'}\t\t";
	printf DATAFILE "%03.2f\t\t", $datasize;
	printf DATAFILE "%03.2f\t\t", $average;
	print DATAFILE "\n";
	$total_vols++;
	$total_data_size += $datasize;

	if($reftable->{$key}->{'Result'} =~ /FAIL/) {
            $run_result = "FAIL";
	    $num_failed ++;
	} else {
	    $num_passed ++;
	}
 }

 # print summary stats entry for these transfers
  print SUMFILE "$total_vols\t";
  print SUMFILE "$run_result\t";
  print SUMFILE "$num_passed\t";
  print SUMFILE "$num_failed\t";
  print SUMFILE "$total_data_size\t";
  print SUMFILE "$total_transfer_duration\t";
  print SUMFILE "$parallel_transfer_duration\t";
  print SUMFILE "$data_transfer_duration\t";
  
  my $data_avg_tput  = ($total_data_size / $data_transfer_duration) ;
  my $end_to_end_avg_tput  = ($total_data_size / $total_transfer_duration) ;

  printf SUMFILE "%03.2f\t\t", $data_avg_tput;
  printf SUMFILE "%03.2f\t\t", $end_to_end_avg_tput;

  print SUMFILE "\n";

}

sub to_dblade_time_format($) {

    my ($time) = @_;
    my ($dayofweek, $mon, $day, $yr, $hr, $min ,$sec ,$zone ) = $time =~
            /(\w+)\s+(\w+)\s+(\d+)\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\S+).*$/;

    if ($zone =~ /-07:00/ ) {
	    $zone = "PDT" ;
    } elsif ($zone =~ /-08:00/ ) {
	    $zone = "PST" ;
    } elsif ($zone =~ /-05:00/ ) {
	    $zone = "EST" ;
    } elsif ($zone =~ /-04:00/ ) {
	    $zone = "EDT" ;
    } elsif ($zone =~ /\+?05:30/ ) {
	    $zone = "IST" ;
    }

    return ("$dayofweek $mon $day $hr:$min:$sec $zone $yr");
}

#something that works
# assumes year month are same
sub oldget_diff_in_sec($$) {
    my($start, $end) = @_;
    if ( !defined($start) || $start=~/^\s*$/ || !defined($end) || $end=~/^\s*$/ ) {
        return "Invalid";
    }
    #print "Start : $start , End: $end \n";
    my ($sday, $shr , $smin ,$ssec ,$szone) = $start =~
            /\w+\s+\w+\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\S+).*$/;
    my ($eday, $ehr , $emin ,$esec ,$ezone) = $end =~
            /\w+\s+\w+\s+(\d+)\s+(\d+):(\d+):(\d+)\s+(\S+).*$/;

    if ($ezone eq $szone ) {
         return ( ($eday-$sday)*86400 + (((($ehr *60) + $emin )*60 +$esec) -
              ((($shr *60) + $smin )*60 +$ssec)));
    }
    else {
         return "Invalid";
    }
}

sub get_diff_in_sec($$) {

    my($start, $end) = @_;

    if ( !defined($start) || $start=~/^\s*$/ || !defined($end) || $end=~/^\s*$/ ) {
        return "Invalid";
    }

    my $time1 = str2time($start);
    my $time2 = str2time($end);
    my $diff = $time2 - $time1;

    return ($diff);
}



sub is_leap_year($)
{
  my ($year) = @_;

  return ($year%4 == 0 && ($year %100 !=0 || $year %400 ==0 ));
}

my $regexpval1 = qr/(\d+)%.?$/; # takes care of 23%  and 23%)
my $regexpval2 = qr/(\d+)KB$/;
my $regexpval3 = qr/(\d+)MB$/;
my $regexpval4 = qr/(\d+\.?\d*)GB$/;
my $regexpval5 = qr/(\d+)B$/;
my $regexpval6 = qr/(\d+)s$/; # seconds??
my $regexpval7 = qr/\D+.*$/;

# This function converts a digit-like string to a number (10% -> 10, 100B -> 100, 100KB -> 102400, etc)

sub getvalue($)
{
        my $str = shift;
        my $val;

        if ( $str =~ $regexpval1 ) { # %
                $val = $1;
        }
        elsif ( $str =~ $regexpval2 ) {
                $val = $1 * 1024;
        }
        elsif ( $str =~ $regexpval3 ) {
                $val = $1 * 1024 * 1024;
        }
        elsif ( $str =~ $regexpval4 ) {
                $val = $1 * 1024 * 1024 * 1024;
        }
        elsif ( $str =~ $regexpval5 ) {
                $val = $1;
        }
        elsif ( $str =~ $regexpval6 ) {
                $val = $1;
        }
        elsif ( $str =~ $regexpval7 ) {
                $val = $str;
        }
        else {
                $val = $str;
        }
        return($val);

}
