# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Acro/FilePosition.pm#1 $
## @summary     Class for determining file postions on Acro volumes
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here

package Acro::FilePosition;

use strict;
use Carp qw(croak);
use Error;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use Module::Common qw(validate_mixed);
use Tharn qw($Log host);
use Data::Dumper;

use vars qw(@ISA);
@ISA = qw(Error);
###############################################################################
# Method:    new
# Objective: create new wex helper object 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class,%args) = @_;
    my (
      $code,  
      $client,
      $errmsg,
      $self,
      $tharnhost,
      $uid,   
    );
    
    $self = bless {}, $class;
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    ### Validate input arguments
    eval {
        ($class,%args) = validate_mixed (
            @_,
            [ { type => SCALAR} ],
            {
                node     => {type => SCALAR, optional=>1 },
                on_err  =>  {type => SCALAR, optional=>1, default=>"die",
                             callbacks=>{
                                'Legal Values'=>sub{shift()=~/die|rtn/i
                            }}},  
                server  =>  {type => OBJECT,  required => 1 ,isa=>'Server', ,
                             callbacks=>{
                                'Type Of Filer is GX'=>sub{
                                   Tharn::host(($args{server}->servers())[0])
                                   ->hosttype()=~/ng/i;              
                            }}},
            }
        );
        
        $self->{on_err}=$args{on_err};
        $self->{server}=$args{server};
        
        if (defined $args{node}){
            $self->{server}->set_execution_server(-Server => $args{node});    
        }
        else {
            $args{node}=$self->{server}->get_execution_server();
        }
        
        $self->{node_name}=$args{node};
        
        $self->{file_of}={};
        $self->{idx_of}={};
        
    };
    if ($@){
       $Log->comment("ERROR OCCURRED!!!!!!". $@); 
       ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
       return $self;
    }
    
    return $self;
}

###############################################################################
# Method:    discover
# Objective: discover files
###############################################################################
sub discover {
    my ($self,%args)=@_;
    my (
        $cmd,
        $dsid,
        $dv_of,
        $loc_of,
        $map_of,
        $members,
        $name,
        $node,
        $raw,
        $type,
        $vols,
        @lines,
    );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node object 
    $node=$self->node_name();    
    
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                dir      => {type => SCALAR, default=>"." },      
                node     => {type => SCALAR, default=>$node },
                volume   => {type => SCALAR }, 
                vserver  => {type => SCALAR, default=>"vs0" },
            }
        );
        
        $self->{server}->set_execution_server(-Server=>$node);
        
        ### Retrieve one member volume by name to get msid
        $vols = $self->{server}->volume_member_show(
            vserver=>$args{vserver},
            volume=>$args{volume}
        );
        
        ### Use msid to retrieve all member volumes 
        $vols = $self->{server}->volume_member_show(
            msid => $vols->[0]{msid}
        );
        
    };
    if ($@){
       ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
        return 0;
    }
    
    ### Determine node location of each DV constituent
    foreach my $vol_of (@$vols){
        my $loc= $self->_get_dsid_location(volume=>$vol_of->{dsid});   
        $map_of->{$vol_of->{dsid}}=$loc;
    }
    
    ### Query each member using wex to locate each fav
    foreach my $vol_of (@$vols){
        
        $cmd="wex dir -dump dsid,name,type ".$args{volume}."./".$args{dir};
        
        ### Execute command to retrieve FAV dsid
        $raw=$self->{server}->shell(cmd=>$cmd);
        $raw=~s/\r\r\n/\n/g;
        @lines=split(/\n/,$raw);
        
        foreach my $line (@lines) {
            
            if ($line=~/dsid/) { 
               ($dsid)=$line=~/\=(\d+)/;
            }
            elsif($line=~/name/){
                ($name)=$line=~/\=(\S+)/;
            }
            elsif ($line=~/type/){
                ($type)=$line=~/\=(\S+)/;
                ### Ignore directories
                next if ($type=~/^[-,2]$/);
                push @{$dv_of->{$dsid}},$name;
                push @{$loc_of->{$map_of->{$dsid}}},$name;
            }
            else {next;}
        }
        
    }
    
    ### maintain file list in object hash
    $self->{dv_of}  = $dv_of;
    $self->{loc_of} = $loc_of;
    $self->{idx_of} = {};
    
    return 1;
}


###############################################################################
# Method:    get_next
# Objective: Retrieve file from discover cache
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_next {
    my ($self,%args)=@_;
    my (
        $dsid,
        $errmsg,
	$idx,
	$file,
        $mnt_pt,
        $node,
        $file_dv_map,
        $file_node_map,
        $idx_of,
        @files,
	);
   
    $self->_clear_err();
    
    ### NOTE: Retrieval of files by dsid has not been validated
    ### using this method yet
     
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                node     => {type => SCALAR, optional => 1 },
                dsid     => {type => SCALAR, optional => 1,
                             callbacks=>{
                                'Is An Integer>0'=>sub{shift()>0}}},
                cnt      => {type => SCALAR, default => 1,
                             callbacks=>{
                                'Is An Integer>0'=>sub{shift()>0}}},
                delete   => {type => SCALAR, default => 0,
                             callbacks=>{
                                'Legal Values'=>
                                   sub{shift()=~/0|1/i }}},
                mnt_pt   => {type => SCALAR, optional => 1},
                rewind_eof =>
                            {type => SCALAR, default => 0,
                             callbacks=>{
                                'Legal Values'=>
                                   sub{shift()=~/0|1/i }}},
            }
        ); 
    };
    if ($@){
       ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
        return (wantarray()) ? () : [];
    }
   
    ### Either a node or dsid needs to be supplied 
    unless (defined $args{node} xor defined $args{dsid}){
        $errmsg="A 'node' or 'dsid' parameter is required";
        ($self->on_err()=~/die/)
        ? die $errmsg
        : $self->_set_err($errmsg);
       
        return (wantarray()) ? () : [];
    }
   
    $file_dv_map   = $self->{dv_of};
    $file_node_map  = $self->{loc_of};
    $idx_of  = $self->{idx_of};
    $dsid    = $args{dsid};
    $node    = $args{node};
    $mnt_pt  = $args{mnt_pt};
    
    $mnt_pt.=(defined $mnt_pt and $mnt_pt !~/\/$/) ? "/" : "";       
    
    ### Retrieve file by either node
    ### location or DV
    foreach (1..$args{cnt}){
        
        if (defined $args{node}) {
            $file=$self->_file_map(
                $file_node_map,$node,$idx_of,$args{delete},$args{rewind_eof});          
        }
        else {
            $file=$self->_file_map(
                $file_dv_map,$dsid,$idx_of,$args{delete},$args{rewind_eof});
            
        }
        
        push @files,$mnt_pt.$file;
    }
    
    if ($args{cnt}==1) {
        return $files[0];
    }
    
    return (wantarray()) ? @files : [@files];

}

###############################################################################
# Method:    node_name
# Objective: Return node_name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub node_name {
    my ($self,%args)=@_;
    
    return $self->{node_name};
}


###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$type)=@_; 
    
    if (defined $type) {
        $self->{on_err}=$type;
    }
    
    return $self->{on_err}; 
}

###############################################################################
# Method:    rewind
# Objective: reset index of file list
# Details:   See POD documentation at the end of this file.
###############################################################################
sub rewind {
    my ($self,%args)=@_;
    my (
        $dsid,
        $errmsg,
	$idx,
        $node,
        $idx_of,
        @files,
	);
   
    $self->_clear_err();
     
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                node     => {type => SCALAR, optional => 1 },
                dsid     => {type => SCALAR, optional => 1,
                             callbacks=>{
                                'Is An Integer>0'=>sub{shift()>0}}},
            }
        ); 
    };
    if ($@){
       ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
       return 0;
    }
   
    unless (defined $args{node} xor defined $args{dsid}){
        $errmsg="A 'node' or 'dsid' parameter is required";
        ($self->on_err()=~/die/)
        ? die $errmsg
        : $self->_set_err($errmsg);
       
        return 0; 
    }
   
    $idx_of  = $self->{idx_of};
    $dsid    = $args{dsid};
    $node    = $args{node};
        
    if (defined $args{node}) {
        $idx_of->{$node} = 0;
    }
    else {
        $idx_of->{$dsid} = 0;
    }
           
    1;         
}

###############################################################################
# Method: _file_map
# Objective: Retrieve file by location or dv filter
###############################################################################
sub _file_map {
    my ($self,$map,$id,$idx_of,$delete_flag,$rewind_flag)=@_;
    my (
        $file,
        $idx,
        );
    
    ### Retrieve next index value
    $idx=(defined $idx_of->{$id})
       ? $idx_of->{$id}
       : 0;
       
    ## Exit loop if no more files exist
    last if ($idx==-1);
    
    ### Get file at index location
    $file=${$map->{$id}}[$idx];
                
    ### Bump the index
    if ($delete_flag) {
        splice @{$map->{$id}},$idx,1; 
    }
    else {
      
        $idx_of->{$id}=($idx>=scalar(@{$map->{$id}})-1)
            ? ($rewind_flag) ? 0 :-1
            : $idx_of->{$id}+1;
        
    }
    
    return $file;
}

###############################################################################
# Method: _get_dsid_location
# Objective: Retrieve node name associated with dsid
###############################################################################
sub _get_dsid_location {
    my ($self,%args)=@_;
    my (
        $cmd,
        $loc,
        $raw,
        @lines,
    );
    
    ### Initialize variables
    $cmd="wex volume ".$args{volume}." -list dsid,location";
    
    ### Execute command to retrieve FAV dsid
    $raw=$self->{server}->shell(cmd=>$cmd);

    $raw=~s/\r\r\n/\n/g;
    @lines=split(/\n/,$raw);
    $Log->comment(@lines);
    
    ### Retrieve volume record name containing dsis
    foreach my $line (@lines) {
        if ($line=~/\s+$args{dsid}\s+/) {
            ($loc)=$line=~/\d+\s+(\S+)/;
        }
    }
    
    unless (defined $loc) {
        croak "convert dsid to location failed";
    }
    
    return $loc;
    
}

1;
############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

Acro::FilePosition.pm

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

Acro::FilePosition.pm provides an API interface to discover and retrieve the
list of files that exist by where thier metadata attribute volume (FAV) is
stored.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error.pm>

=head1 Public Methods

=head2 B<new( B<server>, I<on_err>, I<node> )>

Creates a new Acro::FilePosition object

=over 4

=item Inputs

=over 4

=item * B<server>    

Server object created to manage cluster configuration

=item * I<on_err> (default: "die")    

Error handler to use for returning errors back to the calling script.

=begin html

<ul>
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=item * I<node> (default: first node in array in server->Servers())

Name of node to use for determining what file names exist on a particular DV
or node involved in an acro stripe.

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

    # Example: Create  Acro::FilePosition object
    $Test->nlog("- Creating Acro::FilePostion object);
    my $filePosObj = new Acro::FilePosition(server=>$server,on_err=>'rtn');
    if ($filePosObj->error()){
        $Test->nlog($filePosObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<discover( B<volume>, I<dir>, I<vserver>, I<node> )>

Retrieves names of all files in directory on volume including what node 
and data volume constituent they came from.  Each call to this method 
refreshes the existing object's file name cache.

=over 4

=item Inputs

=over 4

=item * B<volume>    

Name of acro volume to use

=item * I<dir> (default: ".")

Name of directory to locate.  Only files within this directory are returned.  

=item * I<vserver> (default: "vs0")

Name of vserver acro volume is located on

=item * I<node> (default: node specified during new() call)

Name of node to use for performing discover operation.

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

    # Example: Discover files
    $Test->nlog("- Discover existing file names on object");
    $filePosObj->discover(volume=>"p_acro_1", vserver=>"vs0");
    if ($filePosObj->error()){
        $Test->nlog($filePosObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<get_file( B<node|dsid>, I<cnt> , I<delete>, I<mnt_pt>, I<round_robin>)>

Retrieves names of all files on volume including what node and data volume
their FAV is located on.  Method increments and maintains index into discovery
cache, such that each successive get_file() call will return a different file
(if one exists).

=over 4

=item Inputs

=over 4

=item * B<node|dsid>    

Name of the node or volume identifier (dsid) thats maintaining the file's metadata attributes 

=item * I<cnt> (default: 1)    

Number of files to return

=item * I<delete> (default: 0)

Removes the file name from the object's cache after retrieving it.  This does NOT 
remove the file from the filesystem.  This option is to enforce that a file is not 
reused in subsequent testing if the object's cache is re-iterated through.

=item * I<mnt_pt> (default: "")

Prefixes each file name returned with the mount point name.

=item * I<round_robin> (default: 0)

Allows the file cache to be reused once the end of cache is reached.  Otherwise, an 
error is returned if the method call cannot return the requested number of files.

=back

=item Outputs

If cnt is "1" or not set, it returns a scalar value.  If cnt > 1 it returns an array 
or arrayref depending on the specified user's return type. 

=item Examples
    
    # Example: Discover files on volume
    $Test->nlog("- Retrieve 3 files to use");
    @files=$filePosObj->get_file(volume=>"p_acro_1", vserver=>"vs0",mnt_pt=>"/t/mnt", cnt=>3);
    foreach my $file (@files){
	    $Test->nlog("file: ".$file);
	}

=back

=head2 B<node_name()>

This method allows you to retrieve the node name being used for object dsicvoery operation 

=over 4

=item Outputs

Returns a scalar value containing the node name

=item Examples 

 # Get nodename
 $nodename= $filePosObj->node_name();
 $Test->nlog("Node" ". $nodename);

=back

=head2 B<on_err( I<$type> )>

Defines what error handling mechanism to use on Acro::FilePosition instance

=over 4

=item Inputs

=over 4

=item * I<$type>

Indicates what errhandler type to use for all Acro::FilePosition methods
calls.  If no errorhandle is provided, it returns the current handler type 

=begin html

<ul>
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=back

=item Outputs

Returns error handle type  on success, throws error on failure

=item Examples

 # Example 1: Set the defualt error handle from 'die' to 'rtn'   
 $hdlr=$filePosObj->on_err('rtn');
 $test->nlog("Error handler changed to '".$hdlr."'");

=back

=head2 B<rewind( B<node|dsid> )>

Resets the object index of files in the discover cache for node or volume constituent
back to 0. 

=over 4

=item Inputs

=over 4

=item * B<node|dsid>    

Name of the node or volume identifier (dsid) cache to reset to 0 

=item Outputs

Returns 1 on success, 0 on failure

=item Examples

 # Example 1: Reset file cache 
 $filePosObj->rewind(node=>'gi-3040-2');

=back

=cut

