# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Admin/License/Default.pm#1 $

## @summary Storage for Default Licenses and Some More
## @status Shared
## @author dl-central-libs
## @pod here

=head1 NAME

Admin::License::Default - Storage for Default Licenses and Some More

=head1 SYNOPSIS

  use Admin::License::Default;

=head1 DESCRIPTION

First of all, DON'T USE THIS MODULE. This was separated from Admin::License
for internal housekeeping, and therefore there should be absolutely nothing
of interest for the end user.

The only possible reason that you would be looking at this module is because
you want to add, edit, or steal one of these licenses.

The second rule is, DON'T EDIT THIS MODULE, OR COPY LICENSES FROM THIS MODULE.
If you need a license, see the License FAQ at

   http://web.netapp.com/engineering/hhg/Software-4.0/Procedures-3.0/License_FAQ.html
   http://web.netapp.com/engineering/design-depot/appliance-mgmt/license-matrix.html

Only edit this file when you absolutely need to, such as when a new license
has been added, and you need to test it.

=head1 MANAGING THE LICENSES

If you haven't already done so, read the L</DESCRIPTION>. If you shouldn't
be editing this file, you don't have to read the rest of this documentation.

If you are responsible for testing a new feature in which no license is
provided for in this module, you may add it by editing the %DEFAULT_LICENSES
hash. This hash contains a relationship from a hosttype to a hash of
licenses:

  %DEFAULT_LICENSES = (
    filer => {
      nfs => 'CFZRXFF'
      ...
    }
  }

If the service/license is applicable to more than one hosttype, please make
sure to update all relevant types. A lot of code depend on licensing, and
therefore you should take all the attention to make sure that it works as
intended for everybody.

Furthermore, for licenses that may be mutually exclusive with another, you
should setup the %MUTUALLY_EXCLUSIVE hash. This hash should be setup by
using the _add_relationship() private function.

Adding something like

  _add_relationship('snaplock', 'snaplock_enterprise')

will allow Admin::License to intelligently disable mutually exclusive
licenses before enabling the other.

=cut

package Admin::License::Default;
use strict;

# Mostly from http://wikid.netapp.com/w/HHG/Software-4.0/Procedures-3.0/License_FAQ
# 
# XXX - DO NOT COPY LICENSES FROM THIS FILE
#       See the License FAQ instead.
# XXX - Only edit this file when you absolutely need to, such as
#       when a new license has been added, and you need to test it
our %DEFAULT_LICENSES = (
	filer => {
		# please keep this in alphabetical order. it's easier
		# to spot changes that way ;)
       	        a_sis               => 'OSFYRVH',
		cifs                => 'GTLYRVH',
		cluster             => 'KDQLGBN',
		cf                  => 'KDQLGBN',
		cluster_remote      => "GTFJHSI",
                cf_remote           => "GTFJHSI",  # for 7.3 and above
		compression         => 'QTEUWOJ', # new name for wafl_compression >= 8.0
		dafs                => 'OBOLDQC',
		disk_sanitization   => 'CPUPBIL',
		snapvalidator       => 'GDFFMLK',
		fcp                 => 'ICRNCZG',
		flex_cache          => 'KRFUWOJ', # old-style free license
		flexcache_nfs       => 'OSRAREM', # new-style paid license
		flex_clone          => 'QNFYRVH',
		flex_scale          => 'MZGLGBN', # for 7.3 and above		
		pamii               => 'QLAHIJE', # for 7.3.2 or 8.0.1 and above 
		http                => 'AODUWOJ',
		iscsi               => 'IKVAREM',
		nearstore_option    => 'SAXCNCG',
                nearstore_asis1     => 'EJCFMLK',
                nearstore_asis2     => 'OSFYRVH',
		nfs                 => 'UYTCNCG',
		rapid_restore       => 'QBBSXFF', # for 7.1 and above
		smdomino            => 'CFZRXFF',
		smsql               => 'SHEWSMD',
		snaplock            => ['QWXPYWA','IASLGBN'], # for >= 7.001 and < 7.001 resp.
		snaplock_enterprise => 'AOXRXFF',
		snapmanagerexchange => 'GFMYRVH',
		snapmirror          => "UUNLDQC",
		snapmirror_sync     => 'IEOAOTB',
		snaprestore         => "EOYPBIL",
		sv_linux_pri        => 'GROPYWA',
		sv_ontap_pri        => 'YAYCNCG',
		sv_ontap_sec        => 'KVPYRVH',
		sv_nbu_sec          => 'MJWGIJE',
		sv_unix_pri         => 'IUILGBN',
		sv_windows_pri      => 'SLGLDQC',
		syncmirror_local    => "UELJHSI",
		vfiler              => "SPQLGBN",
		multistore          => "SPQLGBN",
		vld                 => "QYOAREM",
		volcopy             => "AAWPYWA",
		wafl_compression    => "KVTAOTB",
	},
	nearstore => {
                a_sis               => 'WQQQZGI',
		cifs                => 'HJIMEAK',
		cluster             => 'ZGDIJTL',
		cluster_remote      => 'RSTBPDJ',
		compression         => "XTBVGUE", # new name for wafl_compression for >= 7.3
		dafs                => 'JSJIJTL',
		disk_sanitization   => 'LNFIJTL',
		snapvalidator       => 'PBQXTWK',
		fcp                 => 'RACGKKH',
		flex_cache          => 'TPQMEAK', # old-style free license
		flexcache_nfs       => 'WQCTYPM', # new-style paid license
		flex_clone          => 'ALSBPDJ',
		http                => 'PRQQZGI',
		iscsi               => 'RIGTYPM',
                nearstore_asis1     => "MHNXTWK",
                nearstore_asis2     => "WQQQZGI",
		nfs                 => 'JCHZPUE',
		smdomino            => 'LDKKFRF',
		smsql               => 'BGPOAYD',
		snaplock            => ['THSZPUE','RYCEOMN'], # for >= 7.001 and < 7.001 resp.
		snaplock_enterprise => 'JMIKFRF',
		snapmanagerexchange => 'RKFVUNG',
		snapmirror          => 'JYAIGIB',
		snapmirror_sync     => 'RCZSVEC',
		snaprestore         => 'FEVDOMN',
		sv_linux_pri        => 'DVBRYNN',
		sv_ontap_pri        => 'JOXSVEC',
		sv_ontap_sec        => 'JEEIJTL',
		sv_nbu_sec          => 'UHHZPUE',
		sv_unix_pri         => 'ZOHVQJB',
		sv_windows_pri      => 'FERMAWE',
		syncmirror_local    => 'LBZDLBD',
		vfiler              => 'TJHIGIB',
		multistore          => 'TJHIGIB',
		vld                 => 'ZWZSYPM',
		wafl_compression    => "XTBVGUE",
	},
	gateway => {
		cifs                => 'DXCQNUJ',
		cluster             => 'VUXLSNL',
		cluster_remote      => 'NGOFYXI',
		dafs                => 'FGEMSNL',
		disk_sanitization   => 'HBAMSNL',
		flex_cache          => 'PDLQNUJ', # old-style free license
		flexcache_nfs       => 'SEXWHKM', # new-style paid license
		flex_clone          => 'WYMFYXI',
		snapvalidator       => 'LPKBDRK',
		gateway_storage     => 'NSOHUVC',
		http                => 'LFLUIBI',
		nfs                 => 'FQBDZOE',
		smdomino            => 'HREOOLF',
		smsql               => 'XTJSJSD',
		snaplock            => ['PVMDZOE','NMXHXGN'], # for >= 7.001 and < 7.001 resp.
		snaplock_enterprise => 'FADOOLF',
		snapmanagerexchange => 'NYZYDIG',
		snapmirror          => 'FMVLPCB',
		snapmirror_sync     => 'NQTWEZB',
		snaprestore         => 'BSPHXGN',
		sv_linux_pri        => 'ZIWUHIN',
		sv_ontap_pri        => 'FCSWEZB',
		sv_ontap_sec        => 'FSYLSNL',
		sv_unix_pri         => 'VCCZZDB',
		sv_windows_pri      => 'BSLQJQE',
		syncmirror_local    => 'HPTHUVC',
		vfiler              => 'PXBMPCB',
		multistore          => 'PXBMPCB',
		vld                 => 'VKUWHKM',
	},
	promoted_simulator => { # this is for promoted filer simulator only
		cifs                => 'DZDACHD',
		cluster             => 'JQIGWWF',
		cluster_remote      => 'BCZZBHD',
		dafs                => 'TBPGWWF',
		disk_sanitization   => 'PZKEAZL',
		snapvalidator       => 'TNVTKCL',
		fcp                 => 'BKHEXNB',
		flex_cache          => 'HFAYFJJ', # old-style free license
		flexcache_nfs       => 'ADIPPVM', # new-style paid license
		flex_clone          => 'ANLEAZL',
		http                => 'NAZOMKC',
		iscsi               => 'BSLRLTG',
		nfs                 => 'BQOEAZL',
		rapid_restore       => 'WSZKRDE',
		smdomino            => 'RKBAFSN',
		smsql               => 'HNGEAZL',
		snaplock            => 'BIICBQH',
		snaplock            => ['ZOJPPVM','BIICBQH'], # for >= 7.001 and < 7.001 resp.
		snaplock_enterprise => 'TVJRLTG',
		snapmanagerexchange => 'BCJEAZL',
		snapmirror          => 'DFVXFJJ',
		snapmirror_sync     => 'XJQIVFK',
		snaprestore         => 'DNDCBQH',
		sv_linux_pri        => 'ZYICXLC',
		sv_ontap_pri        => 'PVOIVFK',
		sv_ontap_sec        => 'PDXMQMI',
		sv_nbu_sec          => 'KCSXFJJ',
		sv_unix_pri         => 'RQAYBFE',
		sv_windows_pri      => 'ZOPRKAM',
		syncmirror_local    => 'RIQTKCL',
		vfiler              => 'NQBYFJJ',
		multistore          => 'NQBYFJJ',
		vld                 => 'JGFRLTG',
	},
	netcache => {
		com_e      => 'CLMEYRI',
		dnfs       => 'SZNNCZG',
		grm_agent  => 'GJOLDQC',
		grm_server => 'LZTGVWH',
		icap       => 'CNHPNOJ',
		netcache   => 'GXVPYWA',
		nntp       => 'WJSRXFF',
		mms        => 'CRZITYG',
		mms_pro    => 'KDUEMLK',
		mpeg4      => 'IIEADLK',
		quicktime  => 'AWPTIVH',
		real       => 'WLREYRI',
	},
	sa_appliance => {
		flexcache_nfs => 'QLULMHM',
	}
);

# XXX - some licenses are mutually exclusive, and therefore license()
# must handle it "intelligently". This hash will store that relationship

# (Note, because I didn't want to have multiple entries to reference
# "this" => "that" and "that" => "this", we're just going to use a 
# function to populate this hash)

our %MUTUALLY_EXCLUSIVE;

sub _add_relationship
{
	my($this, $that) = @_;

	if (!exists $MUTUALLY_EXCLUSIVE{$this}) {
		$MUTUALLY_EXCLUSIVE{ $this } = [];
	}
	if (!exists $MUTUALLY_EXCLUSIVE{$that}) {
		$MUTUALLY_EXCLUSIVE{ $that } = [];
	}

	push @{ $MUTUALLY_EXCLUSIVE{ $this } }, $that;
	push @{ $MUTUALLY_EXCLUSIVE{ $that } }, $this;
}

BEGIN
{
	# populate the MUTUALLY_EXCLUSIVE hash
	_add_relationship('snaplock', 'snaplock_enterprise');
	_add_relationship('sv_ontap_pri', 'sv_ontap_sec');
}


1;


=head1 SEE ALSO

Admin::License, Admin::License::Object

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = none

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    dl-central-libs
	$Id: //depot/prod/test/Rsierranevada/lib/Admin/License/Default.pm#1 $
    Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut
