# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Aggregate/ImplTemplate/RaidInfo_new.pm#1 $
#
# Hey, emacs, this is basically -*-perl-*-
#$Id: //depot/prod/test/Rsierranevada/lib/Aggregate/ImplTemplate/RaidInfo_new.pm#1 $

## @summary     Access routines all raid objects
##
## @description
##      This library provides access to all raid objects (aggregates,
##      volumes, plexes, raid groups, and disks) and internal values
##      associated with these objects. These values are obtained
##      through Ontap's priviledged raid_config info command, 
##      therefore none of the library functionswill depend on the 
##      UI commands.
##      
## @synopsis
## use external functions defined in @raid_subs to get at the objects
## user should not call constructors directly.
##
## @usage
##      Example/testRaidInfoNew.thpl exists to vaguely test this package
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
##
## @status   unmaintained, undocumented junk

package Aggregate::ImplTemplate::RaidInfo_new;
BEGIN {
	$Validation = 1;
	$VERSION    = '0.01';
	
	
	my @raid_subs        = qw(get_all_aggr_raid_config_info 
				  get_all_vol_raid_config_info
				  get_all_plex_raid_config_info
				  get_all_disk_raid_config_info
				  get_all_group_raid_config_info
				  get_aggr_raid_info
				  get_vol_raid_info 
				  get_container_raid_info
				  get_disk_raid_info
				  get_group_raid_info
				  get_plex_raid_info
				  get_spares
				  get_spares_pool);
	
	
	require Exporter;
	our @ISA = qw(Exporter);
	our @EXPORT_OK = (@raid_subs);
	our %EXPORT_TAGS = (raid_subs => [@raid_subs]);
};

use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
use SayWrap qw(sayEOF);
use Volume qw(vol_container);

sub _get_all_type_raid_config_info {
	my $conn = shift;
	my $type = shift;
	my $cmd= "priv set -q diag; raid_config info ";
	if ($type eq 'aggr') {
		$cmd.="listaggr";
	} elsif ($type eq 'vol') {
		$cmd .="listvol";
	} elsif ($type eq 'plex') {
		$cmd .="listplex";
	} elsif ($type eq 'group') {
		$cmd .="listgroup";
	} elsif ($type eq 'disk') {
		$cmd .="listdisk";
	} else {
		die "unknown type $type";
	}
	
	my $all_type_raid_info;
	my( $output ) = sayEOF($conn, $cmd);
	foreach my $one_output (split(/\n/, $output)) {
		my $one_entity_raid_info;
		if ($type eq "aggr") {
			$one_entity_raid_info =	 
				new_with_aggr_output Aggregate::ImplTemplate::AggrRaidInfo($one_output, $conn)
					or die $@;
		} elsif ($type eq 'vol') {
			$one_entity_raid_info = 
				new_with_vol_output Aggregate::ImplTemplate::VolRaidInfo($one_output, $conn)
					or die $@;
		} elsif ($type eq 'plex') {
			$one_entity_raid_info = 
				new_with_plex_output Aggregate::ImplTemplate::PlexInfo($one_output, $conn)
					or die $@;
		} elsif ($type eq 'disk') {
			$one_entity_raid_info =
				new_with_disk_output Aggregate::ImplTemplate::DiskInfo($one_output, $conn)

		} else {
			$one_entity_raid_info =
				new_with_group_output Aggregate::ImplTemplate::GroupInfo($one_output, $conn);
		}

		$all_type_raid_info->{$one_entity_raid_info->name()} = $one_entity_raid_info;
	}
	return $all_type_raid_info;
	
}

sub get_all_aggr_raid_config_info {
	my $conn = shift;
	return _get_all_type_raid_config_info($conn, 'aggr');
}


sub get_all_vol_raid_config_info {
	my $conn = shift;
	return _get_all_type_raid_config_info($conn, 'vol');
}

sub get_all_plex_raid_config_info {
	my $conn = shift;
	return _get_all_type_raid_config_info($conn, 'plex');
}

sub get_all_disk_raid_config_info {
	my $conn =shift;
	return _get_all_type_raid_config_info($conn, 'disk');
}

sub get_all_group_raid_config_info {
	my $conn = shift;
	return _get_all_type_raid_config_info($conn, 'group');
}

sub get_aggr_raid_info {
	my ($conn, $target) = @_;
	return get_target_info  Aggregate::ImplTemplate::AggrRaidInfo($conn, $target);

}

sub get_vol_raid_info {
	my ($conn, $target) = @_;
	return get_target_info Aggregate::ImplTemplate::VolRaidInfo($conn, $target);
} 

sub get_disk_raid_info {
	my ($conn, $target) = @_;
	return get_target_info Aggregate::ImplTemplate::DiskInfo($conn, $target)
}

sub get_spares {
	my ($conn) = @_;
	return get_spares Aggregate::ImplTemplate::DiskInfo($conn)
}

sub get_spares_pool {
	my ($conn, $pool) = @_;
	my $pool_spares;
	my $spares = get_spares($conn);
	foreach my $entry (keys(%{$spares})) {
		if ($spares->{$entry}->disk_container() =~ /$pool/i ) {
			$pool_spares->{$spares->{$entry}->name()} = $spares->{$entry};
		}
	}
	return $pool_spares;
}

sub get_group_raid_info {
	my ($conn, $target) = @_;
	return get_target_info Aggregate::ImplTemplate::GroupInfo($conn, $target);
}

# XXX no interface provided for exactness match

sub get_plex_raid_info {
	my ($conn, $target) = @_;
	return get_target_info Aggregate::ImplTemplate::PlexInfo($conn, $target);
}

#container for flexvol is containing aggr
#container for trad vol is itself
#container for aggr is itself
sub get_container_raid_info {
	my ($conn, $target) = @_;
	#if version is prior to anchorsteam we are dealing with trad vols
	if (get_wafl_version($conn->hostp()) <= WV_SCRIMSHAW_MAX) {
		#container is the volume itself
		return get_vol_raid_info($conn, $target);
	}
	
	#are we an aggregate or a flex vol or a trad vol?
	my $container = vol_container($conn, $target);
	if (!$container) {
		if ($@ =~ /must be a flexible volume/i) {
			# we are a trad vol
			return get_vol_raid_info($conn, $target);
		} elsif ($@ =~ /no flexible volume/i) {
			#we are an aggregate
			return get_aggr_raid_info($conn, $target);
		} else {
			die "Could not determine container of target $target\n";
		}
	} else {
		return get_aggr_raid_info($conn, $container);
	}
	die("failed to find container for target $target");
}

sub get_plex_info {
	my ($conn, $target) = @_;
	return get_target_info Aggregate::ImplTemplate::PlexInfo($conn, $target);
}


1;

############################################################################
package Aggregate::ImplTemplate::AggrRaidInfo;
use     Aggregate::ImplTemplate::RaidInfo_new qw(:raid_subs);  
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
use strict;
our $AUTOLOAD;

sub AUTOLOAD {
    my $sub = $AUTOLOAD;
    (my $attr = $sub) =~ s/.*:://;
    no strict 'refs';
    eval 'sub '.$sub.' { my $self = shift; return $self->{"'.$attr.'"} }';
    goto &$sub;
}
    
sub new_with_aggr_output {
      my ($class, $output, $conn) = @_;
      my ($self) = {};
      $self->{version} = get_wafl_version($conn->hostp());
      
      foreach my $entry (split(/,/, $output)) {
	      my ($key, $value) = split('=', $entry);
	      $self->{$key} = $value;
      }
      bless($self, $class);

      return $self;
}

sub get_target_info {
	my ($class, $conn, $target) = @_;
	#my $self = {};
	#$self->{version} = get_wafl_version($conn->hostp());
	my $all = get_all_aggr_raid_config_info($conn);

	foreach my $aggr_name  ( keys( %{$all} ) ) {
		if ($all->{$aggr_name}->name() eq $target) {
			return $all->{$aggr_name};
			#$self = $all->{$aggr_name};
			#bless $self, $class;
			#return $self;
		}
	}
	die "could not find target aggregate $target in raidinfo output";
}

#wrapper routine to get aggr name
sub name {
	my $self = shift;
	return $self->tree();
}


1;

############################################################################
package Aggregate::ImplTemplate::VolRaidInfo;
use     Aggregate::ImplTemplate::RaidInfo_new qw(:raid_subs);  
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
use strict;
our $AUTOLOAD;

sub AUTOLOAD {
    my $sub = $AUTOLOAD;
    (my $attr = $sub) =~ s/.*:://;
    no strict 'refs';
    *$sub = sub { $_[0]->{$attr} };
    goto &$sub;
}

sub new_with_vol_output {
      my ($class, $output, $conn) = @_;
      my ($self) = {};
      $self->{version} = get_wafl_version($conn->hostp());

      foreach my $entry (split(/,/, $output)) {
	      my ($key, $value) = split('=', $entry);
	      $self->{$key} = $value;
      }
      bless($self, $class);

      return $self;
}

sub get_target_info {
	my ($class, $conn, $target) = @_;
	#my $self = {};
	my $all = get_all_vol_raid_config_info($conn);

	foreach my $vol_name  ( keys( %{$all} ) ) {
		if ($all->{$vol_name}->name() eq $target) {
			return $all->{$vol_name};
		}
	}
	die "could not find target volume $target in raidinfo output";
}

sub vol_container_raid_info {
	my ($self, $conn) = @_;
	if ($self->version() <= WV_SCRIMSHAW_MAX) {
		#trad vol is its own container
		 return $self;
	 }
	
	# we need to get aggregate
	my $parent = $self->raidname();
	return get_aggr_raid_info($conn, $parent);
}

1;

#############################################################################
package Aggregate::ImplTemplate::PlexInfo;
use     Aggregate::ImplTemplate::RaidInfo_new qw(:raid_subs);  
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
use strict;
our $AUTOLOAD;

sub AUTOLOAD {
    my $sub = $AUTOLOAD;
    (my $attr = $sub) =~ s/.*:://;
    no strict 'refs';
    *$sub = sub { $_[0]->{$attr} };
    goto &$sub;
}

#this basic constructor will get executed no matter what
sub new_with_plex_output {
      my ($class, $output, $conn) = @_;
      my ($self) = {};
      $self->{version} = get_wafl_version($conn->hostp());

      foreach my $entry (split(/,/, $output)) {
	      my ($key, $value) = split('=', $entry);
	      $self->{$key} = $value;
      }
      bless($self, $class);

      return $self;
}

sub get_target_info {
	my ($class, $conn, $target, $exact) = @_;
	$exact = 0 if (!defined($exact));
	my $plex_hash = undef;
	# assume target is just a name of tradvol/aggr or a full plex name that is correct
	
	my $self = {};

	# returns a hash of blessed objects, so no need to bless
	my $all = get_all_plex_raid_config_info($conn);
	
	foreach my $plex_name  ( keys( %{$all} ) ) {
		#my $plex = $all->{$plex_name};
		if (($exact && $all->{$plex_name}->container_name() eq $target) || 
		    (!$exact && $all->{$plex_name}->container_name() =~ $target)) 
			{
				#$self = $all->{$plex_name};
				#bless $self, $class;
				$plex_hash->{$all->{$plex_name}->name()} = $all->{$plex_name};
			}
	}
	if (!defined($plex_hash)) {
		die "could not find target plex $target in raidinfo output";
	}
	return $plex_hash;
}

sub name {
	my $self = shift;
       return $self->raid_name();
}

sub container_name {
	my $self = shift;
	return $self->tree();
}

sub container_raid_info {
	my $self = shift;
	if ($self->get_version() <= WV_SCRIMSHAW_MAX) {
		die 'fkdjfkdjfkld';
	}
}

1;

##################################################################
package Aggregate::ImplTemplate::DiskInfo;
use     Aggregate::ImplTemplate::RaidInfo_new qw(:raid_subs);  
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
use strict;
our $AUTOLOAD;

sub AUTOLOAD {
    my $sub = $AUTOLOAD;
    (my $attr = $sub) =~ s/.*:://;
    no strict 'refs';
    *$sub = sub { $_[0]->{$attr} };
    goto &$sub;
}

#this basic constructor will get executed no matter what
sub new_with_disk_output {
      my ($class, $output, $conn) = @_;
      my ($self) = {};
      $self->{version} = get_wafl_version($conn->hostp());

      foreach my $entry (split(/,/, $output)) {
	      my ($key, $value) = split('=', $entry);
	      $self->{$key} = $value;
      }
      bless($self, $class);

      return $self;
}





#target in this context is the containing aggr/tradvol
#XXX will change later to named param so target can be any attribute
sub get_target_info {
	my ($class, $conn, $target) = @_;
	my $disk_hash = undef;
	#my $self = {};
	my $all = get_all_disk_raid_config_info($conn);
	
	foreach my $disk_name  ( keys( %{$all} ) ) {
		if ($all->{$disk_name}->disk_container() eq $target) { 
			#$self = $all->{$disk_name};
			#bless $self, $class;
			#$disk_hash->{$self->name()} = $self;
			$disk_hash->{$all->{$disk_name}->name()} = $all->{$disk_name};
		}
	}
	if (!defined($disk_hash)) {
		die "could not find target disk $target in raidinfo output";
	}
	return $disk_hash;
}

sub disk_container {
	my ($self) = shift;
	return $self->tree();
}


sub get_spares {
	my ($class, $conn) = @_;
	my $disk_hash;
	my $all = get_all_disk_raid_config_info($conn);
	foreach my $disk_name  ( keys( %{$all} ) ) {
		if (
		    #$all->{$disk_name}->disk_container() =~ /POOL:Pool0/ 
		    $all->{$disk_name}->host() == 0) {
			#$self = $all->{$disk_name};
			#bless $self, $class;
			#$disk_hash->{$self->name()} = $self;
			$disk_hash->{$all->{$disk_name}->name()} = $all->{$disk_name};
		}
	}
	return $disk_hash;

}

1;

#################################################################################
package Aggregate::ImplTemplate::GroupInfo;
use     Aggregate::ImplTemplate::RaidInfo_new qw(:raid_subs);  
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
use strict;
our $AUTOLOAD;

sub AUTOLOAD {
    my $sub = $AUTOLOAD;
    (my $attr = $sub) =~ s/.*:://;
    no strict 'refs';
    *$sub = sub { $_[0]->{$attr} };
    goto &$sub;
}
    
#this basic constructor will get executed no matter what
sub new_with_group_output {
      my ($class, $output, $conn) = @_;
      my ($self) = {};
      $self->{version} = get_wafl_version($conn->hostp());

      foreach my $entry (split(/,/, $output)) {
	      my ($key, $value) = split('=', $entry);
	      $self->{$key} = $value;
      }
      bless($self, $class);

      return $self;
}

#target in this context is the containing aggr/tradvol name
#XXX will change later to named param so target can be any attribute
sub get_target_info {
	my ($class, $conn, $target) = @_;
	my $group_hash = undef;
	my $all = get_all_group_raid_config_info($conn);
	
	foreach my $group_name  ( keys( %{$all} ) ) {
		if ($all->{$group_name}->group_container() eq $target) { 
			$group_hash->{$all->{$group_name}->name()} = $all->{$group_name};
		}
	}
	if (!defined($group_hash)) {
		die "could not find target group $target in raidinfo output";
	}
	return $group_hash;
}

sub group_container {
	my ($self) = shift;
	return $self->tree();
}


sub name {
	my ($self) = shift;
	return $self->raid_name();
}

1;
