# $Id: //depot/prod/test/Rsierranevada/lib/Aggregate/Status.pm#1 $
## @summary Library For Aggregate Status/Options Querying
## @author dl-johny@netapp.com (FSS WAFL Quality)
## @cf     false
## @status public
## @pod here

package Aggregate::Status;
use strict;
use Aggregate::ImplTemplate::Status;
use Class::MethodMaker
	new_hash_init => 'new',
	get_set       => [ qw(name status state type) ],
	hash          => [ qw(options) ],
	list          => [ qw(volumes) ],
;

my $impl = Aggregate::ImplTemplate::Status->new(cmd => 'aggr');

sub _xlate_target
{
	my $class = shift;
	my %args  = @_;
	$args{target} = delete $args{aggr};
	return %args;
}

sub fetch_status
{
	my $class = shift;
	$impl->fetch_status($class->_xlate_target(@_));
}

sub fetch_options
{
	my $class = shift;
	$impl->fetch_options($class->_xlate_target(@_));
}

1;

__END__

=head1 NAME

Aggregate::Status - Fetch Aggregate Status and Options

=head1 SYNOPSIS

  my $status = Aggregate::Status->fetch_status(
    conn => $conn,
    aggr => 'aggr1'
  );

  my $name    = $status->name;    # 'aggr1'
  my $status  = $status->status;  # 'raid4', etc
  my $state   = $status->state;   # 'online', 'offline', etc
  my $type    = $status->type;    # 'trad', 'aggr'
  my @volumes = $status->volumes; # list of volumes
  my %options = $status->options; # options

  # you can also access the volumes this way
  my $v = $status->volumes(2); # second volume in list

  # ...and you can also access options this way 
  if ($status->options('root')) {
    print $status->name, " is a root aggr!\n";
  }

  # fetch just the options
  my %options = Aggregate::Status->fetch_options(
    conn => $conn,
    aggr => 'aggr1'
  )

=head1 DESCRIPTION

Aggregate::Status allows you to get to the status of an aggregate.

=head1 METHODS

=head2 Aggregate::Status-E<gt>new(%args)

Creates a Aggregate::Status object, but does NOT populate data automatically.
You can however create this object with dummy values to, for example,
debug your code

=head2 Aggregate::Status-E<gt>fetch_status(%args)

Fetches the current status of an aggregate, and returns a new Aggregate::Status
object that is populated with the appropriate values.

Takes two mandatory parameters:

=over 4

=item conn

The connection object to use to talk to the appliance.

=item aggr

The target aggregate to query for

=back

=head2 Aggregate::Status-E<gt>fetch_options(%args);

Fetches the current option list of an aggregate, and returns a hash containing
key-value pair of the options (boolean options will be set 1 or 0).
Note that unlike fetch_status(), this method does NOT create an object,
but a simple hash.

=over 4

=item conn

The connection object to use to talk to the appliance.

=item aggr

The target aggregate to query for

=back

=head1 ACCESSORS

The following accessors all get/set the appropriate fields.
Please see the documentation for Class::MethodMaker for the various
helper methods available to access these fields.

=head2 $obj-E<gt>name

=head2 $obj-E<gt>status

=head2 $obj-E<gt>state

=head2 $obj-E<gt>type

=head2 $obj-E<gt>volumes

=head2 $obj-E<gt>options

=cut
