# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CConfig/Attr.pm#1 $

## @summary CConfig::Attr
## @author kenneth.gates@netapp.com
## @cf     false
## @status public
## @pod here

=head1 NAME

Attr - Named attribute with value

=head1 SYNOPSIS

  use Attr

=head1 DESCRIPTION

This object represents a name/value pair

=cut

# --------------------------------------------------------
# Class: CConfig::Attr
#   Named attribute with a value. 
# --------------------------------------------------------

package CConfig::Attr;
use strict;
use warnings;
use Params::Validate qw(validate validate_pos OBJECT SCALAR BOOLEAN HASHREF ARRAYREF UNDEF);

# -----------------------------------------------
# Function: new 
#   constructor 
#
# Usage: 
#   CConfig::Attr->new(name,VALUE=>value,REF=>ref,PREFIX=>prefix)
#
# Parameters:
#       name    - attribute name 
#       value   - reference to an attribute value (CConfig::ListIter object) or to a plain scalar
#       ref     - (optional) means that the value of this attribute is a reference to another attribute value
#       prefix  - (optional) character to prefix to attribute value (<,>,etc) 
#
# Return: 
#   blessed ref
# Assumptions:
# -----------------------------------------------
sub new {
    my $invocant = shift;
    my $name=shift;
    my (%args) = validate(@_, {
            VALUE  => { type  => OBJECT, optional=>1, isa=>'CConfig::ListIter'},
            REF    => {type=>SCALAR, optional=>1},
            PREFIX => {type=>SCALAR, optional=>1, regex=>qr/^(<|>|<=|>=|!)$/}
            }
    );
    my $class    = ref($invocant) || $invocant;
    my $self     = {
        NAME=>$name,
        VALUE=>$args{'VALUE'},
        PREFIX=>$args{'PREFIX'},
        REF=>$args{'REF'},
    };

    bless($self, $class);
}

# accessor methods
sub NAME    { my $self = shift; if(@_) { $self->{NAME} = $_[0]; } return $self->{NAME}; }
sub VALUE   { my $self = shift; if(@_) { $self->{VALUE} = $_[0]; } return $self->{VALUE}; }
sub REF     { my $self = shift; if(@_) { $self->{REF} = $_[0]; } return $self->{REF}; }
sub PREFIX  { my $self = shift; if(@_) { $self->{PREFIX} = $_[0]; } return $self->{PREFIX}; }

# -----------------------------------------------
# Function: copy
#   copy of the attribute and its value
#
# Usage:
#   my $attr=$attr->copy(<overrides>)
#
# Parameters:
#
# Return:
#   New copy attribtue and value
# -----------------------------------------------
sub copy {
    my $self=shift;
    my %args = (@_);
    my $copy=__PACKAGE__->new($self->NAME); 
    $copy->VALUE($self->VALUE->copy(%args)) if defined($self->VALUE);
    $copy->REF($self->REF) if defined($self->REF);
    $copy->PREFIX($self->PREFIX) if defined($self->PREFIX);
    return $copy;
}

=head1 FUNCTIONS


=head2 new 

    constructor 

=head3 Usage

    CConfig::Attr->new(name)

=head3 Return

    blessed ref

=head2 copy

    copy of the attribute and its value

=head3 Usage

    my $attr=$attr->copy(<overrides>)

=head3 Return

    New copy attribtue and value

=head3 Parameters:


=over 4


=item overrides

    any of the value iterator attributes: LIST, LISTOP, START, STEP, CHUNK, NUM, DIRECTION, ROOT, DEPTH, WIDTH, KEY, FOREACH
    that you want to change in the copy
    

=back



=head1 SEE ALSO

=head1 HISTORY

=head1 TODO

=cut

1;

