# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CConfig/Impl.pm#3 $

## @summary Impl 
## @author kenneth.gates@netapp.com
## @cf     false
## @status public
## @pod here

=head1 NAME

Impl - Base class for Implementation Functions

=head1 SYNOPSIS

  use Impl 

=head1 DESCRIPTION

This module defines routines related to object action implementation

=cut

# --------------------------------------------------------
# Class: CConfig::Impl
#   Implementation object base class
# --------------------------------------------------------


package CConfig::Impl;
use List::Util;
use strict;
use warnings;
use Data::Dumper;
use NACL::ComponentUtils;
use Params::Validate qw(validate validate_pos OBJECT SCALAR BOOLEAN HASHREF ARRAYREF UNDEF);
use Tharn qw($Log);

use NACL::STask::Node;
use NACL::APISet;
use CConfig::Subtest;

use CConfig::Impl::Aggr;
use CConfig::Impl::AvAccessPolicy;
use CConfig::Impl::AvEngine;
use CConfig::Impl::CclDebug;
use CConfig::Impl::Cluster;
use CConfig::Impl::Command;
use CConfig::Impl::IpAddress;
use CConfig::Impl::Iscsi;
use CConfig::Impl::License;
use CConfig::Impl::Lun;
use CConfig::Impl::LunIgroup;
use CConfig::Impl::LunPortset;
use CConfig::Impl::JobSchedule;
use CConfig::Impl::ExportPolicy;
use CConfig::Impl::ExportPolicyRule;
use CConfig::Impl::FailoverGroup;
use CConfig::Impl::FlexCache;
use CConfig::Impl::FlexClone;
use CConfig::Impl::NameMapping;
use CConfig::Impl::NetInterface;
use CConfig::Impl::NetPort;
use CConfig::Impl::NetPortIfgrp;
use CConfig::Impl::NetRouteGroup;
use CConfig::Impl::NetRoute;
use CConfig::Impl::NodeCommand;
use CConfig::Impl::QosPolicyGroup;
use CConfig::Impl::QosSettingsArchivedWorkload;
use CConfig::Impl::QosSettingsControl;
use CConfig::Impl::QosSettingsReadAhead;
use CConfig::Impl::QosWorkload;
use CConfig::Impl::QuotaPolicy;
use CConfig::Impl::QuotaPolicyRule;
use CConfig::Impl::Qtree;
use CConfig::Impl::SisPolicy;
use CConfig::Impl::Snapshot;
use CConfig::Impl::SnapshotPolicy;
use CConfig::Impl::Snapmirror;
use CConfig::Impl::Vlan;
use CConfig::Impl::Volume;
use CConfig::Impl::Vserver;
use CConfig::Impl::VserverPeer;
use CConfig::Impl::VserverServDNS;
use CConfig::Impl::VserverNFS;
use CConfig::Impl::VserverServUnixUser;
use CConfig::Impl::VserverServUnixGroup;
use CConfig::Impl::VserverCifs;
use CConfig::Impl::VserverCifsShare;
use CConfig::Impl::VserverCifsPrefDc;
use CConfig::Impl::VserverServNISDomain;
use CConfig::Impl::Test;
use Carp;

our %disallow_zapi;
our $prefer_zapi           = 0;
our $shuffle               = 1;

our $_CCL_WORKAROUND_BURT650568 = Tharn::param( "CCL_WORKAROUND_BURT650568" );
our $_MAX_ONTAP_LINE_LENGTH     = Tharn::param( "CCL_MAX_ONTAP_LINE_LENGTH" ) || 512;

# Default this to 1 since Ontap isn't going to fix it until FS, if ever...
if ( !defined($_CCL_WORKAROUND_BURT650568) ) {
    $_CCL_WORKAROUND_BURT650568 = 1;
}

# When adding new implementations, please add in alphabetic order
our %object_2_impl = (
    'aggregate'                         => 'CConfig::Impl::Aggr',
    'av_access_policy'                  => 'CConfig::Impl::AvAccessPolicy',
    'av_engine'                         => 'CConfig::Impl::AvEngine',
    'ccl_debug'                         => 'CConfig::Impl::CclDebug',
    'cifs'                              => 'CConfig::Impl::VserverCifs',
    'cifs_preferred_dc'                 => 'CConfig::Impl::VserverCifsPrefDc',
    'cifs_share'                        => 'CConfig::Impl::VserverCifsShare',
    'cluster'                           => 'CConfig::Impl::Cluster',
    'command'                           => 'CConfig::Impl::Command',
    'dns'                               => 'CConfig::Impl::VserverServDNS',
    'export_policy'                     => 'CConfig::Impl::ExportPolicy',
    'export_policy_rule'                => 'CConfig::Impl::ExportPolicyRule',
    'failover_group'                    => 'CConfig::Impl::FailoverGroup',
    'flex_cache'                        => 'CConfig::Impl::FlexCache',
    'flex_clone'                        => 'CConfig::Impl::FlexClone',
    'interface'                         => 'CConfig::Impl::NetInterface',
    'ip_address'                        => 'CConfig::Impl::IpAddress',
    'iscsi'                             => 'CConfig::Impl::Iscsi',
    'job_schedule'                      => 'CConfig::Impl::JobSchedule',
    'license'                           => 'CConfig::Impl::License',
    'lun'                               => 'CConfig::Impl::Lun',
    'lun_igroup'                        => 'CConfig::Impl::LunIgroup',
    'lun_portset'                       => 'CConfig::Impl::LunPortset',
    'name_mapping'                      => 'CConfig::Impl::NameMapping',
    'nfs'                               => 'CConfig::Impl::VserverNFS',
    'nis_domain'                        => 'CConfig::Impl::VserverServNISDomain',
    'node_command'                      => 'CConfig::Impl::NodeCommand',
    'port'                              => 'CConfig::Impl::NetPort',
    'port_ifgrp'                        => 'CConfig::Impl::NetPortIfgrp',
    'qos_policy_group'                  => 'CConfig::Impl::QosPolicyGroup',
    'qos_settings_archived_workload'    => 'CConfig::Impl::QosSettingsArchivedWorkload',
    'qos_settings_control'              => 'CConfig::Impl::QosSettingsControl',
    'qos_settings_read_ahead'           => 'CConfig::Impl::QosSettingsReadAhead',
    'qos_workload'                      => 'CConfig::Impl::QosWorkload',
    'qtree'                             => 'CConfig::Impl::Qtree',
    'quota_policy_rule'                 => 'CConfig::Impl::QuotaPolicyRule',
    'quota_policy'                      => 'CConfig::Impl::QuotaPolicy',
    'route'                             => 'CConfig::Impl::NetRoute',
    'routing_group'                     => 'CConfig::Impl::NetRouteGroup',
    'sis_policy'                        => 'CConfig::Impl::SisPolicy',
    'snapmirror'                        => 'CConfig::Impl::Snapmirror',
    'snapshot'                          => 'CConfig::Impl::Snapshot',
    'snapshot_policy'                   => 'CConfig::Impl::SnapshotPolicy',
    'test'                              => 'CConfig::Impl::Test',
    'unix_user'                         => 'CConfig::Impl::VserverServUnixUser',
    'unix_group'                        => 'CConfig::Impl::VserverServUnixGroup',
    'vlan'                              => 'CConfig::Impl::Vlan',
    'volume'                            => 'CConfig::Impl::Volume',
    'vserver'                           => 'CConfig::Impl::Vserver',
    'vserver_peer'                      => 'CConfig::Impl::VserverPeer',
);

our %impl_2_object = reverse %object_2_impl;

# To shuffle or not to shuffle
sub shuffle {
    if ( CConfig::Impl->SHUFFLE() ) {
        return List::Util::shuffle( @_ );
    } else {
        return @_;
    }
}

# -----------------------------------------------
# Function: create_impl
#   factory method to create implementation object
#
# Usage: 
#
# Parameters:
#       item - reference to the parser hash
#           item->{'ACTION'} = create | find
#           item->{'OBJECT'} = volume | aggregate | cluster | vserver
#           item->{'WITH'} = CConfig::AttrList of attributes
#
# Return: 
#   CConfig::Impl subclass object
#
# Assumptions:
# -----------------------------------------------
sub create_impl {
    my $item=shift;
    my $me=(caller(0))[3];

    for my $object (lc $item->{'OBJECT'}) {
        my $impl = CConfig::Impl->object2impl( $object );
        if ( $impl ) {
            return $impl->new($item);
        } 
        croak "ERROR $me: Don't know how to create object ".$item->{'OBJECT'};
    }
}

# -----------------------------------------------
# Function: object2impl
#   Class routine to get the impl (i.e. "CConfig::Impl::Volume") from the object
#   (i.e. "volume") 
#
# Usage: 
#   my $impl_name = CConfig::Impl->object2impl("volume")
#
# Parameters:
#       object - CCL object name (i.e. "aggregate", "volume", etc...)
#
# Return: 
#   impl name (i.e. "CConfig::Impl::Volume", given input of "volume")
#
# -----------------------------------------------
sub object2impl {
    my $pkg=shift;
    my $object=shift;
    my $me=(caller(0))[3];

    if ( exists $object_2_impl{$object} ) {
        return $object_2_impl{$object};
    } 
    return undef;
}

# -----------------------------------------------
# Function: impl2object
#   Class routine to get the object (i.e. "volume") from the impl class name
#   (i.e. "CConfig::Impl::Volume") 
#
# Usage: 
#   my $obj_name = CConfig::Impl->impl2object("CConfig::Impl::Volume")
#
# Parameters:
#       impl name - CCL class name of Implementation (i.e. "CConfig::Impl::Aggregate", 
#                     "CConfig::Impl::Volume", etc...)
#
# Return: 
#   object name (i.e. "volume", given input of "CConfig::Impl::Volume")
#
# -----------------------------------------------
sub impl2object {
    my $pkg=shift;
    my $impl=shift;
    my $me=(caller(0))[3];

    if ( exists $impl_2_object{$impl} ) {
        return $impl_2_object{$impl};
    } 
    return undef;
}

# -----------------------------------------------
# Function: new
#   create new object
#
# Usage: 
#   CConfig::Impl->new(KEYS=><keys>,DEFAULT=><default>,COMPAT=><compat>)
#
# Parameters:
#   <keys> - reference to an array of object keys 
#   <default> - reference to an array of default attributes 
#   <compat> - reference to a hash of compatible keys
#
# Return: 
#
# -----------------------------------------------
sub new {
    my $class = shift;
    my (%args) = validate(@_, {
            KEYS    => {type=>ARRAYREF, optional=>1},
            DEFAULT => {type=>ARRAYREF, optional=>1},
            COMPAT  => {type=>HASHREF,  optional=>1}
            }
    );
    $args{'DEFAULT'} ||= $args{'KEYS'};
    if ( !$args{'COMPAT'} ) { 
        $args{'COMPAT'} = {}; 
    } else {
        my $c = $args{'COMPAT'};
        my @keys = keys %{$args{'COMPAT'}};
        # Add reverse compat too
        foreach my $k (@keys) {
            my $compat = $args{'COMPAT'}->{$k};
            $args{'COMPAT'}->{$compat} = $k;
        }
    }

    # create a new object
    my $self= bless {
        RGEN=>new String::Random,
        CMD_INT=>$CConfig::Nodes[0],
        KEYS=>$args{'KEYS'},
        DEFAULT=>$args{'DEFAULT'},
        COMPAT=>$args{'COMPAT'},
        SA_LIST=>[],
        FILER_OBJS_LOADED=>0
    }, $class;

    # Initialize object sets if KEYS is specified
    #    RESULT_OBJS = set of impl objects that were both found and created
    #    CREATED_OBJS = set of impl objects that were created only
    #    FILER_OBJS = set of impl objects that were already on the filer - we don't populate this unless
    #           the action creates an object that does not have a key attribute (ie volume name) set and 
    #           we must generate a unique name
    if (defined($args{'KEYS'})) {
        $self->{'RESULT_OBJS'}  =CConfig::ObjSet->new(KEYS=>$args{'KEYS'},DEFAULT=>$args{'DEFAULT'},COMPAT=>$args{'COMPAT'});
        $self->{'CREATED_OBJS'} =CConfig::ObjSet->new(KEYS=>$args{'KEYS'},DEFAULT=>$args{'DEFAULT'},COMPAT=>$args{'COMPAT'});
        $self->{'FILER_OBJS'} =CConfig::ObjSet->new(KEYS=>$args{'KEYS'},DEFAULT=>$args{'DEFAULT'},COMPAT=>$args{'COMPAT'});
    }
    $self->{UNDO_LIST} = [];

    return $self;
}

# selectors
sub KEYS              { my $self = shift; if(@_) { $self->{KEYS} = $_[0];    } return $self->{KEYS}; }
sub COMPAT            { my $self = shift; if(@_) { $self->{COMPAT} = $_[0];    } return $self->{COMPAT}; }
sub OPTIONS           { my $self = shift; return $CConfig::OPTIONS; }
sub CREATED_OBJS      { my $self = shift; if(@_) { $self->{CREATED_OBJS} = $_[0]; } return $self->{CREATED_OBJS}; }
sub RESULT_OBJS       { my $self = shift; if(@_) { $self->{RESULT_OBJS} = $_[0];   } return $self->{RESULT_OBJS}; }
sub FILER_OBJS        { my $self = shift; if(@_) { $self->{FILER_OBJS} = $_[0];   } return $self->{FILER_OBJS}; }
sub FILER_OBJS_LOADED { my $self = shift; if(@_) { $self->{FILER_OBJS_LOADED} = $_[0];   } return $self->{FILER_OBJS_LOADED}; }

sub COMPAT_KEYS       { my $self = shift; return (@{$self->{KEYS}}, keys %{$self->{COMPAT}}); }

sub BEST_EFFORT     { 
    my $opt="BEST_EFFORT";
    if ($_[0]->OPTIONS->defined(option=>$opt)) { return $_[0]->OPTIONS->get(option=>$opt); }
    return 0;
}
sub STRICT_FIND     { 
    my $opt="STRICT_FIND";
    if ($_[0]->OPTIONS->defined(option=>$opt)) { return $_[0]->OPTIONS->get(option=>$opt); }
    return 0;
}
sub MULTI_CLUSTER   { 
    my $opt="MULTI_CLUSTER";
    if ($_[0]->OPTIONS->defined(option=>$opt)) { return $_[0]->OPTIONS->get(option=>$opt); }
    return 0;
}

# This is the NACL Node object on which to execute the command
sub CMD_INT { 
    my $self = shift; 
    if (@_) { 
        if ( ref($_[0]) =~ /NACL::(C|STask)::Node/ ) {
            $self->{CMD_INT} = $_[0];    
        } else {
            $self->{CMD_INT} = find_node_component( name => $_[0]);    
        }
    } 
    return $self->{CMD_INT}; 
}

# -----------------------------------------------
# Function: check_keys
#   Check that the args contain all required keys 
#
# Usage: 
#   $obj->check_keys(%args)
#
# Parameters:
#       self - object instance
#       %args - hash of args to be checked
#
# Return: 
#       1 - all keys found else die
# -----------------------------------------------
sub check_keys {
    my $self = shift;
    my $args=shift;
    my $me=(caller(0))[3];
    my $hcompat = $self->COMPAT;
    foreach my $k (@{$self->KEYS}) {
        if ( !defined($args->{$k}) ) {
            if ( $hcompat && (defined($hcompat->{$k}) && defined($args->{$hcompat->{$k}})) ) {
                $args->{$k} = $args->{$hcompat->{$k}};
            } else {
                croak "ERROR $me: missing required key $k ";
            }
        } elsif ( $hcompat && (defined($hcompat->{$k})) ) {
            # The key is the primary argument, make sure that the value stored 
            # in the compatible arg matches it
            $args->{$hcompat->{$k}} = $args->{$k};
        }
    }
    return 1;
}

# -----------------------------------------------
# Function: find_node_component
#   Find the NACL Node component given the node name
#
# Usage: 
#   CConfig::Impl::find_node_component( name => 'mynode' )
#
# Parameters:
#       name - name of node
#
# Return: 
#       NACL Node reference or dies
# -----------------------------------------------
sub find_node_component {
    my $me=(caller(0))[3];
    my %opt = ( name => undef, @_ );
    if ( !$opt{name} ) {
        croak "ERROR $me: Must supply \"name\" option";
    }
    my @nodes = @CConfig::Nodes;
    my @node = grep( $_, (map { if ($_->{node} eq $opt{name}) { $_; } } @nodes));
    if ( @node ) {
        return $node[0];
    } else {
        croak "ERROR $me: Could not find NACL Node for node $opt{name}";
    }
}

# -----------------------------------------------
# Function: select_node
#   Selects either the first NACL Node or a random node
#
# Usage: 
#   $obj->select_node();
#
# Parameters:
#       method - first|random
#
# Return: 
#       Nothing
# -----------------------------------------------
sub select_node {
    my $self=shift;
    my $default_method = 'first';
    if ( $self->OPTIONS->get(option=>'RANDOM_NODE') ) {
        $default_method = 'random';
    }
    my %opt = ( method => $default_method, @_ );
    my @nodes = @CConfig::Nodes;

    my $node;
    if ( $opt{method} =~ /first/i ) {
        $node = $nodes[0];
    } else {
        $node = $nodes[rand(@nodes)];
    }
    $self->CMD_INT($node); 
}

# -----------------------------------------------
# Function: invoke_action
#   Call "actions" after performing pre-action actions
#
# Usage: 
#   $obj->invoke_action(item)
#
# Parameters:
#       self - object instance
#       item - reference to the parser hash
#
# Return: 
#       resutls of calling actions
#
# Assumptions:
# -----------------------------------------------
sub invoke_action {
    my $self=shift;
    my $item=shift;
    my $me=(caller(0))[3];
    my $hcompat = $self->COMPAT;

    my $default_method = 'first';
    if ( $self->OPTIONS->get(option=>'RANDOM_NODE') ) {
        $default_method = 'random';
    }

    # examine ON values, see what node or cluster to execute on then set execution node
    my %on=$item->{ON}->to_hash();

    if ($on{'node'}) { 
        $self->CMD_INT($on{'node'}); 
    } elsif ($on{'cluster'}) { 
        PICKNODE: foreach my $cluster (@{CConfig::get('@CLUSTERS')}) {
            if ($cluster->{'_default_'} eq $on{'cluster'}) {
                my $nodes=$cluster->{'nodes'};
                my $node;
                if ( $default_method =~ /first/i ) {
                    $node=$nodes->[0];
                } else {
                    $node=$nodes->[rand(@{$nodes})];
                }
                $self->CMD_INT($node->{'_default_'}); 
                last PICKNODE;
            }
        }
    } else {
        $self->select_node(); 
    }

    my %args = @_;

    # run_command doesn't have KEYS set
    if ( $self->KEYS ) {
        foreach my $k (@{$self->KEYS}) {
            # We define the arguments to be of the names of the oldest release
            # that we want to support, then let APISet's compatability layer
            # handle forward compatabilty for us.  So if we expect "server"
            # and the newer option is "vserver", see if the ruleset specified 
            # "vserver"
            if ( !defined($args{$k}) ) {
                if ( $hcompat && (defined($hcompat->{$k}) && defined($args{$hcompat->{$k}})) ) {
                    $args{$k} = $args{$hcompat->{$k}};
                }
            }
        }
    }

    my $per_rule_best_effort=0;

    # find_attr "best-effort"
    my $be=$item->{'WITH'}->find_attr('-best-effort');
    if ($be) {
        $per_rule_best_effort=$be->VALUE->get_next()->[0];
    } elsif ( $be=$item->{'WITH'}->find_attr('-best_effort') ) {
        $per_rule_best_effort=$be->VALUE->get_next()->[0];
    }

    my $rc=eval { $self->action($item,%args); };
    if ($@) { 
        my $fail=$@; 
        if ($self->BEST_EFFORT || $per_rule_best_effort) { 
            $Log->warn($fail) 
        } else { 
            $Log->warn("ACTION failed with |$fail|, calling POST to cleanup"); 
            eval { $self->post(); };
            if ($@) { 
                $Log->warn("POST Failed with: $@"); 
            }
            die $fail;
        }
    } 
    return $rc;
}

# -----------------------------------------------
# Function: invoke_post
#   Call "post" after performing pre-post actions
#
# Usage: 
#   $obj->invoke_post()
#
# Parameters:
#       self - object instance
#       item - reference to the parser hash
#
# Return: 
#       resutls of calling post
#
# Assumptions:
# -----------------------------------------------
sub invoke_post {
    my $self=shift;

    eval { $self->post(); };
    if ($@) { 
        die $@;
    }

    # do subactions
    eval { 
        foreach my $subaction (@{$self->{'SA_LIST'}}) { 
            # Each Attribute ListIter which was stored in subactions in SA_LIST had
            # it's iterators frozen when they were added.  However, CURRENT at
            # the time they were added is not the same as CURRENT when they are
            # executed.  Update the POS to what the List CURRENT is now.
            foreach my $attr (@{$subaction->{WITH}->DATA}) {
                if ( (ref($attr->{VALUE}) eq "CConfig::ListIter") && 
                     ($attr->{VALUE}->START() =~ /CURRENT/i) ) {
                    $attr->{VALUE}->POS( $attr->{VALUE}->{LIST}->POS() );
                }
            }

            CConfig::Rule::_process_rule($subaction); 
        }
    };
    if ($@) { 
        die $@;
    }
}

# -----------------------------------------------
# Function: action
#   Implement object actions (create/find/sub actions...)
#
# Usage: 
#   $obj->action(item)
#
# Parameters:
#       self - object instance
#       item - reference to the parser hash
#           item->{'ACTION'} = create | find
#           item->{'OBJECT'} = volume | aggregate | cluster | vserver
#           item->{'WITH'} = CConfig::AttrList of attributes
#
# Return: 
#       0
#
# Assumptions:
#   This will be called to implement object actions on each instance of an object
#   Actions will be subtested if NUM is > 1, or the Implementation, or user forbids it
#       subtested jobs will be finalized in the post method
#   Derived class should override this method to provide the desired behavior
# -----------------------------------------------
sub action {
    my $self=shift;
    my $item=shift;
    my $me=(caller(0))[3];
    return 0;
}

# -----------------------------------------------
# Function: post
#       Wait for background object creation to complete
#       Verify objects were successfully created 
#       log undo actions to undo file
#
# Usage: 
#       $obj->post(item)
#
# Parameters:
#       self - object instance
#
# Return: 
#       Nothing
#
# Assumptions:
#       Derived class should override this method
# -----------------------------------------------
sub post {
    my $self=shift;
    my $me=(caller(0))[3];
} 

# -----------------------------------------------
# Function: get_results
#       Update and return results 
#
# Usage: 
#       $obj->get_results(item)
#
# Parameters:
#       self - object instance
#
# Return: 
#       Reference to list of created or found objects
#
# Assumptions:
#       Derived class should override this method
# -----------------------------------------------
sub get_results {
    my $self=shift;
    my $me=(caller(0))[3];
    my @result=();
    return \@result;
} 

# -----------------------------------------------
# Function: add_compatible_keys
#   Add compatible keys.  Key names can change from release to release.
#   CCL cares only to the extent that we keep lists of keys so that
#   we assure that all required keys are specified during object
#   creation.  Individual implementations can keep a list of
#   synonymous key names.  These synonymous key names will be
#   added if the rule did not.
#
# Usage:
#   $obj->add_compatible_keys(href, default )
#
# Parameters:
#       self    - object instance
#       href    - reference to the parser or create args hash
#       default - reference to the hash of defaults for args
#
# Return:
#
# Assumptions:
# -----------------------------------------------
sub add_compatible_keys {
    my $self=shift;
    my $me=(caller(0))[3];
    my $caller=(caller(1))[3];
    my ($href,$default) = @_;

    my $cref = $self->COMPAT;
    my $kref = $self->KEYS;

    foreach my $k (@$kref) {
        my $compat;
        if ( $cref && defined($cref->{$k}) ) {
            $compat = $cref->{$k};
        }

        if (!defined($href->{$k}) && $compat && defined($href->{$compat})) {
            $href->{$k}=$href->{$compat};
        } elsif (defined($href->{$k}) && $compat ) {
            if ( !defined($href->{$compat}) ) {
                $href->{$compat}=$href->{$k};
            }
        } elsif ( $default ) {
            if ( defined($default->{$k}) ) {
                $href->{$k}=$default->{$k};
                if ( $compat ) {
                    $href->{$compat}=$default->{$k};
                }
            } elsif ( $compat && defined($default->{$compat}) ) {
                $href->{$k}     =$default->{$compat};
                $href->{$compat}=$default->{$compat};
            }
        }
    }
}

# -----------------------------------------------
# Function: gen_string
#   Generate a string from a pattern in the form:
#
# Usage:
#   $obj->gen_string( pattern => $pattern, start => $start );
#
# Parameters:
#       self    - object instance
#       href    - reference to the parser or create args hash
#       default - reference to the hash of defaults for args
#
# Return:
#
# Assumptions:
# -----------------------------------------------
sub gen_string {
    my $self=shift;
    my $me=(caller(0))[3];
    my $caller=(caller(1))[3];

    my %options = (
        pattern => "name#rrr",
        start   => undef,
        @_
    );

    my $return = $options{pattern};
    my $orig   = $return;

    # if we can't match this pattern, we will just return the string unchanged
    if ( $options{pattern} =~ /(\S+)([%])\(?([a-zA-Z0-9:]+)\)?(\S*)/ ) {
        my $prefix = $1;
        my $operator = "NUMERIC";
        my $replace = $3;
        my $suffix = $4;

        my $orig_replace = $replace;

        my $start = undef;
        # Option to specify starting point with %(sss:200)
        if ( $replace =~ /([a-zA-Z]+):([a-zA-Z]+)/ ) {
            $operator = "STRING";
           $replace = $1;
           $start = $2;
        } elsif ( $replace =~ /([a-zA-Z]+):(\d+)/ ) {
           $replace = $1;
           $start = $2;
        }

        $replace = lc $replace;

        if ( $operator =~ /NUMERIC/i ) {
            # r is Random, s is sequential
            if ( $replace =~ /r+/i ) {
                my $rgen_pattern = $replace;
                $rgen_pattern =~ s/./n/g;
                $return = $prefix . $self->{RGEN}->randpattern($rgen_pattern);
                $return = $return . $suffix if ( $suffix );
            } elsif ( $replace =~ /s+/i ) {
                my $num = length $replace;

                $return = $prefix;
                my $tag = "PREFIX:$prefix,OPERATOR:NUMERIC,REPLACE:$orig_replace";
                $tag = $tag.",SUFFIX:$suffix" if ( $suffix );

                my $seed;
                # if caller is telling us where to start
                if ( $options{start} ) {
                    $seed = $options{start};
                    # if we have a cached value
                } elsif ( $self->{AUTOGEN_SEEDS}->{$tag} ) {
                    $seed = $self->{AUTOGEN_SEEDS}->{$tag};
                    # if the pattern is telling us where to start
                } elsif ( defined $start ) {
                    $seed = $start;
                } else {
                    $seed = 1;
                }

                my $seq = sprintf( "%0".$num."d", $seed ); $seed++;
                $self->{AUTOGEN_SEEDS}->{$tag} = $seed;

                $return = $return . $seq;
                $return = $return . $suffix if ( $suffix );
            } else {
                die "$me does not understand $replace in $options{pattern}.  Must be some number of \"s\"s or \"r\"s.";
            }

        } else {
            if ( $replace =~ /r+/i ) {
                $self->{RGEN}->{'A'} = [ 'A'..'Z', 'a'..'z' ];
                my $rgen_pattern = "AAA";
                $rgen_pattern = $replace;
                $rgen_pattern =~ s/./A/g;
                $return = $prefix . $self->{RGEN}->randpattern($rgen_pattern);
                $return = $return . $suffix if ( $suffix );
            } elsif ( $replace =~ /s+/i ) {
                my $num = length $replace;
                $return = $prefix;
                my $tag = "PREFIX:$prefix,OPERATOR:STRING,REPLACE:$orig_replace";
                $tag = $tag.",SUFFIX:$suffix" if ( $suffix );

                my $seed;
                # if caller is telling us where to start
                if ( $options{start} ) {
                    $seed = $options{start};
                    # if we have a cached value
                } elsif ( $self->{AUTOGEN_SEEDS}->{$tag} ) {
                    $seed = $self->{AUTOGEN_SEEDS}->{$tag};
                    # if the pattern is telling us where to start
                } elsif ( $start ) {
                    $seed = $start;
                } else {
                    $seed = 'a' x $num;
                }
                my $seq = $seed; $seed++;

                $self->{AUTOGEN_SEEDS}->{$tag} = $seed;

                $return = $return . $seq;
                $return = $return . $suffix if ( $suffix );
            } else {
                die "$me does not understand $replace in $options{pattern}.  Must be some number of \"s\"s or \"r\"s.";
            }
        }
    }

    # If there are multiple % patterns in the string, call again
    if ( ($return =~ /%/ ) && 
         ($return ne $orig) ) {
        $return = $self->gen_string( pattern => $return, start => $options{start} );
    }

    return $return;
}

# -----------------------------------------------
# Function: nacl_component_classname
#   Static method which returns the primary NACL
#   component classname for this implementation
#
# Usage:
#   my $nacl_c_classname = nacl_component_classname();
#
# -----------------------------------------------
sub nacl_component_classname {
    die "CConfig::Impl::nacl_component_classname Must be redefined in derived classes";
}

# -----------------------------------------------
# Function: c2cs
#   Convert Component to ComponentState name
#
# Usage:
#   my $cs = c2cs( 'NACL::C::Volume' );
#
# -----------------------------------------------
sub c2cs {
    my $cType = shift;
    my $csType = $cType;
    $csType =~ s/:C:/:CS:/g;
    $csType =~ s/:STask:/:CS:/g;
    return $csType;
}

# -----------------------------------------------
# Function: subtest_default
#   Looks at the SUBTEST_MIN for the implementation type
#   and the number of objects that we are going to create
#   and if num is >= SUBTES_MIN, then return 1, else 0.
#
# Usage:
#   my $st_def = $obj->subtest_default( num => 2, object => 'interface' );
#
# -----------------------------------------------
sub subtest_default {
    my $self=shift;
    my $me=(caller(0))[3];
    my $subtest = 1;

    my %opt = (
        num    => 1,
        object => undef,
        @_
    );

    my $min = CConfig::Subtest::SUBTEST_MIN( object => $opt{object} );
    my $num = $opt{num};
    if ( $min ) {
        # If num is not an integer (i.e. ALL), default to 1 for the check
        if ( $num !~ /^[0-9]+$/ ) {
            $num = 1;
        }

        if ( $num < $min ) {
            $subtest = 0;
        }
    } else {
        $subtest = 0;
    }

    if ( !exists $self->{subtest_at_a_time} ) {
        $self->{subtest_at_a_time} = CConfig::Subtest::SUBTEST_AT_A_TIME( object => $opt{object} );
    }

    return $subtest;
}

# This is a little different from call_nacl_component_routine as we are calling this
# via the undo mechanism, and do not have an Impl object (i.e. this is strictly a
# static method)
# -----------------------------------------------
# Function: purge
#   Static method used to undo an Impl object
#
# Usage:
#   my $impl_name = 'CConfig::Impl::Volume';
#   CConfig::Impl::purge( $impl_name, -CmdInterface => "csiqa-3070-35", 
#                          vserver => 'vs0', volume => 'vol1' );
#
# Parameters:
#       impl_name - Impl class name (i.e. 'CConfig::Impl::Volume', 'CConfig::Impl::Aggr', etc...)
#       -CmdInterface - Name of node to use as the command-interface
#       args    - reference to the hash containing the args to pass
#                 to the specific ComponentState::create command
#
# Return:
#
# Assumptions:
#    Note: It should be possible to specify a simulation mode where
#          instead of calling a NACL routine, we would instead call
#          a filer simulator that works off the CDEFs
# -----------------------------------------------
sub purge {
    my $class_or_obj=shift;
    my $implclassref = ref($class_or_obj);
    if ( !$implclassref ) {
        $implclassref = $class_or_obj;
    }
    my $object = CConfig::Impl->impl2object( $implclassref );

    #local $Data::Dumper::Maxdepth=2;
    #print "Impl::purge opts =".Dumper(\@_)."\n"; 

    my %opt = ( '-CmdInterface'   => undef,
                '-component_name' => undef,
                '-subtest'        => CConfig::Subtest::SUBTEST_MIN( object => $object ),
                @_ );

    $opt{command_interface} = CConfig::Impl::find_node_component( name => $opt{'-CmdInterface'});
    delete $opt{'-CmdInterface'};

    my $do_subtest = delete $opt{'-subtest'};
    my $compname   = delete $opt{'-component_name'};

    # See if there is a implmentation specific do_purge() or should we use
    # the generic CConfig::Impl::do_purge() ?
    if ( !$compname ) {
        $compname = $implclassref->nacl_component_classname();
    }
    my $methodname = 'CConfig::Impl::do_purge';
    my $symb = \%CConfig::Impl::;

    OUTER: foreach my $key (sort keys %$symb ) {
        if ( 'CConfig::Impl::'.$key eq $implclassref.'::' ) {
            foreach my $subkey (sort keys %{$symb->{$key}} ) {
                if ( $subkey =~ /do_purge/ ) {
                    $methodname = $implclassref.'::do_purge';
                    last OUTER;
                }
            }
        }
    }

    my $allow_zapi  = $implclassref->allow_zapi();
    my $prefer_zapi = $implclassref->PREFER_ZAPI();
    if ( !$allow_zapi && $prefer_zapi ) {
        $opt{'apiset_must'} = { interface => 'CLI' };
        if ( exists $opt{'apiset_should'} ) {
            delete $opt{'apiset_should'};
        }
    }

    # If specified as part of a rule, option will a scalar
    if ( exists $opt{'apiset_must'} && !ref($opt{'apiset_must'}) ) {
        $opt{'apiset_must'} = { interface => $opt{'apiset_must'} };
    }
    if ( exists $opt{'apiset_should'} && !ref($opt{'apiset_should'}) ) {
        $opt{'apiset_should'} = { interface => $opt{'apiset_should'} };
    }

    if ( $do_subtest ) {
        CConfig::Subtest::subtest( method => $methodname, class => $compname, args => \%opt );
    } else {
        my $libname;
        if ( $methodname =~ /(\S+)::do_purge/ ) {
            $libname = $1;
            if ( !defined $CConfig::LOADED{$libname} ) {
                eval "use $libname";
                $CConfig::LOADED{$libname} = 1;
            }

            my @ret;
            eval { @ret = $compname->$methodname( %opt ); };
            my $rc = $@;

            # Ontap declares ERRORs for some things which aren't really errors
            if ( $rc =~ /Job\s+(\d+)\s+(is running in the background|has been queued)/ ) {
                my $jobid = $1;

                my $result = NATE::Result::Warn->new( $rc );
                $result->misc($rc);
                $result->log();

                push(@CConfig::BG_JOBS,$jobid);

                # wait for background jobs to complete
                 if (@CConfig::BG_JOBS) {
                     $Log->trace("Waiting for jobs ".join(',',@CConfig::BG_JOBS));
                     while(CConfig::Job::are_jobs_active(\@CConfig::BG_JOBS)) { sleep 10; }
                 }
     
                 # check bg jobs
                 if (@CConfig::BG_JOBS) { CConfig::Job::check_jobs(\@CConfig::BG_JOBS); }
                 @CConfig::BG_JOBS=();
            } elsif ( $rc ) {
                die $rc;
            }

            return wantarray ? @ret : $ret[0];
        }
    }
}

# -----------------------------------------------
# Function: do_purge
#   Default purge routine which just calls delete on the NACL
#   component.  Some Impls override this routine (like Vserver and Volume).
#
# Usage:
#   do_purge( 'NACL::C::Volume', %opt );
#
# -----------------------------------------------
sub do_purge {
    my $class = shift;
    my %opt = ( 'command_interface' => undef,
                @_ );

     $class->delete(  %opt );
}

# This is a little different from call_nacl_component_routine as we are calling this
# via the undo mechanism, and do not have an Impl object (i.e. this is strictly a
# static method)
# -----------------------------------------------
# Function: undo
#   Static method used to undo an Impl object.  Looks up a "do_undo()" 
#   static method on the specific implementation (say CConfig::Impl::Volume) 
#   or if one is not found defaults to CConfig::Impl::do_undo
#
# Usage:
#   my $impl_name = 'CConfig::Impl::Volume';
#   CConfig::Impl::undo( $impl_name, -CmdInterface => "csiqa-3070-35", cmd => modify,
#                          vserver => 'vs0', volume => 'vol1', size => '50MB' );
#
# Parameters:
#       impl_name     - Impl class name (i.e. 'CConfig::Impl::Volume', 'CConfig::Impl::Aggr', etc...)
#       -CmdInterface - Name of node to use as the command-interface
#       -cmd          - NACL Component command ("create", "modify", etc...)
#       -subtest      - Boolean, subtest undo action, 0|1
#
# Return:
#
# Assumptions:
# -----------------------------------------------
sub undo {
    my $class_or_obj=shift;
    my $implclassref = ref($class_or_obj);
    if ( !$implclassref ) {
        $implclassref = $class_or_obj;
    }
    my $object = CConfig::Impl->impl2object( $implclassref );

    my %opt = ( '-CmdInterface'      => undef,
                '-component_name'    => undef,
                '-cmd'               => undef,
                '-subtest'           => CConfig::Subtest::SUBTEST_MIN( object => $object ),
                '-subtest_at_a_time' => undef,
                @_ );

    $opt{command_interface} = CConfig::Impl::find_node_component( name => $opt{'-CmdInterface'});
    delete $opt{'-CmdInterface'};

    my $cmd        = $opt{'-cmd'};
    my $compname   = delete $opt{'-component_name'};
    my $do_subtest = delete $opt{'-subtest'};
    my $at_a_time  = delete $opt{'-subtest_at_a_time'};

    if ( !$compname ) {
        $compname = $implclassref->nacl_component_classname();
    }

    # We currently do not support a VolumeFile Impl, but it won't hurt
    # anything
    if ( $_CCL_WORKAROUND_BURT650568 ) {
        if ( ($compname =~ /^NACL::C::(Vserver|Volume|Lun|VolumeFile)$/) &&
             (exists $opt{'qos-policy-group'} && 
                ($opt{'qos-policy-group'} =~ /-/)) ) {
    
            # BURT 650568, Vserver, Volume, Lun and File return "-" for
            # qos-policy-group via show, but do not allow "-" via modify
            $Log->comment("CCL_WORKAROUND_BURT650568=1, so for $compname attribute \"qos-policy-group\", \"-\" will be converted to \"none\"");
            $opt{'qos-policy-group'} = 'none';
        }
    }

    my $allow_zapi  = $implclassref->allow_zapi();
    my $prefer_zapi = $implclassref->PREFER_ZAPI();
    if ( !$allow_zapi && $prefer_zapi ) {
        $opt{'apiset_must'} = { interface => 'CLI' };
        if ( exists $opt{'apiset_should'} ) {
            delete $opt{'apiset_should'};
        }
    }

    # If specified as part of a rule, option will a scalar
    if ( exists $opt{'apiset_must'} && !ref($opt{'apiset_must'}) ) {
        $opt{'apiset_must'} = { interface => $opt{'apiset_must'} };
    }
    if ( exists $opt{'apiset_should'} && !ref($opt{'apiset_should'}) ) {
        $opt{'apiset_should'} = { interface => $opt{'apiset_should'} };
    }

    # See if there is a implmentation specific do_undo() or should we use
    # the generic CConfig::Impl::do_undo() ?
    my $methodname = 'CConfig::Impl::do_undo';
    my $symb = \%CConfig::Impl::;

    OUTER: foreach my $key (sort keys %$symb ) {
        if ( 'CConfig::Impl::'.$key eq $implclassref.'::' ) {
            foreach my $subkey (sort keys %{$symb->{$key}} ) {
                if ( $subkey =~ /do_undo/ ) {
                    $methodname = $implclassref.'::do_undo';
                    last OUTER;
                }
            }
        }
    }

    if ( $do_subtest ) {
        my %subtest_opts;
        if ( defined $at_a_time ) {
            $subtest_opts{at_a_time} = $at_a_time;
        }
        CConfig::Subtest::subtest( method => $methodname, class => $compname, args => \%opt, %subtest_opts );
    } else {
        my $libname;
        if ( $methodname =~ /(\S+)::do_undo/ ) {
            $libname = $1;
            if ( !defined $CConfig::LOADED{$libname} ) {
                eval "use $libname";
                $CConfig::LOADED{$libname} = 1;
            }
            $compname->$methodname( %opt );
        }
    }
}

# -----------------------------------------------
# Function: do_undo
#   Default undo routine which just calls the -cmd method on the NACL
#   component.  Impls can override this if they need to do something
#   more involved.  (See Impl/Volume for an example)
#
# Usage:
#   do_undo( 'NACL::C::Volume', '-cmd' => 'modify', %opt );
#
# -----------------------------------------------
sub do_undo {
    my $class = shift;
    my $me=(caller(0))[3];
    my %opt = ( 'command_interface' => undef,
                '-cmd'              => undef,
                @_ );
    my $cmd = delete $opt{'-cmd'};

    if ( !defined $CConfig::LOADED{$class} ) {
        eval "use $class";
        $CConfig::LOADED{$class} = 1;
    }

    $Log->debug( sub {"$me: ".$class."->".$cmd."( ".NACL::ComponentUtils::Dumper( \%opt )." )"} );
    $class->$cmd( %opt );
}

# -----------------------------------------------
# Function: get_required_keys
#
#   Get hash keys required by implementation
# 
# Usage:
#
#   $obj->get_required_keys( action => 'create' );
#
# -----------------------------------------------
sub get_required_keys {
    my $me=(caller(0))[3];
    my $calledby=(caller(1))[3];
    my $class_or_obj=shift;
    my %opt = ( action            => undef,
                @_ );

    if ( !exists $opt{action} ) {
        die "$me ERROR: must call with action option";
    }

    if ( !%CConfig::CDEF_OPTIONS ) {
        build_cdef_options();
    }

    my $implclassref = ref($class_or_obj);
    if ( !$implclassref ) {
        $implclassref = $class_or_obj;
    }

    my $action      = $opt{action};
    my $compname    = $implclassref->nacl_component_classname();

    # Currently the assumption is that this routine will be called with
    # actions which directly map to a command, or that we can figure out the
    # mapping.
    my $cdef_root = $implclassref->cdef_root( component_name => $compname );
    my $full_cmd = $cdef_root."_".$action;

    if ( $full_cmd =~ /fetch$/ ) {
        $full_cmd =~ s/fetch/show/;
    }
    if ( $full_cmd eq 'vserver_cifs_domain_preferred_dc_create' ) {
        $full_cmd =~ s/create/add/;
    }
    $full_cmd =~ s/-/_/g;

    my @required;
    if ( exists $CConfig::CDEF_OPTIONS{$full_cmd}{REQUIRED} ) {
         @required = @{$CConfig::CDEF_OPTIONS{$full_cmd}{REQUIRED}};
    }

    return @required;
}

# -----------------------------------------------
# Function: gen_undo_commands
#
#   Generate the undo commands to be written to 
#   the undo file, from the UNDO_LIST
#   populated during action execution.
#   The commands are pushed on to the TXMGR stack.
#
#   This method is called from the post() routine of each
#   specific Impl object that creates/modifies.
# 
# Usage:
#
#   $obj->gen_undo_commands();
#
# -----------------------------------------------
sub gen_undo_commands {
    my $me=(caller(0))[3];
    my $calledby=(caller(1))[3];
    my $self=shift;
    my %opt = ( @_ );
    my $undo_timeout = 120;


    my $implclassref = ref($self);

    foreach my $r (@{$self->{UNDO_LIST}}) {
        my $compname = $r->{component_name};
        my $undo_cmd;
        # VserverCifsDomainPreferredDc use add/remove instead of create/delete
        if (  ($r->{cmd} =~ /^delete$/i) ||
             (($r->{cmd} =~ /^remove$/i) && ($compname =~ /VserverCifsDomainPreferredDc/)) ) {
            my @primary_keys = $compname->get_primary_keys();
            $undo_cmd = 'CConfig::Impl::purge("'.$implclassref.'",';
        } else {
            $undo_cmd = 'CConfig::Impl::undo("'.$implclassref.'",'.
                                                '-cmd=>"'.$r->{cmd}.'"'.',';
        }

        $undo_cmd .= '-CmdInterface=>"' .$self->{CMD_INT}->{node} .'"'.','.
                     '-component_name=>"'.$compname.'",';

        if ( defined $r->{subtest} ) {
            $undo_cmd .= '-subtest=>'.$r->{subtest}.',';

            if ( defined $r->{subtest_at_a_time} ) {
                $undo_cmd .= '-subtest_at_a_time=>'.$r->{subtest_at_a_time}.',';
            }
        }

        foreach my $field (keys %{$r->{args}}) {
            # When value is like $lif = '\"\"';
            if ( $r->{args}->{$field} && ($r->{args}->{$field} =~ /\"\"/) ) {
                $undo_cmd .= '"'.$field.'"=>"",';
            # Convert an ARRAY ref into an externalizable ARRAY specification
            } elsif ( ref($r->{args}->{$field}) =~ /ARRAY/i ) {
                $undo_cmd .= '"'.$field.'"=>[';
                foreach my $item (@{$r->{args}->{$field}}) {
                    $undo_cmd .= ' "'.$item.'",';
                }
                $undo_cmd .= ' ],';
            } else {
                if ( $field ne 'method-timeout' ) {
                    $undo_cmd .= '"'.$field.'"=>"'.$r->{args}->{$field}.'",';
                } else {
                    if ( $r->{args}->{$field} > $undo_timeout ) {
                        $undo_timeout = $r->{args}->{$field};
                    }
                }
            }
        }
        $undo_cmd .= '"method-timeout"=>' .$undo_timeout;
        $undo_cmd .= ')';
        $CConfig::TXMGR->push( $undo_cmd );
    }
}


# -----------------------------------------------
# Function: add_undo
#
#   Add the data necessary to the Impl UNDO_LIST array to allow
#   rollback of any given change.  The undo process is done in
#   2 stages,
#       1) Each Impl calls add_undo() right before the "do" action
#       2) In each post() method, gen_undo_commands() will be called
#          to actually generate the commands necessary to rollback.
#
#  There are a few issues that need to be addressed when you call
#  add_undo.  
#
#  1) First, since arguments specified for create (for example)
#  might not all be valid for it's undo counterpart (in this case delete)
#  we filter out the "do" options that are not valid for the associated
#  "undo" command.  However, since NACL doesn't make it easy to 
#  distinguish between NACL options (say "username" or "password" for 
#  VserverCifs) and filer options (like "vserver"), if we filter out
#  all of the options based on what the CDEFs allow for the command,
#  we'd end up filtering out those NACL options.  So the
#  "do_not_filter_out" option handles these cases.
#
#  2) Some "undo" arguments are missing from the "do" command
#  entirely, so for that case, use "add_options" (like force => true
#  for NACL::C:Aggregate::delete)
#
#  3) Finally, some CS objects like NACL::CS::AntivirusOnAccessPolicy
#  have a bug where a field (in this case "protocols", should be
#  defined as an array, but is instead defined as a scalar.  In this
#  case, the CS object fetches "protocols" as an arrayref, but the C
#  objects balk at using an array for protocols.  So "check_type"
#  can be used to work-around these type of NACL inconsistencies until
#  they are fixed.
#
# 
# Usage:
#
#   $obj->add_undo( component_name => 'NACL::C::Volume',
#                             cmd => 'modify', args => \%args );
#
# -----------------------------------------------
sub add_undo {
    my $me=(caller(0))[3];
    my $calledby=(caller(1))[3];
    my $self=shift;
    my $object_type = ref($self);
    my %opt = ( cmd               => undef,
                component_name    => undef,
                args              => undef,
                do_not_filter_out => undef,
                add_options       => undef,
                filter_out        => undef,
                check_type        => undef,
                subtest           => undef,
                @_ );

    if ( !exists $opt{cmd} ) {
        die "$me ERROR: must call with cmd option";
    }

    my $args = $opt{args};
    my $implclassref = ref($self);

    my $subtest_at_a_time;

    my $cmd   = $opt{cmd};
    my $compname = $opt{component_name};
    if ( !$compname ) {
        $compname = $implclassref->nacl_component_classname();
    }

    my %cmd_options = ();
    my $allow_zapi  = $implclassref->allow_zapi();
    my $prefer_zapi = $implclassref->PREFER_ZAPI();
    if ( !$allow_zapi && $prefer_zapi ) {
        $cmd_options{'apiset_must'} = { interface => 'CLI' };
    }

    my $undo_args = Storable::dclone($args);

    if ( exists $undo_args->{subtest_pacing} ) {
        $subtest_at_a_time = $undo_args->{subtest_pacing};

        if ( $subtest_at_a_time >= 1 ) {
            $opt{subtest}  = 1;
        } else {
            $opt{subtest}  = 0;
        }

        delete $undo_args->{subtest_pacing};
    }

    # To undo a modify or delete ACTION, we have to query the current object state, 
    # and use those values when reverting/re-creating in the undo.
    # For action=modify, the undo command (cmd option passed in) will ALSO be modify, 
    # for action=delete, the undo command will be create
    if ( $cmd =~ /^(modify|create)$/i ) {
        if ( !defined $CConfig::LOADED{$compname} ) {
            eval "use $compname";
            $CConfig::LOADED{$compname} = 1;
        }
        my $csname = c2cs($compname);
    
        # extract the keys for the fetch filter
        my @primary_keys = $compname->get_primary_keys();

        my %filter;
        my %filter_arg;
        foreach my $key (@primary_keys) {
            $filter{$key} = $undo_args->{$key};
        }

        if ( %filter ) {
            $filter_arg{filter} = { %filter };
        }

        my $retrieved_values = $csname->fetch( command_interface => $self->CMD_INT, 
                                          %filter_arg,
                                          allow_empty => 1,
                                          %cmd_options );

        if ( exists $retrieved_values->{command_interface} ) {
            delete $retrieved_values->{command_interface};
        }

        my @fields_to_set;
        if ( $cmd =~ /^modify$/i ) {
            @fields_to_set = keys %$undo_args;
        } else {
            # When undoing a delete action (i.e. when cmd=create), we want to gather 
            # the current values corresponding to ALL of object's CS fields
            # The get_attributes() method will return all of the show attribute 
            # types (scalar, array, etc...) keyed by attribute name
            # We're only concerned with the keys (i.e. the attribute names)
            my $href = $csname->get_attributes();

            # Attribute names returned as keys of get_attributes() have all of their
            # "-" converted into "_", but the CDEFS use the "-" in their names (just
            # like the Ontap CLI) so we have to swap them back.
            # We also need them swapped back to "-" because the components ALSO use
            # "-" instead of "_" when referencing attribute names.
            @fields_to_set = map { $_ =~ s/_/-/; $_ } (keys %$href);
        }

        foreach my $field (@fields_to_set) {
            # $field name may have dashes in them
            if ( $retrieved_values && $retrieved_values->can( $field ) ) {
                my $undo_arg_value = $undo_args->{$field};
                my $retrieved_value = $retrieved_values->$field();

                $undo_args->{$field} = $retrieved_value;
                if ( ref($undo_args->{$field}) =~ /ARRAY/ ) {
                    # If this is an array, there are times where NACL/ZAPI will 
                    # contain an array of single key hashes.  Normally I would not
                    # try to fix this sort of thing, but in this case it seems
                    # harmless
                    my @arr;
                    foreach my $item (@{$undo_args->{$field}}) {
                        if ( !ref($item) ) {
                            push(@arr, $item);
                        } elsif ( ref($item) =~ /HASH/ ) {
                            if ( scalar(keys %{$item}) == 1 ) {
                                $Log->warn("$me: $object_type has field $field as ".Dumper($item)." but it should be a list of scalars, raise a burt against NACL for the associated NACL CS object");
                                foreach my $key (keys %{$item}) {
                                    push(@arr, $item->{$key});
                                }
                            } else {
                                $Log->warn("$me: $object_type has field $field as ".Dumper($item)." but it should be a list of scalars, raise a burt against NACL for the associated NACL CS object");
                            }
                        }
                    }
                    if ( @arr ) {
                        $undo_args->{$field} = \@arr;
                    }
                }

                # If we are undoing a delete (i.e. the undo command is 'create')
                # and the field is '-' this means "default value" so there is
                # no need to pass it.  Also, there are cases there on create
                # you can only give a subset of all possible values (i.e.
                # you can pass netmask but not netmask-length) so it
                # is really better NOT to pass parameters which only 
                # stand for defaults.
                #
                # Note, for undo of a modify (undo cmd will also be modify)
                # we can't filter out defaults because we may WANT to restore
                # a field to it's default setting
                if ( $cmd =~ /^create$/i ) {
                    if ( $retrieved_value && ($retrieved_value eq '-') ) {
                            delete $undo_args->{$field};
                    }
                }

                # Saw an odd case for "volume efficiency show" where Ontap was tacking on an 
                # asterisk on the end of the policy name, but when you attempt to use that
                # name in a modify/delete/etc...  Ontap complains.  It might be an indicator of
                # some sort, need to look into it
                if ( $undo_arg_value && ($retrieved_value eq $undo_arg_value.'*') ) {
                    $undo_args->{$field} = $undo_arg_value;
                }

                # in case field has unescaped quotes, escape them
                if ( !(defined $undo_args->{$field}) ) {
                    delete $undo_args->{$field};
                } else {
                    # in case field has unescaped quotes, escape them
                    if ( $undo_args->{$field} =~ /\"/ ) {
                        $undo_args->{$field} =~ s/\"/\\"/g;
                    }
                    # in case field has unescaped $, escape them
                    if ( $undo_args->{$field} =~ /\$/ ) {
                        $undo_args->{$field} =~ s/\$/\\\$/g;
                    }
                }
            }
        }

        # Strictly as a hack until C and CS treat all fields consistently.
        # For example, as of 6/1/2011, AntiVirusOnAccessPolicy returns "protocols" as
        # an ARRAYREF, but defines it as a scalar in the AntiVirusOnAccessPolicy.pm class
        if ( $opt{check_type} && (ref($opt{check_type}) =~ /ARRAY/) ) {
            foreach my $field (@{$opt{check_type}}) {
                if ( $retrieved_values && (ref($retrieved_values->$field()) =~ /ARRAY/) ) {
                    my $value = $retrieved_values->$field();
                    my $reset_field = $field. '_reset';
                    $retrieved_values->$reset_field();

                    # Means that field is defined as a scalar, but was set to an array
                    # by ComponentState.pm.  Once reset, the field should revert to the
                    # type defined in the class.
                    if ( !ref($retrieved_values->$field()) ) {
                        $Log->warn( "$me - ComponentState overrode the type of field $field in object $csname, attempting to convert to SCALAR");
                        $undo_args->{$field} = join(",", @$value);
                    }
                }
            }
        }
    }

    # undo cmd may have fewer options than "do" cmd
    # example: Impl::Volume efficiency_stop has fewer options
    #  than    Impl::Volume efficiency_start
    $self->filter_out_invalid( component_name => $compname, cmd=> $cmd, args => $undo_args );

    # Need to pass in allow_extra => 1 because I've seen components complain about
    # synonym arguments (like vif/lif) on Interface create.  Since some of these
    # components are hand-written, it is hard to always be certain that the CDEFS
    # perfectly match up with the component.  And by providing allow_extra, what is
    # the worst that will happen?  Someone passes an option that the cdefs permits
    # but the component ignores.  Seems preferable to the converse (i.e. dying when
    # we pass an option that the component doesn't care about...)

    $undo_args->{allow_extra} = 1;

    # Some commands like Impl::VserverCifs delete require options
    # (username and password), which aren't cdef optional options.
    if ( $opt{do_not_filter_out} && (ref($opt{do_not_filter_out}) =~ /ARRAY/) ) {
        foreach my $field (@{$opt{do_not_filter_out}}) {
            $undo_args->{$field} = $args->{$field};
        }
    }

    # Some commands like (Impl::NetInterface create) The CDEFS
    # may allow netmask and netmask-length, but you can't specify
    # both of them at the same time
    if ( $opt{filter_out} && (ref($opt{filter_out}) =~ /ARRAY/) ) {
        foreach my $field (@{$opt{filter_out}}) {
            if ( exists $undo_args->{$field} ) { 
                delete $undo_args->{$field};
            }
        }
    }

    # You may need to add additional options to the undo, like force => true for aggregate
    # purge which are not present/required on the create
    if ( $opt{add_options} && (ref($opt{add_options}) =~ /HASH/) ) {
        foreach my $field (keys %{$opt{add_options}}) {
            $undo_args->{$field} = $opt{add_options}->{$field};
        }
    }

    my $undo_item = { component_name => $compname, cmd => $cmd, args => $undo_args, subtest => $opt{subtest} };
    if ( $opt{subtest} && defined $subtest_at_a_time ) {
        $undo_item->{subtest_at_a_time} = $subtest_at_a_time;
    }
    push( @{$self->{UNDO_LIST}}, $undo_item );
#print "FWR undo_args = ".NACL::ComponentUtils::Dumper($undo_args)."\n";
#print "FWR UNDO_LIST = ".NACL::ComponentUtils::Dumper($self->{UNDO_LIST})."\n";
}

# -----------------------------------------------
# Function: filter_out_invalid
#
#   Someday hopefully NACL components will allow us to 
#   ignore extra arguments, but for now I have to filter them
#   out or commands will fail.
# 
# Usage:
#
#   $obj->filter_out_invalid( component_name => 'NACL::C::Volume',
#                             cmd => 'create', args => \%args );
#
#   $obj->filter_out_invalid( full_cmd => 'volume_create',
#                             args => \%args );
#
# -----------------------------------------------
sub filter_out_invalid {
    my $me=(caller(0))[3];
    my $calledby=(caller(1))[3];
    my $class_or_obj=shift;
    my %opt = ( cmd            => undef,
                full_cmd       => undef,
                component_name => undef,
                cdef_root      => undef,
                args           => undef,
                @_ );

    if ( ! ((exists $opt{cmd}) || (exists $opt{full_cmd})) ) {
        die "$me ERROR: must call with cmd or full_cmd options";
    }

    my $args = $opt{args};

    my $implclassref = ref($class_or_obj);
    if ( !$implclassref ) {
        $implclassref = $class_or_obj;
    }

    my $cmd      = $opt{cmd};
    my $full_cmd = $opt{full_cmd};
    if ( !$full_cmd ) {
        my $compname = $opt{component_name};
        if ( !$compname ) {
            $compname = $implclassref->nacl_component_classname();
        }
    
        my $cdef_root = $opt{cdef_root};
        if ( !$cdef_root ) {
            $cdef_root = $implclassref->cdef_root( component_name => $compname );
        }

        $full_cmd = $cdef_root."_".$cmd;
    }
    
    if ( !%CConfig::CDEF_OPTIONS ) {
        build_cdef_options();
    }

    # Check to see if the argument is valid
    my @common_args = qw( command_interface method-timeout apiset_should apiset_must subtest_pacing -let_ontap_filter );
    foreach my $field (keys %$args) {
        if ( !grep( /^$field$/, @common_args ) ) {

            my $valid = is_valid_field( full_cmd => $full_cmd, field =>  $field );

            if ( !$valid ) {
                delete $$args{$field};
            }
        }
    }
}

# -----------------------------------------------
# Function: is_valid_field
#
#   Is this field valid for this cdef command?
# 
# Usage:
#
#   my $is_valid = $obj->is_valid_field( full_cmd => 'volume_create', field => 'junk' );
#
# -----------------------------------------------
sub is_valid_field {
    my $me=(caller(0))[3];
    my %opt = ( full_cmd => undef,
                field => undef,
                @_ );

    if ( ! ((exists $opt{full_cmd}) && (exists $opt{field})) ) {
        die "$me ERROR: must call with full_cmd and field options";
    }

    my $full_cmd  = $opt{full_cmd};
    my $field     = $opt{field};

    if ( $full_cmd =~ /fetch$/ ) {
        $full_cmd =~ s/fetch/show/;
    }

    $full_cmd =~ s/-/_/g;

    # NACL fields generally are converted from - to _ so we have to swap back here
    # because the CDEFS will have the names with -
    my $converted_field = $field;
       $converted_field =~ s/_/-/g;

    if ( exists $CConfig::CDEF_OPTIONS{$full_cmd}{ALL} ) {
        if ( grep( /^($field|$converted_field)$/, @{$CConfig::CDEF_OPTIONS{$full_cmd}{ALL}} ) ) {
            return 1;
        }
    } elsif ( $full_cmd eq 'vserver_cifs_domain_preferred_dc_create' ) {
        $full_cmd = 'vserver_cifs_domain_preferred_dc_add';
        if ( exists $CConfig::CDEF_OPTIONS{$full_cmd}{ALL} ) {
            if ( grep( /^($field|$converted_field)$/, @{$CConfig::CDEF_OPTIONS{$full_cmd}{ALL}} ) ) {
                return 1;
            }
        }
    }

    return 0;
}

# -----------------------------------------------
# Function: build_cdef_options
#
#   Build CDEF options hash so we can filter out invalid
#   arguments.  Once NACL allows us to pass in extra arguments
#   and have them ignored, then we can strip out all of this.
# 
# Usage:
#
#   build_cdef_options();
#
# -----------------------------------------------
sub build_cdef_options {
    my $me=(caller(0))[3];

    $SIG{'__WARN__'} = sub { $CConfig::WARNING = $_[0] };

    my $command_interface = $CConfig::Nodes[0];

    my $cli_apiset = $command_interface->apiset(category => "Node", interface => "CLI", set => "CMode");


    my $cdef_cmds = $cli_apiset->{schema}->{commands};

    # Newer NACL APISets do not store the CDEF commands in the APISet object
    # directly
    if ( !$cdef_cmds ) {
        $cdef_cmds = $cli_apiset->get_schema_obj()->get_all_cdef_objects();
    }

    foreach my $cmd (keys %$cdef_cmds) {
        my $file = $cdef_cmds->{$cmd}->{file};

        my %CommandDefinitions;
        my $content = NATE::Inc::get_file($file)
             or NATE::BaseException->throw("Error reading file '$file': $@");

         $CConfig::WARNING = 0;
         eval $content;
         NATE::BaseException->throw(
             "Could not load CDEF file : $file\nReason: $@\n")
             if $@;

         if ( $CConfig::WARNING ) {
             $Log->warn("CDEF file $file had warnings,\n    ".$CConfig::WARNING);
             my @content = split( "\n", $content );
             my $linenum    = 1;
             my $badlinenum = 1;
             if ( $CConfig::WARNING =~ /at .* line (\d+)\.$/ ) {
                 $badlinenum = $1;
             }
             while ( $linenum <= $badlinenum ) {
                 my $line = $content[$linenum-1];
                 print "LINENUM:".$linenum." ".$line."\n";
                 $linenum++;
             }
             $CConfig::WARNING = 0;
         }

         foreach my $cdef ( keys %CommandDefinitions ) {
             my @options;
             if ( $CommandDefinitions{$cdef}{required_options} ) {
                 $CConfig::CDEF_OPTIONS{$cdef}{REQUIRED} = [@{$CommandDefinitions{$cdef}{required_options}}];
                 push( @options, @{$CommandDefinitions{$cdef}{required_options}} );
             }
             if ( $CommandDefinitions{$cdef}{optional_options} ) {
                 push( @options, @{$CommandDefinitions{$cdef}{optional_options}} );
             }
             $CConfig::CDEF_OPTIONS{$cdef}{ALL} = [@options];
         }
    }
}

# -----------------------------------------------
# Function: call_nacl_component_routine
#   Call NACL Component routine
#
# Usage:
#   $obj->call_nacl_component_routine( subtest => 0|1, routine => "modify", args => \%args );
#
# Parameters:
#       obj               - Impl object instance (i.e. CConfig::Impl::Volume, CConfig::Impl::Aggr, etc...)
#       component_name    - NACL::C class name
#       subtest           - Execute this as a subtest? (0|1) default: 0
#       subtest_at_a_time - How many jobs to subtest at a time? default: undef
#       routine           - Routine to execute
#       args              - reference to the hash containing the args to pass
#                           to the specific ComponentState::create command
#
# Return:
#
# Assumptions:
#    Note: It should be possible to specify a simulation mode where
#          instead of calling a NACL routine, we would instead call
#          a filer simulator that works off the CDEFs
# -----------------------------------------------
sub call_nacl_component_routine {
    my $self=shift;
    my $me=(caller(0))[3];
    my $caller=(caller(1))[3];
    my %args = ();

    my %opt = (
        args    => undef,
        component_name   => undef,
        routine => undef,
        subtest => 0,
        subtest_at_a_time => undef,
        filter_out_invalid => 1,
        @_
    );

    if ( $opt{args} ) {
        %args = %{Storable::dclone($opt{args})};
    }

    $args{command_interface} = $self->CMD_INT;

    my $implclassref = ref($self);
    if ( !$implclassref ) {
        $implclassref = $self;
    }

    my $comp_name  = $opt{component_name};
    my $routine    = $opt{routine};
    my $subtest    = $opt{subtest};
    my $at_a_time  = $opt{subtest_at_a_time};
    my $filter_out = $opt{filter_out_invalid};  # Specific Impl might want to do this themselves

    if ( !$at_a_time && (exists $self->{subtest_at_a_time} ) ) {
        $at_a_time = $self->{subtest_at_a_time};
    }

    my $let_ontap_filter=1;
    my $let_ontap_filter_from_rule=0;
    if ( exists $args{filter}{'-let_ontap_filter'} ) {
        $let_ontap_filter=$args{filter}{'-let_ontap_filter'};
        $let_ontap_filter_from_rule=1;
        delete $args{filter}{'-let_ontap_filter'};
    }

    # Generally the NACL Component layer frowns on arguments passed in that are
    # unrecognized.  The "allow_extra" option will eventually allow unknown options
    # to be passed in but ignored.  This is necessary for subactions, as they inherit
    # all of their parent's attributes, but may not use all of them.  We can't just
    # limit what a subaction inherits, because an attribute that is not used by
    # the child subaction, might be required by a grandchild subaction.

    my $allow_zapi  = $implclassref->allow_zapi();
    my $prefer_zapi = $implclassref->PREFER_ZAPI();
    if ( !$CConfig::SIMMODE ) {
        if ( !$comp_name ) {
            $comp_name = $implclassref->nacl_component_classname();
        }
        if ( !$comp_name ) {
            croak "ERROR $me: $implclassref must provide a nacl_component_classname() method or specify \'component_name\' option";
        }

        if ( $routine eq 'fetch' ) {
           $comp_name = c2cs($comp_name);
           # Server accepted -Timeout along with the other arguments, but NACL::CS::fetch takes
           # method-timeout outside of the filter arguments.
           if ( exists $args{filter}{'method-timeout'} ) {
               $args{'method-timeout'} = $args{filter}{'method-timeout'};
               delete $args{filter}{'method-timeout'};
           }

           if ( exists $args{filter}{'apiset_should'} ) {
               $args{'apiset_should'} = { interface => $args{filter}{'apiset_should'} };
               if ( $args{filter}{'apiset_should'} =~ /ZAPI/i ) {
                   if ( !$allow_zapi ) {
                       $args{'apiset_must'} = { interface => 'CLI' };
                       delete $args{'apiset_should'};
                   }
               }
               delete $args{filter}{'apiset_should'};
           }
           if ( exists $args{filter}{'apiset_must'} ) {
               $args{'apiset_must'} = { interface => $args{filter}{'apiset_must'} };
               if ( $args{filter}{'apiset_must'} =~ /ZAPI/i ) {
                   if ( !$allow_zapi ) {
                       $args{'apiset_must'} = { interface => 'CLI' };
                       delete $args{'apiset_should'};
                   }
               }
               delete $args{filter}{'apiset_must'};
           }

           if ( !$allow_zapi && $prefer_zapi ) {
               $args{'apiset_must'} = { interface => 'CLI' };
               if ( exists $args{'apiset_should'} ) {
                   delete $args{'apiset_should'};
               }
           }

           if ( exists $args{filter}{'subtest_pacing'} ) {
               delete $args{filter}{'subtest_pacing'};
           }

           $args{allow_empty} = 1;

           # When both C and CS support the allow_extra option, we should be able to remove the calls to 
           # filter_out_invalid() here, however, do not remove the filter_out_invalid() method, as it could
           # be used in simulation mode
           #     $args{allow_extra} = 1;
           # or possibly 
           #     $args{filter}{allow_extra} = 1;
           if ( $filter_out ) {
               filter_out_invalid( $implclassref, component_name => $comp_name, cmd=> $routine, args => \%{$args{filter}} );
           }

           $subtest = 0;
        } else {
           if ( exists $args{'apiset_should'} ) {
               my $temp = { interface => $args{'apiset_should'} };
               if ( $args{'apiset_should'} =~ /ZAPI/i ) {
                   if ( !$allow_zapi ) {
                       $args{'apiset_must'} = 'CLI';
                       delete $args{'apiset_should'};
                   }
               }
               if ( exists $args{'apiset_should'} ) {
                   $args{'apiset_should'} = $temp;
               }
           }
           if ( exists $args{'apiset_must'} ) {
               my $temp = { interface => $args{'apiset_must'} };
               if ( $args{'apiset_must'} =~ /ZAPI/i ) {
                   if ( !$allow_zapi ) {
                       $temp = { interface => 'CLI' };
                   }
               }
               $args{'apiset_must'} = $temp;

               # If we have set apiset_must and apiset_should is set, 
               # delete apiset_should
               if ( exists $args{'apiset_should'} ) {
                   delete $args{'apiset_should'};
               }
           }

           # Could subtest_pacing be getting filtered out of the fetch and then it
           # would be missing from the create?
           if ( exists $args{'subtest_pacing'} ) {
               $at_a_time = $args{'subtest_pacing'};
               if ( $at_a_time >= 1 ) {
                   $subtest = 1; 
               } else {
                   $subtest = 0; 
               }

               delete $args{'subtest_pacing'};
           }

           if ( !$allow_zapi && $prefer_zapi ) {
               $args{'apiset_must'} = { interface => 'CLI' };
               if ( exists $args{'apiset_should'} ) {
                   delete $args{'apiset_should'};
               }
           }

           if ( $filter_out ) {
               filter_out_invalid( $implclassref, component_name => $comp_name, cmd=> $routine, args => \%args );
           }
        }

        my $methodname = $comp_name."::".$opt{routine};
        if ( !defined $CConfig::LOADED{$comp_name} ) {
            eval "use $comp_name";
            $CConfig::LOADED{$comp_name} = 1;
        }
        if ( $subtest ) {
            if ( exists $args{filter} && 
                 (ref($args{filter}) =~ /HASH/) &&
                 scalar(keys %{$args{filter}} == 0 ) ) {
                delete $args{filter}; 
            }

            $Log->debug( sub {"$me: bg at_a_time = $at_a_time, $methodname( ".NACL::ComponentUtils::Dumper( \%args )." )"} );
            CConfig::Subtest::subtest( method => $methodname, class => $comp_name, at_a_time => $at_a_time, args => \%args );
        } else {
            my @ret;
            my $job;
            # If they don't want to subtest, but command is one which allows a foreground flag,
            # handle this with a job component
            if ( (exists $args{foreground}) && ( $args{foreground} =~ /false/i) ) {
                $args{job_component} = \$job;
            }

            # If we have a ton of filter args (in the thousands)
            # this can cause problems sending such a big command to the filer
            # so we will just send a generic show and then post filter later
            # when we let the filer filter and when we post filter might have
            # to be determined by the length of the filter argument.
            my %filter;
            my $collapsed = 0;
            if ( $routine eq 'fetch' ) {
                my $num_filter_keys = 0;
                my $length_filter_keys = 0;
                foreach my $key (keys %{$args{filter}}) {
                    if ( ref($args{filter}{$key}) =~ /ARRAY/i ) {
                        # When does this happen?
                        $filter{$key} = $args{filter}{$key};
                    } else {
                        my @vals;
                        if ( $args{filter}{$key} =~ /\|/ ) {
                            @vals = split('\|', $args{filter}{$key});
                        } elsif ( $args{filter}{$key} =~ /,/ ) {
                            @vals = split(',', $args{filter}{$key});
                        } else {
                            @vals = ($args{filter}{$key});
                        }
                        # collapse vs0|vs0|vs0|, etc...
                        my %vals = map { $_ => 1 } @vals;
                        my @keys = keys %vals;
                        if ( scalar(@keys) != scalar(@vals) ) {
                            $collapsed = 1;
                        }
                        $num_filter_keys += scalar(@keys);
                        $filter{$key} = join('|', @keys);
                        $length_filter_keys += length( $filter{$key} );

                        # Merge note: lines other than main might still have the issue of
                        # burt 582131 related to NACL::CS::ComponentState->_apply_filter()
                        # not handling '/' (like in file path delimiters)
                    }
                }

                # replace filter with collapsed filter
                if ( %filter && $args{filter} && $collapsed ) {
                    $args{filter} = Storable::dclone(\%filter);
                }

                # Maybe if filter is fairly large, default $let_ontap_filter to 0?
                # Do we want to expose the definition of "fairly large?" to the user?
                # Probably do not need to as the user can always specify where they want
                # filtering to occur (ontap vs in the library)

                # Maybe instead of 50, we want to consider how many characters this is?
                # the only problem with that is they we have to do string lengths of every
                # parameter and that would probably bog things down

                if ( !$let_ontap_filter_from_rule && 
                      $let_ontap_filter && 
                     (($num_filter_keys > 50) || ($length_filter_keys > $_MAX_ONTAP_LINE_LENGTH) ) ) {
                    $let_ontap_filter = 0;
                }


                if ( %filter && !$let_ontap_filter ) {
                    # we will post-filter
                    delete $args{filter};

                    # do we need to increase method-timeout if we aren't going to filter on the filer?
                    if ( $args{'method-timeout'} ) {
                        $args{'method-timeout'} *= 2;
                    }
                }
            }

            if ( exists $args{filter} && 
                 (ref($args{filter}) =~ /HASH/) &&
                 scalar(keys %{$args{filter}} == 0 ) ) {
                delete $args{filter}; 
            }
            $Log->debug( sub {"$me: fg $methodname( ".NACL::ComponentUtils::Dumper( \%args )." )"} );
            eval { @ret = $comp_name->$methodname( %args ); };
            my $rc = $@;

            if ( !$rc ) {
                if ( $routine eq 'fetch' ) {
                    if ( %filter && !$let_ontap_filter ) {
#print "NACL::CS::ComponentState->_apply_filter filter = ".Dumper(\%filter)."\n";
                        NACL::CS::ComponentState->_apply_filter(
                            filter     => \%filter,
                            state_objs => \@ret
                        );
                    }
                }
            }

            # Ontap declares ERRORs for some things which aren't really errors
            # This is a case where the foreground command, was backgrounded by Ontap
            # because it was taking too long.  In this case, wait for the command to 
            # complete.
            # Note: Eventually if this changes (i.e. if backgrounded actions don't throw
            # exceptions) we can always pass in '$args{job_component} = \$job, $args{allow_extra} => 1'
            # or see if we can figure out if job_component is an allowable parameter for the
            # component
            if ( $rc =~ /Job\s+(\d+)\s+(is running in the background|has been queued)/ ) {
                my $jobid = $1;

                my $result = NATE::Result::Warn->new( $rc );
                $result->misc($rc);
                $result->log();

                push(@CConfig::BG_JOBS,$jobid);

                # wait for background jobs to complete
                 if (@CConfig::BG_JOBS) {
                     $Log->trace("Waiting for jobs ".join(',',@CConfig::BG_JOBS));
                     while(CConfig::Job::are_jobs_active(\@CConfig::BG_JOBS)) { sleep 10; }
                 }
     
                 # check bg jobs
                 if (@CConfig::BG_JOBS) { CConfig::Job::check_jobs(\@CConfig::BG_JOBS); }
                 @CConfig::BG_JOBS=();
            } elsif ( $rc ) {
                # Ok, in this case, we just errored, so re-throw the error
                die $rc;
            } else {
                # If we succeeded, and we did this action in the background intentionally
                # wait for the job to complete
                if ( $job ) {
                    my $jobid = $job->id();
                    push(@CConfig::BG_JOBS,$jobid);

                    # wait for background jobs to complete
                     if (@CConfig::BG_JOBS) {
                         $Log->trace("Waiting for jobs ".join(',',@CConfig::BG_JOBS));
                         while(CConfig::Job::are_jobs_active(\@CConfig::BG_JOBS)) { sleep 10; }
                     }
     
                     # check bg jobs
                     if (@CConfig::BG_JOBS) { CConfig::Job::check_jobs(\@CConfig::BG_JOBS); }
                     @CConfig::BG_JOBS=();
                }
            }

            return wantarray ? @ret : $ret[0];
        }
    } else {
        # For future simulator implementation
    }
}

# -----------------------------------------------
# Function: list2arrayrefs
#
#   NACL components want lists to be passed in as Perl 
#   arrays, not comma-separated strings.
#
# Usage:
#
#   $obj->list2arrayrefs( args => \%args, fields => fields => [ qw( disklist mirror-disks ) ] );
#
# -----------------------------------------------
sub list2arrayrefs {
    my $self=shift;
    my $me=(caller(0))[3];

    my %opt = (
        args    => undef,
        fields  => undef,
        @_
    );

    if ( !$opt{args} || !$opt{fields} ) {
        die "$me - Must provide args and fields options";
    }
    my $args        = $opt{args};
    my $fields      = $opt{fields};
    my $object_type = ref($self);

    foreach my $field (@$fields) {
        if ( exists $args->{$field} ) {

            # If field we are dealing with came from a "run command" it might already
            # be an array, so first make sure it is a scalar
            if ( !ref($args->{$field}) ) {
                my @items = split(',', $args->{$field});
                $args->{$field}=\@items;
            } elsif ( ref($args->{$field}) =~ /ARRAY/ ) {
                # If this is an array, there are times where NACL/ZAPI will 
                # contain an array of single key hashes.  Normally I would not
                # try to fix this sort of thing, but in this case it seems
                # harmless
                my @arr;
                foreach my $item (@{$args->{$field}}) {
                    if ( !ref($item) ) {
                        push(@arr, $item);
                    } elsif ( ref($item) =~ /HASH/ ) {
                        if ( scalar(keys %{$item}) == 1 ) {
                            $Log->warn("$me: $object_type has field $field as ".Dumper($item)." but it should be a list of scalars, raise a burt against NACL for the associated NACL CS object");
                            foreach my $key (keys %{$item}) {
                                push(@arr, $item->{$key});
                            }
                        } else {
                            $Log->warn("$me: $object_type has field $field as ".Dumper($item)." but it should be a list of scalars, raise a burt against NACL for the associated NACL CS object");
                        }
                    }
                }
                if ( @arr ) {
                    $args->{$field} = \@arr;
                }
            }
        }
    }
}

# -----------------------------------------------
# Function: convertByteString
#
#  Convert 20MB (for example) to 20971520.
#  Necessary because create accepts 20MB, but show will 
#  show these as purely numeric.  If we didn't convert, 
#  finds would not match.
#
# Usage:
#
#   $obj->convertByteString( "20MB" );
#
# -----------------------------------------------
sub convertByteString {
    my $me=(caller(0))[3];
    my $self=shift;
    my $string=shift;
    my $return = $string;

    if ( $string =~ /(\d+)KB$/i ) {
        $return = $1 * 1024;
    } elsif ( $string =~ /(\d+)MB$/i ) {
        $return = $1 * 1024 * 1024;
    } elsif ( $string =~ /(\d+)GB$/i ) {
        $return = $1 * 1024 * 1024 *1024;
    } elsif ( $string =~ /(\d+)TB$/i ) {
        $return = $1 * 1024 * 1024 *1024 * 1024;
    }

    return $return;
}

# -----------------------------------------------
# Function: noEntries
#
# Return 1 if no entries are found
#
# Usage:
#
#   $obj->noEntries( $@ );
#
# -----------------------------------------------
sub noEntries {
    my $me=(caller(0))[3];
    my $self=shift;
    my $string=shift;

    if ( $string =~ /(There are no entries matching your query|No matching|does not exist|doesn't exist|not found)/i ) {
        return 1;
    }

    if ( $string =~ /NoElementsFound|NoMatchingEntriesException/ ) {
        return 1;
    }

    return 0;
}

# -----------------------------------------------
# Function: resolveAttrList
#
#    This method is intended to resolve references which are based
#    upon auto-generated parameters.
#
#    For example, if you have this rule:
#      find volume with
#                  -default-autogen-pattern={^vserver}_vol%S
#                  junction-path=/{^volume}
#
#   There is no "volume" parameter at the time that Rule.pm resolves
#   the references, but Impl::Volume::action() will auto-create
#   the volume parameter, and so at THAT point it can resolve that
#   the "volume" reference used by "junction-path"
#
# Return An AttrList object
#
# Usage:
#
#   $with = $obj->resolveAttrList( key => 'volume', value => $args{volume}, attr_list => $item->{'WITH'} );
#   %args = $with->to_hash(FORMAT=>"create");
#
# -----------------------------------------------
sub resolveAttrList {
    my $me=(caller(0))[3];
    my $self=shift;
    my %opts = ( key       => undef,
                 value     => undef,
                 attr_list => undef,
                 @_ );

    my $attr = CConfig::Attr->new( $opts{key}, VALUE=>CConfig::ListIter->new(LIST=>CConfig::List->new($opts{value}),STEP=>0) );
    my $attrlist = Storable::dclone($opts{attr_list});
    $attrlist->add($attr);
    $attrlist->resolve();

    return $attrlist;
}

# -----------------------------------------------
# Function: DISALLOW_ZAPI
#   These CCL objects should NOT use ZAPI because of bugs.
#
# Usage:
#   my $allow  = CConfig::Impl::DISALLOW_ZAPI();
#
# -----------------------------------------------
sub DISALLOW_ZAPI   {
    if (@_) {
        my @impls = split( ',', $_[0] );

        foreach my $impl (@impls) {
           $disallow_zapi{$impl} = 1;
        }
    }

    return \%disallow_zapi;
}

# -----------------------------------------------
# Function: allow_zapi
#   Hack to NOT use ZAPI when there are known issues for a given object
#
# Usage:
#   my $allow  = CConfig::Impl::Volume->allow_zapi();
#
# -----------------------------------------------
sub allow_zapi   {
    my $implclassref = shift;
    my $object = CConfig::Impl->impl2object($implclassref);
    if ( exists $disallow_zapi{$object} ) {
        return 0;
    }
    return 1;
}

# -----------------------------------------------
# Function: PREFER_ZAPI
#   Whether ZAPI preference was set via FILER_APISET_SHOULD|MUST_INTERFACE
#
# Usage:
#   my $prefer  = CConfig::Impl->PREFER_ZAPI();
#
# -----------------------------------------------
sub PREFER_ZAPI   {
    my ($pkg, $val, undef)  = @_;
    if ( defined $val ) {
        $prefer_zapi = $val;
    }
    return $prefer_zapi;
}

# -----------------------------------------------
# Function: SHUFFLE
#   Whether to shuffle show commands or not
#
# Usage:
#   my $shuffle  = CConfig::Impl->SHUFFLE();
#
# -----------------------------------------------
sub SHUFFLE   {
    my ($pkg, $val, undef)  = @_;
    if ( defined $val ) {
        $shuffle = $val;
    }
    return $shuffle;
}


=head1 FUNCTIONS



=head2 create_impl

    factory method to create implementation object

=head3 Return

    CConfig::Impl subclass object

=head3 Parameters:


=over 4


=item item

    reference to the parser hash
    item->{'ACTION'} = create | find
    item->{'OBJECT'} = volume | aggregate | cluster | vserver
    item->{'WITH'} = CConfig::AttrList of attributes
    

=back


=head2 new

    create new object

=head3 Usage

    CConfig::Impl->new(KEYS=><keys>,DEFAULT=><default>)

=head3 Parameters:


=over 4


=item <keys>

    reference to an array of object keys 

=item <default>

    reference to an array of default attributes 
    

=back


=head2 check_keys

    Check that the args contain all required keys 

=head3 Usage

    $obj->check_keys(%args)

=head3 Return

    1 - all keys found else die

=head3 Parameters:


=over 4


=item self

    object instance

=item %args

    hash of args to be checked
    

=back


=head2 action

    Implement object actions (create/find/sub actions...)

=head3 Usage

    $obj->action(item)

=head3 Return

    0

=head3 Assumptions

    This will be called to implement object actions on each instance of an object
    Actions will be subtested if NUM is > 1, or the Implementation, or user forbids it
    Subtested jobs will be finalized in the post method
    Derived class should override this method to provide the desired behavior

=head3 Parameters:


=over 4


=item self

    object instance

=item item

    reference to the parser hash
    item->{'ACTION'} = create | find
    item->{'OBJECT'} = volume | aggregate | cluster | vserver
    item->{'WITH'} = CConfig::AttrList of attributes
    

=back


=head2 post

    Wait for background object creation to complete
    Verify objects were successfully created 
    log undo actions to undo file
    execute subactions for each created object

=head3 Usage

    $obj->post(item)

=head3 Return

    0

=head3 Assumptions

    Derived class should override this method

=head3 Parameters:


=over 4


=item self

    object instance
    

=back



=head1 SEE ALSO

=head1 HISTORY

=head1 TODO

=cut

1;
