# -*- perl -*-
# $Id$

## @summary CConfig::Impl::NodeCommand
## @author kenneth.gates@netapp.com
## @cf     false
## @status public
## @pod here

=head1 NAME

NodeCommand 

=head1 SYNOPSIS

  use NodeCommand 

=head1 DESCRIPTION

Implements object actions for NodeCommand objects

=cut


# --------------------------------------------------------
# Class: CConfig::Impl::NodeCommand
# --------------------------------------------------------

package CConfig::Impl::NodeCommand;
use strict;
use warnings;
use Data::Dumper;
use Tharn qw($Log);
use Params::Validate qw(:all);
use String::Random;
use Carp;
use NACL::STask::Node;

use base qw(CConfig::Impl);

# -----------------------------------------------
# Function: new
#   create new object
#
# Parameters:
#       item - reference to the parser hash
#           item->{'ACTION'} = create | find
#           item->{'OBJECT'} = volume | aggregate | cluster | vserver
#           item->{'WITH'} = CConfig::AttrList of attributes
#
# Assumptions: 
# -----------------------------------------------
sub new {
    my $class = shift;
    my $self=$class->SUPER::new();
    return $self;
}


# -----------------------------------------------
# Function: action
#   Implement object actions
#
# Usage: 
#   $obj->action(item)
#
# Parameters:
#       self - object instance
#       item - reference to the parser hash
#           item->{'ACTION'} = create | find
#           item->{'OBJECT'} = volume | aggregate | cluster | vserver
#           item->{'WITH'} = CConfig::AttrList of attributes
# -----------------------------------------------
sub action {
    my $self=shift;
    my $item=shift;
    my $me=(caller(0))[3];
    my (%aargs) = validate(@_, {
                NUM => {type=>SCALAR}
            }
    );

    my @result;
    my %args=$item->{WITH}->to_hash();

    SWITCH: for ($item->{'ACTION'}) {
            /run/i    && do { 
                my $node = $self->CMD_INT;
                my $cli_apiset = $node->apiset(category => "Node", interface => "CLI", set => "CMode");
                my $method = 'system_node_run';

                my ($best_effort, $timeout);
                if ($args{'-best-effort'}) {
                    $best_effort = $args{'-best-effort'};
                    delete $args{'-best-effort'};
                } elsif ($args{'-best_effort'}) {
                    $best_effort = $args{'-best_effort'};
                    delete $args{'-best_effort'};
                }

                if ($args{'method-timeout'}) {
                    $timeout = $args{'method-timeout'};
                    delete $args{'method-timeout'};
                }

                $self->filter_out_invalid( full_cmd=> $method, args => \%args );

                # Add timeout back in
                if ( $timeout ) {
                    $args{'connectrec-timeout'} = $timeout;
                }

                if ($best_effort) {
                    my $response=eval { $cli_apiset->$method(%args) };
                    if ($@) {
                        $Log->warn("$me: -best_effort flag is set, error will be ignored $@");
                    } else {
                        @result=@{$response->get_parsed_output()};
                    }
                } else {
                    my $response=$cli_apiset->$method(%args);
                    @result=@{$response->get_parsed_output()};
                }



                last SWITCH;
            };
            croak "$me :ERROR: Unknown action ".$item->{'ACTION'}." for object ".$item->{'OBJECT'};
    };

    return 1;
}

=head1 FUNCTIONS



=head2 new

    create new object

=head3 Parameters:


=over 4


=item item

    reference to the parser hash
    item->{'ACTION'} = create | find
    item->{'OBJECT'} = volume | aggregate | cluster | vserver
    item->{'WITH'} = CConfig::AttrList of attributes
    

=back


=head2 action

    Implement object actions

=head3 Usage

    $obj->action(item)

=head3 Parameters:


=over 4


=item self

    object instance

=item item

    reference to the parser hash
    item->{'ACTION'} = create | find
    item->{'OBJECT'} = volume | aggregate | cluster | vserver
    item->{'WITH'} = CConfig::AttrList of attributes

=back



=head1 SEE ALSO

=head1 HISTORY

=head1 TODO

=cut


1;
