# -*- perl -*-
# $Id$

## @summary CConfig::Impl::QosSettingsArchivedWorkload
## @author cwilcox@netapp.com
## @cf     false
## @status public
## @pod here

=head1 NAME

AvEngine 

=head1 SYNOPSIS

  use QosSettingsArchivedWorkload; 

=head1 DESCRIPTION

Implements object actions for QosSettingsArchivedWorkload (qos settings archived-workload) objects

Supported actions: find/modify

=cut

package CConfig::Impl::QosSettingsArchivedWorkload;
use strict;
use warnings;
use Data::Dumper;
use Tharn qw($Log);
use Params::Validate qw(:all);
use String::Random;
use Carp;

use base qw(CConfig::Impl);

# -----------------------------------------------
# Function: new
#   create new object
#
# Parameters:
#       item - reference to the parser hash
#           item->{'ACTION'} = find | modify
#           item->{'OBJECT'} = qos_settings_archived_workload
#           item->{'WITH'} = CConfig::AttrList of attributes
#
# Return: 
#   blessed object reference
#
# Assumptions: 
# -----------------------------------------------
sub new {
    my $class = shift;
    my @result;
    my $self=$class->SUPER::new(
        KEYS=>[],
        DEFAULT=>[]
    );

    $self;
}


# -----------------------------------------------
# Function: action
#   Implement object actions
#
# Usage: 
#   $obj->action(item)
#
# Parameters:
#       self - object instance
#       item - reference to the parser hash
#           item->{'ACTION'} = find | modify
#           item->{'OBJECT'} = qos_settings_archived_workload
#           item->{'WITH'} = CConfig::AttrList of attributes
#
# -----------------------------------------------
sub action {
    my $self=shift;
    my $item=shift;
    my $me=(caller(0))[3];
    my (%aargs) = validate(@_, {
                NUM   => {type=>SCALAR},
                LIMIT => {type=>SCALAR,default=>undef}
            }
    );

    my @result=();
    my $strict=0;
    my %args;

    # Can't create an archived workload, so no point in subtesting anything

    # no object to create, so strict_find and find are the same
    SWITCH: for ($item->{'ACTION'}) {
            /find/i    && do { 
                $strict=1;
                # Normally we would pass in args, but engine show tolerates no args passed in
                eval {
                    @result=$self->call_nacl_component_routine( routine => 'fetch' );
                };
                if ($@) {
                    @result=();
                    die $@ if ( !$self->noEntries($@) || $strict); 
                }
                my %sargs;
                foreach my $r (CConfig::Impl::shuffle(@result)) { 
                    # if we haven't picked this obj already then select it
                    if (!$self->RESULT_OBJS->contains($r)) { 
                        $self->RESULT_OBJS->add($r);

                        # generate subactions for this item, passing in primary keys from 'show'
                        CConfig::Rule::gen_subactions($self,$item,%sargs,$self->RESULT_OBJS->getRequired($r));

                        last SWITCH;
                    } 
                }
                last SWITCH;
            };
            /modify/i && do {
                %args=$item->{WITH}->to_hash(FORMAT=>"create");

                $self->add_undo( cmd => "modify", args => \%args );

                $self->call_nacl_component_routine( routine => 'modify', args => \%args );
                last SWITCH;
            };

            croak "$me :ERROR: Unknown action ".$item->{'ACTION'}." for object ".$item->{'OBJECT'};
    };

    return 1;
}

# -----------------------------------------------
# Function: get_results
#   Return created objects
#
# Usage:
#   $obj->get_results()
#
# Parameters:
#       self - object instance
#
# Return:
#         AoH for created/found objects
# -----------------------------------------------
sub get_results {
    my $self=shift;
    my @result=();

    my $resultobjs=$self->RESULT_OBJS;

    return $resultobjs->DATA;
}

# -----------------------------------------------
# Function: post
#   Return found object
#
# Usage: 
#   $obj->post()
#
# Parameters:
#       self - object instance
#       item - reference to the parser hash
#
# -----------------------------------------------
sub post {
    my $self=shift;
    my $me=(caller(0))[3];
    my @result=();

    return 1;
} 

# -----------------------------------------------
# Function: nacl_component_classname
#   Static method which returns the primary NACL
#   component classname for this implementation
#
# Usage:
#   my $nacl_c_classname = nacl_component_classname();
#
# -----------------------------------------------
sub nacl_component_classname {
    return "NACL::C::QosSettingsArchivedWorkload";
}

# -----------------------------------------------
# Function: cdef_root
#   Return cdef_root for this implementation.
#
# Usage:
#   my $cdef_root = cdef_root();
#
#  Note: This is used for parameter checking until
#        NACL supports an "ignore_extra_options" flag
#        and possibly for CCL simulation.
# -----------------------------------------------
sub cdef_root {
    my $pkg = shift;
    my $me=(caller(0))[3];
    my %opt = ( component_name => undef,
                @_ );

    return "qos_settings_archived_workload";
}



=head1 FUNCTIONS



=head2 new

    create new object

=head3 Return

    blessed object reference

=head3 Parameters:


=over 4


=item item

    reference to the parser hash
    item->{'ACTION'} = find | modify
    item->{'OBJECT'} = qos_settings_archived_workload
    item->{'WITH'} = CConfig::AttrList of attributes
    

=back


=head2 action

    Implement object actions

=head3 Usage

    $obj->action(item)

=head3 Parameters:


=over 4


=item self

    object instance

=item item

    reference to the parser hash
    item->{'ACTION'} = find | modify 
    item->{'OBJECT'} = qos_settings_archived_workload
    item->{'WITH'} = CConfig::AttrList of attributes
    

=back


=head2 post

    Return found object

=head3 Usage

    $obj->post(item)

=head3 Parameters:


=over 4


=item self

    object instance

=item item

    reference to the parser hash
    

=back



=head1 SEE ALSO

=head1 HISTORY

=head1 TODO

=cut

1;
