# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CConfig/List.pm#1 $

## @summary CConfig::List 
## @author kenneth.gates@netapp.com
## @cf     false
## @status public
## @pod here

=head1 NAME

List - a list 

=head1 SYNOPSIS

  use List

=head1 DESCRIPTION

An anstraction of a list of values, includes a current list position

=cut

# --------------------------------------------------------
# Class: CConfig::List
#   Class to abstract a list of values and include the concept of a current position
# --------------------------------------------------------

package CConfig::List;
use Params::Validate qw(:all);
use Data::Dumper;
use Carp;
use strict;
use warnings;

# -----------------------------------------------
# Function: new
#   constructor
#
# Usage: 
#   new(list)
#
# Parameters:
#       list      - reference to an array 
#
# Return: blessed reference
# -----------------------------------------------
sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;

    my $self     = {
        POS=>0,
        DATA=>[@_],
    };
    bless($self, $class);
}

# -----------------------------------------------
# Function: add
#   add element to the list 
#
# Usage: 
#   add(element)
#
# Parameters:
#       element      - element to add 
#
# Return: 
# -----------------------------------------------
sub add {
    my $self=shift;
    push(@{$self->DATA()},@_);
}

# -----------------------------------------------
# Function:  DATA - list data
# -----------------------------------------------
sub DATA   { my $self = shift; if(@_) { $self->{DATA} = $_[0]; } return $self->{DATA}; }
# -----------------------------------------------
# Function:  POS  - current cursor into the list
# -----------------------------------------------
sub POS    { my $self = shift; if(@_) { $self->{POS}  = $_[0]; } return $self->{POS}; }
# -----------------------------------------------
# Function:  LEN  - length of data array
# -----------------------------------------------
sub LEN    { my $self = shift; return scalar(@{$self->{DATA}}); }
# -----------------------------------------------
# Function:  LASTI- last index of data in list
# -----------------------------------------------
sub LASTI  { my $self = shift; return $#{$self->{DATA}}; }


# -----------------------------------------------
# Function: get
#   return value of a list element.  
#
# Usage: 
#   get([POS=>pos])
#
# Parameters:
#   pos - optional, list index to return value for, by default
#           will use the current list position
#
# Return: 
#   data element at the current cursor pos
#
# Assumptions:
# -----------------------------------------------
sub get {
    my $self=shift;
    my %args = validate(
        @_,
        {
           POS => { 
              type  => SCALAR,
              default => $self->POS,
           },
        }
    );
    return  $self->DATA()->[$args{'POS'}];
}


# -----------------------------------------------
# Function: copy
#   copy a list 
#
# Usage: 
#   copy()
#
# Parameters:
#
# Return: 
#   ref to copy of self
# -----------------------------------------------
sub copy {
    my $self=shift;
    my $copy=__PACKAGE__->new(@{$self->DATA}); 
    $copy->POS($self->POS);
    return $copy;
}

=head1 FUNCTIONS



=head2 new

    constructor

=head3 Usage

    new(list)

=head3 Return

    blessed reference

=head2 add

    add element to the list 

=head3 Usage

    add(element)

=head2 DATA - list data


=head2 POS  - current cursor into the list


=head2 LEN  - length of data array


=head2 LASTI- last index of data in list


=head2 get

    return value of a list element.  

=head3 Usage

    get([POS=>pos])

=head3 Return

    data element at the current cursor pos

=head3 Parameters:


=over 4


=item pos

    optional, list index to return value for, by default
    will use the current list position
    

=back


=head2 copy

    copy a list 

=head3 Usage

    copy()

=head3 Return

    ref to copy of self


=head1 SEE ALSO

=head1 HISTORY

=head1 TODO

=cut

1;

