
# assume node objects will have IFGRP definitions
# type can be ifgrp_port_add or ifgrp_create
ROUTINE ifgrp_data
   <CODE>
        my %opt  = validate( @_,
                             {
                                 nodes       => { type  => ARRAYREF },
                                 type        => { type  => SCALAR, 
                                                  default => "ifgrp_create", 
                                                  regex => qr/ifgrp_create|ifgrp_port_add/ },
                                 verbose     => { type  => SCALAR, default => 0 },
                             }
                   );

        my @nodes  = @{$opt{nodes}};
        my $type   = $opt{type};
        my @return;

        my $index = 0;
        my %ifgrp;

        foreach my $node ( @nodes ) {
            my @keys = grep(/IFGRP\d*$/, keys %{$node});
            my $nodename = $node->{_default_};
            $ifgrp{node} = $nodename;
            foreach my $key (@keys) {
                foreach my $pair (split(/\s+/,$node->{$key})) {
                    my ($k, $v) = split(/=/, $pair);
                    if ( $k eq "name" ) {
                        $k = "ifgrp";
                    }
                    if ( $k eq "ports" ) {
                        my @ports = split(/,/, $v);
                        $ifgrp{$k} = [@ports];
                    } else {
                        $ifgrp{$k} = $v;
                    }
                }

                if ( $type =~ /ifgrp_port_add/ ) {
                    my $ports = delete $ifgrp{ports};
                    foreach my $port (@$ports) {
                        $ifgrp{port} = $port;
                        push( @return, { %ifgrp } );
                    }
                } elsif ( $type =~ /ifgrp_create/ ) {
                    push( @return, { %ifgrp } );
                }
                %ifgrp = ();
            }
        }

        if ( $opt{verbose} ) {
            print "ifgrp_data $type = ".Dumper(\@return)."\n";
        }

        return @return;
    </CODE>;

ROUTINE replace_ports_with_ifgrps
   <CODE>
        my %opt  = validate( @_,
                             {
                                 lifs           => { type  => ARRAYREF },
                                 ifgrp_ports    => { type  => ARRAYREF },
                                 use_home_port  => { type  => SCALAR, default => 0 },
                                 verbose        => { type  => SCALAR, default => 0 },
                             }
                   );

        my @lifs         = @{$opt{lifs}};
        my @ifgrp_ports  = @{$opt{ifgrp_ports}};
        my @return;
        my ($node, $port) = qw( node port );

        # If the lifs are those found in the CCL NODES objects, use node and port
        # If they are from the CCL interface object, use home_node and home_port
        if ( $opt{use_home_port} ) {
            $node = 'home_node';
            $port = 'home_port';
        }

        my %ifgrp;
        foreach my $item ( @ifgrp_ports ) {
           $ifgrp{$item->{node}}{$item->{port}} = $item->{ifgrp}; 
        }

        foreach my $lif ( @lifs ) {
            if ( exists $ifgrp{$lif->{$node}}{$lif->{$port}} ) {
                $lif->{$port} = $ifgrp{$lif->{$node}}{$lif->{$port}};
            }
            push( @return, { %$lif } );
        }

        if ( $opt{verbose} ) {
            print "lifs with PORTS replaced by IFGRPs = ".Dumper(\@return)."\n";
        }

        return @return;
    </CODE>;

