# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CConfig/TEST/HashOrderedArrayAdapter.pm#1 $

## @summary HashOrderedArrayAdapter 
## @author kenneth.gates@netapp.com
## @cf     false
## @status public
## @pod here

=head1 NAME

HashOrderedArrayAdapter - FIT Eval Adapter

=head1 SYNOPSIS

  use CConfig::TEST::HashOrderedArrayAdapter

=head1 DESCRIPTION

use CConfig::TEST::HashOrderedArrayAdapter

=cut

# --------------------------------------------------------
# Class: CConfig::TEST::HashOrderedArrayAdapter
#   FIT Adapter
#   expect an ordered array
# --------------------------------------------------------


package CConfig::TEST::HashOrderedArrayAdapter;

use Data::Dumper;
use NCPAN::Test::C2FIT;
use HTML::Entities;
use Tharn qw($Log);
use base "NCPAN::Test::C2FIT::TypeAdapter";
use strict;

# -----------------------------------------------------
# Function: parse
#   parse cell text into an array of hashes
#
# Parameters:
#   text - cell text
#
# Returns: 
#   reference to array of hashes
# -----------------------------------------------------
sub parse {
    my $self = shift;
    my $text=decode_entities(shift);
    my $ha=eval($text) || throw NCPAN::Test::C2FIT::Exception( "ERROR parsing $text");
    return $ha;
}

# -----------------------------------------------------
# Function: toString
#   called to convert method result to text for cell error message
#
# Parameters:
#   result - method result, ref to array of hashes
#
# Returns: 
#   string 
# -----------------------------------------------------
sub toString {
    my $self = shift;
    my ($o) = @_;
    return Dumper($o);
}

# -----------------------------------------------------
# Function: equals
#   determines if the first hash array is fully contained
#   within the second
#
# Parameters:
#   ha1 - hash array 1 - from web page
#   ha2 - hash array 2 - from results
#
# Returns: 
#   1 - ha1 is contained in ha2
#   0 otherwise
# -----------------------------------------------------
sub equals {
    my $self = shift;
    my $me=(caller(0))[3];
    my ( $ha1, $ha2 ) = @_;
    $DB::single=1;

    return 0 unless ref($ha1) eq "ARRAY";
    return 0 unless ref($ha2) eq "ARRAY";
    if (scalar(@$ha1)!=scalar(@$ha2)) {
        $Log->debug("$me: Hash Array Length Mismatch: HA1:".Dumper($ha1)."  HA2: ".Dumper($ha2));
        return 0;
    }

    for (my $i=0; $i<length(@$ha1); $i++) {
        if (! $self->_hcomp($ha1->[$i],$ha2->[$i])) { return 0; }
    }
    return 1;
}

# -----------------------------------------------------
# Function: _hcomp
#   utility function to determine if hash 1 elements are all 
#   contained in hash2
#
# Parameters:
#   h1 - hash 1 - from web page
#   h2 - hash 2 - from results
#
# Returns: 
#   1 - h1 elements are contained in h2
#   0 otherwise
# -----------------------------------------------------
sub _hcomp {
    my $self = shift;
    my $me=(caller(0))[3];
    my ( $h1, $h2 ) = @_;

    if (!ref($h1) && !ref($h2)) { return $h1 eq $h2; };

    return 0 unless ref($h1) =~ /^(HASH|NACL::CS)/ && ref($h2) =~ /^(HASH|NACL::CS)/;
    
    foreach my $k (keys(%{$h1})) {
        $h1->{$k} =~ s/^\s*(.*?)\s*$/$1/; # trim
        my $k2 = $k;
        if (!exists($h2->{$k2})) {
            # NACL::CS will convert blah-blah to blah_blah
            my $alternate_k = $k2;
            $alternate_k =~ s/-/_/g;
            if (!exists($h2->{$alternate_k})) {
                $Log->debug("$me: Attribute $k not found");
                return 0;
            } else {
                $k2 = $alternate_k;
            }
        }
        $h2->{$k2} =~ s/^\s*(.*?)\s*$/$1/;

        if ($h2->{$k2} ne $h1->{$k}) { 
            $Log->debug("$me: Attribute $k does not match, expected->".$h1->{$k}."<- ne ->".$h2->{$k2}."<-actual");
            return 0; 
        }
    }

    return 1;
}

1;


