# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CConfig/TEST/ParserFixture.pm#1 $

## @summary ParserFixture 
## @author kenneth.gates@netapp.com
## @cf     false
## @status public
## @pod here

=head1 NAME

ParserFixture - FIT fixture

=head1 SYNOPSIS

  use CConfig::TEST::ParserFixture

=head1 DESCRIPTION

FIT Fixture

=cut

# --------------------------------------------------------
# Class: CConfig::TEST::ParserFixture 
#   FIT Adapter
# --------------------------------------------------------

package CConfig::TEST::ParserFixture;

use Data::Dumper;
use NCPAN::Test::C2FIT;
use NCPAN::Test::Error qw( :try );
use HTML::Entities;

use CConfig;
# Switched from Clone to Storable as Clone sometimes causes seg-faults
use Storable;

use Tharn qw($Log pass_result fail_result);
use Server;

use base 'NCPAN::Test::C2FIT::ActionFixture';
use strict;

sub new {
    my $pkg = shift;
    bless $pkg->SUPER::new( 
        server=>new Server(),
        methodSetterTypeMap => { 
            'server'  => 'CConfig::TEST::EvalAdapter',
        },
        fieldColumnTypeMap => { 
            'server'    => 'CConfig::TEST::HashArrayAdapter',
        }, 
        @_
    ),$pkg;
}

sub do_init {
    my $self = shift;

    $Log->comment("do_init:");

    try {
        CConfig::Rule::init(NODES=>[$self->{'server'}->get_clustered_nodes()]);
    } 
    otherwise {
       my $e = shift;
       throw NCPAN::Test::C2FIT::Exception( "ERROR: $e ");
    };
}

sub do_start_tx {
    my $self = shift;

    my $undofile    = decode_entities($self->{'cells'}->more()->text());
    $Log->comment("do_start_tx: $undofile");
    try {
        my $uffull=$::LOGDIR eq "" ? $undofile : $::LOGDIR."/".$undofile;
        CConfig::start_tx($uffull);
    } 
    otherwise {
       my $e = shift;
       throw NCPAN::Test::C2FIT::Exception( "ERROR: $e ");
    };
};

sub do_commit {
    my $self = shift;

    $Log->comment("do_commit");
    try {
        CConfig::commit();
    } 
    otherwise {
       my $e = shift;
       throw NCPAN::Test::C2FIT::Exception( "ERROR: $e ");
    };
};

sub do_undo {
    my $self = shift;
    my $undofile    = decode_entities($self->{'cells'}->more()->text());
    $Log->comment("do_undo: $undofile");
    CConfig::undo($undofile);
};

sub do_rollback {
    my $self = shift;
    $Log->comment("do_rollback");
    CConfig::rollback();
};

sub do_rule {
    my $self = shift;

    my $var         = decode_entities($self->{'cells'}->more()->text());
    my $rule        = decode_entities($self->{'cells'}->more()->more()->text());
    my $comment     = decode_entities($self->{'cells'}->more()->more()->more()->more()->text());

    # added a comparison mode after the comment.
    my $comp_mod;
    if ( $self->{'cells'}->more()->more()->more()->more()->more() ) {
        $comp_mod    = decode_entities($self->{'cells'}->more()->more()->more()->more()->more()->text());
    }
    $Log->comment("do_rule: $rule");

    my $res;
    try {
        $res=CConfig::Rule::process_rule($rule) || throw NCPAN::Test::C2FIT::Exception( "ERROR processing rule: $rule");
    }  otherwise {
       my $e = shift;
       throw NCPAN::Test::C2FIT::Exception( "ERROR: $e processing rule $rule");
    };

    # Clone DATA because the HashArrayAdapter compare routine will delete items from the array
    # and we don't want to modify the $CConfig::VARS structures
    $self->{$var}=($CConfig::VARS{$var}) ? Storable::dclone($CConfig::VARS{$var}->LIST->DATA) : undef;

    # Currently support no mode, or an "unordered" mode.  The ordered mode requires that the
    # entries compare in the same order in both hashes, unordered just requires that they
    # exist
    if ( $comp_mod && ($comp_mod =~ /unordered/i) ) {
        $self->{'fieldColumnTypeMap'}->{$var}='CConfig::TEST::HashArrayAdapter';
    } else {
        $self->{'fieldColumnTypeMap'}->{$var}='CConfig::TEST::HashOrderedArrayAdapter';
    }
    my $adapter = NCPAN::Test::C2FIT::TypeAdapter->onField( $self, $var );
    $self->check( $self->{'cells'}->more()->more()->more(), $adapter );
};

sub do_server {
    my $self = shift;

    my $server=new Server();

    # this is the do_enter part
    my $method      = $self->camel( $self->{'cells'}->more()->text() );
    my $args        = $self->{'cells'}->more()->more()->text();
    my $comment     = decode_entities($self->{'cells'}->more()->more()->more()->text());
    my $typeAdapter = NCPAN::Test::C2FIT::TypeAdapter->onSetter( $self, "server" );
    my $res         = $server->$method( $typeAdapter->parse($args) );

    $self->{"server"}=$res;

    my $adapter = NCPAN::Test::C2FIT::TypeAdapter->onField( $self, "server" );
    $self->check( $self->{'cells'}->more()->more()->more(), $adapter );
}

sub do_test_mode {
    my $self = shift;

    # this is the do_enter part
    my $flag        = decode_entities($self->{'cells'}->more()->text());
    $Log->comment("do_test_mode: $flag");
    if ($flag =~ /on/i) { $CConfig::TEST_MODE=1; }
    if ($flag =~ /off/i) { $CConfig::TEST_MODE=0; }
}

1;
