package CIFS::AuditLog::Helper;

#
##
#  Modules to import
#

use strict;
use warnings;
use Tharn qw($Log);
use Data::Dumper;
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::MTask::VserverSecurityFileDirectory;
use TCD;

#our $Node;

sub new {
    my $class  = shift;
    my $self   = {};
#    $Node   = shift;
    bless($self, $class);
    return $self;
}

sub get_audit_files {
    my $self  = shift;
    my $WincliObj  = shift;
    my $Winlogpath = shift;
    my $format     = shift;
    my $vserver    = shift;
    my $pattern    = "audit_$vserver";
    my $temp       = "temp.txt";
    my @files;
    
    $Log->enter();

    try {
        ### Below echo is done as the client is not refreshing the share when we do a dir ##
        $WincliObj->execute_raw_command(
            command => "echo refesh please >> $Winlogpath\\$temp");
        my $output = $WincliObj->dir(
            'path'                => "$Winlogpath\\$pattern$format",
            'bare-format'         => 1,
            'file-size-separator' => 1
        );
        @files = split("\n", $output->{'processed_output'});
    }
    otherwise {
        $Log->exit();
        my $e = shift;
        $e->throw();
    };
    $Log->exit();
    return @files;
} ## end sub get_audit_files

sub get_filecount {
    my $self  = shift;
    my $WincliObj  = shift;
    my $Winlogpath = shift;
    my $format     = shift;
    my $vserver    = shift;
    my $filecount;

    $Log->enter();
    try {
        $filecount = get_audit_files($self,$WincliObj, $Winlogpath, $format, $vserver);
    }
    otherwise {
        my $e = shift;
        $Log->exit();
        $e->throw();
    };
    $Log->exit();
    return $filecount;
} ## end sub get_filecount

sub _verify_filecount {
    my $self     = shift;
    my $WincliObj     = shift;
    my $vserver       = shift;
    my $Winlogpath    = shift;
    my $format        = shift;
    my $expectedcount = shift;
    my $actualcount;
    my $result = $TCD::PASS;

    $Log->enter();
    try {
        $actualcount = get_filecount($self,$WincliObj, $Winlogpath, $format, $vserver);
        if ($actualcount != $expectedcount) {
            $Log->comment(
                " Expected $expectedcount files but $actualcount files are generated "
            );
            $result = $TCD::FAIL;
            return;
        } else {
            $Log->comment(
                "Expected $expectedcount files and $actualcount files are generated as expected"
            );
        }
    }
    otherwise {
        my $e = shift;
        if (0 == $expectedcount) {
            if ($e->text() =~ /File Not Found/) {
                $Log->comment(" Expected File Not Found");
                return;
            }
        }
        $Log->exit();
        $e->throw();
    };
    $Log->exit();
    return $result;
} ## end sub _verify_filecount

sub verify_filecount {
    my $self  = shift;
    my %config = @_;
    my $temp = "temp.txt";
    my $result = $TCD::PASS;

    $Log->enter();
    try {
        if (defined $config{rotatecount}) {
            my $rotatecount = $config{rotatecount};
            my $format      = "_D*.$config{format}";
            $result = _verify_filecount( $self, $config{WincliObj}, $config{vserver}, $config{Winlogpath}, $format,
                $rotatecount);
        }
        if ($TCD::FAIL == $result) {
            return;
        }
        if (defined $config{activecount}) {
            my $activecount = $config{activecount};
            my $format      = "_last.$config{format}";
            $result = _verify_filecount( $self, $config{WincliObj},$config{vserver}, $config{Winlogpath}, $format,
                $activecount);
        }
    }
    otherwise {
        my $e = shift;
        $Log->exit();
        $e->throw();
    };
    $Log->exit();

    return $result;
} ## end sub verify_filecount

sub setacl {
    my ($self, %args) = @_;

    $Log->enter();
    my %opt= ( 'command_interface' => $args{command_interface},
               'vserver'   => $args{vserver},
               'path'      => [ $args{path} ],
               'ntfs_mode' => $args{ntfs_mode},
               'cleanup'   => 0);

    if (defined $args{sacl_type})
    {
        $opt{sacl} = [
                         {  'account' => $args{user},
                            'access-type' => $args{sacl_type},
                            'advanced-rights' => $args{sacl_rights},
                         }
                     ];
    }

    if (defined $args{dacl_type})
    {
        $opt{dacl} = [
                         {  'account' => $args{user},
                            'access-type' => $args{dacl_type},
                            'advanced-rights' => $args{dacl_rights},
                         }
                     ];
    }

    try {
        NACL::MTask::VserverSecurityFileDirectory->set_ntfs_acl(%opt);
    }
    otherwise {
        my $exception_object = shift;
        $Log->exit();
        $exception_object->throw();
    };
    $Log->exit();
} ## end sub setacl

sub run_commands {
    my ($self, $object, @commands) = @_;
    @commands = map { s/^\s+|\s+$//g; $_ }
        grep { !/^\s*$/ } map { split /[;\r\n]/ } @commands;
    foreach my $command (@commands) {
        $object->execute_raw_command(command => $command);
    }
} ## end sub run_commands


1;
