#!/usr/local/bin/perl
#
# Copyright (c) 2013 Network Appliance, Inc.
# All Rights reserved
#
## @summary
##  Library for sending the commands to avshim.exe
##
## @description
##  Provides function for sending commands to avshim.exe using NACL.
#
## @dependencies        none
#
## @status              public
#
## @author              bhaskarm@netapp.com

package CIFS::Avshim;
use Data::Dumper;
use strict;
use Params::Validate qw/validate_with SCALAR ARRAYREF/;
use NATE::BaseException qw(:try);
use NACL::APISet;

sub start {
    my ($self) = shift;
    my $obj    = {};
    my %opts   = validate_with(
        params => \@_,
        spec   => {
            hostobj         => { isa  => 'Hostrec::Windows' },
            filer_ip        => { type => SCALAR },
            executable_path => {
                type     => SCALAR,
                optional => 1,
                default  => "x:\\bin\\offbox_av\\avshim.exe",
            }
        }
    );

    #my %opts = @_;
    my $path;
    my $Hostobj    = $opts{hostobj};
    my $APISet_obj = NACL::APISet->new(
        hostobj   => $Hostobj,
        category  => "Host",
        interface => "CLI",
        set       => "Windows",
        connid    => 'login',
        conntype  => 'rsh-persist',
    );

    #print Dumper($APISet_obj);

    $obj->{_Apiset} = $APISet_obj;

    $path = $opts{'executable_path'};
    my $command  = $path . " " . $opts{filer_ip};
    my $response = $APISet_obj->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["avshim>"]
    );

# if ($response =~ /\S/s) {
# NATE::BaseException->throw("Throwing NATE::BaseException: Failed to open executable: $response");
# }
    bless $obj, $self;
    return $obj;
}

sub delay {
    my ($self)     = shift;
    my %opts       = @_;
    my $delay_time = $opts{delay_time};
    my $response   = $self->{_Apiset}->execute_raw_command(
        command              => "d",
        'additional_prompts' => ["avshim>"],
        "connectrec-match_table" =>
            [ "Enter the delay value:" => "$delay_time" ]
    );
    return 0;
}

sub start_logging {
    my ($self) = shift;
    my %opts = @_;

    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "c",
        'additional_prompts' => ["avshim>"]
    );
    return 0;
}

sub stop_logging {
    my ($self) = shift;
    my %opts = @_;

    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "C",
        'additional_prompts' => ["avshim>"]
    );
    return 0;
}

sub get_request_info {
    my ($self) = shift;
    my %opts = @_;
    my @result_array;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "preq",
        'additional_prompts' => ["avshim>"]
    );
    $response =~ s/\[pipe:.*\n//g;
    $response =~ s/\d+\.\d+:.*\n+//g;
    my @responses = split( /\n\n+/, $response );
    foreach my $r (@responses) {
        my %hash;
        my @resp = split( "\n", $r );
        foreach my $res (@resp) {
            if ( $res =~ /mem:\s*(.*)/ ) {
                $hash{"file-name"} = $1;
            } elsif ( $res =~ /(\S+)\s*:\s*(.*)/ ) {
                $hash{$1} = $2;
            }
        }
        push( @result_array, \%hash );
    }
    my @final_res;
    foreach my $res (@result_array) {
        push( @final_res, $res ) if ( $res->{'file-name'} );
    }
    return \@final_res;
}

sub get_response_info {
    my ($self) = shift;
    my %opts = @_;

#Request id 8 of path \\10.225.118.114\ontap_admin$\@@000004620000000000000000d80
#20fc267 completed with status 111, str 'Infected file!'
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "pres",
        'additional_prompts' => ["avshim>"]
    );
    my @arr;
    my @responses = split( "\n", $response );
    foreach my $res (@responses) {
        if ( $res
            =~ /Request id\s+(\S+)\s+of path\s+(\S+)\s+completed with status\s+(\S+),str\s+'(.*)'/
            )
        {
            my %hash;
            $hash{req_id}        = $1;
            $hash{path}          = $2;
            $hash{status}        = $3;
            $hash{status_string} = $4;
            push( @arr, \%hash );
        }
    }
    return \@arr;
}

sub get_scanner_info {
    my ($self) = shift;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "preg",
        'additional_prompts' => ["avshim>"]
    );
    return $response;
}

sub get_update_info {
    my ($self) = shift;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "pupd",
        'additional_prompts' => ["avshim>"]
    );
    return $response;

}

sub loopback {
    my ($self) = shift;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "LB",
        'additional_prompts' => ["avshim>"]
    );
    return $response;
}

sub toggle_loopback {
    my ($self) = shift;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "l",
        'additional_prompts' => ["avshim>"]
    );
    return $response;
}

sub stop {
    my ($self)   = shift;
    my %opts     = @_;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "x",
        'additional_prompts' => ["avshim>"]
    );
    return 0;
}

sub get_raw_request_info {
    my ($self) = shift;
    my %opts = @_;
    my @result_array;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "preq",
        'additional_prompts' => ["avshim>"]
    );
    $response =~ s/\[pipe:.*\n//g;

    return $response;
}

1;
