# $Id$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Logs Component Module (auto-generated by CGT)
## @author ravindrs@netapp.com
## @status shared
## @pod here

package CIFS::BranchCacheCLIZAPIYAML;
use strict;
use warnings;
use YAML;
use YAML::Loader;
use YAML::Dumper;




my $ROOT_TAG = "testcases";
my $NULL = -1;

######################################################
#Instantiate a new YAML Object.
######################################################
sub new {
    my $class = shift;
    my $configfile = shift;
    my $project_name = shift;
    my $cfgfh;
    my @tests = ();
    my $ydata = 0;
    my $bc = 0;

    my $self = {
        configfile => $configfile,
        project_tag => $project_name,
        tests      => \@tests,
        ydata      => 0,
        cfgfh      => $cfgfh,
        root         => 0
    };
    bless $self, $class;
    $self->initilize();
    return $self;
}

######################################################
#Initilizie the yaml object.
######################################################
sub initilize {
    my $self = shift;
    open $self->{cfgfh}, '<', $self->{configfile} or die "Can't open file  $self->configfile: $! ";
    $self->{ydata} = YAML::LoadFile($self->{cfgfh});
    my %cfg = %{$self->{ydata}};
    my %root  = %{$cfg{$ROOT_TAG}};
    $self->{root} = \%root;

    return 0;
}

######################################################
#Returns Operations provided in yaml file
######################################################

sub getUIOps {
    my $self = shift;
    my $uiops;
    print  $self->{root}->{$self->{project_tag}};
    $uiops = $self->{root}->{$self->{project_tag}};
    if(defined $uiops) {
        return split(', ', $uiops);
    }
    return $NULL;
}

######################################################
#Returns ZAPI testcase names in yaml file
######################################################

sub getZapiTcNames {
    my $self = shift;
    my $uiop = shift;
    my $zapitc;
    $zapitc = $self->{root}->{$uiop}{zapi};
    if(defined $zapitc) {
        return split(',', $zapitc);
    } 
    return $NULL;
}

######################################################
#Returns CLI testcase names in yaml file
######################################################
sub getCliTcNames {
    my $self = shift;
    my $uiop = shift;
    my $clitc;
    $clitc = $self->{root}->{$uiop}{cli};
    if(defined $clitc) {
        return split(',', $clitc);
    } 
    return $NULL;
}

###########################################################
#Returns a testcase in yaml file by prociding testcase name
###########################################################

sub getTestcase {
    my $self = shift;
    my $tcname = shift;
    return $self->{root}->{$tcname}{test};
}

###########################################################
#Returns a testcase control info in yaml file
###########################################################

sub getCtrlInfo{
    my $self = shift;
    my $tcname = shift;
    return $self->{root}->{$tcname}{ctrl};
}

###########################################################
#Returns a testcase config info in yaml file
###########################################################
sub getConfigInfo{
    my $self = shift;
    my $tcname = shift;
    return $self->{root}->{$tcname}{config};
}

###########################################################
#Returns CLI testcases as array in yaml file
###########################################################
sub getCliTestcases {
    my $self = shift;
    my $uiop = shift;
    my @tcnames = ();
    my @clitests = ();

    @tcnames = $self->getCliTcNames($uiop);
    foreach my $tcname (@tcnames) {
        push(@clitests, $self->getTestcase($tcname));
    }
    return @clitests;
}

###########################################################
#Returns ZAPI testcases as array in yaml file
###########################################################
sub getZapiTestcases {
    my $self = shift;
    my $uiop = shift;
    my @tcnames = ();
    my @zapitests = ();

    @tcnames = $self->getZapiTcNames($uiop);
    foreach my $tcname (@tcnames) {
        push(@zapitests, $self->getTestcase($tcname));
    }
    return @zapitests;
}

1;
