# $Id: //depot/prod/ontap/main/test/lib/CIFS/Cifs_cmd.pm
#
## @summary Routines for checking different cifs commands.
##
## @description
##              These routines  are used primarily by CIFS tests
##              for checking cifs commands for different version of ontap.
##
##
## use Tharn;
##
## @author  madhu@netapp.com
##
##

require Exporter;
use strict;
use Tharn;
use SayWrap qw(sayEOF);
use CIFS qw(:all);
use Carp qw(croak);
use Filer::Version qw(get_wafl_version WV_ANCHORSTEAM_MIN WV_ANCHORSTEAM_MAX
			  WV_TSINGTAO_MIN  WV_TSINGTAO_MAX  WV_GORDONBIERSCH_MIN
			  WV_GORDONBIERSCH_MAX WV_TRICKY_MIN WV_TRICKY_MAX );

use vars qw (@ISA @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT_OK = qw (
		cifs_help
		cifs_usage
		cifs_access_modify 
		cifs_access_delete
		cifs_change_browse
		cifs_change_nobrowse
		);


## @name ::cifs_help
##
## @summary check for CIFS command for anchorsteam, RtsingtaoN and GB.
##
## @arg $fc
##  $fc - required. filer connection object
##  example: cifs_help($fc)
##

sub cifs_help 
{
	my $fc=shift;
	my $version = get_wafl_version($fc->{hostp});
	my $text=sayEOF($fc, "priv set; cifs help");
	if (($version  >= WV_ANCHORSTEAM_MIN) && ($version  <= WV_ANCHORSTEAM_MAX)){
		if($text =~ /access\s*help\s*restart\s*terminate\s*audit\s*homedir\s*sessions\s*testaddr\s*broadcast\s*lookup\s*setup\s*testdc\s*changefilerpwd\s*nbalias\s*shares\s*top\s*comment\s*prefdc\s*sidcache\s*winreg\s*domaininfo\s*resetdc\s*stat/){
			return 0;}}
        elsif (($version  >=  WV_TSINGTAO_MIN) && ($version  <=  WV_TSINGTAO_MAX)){
                if($text =~ /access\s*gpresult\s*prefdc\s*sidcache\s*audit\s*gpupdate\s*resetdc\s*stat\s*broadcast\s*help\s*restart\s*terminate\s*changefilerpwd\s*homedir\s*sessions\s*testdc\s*comment\s*lookup\s*setup\s*top\s*domaininfo\s*nbalias\s*shares/){
                       	return 0;}} 
        elsif (($version >= WV_GORDONBIERSCH_MIN) && ($version  <=  WV_GORDONBIERSCH_MAX)){
                if($text =~ /access\s*gpresult\s*resetdc\s*stat\s*audit\s*gpupdate\s*restart\s*terminate\s*broadcast\s*help\s*sessions\s*testaddr\s*changefilerpwd\s*homedir\s*setup\s*testdc\s*cndump\s*lookup\s*shares\s*top\s*comment\s*nbalias\s*sidcache\s*winreg\s*domaininfo\s*prefdc/){
                        return 0;	}}
    	elsif ($version >= WV_TRICKY_MIN ){
                if($text =~ /access\s*domaininfo\s*prefdc\s*stat\s*adupdate\s*gpresult\s*resetdc\s*terminate\s*audit\s*gpupdate\s*restart\s*testaddr\s*broadcast\s*help\s*sessions\s*testdc\s*changefilerpwd\s*homedir\s*setup\s*top\s*cndump\s*lookup\s*shares\s*winreg\s*comment\s*nbalias\s*sidcache/){
                        return 0;	}}

	else{
                        return 1;
	}
}


## @name ::cifs_usage
##
## @summary check for CIFS command for anchorsteam, RtsingtaoN and GB.
##
## @arg $fc
##  $fc - required. filer connection object
##  example: cifs_usage($fc)
##


sub cifs_usage
{
        my $fc=shift;
        my $version = get_wafl_version($fc->{hostp});
        my $text=sayEOF($fc, "priv set; cifs help");
        if (($version  >= WV_ANCHORSTEAM_MIN) && ($version  <= WV_ANCHORSTEAM_MAX)){
                if($text =~ /access\s*help\s*restart\s*terminate\s*audit\s*homedir\s*sessions\s*testaddr\s*broadcast\s*lookup\s*setup\s*testdc\s*changefilerpwd\s*nbalias\s*shares\s*top\s*comment\s*prefdc\s*sidcache\s*winreg\s*domaininfo\s*resetdc\s*stat/){
                        return 0;}}
        elsif (($version  >=  WV_TSINGTAO_MIN) && ($version  <=  WV_TSINGTAO_MAX)){
                if($text =~ /access\s*gpresult\s*prefdc\s*sidcache\s*audit\s*gpupdate\s*resetdc\s*stat\s*broadcast\s*help\s*restart\s*terminate\s*changefilerpwd\s*homedir\s*sessions\s*testdc\s*comment\s*lookup\s*setup\s*top\s*domaininfo\s*nbalias\s*shares/){
                        return 0;}}
        elsif (($version >= WV_GORDONBIERSCH_MIN) && ($version  <=  WV_GORDONBIERSCH_MAX)){
                if($text =~ /access\s*gpresult\s*resetdc\s*stat\s*audit\s*gpupdate\s*restart\s*terminate\s*broadcast\s*help\s*sessions\s*testaddr\s*changefilerpwd\s*homedir\s*setup\s*testdc\s*cndump\s*lookup\s*shares\s*top\s*comment\s*nbalias\s*sidcache\s*winreg\s*domaininfo\s*prefdc/){
                        return 0;       }}
    	elsif ($version >= WV_TRICKY_MIN ){
                if($text =~ /access\s*domaininfo\s*prefdc\s*stat\s*adupdate\s*gpresult\s*resetdc\s*terminate\s*audit\s*gpupdate\s*restart\s*testaddr\s*broadcast\s*help\s*sessions\s*testdc\s*changefilerpwd\s*homedir\s*setup\s*top\s*cndump\s*lookup\s*shares\s*winreg\s*comment\s*nbalias\s*sidcache/){
                        return 0;	}}

        else{
                        return 1;
        }
}

## @name ::cifs_access_modify
##
## @summary check for CIFS access -m options for anchorsteam, RtsingtaoN and GB.
##
## @arg $fc,$share
##  $fc - required. filer connection object
##  $share - required share name 
##  example: cifs_access_modify($fc,$share)
##


sub cifs_access_modify
{
	my ($fc ,$share)=@_;
        my $version = get_wafl_version($fc->{hostp});
	my $text=sayEOF($fc, "cifs access $share -m");
	if (($version  >= WV_ANCHORSTEAM_MIN) && ($version<=  WV_TSINGTAO_MAX)){
		if($text eq  ""){
		 return 0;}}
	elsif($version >= WV_GORDONBIERSCH_MIN){
		if($text =~ /1 share\(s\) have been successfully modified/){

			return 0;}}
	else{
		return 1;
	}
}

## @name ::cifs_access_delete
##
## @summary check for CIFS access delete  options for anchorsteam, RtsingtaoN and GB.
##
## @arg $fc, $share
##  $share - required share name 
##  $fc - required. filer connection object
##  example: cifs_access_delete($fc,$share)
##

sub cifs_access_delete
{
        my ($fc ,$share)=@_;
	my $version = get_wafl_version($fc->{hostp});
        my $text=sayEOF($fc, "cifs access -delete $share -m");
        if (($version  >= WV_ANCHORSTEAM_MIN) && ($version<=  WV_TSINGTAO_MAX)){
                if($text eq  ""){
                 return 0;}}
        elsif($version >= WV_GORDONBIERSCH_MIN){
		if($text =~ /1 share\(s\) have been successfully modified/){
                        return 0;}}
        else{
                return 1;
        }
}

## @name ::cifs_change_browse
##
## @summary check for CIFS shares change command for browse 
##              option GB and up versions.
##
## @arg $fc, $share
##  $fc - required. filer connection object
##  $share-required share name
##  example: cifs_change_browse($fc,$share)
##

sub cifs_change_browse
{
        my ($fc ,$share)=@_;
        my $version = get_wafl_version($fc->{hostp});
	if (($version  >= WV_ANCHORSTEAM_MIN) && ($version<=  WV_TSINGTAO_MAX)){
                 return -1;}
        elsif($version >= WV_GORDONBIERSCH_MIN){
        my $text=sayEOF($fc, "cifs shares -change $share -browse");
        if($text=~ /1 share\(s\) have been successfully modified/){
                return 0;}}
        else{
                return 1;
        }
}

## @name ::cifs_change_nobrowse
##
## @summary check for CIFS change command for nobrowse 
##		option GB and up versions.
##
## @arg $fc, $share
##  $fc - required. filer connection object
##  $share-required share name 
##  example: cifs_change_nobrowse($fc,$share)
##

sub cifs_change_nobrowse
{
        my ($fc ,$share)=@_;
        my $version = get_wafl_version($fc->{hostp});
	if (($version  >= WV_ANCHORSTEAM_MIN) && ($version<=  WV_TSINGTAO_MAX)){
                 return -1;}
        elsif($version >= WV_GORDONBIERSCH_MIN){
	my $text=sayEOF($fc, "cifs shares -change $share -nobrowse");
	my $output=sayEOF($fc, "cifs shares $share");
	if(($text=~ /1 share\(s\) have been successfully modified/) && 
			($output =~ /not visible to browsers/)){
		return 0;}}
	else{
		return 1;
	}
}



	 


	


