#!/usr/local/bin/perl
#
# Copyright (c) 2011 Network Appliance, Inc.
# All Rights reserved
#
## @summary
##  Library for sending the commands to cifs_test.exe
##
## @description
##  Provides function for sending commands to cifs_test.exe using NACL.
#
## @dependencies        none
#
## @status              public
#
## @author              alayathu@netapp.com

package CIFS::Cifstest;
use Data::Dumper;
use strict;
use Params::Validate qw/validate_with SCALAR ARRAYREF/;
use NATE::BaseException qw(:try);

sub cifstest_open {
    my ($self) = shift;
    my $obj    = {};
    my %opts   = validate_with(
        params => \@_,
        spec   => {
            hostobj         => { isa  => 'Hostrec::Windows' },
            file_name       => { type => SCALAR },
            share_mode      => { type => SCALAR },
            access_mode     => { type => SCALAR },
            create_options  => { type => SCALAR },
            executable_path => {
                type     => SCALAR,
                optional => 1,
                default  => "x:\\bin\\cifs_test.exe",
            }
        }
    );

    #my %opts = @_;
    my $path;
    my $Hostobj    = $opts{hostobj};
    my $APISet_obj = NACL::APISet->new(
        hostobj   => $Hostobj,
        category  => "Host",
        interface => "CLI",
        set       => "Windows",
        connid    => 'login',
        conntype  => 'rsh-persist',
    );

    #print Dumper($APISet_obj);

    $obj->{_Apiset} = $APISet_obj;

    $path = $opts{'executable_path'};

    my $response = $APISet_obj->execute_raw_command(
        command              => $path,
        'additional_prompts' => ["bingo>"]
    );
    if ( $response =~ /\S/s ) {
        NATE::BaseException->throw(
            "Throwing NATE::BaseException: Failed to open executable: $response"
        );
    }

    my $filename      = $opts{file_name};
    my $sharemode     = $opts{share_mode};
    my $accessmode    = $opts{access_mode};
    my $createoptions = $opts{create_options};

    my $command
        = "o "
        . $filename . " "
        . $sharemode . " "
        . $accessmode . " "
        . $createoptions;

    $response = $APISet_obj->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    print "\n##### OUT :: $response\n command : $command \n";
    $obj->{'file_name'} = $filename;
    if ( $response =~ /Open SUCCESS\sFH *= *(\d+)/ ) {
        $obj->{_fileHandle} = $1;
    } else {
        NATE::BaseException->throw(
            "Throwing NATE::BaseException: The file open failed: $response");
    }

    bless $obj, $self;
    return $obj;
}

sub lock {
    my ($self) = shift;
    my %opts = @_;

    my ( $mode, $waitmode, $offset, $length )
        = ( $opts{mode}, $opts{waitmode}, $opts{offset}, $opts{length} );
    my $command
        = "l "
        . $self->{_fileHandle} . " "
        . $mode . " "
        . $waitmode . " "
        . $offset . " "
        . $length;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    return $response;
}

sub unlock {
    my ($self) = shift;
    my %opts = @_;

    my ( $offset, $length ) = ( $opts{offset}, $opts{length} );
    my $command = "u " . $self->{_fileHandle} . " " . $offset . " " . $length;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    return $response;
}

sub read {
    my ($self) = shift;
    my %opts = @_;

    my ( $offset, $length ) = ( $opts{offset}, $opts{length} );
    my $command = "r " . $self->{_fileHandle} . " " . $offset . " " . $length;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    return $response;
}

sub write {
    my ($self) = shift;
    my %opts = @_;

    my ( $offset, $length ) = ( $opts{offset}, $opts{length} );
    my $command = "w " . $self->{_fileHandle} . " " . $offset . " " . $length;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    return $response;
}

sub delete {
    my ($self) = shift;
    my %opts = @_;

    my $filename = $opts{file_name} || $self->{'file_name'};
    my $command  = "d " . $filename;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    if ( $opts{'quit'} ) {
        $self->{_Apiset}->execute_raw_command( command => "q" );
    }
    return $response;
}

sub close {
    my ($self) = shift;

    my $command  = "c " . $self->{_fileHandle};
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    return $response;
}

sub quit {
    my ($self) = shift;

    my $command = "q ";
    my $response
        = $self->{_Apiset}->execute_raw_command( command => $command );
    return $response;
}

sub hash_request {
    my ($self) = shift;
    my %opts = @_;

    my ( $offset, $length, $max_octl_len )
        = ( $opts{offset}, $opts{length}, $opts{max_ioctl_len} );
    my $command
        = "srh "
        . $self->{_fileHandle} . " "
        . $offset . " "
        . $length . " "
        . $max_octl_len;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["bingo>"]
    );
    return $response;
}

1;
