# $Id$

#
# Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

package CIFS::CommonLib;

## @summary
## @author dl-cifs-qa-svl@netapp.com
## @status shared
## @pod here

=head1 NAME

package CIFS::CommonLib;

=head1 DESCRIPTION
    This module contains subroutines common to most cifs testing scripts.
    QA team members should try to use subroutines from this library as much
    as possible while writing automation.
=cut

#
# Compiler directives.
#

use strict;
use warnings;
use Tharn qw($Log);
use Data::Dumper;
use NATE::BaseException qw(:try);
use NACL::C::Client;
use NACL::C::Node;
use Tharn;
use Options;
use Params::Validate qw(:all);
use NACL::C::Vserver;
use NACL::STask::NetworkInterface;
use NACL::STask::Volume;
use NACL::STask::VserverCifsShare;
use NACL::C::VserverCifsUsersAndGroupsLocalUser;
use NACL::C::VserverCifsUsersAndGroupsLocalGroup;
use NACL::C::VserverCifsUsersAndGroupsPrivilege;
use NACL::C::NetworkRoutingGroupsRoute;
use NACL::C::VserverServicesDns;
use NACL::STask::Client::Mount;
use CIFS::Params_CIFSGX;
use Protocol::ClientFileOperations;
use NACL::C::VserverNameMapping;
use NACL::C::NetworkRoute;
use NACL::C::UnixGroup;
use NACL::C::UnixUser;
use NACL::C::VserverCifs;
use NACL::C::VserverCifsOptions;
use NACL::C::VserverExportPolicy;
use NACL::C::VserverExportPolicyRule;
use NACL::C::VserverNameMapping;
use NACL::C::StorageAggregate;
use NACL::C::Network;
use Net::IP qw(:PROC);
use NACL::C::DiagSecdAuthentication;
use NACL::STask::VserverSecurityFileDirectoryPolicy;
use NACL::STask::VserverSecurityFileDirectoryNtfs;
use NACL::STask::VserverSecurityFileDirectoryPolicyTask;
use NACL::STask::VserverSecurityFileDirectoryNtfsDacl;
use NACL::STask::VserverSecurityFileDirectoryNtfsSacl;
use NACL::C::VserverSecurityFileDirectory;
use NACL::C::VserverSecurityFileDirectoryJob;
use NACL::C::VserverCifsShareAccessControl;
use NACL::C::DiagNbladeReplay;
use NACL::APISet;
use NACL::C::VserverCifsSymlink;
use NACL::C::DiagSecdCache;

my $Options = new Options();
$Options->process();
my $Params = $Options->get_hash();

###############################################################################
# Subroutine  : new
# Input       : none
# Return      : Instance of this class
# Description : This subroutine creates instance of class
###############################################################################
sub new {
    my $class = shift;
    my %args  = Params::Validate::validate( @_, {} );
    my $self  = {};

    bless( $self, $class ) unless ref($class);

    return $self;
} ## end sub new

###############################################################################
# Subroutine  : DESTROY
# Input       : none
# Return      : none
# Description :Currently need to do nothing when this instance of common lib
#              goes away
###############################################################################
sub DESTROY {
    my $self = shift;
}

###############################################################################
# Subroutine  : fetch
# Input       : none
# Return      : $self
# Description : This subroutine populates all cifs testing related objects
###############################################################################

sub fetch {
    my $self = shift;
    $Log->debug('CIFS::CommonLib setup');

    my ( @clients, @nodes );

    @nodes = NACL::C::Node->find();

    $self->{nodes} = [@nodes];

    try {
        @clients = NACL::C::Client->find();
        $self->{clients}         = \@clients;
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->warn("No client(s) found in the testbed");
        $Log->comment( "Caught a " . ref($exception_object) . "!" );
        $Log->comment( "Error is: " . $exception_object->text() );
    };

    if ( defined @clients ) {
        try {
            $self->{unix_clients}    = [ NACL::C::Client->find_unix_clients() ];
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("No unix client(s) found in the testbed");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );
        };

        try {
            $self->{windows_clients} = [ NACL::C::Client->find_windows_clients() ];
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("No windows client(s) found in the testbed");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );
        };
    }

    ## vservers

    my (@vservers);
    logcomment("Vserver");

    @vservers = NACL::C::Vserver->find(
        command_interface => $self->{nodes}->[0],
        filter            => {
            type          => 'data',
            'admin-state' => 'running'
        }
    );

    $self->{vservers} = [@vservers];

    my @vserver_names = map { $_->vserver() } @vservers;

    $self->{vserver_names} = [@vserver_names];

    foreach my $vserver (@vserver_names) {

        ##data lif's
        try {
            $self->get_network_interfaces($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn(
                "Missing network interfaces :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
#        ## network routes
#        try {
#            $self->get_network_routes($vserver);
#        }
#        catch NATE::BaseException with {
#            my $exception_object = shift;
#            $Log->warn("Missing network routes :In the BaseException catch");
#            $Log->comment( "Caught a " . ref($exception_object) . "!" );
#            $Log->comment( "Error is: " . $exception_object->text() );
#
#        };
        ## dns
        try {
            $self->get_dns_entries($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("Missing dns entries :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
        ##unix group
        try {
            $self->get_unix_group($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("Missing unix group :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
        ##unix user
        try {
            $self->get_unix_user($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("Missing unix user :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
        ##cifs options
        try {
            $self->get_cifs_options($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("Missing cifs options :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
        ##export-policy
        try {
            $self->get_export_policy($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("Missing export policy :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
        ##export-policy rule show
        try {
            $self->get_export_policy_rule($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn(
                "Missing export policy rule :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };

        # name-mapping
        try {
            $self->get_name_mapping($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("Missing name mapping :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
        ##cifs server
        try {
            $self->get_cifs_server($vserver);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->warn("Missing cifs server :In the BaseException catch");
            $Log->comment( "Caught a " . ref($exception_object) . "!" );
            $Log->comment( "Error is: " . $exception_object->text() );

        };
    }

    ##

    return $self;
}

###############################################################################
# Subroutine  : create_volume
# Input       : hash of volume parameters
#            'vserver'                # mandatory
#            'junction-path'          # mandatory
#            'nacltask_if_exists'     # optional
#            'volume'                 # mandatory
#            'aggregate'              # mandatory
#            'unix-permissions'       # optional
#            'security-style'         # optional
#            'size'                   # optional
# Return      : volume object - success
# Description : Creates a volume.
###############################################################################
sub create_volume {
    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'            => 1,    # mandatory
            'junction-path'      => 1,    # mandatory
            'nacltask_if_exists' => 0,    # optional
            'volume'             => 1,    # mandatory
            'aggregate'          => 0,    # mandatory
            'unix-permissions'   => 0,    # optional
            'security-style'     => 0,    # optional
            'size'               => 0,    # optional
        }
    );

    my $test_node          = $self->{nodes}->[0];
    my $volume_name        = $args{volume};
    my $vserver_name       = $args{vserver};
    my $nacltask_if_exists = $args{nacltask_if_exists} || 'purge';
    my $junction_path      = $args{'junction-path'};
    my $aggr_name          = $args{aggregate}
      || $self->{vservers}->[0]->get_one_state_attribute('aggregate');
    my $unix_permissions = $args{'unix-permissions'} || '777';
    my $security_style   = $args{'security-style'}   || 'mixed';
    my $volume_size      = $args{'size'}             || '100mb';

    my $test_volume;

    $Log->debug(
"Here is the data for aggregate $aggr_name to be used for creating volume $volume_name"
    );

    NACL::CS::StorageAggregate->fetch(
        command_interface => $test_node,
        filter            => { aggregate => $aggr_name },
        requested_fields =>
          [qw(size usedsize percent-used availsize state capacity nodes)]
    );

    # Volume create

    if ( $security_style =~ m/mixed|unix/i ) {
        $Log->debug(
            "Create volume $volume_name with junction-path $junction_path");
        $test_volume = NACL::STask::Volume->create(
            command_interface  => $test_node,
            volume             => $volume_name,
            vserver            => $vserver_name,
            nacltask_if_exists => $nacltask_if_exists,
            nacltask_wait      => 1,
            state              => 'online',
            'junction-path'    => $junction_path,
            aggregate          => $aggr_name,
            'unix-permissions' => $unix_permissions,
            'security-style'   => $security_style,
            size               => $volume_size
        );
    }
    else {
        $Log->debug(
            "Create volume $volume_name with junction-path $junction_path");
        $test_volume = NACL::STask::Volume->create(
            command_interface  => $test_node,
            volume             => $volume_name,
            vserver            => $vserver_name,
            nacltask_if_exists => $nacltask_if_exists,
            nacltask_wait      => 1,
            state              => 'online',
            'junction-path'    => $junction_path,
            aggregate          => $aggr_name,
            'security-style'   => $security_style,
            size               => $volume_size
        );
    }

    return $test_volume;

}
###############################################################################
# Subroutine  : create_share
# Input       : hash of share parameters
#            'vserver'                      # mandatory
#            'share-name'                   # mandatory
#            'path'                         # mandatory
#            'nacltask_if_exists'           # optional
#            'share-properties'             # optional
#            'nacltask_replay_timeout'      # optional
#            'nacltask_replay_interval'     # optional
#            'symlink-properties'           # optional
# Return      : share object - success
# Description : Creates a cifs share.
###############################################################################
sub create_share {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'                  => 1,    # mandatory
            'share-name'               => 1,    # mandatory
            'path'                     => 1,    # mandatory
            'nacltask_if_exists'       => 0,    # optional
            'share-properties'         => 0,    # optional
            'nacltask_replay_timeout'  => 0,    # optional
            'nacltask_replay_interval' => 0,    # optional
            'symlink-properties'       => 0,    # optional
        }
    );

    my $test_node          = $self->{nodes}->[0];
    my $vserver_name       = $args{'vserver'} || $self->{vserver_names}->[0];
    my $share_name         = $args{'share-name'};
    my $path               = $args{'path'};
    my $nacltask_if_exists = $args{'nacltask_if_exists'} || 'purge';
    my $nacltask_replay_timeout  = $args{'nacltask_replay_timeout'} || 124;
    my $nacltask_replay_interval = $args{'nacltask_replay_interval'} || 7;
    my $share_properties         = $args{'share-properties'}
      || [ 'oplocks', 'browsable', 'changenotify' ];
    my $symlink_properties = $args{'symlink-properties'} || ['enable'];

    my $test_share;

    $Log->debug("Here are the current share on vserver $vserver_name");

    NACL::CS::VserverCifsShare->fetch(
        command_interface => $test_node,
        filter            => { vserver => $vserver_name },
        requested_fields  => [qw(share-name path share-properties)]
    );

    # Share create
    $Log->debug("Find / Create cifs share $share_name");
    try {
        $test_share = NACL::STask::VserverCifsShare->create(
            command_interface        => $test_node,
            vserver                  => $vserver_name,
            'share-name'             => $share_name,
            path                     => $path,
            'share-properties'       => $share_properties,
            'symlink-properties'     => $symlink_properties,
            nacltask_replay_timeout  => $nacltask_replay_timeout,
            nacltask_if_exists       => $nacltask_if_exists,
            nacltask_replay_interval => $nacltask_replay_interval,
        );
    }
    catch NACL::Exceptions::ReplayNotComplete with {
        $Log->warn(
            "Replay not completed for cifs share $share_name creation: ");
        $test_share = NACL::STask::VserverCifsShare->create(
            command_interface        => $test_node,
            vserver                  => $vserver_name,
            'share-name'             => $share_name,
            path                     => $path,
            'share-properties'       => $share_properties,
            nacltask_replay_timeout  => $nacltask_replay_timeout,
            nacltask_if_exists       => $nacltask_if_exists,
            nacltask_replay_interval => $nacltask_replay_interval,
        );
    };

    return $test_share;

}
###############################################################################
# Subroutine  : map_cifs_share
# Input       : hash of parameters required to share cifs share
# Return      : mount point object - success
# Description : This subroutine maps cifs share on windows client
###############################################################################
sub map_cifs_share {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'            => 1,    # mandatory
            'share'              => 1,    # mandatory
            'winclient'          => 1,    # mandatory
            'server'             => 0,    # mandatory
            'nacltask_if_exists' => 0,    # optional
            'nacltask_verify'    => 0,    # optional
            'domain'             => 0,    # optional
            'ad_server_user'     => 0,    # optional
            'ad_server_password' => 0,    # optional
            'retries'            => 0,    # optional
            'cleanup_client'     => 0,    # optional
            'ping_test'          => 0,    # optional
            'address_family'     => 0,    # optional

        }
    );

    my $domain         = $args{'domain'}         || $Params->{CIFS_DOMAIN};
    my $ad_server_user = $args{'ad_server_user'} || $Params->{AD_SERVER_USER};
    my $ad_server_password = $args{'ad_server_password'}
      || $Params->{AD_SERVER_PASSWORD};

    my $test_node          = $self->{nodes}->[0];
    my $vserver_name       = $args{'vserver'} || $self->{vserver_names}->[0];
    my $share_name         = $args{'share'};
    my $cifs_server        = $args{'server'};
    my $cifs_user          = $args{'user'} || $domain . '\\' . $ad_server_user;
    my $cifs_passwd        = $args{'password'} || $ad_server_password;
    my $nacltask_if_exists = $args{'nacltask_if_exists'} || 'purge';
    my $nacltask_verify    = $args{'nacltask_verify'} || 1;
    my $retries            = $args{'retries'} || 10;
    my $win_client         = $args{'winclient'};
    my $client_cleanup     = $args{'cleanup_client'} || undef;
    my $ping_test          = $args{'ping_test'} || undef;
    my $address_family     = $args{'address_family'} || 'ipv4';

    if ( defined $client_cleanup ) {
        $self->delete_mapping(
            'winclient' => $win_client,      # mandatory
            'vserver'   => $vserver_name,    # mandatory
        );
    }

    if ( defined $ping_test ) {
        my $net_interfaces =
          $self->get_network_interfaces( $vserver_name, 'no_refresh' );
        ### ping data lifs
        logcomment("Verify network ping to vserver $vserver_name data-lif");
        foreach my $lif (@$net_interfaces) {
            my $lif_address = $lif->get_one_state_attribute('address');
            if ( Net::IP::ip_is_ipv4($lif_address) ) {
                $self->ping_test($lif_address);
            }
            elsif ( Net::IP::ip_is_ipv6($lif_address) ) {
                $self->ping6_test($lif_address);
            }
        }

        ### ping the client in use by name
        logcomment( "Verify network ping to client: "
              . $win_client->client()
              . "with ip address "
              . $win_client->client_ip() );
        my $client_ip = $win_client->client_ip();
        if ( Net::IP::ip_is_ipv4($client_ip) ) {
            $self->ping_test($client_ip);
        }
        elsif ( Net::IP::ip_is_ipv6($client_ip) ) {
            $self->ping6_test($client_ip);
        }
    }

    if ( defined $cifs_server ) {
        my $output = $win_client->execute( command => "net view $cifs_server" );
        if ( $output !~ m/Shared resources at $cifs_server/ ) {
            $Log->warn( "net view command failed with error " . $output );
        }
    }

    $Log->debug("Validate user $cifs_user");

    my $translate_obj = $self->secd_auth_translate(
        vserver    => $vserver_name,
        'win-name' => $cifs_user,
    );
    logcomment( "SID for cifs user $cifs_user: " . $translate_obj->sid() );

    #    $self->secd_auth_show_creds(
    #        vserver    => $vserver_name,
    #        'win-name' => $cifs_user,
    #    );

    my ($mount_point);

    #Map cifs share
    $Log->debug("Map cifs share $share_name");
    my %mount_params = (
        'share'                    => $share_name,
        'server_command_interface' => $test_node,
        vserver                    => $vserver_name,
        nacltask_if_exists         => $nacltask_if_exists,
        nacltask_verify            => $nacltask_verify,
        retries                    => $retries,
        nacltask_address_family    => $address_family,
        server                     => $cifs_server,
        user                       => $cifs_user,
        password                   => $cifs_passwd
    );

    $mount_point = NACL::STask::Client::Mount->create(
        command_interface => $win_client,
        %mount_params,
    );

    # Get drive letter
    my $mount_state = $mount_point->state();
    my $drive       = $mount_state->mount_point();
    logcomment("CIFS share $share_name is mapped on drive $drive on client ".$win_client->client());

    return $mount_point;

}
###############################################################################
# Subroutine  : delete_mapping
# Input       : windows client nacl object, vserver name
# Return      : None
# Description : This subroutine finds any share mapping for the given vserver and
#               removes it from the client in use.
###############################################################################
sub delete_mapping {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'winclient' => 1,    # mandatory
            'vserver'   => 1,    # mandatory
        }
    );

    my $win_client   = $args{'winclient'};
    my $vserver_name = $args{'vserver'};

    my $server_list    = $self->get_cifs_server($vserver_name);
    my $net_interfaces = $self->get_network_interfaces($vserver_name);

    my ( @servers, @server_ips );

    foreach my $cifs_obj (@$server_list) {
        my $cifs_state       = $cifs_obj->state();
        my $cifs_server_name = $cifs_state->cifs_server();
        push @servers, $cifs_server_name;
    }

    foreach my $lif (@$net_interfaces) {
        my $lif_address = $lif->get_one_state_attribute('address');
        push @server_ips, $lif_address;
    }

    my $ontap_server_list = join( '|', @servers );
    logcomment( "Servers: " . $ontap_server_list );
    my $ontap_server_ip_list = join( '|', @server_ips );
    logcomment( "Server IP's: " . $ontap_server_ip_list );

    logcomment( "Excluding x: drive delete all existing "
          . "mapping on windows client "
          . $win_client->client() );
    my $output = $win_client->execute( command => "net use" );
    while (
        $output =~ /.*([A-Z]:).*($ontap_server_list|$ontap_server_ip_list)/i )
    {
        $output = $win_client->execute( command => "net use $1 /DELETE /Y" );
        if ( $output !~ /$1 was deleted successfully/i ) {
            $Log->warn(
                    "Failed to delete mapped drive $1 from the Windows client."
                  . "\n\nDelete all the mapped shares of $ontap_server_list "
                  . "Filer from the Windows client and rerun the script." );
        }
        $output = $win_client->execute( command => "net use" );
    }
}
###############################################################################
# Subroutine  : ping_test
# Input       : ping destination (accepts all values which are valid on ONTAP)
# Return      : None
# Description : This subroutine perform ping test on ipv4 destination
###############################################################################
sub ping_test {

    my $self = shift;
    my ($destination) = @_;

    my $test_node = $self->{nodes}->[0];
    my ($ping_state);

    $ping_state = NACL::C::Network->ping(
        command_interface => $test_node,
        destination       => $destination,
        node              => $test_node->node(),
        verbose           => 'true',
        'show-detail'     => 'true',
        'count'           => 5,
    );

}

###############################################################################
# Subroutine  : ping6_test
# Input       : ping6 destination (accepts all values which are valid on ONTAP)
# Return      : None
# Description : This subroutine perform ping test on ipv6 destination
###############################################################################
sub ping6_test {

    my $self = shift;
    my ($v6_destination) = @_;

    my $test_node = $self->{nodes}->[0];

    my $ping6_state = NACL::C::Network->ping6(
        command_interface => $test_node,
        destination       => $v6_destination,
        node              => $test_node->node(),
        verbose           => 'true',
        'show-detail'     => 'true',
        'count'           => 5,
    );

}
###############################################################################
# Subroutine  : secd_auth_translate
# Input       : hash of paramters accepted by diag secd authentication translate
#               ontap command [required]
#               e.g.
#               my $translate_obj = $commonlibobject->secd_auth_translate(
#                                       vserver    => $vserver_name,
#                                       'win-name' => $cifs_user,
#                                   );
# Return      : translate object - success
# Description : This subroutine executed diag secd authentication translate
#               command
###############################################################################
sub secd_auth_translate {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'         => 1,    # mandatory
            'uid'             => 0,
            'gid'             => 0,
            'sid'             => 0,
            'unix-user-name'  => 0,
            'unix-group-name' => 0,
            'win-name'        => 0,
        }
    );

    my $test_node = $self->{nodes}->[0];

    logcomment("Run the diag secd authentication translate command");
    my $obj;

    $obj = NACL::C::DiagSecdAuthentication->translate(
        command_interface => $test_node,
        node              => $test_node->node(),
        %args,
    );

    return $obj;
}
###############################################################################
# Subroutine  : secd_auth_show_creds
# Input       : hash of paramters accepted by diag secd authentication show-creds
#               ontap command [required]
#               e.g.
#               my $translate_obj = $commonlibobject->secd_auth_show_creds(
#                                       vserver    => $vserver_name,
#                                       'win-name' => $cifs_user,
#                                   );
# Return      : none - success
# Description : This subroutine executed diag secd authentication show-creds
#               command
###############################################################################
sub secd_auth_show_creds {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'        => 1,    # mandatory
            'uid'            => 0,
            'list-name'      => 0,
            'sid'            => 0,
            'unix-user-name' => 0,
            'list-id'        => 0,
            'win-name'       => 0,
        }
    );

    my $test_node = $self->{nodes}->[0];

    logcomment("Run the diag secd authentication show-creds command");

    NACL::C::DiagSecdAuthentication->show_creds(
        command_interface => $test_node,
        node              => $test_node->node(),
        %args,
    );

}

###############################################################################
# Subroutine  : secd_cache_clear
# Input       : vserver name , secd cache-name
# Return      : None
# Description : subroutine is used to clear a desired secd cahce on a vserver
###############################################################################

sub secd_cache_clear {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'    => 1,    # mandatory
            'cache-name' => 1,    # mandatory
        }
    );

    my $test_node = $self->{nodes}->[0];

    $Log->debug(
        "Clear secd cache $args{'cache-name'} on vserver $args{'vserver'}");

    try {
        NACL::C::DiagSecdCache->clear(
            command_interface => $test_node,
            node              => $test_node->node(),
            'cache-name'      => $args{'cache-name'},
            vserver           => $args{'vserver'},
        );
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->warn("Missing cifs server :In the BaseException catch");
        $Log->comment( "Caught a " . ref($exception_object) . "!" );
        $Log->comment( "Error is: " . $exception_object->text() );
    };
}

###############################################################################
# Subroutine  : create_fileops_object
# Input       : hash (client=><windows client nacl object>[required])
# Return      : fileops object - success
# Description : This subroutine creates a Protocol::ClientFileOperations object
#               for a given windows client
###############################################################################
sub create_fileops_object {

    my $self = shift;
    my %args =
      Params::Validate::validate( @_,
        { client => { isa => 'NACL::C::Client' }, },
      );

    my $fileops_obj;

    $Log->debug("Create a new Protocol::ClientFileOperations fileops object");
    $fileops_obj =
      new Protocol::ClientFileOperations( client => $args{client}, );

    my $client = $args{client}->client();
    $self->{fileops}{$client} = $fileops_obj;

    return $self->{fileops}{$client};

}
###############################################################################
# Subroutine  : create_directory
# Input       : hash (client=><windows client nacl object> [required],
#                     dir_name=><dir path(e.g. z:\myhome)>[required])
# Return      : 1 - success
# Description : This subroutine create a directory
###############################################################################
sub create_directory {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'client' => { isa => 'NACL::C::Client' },    # mandatory
            'dir_name' => 1,
        }
    );

    my $client;
    $client = $args{client}->client();

    my $proto_fileops = $self->{fileops}{$client};

    $Log->debug( "Create directory " . $args{dir_name} );
    $proto_fileops->mkdir( 'directory_name' => $args{dir_name} );

    return 1;
}

###############################################################################
# Subroutine  : file_ops
# Input       :  
########################OPEN FILE##############################################
#
#                 a. Input to open a new file
#
#                 Required arguments to open a file are:
#
#                 file_name
#                 access_mode
#                       valid values : read, write, read-write
#                 open_mode
#                       valid values : open-existing, open-always, 
#                                      create-new, create-always, 
#                                      truncate-existing, open-append
#                 permissions - unix only ((4 bits) for the file being opened)
#
#                 Optional arguments
#
#                 share_mode - windows only
#                       valid values are:  shared-delete, shared-read, 
#                                          shared-write, shared-read-delete, 
#                                          shared-write-delete, shared-read-write,
#                                          shared-read-write-delete, none
#                e.g. (windows)
#
#                $Common_Lib->file_ops(
#                    'client'    => $Win_Client_1,
#                    file_name   => $win_file_name,
#                    access_mode => $access_mode,
#                    open_mode   => $open_mode,
#                );
# 
#                e.g. (unix)
#                $Common_Lib->file_ops(
#                    'client'    => $Unix_Client,
#                    file_name   => $unix_file1,
#                    access_mode => $access_mode,
#                    open_mode   => $open_mode,
#                    close_file  => 1,
#                    permissions => '0777',
#                );
########################READ FILE##############################################
#                b. Input to read a file
#
#                   Required arguments to read a file are:
#                   read_file => 1
#                   offset
#                   num_bytes
#
#                   e.g.
#
#                $Common_Lib->file_ops(
#                    'client'    => $Win_Client_2,
#                    file_name   => $win_file_name2,
#                    access_mode => $access_mode2,
#                    open_mode   => $open_mode2,
#                    read_file   => 1,
#                    close_file  => 1,
#                    num_bytes   => 22,
#                    offset      => 0,
########################WRITE FILE##############################################
#                c. Input to write to a file
#
#                   Required arguments to write to a file are:
#                   write_file => 1 
#                   offset
#                   contents
#
#                e.g.
#
#                $Common_Lib->file_ops(
#                    'client'    => $Win_Client_1,
#                    file_name   => $win_file_name,
#                    access_mode => $access_mode,
#                    open_mode   => $open_mode,
#                    write_file  => 1,
#                    close_file  => 1,
#                    offset      => 0,
#                    contents    => "Testing LUG testcase 1",
#                );
#
# Return      : Reference of hash structure containing:
#               1. file handle object
#               2. number of bytes written (if write_file = 1)
#               3. contents read from the file (if read_file = 1)
#
# Description : This subroutine perform multiple file ops 
#               (e.g. create, read, write and close)
#
#               Do not forget to create a Protocol::ClientFileOperations 
#               object for the client you plan to use
#
#               This subroutine is a limited wrapper for functions inside
#               Protocol::ClientFileOperations
#
#               File handle object which is returned from this function 
#               can be used on all functions inside 
#               a. Protocol::ClientFileOperations.pm
#               b. Protocol::ClientFileHandle.pm
#
###############################################################################

sub file_ops {

    my $self = shift;
    my (%args) = @_;

    my $client        = $args{client}->client();
    my $proto_fileops = $self->{fileops}{$client};

    $Log->debug(
"Open file $args{file_name} with access_mode $args{'access_mode'} and open mode $args{'open_mode'} "
    );

    my ( %return_data, $fh_obj, $bytes_written, $contents );

    if ( $args{client}->is_windows() ) {
        $fh_obj = $proto_fileops->open(
            file_name   => $args{file_name},
            access_mode => $args{access_mode},
            open_mode   => $args{open_mode},
            share_mode  => $args{share_mode} || 'shared-read-write',
        );
    }
    else {
        $fh_obj = $proto_fileops->open(
            file_name   => $args{file_name},
            access_mode => $args{access_mode},
            open_mode   => $args{open_mode},
            permissions => $args{permissions},
        );
    }

    $return_data{'file_handle'} = $fh_obj;

    if (   defined( $args{'write_file'} )
        && $args{'write_file'} == 1
        && defined($fh_obj) )
    {

        $Log->debug(
"Offset to write in file $args{offset} and file contents are $args{'contents'}"
        );
        $bytes_written = $fh_obj->write(
            offset   => $args{offset},
            contents => $args{contents},
        );
        logcomment( "Number of bytes written in $args{'file_name'}: "
              . $bytes_written );
    }

    $return_data{'bytes_written'} = $bytes_written;

    if (   defined( $args{'read_file'} )
        && $args{'read_file'} == 1
        && defined($fh_obj) )
    {
        $Log->debug("Read file $args{file_name}");
        $contents = $fh_obj->read(
            offset    => $args{offset},
            num_bytes => $args{num_bytes},
        );
        logcomment( "Contents of $args{'file_name'}: " . $contents );

    }

    $return_data{'read_contents'} = $contents;

    if (   defined( $args{'close_file'} )
        && $args{'close_file'} == 1
        && defined($fh_obj) )
    {

        $Log->debug("Close file $args{'file_name'}");
        $proto_fileops->close( file_handle => $fh_obj );
    }

    return \%return_data;
}

###############################################################################
# Subroutine  : delete_file
# Input       : hash (client=><windows client nacl object> [required],
#                     dir_name=><dir path(e.g. z:\myhome)>[required])
# Return      : 1 - success
# Description : This subroutine delete a file
###############################################################################

sub delete_file {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'file_name' => 1,                            # mandatory
            'client' => { isa => 'NACL::C::Client' },    # mandatory
        }
    );

    my $client        = $args{client}->client();
    my $proto_fileops = $self->{fileops}{$client};
    my $file_name     = $args{'file_name'};
    $Log->debug("Delete file $file_name");
    $proto_fileops->delete( file_name => $file_name );

}

###############################################################################
# Subroutine  : delete_directory
# Input       : hash (client=><windows client nacl object>,
#                     dir_name=><dir path(e.g. z:\myhome)>)
# Return      : 1 - success
#               exception_object - failure
# Description : This subroutine delete a directory
###############################################################################
sub delete_directory {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'dir_name' => 1,                             # mandatory
            'client' => { isa => 'NACL::C::Client' },    # mandatory
        }
    );

    my $client;
    $client = $args{client}->client();

    my $proto_fileops = $self->{fileops}{$client};

    $Log->debug( "Delete directory " . $args{dir_name} );
    $proto_fileops->rmdir( 'directory_name' => $args{dir_name} );

    return 1;
}

###############################################################################
# Subroutine  : create_name_mapping
# Input       : hash required for name mapping
#                direction (required)
#                pattern (required)
#                replacement (required)
#                position(optional)
# Return      : name mapping nacl object - success
# Description : This subroutine create vserver cifs name-mapping
###############################################################################

sub create_name_mapping {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'direction'   => 1,    # mandatory
            'vserver'     => 1,    # mandatory
            'pattern'     => 1,    # mandatory
            'replacement' => 1,    # mandatory
            'position'    => 0,    # optional
        }
    );

    my $test_node = $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];

    my $direction = $args{'direction'};
    my $position = $args{'position'} || $self->find_name_map_postion(
        'vserver'   => $vserver_name,
        'direction' => $direction,
    );
    my $pattern     = $args{'pattern'};
    my $replacement = $args{'replacement'};

    $Log->debug("Create a new vserver name-mapping at position $position ");
    my $name_mapping_obj;

    $name_mapping_obj = NACL::C::VserverNameMapping->create(
        command_interface => $test_node,
        vserver           => $vserver_name,
        direction         => $direction,
        position          => $position,
        pattern           => $pattern,
        replacement       => $replacement,
    );

    $Log->debug(
        "Here are the updated name-mapping list for vserver $vserver_name ");

    my @name_mapping = NACL::C::VserverNameMapping->find(
        command_interface => $test_node,
        filter            => { vserver => $vserver_name, },
    );

    $self->{name_mapping}{$vserver_name} = [@name_mapping];

    return $name_mapping_obj;

}

###############################################################################
# Subroutine  : find_name_map_postion
# Input       : hash containing following key, value pair
#               1. vserver => <vserver name>
#               2. direction => <win-unix |unix-win>
# Return      : position
# Description : This subroutine find a new empty position required to
#               create a new name-mapping
###############################################################################
sub find_name_map_postion {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'direction' => 1,    # mandatory
            'vserver'   => 1,    # mandatory
        }
    );

    my $test_node    = $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'};
    my $direction    = $args{'direction'};
    my @occupied_position = ();

    $Log->debug( "Let's find a position to create a name-mapping for "
          . "vserver $vserver_name in direction $direction" );
    my (@name_map_cs,$new_position);

    try {
        @name_map_cs = NACL::CS::VserverNameMapping->fetch(
            command_interface => $test_node,
            filter            => {
                vserver     => $vserver_name,
                'direction' => $direction
            },
            requested_fields => [qw(direction position pattern replacement)],
        );
        foreach my $state (@name_map_cs) {
            push @occupied_position, $state->position();
        }
        $new_position = findMissingPositive(@occupied_position);
    }
    catch NACL::Exceptions::NoElementsFound with {
        $new_position = findMissingPositive(@occupied_position);
    };

    return $new_position;
}

###############################################################################
# Subroutine  : findMissingPositive
# Input       : @array with occupied position
# Return      : minimum positive integer
# Description : This subroutine finds minimum positive integer
###############################################################################
sub findMissingPositive {
    my (@arr) = @_;

    my $i;
    my $size = scalar(@arr);

    for ( $i = 0 ; $i < $size ; $i++ ) {
        if ( abs( $arr[$i] ) - 1 < $size && $arr[ abs( $arr[$i] ) - 1 ] > 0 ) {
            $arr[ abs( $arr[$i] ) - 1 ] = -1 * $arr[ abs( $arr[$i] ) - 1 ];
        }
    }

    for ( $i = 0 ; $i < $size ; $i++ ) {
        if ( $arr[$i] > 0 ) {
            return $i + 1;
        }
    }

    return $size+1;
}

###############################################################################
# Subroutine  : get_network_interfaces
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing netowrk interfaces nacl objects
# Description : This subroutine returns referance of array
#               containing netowrk interfaces nacl objects for given vserver
###############################################################################

sub get_network_interfaces {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{data_lifs}{$vserver};
    }
    else {
        logcomment("Network interfaces on vserver $vserver");

        my @lifs = NACL::STask::NetworkInterface->find(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver, },
        );

        $self->{data_lifs}{$vserver} = [@lifs];

        ##show lif data in logs

        NACL::CS::NetworkInterface->fetch(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver },
            requested_fields  => [
                qw(vserver server lif role home-node home-port
                  curr-node curr-port status-oper is-home
                  address netmask netmask-length status-admin
                  data-protocol)
            ],
        );

        return $self->{data_lifs}{$vserver};
    }
}
###############################################################################
# Subroutine  : get_v4_network_interfaces
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing v4 netowrk interfaces nacl objects
# Description : This subroutine returns referance of array
#               containing netowrk interfaces nacl objects for given vserver
###############################################################################

sub get_v4_network_interfaces {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{data_lifs_v4}{$vserver};
    }
    else {
        logcomment("Network interfaces on vserver $vserver");
        try {
            my @lifs = NACL::STask::NetworkInterface->find(
                command_interface => $self->{nodes}->[0],
                filter            => {
                    vserver          => $vserver,
                    'address-family' => 'ipv4',
                },
            );

            $self->{data_lifs_v4}{$vserver} = [@lifs];
        }
        catch NACL::Exceptions::NoElementsFound with {

        };

        ##show lif data in logs

        NACL::CS::NetworkInterface->fetch(
            command_interface => $self->{nodes}->[0],
            filter            => {
                vserver          => $vserver,
                'address-family' => 'ipv4',
            },
            requested_fields => [
                qw(vserver server lif role home-node home-port
                  curr-node curr-port status-oper is-home
                  address netmask netmask-length status-admin
                  data-protocol ipspace)
            ],
        );

        return $self->{data_lifs_v4}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_v6_network_interfaces
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing v6 netowrk interfaces nacl objects
# Description : This subroutine returns referance of array
#               containing netowrk interfaces nacl objects for given vserver
###############################################################################

sub get_v6_network_interfaces {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{data_lifs_v6}{$vserver};
    }
    else {
        logcomment("Network interfaces on vserver $vserver");

        my @lifs = NACL::STask::NetworkInterface->find(
            command_interface => $self->{nodes}->[0],
            filter            => {
                vserver          => $vserver,
                'address-family' => 'ipv6',
            },
        );

        $self->{data_lifs_v6}{$vserver} = [@lifs];

        ##show lif data in logs

        NACL::CS::NetworkInterface->fetch(
            command_interface => $self->{nodes}->[0],
            filter            => {
                vserver          => $vserver,
                'address-family' => 'ipv6',
            },
            requested_fields => [
                qw(vserver server lif role home-node home-port
                  curr-node curr-port status-oper is-home
                  address netmask netmask-length status-admin
                  data-protocol ipspace)
            ],
        );

        return $self->{data_lifs_v6}{$vserver};
    }
}
###############################################################################
# Subroutine  : get_network_routes
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing network routes nacl objects
# Description : This subroutine returns referance of array
#               containing network routes nacl objects for given vserver
###############################################################################

sub get_network_routes {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{network_route}{$vserver};
    }
    else {
        logcomment("Network Route on vserver $vserver");

        my @network_route = NACL::C::NetworkRoute->find(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver, }
        );

        $self->{network_route}{$vserver} = [@network_route];

        return $self->{network_route}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_dns_entries
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing dns entries nacl objects
# Description : This subroutine returns referance of array
#               containing dns entries nacl objects for given vserver
###############################################################################

sub get_dns_entries {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{dns}{$vserver};
    }
    else {
        logcomment("DNS entries on vserver $vserver");

        my $vserver_dns = NACL::C::VserverServicesDns->find(
            command_interface => $self->{nodes}->[0],
            filter            => {
                vserver => $vserver,
                state   => 'enabled',
            }
        );

        $self->{dns}{$vserver} = [$vserver_dns];

        return $self->{dns}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_unix_group
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing unix group nacl objects
# Description : This subroutine returns referance of array
#               containing unix group nacl objects for given vserver
###############################################################################

sub get_unix_group {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{unix_group}{$vserver};
    }
    else {
        logcomment("Unix group on vserver $vserver");
        try {
            my @unix_group = NACL::C::UnixGroup->find(
                command_interface => $self->{nodes}->[0],
                filter            => { vserver => $vserver, },
            );

            $self->{unix_group}{$vserver} = [@unix_group];
        }
        catch NACL::Exceptions::NoElementsFound with {

        };

        ##show unix group in logs

        NACL::CS::VserverServicesUnixGroup->fetch(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver },
            requested_fields  => [qw(name id users)],
        );

        return $self->{unix_group}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_unix_user
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing unix user nacl objects
# Description : This subroutine returns referance of array
#               containing unix user nacl objects for given vserver
###############################################################################

sub get_unix_user {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{unix_user}{$vserver};
    }
    else {
        logcomment("Unix user on vserver $vserver");
        try {
            my @unix_user = NACL::C::UnixUser->find(
                command_interface => $self->{nodes}->[0],
                filter            => { vserver => $vserver, },
            );

            $self->{unix_user}{$vserver} = [@unix_user];
        }
        catch NACL::Exceptions::NoElementsFound with {

        };

        ##show unix user in logs

        NACL::CS::VserverServicesUnixUser->fetch(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver },
            requested_fields  => [qw(id primary-gid group)],
        );

        return $self->{unix_user}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_cifs_options
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing cifs options nacl objects
# Description : This subroutine returns referance of array
#               containing cifs options nacl objects for given vserver
###############################################################################

sub get_cifs_options {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{cifs_options}{$vserver};
    }
    else {
        logcomment("CIFS options on vserver $vserver");

        my $cifs_options = NACL::C::VserverCifsOptions->find(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver, },
        );

        $self->{cifs_options}{$vserver} = [$cifs_options];

        return $self->{cifs_options}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_export_policy
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing export policy nacl objects
# Description : This subroutine returns referance of array
#               containing export policy nacl objects for given vserver
###############################################################################

sub get_export_policy {

    my $self = shift;

    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{export_policy}{$vserver};
    }
    else {
        logcomment("Export policy on vserver $vserver");

        my @export_policy = NACL::C::VserverExportPolicy->find(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver, },
        );

        $self->{export_policy}{$vserver} = [@export_policy];

        return $self->{export_policy}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_export_policy_rule
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing export policy rule nacl objects
# Description : This subroutine returns referance of array
#               containing export policy rule nacl objects for given vserver
###############################################################################

sub get_export_policy_rule {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{export_policy_rule}{$vserver};
    }
    else {
        logcomment("Export policy Rule on vserver $vserver");

        my @export_policy_rule = NACL::C::VserverExportPolicyRule->find(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver, },
        );

        $self->{export_policy_rule}{$vserver} = [@export_policy_rule];

        ##show export-policy rule show in logs

        NACL::CS::VserverExportPolicyRule->fetch(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver },
            requested_fields  => [
                qw(policyname ruleindex protocol clientmatch rorule rwrule anon superuser)
            ],
        );

        return $self->{export_policy_rule}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_name_mapping
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing name mapping nacl objects
# Description : This subroutine returns referance of array
#               containing name mapping nacl objects for given vserver
###############################################################################

sub get_name_mapping {

    my $self = shift;
    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{name_mapping}{$vserver};
    }
    else {
        logcomment("Name-mapping on vserver $vserver");

        my @name_mapping = NACL::C::VserverNameMapping->find(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver, },
        );

        $self->{name_mapping}{$vserver} = [@name_mapping];

        NACL::CS::VserverNameMapping->fetch(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver },
        );

        return $self->{name_mapping}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_cifs_server
# Input       : vserver name - required
#               option ('no_refresh') - optional
# Return      : referance of array containing cifs server nacl objects
# Description : This subroutine returns referance of array
#               containing cifs server nacl objects for given vserver
###############################################################################

sub get_cifs_server {

    my $self = shift;

    my ( $vserver, $option ) = validate_pos( @_, { type => SCALAR }, 0 );

    if ( defined $option && $option =~ m/no_refresh/i ) {
        return $self->{cifs_server}{$vserver};
    }
    else {
        logcomment("CIFS server on vserver $vserver");

        my $cifs_server = NACL::C::VserverCifs->find(
            command_interface => $self->{nodes}->[0],
            filter            => { vserver => $vserver, },
        );

        $self->{cifs_server}{$vserver} = [$cifs_server];

        return $self->{cifs_server}{$vserver};
    }
}

###############################################################################
# Subroutine  : get_unix_clients
# Input       : none
# Return      : Array ref of client objects of Unix/Linux clients
# Description : Returns a list of Linux/Unix clients.
###############################################################################
sub get_unix_clients {

    my $self = shift;
    return $self->{unix_clients};

} ## end sub get_unix_clients

###############################################################################
# Subroutine  : get_win_clients
# Input       : none
# Return      : Array ref of client objects of Windows clients
# Description : Returns a list of Windows clients.
###############################################################################
sub get_win_clients {

    my $self = shift;
    return $self->{windows_clients};
} ## end sub get_win_clients

###############################################################################
# Subroutine  : _get_hostrec
# Description : Gets a hostrec for a host name.
###############################################################################
sub _get_hostrec {
    my $host = shift;
    return ref($host) ? $host : Tharn::host($host);
}

## subroutine for lug project

################################################################################
## Subroutine  : create_local_user
## Input       : vserver name, username, password, description(optional), full name(optional)
## Return      : local user object
## Description : Create a local user
################################################################################
sub create_local_user {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'     => 1,    # mandatory
            'user-name'   => 1,    # mandatory
            'password'    => 1,    # mandatory
            'description' => 0,    # optional
            'full-name'    => 0,   # optional
            'use-existing' => 0,   # optional
        }
    );

    my $test_node        = $self->{nodes}->[0];
    my $vserver_name     = $args{'vserver'};
    my $local_user_name  = $args{'user-name'};
    my $local_user_psswd = $args{'password'};
    my $description      = $args{'description'} || "\"Local User\"";
    my $full_name        = $args{'full-name'} || "\"Full Name\"";
    my $use_existing     = $args{'use-existing'} || 0;
    my $new_local_user;

    $Log->debug(
"Creating a new local user on vserver $vserver_name with name $local_user_name"
    );

    try {
        $new_local_user = NACL::C::VserverCifsUsersAndGroupsLocalUser->find(
              'command_interface' => $test_node,
              filter              => {
                  'vserver'           => $vserver_name,
                  'user-name'         => $local_user_name,
              }
        );
        if (!$use_existing) {
            $new_local_user->delete();
            $new_local_user = NACL::C::VserverCifsUsersAndGroupsLocalUser->create(
                'command_interface' => $test_node,
                'vserver'           => $vserver_name,
                'user-name'         => $local_user_name,
                'password'          => $local_user_psswd,
                'description'       => $description,
                'full-name'         => $full_name,
            );
        } 
    }
    catch NACL::Exceptions::NoElementsFound with {
        $new_local_user = NACL::C::VserverCifsUsersAndGroupsLocalUser->create(
            'command_interface' => $test_node,
            'vserver'           => $vserver_name,
            'user-name'         => $local_user_name,
            'password'          => $local_user_psswd,
            'description'       => $description,
            'full-name'         => $full_name,
        );
    };

    $Log->debug("Show updated local-user list");

    NACL::CS::VserverCifsUsersAndGroupsLocalUser->fetch(
        'command_interface' => $test_node,
        filter              => { vserver => $vserver_name, },
        requested_fields    => [qw(user-name )],

    );

    return $new_local_user;

}
###############################################################################
# Subroutine  : create_local_group
# Input       : Vserver Name, local group name and local group name
# Return      : local user group object
# Description : Create local group
###############################################################################
sub create_local_group {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'     => 1,    # mandatory
            'group-name'  => 1,    # mandatory
            'description' => 0,    # optional
            'use-existing' => 0,   # optional
        }
    );

    my $test_node        = $self->{nodes}->[0];
    my $vserver_name     = $args{'vserver'};
    my $local_group_name = $args{'group-name'};
    my $description      = $args{'description'} || "\"Local Group\"";
    my $use_existing     = $args{'use-existing'} || 0;
    my $new_local_group;

    $Log->debug(
        "Create a local group $local_group_name on vserver $vserver_name");
    try {
        $new_local_group = NACL::C::VserverCifsUsersAndGroupsLocalGroup->find(
              'command_interface' => $test_node,
              filter              => {
                  'vserver'         => $vserver_name,
                  'group-name'      => $local_group_name,
              }
          );
        if (!$use_existing) {
            $new_local_group->delete();
            $new_local_group = NACL::C::VserverCifsUsersAndGroupsLocalGroup->create(
                command_interface => $test_node,
                'vserver'         => $vserver_name,
                'group-name'      => $local_group_name,
                'description'     => $description,
            );
        }
    }
    catch NACL::Exceptions::NoElementsFound with {
        $new_local_group = NACL::C::VserverCifsUsersAndGroupsLocalGroup->create(
            command_interface => $test_node,
            'vserver'         => $vserver_name,
            'group-name'      => $local_group_name,
            'description'     => $description,
        );
    };

    $Log->debug("Show updated local-group list");

    NACL::CS::VserverCifsUsersAndGroupsLocalGroup->fetch(
        'command_interface' => $test_node,
        filter              => { vserver => $vserver_name, },
        requested_fields    => [qw(group-name )],

    );

    return $new_local_group;
}

################################################################################
## Subroutine  : lug_add_privilege
## Input       : vserver name, local user or group name, priveleges
## Return      : none
## Description : Add privileges to local user or group
################################################################################
sub lug_add_privilege {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'user-or-group-name' => 1,    # mandatory
            'vserver'            => 1,    # mandatory
            'privileges'         => 1,    # mandatory
        }
    );

    my $test_node = $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];
    my $local_user_or_group_name = $args{'user-or-group-name'};
    my $privileges               = $args{'privileges'};

    $Log->debug(
"Add privileges $privileges to vserver $vserver_name local user $local_user_or_group_name"
    );

    NACL::C::VserverCifsUsersAndGroupsPrivilege->add_privilege(
        'command_interface'  => $test_node,
        'vserver'            => $vserver_name,
        'user-or-group-name' => $local_user_or_group_name,
        'privileges'         => $privileges,
    );

}

################################################################################
## Subroutine  : delete_local_user
## Input       : vserver name, user name
## Return      : none
## Description : Delete a local user
################################################################################
sub delete_local_user {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'user-name' => 1,    # mandatory
            'vserver'   => 1,    # mandatory
        }
    );

    my $test_node       = $self->{nodes}->[0];
    my $vserver_name    = $args{'vserver'} || $self->{vserver_names}->[0];
    my $local_user_name = $args{'user-name'};

    $Log->debug("Delete the user $local_user_name.");

    NACL::C::VserverCifsUsersAndGroupsLocalUser->delete(
        'command_interface' => $test_node,
        'vserver'           => $vserver_name,
        'user-name'         => $local_user_name,
    );

    $Log->debug("Show updated local-user list");

    NACL::CS::VserverCifsUsersAndGroupsLocalUser->fetch(
        'command_interface' => $test_node,
        filter              => { vserver => $vserver_name, },
        requested_fields    => [qw(user-name )],

    );

}

###############################################################################
# Subroutine  : delete_local_group
# Input       : Vserver Name and local group name
# Return      : None
# Description : Delete local user group
###############################################################################
sub delete_local_group {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'group-name' => 1,    # mandatory
            'vserver'    => 1,    # mandatory
        }
    );

    my $test_node        = $self->{nodes}->[0];
    my $vserver_name     = $args{'vserver'} || $self->{vserver_names}->[0];
    my $local_group_name = $args{'group-name'};

    $Log->debug("Delete local group $local_group_name");

    NACL::C::VserverCifsUsersAndGroupsLocalGroup->delete(
        'command_interface' => $test_node,
        'vserver'           => $vserver_name,
        'group-name'        => $local_group_name,
    );

    $Log->debug("Show updated local-group list");

    NACL::CS::VserverCifsUsersAndGroupsLocalGroup->fetch(
        'command_interface' => $test_node,
        filter              => { vserver => $vserver_name, },
        requested_fields    => [qw(group-name )],

    );

}

################################################################################
## Subroutine  : remove_privilege
## Input       : 
#            'user-or-group-name'     # mandatory
#            'vserver'                # mandatory
#            'privileges'             # mandatory
## Return      : none
## Description : Remove privileges from local user or group
################################################################################
sub lug_remove_privilege {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'user-or-group-name' => 1,    # mandatory
            'vserver'            => 1,    # mandatory
            'privileges'         => 1,    # mandatory
        }
    );

    my $test_node = $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];
    my $local_user_or_group_name = $args{'user-or-group-name'};
    my $privileges               = $args{'privileges'};

    $Log->debug(
"Remove privileges $privileges on vserver $vserver_name local user $local_user_or_group_name"
    );

    NACL::C::VserverCifsUsersAndGroupsPrivilege->remove_privilege(
        'command_interface'  => $test_node,
        'vserver'            => $vserver_name,
        'user-or-group-name' => $local_user_or_group_name,
        'privileges'         => $privileges,
    );

}

###############################################################################
# Subroutine  : local_group_add_members
# Input       : 
#            'group-name'       # mandatory
#            'vserver'          # mandatory
#            'member-names'     # mandatory
# Return      : None
# Description : This subroutine add members in a local group
###############################################################################
sub local_group_add_members {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'group-name'   => 1,    # mandatory
            'vserver'      => 1,    # mandatory
            'member-names' => 1,    # mandatory
        }
    );

    my $test_node        = $self->{nodes}->[0];
    my $vserver_name     = $args{'vserver'} || $self->{vserver_names}->[0];
    my $local_group_name = $args{'group-name'};
    my $members          = $args{'member-names'};

    $Log->debug("Adding member $members to the local group $local_group_name");

    NACL::C::VserverCifsUsersAndGroupsLocalGroup->add_members(
        'command_interface' => $test_node,
        'vserver'           => $vserver_name,
        'group-name'        => $local_group_name,
        'member-names'      => $members
    );

}

###############################################################################
# Subroutine  : local_group_remove_members
# Input       : 
#            'group-name'       # mandatory
#            'vserver'          # mandatory
#            'member-names'     # mandatory
# Return      : None
# Description : This subroutine removes members from a local group
###############################################################################
sub local_group_remove_members {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'group-name'   => 1,    # mandatory
            'vserver'      => 1,    # mandatory
            'member-names' => 1,    # mandatory
        }
    );

    my $test_node        = $self->{nodes}->[0];
    my $vserver_name     = $args{'vserver'} || $self->{vserver_names}->[0];
    my $local_group_name = $args{'group-name'};
    my $members          = $args{'member-names'};

    $Log->debug("Remove member $members to the local group $local_group_name");

    NACL::C::VserverCifsUsersAndGroupsLocalGroup->remove_members(
        'command_interface' => $test_node,
        'vserver'           => $vserver_name,
        'group-name'        => $local_group_name,
        'member-names'      => $members
    );

}

###############################################################################
# Subroutine  : security_file_directory_policy_create
# Input       : Local user group object, member to be added
# Return      : None
# Description : This subroutine adds members to a local group
###############################################################################

sub security_file_directory_policy_create {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'policy-name' => 1,    # mandatory
            'vserver'     => 1,    # mandatory
        }
    );

    my $vserver_name = $args{'vserver'};
    my $policy_name  = $args{'policy-name'};
    my $if_exists    = $args{'nacltask_if_exists'} || 'purge';

    $Log->debug("Create policy $policy_name");
    my $obj = NACL::STask::VserverSecurityFileDirectoryPolicy->create(
        command_interface  => $self->{nodes}->[0],
        vserver            => $vserver_name,
        'policy-name'      => $policy_name,
        nacltask_if_exists => $if_exists,
    );
}

###############################################################################
# Subroutine  : security_file_directory_ntfs_create
# Input       : 
#            'ntfs-sd'     # mandatory
#            'vserver'     # mandatory
# Return      : ntfs sd object
# Description : This subroutine creates a new ntfs sd
###############################################################################

sub security_file_directory_ntfs_create {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'ntfs-sd' => 1,    # mandatory
            'vserver' => 1,    # mandatory
        }
    );

    my $node         = $args{'node'}    || $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];
    my $if_exists = $args{'nacltask_if_exists'} || 'purge';

    $Log->debug("Create security descriptor $args{'ntfs-sd'}");
    my $obj = NACL::STask::VserverSecurityFileDirectoryNtfs->create(
        command_interface  => $node,
        vserver            => $vserver_name,
        nacltask_if_exists => $if_exists,
        %args
    );
}

###############################################################################
# Subroutine  : security_file_directory_policy_task_add
# Input       : 
#            'ntfs-sd'         # mandatory
#            'vserver'         # mandatory
#            'policy-name'     # mandatory
#            'path'            # mandatory
#            'index'           # mandatory
# Return      : task object
# Description : This subroutine adds policy task
###############################################################################

sub security_file_directory_policy_task_add {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'ntfs-sd'     => 1,    # mandatory
            'vserver'     => 1,    # mandatory
            'policy-name' => 1,    # mandatory
            'path'        => 1,    # mandatory
            'index'       => 1,    # mandatory
        }
    );

    my $node         = $args{'node'}    || $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];
    my $if_exists = $args{'nacltask_if_exists'} || 'purge';

    $Log->debug("Add policy task on path $args{'path'}");
    my $obj = NACL::STask::VserverSecurityFileDirectoryPolicyTask->add(
        command_interface  => $node,
        vserver            => $vserver_name,
        nacltask_if_exists => $if_exists,
        %args
    );
}

###############################################################################
# Subroutine  : security_file_directory_ntfs_dacl_add
# Input       : 
#            'ntfs-sd'         # mandatory
#            'vserver'         # mandatory
#            account           # mandatory
#            'access-type'     # mandatory
#            rights            # mandatory
#            'apply-to'        # mandatory
# Return      : ntfs dacl object
# Description : This subroutine is used to add ntfs dacl
###############################################################################

sub security_file_directory_ntfs_dacl_add {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'ntfs-sd'     => 1,    # mandatory
            'vserver'     => 1,    # mandatory
            account       => 1,    # mandatory
            'access-type' => 1,    # mandatory
            rights        => 1,    # mandatory
            'apply-to'    => 1,    # mandatory
        }
    );

    my $node         = $args{'node'}    || $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];
    my $if_exists = $args{'nacltask_if_exists'} || 'purge';

    $Log->debug("Add ntfs dacl");
    my $obj = NACL::STask::VserverSecurityFileDirectoryNtfsDacl->add(
        command_interface  => $node,
        vserver            => $vserver_name,
        nacltask_if_exists => $if_exists,
        %args
    );
}

###############################################################################
# Subroutine  : security_file_directory_ntfs_dacl_purge
# Input       : 
#            'ntfs-sd'         # mandatory
#            'vserver'         # mandatory
#            'account'         # mandatory
#            'access-type'     # mandatory
# Return      : None
# Description : This subroutine purge ntfs dacl
###############################################################################

sub security_file_directory_ntfs_dacl_purge {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'ntfs-sd'     => 1,    # mandatory
            'vserver'     => 1,    # mandatory
            account       => 1,    # mandatory
            'access-type' => 1,    # mandatory
        }
    );

    my $node         = $args{'node'}    || $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];

    $Log->debug("Purge ntfs dacl");
    my $obj = NACL::STask::VserverSecurityFileDirectoryNtfsDacl->purge(
        command_interface => $node,
        vserver           => $vserver_name,
        %args
    );

    $Log->debug("Show ntfs dacl");
    NACL::CS::VserverSecurityFileDirectoryNtfsDacl->fetch(
        command_interface => $node, );
}

###############################################################################
# Subroutine  : security_file_directory_apply
# Input       : 
#            'policy-name' => 1,    # mandatory
#            'vserver'     => 1,    # mandatory
# Return      : None
# Description : This subroutine is used to apply a policy
###############################################################################

sub security_file_directory_apply {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'policy-name' => 1,    # mandatory
            'vserver'     => 1,    # mandatory
        }
    );

    my $node         = $args{'node'}    || $self->{nodes}->[0];
    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];
    my $policy_name  = $args{'policy-name'};

    $Log->debug("Apply policy $policy_name");
    NACL::C::VserverSecurityFileDirectory->apply(
        command_interface => $node,
        vserver           => $vserver_name,
        'policy-name'     => $policy_name,
    );
}

###############################################################################
# Subroutine  : security_file_directory_show
# Input       : 
#            'path'        # mandatory (e.g. unix style path [/vol1/dir1/file1.txt])
#            'vserver'     # mandatory
# Return      : None
# Description : This subroutine executes file-directory show command on a given path
###############################################################################
sub security_file_directory_show {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'path'    => 1,    # mandatory
            'vserver' => 1,    # mandatory
        }
    );

    my $vserver_name = $args{'vserver'} || $self->{vserver_names}->[0];
    my $path = $args{'path'};

    $Log->debug("Show file-directory command output");
    NACL::CS::VserverSecurityFileDirectory->fetch(
        command_interface => $self->{nodes}->[0],
        filter            => {
            vserver => $vserver_name,
            path    => $path,
        },
    );

    try {
        my $job_state =
          NACL::CS::VserverSecurityFileDirectoryJob->fetch(
            command_interface => $self->{nodes}->[0], );
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->warn("In the BaseException catch");
        $Log->comment( "Caught a " . ref($exception_object) . "!" );
    };

    NACL::CS::VserverSecurityFileDirectory->fetch(
        command_interface => $self->{nodes}->[0],
        filter            => {
            vserver => $vserver_name,
            path    => $path,
        },
    );

}
###############################################################################
# Subroutine  : cifs_share_access_control_create
# Input       : 
#            'vserver'           # mandatory
#            'share'             # mandatory
#            'user-or-group'     # mandatory
#            'permission'        # mandatory (No_access or Read or Change or Full_Control)
# Return      : None
# Description : This subroutine create new access control on a share
###############################################################################
sub cifs_share_access_control_create {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'       => 1,    # mandatory
            'share'         => 1,    # mandatory
            'user-or-group' => 1,    # mandatory
            'permission'    => 1,    # mandatory
        }
    );

    my $node = $self->{nodes}->[0];

    $Log->debug("Create new access control for share $args{share}");
    my $access_obj = NACL::C::VserverCifsShareAccessControl->create(
        command_interface => $node,
        %args
    );

    $access_obj->wait_on_attribute(
        attribute_to_check => 'permission',
        till_value         => $args{'permission'},
        'method-timeout'   => 60,
    );

    $self->diag_nblade_cifs_shares(
        'share'      => $args{'share'},
        'permission' => $args{'permission'},
        'vserver'    => $args{'vserver'},
    );

}
###############################################################################
# Subroutine  : cifs_share_access_control_modify
# Input       : 
#            'vserver'           # mandatory
#            'share'             # mandatory
#            'user-or-group'     # mandatory
#            'permission'        # mandatory (No_access or Read or Change or Full_Control)
# Return      : None
# Description : This subroutine modifies access control permission on a share
###############################################################################
sub cifs_share_access_control_modify {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'       => 1,    # mandatory
            'share'         => 1,    # mandatory
            'user-or-group' => 1,    # mandatory
            'permission'    => 1,    # mandatory
        }
    );

    my $node = $self->{nodes}->[0];

    $Log->debug("Modify access control for share $args{share}");
    my $access_obj = NACL::C::VserverCifsShareAccessControl->modify(
        command_interface => $node,
        %args
    );

    $access_obj->wait_on_attribute(
        attribute_to_check => 'permission',
        till_value         => $args{'permission'},
        'method-timeout'   => 60,
    );

    $self->diag_nblade_cifs_shares(
        'share'      => $args{'share'},
        'permission' => $args{'permission'},
        'vserver'    => 1,
    );

}
###############################################################################
# Subroutine  : cifs_share_access_control_find
# Input       : 
#            'vserver'           # optional
#            'share'             # optional
#            'user-or-group'     # optional
#            'permission'        # optional
#            'winsid'            # optional
# Return      : based on filter applied all share access control objects
# Description : This subroutine resturns all share access control objects
###############################################################################
sub cifs_share_access_control_find {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'       => 0,    # optional
            'share'         => 0,    # optional
            'user-or-group' => 0,    # optional
            'permission'    => 0,    # optional
            'winsid'        => 0,    # optional
        }
    );

    my $node = $self->{nodes}->[0];

    $Log->debug("Find all access control permissions");
    my @share_acls = NACL::C::VserverCifsShareAccessControl->find(
        command_interface => $node,
        filter            => {%args}
    );

    return @share_acls;

}

###############################################################################
# Subroutine  : _diag_nblade_cifs_shares
# Input       : 
#            'vserver'    # mandatory
#            'share'      # mandatory
#            'permission' # mandatory (No_access or Read or Change or Full_Control)
# Return      : None
# Description : Wait for ACE to reach a desired state and also waits for
#               diag nblade replay to complete
###############################################################################
sub diag_nblade_cifs_shares {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'    => 1,    # mandatory
            'share'      => 1,    # mandatory
            'permission' => 1,    # mandatory
        }
    );

    my $test_share_name = $args{'share'};
    my $permission      = $args{'permission'};
    my $vserver_name    = $args{'vserver'};

    if ( $permission eq 'Full_Control' ) {
        $permission = 'Full Control';
    }
    elsif ( $permission eq 'No_access' ) {
        $permission = 'No Access';
    }

    $Log->debug( "Wait for Share Name "
          . $test_share_name
          . " to acquire "
          . "access-control permission "
          . $permission );

    my $node      = $self->{nodes}->[0];
    my $node_name = $node->node();
    my $host      = host($node_name);
    my $obj       = NACL::APISet->new(
        hostobj   => $host,
        category  => "Node",
        interface => "CLI",
        set       => "CMode"
    );

    my $j = 0;
    while ( $j == 0 ) {
        sleep
          3;    #let's not fill the log file with diag nblade cifs shares output
        my $response =
          $obj->diag_nblade_cifs_shares( vserver => $vserver_name );

        my $processed_output = $response->get_processed_output();

        my @rawOutputArray = split /\n/, $processed_output;

        #$Test->nlog(Dumper(@rawOutputArray));
        my $i = 0;
        foreach my $var (@rawOutputArray) {

            if ( $var =~ m/Share:\s+$test_share_name/ ) {
                $i = 1;
            }
            elsif (( $var =~ m/Share:/ )
                && ( $var !~ m/Share:\s+$test_share_name/ )
                && ( $var !~ m/Dynamic Share/ ) )
            {
                $i = 0;
            }
            if ( ( $var =~ m/Ace:\s+$permission/ ) && ( $i == 1 ) ) {
                $j = 1;
                $Log->debug( "Wait complete for Share Name "
                      . $test_share_name . " to "
                      . "acquire access-control permission "
                      . $permission );
            }
        }
    }

    $Log->debug("Check output for diag nblade replay show");

    my @DiagNbladeReplay_states = ();
    while ( scalar(@DiagNbladeReplay_states) < 1 ) {
        try {
            @DiagNbladeReplay_states =
              NACL::CS::DiagNbladeReplay->fetch( command_interface => $node );
        }
        catch NACL::Exceptions::NoElementsFound with {
            $Log->debug("No matching diag nblade replay(s) found, lets retry");
        };
    }

    foreach my $state_obj (@DiagNbladeReplay_states) {
        $Log->debug( Dumper( $state_obj->reason() ) );
        $Log->debug( Dumper( $state_obj->rule() ) );
        $Log->debug( Dumper( $state_obj->result() ) );
        $Log->debug( Dumper( $state_obj->completed_at() ) );
    }

}

###############################################################################
# Subroutine  : mount_volume
# Input       : 1. unix_client_object 
#               2. All the arguments which NACL::STask::Client::Mount accepts
# Return      : unix mount point object
# Description : Mounts a volume on a unix client
###############################################################################
sub mount_volume {

    my $self = shift;
    my (%mount_params) = @_;

    my $unix_client = $mount_params{'unix_client'};

    delete $mount_params{'unix_client'};

    my $mount = NACL::STask::Client::Mount->create(
        command_interface => $unix_client,
        %mount_params,
    );

    my $mp = $mount->mount_point();
    logcomment( "Mount point: " . $mp );

    return $mount;
}

###############################################################################
# Subroutine  : unix_symlink_add
# Input       : 
#            'client'       # mandatory
#            'file_name'    # mandatory
#            'link_name'    # mandatory
# Return      : None
# Description : Create a symlink on unix client
###############################################################################

sub unix_symlink_add {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'client' => { isa => 'NACL::C::Client' },    # mandatory
            'file_name' => 1,                            # mandatory
            'link_name' => 1,                            # mandatory
        }
    );

    my $client        = $args{'client'}->client();
    my $proto_fileops = $self->{fileops}{$client};

    delete $args{'client'};

    $Log->debug(
        "Creating symbolic link $args{'link_name'} for file $args{'file_name'}"
    );
    $proto_fileops->symlink(%args);
}

###############################################################################
# Subroutine  : ontap_symlink_add
# Input       : 
#            'vserver'            # mandatory
#            'share-name'         # mandatory
#            'unix-path'          # mandatory
#            'cifs-path'          # mandatory
#            'home-directory'     # optional
#            'locality'           # optional
#            'cifs-server'        # mandatory
# Return      : symlink object - on success
# Description : Create a new symlink definition
###############################################################################

sub ontap_symlink_add {

    my $self = shift;
    my (%args) = validate(
        @_,
        {
            'vserver'        => 1,    # mandatory
            'share-name'     => 1,    # mandatory
            'unix-path'      => 1,    # mandatory
            'cifs-path'      => 1,    # mandatory
            'home-directory' => 0,    # optional
            'locality'       => 0,    # optional
            'cifs-server'    => 1,    # mandatory
        }
    );

    my $cifs_symlink = NACL::C::VserverCifsSymlink->create(
        command_interface => $self->{nodes}->[0],
        vserver           => $args{'vserver'},
        'share-name'      => $args{'share-name'},
        'unix-path'       => $args{'unix-path'},
        'cifs-path'       => $args{'cifs-path'},
        'home-directory'  => $args{'home-directory'} || 'false',
        'locality'        => $args{'locality'} || 'local',
    );

    return $cifs_symlink;
}

1;
