#Hey emacs, this is basically -*- perl -*-
# $Id: $
#
## @summary	Common parameters used in CIFS tests.
## @author	dl-maryv (FSS Protocols QA)
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod here

=head1 NAME

CommonParams - Common parameters used in CIFS tests.

=head1 SYNOPSIS

  use CIFS::CommonParams;

=head1 DESCRIPTION

This sets up correct environment variables. Note that the
variables are coming out of main's name space.

Put the 'use nfs::CommonParams;' right after the BEGIN
block with the thpl script param statements to make sure
the the variables below are defined before other library
modules are used.

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-maryv (FSS Protocols QA)
  $Id: $
  Last revised $Date: $

=head1 COPYRIGHT

  Copyright 2005, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

# The following package declaration is required for these variables
# to be declared global in the main name space. It was done this
# way because 'use vars' doesn't support package names in the
# variable name, ex: $main::variable. Neither does the 'our'
# statement.
package main;

use strict;
use warnings;

# Please keep the list in alphabetic order
use vars qw(
    $AGGR_SIZE
    $CIFS_DOMAIN
    $CIFS_DOMAIN_ADMIN
    $CIFS_DOMAIN_ADMIN_PASSWD
    $CIFS_DRIVE
    $CIFS_PASSWD
    $CIFS_SHARE
    $CIFS_USER
    $CLEANUP
    $CLIENT
    $CMDTIMEOUT
    $EXTERN_SETUP
    $FILER_MODE
    $FILER
    $FILERA
    $FILERB
    $FILER_AGGR
    $FILER_CONN
    $FILER_IP
    $FILER_QTREE
    $FILER_VOL
    $ROOT_VOL
    $FILER_VOL_MNT
    $MOUNTPOINT
    $NBLADE_START
    $NBLADE_HOST
    $NBLADE_DIR
    $ONTAPDIR
    $PFILER
    $TIMEOUT
    $VOL_SIZE
    $TRAD_SIZE
    $FLEX_SIZE
    $VOL_TYPE
    $RAID_TYPE
    $MOUNT_VERS
    $CONTAINER
    $SETUP_DONE
    $WORM_VOL
    $WORM_AGGR
    $WORM_RETENTION
    $WAFLIRON_TIMEOUT
    $SKIP_SNAPLOCK_RETENTION_SETUP
    $SLCC_STRICT
    $HOMEDIR
    $MODE

);

# Change the name space back
package CIFS::CommonParams;

use Tharn qw(host logresult param);

# param sets up the variables in main's name space
param('AGGR_SIZE',      '-default', 5);
param('CIFS_DOMAIN',    '-default', 'cifsqa.lab.netapp.com');
param('CIFS_DOMAIN_ADMIN', '-default', "cifsqa\\administrator");
param('CIFS_DOMAIN_ADMIN_PASSWD', '-default', 'Hello123');
param('CIFS_DRIVE',     '-default', '*');
param('CIFS_PASSWD',    '-default', 'cifspass');
param('CIFS_SHARE',     '-default', 'testshare');
param('CIFS_USER',      '-default', 'administrator');
param('CLEANUP',        '-default', 'test');
param('CLIENT',         '-default', 'localhost');
param('CMDTIMEOUT',     '-default', 120); # 2 mins
param('SETUP_DONE',     '-default', 'reuse'); #this will replace EXTERN_SETUP in the future
param('EXTERN_SETUP',   '-default', $::SETUP_DONE);
param('FILER_AGGR',     '-default', 'testaggr');
param('FILER_CONN',     '-default', '#rsh-socket');
param('FILER_IP',       '-default', undef);
param('FILER_QTREE',    '-default', 'filerqtree');
param('FILER_VOL_MNT',  '-default', undef);
param('MOUNTPOINT',     '-default', undef);
param('NBLADE_START',   '-default', undef());
param('NBLADE_HOST',    '-default', undef());
param('NBLADE_DIR',     '-default', undef());
param('ONTAPDIR',       '-default', '/rlse/mainN');
param('PFILER',         '-default', '');
param('TIMEOUT',        '-default', 600); # 10 mins
param('MOUNT_VERS',     '-default', undef); #could be 4
param('FLEX_SIZE',      '-default', '30m');
param('TRAD_SIZE',      '-default', 3);
param('WORM_RETENTION', '-default', '30s');
param('WORM_AGGR',      '-default', 'wormaggr');
param('WAFLIRON_TIMEOUT', '-default', 2400);
param('SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 0);
param('SLCC_STRICT',    '-default', 1);
param('FILER_MODE',     '-default', 'mono');
param('RAID_TYPE',      '-default', undef);
param('ROOT_VOL',  	'-default', 'vol0');
param('MODE',	  	'-default', '7G');
param('HOMEDIR',  	'-default', 'hometest');


use config_base;
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
my $wv;

if ($::FILER_MODE eq 'mono') {
    param('FILER',      '-mesg', '  hostspec of filer');
} else {
    param('FILERA',     '-mesg', '  FILERA for cluster');
    param('FILERB',     '-mesg', '  FILERB for cluster');
    param('FILER',      '-default', $::FILERA);
}


my $fh = host($::FILER);
eval {
    $wv = get_wafl_version($fh);
};
if ($@ and ! defined($ENV{"VOL_TYPE"}) and ! defined($::VOL_TYPE)) {
    logresult('CONFIG', "filer/sim is down or unreachable -- define \$VOL_TYPE to continue");
} elsif ($wv gt WV_SCRIMSHAW_MAX) {
    param('VOL_TYPE',   '-default', 'flex');
} else {
    param('VOL_TYPE',   '-default', 'trad');
    param('SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 1);
}

if ($::VOL_TYPE eq 'flex') {
    param('VOL_SIZE',   '-default', $::FLEX_SIZE);
    param('FILER_VOL',  '-default', 'testvol_flex');
    param('CONTAINER',  '-default', $::FILER_AGGR);
    param('WORM_VOL',   '-default', 'testvol_worm_flex');
} else {
    param('VOL_SIZE',   '-default', $::TRAD_SIZE);
    param('FILER_VOL',  '-default', 'testvol_trad');
    param('CONTAINER',  '-default', $::FILER_VOL);
    param('WORM_VOL',   '-default', 'testvol_worm_trad');
}

1;

