#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CIFS/Csc.pm#1 $
#
## @summary Csc - Library for the Client Side Caching tests
## @author	  dl-maryv (FSS Protocols QA)
## @notes	  Created by EP Network Storage Performance Lab, Inc.
## @pod here

=head1 NAME

Csc - Library for the Client Side Caching tests

=head1 SYNOPSIS

    use CIFS::Csc;

    csc_setup([mode            => 'multi' | 'ntfs'],
              [create_vol      => 'yes' | 'no'],
              [vol_name        => $default_vol],
              [setup_cifs      => 'yes' | 'no'],
              [default_share   => 'yes' | 'no'],
              [share_name      => $share_name],
              [option          => 'no|auto_document|auto_program'],
              [test_shares     => 'yes | no'],
              [test_share_h    => \%test_share_h],
              [map_test_shares => 'yes | no'],
              [nfs_enable      => 'yes | no'],
              [homedir_chg     => 'yes | no'],
              [homedir_names   => \%homedir_names_h],
              [homedir_load    => 'yes | no'],
               ret_h           => \%setup_h);

    csc_cleanup([fc            => $fc],
                [cifs_drive    => $cifs_drive],
                [options       => \%options],
                [vol           => $volume],
                [drives        => \@drives],
                [shares        => \%shares],
                [test_drives   => \%drives],
                [vols          => \@volumes],
                [dirs          => \@directories],
                [files         => \@files],
                [share_access  => \%share_access],
                [share_acls    => \%share_acls],
                [cifs_users    => \@cifs_users],
                [pids          => \@pids]);

    $dir_path = csc_test_dir_create(target_drive => $tdrive,
                                    dir_name     => $test_dir);

    @pids = csc_bigf(filer => $filer, share => $share);

    @pids = csc_ffiles(filer => $filer, share => $share);

    csc_hammer(filer => $filer, share => $share);

    $my_computer = csc_open_explorer();

    csc_close_explorer(window => $whnd);

    $ret = csc_available_offline(whnd     => $my_computer,
                                 filer_ip => $filer_ip,
                                 share    => $CIFS_SHARE,
                                 drive    => $share_drive);

    $ret = csc_make_available_offline(whnd     => $win,
                                      filer_ip => $filer_ip,
                                      share    => $share,
                                      drive    => $drive,
                                      option   => 'yes');

    $ret = csc_create_files(file_name => $file,
                            file_ext  => $file_extension,
                            file_path => $file_path);

    $ret = csc_edit_file(file_name => $file,
                         file_ext  => $file_extension,
                         file_path => $file_path);

    $ret = csc_sync_folder(filer_ip => $filer_ip,
                           share    => $share,
                           drive    => $drive,
                           whnd     => $whnd);

    @pids = csc_get_pid(task_name => $task);

    csc_kill_task(task_name => $task);

    $win = csc_open_mmc(filer => $filer);

    csc_close_mmc(window => $whnd);

    $msg = csc_verify_caching_button(window     => $win,
                                     share      => $share,
                                     share_attr => $share_attr);

    $msg = csc_verify_path_option(window     => $win,
                                  share      => $share,
                                  share_attr => $share_attr);

    $ret = csc_change_caching_option(window  => $win,
                                     filer   => $filer,
                                     share   => $share,
                                     caching => $share_attr);

    $msg = csc_add_share(window  => $win,
                         filer   => $filer,
                         share   => $share,
                         caching => $share_attr);

    $ret = csc_delete_share(window => $win,
                            filer  => $filer,
                            share  => $share);

    $ret = csc_cifs_sessions(window => $win,
                             filer  => $filer);

    $ret = csc_close_cifs_sessions(window => $win,
                                   filer  => $filer,
                                   user   => $user);

    $ret = csc_browse_files(window => $win,
                            filer  => $filer);

    $ret = csc_browse_local_users(window => $win,
                                  filer  => $filer);

    $ret = csc_browse_local_groups(window => $win,
                                   filer  => $filer);

    $ret = csc_add_local_user(window => $win,
                              filer  => $filer,
                              user   => $user,
                              passwd => $pass);

    $ret = csc_close_open_files(window => $win,
                                filer  => $filer,
                                file   => $file);

    $ret = csc_share_exists(window     => $win,
                            filer      => $filer,
                            share      => $share);

    @perm = csc_share_properties(window => $win,
                                 filer  => $filer,
                                 share  => $share);

    $drive = csc_map_drive(window => $win,
                           filer  => $filer,
                           share  => $share,
                           user   => $user,
                           pass   => $pass);

    $ret = csc_unmap_drive(window => $win,
                           filer  => $filer,
                           share  => $share,
                           drive  => $drive);

    $ret = csc_encryption_support(filer_ip => $filer_ip,
                                  share    => $share,
                                  drive    => $drive,
                                  whnd     => $whnd);

    $ret = csc_view_files(filer_ip => $filer_ip,
                          share    => $share,
                          drive    => $drive,
                          whnd     => $whnd);

    $ret = csc_manual_sync(filer_ip => $filer_ip,
                           share    => $share,
                           drive    => $drive,
                           file     => $file,
                           whnd     => $whnd);

    $ret = csc_sync_via_tools(filer_ip => $filer_ip,
                              share    => $share,
                              drive    => $drive,
                              whnd     => $whnd);

    $ret = csc_offline_shortcut(whnd => $whnd);

    $ret = csc_delete_offline_files(filer_ip => $filer_ip,
                                    share    => $share,
                                    drive    => $drive,
                                    whnd     => $whnd,
                                    option   => $option);

    $ret = csc_advanced_button(filer_ip => $filer_ip,
                               share    => $share,
                               drive    => $drive,
                               whnd     => $whnd,
                               option   => $option);

    @pids = csc_vdthrash_cmdline(filer => $filer, share => $share);

    $win = csc_vdthrash(filer => $filer, share => $share);

    csc_vdthrash_close();

    $ret = sync_via_my_network(filer    => $filer,
                               filer_ip => $filer_ip,
                               share    => $share,
                               drive    => $drive,
                               whnd     => $whnd);

    csc_xcopy(source_path => $source,
              dest_path   => $dest);

    $ret = csc_remove_folder(filer => $filer,
                             share => $share,
                             dir   => $dir);

    $ret = csc_verify_dir_sync(source_path => $source,
                               dest_path   => $dest);

    $ret = csc_folder_offline(whnd     => $win,
                              filer_ip => $filer_ip,
                              share    => $share,
                              drive    => $drive,
                              option   => 'yes');

    $ret = csc_folder_redirect(whnd   => $win,
                               target => $target,
                               item   => $item,
                               drive  => $drive,
                               action => $action);

    csc_copy_file(source  => $source,
                  dest    => $dest);

    $msg = csc_install(file => $file);

    csc_client_setup(setup_h      => \%setup_h,
                     [clients      => \@clients, ]
                     [cifs_clients => \@cifs_clients, ]
                     ret_h         => \%setup_h);

    csc_client_cleanup([  fc => $fc ]
                       [, cc => \@cc ]
                       [, cifs => \@cifs_drives ]
                       [, shares => \%shares ]
                       [, vol => $vol ]
                       [, mtpoint => $mtpoint ]
                       [, files => \@files ]);

=head1 DESCRIPTION

Provides common routines for the Client Side Caching tests.
This library provides wrapper functions to the gui routines in Utils_gui.pm,
used for GUI manipulation of Windows computers.

In general routines in this library should not croak but should
return errors. This allows the caller to cleanup the windows
they know they have open.

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-maryv (FSS Protocols QA)
  $Id: //depot/prod/test/Rsierranevada/lib/CIFS/Csc.pm#1 $
  Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

  Copyright 2006, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package CIFS::Csc;

use strict;
use warnings;
use base qw(Exporter);

use Carp qw(croak);
use CIFS::Ntfs qw(ntfs_cifs_setup);
use ExportFS qw(share_add share_delete);
use filer qw(filer_options_set filer_cifs_access filer_cifs_load_homedir
             filer_qtree_get filer_qtree_security_set);
use FS::Util::Setup qw(create_vol common_cleanup);
use MapShare qw(map_cifs_share unmap_cifs_share);
use Params::Validate qw(validate ARRAYREF SCALAR OBJECT UNDEF HASHREF);
use SPARSE::Util::Logger qw(logStep logWarn);
use Tharn qw(connect host logresult);
use Users qw(removeCIFSUser);
use Volume qw(vol_purge vol_root);
use CLI::FileIO qw(rdfile wrfile);
use GenericMount qw(mount umount);
use nfs::GenericExport qw(export_path unexport_path);
use FileManip qw(remove_file);
use File::Basename;
use File::Path;
use File::stat;
use File::Compare;

BEGIN {
    # Die with a script error when an
    # input parameter fails validation
    Params::Validate::validation_options(
        on_fail => sub { my @msgs = @_;
                         chomp(@msgs);
                         Carp::confess(Tharn::script_result(@msgs)) }
    );
};

our $VERSION = '1.0';

# Functions to be exported
our @EXPORT_OK = qw(
    csc_setup
    csc_cleanup
    csc_client_setup
    csc_client_cleanup
    csc_test_dir_create
    csc_kill_task
    csc_ffiles
    csc_bigf
    csc_hammer
    csc_ffiles
    csc_open_explorer
    csc_close_explorer
    csc_available_offline
    csc_make_available_offline
    csc_create_files
    csc_edit_file
    csc_sync_folder
    csc_open_mmc
    csc_close_mmc
    csc_verify_caching_button
    csc_verify_path_option
    csc_change_caching_option
    csc_add_share
    csc_delete_share
    csc_cifs_sessions
    csc_close_cifs_sessions
    csc_browse_files
    csc_browse_local_users
    csc_browse_local_groups
    csc_add_local_user
    csc_close_open_files
    csc_share_exists
    csc_share_properties
    csc_add_to_local_group
    csc_encryption_support
    csc_view_files
    csc_manual_sync
    csc_sync_via_tools
    csc_offline_shortcut
    csc_delete_offline_files
    csc_advanced_button
    csc_vdthrash_cmdline
    csc_vdthrash
    csc_vdthrash_close
    csc_sync_via_my_network
    csc_xcopy
    csc_remove_folder
    csc_verify_dir_sync
    csc_folder_offline
    csc_folder_redirect
    csc_copy_file
    csc_install
    csc_map_drive
    csc_unmap_drive
    testme
);

#---------------------------------------------------------------------
# Variables used by the setup and cleanup routine
#---------------------------------------------------------------------
my $filer_vol_path;
my $share_name;
my $create_vol_called = 0;
my $filer_homedir_text;
my $cifs_homedir = '/etc/cifs_homedir.cfg';
my $bigf = 'X:\bin\bigf.exe';
my $bigf_task = 'bigf.exe';
my $hammer = 'X:\bin\hammer.exe';
my $ffiles = 'X:\bin\ffile1_ha.exe';
my $ffiles_task = 'ffile1_ha.exe';
my $vdthrash = 'X:\bin\vdthrash.exe';
my $vdthrash_task = 'vdthrash.exe';
my $usermap = '/etc/usermap.cfg';
my $filer_usermap_text;

my @arr;
my %shares_h;
my %drives_h;
my %test_shares;

#---------------------------------------------------------------------
# Function prototypes
#---------------------------------------------------------------------
sub csc_setup(%);
sub csc_cleanup(%);
sub csc_client_setup(%);
sub csc_client_cleanup(%);
sub csc_test_dir_create(%);
sub csc_ffiles(%);
sub csc_get_pid(%);
sub csc_kill_task(%);
sub csc_bigf(%);
sub csc_hammer(%);
sub csc_ffiles(%);
sub csc_open_explorer();
sub csc_close_explorer(%);
sub csc_available_offline(%);
sub csc_make_available_offline(%);
sub csc_create_files(%);
sub csc_edit_file(%);
sub csc_sync_folder(%);
sub csc_open_mmc(%);
sub csc_close_mmc(%);
sub csc_verify_caching_button(%);
sub csc_verify_path_option(%);
sub csc_change_caching_option(%);
sub csc_add_share(%);
sub csc_delete_share(%);
sub csc_cifs_sessions(%);
sub csc_close_cifs_sessions(%);
sub csc_browse_files(%);
sub csc_browse_local_users(%);
sub csc_browse_local_groups(%);
sub csc_add_local_user(%);
sub csc_close_open_files(%);
sub csc_share_exists(%);
sub csc_share_properties(%);
sub csc_add_to_local_group(%);
sub csc_encryption_support(%);
sub csc_view_files(%);
sub csc_manual_sync(%);
sub csc_sync_via_tools(%);
sub csc_offline_shortcut(%);
sub csc_delete_offline_files(%);
sub csc_advanced_button(%);
sub csc_vdthrash_cmdline(%);
sub csc_vdthrash(%);
sub csc_vdthrash_close();
sub csc_sync_via_my_network(%);
sub csc_xcopy(%);
sub csc_remove_folder(%);
sub csc_verify_dir_sync(%);
sub csc_folder_offline(%);
sub csc_folder_redirect(%);
sub csc_copy_file(%);
sub csc_install(%);
sub csc_map_drive(%);
sub csc_unmap_drive(%);
sub testme();

#---------------------------------------------------------------------

=head2 csc_setup

    csc_setup([mode            => 'multi' | 'ntfs'],
              [create_vol      => 'yes' | 'no'],
              [vol_name        => $default_vol],
              [setup_cifs      => 'yes' | 'no'],
              [default_share   => 'yes' | 'no'],
              [share_name      => $share_name],
              [option          => 'no|auto_document|auto_program'],
              [test_shares     => 'yes | no'],
              [test_share_h    => \%test_share_h],
              [map_test_shares => 'yes | no'],
              [nfs_enable      => 'yes | no'],
              [homedir_chg     => 'yes | no'],
              [homedir_names   => \%homedir_names_h],
              [homedir_load    => 'yes | no'],
               ret_h           => \%setup_h);

Runs the common cifs setup. Sets the filer
to multi-protocol or ntfs-only mode.

The following named arguments are accepted:

=over

=item Arguments

=over

=item mode

(Optional) : Protocol setting for the filer can be 'ntfs' or
             'multi'. Defaults to 'multi'.

=item create_vol

(Optional) : If create_vol => 'yes' create default volume.
             If create_vol => 'no' do not create default volume.

=item vol_name

(Optional) : If vol_name create vol_name volume.

=item setup_cifs

(Optional) : If setup_cifs => 'yes' will setup CIFS with the specified mode.
             If setup_cifs => 'no' will not setup CIFS.
             Defaults to 'yes'.

=item default_share

(Optional) : If default_share => 'yes' create and map default share.
             If default_share => 'no' do not create and map default share.

=item share_name

(Optional) : share_name => 'share_name' name of the default share.
             Otherwise name of default share $main::CIFS_SHARE.

=item option

(Optional) : option => 'manual' default share caching option.
             Defaults to manual caching.

=item test_shares

(Optional) : test_shares => 'yes' create test shares for csc test.
Default is no, do not create a default share.

=item test_share_h

(Optional) : hash of test shares to create for test.

=item map_test_shares

(Optional) : map_test_shares => 'yes' map test shares on CIFS client.
Default is not, do not map test shares.

=item nfs_enable

(Optional) : Enable NFS on create_vol.
Default is no.

=item homedir_chg

(Optional) : If homedir_chg save copy of cifs_homedir.cfg, update with new
entries and write. Default is no, do not change.

=item homedir_names

(Optional) : Hash of homedir names to create home directories for.

=item homedir_load

(Optional) : Load the home directories on CIFS.
Defaults to no.

=item ret_h

(Required) : Hash ref passed by caller. To be used by caller and csc_cleanup.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_setup(%) {
    my %opts = validate(
        @_,
        {
            mode            => { type => SCALAR, default => 'multi',
                                 regex => qr/^(ntfs|multi)$/ },
            create_vol      => { type => SCALAR, optional => 1,
                                 regex => qr/^(yes|no)$/ },
            vol_name        => { type => SCALAR, optional => 1 },
            setup_cifs      => { type => SCALAR, default => 'yes',
                                 regex => qr/^(yes|no)$/ },
            default_share   => { type => SCALAR, optional => 1,
                                 regex => qr/^(yes|no)$/ },
            share_name      => { type => SCALAR, optional => 1 },
            option          => { type => SCALAR, default => 'manual',
                                 regex => qr/^(no|auto_document|auto_program|manual)$/ },
            test_shares     => { type => SCALAR, optional => 1,
                                 regex => qr/^(yes|no)$/ },
            test_share_h    => { type => HASHREF, optional => 1 },
            map_test_shares => { type => SCALAR, optional => 1,
                                 regex => qr/^(yes|no)$/ },
            nfs_enable      => { type => SCALAR, default => 'no',
                                 regex => qr/^(yes|no)$/ },
            homedir_chg     => { type => SCALAR, default => 'no',
                                 regex => qr/^(yes|no)$/ },
            homedir_names   => { type => HASHREF, optional => 1 },
            homedir_load    => { type => SCALAR, default => 'no',
                                 regex => qr/^(yes|no)$/ },
            ret_h           => { type => HASHREF }
          }
    );

    my (@args, $cifs_drive, $cifs_user, $fc, $fh, $filer,
        $hostname, $mode, $new_text, $purge, %qtree_out, $share, $volume);

    logStep("BEGIN: csc_setup");

    # Disable the screen lock on the client and just leave it off
    # Croak if the screen lock was on.
    logStep("csc_setup: Verify client has screen saver turned off");
    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(screen_lock);
    screen_lock(setting => 'off', croak_if_on => 'yes');

    # Get connection object to filer
    eval {
        $fh = host($main::FILER);
        $hostname = $fh->hostname();
        $fc = connect($main::FILER_CONN, -timeout, $main::TIMEOUT);
        $fc->timeout($main::CMDTIMEOUT);
    };

    croak("csc_setup: Get connection object to filer $@") if ($@);

    # Save return values
    ${$opts{ret_h}}{fc} = $fc;
    ${$opts{ret_h}}{shares} = \%shares_h;

    if (defined($opts{create_vol}) && $opts{create_vol} eq 'yes') {
        logStep("csc_setup: Create volume");
        $create_vol_called = 1;
        ($fc, $volume) = create_vol(nfs_enable => $opts{nfs_enable})
            or croak("csc_setup: Create vol failed $@");

        ${$opts{ret_h}}{vol} = $volume;

        # At this point the test has asked us to create the vol and the
        # vol already existed or did not exist and was created. Make sure
        # the qtree security style for FILER_VOL matches the protocol mode
        # requested for the CIFS setup.
        %qtree_out = filer_qtree_get($fc, vols => 1);

        # If mode is multi, map name to mixed for comparison
        $mode = $opts{mode};
        $mode = 'mixed' if ($mode =~ /multi/);

        if ($qtree_out{$main::FILER_VOL}{''}{Style} ne $mode) {
            logStep("Set the qtree security mode on $main::FILER_VOL to $mode");
            if (filer_qtree_security_set($fc, "/vol/$volume", $mode) == 0) {
                croak("csc_setup: qtree set mode $mode failed on /vol/$volume");
            }
        }
    }

    if ($opts{homedir_chg} eq 'yes') {
        if (defined($opts{homedir_names})) {
            $filer = $fc->hostp->id;

            logStep("csc_setup: Create the users home directories on filer");
            my $dirname  = "\\\\$filer\\c\$\\users";
            mkpath($dirname);
            if ($@) {
                croak("csc_setup: mkdir failed: $dirname: $@");
            }
            push @{$opts{ret_h}{dirs}}, $dirname;

            %test_shares = %{$opts{homedir_names}};
            foreach $share (keys %test_shares) {
                logStep("csc_setup: Create the home directories for each ".
                        "user on filer");
                my $dirname  = "\\\\$filer\\c\$\\users\\$share";
                mkpath($dirname);
                if ($@) {
                    croak("csc_setup: mkdir failed: $dirname: $@");
                }
            }
        } else {
            croak("csc_setup: no home directories given");
        }

        my $root_vol = vol_root($fc);
        $filer_homedir_text = rdfile($fc, $cifs_homedir);
        logStep("csc_setup: Setup $cifs_homedir file on filer");
        $new_text = "\n/vol/$root_vol/users\n";
        wrfile($fc, $cifs_homedir, $filer_homedir_text. $new_text)
            or croak("csc_setup: can't write $cifs_homedir: $@");

        logStep("csc_setup: load the home directory paths on filer");
        filer_cifs_load_homedir(fc => $fc);
    }

    # Create username to use for cifs based on whether AD is setup or not
    # Default is workgroup
    $hostname =~ tr/a-z/A-Z/;

    $cifs_user = "$hostname\\$main::CIFS_USER";
    # Reassign $main::CIFS_USER and so the fully qualified user name
    # gets propagated to subtests correctly
    $main::CIFS_USER = $cifs_user;
    $new_text = "\n$cifs_user == root\n";

    if (defined($main::CIFS_DOMAIN)) {
        $main::CIFS_DOMAIN =~ s/\s+//g;
        if ($main::CIFS_DOMAIN ne '') {
            # Using Domains
            $new_text .= "$main::CIFS_DOMAIN_ADMIN == root\n";
        }
    }

    logStep("csc_setup: Setup $usermap file on filer");
    $filer_usermap_text = rdfile($fc, $usermap);
    wrfile($fc, $usermap, $filer_usermap_text . $new_text)
        or croak("csc_setup: can't write $usermap: $@");

    if (defined($opts{setup_cifs}) && $opts{setup_cifs} eq 'yes') {
        logStep("csc_setup: Setup filer as $opts{mode}");
        ntfs_cifs_setup(filer    => $main::FILER,
                        mode     => $opts{mode});
        croak("csc_setup: cifs setup failed $@") if $@;

        $filer_vol_path = "/vol/$main::FILER_VOL";

        if (defined($opts{default_share}) && $opts{default_share} eq 'yes') {
            logStep("csc_setup: share add");
            $share_name = defined($opts{share_name}) ?
                $opts{share_name} : $main::CIFS_SHARE;
            $purge = ($share_name =~ /^(etc\$|c\$|home)$/i) ? 0 : 1;
            if ($opts{option} ne 'manual') {
                @args = ( 'caching', $opts{option} );
            } else {
                @args = ();
            }
            share_add($fc, $share_name, $filer_vol_path, verify => 1,
                      purge_existing => $purge, umask => 0, @args)
                or croak("csc_setup: failed to export share $main::CIFS_SHARE");

            $shares_h{$share_name} = $purge;

            logStep("csc_setup: map CIFS share");
            $cifs_drive = map_cifs_share(
                        server   => $fc->connhost(),
                        share    => $share_name,
                        client   => 'localhost',
                        drive    => $main::CIFS_DRIVE,
                        username => $cifs_user,
                        password => $main::CIFS_PASSWD
                        )
                    or croak("csc_setup: could not map the drive $@");

            if (!$cifs_drive) {
                croak("csc_setup: could not map the drive");
            }

            $cifs_drive .= ":" unless $cifs_drive =~ /:$/;
            push @{$opts{ret_h}{drives}}, $cifs_drive;
        }

        if (defined($opts{test_shares}) && ($opts{test_shares} eq 'yes') &&
            defined($opts{test_share_h})) {
            %test_shares = %{$opts{test_share_h}};
            for $share (keys %test_shares) {
                logStep("csc_setup: share add $share");
                if ($test_shares{$share} eq 'manual') {
                    share_add($fc, $share, $filer_vol_path,
                              verify => 1,
                              purge_existing => 1,
                              umask => 0)
                        or croak("csc_setup: failed to export share $share");
                } else {
                    share_add($fc, $share, $filer_vol_path,
                               verify => 1,
                               purge_existing => 1,
                               umask => 0,
                               caching => $test_shares{$share})
                        or croak("csc_setup: failed to export share $share");
                }

                $shares_h{$share} = 1;

                if (defined($opts{map_test_shares}) &&
                    ($opts{map_test_shares} eq 'yes')) {
                    logStep("csc_setup: map $share CIFS share");
                    $cifs_drive = map_cifs_share(server   => $fc->connhost(),
                                                 share    => $share,
                                                 client   => 'localhost',
                                                 drive    => $main::CIFS_DRIVE,
                                                 username => $cifs_user,
                                                 password => $main::CIFS_PASSWD
                                                )
                        or croak("csc_setup: could not map the drive: $@");

                    if (!$cifs_drive) {
                        croak("csc_setup: could not map the drive");
                    }

                    $cifs_drive .= ":" unless $cifs_drive =~ /:$/;
                    push @{$opts{ret_h}{drives}}, $cifs_drive;
                    $drives_h{$share} = $cifs_drive;
                }
            }

            ${$opts{ret_h}}{test_shares} = \%test_shares;
            ${$opts{ret_h}}{test_drives} = \%drives_h;
        }
    }

    logStep("END: csc_setup");
    return;
}

#---------------------------------------------------------------------

=head2 csc_cleanup

    csc_cleanup([fc            => $fc],
                [cifs_drive    => $cifs_drive],
                [options       => \%options],
                [vol           => $volume],
                [drives        => \@drives],
                [shares        => \%shares],
                [test_drives   => \%drives],
                [vols          => \@volumes],
                [dirs          => \@directories],
                [files         => \@files],
                [share_access  => \%share_access],
                [share_acls    => \%share_acls],
                [cifs_users    => \@cifs_users],
                [pids          => \@pids]);

Cleanup after a test run. Unmap CIFS drive. Restore filer options if changed
during test run. Purge volumes created after test run. Purge default volume
if created by csc_setup. Unmap any drives if mapped by test. Delete any
shares added by test. Purge volumes, dirs and files created by test. Restore
share access and acls changed by test. Delete users created by test. Delete
CIFS users added by test. Unlicenses CIFS if csc_setup licensed CIFS.
Unlicenses NFS if csc_setup licensed NFS. Resets filer
wafl.default_security_style to pre-test setting. Use this cleanup
routine if you use csc_setup.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fc

(Optional) : Filer connection object.

=item cifs_drive

(Optional) : Default share to unmap after test run.

=item options

(Optional) : reset any options that were changed in setup.

=item vol

(Optional) : Default volume to purge after test run.

=item drives

(Optional) : drives to unmap after test run.

=item shares

(Optional) : shares to delete after test run.

=item test_shares

(Optional) : test shares created by test run.

=item vols

(Optional) : List of volumes to purge after test run.

=item dirs

(Optional) : List of directories to remove after test run.

=item files

(Optional) : List of files to remove after test run.

=item share_access

(Optional) : Hash of hashes used to set shares access to pretest state.

=item share_acls

(Optional) : Hash of hashes used to delete share level acls added by test.

=item cifs_users

(Optional) : Array of cifs users added by test.

=item pids

(Optional) : Array of windows process ids to kill.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_cleanup(%) {
    my %opts = validate(
        @_,
        {
            fc             => { type => OBJECT   | UNDEF, optional => 1 },
            cifs_drive     => { type => SCALAR   | UNDEF, optional => 1 },
            options        => { type => HASHREF  | UNDEF, optional => 1 },
            vol            => { type => SCALAR   | UNDEF, optional => 1 },
            drives         => { type => ARRAYREF | UNDEF, optional => 1 },
            shares         => { type => HASHREF  | UNDEF, optional => 1 },
            test_shares    => { type => HASHREF  | UNDEF, optional => 1 },
            test_drives    => { type => HASHREF  | UNDEF, optional => 1 },
            vols           => { type => ARRAYREF, optional => 1 },
            dirs           => { type => ARRAYREF, optional => 1 },
            files          => { type => ARRAYREF, optional => 1 },
            share_access   => { type => HASHREF,  optional => 1 },
            share_acls     => { type => HASHREF,  optional => 1 },
            cifs_users     => { type => ARRAYREF, optional => 1 },
            pids           => { type => ARRAYREF | UNDEF, optional => 1 },
        }
    );

    my ($acl_user, $vol, $fc, $fh, $i, $text, $user);

    logStep("CLEANUP == $main::CLEANUP");

    eval {
        # Reconnect to filer
        $fh = host($main::FILER);
        $fc = connect($main::FILER_CONN, -timeout, $main::TIMEOUT);
        $fc->timeout($main::CMDTIMEOUT);

        if ($main::CLEANUP ne 'none') {
            if (defined($opts{pids}) && @{$opts{pids}} > 0) {
                logStep("csc_cleanup: killing windows tasks started by test");
                foreach my $pid (@{$opts{pids}}) {
                    csc_kill_task(pid => $pid);
                    if ($@) {
                        logWarn("csc_cleanup: taskkill failed: $@");
                    }
                }

                @{$opts{pids}} = ();
            }

            if (defined($opts{files}) && @{$opts{files}} > 0) {
                logStep("csc_cleanup: unlinking files");
                unlink @{$opts{files}} or
                    logWarn("csc_cleanup: unlink failed: $!");

                @{$opts{files}} = ();
            }

            if ($fc && $filer_homedir_text) {
                logStep("csc_cleanup: Restore $cifs_homedir file on filer");
                wrfile($fc, $cifs_homedir, $filer_homedir_text)
                    or logWarn("csc_cleanup: can't write $cifs_homedir: $@");

                logStep("csc_cleanup: Delete home directory paths on filer");
                filer_cifs_load_homedir(fc => $fc);
                sleep(5);
                filer_cifs_load_homedir(fc => $fc);
            }

            # restore options
            @arr = keys(%{$opts{options}});
            if ($fc && defined($opts{options}) && @arr > 0) {
                logStep("csc_cleanup: restoring options");
                foreach $i (@arr) {
                    eval {
                        filer_options_set($fc, $i, ${$opts{options}}{$i});
                    };

                    logWarn("csc_cleanup: $@") if $@;
                }
            }

            @arr = keys(%{$opts{share_acls}});
            if ($fc && defined($opts{share_acls}) && @arr > 0) {
                logStep("csc_cleanup: deleting share level acls");
                foreach $i (@arr) {
                    foreach $acl_user (keys %{$opts{share_acls}{$i}{$i}{acl}}) {
                        eval {
                           $text =
                               filer_cifs_access(
                                  $fc,
                                  share       => $i,
                                  user        => $acl_user,
                                  delete_flag => 'yes');
                        };

                        logWarn("csc_cleanup: $@") if $@;
                    }
                }
            }

            @arr = keys(%{$opts{share_access}});
            if (defined($opts{share_access}) && @arr > 0) {
                logStep("csc_cleanup: restoring share permissions");
                foreach $i (@arr) {
                    foreach $acl_user (keys %{$opts{share_access}{$i}{$i}{acl}}) {
                        eval {
                           $text =
                               filer_cifs_access(
                                  $fc,
                                  share  => $i,
                                  user   => $acl_user,
                                  rights =>
                                  $opts{share_access}{$i}{$i}{acl}{$acl_user});
                        };

                        logWarn("csc_cleanup: $@") if $@;
                    }
                }
            }

            if (defined($opts{dirs}) && @{$opts{dirs}} > 0) {
                logStep("csc_cleanup: removing directories");
                # Traverse array in reverse order so that the directories
                # are removed newest to oldest.
                foreach my $dir (@{$opts{dirs}}) {
                    # Change the mode to make the directory removable,
                    # but more importantly to clear out the ACL.
                    # Remove the directory
                    chmod 0777, $dir;
                    rmtree("$dir", 1, 1);
                    if ($@) {
                        logWarn("csc_cleanup: rmdir $dir failed: $@");
                    }
                }

                @{$opts{dirs}} = ();
            }

            @arr = keys(%{$opts{shares}});
            if ($fc && defined($opts{shares}) && @arr > 0) {
                logStep("csc_cleanup: deleting shares");
                foreach $i (@arr) {
                    if (${$opts{shares}}{$i} ==  1) {
                        eval {
                            share_delete($fc, $i);
                        };
                        logWarn("csc_cleanup: $@") if $@;
                    }
                }
            }

            # Unmapping shares is done after the shares are removed
            # which is backwards but is done this way because sometimes
            # windows thinks files are still open even if they have been
            # closed. The force option on unmap_cifs_share doesn't work
            # so this is the only thing that seems to work.
            if (defined($opts{drives}) && @{$opts{drives}} > 0) {
                logStep("csc_cleanup: unmapping drives");
                my $drive;
                while ($drive = pop @{$opts{drives}}) {
                    unmap_cifs_share(mappath => $drive, force => 1);
                    logWarn("csc_cleanup: $@") if $@;
                }
                @{$opts{drives}} = ();
            }

            if ($fc && $filer_usermap_text) {
                logStep("csc_cleanup: Restore $usermap file on filer");
                wrfile($fc, $usermap, $filer_usermap_text)
                    or logWarn("csc_cleanup: can't write $usermap: $@");
            }

            if ($fc && defined($opts{cifs_users}) && @{$opts{cifs_users}} > 0) {
                logStep("csc_cleanup: Remove cifs users added to the filer");
                foreach my $user (@{$opts{cifs_users}}) {
                    removeCIFSUser($fc, $user);
                    logWarn("csc_cleanup: $@") if ($@);
                }
            }

            if ($fc && defined($opts{vols}) && @{$opts{vols}} > 0) {
                # Remove all volumes created by test
                foreach $vol (@{$opts{vols}}) {
                    eval {
                        vol_purge($fc, $vol);
                    };

                    logWarn("csc_cleanup: $@") if $@;
                }
            }
        }

        # Call common cleanup if create_vol was called
        if ($fc && $create_vol_called) {
            common_cleanup($fc);
            logWarn("csc_cleanup: common_cleanup: $@") if $@;
        }
    };

    logWarn("csc_cleanup: $@") if $@;
    logStep("END: csc_cleanup");

    return;
}

#---------------------------------------------------------------------

=head2 csc_test_dir_create

$dir_path = csc_test_dir_create(target_drive => $tdrive,
                                dir_name     => $test_dir);

Create a test directory on the FILER volume.

The following named arguments are accepted:

=over

=item Arguments

=over

=item target_drive

(Required) : target drive for directory and file creation

=item dir_name

(Required) : name of directory to create

=back

=item Returns

Path to created directory if successful

=back

=for html <hr />

=cut
#--------------------------------------------------------------------

sub csc_test_dir_create(%) {
    my %opts = validate(
        @_,
        {
            drive     => { type => SCALAR, regex => qr/^[A-Za-z]\:$/ },
            dir_name  => { type => SCALAR }
        }
    );

    my (@stat_arr, $tpath);

    $tpath = "$opts{drive}/$opts{dir_name}";

    # don't catch error on mkdir because directory may already exist
    mkdir($tpath);

    @stat_arr = stat($tpath)
        or croak("Couldn't stat directory $tpath: $!");

    return $tpath;
}

#---------------------------------------------------------------------

=head2 csc_bigf

@pids = csc_bigf(filer => $filer, share => $share);

This function runs bigf.exe on the filer.

The following named arguments are accepted:

=over

=item Arguments

=over

=item filer

(Required) : Name of filer to run test on.

=item share

(Required) : Share on filer to run test in.

=back

=item Returns

Array of pids

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_bigf(%) {
    my %opts = validate(
        @_,
        {
            filer => { type => SCALAR },
            share => { type => SCALAR },
        }
    );

    my $filer = $opts{filer};
    my $share = $opts{share};
    my (@pid);

    # Execute bigf command
    open(PGM, "$bigf -p\\\\$filer\\$share\\$share\\testfile_bigf -c500|");
    if ($?) {
        croak("bigf command status was: $?");
    }

    @pid = csc_get_pid(task_name => $bigf_task);

    return @pid;
}

#---------------------------------------------------------------------

=head2 csc_ffiles

@pids = csc_ffiles(filer => $filer, share => $share);

This function runs ffiles_ha.exe on the filer.

The following named arguments are accepted:

=over

=item Arguments

=over

=item filer

(Required) : Name of filer to run test on.

=item share

(Required) : Share on filer to run test in.

=back

=item Returns

Array of pids

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_ffiles(%) {
    my %opts = validate(
        @_,
        {
            filer => { type => SCALAR },
            share => { type => SCALAR },
        }
    );

    my $filer = $opts{filer};
    my $share = $opts{share};
    my @pid;

    # Execute ffiles_ha command
    open(PGM, "$ffiles \\\\$filer\\$share\\$share\\testfile_ffiles|");
    if ($?) {
        croak("ffiles_ha command status was: $?");
    }

    @pid = csc_get_pid(task_name => $ffiles_task);

    return @pid;
}

#---------------------------------------------------------------------

=head2 csc_hammer

csc_hammer(filer => $filer, share => $share);

This function runs hammer.exe on the filer.

The following named arguments are accepted:

=over

=item Arguments

=over

=item filer

(Required) : Name of filer to run test on.

=item share

(Required) : Share on filer to run test in.

=item runtime

(Optional) : Length of hammer execution. Defaults to 300 seconds.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_hammer(%) {
    my %opts = validate(
        @_,
        {
            filer   => { type => SCALAR },
            share   => { type => SCALAR },
            drive   => { type => SCALAR },
            runtime => { type => SCALAR, default => 300 },
        }
    );

    my ($path);
    my $filer = $opts{filer};
    my $share = $opts{share};
    my $drive = $opts{drive};
    my $runtime = $opts{runtime};

    $path = "$drive\\$share";
    open(PGM, "$hammer $path -bg -runtime $runtime -filercore $filer|");
    if ($?) {
        croak("hammer command status was: $?");
    }
    return;
}

#---------------------------------------------------------------------

=head2 csc_open_explorer

$my_computer = csc_open_explorer();

Wrapper function used to call open_my_computer in Utils_gui.pm.
This function opens explorer on a windows client.

=over

=item Returns

Returns a windows handle to the explorer window if successful,
otherwise returns a null handle.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub csc_open_explorer() {
    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(open_my_computer);

    return open_my_computer();
}

#---------------------------------------------------------------------

=head2 csc_close_explorer

csc_close_explorer(window => $whnd);

Wrapper function used to call close_my_computer in Utils_gui.pm.
This function close explorer on a windows client.

The following named arguments are accepted:

=over

=item Arguments

=over

=item window

(Required) : The handle to the explorer window.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub csc_close_explorer(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
        }
    );

    my (@pid);

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(close_my_computer);

    close_my_computer(window => $opts{window});

    return;
}

#---------------------------------------------------------------------

=head2 csc_available_offline

$ret = csc_available_offline(whnd     => $my_computer,
                             filer_ip => $filer_ip,
                             share    => $CIFS_SHARE,
                             drive    => $share_drive);

Wrapper function used to call check_available_offline in Utils_gui.pm.
Checks if the share is available offline.

The following named arguments are accepted,

=over

=item Arguments

=over

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to check if available offline

=item drive

(Optional): drive on windows client where share is mapped to

=back

=item Returns

Returns 1 if share is not available offline, 0 if it is available offline.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub csc_available_offline(%) {
    my %opts = validate(
        @_,
        {
            whnd     => { type => SCALAR },
            filer_ip => { type => SCALAR, optional => 1 },
            share    => { type => SCALAR, optional => 1 },
            drive    => { type => SCALAR, optional => 1 },
            item     => { type => SCALAR, optional => 1 },
            target   => { type => SCALAR, optional => 1 }
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(check_available_offline);

    my $item = (defined($opts{item})) ? $opts{item} :
        "$opts{share} on \'$opts{filer_ip}\' ($opts{drive})";

    return check_available_offline(whnd   => $opts{whnd},
                                   target => $opts{target},
                                   item   => $item);
}

#--------------------------------------------------------------

=head2 csc_make_available_offline

$ret = csc_make_available_offline(whnd     => $win,
                                  filer_ip => $filer_ip,
                                  share    => $share,
                                  drive    => $drive,
                                  option   => 'yes');

Wrapper function used to call make_available_offline in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Makes a share available offline.

The following named arguments are accepted,

=over

=item Arguments

=over

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to make available offline

=item drive

(Required): drive on windows client where share is mapped to

=item option

(Optional): option to make share available or not available offline.
Default to make available offline.

=back

=item Returns

Returns 1 if requested action was performed, 0 otherwise.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_make_available_offline(%) {
    my %opts = validate(
        @_,
        {
            whnd     => { type => SCALAR },
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            option   => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(make_available_offline);

    return make_available_offline(whnd     => $opts{whnd},
                                  filer_ip => $opts{filer_ip},
                                  share    => $opts{share},
                                  drive    => $opts{drive},
                                  option   => $opts{option});
}

#--------------------------------------------------------------

=head2 csc_create_files

$ret = csc_create_files(file_name => $file,
                        file_ext  => $file_extension,
                        file_path => $file_path);

Wrapper function used to call create_files in Utils_gui.pm.
Creates a file of type extension in the specified mapped share.

The following named arguments are accepted,

=over

=item Arguments

=over

=item file

(Required): name of file to create

=item file_ext

(Required): type of file to create. Supported types are .doc, .xls, and .ppt

=item file_path

(Required): path where file is to be created

=back

=item Returns

Returns 1 if the file is created, 0 if otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_create_files(%) {
    my %opts = validate(
        @_,
        {
            file_name => { type => SCALAR },
            file_ext  => { type => SCALAR },
            file_path => { type => SCALAR },
        }
    );

    my @pid;
    my $ret = 0;
    my $file = "$opts{file_path}\\$opts{file_name}.$opts{file_ext}";

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(create_files);

    create_files(file_name => $opts{file_name},
                 file_ext  => $opts{file_ext},
                 file_path => $opts{file_path});

    if ($opts{file_ext} eq 'doc') {
        @pid = csc_get_pid(task_name => "WINWORD.EXE");
    } elsif ($opts{file_ext} eq 'xls') {
        @pid = csc_get_pid(task_name => "excel.exe");
    } elsif ($opts{file_ext} eq 'ppt') {
        @pid = csc_get_pid(task_name => "powerpnt.exe");
    } else {
        @pid = csc_get_pid(task_name => "notepad.exe");
    }

    if ($pid[0]) {
        csc_kill_task(pid => $pid[0])
    }

    if (-e $file) {
        $ret = 1;
    }

    return $ret;
}

#--------------------------------------------------------------

=head2 csc_edit_file

$ret = csc_edit_file(file_name => $file,
                     file_ext  => $file_extension,
                     file_path => $file_path);

Wrapper function used to call edit_file in Utils_gui.pm.
Edits a file of type extension in the specified mapped share.

The following named arguments are accepted,

=over

=item Arguments

=over

=item file

(Required): name of file to edit

=item file_ext

(Required): type of file to edit. Supported types are .doc, .xls, and .ppt

=item file_path

(Required): path where file is located

=back

=item Returns

Returns 1 if the file timestamp is updated after editing,
0 if timestamp is not updated after editing.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_edit_file(%) {
    my %opts = validate(
        @_,
        {
            file_name => { type => SCALAR },
            file_ext  => { type => SCALAR },
            file_path => { type => SCALAR },
        }
    );

    my ($before_st, $after_st, @pid);
    my $ret = 0;
    my $file = "$opts{file_path}\\$opts{file_name}.$opts{file_ext}";

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(edit_file);

    #get modification time before edit
    $before_st = stat($file)
        or croak("Couldn't stat $file: $!");

    edit_file(file_name => $opts{file_name},
              file_ext  => $opts{file_ext},
              file_path => $opts{file_path});

    if ($opts{file_ext} eq 'doc') {
        @pid = csc_get_pid(task_name => "WINWORD.EXE");
    } elsif ($opts{file_ext} eq 'xls') {
        @pid = csc_get_pid(task_name => "excel.exe");
    } elsif ($opts{file_ext} eq 'ppt') {
        @pid = csc_get_pid(task_name => "powerpnt.exe");
    } else {
        @pid = csc_get_pid(task_name => "notepad.exe");
    }

    if ($pid[0]) {
        csc_kill_task(pid => $pid[0])
    }

    #get modification time after edit
    $after_st = stat($file)
        or croak("Couldn't stat $file: $!");

    if (($before_st->mtime != $after_st->mtime) &&
        ($after_st->mtime > $before_st->mtime)) {
        $ret = 1;
    }
    return $ret;
}

#--------------------------------------------------------------

=head2 csc_sync_folder

$ret = csc_sync_folder(filer_ip => $filer_ip,
                       share    => $share,
                       drive    => $drive,
                       whnd     => $whnd);

Wrapper function used to call sync_folder in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Syncs a folder with the filer's share.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to make available offline

=item drive

(Required): drive on windows client where share is mapped to

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=back

=item Returns

Returns 1 if the share is in the listview, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_sync_folder(%) {
    my %opts = validate(
        @_,
        {
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            whnd     => { type => SCALAR },
        }
    );

    my ($ret, @pid);

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(sync_folder);

    $ret = sync_folder(filer_ip => $opts{filer_ip},
                       share    => $opts{share},
                       drive    => $opts{drive},
                       whnd     => $opts{whnd});

    @pid = csc_get_pid(task_name => 'mobsync.exe');
    if ($pid[0]) {
        qx(taskkill /f /im mobsync.exe /t 2>&1);
    }
    return $ret;
}

#--------------------------------------------------------------

=head2 csc_get_pid

@pids = csc_get_pid(task_name => $task);

Returns a windows task process id.

The following named arguments are accepted,

=over

=item Arguments

=over

=item task_name

(Required): The name of the windows task to get process id for.

=back

=item Returns

An array of process ids that match the windows task name.
Could be empty if no task that matches name is running.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_get_pid(%) {
    my %opts = validate(
        @_,
        {
            task_name => { type => SCALAR }
        }
    );

    my($task_name, @tasklist, $i, $text);
    my($task, @pids, $pid);

    $task_name = $opts{task_name};
    $i = 0;

    @tasklist = qx(tasklist /fo list /fi "IMAGENAME eq $task_name" 2>&1);
    foreach $i (@tasklist) {
        if ($i =~ /^(PID:)/) {
           ($pid) = ($i =~ m#(\d+)#);
           push @pids, $pid;
        }
    }
    return @pids;
}

#--------------------------------------------------------------

=head2 csc_kill_task

csc_kill_task(task_name => $task);

Kills a windows task. Use this if a windows task is taking to long to complete.

The following named arguments are accepted,

=over

=item Arguments

=over

=item task_name

(Required): The name of the windows task to kill.

=back

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_kill_task(%) {
    my %opts = validate(
        @_,
        {
            pid => { type => SCALAR }
        }
    );

    my($pid);

    $pid = $opts{pid};

    qx(taskkill /f /pid "$pid" 2>&1);
    croak "csc_kill_task: command status was: $?" if $?;

    return;
}

#---------------------------------------------------------------------

=head2 csc_open_mmc

$win = csc_open_mmc(filer => $filer);

Wrapper function used to call open_mmc in Utils_gui.pm.
Opens the "MMC|Computer Management" GUI on the local Windows computer.
This is used in the CSC tests for adding, deleting and changing caching
options for shares on the filer.

The following named arguments are accepted:

=over

=item Arguments

=over

=item filer

(Required) : name of filer to connect to.

=back

=item Returns

Handle to the "Computer Management" window that was opened.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub csc_open_mmc(%) {
    my %opts = validate(
        @_,
        {
            filer => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(open_mmc);

    return open_mmc(filer => $opts{filer});
}

#---------------------------------------------------------------------

=head2 csc_close_mmc

csc_close_mmc(window => $whnd);

Wrapper function used to call close_mmc in Utils_gui.pm.
Passed a window handle to the MMC GUI, closes the window.

The following named arguments are accepted:

=over

=item Arguments

=over

=item window

(Required) : handle to MMC window.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub csc_close_mmc(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
        }
    );

    my (@pid);

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(close_mmc);

    close_mmc(window => $opts{window});

    @pid = csc_get_pid(task_name => "mmc.exe");
    if ($pid[0]) {
        csc_kill_task(pid => $pid[0])
    }
    return;
}

#--------------------------------------------------------------

=head2 csc_verify_caching_button

$msg = csc_verify_caching_button(window     => $win,
                                 share      => $share,
                                 share_attr => $share_attr);

Wrapper function used to call verify_caching_button in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Verifies the MMC caching button displays the correct caching option
for share on the filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to add share to

=item share

(Required): name of share on filer to verify caching option for.

=item caching

(Required): share caching attribute of share on filer.

=back

=item Returns

Returns a text message if unsuccessful or caching option not what expected.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_verify_caching_button(%) {
    my %opts = validate(
        @_,
        {
            window     => { type => SCALAR },
            filer      => { type => SCALAR },
            share      => { type => SCALAR },
            caching    => { type => SCALAR,
                            regex => qr/^(manual|no|auto_document|auto_program)$/ },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(verify_caching_button);

    return verify_caching_button(window  => $opts{window},
                                 filer   => $opts{filer},
                                 share   => $opts{share},
                                 caching => $opts{caching});
}

#--------------------------------------------------------------

=head2 csc_verify_path_option

$msg = csc_verify_path_option(window     => $win,
                              share      => $share,
                              share_attr => $share_attr);

Wrapper function used to call verify_share_path_and_option in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Verifies the share caching option on the filer is displayed on the MMC console.
Also verifies the share name and path is displayed on MMC.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by open_mmc.

=item filer

(Required): filer where share exists.

=item share

(Required): name of share on filer to verify caching option for.

=item path

(Required): path where share on filer should exist.

=item caching

(Required): share caching attribute share should be set to.

=back

=item Returns

Returns a text message if unsuccessful or caching option not what expected.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_verify_path_option(%) {
    my %opts = validate(
        @_,
        {
            window     => { type => SCALAR },
            filer      => { type => SCALAR },
            share      => { type => SCALAR },
            path       => { type => SCALAR },
            caching    => { type => SCALAR,
                            regex => qr/^(manual|no|auto_document|auto_program)$/ },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(verify_share_path_and_option);

    return verify_share_path_and_option(window  => $opts{window},
                                        filer   => $opts{filer},
                                        share   => $opts{share},
                                        path    => $opts{path},
                                        caching => $opts{caching});
}

#--------------------------------------------------------------

=head2 csc_change_caching_option

$ret = csc_change_caching_option(window  => $win,
                                 filer   => $filer,
                                 share   => $share,
                                 caching => $share_attr);

Wrapper function used to call change_caching_option in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Changes the share caching option on the filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by open_my_computer

=item filer

(Required): name of filer to change share caching option on

=item share

(Required): name of share on filer to change caching option on

=item caching

(Required): share caching attribute

=back

=item Returns

Returns a text message if some error occurred or share not
found in listview.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_change_caching_option(%) {
    my %opts = validate(
        @_,
        {
            window     => { type => SCALAR },
            filer      => { type => SCALAR },
            share      => { type => SCALAR },
            caching    => { type => SCALAR,
                            regex =>
                              qr/^(manual|no|document|program)$/ },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(change_caching_option);

    return change_caching_option(window  => $opts{window},
                                 filer   => $opts{filer},
                                 share   => $opts{share},
                                 caching => $opts{caching});
}

#--------------------------------------------------------------

=head2 csc_add_share

$msg = csc_add_share(window  => $win,
                     filer   => $filer,
                     share   => $share,
                     caching => $share_attr);

Wrapper function used to call add_share_xp in Utils_gui.pm.
Passed a window handle to "MMC Computer Management" GUI.
Adds a share to the filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to MMC window opened by
open_my_computer

=item filer

(Required): name of filer to add share to

=item share

(Required): name of share to add to filer

=item caching

(Optional): share caching attribute, defaults to manual_caching

=back

=item Returns

Returns a text message if some error occurred.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_add_share(%) {
    my %opts = validate(
        @_,
        {
            window     => { type => SCALAR },
            filer      => { type => SCALAR },
            vol        => { type => SCALAR },
            share      => { type => SCALAR },
            desc       => { type => SCALAR | UNDEF, optional => 1 },
            security   => { type => SCALAR | UNDEF, optional => 1 },
            caching    => { type => SCALAR, default => 'manual',
                            regex => qr/^(manual|no|document|program)$/ },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(add_share_xp);

    return add_share_xp(window   => $opts{window},
                        filer    => $opts{filer},
                        vol      => $opts{vol},
                        share    => $opts{share},
                        desc     => (defined($opts{desc})) ? $opts{desc} : undef,
                        security => (defined($opts{security})) ? $opts{security} : undef,
                        caching  => $opts{caching});
}

#--------------------------------------------------------------

=head2 csc_delete_share

$ret = csc_delete_share(window => $win,
                        filer  => $filer,
                        share  => $share);

Wrapper function used to call delete_share in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Deletes the share from the filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to delete share from

=item share

(Required): name of share to delete from filer

=back

=item Returns

Returns 1 if share deleted, otherwise 0 if not found in MMC listview.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_delete_share(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
            share  => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(delete_share);

    return delete_share(window => $opts{window},
                        filer  => $opts{filer},
                        share  => $opts{share});
}

#--------------------------------------------------------------

=head2 csc_cifs_sessions

$ret = csc_cifs_sessions(window => $win,
                         filer  => $filer);

Wrapper function used to call mmc_cifs sessions in Utils_gui.pm.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to browse from sessions

=back

=item Returns

Returns 1 if sessions browsed.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_cifs_sessions(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_cifs_sessions);

    return mmc_cifs_sessions(window => $opts{window},
                             filer  => $opts{filer});
}

#--------------------------------------------------------------

=head2 csc_close_cifs_sessions

$ret = csc_close_cifs_sessions(window => $win,
                               filer  => $filer,
                               user   => $user);

Wrapper function used to call mmc_close_cifs_sessions in Utils_gui.pm.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer

=item user

(Required): name of user session to close

=back

=item Returns

Returns 1 if session closed, otherwise 0.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_close_cifs_sessions(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
            user   => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_close_cifs_sessions);

    return mmc_close_cifs_sessions(window => $opts{window},
                                   filer  => $opts{filer},
                                   user   => $opts{user});
}

#--------------------------------------------------------------

=head2 csc_browse_files

$ret = csc_browse_files(window => $win,
                        filer  => $filer);

Wrapper function used to call mmc_browse_files in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Browses filer for open files.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to browse for open files

=back

=item Returns

Returns 1.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_browse_files(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_browse_files);

    return mmc_browse_files(window => $opts{window},
                            filer  => $opts{filer});
}

#--------------------------------------------------------------

=head2 csc_browse_local_users

$ret = csc_browse_local_users(window => $win,
                              filer  => $filer);

Wrapper function used to call mmc_local_users in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Browse local user on filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to browse for local users

=back

=item Returns

Returns 1.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_browse_local_users(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_local_users);

    return mmc_local_users(window => $opts{window},
                           filer  => $opts{filer});
}

#--------------------------------------------------------------

=head2 csc_browse_local_groups

$ret = csc_browse_local_groups(window => $win,
                               filer  => $filer);

Wrapper function used to call mmc_local_groups in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Browse local groups on the filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to browse local groups

=back

=item Returns

Returns array of groups.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_browse_local_groups(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_local_groups);

    return mmc_local_groups(window => $opts{window},
                           filer  => $opts{filer});
}

#--------------------------------------------------------------

=head2 csc_add_local_user

$ret = csc_add_local_user(window => $win,
                          filer  => $filer,
                          user   => $user,
                          passwd => $pass);

Wrapper function used to call csc_add_local_user in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Adds a local user to a local group on filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to delete share from

=item user

(Required): user to add

=item passwd

(Required): password of user

=back

=item Returns

Returns 1 if user added, otherwise 0.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_add_local_user(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
            user   => { type => SCALAR },
            passwd => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_add_local_user);

    return mmc_add_local_user(window => $opts{window},
                              filer  => $opts{filer},
                              user   => $opts{user},
                              passwd => $opts{passwd});
}

#--------------------------------------------------------------

=head2 csc_close_open_files

$ret = csc_close_open_files(window => $win,
                            filer  => $filer,
                            file   => $file);

Wrapper function used to call csc_close_open_files in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Closes an open file on filerr.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to delete share from

=item file

(Required): name of file to close

=back

=item Returns

Returns 1 if file close, otherwise 0 if not found in MMC listview.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_close_open_files(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
            file   => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_close_open_files);

    return mmc_close_open_files(window => $opts{window},
                                filer  => $opts{filer},
                                file   => $opts{file});
}

#--------------------------------------------------------------

=head2 csc_share_exists

$ret = csc_share_exists(window     => $win,
                        filer      => $filer,
                        share      => $share);

Wrapper function used to call share_exists in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Verifies that MMC displays the share.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to look for share

=item share

(Required): name of share on filer.

=back

=item Returns

Returns 1 if share found, otherwise 0.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_share_exists(%) {
    my %opts = validate(
        @_,
        {
            window     => { type => SCALAR },
            filer      => { type => SCALAR },
            share      => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(share_exists);

    return share_exists(window  => $opts{window},
                        filer   => $opts{filer},
                        share   => $opts{share});
}

#--------------------------------------------------------------

=head2 csc_share_properties

@perm = csc_share_properties(window => $win,
                             filer  => $filer,
                             share  => $share);

Wrapper function used to call mmc_share_properties in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Returns a shares's permissions.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to look for share

=item share

(Required): name of share on filer.

=back

=item Returns

Returns an array of share permissions if share found, otherwise empty array.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_share_properties(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
            share   => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_share_properties);

    return mmc_share_properties(window => $opts{window},
                                filer  => $opts{filer},
                                share  => $opts{share});
}

#--------------------------------------------------------------

=head2 csc_add_local_user

$ret = csc_add_local_user(window   => $win,
                          filer    => $filer,
                          location => 'domain',
                          add_user => $user,
                          group    => 'Users',
                          user     => $cifs_user,
                          passwd   => $cifs_user_passwd);

Wrapper function used to call csc_add_local_user in Utils_gui.pm.
Passed a window containing the "MMC Computer Management" GUI.
Adds a local user to a local group on filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer to delete share from

=item location

(Required): location where to search for the user to add to the group.
It can be either 'domain' or 'local' where local searches the filer.

=item add_user

(Required): name of user that already exists locally on the filer

=item group

(Required): name of local group on filer to add user to

=item user

(Required): name of user to log in as, or authenticate as, when doing
the operation on the filer.

=item passwd

(Required): password for authenticated user

=item Returns

Returns 1 if user added, otherwise 0.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_add_to_local_group(%) {
    my %opts = validate(
        @_,
        {
            window   => { type => SCALAR },
            filer    => { type => SCALAR },
            location => { type => SCALAR },
            add_user => { type => SCALAR },
            group    => { type => SCALAR },
            user     => { type => SCALAR },
            passwd   => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(mmc_add_to_local_group);

    return mmc_add_to_local_group(window   => $opts{window},
                                  filer    => $opts{filer},
                                  location => $opts{location},
                                  add_user => $opts{add_user},
                                  group    => $opts{group},
                                  user     => $opts{user},
                                  passwd   => $opts{passwd});
}

#--------------------------------------------------------------

=head2 csc_map_drive

$drive = csc_map_drive(window => $win,
                       filer  => $filer,
                       share  => $share,
                       user   => $user,
                       pass   => $pass);

Wrapper function used to call explorer_map_drive in Utils_gui.pm.
Passed a window containing the "explorer" GUI.
Map a share to drive for specified user.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer where share exists

=item share

(Required): name of share on filer.

=item user

(Required): name of user to map share for.

=item pass

(Required): user's password.

=back

=item Returns

Returns the drive letter where share is mapped to.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_map_drive(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
            share  => { type => SCALAR },
            user   => { type => SCALAR },
            pass   => { type => SCALAR }
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(explorer_map_drive);

    return explorer_map_drive(window => $opts{window},
                              filer  => $opts{filer},
                              share  => $opts{share},
                              user   => $opts{user},
                              pass   => $opts{pass});
}

#--------------------------------------------------------------

=head2 csc_unmap_drive

$ret = csc_unmap_drive(window => $win,
                       filer  => $filer,
                       share  => $share,
                       drive  => $drive);

Wrapper function used to call explorer_unmap_drive in Utils_gui.pm.
Passed a window containing the "explorer" GUI.
Unmap a share drive.

The following named arguments are accepted,

=over

=item Arguments

=over

=item window

(Required): handle to window opened by
open_my_computer or explorer_open_with_target

=item filer

(Required): name of filer where share exists

=item share

(Required): name of share on filer to unmap.

=item drive

(Required): drive letter where share is mapped to.

=back

=item Returns

Returns 1, if share unmapped, otherwise 0.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_unmap_drive(%) {
    my %opts = validate(
        @_,
        {
            window => { type => SCALAR },
            filer  => { type => SCALAR },
            share  => { type => SCALAR },
            drive  => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(explorer_unmap_drive);

    return explorer_unmap_drive(window => $opts{window},
                                filer  => $opts{filer},
                                share  => $opts{share},
                                drive  => $opts{drive});
}

#--------------------------------------------------------------

=head2 csc_encryption_support

$ret = csc_encryption_support(filer_ip => $filer_ip,
                              share    => $share,
                              drive    => $drive,
                              whnd     => $whnd);

Wrapper function used to call encryption_support in Utils_gui.pm.
This function is used to verify the advanced button on a folder's properties.
Used for encrypting a folder's contents.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to where file is stored

=item drive

(Required): drive on windows client where share is mapped to

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=back

=item Returns

Returns 1 if the folder's properties window supports the Advanced
button used for file encryption, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_encryption_support(%) {
    my %opts = validate(
        @_,
        {
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            file     => { type => SCALAR },
            whnd     => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(encryption_support);

    return encryption_support(filer_ip => $opts{filer_ip},
                              share    => $opts{share},
                              drive    => $opts{drive},
                              file     => $opts{file},
                              whnd     => $opts{whnd});
}

#--------------------------------------------------------------

=head2 csc_view_files

$ret = csc_view_files(filer_ip => $filer_ip,
                      share    => $share,
                      drive    => $drive,
                      whnd     => $whnd);

Wrapper function used to call view_files in Utils_gui.pm.
This function is used to view the files in the offline files folder, and to
sync selected files.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to where file is stored

=item drive

(Required): drive on windows client where share is mapped to

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=back

=item Returns

Returns 1 if the offline files folder is displayed and are allowed
to sync selected items, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_view_files(%) {
    my %opts = validate(
        @_,
        {
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            whnd     => { type => SCALAR }
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(view_files);

    return view_files(filer_ip => $opts{filer_ip},
                      share    => $opts{share},
                      drive    => $opts{drive},
                      whnd     => $opts{whnd});
}

#--------------------------------------------------------------

=head2 csc_manual_sync

$ret = csc_manual_sync(filer_ip => $filer_ip,
                       share    => $share,
                       drive    => $drive,
                       file     => $file,
                       whnd     => $whnd);

Wrapper function used to call manual_sync in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Manually syncs local file with net work file.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to where file is stored

=item drive

(Required): drive on windows client where share is mapped to

=item file

(Required): name of file to manually sync

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=back

=item Returns

Returns 1 if the file is in the listview, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_manual_sync(%) {
    my %opts = validate(
        @_,
        {
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            file     => { type => SCALAR },
            whnd     => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(manual_sync);

    return manual_sync(filer_ip => $opts{filer_ip},
                       share    => $opts{share},
                       drive    => $opts{drive},
                       file     => $opts{file},
                       whnd     => $opts{whnd});
}

#--------------------------------------------------------------

=head2 csc_sync_via_tools

$ret = csc_sync_via_tools(filer_ip => $filer_ip,
                          share    => $share,
                          drive    => $drive,
                          whnd     => $whnd);

Wrapper function used to call sync_via_tools in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Sync a offline folder via the tools menu option.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to make available offline

=item drive

(Required): drive on windows client where share is mapped to

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=back

=item Returns

Returns 1 if the item found in listview, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_sync_via_tools(%) {
    my %opts = validate(
        @_,
        {
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            whnd     => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(sync_via_tools);

    return sync_via_tools(filer_ip => $opts{filer_ip},
                          share    => $opts{share},
                          drive    => $opts{drive},
                          whnd     => $opts{whnd});
}

#--------------------------------------------------------------

=head2 csc_offline_shortcut

$ret = csc_offline_shortcut(whnd => $whnd);

Wrapper function used to call offline_shortcut in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Creates an offline files folder shortcut on the desktop.

The following named arguments are accepted,

=over

=item Arguments

=over

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=back

=item Returns

Returns 1 if the offline files folder is in the listview, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_offline_shortcut(%) {
    my %opts = validate(
        @_,
        {
            whnd => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(offline_shortcut);

    return offline_shortcut(whnd => $opts{whnd});
}

#--------------------------------------------------------------

=head2 csc_delete_offline_files

$ret = csc_delete_offline_files(filer_ip => $filer_ip,
                                share    => $share,
                                drive    => $drive,
                                whnd     => $whnd,
                                option   => $option);

Wrapper function used to call delete_offline_files in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Handles the delete button in the folder tools options menu item.
Used to delete offline files.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to make available offline

=item drive

(Required): drive on windows client where share is mapped to

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=item option

(Optional): Option to delete both temporary files or all files.
Defaults to temp.

=back

=item Returns

Returns 1 if the share is in the listview, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_delete_offline_files(%) {
    my %opts = validate(
        @_,
        {
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            whnd     => { type => SCALAR },
            option   => { type => SCALAR, default => 'temp',
                          regex => qr/^(temp|both)$/ }
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(delete_offline_files);

    return delete_offline_files(filer_ip => $opts{filer_ip},
                                share    => $opts{share},
                                drive    => $opts{drive},
                                whnd     => $opts{whnd},
                                option   => $opts{option});
}

#--------------------------------------------------------------

=head2 csc_advanced_button

$ret = csc_advanced_button(filer_ip => $filer_ip,
                           share    => $share,
                           drive    => $drive,
                           whnd     => $whnd,
                           option   => $option);

Wrapper function used to call advanced_button in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Handles the advanced offline settings button.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to make available offline

=item drive

(Required): drive on windows client where share is mapped to

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=item option

(Optional): Option for going offline notification.
Defaults to notify, when going offline.

=back

=item Returns

Returns 1 if the share is in listview, 0 otherwise.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_advanced_button(%) {
    my %opts = validate(
        @_,
        {
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            whnd     => { type => SCALAR },
            option   => { type => SCALAR, default => 'notify',
                          regex => qr/^(notify|never)$/ }
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(advanced_button);

    return advanced_button(filer_ip => $opts{filer_ip},
                           share    => $opts{share},
                           drive    => $opts{drive},
                           whnd     => $opts{whnd},
                           option   => $opts{option});
}

#---------------------------------------------------------------------

=head2 csc_vdthrash_cmdline

$win = csc_vdthrash_cmdline(filer => $filer, share => $share);

This function runs vdthrash.exe on the filer using a command line startup.
For execution of vdthrash.exe using Utils_gui.pm interface, see
csc_vdthrash and csc_vdthrash_close below.

The following named arguments are accepted:

=over

=item Arguments

=over

=item filer

(Required) : Name of filer to run test on.

=item share

(Required) : Share on filer to run test in.

=back

=item Returns

An array of pids to the executing process.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_vdthrash_cmdline(%) {
    my %opts = validate(
        @_,
        {
            filer => { type => SCALAR },
            share => { type => SCALAR },
        }
    );

    my $filer = $opts{filer};
    my $share = $opts{share};
    my (@pid);

    # Execute vdthrash command
    open(PGM, "$vdthrash -f\\\\$filer\\$share\\$share\\testfile_vdthrash -b4 -Ok|");
    if ($?) {
        croak("vdthrash command status was: $?");
    }

    @pid = csc_get_pid(task_name => $vdthrash_task);

    return @pid;
}

#---------------------------------------------------------------------

=head2 csc_vdthrash

$win = csc_vdthrash(filer => $filer, share => $share);

Wrapper function used to call vdthrash in Utils_gui.pm.
This function runs vdthrash.exe on the filer.

The following named arguments are accepted:

=over

=item Arguments

=over

=item filer

(Required) : Name of filer to run test on.

=item share

(Required) : Share on filer to run test in.

=back

=item Returns

Returns handle to vdthrash window if successfully opened, 0 otherwise.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_vdthrash(%) {
    my %opts = validate(
        @_,
        {
            filer => { type => SCALAR },
            share => { type => SCALAR },
            drive => { type => SCALAR },
            exec_path => { type => SCALAR, optional => 1 }
        }
    );
    my (%args);

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(vdthrash);

    if (defined($opts{exec_path})) {
        %args = ( exec_path => "$opts{exec_path}" );
    }
    return vdthrash(filer     => $opts{filer},
                    share     => $opts{share},
                    drive     => $opts{drive},
                    %args);
}

#---------------------------------------------------------------------

=head2 csc_vdthrash_close

csc_vdthrash_close();

Wrapper function used to call vdthrash_close in Utils_gui.pm.
This function closes vdthrash window.

The following named arguments are accepted:

=over

=item Arguments

=over

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------

sub csc_vdthrash_close() {
    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(vdthrash_close);

    vdthrash_close();
    return;
}

#--------------------------------------------------------------

=head2 sync_via_my_network

$ret = sync_via_my_network(filer    => $filer,
                           filer_ip => $filer_ip,
                           share    => $share,
                           drive    => $drive,
                           whnd     => $whnd);

Passed a window handle to "Explorer My Computer" GUI.
Syncs an offline share with filer via my network.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer

(Required): name of the filer

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to make available offline

=item drive

(Required): drive on windows client where share is mapped to

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=back

=item Returns

Returns 1 if sync successful, 0 otherwise.

=back

=for html <hr />

=cut
#--------------------------------------------------------------

sub csc_sync_via_my_network(%) {
    my %opts = validate(
        @_,
        {
            filer    => { type => SCALAR },
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            drive    => { type => SCALAR },
            whnd     => { type => SCALAR },
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(sync_via_my_network);

    return sync_via_my_network(filer    => $opts{filer},
                               filer_ip => $opts{filer_ip},
                               share    => $opts{share},
                               drive    => $opts{drive},
                               whnd     => $opts{whnd});
}

#--------------------------------------------------------------

=head2 csc_xcopy

csc_xcopy(source_path => $source,
          dest_path   => $dest);

Edits a file of type extension in the specified mapped share.

The following named arguments are accepted,

=over

=item Arguments

=over

=item source

(Required): source path of where to xcopy from

=item dest

(Required): destination path of where to xcopy to

=back

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_xcopy( % ) {
    my %opts = validate(
        @_,
        {
            source_path  => { type => SCALAR },
            dest_path    => { type => SCALAR },
        }
    );

    my($source_path, $dest_path, $text);
    my(@files, $files1, $files2);

    $source_path = $opts{source_path};
    $dest_path   = $opts{dest_path};

    $text = qx(xcopy "$source_path" "$dest_path" /E /Q /I /F /C /Y 2>&1);
    croak "xcopy command status was: $?: $text" if $?;

    return;
}

#--------------------------------------------------------------

=head2 csc_remove_folder

$ret = csc_remove_folder(filer => $filer,
                         share => $share,
                         dir   => $dir);

Remove a folder from a mapped share on the filer.

The following named arguments are accepted,

=over

=item Arguments

=over

=item filer

(Required): filer where share exist

=item share

(Required): name of the share on the filer

=item dir

(Required): directory on share to be removed

=back

=item Returns

Returns 1 if directory removed, wise.

=back

=for html <hr />

=cut

#--------------------------------------------------------------
sub csc_remove_folder( % ) {
    my %opts = validate(
        @_,
        {
            filer => { type => SCALAR },
            share => { type => SCALAR },
            dir   => { type => SCALAR }
        }
    );

    my($dest_path, $text, $filer, $share, $dir);

    $filer = $opts{filer};
    $share = $opts{share};
    $dir   = $opts{dir};

    $text = qx(rmdir /S /Q "\\\\$filer\\$share\\$dir" 2>&1);
    $@ = "rmdir command status was: $?: $text" if $?;

    return ($? ? 1 : 0);
}

#--------------------------------------------------------------

=head2 csc_verify_dir_sync

$ret = csc_verify_dir_sync(source_path => $source,
                           dest_path   => $dest);

Verifies if directories are synced. Check if the files match and also
if the number of files match.

The following named arguments are accepted,

=over

=item Arguments

=over

=item source

(Required): the source directory

=item dest

(Required): the destination directory

=back

=item Returns

Returns 1 if directories match, 0 otherwise.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_verify_dir_sync(%) {
    my %opts = validate(
        @_,
        {
            source_files => { type => SCALAR },
            dest_files   => { type => SCALAR }
        }
    );

    my(@source, @dest, $source_count, $dest_count, $ret);
    my(@sorted_dest, $source_file, $dest_file, $i, $j);
    my(@dest_file, $name, $path, $suffix, $matches);
    my($name_source, $path_source, $suffix_source);

    @source = split(/[\r\r\n]/,  $opts{source_files});
    @dest   = split(/[\r\r\n]/,  $opts{dest_files});

    $source_count = @source;
    $dest_count   = @dest;

    if ($source_count != $dest_count) {
        return 0;
    }

    @sorted_dest = sort { lc($a) cmp lc($b)} @dest;

    $matches = 0;
    for ($i = 0; $i < $dest_count; $i++) {
        ($name,$path,$suffix) = fileparse($dest[$i]);
        for ($j = 0; $j < $source_count; $j++) {
            ($name_source,$path_source,$suffix_source) = fileparse($source[$j]);
            if ($name eq $name_source) {
                $ret = compare($dest[$i], $source[$j]);
                if ($ret) {
                    return 0;
                } else {
                    $matches++;
                }
            }
        }
    }

    return ($matches == $dest_count) ? 1 : 0;
}

#--------------------------------------------------------------

=head2 csc_folder_offline

$ret = csc_folder_offline(whnd     => $win,
                          filer_ip => $filer_ip,
                          share    => $share,
                          drive    => $drive,
                          option   => 'yes');

Wrapper function used to call folder_avail_offline in Utils_gui.pm.
Passed a window handle to "Explorer My Computer" GUI.
Makes a folder available offline.

The following named arguments are accepted,

=over

=item Arguments

=over

=item whnd

(Required): handle to Explorer My Computer window opened by
open_my_computer

=item filer_ip

(Required): the filer's ip address

=item share

(Required): name of share to make available offline

=item drive

(Required): drive on windows client where share is mapped to

=item option

(Optional): option to make share available or not available offline.
Default to make available offline.

=back

=item Returns

Returns 1 or 0 depending on option. If option 'yes' returns 1
if share was not available offline, returns 0 if was available offline.
If option 'no' returns 1 if share was available offline, returns 0
if share was not available offline.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_folder_offline(%) {
    my %opts = validate(
        @_,
        {
            whnd     => { type => SCALAR },
            filer_ip => { type => SCALAR },
            share    => { type => SCALAR },
            folder   => { type => SCALAR },
            drive    => { type => SCALAR },
            option   => { type => SCALAR, default => 'yes',
                          regex => qr/^(yes|no)$/  }
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(folder_avail_offline);

    return folder_avail_offline(whnd     => $opts{whnd},
                                filer_ip => $opts{filer_ip},
                                share    => $opts{share},
                                folder   => $opts{folder},
                                drive    => $opts{drive},
                                option   => $opts{option});
}

#--------------------------------------------------------------

=head2 csc_folder_redirect

$ret = csc_folder_redirect(whnd   => $win,
                           target => $target,
                           item   => $item,
                           drive  => $drive,
                           action => $action);

Wrapper function used to call folder_properties in Utils_gui.pm.
Passed a window handle to Explorer, opens a folders properties
window for folder redirection or restoration to default location
based on param action.

The following named arguments are accepted,

=over

=item Arguments

=over

=item whnd

(Required): handle to Explorer window

=item target

(Required): where folder resides, (Desktop)

=item item

(Required): folder in target, folder to opens properties window for.

=item drive

(Required): if action is folder redirection, share drive to redirect
folder to.

=item action

(Optional): action to take, folder redirection or restoration
to default location. Defaults to 'redirect'.

=item wnd_text

(Optional): action to take, folder redirection or restoration
to default location. Defaults to 'redirect'.

=back

=item Returns

Returns 1 if action successful, 0 otherwise.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_folder_redirect(%) {
    my %opts = validate(
        @_,
        {
            whnd     => { type => SCALAR },
            target   => { type => SCALAR },
            item     => { type => SCALAR },
            drive    => { type => SCALAR },
            action   => { type => SCALAR }
        }
    );

    require CIFS::Utils_gui;
    import CIFS::Utils_gui qw(folder_properties);

    return folder_properties(whnd     => $opts{whnd},
                             target   => $opts{target},
                             item     => $opts{item},
                             drive    => $opts{drive},
                             action   => $opts{action});
}

#--------------------------------------------------------------

=head2 csc_copy_file

csc_copy_file(source  => $source,
              dest    => $dest);

Copy a file of type extension in the specified mapped share.

The following named arguments are accepted,

=over

=item Arguments

=over

=item source

(Required): source file

=item dest

(Required): destination file

=back

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_copy_file(%) {
    my %opts = validate(
        @_,
        {
            source => { type => SCALAR },
            dest   => { type => SCALAR },
        }
    );

    my($source, $dest);

    $source = $opts{source};
    $dest   = $opts{dest};

    if (! -e $source) {
        return "source file $source does not exist";
    }

    system("copy /Y $source $dest");
    if ($?) {
        return "copy command status was: $?";
    }

    sleep(10);

    return (undef);
}

#--------------------------------------------------------------

=head2 csc_install

$msg = csc_install(file => $file);

Runs MSIEXEC on the msi file passed in.

The following named arguments are accepted,

=over

=item Arguments

=over

=item file

(Required): name of the msi file to run MSIEXEC on.

=back

=item Returns

Returns error msg if file does not exist.

=back

=for html <hr />

=cut

#--------------------------------------------------------------

sub csc_install(%) {
    my %opts = validate(
        @_,
        {
            file => { type => SCALAR,
                      regex => qr/(msi)$/ },
        }
    );

    my $file = $opts{file};
    my $msiexec = "msiexec /i $file /q";

    if (! -e $file) {
        return "$file does not exist";
    }

    system($msiexec);

    return (undef);
}

#---------------------------------------------------------------------

=head2 csc_client_setup

csc_client_setup(setup_h      => \%setup_h,
                 [clients      => \@clients, ]
                 [cifs_clients => \@cifs_clients, ]
                 ret_h         => \%setup_h);

Sets up the clients for a multi client test. This function
is used by csc_func/edit.thpl. This set calls for 2 CIFS clients.
So test is run by an NFS client as the test server.

The following named arguments are accepted:

=over

=item Arguments

=over

=item mode

(Optional) : Protocol setting for the filer can be 'ntfs' or
             'multi'. Defaults to 'multi'.

=item create_vol

(Optional) : If create_vol => 'yes' create default volume.
             If create_vol => 'no' do not create default volume.

=item clients

(Optional) : Array reference of NFS client hostspecs.

=item cifs_clients

(Optional) : Array reference of CIFS client hostspecs.

=item create_vol

(Optional) : If create_vol => 'yes' create default volume.
             If create_vol => 'no' do not create default volume.

=item nfs_enable

(Optional) : Enable NFS on create_vol.
Default is yes.

=item ret_h

(Required) : Hash ref passed by caller. To be used by caller and csc_cleanup.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_client_setup(%) {
    my %opts = validate(
        @_,
        {
            mode            => { type => SCALAR, default => 'multi',
                                 regex => qr/^(ntfs|multi)$/ },
            clients         => { type => ARRAYREF,  default => ['localhost'] },
            cifs_clients    => { type => ARRAYREF, optional => 1 },
            create_vol      => { type => SCALAR, optional => 1,
                                 regex => qr/^(yes|no)$/ },
            nfs_enable      => { type => SCALAR, default => 'yes',
                                 regex => qr/^(yes|no)$/ },
            ret_h           => { type => HASHREF }
          }
    );

    my($cifs_passwd, $cifs_user, $fc, $fh, $filer_vol_path, $new_text,
       $purge, $op, %shares_h, $hostname, $share_name, $volume,
       $mountpoint);
    my $set_h = $opts{ret_h};

    # Get connection object to filer
    eval {
        $fh = host($main::FILER);
        $hostname = $fh->hostname();
        $fc = connect($main::FILER_CONN, -timeout, $main::TIMEOUT);
        $fc->timeout($main::CMDTIMEOUT);
    };
    croak("csc_setup: Get connection object to filer $@") if ($@);

    $set_h->{fc} = $fc;
    $set_h->{shares} = \%shares_h;

    logStep("csc_client_setup: Create volume");
    ($fc, $volume) = create_vol(nfs_enable => $opts{nfs_enable})
        or croak("csc_client_setup: Create vol failed $@");
    $set_h->{vol} = $volume;

    # Create username to use for cifs based on whether AD is setup or not
    # Default is workgroup
    $hostname =~ tr/a-z/A-Z/;
    $cifs_user = "$hostname\\$main::CIFS_USER";
    $cifs_passwd = "$main::CIFS_PASSWD";
    if (defined($main::CIFS_DOMAIN)) {
        $main::CIFS_DOMAIN =~ s/\s+//g;
        if ($main::CIFS_DOMAIN ne '') {
            # Using Domains
            $cifs_user = "$main::CIFS_DOMAIN_ADMIN";
            $cifs_passwd = "$main::CIFS_DOMAIN_ADMIN_PASSWD";
        }
    }

    logStep("csc_client_setup: Setup $usermap file on filer");
    $filer_usermap_text = rdfile($fc, $usermap);
    $hostname =~ tr/a-z/A-Z/;
    $new_text = "\n$cifs_user == root\n";
    wrfile($fc, $usermap, $filer_usermap_text . $new_text)
        or croak("csc_client_setup: can't write $usermap: $@");

    logStep('Get client connection objects');
    my $cifs_cl = 0;
    foreach my $client ( @{$opts{clients}}, '--', @{$opts{cifs_clients}} ) {
        if ($client eq '--') {
            # Now connecting to CIFS clients
            $cifs_cl = 1;
            next;
        }

        # Validate type the NFS client
        my $fh = host($client);
        if (!$cifs_cl and $fh->hosttype() !~ /^unix/) {
            croak "csc_client_setup: $client is not an NFS client";
        }

        # Get connection object for client
        push @{$set_h->{cc}}, connect('#rsh', '-timeout', $main::TIMEOUT);
        $set_h->{cc}[-1]->timeout($main::CMDTIMEOUT);
    }

    # If CIFS clients are specified, setup CIFS on the filer
    logStep('Setup CIFS on the filer');
    ntfs_cifs_setup(filer    => $main::FILER,
                    mode     => $opts{mode});
    croak("csc_client_setup: cifs setup failed $@") if $@;

    $filer_vol_path = "/vol/$main::FILER_VOL";
    $share_name = $main::CIFS_SHARE;
    $purge = ($share_name =~ /^(etc\$|c\$|home)$/i) ? 0 : 1;
    share_add($fc, $share_name, $filer_vol_path, verify => 1,
              purge_existing => $purge, umask => 0)
        or croak("csc_client_setup: failed to add share $main::CIFS_SHARE");
    $shares_h{$share_name} = $purge;

    my @cifs_clients = @{$opts{cifs_clients}};
    foreach my $client (@cifs_clients) {
        # Map volume on client
        $op = map_cifs_share(server => $fc->connhost(),
                             client => $client,
                             share  => $main::CIFS_SHARE,
                             drive  => '*',
                             username => $cifs_user,
                             password => $cifs_passwd)
            or croak "$@";
        push @{$set_h->{cifs}}, { drive => $op, host => $client };
    }

    my $vol_path = "/vol/$volume";
    ## @step Mount volume
    logStep("Mount volume $vol_path");
    $mountpoint =  mount($fc->connhost(),
                         $vol_path,
                         chmod   => '0777',
                         client => $set_h->{cc}[0],
                         mntopts => 'actimeo=0,noac,hard,intr,noquota')
        or croak "csc_client_setup: $@";

    logStep('Export test volume');
    export_path($fc, $vol_path, OPTIONS => 'rw,anon=0')
      or croak "csc_client_setup: $@";

    # Save mountpoint so if setup should fail
    # it will get properly unmounted
    $set_h->{mtpoint} = $mountpoint;

    return;
}

#---------------------------------------------------------------------

=head2 csc_client_cleanup

csc_client_cleanup([  fc => $fc ]
                   [, cc => \@cc ]
                   [, cifs => \@cifs_drives ]
                   [, shares => \%shares ]
                   [, vol => $vol ]
                   [, mtpoint => $mtpoint ]
                   [, files => \@files ]);

Cleans up the clients from a multi client test. Unmaps the CIFS shares
the were mapped for the CIFS clients. This function is used by
csc_func/edit.thpl.

The following named arguments are accepted:

=over

=item Arguments

=over

=item fc

(Optional) : Filer connection object.

=item cc

(Optional) : Array reference of client connection objects.

=item cifs

(Optional) : Array reference of CIFS drives to unmapped.
Each element is a hash having 'drive' and 'host' as keys.

=item shares

(Optional) : shares to delete after test run.

=item vol

(Optional) : Absolute path of volume.

=item mtpoint

(Optional) : The NFS client mount point of volume.

=item files

(Optional) : Array reference of files to remove.
Each entry can either be a file name or a hash reference having
the file name on the 'file' key.
Files names are relative to the 'volume' option.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub csc_client_cleanup(%) {
    my %opts = validate(
        @_,
        {
            fc      => { type => OBJECT   | UNDEF,  optional => 1 },
            cc      => { type => ARRAYREF | UNDEF,  optional => 1 },
            cifs    => { type => ARRAYREF | UNDEF,  optional => 1 },
            shares  => { type => HASHREF  | UNDEF,  optional => 1 },
            vol     => { type => SCALAR   | SCALAR, optional => 1 },
            mtpoint => { type => SCALAR   | UNDEF,  optional => 1 },
            files   => { type => ARRAYREF | UNDEF,  optional => 1 },
        }
    );

    my(@arr, $fc, $ret_val);

    logStep("CLEANUP == $main::CLEANUP");

    eval {
        logStep('Re-connect to filer');
        host($main::FILER);
        $fc = connect($main::FILER_CONN, '-timeout', $main::TIMEOUT);
        $fc->timeout($main::CMDTIMEOUT);

        if ($main::CLEANUP ne 'none') {
            if ($opts{vol} and $opts{cc}[0] and $opts{mtpoint}) {
                if (defined($opts{files}) && @{$opts{files}} > 0) {
                    logStep("csc_client_cleanup: unlinking files");
                    my $file;
                    logStep('Remove created files');
                    foreach (@{$opts{files}}) {
                        $file = ref($_) eq 'HASH' ? $_->{file} : $_;
                        eval {
                            remove_file(client => $opts{cc}[0],
                                        file   => "$opts{mtpoint}/$file")
                        };

                        logWarn("csc_client_cleanup: $@") if $@;
                    }
                }

                my $vol_path = "/vol/$opts{vol}";
                if ($opts{cc}[0] && $opts{mtpoint}) {
                    logStep("csc_client_cleanup: unmounting volume");
                    $ret_val = umount($opts{mtpoint}, client=>$opts{cc}[0],
                                      force => '1');
                    if (!$ret_val) {
                        logWarn("csc_client_cleanup: $@") if $@;
                    }
                }

                eval {
                    logStep("csc_client_cleanup: unexporting path");
                    $ret_val = unexport_path($fc, $vol_path);
                    if (!$ret_val) {
                        logWarn("csc_client_cleanup: $@") if $@;
                    }
                };
                logWarn("csc_client_cleanup: $@") if $@;
            }

            if ($opts{cifs}) {
                logStep("csc_client_cleanup: deleting shares");
                @arr = keys(%{$opts{shares}});
                foreach my $i (@arr) {
                    if (${$opts{shares}}{$i} ==  1) {
                        eval {
                            share_delete($fc, $i);
                        };

                        logWarn("csc_client_cleanup: $@") if $@;
                    }
                }

                # Unmapping shares is done after the shares are removed
                # which is backwards but is done this way because sometimes
                # windows thinks files are still open even if they have been
                # closed. The force option on unmap_cifs_share doesn't work
                # so this is the only thing that seems to work.
                logStep('Unmap volume on CIFS clients');
                foreach my $cifs (@{$opts{cifs}}) {
                    unmap_cifs_share(mappath => $cifs->{drive},
                                     client  => $cifs->{host},
                                     force => 1);
                    logWarn("csc_client_cleanup: $@") if $@;
                }

            }

            if ($fc && $filer_usermap_text) {
                logStep("csc_client_cleanup: Restore $usermap file on filer");
                wrfile($fc, $usermap, $filer_usermap_text)
                    or logWarn("csc_client_cleanup: can't write $usermap: $@");
            }
        }

        # Call common cleanup if create_vol was called
        if ($fc) {
            common_cleanup($fc);
            logWarn("csc_client_cleanup: common_cleanup: $@") if $@;
        }
    };

    logWarn("csc_client_cleanup: $@") if $@;
    logStep('END: CLEANUP');

    return;
}

#---------------------------------------------------------------------

=head2 testme

Self test routine for this library.

Use test_lib.thpl to invoke this routine.

Example
    ntest -sim PKGNAME="CIFS::Csc" SUBNAME=testme CLIENT='minwin_clnt'
    CIFS_DOMAIN='cifs.lab.domain' CIFS_DOMAIN_ADMIN='admin_name'
    CIFS_DOMAIN_ADMIN_PASSWD='password' FILER=myfiler util/test_pkg.thpl

=over

=item Arguments

=over

=item CLIENT

(Required) : Minwin compatible client on which to run testme.

=item FILER

(Required) : Filer to test.

=back

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub testme() {
    use CIFS::CommonParams;
    use Utils qw(subtest_remote);
    use TestEnv::MinWin qw(minwin_compatible);

    my $host = host($main::CLIENT);
    if ($host->hosttype() !~ /windows/ || !minwin_compatible($main::CLIENT)) {
        logresult('CONFIG', 'The host ' . $host->hostname() .
                  ' is not MinWin compatible, exiting.');
        return;
    }

    if ($main::CLIENT ne 'localhost') {
        eval {
            subtest_remote(test   => 'util/test_pkg.thpl',
                           client => $main::CLIENT);
        };

        logresult('CONFIG', $@) if ($@);
        exit(0);
    }

    my (%setup_h, $fc);

    # Setup environment
    eval {
        csc_setup(mode => 'multi',
                  ret_h => \%setup_h);
    };

    if ($@) {
        my $err = $@;
        # Clean environment
        csc_cleanup(%setup_h);
        logresult('FAIL', $err);
        return;
    }

    # Clean environment
    eval {
        csc_cleanup(%setup_h);
    };

    if ($@) {
        logresult('FAIL', $@);
    } else {
        logresult('PASS', 'testme(): Csc.pm');
    }
    return;
}

1;
