/*
 * Copyright (C) 2012-2013 by NetApp, Inc.  All Rights Reserved.
 *
 * @file fps_schema.cc
 *
 * @author Sandeep Joshi
 *
 * @date 01-25-2012
 *
 * Languages Used : C & C++
 *
 * @brief This file contains code parsing XML message received from
 *        NetApp storage appliance. It doesn't contain code for
 *        validation of messages though.
 */

#define LIBXML_SCHEMAS_ENABLED
#include <libxml/xmlschemastypes.h>
#include <libxml/xmlreader.h>
#include "fps_schemas.h"
#include <assert.h>
#include "fps_notification.h"
#include "fps_server.h"

class FSMConnection;
/* These hold the parsed schema definition for different types of response
 * received from fpolicy server.
 */

/* These hold the parsed schema definition for different types of response
 * received from fpolicy server.
 */

xmlSchemaPtr hdrReqSchema;
xmlSchemaPtr negoReqSchema;
xmlSchemaPtr screenReqSchema;
xmlSchemaPtr statusReqSchema;
xmlSchemaPtr alertsSchema;

extern int32_t  Response;
extern int32_t  delay;
extern int32_t  mode;

/* Schema Definitions for different responses from fpolicy server */
char XSDHeaderReq[] =
"<?xml version=\"1.0\"?>"
"<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">"
"   <xs:simpleType name=\"NotfTypeE\">"
"       <xs:restriction base=\"xs:string\">"
"           <xs:enumeration value=\"NEGO_REQ\"/>"
"           <xs:enumeration value=\"NEGO_RESP\"/>"
"           <xs:enumeration value=\"SCREEN_REQ\"/>"
"           <xs:enumeration value=\"SCREEN_RESP\"/>"
"           <xs:enumeration value=\"VOL_NOTFICATION\"/>"
"           <xs:enumeration value=\"SCREEN_CANCEL\"/>"
"           <xs:enumeration value=\"STATUS_QUERY_REQ\"/>"
"           <xs:enumeration value=\"STATUS_QUERY_RESP\"/>"
"           <xs:enumeration value=\"BACK_PRESSURE_APPLY\"/>"
"           <xs:enumeration value=\"BACK_PRESSURE_REM\"/>"
"           <xs:enumeration value=\"KEEP_ALIVE\"/>"
"           <xs:enumeration value=\"ALERT_MSG\"/>"
"       </xs:restriction>"
"   </xs:simpleType>"
"   <xs:simpleType name=\"DataFormatE\">"
"       <xs:restriction base=\"xs:string\">"
"           <xs:enumeration value=\"XML\"/>"
"       </xs:restriction>"
"   </xs:simpleType>"
"   <xs:element name=\"Header\">"
"       <xs:complexType>"
"           <xs:sequence>"
"               <xs:element name=\"NotfType\" type=\"NotfTypeE\" nillable=\"true\"/>"
"               <xs:element name=\"ContentLen\" type=\"xs:unsignedInt\" nillable=\"true\"/>"
"               <xs:element name=\"DataFormat\" type=\"DataFormatE\" nillable=\"true\"/>"
"           </xs:sequence>"
"       </xs:complexType>"
"   </xs:element>"
"</xs:schema>";

char XSDHandShakeReq[] =
"<?xml version=\"1.0\"?>"
"<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">"
"   <xs:complexType name=\"ProtVersT\">"
"       <xs:sequence>"
"           <xs:element name=\"Vers\" type=\"xs:string\" minOccurs=\"1\" maxOccurs=\"unbounded\"/>"
"       </xs:sequence>"
"   </xs:complexType>"
"   <xs:element name=\"Handshake\">"
"       <xs:complexType>"
"           <xs:sequence>"
"		<xs:element name=\"VsUUID\" nillable=\"true\">"
"                   <xs:simpleType>"
"                       <xs:restriction base=\"xs:string\">"
"                           <xs:minLength value=\"1\"/>"
"                           <xs:maxLength value=\"36\"/>"
"                       </xs:restriction>"
"                   </xs:simpleType>"
"               </xs:element>"
"               <xs:element name=\"PolicyName\" type=\"xs:string\" nillable=\"true\"/>"
"               <xs:element name=\"SessionId\" type=\"xs:string\" nillable=\"true\"/>"
"               <xs:element name=\"ProtVersion\" type=\"ProtVersT\"/>"
"           </xs:sequence>"
"       </xs:complexType>"
"   </xs:element>"
"</xs:schema>";

char XSDScreenReq[] =
"<?xml version=\"1.0\"?>"
"<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">"
"    <xs:simpleType name=\"ReqTypeE\">"
"        <xs:restriction base=\"xs:string\">"
"            <xs:enumeration value=\"SMB_OPEN\"/>"
"            <xs:enumeration value=\"SMB_CLOSE\"/>"
"            <xs:enumeration value=\"SMB_CREAT\"/>"
"            <xs:enumeration value=\"SMB_CREAT_DIR\"/>"
"            <xs:enumeration value=\"SMB_DEL\"/>"
"            <xs:enumeration value=\"SMB_DEL_DIR\"/>"
"            <xs:enumeration value=\"SMB_REN\"/>"
"            <xs:enumeration value=\"SMB_REN_DIR\"/>"
"            <xs:enumeration value=\"SMB_GET_ATTR\"/>"
"            <xs:enumeration value=\"SMB_SET_ATTR\"/>"
"            <xs:enumeration value=\"SMB_RD\"/>"
"            <xs:enumeration value=\"SMB_WR\"/>"
"            <xs:enumeration value=\"NFS_LOOKUP\"/>"
"            <xs:enumeration value=\"NFS_CREAT\"/>"
"            <xs:enumeration value=\"NFS_CREAT_DIR\"/>"
"            <xs:enumeration value=\"NFS_DEL\"/>"
"            <xs:enumeration value=\"NFS_DEL_DIR\"/>"
"            <xs:enumeration value=\"NFS_REN\"/>"
"            <xs:enumeration value=\"NFS_REN_DIR\"/>"
"            <xs:enumeration value=\"NFS_OPEN\"/>"
"            <xs:enumeration value=\"NFS_CLOSE\"/>"
"            <xs:enumeration value=\"NFS_RD\"/>"
"            <xs:enumeration value=\"NFS_WR\"/>"
"            <xs:enumeration value=\"NFS_GET_ATTR\"/>"
"            <xs:enumeration value=\"NFS_SET_ATTR\"/>"
"            <xs:enumeration value=\"NFS_LNK\"/>"
"            <xs:enumeration value=\"NFS_SYM_LNK\"/>"
"        </xs:restriction>"
"    </xs:simpleType>"
"    <xs:simpleType name=\"UsrIdE\">"
"        <xs:restriction base=\"xs:string\">"
"            <xs:enumeration value=\"UNIX_ONLY\"/>"
"            <xs:enumeration value=\"WIN_ONLY\"/>"
"            <xs:enumeration value=\"MAPPED_ID\"/>"
"        </xs:restriction>"
"    </xs:simpleType>"
"    <xs:complexType name=\"MappedIdT\">"
"        <xs:sequence>"
"            <xs:element name=\"Uid\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"WinSid\" type=\"xs:string\" nillable=\"true\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"UsrIdU\">"
"        <xs:choice>"
"            <xs:element name=\"UnixUid\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"WinSid\" type=\"xs:string\" nillable=\"true\"/>"
"            <xs:element name=\"MappedId\" type=\"MappedIdT\"/>"
"        </xs:choice>"
"    </xs:complexType>"
"    <xs:simpleType name=\"FileTypeE\">"
"        <xs:restriction base=\"xs:string\">"
"            <xs:enumeration value=\"FILE\"/>"
"            <xs:enumeration value=\"DIR\"/>"
"            <xs:enumeration value=\"STREAM\"/>"
"            <xs:enumeration value=\"SYMLINK\"/>"
"        </xs:restriction>"
"    </xs:simpleType>"
"    <xs:simpleType name=\"PathNameE\">"
"        <xs:restriction base=\"xs:string\">"
"             <xs:enumeration value=\"WIN_NAME\"/>"
"             <xs:enumeration value=\"WIN_8_3_NAME\"/>"
"             <xs:enumeration value=\"UNIX_NAME\"/>"
"        </xs:restriction>"
"    </xs:simpleType>"
"    <xs:complexType name=\"PathNameT\">"
"        <xs:sequence>"
"            <xs:element name=\"PathNameType\" type=\"PathNameE\" nillable=\"true\"/>"
"            <xs:element name=\"PathName\" type=\"xs:string\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"   <xs:complexType name=\"AccessPathT\">"
"       <xs:sequence>"
"           <xs:element name=\"Path\" type=\"PathNameT\" minOccurs=\"0\" maxOccurs=\"3\"/>"
"       </xs:sequence>"
"   </xs:complexType>"
"    <xs:complexType name=\"ProtCommonInfoT\">"
"        <xs:sequence>"
"            <xs:element name=\"ClientIp\" type=\"xs:string\" nillable=\"true\"/>"
"            <xs:element name=\"UsrIdType\" type=\"UsrIdE\"/>"
"            <xs:element name=\"UsrContext\" type=\"UsrIdU\"/>"
"            <xs:element name=\"FileOwner\" type=\"UsrIdU\" />"
"            <xs:element name=\"AccessPath\" type=\"AccessPathT\"/>"
"            <xs:element name=\"VolMsid\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"FileSize\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"NumHardLnk\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"IsOfflineAttr\" type=\"xs:boolean\"/>"
"            <xs:element name=\"FileType\" type=\"FileTypeE\"/>"
"            <xs:element name=\"IsSparse\" type=\"xs:boolean\"/>"
"            <xs:element name=\"IsDense\" type=\"xs:boolean\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbProtVerT\">"
"        <xs:sequence>"
"            <xs:element name=\"MajorNum\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"MinorNum\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbCommonInfoT\">"
"        <xs:sequence>"
"            <xs:element name=\"ProtCommonInfo\" type=\"ProtCommonInfoT\"/>"
"            <xs:element name=\"DisplayPath\" type=\"xs:string\"/>"
"            <xs:element name=\"ProtVer\" type=\"SmbProtVerT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsProtVerT\">"
"        <xs:sequence>"
"            <xs:element name=\"MajorNum\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"MinorNum\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsCommonInfoT\">"
"        <xs:sequence>"
"            <xs:element name=\"ProtCommonInfo\" type=\"ProtCommonInfoT\"/>"
"            <xs:element name=\"ProtVer\" type=\"NfsProtVerT\"/>"
"            <xs:element name=\"UserGids\" type=\"xs:string\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbOpenT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"            <xs:element name=\"OpenAccmode\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"OpenSharemode\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"OpenOptions\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbCloseT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbDeleteT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbCreatT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"            <xs:element name=\"CreatAccmode\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"CreatSharemode\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"CreatDisposition\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"CreatOptions\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"CreatNewObject\" type=\"xs:boolean\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbRenT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"            <xs:element name=\"TargetAccessPath\" type=\"AccessPathT\" nillable=\"true\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SetAttrTimeT\">"
"        <xs:sequence>"
"            <xs:element name=\"Secs\" type=\"xs:long\"/>"
"            <xs:element name=\"NanoSecs\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbSetAttrT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"            <xs:element name=\"SetAttrChangeAttr\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"SetAttrNewOwner\" minOccurs=\"0\" type=\"UsrIdU\"/>"
"            <xs:element name=\"SetAttrNewGroup\" minOccurs=\"0\" type=\"UsrIdU\"/>"
"            <xs:element name=\"SetAttrFileSize\" minOccurs=\"0\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"SetAttrFileAllocSize\" minOccurs=\"0\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"SetAttrMtime\" minOccurs=\"0\" type=\"SetAttrTimeT\"/>"
"            <xs:element name=\"SetAttrAtime\" minOccurs=\"0\" type=\"SetAttrTimeT\"/>"
"            <xs:element name=\"SetAttrCtime\" minOccurs=\"0\" type=\"SetAttrTimeT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbReadT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"            <xs:element name=\"RdOffset\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"RdLength\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbWriteT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"            <xs:element name=\"WrOffset\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"WrLength\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"SmbGetAttrT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"SmbCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsRenT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"            <xs:element name=\"TargetAccessPath\" type=\"AccessPathT\" nillable=\"true\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsSetAttrCommonT\">"
"        <xs:sequence>"
"            <xs:element name=\"SetAttrMode\" minOccurs=\"0\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"SetAttrUid\" minOccurs=\"0\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"SetAttrGid\" minOccurs=\"0\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"SetAttrSize\" minOccurs=\"0\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"SetAttrMtime\" minOccurs=\"0\" type=\"SetAttrTimeT\"/>"
"            <xs:element name=\"SetAttrAtime\" minOccurs=\"0\" type=\"SetAttrTimeT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsSetAttrT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"            <xs:element name=\"ChangeAttr\" type=\"xs:unsignedInt\"/>"
"            <xs:element name=\"SetAttrCommon\" type=\"NfsSetAttrCommonT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsReadT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"            <xs:element name=\"RdOffset\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"RdLength\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsWriteT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"            <xs:element name=\"WrOffset\" type=\"xs:unsignedLong\"/>"
"            <xs:element name=\"WrLength\" type=\"xs:unsignedInt\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsCreatT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsDelT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsGetAttrT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsLinkT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"            <xs:element name=\"TargetAccessPath\" type=\"AccessPathT\" nillable=\"true\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsSymLinkT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsOpenT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsCloseT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"NfsLookupT\">"
"        <xs:sequence>"
"            <xs:element name=\"CommonInfo\" type=\"NfsCommonInfoT\"/>"
"        </xs:sequence>"
"    </xs:complexType>"
"    <xs:complexType name=\"FileNotfInfoT\">"
"        <xs:choice>"
"            <xs:element name=\"SmbCreatReq\" type=\"SmbCreatT\"/>"
"            <xs:element name=\"SmbOpenReq\" type=\"SmbOpenT\"/>"
"            <xs:element name=\"SmbCloseReq\" type=\"SmbCloseT\"/>"
"            <xs:element name=\"SmbRenReq\" type=\"SmbRenT\"/>"
"            <xs:element name=\"SmbSetAttrReq\" type=\"SmbSetAttrT\"/>"
"            <xs:element name=\"SmbRdReq\" type=\"SmbReadT\"/>"
"            <xs:element name=\"SmbWrReq\" type=\"SmbWriteT\"/>"
"            <xs:element name=\"SmbDelReq\" type=\"SmbDeleteT\"/>"
"            <xs:element name=\"SmbGetAttrReq\" type=\"SmbGetAttrT\"/>"
"            <xs:element name=\"NfsRenReq\" type=\"NfsRenT\"/>"
"            <xs:element name=\"NfsSetAttrReq\" type=\"NfsSetAttrT\"/>"
"            <xs:element name=\"NfsGetAttrReq\" type=\"NfsGetAttrT\"/>"
"            <xs:element name=\"NfsRdReq\" type=\"NfsReadT\"/>"
"            <xs:element name=\"NfsWrReq\" type=\"NfsWriteT\"/>"
"            <xs:element name=\"NfsCreatReq\" type=\"NfsCreatT\"/>"
"            <xs:element name=\"NfsDelReq\" type=\"NfsDelT\"/>"
"            <xs:element name=\"NfsLnkReq\" type=\"NfsLinkT\"/>"
"            <xs:element name=\"NfsSymLnkReq\" type=\"NfsSymLinkT\"/>"
"            <xs:element name=\"NfsOpenReq\" type=\"NfsOpenT\"/>"
"            <xs:element name=\"NfsCloseReq\" type=\"NfsCloseT\"/>"
"            <xs:element name=\"NfsLookupReq\" type=\"NfsLookupT\"/>"
"        </xs:choice>"
"    </xs:complexType>"
"    <xs:element name=\"FscreenReq\">"
"        <xs:complexType>"
"            <xs:sequence>"
"                <xs:element name=\"ReqId\" type=\"xs:unsignedLong\" nillable=\"true\"/>"
"                <xs:element name=\"ReqType\" type=\"ReqTypeE\"/>"
"                <xs:element name=\"NotfInfo\" type=\"FileNotfInfoT\"/>"
"            </xs:sequence>"
"        </xs:complexType>"
"    </xs:element>"
"</xs:schema>";

char XSDStatusReq[] =
"<?xml version=\"1.0\"?>"
"<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">"
"    <xs:simpleType name=\"ReqTypeE\">"
"        <xs:restriction base=\"xs:string\">"
"            <xs:enumeration value=\"SMB_OPEN\"/>"
"            <xs:enumeration value=\"SMB_CLOSE\"/>"
"            <xs:enumeration value=\"SMB_CREAT\"/>"
"            <xs:enumeration value=\"SMB_CREAT_DIR\"/>"
"            <xs:enumeration value=\"SMB_DEL\"/>"
"            <xs:enumeration value=\"SMB_DEL_DIR\"/>"
"            <xs:enumeration value=\"SMB_REN\"/>"
"            <xs:enumeration value=\"SMB_REN_DIR\"/>"
"            <xs:enumeration value=\"SMB_GET_ATTR\"/>"
"            <xs:enumeration value=\"SMB_SET_ATTR\"/>"
"            <xs:enumeration value=\"SMB_RD\"/>"
"            <xs:enumeration value=\"SMB_WR\"/>"
"            <xs:enumeration value=\"NFS_LOOKUP\"/>"
"            <xs:enumeration value=\"NFS_CREAT\"/>"
"            <xs:enumeration value=\"NFS_CREAT_DIR\"/>"
"            <xs:enumeration value=\"NFS_DEL\"/>"
"            <xs:enumeration value=\"NFS_DEL_DIR\"/>"
"            <xs:enumeration value=\"NFS_REN\"/>"
"            <xs:enumeration value=\"NFS_REN_DIR\"/>"
"            <xs:enumeration value=\"NFS_OPEN\"/>"
"            <xs:enumeration value=\"NFS_CLOSE\"/>"
"            <xs:enumeration value=\"NFS_RD\"/>"
"            <xs:enumeration value=\"NFS_WR\"/>"
"            <xs:enumeration value=\"NFS_GET_ATTR\"/>"
"            <xs:enumeration value=\"NFS_SET_ATTR\"/>"
"            <xs:enumeration value=\"NFS_LNK\"/>"
"            <xs:enumeration value=\"NFS_SYM_LNK\"/>"
"        </xs:restriction>"
"    </xs:simpleType>"
"    <xs:element name=\"FscreenQueryStatus\">"
"       <xs:complexType>"
"           <xs:sequence>"
"               <xs:element name=\"ReqId\" type=\"xs:unsignedLong\" nillable=\"true\"/>"
"               <xs:element name=\"ReqType\" type=\"ReqTypeE\"/>"
"           </xs:sequence>"
"       </xs:complexType>"
"    </xs:element>"
"</xs:schema>";

/* Alert text should be one of the enums. Currently making it  as
string as FSM make a custom string from the error it encountered and
send it to fpolicy server */
char XSDAlerts[] =
"<?xml version=\"1.0\"?>"
"<xs:schema xmlns:xs=\"http://www.w3.org/2001/XMLSchema\">"
"   <xs:simpleType name=\"SeverityE\">"
"       <xs:restriction base=\"xs:string\">"
"           <xs:enumeration value=\"FATAL\"/>"
"           <xs:enumeration value=\"WARNING\"/>"
"       </xs:restriction>"
"   </xs:simpleType>"
"   <xs:element name=\"Alert\">"
"        <xs:complexType>"
"            <xs:sequence>"
"                <xs:element name=\"SessionId\" type=\"xs:string\" nillable=\"true\"/>"
"                <xs:element name=\"Severity\" type=\"SeverityE\"/>"
"                <xs:element name=\"AlertMsg\" type=\"xs:string\"/>"
"            </xs:sequence>"
"        </xs:complexType>"
"   </xs:element>"
"</xs:schema>";

static request_type getNotifyTypeEnumFromStr( const char *tagValue );
static fsm_request_type_e getReqTypeEnumFromStr( const char *tagValue );
static data_format_e getReqFormatEnumFromStr( const char *tagValue );
static alertSeverity_e getSevEnumFromString (const char *str );
static std::string  fileReqEnumToString(request_type enumVal);

Result_t parseXmlReqSchemas()
{
    Result_t Result = RESULT_SUCCESS;

    xmlSchemaParserCtxtPtr ctxSchema = NULL;

    //xmlLineNumbersDefault(1);

    /* Parse the HeaderResp Schema */
    ctxSchema = xmlSchemaNewMemParserCtxt(XSDHeaderReq,strlen(XSDHeaderReq));
    xmlSchemaSetParserErrors(ctxSchema, (xmlSchemaValidityErrorFunc) fprintf,
                            (xmlSchemaValidityWarningFunc) fprintf, stderr);
    hdrReqSchema = xmlSchemaParse(ctxSchema);
    xmlSchemaFreeParserCtxt(ctxSchema);
    ctxSchema = NULL;

    /* Parse the HandShakeResp Schema */
    ctxSchema = xmlSchemaNewMemParserCtxt(XSDHandShakeReq,strlen(XSDHandShakeReq));
    xmlSchemaSetParserErrors(ctxSchema, (xmlSchemaValidityErrorFunc) fprintf,
                            (xmlSchemaValidityWarningFunc) fprintf, stderr);
    negoReqSchema = xmlSchemaParse(ctxSchema);
    xmlSchemaFreeParserCtxt(ctxSchema);
    ctxSchema = NULL;

    /* Parse the ScreenReq Schema */
    ctxSchema = xmlSchemaNewMemParserCtxt(XSDScreenReq,strlen(XSDScreenReq));
    xmlSchemaSetParserErrors(ctxSchema, (xmlSchemaValidityErrorFunc) fprintf,
                            (xmlSchemaValidityWarningFunc) fprintf, stderr);
    screenReqSchema = xmlSchemaParse(ctxSchema);
    xmlSchemaFreeParserCtxt(ctxSchema);
    ctxSchema = NULL;

    /* Parse the StatusResp Schema */
    ctxSchema = xmlSchemaNewMemParserCtxt(XSDStatusReq,strlen(XSDStatusReq));
    xmlSchemaSetParserErrors(ctxSchema, (xmlSchemaValidityErrorFunc) fprintf,
                            (xmlSchemaValidityWarningFunc) fprintf, stderr);
    statusReqSchema = xmlSchemaParse(ctxSchema);
    xmlSchemaFreeParserCtxt(ctxSchema);
    ctxSchema = NULL;

    /* Parse thes Alert Schema */
    ctxSchema = xmlSchemaNewMemParserCtxt(XSDAlerts,strlen(XSDAlerts));
    xmlSchemaSetParserErrors(ctxSchema, (xmlSchemaValidityErrorFunc) fprintf,
                            (xmlSchemaValidityWarningFunc) fprintf, stderr);
    alertsSchema = xmlSchemaParse(ctxSchema);
    xmlSchemaFreeParserCtxt(ctxSchema);
    ctxSchema = NULL;

Catch_Errors:
    return Result;
}

/* Should be freed at shutdown */
void freeXmlReqSchemas()
{
    xmlSchemaCleanupTypes();

    // free the resource
    if(hdrReqSchema != NULL)
        xmlSchemaFree(hdrReqSchema);

    if(negoReqSchema != NULL)
        xmlSchemaFree(negoReqSchema);

    if(screenReqSchema != NULL)
        xmlSchemaFree(screenReqSchema);

    if(statusReqSchema != NULL)
        xmlSchemaFree(statusReqSchema);

    if(alertsSchema != NULL)
        xmlSchemaFree(alertsSchema);

}


Result_t validateXmlHdrReqSchema( const char *respBuf )
{
    Result_t Result = RESULT_SUCCESS;
    xmlDocPtr doc = NULL;

    //xmlLineNumbersDefault(1);

    doc = xmlRecoverMemory(respBuf, strlen(respBuf));

    if ( NULL == doc )
    {
        fprintf (stderr, "Could not parse the resp buffer. [0x%p] %s:%d\n",
                       pthread_self(),__FILE__,__LINE__);

        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }
    else
    {
        xmlSchemaValidCtxtPtr ctxt = NULL;
        int ret;

        ctxt = xmlSchemaNewValidCtxt(hdrReqSchema);
        xmlSchemaSetValidErrors(ctxt, (xmlSchemaValidityErrorFunc) fprintf,
                             (xmlSchemaValidityWarningFunc) fprintf, stderr);

        ret = xmlSchemaValidateDoc(ctxt, doc);
        if (ret == 0)
        {
            fprintf (stderr, "XMLResp Header Validates. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
        }
        else if (ret > 0)
        {
            fprintf (stderr, "XMLResp Header Validation Fails. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        }
        else
        {
            fprintf (stderr, "XMLResp Hdr Validation Fails.Internal Err. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        }
        xmlSchemaFreeValidCtxt(ctxt);
    }

Catch_Errors:
    xmlFreeDoc(doc);
    return Result;
}

Result_t validateXmlReqSchema( const char *respBuf,
                               fsm_request_type_e type )
{
    Result_t Result = RESULT_SUCCESS;
    xmlDocPtr doc = NULL;

    //xmlLineNumbersDefault(1);

    doc = xmlRecoverMemory(respBuf, strlen(respBuf));

    if ( NULL == doc )
    {
        fprintf (stderr, "Could not parse the resp buffer. [0x%p] %s:%d\n",
                       pthread_self(),__FILE__,__LINE__);

        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }
    else
    {
        xmlSchemaValidCtxtPtr ctxt = NULL;
        int ret;

        switch(type) {
            case NEGO_MSG_REQ:
                ctxt = xmlSchemaNewValidCtxt(negoReqSchema);
                break;

            case FILE_NOTIFY_REQ:
                ctxt = xmlSchemaNewValidCtxt(screenReqSchema);
                break;

            case STATUS_QUERY_REQ:
                ctxt = xmlSchemaNewValidCtxt(statusReqSchema);
                break;

            case ALERTS:
                ctxt = xmlSchemaNewValidCtxt(alertsSchema);
                break;

            default:
                break;
        }

        xmlSchemaSetValidErrors(ctxt, (xmlSchemaValidityErrorFunc) fprintf,
                             (xmlSchemaValidityWarningFunc) fprintf, stderr);

        ret = xmlSchemaValidateDoc(ctxt, doc);
        if (ret == 0)
        {
            fprintf (stderr, "XMLResp Validates. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
        }
        else if (ret > 0)
        {
            fprintf (stderr, "XMLResp Validation Fails. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        }
        else
        {
            fprintf (stderr, "XMLResp Validation Fails. Internal Error. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        }
        xmlSchemaFreeValidCtxt(ctxt);
    }

Catch_Errors:
    xmlFreeDoc(doc);
    return Result;
}

Result_t FSMXmlRespHdr::getRespHdrFromXML( const char *respBuf )
{
    Result_t Result = RESULT_SUCCESS;
    xmlTextReaderPtr pXmlReader   = NULL;
    int              nodeType = 0;
    int              retVal = 0;

    pXmlReader = xmlReaderForMemory(respBuf, strlen(respBuf) , NULL, NULL, 0);

    while ((retVal = xmlTextReaderRead(pXmlReader)))
    {
        if ( -1 == retVal )
        {
            fprintf ( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
            goto Catch_Errors;
        }

        const char *tagName = NULL;
        const char *tagValue = NULL;

        nodeType = xmlTextReaderNodeType(pXmlReader);
        switch (nodeType)
        {
            /* Response info is contained in element tags */
            case XML_READER_TYPE_ELEMENT:
                tagName = (char *)xmlTextReaderConstName(pXmlReader);

                /* Ignore the element 'Header' as it does not contain any value, It contains
                 * other elements. */
                if ( strcmp(tagName, "Header") )
                {
                    /* Read the next element */
                    retVal = xmlTextReaderRead(pXmlReader);

                    /* Check if we are done with all the tags or some error occured */
                    if ( retVal <= 0 )
                    {
                        if ( -1 == retVal )
                        {
                            fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                        }
                        /* Reached the end of document */
                        goto Catch_Errors;
                    }

                    nodeType = xmlTextReaderNodeType(pXmlReader);
                    if ( nodeType == XML_READER_TYPE_TEXT )
                    {
                        tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                        printf("%s:%s\n", tagName,tagValue);

                        /* Validate the elements */
                        if ( !strcmp(tagName, "NotfType") )
                        {
                            _reqType = getReqTypeEnumFromStr( tagValue );
                        }
                        else if ( !strcmp(tagName, "ContentLen") )
                        {
                            _payloadLen = (uint64_t) strtoull(tagValue,NULL,10);
                        }
                        else if ( !strcmp(tagName, "DataFormat") )
                        {
                            _format = getReqFormatEnumFromStr( tagValue );
                        }
                        else
                        {
                            /* As schema is validated, we should not have any other element */
                            assert(0);
                        }
                    }
                    else
                    {
                       fprintf (stderr,"ERROR: Element %s not followed by value", tagName);
                       Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                       goto Catch_Errors;
                    }
                }

                break;

            default:
                break;
        }
    }

Catch_Errors:
    xmlFreeTextReader(pXmlReader);
    return Result;
}

Result_t FSMXmlHandShakeResp::getHandshakeRespPayloadFromXML( const char *respBuf  )
{
    Result_t Result = RESULT_SUCCESS;
    xmlTextReaderPtr pXmlReader   = NULL;
    int              nodeType = 0;
    int              retVal = 0;

    pXmlReader = xmlReaderForMemory(respBuf, strlen(respBuf) , NULL, NULL, 0);

    while ((retVal = xmlTextReaderRead(pXmlReader)))
    {
        if ( -1 == retVal )
        {
            fprintf ( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
            goto Catch_Errors;
        }

        const char *tagName = NULL;
        const char *tagValue = NULL;

        nodeType = xmlTextReaderNodeType(pXmlReader);
        switch (nodeType)
        {
            /* Response info is contained in element tags */
            case XML_READER_TYPE_ELEMENT:
                tagName = (char *)xmlTextReaderConstName(pXmlReader);


                /* Validate the elements */
                if ( !strcmp(tagName,"VsUUID") )
                {
                        /* Read the next element */
                        retVal = xmlTextReaderRead(pXmlReader);

                        /* Check if we are donewth all the tags or some error occured */
                        if ( retVal <= 0 )
                        {
                            if ( -1 == retVal )
                            {
                                fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                            }
                            /* Reached the end of document */
                            goto Catch_Errors;
                        }

                        nodeType = xmlTextReaderNodeType(pXmlReader);
                        if ( nodeType == XML_READER_TYPE_TEXT )
                        {
                                tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                printf("%s:%s\n", tagName,tagValue);
                                strncpy( _vsUUID, tagValue, FP_MAX_UUID_LEN);
                                fprintf (stderr,"_vsUUID %s\n", _vsUUID);
                        }
                }
                else if ( !strcmp(tagName,"PolicyName") )
                {
                        /* Read the next element */
                        retVal = xmlTextReaderRead(pXmlReader);

                        /* Check if we are done with all the tags or some error occured */
                        if ( retVal <= 0 )
                        {
                            if ( -1 == retVal )
                            {
                                fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                            }
                            /* Reached the end of document */
                            goto Catch_Errors;
                        }

                        nodeType = xmlTextReaderNodeType(pXmlReader);
                        if ( nodeType == XML_READER_TYPE_TEXT )
                        {
                                tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                printf("%s:%s\n", tagName,tagValue);
                                strncpy( _policyName, tagValue, MAX_POLICY_NAME_LEN);
                                fprintf (stderr,"PolicyName %s\n", _policyName);
                        }
                }
                else if ( !strcmp(tagName,"SessionId") )
                {
                        /* Read the next element */
                        retVal = xmlTextReaderRead(pXmlReader);

                        /* Check if we are done with all the tags or some error occured */
                        if ( retVal <= 0 )
                        {
                            if ( -1 == retVal )
                            {
                                fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                            }
                            /* Reached the end of document */
                            goto Catch_Errors;
                        }

                        nodeType = xmlTextReaderNodeType(pXmlReader);
                        if ( nodeType == XML_READER_TYPE_TEXT )
                        {
                                tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                printf("%s:%s\n", tagName,tagValue);
                                strncpy( _sessionId, tagValue, MAX_SESSION_ID_LEN );
                                fprintf (stderr,"SessionId %s\n", _sessionId);
                        }
                }
                else if ( !strcmp(tagName,"ProtVersion") )
                {
                        /* Read the next element */
                        retVal = xmlTextReaderRead(pXmlReader);

                        /* Check if we are done with all the tags or some error occured */
                        if ( retVal <= 0 )
                        {
                            if ( -1 == retVal )
                            {
                                fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                            }
                            /* Reached the end of document */
                            goto Catch_Errors;
                        }

                        nodeType = xmlTextReaderNodeType(pXmlReader);
                        if ( nodeType == XML_READER_TYPE_TEXT )
                        {
                                tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                printf("%s:%s\n", tagName,tagValue);
                                _protocolVersion = tagValue;
                                fprintf (stderr,"ProtVersion %s\n", _protocolVersion.c_str());
                        }
                }
                else
                {
                        /* As schema is validated, we should not have any other element */
                        //assert(0);
                }

                break;

            default:
                break;
        }
    }

Catch_Errors:
    xmlFreeTextReader(pXmlReader);
    return Result;
}

Result_t FSMXmlNotfResp::getNotfyRespPayloadFromXML( const char *respBuf  )
{
    Result_t Result = RESULT_SUCCESS;
    xmlTextReaderPtr pXmlReader   = NULL;
    int              nodeType = 0;
    int              retVal = 0;

    pXmlReader = xmlReaderForMemory(respBuf, strlen(respBuf) , NULL, NULL, 0);

    while ((retVal = xmlTextReaderRead(pXmlReader)))
    {
        if ( -1 == retVal )
        {
            fprintf ( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
            goto Catch_Errors;
        }

        const char *tagName = NULL;
        const char *tagValue = NULL;

        nodeType = xmlTextReaderNodeType(pXmlReader);
        switch (nodeType)
        {
            /* Response info is contained in element tags */
            case XML_READER_TYPE_ELEMENT:
                tagName = (char *)xmlTextReaderConstName(pXmlReader);

                if ( !strcmp(tagName,"ReqId") )
                {
                    /* Read the next element */
                    retVal = xmlTextReaderRead(pXmlReader);

                    /* Check if we are done with all the tags or some error occured */
                    if ( retVal <= 0 )
                    {
                        if ( -1 == retVal )
                        {
                            fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                        }
                        /* Reached the end of document */
                        goto Catch_Errors;
                    }

                    nodeType = xmlTextReaderNodeType(pXmlReader);
                    if ( nodeType == XML_READER_TYPE_TEXT )
                    {
                        tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                        printf("%s:%s\n", tagName,tagValue);

                        _reqId = (uint64_t) strtoull(tagValue,NULL,10);
                        fprintf (stderr,"ReqId %llu\n", (unsigned long long)_reqId );
                    }
                }
                else if ( !strcmp(tagName,"ReqType") )
                {
                    /* Read the next element */
                    retVal = xmlTextReaderRead(pXmlReader);

                    /* Check if we are done with all the tags or some error occured */
                    if ( retVal <= 0 )
                    {
                        if ( -1 == retVal )
                        {
                            fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                        }
                        /* Reached the end of document */
                        goto Catch_Errors;
                    }

                    nodeType = xmlTextReaderNodeType(pXmlReader);
                    if ( nodeType == XML_READER_TYPE_TEXT )
                    {
                        tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                        printf("%s:%s\n", tagName,tagValue);

                        _notifyType = getNotifyTypeEnumFromStr( tagValue );
                        fprintf (stderr,"ReqType %d\n", _notifyType);
                    }
                }
                else if ( !strcmp(tagName,"NotfResp") )
                {
                    //_respCode = (FSMResponseCode) atoi(tagValue);
                    //fprintf (stderr,"NotfResp %d\n", _respCode);
                }
                else
                {
                    /* IGNORE */
                }
                break;

            default:
                break;
        }
    }

Catch_Errors:
    xmlFreeTextReader(pXmlReader);
    //xmlCleanupParser();
    return Result;
}

Result_t FSMXmlStatusResp::getStatusRespPayloadFromXML( const char *respBuf  )
{
    Result_t Result = RESULT_SUCCESS;
    xmlTextReaderPtr pXmlReader   = NULL;
    int              nodeType = 0;
    int              retVal = 0;

    pXmlReader = xmlReaderForMemory(respBuf, strlen(respBuf) , NULL, NULL, 0);

    while ((retVal = xmlTextReaderRead(pXmlReader)))
    {
        if ( -1 == retVal )
        {
            fprintf ( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
            goto Catch_Errors;
        }

        const char *tagName = NULL;
        const char *tagValue = NULL;

        nodeType = xmlTextReaderNodeType(pXmlReader);
        switch (nodeType)
        {
            /* Response info is contained in element tags */
            case XML_READER_TYPE_ELEMENT:
                    tagName = (char *)xmlTextReaderConstName(pXmlReader);

                    if ( !strcmp(tagName,"ReqId") )
                    {
                            /* Read the next element */
                            retVal = xmlTextReaderRead(pXmlReader);

                            /* Check if we are done with all the tags or some error occured */
                            if ( retVal <= 0 )
                            {
                                if ( -1 == retVal )
                                {
                                    fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                    Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                                }
                                /* Reached the end of document */
                                goto Catch_Errors;
                            }

                            nodeType = xmlTextReaderNodeType(pXmlReader);
                            if ( nodeType == XML_READER_TYPE_TEXT )
                            {
                                    tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                    printf("%s:%s\n", tagName,tagValue);
                                    _reqId = (uint64_t) strtoull(tagValue,NULL,10);
                                    fprintf (stderr,"ReqId %llu\n", (unsigned long long)_reqId );
                            }
                    }
                    else if ( !strcmp(tagName,"ReqType") )
                    {
                            /* Read the next element */
                            retVal = xmlTextReaderRead(pXmlReader);

                            /* Check if we are done with all the tags or some error occured */
                            if ( retVal <= 0 )
                            {
                                if ( -1 == retVal )
                                {
                                    fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                    Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                                }
                                /* Reached the end of document */
                                goto Catch_Errors;
                            }

                            nodeType = xmlTextReaderNodeType(pXmlReader);
                            if ( nodeType == XML_READER_TYPE_TEXT )
                            {
                                    tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                    printf("%s:%s\n", tagName,tagValue);
                                    _reqType = getNotifyTypeEnumFromStr( tagValue );
                                    fprintf (stderr,"ReqType %d\n", _reqType);
                            }
                    }
                    else
                    {
                            /* As schema is validated, we should not have any other element */
                            //assert(0);
                    }
                    break;

            default:
                break;
        }
    }

Catch_Errors:
    xmlFreeTextReader(pXmlReader);
    return Result;
}

Result_t FSMXmlAlerts::getAlertPayloadFromXML( const char *respBuf  )
{
    Result_t Result = RESULT_SUCCESS;
    xmlTextReaderPtr pXmlReader   = NULL;
    int              nodeType = 0;
    int              retVal = 0;

    pXmlReader = xmlReaderForMemory(respBuf, strlen(respBuf) , NULL, NULL, 0);

    while ((retVal = xmlTextReaderRead(pXmlReader)))
    {
        if ( -1 == retVal )
        {
            fprintf ( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
            Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
            goto Catch_Errors;
        }

        const char *tagName = NULL;
        const char *tagValue = NULL;

        nodeType = xmlTextReaderNodeType(pXmlReader);
        switch (nodeType)
        {
                /* Response info is contained in element tags */
                case XML_READER_TYPE_ELEMENT:
                        tagName = (char *)xmlTextReaderConstName(pXmlReader);

                        if ( !strcmp(tagName,"SessionId") )
                        {
                                /* Read the next element */
                                retVal = xmlTextReaderRead(pXmlReader);

                                /* Check if we are done with all the tags or some error occured */
                                if ( retVal <= 0 )
                                {
                                    if ( -1 == retVal )
                                    {
                                        fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                                    }
                                    /* Reached the end of document */
                                    goto Catch_Errors;
                                }

                                nodeType = xmlTextReaderNodeType(pXmlReader);
                                if ( nodeType == XML_READER_TYPE_TEXT )
                                {
                                        tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                        printf("%s:%s\n", tagName,tagValue);

                                        strncpy( _sessionId, tagValue, MAX_SESSION_ID_LEN );
                                        fprintf (stderr,"SessionId %s\n", _sessionId);
                                }
                        }
                        if ( !strcmp(tagName,"Severity") )
                        {
                                /* Read the next element */
                                /* Read the next element */
                                retVal = xmlTextReaderRead(pXmlReader);

                                /* Check if we are done with all the tags or some error occured */
                                if ( retVal <= 0 )
                                {
                                    if ( -1 == retVal )
                                    {
                                        fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                                    }
                                    /* Reached the end of document */
                                    goto Catch_Errors;
                                }

                                nodeType = xmlTextReaderNodeType(pXmlReader);
                                if ( nodeType == XML_READER_TYPE_TEXT )
                                {
                                        tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                        printf("%s:%s\n", tagName,tagValue);

                                        _alertSeverity = getSevEnumFromString(tagValue);
                                        fprintf (stderr,"Severity %d\n", _alertSeverity );
                                }
                        }
                        if ( !strcmp(tagName,"AlertMsg") )
                        {
                                /* Read the next element */
                                retVal = xmlTextReaderRead(pXmlReader);

                                /* Check if we are done with all the tags or some error occured */
                                if ( retVal <= 0 )
                                {
                                    if ( -1 == retVal )
                                    {
                                        fprintf( stderr ,"ERROR: xmlTextReaderRead Failed.. \n");
                                        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                                    }
                                    /* Reached the end of document */
                                    goto Catch_Errors;
                                }

                                nodeType = xmlTextReaderNodeType(pXmlReader);
                                if ( nodeType == XML_READER_TYPE_TEXT )
                                {
                                        tagValue = (char *)xmlTextReaderConstValue(pXmlReader);
                                        printf("%s:%s\n", tagName,tagValue);

                                        strncpy( _alertStr, tagValue, MAX_ALERT_STR_LEN);
                                        fprintf (stderr,"AlertMsg %s\n",_alertStr );
                                }
                        }
                        else
                        {
                                /* As schema is validated, we should not have any other element */
                                //assert(0);
                        }
                        break;

            default:
                break;
        }
    }

Catch_Errors:
    xmlFreeTextReader(pXmlReader);
    return Result;
}

static request_type getNotifyTypeEnumFromStr( const char *tagValue )
{
    request_type  notfType = FPOLICY_REQUEST_NULL;

    if ( !strcmp(tagValue,"SMB_OPEN") )
    {
        notfType = SMB_OPEN;
    }
    else if ( !strcmp(tagValue,"SMB_CLOSE") )
    {
        notfType = SMB_CLOSE;
    }
    else if ( !strcmp(tagValue,"SMB_CREAT") )
    {
        notfType = SMB_CREAT;
    }
    else if ( !strcmp(tagValue,"SMB_CREAT_DIR") )
    {
        notfType = SMB_CREAT_DIR;
    }
    else if ( !strcmp(tagValue,"SMB_DEL") )
    {
        notfType = SMB_DEL;
    }
    else if ( !strcmp(tagValue,"SMB_DEL_DIR") )
    {
        notfType = SMB_DEL_DIR;
    }
    else if ( !strcmp(tagValue,"SMB_REN") )
    {
        notfType = SMB_REN;
    }
    else if ( !strcmp(tagValue,"SMB_REN_DIR") )
    {
        notfType = SMB_REN_DIR;
    }
    else if ( !strcmp(tagValue,"SMB_GET_ATTR") )
    {
        notfType = SMB_GET_ATTR;
    }
    else if ( !strcmp(tagValue,"SMB_SET_ATTR") )
    {
        notfType = SMB_SET_ATTR;
    }
    else if ( !strcmp(tagValue,"SMB_RD") )
    {
        notfType = SMB_RD;
    }
    else if ( !strcmp(tagValue,"SMB_WR") )
    {
        notfType = SMB_WR;
    }
    else if ( !strcmp(tagValue,"NFS_LOOKUP") )
    {
        notfType = NFS_LOOKUP;
    }
    else if ( !strcmp(tagValue,"NFS_CREAT") )
    {
        notfType = NFS_CREAT;
    }
    else if ( !strcmp(tagValue,"NFS_CREAT_DIR") )
    {
        notfType = NFS_CREAT_DIR;
    }
    else if ( !strcmp(tagValue,"NFS_DEL") )
    {
        notfType = NFS_DEL;
    }
    else if ( !strcmp(tagValue,"NFS_DEL_DIR") )
    {
        notfType = NFS_DEL_DIR;
    }
    else if ( !strcmp(tagValue,"NFS_REN") )
    {
        notfType = NFS_REN;
    }
    else if ( !strcmp(tagValue,"NFS_REN_DIR") )
    {
        notfType = NFS_REN_DIR;
    }
    else if ( !strcmp(tagValue,"NFS_OPEN") )
    {
        notfType = NFS_OPEN;
    }
    else if ( !strcmp(tagValue,"NFS_CLOSE") )
    {
        notfType = NFS_CLOSE;
    }
    else if ( !strcmp(tagValue,"NFS_RD") )
    {
        notfType = NFS_RD;
    }
    else if ( !strcmp(tagValue,"NFS_WR") )
    {
        notfType = NFS_WR;
    }
    else if ( !strcmp(tagValue,"NFS_GET_ATTR") )
    {
        notfType = NFS_GET_ATTR;
    }
    else if ( !strcmp(tagValue,"NFS_SET_ATTR") )
    {
        notfType = NFS_SET_ATTR;
    }
    else if ( !strcmp(tagValue,"NFS_LNK") )
    {
        notfType = NFS_LNK;
    }
    else if ( !strcmp(tagValue,"NFS_SYM_LNK") )
    {
        notfType = NFS_SYM_LNK;
    }
    else
    {
    }

    return notfType;
}

static fsm_request_type_e getReqTypeEnumFromStr( const char *tagValue )
{
    fsm_request_type_e  reqType = FSM_REQ_TYPE_INVALID;

    if ( !strcmp(tagValue,"NEGO_REQ") )
    {
        reqType = NEGO_MSG_REQ;
    }
    else if ( !strcmp(tagValue,"NEGO_RESP") )
    {
        reqType = NEGO_MSG_RESP;
    }
    else if ( !strcmp(tagValue,"SCREEN_REQ") )
    {
        reqType = FILE_NOTIFY_REQ;
    }
    else if ( !strcmp(tagValue,"SCREEN_RESP") )
    {
        reqType = FILE_NOTIFY_RESP;
    }
    else if ( !strcmp(tagValue,"VOL_NOTFICATION") )
    {
        reqType = VOL_NOTIFY;
    }
    else if ( !strcmp(tagValue,"SCREEN_CANCEL") )
    {
        reqType = FILE_NOTIFY_CANCEL_REQ;
    }
    else if ( !strcmp(tagValue,"STATUS_QUERY_REQ") )
    {
        reqType = STATUS_QUERY_REQ;
    }
    else if ( !strcmp(tagValue,"STATUS_QUERY_RESP") )
    {
        reqType = STATUS_QUERY_RESP;
    }
    else if ( !strcmp(tagValue,"BACK_PRESSURE_APPLY") )
    {
        reqType = BACKBACK_RESSURE_APPLY_REQ;
    }
    else if ( !strcmp(tagValue,"BACK_PRESSURE_REM") )
    {
        reqType = BACK_PRESSURE_REMOVE_REQ;
    }
    if ( !strcmp(tagValue,"KEEP_ALIVE") )
    {
        reqType = KEEP_ALIVE_REQ;
    }
    else if ( !strcmp(tagValue,"ALERT_MSG") )
    {
        reqType = ALERTS;
    }
    else
    {
       //assert(0);
    }

    return reqType;
}


static data_format_e getReqFormatEnumFromStr( const char *tagValue )
{
    data_format_e  reqFormat = FORMAT_UNKNOWN;

    if ( !strcmp(tagValue,"XML") )
    {
        reqFormat = FORMAT_XML;
    }
    else
    {
        /* Currently supporting only XML */
    }

    return reqFormat;
}

Result_t FSMXmlRespHdr::respValidateHeader( void )
{
    Result_t Result = RESULT_SUCCESS;

    /* TBD: check for MAX_PAYLOAD_LENGTH also, as we do not expect
     * very large value for a response */
    if ( ( FSM_REQ_TYPE_INVALID == _reqType ) &&
         ( FORMAT_UNKNOWN == _format ) )
    {
        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }

Catch_Errors:
    return Result;
}

Result_t FSMXmlHandShakeResp::respValidateHandshakePayload( void )
{
    Result_t Result = RESULT_SUCCESS;

    //Chk( FSMGlobals::FSMInst->chkVserver( _vserverId ) );

    /* TBD: Check policy name if it exists */

    /* TBD: Check session id if it exists */

    /* TBD: Check protocol version if it is supported
     * Need to get this from supported protocol versions
     */
    //strncmp(_protocolVersion.c_str(),"1.0",strlen("1.0"));

//Catch_Errors:
    return Result;
}

Result_t FSMXmlNotfResp::respValidateNotfyPayload( void )
{
    Result_t Result = RESULT_SUCCESS;
#if 0
    if ( (_respCode != FPOLICY_ACCESS_ALLOW) &&
         (_respCode != FPOLICY_ACCESS_DENY) )
    {
        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }

Catch_Errors:
#endif
    return Result;
}

Result_t FSMXmlStatusResp::respValidateStatusPayload( void )
{
    Result_t Result = RESULT_SUCCESS;
#if 0
    if ( (_respStatusCode != REQ_STATUS_WORKING) ||
         (_respStatusCode != REQ_STATUS_REJECT) )
    {
        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }

Catch_Errors:
#endif

    return Result;
}


Result_t FSMXmlAlerts::respValidateAlertPayload (void )
{
    Result_t Result = RESULT_SUCCESS;
    /* TBD: */

//Catch_Errors:
    return Result;
}

alertSeverity_e getSevEnumFromString (const char *tagValue )
{
    alertSeverity_e sev = ALERT_SEVERITY_NONE;

    if ( !strcmp(tagValue,"FATAL") )
    {
        sev = ALERT_SEVERITY_FATAL;
    }
    else if ( !strcmp(tagValue,"WARNING") )
    {
        sev = ALERT_SEVERITY_WARNING;
    }
    else
    {
    }

    return sev;
}

FSMXmlRespHdr::FSMXmlRespHdr()
{
    _reqType = FSM_REQ_TYPE_INVALID;
    _payloadLen = 0;
    _format = FORMAT_UNKNOWN;

}

FSMXmlHandShakeResp::FSMXmlHandShakeResp()
{
    memset(_vsUUID,'\0',FP_MAX_UUID_LEN+1);
    memset(_sessionId,'\0',MAX_SESSION_ID_LEN+1);
    memset(_policyName,'\0',MAX_POLICY_NAME_LEN+1);
}

FSMXmlNotfResp::FSMXmlNotfResp()
{
    _reqId = 0;
    _notifyType = FPOLICY_REQUEST_NULL;
    //_respCode = FPOLICY_RESPONSE_CODE_MIN;
}

FSMXmlStatusResp::FSMXmlStatusResp()
{
    _reqId = 0;
    _reqType = FPOLICY_REQUEST_NULL;
    _respStatusCode = REQ_STATUS_INVALID;
}

FSMXmlAlerts::FSMXmlAlerts()
{
    memset(_sessionId,'\0',MAX_SESSION_ID_LEN+1);
    _alertSeverity = ALERT_SEVERITY_NONE;
    memset( _alertStr,  '\0', MAX_ALERT_STR_LEN + 1 );
}

int32_t
FSMConnection::processFSMRequest( unsigned char *respData, int32_t len )
{
    Result_t Result = RESULT_SUCCESS;
    FSMXmlRespHdr respHdr;

    char *respXmlBuf = new char [ len+1 ];

    memcpy(respXmlBuf,respData,len);
    respXmlBuf[len] = '\0';

    char *pSavePtr = NULL;

    /* Get response header and validate the fields */
    char *pBuf = strtok_r(respXmlBuf,"\n", &pSavePtr);
    if ( NULL == pBuf )
    {
        fprintf (stderr, "RESP HEADER is NULL. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);

        Result = RESULT_ERROR_GENERAL_FAILURE;
        goto Catch_Errors;

    }

    fprintf (stderr, "RESP HDR:\n%s\n. [0x%p] %s:%d\n",
               pBuf,pthread_self(),__FILE__,__LINE__);


    if ( 0 != validateXmlHdrReqSchema(pBuf) )
    {
        fprintf (stderr, "RESP HDR:Schema validation Failed\n. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }

    fprintf (stderr, "RESP HDR:\n%s\n. [0x%p] %s:%d\n",
            pBuf,pthread_self(),__FILE__,__LINE__);


    if ( 0 !=  respHdr.getRespHdrFromXML(pBuf) )
    {
        fprintf (stderr, "RESP HDR:getRespHdrFromXML Failed\n. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }

    if ( 0 != respHdr.respValidateHeader() )
    {
        fprintf (stderr, "RESP HDR:respValidateHeader Failed\n. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
        Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
        goto Catch_Errors;
    }


    /* Ignore the keepalive requests */
    if ( KEEP_ALIVE_REQ == respHdr.getReqType() )
    {
        goto Catch_Errors;
    }

    /* Get response payload and validate the fields */
    pBuf = strtok_r(NULL,"\n",&pSavePtr);
    if ( NULL == pBuf )
    {
        fprintf (stderr, "RESP BUFFER is NULL. [0x%p] %s:%d\n",
               pthread_self(),__FILE__,__LINE__);
        Result = RESULT_ERROR_GENERAL_FAILURE;
        goto Catch_Errors;
    }

    switch ( respHdr.getReqType() )
    {
        case FILE_NOTIFY_REQ:
        {
            fprintf (stderr, "\n\nRCVD:: SCREEN REQ from FSM.\n\n"
            "[0x%p] %s:%d \n",
            pthread_self(),__FILE__,__LINE__ );

            FSMXmlNotfResp respScreenXml;
            fprintf (stderr, "NOTIFY PAYLOAD:\n%s\n. [0x%p] %s:%d\n",
                    pBuf,pthread_self(),__FILE__,__LINE__);
            if ( 0 != validateXmlReqSchema(pBuf,FILE_NOTIFY_REQ) )
            {
                fprintf (stderr, "validateXmlReqSchema Failed\n. [0x%p] %s:%d\n",
                        pthread_self(),__FILE__,__LINE__);
                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                goto Catch_Errors;
            }
            ///Form a response to FPolicy Server. As of now we just send
            ///allow or deny.
            if (mode == 0) {
                if ( 0 != respScreenXml.getNotfyRespPayloadFromXML(pBuf) )
                {
                    fprintf (stderr, "getNotfyRespPayloadFromXML Failed\n. [0x%p] %s:%d\n",
                            pthread_self(),__FILE__,__LINE__);
                    Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                    goto Catch_Errors;
                }
                if ( 0 != respScreenXml.respValidateNotfyPayload() )
                {
                    fprintf (stderr, "respValidateNotfyPayload Failed\n. [0x%p] %s:%d\n",
                            pthread_self(),__FILE__,__LINE__);
                    Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                    goto Catch_Errors;
                }
                FSMXmlReq *FpsResponse = populateXmlNotificationResp(respScreenXml.getReqId(),
                                         (unsigned char*)fileReqEnumToString(respScreenXml.getNotifyType()).c_str(),
                                         Response);

                ///Form a message header now
                FSMXmlReq *FpsMsgHeader = populateXmlHeaderMsg(FILE_NOTIFY_RESP,
                                           FpsResponse->getXmlMsgLength(),
                                           FORMAT_XML);
                std::string   nullString;
                nullString += '\0'; ///< Add a null character to nullString.

                std::string separator("\n\n");
                FSMXmlReq reqToSend = (*FpsMsgHeader) + separator + (*FpsResponse) + nullString;

                delete FpsResponse;
                delete FpsMsgHeader;

                sleep( delay );

                int32_t retval = sendMsg(reqToSend);
                if (-1 == retval) {
                    fprintf (stderr, "Error in sendMsg while sending a response."
                            "return with error [0x%p] %s:%d\n", pthread_self(),
                            __FILE__, __LINE__);
                    delete [] respXmlBuf;
                    return(-1);
                }
                fprintf (stderr, "\n\nSENT:: SCREEN to FSM.\n\n"
                                                        "[0x%p] %s:%d \n",
                                                        pthread_self(),__FILE__,__LINE__ );
            }
        }
        break;

        case STATUS_QUERY_REQ:
        {
            fprintf (stderr, "\n\nRCVD:: STATUS_QUERY_REQ from FSM.\n\n"
            "[0x%p] %s:%d \n",
            pthread_self(),__FILE__,__LINE__ );

            FSMXmlStatusResp respStatusXml;
            fprintf (stderr, "STATUS PAYLOAD:\n%s\n. [0x%p] %s:%d\n",
                    pBuf,pthread_self(),__FILE__,__LINE__);

            if ( 0 != validateXmlReqSchema(pBuf,STATUS_QUERY_REQ) )
            {
                fprintf (stderr, "validateXmlReqSchema Failed\n. [0x%p] %s:%d\n",
                        pthread_self(),__FILE__,__LINE__);
                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                goto Catch_Errors;
            }

            if ( 0 != respStatusXml.getStatusRespPayloadFromXML(pBuf) )
            {
                fprintf (stderr, "getStatusRespPayloadFromXML Failed\n. [0x%p] %s:%d\n",
                        pthread_self(),__FILE__,__LINE__);
                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                goto Catch_Errors;
            }

            if ( 0 != respStatusXml.respValidateStatusPayload() )
            {
                fprintf (stderr, "respValidateStatusPayload Failed\n. [0x%p] %s:%d\n",
                        pthread_self(),__FILE__,__LINE__);
                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                goto Catch_Errors;
            }
            if (mode == 0) {
                FSMXmlReq *statusResp = populateStatusResponse( respStatusXml.getReqId(),
                                (unsigned char*)fileReqEnumToString(respStatusXml.getReqType()).c_str(),1);

                ///Form a message header now
                FSMXmlReq *FpsMsgHeader = populateXmlHeaderMsg(STATUS_QUERY_RESP,
                        statusResp->getXmlMsgLength(),
                        FORMAT_XML);
                std::string   nullString;
                nullString += '\0'; ///< Add a null character to nullString.
                std::string separator("\n\n");
                FSMXmlReq reqToSend = (*FpsMsgHeader) + separator + (*statusResp) + nullString;

                delete FpsMsgHeader;
                delete statusResp;

                int32_t retval = sendMsg(reqToSend);
                if (-1 == retval) {
                    fprintf (stderr, "Error in sendMsg while sending a response."
                            "return with error [0x%p] %s:%d\n", pthread_self(),
                            __FILE__, __LINE__);
                            delete [] respXmlBuf;
                    return(-1);
                }
                fprintf (stderr, "\n\nSENT:: STATUS_RESP to FSM.\n\n"
                                                        "[0x%p] %s:%d \n",
                                                        pthread_self(),__FILE__,__LINE__ );
            }
        }
        break;

        case VOL_NOTIFY :
        {
            fprintf (stderr, "\n\nRCVD:: Volume Notification from FSM.\n\n"
            "[0x%p] %s:%d \n",
            pthread_self(),__FILE__,__LINE__ );

            fprintf (stderr, "Volume Notification Payload:\n%s\n. [0x%p] %s:%d\n",
                    pBuf,pthread_self(),__FILE__,__LINE__);
        }
        break;

        case ALERTS:
        {
            fprintf (stderr, "\n\nRCVD:: ALERT from FSM.\n\n"
            "[0x%p] %s:%d \n",
            pthread_self(),__FILE__,__LINE__ );

            FSMXmlAlerts reqAlert;
            fprintf (stderr, "ALERT PAYLOAD:\n%s\n. [0x%p] %s:%d\n",
                    pBuf,pthread_self(),__FILE__,__LINE__);

            if ( 0 != validateXmlReqSchema( pBuf, ALERTS ) )
            {
                fprintf (stderr, "validateXmlReqSchema Failed\n. [0x%p] %s:%d\n",
                        pthread_self(),__FILE__,__LINE__);
                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                goto Catch_Errors;
            }

            if ( 0 != reqAlert.getAlertPayloadFromXML( pBuf ) )
            {
                fprintf (stderr, "getAlertPayloadFromXML Failed\n. [0x%p] %s:%d\n",
                        pthread_self(),__FILE__,__LINE__);
                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                goto Catch_Errors;
            }
            if ( 0 != reqAlert.respValidateAlertPayload() )
            {
                fprintf (stderr, "respValidateAlertPayload Failed\n. [0x%p] %s:%d\n",
                        pthread_self(),__FILE__,__LINE__);
                Result = RESULT_ERROR_FPOLICY_CONN_PARSE_NOTIFICATION;
                goto Catch_Errors;
            }
        }
        break;

        default:
         break;
    }

Catch_Errors:
    delete [] respXmlBuf;

    if ( 0 != Result )
    {
        return -1;
    }
    else
    {
        return 0;
    }

}


static std::string  fileReqEnumToString(request_type enumVal)
{
    std::string fileReqType;

    switch(enumVal) {
        case SMB_OPEN :
        {
            fileReqType = "SMB_OPEN";
            break;
        }
        case SMB_CLOSE :
        {
            fileReqType = "SMB_CLOSE";
            break;
        }
        case SMB_CREAT :
        {
            fileReqType = "SMB_CREAT";
            break;
        }
        case SMB_CREAT_DIR :
        {
            fileReqType = "SMB_CREAT_DIR";
            break;
        }
        case SMB_DEL :
        {
            fileReqType = "SMB_DEL";
            break;
        }
        case SMB_DEL_DIR :
        {
            fileReqType = "SMB_DEL_DIR";
            break;
        }
        case SMB_REN :
        {
            fileReqType = "SMB_REN";
            break;
        }
        case SMB_REN_DIR :
        {
            fileReqType = "SMB_REN_DIR";
            break;
        }
        case SMB_GET_ATTR :
        {
            fileReqType = "SMB_GET_ATTR";
            break;
        }
        case SMB_SET_ATTR :
        {
            fileReqType = "SMB_SET_ATTR";
            break;
        }
        case SMB_RD :
        {
            fileReqType = "SMB_RD";
            break;
        }
        case SMB_WR :
        {
            fileReqType = "SMB_WR";
            break;
        }
        case NFS_LOOKUP :
        {
            fileReqType = "NFS_LOOKUP";
            break;
        }
        case NFS_CREAT :
        {
            fileReqType = "NFS_CREAT";
            break;
        }
        case NFS_CREAT_DIR :
        {
            fileReqType = "NFS_CREAT_DIR";
            break;
    }
        case NFS_DEL :
        {
            fileReqType = "NFS_DEL";
            break;
        }
        case NFS_DEL_DIR :
        {
            fileReqType = "NFS_DEL_DIR";
            break;
        }
        case NFS_REN :
        {
            fileReqType = "NFS_REN";
            break;
        }
        case NFS_REN_DIR :
        {
            fileReqType = "NFS_REN_DIR";
            break;
        }
        case NFS_OPEN :
        {
            fileReqType = "NFS_OPEN";
            break;
        }
        case NFS_CLOSE :
        {
            fileReqType = "NFS_CLOSE";
            break;
        }
        case NFS_RD :
        {
            fileReqType = "NFS_RD";
            break;
        }
        case NFS_WR :
        {
            fileReqType = "NFS_WR";
            break;
        }
        case NFS_GET_ATTR :
        {
            fileReqType = "NFS_GET_ATTR";
            break;
        }
        case NFS_SET_ATTR :
        {
            fileReqType = "NFS_SET_ATTR";
            break;
        }
        case NFS_LNK :
        {
            fileReqType = "NFS_LNK";
            break;
        }
        case NFS_SYM_LNK :
        {
            fileReqType = "NFS_SYM_LNK";
            break;
        }
        /* case VOL_NOTF :
        {
            fileReqType = "VOL_NOTF";
            break;
        } */
        default :
        {
            fileReqType = "UNKNOWN";
            break;
        }
    }

    return fileReqType;
}

