# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CIFS/Filer.pm#1 $

## @summary CIFS Filer routines
## @author dl-ssqa@netapp.com
## @status Shared
## @pod here

=head1 NAME

Cifs::Filer - Filer routines for CIFS.

=head1 DESCRIPTION

This library contains CIFS filer based routines.

=head1 SEE-ALSO

=head1 ROUTINES

=cut

package CIFS::Filer;

use strict;
use warnings;
use Tharn;
$Log->comment("Starting " . __FILE__);
use base qw(Exporter);
our @EXPORT_OK = qw(
  get_netbios_name
  get_vserver_info
  get_cifs_server
  get_preferred_ip
  get_vfiler_info
  create_share
  delete_share
  check_share
  list_shares
  list_share_acls
  create_share_acl
  delete_share_acl
  modify_share_acl
  );
use Options;
use Module::Common qw(validate_mixed);
use Params::Validate qw(
  validate_pos
  validate
  BOOLEAN
  SCALAR
  OBJECT
  ARRAYREF
  HASHREF
  UNDEF);
use Carp qw(croak confess);
use Module::Common qw(validate_mixed);
use Tharn qw(
  say
  disconnect
  script_result
  matchstrings);
use filer qw(
  filer_vfiler_command
  filer_vfiler_status_get
  filer_vfilers_get);
use Utils::DNS qw(get_name_by_ip);
use Utils::TharnHost qw(parse_nate_host_file);
use ExportFS qw(:cifs);
use QtreeCmds;
use Server;
use filer qw(filer_vfiler_context_simulate);
use Data::Dumper;
my $Server;
my $Options = new Options;
$Options->parameters(
      definitions => [
             {name        => 'CIFS_SERVER_ALIASES',
              spec        => "s@",
              default     => undef,
              description => 'Comma separated list of CIFS server names to use',
             },
             {name        => "CIFS_SERVER_ACCESS_MODE",
              description => "Determines the method of CIFS server access",
              spec        => "s",
              default     => "ip",
             },
             {name        => 'FILER_INDEX',
              spec        => "s",
              default     => 0,
              description => 'Specifies the index in the FILER array',
             },
             {name        => 'NODE_INDEX',
              spec        => "s",
              default     => 0,
              description => 'Specifies the index in the FILER array',
             },
             {name        => 'VFILER_INDEX',
              spec        => "s",
              default     => 0,
              description => 'The index of the vfiler to use',
             },
             {name        => 'VSERVER_INDEX',
              spec        => "s",
              default     => 0,
              description => 'The index of the vserver to use',
             },
        ]
        );

=head2 get_netbios_name

Returns the NetBios name of the filer

 use Cifs::Filer qw(get_netbios_name);
 get_netbios_name( conn => $conn )

=over

=item conn

(Required) : Connection object of the filer

=item mode

(Optional) : The method to use when determining the NetBios name (cli, file)

=back

=for html <hr>

=cut

sub get_netbios_name {

    $Log->enter();

    $Options->process();
    my $options = $Options->get_hash();

    my (%opts) = validate_mixed(
        @_,
        [],
        {conn    => {type => OBJECT, optional => 0},
         mode    => {type => SCALAR, optional => 1, default => 'file'},
         'index' => {type => SCALAR, optional => 1, default => '0'},

        }

        );

    my ($line,     $prompt,   $name,  $hostObj, $hostIp,
        $hostType, $hostName, $match, $cmd);

    $hostObj  = $opts{'conn'}->hostp();
    $hostName = $hostObj->hostname();
    $hostType = $hostObj->hosttype();
    $hostIp   = $hostObj->default_ip();

    $Log->comment("Getting the NetBios name for $hostName($hostIp)");

    # Do we have TOP_LOGDIR, if not assume CWD
    my $netBiosNameFile;
    if ($options->{'TOP_LOGDIR'}) {
        if ($opts{'index'}) {
            $netBiosNameFile =
              "$options->{'TOP_LOGDIR'}/netbios_name-$hostName-vf$opts{'index'}.txt";
        } else {
            $netBiosNameFile =
              "$options->{'TOP_LOGDIR'}/netbios_name-$hostName.txt";
        }
    } else {
        $netBiosNameFile = "netbios_name-$hostName.txt";
    }

    # If the NetBios name file exists get the NetBios name from it instead of using
    # cifs sessions which is very expensive in stress situations otherwise grok the
    # NetBios name from the output of cifs sessions.
    if (-e $netBiosNameFile && $opts{'mode'} eq 'file') {
        open(NB, "$netBiosNameFile")
          || die "Error cannot open $netBiosNameFile";
        while (<NB>) {
            chomp;
            if ($_ !~ /^$/ && $_ =~ /\S+/) { $name = $_; }
        }
        close NB;
        if ($name) {
            $Log->comment("Found NetBios name $name in $netBiosNameFile");
        } else {
            $Log->comment("NetBios name not found in $netBiosNameFile");
            $opts{'mode'} = 'cli';
        }
    } else {
        $opts{'mode'} = 'cli';
    }

    if ($opts{'mode'} eq 'cli') {
        $Log->comment("Getting the NetBios name from the CLI");
        my $cmd = 'cifs sessions';
        $cmd =
          filer_vfiler_command($opts{'conn'}, $cmd)
          ;    # Convert command if simulating a vfiler context
        (undef, undef, undef, $prompt) =
          say($opts{'conn'}, $cmd,, '-noprompt', '-prompt',
              "Server Registers as '\\S+'",
              -timeout, 300);
        unless ($opts{'conn'}->is_persay()) {

            # send ctrl-C/ctrl-X and read up to the subsequent command
            # prompt.  It's possible that the command completed before we
            # sent an interrupt character, and so the prompt will appear
            # twice, so gently try reading a second prompt.
            $opts{'conn'}->send_intr();
            say($opts{'conn'}, "");
            say($opts{'conn'}, "", '-timeout', 10, '-ok');
        } ## end unless ($opts{'conn'}->is_persay...
        ($name) = ($prompt =~ /Server Registers as '(\S+)'/);

        if ($name) {

            # Save the NetBios name in a file so other scripts don't need to
            # go through the pain of getting it.
            $Log->comment("Saving NetBios name $name in $netBiosNameFile");
            open(NB, ">$netBiosNameFile")
              || die "Error cannot open $netBiosNameFile";
            print NB "$name\n";
            close NB;
            sleep 5;
        } else {
            die "Unable to get the netBIOS name";
        }
    } ## end if ($opts{'mode'} eq 'cli')

    $Log->exit();

    return $name;

} ## end sub get_netbios_name

=head2 get_cifs_server

Determines how a CIFS server is accessed

 use Cifs::Filer qw(get_cifs_server);
 get_cifs_server(host => $host, mode => $mode);

=over

=item conn

(Required) The connection object to the 1st filer

=item index

(Required) The vserver or vfiler index (Deprecated)

=item mode

(Required) Determines the form of server name or ip returned

=item mode

(Required) The mode of CIFS access 


 CIFS_SERVER_ACCESS_MODE   NET_PREFER   index Returns
 fqdn                      N/A          N/A   hostname.domainname
 hostname-vx.domain        family=inet4       hostname-v4.domainname
 hostname-vx.domain        family=inet6       hostname-v6.domainname
 hostname-vx               family=inet4       hostname-v4
 hostname-vx               family=inet6       hostname-v6
 hostname-vx.domain        family=inet4 x     vfilername-v4.domainname
 hostname-vx.domain        family=inet6 x     vfilername-v6.domainname
 hostname-vx               family=inet4 x     vfilername-v4
 hostname-vx               family=inet6 x     vfilername-v6
 netbios_name              N/A          N/A   NetBIOS name
 vserver-vx.domain         family=inet4 x     vserver-v4.domainname
 vserver-vx.domain         family=inet6 x     vserver-v6.domainname
 vserver-vx                family=inet4 x     vserver-v4
 vserver-vx                family=inet6 x     vserver-v6
 vserver                   N/A          x     vserver name
 ip                        family=inet4       x.x.x.x
 ip                        family=inet6       xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx:xxxx
 data_interfaces           family=inet4       x.x.x.x
 cifs_server_aliases       family=inet4       name


=back

=cut

sub get_cifs_server {

    $Log->enter();

    $Options->process();
    my $options = $Options->get_hash();

    my (%opts) =
      validate(@_,
               {conn  => {type => OBJECT, optional => 1},
                host  => {type => OBJECT, optional => 1},
                index => {type => SCALAR, optional => 1},
                mode  => {
                         type     => SCALAR,
                         optional => 0,
                         default  => $options->{'CIFS_SERVER_ACCESS_MODE'}
                  }
               }
               );

    $Log->comment("CIFS_SERVER_ACCESS_MODE = $opts{'mode'}");

    if ($opts{'conn'}) {
        $opts{'host'} = $opts{'conn'}->hostp();
    }

    #----------------------------------------------------------------------
    # Get he preferred IP address based on the tharnhost file's net blocks
    #----------------------------------------------------------------------
    my $ip = get_preferred_ip('host' => $opts{'host'});

    #--------------------------------------------------------
    # Do a reverse lookup of $ip and get hostname and domain
    #--------------------------------------------------------
    my $info = get_name_by_ip('ip' => $ip);
    $info->{'PTR'} =~ /([^\.]+)\.(.+)/;
    my $name   = $1;
    my $domain = $2;

    #----------------------------------------------------------------------
    # If we are a 10mode filer then the index pertains tothe vserver index
    # if we are 7g/mode then the index pertains to the vfiler index
    #----------------------------------------------------------------------
    my ($index, $vfilerInfo, $vserverInfo);
    if ($opts{'host'}->hosttype() =~ /ng/) {
        if (!$opts{'index'} && $options->{'VSERVER_INDEX'} !~ /^\d+$/) {
            $index = $options->{'VSERVER_INDEX'};
        } else {
            $index = 0;
        }
        $vserverInfo = get_vserver_info('index' => $index);
        $name = $vserverInfo->{'vserver'};
    } elsif ($options->{'VFILER_INDEX'}) {
        if (!$opts{'index'} && $options->{'VFILER_INDEX'} !~ /^\d+$/) {
            $index = $options->{'VFILER_INDEX'};
        } else {
            $index = 0;
        }
        $vfilerInfo =
          get_vfiler_info('conn' => $opts{'conn'}, 'index' => $index);
        $name = $vfilerInfo->{'Name'};
    } ## end elsif ($options->{'VFILER_INDEX'...

    my $server;
    if ($opts{'mode'} eq 'fqdn') {

        # Set $server to be the resolved fqdn
        $server = $info->{'PTR'};
    } elsif ($opts{'mode'} eq 'hostname-vx.domain') {

        # Set $server to be the resolved hostname + -v4 or -V6 + domain based
        # on the address type of $ip.
        if ($ip =~ /:/) {
            $server = $name . '-v6.' . $domain;
        } else {
            $server = $name . '-v4.' . $domain;
        }
    } elsif ($opts{'mode'} eq 'hostname-vx') {

        # Set $server to be the resolved hostname + -v4 or -V6 based on
        # the address type of $ip.

        if ($ip =~ /:/) {
            $server = $name . '-v6';
        } else {
            $server = $name . '-v4';
        }
    } elsif ($opts{'mode'} eq 'netbios_name') {

        if ($opts{'host'}->hosttype() =~ /ng/) {
            die "nebios_name cannot be used with 10mode filers\n";
        } elsif ($index) {
            filer_vfiler_context_simulate($opts{'conn'}, $name);
            $server = get_netbios_name('conn'  => $opts{'conn'},
                                       'index' => $index);

        } else {

            # Get the NetBIOS name of the filer
            $server = get_netbios_name('conn' => $opts{'conn'});
        }

        if ($opts{'host'}->hosttype() !~ /ng/) {
            filer_vfiler_context_simulate($opts{'conn'}, 'vfiler0');
        }
    } elsif ($opts{'mode'} eq 'vserver-vx.domain') {

        # Set $server to be the resolved hostname + -v4 or -V6 + domain based
        # on the address type of $ip.
        if ($ip =~ /:/) {
            $server = $name . '-v6.' . $domain;
        } else {
            $server = $name . '-v4.' . $domain;
        }
    } elsif ($opts{'mode'} eq 'vserver-vx') {

        # Set $server to be the resolved hostname + -v4 or -V6 based on
        # the address type of $ip.

        if ($ip =~ /:/) {
            $server = $name . '-v6';
        } else {
            $server = $name . '-v4';
        }
    } elsif ($opts{'mode'} eq 'vserver') {

        $server = $name;

    } elsif ($opts{'mode'} eq 'vserver.domain') {

        $server = $name . '.' . $domain;

    } elsif ($opts{'mode'} eq 'ip') {

        if (   $index
            && $opts{'host'}->hosttype() !~ /ng/) {
            my $interface;
            my ($ifgrps, $interfaces, $hostFileParams) =
              parse_nate_host_file('host' => $opts{'host'});

            foreach my $i (keys(%{$interfaces->{'ids'}})) {
                my $id = $interfaces->{'ids'}{$i};
                my $key;

                if ($ip =~ /:/) {
                    $key = 'IPV6';
                } else {
                    $key = 'IP';
                }

                if ($ip eq $interfaces->{$id}{$key}) {
                    $interface = $interfaces->{$id}{'IFC'};
                    last;
                }
            } ## end foreach my $i (keys(%{$interfaces...
            foreach my $vfilerIp (@{$vfilerInfo->{'IP Adresses'}{$interface}}) {
                if ($vfilerIp =~ /:/ && $ip =~ /:/) {
                    $server = $vfilerIp;
                    last;
                } elsif ($vfilerIp !~ /:/ && $ip !~ /:/) {
                    $server = $vfilerIp;
                    last;
                }
            }
        } else {

            # Just use the ip address for the server
            $server = $ip;
        }
    } elsif ($opts{'mode'} eq 'data_interfaces') {

        $server = ${$options->{'DATA_INTERFACES'}}[0];

    } elsif ($opts{'mode'} eq 'cifs_server_aliases') {

        $server = ${$options->{'CIFS_SERVER_ALIASES'}}[0];

    }

    $Log->comment("Using $server for CIFS access");
    $Log->exit();

    return $server;

} ## end sub get_cifs_server

=head2 create_share

Creates a share at the specified path and waits until for it to be created
 
 use Cifs::Filer qw(create_share);
 create_share( conn => $conn, share => foo, volume => bar)
 create_share( vserver => vs0, share => foo, volume => bar)

=over

=item conn

(Required) : Connection object of the filer (7G/mode) 

=item client

(Required) : Connection object of a client that will be used to creat the share directory

=item delay

(Required) : Delay between checks

=item path

(Optional) : The path the the share directory/qtree

=item qtree

(Optional) : Creates the qtree to be used at the specified share path.

=item server

(Required) : The server to use when creating the share directory

=item share

(Optional) : The share name 

=item volume

(Optional) : The volume the share lives on

=item vserver

(Optional) : The vserver to use (10mode)

=back

=for html <hr>

=cut

sub create_share {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn    => {type => OBJECT | UNDEF, optional => 1},
                client  => {type => OBJECT | UNDEF, optional => 1},
                server  => {type => SCALAR,         optional => 1},
                volume  => {type => SCALAR,         optional => 1},
                path    => {type => SCALAR,         optional => 0},
                share   => {type => SCALAR,         optional => 0},
                qtree   => {type => BOOLEAN,        optional => 0},
                vserver => {type => SCALAR | UNDEF, optional => 1},
                delay => {type => SCALAR, optional => 1, default => 1},
                reuse => {type => SCALAR, optional => 1, default => 1}
               }
               );

    my $createShare = 0;
    if ($opts{'conn'}) {

        #--------------------------------------------------------------
        # Check if the share already exists and delete it if reuse = 0
        #--------------------------------------------------------------
        if ($opts{'reuse'}) {
            if (!check_share('conn'  => $opts{'conn'},
                             'share' => $opts{'share'})
              ) {
                $Log->comment("Creating share $opts{'share'}");
                $createShare = 1;
            } else {
                $Log->comment("Reusing existing share $opts{'share'}");
            }
        } else {
            delete_share('conn' => $opts{'conn'}, 'share' => $opts{'share'});
            $createShare = 1;

            #--------------------------------
            # Wait for the filer to catch up
            #--------------------------------
            $Log->comment("Delaying $opts{'delay'} seconds for share deletion");
            snooze $opts{'delay'};
        } ## end else [ if ($opts{'reuse'})

        if ($createShare) {

            #-------------------------------------------
            # Create a qtree for the share if specified
            #-------------------------------------------
            if ($opts{'qtree'} && $opts{'path'} ne "/vol/$opts{'share'}") {
                if (!qtreeStatus($opts{'conn'}, $opts{'path'})) {
                    $Log->comment("Creating qtree $opts{'path'}");
                    qtreeCreate($opts{'conn'}, $opts{'path'});
                }
            } elsif ($opts{'path'} ne "/vol/$opts{'share'}") {

                #-------------------------------------------
                # Since qtree=0, create the share directory
                #-------------------------------------------
                my $path = $opts{'path'};
                $path =~ s/\//\\/g;
                $Log->comment(
                         "Searching for the share directory \\\\$opts{'server'}"
                           . $path);
                my $cmd = "cmd.exe /c dir \\\\$opts{'server'}" . "$path\\..";
                my $output;
                eval {
                    (undef, undef, $output) =
                      say($opts{'client'}, $cmd, -timeout, 300, -noverbose);
                };

                if ($@) {
                    $Log->comment("$cmd\n $output\n $@");
                    die "Error listing the share directory";
                } else {
                    $Log->debug("$cmd\n  $output");
                }

                if ($output !~ /<DIR>\s+$opts{'share'}($|\s+$)/) {
                    $Log->comment(
                             "Creating  the share directory \\\\$opts{'server'}"
                               . $path);
                    $cmd =
                        "cmd.exe /c if not exist \\\\$opts{'server'}" 
                      . $path
                      . " mkdir \\\\$opts{'server'}"
                      . $path;

                    eval {
                        (undef, undef, $output) =
                          say($opts{'client'}, $cmd, -timeout, 60, -noverbose);
                    };
                    if ($@ || $output) {
                        $Log->comment("$cmd\n $output");
                        if ($@) { $Log->comment("$@"); }
                        die "Error creating the share directory";
                    } else {
                        $Log->debug("$cmd\n  $output");
                    }
                } else {
                    $Log->comment(
                             "\\\\$opts{'server'}" . $path . " already exists");
                }

            } ## end elsif ($opts{'path'} ne "/vol/$opts{'share'}")

            #------------------
            # Create the share
            #------------------
            $Log->comment("Creating share $opts{'share'} at $opts{'path'}");
            share_add($opts{'conn'}, "$opts{'share'}", $opts{'path'},
                      ('force' => 0, 'purge_existing' => 0));

            while (!check_share('conn'  => $opts{'conn'},
                                'share' => $opts{'share'})
              ) {

                #--------------------------------
                # Wait for the filer to catch up
                #--------------------------------
                $Log->comment(
                          "Delaying $opts{'delay'} seconds for share creation");
                snooze $opts{'delay'};
            } ## end while (!check_share('conn'...

        } ## end if ($createShare)
    } else {
        my ($volInfo, $qtree);
        my $Server = Server->new() if (!$Server);

        #---------------------------------------------------------------------
        # Check if the $%^&* volume exists since 10mode will let you create a
        # share when the path does not eixst.
        #---------------------------------------------------------------------
        my $output;
        eval {
            $Server->volume_show('-Noverbose' => 1,
                                 'output'     => \$output,
                                 volume       => $opts{'volume'}
                                 );
        };
        if ($@) {
            die $Log->comment("$output\n$@");
        } else {
            $Log->debug($output);
        }

        #--------------------------------------------------------------
        # Check if the share already exists and delete it if reuse = 0
        #--------------------------------------------------------------
        if ($opts{'reuse'}) {
            if (!check_share('share'   => $opts{'share'},
                             'vserver' => $opts{'vserver'})
              ) {
                $Log->comment("Creating share $opts{'share'}");
                $createShare = 1;
            } else {
                $Log->comment("Reusing existing share $opts{'share'}");
            }
        } else {
            delete_share('vserver' => $opts{'vserver'},
                         'share'   => $opts{'share'});
            $createShare = 1;

            #--------------------------------
            # Wait for the filer to catch up
            #--------------------------------
            $Log->comment("Delaying $opts{'delay'} seconds for share deletion");
            snooze $opts{'delay'};
        } ## end else [ if ($opts{'reuse'})

        if ($createShare) {

            #-------------------------------
            # Check if qtrees are supported
            #-------------------------------
            if ($opts{'qtree'}) {
                my $output;
                $volInfo =
                  $Server->volume_show('-Noverbose' => 1,
                                       'output'     => \$output,
                                       'volume'     => $opts{'volume'}
                                       );
                if ($@) {
                    die $Log->comment("$output\n$@");
                } else {
                    $Log->debug($output);
                }

                if ($volInfo->[0]->{'volume-style'} =~ /striped/i) {
                    $Log->comment(  "$opts{'volume'} is a striped volume, "
                                  . "turning off qtree creation");
                } else {
                    $qtree = 1;
                }
            } ## end if ($opts{'qtree'})

            #-------------------------------------------------------------
            # If we are using striped aggrs/volumes qtrees are currently
            # not supported so check if we need to create qtrees
            #-------------------------------------------------------------
            if ($qtree && $opts{'qtree'} && $opts{'volume'} ne $opts{'share'}) {
                my $output;
                eval {
                    $Server->volume_qtree_show('-Noverbose' => 1,
                                               'output'     => \$output,
                                               vserver      => $opts{'vserver'},
                                               volume       => $opts{'volume'},
                                               qtree        => $opts{'share'}
                                               );
                };
                if ($@) {
                    $Log->debug("$output\n$@");
                    $Log->comment(
                        "Qtree $opts{'share'} not found. Creating qtree $opts{'share'}"
                        );

                    my $output;
                    $Server->volume_qtree_create('-Noverbose' => 1,
                                                 'output'     => \$output,
                                                 vserver => $opts{'vserver'},
                                                 volume  => $opts{'volume'},
                                                 qtree   => $opts{'share'},
                                                 output  => \$output
                                                 );
                    if ($@ || $output =~ /ERROR/) {
                        die $Log->comment("$output\n$@");
                    } else {
                        $Log->debug($output);
                    }

                } ## end if ($@)
            } elsif ($opts{'volume'} ne $opts{'share'}) {

                #---------------------------------------------------------
                # The share specified is not a volume share so create the
                # target directory
                #---------------------------------------------------------
                my $path = $opts{'path'};
                $path =~ s/\//\\/g;
                $Log->comment(
                         "Searching for the share directory \\\\$opts{'server'}"
                           . $path);
                my $cmd = "cmd.exe /c dir \\\\$opts{'server'}" . "$path\\..";
                my $output;
                eval {
                    (undef, undef, $output) =
                      say($opts{'client'}, $cmd, -timeout, 300, -noverbose);
                };

                if ($@) {
                    $Log->comment("$cmd\n  $output");
                    die "Error listing the share directory: $@";
                } else {
                    $Log->debug("$cmd\n  $output");
                }

                if ($output !~ /<DIR>\s+$opts{'share'}($|\s+$)/) {
                    $Log->comment(
                              "Creating the share directory \\\\$opts{'server'}"
                                . $path);
                    $cmd =
                        "cmd.exe /c if not exist \\\\$opts{'server'}" 
                      . $path
                      . " mkdir \\\\$opts{'server'}"
                      . $path;

                    my $output;
                    eval {
                        (undef, undef, $output) =
                          say($opts{'client'}, $cmd, -timeout, 60, -noverbose);
                    };

                    if ($@ || $output) {
                        $Log->comment("$cmd\n  $output");
                        if ($@) { $Log->comment("$@"); }
                        die "Error creating the share directory";
                    } else {
                        $Log->debug("$cmd\n  $output");
                    }
                } else {
                    $Log->comment(
                             "\\\\$opts{'server'}" . $path . " already exists");
                }

            } ## end elsif ($opts{'volume'} ne...

            #------------------
            # Create the share
            #------------------
            $Log->comment("Creating share $opts{'share'} at $opts{'path'}");
            my $output;
            eval {
                $Server->vserver_cifs_share_create(
                                                 '-Verification' => 0,
                                                 '-Noverbose'    => 1,
                                                 'output'        => \$output,
                                                 vserver => $opts{'vserver'},
                                                 'share-name' => $opts{'share'},
                                                 path         => $opts{'path'},
                                                 );
            };
            if ($@) {
                die $Log->comment("$output\n$@");
            } else {
                $Log->debug($output);
            }

            $Log->comment("Delaying $opts{'delay'} seconds for share creation");
            snooze $opts{'delay'};
            while (!check_share('share'   => $opts{'share'},
                                'vserver' => $opts{'vserver'})
              ) {

                #--------------------------------
                # Wait for the filer to catch up
                #--------------------------------
                $Log->comment(
                          "Delaying $opts{'delay'} seconds for share creation");
                snooze $opts{'delay'};
            } ## end while (!check_share('share'...
        } ## end if ($createShare)

    } ## end else [ if ($opts{'conn'})
    $Log->exit();
} ## end sub create_share

=head2 delete_share

Deletes the specified share and waits until its deleted

 use Cifs::Filer qw(delete_share);
 delete_share( conn => $conn, share => foo, volume => bar)
 delete_share( vserver => vs0, share => foo, volume => bar)

=over

=item conn

(Required) : Connection object of the filer (7G/mode) 

=item delay

(Required) : Delay between checks

=item share

(Optional) : The share name 

=item volume

(Optional) : The volume the share lives on

=item vserver

(Optional) : The vserver to use (10mode)

=back

=for html <hr>

=cut

sub delete_share {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn    => {type => OBJECT | UNDEF, optional => 1},
                share   => {type => SCALAR,         optional => 0},
                volume  => {type => SCALAR,         optional => 1},
                vserver => {type => SCALAR | UNDEF, optional => 1},
                delay => {type => SCALAR, optional => 1, default => 1},
               }
               );

    if ($opts{'conn'}) {
        if (check_share('conn' => $opts{'conn'}, 'share' => $opts{'share'})) {
            $Log->comment("Deleting share $opts{'share'}");
            share_delete($opts{'conn'}, $opts{'share'},
                         {'force' => 1, 'verify' => 0});

            while (check_share('conn'  => $opts{'conn'},
                               'share' => $opts{'share'})
              ) {

                #--------------------------------
                # Wait for the filer to catch up
                #--------------------------------
                $Log->comment(
                          "Delaying $opts{'delay'} seconds for share deletion");
                snooze $opts{'delay'};
            } ## end while (check_share('conn'...
        } ## end if (check_share('conn'...

    } else {
        my $Server = Server->new() if (!$Server);

        #-------------------------------------------------------------
        # For seom reason commands that use wildcards don't work with
        # -Noverbose
        #-------------------------------------------------------------
        if ($opts{'share'} =~ /\*/) {
            $Log->comment("Deleting shares $opts{'share'}");
            my $output;
            eval {
                $Server->vserver_cifs_share_delete(
                                                  vserver => $opts{'vserver'},
                                                  'share-name' => $opts{'share'}
                                                  );
            };

            #--------------------------------
            # Wait for the filer to catch up
            #--------------------------------
            $Log->comment("Delaying $opts{'delay'} seconds for share deletion");
            snooze $opts{'delay'};
        } else {

            if (check_share('share'   => $opts{'share'},
                            'vserver' => $opts{'vserver'})
              ) {
                $Log->comment("Deleting share $opts{'share'}");
                my $output;
                eval {
                    $Server->vserver_cifs_share_delete(
                                                  '-Verification' => 0,
                                                  '-Noverbose'    => 1,
                                                  'output'        => \$output,
                                                  vserver => $opts{'vserver'},
                                                  'share-name' => $opts{'share'}
                                                  );
                };

                if ($@) {
                    die $Log->comment("$output\n$@");
                } else {
                    $Log->debug($output);
                }

                while (check_share('share'   => $opts{'share'},
                                   'vserver' => $opts{'vserver'})
                  ) {

                    #--------------------------------
                    # Wait for the filer to catch up
                    #--------------------------------
                    $Log->comment(
                          "Delaying $opts{'delay'} seconds for share deletion");
                    snooze $opts{'delay'};

                } ## end while (check_share('share'...
            } ## end if (check_share('share'...
        } ## end else [ if ($opts{'share'} =~ ...
    } ## end else [ if ($opts{'conn'})

    $Log->exit();
} ## end sub delete_share

=head2 check_share

Checks if the specified share exists

 use Cifs::Filer qw(check_share);
 check_share( conn => $conn, share => foo)
 check_share( vserver => vs0, share => foo )

=over

=item conn

(Required) : Connection object of the filer (For 7G/mode) 

=item share

(Optional) : The share name 

=item vserver

(Optional) : The vserver to use (cmode only)

=back

=for html <hr>

=cut

sub check_share {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn    => {type => OBJECT | UNDEF, optional => 1},
                share   => {type => SCALAR,         optional => 0},
                vserver => {type => SCALAR | UNDEF, optional => 1},
               }
               );

    $Log->comment("Checking share $opts{'share'}");

    my $shareExists = 0;
    if ($opts{'conn'}) {
        $shareExists = share_exists($opts{'conn'}, $opts{'share'});
    } else {
        my $Server = Server->new() if (!$Server);
        my $output;
        my $shareInfo;
        eval {
            $shareInfo =
              $Server->vserver_cifs_share_show('-Noverbose' => 1,
                                               'output'     => \$output,
                                               vserver      => $opts{'vserver'},
                                               'share-name' => $opts{'share'}
                                               );
        };

        if ($shareInfo) { $shareExists = 1; }
    } ## end else [ if ($opts{'conn'})

    $Log->debug("Returning $shareExists");
    $Log->exit();
    return $shareExists;

} ## end sub check_share

=head2 get_vserver_info

Get the info of the specified vserver

 use Cifs::Filer qw(get_vserver_info);
 get_vserver_info( conn => $conn )

=over

=item index

(Optional) : Determines which vserver to use based on its placement in the 
in the output of C<vserver show -type cluster>.

=back

=for html <hr>

=cut

sub get_vserver_info {

    $Log->enter();

    my %opts = validate(@_,
                        {'index' =>
                           {type => SCALAR, optional => 0, default => 0}
                        }
                        );

    my $Server = Server->new() if (!$Server);
    my $output;
    my $vserverInfo =
      $Server->vserver_show('-Noverbose' => 1,
                            'output'     => \$output,
                            type         => 'cluster'
                            );
    if ($@) {
        die $Log->comment("$output\n$@");
    } else {
        $Log->debug($output);
    }

    my $index = 0;
    my $vserver;
    if ($opts{'index'} > $#{$vserverInfo}) {
        die "Vserver index exceeds the number of existing vservers: "
          . @{$vserverInfo} . "\n";
    }

    foreach my $vs (@{$vserverInfo}) {
        if ($opts{'index'} == $index) {
            $vserver = $vs;
            last;
        }
        $index++;
    }

    $Log->exit();

    return $vserver;

} ## end sub get_vserver_info

=head2 get_vfiler_info

Returns info of the specified vfiler

 use Cifs::Filer qw(get_vfiler_info);
 get_vfiler_info( conn => $conn )

=over

=item conn

(Required) : Connection object of the filer

=item mode

(Optional) : The vfiler index

=back

=for html <hr>

=cut

sub get_vfiler_info {

    $Log->enter();

    my %opts = validate(@_,
                        {conn => {type => OBJECT, optional => 0},
                         'index' =>
                           {type => SCALAR, optional => 0, default => 1}
                        }
                        );
    my %info;
    for (my $i = 1 ; $i <= 100 ; $i++) {
        my $vfiler_get_status = 0;
        my $get_vfilers       = 0;

        my @vfilers = eval { filer_vfilers_get($opts{'conn'}); };
        if ($@ || @vfilers == 0) {
            $Log->comment(" Error getting active vfilers");
            next;
        } else {
            $get_vfilers = 1;
        }
        my $vfiler = $vfilers[$opts{'index'}];
        %info = eval { filer_vfiler_status_get($opts{'conn'}, $vfiler); };
        if ($@ || !keys %info) {
            $Log->comment(" Error getting status of vfilers");
            next;
        } else {
            $info{'Name'} = $vfiler;
            $vfiler_get_status = 1;

        }

        if ($get_vfilers && $vfiler_get_status) {
            last;
        }
    } ## end for (my $i = 1 ; $i <= ...
    $Log->exit();

    return \%info;

} ## end sub get_vfiler_info

=head2 list_shares

Checks if the specified share exists

 use Cifs::Filer qw(list_shares);
 list_shares( conn => $conn, share => foo)
 list_shares( vserver => vs0, share => foo )

=over

=item conn

(Required) : Connection object of the filer (For 7G/mode) 

=item share

(Optional) : The share name 

=item vserver

(Optional) : The vserver to use (cmode only)

=back

=for html <hr>

=cut

sub list_shares {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn    => {type => OBJECT | UNDEF, optional => 1},
                vserver => {type => SCALAR | UNDEF, optional => 1},
                filter  => {type => SCALAR | UNDEF, optional => 1},
               }
               );

    $Log->comment("Listing shares");
    $Log->debug("opts: " . Dumper(\%opts));

    if ($opts{'conn'}) {
        my $output;
        my $cmd = filer_vfiler_command($opts{'conn'}, 'cifs shares');
        eval {
            (undef, undef, $output) = say($opts{'conn'}, $cmd, -timeout, 300);
        };

        if ($@) {
            $Log->comment("$cmd\n $output");
            die "Error listing cifs shares\n";
        } else {
            $Log->debug("Command output: \n$output");
        }

        # Search the commands output
        my (%shares, @shareInfo);
        foreach my $line (split(/\n/, $output)) {

            #Name         Mount Point                       Description
            #----         -----------                       -----------
            #ETC$         /etc                              Remote Administration
            #                        BUILTIN\Administrators / Full Control
            #C$           /                                 Remote Administration
            #			     BUILTIN\Administrators / Full Control

            if ($line =~ /^(\S+)\s+(\/\S+)\s+/) {
                $shares{'share-name'} = $1;
                $shares{'path'}       = $2;
                push(@shareInfo, \%shares);
            }
        } ## end foreach my $line (split(/\n/...
        $Log->exit();
        return \@shareInfo;
    } else {
        my $Server = Server->new() if (!$Server);
        my $output;
        my %args = ('-Noverbose' => 1,
                    'output'     => \$output,
                    vserver      => $opts{'vserver'}
                    );

        if ($opts{'filter'}) { $args{'share-name'} = $opts{'filter'}; }
        my $shareInfo = $Server->vserver_cifs_share_show(%args);
        if ($@) {
            die $Log->comment("$output\n$@");
        } else {
            $Log->debug($output);
        }

        $Log->exit();
        return $shareInfo;
    } ## end else [ if ($opts{'conn'})
    $Log->exit();
} ## end sub list_shares
=head2 get_preferred_ip

 use CIFS::Filer qw(get_preferred_ip);
 get_preferred_ip(conn => $fc);

C<get_preferred_ip> Use NET_PREFER to determine the IP address to use.

=over

=item conn

(Required) The filers connection object

=back

=cut

sub get_preferred_ip {

    $Log->enter();

    $Options->process();
    my $options = $Options->get_hash();
    my (%opts) = validate(@_, {host => {type => OBJECT, optional => 0},});

    my @prefs;

    if ($opts{'host'}->{prefer_family}) {
        push(@prefs,
             {nettype => 'data', family => $opts{'host'}->{prefer_family}},
             {nettype => '7g',   family => $opts{'host'}->{prefer_family}});
    } else {
        push(@prefs, {nettype => 'data',}, {nettype => '7g'});
    }

    my $ip = $opts{'host'}->net_best_address(sort_preference => \@prefs);
    $Log->exit();

    return $ip;
} ## end sub get_preferred_ip

=head2 list_share_acls

Checks if the specified share exists

 use Cifs::Filer qw(list_share_acls);
 list_share_acls( conn => $conn, share => foo)
 list_share_acls( vserver => vs0, share => foo )

=over

=item conn

(Required) : Connection object of the filer (For 7G/mode) 

=item share

(Optional) : The share name 

=item vserver

(Optional) : The vserver to use (cmode only)

=back

=for html <hr>

=cut

sub list_share_acls {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn    => {type => OBJECT | UNDEF, optional => 1},
                vserver => {type => SCALAR | UNDEF, optional => 1},
                share   => {type => SCALAR,         optional => 0},
               }
               );

    $Log->comment("Listing share ACLs for $opts{'share'}");

    if ($opts{'conn'}) {
        my $output;
        my $cmd = filer_vfiler_command($opts{'conn'}, 'cifs shares');
        eval {
            (undef, undef, $output) = say($opts{'conn'}, $cmd, -timeout, 300);
        };

        if ($@) {
            $Log->comment("$cmd\n $output");
            die "Error listing cifs shares\n";
        } else {
            $Log->debug("Command output: \n$output");
        }

        # Search the commands output
        my (%shares, @shareInfo);
        foreach my $line (split(/\n/, $output)) {

            #Name         Mount Point                       Description
            #----         -----------                       -----------
            #ETC$         /etc                              Remote Administration
            #                        BUILTIN\Administrators / Full Control
            #C$           /                                 Remote Administration
            #			     BUILTIN\Administrators / Full Control

            if ($line =~ /^(\S+)\s+(\/\S+)\s+/) {
                $shares{'share-name'} = $1;
                $shares{'path'}       = $2;
                push(@shareInfo, \%shares);
            }
        } ## end foreach my $line (split(/\n/...
        $Log->exit();
        return \@shareInfo;
    } else {

        #rtpqa-fas6030-11::*> vserver cifs share access-control show -share testvol1
        #Virtual        Share       User/Group                  Access
        #Server         Name        Name                        Permission
        #-------------- ----------- --------------------------- -----------
        #A13            testvol1    Everyone                    Full_Control

        my $Server = Server->new() if (!$Server);
        my $output;
        my $shareAclInfo =
          $Server->vserver_cifs_share_access_control_show(
                                                    '-Noverbose' => 1,
                                                    'output'     => \$output,
                                                    vserver => $opts{'vserver'},
                                                    'share' => $opts{'share'}
                                                    );
        if ($@) {
            die $Log->comment("$output\n$@");
        } else {
            $Log->debug($output);
        }

        $Log->exit();
        return $shareAclInfo;
    } ## end else [ if ($opts{'conn'})
    $Log->exit();
} ## end sub list_share_acls

=head2 create_share_acls

Creates a share ACL

 use Cifs::Filer qw(create_share_acl);
 create_share_acl( conn => $conn, share => foo)
 create_share_acl( vserver => vs0, share => foo )

=over

=item conn

(Required) : Connection object of the filer (For 7G/mode) 

=item share

(Optional) : The share name 

=item vserver

(Optional) : The vserver to use (cmode only)

=item user-or-group

(Optional) : The user or group name to use in the ACL

=item permission

(Optional) : The permission to to use in the ACL

=back

=for html <hr>

=cut

sub create_share_acl {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn            => {type => OBJECT | UNDEF, optional => 1},
                vserver         => {type => SCALAR | UNDEF, optional => 1},
                share           => {type => SCALAR,         optional => 0},
                'user-or-group' => {type => SCALAR,         optional => 0},
                permission      => {type => SCALAR,         optional => 0},
               }
               );

    $Log->comment(  "Creating share ACL $opts{'permission'} for "
                  . "group $opts{'user-or-group'} on $opts{'share'}");

    if ($opts{'conn'}) {
        my $output;
        my $cmd = filer_vfiler_command($opts{'conn'},
                                       'cifs access '
                                         . $opts{'share'} . '"'
                                         . $opts{'user-or-group'} . '"' . '"'
                                         . $opts{'permission'} . '"'
                                         );
        eval {
            (undef, undef, $output) = say($opts{'conn'}, $cmd, -timeout, 300);
        };

        if ($@) {
            $Log->comment("$cmd\n $output");
            die "Error creating share ACL\n";
        } else {
            $Log->debug("Command output: \n$output");
        }

    } else {

        my $Server = Server->new() if (!$Server);
        my $output;
        $opts{'permission'}    =~ s/\s+/_/g;
        $opts{'user-or-group'} =~ s/^(.*)$/"$1"/g;
        eval {
            $Server->vserver_cifs_share_access_control_create(
                                      '-Verification' => 0,
                                      '-Noverbose'    => 1,
                                      'output'        => \$output,
                                      'vserver'       => $opts{'vserver'},
                                      'share'         => $opts{'share'},
                                      'user-or-group' => $opts{'user-or-group'},
                                      'permission'    => $opts{'permission'}
                                      );
        };
        if ($@) {
            die $Log->comment("$output\n$@");
        } else {
            $Log->debug($output);
        }

    } ## end else [ if ($opts{'conn'})
    $Log->exit();
} ## end sub create_share_acl

=head2 delete_share_acls

Creates a change ACL

 use Cifs::Filer qw(delete_share_acl);
 delete_share_acl( conn => $conn, share => foo)
 delete_share_acl( vserver => vs0, share => foo )

=over

=item conn

(Required) : Connection object of the filer (For 7G/mode) 

=item share

(Optional) : The share name 

=item vserver

(Optional) : The vserver to use (cmode only)

=item user-or-group

(Optional) : The user or group name to use in the ACL

=back

=for html <hr>

=cut

sub delete_share_acl {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn            => {type => OBJECT | UNDEF, optional => 1},
                vserver         => {type => SCALAR | UNDEF, optional => 1},
                share           => {type => SCALAR,         optional => 0},
                'user-or-group' => {type => SCALAR,         optional => 0},
               }
               );

    $Log->comment(  "Deleting share ACL for user or group "
                  . "$opts{'user-or-group'} on $opts{'share'}");

    if ($opts{'conn'}) {
        my $output;
        my $cmd = filer_vfiler_command($opts{'conn'},
                                       'cifs access '
                                         . $opts{'share'} . '"'
                                         . $opts{'user-or-group'} . '"' . '"'
                                         . $opts{'permission'} . '"'
                                         );
        eval {
            (undef, undef, $output) = say($opts{'conn'}, $cmd, -timeout, 300);
        };

        if ($@) {
            $Log->comment("$cmd\n $output");
            die "Error creating share ACL\n";
        } else {
            $Log->debug("Command output: \n$output");
        }

    } else {

        my $Server = Server->new() if (!$Server);
        my $output;
        $opts{'permission'}    =~ s/\s+/_/g;
        $opts{'user-or-group'} =~ s/^(.*)$/"$1"/g;
        eval {
            $Server->vserver_cifs_share_access_control_delete(
                                       '-Verification' => 0,
                                       '-Noverbose'    => 1,
                                       'output'        => \$output,
                                       'vserver'       => $opts{'vserver'},
                                       'share'         => $opts{'share'},
                                       'user-or-group' => $opts{'user-or-group'}
                                       );
        };
        if ($@) {
            die $Log->comment("$output\n$@");
        } else {
            $Log->debug($output);
        }

    } ## end else [ if ($opts{'conn'})
    $Log->exit();
} ## end sub delete_share_acl

=head2 modify_share_acls

Creates a share ACL

 use Cifs::Filer qw(modify_share_acl);
 modify_share_acl( conn => $conn, share => foo)
 modify_share_acl( vserver => vs0, share => foo )

=over

=item conn

(Required) : Connection object of the filer (For 7G/mode) 

=item share

(Optional) : The share name 

=item vserver

(Optional) : The vserver to use (cmode only)

=item user-or-group

(Optional) : The user or group name to use in the ACL

=item permission

(Optional) : The permission to to use in the ACL

=back

=for html <hr>

=cut

sub modify_share_acl {

    $Log->enter();

    my (%opts) =
      validate(@_,
               {conn            => {type => OBJECT | UNDEF, optional => 1},
                vserver         => {type => SCALAR | UNDEF, optional => 1},
                share           => {type => SCALAR,         optional => 0},
                'user-or-group' => {type => SCALAR,         optional => 0},
                permission      => {type => SCALAR,         optional => 0},
               }
               );

    $Log->comment(  "Modifying share ACL $opts{'permission'} for "
                  . "group $opts{'user-or-group'} on $opts{'share'}");

    if ($opts{'conn'}) {
        my $output;
        my $cmd = filer_vfiler_command($opts{'conn'},
                                       'cifs access '
                                         . $opts{'share'} . '"'
                                         . $opts{'user-or-group'} . '"' . '"'
                                         . $opts{'permission'} . '"'
                                         );
        eval {
            (undef, undef, $output) = say($opts{'conn'}, $cmd, -timeout, 300);
        };

        if ($@) {
            $Log->comment("$cmd\n $output");
            die "Error modifying share ACL\n";
        } else {
            $Log->debug("Command output: \n$output");
        }

    } else {

        my $Server = Server->new() if (!$Server);
        my $output;
        $opts{'permission'}    =~ s/\s+/_/g;
        $opts{'user-or-group'} =~ s/^(.*)$/"$1"/g;
        eval {
            $Server->vserver_cifs_share_access_control_modify(
                                      '-Verification' => 0,
                                      '-Noverbose'    => 1,
                                      'output'        => \$output,
                                      'vserver'       => $opts{'vserver'},
                                      'share'         => $opts{'share'},
                                      'user-or-group' => $opts{'user-or-group'},
                                      'permission'    => $opts{'permission'}
                                      );
        };
        if ($@) {
            die $Log->comment("$output\n$@");
        } else {
            $Log->debug($output);
        }

    } ## end else [ if ($opts{'conn'})
    $Log->exit();
} ## end sub modify_share_acl

=head1 BUGS / ENHANCEMENTS

    Update the routines to use ZAPI.

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = options cifs, options cifs

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    bellizzi@netapp.com
    $Id: //depot/prod/test/Rsierranevada/lib/CIFS/Filer.pm#1 $
    Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

    Copyright 2007, Network Appliance Inc., All Rights Reserved.

=cut

1;
