# -*- perl -*-
# $Id: //depot/prod/test/main/lib/CIFS/MACOS/MacOSMisc.pm#28 $

## @summary CIFS routines specific to MAC OS X client
## @author knshiva@netapp.com
## @status Shared
## @pod here


=head1 NAME

CIFS::MACOS::MacOSMisc - MAC OS setup routines for CIFS.

=head1 SYNOPSIS

    use CIFS::MACOS::MacOSMisc qw (mac_kerb_setup mac_kerb_cleanup);

=head1 DESCRIPTION

    This library contains client setup and cleanup routines for MAC OS client.

=head1 SEE-ALSO

=head1 ROUTINES

=cut

package CIFS::MACOS::MacOSMisc;

use strict;
use warnings;

BEGIN
{
    # doing the export in the BEGIN block to avoid Exporter problems 
    # during run-time due to circular use statements
    use base qw (Exporter);
    our @EXPORT_OK = qw (mac_kerb_setup mac_kerb_cleanup);

    Params::Validate::validation_options(
        on_fail => sub {my @msgs = @_; chomp(@msgs);
                confess(script_result(@msgs))}
        );
}

use Params::Validate v0.70 qw(validate SCALAR OBJECT HASHREF);
use Carp qw (croak confess);
use Tharn qw (logcomment logresult disconnect script_result matchstrings);
use TracePkg;
use SayWrap qw (sayEOF);

=head2 get_version_macos

A routine to setup MAC OS client

=over

=item host

(Required) : The filer DNS name, Hostrec or Connectrec.

=cut


sub mac_kerb_setup {

    my $cc = shift @_;
    my $fc = shift @_;
    
    my $DOMAIN = shift @_;
    my $USER = shift @_;

    my $cli_home_dir = "/Users/$USER/";
    my $mntpt = $cli_home_dir."mnt";

    # client side config
    my $set1 = sayEOF ($cc, "/bin/mkdir -p $mntpt");
    if ((!$set1) || ($set1 =~ /$mntpt/)) {
            # change ownership of user dir created here
            sayEOF ($cc, "/usr/sbin/chown -R \"$DOMAIN\\$USER:$DOMAIN\\Domain Users\" $cli_home_dir");
    }
    else {
            logresult ("FAIL", "setup failed during mount point creation on the client");
            return -1;
    }
    # end of client side config

    # storage system side config

    # add domain user to Filers Administrators group to acces C$ share
    my $set2 = sayEOF ($fc, "useradmin domainuser add $DOMAIN\\$USER -g Administrators");
    if ($set2 =~ /Domain User <$DOMAIN\\$USER> successfully added/) {
            logcomment ("INFO", "added domainuser $DOMAIN\\$USER to local Administrators group"
);
    }
    else {
            logresult ("FAIL", "Failed to add domainuser $DOMAIN\\$USER to local Administrators
group");
            return -1;
    }
    # end of storage system side config
    return 0;
}


sub mac_kerb_cleanup {
    my $cc = shift @_;
    my $fc = shift @_;

    my $DOMAIN = shift @_;
    my $USER = shift @_;

    my $cli_home_dir = "/Users/$USER/";
    my $mntpt = $cli_home_dir."mnt";

    # unmount the C\$ share mounted
    my $cln1 = sayEOF ($cc , "/sbin/umount $mntpt");
    if ((!$cln1) || ($cln1 =~ /not currently mounted/)) {
            logcomment ("INFO", "unmounted C\$ share");
    }
    else {
            logcomment ("WARN", "failed to unmount C\$ share");
    }

    # destory $USER's local home dir
    sayEOF ($cc, "/bin/rm -rf $cli_home_dir");

    my $cln2 = sayEOF ($fc, "useradmin domainuser delete $DOMAIN\\$USER -g Administrators");
    if ($cln2 =~ /Domain User <$DOMAIN\\$USER> successfully deleted/) {
            logcomment ("INFO", "deleted domainuser $DOMAIN\\$USER from local Administrators group");
    }
    else {
            logcomment ("WARN", "could not delete $DOMAIN\\$USER");
    }
}

1;

=head1 AUTHOR

    knshiva@netapp.com
    $Id: //depot/prod/test/main/lib/CIFS/MACOS/MacOSMisc.pm#28 $
    Last revised $Date: 2009/08/21 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

