# $Id$

## @summary Library for miscellaneous CIFS subroutines for NATE on Windows
## @author mueller@netapp.com
## @pod here

=head1 NAME

=head1 SYNOPSIS

    use CIFS::Misc qw(:all);

=head1 ROUTINES

=cut

package CIFS::Misc;
use strict;

BEGIN {
  use Module::Common;
  Module::Common::setup_export_tags(
    ops => [ qw(cifsError
                cifsGetLastError) ],
    errors => [ qw(CIFS_ERR_FILE_NOT_FOUND
	           CIFS_ERR_USER_NOT_FOUND
		   CIFS_ERR_NAME_LOOKUP_FAILED
		   CIFS_ERR_SID_LOOKUP_FAILED
		   CIFS_ERR_INVALID_PARAMETER) ]
  );
}

use Params::Validate v0.70 qw(validate SCALAR SCALARREF HASHREF UNDEF);
use Carp;
use base qw(Exporter);

use constant CIFS_ERR_FILE_NOT_FOUND => "The system cannot find the file specified";
use constant CIFS_ERR_USER_NOT_FOUND => "The account specified could not be resolved";
use constant CIFS_ERR_NAME_LOOKUP_FAILED => "Name lookup failed, cannot resolve to SID";
use constant CIFS_ERR_SID_LOOKUP_FAILED => "SID lookup failed, cannot resolve to account name";
use constant CIFS_ERR_INVALID_PARAMETER => "A parameter is invalid";

sub cifsError {

    my (%args) = validate (@_, {
	sErrorMessage	=> { type => SCALAR },
	sSourceName	=> { type => SCALAR, optional => 1, default => (caller(1))[3] . " [line " . (caller())[2] . "]"},
	bCroak		=> { type => SCALAR, optional => 1, default => 0 }
       });
    my $errMsg = $args{sSourceName} . " | " . $args{sErrorMessage};
    
    print $errMsg . "\n";
    
    if ($args{bCroak}) {
      croak($args{sErrorMessage});
    }
    
    return 1;
}

sub cifsGetLastError {
    ($^E) ? return $^E : return $!;
}

1;
