#!/usr/local/bin/perl
#
# Copyright (c) 2013 Network Appliance, Inc.
# All Rights reserved
#
## @summary
##  Library for sending the commands to nvsc.exe
##
## @description
##  Provides function for sending commands to nvsc.exe using NACL.
#
## @dependencies        none
#
## @status              public
#
## @author              bhaskarm@netapp.com

package CIFS::Nvsc;
use Data::Dumper;
use strict;
use Params::Validate qw/validate_with SCALAR ARRAYREF/;
use NATE::BaseException qw(:try);
use NACL::APISet;

sub start {
    my ($self) = shift;
    my $obj    = {};
    my %opts   = validate_with(
        params => \@_,
        spec   => {
            hostobj  => { isa => 'Hostrec::Windows' },
            filer_ip => {
                type     => SCALAR,
                default  => '127.0.0.1',
                optional => 1
            },
            executable_path => {
                type     => SCALAR,
                optional => 1,
                default  => "x:\\bin\\offbox_av\\nvsc.exe",
            }
        }
    );

    #my %opts = @_;
    my $path;
    my $Hostobj    = $opts{hostobj};
    my $APISet_obj = NACL::APISet->new(
        hostobj   => $Hostobj,
        category  => "Host",
        interface => "CLI",
        set       => "Windows",
        connid    => 'login',
        conntype  => 'rsh-persist',
    );

    #print Dumper($APISet_obj);

    $obj->{_Apiset} = $APISet_obj;

    $path = $opts{'executable_path'};
    my $command  = $path . " " . $opts{filer_ip};
    my $response = $APISet_obj->execute_raw_command(
        command              => $command,
        'additional_prompts' => ["nvsc>"]
    );
    if ( $response =~ /\S/s ) {
        NATE::BaseException->throw(
            "Throwing NATE::BaseException: Failed to open executable: $response"
        );
    }
    bless $obj, $self;
    return $obj;
}

sub delay {
    my ($self)     = shift;
    my %opts       = @_;
    my $delay_time = $opts{delay_time};
    my $response   = $self->{_Apiset}->execute_raw_command(
        command              => "d",
        'additional_prompts' => ["nvsc>"],
        "connectrec-match_table" =>
            [ "Enter the delay value:" => "$delay_time" ]
    );
    return 0;
}

sub mark_as_virus {
    my ($self) = shift;
    my %opts = @_;

    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "v",
        'additional_prompts' => ["nvsc>"]
    );
    return 0;
}

sub mark_as_clean {
    my ($self) = shift;
    my %opts = @_;

    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "V",
        'additional_prompts' => ["nvsc>"]
    );
    return 0;
}

sub update_scanner {
    my ($self)   = shift;
    my %opts     = @_;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "u",
        'additional_prompts' => ["nvsc>"]
    );
    return 0;
}

sub stop {
    my ($self)   = shift;
    my %opts     = @_;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "q",
        'additional_prompts' => ["nvsc>"]
    );
    return 0;
}

sub reset_request_count {
    my ($self) = shift;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "R",
        'additional_prompts' => ["nvsc>"]
    );
    return 0;
}

sub get_request_count {
    my ($self) = shift;
    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "r",
        'additional_prompts' => ["nvsc>"]
    );
    ##my $string=$response->get_raw_output();
    if ( $response =~ /The total count of requests so far is (.+)/ ) {
        return $1;
    }
    return -1;
}

sub stop_logging {
    my ($self) = shift;
    my %opts = @_;

    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "C",
        'additional_prompts' => ["nvsc>"]
    );
    return 0;
}

sub start_logging {
    my ($self) = shift;
    my %opts = @_;

    my $response = $self->{_Apiset}->execute_raw_command(
        command              => "c",
        'additional_prompts' => ["nvsc>"]
    );
    return 0;
}

1;
