# this is basically -*- perl -*-
# $Id: /depot/prod/test/main/lib/CIFS/Params_CIFSGX.pm
#
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.
#
## @author olgav@netapp.com
## @status	design
#
=head1 NAME

Params_CIFSGX - common CIFS definitions for Options.pm parameters

=head1 SYNOPSIS

  use Options;
  use CIFS::Params_CIFSGX;


=head1 DESCRIPTION

This package is used to define the common CIFS parameters used by
GX testers. It can be used by anyone using the L<Options> package
as a convienient way to keep your options all consistent.

=head1 STATUS

  Shared

=head1 AUTHOR

  olgav@netapp.com

=head1 COPYRIGHT

  Copyright 2005, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package CIFS::Params_CIFSGX;


use Options;

my $Options = new Options();
$Options->parameters( definitions => [
    {
        name => "AD_SERVER",
        description => "Name of the Active Directory Server",
        spec        => "s@",
        default     => [qw(CIFSQA-DC-3 CIFSQA-DC-2 CIFSQA-DC-1)],
    },
    {
        name => "AD_SERVER_IP",
        description => "IP address of the Active Directory Server",
        spec        => "s@",
        default     => [qw(172.17.152.42 172.17.152.41 172.17.152.40)],
    },
    {
        name => "AD_SERVER_USER",
        description => "Name of a user in AD that can join machines to the domain",
        default     => "administrator",
    },
    {
        name => "AD_SERVER_PASSWORD",
        description => "Password for user in the Active Directory Server",
        default     => "Hello123",
    },
    {
        name => "POSITION",
        description => "Position number for user",
        default     => "1",
   },
   {
        name  => "CIFS_DOMAIN_LONG",
        description => "Fully qualified name of the CIFS domain",
        default     => "CIFSQA.LAB.NETAPP.COM",
    },
    {
        name => "CIFS_DOMAIN",
        description => "Short name of the CIFS domain",
        default     => "CIFSQA",
    },
    {
        name => "CIFS_SUPERUSERS",
        description => "Comma separated list of CIFS superusers to create",
        spec        => "s@",
        default     => "Administrator",
    },
    {   
        name => "CIFS_SHARE",
        spec        => "s",
        description => "The name (NetBios name) of the CIFS shares",
        default     => "test_share",
    },
    {
        name => "AGGR",
        description => "Comma separated list of Aggr sizes to create on each node in the cluster",
        default     => 3,
    },
    {
        name => "AGGRTYPE",
        description => "Comma separated list of Aggr types (raid4, raid_dp) for the created aggrs.  Last type used if AGGRTYPE list shorter than AGGR list",
        default     => "raid_dp",
    },
    {
        name => "AGGR_NAME",
        description => "Aggregate name",
        default     => "testaggr",
    },
    {
        name => "AGGR_SIZE",
        description => "Aggregate size",
        default     => 3,
    },
    {
        name => "VSERVER",
        description => "Name of the VServer to create",
        default     => "vs0",
    },
    {
        name => "AGGRLOOP",
        description => "How many times to loop around while waiting for an aggregate to be created",
        default     => 60,
    },
    {
        name => "AGGRSLEEP",
        description => "How long to wait between checks of aggregate creation status",
        default     => 3,
    },
    {
        name => "AGGR_NAMES",
        description => "Comma separated list of aggregate name",
        spec        => "s@",
        default     => [qw(testaggr testaggr1 testaggr2)],
    },
    {
        name => "INITIAL_VOLUME_SIZE",
        description => "Initial size for all volumes",
        default     => "200MB",
    },
    {
        name => "INITIAL_VOLUME_FILES",
        description => "Initial number of files in a volume",
        default     => 20000,
    },
    {
        name => "UNIX_PERMISSIONS",
        description => "Unix Permissions",
        default     => "0777",
    },
    {
        name => "DNS_SERVER_IP",
        description => "DNS Server IP address",
        spec        => "s@",
        default     => [qw(172.17.152.42 172.17.152.40)],
    },
    {
        name => "DNS_DOMAIN",
        description => "DNS domain names",
        spec        => "s@",
        default     => [qw(CIFSQA.LAB.NETAPP.COM )],
    },
    {
        name => "NIS_SERVER",
        description => "NIS Server Name or IP addresses",
        spec        => "s",
        default     => "172.19.2.16",
    },
    {
        name => "NIS_DOMAIN",
        description => "NIS domain name",
        default     => "lab.netapp.com",
    },
    {
        name => "TIMEZONE",
        description => "Timezone on node",
        default     => "America/Los_Angeles",
    },
    {
        name => "NS_SWITCH",
        description => "Nameserver switch value",
        valid_values => [ qw(nis file all) ],
        default     => [qw(file nis)],
        spec        => "s@",
    },
    {
        name => "FILER_VOL",
        description => "Name of CIFS test volume",
        default     =>  "testvol_flex",
    },
    {
        name => "JUNCTION_PATH",
        description => "Janction-path to create volume",
    },
    {
        name => "VOLUME_TYPE",
        description => "type of volume",
        valid_values => [ qw(RW LS DR DP TMP TEMP UNKNOWN SNAPSHOT *) ],
        default     => "RW",
    },
    {
        name => "VOL_STRIPE_FORMAT",
        spec        => "s",
        description => "Stripe format for the volume(s) full (Acro) or data (Coral)",
        valid_values => [ qw(data full) ],
        default     => "full",
    },
    {
        name => "VOL_SECURITY_STYLE",
        spec        => "s",
        description => "Security style for the volume(s)",
        valid_values => [ qw(unix ntfs mixed) ],
        default     => "mixed",
    },
    {
        name => "ANTIVIRUS_ON_ACCESS_POLICY",
        description => "Antivirus on_access policy name.",
        default     => "default",
    },
    {
        name => "EXPORT_POLICY",
        description => "Export policy name.",
        default     => "default",
    },
    {
        name => "VOL_STRIPE_FORMATS",
        description => "Comma separated list of formats for striped vol to create",
        spec        => "s@",
    },
    {
        name => "OU",
        description => "Organization Unit. e.g. dpgqa",
        default     => undef 
    },
    {
        name => "VSERVERS",
        description => "Comma separated list of vserver names to create",
        spec        => "s@",
        default     => [qw(vs0 vs1 vs2)],
    },
    {
        name => "CIFS_SERVERS",
        description => "Comma separated list of cifs server names to create",
        spec        => "s@",
        default     => [qw(cifs_vs0 cifs_vs1 cifs_vs2)],
    },
    {
        name => "CIFS_DELETE",
        description => "Flag to delete cifs server",
        default     => 0,
    },
    {
        name => "LDAP_CLIENT_CONFIG",
        description => "Client-config information for LDAP server",
        default     => "OpLdapClient",
    },
    {
        name => "LDAP_SERVER",
        description => "IP Information for LDAP server",
        default     => "172.17.152.197",
    },
    {
        name => "LDAP_SCHEMA",
        description => "LDAP Schema",
        default     => "RFC-2307",
    },
    {
        name => "LDAP_BASE_DN",
        description => "LDAP",
        default     => "dc=lab,dc=netapp,dc=com",
    },
    {
        name => "LDAP_BIND_DN",
        description => "LDAP",
        default     => "cn=admin,dc=lab,dc=netapp,dc=com",
    },
    {
        name => "LDAP_BIND_PASSWORD",
        description => "LDAP",
        default     => "hello123",
    },
    {
        name => "NEW_FUNCTIONALITIES",
        description => "Parameter that shows we are running script on release that has new functional implementation",
        default     => 1,
    },
    {
        name => "NEW_SECD_FUNC",
        description => "Parameter that shows we are running script on release that has new SecD CLI",
        default     => 1,
    },
    {
        name => "NEW_PREFERRED_DC",
        description => "Parameter that shows we are running script on release that has new way to discover LIFs for SecD",
        default     => 1,
    },
    {
        name => "AV_TIMEOUT",
        description => "Timeout value for av job show",
        default     => "150"
    },

    {
        name => "UNIX_CLIENT",
        description => "Unix client name",
        default     => "lnx-rh5-4.cifsqa.lab.netapp.com"
    },

    {   name => "UNIX_USER",
        description => "Solaris User",
        default     => "root",
    },

    {   name => "UNIX_PASSWD",
        description => "Solaris User password",
        default     => "sundance",
    },

    {   name => "FCNTL_LINUX_EXEC_FILE",
        description => "Path to exec file location on Linux",
        default     => "/usr/software/test/noarch/agnostic/cifs/bin/linux/fcntl.Linux",
    },

    {   name => "FCNTL_SUNOS_EXEC_FILE",
        description => "Path to exec file location on Sun",
        default     => "/usr/software/test/noarch/agnostic/cifs/bin/SunOs/fcntl.SunOS",
    },
    {
        name => "NEW_PATH_VALIDATION",
        description => "Parameter that shows we are running script on release that has new functional for cifs share path validation",
        default     => 1,
    },
    {
        name => "INFINITEVOL",
        description => "If tests are to be run against Infinitevol - value 0/1",
        default     => 0,
    },
    {
        name => "SHARE_PATH",
        description => "Infinitevol path where share would be created",
        default     => "/NS",
    },
    {
        name => "SYMLINK_DEFAULT_ENABLED",
        description => "symlink is enabled by default.",
        default     => "1",
    },

]);

# If there are any options that have changed names here is where we would put them
$Options->compatability( definitions=>
    {
    });
1;
