package CIFS::Smoke_Setup;

# Compiler directives.
#
use strict;
use warnings;
#
# Module import.
#
use NACL::C::Node;
use NACL::C::NetworkInterface;
use NACL::C::Vserver;
use NACL::C::VserverCifs;
use NACL::C::VserverNameMapping;
use NACL::C::VserverServicesDns;
use NACL::C::UnixGroup;
use NACL::C::UnixUser;
use NACL::C::ExportPolicy;
use NACL::C::License;
use NACL::C::Component;
use NACL::C::DiagNbladeCifsServer;
use NACL::C::Volume;
use NACL::C::NetworkInterface;
use NACL::C::DiagNbladeCifs;
use NACL::C::VserverCifsHomeDirectory;
use NACL::C::NetworkRoutingGroupsRoute;
use NACL::C::Aggregate;
use NACL::C::Vserver;
use NACL::C::VserverServicesDns;
use NACL::C::ExportPolicyRule;
use NACL::STask::STask;
use Data::Dumper;
use NACL::APISet;
use NACL::MTask::CIFS;
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException;
use Params::Validate qw/validate_with SCALAR ARRAYREF/;
use Client qw(execute);
use Tharn qw($Log);
use Options;
use Server;
use TCD;
use Tharn qw($Log);
use CIFS::Smoke_Setup;

#
# Global Variables.
#
use vars qw(
      $Test
      @TC_LIST
      $nodes
      $primary_node
      $cifs_server_object
      $aggregate
      $domain
      $test_aggregate
      $root_volume
      $vserver
      $primary_lif_name
      $primary_lif_address
      $primary_addresses
      $lif_netmask
      $primary_data_lif_obj
      $routing_group
      $routing_group_object
      $routing_group_gateway
      $routing_group_destination
      $name_servers
      $test_node  
      $Options
      $Params
      $vserver_name
      $cifs_server 
      $cifs_domain_long
      $ad_server_user
      $ad_server_password
      $home_port
      $exportpolicy_rule_object
      @Cleanup
      $Host1
      %all_objects
      );  

####################################################################################
## This module creates setup based on the parameters provided to it. It checks 
## if the objects are already create then use it or if it absent then create it.
## Here are list of object in order setup creates:
## a. Aggregate
## b. Vserver
## c. Data LIF
## d. Network routing group route 
## e. Mtask cifs server object (This build all the component require for cifs server)
## f. Export-policy rule for default export policy.
##
## Test Control Parameters
##
## This program takes the following required parameters:
##
## @param VSERVER                                Required: vserver name
## @param VSERVERS_IP                            Requied : 1 IP address for creating 
##                                                         data lif's. 
## @param NODES                                  Required: 2 comma seperated names 
##                                                         of the nodesin cluster
## @param CIFS_SERVER                            Required: cifs server name
## @param CIFS_DOMAIN_LONG                       Required: cifs domain name
## @param AGGR_NAME                              Required: aggregate name
## @param VSERVER_ROOT_VOLUME                    Required: root volume name on vserver
## @param LIF_NETMASK                            Required: netmask for creating lif
## @param ROUTING_GROUP                          Required: routing group address
## @param ROUTING_GROUP_GATEWAY                  Required: routing group gateway 
## @param ROUTING_GROUP_DESTINATION              Required: routing group destination 
####################################################################################
##
##  a. One cluster with two nodes (choose one node as primary and other as secondary).
##  e.g
##  pinsk::> cluster show
##  Node                  Health  Eligibility
##  --------------------- ------- ------------
##  pinsk-01              true    true
##  pinsk-02              true    true
##  2 entries were displayed.
##
####################################################################################
##
##  b. One aggregate on node chosen as primary node.
##  e.g.
##  pinsk::> aggr show -nodes pinsk-01
##  Aggregate     Size Available Used% State    #Vols Nodes            RAID Status
##  --------- -------- --------- ----- ------- ------ ---------------- ------------
##  aggr1      119.5GB   13.82GB   88% online       7 pinsk-01         raid_dp,
##                                                                     normal
##
####################################################################################
##
##  c. One vserver on an aggregate created on primary node of current
##     cluster
##  e.g.
##                     Admin     Root                  Name    Name
##  Vserver     Type    State     Volume     Aggregate  Service Mapping
##  ----------- ------- --------- ---------- ---------- ------- -------
##  mouse       cluster running   mouseRootV aggr1      file    file
##                                ol
##
####################################################################################
##
##  d. Strictly One data lif created on vserver from step c.
##  e.g.
##              Logical    Status     Network            Current       Current Is
##  Vserver     Interface  Admin/Oper Address/Mask       Node          Port    Home
##  ----------- ---------- ---------- ------------------ ------------- ------- ----
##  mouse
##              mouse_lif    up/up    172.17.91.6/24     pinsk-01      e1a     true
##
####################################################################################
##
##  e. One CIFS server setup on vserver from step 3.
##  e.g.
##              Server          Domain/Workgroup Authentication
##  Vserver     Name            Name             Style
##  ----------- --------------- ---------------- --------------
##  mouse       MOUSE           CIFSQA           domain
##
####################################################################################

###############################################################################
# Subroutine  : smoke_setup
# Input       : $Params a reference variable containing all input variable
# Return      : %all_objects
# Description : This subroutine setup required by cifs smoke. It tries to use
#               if the requested object is already present or if not found 
#               then it creates the object
###############################################################################


sub smoke_setup {

    my ($Params) = @_;
    # General details required to run tests
    $vserver_name             = $Params->{VSERVER};
    $cifs_server              = $Params->{CIFS_SERVER};
    $domain                   = $Params->{CIFS_DOMAIN};
    $cifs_domain_long         = $Params->{CIFS_DOMAIN_LONG};
    $ad_server_user           = $Params->{AD_SERVER_USER};
    $ad_server_password       = $Params->{AD_SERVER_PASSWORD};
    $aggregate                = $Params->{AGGR_NAME};
    $root_volume              = $Params->{VSERVER_ROOT_VOLUME};

    # 2 nodes of the cluster in use
    $nodes                    = $Params->{NODES};
    $primary_node             = $$nodes[0];  

    # Data lif details on primary node 
    $primary_addresses        = $Params->{VSERVERS_IP};
    $primary_lif_name         = $Params->{PRIMARY_NODE_FIRST_DATA_LIF_NAME};
    $primary_lif_address      = $$primary_addresses[0];

    # Details required to create data lif's on 2 nodes of cluster.
    $home_port                 = $Params->{HOME_PORT};
    $lif_netmask               = $Params->{LIF_NETMASK};
    $routing_group             = $Params->{ROUTING_GROUP};
    $routing_group_gateway     = $Params->{ROUTING_GROUP_GATEWAY};
    $routing_group_destination = $Params->{ROUTING_GROUP_DESTINATION};
    $name_servers              = $Params->{NAMESERVERS};

    $test_node = $Params->{NODE};
    $Test      = $Params->{TEST};
    $Test->description("Prepare initial setup for testcase execution. ");

     # Create a test aggregte
     $Test->step("Create a test aggregate");

     try {
         $test_aggregate = NACL::C::Aggregate->find(
             command_interface => $test_node,
             filter            => { aggregate => $aggregate },
         );
     } ## end try
     catch NACL::Exceptions::NoElementsFound with {
         $test_aggregate = NACL::C::Aggregate->create(  command_interface => $test_node,
                                                    aggregate         => $aggregate,
                                                    diskcount         => 3);
         $Test->nlog( "Waiting till the test aggregate is "
                 . "created on node "
                 . $test_node->name()
                 . "\n" );
         my $end_time = time() + 720;
         while ( $test_aggregate->state( apiset_must => { interface => "CLI" } )
             ->state() ne 'online' && time() < $end_time )
         {
             Tharn::snooze(60);
         }
     };## end catch

    # Create a virtual server
    $Test->step("Create a virtual server $vserver_name");

    try {
        $vserver = NACL::C::Vserver->find(
            command_interface => $test_node,
            filter            => { vserver => $vserver_name },
        );
    } ## end try
    catch NACL::Exceptions::NoElementsFound with {
        $vserver = NACL::C::Vserver->create( 
            command_interface => $test_node,
            "method-timeout"  => 300,
            vserver           => $vserver_name,
            rootvolume        => $root_volume,
            aggregate         => $aggregate,
            'ns-switch'       => ["file"],
            'nm-switch'       => ["file"],
            'rootvolume-security-style' => 'mixed',
        );
    };## end catch

    # Create a NetworkInterface
    $Test->step("Create a data LIF on tested vserver $vserver_name node "
                ."$primary_node");
    try {
        $primary_data_lif_obj = NACL::C::NetworkInterface->find(
                                command_interface => $test_node,
                                filter => { 
                                    vserver     => $vserver_name,
                                    lif         => $primary_lif_name,
                                }
                          );
    }
    catch NACL::Exceptions::NoElementsFound with {
        $primary_data_lif_obj  = NACL::C::NetworkInterface->create(  
                               command_interface   => $test_node,
                               vserver             => $vserver_name,
                               lif                 => $primary_lif_name,
                               'data-protocol'     => ['cifs'],
                               role                => 'data',
                               'home-node'         => $primary_node,
                               'home-port'         => $home_port,
                               address             => $primary_lif_address,
                               netmask             => $lif_netmask,
                           );
        sleep 15;
    };## end catch

    # Create a NetworkRoutingGroupsRoute
    $Test->step("Create a network routing group route for tested vserver "
                ."$vserver_name");
    try {
        $routing_group_object  = NACL::C::NetworkRoutingGroupsRoute->find(  
                                    command_interface   =>  $test_node,
                                    filter => { 
                                        vserver             =>  $vserver_name,
                                        'routing-group'     =>  $routing_group,
                                        'destination'       =>  $routing_group_destination,
                                        'gateway'           =>  $routing_group_gateway
                                    }   
                               );
    }
    catch NACL::Exceptions::NoElementsFound with {
        $routing_group_object  = NACL::C::NetworkRoutingGroupsRoute->create(  
                                    command_interface   =>  $test_node,
                                    vserver             =>  $vserver_name,
                                    'routing-group'     =>  $routing_group,
                                    'destination'       =>  $routing_group_destination,
                                    'gateway'           =>  $routing_group_gateway
                               );
    };## end catch


    # Create a CIFS server
    $Test->step(  "Create CIFS server ". $cifs_server ." from tested vserver "
                 ."$vserver_name");

    $cifs_server_object = NACL::MTask::CIFS->setup(
           'command_interface' => $test_node,
           'vserver'           => $vserver_name,
           'cifs-server'       => $cifs_server,
           'cifs_domain'       => $cifs_domain_long,
           'username'          => $ad_server_user,
           'password'          => $ad_server_password,
           'name-servers'      => [$name_servers],
           'policyname'        => 'Sample',
           'if_exists'         => 'reuse',
           'vserver_name_mapping' => 
            [ 
                 {
                    'pattern'     => 'root',
                    'direction'   => 'unix-win',
                    'replacement' => $domain . '\\\\' . $ad_server_user,
                    'position'    => 1,
                 },
            ],
            'unix_users' => [
                                { 
                                  user          => 'nobody',
                                  id            => 65534,
                                  'primary-gid' => 65534,
                                },
                                { 
                                  user          => 'root',
                                  id            => 0,
                                  'primary-gid' => 0,
                                },
                            ],
            'unix_groups' => [
                                 {
                                   name         => "nobody",
                                   id           => 65534
                                 },  
                                 {
                                   name         => "root",
                                   id           => 0
                                 },  
                             ],
    );

    push @Cleanup, $cifs_server_object;

    # Create a rule index for testing purpose.

    $Test->step("Find / Create export policy rule");
    try {
        $exportpolicy_rule_object = NACL::C::ExportPolicyRule->find(
                    command_interface => $test_node,
                    filter => { 
                        vserver           => $vserver_name,
                    }
                 );
    }
    catch NACL::Exceptions::NoElementsFound with {
        $exportpolicy_rule_object =
          NACL::C::ExportPolicyRule->create(
            command_interface => $test_node,
            ruleindex         => 1,
            vserver           => $vserver_name,
            policyname        => 'default',
            clientmatch       => "0.0.0.0/0",
            rorule            => ["any"],
            protocol          => ["any"],
            rwrule            => ["any"]
        );
    };

    sleep 30;
    $all_objects{'aggregate'}           = $test_aggregate;
    $all_objects{'vserver'}             = $vserver;
    $all_objects{'primary_data_lif'}    = $primary_data_lif_obj;
    $all_objects{'routing_group'}       = $routing_group_object;
    $all_objects{'cifs_server'}         = $cifs_server_object;
    $all_objects{'export_policy_rule'}  = $exportpolicy_rule_object;

    return %all_objects;

}

###############################################################################
# Subroutine  : smoke_cleapup
# Input       : $Params, %all_objects
# Return      : None
# Description : This subroutine clean the setup created by subroutine smoke_setup
###############################################################################

sub smoke_cleapup {

    my ($Params, %all_objects) = @_;

    my $test_aggregate           = $all_objects{'aggregate'};
    my $vserver                  = $all_objects{'vserver'};
    my $primary_data_lif_obj     = $all_objects{'primary_data_lif'};
    my $routing_group_object     = $all_objects{'routing_group'};
    my $cifs_server_object       = $all_objects{'cifs_server'};
    my $exportpolicy_rule_object = $all_objects{'export_policy_rule'};


    $test_node                = $Params->{NODE};
    $Test                     = $Params->{TEST};
    $ad_server_user           = $Params->{AD_SERVER_USER};
    $ad_server_password       = $Params->{AD_SERVER_PASSWORD};
    $root_volume              = $Params->{VSERVER_ROOT_VOLUME};
    $vserver_name             = $Params->{VSERVER};

    #Deleting name mapping
    $Test->step("Deleting name mapping");
    foreach my $name_mapping (@{$cifs_server_object->{name_mapping}}) {
        $name_mapping->delete();
    }

    #Deleting unix groups 
    $Test->step("Deleting unix users");
    foreach my $unixUser (@{$cifs_server_object->{unix_user}}) {
        $unixUser->delete();
    }

    #Deleting unix groups 
    $Test->step("Deleting unix groups");
    foreach my $unixGroup (@{$cifs_server_object->{unix_group}}) {
        $unixGroup->delete();
    }

    #Deleting export policy rule delete
    $Test->step("Deleting export policy rule");
    $exportpolicy_rule_object->delete();

    #Deleting export policy
    $Test->step("Deleting export policy");
    $cifs_server_object->{export_policy}->delete();
    
    #Deleting cifs-server 
    $Test->step("Deleting cifs-server");
    $cifs_server_object->{cifs_server}->delete('username' => $ad_server_user,
                                               'password' => $ad_server_password);
                                          

    $Test->step("Deleting Routing Group");
    $routing_group_object->delete();
     
    $Test->step("Deleting Logical Interface");
    $primary_data_lif_obj->delete();
     
    # to delete the vserver the root volume neeeds to be offline and deleted.

    NACL::C::Volume->modify(
        command_interface => $test_node,
        volume => $root_volume,
        vserver => $vserver_name,
        state  => "offline",
        apiset_must => { interface => "CLI",},
    );
    NACL::C::Volume->delete(
        command_interface => $test_node,
        vserver => $vserver_name,
        volume => $root_volume,
        apiset_must => { interface => "CLI",},
    );

    #Deleting Vserver
    $Test->step("Deleting Vserver");
    $vserver->delete();
    
    $Test->step("Deleting Aggregate");
    $test_aggregate->delete();
    #Cleanup done

}

1;