#!/usr/local/bin/perl
## $Id://depot/prod/test/main/cifs/Sparse_File_Attribute/sfaLib.pm#7 $
#
# Copyright (c) 2005 Network Appliance, Inc.
# All Rights reserved
#
## @summary
##  Library for SFA testing
##
## @description
##  Provides functions to test Sparse File Attribute
#
## @dependencies	none
#
## @status	        new
#
## @author	        saurabhg@netapp.com

package CIFS::sfaLib;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT =
  qw(doCleanup setAndTestSparseAttr setupClients setupFiler getFileAttrs getFileSize);

use strict;
use warnings;
use Tharn;
use QtreeCmds;
use MapShare qw(map_cifs_share);

#this library is meant for SFA testing for IC
#Following hard coded paths are being used
our $qtpath       = "/vol/sparsevol/sfatest";
our $volName      = "sparsevol";                #required for vol status command
our $sparseVolume = "/vol/$volName";
our $shareName    = "sfa";
our $exeSetSparse = "X:\\bin\\cifs\\ccover\\SparseFileTest\\setsparseattr";
our $exeQueryInfo = "X:\\bin\\cifs\\ccover\\SparseFileTest\\QueryFileInfo";

sub setAndTestSparseAttr {

    #Function to the core of sparse test functionality
    # This sets the sparse bit and checks it
    #It also tests the expected size difference in the file
    #parameters passed are:
    #1. client connection handle
    #2. Should attribute test be done 0/1
    #3. Should the size test be done  0/1
    #4. Is size change expected       0/1
    #5. Filer connection handle
    #6. Test File name             s:\\testfile

    my ( $handle, $testAttr, $testSize, $exptSizeChange, $flrHandle, $testFile )
      = @_;
    my $sparseStatus = 1;
    my $sizeStatus   = 1;
    my $sizeApparent = my $sizeActual = undef;

    #Query for sparse attribute
    if ($testAttr) {
        my $attrVal = getFileAttrs( $handle, $testFile );
        logcomment("QueryFileInfo says value is:$attrVal");
    }

    if ($testSize) {
        $sizeApparent = getFileSize( $flrHandle, $sparseVolume );
        logcomment( "Size before sparse setting is " . $sizeApparent );
    }

    #set the sparse bit, independent of value of testAttr & testSize
    say( $$handle, "$exeSetSparse $testFile" );

    if ($testAttr) {

        #Query for sparse attribute
        my $attrVal = getFileAttrs( $handle, $testFile );
        logcomment(
"QueryFileInfo after setting sparse attribute says value is:$attrVal"
        );

        unless ( $attrVal eq "220" ) {
            $sparseStatus = 0;
            logresult(
                type  => 'INFO',
                abort => 0,
                msg   => "Attribute value on the file is not 220"
            );
        }
    }

    if ($testSize) {
        $sizeActual = getFileSize( $flrHandle, $sparseVolume );
        logcomment( "Size After sparse setting is " . $sizeActual );

        my $sizeDiff = scalar($sizeApparent) - scalar($sizeActual);

        if (   ( ($exptSizeChange) && ( $sizeDiff <= 0 ) )
            || ( ( $exptSizeChange == 0 ) && ($sizeDiff) ) )
        {
            $sizeStatus = 0;
            logresult(
                type  => 'INFO',
                abort => 0,
                msg   => "Size test of the file has failed"
            );
        }
    }

    logcomment("Sparse Test Passed on the file") if $sparseStatus;
    logcomment("Size Test Passed on the file")   if $sizeStatus;

    if ( $sizeStatus && $sparseStatus ) {
        logcomment("Sparse Attribute set Successfully on the file");
        return 1;
    }
    else {
        logcomment("Sparse Attribute Could Not be set on the file");
        return undef;
    }
}

sub setupFiler {

    #function to setup the filer for testing
    # input parameters:
    #1. Filer connection handle
    #2. qtree Security style
    #
    # Fucntion creates a volume, then a qtree with specified security style

    my ( $handle, $style, $voltype ) = @_;

    if ( $voltype eq "trad" ) {
        createTradVol( $handle, $volName );
    } else {    
        #create volume
        my $match1 = say( $$handle, "vol create $sparseVolume $main::FILER_AGGR 1g" );
        $match1 = say( $$handle, "vol status", "-wait", $volName, "-fail",
                  "sparse volume does not exist, Test aborted" );
    }

    #create qtree
    qtreeCreate( $$handle, $qtpath );
    logresult(
        type  => 'CONFIG',
        abort => 1,
        msg   => "qtree not created, Test Aborted"
    ) unless qtreePresent( $$handle, $qtpath );

    #set security style if not set already
    my $current = qtreeSecurity( $$handle, $qtpath );
    if ( $current =~ m/$style/i ) {
        logcomment("qtree already set to $style security Style");
    }
    else {
        my ( $match, undef, $after ) =
          say( $$handle, "qtree security $qtpath $style");
        logcomment("qtree set to $style security Style");
    }

    #add the qtree for cifs shares with name sfa
    say( $$handle, "cifs shares -add $shareName $qtpath" );
    my ( $match, $before, $after, undef ) =
      say( $$handle, "cifs shares", "-wait", $shareName );

    #print"\n $match $after\n"       ;
    logresult(
        type  => 'CONFIG',
        abort => 1,
        msg   => "Failed to create share, Test Aborted"
    ) unless ($match);
}

sub setupClients {

    # map the required drive on the Windows client as s:
    #Parameters expected are:
    #1. Client connection handle
    #2. IP address of the filer
    #3. Domain
    #4. user for mapping
    #5. Password for the above user

    my ( $chandle, $filerIP, $domain, $user, $password ) = @_;

   #map the filer drive on windows client
    my $mapStatus = undef;
    ( undef, undef, $mapStatus ) = say( $$chandle,
        "net use s: \\\\$filerIP\\$shareName /u:$domain\\$user $password" );

    # sleep(10)    ;
    ( $mapStatus =~ m/The command completed successfully./i )
      ? logcomment("Filer share mapped to s: successfully")
      : logresult(
        type  => 'CONFIG',
        abort => 1,
        msg   => "Unable to map filer share to s:, Test Aborted"
      );
  
}

sub doCleanup {

    #To cleanup all the things created for testing SFA
    #1. delete the mapped drive on windows client
    #2. Disconnect the windows client
    #3. remove the cifs share
    #4. remove the qtree
    #5. Remove the volume
    #6. Disconnect the Filer connection

    my ( $clhandle, $flrHandle ) = @_;

    my ( undef, undef, $mapStatus ) = say( $$clhandle, "net use s: /delete" );

    logcomment($mapStatus);

    #disconnect($$clhandle);

    #remove share, qtree, volume
    say( $$flrHandle, "cifs shares -delete -f $shareName" );
    say( $$flrHandle, "priv set advanced;qtree delete -f $qtpath" );
    say( $$flrHandle, "vol offline $sparseVolume" );
    say( $$flrHandle, "vol destroy $sparseVolume -f" );

    #disconnect($$flrHandle);

}

sub getFileAttrs {

    #To simply get and return the current file attributes
    #Parameters expected are:
    #1. Client Connection Handle
    #2. Path to the file

    my ( $clHandle, $filePath ) = @_;

    my $output = undef;
    ( undef, undef, $output ) = say( $$clHandle, "$exeQueryInfo $filePath" );
    my $attrVal = substr( ( split( /attr =/, $output ) )[1], 8, 3 );
    return $attrVal;
}

sub getFileSize {
    my ( $flrHandle, $sparseVolume ) = @_;
    sleep(5);    # Needed for proper working of df command
    my ( undef, undef, $output, undef ) =
      say( $$flrHandle, "df -r $sparseVolume" );

    my $temp = ( split( /\/vol\/sparsevol\//, $output ) )[1];

    my $size = ( split( /\s+/, $temp ) )[2];
    return $size;
}

sub createTradVol {

    #function to setup the filer with traditional Volumes for testing
    # input parameters:
    #1. Filer connection handle
    #2. qtree Security style
    #
    # Fucntion creates a traditional volume and waits for it to come online

    my ( $handle, $volName ) = @_;

    #create volume
    say( $$handle, "vol create $volName -t raid_dp 3" );
    my $match1 = say( $$handle, "vol status", "-wait", "$volName", "-fail",
        "sparse volume does not exist, Test aborted" );
    my $status = undef;
    do {
        my ( undef, undef, $after, undef ) = say( $$handle, "vol status" );

        $status = ( split( /$volName/, $after ) )[1];
        $status = ( split( /raid_dp/,   $status ) )[0];
        $status =~ s/\s+//g;

        print "\nbefore comapring status is: ".$status    ;

        unless ( $status eq "online" ) {
            logcomment("Probably disk zeroing not over, check sysconfig -r")
              if $status eq "creating";
            logcomment("Volume is not ready yet");
            sleep(300);
        }
    } until ( $status eq "online" );
}


1;
