#
#  $Id: //depot/prod/ontap/main/test/lib/widelink.pm 
#
#  Copyright (c) 2004 Network Appliance, Inc.
#  All rights reserved.
#
##
## @summary Support routines for widelink  tests
##
## @description
##      This file contains routines that
##      let you create widelink on the filer.
##
## @author  dl-maryv (FSS Protocols QA)

package CIFS::widelink;
use  vars ('@ISA', '@EXPORT');
require Exporter;
our @ISA = qw(Exporter);
use strict;
use Tharn;
#use ZapiErrno;
use Zapi;
use TestEnv::MinWin qw(:all);
use misc;
use ZapiTest;
use NaElement;
use  rsh('rsh');
use Module::Common qw();

BEGIN
  {
	     Module::Common::setup_export_tags(
		'basic' => [qw(
		createSymlink
		createWidelink
		validateDirCmd
		validateCdAndDirCmd
		validateFileContents
		grepFile
		pktflush
		appendToFile
		grepContent
		printTestCaseResult
		printTestCase
		printMsg
		deleteQtree
		deleteFile
	) ]
	);
}


sub createSymlink
{
    # Just in case the symlink already exists,
	my $Filer = shift;
	my $Filer_ip = shift;
	my $path = shift;
	my $symlinkvals = shift;
	my $h = host($Filer);
	my $password = $h->{default_password};
	my $server = new ZapiTest($Filer_ip, "root", $password);
	my $o = $server->Zapi::invoke_api("file-delete-file", "path", "$path");

	# Now try to create it
	$o = $server->Zapi::invoke_api("file-create-symlink", "symlink",
				       "$symlinkvals", "path", "$path");
	if ( ! $o->results_ok() ) {
	  printError("Failure in file-create-symlink: "
		     .$o->toString());
	  return -1;
	}

	printMsg("Created symlink $symlinkvals in $path successfully.");
	return 0;
}

sub createWidelink {
  my ($template, $root_vol, $target, $wl_entry, $Filer_ip, $Filer) = @_;
  my $file = "/vol/$root_vol/etc/symlink.translations";
  if (createSymlink($Filer, $Filer_ip, $target, $template) < 0) {
    return -1;
  }

  if (&appendToFile($Filer, $Filer_ip, $file, $wl_entry, -1) < 0) {
    printError("Could not write widelink entry:".
	       "$wl_entry.".
	       "to $file!\n");
    return -1;
  }
  printMsg("Created widelink $wl_entry in $file"); 
  return 0;
}

sub validateDirCmd($$$)
{
	my ($path, $expect, $winclient) = @_;
	&validateCdAndDirCmd("", $path, $expect, $winclient);	
}

sub validateCdAndDirCmd($$$$) {
  my ($drive, $path, $expect, $winclient) = @_;
  my $cmd = "\"pushd $drive\\ & dir $path & popd\"";
  my $ret = execute_cmd($winclient, $cmd);
  my $grep_ret;
  
  printMsg("Validating output of '$path' by expecting $expect to be one".
	   "of the files\n");
  $grep_ret = &grepContent($ret, $expect);
  if ($grep_ret < 0) {
    printError("Expected: $expect in following output of 'dir':".
	       "$grep_ret");
  }
  else {
    printMsg(".....succeeded!");
  }

  return $grep_ret;
}

	 
sub validateFileContents($$$)
{
  my ($path_file, $expect, $winclient) = @_;
  my $cmd = "more $path_file";
  my $ret = execute_cmd($winclient, $cmd);
  my $grep_ret;

  printMsg("Validating contents of $path_file by expecting:\n$expect in it\n");
  $grep_ret = &grepContent($ret, $expect);
  if ($grep_ret < 0) {
    printError("Expected: $expect in file but got the following:".
	       "$ret");
  } else {
    printMsg(".....succeeded!");
  }
  
  return $grep_ret;
}


sub appendToFile
{
  my $Filer= shift;
  my $Filer_ip = shift;
  my $path= shift;
  my $data = shift;
  my $offset= shift;
  my $h = host($Filer);
  my $password = $h->{default_password};
  my $server = new ZapiTest($Filer_ip, "root",$password);
  my $x1 = &str_to_hex($data);
  my $o = $server->invoke_api("file-write-file", "path", "$path",
			      "data", "$x1", "offset","$offset");
  if ( !$o->results_ok()) {
    return -1;
  }
  return 0;
}

sub pktflush($$) {
  my $winclient= shift;
  my $path_to_dfsutil= shift;
  my $cmd="$path_to_dfsutil /pktflush";
  my $ret = execute_cmd($winclient, $cmd);
}

sub str_to_hex($) {
  my $s = shift;
  my $r = "";
  my $leng = length($s);
  for ( my $n=0; $n < $leng; $n++ ) {
    my $c = substr($s,$n,1);
    my $hx = sprintf "%02x", ord("$c");
    $r = $r . $hx;
  }
  return($r);
}
	       
sub hex_to_str($){
	my $s = shift;
	my $r = "";
	my $leng = length($s);
	for ( my $n=0; $n < $leng; $n+=2 )
	{
		my $hx = substr($s,$n,2);
		my $hn = hex("$hx");
		$r = $r . chr($hn);
	}
	return($r);
}


# VV IMP: If $expect contains characters that have special meanings in regular
# expression syntax, eg '\', '.', '/' etc, then MAKE SURE THEY ARE preceded by 
 #a backslash character, or in other words, they are escaped.
#
# e.g., $expect = "dir1\\" will not even cause the code to run, since the pattern
# ends with a bacslash and /dir1\/ confuses PERL.
# So, the right way is to escape it: $expect = "dir1\\\\"
sub grepContent($$)
{
  my ($contents, $expect) = @_;
  return ($contents =~ /$expect/) ? 0 : -1;
}

sub printTestCase($)
{
  my ($case) = @_;
  printMsg($case);
}

sub printTestCaseResult($$$)
{
	my ($case, $fails, $numTests) = @_;
	my $result = ($fails == 0) ? "SUCCESS" : "FAILURE($fails/$numTests)";
	printMsg("Test Case#$case:\t$result!\n");
}

sub printMsg($)
{
  my ($msg) = @_;
  tstlog("-comment", $msg);
}

sub printError($) {
  my ($msg) = @_;
  logcomment("$msg");  
}

sub deleteFile($$$){
	my($Filer, $Filer_ip, $fileName) = @_;
	my $h = host($Filer);
        my $password = $h->{default_password};
        my $server = new ZapiTest($Filer_ip, "root", $password);
        my $o = $server->Zapi::invoke_api("file-delete-file", "path", "$fileName");
	if ( ! $o->results_ok() ) {
          printError("Unable to remove $fileName".
                "Failure in file-delete-file:" .$o->toString());
          return -1;
        }
}

sub deleteQtree($$){
	my($Filer, $qtreePath) = @_;
	my $h = host($Filer);
	rsh($h, "priv set advanced; qtree delete -f $qtreePath");
}


