# $Id: //depot/prod/test/Rsierranevada/lib/CR/C/DebugSmdbTableFactory.pm#1 $
# Copyright (c) 2001-2010 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary VserverNFS Task Module
## @author dl-crqa-auto@netapp.com
## @status private
## @pod here

package CR::C::DebugSmdbTableFactory;

use strict;
use warnings;

use CR::C::DebugSmdbTableRecord;
use List::MoreUtils qw(:all);
use NATE::BaseException;
use Params::Validate qw(:all);
use Readonly;
use Tharn qw($Log);

Readonly our $Default_Delim  => 'X;X';

=head1 NAME

CR::C::DebugSmdbTableFactory

=head1 DESCRIPTION

C<CR::C::DebugSmdbTableFactory> simulate the component state of a row
return by the DebugSmdbTable<bah>

=head1 ATTRIBUTES

=head2 command_interface
(Required) A component object that represents the host to which to send comands.

=head2 -table
(Required) Debug Smdb Table we want to access

=head2 filter
(Optional) Filers to limit the result set return.

=head1 METHODS

=head2 find

  my $rows = CR::C::DebugSmdbTableFactory->find(
        command_interface => $command_interface,
        '-table'          => 'myTable',
        filter            => {
            'row1' => 'value1',
            'row2' => 'value2',
        }
  );

(Class method) get the rows of table column for table C<myTable>
containing row matching C<row1=value1> and C<row2=value2>.

=cut
sub find {
    $Log->enter();
    my %opts = validate_with(
        params      => \@_,
        allow_extra => 1,
        spec        => {
            command_interface   => { type => HASHREF },
            '-table'            => { type => SCALAR  },
            filter              => { type => HASHREF , default => ()},
            allow_empty         => { type => BOOLEAN,  default => 0},
        },
    );

    if ( any { $_ !~ /command_interface|-table|filter|allow_empty|^CR::C/ } (keys %opts) ) {
        $Log->warn('Usage of params other than command_interface, allow_empty, or filter is deprecated for smdb tables and will be disabled at a future time.');
    }

    my $apiset = $opts{'command_interface'}->apiset(
        category    => "Node",
        interface   => "CLI",
        set         => "CMode",
    );

    my $cmd = 'debug smdb table ' . $opts{'-table'} . ' show';
    my $fields_hash = $opts{'filter'};
    my %fields = $fields_hash ? %{$fields_hash} : ();

    foreach my $key (keys %fields) {
        $cmd = $cmd . " -" . $key . " " . $fields{$key};
    }

    my $output = $apiset->execute_raw_command(
        command => $cmd,
    );
    if ($output =~ /Error:/){
       NATE::BaseException->throw("Fail to access table: " . $output);
    }
    my @data = _table_parse($output);
    my @result;
    foreach my $row (@data) {
        my $record = CR::C::DebugSmdbTableRecord->new('data' => $row );
        push( @result, $record );
    }

    if ( !$opts{allow_empty} and !@result ) {
        $Log->warn("No matching entries were found and allow_empty is not true.");
    }
    $Log->exit();
    return wantarray ? @result : $result[0];
}

sub _table_parse
{
    my ($raw_table, $delim) = @_;
    if (!$delim)
    {
        $delim = $Default_Delim;
    }

    my @final_result = ();
    my @rows = split(/\n/,$raw_table);
    my @keywords = split(/$delim/, shift @rows);
    shift @rows; # Second row is long version of keywords

    foreach my $row (@rows)
    {
        my @row_values = split(/$delim/, $row);
        my %row_hash = pairwise { $a => $b } @keywords, @row_values;
        push(@final_result, \%row_hash);
    }

    return @final_result;
}

1;
