# Copyright (c) 2011 Network Appliance, Inc.
# All rights reserved.

## @summary unit test library
## @author  dl-crqa-auto@netapp.com

=head1 NAME

Client Utils

=head1 SYNOPSIS

 use CR::CRClientUtils;


=head1 DESCRIPTION
Client Utils


=head1 STATUS

  Shared

=head1 AUTHOR

  karthik7 (CR QA Automation)
=head1 COPYRIGHT

  Copyright 2010, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package CR::CRClientUtils;

use strict;
use warnings;
use Tharn;
use Options;

use TCD;
use CR::CommonParams;
use Hostrec qw();

use NACL::C::RepositoryVolume;
use NACL::C::Client;
use NACL::C::Client::Mount;
use NACL::C::NetworkInterface;
use Populate qw (create_set delete_set);
use TestUtils;
use Data::Dumper;
use Data::Random qw( rand_words );

use vars qw($Options %SiteData);
our %_wordHits;

=head2 ::new

Creates a CR::ClientUtils object and returns it.

=over

=item Arguments

=item C<<"connection"=>$Node >>

(Mandatory) A connection to one of the nodes. This will be
stored in the object while executing other methods if invoked
as instance methods.

=item Examples

=back

    my $util = CR::ClientUtils->new( connection => $Node, )

=item Returns
=over
Returns a reference to a NACL Repository Volume object.

=cut

sub new {
    my ($class, %args) = @_;
    my $self = {};
    bless $self, $class;
    $Options = new Options;
    $Options->process();
    $self->{_connection} = $args{connection};
    #$self->{_nodes} = $self->get_all_nodes;
    %SiteData = TestUtils::get_site_info();        # From TestUtils
    return $self;
}

=head2 :: mount_client
=over

=item Arguments

=item C<<"connection"=>$Node >>

(Optional) A connection to one of the nodes. This will be
stored in the object while executing other methods if invoked
as instance methods.

=item Examples

=back

    my $mount_obj = CR::ClientUtils->mount_client( connection => $Node,)

=cut


sub mount_client {

    my ($self, %args) = @_;
    my $Node = $args{connection} || $self->{_connection};
    my $Client = $args{client} || $Options->get(option => 'CLIENT');
    my ($Client_Name) = @$Client;
    my $client_host_obj = Hostrec->new(id => "$Client_Name");

    my $Client_obj = NACL::C::Client->new(name => $Client_Name);
    $Node->refresh_command_interface();
    #Find vserver, volume and Junction Path
    #vol show -volume-style infinitevol
    my $repo = NACL::C::Volume->find(command_interface => $Node,
                                     filter            => 
                                        {'volume-style' => 'infinitevol'}
                                    );
    my $Volume = $repo->get_one_state_attribute('volume');
    $Log->comment(" Volume is $Volume ");
    my $Vserver = $repo->get_one_state_attribute('vserver');
    $Log->comment(" Vserver is $Vserver ");
    my $Junction_path = $repo->state()->junction_path();
    $Log->comment(" Junction Path is $Junction_path ");
    #Get Data IP
    my $data_lif = NACL::C::NetworkInterface->find(
                            command_interface=>$Node,
                            filter => { 'vserver' => $Vserver,
                                        role => 'data',
                            },
    );
    my $Data_ip = $data_lif->get_one_state_attribute('address');
    my $data_node = $data_lif->get_one_state_attribute('curr-node');

    my ($mount_obj, $mount_path);
    #Mount Namespace on Client
    if($client_host_obj->{OS} eq 'SunOS'){
        #Work around for burt 507488
        $mount_path='/t/'.$Data_ip.$Junction_path;
        $mount_obj = $mount_path;
        my $Api_Set_Obj_Client = NACL::APISet->new(
                                 name => $Client_Name ,
                                 category => "Host",
                                 interface => "CLI",
                                 set => 'Solaris',
                                 );
        my @commands_on_client_machine = (
            "/usr/software/bin/sudo /usr/sbin/umount $mount_path",
            "/usr/software/bin/sudo mkdir -p $mount_path",
            '/usr/software/bin/sudo /usr/sbin/mount -F nfs -o hard,intr,noquota'
            ." $Data_ip:$Junction_path $mount_path \n",
            "/usr/software/bin/sudo chmod 777 $mount_path",
            "/usr/software/bin/sudo /usr/sbin/mount | grep /t",
        );
        foreach my $command (@commands_on_client_machine){
            my $command_output = $Api_Set_Obj_Client->execute_raw_command(
                command => "$command",
            );
            $Log->comment("$command : $command_output");
        }
    }else{
        $mount_obj = NACL::C::Client::Mount->create(
                    command_interface => $Client_obj,
                    server            => $Data_ip,
                    path              => $Junction_path,
                    );
        $mount_path = $mount_obj->get_one_state_attribute('mount_point');
    }
    $self->mount_obj($mount_obj);
    $self->mount_path($mount_path);

    $Log->comment(" Mount Path is $mount_path");
    return wantarray ? ($mount_obj, $data_node) : $mount_obj;
}

=head2 :: execute_system_command
=over

=item Arguments

=item C<<"mount_path" => $mount_path >>

(Mandatory) Mounted path where the command thas to be executed

=item C<<"command" => $Command >>

(Mandatory) command to execute

=item Examples

=back

    my ($mount_obj,$mount_path,$Junction_path) =
                CR::ClientUtils->execute_system_command(
                                mount_path => $mount_path,
                                command => $Command,
                                );

=cut
#TODO enhance this to do on nay client. Can use APISet
sub execute_system_command {
    my ($self, %args) = @_;
    my $mount_path = $args{mount_path};
    my $command = $args{command};
    my $Client = $args{client} || $Options->get(option => 'CLIENT');
    my ($Client_Name) = @$Client;
    my $client_host_obj = Hostrec->new(id => "$Client_Name");
    $Log->comment("Execute command on host : $Client_Name");
    my $set;
    if($client_host_obj->{OS} eq 'SunOS'){
        $set = 'Solaris';
    }else{
        $set = 'Linux';
    }
    my $Api_Set_Obj_Client = NACL::APISet->new(
                        name => $Client_Name ,
                        category => "Host",
                        interface => "CLI",
                        set => $set,
                        );
    my $command_output = $Api_Set_Obj_Client->execute_raw_command(
                        command => "cd $mount_path; $command",
                        );
    $Log->comment("$command on $mount_path : $command_output");
    return $command_output;
}

=head2 :: do_stat_check
=over

=item Arguments

=item C<<"mount_path" => $Mount_Path >>

(Mandatory) Mounted path where the command thas to be executed

=item C<<"trace_hash" => \%trace_hash >>

(Mandatory) Referece the hash built from tshark
    parser method CR::nfs::Tshark->get_readirplus_hash()

=item C<<"test" => $Test >>

(Mandatory) Test Object

=item Examples

=back
    my ($Mount,$Mount_path,$Junction_path) =
                CR::ClientUtils->do_stat_check(
                                mount_path => $Mount_Path,
                                trace_hash => \%trace_hash,
                                test => $Test
                                );
=cut

sub do_stat_check {
    my ($self, %args) = @_;
    my $Mount_path = $args{mount_path} || $self->mount_path();
    my $trace_hash = $args{trace_hash};
    my $Client = $args{client} || $Options->get(option => 'CLIENT');
    my ($Client_Name) = @$Client;
    my $client_host_obj = Hostrec->new(id => "$Client_Name");
    my $set;
    if($client_host_obj->{OS} eq 'SunOS'){
        $set = 'Solaris';
    }else{
        $set = 'Linux';
    }
    my $Api_Set_Obj_Client = NACL::APISet->new(
                        name => $Client_Name ,
                        category => "Host",
                        interface => "CLI",
                        set => $set,
                        );
    my @failures=();
    my $time_threshold = 2; #milliseconds
    foreach my $file(keys(%$trace_hash)){
        next if ($file eq '.');
        next if ($file eq '..');
        my $command_output = $Api_Set_Obj_Client->execute_raw_command(
                        command => "perl -e 'print join (\" \", stat(\""
                                ."${Mount_path}/${file}\"));'",
                        );
        if(!$command_output){
            $Log->comment("stat check on ${Mount_path}/${file} did not return"
                         .' any value');
        }
        next unless($command_output);
        my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,$atime,$mtime,
            $ctime,$blksize,$blocks) = split(/ /, $command_output);

        if(($size != $trace_hash->{$file}{size})){
            $Log->comment("Failed : Size for $file on client is $size but ".
                    "tracefile is $trace_hash->{$file}{size}");
            push (@failures,"Failed : Size for $file on client is $size but ".
                    "tracefile is $trace_hash->{$file}{size}");
        }

        $mode = sprintf ("%04o", $mode & 07777);
        if($mode != $trace_hash->{$file}{mode}){
            $Log->comment("Failed : Permission for $file on client is $mode"
                    ." but tracefile is $trace_hash->{$file}{mode}"
            );
            push (@failures,"Failed : Permission for $file on client is $mode "
                    ."but tracefile is $trace_hash->{$file}{mode}"
            );
        }

        if(($nlink != $trace_hash->{$file}{nlink})){
            $Log->comment("Failed : Links for $file on client is $nlink but ".
                        "tracefile is $trace_hash->{$file}{nlink}"
            );
            push(@failures,"Failed : Links for $file on client is $nlink but ".
                        "tracefile is $trace_hash->{$file}{nlink}"
            );
        }
        if(($atime =! $trace_hash->{$file}{atime_seconds})){
            if (int ($trace_hash->{$file}{atime_seconds} - $atime)
                    > $time_threshold) {
                $Log->comment("Failed : Atime for $file on client is $atime ".
                        "but tracefile is $trace_hash->{$file}{atime_seconds}"
                );
                push(@failures,"Failed : Atime for $file on client is $atime ".
                        "but tracefile is $trace_hash->{$file}{atime_seconds}"
                );
            }
        }
        if(($mtime != $trace_hash->{$file}{mtime_seconds})){
            if (int ($trace_hash->{$file}{mtime_seconds} - $mtime)
                    > $time_threshold) {
                $Log->comment("Failed : mtime for $file on client is $mtime ".
                        "but tracefile is $trace_hash->{$file}{mtime_seconds}"
                );
                push(@failures,"Failed : mtime for $file on client is $mtime ".
                        "but tracefile is $trace_hash->{$file}{mtime_seconds}"
                );
            }
        }
        if(($ctime != $trace_hash->{$file}{ctime_seconds})){
            if (int ($trace_hash->{$file}{ctime_seconds} - $ctime )
                    > $time_threshold) {
                $Log->comment("Failed : ctime for $file on client is $ctime ".
                        "but tracefile is $trace_hash->{$file}{ctime_seconds}"
                );
                push(@failures,"Failed : ctime for $file on client is $ctime ".
                        "but tracefile is $trace_hash->{$file}{ctime_seconds}"
                );
            }
        }
    }
    return @failures;
}

=head2 :: unmount_client
=over

=cut


#unmount the $mount object ccreated at mount_client
sub unmount_client {
    my ($self, %args) = @_;
    my $Client = $args{client} || $Options->get(option => 'CLIENT');
    my ($Client_Name) = @$Client;
    my $client_host_obj = Hostrec->new(id => "$Client_Name");
    if($client_host_obj->{OS} eq 'SunOS'){
        my $mount_path = $self->{'mount_path'};
        my $Api_Set_Obj_Client = NACL::APISet->new(
                        name => $Client_Name ,
                        category => "Host",
                        interface => "CLI",
                        set => 'Solaris',
                        );
        my $command_output = $Api_Set_Obj_Client->execute_raw_command(
                        command => '/usr/software/bin/sudo /usr/sbin/umount'
                                   ." -f $mount_path",
                        );
    }else{
        my $mount_obj = $self->mount_obj();
        $mount_obj->delete();
    }
}


=head2 mount_obj

Mutator/Accessor method - Sets the $self->{'mount_obj'} value if $mount_obj
is passed. Else returns mount object ($self->{'mount_obj'})

=over

=item Arguments

=item C<$mount_obj)>
(Optional) Required when using as mutator method.

=item Returns

=item C<$mount_obj)>
(Optional) Returns $mount_obj when called as Accessor method

=item Examples
$mount_obj = $Client_Utils_Obj->mount_obj()
$Client_Utils_Obj->mount_obj($mount_obj)

=back

=cut

sub mount_obj {
    my ($self, $mount_obj) = @_;

    if ($mount_obj) {
        $self->{'mount_obj'} = $mount_obj;
    } else {
        return $self->{'mount_obj'};
    }
}

=head2 mount_path

Mutator/Accessor method - Sets the $self->{'mount_path'} value if $mount_path
is passed. Else returns mount path ($self->{'mount_path'})

=over

=item Arguments

=item C<$mount_path)>
(Optional) Required when using as mutator method.

=item Returns

=item C<$mount_path)>
(Optional) Returns $mount_path when called as Accessor method

=item Examples
$mount_path = $Client_Utils_Obj->mount_path()
$Client_Utils_Obj->mount_obj($mount_path)

=back

=cut

sub mount_path {
    my ($self, $mount_path) = @_;

    if ($mount_path) {
        $self->{'mount_path'} = $mount_path;
    } else {
        return $self->{'mount_path'};
    }
}

1;
