## Copyright (c) 2012 NetApp Inc.
## All rights reserved.
##
## @name CdmiDataObjectNameless.pm
##
## @summary Nameless CDMI Data object requests
##
## @description
##   Implements Nameless CDMI Data object requests
##
## @keywords cr, cdmi, http, nameless dataobject
##
## @author Pavan Verma
##
## @example
##
## @status private

=pod

=head1 Package Name

CR::CdmiDataObjectNameless

=head1 Author/Maintainer

Pavan Verma

=head1 Description
Implements Nameless CDMI Data object requests

=cut

package CR::CdmiDataObjectNameless;

# Compiler directives
use strict;
use warnings;

use NATE::Log qw(log_global);
use WWW::Curl;
use WWW::Curl::Easy;
use HTTP::Response qw();
use File::stat;

my $Log = log_global();

################################################################################
# Create a single nameless object
################################################################################
sub Create_Object
{
    my $port = shift(@_);
    my $datafile = shift(@_);
    my $curl = shift(@_);

    my $oid = '';

    open (my $infile, $datafile) or $Log->error("$!");
    $curl->setopt(CURLOPT_READDATA, $infile);
    my $sb = stat($datafile);
    $curl->setopt(CURLOPT_POSTFIELDSIZE, $sb->size);

    my $response_body;
    $curl->setopt(CURLOPT_WRITEDATA,\$response_body);

    # Starts the actual request
    my $retcode = $curl->perform;

    close $infile;

    # Looking at the results...
    if ($retcode == 0)
    {
        my $response_code = $curl->getinfo(CURLINFO_HTTP_CODE);
        if ($response_code != 201)
        {
            $Log->error("Request failed with error $response_code");
            return '';
        }

        # judge result and next action based on $response_code
        my $response = HTTP::Response->parse($response_body);
        my $location = $response->header("Location");
        ($oid) = ($location =~ /.*:$port(\/cdmi_objectid\/\w*)/);
        return $oid;
    }
    else
    {
        # Error code, type of error, error message
        my $errstr = $curl->strerror($retcode);
        my $errbuf = $curl->errbuf;
        $Log->error("An error happened: $retcode, $errstr, $errbuf");
        return '';
    }
}

################################################################################
# Create a bunch of nameless objects
################################################################################
sub Create_Many
{
    my $host = shift(@_);
    my $port = shift(@_);
    my $datafile = shift(@_);
    my $count = shift(@_);

    my $curl = WWW::Curl::Easy->new;
    my $url = 'http://' . $host . ':' . $port . '/cdmi_objectid/';

    $curl->setopt(CURLOPT_HEADER, 1);
    $curl->setopt(CURLOPT_URL, $url);
    $curl->setopt(CURLOPT_POST, 1);
    $curl->setopt(CURLOPT_CONNECTTIMEOUT, 15);
    $curl->setopt(CURLOPT_TIMEOUT, 30);

    my @headers;
    $headers[0] = "Expect:";
    $headers[1] = "Content-Type: text/plain";
    $curl->setopt(CURLOPT_HTTPHEADER, \@headers);

    my @oids;
    my $oid;

    for (my $i = 0; $i < $count; $i++)
    {
        $oid = Create_Object($port, $datafile, $curl);
        if ($oid ne '')
        {
            push(@oids, $oid);
        }
        else
        {
            return $i;
        }
    }

    return ($count, @oids);
}

1;
