package CR::CommonParams;

## @summary
##  Common parameters used in Content Repository tests.
##
##  WARNING:
##  Adding new parameters is fine as long as no other parameters are used the same way.
##  Avoid changing any existing parameters since it may change behaviours of other scripts.

use Params::Common;
use Options;

my $Options = new Options;
$Options->parameters(
    definitions => [
        {   name => "RESETCLUSTER",
            description =>
                "Reset the cluster, this will delete all volume and vservers",
            spec    => "boolean",
            default => 0,
        },
        {   name        => "ALLOW_MULTIPLE_VSERVER",
            description => "Will the script allow multiple vserver",
            spec        => "boolean",
            default     => 1,
        },
        {   name => "DELETE_AGGR",
            description =>
                "Reset the cluster, this will delete all volume and vservers",
            spec    => "boolean",
            default => 0,
        },
        {   name          => "AGGR_CREATE_WAIT",
            description   => "Total wait time in seconds of create_aggr test",
            spec          => "s",
            default       => 7200,
            greater_equal => 0,
        },
        {   name => "AGGR_CHECK_INTERVAL",
            description =>
                "Interval in seconds between each Aggregate state check",
            spec          => "s",
            default       => 30,
            greater_equal => 5,
        },
        {   name          => "TIMEOUT",
            description   => "Set the command timeout value",
            spec          => "s",
            default       => 600,
            greater_equal => 0,
        },

   # AGGR_NAME_LIST will be generated from AGGR_COUNT and AGGR_PREFIX if not define
        {   name        => "AGGR_NAME_LIST",
            description => "List or aggregate we want to use or create",
            spec        => "s@",
            default     => (),
        },
        {   name          => "AGGR_DISK_COUNT",
            description   => "Number of disk per aggregate",
            spec          => "s",
            default       => 3,
            greater_equal => 2,
        },
        {   name        => "VSERVER",
            description => "Vserver name",
            spec        => "s",
            default     => "vs0",
        },
        {   name        => "FLEXVOL_VSERVERS",
            description => "Flex Volume Vserver name",
            spec        => "s",
            default     => "vs1",
        },
        {   name        => "FLEXCACHE_VSERVERS",
            description => "Flex Cache Volume Vserver Name",
            spec        => "s",
            default     => "vs2",
        },
        {   name        => "CVOL_JUNCTION_PATH",
            description => "Junction path for the InfiniteVol",
            spec        => "s",
            default     => "/NS",
        },
        {   name        => "CVOL_SIZE",
            description => "Repository volume size",
            spec        => "s",
        },
        {   name        => "CVOL_NAME",
            description => "Name of the InfiniteVol",
            spec        => "s",
            default     => "repo_vol",
        },
        {   name        => "FLEXVOL_NAME",
            description => "Name of the flexvol",
            spec        => "s",
            default     => "flex_vol",
        },
        {   name        => "CVOL_TYPE",
            description => "InfiniteVol volume type",
            spec        => "s",
            default     => "RW",
        },
        {   name        => "OLS_AGGR_LIST",
            description => "List of OLS aggregate",
            spec        => "s@",
            default     => [ "ols_aggr_1", "ols_aggr_2" ],
        },
        {   name        => "SECURITY_STYLE",
            description => "Security style for Vserver and volume",
            spec        => "s",
            default     => "mixed",
        },
        {   name        => "DATA_LIF_PROTOCOL",
            description => "data-protocol for the data lif",
            spec        => "s@",
            default     => [ 'nfs', 'cifs' ],
        },
        {   name         => "DATA_LIF_NUM",
            description  => "Number of data lif required",
            spec         => "s",
            default      => "AUTO",
            valid_values => [ "AUTO", "0", "1" ],
        },
        {   name         => "NFS_PROTOCOL",
            description  => "List of NFS protocols to enable",
            spec         => "s@",
            default      => ["v3"],
            valid_values => [ "v3", "v4.0", "v4.1", "v4.1-pnfs" ],
        },
        {   name         => "V4_ACL",
            description  => "NFSv4.0 ACL Support",
            spec         => "s",
            default      => "disabled",
            valid_values => [ "enabled", "disabled" ],
        },
        {   name         => "V4_READ_DELEGATION",
            description  => "NFSv4.0 Read Delegation Support",
            spec         => "s",
            default      => "disabled",
            valid_values => [ "enabled", "disabled" ],
        },
        {   name         => "V4_WRITE_DELEGATION",
            description  => "NFSv4.0 Write Delegation Support",
            spec         => "s",
            default      => "disabled",
            valid_values => [ "enabled", "disabled" ],
        },
        {   name        => "V4_ID_DOMAIN",
            description => "NFSv4 ID Mapping Domain",
            spec        => "s",
            default     => "WIN.SMOKE",
        },
        {   name         => "V4_1_ACL",
            description  => "NFSv4.1 ACL Support",
            spec         => "s",
            default      => "disabled",
            valid_values => [ "enabled", "disabled" ],
        },
        {   name        => "LICENSE",
            description => "List of license to add to the cluster",
            spec        => "s@",
            default     => [
                "VMZIVZGKWVMADF", "BAKJZZGKWVMADF",
                "NUHYZZGKWVMADF", "DLOFYZGKWVMADF"
            ],
        },
        {   name        => "FEATURES",
            description => "List of Features to add to the cluster",
            spec        => "s@",
            default     => [ "NFS", "CIFS", "Snapmirror_DP", "SnapRestore" ],
        },
        {   name        => "UNIX_PERMISSIONS",
            description => "unix premissions for vserver and infinvol",
            spec        => "s",
            default     => "---rwxrwxrwx",
        },
        {   name => "AD_SERVER_USER",
            description =>
                "AD server admin name, needed for remove cifs vserver",
            spec    => "s",
            default => "adadmin",
        },
        {   name => "AD_SERVER_PASSWORD",
            description =>
                "AD server admin password, needed for remove cifs vserver",
            spec    => "s",
            default => "adadmin",
        },
        {   name        => "CIFS_DOMAIN",
            description => "Domain name for CIFS server",
            spec        => "s",
            default     => "CNC",
        },
        {   name        => "CIFS_DOMAIN_LONG",
            description => "Domain name for CIFS server",
            spec        => "s",
            default     => "CNC.SPIN.ENG.NETAPP.COM",
        },
        {   name => "AD_SERVER_IP",
            description =>
                "List of IP addresss of the Active Directory servers",
            spec    => "s@",
            default => [ "10.98.203.74", "10.98.202.208" ],
        },
        {   name        => "CIFS_SERVER",
            description => "CIFS server name",
            spec        => "s",
        },
        {   name        => "CIFS_SHARE",
            description => "CIFS SHARE name",
            spec        => "s",
            default     => "cifs_share",
        },
        {   name        => "CIFS_SUPERUSERS",
            description => "CIFS Super User",
            spec        => "s@",
            default     => [],
        },
        {   name        => "LOCAL_UNIX_USERS",
            description => "Local unix user",
            spec        => "s",
            default =>
                'root:0:0:"Local Root",nobody:65534:65534:nobody,smoke:18811:30:smoke',
        },
        {   name        => "LOCAL_UNIX_GROUPS",
            description => "Local unix group",
            spec        => "s",
            default     => 'root:0,nobody:65534,engr:30',
        },
        {   name        => "SKIP_NAME_MAPPING_SETUP",
            description => "Should name-mapping setup be done?",
            spec        => "boolean",
            default     => 0,
        },
        {   name        => "AGGR_PREFIX",
            description => "prefix for aggregate name",
            spec        => "s",
            default     => "aggr",
        },
        {   name          => "AGGR_COUNT",
            description   => "Number of aggregate we want to create",
            spec          => "s",
            default       => 3,
            greater_equal => 1,
        },
        {   name        => "SNAPSHOT_POLICY",
            description => "Snapshot policy for the infinvol",
            spec        => "s",
            default     => "none",
        },
        {   name        => 'SRC_VSERVER',
            description => 'Vserver name from Source Node',
            default     => 'vs0',
        },
        {   name        => 'DST_VSERVER',
            description => 'Vserver name from Destination Node',
            default     => 'vs1',
        },
        {   name        => 'SRC_NODE',
            description => 'One of the nodes from Source Cluster',
        },
        {   name        => 'DST_NODE',
            description => 'One of the nodes from Destination Cluster',
        },
        {   name        => 'DST_NODES',
            description => 'List of the nodes from Destination Cluster',
            spec        => "s@",
        },
        {   name => "CR_VERSION",
            description =>
                "Content Repository Version, default to searchlight",
            spec         => "s",
            default      => "searchlight",
            valid_values => [ "searchlight", "beacon" ],
        },
        {   name        => "CREATE_CIFS",
            description => "Create CIFS",
            spec        => "boolean",
            default     => 0,
        },
        {   name        => "CREATE_CDMI",
            description => "Create CDMI",
            spec        => "boolean",
            default     => 0,
        },
        {   name        => "ENABLE_OBJECT_STORE",
            description => "Enable object store feature",
            spec        => "boolean",
            default     => 0,
        },
        {   name => "DNS_SERVER_IP",
            description =>
                "DNS Server IP, this will override the one provide by the site file",
            spec         => "s@",
        },
        {   name => "DNS_DOMAIN",
            description =>
                "DNS Server DOMAIN, this will override the one provide by the site file",
            spec         => "s@",
        },
        {   name     => "VSERVER_AGGR",
            required => "Aggregate for the VServer root volume is required",
            description => "Aggregate for the VServer root volume",
            spec        => "s",
            default     => 'vs_aggr',
        },
        {
            name        => "NS_SWITCH",
            description => "NS Switches",
            spec        => "s@",
            default     => ['file','nis'],
        },
        {   name => "VSERVER_AGGR_DISK_COUNT",
            description =>
                "Number of disk for vserver aggregate, only use when VSERVER_AGGR is specified",
            spec          => "s",
            greater_equal => 2,
            default       => 3,
        },
        {   name => "DEBUG_CMD",
            description =>
                "command for gathering debug info in a diag_post_test",
            spec => "s",
        },
        {   name => "NUM_REPOS_VSERVER",
            description => "The number of repository vservers to create",
            spec => "s",
            default => 1,
        },
        {   name => "NUM_FLEX_VSERVER",
            description => "The number of flex vservers to create",
            spec => "s",
            default => 0,
        },
        {   name => "VSERVER_NAMES",
            description => "List of vserver names. The first NUM_REPOS_VSERVER "
                . "names are to create repository vservers and then the "
                . "remaining are used for creating flexvol vservers",
            spec        => "s@",
            default     => [],
        },
        {   name        => "ENABLE_SNAPDIFF",
            description => "Enable Snapdiff during InfiniteVol creation",
            spec        => "boolean",
            default     => 0,
        },
        {   name        => 'CREATE_IV_OF_RAND_NAMES',
            description => 'Create IV of different names',
            spec        => "boolean",
            default     => 0,
        },
        {   name        => 'CREATE_IV_OF_INCR_NAMES',
            description => 'Create IV of increamental names',
            spec        => "boolean",
            default     => 0,
        },
    ]
);
$Options->compatability(
    definitions => { SYS_AGGR_LIST => "OLS_AGGR_LIST" } );
$Options->process();

# Generate the AGGR_NAME_LIST from AGGR_COUNT and AGGR_PREFIX if it's not define
if ( scalar( @{ $Options->get( option => "AGGR_NAME_LIST" ) } ) <= 0 ) {
    my @new_aggr_list;
    for (
        my $idx = 1;
        $idx <= $Options->get( option => "AGGR_COUNT" );
        $idx++
        )
    {
        push( @new_aggr_list,
            $Options->get( option => "AGGR_PREFIX" ) . $idx );
    }
    $Options->set( 'AGGR_NAME_LIST' => \@new_aggr_list );
}

1;
