#
# $Id: //depot/prod/test/Rsierranevada/lib/CR/Job.pm#1 $
#
# Copyright (c) 2011 NetApp Inc.
# All rights reserved.
#
## @summary Job Class
## @author davidnew@netapp.com
## @status shared
## @pod here

package CR::Job;

use strict;
use warnings;

use Data::Dumper;
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use Params::Validate qw(:all);
use SPARSE::Util::Logger;
use Tharn qw($Log);

=head1 NAME

Job - The class to encapsulate methods and data of "job" command set

=head1 SYNOPSIS

=begin text

 use Job;

 $cr->job_show(id    => $id,
               name  => $job_name,
               state => $job_state)

 $cr->job_status(name =>$job_name, state => "Running")

=end text

=head1 AUTHOR

David New (davidnew@netapp.com)

=head1 DESCRIPTION

This is a NACL AppBuilder to support test program that uses Content Repository.

This initial release only verifies the syntax of repository rule.

=head1 ROUTINES

=cut

logLevel('info');

#*******************************
# Job Show
#*******************************

=head2 job_show

The purpose of this subroutine is to show job

Optional parameters are

=over 4

=item *
id

id of the job that you would like to monitor

=item *
name

name of the job that you would like to monitor
example: $name of promote job is "repository volume snapshot promote"

=item *
state

state of the job that you would like to monitor

=back

=cut

sub job_show {
  my($self,%args)=@_;
  my $admin_cli = $self->{ngsh};
  my @parsed_output;

  %args = NACL::APISet::validate_params ($self,
                                         params => \%args,
                                         spec   => {
                                           id    => {type => SCALAR, optional => 1},
                                           name  => {type => SCALAR, optional => 1},
                                           state => {type => SCALAR, optional => 1},
                                         });

  @parsed_output=@{$admin_cli->job_show(%args)->get_parsed_output()};
}

#*******************************
# Job Status
#*******************************

=head2 job_status

The purpose of this subroutine is to verify $state of the job

Required parameters are

=over 4

=item *
name

name of the job that you would like to monitor
example: $name of promote job is "repository volume snapshot promote"

=item *
state

The status of the job
state can take one of the values: Complete, Running, Unknown, Failure

=back

=cut

sub job_status {
  my($self,%args)=@_;
  my $admin_cli = $self->{ngsh};
  my @parsed_output;

  %args = NACL::APISet::validate_params ($self,
                                         params => \%args,
                                         spec   => {
                                           name    => {type => SCALAR, optional => 0},
                                           state   => {type => SCALAR, optional => 0},
                                         });
  
  @parsed_output=$self->CR::Job::job_show(name=> $args{name});
  
  if( grep $args{name} eq $_->{name},@parsed_output) {
    if (grep $args{state} eq $_->{state} , @parsed_output) {
      $Log->comment("JOB $args{name} exist");
      return 1;
    } else {
      $Log->comment("JOB $args{name} does not exist");
      return 0;
    }
  } else {
    $Log->comment("JOB $args{name} does not exist");
    return 0;
  }
}
