#
# $Id: //depot/prod/test/Rsierranevada/lib/CR/Repository.pm#1 $
#
# Copyright (c) 2011 NetApp Inc.
# All rights reserved.
#
## @summary Ruleset Class
## @author davidnew@netapp.com
## @status shared
## @pod here

package CR::Repository;

use strict;
use warnings;

use CR::Vserver;
use Data::Dumper;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet;
use Params::Validate qw(:all);
use SPARSE::Util::Logger;
use Tharn qw($Log);

=head1 NAME

Repository - The class to encapsulate methods and data of "repository" commands set

=head1 SYNOPSIS

=begin text

 use Repository;

 $cr->repository_exists(vserver => $vserver,
                        command_interface => $node[0]);

 $cr->repository_show();
 
 $cr->repository_show_constituents();

=end text

=head1 AUTHOR

David New (davidnew@netapp.com)

=cut
logLevel('info');

######################################
# Repository exist check
######################################

=head2 repository_exists

Checking if vserver exists and if yes, check if it bind to the repository

Required parameters are

=over 4

=item *
vserver

Name of the vserver.

=back

=cut

sub repository_exists {
  $Log->enter();
  my ($self, %args) = @_;
  my $admin_cli = $self->{ngsh};
  my @parsed_output;
  
  %args = NACL::APISet::validate_params($self,
                                        params => \%args,
                                        spec => {
                                          vserver => { type => SCALAR, optional => 0 },
                                        },
                                       );


  #  Verify if the vserver and repository exists
#  @parsed_output = @{$admin_cli->vserver_show()->get_parsed_output()};
  @parsed_output=$self->CR::Vserver::vserver_show();
  my $vserver_exists = grep $args{vserver} eq $_->{vserver}, @parsed_output;

  if ($vserver_exists) {
    @parsed_output=$self->CR::Repository::repository_show();

    if (grep $args{vserver} eq $_->{vserver}, @parsed_output) {
      logInfo("Repository and vserver exists\n");
      $Log->exit();
      return 1;
    } else {
      logInfo("Repository for vserver $args{vserver} not exists\n");
      $Log->exit();
      return 0;
    }
  } else {
    logInfo("Vserver $args{vserver} not exists\n");
    $Log->exit();
    return 0;
  }
}

######################################
# Repository Show Constituents
######################################

=head2 repository_show_constituents

Show repository volumes

Required parameters are

None

=back

Optional parameters are

=over 4

=item *
vserver

Name of the vserver.

=item *
volume-role

Volume Role

=back

=cut

sub repository_show_constituents {
  logStep("Calling repository show constituents");
  
  my ($self, %args) = @_;
  my $admin_cli = $self->{ngsh};
  my @parsed_output;

  %args = NACL::APISet::validate_params($self,
                                        params => \%args,
                                        spec => {
                                          vserver => { type => SCALAR, optional => 1 },
                                          'volume-role' => { type => SCALAR, optional => 1 },
                                        },
                                       );

  logInfo("Calling 'repository show constituents'");

  @parsed_output = @{$admin_cli->repository_show_constituents(%args)->get_parsed_output()};
  logInfo("CLI 'repository show constituents' executed successfully.\n");
  return @parsed_output;
}

