#!/usr/software/bin/perl
# $Id$
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
##
## @summary  RestCmds.pm module. This library is used to generate different
##           HTTP commands for the Content Repository project.
##
## @author   dl-cr-dev@netapp.com
##
## @status   Private
##
## @pod      here

package CR::RestCmds;

use strict;
use TracePkg;
use Server;
use Data::Dumper;
use MIME::Base64;

require Exporter;
use vars qw( $VERSION @EXPORT @EXPORT_OK @ISA %EXPORT_TAGS $log_enabled );
our $VERSION = '0.01';
@ISA = qw( Exporter );
@EXPORT = qw(
		$log_enabled
		$http_put_obj
		&dbglog
		&extract_oid_locid
		&extract_domain_id
		&extract_query_complete_and_result_count
		&extract_query_cookie
		&extract_session_id_and_key
		&send_authenticate
		&send_create_obj
		&send_read_obj
		&send_write_obj
		&send_delete_obj
		&send_create_obj_version
		&send_create_path
		&send_read_path
		&send_write_path
		&send_dir_ls
		&send_get_objid
		&send_create_metadata_domain
		&send_close_metadata_domain
		&send_write_metadata
		&send_read_metadata_values
		&send_read_metadata_names
		&send_delete_metadata
		&send_metadata_query
		&send_metadata_query_iteration
);

# Authentication
my ( $http_authenticate );

# Object related
my ( $http_create_obj,
     $http_get_obj,
     $http_put_obj,
     $http_delete_obj,
     $http_create_obj_version );

# Path related
my ( $http_create_path,
     $http_put_path_msg,
     $http_get_path_msg,
     $http_get_dir_ls_msg,
     $http_get_obj_id_msg );

# Metadata related
my ( $http_create_metadata_domain,
     $http_close_metadata_domain,
     $http_write_metadata,
     $http_delete_metadata,
     $http_get_metadata_values,
     $http_get_metadata_names,
     $http_metadata_query,
     $http_metadata_query_iteration,
     $split1,
     $split2 );

my $Trace   = TracePkg->new();

# Authentication related HTTP messages
$http_authenticate = "POST /auth HTTP/1.1\r\nAuthorization: BASE64_ENC_STR\r\nContent-Length: 0\r\n\r\n";

# Object related HTTP messages
$http_create_obj = "POST /object HTTP/1.1\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_get_obj = "GET /object/OBJECT_ID HTTP/1.1\r\nRange: bytes=BYTE_OFFSET1-BYTE_OFFSET2\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_put_obj = "PUT /object/OBJECT_ID HTTP/1.1\r\nx-ntap-locationid: LOCATION_ID\r\nRange: bytes=BYTE_OFFSET1-BYTE_OFFSET2\r\nContent-Length: CONTENT_LENGTH\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_delete_obj = "DELETE /object/OBJECT_ID HTTP/1.1\r\nx-ntap-locationid: LOCATION_ID\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_create_obj_version = "POST /object/version/OBJECT_ID HTTP/1.1\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

# Path related HTTP messages
$http_create_path = "POST /path/OBJECT_PATH HTTP/1.1\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_put_path_msg = "PUT /path/OBJECT_PATH HTTP/1.1\r\nRange: bytes=BYTE_OFFSET1-BYTE_OFFSET2\r\nContent-Length: CONTENT_LENGTH\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_get_path_msg = "GET /path/OBJECT_PATH HTTP/1.1\r\nRange: bytes=BYTE_OFFSET1-BYTE_OFFSET2\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_get_dir_ls_msg = "GET /path/OBJECT_PATH HTTP/1.1\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_get_obj_id_msg = "GET /path/OBJECT_PATH HTTP/1.1\r\nRange: bytes=BYTE_OFFSET1-BYTE_OFFSET2\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

# Metadata domain related HTTP messages
$http_create_metadata_domain = "POST /metadata/domain/DOMAINNAME HTTP/1.1\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_close_metadata_domain = "PUT /metadata/domain/DOMAIN_ID HTTP/1.1\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

# Metadata operations related HTTP messages
$http_write_metadata = "PUT /metadata/OBJECT_ID HTTP/1.1\r\nx-ntap-domainid: DOMAIN_ID\r\nContent-Length: CONTENT_LENGTH\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_delete_metadata = "DELETE /metadata/OBJECT_ID HTTP/1.1\r\nx-ntap-domainid: DOMAIN_ID\r\nContent-Length: CONTENT_LENGTH\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_get_metadata_values = "GET /metadata/values/OBJECT_ID HTTP/1.1\r\nx-ntap-domainid: DOMAIN_ID\r\nContent-Length: CONTENT_LENGTH\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_get_metadata_names = "GET /metadata/names/OBJECT_ID HTTP/1.1\r\nx-ntap-domainid: DOMAIN_ID\r\nContent-Length: 0\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_metadata_query = "POST /metadata/query HTTP/1.1\r\nx-ntap-domainid: DOMAIN_ID\r\nx-ntap-queryname: QUERY_NAME\r\nx-ntap-queryrequestedresultcount: QUERY_REQ_RSLT_COUNT\r\nContent-Length: CONTENT_LENGTH\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$http_metadata_query_iteration = "POST /metadata/query HTTP/1.1\r\nx-ntap-domainid: DOMAIN_ID\r\nx-ntap-queryname: QUERY_NAME\r\nx-ntap-querycookie: QUERY_COOKIE\r\nx-ntap-queryrequestedresultcount: QUERY_REQ_RSLT_COUNT\r\nContent-Length: CONTENT_LENGTH\r\nx-ntap-sessionid: SESSION_ID\r\n\r\n";

$split1 = "GET /metadata/values/Pim5+CpFQTCk2oGWg/E4sQ== HTTP/1.1\r\nx-ntap-sessionid: SESSION_ID\r\nx-ntap-metadata-key: obj";

$split2 = "ectkey\r\nContent-Length: 0\r\n\r\n";

# Debugging
# FIXME: We should move to using the TracePkg library for doing all NATE
#        logging and tracing
sub dbglog
{
	if( $log_enabled eq "1" ) {
		print @_;
	}
}

# Retrieve the response for the HTTP command we just sent
sub get_response {
        my ($sock) = @_;

	my ($hdr, $line);

	dbglog( "\nResponse:\n" );

	# Read header
	while ($line = <$sock>) {
		$hdr .= $line;
		dbglog( $line );
		if ($line =~ /^\r\n/) {
			last;
		}
	}

	$hdr =~ /Content-Length: (\d+)/;
	my $len = $1;
	$hdr =~ /HTTP\/1.1 (\d+) ([\w| ]+)/;
	if ($2 ne "OK") {
		die "Operation failed: $2 ($1)\n";
	}


	if ($len == 0) {
		return ($hdr, '');
	}

	$len -= (length($line)-2);

	my $data;

	dbglog "Waiting for $len bytes of data\n";
	read $sock, $data, $len || die "Failed to read from socket\n";

	return ($hdr, $data);
}

# Helper methods to parse and extract fields from the HTTP response header
sub extract_oid_locid {
	my ($hdr) = @_;
	my ($oid, $locid);

	if ($hdr =~ /x-ntap-objectid: (.*?)\r\n/) {
		$oid = $1;
	} else {
	    die "Failed to extract the x-ntap-objectid field in the "
		."HTTP response header\n";
	}
	if ($hdr =~ /x-ntap-locationid: (.*?)\r\n/) {
		$locid = $1;
	} else {
	    die "Failed to extract the x-ntap-locationid field in the "
		."HTTP response header\n";
	}

	return ($oid, $locid);
}

sub extract_domain_id {
	my ($hdr) = @_;
	my ($domainid);

	if ($hdr =~ /x-ntap-domainid: (.*?)\r\n/) {
		$domainid = $1;
	} else {
	    die "Failed to extract the x-ntap-domainid field in the "
		."HTTP response header\n";
	}

	return ($domainid);
}

sub extract_query_complete_and_result_count {
	my ($hdr) = @_;
	my ($querycomplete, $queryresultcount);

	if ($hdr =~ /x-ntap-querycomplete: (.*?)\r\n/) {
		$querycomplete = $1;
	} else {
	    die "Failed to extract the x-ntap-querycomplete field in the "
		."HTTP response header\n";
	}

	if ($hdr =~ /x-ntap-queryresultcount: (.*?)\r\n/) {
		$queryresultcount = $1;
	} else {
	    die "Failed to extract the x-ntap-queryresultcount field in the "
		."HTTP response header\n";
	}

	return ($querycomplete, $queryresultcount);
}

sub extract_query_cookie {
	my ($hdr) = @_;
	my ($querycookie);

	if ($hdr =~ /x-ntap-querycookie: (.*?)\r\n/) {
		$querycookie = $1;
	} else {
	    die "Failed to extract the x-ntap-querycookie field in the "
		."HTTP response header\n";
	}

	return ($querycookie);
}

sub extract_session_id_and_key {
	my ($hdr) = @_;

	my ($session_id, $session_key);

	if ($hdr =~ /x-ntap-sessionid: (.*?)\r\n/) {
		$session_id = $1;
	} else {
	    die "Failed to extract the x-ntap-sessionid field in the "
		."HTTP response header\n";
	}
	if ($hdr =~ /x-ntap-session-key: (.*?)\r\n/) {
		$session_key = $1;
	} else {
	    die "Failed to extract the x-ntap-session-key field in the "
		."HTTP response header\n";
	}

	return ($session_id, $session_key);
}

# Library wrapper methods to send the HTTP requests to the specified socket
# Authentication related wrapper methods
sub send_authenticate {
        my ($sock, $username, $password) = @_;

	my $msg = $http_authenticate;

	my $input_to_encoder     = $username . ":" . $password;
	my $base64_encoded_creds = encode_base64( $input_to_encoder, "" );
	my $authorization_string = "Basic " . $base64_encoded_creds;

	$msg =~ s/BASE64_ENC_STR/$authorization_string/;

	dbglog "\nAuthenticating user: [$username] and "
	      ."password: [$password]\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

# Object related wrapper methods
sub send_create_obj {
        my ($sock, $session_id) = @_;

	my $msg = $http_create_obj;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Creating object\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_read_obj {
	my ($sock, $oid, $off, $len, $session_id) = @_;

	my $msg = $http_get_obj;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/SESSION_ID/$session_id/;

	$msg =~ s/BYTE_OFFSET1/$off/;
	$off += $len;
	$msg =~ s/BYTE_OFFSET2/$off/;

	dbglog "Reading $len bytes from OBJECT_ID=$oid\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_write_obj {
	my ($sock, $oid, $fh, $off, $data, $session_id) = @_;

	my $msg = $http_put_obj;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/LOCATION_ID/$fh/;
	$msg =~ s/SESSION_ID/$session_id/;

	my $len = length($data);
	$msg =~ s/CONTENT_LENGTH/$len/;

	$msg =~ s/BYTE_OFFSET1/$off/;
	$off += $len;
	$msg =~ s/BYTE_OFFSET2/$off/;

	dbglog "Writing $len bytes to OBJECT_ID=$oid (total_len=", $len + length($msg),")\n\n";
	dbglog "HTTP header      : \n$msg\n";
	dbglog "HTTP message body: \n$data\n";

	print $sock $msg || die "Failed to write to socket\n";
	print $sock $data || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_delete_obj {
	my ($sock, $oid, $fh, $session_id) = @_;

	my $msg = $http_delete_obj;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/LOCATION_ID/$fh/;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Deleting OBJECT_ID=$oid\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_create_obj_version {
	my ($sock, $oid, $fh, $session_id) = @_;

	my $msg = $http_create_obj_version;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/LOCATION_ID/$fh/;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Creating version for OBJECT_ID=$oid\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

# Metadata related wrapper methods
sub send_create_metadata_domain
{
	my ($sock, $domainname, $session_id) = @_;

	my $msg = $http_create_metadata_domain;
	$msg =~ s/DOMAINNAME/$domainname/;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Creating domain $domainname\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	# FIXME: Add logic to wait until you get confirmation of domain
	#        being ready
	sleep( 5 );

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_close_metadata_domain {
	my ($sock, $domainid, $session_id) = @_;

	my $msg = $http_close_metadata_domain;
	$msg =~ s/DOMAIN_ID/$domainid/;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Closing domain having domainID = $domainid\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_write_metadata {
	my ($sock, $oid, $domainid, $data, $session_id) = @_;

	my $msg = $http_write_metadata;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/DOMAIN_ID/$domainid/;
	$msg =~ s/SESSION_ID/$session_id/;

	my $len = length($data);
	$msg =~ s/CONTENT_LENGTH/$len/;

	dbglog "Writing metadata to OBJECT_ID=$oid\n\n";
	dbglog "HTTP header       : \n$msg\n";
	dbglog "HTTP message body : \n$data\n";

	print $sock $msg || die "Failed to write to socket\n";
	print $sock $data || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_read_metadata_values {
	my ($sock, $oid, $domainid, $data, $session_id) = @_;

	my $msg = $http_get_metadata_values;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/DOMAIN_ID/$domainid/;
	$msg =~ s/SESSION_ID/$session_id/;

	my $len = length($data);
	$msg =~ s/CONTENT_LENGTH/$len/;

	dbglog "Reading metadata values from OBJECT_ID=$oid\n\n";
	dbglog "HTTP header       : \n$msg\n";
	dbglog "HTTP message body : \n$data\n";

	print $sock $msg || die "Failed to write to socket\n";
	print $sock $data || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_read_metadata_names {
	my ($sock, $oid, $domainid, $session_id) = @_;

	my $msg = $http_get_metadata_names;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/DOMAIN_ID/$domainid/;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Reading metadata names from OBJECT_ID=$oid\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_delete_metadata {
	my ($sock, $oid, $domainid, $data, $session_id) = @_;

	my $msg = $http_delete_metadata;
	$msg =~ s/OBJECT_ID/$oid/;
	$msg =~ s/DOMAIN_ID/$domainid/;
	$msg =~ s/SESSION_ID/$session_id/;

	my $len = length($data);
	$msg =~ s/CONTENT_LENGTH/$len/;

	dbglog "Deleting metadata from OBJECT_ID=$oid (len=$len)\n\n";
	dbglog "HTTP header       : \n$msg\n";
	dbglog "HTTP message body : \n$data\n";

	print $sock $msg || die "Failed to write to socket\n";
	print $sock $data || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_metadata_query {
	my ( $sock,
	     $domainid,
	     $queryname,
	     $query_req_rslt_count,
	     $data,
	     $session_id ) = @_;

	my $msg = $http_metadata_query;

	$msg =~ s/DOMAIN_ID/$domainid/;
	$msg =~ s/QUERY_NAME/$queryname/;
	$msg =~ s/QUERY_REQ_RSLT_COUNT/$query_req_rslt_count/;
	$msg =~ s/SESSION_ID/$session_id/;

	my $len = length($data);
	$msg =~ s/CONTENT_LENGTH/$len/;

	dbglog "Sending metadata query\n\n";
	dbglog "HTTP header       : \n$msg\n";
	dbglog "HTTP message body : \n$data\n";

	print $sock $msg || die "Failed to write to socket\n";
	print $sock $data || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_metadata_query_iteration {
	my ( $sock,
	     $domainid,
	     $queryname,
	     $query_req_rslt_count,
	     $query_cookie,
	     $data,
	     $session_id ) = @_;

	my $msg = $http_metadata_query_iteration;

	$msg =~ s/DOMAIN_ID/$domainid/;
	$msg =~ s/QUERY_NAME/$queryname/;
	$msg =~ s/QUERY_REQ_RSLT_COUNT/$query_req_rslt_count/;
	$msg =~ s/QUERY_COOKIE/$query_cookie/;
	$msg =~ s/SESSION_ID/$session_id/;

	my $len = length($data);
	$msg =~ s/CONTENT_LENGTH/$len/;

	dbglog "Sending metadata query iteration\n\n";
	dbglog "HTTP header       : \n$msg\n";
	dbglog "HTTP message body : \n$data\n";

	print $sock $msg || die "Failed to write to socket\n";
	print $sock $data || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

# Path related wrapper methods
sub send_create_path {
	my ($sock, $path, $session_id) = @_;

	my $msg = $http_create_path;
	$msg =~ s/OBJECT_PATH/$path/;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Creating path: $path\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_read_path {
	my ($sock, $path, $off, $len, $session_id) = @_;

	my $msg = $http_get_path_msg;
	$msg =~ s/OBJECT_PATH/$path/;
        #$msg =~ s/LOCATION_ID/$fh/;
	$msg =~ s/SESSION_ID/$session_id/;

	$msg =~ s/BYTE_OFFSET1/$off/;
	$off += $len;
	$msg =~ s/BYTE_OFFSET2/$off/;

	dbglog "Reading $len bytes from $path\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_write_path {
	my ($sock, $path, $fh, $off, $data, $session_id) = @_;

	my $msg = $http_put_path_msg;
	$msg =~ s/OBJECT_PATH/$path/;
	$msg =~ s/LOCATION_ID/$fh/;
	$msg =~ s/SESSION_ID/$session_id/;

	my $len = length($data);
	$msg =~ s/CONTENT_LENGTH/$len/;

	$msg =~ s/BYTE_OFFSET1/$off/;
	$off += $len;
	$msg =~ s/BYTE_OFFSET2/$off/;

	dbglog "Writing $len bytes to $path (total_len=", $len + length($msg),")\n\n";
	dbglog "HTTP header       : \n$msg\n";
	dbglog "HTTP message body : \n$data\n";

	print $sock $msg || die "Failed to write to socket\n";
	print $sock $data || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_dir_ls {
	my ($sock, $path, $session_id) = @_;

	my $msg = $http_get_dir_ls_msg;
	$msg =~ s/OBJECT_PATH/$path/;
	$msg =~ s/SESSION_ID/$session_id/;

	dbglog "Reading Dir names from $path\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}

sub send_get_objid {
	my ($sock, $path, $session_id) = @_;

	my $msg = $http_get_obj_id_msg;
	$msg =~ s/OBJECT_PATH/$path/;
	#$msg =~ s/LOCATION_ID/$fh/;
	$msg =~ s/SESSION_ID/$session_id/;

	$msg =~ s/BYTE_OFFSET1/0/;
	$msg =~ s/BYTE_OFFSET2/0/;

	dbglog "Reading object Id from $path\n\n";
	dbglog "HTTP header: \n$msg\n";

	print $sock $msg || die "Failed to write to socket\n";

	my ($hdr, $body) = get_response( $sock );

	return ($hdr, $body);
}
1;
