# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Basic Setup Library
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::BasicSetup

=head1 DESCRIPTION

Module to initialize basic setup configurations for the proper version.

=head1 METHODS

=cut

package CR::Setup::BasicSetup;

use strict;
use warnings;

use CR::Setup::BasicSetup::FullSteam;
use CR::Setup::BasicSetup::SierraNevada;


=head2 new

This factory method initializes the correct basic setup configurations for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $basic_setup;

    if ( $params{version} eq 'latest' ) {
        $basic_setup = CR::Setup::BasicSetup::SierraNevada->new(%params);
    }
    elsif ( $params{version} =~ m/^8\.3\./ ) {
        $basic_setup = CR::Setup::BasicSetup::FullSteam->new(%params);
    }
    elsif ( $params{version} =~ m/^8\.2\./ ) {
        $basic_setup = CR::Setup::BasicSetup::SierraNevada->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::BasicSetup - ' . $params{version}
        );
    }

    return $basic_setup;
}

1;
