# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary CIFSConfig
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::BasicSetup::CIFSConfig

=head1 DESCRIPTION

Module to initialize CIFS configurations for the proper version.

=head1 METHODS

=cut

package CR::Setup::BasicSetup::CIFSConfig;

use strict;
use warnings;

use CR::Setup::BasicSetup::FullSteam;
use CR::Setup::BasicSetup::SierraNevada;


=head2 new

This factory method initializes the correct CIFS configurations for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $cifs_config;

    my $version = delete $params{'version'};

    if ( $version eq 'latest' ) {
        $cifs_config = CR::Setup::BasicSetup::SierraNevada::CIFSConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.3\./ ) {
        $cifs_config = CR::Setup::BasicSetup::FullSteam::CIFSConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.2\./ ) {
        $cifs_config = CR::Setup::BasicSetup::SierraNevada::CIFSConfig->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::BasicSetup::CIFSConfig - ' . $version
        );
    }

    return $cifs_config;
}

1;
