# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Infinitevolconfig
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::BasicSetup::InfinitevolConfig

=head1 DESCRIPTION

Module to initialize infinitevol configurations for the proper version.

=head1 METHODS

=cut

package CR::Setup::BasicSetup::InfinitevolConfig;

use strict;
use warnings;

use CR::Setup::BasicSetup::FullSteam;
use CR::Setup::BasicSetup::SierraNevada;


=head2 new

This factory method initializes the correct infinitevol configurations for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $iv_config;

    my $version = delete $params{'version'};

    if ( $version eq 'latest' ) {
        $iv_config = CR::Setup::BasicSetup::SierraNevada::InfinitevolConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.3\./ ) {
        $iv_config = CR::Setup::BasicSetup::FullSteam::InfinitevolConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.2\./ ) {
        $iv_config = CR::Setup::BasicSetup::SierraNevada::InfinitevolConfig->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::BasicSetup::InfinitevolConfig - ' . $version
        );
    }

    return $iv_config;
}

1;
