# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary BasicSetup Library - LIFManager
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::BasicSetup::LIFManager

=cut

package CR::Setup::BasicSetup::LIFManager;

use strict;
use warnings;

use CR::Setup::BasicSetup::FullSteam;
use CR::Setup::BasicSetup::SierraNevada;

=head2 new

This factory method initializes the correct LIF Manager for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $lif_manager;

    my $version = delete $params{'version'};

    if ( $version eq 'latest' ) {
        $lif_manager = CR::Setup::BasicSetup::SierraNevada::LIFManager->new(%params);
    }
    elsif ( $version =~ m/^8\.3\./ ) {
        $lif_manager = CR::Setup::BasicSetup::FullSteam::LIFManager->new(%params);
    }
    elsif ( $version =~ m/^8\.2\./ ) {
        $lif_manager = CR::Setup::BasicSetup::SierraNevada::LIFManager->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::BasicSetup::LIFManager - ' . $version
        );
    }

    return $lif_manager;
}

1;
