# Copyright (c) 2013 Network Appliance, Inc.
# All rights reserved.

## @summary MultiCluster
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::BasicSetup::MultiCluster

=head1 DESCRIPTION

Module to initialize the MultiCluster module for the proper version.

=head1 METHODS

=cut

package CR::Setup::BasicSetup::MultiCluster;

use strict;
use warnings;

use CR::Setup::BasicSetup::FullSteam;
use CR::Setup::BasicSetup::SierraNevada;


=head2 new

This factory method initializes the correct multicluster module for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $multicluster;

    my $version = delete $params{'version'};

    if ( $version eq 'latest' ) {
        $multicluster = CR::Setup::BasicSetup::SierraNevada::MultiCluster->new(%params);
    }
    elsif ( $version =~ m/^8\.3\./ ) {
        $multicluster = CR::Setup::BasicSetup::FullSteam::MultiCluster->new(%params);
    }
    elsif ( $version =~ m/^8\.2\./ ) {
        $multicluster = CR::Setup::BasicSetup::SierraNevada::MultiCluster->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::BasicSetup::MultiCluster - ' . $version
        );
    }

    return $multicluster;
}

1;
