# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary SnapmirrorConfig
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::BasicSetup::SnapmirrorConfig

=head1 DESCRIPTION

Module to initialize snapmirror configurations for the proper version.

=head1 METHODS

=cut

package CR::Setup::BasicSetup::SnapmirrorConfig;

use strict;
use warnings;

use CR::Setup::BasicSetup::FullSteam;
use CR::Setup::BasicSetup::SierraNevada;


=head2 new

This factory method initializes the correct snapmirror configurations for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $sm_config;

    my $version = delete $params{'version'};

    if ( $version eq 'latest' ) {
        $sm_config = CR::Setup::BasicSetup::SierraNevada::SnapmirrorConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.3\./ ) {
        $sm_config = CR::Setup::BasicSetup::FullSteam::SnapmirrorConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.2\./ ) {
        $sm_config = CR::Setup::BasicSetup::SierraNevada::SnapmirrorConfig->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::BasicSetup::SnapmirrorConfig - ' . $version
        );
    }

    return $sm_config;
}

1;
