# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary StorageServiceConfig
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::BasicSetup::StorageServiceConfig

=head1 DESCRIPTION

Module to initialize storage service configurations for the proper version.

=head1 METHODS

=cut

package CR::Setup::BasicSetup::StorageServiceConfig;

use strict;
use warnings;

use CR::Setup::BasicSetup::FullSteam;
use CR::Setup::BasicSetup::SierraNevada;


=head2 new

This factory method initializes the correct storage service configurations for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $ss_config;

    my $version = delete $params{'version'};

    if ( $version eq 'latest' ) {
        $ss_config = CR::Setup::BasicSetup::SierraNevada::StorageServiceConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.3\./ ) {
        $ss_config = CR::Setup::BasicSetup::FullSteam::StorageServiceConfig->new(%params);
    }
    elsif ( $version =~ m/^8\.2\./ ) {
        $ss_config = CR::Setup::BasicSetup::SierraNevada::StorageServiceConfig->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::BasicSetup::StorageServiceConfig - ' . $version
        );
    }

    return $ss_config;
}

1;
