# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Setup Helper Library
## @author  Adam.Ciapponi@netapp.com
## @pod     here

=head1 NAME

CR::Setup::Helpers

=head1 DESCRIPTION

This module implements a factory that allows for initialization of the right helper object for the version.

=head1 METHODS

=cut

package CR::Setup::Helpers;

# Compiler directives.
use strict;
use warnings;

use CR::Setup::Exception;
use CR::Setup::Helpers::FullSteam;
use CR::Setup::Helpers::SierraNevada;
use Data::Dumper::Concise;
use Tharn qw($Log);

=head2 new

This factory method initializes the correct helper for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $version = delete $params{'version'};
    my $helper;

    if ($version eq 'latest') {
        $helper = CR::Setup::Helpers::SierraNevada->new(%params); # FullSteam isn't active yet, use SN
    } elsif ($version =~ m/^8\.3\./) {
        $helper = CR::Setup::Helpers::FullSteam->new(%params);
    } elsif ($version =~ m/^8\.2\./) {
        $helper = CR::Setup::Helpers::SierraNevada->new(%params);
    } else {
        CR::Setup::Exception->throw(
            'Invalid version for helpers: ' . $version
        );
    }

    return $helper;
}

1;
