# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Space Library
## @author  galane@netapp.com
## @pod     here

=head1 NAME

CR::Setup::Space

=head1 DESCRIPTION

Module to initialize space configurations for the proper version.

=head1 METHODS

=cut

package CR::Setup::Space;

use strict;
use warnings;

use CR::Setup::Exception;
use CR::Setup::Space::FullSteam;
use CR::Setup::Space::SierraNevada;


=head2 new

This factory method initializes the correct space configurations for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $space;

    my $version = delete $params{'version'};

    if ( $version eq 'latest' ) {
        $space = CR::Setup::Space::SierraNevada->new(%params);
    }
    elsif ( $version =~ m/^8\.3\./ ) {
        $space = CR::Setup::Space::FullSteam->new(%params);
    }
    elsif ( $version =~ m/^8\.2\./ ) {
        $space = CR::Setup::Space::SierraNevada->new(%params);
    }
    else {
        CR::Setup::Exception->throw(
            'Invalid version for CR::Setup::Space - ' . $version
        );
    }

    return $space;
}

1;
