# Override for TCD Timer
# Not in use but here because TCD isn't in test/main
package CR::Setup::TCDTimer;

#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rrollingrock/lib/TCD/Timer.pm#1 $

## @summary Timer library for basic time functions
##
## @author knopp@netapp.com
## @status Shared
## @pod here

package Timer;

use lib();

use POSIX qw(strftime);
use Time::HiRes qw(gettimeofday);
use Error;

# Compiler directives.
use strict;

use vars qw( @EXPORT @ISA);
@ISA = qw(Error);


###############################################################################
# Method:    new
# Objective: Create a new Timer object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class,$args) = @_;
    my (
		$self,
	);

    $self = {};
    bless $self;

    return $self;
}

###############################################################################
# Method:    start_timer
# Objective: start named timer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub timer_start{
    my ($self,$name) = @_;

    if (defined $name and length $name){
		$self->{"timer_$name"}{start}=gettimeofday;
		return;
    }
	else {
		$self->{"timer_start"}{start}=gettimeofday;
	}
}

###############################################################################
# Method:    timer_stop
# Objective: Stop named timer
# Details:   See POD documentation at the end of this file.
###############################################################################
sub timer_stop(){
    my ($self,$name) = @_;
    my (
		$errmsg,
		$time,
	);

    $self->_clear_err;

    if (defined $name and length $name){
		unless (defined $self->{"timer_$name"}{start}){
			$errmsg="Error stop_timer(): timer \'$name\' was never started\n";
			$self->_set_err($errmsg);
			return 0;
		}

		$time = gettimeofday - $self->{"timer_$name"}{start};

    }
	else {
		$time = gettimeofday - $self->{"timer_start"}{start};
	}


	return $time;

}


###############################################################
#  Function: timer_format()
#  Objective: Convert epoch seonds into time format (hr:mm:ss).
#  Input: $epoch - Epoch seconds to convert format
#
#  Return: string in format HH:MM:SS
################################################################
sub timer_format()
{
    my ($self,$epoch,$minimize) = @_;
    my ($milli,$seconds,$minutes,$hours,$time_str);

    ($epoch,$milli)=split(/\./,$epoch);

    $seconds    =  $epoch % 60;
    $epoch = ($epoch - $seconds) / 60;
    $minutes    =  $epoch % 60;
    $epoch = ($epoch - $minutes) / 60;
    $hours      =  $epoch % 24;

    $hours = "0".$hours if ($hours < 10);
    $minutes = "0".$minutes if ($minutes < 10);
    $seconds = "0".$seconds if ($seconds < 10);

	if ($minimize){
        if ($hours ne "00") {
			$hours =~ s/^0+//; # strip leading zeros
		    $time_str .= "$hours" . "h";
		}
        if ($minutes ne "00") {
			$minutes =~ s/^0+//; # strip leading zeros
		    $time_str .= "$minutes" . "m";
		}
        if ($seconds ne "00") {
			$seconds =~ s/^0+//; # strip leading zeros
		    $time_str .= "$seconds" . "s";
		}
	} else {
        $time_str .= "$hours:$minutes:$seconds.$milli";
	}

    return "$time_str";
}

1;

=pod

=head1 Package Name

Timer.pm

=head1 Author

Gary Knopp

=head1 Description

Timer provide an API to basic timer functions

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

B<Error.pm>

=head2 B<new()>

This method creates a new timer object.

=over 4

=item Inputs

none

=item Outputs

Returns a Timer object.

=item Examples

    use Timer;

    $timerObj=Timer->new();

=back

=head2 B<timer_format($seconds,$minimize)>

This method returns a number as a formatted time value 'HH:MM:SS'.
If the optional $minimize argument is sent and positive, we ruturn
the time formatted like this: HHh:MMm:SSs without leading zeros

=over 4

=item Inputs

=item * $seconds

Number of seconds to format.

=item * I<$minimize>

Optional - use a format that eliminates leading zeros (3h22m11.1s)

=item outputs

Returns a format time value.  If an error occurred,
the error flag is set, errmsg is set to the failure reason, and 0 is passed back
as the return value.

=item Examples

 # run time
 $duration = $TCase->timer_stop("myTimer");
 print "Time running:".$TCase->$timer_format($duration)." seconds\n";

=back

=head2 B<timer_start($name)>

This method sets time zero for a named timer.  The timer used is a high resolution
timer with millisecond accuracy.

=over 4

=item Inputs

=item * $name

Unique name to identify timer

=item outputs

If an error occurred, the error flag is set, errmsg is set to the failure reason, and 0 is passed back
as the return value.

=item Examples

 # init_time
 $myObj->timer_start("myTimer");
 if ($TCase->error){
     print $TCase->errmsg;
     return 0;
 }

=back

=head2 B<timer_stop($name)>

This method returns the time duration since the timer was started.  The timer used is a high resolution
timer with millisecond accuracy.

=over 4

=item Inputs

=item * $name

Unique name to identify timer

=item outputs

Returns the time in seconds that has expired since timer_start($name).  If an error occurred,
the error flag is set, errmsg is set to the failure reason, and 0 is passed back
as the return value.

=item Examples

 # run time
 my $duration = $myObj->timer_stop("myTimer");
 print "Time running: $duration seconds\n";

=back

=cut
