# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Unbless
## @author  Adam.Ciapponi@netapp.com
## @pod     here

=head1 NAME

CR::Setup::Unbless

=head1 DESCRIPTION

This module implements a function that does the opposite of bless

=head1 METHODS

=cut

package CR::Setup::Unbless;

# Compiler directives.
use strict;
use warnings;

use CR::Setup::Constants;
use Data::Dumper::Concise;
use Params::Validate qw(:all);
use Scalar::Util qw(blessed);

use base qw(Exporter);

our @EXPORT = qw(
    unbless_this
);

=head2 unbless_this

Unblesses an object blessed by CR::Setup::Expectation::this.

=cut

sub unbless_this {
    validate_pos(@_, $TRUE);
    my ( $ref ) = @_;

    if ( blessed $ref ) {
        if ( ref( $ref ) eq 'SCALAR' ) {
            my $t = ${$ref};
            return \$t;
        }
        elsif ( ref( $ref ) eq 'ARRAY' ) {
            my @t = @{$ref};
            return [@t];
        }
        elsif ( ref( $ref ) eq 'HASH' ) {
            my %t = %{$ref};
            return {%t};
        }
        elsif ( ref( $ref ) eq 'CODE' ) {
            # unblesses return value from this_block
            return unbless_this(&{$ref});
        } else {
            # leave actual object alone
            return $ref;
        }
    } else {
        # leave scalar/reference alone
        return $ref;
    }
}
1;
