# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Setup Validation Library
## @author  Adam.Ciapponi@netapp.com
## @pod     here

=head1 NAME

CR::Setup::Validation

=head1 DESCRIPTION

This module implements a factory that allows for initialization of the right helper object for the version.

=head1 METHODS

=cut

package CR::Setup::Validation;

# Compiler directives.
use strict;
use warnings;

use CR::Setup::Exception;
use CR::Setup::Validation::FullSteam;
use CR::Setup::Validation::SierraNevada;
use Data::Dumper::Concise;
use Tharn qw($Log);

=head2 new

This factory method initializes the correct validation library for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $version = $params{'version'};
    my $validation;

    if ($version eq 'latest') {
        $validation = CR::Setup::Validation::SierraNevada->new(%params); # FullSteam isn't active yet, use SN
    } elsif ($version =~ m/^8\.3\./) {
        $validation = CR::Setup::Validation::FullSteam->new(%params);
    } elsif ($version =~ m/^8\.2\./) {
        $validation = CR::Setup::Validation::SierraNevada->new(%params);
    } else {
        CR::Setup::Exception->throw(
            'Invalid version for validation: ' . $version
        );
    }

    return $validation;
}

1;
