# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary InfinteVol balanced creation / expansion library

=head1 NAME

CR::Setup::Validation::Balanced

=head1 DESCRIPTION

This module implements a factory that allows for initialization of the right helper object for the version.

=head1 METHODS

=cut

package CR::Setup::Validation::Balanced;

# Compiler directives.
use strict;
use warnings;

use CR::Setup::Validation::Balanced::SierraNevada;
use CR::Setup::Validation::Balanced::FullSteam;
use Data::Dumper::Concise;
use Tharn qw($Log);

=head2 new

This factory method initializes the correct balanced library for the version specified.

=cut

sub new {
    my $class = shift;
    my %params = @_;

    my $version = delete $params{'version'};
    my $balanced;

    if ($version eq 'latest') {
        $balanced = CR::Setup::Validation::Balanced::FullSteam->new(%params);
    } elsif ($version =~ m/^8\.3\./) {
        $balanced = CR::Setup::Validation::Balanced::FullSteam->new(%params);
    } elsif ($version =~ m/^8\.2\./) {
        $balanced = CR::Setup::Validation::Balanced::SierraNevada->new(%params);
    } else {
        CR::Setup::Exception->throw(
            'Invalid version for Balanced: ' . $version
        );
    }

    return $balanced;
}

1;
