# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.

## @summary Setup Validation Library - 8.2 SierraNevada Release
## @author  Adam.Ciapponi@netapp.com
## @pod     here

=head1 NAME

CR::Setup::Validation::SierraNevada

=head1 DESCRIPTION

This module contains methods for setup validation

=head1 METHODS

=cut

package CR::Setup::Validation::SierraNevada;

# Compiler directives.
use strict;
use warnings;

# Module import.
use CR::Setup::Constants;
use CR::Setup::Constants::SierraNevada;
use CR::Setup::Expectation;
use CR::Setup::Validation::Balanced;
use CR::UnitTest;    #FIXME remove this
use Data::Dumper::Concise;
use Math::Round qw(:all);
use NATE::BaseException qw(:try);
use NACL::APISet;
use NACL::C::Node;
use NACL::C::Snapmirror;
use NACL::C::Volume;
use NACL::C::Vserver;
use Storable qw(dclone);
use Test::Deep::NoTest;
use Tharn qw($Log);

use base qw(CR::Setup::Validation::Base);

=head2 balanced_create_and_validate

This function does balanced creation and verifies the result returning a boolean.

=cut

sub balanced_create_and_validate {
    my $self       = shift;
    my %parameters = @_;

    # Node setup
    my $node    = $parameters{'node'};
    my $vserver = $parameters{'vserver'};

    my $vs_state = NACL::CS::Vserver->fetch(
        command_interface => $node,
        filter            => { vserver => $vserver->vserver }
    );

    my $balanced = CR::Setup::Validation::Balanced->new(
        version   => $self->{'version'},
        node      => $node,
        defaults  => $self->{'defaults'},
        helpers   => $self->{'helpers'},
        operation => 'create',
    );

    $balanced->get_initial_state( vserver => $vs_state );

    $balanced->balanced_create(
        'node'                => $node,
        'cli'                 => $parameters{'cli'},
        'zapi'                => $parameters{'zapi'},
        'vserver'             => $parameters{'vserver'},
        'create_args'         => $parameters{'create_args'},
        'create_to_max'       => $parameters{'create_to_max'},
        'use_zapi_for_create' => $parameters{'use_zapi_for_create'},
        'max_factor'          => $parameters{'max_factor'}
    );

    $balanced->get_final_state( vserver => $vs_state );

    return $balanced->validate_infinitevol();
}

sub balanced_expand_and_validate {
    my $self       = shift;
    my %parameters = @_;

    # Node setup
    my $node    = $parameters{'node'};
    my $vserver = $parameters{'vserver'};

    my $vs_state = NACL::CS::Vserver->fetch(
        command_interface => $node,
        filter            => { vserver => $vserver->vserver }
    );

    my $balanced = CR::Setup::Validation::Balanced->new(
        version   => $self->{'version'},
        node      => $node,
        defaults  => $self->{'defaults'},
        helpers   => $self->{'helpers'},
        operation => 'expand',
    );

    $balanced->get_initial_state( vserver => $vs_state );

    $balanced->balanced_expand(
        'node'                => $node,
        'cli'                 => $parameters{'cli'},
        'zapi'                => $parameters{'zapi'},
        'vserver'             => $parameters{'vserver'},
        'expand_args'         => $parameters{'expand_args'},
        'expand_to_max'       => $parameters{'expand_to_max'},
        'use_zapi_for_expand' => $parameters{'use_zapi_for_expand'},
        'validation_args'     => $parameters{'validation_args'},
        'expand_command'      => $parameters{'expand_command'},
    );

    $balanced->get_final_state( vserver => $vs_state );

    return $balanced->validate_infinitevol();
}

1;
