#
# $Id: //depot/prod/test/Rsierranevada/lib/CR/Volume.pm#1 $
#
# Copyright (c) 2011 NetApp Inc.
# All rights reserved.
#
## @summary Volume Class
## @author davidnew@netapp.com
## @status shared
## @pod here

package CR::Volume;

use strict;
use warnings;

use Data::Dumper;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet;
use Params::Validate qw(:all);
use SPARSE::Util::Logger;
use Tharn qw($Log);

=head1 NAME

Volume - The class to encapsulate methods and data of "volume" commands set

=head1 SYNOPSIS

=begin text

 use Volume;

  $cr->volume_show_constituents(vserver            => $vserver,
                                'constituent-role' => $role);

  $cr->volume_show(vserver => $vserver,
                  'volume-roles' => $roles);
                                 
 =end text

=head1 AUTHOR

David New (davidnew@netapp.com)

=cut

logLevel('info');

##################################################################
# Volume Show Constituents
##################################################################

=head2 volume_show_constituents

Return number of constituent volumes

Required parameters are

=over 4

=item *
vserver

Name of the vserver.

=item *
volume-role

Volume Role

=back

=cut

sub volume_show_constituents {

  my ($self, %args) = @_;
  my $admin_cli = $self->{ngsh};
  my @parsed_output;
  
  %args = NACL::APISet::validate_params($self,
                                        params => \%args,
                                        spec => {
                                          vserver            => { type => SCALAR, optional => 0 },
                                          'constituent-role' => { type => SCALAR, optional => 0 },
                                        },
                                       );

  logInfo("Calling volume show -constituents");

  @parsed_output = @{$admin_cli->volume_show(%args)->get_parsed_output()};;
  logInfo("CLI 'volume show -constituents' executed successfully.\n");
  return @parsed_output;
}


##################################################################
# Volume Show
##################################################################

=head2 volume_show

Purpose of the subroutine is to display the volume details.

Required parameters are

=over 4

=item *
vserver

Name of the infinivol vserver

=item *
volume

name of the volume

=back

=cut

sub volume_show{
  logStep("Calling volume show");

  my ($self,%args)=@_;
  my $admin_cli = $self->{ngsh};
  my @parsed_output;

  %args = NACL::APISet::validate_params ($self,
                                         params => \%args,
                                         spec   => {
                                           vserver => {type => SCALAR, optional => 1},
                                           volume  => {type => SCALAR, optional => 1},
                                         },
                                        );
  if($args{volume}){
    @parsed_output=@{$admin_cli->volume_show(%args,
                                             field=>"size,used-afs")->get_parsed_output()};
    logInfo("Detail of volume on vserver $self->{vserver}:\n");
    return @parsed_output;
  } else{
    @parsed_output=@{$admin_cli->volume_show(vserver=>$args{vserver})->get_parsed_output()};
    logInfo("Detail of volumes on vserver $self->{vserver}:\n");
    return @parsed_output;
  }
}
