# -*- perl -*-
# $Id: //depot/prod/test/main/lib/nfs/CommonParams.pm#4 $

package CR::nfs::CommonParams;

## @summary
##  Common parameters used in NFS tests.
##
## @description
##  This sets up correct environment variables. Note that the
##  variables are coming out of main's name space.
##
## @usage
##  use CR::nfs::CommonParams;
##
## @status      review
##
## @author      dl-nfsqa-only (NFS QA Sunnyvale)
## @notes       Created by EP Network Storage Performance Lab, Inc.
## @notes
##  Put the 'use CR::nfs::CommonParams;' right after the BEGIN
##  block with the thpl script param statements to make sure
##  the the variables below are defined before other library
##  modules are used.

use strict;

#
# The following package declaration is required for these variables
# to be declared global in the main name space. It was done this
# way because 'use vars' doesn't support package names in the
# variable name, ex: $main::variable. Neither does the 'our'
# statement.
package main;

# Please keep the list in alphabetic order
use vars qw(
	    $AGGR_SIZE
            $CIFS_DOMAIN
            $CIFS_DOMAIN_ADMIN
            $CIFS_DOMAIN_ADMIN_PASSWD
	    $CIFS_DRIVE
	    $CIFS_PASSWD
	    $CIFS_SHARE
	    $CIFS_USER
	    $CLEANUP
	    $CLIENT
	    $CMDTIMEOUT
	    $EXTERN_SETUP
	    $FILER_MODE
	    $FILER
	    $FILERA
	    $FILERB
	    $FILER_AGGR
	    $FILER_CONN
	    $FILER_IP
	    $FILER_QTREE
	    $FILER_VOL
	    $FILER_VOL_MNT
	    $MOUNTPOINT
	    $NBLADE_START
	    $NBLADE_HOST
	    $NBLADE_DIR
            $NFS_V4_DELEGATIONS
            $NFS_V4_ID_DOMAIN
            $NIS_DOMAIN
            $NIS_DOMAIN_SERVER
	    $ONTAPDIR
	    $PFILER
	    $TIMEOUT
	    $VOL_SIZE
	    $TRAD_SIZE
	    $FLEX_SIZE
	    $VOL_TYPE
	    $RAID_TYPE
	    $MOUNT_VERS
	    $CONTAINER
	    $SETUP_DONE
	    $WORM_VOL
	    $WORM_AGGR
	    $WORM_RETENTION
	    $WAFLIRON_TIMEOUT
	    $SKIP_SNAPLOCK_RETENTION_SETUP
	    $SLCC_STRICT
        $ADDR_TYPE
	$VOL_GX_TYPE
	$FILER_GX_VOL
	$FILER_GX_AGGR
	$AGGR_GX_TYPE
	$VSERVER_GX_NAME
	$EXPORT_GX_POL
	   );

# Change the name space back
package CR::nfs::CommonParams;

use Tharn;
# param sets up the variables in main's name space
param('AGGR_SIZE',      '-default', 5);
param('CIFS_DOMAIN',    '-default', 'cifsqa.lab.netapp.com');
param('CIFS_DOMAIN_ADMIN', '-default', "$::CIFS_DOMAIN\\administrator");
param('CIFS_DOMAIN_ADMIN_PASSWD', '-default', 'Hello123');
param('CIFS_DRIVE',     '-default', '*');
param('CIFS_PASSWD',    '-default', 'cifspass');
param('CIFS_SHARE',     '-default', 'testshare');
param('CIFS_USER',      '-default', 'administrator');
param('CLEANUP',        '-default', 'test');
param('CLIENT',         '-default', 'localhost');
param('CMDTIMEOUT',     '-default', 120); # 2 mins
param('SETUP_DONE',     '-default', 'reuse'); #this will replace EXTERN_SETUP in the future
param('EXTERN_SETUP',   '-default', $::SETUP_DONE);
param('FILER_AGGR',     '-default', 'testaggr');
param('FILER_CONN',     '-default', '#rsh-socket');
param('FILER_IP',       '-default', undef);
param('FILER_QTREE',    '-default', 'filerqtree');
param('FILER_VOL_MNT',  '-default', undef);
param('MOUNTPOINT',     '-default', undef);
param('NBLADE_START',   '-default', undef());
param('NBLADE_HOST',    '-default', undef());
param('NBLADE_DIR',     '-default', undef());
param('NFS_V4_DELEGATIONS', '-default', 'on');
param('NFS_V4_ID_DOMAIN', '-default', 'lab.netapp.com');
param('NIS_DOMAIN',     '-default', 'lab.netapp.com');
param('NIS_DOMAIN_SERVER', '-default', '172.19.3.11');
param('ONTAPDIR',       '-default', '/rlse/mainN');
param('PFILER',         '-default', '');
param('TIMEOUT',        '-default', 600); # 10 mins
param('MOUNT_VERS',     '-default', undef); #could be 4
param('FLEX_SIZE',      '-default', '30m');
param('TRAD_SIZE',      '-default', 3);
param('WORM_RETENTION', '-default', '30s');
param('WORM_AGGR',      '-default', 'wormaggr');
param('WAFLIRON_TIMEOUT', '-default', 2400);
param('SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 0);
param('SLCC_STRICT',    '-default', 1);
param('FILER_MODE',     '-default', 'mono');
param('RAID_TYPE',      '-default', undef);
param('ADDR_TYPE',      '-default', "inet4");

use config_base;
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
my $wv;
if ($::FILER_MODE eq 'mono') {
    param('FILER',      '-mesg', '  hostspec of filer');
} else {
    param('FILERA',     '-mesg', '  FILERA for cluster');
    param('FILERB',     '-mesg', '  FILERB for cluster');
    param('FILER',      '-set',  $::FILERA);
}

my $fh = host($::FILER);
if ($fh->hosttype() !~ /filer-ng/) {
eval {
    $wv = get_wafl_version($fh);
};
if ($@ and ! defined($ENV{"VOL_TYPE"}) and ! defined($::VOL_TYPE)) {
    logresult('CONFIG', "filer/sim is down or unreachable -- define \$VOL_TYPE to continue");
} elsif ($wv > WV_SCRIMSHAW_MAX) {
    param('VOL_TYPE',   '-default', 'flex');
} else {
    param('VOL_TYPE',   '-default', 'trad');
    param('SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 1);
}

if ($::VOL_TYPE eq 'flex') {
    param('VOL_SIZE',   '-default', $::FLEX_SIZE);
    param('FILER_VOL',  '-default', 'testvol_flex');
    param('CONTAINER',  '-default', $::FILER_AGGR);
    param('WORM_VOL',   '-default', 'testvol_worm_flex');
} else {
    param('VOL_SIZE',   '-default', $::TRAD_SIZE);
    param('FILER_VOL',  '-default', 'testvol_trad');
    param('CONTAINER',  '-default', $::FILER_VOL);
    param('WORM_VOL',   '-default', 'testvol_worm_trad');
}
}

if ($fh->hosttype() =~ /filer-ng/) {


	param('FILER_GX_VOL',	   '-default', 'testvol_flex');
    param('FILER_GX_VOL_NAME', '-default', 'testvol_flex');
	param('FILER_GX_AGGR',	   '-default', 'testaggr');
	param('AGGR_GX_TYPE',	   '-default', 'flex');
	param('VSERVER_GX_NAME',   '-default', 'testvserver');
	param('EXPORT_GX_POL',	   '-default', 'test_pol');

}
1;

