# This PERL module has the code in it that is needed to do CRC checking of
# strings of data.  This code is just about as op for op that I could get as
# the code in libsystest/Utils.cc  So CRC's returned from this code should
# be exactly the same as CRC's returned from C++ programs that use that CRC
# routine.  Will we ever need that functionality...who knows.  But it was
# easy enough to do it this way so I did.

package CRC;

use strict;
use Carp;

local @crc::crc_table;

# This routine takes no paramters.  It fills the @crc_table array with the
# correct data for the CRC generation later.  The arrary containing the
# table data is a global variable that will be available to the code you
# write should you for some reason want to get at it.  The table needs only
# to be generated once, after its generated you won't need to rerun InitCRC
# again.
sub InitCRC{
    my $crc_accum=0;
    my $crc_polynomial=0x04c11db7;
    for(my $i=0;$i<256;$i++){
        $crc_accum=int($i<<24);
        for(my $j=0;$j<8;$j++){
            if($crc_accum & 0x80000000){
                $crc_accum=($crc_accum<<1)^$crc_polynomial;
            }else{
                $crc_accum=($crc_accum<<1);
            }
        }
        $crc::crc_table[$i]=$crc_accum;
    }
    if($#crc::crc_table!=255){ # we should have an array of 255 elements
        croak "Problem generating the CRC Table.\n";
    }
    return 1;
}

# This routine takes the data passed to it and breaks it into characters
# that it then computes a CRC with.  If the CRC table hasn't been populated
# yet, it populates the table before continuing on.  Therefore there is no
# need to call InitCRC from your code at all.  Once the CRC is found, the
# CRC is returned unless the CRC is 0.  In that case we return a different
# predetermined number that isn't 0.
sub UpdateCRC{
    my $data=shift;
    my $crc_accum=undef;
    my @data=split //, $data;
    my $nextdata=0;    
    
    InitCRC() unless @crc::crc_table;
    
    foreach my $j (@data){
        my $i=(int($crc_accum>>24)^(ord($data[$nextdata++])))&0xff;
        $crc_accum=($crc_accum>>8)^$crc::crc_table[$i];
    }
    
    # because 0 is a bad number to test against, never return 0
    if($crc_accum==0){
        return 774662537;
    }else{
        return $crc_accum;
    }
}

1; # all good packages return true
