# $Id: //depot/prod/test/Rsierranevada/lib/CSetup/Snapshot.pm#1 $

## @summary The base Snapshot class,it checks the hosttype and creates 
## the corresponding object of Aggregate or VI or None class 
## @author dl-setupcleanup-libs
## @status Shared
## @pod here


package CSetup::Snapshot;

use strict;
use warnings;

use Tharn qw(script_result);
use Params::Validate qw(validate SCALAR);
BEGIN
{
    Params::Validate::validation_options(
                                         on_fail => sub {
                                             my @msgs = @_; chomp(@msgs);
                                             die script_result([@msgs]);
                                         },
                                        );
}

require Exporter;
our @ISA=qw(Exporter);
our @EXPORT_OK=qw(new);

sub new {
    my $class = shift;
    my (%args) = validate(@_, {
                               host => { isa=>'Hostrec' },
                               type => { type => SCALAR, optional => 1 },
                              });
    my $type = $args{type} || _guess_type($args{host});
    require "CSetup/Snapshot/$type.pm";
    return "CSetup::Snapshot::$type"->new(host => $args{host});
}

#
# local routine which guesses the class of the object to be snapshot
#
sub _guess_type {
    my ($host) = @_;
    if ($host->{VM_HOST}) {
        return "VI";
    }
    if ($host->hosttype =~ /filer/ && $host->hosttype !~ /filer-ng/) {
        return "Aggregate";
    }
    return "None";
}


=head1 NAME

Snapshot - The base Snapshot class, which checks the hosttype and 
creates the corresponding object of Aggregate or VI or None class

=head1 SYNOPSIS

    use CSetup::Snapshot;

    my $host = host($FILER);
    my $snap_object = SnapShot->new(host => $host);

=head1 DESCRIPTION

The calling script, will call the constructor of the Snapshot class 
and passing the host, the Snapshot class will check the hostype 
in the tharnhost file and creates the corresponding
snapshot class object

In case of VMware clients ,the constructor will return an object 
of VI class.  In case of Filer in 7m and 7g, Simulator in 7m and 
Vsims in 7g,the constructor will return an object of Aggregate 
class. For other hosttypes, it returns an object of None class.

For Example:

   use CSetup::Snapshot;
   my $snap_object = SnapShot->new(host => $host);

The constructor of the class, can also take type as an 
optional parameter, in this case it will return an object 
of the classname being passed.

For Example:

   use CSetup::Snapshot;
   my $snap_object = SnapShot->new(host => $host,
				   type => Aggregate);

=head2 new

   my $snap_object = SnapShot->new(host => tada-u1);

=over

=item host

(Required) : Hostspec object.

=item type

(Optional) : The name of the class for which object to be created.
It can take any one of the following options VI, Aggregate and None 

=back

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = none

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    dl-setupcleanup-libs@netapp.com
    $Id: //depot/prod/test/Rsierranevada/lib/CSetup/Snapshot.pm#1 $ 
    Last revised $Date: 2013/01/22 $ 

=head1 COPYRIGHT

    Copyright 2009, Network Appliance Inc., All Rights Reserved.

=cut

1;


