# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/CSetup/Snapshot/None.pm#1 $

## @summary Library of snapshot methods for 10 mode filers and non-vmware clients.
## @author bkavita@netapp.com, dl-setupcleanup-libs
## @status Shared
## @pod here

package CSetup::Snapshot::None;

use strict;
use warnings;
use Tharn qw($Log script_result);
use Params::Validate qw(validate SCALAR);
BEGIN
{
    Params::Validate::validation_options(
                 on_fail => sub {
                                            my @msgs = @_; 
                                            chomp(@msgs);
                                            die script_result([@msgs]);
                 }, );
}

require Exporter;
our @ISA=qw(Exporter);
our @EXPORT_OK=qw(is_available revert_to create delete);

=head1 NAME 

CSetup::Snapshot::None - Snapshot methods for 10 mode filers and non-vmware clients. 

=head1 SYNOPSIS

use CSetup::Snapshot;

my $host = host($filer);

my $obj = CSetup::Snapshot->new(host=>$host);

$obj->is_available(state=>$state);

$obj->create(state=>$state);

$obj->revert_to(state=>$state);

$obj->delete(state=>$state);

=head1 DESCRIPTION

 This library has snapshot methods for 10 mode filers, non-vmware clients, 
 or where we otherwise have no better snapshot facility, or a quick way to make
 sure existing snapshot facility is never used. 

=cut

sub new {
     my ($system) = shift @_;
     my (%attrs) = validate (@_, 
                                       {'host' => { isa => 'Hostrec' }, } );
     my $class = ref($system) || $system;

     my $self = {
                     host => $attrs{host},
     };
     bless($self,$class);
     return $self;
}

=head1 METHODS

=head2 is_available

 This method takes as input the name of the state.

 Since snapshot facility is not supported it will return false.

=cut

sub is_available {
    return 0; 
}

=head2 revert_to

 This method takes as input the name of the state.

 Exit displaying an error. This routine should not have been called as
 snapshot facility is not supported.

=cut

sub revert_to {
     die("Snapshot facility is not supported.");
}

=head2 create

 This method takes as input the name of the state.

 Since snapshot facility is not supported it will not  do anything. 
 Just an empty routine.

=cut

sub create {
     
}

=head2 delete

 This method takes as input the name of the state.

 Since snapshot facility is not supported it will not do anything.

=cut

sub delete {
     
}

=head1 FOR_TESTING
 
 Use Example/testCSetupSnapshotNone.thpl for testing this lib.

=head1 STATUS

 Shared

=cut

1;

=head1 AUTHOR

  bkavita@netapp.com, dl-setupcleanup-libs
  $Id: //depot/prod/test/Rsierranevada/lib/CSetup/Snapshot/None.pm#1 $
  Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

  Copyright 2009, Network Appliance Inc., All Rights Reserved.

=cut
