# -*- perl -*- 
#$Id: //depot/prod/test/main/lib/Class.pm#1 $ 

## @summary Provides some high level features that every Class can use
##
## @owner GX Automation Framework
##
## @author Michael Edmonds (michael.edmonds@netapp.com)
## @author GX Automation Framework (dl-qxautomation@netapp.com)
##
## @change 2007-03-22   medmonds:   Initially created
## @change 2007-04-11   medmonds:   Fixing Up Documentation
## @change 2007-04-11   medmonds:   Fixing Up Documentation ... again
##
#
# Copyright (c) 2007 Network Appliance, Inc., All Rights Reserved
# Permission is granted to use this script on a Network Appliance Product.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from Network Appliance, Inc.
#
##
## @status public
## @pod here

package Class;

use strict;
use Storable;
use Data::Dumper;

use Options;
use TracePkg;

use Err;
our @ISA = qw(Err);

our $Trace;
our $Options;

BEGIN
{
    $Trace = TracePkg->new();
    $Options = Options->new();
}

sub process_options
{
    $Trace->exit();
    my $self = shift;
    $Options->process();
    $Trace->exit();
}
# end process_options()

sub get_options
{
    $Trace->exit();
    my ($self, $opts) = @_;
    if(!(defined($opts)))
    {
        $Trace->exit();
        return $Options->get_hash();
    }
    else
    {
        if(ref($opts) ne "ARRAY")
        {
            $Trace->exit();
            return $Options->get(option => $opts);
        }
        else
        {
            my @opts = $Options->get(option => $opts);
            $Trace->exit();
            return @opts; 
        }
    }
    $Trace->exit();
}
# end get_options()

sub serialize
{
    $Trace->entry();
    my $self = shift;
    if (!(int(@_))){push(@_, $self);}
    $Trace->debug(msg=>"blah = " . Dumper(@_));
    $Trace->exit();
    return Storable::freeze(@_);
}
#end serialize()

sub deserialize
{
    $Trace->entry();
    my $self = shift;
    if (!(int(@_))){push(@_, $self);}
    $Trace->exit();
    return Storable::thaw(@_);
}
#end deserialize()

1;

#############################################################################
#                                   POD
#############################################################################

=pod

=head1 Package Name

Class.pm

=head1 Author

Michael Edmonds

=head1 Owner

GX Automation Framework dl-gxautomation@netapp.com

=head1 Description

Class is intended to be the highest level functionality for all Classes.

=head1 See Also

Options.pm, Storable.pm
Package inherits from the following modules. See these modules for further 
interface detail.

Err.pm

=head1 Synopsis

Here are some examples. 

=over 4

use Class;
our @ISA = qw(Class);

=head1 Public Methods

=head2 C<process_options()>

=over 4

=item Description

Wrapper around Options->Process()

=item Inputs
None

=item Outputs

None

=back

=head2 C<get_options(<$PARAM | \@PARAMS>)>

=over 4

=item Description

Wrapper around Options->gethash() and Options->get()

=item Inputs

void, single option, array of options

=item Outputs

Hash, Array, Scalar

=item Example

    See Options.pm

=back

=head2 C<serialize($something)>

=over 4

=item Description

Wrapper around Storable::freeze()

=item Input

If Void is passes, it serializes $self

=item Outputs

Serialized thing

=item Examples

    See Storable

=back

=head2 C<deserialize($something)>

=over 4 

=item Description

Wrapper around Storable::thaw()

=item Input

Serialized thing

=item Output

deserialized thing

=item Examples

    See Storable

=back

=cut