#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/Client.pm#1 $

## @summary Provide an interface to a HOST machine
## @author lippe@netapp.com
## @status private
## @pod here

package Client;

# Compiler directives.
use strict;

# Module import.
use File::Basename;
use File::Compare;
use Time::HiRes qw(gettimeofday);
use San::SanLog;
use San::SanClient::Lvm;
use San::SanClient::DevMgr;
use San::SanClient::Sanlun;
use San::SanClient::ClientUtil;
use San::SanClient::Initiator;
use Utils qw(make_host make_conn);
use Error;
use Exporter;

use vars qw(	  
	    $_CLIENT_CNT
  	    @EXPORT 
  	    @ISA
  	    );

@ISA = qw(Error SanLog);

###############################################################################
# Public Methods
###############################################################################

###############################################################################
# Method:    new
# Objective: Create new Host object.
# Details:   See POD documentation at the end of this file.
###############################################################################

sub new {
    my ($proto,%args) = @_;
    my $class = ref($proto) || $proto;
    my $self = {}; 
    my $subr;
    my $sanlun_obj={};
    my (
       $execute,
       $host,
       $log,
       $passwd,
       $protocol,
       $recreate_stale,
       $rc,
       $show_cmd,
       $timeout,
       $user,	   
       @protocols,
       %hArgs,
        );

    # init variables
    bless $self,$class; 
    $protocol="nfs";
    $recreate_stale="true";


    # Process arguments.  If an error occurs during the arg processing,
    # The error flag and errmsg values are set and the object is returned.
    # It is up to the caller to check for errors after the new call.
    foreach (keys %args) {
	
	if (/-?host$/) {
	    length($host = $args{$_}) or
		do {
		    $self->_set_err("Client::new invalid value for \"host\":",
				    "$args{$_}");
		    return $self;
		};
	}
	elsif (/^-?user$/) {
	    length($user = $args{$_}) or
		do {
		    $self->_set_err("Client::new invalid value for \"user\":",
				    "$args{$_}");
		    return $self;
		};
	}
	elsif (/^-?passwd$/) {
	    length($passwd = $args{$_}) or
		do {
		    $self->_set_err("Client::new invalid value for \"passwd\":",
				    "$args{$_}");
		    return $self;
		};
	}
	elsif (/-?timeout$/) {
	    length($timeout = $args{$_}) or
		do {
		    $self->_set_err("Client::new invalid value for \"timeout\":",
				    "$args{$_}");
		    return $self;
		};
	}
	elsif (/-?protocol$/) {
	    # Default to nfs if exist and not user defined 
	    # (Note: Undefined get_parameter returns 0)
	    $protocol=$args{$_};
	    if (exists $args{$_} and $args{$_}=~/0/){
		$protocol="nfs";
	    }
	    else {
		unless (exists $args{$_} and $args{$_} =~ /^nfs$|^fcp$|^iscsi$/i){
		    $self->_set_err("Client::new invalid value for \"protocol\":".$args{$_});
		    return $self;

		}
	    }
	}
	elsif (/-?recreate_stale$/) {
	    # Default to true if exist and not user defined 
	    # (Note: Undefined get_parameter returns 0)
	    $recreate_stale=$args{$_};
	    if (exists $args{$_} and $args{$_}=~/0/){
		$recreate_stale='false';
	    }
	    else {
		unless (exists $args{$_} and $args{$_} =~ /^true$|^false$/i){
		    $self->_set_err("Client::new invalid value for \"recreate_stale\":".$args{$_});
		    return $self;

		}
	    }
	}
	elsif (/-?show_cmd$/) {
	    length($show_cmd = $args{$_}) or
		do {
		    $self->_set_err("Client::new invalid value for \"show_cmd\"",
				    "$args{$_}");
		    return $self;
		};
	}
	elsif (/-?loghdl$/) {
	    length($log = $args{$_}) or
		do {
		    $self->_set_err("Client::new invalid value for",
				    "\"log\" $args{$_}");
		    return $self;
		};
	}
	elsif (/-?_obsolete|-?parent/){
	    next;

	}
	else {
	    $self->_set_err("Client::new: Invalid parameter specified: $_");
	    return $self;
	}
    }

    # Initially unconnected.
    $self->{connected} = 0;

    # Show command mode.  By default when printing, the command
    # issued will be displayed.  This can be turned off by the user.
    $self->{show_cmd} = defined $show_cmd ? $show_cmd : 1;

    $self->{show_data}=1;
    $self->{protocol}=$protocol;
    
    # Allow CLI commands to be formed, but not executed.
    $self->{execute} = defined $execute ? $execute : 1;


    # Allow CLI commands to be logged;
    $self->{log}=$log if (defined $log);

    # Allow CLI commands to be logged;
    $self->timeout($timeout) if (defined $timeout);
    

    # Verify mandatory parameters.
    unless (defined $host and length $host) {
	$self->_set_err("Client::new mandatory parameter \"host\"",
			"not specified");
	return $self;
    }

    $self->{tn}=$self->_connect($host,$user,$passwd,$timeout);

    if ($self->error) {
	# Error message is already set, return object so caller
	# can get the error.
	return $self;
    }

    # Get OS Version
    if ($self->hosttype() =~ /aix/){
    	my $ver=$self->execute("uname -v", -hide=>1,-_hostcmd=>1)->[0];
    	my $rel=$self->execute("uname -r", -hide=>1,-_hostcmd=>1)->[0];
    	$self->{tn}{hostp}{version}="$ver.$rel";
    }
    else {
	$self->{tn}{hostp}{version} = $self->execute("uname -r", -hide=>1,-_hostcmd=>1)->[0];
    }

    return $self if ($self->error);
    
    # Store connection parameters in object.

    $self->{host} =  $host;
    $self->{user} =  $user;
    $self->{pass} =  $passwd;
    

    # determine the right platform and require/import/call the appropriate
    # OS object's new call.
    my $hosttype=$self->hosttype();  

    if ($hosttype =~ /unix-solaris/i) {
        require Client::Host::SolarisHost;
        push @ISA,("Client::Host::SolarisHost");
        $self = Client::Host::SolarisHost->_new(-conn=>$self->get_connection(),-self=>$self);
    }
    elsif ($hosttype =~ /unix-hpux/i) {
        require San::SanClient::Host::HPUXHost;
        push @ISA,("San::SanClient::Host::HPUXHost");
        $self = San::SanClient::Host::HPUXHost->_new(-conn=>$self->get_connection(),-self=>$self);
    } 
    elsif ($hosttype =~ /unix-aix/i) {
        require San::SanClient::Host::AixHost;
        push @ISA,("San::SanClient::Host::AixHost");
        $self = San::SanClient::Host::AixHost->_new(-conn=>$self->get_connection(),-self=>$self);
     } 
    elsif ($hosttype =~ /unix-linux/i) {
        require Client::Host::LinuxHost;
        push @ISA,("Client::Host::LinuxHost");
        $self = Client::Host::LinuxHost->_new(-conn=>$self->get_connection(),-self=>$self);
    } 
    else {
        $self->_set_err("Error: Client::new() Host platform not",
                        "currently supported by Client object");
        return $self;     
    }


    # Check if client supports user specified protocol.
    # Retrieve list of supported protocols and compare 
    # to what was  passed in
    @protocols=$self->get_supported_protocols();
    $protocol=$self->get_protocol();

    unless ("@protocols" =~ /$protocol/i){
        $self->_set_err("Error: Client::new() $protocol protocol on ".$self->hosttype(),
                        "is currently not supported by Client object");
        return $self;
    }

    if ($protocol=~/fcp|iscsi/){
 
	$hArgs{-loghdl}=$self->{log} if (defined $self->{log});
	# Create a Sanlun object with the specified protocol
	$sanlun_obj = Sanlun->new(-conn=>$self->get_connection(),-protocol=>$protocol,%hArgs);
	if ($sanlun_obj->error){
	    $self->_set_err($sanlun_obj->errmsg());
	    return $self;                                    
	}

	$self->{sanlun}=$sanlun_obj;

	# Load initiators
	$self->_load_initiators($self->get_protocol());
	return $self if ($self->error);
    }

    # Initialize mount array.  Any found will be pushed here
    @{$self->{mounts}} = ();

    # Re-create stale mount entries from previously failed tests
    if($recreate_stale eq 'true') {
       $self->_load_nfs_mnt;
    }

    return $self;
} 
################################################################################
# Subroutine Name: destroy ()
# Objective:       destroy client object
# Details:         See POD documentation at the end of this file
################################################################################

sub destroy {
    my ($self)=@_;
    if (defined $self->{connect}){
	$self->sanlog(-msg => "Disconnecting Client object '".$self->get_hostname()."'");
	Tharn::disconnect($self->{connect});
    }
}

###############################################################################
# Method:    create_mnt
# Objective: Create and pass back a mntobj handle
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_mnt {
    my($self, %args) = @_;
    my ($found,
	$mntprop,
	$mntObj,
	$path);
    
    $self->_clear_err();

    my $hosttype=$self->hosttype();  

    if ($hosttype =~ /unix-solaris/i) {
        require Client::Mount::SolarisMount;
        $mntObj = Client::Mount::SolarisMount->_new(-parent=>$self, %args);
	return $mntObj if ($mntObj->error);
    }
    elsif ($hosttype =~ /unix-hpux/i) {
        require Client::Mount::HPUXMount;
        $mntObj = Client::Mount::HPUXMount->_new(-parent=>$self, %args);
	return $mntObj if ($mntObj->error);
    } 
    elsif ($hosttype =~ /unix-aix/i) {
        require Client::Mount::AixMount;
        $mntObj = Client::Mount::AixMount->_new(-parent=>$self, %args);
	return $mntObj if ($mntObj->error);
    } 
    elsif ($hosttype =~ /unix-linux/i) {
        require Client::Mount::LinuxMount;
        $mntObj = Client::Mount::LinuxMount->_new(-parent=>$self, %args);
	return $mntObj if ($mntObj->error);
    } 
    else {
        $self->_set_err("Error: Client::create_mnt() Host platform not currently supported by Client object");
        return $self;     
    }

    # Keep pointer to object unless already mounted
    foreach (@{$self->{mounts}}) {
	$mntprop = $mntObj->get_properties;
	$found = 1 if ($mntprop->{path} eq $_->{path});
    }
    push @{$self->{mounts}}, $mntObj unless $found;

    return $mntObj;
}

###############################################################################
# Method:    unmount
# Objective: Unmounts the filesystem and destroys mntObj
# Details:   See POD documentation at the end of this file.
###############################################################################
sub unmount {
    my($self, $mntObj) = @_;
    my (@error);
    
    $self->_clear_err();

    if ($mntObj) {
	# Unmount individual filesystem
	$mntObj->destroy;
	if ($mntObj->error){
	    $self->_set_err($mntObj->errmsg());
	    return $self;                                    
	}
    } else {
	# Unmount all known mounted filesystems
	foreach (@{$self->{mounts}}) {
	    $_->destroy;
	    if ($_->error){
		push @error, $_->errmsg();             
	    }
	}
	$self->_set_err(@error) if (@error);  
    }

    return $self;
}

###############################################################################
# Method:    create_lvm()
# Objective: Create and pass back a lvm handle
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_lvm{
    my($self,%args) = @_;
    my (
        $lvm,
        $lvmObj,
        $loadcfg,
	%hArgs,
        );
    

    $self->_clear_err();

    $loadcfg=1;

    # parse arguments
    foreach (keys %args) {
        if (/-?lvm/) {
            if (not length($lvm = $args{$_})) {
                $self->_set_err("Error: Client::create_lvm()",
                                "Invalid value for argument '-lvm'");
                return $self;
            }        
            next;
        }
        elsif (/-?loadcfg/) {
            if (not length($loadcfg = $args{$_}) or $loadcfg !~ /no|yes|1|0|off|on/i) {
                $loadcfg=1;
            }
            next;
        }
        else {
            $self->_set_err("Error: Client::create_lvm()",
                            "Invalid parameter specified \'$_\'");
            return $self;                            
        }
    }  
    
    # check for existance of required -lvm argument
    unless (defined $lvm and length $lvm) {
        $self->_set_err("Error: Client::create_lvm()",
                        "Missing required argument '-lvm'");
        return $self; 
    }

    $hArgs{-loghdl}=$self->{log} if (defined $self->{log});
    $lvmObj = Lvm->_new(-parent=>$self,-lvm=>$lvm,,-loadcfg=>$loadcfg,%hArgs);
    if ($lvmObj->error){
        $self->_set_err($lvmObj->errmsg());
        return $self;                                    
    }

    # Keep pointer to first snapdrive object created by client
    $self->{obj}{lvm}=$lvmObj if (not defined $self->{obj}{lvm}); # Keep list of child objects created

    return $lvmObj;
}

###############################################################################
# Method:    create_devmgr()
# Objective: Create and pass back a devmgr handle
# Details:   See POD documentation at the end of this file.
###############################################################################
sub create_devmgr{
    my($self) = @_;
    my (
        $dev,
        $dmObj,
	%hArgs,
        );
    

    $self->_clear_err();

    $hArgs{-loghdl}=$self->{log} if (defined $self->{log});
    $dmObj = DevMgr->_new(-parent=>$self,%hArgs);
    if ($dmObj->error){
        $self->_set_err($dmObj->errmsg());
        return $self;                                    
    }

    # Keep pointer to first snapdrive object created by client
    $self->{obj}{devmgr}=$dmObj if (not defined $self->{obj}{devmgr});

    # Keep list of child objects created

    return $dmObj;
}

###############################################################################
# Method:    execute
# Objective: Issue arbitrary command to host and get results.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute {
    my($self, $cmd, %args) = @_;
    my(
       @arr,
       $buffer,
       $hide,
       @rows,
       $cmd_fmt,
       $hostcmd,
       $errmsg,
       $t0,
       $t1,
       );

    $hide=0;


    foreach (keys %args){
        if (/-?hide/) {
            length($hide = $args{$_}) or
                do {
                    $self->_set_err("Error: Client::execute() invalid value for \"hide\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        elsif (/-?_hostcmd/) {
            length($hostcmd = $args{$_}) or
                do {
                    $self->_set_err("Error: Client::execute() invalid value for \"-_hostcmd\":",
                                    "$args{$_}");
                    return $self;
                };
        }
        else {
            $self->_set_err("Error: Client::execute()",
                            "Invalid parameter specified \'$_\'");
            return $self;                            
        }
    }    

    # Clear errors.
    $self->_clear_err;

    unless (length $cmd) {
        $self->_set_err("Error: Client::execute() invalid CLI command");
        return [];
    }

    # Print the command unless the user 
    # has specified otherwise.  
    
    #$cmd_fmt="CLIENT(".$self->hostname().")> ".$cmd."\n";

    if (!$hide and $self->{show_cmd}){   
        $self->sanlog(-msg => $cmd, -use_loghdl => 0); 
    }

    return (wantarray()) ? () : [] unless ($self->{execute});

    # Init execution timer
    $t0=gettimeofday;
    my ($status, @output) = &ClientUtil::execute($self->{tn},$cmd);
    $t1=gettimeofday;
    $self->{command_time}=$t1-$t0;

    # Check for standard set of errors returned from the CLI.
    if ($status) {
        $self->_set_err("Error: ",@output);
        return (wantarray()) ? @output : \@output;
    }

    return (wantarray()) ? @output : \@output;

} # end sub cmd

################################################################################
# Subroutine Name: get_initiators
# Objective:       return array of initiator objects
# Details:         See POD documentation at the end of this file
################################################################################
sub get_initiators {
    my ($self) = @_;
    my (
        @initObjs,
        );
    
    $self->_clear_err();
    if (defined $self->{initiators}) {
        @initObjs = @{$self->{initiators}};
    } 
    else {
        $self->_set_err("Error: Client::get_initiators() Client object has no initiators");
        return ();              
    }

    return (wantarray()) ? @initObjs : \@initObjs;
    
} 

################################################################################
# Subroutine Name: get_mounts
# Objective:       return array of mount objects
# Details:         See POD documentation at the end of this file
################################################################################
sub get_mounts {
    my ($self, %args) = @_;
    my (@mntObjs,
	$mnt_prop,
	$path);
    
    $self->_clear_err();

    foreach (keys %args){
        if (/-?path/) {
            length($path = $args{$_}) or
                do {
                    $self->_set_err("Error: Client::get_mounts() invalid value for \"path\":",
                                    "$args{$_}");
                    return $self;
                };
        } else {
            $self->_set_err("Error: Client::get_mounts()",
                            "Invalid parameter specified \'$_\'");
            return $self;                            
        }
    }

    if (defined $path) {
	foreach (@{$self->{mounts}}) {
	    $mnt_prop = $_->get_properties;
	    if ($mnt_prop->{path} =~ /$path/) {
		return $_;
	    }
	}
	$self->_set_err("Error: Client::get_mounts(): No mounts defined");
	return $self;  
    }

    if (defined $self->{mounts}) {
        @mntObjs = @{$self->{mounts}};
    } else {
        return ();              
    }

    return (wantarray()) ? @mntObjs : \@mntObjs;
    
} 

###############################################################################
# Method:    get_sanlun()
# Objective: Get sanlun object handle
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_sanlun{
    my($self,%args) = @_;

    $self->_clear_err();

    if (not defined $self->{sanlun}) {
        $self->_set_err("Error: Client::get_sanlun()",
                        "Sanlun object was not created.  Client protocol must be set to",
	                "either 'fcp' or 'iscsi'"
	                 );
        return {}; 
    }

    return $self->{sanlun};
  
}

###############################################################################
# Method:    get_connection()
# Objective: Get nate connection object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_connection{
    my($self) = @_;
    return $self->{tn};
}

###############################################################################
# Method:    get_exec_time()
# Objective: Get execution time of snap command statement
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_exec_time{
    my($self) = @_;

    if (length $self->{command_time}){ 
	return sprintf "%.4f",$self->{command_time};
    }

    return ""; 
}

################################################################################
# Subroutine Name: get_protocol
# Objective:       Return protocol type
# Details:         See POD documentation at the end of this file
################################################################################
sub get_protocol {
    my ($self) = @_;

    $self->_clear_err();
    
    if (defined $self->{protocol}){
	return $self->{protocol};
    }
    else {
	$self->_set_err("Error: Client::get_protocol() Client object is missing protocol type");
	return "";
    }
}

###############################################################################
# Method:    hostname()
# Objective: Get client host name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub hostname{
    my($self) = @_;
    return $self->{tn}{hostp}{hostname};
}

###############################################################################
# Method:    hosttype()
# Objective: Get client host type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub hosttype{
    my($self) = @_;
    return $self->{tn}{hostp}{hosttype};
}

###############################################################################
# Method:    os_version()
# Objective: Get execution time of snap command statement
# Details:   See POD documentation at the end of this file.
###############################################################################
sub os_version{
    my($self) = @_;
    return $self->{tn}{hostp}{version};
}

###############################################################################
# Method:    reconnect()
# Objective: reconect to host
# Details:   See POD documentation at the end of this file.
###############################################################################
sub reconnect{
    my ($self)=@_;

    $self->sanlog(-msg => "Reconnecting to host machine '".$self->hostname()."'");
    $self->{tn}=$self->_connect($self->{host},$self->{user},$self->{pass},$self->{timeout});

    return $ClientUtil::FALSE if ($self->error);

    $self->{conn}=$self->{tn};

    # create a new sanlun object otherwise sanlun will have a stale connection
    my $protocol=$self->get_protocol();

    if ($protocol=~/fcp|iscsi/){
        my %hArgs;

        $hArgs{-loghdl}=$self->{log} if (defined $self->{log});
        my $sanlun_obj = Sanlun->new(-conn=>$self->get_connection(),-protocol=>$protocol,%hArgs);
        if ($sanlun_obj->error){
            $self->_set_err("Client::reconnect() ".$sanlun_obj->errmsg());
            return $self;
        }
        $self->{sanlun}=$sanlun_obj;
    }

    return $ClientUtil::TRUE;
}
###############################################################################
# Method:    timeout
# Objective: set or get telnet timeout
# Details:   See POD documentation at the end of this file.
###############################################################################
sub timeout {
    my ($self,$timeout) = @_;

    if (defined $timeout and length $timeout){
	$self->{timeout}=$timeout;
    } 
    else {
	return $self->{timeout};
	
    }
}

###############################################################################
# Private Methods
###############################################################################

###############################################################################
# Method:    _connect
# Objective: Create telnet connection to host
# Details:   This routine handles making the actual Tharn connection. If
#			   the user/passwd are not specified in the call to the client 
#			   new, the the values are read out of the tharnhosts file. 
###############################################################################
sub _connect {
    my($self,$hostname,$user,$passwd,$timeout) = @_;
    my(
       $connect,
       $conn_str,
       $errmsg,
       $host,
       $id,
       );

    # Variable init.
    $self->{connected} = 0;

    $host = Utils::make_host($hostname); 

    unless (defined($timeout) and length($timeout)){
        $self->timeout($host->{connection_fields}{ssh}{timeout});
    }
    
    $id=$Client::_CLIENT_CNT++;

    unless (defined($user))   { $user   = $host->{connection_fields}{ssh}{username}; }
    unless (defined($passwd)) { $passwd = $host->{connection_fields}{ssh}{password}; }

    $conn_str=$hostname."#Client_".$id.";conntype=ssh-persist;".
	"username=".$user.";".
	    "password=".$passwd.";".
		"timeout=".$self->{timeout}.";"; 


    $connect = Tharn::connect(
	$conn_str,
	  -eval=>sub{
	      if (@_) {
		  my $errmsg = join("",@_);
		  $errmsg =~ s/\n//g;
		  $self->_set_err($errmsg);

		  # Display an error in case user doesn't include error 
		  # handler on object instantiation.  
		  $self->sanlog(-msg => "ERROR: Client::_connect() unable to connect to client '".$hostname."'");

		  Tharn::logresult("CONFIG",abort=>0,"\n@_");	

		  return $self;
	      }
	  });

    return $connect;

} # end sub _connect

################################################################################
# Subroutine Name: _get_objs
# Objective:       return array of children objects
# Details:         See POD documentation at the end of this file
################################################################################
sub _get_objs {
    my ($self,%args) = @_;
    my (
        @objs,
        );
    
    $self->_clear_err();

    if (defined $self->{objs}) {
        @objs = @{$self->{objs}};
    } 
    else {
        $self->_set_err("Error: Client::get_objs() No children objects to be logged");
        return  (wantarray()) ? () : [];                
    }

    return (wantarray()) ? @objs : \@objs;
    
} 
###############################################################################
# Method:    _load_initiators()
# Objective: load initiators 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _load_initiators {
    my ($self,$protocol)=@_;            
    my (
        $cmd,
        $key,
        $name,
        $value,
        %hsh,
        @output,
        @slots,
        );


    # retrieve all initiator info from sanlun
    @output=$self->{sanlun}->display_adapters($protocol);
    if ($self->{sanlun}->error){
        $self->_set_err($self->{sanlun}->errmsg);
        return $self;
    }

    # Select specific fields from the output for each 
    # initiator and store in hash
    if ($protocol=~/fcp/i) {
        foreach my $line (@output){
            
            next unless (  
                           $line=~/WWPN\:|^WWNN\:|^model\:|model description\:/ or 
                           $line=~/^driver version\:|^firmware version\:/i
                           );

            if ($line=~/WWPN/i){
                ($name,$key,$value)=$line=~/(\S+)\s+(\w+)\:(\S+)/;
                $hsh{$name}{name}=$name;
            }
            else {
                ($key,$value)=$line=~/(.*):\s+(.*)/i;
            }

            $hsh{$name}{$key}=$value;     
        }
    }
    elsif ($protocol=~/iscsi/i) {
        foreach my $line (@output){
            
            # Remove goofy control chars in output
            $line=~s/\cI\cI/ /g;

            next unless (  
                           $line=~/initiator name\:|model\:|model description\:/ or
                           $line=~/driver version\:|WWPN\:/i
                           );

            if ($line=~/initiator name/i){
                ($name,$value)=$line=~/(\S+)\s+initiator name:\s+(\S+)/;
                $key="WWNN";  # change the key name
                $hsh{$name}{name}=$name;
            }
            else {
                ($key,$value)=$line=~/(.*):\s+(.*)/i;
            }

            $hsh{$name}{$key}=$value;     
        }

        # Make NA the remainder attributes
        # $hsh{$name}{"WWPN"}="NA";
        $hsh{$name}{"firmware version"}="NA";

    }
    
    # Walk hash and create an initiator object for 
    # each initiator  
    @slots=(sort keys %hsh);
    foreach my $slot (@slots){

        $hsh{$slot}{-loghdl}=$self->{log} if (defined $self->{log});
        $hsh{$slot}{-parent}=$self;
        $hsh{$slot}{-protocol}=$protocol;
        # Create initiator object
        my $initiatorObj = Initiator->_new($hsh{$slot});
        if ($initiatorObj->error){
            $self->_set_err($initiatorObj->errmsg());
            return $ClientUtil::FALSE;                                                                  
        }

        push @{$self->{initiators}},$initiatorObj;
        
    }
    
    
    return $ClientUtil::TRUE;   
}

################################################################################
# Subroutine Name: is_reboot()
# Objective:       reboot host
# Details:         See POD documentation at the end of this file
################################################################################
sub is_reboot {
    my ($self) = @_;
    $self->_clear_err();
    
    
    if (defined	$self->{reboot} and $self->{reboot}=~/1|0/){
	return $self->{reboot};
    }
    else {
	$self->{reboot}=0;
    }

    return $self->{reboot};
}

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

Client.pm

=head1 Author

Gary Knopp

=head1 Description

Client.pm provides an API interface to a host machine through the 
use of the Telnet library. 

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<San::SanClient::Host::[I<SolarisHost|HPUXHost|AixHost]> or B<Client::Host::[I<LinuxHost>]>>

B<Error.pm>

=head1 Attributes

=head2 B<connected>

boolean = <1 | 0>

=head2 B<show_cmd>

boolean = <1 | 0> (default: 1 (true) By default log commands executed)

=head2 B<show_data>

boolean = <1 | 0>

=head2 B<protocol>

string = <fcp | iscsi> (default: fcp)

=head2 B<execute>

boolean = <1 | 0> (default: 1)

=head2 B<log>

file handle = File handle used for logging

=head2 B<tn>

connection object = Connection to the host

=head2 B<conn>

connection object = Connection to the host

=head2 B<host>

string = Host name, connection parameter

=head2 B<user>

string = Username, connection parameter

=head2 B<pass>

string = Password, connection parameter

=head2 B<sanlun>

sanlun object = Host sanlun object

=head2 B<obj>

hash = Hash of children objects (devmgr, lvm)

=head2 B<command_time>

integer = time in seconds of the last command executed

=head2 B<initiators>

array = List of initiator objects

=head2 B<timeout>

integer = timeout to use for say calls

=head2 B<no_indent>

boolean = <1 | undef> Do not indent log entries when set

=head1 Public Methods

=head2 B<new(%args)>

Creates a new Client object.  Uses the specified IP address or machine name 
to create a telnet connection. If the connection is successful, the calling 
program will be logged into the CLIENT using either the default user/password
specified in the NATE_HOSTS file or a user and password supplied by the caller.

=over 4

=item Inputs

=item * -host 

Machine name or IP address of the remote machine.

=item * I<-user> (default: Tharnhost username entry).

Override default username for host login.

=item * I<-passwd> (default: Tharnhost password entry).

Override default password for host login.

=item * I<-timeout> (default: Tharnhost telnet timeout entry).

Telnet timeout in seconds

=item * I<-loghdl> 

Filehandle to log output too.

Override the Telnet timeout value.

=item * I<-show_cmd> <1 | 0> (default: 1 (true) ).

By default commands executed will be logged   This can be
turned off by passing the -show_cmd with a value of 0.

=item Outputs

Returns a Client object.  If a successful Telnet connection and login was
achieved, an open Telnet connection is maintained by the object until
the calling program exits, or the B<disconnect> method is explicitly
called.

=item Examples

 # Create a Client object with a connection to a remote machine. 

 $Client = new Client(
        -host => '192.168.x.y',
        -user => 'smith',
        -passwd  => 'mypassword');

  if ($Client->error) {
     $Test->nlog($Client->errmsg());    
     return $TCase::FAIL;     
 }

=back

=head2 B<create_devmgr(%args)>

Creates a new device manager object responsible for keeping track of raw devices
used by a client.

=over 4

=item Inputs

None

=item Outputs

Returns a DevMgr object.  If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

 # Create a DevMgr object

  $DevMgr = $Client->create_devmgr();     
  if ($Client->error) {
     $Test->nlog($Client->errmsg());    
     return $TCase::FAIL;     
 }

=back

=head2 B<create_mnt(%args)>

Creates a new mount object responsible for keeping track of mounts
used by a client.

=over 4

=item Inputs

=item * I<-share>

The filer volume to mount on the host.  Typically of the form "<filer>:/vol/<vol_name>".

=item * I<-path>

Host mount point.

=item * I<-options>

Pass through options to host specific mount command.

=item Outputs

Returns a mount object.  If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

 # Create a mount object

    $mntObj = $Client->create_mnt(-share=>$share);
    if ($mntObj->error) {
        $Test->nlog($mntObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<create_lvm(%args)>

Creates a new Lvm object

=over 4

=item Inputs

=item * -lvm

=item Outputs

Returns a Lvm object.  If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

 # Create a Lvm object

  $Lvm = $Client->create_lvm(
        -lvm=>'vxvm'
        );
  if ($Client->error) {
     $Test->nlog($Client->errmsg());    
     return $TCase::FAIL;     
 }

=back

=head2 B<execute($cmd,I<%args>)>

Executes command

=over 4

=item Inputs

=item * $cmd

Command to execute on the host.

=item * I<$hide> default: 0 

Hide the command from STDOUT

=item Outputs

Returns an array or an array reference to the data returned by executing the command. 
If an error occurred, the error flag is set, errmsg is set to the failure reason, and
an empty array reference is passed back as the return value.      

=item Examples

 # Display command output from host
 $buf=$Client->execute("ls -l");
 if ($Client->error) {
     $Test->nlog($Client->errmsg());    
     return $TCase::FAIL;     
 }


 foreach $line (@$buf){
     print $line."\n";
 }

=back

=head2 B<get_connection()>

This method returns the nate conenction object

=over 4

=item Outputs

Returns a connection object. If an error occured the error flag 
is set, errmsg is set to the failure reason

=item Examples

 #Retrieve execution time
 $connObj = $obj->get_connection();

=back

=head2 B<get_exec_time()>

This method returns the execution time in seconds of the last snap command statement executed.  Its the 
actual time it took to process the statement and does not include any post processing of the data. The
timer used is a high resolution timer with millisecond accuracy. 

=over 4

=item Outputs

Returns a scalar value representing the execution time in seconds.

=item Examples

 # Retrieve execution time

 $time = $obj->get_exec_time();
 print "Execution Time = $time seconds\n";

=back

=head2 B<get_initiators()>

Returns an array of initiator objects

=over 4

=item Inputs

none

=item Outputs

Returns an array or array reference of Initiator objects.  If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

 # Create a Lvm object
 @InitObjs = $Client=>get_initiators();
 if ($Client->error) {
     $Test->nlog($Client->errmsg);    
     return 0;     
 }

=back

=head2 B<get_protocol()>

Retrieve protocol used by client

=over 4

=item Inputs

none

=item Outputs

Returns a scalar containing the protocol. If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

 # Get protocol
 $protocol = $Client=>get_protocol();
 
 $Test->nlog("Protocol: ".$protocol);    

=back

=head2 B<get_sanlun()>

Retrieve Sanlun object handle

=over 4

=item Inputs

none

=item Outputs

Returns a Sanlun object.  If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

 # Create a Lvm object

 $obj = $Client=>get_sanlun();

 if ($obj->error) {
     print $obj->errmsg;    
     return 0;     
 }

=back

=head2 B<hostname()>

This method allows you to retrieve the hostname of the client machine 

=over 4

=item Outputs

Returns a scalar value containing the hostname.  If an error occured the error flag 
is set, errmsg is set to the failure reason, and 0\'s are passed back as the return values.

=item Examples

 # Get build_dt
 $hostname= $Obj->hostname();
 if ($Obj->error){
     ...
 }

 print "The hostname is $hostname.\n";

=back

=head2 B<hosttype()>

This method allows you to retrieve the hosttype of the client machine 

=over 4

=item Outputs

Returns a scalar value containing the hosttype.  If an error occured the error flag 
is set, errmsg is set to the failure reason, and 0\'s are passed back as the return values.

=item Examples

 # Get hosttype
 $hosttype= $Obj->hosttype();
 if ($Obj->error){
     ...
 }

 print "The hosttype is $hosttype.\n";

=back

=head2 B<os_version()>

This method allows you to retrieve the os_version  of the client machine 

=over 4

=item Outputs

Returns a scalar value containing the os_version.  If an error occured the error flag 
is set, errmsg is set to the failure reason, and 0\'s are passed back as the return values.

=item Examples

 # Get os version
 $os_ver= $Obj->os_version();
 if ($Obj->error){
     ...
 }

 print "The version is $os_ver.\n";

=back

=head2 B<timeout($sec)>

This method allows you to change the Telnet timeout.  The default value
for timeout is the value specified in the Tharnhost file.

=over 4

=item Inputs

=item * $sec

Specify number of seconds for timeout value.

=item Outputs

If successful, the timeout in the Telnet object is set to the specified
value and the previously set value is returned.  If an error occurred,
the error flag is set, errmsg is set to the failure reason returned
from the Telnet module and 0 is passed back as the return value.

=item Examples

 # Set Telnet timeout value to 45 seconds.
 $prev = $obj->timeout(45);
 <check for errors>

=back

=head2 B<unmount()>

Unmounts a filesystem and destroys the mount object.

=over 4

=item Examples

 # Unmount filesystem
 $Client->unmount($mntObj);

=back

=head2 B<destroy()>

Destroy client object (disconnect from client).

=over 4

=item Examples

 # Get os version
 $Obj->destroy();

=back

=head2 B<reconnect()>

This method allows you to reconnect to the client machine 

=over 4

=item Outputs

Returns 1 on success. On failure, 0 will be set and the objects error flag will be set.

=item Examples

 # Reconnect to filer
 $Obj->reconnect();
 if ($Obj->error){
     ...
 }

=back

=cut

1;




