# -*- perl -*- 
#$Id: //depot/prod/test/main/lib/ClientInfo.pm#1 $ 

## @summary Provides a functions for returning Client Information from the ENV
##
## @owner GX Automation Framework
##
## @author Michael Edmonds (michael.edmonds@netapp.com)
## @author GX Automation Framework (dl-qxautomation@netapp.com)
##
## @change 2007-03-29   medmonds:   Initially created
## @change 2007-04-11   medmonds:   Fixing Up Documentation
## @change 2007-04-11   medmonds:   Fixing Up Documentation ... again
##
#
# Copyright (c) 2007 Network Appliance, Inc., All Rights Reserved
# Permission is granted to use this script on a Network Appliance Product.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from Network Appliance, Inc.
#
##
## @status public
## @pod here
package ClientInfo;
use strict;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT_OK = qw();

sub os
{
    return ($ENV{OSTYPE} | $ENV{OS});
}

sub arch
{
    return ($ENV{ARCH} | $ENV{PROCESSOR_ARCHITECTURE});
}

sub hostname
{
    return ($ENV{HOSTNAME} | $ENV{COMPUTERNAME});
}

1;

=head1 NAME

ClientInfo.pm

=head1 Author

Michael Edmonds

=head1 Owner

GX Automation Framework dl-gxautomation@netapp.com

=head1 DESCRIPTION

C<ClientInfo> Provides functions to retrieve Client Information from the ENV.

=head1 SYNOPSIS

=head1 Public Methods

=head2 C<os()>

=over 4

=item Description

retrieves the OS from the environment.  

=item Inputs

None

=item Outputs

String
Returns ($ENV{OSTYPE} | $ENV{OS}).  

=item Examples

    my $os = ClientInfo::os();
    if($os =~ qr/windows/i)
    {
        ...
    }
    elsif($os =~ qr/linux/i)
    {
        if(ClientInfo::arch() =~ qr/_64/i)
        {
            ...
        }
        ...
    }

=back

=head2 C<arch()>

=over 4

=item Description

retrieves the achitecture from the environment.  

=item Inputs

None

=item Outputs

String
Returns ($ENV{ARCH} | $ENV{PROCESSOR_ARCHITECTURE}).  

=item Examples

    my $os   = ClientInfo::os();
    my $arch = ClientInfo::arch();
    if($os =~ qr/windows/i)
    {
        ...
    }
    elsif($os =~ qr/linux/i)
    {
        if($arch =~ qr/_64/i)
        {
            ...
        }
        ...
    }

=back

=head2 C<hostname()>

=over 4

=item Description

retrieves the hostname (FQDN actually) from the environment.  

=item Inputs

None

=item Outputs

String
Returns ($ENV{HOSTNAME} | $ENV{COMPUTERNAME}).  

=item Examples

    my $host = ClientInfo::hostname();

=back

=cut

