# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/ClientRPC.pm#1 $

## @summary     Base class providing ClientRPC::Interface helper methods
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here

###############################################
##   NOT READY FOR PUBLIC CONSUMPTION
###############################################
package ClientRPC;

use strict;
use Carp qw(croak);
use Error;
use Exporter;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use Module::Common qw(validate_mixed);
use Utils qw(make_host);
use Tharn qw(subtest $Log);
use Data::Dumper;

use constant UID_TAKEN=>1;

use vars qw(@ISA $_CLIENT_CNT %UID);
@ISA = qw(Error);

sub _new {
    my ($class,%args) = @_;
    my (
      $code,  
      $client,
      $errmsg,
      $self,
      $tharnhost,
      $uid,   
    );
    
    $self = bless {}, $class;
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    ### Validate input arguments
    eval {
        ($class,%args) = validate_mixed (
            @_,
            [ { type => SCALAR} ],
            {
                client   => {type     => SCALAR,
                             callbacks=>{
                              'Valid Tharnhost File'=>sub{
                              $tharnhost=Utils::make_host($args{client});            
                            }}},
                on_err  =>  {type => SCALAR, optional=>1, default=>"die",
                             callbacks=>{
                            'legal values'=>sub{shift()=~/die|rtn/i
                            }}},  
                uid      => {type     => SCALAR, optional=>1,
                             callbacks=>{
                              'Unique Client Id'=>sub{
                                not exists $UID{$args{uid}}             
                            }}},
            }
        );
        
        $self->{on_err}=$args{on_err};
        
        ### Validate that hosttype is supported by library
        unless ($tharnhost->{hosttype}=~/linux|win/){
            croak "Client $client\'s hosttype ".$tharnhost->{hosttype}." ".
                  "is currently not supported";
        }
        $self->{hosttype}=$tharnhost->{hosttype};
        
        
        ### Create a uid if one wasn't passed in. Mark it as used
        ### to prevent a new instance from using the same value.
        $uid=$args{uid};
        if (not defined $uid) {
            while (1) {
                $uid="rpc".++$ClientRPC::_CLIENT_CNT;
                last if (not exists $UID{$uid});    
            }
        }
        $UID{$uid}=UID_TAKEN;
     
        $Log->comment("STARTING UP SERVER on client ".$args{client}); 
        ### Start up the rpc server on the client in the background
        $self->{rpc} = Tharn::subtest(
            '{use NATE::Events qw(eventloop_drain); while(1){eventloop_drain();}}',
            "-bg", 
            "-runid"  => $args{client} ."_".$uid,
            "-client" => $args{client}
        );
      
    };
    if ($@){
       $Log->comment("ERROR OCCURRED!!!!!!". $@); 
       if (my $msg=$self->_input_validation($@)) {
            $errmsg=$msg;
       }
       else {
            $errmsg=$@;
       }
       delete $UID{$uid};
       
      ($self->on_err()=~/die/)
        ? die $errmsg
        : $self->_set_err($errmsg);
       
       return $self;
    }
    
    ### Initialize object metadata
    $self->{hostname}=$args{client};
    $self->{uid}=$uid;
    return $self;
}

###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$type)=@_; 
    
    if (defined $type) {
        $self->{on_err}=$type;
    }
    
    return $self->{on_err}; 
}
  

###############################################################################
# Method:    hostname()
# Objective: Get client host name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub hostname {
    my ($self)=@_;

    return $self->{hostname};
     
}
   
###############################################################################
# Method:    hosttype()
# Objective: Get client host type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub hosttype {
    my ($self)=@_;
    my ($hosttype);
    
    $hosttype=$self->{hosttype};
    
    if ($hosttype=~/linux/) {
        return "linux";
    }
    
    return $hosttype;
}

###############################################################################
# Method:    is_ready();
# Objective: Determine if host rpc server is ready for cmd processing
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_ready {
    my ($self,$_ready)=@_;

    if (defined $_ready){
        $self->{is_ready}=$_ready;
    }
    else {
        $_ready=(defined $self->{is_ready})
            ? $self->{is_ready}
            : 0;
      
        unless ($_ready =~ /^[1|0]$/){   
            ($self->on_err()=~/die/)
                ? die $_ready
                : $self->_set_err($_ready);
        }
    }
    
    return $_ready;
}

###############################################################################
# Method:    is_unix()
# Objective: Determine if host is unix like
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_unix {
    my ($self)=@_;
    
    return ($self->hosttype()!~/win/) ? 1 : 0;
}

###############################################################################
# Method:    is_win()
# Objective: Determine if host is win like
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_win {
    my ($self)=@_;
    
    return ($self->hosttype()=~/win/) ? 1 : 0;
}

###############################################################################
# Method:    load_libs()
# Objective: Load libraries containing rpc server call implementations 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub load_libs {
    my ($self,%args)=@_;
    my (
        $ready,
    );
    
    ### Validate input arguments
    eval {
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                code   => {type  => SCALAR}
                   
            }
        )
    };
    if ($@){
       ($self->on_err()=~/die/)
            ? croak $@
            : $self->_set_err($@);
    }
    
    $args{code}.="return 1;";
    
    ### Import libraries
    ### Traversing NATE_LIB to locate libs is slow
    ### especially on Windows (BURT 340812).
    eval {
        $self->{rpc}->perl_eval(
            code     => $args{code},
            async    => 1,
            callback => sub {
                my ($key) = @_;
                
                eval {
                    $ready = $self->{rpc}->perl_eval_reply_get(key => $key);
                };
                
                ### NATE BUG: calls supporting async need to return error
                ### rather than throw it to prevent affecting other code 
                ### that is running in parallel.
                if ($@) {
                    ### Save the error away and immediately clear
                    ### $@ so it won't influence other catch blocks
                    ### running in parallel on the return.  
                    $self->is_ready($@);
                    undef $@;
                }
                else {
                    $self->is_ready($ready);
                }
            },
        );
    };
    if ($@){
        croak $@; 
    }     
    
    1;      
}
###############################################################################
# Method:    uid()
# Objective: Return a unique identifer  for the client rpc instance
# Details:   See POD documentation at the end of this file.
###############################################################################
sub uid {
    my ($self)=@_;

    return $self->{uid};
     
}

###############################################################################
# P  R  I  V  A  T  E    M  E  T  H  O  D  S
###############################################################################
# Method:    _destroy()
# Objective: STop the rpc server on the client and destroy object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _destroy {
    my ($self)=@_;
    my (
        $rc,
        );
    
    eval {
        $rc=$self->{rpc}->quit();
    };
    if ($@) {
    
     ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
       return $self;
    }
    eval {
        $self->{rpc}->wait_finish(timeout => 120);
    };
    if ($@) {
    
     ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
       return $self;
    }
    return 1;
}

sub _execute_call {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $mth_impl,
        $output,
        $pkg_impl,
        @code_lines,
    );
    
    ### Retrieve package name and remove it from the
    ### arguments list so it doesn't get passed in
    ### as a parameter to the method call as well
    $pkg_impl=$args{pkg};
    delete ($args{pkg});
     
    ### Retrieve method name and remove it from the
    ### arguments list so it doesn't get passed in
    ### as a parameter to the method call as well
    $mth_impl=$args{method};
    delete ($args{method});
    
    ### Generate the code and store in an array 
    @code_lines = $self->_generate_code($pkg_impl,$mth_impl,%args);
    
    ### Execute the code and check for input validation and
    ### runtime errors particular to the module
    eval {
        $output = $self->{rpc}->perl_eval(code => "@code_lines");
    };
    if ($@) {
        my $saved_error = $@; 
        if (my $msg=$self->_input_validation($saved_error)) {
            $errmsg=$msg;
        }
        elsif ($saved_error=~m/(.*) at \(/s){
            ($errmsg)=$@=$1;  
        }
        else {
            ($errmsg)=$saved_error;
        }
    }
    
    ### Return output and errmsg as list
    return ($output,$errmsg);
}

###############################################################################
# Method:    _generate_code()
# Objective: Create and return calling function code block as a scalar
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _generate_code{
    my ($self,$pkg,$method,%args)=@_;
    my (
        $cnt,
        $params,
        @lines,
       );
    
    ### Insure method name doesn't include pkg
    #($method)=$method=~/.*::(\S+)$/;
    
    ### Change args back into key=>value pairs
    $params .= Dumper(\%args);
 
    ### Build the code to pass to the client.  
    push @lines,
         "use Data::Dumper;",
         "my \$ARGS = eval {my $params;};",
         "\$Log->comment('='x40);",
         "\$Log->comment('".$pkg."::$method(%{');",
         "\$Log->comment(Dumper(\$ARGS));",
         "\$Log->comment('})');";
            
    push @lines,
         "my \$rc = ".$pkg."::$method(\%\$ARGS);",
         "\$Log->comment('PID:'.\$\$);",
         "return \$rc;";

    
    return (wantarray()) ? @lines : [@lines];

} 


###############################################################################
# Method:    _input_validation()
# Objective: Parse error message returned via rpc for method input errors
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _input_validation {
    my ($self,$err)=@_;
    my (
        $callback,
        $errmsg,
        $param,
        $value,
        );
    
    if ($err=~/STAF request timed out/){
        $errmsg="STAF request timeout";
    }
    elsif ($err=~/STAF request failed/){
        $errmsg="STAF request failed";
    }
    elsif ($err=~/Mandatory parameter/i){
        ($param)=$err=~/Mandatory parameter '(\S+)'/;
        $errmsg="Mandatory parameter: ".$param;
    }
    elsif ($err=~/was not listed/i){
        ($param)=$err=~/options: (\S+)/;
        $errmsg="Invalid parameter: ".$param;
    }
    elsif ($err=~/callback/i){
        ($param,$value,$callback)=
            $err=~/'(\S+)' parameter \(\"(\S+)\"\).*'(.*)' callback/i;
        $errmsg="Failed callback: '".$callback. "' for ". $param."=>".$value;
    }
    elsif ($err=~/did not pass/i){
        ($param,$value)=$err=~/The '(\S+)' parameter \((\S+)\)/;
        $errmsg="Invalid value: ".$param."=>".$value;
    }
   
    else {
        ### Error was not due to function input
        $errmsg="";
    }
    
    return $errmsg;
}

###############################################################################
# Method:    _method_name
# Objective: Return subroutine method name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _method_name {
    my ($self)=@_;
    my (
        $method,
        );
    
    ($method)=(caller(1))[3]=~/.*::(\S+)$/;
    
    return $method;
}

1;

############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC provides the base class of helper methods for starting up the
ClientRPC::Interface rpc server on the client.  This class SHOULD NOT be
called directly.  It methods are inherited by instantiating the
ClientRPC::Interface package. 

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Utils::Error>

=head1 Public Methods

=head2 B<hostname()>

This method allows you to retrieve the hostname of the client object 

=over 4

=item Outputs

Returns a scalar value containing the hostname.  

=item Examples 

 # Get hostname
 $hostname= $ClientRpcObj->hostname();
 $Test->nlog("Hostname". $hostname);

=back

=head2 B<hosttype()>

This method allows you to retrieve the hosttype of the client machine 

=over 4

=item Outputs

Returns a scalar value containing the hosttype.  

=item Examples

 # Get hosttype
 $hosttype= $ClientRpcObj->hosttype();
 $Test->nlog("Hosttype". $hosttype);

=back

=head2 B<is_ready()>

Determine if a clientRPC object is ready for processing cmd requests

=over 4

=item Outputs

Returns 1 if ready, otherwise 0

=item Examples

 # Example 1: Determine if clientRpcObj is unix.   
 foreach my $clientObj (@ClientObjs){
    sleep 1 unless ($clientObj->is_ready());
 }

=back

=head2 B<is_unix()>

Determine if a clientRPC object is connected to Unix like system

=over 4

=item Outputs

Returns 1 if unix, otherwise 0

=item Examples

 # Example 1: Determine if clientRpcObj is unix.   
 if ($clientRpcObj->is_unix()){
    $path="/t/myvol";
 }

=back

=head2 B<is_win()>

Determine if a clientRPC object is connected to Window's like system

=over 4

=item Outputs

Returns 1 if windows, otherwise 0

=item Examples

 # Example 1: Determine if clientRpcObj is windows.
 if ($clientRpcObj->is_win()){
    $path="\\MYSHARE\DIR";
 }

=back

=head2 B<load_libs( B<code> )>

Load implementation libraries required by rpc server to execute test interface
stub calls.  Required libraries must be passed in as a scalar code block

=over 4

=item Inputs 

=over 4 

=item * B<code>

Code block required by test 

=back

=item Outputs

None

=item Examples

 # Example 1: Load implementation libraries
 $code='use myImplLib;';
 $clientRpcObj->load_libs($code); 

=back

=head2 B<on_err( I<$type> )>

Defines what error handling mechanism to use on ClientRPC instance

=over 4

=item Inputs

=over 4

=item * I<$type>

Indicates what errhandler type to use for all ClientRPC::Interface methods
calls.  If no errorhandle is provided, it returns the current handler type 

=begin html

<ul>
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=back

=item Outputs

Returns error handle type  on success, throws error on failure

=item Examples

 # Example 1: Set the defualt error handle from 'die' to 'rtn'   
 $hdlr=$clientRpcObject->on_err('rtn');
 $test->nlog("Error handler changed to '".$hdlr."'");

=back

=head2 B<uid()>

Retrieve unique identifier associated with object at creation time 

=over 4

=item Outputs

Returns uid value

=item Examples

 # Example 1: Retrieve uid tag.
 $uid=$clientRpcObj->uid();

=back

=cut
