# Hey emacs, this is basically -*- perl -*-
# $Id$

## @summary     Class for basic win32 streams
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here
###############################################
##   NOT READY FOR PUBLIC CONSUMPTION
###############################################
package ClientRPC::Impl::CIFSStream;

use strict;
use Carp qw(croak);
use Data::Dumper;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use Tharn qw ($Log);
use Win32::API;
use Win32API::File qw(
   :Func :Misc :FILE_ :FILE_FLAG_ :FILE_SHARE_ :FILE_ATTRIBUTE_ :GENERIC_ );
#use MapShare qw(map_cifs_share unmap_cifs_share);
use constant KERNEL32 => 'C:\WINDOWS\system32\kernel32.dll';
use RichData;

                  
my (%MapOf,$LockObj,$UnlockObj);
my @rd_opts = (
    dir_acl_pct => 0,
    file_acl_pct => 0,
    dir_stream_pct => 50,
    file_stream_pct => 50,
    stream_avg => 10,

    link_pct => 0,
    slink_pct => 0,
    compress_pct => 0,
    encrypt_pct => 0,
    sparse_pct => 0,
);
###############################################################################
# Method:    make_streams()
# Objective: Make Streams
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub make_streams {
    my (%args)=@_;
    my (
        $fh,
        $file,
        );
    
    ### Validate inout arguments
    (%args) = validate (
        @_,
        {
            path       => { type => SCALAR} ,                
        }
    );
    
    $Log->comment("YOU ARE HERE!!!!");
      
    RichData::populate(path => $args{path},
                        @rd_opts);

    return 1;
}

###############################################################################
# Method:    modify_acls()
# Objective: Modify ACLs
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub modify_streams {
    my (%args)=@_;
    my (
        $fh,
        $file,
        );
    
    ### Validate inout arguments
    (%args) = validate (
        @_,
        {
            path       => { type => SCALAR },                
        }
    );
    
    $Log->comment("YOU ARE HERE!!!!");
    eval { 
    RichData::modify(path => $args{path},
                    @rd_opts);
    };
    if ($@) {
        $Log->comment($@);
    }
    return 1;
}

###############################################################################
# Method:    delete_acls()
# Objective: Delete ACLs
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub delete_streams {
    my (%args)=@_;
    my (
        $fh,
        $file,
        );
    
    ### Validate inout arguments
    (%args) = validate (
        @_,
        {
            path       => { type => SCALAR },                
        }
    );
    
    $Log->comment("YOU ARE HERE!!!!");
  
    eval {
    RichData::modify(path => $args{path},
                    @rd_opts,
                    dir_stream_pct => 0,
                    file_stream_pct => 0,
                    stream_avg => 0);
    }; 
    if ($@) {
        $Log->comment($@);
    }
    return 1;
}




1;

############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC::Impl::CIFSStream

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC::Impl::CIFSStream provides an API interface driving ACL methods 

=head1 See Also

This package is accessible through instantiation of ClientRPC::Interface 
enabling easy execution of Stream driver on remote host via RPC  

B<ClientRPC::Interface>

=head1 Public Methods

=head2 B<make_streams( B<path>)>

Makes streams on volume  

=over 4

=item Inputs

=over 4

=item * B<path>

Path where volume is mounted

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Make streams
    
eval {
   $rc = make_streams  (path=>"/t/9.0.2.39/root-tcp3");
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   


=back

=head2 B<modify_streams(B<path> )>

Modify streams on volume

=over 4

=item Inputs

=over 4

=item * B<path>

Path where volume is mounted

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Modify streams 
    
eval {
   $rc = modify_streams(path=>"/t/9.0.2.39/root-tcp3");
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   


=back

=head2 B<delete_streams(B<path> )>

Delete streams dataset from volume

=over 4

=item Inputs

=over 4

=item * B<path>

Path where volume is mounted


=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Delete streams dataset 
    
eval {
   $rc = delete_streams (
            path=>"/t/9.0.2.39/root-tcp3"
        );
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   

=back

=cut
