# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/ClientRPC/Impl/DtLin.pm#1 $
## @summary     Linux DataTest class
## @author      lippe@netapp.com
## @status      private
###############################################
##   NOT READY FOR PUBLIC CONSUMPTION
###############################################
package ClientRPC::Impl::DtLin;

use strict;
use Data::Dumper;
use Exporter;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use POSIX;
use Tharn qw ($Log);

use vars qw(@Subtests);

###############################################################################
# T E M P O R A R Y   M E T H O D   - WAITING FOR NEW HOME
###############################################################################
# Method:    start_dt()
# Objective: Run DataTest utility on a client share
############################################################################### 
sub start_dt {
    my (%args)=@_;
    my (
	$binary,
	$brief,
	$cmd,
	$name,
	$outfile,
	$seconds,
	$time,
	$tm,
	@output,
	);

    ### Validate input arguments
    (%args) = validate (
        @_,
        {
	    runtime    => { type => SCALAR, optional => 1 },
	    procs      => { type => SCALAR, default => 1 },
	    limit      => { type => SCALAR, default => "2m" },
	    dispose    => { type => SCALAR, default => "delete" }, 
	    block      => { type => SCALAR, default => "128k" },
	    errors     => { type => SCALAR, default => 1 },
	    pattern    => { type => SCALAR, default => "0x39c39c39c" },
	    path       => { type => SCALAR },
	    file       => { type => SCALAR, default => 'datatest' },
	    name       => { type => SCALAR, optional => 1 },
        client     => { type => SCALAR, default=>"localhost" },
        }
    );

    $outfile = $args{path} . '/' . $args{file};
    $name = (defined $args{name}) ? $args{name} : "datatest" . int (rand 10000);

    # $binary = "nice -n 15 /x/eng/localtest/arch/i386-redhat-linux7/bin/dt edelay=1";
    $binary = "nice -n 15 /usr/software/test/bin/dt edelay=1";
    
    $cmd  =  $binary. " ".
	"of=".$outfile." ".
	"limit=".$args{limit}." ".
	"bs=".$args{block}." ".
	"procs=".$args{procs}." ".
	"oncerr=abort ".
	"errors=".$args{errors}." ".
	"dispose=".$args{dispose}." ".
	"pattern=\"".$args{pattern}."\" ";    
    
    $cmd .= "runtime=$args{runtime}" if ($args{runtime});

    # Add a return at end of cmd so datatest is executed and is not effected
    # by additional cmds tacked on the end.  
    $cmd .= "\\\n";

    $Log->comment("DataTest cmd: $cmd");

    push @Subtests, _execute_background($cmd, $name);

    return 1;
}

################################################################################
# Subroutine Name: dt_wait()
# Objective:       check to see if any DataTest processes are running.
################################################################################
sub dt_wait {
    my ($subtest, $fail_cnt, $flag);

    foreach $subtest (@Subtests) {
	while ($subtest->isAlive()){
	    sleep 5;
	}
    }

    # We're checking the results object here because we want to abstract this
    # from the user.  A die here will propagate the error up to the Interface.
    # Remember, implementations always die!!!!!
    foreach $subtest (@Subtests) {
	my @results = $subtest->rslts();

	$Log->comment(Dumper(@results));

	foreach my $res (@results) {
	    if ($res->severity >= 0) {
		$Log->comment("RESULT (" . $res->runid . ") " . $res->type, $res->messages);
		
		if (grep /failure/, $res->messages) {
		    $fail_cnt++;
		}
	    }
	}
    }

    $Log->comment("KRIS: $fail_cnt");

    if ($fail_cnt) {
	die "Dt failures detected!  Check logs for details";
    } else {
	return 0;
    }
}

################################################################################
# Subroutine Name: _execute_background()
# Objective:       execute in background
################################################################################
sub _execute_background {
    my ($cmd, $name)=@_;

    my $subtest = Tharn::subtest( 
				  sub {
				      system($cmd);
				      if ($?) {
					  $Log->comment("Dt returned status: $?");
					  die "Dt failure detected! $?";
				      }
				  },
				  -runid, $name, -bg,
				  -ifscript, -fail, "Dt script failure",
				  -iffatal, -fail, "Dt reported fatal result"
				  );      
    
    return $subtest;
}

1;
