# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/ClientRPC/Impl/Mount.pm#1 $

## @summary     Class for mounting volume
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here
###############################################
##   NOT READY FOR PUBLIC CONSUMPTION
###############################################
package ClientRPC::Impl::Mount;

use strict;
use Carp qw(croak);
use Data::Dumper;
use Exporter;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use POSIX;
use Tharn qw ($Log);
use mount qw (); 
use MapShare qw(map_cifs_share unmap_cifs_share);
my %MountOf;

###############################################################################
# T E M P O R A R Y   M E T H O D   - WAITING FOR NEW HOME
###############################################################################
# Method:    mount()
# Objective: mount
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub mount {
    my (%args)=@_;
    my (
       $mnt_pt,
    );
 
    ### Validate input arguments
    (%args) = validate (
        @_,
        {
            client     => { type => SCALAR, default=>"localhost" },
            server     => { type => SCALAR },
            path       => { type => SCALAR },
            mountpoint => { type => SCALAR, 
                            callbacks=>{
                            'Mount point check'=>sub{
                                not exists $MountOf{$args{mountpoint}}           
                            }}},
            mountopts  => { type => SCALAR, default=>"rw,intr,hard" },
            vers       => { type => SCALAR, default=>4,regex => qr/^\d+$/ },
        }
    );
 
    if ($^O=~/win/i) {
        ### BUG: On ERROR MapShare is NOT throwing an
        ### error.  Need to analyze $mnt_pt. BOGUS
        $mnt_pt = MapShare::map_cifs_share(
            server => $args{server},
            share  => $args{path},
            client => $args{client},
            drive  => $args{mountpoint});
        
        $MountOf{$mnt_pt}=1;
    }
    else {  
        $mnt_pt = mount::mount(
            $args{server},
            $args{path},
            client     => $args{client},
            vers       => $args{vers},
            mountpoint => $args{mountpoint},
            mountopts  => $args{mountopts},
            chmod      => 777);
        
        $MountOf{$args{mountpoint}}=1;
    }
    
    if ($@ or length($mnt_pt)==0) {
        my ($errmsg)=$@=~/^(.*)\n/m;
        $errmsg="Unable to mount" if (not length $errmsg);
        $Log->comment($errmsg);
        croak "Unable to mount ".$args{path}.": ".$@
    }
    
    
    return $mnt_pt; 
     
}

###############################################################################
# Method:    umount()
# Objective: unmount mountpoint
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub umount {
    my (%args)=@_;
    my (
       $mnt_pt,
    );
 
    ### Validate input arguments
    (%args) = validate (
        @_,
        {
            client => {type => SCALAR, default=>"localhost" },
            mountpoint => { type => SCALAR},
                            
        }
    );
    
    if ($^O=~/win/i) {
        MapShare::unmap_cifs_share(
            drive  => $args{mountpoint},
            client => $args{client})
            or croak "Unable to unmap CIFS share ".$args{mountpoint}.":". $@;  
    }
    else {
        mount::umount(
            $args{mountpoint},
            client => $args{client},
            force => 1);
    }

    if ($@) {
        croak "Unable to umount ".$args{mountpoint}.": ".$@
    }
    
    delete $MountOf{$args{mountpoint}};
    
  1;
     
}

1;

############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC::Impl::Mount

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC::Impl::Mount provides an API interface for mounting and unmounting
volume storage on Unix and Win machines

=head1 See Also

This package is accessible through instantiation of any ClientRPC::I<Interface>  
package that inherits the ClientRPC::Intf::Mount stub package.

B<ClientRPC::I<Interface>>

=head1 Public Methods

=head2 B<mount(  B<server>, B<path>, B<mountpoint>, I<vers> )>

Mount (or map if Win32) a shared storage device.  

=over 4

=item Inputs

=over 4

=item * B<mountpoint>  

Directory name on client to attach device file system too. On Win32, the
value '*' can be used to select an unavailable drive to use.

=item * B<path>   

Path to device file system.

=item * B<server> 

Name or IP address of system containing device file system

=item * I<mountopts> (default: rw,intr,hard)

What mount options to use (what to pass to the "-o" option of the
"mount" executable).  The default is 'hard,intr,noquota'.  
Only applicable for NFS mounted file systems

=item * I<vers>  (default: 4)  

Version of protocol to use.  Only applicable for NFS mounted file systems

=back

=item Outputs

Returns mount point on success, 

=item Examples

# Example 1: Mount a device
    
    $mnt_pt=$clientObj->mount(
        server=>"9.0.2.1",
        path=>"/.admin/vol1",
        mountpoint=>"/t/mydir",
        vers=>"4"
    );
    if ($clientObj->error){
        $Test->nlog($clientObj->errmsg());
        return $TCD::FAIL;
    } 


=back

=head2 B<umount( B<mountpoint> )>

Unmount (or unmap if Win32) a shared storage device.  

=over 4

=item Inputs

=over 4

=item * B<mountpoint>

Directory name on client to detach device file system. 

=back

=item Outputs

None

=item Examples

    # Example 1: Unmount a device
    
    $clientObj->umount( 
        mountpoint=>"/t/mydir"
    );
    if ($clientObj->error){
        $Test->nlog($clientObj->errmsg());
        return $TCD::FAIL;
    }


=back

=cut; 
