# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/ClientRPC/Impl/Win32/FileOps.pm#1 $

## @summary     Class for basic Win32 file operations
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here
###############################################
##   NOT READY FOR PUBLIC CONSUMPTION
###############################################
package ClientRPC::Impl::Win32::FileOps;

use strict;
use Carp qw(croak);
use Data::Dumper;
use Exporter;
use File::stat;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use POSIX;
use Tharn qw ($Log);

my %MapOf;
my %MountOf;

###############################################################################
# Method:    link_file()
# Objective: Link file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub link_file {
    my %args = @_;
    my (
        $file,
        $flink,
        $symbolic,
    );

    (%args) = validate (
        @_,
        {
            file     => { type => SCALAR },
            flink    => { type => SCALAR },
            symbolic => { type => SCALAR, default=>0,
                          callbacks=>{
                            'Legal Values'=>
                               sub{shift()=~/1|0/i }}}, 
        }
    );
    
    croak "Symbolic links are not supported on Windows";
   
}

###############################################################################
# Method:    rename_file()
# Objective: rename file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub rename_file {
    my (%args)=@_;
    my (
       $ofile,
       $nfile,
       $nslash,
    );
    
    (%args) = validate (
        @_,
        {
            file  => { type => SCALAR },
            nfile => { type => SCALAR },
        }
    );
    
    ### Initialize variables
    $ofile = $args{file};
    $nfile = $args{nfile};

    ### Do not attempt to rename root directory
    if ( $ofile =~ /^.:\\?$/ or $ofile =~ m{^/$} ) {
        croak "Renaming root directory not allowed";
    }

    ### Get rid off multiple '/'s
    $ofile =~ s{/+}{/};
    
    ### Get rid off trailing '/'
    $ofile =~ s{/$}{};
    
    ### Do not rename file if it is in the root directory on unix,
    ### i.e., contains only one '/'.
    $nslash = ($ofile =~ tr{/}{/});
    if ( $ofile !~ /^.:/ and (($ofile =~ m{^/} and $nslash == 1)
      or getcwd() eq '/' and $nslash == 0) ) {
        croak "Renaming file located in root directory not allowed";
    }

    ### Rename file
    rename $ofile, $nfile or croak "Could not rename file $ofile: $!";

    return 1;
}

###############################################################################
# Method:    sizeof_file()
# Objective: size of file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub sizeof_file {
    my (%args)=@_;
    my (
        $file,
        $size,
        );
    
    ### Validate input arguments
    (%args) = validate (
        @_,
        {
            file     => { type => SCALAR },                     
        }
    );
    
    ### Retrieve $file
    $file=$args{file};
   
    ### Get file size
    eval {
        $size = stat($file)->size;
    };
    if ($@){
        croak $@;
    }
   
    return $size;
}


############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC::Impl::Win32::FileOps

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC::Impl::Win32::FileOps provides an API interface for performing file 
operations. 

=head1 See Also

This package is accessible through the instantiation of ClientRPC::FileOperations 
package enabling easy execution of locking methods on remote host via RPC 

B<ClientRPC::FileOperations>

=head1 Public Methods

=head2 B<rename_file( B<file>, B<nfile>)>

Rename a file. 

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file name 

=item * B<nfile>    

Full path to new file name

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Rename file 
    
eval {
   $rc = rename_file (file=>"S:/f1.txt", nfile=>"S:/f1.txt_new");
);
if ($@) {
   $Test->nlog($@);
   return $TCD::FAIL;
}   

=back

=head2 B<sizeof_file( B<file>)>

Returns the size of file in bytes

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file 

=back

=item Outputs

Returns size of file on success, throws error on failure

=item Examples

# Example 1: Size of file 
    
eval {
   $bytes = sizeof_file (file=>"S:/f1.txt");
);
if ($@) {
   $Test->nlog($@);
   return $TCD::FAIL;
}   

=back

=cut

1;