# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/ClientRPC/Intf/FileOps.pm#1 $

## @summary     Interface class for file operation implementations
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here

package ClientRPC::Intf::FileOps;
use strict;

use Tharn qw($Log);
  
use base qw(
   ClientRPC
);

### Location of package implementation
my $UnixImplLocation="ClientRPC::Impl::Unix::FileOps"; 
my $WinsImplLocation="ClientRPC::Impl::Win32::FileOps"; 


###############################################################################
# Method:    link_file()
# Objective: link file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub link_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    rename_file()
# Objective: Read file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub rename_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    sizeof_file()
# Objective: get size of file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub sizeof_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

1;
############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC::Intf::FileOps

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC::Intf::FileOps is a package wrapper that provides an abstracted
interface for calling the file operation implementation methods via RPC.  It
along with the use of its base class methods, repackage the requested call (with 
arguments) as a stringified code block that is then sent to the remote client
rpc server for execution. The result is then returned back from the remote 
rpc server along with any error message. 

=head1 See Also 

This package is accessible through instantiation of ClientRPC::FileOperations
which provides easy execution of file operation methods on remote host via RPC. 
It supports the following astracted methods... 

=begin html

<ul>
<li><b>rename_file()</b>
<li><b>link_file()</b>
<li><b>sizeof_file()</b>
</ul><p>

=end html

For the method implementation and documentation details, see the module(s)
for the particular operating system of interest.

B<ClientRPC::Impl::Win32::FileOps>

B<ClientRPC::Impl::Unix::FileOps>

This package inherits from the following modules. See these modules for 
further interface detail.

B<ClientRPC>

=cut