# Hey emacs, this is basically -*- perl -*-
# vim: set filetype=perl tabstop=4 shiftwidth=4 shiftround expandtab:
# $Id: //depot/prod/test/Rsierranevada/lib/ClientRPC/Intf/Filelock.pm#1 $

## @summary     Interface class for file lock implementations
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here

package ClientRPC::Intf::Filelock;
use strict;

use Tharn qw($Log);
  
use base qw(
   ClientRPC
);

### Location of package implementation
my $UnixImplLocation="Filelock::Unix"; 
my $WinsImplLocation="Filelock::Win32"; 


###############################################################################
# Method:    close_file()
# Objective: Close file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub close_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    delete_file()
# Objective: delete file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub delete_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    lock_file()
# Objective: lock file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub lock_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}


###############################################################################
# Method:    open_file()
# Objective: Open file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub open_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    read_file()
# Objective: Read file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub read_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    write_file()
# Objective: Write file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub write_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    unlock_file()
# Objective: unlock file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub unlock_file {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    get_file_locks()
# Objective: Get array of existing locks on a file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub get_file_locks {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

1;
############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC::Intf::Filelock

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC::Intf::Filelock is a package wrapper that provides an abstracted
interface for calling the file lock implementation methods via RPC.  It along
with the use of its base class methods, repackage the requested call ( with 
arguments) as a stringified code block that is then sent to the remote client
rpc server for execution. The result is then returned back from the remote 
rpc server along with any error message. 

=head1 See Also 

This package is accessible through instantiation of ClientRPC::Locks 
enabling easy execution of locking methods on remote host via RPC.  It 
supports the following astracted methods... 

=begin html

<ul>
<li><b>close_file()</b>
<li><b>delete_file()</b>
<li><b>lock_file()</b>
<li><b>open_file()</b>
<li><b>unlock_file()</b>
<li><b>read_file()</b>
<li><b>write_file()</b>
<li><b>get_file_locks()</b>
</ul><p>

=end html

For the method implementation and documentation details, see the module(s)
for the particular operating system of interest.

B<Filelock::Win32>

B<Filelock::Unix>

This package inherits from the following modules. See these modules for 
further interface detail.

B<ClientRPC>

=cut
