# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/ClientRPC/Intf/Mount.pm#1 $

## @summary     Interface class for file lock implementations
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here

package ClientRPC::Intf::Mount;
use strict;

use Tharn qw($Log);
  
use base qw(
   ClientRPC
);

### Location of package implementation
my $UnixImplLocation="ClientRPC::Impl::Mount"; 
my $WinsImplLocation="ClientRPC::Impl::Mount";  


###############################################################################
# Method:    mount()
# Objective: mount share
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub mount {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}

###############################################################################
# Method:    umount()
# Objective: umount share
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub umount {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output;  
}

1;
############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC::Intf::Mount

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC::Intf::Mount is a package wrapper that provides an abstracted
interface for calling mount implementations methods via RPC.  It along
with the use of its base class methods, repackages the requested call (with 
arguments) as a stringified code block that is then sent to the remote client
rpc server for execution. The result is then returned back from the remote 
rpc server along with any error message. 

=head1 See Also 

This package is accessible through instantiation of ClientRPC::Locks 
enabling easy execution of locking methods on remote host via RPC.  It 
supports the following astracted methods... 

=begin html

<ul>
<li><b>mount()</b>
<li><b>umount()</b>
</ul><p>

=end html

For the method implementation and documentation details, see the module(s)
for the particular operating system of interest.

B<ClientRPC::Impl::Mount>

This package inherits from the following modules. See these modules for 
further interface detail.

B<ClientRPC>

=cut