# Hey emacs, this is basically -*- perl -*-
# $Id$

## @summary     Interface class for CIFS ACL implementations
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here

package ClientRPC::Intf::Pathdiff;
use strict;

use Tharn qw($Log);
  
use base qw(
   ClientRPC
);

### Location of package implementation
my $UnixImplLocation="ClientRPC::Impl::Pathdiff"; 
my $WinsImplLocation="ClientRPC::Impl::Pathdiff"; 

###############################################################################
# Method:    pathdiff_init()
# Objective: init pathdiff object with specific args
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub pathdiff_init {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}


###############################################################################
# Method:    snapshot()
# Objective: Snapshot of pathdiff 
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub snapshot {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}



###############################################################################
# Method:    compare()
# Objective: Compare 
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub compare {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}


###############################################################################
# Method:    delete_db()
# Objective: Delete database file 
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub delete_db {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $method,
        $output,
        $pkg,
    );
    
    ### Clear error handler
    $self->_clear_err();
    
    ### Determine what package to use
    $pkg=($self->is_win())
        ? $WinsImplLocation
        : $UnixImplLocation;
    
    ### Interface method name to use (no pkg).
    ### Defaults to the current method name.
    $method=$self->_method_name();
    
    ### Execute request
    ($output,$errmsg)=$self->_execute_call(pkg=>$pkg, method=>$method, %args);
    if ($errmsg){
       ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }     
        
    return $output; 
}



1;
############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

ClientRPC::Intf::Pathdiff

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

ClientRPC::Intf::Pathdiff is a package wrapper that provides an abstracted
interface for calling the Pathdiff driver methods via RPC.  It along
with the use of its base class methods, repackage the requested call ( with 
arguments) as a stringified code block that is then sent to the remote client
rpc server for execution. The result is then returned back from the remote 
rpc server along with any error message. 

=head1 See Also 

This package is accessible through instantiation of ClientRPC::Pathdiff 
enabling easy execution of ACL Driver on remote host via RPC.  It 
supports the following astracted methods... 

=begin html

<ul>
<li><b>pathdiff_init(Pathdiff->new arguments)</b>
<li><b>snapshot()</b>
<li><b>compare()</b>
<li><b>delete_db()</b>
</ul><p>

=end html

For the method implementation and documentation details, see the module(s)
for the particular operating system of interest.

B<ClientRPC::Impl::Pathdiff>

This package inherits from the following modules. See these modules for 
further interface detail.

B<ClientRPC>

=cut
