#!/usr/local/bin/perl
# $Id: //depot/prod/test/Rsierranevada/lib/CmdsDriver/TranslationTable.pm#3 $
#
# Summary:
#   map Cmode cdef CLI commands to ZAPI for mode agnostic command testing
#
# Description:
#   The table provides the mapped commands between CDEF and ZAPI-like command
#   This allows users of CmdsDriver to write mode agnostic command testing
#   efficiently.
#
# Dependcies:    Perl
# Status:    private
# Author:   korry@netapp.com, dl-ssan-automation

use strict;
use vars qw(%mapped_cmds);


%mapped_cmds = (
    # Cmode CDEF                 # ZAPI/7Glike
    'cluster_identity_show' => {
        'cmd' => { 'zapi' => 'cluster-identity-get', '7glike' => ''},
        '7g-order' => '',
        },
    'cluster_show' => {
        'cmd'         => { 'zapi' => 'system-node-get-iter',     '7glike' => ''},
        'epsilon'     => { 'zapi' => 'is-epsilon-node',          '7glike' => ''},
        'eligibility' => { 'zapi' => 'is-node-cluster-eligible', '7glike' => ''},
        'health'      => { 'zapi' => 'is-node-healthy',          '7glike' => ''},
        'query'       => 'node-details-info',
        '7g-order'    => '',
        },
    'system_node_show' => {
        'cmd' => { 'zapi' => 'system-node-get', '7glike' => ''},
        '7g-order' => '',
        },
    'storage_disk_show' => {
        'cmd' => { 'zapi' => 'storage-disk-get-iter', '7glike' => ''},
        'query' => 'storage-disk-info',
        '7g-order' => '',
        },
    'storage_disk_zerospares' => {
        'cmd' => { 'zapi' => 'disk-zero-spares', '7glike' => ''},
        'owner' => { 'zapi' => 'node-name', '7glike' => ''},
        '7g-order' => '',
        },
    'storage_aggregate_create' => {
        'cmd' => { 'zapi' => 'aggr-create', '7glike' => ''},
        'aggregate' => { 'zapi' => 'aggregate', '7glike' => ''},
        'mirror' => { 'zapi' => 'is-mirrored', '7glike' => ''},
        'diskcount' => { 'zapi' => 'disk-count', '7glike' => ''},
        'raidtype' => { 'zapi' => 'raid-type', '7glike' => ''},
        'disklist' => { 'zapi' => 'disks', '7glike' => ''},
        'nodes' => { 'zapi' => { 'nodes' => 'node-name'}, '7glike' => ''},
        '7g-order' => '',
        },
    'storage_aggregate_show' => {
        'cmd' => { 'zapi' => 'aggr-get-iter', '7glike' => ''},
        'aggregate' => { 'zapi' => 'aggregate-name', '7glike' => ''},
        'query' => 'aggr-attributes',
        '7g-order' => '',
        },
    'storage_aggregate_modify' => {
        'cmd' => { 'zapi' => 'aggr-set-option', '7glike' => ''},
        '7g-order' => '',
        },
    'storage_aggregate_delete' => {
        'cmd' => { 'zapi' => 'aggr-destroy', '7glike' => ''},
        '7g-order' => '',
        },
    'network_fcp_adapter_show' => {
        'cmd' => { 'zapi' => 'fcp-adapter-get-iter', '7glike' => ''},
        'query' => 'fcp-config-adapter-info',
        '7g-order' => '',
        },
    'network_fcp_adapter_modify_up' => {
        'cmd' => { 'zapi' => 'fcp-adapter-config-up', '7glike' => ''},
        'adapter' => { 'zapi' => 'fcp-adapter', '7glike' => ''},
        '7g-order' => '',
        },
    'network_fcp_adapter_modify_down' => {
        'cmd' => { 'zapi' => 'fcp-adapter-config-down', '7glike' => ''},
        'adapter' => { 'zapi' => 'fcp-adapter', '7glike' => ''},
        '7g-order' => '',
        },
    'network_fcp_adapter_modify_speed' => {
        'cmd' => { 'zapi' => 'fcp-adapter-set-speed', '7glike' => ''},
        'adapter' => { 'zapi' => 'fcp-adapter', '7glike' => ''},
        '7g-order' => '',
        },
    'network_fcp_adapter_stats' => {
        'cmd' => { 'zapi' => 'fcp-adapter-stats-get-iter', '7glike' => ''},
        'query' => 'fcp-adapter-stats-info',
        '7g-order' => '',
        },
    'network_options_ipv6_modify' => {
        'cmd' => { 'zapi' => 'net-options-modify', '7glike' => ''},
        'enabled'=> { 'zapi' => {'net-options'=>{'ipv6-options-info'=>'enabled'} },'7glike' => ''},
        '7g-order' => '',
        },
    'vserver_create' => {
        'cmd' => { 'zapi' => 'vserver-create', '7glike' => ''},
        'vserver' => { 'zapi' => 'vserver-name', '7glike' => ''},
        'aggregate' => { 'zapi' => 'root-volume-aggregate', '7glike' => ''},
        'rootvolume' => { 'zapi' => 'root-volume', '7glike' => ''},
        'rootvolume-security-style' => { 'zapi' => 'root-volume-security-style', '7glike' => ''},
        'ns-switch' => { 'zapi' => { 'name-server-switch' => 'nsswitch'}, '7glike' => ''},
        '7g-order' => '',
       },
    'vserver_show' => {
        'cmd' => { 'zapi' => 'vserver-get-iter', '7glike' => ''},
        'vserver' => { 'zapi' => 'vserver-name', '7glike' => ''},
        'query' => 'vserver-info',
        '7g-order' => '',
        },
    'vserver_delete' => {
        'cmd' => { 'zapi' => 'vserver-destroy', '7glike' => ''},
        'vserver' => { 'zapi' => 'vserver-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_start' => {
        'cmd' => { 'zapi' => 'vserver-start', '7glike' => ''},
        'vserver' => { 'zapi' => 'vserver-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_stop' => {
        'cmd' => { 'zapi' => 'vserver-stop', '7glike' => ''},
        'vserver' => { 'zapi' => 'vserver-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_rename' => {
        'cmd' => { 'zapi' => 'vserver-rename', '7glike' => ''},
        'vserver' => { 'zapi' => 'vserver-name', '7glike' => ''},
        'new-name' => { 'zapi' => 'newname', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_alias' => {
        'cmd'         => { 'zapi' => 'iscsi-target-alias-set-alias','7glike' => ''},
        'target-alias'=> { 'zapi' => 'alias-name',                  '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_alias_get' => {
        'cmd'         => { 'zapi' => 'iscsi-target-alias-get-alias','7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_alias_clear' => {
        'cmd'         => { 'zapi' => 'iscsi-target-alias-clear-alias','7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_show' => {
        'cmd'          => { 'zapi' => 'iscsi-service-get-iter', '7glike' => ''},
        'target-name'  => { 'zapi' => 'node-name',              '7glike' => ''},
        'target-alias' => { 'zapi' => 'alias-name',             '7glike' => ''},
        'status-admin' => { 'zapi' => 'is-available',           '7glike' => '', 'up' => 'true', 'down' => 'false'},
        'query'        => 'iscsi-service-info',
        '7g-order'     => '',
        },
    'vserver_iscsi_create' => {
        'cmd' => { 'zapi' => 'iscsi-service-create', '7glike' => ''},
        'status-admin' => { 'zapi' => 'start', '7glike' => '', 'up' => 'true', 'down' => 'false'},
        'target-name' => { 'zapi' => 'node-name', '7glike' => ''},
        'target-alias' => { 'zapi' => 'alias-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_delete' => {
        'cmd' => { 'zapi' => 'iscsi-service-destroy', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_interface_disable' => {
        'cmd' => { 'zapi' => 'iscsi-interface-disable', '7glike' => ''},
        'lif' => { 'zapi' => 'interface-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_interface_enable' => {
        'cmd' => { 'zapi' => 'iscsi-interface-enable', '7glike' => ''},
        'lif' => { 'zapi' => 'interface-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_start' => {
        'cmd' => { 'zapi' => 'iscsi-service-start', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_stop' => {
        'cmd' => { 'zapi' => 'iscsi-service-stop', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_status' => {
        'cmd'          => { 'zapi' => 'iscsi-service-status', '7glike' => ''},
        'status-admin' => { 'zapi' => '', '7glike' => ''},
        'is-available' => { 'zapi' => '', '7glike' => ''},
        '7g-order'     => '',
        },
    'vserver_iscsi_interface_accesslist_show' => {
        'cmd'            => { 'zapi' => 'iscsi-interface-accesslist-get-iter', '7glike' => ''},
        'initiator-name' => { 'zapi' => 'initiator',                           '7glike' => ''},
        'lif'            => { 'zapi' => 'interface-name',                      '7glike' => ''},
        'query'          => 'iscsi-interface-accesslist-entry-info',
        '7g-order'       => '',
        },
    'vserver_iscsi_interface_accesslist_add' => {
        'cmd'            => { 'zapi' => 'iscsi-interface-accesslist-add', '7glike' => ''},
        'lif'            => { 'zapi' => 'interface-name',                 '7glike' => ''},
        'initiator-name' => { 'zapi' => 'initiator',                      '7glike' => ''},
        '7g-order'       => '',
        },
    'vserver_iscsi_interface_accesslist_remove' => {
        'cmd'            => { 'zapi' => 'iscsi-interface-accesslist-remove', '7glike' => ''},
        'lif'            => { 'zapi' => 'interface-name',                    '7glike' => ''},
        'initiator-name' => { 'zapi' => 'initiator',                         '7glike' => ''},
        '7g-order'       => '',
        },
    'vserver_iscsi_security_show' => {
        'cmd'                => { 'zapi' => 'iscsi-initiator-auth-get-iter', '7glike' => ''},
        'initiator-name'     => { 'zapi' => 'initiator',                     '7glike' => ''},
        'auth-type'          => { 'zapi' => 'auth-type',                     '7glike' => ''},
        'user-name'          => { 'zapi' => 'user-name',                     '7glike' => ''},
        'outbound-user-name' => { 'zapi' => 'outbound-user-name',            '7glike' => ''},
        'auth-chap-policy'   => { 'zapi' => 'auth-chap-policy',              '7glike' => ''},
        'query'          => 'iscsi-security-entry-info',
        '7g-order'       => '',
        },
    'vserver_iscsi_security_create' => {
        'cmd'            => { 'zapi' => 'iscsi-initiator-add-auth', '7glike' => ''},
        'initiator-name' => { 'zapi' => 'initiator',                '7glike' => ''},
        '7g-order'       => '',
        },
    'vserver_iscsi_security_default' => {
        'cmd' => { 'zapi' => 'iscsi-initiator-set-default-auth', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_security_delete' => {
        'cmd'            => { 'zapi' => 'iscsi-initiator-delete-auth', '7glike' => ''},
        'initiator-name' => { 'zapi' => 'initiator',                   '7glike' => ''},
        '7g-order'       => '',
        },
    'vserver_iscsi_security_modify' => {
        'cmd'            => { 'zapi' => 'iscsi-initiator-modify-chap-params', '7glike' => ''},
        'initiator-name' => { 'zapi' => 'initiator',                          '7glike' => ''},
        'auth-type'      => { 'zapi' => '',                                   '7glike' => ''},
        '7g-order'       => '',
        },
    'vserver_iscsi_security_generate' => {
        'cmd' => { 'zapi' => 'iscsi-auth-generate-chap-password', '7glike' => ''},
        '7g-order' => '',
        },
    'test_password_get' => {
        'cmd'            => { 'zapi' => 'test-password-get', '7glike' => ''},
        '7g-order'       => '',
        },
    'vserver_iscsi_interface_show' => {
        'cmd'              => { 'zapi' => 'iscsi-interface-get-iter', '7glike' => ''},
        'lif'              => { 'zapi' => 'interface-name',           '7glike' => ''},
        'enabled'          => { 'zapi' => 'is-interface-enabled',     '7glike' => ''},
        'address'          => { 'zapi' => 'ip-address',               '7glike' => ''},
        'ip-port'          => { 'zapi' => 'ip-port',                  '7glike' => ''},
        'curr-node'        => { 'zapi' => 'current-node',             '7glike' => ''},
        'curr-port'        => { 'zapi' => 'current-port',             '7glike' => ''},
        'tpgroup'          => { 'zapi' => 'tpgroup-name',             '7glike' => ''},
        'tpgroup-tag'      => { 'zapi' => 'tpgroup-tag',              '7glike' => ''},
        'relative-port-id' => { 'zapi' => 'relative-port-id',         '7glike' => ''},
        'query'            => 'iscsi-interface-list-entry-info',
        '7g-order'         => '',
        },
    'vserver_iscsi_portal_show' => {
        'cmd'              => { 'zapi' => 'iscsi-interface-get-iter', '7glike' => ''},
        'lif'              => { 'zapi' => 'interface-name',           '7glike' => ''},
        'enabled'          => { 'zapi' => 'is-interface-enabled',     '7glike' => ''},
        'address'          => { 'zapi' => 'ip-address',               '7glike' => ''},
        'ip-port'          => { 'zapi' => 'ip-port',                  '7glike' => ''},
        'curr-node'        => { 'zapi' => 'current-node',             '7glike' => ''},
        'curr-port'        => { 'zapi' => 'current-port',             '7glike' => ''},
        'tpgroup'          => { 'zapi' => 'tpgroup-name',             '7glike' => ''},
        'tpgroup-tag'      => { 'zapi' => 'tpgroup-tag',              '7glike' => ''},
        'relative-port-id' => { 'zapi' => 'relative-port-id',         '7glike' => ''},
        'query'            => 'iscsi-interface-list-entry-info',
        '7g-order'         => '',
        },
    'vserver_iscsi_session_show' => {
        'cmd'                    => { 'zapi' => 'iscsi-session-get-iter', '7glike' => ''},
        'tpgroup'                => { 'zapi' => 'tpgroup-name',           '7glike' => ''},
        'tsih'                   => { 'zapi' => 'target-session-id',      '7glike' => ''},
        'cmd-window-size'        => { 'zapi' => 'cmd-window-size',        '7glike' => ''},
        'data-pdu-in-order'      => { 'zapi' => 'data-pdu-in-order',      '7glike' => ''},
        'data-sequence-in-order' => { 'zapi' => 'data-sequence-in-order', '7glike' => ''},
        'default-time-to-retain' => { 'zapi' => 'default-time-to-retain', '7glike' => ''},
        'default-time-to-wait'   => { 'zapi' => 'default-time-to-wait',   '7glike' => ''},
        'error-recovery-level'   => { 'zapi' => 'error-recovery-level',   '7glike' => ''},
        'first-burst-length'     => { 'zapi' => 'first-burst-length',     '7glike' => ''},
        'immediate-data-enabled' => { 'zapi' => 'immediate-data-enabled', '7glike' => ''},
        'initiator-alias'        => { 'zapi' => 'initiator-aliasname',    '7glike' => ''},
        'initial-r2t-enabled'    => { 'zapi' => 'initial-r2t-enabled',    '7glike' => ''},
        'initiator-name'         => { 'zapi' => 'initiator-nodename',     '7glike' => ''},
        'max-burst-length'       => { 'zapi' => 'max-burst-length',       '7glike' => ''},
        'max-connections'        => { 'zapi' => 'max-connections',        '7glike' => ''},
        'max-outstanding-r2t'    => { 'zapi' => 'max-outstanding-r2t',    '7glike' => ''},
        'session-type'           => { 'zapi' => 'session-type',           '7glike' => ''},
        'tpgroup-tag'            => { 'zapi' => 'tpgroup-tag',            '7glike' => ''},
        'query'                  => 'iscsi-session-list-entry-info',
        '7g-order'               => '',
        },
    'vserver_iscsi_connection_show' => {
        'cmd'              => { 'zapi' => 'iscsi-connection-get-iter', '7glike' => ''},
        'tpgroup'          => { 'zapi' => 'tpgroup-name',              '7glike' => ''},
        'tsih'             => { 'zapi' => 'session-id',                '7glike' => ''},
        'connection-id'    => { 'zapi' => 'connection-id',             '7glike' => ''},
        'connection-state' => { 'zapi' => 'connection-state',          '7glike' => ''},
        'has-session'      => { 'zapi' => 'has-session',               '7glike' => ''},
        'lif'              => { 'zapi' => 'interface-name',            '7glike' => ''},
        'tpgroup-tag'      => { 'zapi' => 'tpgroup-tag',               '7glike' => ''},
        'local-address'    => { 'zapi' => 'local-ip-address',          '7glike' => ''},
        'local-ip-port'    => { 'zapi' => 'local-ip-port',             '7glike' => ''},
        'remote-address'   => { 'zapi' => 'remote-ip-address',         '7glike' => ''},
        'remote-ip-port'   => { 'zapi' => 'remote-ip-port',            '7glike' => ''},
        'query'            => 'iscsi-connection-list-entry-info',
        '7g-order'         => '',
        },
    'vserver_iscsi_initiator_show' => {
        'cmd'             => { 'zapi' => 'iscsi-initiator-get-iter',                             '7glike' => ''},
        'tpgroup'         => { 'zapi' => 'tpgroup-name',                                         '7glike' => ''},
        'tsih'            => { 'zapi' => 'target-session-id',                                    '7glike' => ''},
        'initiator-name'  => { 'zapi' => 'initiator-nodename',                                   '7glike' => ''},
        'initiator-alias' => { 'zapi' => 'initiator-aliasname',                                  '7glike' => ''},
        'tpgroup-tag'     => { 'zapi' => 'tpgroup-tag',                                          '7glike' => ''},
        'igroup'          => { 'zapi' => {'initiator-group-list-info' =>'initiator-group-name'}, '7glike' => ''},
        'query'           => 'iscsi-initiator-list-entry-info',
        '7g-order'        => '',
        },
    'vserver_iscsi_tpgroup_show' => {
        'cmd'             => { 'zapi' => 'iscsi-tpgroup-get-iter', '7glike' => ''},
        'tpgroup'         => { 'zapi' => 'tpgroup-name',           '7glike' => ''},
        'tpgroup-tag'     => { 'zapi' => 'tpgroup-tag',            '7glike' => ''},
        'lif'             => { 'zapi' => {'interface-list-entries'=>{'interface-list-entry-info'=>'interface-name'} }, '7glike' => ''},
        'is-user-defined' => { 'zapi' => 'is-user-defined',        '7glike' => ''},
        'query'       => 'iscsi-tpgroup-list-entry-info',
        '7g-order'    => '',
        },
    'vserver_iscsi_tpgroup_add' => {
        'cmd' => { 'zapi' => 'iscsi-tpgroup-interface-add', '7glike' => ''},
        'lif' => { 'zapi' => 'interface-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_tpgroup_remove' => {
        'cmd' => { 'zapi' => 'iscsi-tpgroup-interface-remove', '7glike' => ''},
        'lif' => { 'zapi' => 'interface-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_tpgroup_create' => {
        'cmd' => { 'zapi' => 'iscsi-tpgroup-create', '7glike' => ''},
        'server' => { 'zapi' => 'vserver', '7glike' => ''},
        'tpgroup' => { 'zapi' => 'tpgroup-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_tpgroup_delete' => {
        'cmd' => { 'zapi' => 'iscsi-tpgroup-destroy', '7glike' => ''},
        'server' => { 'zapi' => 'vserver', '7glike' => ''},
        'tpgroup-tag' => { 'zapi' => 'tpgroup-tag', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_nodename' => {
        'cmd' => { 'zapi' => 'iscsi-node-set-name', '7glike' => ''},
        'target-name'=> { 'zapi' => 'node-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_nodename_get' => {
        'cmd' => { 'zapi' => 'iscsi-node-get-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_isns_create' => {
        'cmd'          => { 'zapi' => 'iscsi-isns-create', '7glike' => ''},
        'address'      => { 'zapi' => 'isns-ip-addr',      '7glike' => ''},
        'status-admin' => { 'zapi' => 'start',             '7glike' => '', 'up' => 'true', 'down' => 'false'},
        'force'        => { 'zapi' => 'force',             '7glike' => ''},
        '7g-order'     => '',
        },
    'vserver_iscsi_isns_delete' => {
        'cmd'      => { 'zapi' => 'iscsi-isns-destroy', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_isns_modify' => {
        'cmd'          => { 'zapi' => 'iscsi-isns-config', '7glike' => ''},
        'address'      => { 'zapi' => 'isns-ip-addr',      '7glike' => ''},
        'status-admin' => { 'zapi' => '',                  '7glike' => ''},
        'force'        => { 'zapi' => 'force',             '7glike' => ''},
        '7g-order'     => '',
        },
    'vserver_iscsi_isns_show' => {
        'cmd'                    => { 'zapi' => 'iscsi-isns-get-iter',              '7glike' => ''},
        'address'                => { 'zapi' => 'isns-ip-addr',                     '7glike' => ''},
        'status-admin'           => { 'zapi' => 'is-isns-enabled',                  '7glike' => '', 'up' => 'true', 'down' => 'false'},
        'entity-id'              => { 'zapi' => 'isns-entity-id',                   '7glike' => ''},
        'last-successful-update' => { 'zapi' => 'last-successful-update-timestamp', '7glike' => ''},
        'last-update-attempt'    => { 'zapi' => 'last-update-attempt-timestamp',    '7glike' => ''},
        'last-update-result'     => { 'zapi' => 'last-update-result',               '7glike' => ''},
        'query'                  => 'iscsi-isns-info',
        '7g-order'               => '',
        },
    'vserver_iscsi_isns_start' => {
        'cmd'      => { 'zapi' => 'iscsi-isns-start', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_isns_stop' => {
        'cmd'      => { 'zapi' => 'iscsi-isns-stop', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_iscsi_isns_update' => {
        'cmd'      => { 'zapi' => 'iscsi-isns-update', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_show' => {
        'cmd' => { 'zapi' => 'fcp-service-get-iter', '7glike' => ''},
        'query' => 'fcp-service-info',
        '7g-order' => '',
        },
    'vserver_fcp_start' => {
        'cmd' => { 'zapi' => 'fcp-service-start', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_stop' => {
        'cmd' => { 'zapi' => 'fcp-service-stop', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_status' => {
        'cmd' => { 'zapi' => 'fcp-service-status', '7glike' => ''},
        'status-admin' => { 'zapi' => '', '7glike' => ''},
        'is-available' => { 'zapi' => '', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_create' => {
        'cmd' => { 'zapi' => 'fcp-service-create', '7glike' => ''},
        'status-admin' => { 'zapi' => 'start', '7glike' => '', 'up' => 'true', 'down' => 'false'},
        'target-name' => { 'zapi' => 'node-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_delete' => {
        'cmd' => { 'zapi' => 'fcp-service-destroy', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_initiator_show' => {
        'cmd'   => { 'zapi' => 'fcp-initiator-get-iter',                              '7glike' => ''},
        'alias' => { 'zapi' => 'portname-alias',                                      '7glike' => ''},
        'wwnn'  => { 'zapi' => {'fcp-connected-initiators' => 'node-name'},           '7glike' => ''},
        'igroup'=> { 'zapi' => {'initiator-group-list-info' =>'initiator-group-name'},'7glike' => ''},
        'query' => 'fcp-adapter-initiators-info',
        '7g-order' => '',
        },
    'vserver_fcp_nodename' => {
        'cmd' => { 'zapi' => 'fcp-node-set-name', '7glike' => ''},
        'target-name'=> { 'zapi' => 'node-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_nodename_get' => {
        'cmd' => { 'zapi' => 'fcp-node-get-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_wwpn_alias_show' => {
        'cmd' => { 'zapi' => 'fcp-wwpnalias-get-iter', '7glike' => ''},
        'alias' => {'zapi' => 'aliases-alias', '7glike' => ''},
        'wwpn' => {'zapi' => 'aliases-wwpn', '7glike' => ''},
        'query' => 'aliases-info',
        '7g-order' => '',
        },
    'vserver_fcp_wwpn_alias_set' => {
        'cmd' => { 'zapi' => 'fcp-wwpnalias-set', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_wwpn_alias_remove' => {
        'cmd' => { 'zapi' => 'fcp-wwpnalias-remove', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_portname_show' => {
        'cmd' => { 'zapi' => 'fcp-port-name-get-iter', '7glike' => ''},
        'query' => 'fcp-port-name-info',
        '7g-order' => '',
        },
    'vserver_fcp_portname_set' => {
        'cmd' => { 'zapi' => 'fcp-port-name-set', '7glike' => ''},
        'lif' => { 'zapi' => 'fcp-adapter', '7glike' => ''},
        'wwpn' => { 'zapi' => 'port-name', '7glike' => ''},
        '7g-order' => '',
        },
    'vserver_fcp_interface_show' => {
        'cmd'          => { 'zapi' => 'fcp-interface-get-iter','7glike' => ''},
        'lif'          => { 'zapi' => 'interface-name',     '7glike' => ''},
        'wwpn'         => { 'zapi' => 'port-name',          '7glike' => ''},
        'wwnn'         => { 'zapi' => 'node-name',          '7glike' => ''},
        'port-address' => { 'zapi' => 'port-address',       '7glike' => ''},
        'curr-node'       => { 'zapi' => 'current-node',    '7glike' => ''},
        'curr-port'       => { 'zapi' => 'current-port',    '7glike' => ''},
        'relative-port-id'=> { 'zapi' => 'relative-port-id','7glike' => ''},
        'query'    => 'fcp-interface-info',
        '7g-order' => '',
        },
    'network_interface_create' => {
        'cmd' => { 'zapi' => 'net-interface-create', '7glike' => ''},
        'lif' => { 'zapi' => 'interface-name', '7glike' => ''},
        'vserver' => { 'zapi' => 'vserver', '7glike' => ''},
        'data-protocol' => { 'zapi' => {'data-protocols' => 'data-protocol'}, '7glike' => ''},
        'routing-group' => { 'zapi' => 'routing-group-name', '7glike' => ''},
        'status-admin' => { 'zapi' => 'administrative-status', '7glike' => ''},
        'auto-revert' => { 'zapi' => 'is-auto-revert', '7glike' => ''},
        'dns-zone' => { 'zapi' => 'dns-domain-name', '7glike' => ''},
        'allow-lb-migrate' => { 'zapi' => '', '7glike' => ''},
        'lb-weight' => { 'zapi' => '', '7glike' => ''},
        'wwpn'                 => { 'zapi' => '',                                    '7glike' => ''},
        '7g-order' => '',
        },
    'network_interface_delete' => {
        'cmd' => { 'zapi' => 'net-interface-delete', '7glike' => ''},
        'lif' => { 'zapi' => 'interface-name', '7glike' => ''},
        'server' => { 'zapi' => 'vserver', '7glike' => ''},
        '7g-order' => '',
        },
    'network_interface_show' => {
        'cmd'                  => { 'zapi' => 'net-interface-get-iter',              '7glike' => ''},
        'lif'                  => { 'zapi' => 'interface-name',                      '7glike' => ''},
        'server'               => { 'zapi' => 'vserver',                             '7glike' => ''},
        'vserver'              => { 'zapi' => 'vserver',                             '7glike' => ''},
        'role'                 => { 'zapi' => 'role',                                '7glike' => ''},
        'data-protocol'        => { 'zapi' => {'data-protocols' => 'data-protocol'}, '7glike' => ''},
        'home-node'            => { 'zapi' => 'home-node',                           '7glike' => ''},
        'home-port'            => { 'zapi' => 'home-port',                           '7glike' => ''},
        'curr-node'            => { 'zapi' => 'current-node',                        '7glike' => ''},
        'curr-port'            => { 'zapi' => 'current-port',                        '7glike' => ''},
        'status-oper'          => { 'zapi' => 'operational-status',                  '7glike' => ''},
        'status-extended'      => { 'zapi' => '',                                    '7glike' => ''},
        'numeric-id'           => { 'zapi' => '',                                    '7glike' => ''},
        'is-home'              => { 'zapi' => 'is-home',                             '7glike' => ''},
        'address'              => { 'zapi' => 'address',                             '7glike' => ''},
        'netmask'              => { 'zapi' => 'netmask',                             '7glike' => ''},
        'netmask-length'       => { 'zapi' => 'netmask-length',                      '7glike' => ''},
        'auto'                 => { 'zapi' => 'is-ipv4-link-local',                  '7glike' => ''},
        'routing-group'        => { 'zapi' => 'routing-group-name',                  '7glike' => ''},
        'status-admin'         => { 'zapi' => 'administrative-status',               '7glike' => ''},
        'failover-policy'      => { 'zapi' => 'failover-policy',                     '7glike' => ''},
        'firewall-policy'      => { 'zapi' => 'firewall-policy',                     '7glike' => ''},
        'auto-revert'          => { 'zapi' => 'is-auto-revert',                      '7glike' => ''},
        'sticky'               => { 'zapi' => '',                                    '7glike' => ''},
        'dns-zone'             => { 'zapi' => 'dns-domain-name ',                    '7glike' => ''},
        'listen-for-dns-query' => { 'zapi' => 'listen-for-dns-query',                '7glike' => ''},
        'allow-lb-migrate'     => { 'zapi' => '',                                    '7glike' => ''},
        'lb-weight'            => { 'zapi' => '',                                    '7glike' => ''},
        'failover-group'       => { 'zapi' => 'failover-group',                  '7glike' => ''},
        'wwpn'                 => { 'zapi' => '',                                    '7glike' => ''},
        'address-family'       => { 'zapi' => 'address-family',                      '7glike' => ''},
        'comment'              => { 'zapi' => 'comment',                             '7glike' => ''},
        'query'                => 'net-interface-info',
        '7g-order'             => '',
        },
    'network_interface_modify' => {
        'cmd' => { 'zapi' => 'net-interface-modify', '7glike' => ''},
        'lif' => { 'zapi' => 'interface-name', '7glike' => ''},
        'server' => { 'zapi' => 'vserver', '7glike' => ''},
        'auto' => {'zapi' => 'is-ipv4-link-local', '7glike' => ''},
        'routing-group' => {'zapi' => 'routing-group-name', '7glike' => ''},
        'status-admin' => {'zapi' => 'administrative-status', '7glike' => ''},
        'auto-revert' => {'zapi' => 'is-auto-revert', '7glike' => ''},
        'dns-zone' => {'zapi' => 'dns-domain-name', '7glike' => ''},
        'allow-lb-migrate'     => { 'zapi' => '',                                    '7glike' => ''},
        'lb-weight'            => { 'zapi' => '',                                    '7glike' => ''},
        'wwpn'                 => { 'zapi' => '',                                    '7glike' => ''},
        '7g-order' => '',
        },
    # FYI - This call is hard-coded to cli in VserverHelper.pm.  There are not
    # zapis for routing group cmds.  Note the cmd listed below is for routes.
    # routing-group zapis may arrive in SN0.
    'network_routing_groups_show' => {
        'cmd'     => {'zapi' => 'net-routing-group-route-get-iter','7glike' => ''},
        'vserver' => {'zapi' => 'vserver',                         '7glike' => ''},
        'query'   => 'routing-group-route-info',
        '7g-order'=> '',
        },
    # FYI - This call is hard-coded to cli in VserverHelper.pm & Lif.pm.  There are not
    # zapis for routing group cmds.  Note the cmd listed below is for routes.
    # routing-group zapis may arrive in SN0.
    'network_routing_groups_delete' => {
        'cmd'        => {'zapi' => 'net-routing-group-route-destroy','7glike' => ''},
        'destination'=> {'zapi' => 'destination-address',            '7glike' => ''},
        '7g-order'   => '',
        },
    'network_routing_groups_route_show' => {
        'cmd'        => {'zapi' => 'net-routing-group-route-get-iter','7glike' => ''},
        'destination'=> {'zapi' => 'destination-address',             '7glike' => ''},
        'query'      => 'routing-group-route-info',
        '7g-order'=> '',
        },
    'network_routing_groups_route_create' => {
        'cmd'          => {'zapi' => 'net-routing-group-route-create','7glike' => ''},
        'destination'  => {'zapi' => 'destination-address',           '7glike' => ''},
        'gateway'      => {'zapi' => 'gateway-address',               '7glike' => ''},
        'routing-group'=> {'zapi' => 'routing-group',                 '7glike' => ''},
        '7g-order' => '',
        },
    'network_routing_groups_route_delete' => {
        'cmd'        => {'zapi' => 'net-routing-group-route-destroy', '7glike' => ''},
        'destination'=> {'zapi' => 'destination-address',             '7glike' => ''},
        '7g-order'   => '',
        },
    'network_interface_failover_create' => {
        'cmd' => { 'zapi' => 'net-interface-failover-create', '7glike' => ''},
        '7g-order' => '',
        },
    'network_interface_failover_delete' => {
        'cmd' => { 'zapi' => 'net-interface-failover-delete', '7glike' => ''},
        '7g-order' => '',
        },
    'network_interface_failover_modify' => {
        'cmd' => { 'zapi' => 'net-interface-failover-modify', '7glike' => ''},
        '7g-order' => '',
        },
    'network_port_show' => {
        'cmd'                => { 'zapi' => 'net-port-get-iter',               '7glike' => ''},
        'duplex-admin'       => { 'zapi' => 'administrative-duplex',           '7glike' => ''},
        'flowcontrol-admin'  => { 'zapi' => 'administrative-flowcontrol',      '7glike' => ''},
        'speed-admin'        => { 'zapi' => 'administrative-speed',            '7glike' => ''},
        'ifgrp-distr-func'   => { 'zapi' => 'ifgrp-distribution-function',     '7glike' => ''},
        'autonegotiate-admin'=> { 'zapi' => 'is-administrative-auto-negotiate','7glike' => ''},
        'up-admin'           => { 'zapi' => 'is-administrative-up',            '7glike' => ''},
        'autonegotiate-oper' => { 'zapi' => 'is-operational-auto-negotiate',   '7glike' => ''},
        'link'               => { 'zapi' => 'link-status',                     '7glike' => ''},
        'mac'                => { 'zapi' => 'mac-address',                     '7glike' => ''},
        'duplex-oper'        => { 'zapi' => 'operational-duplex',              '7glike' => ''},
        'flowcontrol-oper'   => { 'zapi' => 'operational-flowcontrol',         '7glike' => ''},
        'speed-oper'         => { 'zapi' => 'operational-speed',               '7glike' => ''},
        'type'               => { 'zapi' => 'port-type',                       '7glike' => ''},
        'vlan-tag'           => { 'zapi' => 'vlan-id',                         '7glike' => ''},
        'query'    => 'net-port-info',
        '7g-order' => '',
        },
    'network_port_modify' => {
        'cmd'                => { 'zapi' => 'net-port-modify',                 '7glike' => ''},
        'duplex-admin'       => { 'zapi' => 'administrative-duplex',           '7glike' => ''},
        'flowcontrol-admin'  => { 'zapi' => 'administrative-flowcontrol',      '7glike' => ''},
        'speed-admin'        => { 'zapi' => 'administrative-speed',            '7glike' => ''},
        'autonegotiate-admin'=> { 'zapi' => 'is-administrative-auto-negotiate','7glike' => ''},
        'up-admin'           => { 'zapi' => 'is-administrative-up',            '7glike' => ''},
        '7g-order'           => '',
        },
    'volume_create' => {
        'cmd' => { 'zapi' => 'volume-create', '7glike' => ''},
        'aggregate' => { 'zapi' => 'containing-aggr-name', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_delete' => {
        'cmd' => { 'zapi' => 'volume-destroy', '7glike' => ''},
        'volume' => { 'zapi' => 'name', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_show' => {
        'cmd' => { 'zapi' => 'volume-get-iter', '7glike' => ''},
        'query' => { 'volume-attributes' => 'volume-id-attributes' },
        '7g-order' => '',
        },
    'volume_modify' => {
        'cmd' => { 'zapi' => 'volume-modify-iter', '7glike' => ''},
        'volume' => { 'zapi' => 'name', '7glike' => ''},
        'state' => { 'zapi' => { 'volume-state-attributes' => 'state' }, '7glike' => ''},
        'percent-snapshot-space' => { 'zapi' => { 'volume-space-attributes' => 'percentage-snapshot-reserved' }, '7glike' => ''},
        'query' => { 'volume-attributes' => 'volume-id-attributes' },
        '7g-order' => '',
        },
    'volume_efficiency_off' => {
        'cmd' => { 'zapi' => 'sis-disable', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_efficiency_on' => {
        'cmd' => { 'zapi' => 'sis-enable', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_efficiency_modify' => {
        'cmd' => { 'zapi' => 'sis-set-config', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_efficiency_show' => {
        'cmd' => { 'zapi' => 'sis-get-iter', '7glike' => ''},
        'query' => 'sis-status-info',
        '7g-order' => '',
        },
    'volume_mount' => {
        'cmd' => { 'zapi' => 'volume-mount', '7glike' => ''},
        'volume' => { 'zapi' => 'volume-name', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_unmount' => {
        'cmd' => { 'zapi' => 'volume-unmount', '7glike' => ''},
        'volume' => { 'zapi' => 'volume-name', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_move_start' => {
        'cmd' => { 'zapi' => 'volume-move-start', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_online' => {
        'cmd' => { 'zapi' => 'volume-online', '7glike' => ''},
        'volume' => { 'zapi' => 'name', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_offline' => {
        'cmd' => { 'zapi' => 'volume-offline', '7glike' => ''},
        'volume' => { 'zapi' => 'name', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_file_clone_create' => {
        'cmd'  => { 'zapi' => 'clone-create', '7glike' => ''},
        'range'=> { 'zapi' => 'block-ranges', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_add' => {
        'cmd' => { 'zapi' => 'igroup-add', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        'initiator' => { 'zapi' => 'initiator', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_create' => {
        'cmd' => { 'zapi' => 'igroup-create', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        'protocol' => { 'zapi' => 'initiator-group-type', '7glike' => ''},
        'ostype' => { 'zapi' => 'os-type', '7glike' => ''},
        'portset' => { 'zapi' => 'bind-portset', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_remove' => {
        'cmd' => { 'zapi' => 'igroup-remove', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_rename' => {
        'cmd' => { 'zapi' => 'igroup-rename', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        'new-name' => { 'zapi' => 'initiator-group-new-name', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_delete' => {
        'cmd' => { 'zapi' => 'igroup-destroy', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_show' => {
        'cmd' => { 'zapi' => 'igroup-get-iter', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        'query' => 'initiator-group-info',
        '7g-order' => '',
        },
    'lun_igroup_set' => {
        'cmd' => { 'zapi' => 'igroup-set-attribute', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_modify' => {
        'cmd' => { 'zapi' => 'igroup-set-attribute', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        'ostype' => { 'zapi' => 'os-type', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_bind' => {
        'cmd' => { 'zapi' => 'igroup-bind-portset', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        'portset' => { 'zapi' => 'portset-name', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_igroup_unbind' => {
        'cmd' => { 'zapi' => 'igroup-unbind-portset', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group-name', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_portset_create' => {
        'cmd' => { 'zapi' => 'portset-create', '7glike' => 'lun_portset_create'},
        'portset' => { 'zapi' => 'portset-name', '7glike' => ''},
        'protocol' => { 'zapi' => 'portset-type', '7glike' => '-i|-f'},
        '7g-order' => '',
        },
    'lun_portset_add' => {
        'cmd' => { 'zapi' => 'portset-add', '7glike' => 'lun_portset_add'},
        'portset' => { 'zapi' => 'portset-name', '7glike' => ''},
        'port-name' => { 'zapi' => 'portset-port-name', '7glike' => '-f'},
        '7g-order' => '',
        },
    'lun_portset_delete' => {
        'cmd' => { 'zapi' => 'portset-destroy', '7glike' => 'lun_portset_destroy'},
        'portset' => { 'zapi' => 'portset-name', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_portset_remove' => {
        'cmd' => { 'zapi' => 'portset-remove', '7glike' => 'lun_portset_remove'},
        'portset' => { 'zapi' => 'portset-name', '7glike' => ''},
        'port-name' => { 'zapi' => 'portset-port-name', '7glike' => '-f'},
        'tpg' => { 'zapi' => 'portset-tpg-tag', '7glike' => '-i'},
        '7g-order' => '',
        },
    'lun_portset_show' => {
        'cmd' => { 'zapi' => 'portset-get-iter', '7glike' => 'lun_portset_show'},
        'portset' => { 'zapi' => 'portset-name', '7glike' => ''},
        'protocol' => { 'zapi' => 'portset-type', '7glike' => '-i|-f'},
        'fc-lif' => { 'zapi' => 'portset-port-name', '7glike' => '-f'},
        'tpg' => { 'zapi' => 'portset-tpg-tag', '7glike' => '-i'},
        'query' => 'portset-info',
        '7g-order' => '',
        },
    'lun_create' => {
        'cmd' => { 'zapi' => 'lun-create-by-size', '7glike' => 'lun_create'},
        'ostype' => { 'zapi' => 'ostype', '7glike' => '-t'},
        'path' => { 'zapi' => 'path', '7glike' => '-p'},
        'space-reserve' => { 'zapi' => 'space-reservation-enabled', '7glike' => ''},
        '7g-order' => '-t -p',
        },
    'lun_create_from_file' => {
        'cmd' => { 'zapi' => 'lun-create-from-file', '7glike' => 'lun_create'},
        'ostype' => { 'zapi' => 'ostype', '7glike' => '-t'},
        'path' => { 'zapi' => 'path', '7glike' => '-p'},
        'space-reserve' => { 'zapi' => 'space-reservation-enabled', '7glike' => ''},
        'file-path' => { 'zapi' => 'file-name', '7glike' => ''},
        '7g-order' => '-t -p',
        },
    'lun_delete' => {
        'cmd' => { 'zapi' => 'lun-destroy', '7glike' => ''},
        'lun-path' => { 'zapi' => 'path', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_map' => {
        'cmd' => { 'zapi' => 'lun-map', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_unmap' => {
        'cmd' => { 'zapi' => 'lun-unmap', '7glike' => ''},
        'igroup' => { 'zapi' => 'initiator-group', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_online' => {
        'cmd' => { 'zapi' => 'lun-online', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_offline' => {
        'cmd' => { 'zapi' => 'lun-offline', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_show' => {
        'cmd' => { 'zapi' => 'lun-get-iter', '7glike' => ''},
        'query' => 'lun-info',
        '7g-order' => '',
        },
    'lun_mapped_show' => {
        'cmd' => { 'zapi' => 'lun-map-get-iter', '7glike' => ''},
        'query' => 'lun-map-info',
        '7g-order' => '',
        },
    'lun_move' => {
        'cmd' => { 'zapi' => 'lun-move', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_resize' => {
        'cmd' => { 'zapi' => 'lun-resize', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_comment' => {
        'cmd' => { 'zapi' => 'lun-get-comment', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_set_comment' => {
        'cmd'  => { 'zapi' => 'lun-set-comment','7glike' => ''},
        'path' => { 'zapi' => 'path',           '7glike' => ''},
        '7g-order' => '',
        },
    'lun_set_reservation' => {
        'cmd'          => { 'zapi' => 'lun-set-space-reservation-info', '7glike' => ''},
        'space-reserve'=> { 'zapi' => 'enable',                         '7glike' => ''},
        '7g-order' => '',
        },
    'lun_get_reservation' => {
        'cmd' => { 'zapi' => 'lun-get-space-reservation-info','7glike' => ''},
        'path'=> { 'zapi' => 'path',                          '7glike' => ''},
        '7g-order' => '',
        },
    'lun_serial' => {
        'cmd' => { 'zapi' => 'lun-get-serial-number', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_set_serial' => {
        'cmd'   => { 'zapi' => 'lun-set-serial-number','7glike' => ''},
        'path'  => { 'zapi' => 'path',                 '7glike' => ''},
        'serial'=> { 'zapi' => 'serial-number',        '7glike' => ''},
        '7g-order' => '',
        },
    'lun_set_device_id' => {
        'cmd' => { 'zapi' => 'lun-set-device-id', '7glike' => ''},
        'type' => { 'zapi' => 'device-id-type', '7glike' => ''},
        'device-binary-id' => { 'zapi' => 'device-id', '7glike' => ''},
        'device-text-id' => { 'zapi' => 'device-id', '7glike' => ''},
        'device-legacy-id' => { 'zapi' => 'device-id', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_unset_device_id' => {
        'cmd' => { 'zapi' => 'lun-unset-device-id', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_maxsize' => {
        'cmd' => { 'zapi' => 'lun-get-maxsize', '7glike' => ''},
        'path' => { 'zapi' => 'path', '7glike' => ''},
        'ostype' => { 'zapi' => 'type', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_minsize' => {
        'cmd' => { 'zapi' => 'lun-get-minsize', '7glike' => ''},
        'ostype' => { 'zapi' => 'type', '7glike' => ''},
        '7g-order' => '',
        },
    'lun_geometry' => {
        'cmd' => { 'zapi' => 'lun-get-geometry', '7glike' => ''},
        },
    'lun_stats' => {
        'cmd' => { 'zapi' => 'lun-stats-get-iter', '7glike' => ''},
        'query' => 'lun-stats-info',
        '7g-order' => '',
        },
    'volume_qtree_create' => {
        'cmd' => { 'zapi' => 'qtree-create', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_qtree_delete' => {
        'cmd' => { 'zapi' => 'qtree-delete', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_qtree_show' => {
        'cmd'         => { 'zapi' => 'qtree-list-iter', '7glike' => ''},
        'query'       => 'qtree-info',
        'oplock-mode' => {'zapi'  => 'oplocks', '7glike' => ''},
        'qtree-path'  => { 'zapi' => 'qtree', '7glike'   => ''},
        '7g-order'    => '',
        },
    'volume_qtree_rename' => {
        'cmd' => { 'zapi' => 'qtree-rename', '7glike' => ''},
        'newname' => { 'zapi' => 'new-qtree-name', '7glike' => ''},
        'qtree-path' => { 'zapi' => 'qtree', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_create' => {
        'cmd' => { 'zapi' => 'snapshot-create', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_delete' => {
        'cmd' => { 'zapi' => 'snapshot-delete', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_show' => {
        'cmd' => { 'zapi' => 'snapshot-get-iter', '7glike' => ''},
        'query' => 'snapshot-info',
        '7g-order' => '',
        },
    'volume_snapshot_rename' => {
        'cmd' => { 'zapi' => 'snapshot-rename', '7glike' => ''},
        'snapshot' => { 'zapi' => 'current-name', '7glike' => ''},
        'new-name' => { 'zapi' => 'new-name', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_restore' => {
        'cmd' => { 'zapi' => 'snapshot-restore-volume', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_restore_file' => {
        'cmd' => { 'zapi' => 'snapshot-restore-file', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_restore_file_info' => {
        'cmd' => { 'zapi' => 'snapshot-restore-file-info', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_partial_restore_file' => {
        'cmd' => { 'zapi' => 'snapshot-partial-restore-file', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_partial_restore_file_list_info' => {
        'cmd' => { 'zapi' => 'snapshot-partial-restore-file-list-info', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_autodelete_show' => {
        'cmd' => { 'zapi' => 'snapshot-autodelete-list-info', '7glike' => ''},
        '7g-order' => '',
        },
    'volume_snapshot_autodelete_modify' => {
        'cmd' => { 'zapi' => 'snapshot-autodelete-set-option', '7glike' => ''},
        '7g-order' => '',
        },
);

1;

#############################################################################
#                             POD Documentation
#############################################################################
## @pod here

=pod

=head1 Package Name

TranslationTable.pm

=head1 Author

Korry Nguyen (dl-ssan-test)

=head1 Description

TranslationTable.pm provides a hash table that mapped CDEF commands and options to ZAPI/7Glike commands and options. This allows libraries/test scripts to be written agnostically, meaning tests can run on 7mode and Cmode.

=head1 See Also

See CmdsDriver.pm for further interface detail.

=head1 Usage

=over 4

=item Examples

  #
  # Create a CmdsDriver object using filer name
  #
  $cmd_obj = new CmdsDriver(-log         => \*ntest::NTEST_STDOUT,
                            -filer       => "rtpqa-fas2020-3",
                            -debug_level => $debug_level);
  if ($cmd_obj->error())
  {
      logresult('FATAL', "Failed to instantiate CmdsDriver object" . $cmd_obj->errmsg());
  }

  my $cmd = ($cmd_obj->filer_mode() =~ /7mode/i) : "lun-create-by-size" ? "lun_create_bysize";
  $resultsHash_ref =
    $cmd_obj->execute_cmd(
        -cmd               => $cmd,
        -cmd_filer         => $filer,
        -cmd_filer_version => $version,
        -cmd_interface     => "cli",
        -size              => "100m",
        -type              => "linux",
        -check_output      => "yes",
        -check_result      => "yes",
        -expected_result   => "255 0",
        -path              => "/vol/cmdsTest/cli_lun_100m" . "$$");

  if ($cmd_obj->error())
  {
      my $errmsg = join "\n", $cmd_obj->errmsg();
      $Test->nlog("ERROR: Failed calling execute_cmd: " . $errmsg);
      return $TCD::FAIL;
  }

  #
  # To run the above execute_cmd on Cmode, the 'lun-create-by-size'
  # hash must be created in the %mapped_cmd hash table as shown below:
  #
  %mapped_cmds = (
      #
      # Cmode CDEF                 # ZAPI/7Glike
      #
      'lun_create_bysize' => {
         'cmd' => { 'zapi' => 'lun-create-by-size', '7glike' => 'lun_create_bysize'},
         'ostype' => { 'zapi' => 'type', '7glike' => '-t'},
         'lun-path' => { 'zapi' => 'path', '7glike' => '-p'},
         '7g-order' => '-t -p',
         },
  );

  ADDING arguments TO TRANSLATION TABLE

  Introduction.
  TranslationTable is, basically, a Hash (defined as %mapped_cmds) that holds hashes
  of most CLI commands. The name of the command (in CLI) is the key for each sub-hash.
  The sub-hash or value of the cli_command_key has all the information needed for
  translating from CLI to either ZAPI or 7Mode.

  Structure.
  1. The key of the command hash (sub-hash) most be the name of the command.
  2. The command hash (sub-hash) most contain a hash with keys. This keys are usually
     the arguments of the command in CLI.
  3. Each sub-command hash (the hash that is hold by each key, under the command key)
     most have a two keys, 'zapi' and '7glike'.
  4. Each command hash (sub-hash) most have a 'cmd' key holding the equivalent command in
     either ZAPI or 7Mode.
  5. The value of the ZAPI key is the argument name (or nested structure) of the ZAPI equivalent
     to the CLI argument.

     * In case the argument is nested under more than one argument the following structure
      most be constructed and passed in as value of the ZAPI key.

        Example 1: Nested under one argument
        We need to translate the argument 'ns-switch (cli) to nsswitch (ZAPI)
        for command 'vserver_create'. Argument 'nsswitch' is defined under
        argument 'name-server-switch'.

        So, the XML will look like this:
            <name-server-switch>
                    <nsswitch>argument_value</nsswitch>
            </name-server-switch>

        And we want to translate it to:

            'name-server-switch' => {
                                     'nsswitch' => 'argument_value'
                                    },

        So, on translation table the value hash key-value pair most be:
            'ns-switch' => {'zapi' => { 'name-server-switch' => 'nsswitch'}, ...}


        Example 2: Nested under more than one argument
        We need to translate the argument LIF (cli) to interface-name (ZAPI) that
        is nested under argument 'interface-list-entry-info' that is under argument
        'interface-list-entries'

        So, the XML will look like this:
            <interface-list-entries>
                    <interface-list-entry-info>
                            <interface-name>argument_value</interface-name>
                    </interface-list-entry-info>
            </interface-list-entries>

        And we want to translate it to:

            'interface-list-entries' => {
                                         'interface-list-entry-info' => {
                                                                         'interface-name' => 'argument_value'
                                                                        }
                                        }

        So, on translation table the value hash key-value pair most be:
            'lif' => { 'zapi' => {'interface-list-entries'=>{'interface-list-entry-info'=>'interface-name'} },...}

  6. If all the arguments are nested under one single argument (in ZAPI), then the key
     'query' should be added and it's value is the name of the argument where
    the code will search (query) for the arguments.

    The ZAPI XML will look like this:
            <vserver-info>
                    <aggr-list>argument_value</aggr-list>
                    <nisdomain>argument_value</nisdomain>
                    <quota-policy>argument_value</quota-policy>
                    <language-str>argument_value</language-str>
                    <max-volumes>argument_value</max-volumes>
                    <uuid>argument_value</uuid>
                    <nm-switch>argument_value</nm-switch>
                    <vserver-name>argument_value</vserver-name>
                    <id>argument_value</id>
                    ....
            </vserver-info>

    INSTEAD of doing this for every argument:
        'vserver' => { 'zapi' => {'vserver-info'=>'vserver-name'} },...}

    JUST add the key 'query' and assign simple values to the arguments
        'vserver'=> { 'zapi' => 'vserver-name', '7glike' => ''},
        'query'  => 'vserver-info',

=cut
