#-*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Cmode/CifsAccess.pm#2 $
#  Copyright (c) 2011 Network Appliance, Inc.
#  All rights reserved.
#
## @summary Routines for cifs setup and data access using SIO tool
##
## @description
##	Functions for the cifs setup and data access using SIO tool automation
##      The library conatins the following subroutines : cifs_setup verify_traffic cleanup_shares filer_sysstat_get_cmode shares_create sio_write
##
## @author Rahula.Muthiah@netapp.com

package Cmode::CifsAccess;
use strict;
use warnings;
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use Tharn;
use Data::Dumper;
use Server;
use MapShare qw(map_cifs_share unmap_cifs_share);
use SayWrap qw(prompt_loop);
use base qw(Exporter);
use Sys::Hostname;
use Config::Cluster;
use Cmode::Cmode_Utils qw(Filer_saycmd);
use NACL::C::VserverCifs;
use NACL::STask::Client::Mount;
use constant LICENSE_CODE_CIFS => "JAMHCKPJYBFDAA";
use constant PROTOCOL          => "cifs";
use constant STATE_DOMAIN      => "enabled";
use constant CLIENT_MATCH      => "0.0.0.0/0";
use constant RORULE            => "any";
use constant RWRULE            => "any";
use constant SUPERUSER         => "any";
use constant DIRECTION_WU      => "win-unix";
use constant DIRECTION_UW      => "unix-win";
use constant POLICYNAME        => "default";
BEGIN {
    our @EXPORT =
      qw(cifs_setup cleanup_shares verify_traffic shares_create sio_write);
}
##Declaration of global variables
my ($drive);
## @Function
##      cifs_setup - Does the CIFS setup,creates the CIFS shares and maps it during SIO operation
##
## @usage
##         cifs_setup(server_object => $Server,
##            		domain        => $CIFS_DOMAIN ,
## 					vserver       => $vserver1,
##					win_client    => $WINCLIENT);
##
## @arguments
##      server_object     Required    Server object
##      domain            Required    CIFS Domain name
##      vserver           Required    Vserver name
##      win_client        Required    Windows Client on which shares are created
##      nameservers       Required    Nameservers ip addresses for dns services
##	user		  optional    The unix-user to create
##	cifs_server 	  optional    The server name to be created
##	unix_user_id  	  optional    The Unix-user id
##	unix_group_id 	  optional    The Unix-group id
##	position  	  optional    The position in the queue
##
## @scope
##      Exported from this module
##
sub cifs_setup {
    my (%opts) =
      validate(@_,
               {server_object => {type => OBJECT, isa => 'Server'},
                domain        => {type => SCALAR},
                vserver       => {type => SCALAR},
                win_client    => {type => SCALAR},
                nameservers   => {type => SCALAR},
                cifs_domain   => {type => SCALAR},
                user => {type => SCALAR, optional => 1, default => "root"},
                cifs_server =>
                  {type => SCALAR, optional => 1, default => 'VS2'},
                unix_user_id  => {type => SCALAR, default => '0'},
                unix_group_id => {type => SCALAR, default => '1'},
                position      => {type => SCALAR, default => '1'}});
    my $Server        = $opts{server_object};
    my $vserver       = $opts{vserver};
    my $domain        = $opts{domain};
    my $winclient     = $opts{win_client};
    my $user          = $opts{user};
    my $cifs_server   = $opts{cifs_server};
    my $unix_user_id  = $opts{unix_user_id};
    my $unix_group_id = $opts{unix_group_id};
    my $position      = $opts{position};
    my $nameservers   = $opts{nameservers};
    my $cifs_domain   = $opts{cifs_domain};
    my ($ch_cifs, $cc_cifs, $domain_ad, $host);
    eval {
        $host = host($winclient)
          or
          die("Configuration error while creating host object of $winclient");
        $cc_cifs = connect("#rsh")
          or
          die("Configuration error while connecting through rsh to $winclient");
        if ($cifs_domain =~ /([A-Za-z]+)\.[A-Za-z]+\.[A-Za-z]+/) {
            $domain_ad = $1;
        }
        $Log->comment("Checking if cifs license exists");
        eval { $Server->system_license_show(feature => "CIFS"); };
        if ($@ =~ /There are no entries matching your query/) {
            $Log->comment("CIFS license not exists creating...");
            $Server->system_license_add('license-code' => LICENSE_CODE_CIFS);
        } else {
            $Log->comment("CIFS license already exists");
        }
	 eval { $Server->vserver_export_policy_show(vserver => $vserver,
                                                   policyname =>POLICYNAME); };
        if ($@=~ /There are no entries matching your query/) {
            logcomment("Creating export-policy ");
            $Server->vserver_export_policy_create(vserver    => $vserver,
                                                  policyname => POLICYNAME);
        }else{
            $Log->comment("Export-policy POLICYNAME already exists for $vserver");
        }
		eval { $Server->vserver_export_policy_rule_show(vserver => $vserver,
                                                        protocol => PROTOCOL); };
		if($@ =~ /There are no entries matching your query/) {										   
            $Log->comment("Create export-policy rule");
            $Server->vserver_export_policy_rule_create(
                                                    vserver     => $vserver,
                                                    policyname  => POLICYNAME,
                                                    clientmatch => CLIENT_MATCH,
                                                    rorule      => RORULE,
                                                    rwrule      => RWRULE,
                                                    anon      => $unix_user_id,
                                                    protocol  => PROTOCOL,
                                                    superuser => SUPERUSER);
		}
        else {
		   $Log->comment("Export policy rule already exists for $vserver for the cifs protocol");
       	 }		 
        eval {
            $Server->vserver_services_dns_show(vserver => $vserver,
                                               domains => $domain);
        };
        if ($@ =~ /There are no entries matching your query/) {
            $Server->vserver_services_dns_create(vserver        => $vserver,
                                                 domains        => $domain,
                                                 state          => STATE_DOMAIN,
                                                 'name-servers' => $nameservers
                                                );
        } else {
            $Log->comment("DNS already configured for $vserver vserver");
        }
        #$Server->vserver_services_unix_user_create(
                                                 #vserver       => $vserver,
                                                 #user          => $user,
                                                 #id            => $unix_user_id,
                                                 #'primary-gid' => $unix_group_id
        #);

        #$Server->vserver_services_unix_group_create(vserver => $vserver,
                                                    #name    => $user,
                                                    #id      => $unix_group_id);

        $Server->vserver_name_mapping_create(
                              vserver   => $vserver,
                              direction => DIRECTION_WU,
                              position  => $position,
                              pattern => "$domain_ad\\\\$cc_cifs->{'username'}",
                              replacement => $user);

        $Server->vserver_name_mapping_create(
                           vserver     => $vserver,
                           direction   => DIRECTION_UW,
                           position    => $position,
                           pattern     => $user,
                           replacement => "$domain_ad\\\\$cc_cifs->{'username'}"
        );
        $Server->cli(
            cmd =>
              "cifs create -vserver $vserver -cifs-server $cifs_server -domain $cifs_domain -ou CN=Computers -default-site \"\" ",
            additional_prompts_timeout => 300,
            additional_prompts         => {
                              "Enter the user name.*" => $cc_cifs->{'username'},
                              "Enter the password"    => $cc_cifs->{'password'},
                              "(.*?)::(.*?)"          => 'FINAL_PROMPT'});
    };
    if ($@) {
        die("CIFS setup failed with error : $@");
    }
} ## end sub cifs_setup
##
##@Function
##    verify_traffic :: checks the protocol traffic
##
##@Usage
##    verify_traffic($conn,'CIFS');
##
##@Arguments
##    $obj        Reqiured   Connection object
##    $type       Required   Type of protocol
##
##@Return value
##    $avg    -   The average value of the particular type of traffic
##
##@Scope
##    Exported from this module
##
sub verify_traffic {
    my ($obj, $type) = @_;
    my ($avg, $end_time, %stats);
    $end_time = time() + 20;
    do {

        # Collect 4 samples of stats every 5 secs
        %stats = filer_sysstat_get_cmode($obj,
                                         {'FILER_SYSSTAT_COUNT'    => 4,
                                          'FILER_SYSSTAT_INTERVAL' => 5});
        if ($type =~ /CIFS/) {
            $avg = $stats{$type}{Avg};
        } elsif ($type =~ /NFS/) {
            $avg = $stats{$type}{Avg};
        } elsif ($type =~ /iSCSI/) {
            $avg = $stats{$type}{Avg};
        }
        logcomment(Dumper(\%stats));
    } while (($avg == 0) && (time() < $end_time));
    #return $avg;
     return 1;
} ## end sub verify_traffic
##
##@Function
##   cleanup_shares  :: cleans up the shares and stopes the SIO process
##
##@Usage
##   		cleanup_shares(drive     => $drive,
##		               winclient => $WINCLIENT,
##					   vserver   => $vserver1,
##					   server_obj=> $Server);
##
##@Arguments
##    drive           Reqiured   The drive name to which the shares are mapped
##    winclient       Required   The windows client
##    vserver         Required   The vserver name
##    server_obj      Required   Server object on which execution is done
##
##@Scope
##    Exported from this module
##
sub cleanup_shares {
    my (%opts) =
      validate(@_,
               {drive      => {type => SCALAR},
                winclient  => {type => SCALAR},
                vserver    => {type => SCALAR},
                server_obj => {type => OBJECT, isa => 'Server'}});
    my $drive     = $opts{drive};
    my $winclient = $opts{winclient};
    my $vserver   = $opts{vserver};
    my $Server    = $opts{server_obj};
    my $ch_cifs   = host($winclient)
      or die("Configuration error while creating host object of $winclient");
    my $cc_cifs = connect("#rsh")
      or die("Configuration error while connecting through rsh to $winclient");
    my $killsio = "TASKKILL /IM sio.exe";
    eval { $cc_cifs->execute(command => $killsio); };

    if ($@) {
        die("Task kill SIO operation failed with error : $@ ");
    }
    $Log->comment("Unmapping the share");
    eval {
        unmap_cifs_share(drive => $drive, client => $cc_cifs);
        $Server->vserver_cifs_delete(
                                      vserver   => $vserver,
                                      -userid   => $cc_cifs->{'username'},
                                     -password  => $cc_cifs->{'password'}
        );
    };
    if ($@) {
        die("CIFS unmapping shares and vserver cifs deletion failed with error : $@"
           );
    }

} ## end sub cleanup_shares

sub filer_sysstat_get_cmode ($;$) {
    my ($fc, $options) = @_;
    my $count = $options->{'FILER_SYSSTAT_COUNT'};
    $count = 1 unless ((defined($count)) && ($count ne ""));
    my $interval = $options->{'FILER_SYSSTAT_INTERVAL'};
    $interval = 15 unless ((defined($interval)) && ($interval ne ""));

    my $after;
    my @protocols = qw(CPU NFS CIFS HTTP FCP iSCSI DAFS DISK_UTIL);
    my %sysstatHash;

    # intialize all the values
    foreach my $protocol (@protocols) {
        $sysstatHash{$protocol}{'Total'} = 0;
        $sysstatHash{$protocol}{'Avg'}   = 0;
        $sysstatHash{$protocol}{'Min'}   = 0;
        $sysstatHash{$protocol}{'Max'}   = 0;
    }

    my $cmd = "run local sysstat -c $count -s -x $interval";
    my ($text, $result, undef) =
      Filer_saycmd(conn    => $fc,
                   cmd     => $cmd,
                   timeout => $count * $interval + 60);
    my $value_type = "Total";
    my $got_DAFS   = 0;
    foreach my $line (split(/\n/, $text)) {
        if ($line =~ m/\sDAFS|OTHER\s/) {
            $got_DAFS = 1;
        }
        if ($line =~ /%/) {
            $line =~ s/^\s+//g;
            my @opsline = split(/\s+/, $line);
            $opsline[0] =~ s/%//;
            $sysstatHash{'CPU'}{$value_type} =
              $sysstatHash{'CPU'}{$value_type} + $opsline[0];
            $sysstatHash{'NFS'}{$value_type} =
              $sysstatHash{'NFS'}{$value_type} + $opsline[1];
            $sysstatHash{'CIFS'}{$value_type} =
              $sysstatHash{'CIFS'}{$value_type} + $opsline[2];
            $sysstatHash{'HTTP'}{$value_type} =
              $sysstatHash{'HTTP'}{$value_type} + $opsline[3];
            $opsline[15] =~ s/%//;
            $sysstatHash{'DISK_UTIL'}{$value_type} =
              $sysstatHash{'DISK_UTIL'}{$value_type} + $opsline[15];

            if ($got_DAFS) {

                # DAFS is absolete in GB release
                $sysstatHash{'DAFS'}{$value_type} =
                  $sysstatHash{'DAFS'}{$value_type} + $opsline[16];
            }
            $sysstatHash{'FCP'}{$value_type} =
              $sysstatHash{'FCP'}{$value_type} + $opsline[16 + $got_DAFS];
            $sysstatHash{'iSCSI'}{$value_type} =
              $sysstatHash{'iSCSI'}{$value_type} + $opsline[17 + $got_DAFS];
        } elsif ($line =~ /Min/) {
            $value_type = "Min";
        } elsif ($line =~ /Max/) {
            $value_type = "Max";
        } elsif ($line =~ /Avg/) {
            $value_type = "Avg";
        } elsif ($line =~ /Summary Statistics/) {
            $sysstatHash{'Summary Statistics'} = $line;
        }
    } ## end foreach my $line (split(/\n/...
    return %sysstatHash;
} ## end sub filer_sysstat_get_cmode ($;$)
##
##@Function
##   shares_create  :: Creates the mount - point , shares and the mapping of shares
##
##@Usage
##   		shares_create(server_obj    =>$Server,
##                	  vserver       =>$vserver1,
##					  junction_path =>$junction_path,
##					  volume        =>$vol_cifs,
##					  winclient     =>$WINCLIENT,
##					  domain        =>$CIFS_DOMAIN,
##					  ipaddr        =>$Node_details{$Nodes[0]}{'DATA_IP1'});
##
##@Arguments
##    winclient       Required   The windows client
##    vserver         Required   The vserver name
##    server_obj      Required   Server object on which execution is done
##    junction_path   Required   The path where the mounting is done
##    volume          Required   The volume needed for mounting
##    domain          Required   The domain name on which CIFS is created
##    ipaddr          Required   The LIF ipaddress on which shares are mapped
##    sharename       Optional   The shares name on which IO is performed
##
##@Return value
##    $drive  - The drive name to which the shares are mapped
##
##@Scope
##    Exported from this module
##
sub shares_create {
    my (%opts) =
      validate(@_,
               {server_obj    => {type => OBJECT, isa => 'Server'},
                vserver       => {type => SCALAR},
                junction_path => {type => SCALAR},
                volume        => {type => SCALAR},
                winclient     => {type => SCALAR},
                domain        => {type => SCALAR},
                ipaddr        => {type => SCALAR},
                sharename =>
                  {type => SCALAR, optional => 1, default => "CIFS_SHARE"}});
    my $vol_cifs      = $opts{volume};
    my $junction_path = $opts{junction_path};
    my $vserver       = $opts{vserver};
    my $Server        = $opts{server_obj};
    my $winclient     = $opts{winclient};
    my $domain        = $opts{domain};
    my $ipadd         = $opts{ipaddr};
    my $share_name    = $opts{sharename};

    #----------------------------------------------------------------------
    #  Setup CIFS from the client side and server side
    #----------------------------------------------------------------------
    $Log->comment(
                 "Establishing RSH connection to Windows client for the tests");
    my $ch_cifs = host($winclient)
      or die("Configuration error while creating host object of $winclient");
    my $cc_cifs = connect("#rsh")
      or die("Configuration error while connecting through rsh to $winclient");
    eval {
	$Server->volume_modify(vserver => $vserver,
                               policy  => POLICYNAME,
                               volume  => $vol_cifs);
        $Server->volume_mount(volume          => $vol_cifs,
                              vserver         => $vserver,
                              'junction-path' => $junction_path);
        $Server->vserver_cifs_share_create(vserver      => $vserver,
                                           'share-name' => $share_name,
                                           path         => $junction_path);

        #----------------------------------------------------------------------
        #  Map the cifs shares
        #----------------------------------------------------------------------
        my $val = "false";
        my $i = 0;
        while ( $i <= 3) {
        if ( $val = "false") {

        eval {
        $Log->step("Map the CIFS shares");

        my @vals = split('\.', $domain);
        my $CIFS_DOMAIN_NAME = $vals[0];
        my $Map_Name = "\\\\$ipadd\\$share_name";
		    $drive = map_cifs_share(share  => $Map_Name,
                                    domain => $CIFS_DOMAIN_NAME,
                                    username => $cc_cifs->{'username'},
				    password => $cc_cifs->{'password'}, 
                                    client => $cc_cifs,
                                    drive  => '*');
        };
        if($drive) {
        $Log->comment("successfully in mounting the cifs share");
        $val = "true";
        last;
        } else {
        $val = "false";
        $i++;
        $Log->comment("Unsuccessfully in mounting the cifs share in $i retry ");
        }
        } elsif ($i = 3) {
        $Log->comment("Unsuccessfully in mounting the cifs share: CIFS_SHARE ");
        die "Mounting cifs share failed: $@";
        }
        }

    };
    if ($@) {
        die("CIFS shares creation and mapping failed with error : $@");
    }
    return $drive;
} ## end sub shares_create
##
##@Function
##   sio_write  :: Performs IO operation using SIO tool
##
##@Usage
##   		          sio_write(server_obj =>$Server,
##		              filer      =>$Nodes[0],
##				      winclient  =>$WINCLIENT,
##				      vserver    =>$vserver1);
##
##@Arguments
##    filer           Reqiured   The filer on which execution is performed
##    winclient       Required   The windows client
##    vserver         Required   The vserver name
##    server_obj      Required   Server object on which execution is done
##
##@Scope
##    Exported from this module
##
sub sio_write {
    my (%opts) =
      validate(@_,
               {server_obj => {type => OBJECT, isa => 'Server'},
                filer      => {type => SCALAR},
                winclient  => {type => SCALAR},
                vserver    => {type => SCALAR},
                file_size => {type => SCALAR, optional => 1, default => '3m'},
                file_name =>
                  {type => SCALAR, optional => 1, default => 'test1.data'},
                blk_sz => {type => SCALAR, optional => 1, default => '4k'},
                sio_runtime =>
                  {type => SCALAR, optional => 1, default => '300'},
                rand_access =>
                  {type => SCALAR, optional => 1, default => '100'},
                threads => {type => SCALAR, optional => 1, default => '1'},
                sio_path =>
                  {type => SCALAR, optional => 1, default => 'C:\sio.exe'}});
    my $winclient   = $opts{winclient};
    my $file_size   = $opts{file_size};
    my $file_name   = $opts{file_name};
    my $blk_sz      = $opts{blk_sz};
    my $file_qty    = $opts{file_qty};
    my $sio_runtime = $opts{sio_runtime};
    my $threads     = $opts{threads};
    my $sio_path    = $opts{sio_path};
    my $rand_access = $opts{rand_access};
    my $vserver     = $opts{vserver};
    my $Server      = $opts{server_obj};
    my $filer       = $opts{filer};
    my ($ch_cifs,     $cc_cifs, $initpath, $initdrive,
        $create_file, $output,  $initsize, $sio,
        $avg,         $conn,    $endsize,  $host);
    eval {

        #----------------------------------------------------------------------
        #  Setup CIFS from the client side and server side
        #----------------------------------------------------------------------
        $Log->comment(
                 "Establishing RSH connection to Windows client for the tests");
        $ch_cifs = host($winclient)
          or
          die("Configuration error while creating host object of $winclient");
        $cc_cifs = connect("#rsh")
          or
          die("Configuration error while connecting through rsh to $winclient");

        #----------------------------------------------------------------------
        #  Write data on CIFS shares by performing SIO operation
        #----------------------------------------------------------------------
        logcomment("Finding the initial path information");
         my $verbose_flag = '-verbose';
        my (undef,undef,$initpath) = say($cc_cifs,"echo \> $drive\\$file_name",'-timeout',"100",$verbose_flag);;
         my (undef,undef,$initpath) = say($cc_cifs,"echo \> $drive\\$file_name",'-timeout',"100",$verbose_flag);
         my (undef,undef,$initpath) = say($cc_cifs,"echo \> $drive\\$file_name",'-timeout',"100",$verbose_flag);
        #$initpath = $cc_cifs->execute(command => "cd");
        if ($initpath =~ /([A-Z]:\\[A-Za-z0-9]+)/) {
            $initdrive = $1;
        }
        logcomment("Creating file for SIO operation");
        $create_file = "echo \> $drive\\$file_name";
        my $val = "false";
        my $i = 0;
        while ( $i <= 3) {
        if ( $val = "false") {

        eval { $cc_cifs->execute(command => $create_file); };
        if ($@) {
        $val = "false";
        $i++;
        $Log->comment("Unsuccessfully in creating the file in cifs share in $i retry ");
        } else {
        $val = "true";
        last;
   	}
        } elsif ($i = 3) {
        $Log->comment("Unsuccessfully in  creating the file in the cifs share: CIFS_SHARE ");
        die " creating the file in cifs share failed: $@";
        }
        }

        #$cc_cifs->execute(command => "cd $drive");
        $cc_cifs->execute(command => "$drive");
        $output = $cc_cifs->execute(command => "dir $drive");
        if ($output =~ /([\d]+)[\s]$file_name/) {
            $initsize = $1;
            $initsize =~ s/[\D]+//g;
            logcomment(
                "The file $file_name has been created with size $initsize bytes"
            );
        } else {
            die("The file $file_name has not been created in $drive");
        }
        logcomment("Start SIO operation in background");
        $sio =
          "sio.exe 0 $rand_access $blk_sz 0 $file_size $sio_runtime $threads $drive\\$file_name ";
        eval { $cc_cifs->execute(command => $sio); };
        if ($@) {
            die("SIO operation failed with error : $@ ");
        }
        logcomment("Checking the CIFS traffic");
        $host = host($filer)
          or die("Configuration error while creating host object of $filer");
        $conn = connect("#console");
        $avg = verify_traffic($conn, 'CIFS');
        if ($avg > 0) {
            logcomment(
                      "CIFS traffic occured with average traffic value = $avg");
        } else {
            die("CIFS traffic has not occured");
        }
        logcomment("Checking the filesize after SIO");
        $cc_cifs->execute(command => "cd $drive");
        $output = $cc_cifs->execute(command => "dir $drive");
        if ($output =~ /([\d\W]+)[\s]$file_name/) {
            $endsize = $1;
            $endsize =~ s/[\D]+//g;
            if ($endsize > $initsize) {
                logcomment(
                    "The SIO has been done on file $file_name with size $endsize bytes"
                );
            } else {
                die("File size of $file_name has not increased in SIO operation"
                );
            }
        } ## end if ($output =~ /([\d\W]+)[\s]$file_name/)
        if ($initdrive !~ /C:.*/) {
            $cc_cifs->execute(command => "cd C:");
        } else {
            $cc_cifs->execute(command => "cd $initdrive");
        }
    };
    if ($@) {
        die("SIO random access write operation failed with error : $@");
    }

} ## end sub sio_write

1;
