#-*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Cmode/Cmode_Utils.pm#1 $
#
## @summary Routines for cmode utilities
##
## @description
##	functions for the cmode testcase automation
##
## @keywords cmode
##
## @status public
##
## @dependencies
##	
## @author sivakum1@netapp.com

package Cmode::Cmode_Utils;

use strict;
use warnings;
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use Tharn;
use base qw(Exporter);

BEGIN
{
	our @EXPORT = qw(
        Filer_saycmd
    );
}
=head2 Filer_saycmd

Filer_saycmd(conn=>$conn, cmd=>$cmd, options=>$options, prompt=>$prompt, timeout=>$timeout);

This subroutine run any filer command using the connection object which we pass it on

It returns o/p of the command if any

The following arguments are accepted:

=item conn

(Optional): Connection object to use.

=item cmd

(Optional): filer command.

=item options

(Optional): filer options like -noprompt, -nocr.

=item prompt

(Optional): filer prompt like .*> (or) waitstring.

=item timeout

(Optional): timeout value in secs.

=back

=cut
#--------------------------------------------------------------------------------------------------

sub Filer_saycmd {
    my (%opts) = validate(@_, {
                        conn    => {type => OBJECT, isa=>'Connectrec'},
                        cmd     => {type => SCALAR},
                        options => {type => SCALAR, optional => 1},
                        prompt  => {type => SCALAR, optional => 1},
                        timeout => {type => SCALAR, default  => "60", optional=>1},
                              });
    my $conn = $opts{conn};
    my $cmd = $opts{cmd};
    my $options   = $opts{options};
    my $prompt   = $opts{prompt};
    my $timeout = $opts{timeout};

    my $output;
    if ((!defined($options))&&(!defined($prompt))) {
        (undef, undef, $output) = Tharn::say($conn, "$cmd", -timeout, $timeout);
    } elsif ((defined($options))&&(!defined($prompt))) {
        (undef, undef, $output) = Tharn::say($conn, "$cmd", "$options", -timeout, $timeout);
    } elsif ((!defined($options))&&(defined($prompt))) {
        (undef, undef, $output) = Tharn::say($conn, "$cmd", -prompt, $prompt, -timeout, $timeout);
    } else {
        (undef, undef, $output) = Tharn::say($conn, "$cmd", -prompt, $prompt, $options, -timeout, $timeout);
    }
    return $output;
}

1;
