# Hey emacs, this is basically -*- perl -*-
# $Id: dblade/wafl/dense/du/DU_Cmode.pm#1 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      DU_Cmode.pm - cmode dense space reporting 
#
# DESCRIPTION
#      This perl module provide wrapper interfaces to c-mode 
#       "du" cli command
#
# AUTHOR
#       gourab@netapp.com
#
#
## @pod here
##
##

=pod

=head1 NAME

    DU_Cmode.pm - provide wrapper interfaces to C-Mode du command

=head1 SYNOPSIS

    use Cmode::DU_Cmode qw(:all);
    OR
    use Cmode::DU_Cmode qw(:dense_fast_clone_start);

=head1 DESCRIPTION

=head1 FUNCTIONS

Cmode file clone Functions

=cut

package Cmode::DU_Cmode;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($VERSION);

#
# Define constants
#

use warnings;
use strict qw(vars subs refs);
use base   qw(Exporter);
use Tharn;

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $VERSION    = '0.01';

    my @du_cmode_sub = qw(
                        &disk_usage_cmode
                    );

    our %EXPORT_TAGS = (
        'DU_Cmode'      => [ @du_cmode_sub ],
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate qw(validate SCALAR ARRAYREF SCALARREF OBJECT);

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub disk_usage_cmode;


#------------------------------------------------------
#     Local Variables
#------------------------------------------------------


#------------------------------------------------------
#     Subroutines Definition
#------------------------------------------------------

=head2 disk_usage_cmode()

Report disk usage for a file

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns total and unique block count in specified units

=back

=cut

sub disk_usage_cmode {

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'vserver'          => {type => SCALAR},
            'volume'           => {type => SCALAR},
            'path'             => {type => SCALAR},
            'unique-bytes'     => {type => SCALAR, optional => 1},
            'range'            => {type => SCALAR, optional => 1},
            '-Timeout'         => {type => SCALAR, default  => 300},
        );
    
    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    foreach my $key ( keys %args_validn_spec ) {
        if ( $key ne 'server' && defined($options{$key})) {
            $input_arg{$key} = $options{$key};
        }
    }
    
    my $return_text;
    
    $input_arg{'-Verification'}=undef;

    $return_text = $server->volume_file_du( %input_arg );
  
    #We only need the the 3rd line of the ouput here 
    #
    my @output_array = split("\n",$return_text);

    #Splitting the third line on spaces
    #
    @output_array = split (/\s+/,$output_array[2]);

    #Stripping off KB from the output fields
    #
    my ($total_blocks)= split("KB",$output_array[1]);
    my ($unique_blocks)= split("KB",$output_array[2]);

    $Log->comment("Total Blocks:*$total_blocks* Unique Blocks:*$unique_blocks*");
    
    #Return integer output in KB 
    #
    return ($total_blocks,$unique_blocks);
}

1;
