# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/Cmode/Fileclone.pm#1 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      Fileclone.pm - cmode file clone
#
# DESCRIPTION
#      This perl module provide wrapper interfaces to c-mode 
#       file clone cli command
#
# AUTHOR
#       sumedh@netapp.com
#
#
## @pod here
##
##

=pod

=head1 NAME

    Fileclone.pm - provide wrapper interfaces to C-Mode file clone cli command

=head1 SYNOPSIS

    use Cmode::Fileclone qw(:all);
    OR
    use Cmode::Fileclone qw(:dense_fast_clone_start);

=head1 DESCRIPTION

=head1 FUNCTIONS

Cmode file clone Functions

=cut

package Cmode::Fileclone ;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($VERSION);

#
# Define constants
#

use warnings;
use strict qw(vars subs refs);
use base   qw(Exporter);
use SFOTest  qw(run_nodescope_command );
use Utils::Units qw(convert_to_bytes);

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $VERSION    = '0.01';

    my @fileclone_sub = qw(
                        &dense_fast_clone_start
                        &dense_fast_clone_status
                        &dense_fast_clone_wait_till_complete
                        &enable_clone_split
                        &disable_clone_split
                        &get_clone_header_stats
                        &get_range
                        &check_vol_size
                    );

    our %EXPORT_TAGS = (
        'Fileclone'      => [ @fileclone_sub ],
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate qw(validate SCALAR ARRAYREF SCALARREF OBJECT);

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub dense_fast_clone_start;
sub dense_fast_clone_status;
sub dense_fast_clone_wait_till_complete;
sub enable_clone_split;
sub disable_clone_split;
sub get_clone_header_stats;
sub get_range;
sub check_vol_size;
#------------------------------------------------------
#     Local Variables
#------------------------------------------------------


#------------------------------------------------------
#     Subroutines Definition
#------------------------------------------------------

=head2 dense_fast_clone_start()

Create a file clone

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise dies

=back

=cut

sub dense_fast_clone_start {

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'vserver'          => {type => SCALAR},
            'volume'           => {type => SCALAR},
            'source-path'      => {type => SCALAR},
            'destination-path' => {type => SCALAR, optional => 1},
            'snapshot-name'    => {type => SCALAR, optional => 1},
            'range'            => {type => SCALAR, optional => 1},
            'no-reserve'        => {type => SCALAR, optional => 1},
            '-Timeout'         => {type => SCALAR, default  => 300},
            'error_flag'       => {type => SCALAR, optional => 1, default => 0},
            'error_msg'        => {type => SCALARREF, optional => 1},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $error_ref = $options{'error_msg'};
    my $error_flag = $options{'error_flag'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( $key ne 'server' && defined($options{$key})) {
            $input_arg{$key} = $options{$key};
        }
    }

    $input_arg{'-Verification'}=undef;

    if ( $error_flag == 1 ) {
        eval {
            $server->volume_file_clone_create( %input_arg );
        };
        if ($@) {
            $$error_ref="$@";
            return 1;
        }
    } else {
        eval {
            $server->volume_file_clone_create( %input_arg );
        };
        if ($@) {
            die("Unable to create file clone: $@");
        }
        return 1;
   }
}

=head2 dense_fast_clone_status()

Use the cmf_test command to check the status of clone creation

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise dies

=back

=cut

sub dense_fast_clone_status {

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'volume'           => {type => SCALAR},
            'file'             => {type => SCALAR},
            'start_fbn'        => { type => SCALAR, optional => 1},
            'fbn_cnt'          => {type => SCALAR, optional => 1},
        );
    my %opts = validate(@_, \%args_validn_spec);
    my $server   = $opts{'server'}; # server object
    my $volume = $opts{volume};
    my $file = $opts{file};
    my $start_fbn = $opts{start_fbn};
    my $fbn_cnt = $opts{fbn_cnt};
    my $cmd;

    $cmd = "priv set test; slist /vol/$volume/$file";
    my $file_inode;
    my ($return_text) = $server->dblade(cmd => $cmd);
    if( $return_text =~ m/(\d+)\s+(\d+)/ ) {
        $file_inode = $1;
    }
    
 

    $cmd = "priv set test; cmf_test $volume";
    if ( defined($start_fbn) && defined($fbn_cnt) ) {
        $cmd = $cmd . " " . "lookup " . $file_inode . " " . $start_fbn . " " . $fbn_cnt;
        ($return_text) = $server->dblade(cmd => $cmd);
        if ( $return_text =~ m/Key not found/ ) {
            logcomment("Clone operation has completed");
            return 1;
        }
        else {
             return 0;
        }
    } else {
        $cmd = $cmd . " " . "list";
        ($return_text) = $server->dblade(cmd => $cmd);
        if($return_text =~ m/Clone-inum = $file_inode/) {
            return 0;
        }
        else {
            logcomment("Clone operation has completed");
            return 1;
       } 
    }
}

=head2 dense_fast_clone_wait_till_complete()

Use the cmf_test command to wait till the clone creation is complete

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise dies

=back

=cut

sub dense_fast_clone_wait_till_complete {
 
   my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'volume'           => {type => SCALAR},
            'file'             => {type => SCALAR},
            'start_fbn'        => { type => SCALAR, optional => 1},
            'fbn_cnt'          => {type => SCALAR, optional => 1},
            'timeout'          => {type => SCALAR, default =>600},
            'interval'         => {type => SCALAR, default =>5}
        );
    my %opts = validate(@_, \%args_validn_spec);
    my $server   = $opts{'server'}; # server object
    my $volume = $opts{volume};
    my $file = $opts{file};
    my $start_fbn = $opts{start_fbn};
    my $fbn_cnt = $opts{fbn_cnt};
    my $timeout = $opts{timeout};
    my $interval  = $opts{interval};
    while($timeout >0) {
        if ( defined($start_fbn) && defined($fbn_cnt) ) {
            if (dense_fast_clone_status(server => $server, volume => $volume, file => $file,
                                    start_fbn => $start_fbn, fbn_cnt=> $fbn_cnt,
                                    ) == 1 ) {
                last;
            }
        }
        else {
            if (dense_fast_clone_status(server => $server, volume => $volume, file => $file)
                                     == 1 ) {
                last;
            }
        } 

        sleep $interval;
        $timeout = $timeout - $interval;
    }
        if($timeout <=0) {
            die ("sis clone has not completed yet");
        }
}

=head2 enable_clone_split()

Enable file clone split

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise dies

=back

=cut

sub enable_clone_split {

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
        );
    my %opts = validate(@_, \%args_validn_spec);
    my $server   = $opts{'server'}; # server object
    my $cmd;

    $cmd = "priv set -q test; setflag wafl_disable_sis_clone_fixup 0";
    my ($return_text) = $server->dblade(cmd => $cmd);
    if( $return_text =~ /no such flag/ || $return_text =~ /usage/ ) {
        die($return_text); 
    } else {
        return 1;
    }
}

=head2 disable_clone_split()

Disable file clone split

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise dies

=back

=cut

sub disable_clone_split {

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
        );
    my %opts = validate(@_, \%args_validn_spec);
    my $server   = $opts{'server'}; # server object
    my $cmd;

    $cmd = "priv set -q test; setflag wafl_disable_sis_clone_fixup 1";
    my ($return_text) = $server->dblade(cmd => $cmd);
    if( $return_text =~ /no such flag/ || $return_text =~ /usage/ ) {
        die($return_text);
    } else {
        return 1;
    }
}


=head2 get_clone_header_stats (host=>$host, volume=>$vol,)

Use the cmf_test command to obtain the clone stats of a volume

=cut

sub get_clone_header_stats {

   my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'volume'           => {type => SCALAR},
            'filer'            => {type => SCALAR},  
        );
   my %opts = validate(@_, \%args_validn_spec);
   
   my $server = $opts{'server'};
   my $volume = $opts{'volume'};
   my $filer = $opts{'filer'};

   my $cmd = "priv set test; cmf_test $volume header";

   my ($return_text, $result_code)= run_nodescope_command(serverobj  => $server,
                                                          filername  => $filer,
                                                          command    => $cmd);

    my @output_array =  split(/\n/,$return_text);

    my %clone_header_hash;

    for(my $i=6 ; $i<=8 ; $i++) {
        my @temp = split(":",$output_array[$i]);
        $clone_header_hash{$temp[0]}=$temp[1];
    }

    return \%clone_header_hash;
  
}

=head2 get_range ( file_size => $size)

Get a valid range for sub-file cloning on the same file.

=cut

sub get_range {

    my (%opts) = validate ( @_, {
                                file_size => { type => SCALAR }
                                }
                          );

    my $file_size = $opts{file_size};
    my $size_in_bytes = convert_to_bytes($file_size);
    my $range;

    logcomment("Size is $file_size");
    my $blks = int($size_in_bytes/(1024*4));

    logcomment("Number of blocks is $blks");

    while(1)
        {
        my $src_fbn  = int(rand($blks));
        my $dstn_fbn = int(rand($blks));
        my $num_blks = (int(rand($blks) - ($src_fbn)) + 1);

        if ( $src_fbn < 0 || $dstn_fbn < 0 || $num_blks < 1 )
            { next; }
        else {
            if ( $src_fbn > $dstn_fbn ) {
                if ( $dstn_fbn + $num_blks <= $src_fbn && $src_fbn + $num_blks <= $blks) {
                    $range = "$src_fbn:$dstn_fbn:$num_blks";
                    logcomment("Range is $range");
                    return $range;
                }
                next;
            }

            if ( $dstn_fbn > $src_fbn ) {
                if ( $num_blks + $dstn_fbn <= $blks && $src_fbn + $num_blks <= $dstn_fbn ) {
                    $range = "$src_fbn:$dstn_fbn:$num_blks";
                    logcomment("Range is $range");
                    return $range;
                }
                next;
            }
            else
                { next; }
         }
    }
}

=head2 check_vol_size ( file_size => $size, num_file => $NUM_FILES, vol_size => $VOL_SIZE)

Checking for valid vol size.

=cut

sub check_vol_size {
    my (%opts) = validate ( @_, {
                                'file_size'        => { type => SCALAR },
                                'num_file'         => { type => SCALAR},
                                'vol_size'         => { type => SCALAR},
                                'free_space_delta' => { type => SCALAR},
								'stream_size'      => { type => SCALAR , optional => 1 },
								'num_streams'      => { type => SCALAR , optional => 1 },
                                }
                          );

    my $file_size = $opts{file_size};
    my $size_in_bytes_file = convert_to_bytes($file_size);

    my $vol_size = $opts{vol_size};
    my $size_in_bytes_vol = convert_to_bytes($vol_size);

    my $num_file = $opts{num_file};
    my $free_space_delta = $opts{free_space_delta};

	my ($num_streams,$size_in_bytes_stream); 
	if ($opts{num_streams}) {
		$num_streams = $opts{num_streams};
	    $size_in_bytes_stream = convert_to_bytes($opts{stream_size});
	} else { 
	    $num_streams = 0;
		$size_in_bytes_stream = 0;
	}
    
	my $margin_space = ($free_space_delta/100) + 1;
    my $min_size_require = int((($size_in_bytes_file * $num_file) + ($size_in_bytes_stream * $num_streams * $num_file)) * $margin_space) ;
    
	if ($min_size_require < $size_in_bytes_vol ){
		logcomment("Minimum volume size criteria met");
        return 1;
    } else {
        logcomment("Minimum volume size required:$min_size_require bytes");
        return 0;
    }
}
1;
