# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Cmode/Flexclone.pm#2 $
#
# $Change: 2013620 $
#
# $DateTime: 2013/04/16 03:29:39 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      Flexclone.pm - cmode flexclone library
#
# DESCRIPTION
#      This perl module provide interfaces to test flexclone's
#      functionality like clone creation, clone split, clone move,
#      and clone verification, and for automation utility etc.
#      This module also provide wrapper interfaces around volume,
#      snapshot, snapshot policy, wafliron cli command.
#
# AUTHOR
#       dl-nb-capacity-qa@netapp.com
#
#
## @pod here
##
##

=pod

=head1 NAME

    Flexclone.pm - C-Mode flexclone library to test FlexClone functionality

=head1 SYNOPSIS

    use Cmode::Flexclone qw(:all);
    OR
    use Cmode::Flexclone qw(:flexclone);
    OR
    use Cmode::Flexclone qw(:volume);
    OR
    use Cmode::Flexclone qw(:snapshot);
    OR
    use Cmode::Flexclone qw(:policy);
    OR
    use Cmode::Flexclone qw(:flexcache);
    OR
    use Cmode::Flexclone qw(:util);
    OR
    use Cmode::Flexclone qw(:wafliron);
    OR
    use Cmode::Flexclone qw(:constants);


=head1 DESCRIPTION

=head1 FUNCTIONS

Cmode Flexclone Functions

=cut


package Cmode::Flexclone;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($Validation, $VERSION);

#
# Define constants
#
use constant FC_ERROR => undef;
use constant FC_TRUE  => 1;
use constant FC_FALSE => 0;

use constant IRON_STARTING    => 3;
use constant IRON_SCANNING    => 4;
use constant IRON_FINISHING   => 5;
use constant IRON_NOT_RUNNING => 6;
use constant IRON_UNKNOWN     => 7;

use constant PARENT_MOVE => 1;
use constant CLONE_MOVE  => 2;

use constant AGGR_DOES_NOT_EXIST => 0;
use constant AGGR_EXIST => 1;
use constant AGGR_EXIST_WITH_FIELD_MISMATCH => 2;

use constant HA_ENABLE  => 1;
use constant HA_DISABLE => 0;

use warnings;
use strict qw(vars subs refs);
use base   qw(Exporter);
use Utils::Units qw(:constants convert_units strip_unit_spec guess_unit);

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $Validation = 1; # unused as of now
    $VERSION    = '0.01';

    my @clone_sub = qw(
                        &get_clone_creation_vldn_hash
                        &get_clone_split_vldn_hash
                        &get_parent_or_clone_move_vldn_hash
                        &volume_clone_show
                        &verify_cli_cmd_op
                        &volume_clone_create
                        &volume_clone_split
                        &volume_clone_split_stop
                        &volume_clone_split_estimate
                        &volume_clone_split_status
                        &is_clone_creation_ok
                        &is_clone_split_ok
                        &is_clone_split_abortion_ok
                        &is_parent_clone_move_ok
                    );

    my @volume_sub = qw(
                        &volume_copy
                        &volume_create
                        &volume_delete
                        &volume_online
                        &volume_modify
                        &volume_move
                        &volume_rename
                        &volume_mount
                        &volume_unmount
                        &volume_remove_junction
                        &volume_show
                    );

    my @snapshot_sub = qw(
                        &volume_snapshot_create
                        &volume_snapshot_delete
                        &volume_snapshot_modify
                        &volume_snapshot_promote
                        &volume_snapshot_remove_owner
                        &volume_snapshot_rename
                        &volume_snapshot_show
                        &volume_snapshot_autodelete_modify
                        &volume_snapshot_autodelete_show
                        &volume_snapshot_restore_file
                        );

    my @policy_sub = qw(
                        &volume_snapshot_policy_create
                        &volume_snapshot_policy_delete
                        &volume_snapshot_policy_modify
                        &volume_snapshot_policy_add_schedule
                        &volume_snapshot_policy_modify_schedule
                        &volume_snapshot_policy_remove_schedule
                        &volume_snapshot_policy_show
                    );

    my @flexcache_sub = qw(
                            &volume_flexcache_create
                            &volume_flexcache_delete
                            &volume_flexcache_show
                            &volume_flexcache_cache_policy_create
                            &volume_flexcache_cache_policy_delete
                            &volume_flexcache_cache_policy_modify
                            &volume_flexcache_cache_policy_show
                        );

    my @wafliron_sub = qw(
                        &aggregate_wafliron_start
                        &aggregate_wafliron_show
                        &aggregate_wafliron_status
                        &aggregate_wafliron_stop
                        &aggregate_wafliron_wait
                    );

    my @vserver_sub = qw(
                        &vserver_show
                        &vserver_modify
                    );

    my @util_sub = qw(
                        &does_vserver_exist
                        &does_aggr_exists
                        &get_filer_aggr_name
                        &get_unit_conversion
                        &is_space_available_in_aggr
                        &does_files_exist_in_vol
                        &fill_aggr
                        &is_corruption_found_in_ironlogs
                        &set_flags
                        &get_vserver_data_ip_list
                        &get_vol_count_by_node
                        &start_mlog_msg_reader
                        &stop_mlog_msg_reader
                        &get_wafl_scan_speed
                        &set_wafl_scan_speed
                        &get_cluster_name
                        &get_mkfile_filename_list
                        &get_flexcache_stat
                        &open_remote_nfs_client_connection
                        &aggregate_show
                        &enableHA
                        &disableHA
                        &send_interface_home
                    );

    my @fc_constants   = qw(FC_ERROR FC_TRUE FC_FALSE);

    my @iron_constants = qw(IRON_NOT_RUNNING IRON_STARTING
                            IRON_SCANNING IRON_FINISHING);

    my @move_constants = qw(PARENT_MOVE CLONE_MOVE);

    my @unit_constants = qw(UNIT_B UNIT_KB UNIT_MB UNIT_TB);

    my @aggr_constants = qw(AGGR_EXIST AGGR_DOES_NOT_EXIST 
                            AGGR_EXIST_WITH_FIELD_MISMATCH);

    my @ha_constants   = qw(HA_ENABLE HA_DISABLE);

    my @all_constants  = (@fc_constants, @iron_constants,
                            @move_constants, @unit_constants,
                            @aggr_constants, @ha_constants,
                         );

    our %EXPORT_TAGS = (
        'flexclone' => [ @clone_sub ],
        'volume'    => [ @volume_sub ],
        'snapshot ' => [ @snapshot_sub ],
        'policy'    => [ @policy_sub ],
        'flexcache' => [ @flexcache_sub ],
        'util'      => [ @util_sub ],
        'wafliron'  => [ @wafliron_sub ],
        'vserver'   => [ @vserver_sub ],
        'constants' => [ @all_constants ],
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate   qw(validate validate_pos SCALAR ARRAYREF
                          SCALARREF OBJECT HASHREF CODEREF);
use LogReader::MlogMessages;


#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub get_clone_creation_vldn_hash;
sub get_clone_split_vldn_hash;
sub get_parent_or_clone_move_vldn_hash;
sub volume_clone_show;
sub verify_cli_cmd_op;
sub volume_clone_create;
sub volume_clone_split;
sub volume_clone_split_stop;
sub volume_clone_split_estimate;
sub volume_clone_split_status;
sub is_clone_creation_ok;
sub is_clone_split_ok;
sub is_clone_split_abortion_ok;
sub is_parent_clone_move_ok;

sub volume_copy;
sub volume_create;
sub volume_delete;
sub volume_online;
sub volume_modify;
sub volume_move;
sub volume_rename;
sub volume_mount;
sub volume_unmount;
sub volume_remove_junction;
sub volume_show;

sub volume_snapshot_create;
sub volume_snapshot_delete;
sub volume_snapshot_modify;
sub volume_snapshot_promote;
sub volume_snapshot_remove_owner;
sub volume_snapshot_rename;
sub volume_snapshot_show;
sub volume_snapshot_autodelete_modify;
sub volume_snapshot_autodelete_show;
sub volume_snapshot_restore_file;

sub volume_snapshot_policy_create;
sub volume_snapshot_policy_delete;
sub volume_snapshot_policy_modify;
sub volume_snapshot_policy_add_schedule;
sub volume_snapshot_policy_modify_schedule;
sub volume_snapshot_policy_remove_schedule;
sub volume_snapshot_policy_show;

sub volume_flexcache_create;
sub volume_flexcache_delete;
sub volume_flexcache_show;
sub volume_flexcache_cache_policy_create;
sub volume_flexcache_cache_policy_delete;
sub volume_flexcache_cache_policy_modify;
sub volume_flexcache_cache_policy_show;

sub aggregate_wafliron_start;
sub aggregate_wafliron_show;
sub aggregate_wafliron_status;
sub aggregate_wafliron_stop;
sub aggregate_wafliron_wait;

#
# utility functions
#
sub does_vserver_exist;
sub does_aggr_exists;
sub get_filer_aggr_name;
sub get_unit_conversion;
sub is_space_available_in_aggr;
sub does_files_exist_in_vol;
sub fill_aggr;
sub is_corruption_found_in_ironlogs;
sub set_flags;
sub get_vserver_data_ip_list;
sub get_vol_count_by_node;
sub start_mlog_msg_reader;
sub stop_mlog_msg_reader;
sub get_wafl_scan_speed;
sub set_wafl_scan_speed;
sub get_cluster_name;
sub get_mkfile_filename_list;
sub get_flexcache_stat;
sub open_remote_nfs_client_connection;
sub aggregate_show;
sub enableHA;
sub disableHA;
sub send_interface_home;

#------------------------------------------------------
#     Local Variables
#------------------------------------------------------

my %clone_creation_vldn_hash = (
        'parent_snapshot' =>
            {
                'busy' => 'true',
            },
        'clone_snapshot' =>
            {
                'busy' => 'false',
            },
        'vol_show_clone_vol' =>
            {
                'clone-volume' => 'true',
                'state' => 'online',
            },
        'vol_clone_show' =>
            {
                # Actual value to these parameters will be assigned
                # in the is_clone_creation_ok function, as these
                # values are needed from the running script

                'parent-volume'   => FC_TRUE, 
                'parent-snapshot' => FC_TRUE,
                'flexclone'       => FC_TRUE,
            },
    );

my %clone_split_vldn_hash = (
        'parent_snapshot' =>
            {
                'busy' => 'false',
            },
        'vol_show_clone_vol' =>
            {
                'clone-volume' => 'false',
                'state' => 'online',
            },
    );


my %parent_or_clone_move_vldn_hash = (
        'parent_snapshot' =>
            {
                'busy' => 'false',
            },
        'clone_snapshot' =>
            {
                'busy' => 'false',
            },
        'vol_show_clone_vol' =>
            {
                'clone-volume' => 'false',
                'state' => 'online',
            },
    );

#
# mlog message reader flag
# set to 1, when reader is started
# set to 0, when reader is stopped
#
my $mlog_rdr_started = 0;

#
# mlog message reader object
#
my $mlog_msg_rdr_obj = undef;



=head2 get_clone_creation_vldn_hash

Returns the copy of validation hash used to verify 
clone creation. 

=over 4

=item Arguments:

=over 4

None

=back

=item Returns:

Returns HASH

=back

=cut


sub get_clone_creation_vldn_hash {
    my %change_vldn_hash;

    my %chng_parent_snap    = %{$clone_creation_vldn_hash{'parent_snapshot'}};
    my %chng_clone_snap     = %{$clone_creation_vldn_hash{'clone_snapshot'}} ;
    my %chng_vol_show_clone = %{$clone_creation_vldn_hash{'vol_show_clone_vol'}};
    my %chng_clone_show     = %{$clone_creation_vldn_hash{'vol_clone_show'}};

    # creating a copy of %clone_creation_vldn_hash
    # for script user to work
    $change_vldn_hash{'parent_snapshot'}    = \%chng_parent_snap;
    $change_vldn_hash{'clone_snapshot'}     = \%chng_clone_snap;
    $change_vldn_hash{'vol_show_clone_vol'} = \%chng_vol_show_clone;
    $change_vldn_hash{'vol_clone_show'}     = \%chng_clone_show;

    return %change_vldn_hash;
}


=head2 get_clone_split_vldn_hash

Returns the copy of validation hash used to verify
parent and clone flexvol after clone split.

=over 4

=item Arguments:

=over 4

None

=back

=item Returns:

Returns HASH

=back

=cut

sub get_clone_split_vldn_hash {
    my %change_vldn_hash;

    my %chng_parent_snap    = %{$clone_split_vldn_hash{'parent_snapshot'}};
    my %chng_vol_show_clone = %{$clone_split_vldn_hash{'vol_show_clone_vol'}};

    # creating a copy of %clone_split_vldn_hash
    # for script user to work
    $change_vldn_hash{'parent_snapshot'}    = \%chng_parent_snap;
    $change_vldn_hash{'vol_show_clone_vol'} = \%chng_vol_show_clone;

    return %change_vldn_hash;
}

=head2 get_parent_or_clone_move_vldn_hash

Returns the copy of validation hash used to verify
parent and clone flexvol after parent or clone move.

=over 4

=item Arguments:

=over 4

None

=back

=item Returns:

Returns HASH

=back

=cut

sub get_parent_or_clone_move_vldn_hash {
    my %change_vldn_hash;

    my %chng_parent_snap    = %{$parent_or_clone_move_vldn_hash{'parent_snapshot'}};
    my %chng_clone_snap     = %{$parent_or_clone_move_vldn_hash{'clone_snapshot'}} ;
    my %chng_vol_show_clone = %{$parent_or_clone_move_vldn_hash{'vol_show_clone_vol'}};

    # creating a copy of %parent_or_clone_move_vldn_hash
    # for script user to work
    $change_vldn_hash{'parent_snapshot'}    = \%chng_parent_snap;
    $change_vldn_hash{'clone_snapshot'}     = \%chng_clone_snap;
    $change_vldn_hash{'vol_show_clone_vol'} = \%chng_vol_show_clone;

    return %change_vldn_hash;
}

=head2 volume_clone_show()

Retrieve the clone information

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_clone_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'                  => {type => OBJECT},
            'vserver'                 => {type => SCALAR, optional => 1},
            'flexclone'               => {type => SCALAR, optional => 1},
            'parent-volume'           => {type => SCALAR, optional => 1},
            'parent-snapshot'         => {type => SCALAR, optional => 1},
            'junction-path'           => {type => SCALAR, optional => 1},
            'junction-active'         => {type => SCALAR, optional => 1},
            'space-guarantee'         => {type => SCALAR, optional => 1},
            'space-guarantee-enabled' => {type => SCALAR, optional => 1},
            'aggregate'               => {type => SCALAR, optional => 1},
            'dsid'                    => {type => SCALAR, optional => 1},
            'msid'                    => {type => SCALAR, optional => 1},
            'size'                    => {type => SCALAR, optional => 1},
            'used'                    => {type => SCALAR, optional => 1},
            'split-estimate'          => {type => SCALAR, optional => 1},
            'comment'                 => {type => SCALAR, optional => 1},
            'errmsg'                  => {type => SCALARREF},
            'cli_op'                  => {type => ARRAYREF},
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 300,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_clone_show( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_clone_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
} ### end sub volume_clone_show


=head2 verify_cli_cmd_op()

call the given method and validate the received
field's value with given fields value

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    errmsg        => referece to error scalar variable
    inputarg      => reference to hash having args for calling sub 
    fields        => reference to hash used for validation
    method        => reference to calling sub

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub verify_cli_cmd_op {
    my %options = validate(@_,
                           {
                            errmsg    => {type => SCALARREF},
                            inputarg  => {type => HASHREF},
                            fields    => {type => HASHREF},
                            method    => {type => CODEREF},
                           });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable
    my %inHash  = %{$options{'fields'}}; # hash of fields to verify

    if ( FC_FALSE == &{$options{'method'}}( %{$options{'inputarg'}} ) ) {
        $$err_ref = ${$options{'inputarg'}{'errmsg'}};
        return FC_FALSE;
    }

    # Retrieve the cli command output
    my @cmdOp = @{$options{'inputarg'}{'cli_op'}};
 
    $$err_ref = "";

    # Retrieve the first hash from cli command output
    my %opHash = %{$cmdOp[0]};

    foreach my $field ( keys %inHash ) {
        # replace " char
        $opHash{$field} =~ s/\"//g;

        if( $inHash{$field} ne $opHash{$field} ) {
            my $mesg = "Mismatch in key: $field value, act:$opHash{$field}"
                            . " exp:$inHash{$field}\n";
            $$err_ref .= $mesg;
        } else {
            logcomment("*** key: $field, value: $inHash{$field} is ok ***\n");
        }

    } ### End of foreach

    if ($$err_ref ne ""){
        return FC_FALSE;
    }

    return FC_TRUE;
} ### end of sub verify_cli_cmd_op


=head2 volume_clone_create()

create the clone volume or flexclone 

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    parent_vol     => parent flexvol name
    clone_vol      => flexclone name
    snapshot       => snapshot  name
    junc_path      => junc_path  of clone vol 
    junc_act       => activate junction of clone vol 
    guarantee      => guarantee  of clone vol 
    comment        => comment for clone vol 
    foreground     => run the command in foreground
    cli_op         => referece to scalar variable to return cli o/p
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]


=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_clone_create {
    my %options = validate(@_,
                    {
                        server     => {type => OBJECT},
                        vserver    => {type => SCALAR},
                        parent_vol => {type => SCALAR},
                        clone_vol  => {type => SCALAR},
                        snapshot   => {type => SCALAR, optional =>1},
                        junc_path  => {type => SCALAR, optional =>1},
                        junc_act   => {type => SCALAR, optional =>1},
                        guarantee  => {type => SCALAR, optional =>1},
                        foreground => {type => SCALAR, default  => 'true'},
                        comment    => {type => SCALAR, optional =>1},
                        cli_op     => {type => SCALARREF, optional => 1},
                        errmsg     => {type => SCALARREF},
                        timeout    =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                        },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'vserver'}       = $options{'vserver'};   
    $input_arg{'parent-volume'} = $options{'parent_vol'};   
    $input_arg{'flexclone'}     = $options{'clone_vol'};   
    $input_arg{'foreground'}    = $options{'foreground'};   
    $input_arg{'-Timeout'}      = $options{'timeout'};   

    if (defined $options{'snapshot'}) {
        $input_arg{'parent-snapshot'} = $options{'snapshot'}; 
    }
    if (defined $options{'junc_act'}) {
        $input_arg{'junction-active'} = $options{'junc_act'};
    }
    if (defined $options{'junc_path'}) {
        $input_arg{'junction-path'} = $options{'junc_path'};
    }
    if (defined $options{'guarantee'}) {
        $input_arg{'space-guarantee'} = $options{'guarantee'};
    }
    if (defined $options{'comment'}) {
        $input_arg{'comment'} = $options{'comment'};
    }
    

    # reset the error message
    $$err_ref = "none";

    eval {
        my $cmd_op = $server->volume_clone_create( %input_arg );

        if ( exists($options{'cli_op'}) ) {
            ${$options{'cli_op'}} = $cmd_op;
        }
    };
    if($@) {
        $$err_ref = "volume_clone_create() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_clone_split()

split the clone volume or flexclone

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    clone_vol      => flexclone name
    foreground     => run the command in foreground
    space_check    => override space check while splitting
    cli_op         => referece to scalar variable to return cli o/p
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_clone_split {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        clone_vol   => {type => SCALAR},
                        foreground  => {type => SCALAR, optional =>1, default =>'false'},
                        space_check => {type => SCALAR, optional =>1, default =>'false'},
                        cli_op      => {type => SCALARREF, optional => 1},
                        errmsg      => {type => SCALARREF},
                        timeout    =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                        },
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'}   = $options{'timeout'}; 
    $input_arg{'vserver'}    = $options{'vserver'};
    $input_arg{'flexclone'}  = $options{'clone_vol'};
    $input_arg{'foreground'} = $options{'foreground'}; 
    $input_arg{'override-space-check'} = $options{'space_check'};

    # reset the error message
    $$err_ref = "none";

    eval {
        my $cmd_op = $server->volume_clone_split_start( %input_arg );

        if ( exists($options{'cli_op'}) ) {
            ${$options{'cli_op'}} = $cmd_op;
        }
    };

    if($@) {
        $$err_ref = "volume_clone_split() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_clone_split_stop()

stop clone split fori given flexclone

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    flexclone      => flexclone name
    errmsg         => scalar referece to return error message
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_clone_split_stop {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        flexclone   => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        timeout     => { type => SCALAR, default => 60},
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'}   = $options{'timeout'};
    $input_arg{'vserver'}    = $options{'vserver'};
    $input_arg{'flexclone'}  = $options{'flexclone'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_clone_split_stop( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_clone_split() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_clone_split_estimate()

retrieve split estimate for given flexclone

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    flexclone      => flexclone name
    errmsg         => scalar referece to return error message
    cli_op         => hash referece to return parsed cli output 
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_clone_split_estimate {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        flexclone   => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        cli_op      => {type => HASHREF},
                        timeout     => { type => SCALAR, default => 60},
                    });

    my $cli_op; # scalar to store cli cmd o/p
    my @op_list; # array to hold cli cmd o/p by lines
    my @value;   # array to hold parse cmd o/p

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref hash array

    # reset the error message
    $$err_ref = "none";

    my $cmd = "volume clone split estimate "
                . " -vserver   $options{'vserver'} "
                . " -flexclone $options{'flexclone'}" ;

    eval {
        $server->cli(   'cmd' => $cmd,
                        'output' => \$cli_op,
                        '-Timeout' => $options{'timeout'},
                     );

        # chop the cli output based on newline char 
        @op_list = split('\n', $cli_op);

        # pick up the value from last line of cli output
        @value = split(/\s+/, $op_list[-1]);

        $cmdOpRef->{'vserver'}        = $value[0];
        $cmdOpRef->{'flexclone'}      = $value[1];
        $cmdOpRef->{'split_estimate'} = $value[2];
    };

    if($@) {
        $$err_ref = "volume_clone_split_estimate() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_clone_split_status()

retrieve split status for given flexclone

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    flexclone      => flexclone name
    errmsg         => scalar referece to return error message
    cli_op         => hash referece to return parsed cli output 
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_clone_split_status {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        flexclone   => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        cli_op      => {type => HASHREF},
                        timeout     => { type => SCALAR, default => 60},
                    });

    my $cli_op; # scalar to store cli cmd o/p
    my @op_list; # array to hold cli cmd o/p by lines
    my @keyList; # list of keys in o/p
    my @valList; # list of values in o/p

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref hash array

    # reset the error message
    $$err_ref = "none";

    my $cmd = "volume clone split status "
                . " -vserver   $options{'vserver'} "
                . " -flexclone $options{'flexclone'}" ;

    eval {
        $server->cli(   'cmd' => $cmd,
                        'output' => \$cli_op,
                        '-Timeout' => $options{'timeout'},
                     );

        # chop the cli output based on newline char 
        @op_list = split('\n', $cli_op);

        # pick up the value from last line of cli output
        @keyList = split('X;X', $op_list[0]);

        # pick up the value from last line of cli output
        @valList = split('X;X', $op_list[-1]);

        foreach my $i ( 0..$#keyList ) {
            $cmdOpRef->{$keyList[$i]} = $valList[$i];
        }
    };

    if($@) {
        $$err_ref = "volume_clone_split_status() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_copy()

copy from source volume to a destination volume on destination aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => source volume 
    dstn_vol       => destination volume 
    dstn_aggr      => destination aggregate name
    stripe_frmt    => destination stripe format 
    foreground     => run the command in foreground
    cli_op         => referece to scalar variable to return cli o/p
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [600]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_copy {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        dstn_vol    => {type => SCALAR},
                        dstn_aggr   => {type => SCALAR},
                        stripe_frmt =>
                        {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(data|full)$/,
                        },
                        foreground  =>
                        {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(true|false)$/,
                            default => 'false',
                        },
                        errmsg  => {type => SCALARREF},
                        timeout => {type => SCALAR, default => 600},
                        cli_op  => {type => SCALARREF, optional => 1},
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # setting input argument for cli command
    $input_arg{'-Timeout'}   = $options{'timeout'};
    $input_arg{'vserver'}    = $options{'vserver'};
    $input_arg{'volume'}     = $options{'volume'};
    $input_arg{'destination-volume'}    = $options{'dstn_vol'};
    $input_arg{'destination-aggregate'} = $options{'dstn_aggr'};

    if ( defined $options{'stripe_frmt'} ) {
        $input_arg{'destination-stripe-format'} = $options{'stripe_frmt'};
    }
    if ( defined $options{'foreground'} ) {
        $input_arg{'foreground'} = $options{'foreground'};
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        my $cmd_op = $server->volume_copy( %input_arg );

        if ( exists($options{'cli_op'}) ) {
            ${$options{'cli_op'}} = $cmd_op;
        }
    };

    if($@) {
        $$err_ref = "volume_copy() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_create()

create volume

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_create {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'vserver'          => {type => SCALAR},
            'volume'           => {type => SCALAR},
            'aggregate'        => {type => SCALAR},
            'size'             => {type => SCALAR, optional => 1},
            'state'            => {type => SCALAR, optional => 1},
            'type'             => {type => SCALAR, optional => 1},
            'policy'           => {type => SCALAR, optional => 1},
            'user'             => {type => SCALAR, optional => 1},
            'group'            => {type => SCALAR, optional => 1},
            'security-style'   => {type => SCALAR, optional => 1},
            'unix-permissions' => {type => SCALAR, optional => 1},
            'junction-path'    => {type => SCALAR, optional => 1},
            'junction-active'  => {type => SCALAR, optional => 1},
            'vsroot'           => {type => SCALAR, optional => 1},
            'comment'          => {type => SCALAR, optional => 1},
            'maxdir-size'      => {type => SCALAR, optional => 1},
            'space-guarantee'  => {type => SCALAR, optional => 1},
            'snapshot-policy'  => {type => SCALAR, optional => 1},
            'stripe-count'     => {type => SCALAR, optional => 1},
            'stripe-width'     => {type => SCALAR, optional => 1},
            'concurrency'      => {type => SCALAR, optional => 1},
            'optimize'         => {type => SCALAR, optional => 1},
            'foreground'       => {type => SCALAR, optional => 1},     
            'nvfail'           => {type => SCALAR, optional => 1},     
            'antivirus-on-access-policy' => {type => SCALAR, optional => 1},
            'flexcache-cache-policy'     => {type => SCALAR, optional => 1},
            'flexcache-fill-policy'      => {type => SCALAR, optional => 1},
            'flexcache-origin-volume'    => {type => SCALAR, optional => 1},
            'stripe-format'          => {type => SCALAR, optional => 1},
            'max-write-alloc-blocks' => {type => SCALAR, optional => 1},
            'errmsg'                 => {type => SCALARREF},
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 300,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_create( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_create() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_delete()

delete given volume in a given vserver 

=over 4

=item Arguments:

=over 4

    server     => server object
    vserver    => vserver name
    volume     => volume name
    foreground => run the command in foreground
    errmsg     => referece to error scalar variable
    timeout    => timeout in seconds [600]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_delete {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'     => {type => OBJECT},
            'vserver'    => {type => SCALAR},
            'volume'     => {type => SCALAR},
            'foreground' => {type => SCALAR, optional => 1},
            'errmsg'     => {type => SCALARREF},
            'timeout'    => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_delete() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_online()

online given volume in a given vserver 

=over 4

=item Arguments:

=over 4

    server     => server object
    vserver    => vserver name
    volume     => volume name
    foreground => run the command in foreground
    errmsg     => referece to error scalar variable
    timeout    => timeout in seconds [600]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_online {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'     => {type => OBJECT},
            'vserver'    => {type => SCALAR},
            'volume'     => {type => SCALAR},
            'foreground' => {type => SCALAR, optional => 1},
            'errmsg'     => {type => SCALARREF},
            'timeout'    => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_online( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_online() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_modify()

modify volume attributes

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_modify {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'  => {type => OBJECT},
            'vserver' => {type => SCALAR},
            'volume'  => {type => SCALAR},
            'size'    => {type => SCALAR, optional => 1},
            'state'   =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(online|restricted|offline|force-online)$/,
                },
            'policy'            => {type => SCALAR, optional => 1},
            'user'              => {type => SCALAR, optional => 1},
            'group'             => {type => SCALAR, optional => 1},
            'security-style'    => {type => SCALAR, optional => 1},
            'unix-permissions'  => {type => SCALAR, optional => 1},
            'junction-active'   => {type => SCALAR, optional => 1},
            'comment'           => {type => SCALAR, optional => 1},
            'autosize' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            'max-autosize'       => {type => SCALAR, optional => 1},
            'autosize-increment' => {type => SCALAR, optional => 1},
            'files' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(\d+)$/,
                },
            'maxdir-size'     => {type => SCALAR, optional => 1},
            'space-guarantee' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(none|volume|file)$/,
                },
            'min-readahead' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            'atime-update' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            'snapdir-access'         => {type => SCALAR, optional => 1},
            'percent-snapshot-space' => {type => SCALAR, optional => 1},
            'snapshot-policy' => {type => SCALAR, optional => 1},
            'stripe-width'    => {type => SCALAR, optional => 1},
            'concurrency' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(low|medium|high|peak)$/,
                },
            'optimize' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^([max-]?performance|space)$/,
                },
            'antivirus-on-access-policy' => {type => SCALAR, optional => 1},
            'index-dir-enabled' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            'space-mgmt-try-first'   => {type => SCALAR, optional => 1},
            'max-write-alloc-blocks' => {type => SCALAR, optional => 1},
            'nvfail'                 => {type => SCALAR, optional => 1},
            'in-nvfailed-state'      => {type => SCALAR, optional => 1},
            'filesys-size-fixed'     => {type => SCALAR, optional => 1},
            'fractional-reserve'     => {type => SCALAR, optional => 1},
            'auto-snapshots-enabled' => {type => SCALAR, optional => 1},
            'convert-ucode'          => {type => SCALAR, optional => 1},
            'sched-snap-name'        => {type => SCALAR, optional => 1},
            'instance-uuid'          => {type => SCALAR, optional => 1},
            'provenance-uuid'        => {type => SCALAR, optional => 1},
            'flexcache-cache-policy' => {type => SCALAR, optional => 1},
            'flexcache-fill-policy'  => {type => SCALAR, optional => 1},
            'snapshot-clone-dependency' => {type => SCALAR, optional => 1},
            'errmsg'  => {type => SCALARREF},
            'timeout' => {type => SCALAR, default => 60},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_modify( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_modify() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_move()

move volume to destination aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume to be moved
    dstn_aggr      => destination aggregate name
    foreground     => run the command in foreground [false]
    cli_op         => referece to scalar variable to return cli o/p
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_move {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        dstn_aggr   => {type => SCALAR},
                        foreground  => {
                                        type => SCALAR, 
                                        optional =>1, 
                                        default =>'false',
                                       },
                        errmsg      => {type => SCALARREF},
                        cli_op      => {type => SCALARREF, optional => 1},
                        timeout     =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                        },
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'}   = $options{'timeout'};
    $input_arg{'vserver'}    = $options{'vserver'};
    $input_arg{'volume'}     = $options{'volume'};
    $input_arg{'foreground'} = $options{'foreground'};
    $input_arg{'destination-aggregate'} = $options{'dstn_aggr'};

    # reset the error message
    $$err_ref = "none";

    eval {
        my $cmd_op = $server->volume_move( %input_arg );
        
        if ( exists($options{'cli_op'}) ) {
            ${$options{'cli_op'}} = $cmd_op;
        }
    };

    if($@) {
        $$err_ref = "volume_move() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_rename()

rename volume

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => old name of volume 
    newname        => new name of volume
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_rename {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        newname     => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        timeout     =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                        },
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'vserver'}  = $options{'vserver'};
    $input_arg{'volume'}   = $options{'volume'};
    $input_arg{'newname'}  = $options{'newname'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_rename( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_rename() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_mount()

mount volume

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume to be mounted 
    junc_path      => junction-path of volume
    junc_act       => activate junction path 
    pol_ovrride    => override export policy
    foreground     => run the command in foreground [false]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_mount {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        junc_path   => {type => SCALAR},
                        junc_act    =>
                        {
                            type => SCALAR,
                            optional => 1,
                            regex => qr/^(true|false)$/,
                        },
                        pol_ovrride =>
                        {
                            type => SCALAR,
                            optional => 1,
                            regex => qr/^(true|false)$/,
                        },
                        errmsg      => {type => SCALARREF},
                        timeout     =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                        },
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'vserver'}  = $options{'vserver'};
    $input_arg{'volume'}   = $options{'volume'};
    $input_arg{'junction-path'} = $options{'junc_path'};

    if ( defined $options{'junc_act'} ) {
        $input_arg{'active'} = $options{'junc_act'};
    }
    if ( defined $options{'pol_ovrride'} ) {
        $input_arg{'policy-override'} = $options{'pol_ovrride'};
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_mount( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_mount() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_unmount()

unmount volume

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume to be unmounted
    force          => force unmount 
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_unmount {
    my %options = validate(@_,
                    {
                        server  => {type => OBJECT},
                        vserver => {type => SCALAR},
                        volume  => {type => SCALAR},
                        force   =>
                        {
                            type => SCALAR,
                            optional => 1,
                            regex => qr/^(true|false)$/,
                            default => 'false',
                        },
                        errmsg  => {type => SCALARREF},
                        timeout =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                        },
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'vserver'}  = $options{'vserver'};
    $input_arg{'volume'}   = $options{'volume'};

    if ( defined $options{'force'} ) {
        $input_arg{'force'} = $options{'force'};
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_unmount( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_unmount() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_remove_junction()

remove dead junction

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    junc_path      => junction-path
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_remove_junction {
    my %options = validate(@_,
                    {
                        server    => {type => OBJECT},
                        vserver   => {type => SCALAR},
                        junc_path => {type => SCALAR},
                        errmsg  => {type => SCALARREF},
                        timeout =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                        },
                    });

    my $server  = $options{'server'}; # server obj
    my $err_ref = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'vserver'}  = $options{'vserver'};
    $input_arg{'junction-path'} = $options{'junc_path'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_remove_junction( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_remove_junction() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_show()

Execute 'volume show' command and set the cli_op key to 
array of hash reference

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume to be mounted
    cli_op         => referece to output array variable
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [600]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_show {
    my %options = validate(@_,
                    {
                        server  => {type => OBJECT},
                        vserver => {type => SCALAR, optional => 1},
                        volume  => {type => SCALAR, optional => 1},
                        errmsg  => {type => SCALARREF},
                        cli_op  => {type => ARRAYREF},
                        timeout    =>
                        {
                            type => SCALAR,
                            optional => 1,
                            default  => 600,
                        },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    my %input_arg; # hash of input argument to cli command

    $input_arg{'-Timeout'} = $options{'timeout'};

    if ( defined $options{'vserver'} ) {
        $input_arg{'vserver'} = $options{'vserver'};
    } 
    if ( defined $options{'volume'} ) {
        $input_arg{'volume'} = $options{'volume'};
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_show( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
} ### end sub volume_show


=head2 volume_snapshot_create()

create volume snapshot

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume name
    snapshot       => snapshot name
    comment        => comment for snapshot 
    foreground     => run the command in foreground [true]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back


=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_create {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        snapshot    => {type => SCALAR},
                        comment     => {type => SCALAR, optional => 1},
                        foreground  => {
                                        type => SCALAR,
                                        optional => 1,
                                        default => 'true',
                                       },
                        timeout     => {
                                        type => SCALAR,
                                        optional => 1,
                                        default  => 60,
                                       },
                        errmsg      => {type => SCALARREF},
                    });

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'vserver'}    = $options{'vserver'};
    $input_arg{'volume'}     = $options{'volume'};
    $input_arg{'foreground'} = $options{'foreground'};
    $input_arg{'snapshot'}   = $options{'snapshot'};
    $input_arg{'-Timeout'}   = $options{'timeout'};

    if ( defined($options{'comment'}) ) {
        $input_arg{'comment'} = $options{'comment'} 
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_create( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_create() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_snapshot_delete()

delete volume snapshot

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume name
    snapshot       => snapshot name 
    force          => force snapshot delete 
    ignr_owners    => ignore snapshot owners [false] 
    foreground     => run the command in foreground [false]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_delete {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        snapshot    => {type => SCALAR},
                        foreground  => {
                                        type => SCALAR,
                                        optional =>1,
                                        regex => qr/^(true|false)$/,
                                        default =>'true',
                                       },
                        force       => {
                                        type => SCALAR,
                                        optional =>1,
                                        regex => qr/^(true|false)$/,
                                        default =>'false',
                                       },
                        ignr_owners => {
                                        type => SCALAR,
                                        optional =>1,
                                        regex => qr/^(true|false)$/,
                                        default =>'false',
                                       },
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 120,
                                       },
                        errmsg      => {type => SCALARREF},
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'vserver'}    = $options{'vserver'};
    $input_arg{'volume'}     = $options{'volume'};
    $input_arg{'snapshot'}   = $options{'snapshot'};
    $input_arg{'-Timeout'}   = $options{'timeout'};
    $input_arg{'foreground'} = $options{'foreground'};
    $input_arg{'force'}      = $options{'force'};
    $input_arg{'ignore-owners'} = $options{'ignr_owners'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_delete() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_snapshot_modify()

modify volume snapshot

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume name
    snapshot       => snapshot name
    comment        => snapshot comment 
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_modify {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        snapshot    => {type => SCALAR},
                        comment     => {type => SCALAR},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                        errmsg      => {type => SCALARREF},
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'vserver'}    = $options{'vserver'};
    $input_arg{'volume'}     = $options{'volume'};
    $input_arg{'snapshot'}   = $options{'snapshot'};
    $input_arg{'comment'}    = $options{'comment'};
    $input_arg{'-Timeout'}   = $options{'timeout'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_modify( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_modify() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_promote()

restore volume snapshot

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume name
    snapshot       => snapshot name
    force          => force snapshot promote
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back


=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_promote {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        snapshot    => {type => SCALAR},
                        force       => {
                                        type => SCALAR,
                                        optional =>1,
                                        regex => qr/^(true|false)$/,
                                        #default =>'false',
                                       },
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                        errmsg      => {type => SCALARREF},
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'vserver'}  = $options{'vserver'};
    $input_arg{'volume'}   = $options{'volume'};
    $input_arg{'snapshot'} = $options{'snapshot'};
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'force'}    = $options{'force'} if (defined($options{'force'}));

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_promote( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_promote() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_snapshot_remove_owner()

remove volume snapshot owner

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume name
    snapshot       => snapshot name
    owner          => snapshot owner
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_remove_owner {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        snapshot    => {type => SCALAR},
                        owner       => {type => SCALAR},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                        errmsg      => {type => SCALARREF},
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'vserver'}  = $options{'vserver'};
    $input_arg{'volume'}   = $options{'volume'};
    $input_arg{'snapshot'} = $options{'snapshot'};
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'owner'}    = $options{'owner'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_remove_owner( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_remove_owner() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_rename()

rename volume snapshot name

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    volume         => volume name
    snapshot       => snapshot name
    new_name       => snapshot new name 
    foreground     => run the command in foreground [false]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_rename {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        vserver     => {type => SCALAR},
                        volume      => {type => SCALAR},
                        snapshot    => {type => SCALAR},
                        new_name    => {type => SCALAR},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                        errmsg      => {type => SCALARREF},
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'vserver'}  = $options{'vserver'};
    $input_arg{'volume'}   = $options{'volume'};
    $input_arg{'snapshot'} = $options{'snapshot'};
    $input_arg{'new-name'} = $options{'new_name'};
    $input_arg{'-Timeout'} = $options{'timeout'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_rename( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_rename() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_show()

show volume snapshots 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'      => {type => OBJECT},
            'vserver'     => {type => SCALAR, optional => 1},
            'volume'      => {type => SCALAR, optional => 1},
            'snapshot'    => {type => SCALAR, optional => 1},
            'dsid'        => {type => SCALAR, optional => 1},
            'msid'        => {type => SCALAR, optional => 1},
            'busy' =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(true|false)$/,
                },
            'owners'           => {type => SCALAR, optional => 1},
            'size'             => {type => SCALAR, optional => 1},
            'blocks'           => {type => SCALAR, optional => 1},
            'usedblocks '      => {type => SCALAR, optional => 1},
            'cpcount'          => {type => SCALAR, optional => 1},
            'internal-status'  => {type => SCALAR, optional => 1},
            'comment'          => {type => SCALAR, optional => 1},
            'fs-version'       => {type => SCALAR, optional => 1},
            'physical-snap-id' => {type => SCALAR, optional => 1},
            'logical-snap-id'  => {type => SCALAR, optional => 1},
            'record-owner'     => {type => SCALAR, optional => 1},
            'tags'             => {type => SCALAR, optional => 1},
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
            'errmsg' => {type => SCALARREF},
            'cli_op' => {type => ARRAYREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {

        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_snapshot_show( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_snapshot_autodelete_modify()

modify volume snapshot autodelete policy

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_autodelete_modify {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'      => {type => OBJECT},
            'vserver'     => {type => SCALAR, optional => 1},
            'volume'      => {type => SCALAR, optional => 1},
            'enabled'     => {type => SCALAR, optional => 1},
            'commitment'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(try|disrupt|destroy)$/,
                },
            'defer-delete'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(scheduled|user_created|prefix|none)$/,
                },
            'delete-order'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(newest_first|oldest_first)$/,
                },
            'trigger'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(volume|snap_reserve|space_reserve)$/,
                },
            'destroy-list'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(lun_clone|vol_clone|cifs_share|file_clone)$/,
                },
            'defer-delete-prefix' => {type => SCALAR, optional => 1},
            'target-free-space'   => {type => SCALAR, optional => 1},
            'timeout' => {type => SCALAR, default => 60},
            'errmsg'  => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_autodelete_modify(%input_arg);
    };

    if($@) {
        $$err_ref = "volume_snapshot_autodelete_modify() failed, error: $@";
        return FC_FALSE;
    }
    return FC_TRUE;
}

=head2 volume_snapshot_autodelete_show()

show volume snapshot autodelete policy

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_autodelete_show {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'      => {type => OBJECT},
            'vserver'     => {type => SCALAR, optional => 1},
            'volume'      => {type => SCALAR, optional => 1},
            'enabled'     => {type => SCALAR, optional => 1},
            'commitment'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(try|disrupt|destroy)$/,
                },
            'defer-delete'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(scheduled|user_created|prefix|none)$/,
                },
            'delete-order'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(newest_first|oldest_first)$/,
                },
            'trigger'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(volume|snap_reserve|space_reserve)$/,
                },
            'destroy-list'  =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex => qr/^(lun_clone|vol_clone|cifs_share|file_clone)$/,
                },
            'defer-delete-prefix' => {type => SCALAR, optional => 1},
            'target-free-space'   => {type => SCALAR, optional => 1},
            'timeout' => {type => SCALAR, default => 300},
            'errmsg'  => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_snapshot_autodelete_show(%input_arg);
    };

    if($@) {
        $$err_ref = "volume_snapshot_autodelete_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_snapshot_restore_file()

restore file from a given snapshot in a volume 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_restore_file {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'vserver'        => {type => SCALAR},
            'snapshot'       => {type => SCALAR},
            'path'           => {type => SCALAR},
            'volume'         => {type => SCALAR, optional => 1},
            'volume-dsid'    => {type => SCALAR, optional => 1},
            'instance-uuid'  => {type => SCALAR, optional => 1},
            'restore-path'   => {type => SCALAR, optional => 1},
            'split-disabled' => {type => SCALAR, optional => 1},
            'timeout' => {type => SCALAR, default => 300},
            'errmsg'  => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_restore_file(%input_arg);
    };

    if($@) {
        $$err_ref = "volume_snapshot_restore_file() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_policy_create()

create volume snapshot policy name

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_policy_create {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            policy      => {type => SCALAR},
            enabled     => {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(true|false)$/,
                           },
            comment     => {type => SCALAR, optional => 1},
            schedule1   => {type => SCALAR },
            count1      => {
                            type => SCALAR,
                            regex => qr/^(\d+)$/,
                           },
            schedule2   => {
                            type => SCALAR,
                            optional => 1,
                            depends  => ['count2'],
                           },
            count2      => {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(\d+)$/,
                           },
            schedule3   => {
                            type => SCALAR,
                            optional => 1,
                            depends  => ['count3'],
                           },
            count3      => {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(\d+)$/,
                           },
            schedule4   => {
                            type => SCALAR,
                            optional => 1,
                            depends  => ['count4'],
                           },
            count4      => {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(\d+)$/,
                           },
            schedule5   => {
                            type => SCALAR,
                            optional => 1,
                            depends  => ['count4'],
                           },
            count5      => {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(\d+)$/,
                           },
            timeout     => {
                            type => SCALAR,
                            optional =>1,
                            default  => 60,
                           },
            errmsg      => {type => SCALARREF},
            'verify'    => {type => SCALAR, default => 0},
                    
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'-Verification'} = $options{'verify'};

    foreach my $key ( keys %args_validn_spec ) {

        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";
    eval {
        $server->volume_snapshot_policy_create( %input_arg );
    };
    if($@) {
        $$err_ref = "volume_snapshot_policy_create() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_policy_delete()
 
delete volume snapshot policy name

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    policy         => policy name
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_policy_delete {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        policy      => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'policy'}   = $options{'policy'};
    $input_arg{'-Timeout'} = $options{'timeout'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_policy_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_policy_delete() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_snapshot_policy_modify()

modify volume snapshot policy

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    policy         => policy name
    enabled        => enable policy
    comment        => comment to policy
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_policy_modify {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        policy      => {type => SCALAR},
                        enabled     => {
                                        type => SCALAR,
                                        optional =>1,
                                        regex => qr/^(true|false)$/,
                                       },
                        comment     => {type => SCALAR, optional => 1},
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                    });


    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'policy'}   = $options{'policy'};
    $input_arg{'-Timeout'} = $options{'timeout'};

    if ( defined $options{'enabled'} ) {
        $input_arg{'enabled'} = $options{'enabled'};
    }

    if ( defined $options{'comment'} ) {
        $input_arg{'comment'} = $options{'comment'};
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_policy_modify( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_policy_modify() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_policy_add_schedule()

add schedule to volume snapshot policy

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    policy         => policy  name
    schedule       => schedule name
    count          => snapshot count
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_policy_add_schedule {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        policy      => {type => SCALAR},
                        schedule    => {type => SCALAR},
                        count       => {
                                        type => SCALAR,
                                        optional =>1,
                                        regex => qr/^(\d+)$/,
                                       },
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'policy'}   = $options{'policy'};
    $input_arg{'schedule'} = $options{'schedule'};
    $input_arg{'count'}    = $options{'count'};
    $input_arg{'-Timeout'} = $options{'timeout'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_policy_add_schedule( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_policy_add_schedule() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_policy_modify_schedule()

modify schedule to volume snapshot policy

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    policy         => policy  name
    schedule       => schedule name
    newcount       => snapshot new count
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_policy_modify_schedule {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        policy      => {type => SCALAR},
                        schedule    => {type => SCALAR},
                        newcount    => {
                                        type => SCALAR,
                                        regex => qr/^(\d+)$/,
                                       },
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'policy'}   = $options{'policy'};
    $input_arg{'schedule'} = $options{'schedule'};
    $input_arg{'newcount'} = $options{'newcount'};
    $input_arg{'-Timeout'} = $options{'timeout'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_policy_modify_schedule( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_policy_modify_schedule() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_policy_remove_schedule()

remove schedule from volume snapshot policy

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    policy         => policy  name
    schedule       => schedule name
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_policy_remove_schedule {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        policy      => {type => SCALAR},
                        schedule    => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    # set the input arguments to cli command
    $input_arg{'policy'}   = $options{'policy'};
    $input_arg{'schedule'} = $options{'schedule'};
    $input_arg{'-Timeout'} = $options{'timeout'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_snapshot_policy_remove_schedule( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_policy_remove_schedule() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 volume_snapshot_policy_show()

show volume snapshot policy

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_snapshot_policy_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'  => {type => OBJECT},
            'policy'  => {type => SCALAR, optional => 1},
            'enabled' =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex    => qr/^(true|false)$/,
                },
            'comment' => {type => SCALAR, optional => 1},
            'total-schedules' =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex    => qr/^(\d+)$/,
                },
            'schedules' => {type => SCALAR, optional => 1},
            'counts'    =>
                {
                    type => SCALAR,
                    optional =>1,
                    regex    => qr/^(\d+)$/,
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional =>1,
                    default  => 60,
                },
            'errmsg' => {type => SCALARREF},
            'cli_op' => {type => ARRAYREF},
        );


    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {

        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_snapshot_policy_show( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_snapshot_policy_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 is_clone_creation_ok()

This interface validates parent and clone flexvol
status after clone creation.

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    parent_vol     => parent vol name
    clone_vol      => clone vol name
    snapshot       => snapshot name
    field_hash     => referece to validation hash variable [ %clone_creation_vldn_hash ]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub is_clone_creation_ok {
    my %options = validate(@_,
                    {
                        server     => {type => OBJECT},
                        vserver    => {type => SCALAR},
                        parent_vol => {type => SCALAR},
                        clone_vol  => {type => SCALAR},
                        snapshot   => {type => SCALAR},
                        errmsg     => {type => SCALARREF},
                        field_hash => {type => HASHREF, optional => 1},
                    });

    my $inHashref; #Hash of fields to verify
    my $errStr1; # error string 1
    my $errStr2; # error string 2
    my %fields ; # hash of fields to verify
    my %inArgs ; # hash of input args to called method
    my @cliCmdOp ; # array of hash received from cli command op

    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable


    # setting up default arg required by all calls
    %inArgs = (
                server   => $options{'server'},
                vserver  => $options{'vserver'},
                errmsg   => \$errStr1,
                cli_op   => \@cliCmdOp,
               );

    $inHashref = ((defined $options{'field_hash'})
                    ? $options{'field_hash'} 
                    : \%clone_creation_vldn_hash);

    ## snapshot is marked busy in parent flexvol
    logcomment("-> Verifying base snapshot in parent flexvol <-");
    %fields = %{$inHashref->{'parent_snapshot'}};

    $inArgs{'volume'}   = $options{'parent_vol'} ;
    $inArgs{'snapshot'} = $options{'snapshot'} ;

    if ( FC_TRUE == 
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_snapshot_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> parent vol snapshot verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> parent vol snapshot verification is not ok <-");
        return FC_FALSE;
    }

    # cleanup keys, which are not required
    delete $inArgs{'volume'};

    ## snapshot is available in clone flexvol
    logcomment("-> Verifying base snapshot in clone flexvol <-");
    %fields = %{$inHashref->{'clone_snapshot'}};
    $inArgs{'volume'} = $options{'clone_vol'} ;

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_snapshot_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> clone vol snapshot verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> clone vol snapshot verification is not ok <-");
        return FC_FALSE;
    }

    # cleanup keys, which are not required
    delete $inArgs{'snapshot'};

    ## clone-volume is set to true in clone_vol
    logcomment("-> Verifying clone flexvol fields <-");
    %fields = %{$inHashref->{'vol_show_clone_vol'}};
    $inArgs{'volume'} = $options{'clone_vol'} ;

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> clone vol verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> clone vol verification is not ok <-");
        return FC_FALSE;
    }

    # cleanup keys, which are not required
    delete $inArgs{'volume'};

    ## "vol clone show" command shows the clone_vol
    logcomment("-> Verifying clone in vol_clone_show cmd op <-");
    %fields = %{$inHashref->{'vol_clone_show'}};
    
    ## assigning value to the undefined parameters in clone_creation_vldn_hash
    $fields{'parent-volume'}    = $options{'parent_vol'} ;
    $fields{'parent-snapshot'}  = $options{'snapshot'}; 
    $fields{'flexclone'}        = $options{'clone_vol'}; 

    ## Input Arguments for volume_clone_show function.
    $inArgs{'flexclone'} = $options{'clone_vol'} ;

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_clone_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> vol clone show verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> vol clone show verification is not ok <-");
        return FC_FALSE;
    }

    return FC_TRUE;
} ### end sub is_clone_creation_ok


=head2 is_clone_split_ok()

This interface validates parent and clone flexvol
status after clone split operation.

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    parent_vol     => parent vol name
    clone_vol      => clone vol name
    snapshot       => snapshot name
    field_hash     => referece to validation hash variable [ %clone_split_vldn_hash ]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub is_clone_split_ok {
    my %options = validate(@_,
                    {
                        server     => {type => OBJECT},
                        vserver    => {type => SCALAR},
                        parent_vol => {type => SCALAR},
                        clone_vol  => {type => SCALAR},
                        snapshot   => {type => SCALAR},
                        errmsg     => {type => SCALARREF},
                        field_hash => {type => HASHREF, optional => 1},
                    });

    my $inHashref; #Hash of fields to verify
    my $errStr1; # error string 1
    my $errStr2; # error string 2
    my %fields ; # hash of fields to verify
    my %inArgs ; # hash of input args to called method
    my @cliCmdOp ; # array of hash received from cli command op

    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # setting up default arg required by all calls
    %inArgs = (
                server   => $options{'server'},
                vserver  => $options{'vserver'},
                errmsg   => \$errStr1,
                cli_op   => \@cliCmdOp,
               );

    $inHashref = ((defined $options{'field_hash'})
                    ? $options{'field_hash'}
                    : \%clone_split_vldn_hash);

    ## snapshot is not marked busy in parent flexvol
    logcomment("-> Verifying base snapshot in parent flexvol <-");
    %fields = %{$inHashref->{'parent_snapshot'}};

    $inArgs{'volume'}   = $options{'parent_vol'};
    $inArgs{'snapshot'} = $options{'snapshot'};

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_snapshot_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> parent vol snapshot verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> parent vol snapshot verification is not ok <-");
        return FC_FALSE;
    }
    # cleanup keys, which are not required
    delete $inArgs{'volume'};

    ## base snapshot is not available in clone flexvol
    logcomment("-> Verifying base snapshot in clone flexvol <-");
    $inArgs{'volume'} = $options{'clone_vol'};

    if ( volume_snapshot_show( %inArgs ) == FC_FALSE ) {
        logcomment("-> base snapshot not found in clone flexvol <-");
    } else {
        $$err_ref = "base snapshot info found";
        $Log->error("snapshot info:\n" . Dumper $cliCmdOp[0]);
        logcomment("-> base snapshot found in clone flexvol <-");
        return FC_FALSE;
    }
    # cleanup keys, which are not required
    delete $inArgs{'snapshot'};

    ## clone-volume is set to false in cloned flexvol
    logcomment("-> Verifying clone flexvol fields <-");
    %fields = %{$inHashref->{'vol_show_clone_vol'}};
    $inArgs{'volume'} = $options{'clone_vol'};

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> clone vol verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> clone vol verification is not ok <-");
        return FC_FALSE;
    }
    # cleanup keys, which are not required
    delete $inArgs{'volume'};

    ## "vol clone show" command shows nothing
    logcomment("-> Verifying clone is not present <-");
    $inArgs{'flexclone'} = $options{'clone_vol'} ;

    if (volume_clone_show( %inArgs ) == FC_FALSE ) {
        logcomment("-> No clone flexvol found after clone split <-");
    } else {
        $$err_ref = "Clone info found";
        $Log->error("clone info:\n" . Dumper $cliCmdOp[0]);
        logcomment("-> clone flexvol found after clone split <-");
        return FC_FALSE;
    }
    return FC_TRUE;
} ### end sub is_clone_split_ok

=head2 is_clone_split_abortion_ok()

This interface validates parent and clone flexvol
status after clone split abortion.

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    parent_vol     => parent vol name
    clone_vol      => clone vol name
    snapshot       => snapshot name
    field_hash     => referece to validation hash variable [ %clone_creation_vldn_hash ]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub is_clone_split_abortion_ok {
    my %options = validate(@_,
                    {
                        server     => {type => OBJECT},
                        vserver    => {type => SCALAR},
                        parent_vol => {type => SCALAR},
                        clone_vol  => {type => SCALAR},
                        snapshot   => {type => SCALAR},
                        errmsg     => {type => SCALARREF},
                        field_hash => {type => HASHREF, optional => 1},
                    });

    my $inHashref; #Hash of fields to verify
    my $errStr1; # error string 1
    my $errStr2; # error string 2
    my %fields ; # hash of fields to verify
    my %inArgs ; # hash of input args to called method
    my @cliCmdOp ; # array of hash received from cli command op

    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable


    # setting up default arg required by all calls
    %inArgs = (
                server   => $options{'server'},
                vserver  => $options{'vserver'},
                errmsg   => \$errStr1,
                cli_op   => \@cliCmdOp,
               );

    $inHashref = ((defined $options{'field_hash'})
                    ? $options{'field_hash'} 
                    : \%clone_creation_vldn_hash);

    ## snapshot is marked busy in parent flexvol
    logcomment("-> Verifying base snapshot in parent flexvol <-");
    %fields = %{$inHashref->{'parent_snapshot'}};

    $inArgs{'volume'}   = $options{'parent_vol'} ;
    $inArgs{'snapshot'} = $options{'snapshot'} ;

    if ( FC_TRUE == 
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_snapshot_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> parent vol snapshot verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> parent vol snapshot verification is not ok <-");
        return FC_FALSE;
    }

    # cleanup keys, which are not required
    delete $inArgs{'volume'};
    delete $inArgs{'snapshot'};

    ## snapshot is not available in clone flexvol
    logcomment("-> Verifying base snapshot is not clone flexvol <-");

    if ( FC_FALSE == volume_snapshot_show(
                        server   => $options{'server'},
                        vserver  => $options{'vserver'},
                        errmsg   => \$errStr2,
                        cli_op   => \@cliCmdOp,
                        snapshot => $options{'snapshot'},
                        volume   => $options{'clone_vol'},
                        ) ) {
        logcomment("-> base snapshot not found in clone flexvol <-");
    } else {
        logcomment("-> base snapshot found in clone flexvol <-");
        $$err_ref = "base snapshot found in clone flexvol,"
                    . "cli_op: " . Dumper @cliCmdOp ;
        return FC_FALSE;
    }

    ## clone-volume is set to true in clone_vol
    logcomment("-> Verifying clone flexvol fields <-");
    %fields = %{$inHashref->{'vol_show_clone_vol'}};
    $inArgs{'volume'} = $options{'clone_vol'} ;

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> clone vol verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> clone vol verification is not ok <-");
        return FC_FALSE;
    }

    # cleanup keys, which are not required
    delete $inArgs{'volume'};

    ## "vol clone show" command shows the clone_vol
    logcomment("-> Verifying clone in vol_clone_show cmd op <-");
    %fields = %{$inHashref->{'vol_clone_show'}};
    
    ## assigning value to the undefined parameters in clone_creation_vldn_hash
    $fields{'parent-volume'}    = $options{'parent_vol'} ;
    $fields{'parent-snapshot'}  = $options{'snapshot'}; 
    $fields{'flexclone'}        = $options{'clone_vol'}; 

    ## Input Arguments for volume_clone_show function.
    $inArgs{'flexclone'} = $options{'clone_vol'} ;

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_clone_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> vol clone show verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> vol clone show verification is not ok <-");
        return FC_FALSE;
    }

    return FC_TRUE;
} ### end sub is_clone_split_abortion_ok


=head2 is_parent_clone_move_ok()

This interface validates parent and clone flexvol
status after clone move operation.

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    parent_vol     => parent vol name
    clone_vol      => clone vol name
    snapshot       => snapshot name
    move_type      => type of move [ PARENT_MOVE | CLONE_MOVE ] 
    field_hash     => referece to validation hash variable [ %clone_split_vldn_hash ]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub is_parent_clone_move_ok {
    my %options = validate(@_,
                    {
                        server     => {type => OBJECT},
                        vserver    => {type => SCALAR},
                        parent_vol => {type => SCALAR},
                        clone_vol  => {type => SCALAR},
                        snapshot   => {type => SCALAR},
                        move_type  => {type => SCALAR},
                        temp_vol   => {type => SCALARREF, default => undef},
                        errmsg     => {type => SCALARREF},
                        field_hash => {type => HASHREF, optional => 1},
                    });

    my $inHashref; #Hash of fields to verify
    my $errStr1; # error string 1
    my $errStr2; # error string 2
    my %fields ; # hash of fields to verify
    my %inArgs ; # hash of input args to called method
    my @cliCmdOp ; # array of hash received from cli command op

    my $err_ref   = $options{'errmsg'}; # refernce to error msg variable
    my $move_type = $options{'move_type'};
    my $temp_vol_ref   = $options{'temp_vol'}; # refernce to temp vol name

    # setting up default arg required by all calls
    %inArgs = (
                server   => $options{'server'},
                vserver  => $options{'vserver'},
                errmsg   => \$errStr1,
                cli_op   => \@cliCmdOp,
               );

    $inHashref = ((defined $options{'field_hash'})
                    ? $options{'field_hash'}
                    : \%parent_or_clone_move_vldn_hash);

    ############################################################
    # in case of parent move,
    #    - a temporary flexvol is left and clone is still a clone
    #    - base snapshot is not marked busy in parent volume
    # in case of clone move,
    #    - snapshot in parent vol is not marked busy
    #    - snapshot is available in clone
    #    - clone is not available
    #    - clone-volume is set to false in clone volume
    ############################################################

    ############################################################
    ## verify parent vol's snapshot is not marked busy
    ############################################################
    logcomment("-> Verifying base snapshot in parent flexvol <-");
    %fields = %{$inHashref->{'parent_snapshot'}};

    $inArgs{'volume'}   = $options{'parent_vol'};
    $inArgs{'snapshot'} = $options{'snapshot'};

    if ( FC_TRUE ==
            verify_cli_cmd_op(
                inputarg => \%inArgs,
                fields   => \%fields,
                method   => \&volume_snapshot_show,
                errmsg   => \$errStr2) ) {
        logcomment("-> parent vol snapshot verification is ok <-");
    } else {
        $$err_ref = $errStr2;
        logcomment("-> parent vol snapshot verification is not ok <-");
        return FC_FALSE;
    }
    # cleanup keys, which are not required
    delete $inArgs{'volume'};

    if ( $move_type == PARENT_MOVE ) {

        ##################################
        ## parent has left temporary volume
        ##################################
        if ( volume_clone_show(
                server    => $options{'server'},
                vserver   => $options{'vserver'},
                flexclone => $options{'clone_vol'},
                cli_op    => \@cliCmdOp,
                errmsg    => \$errStr2) == FC_FALSE ) {
            $$err_ref = "no clone flexvol found, error: $errStr2";
            logcomment("-> after parent vol move, no clone found <-");
            return FC_FALSE;
        }

        logcomment( Dumper \@cliCmdOp );

        my $tmpVolName= $cliCmdOp[0]->{'parent-volume'};
    
        if ( defined($temp_vol_ref) ) {
           $$temp_vol_ref = $tmpVolName;
        }

	## modify tmpvol to online if its offline.
	my @vol_state;
	my $err1;
	volume_show(
	    server    => $options{'server'},
	    vserver   => $options{'vserver'},
	    volume   => $tmpVolName,
	    cli_op   => \@vol_state,
	    errmsg   => \$err1,
	    timeout  => "300",
	) or die("volume_show failed, error:$err1");

        if ( $vol_state[0]->{'state'} eq 'offline' ) {
	    logcomment("-> Bring $tmpVolName to online <-");
	    volume_modify(
                server   => $options{'server'},
                vserver  => $options{'vserver'},
                volume   => $tmpVolName,
                state    => 'online',
                errmsg   => \$err1)
                or die("Failed to online $tmpVolName error:$err1");
	}

        ##################################
        ## verify clone relationship
        ##################################
        logcomment("-> Verifying clone relationship with tmporary flexvol <-");
        is_clone_creation_ok(
            server     => $options{'server'},
            vserver    => $options{'vserver'},
            parent_vol => $tmpVolName,
            clone_vol  => $options{'clone_vol'},
            snapshot   => $options{'snapshot'},
            errmsg     => \$errStr2)
                or die("clone verification failed, error:$errStr2");
        logcomment("-> clone relationhip is ok,after parent move <-");

    } else {
        ##########################################
        ## snapshot is available in clone flexvol
        ##########################################
        logcomment("-> Verifying base snapshot in clone flexvol <-");
        %fields = %{$inHashref->{'clone_snapshot'}};
        $inArgs{'volume'} = $options{'clone_vol'} ;

        if ( FC_TRUE ==
                verify_cli_cmd_op(
                    inputarg => \%inArgs,
                    fields   => \%fields,
                    method   => \&volume_snapshot_show,
                    errmsg   => \$errStr2) ) {
            logcomment("-> clone vol snapshot verification is ok <-");
        } else {
            $$err_ref = $errStr2;
            logcomment("-> clone vol snapshot verification is not ok <-");
            return FC_FALSE;
        }

        # cleanup keys, which are not required
        delete $inArgs{'snapshot'};

        ############################################################
        ## verify clone flexvol fields 
        ## clone-volume is set to false
        ############################################################
        logcomment("-> Verifying clone flexvol fields <-");
        %fields = %{$inHashref->{'vol_show_clone_vol'}};
        $inArgs{'volume'} = $options{'clone_vol'};

        if ( FC_TRUE ==
                verify_cli_cmd_op(
                    inputarg => \%inArgs,
                    fields   => \%fields,
                    method   => \&volume_show,
                    errmsg   => \$errStr2) ) {
            logcomment("-> clone vol verification is ok <-");
        } else {
            $$err_ref = $errStr2;
            logcomment("-> clone vol verification is not ok <-");
            return FC_FALSE;
        }
        # cleanup keys, which are not required
        delete $inArgs{'volume'};

        ############################################################
        ## verify "vol clone show" command shows nothing
        ############################################################
        logcomment("-> Verifying clone is not present <-");
        $inArgs{'flexclone'} = $options{'clone_vol'} ;

        if (volume_clone_show( %inArgs ) == FC_FALSE ) {
            logcomment("-> No clone flexvol found after clone move <-");
        } else {
            $$err_ref = "Clone info found";
            $Log->error("clone info:\n" . Dumper $cliCmdOp[0]);
            logcomment("-> clone flexvol found after clone move <-");
            return FC_FALSE;
        }
    }

    return FC_TRUE;
} ### end sub is_parent_clone_move_ok


=head2 volume_flexcache_create()

create the flexcache for given origin volume

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    origin-volume  => name of origin volume
    errmsg         => scalar referece to return error message 
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_flexcache_create {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            verify  => 1,
        );

    my %args_validn_spec =
        (
            'server'        => {type => OBJECT},
            'vserver'       => {type => SCALAR},
            'origin-volume' => {type => SCALAR},
            'timeout'       => {type => SCALAR, default => 60},
            'verify'        => {type => SCALAR, default => 0},
            'errmsg'        => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'-Verification'} = $options{'verify'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_flexcache_create( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_flexcache_create() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_flexcache_delete()

create the flexcache for given origin volume

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    origin-volume  => name of origin volume
    errmsg         => scalar referece to return error message
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_flexcache_delete {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'        => {type => OBJECT},
            'vserver'       => {type => SCALAR},
            'origin-volume' => {type => SCALAR},
            'timeout'       => {type => SCALAR, default => 60},
            'errmsg'        => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_flexcache_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_flexcache_delete() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_flexcache_show()

show flexcache for given origin volume

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_flexcache_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'             => {type => OBJECT},
            'vserver'            => {type => SCALAR, optional => 1},
            'cache-volume'       => {type => SCALAR, optional => 1},
            'origin-volume'      => {type => SCALAR, optional => 1},
            'cache-aggregate'    => {type => SCALAR, optional => 1},
            'cache-size'         => {type => SCALAR, optional => 1},
            'cache-state'        => {type => SCALAR, optional => 1},
            'cache-available'    => {type => SCALAR, optional => 1},
            'cache-percent-used' => {type => SCALAR, optional => 1},
            'connection-status'  => {type => SCALAR, optional => 1},
            'origin-aggregate'   => {type => SCALAR, optional => 1},
            'origin-state'       => {type => SCALAR, optional => 1},
            'timeout'            => {type => SCALAR, default => 300},
            'errmsg'             => {type => SCALARREF},
            'cli_op'             => {type => ARRAYREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_flexcache_show( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_flexcache_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_flexcache_cache_policy_create()

create flexcache cache-policy 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_flexcache_cache_policy_create {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'             => {type => OBJECT},
            'vserver'            => {type => SCALAR},
            'policy'             => {type => SCALAR},
            'reg-ttl'            => {type => SCALAR, default => 15},
            'dir-ttl'            => {type => SCALAR, default => 15},
            'timeout'            => {type => SCALAR, default => 300},
            'errmsg'             => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_flexcache_cache_policy_create( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_flexcache_cache_policy_create() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_flexcache_cache_policy_delete()

delete flexcache cache-policy

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_flexcache_cache_policy_delete {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'             => {type => OBJECT},
            'vserver'            => {type => SCALAR},
            'policy'             => {type => SCALAR},
            'timeout'            => {type => SCALAR, default => 300},
            'errmsg'             => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_flexcache_cache_policy_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_flexcache_cache_policy_delete() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_flexcache_cache_policy_modify()

modify flexcache cache-policy

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_flexcache_cache_policy_modify {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'             => {type => OBJECT},
            'vserver'            => {type => SCALAR},
            'policy'             => {type => SCALAR},
            'reg-ttl'            => {type => SCALAR, default => 15},
            'dir-ttl'            => {type => SCALAR, default => 15},
            'timeout'            => {type => SCALAR, default => 300},
            'errmsg'             => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( exists($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_flexcache_cache_policy_modify( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_flexcache_cache_policy_modify() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 volume_flexcache_cache_policy_show()

show flexcache cache-policy

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_flexcache_cache_policy_show {
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'             => {type => OBJECT},
            'vserver'            => {type => SCALAR},
            'policy'             => {type => SCALAR},
            'reg-tt'             => {type => SCALAR, optional => 1},
            'dir-tt'             => {type => SCALAR, optional => 1},
            'timeout'            => {type => SCALAR, default => 300},
            'errmsg'             => {type => SCALARREF},
            'cli_op'             => {type => ARRAYREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_flexcache_cache_policy_show( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_flexcache_cache_policy_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;

}

=head2 aggregate_wafliron_start()

start the wafliron on specified aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    aggregate      => aggregate name
    include_mirror => include mirrors 
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub aggregate_wafliron_start {
    my %options = validate(@_,
                    {
                        server          => {type => OBJECT},
                        aggregate       => {type => SCALAR},
                        include_mirror  => {
                                            type => SCALAR,
                                            optional =>1,
                                            regex => qr/^(true|false)$/,
                                            default => 'false',
                                           },
                        errmsg          => {type => SCALARREF},
                        timeout         => {
                                            type => SCALAR,
                                            optional =>1,
                                            default  => 60,
                                           },
                    });

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'}  = $options{'timeout'};
    $input_arg{'aggregate'} = $options{'aggregate'};
    $input_arg{'include-mirrors'} = $options{'include_mirror'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->storage_aggregate_wafliron_start( %input_arg );
    };

    if ($@) {
        $$err_ref = "aggregate_wafliron_start() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 aggregate_wafliron_show()

retrieve the wafliron status on specified aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub aggregate_wafliron_show {

    my @state_list = qw( online  creating  mounting  quiesced  quiescing
                     unmounted  unmounting  destroying  partial  frozen
                     reverted  restricted  inconsistent iron_restricted 
                     unknown  offline failed );

    my $states = join('|', @state_list);

    my $match_states = qr/^($states)$/;

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'    => {type => OBJECT},
            'aggregate' => {type => SCALAR, optional => 1},
            'state'     =>
                {
                    type => SCALAR, 
                    optional => 1, 
                    regex  => $match_states,
                },
            'iron-status'                => {type => SCALAR, optional => 1},
            'iron-scan-percentage'       => {type => SCALAR, optional => 1},
            'iron-last-start-errno'      => {type => SCALAR, optional => 1},
            'iron-last-start-error-info' => {type => SCALAR, optional => 1},
            'errmsg'                     => {type => SCALARREF},
            'cli_op'                     => {type => ARRAYREF},
            'timeout'                    => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

   foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = undef;
        @{$cmdOpRef} = $server->storage_aggregate_wafliron_show( %input_arg );
    };

    if($@) {
        $$err_ref = "aggregate_wafliron_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 aggregate_wafliron_status()

return the wafliron state on specified aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys 
    server    => server object
    aggregate => aggregate name
    errmsg    => reference to scalar variable to store error mesg
    timeout   => timeout in sec [ default : 300 ]
    iron_info => reference to hash to store iron information [ optional ]

=back

=item Returns:

Returns 0 if failed, otherwise iron states

=back

=cut

sub aggregate_wafliron_status {

    my %args_validn_spec =
        (
            'server'    => {type => OBJECT},
            'aggregate' => {type => SCALAR},
            'errmsg'    => {type => SCALARREF},
            'timeout'   => {type => SCALAR, default => 300},
            'iron_info' => {type => HASHREF, optional => 1},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $aggrName = $options{'aggregate'}; # server object
    my $err_ref  = $options{'errmsg'}; # reference to error msg variable
    my $timeout  = $options{'timeout'}; # reference to status variable 

    my $status ; # stores wafliron status 
    my $err_str; # variable to store error message
    my @cmd_op;  # array to store wafliron_show cmd o/p

    eval {
        aggregate_wafliron_show(
            'server'    => $server,
            'aggregate' => $aggrName,
            'timeout'   => $timeout,
            'errmsg'    => \$err_str,
            'cli_op'    => \@cmd_op,
        ) or die ("aggregate_wafliron_show failed, error: $err_str");
        
        if ( $cmd_op[0]->{'iron-status'} eq "not_running") {
            $status = IRON_NOT_RUNNING;
        } elsif ($cmd_op[0]->{'iron-status'} eq "starting") {
            $status = IRON_STARTING;
        } elsif ($cmd_op[0]->{'iron-status'} eq "scanning") {
            $status = IRON_SCANNING;
        } elsif ($cmd_op[0]->{'iron-status'} eq "finishing") {
            $status = IRON_FINISHING;
        } else {
            $status = IRON_UNKNOWN;
        }

        if( defined $options{'iron_info'} ) {
            %{$options{'iron_info'}} = %{$cmd_op[0]};
        }
    };

    if ($@) {
        $$err_ref = $@;
        return FC_FALSE;
    }

    return $status;
}


=head2 aggregate_wafliron_stop()

stops the wafliron on specified aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server     => server object
    aggregate  => aggregate name
    errmsg     => referece to error scalar variable
    timeout    => timeout in seconds for cli call [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub aggregate_wafliron_stop {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        aggregate   => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional => 1,
                                        default  => 300,
                                       },
                    });

    my $server    = $options{'server'};
    my $aggr_name = $options{'aggregate'};
    my $err_ref   = $options{'errmsg'};
    my $timeout   = $options{'timeout'};

    eval {
        my $status = aggregate_wafliron_status(
                        server => $server,
                        aggregate => $aggr_name,
                        errmsg    => $err_ref,
                        );

        if ( $status == FC_FALSE ) {
            die("aggregate_wafliron_status failed, error: $$err_ref");
        } elsif ( $status != IRON_NOT_RUNNING ) {
            $server->storage_aggregate_wafliron_stop(
                    'aggregate' => $aggr_name,
                    '-Timeout'  => $timeout );
        } else {
            $$err_ref = "Wafliron is not running on aggregate";
        }
    };

    if ($@) {
       $$err_ref = "aggregate_wafliron_stop() failed, error: $@";
       return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 aggregate_wafliron_wait()

wait for wafliron to complete on given aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server     => server object
    aggregate  => aggregate name
    errmsg     => referece to error scalar variable
    timeout    => timeout in seconds for cli call [60]
    interval   => interval in seconds between two calls [1]
    iron_info  => reference to a hash to get wafliron status, if required

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub aggregate_wafliron_wait {

    my %args_validn_spec =
        (
            'server'     => {type => OBJECT},
            'aggregate'  => {type => SCALAR},
            'errmsg'     => {type => SCALARREF},
            'interval'   => {type => SCALAR, default => 1},
            'timeout'    => {type => SCALAR, default => 60},
            'iron_info'  => {type => HASHREF, optional =>1},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $server   = $options{'server'};    # server object
    my $aggrName = $options{'aggregate'}; # aggr name
    my $timeout  = $options{'timeout'};   # timeout for cli command
    my $err_ref  = $options{'errmsg'};    # reference error msg variable
    my $interval = $options{'interval'};  # interval between two calls

    my %iron_info; # hash to store iron information

    my $completed = 0;

    # undef the iron_info hash, if available
    if ( exists $options{'iron_info'} ) {
        %{$options{'iron_info'}} = ();
    }

    my $finish = time() + $timeout;

    while ( time() <=  $finish ) {

        my $status = aggregate_wafliron_status(
                        server    => $server,
                        aggregate => $aggrName,
                        errmsg    => $err_ref,
                        iron_info => \%iron_info,
                        );

        if ( $status == IRON_NOT_RUNNING ) {
            logcomment("Wafliron is not running");
            $completed = 1;
            if ( defined $options{'iron_info'} ) {
                %{$options{'iron_info'}} = %iron_info;
            }
            last;
        } elsif ( $status == FC_FALSE ) {
            logcomment("aggregate_wafliron_status failed, error: $$err_ref");
            return FC_FALSE;
        } else {
            logcomment("Wafliron is running");
            sleep($interval);
        }
    }

    if ( $completed ) {
        return FC_TRUE;
    } else {
        return FC_FALSE;
    }
}

=head2 vserver_show()

Retrieve vserver information

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub vserver_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'                     => {type => OBJECT},
            'vserver'                    => {type => SCALAR, optional => 1},
            'type'                       => {type => SCALAR, optional => 1},
            'uuid'                       => {type => SCALAR, optional => 1},
            'rootvolume'                 => {type => SCALAR, optional => 1},
            'aggregate'                  => {type => SCALAR, optional => 1},
            'ns-switch'                  => {type => SCALAR, optional => 1},
            'nm-switch'                  => {type => SCALAR, optional => 1},
            'nisdomain'                  => {type => SCALAR, optional => 1},
            'rootvolume-security-style'  => {type => SCALAR, optional => 1},
            'ldap-client'                => {type => SCALAR, optional => 1},
            'language'                   => {type => SCALAR, optional => 1},
            'snapshot-policy'            => {type => SCALAR, optional => 1},
            'comment'                    => {type => SCALAR, optional => 1},
            'antivirus-on-access-policy' => {type => SCALAR, optional => 1},
            'quota-policy'               => {type => SCALAR, optional => 1},
            'errmsg'                     => {type => SCALARREF},
            'cli_op'                     => {type => ARRAYREF},
            'timeout'                    => {type => SCALAR, default  => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->vserver_show( %input_arg );
    };

    if($@) {
        $$err_ref = "vserver_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
} ### end sub vserver_show

=head2 vserver_modify()

modify vserver information

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub vserver_modify {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'                     => {type => OBJECT},
            'vserver'                    => {type => SCALAR},
            'ns-switch'                  => {type => SCALAR, optional => 1},
            'nm-switch'                  => {type => SCALAR, optional => 1},
            'snapshot-policy'            => {type => SCALAR, optional => 1},
            'comment'                    => {type => SCALAR, optional => 1},
            'antivirus-on-access-policy' => {type => SCALAR, optional => 1},
            'quota-policy'               => {type => SCALAR, optional => 1},
            'errmsg'                     => {type => SCALARREF},
            'timeout'                    => {type => SCALAR, default  => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->vserver_modify( %input_arg );
    };

    if($@) {
        $$err_ref = "vserver_modify() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 does_vserver_exist()

checks, if given vserver exists

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub does_vserver_exist {
    my %options = validate(@_,
                    {
                        server     => {type => OBJECT},
                        vserver    => {type => SCALAR},
                        errmsg     => {type => SCALARREF},
                        timeout    => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                      },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my %input_arg; # hash of input argument to cli command

    $input_arg{'vserver'} = $options{'vserver'};
    $input_arg{'-Timeout'} = $options{'timeout'};

    eval {
        $server->vserver_show( %input_arg );
    };

    if ($@) {
        $$err_ref = "does_vserver_exist() failed, error: $@";
        return FC_FALSE;
    }
    return FC_TRUE;
}

=head2 does_aggr_exists()

verify given aggregate presence and its attributes

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server         => server object
    vserver        => vserver name
    aggr_name      => aggregate name
    field_hash     => referece to validation hash variable [optional]
    errmsg         => referece to error scalar variable
    timeout        => timeout in seconds [60]

=back

=item Returns:

Returns 0 if aggregate does not exist,
Returns 1 if aggregate exists with correct field hash,
Returns 2 if aggregate exists but with mismatch in field hash 

=back

=cut

sub does_aggr_exists {
    my %options = validate(@_,
                    {
                        server     => {type => OBJECT},
                        aggr_name  => {type => SCALAR},
                        errmsg     => {type => SCALARREF},
                        field_hash => 
                        {
                            type => HASHREF,
                            optional => 1,
                            default =>
                            {
                                'volume-style' => 'flex',
                            },
                                        
                        },
                        timeout => 
                        {
                            type => SCALAR,
                            optional =>1,
                            default  => 60,
                        },
                    });

    my $server   = $options{'server'}; # server obj
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $aggr_name    = $options{'aggr_name'};
    my %in_aggr_info = %{$options{'field_hash'}};

    my %input_arg; # hash of input argument to cli command

    $input_arg{'aggregate'} = $aggr_name;
    $input_arg{'-Timeout'} = $options{'timeout'};

    eval {
        $$err_ref = ""; 

        my @aggrs = $server->storage_aggregate_show( %input_arg );
        my %cli_aggr_info = %{$aggrs[0]};

        foreach my $field ( keys %in_aggr_info ) {
            $cli_aggr_info{$field} =~ s/\"//g;

            if( $in_aggr_info{$field} ne $cli_aggr_info{$field} ) {
                my $mesg = "Mismatch in $field value,"
                                . " act:$cli_aggr_info{$field}"
                                . " exp:$in_aggr_info{$field}\n";
                $$err_ref .= $mesg;
            } else {
                logcomment("*** aggregate key: $field, "
                    . "value: $in_aggr_info{$field} is ok***\n");
            }

        } ### end of foreach
    };

    if ($@) {
        $$err_ref = "$aggr_name aggr does not exist, error: $@";
        return AGGR_DOES_NOT_EXIST;
    }

    if ($$err_ref ne ""){
        return AGGR_EXIST_WITH_FIELD_MISMATCH;
    }

    return AGGR_EXIST;
}


=head2 get_filer_aggr_name()

return the filer aggregate name

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    node_name      => node name
    aggr_name      => aggregate name [ aggr1 ]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_filer_aggr_name {
    my %options = validate(@_,
                    {
                        node_name => {type => SCALAR},
                        aggr_name =>
                        {
                            type => SCALAR, 
                            optional => 1, 
                            default => 'aggr1'
                        },
                    });

    my $node_name = $options{'node_name'};
    my $aggr_name = $options{'aggr_name'};

    $node_name =~ s/-/_/g;
    $aggr_name = $node_name . "_" . $aggr_name;

    return($aggr_name);
}


=head2 get_unit_conversion()

converts given input unit to the required unit

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    input_unit  => input unit of type 100K|100KB|100kb|100mb|100M 
    convert_to  => convert to unit UNIT_B|UNIT_KB|UNIT_MB|UNIT_TB
    output_unit => reference to scalar to store output
    errmsg      => reference to scalar to store error string

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_unit_conversion {
    my %options = validate(@_,
                    {
                        input_unit => {type => SCALAR},
                        convert_to => {type => SCALAR},
                        output_unit => {type => SCALARREF},
                        errmsg      => {type => SCALARREF},
                    });

    my ($size, $unit_to, $unit_from, $op_ref, $err_ref);

    $size    = $options{'input_unit'};
    $unit_to = $options{'convert_to'};
    $op_ref  = $options{'output_unit'};
    $err_ref = $options{'errmsg'};

    eval {
        if ( $size =~ /(\d+)([kmgtKMGT][bB]?)/ ) {

            logcomment("input_unit: $1 is in $2");

            $size =~ s/((\d+)[kmgtKMGT])[bB]?/$1/;

            $unit_from = guess_unit($size)
                or die("input vol size has wrong format,error:$@");

            $size = strip_unit_spec($size);

        } else {
            logcomment("input_unit: $size is in bytes");
            $unit_from = UNIT_B;
        }

        $$op_ref = convert_units($size, $unit_from, $unit_to)
            or die("convert_units failed for input size,error:$@");
    };

    if ($@) {
        $$err_ref = "unit conversion failed, error:$@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 is_space_available_in_aggr()

checks, if required space is available in given aggregate

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys
    server    => server object
    aggrname  => aggregate name
    unitsize  => unit size of required space
    maxunits  => number of maximum size [ default: 1 ]
    errmsg    => reference to scalar to store error string
    timeout   => timeout for cli command [ default: 300 ]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub is_space_available_in_aggr {

    my %args_validn_spec =
        (
            server     => {type => OBJECT},
            aggrname   => {type => SCALAR},
            unitsize   => {type => SCALAR},
            maxunits   => {type => SCALAR, default => 1},
            errmsg     => {type => SCALARREF},
            timeout    => {type => SCALAR, default  => 300},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $server   = $options{'server'};   # server object
    my $aggrname = $options{'aggrname'}; # aggr name
    my $unitsize = $options{'unitsize'}; # required unit size
    my $maxunits = $options{'maxunits'}; # total units
    my $err_ref  = $options{'errmsg'};   # reference error msg variable
    my $timeout  = $options{'timeout'};  # timeout for cli command

    eval {

        my @arr = $server->storage_aggregate_show(
                        'aggregate' => $aggrname,
                        '-Timeout'  => $timeout);

        my $space_avail = $arr[0]->{availsize};

        get_unit_conversion(
                input_unit  => $unitsize,
                convert_to  => UNIT_B,
                output_unit => \$unitsize,
                errmsg      => $err_ref)
                or die("get_unit_conversion() failed, error: $$err_ref");

        my $space_reqd = ($unitsize * $maxunits);

        logcomment("Aggregate available space: $space_avail");
        logcomment("Space required: $space_reqd");
        if ( $space_avail > $space_reqd ) {
            logcomment("Required space is available on $aggrname aggr");
            return FC_TRUE;
        } else {
            die("No space availabe in aggr,"
                . " Avail space:$space_avail Reqd space:$space_reqd");
            return FC_FALSE;
        }
    };

    if ($@) {
        $$err_ref = $@;
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 does_files_exist_in_vol()

checks, if files from given file list are available in
given flexvol

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys
    server    => server object
    volpath   => vol path like /vol/vol_name
    files     => reference to array of input file list
    errmsg    => reference to scalar to store error string
    timeout   => timeout for cli command [ default: 300 ]
    setfiler  => set execution server to setfiler [undef]
    resetfiler=> reset execution server to resetfiler [undef]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub does_files_exist_in_vol {

    my %args_validn_spec =
        (
            server     => {type => OBJECT},
            volpath    => {type => SCALAR},
            files      => {type => ARRAYREF},
            errmsg     => {type => SCALARREF},
            timeout    => {type => SCALAR, default  => 300},
            setfiler   => {type => SCALAR, default => undef},
            resetfiler => {type => SCALAR, default => undef},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $server   = $options{'server'};   # server object
    my $volpath  = $options{'volpath'};  # dblade vol path
    my @fileList = @{$options{'files'}}; # list of input files
    my $err_ref  = $options{'errmsg'};   # scalar ref to store error
    my $timeout  = $options{'timeout'};  # cli timeout
    my $setFiler = $options{'setfiler'}; # set filer name
    my $resetFiler = $options{'resetfiler'}; # reset filer name

    eval {

        #
        # set execution server to setFiler
        #
        if ( defined($setFiler) ) {
            $server->set_execution_server( '-Server' => $setFiler );
        }

        $server->dblade_set_privilege(
            level=> 'test',
            quiet => 'on' );

        my $cmdop = $server->dblade(
                    'cmd'      => "ls $volpath",
                    '-Timeout' =>  $timeout );
        #
        # reset execution server back to resetFiler
        #
        if ( defined($resetFiler) ) {
            $server->set_execution_server( '-Server' => $resetFiler );
        }

        my @opfiles = split('\s+',$cmdop);
        $$err_ref = "";
        
        if ( $cmdop =~ /No such file or directory/ ) {
            $$err_ref .= "file not found at $volpath\n";
        } else {
            foreach my $file (@fileList) {
                if ( not grep(/^$file$/, @opfiles) ) {
                    logcomment("we entered the if"); 
                    $$err_ref .= "$file not found at $volpath\n";
                }
            }
        }
        if ( $$err_ref ne "" ) {
            # we got error
            die("$$err_ref");
        }
    };

    if ($@) {
        $$err_ref = "$@";
        return FC_FALSE;
    }
    return FC_TRUE;
}

=head2 fill_aggr()

fills the given aggregate space by creating tmp flexvol
until required space is left on the aggregate.

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys
    server         => server object
    vserver        => vserver name
    aggrname       => aggregate name
    reqd_space     => required free space in aggregate
    tmpvolname     => name of temp flexvol to be created
    aggrfreespace  => available space in aggregate after adjustment
    files          => reference to array of input file list
    errmsg         => reference to scalar to store error string
    timeout        => timeout for cli command [ default: 300 ]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub fill_aggr {

    my %args_validn_spec =
        (
            server        => {type => OBJECT},
            vserver       => {type => SCALAR},
            aggrname      => {type => SCALAR},
            reqd_space    => {type => SCALAR},
            tmpvolname    => {type => SCALAR},
            aggrfreespace => {type => SCALARREF, optional => 1},
            errmsg        => {type => SCALARREF},
            timeout       => {type => SCALAR, default  => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my $server     = $options{'server'};   # server object
    my $vserver    = $options{'vserver'};  # vserver name
    my $aggrName   = $options{'aggrname'}; # aggr name path
    my $volName    = $options{'tmpvolname'}; # tmp vol name
    my $reqd_space = $options{'reqd_space'}; # space required in aggregate
    my $err_ref    = $options{'errmsg'}; # ref to scalar to store error
    my $timeout    = $options{'timeout'}; # cli timeout

    eval {

        my (@vols, $vol_size, $err_str);

        my @arr = $server->storage_aggregate_show(
                            aggregate => $aggrName,
                            '-Timeout' => $timeout);

        my $space_avail = $arr[0]->{availsize};

        get_unit_conversion(
                input_unit  => $reqd_space,
                convert_to  => UNIT_B,
                output_unit => \$reqd_space,
                errmsg      => $err_ref)
                or die("get_unit_conversion() failed, error: $$err_ref");

        volume_create(
            'server'          => $server,
            'vserver'         => $vserver,
            'volume'          => $volName,
            'aggregate'       => $aggrName,
            'size'            => '20m',
            'space-guarantee' => 'volume',
            'type'            => "RW",
            'security-style'  => "mixed",
            'errmsg'          => $err_ref,
        ) or die("$volName volume creation FAILED, error: $$err_ref");

        while ( $reqd_space <= $space_avail ) {

            $space_avail = int ($space_avail/2);

            volume_show(
                server  => $server,
                vserver => $vserver,
                volume  => $volName,
                cli_op  => \@vols,
                errmsg  => $err_ref)
                or die("volume_show failed, error:$$err_ref");

            $vol_size = $vols[0]->{size}; # there is only one entry

            $vol_size += $space_avail;

            volume_modify(
                server  => $server,
                vserver => $vserver,
                volume  => $volName,
                size    => $vol_size,
                errmsg  => $err_ref)
                or die("volume_modify failed, error: $$err_ref");

            @arr = $server->storage_aggregate_show(
                            aggregate => $aggrName,
                            '-Timeout' => $timeout);

            $space_avail = $arr[0]->{availsize};

        } ## end while

        if ( defined $options{'aggrfreespace'} ) {
            ${$options{'aggrfreespace'}} = $space_avail;
        }
    };

    if ($@) {
        $$err_ref = $@;
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 is_corruption_found_in_ironlogs()

checks for corruption or error message in given wafliron logs

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    logs => iron logs

=back

=item Returns:

Returns 1 if corruption found, otherwise 0

=back

=cut

sub is_corruption_found_in_ironlogs {

    my %args_validn_spec =
        (
            logs   => {type => SCALAR},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $op = $options{'logs'};

    if ( $op =~ /(wafl\.check\.info\:ALERT)/ ||
         $op =~ /(wafl\.check\.info\:error)/ ||
        ($op =~ /(wafl\.iron.*:CRITICAL)/ &&
             $op !~ /wafl\.iron\.oc\.[deletedChangeLog|noCorruption]\:CRITICAL/)) {
        logcomment("Corruption matching string: $1");
        return FC_TRUE;
    }
    return FC_FALSE;
}

=head2 set_flags()

set given flags in dblade cli

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server    => server object
    filer     => filer name
    priv      => filer priv mode
    flag_hash => reference to hash of flag { flag_name => value }
    errmsg    => reference to scalar to save error message
    timeout   => cli timeout [300]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub set_flags {

   my %args_validn_spec =
        (
            server    => {type => OBJECT},
            filer     => {type => SCALAR},
	    priv      => {type => SCALAR, optional => 1,
			  default => "diag"}, 
            flag_hash => {type => HASHREF},
            errmsg    => {type => SCALARREF},
            timeout   => {type => SCALAR, default  => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my $server   = $options{'server'};   # server object
    my $filer    = $options{'filer'};  # filer name
    my $priv     = $options{'priv'};  # filer priv mode
    my %flaghash = %{$options{'flag_hash'}}; # hash of flags to be set
    my $err_ref  = $options{'errmsg'}; # scalar reference to store error
    my $timeout  = $options{'timeout'}; # cli timeout

    my ( $cmd, $value);

    foreach my $flag ( keys %flaghash ) {
        $value = $flaghash{$flag};
        $cmd = "\"priv set $priv; setflag  $flag $value\"";
        eval  {
            $server->system_node_run(
                        'node'     => $filer,
                        'command'  => $cmd,
                        '-Timeout' => $timeout);
            logcomment("$flag is set to $value successfully");
        };

        if ($@)  {
            $$err_ref = "Could not set $flag, error: $@";
            return FC_FALSE;
        }
    } ## end foreach

    return FC_TRUE;
}

=head2 get_vserver_data_ip_list()

retrieve the data ip address for given vserver and filer name

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server     => server object
    vserver    => vserver name
    home_node  => filer name
    role       => interface role [data]
    dataip     => reference to array to return data ip list 
    cli_op     => reference to array to return cli op  [optional]
    errmsg     => reference to scalar to save error message
    timeout    => cli timeout [300]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_vserver_data_ip_list {

    my %args_validn_spec =
        (
            'server'    => {type => OBJECT},
            'vserver'   => {type => SCALAR},
            'home_node' => {type => SCALAR, optional => 1},
            'errmsg'    => {type => SCALARREF},
            'dataip'    => {type => ARRAYREF},
            'cli_op'    => {type => ARRAYREF, optional => 1},
            'role'      => {type => SCALAR, default => 'data'},
            'timeout'   => {type => SCALAR, default  => 300 },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command
    
    my @ifList; # list of interface recvd from cli o/p

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $ip_ref   = $options{'dataip'}; # refernce to data ip returning list 

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'server'}   = $options{'vserver'};
    $input_arg{'role'}     = $options{'role'};

    if ( defined $options{'home_node'} ) {
        $input_arg{'home-node'} = $options{'home_node'};
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @ifList = $server->network_interface_show( %input_arg );

        if ( defined $options{'cli_op'} ) {
            @{$options{'cli_op'}} = @ifList;
        }
    };

    if($@) {
        $$err_ref = "network_interface_show() failed, error: $@";
        return FC_FALSE;
    }

    foreach my $entry ( @ifList ) {
        if ( $entry->{'status-oper'} eq "up" ) {
            push(@{$ip_ref}, $entry->{'address'});
        }
    }

    if ( ! @{$ip_ref} || ( scalar(@{$ip_ref}) == 0) ) {
        $$err_ref = "No valid data ip found for vserver: $options{'vserver'}";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 get_vol_count_by_node()

calculate the number of existing flexvols for given filer/node

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server         => server object
    filer          => filer name
    volcount       => scalr reference to return vol count 
    aggr_vol_hash  => hash reference to return aggr<->vol data [optional]
    errmsg         => scalar reference to return error message
    timeout        => cli timeout [600]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_vol_count_by_node {

    my %args_validn_spec =
        (
            'server'        => {type => OBJECT},
            'filer'         => {type => SCALAR},
            'volcount'      => {type => SCALARREF},
            'aggr_vol_hash' => {type => HASHREF, optional => 1},
            'errmsg'        => {type => SCALARREF},
            'timeout'       => {type => SCALAR, default => 600},
        );

    my %options = validate(@_, \%args_validn_spec);

    my $server       = $options{'server'}; # server object
    my $filer        = $options{'filer'};  # filer name
    my $vol_cnt_ref  = $options{'volcount'}; # refernce to vol count variable
    my $err_ref      = $options{'errmsg'}; # refernce to error msg variable
    my $ip_ref       = $options{'dataip'}; # refernce to data ip returning list
    my $timeout      = $options{'timeout'}; # cli timeout

    my (%aggr_vol_hash);

    $$vol_cnt_ref = 0;

    eval {

        my @aggrs = $server->storage_aggregate_show(
                        'nodes' => $filer,
                        '-Timeout' => $timeout,);

        foreach my $entry ( @aggrs ) {
            my $aggr = $entry->{'aggregate'};

            eval {
                my @vols = $server->volume_show(
                                'aggregate' => $aggr,
                                '-Timeout' => $timeout,);

                my @vol_list; # tmp vol list

                foreach my $entry ( @vols ) {
                    push(@vol_list, $entry->{'volume'});
                }

                $$vol_cnt_ref += scalar(@vol_list);
                $aggr_vol_hash{$aggr}{count}   = scalar(@vol_list);
                $aggr_vol_hash{$aggr}{vollist} = \@vol_list;
            };

            if ($@) {
                logcomment("$aggr does not have any volumes");
                $aggr_vol_hash{$aggr}{count}   = 0;
                $aggr_vol_hash{$aggr}{vollist} = [];
            }

        } ## end foreach
    };

    if ( $@ ) {
        $$err_ref = "Error: $@" ;
        return FC_FALSE;
    }

    if ( defined $options{'aggr_vol_hash'} ) {
        %{$options{'aggr_vol_hash'}} = %aggr_vol_hash;
    }

    return FC_TRUE;
}

=head2 start_mlog_msg_reader()

start mlog message reader

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server => server object

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub start_mlog_msg_reader {

    my %args_validn_spec =
        (
            server  => {type => OBJECT},
        );

    my %options  = validate(@_, \%args_validn_spec);
    my $server   = $options{'server'};  # server object
    my $out;

    if ( 1 == $mlog_rdr_started ) {
        # reader already started
        logcomment("Reader is already started");
        return FC_FALSE;
    }

    $mlog_msg_rdr_obj = LogReader::MlogMessages->new($server);

    $server->shell(
        'cmd'        => "echo \"\" > /mroot/etc/mlog/messages.log",
        'output'     => \$out,
        '-Noverbose' => 1);

    $mlog_msg_rdr_obj->start();

    $mlog_rdr_started = 1;

    return FC_TRUE;
}

=head2 stop_mlog_msg_reader()

stop mlog message reader and set message logs to output variable

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    output =>  a scalar reference to have message logs
    errmsg =>  a scalar reference to have error message 

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub stop_mlog_msg_reader {

    my %args_validn_spec =
        (
            output   => {type => SCALARREF},
            errmsg   => {type => SCALARREF},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $op_ref  = $options{'output'}; # reference to output variable
    my $err_ref = $options{'errmsg'}; # reference to error msg variable


    if ( 1 != $mlog_rdr_started ) {
        $$err_ref = "log reader not started";
        return FC_FALSE;
    }

    $mlog_msg_rdr_obj->stop();

    $mlog_rdr_started = 0;

    $$op_ref = $mlog_msg_rdr_obj->to_string();

    $mlog_msg_rdr_obj = undef;

    return FC_TRUE;
}

=head2 get_wafl_scan_speed()

retrieve the wafl scanner speed from dblade 

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server =>  Server object
    filer  =>  node name
    output =>  scalar reference to store scanner speed
    errmsg =>  scalar reference to store error message

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_wafl_scan_speed {

    my %args_validn_spec =
        (
            server   => {type => OBJECT},
            filer    => {type => SCALAR},
            output   => {type => SCALARREF},
            errmsg   => {type => SCALARREF},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $server  = $options{'server'}; # server object
    my $filer   = $options{'filer'};  # node name
    my $op_ref  = $options{'output'}; # reference to output variable
    my $err_ref = $options{'errmsg'}; # reference to error msg variable

    eval {
        my $op;
        $server->system_node_run(
            'output'  => \$op,
            'node'    => $filer,
            'command' => "wafl scan speed");

        if ($op =~ m/WAFL\s*scan\s*speed\s*is\s*(\d+)\s*/) {
            $$op_ref = $1;
        } else {
            die("Did not get scan speed");
        }

    };

    if ($@) {
        $$err_ref = "Error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 set_wafl_scan_speed()

set the wafl scanner speed

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server =>  Server object
    filer  =>  node name
    speed  =>  scanner speed to be set
    errmsg =>  scalar reference to store error message

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub set_wafl_scan_speed {

    my %args_validn_spec =
        (
            server   => {type => OBJECT},
            filer    => {type => SCALAR},
            speed    => {type => SCALAR},
            errmsg   => {type => SCALARREF},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $server  = $options{'server'}; # server object
    my $filer   = $options{'filer'};  # node name
    my $speed   = $options{'speed'};  # scanner speed
    my $err_ref = $options{'errmsg'}; # reference to error msg variable

    eval {
        $server->system_node_run(
            'node'    => $filer,
            'command' => "wafl scan speed $speed");

    };

    if ($@) {
        $$err_ref = "Error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 get_cluster_name()

retrieve the cluster name

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server =>  Server object
    name   =>  scalar reference to store cluster name 
    errmsg =>  scalar reference to store error message

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_cluster_name {
    my %args_validn_spec =
        (
            server   => {type => OBJECT},
            name     => {type => SCALARREF},
            errmsg   => {type => SCALARREF},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $server  = $options{'server'}; # server object
    my $name    = $options{'name'};   # scalar ref to return cluster name 
    my $err_ref = $options{'errmsg'}; # scalar ref to return error msg 

    eval {
        $$name = $server->cluster_identity_show()->[0]->{name};
    };

    if ($@) {
        $$err_ref = "Error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 get_mkfile_filename_list()

return a list of file names similar to file names
created by mkfile command on filer

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    filename   =>  file prefix name used to create files 
    filecount  =>  total number of files

    example :
         mkfile 4k /vol/TestVol/TstFile 100
         filename  => "TstFile"
         filecount => 100 
=back

=item Returns:

Returns an array of file names 

=back

=cut

sub get_mkfile_filename_list {

   my %args_validn_spec =
        (
            filename  => {type => SCALAR},
            filecount => {type => SCALAR},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $filename  = $options{'filename'}; # file name prefix 
    my $filecount = $options{'filecount'}; # total number of files

    my (@file_list, $tmpname, $file_extn, $len, $loop);

    $len = length($filecount - 1);

    if ( $filecount <= 1 ) {
        push @file_list, $filename;
    } else {
        for ( $loop = 0; $loop < $filecount; $loop += 1 ) {
            $file_extn = ("0" x ($len - length($loop))) . "$loop";
            $tmpname = $filename . ".$file_extn";
            push @file_list, $tmpname;
        }
    }

    return @file_list;
}

=head2 get_flexcache_stat()

retrieve flexcache stat for given file and flexcache
and return local block counts in scalar referece variable

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server     =>  server object 
    flexcache  =>  flexcache name
    filename   =>  file name 
    filer      =>  filer name 
    errmsg     =>  scalar reference to return error message 
    local_blks =>  scalar reference to return local blks count  
    timeout    =>  timeout for cli command [300] 

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_flexcache_stat {

    my %args_validn_spec =
        (
            server     => {type => OBJECT},
            flexcache  => {type => SCALAR},
            filename   => {type => SCALAR},
            filer      => {type => SCALAR},
            errmsg     => {type => SCALARREF},
            local_blks => {type => SCALARREF},
            timeout    => {type => SCALAR, default => 300},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $server    = $options{'server'};    # server object
    my $flexcache = $options{'flexcache'}; # flexcache name
    my $filename  = $options{'filename'};  # filename vol
    my $filer     = $options{'filer'};     # filer name
    my $timeout   = $options{'timeout'};   # timeout for cli command
    my $err_ref   = $options{'errmsg'};    # reference error msg variable
    my $loc_blks_ref  = $options{'local_blks'};   # reference error msg variable

    my $cmd = "flexcache fstat /vol/$flexcache/$filename";

    my $op; # cli output after command execution

    eval {

        $server->set_execution_server(-Server => $filer);
    
        $server->system_node_run(
            'node'   =>  $filer,
            'command'  => $cmd,
            '-Timeout' => $timeout,
            'output'   => \$op );

        logcomment("$flexcache Stats: \n");
        logcomment(Dumper $op);

        if ( $op =~ /Object does not exist/i ) {
            logcomment("Error: $filename not found in $flexcache flexcache");
            die("Error: $filename not found in $flexcache flexcache");
        }

        $op =~ /Filename:.*\/(.*?)\n/;
        if ( $1 eq $filename ) {
            logcomment("file read from flexcache is matched with input");
        } else {
            die("file read from flexcache is not matched with input");
        }

        $op =~ /FlexCache Local Blocks:\s*(\d+).*\n/;

        $$loc_blks_ref = $1;
    };

    if ($@) {
        $$err_ref = "Error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}

=head2 open_remote_nfs_client_connection()

opens a connection to remote NFS client
and returns a connection handle to a scalar reference variable

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    client     =>  nfs client name
    errmsg     =>  scalar reference to return error message
    handle     =>  scalar reference to return connection object 
    timeout    =>  timeout for cli command [300]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub open_remote_nfs_client_connection {

    my %args_validn_spec =
        (
            client  => {type => SCALAR},
            errmsg  => {type => SCALARREF},
            handle  => {type => SCALARREF},
            timeout => {type => SCALAR, default => 300},
        );

    my %options  = validate(@_, \%args_validn_spec);

    my $client    = $options{'client'};
    my $err_ref   = $options{'errmsg'};
    my $timeout   = $options{'timeout'};

    eval {

        my $connObj;

        $connObj = new Client(
                        -host    => $client,
                        -timeout => $timeout,
                        -recreate_stale => "false",
                        -protocol => "nfs");

        if ($connObj->error) {
            logcomment("Client conn error: " . $connObj->errmsg);
            die("Problem in connecting to $client, error:" . $connObj->errmsg);
        }

        ${$options{'handle'}} = $connObj;
    };

    if ($@) {
        $$err_ref = "Error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 aggregate_show()

shows aggregate information for give aggregate name

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server      =>  server object
    aggregate   =>  name of aggregate
    errmsg      =>  scalar reference to return error string
    cli_op      =>  array  reference to return aggregate information 
    timeout     =>  cli command timeout [60] 

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub aggregate_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'    => {type => OBJECT},
            'aggregate' => {type => SCALAR},
            'timeout'   => {type => SCALAR, default => 60},
            'errmsg'    => {type => SCALARREF},
            'cli_op'    => {type => ARRAYREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {

        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->storage_aggregate_show( %input_arg );
    };

    if($@) {
        $$err_ref = "aggregate_show() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 disableHA()

disable HA on given node, if it is enabled

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server      =>  server object
    node        =>  name of node
    ha_state    =>  scalar reference to return HA state before disabling 
    errmsg      =>  scalar reference to return error string

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub disableHA {

    my %args_validn_spec =
        (
            'server'    => {type => OBJECT},
            'node'      => {type => SCALAR},
            'ha_state'  => {type => SCALARREF, optional => 1},
            'errmsg'    => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my $server   = $options{'server'}; # server object
    my $filer    = $options{'node'};   # node name
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # reset the error message
    $$err_ref = "none";

    eval {

        my (@op, $ha_state);

        $ha_state = HA_DISABLE;

        @op = $server->cluster_ha_show();

        if ( $op[0]->{'configured'} eq "true" ) {
            logcomment("HA is enabled, so disabling it..");

            $ha_state = HA_ENABLE;

            $server->cluster_ha_modify(configured => 'false');

            $server->storage_failover_modify(node    => $filer,
                                             enabled => 'false');
        } else {
            logcomment("HA is already disabled");
        }

        if ( exists $options{'ha_state'} ) {
            ${$options{'ha_state'}} = $ha_state;
        }
    };

    if($@) {
        $$err_ref = "disableHA() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 enableHA()

enable HA on given node

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server      =>  server object
    node        =>  name of node
    errmsg      =>  scalar reference to return error string

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub enableHA {

    my %args_validn_spec =
        (
            'server'    => {type => OBJECT},
            'node'      => {type => SCALAR},
            'errmsg'    => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my $server   = $options{'server'}; # server object
    my $filer    = $options{'node'};   # node name
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # reset the error message
    $$err_ref = "none";

    eval {

        logcomment("Enabling HA..");

        $server->cluster_ha_modify(configured => 'true');

        $server->storage_failover_modify(node    => $filer,
                                         enabled => 'true');
        logcomment("HA is enabled");
    };

    if($@) {
        $$err_ref = "enableHA() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


=head2 send_interface_home()

send given interface in given vserver to the home port

=over 4

=item Arguments:

=over 4

takes a hash as input with following keys

    server      =>  server object
    vserver     =>  vserver name [ Default: * ]
    lif         =>  lif     name [ Default: * ]
    errmsg      =>  scalar reference to return error string
    timeout     =>  cli command execution timeout [ Default: 120] 

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub send_interface_home {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'    => {type => OBJECT},
            'vserver'   => {type => SCALAR, default => "*"},
            'lif'       => {type => SCALAR, default => "*"},
            'timeout'   => {type => SCALAR, default => 120},
            'errmsg'    => {type => SCALARREF},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    foreach my $key ( keys %args_validn_spec ) {

        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->network_interface_revert( %input_arg );
    };

    if($@) {
        $$err_ref = "network_interface_revert() failed, error: $@";
        return FC_FALSE;
    }

    return FC_TRUE;
}


1;

=head1 SEE ALSO

    None

=head1 STATUS

    Shared

=head1 AUTHOR

    Tarun Garg (tarung@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Cmode/Flexclone.pm#2 $
    Last $Change: 2013620 $
    Last Revised $DateTime: 2013/04/16 03:29:39 $

=head1 HISTORY

    2009-12-24 author: tarung      original lib created

=head1 COPYRIGHT

Copyright 2009, Network Appliance., All Rights Reserved

=cut

