# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Cmode/License.pm#1 $
#
# $Change: 1917575 $
#
# $DateTime: 2013/01/22 11:22:46 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      License.pm - provide interfaces to C-Mode license cli command 
#
# DESCRIPTION
#      This perl module provide wrapper interfaces to c-mode license cli command
#
#
# AUTHOR
#       dl-nb-capacity-qa@netapp.com
#
#
#
## @pod here
##
##

=pod

=head1 NAME

    License.pm -  provide interfaces to C-Mode license cli command

=head1 SYNOPSIS

    use Cmode::License qw(:all);
    or
    use Cmode::License qw(:license);

    use Server;

    my $Server = Server->new() or die("Unable to connect to Server");

    my ($code, $errmsg, %info, @info, $feature);
    get_license_code(
        feature => "FlexClone",
        errmsg  => \$errmsg,
        code    => \$code)
        or die(" get_license_code failed(), $errmsg");

    get_feature_name_by_code(
        feature => \$feature,
        errmsg  => \$errmsg,
        code    => $code)
        or die("get_feature_name_by_code failed(), $errmsg");

    license_add(
        server => $Server,
        code => $code,
        errmsg  => \$errmsg)
        or die("license_add failed, error: $errmsg");

   license_delete(
        server => $Server,
        feature => "FlexClone",
        errmsg  => \$errmsg)
        or die("license_delete() failed, error: $errmsg");

    is_license_configured(
        server => $Server,
        feature => "FlexClone",
        errmsg  => \$errmsg,
        licinfo => \%info)
        or die("is_license_configured failed, error: $errmsg");

    license_show(
        server => $Server,
        errmsg  => \$errmsg,
        cli_op  => \@info)
        or die("license_show failed, error: $errmsg");

    configure_licenses(
        server => $Server,
        features => [ "FlexClone","NFS" ],
        errmsg  => \$errmsg)
        or die("configure_licenses() failed, error: $errmsg");
    OR
    configure_licenses(
        server => $Server,
        features => "FlexClone",
        errmsg  => \$errmsg)
        or die("configure_licenses() failed, error: $errmsg");

    cleanup_license(
        server => $Server,
        errmsg  => \$errmsg)
        or die("cleanup_license() failed, error: $errmsg");

=head1 DESCRIPTION

    This library implements wrapper interface around license cli commands and 
    also provide useful utility interface.

=head1 FUNCTIONS

Cmode License Functions

=cut

package Cmode::License ;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($Validation, $VERSION);

#
# Define constants
#
use constant LC_ERROR => undef;
use constant LC_TRUE  => 1;
use constant LC_FALSE => 0;


use warnings;
use strict qw(vars subs refs);
use base   qw(Exporter);

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $Validation = 1; # unused as of now
    $VERSION    = '0.01';

    my @license_sub = qw(
                        &license_add
                        &license_delete
                        &license_show
                        &is_license_configured
                        &get_license_code
                        &get_feature_list
                        &get_feature_name_by_code
                        &configure_licenses
                        &cleanup_license
                    );

    my @constants = qw(LC_ERROR LC_TRUE LC_FALSE);

    our %EXPORT_TAGS = (
        'license'   => [ @license_sub ],
        'constants' => [ @constants ],
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate qw(validate SCALAR ARRAYREF HASHREF
                         SCALARREF OBJECT CODEREF);

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub license_add;
sub license_delete;
sub license_show;
sub is_license_configured;
sub get_license_code;
sub get_feature_list;
sub get_feature_name_by_code;
sub configure_licenses;
sub cleanup_license;


#------------------------------------------------------
#     Local Variables
#------------------------------------------------------

#
# Add new feature license code here
# 
#
my %license_code = (
                        '1-80-123456' => {
                            'CDMI' 	      => 'WSQFRWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'CIFS' 	      => 'MJXMPWDUSAFDAAAAAAAAAAAAAAAA',
                            'FCP' 	      => 'KYSQQWDUSAFDAAAAAAAAAAAAAAAA',
                            'FlexClone'   => 'GCKYSWDUSAFDAAAAAAAAAAAAAAAA',	
                            'iSCSI'       => 'YDVBQWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'NFS' 	      => 'APZXOWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapLock' 	  => 'ERFCUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapManagerSuite' => 	'QLDRUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapMirror'  =>  'UHMJSWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapProtectApps' => 'CGBGVWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapRestore' => 'INOURWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapVault'   => 'SWHNTWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'V_Series' 	  => 'AXFTWDBBVAAAAAAAAAAAAAAAAAAA',
                        },

                        '1-80-000011' => {
                            'CDMI' 	      => 'WSQFRWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'CIFS' 	      => 'MJXMPWDUSAFDAAAAAAAAAAAAAAAA',
                            'FCP' 	      => 'KYSQQWDUSAFDAAAAAAAAAAAAAAAA',
                            'FlexClone'   => 'GCKYSWDUSAFDAAAAAAAAAAAAAAAA',	
                            'iSCSI'       => 'YDVBQWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'NFS' 	      => 'APZXOWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapLock' 	  => 'ERFCUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapManagerSuite' => 	'QLDRUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapMirror'  =>  'UHMJSWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapProtectApps' => 'CGBGVWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapRestore' => 'INOURWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapVault'   => 'SWHNTWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'V_Series' 	  => 'AXFTWDBBVAAAAAAAAAAAAAAAAAAA',
                        },

                        '1-80-000013' => {
                            'CDMI' 	      => 'WSQFRWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'CIFS' 	      => 'MJXMPWDUSAFDAAAAAAAAAAAAAAAA',
                            'FCP' 	      => 'KYSQQWDUSAFDAAAAAAAAAAAAAAAA',
                            'FlexClone'   => 'GCKYSWDUSAFDAAAAAAAAAAAAAAAA',	
                            'iSCSI'       => 'YDVBQWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'NFS' 	      => 'APZXOWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapLock' 	  => 'ERFCUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapManagerSuite' => 	'QLDRUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapMirror'  =>  'UHMJSWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapProtectApps' => 'CGBGVWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapRestore' => 'INOURWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapVault'   => 'SWHNTWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'V_Series' 	  => 'AXFTWDBBVAAAAAAAAAAAAAAAAAAA',
                        },

                        '1-80-000017' => {
                            'CDMI' 	      => 'WSQFRWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'CIFS' 	      => 'MJXMPWDUSAFDAAAAAAAAAAAAAAAA',
                            'FCP' 	      => 'KYSQQWDUSAFDAAAAAAAAAAAAAAAA',
                            'FlexClone'   => 'GCKYSWDUSAFDAAAAAAAAAAAAAAAA',	
                            'iSCSI'       => 'YDVBQWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'NFS' 	      => 'APZXOWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapLock' 	  => 'ERFCUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapManagerSuite' => 	'QLDRUWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapMirror'  =>  'UHMJSWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapProtectApps' => 'CGBGVWDUSAFDAAAAAAAAAAAAAAAA',
                            'SnapRestore' => 'INOURWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'SnapVault'   => 'SWHNTWDUSAFDAAAAAAAAAAAAAAAA', 	
                            'V_Series' 	  => 'AXFTWDBBVAAAAAAAAAAAAAAAAAAA',
                            'V_Series' 	  => 'AXFTWDBBVAAAAAAAAAAAAAAAAAAA',
                        },

                    );

#
# list of license configured by test
#
my %license_cfged_by_test;


#------------------------------------------------------
#     Subroutines Definition 
#------------------------------------------------------

=head2 license_add

 add the given license code

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server  => server object
    code    => license code 
    errmsg  => reference to scalar to store error message
    timeout => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub license_add {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        code        => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                    });

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $code     = $options{'code'};   # license code 

    # set the input arguments to cli command
    $input_arg{'-Timeout'}     = $options{'timeout'};
    $input_arg{'license-code'} = $options{'code'};

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->license_add( %input_arg );
    };

    if($@) {
        $$err_ref = "license_add() failed, error: $@";
        return LC_FALSE;
    }

    return LC_TRUE;
}


=head2 license_delete

deletes the license for given feature

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server  => server object
    feature => feature  name
    errmsg  => reference to scalar to store error message
    timeout => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub license_delete {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        feature     => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        timeout     => {
                                        type => SCALAR,
                                        optional =>1,
                                        default  => 60,
                                       },
                    });

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $feature  = $options{'feature'}; # feature name

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'package'}  = $options{'feature'};

    $input_arg{'-Verification'}=undef;

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->license_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "license_delete() failed, error: $@";
        return LC_FALSE;
    }

    return LC_TRUE;
}

=head2 license_show

show license configured on the filer 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub license_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1, 
        );

    my %args_validn_spec = 
        (
            'server'           => {type => OBJECT},
            'package'          => {type => SCALAR, optional => 1},
            'license-code'     => {type => SCALAR, optional => 1},
            'description'      => {type => SCALAR, optional => 1},
            'serial-number'    => {type => SCALAR, optional => 1},
            'limit'            => {type => SCALAR, optional => 1},
            'expiration-date'  => {type => SCALAR, optional => 1},
            'errmsg'           => {type => SCALARREF},
            'cli_op'           => {type => ARRAYREF},
            'timeout'          => {
                                    type => SCALAR,
                                    optional =>1,
                                    default  => 300,
                                  },
        );
                        
    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";
    eval {
        @{$cmdOpRef} = $server->license_show( %input_arg );
    };

    if($@) {
        $$err_ref = "license_show() failed, error: $@";
        return LC_FALSE;
    }

    return LC_TRUE;
}

=head2 is_license_configured

verify if license for a given feature is enabled

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server  => server object
    feature => feature  name
    errmsg  => reference to scalar to store error message
    licinfo => reference to  hash to have license information [ optional ]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub is_license_configured {
    my %options = validate(@_,
                    {
                        server  => {type => OBJECT},
                        feature => {type => SCALAR},
                        errmsg  => {type => SCALARREF},
                        licinfo => {type => HASHREF, optional => 1},
                    });

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $feature  = $options{'feature'}; # feature name
    
    my @lic_info;

    eval {
        license_show(
            server => $server,
            package => $feature,
            errmsg  => $err_ref,
            cli_op  => \@lic_info)
            or die("license_show() failed, error: $$err_ref");
        
        if ( defined($options{'licinfo'}) ) {
            %{$options{'licinfo'}} = %{$lic_info[0]};
        } 
    };

    if ($@) {
        $$err_ref = $@;
        return LC_FALSE;
    }
    return LC_TRUE;
}

=head2 configure_licenses

configure licenses for a given features list and
keep the track of license configured by test

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server   => server object
    features => reference to array of features name
    errmsg   => reference to scalar to store error message

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub configure_licenses {

    my %options = validate(@_,
                    {
                        server   => {type => OBJECT},
                        features => {type => SCALAR | ARRAYREF},
                        errmsg   => {type => SCALARREF},
                    });

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $features = $options{'features'}; # feature name or list

    my @featList; # list of features
    my $err_msg;  # scalar to store error mesg
    my $code;     # scalar to store license code

    if ( ref($features) eq "ARRAY") {
      push(@featList, @{$features});
    } else {
      push(@featList, $features);
    }

    eval {

        logcomment("configuring license for given features list");
        foreach my $feature ( @featList ) {
            logcomment("enabling license for $feature");

            if ( is_license_configured(
                    server => $server,
                    feature => $feature,
                    errmsg  => \$err_msg) ) {
                logcomment("license for $feature is already enabled");
            } else {
                get_license_code(
                    server  => $server,
                    feature => $feature,
                    errmsg  => \$err_msg,
                    code    => \$code)
                    or die("get_license_code() failed, error: $err_msg");

                license_add(
                    server => $server,
                    code   => $code,
                    errmsg  => \$err_msg)
                    or die("license_add() failed for $feature,"
                            . " error: $err_msg");

                # push the feature in tracking hash
                $license_cfged_by_test{$feature} = $code;
                logcomment("license for $feature is enabled");
            }
        }
    };
    if ($@) {
        $$err_ref = $@;
        return LC_FALSE;
    }

    return LC_TRUE;
}

=head2 cleanup_license

cleanup licenses, which were configured by configure_licenses() call

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server   => server object
    errmsg   => reference to scalar to store error message

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub cleanup_license {
    my %options = validate(@_,
                    {
                        server   => {type => OBJECT},
                        errmsg   => {type => SCALARREF},
                    });

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    my $error = "";

    my $failCount = 0;

    eval {

        logcomment("performing license cleanup..");
        foreach my $feature ( keys %license_cfged_by_test ) {
            logcomment("disabling license for $feature");
            if ( license_delete(
                    server  => $server,
                    feature => $feature,
                    errmsg  => $err_ref) ) {
                logcomment("license for $feature is disabled");
            } else {
                $error .= "$feature license delete failed, error: $$err_ref\n";
                $failCount++;
            }

            # delete the feature from tracking hash
            delete $license_cfged_by_test{$feature};
        }
        die("$error") if ( $failCount );
    };

    if ($@) {
        $$err_ref = $@;
        return LC_FALSE;
    }

    return LC_TRUE;
}

=head2 get_license_code

return the license code for given feature

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server     =>  server object [optional]
    serial_nbr => serial number [ 1-80-000011 ]
    feature    => features name
    errmsg     => reference to scalar to store error message
    code       => reference to scalar to store license code 

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub get_license_code {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT, optional => 1},
                        serial_nbr  => {type => SCALAR, default => '1-80-000011'},
                        feature     => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        code        => {type => SCALARREF},
                    });
    my $feature    = $options{'feature'}; # feature name
    my $err_ref    = $options{'errmsg'};  # refernce to scalar to store error
    my $code_ref   = $options{'code'};    # refernce to scalar to store code
    my $serial_nbr = $options{'serial_nbr'}; # default serial number
    my $server     = $options{'server'}; # server object

    #
    # if server object is given, then get serial nbr
    # from filer, otherwise use base serial number
    # as 1-80-000011, as it is used by deux_init
    #
    eval {

        if ( defined($options{'server'}) ) {
            my ($error, @lic_info);

            license_show(
                server => $server,
                package => "Base",
                errmsg  => \$error,
                cli_op  => \@lic_info)
                or die("license_show() failed, error: $error");

            $serial_nbr = $lic_info[0]->{'serial-number'};
        }
    };
    if ($@) {
        $$err_ref = "$@";
        return LC_FALSE;
    }
    
    if ( ! defined($license_code{$serial_nbr}) ) {
        $$err_ref = "serial nbr: $serial_nbr does not exist";
        return LC_FALSE;
    }
   
    if ( defined($license_code{$serial_nbr}->{$feature}) ) {
        $$code_ref = $license_code{$serial_nbr}->{$feature};
    } else {
        $$err_ref = "Unknown feature $feature for serial nbr: $serial_nbr";
        return LC_FALSE;
    } 
    return LC_TRUE;
}

=head2 get_feature_list

return the list of features name available to library

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    serial_nbr => serial number [ 1-80-000011 ]

=back

=item Returns:

Returns an array of features name

=back

=cut

sub get_feature_list {

    my %options = validate(@_,
                    {
                        serial_nbr  => {type => SCALAR, default => '1-80-000011'},
                    });

    my $serial_nbr = $options{'serial_nbr'}; # default serial number

    my @feature_list = keys %{$license_code{$serial_nbr}} ;

    return @feature_list;
}

=head2 get_feature_name_by_code

return the feature's name for given code

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    serial_nbr => serial number [ 1-80-000011 ]
    code       => license code 
    errmsg     => reference to scalar to store error message
    feature    => reference to scalar to store license code

=back

=item Returns:

Returns 1 if successful, otherwise 0.

=back

=cut

sub get_feature_name_by_code {
    my %options = validate(@_,
                    {
                        serial_nbr  => {type => SCALAR, default => '1-80-000011'},
                        code        => {type => SCALAR},
                        errmsg      => {type => SCALARREF},
                        feature     => {type => SCALARREF},
                    });
    my $feat_ref = $options{'feature'}; # refernce to scalar to store feature 
    my $err_ref  = $options{'errmsg'};  # refernce to scalar to store error
    my $code     = $options{'code'};    # code name
    my $serial_nbr = $options{'serial_nbr'}; # default serial number

    my %code_hash = reverse %{$license_code{$serial_nbr}};

    if ( defined($code_hash{$code}) ) {
        $$feat_ref = $code_hash{$code};
    } else {
        $$err_ref = "Unknown code $code";
        return LC_FALSE;
    }
    return LC_TRUE;
}


1;


=head1 SEE ALSO

    None

=head1 STATUS

    Shared

=head1 AUTHOR

    Tarun Garg (tarung@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Cmode/License.pm#1 $
    Last $Change: 1917575 $
    Last Revised $DateTime: 2013/01/22 11:22:46 $

=head1 HISTORY

    2010-01-20 author: tarung      original lib created

=head1 COPYRIGHT

Copyright 2009, Network Appliance., All Rights Reserved

=cut
