package Cmode::Panamax_Clone_Utils;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($VERSION);

#
# Define constants
#

use warnings;
use strict qw(vars subs refs);
use base qw(Exporter);
use SFOTest qw(run_nodescope_command );
use Utils::Units qw(convert_to_bytes);
use NATE::BaseException qw(:try);
use Cmode::HyA_Clone_Utils qw(:all);

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate qw(validate SCALAR ARRAYREF SCALARREF OBJECT HASHREF);

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub get_num_hier_entries;
sub get_num_snap_entries;
sub get_num_afs_entries;
sub fixup_test_list;
sub fixup_test_header;
sub get_split_progress_using_list;
sub check_hier_queue_progress;
sub check_snap_queue_progress;
sub check_afs_queue_progress;
sub get_average_num_splits;
sub delete_file;
sub wafl_touch;

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $VERSION = '0.01';

    my @myutils_sub = qw(
        &get_num_hier_entries
        &get_num_snap_entries
        &get_num_afs_entries
        &fixup_test_list
        &fixup_test_header
        &get_split_progress_using_list
        &check_hier_queue_progress
        &check_snap_queue_progress
        &check_afs_queue_progress
        &get_average_num_splits
        &delete_file
        &wafl_touch
    );

    our %EXPORT_TAGS = ('Panamax_Clone_Utils' => [@myutils_sub],);

    $EXPORT_TAGS{all} = [map { @{$_} } values %EXPORT_TAGS];

    Exporter::export_ok_tags('all');
}




=head2 fixup_test_list

Returns a hash of list entries returned by "fixup_test list print"

=back

=item Returns:

Returns the statistics in the following format.

$stats{v1 => 
	 	{ 'hier'  => value
		  'snap'  => value
		  'afs'   => value
		  'purge' => value
		},
	v2 =>
                { 'hier'  => value
                  'snap'  => value
                  'afs'   => value
                  'purge' => value
                },
	}
=cut

sub fixup_test_list {

    my %args_validn_spec = ('command_interface' => {type => OBJECT},);

    my %opts = validate(@_, \%args_validn_spec);
    my $Node = $opts{'command_interface'};

    my %stats;

    my $runq_entries = $Node->run(
        command => "\"priv set test;" . "fixup_test list print\"");
    my @fixup_entries = split('\n', $runq_entries);

    for (my $i = 0; $i <= $#fixup_entries; $i++) {
        if ($fixup_entries[$i] =~ /Entries in runq 2/) {
            my $j = $i;
            $j++;
            while ($fixup_entries[$j] =~ /Volume/) {
                my @vol_entry = split(' ', $fixup_entries[$j]);
                for (my $index = 0; $index <= $#vol_entry; $index++) {
                    if ($vol_entry[$index] =~ /Volume/) {
                        $index++;
                        my $vol = $vol_entry[$index];
                        $index++;
                        my $entry = $vol_entry[$index];
                        $index++;
                        my $num_entries = $vol_entry[$index];
                        $stats{$vol}{"hier"} = $num_entries;
                    }
                }
                $j++;
            }
            $i = $j - 1;
        } elsif ($fixup_entries[$i] =~ /Entries in runq 1/) {
            my $j = $i;
            $j++;
            while ($fixup_entries[$j] =~ /Volume/) {
                my @vol_entry = split(' ', $fixup_entries[$j]);
                for (my $index = 0; $index <= $#vol_entry; $index++) {
                    if ($vol_entry[$index] =~ /Volume/) {
                        $index++;
                        my $vol = $vol_entry[$index];
                        $index++;
                        my $entry = $vol_entry[$index];
                        $index++;
                        my $num_entries = $vol_entry[$index];
                        $stats{$vol}{"snap"} = $num_entries;
                    }
                }
                $j++;
            }
            $i = $j - 1;
        } elsif ($fixup_entries[$i] =~ /Entries in runq 0/) {
            my $j = $i;
            $j++;
            while ($fixup_entries[$j] =~ /Volume/) {
                my @vol_entry = split(' ', $fixup_entries[$j]);
                for (my $index = 0; $index <= $#vol_entry; $index++) {
                    if ($vol_entry[$index] =~ /Volume/) {
                        $index++;
                        my $vol = $vol_entry[$index];
                        $index++;
                        my $entry = $vol_entry[$index];
                        $index++;
                        my $num_entries = $vol_entry[$index];
                        $stats{$vol}{"afs"} = $num_entries;

                        $index = $index + 2;  ## to extract the purge entries.
                        $num_entries = $vol_entry[$index];
                        $stats{$vol}{"purge"} = $num_entries;
                    }
                }
                $j++;
            }
            $i = $j - 1;
        }
    }

    ## Initialize to zero if vol is not enqueud in a runq .
    foreach my $vol (sort { lc($a) cmp lc($b) } keys %{stats}) {
        if (exists $stats{$vol}{'hier'}) {
            $Log->comment("Volume $vol is enqued in hier queue");
        } else {
            $Log->comment("Volume $vol is not enqueued in hier queue");
            $stats{$vol}{'hier'} = 0;    #Initialize hier entry to zero
        }

        if (exists $stats{$vol}{'snap'}) {
            $Log->comment("Volume $vol is enqued in snap queue");
        } else {
            $Log->comment("Volume $vol is not enqueued in snap queue");
            $stats{$vol}{'snap'} = 0;    #Initialize snap entry to zero
        }

        if (exists $stats{$vol}{'afs'}) {
            $Log->comment("Volume $vol is enqued in afs queue");
        } else {
            $Log->comment("Volume $vol is not enqueued in afs queue");
            $stats{$vol}{'afs'} = 0;     #Initialize afs entry to zero
        }
        if (exists $stats{$vol}{'purge'}) {
            $Log->comment("Volume $vol has purge entries");
        } else {
            $Log->comment("Volume $vol do not have purge entries");
            $stats{$vol}{'purge'} = 0;    #Initialize purge entry to zero
        }
    }

    foreach my $vol (sort { lc($a) cmp lc($b) } keys %{stats}) {
        $Log->comment("Volume $vol = hier  = $stats{$vol}{'hier'} ");
        $Log->comment("Volume $vol = snap  = $stats{$vol}{'snap'} ");
        $Log->comment("Volume $vol = afs   = $stats{$vol}{'afs'} ");
        $Log->comment("Volume $vol = purge = $stats{$vol}{'purge'} ");
    }

    return %stats;
} ## end sub fixup_test_list


=head2 get_split_progress_using_list

This api does 3 things
1) collect the outstanding split entries across all the vols at various intervals t0 - tn
2) From the above collected data extract the overall progress in each runq.
3) Also calculate actual number of splits completed at various intervals t0 - tn.

=item Returns:

Returns HASH

=back

=cut

sub get_split_progress_using_list {
    my %args_validn_spec = (
        'command_interface' => {type => OBJECT},
        'interval'          => {type => SCALAR},    #in seconds
        'nstats'            => {type => SCALAR},    #in seconds
    );

    my %opts      = validate(@_, \%args_validn_spec);
    my $Node      = $opts{'command_interface'};
    my $interval  = $opts{'interval'};
    my $NUM_STATS = ($opts{'nstats'});

    $Log->comment("Initial entries in all runq's...");
    $Node->run(command => "\"priv set test;fixup_test list print\"");

    ### collect initial pending splits at index 0 of %stats (time t0)
    my (%stats);

    ## This hash contains statistics in the following format
    #	$stats{  0 => {	v1 =>
    #                		{ 'hier'  => value
    #                  		  'snap'  => value
    #                  		  'afs'   => value
    #                  		  'purge' => value
    #                		},
    #        		v2 =>
    #                		{ 'hier'  => value
    #                  		  'snap'  => value
    #                  		  'afs'   => value
    #                  		  'purge' => value
    #                		},
    #			......
    #			......
    #        		},
    #		 1 => {  v1 =>
    #                                { 'hier'  => value
    #                                  'snap'  => value
    #                                  'afs'   => value
    #                                  'purge' => value
    #                                },
    #                        v2 =>
    #                                { 'hier'  => value
    #                                  'snap'  => value
    #                                  'afs'   => value
    #                                  'purge' => value
    #                                },
    #                        ......
    #                        ......
    #                        },
    #	      };

    my (%stats_collect);
    $Log->comment("Collecting stats for entry 0");
    %stats_collect = fixup_test_list(command_interface => $Node);
    $stats{"0"} = {%stats_collect};

    ### initialize to zero all the entries from t1 to tn
    my $stat_index = 1;
    while ($stat_index <= $NUM_STATS) {
        foreach my $volume (sort keys %{$stats{0}}) {
            foreach my $entries (sort keys %{$stats{0}{$volume}}) {
                $Log->comment(
                    "Initializing $stat_index $volume $entries = 0");
                $stats{$stat_index}{$volume}{$entries} = 0;
            }
        }
        $stat_index++;
    }

    ### enable fixup
    $Log->comment("Enable clone split...");
    die "enable clone split failed..."
        if (enable_fixup(node => $Node));

    $Log->comment("Outstanding split entries...");
    $Node->run(command => "\"priv set test;fixup_test list header\"");

    ###### collect the pending splits from index t1 to tn #############
    ###### with $INTERVAL seconds in between
    my $delay = ($interval);
    $stat_index = 1;
    while ($stat_index <= $NUM_STATS) {
        $Log->comment(
            "Sleeping for $delay seconds before disabling fixup...");
        $Node->run(command => "\"priv set test;fixup_test list dispatched\"");

        sleep($delay);

        $Node->run(command => "\"priv set test;fixup_test list dispatched\"");

        $Log->comment("Disable fixup...");
        die("disable clone split failed...")
            if (disable_fixup(node => $Node));

        $Log->comment("Wait for 60 seconds before collecting "
                . "stats for entry $stat_index");
        sleep(60);
        $Log->comment("Collecting stats for entry $stat_index");
        %stats_collect = fixup_test_list(command_interface => $Node);
        $stats{$stat_index} = {%stats_collect};

        %stats_collect = fixup_test_header(command_interface => $Node);
        my %stats_header = %stats_collect;
        $Log->comment("hier = $stats_header{'hier'}");
        $Log->comment("snap = $stats_header{'snap'}");
        $Log->comment("afs  = $stats_header{'afs'}");

        if ($stat_index < $NUM_STATS) {
            $Log->comment("Enable fixup...");
            die "enable clone split failed..."
                if (enable_fixup(node => $Node));

            $Log->comment("Outstanding split entries...");
            $Node->run(
                command => "\"priv set test;" . "fixup_test list print\"");
        }
        $stat_index++;
    }    ## END of while loop for collecting stats from t1-tn

    my %runq_progress;
    # This hash is used to hold the split progress in each vol
    # and in each runq in each queue at t0-tn and split
    # progress in each runq from t0 - tn
    #
    # $runq_progress{'vol1' =>
    #                           { 'hier'  } => (t0 t1 t2 ...)
    #                           { 'snap'  } => (t0 t1 t2 ...)
    #                           { 'afs'   } => (t0 t1 t2 ...)
    #                           { 'purge' } => (t0 t1 t2 ...)
    #                'vol2' =>
    #                           { 'hier' } => ...
    #                           { 'snap' } => ...
    #                           { 'afs' }  => ...
    #                           { 'purge' }=> ...
    #		'runq' =>
    #                           { 'hier'  } => (t0 t1 t2 ...)
    #                           { 'snap'  } => (t0 t1 t2 ...)
    #                           { 'afs'   } => (t0 t1 t2 ...)
    #                           { 'purge' } => (t0 t1 t2 ...)
    #                'split' =>
    #                           { 'hier' } => ...
    #                           { 'snap' } => ...
    #                           { 'afs' }  => ...
    #                           { 'purge' }=> ...
    #              }

    ######################## Collect each runq progress ########################
    $Log->comment("Overall run queue progress...");

    my %total_entries; ## sum of all hiers, and snap and afs and purge entries
    ## of all the volumes at various intervals t0 - tn

    foreach my $index (sort { $a <=> $b } keys %{stats}) {
        foreach my $volume (sort keys %{$stats{$index}}) {
            $total_entries{$index}{'hier'}  = 0;
            $total_entries{$index}{'snap'}  = 0;
            $total_entries{$index}{'afs'}   = 0;
            $total_entries{$index}{'purge'} = 0;
        }
    }

    $Log->comment("calculating total entries t0 - tn ...");
    foreach my $index (sort { $a <=> $b } keys %{stats}) {
        foreach my $volume (sort keys %{$stats{$index}}) {
            $total_entries{$index}{'hier'} = $total_entries{$index}{'hier'}
                + $stats{$index}{$volume}{'hier'};
            $total_entries{$index}{'snap'} = $total_entries{$index}{'snap'}
                + $stats{$index}{$volume}{'snap'};
            $total_entries{$index}{'afs'} = $total_entries{$index}{'afs'}
                + $stats{$index}{$volume}{'afs'};
            $total_entries{$index}{'purge'} = $total_entries{$index}{'purge'}
                + $stats{$index}{$volume}{'purge'};
        }
    }

    $Log->comment("Total entries t0 - tn ...");
    my $hier_entries_progress  = "";
    my $snap_entries_progress  = "";
    my $afs_entries_progress   = "";
    my $purge_entries_progress = "";

    foreach my $index (sort { $a <=> $b } keys %{total_entries}) {
        $hier_entries_progress
            = "$hier_entries_progress " . "$total_entries{$index}{'hier'}";
        $snap_entries_progress
            = "$snap_entries_progress " . "$total_entries{$index}{'snap'}";
        $afs_entries_progress
            = "$afs_entries_progress " . "$total_entries{$index}{'afs'}";
        $purge_entries_progress
            = "$purge_entries_progress " . "$total_entries{$index}{'purge'}";
    }

    $runq_progress{"runq"}{"hier"}  = $hier_entries_progress;
    $runq_progress{"runq"}{"snap"}  = $snap_entries_progress;
    $runq_progress{"runq"}{"afs"}   = $afs_entries_progress;
    $runq_progress{"runq"}{"purge"} = $purge_entries_progress;

    ############ Collect each runq split progress
    ############ (i.e number of splits completed between n and n-1
    $Log->comment("Actual number of split happened...");

    my $hier_diff  = "";
    my $snap_diff  = "";
    my $afs_diff   = "";
    my $purge_diff = "";
    my $next;

    for (my $i = 0; $i < ($NUM_STATS); $i++) {
        $next = ($i + 1);
        my $diff
            = ($total_entries{$i}{'hier'} - $total_entries{$next}{'hier'});
        $hier_diff = "$hier_diff " . "$diff";
    }

    for (my $i = 0; $i < ($NUM_STATS); $i++) {
        $next = ($i + 1);
        my $diff
            = ($total_entries{$i}{'snap'} - $total_entries{$next}{'snap'});
        $snap_diff = "$snap_diff " . "$diff";
    }

    for (my $i = 0; $i < ($NUM_STATS); $i++) {
        $next = ($i + 1);
        my $diff = ($total_entries{$i}{'afs'} - $total_entries{$next}{'afs'});
        $afs_diff = "$afs_diff " . "$diff";
    }

    $runq_progress{"split"}{"hier"} = $hier_diff;
    $runq_progress{"split"}{"snap"} = $snap_diff;
    $runq_progress{"split"}{"afs"}  = $afs_diff;

    ######################## Collect progress per vol #########################
    my %vol_progress;
    foreach my $volume (sort keys %{$stats{0}}) {
        $vol_progress{$volume}{'hier'}  = "";
        $vol_progress{$volume}{'snap'}  = "";
        $vol_progress{$volume}{'afs'}   = "";
        $vol_progress{$volume}{'purge'} = "";
    }

    foreach my $volume (sort keys %{$stats{0}}) {
        for (my $ind = 0; $ind < $NUM_STATS; $ind++) {
            $vol_progress{$volume}{'hier'} = "$vol_progress{$volume}{'hier'} "
                . "$stats{$ind}{$volume}{'hier'}";
            $vol_progress{$volume}{'snap'} = "$vol_progress{$volume}{'snap'} "
                . "$stats{$ind}{$volume}{'snap'}";
            $vol_progress{$volume}{'afs'} = "$vol_progress{$volume}{'afs'} "
                . "$stats{$ind}{$volume}{'afs'}";
            $vol_progress{$volume}{'purge'}
                = "$vol_progress{$volume}{'purge'} "
                . "$stats{$ind}{$volume}{'purge'}";
        }
    }

    foreach my $volume (sort keys %{vol_progress}) {
        $runq_progress{$volume}{'hier'}  = $vol_progress{$volume}{'hier'};
        $runq_progress{$volume}{'snap'}  = $vol_progress{$volume}{'snap'};
        $runq_progress{$volume}{'afs'}   = $vol_progress{$volume}{'afs'};
        $runq_progress{$volume}{'purge'} = $vol_progress{$volume}{'purge'};
    }

    return %runq_progress;
} ## end sub get_split_progress_using_list



=head2 fixup_test_header

Returns a hash of "fixup_test list header" command output

=item Returns:

Returns HASH

=back

=cut

sub fixup_test_header {
    my %args_validn_spec = ('command_interface' => {type => OBJECT},);

    my %opts = validate(@_, \%args_validn_spec);
    my $Node = $opts{'command_interface'};

    my %stats;

    my $runq_entries = $Node->run(
        command => "\"priv set test;" . "fixup_test list header\"");
    my @fixup_entries = split('\n', $runq_entries);

    for (my $i = 0; $i <= $#fixup_entries; $i++) {
        if ($fixup_entries[$i] =~ /Entries in runq 2/) {
            my $j = $i;
            $j++;
            if ($fixup_entries[$j] =~ /Unsplit_entries/) {
                my @entry = split(' ', $fixup_entries[$j]);
                $stats{"hier"} = $entry[1];
            }
            $i = $j;
        } elsif ($fixup_entries[$i] =~ /Entries in runq 1/) {
            my $j = $i;
            $j++;
            if ($fixup_entries[$j] =~ /Unsplit_entries/) {
                my @entry = split(' ', $fixup_entries[$j]);
                $stats{"snap"} = $entry[1];
            }
            $i = $j;
        } elsif ($fixup_entries[$i] =~ /Entries in runq 0/) {
            my $j = $i;
            $j++;
            if ($fixup_entries[$j] =~ /Unsplit_entries/) {
                my @entry = split(' ', $fixup_entries[$j]);
                $stats{"afs"} = $entry[1];
            }
            $i = $j;
        }
    }
    $Log->comment("fixup_test_header hier = $stats{'hier'}");
    $Log->comment("fixup_test_header snap = $stats{'snap'}");
    $Log->comment("fixup_test_header afs  = $stats{'afs'}");

    #$stats{
    #        "hier" => n1,
    #	     "snap" => n2,
    #	     "afs"  => n3,
    #      }
    return %stats;
} ## end sub fixup_test_header


=head2 get_num_hier_entries

Returns total number of hierarchy entries in hierarchy queue.

=item Returns:

Returns a scalar

=back

=cut

sub get_num_hier_entries {
    my %args_validn_spec = ('ci' => {type => OBJECT},);

    my %opts = validate(@_, \%args_validn_spec);
    my $Node = $opts{'ci'};

    my $num_hier_entries = 0;

    my $runq_entries = $Node->run(
        command => "\"priv set test; fixup_test list header\"");

    my @fixup_entries = split('\n', $runq_entries);
    for (my $i = 0; $i <= $#fixup_entries; $i++) {
        if ($fixup_entries[$i] =~ /Entries in runq 2/) {
            my $j = $i;
            $j++;
            if ($fixup_entries[$j] =~ /Unsplit_entries/) {
                my @entry = split(' ', $fixup_entries[$j]);
                $num_hier_entries = $entry[1];
            }
        }
    }
    return $num_hier_entries;
} ## end sub get_num_hier_entries


=head2 get_num_snap_entries

Returns total number of snapshot entries in snapshot queue.

=item Returns:

Returns scalar

=back

=cut

sub get_num_snap_entries {
    my %args_validn_spec = ('ci' => {type => OBJECT},);

    my %opts = validate(@_, \%args_validn_spec);
    my $Node = $opts{'ci'};

    my $num_snap_entries = 0;

    my $runq_entries = $Node->run(
        command => "\"priv set test; fixup_test list header\"");

    my @fixup_entries = split('\n', $runq_entries);
    for (my $i = 0; $i <= $#fixup_entries; $i++) {
        if ($fixup_entries[$i] =~ /Entries in runq 1/) {
            my $j = $i;
            $j++;
            if ($fixup_entries[$j] =~ /Unsplit_entries/) {
                my @entry = split(' ', $fixup_entries[$j]);
                $num_snap_entries = $entry[1];
            }
        }
    }
    return $num_snap_entries;
} ## end sub get_num_snap_entries

=head2 get_num_afs_entries

Returns total number of AFS entries in AFS queue.

=item Returns:

Returns scalar

=back

=cut

sub get_num_afs_entries {
    my %args_validn_spec = ('ci' => {type => OBJECT},);

    my %opts = validate(@_, \%args_validn_spec);
    my $Node = $opts{'ci'};

    my $num_afs_entries = 0;

    my $runq_entries = $Node->run(
        command => "\"priv set test; fixup_test list header\"");

    my @fixup_entries = split('\n', $runq_entries);
    for (my $i = 0; $i <= $#fixup_entries; $i++) {
        if ($fixup_entries[$i] =~ /Entries in runq 0/) {
            my $j = $i;
            $j++;
            if ($fixup_entries[$j] =~ /Unsplit_entries/) {
                my @entry = split(' ', $fixup_entries[$j]);
                $num_afs_entries = $entry[1];
            }
        }
    }
    return $num_afs_entries;
} ## end sub get_num_afs_entries

=head2 check_hier_queue_progress

Checks if splits in Hierarchy queue progressed smoothly
and returns 0 on success and 1 on failure.

=item Returns:

Returns scalar

=back

=cut

sub check_hier_queue_progress {

    my %args_validn_spec = (
        'runq_hash'      => {type => HASHREF},
        'avg_num_splits' => {type => SCALAR},
    );

    my %opts = validate(@_, \%args_validn_spec);
    my ($ref_runq_progress) = $opts{'runq_hash'};
    my (%runq_progress)     = %$ref_runq_progress;
    my ($avg_num_splits)    = $opts{'avg_num_splits'};

    my @runq_hier = split(' ', $runq_progress{'runq'}{'hier'});
    my @runq_snap = split(' ', $runq_progress{'runq'}{'snap'});
    my @runq_afs  = split(' ', $runq_progress{'runq'}{'afs'});

    my @split_hier = split(' ', $runq_progress{'split'}{'hier'});
    my @split_snap = split(' ', $runq_progress{'split'}{'snap'});
    my @split_afs  = split(' ', $runq_progress{'split'}{'afs'});

    for (my $i = 0; $i < $#runq_hier; $i++) {
        ## check hier runq progress
        if ($runq_hier[$i]) {
            $Log->comment("---------------------------------------------------");
            $Log->comment(
                "Checking the hierarchy entries progress at time t$i");
            if (($runq_hier[$i] <= (int($avg_num_splits / 2)))) {
                ## if avg splits is more than double the hier entries
                ## than all the hier entries shuld be drained out
                if ($split_hier[$i] == $runq_hier[$i]) {
                    $Log->comment(
                        "Hier entries progressed...with avg splits ");
                    $Log->comment("more than double the hier entries");
                    $Log->comment("avg_num_splits = $avg_num_splits ");
                    $Log->comment("hier entries   = $runq_hier[$i] ");
                    $Log->comment("split entries  = $split_hier[$i]");
                } else {
                    $Log->comment("Hier entries made less progress? when ");
                    $Log->comment(
                        "avg splits is more than double the hier entries");
                    $Log->comment("avg_num_splits = $avg_num_splits ");
                    $Log->comment("hier entries   = $runq_hier[$i] ");
                    $Log->comment("split entries  = $split_hier[$i]");

                    #die "Please check the hier entry progress";
                    return 1;
                }
            } else {
                ## hier splits shuldbe more than any other queue
                if (   ($split_hier[$i] > $split_snap[$i])
                    && ($split_hier[$i] > $split_afs[$i])) {
                    $Log->comment("Hier entries progressed... "
                            . "when avg splits less than hier entries");
                    $Log->comment("avg_num_splits = $avg_num_splits ");
                    $Log->comment("hier entries   = $runq_hier[$i] ");
                    $Log->comment("split entries  = $split_hier[$i]");
                } else {
                    $Log->comment("Hier entries made less progress? ");
                    $Log->comment("avg_num_splits = $avg_num_splits ");
                    $Log->comment("hier entries   = $runq_hier[$i] ");
                    $Log->comment("split entries  = $split_hier[$i]");

                    #die "Please check the hier entry progress";
                    return 1;
                }
            }
        }
    }
    return 0;
} ## end sub check_hier_queue_progress

=head2 check_snap_queue_progress

Checks if splits in Snapshot queue progresses smoothly
and returns 0 on success and 1 on failure.

=item Returns:

Returns scalar

=back

=cut

sub check_snap_queue_progress {
    my %args_validn_spec = (
        'runq_hash'      => {type => HASHREF},
        'avg_num_splits' => {type => SCALAR},
    );

    my %opts = validate(@_, \%args_validn_spec);

    #    my %runq_progress = %$opts{'runq_hash'};
    my ($ref_runq_progress) = $opts{'runq_hash'};
    my (%runq_progress)     = %$ref_runq_progress;

    my $avg_num_splits = $opts{'avg_num_splits'};

    my @runq_hier = split(' ', $runq_progress{'runq'}{'hier'});
    my @runq_snap = split(' ', $runq_progress{'runq'}{'snap'});
    my @runq_afs  = split(' ', $runq_progress{'runq'}{'afs'});

    my @split_hier = split(' ', $runq_progress{'split'}{'hier'});
    my @split_snap = split(' ', $runq_progress{'split'}{'snap'});
    my @split_afs  = split(' ', $runq_progress{'split'}{'afs'});

    for (my $i = 0; $i < $#runq_hier; $i++) {
        ## check snap runq progress
        if ($runq_snap[$i]) {
            $Log->comment("---------------------------------------------------");
            $Log->comment(
                "Checking the snapshot entries progress at time t$i");
            if (   ($runq_snap[$i] > $avg_num_splits)
                && ($runq_afs[$i] > $avg_num_splits)) {
                if ($split_snap[$i] > $split_afs[$i]) {
                    $Log->comment("Snapshot entries progressed ...");
                    $Log->comment("avg_num_splits = $avg_num_splits ");
                    $Log->comment("snap entries   = $runq_snap[$i] ");
                    $Log->comment("split entries  = $split_snap[$i]");

                    my $ratio = int($split_snap[$i] / $split_afs[$i]);

                    $Log->comment("Snapshot splits progress at interval "
                            . "t$i is $ratio times afs splits");
                } else {
                    $Log->comment(
                        "splits progressed more in AFs than snapshot queue!!"
                    );
                    $Log->comment(
                        "Interval t$i snap_splits = $split_snap[$i] ");
                    $Log->comment("afs_splits = $split_afs[$i]");
                    $Log->comment("avg_num_splits = $avg_num_splits ");
                    $Log->comment("snap entries   = $runq_snap[$i] ");
                    $Log->comment("split entries  = $split_snap[$i]");

                    #die "snapshot queue is not sceduled properly";
                    return 1;
                }
            }
        }
    }
    return 0;
} ## end sub check_snap_queue_progress

=head2 check_afs_queue_progress

Checks if splits in AFS queue progressed smoothly
and returns 0 on success and 1 on failure.

=item Returns:

Returns scalar

=back

=cut

sub check_afs_queue_progress {
    my %args_validn_spec = (
        'runq_hash'      => {type => HASHREF},
        'avg_num_splits' => {type => SCALAR},
    );

    my %opts = validate(@_, \%args_validn_spec);

    #    my %runq_progress = %$opts{'runq_hash'};
    my ($ref_runq_progress) = $opts{'runq_hash'};
    my (%runq_progress)     = %$ref_runq_progress;

    my $avg_num_splits = $opts{'avg_num_splits'};

    my @runq_hier = split(' ', $runq_progress{'runq'}{'hier'});
    my @runq_snap = split(' ', $runq_progress{'runq'}{'snap'});
    my @runq_afs  = split(' ', $runq_progress{'runq'}{'afs'});

    my @split_hier = split(' ', $runq_progress{'split'}{'hier'});
    my @split_snap = split(' ', $runq_progress{'split'}{'snap'});
    my @split_afs  = split(' ', $runq_progress{'split'}{'afs'});

    for (my $i = 0; $i < $#runq_hier; $i++) {
        ## check AFS queue progress
        if ($runq_afs[$i]) {
            $Log->comment("---------------------------------------------------");
            $Log->comment("Checking the AFS entries progress at time t$i");
            if ($runq_afs[$i] > $runq_afs[$i + 1]) {
                $Log->comment("AFS queue progressed...");
                $Log->comment("avg_num_splits = $avg_num_splits "
                        . "afs entries    = $runq_afs[$i] "
                        . "split entries  = $split_afs[$i]");

            } else {
                my $j = $i + 1;
                $Log->comment(
                    "Splits did not progress at interval t$i to t$j");

                #die "AFS was not sceduled properly";
                return 1;
            }
        }

    } ## end of for

    return 0;
} ## end sub check_afs_queue_progress

=head2 get_average_num_splits

computes average num of splits happend between time interval t0 - tn

=item Returns:

Returns scalar

=back

=cut

sub get_average_num_splits {

    my %args_validn_spec = ('runq_hash' => {type => HASHREF},);

    my %opts = validate(@_, \%args_validn_spec);
    my ($ref_runq_progress) = $opts{'runq_hash'};
    my (%runq_progress)     = %$ref_runq_progress;

    my $item;

    my @runq_hier = split(' ', $runq_progress{'runq'}{'hier'});
    my @runq_snap = split(' ', $runq_progress{'runq'}{'snap'});
    my @runq_afs  = split(' ', $runq_progress{'runq'}{'afs'});

    my @split_hier = split(' ', $runq_progress{'split'}{'hier'});
    my @split_snap = split(' ', $runq_progress{'split'}{'snap'});
    my @split_afs  = split(' ', $runq_progress{'split'}{'afs'});

    ## @split_t Contains sum of all the splits in each runq at each interval
    ## Num of splits happened at t0 - tn are present in split_t[0] - split_t[n]
    my @split_t;
    for (my $i = 0; $i < $#runq_hier; $i++) {
        if ($split_hier[$i] || $split_snap[$i] || $split_afs[$i]) {
            $split_t[$i] = $split_hier[$i] + $split_snap[$i] + $split_afs[$i];
        } else {
            last;
        }
    }

    my $total_split = 0;
    foreach my $j (@split_t) {
        $total_split = $total_split + $j;
    }

    $Log->comment("split_t = @split_t");
    $Log->comment("Total splits = $total_split");

    my $average = int($total_split / ($#split_t + 1));

    $Log->comment("average = $average");

    return $average;
} ## end sub get_average_num_splits

=head2 delete_file

This routine is used to delete a file and return 0 on success and 1 on failure

=item Returns:

Returns scalar

=back

=cut

sub delete_file {

    my %args_validn_spec = (
        'node_obj'  => {type => OBJECT},
        'vol_name'  => {type => SCALAR},
        'file_name' => {type => SCALAR},
        'timeout'   => {type => SCALAR, default => "36000"},
    );

    my %opts      = validate(@_, \%args_validn_spec);
    my $node_obj  = $opts{'node_obj'};
    my $vol_name  = $opts{'vol_name'};
    my $file_name = $opts{'file_name'};
    my $timeout   = $opts{'timeout'};

    ### delete the clone file ###
    my $output = $node_obj->execute_raw_command(
        command => "run local \"priv set test;"
            . "slist /vol/$vol_name/$file_name\"",
        'timeout' => $timeout
    );
    if ($output =~ /Invalid/) {
        $Log->comment("file $file_name does not exist");
        return 1;
    } else {
        $node_obj->execute_raw_command(
            command => "run local \"priv set test;"
                . "rm /vol/$vol_name/$file_name\"",
            'timeout' => $timeout
        );

        $output = $node_obj->execute_raw_command(
            command => "run local \"priv set test;"
                . "slist /vol/$vol_name/$file_name\"",
            'timeout' => $timeout
        );
        if ($output =~ /Invalid/) {
            $Log->comment("file $file_name deleted!");
        } elsif ($output =~ /Inode/) {
            $Log->comment("Failed to delete file $file_name");
            return 1;
        } else {
            $Log->comment("Unknown error...slist for "
                    . "$file_name returned - $output");
            return 1;
        }
    }
    return 0;
} ## end sub delete_file

=head2 wafl_touch

Wafl touch a file by name and return 0 on success and 1 on failure

=item Returns:

Returns scalar

=back

wafl_touch

=cut

sub wafl_touch {

    my %args_validn_spec = (
        'node_obj'  => {type => OBJECT},
        'vol_name'  => {type => SCALAR},
        'file_name' => {type => SCALAR},
        'timeout'   => {type => SCALAR, default => "36000"},
    );

    my %opts      = validate(@_, \%args_validn_spec);
    my $node_obj  = $opts{'node_obj'};
    my $vol_name  = $opts{'vol_name'};
    my $file_name = $opts{'file_name'};
    my $timeout   = $opts{'timeout'};

    my ($inum);

WAFL_TOUCH:
    try {
        my $out = $node_obj->execute_raw_command(
            command => "run local \"priv set test;"
                . "slist /vol/$vol_name/$file_name\"",
            'timeout' => $timeout
        );

        if ($out =~ /Invalid/) {
            $Log->comment(
                "source file $file_name " . "does not exist in $vol_name");
            return 1;
        } else {
            $out = $node_obj->execute_raw_command(
                command => "run local \"priv set test;"
                    . "wafl touch -V $vol_name "
                    . "-f /vol/$vol_name/$file_name 0 0 0\"",
                'timeout' => $timeout
            );

            if ($out =~ /wafl touch succeeded/) {
                $Log->comment(
                    "wafl touch suceeded on " . "/vol/$vol_name/$file_name ");
            } else {
                $Log->comment("failed to touch the file $file_name");
                return 1;
            }
        }
    }
    catch NATE::BaseException with {
        my $e = shift;
        if ($e->text() =~ /Could not create connection/) {
            $Log->comment("SSH could not connect...retry");
            goto WAFL_TOUCH;
        } else {
            logresult(
                type => 'FAIL',
                msg  => "wafl touch $vol_name $inum failed: " . $e->text(),
            );
            $Log->comment("Failed to touch file $file_name");
            return 1;
        }
    };
    return 0;
} ## end sub wafl_touch
