# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Cmode/Qtree.pm#1 $
#
# $Change: 1917575 $
#
# $DateTime: 2013/01/22 11:22:46 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      Qtree.pm - cmode qtree library
#
# DESCRIPTION
#      This perl module provide wrapper interfaces to c-mode 
#       volume qtree cli command
#
# AUTHOR
#       tarung@netapp.com
#
#
## @pod here
##
##

=pod

=head1 NAME

    Qtree.pm - provide wrapper interfaces to C-Mode volume-qtree cli command

=head1 SYNOPSIS

    use Cmode::Qtree qw(:all);
    OR
    use Cmode::Qtree qw(:qtree);

=head1 DESCRIPTION

=head1 FUNCTIONS

Cmode Volume Qtree Functions

=cut

package Cmode::Qtree ;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($Validation, $VERSION);

#
# Define constants
#
use constant QT_ERROR => undef;
use constant QT_TRUE  => 1;
use constant QT_FALSE => 0;

use warnings;
use strict qw(vars subs refs);
use base   qw(Exporter);

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $Validation = 1; # unused as of now
    $VERSION    = '0.01';

    my @qtree_sub = qw(
                        &volume_qtree_create
                        &volume_qtree_delete
                        &volume_qtree_modify
                        &volume_qtree_rename
                        &volume_qtree_show
                        &volume_qtree_statistics
                        &volume_qtree_statistics_reset
                    );

    my @constants = qw(QT_ERROR QT_TRUE QT_FALSE);

    our %EXPORT_TAGS = (
        'qtree'      => [ @qtree_sub ],
        'constants'  => [ @constants ],
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate qw(validate SCALAR ARRAYREF SCALARREF OBJECT );

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub volume_qtree_create;
sub volume_qtree_delete;
sub volume_qtree_modify;
sub volume_qtree_rename;
sub volume_qtree_show;
sub volume_qtree_statistics;
sub volume_qtree_statistics_reset;


#------------------------------------------------------
#     Local Variables
#------------------------------------------------------


#------------------------------------------------------
#     Subroutines Definition
#------------------------------------------------------

=head2 volume_qtree_create()

create qtree in given flexvol and vserver 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_qtree_create {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            verify  => 1,
        );

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'vserver'          => {type => SCALAR},
            'qtree'            => {type => SCALAR, optional => 1},
            'volume'           => {type => SCALAR},
            'security-style'   => {type => SCALAR, default => 'mixed' },
            'oplock-mode'      => {type => SCALAR, default => 'enable'},
            'verify'           => {type => SCALAR, default => 1},
            'unix-permissions' => {type => SCALAR, default => '---rwxrwxrwx'}, 
            'errmsg'           => {type => SCALARREF},
            'timeout'          => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'-Verification'} = $options{'verify'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_qtree_create( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_qtree_create() failed, error: $@";
        return QT_FALSE;
    }

    return QT_TRUE;
}

=head2 volume_qtree_delete()

delete qtree in given flexvol and vserver

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_qtree_delete {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'     => {type => OBJECT},
            'vserver'    => {type => SCALAR},
            'volume'     => {type => SCALAR},
            'qtree'      => {type => SCALAR},
            'force'      => {type => SCALAR, default => 'false'},
            'foreground' => {type => SCALAR, default => 'false'},
            'errmsg'     => {type => SCALARREF},
            'timeout'    => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_qtree_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_qtree_delete() failed, error: $@";
        return QT_FALSE;
    }

    return QT_TRUE;
}


=head2 volume_qtree_modify()

modify qtree attributes in given flexvol and vserver

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_qtree_modify {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'vserver'          => {type => SCALAR},
            'qtree'            => {type => SCALAR},
            'volume'           => {type => SCALAR},
            'security-style'   => {type => SCALAR, optional => 1},
            'oplock-mode'      => {type => SCALAR, optional => 1},
            'unix-permissions' => {type => SCALAR, optional => 1},
            'errmsg'           => {type => SCALARREF},
            'timeout'          => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_qtree_modify( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_qtree_modify() failed, error: $@";
        return QT_FALSE;
    }

    return QT_TRUE;
}


sub volume_qtree_rename {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'vserver'          => {type => SCALAR},
            'volume'           => {type => SCALAR},
            'qtree'            => {type => SCALAR},
            'newname'          => {type => SCALAR},
            'errmsg'           => {type => SCALARREF},
            'timeout'          => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->volume_qtree_rename( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_qtree_rename() failed, error: $@";
        return QT_FALSE;
    }

    return QT_TRUE;
}


=head2 volume_qtree_show()

show qtrees in given flexvol and vserver 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_qtree_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'           => {type => OBJECT},
            'vserver'          => {type => SCALAR, optional => 1},
            'volume'           => {type => SCALAR, optional => 1},
            'qtree'            => {type => SCALAR, optional => 1},
            'security-style'   => {type => SCALAR, optional => 1},
            'oplock-mode'      => {type => SCALAR, optional => 1},
            'unix-permissions' => {type => SCALAR, optional => 1},
            'qtree-id'         => {type => SCALAR, optional => 1},
            'status'           => {type => SCALAR, optional => 1},
            'errmsg'           => {type => SCALARREF},
            'cli_op'           => {type => ARRAYREF},
            'timeout'          => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_qtree_show( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_qtree_show() failed, error: $@";
        return QT_FALSE;
    }

    return QT_TRUE;
}

=head2 volume_qtree_statistics()

show statistics for a given qtree in given flexvol and vserver

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_qtree_statistics {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'                => {type => OBJECT},
            'vserver'               => {type => SCALAR, optional => 1},
            'volume'                => {type => SCALAR, optional => 1},
            'qtree'                 => {type => SCALAR, optional => 1},
            'nfs-ops'               => {type => SCALAR, optional => 1},
            'cifs-ops'              => {type => SCALAR, optional => 1},
            'internal-ops'          => {type => SCALAR, optional => 1},
            'no-reset-nfs-ops'      => {type => SCALAR, optional => 1},
            'no-reset-cifs-ops'     => {type => SCALAR, optional => 1},
            'no-reset-internal-ops' => {type => SCALAR, optional => 1},
            'errmsg'                => {type => SCALARREF},
            'cli_op'                => {type => ARRAYREF},
            'timeout'               => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
     $$err_ref = "none";

    eval {
        @{$cmdOpRef} = $server->volume_qtree_statistics( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_qtree_statistics() failed, error: $@";
        return QT_FALSE;
    }

    return QT_TRUE;
}


=head2 volume_qtree_statistics_reset()

reset statistics for given flexvol

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub volume_qtree_statistics_reset {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'  => {type => OBJECT},
            'vserver' => {type => SCALAR, optional => 1},
            'volume'  => {type => SCALAR, optional => 1},
            'errmsg'  => {type => SCALARREF},
            'timeout' => {type => SCALAR, default => 300},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
     $$err_ref = "none";

    eval {
        $server->volume_qtree_statistics_reset( %input_arg );
    };

    if($@) {
        $$err_ref = "volume_qtree_statistics_reset() failed, error: $@";
        return QT_FALSE;
    }

    return QT_TRUE;
}


1;


=head1 SEE ALSO

    None

=head1 STATUS

    Shared

=head1 AUTHOR

    Tarun Garg (tarung@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Cmode/Qtree.pm#1 $
    Last $Change: 1917575 $
    Last Revised $DateTime: 2013/01/22 11:22:46 $

=head1 HISTORY

    2010-02-10 author: tarung      original lib created

=head1 COPYRIGHT

Copyright 2009, Network Appliance., All Rights Reserved

=cut

