# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/Cmode/SFXOD.pm#1 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      SFXOD.pm
#
# DESCRIPTION
#      This perl module provide wrapper interfaces to c-mode 
#      volume file move command 
#
# AUTHOR
#       gourab@netapp.com
#
#
## @pod here
##
##

=pod

=head1 NAME

    SFXOD.pm - provide wrapper interfaces to C-Mode file volume file move
    command

=head1 SYNOPSIS

    use Cmode::SFXOD qw(:all);
    OR
    use Cmode::SFXOD qw(:single_file_move);

=head1 DESCRIPTION

=head1 FUNCTIONS

Cmode single file move on demand functions

=cut

package Cmode::SFXOD ;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($VERSION);

#
# Define constants
#

use warnings;
use strict qw(vars subs refs);
use base   qw(Exporter);
use SFOTest  qw(run_nodescope_command );
use Utils::Units qw(convert_to_bytes);

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $VERSION    = '0.01';

    my @filemove_sub = qw(
                        &single_file_move
                    );

    our %EXPORT_TAGS = (
        'SFCOD'      => [ @filemove_sub ],
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate qw(validate SCALAR ARRAYREF SCALARREF OBJECT);

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub single_file_move;

#------------------------------------------------------
#     Local Variables
#------------------------------------------------------


#------------------------------------------------------
#     Subroutines Definition
#------------------------------------------------------

=head2 single_file_move()

Create a SFMOD job

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise dies

=back

=cut

sub single_file_move {

    my %args_validn_spec =
        (   'server'           => {type => OBJECT},
            'source-paths'     => {type => SCALAR},
            'destination-paths'=> {type => SCALAR},
            'max-cutover-time'=> {type => SCALAR, optional => 1},
            'max-throughput'  => {type => SCALAR, optional => 1},
            'scanner-paused'  => {type => SCALAR, optional => 1},
            '-Timeout'         => {type => SCALAR, default  => 300},
            'error_flag'       => {type => SCALAR, optional => 1, default => 0},
            'error_msg'        => {type => SCALARREF, optional => 1},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $error_ref = $options{'error_msg'};
    my $error_flag = $options{'error_flag'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( $key ne 'server' && defined($options{$key})) {
            $input_arg{$key} = $options{$key};
        }
    }

    $input_arg{'-Verification'}=undef;
    
    my %test_options = (
                         'server' => $server,
                         'privilege' => 'test',
                        );
                        
    $server->set( %test_options ) ;
    
    if ( $error_flag == 1 ) {
        eval {
            $server->volume_file_move_start( %input_arg );
        };
        if ($@) {
            $$error_ref="$@";
            return 1;
        }
    } else {
        eval {
            $server->volume_file_move_start( %input_arg );
        };
        if ($@) {
            die("SFMOD start failed: $@");
        }
        return 1;
   }
}

1;
