# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Cmode/Snapmirror.pm#2 $
#
# $Change: 2013620 $
#
# $DateTime: 2013/04/16 03:29:39 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
#
#
# NAME
#      Snapmirror.pm - provide interfaces to C-Mode job cli command 
#
# DESCRIPTION
#      This perl module provide wrapper interfaces to c-mode snapmirror cli command
#
#
#
# AUTHOR
#       dl-nb-capacity-qa@netapp.com
#
#
#
## @pod here
##
##

=pod

=head1 NAME

    Snapmirror.pm - provide interfaces to C-Mode snapmirror cli command

=head1 SYNOPSIS

    use Cmode::Snapmirror qw(:all);

=head1 DESCRIPTION

=head1 FUNCTIONS

Cmode Snapmirror Functions

=cut


package Cmode::Snapmirror;

#------------------------------------------------------
#   Module Vars
#------------------------------------------------------
our ($Validation, $VERSION);

#
# Define constants
#
use constant SM_ERROR => undef;
use constant SM_TRUE  => 1;
use constant SM_FALSE => 0;


use warnings;
use strict qw(vars subs refs);
use base   qw(Exporter);

BEGIN {

    #--------------------------------------------
    #   Exports
    #--------------------------------------------

    $Validation = 1; # unused as of now
    $VERSION    = '0.01';

    my @all_sub = qw(
                &snapmirror_abort
                &snapmirror_check
                &snapmirror_create
                &snapmirror_delete
				&snapmirror_release
                &snapmirror_initialize
                &snapmirror_initialize_ls_set
                &snapmirror_modify
                &snapmirror_promote
                &snapmirror_resync
                &snapmirror_show
                &snapmirror_update
                &snapmirror_update_ls_set
            );

    my @constants = qw(SM_ERROR SM_TRUE SM_FALSE);

    our %EXPORT_TAGS = (
        'snapmirror' => [ @all_sub ],
        'constants'  => [ @constants ],
    );

    $EXPORT_TAGS{all} = [ map{ @{$_} } values %EXPORT_TAGS ];

    Exporter::export_ok_tags('all');
}

#------------------------------------------------------
#   Imports
#------------------------------------------------------
use Tharn;
use Data::Dumper;
use Params::Validate qw(validate validate_pos SCALAR ARRAYREF
                        SCALARREF OBJECT HASHREF CODEREF);

#------------------------------------------------------
#     Prototypes
#------------------------------------------------------
sub snapmirror_abort;
sub snapmirror_check;
sub snapmirror_create;
sub snapmirror_delete;
sub snapmirror_release;
sub snapmirror_initialize;
sub snapmirror_initialize_ls_set;
sub snapmirror_modify;
sub snapmirror_promote;
sub snapmirror_resync;
sub snapmirror_show;
sub snapmirror_update;
sub snapmirror_update_ls_set;

=head2 snapmirror_abort()

abort snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_abort {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' => 
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'hard' => 
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            timeout =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->snapmirror_abort( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_abort() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_check()

start check operation 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_check {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'target-snapshot'     => {type => SCALAR, optional => 1},
            'foreground' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            timeout =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->snapmirror_check( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_check() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_create()

create snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_create {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            verify  => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'type' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 'DP',
                },
            'vserver'  => {type => SCALAR, optional => 1},
            'schedule' => {type => SCALAR, optional => 1},
            'tries' =>
                {
                    type => SCALAR,
                    optional => 1,
                },
            'throttle' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                    default  => 'unlimited',
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
            'verify' => {type => SCALAR, default => 1},
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'-Verification'} = $options{'verify'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->snapmirror_create( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_create() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_initialize()

initialize snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_initialize {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'type' =>
                {
                    type => SCALAR,
                    regex => qr/^(DP|LS)$/,
                    default  => 'DP',
                },
            'throttle' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                },
            'foreground' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                    default  => 'false',
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->snapmirror_initialize( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_initialize() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_initialize_ls_set()

initialize load sharing snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_initialize_ls_set {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver' => {type => SCALAR, optional => 1},
            'source-volume'  => {type => SCALAR, optional => 1},
            'foreground' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                    default  => 'false',
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->snapmirror_initialize_ls_set( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_initialize_ls_set() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_delete()

delete snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_delete {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->snapmirror_delete( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_delete() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

sub snapmirror_release {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    eval {
        $server->snapmirror_release( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_delete() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_modify()

modify snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_modify {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'vserver'             => {type => SCALAR, optional => 1},
            'schedule'            => {type => SCALAR, optional => 1},
            'tries' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                },
            'throttle' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    # invoke the call
    eval {
        $server->snapmirror_modify( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_modify() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}


=head2 snapmirror_promote()

promote snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_promote {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'force' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                    default  => 'false',
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    # invoke the call
    eval {
        $server->snapmirror_promote( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_promote() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}


=head2 snapmirror_resync()

resync snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_resync {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'force' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            'throttle' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                },
            'foreground' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    # invoke the call
    eval {
        $server->snapmirror_resync( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_resync() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}


=head2 snapmirror_show()

show snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'type' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(DP|LS)$/,
                },
            'vserver'  => {type => SCALAR, optional => 1},
            'schedule' => {type => SCALAR, optional => 1},
            'tries' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                },
            'throttle' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                },
            'state' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(Uninitialized|Snapmirrored|Broken-off|Unknown)$/,
                },
            'mirror-timestamp'   => {type => SCALAR, optional => 1},
            'status' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(Idle|Transfer|Restore|Check)$/,
                },
            'transfer-snapshot'   => {type => SCALAR, optional => 1},
            'snapshot-progress'   => {type => SCALAR, optional => 1},
            'snapshot-checkpoint' => {type => SCALAR, optional => 1},
            'common-snapshot'     => {type => SCALAR, optional => 1},
            'exported-snapshot'   => {type => SCALAR, optional => 1},
            'cli_op'              => {type => ARRAYREF, optional => 1},
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    my $cmdOpRef = $options{'cli_op'}; # cli output ref array

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    # invoke the call
    eval {
        @{$cmdOpRef} = $server->snapmirror_show( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_show() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_update()

update snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_update {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver'      => {type => SCALAR, optional => 1},
            'source-volume'       => {type => SCALAR, optional => 1},
            'destination-path'    => {type => SCALAR, optional => 1},
            'destination-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['destination-vserver', 'destination-volume'],
                },
            'destination-vserver' => {type => SCALAR, optional => 1},
            'destination-volume'  => {type => SCALAR, optional => 1},
            'throttle' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^((\d+)|unlimited)$/,
                },
            'foreground' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # reset the error message
    $$err_ref = "none";

    # invoke the call
    eval {
        $server->snapmirror_update( %input_arg );
    };
    if($@) {
        $$err_ref = "snapmirror_update() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}

=head2 snapmirror_update_ls_set()

update load sharing snapmirror relationship

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub snapmirror_update_ls_set {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            'server'         => {type => OBJECT},
            'errmsg'         => {type => SCALARREF},
            'source-path'    => {type => SCALAR, optional => 1},
            'source-cluster' =>
                {
                    type => SCALAR,
                    optional => 1,
                    depends  => ['source-vserver', 'source-volume'],
                },
            'source-vserver' => {type => SCALAR, optional => 1},
            'source-volume'  => {type => SCALAR, optional => 1},
            'foreground' =>
                {
                    type => SCALAR,
                    optional => 1,
                    regex => qr/^(true|false)$/,
                    default  => 'false',
                },
            'timeout' =>
                {
                    type => SCALAR,
                    optional => 1,
                    default  => 60,
                },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = "\"" . $options{$key} . "\"";
            }
        }
    }

    # invoke the cli call
    eval {
        $server->snapmirror_update_ls_set( %input_arg );
    };

    if($@) {
        $$err_ref = "snapmirror_update_ls_set() failed, error: $@";
        return SM_FALSE;
    }

    return SM_TRUE;
}


1;

=head1 SEE ALSO

    None

=head1 STATUS

    Shared

=head1 AUTHOR

    Tarun Garg (tarung@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Cmode/Snapmirror.pm#2 $
    Last $Change: 2013620 $
    Last Revised $DateTime: 2013/04/16 03:29:39 $

=head1 HISTORY

    2009-12-24 author: tarung      original lib created

=head1 COPYRIGHT

Copyright 2009, Network Appliance., All Rights Reserved

=cut

