package CmodeCLIOutputParser;
use Data::Dumper;
use Tharn qw($Log);

#Write class method here i.e. constructor
sub new(){
	my ($class, %args) = @_;
	
	return bless(\%args,$class);
}

#Write object methods here 

##############################################################################
# Method:  _parse_output
# Purpose: Parse the provided raw output into a standard structure - array 
#          of hashes. The keys of the hashes being the keys and each entry
#          in the array for one row in the output.
##############################################################################
sub _parse_output {
    $Log->enter();

    my ( $self, %args ) = @_;

    my $want_array = [];

    my $output        = $self->{output};
    my $CLI_separator = $self->{CLI_separator};
    
    my ($fields, $heading, @tmp_output) = split( /\n/, $output );
    my (@fields, @heading);
    if (defined $fields) {
        @fields        = split( $CLI_separator, $fields );
    }
    if (defined $heading) {
        @heading       = split( $CLI_separator, $heading );
    }

    my @not_parsed    = ();

    $Log->debug( "Fields:\n" . Dumper(@fields) );
    $Log->debug( "Heading:\n" . Dumper(@heading) );

   # Some output still contains embedded newlines in the fields so we cannot
   #  totally depend on the @tmp_output containing a single line for each row in
   #  in the CLI. So, loop through it once and find any elements of @array
   #  (the row) which do not contain the right number of elements (values
   #  for the row). The right number should be the number of elements of
   #  @heading.
   #
   # For any discrepencies, try to reform the row by combining multiple
   #  elements of @tmp_output into a new @output.
    my @output;
    my $partial_line = "";

    foreach my $line (@tmp_output) {

# We're here to parse the output into key/value structures, save off skipped lines
        if ( $line =~ /^INFO/ ) {
            $Log->debug( "Found INFO line. Ignoring: \"$line\"" );
            push( @not_parsed, $line );
            next;
        } elsif ( $line =~ /-showseparator \"$CLI_separator\"/ ) {

         #Ignore any separator entry that is returned via the debug_log_show cmd
            $Log->debug( 
                  "Found output separator config line. Ignoring: \"$line\"" );
            push( @not_parsed, $line );
            next;
        }
        $self->{not_parsed} = \@not_parsed;

        my @values = split( $CLI_separator, $line );
        if ( @values != @heading ) {
            $partial_line .= $line;

            # make sure we have seen the end of the row
            next if ( $partial_line !~ /$CLI_separator$/ );

            my @partial = split( $CLI_separator, $partial_line );
            if ( @partial == @heading ) {
                $Log->debug( "done with merge: $partial_line" );
                push( @output, $partial_line );
                $partial_line = "";
            } elsif ( @partial > @heading ) {
                my $error_text;
                $error_text = "Error while parsing CMode CLI output!\n";
                $error_text .= "Failed to handle embedded newlines in the " . 
                              "CLI output\n";
                $error_text .= "line=$line\n";
                $error_text .= "partial_line=$partial_line";
#                NACL::APISet::Exceptions::ResponseException->throw(
#                    text => $error_text, response_object => $self);

                printf("################################\n");
                    printf("ERROR_TEXT::[$error_text]\n");
                    printf Dumper("PARSER_OBJ::[$self]\n");
                printf("################################\n");
                
            }
        } else {
            push( @output, $line );
        }
    }

    foreach my $line (@output) {
        my %output = ();
        my $index  = 0;
        foreach my $field ( split( $CLI_separator, $line ) ) {
            ##################################################################
            # NOTE: The splitting of a list field on a comma will break
            # if the field is of type list AND accepts a list of text values
            # (or some other field type) that supports a comma.
            # We have not seen instances of this type of field yet but will
            # evaluate if problems occur
            ##################################################################

            ##################################################################
            # If the field starts with quotes, split the individual strings
            # separated by comma-space. Otherwise, treat entire thing as one
            # string. Some job output is missing the trialing ".
            ##################################################################
            if ( $field =~ /^\".+\"?$/ || $field =~ /^\{.+\}$/ ) {
                if ( $field =~ /, \"/ ) {
                    my $array_ref = [ split( /, /, $field ) ];
                    $Log->debug( "array formed because of '$field'" );
                    $output{ $fields[$index] } = $array_ref;
                } else {
                    $output{ $fields[$index] } = $field;
                }
            } elsif ( $field =~ /, ?/ ) {
                my $array_ref = [ split( /, ?/, $field ) ];
                $Log->debug( "array formed because of '$field'" );
                $output{ $fields[$index] } = $array_ref;
            } else {
                $output{ $fields[$index] } = $field;
            }
            $index++;
        }

        # Below line removes hash entry with key as "\r" that is left
        # after parsing all other lines for key value pairs.
        delete $output{"\r"} if exists( $output{"\r"} );

        push( @$want_array, \%output );
    }
    $Log->debug( "Parsed output is:\n" . Dumper($want_array) );
    $Log->exit();

    $self->{parsed_output} = $want_array;
}


1;
