#
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/7Mode/LunClone.pm#1 $
#

# Documentation is found in Compression::LunClone
# This module contains the filer-side methods needed for clone luns
package Compression::7Mode::LunClone;

use strict;
use warnings;

use Tharn;
use Params::Validate qw(validate validate_with SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);
use Data::Dumper;
use Scalar::Util qw(blessed);

use Compression::Constants qw(:all);
use Compression::7Mode::Lun;
use blocks_filer;

our @ISA = ("Compression::7Mode::Lun");

BEGIN
{
	require "Compression/base_import.pl";
}


sub new
{
	my $class = shift;
	my (%args) = validate( @_,
	{
		source => {type => OBJECT,
			   isa => "Compression::Lun",},
		
		parent => {type => OBJECT,
			   isa  => "Compression::Compress",
			   optional => 1,},

		name => {type => SCALAR,
			 optional => 1,},
	});

	# Start building our object:
	my $self = {};
	my $parent = delete($args{parent});
	my $lun_source = delete($args{source});
	$self->{_debug} = $lun_source->{_debug};
	$self->{_disable_cleanup} = $lun_source->{_disable_cleanup};
	$self->{_igroup_name} = $lun_source->{_igroup_name};
	$self->{_ui} = $lun_source->{_lun}->{_ui};
	$self->{_creator_pid} = $$;
	$self->{_mapped} = FALSE;
	if (defined($parent))
	{
		$args{Parent} = $parent->{volume}->_get_underlying_vol_obj();
	}
	
	$args{splitclone} = 0;
	$self->{_note_lun} = $lun_source->{_lun}->{_note_lun}->clone(%args);
	$self->{_name} = $self->{_note_lun}->name();
	$self->{_full_path} = $self->{_note_lun}->mount_path();
	$self->{_conn} = $self->{_note_lun}->Filer->conn();
	$self->{_filer_ip} = $self->{_note_lun}->Filer->defaultip;
	
	bless($self, $class);

	return $self;

}


sub split_start
{
	my $class = shift;
	my (%args) = validate( @_,
	{
		wait => {type => SCALAR, default => TRUE,}
	} );
	
	logcomment("Starting split of lun clone " . $class->{_name});
	
	blocks_filer::filer_lun_clone_split_start($class->{_conn},
						  $class->{_full_path});
	
	if (TRUE == $args{wait})
	{
		$class->split_wait();
	}
}


sub split_wait
{
	my $class = shift;
	my (%args) = validate( @_,
	{
		# None
	} );
	
	logcomment("Waiting split of lun clone " . $class->{_name} . " to complete");
	
	blocks_filer::filer_lun_clone_wait($class->{_conn},
					   $class->{_full_path});
}


sub split_status
{
	my $class = shift;
	my (%args) = validate( @_,
	{
		expected => {type => SCALAR, default => C_RC_SUCCESS,},
	} );
	
	my $rc = C_RC_SUCCESS;
	my $name = $class->{_name};
	my $percent_complete = blocks_filer::filer_lun_clone_percentcomplete_get($class->{_conn},
										 $class->{_full_path});
	logcomment("Percent complete for lun clone split of $name: $percent_complete");
	if ($percent_complete == 100)
	{
		$rc = C_RC_SCANNER_IDLE;
	}
	
	return $class->{_ui}->check_for_expected_rc(obtained_rc => $rc,
						    expected_rc => $args{expected},
						    log_message => "Obtain lun clone split status for $name",
						    output      => "Percentage complete: $percent_complete");
}


sub _attach_to_lun_in_cloned_volume
{
	my $class = shift;
	my (%args) = validate( @_,
	{
		source => {type => OBJECT,
			   isa => "Compression::Lun",},
		
		cloned_vol => {type => OBJECT,
			       isa  => "Compression::VolClone",
			       },

		name => {type => SCALAR,
			 optional => 1,},
	});
	
	if (blessed($class))
	{
		logresult('script', "This method should not be called on existent objects");
	}

	# Start building our object:
	my $self = {};
	my $parent = $args{cloned_vol};
	my $lun_source = $args{source};
	$self->{_debug} = $lun_source->{_debug};
	$self->{_disable_cleanup} = $lun_source->{_disable_cleanup};
	$self->{_igroup_name} = $lun_source->{_igroup_name};
	$self->{_ui} = $lun_source->{_lun}->{_ui};
	$self->{_creator_pid} = $$;
	$self->{_mapped} = FALSE;
	$self->{_name} = $lun_source->{name};
	
	my $vol = NOTE::Volume->attach(Filer => $parent->{filer}->_get_underlying_filer_obj(),
				       name => $parent->{vol_name},);
	$self->{_note_lun} = NOTE::LUN->attach(Parent => $vol,
					       name   => $self->{_name},
					       );
	$self->{_full_path} = $self->{_note_lun}->mount_path();
	$self->{_conn} = $self->{_note_lun}->Filer->conn();
	$self->{_filer_ip} = $self->{_note_lun}->Filer->defaultip;
	
	bless($self, $class);

	return $self;
}




1;
