# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/CMode/FileClone.pm#1 $
#
## @summary
##  Utility wrapper module for C-mode file clone
##
## @author @author voicu@netapp.com, dl-compression-qa@netapp.com
## @status private (internal)

package Compression::CMode::FileClone;

BEGIN
{
	require "Compression/base_import.pl";

}

use strict;
use warnings;
use Tharn;
use Params::Validate qw(validate validate_with SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);
use Data::Dumper;

use Compression::Constants qw(:all);
use Cmode::Fileclone qw(dense_fast_clone_start);




# Documentation in Compression::FileClone
sub new
{
	my $class = shift;
	my (%args) = validate( @_,
	{
		filer => {
                        type => OBJECT,
			isa => "NOTE::GX::Node",},

                volume => {
			type => SCALAR,},
		
		vserver => {
			type => SCALAR,},

		debug => {
			type => BOOLEAN,
                        optional => 1,
			default => FALSE,},
                
                ui => {type => OBJECT,
		       isa => "Compression::UI",
		       },
	} );

	# Start building our object:
	my $self = {};
        $self->{_filer} = $args{filer};
	$self->{_vserver} = $args{vserver};
        $self->{_volume} = $args{volume};
        $self->{_debug} = $args{debug};
        $self->{_creator_pid} = $$;
	$self->{_ui} = $args{ui};
        

	bless($self, $class);

	return $self;
}


sub start
{
        my $class = shift;
        my (%args) = validate( @_,
        {
                source => {type => SCALAR,},
		clone => {type => SCALAR, optional => 1,},
		expected => {type => SCALAR, default => C_RC_SUCCESS,},
		snapshot => {type => SCALAR, optional => 1,},
		blk_range => {type => SCALAR, optional => 1,},
        } );
	
	my $expected = $args{expected};
	my $source = $args{source};
	my $clone = $args{clone};
	my $vol_name = $class->{_volume};
	
	if ((!exists($args{clone})) && (!exists($args{blk_range})))
	{
		logresult('script', "One of the \"clone\" or \"blk_range\" must be specified for the file clone start operation");
	}
	
	# For backwards compatibility with 7-mode, the source/clone path is the filer-mounted path.
	# For c-mode, we have to only keep the file's relative path.
	$source =~ s/.*\/$vol_name\//\//;

	my %fileclone_args = ('server' => $class->{_filer}->server(),
			      'vserver' => $class->{_vserver},
			      'volume' => $class->{_volume},
			      'source-path' => $source,
			      );
	
	if (defined($clone))
	{
		$clone =~ s/.*\/$vol_name\//\//;
		$fileclone_args{'destination-path'} = $clone;
	}
	
	if (exists($args{blk_range}))
	{
		$fileclone_args{'range'} = $args{blk_range};
	}
	
	if (exists($args{snapshot}))
	{
		$fileclone_args{'snapshot-name'} = $args{snapshot};
	}
	
	eval
	{
		Cmode::Fileclone::dense_fast_clone_start(%fileclone_args);
	};
	
	my $output = $@;
	return $class->{_ui}->process_output_string(code => C_CMD_FILE_CLONE_START,
						    output => $output,
						    expected_rc => $expected,
						    log_message => "File clone start for file: $source (volume $vol_name)");
}


sub status
{
        my $class = shift;
	
	logresult('NA', "NOT APPLICABLE: File clone status is not supported in C-mode");
}


sub clear
{
        my $class = shift;
        
	logresult('NA', "NOT APPLICABLE: File clone clear is not supported in C-mode");
}


sub stop
{
        my $class = shift;
	
	logresult('NA', "NOT APPLICABLE: File clone stop is not supported in C-mode");
}



sub wait_for_completion
{
        my $class = shift;
	
	# The clone operation is synchronous in C-mode.
	# Make this method a no-op...
	
	return (rc => C_RC_SUCCESS);
}


sub is_bg_clone_in_progress
{
	my $class = shift;
	my (%args) = validate( @_,
        {
                inode_num => {type => SCALAR,},
		expected => {type => SCALAR, default => C_RC_SUCCESS,},
        } );
	
	$class->{_ui}->{version}->check_for_required_features(features => [C_FEATURE_CTIME_FILE_CLONE,]);
	
	my $vol = $class->{_volume};
	my $inum = $args{inode_num};
	my $expected = $args{expected};
	
	my $cmd_code = C_CMD_BG_FILE_CLONE_STATUS;
	my $command = $class->{_ui}->formulate_command(code   => $cmd_code,
                                                       volume => $vol,);
	
	my $output = Compression::Utils::_send_cmd_to_filer(conn => $class->{_filer},
                                                            boot_mode => C_ONTAP_CMODE,
                                                            cmd => $command,
							    node_run => $class->{_ui}->is_dblade_level_command(code => $cmd_code),);
	
	my $bg_clone_prefix = $class->{_ui}->get_constant_value(constant_code => 'BG_FILE_CLONE_ACTIVE_PREFIX');
	$bg_clone_prefix .= $inum;
	
	my $rc = ($output =~ /$bg_clone_prefix/) ? C_RC_SUCCESS : C_RC_SCANNER_IDLE;

	return $class->{_ui}->check_for_expected_rc(obtained_rc => $rc,
						    expected_rc => $expected,
						    log_message => "Obtain background file clone status for volume " . $vol,
						    output      => $output);
}




1;