#
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/Filer/License.pm#1 $
#

## @summary
## Utility module for compression licensing.
##
## @description
## Utility module for compression licensing.
## Keeps track of all the licensing prerequisites for compression.
##
## @synopsis
##
## use Compression::Constants;
## use Compression::Version;
##
## 
##
## @examples 
## 
## 
## @author voicu@netapp.com, dl-compression-qa@netapp.com
## @status private (in-progress)

package Compression::Filer::License;


use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);

use Tharn;
use Module::Common;
use Compression::Constants qw(:all);
use Compression::Version;


BEGIN
{
	require "Compression/base_import.pl";
	require Exporter;
	our @ISA = qw(Exporter);
}


## @name ::new
##
## @summary Compression::Filer::License Constructor.
##
## @return Object of type Compression::Filer::License.
##
## @arg version Required: Compression::Version object for the hosting filer.
##
## @arg debug Optional: Controls verbosity of this module.
## Defaults to FALSE.
##
## @example
## my $obj = Compression::Filer::License->new( version => $version );
##
## @author voicu@netapp.com, dl-compression-qa@netapp.com
## @status private (in-progress)
##

sub new
{
	my $class = shift;
	my %args = validate(@_,
	{
                version => {type => OBJECT,
                            isa => "Compression::Version",},
                
                debug => {type => SCALAR,
                          optional => 1,
                          default  => FALSE,},
                
                boot_mode => {type => SCALAR,
                              optional => 1,
                              default => C_ONTAP_BOOT_MODE},
	} );

	my $self = {};
        $self->{_debug} = $args{debug};
        $self->{_version} = $args{version};
        $self->{_boot_mode} = $args{boot_mode};

	bless $self, $class;
	return $self;
}


## @name ::get_licensing_info
## @summary Generates release-specific licensing information for compression.
##
## @description
## Generates release-specific licensing information for compression
## (if licensing is needed and if the compression license is dependent on other licenses).
##
## @return Array containing the codes of all the licenses necessary for compression
## (including compression). 
##
## @author voicu@netapp.com, dl-compression-qa@netapp.com
## @status private (in-progress)
##

sub get_licensing_info
{
	my $class = shift;
	my %args = validate(@_,
	{
		# No args
	} );
        
        my $ontap_ver = $class->{_version}->{ontap};
        my $boot_mode = $class->{_boot_mode};
	
	my $licensing_info;
        
        if ($boot_mode eq C_ONTAP_CMODE)
        {
                $licensing_info = {'default' => [],
                                   }
        }
        else
        {
                $licensing_info = {'default' => [],
                                   '7.3' => [C_LICENSE_NEARSTORE, C_LICENSE_COMPRESSION],
                                   '8.0' => [C_LICENSE_ASIS, C_LICENSE_COMPRESSION],
                                   }
        }
	
	my $release_specific_info = $licensing_info->{$ontap_ver};
	
	if (defined($release_specific_info))
	{
		return $release_specific_info;
	}
	else
	{
		logcomment("License: Licensing info for $ontap_ver not found. Returning default") if ($class->{_debug});
		return $licensing_info->{default};
	}
}


# This should be a temporary solution
sub _get_licensed_feature_option_info
{
        my $class = shift;
	my %args = validate(@_,
	{
		# No args
	} );
        
        my $ontap_ver = $class->{_version}->{ontap};
        my $boot_mode = $class->{_boot_mode};
	
	my $options_info;
        
        if ($boot_mode eq C_ONTAP_CMODE)
        {
                $options_info = {'default' => ['licensed_feature.a_sis.enable'],
                                   }
        }
        else
        {
                $options_info = {'default' => ['licensed_feature.a_sis.enable'],
                                   '7.3' => [],
                                   '8.0' => [],
                                   }
        }
	
	my $release_specific_info = $options_info->{$ontap_ver};
	
	if (defined($release_specific_info))
	{
		return $release_specific_info;
	}
	else
	{
		logcomment("License: Licensed feature options info for $ontap_ver not found. Returning default") if ($class->{_debug});
		return $options_info->{default};
	}
}


1;
