#
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/Qtree.pm#1 $
#
## @summary Compression Qtree routines.
##  
##
## @description
## Wrapper around NOTE::Qtree.
## The main reason for not using NOTE::Qtree directly is to provide standardized output
##    and to enable specific processing on some failures, otherwise fatal.
##    This is required for handling compression-specific failures.
##
## @synopsis
## use Compression::Qtree;
## use vars qw( $FILER  );
##
## BEGIN { require "Compression/setup_core.pl"; }
##
## my $obj = Compression::Qtree->new (host => $FILER,
##				            initialize => TRUE );
##
## @author dl-compression-qa@netapp.com, waroff@netapp.com
## @status private (in-progress)

package Compression::Qtree;

use strict;
use warnings;
use Tharn;
use Params::Validate qw(validate validate_with SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);
use Data::Dumper;

use Compression::Constants qw(:all);
use Compression::Utils qw(_c_interpret_logresult);


BEGIN
{
	require "Compression/base_import.pl";

}


## @name:: new
## @summary 
##
## @description
## 
##
## @return Object of type Compression::Qtree.
##
## @arg compress Required: The compression object from Compression::Compress::setup
## @arg name Required: Name of the qtree
## @arg debug Optional: Controls verbosity of this module (defaults to FALSE)
## @arg destination Optional: Set this to TRUE if the qtree will be a destination of QSM, Snapvault, etc.  In
##                            this case only the object will initially be created.
##
## @example
## my $obj = Compression::Qtree->new (	filer => $conn,
##                                      volume => $self->{volume}->{volume},
##                                      debug => $args{debug} );
sub new
{
	my $class 	= shift;
	my (%args) = validate (@_, {	name		 => { type=>SCALAR },
									compress_obj => { type=>OBJECT },
									destination  => { type=>SCALAR, optional => 1, default=>FALSE, regex => qr/^0|1$/, },
									debug		 => { type=>SCALAR, optional => 1, default=>FALSE },
	} );
	my $name = $args{name};
	my $compress_obj = $args{compress_obj};
	my $destination = $args{destination};

	my $qtree_obj;
	if ($destination == TRUE) {
		$qtree_obj = NOTE::Qtree->new(Volume=>$compress_obj->{volume}->_get_underlying_vol_obj(), name=>$name);
	} else {
		$qtree_obj = NOTE::Qtree->create(Volume=>$compress_obj->{volume}->_get_underlying_vol_obj(), name=>$name);
	}

	
	my $self = {};
	$self->{ID} 	= $$;
	$self->{_debug}	= $args{debug};
	$self->{_qtree}	= $qtree_obj;
	$self->{_compress} = $compress_obj;
	$self->{name}	= $name;
	bless($self, $class);

	logcomment("Qtree: Setup Complete..\n\n") if ($self->{_debug});
	return $self;
}


1;