#
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/UI/CLI/Messages/Constants_7_3.pm#1 $
#


package Compression::UI::CLI::Messages::Constants_7_3;

  
use Tharn;

use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);

use Compression::Constants qw(:all);


BEGIN
{
	require "Compression/base_import.pl";
	require Exporter;
	our @ISA = qw(Exporter);
        
};


sub new
{
	my $class = shift;
	my %args = validate(@_,
	{       
                debug  => {  
			type 	 => SCALAR,
			optional => 1,
                        default  => FALSE,
			},
                
                boot_mode => {type => SCALAR,
                              optional => 1,
                              default  => C_ONTAP_BOOT_MODE,
                              },
	} );
        
	my $self = {};
        $self->{_debug} = $args{debug};
        
        my $boot_mode = $args{boot_mode};
        $self->{_boot_mode} = $boot_mode;
        
        if ($boot_mode ne C_ONTAP_7MODE)
        {
                logresult('script', "There is no support for \"$boot_mode\" boot mode in 7.3");
        }
        
        $self->{messages} = {
                OPTION_SET => {'Operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                               'Operation failed because volume is being decompressed' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
                               'Volume \S+ is not online' => C_RC_VOLUME_NOT_ONLINE,
                               'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                               'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                               },
                DECOMPRESS_START => {'decompress of volume \S+ started' => C_RC_SUCCESS,
                                     'Please turn off the "compression" option for volume' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                     'No compressed data in volume' => C_RC_NO_DATA_TO_DECOMPRESS,
                                     'Not enough disk space to decompress volume' => C_RC_INSUFFICIENT_SPACE,
                                    },
                DECOMPRESS_STOP => {'' => C_RC_SUCCESS,
                                    'No decompression in progress for volume ' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                    'Invalid volume name \S+ specified' => C_RC_VOLUME_NOT_FOUND,
                                   },
                DECOMPRESS_STATUS => {'No volume decompression in progress' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                     },
                LICENSE_ADD => {'This license is only supported either on NearStore platforms' => C_RC_NO_NEARSTORE_LICENSE,
                               },
                LICENSE_REMOVE => {'Compression must be disabled .+ before removing the license' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                  },
                GET_SPACE_SAVINGS => {'df: \S+: No such file or directory' => C_RC_VOLUME_NOT_FOUND,
                                      ': unrecognized argument ' => C_RC_INVALID_COMMAND_ARGUMENT,
                                      },
                VOL_ONLINE => {'' => C_RC_SUCCESS,
                               'Unable to online volume as space cannot be guaranteed for the volume' => C_RC_INSUFFICIENT_SPACE,
                               },
		AUTOSIZE => {'autosize settings UPDATED' => C_RC_SUCCESS,
			     },
		VOL_CLONE_SPLIT_STOP => {'' => C_RC_SUCCESS,
                                         'Clone volume is not being split from its parent' => C_RC_VOLUME_IS_NOT_BEING_SPLIT,
                                         'The volume is not a clone' => C_RC_VOLUME_IS_NOT_A_CLONE,
                                         },
                VOL_CLONE_SPLIT_START => {'' => C_RC_SUCCESS,
                                          },
                OPTION_SET => {'Operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                               'Operation failed because volume is being decompressed' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
                               'Volume \S+ is not online' => C_RC_VOLUME_NOT_ONLINE,
                               'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                               'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                               },
                ENABLE_COMPRESSION => {'Operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                                       'Operation failed because volume is being decompressed' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
                                       'Volume \S+ is not online' => C_RC_VOLUME_NOT_ONLINE,
                                       'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                       'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                                       '' => C_RC_SUCCESS,
                                       },
                DISABLE_COMPRESSION => {'Operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                                        'Volume \S+ is not online' => C_RC_VOLUME_NOT_ONLINE,
                                        'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                        'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                                        '' => C_RC_SUCCESS,
                                        },
                IS_COMPRESSION_ENABLED => {'compression=on' => _C_RC_COMPR_ON_FULLY_FUNCTIONAL,
                                           'compression=off' => _C_RC_COMPR_OFF,
                                           },
	};
        
	
        logcomment("Created new Messages :: Constants_7_3 object\n" . Dumper($self)) if ($self->{_debug});
        
	bless $self, $class;
	return $self;
}



1;