#
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/UI/CLI/Messages/Constants_8_1.pm#1 $
#

package Compression::UI::CLI::Messages::Constants_8_1;

  
use Tharn;

use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);

use Compression::Constants qw(:all);


BEGIN
{
	require "Compression/base_import.pl";
	require Exporter;
	our @ISA = qw(Exporter);
        
};


sub new
{
	my $class = shift;
	my %args = validate(@_,
	{       
                debug  => {  
			type 	 => SCALAR,
			optional => 1,
                        default  => FALSE,
			},
                
                boot_mode => {type => SCALAR,
                              optional => 1,
                              default  => C_ONTAP_BOOT_MODE,
                              },
	} );
        
	my $self = {};
        $self->{_debug} = $args{debug};
        
        my $boot_mode = $args{boot_mode};
        $self->{_boot_mode} = $boot_mode;
        
        if ($boot_mode eq C_ONTAP_7MODE)
        {
                #####################################
                ####### 7-mode output strings #######
                #####################################
                $self->{messages} = {
                        OPTION_SET => {'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                                       'No volume named \S+ exists' => C_RC_VOLUME_NOT_FOUND,
                                       'CR_OP_NOT_ALLOWED_ON_ROOT_VOL' =>  C_RC_OP_NOT_ALLOWED_ON_ROOT_VOL,
                                       },
                        IS_COMPRESSION_ENABLED => {'Invalid path specification: \S+' => C_RC_INVALID_PATH,
                                                   'Compression:\s+Disabled\nInline Compression:\s+Disabled' => _C_RC_COMPR_OFF,
                                                   'Compression:\s+Enabled\nInline Compression:\s+Disabled' => _C_RC_COMPR_ON_NO_INLINE,
                                                   'Compression:\s+Enabled\nInline Compression:\s+Enabled' => _C_RC_COMPR_ON_FULLY_FUNCTIONAL,
                                                   'No status entry found' => _C_RC_COMPR_OFF,
						   'Volume state transition is in progress' => C_RC_VOL_STATE_TRANSITION_IN_PROGRESS,
                                                   },
                        ENABLE_COMPRESSION => {'This operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                                               'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
                                               'Compression is not allowed at this time because undo scanner is in progress on this volume' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
                                               'Volume type has to be Flexvol:' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                               #'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                                               'Compression is not allowed on a root volume' =>  C_RC_OP_NOT_ALLOWED_ON_ROOT_VOL,
                                               '' => C_RC_SUCCESS,
                                               'Compression can only be enabled on a 64-bit volume contained in a 64-bit aggregate' => C_RC_COMPRESSION_ALLOWED_ONLY_ON_64_BIT,
                                               'Inline compression cannot be enabled when background compression is disabled' => C_RC_COMPRESSION_NOT_TURNED_ON,
                                               'Compression is not allowed on a volume that is part of a Remote SnapMirror relationship' => C_RC_COMPRESSION_NOT_ALLOWED_ON_RSM_VOL,
                                               'Compression is not allowed at this time because a clone operation is in progress on this volume' => C_RC_VOL_CLONE_IN_PROGRESS,
                                               },
                        DISABLE_COMPRESSION => {'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
                                                'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                                'Background compression cannot be disabled when inline compression is enabled on the volume' => C_RC_INLINE_COMPRESSION_NOT_OFF,
						'Compression cannot be disabled while an efficiency operation is in progress on the volume' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
                                                '' => C_RC_SUCCESS,
                                                },
                        GET_SPACE_SAVINGS => {'df: \S+: No such file or directory' => C_RC_VOLUME_NOT_FOUND,
                                              ': unrecognized argument ' => C_RC_INVALID_COMMAND_ARGUMENT,
                                              },
                        VOL_ONLINE => {'' => C_RC_SUCCESS,
                                       'Unable to online volume as space cannot be guaranteed for the volume' => C_RC_INSUFFICIENT_SPACE,
				       'Unable to online volume because there is not enough space in the aggregate' => C_RC_INSUFFICIENT_SPACE,
                                       },
                        AUTOSIZE => {'autosize settings UPDATED' => C_RC_SUCCESS,
                                     },
                        VOL_CLONE_SPLIT_STOP => {'' => C_RC_SUCCESS,
                                                 'Clone volume is not being split from its parent' => C_RC_VOLUME_IS_NOT_BEING_SPLIT,
                                                 'The volume is not a clone' => C_RC_VOLUME_IS_NOT_A_CLONE,
                                                 },
                        VOL_CLONE_SPLIT_START => {'' => C_RC_SUCCESS,
						  'The volume has restricted snapshots' => C_RC_VOLUME_HAS_RESTRICTED_SNAPSHOTS,
                                                  },
                        SFSR_START => {'No space left on device' => C_RC_INSUFFICIENT_SPACE,
                                       },
                        LICENSE_ADD => {'This license is only supported either on NearStore platforms' => C_RC_NO_NEARSTORE_LICENSE,
                                       },
                        LICENSE_REMOVE => {'Compression must be disabled .+ before removing the license' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                          },
                        COMPRESS_START => {'The SIS operation for \S+ is started' => C_RC_SUCCESS,
                                           'Operation is currently active: \S+' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
                                           'Compression not enabled: \S+' => C_RC_COMPRESSION_NOT_TURNED_ON,
					   'Invalid operation on a SnapVault secondary volume: \S+' => C_RC_NOT_ALLOWED_ON_SNAPVAULT_SECONDARY,
					   'The SIS operation for \S+ is queued' => C_RC_SE_OPERATION_IS_QUEUED,
                                           },
                        COMPRESS_STATUS => {'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
                                           },
                        COMPRESS_STOP => {'The operation on \S+ is being stopped' => C_RC_SUCCESS,
					  'Operation is currently idle: \S+' => C_RC_SCANNER_IDLE,
					  'Volume is offline: \S+' => C_RC_VOL_IS_OFFLINE,
                                          },
                        COMPRESS_STATUS_CHECKPOINT => {
                                                       },
                        _PARSE_CHECKPOINT_TIME => {'No Checkpoint' => C_RC_CHECKPOINT_FILE_EMPTY,
                                                   '\w+\s+\w+\s+\d+\s+\d+:\d+:\d+\s+\S+\s+\d+' => C_RC_SUCCESS,
                                                   },
                        DECOMPRESS_START => {'' => C_RC_SUCCESS,
                                             'Compression is not disabled on the volume' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                             'No compressed data in the volume' => C_RC_NO_DATA_TO_DECOMPRESS,
                                             'Not enough disk space for the operation' => C_RC_INSUFFICIENT_SPACE,
                                             'Volume \S+ is readonly' => C_RC_VOLUME_IS_READ_ONLY,
                                             #'Please break all the snapmirror relationships in volume \S+' => C_RC_VOLUME_HAS_RSM_RELATIONSHIPS,
                                            },
                        DECOMPRESS_STOP => {'The operation on \S+ is being stopped' => C_RC_SUCCESS,
                                            'Operation is currently idle: \S+' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                            'Volume does not exist: \S+' => C_RC_VOLUME_NOT_FOUND,
                                           },
                        DECOMPRESS_STATUS => {'No status entry found' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                             },
                        DEDUPE_START => {'' => C_RC_SUCCESS,
                                         'Operation is currently active: \S+' => C_RC_SCANNER_STILL_ACTIVE,
                                         },
                        DEDUPE_SET_STATE => {'' => C_RC_SUCCESS,
                                             },
                        DEDUPE_STOP => {'' => C_RC_SUCCESS,
                                       },
			DEDUPE_OFF_UNDO => {'' => C_RC_SUCCESS,
					    },
                        DUMPBLOCK_FILE => {'This is compressed block' => C_RC_DUMPED_COMPRESSED_BLOCK,
                                           '\s0x\d+\s' => C_RC_DUMPED_UNCOMPRESSED_BLOCK,
                                           },
                        FILE_RESERVATION => {'space reservations for file \S+: off' => C_RC_FILE_RESERVATION_OFF,
                                             'space reservations for file \S+: on' => C_RC_FILE_RESERVATION_ON,
                                             },
                        FILE_FOLDING_START => {'' => C_RC_SUCCESS,
                                               },
                        FILE_FOLDING_ABORT => {'' => C_RC_SUCCESS,
                                               },
                        SET_SNAPLOCK_AUTO_COMMIT => {'^0$' => C_RC_SUCCESS,
                                                     },
                        SNAPMIRROR_RESYNC => {'' => C_RC_SUCCESS,
                                              'process was aborted' => C_RC_PROCESS_WAS_ABORTED,
                                              },
                        SNAPVAULT_UPDATE => {'' => C_RC_SUCCESS,
                                             'No space left on device' => C_RC_INSUFFICIENT_SPACE,
                                             'qtree is currently involved in another operation' => C_RC_QTREE_INVOLVED_IN_OTHER_OPERATION,
                                             },
                        SET_WAFL_SCAN_SPEED => {'' => C_RC_SUCCESS,
                                                },
                        VOL_COPY_START => {'' => C_RC_SUCCESS,
                                           'Aborting: Destination volume, \S+, is too small' => C_RC_INSUFFICIENT_SPACE,
                                           'Aborting: the aggregate that contains destination volume \S+ does not support compression' => C_RC_COMPRESSION_NOT_SUPPORTED_ON_DEST,
                                           },
                        AGGR_COPY_START => {'' => C_RC_SUCCESS,
                                            },
			VOL_SNAP_RESTORE => {'' => C_RC_SUCCESS,
					     },
			SNAPMIRROR_INITIALIZE => {'The aggregate that contains the destination volume does not support compression' => C_RC_COMPRESSION_NOT_SUPPORTED_ON_DEST,
						  },
			DELETE_FILE => {'Read-only file system' => C_RC_READ_ONLY_FILESYSTEM,
					},
			FILE_CLONE_START => {'^\d+$' => C_RC_SUCCESS,
					     'clone start: No space' => C_RC_INSUFFICIENT_SPACE,
					     },
			FILE_CLONE_CLEAR => {'' => C_RC_SUCCESS,
					     },
			FILE_CLONE_STOP => {'' => C_RC_SUCCESS,
					    },
			AGGR_BLOCK_CHECK_START => {'block_check already running for aggregate \S+' => C_RC_AGGR_BLOCK_CHECK_IN_PROGRESS,
						   },
			VOL_MOVE_START => {'' => C_RC_SUCCESS,
					   'Source vol has a flexclone and hence will not be deleted after vol move operation completion' => C_RC_VOL_MOVE_SRC_HAS_CLONE,
					   'Specified source volume is a flexclone' => C_RC_VOL_MOVE_SRC_IS_CLONE,
					   'Specified source volume is a SnapLock volume' => C_RC_VOL_MOVE_SRC_IS_SNAPLOCK,
					   'Specified source volume is read only' => C_RC_VOLUME_IS_READ_ONLY,
					   'Aggregate holding the source volume and the destination aggregate are not homogeneous' => C_RC_SRC_AND_DEST_VOLS_NOT_HOMOGENEOUS,
					   },
			VOL_MOVE_STATUS => {'Specified source volume is not being moved' => C_RC_VOL_IS_NOT_BEING_MOVED,
					   },
			VOL_MOVE_CUTOVER => {'' => C_RC_SUCCESS,
					     },
			VOL_MOVE_PAUSE => {'' => C_RC_SUCCESS,
					   },
			VOL_MOVE_IS_PAUSED => {'\(paused\)' => C_RC_SUCCESS,
					       'Source\s+Destination\s+CO Attempts\s+CO Time\s+State' => C_RC_VOL_MOVE_IS_IN_PROGRESS,
					       'Specified source volume is not being moved' => C_RC_VOL_IS_NOT_BEING_MOVED,
					       },
			VOL_ONLINE_RO => {'Volume \S+ is now online' => C_RC_SUCCESS,
					  },
			COMPRESSION_STATS_START => {'Stats identifier name is\s+\S+' => C_RC_SUCCESS,
					     },
			'WAFL_PUNCH' => {'wafl punch: status=0' => C_RC_SUCCESS,
					 },
			'GET_VBN_INFO_ERROR' => {'Error: fbn \d+ is not accessible\. Error code: 302' => C_RC_FBN_NOT_ACCESSIBLE_ERROR_302,
						 'fbn\s+\d+\s+is\s+vbn\s+\d+,\s+vvbn\s+\d+\s*$' => C_RC_SUCCESS,},
                };
        }
        elsif ($boot_mode eq C_ONTAP_CMODE)
        {
                #####################################
                ####### C-mode output strings #######
                #####################################
                $self->{messages} = {
                        OPTION_SET => {'' => C_RC_SUCCESS,
                                       'volume \S+ does not exist' => C_RC_VOLUME_NOT_FOUND,
                                       },
                        IS_COMPRESSION_ENABLED => {'Invalid path specification: \S+' => C_RC_INVALID_PATH,
                                                   'Compression:\s+Disabled\nInline Compression:\s+Disabled' => _C_RC_COMPR_OFF,
                                                   'Compression:\s+Enabled\nInline Compression:\s+Disabled' => _C_RC_COMPR_ON_NO_INLINE,
                                                   'Compression:\s+Enabled\nInline Compression:\s+Enabled' => _C_RC_COMPR_ON_FULLY_FUNCTIONAL,
                                                   'There are no entries matching your query' => _C_RC_COMPR_OFF,
                                                   },
                        ENABLE_COMPRESSION => {'This operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                                               '' => C_RC_SUCCESS,
                                               'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
                                               'Compression is not allowed at this time because undo scanner is in progress on this volume' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
                                               'Volume type has to be Flexvol:' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                               'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                                               'Compression is not allowed on a root volume' =>  C_RC_OP_NOT_ALLOWED_ON_ROOT_VOL,
                                               'Compression can only be enabled on a 64-bit volume contained in a 64-bit aggregate' => C_RC_COMPRESSION_ALLOWED_ONLY_ON_64_BIT,
                                               'Inline compression cannot be enabled when background compression is disabled' => C_RC_COMPRESSION_NOT_TURNED_ON,
                                               'Compression is not allowed on a volume that is part of a Remote SnapMirror relationship' => C_RC_COMPRESSION_NOT_ALLOWED_ON_RSM_VOL,
                                               'Compression is not allowed at this time because a clone operation is in progress on this volume' => C_RC_VOL_CLONE_IN_PROGRESS,
                                               },
                        DISABLE_COMPRESSION => {'' => C_RC_SUCCESS,
                                                'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
                                                #'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                                'Background compression cannot be disabled when inline compression is enabled on the volume' => C_RC_INLINE_COMPRESSION_NOT_OFF,
						'Compression cannot be disabled while an efficiency operation is in progress on the volume' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
                                                },
                        GET_SPACE_SAVINGS => {'df: \S+: No such file or directory' => C_RC_VOLUME_NOT_FOUND,
                                              'unrecognized argument' => C_RC_INVALID_COMMAND_ARGUMENT,
                                              },
                        VOL_ONLINE => {'' => C_RC_SUCCESS,
                                       'Unable to online volume because there is not enough space in the aggregate' => C_RC_INSUFFICIENT_SPACE,
                                       },
                        AUTOSIZE => {'' => C_RC_SUCCESS,
                                     },
                        VOL_CLONE_SPLIT_STOP => {'' => C_RC_SUCCESS,
                                                 'command failed: Not found' => C_RC_VOLUME_IS_NOT_BEING_SPLIT,
                                                 'Flexclone \S+ not found. Reason: entry doesn\'t exist' => C_RC_VOLUME_IS_NOT_A_CLONE,
                                                 },
                        VOL_CLONE_SPLIT_START => {'' => C_RC_SUCCESS,
                                                  'The volume has restricted snapshots' => C_RC_VOLUME_HAS_RESTRICTED_SNAPSHOTS,
						  'The volume is not a clone' => C_RC_VOLUME_IS_NOT_A_CLONE,
                                                  },
                        SFSR_START => {'' => C_RC_SUCCESS,
                                       # Unfortunatelly, the error message for sfsr is too generic. Leave like this for now, until it gets fixed
                                       'Job failed: Failed to restore file from snapshot .+' => C_RC_INSUFFICIENT_SPACE,
                                       },
                        SFSR_TO_PATH_START => {'Job succeeded' => C_RC_SUCCESS,},
                        #LICENSE_REMOVE => {'Compression must be disabled .+ before removing the license' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                        #                  },
                        COMPRESS_START => {'The efficiency operation for volume \S+ of vserver \S+ has started' => C_RC_SUCCESS,
                                           'Failed to start efficiency on volume .+ Operation is currently active' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
                                           'Failed to start efficiency on volume .+ Background compression not enabled' => C_RC_COMPRESSION_NOT_TURNED_ON,
                                           },
                        COMPRESS_STATUS => {'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
                                           },
                        COMPRESS_STOP => {'The efficiency operation for volume .+ is being stopped' => C_RC_SUCCESS,
                                          'Failed to stop efficiency on volume .+ Operation is currently idle' => C_RC_SCANNER_IDLE, 
                                          },
                        _PARSE_CHECKPOINT_TIME => {'No Checkpoint' => C_RC_CHECKPOINT_FILE_EMPTY,
                                                   '\w+\s+\w+\s+\d+\s+\d+:\d+:\d+\s+\S+\s+\d+' => C_RC_SUCCESS,
                                                   },
                        DECOMPRESS_START => {'The efficiency undo operation for volume \S+ of vserver \S+ has started' => C_RC_SUCCESS,
                                             #'Compression is not disabled on the volume' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                             'No compressed data in the volume' => C_RC_NO_DATA_TO_DECOMPRESS,
                                             'Insufficient free space available for the \"efficiency undo\" operation to complete on volume' => C_RC_INSUFFICIENT_SPACE,
                                             #'Please break all the snapmirror relationships in volume \S+' => C_RC_VOLUME_HAS_RSM_RELATIONSHIPS,
                                            },
                        DECOMPRESS_STOP => {'The efficiency operation for volume .+ is being stopped' => C_RC_SUCCESS,
                                            'Failed to stop efficiency on volume .+ Operation is currently idle' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                            'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
                                           },
                        DECOMPRESS_STATUS => {'There are no entries matching your query' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                             },
                        DEDUPE_START => {'' => C_RC_SUCCESS,
                                         #'Operation is currently active: \S+' => C_RC_SCANNER_STILL_ACTIVE,
					 'Operation is not allowed because a volume move is transferring SIS metadata' => C_RC_NOT_ALLOWED_VOL_MOVE_TRANSFER_SIS_METADATA,
                                         },
                        DEDUPE_SET_STATE => {'' => C_RC_SUCCESS,
                                             # Do not fail if sis is already on/off
                                             'Operation has already been' => C_RC_SUCCESS,
					     'Volume state transition is in progress' => C_RC_VOL_STATE_TRANSITION_IN_PROGRESS,
					     'Operation is not allowed because a volume move is in progress' => C_RC_NOT_ALLOWED_VOL_MOVE_IN_PROGRESS,
                                             },
                        DEDUPE_STOP => {'' => C_RC_SUCCESS,
                                       },
			DEDUPE_OFF_UNDO => {'' => C_RC_SUCCESS,
					    },
                        DEDUPE_WAIT => {'' => C_RC_SUCCESS,
                                        'There are no entries matching your query' => C_RC_SCANNER_IDLE,
                                        },
                        DUMPBLOCK_FILE => {'This is compressed block' => C_RC_DUMPED_COMPRESSED_BLOCK,
                                           '\s0x\d+\s' => C_RC_DUMPED_UNCOMPRESSED_BLOCK,
                                           },
                        SYNC => {'' => C_RC_SUCCESS,
                                },
                        AGGR_SNAP_DELETE => {'deleting snapshot' => C_RC_SUCCESS,
                                            },
                        DETECT_ROOT_VOL => {'root,\s' => C_RC_ROOT_VOLUME_DETECTED,
                                           },
                        FILE_FOLDING_START => {'' => C_RC_SUCCESS,
                                               },
                        FILE_FOLDING_ABORT => {'' => C_RC_SUCCESS,
                                               },
                        FILE_RESERVATION => {'space reservations for file \S+: off' => C_RC_FILE_RESERVATION_OFF,
                                             'space reservations for file \S+: on' => C_RC_FILE_RESERVATION_ON,
                                             },
                        SIS_CHECK => {'Fingerprints of \S+ are being verified' => C_RC_SUCCESS,
                                      'Number of changelog buckets' => C_RC_SUCCESS,
                                      },
                        LA_IPU_STATUS => {'' => C_RC_SUCCESS,
                                          '64-bit upgrade is not currently in progress' => C_RC_LA_IPU_NOT_IN_PROGRESS,
                                          },
                        SET_WAFL_SCAN_SPEED => {'' => C_RC_SUCCESS,
                                                },
                        VOL_COPY_START => {'' => C_RC_SUCCESS,
                                           },
			VOL_SNAP_RESTORE => {'' => C_RC_SUCCESS,
					     },
			DELETE_FILE => {'Read-only file system' => C_RC_READ_ONLY_FILESYSTEM,
					},
			FILE_CLONE_START => {'' => C_RC_SUCCESS,
					     'No space' => C_RC_INSUFFICIENT_SPACE,},
			AGGR_BLOCK_CHECK_START => {'block_check already running for aggregate \S+' => C_RC_AGGR_BLOCK_CHECK_IN_PROGRESS,
						   },
			SNAPMIRROR_INITIALIZE => { '' => C_RC_SUCCESS,
						  'Replication cannot be performed because the destination volume .+ is in an aggregate that does not support compression' => C_RC_COMPRESSION_NOT_SUPPORTED_ON_DEST,
						   },
			VOL_MOVE_START => {'' => C_RC_SUCCESS,
					   'Use the \"job show -id \d+\" command to view the status of this operation'=> C_RC_SUCCESS,
					   },
			FIND_VOL_MOVE_STATE => {'Volume move job in setup' => _C_RC_VOL_MOVE__SETUP_STATE,
						'\"Volume move job preparing transfer' => _C_RC_VOL_MOVE__SETUP_STATE,
						'\"Transferring data' => _C_RC_VOL_MOVE__MOVE_STATE,
						'Cutover Deferred' => _C_RC_VOL_MOVE__CUTOVER_DEFERRED_STATE,
						'Cutover Started' => _C_RC_VOL_MOVE__CUTOVER_STATE,
						},
			VOL_MOVE_CUTOVER => {'' => C_RC_SUCCESS,
					     },
			COMPRESSION_STATS_START => {'Stats identifier name is\s+\S+' => C_RC_SUCCESS,
					     },
			'WAFL_PUNCH' => {'wafl punch: status=0' => C_RC_SUCCESS,
					 },
			'GET_VBN_INFO_ERROR' => {'Error: fbn \d+ is not accessible\. Error code: 302' => C_RC_FBN_NOT_ACCESSIBLE_ERROR_302,
						 'fbn\s+\d+\s+is\s+vbn\s+\d+,\s+vvbn\s+\d+\s*$' => C_RC_SUCCESS,},
			'SNAPMIRROR_UPDATE' => {'' => C_RC_SUCCESS,},
                };
        }
        else
        {
                logresult('script', "Unrecognized boot_mode: \"$boot_mode\"");
        }
        
	
        logcomment("Created new Messages :: Constants_8_1 object\n" . Dumper($self)) if ($self->{_debug});
        
	bless $self, $class;
	return $self;
}



1;
