#
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/UI/CLI/Messages/Constants_BR_0.pm#1 $
#

package Compression::UI::CLI::Messages::Constants_BR_0;

  
use Tharn;

use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);

use Compression::Constants qw(:all);


BEGIN
{
	require "Compression/base_import.pl";
	require Exporter;
	our @ISA = qw(Exporter);
        
};


sub new
{
	my $class = shift;
	my %args = validate(@_,
	{       
                debug  => {  
			type 	 => SCALAR,
			optional => 1,
                        default  => FALSE,
			},
                
                boot_mode => {type => SCALAR,
                              optional => 1,
                              default  => C_ONTAP_BOOT_MODE,
                              },
	} );
        
	my $self = {};
        $self->{_debug} = $args{debug};
        
        my $boot_mode = $args{boot_mode};
        $self->{_boot_mode} = $boot_mode;
        
        if ($boot_mode ne C_ONTAP_7MODE)
        {
                logresult('script', "There is no support for \"$boot_mode\" boot mode in 8.0");
        }
        
        # TODO: rename this module to Constants_8_0
        $self->{messages} = {
                OPTION_SET => {'Operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                               'Operation failed because volume is being decompressed' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
                               'Volume \S+ is not online' => C_RC_VOLUME_NOT_ONLINE,
			       'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                               'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                               '\'compression\' is not enabled for volume' => C_RC_COMPRESSION_NOT_TURNED_ON,
                               '\'compression\' is already enabled for volume' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                               'No volume named \S+ exists' => C_RC_VOLUME_NOT_FOUND,
                               'CR_OP_NOT_ALLOWED_ON_ROOT_VOL' => C_RC_OP_NOT_ALLOWED_ON_ROOT_VOL,
                               },
                IS_COMPRESSION_ENABLED => {'compression=on' => _C_RC_COMPR_ON_FULLY_FUNCTIONAL,
                                           'compression=off' => _C_RC_COMPR_OFF,
                                           },
                ENABLE_COMPRESSION => {'Operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                                       'Operation failed because volume is being decompressed' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
                                       'Volume \S+ is not online' => C_RC_VOLUME_NOT_ONLINE,
                                       'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                       'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                                       'Volume \S+ is a snapmirrored volume' => C_RC_COMPRESSION_NOT_ALLOWED_ON_RSM_VOL,
                                       '' => C_RC_SUCCESS,
                                       },
                DISABLE_COMPRESSION => {'Operation requires a compression license' => C_RC_NO_COMPRESSION_LICENSE,
                                       'Volume \S+ is not online' => C_RC_VOLUME_NOT_ONLINE,
                                       'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
                                       'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
                                       'Compression of existing data is currently active on volume \S+' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
                                       '' => C_RC_SUCCESS,
                                       },
                DECOMPRESS_START => {'decompress of volume \S+ started' => C_RC_SUCCESS,
                                     'Please turn off the "compression" option for volume' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                     'No compressed data in volume' => C_RC_NO_DATA_TO_DECOMPRESS,
                                     'Not enough disk space to decompress volume' => C_RC_INSUFFICIENT_SPACE,
                                     'Volume \S+ is readonly' => C_RC_VOLUME_IS_READ_ONLY,
                                     'Please break all the snapmirror relationships in volume \S+' => C_RC_VOLUME_HAS_RSM_RELATIONSHIPS,
                                    },
                DECOMPRESS_STOP => {'' => C_RC_SUCCESS,
                                    'No decompression in progress for volume ' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                    'Invalid volume name \S+ specified' => C_RC_VOLUME_NOT_FOUND,
                                   },
                DECOMPRESS_STATUS => {'No volume decompression in progress' => C_RC_NO_DECOMPRESSION_IN_PROGR,
                                     },
                COMPRESS_START => {'' => C_RC_SUCCESS,
                                   'Compression of existing data is currently active on volume \S+' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
                                   'Operation failed because compression option is not enabled on volume \S+' => C_RC_COMPRESSION_NOT_TURNED_ON,
                                   'A-SIS gathering operation is active on volume \S+, compression of existing data is not allowed to proceed' => C_RC_SIS_SCANNER_IS_ACTIVE,
                                   },
                COMPRESS_STOP => {'' => C_RC_SUCCESS,
                                  'Compression of existing data  for volume \S+ is not active' => C_RC_SCANNER_IDLE,
                                  'No volume named \S+ was found' => C_RC_VOLUME_NOT_FOUND,
                                  'Operation failed because compression option is not enabled on volume \S+' => C_RC_COMPRESSION_NOT_TURNED_ON,
                                  },
                COMPRESS_STATUS => {'Compression of existing data is not active on volume \S+' => C_RC_SCANNER_IDLE,
                                    },
                COMPRESS_STATUS_CHECKPOINT => {'Compression of existing data is not active on volume \S+' => C_RC_SCANNER_IDLE,
                                               'Checkpoint file is empty' => C_RC_CHECKPOINT_FILE_EMPTY,
                                               'Checkpoint does not exist' => C_RC_CHECKPOINT_DOES_NOT_EXIST,
                                               },
                _PARSE_CHECKPOINT_TIME => {'No Checkpoint' => C_RC_CHECKPOINT_FILE_EMPTY,
                                           '\w+\s+\w+\s+\d+\s+\d+:\d+:\d+\s+\S+\s+\d+' => C_RC_SUCCESS,
                                           },
                LICENSE_ADD => {'This license is only supported either on NearStore platforms' => C_RC_NO_NEARSTORE_LICENSE,
                               },
                LICENSE_REMOVE => {'Compression must be disabled .+ before removing the license' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                  },
                GET_SPACE_SAVINGS => {'df: \S+: No such file or directory' => C_RC_VOLUME_NOT_FOUND,
                                      ': unrecognized argument ' => C_RC_INVALID_COMMAND_ARGUMENT,
                                      },
                VOL_ONLINE => {'' => C_RC_SUCCESS,
                               'Unable to online volume as space cannot be guaranteed for the volume' => C_RC_INSUFFICIENT_SPACE,
			       'Compression is\s*\n*\s*not supported in this version of Data ONTAP' => C_RC_COMPRESSION_NOT_SUPPORTED,
                               },
		AUTOSIZE => {'autosize settings UPDATED' => C_RC_SUCCESS,
                             },
                VOL_CLONE_SPLIT_STOP => {'' => C_RC_SUCCESS,
                                         'Clone volume is not being split from its parent' => C_RC_VOLUME_IS_NOT_BEING_SPLIT,
                                         'The volume is not a clone' => C_RC_VOLUME_IS_NOT_A_CLONE,
                                         },
                VOL_CLONE_SPLIT_START => {'' => C_RC_SUCCESS,
					  'The volume has restricted snapshots' => C_RC_VOLUME_HAS_RESTRICTED_SNAPSHOTS,
                                          },
                SFSR_START => {'No space left on device' => C_RC_INSUFFICIENT_SPACE,
                               },
		DEDUPE_START => {'' => C_RC_SUCCESS,
				 'Operation is currently active: \S+' => C_RC_SCANNER_STILL_ACTIVE,
				 'TODO need to update error message when available' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
				 'Operation is not enabled: \S+' => C_RC_SIS_NOT_TURNED_ON,
				 },
		DEDUPE_SET_STATE => {'' => C_RC_SUCCESS,
				     'Cannot disable SIS when FAS compression is enabled on volume : \S+' => C_RC_COMPRESSION_NOT_TURNED_OFF,
				     },
		DEDUPE_STOP => {'' => C_RC_SUCCESS,
			       },
		DEDUPE_OFF_UNDO => {'' => C_RC_SUCCESS,
				    },
                DUMPBLOCK_FILE => {'This is compressed block' => C_RC_DUMPED_COMPRESSED_BLOCK,
                                   '\s0x\d+\s' => C_RC_DUMPED_UNCOMPRESSED_BLOCK,
                                   },
                FILE_RESERVATION => {'space reservations for file \S+: off' => C_RC_FILE_RESERVATION_OFF,
                                     'space reservations for file \S+: on' => C_RC_FILE_RESERVATION_ON,
                                     },
                FILE_FOLDING_START => {'' => C_RC_SUCCESS,
                                       },
                FILE_FOLDING_ABORT => {'' => C_RC_SUCCESS,
                                       },
		LA_IPU_STATUS => {'' => C_RC_SUCCESS,
				  '64-bit upgrade is not currently in progress' => C_RC_LA_IPU_NOT_IN_PROGRESS,
                                  },
                DECOMPRESS_REVERT => {'Please turn off the "compression" option for volume' => C_RC_COMPRESSION_NOT_TURNED_OFF,
                                      '' => C_RC_SUCCESS,
                                      },
                SET_SNAPLOCK_AUTO_COMMIT => {'^0$' => C_RC_SUCCESS,
                                             },
                SNAPMIRROR_RESYNC => {'' => C_RC_SUCCESS,
                                      'process was aborted' => C_RC_PROCESS_WAS_ABORTED,
                                      },
                SNAPVAULT_UPDATE => {'' => C_RC_SUCCESS,
                                     'No space left on device' => C_RC_INSUFFICIENT_SPACE,
                                     'qtree is currently involved in another operation' => C_RC_QTREE_INVOLVED_IN_OTHER_OPERATION,},
                SET_WAFL_SCAN_SPEED => {'' => C_RC_SUCCESS,
                                        },
                VOL_COPY_START => {'' => C_RC_SUCCESS,
                                   'Aborting: Destination volume, \S+, is too small' => C_RC_INSUFFICIENT_SPACE,
                                   'Aborting: the aggregate that contains destination volume \S+ does not support compression' => C_RC_COMPRESSION_NOT_SUPPORTED_ON_DEST,
                                   },
                AGGR_COPY_START => {'' => C_RC_SUCCESS,
                                    },
		VOL_SNAP_RESTORE => {'' => C_RC_SUCCESS,
				     'The following prevents snap restore\s+.*\s+compression scanner running' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
				     },
		DELETE_FILE => {'Read-only file system' => C_RC_READ_ONLY_FILESYSTEM,
				},
		FILE_CLONE_START => {'^\d+$' => C_RC_SUCCESS,
				     'clone start: No space' => C_RC_INSUFFICIENT_SPACE,
				     },
		FILE_CLONE_CLEAR => {'' => C_RC_SUCCESS,
				     },
		FILE_CLONE_STOP => {'' => C_RC_SUCCESS,
				    },
		AGGR_BLOCK_CHECK_START => {'block_check already running for aggregate \S+' => C_RC_AGGR_BLOCK_CHECK_IN_PROGRESS,
					   },
		VOL_MOVE_START => {'' => C_RC_SUCCESS,
				   },
		VOL_MOVE_STATUS => {'Specified source volume is not being moved' => C_RC_VOL_IS_NOT_BEING_MOVED,
				    },
		VOL_MOVE_CUTOVER => {'' => C_RC_SUCCESS,
				     },
		VOL_MOVE_PAUSE => {'' => C_RC_SUCCESS,
				   },
		VOL_MOVE_IS_PAUSED => {'\(paused\)' => C_RC_SUCCESS,
				       'Source\s+Destination\s+CO Attempts\s+CO Time\s+State' => C_RC_VOL_MOVE_IS_IN_PROGRESS,
				       'Specified source volume is not being moved' => C_RC_VOL_IS_NOT_BEING_MOVED,
				       },
		VOL_ONLINE_RO => {'Volume \S+ is now online' => C_RC_SUCCESS,
				  },
		'WAFL_PUNCH' => {'wafl punch: status=0' => C_RC_SUCCESS,
				 },
	};
        
	
        logcomment("Created new Messages :: Constants_BR_0 object\n" . Dumper($self)) if ($self->{_debug});
        
	bless $self, $class;
	return $self;
}



1;
