#
# $Id: //depot/prod/test/Rsierranevada/lib/Compression/UI/ZAPI/Messages/Constants_8_2.pm#2 $
#

package Compression::UI::ZAPI::Messages::Constants_8_2;

  
use Tharn;

use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);

use Compression::Constants qw(:all);


BEGIN
{
	require "Compression/base_import.pl";
};


sub new
{
	my $class = shift;
	my %args = validate(@_,
	{
		debug => {type 	 => SCALAR,
			  optional => 1,
			  default  => FALSE,},
		
		boot_mode => {type => SCALAR,
			      optional => 1,
			      default  => C_ONTAP_BOOT_MODE,
			      },
	});
        
	my $self = {};
        $self->{_debug} = $args{debug};
        
        my $boot_mode = $args{boot_mode};
        $self->{_boot_mode} = $boot_mode;
        
        if ($boot_mode eq C_ONTAP_7MODE)
        {
                #####################################
                ####### 7-mode output strings #######
                #####################################
                $self->{messages} = {
                        IS_COMPRESSION_ENABLED => {'Invalid path specification' => C_RC_INVALID_PATH,
                                                   '<is-compression-enabled>false<.+<is-inline-compression-enabled>false<' => _C_RC_COMPR_OFF,
                                                   '<is-compression-enabled>true<.+<is-inline-compression-enabled>false<' => _C_RC_COMPR_ON_NO_INLINE,
                                                   '<is-compression-enabled>true<.+<is-inline-compression-enabled>true<' => _C_RC_COMPR_ON_FULLY_FUNCTIONAL,
                                                   'No status entry found' => _C_RC_COMPR_OFF,
						   'Volume state transition is in progress' => C_RC_VOL_STATE_TRANSITION_IN_PROGRESS,
                                                   },
			ENABLE_COMPRESSION => {'Volume does not exist' => C_RC_VOLUME_NOT_FOUND,
					       '<results status="passed"></results>' => C_RC_SUCCESS,
					       'Compression is not allowed at this time because undo scanner is in progress on this volume' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
					       'Volume type has to be Flexvol' => C_RC_VOLUME_TYPE_NOT_FLEX,
					       'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
					       'Compression is not allowed on a root volume' =>  C_RC_OP_NOT_ALLOWED_ON_ROOT_VOL,
					       'Compression can only be enabled on a 64-bit volume contained in a 64-bit aggregate' => C_RC_COMPRESSION_ALLOWED_ONLY_ON_64_BIT,
					       'Inline compression cannot be enabled when background compression is disabled' => C_RC_COMPRESSION_NOT_TURNED_ON,
					       'Compression is not allowed on a volume that is part of a Remote SnapMirror relationship' => C_RC_COMPRESSION_NOT_ALLOWED_ON_RSM_VOL,
					       'Compression is not allowed at this time because a clone operation is in progress on this volume' => C_RC_VOL_CLONE_IN_PROGRESS,
					      },
			DISABLE_COMPRESSION => {'Volume does not exist' => C_RC_VOLUME_NOT_FOUND,
						'<results status="passed"></results>' => C_RC_SUCCESS,
						'Compression cannot be disabled when inline compression is enabled on the volume' => C_RC_INLINE_COMPRESSION_NOT_OFF,
						'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
						'Compression cannot be disabled while an efficiency operation is in progress on the volume' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
					       },
			COMPRESS_START => {'<results status="passed"></results>' => C_RC_SUCCESS,
					   'Invalid operation on a SnapVault secondary volume: \S+' => C_RC_NOT_ALLOWED_ON_SNAPVAULT_SECONDARY,
					   'Operation is currently active' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
					   'Volume is offline' => C_RC_VOL_IS_OFFLINE,
					   'Volume is restricted' => C_RC_VOL_IS_RESTRICTED,
					   },
			COMPRESS_STOP => {'<results status="passed"></results>' => C_RC_SUCCESS,
					  'Operation is currently idle' => C_RC_SCANNER_IDLE,
					  'Volume does not exist' => C_RC_VOLUME_NOT_FOUND,
					  'Volume is offline' => C_RC_VOL_IS_OFFLINE,
					  'Volume is restricted' => C_RC_VOL_IS_RESTRICTED,
					  },
			DECOMPRESS_STOP => {'<results status="passed"></results>' => C_RC_SUCCESS,
					    'Operation is currently idle: \S+' => C_RC_NO_DECOMPRESSION_IN_PROGR,
					    'Volume does not exist' => C_RC_VOLUME_NOT_FOUND,
					    },
			COMPRESS_STATUS => {'Volume does not exist' => C_RC_VOLUME_NOT_FOUND,
					    'No Status entry found' => C_RC_VOLUME_NOT_FOUND,
					    },
			DECOMPRESS_STATUS => {'No status entry found' => C_RC_NO_DECOMPRESSION_IN_PROGR,
					      },
			GET_SPACE_SAVINGS => {'No volume named \S+ exists' => C_RC_VOLUME_NOT_FOUND,
					      },
			_PARSE_CHECKPOINT_TIME => {'^0$' => C_RC_CHECKPOINT_FILE_EMPTY,
						   '\d+\d+' => C_RC_SUCCESS,
						   },
			CONFIGURE_IDD => {'<results status="passed"></results>' => C_RC_SUCCESS,
					  'Inline compression or Incompressible data detection cannot be enabled when compression is disabled' => C_RC_COMPRESSION_NOT_TURNED_ON,
					  },
			IS_IDD_ENABLED => {'<is-idd-enabled>true</is-idd-enabled>' => C_RC_IDD_ON,
					   '<is-idd-enabled>false</is-idd-enabled>' => C_RC_IDD_OFF,
					   },
                };
        }
        elsif ($boot_mode eq C_ONTAP_CMODE)
        {
                #####################################
                ####### C-mode output strings #######
                #####################################
                $self->{messages} = {
			IS_COMPRESSION_ENABLED => {'Invalid path specification' => C_RC_INVALID_PATH,
						   '<is-compression-enabled>false<.+<is-inline-compression-enabled>false<' => _C_RC_COMPR_OFF,
						   '<is-compression-enabled>true<.+<is-inline-compression-enabled>false<' => _C_RC_COMPR_ON_NO_INLINE,
						   '<is-compression-enabled>true<.+<is-inline-compression-enabled>true<' => _C_RC_COMPR_ON_FULLY_FUNCTIONAL,
						   'No status entry found' => _C_RC_COMPR_OFF,
						   'Volume state transition is in progress' => C_RC_VOL_STATE_TRANSITION_IN_PROGRESS,
						   },
			ENABLE_COMPRESSION => {'<results status="passed"></results>' => C_RC_SUCCESS,
					       'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
					       'Compression is not allowed at this time because undo scanner is in progress on this volume' => C_RC_DECOMPRESSION_STILL_IN_PROGRESS,
					       'Volume type has to be Flexvol' => C_RC_VOLUME_TYPE_NOT_FLEX,
					       'Illegal option for SnapLock volume' => C_RC_ILLEGAL_OPTION_FOR_SNAPLOCK_VOL,
					       'Compression is not allowed on a root volume' =>  C_RC_OP_NOT_ALLOWED_ON_ROOT_VOL,
					       'Compression can only be enabled on a 64-bit volume contained in a 64-bit aggregate' => C_RC_COMPRESSION_ALLOWED_ONLY_ON_64_BIT,
					       'Inline compression cannot be enabled when background compression is disabled' => C_RC_COMPRESSION_NOT_TURNED_ON,
					       'Compression is not allowed on a volume that is part of a Remote SnapMirror relationship' => C_RC_COMPRESSION_NOT_ALLOWED_ON_RSM_VOL,
					       'Compression is not allowed at this time because a clone operation is in progress on this volume' => C_RC_VOL_CLONE_IN_PROGRESS,
					       },
			DISABLE_COMPRESSION => {'<results status="passed"></results>' => C_RC_SUCCESS,
						'Volume does not exist:\S+' => C_RC_VOLUME_NOT_FOUND,
						'Compression cannot be disabled when inline compression is enabled on the volume' => C_RC_INLINE_COMPRESSION_NOT_OFF,
						'A flexible volume is required for this operation' => C_RC_VOLUME_TYPE_NOT_FLEX,
						'Compression cannot be disabled while an efficiency operation is in progress on the volume' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
						},
			GET_SPACE_SAVINGS => {'No volume named \S+ exists' => C_RC_VOLUME_NOT_FOUND,
					      },
			COMPRESS_START => {'<results status="passed"></results>' => C_RC_SUCCESS,
					   'Invalid operation on a SnapVault secondary volume: \S+' => C_RC_NOT_ALLOWED_ON_SNAPVAULT_SECONDARY,
					   'Operation is currently active' => C_RC_COMPRESSION_STILL_IN_PROGRESS,
					   },
			COMPRESS_STATUS => {'Volume does not exist' => C_RC_VOLUME_NOT_FOUND,
					    },
			COMPRESS_STOP => {'<results status="passed"></results>' => C_RC_SUCCESS,
					  'Volume \S+ does not exist' => C_RC_VOLUME_NOT_FOUND,
					  'Operation is currently idle' => C_RC_SCANNER_IDLE,
					  },
			_PARSE_CHECKPOINT_TIME => {'^0$' => C_RC_CHECKPOINT_FILE_EMPTY,
						   '\d+\d+' => C_RC_SUCCESS,
						   },
			DECOMPRESS_STATUS => {'No status entry found' => C_RC_NO_DECOMPRESSION_IN_PROGR,
					      },
			DECOMPRESS_STOP => {'<results status="passed"></results>' => C_RC_SUCCESS,
					    'Operation is currently idle: \S+' => C_RC_NO_DECOMPRESSION_IN_PROGR,
					    'Volume does not exist' => C_RC_VOLUME_NOT_FOUND,
					    },
			CONFIGURE_IDD => {'<results status="passed"></results>' => C_RC_SUCCESS,
					  'Inline compression or Incompressible data detection cannot be enabled when compression is disabled' => C_RC_COMPRESSION_NOT_TURNED_ON,
					  },
			IS_IDD_ENABLED => {'<is-idd-enabled>true</is-idd-enabled>' => C_RC_IDD_ON,
					   '<is-idd-enabled>false</is-idd-enabled>' => C_RC_IDD_OFF,
					   },
                };
        }
        else
        {
                logresult('script', "Unrecognized boot_mode: \"$boot_mode\"");
        }
        
	
        logcomment("Created new Messages :: Constants_8_2 object\n" . Dumper($self)) if ($self->{_debug});
        
	bless $self, $class;
	return $self;
}



1;
