#!perl -w
# Hey emacs, this is basically -*- perl -*-
#
#Copyright (c) 2009 Network Appliance, Inc.
#All rights reserved.
#
## @summary Dover the Dispatcher 
##
## @description
##
## Below is the the Dispatcher for running NATE scripts to test compression on filers 
##
## @usage
##
## @keywords StressBed TestBed Dispatcher 
## @dependencies
## @author pade@netapp.com
##
## @status private
##

package Compression::tools::Dispatcher;

use strict;
use warnings;

use Tharn;
use XML::Simple qw(XMLin); 
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF BOOLEAN UNDEF OBJECT);
use Carp qw(croak);

##Global Variable
my $tabCount = 0;

## @name ::run
##
## @summary main subroutine
##
## @description
## Parses the xml and calls subroutines and libraries to handle the setup and execution
##
## @return 1
##
## @arg configfile Required: path to the configuration xml file
##
## @examples
## use Compression::Testbed::Dispatcher;
## run( configfile => "/path/to/testbed.xml"' );
##
## @author pade@netapp.com, dl-compression-qa@netapp.com

sub run
{
	my %args = validate (@_, { configfile => { type => SCALAR }});
	my $configFile = $args{configfile};
	# validate the XML 
	my $config = XMLin( $configFile ); 
	logcomment("configuration file: $configFile");
	open(XML, $configFile) or die("Could not open file $configFile");
	my @file = <XML>;
	close(XML);
	
	my $totalLineCount = @file;
	my $currentLineCount = 0;
	open(FILE, "+>>output.stest") or die ("Could not create output.stest file");

	while ($currentLineCount != $totalLineCount) {
		my $fileOutput;
		if($file[$currentLineCount] =~/<serial>/) {
			$fileOutput = &_insert_tab($tabCount);
			$fileOutput .= "serial {\n";
			if($file[($currentLineCount+1)] =~/<attributes>/) {
				my $formatString = &_attribute_create($currentLineCount, \@file);
				$fileOutput = &_insert_tab($tabCount);
				$fileOutput .= "serial$formatString {\n";
			}
			$tabCount++;
		} elsif ($file[$currentLineCount] =~/<\/serial>/) {
			$tabCount--;
			$fileOutput = &_insert_tab($tabCount);
			$fileOutput .= "}\n";
		} elsif ($file[$currentLineCount] =~/<parallel>/) {
			$fileOutput = &_insert_tab($tabCount);
			$fileOutput .= "parallel {\n";
			if($file[($currentLineCount+1)] =~/<attributes>/) {
				my $formatString = &_attribute_create($currentLineCount, \@file);
				$fileOutput = &_insert_tab($tabCount);
				$fileOutput .= "parallel$formatString {\n";
			}
			$tabCount++;
		} elsif ($file[$currentLineCount] =~/<\/parallel>/) {
			$tabCount--;
			$fileOutput = &_insert_tab($tabCount);
			$fileOutput .= "}\n";
		} elsif ($file[$currentLineCount] =~ /<test>(.+)<\/test>/) {
			$fileOutput = &_insert_tab($tabCount);
			$fileOutput .= $1 . "\n";
		}

		if(defined($fileOutput)) {
			print FILE $fileOutput;
		}
		$currentLineCount++;
	}
	close FILE;
	
	##execute the created stest file
	open(DATA, "+>>data.dat") or die ("Could not create data.dat file");
	my $dump = `ntest LOGDIR=- stest/stest.thpl STEST_FILE="output.stest"`;
	print DATA $dump; 
	close(DATA);
	 
	return 1;
}

## @name ::_attribute_create
##
## @summary (INTERNAL) generates the attributes for serial and parallel
##
## @description
## parses the XML file for certain tags and gathers the information between
## the tags to change into a .stest format. Currenlty does minimal checking of
## correct input, assuming that the users XML is perfect.
##
## @return scalar of the list of attributes to use 
##
## @arg lineSpot the current line we are on in the file 
## @arg file the array of lines in the file 
##
## @examples
## &_attribute_create($currentLineCount, \@file);

sub _attribute_create
{
#	my %args = validate (@_, line => {type => SCALAR}, file => {type => ARRAYREF}); validatation doesn't work 
	my $lineSpot = $_[0];
	my @file = @{$_[1]};
#	my $lineSpot = %args{line};
#	my @file = %args{file};

	my $attributeString = "";

	while ($file[$lineSpot] !~ /<\/attributes>/) {
		if($file[$lineSpot] =~ /<runtime>([0-9dhms]+)<\/runtime>/) {
			$attributeString .= " runtime=$1";
		} elsif ($file[$lineSpot] =~ /<iterations>([0-9]+)<\/iterations>/) {
			$attributeString .= " iterations=$1";
		} elsif ($file[$lineSpot] =~ /<alarm>([0-9dhms]+)<\/alarm>/) {
			$attributeString .= " alarm=$1";
		} elsif ($file[$lineSpot] =~ /<exit_on_errors>([\w|]+)<\/exit_on_errors>/) { #assumes the users entered it correctly
			$attributeString .= " exit_on_errors=$1";
		} elsif ($file[$lineSpot] =~ /<clients>([\w,-]+)<\/clients>/) {
			$attributeString .= " clients=$1";
		} elsif ($file[$lineSpot] =~ /<filers>([\w,-]+)<\/filers>/) {
			$attributeString .= " filers=$1";
		} elsif ($file[$lineSpot] =~ /<switches>(\w+)<\/switches>/) {
			$attributeString .= " switches?$1";
		} elsif ($file[$lineSpot] =~ /<runid>([\w-]+)<\/runid>/) {
			$attributeString .= " runid=$1";
		} elsif ($file[$lineSpot] =~ /<dirname>(\w+)<\/dirname>/) {
			$attributeString .= " dirname=$1";
		} elsif ($file[$lineSpot] =~ /<debug>(\w+)<\/debug>/) {
			$attributeString .= " debug=$1";
		} elsif ($file[$lineSpot] =~ /<run_block_value>(\w+)<\/run_block_value>/) {
			$attributeString .= " runtime=$1";
		} elsif ($file[$lineSpot] =~ /<test_controllers>([\w,-]+)<\/test_controllers>/) {
			$attributeString .= " test_controllers=$1";
		} elsif ($file[$lineSpot] =~ /<load_distrobution>(\w+)<\/load_distrobution>/) {
			$attributeString .= " load_distribution=$1";
		}
		$lineSpot++;
	}
	return $attributeString;
}

# Internal function
# name ::_insert_tab
#
# summary (INTERNAL) takes the input number and makes n tabs 
#
# description
# called to help format the output to the file to make it human readable
#
# @arg $amountOfTabs the number of tabs to place in the line 
# 
#	my %args	= validate (@_,	{ line => { type => SCALAR }});
#	my $line	= $args{line};
sub _insert_tab
{
	my $amountOfTabs = $_[0];
	my $output = "";
	for (my $i = 0; $i < $amountOfTabs; $i++) {
		$output .= "\t";
	}
	return $output;
}

return 1;
