# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Config/Cluster.pm#2 $

## @summary     Class for basic NFS configuration on GX
## @author      dl-afd@netapp.com
## @status      private
## @pod here

package Config::Cluster;

use strict;
### Modules for use

use Data::Dumper qw(Dumper);
use Utils::Error;
use Exporter;
use File::Basename;
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use TracePkg;
use Tharn;

### Global
use vars qw($Server $Trace @Topologies %Lastid_of 
            $Vs_Cnt %Cnt_of $Vol_Cnt);

### Constants
use constant PER_INSTANCE => "per_instance";
use constant SINGLE       => "single";
use constant FLEX         => "flex";
use constant STRIPED      => "striped";
use constant RAID4        => "raid4";
use constant RAID_DP      => "raid_dp";
use constant DATA         => "data";
use constant FULL         => "full";
use constant ON           => 1;

our @EXPORT=qw(PER_INSTANCE SINGLE FLEX STRIPED RAID4 RAID_DP DATA FULL);
 
our @ISA = qw(Exporter Utils::Error);

$Trace=new TracePkg();
######################################################################
#  P U B L I C   M E T H O D S
######################################################################
################################################################
# Subroutine Name: new()
# Objective:       create a config::config object
# Details:         See POD documentation at the end of this file
################################################################
sub new {
    my ($class,%args)=@_;
    my (
        $log,
        $self,
        );
    
    $Trace=new TracePkg();
    $Trace->entry();
    
    ### parse arguments
    $self={};
    
    bless $self,$class;
    $self->_clear_err();
    $self->_set_err_stack_depth(1);
    $self->_set_err_stack_trace(ON);
    
    eval{
        ($class, %args) = validate_mixed(
            @_,
            [ { type => SCALAR} ],
            {
            on_err  =>  {type => SCALAR,  required => 1, default=>"die",
                         callbacks=>{
                            'legal values'=>sub{shift()=~/die|rtn/i}}},       
            server  =>  {type => OBJECT,  required => 1 ,isa=>'Server', ,
                         callbacks=>{
                            'Type Of Filer is GX'=>sub{
                                Tharn::host(($args{server}->servers())[0])
                                ->hosttype()=~/ng/i;              
                         }}},
            }                
        );
    }; 
    if ($@){  
       $self->{on_err}=$args{on_err};   
       $self->_set_err(msg=>$@,stack_depth=>1);
       return $self;
    }
 
    $self->{node_cnt}=scalar($args{server}->servers());
    $self->{on_err}=$args{on_err};
    $self->{topology}={};
    $Server=$args{server};
    
    $Trace->exit();
    return $self;
}


################################################################
# Subroutine Name: create()
# Objective:       create test topology on cluster
# Details:         See POD documentation at the end of this file
################################################################
sub create{  
    my ($self,%args)=@_;
    my (
        $topology,
       );
    $Trace->entry();
    $self->_clear_err();
    
    eval {
        if ($self->{require_fresh}) {
            $Trace->comment(msg => 'Require Fresh');
            my @already_exists = (); 
            $topology = $Server->discover_topology(cmd_nodes=>["volume"]);
            foreach my $vol_of (@{$topology->{cluster}->{volume}}) {
                foreach my $name (keys %{$self->{client_vol_map}}) {
                    foreach my $vol_hsh (@{$self->{client_vol_map}{$name}}) {
                        if ($vol_hsh->{volume}->{volume}=~/^$vol_of->{volume}$/) {
                            push @already_exists , $vol_hsh->{volume}->{volume};
                        }
                    }
                }
            }
            if (scalar(@already_exists)) {
                my $vol_string = join ",", @already_exists;
                die "Volumes $vol_string already exist. Unable to satisfy the request for fresh volumes.";
            }
        }
        ### BUG: function doesn't return everything it creates.
        ### Need to discover again for specific cmd node.
        $Server->create_topology(
                    print_node=>undef,
                    topology=>$self->{topology});
        
        $topology = $Server->discover_topology(cmd_nodes=>["volume"]);
    };
    if ($@){
        $self->_set_err(msg=>$@,stack_depth=>1);
        return 0;
    }
    
    $self->_log("updating client_vol_map with actual available volume space");
    foreach my $vol_of (@{$topology->{volume}}){
        next if ($vol_of->{volume}!~/^p_/);
        foreach my $name (keys %{$self->{client_vol_map}}){
            foreach my $vol_hsh (@{$self->{client_vol_map}{$name}}){
                if ($vol_hsh->{vol_name}=~/^$vol_of->{volume}$/){
                    #push @{$self->{vol_of}=$vol_of;
                    $vol_hsh->{vol_availsize}=
                        $self->_size_to_mbytes($vol_of->{available})."mb";
                    last;
                }
            }
        }
    }
    
    #print ntest::NTEST_STDOUT Dumper($self->{vol_of});
    
    $Trace->exit();
    
    return 1;
}
################################################################
# Subroutine Name: define()
# Objective:       define test topology to be created by create()
# Details:         See POD documentation at the end of this file
################################################################
sub define {
    my ($self,%args)=@_;
    my (  
        $base,
        $name,
        $aggrs_of,
        $client_of,
        $exports_of,
        $prefix,
        %topology,
        $vifs_of,
        $vs_of,
        $vss_of,
        $vols_of,
        $rule_set,
        $shares_of,
        );
    
    $Trace->entry();
    
    # clear errors
    $self->_clear_err();
    
   
    # parse arguments
    eval{
        ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            aggregates         =>  {type => ARRAYREF|SCALAR, optional => 1},
            clients            =>  {type => ARRAYREF,
                                    callbacks=>{
                                     'Specifed 1 or more values'=>sub{                      
                                        scalar(@{shift()}>0)}}},                              
            data_vifs          =>  {type => ARRAYREF|SCALAR, optional => 1},
            instance           =>  {type => SCALAR, default => 1,
                                    callbacks=>{
                                     'Is An Integer>0'=>sub{shift()>0}}},                                                 
            mnt_mode           =>  {type => SCALAR, default => "777",
                                    callbacks=>{ 
                                     'Valid 3 digit permission'=>sub{
                                        shift()=~/([0-7]{3})/}}},           
            mnt_opts           =>  {type => SCALAR, optional => 1},
            prefix             =>  {type => SCALAR, optional => 1},
            raid_type          =>  {type => SCALAR, optional => 1,
                                    callbacks=>{
                                     'Legal Values'=>
                                        sub{shift()=~/RAID4|RAID_DP/i }}}, 
            timeout            =>  {type => SCALAR,  default => 300},            
            vol_cnt            =>  {type => SCALAR, 
                                    callbacks=>{
                                     'Is An Integer>0'=>sub{shift()>0}}},   
            vol_files          =>  {type => SCALAR, optional => 1},
            vol_format         =>  {type => SCALAR, optional => 1,
                                    callbacks=>{
                                     'Legal Values'=>
                                        sub{shift()=~/DATA|FULL/i or
                                            length($args{vol_format})==0}}}, 
            vol_group          =>  {type => SCALAR, optional => 1},            
            vol_layout         =>  {type => SCALAR, default => PER_INSTANCE,
                                    callbacks=>{
                                     'Legal Values'=>
                                        sub{shift()=~/PER_INSTANCE|SINGLE/i }}},           
            vol_maxdir         =>  {type => SCALAR, optional => 1},
            vol_owner          =>  {type => SCALAR, optional => 1},
            vol_perms          =>  {type => SCALAR, optional => 1 ,default => "777",
                                    callbacks=>{ 
                                     'Valid 3 digit permission'=>sub{
                                        shift()=~/([0-7]{3})/}}},                 
            vol_security_style =>  {type => SCALAR, optional => 1,
                                    callbacks=>{
                                     'Legal Values'=>
                                        sub{shift()=~/unix|mixed|ntfs/i }}},
            vol_size           =>  {type => SCALAR, default => "1gb"},
            vol_stripe_count   =>  {type => SCALAR, optional => 1,
                                     callbacks=>{
                                     'Between 2 and 64'=>
                                        sub{shift() =~/([2-64])/}}},
            vol_stripe_width   =>  {type => SCALAR, optional => 1},
            vol_style          =>  {type => SCALAR, default => FLEX,
                                    callbacks=>{
                                     'Legal Values'=>
                                        sub{shift()=~/STRIPED|FLEX/i }}},         
            vol_index_dir_enabled =>  {type => SCALAR, optional => 1,
                                    regex => qr/^(true|false)$/i},
            vservers           =>  {type => SCALAR, optional => 1},
            require_fresh      =>  {type => SCALAR, optional => 1, default => 0},
        }
        );
    }; 
    if ($@){
      $self->_set_err(msg=>$@,stack_depth=>10);
      return 0;
    } 
    
    $self->{require_fresh} = $args{require_fresh};

    # Create client hash
    $client_of=$self->_init_client_hash(@{$args{clients}}); 
    return 0 if ($self->error);
    
    # Define prefix name to use for created resources
    if (defined $args{prefix}) {
        $prefix=$args{prefix};
    }
    else {  
        ($prefix,undef,undef)=fileparse($ENV{TESTNAME},'\..*');     
    }
        
    $name = $self->_normalize_name(@{$args{clients}}[0]);
    if ($args{instance}) {
        $self->{rule_set} = "p_".$prefix."_".$name."_".$args{instance};
    } else {
        $self->{rule_set} = "p_".$prefix."_".$name."_1";
    }
    
    # get cluster topology 
    if (not defined $self->{cluster_topology}){
        $self->{cluster_topology}= $self->_get_cluster_topology();   
        return 0 if ($self->error);
    }
     
    # get aggregates 
    $aggrs_of=$self->_get_aggregates(
        aggregates => $args{aggregates},
        raid_type  => $args{raid_type},
        vol_style  => $args{vol_style}
        );
    return 0 if ($self->error);                                     
              
    # get vservers 
    $vss_of=$self->_get_vservers(
        vservers => $args{vservers},
        vifs_of  => $vifs_of
        );
    return 0 if ($self->error);
    
    # get data interfaces 
    $vifs_of=$self->_get_vifs(
        data_vifs => $args{data_vifs}
        );
    return 0 if ($self->error);
    
    # create vserver to vif map
    $vs_of=$self->_create_vserver_vif_map(
        vss_of  => $vss_of,
        vifs_of => $vifs_of,
        );
    return 0 if ($self->error);

    # create export rules 
    $exports_of=$self->_create_export_ruleset(
        vss_of   => $vss_of,
        rule_set => $self->{rule_set}               
        );
    return 0 if ($self->error);

    if ($args{vol_index_dir_enabled} =~ qr/true/i) {
        $Server->cli(cmd => 'set -privilege test');
    }
    
    # create volume layout 
    $vols_of=$self->_create_client_vol_layout(
        client_of  => $client_of,                                          
        instance   => $args{instance},
        prefix     => $args{prefix},
        vol_cnt    => $args{vol_cnt},
        vol_files  => $args{vol_files},            
        vol_layout => $args{vol_layout},
        vol_size   => $args{vol_size},
        vol_style   => $args{vol_style},
        vol_index_dir_enabled => $args{vol_index_dir_enabled},
        );
    if ($self->error){
        $self->_set_err(msg=>[$self->errmsg()], stack_trace=>0);
        $self->_clear_err();
        return 0;
    }

    # update volume layout
    $vols_of=$self->_update_volume_attributes(
        %args,
        vs_of    => $vs_of,
        vols_of  => $vols_of,
        aggrs_of => $aggrs_of,
        rule_set => $self->{rule_set},
        );

    # create cifs shares layout
    $shares_of=$self->_create_client_share_layout(
        client_of  => $client_of,
        vols_of    => $vols_of,
        vol_layout => $args{vol_layout},
        );

    # append add() definitions together
    push @{$self->{topology}{cluster}{volume}}, @{$vols_of->{volume}};

    push(@{$self->{topology}{cluster}{cifs_share}}, @{$shares_of->{cifs_share}});

    if (%{$exports_of}){
        push @{$self->{topology}{cluster}{exportruleset}},
             @{$exports_of->{exportruleset}};
        push @{$self->{topology}{cluster}{exportruleset_rules}},
             @{$exports_of->{exportruleset_rules}};
    }

    # create client to vol map
    $self->_create_client_vol_map($client_of, $vs_of, $vols_of, $shares_of);

    if ($self->error){
        my @errmsg=$self->errmsg();
        $self->_clear_err();
        push my @errors,
            @errmsg,      
            join("\n",$self->get_client_map(format=>1)),"\n",
            join("\n",$self->_get_aggregate_sizes($aggrs_of));
        $self->_set_err(msg=>[@errors],stack_trace=>0);    
        return 0;
    }
    
    $Trace->exit();
    
    return 1;
}

################################################################
# Subroutine Name: delete()
# Objective:       remove test topology from cluster
# Details:         See POD documentation at the end of this file
################################################################
sub delete {
    my ($self,%args)=@_;
    my (
        @cmds,
        );
    
    $Trace->entry();

    $self->_clear_err();
    
    @cmds=qw(cifs_share volume exportruleset);

    foreach my $cmd (@cmds){
        my $topology_of;
        $topology_of->{cluster}{$cmd}=$self->{topology}{cluster}{$cmd};
        eval {
            $Server->delete_topology(
                print_node => undef,
                topology   => $topology_of
            );
        };
        if ($@) {
            $self->_set_err(msg=>$@,stack_depth=>1);
            return 0;
        }
    }
    
        
    ### clear cache, need to reread
    if (!$args{noundefine}) {
        undefine($self,%args);
    }
    $Trace->exit();
    
    return 1;  
}

################################################################
# Subroutine Name: undefine()
# Objective:       undefine the define after delete
# Details:         See POD documentation at the end of this file
################################################################
sub undefine {
    my ($self,%args)=@_;
    my (
        @cmds,
        );
    
    $Trace->entry();

    $self->_clear_err();

    @cmds=qw(cifs_share volume exportruleset);

    foreach my $cmd(@cmds) {
        delete $self->{topology}{cluster}{$cmd};
    }

    $self->{cluster_topology}=undef;
    $self->{topology}=undef;
    $self->{client_vol_map}=undef;
    %Lastid_of=undef;
    %Cnt_of=undef;
    $Vs_Cnt=0 ; 
    $Vol_Cnt=0;

    $Trace->exit();
    
    return 1;  


}


################################################################
# Subroutine Name: get_client_map()
# Objective:       return client vol map either as raw or format
# Details:         See POD documentation at the end of this file
################################################################
sub get_client_map {
    my ($self,%args)=@_;
    my (
        $last,
        $length,
        $max,
        $header,
        @buffer,
        %last_of,
        %size_of,
       );
    
    $Trace->entry();
     
    $self->_clear_err();
    
    eval{
        ($self, %args) = validate_mixed(
            @_,
            [ { type => OBJECT} ],
            {
                format  =>  {type => SCALAR, default=>0},   
            }
        );
    }; 
    if ($@){
        $self->_set_err(msg=>$@,stack_depth=>1);
        return (wantarray()) ? () : [];
    } 
    
    
    if ($args{format}==0){
        return $self->{client_vol_map};        
    }    
    else {
        # Determine maximum length of field values
        foreach my $client_name (sort keys %{$self->{client_vol_map}}){
            foreach my $rec (@{$self->{client_vol_map}{$client_name}}){
                foreach my $field (qw(volume vserver aggregate)){
                    $last_of{$field}=length($rec->{volume}{$field});
                    $size_of{$field}=$last_of{$field}
                        if ($last_of{$field}>$size_of{$field});
                }
            }
        }
        
        # Create output display
        push @buffer,sprintf("%sMap Detail:\n\n".
            "%s%-12.12s\n%s%-12.12s"," "x2," "x4,"CLIENTS"," "x4,"="x12);
    
        foreach my $client_name (sort keys %{$self->{client_vol_map}}){
            push @buffer,
                sprintf("%s%-s:\n"," "x4,($client_name eq $last) ? "" : $client_name);
           
            push @buffer,sprintf(
                "\t%-*s  %-1.1s  %-8.8s  %4s %-*s %-12.12s %-*s  %12.12s\n".
                "\t%-*s  %-1.1s  %-8.8s  %4s %-*s %-12.12s %-*s  %12.12s",
                $size_of{volume},"VOLUME","S","SIZE(mb)","MNTS",
                $size_of{vserver},"VS","DATA_IP",
                $size_of{aggregate},"AGGREGATE","AGGR_AVL(mb)",
                $size_of{volume},"="x$size_of{volume},
                "="x1,"="x8,"="x4,
                $size_of{vserver},"="x$size_of{vserver},
                "="x12,
                $size_of{aggregate},"="x$size_of{aggregate},
                "="x12);
           
            foreach my $rec (@{$self->{client_vol_map}{$client_name}}){
                   
               push @buffer,
                   sprintf("\t%-*s  %-1.1s  %8d  %3d  %-*s ".
                           "%-12.12s %-*s  %8d",
                    $size_of{volume},$rec->{volume}{volume},
                    uc($rec->{volume}{style}),
                    $rec->{volume}{size},
                    $rec->{mnt_cnt},
                    $size_of{vserver},$rec->{volume}{vserver},
                    $rec->{data_ip},
                    $size_of{aggregate},$rec->{volume}{aggregate},
                    $rec->{volume}{aggr_availsize}
                    ); 
                                                
            }
            
            push @buffer,"\n"; 
            $last=$client_name;    
        }
        
        
        return (wantarray()) ? @buffer : [@buffer];
    }
    
    $Trace->exit();
}


################################################################
# Subroutine Name: get_vol_names()
# Objective:       return list of volumes
# Details:         See POD documentation at the end of this file
################################################################
sub get_vol_names { 
    my ($self)=@_;
    my (
        $topology,
        @volumes,
       );
    
    $Trace->entry();
    $topology=$self->{topology}{cluster};
    
    foreach my $vol_hsh (@{$topology->{volume}}){
        push @volumes,$vol_hsh->{volume};
    }
    
    $Trace->exit();
    
    return (wantarray()) ? @volumes : [@volumes];
}

###############################################################
# Subroutine Name: get_vol_of()
# Objective:       return hash of volumes
# Details:         See POD documentation at the end of this file
################################################################
sub get_vol_of {
    my ($self)=@_;
    my (
        $topology,
        %vol_of,
       );
    
    $Trace->entry();
    
    $topology=$self->{topology}{cluster};
    
    foreach my $vol_hsh (@{$topology->{volume}}){
        $vol_of{$vol_hsh->{volume}}=$vol_hsh;    
    }
    
    $Trace->exit();
     
    return \%vol_of;
}

###############################################################
# Subroutine Name: get_volumes_of()
# Objective:       return arrays of volume hashes
# Details:         See POD documentation at the end of this file
################################################################
sub get_volumes_of {
    my ($self)=@_;
    my (
        $topology,
       );
    
    $Trace->entry();
    
    $topology=$self->{topology}{cluster};
    
    $Trace->exit();
     
    return (wantarray()) ? @{$topology->{volume}} : $topology->{volume};
}

###############################################################
# P R I V A T E   M E T H O D S
###############################################################

################################################################
# Subroutine Name: _create_client_vol_map()
# Objective:       Create a hash map of client volumes to mount
# Details:         See POD documentation at the end of this file
################################################################
sub _create_client_vol_map {
    my ($self, $client_vol_map, $vs_of, $topology, $share_of)=@_;
    my (      
        $idx,
        $record,
        $vs_name,
       );
    $Trace->entry();

    $self->_log("creating client to volume map");
    my $i = 0;
    foreach my $client_hsh (@{$client_vol_map->{clients}}) {
        foreach my $vol_hsh (@{$topology->{volume}}){
            next if ($vol_hsh->{data_ip});
            my %vol_of = %{$vol_hsh}; # had to do this because I was having problems with the ref
            if ($client_hsh->{name} eq $vol_hsh->{host} ||
                $vol_hsh->{host} eq "*"){
                $vs_name=$vol_hsh->{vserver};
                $idx=$Cnt_of{$vs_name} % scalar(@{$vs_of->{$vs_name}});
                $record={
                    data_ip=>$vs_of->{$vs_name}[$idx]{address},
                    mnt_cnt=>$vol_hsh->{mnt_cnt},
                    node=>$vs_of->{$vs_name}[$idx]{'curr-node'},
                    volume=>\%vol_of,
                    share => $share_of->{cifs_share}[$i % @{$topology->{volume}}]->{'share-name'},
                    };

                push @{$self->{client_vol_map}{$client_hsh->{name}}},$record;
                $Cnt_of{$vs_name}++;
                $i++;
            }
        }
    }
    #print ntest::NTEST_STDOUT "DMP:".Dumper($self->{client_vol_map});
    $Trace->exit();
    return $self->{client_vol_map};  
}

################################################################
# Subroutine Name: _create_export_ruleset()
# Objective:       Generate ruleset definition for each vserver  
# Details:         See POD documentation at the end of this file
################################################################s
sub _create_export_ruleset{
    my ($self,%args)=@_;
    my (
        %exports_of,
        %seen_of,
       );
    $Trace->entry();
    
    foreach my $vs_of (@{$args{vss_of}}){
        next if ($vs_of->{type} !~ /^cluster$|^data$/);
        next if ($seen_of{$vs_of->{vserver}}); 
        push(@{$exports_of{exportruleset}}, {
            vserver => $vs_of->{vserver},
            rulesetname => $args{rule_set},
        }); 
        
        push(@{$exports_of{exportruleset_rules}}, {
            ruleindex => 1,
            vserver => $vs_of->{vserver},
            rulesetname => $args{rule_set},
            anon => 0,
            protocol => "any",
            clientmatch => "0.0.0.0/0",
            rorule => "any",
            rwrule => "any",
            superuser => "any",
        });
        $seen_of{$vs_of->{vserver}}++;
    }
    
    $Trace->exit();
    
    return \%exports_of;  
}

################################################################
# Subroutine Name: _create_client_vol_layout()
# Objective:       Generate array ref of vol hashes for creation 
# Details:         See POD documentation at the end of this file
################################################################s
sub _create_client_vol_layout {
    my ($self,%args)=@_;
    my (
        $clients_href,
        $dflt_name,
        $instance,
        $files,
        $size,
        $vol_name,
        $vol_size,
        $vol_unit,
        %vols_of,
       );

    $Trace->entry();

    $self->_log("defining test topology");

    $clients_href=$args{client_of}{clients};

    # determine default prefix for new vol resources
    if (defined $args{prefix}){
        $dflt_name="p_".$args{prefix};
    }
    else {
        $dflt_name = "p_".basename($ENV{UTESTNAME}) if (defined($ENV{UTESTNAME}));
        $dflt_name =~ s/.thpl//;
    }
    
    $instance=($args{instance}) ? "_".$args{instance} : "";

    if ($args{vol_layout} eq PER_INSTANCE) {
      foreach my $client_href (@$clients_href) {    
        foreach my $cnt (1..$args{vol_cnt}){
            $Lastid_of{$client_href->{name}}+=1;
            $vol_name =
                $dflt_name."_".
                $self->_normalize_name($client_href->{name}).                  
                    "_v".$Lastid_of{$client_href->{name}}.
                    $instance;

            my $vol_hptr = {
                volume => $vol_name,
                size => $self->_size_to_mbytes($args{vol_size})."mb",
                delete_volume => 1,
                mnt_cnt=>1,
                host=>$client_href->{name},
                style=>$args{vol_style}
            };
            if ($args{vol_index_dir_enabled} =~ qr/true/i) {
                $vol_hptr->{'index-dir-enabled'} = $args{vol_index_dir_enabled};
            }
            push(@{$vols_of{volume}},$vol_hptr);
	}  
      } 
    }
    elsif ($args{vol_layout} eq SINGLE) {           
	foreach my $cnt (1..$args{vol_cnt}){  
            $files=0;
            $size=0;
            ($vol_size,$vol_unit)=$args{vol_size} =~ /(\d{0,})\s{0,}(\w+)/;
            
            foreach my $client_href (@$clients_href) {       
                $size += $vol_size;
                $files += $args{vol_files};
            }
            $Lastid_of{single}+=1;
            $vol_name =$dflt_name."_shared_"."v".$Lastid_of{single}.$instance;
                
            my $vol_hptr = {
                volume => $vol_name,
                size => $self->_size_to_mbytes($size.$vol_unit)."mb",
                files => $files,
                delete_volume => 1,
                mnt_cnt=>scalar(@$clients_href),
                host=>"*",
                style=>$args{vol_style}
            };
            if ($args{vol_index_dir_enabled} =~ qr/true/i) {
                $vol_hptr->{'index-dir-enabled'} = $args{vol_index_dir_enabled};
            }
            push(@{$vols_of{volume}},$vol_hptr);
        }
    }

    $Trace->exit();
    
    return \%vols_of;
}

################################################################
# Subroutine Name: _create_client_share_layout()
# Objective:       Generate array ref of cifs share hashes for creation 
################################################################s
sub _create_client_share_layout {
    $Trace->entry();

    my ($self,%args)=@_;
    my (%shares_of);

    $self->_log("defining CIFS topology");

    my $i = 0;
    foreach my $vol_of (@{$args{vols_of}->{volume}})
    {
        my $share_name = uc($vol_of->{volume});
        if(length($vol_of->{volume}) > 15)
        {
            $share_name = uc(substr($vol_of->{volume}, 0, 13) . "_" . $i);
        }
        $i++;
        my $share_of = {'share-name' => $share_name,
                        vserver      => $vol_of->{vserver},
                        path         => $vol_of->{'junction-path'}};
        push(@{$shares_of{cifs_share}}, $share_of);
    }

    $Trace->exit();
    
    return \%shares_of;
}

################################################################
# Subroutine Name: _get_aggregates()
# Objective:       Return hash of aggregate records
# Details:         See POD documentation at the end of this file
################################################################
sub _get_aggregates {
    my ($self,%args) = @_;
    my (
        $aggrs_use,
        $aggrs_ignore,
        $min_size,
        @aggrs_of,
        @aggregates_of,
       );

    $Trace->entry();

    $self->_log("retrieving aggregates"); 
    $min_size="200";

    eval {

        #@aggrs_of = $Server->storage_aggregate_show(
        #    state=>"online",
        #    avail_size=>$min_size,
        #    voltype=>$args{vol_style},
        #    );
        #
        @aggrs_of = $Server->pick_aggregates(
            state=>"online",
            avail_size=>$min_size,
            voltype=>$args{vol_style},
            );

        ($aggrs_use,$aggrs_ignore)=
            $self->_use_or_ignore($args{aggregates});

        # skip if private | aggr user requested | requested by user to skip
        foreach my $aggr_of (@aggrs_of) {
            next if (defined $args{aggr_type} &&
                     $args{aggr_type} != $aggr_of->{aggrtype});
            next if ($aggr_of->{aggregate} =~ /^p_/); 
            next if (@$aggrs_use &&
                     !grep($aggr_of->{aggregate} =~ /$_/,@$aggrs_use));   
            next if (@$aggrs_ignore &&
                     grep($aggr_of->{aggregate} =~ /$_/,@$aggrs_ignore));       
            push @aggregates_of, $self->_convert_aggregate_sizes($aggr_of);
        }
    };
    if ($@){
        if ("$@"=~/Could not find any aggregates/i){
            push my @msg, "Unable to find any aggregates that matched your ".
                          "requirements.";

            foreach my $aggr_of (@{$self->{cluster_topology}{cluster}{aggregate}}){
                push @aggregates_of, $self->_convert_aggregate_sizes($aggr_of);
            }

            push @msg, $self->_get_aggregate_sizes([@aggregates_of]); 

            if ($args{vol_style} eq STRIPED){
                push @msg,"\n",
                 "All node participants for a coral aggregate must be supplied",
                 "to the FILER parameter for it to be used for striped volume",
                 "creation.";
            } 

            $self->_set_err(msg=>[@msg],stack_depth=>3);
        }
        else {
            $self->_set_err(msg=>$@,stack_depth=>1);
        }
        return [];
    }

    $Trace->exit();

    return [@aggregates_of];

}
################################################################
# Subroutine Name: _convert_aggregate_sizes()
# Objective:       Convert Aggregate sizes
# Details:         See POD documentation at the end of this file
################################################################
sub _convert_aggregate_sizes {
    my ($self,$aggr_of)=@_;
    
    ## TODO:  Need code to determine Tricky release. Since aggr
    ## conversion is different whether Tricky.1 or Tricky.2

    if (not $aggr_of->{size_to_mbytes}){
        #$aggr_of->{availsize}=$self->_size_to_mbytes($aggr_of->{availsize});
        $aggr_of->{size}=$self->_size_to_mbytes($aggr_of->{size});
        $aggr_of->{usedsize}=$self->_size_to_mbytes($aggr_of->{usedsize});
        $aggr_of->{size_to_mbytes}="converted by Config::Cluster.pm";
    }
    return $aggr_of;
}


################################################################
# Subroutine Name: _get_aggregate_sizes()
# Objective:       Return hash of aggregate records
# Details:         See POD documentation at the end of this file
################################################################
sub _get_aggregate_sizes {
    my ($self,$aggr_ref)=@_;
    my (
        @buffer,
       );

    $Trace->entry();

    $self->_log("retrieving aggregate sizes");
    push @buffer,sprintf("\tAggr Detail:\n".
        "\t%-20.20s  %-7.7s  %-7.7s  %-12.12s  %-25.25s\n".
        "\t%-20.20s  %-7.7s  %-7.7s  %-12.12s  %-25.25s",
        "AGGREGATE","TYPE","RAID","AVL_SIZE(mb)","NODES",
        "="x20,"="x7,"="x7,"="x12,"="x25);

    foreach my $aggr_hsh (@{$aggr_ref}){
        push @buffer,
            sprintf("\t%-20.20s  %-7.7s  %-7.7s  %12d  %-s",  
                $aggr_hsh->{aggregate},
                $aggr_hsh->{voltype},
                $aggr_hsh->{raidtype},
                $self->_size_to_mbytes($aggr_hsh->{availsize}),
                (ref($aggr_hsh->{nodes}) eq "ARRAY")
                    ? join(", ",@{$aggr_hsh->{nodes}})
                    : $aggr_hsh->{nodes},         
               );                     
    }
    
    $Trace->exit();
    
    return @buffer;
}

################################################################
# Subroutine Name: _get_aggr_striped_limit
# Objective:       return hash of aggregate min size limits
# Details:         See POD documentation at the end of this file
################################################################
sub _get_aggr_striped_limit {
    my ($self, $stripe_count,@aggrs_hsh)=@_;
    my (
        %aggr_limit_of,
       );
    $Trace->entry();

    $self->_log("retrieving aggr min size limits");
    foreach my $aggr_hsh  (@aggrs_hsh) {
    
        # Keep the minsize around to check against size requests
        # If a stripe count is specified, calculate the minsize from it.  
        $aggr_limit_of{$aggr_hsh->{aggregate}}{minperm}= $stripe_count
            ? $stripe_count * (1.2 * 1024)
            : $Server->find_min_max_volume_size($aggr_hsh->{aggregate}, 'minperm');
                 
        if ($aggr_limit_of{$aggr_hsh->{aggregate}}{minperm} =~  /^(\d+\.?\d*)/) {
            $aggr_limit_of{$aggr_hsh->{aggregate}}{minperm} = $1;
        } else {
            #$Trace->fatal(result=>"CONFIG", msg=>"Couldn't determine minimum size" );
        }   
    }
    
    $Trace->exit();
    return \%aggr_limit_of;
}

################################################################
# Subroutine Name: _get_cluster_topology()
# Objective:       Return cluster topology of requested cmd nodes
# Details:         See POD documentation at the end of this file
################################################################
sub _get_cluster_topology {
    my ($self,%args) = @_;
    my (
        $topology,
        @cmd_nodes,
       );
    $Trace->entry();
    
    $self->_log("retrieving cluster topology"); 
    # Define the base list of command nodes to discover in the cluster
    @cmd_nodes = qw(aggregate vserver network_interfaces);

    # Add the cifs nodes to the list if the user is running a CIFS test
    push(@cmd_nodes, ("cifs_server", "cifs_share", "cifs_domain",
                      "kerberos","cifs_superuser")) if ($args{cifs});
    
    eval {
        $topology = $Server->discover_topology(cmd_nodes=>\@cmd_nodes);
    };
    if ($@){
        $self->_set_err(msg=>$@);
        return {};
    }
    
    $Trace->exit();
    
    return $topology;
}



################################################################
# Subroutine Name: _get_vifs()
# Objective:       return array ref list containing vifs 
# Details:         See POD documentation at the end of this file
################################################################
sub _get_vifs {
    my ($self,%args) = @_;
    my (
        $discovered,
        $data_vifs_use,
        $data_vifs_ignore,
        @data_vifs_of,
       );

    $Trace->entry();

    $self->_log("retrieving vifs"); 
    $discovered=$self->{cluster_topology}{cluster};

    ($data_vifs_use,$data_vifs_ignore)=
        $self->_use_or_ignore($args{data_vifs});

    foreach my $vif_of (@{$discovered->{network_interfaces}}) {       
        $self->_log("Checking vif: ".$vif_of->{vif});
        # skip if private | address not in interface | requested by user to skip
	next if ($vif_of->{vif} =~ /^p_/);
	next if (@$data_vifs_use && !grep($vif_of->{address} =~ /$_/,@$data_vifs_use));   
        next if (grep($vif_of->{address} =~ /$_/,@$data_vifs_ignore));       
        my $data_protocol;
        if ('ARRAY' eq ref $vif_of->{'data-protocol'}) {
            $data_protocol = join ',', @{$vif_of->{'data-protocol'}};
        } else {
            $data_protocol = $vif_of->{'data-protocol'};
        } 
        $self->_log("vif: ".$vif_of->{vif}." data-protocol: ".$data_protocol);

        push(@data_vifs_of, $vif_of)
            if ($vif_of->{role} eq "data" && $vif_of->{"status-admin"} eq "up" && $data_protocol =~ /nfs/ && $data_protocol =~ /cifs/);                                     
    }

    # if interfaces were specified, verify that they were all found
    if ($args{data_vifs}) {
	my @failures;
	foreach my $data_vif_use (@$data_vifs_use) {
	    push(@failures, $data_vif_use)
                if (!grep($_->{address} =~ /$data_vif_use/, @data_vifs_of)); 
	}
	if (@failures) {
            my $message="INVALID_DATA_INTERFACE - ".
                        "Did not find specified data interfaces\n".
                         join("\n", @failures);
            $self->_set_err(msg=>$message);
            return [];
	}
    }

    # catch the case where we couldn't find a data_interfaces
    if (!scalar(@data_vifs_of)) {
        my $message="NO_DATA_VIFS_FOUND - Could not find data interfaces";
        $self->_set_err(msg=>$message);
        return ([]);
    }

    $Trace->exit();

    return [@data_vifs_of];

}

################################################################
# Subroutine Name: _get_vservers()
# Objective:       return array ref list containing vifs and
#                  vserver
# Details:         See POD documentation at the end of this file
################################################################
sub _get_vservers {
    my ($self,%args) = @_;
    my (
        $discovered,
        @vss_of,
        $vservers_use,
        $vservers_ignore,
        @vservers_of,
       );
    
    $Trace->entry();
       
    $self->_log("retrieving vservers"); 

    ($vservers_use,$vservers_ignore)=
        $self->_use_or_ignore($args{vservers});


    # Retrieve existing vservers from cluster
    $discovered=$self->{cluster_topology}{cluster};
    @vss_of = @{$discovered->{vserver}};
   
        
    foreach my $vs_of (@vss_of) {       
        # skip if private | address not in interface | requested by user to skip
	next if ($vs_of->{vserver} =~ /^p_/);
	next if (@$vservers_use && !grep($vs_of->{vserver} =~ /$_/,@$vservers_use));   
        next if (grep($vs_of->{vserver} =~ /$_/,@$vservers_ignore));       
	    
        push @vservers_of, $vs_of;                                   
    }   
        
    # catch the case where we couldn't find a virtual server 
    if (!@vservers_of) {
        my $message="NO_VSERVER_FOUND - Could not find virtual server ".$args{vservers};
        $self->_set_err(msg=>$message);
        return [];
    }      
      
    $Trace->exit();
    
    return [@vservers_of];
      
}

################################################################
# Subroutine Name: _create_vserver_vif_map()
# Objective:       return array ref list containing vifs and
#                  vserver
# Details:         See POD documentation at the end of this file
################################################################
sub _create_vserver_vif_map {
    my ($self,%args) = @_;
    my (
        %vs_of,
       );
    
    $Trace->entry();
       
    $self->_log("creating vserver data interface map"); 
    
    # Make sure user supplier vifs only select
    # from the associated vserver
    foreach my $vif_of (@{$args{vifs_of}}) {
        my $name = $vif_of->{server};
        next if (!grep($name eq $_->{vserver},@{$args{vss_of}}));
        push @{$vs_of{$vif_of->{server}}}, $vif_of;
    } 
    
     # catch the case where we couldn't find a virtual server 
    if (!keys %vs_of) {
        my $message="NO_VSERVER_VIF_MAP - ".
                    "Unable to create map with available resources";
        $self->_set_err(msg=>$message);
        return [];
    }
    
    $Trace->exit();
    
    return \%vs_of;
      
}

################################################################
# Subroutine Name: _init_client_hash()
# Objective:       create a hash of clients containing hostname
#                  and hosttype
# Details:         See POD documentation at the end of this file
################################################################
sub _init_client_hash {
    my ($self,@names)=@_;   
    my (
        $host,
        $hosttype,
        %id_of,        
        %name_of,
       );
    
    $Trace->entry();
       
    $self->_log("retrieving client host types");   
    foreach my $name (sort @names) {
        # remove domainname
        $name =~s/[\?\.].*//;
        
        # replace localhost name with real name
	chomp($name = `hostname`) if ($name =~ "localhost");
        
        # Determine client hosttype
        if (not $name_of{$name}){
            eval {
                $host=Tharn::host($name);
                $hosttype=($host->hosttype()=~/linux/i)
                    ? "linux"
                    : $host->hosttype();
            };
            if ($@){
                $self->_set_err(msg=>$@,stack_depth=>1);  
                return \{};
            }
        }
             
        # use unique id for each duplicate client name
	push @{$name_of{clients}}, {
                            name=>$name,
                            hosttype=>$hosttype,
                            };
                                            	 
    }
    
    $Trace->exit();
    
    return \%name_of;
}
################################################################
# Subroutine Name: _log()
# Objective:       Convert value containing dashes with underscore
# Details:         See POD documentation at the end of this file
################################################################
sub _log {
    my ($self,$msg)=@_;

    print ntest::NTEST_STDOUT "\t- ".$msg."\n"
        if ($ENV{LOG}==1); 
    $Log->trace($msg);
}

################################################################
# Subroutine Name: _normalize_name()
# Objective:       Convert value containing dashes with underscore
# Details:         See POD documentation at the end of this file
################################################################
sub _normalize_name {
    my ($self,$value)=@_;
    $value =~ s/-/_/g;
    return $value
}

################################################################
# Subroutine Name: _size_to_bytes()
# Objective:       convert size strings to bytes.
# Details:         See POD documentation at the end of this file
################################################################
sub _size_to_bytes {
    my ($self,$size)=@_;
    my (
        $bytes,
        $nbr,
        $unit
    );
    
   ### If no units then byte 
   $size .= "b" if ($size=~/^\d+$/);
   
   ### Separate number and units
   ($nbr,$unit)=$size=~/^(\d+\.?\d*)\s{0,}(\w+)/;
   return $nbr if ($unit=~/^b$/i);

   if ($unit =~/^k/i) {
    $bytes = $nbr * 1024;
   }
   elsif ($unit=~/^m/i){
    $bytes = $nbr * 1024**2;
   }
   elsif ($unit=~/^g/i){
    $bytes = $nbr * 1024**3;
   }
   elsif ($unit=~/^t/i){
    $bytes = $nbr * 1024**4;
   }
   return int($bytes);
}

################################################################
# Subroutine Name: _size_to_mbytes()
# Objective:       convert size strings to mbytes.
# Details:         See POD documentation at the end of this file
################################################################
sub _size_to_mbytes {
    my ($self,$size)=@_;
    my (
        $mbytes,
        $nbr,
        $unit
    );
    
   ### If no units then byte 
   $size .= "b" if ($size=~/^\d+$/);
   
   ### Separate number and units
   ($nbr,$unit)=$size=~/^(\d+\.?\d*)\s{0,}(\w+)/;
   return $nbr if ($unit=~/m/i);

   if ($unit =~/^b/i) {
    $mbytes = $nbr/1024**2;
   }
   elsif ($unit =~/^k/i) {
    $mbytes = $nbr/1024;
   }
   elsif ($unit=~/^g/i){
    $mbytes = $nbr * 1024;
   }
   elsif ($unit=~/^t/i){
    $mbytes = $nbr * 1024**2;
   }
   return int($mbytes);
}

################################################################
# Subroutine Name: _update_volume_attributes()
# Objective:       update volume hashes 
# Details:         See POD documentation at the end of this file
################################################################
sub _update_volume_attributes {
    my ($self,%args) = @_;
    my (
        $aggr_hsh,
        $aggr_limit_of,
        $inode_space,
        $no_aggr_cnt,
        $vol_size,
        $vols_of,
       );

    $Trace->entry();

    $self->_log("updating volume data");

    $vols_of=$args{vols_of};

    # Create a hash to store the minimum volume size allowed by the aggregates
    # Or calculate the minimum if option VOL_STRIPE_COUNT is provided.
    $aggr_limit_of=
        $self->_get_aggr_striped_limit($args{'stripe-count'},@{$args{aggrs_of}})
            if ($args{vol_style} eq STRIPED);

    foreach my $vol_hsh (@{$vols_of->{volume}}) {
        next if ($vol_hsh->{comment}=~/Config::Cluster/i);

        # name
        $vol_hsh->{volume} = $self->_normalize_name($vol_hsh->{volume});

        # size  
        $vol_size = $self->_size_to_mbytes($vol_hsh->{size})
            if (defined($vol_hsh->{size}));           
        $vol_hsh->{size} = "300MB" if ($vol_size < 300);

        # type
        if ($args{vol_style} eq STRIPED) {
            if ( defined ($args{vol_stripe_width}) ) {
                $vol_hsh->{'stripe-width'} = $args{vol_stripe_width};
            }		
            if( defined ($args{vol_stripe_count}) ) {
                $vol_hsh->{'stripe-count'} = $args{vol_stripe_count};
            }
        }

        # maxdir size
        if (!defined($vol_hsh->{'maxdir-size'}) && $args{vol_maxdir}) {
            $vol_hsh->{'maxdir-size'} =
                $self->_size_to_mbytes($args{vol_maxdir})."MB";
        }

        # file maximum
        $vol_hsh->{files} = $args{vol_files} if (!defined($vol_hsh->{files}));

        if (defined($vol_hsh->{files})) {
            # each inode needs 4K so make sure we set the
            # volume size large enough
            $inode_space =
                $self->_size_to_mbytes($vol_hsh->{files} * 4096);
            if ($inode_space > $vol_size) {
                $vol_size = $inode_space;
                $vol_hsh->{size} = $inode_space."MB";
            }
        }

        # get aggregates that satisfy volume size rqmt
        my @usable_aggrs = 
            grep( { $self->_size_to_mbytes($_->{availsize}) >= $vol_size} @{$args{aggrs_of}} );

        # insure volume size is greater than the aggr minimum size limit
        if ($args{vol_style} eq STRIPED) {

            @usable_aggrs = grep( { $vol_hsh->{availsize} >=
                    $aggr_limit_of->{$_->{aggregate}}{minsize}}
                                 @usable_aggrs);
        }

        # return failure if no aggregates
        if (!scalar(@usable_aggrs)) {
            $vol_hsh->{aggregate}="NO AGGR FOUND";
            $vol_hsh->{aggr_availsize}=0;
            $no_aggr_cnt++;
        }
        else {
            # pick aggregates in a round-robin
            $aggr_hsh = $usable_aggrs[$Vol_Cnt++ % (scalar @usable_aggrs)];

            # adjust striped volume sizes that are under the aggr minimum
            # allowed size
            if ($args{vol_style} eq STRIPED ) {

                my $tmp = $vol_hsh->{'stripe-count'}
                        ? ($vol_hsh->{'stripe-count'}) * (1.2 * 1024)
                        : $aggr_limit_of->{$aggr_hsh->{aggregate}}{minperm};

                my ($minsize) = $tmp =~ /^(\d+\.?\d*)/; 
                my $reqsize = $self->_size_to_mbytes($vol_hsh->{'size'});
                if ($reqsize < $minsize) {
                    # increase the size to avoid filer conversion problems
                    $minsize = ($minsize + 100);
                    $vol_hsh->{'size'} = $minsize."mb";
                    $vol_size = $minsize;
                }
            }

            # volume aggregate name
            $vol_hsh->{aggregate} = $aggr_hsh->{aggregate}
                if (!$vol_hsh->{aggregate});

            # subtract vol size from aggr available size
            $vol_hsh->{aggr_availsize}=
                $self->_size_to_mbytes($aggr_hsh->{availsize})."mb";
            $aggr_hsh->{availsize} -= $self->_size_to_bytes("$vol_size"."mb");     
        }

        # vserver name
        $vol_hsh->{vserver} = (keys %{$args{vs_of}})[$Vs_Cnt++ %(scalar keys %{$args{vs_of}})];

        # security_style
        if ( !$vol_hsh->{'security-style'} ) {
            $vol_hsh->{'security-style'} = $args{vol_security_style};
        }

        # user/group 
        if ($args{vol_owner} eq 'self') {
            $vol_hsh->{user} = $ENV{LOGNAME};
            (undef,undef,undef,$vol_hsh->{group}, undef, undef,
             undef(), undef(), undef()) = getpwnam($ENV{LOGNAME});
        }

        # unix permissions
        if (!defined($vol_hsh->{'unix-permissions'}) && $args{vol_perms} &&
            $vol_hsh->{'security-style'} !~ /^ntfs$/i) {
            $vol_hsh->{'unix-permissions'} =$args{vol_perms};
        }

        # stripe format
        if (!defined($vol_hsh->{'stripe-format'}) && $args{vol_format}) {
            $vol_hsh->{'stripe-format'} =$args{vol_format};
        }

        # junction-path 
        $vol_hsh->{'junction-path'} = (defined $vol_hsh->{'junction-path'})
            ? $vol_hsh->{'junction-path'}
            : "/$vol_hsh->{volume}";

        # export rule
        $vol_hsh->{ruleset} = $args{rule_set} if (!$vol_hsh->{ruleset});

        # Indicate volume created by Config::Cluster
        $vol_hsh->{comment} =~ s/['"]//g if ($vol_hsh->{comment});
        $vol_hsh->{comment}.='"(created using Config::Cluster)"';
    }

    # remove undefined keys
    foreach my $vol_hsh (@{$vols_of->{volume}}) {
        foreach my $field (keys %{$vol_hsh}){
            delete $vol_hsh->{$field} if (not defined $vol_hsh->{$field});
        }
    }

    # Return message
    if ($no_aggr_cnt){
        $self->_set_err(
            msg=>"NO_AGGR_FOUND - Unable to fullfill test definition",
            on_err=>'rtn',
            stack_trace=>1,
            stack_depth=>7,
            );
    }

    $Trace->exit();

    return $vols_of;
}

################################################################
# Subroutine Name: _use_or_ignore()
# Objective:       filter value on whether to use or ignore
# Details:         See POD documentation at the end of this file
################################################################
sub _use_or_ignore {
    my ($self,$value)=@_;
    my (
	@use,
	@ignore,
	@values,
	);

    if (ref($value) eq "ARRAY"){
	@values=@$value;	
    }
    else {
	@values=split(',',$value);
    }

    foreach my $value(@values){
	if ($value=~/^(!|~)/){
            $value=~s/(!|~)//g;
	    push @ignore, $value;
	}
	elsif ($value=~m!^/(.+)/$!) {
            $value=~s/\///g;
	    push @use, $value;
	}
        else {
            push @use, '^'.$value.'$';
        }
    }

    return ([@use],[@ignore]);
}

1;

############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

Config::Cluster.pm

=head1 Author

dl-afd@netapp.com

=head1 Description

Config::Cluster.pm provides an API interface for defining, creating, and deleting
a cluster topology 

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Utils::Error.pm>

=head1 Exported Constants

raid_type:  B<RAID_DP>, B<RAID4>

vol_layout: B<PER_INSTANCE>, B<SINGLE>

vol_style:   B<FLEX>, B<STRIPED>

vol_format:  B<DATA>, B<FULL>

=head1 Public Methods

=head2 B<new(server, I<err_on>)>

Creates a new Config::Cluster object. 

=over 4

=item Inputs

=over 4

=item * B<server>

Server object for communicating with cluster. 

=item * I<err_on>  (default: die)   

Type of error handling to use for handling configuration errors. By default, 'die'  
will be used to throw errors and require the function caller to catch them using
an eval block.  Specifying err_on=>'rtn' will require the caller after the function 
returns to make a inquiry on the Config::Cluster object whether an error occurred.

=back

=item Outputs

Returns a Config::Cluster object.

=item Examples

    # Example 1: Create a cfg object which dies on error.
    eval {
	$cfg_obj = Config::Cluster->new(
	    server=>$server_obj,
	    on_err=>'die'
	);
    };
    if ($@) {
	$Test->nlog($@);
	return $TCD:FAIL;
    }	


    # Example 2: Create a cfg object which returns on error.
    $cfg_obj = Config::Cluster->new(
	server=>$server_obj,
	on_err=>'rtn'
    );
    if ($cfg_obj->error) {
	$Test->nlog($cfg_obj->errmsg());
	return $TCD:FAIL;
    }	


=back

=head2 B<define(%args)>

This method defines the configuration definition that will be created by the Config::Cluster
create() method.  

It allows you create a NFS definition that includes the number of volumes, volume attributes,
and client-to-volume layout which can be distributed across the existing aggregates, vservers,
and data interfaces in the cluster.

Multiple calls to define() will append the new definition to the previous definition.    

=over 4

=item Inputs

=over 4

B<Required:>

=over 4

=item * B<clients> 

An arrayRef of client names to use for defining the client to volume relationship. 

    clients=[@clients];

=item * B<vol_cnt> 

The number of volumes (scalar value) to create B<for each client> name provided by
the parameter 'clients=>[@clients]'.  

    vol_cnt=>10

=back 

B<Optional:>

=over 4

=item * I<aggregates> 

Lists of aggregate names (arrayref or scalar with comma separated values) to select from for volume
creation.  By default all aggregate values will become part of the selection pool. Regular expressions 
can also be used to identify the aggregate selction.  To exclude an aggregate from use,  prefix the 
aggregate value with an '!' or '~'.   

            # Only use the following aggregates for volume selection 
            aggregates=>"nqa-f3050-7_aggr1,nqa-f3050-8_aggr2"

	    # Only use aggregates on node nqa-f3050-7 and exlude aggr1
	    aggregates=>"/f3050-7/,!nqa-f3050-7_aggr1";

            # Ignore the following aggregates for volume selection
            aggregates=>"!nqa-f3050-7_aggr1,!nqa-f3050-8_aggr2"


=item * I<data_vifs>

Lists of data vif addresses (arrayref or scalar with comma separated values) to select from for a volume
interface.  By default all data vif address values will become part of the selection pool.  Regular expressions 
can also be used to identify the data vifs selection.  To exclude an data vif from use,  prefix the 
data vif value with an '!' or '~'.   

            # Only use the following data_vifs
            data_vifs=>"172.16.7.1,172.16.7.2";

	    # Only use data_vifs *.*.7.* and excluding 7.1
	    data_vifs=>"/^172.16.7/,~172.16.7.1";

            # Ignore the data_vifs
            data_vifs=>"!172.16.8.1,~172.16.8.2";

=item * I<instance> (default: 1) 

A value used to define a unique instance (scalar value) of the configuration resources created. The 
instance value is appended to each named resource created by the test. By specifying a
different instance value for each test run, multiple instances of the same test can be run in parallel 
using the same equipment resources.  The value specified must be > 0 or an error will occur. 

The following format will be used for creating new volume and export rule resource entries on the node.

=begin html
<table border=1>
<th bgcolor=#F4F4F4>vol_layout=></th><th bgcolor=#F4F4F4>format</th><th bgcolor=#F4F4F4>example</th>
<tr>
<td>per_instance</td>
<td><b>p_</b><i>[scriptname|prefix]</i><b>_</b><i>[client]</i><b>_v</b><i>[cnt]</i><b>_</b><i>[instance]</></td>
<td>p_mytest_nanex86_v1_1</td>
<tr>
<td>single</td>
<td><b>p_</b><i>[scriptname|prefix]</i><b>_shared_v</b><i>[cnt]</i><b>_</b><i>[instance]</></td>
<td>p_mytest_shared_v1_1</td>
</table>
<br>
Definitions:
<ul>
    <li> <b>p_</b>:           denotes the named resource is private
    <li> <i>[scriptname|prefix]</i>: name of test script w/o .thpl or user specified prefix
    <li> <i>[client]</i>:     name of client that will be mounting volume
    <li> <b>[cnt]</b>:        incrementing number for volumes created on client (base 1)
    <li> <b>shared</b>:       volume is mounted by multiple clients 
</ul>

=end html

=item * I<mnt_mode> (default: 777)

The client permissions (scalar value) to be assigned to the mounted volume directory. 

=item * I<mnt_opts> 

The mount options (scalar value) to be supplied during the client mount operation of the volume.

=item * I<prefix> (default: p_I<ScriptName> (w/o '.thpl'))

Prefix to use for all created resources by test.  All resources are
automatically marked as private by pre-appending a 'p_' to the prefix value.

=item * I<timeout> (default: 300) 

The amount of time in seconds (scalar value) to wait for the creation of a resource to complete. An 
error will result once the timeout has been exceeded. 

=item * I<vol_files>

The maximum number of files that can be created on the volume.

=item * I<vol_format>

The striped volume format to use for the volume.  Valid values are DATA|FULL.

=item * I<vol_group>

An existing group\'s integer ID or character-string name (scalar value).  This value
defines the new volume\'s primary group membership 

=item * I<vol_index_dir_enabled> 

Turn on sidewinder indexed directories. This will cause test mode to be set 
and is not currently compatible with promoted builds. 

Valid values are true|false

=item * I<vol_layout> (default: 'per_instance')

The volume layout to create and map to the client names provided.  A scalar value of 
'per_instance' creates and map 1 volume for each client.  Wheras a value of 'single' 
creates 1 volume for all clients to mount.  If the user specifies a volume count > 1 
(vol_cnt>1) then the layout will be repeated for each additional set of volumes.

The size of the volume for a 'single' volume layout will be increased in size by the
number of clients that will be mounting it.  For example, if vol_size definition is 
2g and 5 clients are specified, a 10g volume will be created.  

=item * I<vol_maxdir>

The maximum number of directories (scalar value) that can be created on a volume.

=item * I<vol_owner>

An existing user\'s integer ID or character-string name (scalar value). This value 
defines the new volume\'s primary owner.

=item * I<vol_perms> (default: '777')

Unix permissions to set on the volume. 

=item * I<vol_security_style> 

The security-style to use for accessing the volume.  Legal values are unix|mixed|ntfs.

=item * I<vol_size> (default: '1gb')

The size of the volume to create.  If the volume layout is specified as 'single' the size 
will be multiplied by the number of clients (2g vol_szie* 5 clients = 10g volume)

=item * I<vol_stripe_count>

The number of flex vol participants to create across the set of member aggregates   
in a round-robin fashion.  The volume size will automatically be adjusted by a 
multiple of 1.2g per volume particpant

=item * I<vol_stripe_width>

The stripe width size to use for each flex vol participant in the stripe.

=item * I<vol_style>  (default: 'flex')

The style of volume to create.  By default a 'flex' volume will be created.  However, a
value of 'striped' will look for coral aggregates to use for volume creation.  Please
note striped volumes require all node participants in the aggregate to be specified
on the FILER command line, otherwise the coral aggregate will be ignored.

=item * I<vservers>

Lists of data vservers (arrayref or scalar with comma separated values) to select from.
By default all vserver values will become part of the selection pool.  Regular expressions 
can also be used to identify the vserver selection.  To exclude an vserver from use, 
prefix the data vif value with an '!' or '~'.   

            # Only use the following vservers
            vservers=>"vs1,vs2";

	    # Only use vserver starting with vs
	    vservers=>"/^vs/";

            # Ignore the vserver
            vservers=>"!vs2";

=item * I<require_fresh>

Require fresh volumes. Fail if the volumes already exist.

=back

=item Outputs

Returns 1 on success and 0 on failure.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
argument 'err_on' as defined during object instantiation.

If the err_on type is to 'rtn' during object instantiation, 
otherwise an error is thrown to be caught by an eval block around the calling function in the test script.
On error, an error message is returned identifying the specific issue.

If there are is not enough space on the existing aggregates to satisfy the nfs definition an error 
will occur.  A detailed explanation of where it failed will be retruned to the caller. An example of 
the output is listed below.

=back 

=begin html

<pre><small>
ERROR: NO_AGGR_FOUND - Unable to fullfill test definition
at /u/knopp/p4/afd/lib/Config/Cluster.pm::Config::Cluster::_update_volume_data: 555
<p>
Map Detail:
CLIENT        VOLUME                     STYLE    SIZE(mb)  MNTS VS      DATA_IP      AGGREGATE             AGGR_AVL(mb)
============  =========================  =======  ========  ==== ======= ============ ====================  ============
naneqa-2k6    p_NFS_naneqa_2k6_v1_1      flex        51200    1  vs0     172.16.7.1    nqa_f3050_7_aggr1       115241
              p_NFS_naneqa_2k6_v2_1      flex        51200    1  vs1     172.16.7.3    nqa_f3050_8_aggr1       116262
              p_NFS_naneqa_2k6_v3_1      flex        51200    1  vs0     172.16.7.2    nqa_f3050_8_aggr3       116262
              p_NFS_naneqa_2k6_v4_1      flex        51200    1  vs1     172.16.7.4    nqa_f3050_7_aggr1        64041
              p_NFS_naneqa_2k6_v5_1      flex        51200    1  vs0     172.16.8.1    nqa_f3050_8_aggr1        65062
naneqa-2k8    p_NFS_naneqa_2k8_v1_1      flex        51200    1  vs1     172.16.8.3    nqa_f3050_8_aggr3        65062
              p_NFS_naneqa_2k8_v2_1      flex        51200    1  vs0     172.16.8.2    NO AGGR FOUND                0
              p_NFS_naneqa_2k8_v3_1      flex        51200    1  vs1     172.16.8.4    NO AGGR FOUND                0
              p_NFS_naneqa_2k8_v4_1      flex        51200    1  vs0     172.16.7.1    NO AGGR FOUND                0
              p_NFS_naneqa_2k8_v5_1      flex        51200    1  vs1     172.16.7.3    NO AGGR FOUND                0
<p>
Aggr Detail:
AGGREGATE             TYPE     AVL_SIZE(mb) 
====================  =======  ============  
aggr0_nqa_f3050_7     flex        13932
aggr0_nqa_f3050_8     flex        13934
nqa_f3050_7_aggr1     flex        12841
nqa_f3050_7_aggr2     striped    348786
nqa_f3050_8_aggr1     flex        13862
nqa_f3050_8_aggr3     flex        13862
</pre></small>

=end html

=item Examples

  eval {
      # Create 2 striped volumes for each client.
      # (if there are 3 clients, 6 volumes will be created)
      $cfg_obj->define(
		    clients=>[@clients],
		    vol_cnt=>2,
		    vol_layout=>'per_instance',
		    vol_style=>'striped'
		    );

      # Create 3 flex volumes that are mounted by all clients.
      # (regardless of client number, only 3 volumes will be created)
      $cfg_obj->define(
		    clients=>[@clients],
		    vol_cnt=>2,
		    vol_layout=>'single',
		    vol_style=>'flex'
		    );
  };
  if ($@){
      $Test->nlog($@);
      return $TCD::FAIL;
  }

=back

=head2 B<create()>

Create the cluster portion of the configuration that was defined by the define() method.
Once created the 'client volume map' is updated with the actual available volume size to 
inform other packages that use the definition for client=side configuration how much space
is avaiable for data.

=over 4

=item Inputs

None

=item Outputs

Returns 1 on success and 0 on failure.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
argument 'err_on' as defined during object instantiation.

=item Examples

    $cfg_obj->create();

=back

=head2 B<delete()>

Delete the cluster portion of the conifugration that was defined by the define() method. 

=over 4

=item Inputs

none

=item Outputs

Returns 1 on success and 0 on failure.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
argument 'err_on' as defined during object instantiation.

=item Examples

   $cfg_obj->delete();

=back

=head2 B<undefine()>

Undefine the cluster defined by define

=over 4

=item Inputs

=item * I<noundefine>

Set to one to prevent undefine so we can run create again without define

=item Outputs

Returns 1 on success and 0 on failure.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
argument 'err_on' as defined during object instantiation.

=item Examples

   $cfg_obj->undefine();

=back


=head2 B<get_client_map(I<format>)>

Return the client portion of the configuration definition to be created by a separate
package that is responsible for client-side operations.  

=over 4

=item Inputs

=over 4

=item * I<format> (default: 0)

Setting this value to 0, returns an arrayref of client volume mappings to be created.
If the value is set to 1, the definition is returned in a user readable format that 
can be logged to show the current client volume mapping that have been defined.

=back

=item Outputs

=over 4

Returns an array reference of hashes containing client volume mappings.

=back 

=item Examples

=over 4

 # Example 1:
 # display friendly format of mapping definition  
 @output=$cfg_obj->get_client_map(format=>1);

=back 

=begin html

<i>Output: </i>
<small>
<pre>
Map Detail:
CLIENT        VOLUME                     STYLE     SIZE(mb)  MNTS VS      DATA_IP      AGGREGATE             AGGR_AVL(mb)
============  =========================  =======  ========  ==== ======= ============ ====================  ============
naneqa-2k6    p_NFS_naneqa_2k6_v1_1      flex         1024    1  vs0     172.16.7.1    aggr0_nqa_f3050_7        13932
              p_NFS_naneqa_2k6_v2_1      flex         1024    1  vs1     172.16.7.3    aggr0_nqa_f3050_8        13934
              p_NFS_naneqa_2k6_v3_1      flex         4096    1  vs0     172.16.8.1    aggr0_nqa_f3050_7        12908
              p_NFS_naneqa_2k6_v4_1      flex         4096    1  vs1     172.16.8.3    aggr0_nqa_f3050_8        12910
              p_NFS_naneqa_2k6_v5_1      striped      3786    1  vs0     172.16.7.1    nqa_f3050_7_aggr2       348786
              p_NFS_naneqa_2k6_v6_1      striped      3786    1  vs1     172.16.7.3    nqa_f3050_7_aggr2       344999
              p_NFS_shared_v1_1          flex        14336    2  vs0     172.16.8.1    aggr0_nqa_f3050_7         8812
naneqa-2k8    p_NFS_naneqa_2k8_v1_1      flex         1024    1  vs0     172.16.7.2    nqa_f3050_7_aggr1       115241
              p_NFS_naneqa_2k8_v2_1      flex         1024    1  vs1     172.16.7.4    nqa_f3050_8_aggr1       116262
              p_NFS_naneqa_2k8_v3_1      flex         4096    1  vs0     172.16.8.2    nqa_f3050_7_aggr1       114217
              p_NFS_naneqa_2k8_v4_1      flex         4096    1  vs1     172.16.8.4    nqa_f3050_8_aggr1       115238
              p_NFS_naneqa_2k8_v5_1      striped      3786    1  vs0     172.16.7.2    nqa_f3050_7_aggr2       341213
              p_NFS_naneqa_2k8_v6_1      striped      3786    1  vs1     172.16.7.4    nqa_f3050_7_aggr2       337426
              p_NFS_shared_v1_1          flex        14336    2  vs0     172.16.8.2    aggr0_nqa_f3050_7         8812
</pre></small>

=end html

 # Example 2:
 # provide client volume map to pgm responsible for client-side operations
 $hash=$cfg_obj->get_client_map();
 ...client-side code using $hash output goes here...

=begin html
<b>Output: </b>
<pre>
'naneqa-2k8' => [
 {
  'data_ip' => '172.16.7.2',                  # data interface
  'mnt_cnt' => 1,                             # nbr of clients assigned to mount volume.
  'node' => 'nqa-f3050-7',                    # node name that data_ip belongs too    
  'volume' => {
    'aggregate' => 'aggr0_nqa_f3050_8',       # aggregate name volume is on
    'aggr_availsize' => '13938mb',            # available aggr space for new volumes
    'volume' => 'p_NFS_naneqa_2k8_v1_1'       # name of volume 
    'vol_availsize' => '1600mb',                 # available volume space to write data.
    'junction-path' => '/p_NFS_naneqa_2k8_v1_1', # junction path to mount
    'size' => '2048mb',                       # actual size of volume
    'type' => 'flex',                         # type of volume
    'vserver' => 'vs0',                       # virtual server 
   }
 },
 ...
 ],
</pre>

=end html

=back 

=head2 B<get_vol_names()>

Returns volume names defined or created by test.

=over 4

=item Inputs

none

=back

=over 4

=item Outputs

=over 4

Returns an array or array reference of volume names created by test.

=back 

=item Examples

 # retrieve volume names
 @vol_names=$cfg_obj->get_vol_names();


=back

=head2 B<get_vol_of()>

Return the volumes that were defined  or created by the test
as a hash reference.    

=over 4

=item Inputs

=over 4

none

=back

=item Outputs

=over 4

Returns a hash ref of keyed hashes by volume name

=back 

=item Examples

 # retrieve hash of test volumes  
 $vol_of=$cfg_obj->get_vol_of();

=back

=begin html
<i>Output: </i>
<pre>
  'p_Cluster_long_name_naneqa_2k6_v1_1' => {
	     'mnt_cnt' => 1,
	     'volume' => 'p_Cluster_long_name_naneqa_2k6_v1_1',
	     'vserver' => 'vs0',
	     'delete_volume' => 1,
	     'aggr_availsize' => '13959mb',
	     'junction-path' => '/glk',
	     'host' => 'naneqa-2k6',
	     'size' => '1024mb',
	     'ruleset' => 'p_Cluster_long_name_naneqa_2k6_1',
	     'comment' => '"(created using Config::Cluster)"',
	     'type' => 'flex',
	     'aggregate' => 'nqa_f3050_7_aggr0'
	   }
</pre>

=end html



