package ConfigBroker::CbUtils;

use strict;
use warnings;
use DBI qw();
use Tharn qw($Log);
use Tharn;
use Data::Dumper;
our @ISA = qw (Exporter);
use ConfigBroker::ResourceGroup;
use ConfigBroker::Schema;


sub get_all_resource_groups
{
   my ($dbh, $_private) = @_;
   $Log->enter();

   my $get_RGlist_cmd = "SELECT name FROM resourcegroup";
   my @RGlist = @{$dbh->selectcol_arrayref($get_RGlist_cmd)};

   # Exclude all private resource group by default: these are denoted with an
   # underscore at the beginning of a resource group 
   if (!$_private) {
      @RGlist = grep { !m/^_/} @RGlist;
   }

   $Log->exit();
   return @RGlist;
}

sub get_resource_group
{
   my ($dbh, $group) = @_;
   $Log->enter();

   my $get_RG_cmd = "SELECT * FROM resourcegroup ".
                        "WHERE name='$group'";

   $Log->debug($get_RG_cmd);

   my $RGref = $dbh->selectrow_hashref($get_RG_cmd);

   
   $Log->exit();
   return $RGref;
}

sub get_all_supergroups
{
   my ($dbh) = @_;
   $Log->enter();

   my $get_supergroup_cmd = "SELECT name FROM supergroup";

   my @superlist = @{$dbh->selectcol_arrayref($get_supergroup_cmd)};
   $Log->exit();
   return @superlist;
}

sub does_supergroup_exist
{
   my ($dbh, $groupname) = @_;
   $Log->enter();

   my @currentgroups = get_all_supergroups($dbh);
   my $found = 0;
   foreach my $grp (@currentgroups)
   {
      $found = 1 if ($grp eq $groupname);
   }
   $Log->exit();
   return $found;
}

sub create_table_sqlite
{
   my ($dbh, $table_name, $table_defn) = @_;
   $Log->enter();
   my $ret = 0;
   if ( &verify_table_exists($dbh, $table_name) )
   {
      $Log->debug("table $table_name exists\n");
   }
   else
   {
      eval {
         my $create_cmd = "CREATE TABLE \"$table_name\" \($table_defn\)";
         $Log->debug("TABLE CREATE CMD IS $create_cmd\n");
         $dbh->do($create_cmd);
      };
      if ($@) {
         $Log->warn("Error creating table \"resourcegroup\". Error is $@");
         $ret = 1;
      }
   }
   $Log->exit();
   return $ret;
}

sub drop_table { 
   my ($dbh, $table_name) = @_;
   $Log->enter();
   my $ret = 0;

    my $crossreftable = $table_name . "crossref";
    my $drop_cmd = "DROP TABLE IF EXISTS $crossreftable";
    $Log->debug("TABLE DROP CMD IS: $drop_cmd");
    eval {$dbh->do($drop_cmd); };
    if ($@) {
        $Log->warn("Error dropping table $table_name. Error is $@");
        $ret = 0;
    }
    $drop_cmd = "DROP TABLE IF EXISTS $table_name";
    $Log->debug("TABLE DROP CMD IS: $drop_cmd");
    eval {$dbh->do($drop_cmd); };
    if ($@) {
       $Log->warn("Error dropping table $table_name. Error is $@");
       $ret = 0;
    }
    return $ret;
}


sub create_table
{
   my ($dbh, $table_name, $table_defn) = @_;
   $Log->enter();
   my $ret = 0;
   eval {
      my $create_cmd = "CREATE TABLE IF NOT EXISTS $table_name $table_defn ENGINE=INNODB";
      $Log->debug("TABLE CREATE CMD IS $create_cmd\n");
      $dbh->do($create_cmd);
   };
   if ($@) {
      $Log->warn("Error creating table \"$table_name\". Error is $@");
      $ret = 1;
   }
   $Log->exit();
   return $ret;
}

sub verify_table_exists
{
   my ($dbh, $table_name) = @_;
   $Log->enter();
   my $ret = 0;

   my $get_tables_cmd = qq[SELECT table_name FROM information_schema.tables ].
                        qq[WHERE table_schema = 'CB' AND table_name = '$table_name'];

   my $tables = $dbh->selectcol_arrayref($get_tables_cmd);

   $Log->debug("get_tables_cmd: $get_tables_cmd\noutput:\n". Dumper [$tables] );

   if (ref $tables eq "ARRAY") {
      if (defined $tables->[0]) {
         $ret = $tables->[0] eq $table_name;
      }
   }

   $Log->exit();
   return $ret;
}

sub verify_resource_exists()
{
   my ($dbh, $table_name, $resource_name, $resource_group) = @_;
   $Log->enter();
   my $ret = 0;

   my $get_res_cmd = "SELECT name FROM $table_name ".
                        "WHERE name=\'$resource_name\' AND resourcegroup=\'$resource_group\'";
   my $resource = $dbh->selectcol_arrayref($get_res_cmd);
   if ($resource)
   {
      $ret = 1 if ($$resource[0]);
   }
   $Log->exit();
   return $ret;
}

sub get_resourcegroup_name()
{
   my ($dbh, $table_name, $maxrecords) = @_;
   $Log->enter();

   my $get_res_cmd = "SELECT * FROM $table_name ";
   #my $get_res_cmd = "SELECT * FROM $table_name ".
   #                      "WHERE status=\'READY\'" ;
 
   # This gives you 1 record from the table as an array of hashrefs with key/value pair
   my $resourcearrayref = $dbh->selectall_arrayref($get_res_cmd, {Slice => {} , MaxRows => $maxrecords});
   my $rgname;
   if ($resourcearrayref)
   {
      $rgname =  $$resourcearrayref[0]->{'resourcegroup'}; 
      $Log->debug( "RGNAME IS $rgname");
   }
   $Log->exit();
   return $rgname;
}

sub get_resource()
{
   my ($dbh, $table_name, $resource_name) = @_;
   $Log->enter();

   my $resources = &get_resources($dbh, $table_name, $resource_name);
   $Log->exit();
   return $$resources[0];
}

sub get_resources()
{
   my ($dbh, $table_name, $resource_name) = @_;
   $Log->enter();

   my $get_res_cmd = "SELECT * FROM $table_name ".
                        "WHERE name=\'$resource_name\'";
   $Log->debug("CbUtils: get_resource() query $get_res_cmd");
   my $resources;
   eval {
     $resources = $dbh->selectrow_arrayref($get_res_cmd);
   };
   if ( ($@) || ($resources eq 'undef')) {
      die "error retrieving data in CbUtils::get_resources method\n";
   }
   $Log->exit();
   return $resources;
}

sub _get_resourcegroups_for_type
{
   my (%getparams) = @_;

   my $dbh        = $getparams{'dbh'};
   my $table      = $getparams{'type'};
   my $exactmatch = $getparams{'match_filters'};
   my $regexmatch = $getparams{'regex_filters'};
   $Log->enter();

   my $crossreftable = $table . "crossref";
   #my $dbstring = "WHERE $crossreftable.resourcegroup=\'$rgname\' ";
   my $dbstring = "WHERE ";
   if (defined $exactmatch)
   {
      my %fieldshash = %{$exactmatch};
      my $first = 1;
      foreach my $thiskey (keys (%fieldshash)) {
         #$thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         $thiskey = &ConfigBroker::Schema::convert_reserved_word($thiskey) if (&ConfigBroker::Schema::is_reserved_word($thiskey));
         # is this an array attribute?
         my $isarray = &ConfigBroker::Schema::is_array_attribute($table, $thiskey, $dbh);
         if ($isarray) {
            # we store the arrays as as " " separated string in the DB.
            # so the value will have a " " either in the front or the back  
            my $compare_string = "($thiskey=\' $fieldshash{$thiskey}\' OR $thiskey=\'$fieldshash{$thiskey} \' OR $thiskey=\'$fieldshash{$thiskey}\')";
            #$dbstring = "$dbstring " . "AND $compare_string " ;
            if ($first) {
                $dbstring = "$dbstring " . "$compare_string " ;
            }
            else {
                $dbstring = "$dbstring " . "AND $compare_string " ;
            } 
         }
         else {
            if ($first) {
                $dbstring = "$dbstring " . "$thiskey=\'$fieldshash{$thiskey}\' " ;
            }
            else {
                $dbstring = "$dbstring " . "AND $thiskey=\'$fieldshash{$thiskey}\' " ;
            }
         }
         $first = 0 if ($first);
      }
   }

   if (defined $regexmatch)
   {
      my %fieldshash = %{$regexmatch};
      foreach my $thiskey (keys (%fieldshash)) {
         $thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         $dbstring = "$dbstring " . "AND $thiskey REGEXP \'$fieldshash{$thiskey}\' " ;
      }
   }

   $Log->debug("DBSTRING IS $dbstring\n");

   my $get_cmd = "SELECT resourcegroup FROM $crossreftable " .
                 "JOIN $table ON $table.uniq_resource_id=$crossreftable.resourceid " .
                 "$dbstring"; 
   $Log->debug("DBCMD IS $get_cmd\n");

   my $resarrayref;
   eval {
      $resarrayref = $dbh->selectcol_arrayref($get_cmd);
   };
   if ( ($@) || ($resarrayref eq 'undef')) {
      die "error retrieving data in CbUtils::_get_resourcegroups_for_type method\n";
   }
   $Log->exit();
   return $resarrayref;
}


sub _get_resources_arrayref_from_db
{
   my (%getparams) = @_;

   my $rgname     = $getparams{'resource_group'};
   my $dbh        = $getparams{'dbh'};
   my $table      = $getparams{'table'};
   my $maxrecords = $getparams{'resource_count'};
   my $exactmatch = $getparams{'exact_filters'};
   my $regexmatch = $getparams{'regex_filters'};
   my $randomize  = $getparams{'randomize'};
   my $offset     = $getparams{'offset'};
   $Log->enter();

   my $crossreftable = $table . "crossref";
   my $dbstring = "WHERE $crossreftable.resourcegroup=\'$rgname\' ";
   if (defined $exactmatch)
   {
      my %fieldshash = %{$exactmatch};
      my $first = 1;
      foreach my $thiskey (keys (%fieldshash)) {
         #$thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         $thiskey = &ConfigBroker::Schema::convert_reserved_word($thiskey) if (&ConfigBroker::Schema::is_reserved_word($thiskey));
         # is this an array attribute?
         my $isarray = &ConfigBroker::Schema::is_array_attribute($table, $thiskey, $dbh);
         if ($isarray) {
            # we store the arrays as as " " separated string in the DB.
            # so the value will have a " " either in the front or the back  
            my $compare_string = "($thiskey=\' $fieldshash{$thiskey}\' OR $thiskey=\'$fieldshash{$thiskey} \' OR $thiskey=\'$fieldshash{$thiskey}\')";
            $dbstring = "$dbstring " . "AND $compare_string " ;
         }
         else {   
            $dbstring = "$dbstring " . "AND $thiskey=\'$fieldshash{$thiskey}\' " ;
         }
      }
   }

   if (defined $regexmatch)
   {
      my %fieldshash = %{$regexmatch};
      foreach my $thiskey (keys (%fieldshash)) {
         $thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         $dbstring = "$dbstring " . "AND $thiskey REGEXP \'$fieldshash{$thiskey}\' " ;
      }
   }

   my $add_on = "";
   if ($randomize) {
       $add_on = $add_on . "ORDER BY RAND() ";
   }
   if ($maxrecords) {
       $add_on = $add_on . "LIMIT $offset, $maxrecords ";
   }
   $Log->debug("DBSTRING IS $dbstring\n");

   my $get_cmd = "SELECT * FROM $table " .
                 "STRAIGHT_JOIN $crossreftable ON $table.uniq_resource_id=$crossreftable.resourceid " .
                 "$dbstring" .
                 "$add_on" ;
   $Log->debug("DBCMD IS $get_cmd\n");

   ### add %attr parameter to the selectall_arrayref and include value for
   ### "slice" key as an empty hash to get each row as a hash.
   my $resarrayref;
   eval {
      $resarrayref = $dbh->selectall_arrayref($get_cmd, {Slice => {}});
   };
   if ( ($@) || ($resarrayref eq 'undef')) {
      die "error retrieving data in CbUtils::_get_resources_arrayref_from_db method\n";
   }
   $Log->exit();
   return $resarrayref;
}

###############################################################################
# Helper method to create a new instance of a specific NACL obj.
###############################################################################
sub _create_nacl_obj {
    my (%args) = @_;
    $Log->enter();

    # Locals
    my $type     = undef;
    my $node     = undef;
    my $resource = undef;

    # Assign values
    if (!defined($type = $args{'type'})) {
        NATE::Exceptions::Argument->throw("Missing required field 'type'");
    }
    if (!defined($resource = $args{'resource'})) {
        NATE::Exceptions::Argument->throw(
            "Missing required field 'resource'");
    }
    if (!defined($node = $args{'node'})) {
        NATE::Exceptions::Argument->throw("Missing required field 'node'");
    }

    # Switch/Case implementation
    if ($type =~ /aggregate/) {
        my $aggr = $resource->{'name'};
        return NACL::C::StorageAggregate->new(
            'command_interface' => $node,
            'aggregate'         => $aggr
        );
    } elsif ($type =~ /vserver/) {
        return NACL::C::Vserver->new(
            'command_interface' => $node,
            'vserver'           => $resource->{'name'}
        );
    } elsif ($type =~ /volume/) {
        my $vserver = $resource->{'vserver'};
        my $vol     = $resource->{'name'};
        return NACL::C::Volume->new(
        #return NACL::CS::Volume->fetch(
            'command_interface' => $node,
            'volume'            => $vol,
            #'vserver'           => $vserver,
        );
    } elsif ($type =~ /lun/) {
        my $vol = $resource->{'volume'};
        my $lun = $resource->{'NAME'};
        return NACL::C::Lun->new(
            'command_interface' => $node,
            'vserver'           => $resource->{'vserver'},
            'path'              => "/vol/$vol/$lun"
        );
    } elsif ($type =~ /igroup/) {
        my $vserver = $resource->{'vserver'};
        my $igroup  = $resource->{'name'};
        NACL::C::LunIgroup->new(
            command_interface => $node,
            vserver           => $vserver,
            igroup            => $igroup,
        );
    } else {
        NATE::BaseException->throw(
            "Type not recognized or supported '$type'");
    }
    $Log->exit();
    return 0;
} ## end sub _get_nacl_obj

sub get_resource_group_object
{
   my ($dbh, $groupname, $dsn) = @_;
   $Log->enter();

   my %res_info;
   $res_info{'dbh'} = $dbh;
   $res_info{'name'}= $groupname;
   $res_info{'dsn'} = $dsn if (defined $dsn);

   my $RG = new ConfigBroker::ResourceGroup(%res_info);
   $Log->exit();
   return $RG;
}


1;

