package ConfigBroker::Engine;

use strict;
use warnings;
use DBI qw();
use Tharn qw($Log);
use Tharn;
use Data::Dumper;
our @ISA = qw (Exporter);
use ConfigBroker::Engine::EngineEnGenX;
use ConfigBroker::Engine::EngineCCL;
use ConfigBroker::Engine::EngineDiscover;
use ConfigBroker::Engine::EngineDiscover7M;
use ConfigBroker::Engine::EngineDiscoverClients;
use ConfigBroker::Engine::EngineDiscoverUsers;

sub new
{
   my ($pkg, $engineinfo) = @_;
   $Log->enter();

   $Log->debug(Dumper($engineinfo));
   my $class = ref($pkg) || $pkg ;
   my $self = {};
   if ((defined $engineinfo->{'discover'}) && ($engineinfo->{'discover'} == 1)) {
      $self->{'discover'}      = $engineinfo->{'discover'};
      $self->{'discoverobjs'}  = $engineinfo->{'discoverobjs'} if (defined $engineinfo->{'discoverobjs'});
      $self->{'excludeobjs'}   = $engineinfo->{'excludeobjs'}  if (defined $engineinfo->{'excludeobjs'});
      $self->{'clients'}       = $engineinfo->{'clients'}      if (defined $engineinfo->{'clients'});
      $self->{'users'}         = $engineinfo->{'users'}        if (defined $engineinfo->{'users'});
   }
   else {
      if ((! defined $engineinfo->{'configfile'}) && (!defined $engineinfo->{'cleanupfile'})) {
         $Log->error("Either configfile or cleanupfile must be defined.\n");
         exit 1;
      }
   }
   if (! defined $engineinfo->{'dbh'}) {
      $Log->error("dbh must be defined.\n");
      exit 1;
   }
   if (! defined $engineinfo->{'resourcegroup'}) {
      $Log->error("resourcegroup must be defined.\n");
      exit 1;
   }
 
   $self->{'node'}           = $engineinfo->{'node'}          if (defined $engineinfo->{'node'});
   $self->{'resourcegroup'}  = $engineinfo->{'resourcegroup'} if (defined $engineinfo->{'resourcegroup'});
   $self->{'configfile'}     = $engineinfo->{'configfile'}    if (defined $engineinfo->{'configfile'});
   $self->{'cleanupfile'}    = $engineinfo->{'cleanupfile'}   if (defined $engineinfo->{'cleanupfile'});
   $self->{'cmdlinestring'}  = $engineinfo->{'cmdlinestring'} if (defined $engineinfo->{'cmdlinestring'});
   $self->{'dbh'}            = $engineinfo->{'dbh'}           if (defined $engineinfo->{'dbh'});
   $self->{'clients'}        = $engineinfo->{'clients'}       if (defined $engineinfo->{'clients'});
   $self->{'users'}          = $engineinfo->{'users'}         if (defined $engineinfo->{'users'});
   bless ($self, $class);

   $Log->exit();
   return $self;
}

# Class method
sub get_engine_object
{
   my ($engineinfo) = @_;
   $Log->enter();

   my $engine;
   if ($engineinfo->{'enginename'} eq 'EnGenX') {
      $engine = new ConfigBroker::Engine::EngineEnGenX($engineinfo);
   }
   elsif ($engineinfo->{'enginename'} eq 'CCL') {
      $engine = new ConfigBroker::Engine::EngineCCL($engineinfo);
   }
   elsif ($engineinfo->{'enginename'} eq 'Discover') {
      $engine = new ConfigBroker::Engine::EngineDiscover($engineinfo);
   }
   elsif ($engineinfo->{'enginename'} eq 'Discover7M') {
      $engine = new ConfigBroker::Engine::EngineDiscover7M($engineinfo);
   }
   elsif ($engineinfo->{'enginename'} eq 'DiscoverClients') {
      $engine = new ConfigBroker::Engine::EngineDiscoverClients($engineinfo);
   }
   elsif ($engineinfo->{'enginename'} eq 'DiscoverUsers') {
      $engine = new ConfigBroker::Engine::EngineDiscoverUsers($engineinfo);
   }

   $Log->exit();
   return $engine;
}

sub get_cleanupfile()
{
   my ($self) = @_;

   return $self->{'cleanupfile'};
}

###  this method to be overridden by the derived class to do the actual setup
sub run_setup()
{
   my ($self) = @_;

   my $ret = 0;
   return $ret;
}

###  this method to be overridden by the derived class to do the actual cleanup
sub run_cleanup()
{
   my ($self) = @_;

   my $ret = 0;
   return $ret;
}
   
###  this method to be overridden by the derived class to do the actual discover
sub run_discovery()
{
   my ($self) = @_;

   my $ret = 0;
   return $ret;
}

1;
