package ConfigBroker::Engine::EngineCCL;

use strict;
use warnings;
use DBI qw();
use File::Path;
use File::Basename;
use Tharn qw($Log);
use Tharn;
use NATE::Result;
use Data::Dumper;
our @ISA = qw (ConfigBroker::Engine Exporter);
use ConfigBroker::CbUtils;
use YAML;

sub new
{
   my ($pkg, $configinfo) = @_;
   $Log->enter();
   my $class = ref($pkg) || $pkg ;
   my $self=$class->SUPER::new($configinfo);
   bless ($self, $class);

   $Log->exit();
   return $self;
}

sub run_setup()
{
   my ($self) = @_;
   $Log->enter();

   my $ret = 0;
   my $cmdline = $self->{'cmdlinestring'};

   ### Create a directory for ccl to dump resource files. The directory is named as the resourcegroup.
   ### The resource files will be named by the type of resource by CCL.
   my $umask = umask 0;
   my $logdir = $ENV{LOGDIR} || ".";
   my $dir = "$logdir/$self->{'resourcegroup'}";
   mkpath "$dir";
   umask $umask;

   ### For CCL we split the the cmdline string and give it to the subtest for passing into the ccl engine call.
   my @cmdlinearray = split(" ", $cmdline);
   my $subtest_string = "tools/CCL/ccl_engine.thpl";
   my $subtest_object = Tharn::subtest(
                          $subtest_string,
                          @cmdlinearray,
                          "OBJECT_YAML_DIR=$dir",
                          '-runid',
                          "CCLENGINE",
                          -onexit, 
                          \&subtest_exit_handler
   );
   my @child_results = $subtest_object->results();
   my $failure = 0;
   foreach my $result (@child_results) {
      if ( ref($result) =~ /^NATE::Result::(Fatal|Fail|Script|Config)$/ ) {
         $Log->comment("Subtest failure. Returning failure\n");
         return 1;
      }
   }

   my $config = $self->{'configfile'};
   $config=basename($config,(".rs"));
   my $undofile = $logdir . "/" . "$config.undo";
   $self->{'cleanupfile'} = $undofile;

   #my $subtest_object = Tharn::subtest(
   #                       $subtest_string,
   #                       "RULESET=$self->{'configfile'}",
   #                       "UNDO_ON_FAIL=1",
   #                       "CONFIGBROKER=$self->{'resourcegroup'}",
   #                       '-runid',
   #                       "CCLENGINE"
   #);

   my $rg_obj = &ConfigBroker::CbUtils::get_resource_group_object($self->{'dbh'}, $self->{'resourcegroup'});
   $rg_obj->update_cleanupfilename($self->{'cleanupfile'});

   ### CCL does not update the database.  They will write to Yaml files under the RESOURCEINFODIR.
   ### So, read them back from yaml and do add resources.

   my $files = `ls $dir`;
   my @filelist = split(' ', $files);
   foreach my $file (@filelist) {
      ### Load yaml back into an array
      ### Each file contains array of hashes of that resource type
      my ($arrayref, $string) = YAML::LoadFile("$dir/$file");

      ### Handle naming discrepencies between CCL names and NACL names. CCL uses NACL, but the resource
      ### names doesn't always match.
      $file = "vservercifsshare" if ($file eq "cifs_share"); 

      $rg_obj->add_resources(resource_type => $file,
                          resourcearrayref =>  $arrayref);
   }
      
   $Log->exit();
   return $ret;

}

###  this method to be overridden by the derived class to do the actual cleanup
sub run_cleanup()
{
   my ($self) = @_;
   $Log->enter();

   my $ret = 0;
   my $subtest_string = "tools/CCL/ccl_undo.thpl";
   my $subtest_object = Tharn::subtest(
                          $subtest_string,
                          "UNDO_FILE=$self->{'cleanupfile'}",
                          "CONFIGBROKER=$self->{'resourcegroup'}",
                          '-runid',
                          "CCLUNDO"
   );
   $Log->exit();
   return $ret;
}

sub subtest_exit_handler ()
{
   ### Just an empty exit handler to catch the flow so
   ### we can pass on the failure to the caller where 
   ### failure scenario is handled.
}

1;
