package ConfigBroker::Engine::EngineDiscover;

use strict;
use warnings;
use DBI qw();
use File::Path;
use File::Basename;
use Tharn qw($Log);
use Tharn;
use Data::Dumper;
our @ISA = qw (ConfigBroker::Engine Exporter);
use cluster::enterprise::TestTriage::DiscoverConfiguration qw(getObjectList discoverObject);
use ConfigBroker::CbUtils;
use Params::Validate qw(validate SCALAR OBJECT HASHREF BOOLEAN ARRAYREF UNDEF);
use Module::Common qw(validate_mixed);
use YAML;

sub new
{
   my ($pkg, $configinfo) = @_;
   $Log->enter();
   my $class = ref($pkg) || $pkg ;
   my $self=$class->SUPER::new($configinfo);
   bless ($self, $class);

   $Log->exit();
   return $self;
}

sub run_discovery()
{
   my ($self) = @_;
   $Log->enter();

   my $ret = 0;

   my $get_fields_cmd = "SELECT typename FROM typedefn";
   my $fields = $self->{'dbh'}->selectcol_arrayref($get_fields_cmd);
   my $RG = &ConfigBroker::CbUtils::get_resource_group_object($self->{'dbh'}, $self->{'resourcegroup'});

   ### Use <test>/cluster/enterprise/TestTriage/DiscoverConfiguration.pm for discovery
   my $obj_arrayref;
   my @objects;
   if ((defined $self->{'discoverobjs'}) && ($self->{'discoverobjs'} ne "")) {
      $obj_arrayref = $self->{'discoverobjs'};
   }
   else {
      $obj_arrayref = &getObjectList();
   }
   foreach my $obj (@{$obj_arrayref}) {
      my $supported = 0;
      foreach (@$fields) {
         if (lc($obj) eq $_)  {
            $Log->debug ("Supported type $_\n");
            $supported = 1;
            last;
         }
      }
      if ($supported == 1) {
         my $exclude = 0;
         foreach my $exclude_obj (@{$self->{'excludeobjs'}}) {
            if ($obj eq $exclude_obj) { # This object should be excluded.  Do NOT add to the final discover object list.
               $exclude = 1;
               $Log->debug("Excluding $obj\n");
               last;
            }
         }
         if ($exclude == 0) {
            $Log->debug ("Including $obj to discover array\n");
            push (@objects, $obj);
         }
      }
   }
   foreach my $objtype (@objects) {
      $Log->comment("Discovering $objtype\n");
      my $objarray = &discoverObject(conn => $self->{'node'}, object => $objtype);
      if (ref($objarray) && scalar(@$objarray) > 0) {
         $Log->debug("Adding Objecttype $objtype\n");
         #$objarray = $self->process_exceptions(type => $objtype, objarrayref => $objarray);
         $RG->add_resources(resource_type => lc($objtype),
                         resourcearrayref => $objarray);
      }
   }

   $Log->exit();
   return $ret;
}

sub process_exceptions
{
   my ($self, %opts) =
          validate_mixed(@_,
                  [{type=> OBJECT},],
                  {type => {type => SCALAR,},
                   objarrayref => {type => ARRAYREF, optional => 1,},},);

   my $type = $opts{'type'};
   my $returnarrayref = $opts{'objarrayref'};
   #if (lc($type) eq "vfiler") {
   #   $returnarrayref = $self->process_7mode_vfiler($returnarrayref);
   #   $Log->debug(Dumper($returnarrayref));
   #}
   return $returnarrayref;

}


#sub process_7mode_vfiler
#{
#   my ($self, $vfilerarrayref) = @_;

#   my @returnarray; 
#   foreach my $vfilerhashref (@{$vfilerarrayref}) {
#      my @vfilerkeys = keys(%{$vfilerhashref});
#      my %vfilerhash;
#      my $ipaddrarrayref = $vfilerhashref->{'ip_address_configuration'};
#      foreach my $ipaddrhashref (@{$ipaddrarrayref}) {
#         my %newhash = ();
#         $newhash{'ip_address'} = $ipaddrhashref->{'ip_address'};
#         $newhash{'interface'}  = $ipaddrhashref->{'interface'};
#         foreach my $thiskey (@vfilerkeys) {
#               $newhash{$thiskey} =  $vfilerhashref->{$thiskey};
#         }  
#         push (@returnarray, \%newhash);
#      }  
#   }
#   return \@returnarray;
#}

1;
