package ConfigBroker::Engine::EngineDiscover7M;

use strict;
use warnings;
use DBI qw();
use File::Path;
use File::Basename;
use Tharn qw($Log);
use Tharn;
use Data::Dumper;
our @ISA = qw (ConfigBroker::Engine Exporter);
use cluster::enterprise::TestTriage::DiscoverConfiguration qw(getObjectList discoverObject);
#use Topology;
#use Topology::Config::7M;
use ConfigBroker::CbUtils;
use Options;
use NACL::C::Node;
use YAML;

sub new
{
   my ($pkg, $configinfo) = @_;
   $Log->enter();
   my $class = ref($pkg) || $pkg ;
   my $self=$class->SUPER::new($configinfo);
   bless ($self, $class);

   $Log->exit();
   return $self;
}

sub run_discovery()
{
   my ($self) = @_;
   $Log->enter();

   my $ret = 0;

   my $get_fields_cmd = "SELECT typename FROM typedefn";
   my $fields = $self->{'dbh'}->selectcol_arrayref($get_fields_cmd);
   my $RG = &ConfigBroker::CbUtils::get_resource_group_object($self->{'dbh'}, $self->{'resourcegroup'});

   my @targets = ( 'cifs', 'net', 'shares', 'vfilers', 'volumes', 'useradmin');

   my @nodenames;
   foreach my $nodeobj (@{$self->{'node'}}) {
      push (@nodenames, $nodeobj->hostrec->id()); 
   }

   my $discovered = {};
   $discovered = $self->discover_7m();

   foreach my $thisnode (@nodenames) {

      my $vfilers = $discovered->{$thisnode}->{'vfiler-info'};
      my $allowed_protocols = $discovered->{$thisnode}->{'vfiler-get-allowed-protocols'};
      my $vfilerhash = $self->process_vfiler_resources($vfilers, $allowed_protocols, $thisnode);
      $RG->add_resources(resource_type => "vfiler", resourcearrayref => $vfilerhash->{'vfiler'});
      $RG->add_resources(resource_type => "vfdns", resourcearrayref => $vfilerhash->{'vfdns'});
      $RG->add_resources(resource_type => "vfnis", resourcearrayref => $vfilerhash->{'vfnis'});
      $RG->add_resources(resource_type => "vfstore", resourcearrayref => $vfilerhash->{'vfstore'});
      $RG->add_resources(resource_type => "vfnet", resourcearrayref => $vfilerhash->{'vfnet'});

      my $options = $discovered->{$thisnode}->{'options-list-info'};
      $options = $self->process_options_resources($options, $thisnode);
      $RG->add_resources(resource_type => "options7m",     resourcearrayref => $options);

      my $net     = $discovered->{$thisnode}->{'interface-config-info'};
      $net = $self->process_net_resources($net, $thisnode);
      $RG->add_resources(resource_type => "net",     resourcearrayref => $net);

      my $shares  = $discovered->{$thisnode}->{'cifs-share-info'};
      $shares = $self->process_shares_resources($shares, $thisnode);
      $RG->add_resources(resource_type => "shares",  resourcearrayref => $shares);

      my $cifs    = $discovered->{$thisnode}->{'cifs-list-config'};
      $cifs = $self->process_cifs_resources($cifs, $thisnode);
      $RG->add_resources(resource_type => "cifs",   resourcearrayref => $cifs);

      my $volumes = $discovered->{$thisnode}->{'volume_info'};
      $volumes = $self->process_volume_resources($volumes, $thisnode);
      $RG->add_resources(resource_type => "volume", resourcearrayref => $volumes);

      my $aggrs = $discovered->{$thisnode}->{'aggr_info'};
      $aggrs = $self->process_aggregate_resources($aggrs);
      $RG->add_resources(resource_type => "aggregate", resourcearrayref => $aggrs);
    
      my $user_info = [];
      $user_info = $discovered->{$thisnode}->{'user_info'};
      if (@{$user_info}) {
          $RG->add_resources(
              resource_type    => "useradmin",
              resourcearrayref => $user_info
         );
      }
   }

   $Log->exit();
   return $ret;
}

sub discover_7m {

    $Log->enter();
    my ($self) = @_;
    my $discover = {};
    my $status;

    my @nodenames;
    foreach my $nodeobj (@{$self->{'node'}}) {
       push (@nodenames, $nodeobj->hostrec->id()); 
    }

    my $Options = new Options;
    $Options->process();
    my $Options_hash = $Options->get_hash();

    if (defined @{$Options_hash->{'NODES'}}) {
       @nodenames =  @{$Options_hash->{'NODES'}};
    } 

    # Discover all existing vfilers and volumes
    foreach my $node (@nodenames) {
        $Log->comment("Creating ZAPI connection object for $node");
        my $zapi = NACL::APISet->new(
            hostobj   => host($node),
            category  => "Node",
            interface => "ZAPI",
            set       => "7Mode"
        );
	my $hosttype = eval {host($node)->hosttype();};

        my $parsed_output;
        # check to ensure Zapi is enabled, if not, try to enable it
        my $response;
        try {
            $response = $zapi->system_get_version();
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = shift;

            # if zapi connect fails to find socket, attempt to enable it
            if ($exception->text() =~ /cannot connect to socket/) {
                $Log->comment("Enabling Zapi");
                die;
            } else {
                die config_result($exception->text());
            }
        };

        #$Log->comment("Discovering licenses on $node");
        #$response = $zapi->license_v2_list_info();
        #$parsed_output = $response->get_parsed_output();
        #$discover->{$node}->{'license-v2-info'}
        #    = $parsed_output->[0]->{'licenses'}->[0]->{'license-v2-info'};

        # Set the vfiler context for the next ZAPI calls
        try {
            $response = $zapi->set_vfiler('vfiler-name' => 'vfiler0');
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception = shift;
            die config_result($exception->text());
        };

        $Log->comment("Discovering interfaces on $node");
        $response      = $zapi->net_ifconfig_get();
        $parsed_output = $response->get_parsed_output();
        $discover->{$node}->{'interface-config-info'}
            = $parsed_output->[0]->{'interface-config-info'}->[0]->{'interface-config-info'};

        $Log->comment("Discovering vfilers on $node");
        $response      = $zapi->vfiler_list_info();
        $parsed_output = $response->get_parsed_output();
        $discover->{$node}->{'vfiler-info'}
            = $parsed_output->[0]->{'vfilers'}->[0]->{'vfiler-info'};

        $Log->comment("Discovering volumes on $node");
        $response      = $zapi->volume_list_info_iter_start();
        $parsed_output = $response->get_parsed_output();
        my $records = ${$parsed_output}[0]->{'records'};
        my $tag     = ${$parsed_output}[0]->{'tag'};
        $Log->comment("$records volumes discovered");

        $response = $zapi->volume_list_info_iter_next(
            'tag'     => $tag,
            'maximum' => $records
        );
        my $volumes = $response->get_parsed_output('tag' => $tag);
        my @volume_info = @{$volumes->[0]->{'volumes'}->[0]->{'volume-info'}};

        foreach my $volume (@volume_info) {
            push(@{$discover->{$node}->{'volume_info'}}, $volume);
        }
        $response = $zapi->volume_list_info_iter_end('tag' => $tag);

        $Log->comment("Discovering aggregates on $node");
        $response = $zapi->aggr_list_info();
        my $aggregates = $response->get_parsed_output();
        my @aggr_info
            = @{$aggregates->[0]->{'aggregates'}->[0]->{'aggr-info'}};
        foreach my $aggr (@aggr_info) {
            push(@{$discover->{$node}->{'aggr_info'}}, $aggr);
        }

        foreach my $vfiler (@{$discover->{$node}->{'vfiler-info'}}) {

            # Set the vfiler context for the next ZAPI calls
            try {
                $response
                    = $zapi->set_vfiler('vfiler-name' => $vfiler->{'name'});
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception = shift;
                die config_result($exception->text());
            };

            # Discover vfiler allowed protocols
            $Log->comment("Discovering allowed protocols on vfiler $vfiler->{'name'}");
            eval {
                $response = $zapi->vfiler_get_allowed_protocols(vfiler => $vfiler->{'name'});
            };

            if (!$@) {
                $parsed_output = $response->get_parsed_output();
                $parsed_output->[0]->{'vfiler'} = $vfiler->{'name'};
                push( 
                    @{$discover->{$node}->{'vfiler-get-allowed-protocols'}}, 
                    $parsed_output->[0]
                );
            }

            # Set the vfiler context for the next ZAPI calls
            try {
                $response
                    = $zapi->set_vfiler('vfiler-name' => $vfiler->{'name'});
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception = shift;
                die config_result($exception->text());
            };

            # Discover options
            $Log->comment("Discovering options on vfiler $vfiler->{'name'}");

            eval { $response = $zapi->options_list_info(); };
            if (!$@) {

                my $parsed_output = $response->get_parsed_output();
                $parsed_output->[0]->{options}->[0]->{'vfiler'} = $vfiler->{'name'};
                push(
                    @{  $discover->{$node}->{'options-list-info'}},
                    $parsed_output->[0]->{options}->[0]
                );
            }

            # Check if CIFS is configured
	    my $cifs_server; 
            eval { $response = $zapi->cifs_list_config(); };
            if (!$@) {

                $Log->comment("Discovering CIFS on vfiler $vfiler->{'name'}");

                $parsed_output = $response->get_parsed_output();
		$cifs_server = $parsed_output->[0]->{'NetBIOS-servername'};
                $parsed_output->[0]->{'vfiler'} = $vfiler->{'name'};
                push(
                    @{$discover->{$node}->{'cifs-list-config'}},
                    $parsed_output->[0]
                );

                $response = $zapi->cifs_status();
                $status   = $response->get_parsed_output();
                if (!grep(/started/, @{$status->[0]->{'status'}})) {
                    next;
                }

                $Log->comment(
                    "Discovering CIFS shares on node $node vfiler $vfiler->{'name'}"
                );

                # Get the current share config
                my ($parsed_output, $records, $tag);

                $response = $zapi->cifs_share_list_iter_start(
                    'zapi-timeout' => 3600);
                $parsed_output = $response->get_parsed_output();
                $records       = ${$parsed_output}[0]->{'records'};
                $tag           = ${$parsed_output}[0]->{'tag'};

                my $max_records_per_call = 10000;
                while ($records) {

                    my $records_this_call;
                    if ($max_records_per_call >= $records) {
                        $records_this_call = $records;
                    } else {
                        $records_this_call = $max_records_per_call;

                    }
                    $records -= $records_this_call;

                    my $shares;
                    $response = $zapi->cifs_share_list_iter_next(
                        'tag'     => $tag,
                        'maximum' => $records_this_call
                    );
                    $shares = $response->get_parsed_output('tag' => $tag);

                    if ($shares->[0]->{'records'}) {
                        my @share_info = @{$shares->[0]->{'cifs-shares'}->[0]
                                ->{'cifs-share-info'}};

                        foreach my $share (@share_info) {
                            $share->{'vfiler'} = $vfiler->{'name'};
                            push(@{$discover->{$node}->{'cifs-share-info'}}, $share);
                        }
                    }
                    snooze 10;
                }

                $response = $zapi->cifs_share_list_iter_end('tag' => $tag);
            }
        
            $Log->comment("Discovering users on vfiler $vfiler->{'name'}");
            $response = $zapi->useradmin_user_list(verbose => 1);
            $parsed_output = $response->get_parsed_output();
            my @user_info = @{$parsed_output->[0]->{'useradmin-users'}->[0]
                    ->{'useradmin-user-info'}};
            foreach my $user (@user_info) {

                my $comment = $user->{'comment'};
                if (ref($user->{'comment'}) eq 'ARRAY') {
                    $comment = "";
                }

                my $full_name = $user->{'full-name'};
                if (ref($user->{'full-name'}) eq 'ARRAY') { $full_name = ""; }
                if (!$full_name) { next; }

                my $homedir;
                $response = $zapi->cifs_homedir_path_get_for_user(
                    'user-name' => $user->{'name'});
                $parsed_output = $response->get_parsed_output();
                $homedir       = $parsed_output->[0]->{'homedir-path-user'};

                push(
                    @{$discover->{$node}->{'user_info'}},
                    {   vfiler    => $vfiler->{'name'},
                        node      => $node,
                        comment   => $comment,
                        name      => $user->{'name'},
                        full_name => $full_name,
                        homedir   => $homedir
                    }
                );
            }
        }
    }
    #$Log->comment(Dumper($discover));
    $Log->exit();
    return $discover;
} ## end sub discover_7m

sub process_net_resources
{
   my ($self, $netarrayref, $node) = @_;

   my @returnarray;
   foreach my $nethashref (@{$netarrayref}) {
      my @netkeys = keys(%{$nethashref});
      my %nethash;
      #my $ipaddrarrayref = $nethashref->{'v4-primary-address'}->{'ip-address-info'};
      my $v4primaryarrayref = $nethashref->{'v4-primary-address'};
      foreach my $v4hashref (@{$v4primaryarrayref}) {
         my $ipaddrarrayref = $v4hashref->{'ip-address-info'};
         foreach my $ipaddrhashref (@{$ipaddrarrayref}) {
            my %newhash = ();
            $newhash{'broadcast'}    = $ipaddrhashref->{'broadcast'};
            $newhash{'addr_family'}  = $ipaddrhashref->{'addr-family'};
            $newhash{'creator'}      = $ipaddrhashref->{'creator'};
            $newhash{'netmask_or_prefix'}      = $ipaddrhashref->{'netmask-or-prefix'};
            $newhash{'address'}      = $ipaddrhashref->{'address'};
            foreach my $thiskey (@netkeys) {
               # convert any "-" in the inputkeys to "_" to accomodate mysql table field.
               my $newkey = $thiskey;
               $newkey =~ s/-/_/g ; 
               $newhash{$newkey} =  $nethashref->{$thiskey};
            }
            $newhash{'node'}     = $node;
            push (@returnarray, \%newhash);
         }
      }
   }
   return \@returnarray;
}

sub process_shares_resources
{
   my ($self, $sharesarrayref, $node) = @_;

   my @returnarray;
   foreach my $shareshashref (@{$sharesarrayref}) {
      my @shareskeys = keys(%{$shareshashref});
      my %shareshash;
      my %newhash = ();
      foreach my $thiskey (@shareskeys) {
        # convert any "-" in the inputkeys to "_" to accomodate mysql table field.
        my $newkey = $thiskey;
        $newkey =~ s/-/_/g ;
        $newhash{$newkey} =  $shareshashref->{$thiskey};
      }
      $newhash{'node'}     = $node;
      push (@returnarray, \%newhash);
   }
   return \@returnarray;
}

sub process_volume_resources
{
   my ($self, $volumearrayref, $node) = @_;

   my @returnarray;
   foreach my $volumehashref (@{$volumearrayref}) {
      my @volumekeys = keys(%{$volumehashref});
      my %newhash = ();
      foreach my $thiskey (@volumekeys) {
            # convert any "-" in the inputkeys to "_" to accomodate mysql table field.
            my $newkey = $thiskey;
            $newkey =~ s/-/_/g ;
            $newhash{$newkey} =  $volumehashref->{$thiskey};
      }
      $newhash{'aggregate'} = $volumehashref->{'containing-aggregate'};
      $newhash{'volume'} = $volumehashref->{'name'};
      $newhash{'node'}   = $node;
      push (@returnarray, \%newhash);
   }
   return \@returnarray;
}

sub process_aggregate_resources
{
   my ($self, $aggrarrayref) = @_;

   my @returnarray;
   foreach my $aggrhashref (@{$aggrarrayref}) {
      my @aggrkeys = keys(%{$aggrhashref});
      my %newhash = ();
      foreach my $thiskey (@aggrkeys) {
            # convert any "-" in the inputkeys to "_" to accomodate mysql table field.
            my $newkey = $thiskey;
            $newkey =~ s/-/_/g ;
            $newhash{$newkey} =  $aggrhashref->{$thiskey};
      }
      $newhash{'aggregate'} = $aggrhashref->{'name'};
      push (@returnarray, \%newhash);
   }
   return \@returnarray;
}

sub process_options_resources
{
   my ($self, $optionsarrayref, $node) = @_;

   my @returnarray;
   foreach my $optionshashref (@{$optionsarrayref}) {
      my $optionsinfo_arrayref = $optionshashref->{'option-info'};
      my $vfiler               = $optionshashref->{'vfiler'};
      foreach my $optionsinfo_hashref (@{$optionsinfo_arrayref}) {
         my @optionskeys = keys(%{$optionsinfo_hashref});
         my %newhash = ();
         foreach my $thiskey (@optionskeys) {
               # convert any "-" in the inputkeys to "_" to accomodate mysql table field.
               my $newkey = $thiskey;
               $newkey =~ s/-/_/g ;
               $newhash{$newkey} =  $optionsinfo_hashref->{$thiskey};
         }
         $newhash{'vfiler'} = $vfiler;
         $newhash{'node'}   = $node;
         push (@returnarray, \%newhash);
      }
   }
   return \@returnarray;
}

sub process_cifs_resources
{
   my ($self, $cifsarrayref, $node) = @_;

   my @returnarray;
   foreach my $cifshashref (@{$cifsarrayref}) {
      my @cifskeys = keys(%{$cifshashref});
         my %newhash = ();
         foreach my $thiskey (@cifskeys) {
               # convert any "-" in the inputkeys to "_" to accomodate mysql table field.
               my $newkey = $thiskey;
               $newkey =~ s/-/_/g ;
               $newhash{$newkey} =  $cifshashref->{$thiskey};
         }
         $newhash{'node'}   = $node;
         push (@returnarray, \%newhash);
   }
   return \@returnarray;
}


sub process_vfiler_resources
{
   my ($self, $vfilerarrayref, $allowedprotoarrayref, $node) = @_;

   my @vfilerarray;
   my @nisarray;
   my @dnsarray;
   my @vfstoresarray;
   my @vfnetarray;
   my @protoarray;

   foreach my $vfilerhashref (@{$vfilerarrayref}) {
      my @vfilerkeys = keys(%{$vfilerhashref});
      my $nisinfo_arrayref = $vfilerhashref->{'nis-info'};
      my $dnsinfo_arrayref = $vfilerhashref->{'dns-info'};
      my $vfstores_arrayref = $vfilerhashref->{'vfstores'};
      my $vfnet_arrayref    = $vfilerhashref->{'vfnets'};

      foreach my $allowed_proto_hashref (@{$allowedprotoarrayref}) {
         my $apinfo_arrayref = $allowed_proto_hashref->{'allowed-protocols'};
         foreach my $ap_hashref (@{$apinfo_arrayref}) {
            my %protohash = ();
            my $ap_arrayref = $ap_hashref->{'protocol-info'};
            my $allowed_protocols = "";
            if (ref ($ap_arrayref) eq "ARRAY") {
               foreach my $proto_hashref (@{$ap_arrayref}) {
                  my $proto = $proto_hashref->{'protocol'};
                  $allowed_protocols = "$allowed_protocols $proto";
               }
            }
            else {
               # ZAPI gives a scalar value if there is only one. 
               $allowed_protocols = $ap_arrayref;
            }
            
            $vfilerhashref->{'allowed_protocols'} = $allowed_protocols; 
            $protohash{'vfiler'}            = $vfilerhashref->{'name'};;
            $protohash{'allowed_protocols'} = $allowed_protocols;
            $protohash{'node'}              = $node;
            push (@protoarray, \%protohash);
         }
      }

      foreach my $nisinfo_hashref (@{$nisinfo_arrayref}) {
         my $nisserver_arrayref = $nisinfo_hashref->{'nis-servers'};
         foreach my $nisserver_hashref (@{$nisserver_arrayref}) {
            my %nishash = ();
            my $server_arrayref = $nisserver_hashref->{'server'};
            my $nis_servers = "";
            if (ref ($server_arrayref) eq "ARRAY") {
               foreach my $ip (@{$server_arrayref}) {
                  $nis_servers = "$nis_servers $ip";
               }
            }
            else {
               $nis_servers = $server_arrayref;
            }
            $nishash{'vfiler'}            = $vfilerhashref->{'name'};;
            $nishash{'nis_servers'}       = $nis_servers;
            $nishash{'nis_domain_name'}   = $nisinfo_hashref->{'nis-domain-name'};
            $nishash{'node'}              = $node;
            push (@nisarray, \%nishash);
         }
      }
      foreach my $dnsinfo_hashref (@{$dnsinfo_arrayref}) {
         my $dnsserver_arrayref = $dnsinfo_hashref->{'dns-servers'};
         foreach my $dnsserver_hashref (@{$dnsserver_arrayref}) {
            my %dnshash = ();
            my $server_arrayref = $dnsserver_hashref->{'server'};
            my $dns_servers = "";
            if (ref ($server_arrayref) eq "ARRAY") {
               foreach my $ip (@{$server_arrayref}) {
                  $dns_servers = "$dns_servers $ip";
               }
            }
            else {
               $dns_servers = $server_arrayref;
            }
            $dnshash{'vfiler'}            = $vfilerhashref->{'name'};;
            $dnshash{'dns_servers'}       = $dns_servers;
            $dnshash{'dns_domain_name'}   = $dnsinfo_hashref->{'dns-domain-name'};
            $dnshash{'node'}              = $node;
            push (@dnsarray, \%dnshash);
         }
      }

      foreach my $vfstores_hashref (@{$vfstores_arrayref}) {
          my $vfstoreinfo_arrayref = $vfstores_hashref->{'vfstore-info'};
          foreach my $vfstoreinfo_hashref (@{$vfstoreinfo_arrayref}) {
            my %vfstorehash = ();
            $vfstorehash{'vfiler'}            = $vfilerhashref->{'name'};;
            $vfstorehash{'path'}              = $vfstoreinfo_hashref->{'path'};
            $vfstorehash{'is_etc'}            = $vfstoreinfo_hashref->{'is-etc'};
            $vfstorehash{'status'}            = $vfstoreinfo_hashref->{'status'};
            $vfstorehash{'node'}              = $node;
            push (@vfstoresarray, \%vfstorehash);
          }
      }

      foreach my $vfnet_hashref (@{$vfnet_arrayref}) {
          my $vfnetinfo_arrayref = $vfnet_hashref->{'vfnet-info'};
          foreach my $vfnetinfo_hashref (@{$vfnetinfo_arrayref}) {
            my %vfnethash = ();
            $vfnethash{'vfiler'}            = $vfilerhashref->{'name'};;
            $vfnethash{'ipaddress'}         = $vfnetinfo_hashref->{'ipaddress'};
            $vfnethash{'interface'}         = $vfnetinfo_hashref->{'interface'};
            $vfnethash{'netmask'}           = $vfnetinfo_hashref->{'netmask'};
            $vfnethash{'node'}              = $node;
            push (@vfnetarray, \%vfnethash);
          }
      }

      #foreach my $thiskey (@vfilerkeys) {
      #    # convert any "-" in the inputkeys to "_" to accomodate mysql table field.
      #    my $newkey = $thiskey;
      #    $newkey =~ s/-/_/g ;
      #    $newhash{$newkey} =  $vfilerhashref->{$thiskey};
      #}
      my %newhash = ();
      $newhash{'ipspace'}    = $vfilerhashref->{'ipspace'};
      $newhash{'name'}       = $vfilerhashref->{'name'};
      $newhash{'uuid'}       = $vfilerhashref->{'uuid'};
      foreach my $protohashref (@{protoarray}) {
         if ($protohashref->{'vfiler'} eq $vfilerhashref->{'name'}) {
            $newhash{'allowed_protocols'} = $protohashref->{'allowed_protocols'};
         }
      }
      $newhash{'node'}       = $node;
      push (@vfilerarray, \%newhash);
   }

   #print "DUMPING VFILERS\n";
   #print Dumper(\@vfilerarray);
   #print "DUMPING VFSTORE\n";
   #print Dumper(\@vfstoresarray);
   #print "DUMPING VFNET\n";
   #print Dumper(\@vfnetarray);

   my %returnhash;
   $returnhash{'vfiler'}   = \@vfilerarray;
   $returnhash{'vfdns'}    = \@dnsarray;
   $returnhash{'vfnis'}    = \@nisarray;
   $returnhash{'vfstore'}  = \@vfstoresarray;
   $returnhash{'vfnet'}    = \@vfnetarray;

   return \%returnhash;
}

1;
