package ConfigBroker::Engine::EngineDiscoverClients;

use strict;
use warnings;
use Tharn qw($Log script_result);
use Data::Dumper;
our @ISA = qw (ConfigBroker::Engine Exporter);
use ConfigBroker::CbUtils;
use Params::Validate qw(validate SCALAR OBJECT HASHREF BOOLEAN ARRAYREF UNDEF);
use Module::Common qw(validate_mixed);
use Options;
use Params::Common;
use Hostrec;
use NATE::ParamSet;
use NACL::C::Client;
use NACL::APISet::Host::CLI::Windows::default::ipconfig;

sub new
{
   my ($pkg, $configinfo) = @_;
   $Log->enter();
   my $class = ref($pkg) || $pkg ;
   my $self=$class->SUPER::new($configinfo);
   bless ($self, $class);

   $Log->exit();
   return $self;
}

sub run_discovery()
{
   my ($self) = @_;
   $Log->enter();

   my $ret = 0;

   my $get_fields_cmd = "SELECT typename FROM typedefn";
   my $fields = $self->{'dbh'}->selectcol_arrayref($get_fields_cmd);
   my $RG = &ConfigBroker::CbUtils::get_resource_group_object($self->{'dbh'}, $self->{'resourcegroup'});

   ### Get the clientname from the Options CLIENTFILE or CLIENTS or CLIENT in that order.
   my $Options = new Options;
   $Options->process();
   my $options_hash = $Options->get_hash();

   my @clients;
   @clients = @{$self->{'clients'}} if (defined $self->{'clients'});

   ### If any of the options params CLIENTFILE, CLIENTS, CLIENT are defined, 
   ### get them in that order of preference. These take precedence over what is passed in thru
   ### CB method.
   if ((defined $options_hash->{'CLIENTFILE'}) && ($options_hash->{'CLIENTFILE'} ne "")) {
      my $file = $options_hash->{'CLIENTFILE'};
      ### Process clients file to get the clients stuff
      if (open (FILE, "<$file"))
      {
         my @data = <FILE>;
         chomp @data;
         close(FILE);
         @clients = @data;
      }
   }
   elsif (defined ($options_hash->{'CLIENTS'}->[0])) {
      @clients = @{$options_hash->{'CLIENTS'}};
   }
   elsif (defined ($options_hash->{'CLIENT'}->[0])) {
      @clients = @{$options_hash->{'CLIENT'}};
   }

   if (! defined ($clients[0])) {
      $Log->comment("Names of clients to be discovered not provided\n");
      return 1;
   }
   my @clientinfo;  
   foreach my $thisclient (@clients) {
      ### Get whatever information about each client here
      ### for adding to the cb table.
      my $clientobj = NACL::C::Client->new(name => $thisclient);
      my $host = Hostrec->new(id=>$thisclient);
      my %clienthash;
      $Log->comment("DUMPING client $thisclient\n");
      $Log->comment(Dumper($host));
      my $iphashref = {};
      if ($host->hosttype() =~ /win/) {
         $iphashref = get_win_client_ipinfo($host);
      }
      else {
         $iphashref = get_client_ipinfo($clientobj);
      }
      $clienthash{'ipconfig'}    = $iphashref->{'ipconfig'}; 
      $clienthash{'ipv4'}        = $iphashref->{'ipv4'}; 
      $clienthash{'ipv6'}        = $iphashref->{'ipv6'}; 
      $clienthash{'client_name'} = $host->{'hostname'}; 
      $clienthash{'hosttype'}    = $host->{'hosttype'}; 
      $clienthash{'os'}          = $host->{'OS'}; 
      $clienthash{'os_version'}  = $host->{'OS_VER'}; 
      push (@clientinfo, \%clienthash);
   }  
   my $objtype = "client";
   if (scalar(@clientinfo) > 0) {
      $Log->debug("Adding objecttype $objtype\n");
      $RG->add_resources(resource_type => lc($objtype),
                         resourcearrayref => \@clientinfo);
   }

   $Log->exit();
   return $ret;
}

sub get_win_client_ipinfo {
    my $host = shift;
    $Log->debug("Creating apiset object");
    my $apiset = NACL::APISet->new(
       hostobj   => $host,
       category  => "Host",
       interface => "CLI",
       conntype  => "staf-persay",
       connid    => "login",
       set       => "Windows"
    );

    my $resp = $apiset->ipconfig();
    my $parsed    = $resp->{parsed_output};
    my $processed = $resp->{processed_output};
    my %ipinfo;
    $ipinfo{'ipconfig'} = $processed;
    #The parsed is an array of hashes.  We just need the one entry with IPV6/4 and take
    # the first entry of those;
    foreach my $confighashref (@{$parsed}) {
       if (defined $confighashref->{ipv6}) {
          $ipinfo{'ipv6'} = $confighashref->{ipv6}->[0];
       }
       if (defined $confighashref->{ipv4}) {
          $ipinfo{'ipv4'} = $confighashref->{ipv4}->[0];
       }
    }
    return \%ipinfo;
}

sub get_client_ipinfo {
    my $co = shift;
    my %ipinfo;
    my $conn = $co->hostrec()->connect(
        template => "local",
        conntype => "rlogin",
        conncmd => ["/bin/sh", "-i"]
    );
    my $command = "/sbin/ifconfig";
    eval {
         $ipinfo{'ipconfig'} = $conn->execute(
            command          => $command,
         );
    };
    if ($@) { die script_result($@); }

    $command = "/sbin/ifconfig  | grep -e \"inet6.*Link\"";
    my $response;
    eval {
         $response = $conn->execute(
             command          => $command,
         );
    };
    if ($@) { die script_result($@); }
    my @temp = split("Scope", $response);
    @temp    = split(" ", $temp[0]);
    $ipinfo{'ipv6'} = $temp[2];

    $command = "/sbin/ifconfig  | grep -e \"inet addr.*Bcast\"";
    eval {
         $response = $conn->execute(
             command          => $command,
         );
    };
    if ($@) { die script_result($@); }
    @temp = split("Bcast", $response);
    @temp    = split(":", $temp[0]);
    $ipinfo{'ipv4'}  = $temp[1];

    return \%ipinfo;
}

1;
