package ConfigBroker::Engine::EngineEnGenX;

use strict;
use warnings;
use DBI qw();
use Tharn qw($Log);
use Tharn;
use Data::Dumper;
our @ISA = qw (Exporter ConfigBroker::Engine);
use ConfigBroker::ResourceGroup;

sub new
{
   my ($pkg, $configinfo) = @_;
   $Log->enter();
   my $class = ref($pkg) || $pkg ;
   my $self=$class->SUPER::new($configinfo);
   bless ($self, $class);

   $Log->exit();
   return $self;
}

sub run_setup()
{
   my ($self) = @_;
   $Log->enter();

   my $ret = 0;
   my $Debug_level = 2;
   require EnGenX;
   my $egxobj = new EnGenX(file   => $self->{'configfile'},
               configbroker_group => $self->{'resourcegroup'},
               load_resources     => 1
             );
   $Log->comment("Running EnGenX ::config(): config environment using $self->{'configfile'} file\n");
   ### EnGenX returns TRUE (1) for success and FALSE (0) for failure.
   my $engine_rc = $egxobj->config();
   if (! $engine_rc) {
      $Log->error("EnGenX returned failure for config. ERROR...\n");
      $ret = 1;
   }
   if (! $ret) { 
      $Log->comment("Running EnGenX ::setup(): config environment using $self->{'configfile'} file\n");
      $engine_rc = $egxobj->setup(-debug_level => $Debug_level);
      if (! $engine_rc) {
         $Log->error("EnGenX returned failure for setup $ret. ERROR...\n");
         $ret = 1;
      }
   }
   if (! $ret) {
      $self->{'cleanupfile'} = $self->{'configfile'};
      my $rg_obj = &ConfigBroker::CbUtils::get_resource_group_object($self->{'dbh'}, $self->{'resourcegroup'});
      $rg_obj->update_cleanupfilename($self->{'cleanupfile'});
   }

   $Log->exit();
   return $ret;
}

sub run_cleanup()
{
   my ($self) = @_;
   $Log->enter();
   my $ret = 0;
   require EnGenX;
   my $egxobj = new EnGenX(file  => $self->{'cleanupfile'},
               );
   #$Log->comment("Running EnGenX ::deconfig(): using $self->{'cleanupfile'} file\n");
   #$egxobj->deconfig();
   $Log->comment("Running EnGenX ::cleanup(): using $self->{'cleanupfile'} file\n");
   $egxobj->cleanup();

   $Log->exit();
   return $ret;
}
   
1;
