package ConfigBroker::ResourceGroup;

use strict;
use warnings;
use DBI qw();
use Tharn qw($Log);
use Data::Dumper;
our @ISA = qw (Exporter);
#use lib "/u/veena/p4/test/lib";
use ConfigBroker::CbUtils;
use ConfigBroker::Schema;
use Params::Validate qw(validate SCALAR OBJECT HASHREF BOOLEAN ARRAYREF UNDEF);
use Module::Common qw(validate_mixed);


## @summary ResourceGroup	
## @author veena@netapp.com
## @status public
## @pod here

=head1 NAME

  ResourceGroup - class for accessing resourcegroup objects

=head1 SYNOPSIS

  use ResourceGroup;

  my $CB = new ConfigBroker();
  my $RG = $CB->create_resource_group(<params>);
  my $RG = $CB->discover_and_create_resource_group(<params>);

=head1 DESCRIPTION

  class module to call resourcegroup object methods.

=cut


sub new
{
   $Log->enter();
   my ($pkg, %res_info) = @_;
   my $class = ref($pkg) || $pkg ;
   my $self = {};
   $self->{'dbh'}   = $res_info{'dbh'};
   $self->{'name'}  = $res_info{'name'};
   bless ($self, $class);
   $self->{'reuse'}          = "REUSE";
   $self->{'share'}          = "SHARE";
   $self->{'dsn'}            = "";
   $self->{'configfile'}     = "";
   $self->{'configengine'}   = "";
   $self->{'cleanupfile'}    = "";
   $self->{'cmdlinestring'}  = "";
   $self->{'status'}         = "";

   my $group = &ConfigBroker::CbUtils::get_resource_group($self->{'dbh'}, $self->{'name'});
   
   if (defined $group)
   {
      my %resgroup = %{$group};
      if ($resgroup{'name'} eq $self->{'name'})
      {
         $Log->debug("RESOURCE GROUP $self->{'name'} EXISTS.\n"); 
         #if ($resgroup{'share'} eq "SHARE") {
         #   $Log->debug("Resource Group $self->{'name'} can be SHARED\n"); 
            $self->{'reuse'} = $resgroup{'reuse'};
            $self->{'share'} = $resgroup{'share'};
            $self->{'dsn'}   = $resgroup{'dsn'};
            $self->{'configfile'}     = $resgroup{'configfile'};
            $self->{'configengine'}   = $resgroup{'configengine'};
            $self->{'cleanupfile'}    = $resgroup{'cleanupfile'};
            $self->{'cmdlinestring'}  = $resgroup{'cmdlinestring'};
            $self->{'status'}         = $resgroup{'status'};
         #}
         #else {
         #   $Log->comment("Resource Group $self->{'name'} exists and CANNOT be SHARED\n"); 
         #   $Log->error("EXITING\n");
         #   exit 1;
         #}
      }
      else
      {
         $Log->error("$resgroup{'name'} is not $self->{'name'}. Something is wrong.\n"); 
      }
   }
   else
   {
      $self->{'reuse'}           = $res_info{'reuse'} if defined ($res_info{'reuse'});
      $self->{'share'}           = $res_info{'share'} if defined ($res_info{'share'});
      $self->{'dsn'}             = $res_info{'dsn'}   if defined ($res_info{'dsn'});
      $self->{'configfile'}      = $res_info{'configfile'}   if defined ($res_info{'configfile'});
      $self->{'configengine'}    = $res_info{'configengine'} if defined ($res_info{'configengine'});
      $self->{'cleanupfile'}     = $res_info{'cleanupfile'}  if defined ($res_info{'cleanupfile'});
      $self->{'cmdlinestring'}   = $res_info{'cmdlinestring'}  if defined ($res_info{'cmdlinestring'});
      $self->{'status'}          = $res_info{'status'}         if defined ($res_info{'status'});
      $Log->debug (" Resource Group $self->{'name'} doesn't exist already.  Creating new\n"); 
      $self->_add_resource_group_to_db();
   }

   $Log->exit();
   return $self;
}

sub get_group  
{
   my $self = shift;
   my $group = &ConfigBroker::CbUtils::get_resource_group($self->{'dbh'}, $self->{'name'}); 
   return $group;
}

sub _add_resource_group_to_db
{
   my $self = shift;
   $Log->enter();

   #$Log->comment(Dumper($self));
   eval {
      my $insert_cmd = "INSERT INTO resourcegroup (name, reuse, share, dsn, configfile, configengine, cleanupfile, cmdlinestring, status) " .
                       "VALUES('$self->{'name'}', '$self->{'reuse'}', '$self->{'share'}', '$self->{'dsn'}', '$self->{'configfile'}', '$self->{'configengine'}', '$self->{'cleanupfile'}', '$self->{'cmdlinestring'}', '$self->{'status'}')";
      $self->{'dbh'}->do($insert_cmd);
   };
   if ($@) {
      $Log->warn("Error inserting row into table \"resourcegroup\". Error is $@");
   }

   $Log->exit();
}

sub edit_resourcegroup                                       ## This is for GUI
{
   my ($self, %opts) = 
		validate_mixed(@_,
			[{type => OBJECT},],
			{newstatus => {type => SCALAR},
			newreuse => {type =>SCALAR},
			newshare=> {type => SCALAR},},);
   my $ret = 0;
   $Log->enter();
   $self->{'status'} = $opts{'newstatus'} if (defined $opts{'newstatus'});
   $self->{'share'} = $opts{'newshare'} if (defined $opts{'newshare'});
   $self->{'reuse'} = $opts{'newreuse'} if (defined $opts{'newreuse'});
   eval {
      my $update_cmd = "UPDATE config " .
                       "SET status=\'$self->{'status'}\' ,"."share=\'$self->{'share'}\' ,"."reuse=\'$self->{'reuse'}\' ".
                       "WHERE name=\'$self->{'name'}\'";
      $Log->debug($update_cmd);
      $self->{'dbh'}->do($update_cmd);
   };
   if ($@) {
      $Log->warn("Error updating status column into table \"config\". Error is $@");
      $ret = 1;
   }
   $Log->exit();
   return $ret;
}


sub update_cleanupfilename
{
   my ($self, $cleanupfile) = @_;
   $Log->enter();
   $cleanupfile = $self->{'cleanupfile'} if (!defined $cleanupfile);
   eval {
      my $update_cmd = "UPDATE resourcegroup " .
                       "SET cleanupfile=\'$cleanupfile\' ".
                       "WHERE name=\'$self->{'name'}\'";
      $self->{'dbh'}->do($update_cmd);
   };
   if ($@) {
      $Log->warn("Error updating cleanup file column into table \"resourcegroup\". Error is $@");
   }
   $Log->exit();
   return 0;
}



sub update_status
{
   my ($self, %opts) = 
		validate_mixed(@_,
			[{type => OBJECT},],
			{newstatus => {type => SCALAR},},);
   my $ret = 0;
   $Log->enter();
   $self->{'status'} = $opts{'newstatus'} if (defined $opts{'newstatus'});
   eval {
      my $update_cmd = "UPDATE resourcegroup " .
                       "SET status=\'$self->{'status'}\' ".
                       "WHERE name=\'$self->{'name'}\'";
      $self->{'dbh'}->do($update_cmd);
   };
   if ($@) {
      $Log->warn("Error updating status column into table \"config\". Error is $@");
      $ret = 1;
   }
   $Log->exit();
   return $ret;
}


sub reserve
{
   my ($self, %opts) = 
			validate_mixed(@_,
			[{type => OBJECT},],
			{reserve_code => {type => SCALAR, optional =>1, default => 0},},); 
   my $ret = 0;
   $Log->enter();
   my $code  = "INUSE_" . "$opts{'reserve_code'}" ;
   $ret = $self->update_status(newstatus => "$code");
   $Log->exit();
   return $ret;
}

sub unreserve
{
      my ($self, %opts) =
                        validate_mixed(@_,
                        [{type => OBJECT},],
                        {reserve_code => {type => SCALAR, optional =>1, default => 0},},);
   my $ret = 0; 
   $Log->enter();
   my $verify_code = "INUSE_" . "$opts{'reserve_code'}";
   if ( $self->{'status'} ne $verify_code) {

      $Log->error("reservation code supplied is not the same used to reserve the group\n"); 
      $Log->error("Could not unreserve\n");
      $ret = 1;
   }
   else {
      
		$ret = $self->update_status(newstatus => "READY");
   }
   $Log->exit();
   return $ret;
}

sub add_resources()
{  
   $Log->enter();
    my ($self, %opts) =
          validate_mixed(@_,
                  [{type => OBJECT},],
                  {resource_type => {type => SCALAR,},
                   resourcearrayref => {type => ARRAYREF, optional => 1,},},);
 
   my $typetable = $opts{resource_type}; 
   my $crossreftable = "$typetable" . "crossref";

   my $crossreftrig       = "$typetable" . "crossreftrig";
   my $crossreftrigupdate = "$typetable" . "crossreftrigupdate";

   ### Now add the trigger to insert into the crossref table when new resources are added to the resource table
   my $tempname = $self->{'name'};
   my $triggercmd =  "CREATE TRIGGER $crossreftrig AFTER INSERT ON $typetable FOR EACH ROW INSERT INTO $crossreftable (resourceid, resourcegroup) VALUES (NEW.uniq_resource_id, \'$tempname\')"; 
   $Log->debug("CREATING AFTER INSERT TRIGGER $triggercmd\n");
   $self->{'dbh'}->do($triggercmd);

   ### Now add the trigger to insert into the crossref table when resources are updated in the resource table
   ### In case of already existing resource being added to new RG, we just update the referece count in the resource table.
   ### This is to handle that case
   $triggercmd =  "CREATE TRIGGER $crossreftrigupdate AFTER UPDATE ON $typetable FOR EACH ROW INSERT INTO $crossreftable (resourceid, resourcegroup) VALUES (NEW.uniq_resource_id, \'$tempname\')"; 
   $Log->debug("CREATING AFTER UPDATE TRIGGER $triggercmd\n");
   $self->{'dbh'}->do($triggercmd);

   $self->_add_resource_bulk_mysql($opts{resource_type}, $opts{resourcearrayref});

   my $droptriggercmd =  "DROP TRIGGER $crossreftrig"; 
   $Log->debug("DROPPING AFTER INSERT TRIGGER\n");
   $self->{'dbh'}->do($droptriggercmd);
   $droptriggercmd =  "DROP TRIGGER $crossreftrigupdate"; 
   $Log->debug("DROPPING AFTER UPDATE TRIGGER\n");
   $self->{'dbh'}->do($droptriggercmd);

   $Log->exit();
   return 0;
}

sub _add_resource_bulk_mysql
{
   my ($self, $restype, $resourcearrayref) = @_;
   $Log->enter();
   my $addbulk;
   my $count = 0;
   my $get_fields_cmd = "SELECT typefields FROM typedefn ".
                        "WHERE typename=\'$restype\'";
   $Log->debug ("GETFIELDSCMD is $get_fields_cmd\n");
   my $fields = $self->{'dbh'}->selectcol_arrayref($get_fields_cmd);
   my $fieldzero = $fields->[0];
   my @reskeys = split(" ", $fieldzero);

   my $numres = scalar(@{$resourcearrayref});

   foreach (@{$resourcearrayref})
   {
      my %res_input = %{$_};
      my %res_info;
      #### Get everything into a hash with lowercase keys regardless of what came in..
      foreach my $inputkey (keys(%res_input))
      {
         my $nocase = lc($inputkey);
         $nocase = &ConfigBroker::Schema::convert_reserved_word($nocase) if(&ConfigBroker::Schema::is_reserved_word($nocase));
         $res_info{$nocase} = $res_input{$inputkey};
         #$Log->comment ("ENGINE KEY IS $inputkey; ENGINE VALUE IS $res_info{$nocase}\n");
      }
      ### We use "name" column in the db to store the name of the resource (e.g. vol1, "myaggr1", "vol002", etc)
      ### NACL stores them in the attr field called by the resource type (e.g: 'volume' => "vol1")
      ### take care of this discrepany.
      $res_info{$restype} = $res_info{'name'} if ((defined $res_info{'name'}) && (! defined $res_info{$restype}));
      $res_info{'name'} = $res_info{$restype} if (! defined $res_info{'name'});
      $res_info{'name'} = $res_info{'port'}   if ( (! defined $res_info{'name'}) && ($restype eq "networkport"));
      $res_info{'name'} = $res_info{'lif'}    if ( (! defined $res_info{'name'}) && ($restype eq "networkinterface"));
      $res_info{'name'} = $res_info{'policyname'}    if ( (! defined $res_info{'name'}) && ($restype eq "exportpolicy"));
      $res_info{'name'} = $res_info{'node'}   if ( (! defined $res_info{'name'}) && ($restype eq "cluster"));
      $res_info{'name'} = $res_info{'workload'}   if ( (! defined $res_info{'name'}) && ($restype eq "qosworkload"));
      $res_info{'name'} = $res_info{'policy-name'}   if ( (! defined $res_info{'name'}) && ($restype eq "quotapolicy"));
      $res_info{'name'} = $res_info{'policy'}   if ( (! defined $res_info{'name'}) && ($restype eq "snapshotpolicy"));
      $res_info{'name'} = $res_info{'share_name'}   if ( (! defined $res_info{'name'}) && ($restype eq "vservercifsshare"));
      $res_info{'name'} = $res_info{'policyname'}   if ( (! defined $res_info{'name'}) && ($restype eq "exportpolicyrule"));
      $res_info{'licenselimit'} = $res_info{'limit'}   if ( (defined $res_info{'limit'}) && ($restype eq "license"));
      $res_info{'group_ref_count'} = 1;

      ### set resource_priority to 0 (equal priority - default) if not set
      $res_info{'resource_priority'} = 0 if((! defined $res_info{'resource_priority'}) || ($res_info{'resource_priority'} eq ""));

      ### set resource_status if not set
      $res_info{'resource_status'} = "READY" if((! defined $res_info{'resource_status'}) || ($res_info{'resource_status'} eq ""));

      if ($count == 0)
      {
         $addbulk = qq{ INSERT INTO $restype };
         my $numkeys = scalar(@reskeys);
         my $keycount = 0;
         my $fieldnames = "(" ; 
         my $fieldvalues = "VALUES (" ; 
         foreach my $thiskey (@reskeys)
         {
            $keycount++ ;
            next if ($thiskey eq "dbh");
            #next if ($thiskey eq "type");
            ### Initialize the value to an empty string if it is not defined.
            $res_info{$thiskey} = "" if (! defined $res_info{$thiskey});
            $res_info{$thiskey} =~ s/(\'|\!|\\)/\\$1/g if(defined $res_info{$thiskey});

            if ($thiskey eq "resourcegroup") {
               $fieldnames  = $fieldnames  . $thiskey ;
               $fieldvalues = $fieldvalues . "\'$self->{'name'}\' " ;
            }
            else {
               $fieldnames  = $fieldnames  . $thiskey ;
               my $value = $res_info{$thiskey};
               my $ref = ref $value;
               if ($ref eq 'ARRAY') {
                  #$value = Dumper($value);
                  $value = "@$value";
               }
               $fieldvalues = $fieldvalues . "\'$value\'" ;

            }
            if ($numkeys == $keycount) {
               $fieldnames  = $fieldnames  . ") " ;
               $fieldvalues = $fieldvalues . ") " ;
            }
            else {
               $fieldnames  = $fieldnames  . ", " ;
               $fieldvalues = $fieldvalues . ", " ;
            }
         } #for each field from the table columnlist 
         $addbulk .= $fieldnames;
         $addbulk .= $fieldvalues;
      }
      else {
         my $numkeys = scalar(@reskeys);
         my $keycount = 0;
         my $fieldvalues = "(" ;
         foreach my $thiskey (@reskeys)
         {
            $keycount++ ;
            next if ($thiskey eq "dbh");
            #next if ($thiskey eq "type");
            ### Initialize the value to an empty string if it is not defined.
            $res_info{$thiskey} = "" if (! defined $res_info{$thiskey});
            $res_info{$thiskey} =~ s/(\'|\!|\\)/\\$1/g if(defined $res_info{$thiskey});

            if ($thiskey eq "resourcegroup") {
               $fieldvalues .= "\'$self->{'name'}\' " ;
            }
            else {
               #$fieldvalues .= "\'$res_info{$thiskey}\'" ;
               my $value = $res_info{$thiskey};
               my $ref = ref $value;
               if ($ref eq 'ARRAY') {
                  #$value = Dumper($value);
                  $value = "@$value";
               }
               $fieldvalues = $fieldvalues . "\'$value\'" ;
            }
            if ($numkeys == $keycount) {
               $fieldvalues .= ") " ;
            }
            else {
               $fieldvalues .= ", " ;
            }
         } #for each field from the table columnlist
         $addbulk .= $fieldvalues;
      }
      $count ++;
      if ($count < $numres) {
        $addbulk .= ", " ;
      }
   } # for each input resource
   
   if (defined($addbulk) && $addbulk ne "")
   {
      ### add on duplicate key clause.
      $addbulk .= " ON DUPLICATE KEY UPDATE group_ref_count=group_ref_count+1";

      #my $rg = $self->{'name'};
      #my $tablecrossref = $restype . "crossref";
      #$addbulk .= " ON DUPLICATE KEY INSERT INTO $tablecrossref (resourceid, resourcegroup) VALUES(uniq_resource_id, \'$rg\')";

      $Log->debug("ADDBULK IS $addbulk\n");
      eval {
         $self->{'dbh'}->do($addbulk);
      };
      if ($@) {
         $Log->warn("Error adding resources. Error is $@");
      }
   }
   $Log->exit();
   return 0;
}
   
sub _add_resource_bulk_sqlite
{
   my ($self, $restype, $resourcearrayref) = @_;
   $Log->enter();

   my $addbulk;
   my $count = 0;
   my $get_fields_cmd = "SELECT typefields FROM typedefn ".
                        "WHERE typename=\'$restype\'";
   my $fields = $self->{'dbh'}->selectcol_arrayref($get_fields_cmd);
   my @reskeys = split(" ", @$fields[0]);

   foreach (@{$resourcearrayref})
   {
      my %res_input = %{$_};
      my %res_info; 
      #$res_info{'resourcegroup'} = $self->{'name'};
      #### Get everything into a hash with lowercase keys regardless of what came in..
      foreach my $inputkey (keys(%res_input))
      {
         my $nocase = lc($inputkey);
         $nocase = &ConfigBroker::Schema::convert_reserved_word($nocase) if(&ConfigBroker::Schema::is_reserved_word($nocase));
         $res_info{$nocase} = $res_input{$inputkey};
         #$Log->comment ("ENGINE KEY IS $inputkey; ENGINE VALUE IS $res_info{$nocase}\n");
      }
      ### We use "name" column in the db to store the name of the resource (e.g. vol1, "myaggr1", "vol002", etc)
      ### NACL stores them in the attr field called by the resource type (e.g: 'volume' => "vol1")
      ### take care of this discrepany.
      $res_info{'name'} = $res_info{$restype} if (! defined $res_info{'name'});
      $res_info{'name'} = $res_info{'port'}   if ( (! defined $res_info{'name'}) && ($restype eq "networkport"));
      $res_info{'name'} = $res_info{'lif'}    if ( (! defined $res_info{'name'}) && ($restype eq "networkinterface"));
      $res_info{'name'} = $res_info{'policyname'}    if ( (! defined $res_info{'name'}) && ($restype eq "exportpolicy"));
      $res_info{'name'} = $res_info{'node'}   if ( (! defined $res_info{'name'}) && ($restype eq "cluster"));
      $res_info{'name'} = $res_info{'workload'}   if ( (! defined $res_info{'name'}) && ($restype eq "qosworkload"));
      $res_info{'name'} = $res_info{'policy-name'}   if ( (! defined $res_info{'name'}) && ($restype eq "quotapolicy"));
      $res_info{'name'} = $res_info{'policy'}   if ( (! defined $res_info{'name'}) && ($restype eq "snapshotpolicy"));
      $res_info{'name'} = $res_info{'share_name'}   if ( (! defined $res_info{'name'}) && ($restype eq "vservercifsshare"));
      $res_info{'name'} = $res_info{'policyname'}   if ( (! defined $res_info{'name'}) && ($restype eq "exportpolicyrule"));
      $res_info{'licenselimit'} = $res_info{'limit'}   if ( (defined $res_info{'limit'}) && ($restype eq "license"));

      ### set resource_priority to 0 (equal priority - default) if not set
      $res_info{'resource_priority'} = 0 if((! defined $res_info{'resource_priority'}) || ($res_info{'resource_priority'} eq ""));

      ### set resource_status if not set
      $res_info{'resource_status'} = "READY" if((! defined $res_info{'resource_status'}) || ($res_info{'resource_status'} eq ""));

      if ($count == 0)
      {
         $addbulk = qq{ INSERT INTO $restype };
         $addbulk .= "SELECT";
         my $numkeys = scalar(@reskeys);
         my $keycount = 0;
         foreach my $thiskey (@reskeys)
         {
            $keycount++ ;
            next if ($thiskey eq "dbh");
            #next if ($thiskey eq "type");
            ### Initialize the value to an empty string if it is not defined. 
            $res_info{$thiskey} = "" if (! defined $res_info{$thiskey});
            $res_info{$thiskey} =~ s/'//g if(defined $res_info{$thiskey});
            $res_info{$thiskey} =~ s/!//g if(defined $res_info{$thiskey});

            if ($thiskey eq "resourcegroup") {
               $addbulk .= qq{ \'$self->{'name'}\' as \'$thiskey\' };
            }
            else
            {
               my $value = $res_info{$thiskey};
               my $ref = ref $value;
               if ($ref eq 'ARRAY') {
                  #$value = Dumper($value);
                  $value = "@$value";
               }
               #$addbulk .= qq{ \'$res_info{$thiskey}\' as \'$thiskey\' };
               $addbulk .= qq{ \'$value\' as \'$thiskey\' };
            }
            $addbulk .= ", " if ($keycount < $numkeys);
         }
      }
      else
      {
         $addbulk .= " UNION SELECT ";
         my @values;
         foreach my $thiskey (@reskeys)
         {
            next if ($thiskey eq "dbh");
            #next if ($thiskey eq "type");
            ### Initialize the value to an empty string if it is not defined. 
            $res_info{$thiskey} = "" if (! defined $res_info{$thiskey});
            $res_info{$thiskey} =~ s/'//g if(defined $res_info{$thiskey});
            $res_info{$thiskey} =~ s/!//g if(defined $res_info{$thiskey});
            if ($thiskey eq "resourcegroup") {
               push(@values, $self->{'name'});
               next;
            }
            my $value = $res_info{$thiskey};
            my $ref = ref $value;
            if ($ref eq 'ARRAY') {
               $value = "@$value";
            }
            push(@values, $value);
         }
         $addbulk .= join(", ", map { $self->{'dbh'}->quote($_) } @values );
      }
      $count ++;
      if ($count == 450)
      {
         $self->{'dbh'}->do($addbulk);
         $count = 0;
         $addbulk = "";
      }
   }
   #$Log->comment("The DB command to add resources in bulk is :\n$addbulk\n");
   if ($addbulk ne "")
   {
      $Log->debug("ADD CMD IS $addbulk\n");
      eval { 
         $self->{'dbh'}->do($addbulk);
      };

      if ($@) {
         $Log->warn("Error adding resources to the table \"$restype\". Error is $@");
      }
      else {
         $Log->debug("Successfully added resources to \"$restype\".");
      }

   }
   $Log->exit();
   return 0;
}

sub add_resource()
{
   my ($self, %opts) = 
			validate_mixed(@_,
				[{type => OBJECT},],
				{type => {type => SCALAR,},},);
   $Log->enter();

   ### the caller will determine if the resource exist and administer the
   ### reuse/share policy before deciding to create a new one.
   ### so assume when we are here, a resource needs to be created.
   ### create a new resource and return the resource object to the caller.

   ### validate the input and create the resource group.

   my $restype = $opts{'type'};
   my @resarray; 
   push (@resarray, \%opts);
  
   $self->_add_resource_bulk_mysql($restype, \@resarray); 
   $Log->exit(); 
   return 0;
}

sub verify_resource_exist()
{
   my ($self, %opts) = 
			validate_mixed(@_,
			[{type => OBJECT},],
			{restype => {type => SCALAR,},
			resname => {type => SCALAR,},},);
   $Log->enter();

   if (&ConfigBroker::CbUtils::verify_resource_exists($self->{'dbh'}, $opts{'restype'}, $opts{'resname'}, $self->{'name'}))
   {
      $Log->debug("Resource $opts{'resname'} exists\n");
      return 1;
   }
   $Log->debug("Resource $opts{'resname'} does not exist\n");
   $Log->exit();
   return 0;
}

### Get all resources of a specific type 
### possibly narrow down using paramhash by mactching specific colomn values.
### return an array reference to an array of hashes.
sub get_resources
{
   $Log->enter();
   my ($self, %opts) =
          validate_mixed(@_,
                  [{type=> OBJECT},],
                  {count         => {type => SCALAR, optional => 1,},
                   randomize     => {type => SCALAR, optional => 1, default => 0,},
                   offset        => {type => SCALAR, optional => 1, default => 0,},
                   type          => {type => SCALAR},
                   match_filters => {type => HASHREF, optional => 1,},
                   regex_filters => {type => HASHREF, optional => 1,},},);

   my $restype = $opts{'type'};
   my $maxrows = $opts{'count'} ? $opts{'count'} : 0;
   my $filters = ();   
   $filters = $opts{'match_filters'} if ($opts{'match_filters'});
   my $regex_filters = ();
   $regex_filters = $opts{'regex_filters'} if ($opts{'regex_filters'});
  
   my $resarrayref = &ConfigBroker::CbUtils::_get_resources_arrayref_from_db(resource_group => $self->{'name'}, dbh => $self->{'dbh'}, table => $restype, resource_count => $maxrows, exact_filters => $filters, regex_filters => $regex_filters, randomize => $opts{'randomize'}, offset => $opts{'offset'});
   $Log->exit();
   return $resarrayref;
}

### Get an iterator for all resources of a specific type 
### possibly narrow down using paramhash by mactching specific colomn values.
### returns an array reference to an array of hashes that can be looped through
### or fetched more .
sub get_resources_iterator_PENDING 
{
   my ($self, $restype, $maxrows, $paramhash ) = @_;
   $Log->enter();

   ######## TO DO #################
   ##### Pending installation of Class::DBI module to std perl installation #########
   #Class::DBI->Connection($self->{'dsn'}); 
   #Class::DBI->table($restype);
   #my @objs = Class::DBI->retrieve_all();
   $Log->exit();
   return 0;
}

sub get_resource_NACL_objects
{
   my ($self, %opts) = 
		 validate_mixed(@_,
                  [{type=> OBJECT},],
                  {count         => {type => SCALAR, optional => 1,},
                   randomize     => {type => SCALAR, optional => 1, default => 0,},
                   offset        => {type => SCALAR, optional => 1, default => 0,},
                   type          => {type => SCALAR},
                   match_filters => {type => HASHREF, optional => 1,},
                   regex_filters => {type => HASHREF, optional => 1,},},);
   $Log->enter();

   my $restype = $opts{'type'};
   my $maxrows = 0;
   $maxrows = $opts{'count'} if (defined $opts{'count'});
   my $filters = ();
   $filters = $opts{'match_filters'} if (defined $opts{'match_filters'});
   my $regex_filters = ();
   $regex_filters = $opts{'regex_filters'} if (defined $opts{'regex_filters'});
   $maxrows = 0 if (! defined $maxrows);

   my @objs;
   my $resource_arrayref = &ConfigBroker::CbUtils::_get_resources_arrayref_from_db(resource_group => $self->{'name'}, dbh => $self->{'dbh'}, table => $restype, resource_count => $maxrows, exact_filters => $filters, regex_filters => $regex_filters, randomize => $opts{'randomize'}, offset => $opts{'offset'});

   if (scalar(@{$resource_arrayref})) {
      my $node = NACL::C::Node->find();
      foreach my $resource (@{$resource_arrayref}) {
         # Create NACL objects
         #my $obj = $self->_create_nacl_obj(
         my $obj = &ConfigBroker::CbUtils::_create_nacl_obj(
            'type'     => $restype,
            'resource' => $resource,
            'node'     => $node
         );
         push(@objs, $obj);
      }
   } 
   else {
      $Log->error("Could not create NACL object for resource\n"); 
      exit 1;
   }

   ### TO DO - Other category of resources i.e. non cluster controller ones like switches etc.

   $Log->exit();
   return @objs;
}

###############################################################################
# Helper method to create a new instance of a specific NACL obj.
###############################################################################
sub _create_nacl_obj_obsolete {
    my ($self, %args) = @_;
    $Log->enter();

    # Locals
    my $type     = undef;
    my $node     = undef;
    my $resource = undef;

    # Assign values
    if (!defined($type = $args{'type'})) {
        NATE::Exceptions::Argument->throw("Missing required field 'type'");
    }
    if (!defined($resource = $args{'resource'})) {
        NATE::Exceptions::Argument->throw(
            "Missing required field 'resource'");
    }
    if (!defined($node = $args{'node'})) {
        NATE::Exceptions::Argument->throw("Missing required field 'node'");
    }

    # Switch/Case implementation
    if ($type =~ /aggregate/) {
        my $aggr = $resource->{'name'};
        return NACL::C::StorageAggregate->new(
            'command_interface' => $node,
            'aggregate'         => $aggr
        );
    } elsif ($type =~ /vserver/) {
        return NACL::C::Vserver->new(
            'command_interface' => $node,
            'vserver'           => $resource->{'name'}
        );
    } elsif ($type =~ /volume/) {
        my $vserver = $resource->{'vserver'};
        my $vol     = $resource->{'name'};
        #return NACL::C::Volume->new(
        return NACL::CS::Volume->fetch(
            'command_interface' => $node,
            'volume'            => $vol,
            #'vserver'           => $vserver,
        );
    } elsif ($type =~ /lun/) {
        my $vol = $resource->{'volume'};
        my $lun = $resource->{'NAME'};
        return NACL::C::Lun->new(
            'command_interface' => $node,
            'vserver'           => $resource->{'vserver'},
            'path'              => "/vol/$vol/$lun"
        );
    } elsif ($type =~ /igroup/) {
        my $vserver = $resource->{'vserver'};
        my $igroup  = $resource->{'name'};
        NACL::C::LunIgroup->new(
            command_interface => $node,
            vserver           => $vserver,
            igroup            => $igroup,
        );
    } else {
        NATE::BaseException->throw(
            "Type not recognized or supported '$type'");
    }

} ## end sub _get_nacl_obj

### Get a specific resource by name and resourcetype
### and return a NACL object back.
sub get_resource_NACL_object_by_name
{
   my ($self, $restype, $resname) = @_;

}

##########
## Delete all resources of given type with filters matching in this resource group
#########
sub delete_resources
{
   my ($self, %opts) =
          validate_mixed(@_,
                  [{type=> OBJECT},],
                  {type => {type => SCALAR,},
                   match_filters => {type => HASHREF, optional => 1,},
                   regex_filters => {type => HASHREF, optional => 1,},},);

   my $type = $opts{'type'};
   my $filters = $opts{'match_filters'} if (defined $opts{'match_filters'});
   my $regex_filters = $opts{'regex_filters'} if (defined $opts{'regex_filters'});

   my $crossreftable = $type . "crossref";
   my $crossref_matchstring = "WHERE $crossreftable.resourceid=$type.uniq_resource_id ".
                              "AND $crossreftable.resourcegroup=\'$self->{'name'}\' ";
   my $matchstring = "";

   if ($filters)
   {
      my %matchhash  = %{$filters};
      foreach my $thiskey (keys (%matchhash)) {
         $thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         # is this an array attribute?
         my $value = $matchhash{$thiskey};
         my $isarray = &ConfigBroker::Schema::is_array_attribute($type, $thiskey, $self->{'dbh'});
         if ($isarray) {
            # we store the arrays as as " " separated string in the DB.
            # so the value will have a " " either in the front or the back
            my $compare_string = "($thiskey=\' $value\' OR $thiskey=\'$value \' OR $thiskey=\'$value\')";
            $matchstring = "$matchstring" . "AND $compare_string " ;
         }
         else {
            $matchstring = "$matchstring" . "AND $thiskey=\'$value\' " ;
         }
      }
   }

   if ($regex_filters)
   {
      my %matchhash = %{$regex_filters};
      foreach my $thiskey (keys (%matchhash)) {
         $thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         $matchstring = "$matchstring " . "AND $thiskey LIKE \'$matchhash{$thiskey}\' " ;
      }
   }

   my $ret = 0;
   my $delete_cmd = "DELETE $crossreftable FROM $crossreftable ".
                    "JOIN $type ON $crossreftable.resourceid=$type.uniq_resource_id " .
                    "$crossref_matchstring ".
                    "$matchstring";
   my $update_cmd = "UPDATE $type ".
                    "JOIN $crossreftable ON $type.uniq_resource_id=$crossreftable.resourceid " .
                    "SET group_ref_count=group_ref_count-1 ".
                    "$crossref_matchstring ".
                    "$matchstring";

   eval {
         $Log->debug ("Updating resource reference count in resource table before the delete\n");
         $Log->debug ("UPDATE CMD IS $update_cmd\n");
         $self->{'dbh'}->do($update_cmd);
   };
   if ($@) {
      $Log->error("Error update on the table \"$type\". Error is $@");
      $ret = 1;
   } 
   eval {
      $Log->debug("Deleting the resourcegroup reference from the ref table\n");
      $Log->debug("DELETE CMD IS $delete_cmd\n");
      $self->{'dbh'}->do($delete_cmd);
   };
   if ($@) {
      $Log->error("Error deleting from the table \"$crossreftable\". Error is $@");
      $ret = 1;
   }
   if ($ret == 1) {
      ### If the resource still had references to other groups, update the count.
      eval {
         my $update_cmd = "UPDATE $type ".
                    "JOIN $crossreftable ON $type.uniq_resource_id=$crossreftable.resourceid " .
                    "SET group_ref_count=group_ref_count+1 ".
                    "WHERE $type.uniq_resource_id=$crossreftable.resourceid ".
                    "$matchstring";
         $Log->debug("UPDATE CMD IS $update_cmd\n");
         $self->{'dbh'}->do($update_cmd);
      };
      if ($@) {
         $Log->error("Error reverting earlier update on the table \"$type\". Error is $@");
         $ret = 1;
      }
   }
   #### Now go ahead and delete resources that do not have any groups associated with it in the resource table.
   else{
      $Log->debug("Deleting resources from $type where group_ref_count is now 0 \n");
      eval {
         my $delete_cmd = "DELETE FROM $type ".
                          "WHERE group_ref_count=0";
         $self->{'dbh'}->do($delete_cmd);
      };
      if ($@) {
         $Log->debug("Error deleting resource with group_ref_count=0 from the table \"$type\". Error is $@");
      }
   }

   $Log->exit();
   return $ret;
}

sub delete_all_resources_from_group
{
   my ($self) = @_;
   $Log->enter();

   ### get the various tables in the database 
   my $get_types_cmd = "SELECT typename FROM typedefn ";
   my $types = $self->{'dbh'}->selectcol_arrayref($get_types_cmd);

   ### for each table type (vol, aggr, etc. etc.) delete all the rows pertaining to this resource group
   my @tabletypes = @{$types}; 
   foreach my $table (@tabletypes) {
      $self->delete_resources_for_type_from_group(type=>$table); 
   }
   $Log->exit();
   return 0;
}

sub delete_resources_from_db
{
   my ($self) = @_;
   $Log->enter();

   ### get the various tables in the database 
   my $get_types_cmd = "SELECT typename FROM typedefn ";
   my $types = $self->{'dbh'}->selectcol_arrayref($get_types_cmd);

   ### for each table type (vol, aggr, etc. etc.) delete all the rows pertaining to this resource group
   my @tabletypes = @{$types}; 
   foreach my $table (@tabletypes) {
      $self->delete_resources_for_type_from_group(type=>$table); 
   }
   $Log->exit();
   return 0;
}

sub delete_resources_for_type_from_group
{
   my ($self, %opts) = 
		validate_mixed(@_,
			[{type => OBJECT},],
			{type => {type => SCALAR,},},);
   $Log->enter();
   #$opts{'type'}=lc($opts{'type'});
   my $type = lc($opts{'type'});
   my $ret = $self->delete_resources(type => $type);

   $Log->exit();
   return $ret;
}

sub modify_all_resources
{
   my ($self, %opts) = 
			validate_mixed(@_,
				[{type => OBJECT},],
				{type => {type => SCALAR,},
				updatehashref => {type => HASHREF},},);
   $Log->enter();

   my $matchhashref = "";
   $self->modify_resources($opts{'type'}, $matchhashref, $opts{'updatehashref'});
   $Log->exit();
   return 0;
}

sub modify_resources
{
   $Log->enter();
   my ($self, %opts) =
          validate_mixed(@_,
                  [{type=> OBJECT},],
                  {type => {type => SCALAR,},
                   count => {type => SCALAR, optional => 1, default => 0},
                   match_filters => {type => HASHREF, optional => 1,},
                   modify_info => {type => HASHREF,},
                   regex_filters => {type => HASHREF, optional => 1,},},);

   my $restype = $opts{'type'};
   my $maxrows = $opts{'count'};
   my $filters = $opts{'match_filters'} if (defined $opts{'match_filters'});
   my $regex_filters = $opts{'regex_filters'} if (defined $opts{'regex_filters'});
   my $modify_info = $opts{'modify_info'};

   my $ret = 0;
   my %updatehash = %{$modify_info};
   my $updatestring = "";
   foreach my $col (keys(%updatehash)) {
      $col = "_" . "$col" if ($col eq "group");
      my $value = $updatehash{$col};
         my $ref = ref $value;
      if ($ref eq 'ARRAY') {
         $value = "@$value";
      }
      if ($updatestring eq "") {
         $updatestring = "SET " . "$col=\'$value\'" ;
      }
      else {
         $updatestring = "$updatestring" . ",$col=\'$value\'" ;
      }
   }

   my $crossreftable = $restype . "crossref";
   my $matchstring = "WHERE $crossreftable.resourcegroup=\'$self->{'name'}\' ";
   if ($filters)
   {
      my %matchhash  = %{$filters};
      foreach my $thiskey (keys (%matchhash)) {
         #$thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         $thiskey = &ConfigBroker::Schema::convert_reserved_word($thiskey) if (&ConfigBroker::Schema::is_reserved_word($thiskey));
         # is this an array attribute?
         my $value = $matchhash{$thiskey};
         my $isarray = &ConfigBroker::Schema::is_array_attribute($restype, $thiskey, $self->{'dbh'});
         if ($isarray) {
            # we store the arrays as as " " separated string in the DB.
            # so the value will have a " " either in the front or the back
            my $compare_string = "($thiskey=\' $value\' OR $thiskey=\'$value \' OR $thiskey=\'$value\')";
            $matchstring = "$matchstring" . "AND $compare_string " ;
         }
         else {
            $matchstring = "$matchstring" . "AND $thiskey=\'$value\' " ;
         }
      }
   }
   if ($regex_filters)
   {
      my %matchhash = %{$regex_filters};
      foreach my $thiskey (keys (%matchhash)) {
         $thiskey = "_" . "$thiskey" if ($thiskey eq "group");
         $matchstring = "$matchstring " . "AND $thiskey LIKE \'$matchhash{$thiskey}\' " ;
      }
   }

   my $update_cmd = "UPDATE $restype ".
                    "JOIN $crossreftable ON $restype.uniq_resource_id=$crossreftable.resourceid ".
                    "$updatestring ".
                    "$matchstring";

   eval {
      $self->{'dbh'}->do($update_cmd);
   };
   if ($@) {
      $Log->warn("Error updating from the table \"$restype\". Error is $@");
      $ret = 1;
   }
   $Log->enter();
   return $ret;
}

sub discover_clients_into_group {
   # Discover clients specified in the Options or input
   # and add them to this group.
   $Log->enter();
   my ($self, %opts) =
          validate_mixed(@_,
                  [{type => OBJECT},],
                  {reuse   => {type => SCALAR, optional => 1, default => 0},
                   clients => {type => ARRAYREF, optional => 1,},},);

   ### Check if there are any existing clients in this RG.  
   my $existing_clients = $self->get_resources(type => 'client');
   if (defined $existing_clients->[0]) {
      ### If there are clients, if reuse is requested, just return.
      if ($opts{'reuse'}) {
         $Log->debug("There are clients in the RG. Reusing them.\n");
         return 0;
      }
      ### Delete the existing clients so we can rediscover 
      my $ret = $self->delete_resources(type => 'client');
      if ($ret) {
         $Log->error("Deleting existing clients before redisover failed\n");
         return 1;
      }
   }

   my $clients = [];
   $clients = $opts{'clients'} if (defined $opts{'clients'});
   my %engineinfo = (
       'enginename'        => "DiscoverClients",
       'discover'          => 1,
       'clients'           => $clients,
       'resourcegroup'     => $self->{'name'},
       'dbh'               => $self->{'dbh'},
   );

   require ConfigBroker::Engine;
   my $engine_obj = ConfigBroker::Engine::get_engine_object(\%engineinfo);
   my $rc = $engine_obj->run_discovery();
   if ($rc != 0) {
       $Log->error("Failure returned from clients discovery.\n");
       return 1;
    }
    return 0;
}

sub discover_users_into_group {
   # Discover users specified in the Options or input
   # and add them to this group.
   $Log->enter();
   my ($self, %opts) =
          validate_mixed(@_,
                  [{type => OBJECT},],
                  {reuse   => {type => SCALAR, optional => 1, default => 0},
                   },);

   ### Check if there are any existing users in this RG.  
   my $existing_users = $self->get_resources(type => 'user');
   if (defined($existing_users) && @{$existing_users}) {
      ### If there are users, if reuse is requested, just return.
      if ($opts{'reuse'}) {
         $Log->comment("There are users in the RG and reuse flag is set. Reusing them.\n");
         return 0;
      }
      ### Delete the existing users so we can rediscover 
      my $ret = $self->delete_resources(type => 'user');
      if ($ret) {
         $Log->error("Deleting existing users before redisover failed\n");
         return 1;
      }
   }

   my %engineinfo = (
       'enginename'        => "DiscoverUsers",
       'discover'          => 1,
       'resourcegroup'     => $self->{'name'},
       'dbh'               => $self->{'dbh'},
   );

   require ConfigBroker::Engine;
   my $engine_obj = ConfigBroker::Engine::get_engine_object(\%engineinfo);
   my $rc = $engine_obj->run_discovery();
   if ($rc != 0) {
       $Log->error("Failure returned from user discovery.\n");
       return 1;
    }
    return 0;
}
=head2 add_resources

    Object method; Add resources of particular resource type to this ResourceGroup.

=head3 Usage

    $RG->add_resources(%opts)
    $opts{'resource_type'} - Type of resource to be added
    $opts{'resourceinfo_arrayref'} - Arrary reference containing the resource information to be added.

=head3 Return

    0 - Success.

=head2 verify_resource_exists

    Object method; Verify if a given reosurce of type & name exists in the ResourceGroup.

=head3 Usage

    $RG->verify_resource_exists(%opts)
    $opts{'resource_type'} - Type of resource to be searched for.
    $opts {'resource_name'} - Name of the resource to be searched for.

=head3 Return

    1 - ResourceExists ;  0 - Resource does NOT exist.

=head2 get_resources 

    Object method; Get resources of type $resource_type from this ResourceGroup  
    Filter retrieval based on %filter_params 

=head3 Usage

    $RG->get_resources(%opts);
    $opts{'type'} - resource type to retrieve
    $opts{'count'} - max number of resources to retrieve
    $opts{'offset'} - offset the result of the query returned by given number.  if specificed, the query results will be returned from
                      the item starting at the offset number.
    $opts{'randomize'} - randomize the query results order. default is 0. if set to 1, the query results will be in random order.
    $opts{'match_filters'} - Hash reference to match resource attribute key/values for an exact match 
    $opts{'regex_filters'} - Hash reference to match resource attribute key/values for matching the given regular expression.

=head3 Return

    Reference to an array of resources(hash) retrieved.

=head2 modify_resources 

    Object method; Modify resources in this ResourceGroup.
    NOTE: Update only the configbroker data.  Does not actually modify the resource.
    Actual modification of resources to be done by users using NACL methods or filer commands.

=head3 Usage

    $RG->modify_resources(%opts);
    $opts{'type'} - resource type to retrieve
    $opts{'count'} - max number of resources to retrieve
    $opts{'match_filters'} - Hash reference to match resource attribute key/values for an exact match 
    $opts{'regex_filters'} - Hash reference to match resource attribute key/values for matching the given regular expression.
    $opts{'modify_info'} - Hash reference to attributes key/values that are to be modified. 
                             Modify the key attribute with the given new value.

    Filter resources to modify using $opts{'match_filters'} & $opts{'regex_filters'}
    Update resources with values provided by $opts{'modify_info'}

=head3 Return

    0 - Success ; 1 - Failure.

=head2 delete_resources 

    Object method; Deletes all the resources pertaining to this ResourceGroup.
    NOTE: deletes only the configbroker data.  Does not actually delete the resource.
    Actual deletion of specific resources to be done by users using NACL methods or filer commands.


=head3 Usage

    $RG->delete_resources();

=head3 Return

    0 - Success ; 1 - Failure.

=head2 discover_clients_into_group

    Object method; Discover clients and add them to this group.  The clientname is provided through %opts or 
    through input options CLIENT / CLIENTS / CLIENTFILE..

=head3 Usage

    $RG->discover_clients_into_group(%opts);
    $opts{'clients'} - Reference to the optional array containing clientnames.
                       If this is not provided, input options CLIENTFILE / CLIENT / CLIENTS will be used.
                       If none are provided, returns error.
    $opts{'reuse'}   - Boolean to indicate reusing existing clients in the group or clean out the existing and rediscover.
                       default is 0, causing rediscover.

=head3 Return

    0 - Success ; 1 - Failure.


=head2 reserve

    Object method; Reserved this "sharable" ResourceGroup as being currently in use; When not in use, others can share.

=head3 Usage

    $RG->reserve(%opts);
    %opts{'res_code'} - Unique Code to be placed on the resource group to reserve it . Mandatory Parameter. It is of the form INUSE_<res_code>

=head3 Return

    0 - Success ; 1 - Failure.

=head2 unreserve

    Object method; Unreserve this "shareable" ResourceGroup as READY for others to use. 

=head3 Usage

    $RG->unreserve();
    %opts{'res_code'} - Unique Code to be search on the resource group to unreserve it . Mandatory Parameter. It is of the form INUSE_<res_code>


=head3 Return

    0 - Success ; 1 - Failure.

=head2 get_group

    Object method; get the information (data used to create the gorup) about the group itself.

=head3 Usage

    $RG->get_group();

=head3 Return

    Returns a hashref containing the group info data (name, configengine, configfile, share, reuse, status, cmdlinestring).

=head2 update_status

    Object method; used to update the status of the resource group 

=head3 Usage

    $RG->update_status(%opts);
    $opts{'status'} is a mandatory parameter. 
    The possible values for this are: READY, INUSE, PENDING.  

=head3 Return

    Returns 0 upon success; 1 upon failure.

=cut

1;
