package ConfigBroker::Schema;

use strict;
use warnings;
use DBI qw();
use Tharn qw($Log connect config_result script_result host say snooze);
use Data::Dumper;
our @ISA = qw (Exporter);
use Data::Dumper;
use Options;
use YAML;
use NACL::APISet;

use Params::Validate qw(validate_with SCALAR);
use Class::Inspector;

my @reserved_words = (
 "force",
 "group",
 "option",
 "optimize",
 "trigger",
 "write",
);

my @defaulttableattrs = ("name", "uniq_resource_id", "group_ref_count", "_category", "_status", "_priority", "_comment");

my %task_tables = (
 tasks  => "task_name VARCHAR(100), task_type VARCHAR(100), task_action VARCHAR(20), timestamp VARCHAR(50), resource_id VARCHAR(500), resource_type VARCHAR(100), reporting_module VARCHAR(25), PRIMARY KEY(task_name, task_action, resource_id, timestamp, reporting_module)"
);

my %non_nacl_resourcetables = ( 
  net	      => ["interface_name", "mac_address", "mediatype", "flowcontrol", "ipspace_name", "addr_family", "address", "broadcast", "netmask_or_prefix", "creator", "node" ],
  cifs	      => ["netbios_servername", "auth_type", "security_style", "netbios_domainname", "dns_domainname", "windows_type", "ad_site", "vfiler", "node" ],
  shares      => ["share_name", "mount_point", "description", "vfiler", "node" ],
  vfiler      => ["ipspace", "uuid", "allowed_protocols", "node" ],
  vfdns       => ["dns_domain_name", "dns_servers", "vfiler", "node" ],
  vfnis       => ["nis_domain_name", "nis_servers", "vfiler", "node" ],
  vfstore     => ["status", "path", "is_etc", "vfiler", "node" ],
  vfnet       => ["interface", "ipaddress", "netmask", "vfiler", "node" ],
  client      => ["client_name", "hosttype", "os_version", "os", "ipv4", "ipv6", "ipconfig" ],
  device      => ["client_name", "device", "protocols" ],
  options7m   => ["vfiler", "value", "cluster_constraint", "node" ],
  useradmin   => ["comment", "full_name", "homedir", "node", "vfiler"],
  user        => ["id", "user_name", "password", "ad", "nis", "host", "hosttype", "homedir", "vserver", "protocol"]
);

my @nacl_resourcetables = (
 "Aggregate",
 "Cluster",
 "ExportPolicy",
 "ExportPolicyRule",
 #"FlexClone", ## volume clone in nacl
 "Ifconfig",
 # "JobSchedule", ## name is an attribute for this object in nacl
 "License", ## limit attribute causes syntax error.  mysql keyword?
 "Lun",
 "LunBind",
 "LunIgroup",
 "LunMapped",
 "LunPortset",
 "NetworkInterface",
 "NetworkPortIfgrp",
 "NetworkPort",
 #"NetworkRouteGroup",
 #"NetworkRoute",
 "Options",
 "QosWorkload",
 "QosPolicyGroup",
 "Quota",
 "QuotaPolicy",
 "QuotaPolicyRule",
 # "Snapmirror", ## has QOS_policy & qos_policy attributes.
 "Snapshot",
 "SnapshotPolicy",
 "Volume",
 "VolumeQtree",
 "VserverAudit",
 "VserverCifs",
 "VserverCifsHomeDirectorySearchPath",
 "VserverCifsShare",
 "VserverCifsUsersAndGroupsLocalUser",
 "VserverFcp",
 "VserverIscsi",
 "VserverNFS",
 "VserverNfsKerberosConfig",
 "Vserver",
);

my @nacl_resourcetables_temp = (
 "Volume",
 "Aggregate",
);

sub new
{
   $Log->enter();
   my ($pkg, $schemafile) = @_;
   my $class = ref($pkg) || $pkg ;
   my $self = {};

   $self->{'schemafile'}           = $schemafile if(defined $schemafile);
   $self->{'resourceschema'}       = {}; 
   bless ($self, $class);
   @{$self->{defaulttableattrs}} = @defaulttableattrs;
   $self->_create_schema();

   $Log->exit();
   return $self;
}

sub _create_schema()
{
   $Log->enter();
   my $self = shift;
   
   if (! defined $self->{'schemafile'}) 
   {
      ### get the default schema 
      my %schemahash;
      foreach my $table (@nacl_resourcetables)
      {
         my $attrhashref;
         my @attrs;
         $attrhashref  = &get_attributes($table);
         @attrs = (@{$attrhashref->{'scalar_attributes'}}, @{$attrhashref->{'array_attributes'}});
         my $attrstring .= $self->get_default_table_attrs() . ' ';

         if ($table eq "License") {
            my @newarray;
            foreach my $attr (@attrs) {
               if ($attr eq "limit") {
                  push (@newarray, "license_limit");
               }
               else {
                  push (@newarray, $attr);
               }
            }
            $attrstring .=  "@newarray"; 
         }
         elsif ($table eq "Options") {
            my @newarray;
            foreach my $attr (@attrs) {
               if ($attr ne "name") {
                  push (@newarray, $attr);
               }
            }
            $attrstring .= "@newarray"; 
         }
         else {
            $attrstring .= "@attrs"; 
         }
            
         $schemahash{lc($table)} = $attrstring; 
      }
      foreach my $table (keys (%non_nacl_resourcetables)) {
         my $attrarrayref;
         $attrarrayref  = $non_nacl_resourcetables{$table};
         my $attrstring .= $self->get_default_table_attrs() . ' ';
         $attrstring .= "@{$attrarrayref}"; 
         $schemahash{lc($table)} = $attrstring; 
      }
      $self->{'resourceschema'} = \%schemahash;
      
   } 
   elsif (! -e $self->{'schemafile'})
   {
      ### Error out
   }
   else
   {
      ### Read in the file into a hash 
      ### file lines should be TYPE : "fields string" for example, vol : "name resourcegroup vserver size"
      ### in YAML format

      $Log->debug("schema file found $self->{'schemafile'}\n");
      my ($hashref, $arrayref, $string) = YAML::LoadFile($self->{'schemafile'}); 
      my %schemahash = %$hashref;
      foreach my $mykey (keys (%schemahash))
      {
         my $attrstring = "name ";
         my $value = $schemahash{$mykey};
         #$attrstring = "@$value"; 
         $attrstring = "name " . "@$value "; 
         $schemahash{$mykey} = $attrstring; 
      }
      $self->_validate_schema($hashref);
      $self->{'resourceschema'} = \%schemahash;
   }    
   $Log->exit();
   return 0;
}
sub get_nacl_resource_names
{
   $Log->enter();
   #my $self=shift;
   return \@nacl_resourcetables;
}


sub _validate_schema()
{
   my ($self, $hashref) = @_;

   ### TO BE IMPLEMENTED
   ### make sure each type has "name" and "resourcegroup" part of the schema.  If not, add those.
   return;
}

sub get_tasks_table_schema(){
	return \%task_tables;
}
sub get_resource_schema()
{
   my $self = shift;
   return $self->{'resourceschema'};
}

sub get_default_table_attrs
{
    my ($self) = @_;

    if ($self) { 
        return wantarray ? $self->{'defaulttableattrs'} : join(' ', @{$self->{'defaulttableattrs'}});
    } 
    else { 
        return wantarray ? \@defaulttableattrs : join(' ', @{defaulttableattrs});
    }
}

sub get_primary_keys {
   my ($typename) = @_; 

   my @primarykeys;

   $Log->comment("Getting primary keys for $typename\n");
   # use the component library from nacl
   # the typename that comes as input is all lowercase.
   # find the nacl component from our @nacl_resourcetables
   my $nacltype = "" ;
   foreach (@nacl_resourcetables) {
      if (lc($_) eq $typename) {
         $nacltype = $_;
         last;
      }   
   }
   my $comp = "NACL::C::$nacltype";

   my $primarykeys;
   my $naclkeys_found = 0;
   eval "require $comp";
   if (! $@) {
      $Log->debug("Trying to find NACL keys\n");
      @primarykeys = $comp->get_primary_keys();
      if (scalar(@primarykeys) > 0) { 
         $naclkeys_found = 1;
         #push (@primarykeys, "resourcegroup");
      }
      ### The type "cluster" gets "node" as the primary key.  It is wrong.
      ### fix this here.
      if (lc($nacltype) eq "cluster") {
         @primarykeys = ("name");
      }
      else {
         my @pkeys; 
         foreach my $thiskey (@primarykeys) {
            $thiskey = &convert_reserved_word($thiskey) if (&is_reserved_word($thiskey)); 
	    $thiskey =~ s/-/_/g ;
            push (@pkeys, $thiskey);
         }
         @primarykeys = @pkeys;
      }
   }
   if ($nacltype eq "Volume") {
      ### This is to make it work for 7mode also.
      push(@primarykeys, "node");
   }
   if ($typename eq "options") {
      ### NACL doesn't seem to return primary keys for this even though it is a NACL type
      @primarykeys = ("option_name", "vserver", "option_value", "cluster_constraint");
   }
   if (! $naclkeys_found) {
      if ($typename eq "options7m") {
         @primarykeys = ("name", "vfiler", "node");
      }
      elsif ($typename eq "volume") {
         #@primarykeys = ("name", "resourcegroup", "aggregate", "vserver");
         @primarykeys = ("name", "aggregate", "vserver");
      }
      elsif ($typename eq "exportpolicy") {
         #@primarykeys = ("policyname", "resourcegroup", "vserver");
         @primarykeys = ("policyname", "vserver");
      }
      elsif ($typename eq "vserver") {
         #@primarykeys = ("vserver", "resourcegroup");
         @primarykeys = ("vserver");
      }
      elsif ($typename eq "vservercifs") {
         #@primarykeys = ("name", "resourcegroup", "vserver");
         @primarykeys = ("name", "vserver");
      }
      elsif ($typename eq "vservercifsshare") {
         #@primarykeys = ("name", "resourcegroup", "share_name", "vserver");
         @primarykeys = ("name", "share_name", "vserver");
      }
      elsif ($typename eq "vservernfs") {
         #@primarykeys = ("name", "resourcegroup", "vserver");
         @primarykeys = ("name", "vserver");
      }
      elsif ($typename eq "volumeqtree") {
         #@primarykeys = ("name", "resourcegroup", "vserver", "volume");
         @primarykeys = ("name", "vserver", "volume");
      }
      elsif (($typename eq "networkinterface") || ($typename eq "interface")) {
         #@primarykeys = ("lif", "resourcegroup", "vserver", "server");
         @primarykeys = ("lif", "vserver", "server");
      }
      elsif ($typename eq "lun") {					
         #@primarykeys = ("vserver", "resourcegroup","path","name");
         @primarykeys = ("vserver", "path", "name");
      }
      elsif ($typename eq "networkport") {
         #@primarykeys = ("port", "resourcegroup", "node");
         @primarykeys = ("port", "node");
      }
      elsif ($typename eq "qosworkload") {
         #@primarykeys = ("workload", "resourcegroup");
         @primarykeys = ("workload");
      }
      elsif ($typename eq "quota") {
         #@primarykeys = ("resourcegroup", "volume", "vserver");
         @primarykeys = ("volume", "vserver");
      }
      elsif ($typename eq "quotapolicy") {
         #@primarykeys = ("name", "resourcegroup", "vserver");
         @primarykeys = ("name", "vserver", "policy_name");
      }
      elsif ($typename eq "qospolicygroup") {
         #@primarykeys = ("resourcegroup", "policy_group");
         @primarykeys = ("policy_group");
      }
      elsif ($typename eq "quotapolicyrule") {
         #@primarykeys = ("resourcegroup", "vserver", "target", "qtree", "volume", "type", "policy_name");
         @primarykeys = ("vserver", "target", "qtree", "volume", "type", "policy_name");
      }
      elsif ($typename eq "exportpolicyrule") {
         #@primarykeys = ("name", "resourcegroup", "vserver", "ruleindex", "policyname");
         @primarykeys = ("name", "vserver", "ruleindex", "policyname");
      }
      elsif ($typename eq "snapshot") {
         #@primarykeys = ("snapshot", "resourcegroup", "vserver", "volume");
         @primarykeys = ("snapshot", "vserver", "volume");
      }
      elsif ($typename eq "snapshotpolicy") {
         #@primarykeys = ("policy", "resourcegroup", "vserver");
         @primarykeys = ("policy", "vserver");
      }
      elsif ($typename eq "license") {
         #@primarykeys = ("feature", "serial_number", "package", "resourcegroup");
         @primarykeys = ("feature", "serial_number", "package");
      }
      elsif ($typename eq "ifconfig") {
         #@primarykeys = ("interface", "resourcegroup");
         @primarykeys = ("interface");
      }
      elsif ($typename eq "vfiler") {
         #@primarykeys = ("name", "resourcegroup", "ip_address", "interface");
         @primarykeys = ("name", "node");
      }
      elsif ($typename eq "vfnis") {
         #@primarykeys = ("nis_domain_name", "vfiler", "resourcegroup");
         @primarykeys = ("nis_domain_name", "vfiler", "node");
      }
      elsif ($typename eq "vfdns") {
         #@primarykeys = ("dns_domain_name", "vfiler", "resourcegroup", "node");
         @primarykeys = ("dns_domain_name", "vfiler", "node");
      }
      elsif ($typename eq "vfstore") {
         #@primarykeys = ("path", "vfiler", "resourcegroup", "node");
         @primarykeys = ("path", "vfiler", "node");
      }
      elsif ($typename eq "vfnet") {
         #@primarykeys = ("ipaddress", "vfiler", "resourcegroup", "node");
         @primarykeys = ("ipaddress", "vfiler", "node");
      }
      elsif ($typename eq "cifs") {
         #@primarykeys = ("netbios_servername", "resourcegroup", "vfiler", "node");
         @primarykeys = ("netbios_servername", "vfiler", "node");
      }
      elsif ($typename eq "shares") {
         #@primarykeys = ("share_name", "vfiler", "resourcegroup", "node");
         @primarykeys = ("share_name", "vfiler", "node");
      }
      elsif ($typename eq "net") {
         #@primarykeys = ("interface_name", "address", "resourcegroup", "node");
         @primarykeys = ("interface_name", "address", "node");
      }
      elsif ($typename eq "client") {
         #@primarykeys = ("client_name", "resourcegroup");
         @primarykeys = ("client_name");
      }
      elsif ($typename eq "device") {
         @primarykeys = ("device");
      }
      elsif ($typename eq "options") {
         ### NACL doesn't seem to return primary keys for this even though it is a NACL type
         @primarykeys = ("option_name", "vserver", "option_value", "cluster_constraint");
      }
      elsif ($typename eq "user") {
         @primarykeys = ("user_name", "id", "vserver");
      }
      elsif ($typename eq "useradmin") {
         @primarykeys = ("name", "node", "vfiler");
      }
      else {
         #@primarykeys = ("name", "resourcegroup") ;
         @primarykeys = ("name") ;
      }
   }
   return \@primarykeys;
}

### keep this method for now. We are not going to use it
### but may want this in the future.
sub get_zapi_attributes {
   $Log->enter();
   my $self = shift;

   #my $node = NACL::C::Node->find();
   my $host = host("nst-vsa-35");
   my $zapiapi  = NACL::APISet->new(
            hostobj   => $host,
            category  => "Node",
            interface => "ZAPI",
            set       => "CMode",
   );

   #my $apiarray = $zapiapi->system_api_list();
   #my $apiarray = $zapiapi->system_api_get_elements('api-list' => ['volume-get-iter', 'aggr-get-iter']);
   #my $apiarray = $zapiapi->system_api_get_elements('api-list' => ['volume-info']);
   my $apiarray = $zapiapi->system_api_list_types();
   #print "APIARRAY IS: \n";
   #print Dumper($apiarray);
   #print "APIARRAY END: \n";
}

### CLASS METHOD ###
sub get_attributes {
   $Log->enter();
   my ($type, $dbh) = @_;
   my (@array_attributes, @scalar_attributes);
   my $pkg = "NACL::CS::$type";
   eval "require $pkg";
   if ($@) {  
       # The NACL CS objects allow us to call get_attributes(), and we 
       # can determine if the column data should be an array or a scalar. 
       # Custom tables won't have NACL CS objects associated with them, 
       # and thus the include will fail. For now, we are not going to support
       # arrays in custom tables. At least the get won't work properly with the match 
       # option specified. It probably only affects exact match at that. 
       # Seems we can live with this for a while. 
       $Log->debug("require $pkg did not work. It must be a custom table or non nacl default table. Proceeding");  
       $@ = "";

       my $resourcename = $type;
       my $attributes = &ConfigBroker::CbUtils::get_resources($dbh, 'typedefn', $type);
       # get_resources() returns an arrayref of scalars from typedefn
       # position one is the name, two is the type string seperated by spaces
       @scalar_attributes = split(/ /, $attributes->[1]);
     
   } else { 
       my $attrhashref  = $pkg->get_attributes();
       my %attrs = %$attrhashref;

       foreach my $thiskey (keys(%attrs)) {
          #$thiskey = "_" . "$thiskey" if ($thiskey eq "group");
          my $value = $attrs{$thiskey};
          $thiskey = &convert_reserved_word($thiskey) if (&is_reserved_word($thiskey));
          if ($value eq 'array') {
              push @array_attributes, $thiskey;
          } 
          else {
              push @scalar_attributes, $thiskey;
          }
       }
   }
   my $specialattrs = get_attribute_exceptions($type);
   if (scalar(@{$specialattrs})) {
      $Log->debug("special... $type\n");
      push (@scalar_attributes, @{$specialattrs});
   }
   my %attributes = (
      array_attributes  => \@array_attributes,
      scalar_attributes => \@scalar_attributes
   );
   $Log->exit();
   return \%attributes;
}

sub get_attribute_exceptions {
   my $type = shift;

   my $attr_exception = [];
   if (lc($type) eq "aggregate") {
      $attr_exception = ["wafliron"];
   }
   if (lc($type) eq "volume") {
      $attr_exception = ["owning_vfiler", "autosize_percent", "use_default_qos_policy_group"];
   }
   if (lc($type) eq "vserver") {
      $attr_exception = ["use_default_qos_policy_group", "fcp_enabled", "iscsi_enabled", "vs_type"];
   }
   if (lc($type) eq "lun") {
      $attr_exception = ["bind_list", "create_from_file", "lun_id", "lun_id_range", "require_same_lun_id_per_igroup", "use_default_qos_policy_group"];
   }
   return $attr_exception;
} 
      
sub is_array_attribute 
{
   my ($type, $attr,$dbh) = @_;
 
   my $pkg = $type; 
   foreach (@nacl_resourcetables) {
      if (lc($_) eq lc($type)) {
         $pkg = $_;
      }
   }
   my $attrhashref = &get_attributes($pkg,$dbh);
   my $array_attributes = $attrhashref->{'array_attributes'};

   my @arrayattrs = @$array_attributes;
   my $is_array = 0;
   foreach (@arrayattrs) {
      if ($attr eq $_) {
         $is_array = 1;
         last;
      }
   }
   return $is_array;
}

sub is_reserved_word
{
   my ($field) = @_;  

   foreach my $word (@reserved_words) {
      return 1 if ($word eq $field); 
   }
   return 0;
}

sub convert_reserved_word
{
   my ($field) = @_;
   
   $field = "_" . "$field";
   return $field;
}


1;
