package ConfigBroker::SuperGroup;

use strict;
use warnings;
use DBI qw();
use Tharn qw($Log);
use Data::Dumper;
our @ISA = qw (Exporter);
use ConfigBroker::CbUtils;

## @summary SuperGroup
## @author veena@netapp.com
## @status public
## @pod here

=head1 NAME

  SuperGroup

=head1 SYNOPSIS

  use SuperGroup;

  my $CB = new ConfigBroker();
  my $RG = $CB->create_supergroup($supergroupname, $subgroups_arrayref);

=head1 DESCRIPTION

  class for creating supergroup objects and accessing its methods.

=cut


sub new
{
   $Log->enter();
   my ($pkg, $name, $dbh, $subgroups) = @_;
   my $class = ref($pkg) || $pkg ;
   my $self = {};
   $self->{'dbh'}     = $dbh;
   $self->{'name'}    = $name;
   $self->{'groups'}  = "";
   if (defined $subgroups) {
      $self->{'groups'} = "@$subgroups";
   }
   bless ($self, $class);
   $self->_add_supergroup_to_db();
   $Log->exit();
   return $self;
}

sub _add_supergroup_to_db
{
   my $self = shift;
   $Log->enter();

   $Log->debug(Dumper($self));
   eval {
      my $insert_cmd = "INSERT INTO \"supergroup\" (name, groups)" .
                       "VALUES(\'$self->{'name'}\', \'$self->{'groups'}\')";
      $self->{'dbh'}->do($insert_cmd);
   };
   if ($@) {
      $Log->debug("Error inserting row into table \"supergroup\". Error is $@");
   }

   $Log->exit();
}


sub add_subgroups
{
   my ($self, $groups) = @_;
   $Log->enter();
  
   $self->{'groups'}         = $groups;
   my @subgroups = $self->subgroups();
   my $subgroupstring;
   if (scalar(@subgroups) > 0) { ### we have a list of subgroups already; append to it.
      $subgroupstring = "@subgroups " . $groups; 
   }
   else {
      $subgroupstring = $groups;
   }
   eval {
      my $update_cmd = "UPDATE supergroup " .
                       "SET groups=\'$subgroupstring\' ".
                       "WHERE name=\'$self->{'name'}\'";
      $self->{'dbh'}->do($update_cmd);
   };
   if ($@) {
      $Log->debug("Error updating groups into table \"supergroup\". Error is $@");
   }
}

sub get_subgroups
{
   my ($self) = @_;
   $Log->enter();

   my $get_cmd = "SELECT gropus FROM supergroups".
                 "WHERE name=\'$self->{'name'}\' ";

   my @groupsarray = @{$self->{'dbh'}->selectcol_arrayref($get_cmd)};
   my @groups= split(" ", $groupsarray[0]);

   $Log->exit();
   return @groups;
}

### Get all resources of a specific type
### possibly narrow down using paramhash by mactching specific colomn values.
### return an array reference to an array of hashes.
sub get_resources
{
   my ($self, $restype, $maxrows, $paramhash ) = @_;
   $Log->enter();
   my @all_resources;

   ### validate $restype and $maxrows here
   ### TO DO

   ### Get the subgroups and call get_resources on each of them and aggregate.
   my @subgroups = $self->get_subgroups();
   foreach my $group (@subgroups)
   {
      my $resarrayref = ConfigBroker::CbUtils::_get_resources_arrayref_from_db($group, $self->{'dbh'}, $restype, $maxrows, $paramhash);
      push (@all_resources, @$resarrayref);
   }
   $Log->exit();
   return \@all_resources;
}

sub get_resource_NACL_objects
{
   my ($self, $type_name, $maxresources, $paramhashref) = @_;
   $Log->enter();

   my @objs;
   ### Get the subgroups and call get_resources on each of them and aggregate.
   my @subgroups = $self->get_subgroups();
   foreach my $group (@subgroups)
   {
      my $resource_arrayref = ConfigBroker::CbUtils::_get_resources_arrayref_from_db($group, $self->{'dbh'}, $type_name, $maxresources, $paramhashref);
      if (scalar(@{$resource_arrayref})) {
         my $node = NACL::C::Node->find();
         foreach my $resource (@{$resource_arrayref}) {
            # Create NACL objects
            my $obj = &ConfigBroker::CbUtils::_create_nacl_obj(
               'type'     => $type_name,
               'resource' => $resource,
               'node'     => $node
            );
            push(@objs, $obj);
         }
      }
      else {
         $Log->error("Could not create NACL object for resource\n");
         exit 1;
      }
   }
   ### TO DO - Other category of resources i.e. non cluster controller ones like switches etc.

   return @objs;
   $Log->exit();
}

1;
