package CBTemplate;

# Import packages
use strict;
use warnings;
use Data::Dumper;
use DBI qw();

BEGIN {
   use Exporter();
   our @ISA = qw(Exporter);
   our $VERSION = '1.0.0';
   our @EXPORT = qw(log_creation create_resource_group discover_create get_resourcegroup_names update_resourcegroup reserve_resourcegroup unreserve_resourcegroup delete_resource_group create_resource_type delete_resource_type get_custom_resourcetypes get_all_resourcetypes get_all_custom_restypefields get_resource_type_names get_resourcegroup_type get_resourcetypes_discover get_resource_fields get_resource_fields_name remove_resource_name modify_resource add_resource add_new_resource);
   our @EXPORT_OK  = qw();
}

## Pushing the paths of ConfigLibraries and the NATE Libraries to the INC array .

#BEGIN {push (@INC, '/u/sreekuma/p4/main/lib/','/u/sreekuma/p4/main','/usr/software/test/share/nate/latest/lib/');} ##Hard Coding needs to Go
BEGIN {push(@INC, '/usr/software/test/agnostic/', '/usr/software/test/agnostic/lib/', '/usr/software/test/share/nate/latest/lib/');}

##Loading the appropriate ConfgiBroker packages and NATE packages

use NATE::Log qw(log_global);
use NATE::BaseException;
use NATE::ParamSet;
use ConfigBroker;
use ConfigBroker::Schema;
# $Id: $
## @summary Perl module for ConfigBroker GUI
## @status Public
## @author Sreekumar Methil (sreekuma@netapp.com, dl-nst-tools@netapp.com)
## @pod here

=head1 NAME

CBTemplate.pm 

=head1 DESCRIPTION

CBTemplate is a library for implementing ConfigBroker functions on a GUI . It has several methods which interacts with 
ConfigBroker Libraries and post the results on a GUI.

=head1 SYNOPSIS

To call the routines in CBTemplate we need to pass the load the library in the CGI script and call appropriate methods .
Here is a snippet of the way to call the routines in CBTemplate.pm
    Eg : CBTemplate::get_resource_type_names(testbed => "$testbed",typename=>"$resource_type");

=cut




## Global variables to be used in the Library throughout.

my $Log = log_global();


### Function to create Log and instantiate a Log object ,to capture all the NATE logs. This function creates a logdirectory called
### cbgui under the testbed if not present and creates a logfile . It returns the full link of the location of log file to the user.

sub log_creation{
	
	
	my %opts = (
	testbed => undef,
	@_,
	);
		
		my $logfile;
		my $testbed=$opts{'testbed'};
		
			##Checks if a directory called cbgui is present under database otherwise it creates it .
		
			if(-d "$testbed/cbgui")
				{
					$logfile = $testbed."/cbgui/cbgui".time().".log";
				}

		else 
				{
					
					
					mkdir "$testbed/cbgui" or die "Not enough permissions to create a logfile or database under $testbed.Please give the appropriate permissions to the folder";
					$logfile = $testbed."/cbgui/cbgui".time().".log";
				
		}
			
			return $logfile;
	}




## Function to create a resource group . With this function we will be able to create only the resource group with name currently.
## Note : Because of the ntest subtest issue . As of now the config creation using CCL and EngenX is disabled. 

sub create_resource_group{

	my %opts = (
		cb => undef,	
		name => undef,
		testbed =>undef,
		configfile=>undef,
		configengine=>undef,
		reuse=>undef,
		share=>undef,
		cleanupfile=>undef,
		cmdlinestring=>undef,
		status=>undef,
		@_,
		);

	$Log->enter();
	$Log->comment("################################CREATING RESOURCE GROUP #################################");
	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $name = $opts{'name'};
	my $configfile = $opts{'configfile'};
	my $configengine = $opts{'configengine'};
	my $returnmessage;
	## IF condition to check if the name is given by user and return error message if error is not present.
	
	if($name){
	
	my $rg=$cb->create_resource_group(name => "$name");

		## Checks if a hash reference is returned which indicates creation of resource group
			if($rg =~ /.*HASH\(\w+\)/i)
				{
					$returnmessage=" Resource Group Created ".$name;
				}
			else 
				{
					$returnmessage=$rg;
				}}
			else {
				$returnmessage="Name should be specified to create a resource group";
			}
	
	$Log->exit();
	return "$returnmessage";
	}


##Function to discover resource type and add it as part of resource group.This function is used to discover existing NACL resource
## on the filer.

sub discover_create{
	my %opts = (
		testbed =>undef,
		name => undef,
		node => undef,	
		distype =>undef,
		reuse =>undef,
		share => undef,
		@_,
			);
	
	$Log->enter();
	$Log->comment("################################ DISCOVERING & CREATING RESOURCE GROUP #################################");
	
	## Assigning value to NATE_HOSTS as the discover script looks for NATE hosts files using this variable.
	
	my $natehostindex=index($opts{'testbed'},'testbeds');
	my $natehostpath=substr($opts{'testbed'},0,$natehostindex+length("testbeds"));
	my $natehostfiles="$natehostpath/".'nate_hosts';
	my @natehostarray=("$natehostfiles");
	my $nate =join(";",@natehostarray);
	my $DISCRG;	
	
	## If groupname is not defined , the configbroker library gives a default name of discover_<nodename>
	my $returnmessage;
	my $testbeddir=$opts{'testbed'};
	my $distype = $opts{'distype'};
	my @disarray=split(' ',$distype);
	my $groupname = $opts{'name'};
	my $nodename=$opts{'node'};
	$ENV{'FILER'} = "$nodename";
	my $p = NATE::ParamSet->new(global=>1);

		eval
			{
	## Setting NATE_HOSTS and NATE_Lib files to appropriate values
		$p->set (key =>"NATE_LIB", value =>"/usr/software/test/agnostic"); ## This needs to be changed once all changes to CB are checked in .
		Tharn::param('NATE_HOSTS', '-set',Tharn::param('NATE_HOSTS').";$nate");
		if($nodename){
		my $cb = ConfigBroker->new("$testbeddir");
			
			## Condition to check if user wants to add all discover and add all NACL resources or specific resources
			
			if ($opts{'distype'} eq 'all')
				{
					$DISCRG = $cb->discover_and_create_resource_group(groupname => "$groupname",,reuse =>"$opts{'reuse'}",share =>"$opts{'share'}");
    			}
			
			else
				{

       				 $DISCRG = $cb->discover_and_create_resource_group(groupname => "$groupname",reuse =>"$opts{'reuse'}",share =>"$opts{'share'}",nodename =>"$nodename",objarrayref => \@disarray);
  
				}

				if($DISCRG =~ /.*HASH\(\w+\)/i)
					{
						$returnmessage=" Resource Group Created ".$groupname;}
						
				else 
							{
								$returnmessage=$DISCRG;
							}}
							else {
								$returnmessage="Specify a node name to run discover on";
							}
			};
    if ($@) 
    {
        print "Error: $@";
        exit;
    };				
					$Log->exit();
					return "$returnmessage";
}


## Function to get all the resource group which is present in the database . This is to be displayed on the left pane of the GUI
## to list all the resource groups like a tree structure in GUI.

sub get_resourcegroup_names{
	my %opts = (
		testbed =>undef,	
		@_,
	);
	
	$Log->enter();
	$Log->comment("################################RETRIEVING RESOURCE GROUP NAME FROM CB Db#################################");
	my @rg;
	my $index=0;
	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my @rgs = $cb->get_all_resource_groups;

	foreach my $resource_name(@rgs){
		$rg[$index]=$cb->get_resource_group(name => "$resource_name");
		$index=$index+1;
	}
	
		$Log->exit();
		return @rg;
}

## Used to update the status,share,reuse of the resource group. The possible status of the resources are ACTIVE,INUSE and PENDING.
## The possible value for resue is REUSE,NOREUSE,CREATENEW .The possible values of share are SHARE,EXCLUSIVE.

sub update_resourcegroup{
	my %opts = (
		name =>undef,
		testbed =>undef,
		status =>undef,	
		reuse => undef,
		share =>undef,
		@_,
		);

	
	$Log->enter();
	$Log->comment("################################UPDATING STATUS,SHARE AND REUSE OF RESOURCE GROUP #################################");
	my $testbeddir=$opts{'testbed'};
	my $returnmessage;
	my $resource_group=$opts{'name'};
	eval{
		my $cb = ConfigBroker->new("$testbeddir");
		my $rg = $cb->get_resource_group(name => "$resource_group");
		$returnmessage =$rg->edit_resourcegroup(newstatus => "$opts{'status'}",newreuse => "$opts{'reuse'}",newshare => "$opts{'share'}");
		};
    
    if ($@) 
    {
        print "Error: $@";
        exit;
    };
	if($returnmessage ==0)
	{
		$returnmessage ="Status of resource group ".$resource_group." Updated";
	} 
	
	else 
	{
		$returnmessage ="Update Status failed . Please see log for more details";
	}

		$Log->exit();
		return "$returnmessage";
}

##Function to reserve a resource group with a reservation code . The reservation code is supplied by the user.This is used in
## edit resource group in GUI.

sub reserve_resourcegroup{
	
	my %opts = (
		name =>undef,
		testbed =>undef,
		reservecode =>undef,	
		@_,
		);

	
	$Log->enter();
	$Log->comment("################################ RESERVING RESOURCE GROUP #################################");
	my $testbeddir=$opts{'testbed'};
	my $returnmessage;
	my $cb = ConfigBroker->new("$testbeddir");
	my $rg = $cb->get_resource_group(name => "$opts{'name'}");
	
		if($opts{'reservecode'})
			{
				$returnmessage =$rg -> reserve(reserve_code => "$opts{'reservecode'}");
			}
		else 
			{
				$returnmessage=2;
			}
	
	if($returnmessage ==0)
		{
			$returnmessage ="Resource group "."$opts{'name'} ". "reserved with code "."$opts{'reservecode'}";
		} 
	elsif($returnmessage ==2) 
		{
			$returnmessage="Reserve code is empty or not defined";
			
		}
	else
		{
			$returnmessage ="Reservation failed . Please see log for more details";
		}

		$Log->exit();
		return "$returnmessage";

}

## Function to unreserve a reserve group . The reservation code is removed from the resource group and the resource group is put 
## to ACTIVE state. This is used in Edit Resource group in GUI.

sub unreserve_resourcegroup{
	my %opts = (
		name =>undef,
		testbed =>undef,
		reservecode =>undef,	
		@_,
		);
		
	$Log->enter();
	$Log->comment("################################ UNRESERVING RESOURCE GROUP #################################");
	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $rg = $cb->get_resource_group(name => "$opts{'name'}");
	my $returnmessage =$rg -> unreserve(reserve_code => "$opts{'reservecode'}");
	
	if($returnmessage ==0)
		{
			$returnmessage ="Resource group "."$opts{'name'} ". "unreserved with code "."$opts{'reservecode'}";
		} 
	else 
		{
			$returnmessage ="Un-Reservation failed . Please see log for more details";
		}
	
		$Log->exit();
		return "$returnmessage";
}

## Used to delete a resource group .NOTE: This function deletes the resource group from CB db only and it does not do clean up of resources on filer

sub delete_resource_group{
	my %opts = (
		name =>undef,
		testbed =>undef,
		cleanupfile =>undef,	
		@_,
		);
		
	$Log->enter();
	$Log->comment("################################ DELETING RESOURCE GROUP #################################");	
	my $testbeddir=$opts{'testbed'};
	my @resource_group=split(" ",$opts{'name'});
	my $cb = ConfigBroker->new("$testbeddir");
	my $returnmessage="";
	for my $rgname(@resource_group)
		{
	 $returnmessage = $cb->delete_resource_group(groupname => "$rgname");
		
	if($returnmessage ==0)
		{
			$returnmessage ="Resource group deleted $opts{'name'} ";
		} 
	else 
		{
			$returnmessage ="Delete failed . Please see log for more details";
		}
		}
		
		$Log->exit();
		return "$returnmessage";
}

## Function used to create a resource type table . This can be custom tables apart from the normal NACL object tables.


sub create_resource_type{
	my %opts = (
		name =>undef,
		testbed =>undef,
		primarykey =>undef,	
		tabledefn =>undef,
		@_,
		);
		
	
	$Log->enter();
	$Log->comment("################################ CREATING RESOURCE TYPE TABLE #################################");
	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $returnmessage;
	my $primary;
	
	## The primary keys include by default the resourcegroup and name . User can add primary keys to it .
	if($opts{'primarykey'})
		{
			 $primary="$opts{'primarykey'},"."name,resourcegroup";
		}
	else {
		 $primary="name,resourcegroup";
	}
	my @primary_keys = split(",","$primary");
	
	##By default the tabledefn will have following attributes : name,resourcegroup,_status,_comment,_priority.
	my @tabledefn = split(",","$opts{'tabledefn'}");
	my $customtype_name= "$opts{'name'}";
	if($customtype_name){
	 $returnmessage = $cb ->create_resource_type(name => $customtype_name, primarykey => \@primary_keys ,tabledefn => \@tabledefn);

	if($returnmessage ==0)
		{
			$returnmessage ="New Resource Type Created ".$customtype_name;
		} 
		else 
		{
			$returnmessage ="New Resource Type creation failed. Please see log for more details";
		}
	}
		
		else 
		{
			$returnmessage ="Specify a name for the custom resource type";
		}
	

		$Log->exit();
		return $returnmessage;
}

## Deletes a resource type which is created on ConfigBroker DB .This returns all the resource type table names created by the user only

sub delete_resource_type
	{
	my %opts = (
		resourcetype =>undef,
		testbed =>undef,
		@_,
	);
	
	$Log->enter();
	$Log->comment("################################ DELETING RESOURCE TYPE TABLE #################################");
	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my @resourcetypearray=split(' ',$opts{'resourcetype'});
	my $returnmessage;

		for my $type(@resourcetypearray)
			{
				$returnmessage = $cb->delete_resource_type(name => "$type");
			}
	if($returnmessage ==0)
	{
		$returnmessage ="New Resource Type deleted "."$opts{'resourcetype'}";
	} 
	else 
	{
		$returnmessage ="New Resource Type deletion failed. Please see log for more details";
	}
	
		$Log->exit();
		return "$returnmessage";
}

## Function used to retrive the custom reosource table names created by the user . It is used in main.cgi for user to choose to delete a custom resource type

sub get_custom_resourcetypes{
	my %opts = (
		testbed =>undef,
		@_,
		);
	
	$Log->enter();
	$Log->comment("################################ RETIEVING CUSTOM RESOURCE TYPE NAMES#################################");

	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $schema_object=new ConfigBroker::Schema();
	my @types=$schema_object->_get_nacl_resource_names();
	my $alltypes=$cb ->get_typename();
	my @smalltype;
	my @custom_resourcetypes;	
		for my $type(@types)
			{
				push(@smalltype,lc($type));
			}
		for my $type(@$alltypes)
			{
				
				if(!grep /^$type$/,@smalltype) {
					push(@custom_resourcetypes,$type);
				}
			}
			
				
				
		$Log->exit();
		return @custom_resourcetypes;
}

## Function used to retrieve the primary keys for resource types in the CB table from Schema.pm. This is to display all the resource
## types without name by a combination of primary keys 

sub get_primarykeys{
	my %opts = (
		typename =>undef,
		@_,
		);
	
	$Log->enter();
	$Log->comment("################################ RETIEVING PRIMARY KEYS FOR THE DATABASE#################################");
	my $typename=$opts{'typename'};
	$Log->comment($typename);
	my $primary_keys=&ConfigBroker::Schema::get_primary_keys($typename);
	$Log->comment(Dumper($primary_keys));

	$Log->exit;
	return $primary_keys;				

}
	
## Function to retreive all resource types names which is the same as the table name in ConfigBroker DB. This is used in multiple
## functions where the user is given an option to choose resource type.
sub get_all_resourcetypes{
	my %opts = (
		testbed =>undef,
		@_,
		);	
		
		$Log->enter();
	$Log->comment("################################ RETIEVING ALL RESOURCE TYPES FROM CB Db #################################");

	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $types = $cb ->get_typename();
	
	$Log->exit();
	return @$types;
}

## Function used to get all resource fields from the resource table for custom resource type. This is displayed as a form 
## to get user input to insert resource records to field.

sub get_all_custom_restypefields{
	my %opts = (
		testbed =>undef,
		resource_type=>undef,
		@_,
		);	
		
		$Log->enter();
	$Log->comment("################################ RETIEVING CUSTOM RESOURCE TYPE FIELDS FROM CB Db #################################");
	
	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $types = $cb->get_customtype_fields($opts{'resource_type'});

	$Log->exit();
	return @$types;
}

#Function to get all resource fields of particular resource type . Used in Add resources in leftpane.cgi

sub get_resource_type_names{
	my %opts = (	
		testbed =>undef,
		typename =>undef,
		@_,
		);	

$Log->enter();
	$Log->comment("################################ RETRIEVING RESOURCE TYPES NAMES FROM TABLE #################################");

	my $testbeddir=$opts{'testbed'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $type_names = $cb->get_resname_type("$opts{'typename'}");

	$Log->exit();
	return $type_names;
}

##Function used to retrieve only the resource types present in a resource group .Used to print all the resource types under a 
## resource group on the leftpane of the gui

sub get_resourcegroup_type{

	my %opts = (
		name =>undef,
		testbed =>undef,
		@_,
	);
	
	$Log->enter();
	$Log->comment("################################ RETRIEVE ALL RESOURCE TYPES UNDER A RESOURCE GROUP #################################");

	my $testbeddir=$opts{'testbed'};
	my $rgname =$opts{'name'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $types = $cb->get_typename();
	
	my $resource_group;
	$resource_group=$cb->get_resource_group(name=>$rgname);
	
	my @rgtype;
	
	for my $typ(@$types)
		{
			
			$Log->comment("Type= ".$typ);
			my $resarrayref=$resource_group->get_resources(type=>$typ);
			my $resourcehash=@$resarrayref;
			##Pushes only resourcetypes which has particular entry in the resource group
				if($resourcehash)
					{
						push(@rgtype,$typ);
					}
		}

			$Log->exit();
			return @rgtype;

		}

## Get resourcenames as stored in the NACL libraries for discovering and creating resource group .It retrieves all the types from Schema.pm
## For eg :to discover aggregates we need to input in format "Aggregates"

sub get_resourcetypes_discover{
	my %opts = (
		testbed =>undef,
		@_,
		);
		
		$Log->enter();
	$Log->comment("################################ RETRIEVING ALL RESOURCE TYPES FOR DISCOVER FUNCTION #################################");

	my $testbeddir=$opts{'testbed'};
	my $schema_object=new ConfigBroker::Schema();
	my @types=$schema_object->_get_nacl_resource_names();
	

	$Log->exit();
	return @types;
}

##Get all resources of particular type in form of hash to print out in form of tables . It is used in lefpane.cgi to display
## all the resource type table in a paritcular resource group

sub get_resource_fields{
	my %opts = (
		name =>undef,
		type =>undef,
		testbed =>undef,
		@_,
		);
			
	my $testbeddir=$opts{'testbed'};
	my $rgname =$opts{'name'};
	my $restype=$opts{'type'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $resource_group=$cb ->get_resource_group(name=>$rgname);
	my $resarrayref=$resource_group->get_resources(type=>$restype);
	return $resarrayref;
}

## Get all resource fields in form of table when resource type and resource field name is given . It is used in lefpane.cgi
##to display individual tables of particular name .
sub get_resource_fields_name{
	my %opts = (
		resource_name =>undef,
		name =>undef,
		type =>undef,
		testbed =>undef,
		@_,
	);
	
	$Log->enter();
	$Log->comment("################################RETIREVING ALL RESOURCE NAMES FROM THE CB Table #################################");

	my $resource_name =$opts{'resource_name'};
	my $testbeddir=$opts{'testbed'};
	my $rgname =$opts{'name'};
	my $typename=$opts{'type'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $resarrayref;
	my $matchstring="";
	my $resource_group=$cb->get_resource_group(name=>$rgname);
	
	##Used to split the resource identifier into an array as some of the resource identifier maybe a combination of primary keys
	## when the name parameter is not defined for the resource
	
	my @resource_identifier=split(",",$resource_name);
	
	## Get primary keys for uniquely identifying a resource in the resource table
	
	my $primary_key_array=get_primarykeys(typename=>$typename);
	my @primary_keys=@$primary_key_array;
	my $index=0;
	
	for my $resourcekeys(@primary_keys)
	{
		if($resourcekeys ne 'resourcegroup'){
			if ($resource_identifier[$index] eq ""){
					$resource_identifier[$index]="''";	
			}else{
		$matchstring=$matchstring."$resourcekeys"."="."$resource_identifier[$index] ";
			}
		$index=$index+1;
	
		}
	}
	##Creates hash which is passes as match filter to get the matching resource
	
	my %match_filter = map {split("=",$_)} split(" ",$matchstring);
	$Log->comment(%match_filter);
	$resarrayref=$resource_group->get_resources(type => $typename,count=>0,match_filters => \%match_filter);	
	$Log->comment("Res".Dumper($resarrayref));
	return $resarrayref;
}




## Function used to delete a particular resource name from a resource group only . It deletes the entry from the database only.

sub remove_resource_name{
	my %opts = (
		resourceidentifiers =>undef,
		groupname =>undef,
		restype =>undef,
		testbed =>undef,
		selection_resources => undef,
		@_,
	);

$Log->enter();
	$Log->comment("################################ DELETING RESOURCE NAMES FROM THE CB Table #################################");

	#my $resource_identifier =$opts{'resourceidentifier'};	
	my $testbeddir=$opts{'testbed'};
	my $rgname =$opts{'groupname'};
	my $typename=$opts{'restype'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $matchstring="";
	my $returnmessage;
	my $result;
	my $resource_group=$cb->get_resource_group(name=>$rgname);
	
	##Used to convert the select resource numbers to array ,so that the operation can be performed on a particular resource sequentially
	
	my @selection=split(" ",$opts{'selection_resources'});
	my @resource_identifierarray=split(" ",$opts{'resourceidentifiers'});
		
		##Check condition to see if a selection is made on submitting the form 
			if(@selection)
				{
		
		## Do the remove operation on each of the selected resource 
					for my $select(@selection)
						{	
	##Used to split the resource identifier into an array as some of the resource identifier maybe a combination of primary keys
	## when the name parameter is not defined for the resource
	
		my @resource_identifier=split(",",$resource_identifierarray[$select]);
		
		## Get primary keys for uniquely identifying a resource in the resource table
	my $primary_key_array=get_primarykeys(typename=>$typename);
	my @primary_keys=@$primary_key_array;
	my $index=0;
	
	for my $resourcekeys(@primary_keys)
	{
		if($resourcekeys ne 'resourcegroup'){
			if ($resource_identifier[$index] eq ""){
					$resource_identifier[$index]="''";	
			}else{
				## Makes the hash element to be added to as part of match filters
		$matchstring=$matchstring."$resourcekeys"."="."$resource_identifier[$index] ";
			}
		$index=$index+1;
	
		}
	}
	
	##Adds element to match_filter 
	my %match_filter = map {split("=",$_)} split(" ",$matchstring);
	$Log->comment(%match_filter);
	$returnmessage=$resource_group->delete_resources(type => $typename,match_filters => \%match_filter);	

				
			if ($returnmessage ==0)
			{
				$result .="Resource $resource_identifierarray[$select] "." deleted from "."$rgname. ";
			}
			
			else 
			{
				$result .="Delete failed on $resource_identifierarray[$select] . Check Log to see error messages. ";
			}
	}}
		else 
		{
			$result="Select a resource type to delete";
		}
		
		$Log->exit();
		return $result;
}




## Function used to modify resource . As of now we are disabling all fields and user can only change the resource status 
## and resource group name .
sub modify_resource{
	
	my %opts = (
		selection_resources => undef,
		resource_identifiers =>undef,
		resource_status =>undef,
		rg_grp=>undef,
		groupname =>undef,
		restype =>undef,
		testbed =>undef,
		@_,
		);
		
	$Log->enter();
	$Log->comment("################################ MODIFYING RESOURCE IN RESOURCE GROUP #################################");

	
	my $resource_status =$opts{'resource_status'};
	my $testbeddir=$opts{'testbed'};
	my $typename=$opts{'restype'};
	my $rg_grp=$opts{'rg_grp'};
	my $cb = ConfigBroker->new("$testbeddir");
	my $matchstring;
	my $returnmessage;
	my $result;
	my $resource_group=$cb->get_resource_group(name=>$rg_grp);
	my @rgs=$cb->get_all_resource_groups();
	
	##Used to convert the select resource numbers to array ,so that the operation can be performed on a particular resource sequentially
	my @selection=split(" ",$opts{'selection_resources'});
	my @resource_identifierarray=split(" ",$opts{'resource_identifiers'});
	my @resource_grouparray=split(" ",$opts{'groupname'});
		
		##Check condition to see if a selection is made on submitting the form 
		if(@selection)
				{
		## Do the remove operation on each of the selected resource 
				for my $select(@selection)
					{	
	##Used to split the resource identifier into an array as some of the resource identifier maybe a combination of primary keys
	## when the name parameter is not defined for the resource
	
		my @resource_identifier=split(",",$resource_identifierarray[$select]);

	
	## Get primary keys for uniquely identifying a resource in the resource table
	my $primary_key_array=get_primarykeys(typename=>$typename);
	my @primary_keys=@$primary_key_array;
	my $index=0;
	
	for my $resourcekeys(@primary_keys)
	{
		if($resourcekeys ne 'resourcegroup'){
			if ($resource_identifier[$index] eq ""){
					$resource_identifier[$index]="''";	
			}else{
		$matchstring=$matchstring."$resourcekeys"."="."$resource_identifier[$index] ";
			}
		$index=$index+1;
	
		}
	}
	my %match_filter = map {split("=",$_)} split(" ",$matchstring);

	if(grep /^$resource_grouparray[$select]$/,@rgs)
	 {
	$returnmessage=$resource_group->modify_resources(type => $typename, match_filters => \%match_filter,modify_info =>{resourcegroup=>$resource_grouparray[$select]});
		}

	else 
	{
		$returnmessage=2;
	}


	if ($returnmessage ==0)
	{
		$result .="Resource updated $resource_identifierarray[$select] ."."\n";
	}
	elsif ($returnmessage ==2)
	{
	$result .="Update failed on resource $resource_identifierarray[$select],The $resource_grouparray[$select] is not present in database \n.";
	}
	else
	{	
	$result .="Update failed $resource_identifierarray[$select] . Check Log to see error messages \n.";
	}
					}
				}
	else 
	{
	$result="Select a resource type to modify";
	}

		$Log->exit();
		return $result;
}



## Function used to add existing resource in the database to a resource group .
sub add_resource{
	my %opts = (
		resourceidentifier =>undef,
		groupname =>undef,
		restype =>undef,
		testbed =>undef,
		@_,
		);

	$Log->enter();
	$Log->comment("################################ ADDING RESOURCE TO A  RESOURCE GROUP #################################");

	my @resource_identifier =split(" ",$opts{'resourceidentifier'});
	my $testbeddir=$opts{'testbed'};
	my $rgname =$opts{'groupname'};
	my $typename=$opts{'restype'};
	my $returnmessage;
	my $resarrayref;
	my $cb = ConfigBroker->new("$testbeddir");
	my $resource_group=$cb->get_resource_group(name=>$rgname);
	for my $resource_identifier(@resource_identifier){
	my $matchstring;
	
	
	##Used to split the resource identifier into an array as some of the resource identifier maybe a combination of primary keys
	## when the name parameter is not defined for the resource
	
	my @resource_identifier=split(",",$resource_identifier);
	
	## Get primary keys for uniquely identifying a resource in the resource table
	my $primary_key_array=get_primarykeys(typename=>$typename);
	my @primary_keys=@$primary_key_array;
	my $index=0;
	
	for my $resourcekeys(@primary_keys)
	{
		if($resourcekeys ne 'resourcegroup'){
			if ($resource_identifier[$index] eq ""){
					$resource_identifier[$index]="''";	
			}else{
		$matchstring=$matchstring."$resourcekeys"."="."\'$resource_identifier[$index]\' ";
			}
		$index=$index+1;
	
		}
	}
	
	##Creats the matching condition to retrieve the resource arrayref from the database
	$matchstring=substr $matchstring,0,length($matchstring)-1;
	$matchstring=~ s/ / AND /g;
	$resarrayref=$cb ->get_resource_fields_name("$typename","$matchstring");
	$returnmessage=$resource_group->add_resources(resource_type=>$typename,resourcearrayref => [$resarrayref]);

	if ($returnmessage ==0)
	{
		$returnmessage ="Resource added to "."$rgname";
	}
	else 
	{
	$returnmessage="Addition ".$resource_identifier."failed . Check Log to see error messages";
	}

	}

	$Log->exit();
	return $returnmessage;
	
}



##Function to add new custom type resource . It displays a form to the user and then gets user input in form of hash to update
## the resource table

sub add_new_resource{
	my %opts = (
		groupname =>undef,
		restype =>undef,
		testbed =>undef,
		resoucehash=>undef,
		@_,
		);
		
		$Log->enter();
	$Log->comment("################################ADDING CUSTOM RESOURCE TO A RESOURCE GROUP #################################");

	my $resourcehash =$opts{'resourcehash'};
	my $testbeddir=$opts{'testbed'};
	my $rgname =$opts{'groupname'};
	my $restype=$opts{'restype'};
	my $returnmessage;
	my %resource=%$resourcehash;
	if($resource{'name'}){
	my $cb = ConfigBroker->new("$testbeddir");
	my $resource_group=$cb->get_resource_group(name=>$rgname);

	$returnmessage=$resource_group->add_resources(resource_type=>$restype,resourcearrayref => [$resourcehash]);

	if ($returnmessage ==0)
		{
			$returnmessage ="Resource added to "."$rgname";
		}
	else 
		{
			$returnmessage="Addition failed . Check Log to see error messages";
		}
	}
	else 
		{
			$returnmessage="Name is a primary key. Need to give a name for the resource type to add to database";
		}

	$Log->exit();
	return $returnmessage;
}

=head1 ROUTINES

=head2 log_creation

=head3 Usage

CBTemplate::log_creation(testbed => "$testbed");

$opts{'testbed'} = Mandatory Parameter. Testbed location . It creates a log file for a session under a directory called cbgui with filename being a default time stamp
eg : /x/eng/rtpeng-testbeds/csiqa/rtpeng_csiqa_sreekuma_vsim12/cbgui/cbgui1345036304.log

=head3 Return

Returns the location of logfile .

=head2 create_resource_group

    Object method

=head3 Usage

   	CBTemplate::create_resource_group(name => "$groupname",testbed =>"$testbed",reuse =>"$reuse",status => "$status",share=>"$share");
    $opts{'groupname'} - name of the resource group.  This is a mandatory parameter.
    $opts{'reuse'}  - Whether the resource group can be reused if it was already in the DB. 
    $opts{'share'} - Whether this resource group can be shared. 
    $opts{'status'}- Status of the resource group to be set.


=head3 Return

    Returns the message if the resource group is created or not.

=head2 discover_create

	Method
	
=head2 Usage
    
    CBTemplate::discover_create(name => "$groupname",testbed =>"$testbed",node=>"$nodename",reuse=>"$reuse",share=>"$share",distype=>"@distype")
    
    $opts{'name'} - Name of the resource group to be created.
    $opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
    $opts{'reuse'}  - Whether the resource group can be reused if it was already in the DB. 
    $opts{'share'} - Whether this resource group can be shared.
    $opts{'distype'} - Array reference to the NACL objects which needs to be discovered and added as part of the resource group.
     
=head3 Return

	 Returns the message if the resource group is created or not.
	
=head2 update_resourcegroup

	Method
	
=head2 Usage 

	CBTemplate::update_resourcegroup(name => "$groupname",testbed =>"$testbed",status =>"$status",share =>"$share",reuse =>"$reuse");
	 $opts{'name'} - Name of the resource group to be updated.
    $opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
    $opts{'reuse'}  - Whether the resource group can be reused if it was already in the DB. 
    $opts{'share'} - Whether this resource group can be shared.	
     $opts{'status'}- Status of the resource group to be set.

=head3 Return

	Returns the message if the resource group parameters are updated or not .
	
=head2 reserve_resourcegroup

=head2 Usage

	CBTemplate::reserve_resourcegroup(name => "$groupname",testbed =>"$testbed",reservecode =>"$reservecode");
	
	$opts{'name'} - Name of the resource group to be updated.
    $opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
    $opts{'reservecode'} - Reservation code supplied by the user . On accessing this function the user changes the status of
    the resource group to INUSE_<reservecode>

=head3 Return

	Returns the message if the resource group parameters are updated or not .

=head2 unreserve_resourcegroup

	Method
=head2 Usage

	CBTemplate::unreserve_resourcegroup(name => "$groupname",testbed =>"$testbed",reservecode =>"$reservecode");
	
	$opts{'name'} - Name of the resource group to be updated.
    $opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
    $opts{'reservecode'} - Reservation code supplied by the user . On accessing this function the user changes the status of
    the resource group to ACTIVE.

=head3 Return

	Returns the message if the resource group parameters are updated or not .

=head2 delete_resource_group

	Method
	
=head2 Usage 

	CBTemplate::delete_resource_group(name => "$groupname",testbed =>"$testbed",cleanupfile =>"$cleanupfile");
	
	$opts{'name'} - Name of the resource group to be deleted
	$opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
	$opts{''cleanupfile} - Optional.Not implemented currently in GUI now
	
=head3 Return

	Returns the message if the resource group parameters are updated or not .
	
=head2 create_resource_type

	Method
	
=head2 Usage 

	CBTemplate::create_resource_type(name => "$name",testbed =>"$testbed",primarykey =>"$primarykey",tabledefn =>"$tabledefn");
	
		$opts{'name'} - Name of the resource type to be created
	$opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
	$opts{'primarykey'} - List of primary keys to be added for this resource type . By default it will contain the resourcegroup and the name of the resource.
	$opts{'tabledefn'} - List of columns to be part of the table which defines the attribute of the resource type.
	
=head3 Return 

	Returns the message if the resource group parameters are updated or not .
	
=head2 delete_resource_type

	Method
	
=head2 Usage 

	CBTemplate::delete_resource_type(resourcetype => "@resourcetype",testbed =>"$testbed");
	
	$opts{'resourcetype'} - Array reference to the type of the resource to be deleted from the table.
	$opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
	
=head3 Return 

	Returns the message if the resource group parameters are updated or not .
	
=head2 remove_resource_name

	Method
	
=head3 Usage

	CBTemplate::remove_resource_name(resourceidentifier => "$res_identifier",testbed =>"$testbed",restype =>"$restype",groupname =>"$groupname")
	
	$opts{'selection_resources'}--Selected resource no on which the operation needs to be performed.
	$opts{'resourceidentifier'} - Unique way to identify a resource . Some of the resource dont have a name defined , so we identify those resource
	by the combination of primary keys as defined in Schema.pm
	$opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
	$opts{'restype'} - Type of resource to be identified.
	$opts{'groupname'} - Name of the resourcegroup from which the resource needs to be removed.
	
=head3 Return 

	Returns the message if the resource group parameters are updated or not .
	
=head2 modify resource

=head2 Usage

	CBTemplate::modify_resource(rg_grp=>"$rg_grp",resource_identifier => "$res_identifier",resource_status=>"$resource_status",testbed =>"$testbed",restype =>"$restype",groupname =>"$groupname")
	
	$opts{'selection_resources'}--Selected resource no on which the operation needs to be performed.
	$opts{'rg_grp'} - Name of the resource group where the resource needs to be modified.
	$opts{'resourceidentifier'} - Unique way to identify a resource . Some of the resource dont have a name defined , so we identify those resource
	by the combination of primary keys as defined in Schema.pm
	$opts{'resource_status'} - Status of the resource which needs to be modified,
	$opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
	$opts{'restype'} - Type of resource to be identified.
	$opts{'groupname'} - Name of the resource group to be updated in the resource table
	
=head3 Return 

	Returns the message if the resource group parameters are updated or not .
	
=head2 add_resource 

	Method
	
=head2 Usage

	CBTemplate::add_resource(groupname=>"$groupname",resourceidentifier => "@resource_identifier",testbed =>"$testbed",restype =>"$restype")

	$opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
	$opts{'restype'} - Type of resource to be added.
	$opts{'groupname'} - Name of the resource group to which the resource is to be added in the resource table
	$opts{'resourceidentifier'} - Unique way to identify a resource . Some of the resource dont have a name defined , so we identify those resource
	by the combination of primary keys as defined in Schema.pm

=head3 Return 

	Returns the message if the resource group parameters are updated or not .
	
=head2 add_new_resource

	Method
	
=head Usage

	CBTemplate::add_new_resource(restype =>$resource_type,testbed=>"$testbed",groupname=>"$groupname",resourcehash=>\%resourcehash)
	
	$opts{'testbed'} - Testbed location so that ConfigBroker can look for the database.
	$opts{'restype'} - Type of resource to be added.
	$opts{'groupname'} - Name of the resource group to which the resource is to be added in the resource table
	$opts{'resourceidentifier'} - Unique way to identify a resource . Some of the resource dont have a name defined , so we identify those resource
	by the combination of primary keys as defined in Schema.pm
	$opts{'resourcehash'}- Hash reference of the resource which needs to be added as part of the resource group.

=cut
1;
