package Conlog;

use strict;                                     # Always use strict!
use Options;
use Tharn;
use Data::Dumper;


require Exporter;                               # Exporter allows us to provide
# functionality to scripts.

our @ISA = qw(Exporter);                        # Specify this package as one the
# an Exporter derived type.

our @EXPORT = qw();                             # Add our package variable to the
# list of symbols automatically 
# exported.

our @EXPORT_OK = qw();                          # Add the subroutine to the list
# of subs available for export.

#Setup some package global variables											
my $Options = Options->new();
my $Trace = TracePkg->new();

#Grab these parameters passed in from either Nate or the Testbed
$Options->parameters(definitions=>
	[
	{	
		name => "TEST_NAME",
		description	=> "NATE's TEST_NAME variable", 
		default		=> undef,
	},
	{
		name		=> "LOGDIR",
		description	=> "NATE's LOGDIR variable", 
		default		=> undef,
	},
    {
		name		=> "CONSERVER",
		description	=> "Name of this site's console server (set in the testbed)", 
		default		=> undef,
	},
		{
		name		=> "CONSERVER_LOGDIR",
		description	=> "What directory on the conserver are the logs stored in? (set in the testbed)", 
		default		=> undef,
	},
	]
);

$Options->process();
my $P = $Options->get_hash();
my %Ssh_handle;
my $Handle=undef;


#ssh to a conserver unless there is already a line open
#
#Takes no params (reads conserver from testbed)
#
#returns either a handle on success, or -1 on failure.
sub _connect {
	$Trace->entry();


	if (defined($Handle)) {
		$Trace->trace(msg=>"Handle was defined");
		if ($Handle==-1) {
			return -1;#tell the rest of the attempts not to bother.
		} else {
			return $Handle;
		}
	}
	elsif (not defined $P->{'CONSERVER'}) { 
		$Trace->warn(msg=>"A console server was not found.  This is a setting that should be made in the testbed.\n  ex.  CONSERVER=rtpgx-smk1  (is the value set in the .ntest.tb file for the RTP smokes testbed.) ");
		return -1; #tell the rest of the attempts not to bother.
	} else {
		$Trace->trace(msg=>"Handle was undefined");
		my $localhandle=undef;
		host($P->{'CONSERVER'});
		eval {
			$localhandle=connect("#ssh"); #connect to the machine we just ran host on
			$Trace->debug(msg=>"immediate value of localhandle=$localhandle");
		};  
		if ($@) {
            $Trace->fatal(msg=>"Error: I was Unable to connect to the console logging server.  This is an infrastructure problem.  That service should be available.  Without it, scripts like did_bad_happen can't detect why a failure happened. (because it uses this script to mark the spot where this run started writing to the console log) The error was: '$@'");
			return -1;#tell the rest of the attempts not to bother.
		} else {
			return $localhandle;
		}

	}
}



#what is the filesize of the console log for this node?
sub filesize {
	$Trace->entry();
	my %params=@_; # valid params: node


	$Handle=_connect() unless defined($Handle);
	$Trace->debug(msg=>"returned value of Handle=$Handle");

	if ($Handle != -1) { #if we have a good handle
		my @rvalue;

		eval {
			@rvalue=say($Handle,"ls -l $P->{CONSERVER_LOGDIR}/$params{node} | awk '{print \"X;X\" \$5 \"X;X\"}'",'-swallow'); #separate the filesize for easy grabbing
		};
		if ($@) {
			$Trace->warn(msg=>"Failed to grab the filesize, error is $@");
			$Handle=-1; #tell future requests not to bother
			return -1;
                        
		} else {

			if ($rvalue[2] =~ /X;X(\d+)X;X/) { #grab the filesize
				$Trace->debug(msg=>"string=$rvalue[2]");
				$Trace->debug(msg=>"size=$1");
				return $1;
			} else {
				#We might have got here for a few reasons:
				#1) the console server is up and running, but not configured to log for this node
				#2) The logs were just rotated, and nothing has written to this file yet

				$Trace->warn(msg=>"I was able to run ls on $P->{CONSERVER_LOGDIR}/$params{node} but found no filesize.  Not a connection problem.  The file might not be there.\n");
				return undef;
			}
		}
	}
}


#Grab a section of the console server's log file, and write it to a file in the logdir
sub print_section_of_log {
	$Trace->entry();
	my %params=@_; # valid params: node, start, finish

	debug_print();

	$Handle=_connect unless defined($Handle);

	if ($Handle != -1) { #if we have a good handle
        if ( defined(my $dest_dir=Testinfo::dir()) ) {
            Testinfo::make_open_dir("$dest_dir/$params{node}");
            eval {
                say($Handle,"sudo /x/eng/localtest/noarch/bin/point2point --infile $P->{CONSERVER_LOGDIR}/$params{node} --outfile $dest_dir/$params{node}/console.log --start $params{start} --finish $params{finish}",'-swallow');
                $Trace->debug(msg=>"returning: $dest_dir/$params{node}/console.log");
            };
            if ($@) {
                $Trace->warn(msg=>"Unable to connect to the console logger to print the section of log, error is $@");
                return "Error: No file written, couldn't connect to the console logging server.";
            } else {
                return ("$dest_dir/$params{node}/$params{node}/console.log");
            }
        } else {
            $Trace->exit(msg=>"unable to get a directory to write to.  Check the logs from Testinfo.pm to see why.", result=>"FAIL");
        }
    }
}

#Delete all but the most recent 10mg of the log to keep it from growing too large
sub truncate_log {
	$Trace->entry();
	my %params=@_; # valid params: node, start, finish
	debug_print();

	$Handle=_connect unless defined($Handle);

	if ($Handle != -1) { #if we have a good handle
            my $tempfile = "temp_" . $params{node};
            #Copy last 10mb of file to temp file
            eval {
                say($Handle,"sudo /x/eng/localtest/noarch/bin/point2point --infile $P->{CONSERVER_LOGDIR}/$params{node} --outfile $P->{CONSERVER_LOGDIR}/$tempfile --start $params{start} --finish $params{finish}",'-swallow');
            };
            if ($@) {
                $Trace->warn(msg=>"Unable to connect to the temp file to copy the conserver file, error is $@");
                return -1;
            }
            #Overlay conserver file with newly created temp file
            eval {
                say($Handle,"sudo /x/eng/localtest/noarch/bin/point2point --infile $P->{CONSERVER_LOGDIR}/$tempfile --outfile $P->{CONSERVER_LOGDIR}/$params{node} --start 0 --finish 10000000",'-swallow');
            };
            if ($@) {
                $Trace->warn(msg=>"Unable to connect to the conserver file to copy the temp file, error is $@");
                return -1;
            }
            #Delete temp file
	    eval {
		say($Handle,"sudo rm $P->{CONSERVER_LOGDIR}/$tempfile",'-swallow');
	    };
	    if ($@) {
		$Trace->warn(msg=>"Failed to delete the temp file, error is $@");
		return -1;
            }
        }
}


sub debug_print {
	$Trace->comment(msg=>"testname=" . $P->{'TEST_NAME'});
	$Trace->comment(msg=>"logdir=" . $P->{'LOGDIR'});
}


#Write a marker line into the console server log file for this node
sub write_test_starting2log {

	my %params = @_; #valid params: node
	$Trace->entry();

	$Handle=_connect unless defined($Handle);

	if ($Handle != -1) { #if we have a good handle
		$Trace->trace(msg=>"P->{HOSTRECS}{$params{node}}{SITE}=$P->{HOSTRECS}{$params{node}}{SITE}" . "\n");
		$Trace->trace(msg=>"P->{TEST_NAME}=" . $P->{'TEST_NAME'} . "\n");
		$Trace->trace(msg=>"P->{LOGDIR}=" . $P->{'LOGDIR'} . "\n");
		$Trace->trace(msg=>"P->{CONSERVER}=$P->{CONSERVER}\n");
		$Trace->trace(msg=>"P->{CONSERVER_LOGDIR}=$P->{CONSERVER_LOGDIR}\n");

		if (defined $P->{'TEST_NAME'}) { #available if we ran from a testbed
			eval {
				say($Handle,"echo -e \"\n\n##TEST IS STARTING[$P->{'TEST_NAME'}]\">>$P->{CONSERVER_LOGDIR}/$params{node}" ,'-swallow');
			};
			if ($@) {
				$Trace->warn(msg=>"We have a handle to the console server, but we were unable to poke a 'test starting' message into the log.  The error is $@");
			}

		} elsif (defined $P->{'LOGDIR'}) { #available unless someone ran with LOGDIR=-
			eval {
				say($Handle,"echo -e \"\n\n##TEST IS STARTING[$P->{'LOGDIR'}]\">>$P->{CONSERVER_LOGDIR}/$params{node}",'-swallow');
			};
			if ($@) {
				$Trace->warn(msg=>"We have a handle to the console server, but we were unable to poke a 'test starting' message into the log.  The error is $@");
			}

		} else { #at least show that we ran, but without much good info
			eval {
				say($Handle,"echo -e \"\n\n##TEST IS STARTING[LOGDIR-]\">>$P->{CONSERVER_LOGDIR}/$params{node}",'-swallow');
			};
			if ($@) {
				$Trace->warn(msg=>"We have a handle to the console server, but we were unable to poke a 'test starting' message into the log.  The error is $@");
			}
		}

		eval {
			say($Handle,"echo -e \"\n\n##LOGDIR[$P->{'LOGDIR'}]\n\">>$P->{CONSERVER_LOGDIR}/$params{node}",'-swallow'); };
		if ($@) {
			$Trace->warn(msg=>"We have a handle to the console server, but we were unable to poke a 'LOGDIR' message into the log.  The error is $@");
		}
	}
}



return 1;

##########################Perldoc##########################


=head1 NAME
Conlog - an interface to the conserver server/logs

=head1 EXTERNAL FUNCTIONS

=over 8

=item filesize(node=>node)

This sub routine takes a single argument (node).  Finds the server that
logs the console for this node (if any), login to it, and RETURN the 
filesize of the log for this node.

ex. $size=Conlog::filesize($node);

=item print_section_of_log(node=>node, start=>size of file to start at, finish=>size of file to end at)

This sub routine prints the section of the console log for a node from a byte location, and to a byte
location.  If it is available, this will write to the logdir set by nate and a file called conlog.  If
logdir isn't available, it will write to standard out.

=item write_test_starting2log(node=>node) 

This sub routine writes a marker line to the logfile for a certain node.  It automatically figures out what
console server is responsible for this node.


=back

=head1 AUTHORS

Mike Garrett


