package Common;

use 5.006;
use strict;
use warnings;
use Data::Dumper;


# NetApp Specific Libs
use Server;
use TracePkg;
use Params::Common;
use Module::Common qw(validate_mixed);
use Params::Validate qw(CODEREF ARRAYREF HASHREF SCALAR UNDEF);


##############################################################################
#                          GLOBALS
##############################################################################

my $Trace = TracePkg->new();
my $Server = Server->new();


require Exporter;
use AutoLoader qw(AUTOLOAD);

our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Common ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = ( 'all' => [ qw(
	
) ] );

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw(
	
);
our $VERSION = '0.01';


# Preloaded methods go here.

sub stripe_vol_create 
{
 $Trace->entry();

    my %vol_params = validate_mixed (
	@_,
	[],
	{
	    'aggregate' => {type=>SCALAR, optional=>0 },
	    'vserver' => {type=>SCALAR,default=>'vs0'},
	    'volume' => {type=>SCALAR,optional=>0},
	    'junction-path' => {type=>SCALAR,optional=>1},
	    'size' => {type=>SCALAR,optional=>1},
	    'state' => {type=>SCALAR,optional=>1},
	    'policy' => {type=>SCALAR,optional=>1},
	    'user' => {type=>SCALAR,optional=>1},
	    'group' => {type=>SCALAR,optional=>1},
	    'security-style' => {type=>SCALAR,optional=>1},
	    'unix-permissions' => {type=>SCALAR,optional=>1},
	    'junction-active' => {type=>SCALAR,optional=>1},
	    'vsroot' => {type=>SCALAR,optional=>1},
	    'comment' => {type=>SCALAR,optional=>1},
	    'maxdir-size' => {type=>SCALAR,optional=>1},
	    'space-guarantee' => {type=>SCALAR,optional=>1},
	    'snapshot-policy' => {type=>SCALAR,optional=>1},
	    'stripe-count' => {type=>SCALAR,optional=>1},
	    'stripe-width' => {type=>SCALAR,optional=>1},
	    'lookup-cache' => {type=>SCALAR,optional=>1},
	    'foreground' => {type=>SCALAR,optional=>1},
	}
    );


    print "\n\n";
    $Trace->debug(msg=>"Dumping the \\%vol_params hash: " . Dumper \%vol_params);
    print "\n\n";

#The server command that was sent to this function  
my $result = "true";
eval
{
    $Server->volume_new(%vol_params);
};
if ($@) {
    $Trace->comment(msg=>"Volume create for: %vol_params{volume} failed with this error: $@");
    $result = "false"; 
}
return $result;

		    
   # TODO 
   #
   # If the volume already exists need to pass reuse flag 
   #
   # or delete the volume first (if flag is set) then recreate volume for test case 


    $Trace->exit();
}

sub get_epoch {
    my $output;
    my $epoch;
    my $return_var;
    my $max_epoch;
    my $msid_var = shift;
    $Trace->entry();

    $Server->shell(cmd => "vldbtest dump -e | grep $msid_var", output => \$output);
    $Trace->comment(msg=>"This is the vldbtest dump -e command $output ");

    if ($output =~ /.*epoch:*./){
	#if ($output =~ /.*epoch:\s=(\d+)$/)
	$Trace->comment(msg=>"Found an epoch for the volume");
	$epoch = $output;
	my @fields = split / /, $epoch;
	$Trace->debug(msg=>"This is my fields array: ".Dumper(\@fields));
	$Trace->comment(msg=>"My current epoch is: $fields[5]");
	$return_var = $fields[5];
    } else {
	$Trace->fatal(msg=>"No epoch was listed at vldbtest dump -e | grep $msid_var " . Dumper(\$output));
    };
    if ($return_var == 255){
	$Trace->comment(msg=>"Can not increment the value for epoch. It is already at the MAX : " . $return_var);
	#setting max_epoch to true if it is at 255
	$max_epoch = "true";
    }

    $Trace->exit();
    return ($return_var, $max_epoch);
}


# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is stub documentation for Coral::Common.pm

=head1 NAME

Common.pm -  Library of common utility subroutines that will work for stripe aggregates/volumes.

=head1 SYNOPSIS

  use Common;
  blah blah blah

=head1 DESCRIPTION

Stub documentation for Common, created by h2xs. It looks like the
author of the extension was negligent enough to leave the stub
unedited.

Blah blah blah.

=head2 EXPORT

None by default.


=head1 AUTHOR

A. U. Thor, E<lt>a.u.thor@a.galaxy.far.far.awayE<gt>

=head1 SEE ALSO

L<perl>.

=cut
