# -*- perl -*- 
#$Id: //depot/prod/test/Rsierranevada/lib/Coral/CoralEvents.pm#1 $ 

## @summary Helper functions for coral tests
##
## @owner Coral QA (dl-coral-qa@netapp.com)
##
## @author Michael Edmonds (michael.edmonds@netapp.com)
## @author Lamb-diggity (kevin.lambright@netapp.com)
## @author GX Automation Framework (dl-afd@netapp.com)
##
## @change 2007-05-07   medmonds:   Initially created
## @change 2007-05-10   medmonds:   added make_aggr_mode() and
##                                      modified add_member_aggr()
## @change 2007-05-11   medmonds:   added get_aggr_mems()
## 
##
#
# Copyright (c) 2007 Network Appliance, Inc., All Rights Reserved
# Permission is granted to use this script on a Network Appliance Product.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from Network Appliance, Inc.
#
##
## @status private
## @pod here

=pod

=head1 Author

GX Automation Framework (dl-afd@netapp.com)

=head1 Owner
Coral QA (dl-coral-qa@netapp.com)

=head1 NAME

CoralEvents.pm

=head1 DESCRIPTION

C<CoralEvents.pm> takes care of Coral specific thingies.

=head1 See Also


=head1 Public Functions

=cut

package Coral::CoralEvents;

use strict;
use Data::Dumper;
use Params::Validate qw(:all);

use Tharn;
use DataStore;
use TracePkg;
our $Trace;

use vars qw(
	    $Mon_period
	    );

BEGIN
{
    $Trace = TracePkg->new();
}
# end BEGIN

# Global init.
$Mon_period = 30;   # Monitor I/O every 30 seconds.

###########################
# Events
###########################

sub add_member_aggr_mixed {
    my($server_obj, $cfg_obj) = @_;
    my(
       $dataref,
       $output,
       $node,
       $vol_hash,
       @aggr_members,
       @striped_aggr,
       );

    $Trace->entry();

    # Get dataref from parent.
    $dataref = &DataStore::get_data;
    $Trace->debug(msg => "Data received from parent: " . Dumper $dataref);

    $vol_hash = $cfg_obj->get_vol_of();
    foreach my $vol (keys %{$vol_hash})
    {
        if ($vol_hash->{$vol}{type} eq "striped")
        {
            push @striped_aggr, $vol_hash->{$vol};
        }
    }

    # Add aggr members. Value of -add-member-volumes comes from the
    # dataref.
    $node = shift @{$dataref->{nodes}};
    $server_obj->cli(cmd => "storage aggregate add-members" .
		     " -aggregate " . $striped_aggr[0]->{aggregate} .
		     " -nodes " . $node .
		     " -add-member-volumes " . $dataref->{add_member_vols});

    sleep(15);

    $Trace->comment(msg => "aggr_mem = " .
		    Dumper($server_obj->storage_aggregate_member_show()));
    @aggr_members =
	$server_obj->storage_aggregate_member_show(node => $node);
    $Trace->comment(msg => "vol_mem = " .
		    Dumper($server_obj->volume_member_show()));

    foreach my $vol (keys %{$vol_hash})
    {
        if ($vol_hash->{$vol}{type} eq "striped")
        {
            $Trace->comment(msg => "is_vol_proper = " .
			    is_vol_proper($server_obj,
					  $vol_hash->{$vol}{volume}));
        }
    }

    $Trace->comment(msg=>"aggr_proper = " .
		    Dumper(is_aggr_proper($server_obj,
					  $dataref->{aggregate})));
    
    $Trace->exit();

    return 1;
} # end sub add_member_aggr_mixed


sub cthon {
    my($server_obj, $cfg_obj) = @_;
    my(
       $cl_stopped,
       $clientmgr_obj,
       $cthon,
       $dataref,
       @status,
       );

    $Trace->entry();

    $dataref = &DataStore::get_data;
    
    $Trace->debug(msg => "Adding CTHON load");
    $clientmgr_obj = new ClientMgr(clients => $dataref->{clients});
    $cthon = $clientmgr_obj->add_load(type => ClientMgr->CTHON());

    $Trace->debug(msg => "Mounting on clients " .
		  Dumper($dataref->{clients}));
    $clientmgr_obj->mount(map => $cfg_obj->get_client_map());

    $Trace->debug(msg => "Starting CTHON IO");
    $clientmgr_obj->start();

    # Monitor I/O.
    for (my $i = 0; $i < ($dataref->{io_duration} / $Mon_period); $i++) {
	@status = $clientmgr_obj->status;
	foreach my $client_status (@status) {
	    if (@{$client_status->{stopped}}) {
		$cl_stopped = join("\n", @{$client_status->{stopped}});
		$Trace->warn(msg => "The following clients have stopped: \n" .
			     $cl_stopped . "\n");
		&Tharn::addresult(type => "FAIL",
				  msg  => "CTHON i/o not successful, one or " .
				  "more clients stopped or failed");
		return;
	    }
	}

	sleep $Mon_period;
    }

    $Trace->debug(msg => "Stopping CTHON IO");
    $clientmgr_obj->stop();

    &Tharn::addresult(type => "PASS", "CTHON run successfully");

    return 1;
} # end sub cthon


sub aggr_cleanup {
    my($server_obj, $cfg_obj) = @_;
    my(
       $dataref,
       $node,
       @aggr_mem,
       );

    $dataref = &DataStore::get_data;
    $node = shift @{$dataref->{nodes}};

    @aggr_mem =	$server_obj->storage_aggregate_member_show(node => $node);
    $server_obj->storage_aggregate_remove_members(aggregate =>
						  $dataref->{aggregate},
						  members =>
						  $aggr_mem[0]->{aggregate});

    return 1;
} # end sub aggr_cleanup


###########################
# Helper functions
###########################


sub is_vol_proper {
    my ($server_obj, $vol) = @_;
    my(
       @member_vols,
       %aggr_hash,
       );

    $Trace->entry();

    @member_vols = $server_obj->volume_member_show(volume => $vol . "_*");
    $Trace->comment(msg => "MEMBER VOLS = " . Dumper(@member_vols));
    foreach my $member_vol (@member_vols) {
        if (defined $aggr_hash{$member_vol->{aggregate}}) {
            $Trace->exit();
            return 0;
        }
        $aggr_hash{$member_vol->{aggregate}} = $member_vol->{volume};
    }

    $Trace->exit();
    return 1;
} # end is_vol_proper


sub is_aggr_proper {
    my($server_obj, $aggr) = @_;
    my(
       @vols,
       $proper,
       $improper,
       $rv,
       );

    $Trace->entry();

    # Retrieve existing volumes in aggr.
    @vols = get_vols($server_obj, $aggr);

    $proper   = 0;
    $improper = 0;
    foreach my $vol (@vols) {
        $Trace->comment(msg => "VOLUME = " . Dumper($vol));
        if (is_vol_proper($server_obj, $vol->{volume})) {
            $proper++;
        }
        else {
            $improper++;
        }
    }

    if (($improper > 0) && ($proper > 0)) {
        $rv = 2;
    }
    elsif ($proper > 0) {
        $rv = 1;
    }
    elsif ($improper > 0) {
        $rv = 0;
    }
    else {
        $Trace->debug(msg => $aggr . " Contains NO VOLUMES.");
        $rv = -1;
    }

    $Trace->exit(msg=>"Return Value = " . $rv);
    return $rv;
}
# end is_aggr_proper

# UNDER CONSTRUCTION
sub make_aggr_proper
{
    my $hash = {server    => {optional => 0, type => OBJECT},
                cfg       => {optional => 0, type => OBJECT},
                trace     => {optional => 0, type => OBJECT},
                aggregate => {optional => 0, type => SCALAR},};
    my %params = validate(@_, $hash);

    $params{trace}->entry();

    my %aggr_hash; # add to this later to keep track if we have Multi-DVs
    my @vols;
    eval{@vols = get_vols($params{aggregate})};
    foreach my $vol (@vols)
    {
        my @member_vols = $params{server}->volume_member_show(volume => $vol . "_*");
        foreach my $mem (@member_vols)
        {
            push(@{$aggr_hash{$mem->{aggregate}}}, $mem->{volume});
        }
        foreach my $aggregate (keys(%aggr_hash))
        {
            $params{trace}->comment(msg => "DVs = " . Dumper($aggr_hash{$aggregate}));
            if(scalar(@{$aggr_hash{$aggregate}}) > 1)
            {
                for(my $i = 0 ; $i < (scalar(@{$aggr_hash{$aggregate}}) - 1) ; $i++)
                {
                    # Still trying to figure it out
                }
            }
        }
    }

    $params{trace}->exit();
    return 1;
}
# end make_aggr_proper()

sub make_aggr_mode
{
    $Trace->entry();
    my $hash = {server    => {optional => 0, type => OBJECT},
                cfg       => {optional => 0, type => OBJECT},
                clients   => {optional => 0, type => ARRAYREF},
                aggregate => {optional => 0, type => SCALAR},
                mode      => {optional => 1, type => SCALAR, default => 2,
                              callbacks => {test => sub {(int($_[0]) > -1)
                                                                &&
                                                         (int($_[0]) < 3)}}},};
    my %params = validate(@_, $hash);

    my($aggr_hash, @nodes,);

    my $prop = is_aggr_proper($params{server}, $params{aggregate});
    while($prop != $params{mode})
    {
        if($prop == -1)
        {
	    # No vols.
	    $aggr_hash = get_aggr($params{server}, $params{aggregate});
            @nodes = @{$aggr_hash->{nodes}};
            make_volume($params{server},
			$params{cfg},
			$params{clients},
			$params{aggregate},
                        (scalar(@nodes) + (scalar(@nodes) % 2)));
        }
        elsif($prop == 1)
        {
	    # Proper Only
            if($params{mode} != 1)
            {
                my @vols = get_vols($params{server}, $params{aggregate});
                if($params{mode} == 2)
                {
                    add_member_vol($params{server}, $vols[0]);
                }
                else
                {
                    foreach my $vol (@vols)
                    {
                        add_member_vol($params{server}, $vol);
                    }
                }
            }
        }
        elsif($prop == 0)
        {
	    # Improper Only
            if($params{mode} != 0)
            {
                if($params{mode} == 2)
                {
                    make_volume($params{server},
                                $params{cfg},
                                $params{clients},
                                $params{aggregate},
                                1);
                }
                else
                {
                    # Still trying to figure out how to "proper-tize" the vols
                }
            }
        }

        sleep(45);
        $prop = is_aggr_proper($params{server}, $params{aggregate});
    }
    $Trace->exit();
    return 1;
}
# end sub make_aggr_mode

sub make_aggr_mixed {
    my($server_obj, $cfg_obj, $client_ref, $aggr) = @_;
    my(
       $aggr_hash,
       @nodes,
       );

    $Trace->entry();

    my $prop = is_aggr_proper($server_obj, $aggr);
    while($prop != 2) {
        if($prop == -1) {
	    # No vols.
	    $aggr_hash = get_aggr($server_obj, $aggr);
            @nodes = @{$aggr_hash->{nodes}};
            make_volume($server_obj,
			$cfg_obj,
			$client_ref,
			$aggr,
			(scalar(@nodes) + (scalar(@nodes) % 2)));
            #add_member_vol((get_vols($Aggregate))[0]);
        }
        elsif($prop == 1) {
	    # Proper Only
            add_member_vol($server_obj, (get_vols($server_obj, $aggr))[0]);
        }
        elsif($prop == 0) {
	    # Improper Only
            make_volume($server_obj, $cfg_obj, $client_ref, $aggr, 1);
        }

        sleep(45);
        $prop = is_aggr_proper($server_obj, $aggr);
    }
    $Trace->exit();
    return 1;
} # end sub make_aggr_mixed

sub add_member_vol {
    my ($server_obj, $vol_hash, $count) = @_;

    $Trace->entry();

    $count = 1 unless ($count);
    $Trace->comment(msg=>"vol_hash = " . Dumper($vol_hash));
    $server_obj->volume_add_members(vserver    => $vol_hash->{vserver},
				    volume     => $vol_hash->{volume},
				    foreground => "true",
				    count      => $count);
    $Trace->exit();
}
# end sub add_member_vol

sub make_aggr
{
    my $hash = {server    => {optional => 0, type => OBJECT},
                raidtype  => {optional => 1, type => SCALAR, default=>"raid_dp",
                              callbacks => {test => sub {($_[0] eq "raid_dp")
                                                                ||
                                                         ($_[0] eq "raid4")}},},
                trace     => {optional => 0, type => OBJECT},
                aggregate => {optional => 0, type => SCALAR},
                disks_per_mem => {optional => 1, type => SCALAR, default => 3,},
                aggr_nodes => {optional => 0, type => SCALAR},
                retry_sleep => {optional => 1, type => SCALAR, default => 15,},
                retries => {optional => 1, type => SCALAR, default => 10,},};
    my %params = validate(@_, $hash);

    $params{trace}->entry();

    my (@nodes,
        $disks,
        $found,
        $no_entry_err,
        $dup_entry_err,
    );

    @nodes = split(",", $params{aggr_nodes});
    $disks = ($params{disks_per_mem} * scalar(@nodes));
    $found = 0;
    $no_entry_err = "there are no entries matching your query";
    $dup_entry_err = "an aggregate already uses ".$params{aggregate}." as name";

    eval
    {
        $params{server}->storage_aggregate_new(aggregate       => $params{aggregate},
                                                diskcount      => $disks,
                                                'volume-style' => 'striped',
                                                nodes          => $params{aggr_nodes},
                                                raidtype       => $params{raidtype});
    };
    if ($@)
    {
        if($@ =~ /$no_entry_err/i)
        {
            for (my $i = 0; $i < $params{retries}; $i++)
            {
                sleep($params{retry_sleep});
                # This will continue to fail until the aggr is visible
                # Retry some number of times with delay between shows.
                eval
                {
                    $params{server}->storage_aggregate_show(aggregate => $params{aggregate});
                };
                if ($@)
                {
                    if($@ !~ /$no_entry_err/i)
                    {
                        $params{trace}->warn(msg => $@);
                        return 0;
                    }
                }
                else
                {
                    $found = 1;
                    $i = $params{retries}
                }
            }
        }
        elsif($@ =~ /$dup_entry_err/i)
        {
            $found = 1;
        }
        else
        {
            $params{trace}->warn(msg => $@);
            return 0;
        }
    }
    else
    {
        $found = 1;
    }

    if(!($found))
    {
        $params{trace}->warn(msg => "Aggregate " . $params{aggregate} .
                             " not visible after " . $params{retries} .
                             " retries ($params{retry_sleep} delay" .
                             " between retries)");
        return 0;
    }
    $params{trace}->exit();
    return 1;
}
# end make_aggr()

sub wait_aggr_complete
{
    my $hash = {server    => {optional => 0, type => OBJECT},
                raidtype  => {optional => 1,
                              type => SCALAR, default=>"raid_dp",
                              callbacks => {test => sub {($_[0] eq "raid_dp")
                                                                ||
                                                         ($_[0] eq "raid4")}},
                              },
                trace     => {optional => 0, type => OBJECT},
                aggregate => {optional => 0, type => SCALAR},};
    my %params = validate(@_, $hash);

    $params{trace}->entry();

    my $error = 1;
    while($error)
    {
        # Going to be a while if are using init'd disks
        eval
        {
            $params{server}->storage_aggregate_show(aggregate => $params{aggregate},
                                                     raidtype  => $params{raidtype});
        };
        if($@)
        {
            $error = $@;
            sleep(1);
        }
        else
        {
            $error = 0;
        }
    }
    $params{trace}->exit();
    return 1;
}
# end wait_aggr_complete

sub add_member_aggr
{
    my ($server, $aggr, $nodes, $mem_vol, $trace) = @_;

    $trace->entry();

    my (@nodes, @members);

    @nodes = split(",", $nodes);

    eval{$server->cli(cmd => "storage aggregate add-members"  .
                             " -aggregate "          . $aggr  .
                             " -nodes "              . $nodes .
                             " -add-member-volumes " . $mem_vol,
                             -Timeout=>300);};
    if($@)
    {
        return $@;
    }

    foreach my $node (@nodes)
    {
        my @temp_mem = get_aggr_mems($server, $trace, $aggr, $node);
        if($temp_mem[0] =~ /error|fail/i)
        {
            return @temp_mem;
        }
        push(@members, @temp_mem);
    }
    $trace->exit();
    return @members;
}
# end add_member_aggr()

sub delete_aggr
{
    my ($server, $aggr, $by_member, $trace) = @_;
    $trace->entry();
    if($by_member)
    {
        my @members = $server->storage_aggregate_member_show(aggregate => $aggr."*");
        foreach(@members)
        {
            $server->storage_aggregate_remove_members(aggregate => $aggr,
                                                      members   => $_->{aggregate});
        }
        eval{$server->storage_aggregate_show(aggregate => $aggr);};
        if($@ !~ qr/there are no entries matching your query/i)
        {
            $trace->warn(msg => "It does Not appear that " . $aggr . " was " .
                         "properly removed when all of the memebers were " .
                         "deleted.");
            return 0;
        }
    }
    else
    {
        $server->storage_aggregate_delete(aggregate => $aggr);
    }
    $trace->exit();

    return 1;
}
# end delete_member_aggr()

sub delete_aggr_mem
{
    my $hash = {server    => {optional => 0, type => OBJECT},
                member    => {optional => 0, type => SCALAR}};
    my %params = validate(@_, $hash);
    my @pieces = split("_", $params{member});
    pop(@pieces);
    my $aggregate = join("_", @pieces);
    return eval
    {
        $params{server}->storage_aggregate_remove_members(aggregate => $aggregate,
                                                          members   => $params{member});
    }
}

sub get_aggr_mems
{
    my ($server, $trace, $aggr, $node) = @_;
    $trace->entry();

    my (@members, $sleep_time, $tries);

    $sleep_time = 15;
    $tries = 10;

    for(my $i = 0 ; $i < $tries ; $i++)
    {
        sleep($sleep_time);
        eval{@members = $server->storage_aggregate_member_show(aggregate => $aggr."_*",
                                                                node      => $node);};
        my $error = $@;
        if($error)
        {
            if($error =~ qr/there are no entries matching your query/i)
            {
                if($i == ($tries - 1))
                {
                    return $error;
                }
                sleep($sleep_time);
            }
            #else
            #{
            #    return $error;
            #}
        }
        elsif(($members[0]->{aggregate} =~ /$aggr/) && ($members[0]->{node} =~ /$node/))
        {
            $trace->comment(msg=>"members = " . Dumper(@members));
            $trace->exit();
            return @members;
        }
    }
}
# end get_aggr_mems()

sub make_volume {
    my ($server_obj, $cfg_obj, $client_ref, $aggr, $quantity) = @_;

    $Trace->entry();

    $cfg_obj->define(aggregates => $aggr,
		     vol_layout => 'single',
		     vol_cnt    => $quantity,
		     vol_type   => 'striped',
		     vol_size   => '100m',
		     clients    => $client_ref);
    $cfg_obj->create();

    $Trace->exit();

    return 1;
} # end sub make_volume


sub get_vols {
    my($server_obj, $aggr) = @_;
    my(
       @vols,
       );

    $Trace->entry();

    eval{
	@vols = $server_obj->volume_show(aggregate => $aggr);
    };
    if ($@ and $@ !~ /no entries matching your query/i) {
	# We expect we might get the error stating there are no vols
	# in the aggr.  This is a different error.
	die $@;
    }

    $Trace->exit();
    return @vols;
} # end sub get_vols


sub get_aggr {
    my($server_obj, $aggr) = @_;
    my(
       @aggrs,
       );

    $Trace->entry();

    @aggrs = $server_obj->storage_aggregate_show(aggregate => $aggr);


    $Trace->exit();

    return $aggrs[0];
} # end sub get_aggr

1;
