package Restripe;

use 5.006;
use strict;
use warnings;
use Data::Dumper;


# NetApp Specific Libs
use TCD;
use Server;
use TracePkg;
use Options;
use Params::Common;
use Module::Common qw(validate_mixed);
use Params::Validate qw(CODEREF ARRAYREF HASHREF SCALAR UNDEF);


##############################################################################
#                          GLOBALS
##############################################################################

my $Trace = TracePkg->new();
my $Options = Options->new();
my $Server = Server->new();
my $Params = $Options->get_hash();
 

require Exporter;
#use AutoLoader qw(AUTOLOAD);

#our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Restripe ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.

#our %EXPORT_TAGS = ( 'all' => [ qw(

#) ] );

#our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

#our @EXPORT_OK = qw(hello);


our @EXPORT = qw(

);
our $VERSION = '0.01';


# Preloaded methods here.

sub modify_stripe_width 
{
    $Trace->entry();

    ##Retrieve current sub routine from stack for clarity in log messages.
    #my $caller=(caller(0))[3]."()";

    my %thpl_params = validate_mixed (
	@_,
	[],
	{
	    aggr=> {type=>SCALAR, optional=>0 },
	    vs=> {type=>SCALAR,default=>'vs0'},
	    vol=> {type=>SCALAR,optional=>0},
	    sub_count=> {type=>SCALAR,default=>1},
	    sw_step=>{type=>SCALAR,default=>4096},
	    iterate_sleep=>{type=>SCALAR,default=>'0'},
	}	
    );



    print "\n\n";
    $Trace->debug(msg=>"Dumping the \%thpl_params hash: " . Dumper \%thpl_params);
    print "\n\n";

    #This is the loop for the modify_stripe_width sub routine. 
    #The value for the loop comes from $thpl_params{sub_count} and the default is 1.
    for (my $i = 1; $i <= $thpl_params{sub_count}; $i++ ) {

	$Trace->trace(msg=>"This is the: $i / $thpl_params{sub_count} time through the modify_stripe_width sub");

	my $vol_hash = $Server->volume_show(volume    => $thpl_params{vol},
	    vserver   => $thpl_params{vs},
	    aggregate => $thpl_params{aggr}
	);

	$Trace->debug(msg=>"Dumping the vol_hash to get the stripe_width: ".Dumper($vol_hash));

	#need to get the msid of the volume
	my $msid = $vol_hash->[0]{'msid'};
	$Trace->comment(msg=>"This is the msid: $msid");

	#Sending the msid that we got to find out the epoch
	my (@epoch_1, @epoch_2, $max_epoch);
	@epoch_1 = Common::get_epoch($msid);

	$max_epoch = $epoch_1[1];

	if ($max_epoch eq "true"){
	    $Trace->exit("The value for epoch is already at the MAX : " .$epoch_1[0]);      
	    #Tharn::addresult(type=>"CONFIG", abort=>0, $caller." epoch is already at the MAX 'EPOCH: ".@epoch_1[0]."' failed");
	    #return 1;
	    #return $TCD::FAIL;
	}


	#need to get the stripe-width of the volume
	my $stripe_width_1 = $vol_hash->[0]{'stripe-width'}; 

	$Trace->comment(msg=>"The epoch and stripe-width for volume $thpl_params{vol} BEFORE the stripe-width is incremented. EPOCH: " . $epoch_1[0] . " and stripe-width of: " . $stripe_width_1 . " in Bytes.");

	#Setting the stripe-width step default to 4KB
	#The default unit that we get back in our hash is Bytes
	my $sw_step = 4096;

	my $new_stripe_width = $stripe_width_1 + $sw_step;

	$Trace->comment(msg=>"The the new stripe-width to be set :" . $new_stripe_width . " Bytes");


	$Server->volume_modify('volume'        => $thpl_params{vol},
	    'vserver'       => $thpl_params{vs},
	    'stripe-width'  => $new_stripe_width

	);

	$vol_hash = $Server->volume_show(volume    => $thpl_params{vol},
	    vserver   => $thpl_params{vs},
	    aggregate => $thpl_params{aggr}
	);


	my $stripe_width_2 = $vol_hash->[0]{'stripe-width'};

	@epoch_2 = Common::get_epoch($msid);
	$Trace->comment(msg=>"The epoch and stripe-width for volume $thpl_params{vol} AFTER the stripe-width is incremented. EPOCH: " . $epoch_2[0] . " and stripe-width of: " . $stripe_width_2 . " in Bytes.");


	#Making sure that the striped-width is the width that was requested.
	if ($new_stripe_width == $stripe_width_2){
	    $Trace->comment(msg=>"The difference between the new requested stripe width: " .$new_stripe_width . " and the current stripe width: " .$stripe_width_2 . " should be ZERO and it is.");
	} else {
	    $Trace->exit(msg=>"The the new_stripe_width and stripe_width_2  are not equal. The requested striped-width is: ". $new_stripe_width . "Bytes. The actually stripe-width is: ".$stripe_width_2." Bytes" );
	    #return $TCD::FAIL;
	}


	#Makeing sure the the epoch got bumped up by one.
	my $epoch_diff = $epoch_2[0] - $epoch_1[0];	
	if ($epoch_diff == 1){
	    $Trace->comment(msg=>"The value for epoch_diff should be 1 it is: " .$epoch_diff);
	} else {
	    $Trace->exit(msg=>"The epoch WAS NOT incremented. The epoch_diff should be 1 it was: ". $epoch_diff);
	    #return $TCD::FAIL;
	}

	#Make sure the volume came back online  
	if ($vol_hash->[0]{'state'} eq "online"){
	    $Trace->comment(msg=>"The striped volume state for: $thpl_params{vol} is: $vol_hash->[0]{'state'} after changing the stripe-width");
	} else {
	    $Trace->exit(msg=>"The striped volume state for: $thpl_params{vol} is: $vol_hash->[0]{'state'} after changing the stripe-width");
	    #return $TCD::FAIL;
	}
    
    sleep $thpl_params{iterate_sleep};
    
    }
        $Trace->exit();
	return ("pass");
	#return $TCD::PASS;
}
#end modify_stripe_width()

sub volume_add_members {

    $Trace->entry();

    ##Retrieve current sub routine from stack for clarity in log messages.
    #my $caller=(caller(0))[3]."()";

    my %thpl_params = validate_mixed (
	@_,
	[],
	{
	    aggr=> {type=>SCALAR, optional=>0 },
	    vs=> {type=>SCALAR,default=>'vs0'},
	    vol=> {type=>SCALAR,optional=>0},
	    sub_count=> {type=>SCALAR,default=>1},
	    member_count=>{type=>SCALAR,default=>1},
	    member_foreground=>{type=>SCALAR,default=>'true'},
	    iterate_sleep=>{type=>SCALAR,default=>'0'},
	}
    );

    #TODO:if you set the member_foreground to false then the job id will need to be monitored before the second check of the Common::get_epoch()  

    print "\n\n";
    $Trace->debug(msg=>"Dumping the \%thpl_params hash: " . Dumper \%thpl_params);
    print "\n\n";

    #This is the loop for the modify_stripe_width sub routine.
    #The value for the loop comes from $thpl_params{sub_count} and the default is 1.
    for (my $i = 1; $i <= $thpl_params{sub_count}; $i++ ) {

    $Trace->trace(msg=>"This is the: $i / $thpl_params{sub_count} time through the volume_add_members sub");
	#$Trace->debug(msg=>"Here are the params: ".Dumper(\$Params));
	my $vol_hash = $Server->volume_show(volume    => $thpl_params{vol},
	    vserver   => $thpl_params{vs},
	    aggregate => $thpl_params{aggr} 
	);

	$Trace->debug(msg=>"Dumping the vol_hash: ".Dumper($vol_hash));

	#Need to get the msid of the volume we created
	my $msid = $vol_hash->[0]{'msid'};
	$Trace->comment(msg=>"This is the msid: $msid");

	#Sending the msid that we got to find out the epoch
	my (@epoch_1, @epoch_2, $max_epoch);
	@epoch_1 = Common::get_epoch($msid);

	$max_epoch = $epoch_1[1];

	if ($max_epoch eq "true"){
	    $Trace->exit(msg=>"The value for epoch is already at the MAX : " .$epoch_1[0]);
	    #return $TCD::FAIL;
	}

	$Trace->comment(msg=>"The epoch for $thpl_params{vol} BEFORE volume add-member is: " . $epoch_1[0]);


	$Trace->comment(msg=>"Starting volume add-member on striped volume named: " . $thpl_params{vol});


	#This Server call is used to add volume members to a exsisting coral set.
	$Server->volume_add_members('volume'     => $thpl_params{vol},
	                            'vserver'    => $thpl_params{vs},
	                            'count'      => $thpl_params{member_count},
	                            'foreground' => $thpl_params{member_foreground}
	);

	@epoch_2 = Common::get_epoch($msid);
	$Trace->comment(msg=>"The epoch for $thpl_params{vol} AFTER volume add-member is " . $epoch_2[0]);

	$Trace->debug(msg=>"epoch_1 is: $epoch_1[0]"); 
	$Trace->debug(msg=>"epoch_2 is: $epoch_2[0]"); 

	#Makeing sure the the epoch got bumped up by one.
	my $epoch_diff = $epoch_2[0] - $epoch_1[0];
	if ($epoch_diff == 1){
	    $Trace->comment(msg=>"The value for epoch_diff should be 1 it is: " .$epoch_diff);
	} else {
	    $Trace->exit(msg=>"The epoch WAS NOT incremented. The epoch_diff should be 1 it was: ". $epoch_diff);
	    #return $TCD::FAIL;
	}

	#Make sure the volume came back online
	if ($vol_hash->[0]{'state'} eq "online"){
	    $Trace->comment(msg=>"The striped volume state for: $thpl_params{vol} is: $vol_hash->[0]{'state'} after changing the stripe-count");
	} else {
	    $Trace->exit(msg=>"The striped volume state for: $thpl_params{vol} is: $vol_hash->[0]{'state'} after changing the stripe-count");
	    #return $TCD::FAIL;
	}
	#Hitting a sleep in the for loop - for some defined duration
	sleep $thpl_params{iterate_sleep};
    }
    	$Trace->exit();
	return ("pass");
	#return $TCD::PASS;
}
#end volume_add_members()


#############################################################################
#              P R I V A T E   M E T H O D S
#############################################################################


# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is stub documentation for Restripe.pm

=head1 NAME

Restripe.pm - Library of common worker subroutines that restripe volumes.

=head1 SYNOPSIS

  use Restripe qw();

=head1 DESCRIPTION

"Restripe" provides common routines that used for restriping volumes.

=head1 ROUTINES

  volume_add_members
  Adds a volume member, Finds the epoch, Verifies that the epoch incremented.


  modify_stripe_width: 
  Finds the stripe-width, Finds the epoch, Increments the stripe-width, Verifies that the stripe-width incremented, Verifies that the epoch incremented.



=head2 EXPORT

None by default.

=head1 AUTHOR

Frank Danyo, E<lt>danyo@netapp.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007, Network Appliance., All Rights Reserved

=cut
