/* $Id: //depot/prod/test/Rsierranevada/lib/Coral/Verify.pm#1 $ */
package Verify;

use 5.006;
use strict;
#use warnings;
use Data::Dumper;


# NetApp Specific Libs
use TCD;
use Server;
use TracePkg;
use Options;
use Params::Common;
use Module::Common qw(validate_mixed);
use Params::Validate qw(CODEREF ARRAYREF HASHREF SCALAR UNDEF);


##############################################################################
#                          GLOBALS
##############################################################################

my $Trace = TracePkg->new();
my $Options = Options->new();
my $Server = Server->new();
my $Params = $Options->get_hash();
 

require Exporter;
#use AutoLoader qw(AUTOLOAD);

#our @ISA = qw(Exporter);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use Restripe ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.

#our %EXPORT_TAGS = ( 'all' => [ qw(

#) ] );

#our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

#our @EXPORT_OK = qw(hello);


our @EXPORT = qw(

);
our $VERSION = '0.01';


# Preloaded methods here.

sub verify_volume_member_attributes {
    my %in_hash = (
                    @_,
                  );
    my %vol_params = %{$in_hash{vol_params}};
    my $need_attrs = "volume|vserver";
    #filter out the attributes we don't need
    #If the volume has changed since the caller used this info, our query will fail
    map(delete($vol_params{$_}),grep($_ !~ /$need_attrs/,keys(%vol_params)));
        
    my $result = 1;  # We'll use this var and binary AND against it to return a pass/fail at the end.
    my $mdv_attrs;
    my @dv_members;
    #my @skip_mdv_attrs = qw(aggregate lookup-cache junction-path junction-path-source junction-active striping-epoch stripe-count parent properly-striped percent-used used available size total space-guarantee);
    #my @skip_dv_attrs = qw(aggregate volume stripe-count unix-permissions striping-epoch security-style lookup-cache user group properly-striped percent-used used available size total files-used parent junction-path junction-path-source junction-active dsid);
    my @common_attrs = qw(aggregate available comment concurrency crtime create-time description junction-active junction-path junction-path-source lookup-cache max-autosize autosize-increment parent percent-used properly-striped size stripe-count striping-epoch total used);
    my @skip_mdv_attrs = (@common_attrs, qw(space-guarantee));
    my @skip_dv_attrs = (@common_attrs, qw(dsid files-used group security-style unix-permissions user volume));
    
    
    
    
    $Trace->entry();
    $Trace->debug(msg=>"Args: ".Dumper(@_));
    my $striped_vol = ($Server->volume_show(%vol_params))[0];
    $Trace->debug(msg=>"volume".Dumper($striped_vol));
    my $striped_vol_members = $Server->volume_member_show(msid=>$striped_vol->{msid});
    $Trace->debug(msg=>"volume_members".Dumper($striped_vol_members));
    $mdv_attrs = (grep($_->{volume} =~ /^$striped_vol->{volume}$/,@$striped_vol_members))[0]; 
    @dv_members = grep($_->{volume} !~ /^$striped_vol->{volume}$/,@$striped_vol_members);
        $Trace->debug(msg=>"MDV attrs".Dumper($mdv_attrs));
        $Trace->debug(msg=>"DV attrs".Dumper(@dv_members));
        
    #Walk through the simple attributes, make sure everything matches up.    
    DVCOMPARE: for my $member_vol (@$striped_vol_members) {
        ATTRCOMPARE: for my $attribute (keys(%$striped_vol)) {
            my $mvol = $striped_vol->{volume};
            $Trace->debug(msg=>"VOL: $member_vol->{volume} Attr: $attribute member: $member_vol->{$attribute} striped_vol: $striped_vol->{$attribute} $mvol");

            # Check if the volume is an MDV, or a DV, and we are on one of the attributes we should skip
            #    If the volume name = the striped volume name, it is an MDV. If it is volume_### then it is a DV
            if (($member_vol->{volume} eq $striped_vol->{volume} && grep(/$attribute/, @skip_mdv_attrs)) || ($member_vol->{volume} =~ /${mvol}_[0-9]{3}/ && grep(/$attribute/, @skip_dv_attrs))) {
                $Trace->trace(msg=>"Skipping $attribute");
                next ATTRCOMPARE;
            }    
            
            #Replaces This
            #if ($member_vol->{volume} eq $mdv_attrs->{volume}) {
            #    $Trace->comment(msg=>"FOUND MDV");
            #    if (grep(/$attribute/, @skip_mdv_attrs)) {
            #        $Trace->comment(msg=>"Skipping $attribute");
            #        next ATTRCOMPARE 
            #    }
            #    
            #}
            #else {
            #    $Trace->comment(msg=>"FOUND DV");
            #    if (grep(/$attribute/, @skip_dv_attrs)) {
            #        $Trace->comment(msg=>"Skipping $attribute");
            #        next ATTRCOMPARE
            #    }
            #}
            if ($striped_vol->{$attribute} eq $member_vol->{$attribute}) {
                $Trace->debug(msg=>"Attribute $attribute is ok on member: $member_vol->{'volume'}");
                $result &= 1;
            }
            else {
                $Trace->warn(msg=>"Attribute: $attribute on member volume: $member_vol->{'volume'} does not match the striped volume value \nmember: $member_vol->{$attribute} striped_vol: $striped_vol->{$attribute}");
                $result &= 0;
            }    
        }
        
    }
    
    
    if ($mdv_attrs->{'space-guarantee'} ne 'none') {
        $Trace->comment(warn=>"space guarantee on MDV is $mdv_attrs->{'space-guarantee'} should be none");
        $result &= 0;
    }
    else {
        $result &= 1;
    }    

    #Checking sizes
    
    
    #get the member size from striped volume total size/stripe count.
    #compare all members
    my $stripe_size = $striped_vol->{size}/$striped_vol->{'stripe-count'};
    for my $member (@$striped_vol_members) {
        if ($stripe_size != $member->{size} ) {
            $Trace->warn(warn=>"member volume $member->{volume} size is: $member->{size} should be: $stripe_size");
            
            $result &= 0;
        }
        else {
             $result &= 1;
        }
        if ($striped_vol->{'files-used'} < $member->{'files-used'}) {
             $Trace->warn(warn=>"member volume $member->{volume} files-used is: $member->{'files-used'} is greater than the striped volume: $striped_vol->{'files-used'}");
             $result &= 0;
        }
        
        
        else {
            $result &= 1;
        }
             
    
    }
    
    #get sum of member sizes, compare sum with striped volume size
    my %size_sum;
    for my $size_attr (qw (size used total max-autosize autosize-increment)) {
        map ($size_sum{$size_attr} += $_->{$size_attr},@dv_members);
        if ($size_sum{$size_attr} != $striped_vol->{$size_attr} ) {
            $Trace->warn(msg=>"$size_attr sum  of volume members does not match \nsum of $size_attr: $size_sum{$size_attr} striped $size_attr:$striped_vol->{$size_attr} ");
            $result &= 0;
        }
        else {
             $result &= 1;
        }     
    
    }

    #find smallest available size * stripe count, compare with striped volume available size.
#BIG MAP
#    map ($size_sum{available} = $_->{available}*scalar(@dv_members) < $size_sum{available} || !defined($size_sum{available} ) ? $_->{available}*scalar(@dv_members) : $size_sum{available} ,@dv_members);
#    $Trace->comment(msg=>"sum of available: $size_sum{available} striped available:$striped_vol->{available} ");
#OR FOR LOOP?
    for (@dv_members) {
        if ($_->{available}*scalar(@dv_members) < $size_sum{available} || !defined($size_sum{available} ) ) {
            $size_sum{available} = $_->{available}*scalar(@dv_members);
        }

    }
    
    if ($size_sum{available} != $striped_vol->{available})  {
        $Trace->warn(msg=>"Sum of member available size does not match \nsum of available: $size_sum{available} striped available:$striped_vol->{available} ");
        $result &= 0;
    }
    else {
    
        $result &= 1;
    }    
    $Trace->exit();
    return $result;
}   

#############################################################################
#              P R I V A T E   M E T H O D S
#############################################################################


# Autoload methods go after =cut, and are processed by the autosplit program.

1;
__END__
# Below is stub documentation for Restripe.pm

=head1 NAME

Verify.pm - Library of common worker subroutines that verify striped volumes.

=head1 SYNOPSIS

  use Coral::Verify qw();

=head1 DESCRIPTION

"Verify" provides common routines that used for checking restriping volumes.

=head1 ROUTINES

=item verify_volume_member_attributes

verify_volume_member_attributes()

=item Description 

    Checks that member volume attributes all match the striped volume returns
    boolean result.

=item Input

=item * vol_params => $vol_hash
    hash reference of volume parameters as used in volume operations when
    calling Server.pm must contain 'vserver' and 'volume' params.

=item Output

    boolean
    A boolean result is passed back to indicate successful
    
    $result = Coral::Verify( vol_params => $vol_params );
    
    
  

=head2 EXPORT

None by default.

=head1 AUTHOR

Albert Straub, E<lt>astraub@netapp.comE<gt>

=head1 COPYRIGHT AND LICENSE

Copyright (C) 2007, Network Appliance., All Rights Reserved

=cut
