#
#  $Id: //depot/prod/test/Rsierranevada/lib/DBlade/dLock.pm#1 $
#
#  Copyright (c) 2008 Network Appliance, Inc.
#  All rights reserved.
#

# -----------------------------------------------------------
##
## @summary Support routines for DBlade Zapi Lock tests
##
## @description
##	This file contains routines that let you exercise the
##	internal API of the filer using ssh access to the FreeBSD
##	shell command zsmcli.
##
##	Contains interface functions to d-blade Zapis that take MSIDs
##	or DSIDs as arguments or return values.
##
## @author shenry@netapp.com

=head Summary

Support routines for d-blade Zapi d-lock tests.

=head Description

This file contains routines that let you exercise the internal
(d-blade) lock API of the filer using ssh access to the FreeBSD
shell command zsmcli.

=head2 Author

shenry@netapp.com

=cut

package DBlade::dSnapshot;

use	strict;
use	Tharn;
use	Socket;
use	LWP::UserAgent;
use	XML::Parser;
use	NaElement;
use	SPARSE::Util::Logger qw( logWarn logDbg logTrc );
use	Data::Dumper;
use	DBlade::dZapi;

use	vars ('@ISA', '@EXPORT');


=head2 new

  use DBlade::dSnapshot;
  my $zapi = new DBlade::dLock($FILER, $user, $password);

Create a new instance of the dblade zapi interface.
All access is through member functions.

=over

=item $FILER

(Required): Hostspec or name of the converged filer.

=item $user

(Optional): Username to login to the filer's BSD shell with.
Defaults to "root".

=item $password

(Optional): password for the login account. Takes the password from
the nate_host file if not specified.

=back

=cut

#' emacs-fodder

sub new {
    my ($class, $server, $user, $password) = @_;

    my $zapi = new DBlade::dZapi($server, $user, $password);

    my $self = {
                zapi => $zapi,
               };
    bless $self, $class;
    return $self;
}


=head2 d_lock_break

  $bool = $zapi->d_lock_break(%args);

Breaks one or more locks. %args are filters. If none are specified,
then all locks in the system are broken. All %args are optional.

Returns true if there are more locks to break, false otherwise.

=over

=item 'file-name'

(Optional): Path to file from root of volume (volume-dsid required).

=item 'host'

(Optional): Identity of client machine, opaque string.

=item 'owner'

(Optional): opaque string that is owner of file.

=item 'vif'

(Optional): ID of the interface that the lock was associated with. Integer.

=item 'volume-dsid'

(Optional): Volume for which locks are to be broken. If missing, all
volumes.

=item 'vserver'

(Optional): the vserver ID which created the lock. An integer not the UUID.

=back

Most of the inputs are knowable only to the n-blade which set them, so
are unlikely to be settable for testing...

=cut

sub d_lock_break ($%)
{
    my ($self, %args) = @_;

    my $ne = $self->zapi->invoke("d-lock-break", \%args);

    if (! $ne->results_ok() ) {
        fail_log( "d-lock-break()", $ne);
    }

    my $more = 0;
    my $r = $ne->child_get("has-more-locks");
    if ($r) {
        $more = ($r->{content} =~ /true|1/i) ? 1 : 0;
    }
    return $more;
}

=head2 d_lock_status_list

  my @locks = d_lock_status_list(%args);

Gives information about one or more locks on files. One or more
filters can be specified in combination to get matching locks. If no
inputs are specified, then all the locks in the system are returned.

=over

=item 'file-name'

(Optional): name of the file on which locks are held.

=item 'host'

(Optional): opaque string host id that owns the lock(s).

=item 'owner'

(Optional):Opaque string representation of lock owner

=item 'vif'

(Optional): Opaque unique VIF ID associated with the lock(s).

=item 'volume-dsid'

(Optional): DSID of the volume containg the files with lock(s).

=item 'vserver'

(Optional): Opaque integer ID for the vserver associated with the lock(s).

=back

Due to the generated nature of many of these fields, they are probably
not useful in a test script...

=cut

sub d_lock_status_list ($%)
{
    my ($self, %args) = @_;

    $args{'max-locks'} = 20;
    $args{'tag'} = 0;

    my $ne = $self->zapi->invoke('d-lock-status-list-iter', \%args);

    if (! $ne->results_ok() ) {
        fail_log( "d-lock-status-list()", $ne);
    }
}
