#
#  $Id: //depot/prod/test/Rsierranevada/lib/DBlade/dSystem.pm#1 $
#
#  Copyright (c) 2008 Network Appliance, Inc.
#  All rights reserved.
#

# -----------------------------------------------------------
##
## @summary Support routines for DBlade Zapi System functions
##
## @description
##	This file contains routines that let you exercise the
##	internal API of the filer using ssh access to the FreeBSD
##	shell command zsmcli.
##
## @author shenry@netapp.com

=head Summary

Support routines for d-blade Zapi d-system tests.

=head Description

This file contains routines that let you exercise the internal
(d-blade) system API of the filer using ssh access to the FreeBSD
shell command zsmcli.

=head2 Author

shenry@netapp.com

=cut

package DBlade::dSystem;

use	strict;
use	Tharn;
use	Socket;
use	LWP::UserAgent;
use	XML::Parser;
use	NaElement;
use	SPARSE::Util::Logger qw( logWarn logDbg logTrc );
use	Data::Dumper;
use	DBlade::dZapi;

use	vars ('@ISA', '@EXPORT');


=head2 new

  use DBlade::dSystem;
  my $zapi = new DBlade::dSystem($FILER, $user, $password);

Create a new instance of the dblade zapi interface.
All access is through member functions.

=over

=item $FILER

(Required): Hostspec or name of the converged filer.

=item $user

(Optional): Username to login to the filer's BSD shell with.
Defaults to "root".

=item $password

(Optional): password for the login account. Takes the password from
the nate_host file if not specified.

=back

=cut

#' emacs-fodder

sub new {
    my ($class, $server, $user, $password) = @_;

    my $zapi = new DBlade::dZapi($server, $user, $password);

    my $self = {
                zapi => $zapi,
               };
    bless $self, $class;
    return $self;
}


=head2 system_api_list

  use DBlade::dZapi;
  my $dzapi = new DBlade::dZapi($FILER, $user, $passwd);
  my @api_list = system_api_list();

Returns the array of available d-blade API command names.
Dies with $@ if the routine will not complete for some reason.

=cut

sub system_api_list
{
    my $self = shift;

    my $ne = $self->zapi->invoke("system-api-list", undef);

    if (! $ne->results_ok() ) {
        fail_log("system-api-list()", $ne);
    }

    my (@array, $elem, $nam);
    my $ar1 = $ne->child_get("apis");
    my $num1 = $ar1->child_max_number();
    for (my $i=0; $i < $num1; $i++) {
        $elem = $ar1->child_get_nth($i);
        $nam = $elem->child_get("name")->{content};
        push(@array, $nam);
    }

    return @array;
}


=head2 system_api_list_types

  use DBlade::dZapi;
  my $dzapi = new DBlade::dZapi($FILER, $user, $passwd);
  my %api_list_types = system_api_list_types();

Returns a hash of hash-refs of the available d-blade API
command names and their argument types. The top-level hash keys are
the API command name, the values are a hash ref of the argument
names (keys) and types (value). The type may be followed by the
string " optional" if it is an optional argument.

Dies with $@ if the routine will not complete for some reason.

=cut

sub system_api_list_types
{
    my $self = shift;

    my $ne = $self->zapi->invoke("system-api-list-types", undef);

    if (! $ne->results_ok() ) {
        fail_log( "system-api-list-types()", $ne);
    }

    my (%ret, $elem, $apiname, $typarr, $t, $typnam, $typtyp, $opt);
    my $ar0 = $ne->child_get("type-entries");
    my $num0 = $ar0->child_max_number();
    logDbg("system_api_list_types() returns $num0 apis");
    for (my $i=0; $i < $num0; $i++) {
        $elem = $ar0->child_get_nth($i);
        $apiname = $elem->child_get("name")->{content};
        $typarr = $elem->child_get("type-elements");
        my $num1 = $typarr->child_max_number();
        my %vals = ();
        for (my $j=0; $j < $num1; $j++) {
            $t = $typarr->child_get_nth($j);
            $typnam = $t->child_get("name")->{content};
            $typtyp = $t->child_get("type")->{content};
            $opt = $t->child_get("is-optional");
            if ( defined($opt) and $opt) {
                $vals{$typnam} = "$typtyp optional";
            } else {
                $vals{$typnam} = $typtyp;
            }
        }
        $ret{$apiname} = \%vals;
    }

    return %ret;
}

1;
