package MyTypes::PerfDynamicDataSources;
use strict;
use warnings;


__PACKAGE__->_set_element_form_qualified(1);

sub get_xmlns { 'http://www.netapp.com/management/v1' };

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %Color_of :ATTR(:get<Color>);
my %DataSource_of :ATTR(:get<DataSource>);
my %MaximumInstances_of :ATTR(:get<MaximumInstances>);
my %SelectAllInstances_of :ATTR(:get<SelectAllInstances>);
my %SortOrder_of :ATTR(:get<SortOrder>);
my %TimeInterval_of :ATTR(:get<TimeInterval>);

__PACKAGE__->_factory(
    [ qw(        Color
        DataSource
        MaximumInstances
        SelectAllInstances
        SortOrder
        TimeInterval

    ) ],
    {
        'Color' => \%Color_of,
        'DataSource' => \%DataSource_of,
        'MaximumInstances' => \%MaximumInstances_of,
        'SelectAllInstances' => \%SelectAllInstances_of,
        'SortOrder' => \%SortOrder_of,
        'TimeInterval' => \%TimeInterval_of,
    },
    {
        'Color' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
        'DataSource' => 'MyTypes::WrapperOfPerfDataSource',
        'MaximumInstances' => 'SOAP::WSDL::XSD::Typelib::Builtin::integer',
        'SelectAllInstances' => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
        'SortOrder' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
        'TimeInterval' => 'SOAP::WSDL::XSD::Typelib::Builtin::integer',
    },
    {

        'Color' => 'Color',
        'DataSource' => 'DataSource',
        'MaximumInstances' => 'MaximumInstances',
        'SelectAllInstances' => 'SelectAllInstances',
        'SortOrder' => 'SortOrder',
        'TimeInterval' => 'TimeInterval',
    }
);

} # end BLOCK







1;


=pod

=head1 NAME

MyTypes::PerfDynamicDataSources

=head1 DESCRIPTION

Perl data type class for the XML Schema defined complexType
PerfDynamicDataSources from the namespace http://www.netapp.com/management/v1.

A specification for selecting top-n Instances for a single counter. Data for each Instance is drawn on the chart as one line. e.g. top 5 busy storage systems in a Group.




=head2 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * Color


=item * DataSource


=item * MaximumInstances


=item * SelectAllInstances


=item * SortOrder


=item * TimeInterval




=back


=head1 METHODS

=head2 new

Constructor. The following data structure may be passed to new():

 { # MyTypes::PerfDynamicDataSources
   Color =>  $some_value, # string
   DataSource =>  { # MyTypes::WrapperOfPerfDataSource
     PerfDataSource =>  { # MyTypes::PerfDataSource
       ApplianceNameOrId =>  $some_value, # string
       CounterName =>  $some_value, # string
       GroupNameOrId =>  $some_value, # string
       InstanceName =>  $some_value, # string
       LabelNames =>  $some_value, # string
       ObjectFullName => $some_value, # ObjFullName
       ObjectName =>  $some_value, # string
       ObjectNameOrId => $some_value, # ObjNameOrId
       ObjectType =>  $some_value, # string
     },
   },
   MaximumInstances =>  $some_value, # integer
   SelectAllInstances =>  $some_value, # boolean
   SortOrder =>  $some_value, # string
   TimeInterval =>  $some_value, # integer
 },




=head1 AUTHOR

Generated by SOAP::WSDL

=cut

