package MyTypes::ResourceIdentifier;
use strict;
use warnings;


__PACKAGE__->_set_element_form_qualified(1);

sub get_xmlns { 'http://www.netapp.com/management/v1' };

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %Aggregate_of :ATTR(:get<Aggregate>);
my %Dataset_of :ATTR(:get<Dataset>);
my %Filer_of :ATTR(:get<Filer>);
my %Group_of :ATTR(:get<Group>);
my %Host_of :ATTR(:get<Host>);
my %Lun_of :ATTR(:get<Lun>);
my %Policy_of :ATTR(:get<Policy>);
my %Qtree_of :ATTR(:get<Qtree>);
my %RbacRole_of :ATTR(:get<RbacRole>);
my %ResourceId_of :ATTR(:get<ResourceId>);
my %ResourcePool_of :ATTR(:get<ResourcePool>);
my %StorageService_of :ATTR(:get<StorageService>);
my %Vfiler_of :ATTR(:get<Vfiler>);
my %VfilerTemplate_of :ATTR(:get<VfilerTemplate>);
my %Volume_of :ATTR(:get<Volume>);

__PACKAGE__->_factory(
    [ qw(        Aggregate
        Dataset
        Filer
        Group
        Host
        Lun
        Policy
        Qtree
        RbacRole
        ResourceId
        ResourcePool
        StorageService
        Vfiler
        VfilerTemplate
        Volume

    ) ],
    {
        'Aggregate' => \%Aggregate_of,
        'Dataset' => \%Dataset_of,
        'Filer' => \%Filer_of,
        'Group' => \%Group_of,
        'Host' => \%Host_of,
        'Lun' => \%Lun_of,
        'Policy' => \%Policy_of,
        'Qtree' => \%Qtree_of,
        'RbacRole' => \%RbacRole_of,
        'ResourceId' => \%ResourceId_of,
        'ResourcePool' => \%ResourcePool_of,
        'StorageService' => \%StorageService_of,
        'Vfiler' => \%Vfiler_of,
        'VfilerTemplate' => \%VfilerTemplate_of,
        'Volume' => \%Volume_of,
    },
    {
        'Aggregate' => 'MyTypes::WrapperOfAggregateResource',
        'Dataset' => 'MyTypes::WrapperOfDatasetResource',
        'Filer' => 'MyTypes::WrapperOfFilerResource',
        'Group' => 'MyTypes::WrapperOfGroupResource',
        'Host' => 'MyTypes::WrapperOfHostResource',
        'Lun' => 'MyTypes::WrapperOfLunResource',
        'Policy' => 'MyTypes::WrapperOfPolicyResource',
        'Qtree' => 'MyTypes::WrapperOfQtreeResource',
        'RbacRole' => 'MyTypes::WrapperOfRbacRoleResource',
        'ResourceId' => 'SOAP::WSDL::XSD::Typelib::Builtin::unsignedInt',
        'ResourcePool' => 'MyTypes::WrapperOfResourcePoolResource',
        'StorageService' => 'MyTypes::WrapperOfStorageServiceResource',
        'Vfiler' => 'MyTypes::WrapperOfVfilerResource',
        'VfilerTemplate' => 'MyTypes::WrapperOfVfilerTemplateResource',
        'Volume' => 'MyTypes::WrapperOfVolumeResource',
    },
    {

        'Aggregate' => 'Aggregate',
        'Dataset' => 'Dataset',
        'Filer' => 'Filer',
        'Group' => 'Group',
        'Host' => 'Host',
        'Lun' => 'Lun',
        'Policy' => 'Policy',
        'Qtree' => 'Qtree',
        'RbacRole' => 'RbacRole',
        'ResourceId' => 'ResourceId',
        'ResourcePool' => 'ResourcePool',
        'StorageService' => 'StorageService',
        'Vfiler' => 'Vfiler',
        'VfilerTemplate' => 'VfilerTemplate',
        'Volume' => 'Volume',
    }
);

} # end BLOCK







1;


=pod

=head1 NAME

MyTypes::ResourceIdentifier

=head1 DESCRIPTION

Perl data type class for the XML Schema defined complexType
ResourceIdentifier from the namespace http://www.netapp.com/management/v1.

Identifies a Resource. Only one Resource field must be set. i.e. one of ResourceId, RbacRole, Host, Group, storage system, Vfiler, Aggregate, Volume, ResourcePool, Dataset, Qtree, protection Policy, provisioning Policy, Lun or vFiler template. When an object Id is specified, it refers To the object Id field in the Objects table From the DFM database.




=head2 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * Aggregate


=item * Dataset


=item * Filer


=item * Group


=item * Host


=item * Lun


=item * Policy


=item * Qtree


=item * RbacRole


=item * ResourceId


=item * ResourcePool


=item * StorageService


=item * Vfiler


=item * VfilerTemplate


=item * Volume




=back


=head1 METHODS

=head2 new

Constructor. The following data structure may be passed to new():

 { # MyTypes::ResourceIdentifier
   Aggregate =>  { # MyTypes::WrapperOfAggregateResource
     AggregateResource =>  { # MyTypes::AggregateResource
       AggregateResourceNameOrId =>  { # MyTypes::WrapperOfAggregateNameOrId
         AggregateNameOrId =>  { # MyTypes::AggregateNameOrId
           AggregateId =>  $some_value, # unsignedInt
           AggregateName =>  $some_value, # string
         },
       },
       FilerIdentifier =>  { # MyTypes::WrapperOfFilerResource
         FilerResource =>  { # MyTypes::FilerResource
           FilerId =>  $some_value, # unsignedInt
           FilerName =>  $some_value, # string
         },
       },
     },
   },
   Dataset =>  { # MyTypes::WrapperOfDatasetResource
     DatasetResource =>  { # MyTypes::DatasetResource
       DatasetId =>  $some_value, # unsignedInt
       DatasetName =>  $some_value, # string
     },
   },
   Filer =>  { # MyTypes::WrapperOfFilerResource
     FilerResource =>  { # MyTypes::FilerResource
       FilerId =>  $some_value, # unsignedInt
       FilerName =>  $some_value, # string
     },
   },
   Group =>  { # MyTypes::WrapperOfGroupResource
     GroupResource =>  { # MyTypes::GroupResource
       GroupId =>  $some_value, # unsignedInt
       GroupName =>  $some_value, # string
     },
   },
   Host =>  { # MyTypes::WrapperOfHostResource
     HostResource =>  { # MyTypes::HostResource
       HostId =>  $some_value, # unsignedInt
       HostName =>  $some_value, # string
     },
   },
   Lun =>  { # MyTypes::WrapperOfLunResource
     LunResource =>  { # MyTypes::LunResource
       HostIdentifier =>  { # MyTypes::WrapperOfHostResource
         HostResource =>  { # MyTypes::HostResource
           HostId =>  $some_value, # unsignedInt
           HostName =>  $some_value, # string
         },
       },
       LunIdentifierNameOrId =>  { # MyTypes::WrapperOfLunNameOrId
         LunNameOrId =>  { # MyTypes::LunNameOrId
           LunId =>  $some_value, # unsignedInt
           LunName =>  $some_value, # string
         },
       },
       VolumeIdentifier =>  { # MyTypes::WrapperOfVolumeResource
         VolumeResource =>  { # MyTypes::VolumeResource
           AggregateIdentifier =>  { # MyTypes::WrapperOfAggregateResource
             AggregateResource =>  { # MyTypes::AggregateResource
               AggregateResourceNameOrId =>  { # MyTypes::WrapperOfAggregateNameOrId
                 AggregateNameOrId =>  { # MyTypes::AggregateNameOrId
                   AggregateId =>  $some_value, # unsignedInt
                   AggregateName =>  $some_value, # string
                 },
               },
               FilerIdentifier =>  { # MyTypes::WrapperOfFilerResource
                 FilerResource =>  { # MyTypes::FilerResource
                   FilerId =>  $some_value, # unsignedInt
                   FilerName =>  $some_value, # string
                 },
               },
             },
           },
           HostIdentifier =>  { # MyTypes::WrapperOfHostResource
             HostResource =>  { # MyTypes::HostResource
               HostId =>  $some_value, # unsignedInt
               HostName =>  $some_value, # string
             },
           },
           VfilerIdentifier =>  { # MyTypes::WrapperOfVfilerResource
             VfilerResource =>  { # MyTypes::VfilerResource
               VfilerId =>  $some_value, # unsignedInt
               VfilerNameOrUuid =>  $some_value, # string
             },
           },
           VolumeIdentifierNameOrId =>  { # MyTypes::WrapperOfVolumeNameOrId
             VolumeNameOrId =>  { # MyTypes::VolumeNameOrId
               VolumeId =>  $some_value, # unsignedInt
               VolumeName =>  $some_value, # string
             },
           },
         },
       },
     },
   },
   Policy =>  { # MyTypes::WrapperOfPolicyResource
     PolicyResource =>  { # MyTypes::PolicyResource
       PolicyId => $some_value, # ObjId
       PolicyName => $some_value, # ObjName
     },
   },
   Qtree =>  { # MyTypes::WrapperOfQtreeResource
     QtreeResource =>  { # MyTypes::QtreeResource
       HostIdentifier =>  { # MyTypes::WrapperOfHostResource
         HostResource =>  { # MyTypes::HostResource
           HostId =>  $some_value, # unsignedInt
           HostName =>  $some_value, # string
         },
       },
       QtreeIdentifierNameOrId =>  { # MyTypes::WrapperOfQtreeNameOrId
         QtreeNameOrId =>  { # MyTypes::QtreeNameOrId
           QtreeId => $some_value, # ObjId
           QtreeName => $some_value, # ObjName
         },
       },
       VolumeIdentifier =>  { # MyTypes::WrapperOfVolumeResource
         VolumeResource =>  { # MyTypes::VolumeResource
           AggregateIdentifier =>  { # MyTypes::WrapperOfAggregateResource
             AggregateResource =>  { # MyTypes::AggregateResource
               AggregateResourceNameOrId =>  { # MyTypes::WrapperOfAggregateNameOrId
                 AggregateNameOrId =>  { # MyTypes::AggregateNameOrId
                   AggregateId =>  $some_value, # unsignedInt
                   AggregateName =>  $some_value, # string
                 },
               },
               FilerIdentifier =>  { # MyTypes::WrapperOfFilerResource
                 FilerResource =>  { # MyTypes::FilerResource
                   FilerId =>  $some_value, # unsignedInt
                   FilerName =>  $some_value, # string
                 },
               },
             },
           },
           HostIdentifier =>  { # MyTypes::WrapperOfHostResource
             HostResource =>  { # MyTypes::HostResource
               HostId =>  $some_value, # unsignedInt
               HostName =>  $some_value, # string
             },
           },
           VfilerIdentifier =>  { # MyTypes::WrapperOfVfilerResource
             VfilerResource =>  { # MyTypes::VfilerResource
               VfilerId =>  $some_value, # unsignedInt
               VfilerNameOrUuid =>  $some_value, # string
             },
           },
           VolumeIdentifierNameOrId =>  { # MyTypes::WrapperOfVolumeNameOrId
             VolumeNameOrId =>  { # MyTypes::VolumeNameOrId
               VolumeId =>  $some_value, # unsignedInt
               VolumeName =>  $some_value, # string
             },
           },
         },
       },
     },
   },
   RbacRole =>  { # MyTypes::WrapperOfRbacRoleResource
     RbacRoleResource =>  { # MyTypes::RbacRoleResource
       RbacRoleId =>  $some_value, # unsignedInt
       RbacRoleName =>  $some_value, # string
     },
   },
   ResourceId =>  $some_value, # unsignedInt
   ResourcePool =>  { # MyTypes::WrapperOfResourcePoolResource
     ResourcePoolResource =>  { # MyTypes::ResourcePoolResource
       ResourcePoolId =>  $some_value, # unsignedInt
       ResourcePoolName =>  $some_value, # string
     },
   },
   StorageService =>  { # MyTypes::WrapperOfStorageServiceResource
     StorageServiceResource =>  { # MyTypes::StorageServiceResource
       StorageServiceId => $some_value, # ObjId
       StorageServiceName => $some_value, # ObjName
     },
   },
   Vfiler =>  { # MyTypes::WrapperOfVfilerResource
     VfilerResource =>  { # MyTypes::VfilerResource
       VfilerId =>  $some_value, # unsignedInt
       VfilerNameOrUuid =>  $some_value, # string
     },
   },
   VfilerTemplate =>  { # MyTypes::WrapperOfVfilerTemplateResource
     VfilerTemplateResource =>  { # MyTypes::VfilerTemplateResource
       VfilerTemplateId => $some_value, # ObjId
       VfilerTemplateName => $some_value, # ObjName
     },
   },
   Volume =>  { # MyTypes::WrapperOfVolumeResource
     VolumeResource =>  { # MyTypes::VolumeResource
       AggregateIdentifier =>  { # MyTypes::WrapperOfAggregateResource
         AggregateResource =>  { # MyTypes::AggregateResource
           AggregateResourceNameOrId =>  { # MyTypes::WrapperOfAggregateNameOrId
             AggregateNameOrId =>  { # MyTypes::AggregateNameOrId
               AggregateId =>  $some_value, # unsignedInt
               AggregateName =>  $some_value, # string
             },
           },
           FilerIdentifier =>  { # MyTypes::WrapperOfFilerResource
             FilerResource =>  { # MyTypes::FilerResource
               FilerId =>  $some_value, # unsignedInt
               FilerName =>  $some_value, # string
             },
           },
         },
       },
       HostIdentifier =>  { # MyTypes::WrapperOfHostResource
         HostResource =>  { # MyTypes::HostResource
           HostId =>  $some_value, # unsignedInt
           HostName =>  $some_value, # string
         },
       },
       VfilerIdentifier =>  { # MyTypes::WrapperOfVfilerResource
         VfilerResource =>  { # MyTypes::VfilerResource
           VfilerId =>  $some_value, # unsignedInt
           VfilerNameOrUuid =>  $some_value, # string
         },
       },
       VolumeIdentifierNameOrId =>  { # MyTypes::WrapperOfVolumeNameOrId
         VolumeNameOrId =>  { # MyTypes::VolumeNameOrId
           VolumeId =>  $some_value, # unsignedInt
           VolumeName =>  $some_value, # string
         },
       },
     },
   },
 },




=head1 AUTHOR

Generated by SOAP::WSDL

=cut

