package MyTypes::ScheduleContentInfo;
use strict;
use warnings;


__PACKAGE__->_set_element_form_qualified(1);

sub get_xmlns { 'http://www.netapp.com/management/v1' };

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %DailyList_of :ATTR(:get<DailyList>);
my %HourlyList_of :ATTR(:get<HourlyList>);
my %IsModifiable_of :ATTR(:get<IsModifiable>);
my %MonthlyList_of :ATTR(:get<MonthlyList>);
my %MonthlySubscheduleList_of :ATTR(:get<MonthlySubscheduleList>);
my %ScheduleCategory_of :ATTR(:get<ScheduleCategory>);
my %ScheduleDescription_of :ATTR(:get<ScheduleDescription>);
my %ScheduleId_of :ATTR(:get<ScheduleId>);
my %ScheduleName_of :ATTR(:get<ScheduleName>);
my %ScheduleType_of :ATTR(:get<ScheduleType>);
my %WeeklyList_of :ATTR(:get<WeeklyList>);
my %WeeklySubscheduleList_of :ATTR(:get<WeeklySubscheduleList>);

__PACKAGE__->_factory(
    [ qw(        DailyList
        HourlyList
        IsModifiable
        MonthlyList
        MonthlySubscheduleList
        ScheduleCategory
        ScheduleDescription
        ScheduleId
        ScheduleName
        ScheduleType
        WeeklyList
        WeeklySubscheduleList

    ) ],
    {
        'DailyList' => \%DailyList_of,
        'HourlyList' => \%HourlyList_of,
        'IsModifiable' => \%IsModifiable_of,
        'MonthlyList' => \%MonthlyList_of,
        'MonthlySubscheduleList' => \%MonthlySubscheduleList_of,
        'ScheduleCategory' => \%ScheduleCategory_of,
        'ScheduleDescription' => \%ScheduleDescription_of,
        'ScheduleId' => \%ScheduleId_of,
        'ScheduleName' => \%ScheduleName_of,
        'ScheduleType' => \%ScheduleType_of,
        'WeeklyList' => \%WeeklyList_of,
        'WeeklySubscheduleList' => \%WeeklySubscheduleList_of,
    },
    {
        'DailyList' => 'MyTypes::ArrayOfDailyInfo',
        'HourlyList' => 'MyTypes::ArrayOfHourlyInfo',
        'IsModifiable' => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
        'MonthlyList' => 'MyTypes::ArrayOfMonthlyInfo',
        'MonthlySubscheduleList' => 'MyTypes::ArrayOfMonthlySubscheduleInfo',
        'ScheduleCategory' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
        'ScheduleDescription' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
        'ScheduleId' => 'MyTypes::ObjId',
        'ScheduleName' => 'MyTypes::ObjName',
        'ScheduleType' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
        'WeeklyList' => 'MyTypes::ArrayOfWeeklyInfo',
        'WeeklySubscheduleList' => 'MyTypes::ArrayOfWeeklySubscheduleInfo',
    },
    {

        'DailyList' => 'DailyList',
        'HourlyList' => 'HourlyList',
        'IsModifiable' => 'IsModifiable',
        'MonthlyList' => 'MonthlyList',
        'MonthlySubscheduleList' => 'MonthlySubscheduleList',
        'ScheduleCategory' => 'ScheduleCategory',
        'ScheduleDescription' => 'ScheduleDescription',
        'ScheduleId' => 'ScheduleId',
        'ScheduleName' => 'ScheduleName',
        'ScheduleType' => 'ScheduleType',
        'WeeklyList' => 'WeeklyList',
        'WeeklySubscheduleList' => 'WeeklySubscheduleList',
    }
);

} # end BLOCK







1;


=pod

=head1 NAME

MyTypes::ScheduleContentInfo

=head1 DESCRIPTION

Perl data type class for the XML Schema defined complexType
ScheduleContentInfo from the namespace http://www.netapp.com/management/v1.

Detailed schedule contents. A schedule can be a daily, weekly, or monthly schedule. A daily schedule may include both multiple recurring hourly schedules (HourlyList element) and/or individual non-recurring daily schedules (DailyList element). A weekly schedule may include multiple references To daily schedules To be run on specific Days of the week (WeeklySubscheduleList element) and/or individual non-recurring weekly schedules (WeeklyList element). A monthly schedule may include multiple non-recurring monthly schedules (MonthlyList element). In addition, a monthly schedule may include a single reference To either a daily schedule or a montly schedule, not both. User may only specify one MonthlySubscheduleInfo element in the MonthlySubscheduleList.




=head2 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * DailyList


=item * HourlyList


=item * IsModifiable


=item * MonthlyList


=item * MonthlySubscheduleList


=item * ScheduleCategory


=item * ScheduleDescription


=item * ScheduleId


=item * ScheduleName


=item * ScheduleType


=item * WeeklyList


=item * WeeklySubscheduleList




=back


=head1 METHODS

=head2 new

Constructor. The following data structure may be passed to new():

 { # MyTypes::ScheduleContentInfo
   DailyList =>  { # MyTypes::ArrayOfDailyInfo
     DailyInfo =>  { # MyTypes::DailyInfo
       ItemId => $some_value, # atomic
       StartHour => $some_value, # atomic
       StartMinute => $some_value, # atomic
     },
   },
   HourlyList =>  { # MyTypes::ArrayOfHourlyInfo
     HourlyInfo =>  { # MyTypes::HourlyInfo
       EndHour => $some_value, # atomic
       EndMinute => $some_value, # atomic
       Frequency => $some_value, # atomic
       ItemId => $some_value, # atomic
       StartHour => $some_value, # atomic
       StartMinute => $some_value, # atomic
     },
   },
   IsModifiable =>  $some_value, # boolean
   MonthlyList =>  { # MyTypes::ArrayOfMonthlyInfo
     MonthlyInfo =>  { # MyTypes::MonthlyInfo
       DayOfMonth => $some_value, # atomic
       DayOfWeek => $some_value, # atomic
       ItemId => $some_value, # atomic
       StartHour => $some_value, # atomic
       StartMinute => $some_value, # atomic
       WeekOfMonth => $some_value, # atomic
     },
   },
   MonthlySubscheduleList =>  { # MyTypes::ArrayOfMonthlySubscheduleInfo
     MonthlySubscheduleInfo =>  { # MyTypes::MonthlySubscheduleInfo
       SubscheduleId => $some_value, # ObjId
       SubscheduleName => $some_value, # ObjName
     },
   },
   ScheduleCategory =>  $some_value, # string
   ScheduleDescription =>  $some_value, # string
   ScheduleId => $some_value, # ObjId
   ScheduleName => $some_value, # ObjName
   ScheduleType =>  $some_value, # string
   WeeklyList =>  { # MyTypes::ArrayOfWeeklyInfo
     WeeklyInfo =>  { # MyTypes::WeeklyInfo
       DayOfWeek => $some_value, # atomic
       ItemId => $some_value, # atomic
       StartHour => $some_value, # atomic
       StartMinute => $some_value, # atomic
     },
   },
   WeeklySubscheduleList =>  { # MyTypes::ArrayOfWeeklySubscheduleInfo
     WeeklySubscheduleInfo =>  { # MyTypes::WeeklySubscheduleInfo
       EndDayOfWeek => $some_value, # atomic
       ItemId => $some_value, # atomic
       StartDayOfWeek => $some_value, # atomic
       SubscheduleId => $some_value, # ObjId
       SubscheduleName => $some_value, # ObjName
     },
   },
 },




=head1 AUTHOR

Generated by SOAP::WSDL

=cut

