package MyTypes::StorageReliability;
use strict;
use warnings;


__PACKAGE__->_set_element_form_qualified(1);

sub get_xmlns { 'http://www.netapp.com/management/v1' };

our $XML_ATTRIBUTE_CLASS;
undef $XML_ATTRIBUTE_CLASS;

sub __get_attr_class {
    return $XML_ATTRIBUTE_CLASS;
}

use Class::Std::Fast::Storable constructor => 'none';
use base qw(SOAP::WSDL::XSD::Typelib::ComplexType);

Class::Std::initialize();

{ # BLOCK to scope variables

my %ControllerFailure_of :ATTR(:get<ControllerFailure>);
my %DiskFailure_of :ATTR(:get<DiskFailure>);
my %SubSystemFailure_of :ATTR(:get<SubSystemFailure>);

__PACKAGE__->_factory(
    [ qw(        ControllerFailure
        DiskFailure
        SubSystemFailure

    ) ],
    {
        'ControllerFailure' => \%ControllerFailure_of,
        'DiskFailure' => \%DiskFailure_of,
        'SubSystemFailure' => \%SubSystemFailure_of,
    },
    {
        'ControllerFailure' => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
        'DiskFailure' => 'SOAP::WSDL::XSD::Typelib::Builtin::string',
        'SubSystemFailure' => 'SOAP::WSDL::XSD::Typelib::Builtin::boolean',
    },
    {

        'ControllerFailure' => 'ControllerFailure',
        'DiskFailure' => 'DiskFailure',
        'SubSystemFailure' => 'SubSystemFailure',
    }
);

} # end BLOCK







1;


=pod

=head1 NAME

MyTypes::StorageReliability

=head1 DESCRIPTION

Perl data type class for the XML Schema defined complexType
StorageReliability from the namespace http://www.netapp.com/management/v1.

NetApp Storage Systems offer a wide range of storage availability features which provide protection against various failure like disk drive failures, shelf failures, controller failures or site failures. The required level of reliability of the Dataset can be specified in the provisioning Policy.




=head2 PROPERTIES

The following properties may be accessed using get_PROPERTY / set_PROPERTY
methods:

=over

=item * ControllerFailure


=item * DiskFailure


=item * SubSystemFailure




=back


=head1 METHODS

=head2 new

Constructor. The following data structure may be passed to new():

 { # MyTypes::StorageReliability
   ControllerFailure =>  $some_value, # boolean
   DiskFailure =>  $some_value, # string
   SubSystemFailure =>  $some_value, # boolean
 },




=head1 AUTHOR

Generated by SOAP::WSDL

=cut

