# $Id: //depot/prod/test/main/lib/DPGNACL/FindNACLObj.pm#1 $

#
# Copyright (c) 2010 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary Finding NACL objects.
## @author gmanish@netapp.com;dl-nb-tape-qa
## @status shared
## @pod here

=head1 NAME

DPGNACL::FindNACLObj

=head1 DESCRIPTION

=cut

=head1 METHODS

=head2 FindNACLObj

    DPGNACL::FindNACLObj->find_obj(
        TYPE       => <Element type which we are fetching>,
        OBJECT     => <Name of the element>,
        NODE       => <Node on which the element resides>,
        );

(Class method)
This method is used to fetch  the NACL object using NACL Find in conjuction.
with FILTER.
User will provide the Element Type, Element name and the Node on which it resides.
The library will fetch the respective object from the System and return an obj.

For ex : $Returnobj = DPGNACL::FindNACLObj->find_obj (TYPE=>'vserver',
                                                      OBJECT=>"$VserverName",
                                                      NODE=>$Node);
=over

=item Options

=over

=item C<< TYPE => $type >>

(Required)  element type [ vserver | volume | aggregate |Snapshot ]

=item C<< OBJECT =>  $element name >>

(Optional) name of the element, for ex: name of the volume

=item C<< NODE => $NODE  >>

(Required) Node on which element resides

=item C<< VSERVER => $vserver  >>

(Required) Required for snapshot and volume. 

=item C<< VOLUME => $volume  >>

(Required) Required only for only snapshots.Volume on which snapshot resides

=back

=back

=cut

package DPGNACL::FindNACLObj;

use strict;
use warnings;
use Tharn qw($Log);
use NATE::BaseException qw(:try);
use NACL::C::Node;
use NACL::STask::Aggregate;
use NACL::STask::Volume;
use NACL::STask::Vserver;
use NACL::ComponentUtils qw (Dumper);
use NACL::C::Snapshot;
use NACL::APISet::Exceptions::ResponseException;
use NACL::APISet::Exceptions::ResponseException;

sub find_obj {
    my $pkg = shift;
    my (%args) = @_;
    my $Vserver_obj;
    my $Volume_obj;
    my $Aggr_obj;
    my $Snap_obj;
    my $type    = $args{'TYPE'};
    my $object  = $args{'OBJECT'};
    my $node    = $args{'NODE'};
    my $vserver = $args{'VSERVER'};
    my $volume  = $args{'VOLUME'};
    $Log->enter();

    if ($type eq 'vserver') {
        try {
            $Vserver_obj =
              NACL::STask::Vserver->find(command_interface => $node,
                                         filter => {vserver => "$object",});
        }
        catch NACL::Exceptions::NoElementsFound with {
            my $error = shift;
            $Log->debug("Error is : " . $error->text());
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $error = shift;
            if ($error->text =~ /There are no entries matching your query/i) {
                $Log->debug("Vserver dosenot exist.");
            } else {
                $Log->error("Error is : " . $error->text());
                $error->throw($error->text);
            }
        };

        $Log->debug(' Vserver Object value : \n' . Dumper($Vserver_obj));
        return $Vserver_obj;
    }

    if ($type eq 'volume') {
        try {
            $Volume_obj =
              NACL::STask::Volume->find(command_interface => $node,
                                        filter => {volume => "$object",vserver  => "$vserver"});
        }
        catch NACL::Exceptions::NoElementsFound with {
            my $error = shift;
            $Log->debug("Error is : " . $error->text());
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $error = shift;
            if ($error->text =~ /No matching volume(s) found/i) {
                $Log->debug("Volume  dose not exist.");
            } else {
                $Log->error("Error is : " . $error->text());
                $error->throw($error->text);
            }
        };

        $Log->debug(' Volume Object value : \n' . Dumper($Volume_obj));
        return $Volume_obj;
    }

    if ($type eq 'aggregate') {
        try {
            $Aggr_obj =
              NACL::STask::Aggregate->find(command_interface => $node,
                                           filter => {aggregate => "$object",});
        }
        catch NACL::Exceptions::NoElementsFound with {
            my $error = shift;
            $Log->debug("Error is : " . $error->text());
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $error = shift;
            if ($error->text =~ /does not exist/i) {
                $Log->debug("Aggregate  dose not exist.");
            } else {
                $Log->error("Error is : " . $error->text());
                $error->throw($error->text);
            }
        };
        $Log->debug(' Aggregate Object value : \n' . Dumper($Aggr_obj));
        return $Aggr_obj;
    }

    if ($type eq 'snapshot') {
        if (!defined($vserver)) {
            $Log->error('Snapshot Find :Vserver value is mandatory to specify');
        }
        if (!defined($volume)) {
            $Log->error('Snapshot Find :Volume value is mandatory to specify');
        }
        try {
            $Snap_obj =
              NACL::C::Snapshot->find(command_interface => $node,
                                      filter            => {
                                                 snapshot => "$object",
                                                 vserver  => "$vserver",
                                                 volume   => "$volume",});
        }
        catch NACL::Exceptions::NoElementsFound with {
            my $error = shift;
            $Log->debug("Error is : " . $error->text());
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $error = shift;
            if ($error->text =~ /There are no entries matching your query/i) {
                $Log->debug("SnapShot dose not exist.");
            } else {
                $Log->error("Error is : " . $error->text());
                $error->throw($error->text);
            }
        };        
        $Log->debug(' SnapShot Object value : \n' . Dumper($Snap_obj));
        return $Snap_obj;
    }
}
1;
