package DPGNACL::LoadData;

use FSR::FSRutils;
use NACL::C::Restore;
use NACL::APISet::Exceptions::CommandFailedException;
use Data::Dumper;
use Tharn;
use File::Basename;

use vars qw(
    $hostrec
    $filer
    $filer_ipaddress
    $Api_Set_Obj
    $command_interface
    $cmd1
    $cmd2
    $cmd3
    $data_path
    $restore_location
    $System_Shell_password
    $status
    $output
    @match_table
    $Api_Set_Obj_SystemShell
   );
    
sub loaddata {
    $Log->enter();
    my $pkg = shift;

    my (%opts) = @_;
    my $path =
      '/x/eng/http/html/engineering/qa/fsr/dump_images/new_compat_data.dmp.gz';
    my $timeout = defined $opts{timeout} ? $opts{timeout} : 500;

    $command_interface = $opts{command_interface};
    $hostrec           = $command_interface->hostrec;
    $filer             = $hostrec->hostname;
    $filer_ipaddress   = $hostrec->default_ip;
    $restore_location  = $opts{restore_location};
    $data_path         = defined $opts{default_sample_path} ? $opts{default_sample_path} : $path;


    $mode = $command_interface->mode();
    if ( $mode =~ /CMode/i ) {
    
        $Api_Set_Obj_SystemShell = NACL::APISet->new(
            hostobj   => $hostrec,
            category  => 'Node',
            interface => 'CLI',
            set       => 'Systemshell',
        );

        $Api_Set_Obj = $command_interface->apiset(
            category  => 'Node',
            interface => 'CLI',
            set       => 'CMode'
    );    ## APIset obj

    
    #fetching systemshell password
    $System_Shell_password =
      $Api_Set_Obj->get_connection()->{prompt_password};

    @match_table = (
        '.*[Pp]assword.*:' => "$System_Shell_password",
        'The authenticity of host(.|\n)*?Are you sure you want to continue connecting \(yes\/no\)\?\s*' => 'yes',
        '.*(do you wish to overwrite (y or n)?).*' => 'y'
    );
        #connection to local client machine where ntest is running
        my $conn =
          connect("localhost#local;conntype=rsh-persay;conncmd=/bin/sh,-c");
        #getting base filename from the path 
        my $file_name = basename($data_path); 
        ## copy data to mroot of the C-Mode filer.
        $cmd1 = "scp $data_path diag\@$filer_ipaddress:/mroot";
        ##gunzip the DMP file in the mroot
        $cmd2 = "gunzip /mroot/$file_name";
        #getting the unzip dump file by cutting gz extension
        ($file_name) = $file_name =~ /(.+)\.gz/;
        ##removing the data from the mroot volume
        $cmd3 = "rm /mroot/$file_name";
        my $ssh_keygen_cmd  = "/usr/bin/ssh-keygen -R $filer_ipaddress";

        $output = $conn->execute(
                  command       => $ssh_keygen_cmd,
                  get_status    => \$status,
                  'timeout'     => $timeout
        );
	if (!defined $output) {
          NACL::APISet::Exceptions::CommandFailedException->throw($output);
        }

        $output = $conn->execute(
            command       => $cmd1,
            get_status => \$status,
            'match_table' => \@match_table,
            'timeout'     => $timeout
        );

        if ($output =~ /Name or service not known/i) {
        NACL::APISet::Exceptions::CommandFailedException->throw($output);
        }

        $output = $Api_Set_Obj_SystemShell->execute_raw_command(
            command                  => $cmd2,
            'connectrec-match_table' => \@match_table,
            'connectrec-timeout'     => $timeout
        );
        if ($output =~ /No such file or directory/i) {
        NACL::APISet::Exceptions::CommandFailedException->throw($output);
        }


        ## restore the data from the mroot to the specified restore_location
        NACL::C::Restore->restore(
            command_interface => $command_interface,
            'block-size'      => 'rfD',
            'tape-name'       => "/vol/vol0/$file_name",
            'target-location' => "$restore_location",
            'method-timeout'  => $timeout,

        );

        $Api_Set_Obj_SystemShell->execute_raw_command(
            command                  => $cmd3,
            'connectrec-match_table' => \@match_table,
            'connectrec-timeout'     => $timeout
        );    # deleting the unzipped folder

    }
    elsif ( $mode =~ /7Mode/i ) {
        eval { load_compat_data( $filer, $restore_location, $data_path ) }; ##calling NATE library for 7 Mode.
        if ($@) { NACL::APISet::Exceptions::CommandFailedException->throw($@); }
    }
    $Log->exit();
    return;
}

=head1 METHODS

=head2 LoadData

    DPGNACL::LoadData->loaddata(
                   command_interface    => $ci,
                   restore_location     => $restore_location,
                  );

(Class method)
This method is used to restore compatdata.dmp file located at shared location
on the given filer.

=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< "restore_location" => $restore_location >>
(Required) Restore location where script will restore the dmp file
=back

=cut
1;
