# $Id: //depot/prod/test/main/lib/DPGNACL/NdmpTask.pm#1 $

#
# Copyright (c) 2010 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary 
##  backup  - NDMP based 3 way backup and local backup
##  resotre - NDMP based 3 way restore and local restore
## Scope of the library is to Initiate NDMP backup /restore
## If NDMP server returns any error message then library will raise
## command failed execption.Dump engine log verification
## is out of scope of the library.Library will return the log
## message back to the user in the form of hash and user has to
## verify whether dump/restore is successful.
## @author dl-dpg-nb-automation;
## @status shared

package DPGNACL::NdmpTask;

use NDMP::NdmpCmds;
use NACL::APISet::Exceptions::CommandFailedException;
use Data::Dumper;
use Tharn;

use vars qw(
    $srcVol
    $OPTS
    $command_interface
    $mode
    $srcVol_hostname
    $data_filer
    $tapefiler
    $tape_filer
    $volumeName
    $vol_mount_path
    $user
    $password
    $hostrec
    $dstVol
    $dstVol_hostname
    $restoreTo
);

sub backup
{
    $Log->enter();
    my ($self,%opts) = @_;

    $srcVol =$opts{srcVol};
    delete $opts{srcVol};

    #from source volume object, we will retrieve filer information
    $command_interface = $srcVol->command_interface();
    $mode = $command_interface->mode();
    $Log->debug( 'Mode detected as:' . Dumper( $mode ) );
    $srcVol_hostname = $command_interface->hostrec()->hostname();

    ## Data filer from which we will initate backup
    $data_filer=defined $opts{data_filer} ? $opts{data_filer}:$srcVol_hostname;
    if($opts{data_filer}) {delete $opts{data_filer}};

    my $tapefiler=defined $opts{tapefiler}? $opts{tapefiler}:$command_interface;
    if($opts{tapefiler}) {delete $opts{tapefiler}};

    ## retrieving user name and password and hostname for the tape filer
    #$hostrec = $tapefiler->hostrec;
    $hostrec = $command_interface->hostrec();
    $tape_filer = $tapefiler->hostrec()->hostname();
    $user = $hostrec->default_username;
    $password  = $hostrec->default_password ;

    # reading option passed from the user and deleting it from the options hash.
    $user=defined $opts{src_username} ? $opts{src_username}:$user;
    if($opts{src_username}) {delete $opts{src_username}};

    my $vserver=$opts{vserver};
    if($opts{vserver}) {delete $opts{vserver}};

    $password=defined $opts{src_password} ? $opts{src_password}:$password;
    if($opts{src_password}) {delete $opts{src_password}};

    my $protocol=$opts{version}? $opts{version}:'4';
    if($opts{version}) {delete $opts{version}};

    my $mover_record=$opts{mover_record}?$opts{mover_record}:'64512' ;
    if($opts{mover_record}) {delete $opts{mover_record}};

    my $mover_window=$opts{mover_window}?$opts{mover_window}:'-1';
    if($opts{mover_window}) {delete $opts{mover_window}};

    my $tape=$opts{tape};
    if($opts{tape}) {delete $opts{tape}};

    my $isIncr=$opts{isIncr}?$opts{isIncr}:'0';
    if($opts{isIncr}) {delete $opts{isIncr}};

    my $want_log=$opts{want_log} ?$opts{want_log} :'1';
    if($opts{want_log}) {delete $opts{want_log}};

    my $EXCLUDE=$opts{EXCLUDE} ?$opts{EXCLUDE} :'1';

    # opts hash contain all the variable passed by the user.
    # options passed except deleted from opts hash [ in above stmts] will
    # be considered as env variable
    my $env=$opts{env};
    $OPTS{$_} = $env->{$_} for ( keys %$env );

    ## based on Mode, will build volume name - for c-mode it will be \verserver\volume
    ## and for 7-mode it will be \vol\volume name
    if ($mode =~ /CMode/i) {

        #constructing volume name. Vserver name will be supplied by the user
        $volumeName = "\/$vserver\/".$srcVol->state()->volume;
        $Log->debug( 'Volume Name for C-Mode is:' . Dumper( $volumeName ) );

    }  elsif ($mode =~ /7Mode/i) {

        #constructing volume name.
        $volumeName = '\/vol\/'.$srcVol->state()->volume;
        $Log->debug( 'Volume Name for 7-Mode is:' . Dumper( $volumeName ) );
    }

    if($opts{snapshot})
    {
        die "Snapshot provided does not exist, pleae provide the correct snapshot " unless($srcVol->snapshot_exists($opts{snapshot}));
        $vol_mount_path=$volumeName."\/\.snapshot\/".$opts{snapshot};
    }
    else {
        $vol_mount_path=$volumeName;
    }

    ## Calling ndmpLocalBackup and ndmp3WayBackup supported in NdmpCmds

    ## If source voulme filer from where data to be backed up is same as
    ## tape file, we will call localbackup
    if($data_filer=~/$tape_filer/)
        {
        $Log->debug( "Calling local backup, both tape and restore path is on" . Dumper( $data_filer ) );
            eval {
                     my %ret =NdmpCmds::ndmpLocalBackup($data_filer,$user,$password,$protocol,$mover_record,$mover_window,$tape,$isIncr,FILESYSTEM=>$vol_mount_path,WANT_LOGS=>$want_log,%opts);
            };
            if ($@) {NACL::APISet::Exceptions::CommandFailedException->throw($@);}
        return  \%ret;
        }
    ## If source voulme filer from where data to be backed up is different from
    ## tape filer, we will call 3way backup
    else{
        $Log->debug( 'Calling 3 Way Backup....');
         my $tape_user     = defined $opts{tape_username} ? $opts{tape_username}:$tapefiler->hostrec()->default_username;
         my $tape_password = defined $opts{tape_password} ? $opts{tape_password}:$tapefiler->hostrec()->default_password;
        $Log->debug( 'Calling 3Way backup , tape is on ' . Dumper( $tapefiler_host ) . " and restore path is on" . Dumper( $data_filer ) );
        eval {        
            my %ret =NdmpCmds::ndmp3WayBackup($data_filer,$tape_filer,$user,$tape_user,$password,$tape_password,$protocol,$mover_record,$mover_window,$tape,$isIncr,FILESYSTEM=>$vol_mount_path,WANT_LOGS=>$want_log,%OPTS);
        };
        if ($@) {NACL::APISet::Exceptions::CommandFailedException->throw($@);}
        return  \%ret;
        }
    $Log->exit();
=head1 METHODS
=head2 ::backup


    DPGNACL::NdmpTask->backup(
                src          => $Volume_Obj,
                tapefiler    => $node,
                data_filer   => $datafiler_IP,
                src_username => $datafiler_username,
                src_password => $datafiler_password,
                mover_record => NDMP_MOVER_RECORD_SIZE,
                mover_window => NDMP_MOVER_WINDOW_SIZE,
                tape         => $TAPE,
                isIncr       => '1',
                LEVEL        => '0',
                HIST         => 'Y',
            );
NdmpTask->backup perform local backup OR 3 way backup operation based on the input parametes and return a hash,
which has one of the element 'messages' that captures all the logs doing a restore operation is called.
For local backup tape and the source volume should be on the same filer .Source volume object is required
For 3wayBack up, tapefiler(command interface of the filer on which tape resides) is required .

=over

=item Arguments

B<src> =>I< Volume object of the volume to be backed up>
I<Required.>

B<data_filer> => I<Filer name or IP on which the volume to be backed up is resides, required only for negative testing, when the volume object passed does not reside on the data_filer, Also used in case of vfiler>
I<Optional.>

B<tapefiler> => I< Filer object [command interface] on which tape resides,defaults to the filer on which volume to be backed up resides.
In that cases local backup is done.Is a mandatory parameter in case 3 way backup is to be done>
I<Required.>

B<src_username> => I<Username for the src filer>
I<Optional.>
Defaults to src filer user name from tharnhost.Optional if LocalBackup is done
For C-Mode system, ndmdp server credetials are different from filer credetials hence madatory for C-Mode systems

B<src_password> => I<Password for the source filer>
I<Optional.>
Defaults to source filer password name from tharnhost.Optional if LocalBackup is done
For C-Mode system, ndmdp server credetials are different from filer credetials hence madatory for C-Mode systems

B<tape_username> => I<Username for the tape filer>
I<Optional.>
Defaults to tape filer user name from tharnhost.Optional if 3wayBackup is done

B<tape_password> => I<Password for the tape filer>
I<Optional.>
Defaults to tape filer password name from tharnhost.Optional if 3wayBackup is done

B<version> => I<Ndmp version used for doing tape backup>
I<Optional.>
Default NDMP version is 4. Otherwise provide the NDMP version

B<mover_record> => I<mover record size>
I<Optional>
Defaults to 64512 ,mover required size

B<mover_window> => I<mover window size>
I<Optional.>
Defaults to -1,mover window size

B<tape> => I<Tape name>
I<Required.>
Name of the tape where the backup is to be done

B<isIncr> => I<Is increment value>
I<Optional.>
Defaults to 0.Is it an incremental backup.

B<snapshot> => I<snap name on the source volume>
I<Optional.>
If given then src path given for ndmp backup will be /vol/<volname>/.snapshot/<snapshot>

B<env> => I<Hash of the environment value>
I<Optional.>
Hash of the environment values.

B<vserver> => I<VSever name for C-Mode systems>
I<Required.>
For C-Mode system, specify the vserver name on which source volume resides
=cut

sub restore
{
    $Log->enter();
    my ($self,%opts) = @_;

    $dstVol =$opts{dstVol};
    delete $opts{dstVol};

    $command_interface = $dstVol->command_interface();   #from source volume object, we will retrieve filer information
    $mode = $command_interface->mode();
    $Log->debug( 'Mode detected as:' . Dumper( $mode ) );
    $dstVol_hostname = $command_interface->hostrec()->hostname();
    
    ## Data filer from which we will initate restore
    $data_filer=defined $opts{data_filer} ? $opts{data_filer}:$dstVol_hostname;
    if($opts{data_filer}) {delete $opts{data_filer}};

    my $tapefiler=defined $opts{tapefiler}? $opts{tapefiler}:$command_interface;
    if($opts{tapefiler}) {delete $opts{tapefiler}};
    
    ## retrieving user name and password and hostname for the tape filer
    #$hostrec = $tapefiler->hostrec;
    $hostrec = $command_interface->hostrec();
    $tape_filer = $tapefiler->hostrec()->hostname();
    $user = $hostrec->default_username;
    $password  = $hostrec->default_password ;

    $user=defined $opts{dst_username} ? $opts{dst_username}:$user;
    if($opts{dst_username}) {delete $opts{dst_username}};

    my $vserver=$opts{vserver};
    if($opts{vserver}) {delete $opts{vserver}};

    my $password=defined $opts{dst_password} ? $opts{dst_password}:$password;
    if($opts{dst_password}) {delete $opts{dst_password}};

    my $protocol=$opts{version}? $opts{version}:'4';
    if($opts{version}) {delete $opts{version}};
    
    my $mover_record=$opts{mover_record}?$opts{mover_record}:'64512' ;
    if($opts{mover_record}) {delete $opts{mover_record}};

    my $mover_window=$opts{mover_window}?$opts{mover_window}:'-1';
    if($opts{mover_window}) {delete $opts{mover_window}};

    my $tape=$opts{tape};
    if($opts{tape}) {delete $opts{tape}};
    
    my $isIncr=$opts{isIncr}?$opts{isIncr}:'0';
    if($opts{isIncr}) {delete $opts{isIncr}};
    
    my $want_log=$opts{want_log} ?$opts{want_log} :'1';
    if($opts{want_log}) {delete $opts{want_log}};
    
    my $EXCLUDE=$opts{EXCLUDE} ?$opts{EXCLUDE} :'1';
    
    my $env=$opts{env};
    $OPTS{$_} = $env->{$_} for ( keys %$env );

    ## based on Mode, will build volume name - for c-mode it will be \verserver\volume
    ## and for 7-mode it will be \vol\volume name
    if ($mode =~ /CMode/i) {

        #constructing volume name. Vserver name will be supplied by the user        
        $volumeName = "\/$vserver\/".$dstVol->state()->volume;
        $Log->debug( 'Volume Name for C-Mode is:' . Dumper( $volumeName ) );

    }  elsif ($mode =~ /7Mode/i) {

        #constructing volume name.
        $volumeName = '\/vol\/'.$dstVol->state()->volume;
        $Log->debug( 'Volume Name for 7-Mode is:' . Dumper( $volumeName ) );
    }

    $restoreTo = $volumeName;
    ## Calling ndmpLocalRestore and ndmp3WayRestore supported in NdmpCmds

    ## If source voulme filer from where data to be backed up is same as
    ## tape filer, we will call localrestore

    if($data_filer=~/$tape_filer/)
        {
        $Log->debug( 'Calling local restore....');
        $Log->debug( 'Calling local restore, both tape and restore path is on' . Dumper( $data_filer ) );

        eval {
        my %ret =NdmpCmds::ndmpLocalRestore($data_filer,$user,$password,$protocol,$mover_record,$mover_window,$tape,$restoreTo,$isIncr,WANT_LOGS=>$want_log,%opts);
        };
        if ($@) {NACL::APISet::Exceptions::CommandFailedException->throw($@);}
        return  \%ret;
        }
    else{
        $Log->debug( 'Calling 3 Way restore....');
         my $tape_user     = defined $opts{tape_username} ? $opts{tape_username}:$tapefiler->hostrec()->default_username;
         my $tape_password = defined $opts{tape_password} ? $opts{tape_password}:$tapefiler->hostrec()->default_password;
        $Log->debug( "Calling 3Way restore , tape is on " . Dumper( $tapefiler_host ) . " and restore path is on" . Dumper( $data_filer ) );
        eval {
            my %ret =NdmpCmds::ndmp3WayRestore($data_filer,$tape_filer,$user,$tape_user,$password,$tape_password,$protocol,$mover_record,$mover_window,$tape,$restoreTo,$isIncr,WANT_LOGS=>$want_log,%OPTS);
        };
        if ($@) {NACL::APISet::Exceptions::CommandFailedException->throw($@);}
        return  \%ret;
        }
    $Log->exit();
}


=head1 METHODS
=head2 ::restore
    $return = DPGNACL::NdmpTask->restore(
                dstVol          => $Volume_Obj,
                tapefiler    => $node,
                data_filer   => $datafiler_IP,
                dst_username => $datafiler_username,
                dst_password => $datafiler_password,
                mover_record => NDMP_MOVER_RECORD_SIZE,
                mover_window => NDMP_MOVER_WINDOW_SIZE,
                tape         => $TAPE,
                isIncr       => '1',
                LEVEL        => '0',
                HIST         => 'Y',
            );
NdmpTask->restore perform local restore OR 3 way restore operation based on the input parametes and and return a hash,
which has one of the element 'messages' that captures all the logs doing a restore operation is called.
For local restore tape and the destiantion volume should be on the same filer .destination volume object is required
For 3wayrestore, tapefiler(command interface of the filer on which tape resides) is required .

=over

=item Arguments

B<dstVol> =>I< Destination Volume object of the volume on which resotre is to be done>
I<Required.>

B<data_filer> => I<Filer name or IP on which the volume on which we will perform resotre resides, required only for negative 
testing, when the volume object passed does not reside on the data_filer, Also used in case of vfiler>
I<Optional.>

B<tapefiler> => I< Filer object [command interface] on which tape resides,defaults to the filer on which volume resides to which we are
performing restore.
In that cases local restore is done.Is a mandatory parameter in case 3 way resotre operation to be performed>
I<Required.>

B<dst_username> => I<Username for the destination filer>
I<Optional.>
Defaults to destiantion  filer user name from tharnhost.Optional if LocalRestore is done
For C-Mode system, ndmdp server credetials are different from filer credetials hence madatory for C-Mode systems

B<dst_password> => I<Password for the destination filer>
I<Optional.>
Defaults to destination filer password name from tharnhost.Optional if Localrestore is done
For C-Mode system, ndmdp server credetials are different from filer credetials hence madatory for C-Mode systems

B<tape_username> => I<Username for the tape filer>
I<Optional.>
Defaults to tape filer user name from tharnhost.Optional if 3wayRestore is done

B<tape_password> => I<Password for the tape filer>
I<Optional.>
Defaults to tape filer password name from tharnhost.Optional if 3wayRestore is done

B<version> => I<Ndmp version used for doing tape resoter>
I<Optional.>
Default NDMP version is 4. Otherwise provide the NDMP version

B<mover_record> => I<mover record size>
I<Optional>
Defaults to 64512 ,mover required size

B<mover_window> => I<mover window size>
I<Optional.>
Defaults to -1,mover window size

B<tape> => I<Tape name>
I<Required.>
Name of the tape from where restore is to be done

B<isIncr> => I<Is increment value>
I<Optional.>
Defaults to 0.

B<env> => I<Hash of the environment value>
I<Optional.>
Hash of the environment values.

B<vserver> => I<VSever name for C-Mode systems>
I<Required.>
For C-Mode system, specify the vserver name on which source volume resides
=item Examples

=back
    For C-Mdoe Local restore
    $return = DPGNACL::NdmpRestore->restore(
                dstVol         => $testvol, 
                tape         => 'nrst0a',
                LEVEL        => '0',
                HIST         => 'Y',
                vserver      => $TestVserver,
                # in c-mode ndmp server credentials are different
                # so we will over ride tharhost defined username
                #and password
                dst_username  => 'root',
                dst_password  => 'admin',
);


For C-Mode 3 way restore
    $return =  DPGNACL::NdmpRestore->restore(
                dstVol        => $testvol, 
                tapefiler     => $node[0],  ## node [ filer connection object]
                tape          => 'nrst0a',
                LEVEL         => '0',
                HIST          => 'Y',
                vserver       => $TestVserver,
                tape_username => 'root',
                tape_password => 'admin',
                dst_username  => 'root',
                dst_password  => 'admin',
);

=cut

}
1;