#! /usr/local/bin/perl
# $Id: //depot/prod/test/main/lib/DPGNACL/Treecompare.pm#1 $

# Copyright (c) 2010 NetApp, Inc., All Rights Reserved
# Any use, modification, or distribution is prohibited
# without prior written consent from NetApp, Inc.
#

## @summary treecompare Component Module
## @author dl-dpg-nb-automation, gmanish@netapp.com
## @status shared
## @pod here
package DPGNACL::Treecompare;

use NACL::APISet::Exceptions::CommandFailedException;
use NACL::Exceptions::VerifyFailure;
use Tharn qw($Log);
use NACL::CS::Volume;
use NACL::C::Options;
use NACL::C::Treecompare;
use Data::Dumper;
use NACL::C::Volume;
use NACL::CS::Aggregate;
use NACL::STask::SystemLicense;
use vars qw(
    $command_interface
    $return_value
    $srcVolObj
    $dstVolObj
    $src_snap
    $dst_snap
    $was_created
    $timeout
    $ip
    $src_path
    $dst_path
    $src_volume_name
    $dst_volume_name
    $exclude
);

sub verify {
    $Log->enter();
    my $pkg = shift;
    $timeout = 1800;
    $ip = "127.0.0.1";
    my (%opts) = @_;
    $was_created = 0;
    ##reading the params passed to the lib
    $command_interface = $opts{command_interface};
    $srcVolObj         = $opts{'source_path'};
    $dstVolObj         = $opts{'destination_path'};
    $exclude           = $opts{'exclude'};
    $timeout = defined $opts{'Timeout'} ? $opts{'Timeout'} : $timeout;
    $ip      = defined $opts{'src_ip'} ? $opts{'src_ip'} : $ip;
    ## deleting the params from opts hash which we read above
    delete $opts{'source_path'};
    delete $opts{'destination_path'};
    delete $opts{'command_interface'};
    delete $opts{'Timeout'} if defined( $opts{'Timeout'} );
    delete $opts{'exclude'} if defined( $opts{'exclude'} );

    if (defined( $opts{'src_ip'} )){
      $opts{'remote'} = 1;
    }

    ## Verifying nature of the source-path
    ## Based on that defining source and dest path for treecompare.
    if (   ( $srcVolObj =~ /.*\/.snapshot\/.*/ )
        && ( $dstVolObj =~ /.*\/.snapshot\/.*/ ) ) {
        $src_path = "$ip:" . $srcVolObj;
        $dst_path = $dstVolObj;
        $srcVolObj =~ /.*\/.snapshot\/.*/;
        $src_volume_name = $2;
        $dstVolObj =~ /.*\/.snapshot\/.*/;
        $dst_volume_name = $2;
    } elsif (
        ( blessed $srcVolObj)
        && (   ( $srcVolObj->isa('NACL::C::Volume') )
            || ( $srcVolObj->isa('NACL::CS::Volume') ) )
        && ( blessed $dstVolObj)
        && (   ( $dstVolObj->isa('NACL::C::Volume') )
            || ( $dstVolObj->isa('NACL::CS::Volume') ) )
        ) {

        # Creating Snapshot for source-volume and destination-volume
        my $Snapshot_name = 'Treecompare_Snapshot_' . time();
        $was_created = 1;
        $src_snap
            = $srcVolObj->snapshot_create( snapshot => $Snapshot_name, );
        $dst_snap
            = $dstVolObj->snapshot_create( snapshot => $Snapshot_name, );

        $src_path
            = "$ip:/vol/"
            . $srcVolObj->volume
            . '/.snapshot/'
            . $Snapshot_name;
        $dst_path
            = "/vol/" . $dstVolObj->volume . "/.snapshot/" . $Snapshot_name;
        $src_volume_name = $srcVolObj->volume;
        $dst_volume_name = $dstVolObj->volume;
    } else {
        $Log->comment(
            'Source Path and destination path are in unspecified format');
        NACL::APISet::Exceptions::CommandFailedException->throw(
            'Source Path and destination path are in unspecified format');
    }

   if(!defined( $opts{'src_ip'} )){
     ## Verifying where volumes are co-located or not

    my $aggr_src_name_obj = NACL::CS::Volume->fetch(
        command_interface => $command_interface,
        filter            => { volume => $src_volume_name },
        requested_fields  => ['aggregate']
    );
    my $src_node = NACL::CS::Aggregate->fetch(
        command_interface => $command_interface,
        filter            => { aggregate => $aggr_src_name_obj->aggregate },
        requested_fields  => ['nodes']
    );

    my $aggr_dst_name_obj = NACL::CS::Volume->fetch(
        command_interface => $command_interface,
        filter            => { volume => $dst_volume_name },
        requested_fields  => ['aggregate']
    );
    my $dst_node = NACL::CS::Aggregate->fetch(
        command_interface => $command_interface,
        filter            => { aggregate => $aggr_dst_name_obj->aggregate },
        requested_fields  => ['nodes']
    );

    if ( $src_node->nodes->[0] !~ $dst_node->nodes->[0] ) {
        $Log->comment('Volumes are not collacted');
        $Log->comment('Not supported by lib');
        NACL::APISet::Exceptions::CommandFailedException->throw();
    }
}
    delete $opts{'src_ip'} if defined( $opts{'src_ip'} );
 
    # Enable Snapmirror
    NACL::STask::SystemLicense->add(
        command_interface    => $command_interface,
        feature              => 'SnapMirror',
        nacltask_verify      => 1,
        nacltask_if_licensed => 'pass',
    );

    NACL::C::Options->option(
        command_interface => $command_interface,
        apiset_should     => { set => 'Nodescope' },
        'option-name'     => 'snapmirror.enable',
        'option-value'    => 'on'
    );

    NACL::C::Options->option(
        command_interface => $command_interface,
        apiset_should     => { set => 'Nodescope' },
        'option-name'     => 'snapmirror.access',
        'option-value'    => 'all'
    );

    ## Initating Treecompare
    $Log->comment( 'Treecompare Tree 1 path:' . $src_path );
    $Log->comment( 'Treecompare Tree 2 path:' . $dst_path );

    my $response = NACL::C::Treecompare->start(
        command_interface   => $command_interface,
        'treepath1'         => $src_path,
        'treepath2'         => $dst_path,
        'dc-ctime'          => 1,
        'dc-vdisk-metadata' => 1,
        'method-timeout'    => $timeout,
        'privilege-level'   => 'diag',
        apiset_should       => { set => 'Nodescope' },
        %opts
    );
    my $raw_output = $response->get_raw_output();

    @logs_array = split( /TREECMP:/, $raw_output );
    my ( @restore_symbol_table_array, @data_mismatch_array );

    if ( !defined($exclude) ) {
        foreach (@logs_array) {
            if ( $_
                =~ /Found.*restore_symboltable|rstab|Found unnecessary disk usage/i
                ) {
                push @restore_symbol_table_array, $_;
            } elsif ( $_ =~ /Found/i ) {
                push @data_mismatch_array, $_;
            }
        }
    } else {

        my @exclude_list = @$exclude;
        $Log->comment("Exclude list is:\n");
        foreach (@exclude_list) {
            $Log->comment(" $_ \n");
        }

        $Log->comment(
            'Exclude list is provided,ignoring them from treecomparemismatch'
        );
        foreach my $log_line (@logs_array) {
            if ( $log_line
                =~ /Found.*restore_symboltable|rstab|Found unnecessary disk usage/i
                ) {
                push @restore_symbol_table_array, $log_line;
            } elsif ( check( $log_line, @exclude_list ) ) {
                push @restore_symbol_table_array, $log_line;
            } elsif ( $log_line =~ /Found/i ) {
                push @data_mismatch_array, $log_line;
            }
        }

    }
    if ( ( $#data_mismatch_array + 1 ) > 0 ) {
        $return_value = 'FAIL';
        $Log->comment( 'DATA Mismatch found' . Dumper(@data_mismatch_array) );
        NACL::Exceptions::VerifyFailure->throw('Data Mismtach Found');
    } elsif ( ( $#restore_symbol_table_array + 1 ) > 0 ) {
        $return_value = 'PASS';
        $Log->comment('No mismatch observed except restore_symbol table');
    } else {
        $return_value = 'PASS';
        $Log->comment('Treecomapare succeed');
    }

    if ( $was_created == 1 ) {
        $src_snap->delete();
        $dst_snap->delete();
    }
    $Log->exit();
    return $return_value;
} ## end sub verify

sub check() {
    my ( $log_line, @exclude_list ) = @_;

    foreach (@exclude_list) {
        if ( $log_line =~ /$_/ ) { return 1; }
    }

    return 0;
} ## end sub check

=head1 NAME
DPGNACL::Treecompare
=head1 DESCRIPTION
It builds on top of, and is a derived class of C<NACL::C::Treecompare>,
and so it also provides methods that are more in the scope of
individual treecompare-related commands.  See C<NACL::C::Treecompare> for
details.

=head1 METHODS

=head2 Treecompare

    DPGNACL::Treecompare->verify(
                    command_interface    => $ci,
                    source_path =>  $vol_A_obj,
                    destination_path =>  $vol_B_obj,
                    exclude  => \@exclude_array,
                    src_ip   => $src_ip,
                  );

	DPGNACL::Treecompare->verify(
                    command_interface    => $ci,
                    source_path =>  "/vol/vol1/.snapshot/snapshot1",
                    destination_path =>  "/vol/vol1/.snapshot/snapshot1",
                    exclude  => \@exclude_array,
                    src_ip   => "127.0.0.1",
                  );
(Class method)
This method is used for compare two filesystem specified by the user
Currently supproted only if both volumes are co located
Invokes "treecompare start" command for CMode using node scope
=over

=item Options

=over

=item C<< command_interface => $command_interface >>

(Required) See L<NACL::C::Component|lib-NACL-C-Component-pm/command_interface>

=item C<< "source_path" => $source_path >>
(Required) source location
can be a vol object, snapshot location or file path
=item C<< "destination_path" => $source_path >>
(Required) destination location
can be a vol object, snapshot location or file path
=item C<< "exclude" => \@exclude_list >>
(Optional) Refernce of an array consisting of exlude strings 
can be a vol object, snapshot location or file path
=item C<< "Timeout" => $timeout >>
(Optional) Timeout value
Required when treecompare operation is taking long time.
by default set to 1800 seconds
=item C<< "src_ip => $src_ip >>
(Optional) set when 3way treecompare is needed
Required when treecompare operations should be carried out on volumes residing on different nodes
 When this option is given it automatically sets  "remote => 1" during treecompare
=item C<< "%opts" => $scalar >>
(Optional) Timeout value
The new timeout value..
by default set to 1800 seconds
(Optional) remote 
by default is 0, when src_ip is passed it is set to 1

=back

=cut

1;
