package Data::ASCIIFileChurner;

use Data::Dumper;
use Carp;
use strict;
use Data::File::ASCII;

our $DEBUG = 0;

=head1 NAME

  Data::ASCIIFileChurner - Short Description
  
=head1 DESCRIPTION

  Put description here. 
  
=head1 SYNOPSIS

  use Data::ASCIIFileChurner;

=head1 METHODS

=head2 new()

  The constructor.  Takes no arguments.

=cut

sub new {

	my ( $class ) = @_;

	my $self = bless {}, $class;

	return $self;
}

sub set_file {
	my ( $self, $file ) = @_;
	croak "file must be an instance of Data::File::ASCII" unless ( $file->isa( 'Data::File::ASCII' ) );
	$self->{file} = $file;
}
sub get_file { my ( $self ) = @_; return $self->{file} }

sub set_size {
	my ( $self, $size ) = @_;
	croak "size must be a non-negative integer (bytes): $size" if ( $size =~ /\D/ || $size < 0 );
	croak "You must set file before size" unless $self->get_file;
	croak "size cannot be larger than file's size." if $size > $self->get_file->get_size;
	$self->{size} = $size;
}
sub get_size { my ( $self ) = @_; return $self->{size} }

=head2 churn()

  Instance method.
  
  Takes nothing.  This overwrites some of the existing data in the file, 
  replacing it with a random string, beginning at a random offset.  The 
  offset is not allowed to result in the file growing in size.

=cut

sub churn {

	my ( $self ) = @_;
	
	my $start_range = $self->get_file->get_size - $self->get_size;
		
	my $start = int rand( $start_range );
	my $end = $start + $self->get_size - 1;
	
	my $filename = $self->get_file->get_name;

	my $b = `cat $filename` if $DEBUG;

	open( FILE, "+<$filename" ) || croak "Couldn't open file: $filename";
	
	seek( FILE, 0, 0 );
	
	for ( my $i = $start; $i <= $end; $i++ ) {
		seek( FILE, $i, 0 );
		#print FILE Data::File::ASCII::random_char();
		print FILE '+';
	}
	
	close FILE;
	
	if ( $DEBUG ) {	
		my $a = `cat $filename`;
		print "$filename\n$b\n$a\n" . " " x $start . ( "-" x ( $end - $start + 1 ) ) ."\n";
	}
}

=head2 read_all()

  Instance method.
  
  Takes nothing.  Reads an entire file.  Doesn't do anything with the data.
  Returns nothing. 

=cut

sub read_all {

	my ( $self ) = @_;
	
	my $filename = $self->get_file->get_name;
	
	open( FILE, "<$filename" ) || croak "Couldn't open file: $filename";
	while ( <FILE> ) {
		;
	}	
	close FILE;
}

=head2 to_string()

  Takes no arguments.  Returns a string of keys/values for this object instance.

=cut

sub to_string {

	my ( $self ) = @_;

	my $string = "Instance of \"" . __PACKAGE__ . "\"\n";
	$string .= "=" x ( ( length $string ) - 1 ) . "\n";

	my $longest = '';
	my $i;
	for ( keys %$self ) {
		$longest = $_ unless $i;
		$longest = $_ if ( length $_ >= length $longest );
		$i++;
	}

	for ( sort keys %$self ) {
		my $pad = ( length $longest ) - ( length $_ );
		$string .= "$_ " . " " x $pad . ": $self->{$_}\n";
	}
	
	$string .= "\n";

	return $string;
}

1;

=head1 TODO

  Make a TODO list.
  
=head1 DEPENDENCIES

  Data::Dumper.pm

=head1 AUTHOR

  Todd Shoenfelt (todd@netapp.com)
  
=cut
















