# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/DataManip/defaults.pm#1 $

## @summary environment type definitions for DataManip
## @author remi@netapp.com
## @status public
## @pod here

=head1 NAME

DataManip::defaults - environment type definitions for DataManip

=head1 DESCRIPTION

This package defines the available environment types for DataManip.
An "environment type" is really just a set of default values for all
the options available to the DataManip routines.

=cut

package DataManip::defaults;
use warnings;
use strict;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(%defaults);
our @EXPORT_OK = qw();

# Table of default option values
our %defaults = ();

my @required_options =
  qw(DIR_AVG  FILE_AVG  DEPTH_AVG  BREADTH_AVG
     DIR_DEV  FILE_DEV  DEPTH_DEV  BREADTH_DEV
     CREATE_FILE_AVG  CREATE_DIR_AVG  CREATE_FILE_SIZE_AVG  SUBTREE_DEPTH_AVG
     CREATE_FILE_DEV  CREATE_DIR_DEV  CREATE_FILE_SIZE_DEV  SUBTREE_DEPTH_DEV
     TOTAL_MOD  TOTAL_GROW  TOTAL_SHRINK  TOTAL_DEL   TOTAL_SUBTREE_DEL
     MOD_AVG    GROW_AVG    SHRINK_AVG    FRAGMENT_AVG
     MOD_DEV    GROW_DEV    SHRINK_DEV    FRAGMENT_DEV
     MOD_RAW    GROW_RAW    SHRINK_RAW    FRAGMENT_RAW
     DEPTH_BASE MIN_DIR_SIZE
     FLOOD SCRAMBLE
     FORK_COUNT PERSEVERE VERBOSE);
my %req_opts = map { $_ => 1 } @required_options;

=head1 ENVIRONMENT TYPES

Each environment type is described below.
Everyone is encouraged to add their own.

=head2 nil

This is the environment type where every option is set to zero.  It is
the environment type used when no environment type is given and the
default environment type is avoided because other data-modifying
options have been set.

=head2 filer

This is a generic environment type, intended for users that are just
looking for a good mix of everything and do not care about specifics.
However, the dataset generated will typically be larger than what
people are patient enough to handle on simulators.  Hence, it is
primarily meant for use in populating datasets on real filers or
locally-attached disks.

This is also the "default" environment type.  Actually, it is only
ever used as such when no environment type is specified B<and> no
data-modifying options are set.

=head2 simulator

This is also a generic environment type, only on a small scale.  It
generates a similar kind of dataset as the C<filer> environment type,
but tries to keep the dataset small enough to fit on simulators.
Hence, this environment type is primarily meant for populating
datasets on simulators (or where the user does not want to wait very
long).

=cut

my $base = <<'BASE';
                                nil      filer  simulator
             BREADTH_AVG          0      32768      16384
             BREADTH_DEV          0      16384       4096
          CREATE_DIR_AVG          0        0.3        0.3
          CREATE_DIR_DEV          0        0.1  0.0833333
         CREATE_FILE_AVG          0          1        0.5
         CREATE_FILE_DEV          0        0.4       0.15
    CREATE_FILE_SIZE_AVG          0      65536       8192
    CREATE_FILE_SIZE_DEV          0      12288       4096
               DEPTH_AVG          0        4.3        4.3
              DEPTH_BASE          0          0          0
               DEPTH_DEV          0          1          1
                 DIR_AVG          0        3.2        3.2
                 DIR_DEV          0          1          1
                FILE_AVG          0          5          4
                FILE_DEV          0          2          2
                   FLOOD          0          0          0
              FORK_COUNT          0          0          0
            FRAGMENT_AVG          0       8192          1
            FRAGMENT_DEV          0        512          0
            FRAGMENT_RAW          0          1          0
                GROW_AVG          0       0.05       0.05
                GROW_DEV          0  0.0066666  0.0066666
                GROW_RAW          0          0          0
            MIN_DIR_SIZE          0          0          0
                 MOD_AVG          0       0.25       0.25
                 MOD_DEV          0          0          0
                 MOD_RAW          0          0          0
               PERSEVERE          0          0          0
                SCRAMBLE          0          0          0
              SHRINK_AVG          0       0.05       0.05
              SHRINK_DEV          0  0.0066666  0.0066666
              SHRINK_RAW          0          0          0
       SUBTREE_DEPTH_AVG          0          6        3.5
       SUBTREE_DEPTH_DEV          0        0.4       0.25
               TOTAL_DEL          0        0.2      0.125
              TOTAL_GROW          0        0.1        0.1
               TOTAL_MOD          0        0.5        0.5
            TOTAL_SHRINK          0        0.1        0.1
       TOTAL_SUBTREE_DEL          0       0.06      0.075
                 VERBOSE          0          0          0
BASE

=head2 gigabytes

This environment type is intended for large-scale testing.  As the
name suggests, it creates a dataset many gigabytes in size.

=head2 apple

This environment type was used for testing during an escalation at
Apple.  The dataset is huge (about a terabyte), but somewhat
uninteresting in terms of data or files.  Modifications with this
environment type focus on doing lots of small (1 KB) random overwrites
to heavily fragment the disks, but do not grow or shrink any files.

=head2 dpg_4tb

This is a dataset for DPG large-scale testing.
Currently, it only has parameters for populate, not modify.
On average, it creates a dataset that is around 4 TB in total size,
with around 30 million inodes (1.1 million directories and 28.9
million files).  The files average around 140 KB each.

=cut

my $large = <<'LARGE';
                          gigabytes      apple    dpg_4tb
             BREADTH_AVG      65536      60416     146243
             BREADTH_DEV      16384       2048      48747
          CREATE_DIR_AVG          0          0          0
          CREATE_DIR_DEV          0          0    0.30303
         CREATE_FILE_AVG          0          0          3
         CREATE_FILE_DEV          0          0          1
    CREATE_FILE_SIZE_AVG          0          0     146243
    CREATE_FILE_SIZE_DEV          0          0      48747
               DEPTH_AVG          6       4.75        6.0
              DEPTH_BASE          0          0          0
               DEPTH_DEV        0.5     0.1056 0.33333333
                 DIR_AVG        4.5          5       10.0
                 DIR_DEV       0.75      0.175        0.5
                FILE_AVG          7       1438         26
                FILE_DEV       1.75          8          9
                   FLOOD          0          1          0
              FORK_COUNT          0         25          0
            FRAGMENT_AVG      16384       1024        1.5
            FRAGMENT_DEV       1024          0       0.33
            FRAGMENT_RAW          0          1          0
                GROW_AVG       0.05          0          0
                GROW_DEV  0.0066666          0          0
                GROW_RAW          0          0          0
            MIN_DIR_SIZE          0          0          0
                 MOD_AVG       0.25     0.0625       0.21
                 MOD_DEV      0.025          0       0.05
                 MOD_RAW          0          0          0
               PERSEVERE          0          0          0
                SCRAMBLE          0          0          0
              SHRINK_AVG       0.05          0          0
              SHRINK_DEV  0.0066666          0          0
              SHRINK_RAW          0          0          0
       SUBTREE_DEPTH_AVG          0          0        6.0
       SUBTREE_DEPTH_DEV          0          0 0.33333333
               TOTAL_DEL          0          0     0.1153
              TOTAL_GROW        0.1          0          0
               TOTAL_MOD        0.5      0.039      0.346
            TOTAL_SHRINK        0.1          0          0
       TOTAL_SUBTREE_DEL          0          0     0.0495
                 VERBOSE          0          1          0
LARGE

=head2 VP3 environment types

These three environment types (vp3-macro, macromod3, and macromod7)
are for creating and modifying the VERITAS P3 performance benchmark
dataset.  For documentation on how all these values were constructed,
and how to use these types, see the performance test specification at:

L<http://web.netapp.com/engineering/design-depot/dam/testing/devel/testplans/TsingTao_VP3_Performance.html>

=cut

my $vp3 = <<'VP3';
                          vp3-macro  macromod3  macromod7
             BREADTH_AVG      65536     262144     262144
             BREADTH_DEV       4096      43690      43690
          CREATE_DIR_AVG          0          0          0
          CREATE_DIR_DEV          0      0.275      0.275
         CREATE_FILE_AVG        1.8      0.162      0.378
         CREATE_FILE_DEV 0.33333333 0.33333333 0.33333333
    CREATE_FILE_SIZE_AVG    1048576     262144     262144
    CREATE_FILE_SIZE_DEV     174763      43690      43690
               DEPTH_AVG          8          8          8
              DEPTH_BASE          0          0          0
               DEPTH_DEV        0.5        0.5        0.5
                 DIR_AVG       3.06       3.06       3.06
                 DIR_DEV       0.02       0.02       0.02
                FILE_AVG        7.2          9          9
                FILE_DEV 0.33333333 0.33333333 0.33333333
                   FLOOD          0          0          0
              FORK_COUNT          0          0          0
            FRAGMENT_AVG          0      30720      30720
            FRAGMENT_DEV          0       3414       3414
            FRAGMENT_RAW          0          1          1
                GROW_AVG          0          0          0
                GROW_DEV          0          0          0
                GROW_RAW          0          0          0
            MIN_DIR_SIZE          0          0          0
                 MOD_AVG          0     262144     262144
                 MOD_DEV          0      43690      43690
                 MOD_RAW          0          1          1
               PERSEVERE          0          0          0
                SCRAMBLE          0          0          0
              SHRINK_AVG          0          0          0
              SHRINK_DEV          0          0          0
              SHRINK_RAW          0          0          0
       SUBTREE_DEPTH_AVG          0          8          8
       SUBTREE_DEPTH_DEV          0        0.5        0.5
               TOTAL_DEL          0      0.018      0.042
              TOTAL_GROW          0          0          0
               TOTAL_MOD          0 0.01221996 0.02922755
            TOTAL_SHRINK          0          0          0
       TOTAL_SUBTREE_DEL          0   0.011375   0.011375
                 VERBOSE          0          0          0
VP3

=pod

The environment type definitions themselves are just text blocks which
this library exports as a hash table.

=cut

sub split_block( $$ ) {
  my ($text, $block_name) = @_;
  my %types;

  my @lines = split(/\n/, $text);
  my $header = shift @lines;
  $header =~ s/\A\s+//;
  $header =~ s/\s+\z//;
  my @names = split(/\s+/, $header);

  foreach my $line (@lines) {
    $line =~ s/\A\s+//;
    $line =~ s/\s+\z//;
    next unless (length($line) > 0);

    my @fields = split(/\s+/, $line);
    my $opt = shift @fields;

    die "Unrecognized option \"$opt\" in block $block_name\n"
      unless (defined $opt && exists $req_opts{$opt});
    die "Incomplete table line in block $block_name for option $opt.\n"
      if (scalar(@fields) < scalar(@names));
    die "Extraneous fields in block $block_name for option $opt.\n"
      if (scalar(@fields) > scalar(@names));

    foreach my $name (@names) {
      $types{$name}->{$opt} = shift @fields;
    }
  }
  return %types;
}

%defaults = (split_block($base, 'BASE'),
	  split_block($large, 'LARGE'),
          split_block($vp3, 'VP3')
	 );


=head1 UPDATES

Everyone is invited to add their own environment types to this
package, for convenience if not for necessity.  To do so, merely
add a new key to the environment type hash, and define all the
necessary options.  This package itself will refuse to compile
if all the required options are not defined.

When adding a new option to DataManip, the option must be added to the
list of required options, and must be defined in all existing
environment types.  By default, new options should be set to zero in
existing environment types to avoid changing their behavior.

=cut

my ($env_type, $hashref);
my $errors = "";
while (($env_type, $hashref) = each %defaults) {
  foreach my $opt (@required_options) {
    unless (exists $hashref->{$opt}) {
      $errors .= "Option $opt does not exist " .
	"in environment type \"$env_type\".\n";
      next;
    }
    my $val = $hashref->{$opt};
    unless (defined $val && $val =~ /\A-?\d+(?:\.\d+)?\z/) {
      $errors .= "Bad value \"$val\" for option $opt" .
	" in environment type \"$env_type\".\n";
      next;
    }
  }
}
die $errors if ($errors);


=head1 BUGS

The C<filer> and C<simulator> environment types are B<heavily>
variable.  Someone needs to run each a bunch of times, and tweak the
deviations so that their sizes are a bit more predictable.

The descriptions above do not talk much about how the environment
types modify their datasets (probably because the current code to
modify datasets is a little feature-hungry).

=cut


1;
