# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide a database utility package for FabricMgr
## @author derricks@netapp.com
## @status private
## @pod here
#
# $Id: //depot/prod/test/Rsierranevada/lib/DbUtil.pm#1 $
package DbUtil;

# Compiler directives.
use strict;

# Module import.
use Tharn;
use Error;
use San::SanLog;
use Data::Dumper;
use ParamsMgr;

use vars qw(@ISA);

@ISA = qw(Error SanLog);

###############################################################################
#                               Public Methods
###############################################################################
###############################################################################
# Method:    new
# Objective: Create a new DbUtil object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class, %args) = @_;

    my $self = {};

    # bless the object upfront in case an error occurs.
    bless $self, $class;

    # Clear out the errors
    $self->_clear_err();

    # need to tell Error class to handle errors in a special way
    # this will allow us to invoke set err over and over to create an error stack
    # without overwriting the previous error
    $self->{error_dont_overwrite_errors} = 1;

    # Make sure the stack style is normal
    $self->{error_stack_style} = "normal";

    &ParamsMgr::process_args(%args);
    my $type = &ParamsMgr::get_arg(-arg => "type", -required => 1, -default => "");

    if (&ParamsMgr::error()) {
        $self->_set_err("Args parsing failed", &ParamsMgr::get_error());
        return $self;
    }

    if (lc($type) eq "postgres") {
        require DbUtil::postgres;
        $self = DbUtil::postgres->_new(%args);
    } else {
        # Invalid database type provided
        $self->_set_err("Database type '$type' is not supported");
        return $self;
    }

    #@ISA = qw(Error SanLog DbUtil);

    return $self;
}
