{ package   Dense::Agnostic::7mode::Aggr;
    use strict;
    use warnings;
    
    use Data::Dumper;

    use constant    TYPE_7M_NODE    => 'Dense::Agnostic::7mode::Node';

    use Tharn   qw($Log);
    use Aggregate  qw(
                    aggr_exists
                    aggr_create
                    aggr_add
                    aggr_status
                    aggr_show_space
		    aggr_offline
		    aggr_destroy
                );
    use HybridBuftree;
    use Dense::Agnostic::7mode::Aggr::Wafliron;
    use Dense::Agnostic::Constants;
    use Dense::Agnostic::7mode::Aggr::Fmap;


    sub _check_aggrnew_args  ($)
    {
        my  ($F, $A, $opts);
        my  ($emsg);
        
        $opts = $_[0];

        #Mandatory args
        if ( (! defined $opts->{Aggrname})    ||
             (! defined $opts->{Node})        ||
             (! defined $opts->{Ndisks}) )  {
            my  $emsg = "Aggr::new: need Aggrname, Node and Ndisks args";
            $Log->comment($emsg);
            die($emsg);
        }

        #Type checks
        $F  = $opts->{Node};
        $A  = $opts->{Aggr};
        if (ref($F) ne TYPE_7M_NODE) {
            $emsg = "Aggr::new: type mismatch: arg Node has bad type";
            $Log->comment($emsg);
            die($emsg);
        }

        #Other options: Raidtype, Snaplock, Indirect_type.
        #Replace the leading capital by a lowercase letter to conform to
        #Aggregate.pm's arg convention.
        if (defined $opts->{Raidtype}) {
            $opts->{raidtype} = delete $opts->{Raidtype};
            if (($opts->{raidtype} != 'raid4')  &&
                ($opts->{raidtype} != 'raid_dp')){
                $emsg = "Aggr::new: raidtype is neither raid4 nor raid_dp";
                $Log->comment($emsg);
                die($emsg);
            }
        }

        if (defined $opts->{Snaplock}) {
            $opts->{snaplock} = delete $opts->{Snaplock};
        }

        if (defined $opts->{IndirectType}) {
            $opts->{indirect_type} = delete $opts->{IndirectType};
            if (($opts->{indirect_type} != 32) &&
                ($opts->{indirect_type} != 64)){
                $emsg = "Aggr::new: Indirect type is neither 32 nor 64";
                $Log->comment($emsg);
                die($emsg);
            }
        }

    } #sub _check_aggrnew_args


    sub new (@)
    {
        my  ($A, $class, %opts);

        $class  = shift();
        %opts   = @_;

        eval {
            _check_aggrnew_args(\%opts);
        };
        if ($@) {
            die($@);
        }

        #Newbie note: here, "delete" on a hash performs a function similar
        #to "pop" on an array.
        $A  = {
            F       => delete $opts{Node},
            name    => delete $opts{Aggrname},
            ndisks  => delete $opts{Ndisks},
            reuse   => delete $opts{Reuse},
        };


        if (aggr_exists($A->{F}{fc}, $A->{name}) && $A->{reuse}) {
            #XXX: check to see if the existing aggr has as many disks as we
            #would like it to have. If not, try to add disks to the aggr. If
            #that doesn't succeed, bail out.
            goto OUT;
        }

        #Aggr does not already exist
        eval    {
            my $ret;
            #XXX check %opts for sanity
            $ret    = aggr_create($A->{F}{fc}, $A->{name}, $A->{ndisks}, %opts);
            if (! $ret) {
                die("Creation of aggr $A->{name} failed");
            }
        };
        if ($@) {
            die($@);
        }
OUT:
		$A->{params} = {
			fh			=> $A->{F}{fh},
			host		=> $A->{F}{name},
			filer		=> $A->{F}{fh},
			conn		=> $A->{F}{fc},
			aggregate	=> $A->{name},
			aggr_name	=> $A->{name},
		};
        bless($A, $class);
        return $A;

    } # sub new

	# can be used to restrict the aggr before doing aggr_copy
    # This routine can be called like this: $AGGR_DST->aggr_restrict(); where $AGGR_DST is the aggr object of the aggr which needs to be restricted.
	sub aggr_restrict()
	{
	  my $A = shift();
		my $ret = $A->{F}->say(command=>"aggr restrict $A->{name}");
	    if (!$ret)
		{
			die("aggr restrict failed");
		}	
		
	}
	# Can be used to online an aggr
 	# Call this method like this: $A->aggr_online();, where $A is the aggr object
	sub aggr_online()
	{
		my ($A, $F);
		$A = shift();
		my $ret = $A->{F}->say(command=>"aggr online $A->{name}");
		if(!$ret){ die("Aggr online failed");}
	}

	# Can be used to perform aggr copy function with two filer
	# This routine can be called like this: $A->aggr_copy(dst_aggr=>$AGGR_DST->{name}); where $A is source aggr object and $AGGR_DST is the aggr to where source aggr will be copied. Before  calling aggr_copy routine, destination aggr has to be restricted using $AGGR_DST->aggr_restrict(); routine.
	sub aggr_copy(@)
	{
		my ($F, $A, $V);
        $A = shift();
		my %opts=@_;
		my $dst_aggr = $opts{dst_aggr};
		my $dst_filer = $opts{dst_filer};
		eval{
		my $ret = $A->{F}->say(command=>"aggr copy start $A->{name} $dst_filer:$dst_aggr");
		};
		if ($@)
		{
			die("aggr copy failed with error:$@");
		}

	}

    sub show  (@)
    {
        my  ($A, $as, $options, @arr, $len);
        $A   = shift();
        $as  = aggr_status($A->{F}{fc}, $A->{name});
        print Dumper($as);
        @arr = split(/,/, $as->{'status'});
        $len = @arr;
        $options->{'raidtype'} = $arr[0];
        
        ## aggr_status() shows [raid_dp, aggr, parity uninit\'d!, 32-bit] 
        ## in case of an un-initialized disk. So 'raidstatus' will be shown at the index 2. 
        ##The other case, [raid_dp, aggr, 64-bit] for initialized disks. i
        ##So 'raidstatus' will be empty.  
        if ($len == 3) {
            $options->{'block-type'} = $arr[2]; 
            $options->{'raidstatus'} = "normal"; 
        }
        elsif ($len == 4) {
            $options->{'block-type'} = $arr[3]; 
            $options->{'raidstatus'} = $arr[2]; 
        }
        return $options;
        
    }

    sub status  (@)
    {
        my  ($A, $as);

        $A   = shift();

        $as     = aggr_status($A->{F}{fc}, $A->{name});
        return $as->state();
    }

    sub is_upgradable (@)
    {
	my ($A, $fc);
	$A = shift();
	$fc = $A->{F}{fc};

	my $ret = aggr_is_convertible( $fc, $A->{name});

	return $ret;
    }

    sub aggr_upgrade (@)
    {
	my ($A, $fc, $disable_scan, %opts);
	$A = shift();
	$fc = $A->{F}{fc};
	%opts = @_;
	my $start_scan = $opts{start_scan};

	if (aggr_convert($fc, $A->{name}, '0')) {
	    return 1;
	} else {
	    return 0;
	}
    }

    sub purge (@)
    {
	my ($A, $fc);
	$A = shift();
	$fc = $A->{F}{fc};

	aggr_offline($fc, $A->{name});
	my $ret = aggr_destroy($fc, $A->{name});
	$A->{F}->say(command=>"disk zero spares");

	return $ret;
    }
    sub AUTOLOAD (@)
	{
		our $AUTOLOAD;
		my  ($self, @req_params, @opt_params, %default_params);
		my  ($in_fn, %in_opts, %exec_opts, $exec_fn, @ret);

		$self       = shift();
		%in_opts    = @_;
		$in_fn      = $AUTOLOAD;
		%exec_opts  = %in_opts;

		$Log->enter();

		$in_fn =~ s/.*:://;

		$exec_fn = $Fmap->{$in_fn}{fname};
		if (!defined $exec_fn) {
			my $emsg = "UNDEFINED: 7mode::Volume::Autoload::$in_fn";
			$Log->comment($emsg);
			die($emsg);
		}

		if (! defined $Fmap->{$in_fn}{req_params}) {
			$Log->debug("7mode::Volume::Autoload: no mandatory args for $in_fn\[$exec_fn\]");
		} else {
			@req_params = @{$Fmap->{$in_fn}{req_params}};
			foreach my $param (@req_params) {
				if ((! $exec_opts{$param}) && (! defined $self->{params}{$param})) {
					my $errmsg = "7mode::Volume::Autoload: Can't resolve param\
								  $param of $in_fn [$exec_fn]";
					$Log->debug($errmsg);
					die($errmsg);
				}
				$exec_opts{$param} = $self->{params}{$param};
			}
		}

		if (! defined $Fmap->{$in_fn}{default_params}) {
			$Log->debug("7mode::Volume::Autoload: no default args for $in_fn\[$exec_fn\]");
		} else {
			%default_params = %{$Fmap->{$in_fn}{default_params}};
			foreach my $param (keys(%default_params)) {
				if (! defined $exec_opts{$param}) {
					$exec_opts{$param} = $default_params{$param};
				}
			}
		}

		if (! defined $Fmap->{$in_fn}{optional_params}) {
			$Log->debug("7mode::Volume::Autoload: no optional params for $in_fn\[$exec_fn\]")
		} else {
			@opt_params = @{$Fmap->{$in_fn}{optional_params}};
			foreach my $param (@opt_params) {
				if(! defined $exec_opts{$param}) {
					$exec_opts{$param} = $default_params{$param};
				}
			}
		}

		if (%exec_opts) {
			@ret = $exec_fn->(%exec_opts);
		} else {
			@ret = $exec_fn->();
		}

		$Log->exit();
		return @ret;

	}
    return 1;
} #package Dense::Agnostic::7mode::Aggr
