{ package   Dense::Agnostic::7mode::Volume;
    use strict;
    use warnings;

    use constant    TYPE_7M_NODE    => 'Dense::Agnostic::7mode::Node';
    use constant    TYPE_7M_AGGR    => 'Dense::Agnostic::7mode::Aggr';

    use Data::Dumper;
    use Tharn   qw($Log);
    use Volume  qw(
                    vol_create_flex
                    vol_destroy
                    vol_exists
                    vol_offline
                    vol_online 
                    vol_rename
                    vol_restrict
                    vol_purge
                    volume_set_option
                    volume_options_list_info
		    vol_status
                );
    use Params::Validate qw(validate SCALAR);
    use Dense::Agnostic::Constants;
    use Dense::Agnostic::7mode::Volume::Snapshot;
    use Dense::Agnostic::7mode::Volume::Fmap;
    use Dense::Agnostic::7mode::Volume::Sis;
    use Dense::Agnostic::7mode::Volume::Utils;
    use Dense::Agnostic::7mode::Volume::Qtree;
    use Dense::Agnostic::7mode::Volume::Lun;
    use HybridBuftree;


    sub _check_volnew_args  ($)
    {
        my  ($F, $A, $opts);
        my  ($emsg);
        
        $opts = $_[0];

        if ( (! defined $opts->{Volname}) ||
             (! defined $opts->{Aggr})    ||
             (! defined $opts->{Size}) )  {
            my  $emsg = "Volume::new: need Volname, Aggr and Volsize args";
            $Log->comment($emsg);
            die($emsg);
        }

        $A  = $opts->{Aggr};
        if (ref($A) ne TYPE_7M_AGGR) {
            $emsg = "Volume::new: type mismatch: arg Aggr has bad type";
            $Log->comment($emsg);
            die($emsg);
        }

    } #sub _check_volnew_args


    sub new (@)
    {
        my  ($V, $F, $A, $class, %opts);
        my  ($emsg);

        $class  = shift();
        %opts   = @_;

        eval {
            _check_volnew_args(\%opts);
        };
        if ($@) {
            die($@);
        }

        $V  = {
            A       => delete $opts{Aggr},
            name    => delete $opts{Volname},
            size    => delete $opts{Size},
        };
        $V->{F} = $V->{A}{F};
        $F  = $V->{F};

        eval    {
            my ($reuse, $ret);

            $reuse  = $opts{Reuse};
            delete $opts{Reuse};
            $Log->comment("Reuse option is : $reuse");

            if (vol_exists($F->{fc}, $V->{name})) {
		 if(! $reuse){
                     vol_purge($F->{fc}, $V->{name});
		     $ret = vol_create_flex($V->{F}{fc}, $V->{name}, $V->{A}{name}, $V->{size}, %opts);
		     if (! $ret) {
			 die("Creation of flexvol $V->{name} failed");
		     }
		 }
            } else {
            	$ret = vol_create_flex($V->{F}{fc}, $V->{name}, $V->{A}{name}, $V->{size}, %opts);
            	if (! $ret) {
                	die("Creation of flexvol $V->{name} failed");
            	}
	    }	
        };
        if ($@) {
            die($@);
        }

        #This is for the (dense) param resolution business.
        $V->{params}    = {
            fh          => $F->{fh},
            host        => $F->{name},
            filer       => $F->{fh},
            conn        => $F->{fc},
            aggregate   => $V->{A}{name},
            volume      => $V->{name},
            volName     => $V->{name},
            vol_name    => $V->{name},
            volname    => $V->{name},
            path_name   => "/vol/$V->{name}",
            path        => "/vol/$V->{name}",
            vol_path    => "/vol/$V->{name}",
        };

        bless($V, $class);
        return $V;

    } # sub new


    sub unmount (@)
    {
        my ($V);

        $V = shift();
        $Log->comment('Dummy function for Cmode parity of Agnostic scripts');
    }

    sub offline (@)
    {
        my  ($V, $ret);

        $V  = shift();
        $ret = vol_offline($V->{F}{fc}, $V->{name});
        if (! $ret) {
            die("Unable to offline flexvol $V->{name}");
        }
    }

    sub online (@)
    {
        my  ($V, $ret);

        $V  = shift();
        $ret = vol_online($V->{F}{fc}, $V->{name});
        if (! $ret) {
            die("Unable to online flexvol $V->{name}");
        }
    }

    sub destroy (@)
    {
        my  ($V, $ret);

        $V  = shift();
        $ret = vol_destroy($V->{F}{fc}, $V->{name});
        if (! $ret) {
            die("Unable to offline flexvol $V->{name}");
        }

        for my $k (keys(%$V)) {
            delete $V->{$k};
        }
    }

    sub restrict (@)
    {
    my  ($V, $ret);

    $V  = shift();
    $ret = vol_restrict($V->{F}{fc}, $V->{name});
    if (! $ret) {
        die("Unable to restrict flexvol $V->{name}");
          }
    }

   sub show  (@)
   {
     my  ($V, $ret, $options, @arr, $len);
     $V   = shift();
     $ret  = vol_status($V->{F}{fc}, $V->{name});
     @arr = split(/,/, $ret->{'status'});
     $len = @arr;
     $options->{'type'} = $arr[0]; 
     $options->{'volume-style'}   = $arr[1];
     
     ## vol_status() shows ['raid_dp, flex,64-bit, 
     ## Volume UUID: a7683926-8297-11e0-8235-123478563412',]
     ## for normal volume.
     ## The other case ,['raid_dp, flex, sis, 64-bit,
     ## Volume UUID: a7683926-8297-11e0-8235-123478563412',]
     ## for sis enabled volume.
     ## The other case, ['raid_dp, flex, wafl inconsistent , sis, 64-bit,
     ## Volume UUID: a7683926-8297-11e0-8235-123478563412',] 
     ## for wafl inconsistent volume.
     
     if ($len == 4) {
        $options->{'block-type'} = $arr[2];
        $options->{'is-inconsistent'} = "normal";
     }
     elsif ($len == 5) {
        $options->{'block-type'} = $arr[3];
        $options->{'is-sis-volume'} = $arr[2];
        $options->{'is-inconsistent'} = "normal";
     }
     elsif ($len == 6) {
        $options->{'is-inconsistent'} = $arr[2];
        $options->{'is-sis-volume'} = $arr[3];
        $options->{'block-type'} = $arr[4];
                
    }
       return $options;
  }
                                        
    sub purge (@)
    {
	    my ($V, $ret);
	    $V = shift();
	    
	    $V->offline();
	    $V->destroy(force => 1);
    }

    sub rename  (@)
    {
        my  ($V, $newname, $ret);

        $V  = shift();
        $newname = shift();
        $ret = vol_rename($V->{F}{fc}, $V->{name}, $newname);
        if (! $ret) {
            die("Unable to rename volume $V->{name}");
        }
        $V->{name} = $newname;
    }

    sub mkfile  (@)
    {
        my  ($V, %opts, $ret);
        my  ($filepath, $filecount, $options, $filesz, $stepsz);
        my  ($cmdstring);

        $V  = shift();
        %opts = @_;

        $filesz     = $opts{Size};
        $filepath   = $opts{Path};
        $filecount  = $opts{Count};
        $stepsz     = $opts{Stepsize};
        $options    = $opts{Option};
        
	#mkfile -x stream_name is not handled here
	$cmdstring = "mkfile $options $filesz $filepath $filecount"; 

        if (defined $stepsz) {
            $cmdstring = $cmdstring . " $stepsz";
        }

        eval {
            $V->{F}->say(command => $cmdstring);
        };

        if ($@) {
            die($@);
        }
        
        #Sync data
		if(!(defined($opts{Nosync})))
		{
			$V->{F}->say(command => "sync;sync;sync");
		}	

    }

    sub rm  (@)
    {
        my  ($V, $filepath);
        my  ($cmdstring);

        $V = shift();
        $filepath = shift();

        $cmdstring = "rm $filepath";
        eval {
            $V->{F}->say(command => $cmdstring);
        };
        if ($@) {
            die($@);
        }
    }

     sub upgrade (@)
	 {
    
	 my ($V, %opts, $error_msg, $start_scan,
		           $time, $endtime, $buftree_type, $buftree_status, $wait, $status);

	 $V = shift();

	 %opts = validate (@_,{ 'start_scan' => {type => SCALAR, default => 0},
				                             'wait'       => {type => SCALAR, default => 1},},);
					
     $start_scan = $opts{'start_scan'};
	 $wait = $opts{'wait'};

     if ($start_scan) {
	           $V->{F}->say(command=>'setflag wafl_64bit_upgrade_scan_disable 0');
	      } else {
	           $V->{F}->say(command=>'setflag wafl_64bit_upgrade_scan_disable 1');
		}
	
     eval{
       #upgrade the volume
       $status = $V->{F}->say(command => "vol 64bit-upgrade start $V->{name} -mode normal");
     };
     
	 if($status =~ /Started 64-bit upgrade on volume/){
		$Log->comment("64-bit upgrade started on the volume $V->{name}");
	 } else{
	    die "64-bit upgrade on $V->{name} Failed: $status";
	 }
	
	  if($wait){
         #check if upgrading is complete
         my $vol_hash = $V->{F}->say(command=> "vol_db iter name $V->{name}");
         $endtime = time() + 100;
         $buftree_type = $V->{F}->say(command=> "vol_db get buftree_type $vol_hash");
	     
		 if ($buftree_type =~ m/upgrading/) {
				             #If volume is upgrading,wait till upgrading is over
		           do{
			        $buftree_status = $V->{F}->say(command=> "vol_db get buftree_type $vol_hash");
			        if($buftree_status !~ m/upgrading/ && $buftree_status =~ /dual64|trad64/){
			              $Log->comment("Upgrade complete");
						  return 1;
					} elsif($buftree_status =~  m/upgrading/) {
					      sleep 10;
					} $time = time()+10;
				 } while($time < $endtime);
			  $Log->comment("Upgrading not complete");
			 return 0;
		}
        #If the upgrading happens very fast so that 'upgrading' status is not shown
		elsif($buftree_type !~ m/upgrading/ && $buftree_type =~  /dual64|trad64/) {
		         $Log->comment("Volume upgraded");
		         return 1;
		}
		else {
		         return 0;
		}
	}
	  return 1;
   }

    sub mixed_buftree (@)
    {
	my ($V, $fc, $file, %opts);
	$V = shift();
	$fc = $V->{F}{fc};
	$file = shift();;
	%opts = @_;
	my $vname = $V->{name};
	print "Volume name : $V->{name}, File : $file \n";
	if (file_hybridize($V->{F}{fc}, $vname, $file, %opts)) {
	    return 1;
	} else {
	    return 0;
	}
    }

    sub file_check_buftree (@) {

      my ($V, $file, %opts);

      $V = shift();
      $file = shift();
      %opts = @_;

      my $vname = $V->{name};

      my $ret = file_buftree_check($V->{F}{fc}, $vname, $file, %opts);

      return $ret;
	  }

    sub is_hybrid ()
    {
	my ($V, $fc);
	$V = shift();
	$fc = $V->{F}{fc};

	my $ret = volume_is_hybrid( $fc, $V->{name} );

	return $ret;
    }

    #############################################

    sub AUTOLOAD (@)
    {
        our $AUTOLOAD;
        my  ($self, @req_params, @opt_params, %default_params);
        my  ($in_fn, %in_opts, %exec_opts, $exec_fn);

        $self       = shift();
        %in_opts    = @_;
        $in_fn      = $AUTOLOAD;
        %exec_opts  = %in_opts;

        $Log->enter();

        $in_fn =~ s/.*:://; #Chop off the "Dense::Agnostic::7mode::" prefix.

        #Resolve in_fn to the function that we should actually invoke.
        $exec_fn = $Fmap->{$in_fn}{fname};
        if (!defined $exec_fn) {
            my $emsg = "UNDEFINED: 7mode::Volume::Autoload::$in_fn";
            $Log->comment($emsg);
            die($emsg);
        }

        #Resolve $exec_fn's mandatory params
        if (! defined $Fmap->{$in_fn}{req_params}) {
            $Log->debug("7mode::Volume::Autoload: no mandatory args for $in_fn\[$exec_fn\]");
        } else {
            @req_params = @{$Fmap->{$in_fn}{req_params}};
            foreach my $param (@req_params) {
                if ((! $exec_opts{$param}) && (! defined $self->{params}{$param})) {
                    my $errmsg = "7mode::Volume::Autoload: Can't resolve param\
                                  $param of $in_fn [$exec_fn]";
                    $Log->debug($errmsg);
                    die($errmsg);
                } 

                $exec_opts{$param} = $self->{params}{$param};
            } #foreach
        }

        #Resolve $exec_fn's default params
	if (! defined $Fmap->{$in_fn}{default_params}) {
            $Log->debug("7mode::Volume::Autoload: no default args for $in_fn\[$exec_fn\]");
        } else {
	    %default_params = %{$Fmap->{$in_fn}{default_params}};
            foreach my $param (keys(%default_params)) {
                if (! defined $exec_opts{$param}) {
                    $exec_opts{$param} = $default_params{$param};
                }
            }
        } #else

        #Resolve $exec_fn's optional params
        if (! defined $Fmap->{$in_fn}{optional_params}) {
            $Log->debug("7mode::Volume::Autoload: no optional params for $in_fn\[$exec_fn\]");
        } else {
            @opt_params = @{$Fmap->{$in_fn}{optional_params}};
            my  ($Pmap, $p, $v);
            $Pmap    = $Fmap->{$in_fn}{pmap};
            foreach my $op (@opt_params) {
                $p  = $Pmap->{$op}{pname};
                $v  = $exec_opts{$p};
                if ($exec_opts{$p}) {
                    $exec_opts{$op} = $Pmap->{$op}{$v} ? $Pmap->{$op}{$v} : $v;
                }

                if ($p ne $op) {
                    delete $exec_opts{$p};
                }
            }
        } #else

        #All our params should now be in %exec_opts. Go forth and execute our
        #little function.
        if (%exec_opts) {
            return $exec_fn->(%exec_opts);
        } else {
            return $exec_fn->();
        }

        $Log->exit();

    } #AUTOLOAD

    sub vol_wait_online (@) {
	my ($V, %opts, $sleep, $timeout, $interval);

	$V = shift();
	%opts = validate (@_, { timeout => { type=>SCALAR, default=>90, optional=>1 },
		       		time_gap => { type=>SCALAR, optional=>1, default=>5} });
	$timeout = $opts{timeout};
	$sleep = $opts{time_gap};
	my $conn = $V->{F}{fc};
	my $vol = $V->{name};
	$Log->comment("Connection object : $conn, Vol name : $vol");

	for( $interval=$timeout; $interval>0; $interval=-$sleep) {
	    if (vol_status($conn, $vol)->state() eq 'online') {
		return 1;
	    } else {
		sleep ($sleep);
	    }
	}
	die "Volume did not come online within specified interval\n";
    }

    return 1;
} #package Dense::Agnostic::7mode::Volume
