{ package   Dense::Agnostic::7mode::Volume;
    use strict; 
    use warnings;

    use Dense ();
    use Dense::Utils (); 
    use Dense::fiji ();
    use dense::performance::SORT ();
    use FS::Util::Utilities ();

    use vars qw($Fmap $Sis_Fmap $Utils_Fmap);

    $Sis_Fmap   = {

        sis_on => {
            fname               => \&Dense::dense_toggle,
            req_params          => [qw(host path_name)],
            default_params      => { state => "on" },    
            return_value        => undef,
        },

        sis_off => {
            fname               => \&Dense::dense_toggle,
            req_params          => [qw(host path_name)],
            default_params      => { state => "off" },    
            return_value        => undef,
        },

        sis_start => {
            fname               => \&Dense::dense_start,
            req_params          => [qw( host path_name )],
            default_params      => { nosync => 0 },    
            optional_params     => [qw( scan reuse_ckpt del_ckpt )],
            return_value        => undef,
        },

        sis_check => {
            fname               => \&Dense::dense_check,
            req_params          => [qw(host path_name)],
            default_params      => undef,
            return_value        => undef,
        },

        sis_config => {
            fname               => \&Dense::dense_config,
            req_params          => [qw(host path_name)],
            default_params      => undef,
            return_value        => undef,
        },


        sis_stop => {
            fname               => \&Dense::dense_stop,
            req_params          => [qw(host path_name)],
            default_params      => undef,
            return_value        => undef,
        },

        sis_wait_for => {
            fname               => \&Dense::dense_wait_for,
            req_params          => [qw( host path_name )],
            default_params      => { status=>"IDLE", time_out=>'3600' },    
            return_value        => undef,
        },

        sis_check_records => {
            fname               => \&Dense::parse_dense_check,
            req_params          => [qw( host path_name )],
            default_params      => undef,
            return_value        => undef,
        },

        sis_undo => {
            fname               => \&Dense::dense_undo,
            req_params          => [qw(host path_name)],
            default_params      => undef,
            return_value        => undef,
        },

	sis_stat => {
	    fname               => \&Dense::dense_stat,
	    req_params          => [qw( host path )],
	    optional_params     => [qw( block long verbose diag_mode)],
	    return_value        => "HASH",
	},

	is_dense_enabled => {
	    fname               => \&Dense::Utils::is_dense_enabled,
	    req_params          => [qw(host vol_path)],
	    return_value        => "SCALAR",
	},

	sis_config => {
	    fname		=> \&Dense::dense_config,
	    req_params		=> [qw( host path_name)],
	    default_params	=> {test_mode=>'1'},
	    return_value	=> "HASH",
	},

	sis_options_set => {
	    fname		=> \&Dense::Utils::set_sis_options,
	    req_params		=> [qw( conn )],
	    default_params	=> {},
	    optional_params     => [qw(value option_name)],
	    return_value	=> "HASH",
	},
	sis_checkpoint => {
	    fname		=> \&Dense::dense_create_checkpoint,
	    req_params		=> [qw( host path_name )],
	    default_params	=> { stage=>"DENSE_DEDUPING_P2" },
	    optional_params     => [qw(checkpoint_timeout sleep)],
	    return_value	=> "HASH",
	},
	 sis_revert_to => {
            fname               => \&Dense::dense_revert_to,
            req_params          => [qw(host path_name)],
            default_params      => { cleanup =>0 },
            return_value        => undef,
        },

    sis_get_maxref_cnt => {
        fname               => \&Dense::Utils::get_max_supported_ref_count,
        req_params          => undef,
        optional_params     => undef,
        return_value        => undef,
    }
    };

    $Utils_Fmap = {

        sis_stage_fiji_set => {
            fname                   =>  \&Dense::fiji::fiji_dense_stage,
            req_params              => [qw( host vol_name )],
            optional_params         => [qw( dense_stage )], 
            return_value            => undef,
        },

        set_overlap_request     => {
            fname                   =>  \&Dense::Utils::set_overlap_requests,
            req_params              => [qw( conn )],
            optional_params         => undef,
            return_value            => undef,
        },

	wafl_stat_counter => {
	    fname		=> \&dense::performance::SORT::get_wafl_stat_blkshare_counters,
	    req_params		=> [qw(conn host)],
	    optional_params	=> [qw(flags)],
	    return_value	=> "ARRAYREF",
	},

        wafl_showvbns     => {
            fname                   =>  \&Dense::Utils::dense_parse_showvbns,
            req_params              => [qw( conn )],
            optional_params         => undef,
            return_value            => undef,
        },
        
        stats_show     => {
            fname                   =>  \&Dense::Utils::dense_get_vol_stats,
            req_params              => [qw( conn volname)],
            return_value            => undef,
        },

        dump_block       => {
            fname                   =>  \&FS::Util::Utilities::dumpblock,
            req_params              => [qw( conn )],
            optional_params         => undef,
            return_value            => undef,
        },
       
        syslog_rotate     => {
            fname                   =>  \&Dense::Utils::dense_syslog_rotate,
            req_params              => [qw( conn )],
            optional_params         => undef,
            return_value            => undef,
        },

   
        get_inode_num     => {
            fname                   =>  \&Dense::Utils::dense_get_inode_num,
            req_params              => [qw( conn )],
            optional_params         => undef,
            return_value            => "SCALAR",
        },

	wafl_stat_zero => {
	    fname		=> \&dense::performance::SORT::make_wafl_stat_zero,
	    req_params		=> [qw( conn )],
	    return_value	=> undef,
	},

	set_wp1m_delay_fiji => {
	    fname                   =>  \&Dense::fiji::set_wp1m_delay_fiji,
	    req_params              => [qw( host vol_name)],
	    optional_params         => [qw( msg_type sleep)],
	    return_value            => undef,
	},

	set_wafl_scan_speed => {
	    fname                   =>  \&FS::Util::Utilities::set_wafl_scan_speed,
	    req_params              => [qw( conn )],
	    optional_params         => [qw( speed )],
	    return_value            => undef,
	},

	get_wafl_scan_speed => {
	    fname                   =>  \&FS::Util::Utilities::get_wafl_scan_speed,
	    req_params              => [qw( conn )],
	    optional_params         => undef,
	    return_value            => undef,
	},
	
    };

    $Fmap = {(@{[%$Sis_Fmap]}, @{[%$Utils_Fmap]})};

    return (1);
}

