#$Id: //depot/prod/test/main/lib/Dense/Agnostic/7mode/Volume/Lun.pm#1 $
{ package Dense::Agnostic::7mode::Volume;

    use Tharn qw($Log);
    use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
    use Data::Dumper qw(Dumper);
    use Dense::SAN;
    use blocks_filer qw(filer_lun_create filer_lun_exists);

=head2 lun_create()

Description : Function to create the Lun and setup iscsi 

=over 4

=item lun 

Name of the Lun , mandatory parameter

=item size

Size of the Lun , default value is 4m

=item ostype

Type of Lun (solaris, windows, hpux, aix, linux, netware, vmware, windows_gpt, windows_2008, xen, hyper_v, solaris_efi and openvms)

=item iscsi_setup

This param will determine whether to setup iscsi or not

=item iname

Igroup name

=item initiator

Initiator name in the Client

=item mount

This param will determine whther to mount the lun or not

=item client

Client name to mount the lun

=over 4

Return Value

Return with the client mount point when "mount" param is set

=back

=cut


	sub lun_create(@)
    {
        my ($V, %opts, $filer_conn, $volname, $lun_path, $mountdir, $disk);
        $V = shift();

        %opts = validate (@_, { 
                                lun         => { type=>SCALAR},
                                size        => { type=>SCALAR, optional=>1, default=>"4m"},
                                ostype      => { type=>SCALAR, optional=>1, default=>"linux"},
                                iscsi_setup => { type=>SCALAR, optional=>1, default=>0},
                                iname       => { type=>SCALAR, optional=>1, default=>"dense_igroup"},
                                initiator   => { type=>SCALAR, optional=>1},
                                mount       => { type=>SCALAR, optional=>1, default=>0},
                                client      => { type=>SCALAR, optional=>1, default=>"localhost"},
                              });

        $filer_conn = $V->{params}->{conn}; 
        $volname = $V->{name};
        $lun_path = "/vol/".$volname."/".$opts{lun};

        if (filer_lun_exists( $filer_conn, $lun_path )) {
            filer_lun_destroy($filer_conn, $lun_path );
        }

        filer_lun_create($filer_conn, $lun_path, $opts{size}, $opts{ostype});

        if ($opts{iscsi_setup}) {
            dense_filer_iscsi_setup (conn           =>$filer_conn,
                                     lun_name       =>$opts{lun},
                                     lun_group      =>$opts{iname},
                                     lun_platform   =>$opts{ostype},
                                     vol_name       =>$volname,
                                     iname          =>$opts{initiator});

        }

        if ($opts{mount}) {
            ($mountdir, $disk) = dense_lun_mount(filer    => $V->{F}->{name},
                                                 lun_name => $opts{lun},
                                                 vol_name => $volname,
                                                 client   => $opts{client});
            return $mountdir;
        }

    }
		    
    return 1;
}
