{ package   Dense::Agnostic::7mode::Volume;
    use strict;
    use warnings;

    use Tharn   qw($Log);

    use Params::Validate qw(validate SCALAR);
    use Data::Dumper;
    use filer;
    use Dense::Fast_Clones qw(dense_fast_clone_start dense_fast_clone_wait_till_complete);
    use Dense::fiji qw(fiji_clear);
    use filer qw(filer_cli_test_say);
    use Tharn;
    use Dense qw(dense_wait_for_field);

=head get_savings()
Executes "df -s" command and returns back used and saved.

$savings = $V->get_savings();
$used = $savings->{used};
$saved = $savings->{saved};

=cut	
    sub get_savings (@)
    {
        my  ($V, $ret, %savings, @lines);

        $V  = shift();
   
     	$ret = $V->{F}->say(command => "sync;sync;sync");
       	$ret = $V->{F}->say(command => "df -S $V->{name}");
        @lines = split(/\n/,$ret);

	if ($lines[1] =~ /^\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s*/) {
               	%savings = (
           		used    =>  $2,
                    	saved   =>  $5,
                    	dedupe_saved_percentage => $6,
                        saved_compress => $7,
			vbn_zero => $9,
                	);
        } elsif($lines[1] =~ /^\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s*/) {
		%savings = (
                        used    =>  $2,
                        saved   =>  $5,
                        dedupe_saved_percentage => $6,
                        saved_compress => $7,
                        );
       }
	
        return \%savings;
    }

=head2 dense_sort_gen()
Executes sis sort gen for bucket sort or merge sort.

$V->dense_sort_gen(bucket=>'true', num_records_KB=>$numrec) - bucket sort

$V->dense_sort_gen(num_records_KB=>$numrec) - merge sort

=cut

sub dense_sort_gen (%) {
        my  ($V, $ret, %opts, $cmd);

        $V  = shift();
        (%opts) = validate (@_, {
                                 'bucket' => { type=>SCALAR },
                                 'num_records_KB' => { type=>SCALAR },
                                }
                          );
        if ($opts{bucket} =~ /true/i) {
		$cmd = "sis sort gen bucket $opts{num_records_KB} $V->{params}{path}";
	} else {
		$cmd = "sis sort gen $opts{num_records_KB} $V->{params}{path}";
	}

	eval {
            $ret = $V->{F}->say(command=>$cmd, timeout=>900);
	};
	if( $@ ){
	    die ($@);
	}
	print "Output of sis sort run: $ret \n";
	my @lines = split(/\n/, $ret);
	print "Array output : @lines\n";
}
	
=head2 dense_sort_run()
=cut

sub dense_sort_run (%) {
        my  ($V, $ret, %opts, $cmd, %sort_counters);

        $V  = shift();
        (%opts) = validate (@_, {
                                 'bucket' => { type=>SCALAR },
                                 'num_records_KB' => { type=>SCALAR },
                                 'num_workers' => { type=>SCALAR, optional=>1, default=>2 },
				 'input_file' => { type=>SCALAR, optional=>1 },
				 'output_file' => { type=>SCALAR },
                                }
                          );
        if ($opts{bucket} =~ /true/i) {
		$cmd = "sis sort run bucket -n $opts{num_records_KB} -w $opts{num_workers} -i $V->{params}{path} -o $opts{output_file}";
	} else {
		$cmd = "sis sort run -n $opts{num_records_KB} -w $opts{num_workers} -i $opts{input_file} -o $opts{output_file}";
	}
	eval {
            $ret = $V->{F}->say(command=>$cmd, timeout=>900);
	};
	if( $@ ){
	    die ($@);
	}

	print "Output of sis sort run: $ret \n";
	my @lines = split(/\n/, $ret);
	print "Array output : @lines\n";
	foreach my $line (@lines) {
	    if( $line =~ /^\s*(\S.+):\s+(.+?)\s*$/ ){
		my $key = $1;
		my $value = $2;
		$key = Dense::remove_leading_trailing_spaces($key);
		$value = Dense::remove_leading_trailing_spaces($value);
		$sort_counters{$key} = $value;
	    }
	}

	return \%sort_counters;
	
}

=head2 get_sis_stat_global ()

This function execute "sis stat -g" and parses the output to returns a hash

=over 4

Failure

Returns if the output is not correct/in desired format

This function returns a Hash of all the fields of the output 

=back

=cut

    sub get_sis_stat_global (@)
    {
	my  ($V, $cmd, $ret);
	$V = shift();
	my %stat_info = ();
	my %op_status = ();
	my %status_map = ();

	$cmd = "sis stat -g";
			                               
	$ret = $V->{F}->say(command => $cmd);
	my @process = filer_cli_split($ret);
	foreach my $line (@process) {
	    if ($line =~ /Change log buffer stats/) {
	         next;
	    }
	    if ( $line ne "") {
	        if ($line =~ /^\s*(\S.+)\s+(.+?)\s*$/) {
		    my $field = $1;
		    my $data = $2;
		    $field =~ s/://g;
		    $field = Dense::remove_leading_trailing_spaces($field);

		    $stat_info{$field} = $data;
		 } else {
	            die "Unparsable line in status output: $line\n";
		 }
	    }
	}

	my $stat = \%stat_info;

	%status_map = (
	        "max-dedupe-proc"               => "Max SIS Ops",
		"max-share-blocks"              => "Max Share Blocks",
		"pending-dedupe-proc"           => "Pending SIS Ops",
		"running-dedupe-proc"           => "Running SIS Ops",
		"total-config-volumes"          => "Total Configured",
		"num-succeeded-ops"          	=> "Succeeded Ops",
		"num-started-ops"         	=> "Started Ops",
		"num-failed-ops"		=> "Failed Ops",
		"num-deferred-ops"		=> "Deferred Ops",
		"num-stopped-ops"		=> "Stopped Ops",
		"dropped-clogs"			=> "Dropped Change Logs",
		"generated-clogs"		=> "Change Log Generated",
		"flushed-clogs"			=> "Change Log Flushed",
		"pending-clogs"			=> "Change Log Pending",
		"BCMP in Exempt"		=> "BCMP in Exempt",
	     );
	%op_status = map { $_ => $stat->{$status_map{$_}}} keys %status_map;

	print Dumper (\%op_status);
	return (\%op_status);
    }

=head2 verify_bcmp_counter ()

=cut

    sub verify_bcmp_counter (@)
    {
       my ($V, $ret, %opts);
       $V = shift();
	
       %opts = validate (@_, {initial => { type=>SCALAR, optional=>1},
                              expected_value => { type=>SCALAR, optional=>1} } );
       my $exp_value = $opts{expected_value};
       my $init_bcmp = $opts{initial};
       my $sis_stat = $V->get_sis_stat_global();
       my $curr_bcmp = $sis_stat->{'BCMP in Exempt'};

       if ( ($curr_bcmp - $init_bcmp) != $exp_value) {
           die "Bcmp_in_Exempt counter does not have expected value. Obtained : ($curr_bcmp - $init_bcmp) Expected : $exp_value";
       } else {
           return 1;
       }
    }

=head2 vol_size ()

Function to increase or decrease the volume size. if not passed any argument , returns the size of the volume

=over 4

=item size

size with which we want to increase/decrease the volume

=over 4

Failure

Return with an error message if command is not able to execute correctly

=back

=cut

    sub vol_size (@)
    {
	my  ($V, $cmd, $ret, %opts);
	$V = shift();

	%opts = validate (@_, {size => { type=>SCALAR, optional=>1}} );
	my $size_vol = qr{\d+\S\D};

	my ($size) = $opts{size};
	if ( $opts{size} ){
	    $cmd = "vol size $V->{name}";
	    $cmd = $cmd . " $size";
	    $ret = $V->{F}->say(command => $cmd);
	    if ($ret !~ /vol size: Flexible volume '$V->{name}' size set to/s) {
		die ("vol size did not executed correctly");
	    }
	} else {
	    $cmd = "vol size $V->{name}";
	    $ret = $V->{F}->say(command => $cmd);
	    if($ret !~ /vol\s+size:\s+Flexible\s+volume\s+'$V->{name}'\s+has\s+size\s+($size_vol)/) {
		die ("vol size did not executed correctly");
	    }
	    return ($1);
	}
    }

    sub clone_start (@)
    {
        my  ($V, $ret, %opts, %options, $clone_id);

        $V  = shift();
        (%opts) = validate (@_, {
                                 'source-path' => { type=>SCALAR },
                                 'destination-path' => { type=>SCALAR, optional=>1 },
                                 range => { type=>SCALAR, optional=>1 },
                                 snapshot => {type=>SCALAR, optional => 1},
                                }
                          );

         %options = (source => "/vol/".$V->{name}."/".$opts{'source-path'});

         $options{clone} = "/vol/".$V->{name}."/".$opts{'destination-path'} if($opts{'destination-path'});
         $options{range} = $opts{range} if($opts{range});
         $options{snapshot} =$opts{snapshot} if($opts{snapshot});
         $clone_id = dense_fast_clone_start(host => $V->{params}{host}, %options);
         return $clone_id;
    }


    sub clone_wait_for_complete(@)
    { 
        my ($V, %opts, %options);
        $V = shift();
        %opts = validate (@_, { 
                                file => { type => SCALAR },
                                start_fbn => { type => SCALAR, optional => 1, },
                                fbn_cnt => { type => SCALAR, optional => 1, },
                                timeout => { type => SCALAR, optional => 1, },
                                }
                        );
        
        $opts{host}      = $V->{params}{host};
        $opts{volume}    = $V->{name};

        dense_fast_clone_wait_till_complete(%opts); 

    }                   

    
=head2 $V->sis_stat(block=>0, long=>1, verbose=>1)

Function to collect sis stat of a particular volume for the options specified in the function call

=over 4

=item block

=head2 get_sis_status ()

The output fields of sis status are different in 7-mode and C-mode . 
This function maps the 7-mode output fields to c-mode fields.

=over 4

Failure

Return a hash of the mapped fields 

=back

=cut
	
    sub get_sis_status (@)
    {
	my ($V, $ret, %tmp_status, %status_map , %op_status);
	$V = shift();

	%tmp_status = Dense::dense_status(host => $V->{params}{host}, path => $V->{params}{path}, long => 1);
	my $status = $tmp_status{$V->{params}{path}};

    %status_map = (
        "op-status"                    => "Status",
        "state"                        => "State",
        "progress"                     => "Progress",
        "last-op-error"                => "Last Operation Error",
        "last-op-size"                 => "Last Operation Size",
        "verify-suspend-count"         => "Verify Suspend Count",
        "fingerprints-gathered"        => "Fingerprints Gathered",
        "fingerprints-sorted"          => "Fingerprints Sorted",
        "background-compression"       => "Background Compression",
        "inline-compression"           => "Inline Compression",
        "min-blocks-shared"            => "Minimum Blocks Shared",
        "blks-skipped-sharing"         => "Blocks Skipped Sharing",
        "blocks-deduplicated"          => "Blocks De-duplicated",
        "duplicate-blocks-found"       => "Duplicate Blocks Found",
        "fingerprints-deleted"         => "Fingerprints Deleted",
        "stale-fingerprint-percentage" => "Stale Fingerprints",
        "checkpoint-stage"             => "Checkpoint Stage",
        "checkpoint-sub-stage"         => "Checkpoint Sub-stage",
        "checkpoint-progress"          => "Checkpoint Progress",
        "queued-job"                   => "Queued Jobs",
        "stage"                        => "Stage",
        "changelog-usage"              => "Change Log Usage",
    );

	%op_status = map { $_ => $status->{$status_map{$_}}} keys %status_map;
	return \%op_status;
    }

=item long

returns sis stat output with options "-l" (sis stat -l /vol/volname)

=item verbose

returns sis stat output with options "-v" (sis stat -v /vol/volname)

=over 4

Failure

Return with an error message if command is not able to execute correctly

=back

=cut
    sub sis_stat (@)
    {
        my ($V, %opts, $stat, %tmp_stat, %stat_map , %op_stat);

        $V = shift();
        %opts = validate (@_, {
				block => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ },
				long => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ },
				verbose => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|1)$/ },
                                }
                          );

	my ($long) = $opts{long};
        my ($verbose) = $opts{verbose};
	my ($block) = $opts{block};	

        %tmp_stat = Dense::dense_stat(host => $V->{params}{host}, path => $V->{params}{path}, long => $long, verbose=>$verbose, block=>$block);

	print Dumper(\%tmp_stat);

        $stat = $tmp_stat{$V->{"params"}{"path"}};

	print Dumper($stat);

        %stat_map = (
                "unflushed-clogs" => "Un-Flushed Change Logs",
                "space-saved"     => "Saving",
                "saved-percentage"  => "Shared",
		"num-p1-merges" => "P1Merges",
		"num-merged-bins" => "Merged Bins",
		"used-0-splits"	=> "Used 0 Splits",
		"used-1-split" => "Used 1 Split",
		"used-2-splits" => "Used 2 Splits",
		"used-1-pass" => "Used 1 Pass", 
		"used-2-passes" => "Used 2 Passes",
		"used-3-passes" => "Used 3 Passes",
		"records-sorted" => "Records Sorted",
		"fp-mismatch-overwrites" => "Mismatch By Overwrite",
		"mismatch-data-blks" => "Mismatched Data",
        "skip-share-blocks-upper" =>"Skip Share Blocks Upper",
        "skip-share-blocks-delta" =>"Skip Share Blocks Delta",
        "same-data-blocks" =>"Same Data",
	"num-sorted-blocks" =>"Total Sorted Blocks",
        "num-stale-recipients" => "Staled Recipient",
        "num-stale-donors" => "Staled Donor",
        "num-mismatch-fps"              => "FP False Match",
        );

        %op_stat = map { $_ => $stat->{$stat_map{$_}}} keys %stat_map;
	print Dumper(\%op_stat);

	return \%op_stat;
   }

=head2 $V->sis_verify_changelog(records=>$no_of_fps)

Function to check whether one of the changelog directories is populated with required number of FP records

=over 4

=item records

Number of FP records expected in changelog

=item timeout

Duration to wait for FP records to get flushed to changelog

Failure
fails when changelog is not poulated with expected number of FP records within the given timeout

=cut
   sub sis_verify_changelog (@)
   {
	my ($V, %opts, @clog, $clog_entry);

	$V = shift();
        %opts = validate (@_, { records => { type=>SCALAR },
				timeout => { type=>SCALAR, default=>'90' },
			      }
			);
	my $timeout = time() + $opts{timeout};
	my $records = $opts{records};

	while( time() < $timeout ) {
	    @clog = $V->sis_check_records();
	    $clog_entry = $clog[0]->{'active'}->{cl_1};
	    if ($clog_entry >= $records) {
		return;
	    }
	    sleep 5;
	}

	die "Changelog file is not populated with all written records";
   }

=head2 $V->set_fiji(fiji_stage=>$fiji_stage,return=>$return)

	Function to set fiji

=cut
    sub set_fiji (@)
    {
	my ($V, %opts, $cmd);
	$V = shift();
	%opts = validate(@_, { fiji_stage=>{type=>SCALAR, optional => 1 },
			       stage=>{type=>SCALAR, optional => 1 },
			       name=>{type=>SCALAR, optional => 1},
			       return=>{type=>SCALAR},
                   repeat => { type=>SCALAR, optional=>1, default=>1 },
			       } );
	my $fiji_stage = $opts{fiji_stage};
	my $stage = $opts{stage};
        my $name= $opts{name};
	my $ret_value = $opts{return};
	my $vol_name = $V->{name};
    	if ($stage) {
            $cmd = "fiji filter \'(repeat $opts{repeat} ($name (&& (eq \@sleep_stage $stage) (eq \@vol $vol_name)) (return $ret_value)))\'";
        } else {
	    $cmd = "fiji filter \"(repeat $opts{repeat} ($fiji_stage (eq \@vol $vol_name) (return $ret_value)))\"";
        }
	eval {
            my $ret = $V->{F}->say(command=>$cmd, timeout=>900);
	};
	if( $@ ){
	    die ($@);
	}
	return 1;
    }
	

    sub sis_status (@)
    {
        my ($V, $ret, %tmp_status, %status_map , %op_status);
        
       $V = shift(); 
        %tmp_status = Dense::dense_status(host => $V->{params}{host}, path => $V->{params}{path}, long => 1);
        
        my $status = $tmp_status{$V->{params}{path}};

        %status_map = (
                "op-status"                     => "Status",
                "state"                         => "State",
                "stage"                         => "Stage",
                "progress"                      => "Progress",
                "last-op-state"                 => "Last Operation State",
                "last-op-error"                 => "Last Operation Error",
                "last-op-size"                  => "Last Operation Size",
                "checkpoint-progress"           => "Checkpoint Progress",
                "checkpoint-stage"              => "Checkpoint Stage",
                "checkpoint-sub-stage"          => "Checkpoint Substage",
                "checkpoint-progress"           => "Checkpoint Progress",
                "logical-data-size"             => "Logical Data",
                "stale-fingerprint-percentage"  => "Stale Fingerprints", 
                "verify-trigger-rate"           => "Verify Trigger Rate",
                "verify-suspend-count"          => "Verify Suspend Count",
               );

        %op_status = map { $_ => $status->{$status_map{$_}}} keys %status_map;
        
        return \%op_status;
        
    }

   sub fiji_clear (@)
   {
	my $V;
	
	$V = shift();
        my (%opts) = validate (@_, { slot => { type=>SCALAR, optional=>1, default=>'all' }});

	my $slot = $opts{slot};
	$V->{F}->say(command => "fiji clear $slot");

	return 1;
   }
   sub set_ric_fiji (@) {
        my $V;
        $V = shift();
        my $conn = $V->{F}{fc};
        my $host = $V->{F}{name};
        my $path = $V->{params}{path};
	    $V->sis_start();
        # To ensure CP is elongated when block sharing is active. wait for Saving_sharing phase 
        dense_wait_for_field(host=>$host, path_name=>$path, wait_on=>"Checkpoint Stage", wait_for=>"Saving_sharing", time_out=>"120");
        filer_cli_test_say( $conn, "dbg level dense=0;fiji filter \"(wafl.wp1m.delay.mdp (&& (eq \@type WAFL_CLEAN_ALLOC_FILE) (eq \@vol $V->{name}) (eq \@fileid 73)) (return 200))\";mkfile 4k /vol/$V->{name}/file2;sync",'720');
        $Log->comment("Done with fiji set for RIC");
        return 1;
   }	

 sub sis_reset (@) {

        my  ($V,%opts,$vol_path,$metafile,$location);
        my  ($cmdstring);
        $V = shift;

        %opts = validate (@_, {
                                vol_path => { type=>SCALAR},
                                metafile => { type=>SCALAR},
                                location => { type=>SCALAR},
                                }
                          );

        $vol_path = $opts{vol_path};
        $metafile = $opts{metafile};
        $location = $opts{location};

        $cmdstring = "sis reset $vol_path $metafile $location";
        eval {
            $V->{F}->say(command => $cmdstring);
        };
        if ($@) {
            die($@);
        }
   }

    sub sis_start_schedule(@) {
        my ($V,%opts);
        $V = shift;
        
        $V->sis_options_set(option_name=>'sis.minutely_schedule', value=>'on');
        $V->sis_config(schedule_string => 'sun-sat@0-23@~0-59/1');
        $V->sis_wait_for(status=>"ACTIVE");
        $V->sis_config(schedule_string => 'sun-sat@0');
        $V->sis_options_set(option_name=>'sis.minutely_schedule', value=>'off');
        
        return 1;
    }
    
    sub sis_ls(@) {
        my ($V,%opts);
        my %metafile;
        $V = shift;

        my $volpath="/vol/".$V->{name};

        my ($return_text, $result_code) = $V->{F}->say(command =>"sis ls $volpath");
        return undef if(!$return_text);

        my @lines=split(/\n/, $return_text);
        shift(@lines);
        foreach(@lines) {
            next if(!/\S/); #Check if it is an empty line
            if(/Total Blocks Used By $V->{name} sis Metafiles on the aggregate/) {
                $metafile{TotalBlocksUsed}=(split)[-1];
            } else {
                my ($name, $size, $blocks)=split;
                $metafile{$name}{size}=$size;
                $metafile{$name}{blocks}=$blocks;
            }
        }
        return(\%metafile);
    }

    return 1;
}

