{ package Dense::Agnostic::7mode::Volume;
    use Tharn qw(
	    logcomment
    );
    use Params::Validate qw(
	    validate
	    OBJECT
	    SCALAR
	    ARRAYREF
	    HASHREF
	    UNDEF
    );
    use SnapCmds qw(
            snapshot_create
            snapshot_delete
            snapshot_exists
            snapshot_list_b
            snapshot_rename
	    snapsched_volume
            snaprestore_file
            snaprestore_volume
    );
    use wafl_utils qw(snap_status);
    
    sub snap_status(%)
    {
        my ($V, @status);
        $V = shift();
        @status = wafl_utils::snap_status($V->{F}{fc}, $V->{name});
        return @status;
    }

    sub snap_create (%)
    {
	my ($V, $snapname, %opts, $ret);

        $V  = shift();
	%opts = validate (@_, { snapshot => { type=>SCALAR } });

	$snapname = $opts{snapshot};

        $ret = snapshot_create($V->{F}{fc}, $V->{name}, $snapname);
        if (! $ret) {
            die("snap create of snapshot $snapname failed on $V->{name}");
        }
    }

    sub nosnap (%)
    {
	my  ($V,$ret);

	$V = shift();
	my @schedule = ("0","0","0");

	$ret = snapsched_volume($V->{F}{fc}, $V->{name}, @schedule);
	if (! $ret) {
	    die("snap sched failed on $V->{name}");
	}
    }

    sub snap_delete (%)
    {
        my  ($V, $snapname, %opts, $ret);

	$V = shift();
	%opts = validate (@_, { Snapname => { type=>SCALAR },
			        Options => { type=>SCALAR, optional=>1 },
			        Zapi => { type=>SCALAR, optional=>1} }
			  );
	
	$snapname = $opts{Snapname};
	$options = $opts{Options};
	$use_zapi = $opts{Zapi};

        $ret = snapshot_delete($V->{F}{fc}, $V->{name}, $snapname, $options, 
			$use_zapi);
        if (! $ret) {
            die("snap delete of snapshot $snapname failed on $V->{name}");
        }
    }

    sub snap_list_b (%)
    {
	my  ($V, @ret);

        $V = shift();

	@ret = snapshot_list_b($V->{F}{fc}, $V->{name});
	return @ret;
    }

    sub snap_rename (%)
    {
        my  ($V, $old_snapname, $new_snapname, $ret);

        $V = shift();
	%opts = validate (@_, { Oldname => { type=>SCALAR },
			        Newname => { type=>SCALAR } }
			 );

	$old_snapname = $opts{Oldname};
	$new_snapname = $opts{Newname};
	
        $ret = snapshot_rename($V->{F}{fc}, $V->{name}, $old_snapname, 
                $new_snapname);
        if (! $ret) {
            die("snap rename of snapshot $old_snapname to $new_snapname
                    failed on $V->{name}");
        }
    }

    sub snap_restore (@)
    {
        my  ($V, $snapname, %opts, $ret);

        $V  = shift();
	%opts = validate (@_, { snapshot => { type=>SCALAR }} );
	
        $snapname = $opts{snapshot};
        
	$ret = snaprestore_volume($V->{F}{fc}, $V->{name}, $snapname);
        if (! $ret) {
            die("snap restore of volume $V->{name} from snapshot $snapname
                    failed");
        }
    }

    sub sfsr (%)
    {
        my  ($V, $snapname, $filepath, $restorepath, %opts, $ret);

        $V  = shift();
        %opts = validate (@_, { snapshot => { type=>SCALAR },
			 	path => { type=>SCALAR },
				restore-path => { type=>SCALAR, optional=>1 } }
			  );

        $snapname = $opts{snapshot};
        $filepath = $opts{path};
        $restorepath = $opts{restore-path};

        $ret = snaprestore_file($V->{F}{fc}, $filepath, $snapname,
                restore_as_path=>$restorepath);
        if (! $ret) {
            die("sfsr of $filepath to $restorepath on $V->{name}, from Snapshot
                    $snapname failed");
        }
    }

    return 1;
}
