#$Id: //depot/prod/test/Rsierranevada/lib/Dense/Agnostic/7mode/Volume/Utils.pm#2 $
{ package Dense::Agnostic::7mode::Volume;

    use Tharn qw($Log);
    use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
    use Data::Dumper qw(Dumper);
    use SnapVault qw(:xfer);
	use Volume::Clone qw(:all);
    use filecheck;
    use Dense::Utils qw(check_inode_dense_flag);
        
    sub is_dense_volume (@)
    {
	my ($vol, $ret);

        $vol = shift;

        $ret = $vol->{F}->say(command=>"vol status $vol->{name}");
        return ($ret =~ /sis/);
    }

    sub wafl_punch_hole (@)
    {
        my ($vol, $path, %opts, $cmd, $ret);
        $vol = shift;
        (%opts) = validate(@_, {
                                file   => { type=>SCALAR },
                                fbn    => { type=>SCALAR },
                                count  => { type=>SCALAR },
                               });

         $path = "/vol/".$vol->{name}."/".$opts{file};
         $cmd = "wafl punch $path ";
         $cmd .= $opts{fbn}." ";
         $cmd .= $opts{count}." ";

         $ret = $vol->{F}->say(command=>$cmd);
         die("wafl punch failed with $ret") if($ret ne "wafl punch: status=0");
         $ret = $vol->{F}->say(command=>"sync;sync;sync");
    }

    sub file_punch_hole (@)
    {
        my ($vol, $path, %opts, $cmd, $ret);
        $vol = shift;
        (%opts) = validate(@_, {
                                file   => { type=>SCALAR },
                                offset    => { type=>SCALAR },
                                count  => { type=>SCALAR },
                               });

        $path = "/vol/".$vol->{name}."/".$opts{file};
        $cmd = "file punchhole $path ";
        $cmd .= $opts{offset}." ";
        $cmd .= $opts{count}." ";

        $ret = $vol->{F}->say(command=>$cmd);
        die("file punch failed with $ret") if ($ret != 0);
    }

=head2 vol_snap_reserve ()

Function to change snap reserve percentage of volume. if not passed any argument , returns the current value of snap reserve

=over 4

=item reserve_pct

reserve percentage value with which we want to change the snapshot reservation

=over 4

Failure

Return with an error message if command is not able to execute correctly

=back

=cut

    sub vol_snap_reserve (@)
    {
	my  ($V, $cmd, $ret, %opts);
	$V = shift();

	%opts = validate (@_, {reserve_pct => { type=>SCALAR, optional=>1}} );
	my ($reserve_pct) = $opts{reserve_pct};

	if ( defined($reserve_pct)) {
	    $cmd = "snap reserve -V $V->{name} $reserve_pct";
	    eval {
	        $ret = $V->{F}->say(command => $cmd);
	    };
	    if ($@) {  die($@);  }
	    return 1;
	} else {
	    $cmd = "snap reserve -V $V->{name}";
	    $ret = $V->{F}->say(command => $cmd);
	    if ($ret !~ s/Volume $V->{name}: current snapshot reserve is (\d+)% or (\d+) k-bytes\.//) {
		die ("vol size did not executed correctly");
	    }
	    return ($1);
	}
    }


    sub snapvault_start(@)
    {
        my ($V, %opts);
        $V = shift();

        %opts = validate (@_, { src_node => { type=>SCALAR}, 
                                src_qt   => { type=>SCALAR},
                                dst_qt   => { type=>SCALAR},
                                complete => { type=>SCALAR, optional=>1, default=>1},
                              });
        
        sv_start($V->{F}{fc}, $opts{dst_qt}, SOURCE => "$opts{src_node}:$opts{src_qt}", COMPLETE => $opts{complete});
    }

    sub snapvault_stop(@)
    {
        my ($V, %opts);
        $V = shift();

        %opts = validate (@_, {
                dst_qt   => { type=>SCALAR},
                complete => { type=>SCALAR, optional=>1, default=>1},
                });

        sv_stop($V->{F}{fc}, $opts{dst_qt}, COMPLETE => $opts{complete});
    }

     sub snapvault_update(@)
    {
        my ($V, %opts);
        $V = shift();

        %opts = validate (@_, {
                                dst_qt   => { type=>SCALAR},
                                complete => { type=>SCALAR, optional=>1, default=>1},
                              });

        sv_update($V->{F}{fc}, $opts{dst_qt}, COMPLETE => $opts{complete});
    }



############################################
### Check dense flag for a file
############################################

    sub check_dense_flag (@) {

        my ($V, $ret, $conn, %opts);

        $V = shift();

        (%opts) = validate( @_, {
                            file_name     => { type => SCALAR },
                            });
        $ret  = check_inode_dense_flag(conn => $V->{F}{fc}, vol =>$V, %opts );

        return $ret;
    }

    return 1;
}

##Function to call file check and parses the output of file_check
    sub file_check (@)
    {
        my ($V, %opts, $path, $errcode, $file_check_path, $count, $total, 
        $filecheck_vbnlist, @snapshot_list);
        $V = shift();
        %opts = validate (@_, {
                                args => {type => SCALAR},
                                parse => { type=>SCALAR, optional=>1},
                                                                
                              });

        $path = "/vol/".$V->{name}."/".$opts{args};
        $Log->comment("file name: $path");
                                        
        ($errcode, $file_check_path) = filecheck::file_check(conn => $V->{F}{fc}, args => $path);
        if ($errcode != 0) {
            die("File check error: $file_check_path");
        }
       
        if (($opts{parse})) {
            ($count, $total, $filecheck_vbnlist, @snapshot_list) =
                file_check_read_output(conn => $V->{F}{fc}, path => $file_check_path);
            return $count, $total, $filecheck_vbnlist, @snapshot_list;
        }
        else {
            return $file_check_path;
        }
    }
    
# Function to call volume clone

sub clone_create(@)
{
    my $V = shift();
    my %opts = validate (@_, {
                                flexclone => { type=>SCALAR},
                                'parent-volume' => {type=>SCALAR},
                              });
    vol_clone_create(conn=>$V->{F}{fc}, parent=>$opts{'parent-volume'}, clone=>$opts{flexclone});
    $Vc=$V->{F}->vol_create(
                    Aggr        => $V->{A}, 
                    Volname     => $opts{flexclone},
                    Size        => $V->{size},
                    Reuse       => '1'
        );
    $Log->comment("$Vc->{name}");
    return $Vc;

}

sub clone_split(@)
{
    my $V = shift();
    my %opts = validate (@_, {  flexclone => {type=>SCALAR}, 
                                foreground=> {type=>SCALAR,default=>"true"}
                             });
    my $wait=($opts{"forground"} == "false") ? 0 : 1;
    vol_clone_split_start(conn=>$V->{F}{fc}, clone=>$opts{flexclone}, wait_to_complete=>$wait);
    return 1;
}

############################################
## Description: 
##          This function will run waf lscan measure_layout command on a file.
## Inputs: 
##          Inputs to this function are "file_name" param
## Return value:
##          Returns the layout ration for particular file.
##
############################################

sub wafl_scan_measure_layout(@)
{
    my ($V, %opts, $ret, $layout);
    $V = shift();
    %opts = validate (@_, {file_name => {type=>SCALAR, optional=>1 },
                        });
    my $record = $V->{F}->start_log(file=>'messages');

    $ret = $V->{F}->say(command=>"wafl scan measure_layout $opts{file_name}");

    my $inum = $V->get_inode_num(%opts);
    my $str = "WAFL layout ratio for volume $V->{name}, public inode $inum is ";
    $record->stop();
    my $result = $record->to_log();
    my @result = split(/\n/, $result);
    my @index = grep(/$str/, @result);
    if ($index[0] =~ m/public inode (\d+) is (\d+)/ ) {
        $layout = $2;
        $Log->comment("Layouti ratio is $layout");
    } else {
        die "Didn't got the layout ratio";
    }
    return $layout;
}


############################################
## Description:
##          Invokes "wafl touch" on the inode and the fbn(s) provided
## Input params:
##          inum => Inode number of the file
##          level => the level of the block in the buftree(0 being L0)
##          start_fbn => starting fbn number 
##          end_fbn => end fbn number 
##          private => if the param is set to 1 then the inum will be treated as private
## Return value
##          Returns the command output of "wafl touch"
##
############################################

sub wafl_touch(@)
{
    my ($V, $cmd, %opts);
    $V = shift;
    %opts = validate (@_, {
                                inum    => { type=>SCALAR},
                                level   => { type=>SCALAR},
                                start_fbn => {type=>SCALAR},
                                end_fbn => {type=>SCALAR},
                                private => {type=>SCALAR, optional=>1 },
                            });
    
    if ($opts{private}) {
        $cmd = "wafl touch -V $V->{name} $opts{inum} -private $opts{level} $opts{start_fbn} $opts{end_fbn}";
    } else {
        $cmd = "wafl touch -V $V->{name} $opts{inum} $opts{level} $opts{start_fbn} $opts{end_fbn}";
    }
    $ret = $V->{F}->say(command=>$cmd);
    
    return $ret;    
}

