
{package Dense::Agnostic::Client;

    use strict;
    use Data::Dumper;
    use Params::Validate qw(validate SCALAR HASHREF);
    use Tharn 	qw($Log); 
    use NACL::APISet::Host::CLI qw(execute_raw_command);
    use MapShare qw(map_cifs_share );
    use DataManip::local qw(writeFile);

    # new (Name=>client_hostname, Type=>host_type)
    # default hosttype Linux.
    sub new (@)
    {
        my  ($class, $self, %opts, %defaults);

        %defaults   = {
            Type    => 'Linux',
        };

        $class  = shift();
        %opts   = @_;

        for my $k (keys(%defaults)) {
            if (! defined $opts{$k}) {
                $opts{$k} = $defaults{$k};
            }
        }

        $self = {};
        $self->{api} = NACL::APISet->new(
                    hostobj=>Tharn::host($opts{Name}),
                    category=>'Host',
                    interface=>'CLI',
                    set=>$opts{Type},
                );
        $self->{api}->set_timeout( "connectrec-timeout" => '300' );
        bless($self, $class);
        return $self;
    }

    sub run (@)
    {
        my  ($self, $cmdstring, $out, $timeout);

        $self = shift();
        $cmdstring = shift();
		$timeout = shift();

		if (! defined ($timeout)) {
			$timeout = '300';
		}
        if (! defined $cmdstring) {
            die("Expected cmdstring, got nothing");
        }
        my $output =($self->{'api'}->{'hostobj'}->{'hosttype'});
        if($output =~ /unix-linux/)
        {
        $cmdstring = 'sudo ' . $cmdstring;
        }
 
        elsif($output =~ /windows/)
        {
        $cmdstring =  $cmdstring;
        }
        $self->{api}->set_timeout( "connectrec-timeout" => $timeout );
        $out = $self->{api}->execute_raw_command(command=>$cmdstring);
        return $out;
    }

    sub mount_vol_nfs (@)
    {
        my  ($self, $F, $V);
        my  ($host, $cmd, $mntpt, @res, $vserver, $vserver_ip, $ret);

        $self   = shift();
        $V      = shift();

        $F      = $V->{F};
        $host   = $F->{name};   #XXX make this IP instead. Safer.

        $mntpt  = "/t/$F->{name}/$V->{name}";

	$self->umount_vol_nfs($mntpt);
        #Check that the mountpoint is visible
        @res = $self->run("showmount -e $host");
        #XXX grep $res for $V->{name}

	print "Vserver name : $vserver\n";
        #XXX check that $mntpt exists, and if not, create it.        
        @res = $self->run("mkdir -p $mntpt");
        
        if ($V->{Vs}) { ## Need to differentiate b/w cmode and 7mode
	    $vserver = $V->{Vs}{name};
            $ret = $F->network_interface_show(role =>"data", vserver=>$vserver);
            $host = $ret->{address};
            @res = $self->run("mount $host:/$V->{'name'} $mntpt;sudo chmod 777 $mntpt");
        } else {
            $host = $F->{params}->{conn}->{hostp}->{default_ip} || $F->{name};
            $F->say(command => "exportfs -io anon=0 /vol/$V->{name}");
            @res = $self->run("mount $host:/vol/$V->{name} $mntpt;sudo chmod 777 $mntpt");
        }
        
        #XXX check res for success/failure etc.
        return $mntpt;
    }


    sub umount_vol_nfs (@)
    {
        my ($self, $mntpt, $ret);
       
        $self  = shift();  
        $mntpt = shift();

        $ret = $self->run("umount $mntpt");
	$ret = $self->run("r rmdir $mntpt");
        
    }

   sub map_cifs_share()
   {
      my $F = shift();
      my (%opts) = validate(@_, { server       => { type => SCALAR, optional => 1 , reqex => qr/\w+|\d+.\d+.\d+.\d+/, default=>undef },
                                client       => { type => SCALAR|HASHREF ,default =>'localhost'},
                                share        => { type => SCALAR },
                                username     => { type => SCALAR, optional => 1},
                                password     => { type => SCALAR, optional => 1 },
                                drive        => { type => SCALAR, optional => 1},
                               });

     my $server = $opts{server};
     my $client = $opts{client};
     my $share = $opts{share};
     my $username = $opts{username};
     my $password = $opts{password};
     my $drive = $opts{drive};

     MapShare::map_cifs_share(server => $server , client => $client , share => $share , username => $username ,
                                        password => $password , drive => $drive );

   }


    sub wrfile (@)
	    {
		 my ($V, $filename, $byte_count, %opts);

		$V = shift();

		%opts = validate(@_, {filename=> {type => SCALAR,},
		                                 byte_count => {type => SCALAR,},
		                                FLOOD => {type => SCALAR, default=>0,},
		                                TRUNC => {type => SCALAR,optional =>1},
		                               SCRAMBLE => {type => SCALAR,},});

        writeFile($opts{filename}, $opts{byte_count}, %opts) or die "Failed to write data to the volume";
    }


=head2 dd(mount_point=>$mnt, input_file_name=>"ZERO", output_file_name=>$file_name, count=>$no_of_blocks, seek=>$seek, conv=>$no_trunc);

Executes dd command at the mount point specified 

=over 4

=item mount_point 

required

=item output_file_name

required

=item count

required

=item <input_file_name,seek,skip,block_size,conv> 

optional 

Success

=over 4

Returns nothing

=back

Failure

=over 4

Handled by NACL folks

=back

=back

=cut
	
	sub dd (@)
	{
		my $self = shift();
		my %opts = validate (@_, 
				{
					mount_point => {type=>SCALAR},
					input_file_name	=> {type=>SCALAR},
					output_file_name  => {type=>SCALAR},
					count => {type=>SCALAR},
					block_size => {type=>SCALAR, optional=>1, default=>4096}, 
					seek => {type=>SCALAR, optional=>1},
					skip => {type=>SCALAR, optional=>1},
					conv => {type=>SCALAR, optional=>1}
				}
			);
		 
		my ($mnt_point, $if, $of, $bs, $no_of_blocks,$seek,$skip,$conv) ;
		$conv = undef ; 
		$mnt_point = $opts{mount_point};
         		
		if($opts{input_file_name} =~ /ZERO/)
		{
			$if = "/dev/zero" ; 
		}elsif($opts{input_file_name} =~ /RANDOM/)
		{
			$if = "/dev/urandom";
		}else
		{
			$if = $mnt_point."/".$opts{input_file_name};
		}
				
		$bs = $opts{block_size};
		
		if (defined($opts{conv}))
		{
			$conv = $opts{conv};	
			$Log->comment("The value of conv is $conv");
		}
		
		
		$of = $mnt_point."/".$opts{output_file_name};
		$no_of_blocks = $opts{count};
		
		$Log->comment("Mount point is $mnt_point ");
		$Log->comment("Input file path is $if\n");
		$Log->comment("Output file path is $of\n");
		$Log->comment("No of blocks is $no_of_blocks\n");
		
		if(defined($opts{seek}) && defined($opts{skip} ))
		{
			$seek = $opts{seek} ;
			$skip = $opts{skip};
			$Log->comment("Value of seek is $seek\n");
			$Log->comment("Value of skip is $skip\n");
			if(defined($conv))
			{
				$self->run("dd if=$if of=$of bs=$bs skip=$skip seek=$seek  count=$no_of_blocks conv=$conv");
			}else
			{			
				$self->run("dd if=$if of=$of bs=$bs skip=$skip seek=$seek  count=$no_of_blocks");
			}
			
		}elsif(defined($opts{seek}))
		{
			$seek = $opts{seek} ;
			$Log->comment("Value of seek is $seek\n");
			if(defined($conv))
			{
				$self->run("dd if=$if of=$of bs=$bs seek=$seek count=$no_of_blocks conv=$conv");
			}else
			{			
				$self->run("dd if=$if of=$of bs=$bs seek=$seek count=$no_of_blocks");
			}
		}elsif(defined($opts{skip}))
		{
			$skip = $opts{skip};
			$Log->comment("Value of skip is $skip\n");
			if(defined($conv))
			{
				$self->run("dd if=$if of=$of bs=$bs skip=$skip count=$no_of_blocks conv=$conv");
			}else
			{			
				$self->run("dd if=$if of=$of bs=$bs skip=$skip count=$no_of_blocks ");
			}
			
		}else
		{	if(defined($conv))
			{
				$self->run("dd if=$if of=$of bs=$bs count=$no_of_blocks conv=$conv");
			}else
			{
				$self->run("dd if=$if of=$of bs=$bs count=$no_of_blocks");
			}			
		}
			
	}

    return 1;
}
