{ package   Dense::Agnostic::Cmode::Aggr;

    use strict;
    use warnings;

    use Data::Dumper;

    use constant    TYPE_CM_NODE    => 'Dense::Agnostic::Cmode::Node';

    use Tharn   qw($Log);
    use Dense::Agnostic::Constants;

    use Dense::Agnostic::Cmode::Aggr::Wafliron     qw(wafliron_start);
    use Dense::Agnostic::Cmode::Aggr::Fmap;

    sub _check_aggrnew_args ($)
    {
        my  ($F, $A, $opts);
        my  ($epfx, $emsg);

        $opts = $_[0];

        #Mandatory args
        $epfx = 'Dense::Agnostic::Cmode::Aggr::';
        if ( (! defined $opts->{Aggrname}) ||
             (! defined $opts->{Node})     ||
             (! defined $opts->{Ndisks}) ) {
            $emsg = "$epfx new need Aggrname. Node and \
                Ndisks args";
            $Log->comment($emsg);
            die($emsg);
        }

        #Type checks
        $F = $opts->{Node};
        $A = $opts->{Aggr};
        if (ref($F) ne TYPE_CM_NODE) {
            $emsg = "$epfx new: type mismatch: arg Node \
                has bad type";
            $Log->comment($emsg);
            die($emsg);
        }

        #Other options: Raidtype, Snaplock, Indirect_type.
        #Replace the leading capital by a lowercase letter to conform to the
        #APISet naming convention. Perhaps we should've followed the same?
        if (defined $opts->{Raidtype}) {
            $opts->{raidtype} = delete $opts->{Raidtype};
            if (($opts->{raidtype} != 'raid4') &&
                ($opts->{raidtype} != 'raid_dp')) {
                $emsg = "$epfx new: raidtype is neither raid4 nor raid_dp";
            }
            
            if (defined $opts->{Snaplock}) {
                delete $opts->{Snaplock};
                #XXX do we support Snaplock in CMode?
            }
	}

	
            if (defined $opts->{IndirectType}) {
                $opts->{indirect_type} = delete $opts->{IndirectType};
                if  ($opts->{indirect_type} == 32) {
                    $opts->{indirect_type} = '32-bit';
                } elsif ($opts->{indirect_type} == 64) {
                    $opts->{indirect_type} = '64-bit';
                } else {
                    $emsg = "$epfx new: Indirect type is neither 32 nor 64";
                    $Log->comment($emsg);
                    die($emsg);
                }
            }
    }

    sub purge (@)
    {
	my ($A, $ret);
	$A = shift();
	$A->offline();
	$A->destroy();
	$A->{F}->say(command=>"disk zero spares");
    }

    sub is_large_aggr (@)
    {
	my ($A, $ret);
	my $block_type;;

	$A = shift();
	$ret = $A->show(fields=>'block-type');
	print Dumper($ret);
	
	$block_type = $ret->{'block-type'};
	$Log->comment("Block type of aggregate : $block_type");

	if( $block_type =~ /64-bit/ ){
	    return 1;
	} else {
	    return 0;
	}
    }

    sub is_upgradable (@)
    {
	my $A;
        $A = shift();
	if ($A->is_large_aggr()) {
		return 0;
	} else {
		return 1;
	}
    }

    sub aggr_upgrade (@)
    {
	my ($A, %opts);
	$A = shift();

	%opts = @_;
	my $start_scan = $opts{'start_scan'} ? $opts{'start_scan'} : 0;

	if ($start_scan) {
	    $A->{F}->say(command=>'setflag wafl_64bit_upgrade_scan_disable 0');
	} else {
	    $A->{F}->say(command=>'setflag wafl_64bit_upgrade_scan_disable 1');
	}
 
	# Convert aggregate
	$A->upgrade();
	$A->upgrade_status();
    }

    sub new (@)
    {
        my  ($class, %opts);
        my  ($A, @ret, $wret, $reuse, $timeout);

        $class  = shift();
        %opts   = @_;
        eval {
            _check_aggrnew_args(\%opts);
        };
        if ($@) {
            die("Dense::Agnostic::Cmode::Aggr::new: $@");
        }

        $A  = {
            F       => delete $opts{Node},
            name    => delete $opts{Aggrname},
            ndisks  => delete $opts{Ndisks},
            reuse   => delete $opts{Reuse},
	    block   => delete $opts{indirect_type},
	    raidtype=> delete $opts{raidtype},
        };

        @ret = $A->{F}->storage_aggregate_show();
        print "XXXX".Dumper(\@ret);
        $reuse = delete $A->{F}->{reuse};
        #Check to see if aggr exists.
        for my $a (@ret) {
            if (grep(/$A->{F}->{name}/, $a->{nodes})) {
                #Node matches
                if (grep(/$A->{name}/, $a->{aggregate})) {
                    #Aggr matches too - aggr exists
                    #XXX Try add disks to aggr. If that doesn't succeed, die.
                    if (! $reuse) {
                        goto OUT;
                    } else {
                        $A->{F}->storage_aggregate_delete(
                                aggregate=>$A->{name},
                                force=>'true',
                                );
                    }
                }
            }
        }


        eval    {
            my  (%in_opts, $ret);

            %in_opts = (
                    aggregate       => $A->{name},
                    'volume-style'  => 'flex',
                    nodes           => $A->{F}->{name},
                    diskcount       => $A->{ndisks},
                    raidtype        => $A->{raidtype},
                   'block-type'   => $A->{block},
            );
	    $ret = $A->{F}->storage_aggregate_create(%in_opts);
            if (! $ret) {
               die("Creation of aggr $A->{name} failed");
            }

            $timeout = 60;
            while ($timeout--) {
#                for my $a (@{$A->{F}->storage_aggregate_show()}) {
                for my $a ( $A->{F}->storage_aggregate_show() ) {
                    if ($a->{aggregate} eq $A->{name}) {
                        $Log->comment(">>>> Aggregate $A->{name} created!");
                        goto OUT;
                    }
                }
                sleep(1);
            }
            die("Error creating aggregate $A->{name}");
        };
        if ($@) {
            die("Dense::Agnostic::Cmode::Aggr::new: $@");
        }

OUT:
        bless($A, $class);
        return $A;
    }

    sub	AUTOLOAD	(@)
    {
	    our	$AUTOLOAD;
	    my	($A, $F);
	    my  ($in_fn, %in_opts, $exec_fn, %exec_opts, $ret);
	    my  ($k, $v);

	    $A	= shift();
	    %in_opts = @_;
	    $in_fn = $AUTOLOAD;
	    $in_fn =~ s/.*:://;

	    $F = $A->{F};
	    if (ref($F) !~ 'Cmode::Node') {
		    die('Cmode::Volume::AUTOLOAD: $F has wrong type');
	    }

	    if (! grep(/$in_fn/,keys(%$Fmap))) {
		    if ($in_fn eq 'DESTROY') {
			    $Log->comment("Dense::Agnostic::Cmode::Volume: recd Destroy?");
			    return;
		    } else {
			    $Log->debug(print "Cmode::Aggregate: Unknown method $in_fn");
			    die("Dense::Agnostic::Cmode::Aggr: Unknown method $in_fn");
		    }
	    }

	    $exec_fn = $Fmap->{$in_fn}{fname};
	    $Log->comment("Cmode::Aggr::AUTOLOAD: evaluated $in_fn -> $exec_fn\n");

	    $exec_opts{aggregate} = $A->{name};
	    while(($k, $v) = each(%in_opts)) {
		    $exec_opts{lc($k)} = $v;
	    }

	    foreach my $p (@{$Fmap->{$in_fn}{m_args}}) {
		if (! defined $exec_opts{$p}) {
		    die("Cmode::Aggr->$in_fn: mandatory arg $p missing");
		}
	    }

	    while (($k, $v) = each(%{$Fmap->{$in_fn}{d_args}})) {
		if (! defined $exec_opts{$k}) {
		    $exec_opts{$k} = $v;
		}
	    }

	    $Log->comment("Cmode::Aggr::AUTOLOAD: Evaluating $exec_fn with the following args:");
	    if (%exec_opts) {
		$Log->debug(print Dumper(\%exec_opts));
		$ret = $F->$exec_fn(%exec_opts)
	    } else {
		$ret = $F->$exec_fn();
	    }

	    return $ret;
	    
    }

    return 1;
} #Package
