{ package Dense::Agnostic::Cmode::Aggr;

    use Tharn;
    use Data::Dumper;
    
    use constant DENSE_DEFAULT_TIMEOUT  => 3600;
    use constant GAP_INTERVAL           => 5;
    use constant WAFLIRON_IDLE          => "not_running";
    use Params::Validate qw(validate SCALAR);

=head2 $A->wafliron_start()
starts wafliron on an aggregate

=item speed

=item wait
Wait for wafliron to complete. Optional. Default is 1
=cut
    sub wafliron_start (@)
    {
        my  ($A, %opts, $filer, $timeout, $count, $ret, $aggr_name, $completed);

        $A  = shift();
        %opts = validate(@_, {  speed => { type=>SCALAR, default=>2000, regex=>qr/^\d+$/ },
				Aggrname=> { type=>SCALAR, optional=>1, default=>"$A->{name}" },
				Nodename => { type=>SCALAR },
				wait => { type=>SCALAR, default=>1, regex=>qr/^(0|1)$/ } }
			);	

	$Log->comment("Wait : $opts{wait}");
        $timeout = $opts{timeout} ? $opts{timeout} : DENSE_DEFAULT_TIMEOUT;
	if ($opts{speed}) {
	    $A->{F}->say(command=>"wafl scan speed $opts{speed}");
	}
        $aggr_name = $A->{name}; 

        $A->{F}->storage_aggregate_wafliron_start(aggregate=>$aggr_name);

        if (! $opts{wait}) {
	    $Log->comment("Wait is not set");
            goto OUT ;
        }
	$Log->comment("Wait is set");
        for($count = 0 ; $count < $timeout ; $count++) {
            # Need to use NACL exceptions.
            eval {
                $ret = $A->{F}->storage_aggregate_wafliron_show(aggregate=>$aggr_name);
            };
            if($@) {
                die("Not able to check the wafliron status:".$@);
            }
            $wafliron_status = $ret->{"iron-status"};
            logcomment("Wafliron Status on Filer is = ".$wafliron_status);
            print "$wafliron_status :XXX\n";
            if($wafliron_status !~ /WAFLIRON_IDLE|not_running/) {
                $Log->step("Wafliron status is still running");
            } else {
                $Log->step("Wafliron seems to have completed");
                return 1;
            }
            sleep(GAP_INTERVAL);
        }
        sleep(GAP_INTERVAL);

OUT:
    } #End wafliron_start()
=head2 $A->wafliron_wait(speed=>10, timeout=>120)
Wait for wafliron to complete within the given timeout

=item speed

Optional parameter to set wafl scan speed before waiting for wafliron run to complete

=item timeout

Duration within which wafliron has to complete

=cut
    sub wafliron_wait (@)
    {
	my ($A, %opts, $speed, $wafliron_status);

	$A = shift();
	%opts = validate (@_, { speed => { type=>SCALAR, optional=>1 },
				timeout => { type=>SCALAR, optional=>1, default=>900 }, }
			);
	my $timeout = time() + $opts{timeout};
	$speed = $opts{speed};
        if ($speed) {
	    $A->{F}->say(command=>"wafl scan speed $speed");
	}
	while (time() < $timeout) {
	    $wafliron_status = $A->{F}->say(command=>"aggr wafliron status $A->{name}");
	    if ($wafliron_status =~ /wafliron is not currently active/) {
		return 1;
	    } else {
		sleep 5;
	    }
	}
	die "Wafliron has not completed";
    }

=head2 $A->is_iron_active()
Returns 1 if wafliron is active on the aggr, else returns 0
=cut
    sub is_iron_active()
    {
	my ($A, $status);

	$A = shift();
	$status = $A->{F}->say(command=>"aggr wafliron status $A->{name}");
	if ($status =~ /wafliron is not currently active/) {
	    return 0;
	} else {
	    return 1;
	}
    }
    return 1;

}#End package Dense::Agnostic::Cmode::Aggr
