#!/usr/local/bin/perl
#
# $Id: //depot/prod/test/Rsierranevada/lib/Dense/Agnostic/Cmode/Utils.pm#1 $
#
# Copyright (c) 2008 NetApp,  Inc.
# All rights reserved.

## @summary Dedupe Cmode utility module
##
## @description
##    This module provides utility functions required for dedupe work flow
##
## @synopsis
##
## use Dense::Constant::Message;
## use Dense::Agnostic::Constants;
## use Dense::Agnostic::Cmode::Utils;
## 
## 
## @author khung@netapp.com
## @status private (in-progress)

{   package Dense::Agnostic::Cmode::Utils;
    use strict;
	use warnings;

    use Data::Dumper;

    use Params::Validate qw( validate SCALAR );

    use Dense::Agnostic::Constants;

    use Tharn   qw($Log);

    use vars ('@ISA', '@EXPORT');
    require Exporter;
    @ISA    = qw(Exporter);
    @EXPORT = qw(validate_message
                 get_error_message);

	
	## @name ::validate_message
	## @summary Compares two messages.
	## 
	## @description
	## Compares two message strings.
	##
	## 
	## @return TRUE/FALSE.
	##
	## @arg obtained_value Required: Value to be compared.
	##
	## @arg expected_value Required: Value to be compared against.
	##
	##
	## @examples
	## my $result = validate_message( obtained_value => $obtained,
	##                                expected_value => $expected);
	## 
	## @author khung@netapp.com
	## @status private (in-progress)
	##
	sub validate_message {
		my ( $expected_value, $obtained_value );

		$expected_value = shift;
		$obtained_value = shift;

		$Log->comment("===== OBTAINED MESSAGE ====\n  $obtained_value \n ==== EXPECTED MESSAGE ==== \n $expected_value \n====");

		if ( $obtained_value =~ $expected_value || ( $expected_value eq "" && $obtained_value == 1 ) ) {
			$Log->comment("The expected and the obtained error messages are the same.");
            return TRUE;
		}
		else {
			$Log->comment("The expected and the obtained error messages are different.");
             return FALSE;
		}
	}

	## @name ::get_error_message
	## @summary Gets the message from library Dense::Constant::Message
	## 
	## @description
	## Gets and returns the specified message from library Dense::Constant::Message.
	##
	## 
	## @return message string.
	##
	## @arg code Required: Name of message to be returned.
	##
	## @arg host Required: Name of filer/node.
	##
	##
	## @examples
	## my $msg = get_error_message( code => SIS_ENABLE_ON_VOLUME,
	##                              host => $FILER);
	## 
	## @author khung@netapp.com
	## @status private (in-progress)
	##
	sub get_error_message {
		my (%opts) = @_;

		return Dense::Constant::Message::GetMessage( code => $opts{code}, host => $opts{host} );
	}


    return 1;
}
