{ package   Dense::Agnostic::Cmode::Volume;
    use strict;
    use warnings;

    use Data::Dumper;

    use Tharn   qw($Log);

    use Dense::Agnostic::Cmode::Volume::Fmap;
    use Dense::Agnostic::Cmode::Volume::Fmap_zapi;
    use Dense::Agnostic::Cmode::Volume::Snap qw( snap_list_b );
    use Dense::Agnostic::Cmode::Volume::Sis     qw(
                                                    sis_wait_for 
                                                    get_savings
                                                    sis_check_records
                                                    set_overlap_request
                                                  );
    use Dense::Agnostic::Cmode::Volume::Utils;
    use FS::Util::Utilities qw(util_stat);

    sub new (@)
    {
        my  ($class, %in_opts, %opts);
        my  ($F, $A, $Vs, $V);
        my  ($reuse, $timeout, @ret, $vret);

        $class = shift();
        %in_opts = @_;
        $V  = {};

        $A  = delete $in_opts{Aggr};
        if (ref($A) !~ 'Aggr') {
            die("Cmode::Volume->new: Argument Aggr has wrong type");
        }
        $F  = $A->{F};

        $Vs = delete $in_opts{Vserver};
        if (ref($Vs) !~ 'Vserver') {
            die("Cmode::Volume->new: Argument Vserver has wrong type");
        }

        $reuse  = delete $in_opts{Reuse};

        $opts{volume}       = $in_opts{Volname};
        $opts{vserver}      = $Vs->{name};
        $opts{aggregate}    = $A->{name};
        $opts{size}         = $in_opts{Size};
	
	### Junction paths cannot be defined for DP volumes 
	if( !defined($in_opts{Vol_opts}) || ($in_opts{Vol_opts}->{type} !~ /DP/) )
	{
				
        	$opts{'unix-permissions'} = "---rwxrwxrwx";
        	if (defined $in_opts{Junctionpath}) {
            		$opts{'junction-path'} = $in_opts{Junctionpath};
        	} else {
            		$opts{'junction-path'} = "/".$in_opts{Volname};
        	}
	}
	## All arguments specified in the Vol_opts hash has to be copied to the
	## %opts hash if any specific vol option is defined 
	if ( defined ( $in_opts{Vol_opts} ) )
	{
		foreach my $key ( keys ( %{$in_opts{Vol_opts}} ) )
		{
			$opts{$key} = $in_opts{Vol_opts}->{$key} ;
		}
	}
	
        #XXX we need this weirdness because of the following:
        #if one executes volume_show() on a volume that does not exist,
        #the APISet throws an exception and does some funny things with
        #destroy which more or less kills us.
            
        @ret = $F->volume_show( vserver => $opts{vserver} );

        for my $v (@ret) {
            if ($opts{volume} eq $v->{volume}) {
		        if ($reuse) {
                    goto OUT;
		        } else {
		            $F->volume_unmount(vserver=>$opts{vserver},volume=>$opts{volume});
		            $F->volume_offline(vserver=>$opts{vserver},volume=>$opts{volume});
		            $F->volume_destroy(vserver=>$opts{vserver},volume=>$opts{volume});
            	    goto CREATE_VOL;
		        }
            }
        }

CREATE_VOL:
        eval {
            $vret = $F->volume_create(%opts);
        };

        $timeout = 30;
        while ($timeout--) {
            for my $v ( $F->volume_show( vserver=>$opts{vserver} ) ) {
                if ($v->{volume} eq $in_opts{Volname}) {
                    goto OUT;
                }
            }
            sleep(1);
        }

OUT:
        $V->{name}  = $opts{volume};
        $V->{path}  = '/vol/'.$opts{volume};
        $V->{A}     = $A;
        $V->{Vs}    = $Vs;
        $V->{F}     = $F;

        bless($V, $class);
        return $V;
    }

    sub vol_wait_online (@) {
	my ($V, $ret);
	my $state;

	$V = shift();
	my $timeout = time() + 90;
	while (time() < $timeout) {
	$ret = $V->show(fields=>'state');
	print Dumper($ret);
	$state = $ret->{'state'};
	if ($state =~ /online/) {
	    return;
	}
	}
	die "Volume is still not online";
    }
    #XXX this shouldn't really belong here, move it out to the right place.
    sub mkfile  (@)
    {
        my  (%opts);
        my  ($V, $F, $cmd, $ret, $filepath);

        $V  = shift();
        %opts = @_;

        if ((! defined $opts{Size})     ||
            (! defined $opts{Path}))  {
            die("Cmode::Volume->mkfile: need Size and Path args");
        }
        
        $filepath = $opts{Path};

        if( $filepath !~ /\/vol\//) {
            $filepath = "/vol/$V->{name}/$opts{Path}";
        }


        $F = $V->{F};

	$cmd = "mkfile $opts{Option} $opts{Size} $filepath $opts{Count}";
        $ret = $F->system_node_run(node=>$F->{name}, command=>$cmd);

        if(!(defined($opts{Nosync})))
		{
			$ret = $F->system_node_run(node=>$F->{name}, command=>'sync');
			$ret = $F->system_node_run(node=>$F->{name}, command=>'sync');
			$ret = $F->system_node_run(node=>$F->{name}, command=>'sync');
			return $ret;
		}	
    }

    sub rm  (@)
    {
        my  ($V, $filepath);
        my  ($cmdstring);

        $V = shift();
        $filepath = shift();

        $cmdstring = "rm $filepath";
        eval {
            $V->{F}->say(command => $cmdstring);
        };
        if ($@) {
            die($@);
        }
    }

    sub purge (@)
    {
	    my $V;
	    $V = shift();

	    $V->unmount();
	    sleep(5);	
	    $V->offline();
	    $V->destroy();
    }

    sub dense_purge (@) {
        my $V;
        $V = shift();

        if ( $V->is_dense_enabled() ) {

            #Forcing a sis stop
            eval {
                $V->sis_stop();
            };
            $V->sis_wait_for();
        }

        $V->unmount();
        $V->offline();
        $V->destroy();
    }

    sub is_hybrid (@)
    {
        my $V;
        $V = shift();

        my $ret = $V->{F}->say(command=>"vol_db iter");
	my $vol_name = $V->{name};

        my @output = split("\r*\n", $ret);
        foreach my $line (@output) {
	    my $name = $V->{F}->say(command=>"vol_db get name $line");
	    if ($name =~ /$vol_name/) {
		my $buftree = $V->{F}->say(command=>"vol_db get buftree_type $line");
		if ($buftree =~ /upgrading/) {
		    return 1;
		}
		return 0;
	    }
        }
	return 0;
    }

    sub mixed_buftree (@)
    {
	my ($V, $file, %opts);
	$V = shift();
	$file = shift();
	%opts = validate( @_, {'level' => { type => SCALAR, default => 0, optional => 1},
			      'fbn-start' => { type=>SCALAR, optional=>1, default=>0 },
			      'fbn-end' => { type=>SCALAR, optional=>1 },
			      'fbn-skip' => { type=>SCALAR, optional=>1 },
			      });

	my $vname = $V->{name};
	my $F = $V->{F};
	my $filer_name = $F->{name};

	if (! $V->is_hybrid()) {
	    die "Mixed buftree is not supported for non-hybrid volume\n";
	}
        my $file_name = "/t/$filer_name/$vname/".$file;
	
	my @stats = util_stat($file_name);

	my $level = $opts{'level'};
	my $fbn_start = $opts{'fbn-start'};
	my $fbn_end = $opts{'fbn-end'} ? $opts{'fbn-end'} : POSIX::floor($stats[7] / 4096);
	my @skips = (2048, 2097152, 2147483648, 4286529242, 4286529242, 4286529242, 4286529242, 4286529242);
	my $fbn_skip = $opts{'fbn-skip'} ? $opts{'fbn-skip'} : $skips[$level];
	my $inodenum = $V->get_inode_num(file_name=>"/vol/$vname/$file");
	print "Inode number is $inodenum \n";
	# Issue wafl touch command
	$F->say(command=>"wafl touch $vname $inodenum $level $fbn_start $fbn_end $fbn_skip");
    }


##########################################################################
# Function     : get_vol_space()
# Input        : dense agnostic vol object
# Return value : Returns df -r output hash
# Summary      : This function will parse the d-blade df -r output and
#                   return it in a hash format
##########################################################################
    sub get_vol_space (@)
    {
        my  ($V, %ret, $cmd, $ret_val, @df_lines);

        $V  = shift();

        $V->{F}->say(command => "sync;sync;sync");
        $cmd = 'df -r '.$V->{'name'};
        $ret_val = $V->{F}->say(command => $cmd);

        $Log->debug( print Dumper($ret_val) );
        @df_lines = split("\n", $ret_val);

        my @df_fields = split( '\s+', $df_lines[1]);
        my @df_values = split( '\s+', $df_lines[2]);

        my $fields = scalar @df_fields;

        for(my $i = 0; $i < $fields ; $i++) {
            $ret{$df_fields[$i]} = $df_values[$i];
        }
            
        return \%ret;
    }


    sub AUTOLOAD (@)
    {
        our $AUTOLOAD;
        my  ($V, $F);
        my  ($in_fn, %in_opts, $exec_fn, %exec_opts, $ret, @ret);
        my  ($k, $v);

        $V = shift();
        %in_opts   = @_;
        $in_fn  = $AUTOLOAD;
        $in_fn  =~ s/.*:://;
            #Chop off the leading Foo::bar::blah

        $F  = $V->{F};
        if (ref($F) !~ 'Cmode::Node') {
            die('Cmode::Volume::AUTOLOAD: $F has wrong type');
        }

        if ($in_fn =~ /^zapi_/ ) {
            $exec_fn = $Fmap_zapi->{$in_fn}{fname};
        } else {
            $exec_fn = $Fmap->{$in_fn}{fname};
        }
        
        if ( !defined $exec_fn ) {
            if ($in_fn eq 'DESTROY') {
                $Log->comment("Dense::Agnostic::Cmode::Volume: recd Destroy?");
            } else {
                die("Dense::Agnostic::Cmode::Volume: Unknown method $in_fn");
            }
        }

        $Log->comment("Cmode::Volume::AUTOLOAD: evaluated $in_fn -> $exec_fn\n");

        #Anything invoked on a volume should have the following at the
        #minimum...
        #option volume is a default argument except when option path is used

        if ( $in_fn =~ /^zapi_/ ) {
            $exec_opts{zapi} = 1;
            if ( grep /^path$/, @{$Fmap_zapi->{$in_fn}{m_args}} ) {
                $exec_opts{path} = $V->{path};
            }

            if ( grep /^vserver$/, @{$Fmap_zapi->{$in_fn}{m_args}} ) {
                $exec_opts{vserver} = $V->{Vs}->{name};
            }
        } else {
        if (! exists $in_opts{path} ) {
            $exec_opts{volume} = $V->{name} if (grep /^volume$/, @{$Fmap->{$in_fn}{m_args}});
        }
            $exec_opts{vserver} = $V->{Vs}->{name};
        }

        while (($k, $v) = each(%in_opts)) {
#            $exec_opts{lc($k)} = $v;
            $exec_opts{$k} = $v;
        }
        #How about map($exec_opts{lc($_)} = $in_opts{$_}, keys(%in_opts))

        #Check for mandatory params
        foreach my $p (@{$Fmap->{$in_fn}{m_args}}) {
            if (! defined $exec_opts{$p}) {
				if ( $p == "volume" ) {
					next if defined $exec_opts{path};
				}
                die("Cmode::Volume->$in_fn: mandatory arg $p missing");
            }
        }

        #Resolve default params if any
        while (($k, $v) = each(%{$Fmap->{$in_fn}{d_args}})) {
            if (! defined $exec_opts{$k}) {
                $exec_opts{$k} = $v;
            }
        }

        #Now we should have a hash with all params reqd. to execute $in_fn.
        $Log->comment("Cmode::Volume::AUTOLOAD: Evaluating $exec_fn with the following args:");

        if ( wantarray() ) {
            if (%exec_opts) {
                $Log->debug(print Dumper(\%exec_opts));
                @ret = $F->$exec_fn(%exec_opts);
            } else {
                @ret = $F->$exec_fn();
            }
            return @ret;
        } else {
            if (%exec_opts) {
                $Log->debug(print Dumper(\%exec_opts));
                $ret = $F->$exec_fn(%exec_opts);
            } else {
                $ret = $F->$exec_fn();
            }
            return $ret;
        }

        #if (%exec_opts) {
        #    eval {
        #        $ret = $F->$exec_fn(%exec_opts);
        #    }; 
        #    if ($@) {
        #        $Log->debug(print ">>>>>> Cmode::Volume::AUTOLOAD: CMD $in_fn FAILED <<<<<<\n");
        #        $ret = "ERR: Cmode::Volume::$exec_fn FAILED: $@";
        #        die($ret);
        #    }
        #} else {
        #    $ret = $F->$exec_fn->();
        #}

#        return $ret;
    }

    return 1;
} #Package
