{ package   Dense::Agnostic::Cmode::Volume;
    use strict;
    use warnings;

    use Tharn   qw($Log);
    use Dense::Constant::Command;

    use Params::Validate qw(SCALAR ARRAYREF);
    use NACL::APISet;
    use Data::Dumper;
    use NACL::CS::VolumeEfficiency qw(fetch);
    use NACL::CS::VolumeEfficiencyStat qw(fetch);
    use NACL::STask::VolumeEfficiency qw(wait_for_changelog);
    
    sub sis_wait_for    (@)
    {
        my  (%opts);
        my  ($F, $V, $Vs, $cur_status, $wait_on, $wait_for);
        my  ($ret, $s_out, $timeout);

        $V  = shift();
        %opts   = validate (@_, { status => {type=>SCALAR, optional=>1, default=>"Idle"},
				  stage  => {type=>SCALAR, optional=>1},
                  		  'sub-stage' => {type=>SCALAR, optional=>1},
				  progress => {type=>SCALAR, optional=>1},
				  lastoperr => {type=>SCALAR, optional=>1},
				  timeout => {type=>SCALAR, optional=>1, default=>3600},
				  timegap => {type=>SCALAR, optional=>1, default=>5} }
			);

        $F  = $V->{F};
        $Vs = $opts{Vserver};

        $wait_on = $opts{stage} ? 'stage' : 'op-status';
	$wait_on = $opts{progress} ? 'progress' : $wait_on;
        $wait_for = $opts{stage} ? $opts{stage} : $opts{status};
	$wait_for = $opts{progress} ? $opts{progress} : $wait_for;
	
        if (ref($F) !~ 'Cmode::Node') {
            die("Sis_Fmap::sis_wait_for: bad type for Node $F->{name}");
        }

        if ($opts{timeout}) {
            $timeout = $opts{timeout};
        }

        eval {
           $ret = $V->sis_status();
        };
        $Log->debug(print Dumper($ret));

        if($@) {
           goto OUT if(($opts{status} eq 'NOSTATUS') && ($@ =~/There are no entries matching your query/));
           die($@);
        }
#        $cur_status = {@$ret[0]}->{'op-status'};
        $cur_status = $ret->{'op-status'};
        $Log->debug(">>>>>>>>>> Current status : $cur_status");

        #XXX We'd like to err on the side of caution
        if ($cur_status eq $wait_for) {
            sleep(5);
        }

        $Log->debug(print ">>>>> Waiting $timeout sec to reach status $wait_for\n");
        while ($timeout--) {
            eval {
            $ret = $V->sis_status();
            };
            if($@) {
               goto OUT if(($opts{status} eq 'NOSTATUS') && ($@ =~/There are no entries matching your query/));
               die($@);
            }
            $Log->debug(print ">>>>>> $timeout [$ret->{'op-status'}]\n");
            $Log->debug(print "Status: $ret->{'op-status'}, Stage: $ret->{'stage'}, Progress: $ret->{'progress'}, Checkpoint Stage: $ret->{'checkpoint-stage'} \n");
            if (defined($opts{'sub-stage'})) {
                $Log->debug(print "Current Checkpoint Stage is $ret->{'checkpoint-stage'}, Expected value is $opts{'sub-stage'}");
                if ($ret->{$wait_on} =~ /$wait_for/i && $ret->{'checkpoint-stage'} =~ /$opts{'sub-stage'}/i) {
                    $Log->debug(print ">>>>>>>>>>>>> Got expected status $wait_for\n");
                    goto OUT;
                }
            } elsif ($ret->{$wait_on} =~ /$wait_for/i) {
                $Log->debug(print ">>>>>>>>>>>>> Got expected status $wait_for\n");
                goto OUT;
            }
            sleep ($opts{timegap});
        }
        goto ERR_OUT;

OUT:
        if (defined($opts{lastoperr})) {
            if ($ret->{'last-op-error'} !~ /$opts{lastoperr}/) {
                die("Sis operation was unsuccessfull");
            }
        }

        $ret = 1;
        return $ret;

ERR_OUT:
        $ret = 0;
        return $ret;
    }

    sub sis_stat (@)
    {
        my  (%opts);
        my  ($F, $V, $Vs, $cur_stat, %options);
        my  ($ret);

        $V  = shift();
        %opts   = validate (@_, { long => {type=>SCALAR, optional=>1},
				  verbose => {type=>SCALAR, optional=>1},
				  block => {type=>SCALAR, optional=>1},}
			);
	$F = $V->{F};
	$Vs = $opts{Vserver};

   # my $Node = $F->{ci};
   # $ret = NACL::CS::VolumeEfficiencyStat->fetch(
   #     command_interface => $Node,
   #     filter => { path=>$V->{params}{path}, vserver=>$V->{Vs}{name} },
   #     );
            
   $ret = $V->get_sis_stat(%options);
	print Dumper($ret);

	return $ret;
    }

    sub dense_sort_gen (%)
    {
	my  ($V, $ret, %opts, $cmd);
	$V  = shift();
	(%opts) = validate (@_, {
			    'bucket' => { type=>SCALAR },
			    'num_records_KB' => { type=>SCALAR },
			    }
			   );
	if ($opts{bucket} =~ /true/i) {
		$cmd = "sis sort gen bucket $opts{num_records_KB} $V->{params}{path}";
	} else {
		$cmd = "sis sort gen $opts{num_records_KB} $V->{params}{path}";
	}

	eval {
	    $ret = $V->{F}->say(command=>$cmd);
	};
	if( $@ ){
	    die ($@);
        }
	return 1;
    }
    sub dense_sort_run (%)
    {
	my  ($V, $ret, %opts, $cmd, %sort_counters);
	$V  = shift();
	(%opts) = validate (@_, {
			    'bucket' => { type=>SCALAR },
			    'num_records_KB' => { type=>SCALAR },
			    'num_workers' => { type=>SCALAR, optional=>1, default=>2 },
			    'input_file' => { type=>SCALAR, optional=>1 },
			    'output_file' => { type=>SCALAR },
			    'sorted_fp_file' => { type=>SCALAR, optional=>1 },
			    }
			   );
	if ($opts{bucket} =~ /true/i) {
		$cmd = "sis sort run bucket $opts{num_records_KB} $opts{num_workers} $V->{params}{path} $opts{output_file} $opts{sorted_fp_file}";
	} else {
		$cmd = "sis sort run $opts{num_records_KB} $opts{num_workers} $opts{input_file} $opts{output_file} $opts{sorted_fp_file}";
	}

	eval {
	    $ret = $V->{F}->say(command=>$cmd);
	};
	if( $@ ){
	    die ($@);
        }

	my @lines = split(/\n/, $ret);
	foreach my $line (@lines) {
	    if( $line =~ /^\s*(\S.+):\s+(.+?)\s*$/ ){
		my $key = $1;
		my $value = $2;
		$key = Dense::remove_leading_trailing_spaces($key);
		$value = Dense::remove_leading_trailing_spaces($value);
		$sort_counters{$key} = $value;
	    }
	}

	return \%sort_counters;
    }

=head2 vol_size ()

Function to increase or decrease the volume size. if not passed any argument , returns the size of the volume

=over 4

=item size

size with which we want to increase/decrease the volume

=over 4

Failure

Return with an error message if command is not able to execute correctly

=back

=cut

    sub vol_size (@)
    {
	my ($V, $cmd, $ret, %opts);
	$V = shift();

	%opts = validate (@_, {size => { type=>SCALAR, optional=>1 } });
	my $size_vol = qr{\d+\S\D};

	my ($size) = $opts{size};
	if ( $opts{size} ){
	    $V->{F}->volume_size(vserver=>$V->{Vs}{name}, volume=>$V->{name},'new-size'=>$size);
	} else {
	    $cmd = "vol size $V->{name}";
	    $ret = $V->{F}->say(command => $cmd);
	    if($ret !~ /vol\s+size:\s+Flexible\s+volume\s+'$V->{name}'\s+has\s+size\s+($size_vol)/) {
		die ("vol size did not executed correctly");
	    }
	    return ($1);
	}
    }

=head2 get_sis_status ()

sis_status returns an array of hash , so this function is just a wrapper to access hash from the array

=over 4

Failure

Return a hash of the mapped fields

=back

=cut

    sub get_sis_status (@)
    {
	my  (%opts);
	my  ($F, $V, $Vs, $cur_stat, %options);
	my  ($ret);

	$V  = shift();
  	$F = $V->{F};

	$ret = $V->sis_status();
	return $ret;
    }

	sub get_savings (@)
    {
        my  (%opts, $V, $ret, %savings, $saved_kb_dedupe, $saved_kb_compress, $saved_kb_vbnzero, $dedupe_saved_percent, @lines);

        $V  = shift();
        %opts   = validate (@_, { df => {type=>SCALAR, optional=>1, default=>0} });
        $V->{F}->say(command => "sync;sync;sync");

	if ($opts{df} == 1){
	      $ret = $V->{F}->say(command => "df -S $V->{name}");
              @lines = split(/\n/,$ret);
              if ($lines[$#lines] =~ /^\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s*/) {
             	   %savings = (
                        used    =>  $2,
                        saved   =>  $5,
                        saved_compress => $7,
                        vbn_zero => $9,
                        );
              } elsif($lines[$#lines] =~ /^\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s+(\d+)\s+(\d+)\%\s*/) {
             	   %savings = (
                        used    =>  $2,
                        saved   =>  $5,
                        saved_compress => $7,
                        );
       		}
	}
	else {
              $ret = $V->show("privilege-level" => "test");
              if ( $ret->{'dedupe-space-saved'} eq '-' ) {
            	  $saved_kb_dedupe = 0;
		} else {
            	  $saved_kb_dedupe = ($ret->{'dedupe-space-saved'})/1024;
        	}
        
		if ( $ret->{'vbn-zero-space-saved'} eq '-' ) {
            	    $saved_kb_vbnzero = 0;
		} else {
            	  $saved_kb_vbnzero = ($ret->{'vbn-zero-space-saved'})/1024;
        	}

        	if ( $ret->{'compression-space-saved'} eq '-' ) {
            	    $saved_kb_compress = 0;
		} else {
            	    $saved_kb_compress = ($ret->{'compression-space-saved'})/1024;
        	}
     
        	if ( $ret->{'dedupe-space-saved-percent'} eq '0%' ) {
            	    $dedupe_saved_percent = 0;
        	} else {
            	    $dedupe_saved_percent = int($ret->{'dedupe-space-saved-percent'});
        	}

        	%savings = (
            		used           =>  $ret->{used},
            		saved   =>  $saved_kb_dedupe,
            		saved_compress =>  $saved_kb_compress,
	        	vbn_zero => $saved_kb_vbnzero,
	        	dedupe_saved_percentage => $dedupe_saved_percent,
        	);
	}

        $Log->debug( print Dumper($ret) );
        return \%savings;
    }

    sub sis_options_set (@)
    {
	my ($V, @ret, $value, $option, $set_op, $output, $ret_val);
	my %opts;

	$V = shift();
	my $F = $V->{F};
        %opts   = validate (@_, { value => {type=>SCALAR, optional=>1},
				  option_name => {type=>SCALAR},}
			);
	$value = $opts{value};
	$option = $opts{option_name};
	
	my $cmd = "registry walk options.".$opts{option_name};
	my $result = $F->say(command=>$cmd);
	if ($result =~ /no such option/i) {
	    die "Options $option is not supported in current ONTAP version";
	}
	if (defined($value) && ($value ne "")) {
	    $Log->comment("Setting $option to $value");
	    $set_op = $F->say(command=>"registry set options.".$option." ".$value);
	}
	$output = $F->say(command=>$cmd);
	@ret = split('=', $output);
	$ret_val = $ret[1];

	$Log->comment("Option $option value is = $ret_val");

	if (defined($value) && ($value ne "")) {
	    $ret_val = Dense::remove_leading_trailing_spaces($ret_val);
	    $value = Dense::remove_leading_trailing_spaces($value);
	    if ($ret_val ne $value) {
		die "$cmd was not set successfully : $set_op";
	    } else {
		$Log->comment("$cmd was set successfully : $value");
	    }
	}
	return $ret_val;
    }
    sub set_wp1m_delay_fiji (@)
    {
	    my $V;
	    $V = shift();
	    my %opts = @_;
	    my $sleep = $opts{sleep} ? $opts{sleep} : 60;
	    my $repeat = $opts{repeat} ? $opts{repeat} : 1;

	    my $cmd = "fiji filter \'(repeat $repeat (wafl.wp1m.delay.mdp (&& (eq \@type SHARING_MSG_COMPLETE) (eq \@vol $V->{name})) (return $sleep)))\'";
	    $V->{F}->say(command=>$cmd);
	    return 1;
    }
    sub sis_stage_fiji_set (@)
    {
         my ($V, $node, $fiji_string, $dense_stage, $wait, $repeat );

		 $V = shift();
		 
        my (%opts) = validate( @_,
            {
                dense_stage => { type => SCALAR },
                wait        => { type => SCALAR, default => 60 },
                repeat => { type => SCALAR, default => "1", optional => 1 },
                clear => { type => SCALAR, default => "1", optional => 1 },
            }
        );


		$fiji_string = "fiji filter \'(repeat $opts{repeat} (dense.sleep.stage (&& (eq \@sleep_stage $opts{dense_stage}) (eq \@vol $V->{name})) (return $opts{wait})))\'";
		
		eval {
            if ( $opts{'clear'} ) {
                $V->{F}->say( command => "fiji clear all" );
            }
			$V->{F}->say(command => $fiji_string);
		};
		if ($@) {
			die($@);
		}
		return 1;
	}
       
    sub fiji_clear (@)
    {
	my $V;

	$V = shift();
	my (%opts) = validate(@_, { slot => {type=>SCALAR, optional=>1, default=>'all'} });

	my $slot = $opts{slot};

	$V->{F}->say(command => "fiji clear $slot");
	return 1;
    }
    
    sub sis_check_records(@)
    {
        my ($V, $F, $Vs, $ret, $volname, $cmd_opt);
        my (%opts, @output, $line, %element_key, $cur_stage, %final, $got_stage, $segment,@list,@list1,%ret_output);
        my ($ver_key, $down_flag, $type, %ver_map, %map_ver);
        $V  = shift();
        %opts   = @_;
        
        $ver_key='active';        
        $down_flag = 0;
        $type="detail";
        $F  = $V->{F};
        $Vs = $opts{Vserver};
        $volname = $V->{name};
        #$final{active} = {};
        #$final{detail} = {};
        
        %ver_map = (
            'rr'    => '8.1',
            'sn'    => '8.2',
            );
        
        %map_ver = reverse %ver_map;
        
        %element_key = (
            'fingerprint'                   => 'fp',
            'fingerprint.next'		    => 'fp_next',
            'fingerprint.vvol'		    => 'fp_vvol',
            'fingerprint.vvol.next'	    => 'fp_vvol_next',
            'fingerprint.vvol.delta'	    => 'fp_vvol_delta',
            'fingerprint.vvol.delta.old'    => 'fp_vvol_delta_old',
            'fingerprint.vvol.delta.dwx'    => 'fp_vvol_delta_dwx',
            'fingerprint.vvol.temp'	    => 'fp_vvol_temp',
            'fingerprint.vvol.gen'	    => 'fp_vvol_gen',
            'fingerprint.stale'		    => 'fp_stale',
            'fingerprint.stale.sorted'	    => 'fp_stale_sorted',
            'fingerprint.stale.sorted.old'  => 'fp_stale_sorted_old',
            'clogdir.0'                     => 'cl_0',
            'clogdir.1'                     => 'cl_1',
            'gather'		            => 'gather',
            'duplicate.sorted'		    => 'dedup_sort',
            'stale_fbns'                    => 'stale_fbns',
            'stale_fbns.old'                => 'stale_fbns.old',
            'stale_fbns.out'                => 'stale_fbns.out',
            'fingerprint.rr'            => 'fp_rr',
            'fingerprint.rr.vvol'       => 'fp_rr_vvol',
            'fingerprint.vvol.rr.rv'       => 'fp_vvol_rr_rv',
            'fingerprint.rr.vvol.delta' => 'fp_rr_vvol_delta',
            'fingerprint.vvol.delta.rr.rv' => 'fp_vvol_delta_rr_rv',
            'clogdir.0.rr'              => 'cl_0_rr',
            'clogdir.1.rr'              => 'cl_1_rr',
            'clogdir.0.rr.rv'           => 'cl_0_rr_rv',
            'clogdir.1.rr.rv'           => 'cl_1_rr_rv',
            
            );


        $cmd_opt = $opts{'verbose'} ? "-cv" : "-c";
   
        $ret = $F->say(command => "sis check $cmd_opt /vol/$volname");
        @output = split("\r*\n", $ret);
        foreach $line (@output) {
            if ($line =~ "Checking Semi Upgraded"){
                print "Semi upgraded file: $line\n";
            } else {
                # Active metafile records
                if ( $line =~ /^Checking\s+(\S+)\s+/ ) {
                    my $temp = $1;
                    if ($line =~ /^Checking\s+Downgraded\s+temporary\s+metafiles\s+\((\S+)\)/){
                        if (!$opts{'downgraded'}) {
                                last;
                            }
                        $ver_key = $1;
                        $down_flag = 1;
                        print "Downgraded metafile: $line\n";
                    } elsif ( $line =~ /(\d+)\s+records/i ) {
                        my $rec_val = $1;
                        $cur_stage = $temp;
                        if($cur_stage !~ /\.rv/) {
                            $ver_key = 'active';
                        }
                        if($cur_stage =~ /\S+\.\d+\.(\S+)\.rv/){
                            $ver_key = $ver_map{"$1"};
                            $down_flag = 1;
                        }
                        
                        ## below condition is to remove cl_0_rr_rv coming from reverted metafiles
                        ## changing from $final->{8.1}->{'fp_vvol_rr_rv'} to $final->{8.1}->{'fp_vvol'}
                        ## Just to maintain agnostic.
                                                
                        if($opts{'downgraded'} && ($ver_key =~ /\d+/)) {
                            my $str = $element_key{$cur_stage};
                            my $str1 = "_".$map_ver{"$ver_key"}."_rv";
                            $str =~ s/$str1//g;
                            $final{$ver_key}{$str} = $rec_val;
                        } else {
                            $final{$ver_key}{$element_key{$cur_stage}} = $rec_val;    
                        }
                        
                    }
                } elsif ($line =~ /Number of Segments:\s+(\d+)/ ) {
                    $final{$ver_key}{'total_segments'} = $1;
                } elsif ($line =~ /Number of Records:/ ) {
                    @list = map {/(\d+)/} split (/\s+/,$line);
                } elsif ($line =~ /^\s+\d+/) {
                    @list1 =map {/(\d+)/} split (/\s+/,$line);
                    push (@list, @list1);
                 }
            }

            if (($#list >= 0) && ($final{$ver_key}{'total_segments'} == ($#list + 1))) {
                   $final{$ver_key}{'segment_list'} = \@list;
             }

        }
        

        if ( $opts{verbose}) {
            print "Verbose value set\n";
            foreach $line (@output) {
                if ( $line =~ /Number of|contents of/ ) {
                   next;
                } elsif ( $line =~ /^Checking\s+(\S+)\s+/ ) {
                    my $temp = $1;
                        if ( $line =~ /(\d+)\s+records/i ) {
                                $got_stage = $temp;
                        }
                        if($got_stage !~ /\.rv/) {
                            $ver_key = 'active';
                            #$down_flag = 0;
                        }
                        #This part of code requires later when we fix the 7 mode library
                        #Keeping this code for future use.
                        #if($got_stage =~ /\S+\.\d+\.(\S+)\.rv/){
                        #    $ver_key = $ver_map{"$1"};
                        #    $down_flag = 1;
                        #}
                        if ($line =~ /^Checking\s+Downgraded\s+temporary\s+metafiles\s+\((\S+)\)/) {
                            if (!$opts{'downgraded'}) {
                                last;
                            }
                        $ver_key = $1;
                        $down_flag = 1;
                    }
                    next;
                } elsif ($line =~ /Contents of Segment:\s+(\d+)/ ) {
                    $segment = $1;
                    next;
                } elsif ($line =~ /data log file:\s+(\d+)/ ) {
		            $final{$ver_key}{'data_clog'} = $1;
		            next;
		        } elsif ($line =~ /reference log file:\s+(\d+)/ ) {
		            $final{$ver_key}{'ref_clog'} = $1;
		            next;
		        } elsif ($line =~ /pre-processed transfer log file:\s+(\d+)/ ) {
		            $final{$ver_key}{'vault_clog'} = $1;
		            next;
		        }
       if ( $line =~ /^\s*(\w+)\s+(\d+)\s+(\d+)$/ ) {
            print "Line obtained is $line \n";
            if ($got_stage eq "fingerprint") {
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{fp}},$1;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{vvbn}},$2;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{cp_cnt}},$3;
            } elsif ($got_stage eq "fingerprint.vvol" || $got_stage eq "fingerprint.stale.sorted") {
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{vvbn}},$2;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{cp_cnt}},$3;
            } else {
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{vvbn}},$2;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{cp_cnt}},$3;
            }
        } elsif ( $line =~ /^\s*(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/ ) {
            print "Line obtaines is $line \n";

                if ($got_stage eq "fingerprint.rr" || $got_stage eq "fingerprint") {
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{fp}},$1;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{inode}},$2;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{fbn}},$3;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{wi_gen}},$4;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{cp_cnt}},$5;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{$segment}->{compress_attempted}},$6;
                } elsif ($got_stage eq "fingerprint.rr.vvol" || $got_stage eq "fingerprint.rr.vvol.delta" || $got_stage eq "clodir.0.rr" || $got_stage eq "clogdir.1.rr" || $got_stage eq "fingerprint.vvol" || $got_stage eq "clogdir.0" || $got_stage eq "fingerprint.vvol.delta" || $got_stage eq "clogdir.1" ) {
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fp}},$1;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{inode}},$2;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fbn}},$3;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{wi_gen}},$4;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{cp_cnt}},$5;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{compress_attempted}},$6;
                } else {
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fp}},$1;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{inode}},$2;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fbn}},$3;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{wi_gen}},$4;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{cp_cnt}},$5;
                    push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{compress_attempted}},$6;
                }
       } elsif ( $line =~ /^\s*(\w+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)$/ ) {
            print "Line obtaines is $line \n";
            if ($got_stage eq "clogdir.0" || $got_stage eq "clogdir.1") {
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{inode}},$2;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fbn}},$3;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{vvbn}},$4;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{wi_gen}},$5;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{cp_cnt}},$6;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{compress_attempted}},$7;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{unprocessed}},$8;
            } else {
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fp}},$1;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{inode}},$2;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{fbn}},$3;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{vvbn}},$4;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{wi_gen}},$5;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{cp_cnt}},$6;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{compress_attempted}},$7;
                push @{$ret_output{$ver_key}{$type}{$element_key{$got_stage}}->{unprocessed}},$8;
            }
        }
        
         
    }
        ##Below code will return the hash like $final{'detail'}, instead of $final{'active'}{'detail'} and
        ## for reverted metafiles $final{'8.1'}{'detail'}
        print Dumper(\%ret_output);
        $final{$type} = $ret_output{'active'}{$type};
        if($opts{'downgraded'} && $down_flag) {
            if ($ver_key =~ /^\d+/){
                $final{$ver_key}{$type} = $ret_output{$ver_key}{$type};
            } else {
                print "Something wrong here...";
            }
        }
    }
        print Dumper(\%final);
        
        return \%final;
    }
 

    sub set_overlap_request(@)
    {
        my ($V, %opts, $cmd);
   
        $V    = shift();
        %opts = @_;     
        
        $cmd = "registry set options.sis.overlap_requests $opts{state}";

        $V->{F}->say(command => $cmd);

    }
    sub wafl_stat_counter(@) {
	my ($F);

	my $V = shift();
        my (%opts) = validate (@_, { flags      => { type=>ARRAYREF } });
	
	$F = $V->{F};
	my $FLAGS       = $opts{'flags'};
	my @values;
	my $wafl_stats = $F->say(command => "wafl stats");
	my @array =  split(/\r*\n/,$wafl_stats);
	foreach my $flag (@$FLAGS) {
	    foreach my $line (@array) {
		if ($line =~ /$flag/) {
		    my @value = split(/=/, $line);
		    $value[1] = Dense::remove_leading_trailing_spaces($value[1]);
		    push (@values, $value[1]);
		    last;
		}
	    }
	}
	$Log->comment("Array returned is @values");
	return \@values;
    }

    sub verify_bcmp_counter (@) {
	    return 1;
    }
    sub wafl_stat_zero (@) {
	my $F;

	my $V = shift();
	$F = $V->{F};

	$F->say( command=>"wafl stats zero" );
    }

=head2 set_wafl_scan_speed ()

Function to set the wafl scan speed to the given value

=over 4

=item speed

scan speed which we want to set

=over 4

Failure

Return with an error message if command is not able to execute correctly

=back

=cut

    sub set_wafl_scan_speed(@) {
        my ($F);
	my $V = shift();

	my (%opts) = validate (@_, { speed	=> { type=>SCALAR }});
	$F = $V->{F};
	my $scan_speed = $opts{'speed'};
	
	eval {
	    $F->say( command=>"wafl scan speed $scan_speed ");
	};

	if ($@) {
	    die($@);
	}
    }


=head2 get_wafl_scan_speed ()

Function to get the wafl scan speed value

=over 4

Failure

Return with an error message if command is not able to execute correctly

=back

=cut

    sub get_wafl_scan_speed(@) {
	my ($F);
	my $V = shift();

	$F = $V->{F};
	my ($scanner_speed, $err_msg);
	my $output = $F->say( command=>"wafl scan speed");

	if ( $output =~ m/WAFL\s+scan\s+speed\s+is\s+([\d]+)/i ) {
	    $scanner_speed = $1;
	    Dense::remove_leading_trailing_spaces ($scanner_speed);
	    $err_msg = "";
	} else {
	    $scanner_speed = -1;
	    $err_msg = "Could not retrieve wafl scan speed, error: $output";
	}
	return ($scanner_speed, $err_msg);
    }

    sub sis_verify_changelog (@)
    {
	my ($V, %opts, @clog, $clog_entry);

	$V = shift();
    my $vname = $V->{name};
    my $vpath = $V->{params}{path};
    $Log->comment("Volume name is $vname");
    
	%opts = validate (@_, { records => { type=>SCALAR },
				timeout => { type=>SCALAR, default=>'90' },
				}
			);
	my $timeout = time() + $opts{timeout};
	my $records = $opts{records};

	while( time() < $timeout ) {
	    @clog = $V->sis_check_records();
	    $clog_entry = $clog[0]->{'active'}->{cl_1} ? $clog[0]->{'active'}->{cl_1}:$clog[0]->{'active'}->{cl_0};
	    if ($clog_entry >= $records) {
		return;
	    }
	    sleep 5;
	}
	die "Changelog file is not populated with all written records";
    }

    sub set_fiji (@)
    {
	my ($V, %opts, $cmd);
	$V = shift();
	%opts = validate(@_, { fiji_stage => { type=>SCALAR, optional => 1},
                           stage=>{type=>SCALAR, optional => 1 },
                           name=>{type=>SCALAR, optional => 1},
        			       return => { type=>SCALAR  },
                           repeat => { type=>SCALAR, optional=>1, default=>1 },
			       } );
	my $fiji_stage = $opts{fiji_stage};
        my $stage = $opts{stage};
        my $name= $opts{name};
	my $ret_value = $opts{return};
	my $vol_name = $V->{name};
        if ($stage) {
            
            $cmd = "fiji filter \'(repeat $opts{repeat} ($name (&& (eq \@sleep_stage $stage) (eq \@vol $vol_name)) (return $ret_value)))\'";
        } else {
            $cmd = "fiji filter \'(repeat $opts{repeat} ($fiji_stage (eq \@vol $vol_name) (return $ret_value)))\'";
        }
	eval {
            my $ret = $V->{F}->say(command=>$cmd);
	};
	if( $@ ){
	    die ($@);
        }
	return 1;
    }

    sub set_ric_fiji (@) {
	my ($V, $F);
	$V = shift();
	$F = $V->{F};
	$V->sis_start();
    # To ensure that CP gets elongated in block sharing phase, wait for Saving_Sharing phase
    $V->sis_wait_for(stage=>"SAVING", 'sub-stage'=>"SAVING_SHARING", timeout=>120);
	my $cmd = "fiji filter \'(wafl.wp1m.delay.mdp (&& (eq \@type WAFL_CLEAN_ALLOC_FILE) (eq \@vol $V->{name}) (eq \@fileid 73)) (return 200))\';mkfile 4k /vol/$V->{name}/file2;sync";
	$F->say(command=>$cmd);
	return 1;
    }

    sub sis_reset (@) {

        my  ($V,%opts,$vol_path,$metafile,$location);
        my  ($cmdstring);
        $V = shift;

        %opts = validate (@_, {
                                vol_path => { type=>SCALAR},
                                metafile => { type=>SCALAR},
                                location => { type=>SCALAR},
                                }
                          );

        $vol_path = $opts{vol_path};
        $metafile = $opts{metafile};
        $location = $opts{location};

        $cmdstring = "sis reset $vol_path $metafile $location)";
        eval {
            $V->{F}->say(command => $cmdstring);
        };
        if ($@) {
            die($@);
        }
   }

    sub get_sis_stat_global() {
	my ($self, $ret);
	$self = shift();

	@$ret = $self->{F}->volume_efficiency_stat(g=>'true');
	foreach my $line (@$ret) {
	    print Dumper ($line);
	    if ($line->{node} eq $self->{F}{name}) {
		return $line;
	    }
	}
	die "Did not get sis stat -g counters for node : $self->{F}{name} \n";
    }

    sub sis_checkpoint (%) {
        my ( $V, %opts );
        $V    = shift();
        %opts = validate( @_, {
                stage => { type => SCALAR },
                sub_stage => { type => SCALAR, optional => 1 },
                checkpoint_timeout => { type => SCALAR, optional => 1, default => 60 },
                sleep  => { type => SCALAR, optional => 1, default => 0 },
                repeat => { type => SCALAR, optional => 1, default => 1 },
                scan   => { type => SCALAR, optional => 1, default => 'true' },
                delete_cp   => { type => SCALAR, optional => 1, default => 'false' },
                zapi   => { type => SCALAR, optional => 1, default => 0 },
                wafl_scan_speed => { type => SCALAR, optional => 1, default => 10 },
            });

        my $F         = $V->{F};
        my $vol_name  = $V->{name};
        my $vol_path  = $V->{params}{path};
        my $stage     = $opts{stage};
        my $sub_stage = $opts{sub_stage};
        my $ckpt_time = $opts{checkpoint_timeout};
        my $sleep     = $opts{sleep};
        my $swss      = $opts{wafl_scan_speed};

        # Compute the next stage for a given stage.
        my %checkpoint_stages = (
            GATHERING   => "Gathering_phase2",
            SORTING     => "Sorting",
            DEDUPING_P1 => "Saving_pass1",
            DEDUPING_P2 => "Saving_pass2",
            DEDUPING_SHARING => "Saving_sharing",
            CHECKING_P1 => "Checking_pass2",
        );

        my $value = Dense::Constant::Command::DENSE_CKPT_STAGE_VALUE->{$stage};
        my ( $fiji_stringr, $WSS, $err, $CPIV, $errs, %checkpoint_stats );
        my $key  = 'options.sis.checkpoint_interval';
        my $keyv = $ckpt_time;
        eval {
            if ( $stage eq "GATHERING" ) {
                $V->sis_stage_fiji_set(
                    dense_stage => "DENSE_STAGE_GATHERING",
                    wait        => ($ckpt_time - 10),
                );

                $CPIV = $F->registry_get( key => $key );
                $Log->comment( "The default value of $key is $CPIV, setting it to $keyv");
                $F->registry_set(
                    key   => $key,
                    value => $keyv
                );

                ( $WSS, $err ) = $V->get_wafl_scan_speed();
                $Log->comment( "The default value of wafl scan speed is $WSS, setting the speed to $swss");
                $V->set_wafl_scan_speed( speed => $swss );
            } elsif ($sub_stage eq "BUCKET_SORT") {
                my $fiji_string = "fiji filter \'(repeat 3 (dense.delay (eq \@FUNC_NAME bucket_sort_worker_sort)(sleep $ckpt_time)))\'";
                $F->say( command => "fiji clear all" );
                $F->say( command => $fiji_string );

                $CPIV = $F->registry_get( key => $key );
                $Log->comment( "The default value of $key is $CPIV, setting it to $keyv");
                $F->registry_set(
                    key   => $key,
                    value => 60, 
                );

            } elsif ( $stage eq "CHECKING_P1" ){
                $value--;
                $F->say( command => "fiji clear all" );
                $V->set_fiji(name=>"dense.sleep.stage", stage=> "DENSE_STAGE_CHECKING_P1", return=>60);
            } elsif ($stage eq "DEDUPING_P2") {
                my $fiji_string = "fiji filter \'(repeat $opts{repeat} (dense.sleep.before.record.checkpoint (&& (eq \@sleep_stage DENSE_STAGE_DEDUPING_P1) (eq \@vol $vol_name)) (sleep $ckpt_time)))\'";
                $F->say( command => "fiji clear all" );
                $F->say( command => $fiji_string );
            } else {
                my $fiji_string = "fiji filter \'(repeat $opts{repeat} (dense.sleep.before.record.checkpoint (&& (eq \@sleep_stage DENSE_STAGE_$stage) (eq \@vol $vol_name)) (sleep $ckpt_time)))\'";
                $F->say( command => "fiji clear all" );
                $F->say( command => $fiji_string );
            }

            $F->sync();
            $Log->comment( "Stage to create checkpoint is $stage and equivalent value is $value");

            my ( $checkpoint_stage, $stage_expected );
            if ( $value == 2 ) {
                #$checkpoint_stage = "-";
                $checkpoint_stage = "Gathering_phase2";
                $stage_expected   = "Gathering";
            } elsif ( $value == 3 ) {
                $checkpoint_stage = "Sorting";
                $stage_expected   = "Sorting";
            } elsif ( $value == 4 ) {
                $checkpoint_stage = "Saving_pass1";
                $stage_expected   = "Saving";
            } elsif ( $value == 5 ) {
                $checkpoint_stage = "Saving_pass2";
                $stage_expected   = "Saving";
            } elsif ( $value == 6 ) {
                $value++;
                $checkpoint_stage = "-";
                $stage_expected   = "Checking_P1";
            } elsif($stage eq "DEDUPING_SHARING") {
                $checkpoint_stage = $checkpoint_stages{$stage};
                $stage_expected = "Saving";
            } else {
                die("Unable to get checkpoint stage value \n");
            }

            if ( $opts{'zapi'} ) {
                $V->zapi_sis_start(
                    'scan'              => $opts{'scan'},
                    'delete-checkpoint' => $opts{'delete_cp'}
                );
            } else {
                $V->sis_start(
                    'scan'              => $opts{'scan'},
                    'delete-checkpoint' => $opts{'delete_cp'}
                );
            }

            my ( $status_info, $stage_curr, $checkpoint_stage_curr, $status );
            if ( $stage eq "GATHERING" ) {
                $V->sis_wait_for( stage => "GATHERING" );
            } elsif ( $stage =~ /CHECKING_P1/ ) {
                $V->sis_wait_for();
                $V->sis_check();
            }

            $status_info           = $V->sis_status();
            $status                = $status_info->{'op-status'};
            $stage_curr            = $status_info->{'stage'};
            $checkpoint_stage_curr = $status_info->{'checkpoint-stage'};

            $Log->comment( "Dense stage is :$stage_curr ,Checkpoint stage is :$checkpoint_stage_curr, Expecting dense stage as $stage_expected");
            until ( ( ( $stage_curr =~ /$stage_expected/i ) && ( $checkpoint_stage_curr =~ /$checkpoint_stage/i )) || ( $status eq "Idle" )) {
#$F->sync();
                $status_info           = $V->sis_status();
                $status                = $status_info->{'op-status'};
                $stage_curr            = $status_info->{'stage'};
                $checkpoint_stage_curr = $status_info->{'checkpoint-stage'};
                $Log->comment( "Dense stage is :$stage_curr ,Checkpoint stage :$checkpoint_stage_curr, SIS status =$status, Expecting dense stage :$stage_expected ");
            }

            if ( $status ne "Idle" ) {
                $V->sis_stop();
                $V->sis_wait_for();
            } else {
                die "Restart Checkpiont is not created for the stage [$stage] \n";
            }

            $status_info = $V->sis_status();
            $Log->comment("Checking status_info output: ".Dumper($status_info));
            $checkpoint_stats{'Checkpoint Op Type'} = $status_info->{'checkpoint-op-type'};
            $checkpoint_stats{'Checkpoint stage'} = $status_info->{'checkpoint-stage'};
            $checkpoint_stats{'Checkpoint Sub-stage'} = $status_info->{'checkpoint-sub-stage'};
            $checkpoint_stats{'Checkpoint Progress'} = $status_info->{'checkpoint-progress'};
            $Log->comment( "dense_create_checkpoint : Checkpoint stats are: \n" . Dumper( \%checkpoint_stats ) );

            if ( $checkpoint_stats{'Checkpoint stage'} ne $checkpoint_stages{$stage} ) {
                die( "Checkpoint is created for different stage : ".$status_info->{'checkpoint-stage'});
            }
        };
        if ($@) {
            $errs = $@;
        }

        if ( $stage eq "GATHERING" ) {
            $Log->comment("Restoring the default value of $key to $CPIV");
            $F->registry_set(
                key   => $key,
                value => $CPIV
            );

            $Log->comment("Restoring the default value of wafl scan speed to $WSS");
            $V->set_wafl_scan_speed( speed => $WSS );
        }

        if ($errs) {
            die($errs);
        }
        return ( \%checkpoint_stats );
    }


    sub clone_status(@) {

        my ($V, $file_inode, %opts, $volume, $file, $start_fbn, $fbn_cnt, $timeout, $cmd);
        $V = shift();
        %opts = validate ( @_, { 
                                  file => { type => SCALAR },
                                  start_fbn => { type => SCALAR, optional => 1, },
                                  fbn_cnt => { type => SCALAR, optional => 1, },
                                  timeout => { type => SCALAR, optional => 1, },
                                }
                            );
                            
        $volume = $V->{name};
        $file = $opts{file};
        $start_fbn = $opts{start_fbn};
        $fbn_cnt = $opts{fbn_cnt};
        $timeout = $opts{timeout};
        
        $timeout = 3600 unless $timeout;
        
        $cmd = "priv set test; slist /vol/$volume/$file";
        my $return_text = $V->{F}->say( command => $cmd );
        if( $return_text =~ m/(\d+)\s+(\d+)/ ) {
            $file_inode = $1;
        }
        
        $cmd = "priv set test; cmf_test $volume";
        if ( defined($start_fbn) && defined($fbn_cnt) ) {
            $cmd = $cmd . " " . "lookup " . $file_inode . " " . $start_fbn . " " . $fbn_cnt;
            $return_text = $V->{F}->say(command => $cmd);
            if ( $return_text =~ m/Key not found/ ) {
                $Log->comment("Clone operation has completed");
                return 1;
            } else {
                 return 0;
            }
        } else {
            $cmd = $cmd . " " . "list";
            $return_text = $V->{F}->say(command => $cmd);;
            if($return_text =~ m/Clone-inum = $file_inode/) {
                return 0;
            } else {
                $Log->comment("Clone operation has completed");
                return 1;
           }
        }
    }


    sub clone_wait_for_complete(@) {
        my ($V, %opts, $timeout, $wait_time, $ret );
        $V = shift();

        %opts = validate (@_, { 
                                file => { type => SCALAR },
                                start_fbn => { type => SCALAR, optional => 1, },
                                fbn_cnt => { type => SCALAR, optional => 1, },
                                timeout => { type => SCALAR, optional => 1, },
                                wait_time => { type => SCALAR, optional => 1, },
                                }
                        );
       
        $timeout    = delete $opts{timeout};
        $wait_time  = delete $opts{wait_time};

        $timeout    = 3600 unless $timeout;
        $wait_time  = 2 unless $wait_time;

         while($timeout >0) {
            $ret = $V->clone_status(%opts);
            if ($ret == 1) {
                last;
            }
            sleep $wait_time;
            $timeout = $timeout - $wait_time;
         }
         if($timeout <=0) {
            die ("sis clone has not completed yet");
         }
    }


    sub sis_start_schedule(@) {
        my ($V, $Vs, $ret, %opts);
        $V = shift;

        %opts = validate (@_, { 
                                complete => { type => SCALAR, optional => 1, },
                              }
                        );
        
        $Vs = $V->{Vs};

        $ret = $Vs->sis_policy_sched_create(min => 1);

        $V->sis_stage_fiji_set(dense_stage => "DENSE_STAGE_SORTING");
        
        $V->sis_config(policy => $ret->{'policy_name'}); 

        $V->sis_wait_for(status => 'ACTIVE', timegap => 1);
        
        $V->sis_config(schedule => 'sun-sat@0'); 

        if ($opts{complete}) {
            $V->sis_wait_for();
            $V->{F}->fiji_clear();
        }

        return $ret;
    } 

    sub sis_start(@) {
        my ($self, $ret);
        $self = shift();
        my %opts = @_;
        if (defined($opts{scan})) {
                $opts{'scan-old-data'} = delete($opts{scan});
        }
        $ret = $self->{F}->volume_efficiency_start(volume=>$self->{name}, vserver=>$self->{Vs}{name}, %opts);
        return $ret;
    }
    sub verify_queue(@) {
        my ($V, $flag);
        $V = shift;

        my @queues = @_;

        $V->{F}->set_privilege( level => "test" );
        foreach my $queue (@queues) {
            $flag = 0;
            my $ret = $V->{F}->volume_efficiency_show_queue_show(node=>"local",queue => $queue);
            print Dumper($ret);
            my @arg = split(/\s+/,$ret->{'queue-output'});
            for (my $i=8; $i<=$#arg;$i++) {
                print "Queue : $arg[$i]\n";
                if ($arg[$i+1] =~ /Process/) {$i=$i+3;} else {$i=$i+2;}
                print "Volume : $arg[$i]\n";
                if ($arg[$i] =~ /$V->{name}/) {
                    $flag++ ;
                    $Log->comment("Volume $V->{name} is found in $queue queue");
                }
            }
            if ($flag == 0) {
                die("Volume $V->{name} is not found in $queue queue");
            }
        }
        return $flag;        
    }

    sub sis_ls(@) {
        my ($V,%opts);
        my %metafile;
        $V = shift;

        my $volpath="/vol/".$V->{name};

        my ($return_text, $result_code) = $V->{F}->say(command =>"sis ls $volpath");
        return undef if(!$return_text);

        my @lines=split(/\n/, $return_text);
        shift(@lines);
        foreach(@lines) {
            next if(!/\S/); #Check if it is an empty line
            if(/Total Blocks Used By $V->{name} sis Metafiles on the aggregate/) {
                $metafile{TotalBlocksUsed}=(split)[-1];
            } else {
                my ($name, $size, $blocks)=split;
                $metafile{$name}{size}=$size;
                $metafile{$name}{blocks}=$blocks;
            }
        }
        return(\%metafile);
    }
    
    return 1;
}
