{ package   Dense::Agnostic::Cmode::Volume;
    use strict;
    use warnings;

    use Tharn   qw($Log);

    use Dense::Agnostic::Constants;
    use NATE::Exceptions::Argument qw(:try);
    use NACL::APISet::Exceptions::NoMatchingEntriesException ();

    use Params::Validate qw(
            validate
            SCALAR
    );

    sub snap_list_b (@)
    {
        my  ($V, $ret);
	my @result = ();

        $V  = shift();
        $ret = $V->snap_list_owners();
	
		foreach my $entry (@$ret) {
			my %snap;
			$snap{'name'} = $entry->{'snapshot'};
			$snap{'owner'} = $entry->{'owners'};
			push (@result, {%snap});
		}

        return @result;
    }

    sub snapshot_create (%) 
    {
        my $V = shift;
        my (%opts) = validate(@_, {
                    snapname    => { type => SCALAR },
                    options     => { type => SCALAR, optional => 1 }
                } );

        my $snapname = $opts{snapname};

        # Create a volume snapshot.
        $Log->comment("Creating snapshot on volume $V->{name}");
        $V->snap_create( 'volume' => $V->{name}, 'vserver' => $V->{Vs}->{name}, 'snapshot' => $snapname);

        # Verify snapshot was created.
        $Log->comment("Verify that snapshot $snapname was successfully created");
        my @output = $V->snap_show( 'volume' => $V->{name}, 'vserver' => $V->{Vs}->{name}, 'snapshot' => $snapname);
        $Log->debug( print Dumper(@output) );

        if ( !( scalar @output ) ) {
            $Log->error("Snapshot $snapname for volume $V->{name} was not found.");
            return 0;
        }

        return 1;
    }

    sub snap_delete (%)
    {
	    my  ($V, $snapname, %opts, $ret);

	    $V = shift();
	    %opts = validate (@_, { Snapname => {type=>SCALAR},
			    	    Options => { type=>SCALAR, optional=>1 } }
			      );
	    $snapname = $opts{Snapname};
	    my $options = $opts{Options};

	    if( $options eq "-a" ){
		    $ret = $V->snap_delete_opts(snapshot=>"*",force=>"true");
	    }

	    if( !defined($opts{Options})){
		    $ret = $V->snap_delete_opts(snapshot=>$snapname,force=>"true");
	    }
            if (! $ret) {
                die("snap delete of snapshot $snapname failed on $V->{name}");
            }
    }	

    sub nosnap (%)
    {
	    my ($V, $F, $snapname, $ret, $key, $policy);

	    $V = shift();
	    $F = $V->{F};
        try {
	        $policy = $F->volume_snapshot_policy_show(fields=>"policy", policy=>"nosnap");
        }
        catch NACL::APISet::Exceptions::NoMatchingEntriesException with {
            my $exception_object = shift;
            my $type = ref $exception_object;
            my $error = $exception_object->text();
            if ($error =~ /There are no entries matching your query/) {
	                $F->volume_snapshot_policy_create(policy=>"nosnap", enabled=>"true", schedule1=>"weekly", count1=>"0");
            } else {
                $Log->exit();
                $exception_object->throw();
            }
        };
		$Log->comment("Setting no snapshot policy on the volume");
        $V->modify('snapshot-policy'=>'nosnap');
        return 1;
    }


    return 1;
}
