#$Id: //depot/prod/test/Rsierranevada/lib/Dense/Agnostic/Cmode/Volume/Utils.pm#2 $
{ package   Dense::Agnostic::Cmode::Volume;
    use strict;
    use warnings;

    use Tharn   qw($Log);
    use Params::Validate qw(validate SCALAR ARRAYREF HASHREF);
    use Data::Dumper;
    use NACL::C::Node;
    use NACL::MTask::EventLogDetector;
    use NACL::APISet::Exceptions::ResponseException qw(:try);
    use Storage::UiUtils;
    use Server;
    use Dense::Constants;
    use Dense::Constant::Command;
    use Dense::Constant::Message;
    use Dense::Constant::Clone_Command;
    use Dense::Constant::Clone_Message;
    use Dense::Agnostic::Constants;
    use Dense::Agnostic::Cmode;
    use Dense::Agnostic::Client;
    use Dense::Utils;
    use File::Find;
    use File::Spec;
    
    sub is_dense_volume (@)
    {
        my ($V, $ret);
        $V  = shift();
        $ret = $V->show();
        $Log->comment("volume show returned: ".Dumper($ret));
        return($ret->{'is-sis-volume'} eq 'true');
    }

    sub is_dense_enabled (@)
    {
        my ($V, $err, $ret);
        $V  = shift();
        eval {
        $ret = $V->sis_show();
        };
        $err = $@;
        $Log->comment("volume efficiency show returned: ".Dumper($ret));
        return ($err?0:1);
    }

sub stats_show (@)
{
        my ($vol, %vol_stats, $stats_cmd);
        $vol = shift;
        my (%opts) = validate   (@_, {  
                                        timeout         => {type=>SCALAR, default=>3600, optional=>1},
                                      }
                                  );
        my $timeout     = $opts{'timeout'};
        my $volname     = $vol->{name};
        
        $stats_cmd = "stats show volume:$volname";
        my ($stats_text, undef) = $vol->{F}->say(command=>$stats_cmd);
        if ($stats_text =~ /No instance|invalid object name|Unknown/i) {
                die fatal_result ("Could not get volume stats: $stats_text");
        }
        my @vol_stats_output = split(/\n/,$stats_text);
        foreach my $stats_line (@vol_stats_output) {
                my (undef, $name, $key, $value) = split(/:/, $stats_line);
                if (defined($name) && defined($key) && defined($value)) {
                $vol_stats{$key} = $value;
                }
        }
    if (!%vol_stats) {
        die fatal_result ("Could not get volume stats");
    }
    return \%vol_stats;
}

    sub wafl_showvbns (@)
    {
        my ($vol, $ret, $path, $options, %opts, %vbns, @lines, @tmp);
        $vol = shift;
        (%opts) = validate(@_, {
                                 file_name=>{type=>SCALAR},
                                 fbn_range=>{type=>SCALAR | ARRAYREF, optional=>1},
                                 option => {type=>SCALAR, optional=>1}
                               });

        # Since the 7mode function dense_parse_showvbns is taking file_name as path, we had to do this way here
#       $path = "/vol/$vol->{name}/$opts{file_name}";
        $path = $opts{file_name};
	$options = $opts{option};
        
        if($opts{fbn_range}) {
           my $fbn_range = $opts{fbn_range};
           if(ref($fbn_range) eq "ARRAY") {
              foreach(@$fbn_range) {
                 $ret = $vol->{F}->say(command=>"wafl showvbns $options -s $path $_");
                 push(@lines, $ret);
              }
           } else {
              $ret = $vol->{F}->say(command=>"wafl showvbns $options -s $path $fbn_range");
              @lines = ($ret);
           }
        } else {
           $ret = $vol->{F}->say(command=>"wafl showvbns $options -s $path");
           @tmp = split(/\n/, $ret);
           @lines = grep(/^fbn/,@tmp); # The first and last lines are sometimes empty in cmode
        }

        $Log->comment("wafl showvbns returning after  ".Dumper(\@lines));
        foreach(@lines) {
           if(/fbn (\d+) is vbn (\d+), vvbn (\d+), refcount (\d+)/) {
              $vbns{$1}           = {};
              $vbns{$1}{fbn}      = $1;
              $vbns{$1}{vbn}      = $2;
              $vbns{$1}{vvbn}     = $3;
              $vbns{$1}{refcount} = $4;
              $Log->comment("DEBUG: Storing showvbns {$1}{fbn} = $vbns{$1}{fbn} -- {$1}{vbn} = $vbns{$1}{vbn}, {$1}{vvbn} = $vbns{$1}{vvbn} , {$1}{refcount} = $vbns{$1}{refcount}");
           }elsif(/fbn (\d+) is vbn (\w+), vvbn (\w+), refcount (\d+)/) {
              $vbns{$1}           = {};
              $vbns{$1}{fbn}      = $1;
              $vbns{$1}{vbn}      = $2;
              $vbns{$1}{vvbn}     = $3;
              $vbns{$1}{refcount} = $4;
              $Log->comment("DEBUG: Storing showvbns {$1}{fbn} = $vbns{$1}{fbn} -- {$1}{vbn} = $vbns{$1}{vbn}, {$1}{vvbn} = $vbns{$1}{vvbn} , {$1}{refcount} = $vbns{$1}{refcount}");
           } else {
              die("Invalid showvbns output format:  Obtained => $_");
           }
        }
        $Log->comment("wafl showvbns returning ".Dumper(\%vbns));
        return(%vbns);
    }

    sub wafl_punch_hole (@)
    {
        my ($vol, $path, %opts, $cmd, $ret);
        $vol = shift;
        (%opts) = validate(@_, {
                                file   => { type=>SCALAR },
                                fbn    => { type=>SCALAR },
                                count  => { type=>SCALAR },
                               });

        $path = "/vol/".$vol->{name}."/".$opts{file};
        $cmd = "wafl punch $path ";
        $cmd .= $opts{fbn}." ";
        $cmd .= $opts{count}." ";

        $ret = $vol->{F}->say(command=>$cmd);
        die("wafl punch failed with $ret") if($ret !~ /wafl punch: status=0/);
    }

    sub file_punch_hole (@)
    {
        my ($vol, $path, %opts, $cmd, $ret);
        $vol = shift;
        (%opts) = validate(@_, {
                                file   => { type=>SCALAR },
                                offset    => { type=>SCALAR },
                                count  => { type=>SCALAR },
                               });

        $path = "/vol/".$vol->{name}."/".$opts{file};
        $cmd = "file punchhole $path ";
        $cmd .= $opts{offset}." ";
        $cmd .= $opts{count}." ";

        $ret = $vol->{F}->say(command=>$cmd);
        die("file punch failed with $ret") if ($ret != 0);
    }

    sub get_inode_num (@)
    {
        my ($vol, $path, %opts, $cmd, $ret, $hex_num);
        $vol = shift;
        (%opts) = validate(@_, {
                                file_name   => { type=>SCALAR },
                               });

        # Since the 7mode function dense_get_inode_num is taking file_name as path, we had to do this way here
#         $path = "/vol/".$vol->{name}."/".$opts{file};
        $path = $opts{file_name};
        $cmd = "showfh $path ";

        $ret = $vol->{F}->say(command=>$cmd);
        die("showfh did not return the hex inode number, but returned <<$ret>>") if($ret !~ /fileid=(.*)/);
        $hex_num = $1;
        return hex($hex_num);
    }

=head2 vol_snap_reserve ()

Function to change snap reserve percentage of volume. if not passed any argument , returns the current value of snap reserve

=over 4

=item reserve_pct

reserve percentage value with which we want to change the snapshot reservation

=over 4

Failure

Return with an error message if command is not able to execute correctly

=back

=cut

    sub vol_snap_reserve (@)
    {
	my  ($V, $cmd, $ret, %opts);
	$V = shift();

	%opts = validate (@_, {reserve_pct => { type=>SCALAR, optional=>1}} );
	my ($reserve_pct) = $opts{reserve_pct};

	if ( defined($reserve_pct)) {
	    $cmd = "snap reserve -V $V->{name} $reserve_pct";
	    eval {
		$ret = $V->{F}->say(command => $cmd);
	    };
	    if ($@) {  die($@);  }
	    return 1;
	} else {
	    $cmd = "snap reserve -V $V->{name}";
	    $ret = $V->{F}->say(command => $cmd);
	    if ($ret !~ s/Volume $V->{name}: current snapshot reserve is (\d+)% or (\d+) k-bytes\.//) {
		die ("vol size did not executed correctly");
	    }
	    return ($1);
	}
    }

    return 1;
}

###
### Generate and execute send_blocks command
###
=head2 send_blocks();
   $V->send_blocks(id=> "0x0019a2", type=>"1", gen=>"0x19b1ef01",
                         offset=>"0", size=>"32768", fsid=>"0x3c3e2321",
                        sym_link_path=>1, subtype=>1,
                         file_space=>1, meta_file_space=>1 );

Generate and execute send_blocks command
	send_blocks command is used to generate WAFL_SPINNP_REPL_SEND_BLOCKS message
	WAFL_SPINNP_REPL_SEND_BLOCKS is a new WAFL message to update files on a paloma
	destination volume, which is very similar to WAFL_SPINNP_WRITE. It's a message
	for 10-mode volumes only

usage: send_blocks -i {public_fileid | metafile_id} -t type -g gen -o offset -s size -f fsid [-p sym_link_path (needed if type is 5, offset and size are ignored in this case)] [-st subtype (needed if type is 11; 1 for cifs and 2 for nfsv4 acls] [-fs file_space (use 0 for public files and 1 for meta files; defaults to 0)]
[-m meta_file_space (use 0 for wire metafiles and 1 for local metafiles; defaults to 0)]

Return value:
    Return the 'send_blocks' command output in scalar. Validation should happen in the calling function/script
=cut

sub send_blocks(@)
{
    my  ($V, $cmd, $ret_value, %opts);
    $V = shift();
	%opts = validate(@_, {
					id				=>{type=>SCALAR},
					type			=>{type=>SCALAR},
					gen				=>{type=>SCALAR},
					offset			=>{type=>SCALAR},
					size			=>{type=>SCALAR},
					fsid			=>{type=>SCALAR},
					sym_link_path	=>{type=>SCALAR, optional=>1},
					subtype			=>{type=>SCALAR, optional=>1},
					file_space		=>{type=>SCALAR, optional=>1},
					meta_file_space	=>{type=>SCALAR, optional=>1},
			});

	my $id				= $opts{id};
	my $type			= $opts{type};
	my $gen				= $opts{gen};
	my $offset			= $opts{offset};
	my $size			= $opts{size};
	my $fsid			= $opts{fsid};
	my $sym_link_path	= $opts{sym_link_path};
	my $subtype			= $opts{subtype};
	my $file_space		= $opts{file_space};
	my $meta_file_space	= $opts{meta_file_space};

	$cmd = "send_blocks -i $id -t $type -g $gen -o $offset -s $size -f $fsid ";

	if(defined($sym_link_path))		{ $cmd	= "$cmd " . "-p $sym_link_path"; }
	if(defined($subtype))			{ $cmd	= "$cmd " . "-st $subtype"; }

	$cmd	= "$cmd " . "-fs $file_space -m $meta_file_space";

	$Log->comment("send_blocks: send_blocks command = $cmd");

	$ret_value = $V->{F}->say(command => $cmd);
	return $ret_value;
}

###
### Create DataWarehouse file
###
=head2 create_dwfile(volname=>$vol_name, sleep=> $delay_in_sec, action => $slow)

Create DataWarehouse file

usage: file datawarehouse create <volname> [<dwdir>] <dwname>

Return value:

	Success	: Return Hashref with DW directory and DW file location
	Failure	: Die
=cut

sub create_dwfile($@)
{
	my ($V, $cmd, $ret_value, %opts, %dwfile_loc);
	$V = shift();
	%opts = validate(@_, {
            		        dwdir	=>{type=>SCALAR, optional=>1},
            		        dwname	=>{type=>SCALAR}
            		});

	my $dwdir	= $opts{dwdir};
	my $dwname	= $opts{dwname};

	if(defined($dwdir)){
		$cmd = "file datawarehouse create  $V->{name} $dwdir $dwname";		
	}else{
		$cmd = "file datawarehouse create  $V->{name} $dwname";
	}
	
	$Log->comment("create_dwfile: DW file create command = $cmd");
	$ret_value = $V->{F}->say(command => $cmd);
	
	if (($ret_value !~ /FILE SUCCESS/) || ($ret_value !~ /MAP SUCCESS/)){
		die ("Creating datawarehouse failed");
	}
	if(defined($dwdir)){
		$dwfile_loc{'dw_dir'}	=	 "/vol/$V->{name}/metadir/datawarehouse/$dwdir";
		$dwfile_loc{'dw_file'}	=	 "/vol/$V->{name}/metadir/datawarehouse/$dwdir/$dwname" . "_FILE";
		$dwfile_loc{'dw_map'}	=	 "/vol/$V->{name}/metadir/datawarehouse/$dwdir/$dwname" . "_MAP";
	}else{
        $dwfile_loc{'dw_dir'}   =    "/vol/$V->{name}/metadir/datawarehouse";
        $dwfile_loc{'dw_file'}  =    "/vol/$V->{name}/metadir/datawarehouse/$dwname" . "_FILE";
        $dwfile_loc{'dw_map'}   =    "/vol/$V->{name}/metadir/datawarehouse/$dwname" . "_MAP";
	}
	return (\%dwfile_loc);
}

###
### Delete DataWarehouse file
###
=head2	$V->delete_dwfile(dwdir=>"dw");

Delete DataWarehouse file

usage: file datawarehouse delete <volname> [<dwpath>]

Return value:
    Success Or Failure	:	Return the command output
							Validation should happen in the caling function/script
=cut

sub delete_dwfile($@)
{
	my ($V, $cmd, $ret_value, %opts);
	$V = shift();
	%opts = validate(@_, {dwdir   =>{type=>SCALAR, optional=>1} });

	my $dwdir   = $opts{dwdir};
	if(defined($dwdir)){
		$cmd = "file datawarehouse delete $V->{name} $dwdir";
	}else{
		$cmd = "file datawarehouse delete $V->{name}";
	}
    
	$Log->comment("delete_dwfile: DW file delete command = $cmd");
	$ret_value = $V->{F}->say(command => $cmd);
	
	return $ret_value;
}

###
### Parse showfh command to get inode flags, snapid, fileid, gen & fsid
###
=head2 parse_showfh(file_path=>$file_path)

Parse showfh command to get inode flags, snapid, fileid, gen & fsid

usage: showfh <file_path>

Return value:

    Success : Return Hash with inode properties (flags, snapid, fileid, gen & fsid)
    Failure : Die

=cut

sub parse_showfh($@)
{
	my ($V, $cmd, $ret_value, %opts, %inode_prop);
	$V = shift();
	%opts = validate(@_, {file_path =>{type=>SCALAR} });

	my $file_path   = $opts{file_path};
	$cmd = "showfh $file_path";

	$Log->comment("parse_showfh: showfh command = $cmd");
	$ret_value = $V->{F}->say(command => $cmd);
	die("showfh command failed. $cmd Output: $ret_value") if($ret_value !~ /fileid=(.*)/);
	
	if ($ret_value =~ "flags=([0-9a-z]+)")	{$inode_prop{flags}	= $1;}
	if ($ret_value =~ "snapid=([0-9a-z]+)")	{$inode_prop{snapid}= $1;}
	if ($ret_value =~ "fileid=([0-9a-z]+)")	{$inode_prop{fileid}= $1;}
	if ($ret_value =~ "gen=([0-9a-z]+)")	{$inode_prop{gen}	= $1;}
	if ($ret_value =~ "fsid=([0-9a-z]+)")	{$inode_prop{fsid}	= $1;}
	if ($ret_value =~ "dsid=([0-9a-z]+)")	{$inode_prop{dsid}	= $1;}

	return (\%inode_prop);
}

###
### Generate and execute send_file_update_byname command
###
=head2 send_file_update_byname();
        $V->send_file_update_byname(fsid=> "0x560863f4", don_fileid=>"0x002d13", don_gen=>"0x002d13",
                              recep_fileid=>"0x000060", recep_gen=>"0x199aec53", recep_offset=>10,
                              don_offset=>5, num_blks=>20 );

Generate and execute send_file_update_byname command

usage:
send_file_update_byname -f fsid -doi <donor_fileid> -dog <donor_gen> [-ds <donor_snapid>] [-dpub] -ri <recepient_fileid> -rg <recepient_gen> [-rt <recepient type>] [-rpriv] -ro <recepient offset> {-do <donor offset> -nb <num blocks>} | {-df <d1,d2,d3...d128>}

Return value:
    Return the 'send_file_update_byname' command output. Validation should happen in the caling function/script
=cut

sub send_file_update_byname($@)
{
	my  ($V, $cmd, $ret_value, %opts);
	$V = shift();
	%opts = validate(@_, {
			fsid			=>{type=>SCALAR},
			don_fileid		=>{type=>SCALAR},
			don_gen			=>{type=>SCALAR},
			don_snapid		=>{type=>SCALAR, optional=>1},
			dpub			=>{type=>SCALAR, optional=>1, regex => qr/^(0|1)$/},
			recep_fileid		=>{type=>SCALAR},
			recep_gen		=>{type=>SCALAR},
			recep_type		=>{type=>SCALAR, optional=>1},
			rpriv			=>{type=>SCALAR, optional=>1, regex => qr/^(0|1)$/},
			recep_offset		=>{type=>SCALAR, optional=>1},
			don_offset		=>{type=>SCALAR, optional=>1}, 
			num_blks		=>{type=>SCALAR, optional=>1},
			discrete_fbns		=>{type=>SCALAR, optional=>1},
			recep_fbns		=>{type=>SCALAR, optional=>1},
			transfer_type		=>{type=>SCALAR},
			cgs			=>{type=>SCALAR, optional=>1},
		});

	my $fsid		= $opts{fsid};
	my $don_fileid		= $opts{don_fileid};
	my $don_gen		= $opts{don_gen};
	my $don_snapid		= $opts{don_snapid};
	my $dpub		= $opts{dpub};
	my $recep_fileid	= $opts{recep_fileid};
	my $recep_gen		= $opts{recep_gen};
	my $recep_type		= $opts{recep_type};
	my $rpriv		= $opts{rpriv};
	my $recep_offset	= $opts{recep_offset};
	my $don_offset		= $opts{don_offset};
	my $num_blks		= $opts{num_blks};
	my $discrete_fbns	= $opts{discrete_fbns};
	my $recep_fbns		= $opts{recep_fbns};
	my $transfer_type	= $opts{transfer_type};
	my $cgs			= $opts{cgs};

	$cmd = "send_file_update_byname -f $fsid -doi $don_fileid -dog $don_gen ";

	if(	((defined($don_offset)) || (defined($num_blks))) && (defined($discrete_fbns)) ){
		$Log->comment("Should not pass both contiguous & discrete fbns values"); 
		die("Function send_file_update_byname: Pass either don_offset & num_blks OR  discrete_fbns")
	}

	if(defined($don_snapid))	{$cmd = "$cmd " . "-ds $don_snapid ";}
	if(defined($dpub))		{$cmd = "$cmd " . "-dpub ";}

	$cmd = "$cmd " . "-ri $recep_fileid " . "-rg $recep_gen";

	if(defined($recep_type))	{$cmd = "$cmd " . "-rt $recep_type ";}
	if(defined($rpriv))		{$cmd = "$cmd " . "-rpriv ";}

	if(defined($recep_fbns))      	{$cmd = "$cmd " . "-rf $recep_fbns ";}
	elsif(defined($recep_offset))	{$cmd = "$cmd " . "-ro $recep_offset ";}
	else				{$Log->comment("Recipient  Range: Missing values for one of the parameter: recep_offset or recep_fbns");
					die("Function send_file_update_byname: Pass either recep_offset or recep_fbns");}
	
	if(defined($discrete_fbns)){
		$cmd = "$cmd " . "-df $discrete_fbns ";
	}elsif((defined($don_offset)) && (defined($num_blks))){
		$cmd = "$cmd " . "-do $don_offset " . "-nb $num_blks ";
	}else{
		$Log->comment("Contiguous Donor Range: Missing values for one of the parameter: don_offset or num_blks");
		die("Function send_file_update_byname: Pass both don_offset & num_blks");
	}
        $cmd = "$cmd " . "$transfer_type ";
	if(defined($cgs))		{$cmd = "$cmd " . "-cgs $cgs ";}

	$Log->comment("send_file_update_byname: send_file_update_byname command = $cmd");

	$ret_value = $V->{F}->say(command => $cmd);
	return $ret_value;
}

############################################
## Description: 
##          This function will run waf lscan measure_layout command on a file.
## Inputs: 
##          Inputs to this function are "file_name" param
## Return value:
##          Returns the layout ration for particular file.
##
############################################

sub wafl_scan_measure_layout(@)
{
    my ($V, %opts, $ret, $layout);
    $V = shift();
    %opts = validate (@_, {file_name => {type=>SCALAR, optional=>1 },
                        });
    my $record = $V->{F}->start_log(file=>'ems');

    $V->{F}->say(command=>"wafl scan measure_layout $opts{file_name}");
    sleep(10);
    
    my $inum = $V->get_inode_num(%opts);
    my $str1 = "ratio1";
    $record->stop();
    my $result = $record->to_log();
    print Dumper($result);
    my @result = split(/\n/, $result);
    my @index = grep(/$str1/, @result);
    
    print Dumper(\@index);
    if ($index[0] =~ m/ratio1=\"(\d+)\"/ ) {
        $layout = $1;
        $Log->comment("Layout ratio is $layout");
    } else {
        die "Didn't got the layout ratio";
    }
    return $layout;
}


############################################
## Description:
##          Invokes "wafl touch" on the inode and the fbn(s) provided
## Input params:
##          inum => Inode number of the file
##          level => the level of the block in the buftree(0 being L0)
##          start_fbn => starting fbn number 
##          end_fbn => end fbn number 
##          private => if the param is set to 1 then the inum will be treated as private
## Return value
##          Returns the command output of "wafl touch"
##
############################################

sub wafl_touch(@)
{
    my ($V, $cmd, %opts);
    $V = shift;
    %opts = validate (@_, {
                                inum    => { type=>SCALAR},
                                level   => { type=>SCALAR},
                                start_fbn => {type=>SCALAR},
                                end_fbn => {type=>SCALAR},
                                private => {type=>SCALAR, optional=>1 },
                            });
    
    if ($opts{private}) {
        $cmd = "wafl touch -V $V->{name} $opts{inum} -private $opts{level} $opts{start_fbn} $opts{end_fbn}";
    } else {
        $cmd = "wafl touch -V $V->{name} $opts{inum} $opts{level} $opts{start_fbn} $opts{end_fbn}";
    }
    $ret = $V->{F}->say(command=>$cmd);
    
    return $ret;    
}

############################################
## Description:
##      lrse_val_func is a function used for validating LRSE transfer
## Input params:
##      srcvol => Source volume object
##      sisflag_src => Flag to specify whether dedupe is run on the source volume, set to 0 if dedupe operation is not run on the source volume, default is 1        Use this flag to run dedupe on source volume after lrse transfer
##      filelist => Optional parameter to specify the list of files in source volume that needs to be validated
##      srcsnap => Optional parameter to specify the source snapshot
##      client => Optional parameter to specify the client on which the volume should be mounted
##      client_type => Optional parameter to specify the client type on which the volume should be mounted
## Return Value
##      Returns Success or Failure
############################################

sub     lrse_val_func (@)
{
        $Log->comment("Entered lrse_val_func");
        my (%opts);
        my ($file, %src_vbn, %dst_vbn, $status_info, $src_snap, @abs, @files, $out, $mntdir, $comp_result);
        my ($src_vol, $dst_vol, $src_vol_name, $dst_vol_name, $voldir, $src_file_path, $dst_file_path, %exp_vbn);

        $dst_vol = shift();
        %opts = validate (@_, { srcvol => {type=>HASHREF},
                                sisflag_src => {type=>SCALAR, optional=>1, default=>1},
                                filelist => {type=>ARRAYREF, optional=>1},
                                srcsnap => {type=>SCALAR, optional=>1},
                                client => {type=>SCALAR, optional=>1, default=>'localhost'},
                                client_type => {type=>SCALAR, optional=>1, default=>'Linux'},
                         });
        $src_vol = $opts{srcvol};
        $src_vol_name = "$src_vol->{name}";
        $dst_vol_name = "$dst_vol->{name}";

        # Check for LRE condition, LRE validation is not supported by this function
        $status_info = $dst_vol->sis_status();
        if ($status_info->{'compression'} eq "true"){
            die ("LRE validation is not supported by this function");
        }

        #Check if source volume filelist is passed to the function else mount the volume on a client and get the file list
        if (defined($opts{filelist})){
            @files = @{$opts{filelist}};
            if (defined($opts{srcsnap})){
                $src_snap = $opts{srcsnap};
                $voldir = "/vol/$src_vol_name/.snapshot/$src_snap/";
            }
            else {
                 $voldir="/vol/$src_vol_name/";
            }
        }
        else {
             eval {
                  $Client = Dense::Agnostic::Client->new(Name => $opts{client}, Type => $opts{client_type});
             };
             if ($@) {
                 die($@);
             }
             my $mntpnt = $Client->mount_vol_nfs($src_vol);
             # Get files in the source directory
             if (defined($opts{srcsnap})){
                 $src_snap = $opts{srcsnap};
                 $mntdir = $mntpnt."/.snapshot/"."$src_snap/";
                 $voldir = "/vol/$src_vol_name/.snapshot/$src_snap/";
             }
             else {
                   $mntdir=$mntpnt;
                   $voldir="/vol/$src_vol_name/";
             }

             find ({
                   wanted => sub { push @abs, $_ if -f and -r },
                   no_chdir => 1,
             }, $mntdir);
             @files = map { File::Spec->abs2rel($_, $mntdir) } @abs;
             $Client->umount_vol_nfs($mntpnt);
        }
        #Run sis on src volume if it is not run before LRSE transfer
        if ($opts{sisflag_src} == 0) {
            $src_vol->sis_start();
            # Wait for Sis to be idle
            $src_vol->sis_wait_for();
        }
        #Get the sis status of source volume
        $status_info = $src_vol->sis_status();

        # Wait for Sis to be idle on destination volume
        $dst_vol->sis_wait_for();

        $Log->comment("Files : @files");
        #For each file in the source volume verify whether LRSE transfer is correct
        foreach $file (@files) {
             $src_file_path = "$voldir"."$file";
             #Get the wafl showvbns output of source file
             %src_vbn = $src_vol->wafl_showvbns(file_name=>$src_file_path);
             $Log->comment("src_file_path = $src_file_path, voldir = $voldir");
             $dst_file_path = "/vol/$dst_vol_name/".$file;
             #Get the wafl showvbns output of destination file
             %dst_vbn = $dst_vol->wafl_showvbns(file_name=>$dst_file_path);
             #Get the expected vbns output of destination file
             %exp_vbn = $dst_vol->lrse_cal_exp_vbns(srcwsv => \%src_vbn, dstwsv => \%dst_vbn, srcvol_status => $status_info, lrseflag => $opts{sisflag_src});
             #Compare the destination vbns with expected vbns
             $comp_result = dense_compare_wsv (hash1 => \%dst_vbn, hash2 =>\%exp_vbn);
             if($comp_result ne DENSE_CLONE_SUCCESS){
                die("vbns of files on destination volume $dst_vol_name are not as expected");
             }
        }
}

############################################
## Description:
##      lrse_cal_exp_vbns is a function used to create expected fbn, vbn, vvbn, ref count output for the destination files
## Input params:
##      srcwsv => Hash Reference of Source volume wafl showvbn
##      dstwsv => Hash Reference of Destination volume wafl showvbn
##      srcvol_status => SIS status of src vol
##      lrseflag => Flag to specify whether dedupe is triggered before or after lrse transfer
## Return Value
##      Returns Expected hash
############################################

sub     lrse_cal_exp_vbns (@)
{
        my (%opts);
        my (%dest_vol, $dst_vol_name, %s_wsv, %d_wsv, $dch_vbn, $dch_vvbn, $dct, $filer);
        $dest_vol = shift();
        %opts = validate (@_, { srcwsv => {type=>HASHREF},
                                dstwsv => {type=>HASHREF},
                                srcvol_status => {type=>HASHREF},
                                lrseflag => {type=>SCALAR},
                                });
        $s_wsv = $opts{srcwsv};
        $d_wsv = $opts{dstwsv};
        $dst_vol_name = "$dest_vol->{name}";
        $filer = $dest_vol->{F}->{name};
	$dch_vbn = 281474976710654;
	$dch_vvbn = 281474976710646;
	$dct = 281474976710652;

        ### Construct hashes 'fbn to vbn', 'fbn to vvbn' & 'vbn to refcnt' for source & clone file
        my ($sfbn, $svbn, $svvbn, $srefc, $dw_path, %dst_exp, $srcstatus_info, $dststatus_info, %dwwsv, $maxref_flag, $prev_vvbn);

        $maxref_flag = 0;
        $dststatus_info = $dest_vol->sis_status();
        $srcstatus_info = $opts{srcvol_status};
        $Log->comment("srcvol_status = $srcstatus_info->{'state'}, dest_vol_status = $dststatus_info->{'state'}");
        $dw_path =  "/vol/$dst_vol_name/".DWPATH;
        #Set wafl_metadata_visible to access dw file
        $dest_vol->{F}->say(command => "setflag wafl_metadata_visible 1");
        #For every block of source volume calculate the expected output of fbn, vbn, vvbn, ref count on destination volume
        for my $idx (sort keys(%$s_wsv)) {
                $sfbn    = $s_wsv->{$idx}->{fbn};
                $svbn    = $s_wsv->{$idx}->{vbn};
                $svvbn   = $s_wsv->{$idx}->{vvbn};
                $srefc   = $s_wsv->{$idx}->{refcount};
                $Log->comment("sfbn = $sfbn, svbn = $svbn, svvbn = $svvbn, srefc = $srefc, idx = $idx");
                #Get the expected vbns output of datawarehouse file
                if ($svvbn != $dch_vvbn && $svvbn != $dct){
                    %dwwsv   = $dest_vol->wafl_showvbns(file_name=>$dw_path, fbn_range=>$svvbn);
                }
                #Create a hash of expected output of fbn, vbn, vvbn, ref count of destination volume
                $dst_exp{$sfbn}        = {};
                $dst_exp{$sfbn}{fbn}   = $sfbn;
                $dst_exp{$sfbn}{vbn}   = $dwwsv{$svvbn}{vbn};
                $dst_exp{$sfbn}{vvbn}  = $dwwsv{$svvbn}{vvbn};
                #Calculate the ref count of data block in destination volume when dedupe is disabled on both source and destination volume
                if ($srcstatus_info->{'state'} eq "Disabled" && $dststatus_info->{'state'} eq "Disabled"){
                #If the data is a hole
                    if ($svbn == 0 && $svvbn == 0 && $srefc == 0) {
                        $dst_exp{$sfbn}{refcount} = 0;
                    }
                    else {
                        $dst_exp{$sfbn}{refcount} = $srefc + 1;
                    }
                }
                #Calculate the ref count of data block in destination volume when dedupe is enabled on source and disabled on destination volume
                elsif ($srcstatus_info->{'state'} eq "Enabled" && $dststatus_info->{'state'} eq "Disabled"){
                #If the data is a hole
                    if ($svbn == 0 && $svvbn == 0 && $srefc == 0) {
                        $dst_exp{$sfbn}{refcount} = 0;
                    }
                #If compression is enabled on the source volume
                    elsif ($srcstatus_info->{'compression'} eq "true"){
		#If the vbn, vvbn are compression header
                        if ($svbn == $dch_vbn && $svvbn == $dch_vvbn){
                            $dst_exp{$sfbn}{vbn}   = $dch_vbn;
                            $dst_exp{$sfbn}{vvbn}  = $dch_vvbn;
                            $dst_exp{$sfbn}{refcount} = 0;
                        }
		#If the vbn, vvbn are compression trailer
                        elsif ($svbn == $dct && $svvbn == $dct){
                            $dst_exp{$sfbn}{vbn}   = $dct;
                            $dst_exp{$sfbn}{vvbn}  = $dct;
                            $dst_exp{$sfbn}{refcount} = 0;
                        }
                        else {
                # If ref count on source fbn is MAX REF COUNT (32767)
                    	    if ($srefc == MAX_REF_COUNT - 1) {
		# Due to Burt637223 code to handle max ref count needs to enhanced but keeping the code as it is for now and returning from the function if it is max ref count
				return %{$d_wsv}; 
                        	if (!$maxref_flag) {
                            	    $prev_vvbn = $d_wsv->{$idx}->{vvbn};
                            	    $maxref_flag = 1;
                        	}
                        	$dst_exp{$sfbn}{vbn}   = $d_wsv->{$idx}->{vbn};
                        	$dst_exp{$sfbn}{vvbn}  = $d_wsv->{$idx}->{vvbn};
                        	if ($prev_vvbn != $d_wsv->{$idx}->{vvbn}){
                            	    $dst_exp{$sfbn}{refcount} = 1;
                            	    $maxref_flag = 0;
                        	}
                        	else {
                            	    $dst_exp{$sfbn}{refcount} = MAX_REF_COUNT - 2;
                        	}
                        	$prev_vvbn = $d_wsv->{$idx}->{vvbn};
                    	    }
			    else {
                            	$dst_exp{$sfbn}{refcount} = $srefc + 1;
                            }
			}
                    }
                # If ref count on source fbn is MAX REF COUNT (32767)
                    elsif ($srefc == MAX_REF_COUNT - 1) {
                # Due to Burt637223 code to handle max ref count needs to enhanced but keeping the code as it is for now and returning from the function if it is max ref count
                        return %{$d_wsv};
			if (!$maxref_flag) {
			    $prev_vvbn = $d_wsv->{$idx}->{vvbn};
                            $maxref_flag = 1;
			}
                        $dst_exp{$sfbn}{vbn}   = $d_wsv->{$idx}->{vbn};
                        $dst_exp{$sfbn}{vvbn}  = $d_wsv->{$idx}->{vvbn};
			if ($prev_vvbn != $d_wsv->{$idx}->{vvbn}){
			    $dst_exp{$sfbn}{refcount} = 1;
			    $maxref_flag = 0;
			}
			else {
                            $dst_exp{$sfbn}{refcount} = MAX_REF_COUNT - 2;
			}
                        $prev_vvbn = $d_wsv->{$idx}->{vvbn};
                    }
                    else {
                        $dst_exp{$sfbn}{refcount} = $srefc + 1;
                    }
                }
                #Calculate the ref count of data block in destination volume when dedupe is disabled on source and enabled on destination volume
                elsif ($srcstatus_info->{'state'} eq "Disabled" && $dststatus_info->{'state'} eq "Enabled"){
                #If the data is a hole
                    if ($svbn == 0 && $svvbn == 0 && $srefc == 0) {
                        $dst_exp{$sfbn}{refcount} = 0;
                    }
                    else {
                        $dst_exp{$sfbn}{refcount} = $dwwsv{$svvbn}{refcount};
                    }
                }
                #Calculate the ref count of data block in destination volume when dedupe is enabled on both source and destination volume
                elsif ($srcstatus_info->{'state'} eq "Enabled" && $dststatus_info->{'state'} eq "Enabled"){
                #If the data is a hole
                    if ($svbn == 0 && $svvbn == 0 && $srefc == 0) {
                        $dst_exp{$sfbn}{refcount} = 0;
                    }
                #If dedupe is run on source volume after LRSE transfer
                    elsif ($opts{lrseflag} == 0){
                        $Log->comment ("dedupe is run on source volume after LRSE transfer");
                #If compression is enabled on the source volume
                        if ($srcstatus_info->{'compression'} eq "true"){
		#If the vbn, vvbn are compression header
                            if ($svbn == $dch_vbn && $svvbn == $dch_vvbn){
                                $dst_exp{$sfbn}{vbn} = $dch_vbn;
                                $dst_exp{$sfbn}{vvbn}  = $dch_vvbn;
                                $dst_exp{$sfbn}{refcount} = 0;
                            }
		#If the vbn, vvbn are compression trailer
                            elsif ($svbn == $dct && $svvbn == $dct){
                                $dst_exp{$sfbn}{vbn}   = $dct;
                                $dst_exp{$sfbn}{vvbn}  = $dct;
                                $dst_exp{$sfbn}{refcount} = 0;
                            }
                            else {
				if ($srefc == MAX_REF_COUNT - 1) {
                # Due to Burt637223 code to handle max ref count needs to enhanced but keeping the code as it is for now and returning from the function if it is max ref count
                                    return %{$d_wsv};
                            	    $dst_exp{$sfbn}{vbn}   = $d_wsv->{$idx}->{vbn};
                            	    $dst_exp{$sfbn}{vvbn}  = $d_wsv->{$idx}->{vvbn};
                            	    $dst_exp{$sfbn}{refcount} = MAX_REF_COUNT - 1;
                        	}
                        	else {
                                    $dst_exp{$sfbn}{refcount} = $dwwsv{$svvbn}{refcount};
				}
                            }
                        }
                # If ref count on source fbn is MAX REF COUNT (32767)
                        elsif ($srefc == MAX_REF_COUNT - 1) {
                # Due to Burt637223 code to handle max ref count needs to enhanced but keeping the code as it is for now and returning from the function if it is max ref count
                            return %{$d_wsv};
                            $dst_exp{$sfbn}{vbn}   = $d_wsv->{$idx}->{vbn};
                            $dst_exp{$sfbn}{vvbn}  = $d_wsv->{$idx}->{vvbn};
                            $dst_exp{$sfbn}{refcount} = MAX_REF_COUNT - 1;
                        }
                        else {
                            $dst_exp{$sfbn}{refcount} = ($srefc*2) + 1;
                        }
                    }
                #If dedupe is run on source volume before LRSE transfer
                    elsif ($opts{lrseflag} == 1){
                        $Log->comment ("dedupe is run on source volume before LRSE transfer");
                #If compression is enabled on the source volume
                        if ($srcstatus_info->{'compression'} eq "true"){
		#If the vbn, vvbn are compression header
                            if ($svbn == $dch_vbn && $svvbn == $dch_vvbn){
                                $dst_exp{$sfbn}{vbn}   = $dch_vbn;
                                $dst_exp{$sfbn}{vvbn}  = $dch_vvbn;
                                $dst_exp{$sfbn}{refcount} = 0;
                           }
		#If the vbn, vvbn are compression trailer
                           elsif ($svbn == $dct && $svvbn == $dct){
                                $dst_exp{$sfbn}{vbn}   = $dct;
                                $dst_exp{$sfbn}{vvbn}  = $dct;
                                $dst_exp{$sfbn}{refcount} = 0;
                           }
                           else {
                # If ref count on source fbn is MAX REF COUNT (32767)
                            	if ($srefc == MAX_REF_COUNT - 1) {
                # Due to Burt637223 code to handle max ref count needs to enhanced but keeping the code as it is for now and returning from the function if it is max ref count
                                    return %{$d_wsv};
                                    if (!$maxref_flag) {
                                    	$prev_vvbn = $d_wsv->{$idx}->{vvbn};
                                    	$maxref_flag = 1;
                                    }
                                    $dst_exp{$sfbn}{vbn}   = $d_wsv->{$idx}->{vbn};
                                    $dst_exp{$sfbn}{vvbn}  = $d_wsv->{$idx}->{vvbn};
                                    if ($prev_vvbn != $d_wsv->{$idx}->{vvbn}){
                                    	$dst_exp{$sfbn}{refcount} = 1;
                                    	$maxref_flag = 0;
                                    }
                                    else {
                                    	$dst_exp{$sfbn}{refcount} = MAX_REF_COUNT - 2;
                                    }
                                    $prev_vvbn = $d_wsv->{$idx}->{vvbn};
                            	}
                            	else {
                                    $dst_exp{$sfbn}{refcount} = $srefc + 1;
                            	}
                           }
                    	}
                # If ref count on source fbn is MAX REF COUNT (32767)
                    	elsif ($srefc == MAX_REF_COUNT - 1) {
                # Due to Burt637223 code to handle max ref count needs to enhanced but keeping the code as it is for now and returning from the function if it is max ref count
                            return %{$d_wsv};
                            if (!$maxref_flag) {
                            	$prev_vvbn = $d_wsv->{$idx}->{vvbn};
                            	$maxref_flag = 1;
                            }
                            $dst_exp{$sfbn}{vbn}   = $d_wsv->{$idx}->{vbn};
                            $dst_exp{$sfbn}{vvbn}  = $d_wsv->{$idx}->{vvbn};
                            if ($prev_vvbn != $d_wsv->{$idx}->{vvbn}){
                            	$dst_exp{$sfbn}{refcount} = 1;
                            	$maxref_flag = 0;
                            }
                            else {
                            	$dst_exp{$sfbn}{refcount} = MAX_REF_COUNT - 2;
                            }
                            $prev_vvbn = $d_wsv->{$idx}->{vvbn};
                    	}
                    	else {
                            $dst_exp{$sfbn}{refcount} = $srefc + 1;
                    	}
                    }
                }
        }
        $Log->comment("wafl showvbns of expected hash of destination volume returning ".Dumper(\%dst_exp));
        return %dst_exp;
}
