{ package   Dense::Agnostic::Cmode::Vserver;
    use strict;
    use warnings;

    use Tharn;
    use Data::Dumper;

    use Dense::Agnostic::Constants;
    use Dense::Agnostic::Cmode::Volume;
    use Dense::Agnostic::Cmode::Vserver::Fmap;
    use Dense::Agnostic::Cmode::Vserver::Sis_Policy qw(sis_policy_exists
            sis_policy_sched_create);

    use VifMgr::VifConfig;
    
    use constant CLIENTMATCH    => "0.0.0.0/0";

    #Vserver->new(
    #   Vservername             => <vserver_name>,
    #   Node                    => <node>,
    #   Rootvolume              => <root_volume>,
    #   Aggr                    => <aggregate>,
    #   Ns-switch               => <ns-switch>,
    #   Rootvol_security_style  => <rootvolume_security_style>
    #   Reuse                   => 1
    sub new (@)
    {
        my  ($class, %in_opts, %opts);
        my  ($F, $Vs);
        my  ($timeout, $ret);

        $class  = shift();
        %in_opts   = @_;
        %opts   = (
            'ns-switch'                 => 'file',
            'rootvolume-security-style' => 'unix',
        );
        $Vs = {};

        $timeout = 30;
        $F  = delete $in_opts{Node};
        if (ref($F) ne 'Dense::Agnostic::Cmode::Node') {
            die("Vserver::new: Node must be an Agnostic::Cmode::Node object");
        }

        $opts{vserver}      = $in_opts{Vservername};
        $opts{rootvolume}   = $in_opts{Rootvolume};
        if (ref($in_opts{Aggr}) =~ 'Aggr') {
            $opts{aggregate}    = $in_opts{Aggr}->{name};
        } else {
            $opts{aggregate}    = $in_opts{Aggr};
        }
        if ($in_opts{'Ns-switch'}) {
            $opts{'ns-switch'} = $in_opts{'ns-switch'};
        }

        if ($in_opts{'Rootvolume-security-style'}) {
            $opts{'rootvolume-security-style'} = 
                $in_opts{'rootvolume-security-style'};
        }

        #Since we have the misfortune of our method being named vserver_create,
        #we have to resort to invoking the "actual" vserver_create method
        #through the APISet object directly.
        if ($in_opts{Reuse} == 1) {
#            for my $v (@{$F->vserver_show()}) {
             for my $v ( $F->vserver_show() ) {
                if ($v->{vserver} eq $opts{vserver}) {
                    $Log->debug(">>> Vserver->new(): Reuse: match\n");
                    goto OUT;
                }
            }
        }
        eval {
            $ret = $F->{api}->vserver_create(%opts);
        };
        if ($@) {
            die("Vserver->new: $@");
        }

        my  $found = 0;
        while ($found < $timeout) {
#            for my $v (@{$F->vserver_show()}) {
            for my $v ( $F->vserver_show() ) {
                if ($v->{vserver} eq $opts{vserver}) {
                    goto OUT;
                }
            }
            sleep(1);
        }
        die("Error creating vserver $opts{vserver}");

OUT:
        $Vs->{F}    = $F;
        $Vs->{name} = $opts{vserver};

        #XXX uncomment when Volume.pm is up. This doesn't really create the
        #rootvol; the vserver_create should've taken care of that. This is more
        #of the "find" kind of method.
         #eval {
         #    $Vs->{rootvol} = Dense::Agnostic::Volume->new(
         #                Volname => $opts{rootvolume},
         #                Reuse   => 1,
         #            );
         #};
         #if ($@) {
         #    die($@);
         #}

        my $host_info = host($F->{name});
        my $vif_config = new VifMgr::VifConfig();
        my $address = $host_info->{DATA_IP1} || param("VSERVER_IP","-mesg","Vserver IP");

        eval {
            $vif_config->create_vif(
                            server      => $Vs->{name},
                            vif         => $Vs->{name}."_data",
                            role        => "DATA",
                            home_node   => $F->{name},
                            home_port   => $host_info->{DATA_PORT1},
                            address     => $address,
                            if_netmask  => $host_info->{DATA_MASK1},
                            #server_args => \@server_add_args,
                            gateway     => $host_info->{DATA_GATEWAY1},
                    );

            $F->vserver_export_policy_rule_new(
                            vserver      => $Vs->{name},
                            policyname   => "default",
                            clientmatch  => CLIENTMATCH,
                            rorule       => "any",
                            rwrule       => "any",
                            superuser    => "any",
                            "allow-suid" => "true",
                            "allow-dev"  => "true",
                    );

            $F->vserver_nfs_new(
                            vserver  => $Vs->{name},
                            access   => "true",
                            v3       => "enabled",
                            udp      => "enabled",
                            tcp      => "enabled",
                            spinauth => "enabled",
                    );

        };
        
        if ($@) {
            if ($in_opts{Reuse} != 1) { $Log->warn("Vserver setup failed"); }
        }

        bless($Vs, $class);
        return $Vs;
    } #sub new

    #XXX An fmap to implement cifs/fcp/etc/etc/etc - all other vserver methods?

    sub AUTOLOAD (@)
    {
        our $AUTOLOAD;
        my  ($Vs, $F);
        my  ($in_fn, %in_opts, $exec_fn, $ret, @ret);
        my  ($k, $v);

        $Vs = shift();
        %in_opts   = @_;
        $in_fn  = $AUTOLOAD;
        $in_fn  =~ s/.*:://; #Chop off the leading Foo::bar::blah

        $F  = $Vs->{F};
        if (ref($F) !~ 'Cmode::Node') {
            die('Cmode::Vserver::AUTOLOAD: $F has wrong type');
        }
        
        $exec_fn = $in_fn;

        # Check if command is in Vserver::Fmap
        # If yes, process according to Fmap. If no, execute command exactly as passed in.
        if ( grep(/$in_fn/, keys(%$Fmap)) ) {
            $exec_fn = $Fmap->{$in_fn}{fname};
            $Log->comment("Cmode::Vserver::AUTOLOAD: evaluated $in_fn -> $exec_fn\n");

            #Check for mandatory params
            foreach my $p (@{$Fmap->{$in_fn}{m_args}}) {
                if ($p eq 'vserver') {
                    $in_opts{$p} = $Vs->{'name'};
                }
                if (! defined $in_opts{$p}) {
                    die("Cmode::Vserver->$in_fn: mandatory arg $p missing");
                }
            }

            #Resolve default params if any
            while (($k, $v) = each(%{$Fmap->{$in_fn}{d_args}})) {
                if (! defined $in_opts{$k}) {
                    $in_opts{$k} = $v;
                }
            }
        }

        #Now we should have a hash with all params reqd. to execute $in_fn.
        $Log->comment("Cmode::Vserver::AUTOLOAD: Evaluating $exec_fn with the following args:");

        $Log->debug(print Dumper(\%in_opts));
#        $ret = $F->$exec_fn(%in_opts);
        
#        return $ret;
         if ( wantarray() ) {
             $Log->debug("Cmode::Vserver::AUTOLOAD::wantarray: Array");
             @ret = $F->$exec_fn(%in_opts);
             return @ret;
		 }
		 else {
             $Log->debug("Cmode::Vserver::AUTOLOAD::wantarray: First element");
             $ret = $F->$exec_fn(%in_opts);
             return $ret;
		 }
    }

    return  1;
} #Package
