# -*- perl -*-
# $Id$
## @summary Filer/Volume/Utility routines for dense volume
## @usage
##      use Dense::Agnostic::Utils;
## @author 
## @cf     false
## @status public
## @pod here


=head1 NAME

Utility routines for dense test scripts.

=head1 DESCRIPTION

This module defines utility routines for dense test scripts use.
The routines are built on top of the SPARSE test infrastructure.

=head1 FUNCTIONS

=cut

package Dense::Agnostic::Utils;

require Exporter;
our @ISA= qw(Exporter);
our @EXPORT = qw (
  calculate_expected_savings
  wafl_get_active_vvbn
  is_gx
  check_debug
);
use Tharn;
use Utils;
use Params::Validate qw(validate validate_pos OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use Dense::Agnostic;
use Dense::Agnostic::Constants;
use Data::Dumper;
Params::Validate::validation_options( allow_extra => 1);
BEGIN {
Params::Validate::validation_options(
    # allow presence of options other than those named in the validation specification
    allow_extra => 1,
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
}

=head2 calculate_expected_savings(blocks=>$no_of_blocks,refcnt=>$max_refcnt)

Calculates and returns the expected savings for given number of blocks (only for a file with 100% duplicate) with a specific refcount

=cut

sub calculate_expected_savings(%)
{
    my (%opts) = validate(@_, {blocks=> {type=>SCALAR},
                               max_refcnt=> {type=>SCALAR, default=>MAX_REF_COUNT},}
                    );
    my $no_of_blocks = $opts{blocks};
    my $max_refcnt = $opts{max_refcnt};
    my ($lost_blocks, $saved_blocks, $remaining_blocks);
    my $savings;

    $remaining_blocks = 0;
    $Log->comment("No of blocks : $no_of_blocks");
    $lost_blocks = int($no_of_blocks/$max_refcnt);
    $Log->comment("Lost blocks is $lost_blocks");
    $remaining_blocks = $no_of_blocks%$max_refcnt;
    $Log->comment("Remaining blocks is $remaining_blocks");

    if ($remaining_blocks == 0) {
         $saved_blocks = $no_of_blocks - $lost_blocks;
         $savings = $saved_blocks * 4;
         return $savings;
    } else {
         $lost_blocks++;
         $saved_blocks = $no_of_blocks - $lost_blocks;
         $savings = $saved_blocks * 4;
         return $savings;
    }
}


=head2 calc_savings_inlinecompress_mkfile(blocks=>$no_of_blocks)

Calculates and returns the expected (compression+dedupe) savings for given number of blocks(for a single file ) 
Please Note :The function as it stands today will work /only/ for 
1) with inline compression (only offline compression enabled will not work) 
2) working on a dataset that has all-identical blocks (like an mkfile dataset)
3) which does not span L1 boundaries(blocks should be < 248)
 
=cut

sub calc_savings_inlinecompress_mkfile(%)
{
    my (%opts) = validate(@_, {blocks=> {type=>SCALAR}, } );
    my $no_of_blocks = $opts{blocks};
    my ($compressed_blocks, $saved_blocks, $remaining_blocks, $no_of_cg);
    my ($savings, $compressed_savings, $dedupe_savings);

    $remaining_blocks = 0;
    $Log->comment("No of blocks : $no_of_blocks");
    $no_of_cg = int($no_of_blocks / BLKS_PER_CG);
    $Log->comment("No of CG blocks is $no_of_cg");
    $remaining_blocks = $no_of_blocks % BLKS_PER_CG;
    $Log->comment("Remaining blocks is $remaining_blocks");
         
    $compressed_blocks = $no_of_blocks - $remaining_blocks - $no_of_cg;
    #calculating savings in KB hence multiplying  by 4, since each block is 4k
    $compressed_savings = $compressed_blocks * 4;
    $Log->comment("compression savings is  $compressed_savings");
    
    if ($remaining_blocks > 1) {
         $saved_blocks = $remaining_blocks - 1;
    }
       
    $Log->comment("Noncompressed blocks deduped  is  $saved_blocks");
    $saved_blocks +=  ($no_of_cg - 1);
    $Log->comment("total blocks deduped  is  $saved_blocks");
    #calculating savings in KB hence multiplying  by 4, since each block is 4k
    $dedupe_savings = $saved_blocks * 4;
    $Log->comment("Dedupe savings  is  $dedupe_savings");
    $savings = $compressed_savings + $dedupe_savings;

    return $savings;
}

=head2 wafl_get_active_vvbn(file_vbns=>$showvbns_output_file, active_vvbn=>curr_vvbn_output)

    Returns a hash with entries <vbn, vvbn, refcnt> for every unique VVBN present in the %file_vbns hash input passed to it.
    $active_vvbn_curr (optional): Hash reference with already existing active VVBNs present in the volume. The unique VVBNs from the %file_vbns of current file will get appended to this hash

=cut

sub wafl_get_active_vvbn(%)
{
    my (%opts) = validate(@_, { file_vbns => {type=>HASHREF},
                                active_vvbn_curr => {type=>HASHREF, optional=>1} , } );
    my $file_vbns_ref = $opts{file_vbns};
    my %vbn_file = %$file_vbns_ref;
    my %active_vvbn;
    if( defined($opts{active_vvbn_curr}) ) {
        my $active_vvbn_ref = $opts{active_vvbn_curr};
        %active_vvbn = %$active_vvbn_ref;
    }
    foreach $key1 (keys %vbn_file)
    {
        my $present = 0;
        if (!keys %vbn_file)
        {
            $active_vvbn{0}{vvbn} = $vbn_file{$key1}{vvbn};
            $active_vvbn{0}{vbn} = $vbn_file{$key1}{vbn};
            $active_vvbn{0}{refcount} = $vbn_file{$key1}{refcount};
            print "Creating active_vvbn, populated first element \n";
        } else {
            foreach $key2 (keys %active_vvbn)
            {
                if ( $vbn_file{$key1}{refcount} eq $active_vvbn{$key2}{refcount} && $vbn_file{$key1}{vbn} eq $active_vvbn{$key2}{vbn} && $vbn_file{$key1}{vvbn} eq $active_vvbn{$key2}{vvbn}) { 
                    $present = 1;
                    last;
                }
            }
            if( !$present )
            {
                print "vbn is $active_vvbn{$key2}{vbn}, vvbn is $active_vvbn{$key2}{vvbn}, refcount is $active_vvbn{$key2}{refcount} \n";
                print "vbn is $vbn_file{$key1}{vbn}, vvbn is $vbn_file{$key1}{vvbn}, refcount is $vbn_file{$key1}{refcount}  \n";
                my $var = scalar keys %active_vvbn;
                print "Making a new entry in vbn_file for key: $var \n";
                $active_vvbn{$var}{vvbn} = $vbn_file{$key1}{vvbn};
                $active_vvbn{$var}{vbn} = $vbn_file{$key1}{vbn};
                $active_vvbn{$var}{refcount} = $vbn_file{$key1}{refcount};
            }
        }
    }

    print "Active VVBN hash from wafl_get_active_vvbn() is  \n";
    print Dumper(\%active_vvbn);
    return(\%active_vvbn);
}

=head2 is_gx()
	Returns 1 if filer is running Clustered ONTAP else 0
=cut

sub is_gx($)
{
    my $filer = shift;
    my $fh = make_host($filer);

    if ($fh->hosttype =~ HOSTTYPE_CMODE){
        return 1;
    }else{
        return 0;
    }
}

=head2 check_debug()
        Returns 1 if filer is running debug build else 0
=cut

sub check_debug($)
{
    my $filer = shift;
    my $fh = make_host($filer);

    return $fh->is_debug();
}

1;

