# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/Dense/Constant/Clone_Command.pm#1 $
#
## @summary	Library that defines Clone command & constants
##
## @description
##	        This package defines generic clone commands & constants
## @usage
##	use Dense::Constant::Clone_Command;
## @author mathan@netapp.com
## @status in progress
## @pod here

=head1 NAME

 Message - Command & Constat file for Clone

=head1 SYNOPSIS

 Use Dense::Constant::Clone_Command

=cut

package Dense::Constant::Clone_Command;
use strict;
use SPARSE::Util::Logger;
use filer;
use Tharn;
use Tharn qw(logcomment);
use Dense::Constant::Message;
use Dense::Constant::Command;
use Dense::Constant::Clone_Message;

BEGIN
	{
		Params::Validate::validation_options(
							on_fail => sub	{
							my @msgs = @_; 
							chomp(@msgs);
							Carp::confess(Tharn::script_result(@msgs))
									}
					  	    );
	} #BEGIN #1

use base qw(Exporter);

BEGIN	
	{
		my @action 	= qw (	GenerateCloneCmd
					ExecuteCloneCmd
					CloneCommandTemplate
				     ); 
		my @util	= qw ();
		my @verify 	= qw ();
		our %EXPORT_TAGS= (	action => [ @action ],
					util   => [ @util ],
				  );
		$EXPORT_TAGS{'all'} = [@action, @util, @verify,];
		Exporter::export_ok_tags('all');
	}#BEGIN #2
	
# Constants

use constant DENSE_CLONE_DEFAULT_TIMEOUT	=> 3600;
use constant DENSE_CLONE_DEFAULT_PARTNER	=> 0;
use constant DENSE_CLONE_INTERVAL			=> 3;
use constant DENSE_CLONE_MAX_TIME_TO_WAIT	=> 100;
use constant DENSE_CLONE_SUCCESS			=> "SUCCESS";
use constant DENSE_CLONE_FAIL				=> "FAIL";
use constant DENSE_FIJI_DEFAULT_DELAY		=> 30;
use constant DENSE_CLONE_NOT_SUPP_MSG		=>	"SIS Clone is not supported on this ONTAP Version. Supported from IC.1";
use constant DENSE_CLONE_IN_PROGRESS_FIELD	=>	'wvdc_dense_clone_inprog';
use constant AGGR_SHOWSPACE_AGGR_FIELDS		=>	"Total space    WAFL reserve    Snap reserve    Usable space       BSR NVLOG           A-SIS";
use constant AGGR_SHOWSPACE_VOL_FOUND		=>	"Space allocated to volumes in the aggregate";
use constant AGGR_SHOWSPACE_VOL_NOTFOUND	=>	"This aggregate does not contain any volumes";
use constant AGGR_SHOWSPACE_AGGR_TITLE		=>	"Aggregate                       Allocated            Used           Avail";
use constant AGGR_SHOWSPACE_VOL_TITLE		=>	"Volume                          Allocated            Used       Guarantee";
use vars ('@ISA', '@EXPORT');
require Exporter;

@ISA    = qw(Exporter);
@EXPORT = qw(
		DENSE_CLONE_DEFAULT_TIMEOUT
		DENSE_CLONE_DEFAULT_PARTNER
		DENSE_CLONE_PARTNER
		DENSE_CLONE_INTERVAL
		DENSE_CLONE_MAX_TIME_TO_WAIT
		DENSE_CLONE_SUCCESS
		DENSE_CLONE_FAIL
		DENSE_FIJI_DEFAULT_DELAY
		DENSE_CLONE_NOT_SUPP_MSG
		DENSE_CLONE_IN_PROGRESS_FIELD
		AGGR_SHOWSPACE_AGGR_FIELDS
		AGGR_SHOWSPACE_VOL_FOUND
		AGGR_SHOWSPACE_VOL_NOTFOUND
		AGGR_SHOWSPACE_AGGR_TITLE
		AGGR_SHOWSPACE_VOL_TITLE
		@clone_metafile_fields
		@clone_types
);

#########################################
### This is used for Generateing commands
#########################################

our %BaseCloneCmdTemplate = (
	clone	=> "<command> ",
	clear	=> "<command> <cmd_type> <vol_name> <id> ",
	help 	=> "<command> <cmd_type> <command> ",
	start	=> "<command> <cmd_type> <src_file> <clone_file> <options> <blk_range> ",
	stop	=> "<command> <cmd_type> <vol_name> <id> ",
	status	=> "<command> <cmd_type> <vol_name> <id> ",
	print_information	=> "<command> <cmd_type> <vol_name> ",
	corrupt_header	=> "<command> <cmd_type> <vol_name> <entry_index> <offset> <value>",
);

our %CloneCmdTemplate = 
(
	default	=> \%BaseCloneCmdTemplate,
	BR		=> \%BaseCloneCmdTemplate,
	IC		=> \%BaseCloneCmdTemplate,
	RR		=> \%BaseCloneCmdTemplate,
	SN 		=> \%BaseCloneCmdTemplate
);

#########################################
### Metafile fields to be got from the clone metafile
#########################################

our @clone_types=qw/DENSE_CLONE_NONE DENSE_CLONE_FILE DENSE_CLONE_SUB_FILE DENSE_CLONE_LUN DENSE_CLONE_SUB_LUN/;

our @clone_metafile_fields=qw(
			magic
			snap_index
			entry_index 
			type 
			err 
			flags 
			op_id1 
			op_id2 
			snapid 
			snap_cp_cnt 
			source_fileid 
			source_generation 
			clone_fileid 
			clone_generation 
			sfile_len 
			cfile_len 
			num_blk_ranges
                        vfiler_uid_1
			vfiler_uid_2
			vfiler_uid_3
			vfiler_uid_4
			);

### Get the inputs from functions and generate the commands

sub GenerateCloneCmd
{
	my @options = @_;
	my $cmd = "";

	## making the command template Generic
	push @options, "command";
	push @options, "clone";
	logcomment("DEBUG: GenerateCloneCmd options = @options");

	# get the command from the template
	$cmd = CloneCommandTemplate (@options);

	logcomment("DEBUG: Generated Command: [$cmd]");
	return $cmd;
}

sub ExecuteCloneCmd
{
	my @options		= @_;
	my %cmd_inputs	= @_;
	my $conn 		= $cmd_inputs{conn};
	my $partner 	= $cmd_inputs{partner};
	my $timeout 	= $cmd_inputs{timeout};
	my $test_mode 	= $cmd_inputs{test_mode};
	my $vfiler 	= $cmd_inputs{vfiler};
	my ($return_text, $result_code);
	
	### Generate the command
	my $clone_cmd = GenerateCloneCmd (@options);
	
	### Check partner mode
	if ($partner)
	{
		$clone_cmd  = " partner " . (($test_mode)?DENSE_TEST_CMD:DENSE_ADV_CMD) . " partner " . "$clone_cmd"; 
	}
	if($vfiler)
	{
	    $clone_cmd = "vfiler run $vfiler $clone_cmd";
	}
	if (!$timeout){
		$timeout  = DENSE_CLONE_DEFAULT_TIMEOUT;
	}
	logcomment("DEBUG: Clone command is: $clone_cmd");
	
	if($test_mode) {
		($return_text, $result_code) = filer_cli_test_say($conn, $clone_cmd, {timeout=>$timeout});
	} else {
		($return_text, $result_code) = filer_cli_advanced_say($conn, $clone_cmd, {timeout=>$timeout});
	}
	return ($return_text);

}

sub CloneCommandTemplate (@)
{
	my %NV 		= @_;
	my @options = @_;
	my $code 	= $NV{cmd_type};
	my $conn 	= $NV{conn};
	
	### Get the Filer name from 'conn' to find the ONTAP version
	my $filer = $conn->hostp()->hostname();
	logcomment("DEBUG: HostName is $filer");	
	my $version = getwv(host=>$filer);
	logcomment("DEBUG: Ontap Version is $version");
		
	# version control for command code lookup from template
	my $ref = ($version ne "")? 
			$CloneCmdTemplate{$version} :
			$CloneCmdTemplate{default} ;
			
	logcomment("DEBUG: Values: code = $code,  ref = $ref, $version");
	
	# map the command from template
	my $cmd = ${$ref}{$code};
	logcomment("DEBUG: Command from Template for version $version: [$cmd]");
	
	# populate the standard tokens for the command as defined in the template
	foreach my $key (keys %NV){ 
		my $tmpstr = $NV{$key};
		if(ref($NV{$key}) eq "ARRAY"){
			my $str = "";
			foreach my $k (@{$NV{$key}})
			{$str .= "$k,";}
			chop $str;
			logcomment("DEBUG: Converted opspec $tmpstr from array $NV{$key} 
				to str = $str, arraysize = ". $#$tmpstr);
			$NV{$key} = $str;
		}
		# convert the tag <name> to actual value(s) for this instance
		$cmd =~ s/\<$key\>/$NV{$key}/g; delete $NV{$key}; 
	}
	# return the formed command
	return $cmd;
}

###############################################################################
# end of file 
1;

