# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Dense/Constant/Clone_Message.pm#1 $
## @summary	Library that defines Clone Messages
##
## @description
##	This package checks for the ontap version running on the filer, and
##	appropriately returns message definitions.
##
## @usage
##	use Dense::Constant::Clone_Message; 
##
## @author mathan@netapp.com
## @status in progress
## @pod here

=head1 NAME

 Message - Command & Constat file for Clone

=head1 SYNOPSIS

 Use Dense::Constant::Clone_Message

=cut

package Dense::Constant::Clone_Message;
use strict;

use	Tharn qw(logcomment host);
use 	Params::Validate qw(validate validate_pos OBJECT SCALAR HASHREF ARRAYREF UNDEF);
use	Filer::Version;
use	Dense::Constant::Message;

# Constants

#use constant DENSE_CLONE_DEFAULT_TIMEOUT	=> 3600;

BEGIN {
	use vars ('@ISA', '@EXPORT');
	require Exporter;

	@ISA    = qw(Exporter);
	@EXPORT = qw( 	
			GetCloneMessage
		    );
}
	
## print "Using ". __FILE__;

our %CloneDefaultMessage = (
# the following messages are from common/dense/utils.c file
	DENSE_CLONE_GETID_REGEXP		=> "ID:\\s*(\\d+)",
	DENSE_CLONE_OK                	=> "Clone operation started successfully. ID:",
	DENSE_STALE_CLONE_DIR_INODE 	=> "stale clone file directory",
	DENSE_CLONE_SPLITTING			=> "Clone split is running on this volume",
	DENSE_CLONE_SFSR_INPROG			=> "SFSR is in progress on the volume",
	DENSE_CLONE_FILE_FOLD_INPROG	=> "File folding is in progress on the volume",
	DENSE_CLONE_NOT_LICENSED		=> "Flexclone not licensed",
	DENSE_CLONE_LICENSE_EXPIRED		=> "Flexclone license has expired",
	DENSE_CLONE_STOPPED				=> "Clone operation aborted by user",
	DENSE_CLONE_CLEANUP				=> "Clone operation not started yet",
	DENSE_CLONE_QUIT				=> "Clone operation quitted",
	DENSE_CLONE_SFILE_NONEXIST 		=> "Source file does not exist",
	DENSE_CLONE_CFILE_NONEXIST 		=> "Clone file does not exist",
	DENSE_CLONE_NO_SLOT				=> "No free slot to log clone operation",
	DENSE_CLONE_NO_ARGS				=> "There is no argument specified",
	DENSE_CLONE_DIFF_VOLUME			=> "Either source or destination file is in diffrent volume",
	DENSE_CLONE_SAME_FILE			=> "Source and destination files are same",
	DENSE_CLONE_VOL_REVERTING		=> "Dense revert is in progress on the volume",
	DENSE_CLONE_IN_PROG				=> "Dense clone operation is running",
	DENSE_CLONE_SNAPNAME			=> "Snap name could not be calculated",
	DENSE_CLONE_SNAPPATH			=> "Snap path could not be calculated",
	DENSE_CLONE_SNAP_FH				=> "Error in getting file handle of the source file in snapshot",
	DENSE_CLONE_SNAP_ATTR			=> "Error in getting file attribute of the source file the snapshot",
	DENSE_CLONE_SFILE_ATTR			=> "Error in getting file attribute of the source file",
	DENSE_CLONE_CFILE_ATTR			=> "Error in getting file attribute of the clone file",
	DENSE_CLONE_EXIST				=> "Clone file already exists",
	DENSE_CLONE_METAFILE			=> "Could not create dense clone metafile",
	DENSE_CLONE_METAFILE_FH			=> "Could not get file handle of dense clone metafile",
	DENSE_CLONE_METAFILE_WRITE		=> "Could not write to dense clone metafile",
	DENSE_CLONE_LARGE_INFO			=> "The clone information is too big to save persistently",
	DENSE_CLONE_METAFILE_ATTR		=> "Error in getting attribute of dense clone metafile",
	DENSE_CLONE_METAFILE_READ		=> "Error in reading dense clone metafile", 
	DENSE_CLONE_SFILE_READ			=> "Error in reading source file",
	DENSE_CLONE_CFILE_WRITE 		=> "Error in writing clone file",
	DENSE_CLONE_FH					=> "Error in getting file handle of the clone file",
	DENSE_CLONE_DIR_FH				=> "Error in getting file handle of the clone file's directory",
	DENSE_CLONE_CREATE				=> "Could not create clone file",
	DENSE_CLONE_SETATTR				=> "Could not set attribute for clone file",
	DENSE_CLONE_SOURCE_FH			=> "Error in getting file handle of the source file",
	DENSE_CLONE_NOT_SUPP			=> "This clone operation is not supported",
	DENSE_CLONE_PROC 				=> "Failed to create dense clone process",
	DENSE_CLONE_MAX_OPS_VOL 		=> "Maximum concurrent clone operations already running on the volume",
	DENSE_CLONE_TYPE_SUPP			=> "This source file type is not supported",
	DENSE_CLONE_SRC_OUTOF_RANGE		=> "Source block range out of file",
	DENSE_CLONE_DEST_OUTOF_RANGE	=> "Destination block range out of file",
	DENSE_CLONE_OVERLAP_RANGE		=> "Source and destination block ranges are overlapping",
	DENSE_CLONE_INVALID_RANGE		=> "Invalid block range",
	DENSE_CLONE_RUNNING_OP			=> "The clone operation is running",
	DENSE_CLONE_NOT_RUNNING 		=> "The clone operation is not running",
	DENSE_CLONE_NO_OP				=> "No such clone operation",
	DENSE_CLONE_NOT_FAILED			=> "Not a failed clone operations",
	DENSE_CLONE_EXISTING_OP			=> "Clone operation with same arguments is already running",
	DENSE_CLONE_DEL_INFO 			=> "Error in deleting dense clone info",
	DENSE_CLONE_PENDING_SNAP		=> "Pending snapshot creation on the volume",
	DENSE_CLONE_SPLIT_MAP 			=> "Volume split blockmap upgrade is in progress",
	DENSE_CLONE_INVALID_ID			=> "Invalid clone ID",
	DENSE_CLONE_INVALID_UUID		=> "Invalid volume uuid",
	DENSE_CLONE_INVALID_SOURCE		=> "Invalid source file",
	DENSE_CLONE_INVALID_CLONE		=> "Invalid clone file",
	DENSE_CLONE_INVALID_DIR			=> "Directory does not exist",
	DENSE_CLONE_NOT_QTREE_ROOT		=> "Directory is not qtree root",
	DENSE_CLONE_SNAP_DEL			=> "Snapshot used for cloning has been deleted",
	DENSE_CLONE_GET_SNAPID		 	=> "Error in getting snapid of the snapshot used for cloning",
	DENSE_CLONE_SMALL_DATA			=> "Data is too small for cloning",
	DENSE_CLONE_WORM_VOL			=> "Volume is a SnapLock volume",
	DENSE_CLONE_CHANGELOG_OFF		=> "Change logging is off on the volume",
	DENSE_CLONE_BOOT_STOP			=> "Clone operation restart was aborted by user",
	DENSE_CLONE_MAX_OPS_FILER		=> "Maximum concurrent clone operations already running on the filer",
	DENSE_CLONE_MAX_OPS_VFILER		=> "The maximum number of concurrent clone operations already running in vFiler context",
	DENSE_VDISK_SRC_LUN_CLONE 		=> "Source LUN is a LUN clone backed by snapshot",
	DENSE_LOG_CLONE_START			=> "clone started",
	DENSE_LOG_CLONE_DONE			=> "Clone operation on file %file% completed successfully. The clone operation ID was %id%.",
	DENSE_LOG_CLONE_RESTART			=> "clone operation restarted.",
	DENSE_LOG_CLONE_STOP			=> "clone operation stopped by user.",
	DENSE_LOG_CLONE_RESTART			=> "Clone operation on file %file% restarted. The clone operation ID is %id%.",
	DENSE_LOG_CLONE_RESTART_FAIL	=> "Clone operation on file %file% failed to restart with error %error%. The clone operation ID is %id%.",
	DENSE_LOG_CLONE_STOP			=> "Clone operation on file %file% was aborted successfully. The clone operation ID was %id%.",
	DENSE_LOG_CLONE_FAIL			=> "Clone operation on file %file% failed with error %error%. The clone operation ID is %id%.",
	DENSE_LOG_CLONE_RESTART			=> "Clone operation on file '%file%' restarted. The clone operation ID is %id%.",
	DENSE_LOG_CLONE_RESTART_FAIL	=> "Clone operation on file '%file%' failed to restart with error '%error%'. The clone operation ID is %id%.",
	DENSE_LOG_CLONE_STOP			=> "Clone operation on file '%file%' was aborted successfully. The clone operation ID was %id%.",
	DENSE_LOG_CLONE_FAIL			=> "Clone operation on file '%file%' failed with error '%error%'. The clone operation ID is %id%.",
	DENSE_LOG_CLONE_BOOT			=> "Boot work for clone was unsuccessful",

	DENSE_FOLD_CLONE_INPRG			=> "fold: clone operation is running on the volume",
	DENSE_SNAPRESTORE_CLONE_INPRG	=> "snap restore: clone operation is running on the volume",
	DENSE_CLONE_SFILE_SMALL			=> "The donor file is too small",
	DENSE_CLONE_VCLONE_SPLIT_INPROG	=> "Clone split is running on the volume",
	
	DENSE_CLONE_START_CIFS_RESERV	=> "The recipient file has cifs hole reservation",
	clog0_name 						=> "clogdir.0",
	clog1_name 						=> "clogdir.1",

	DENSE_CLONE_VFILER_NO_ACCESS		=> "The vFiler does not have access to the source path",
	DENSE_CLONE_NO_SUCH_OPERATION		=> "No such clone operation",
	
	DENSE_COMPRESSION_HEADER			=>  281474976710654,
	DENSE_COMPRESSION_TRAILER			=>  281474976710652,
);

our %CloneMessage = (
	DEFAULT	=> \%CloneDefaultMessage,
	IC		=> \%CloneDefaultMessage,
	BR		=> \%CloneDefaultMessage,
	RR		=> \%CloneDefaultMessage,
	SN		=> \%CloneDefaultMessage,
);

sub GetCloneMessage 
{
	my ( %opts ) = validate( @_, { 
									message	=> {type => SCALAR},
									host	=> {type => SCALAR},
									vol		=> {type => SCALAR, optional => 1},
									file	=> {type => SCALAR, optional => 1},
									id		=> {type => SCALAR, optional => 1},
									error	=> {type => SCALAR, optional => 1},
								}
							);

	my $version = getwv(host=>$opts{host});
	my $message = $opts{message};
        my $msg = "";

	if( $version )
	{
		$msg = $CloneMessage{$version}{$message};
		logcomment("Version=[$version], msg=[$msg]");
	}
	else
	{
		$msg = $CloneMessage{"DEFAULT"}{$message};
		logcomment("Version could not be retrieved: $version, falling back to DEFAULT error message: [$msg]");
	}

	$msg =~ s/%vol%/$opts{vol}/g if( defined $opts{vol} );
	$msg =~ s/%file%/$opts{file}/g if( defined $opts{file} );
	$msg =~ s/%id%/$opts{id}/g if( defined $opts{id} );
	$msg =~ s/%error%/$opts{error}/g if( defined $opts{error} );
	return "$msg";
}

# end of file 
1;

