# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Dense/Constant/Command.pm#1 $
## @summary	Library that defines dense command 
## @description
##	        This package defines generic dense command 
##
## @usage
##	use Dense::Constant::Command;
## @author dl-dpgqa-automation@netapp.com
## @status in progress
## @pod here

=head1 NAME

 Message - Command file for dense 

=head1 SYNOPSIS

 Use Dense::Constant::Command

=cut

package Dense::Constant::Command;
use strict;

# the following messages are from common/cmds/dense.c file 

use constant DENSE_DEFAULT_TIMEOUT        => 3600;
use constant DENSE_NAME        => "a_sis";
use constant DENSE_OLD_NAME        => "nearstore_asis2";
use constant COMPRESSION_NAME        => "compression";
use constant COMPRESSION_LICENSE_CODE        => "KVTAOTB";
use constant DENSE_CMD         => "priv set -q test; ";
use constant DENSE_ADV_CMD     => "priv set -q advanced; ";
use constant DENSE_DIAG_CMD    => "priv set -q diag; ";
use constant DENSE_TEST_CMD    => "priv set -q test; ";
use constant DENSE_CONFIG_NAME => "sis config "; 
use constant DENSE_ON_NAME     => "sis on "; 
use constant DENSE_OFF_NAME    => "sis off "; 
use constant DENSE_START_NAME  => "sis start "; 
use constant DENSE_STOP_NAME   => "sis stop "; 
use constant DENSE_STATUS_NAME => "sis status "; 
use constant DENSE_STAT_NAME   => "sis stat "; 
use constant DENSE_UNDO_NAME   => "sis undo ";
use constant DENSE_CHECK_NAME  => "sis check "; 
use constant DENSE_VOL_LIMIT_NAME  => "sis max_vol_size "; 
use constant DENSE_REVERT_NAME  => "sis revert_to"; 
use constant DENSE_SORT_NAME   => "sis sort ";
use constant DENSE_SHARE_NAME  => "sis share ";
use constant DENSE_UNPACK_NAME => "sis unpack ";
use constant DENSE_CLONE_NAME => "sis clone ";
use constant DENSE_CLONE_START => "clone start";
use constant DENSE_CLONE_STOP =>  "clone stop";
use constant DENSE_CLONE_STATUS => "clone status";
use constant DENSE_HELP_NAME => "sis help";
use constant VOL_COMPRESS_START => "vol compress start";
use constant VOL_COMPRESS_STATUS => "vol compress status";

### Snap status command to check sfsr status
use constant DENSE_SFSR_STATUS => "snap status -r ";

# the following is added for internal use

# the status strings
use constant DENSE_STATUS_INIT       => "Initializing";
use constant DENSE_STATUS_IDLE       => "Idle";
use constant DENSE_STATUS_ACTIVE     => "Active";
use constant DENSE_STATUS_UNDOING    => "Undoing";
use constant DENSE_STATUS_PENDING    => "Pending";
#use constant DENSE_STATUS_GATHERING => "Gathering";
#use constant DENSE_STATUS_SORTING   => "Sorting";
#use constant DENSE_STATUS_MERGING   => "Merging";

# the stage strings
use constant DENSE_STAGE_GATHERING      => "Gathering";
use constant DENSE_STAGE_SORTING        => "Sorting";
use constant DENSE_STAGE_SAVING         => "Saving";
use constant DENSE_STAGE_CHECKING_P0    => "Checking_P0";
use constant DENSE_STAGE_CHECKING_P1    => "Checking_P1";
use constant DENSE_STAGE_CHECKING_P2    => "Checking_P2";
use constant DENSE_STAGE_DONE           => "Done";
use constant DENSE_STAGE_DASH           => "-";

use constant DENSE_CKPT_STAGE_VALUE  => {
					 GATHERING => "2",
                                         SORTING => "3", 
			  	         DEDUPING_P1 => "4",  
				         DEDUPING_P2 => "5",  
				         CHECKING_P1 => "7", 
                                         CHECKING_P2 => "8", 
                         };


# Define a constant for Simulator hosttypes
# filer-simulator for simulators and filer-vsim for VSIMs
use constant DENSE_SIMULATOR_NAME	=> "sim";

# Dedupe is not supported in earlier releases
use constant MIN_SUPPORTED_DEDUPE_RELEASE	=> "7.2";

# Number of Active Ops supported on Vfiler
use constant SIS_MAX_ACTIVE_VFILER_OPS                => "sis.max_vfiler_active_ops";

# the state strings
use constant DENSE_STATE_ENABLED    => "Enabled";
use constant DENSE_STATE_DISABLED   => "Disabled";

# the vol size limit strings
# NOTE these constants are here to support old scripts.  They are deprecated
# for new scripts; use VOL_SIZE_LIMIT in Message_XX.pm instead.  These constants
# should be eliminated when possible.
use constant VOL_SIZE_R200	=> '4096g';
use constant VOL_SIZE_R150	=> '4096g';
use constant VOL_SIZE_FAS3020	=> '1024g';
use constant VOL_SIZE_FAS3050   => '2048g';
use constant VOL_SIZE_FAS940   => '2048g';
use constant VOL_SIZE_FAS2020   => '512g';
use constant VOL_SIZE_FAS2050   => '1024g';
use constant VOL_SIZE_FAS2040   => '3072g';
use constant VOL_SIZE_FAS3040   => '3072g';
use constant VOL_SIZE_FAS3070   => '6144g';
use constant VOL_SIZE_FAS6070   => '13312g'; 
## use constant VOL_SIZE_FAS6080   => '13312g'; 
use constant VOL_SIZE_FAS6080   => '14716g'; 
use constant VOL_SIZE_FAS6030   => '10240g'; 
use constant VOL_SIZE_FAS6040   => '10240g'; 

# vol size limit HashRef
# NOTE these constants are here to support old scripts.  They are deprecated
# for new scripts; use VOL_SIZE_LIMIT in Message_XX.pm instead.  These constants
# should be eliminated when possible.
use constant VOL_SIZE_LIMIT => { 
			   R200      => '4096g',
			   R150      => '4096g',
			   FAS940   => '2048g',
			   FAS2020  => '512g',
			   FAS2050  => '1024g',
			   FAS3020   => '1024g',
			   FAS3050   => '2048g',
			   FAS2040   => '3072g',
			   FAS3040   => '3072g',
			   FAS3070   => '6144g',
			   FAS6070   => '13312g',
## 			   FAS6080   => '13312g',
 			   FAS6080   => '14716g',
			   FAS6030   => '10240g', 
			   FAS6040   => '10240g' 
			 };


use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(
              DENSE_NAME
              DENSE_OLD_NAME
	      COMPRESSION_NAME	
	      COMPRESSION_LICENSE_CODE
              DENSE_CMD
	      DENSE_ADV_CMD
	      DENSE_DIAG_CMD
	      DENSE_TEST_CMD
              DENSE_HELP_NAME
              DENSE_CONFIG_NAME 
              DENSE_ON_NAME 
              DENSE_OFF_NAME
              DENSE_START_NAME
              DENSE_STOP_NAME
              DENSE_STATUS_NAME
              DENSE_STAT_NAME
              DENSE_UNDO_NAME
	      DENSE_REVERT_NAME
              DENSE_CHECK_NAME
              DENSE_VOL_LIMIT_NAME
              DENSE_SORT_NAME
	      DENSE_SHARE_NAME
	      DENSE_UNPACK_NAME
	      DENSE_CLONE_NAME
	      DENSE_STATUS_INIT
              DENSE_STATUS_IDLE
              DENSE_STATUS_ACTIVE
              DENSE_STATUS_UNDOING
              DENSE_STATUS_PENDING
              DENSE_STATE_ENABLED
              DENSE_STATE_DISABLED
              DENSE_STATE_ENABLED
              DENSE_STATE_DISABLED
              DENSE_STAGE_GATHERING
              DENSE_STAGE_SORTING
              DENSE_STAGE_SAVING
              DENSE_STAGE_DONE
              DENSE_STAGE_DASH
              DENSE_STAGE_CHECKING_P0 
              DENSE_STAGE_CHECKING_P1 
              DENSE_STAGE_CHECKING_P2 
              DENSE_CLONE_START
              DENSE_CLONE_STOP
              DENSE_CLONE_STATUS
	      DENSE_SIMULATOR_NAME
	      MIN_SUPPORTED_DEDUPE_RELEASE
	      VOL_SIZE_R200
	      VOL_SIZE_R150
	      VOL_SIZE_FAS940
	      VOL_SIZE_FAS3020
              VOL_SIZE_FAS3050
	      VOL_SIZE_FAS2020
	      VOL_SIZE_FAS2050
	      VOL_SIZE_FAS2040
	      VOL_SIZE_FAS3040
	      VOL_SIZE_FAS3070
	      VOL_SIZE_FAS6030
	      VOL_SIZE_FAS6070
	      VOL_SIZE_FAS6080
	      VOL_SIZE_FAS6040
	      VOL_SIZE_LIMIT
              DENSE_STAGE_VALUE
	      SIS_MAX_ACTIVE_VFILER_OPS
		  DENSE_SFSR_STATUS
              DENSE_DEFAULT_TIMEOUT
	      VOL_COMPRESS_START
	      VOL_COMPRESS_STATUS
          );

# end of file 
1;
