# perl -*-
## @summary	Library that defines dense messages in FS.
## @description
##	        This file defines dense messages used by FS.
##
## @usage
##	Not to be used directly. To be invoked by lib/Dense/Constant/Message.pm.
##  Make sure that the @EXPORT variable defined in that file contains ALL
##  constants defined here.
## @author dl-drg-qa@netapp.com
## @author dshiva@netapp.com
## @status review
## @pod here

=head1 NAME

 Message_fs.pm - Message file for dense operation. Contains FS specific
 messages.

=head1 SYNOPSIS

 Not to be used directly. To be invoked by lib/Dense/Constant/Message.pm.
 Make sure that the @EXPORT variable defined in that file contains ALL
 constants defined here.

=cut

package Dense::Constant::Message_fs;

use strict;

#BEGIN {
#	use vars ('@ISA', '@EXPORT');
#	require	Exporter;
#
#	@ISA	= qw(Exporter);
#	@EXPORT	= qw(
#			FSConsts
#		    );
#}

our @Filer_Models = (
                       "FAS2040",
                       "FAS3040",
                       "FAS3070",
                       "FAS3140",
                       "FAS3160",
                       "FAS3170",
                       "FAS3210",
                       "FAS3240",
                       "FAS3260",
                       "FAS6030",
                       "FAS6040",
                       "FAS6070",
                       "FAS6080",
                       "FAS6210",
                       "FAS6240",
                       "FAS6280",
                       "VSA1000",
                       "VSA2000",
    # IBM Names
                       "N3400",    # equivalent to FAS2040
                       "N5300",    # equivalent to FAS3040
                       "N5600",    # equivalent to FAS3070
                       "N6040",    # equivalent to FAS3140
                       "N6060",    # equivalent to FAS3160
                       "N6070",    # equivalent to FAS3170
                       "N6210",    # equivalent to FAS3210
                       "N6240",    # equivalent to FAS3240
                       "N6260",    # equivalent to FAS3260
                       "N7600",    # equivalent to FAS6030
                       "N7700",    # equivalent to FAS6040
                       "N7800",    # equivalent to FAS6070
                       "N7900",    # equivalent to FAS6080
                       "N7550T",   # equivalent to FAS6210
                       "N7750T",   # equivalent to FAS6240
                       "N7950T",   # equivalent to FAS6280
                       );

our %FSConsts = (
# the following messages are from common/dense/utils.c file
"-"                     => "-",
DENSE_OK                => "Operation succeeded",
DENSE_STOP              => "Operation was stopped",
DENSE_STOP_IDLE         => "Operation is currently idle",
DENSE_REVERTING         => "Revert is in progress",
DENSE_IRON_RUNNING      => "WAFL_Iron is running",
DENSE_VOL_NONEXIST      => "Volume does not exist",
DENSE_VOL_OFFLINE       => "Volume is offline",
DENSE_VOL_NOFLEX        => "Volume type has to be Flexvol",
DENSE_VOL_RDONLY        => "Volume is read-only",
DENSE_VOL_NOFLAG        => "Volume is not SIS enabled",
DENSE_VOL_BIG           => "Volume or maxfiles exceeded max allowed for SIS",
DENSE_VOL_TRANSITING    => "Volume state transition is in progress",
DENSE_VOL_RESTRICTED    => "Volume is restricted",
DENSE_NO_CHANGELOGGING  => "SIS logging is not enabled",
DENSE_SNAPSHOT_CREATE   => "Snapshot creation failed",
DENSE_SNAPSHOT_DELETE   => "Snapshot deletion failed",
DENSE_SNAPSHOT_NONEXIST => "Snapshot does not exist",
DENSE_METAFILE_CREATE   => "Creation of metafile failed",
DENSE_METAFILE_DELETE   => "Deletion of metafile failed",
DENSE_METAFILE_NONEXIST => "Metafile does not exist",
DENSE_METAFILE_READ     => "Read from metafile failed",
DENSE_METAFILE_WRITE    => "Write to metafile failed",
DENSE_METAFILE_GETATTR  => "Getattr on metafile failed",
DENSE_METAFILE_SETATTR  => "Setattr on metafile failed",
DENSE_METAFILE_RENAME   => "Rename of metafile failed",
DENSE_METAFILE_CHECK    => "Metafile check operation failed",
DENSE_METAFILE_INVAL_HDR=> "Metafile has invalid header",
DENSE_INVAL_SCHED       => "Invalid schedule string specified",
DENSE_NOT_CONFIGURED    => "Operation is not configured",
DENSE_NOT_ENABLED       => "Operation is not enabled",
DENSE_NOT_DISABLED      => "Operation is not disabled",
DENSE_ALREADY_ENABLED   => "Operation has already been enabled",
DENSE_ALREADY_DISABLED  => "Operation has already been disabled",
DENSE_REG_SET_FAILED    => "Failed to save configuration",
DENSE_REG_GET_FAILED    => "Failed to retrieve configuration",
DENSE_BUSY              => "Another sis operation is currently active",
DENSE_IDLE              => "Operation is currently idle",
DENSE_CONCURRENT_LIMIT  => "Too many concurrent operations",
DENSE_SCAN_FAILED       => "Failed to start scanner",
DENSE_ABORTED           => "Operation was aborted",
DENSE_ABORT_PENDING     => "Operation was no longer pending",
DENSE_NOMEM             => "Not enough memory",
DENSE_PARTNER_UNSUP_OP => "Operation not supported on partner volume",
DENSE_IS_SV_TYPE        => "Invalid operation on a SnapVault secondary volume",
DENSE_UNINITIALIZED     => "SIS is not initialized",
DENSE_NOT_LICENSED      => "SIS is not licensed",
DENSE_LICENSE_EXPIRED   => "SIS license has expired",
DENSE_INVAL_CONFIG_FILE => "SIS configuration file content is invalid",
DENSE_CREATE_MISSING_CONFIG => "Creating missing configuration for SIS volume",
DENSE_PROCESS_CREATE    => "Not enough internal resources",
DENSE_LOGFILE_WRITE     => "Not enough disk space in root volume to log information",
DENSE_LOGFILE_CREATE    => "Creation of undo log file failed",
DENSE_RESTART_FAIL      => "Checkpoint restore failed",
DENSE_CHKPNT_EXPIRED    => "Checkpoint is older than expected",

DENSE_STALE_DINODE      => "Stale donor inode",
DENSE_STALE_RINODE      => "Stale recipient inode",
DENSE_STALE_DBUF        => "Donor block no longer exists",
DENSE_STALE_RBUF        => "Recipient block no longer exists",
DENSE_DATA_MISMATCH     => "Data block mismatch",
DENSE_MAX_REFCOUNT      => "Too many references",
DENSE_HOLE_RERVD        => "The recipient file has hole reservation",
DENSE_VOL_DIFF          => "Donor and recipient exist on different volume",
DENSE_DONOR_TOO_SMALL   => "The donor file is too small",
DENSE_RECIPIENT_TOO_SMALL => "The recipient file is too small",
DENSE_NO_SPACE          =>  "Not enough disk space for the operation",
DENSE_SHARE_NO_SPACE          =>  "Not enough disk space for the operation",
DENSE_NO_INODE          => "Not enough inodes exist for the operation",
DENSE_SHUTDOWN          => "System is shutting down",
DENSE_FAILOVER          => "System is doing takeover or giveback",
DENSE_DONOR_RANGE       => "Donor file is out of range",
DENSE_RECPT_RANGE       => "Recipient file is out of range",
DENSE_VFILER_ACCESS     => "VFiler doesn't have access to the Volume",
DENSE_VFILER_LIMIT      => "Maximum SIS operation limit reached on VFiler",
DENSE_VFILER_NOEXIST    => "VFiler doesn't exist",
DENSE_VFILER_NO_ACCESS	=> "VFiler does not have access to the Volume",
MAX_DENSE_VFILER_OPS	=> "8",

DENSE_SAME_VBN          => "Blocks have already been shared",
DENSE_SAME_CONTENT      => "Blocks have the same content",
DENSE_UNSUPPORTED_VOLUME => "ASIS is not supported on this volume Type",
DENSE_CLONE_SPLITTING   => "Clone split is running on the volume",
DENSE_CLONE_SRC_RANGE_CHECK => "Source block range out of file",
DENSE_CLONE_DST_RANGE_CHECK => "Destination block range out of file",
DENSE_QUOTA_CHECK		=> "Quota check failed",

# the following messages are strings returned by command line.
DENSE_SIS_FOR           => "SIS for",
DENSE_OP_MESSAGE        => "The SIS operation for",
DENSE_OP_STARTED        => "is started.",
DENSE_OP_ON 	         => "The operation on",
DENSE_OP_STOPPED        => "is being stopped.",
DENSE_OP_ENABLED	     => "is enabled.",
DENSE_OP_DISABLED	     => "is disabled.",

# These are SIS related messages returned by the revert_to command on the filer
DENSE_CANNOT_REVERT     => "Cannot revert the volume (\\S+) because volume size or maxfiles exceeded max allowed for SIS.",
DENSE_RUN_REVERT        => "Please revert SIS metafiles on Volume (\\S+)",
DENSE_UPGRADE_NOT_DONE  => "SIS metafiles upgrade not yet done",
DENSE_DOWNGRADE_EXIST   => "Previously downgraded SIS metafiles exists",
DENSE_REVERT_SUCCESS	=> "'sis revert_to' command has succeeded on volume",

# JSM - these are new.  they replace the three messages below.  these actually
# are different between GB and IC/BR/RR.
WAFL_MAX_DENSE_SIZE_EXCEEDED     => "vol size: Selected SIS volume size is too large",
WAFL_MAX_DENSE_AUTOSIZE_EXCEEDED => "vol autosize: Selected SIS volume size is too large",
WAFL_MAX_VOL_SIZE_EXCEEDED		 => "vol size: Selected volume size is too large",
WAFL_MAX_VOL_AUTOSIZE_EXCEEDED	 => "vol autosize: Selected volume size is too large",

# the following message are strings from cmds/vol.c
# JSM - These should go away.  They remain here for legacy support:
WAFL_MAX_DENSE_SIZE_MB_SPECIFIED => "vol size: Specified SIS volume size is too large",
WAFL_MAX_DENSE_SIZE_MB_SELECTED  => "vol autosize: Selected SIS volume size is too large",
WAFL_MAX_DENSE_SIZE_IC_SPECIFIED => "Selected Volume size is too large",

# the following message are strings from cmds/dense.c
DENSE_CONFIG_NOT_FOUND  => "No Config entry found",
DENSE_STAT_NOT_FOUND    => "No statistics entry found",
DENSE_STATUS_NOT_FOUND  => "No status entry found",
DENSE_FINGER_PRINT      => "Fingerprints of",
DENSE_VERIFY            => "are being verified.",
DENSE_RUN_SIS_START_FIRST	=> "'sis start' needs to be run before this operation",

# the following message are strings from dense/admin.c
DENSE_ASIS2_DEFAULT_SCHED_STR => 'sun-sat@0',
DENSE_ASIS1_DEFAULT_SCHED_STR => '-',

# The following constants are for options
DENSE_OPT_ALLOW_PLATFORMS => "sis.allow_platforms",
DENSE_OPT_ON => "on",
DENSE_OPT_OFF => "off",
SIS_MAX_ACTIVE_OPS => "8",

clog0_name  =>  "clogdir.0",
clog1_name  =>  "clogdir.1",

# CMODE - Following constants are for CMODE 
SIS_ENABLE_ON_RESTRICTED_VOLUME         => "Failed to enable efficiency on volume (\\S+) of Vserver (\\S+) Volume is restricted.",
SIS_ENABLE_ON_OFFLINE_VOLUME            => "Failed to enable efficiency on volume (\\S+) of Vserver (\\S+) Volume is offline.",
SIS_ENABLE_ON_VOLUME                    => "Efficiency for volume (\\S+) of Vserver (\\S+) is enabled. (\\n)Already existing data could be processed by running \"volume efficiency start \-vserver (\\S+) \-volume (\\S+) \-scan\-old\-data true\"",
SIS_ENABLE_ON_ACTIVE_DENSE_VOLUME       => "Failed to enable efficiency on volume (\\S+) of Vserver (\\S+) Another sis operation is currently active.",
SIS_USING_INVALID_PARAMS                => "Volume \\S+ does not exist in Vserver \\S+. Reason: entry doesn't exist",
SIS_USING_INVALID_VSERVER               => "Vserver (\\S+) does not exist.",
SIS_ENABLE_ON_AFTER_DELETING_VOLUME     => "Failed to enable efficiency on volume (\\S+) of Vserver (\\S+): Illegal dsid: Volume does not exist.",
SIS_ENABLE_ON_NOT_REACHABLE_VOLUME      => "Failed to enable efficiency on volume (\\S+) of Vserver (\\S+):",
SIS_DISABLE_ON_VOLUME                   => "Efficiency for volume (\\S+) of Vserver (\\S+) is disabled.",
SIS_DISABLE_ON_RESTRICTED_VOLUME        => "Failed to disable efficiency on volume (\\S+) of Vserver (\\S+) Volume is restricted.",
SIS_DISABLE_ON_OFFLINE_VOLUME           => "Failed to disable efficiency on volume (\\S+) of Vserver (\\S+) Volume is offline.",
SIS_DISABLE_ON_ACTIVE_DENSE_VOLUME      => "Failed to disable efficiency on volume (\\S+) of Vserver (\\S+) Another sis operation is currently active.",
SIS_STOP_ON_DENSE_DISABLED_VOLUME       => "Failed to stop efficiency on volume \"\\S+\" of Vserver \"\\S+\": Operation is currently idle.",
SIS_STOP_ON_DENSE_ENABLED_VOLUME        => "Failed to stop efficiency on volume \"\\S+\" of Vserver \"\\S+\": Operation is currently idle.",
SIS_STOP_ON_RESTRICTED_VOLUME           => "Failed to stop efficiency on volume \"\\S+\" of Vserver \"\\S+\": Volume is restricted.",
SIS_START_ON_EFFICIENCY_DISABLED_VOLUME => "Failed to start efficiency on volume \"\\S+\" of Vserver \"\\S+\": Operation is not enabled.",
SIS_START_ON_COMPRESSION_DISABLED_VOLUME => "command failed: Failed to start efficiency on volume \"\\S+\" of Vserver \"\\S+\": Compression not enabled.",

SIS_NO_MATCHING_ENTRIES                 => "There are no entries matching your query.",
SIS_MISSING_REQUIRED_FIELD              => "\"\\S+\" is a required field",
SIS_ENTRY_DOES_NOT_EXIST                => "entry doesn't exist",

SIS_OFFLINE_COMPRESSION_CANNOT_BE_DISABLED => "Background compression cannot be disabled when inline compression is enabled on the volume. Disable inline compression and retry the command",
SIS_INLINE_COMPRESSION_CANNOT_BE_ENABLED   => "Inline compression or incompressible data detection cannot be enabled when compression is disabled. Also to modify quick check file size compression should be enabled. Enable compression and retry the command",

                                           
SIS_INVALID_OPTIONS                     => "invalid argument \"\\S+\"",
SIS_INVALID_OPTION_VALUE                => "\"\\S+\" is an invalid value for field",
SIS_INVALID_OPTION_VALUE_1              => "invalid argument \"\\S+\"",
SIS_INVALID_OPTION_VALUE_FORMAT_2       => "The value \'\\S+\' is invalid for field",

SIS_START_COMPRESSION_DEDUPE_FALSE		=> "\QAtleast one of options -compression|C and -dedupe|D must be true.",
SIS_START_WHEN_SCAN_IS_FALSE            => "command failed: Options \"\-compression\|\-C\"\, \"\-dedupe\|\-D\"\, \"\-shared\-blocks\|\-a\" and \"\-snapshot\-blocks\|\-b\" are only valid when \"\-scan\-old\-data\|\-s\" is true",


SIS_POLICY_FOR_ASSIGNED_VOLUME_DELETE	=> "The policy is in use by at least one volume.",

SIS_POLICY_MODIFY_INVALID_SCHEDULE		=> "Schedule \\S+ not found",
SIS_POLICY_MODIFY_MISSING_VALUE			=> "Missing value for \\S+.",
SIS_POLICY_MODIFY_NO_VALUES				=> "You need to provide some values to apply.",

SIS_CONFIG_INVALID_SCHEDULE             => "Invalid schedule \"\\S+\": invalid day specification",

SIS_SHOW_FIELDS_ADMIN_MODE => [
    "background-compression", "blks-skipped-sharing",
    "changelog-usage",        "inline-compression",
    "last-op-begin",          "last-op-end",
    "last-op-error",          "last-op-size",
    "logical-data-limit",     "logical-data-percent",
    "logical-data-size",      "min-blocks-shared",
    "op-status",              "path",
    "policy",                 "progress",
    "queued-job",             "ruleset",
    "schedule",               "stale-fingerprint-percentage",
    "state",                  "type",
    "vserver",                "volume",
    "blocks-deduplicated",    "checking-begin-time",
    "checkpoint-op-type",     "checkpoint-progress",
    "checkpoint-stage",       "checkpoint-sub-stage",
    "checkpoint-time",        "compression",
    "dedupe-begin-time",      "duplicate-blocks-found",
    "fingerprints-deleted",   "fingerprints-gathered",
    "fingerprints-sorted",    "gathering-begin-time",
    "last-op-state",          "last-success-op-begin",
    "last-success-op-end",    "policy-id",
    "sorting-begin-time",     "stage",
    "verify-trigger-rate",    "total-verify-time",
    "verify-suspend-count",   "idd",
    "changelog-size",         "changelog-overflow",     
    "vault-transfer-log-size",      "quick-check-fsize",      
    "op-frequency",           "is-constituent",         
    "optimize",               "gathering-phase2-begin-time",
    "compression-changelog-size"
  ],

SIS_STAT_FIELDS_DIAGNOSTIC_MODE => [
    "additional-sharing-messages",      "allocated-data",
    "average-decompress-time",          "avg-compress-cg-time",
    "blks-saved-compression",           "block-sharing-average",
    "block-sharing-histo",              "extra-cp-reads",
    "fp-mismatch-overwrites",           "max-ref-count",
    "mismatch-data-blks",               "num-cg-decompressed",
    "num-check-fail-ops",               "num-check-ok-ops",
    "num-check-susp-ops",               "num-compress-attempts",
    "num-compress-fail",                "num-compressed-bg",
    "num-compressed-blks",              "num-compressed-inline",
    "num-decompress-mark-sw-bad",       "num-decompress-read-bad-blks",
    "num-deferred-ops",                 "num-deleted-ino-recs",
    "num-failed-ops",                   "num-files-too-small",
    "num-fps-deleted",                  "num-max-ref-hits",
    "num-merged-bins",                  "num-mismatch-fps",
    "num-not-enough-savings",           "num-out-of-space-errors",
    "num-p1-merges",                    "num-partial-cg-modifies",
    "num-partial-cg-new-writes",        "num-shared-skipped",
    "num-sis-files",                    "num-snap-skipped",
    "num-sorted-blocks",                "num-stale-donors",
    "num-stale-recipients",             "num-started-ops",
    "num-stopped-ops",                  "num-succeeded-ops",
    "num-unaligned-compression-blocks", "num-uncompressed-blks",
    "overlapped-blocks",                "path",
    "records-sorted",                   "ref-count-average",
    "ref-count-histo",                  "same-data-blocks",
    "same-fp-blocks",                   "same-fbn-blocks",
    "same-vbn-blocks",                  "saved-percentage",
    "shared-data",                      "space-saved",
    "total-process-time",               "total-processed-data",
    "total-verify-time",                "volume",
    "vserver",                          "unflushed-clogs",
    "used-1-pass",                      "used-2-passes",
    "used-3-passes",                    "used-0-splits",
    "used-1-split",                     "used-2-splits",
    "total-space-saved",                "num-qcheck-fail",
    "num-icg-inline",                   "avg-idd-qcheck-time",
    "avg-cd-qcheck-time",               "bce-policy-stage",
    "bce-err-post-processing-nospace",  "num-skipshare-blocks-upper",
    "bce-err-nospace",                  "bce-recieved-in-exempt",
    "num-compressed-bg-cg",             "bce-post-processing-bdata-rejections",
    "bce-err-post-processing-abort",    "bce-post-processing-stage",
    "bce-sending-to-exempt",            "bce-err-abort",
    "bce-fbn-invalid",                  "num-skipshare-blocks-delta",
    "bce-compress-stage",               "bce-err-post-processing-stale-inode",
    "bce-err-stale-inode"
  ],

SIS_SHOW_CONSTANTS => {
                state => "Enabled",
                "background-compression" => "false",
                "inline-compression" => "false",
                "op-status" => "Idle",
                type => "Regular",
                schedule => "sun-sat\@0",
                policy => "-",
                "min-blocks-shared" => "1",
                "blks-skipped-sharing" => "0",
                "queued-job" => "-",
                "ruleset" => "-",
        },


# Release specific failure messages can be catagorized here

# Starting sis on snapmirror secondary gives different errors in different releases
DENSE_START_SECONDARY    => "Volume is read-only",

# Sis status returns different messages for renamed volumes in GB/IC and BR/RR
DENSE_VOL_RENAMED       => "Volume does not exist", 
# Sis status returns different messages for non existant volumes in different releases
DENSE_NONEXISTENT_VOL	=> "Volume does not exist", 

# The following messages are returned in /etc/messages
MESG_DELETE_SNAP => "deleted by the Data ONTAP function dense_delete_snapshot",

# Maximum volume size possible on BR/RR codelines for 32 bit volumes, in GB
MAX_DENSE_VOL_SIZE      => "14716",

# Maximum Volume Size Table, moved from Command.pm
# NOTE these values apply to RR.0 with 32-bit aggregates
VOL_SIZE_LIMIT => { map {$_=>'14716g'} @Filer_Models },
    # RRSizes
# Maximum Volume Size Table, moved from Command.pm
# NOTE these values apply to RR.0 64-bit aggregates
LA_VOL_SIZE_LIMIT => {
        FAS2040 => '27647g',
        FAS3140 => '46079g',
        FAS3160 => '46079g',
        FAS3170 => '64511g',
        FAS6030 => '64511g',
        FAS6040 => '64511g',
        FAS6070 => '92159g',
        FAS6080 => '92159g',
    # IBM Names
        N3400   => '27647g',    # equivalent to FAS2040
        N6040   => '46079g',    # equivalent to FAS3140
        N6060   => '46079g',    # equivalent to FAS3160
        N6070   => '64511g',    # equivalent to FAS3170
        N7600   => '64511g',    # equivalent to FAS6030
        N7700   => '64511g',    # equivalent to FAS6040
        N7800   => '92159g',    # equivalent to FAS6070
        N7900   => '92159g',    # equivalent to FAS6080
    },
);
#FSConsts

# end of file
1;
