# -*- perl -*-
## @summary	Library that defines dense messages in GB.
## @description
##	        This file defines dense messages used by GB.
##
## @usage
##	Not to be used directly. To be invoked by lib/Dense/Constant/Message.pm.
##  Make sure that the @EXPORT variable defined in that file contains ALL
##  constants defined here.
## @author dl-drg-qa@netapp.com
## @author karthiks@netapp.com
## @status review
## @pod here

=head1 NAME

 Message_gb.pm - Message file for dense operation. Contains GB specific
 messages.

=head1 SYNOPSIS

 Not to be used directly. To be invoked by lib/Dense/Constant/Message.pm.
 Make sure that the @EXPORT variable defined in that file contains ALL
 constants defined here.

=cut

package	Dense::Constant::Message_gb;

use strict;

BEGIN {
	use vars ('@ISA', '@EXPORT');
	require	Exporter;

	@ISA	= qw(Exporter);
	@EXPORT	= qw(
			%GBConsts
			GBConsts
		    );
}

our %GBConsts = (
# the following messages are from common/dense/utils.c file
"-"                     => "-",
DENSE_OK                => "Operation succeeded",
DENSE_STOP              => "Operation was stopped",
DENSE_STOP_IDLE         => "Operation is currently idle",
DENSE_REVERTING         => "Revert is in progress",
DENSE_IRON_RUNNING      => "WAFL_Iron is running",
DENSE_VOL_NONEXIST      => "Volume does not exist",
DENSE_VOL_OFFLINE       => "Volume is offline",
DENSE_VOL_NOFLEX        => "Volume is not flexible",
DENSE_VOL_RDONLY        => "Volume is read-only",
DENSE_VOL_NOFLAG        => "Volume is not SIS enabled",
DENSE_VOL_BIG           => "Volume or maxfiles exceeded max allowed for SIS",
DENSE_VOL_TRANSITING    => "Volume state transition is in progress",
DENSE_VOL_RESTRICTED    => "Volume is restricted",
DENSE_NO_CHANGELOGGING  => "SIS logging is not enabled",
DENSE_SNAPSHOT_CREATE   => "Snapshot creation failed",
DENSE_SNAPSHOT_DELETE   => "Snapshot deletion failed",
DENSE_SNAPSHOT_NONEXIST => "Snapshot does not exist",
DENSE_METAFILE_CREATE   => "Creation of metafile failed",
DENSE_METAFILE_DELETE   => "Deletion of metafile failed",
DENSE_METAFILE_NONEXIST => "Metafile does not exist",
DENSE_METAFILE_READ     => "Read from metafile failed",
DENSE_METAFILE_WRITE    => "Write to metafile failed",
DENSE_METAFILE_GETATTR  => "Getattr on metafile failed",
DENSE_METAFILE_SETATTR  => "Setattr on metafile failed",
DENSE_METAFILE_RENAME   => "Rename of metafile failed",
DENSE_METAFILE_CHECK    => "Metafile check operation failed",
DENSE_METAFILE_INVAL_HDR=> "Metafile has invalid header",
DENSE_INVAL_SCHED       => "Invalid schedule string specified",
DENSE_NOT_CONFIGURED    => "Operation is not configured",
DENSE_NOT_ENABLED       => "Operation is not enabled",
DENSE_NOT_DISABLED      => "Operation is not disabled",
DENSE_ALREADY_ENABLED   => "Operation has already been enabled",
DENSE_ALREADY_DISABLED  => "Operation has already been disabled",
DENSE_REG_SET_FAILED    => "Failed to save configuration",
DENSE_REG_GET_FAILED    => "Failed to retrieve configuration",
DENSE_BUSY              => "Operation is currently active",
DENSE_IDLE              => "Operation is currently idle",
DENSE_CONCURRENT_LIMIT  => "Too many concurrent operations",
DENSE_SCAN_FAILED       => "Failed to start scanner",
DENSE_ABORTED           => "Operation was aborted",
DENSE_ABORT_PENDING     => "Operation was no longer pending",
DENSE_NOMEM             => "Not enough memory",
DENSE_NOT_ENOUGH        => "Not enough new data to process",
DENSE_PARTNER_UNSUP_OP => "Operation not supported on partner volume",
DENSE_IS_SV_TYPE        => "Invalid operation on a SnapVault secondary volume",
DENSE_UNINITIALIZED     => "SIS is not initialized",
DENSE_NOT_LICENSED      => "SIS is not licensed",
DENSE_LICENSE_EXPIRED   => "SIS license has expired",
DENSE_PROCESS_CREATE    => "Not enough internal resources",
DENSE_LOGFILE_WRITE     => "Not enough disk space in root volume to log information",
DENSE_LOGFILE_CREATE    => "Creation of undo log file failed",

DENSE_STALE_DINODE      => "Stale donor inode",
DENSE_STALE_RINODE      => "Stale recipient inode",
DENSE_STALE_DBUF        => "Donor block no longer exists",
DENSE_STALE_RBUF        => "Recipient block no longer exists",
DENSE_DATA_MISMATCH     => "Data block mismatch",
DENSE_MAX_REFCOUNT      => "Too many references",
DENSE_HOLE_RERVD        => "The recipient file has hole reservation",
DENSE_VOL_DIFF          => "Donor and recipient exist on different volume",
DENSE_DONOR_TOO_SMALL   => "The donor file is too small",
DENSE_RECIPIENT_TOO_SMALL => "The recipient file is too small",
DENSE_NO_SPACE          =>  "No enough disk space for the operation",
DENSE_SHARE_NO_SPACE          =>  "No enough disk space for the operation",
DENSE_SHUTDOWN          => "System is shutting down",
DENSE_FAILOVER          => "System is doing takeover or giveback",
DENSE_DONOR_RANGE       => "Donor file is out of range",
DENSE_RECPT_RANGE       => "Recipient file is out of range",
DENSE_SAME_VBN          => "Blocks have already been shared",
DENSE_SAME_CONTENT      => "Blocks have the same content",
DENSE_UNSUPPORTED_VOLUME => "ASIS is not supported on this volume Type",
DENSE_CLONE_SPLITTING   => "Clone split is running on the volume",

# the following messages are strings returned by command line.
DENSE_SIS_FOR           => "SIS for",
DENSE_OP_MESSAGE        => "The SIS operation for",
DENSE_OP_STARTED        => "is started.",
DENSE_OP_ON 	         => "The operation on",
DENSE_OP_STOPPED        => "is being stopped.",
DENSE_OP_ENABLED	     => "is enabled.",
DENSE_OP_DISABLED	     => "is disabled.",

# JSM - these are new.  they replace the three messages below.  these actually
# are different between GB and IC/BR.
WAFL_MAX_DENSE_SIZE_EXCEEDED     => "vol size: Specified SIS volume size is too large",
WAFL_MAX_DENSE_AUTOSIZE_EXCEEDED => "vol autosize: Selected SIS volume size is too large",
WAFL_MAX_VOL_SIZE_EXCEEDED		 => "Selected Volume size is too large",
WAFL_MAX_VOL_AUTOSIZE_EXCEEDED	 => "vol autosize: Selected volume size is too large",

# the following message are strings from cmds/vol.c
# JSM - These should go away.  They remain here for legacy support:
WAFL_MAX_DENSE_SIZE_MB_SPECIFIED => "vol size: Specified SIS volume size is too large",
WAFL_MAX_DENSE_SIZE_MB_SELECTED  => "vol autosize: Selected SIS volume size is too large",
WAFL_MAX_DENSE_SIZE_IC_SPECIFIED => "Selected Volume size is too large",

# the following message are strings from cmds/dense.c
DENSE_CONFIG_NOT_FOUND  => "No Config entry found",
DENSE_STAT_NOT_FOUND    => "No statistics entry found",
DENSE_STATUS_NOT_FOUND  => "No status entry found",
DENSE_FINGER_PRINT      => "Fingerprints of",
DENSE_VERIFY            => "are being verified.",

# the following message are strings from dense/admin.c
DENSE_ASIS2_DEFAULT_SCHED_STR => 'sun-sat@0',
DENSE_ASIS1_DEFAULT_SCHED_STR => '-',

# The following constants are for options
DENSE_OPT_ALLOW_PLATFORMS => "sis.allow_platforms",
DENSE_OPT_ON => "on",
DENSE_OPT_OFF => "off",

clog0_name  => "changelog.0",  
clog1_name  => "changelog.1",

# Release specific failure messages can be catagorized here
DENSE_START_FAILURE    => "Operation is not configured",

# Starting sis on snapmirror secondary gives different errors in different releases
DENSE_START_SECONDARY    => "Volume is read-only",

# Sis status returns different messages for renamed volumes in GB/IC and BR
DENSE_VOL_RENAMED       => "No status entry found",
# Sis status returns different messages for non existant volumes in different releases
DENSE_NONEXISTENT_VOL	=> "No status entry found",

# Maximum volume size possible on GB codelines, in GB
MAX_DENSE_VOL_SIZE	=> "14716",

# Maximum Volume Size Table, moved from Command.pm
# NOTE these values apply to all GB releases
VOL_SIZE_LIMIT => {
	R150    =>  '4096g',
	R200    =>  '4096g',
	FAS940  =>  '2048g',
	FAS2020 =>   '512g',
	FAS2050 =>  '1024g',
	FAS3020 =>  '1024g',
	FAS3040 =>  '3072g',
	FAS2040 =>  '3072g',
	FAS3050 =>  '2048g',
	FAS3070 =>  '6144g',
	FAS3140 => '3072g',
	FAS3160 => '6144g',
	FAS3170 => '10240g',
	FAS6030 => '10240g',
	FAS6040 => '10240g',
	FAS6070 => '14716g',
	FAS6080 => '14716g',
    # IBM Names
	N3300   =>   '512g',	# equivalent to FAS2020
	N3600   =>  '1024g',	# equivalent to FAS2050
	N5200   =>  '1024g',    # equivalent to FAS3020
	N5300   =>  '3072g',    # equivalent to FAS3040
	N3400   =>  '3072g',    # equivalent to FAS2040
	N5500   =>  '2048g',    # equivalent to FAS3050
	N5600   =>  '6144g',    # equivalent to FAS3070
	N7600   => '10240g',    # equivalent to FAS6030
	N7700   => '10240g',    # equivalent to FAS6040
	N7800   => '13312g',    # equivalent to FAS6070
	N7900   => '14716g',    # equivalent to FAS6080
    },
    # GBSizes
);
#GBConsts


# end of file
1;
