# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/Dense/Fast_Clones.pm#1 $
#
## @summary 
##    Library routines for constant time sis clones. (7-mode) RR.0 >= 
##
## @author dl-nb-capacity-qa@netapp.com
## @author gourab@netapp.com
## @status in-progress
## @pod here

=head1 NAME

Dense - Basic routines for fast sis clones 

=head1 DESCRIPTION

This module defines routines for fast is clones  

=head1 FUNCTIONS

fast sis clone funtions

=cut

package Dense::Fast_Clones;

use strict;
use SPARSE::Util::Logger;
use filer;
use Tharn qw(connect host say logcomment logresult param tstlog addresult fatal_result);
use Utils qw(make_conn);
use Dense qw(:all);
use Dense::Status; 
use Dense::Constant::Command;
use Dense::Constant::Message;
use Filer::Version;
use Params::Validate qw(validate validate_pos OBJECT SCALAR BOOLEAN 
                        HASHREF ARRAYREF UNDEF SCALARREF);
use Storage::Misc qw(misc_setflag);
use Utils::Units qw(convert_to_bytes);

BEGIN {
Params::Validate::validation_options(
    on_fail => sub {
        my @msgs = @_; 
        chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))
    }
   );
} 
#BEGIN #1

use base qw(Exporter);

BEGIN {
    my  @conf = qw(
                   );

    my @action = qw(
                    dense_fast_clone_start
                    ); 

    my @status = qw(
                    dense_fast_clone_status
                    );
    my @state = qw(
                   );

    my @util   = qw(
                    _get_parent_inum
                    _get_clone_header_stats
                    _setflag_disable_sis_clone_fixup
                    _get_range
                    );


    my @verify = qw(
                    dense_fast_clone_wait_till_complete
                    );

    my @comp   = qw(
                    );

    our %EXPORT_TAGS = ( conf   => [ @conf ],
                         action => [ @action ],
                         status => [ @status ],
                         state  => [ @state ],
                         util   => [ @util ], 
                         verify => [ @verify ],
                         comp   => [ @comp ]
                       );

    $EXPORT_TAGS{'all'} = [@conf, @status, @action, @state, @util, @verify, @comp];

    Exporter::export_ok_tags('all');
}#BEGIN #2

use FS::Util::filer_messages;
use Volume qw (vol_list vol_status);
use vars qw($ENABLE_COMPRESSION $DEFAULT_DLOG_SIS_FILES $DENSE_ONTAP_VERSION $RR_BRANCH);


=head2 dense_fast_clone_start(host=>$host, source=>$path_name, clone=> $clone partner=>$partner 
                          options => '-n' timeout => 36000, snapshot =>$snapshot, $range => "x:y:z a:b:c ...",
                          )

start clone operation

=over 4

=item host

Filer HostSpec

=item source

clone source

=item clone

clone destination

=item partner

cluster partner

=item options

the -n option

=item timeout

Timeout

=item range

string of triplets x:y:z src_fbn:dst_fbn:fbn_cnt

=item Returns:

Success

=over 4

=back

=back

=cut


sub dense_fast_clone_start($@)
{

    my (%opts) = validate (@_, { host => { type=>SCALAR | HASHREF },
                                 source => { type=>SCALAR },
                                 clone => { type=>SCALAR, optional=>1 },
                                 options => { type=>SCALAR, optional=>1 },
                                 timeout => { type=>SCALAR, default=>"3600", optional=>1 },
                                 partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }, 
                                 range => { type=>SCALAR, optional=>1 },
                                 snapshot => {type=>SCALAR, optional => 1},
                                 error_flag => {type=>SCALAR, optional =>1, default=>0},
                                 error_msg => {type=>SCALARREF, optional=>1},
                                }
                          );

    my $host = $opts{host};
    my $source = $opts{source};
    my $clone = $opts{clone};
    my $timeout = $opts{timeout};
    my $options = $opts{options};
    my $range = $opts{range};
    my $snapshot = $opts{snapshot};
    my $error_flag = $opts{error_flag};

    my ($conn) = make_conn($host,"#rsh");
    my $cmd;
    my $cmd_suffix;
    $timeout = 3600 unless $timeout;
    
    if(defined($options)) {
        $options = " " . $options . " ";
    } else {
        $options = " ";
    }

    if (defined($snapshot)) {
        $options = $options . " ". '-s' . " " . $snapshot . " ";
    } 
    
    # "clone" or [dest_path] need not be defined for 
    # clone start <src_path> [dest_path] [-n] <-r <src_blk>:<dest_blk>:<blk_cnt> ...>
    # clone start <src_path> [dest_path] [ -s <snapshot_name> ] <-r <src_blk>:<dest_blk>:<blk_cnt> ...
    # in all other cases check for clone/dest_path
    #
    if(defined($range)) {
       $cmd_suffix = $range;
       $options = $options . " " . '-r' . " ";
    } 

    if($opts{partner}) {
        $cmd = "partner " . DENSE_CLONE_START . " " .  $source . " " . $clone . $options . $cmd_suffix;
    } else {
        $cmd = DENSE_CLONE_START . " " .  $source . " " . $clone . $options . $cmd_suffix;
    }
    my ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd, {timeout=>$timeout});

    if($error_flag == 1) {
       return 0 if($return_text =~ m/failed/ | $return_text =~ m/Invalid/);
    } else { 
       if($return_text =~ m/Clone operation started successfully. ID:\s*(\d+)/) {
            return($1);
       } else {
            die("Unable to start clone operation: $return_text");
       }
    }
}


=head2 dense_fast_clone_status(host=>$host, volume=>$vol, file=>$filename, start_fbn=>100, fbn_cnt=>200 )

Use the cmf_test command to check the status of clone creation

=cut

sub dense_fast_clone_status {
    
    my (%opts) = validate ( @_, { host => { type => SCALAR|HASHREF },
                                  volume =>  { type => SCALAR },
                                  file => { type => SCALAR },
                                  start_fbn => { type => SCALAR, optional => 1, },
                                  fbn_cnt => { type => SCALAR, optional => 1, },
                                  partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }, 
                                  timeout => { type => SCALAR, optional => 1, },
                                }
                            );
                            
    my $host = $opts{host};
    my $volume = $opts{volume};
    my $file = $opts{file};
    my $start_fbn = $opts{start_fbn};
    my $fbn_cnt = $opts{fbn_cnt};
    my $timeout = $opts{timeout};
    
    
    my ($conn) = make_conn($host,"#rsh");
    my $cmd;
    
    $timeout = 3600 unless $timeout;
    
    $cmd = "priv set test; slist /vol/$volume/$file";
    if($opts{partner}) {
        $cmd = "partner priv set test;partner slist /vol/$volume/$file";
    }
    my $file_inode;
    my ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd, {timeout=>$timeout});
    if( $return_text =~ m/(\d+)\s+(\d+)/ ) {
        $file_inode = $1;
    }
    
    $cmd = "priv set test; cmf_test $volume";
    if ( defined($start_fbn) && defined($fbn_cnt) ) {
        if($opts{partner}) {
            $cmd  = "partner priv set test;partner cmf_test $volume";
        }
        $cmd = $cmd . " " . "lookup " . $file_inode . " " . $start_fbn . " " . $fbn_cnt;
        ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd, {timeout=>$timeout});
        if ( $return_text =~ m/No keys found/ ) {
            logcomment("Clone operation has completed");
            return 1;
        }
        else {
             return 0;
        }
    } else {
        $cmd = $cmd . " " . "list";
        if($opts{partner}) {
            $cmd  = "partner priv set test;partner cmf_test $volume list";
        }
        ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd, {timeout=>$timeout});;
        if($return_text =~ m/inum = $file_inode/) {
            return 0;
        }
        else {
            logcomment("Clone operation has completed");
            return 1;
       }
    }

    
}

=head2 dense_fast_clone_wait_till_complete(host=>$host, volume=>$vol, file=>$filename)

Use the cmf_test command to wait till the clone creation is complete

=cut

sub dense_fast_clone_wait_till_complete {
    
    my (%opts) = validate ( @_, { host => { type => SCALAR|HASHREF },
                                  volume =>  { type => SCALAR },
                                  file => { type => SCALAR },
                                  timeout => { type => SCALAR, optional => 1},
                                  start_fbn => { type => SCALAR, optional => 1},
                                  fbn_cnt => { type => SCALAR, optional => 1},
                                  wait_time => { type => SCALAR, optional => 1},
                                }
                            );
                            
    my $host = $opts{host};
    my $volume = $opts{volume};
    my $file = $opts{file};
    my $timeout = $opts{timeout};
    my $wait_time = $opts{wait_time};
    my $start_fbn = $opts{start_fbn};
    my $fbn_cnt = $opts{fbn_cnt};
    
    $timeout = 3600 unless $timeout;
    $wait_time = 2 unless $wait_time;
    
    while($timeout >0) {
        if ( defined($start_fbn) && defined($fbn_cnt) ) {
            if (dense_fast_clone_status(host => $host, volume => $volume, file => $file,
                                    start_fbn => $start_fbn, fbn_cnt=> $fbn_cnt,
                                    ) == 1 ) {
                last;
            }
        }
        else {
            if (dense_fast_clone_status(host => $host, volume => $volume, file => $file)
                                     == 1 ) {
                last;
            }
        }

        sleep $wait_time;
        $timeout = $timeout - $wait_time;
    }
    if($timeout <=0) {
         die ("sis clone has not completed yet");
    }
}

=head2 _get_parent_inum (host=>$host, volume=>$vol,)

Use the cmf_test command to obtain the parent inum for the clone split operation

=cut

sub _get_parent_inum {

    my (%opts) = validate ( @_, { host => { type => SCALAR|HASHREF },
                                  volume =>  { type => SCALAR },
				}
			  );
  
    my $host = $opts{host};
    my $volume = $opts{volume};
 
    my $cmd = "priv set test; cmf_test $volume list";
   
    my ($conn) = make_conn($host,"#rsh");
    my ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd,);
    if($return_text =~ m/Clone-inum/) {
    	my @output_array = split(/\n/,$return_text);
	my $i = 0;
	while ($i < $#output_array){
	    if($output_array[$i] =~ /Parent..*inum/){
	 	@output_array = split(/\s+/,$output_array[$i]);
		return $output_array[3];
	    }
	    $i++;
	}
	return 0;
   } else {
        return 0; #for failure
   }
    
}

=head2 _get_clone_header_stats (host=>$host, volume=>$vol,)

Use the cmf_test command to obtain the clone stats of a volume

=cut 

sub _get_clone_header_stats {

    my (%opts) = validate ( @_, { host => { type => SCALAR|HASHREF },
                                  volume =>  { type => SCALAR },
				}
			  );
  
    my $host = $opts{host};
    my $volume = $opts{volume};
 
    my $cmd = "priv set test; cmf_test $volume header";
   
    my ($conn) = make_conn($host,"#rsh");
    my ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd,);
    
    my @output_array =  split(/\n/,$return_text);
    
    my %clone_header_hash;

    for(my $i=4 ; $i<=6 ; $i++) {
        my @temp = split(":",$output_array[$i]);
        $clone_header_hash{$temp[0]}=$temp[1];
    }                

    return \%clone_header_hash;
}

=head2 _setflag_disable_sis_clone_fixup (host=>$host, value=> 1|0)

Set the wafl_disable_sis_clone_fixup flag

=cut

sub _setflag_disable_sis_clone_fixup {

    my (%opts) = validate ( @_, { 
                                host => { type => SCALAR|HASHREF },
                                value => { type => SCALAR },
                                }
                          );
    
    my $host = $opts{host};
    my $value = $opts{value};

    my $cmd = "priv set test; setflag wafl_disable_sis_clone_fixup $value";

    my ($conn) = make_conn($host,"#rsh");
    my ($return_text, $result_code) = filer_cli_advanced_say($conn, $cmd,);
     
}

=head2 _get_range ( file_size => $size)

Get a valid range for sub-file cloning on the same file.

=cut

sub _get_range {

    my (%opts) = validate ( @_, {
                                file_size => { type => SCALAR }
                                }
                          );

    my $file_size = $opts{file_size};
    my $size_in_bytes = convert_to_bytes($file_size);
    my $range;

    logcomment("Size is $file_size");
    my $blks = int($size_in_bytes/(1024*4));

    logcomment("Number of blocks is $blks");

    while(1)
        {
        my $src_fbn  = int(rand($blks));
        my $dstn_fbn = int(rand($blks));
        my $num_blks = (int(rand($blks) - ($src_fbn)) + 1);

        if ( $src_fbn < 0 || $dstn_fbn < 0 || $num_blks < 1 )
            { next; }
        else {
            if ( $src_fbn > $dstn_fbn ) {
                if ( $dstn_fbn + $num_blks <= $src_fbn && $src_fbn + $num_blks <= $blks) {
                    $range = "$src_fbn:$dstn_fbn:$num_blks";
                    logcomment("Range is $range");
                    return $range;
                }
                next;
            }

            if ( $dstn_fbn > $src_fbn ) {
                if ( $num_blks + $dstn_fbn <= $blks && $src_fbn + $num_blks <= $dstn_fbn ) {
                    $range = "$src_fbn:$dstn_fbn:$num_blks";
                    logcomment("Range is $range");
                    return $range;
                }
                next;
            }
            else
                { next; }
         }
    }
}
    

=head1 AUTHOR

    Gourab Baksi gourab@netapp.com
    $Id: //depot/prod/test/Rsierranevada/lib/Dense/Fast_Clones.pm#1 $
    Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;
