# -*- perl -*-
## @summary Status/Stat routines for dense volume 
## @author dl-dpgqa-automation@netapp.com
## @cf     false
## @status under development
## @pod here


=head1 NAME

Perl routines for Dense.pm. This library only do data manipulation. 

=head1 DESCRIPTION

This module defines routines for Dense.pm use.
This library only do data manipulation and is independent to nate  

=head1 FUNCTIONS

=cut

package Dense::Status;
use Tharn qw(connect host say logcomment logresult param tstlog);
use Utils qw(make_conn);
use Data::Dumper qw(Dumper);
use Params::Validate qw(validate validate_pos OBJECT SCALAR ARRAYREF HASHREF);
use SPARSE::Util::Logger;
BEGIN {
Params::Validate::validation_options(
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
}

#use base qw(Exporter);
require Exporter;
our @ISA= qw(Exporter);
our @EXPORT = qw(
                   dense_parse_config
                   dense_parse_stat
                   dense_parse_stat_detail
                   dense_parse_status
                   dense_parse_status_detail
                   );
our @EXPORT_OK = qw(
                   $PARSE_STRICT
		   );

#require Exporter;
#our @ISA= qw(Exporter);
#our @EXPORT = qw ( 
#                   dense_parse_config
#                   dense_parse_stat
#                   dense_parse_stat_detail
#                   dense_parse_status
#                   dense_parse_status_detail
#                 );

use strict;

use vars qw($PARSE_STRICT);

=head2 dense_parse_config(config_array=>\@config_array)

Parse dense config 

=over4

=item config_array

Array Reference

=item Returns:

Success

=over 4

Return hash of a hash.

libai*> sis config
Path                           Schedule       
/vol/large                     sun-sat@0-23/2 
/vol/s/                        sun-sat@0      
/vol/test/                     sun-sat@0 

The key is the path_name, such as "/vol/large".
Value is the schedule, such as "sun-sat@0-23/2".

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_parse_config
{
    my (%opts) = validate (@_, { config_ref => { type=>ARRAYREF } } );

    my $config_ref = $opts{config_ref};
    my @process = @$config_ref;

    my %config_info=();

    # skip the first line
    # Path                           Schedule       
    @process = dense_strip_header(type=>"CONFIG", array=>$config_ref);

    foreach my $line (@process) {

        # Parse them into hash
        # /vol/large                     sun-sat@0-23/2 
	if ($line =~ /^\s*(.*?)\s+(.*?)\s+(.*?)\s+(.*?)\s+(.*)$/){
	    my $key = $1;
	    my $value = $2;
	    my $bg_compression = $3;
	    my $inl_compression = $4;

	    logcomment("Key=$key, Value=$value, bg-comp=$bg_compression, in-comp=$inl_compression");
	    # get rid of empty space at the end
	    $key =~ s/\s*$//;
	    $value =~ s/\s*$//;
	    $value =~ s/@/\@/;
	    $bg_compression =~ s/\s*$//;
	    $inl_compression =~ s/\s*$//;

	    $config_info{$key} = $value;
	}elsif ($line =~ /^\s*(\S.+)\s+(\S.+)\s+(\d+)\s*$/){
            my $key = $1;
            my $value = $2;
	    my $mbs = $3;

	    logcomment("Key=$key, Value=$value, mbs=$mbs");
            # get rid of empty space at the end
            $key =~ s/\s*$//;
            $value =~ s/\s*$//;
            $value =~ s/@/\@/;
            $mbs =~ s/\s*$//;

            $config_info{$key} = $value;
	    # further when MBS is required, we will have to parse the schedule into days, hours, mins
	    # to satisfy hash refs
	    #$config_info{$key}{mbs} = $mbs;
        }elsif ($line =~ /^\s*(\S.+)\s+(\S.+)\s*$/){
            my $key = $1;
            my $value = $2;

	    logcomment("Key=$key, Value=$value");
            # get rid of empty space at the end
            $key =~ s/\s*$//;
            $value =~ s/\s*$//;

            $config_info{$key} = $value;
        }else{
            die "Unable to parse config:$line\n";
        }
    } # end foreach

    return %config_info;
}

=head2 dense_parse_stat(stat_array=>\@stat_array)

Parse dense stat 

=over4

=item stat_array

Array reference

=item Returns:

Success

=over 4

Return hash of a hash. 

fsr-dedup> dense stat
Path                                 Allocated          Saving       %Saved
/vol/spring                                  0               0          0 %
/vol/summer                                  0               0          0 %
/vol/vol0                                    0               0          0 %
/vol/vol1                                    0               0          0 %
/vol/vol2                                    0               0          0 %


The first level key is the path_name, such as "/vol/summer".
The second level key is: "Allocated", "Saving" and "%Saved" 
$stat_info{$key}{"Allocated"} = 500;
$stat_info{$key}{"Saving"} = 400;
$stat_info{$key}{"%Saved"} = 5 %;

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_parse_stat
{
    my (%opts) = validate (@_, { stat_array => { type=>ARRAYREF } } );

    my $stat_ref = $opts{stat_array};
    my @process = @$stat_ref;

    my %stat_info=();

    # skip the first line
    # Path  Allocated    Saving   %Saved

    @process = dense_strip_header(type=>"STAT", array=>$stat_ref);

    foreach my $line (@process) {

        # Parse them into hash
        # /vol/vol0     352 MB   176 MB    33 %        
        #if ($line =~ /^\s*(.+)\s+(\d+\s\wB)\s+(\d+\s\wB)\s+(\d+\s\%)\s*$/){
        if ($line =~ /^\s*(.+)\s+(\d+\s\w*)\s+(\d+\s\w*)\s+(\d+\s\%)\s*$/){
            my $key = $1;
            my $used = $2;
            my $total_saving = $3;
            my $total_percent = $4;

           # get rid of empty space at the end
           $key =~ s/\s*$//;

           $stat_info{$key}{"Allocated"} = $used;
           $stat_info{$key}{"Saving"} = $total_saving;
           $stat_info{$key}{"%Saved"} = $total_percent;
        }else{
           die "Unable to parse status:$line\n";
        }
    } # end foreach

    return %stat_info;
}

=head2 dense_parse_stat_detail(stat_array=>\@stat_array)

Parse dense detail stat

=over4

=item stat_array 

Array Reference

=item Returns:

Success

=over 4

Return hash of a hash.

fsr-dedup> dense stat -l /vol/summer /vol/spring

Path:                   /vol/summer
Allocated:              0
Shared:                 0
Saving:           	0
%Saved:          	0 %

Path:                   /vol/spring
Allocated:              0
Shared:                 0
Saving:           	0
%Saved:          	0 %


The first level key is the path_name, such as "/vol/summer".
The second level key is: "Allocated", "Shared", "%Saved", etc.

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_parse_stat_detail
{
    my (%opts) = validate (@_, { stat_array => { type=>ARRAYREF } } );
   
    my $stat_array = $opts{stat_array};
    my @process = @$stat_array;

    my %stat_info = ();
    my $key="UNKNOWN";


    foreach my $line (@process) {
        # Path:               /vol/vol1
        # State:              Disabled
        # Base Snapshot:      -

        # It starts with an empty string
	#if (($line =~ /^\s*\d/ )||($line =~ /^\s*</ )||($line =~ /histogram/)||($line =~ /Bucket Sort Stats/)||($line =~ /Sort Stats/)){
	if (($line =~ /Bucket Sort Stats/)||($line =~ /Sort Stats/)){
 		next;
        }
	if ($line =~ /histogram/ ){
		last ;
		}
	if ( $line ne ""){

            # Begin with non-white space character
            if ($line =~ /^\s*(\S.+):\s*(.+?)\s*$/) {

                my $field = $1;
                my $data = $2;

                # Check if line contain path
                # The path is the top leval key, such as /vol/volone
                if ( $field eq "Path"){
                    $key = $data;
                }
                $stat_info{$key}{$field} = $data;
            }else {
                die "Unparsable line in status output: $line\n";
            }
         } # end if empty string
    } # end foreach
    return %stat_info;
}


=head2 dense_parse_status(status_array=>\@status_array)

Parse dense status

=over4

=item status_array

Array Reference

=item Returns:

Success

=over 4

Return hash of a hash.

fsr-dedup> dense status
Path                           State      Status
/vol/spring                    Enabled    Idle
/vol/summer                    Enabled    Idle
/vol/vol0                      Enabled    Idle
/vol/vol1                      Disabled   Idle
/vol/vol2                      Disabled   Idle
/vol/another                   Enabled    Active (14 Blocks processed)\r\n


The first level key is the path_name, such as "/vol/summer".
The second level key is: Path, State, and Status

=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_parse_status
{
    my (%opts) = validate (@_, { status_array => { type=>ARRAYREF } } );

    my $status_array = $opts{status_array};
    my @process = @$status_array;

    my %status_info=();

    # skip the header line
    # Path  State    Status
    @process = dense_strip_header(type=>"STATUS", array=>$status_array);

 
    foreach my $line (@process) {
        #skip first empty line
        # Parse them into hash 

	## Old format
        ## if ($line =~ /^\s*(\S+)\s+(\w+)\s+(.+)\s*$/){
        if ($line =~ /^\s*(\S+)\s+(\w+)\s+(\w+)\s+(.+)\s*$/){
            my $key = $1;
            my $state = $2;
            my $status = $3;
	    my $progress = $4;

            # get rid of empty space at the end, 
            $key =~ s/\s*$//;

            $status_info{$key}{State} = $state;
            $status_info{$key}{Status} = $status;
            $status_info{$key}{Progress} = $progress;

        }else{
            die "Unable to parse status:$line\n";
        }
    } # end foreach

    return %status_info;
}

=head2 dense_parse_status_detail(status_array=>\@status_array)

Parse dense detail status

=over4

=item status_array

Array Reference

=item Returns:

Success

=over 4

Return hash reference 

$VAR1 = {
          '/vol/dense_vol' => {
                                'Sorting Begin' => '-',
                                'Checkpoint stage' => 'Gathering',
                                'Last Operation End' => 'Thu May 21 08:24:19 GMT 2009',
                                'Status' => 'Active',
                                'De-duping Begin' => '-',
                                'Type' => 'Regular',
                                'Checkpoint Progress' => '76000 KB Scanned',
                                'Duplicate Blocks Found' => '0',
                                'Gathering Begin' => 'Thu May 21 08:27:19 GMT 2009',
                                'Global Scanner progress' => {
                                                               'MAX  Inode Num' => '65520',
                                                               'Next Inode Num' => '7050',
                                                               'Blocks Gathered' => '19870',
                                                               'Num scanners aborted' => '0'
                                                             },
                                'Fingerprints Sorted' => '0',
                                'Fingerprints Deleted' => '0',
                                'Checkpoint Data Information' => {
                                                                   'Individual sub scanner checkpoint progress' => {
                                                                                                                     'sub_scanner_3' => {
                                                                                                                                    'Inode Num' => '6668',
                                                                                                                                    'End_Fbn' => '9',
                                                                                                                                    'Fbn' => '0'
                                                                                                                                        },
                                                                                                                     'sub_scanner_1' => {
                                                                                                                                    'Inode Num' => '6665',
                                                                                                                                    'End_Fbn' => '9',
                                                                                                                                    'Fbn' => '10'
                                                                                                                                        },
                                                                                                                     'sub_scanner_4' => {
                                                                                                                                    'Inode Num' => '6667',
                                                                                                                                    'End_Fbn' => '9',
                                                                                                                                    'Fbn' => '0'
                                                                                                                                        },
                                                                                                                     'sub_scanner_2' => {
                                                                                                                                    'Inode Num' => '6666',
                                                                                                                                    'End_Fbn' => '9',
                                                                                                                                    'Fbn' => '0'
                                                                                                                                       }
                                                                                                                   },
                                                                   'Global checkpoint progress' => {
                                                                                                     'Inode Num' => '6668',
                                                                                                     'Blocks Gathered' => '19000',
                                                                                                     'Fbn' => '10',
                                                                                                     'Bucket Count' => '16'
                                                                                                   }
                                                                 },
                                'Blocks De-duplicated' => '0',
                                'Changelog Usage' => '0%',
                                'Stage' => 'Gathering',
                                'Blocks Skipped Sharing' => '0',
                                'Individual Scanner progress' => {
                                                                   'scanner_4' => {
                                                                                    'Next Fbn' => '0',
                                                                                    'Inum' => '7047',
                                                                                    'SubScan Id' => '1'
                                                                                  },
                                                                   'scanner_3' => {
                                                                                    'Next Fbn' => '0',
                                                                                    'Inum' => '7048',
                                                                                    'SubScan Id' => '2'
                                                                                  },
                                                                   'scanner_2' => {
                                                                                    'Next Fbn' => '0',
                                                                                    'Inum' => '7049',
                                                                                    'SubScan Id' => '3'
                                                                                  },
                                                                   'scanner_1' => {
                                                                                    'Next Fbn' => '0',
                                                                                    'Inum' => '7050',
                                                                                    'SubScan Id' => '0'
                                                                                  }
                                                                 },
                                'Path' => '/vol/dense_vol',
                                'Schedule' => '-',
                                'Checkpoint Op Type' => 'Scan',
                                'Checkpoint Sub-stage' => '-',
                                'Last Operation Size' => '0 KB',
                                'Checkpoint Header Information' => {
                                                                     'Checkpoint Time' => 'Thu May 21 02:27:00 GMT 2009',
                                                                     'Checkpoint Magic' => '444e5345',
                                                                     'Checkpoint Checksum' => '1761766084',
                                                                     'Checkpoint Version' => '4',
                                                                     'Checkpoint Op Type' => 'Scan',
                                                                     'Checkpoint Stage' => 'Gathering',
                                                                     'Checkpoint Size' => '1816'
                                                                   },
                                'Last Operation Begin' => 'Thu May 21 08:24:19 GMT 2009',
                                'Last Operation Error' => '-',
                                'State' => 'Enabled',
                                'Checkpoint Time' => 'Thu May 21 02:27:00 GMT 2009',
                                'Minimum Blocks Shared' => '1',
                                'Checking Begin' => '-',
                                'Progress' => '79440 KB Scanned',
                                'Fingerprints Gathered' => '19860'
                              }
        };


=back

Failure

=over 4

Return with an error message in C<$@>

=back

=back

=cut

sub dense_parse_status_detail
{
	logcomment("DEBUG: PARSE_STRICT => $PARSE_STRICT");
	my (%opts) = validate (@_, { status_array => { type=>ARRAYREF } } );

	my $status_array = $opts{status_array};
	my @to_process = @$status_array;

	my $num = scalar(@to_process);

	my %status_info = ();

	my $key="UNKNOWN";
        my ($ckpt_header, $ckpt_data, $progress, $count);
 
	foreach my $line (@to_process) {
		# Path:               /vol/vol1
		# State:              Disabled
		# Base Snapshot:      -
		
		# It starts with a non empty string
		if ($line ne "") {
			if ($line =~ /Checkpoint Header Information/) {
                                $ckpt_header = "Checkpoint Header Information"; 
				next;	
			} elsif ($line =~ /^=+$/) {
                                next;
                        } elsif ($line =~ /Checkpoint Data Information/) {
                                $ckpt_data = "Checkpoint Data Information";
                                $ckpt_header = "";
                                next;
                        } elsif ($line =~ /Global checkpoint progress/) {
                                $progress = "Global checkpoint progress";
                                next;
                        } elsif ($line =~ /Individual sub scanner checkpoint progress/) {
                                $progress = "Individual sub scanner checkpoint progress";
                                $count = 0;
                                next;
                        } elsif ($line =~ /Global Scanner progress/) {
                                $progress = "Global Scanner progress";
                                $ckpt_data = "";
                                next;
                        } elsif ($line =~ /Individual Scanner progress/) {
                                $progress = "Individual Scanner progress";
                                $count = 0;
                                next;
                        } elsif ($line =~ /Verify Check Point Info/) {
				next;
			}

			# Begin with non-white space character
            		if ($line =~ /^\s*(\S.+?):\s*(.+?)\s*$/) {
				my $field = $1;
				my $data = $2;

				# The path is the top level key, such as /vol/volone
				if ( $field eq "Path" ) {
				    $key = $data; 
				    $ckpt_header = "";
				    $ckpt_data = "";
				    $progress = "";
				}
                                    
                                if ($ckpt_header =~ /Checkpoint Header Information/) {  
                                    $status_info{$key}{$ckpt_header}{$field} = $data;
                                } elsif ($ckpt_data =~ /Checkpoint Data Information/) {
                                     if ($progress =~ /Global checkpoint progress/) {
                                         $status_info{$key}{$ckpt_data}{$progress}{$field} = $data;
                                     } elsif ($progress =~ /Individual sub scanner checkpoint progress/) {
                                         my $sub_scanner = "sub_scanner";
                                         $count++ if ($field =~ /Inode Num/);
                                         $sub_scanner = $sub_scanner . "_" . $count;
                                         $status_info{$key}{$ckpt_data}{$progress}{$sub_scanner}{$field} = $data;
                                     } 
                                 } elsif ($progress =~ /Global Scanner progress/) {
                                         $status_info{$key}{$progress}{$field} = $data;
                                 } elsif ($progress =~ /Individual Scanner progress/) {
                                         my $scanner = "scanner";
                                         $count++ if ($field =~ /SubScan Id/);
                                         $scanner = $scanner . "_" . $count;
                                         $status_info{$key}{$progress}{$scanner}{$field} = $data;
                                } else {
                                      $status_info{$key}{$field} = $data;
                                }
                           } else {
				if( $PARSE_STRICT ){
				    die "Unparsable line in status output: $line\n";
				} else {
				    next;
				}
                           }
		} # if empty

   } # end foreach

   return %status_info;

}

=head2 dense_strip_header(type=>$type, array=>\@array)

Remove the header from the array of the config, stat or status list 

=over4

=item type

One of the following, CONFIG, STAT, STATUS

=item array

Reference to Array of entries

=item Returns:

array - Might be empty

=back

=cut

sub dense_strip_header(%)
{
    my (%opts) = validate (@_, { type => { type=>SCALAR },
				 array => { type=>ARRAYREF } }
			  );
    
    my $type = $opts{type};
    my $array = $opts{array};
    my (@content) = @$array;

    my $header;
    if ($type eq "CONFIG"){
	if ($Dense::DENSE_ONTAP_VERSION eq "RR"){
	    $header = qr/^\s*Path\s+Schedule(\s+Minimum Blocks Shared)?(\s+Compression)?(\s+Compression)?\b/;
	} else {
            $header = qr/^\s*Path\s+Schedule(\s+Minimum Blocks Shared)?\b/;
	}
    }elsif($type eq "STAT"){
        $header = qr/^\s*Path\s+Allocated\s+Saving\s+\%Saved\b/;
    }elsif($type eq "STATUS"){
        $header = qr/^\s*Path\s+State\s+\Status\b/;
    } 
logcomment("DENSE ONTAP VERSION -> $Dense::DENSE_ONTAP_VERSION");
logcomment("Array: @content");
    
    if ($type eq "CONFIG") {
	   if ($content[1] !~ /$header/) {
		  die "Unrecgnized format for $type header: $content[0]\n";
	   } else {
		   shift @content; # To skip Inline in RR
		   shift @content; # To skip Header line (Path Schedule Compression Compression)
		   shift @content; # To skip "------"
	   }
    } else {
    if ($content[0] !~ /$header/){
        die "Unrecognized format for $type header: $content[0]\n";
    }else{
        shift @content;
    }
    }
    return @content;
}

=head2 print_hash(hashofhash=>\%hashofhash)

Internal debug use

=over4

=item hashofhash 

hash reference

=item Returns: 

Print out the hash table

=back

=cut

# for debug only
sub print_hash(%)
{
    my (%opts) = validate (@_, { hashofhash => { type=>HASHREF } } );

    my $myKey;
    my $item;

    my $hashref = $opts{hashofhash};
    my %test = %$hashref;

    foreach $myKey (keys  %test) {
        print "$myKey: \n";

        foreach $item (keys %{$test{$myKey}}) {

            print "item is :$item\n";
            print "value is :$test{$myKey}{$item}\n";

        }
    print "\n";
    }
}

=head2 dense_kb_atoi(kb=>$kb)

Convert KB in string format to an actual number

=over4

=item kb

Number of KB in string format

=item Returns: 

item number - Number of KB

=back

=cut

# for debug only
sub dense_kb_atoi(%)
{
    my (%opts) = validate (@_, { kb => { type=>SCALAR } } );

    my ($value) = $opts{kb};

    my $number;
    my $unit;
    if ($value =~ /(\d+)\s*([K|M|G|T])B?/i) {
        $number = $1;
        $unit = uc($2);
    }else{
	die "No value found ($value).\n"; 
    }
	
    # this part might not be needed. Long form always pass in KB
    if ($unit eq "M"){
	$number = $number * 1024;
    }elsif( $unit eq "G"){
	$number = $number * 1024 * 1024;
    }elsif ( $unit eq "T"){
	$number = $number * 1024 * 1024 * 1024;
    }
    return $number;
}

1;
