# -*- perl -*-
# $Id$
## @summary Filer/Volume/Utility routines for dense volume 
## @usage
##	use Dense::Utils;
## @author dl-dpgqa-automation@netapp.com rani@netapp.com abhayjit@netapp.com
## @cf     false
## @status public
## @pod here


=head1 NAME

Utility routines for dense test scripts.

=head1 DESCRIPTION

This module defines utility routines for dense test scripts use.
The routines are built on top of the SPARSE test infrastructure.

=head1 FUNCTIONS

=cut

package Dense::Utils;

require Exporter;
our @ISA= qw(Exporter);
our @EXPORT = qw (
  wafl_scan_speed
  dense_new_filer
  dense_new_vol
  dense_setup_test_vvol
  dense_setup_all
  dense_destroy_vol
  dense_cleanup_vol
  dense_verify_stat
  dense_verify_failure
  dense_get_file_size_and_blkcnt
  dense_verify_file_size_and_blkcnt
  get_snapshot_with_time
  set_metafile_visible
  set_flag_wafliron_panic
  set_iron_scan_suspend_duration
  set_iron_on_demand
  unset_iron_on_demand
  get_wafl_scan_id
  abort_wafl_scan
  wait_for_scan_done
  verify_scan
  create_testdir
  get_vol_savings
  get_all_savings
  is_dense_enabled
  unmount_and_cleanup_vol
  cf_check_state
  dense_cf_status
  set_maxfiles
  parse_showvbns
  sort_spares
  create_new_aggr
  check_old_aggr
  aggr_space_check
  dense_parse_showvbns
  dense_get_unshared_blkcnt
  dense_compare_hashes
  dense_get_inode_num
  dense_parse_dumpblock
  check_inode_dense_flag
  dense_get_quota_index
  dense_is_larger_aggr
  get_subtest_results
  set_overlap_requests
  set_inode_hash
  get_fingerprint_count
  reboot_verify
  dense_syslog_rotate
  dense_get_vol_stats
  dense_check_fsflags
  dense_get_fsflags
  get_sisls_details
  dense_compare_wsv
  dense_wait_sfsr_complete
  get_max_supported_ref_count
  is_debug
  get_snap_id
  setflag_hash_optimization
  dense_file_sync_last_blk
  dump_data
  get_filer_model_info
  set_p1merge_bin_value
  get_p1merge_bin_value
  validate_max_refcount
  tp_clear
  tp_delay
  set_sis_options
  dense_filer_dd
  bootargs
  calculate_expected_savings
  calculate_merge_counter_values
  panic
);

our @EXPORT_OK = qw(
                    $ENABLE_COMPRESSION
                    $DENSE_ONTAP_VERSION
                    $CHECK_32K_REFCNT
                   );

use Sys::Hostname;
use mount;
use cluster;
use Aggregate;
use Aggregate qw( :df aggr_status );
use SPARSE::Resource::Volume;
use SPARSE::Resource::Filer;
use SPARSE::Util::FileIO;
use SPARSE::Util::Logger;
use FS::Util::snapCmds;
use FSR::FSRutils;
use Dense::Constants;
use Dense::Constant::Command;
use Dense::Constant::Message;
use Dense::Constant::Message qw(GetMessage);
use Dense::TestData;
use Dense qw(:all);
#use Tharn qw(connect host say logcomment logresult param tstlog);
use Tharn;
use Utils qw(make_conn);
use Volume qw(vol_exists vol_max_files);
use filer;
use Data::Dumper qw(Dumper);
use Params::Validate qw(validate validate_pos OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use Wafliron;
use Dense::Constant::Clone_Command;
use boot qw(reboot_to boot_state transit_to wait_for_rsh);
use compression;
use FileManip qw(get_file_size setattr_file);
use POSIX;
use ModelInfo;

Params::Validate::validation_options( allow_extra => 1);
BEGIN {
Params::Validate::validation_options(
    # allow presence of options other than those named in the validation specification 
    allow_extra => 1,
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
   @Dense::Utils::EXPORT_OK   = qw($SISUNDO_AT_CLEANUP);
   @Dense::Utils::EXPORT_OK   = qw($WAFLIRON_AT_CLEANUP);
}

use strict;

use vars qw($ENABLE_COMPRESSION);
use vars qw($SISUNDO_AT_CLEANUP);
use vars qw($WAFLIRON_AT_CLEANUP);
use vars qw($DENSE_ONTAP_VERSION);
use vars qw($CHECK_32K_REFCNT);


=head2 dense_new_filer(host=>$hostspec, nfsAcls=>$nfsAcls, filerConn=>$filerConn, noLicense=>$noLicense)

Connect to filer, instantiate a new filer object, and license dense.
A wrapper function for SPARSE::Resource::Filer::sparse_filer().

=over 4

=item hostspec 

Filer name, required.

=item nfsAcls

Options to pass down to sparse_filer(), optional. 
Currently only one option supported {nfsAcls=>1}.
Keep it the same format so it can be passed to sparse_filer directly.

=item filerConn

Options to pass down to sparse_filer(), optional. 
Filer Connection.

=item noLicense

Options to pass down to sparse_filer(), optional. 
Remove Dense License or not.

=item Returns:

Filer object

=back

=cut

sub dense_new_filer(%)
{

# No Validate for wrappers to other functions
#    my (%opts) = validate (@_, { host => { type=>SCALAR },
#				 nfsAcls => { type=>SCALAR, default=>"1", optional=>1 },
#				 filerConn => { type=>SCALAR, optional=>1 },
#				 noLicense => { type=>SCALAR, default=>"0", optional=>1 },
#				 subTest => { type=>SCALAR | UNDEF, optional=>1 } }
#			  );
    my (%opts) = @_;

    my ($hostspec) = $opts{host};
    my ($nfsAcls) = $opts{nfsAcls};
    my ($filerConn) = $opts{filerConn};
    my ($noLicense) = $opts{noLicense};
    my ($mbs) = Dense::Constants::MIN_SHARE_BLKS;

    my ($hashref) = \%opts;

    # use rsh-socket instead of rsh to work around Burt 68700
    $hashref->{filerConn} = DFLT_FILER_CONN	unless $hashref->{filerConn};
    $hashref->{subTest} = 1;

    my $filer = sparse_filer($hostspec, $hashref);
    # Not necessary - never used anywhere -  my $conn = $filer->{conn};

    # Check if license is required, and if it should be licensed.
    # In SN no license code is required for Dedupe
#    if ($hashref->{noLicense}) {
#    	dense_remove_license(host=>$hostspec);
#    } else {
#        dense_set_license(host=>$hostspec);
#    }

	logcomment('Setting dense dbg level to 0'); 
	if(filer_dbg_level_set($filer->{conn}, "dense", 0)) {
		die('Unable to set dense dbg level to 0');
	}
    if($mbs > 0) 
    {
	    logcomment("Setting the MinShareBlks to $mbs");
	    filer_cli_say($filer->{conn}, "options sis.min_share_blks");
	    filer_cli_say($filer->{conn}, "options sis.min_share_blks $mbs");
	    filer_cli_say($filer->{conn}, "options sis.min_share_blks");
    }
    else
    {
	    logcomment("Getting the default MinShareBlks filer-wide");
	    filer_cli_say($filer->{conn}, "options sis.min_share_blks");
    }

    return ($filer);
}

=head2 dense_new_vol(filer=>$filer, volName=>$volName, tradVol=>$tradVol, volSize=>$volSize, purge=>$purge, aggr=>$aggr, aggrSize=>$aggrSize, aggrMinSize=>$aggrMinSize, aggrPurge=>aggrPurge, client=>$client)

Create a volume object for the specified volume, create it if the volume
doesn't exist, and mount the volume.
A wrapper function for SPARSE::Resource::Volume::sparse_volume().

=over 4

=item filer 

Filer object, required.

=item volName 

Name of the new volume.
The volume name to be used will be determined as the following:

use volName if specified in opts, otherwise

use PATH_NAME if defined, otherwise

use DFLT_VOLNAME if not user defined

=item tradVol

To create a traditional volume. Default is flex volume.

=item volSize

Size of the new volume. 
If not specified, the default size is 20MB.

=item purge

If the volume already exists, destroy then re-create it.

=item aggr 

Name of the aggrgate.
If not specified, use "dense_aggr".
If the aggrgate doesn't exist, it will be craeted.

=item aggrSize 

Size of the aggrgate in number of disks.
The default is 5 disks but no less than 50MB 
(this is the default value of aggrMinSize).
If the aggregate exists and aggrPurge is not 1, 
then this parameter is ignored.

=item aggrMinSize

The minimum size of the aggregate if aggregate creation is required.
If the aggregate exists and aggrPurge is not 1, then this parameter 
is ignored. Default value is 51200KB.

=item aggrPurge

If the aggregate already exists, destroy then re-create it.

=item client

client spec, optional. Mount on client instead of on "localhost".

=item Returns:

volume object

=back

=cut

sub dense_new_vol(%)
{
# No Validate for wrappers to other functions
#    my (%opts) = validate (@_, { filer => { type=>HASHREF }, 
#				 volName => { type=>SCALAR | UNDEF, optional=>1 }, 
#				 tradVol => { type=>SCALAR, optional=>1, default=>"0" }, 
#				 volSize => { type=>SCALAR, optional=>1, default=>"3g" }, 
#				 purge => { type=>SCALAR | UNDEF, optional=>1 }, 
#				 aggr => { type=>SCALAR, optional=>1, default=>"dense_aggr" }, 
#				 aggrSize => { type=>SCALAR, optional=>1, default=>"5" }, 
#				 aggrMinSize => { type=>SCALAR, optional=>1, default=>"51200KB" }, 
#				 aggrPurge => { type=>SCALAR, optional=>1, default=>"0" }, 
#				 aggrRaidType => { type=>SCALAR | UNDEF, optional=>1},
#				 subTest => { type=>SCALAR | UNDEF, optional=>1 },
#				 client => { type=>SCALAR | UNDEF, optional=>1 } }
#			  );
    my (%opts) = @_;
    my ($filer) = $opts{filer};
    my ($client) = $opts{client};
    my ($opts_ref) = \%opts;

    if ($opts{client} eq 'localhost') {
        $opts{client} = hostname();
        my @clienta = split(/\./,$opts{client});
        $opts{client} = $clienta[0];
        logcomment("Hostname: $opts{client}");
    }

    if ($main::PATH_NAME) {
	(undef, undef, $opts{volName}) = split('/', $main::PATH_NAME) unless $opts{volName};
    } 

    ## logcomment("DEBUG: Aggr Size is set to $opts{aggrSize}");
    my $sim = DENSE_SIMULATOR_NAME;

    if(! ($opts{aggrSize}) ) {
	    ## If Filer Hosttype is 'filer-simulator', then change default aggrSize to 15
	    if($filer->{conn}->{hostp}->{hosttype} =~ /$sim/) {
		 ## Get List of spares, sort by total avail space and supply the highest to aggrSize	
		 my %spares = sort_spares(host=>$filer->{hostName});
		 if( !(scalar keys %spares) ) {
		     logcomment("WARNING: No spare disks available on filer");
		     $opts{aggrSize} = 4;
		 }else {
		     my ($key, $count, %total_size);
	             foreach $key ( keys %spares ) {
           	         $count = @{ $spares{$key} };
		         $total_size{$key} = ($key * $count) / 1024;
        	     }
		     my @sorted = sort { $total_size {$b} <=> $total_size {$a} } keys %total_size; 
 		     $key = shift(@sorted);
		     $count =  scalar @{ $spares{$key} }; 
		     $opts{aggrSize} = $count;
		 }
    	}
    }

   if(! ($opts{volSize}) ) {
            ## If Filer Hosttype is 'filer-simulator', then change default volSize to 20m
            if($filer->{conn}->{hostp}->{hosttype} =~ /$sim/) {
                 logcomment("Hosttype is $filer->{conn}->{hostp}->{hosttype}. Setting volSize to 20m");
                 $opts{volSize} = "20m";
        }
    }

    
    ## Default aggr size is 4
    if($filer->{conn}->{hostp}->{hosttype} !~ /$sim/) {
	$opts{aggrSize} = 4 unless $opts{aggrSize};
    }

    if ($opts{purge}) {
	my $path = "/vol/" . $opts{volName};
	## Changing the mount point to default_ip because to make it run on the Smoke environment
	my $mountPoint = "/t/" . $filer->{conn}->{hostp}->{default_ip} . "/vol_" . $opts{volName};

	logcomment("Unmounting $mountPoint from $client");
        umount($mountPoint, {client=>$client});
        if($@) { logcomment("Could not unmount $mountPoint on $client: $@"); }

    } 

    $opts{volName} = DFLT_VOLNAME	unless $opts{volName};
    $opts{volSize} = DFLT_VOLSIZE	unless $opts{volSize};
    $opts{aggr} = DFLT_AGGRNAME	unless $opts{aggr};
    $opts{aggrSize} = DFLT_AGGRSIZE   unless $opts{aggrSize};
    $opts{aggrMinSize} = DFLT_MIN_AGGRSIZE   unless $opts{aggrMinSize};
    $opts{aggrBasic} = 1 unless $opts{aggrBasic};
    
    if ( defined($client) ) { $opts{nomount} = 1; }## Never mount the volume if client is specified

    logcomment("Calling dense_new_vol with params = dense_new_vol(filer=>$filer, volName=>$opts{volName}, tradVol=>$opts{tradVol}, volSize=>$opts{volSize}, purge=>$opts{purge}, aggr=>$opts{aggr}, aggrSize=>$opts{aggrSize}, aggrMinSize=>$opts{aggrMinSize}, aggrPurge=>$opts{aggrPurge}, client=>$opts{client}");

    # don't try to make the exportfs options persistent
    $opts{noPersistExport} = 1;

    my $dvol = sparse_volume($filer, $opts{volName}, $opts_ref);

    ###########################################################################
    # Changes for Enabling compression on the volume
    logcomment("Compression flag = $ENABLE_COMPRESSION");
    if($ENABLE_COMPRESSION)
    {
	my $licensed = filer_service_is_licensed($filer->{conn},"compression");
	if(!$licensed){
	    logcomment("Compression not licensed. Adding license string now");
	    filer_service_license($filer->{conn},"compression");
	}
    	logcomment("Enabling Compression for the Volume $opts{volName}");
	enable_compression_option(filer => $filer->{hostName}, volname => $opts{volName});
    }
    ###########################################################################

    if ( defined($client) ) { 

	logcomment("DEBUG: dense_new_vol - Client is $client, Volume = $dvol->{name}");

	say($filer->{conn}, "exportfs -p sec=sys,rw,anon=0,nosuid /vol/$opts{volName}");
	## Changing the mount point to default_ip because to make it run on the Smoke environment
	logcomment("Trying to mount using mount ($filer->{conn}->{hostp}->{default_ip},\"$dvol->{name}\",client=>$client)");
	$dvol->{mountPoint} = mount ($filer->{conn}->{hostp}->{default_ip},"/vol/$dvol->{name}",client=>$client,chmod =>'0777') or
	    die("Could not mount $filer->{conn}->{hostp}->{default_ip} from $client: $@\n");
	logcomment ("Mount point for volume $dvol->{name} is $dvol->{mountPoint}");
    
    }

    return ($dvol);
}

=head2 dense_setup_test_vvol(filer=>$filer, volName=>$volName, tradVol=>$tradVol, volSize=>$volSize, purge=>$purge, aggr=>$aggr, aggrSize=>$aggrSize, aggrMinSize=>$aggrMinSize, aggrPurge=>aggrPurge)

Make a flex dense volume, turn off snap schedule, enable dense operation, 
turn off dense schedule.

=over 4

=item filer 

Filer object, required.

=item options 

Options for creating volumes, optional.
Same as in the dense_new_vol(), except that tradVol is not allowed.

=item Returns:

volume object

=back

=cut

sub dense_setup_test_vvol(%)
{

# No Validate for wrappers to other functions
#    my (%opts) = validate (@_, { filer => { type=>HASHREF },
#                                 volName => { type=>SCALAR | UNDEF, optional=>1 },
#                                 tradVol => { type=>SCALAR, optional=>1, default=>"0" },   
#                                 volSize => { type=>SCALAR, optional=>1, default=>"20m" },
#                                 purge => { type=>SCALAR | UNDEF, optional=>1, default=>0 },
#                                 aggr => { type=>SCALAR, optional=>1, default=>"dense_aggr" },
#                                 aggrSize => { type=>SCALAR, optional=>1, default=>"4" },
#                                 aggrMinSize => { type=>SCALAR, optional=>1, default=>"51200KB" },
#                                 aggrPurge => { type=>SCALAR, optional=>1, default=>"0" },
#				  aggrRaidType => { type=>SCALAR | UNDEF, optional=>1 }, 
#				  subTest => { type=>SCALAR | UNDEF, optional=>1 }, 
#				  client => { type=>SCALAR | UNDEF, optional=>1 } } 
#                          );

    my (%opts) = @_;
    my ($filer) = $opts{filer};
    my ($opts_ref) = \%opts;
   
    logcomment("DEBUG: dense_setup_test_vvol - Client is $opts{client}");
    if (!defined($opts{client}) ) { $opts{client} = 'localhost'; } 

    if ($opts{client} eq 'localhost') {
	$opts{client} = hostname();
	my @clienta = split(/\./,$opts{client});
	$opts{client} = $clienta[0];
	logcomment("Hostname: $opts{client}");
    }

    # only create flex volume
    if ($opts{tradVol}) {
	die "dense_setup_test_vvol only create flex volumes.\n";
    }

    $opts{volName} = DFLT_VOLNAME unless $opts{volName};

    # Create a new vvol for dense operations.
    my $dvol = dense_new_vol(filer=>$filer, volName=>$opts{volName}, tradVol=>$opts{tradVol}, volSize=>$opts{volSize}, purge=>$opts{purge}, aggr=>$opts{aggr}, aggrSize=>$opts{aggrSize}, aggrMinSize=>$opts{aggrMinSize}, aggrPurge=>$opts{aggrPurge}, client=>$opts{client});

    logcomment("returned by dense_new_vol = $dvol");

    # Make a default test dir
    # $dvol->make_testdir();
    logcomment("Calling create_testdir with params = create_testdir(vol=>$dvol, client=>$opts{client})");
    create_testdir(vol=>$dvol, client=>$opts{client});

    my $volpath = "/vol/" . $dvol->{name};

    # turn off snap sched
    $filer->filer_say("snap sched $opts{volName} 0 0 0");

    # Enable dense on the test volume.
    dense_toggle(host=>$filer->{hostName}, path_name=>$volpath, state=>'on');

    # disable dense schedule
    dense_config_unsched(host=>$filer->{hostName}, path_name=>$volpath);

    return ($dvol);
}

=head2 dense_setup_all(host=>$hostspec, nfsAcls=>$nfsAcls, all args for dense_new_vol also valid here)

Connect to filer, set up volume and populate data set

=over 4

=item hostspec 

Filer name, required.

=item opts 

Options to pass down to sparse_filer(), optional. 
Currently only one option supported {nfsAcls=>1}.
Keep it the same format so it can be passed to sparse_filer directly.

=item Returns:

volume object

data set

=back

=cut

sub dense_setup_all(%)
{

# No Validate for wrappers to other functions
#    my (%arg) = validate (@_, { host => { type=>SCALAR },
#				 nfsAcls => { type=>SCALAR, optional=>1, default=>"1" },
#                                 volName => { type=>SCALAR | UNDEF, optional=>1 },
#                                 tradVol => { type=>SCALAR, optional=>1, default=>"0" },   
#                                 volSize => { type=>SCALAR, optional=>1, default=>"24m" },
#                                 purge => { type=>SCALAR | UNDEF, optional=>1 },
#                                 aggr => { type=>SCALAR, optional=>1, default=>"dense_aggr" },
#                                 aggrSize => { type=>SCALAR, optional=>1, default=>"5" },
#                                 aggrMinSize => { type=>SCALAR, optional=>1, default=>"51200KB" },
#                                 aggrPurge => { type=>SCALAR, optional=>1, default=>"0" }, 
#                                 client => { type=>SCALAR | UNDEF, optional=>1 } }
#                          );

    my (%arg) = @_;
    my ($hostspec) = $arg{host};
    my ($opts) = \%arg;

    if ($arg{client} eq 'localhost') {
	$arg{client} = hostname();
	my @clienta = split(/\./,$arg{client});
	$arg{client} = $clienta[0];
	print "\n Hostname = $arg{client}\n";
    }


    my $filer = dense_new_filer(host=>$hostspec, nfsAcls=>$arg{nfsAcls});

    my $vol = dense_setup_test_vvol(filer=>$filer, volName=>$arg{volName}, tradVol=>$arg{tradVol}, volSize=>$arg{volSize}, purge=>$arg{purge}, aggr=>$arg{aggr}, aggrSize=>$arg{aggrSize}, aggrMinSize=>$arg{aggrMinSize}, aggrPurge=>$arg{aggrPurge}, client=>$arg{client} );

    my @fs = dense_small_dataset(vol=>$vol, host=>$hostspec);
    set_filer_sync(host=>$hostspec);

    return ($vol, @fs);
}


=head2 dense_destroy_vol(vol=>$vol)

Umount, offline, and destroy a volume

=over 4

=item vol 

Volume object, required.

=back

=cut

sub dense_destroy_vol(%)
{
    my (%opts) = validate (@_, { vol => { type=>HASHREF } } );
    my ($dvol) = $opts{vol};

    $dvol->umount();
#    $dvol->offline();
    $dvol->destroy();
}

=head2 dense_cleanup_vol(host=>$host, filerConn=>$filerConn, path=>$path, vol=>$vol)

Cleanup a dense volume. Turn off dense operation, revert all the
data sharing. The volume is then destroyed for now. This won't be
necessary when the undo works.

=over 4

=item host

Filer HostSpec, Required.

=item filerConn 

Filer connection, Optional.

=item path 

Full path of the dense volume, /vol/<volname>, required.

=item vol 

Volume object, Optional.

=back

=cut

sub dense_cleanup_vol(%)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
				 filerConn => { type=>HASHREF, optional=>1 },
				 path => { type=>SCALAR },
				 vol => { type=>HASHREF, optional=>1 } }
			  );				

    my ($host) = $opts{host};
    my ($filerConn) = $opts{filerConn};
    my ($path) = $opts{path};
    my ($vol) = $opts{vol};

    my ($fh) = host($host);
    my ($conn) = connect("#rsh") unless $opts{filerConn};

    dense_off_undo(host=>$host, path_name=>$path);
    dense_wait_for_status(host=>$host, path_name=>$path, status=>"NOSTATUS");
}


=head2 dense_verify_failure(returned_error=>$returned_error, expected_error=>$expected_error)

Verify error messages of a failed command.

=over 4

=item returned_error

Returned error message, required.

=item expected_error

Expected error message, optional

=back

=cut

sub dense_verify_failure(%)
{
    my ( %opts ) = validate ( @_, { returned_error => { type=>SCALAR },
				    expected_error  => { type=>SCALAR|ARRAYREF, optional=>1 } ,
				    vol  => { type=>SCALAR, optional => 1 } ,
				    host  => { type=>SCALAR, optional => 1 } }
			    );

    my $returned_error = $opts{returned_error};
    $returned_error =~ s/^\s+//g;
    $returned_error =~ s/\s+$//g;
    my $expected_error = $opts{expected_error};

    my $fatal_unexpected = 1;
    my $fatal_invalid = 0;

    # first check if there is an expected error, if so check against returned error
    # if not then throw exception

    my @expected_errors = ();
    if( ref($expected_error) eq "ARRAY" )
    {
         logcomment("DEBUG: dense_verify_status: Multiple messages are passed as expected errors:[$opts{expected_error}]");
         @expected_errors = @{$expected_error};
    }
    else
    {
         logcomment("DEBUG: dense_verify_status: Only one message is passed as expected errors:[$opts{expected_error}]");
         push @expected_errors, $expected_error;
    }

    if ( !$returned_error ) 
    { 
        $fatal_invalid = 1;
    }

    logcomment("DEBUG: dense_verify_failure: vol=[$opts{vol}], obtained=[$returned_error], expected=[@expected_errors]");

    if( !$fatal_invalid and $#expected_errors >= 0 )
    {
	    foreach my $ee ( @expected_errors ) 
	    {
		my $err = $ee;
		$err = "${ee}:$opts{vol}" if(defined $opts{vol});
		logcomment("DEBUG: dense_verify_failure: expected=[$err], obtained=[$returned_error]");
		# relaxing the condition to accomodate the dense_share output
		# which contains the block count
		if ( "$returned_error" eq "$err" or $returned_error =~ /$err/)
		{
		    $fatal_unexpected = 0;
		    last;
		}
          }
    }

    if( $fatal_unexpected )
    {
	    die ("Unexpected error message: $returned_error\n");
    }
    elsif( $fatal_invalid )
    {
	die ("Operation succeeded unexpectedly\n"); 
    }
}

######################################################
### Get the WAFL scan id for a particular process  ###
######################################################
=head2 get_wafl_scan_id
 
get the WAFL scan id for a particular process specified
 
Inputs:
1. conn - filer connection
2. vol_name - the name of the volume on which the specifed process needs to be found
3. match_string - the process string (in wafl scan status) that needs to be found

Return value:
Sucess:
The scan id of the requested process in the volume
Failure:
the scan id will be zero
 
=cut

sub get_wafl_scan_id(%) {
    my (%opts) = validate (@_, 
		    { 
			conn => { type=>HASHREF }, 
			vol_name => { type=>SCALAR },
			match_string => { type=>SCALAR },
		    }
		);
    my ($scan_id, $out, @out_array, $line, $retry);
    $scan_id = 0;
    $retry = 10;
    
    if (remove_leading_trailing_spaces($opts{'match_string'}) eq "") {
	return $scan_id;
    }
    do {
	$retry--;
    	$out = filer_cli_test_say($opts{'conn'}, "wafl scan status -V ".$opts{'$vol_name'});
    	@out_array = split("\n", $out);
    	foreach $line (@out_array) {
	    if ($line =~/$opts{'match_string'}/) {
		$scan_id = (split(/\s+/, remove_leading_trailing_spaces($line)))[0];
		last;
	    }
    	}
    sleep 1;
    } while (($scan_id == 0) && ($retry !=0));

    return $scan_id;
}



###############################################################################
### Abort the particular process specified by the ID in the wafl scan space ###
###############################################################################
=head2 abort_wafl_scan
 
abort the particular process specified by the ID in the wafl scan space
 
Inputs:
1. conn - filer connection
2. volname - the name of the volume on which the specifed process needs to be found
3. scan_id - the id of the process that needs to be aborted
4. scan_name - the process string (in wafl scan status) that needs to be found
5. timeout - timeout for this process to complete

Return value:
Sucess: 1
Failure: 0

=cut

sub abort_wafl_scan(%) {
    my (%opts) = validate (@_, 
			    {	conn => {type=>HASHREF},
				volname => {type=>SCALAR},
			        scan_id => {type=>SCALAR}, 
			        scan_name => {type=>SCALAR}, 
			        timeout => {type=>SCALAR, default=>300}, 
			    }
		        ); 
     
    my $ret_val = 1; 
     
    filer_cli_test_say($opts{'conn'}, "wafl scan abort ".$opts{'scan_id'});
    wait_for_scan_done(host=>$opts{'conn'}->hostp()->hostname(), volname=>$opts{'volname'}, scan=>$opts{'scan_name'}, timeout=>$opts{'timeout'});
     
    if(get_wafl_scan_id(conn=>$opts{'conn'}, vol_name=>$opts{'volname'}, match_string=>$opts{'scan_name'})) {
        $ret_val = 0;
    }
    
    return $ret_val; 
}

=head2 wait_for_scan_done(host=>$host, volname=>$volname, scan=>$scan)

Wait for the completion of scan

=over 4

=item host

Filer HostSpec, required.

=item conn

Filer connection, optional.

=item volname

The name of volume, optional. If not specified checks filer wide.

=item scan

The status string of the scan, required.

=item timeout

Abort with an error if scan does not finish within the specified timeout.
A value of 0 will wait forever. Defaults to 0.

=item period

Wait time in seconds between consecutive wafl scan status checks. A larger
period will reduce log size for longer scans. Defaults to 5.

=back

=cut

sub wait_for_scan_done(%)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
				 volname => { type=>SCALAR, optional=>1, default=>"" },
				 scan => { type=>SCALAR },
                                 timeout => { type=>SCALAR, default=>0 },
                                 period => { type=>SCALAR, default=>5 } }
			  );

    my ($host) = $opts{host};
    my ($volname) = $opts{volname};
    my ($scan) = $opts{scan};
    my ($timeout) = $opts{timeout};
    my ($period) = 0;
    my ($filer) = host($host);
    my ($conn) = connect("#rsh");
    my $text;
    my $result;
    my $start_time = time;

    do {
	sleep($period); # check every $period seconds
                        # period is 0 first time through

	($text,$result) = 
	    filer_cli_advanced_say($conn, "wafl scan status $volname");

        if ($timeout) {
            if (time() - $start_time > $timeout) {
                Carp::croak("Timeout waiting for wafl scan $scan " .
                            "to complete on $volname");
            }
        }

        $period = $opts{period};
    } while ($text =~ /$scan/);

}

=head2 verify_scan(host=>$host, volname=>$volname, scan=>$scan)

Verify if certain wafl scan is still going on, such as dense scan 

=over 4

=item host

Filer HostSpec, required.

=item conn

Filer connection, optional.

=item volname

The name of volume, required.

=item scan

The name (string) of the scan, required, such as defragmentation

=back

=cut

sub verify_scan(%)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
                                 volname => { type=>SCALAR },
                                 scan => { type=>SCALAR } }
                          );

    my ($host) = $opts{host}; 
    my ($volname) = $opts{volname}; 
    my ($scan) = $opts{scan};
    my ($fh) = host($host);
    my ($conn) = connect("#rsh");
    my $text;
    my $result;

   ($text,$result) =
            filer_cli_advanced_say($conn, "wafl scan status $volname");
   if ($text !~ /$scan/){
       die "Does not match scan string $scan: $text";
   }

}

=head2 dense_get_file_size_and_blkcnt(vol=>$vol, filename=>$filename)

Get the size and block count of a file.

=over 4

=item vol 

Volume object, required.

=item filename

Name of the file, required.

=item Returns:

size

blk_cnt

=back

=cut

sub dense_get_file_size_and_blkcnt(%)
{
    my (%opts) = validate (@_, { vol => { type=>HASHREF },
				 filename => { type=>SCALAR } }
			  ); 

    my ($vol) = $opts{vol};
    my ($filename) = $opts{filename};

    my $fio = fioOpen($vol, $filename);
    $fio->Stat();
    my $blk_cnt = $fio->{Stat}{blocks} / 8;
    my $size = $fio->{Stat}{size};
    $fio->Close();

    return ( $size, $blk_cnt);
}


=head2 dense_verify_file_size_and_blkcnt(vol=>$vol, filename=>$filename, exp_size=>$exp_size, exp_blk_cnt=>$exp_blk_cnt)

Compare the size and block count against the expected size and block count.

=over 4

=item vol 

Volume object, required.

=item filename

Name of the file, required.

=item exp_size

Expected file size in bytes, required.

=item exp_blk_cnt

Expected file block count in 4KB blocks, required.

=back

=cut

sub dense_verify_file_size_and_blkcnt(%)
{
    my (%opts) = validate (@_, { vol => { type=>HASHREF },
				 filename => { type=>SCALAR },
				 exp_size => { type=>SCALAR },
				 exp_blk_cnt => { type=>SCALAR } }
			  );

    my ($vol) = $opts{vol};
    my ($filename) = $opts{filename};
    my ($exp_size) = $opts{exp_size};
    my ($exp_blk_cnt) = $opts{exp_blk_cnt};

    my ($size, $blk_cnt) = dense_get_file_size_and_blkcnt(vol=>$vol, filename=>$filename);

    if ($size != $exp_size) {
	die ("File size is $size, expected size is $exp_size");
    }

    if ($blk_cnt != $exp_blk_cnt) {
	die ("File block count is $blk_cnt, expected block count is $exp_blk_cnt");
    }
}

=head2 get_snapshot_with_time(host=>$host, volname=>$volname, snapname=>$snapname)

Get snapshot along with its timestamp.
The returned value will looks like this:
"05 25 20:00|sv_target.0".

=item host

Filer HostSpec, required.

=item conn 

Filer Connection, optional.

=item volname

Name of the volume, required.

=item snapname

Name of the snapshot, required.

=back

=cut

sub get_snapshot_with_time(%)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR },
                                 volname => { type=>SCALAR },
                                 snapname => { type=>SCALAR } }
                          );

    my ($host) = $opts{host};
    my ($volname) = $opts{volname};
    my ($snapname) = $opts{snapname};
    my ($fh) = host($host);
    my ($conn) = connect("#rsh");

    my $snapVol = new snapCmds($conn,'Vol');

    # get snapshots along with their time data 
    my @SnapList = $snapVol->snapshot_list_wtime($conn, $volname);
    my $count = $#SnapList;
    my $i = 0;
    while ($i <= $count) {
        my $snap = shift(@SnapList);
        if ($snap =~ /$snapname/) {
            logInfo("found specified snapshot: $snap");
            return $snap;
        }
        $i++;
    }
    return 0;
}

=head2 set_metafile_visible(host=>$host, partner=>$partner)

Make metafile visiable to user by issusing 
"priv set -q test; setflag wafl_metadata_visible 1"
Only work on a debug kernel

=item host

Filer HostSpec, required.

=back

=cut

sub set_metafile_visible(%)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR }, 
				partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } );

    my ($host) = $opts{host};
    my ($fh) = host($host);
    my $conn = connect("#rsh");
    
    my $cmd;

    if ($opts{partner}) {
    	$cmd  = "partner". " " . DENSE_TEST_CMD . "partner" . " ". "setflag wafl_metadata_visible 1"; 
    } else {
    	$cmd  = "setflag wafl_metadata_visible 1"; 
    }
  
    my ($return_text, $result_code) = filer_cli_test_say($conn, $cmd);

    # Nothing can be checked.
}

=head2 calculate_expected_savings(blocks=>$no_of_blocks,refcnt=>$max_refcnt)

Calculates and returns the expected savings for given number of blocks (only for a file with 100% duplicate) with a specific refcount

=cut

sub calculate_expected_savings(%)
{
    my (%opts) = validate(@_, {blocks=> {type=>SCALAR},
		    	       max_refcnt=> {type=>SCALAR, default=>32768},}
		    );
    my $no_of_blocks = $opts{blocks};
    my $max_refcnt = $opts{max_refcnt};
    my ($lost_blocks, $saved_blocks, $remaining_blocks);
    my $savings;

    $remaining_blocks = 0;
    $Log->comment("No of blocks : $no_of_blocks");
    $lost_blocks = int($no_of_blocks/$max_refcnt);
    $Log->comment("Lost blocks is $lost_blocks");
    $remaining_blocks = $no_of_blocks%$max_refcnt;
    $Log->comment("Remaining blocks is $remaining_blocks");

    if ($remaining_blocks == 0) {
	 $saved_blocks = $no_of_blocks - $lost_blocks;
	 $savings = $saved_blocks * 4;
	 return $savings;
    } else {
	 $lost_blocks++;
	 $saved_blocks = $no_of_blocks - $lost_blocks;
	 $savings = $saved_blocks * 4;
	 return $savings;
    }
}

## @name ::panic
## @summary Panics the filer
## @arg $fc Required: The filer connection object for the command line interface

sub panic (%) {
    my %opts = @_;
    my($fc) = $opts{conn};
    my $filer = filer_name($fc);
    my $timeout = $opts{timeout} ?  $opts{timeout} : 900;
    my $delta = $opts{delta} ? $opts{delta} : 10;
    
    say($fc, "priv set test;panic",-blind);
    sleep(60);
    wait_for_rsh($filer, timeout=>$timeout, delta=>$delta);
}

=head2 calculate_merge_counter_values(fps=>$no_of_fps, p1merge_bins=>$p1_merge_bin_value, initial_sis=>1)

Calculates and returns the expected values for num_p1_merges, num_p1_bins

=cut

sub calculate_merge_counter_values(%)
{
    my (%opts) = validate(@_, {fps=> {type=>SCALAR},
		    	       p1merge_bins => {type=>SCALAR},
			       initial_sis=> {type=>SCALAR, default=>1},}
		    );
    my $no_of_fps = $opts{fps};
    my $p1merge_bins = $opts{p1merge_bins};
    my $initial_sis = $opts{initial_sis};
    my ($exp_p1_bins, $exp_p1_bins_merge, $exp_bins_merged, $exp_num_merges, $unmerged_bins, $mem_allocated, $exp_p1_bins_second);
    
    my $no_of_kilo_fps = ceil($no_of_fps/1024);
    if($no_of_kilo_fps <= 256 )
    {
	$mem_allocated = 1;
    }elsif($no_of_kilo_fps <= 1024 )
    {
	$mem_allocated = 2;
    }elsif($no_of_kilo_fps <= 4096 )
    {
	$mem_allocated = 4;
    }elsif($no_of_kilo_fps <= 16384 )
    {
	$mem_allocated = 8;
    }elsif($no_of_kilo_fps <= 65536 )
    {
	$mem_allocated = 16;
    }elsif($no_of_kilo_fps <= 262144 )
    {
	$mem_allocated = 32;
    }elsif($no_of_kilo_fps > 262144 )
    {
	$mem_allocated = 64;
    }
    
    $exp_p1_bins = ceil($no_of_kilo_fps/($mem_allocated*8));
    $exp_num_merges = int($exp_p1_bins/$p1merge_bins);
    $unmerged_bins = $exp_p1_bins%$p1merge_bins;

    $Log->comment("For p1 there should be $exp_p1_bins bins");
    $Log->comment("After first p1m number of merges which should happen is $exp_num_merges");
    $Log->comment("Unmerged Bins after first pass is $unmerged_bins");
    $exp_bins_merged = $exp_p1_bins - $unmerged_bins;
    $Log->comment("After first p1m number of bins which should be merged is $exp_bins_merged");

    $exp_p1_bins_second = ceil($exp_p1_bins/2);
    $exp_num_merges += int($exp_p1_bins_second/$p1merge_bins);
    $unmerged_bins = $exp_p1_bins_second%$p1merge_bins;
    $Log->comment("For second p1 there should be $exp_p1_bins_second bins");
    $Log->comment("After second p1m value of number of merges should be $exp_num_merges");
    $Log->comment("Unmerged Bins after second merge is $unmerged_bins");
    $exp_bins_merged += $exp_p1_bins_second - $unmerged_bins;

    if ($initial_sis) {
	$Log->comment("The same round of sorting as above repeats when the vvol FPDB is being created from the aggr FPDB");
	$exp_p1_bins_second = ceil($exp_p1_bins/2);
        $exp_num_merges += int($exp_p1_bins_second/$p1merge_bins);
        $unmerged_bins = $exp_p1_bins_second%$p1merge_bins;
	$exp_bins_merged += $exp_p1_bins_second - $unmerged_bins;
    }

    my @return;
    push @return, $exp_num_merges;
    push @return, $exp_bins_merged;

    return @return;
}

=head2 set_iron_on_demand(host=>$host, partner=>$partner)

Enable IOD on the filer
"priv set -q test; setflag wafl_iron_buf_on_demand 1"
Only work on a debug kernel

=item host

Filer HostSpec, required.

=back

=cut

sub set_iron_on_demand(%)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR },
                                partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } );

    my ($host) = $opts{host};
    my ($fh) = host($host);
    my $conn = connect("#rsh");

    my $cmd;

    if ($opts{partner}) {
        $cmd  = "partner". " " . DENSE_TEST_CMD . "partner" . " ". "setflag wafl_iron_buf_on_demand 1";
    } else {
        $cmd  = "setflag wafl_iron_buf_on_demand 1";
    }

    my ($return_text, $result_code) = filer_cli_test_say($conn, $cmd);

}

=head2 wafl_scan_sped(conn=>$conn, value=>$value)

Gets or sets the value of wafl scan speed.

=item conn
Connection object, required.

=item value
To set wafl scan speed value, optional. Default: returns the current scan speed.

=cut

sub wafl_scan_speed(%)
{
    my (%opts) = validate(@_, { conn => { type=>HASHREF },
		    		value => { type=>SCALAR, optional=>1, regex=>qr/^[0-9]+$/ } } );
    my $conn = $opts{conn};
    my $value = $opts{value};

    my $return;

    if(! $value){
	$return = filer_cli_test_say($conn, "wafl scan speed");
	$return =~ s/[^\d]+//g;
	return $return;
    } else {
	$return = filer_cli_test_say($conn, "wafl scan speed $value");
    }
}
=head2 unset_iron_on_demand(host=>$host, partner=>$partner)

Enable IOD on the filer
"priv set -q test; setflag wafl_iron_buf_on_demand 0"
Only work on a debug kernel

=item host

Filer HostSpec, required.

=back

=cut

sub unset_iron_on_demand(%)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR },
                                partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } );

    my ($host) = $opts{host};
    my ($fh) = host($host);
    my $conn = connect("#rsh");

    my $cmd;

    if ($opts{partner}) {
        $cmd  = "partner". " " . DENSE_TEST_CMD . "partner" . " ". "setflag wafl_iron_buf_on_demand 0";
    } else {
        $cmd  = "setflag wafl_iron_buf_on_demand 0";
    }

    my ($return_text, $result_code) = filer_cli_test_say($conn, $cmd);

}

=head2 set_iron_scan_suspend_duration(host=>$host, value=>$value, partner=>$partner)

Set wafl_iron_scan_suspend_duration flag to user desired value
"priv set -q test; setflag wafl_metadata_visible 150"
Only work on a debug kernel

=item host

Filer HostSpec, required.

=back

=cut

sub set_iron_scan_suspend_duration(%)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR },
				value => { type=>SCALAR, optional=>1, default=>0 },
                                partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } );

    my ($host) = $opts{host};
    my ($fh) = host($host);
    my $conn = connect("#rsh");
    my ($value) = $opts{value};

    my $cmd;

    if ($opts{partner}) {
        $cmd  = "partner". " " . DENSE_TEST_CMD . "partner" . " ". "setflag wafl_iron_scan_suspend_duration $value";
    } else {
        $cmd  = "setflag wafl_iron_scan_suspend_duration $value";
    }

    my ($return_text, $result_code) = filer_cli_test_say($conn, $cmd);

}

=head2 set_flag_wafliron_panic(host=>$host, partner=>$partner)

Turn off wafliron panic on corruption flag
"priv set -q test; setflag wafl_iron_panic_on_corruption 0"
Only work on a debug kernel

=item host

Filer HostSpec, required.

=item Returns:

On success - returns 1
On failure - returns 0

=back

=cut

sub set_flag_wafliron_panic(%)
{
    my (%opts) = validate(@_, { host => { type=>SCALAR },
                                partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ } } );

    my ($host) = $opts{host};
    my ($fh) = host($host);
    my $conn = connect("#rsh");
   
    my $cmd;

    if ($opts{partner}) {
        $cmd  = "partner". " " . DENSE_TEST_CMD . "partner" . " ". "setflag wafl_iron_panic_on_corruption 0";
    } else {
        $cmd  = "setflag wafl_iron_panic_on_corruption 0";
    }

    my ($return_text, $result_code) = filer_cli_test_say($conn, $cmd);

    my $print_cmd = "printflag wafl_iron_panic_on_corruption";
    my ($text, $code) = filer_cli_test_say($conn, $print_cmd);
    
    my @result = split(/ /, $text);
    if ($result[2] eq 0) { return 1; } else { return 0; }

}

=head2 create_testdir(vol=>$vol, dir=>$dir, client=>$client)

Create Test Dir for a volume from the client specified via NATE

=item vol

volume object, Required.

=item dir

Test Dir Name, Optional. Default is TEST.

=item client

client hostname, Optional. Default is localhost.

=back

=cut

sub create_testdir(%)
{
    my (%opts) = validate(@_, { vol => { type=>HASHREF },
                                dir => { type=>SCALAR, optional=>1, default=>"TEST"},
				client => { type=>SCALAR | UNDEF, optional=>1, default=>"localhost" } }
			 );

	my ($vol) = $opts{vol};
	my ($dir) = $opts{dir};
	my ($client) = $opts{client};
	my $sudo_path = param('SUDO_PATH','-mesg','(NATE is expected to have set this)');

	if ($client eq 'localhost') {
		$client = hostname();
		my @clienta = split(/\./,$client);
		$client = $clienta[0];
		logcomment("Hostname: $client");
	}

	## set test dir on volume
        $vol->{testDirBaseName} = $dir;
        $vol->{testDir} = "$vol->{mountPoint}/$dir";
        $vol->{testDirVolName} = "/vol/$vol->{name}/$dir";
        logTrc("set test dir name=$vol->{testDir}");

	my $h = host($client);
	my $c = connect("#rsh");

	## create test dir on volume
        # set up dir if needed
        if (!-e $vol->{testDir}) { 
		logTrc("create test dir name=$vol->{testDir}");
	my(undef,undef,$output) = say($c, "$sudo_path mkdir -p $vol->{testDir} && /bin/echo _mkdir_ok_", "_mkdir_ok_", -ignore, "/bin/echo _mkdir_ok_");
        ## if($output ne "_mkdir_ok_") { logError("mkdir $vol->{testDir} failed"); }
        
	(undef,undef,$output) = say($c, "$sudo_path chmod 0777 $vol->{testDir} && /bin/echo _chmod_ok_", "_chmod_ok_", -ignore, "/bin/echo _chmod_ok_");
        ## if($output ne "_chmod_ok_") { logError("chmod $vol->{testDir} failed"); }
        }

}

=head2 get_vol_savings(host=>$host, vol_name=>$vol_name)

Get the savings of a volume(if provided) or all the volumes

=over 4

=item host

Filer HostSpec (Required)

=item vol_name

Volume name (Optional)

=item Returns:

Success

=over 4

Returns 1 and hash containing vol savings

=back

Failure

=over 4

Returns 0

=back

=back

=cut

####################### df -s ####################

sub get_vol_savings
{
  my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                               vol_name => { type=>SCALAR, optional=>1 },
				partner => { type=>SCALAR, optional=>1 }}
                         );


    my $cfiler;
    if (ref($opts{host})) {
       $cfiler = $opts{host};
    } else {
       # Connect uses the host most recently created or referenced by host()
       my $host = host($opts{host});
       $cfiler = connect("#rsh");
    }
    my $cmd="";
    if($opts{vol_name})
    {
       $cmd = "df -s" . " $opts{vol_name}";
    }
    else
    {
       $cmd = "df -s";
    }

    if($opts{partner}) {
	$cmd = "partner " . $cmd;
	logcomment("DEBUG: Command is $cmd");
    }

    my ($df_result, $df_exit__code) = filer_cli_say($cfiler,$cmd);

my %df_result=();
$df_result{result}=0;

# Return failure if df command failed.
return (%df_result)  if ((split(/\n/,$df_result))[0] !~ /^Filesystem\s+used\s+saved\s+\%saved/);
# Test Passed. Fill success bit (1) in $df_result{result} and populate %df_result
$df_result{result}=1;


# Returning the result in the hash which looks like this
#	%df_result={	fs=>{  vol1=>{used=>value,saved=>value,%saved=>value},
#		               vol2=>{used=>value,saved=>value,%saved=>value}
#		            }
#		        result=>0/1	
#                  }
#

    foreach my $line (split(/\n/, $df_result)) 
    {
	if($line =~ /^\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+\%)\s*/)
	{
		$df_result{fs}->{$1}->{used}=$2;
		$df_result{fs}->{$1}->{saved}=$3;
		$df_result{fs}->{$1}->{"%saved"}=$4;
	}
    }
	
   return %df_result;
}

=head2 get_all_savings(host=>$host, vol_name=>$vol_name)

Get the ASIS and Compression savings of a volume(if provided) or all the volumes

=over 4

=item host

Filer HostSpec (Required)

=item vol_name

Volume name (Optional)

=item Returns:

Success

=over 4

Returns 1 and hash containing ASIS and compression savings

=back

Failure

=over 4

Returns 0

=back

=back

=cut

####################### df -S ####################

sub get_all_savings
{
  my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                              vol_name => { type=>SCALAR, optional=>1 },
			      partner => {type=>SCALAR, optional=>1 }}
                         );


    my $cfiler;
    if (ref($opts{host})) {
       $cfiler = $opts{host};
    } else {
       my $host =$opts{host};
       $cfiler = connect("#rsh");
    }
    my $cmd="";
    if($opts{vol_name})
    {
       $cmd = "df -S" . " $opts{vol_name}";
    }
    else
    {
       $cmd = "df -S";
    }
    if($opts{partner}) {
        $cmd = "partner " . $cmd;
        logcomment("DEBUG: Command is $cmd");
    }


    my ($df_result, $df_exit__code) = filer_cli_say($cfiler,$cmd);

my %df_result=();
$df_result{result}=0;

# Return failure if df command failed.
return (%df_result)  if ((split(/\n/,$df_result))[0] !~ /^Filesystem\s+used\s+compressed\s+a-sis\s+\%saved/);
# Test Passed. Fill success bit (1) in $df_result{result} and populate %df_result
$df_result{result}=1;

# Returning the result in the hash which looks like this
#	%df_result={	fs=>{  vol1=>{used=>value,compressed=>value,asis=>value,,%saved=>value},
#		               vol2=>{used=>value,compressed=>value,asis=>value,,%saved=>value},
#		            }
#		        result=>0/1	
#                  }
#

    foreach my $line (split(/\n/, $df_result)) 
    {
	if($line =~ /^\s*(\S+)\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+\%)\s*/)
	{
		$df_result{fs}->{$1}->{"used"}=$2;
		$df_result{fs}->{$1}->{"compressed"}=$3;
		$df_result{fs}->{$1}->{"asis"}=$4;
		$df_result{fs}->{$1}->{"saved"}=$5;
	}
    }
   return %df_result;
}

=head2 is_dense_enabled(host=>$host, vol_path=>$vol_path)

Check if the volume has dense enabled or not

Parameters:

=over 4

=item host

Filer HostSpec

=item volpath

Volume Path

=back

Return:

=over 4

=item Dense is enabled

Return 1 if the volume has sis enabled

=item Dense flag is not set

Return 0 if the volume has sis disabled

=back

=cut


sub is_dense_enabled($@)
{
    my (%opts) = validate (@_, { host => { type=>SCALAR|HASHREF },
                                 vol_path => { type=>SCALAR } }
                          );

    my $cfiler;
    if (ref($opts{host})) {
       $cfiler = $opts{host};
    } else {
       my $host =$opts{host};
       $cfiler = connect("$host#rsh");
    }
    my ($vol_path)=$opts{vol_path};
    my $cmd="sis status $vol_path";


    my ($df_result, $df_exit__code) = filer_cli_say($cfiler,$cmd);

# Return failure if command failed.
if ((split(/\n/,$df_result))[0] !~ /^Path\s+State\s+Status\s+Progress/)
{
	return 0;
}
else
{	
# Check if volume is enabled or disabled.
	if (((split(/\s+/,((split(/\n/,$df_result))[1])))[1]) =~ /Enabled/)
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

}

## unmount_and_cleanup_vol($host, $volume_path, $vol_obj, $client);
sub unmount_and_cleanup_vol($$$$) {

        my ($host, $path, $vol, $client_name) = @_;
	host($host);
        my $conn = connect("#rsh");

	if (!vol_exists($conn, $path)) { 
		logcomment("Volume to be cleaned up does not exist: $path");
		return;
	}
	
	logcomment("DEBUG: unmount_and_cleanup_vol - Client is $client_name");
    	logcomment("DEBUG: Aggr name is :$vol->{aggrName}");
	if (!defined($client_name) ) { $client_name = 'localhost'; } 

    	if ($client_name eq 'localhost') {
		$client_name = hostname();
		my @clienta = split(/\./,$client_name);
		$client_name = $clienta[0];
		logcomment("Hostname: $client_name");
    	}

        logcomment("Unmounting volume $path from $client_name");

        umount($vol->{mountPoint}, {client=>$client_name});
        if($@) { logcomment("Could not unmount $vol->{mountPoint} on $client_name: $@"); }

	host($host);
        $conn = connect("#rsh");

        logcomment("Cleaning up volume $path on filer $host");

        if (vol_exists($conn, $vol->{name})) {
	
## 		if($ENABLE_COMPRESSION) {
##         		logcomment("Compression is enabled on the volume $vol->{name}. Decompressing volume now");
##         		filer_cli_say($conn, "vol options $vol->{name}; vol status $vol->{name}");
## 
## 			logcomment("Turning off compression option on the volume...");
## 			filer_cli_say($conn, "vol options $vol->{name} compression off");
##         		filer_cli_say($conn, "vol options $vol->{name}; vol status $vol->{name}");
## 			
## 			logcomment("Starting decompress now...");
## 			filer_cli_say($conn, "vol decompress $vol->{name}");	
## 
## 			## Wait till decompress is done before proceeding ??
##     		}
## 
                if ($ENV{SISUNDO_AT_CLEANUP} || $SISUNDO_AT_CLEANUP) {
                        logcomment("SISUNDO_AT_CLEANUP is set");
                        if(is_dense_enabled(host=>$host, vol_path=>$path)) {
                                logcomment("Starting dense undo on $path");
                                dense_stop(host=>$host, path_name=>$path);
                                eval {
                                        dense_wait_for_status(host=>$host, path_name=>$path, status=>'IDLE', time_gap=>30);
                                };
                                if($@) { logcomment("Error while waiting for SIS status on $path: $@"); }
                                eval {
                                        dense_off_undo(host=>$host, path_name=>$path);
                                };
                                if($@) {
                                        logcomment("Error while undoing SIS on $path: $@");
                                } else {
                                        eval {
                                                dense_wait_for_status(host=>$host, path_name=>$path, status=>'NOSTATUS', time_gap=>120);
                                        };
                                        if($@) { logcomment("Error while waiting for SIS status on $path: $@"); }
                                }
                        }
                }
                if ($ENV{WAFLIRON_AT_CLEANUP} || $WAFLIRON_AT_CLEANUP) {
                        logcomment("WAFLIRON_AT_CLEANUP is set");
                        my ($cor_found, $cor_messages);
                        eval {
                                ($cor_found, $cor_messages) = wafliron_start($conn , $vol->{aggrName} , oc => 1 );
                        };
                        if ($@) {
                                ($cor_found, $cor_messages) = wafliron_start($conn , $vol->{aggrName} );
                        }
                        if (wafliron_status($conn , $vol->{aggrName}) == 2) {
                                wafliron_reject($conn , $vol->{aggrName});
                        }
                        if ($cor_found) {
                                logresult("FAIL","Wafliron found corruption");
                        }
                        filer_cli_say($conn, "aggr online $vol->{aggrName}");
                }
                filer_cli_say($conn, "vol offline $vol->{name}");
                filer_cli_say($conn, "vol destroy $vol->{name} -f");
        }
}

sub cf_check_state($$){
	my @hosts = @_;
	my ($filer, $filer_partner) = @hosts;
	
	foreach my $host (@hosts){
		my $conn=make_conn($host, "#rsh");
		
		# For NearStore products
		logStep("Checking if the filer is a NearStore platform");
		my $filer_model = $conn->{hostp}->{'MODEL'};
		if (!defined($filer_model) ) { $filer_model = $conn->{hostp}->{'model'}; }
		if (!defined($filer_model) ) { $filer_model = filer_model_get($conn); }
		if ($filer_model =~ m/^R/i ) {
			die "\nThis test cannot be run on Nearstore platforms";
		}

		logStep("Checking if cluster is licensed on filer");

		my $licensed = filer_service_is_licensed($conn,"cluster");
		# Bring both filers up (if they're not already), and make sure any
		# which are in takeover state have given back to each other.
		sleep(10);
		boot_with_giveback($filer, $filer_partner) or die("Cluster setup misconfiguration : $@\n"); 	

		my ($text,$result) = filer_cli_say($conn, "cf status");
		if ($text =~ /reboot/ ) { $conn = filer_reboot($conn); }
	}

	foreach my $host (@hosts){
		my $conn=make_conn($host, "#rsh");
		logStep("Checking cluster state on filer");
		my $cf_state = get_cf_state($conn);
	
	        if( $cf_state=~/DISABLED/ ) {
	                logcomment("Enabling cluster on the filer $host");
	                cf_enable($conn) or die ("Could not enable CF on filer $host: $@\n");
	        } elsif( $cf_state=~/UNREACH/ ) {
	                die ("Filer is not in a reachable state. Please try test when filer is up\n");
	        } elsif( $cf_state=~/DEAD/ ) {
	                die ("Filer has been takenover by its partner\n");
		}
	}

}

##############################################################################
##########   To get / enable / disable cf on a filer  data         ###########
##############################################################################

=head2 dense_cf_status(conn=>$conn) or dense_cf_status(conn=>$conn, set=>$status)
    name : dense_cf_status

Description:
    To get / enable / disable cf on a filer  data

Input value:
    conn => mandatory => the filer connection
    set  => options   => if not defined get the cf status else set cf status to specified

Return value:
    if "set" is not defined
	the status of the cf
    if "set" is not defined
	Success : Return 1
	Fail    : Return 0
=cut

sub dense_cf_status($@) {
    my (%opts) = validate (@_, {
		    conn    =>	{type=>HASHREF},
		    set	    =>	{type=>SCALAR, optional=>1,  default=>"NOVAL"},
		});

    my ($conn, $set, $ret_val);

    $conn = $opts{'conn'};
    $set = $opts{'set'};

    if ($set ne "NOVAL") {
	# Setting the cf status on the filer
	logcomment ("Trying to set cf status to $set");
	if ($set =~ /enable|up/i) {
	    $ret_val = cf_enable($conn);
	} elsif ($set =~ /disable/i) {
	    $ret_val = cf_disable($conn);
	} else {
	    $ret_val = 0;
	    logcomment("cf status $set not recognized");
	}

	if ($ret_val) {
	    logcomment("setting cf status to $set was successful");
	} else {
	    logcomment("setting cf status to $set failed");
	}

    } else {
	# Getting the cf status on the filer
	# return value can be  "UP", "TAKEOVER", "DISABLED", "DEAD" or "UNREACH"
	$ret_val = get_cf_state($conn);
    }

    return $ret_val;
}

sub set_maxfiles ($$) {

my ($host, $vol_name) = @_;
my $conn = make_conn($host, "#rsh");

my $text = "nothing";
my $maxfiles = '999999999999';

my ($max, $used) = vol_max_files($conn, $vol_name);

logcomment("Max is $max and Used is $used");

        eval {
                vol_max_files($conn, $vol_name, $maxfiles);
                $text = $@ if ($@);
        };
        if ($text =~ /maximum inode count should be between (\d+) and (\d+)/) {
                $maxfiles=$2;
                vol_max_files($conn, $vol_name, $maxfiles) or die("Maxfiles $maxfiles failed: $@");
                logcomment("Maxfiles is now $maxfiles");
        } elsif ($text =~ /Maxfiles remains at \d/) {
                logcomment("Maxfiles was already maxed out");
        } else {
                die("Maxfiles said something unexpected: $text");
        }

}

## From compression.pm
sub parse_showvbns ($$$) {
        
##   my (%opts) = validate(@_, { host => { type=>SCALAR },
##                                file_name => { type=>SCALAR },
##                                 partner => { type=>SCALAR, optional=>1 }}
##                          );
## 
## 	my ($host) = $opts{host};
##      my ($file) = $opts{file_name};

	my ($host, $file, $partner) = @_;
	my $conn = make_conn($host, "#rsh");
	my $cmd;

## 	if ($opts{partner}) {
 	if ($partner) {
		$cmd  = "partner". " " . DENSE_TEST_CMD . "partner" . " ". "wafl showvbns -s" . " " . $file;
	} else {
		$cmd = "wafl showvbns -s". " " . $file;
	}

	my $buf = filer_cli_test_say($conn, $cmd);
        
	my %val= ();
        my @bufs = split /\n/, $buf;

	## Parse the command output
        foreach my $line (@bufs)
        {
                if($line =~ /fbn (\d+) is vbn (\d+), vvbn (\d+), refcount (\d+)/)
                {
                        $val{$1} = {};
                        $val{$1}{vbn} = $2;
                        $val{$1}{vvbn} = $3;
                        $val{$1}{refcount} = $4;
                        logcomment("Storing {$1}{vbn} = $val{$1}{vbn}, {$1}{vvbn} = $val{$1}{vvbn} , {$1}{refcount} = $val{$1}{refcount}");
                } else {
			die("Invalid output format: Expected => fbn (d+) is vbn (d+), vvbn (d+), refcount (d+) Obtained => $line\n");
                }
        }
        return %val;

}


sub sort_spares (%) {

        my (%opts) = validate(@_, { host => { type=>SCALAR } });
        my $host = $opts{host};
        my $conn = make_conn($host, "#rsh");

        my (@sizes, $i);
        my %list_of_spares_by_size;

        my $spares = Aggregate::aggr_spare_status($conn);

        foreach my $spare (@{$spares->{0}}) {
                my $push = 1;
                foreach $i (@sizes) {
                        if ( $i == $spare->{used_mb} ) {
                                $push = 0;
                        }
                }
                if ($push) { push(@sizes, $spare->{used_mb}); }
        }

        foreach $i (@sizes) {
                foreach my $spare (@{$spares->{0}}) {
                        if ($spare->{used_mb} eq $i) {
                                push @{ $list_of_spares_by_size{$i} }, $spare->{name};
                        }
                }
        }

	return %list_of_spares_by_size;

}

sub create_new_aggr (%) {
	my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                              volSize => { type=>SCALAR },
                              aggrName => {type=>SCALAR }}
                         );
	my $host=$opts{host};
	my $req_size=$opts{volSize};
	my $aggrname=$opts{aggrName};
	
	my $conn = make_conn($host, "#rsh");

	my (@sizes, $i, $success);
	my %total_size;
	my %list_of_spares_by_size;
	
	my $spares = Aggregate::aggr_spare_status($conn);

        foreach my $spare (@{$spares->{0}}) {
                my $push = 1;
                foreach $i (@sizes) {
                        if ( $i == $spare->{used_mb} ) {
                                $push = 0;
                        }
                }
                if ($push) { push(@sizes, $spare->{used_mb}); }
        }

        logcomment("Sizes are @sizes");

        foreach $i (@sizes) {
                foreach my $spare (@{$spares->{0}}) {
                        if ($spare->{used_mb} eq $i) {
                                push @{ $list_of_spares_by_size{$i} }, $spare->{name};
                        }
                }
        }
        $success = 0;
        for my $key ( keys %list_of_spares_by_size ) {
                my $count = @{ $list_of_spares_by_size{$key} };
                my @disks = @{ $list_of_spares_by_size{$key} };
                logcomment("Number of spares of size $key MB is $count");
                logcomment("Spares of size $key MB are @disks");
                $total_size{$key} = ($key * $count) / 1024;
                if ($total_size{$key} > $req_size) {
                        logcomment("Total spare size is $total_size{$key}");
                        my $num_spares_reqd = ($req_size * 1024) / ($key);
                        my $num_spares = POSIX::ceil($num_spares_reqd);
                        $num_spares = $num_spares + 2; ##for raid-dp aggregates
                        if ($count < $num_spares) {
                                die ("Required:$num_spares Available:$count - Not enough spare disks to create new aggregate $aggrname");
                                return ($aggrname, 0);
                        }
                        logcomment("Number of spare disks required to create new aggr for $req_size g volume is $num_spares");
                        my @req_disks;
                        while ($num_spares) {
                                push(@req_disks, pop(@disks));
                                $num_spares--;
                        }
                        logcomment("Creating new aggr with spare disks: @req_disks");
                        my $aggr_create_result;
                        eval {
                                $aggr_create_result = Aggregate::aggr_create($conn, $aggrname, \@req_disks);
                        };
                        if($@) {
                                die ("$aggr_create_result : Could not create aggregate $aggrname successfully: $@");
                        }
                        $success = 1;
                        return ($aggrname, $success);
                }
        }

}


sub check_old_aggr (%) {
    my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                              volSize => { type=>SCALAR },
                              aggrName => {type=>SCALAR }}
                         );
    my $host=$opts{host};
    my $req_size=$opts{volSize};
    my $aggrname=$opts{aggrName};

	my $conn = make_conn($host, "#rsh");

	## Get aggr free space
	my %aggr_val = filer_aggr_spaceinfo_get($conn, $aggrname);
	my $aggr_space_in_gb =  $aggr_val{FREE} / (1024 * 1024);

	logcomment("Usable space in this aggr is $aggr_space_in_gb GB");
	my $created = 0;

        if ( $aggr_space_in_gb > $req_size ) {
                logcomment("Enough space is available on $aggrname to create volume of $req_size g");
                $created = 1;
                return ($aggrname, $created);
        }
        else {
                logcomment("Checking if there are enough spare disks to add to aggr $aggrname to create $req_size g volume");

                my @disk_arr = Aggregate::aggr_disks($conn, $aggrname);
                my ($disk, $size, @spares_for_aggr, $spares, $spare_cnt);
                foreach $disk ( @disk_arr ) {
                                $size = $disk->{size};
                }

                my $spare_size_in_gb;
                $spares = Aggregate::aggr_spare_status($conn);
                foreach my $spare (@{$spares->{0}}) {
                        my $spare_size = $spare->{used_mb} . "/" . $spare->{used_blks};
                        if ($spare_size eq $size) {
                                push(@spares_for_aggr, $spare->{name});
                                $spare_size_in_gb = $spare->{used_mb} / 1024;
                        }
                }
                $spare_cnt = @spares_for_aggr;
                logcomment("Available spares are @spares_for_aggr");
                logcomment("Number of spares is $spare_cnt");
                if ($spare_cnt) {
                                my $total_spare_space = ($spare_cnt * $spare_size_in_gb);
                                logcomment("Total spare space is $total_spare_space g");
                                if ($total_spare_space > ($req_size - $aggr_space_in_gb)) {
                                        my $num_spares_reqd = ($req_size - $aggr_space_in_gb) / $spare_size_in_gb;
                                        my $num_spares = POSIX::ceil($num_spares_reqd);
                                        my @req_disks;
                                        while ($num_spares) {
                                                push(@req_disks, pop(@spares_for_aggr));
                                                $num_spares--;
                                        }
                                        logcomment("Adding spare disks to aggregate $aggrname: @req_disks");
                                        eval {
                                                Aggregate::aggr_add($conn, $aggrname, \@req_disks);
                                        };
                                        if($@) { logcomment("Could not successfully add disks @req_disks to aggr $aggrname: $@"); }
                                        $created = 1;
                                        return ($aggrname, $created);
                                } else {
					logcomment("Even by adding all spare disks we cannot get the required size for aggregate $aggrname");
				}
		
                } else {
                        logcomment("Volume cannot be created on aggregate: $aggrname");
                        return ($aggrname, $created);
                }
        }

}

sub aggr_space_check(%) {
    my (%opts) = validate(@_, { host => { type=>SCALAR|HASHREF },
                              volSize => { type=>SCALAR },
                              aggrName => {type=>SCALAR }}
                         );
    my $host=$opts{host};
    my $req_size=$opts{volSize};
    my $aggrname=$opts{aggrName};

#	my ($host, $req_size, $aggrname) = @_;
	my ($create_new,$created)=0;
	my $conn=make_conn($host,"#rsh");
	my $root_aggr=Aggregate::aggr_root($conn);
	if (!Aggregate::aggr_exists($conn,$aggrname)) {
                logcomment("Trying to create new aggregate");
        	($aggrname, $create_new) = create_new_aggr( host=>$host , volSize=>$req_size , aggrName=>$aggrname );
	} else {
		logcomment("Checking the existing aggregate");
		($aggrname, $created) = check_old_aggr( host=>$host ,volSize=>$req_size, aggrName=>$aggrname);
	}
	
        if (!$create_new && !$created) {
                logcomment("Existing aggregate doesnot have enough space or Could not create new aggregate - Not enough spare disks");
		logcomment("Check if other aggregates have enough space");
		
                my @aggr_lists = filer_aggrs_get($conn);
                foreach my $aggr (@aggr_lists) {
                        if ($aggr eq $root_aggr || $aggr eq $aggrname) {
                                next;
                        }
                        ($aggrname, $created) = check_old_aggr(host=>$host, volSize=>$req_size, aggrName=>$aggr);
                        if ($created) {
                                $create_new = 1;
                                last;
                        }
                }
        }

        if(($created == 0) && ($create_new == 0)) { 
		die ("Not enough space to create volume of required size: $req_size"); 
	}
	return $aggrname;
}

########################################################
### Parse wafl showvbns output
########################################################
=head2 dense_parse_showvbns(conn=>$conn, file_name=>$file_name, fbn_range => $fbn_range, partner=> 1)
Parse wafl showvbns output

Input fbn_range

	if fbn_range has a single fbn	--> Print showvbns for that particular fbn
	if fbn_range has a fbn range	--> Print showvbns for that particular range of fbn (syntax - 5:10)
	if fbn_range has a empty string	--> Print showvbns for all the fbns in that file

Return value:

	FBN value	: $val{$1}{fbn} = $1;
	VBN value	: $val{$1}{vbn} = $2;
	VVBN value	: $val{$1}{vvbn} = $3;
	Refcount	: $val{$1}{refcount} = $4;

=cut


sub dense_parse_showvbns($@)
{

	my (%opts) = validate 	(@_, { 	conn		=> {type=>SCALAR | HASHREF},
					file_name 	=> {type=>SCALAR},
					option	 	=> {type=>SCALAR, optional=>1},
					fbn_range	=> {type=>SCALAR | ARRAYREF, optional=>1},
					partner 	=> {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/}
                                      }
	                         );


	
	my $conn 	= $opts{'conn'};
	my $file_name   = $opts{'file_name'};
	my ($fbn_range)	= $opts{'fbn_range'};
	my $partner	= $opts{partner};
        my $options     = $opts{'option'};
	
	my $cmd;
	my $i;
	my $full_cmd;
	my @buf;
	
       	if ($partner) {
		$cmd  = " partner " . DENSE_TEST_CMD . " partner " . "wafl showvbns $options -s " . $file_name;
       	}else{
		$cmd = "wafl showvbns $options -s " . $file_name;
       	}
		
	### Run wafl showvbns depends on the input fbn range
	
	my @fbn_range_to_show;
	my $tmp_buf;
	
	if($fbn_range) {
	    my $refType = ref($fbn_range);
		if ($refType !~ /ARRAY/i) {
		    logcomment("fbn_range type = SCALAR");
		    @fbn_range_to_show = split /:/, $fbn_range;
		    if($fbn_range_to_show[1]){
			my $init = 1;
			### Run wafl showvbns for the specified fbn range
			for($i = $fbn_range_to_show[0]; $i <= $fbn_range_to_show[1]; $i++){
				$full_cmd = "$cmd" . " " . $i;
				#push @buf,filer_cli_test_say($conn, $full_cmd);
				if ($init){
					$tmp_buf = filer_cli_test_say($conn, $full_cmd);
					$init = 0;
				}else{
					$tmp_buf = $tmp_buf . "\n" . filer_cli_test_say($conn, $full_cmd);
				}
			}
			### Store the showvbn lists in an array
			@buf = split /\n/, $tmp_buf;
		    }else{
			### Run wafl showvbns for the particular fbn if only one value is specified
			$full_cmd = "$cmd" . " " . $fbn_range_to_show[0];
			$tmp_buf = filer_cli_test_say($conn, $full_cmd);
			@buf = split /\n/, $tmp_buf;
		    }
		} else { 
		    my $init = 1;
		    my $length = scalar (@$fbn_range);
		    logcomment("fbn_range type = ARRAYREF");

		    for($i = 0; $i < $length; $i++) {
			$full_cmd = "$cmd" . " " . $fbn_range->[$i];
			if ($init){
			    $tmp_buf = filer_cli_test_say($conn, $full_cmd);
			    $init = 0;
			}else{
			    $tmp_buf = $tmp_buf . "\n" . filer_cli_test_say($conn, $full_cmd);
			}
		    ### Store the showvbn lists in an array
		    @buf = split /\n/, $tmp_buf;
		    }
		}
	}else{
	    ### Run wafl showvbns without fbn if fbn is not passed to this function
	    $tmp_buf = filer_cli_test_say($conn, $cmd);
	    @buf = split /\n/, $tmp_buf;
	}

        my %showvbns_info= ();
        
        ## Parse the showvbns
	logStep("Parse the showvbns output");
        foreach my $line (@buf)
        {
        	if($line =~ /fbn (\d+) is vbn (\d+), vvbn (\d+), refcount (\d+)/){
                        $showvbns_info{$1} = {};
                        $showvbns_info{$1}{fbn} = $1;
                        $showvbns_info{$1}{vbn} = $2;
                        $showvbns_info{$1}{vvbn} = $3;
                        $showvbns_info{$1}{refcount} = $4;
                        logcomment("DEBUG: Storing showvbns {$1}{fbn} = $showvbns_info{$1}{fbn} -- {$1}{vbn} = $showvbns_info{$1}{vbn}, {$1}{vvbn} = $showvbns_info{$1}{vvbn} , {$1}{refcount} = $showvbns_info{$1}{refcount}");
        	} elsif($line =~ /fbn (\d+) is vbn (\w+), vvbn (\w+), refcount (\d+)/) {
                        $showvbns_info{$1} = {};
                        $showvbns_info{$1}{fbn} = $1;
                        $showvbns_info{$1}{vbn} = $2;
                        $showvbns_info{$1}{vvbn} = $3;
                        $showvbns_info{$1}{refcount} = $4;
                        logcomment("DEBUG: Storing showvbns {$1}{fbn} = $showvbns_info{$1}{fbn} -- {$1}{vbn} = $showvbns_info{$1}{vbn}, {$1}{vvbn} = $showvbns_info{$1}{vvbn}, {$1}{refcount} = $showvbns_info{$1}{refcount} ");
                }else{
                	die fatal_result ("Invalid showvbns output format: Obtained => $line");                        
                }
        }
        return %showvbns_info;

}

########################################################
### Get number of blocks with refcnt zero
########################################################
=head2 dense_get_unshared_blkcnt(conn=>$conn, file_name=>$file_name, fbn_range => $fbn_range, partner=> 1)
Note: The parameters are same as for dense_parse_showvbns and it gets validated in that function.
Parse wafl showvbns output

Input fbn_range

	if fbn_range has a single fbn	--> return refcnt of that particular fbn
	if fbn_range has a fbn range	--> return number of fbns in that particular range of fbn (syntax - 5:10), that has refcnt zero
	if fbn_range has a empty string	--> return number of fbns in the whole file, that has refcnt zero

Return value:

	number of blocks in the file within the fbn range, which are not shared

=cut

sub dense_get_unshared_blkcnt($@)
{
	my %vbns_info	= &dense_parse_showvbns;
	my $num_unshared_blks	= grep { $_->{refcount}==0 } (values %vbns_info);
	return($num_unshared_blks);
}

###
### Compare hashes
###
=head2 dense_get_inode_num(hash1=>$hash1, hash2=>$hash2)

Description:
	Compare two hashes
Return value:
	Success	: Return Succuss
	Fail	: Return Fail
=cut
sub dense_compare_hashes ($@)
{

	my (%opts) = validate (@_,{	hash1		=> {type =>HASHREF},
					hash2		=> {type =>HASHREF},
				  }
			       );
	
	my $hash1	= $opts{'hash1'};
	my $hash2	= $opts{'hash2'};
	my  (@keys);
	map {
	    if ($hash1->{$_} cmp $hash2->{$_}) {
		return DENSE_CLONE_FAIL;
	    }
	} @keys = keys(%$hash1);
	return DENSE_CLONE_SUCCESS; 
}

########################################################
### Compare vbn, vvbn and refcount (from wafl showvbns output)
########################################################
=head2 dense_compare_wsv(hash1=>$hash1, hash2=> $hash2)

Description:
	Compare wafl showvbns output for 2 hashes

Return value:
	Success	: Return Success if fbn, vbn, vvbn & refcount are same
	Fail	: Return Fail if fbn, vbn, vvbn & refcount are different
=cut

sub dense_compare_wsv ($@)
{

	my (%opts) = validate (@_,{	hash1		=> {type =>HASHREF},
								hash2		=> {type =>HASHREF},
								}
							);
	my $hash1	= $opts{'hash1'};
	my $hash2	= $opts{'hash2'};
	
	foreach my $fbn (keys %$hash1){
		my $comp_result = dense_compare_hashes (hash1 => $hash1->{$fbn}, hash2 => $hash2->{$fbn});
		if($comp_result ne DENSE_CLONE_SUCCESS){
			logcomment("DEBUG: Hash1 and Hash2 has different values");
			logcomment("DEBUG: Hash1 Values: fbn:$fbn, vbn: $hash1->{$fbn}->{vbn}, vvbn:$hash1->{$fbn}->{vvbn}, refcnt: $hash1->{$fbn}->{refcount} ");
			logcomment("DEBUG: Hash2 Values: fbn:$fbn, vbn:$hash2->{$fbn}->{vbn}, vvbn:$hash2->{$fbn}->{vvbn}, refcnt:$hash2->{$fbn}->{refcount} ");
			return  DENSE_CLONE_FAIL;
		}
	}
	return  DENSE_CLONE_SUCCESS;
}

######################
### Get Inode number
######################
=head2 dense_get_inode_num(conn=>$conn, file_name => $file_name,  timeout => $timeout, partner=> 1)

Description:
	
	Get the file name (absolute path) and connection and return the inode number
	
Return value:

	Success	: Return the inode number
	Fail	: Return the error message

=cut

sub dense_get_inode_num
{

	my (%opts) = validate (@_,{	conn		=> {type =>SCALAR|HASHREF}, 
					file_name	=> {type=>SCALAR},
					timeout 	=> {type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1},
					partner		=> {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/},
				  }
			       );

	my $conn 	= $opts{'conn'};
	my $file_name	= $opts{'file_name'};
	my $timeout	= $opts{timeout};
	my $partner	= $opts{partner};
	
	my $cmd;
	my $inode_number;
	my $hex_inode_number;
	
	$cmd = "showfh $file_name";

	### Check partner mode
	if ($partner){
		$cmd  = " partner " . DENSE_ADV_CMD . " partner " . "$cmd"; 
	}
	
	my ($showfh_output, $result_code) = filer_cli_advanced_say($conn, $cmd, {timeout=>$timeout});
	
	my @showfh_command = split(/ /, $showfh_output);
	foreach my $entry (@showfh_command)
	{
		if ($entry =~ /fileid=(.*)/){
		$hex_inode_number=$1;
		logcomment("DEBUG: File name: $file_name, Hex inode number: $hex_inode_number");
	}
		next;
	}

	### Convert Hex value to decimal value
	$inode_number = hex($hex_inode_number);
	if ($inode_number){
		return $inode_number;
	}else{
		return $showfh_output;
	}
}

############################################
### Get inode details from dumpblock output
############################################

=head2 dense_parse_dumpblock(conn=>$conn, vol => $vol,  inum =>$inum, timeout => $timeout, partner=> 1)

Description:
	
	Get inode attributes from dumpblock output and return the inode_info hash reference
	
Return value:

	Success	: Return the inode_info hash reference
	Fail	: Return the inode_info hash reference

=cut


sub dense_parse_dumpblock {

    my (%opts) = validate( @_, {
            conn => { type => SCALAR | HASHREF },
            vol  => { type => SCALAR },
            inum => { type => SCALAR, regex => qr/^[1-9]\d*$/ },
            private_inode => { type => SCALAR, optional => 1, default => 0, regex => qr/^(0|1)$/ },
            timeout => { type => SCALAR, default => DENSE_CLONE_DEFAULT_TIMEOUT, optional => 1 },
            partner => { type => SCALAR, optional => 1, default => DENSE_CLONE_DEFAULT_PARTNER, regex => qr/^(0|1)$/ },
        }
    );

    my $conn          = $opts{conn};
    my $vol           = $opts{vol};
    my $inum          = $opts{inum};
    my $timeout       = $opts{timeout};
    my $partner       = $opts{partner};
    my $private_inode = $opts{private_inode};
    
    my $dump_cmd = "dumpblock ifile $vol $inum";
    if ($private_inode) {
        logcomment('Executing dumpblock in private inode space');
        $dump_cmd = $dump_cmd . ' -private';
    }

    ### Check partner mode
    if ($partner) {
        $dump_cmd = " partner " . DENSE_DIAG_CMD . " partner " . "$dump_cmd";
    }

	my $dumpblk_restart = 3;
	my %inode_info= ();
	$inode_info{'version'} = 0;
	while (($inode_info{'version'} == 0) && ($dumpblk_restart > 0)){
		my ($dump_output, $return_code) = filer_cli_diag_say($conn, $dump_cmd, {timeout=>$timeout});

		my @blocks = split(/\n/, $dump_output);
		
		# Parse dumpblock's output.

		my $BLKSZ		= 4096;
		my $INOSZ		= 192;
		my $HDRSZ		= $BLKSZ % $INOSZ;
		my $INODESPERBLK	= (($BLKSZ - $HDRSZ) / $INOSZ);

		#dumpblock <ifile> <inum> prints out 4192 bytes (including the
		#checksum etc.) At any rate, we should NOT get less than 4K of data.
		#IOW, 262 (or 256) lines of 4 x 4-byte numbers.
		my $MAXBUFLEN	= 262;
		my $MINBUFLEN	= 256;
		
		my $inum_dec_offset = $HDRSZ + ($INOSZ * ($inum % $INODESPERBLK));
		my $inum_hex_offset =  sprintf("%X", $inum_dec_offset);
		
		### Convert the hex value to 4 digit number
		my $string_len =  length($inum_hex_offset);
		for(my $b=4;$b>$string_len;$b--){
			$inum_hex_offset = 0 . $inum_hex_offset;
		}
		
		print "dec offset is $inum_dec_offset \n";
		print "hex offset is $inum_hex_offset \n";

		#### Get the line number for the particular Inode
		my ($inum_dump, $inum_dump1, $inum_dump2, $inum_dump3, $inum_dump4,$inum_dump5);

		my $line_cnt = 0;
		my $inum_line_no = 0;
		while ($blocks[$line_cnt]) {
			if ($blocks[$line_cnt] =~ m/0x$inum_hex_offset\:/i){
				$inum_line_no = $line_cnt;
			}
			$line_cnt++;
		}

		#### Get the subsequent lines for the inode to get more attributes 
		$inum_dump = $blocks[$inum_line_no];
		$inum_line_no++;
		$inum_dump1 = $blocks[$inum_line_no];
		$inum_line_no++;
		$inum_dump2 = $blocks[$inum_line_no];
		$inum_line_no++;
		$inum_dump3 = $blocks[$inum_line_no];
		$inum_line_no++;
		$inum_dump4 = $blocks[$inum_line_no];
		$inum_line_no++;
		$inum_dump5 = $blocks[$inum_line_no];
		
		#### Split the lines to get the each inode attributes
		
		my @inum_values = split(' ',$inum_dump);
		my @inum_values1 = split(' ',$inum_dump1);
		my @inum_values2 = split(' ',$inum_dump2);
		my @inum_values3 = split(' ',$inum_dump3);
		my @inum_values4 = split(' ',$inum_dump4);
		my @inum_values5 = split(' ',$inum_dump5);

		#### Get each inode attributes
			my $inum_links 	= $inum_values1[1];
			my $inum_gentime= $inum_values1[2];
			my $inum_uid 	= $inum_values1[3];
			my $inum_gid	= $inum_values1[4];
			
			my $inum_atime	= $inum_values2[1];
			my $inum_mtime	= $inum_values2[3];
			
			my $inum_ctime	= $inum_values3[1];
			my $inum_crtime	= $inum_values3[3];
			
			my $inum_size	= $inum_values4[1];
			my $inum_blkcnt	= $inum_values4[3];
			my ($inum_xinode, $inum_sinode);
			my $filername = $conn->hostp()->hostname();
            if ( getwv( host => $filername ) eq "BR" ) {
                $inum_xinode = $inum_values5[1];
                $inum_sinode = $inum_values5[2];
            } else {
                $inum_xinode = $inum_values4[4];
                $inum_sinode = $inum_values5[1];
            }
		
		### Construct the inode_info hash
			$inode_info{'links'} 	= $inum_links;
			$inode_info{'gentime'} 	= $inum_gentime;
			$inode_info{'uid'} 		= $inum_uid;
			$inode_info{'gid'}		= $inum_gid;
			
			$inode_info{'atime'}  	= $inum_atime;
			$inode_info{'mtime'}  	= $inum_mtime;

			$inode_info{'ctime'}  	= $inum_ctime;
			$inode_info{'crtime'} 	= $inum_crtime;

			$inode_info{'size'}   	= $inum_size;
			$inode_info{'blkcnt'} 	= $inum_blkcnt;
			$inode_info{'xinode'} 	= $inum_xinode;

			$inode_info{'sinode'} 	= $inum_sinode;
		
		#### Convert the values to decimal
			for my $inode_key ( keys %inode_info ) {
				$inode_info{$inode_key} = (sprintf "%d",hex($inode_info{$inode_key}));
			}
		
		#### Get other inode attributes
			#### inode number
			$inode_info{'inum'} = $inum;
			
			#### Convert permission to Octal value
			my $inum_perms_hex 			= $inum_values[2];
			my $inum_perms_dec			= (sprintf "%d",hex($inum_perms_hex));
			$inode_info{'permission'}	= (sprintf "%o",$inum_perms_dec);
				
			#### Keep the flag values in Hex	
			$inode_info{'dflags'}		= $inum_values[4];

			#### Other inode values
			my $bin_inode_values= unpack("B*", pack("H*", $inum_values[1]));
			
			$inode_info{'version'}	=  unpack("N", pack("B32", substr("0" x 32 . (substr $bin_inode_values, 32, 8), -32)));
			$inode_info{'type'}		=  unpack("N", pack("B32", substr("0" x 32 . (substr $bin_inode_values, 24, 8), -32)));
			$inode_info{'subtype'}	=  unpack("N", pack("B32", substr("0" x 32 . (substr $bin_inode_values, 16, 8), -32)));
			$inode_info{'level'}  	=  unpack("N", pack("B32", substr("0" x 32 . (substr $bin_inode_values, 8, 8), -32)));
	$dumpblk_restart--;	
	}
	return \%inode_info;
}

############################################
### Check dense flag for a file
############################################

=head2 check_inode_dense_flag(conn=>$conn, file_name => $file_name,  timeout => $timeout, partner=> 1)

Description:
	
	Check dense flag for the input file
	
Return value:

	Success	: Return the dense bit from the inode
	Fail	: Return the dense bit from the inode

=cut


sub check_inode_dense_flag {

    my (%opts) = validate( @_, {
            conn          => { type => SCALAR | HASHREF },
            vol           => { type => HASHREF },
            file_name     => { type => SCALAR },
            private_inode => { type => SCALAR, optional => 1, default => 0, regex => qr/^(0|1)$/ },
            timeout => { type => SCALAR, default => DENSE_CLONE_DEFAULT_TIMEOUT, optional => 1 },
            partner => { type => SCALAR, optional => 1, default => DENSE_CLONE_DEFAULT_PARTNER, regex => qr/^(0|1)$/ }
        }
    );

    my $conn          = $opts{conn};
    my $vol           = $opts{vol};
    my $file_name     = $opts{file_name};
    my $private_inode = $opts{private_inode};
    my $timeout       = $opts{timeout};
    my $partner       = $opts{partner};

    my $dump_cmd;
    my $inode_info;
    my $inode_flag;

    my $inode_num = dense_get_inode_num(
        conn      => $conn,
        file_name => $file_name,
        timeout   => $timeout,
        partner   => $partner
    ) or warn fatal_result("Getting inode for file $file_name Failed");

    if ( $inode_num =~ /^[0-9]+$/ ) {
        logcomment("DEBUG: node numbers for file $file_name is $inode_num");
    } else {
        die("Getting inode for file $file_name Failed. $inode_num\n");
    }

    $inode_info = dense_parse_dumpblock(
        conn          => $conn,
        vol           => $vol->{name},
        inum          => $inode_num,
        private_inode => $private_inode,
        timeout       => $timeout,
        partner       => $partner
    ) or warn fatal_result("Parsing dumpblock dense_parse_dumpblock Failed");

    $inode_flag = $inode_info->{dflags};

    my $bin_inode_flag = unpack( "B*", pack( "H*", $inode_flag ) );
    my $dense_bit = substr $bin_inode_flag, -16, -15;

    if ($dense_bit) {
        logcomment("DEBUG: File $file_name has a dense flag");
    } else {
        logcomment("DEBUG: File $file_name does not have a dense flag");
    }
    return $dense_bit;
}

############################################
### Get the index for the specified quota
############################################

=head2 dense_get_quota_index(volname=>$volname, qtree => $qtree,  quota_type => $quota_type, quota_report=> \@quota_report)

Description:
	Get the index for the specified quota

Note:
	We will face an issue if the return value is '0'. So this function returns the variable reverence.
	The Calling script/function should dereference the varibele
Return value:

	Success	: Return the variable reverence
	Fail	: Return the eror message

=cut


sub dense_get_quota_index ($@)
{
	my (%opts) = validate 	(@_, {	volname 	=> {type=>SCALAR},
									qtree		=> {type=>SCALAR},
									quota_type 	=> {type=>SCALAR},
									quota_report=> {type=>ARRAYREF},
	                              }
	                          );	
	my $vol		= $opts{volname};
	my $qtree	= $opts{qtree};
	my $type	= $opts{quota_type};
	my @qr		= @{$opts{quota_report}};
	my $index 	= 0;
	
	foreach my $thisqr (@qr) {
        logcomment("searching $thisqr->{id} for $qtree");
		if (($thisqr->{volume} eq $vol) && ($thisqr->{tree} eq $qtree) && ($thisqr->{type} eq $type)){
			logcomment("index: $index, vol: $thisqr->{volume} , tree $thisqr->{tree}, type $thisqr->{type}");
			return \$index;
		}else{
			$index++;
		}
	}
	die(fatal_result("Can't find index for $type, $qtree in quota report"));
}

############################################
### Check for LA
############################################

=head2 dense_is_larger_aggr(conn=>$conn, aggr_name => $aggr_name)

Description:
	Check for 64-bit aggregate

Return value:

	Return 1 : If its a 64-bit aggregate
	Return 0 : If its a 32-bit aggregate

=cut

sub dense_is_larger_aggr ($@)
{
	my (%opts) = validate 	(@_, {	conn 		=> {type=>HASHREF},
									aggr_name	=> {type=>SCALAR},
									timeout 	=> {type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1},
	                              }
	                          );	
	my $conn		= $opts{conn};
	my $aggr		= $opts{aggr_name};
	my $timeout		= $opts{timeout};
	my $partner		= $opts{partner};

	my $aggr_status_obj = aggr_status($conn, $aggr) or die $@;
	my $aggrstatus = $aggr_status_obj->{status};

	logcomment("Aggr status for $aggr: $aggrstatus");
	if ($aggrstatus =~m /64-bit/){
		return 1;
	}else{
		return 0;
	}
}

############################################
### Get Subtest results
############################################

sub get_subtest_results($@) {
  my ($obj, $type, @expected) = @_;

  return eval {
    my $rslts;

	$rslts = extractResults($obj, $type, qw(EVAL_ERROR));
	logcomment("From extractResults is :  $rslts->{'EVAL_ERROR'} and other value is $@ end");

	die $@ if ($@);
	die $rslts->{'EVAL_ERROR'} . "\n";
  }
}

############################################
### Set overlap requests to off or on
############################################

sub set_overlap_requests($@) {
	my (%opts) = validate(@_, {	conn => {type=>HASHREF},
	state => {type=>SCALAR, optional=>1, default=>"on", regex=>qr/^o(n|ff)$/ },
	partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }
	                              }
	                          );

	my ($state) = $opts{state};
	my $cmd;
    logcomment("Setting option sis.overlap_requests to $state");
	
	$cmd="partner " if($opts{partner});
	$cmd.="options sis.overlap_requests";

    filer_cli_say($opts{conn}, "$cmd $state");
	# Checking the state is set correctly
    my ($text, $code) = filer_cli_say($opts{conn}, $cmd);

    # No errror is return
	if( $text =~ /$state/ ){
		return $text;
	}else{
		die("$text\n");
	}
}


############################################
### Set sis.p1merge_bins value
### This functions sets options sis.p1merge_bins to given value
############################################

sub set_p1merge_bin_value($@) {
    my (%opts) = validate(@_, { conn => {type=>HASHREF},
    value => {type=>SCALAR, optional=>1, default=>256, regex=>qr/^[\d]/ },
    partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }
                                  }
                              );

    my ($value) = $opts{value};
    my $cmd;
    logcomment("Setting option sis.p1merge_bins to $value");

    $cmd="partner " if($opts{partner});
    $cmd.="options sis.p1merge_bins";

    filer_cli_say($opts{conn}, "$cmd $value");
    # Checking the state is set correctly
    my ($text, $code) = filer_cli_say($opts{conn}, $cmd);

    # No errror is return
    if( $text =~ /$value/ ){
        return $text;
    }else{
        die("$text\n");
    }
}

############################################
### Get sis.p1merge_bins value
### This functions get options sis.p1merge_bins value
############################################

sub get_p1merge_bin_value($@) {
    my (%opts) = validate(@_, { conn => {type=>HASHREF},
    partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }
                                  }
                              );
	my $cmd;
   	logcomment("Getting option sis.p1merge_bins");
    $cmd = "partner " if($opts{partner});
    $cmd.="options sis.p1merge_bins";    
	filer_cli_say($opts{conn}, "$cmd");
	my ($text, $code) = filer_cli_say($opts{conn}, $cmd);
	my @array =  split(/\n/,$text);
        for my $line (@array) {
                my @value =  split(' ',$line);
                return $value[1];
        }
}


sub set_inode_hash($@) {
	my (%opts) = validate(@_, {	conn => {type=>HASHREF},
	state => {type=>SCALAR, optional=>1, default=>"on", regex=>qr/^o(n|ff)$/ },
	partner => { type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/ }
	                              }
	                          );

	my ($state) = $opts{state};
	my $cmd;
    logcomment("Setting option sis.deleted_inode_hash to $state");
	
	$cmd="partner " if($opts{partner});
	$cmd.="options sis.deleted_inode_hash";

    filer_cli_say($opts{conn}, "$cmd $state");
	# Checking the state is set correctly
    my ($text, $code) = filer_cli_say($opts{conn}, $cmd);

    # No errror is return
	if( $text =~ /$state/ ){
		return $text;
	}else{
		die("$text\n");
	}
}

##########################################################################  
### Functions to set or get the sis options value on the filer         ###  
##########################################################################  
=head2 set_sis_options

	set or get the sis options value on the filer  

Inputs:  
    1. conn - filer connection
    2. set_value (optional) - value of the option that needs to be set  

Return value:  
    The value of the options read or set  

=cut  

sub set_sis_options($@) {  
	my (%opts) = validate (@_, { 	conn		=> {type=>SCALAR | HASHREF},
                             		option_name	=> {type=>SCALAR},
									value		=> {type=>SCALAR,optional=>1},
									partner		=> {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/},
									vfiler  	=> {type=>SCALAR, optional=>1},
								}
							);
									
    my $conn		= $opts{conn};
	my $option_name	= $opts{option_name};
	my $value		= $opts{value};
	
    my ($ret_val, $set_op, $output, $cmd);
    
    $ret_val = 0; 
	$cmd="partner " if($opts{partner});
	$cmd.="vfiler run $opts{vfiler} " if($opts{vfiler});
	$cmd.="options $option_name";
    
    $output = filer_cli_test_say($conn, $cmd);  
    logcomment("options exec output: ".$output);
    if ($output =~ /no such option/i) {
		my $version = getwv(host=>{($opts{'conn'})->{'hostname'}}); 
		die("Option $option_name is not supported on $version release");
    } 
    if (defined($value) && ($value ne "")) {  
        logcomment("Setting $option_name to $value");  
        $set_op = filer_cli_test_say($conn, $cmd." ".$value); 
    } 
    $output		= filer_cli_test_say($conn, $cmd);  
	my @ct_value	= split(' ',$output);
	$ret_val		= $ct_value[1];
		
    logcomment("Option $option_name value is = $ret_val"); 
    if (defined($value) && ($value ne "")) {
		$ret_val	= remove_leading_trailing_spaces($ret_val);
		$value		= remove_leading_trailing_spaces($value);
		if ($ret_val ne $value){
			die($cmd." was not set successfully : ".$set_op);  
		} else {  
			logcomment($cmd." was set successfully ".$value)
		}
    } 
    return $ret_val; 
}

########################################################
### Enable or disable hash optimization
########################################################
=head2 setflag_hash_optimization(conn=>$conn, state=>$state)

Enable or disable hash optimization based on the state value

=over 4
=item conn 
Filer connection

=item state 
The state of the hash optimization (either 0 or 1)


Return value: 
Success : 0
Fail: 1

=cut

sub setflag_hash_optimization ($@) { 
    my(%opts) = validate(@_, 
		    {
			conn => {type=>HASHREF}, 
    		    	state => {type=>SCALAR, optional=>1, default=>"1", regex=>qr/^(1|0)$/ }, 
    		    }
		); 
    my ($flag, $state, $cmd, $text, $code);

    logcomment("Version: $DENSE_ONTAP_VERSION");
    
    $state = $opts{state};
    
    if($DENSE_ONTAP_VERSION eq "GB") {
	$flag = "wafl_dense_use_vvbn_hash"; 
    } else {
	$flag = "wafl_read_from_pvbn_hash"; 
    } 
    
    $cmd = "setflag $flag $state";
    
    logcomment("$DENSE_ONTAP_VERSION: Setting: $cmd");
    filer_cli_test_say($opts{conn}, "$cmd");
    
    
    # Checking the state is set correctly 
    $cmd = "printflag $flag"; 
    ($text, $code) = filer_cli_test_say($opts{conn}, $cmd); 
    
    if((split(/\s+/,$text))[2] eq $state ){
	logcomment("Successfully set: $text");
	return 0 ; 
    } else {
	logcomment("Failed to set: $text"); 
	return 1; 
    } 
}

sub get_fingerprint_count($@) {
	my (%opts) = validate(@_, {
		        host => {type=>SCALAR},
                        path_name => {type=>SCALAR},
                        }
                      );
        my ($host) = $opts{host};
        my ($path_name) = $opts{path_name};
	my $check_status = dense_check('host' => $host, 'path_name' => $path_name , 'cmd_opt' => '-c');
   	my @array =  split(/\r*\n/,$check_status);
    	for my $line (@array) {
        	if ($line  =~ /(\d+)\s+records$/) {
           	return $1;
          	}
      }
 }

=head2 reboot_verify ($hostspec, ...options...)

Reboot the filer and verify if the desired state is achieved .

=over 4

=item host

Filer HostSpec, required.

=item options

Complete list available in transit_to() in boot.pm

=back

=cut


sub reboot_verify ( $@ ) {
    my ($spec, @opts) = @_;
	my $ret;
	eval{
		$ret = reboot_to($spec,"UP",@opts);
	};
	my $bootstate = boot_state($spec);

    if ( $bootstate ne "UP"){
		logcomment("Filer in state: $bootstate, reboot not complete or login prompt missing. Retrying.....");
		push @opts, (powercycle => 0);
		$ret = transit_to($spec,"UP",@opts);
	}
	return $ret;
}

############################################
### Rotate all syslog messages 
############################################

=head2 dense_syslog_rotate(conn=>$conn)

Description:
       	Rotates all the syslog messages 

Return value:

        Return 1 : If successfully able to rotate 	 
        Return 0 : If not.  

=cut

sub dense_syslog_rotate( $@ ) {
    my (%opts) = validate   (@_, { conn => {type=>HASHREF } } );
    my $conn = $opts{conn};
    my $iter=0;		
    my $ret=0;
    do {
        eval {
            filer_cli_test_say($conn," syslog rotate_syslog");
        };
	logcomment("Command status is $@");
	sleep(30);
	my $cmd = "ems log dump 1s";
        my (undef, undef, $output) = say($conn, "$cmd",-timeout,300);
	$ret = grep /<NR/, $output;
	$iter++;
    } while ( (!$ret) && ( $iter <= 3) );
    if ($@ =~ /command failed/) {
        logcomment("Could not rotate the syslog \n");
        return 0;
    }
    else {
        return 1;
    }
}

#################################
### Get the volume statistics
#################################

=head2 dense_get_vol_stats(conn=>$conn, volname => $volname,  timeout => $timeout, partner=> $partner)

name : dense_get_vol_stats

Description:
	Show volume statistics
	Runs 'stats show volume:<vol>' to gather volume statistics
	$volnams is Optional: Show stats of volume name. Default is all volumes
	
Return value:
	Success	: Returns volume statistics hash
	Fail	: Returns the eror message

=cut

sub dense_get_vol_stats ($@)
{
	my (%opts) = validate 	(@_, {	conn 		=> {type=>HASHREF},
									volname 	=> {type=>SCALAR,optional=>1},
									timeout 	=> {type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1},
									partner		=> {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/},
	                              }
	                          );	
	my $conn	= $opts{'conn'};
	my $volname	= $opts{'volname'};
	my $timeout	= $opts{'timeout'};
	my $partner	= $opts{'partner'};

	my %vol_stats;
	my $stats_cmd = "stats show volume:$volname";
   	### Check partner mode
	if ($partner){
		$stats_cmd  = " partner " . DENSE_DIAG_CMD . " partner " . "$stats_cmd"; 
	}
	my ($stats_text, undef) = filer_cli_diag_say($conn,$stats_cmd,{retry=>'ok'});
	if ($stats_text =~ /No instance|invalid object name|Unknown/i) {
		die fatal_result ("Could not get volume stats: $stats_text");
	}
	my @vol_stats_output = filer_cli_split($stats_text);
	foreach my $stats_line (@vol_stats_output) {
		my (undef, $name, $key, $value) = split(/:/, $stats_line);
		if (defined($name) && defined($key) && defined($value)) {
		$vol_stats{$key} = $value;
		}
	}
    if (!%vol_stats) {
        die fatal_result ("Could not get volume stats");
    }
    return \%vol_stats;
}

#################################
### Get FSFLAGS
#################################

=head2 dense_get_fsflags(conn=>$conn, volname => $volname,  timeout => $timeout, partner=> $partner)

name : dense_get_fsflags

Description:
	Get the FSFLAG values for the given volume and return the hash reference with 
	FSFLAGs name and its values.
	
Return value:
	Return hash reference with the list of FSFLAGS & its value

=cut

sub dense_get_fsflags ($@)
{
	my (%opts) = validate 	(@_, {	conn 		=> {type=>HASHREF},
									volname 	=> {type=>SCALAR},
									timeout 	=> {type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1},
									partner		=> {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/},
	                              }
	                          );
	my $conn		= $opts{'conn'};
	my $volname		= $opts{'volname'};
	my $timeout		= $opts{'timeout'};
	my $partner		= $opts{'partner'};
	my %fsflag_status;

	### Get Volume statistics
	my $vol_stats_ref = dense_get_vol_stats (conn => $conn, volname => $volname, timeout=> $timeout, partner=> $partner)
					or die fatal_result ("Getting vol stats for volume $volname Failed");
	my %vol_stats = %$vol_stats_ref;
					
	### Get FSFLAG value for the volume (hex value)
	my $vol_fsflags_hex	= hex($vol_stats{'wv_volinfo_fs_flags'});

	### Check all the FSFLAGS and create a hash with FSFLAG name and its value
	my $fsflag_ref = DENSE_FSFLAGS;
	my %fsflag_list = %$fsflag_ref;
	foreach my $flag_name (keys %fsflag_list){
		my $fsflag_hex 		= $fsflag_list{$flag_name};
		if ($vol_fsflags_hex & $fsflag_hex) {
			$fsflag_status{$flag_name} = 1;
		}else{
			$fsflag_status{$flag_name} = 0;
		}
	}
	return \%fsflag_status;
}

#################################
### Check FSFLAGS
#################################

=head2 dense_check_fsflags(filer=>$filer, volname => $volname,  exp_fsflags => $flags_to_check, timeout => $timeout, partner=> $partner)

name : dense_check_fsflags

Description:
	Get fsflags through dense_get_fsflags and check if it is set as expected from exp_fsflags
	
Return value:
	the list of flags which is not matching the expected value
=cut

sub dense_check_fsflags ($@)
{
	my (%opts) = validate 	(@_, {	filer 		=> {type=>HASHREF},
									volname 	=> {type=>SCALAR},
									exp_fsflags	=> {type=>HASHREF},
									timeout 	=> {type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1},
									partner		=> {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/},
	                              }
	                          );
	my $filer		= $opts{'filer'};
	my $volname		= $opts{'volname'};
	my $exp_fsflags	= $opts{'exp_fsflags'};
	my $timeout		= $opts{'timeout'};
	my $partner		= $opts{'partner'};
	my @unmatched_flags;
	
	### Get Volume statistics
	my $fsflag_status_ref = dense_get_fsflags (conn => $filer->{conn}, volname => $volname,timeout=> $timeout, partner=> $partner) or die fatal_result ("Getting vol stats for volume $volname Failed");

	foreach my $fsflag (keys %$exp_fsflags) {
		logcomment("DEBUG: Flag $fsflag value: $fsflag_status_ref->{$fsflag} ");
		if($exp_fsflags->{$fsflag} != $fsflag_status_ref->{$fsflag}) {
			logcomment("Dense fsflag $fsflag expected $exp_fsflags->{$fsflag}, but found $fsflag_status_ref->{$fsflag}");
			push(@unmatched_flags, $fsflag);
		}
	}

	return @unmatched_flags;
}

#################################
### Get sis ls information in the form of an hash
#################################

=head2 get_sisls_details(filer=>$filer, volname => $volname,  timeout => $timeout, partner=> $partner)

name : get_sisls_details

Description:
	parses the sis ls ouput from the filer and returns it in the form of hashref
	
Return value:
	hashref containing the parsed sis ls output
=cut

sub get_sisls_details {
	my (%opts) = validate 	(@_, { 	filer	=> {type=>HASHREF},
									volname	=> {type=>SCALAR},
									timeout	=> {type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1},
									partner => {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/},
								}
						   );
	my $volpath="/vol/".$opts{volname};
	my %metafile;
	my ($return_text, $result_code) = filer_cli_test_say($opts{filer}->{conn}, "sis ls $volpath");
	return undef if(!$return_text);
	my @lines=split(/\n/, $return_text);
	shift(@lines);
	foreach(@lines) {
		next if(!/\S/); #Check if it is an empty line
		if(/Total Blocks Used By $opts{volname} sis Metafiles on the aggregate/) {
			$metafile{TotalBlocksUsed}=(split)[-1];
		} else {
			my ($name, $size, $blocks)=split;
			$metafile{$name}{size}=$size;
			$metafile{$name}{blocks}=$blocks;
		}
	}
	return(\%metafile);
}
########################################################
### Wait for sfsr operation to complete
########################################################
=head2 dense_wait_sfsr_complete(conn=>$conn, volname=>$volname, timeout => 3600, partner=> 1
					interval => 3, max_wait=>30)

Get the volname and check the sfsr status in a loop (interval:3sec, max time to wait:30 sec) till
all the sfsr operation is completes.

Return value:
	Success	: Return 0
	Fail	: Return 1

=cut

sub dense_wait_sfsr_complete($@)
{
	my (%opts) = validate (@_, {conn 		=> {type=>SCALAR | HASHREF},
								volname		=> {type=>SCALAR},
								interval	=> {type=>SCALAR, default =>DENSE_CLONE_INTERVAL, optional =>1},
								max_wait	=> {type=>SCALAR, default =>DENSE_CLONE_MAX_TIME_TO_WAIT, optional =>1},
								timeout 	=> {type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1},
								partner		=> {type=>SCALAR, optional=>1, default=>DENSE_CLONE_DEFAULT_PARTNER, regex=>qr/^(0|1)$/},
							}
						);
						

	my $conn		= $opts{conn};
	my $volname		= $opts{volname};
	my $interval	= $opts{interval};
	my $max_wait	= $opts{max_wait};
	my $timeout		= $opts{timeout};
	my $partner		= $opts{partner};

	my $sfsr_cmd 	= DENSE_SFSR_STATUS . " $volname";
	if ($partner){
		$sfsr_cmd  = " partner " . DENSE_TEST_CMD . "$sfsr_cmd"; 
	}
	my $sfsr_out 	= filer_cli_say($conn, "priv set -q test; $sfsr_cmd");
	my $sfsr_string = "0 single-file snap restore";
	
	while(($max_wait >= 0)){
		if ($sfsr_out =~ $sfsr_string){
			return 1;
		}
		logcomment("SFSR status time-out: $max_wait");
		sleep($interval);
		$sfsr_out = filer_cli_say($conn, "priv set -q test; $sfsr_cmd");
		$max_wait-= $interval;
	}
	die fatal_result("Waiting for SFSR to complete timed out");
}

#################################
### Check if the filer is installed with promoted build 
#################################

=head2 is_debug(conn=>$conn)

name : is_debug

Description:
	Check whether the installed build in the filer is promoted or not.	
	
Return value:
	Return 1 if debug else 0	
=cut

sub is_debug(%) {
    my (%opts) = validate   (@_, {  conn   => {type=>HASHREF},
				    } );

    my $conn = $opts{conn};
    my $return_text = filer_cli_test_say($conn,"version");	 
    if ($return_text =~ /<.*d.*>/) {
		return 1;
    } else {
		return 0;
    }
}
   
#################################
### Get the snapid of the matching snapshot 
#################################

=head2 get_snap_id($conn,$VOL_NAME,$match_snap);

name : get_snap_id 

Description:
	Get the snapid of the snapshot which matches the match string.	
	
Return value:
	Return snap_id  
=cut

sub get_snap_id($$$) {
    my($conn,$vol,$pattern)= @_;
    my $snapid;
    my @snap_status = snapshot_status($conn,$vol);
    foreach my $snap_rec (@snap_status) {
	if ($$snap_rec{name} =~ /$pattern/){
	    $snapid = $$snap_rec{snapid};
	    return $snapid;
	}
    }
}

#################################
### Get the filer info 
#################################

=head2 get_filer_model_info(filer=>$FILER,opt=>"arch"); 

name : get_filer_model_info 

Description:
	Get the filer model information for the given option 	
	
Return value:
	Return options specified -> opt can be "cluster" , "floppy" , "arch" , "bootcard" , "brand" , "firmware" , "boot" , "processors" , "appliance" .   
=cut

sub get_filer_model_info($@) {
    my (%opts) = validate(@_, { filer => { type=>SCALAR },
				opt   => { type=>SCALAR }}
			  );
    my ($filer) = $opts{filer};
    my ($opt) = $opts{opt};
    my %params = host($filer)->param_fields();
    my $mi = new ModelInfo($params{MODEL});
	return $mi->{$opt};
}


##############################################################################
######    To make sure that the file has synced to the last block      #######
##############################################################################

=head2 dense_file_sync_last_blk(conn=>$conn, volname=>$volname, filePath=>$path
			timeout =>3600, interval => 3, max_wait=>30)
name : dense_file_sync_last_blk

Description:
    To make sure that the file has synced to the last block

Return value:
        Success : Return 1
        Fail    : Return 0

=cut



sub dense_file_sync_last_blk(%) {
    my (%opts) = validate (@_, {
		    conn => { type=>HASHREF },
		    volume => { type=>HASHREF },
		    filePath => { type=>SCALAR },
		    timeout => { type=>SCALAR, optional=>1, default=>"300" },
		    interval => { type=>SCALAR, optional=>1, default=>"5" },
		});
    
    my ($conn, $vol_obj, $file_vol_path, $max_timeout, $interval, $result);
    my ($file_mount_path, %parse_vbns, $cur_time, $wafl_blk_size, $file_size, $last_blk, $search_string);
    
    $conn = $opts{'conn'};
    $vol_obj = $opts{'volume'};
    $file_vol_path = $opts{'filePath'};
    $max_timeout = $opts{'timeout'};
    $interval = $opts{'interval'};
    
    ##Building the required Path
    $file_mount_path = $opts{'filePath'};
    #logcomment("file_mount_path 1: $file_mount_path");
    $search_string = "\/vol\/".$vol_obj->{name};
    $file_mount_path =~ s/$search_string//;
    #logcomment("file_mount_path 2: $file_mount_path");
    $file_mount_path = $vol_obj->{mountPoint}.$file_mount_path;
    #logcomment("file_mount_path 3: $file_mount_path");
    
    $cur_time = 0;
    $result = 0;
    $wafl_blk_size = 4096;
    
    logcomment("File path related to vol is = $file_vol_path and mount is = $file_mount_path");
    
   # if (!setattr_file(file=>$file_mount_path, uid=>"0", mode=>"0777")) {
   #	die("Set attr failed on $file_mount_path");
   # } else {
   #	logcomment("Set attr passed on $file_mount_path");
   #   }
    
    $file_size = get_file_size(file=>$file_mount_path);
    $last_blk = floor($file_size/$wafl_blk_size) - 1; 
    
    logcomment("File size of $file_mount_path = $file_size ; last block index= $last_blk");
    logcomment("Waiting for the file to be completly synced");
    while (($cur_time < $max_timeout) && (! $result)) {
	filer_sync($conn);
    	%parse_vbns = dense_parse_showvbns (
			conn        => $conn,
	        	file_name   => $file_vol_path,
	        	fbn_range   =>$last_blk,
		    );
        if (($parse_vbns{$last_blk}{'vbn'} == 0) ||  ($parse_vbns{$last_blk}{'vvbn'} == 0)) {
            $cur_time += $interval;
            logcomment("File hasnt synced, hence waiting for $interval sec and then retrying");
        sleep $interval;
        } else {
            $result = 1;
        }
    }
    return $result;
}

##############################################################################
####################         To dump data         ############################
##############################################################################

=head2 dump_data(conn=>$conn, device=>$device, files=>$files, options=>$options
                        level =>level, calc_time => 0, timeout=>300)
name : dump_data

Description:
    To dump data using the dump command

Return value: 
    HashRef
	Keys:
	    result -> indicates the result of the test
		Success : Return 1
		Fail    : Return 0
	    time -> gives them amount of time taken to execute the dump command
=cut

sub dump_data ($@) {
    my (%opts) = validate (@_, {
		    conn=> {type=>HASHREF},
    		    device=> {type=>SCALAR},
    		    files=> {type=>SCALAR},
    		    level=> {type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/},
    		    options => {type=>SCALAR, optional=>1}, 
    		    calc_time=> {type=>SCALAR, optional=>1, default=>0, regex=>qr/^(0|1)$/},
    		    timeout=> {type=>SCALAR, optional=>1, default=>DENSE_DEFAULT_TIMEOUT}
		});
    
    my ($conn, $level, $device, $files, $timeout, $options, $calc_time, $cmd);
    my ($returnVal, %returnV, $lTime);
    $conn = $opts{'conn'};
    $level = $opts{'level'};
    $device = $opts{'device'};
    $files = $opts{'files'};
    $timeout = $opts{'timeout'};
    $options = $opts{'options'};
    $calc_time = $opts{'calc_time'};

    $cmd = "dump $level".$options."f $device $files";
    if ($calc_time) {
	$cmd = "time ".$cmd;
    }
    
    
    $returnVal = filer_cli_test_say($conn, $cmd, $timeout);
    $returnV{'result'}  = ($returnVal =~ /DUMP IS DONE/);
    if ($calc_time) {
    	($lTime) = ($returnVal =~ /(\d+)ms/);
	$returnV{'time'} = $lTime;
    }
    
    return \%returnV;
}

##############################################################################
###########   To get max supported ref count for the release     #############
##############################################################################

=head2 get_max_supported_ref_count($DENSE_ONTAP_VERSION)
    name get_max_supported_ref_count
    Description:
        To get max supported ref count for the release passed
    Return value:
        Success : The max supported ref count for the release 
        Fail    : die
=cut

sub get_max_supported_ref_count($) {
    my ($dense_ver) = @_;
    my %ref_cnts = (
        "GB" => 255,
        "IC" => 255,
        "BR" => 255,
        "RR" => 32767,
    );

    if ( !exists $ref_cnts{$dense_ver} ) {
        logcomment ("Returning 32K refcount for DATA ONTAP VERSION SN and above");
        return 32767;
    }
    logcomment("CHECK_32K_REFCNT=$CHECK_32K_REFCNT; DENSE_ONTAP_VERSION=$dense_ver");
    if (($dense_ver eq 'RR') && (! $CHECK_32K_REFCNT)) {
        return 255;
    } else {
        return $ref_cnts{$dense_ver};
    }
}

sub tp_clear($$) {
	my ($conn,$cmd) = @_;
	my $return_text = filer_cli_test_say($conn,$cmd);
	my $ret_val ; 
	if( $return_text =~ /No testpoints enabled/){
	logcomment("tps have been cleared");
	$ret_val = 0 ;
	}else{
	logcomment("tp could not be cleared");
	$ret_val = 1 ; 
	}#not checking if the 
	#not checking if the command executed properly  since this command always returns status code 255 
	}


###########################################################################################################
## Check refcount exceeds the limit & Get the PVBN, VVBN, refcount, fbn mapping for a file/list of files ##
###########################################################################################################

=head2 validate_max_refcount($DENSE_ONTAP_VERSION)
    name validate_max_refcount
    Description:
        1. Check refcount exceeds the limit and if it exceeds show many PVBNs
        2. Get the PVBN, VVBN, refcount, fbn mapping for a file/list of files
    Return value:
        Success: 'file_path'->{result}=0 (i.e, when no block exceeds the max ref count)
        Failure: 'file_path'->{result}=(no. of blocks that exceeds he max ref count)
        
        Hash layout: 
        parsed_output = {   'file_path1'    {   ->  {result}    ->  {value}
                                                ->  {pvbn}      ->  {   {vvbn}
                                                                        {refcount}
                                                                        {fbns}
                                                                    }
                                            }
                            'file_path2'    {   ->  {result}    ->  {value}
                                                ->  {pvbn}      ->  {
                                                                        {vvbn}
                                                                        {refcount}
                                                                        {fbns}
                                                                    }
                       }
=cut

sub validate_max_refcount($@) {

    my (%opts) = validate (@_, {
                            host            =>  {type=>SCALAR},
                            file_path       =>  {type=>SCALAR | ARRAYREF},
                            max_refcount    =>  {type=>SCALAR, optional=>1, default=>255},
                            timeout         =>  {type=>SCALAR, optional=>1, default=>DENSE_DEFAULT_TIMEOUT},
                            partner         =>  {type=>SCALAR, optional=>1},
                    });
    my ($host, $file_path, $file, $max_refcount, $timeout, $partner, $file_paths); 
    my %parsed_output;

    $host = $opts{'host'};
    $file_path = $opts{'file_path'};
    $max_refcount = $opts{'max_refcount'};
    $timeout = $opts{'timeout'};
    $partner = $opts{'partner'};

    if ( ref($file_path) ) {
        $file_paths = $file_path;
        logcomment("This input type of file_path is ARRAYREF");
    } else {
        push( @{$file_paths}, $file_path );
        logcomment("This input type of file_path is SCALAR");
    }

    foreach $file (@$file_paths) {
        my ( %showvbns, @fbns, $fbn, $pvbn, $vvbn, $refcount, %exceed_count);
        %showvbns = parse_showvbns($host, $file, $partner);
        @fbns = keys %showvbns;
        foreach $fbn (@fbns) {
            $pvbn     = $showvbns{$fbn}{'vbn'};
            $vvbn     = $showvbns{$fbn}{'vvbn'};
            $refcount = $showvbns{$fbn}{'refcount'};

            $parsed_output{$file}{$pvbn}{'vvbn'}     = $vvbn;
            $parsed_output{$file}{$pvbn}{'refcount'} = $refcount;
            push( @{ $parsed_output{$file}{$pvbn}{'fbns'} }, $fbn );

            if ( $refcount > $max_refcount || $refcount <0 ) {
                $exceed_count{$pvbn} = $refcount;
                logcomment("This block pvbn = $pvbn , vvbn = $vvbn , fbn = $fbn has refcount $refcount");
            }
        }

        $parsed_output{$file}{'result'} = scalar keys %exceed_count;
        if ( $parsed_output{$file}{'result'} ) {
            logcomment('WARNING: The number of element(s) not in the range of 0 to
                    $max_refcount is/are'.
                    $parsed_output{$file}{'result'}.'\n'.Dumper(\%exceed_count));
        }
    }

    return \%parsed_output;
}
   
#################################################################
##########     To copy file with filer dd command      ##########
#################################################################

=head2 dense_filer_dd(conn=>$conn, input_file=>$input_file,
        output_file=>$output_file, blks_to_copy=>$blks_to_copy,
        verify_file=>$verify_file, timeout=>$timeout)

    Name: dense_filer_dd
    Description:
        To copy file with filer dd command
    Return value:
        Success : Nothing
        Failure : die
=cut 

sub dense_filer_dd($@) {

    my (%opts) = validate( @_,
        {
            conn         => { type => HASHREF },
            input_file   => { type => SCALAR },
            output_file  => { type => SCALAR },
            blks_to_copy => { type => SCALAR, optional => 1 },
            verify_file  => { type => SCALAR, optional => 1, default => 0 },
            timeout      => { type => SCALAR, optional => 1, default => DENSE_DEFAULT_TIMEOUT },
        }
    );

    my $cmdname = 'dd';
    my $conn = $opts{'conn'};
    my $cmd  = "$cmdname if=" . $opts{'input_file'} . " of=" . $opts{'output_file'};

    if ( $opts{'blks_to_copy'} ) {
        $cmd = $cmd . " count=" . $opts{'blks_to_copy'};
    }

    my ( $text, $result ) = filer_cli_test_say( $conn, $cmd, $opts{'timeout'} );

    my @info = filer_cli_split($text);
    foreach my $line (@info) {
        if ( $line =~ /$cmdname:/ ) {
            die("Unexpected error from $cmd: $line");
        }
    }

    if ( $opts{'verify_file'} ) {
        if ( !filer_file_exists( $conn, $opts{'output_file'} ) ) {
            die( 'Failed to verify the required file at: ' . $opts{'output_file'} );
        }
    }

    return;
}

######################################################
###########   get or set bootargs values   ###########
######################################################
=head2 bootargs                                                                                                                                                                                       
                                                                                                                                                                                                               
get or set bootargs values
                                                                                                                                                                                                               
Inputs:                                                                                                                                                                                                        
    * conn - filer connection
    * name - bootargs parameter name
    * value - bootargs parameter value 
                (optional, if provided will set the value else will get the value);
Return value:
    * value got (after setting in case of set)

=cut

sub bootargs ($@) {
    my (%opts) = validate(@_,
        {
            conn  => { type => HASHREF },
            name  => { type => SCALAR },
            value => { type => SCALAR, optional => 1, default => "" },
        }
    );

    my ( $step, $conn, $name, $value, $cmd, $cli_ret, $val );
    $conn  = $opts{'conn'};
    $name  = $opts{'name'};
    $value = remove_leading_trailing_spaces( $opts{'value'} );

    if ($value) {
        $cmd  = "bootargs set $name $value";
        $step = "Setting bootargs $name to $value";
    } else {
        $cmd  = "bootargs get $name";
        $step = "Getting bootargs $name";
    }

    logcomment($step);

    $cli_ret = filer_cli_test_say( $conn, $cmd );
    logcomment("Return text : $cli_ret");
    $val = remove_leading_trailing_spaces( ( split( /==/, $cli_ret ) )[-1] );
    logcomment("Got => $name = $val");

    return $val;
}


sub tp_delay($$) {
	my ($conn,$cmd) = @_;
	my $return_text = filer_cli_test_say($conn,$cmd);
	my $ret_val ; 
	if( $return_text =~ /tps enabled/){
	logcomment("tp delay has been introduced");
	$ret_val = 0 ;
	}else{
	logcomment("tp not set");
	$ret_val = 1 ; 
	}#not checking if the command executed properly  since this command always returns status code 255	
}	

#----------------------------------------------------------------------
# The testme routine to test the exported functions of this library
#
# ntest PKGNAME=Dense::Utils SUBNAME=testme util/test_pkg.thpl
#
# The routine can also be invoked with many optional parameters, such as:
#
# ntest PKGNAME=Dense::Utils SUBNAME=testme FUNCTION_NAME=dense_new_filer util/test_pkg.thpl
#
#----------------------------------------------------------------------

sub testme() {

        package main;

        our ($FILER, $FUNCTION_NAME, $FILER_VOL, $FILER_CONN, $CLIENT, $PARTNER);

        param('FILER',       '-mesg',    "Filer to test");
        param('FILER_CONN','-default',"#rsh");
        param('FUNCTION_NAME',   '-default',    "all");
        param('FILER_VOL',   '-default', 'vol0');
        param('CLIENT',   '-default', '');
        param('PARTNER',   '-default', '0');

        package Dense::Utils;

        logcomment("Testing on FILER $FILER, Function $FUNCTION_NAME");
        my $ret;
        my $path_name = "/vol/".$FILER_VOL;

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_new_filer"){

                eval {
                        my $filer_obj = dense_new_filer(host=>$FILER, nfsAcls=>1, noLicense=>0);
			# logcomment("Filer hostname is $filer_obj->{hostName}");
			filer_cli_say($filer_obj->{conn}, "version -v");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_new_vol"){

		my $filer_obj = dense_new_filer(host=>$FILER, nfsAcls=>1, noLicense=>0);

		# Create a Tradvol
                eval {
			logcomment("Creating Trad Vol...");
			my $ret = dense_new_vol(filer=>$filer_obj , tradVol=>1);
			filer_cli_say($filer_obj->{conn}, "vol status");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

		# Create a flexvol
                eval {
			logcomment("Creating Flex Vol...");
                        my $ret = dense_new_vol(filer=>$filer_obj, volName=>"Rani_flex", volSize=>"20m", purge=>1, aggr=>"autotest", client=>$CLIENT);
			filer_cli_say($filer_obj->{conn}, "vol status");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        print Dumper($ret);
                        logresult('pass', "Got expected result");
                }

        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_setup_test_vvol"){

                eval {
                        my $filer_obj = dense_new_filer(host=>$FILER, nfsAcls=>1, noLicense=>0);
			my $ret = dense_setup_test_vvol(filer=>$filer_obj , volName=>"Rani_Test", volSize=>"20m", purge=>1, aggr=>"autotest", aggrPurge=>0);
                        filer_cli_say($filer_obj->{conn}, "snap sched Rani_Test");
                        filer_cli_say($filer_obj->{conn}, "sis status");
                        filer_cli_say($filer_obj->{conn}, "sis config");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_setup_all"){

                eval {
			my ($vol, @fs) = dense_setup_all(host=>$FILER, nfsAcls=>1, volName=>"Rani_Test", purge=>1, aggr=>"autotest", aggrPurge=>0);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_destroy_vol"){

                eval {
			my $filer_obj = dense_new_filer(host=>$FILER, nfsAcls=>1, noLicense=>0);
			my $vol = dense_new_vol(filer=>$filer_obj , tradVol=>0);	
			my $ret = dense_destroy_vol(vol=>$vol);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_cleanup_vol"){

                eval {
			my $ret = dense_cleanup_vol(host=>$FILER, path=>$path_name);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
			print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

        if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_failure"){

                eval {
                        my $ret = dense_verify_failure(returned_error=>"", expected_error=>"None");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }
	
	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "wait_for_scan_done"){

                eval {
                        my $ret = wait_for_scan_done(host=>$FILER, volname=>$FILER_VOL, scan=>"wafliron");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "verify_scan"){

                eval {
                        my $ret = verify_scan(host=>$FILER, volname=>$FILER_VOL, scan=>"active bitmap rearrangement");
		 };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_get_file_size_and_blkcnt"){

                eval {
			my $filer_obj = dense_new_filer(host=>$FILER, nfsAcls=>1, noLicense=>0);
			my $vol = dense_new_vol(filer=>$filer_obj , volName=>"vol1", purge=>0, aggr=>"autotest", aggrPurge=>0);	
                        my ($size, $blk_cnt) = dense_get_file_size_and_blkcnt(vol=>$vol, filename=>"fileA");
			logcomment("File size is $size and Block count is $blk_cnt");
		};

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "dense_verify_file_size_and_blkcnt"){

                eval {
			my $filer_obj = dense_new_filer(host=>$FILER, nfsAcls=>1, noLicense=>0);
                        my $vol = dense_new_vol(filer=>$filer_obj , volName=>"vol1", purge=>0, aggr=>"autotest", aggrPurge=>0);  
			my $ret = dense_verify_file_size_and_blkcnt(vol=>$vol, filename=>"fileA", exp_size=>"17194", exp_blk_cnt=>"5");
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "get_snapshot_with_time"){
        
                eval {
	 		my $ret = get_snapshot_with_time(host=>$FILER, volname=>$FILER_VOL, snapname=>"hourly.0");
			logcomment("The snapshot is $ret");
		};

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        logresult('pass', "Got expected result");
                }
        }

	if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "set_metafile_visible"){

                eval {
                        my $ret = set_metafile_visible(host=>$FILER, partner=>$PARTNER);
                };

                if ($@ =~ /Connection timed out/) {
                        logresult('pass', "Expected fail. Check filer and connection type");
                } elsif ($@) {
                        logresult('fail', "$@");
                } else {
                        print Dumper($ret);
                        logresult('pass', "Got expected result");
                }
        }


} ## End of Testme Routine

=head1 SEE ALSO

    Where additional information can be found.

=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 AUTHOR

    Rani Palanisami (rani@netapp.com)
    $Id$
    Last revised $Date$

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;
