# -*- perl -*-
# $Id$

## @summary
## @author kasasrik@netapp.com
## @status draft
## @pod here



package Dense::cmode::Utils;
require Exporter;
our @ISA= qw(Exporter);
our @EXPORT = qw (
	dense_volume_exists
	dense_volume_new
	dense_volume_mount
	dense_volume_umount
	dense_volume_delete
	dense_set_volume_state
	dense_volume_rename
	
	dense_aggregate_exists	
	dense_aggregate_new
	dense_aggregate_rename
	dense_aggregate_delete
	dense_set_aggregate_state
	
	dense_vserver_exists	
	dense_vserver_new
	dense_vserver_delete
	
	dense_cluster_exists
	dense_cluster_new
	dense_cluster_license

	setup_basic
	cleanup_basic
	
        dense_execute_node_run
	
);



use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use Server;
use GNOTE;
use Dense::Constants;
use Tharn;
use VifMgr::VifConfig;
use Data::Dumper;
use TracePkg;
use constant CLUSTER_LICENSE    => "NWUZZJPJYBFDAA";
use constant DFLT_VSERVER_NAME 	=> "dense_vs";
use constant DFLT_LICENSE => "NFS";
use constant DFLT_CLUSTER_NAME => "dense_cluster";
use constant SIS_LICENSE_7MODE => "OSFYRVH";
use constant TRUE	=> 1;
use constant FALSE	=> 0;
use constant CLIENTMATCH => "0.0.0.0/0";

my (@Nodes);
my $trace = new TracePkg;



##
##
## Basic Setup Utility for generic purpose 
##
##

sub setup_basic
{
     my (%opts) = validate( @_,
     			{
                            Node     => {type => SCALAR|HASHREF,
                                         },
                            address   => {type => SCALAR|HASHREF,
                                         },
                            vserver   => {type => SCALAR|HASHREF,
                                          optional => 1,
                                          default => DFLT_VSERVER_NAME,
                                         },
                            volume    => {type => SCALAR|HASHREF,
                                          optional => 1,
                                          default => DFLT_VOLNAME,
                                         },
                            aggregate => {type => SCALAR|HASHREF,
                                          optional => 1,
                                          default => DFLT_AGGRNAME,
                                         },
                            debug    =>  {type => SCALAR,
                                          optional => 1,
                                          default => 0,
                                         },
                        } );

    
    NOTE::logcomment("seutp_basic is called with params : %s", Dumper(%opts) ) if($opts{debug});	
    my(%cl,%vsopts,$vs,%volopts,$vol); 
    
    eval {
	
        dense_cluster_new(Node=>$opts{Node});

	## Create vserver with specified vserver name and ip address
	%vsopts = %opts;
	delete $vsopts{volume};
        $vs = dense_vserver_new(%vsopts);

	##Setup Dense License on Node - for MS4
	_dense_setup_node($vs);
        
	##Set the reuse to 0
        $opts{reuse}=0;
        
	## Create the Volume with specified Vserver Name 
        %volopts = %opts;
	delete $volopts{address};
	$volopts{vserver} = $vs->{vserver};
	$vol = dense_volume_new(%volopts);

	## Mount the Volume created for dataset creation
	dense_volume_mount($vol);

	#Fill in the hash
	$vol->{server} = $vs->{server};

    };
    if($@) {
        die"Inside setup_basic : Failed to create the setup : $@ ";
    } else {
        NOTE::logcomment("Inside setup_basic : Created Setup Successfully :%s",Dumper(%vol));
    }
    return $vol;
    
    
}


##
## Cleanup utility based on the setup_basic 
##

sub cleanup_basic
{
    my ($opts) =  @_;
    my (%hash);
    $opts->{cleanup} = "volume" unless $opts->{cleanup};
    my $type = $opts->{cleanup};

    NOTE::logcomment("Inside cleanup_basic: Requested Cleanup type \[Suppored : volume\|vserver\] : %s",$type);
    eval { 
	    dense_volume_umount($opts);	
            dense_volume_delete($opts);
	    NOTE::logcomment("Inside cleanup_basic: Successfully deleted the Volume");	
            $opts->{volume_cleanup}=1;
    };
    if($@) {
        $opts->{volume_cleanup}=0;
	$opts->{volume_output}=$@;
        die"Inside cleanup_basic: Failed to cleanup the volumes : $@";
    }
    if($type eq "vserver" || $type eq "all")
    {
        eval { 
            dense_vserver_delete($opts);
            $opts->{vserver_cleanup}=1;
	    NOTE::logcomment("Inside cleanup_basic: Successfully deleted the Vserver");	
        };
	if($@) {
        	$opts->{vserver_cleanup}=0;
		$opts->{vserver_output}=$@;
        	die"Inside cleanup_basic: Failed to cleanup the vserver : $@";
	}
    }
}


##
##  Private Function - Just for MS4 
##  7Mode asis license needs to be added for C-Mode sis commands to work
##
sub _dense_setup_node
{
	my ($opts) = @_;

        my ($out,$server,$cmd);
        $opts->{server} = new Server(nodes => $opts->{Node}) unless $opts->{server};
        $server = $opts->{server};
	$cmd = "license add ".SIS_LICENSE_7MODE;

        $out = $server->system_node_run(node => $opts->{Node},
					 command => $cmd,
					);
}

## End of _dense_setup_node



##
## Wrapper to execute node run
##

sub dense_execute_node_run
{
    my ($opts) = @_;

    die "Expecting the \'command\' parameter" unless ($opts->{command}); 

    $opts->{server} = new Server(nodes=>$opts->{Node}) unless ($opts->{server});
    return $opts->{server}->system_node_run(
                                            node => $opts->{Node},
                                            command => $opts->{command},
                                           );
}












################################################
######### VOLUME Related Utilities #############



##
## Verifies whether volume exists and returns TRUE(1) or FALSE(0)
## 
##

sub dense_volume_exists
{
    my ($opts) = @_;	

    logcomment("Utils::dense_set_volume_state called with :" . Dumper($opts)) if($opts->{debug});
    _dense_volume_attach($opts);
    return $opts->{volume_obj}->exists if(exists $opts->{volume_obj});
    return FALSE;
}


##
## Creates a Volume 
## Filer is the Required paramter
## All Others are optional
##

sub dense_volume_new 
{

    my (%opts) = validate( @_,
                {
                        Node     => {type => SCALAR|HASHREF,
                                    },
                        volume     => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_VOLNAME,
                                    },
                        size     => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_VOLSIZE,
                                    },
                        vserver     => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_VSERVER_NAME,
                                    },
                        aggregate     => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_AGGRNAME,
                                    },
                        reuse => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => 1,
                                    },
                        debug    => {type => SCALAR,
                                    optional => 1,
                                    default => 0,
                                    },
                } );

    
    logcomment("Utils::dense_volume_new called with :" . Dumper(%opts)) if($opts{debug});

    NOTE::logcomment("Trying to create the volume : %s",$opts{volume} ) if($opts{debug}) ;

    dense_volume_delete(\%opts) unless ($opts{reuse});

    my $aggr = dense_aggregate_new(Node=>$opts{Node},aggregate=>$opts{aggregate});
    $opts{aggregate_obj} = $aggr->{aggregate_obj};
    $opts{aggregate} = $aggr->{aggregate};

    $opts{volume_obj} = NOTE::Volume->create(
			Vserver => $opts{vserver},
			name => $opts{volume},
			Node => $opts{Node},
			size => $opts{size},
			Aggregate => $aggr->{aggregate_obj},
			);

    ## TBD : Try to setup the volume based on the parameters 
    ## 
    ##
    
    return \%opts;	
}


##
## Sets the volume state to the desired ( offline/restrict/online)
##

sub dense_set_volume_state
{
    my($opts) = @_;

    logcomment("Utils::dense_set_volume_state called with :" . Dumper($opts)) if($opts->{debug});
	
    die "volume_state is not set which is required parameter for dense_set_volume_state" if(! $opts->{volume_state} );

    if($opts->{volume_obj})
    {
            NOTE::logcomment("Trying to set the volume state to : %s ",$opts->{volume_state});

	    if($opts->{volume_state} eq "online") {	
	    	return $opts->{volume_obj}->online;
	    }
	    if($opts->{volume_state} eq "offline") {	
	    	return $opts->{volume_obj}->offline;
	    }
	    if($opts->{volume_state} eq "restrict") {
		#return $opts->{volume_obj}->restrict;
		## NOTE restrict is not working so using server as a work around

		$opts->{server} = new Server(nodes=>$opts->{server}) unless ($opts->{server});
		$opts->{server}->volume_unmount(vserver => $opts->{vserver}, volume => $opts->{volume});
		return $opts->{server}->volume_modify(vserver => $opts->{vserver}, volume => $opts->{volume} , state => "restricted");
	    }
	    die "volume_state passed is Unknown, Accepted values are [online|offline|restrict]\n";
		
	} else {
		die "Looks like script didn't use dense_volume_new to create the volume, Expecting a NOTE Volume Object";
	}
}



##
##
## Delete the volume 
##
##


sub dense_volume_delete
{
    my($opts) = @_;
    
    logcomment("Utils::dense_volume_delete called with :" . Dumper($opts)) if($opts->{debug});

    	
    ## Verify whether NOTE Volume object exists, if not attach the volume to NOTE Object		
    _dense_volume_attach($opts);	
    ## Destroy the volume and return the value
    return $opts->{volume_obj}->destroy() if(exists $opts->{volume_obj});	

	
}


##
##
## Mount the volume on the given client, defaults to localhost
##
##


sub dense_volume_mount
{
    my($opts) = @_;
    
    #Local Variables
    my($vol,$mount_point);

    logcomment("Utils::dense_volume_mount called with :" . Dumper($opts)) if($opts->{debug});
    
    #Set the client as local host if not specified in the params
    $opts->{client} = "localhost" unless $opts->{client};

    if(exists $opts->{volume_obj}){
        $vol = $opts->{volume_obj};

        ## Need to fix this once NOTE works properly
        ## Get the vserver lif ip and sent it to NOTE volume mount call
        ## this should ideally be done by NOTE but its not working right now

        #my $vs = $vol->Vserver;
	#my $ip = $vs->lif_address();
	#NOTE::logcomment("Inside dense_volume_mount : Value of vserver ip : $ip : $vs");
        #$vol->mount(client=> $opts->{client} ,RW=>1, ip => $ip);
        $vol->mount(client=> $opts->{client} ,RW=>1);

        $mount_point = $vol->mountpoint();
        NOTE::logcomment(" Volume :%s is mounted under : %s",$vol->name, $mount_point);
        $mount_point =~ s/\/\//\//g;
        NOTE::logcomment(" Mount path  : %s", $mount_point);
        $opts->{mountPoint} = $mount_point;
    } else {
        die "Looks like script didn't use dense_volume_new to create the volume, Expecting a NOTE Volume Object";	
    } # end of if-else	
}

##
## unmount the volume from the given client, defaults to localhost
##


sub dense_volume_umount
{
    my($opts) = @_;

    logcomment("Utils::dense_volume_umount called with :" . Dumper($opts)) if($opts->{debug});

    die "Looks like script didn't use dense_volume_new to create the volume, Expecting a NOTE Volume Object" unless($opts->{volume_obj});

    #Verify whether client parameter is passed and unmount the volume from client
    if(exists $opts->{client}) {
        return $opts->{volume_obj}->umount(client=> $opts->{client});
    } else {
        die "Looks like client param is not set, script didn't use dense_volume_mount to mount the volume";
    }
}


##
## private function to attach the volume name to NOTE Object 
##
sub _dense_volume_attach
{
    my ($opts) = @_;

    logcomment("Utils::_dense_volume_attach called with :" . Dumper($opts)) if($opts->{debug});

    ## Verify whether NOTE Volume object exists, if not attach the volume to NOTE Object
    if(! opts->{volume_obj}) {
        eval { $opts->{volume_obj} = NOTE::Volume->attach(
                        Node => $opts->{Node},
                        name => $opts->{volume},
                        Vserver => $opts->{vserver},
                        );
        };
    }

}

###################################################
######### AGGREGATE Related Utilities #############


##
## Creates Aggregate 
## Filer is the Required paramter
## All Others are optional
##

sub dense_aggregate_new
{
    my (%opts) = validate( @_,
                {
                        Node    => {type => SCALAR|HASHREF,
                                    },
                        aggregate     => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_AGGRNAME,
                                    },
                        raidtype => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => "raid_dp",
                                    },
                        disks    => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => 3,
                                    },
                        nodes    => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    },
                        reuse => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => 1,
                                    },
                        debug    => {type => SCALAR,
                                    optional => 1,
                                    default => 0,
                                    },
                } );
    my ($aggr,$server);
    logcomment("Utils::dense_aggregate_new called with :" . Dumper(%opts)) if($opts{debug});
    
    if($opts{nodes}) {
        $server = new Server(nodes=>$opts{nodes});
        $opts{server}=$server;
    }  
 
    dense_aggregate_delete(\%opts) unless ($opts{reuse});
    $opts{aggregate_obj} = NOTE::Aggregate->create(
                                name => $opts{aggregate},
                                Node => $opts{Node},
                                );
    return \%opts;

}

##
## Verified whether aggregate exists 
## Returns TRUE(1) if exists else FALSE(0)
##

sub dense_aggregate_exists
{
    my ($opts) = @_;
	
    logcomment("Utils::dense_aggregate_exists called with :" . Dumper($opts)) if($opts->{debug});
    _dense_aggregate_attach($opts);
    return $opts->{aggregate_obj}->exists if($opts->{aggregate_obj});
    return FALSE;
}

##
## set the aggregate state to the desired state (offline/restrict/online)
##
##

sub dense_set_aggregate_state
{
    my($opts) = @_;
	
    logcomment("Utils::dense_set_aggregate_state called with :" . Dumper($opts)) if($opts->{debug});
    die "aggregate_state is not set which is required parameter for dense_set_aggregate_state" if(! $opts{aggregate_state} );

    if(exists $opts->{aggregate_obj}){
        if($opts->{aggregate_state} eq "online") {	
            return $opts->{aggregate_obj}->online;
	}
	if($opts->{aggregate_state} eq "offline") {	
	    return $opts->{aggregate_obj}->offline;
	}
	if($opts->{aggregate_state} eq "restrict") {	
	    return $opts->{volume_obj}->restrict;
	}
	die "aggregate_state passed is Unknown, Accepted values are [online|offline|restrict]\n";
    } else {
	die "Looks like script didn't use dense_aggregate_new to create the volume, Expecting a NOTE Volume Object";
    }
}


##
## Deletes the aggregate
##

sub dense_aggregate_delete
{
    my ($opts) = @_;
	
    logcomment("Utils::dense_set_aggregate_delete called with :" . Dumper($opts)) if($opts->{debug});

    _dense_aggregate_attach($opts);
    return $opts->{aggregate_obj}->destroy() if(exists $opts->{aggregate_obj}); 
}

##
## Private function for attach the given aggregate to a NOTE Object
##

sub _dense_aggregate_attach
{

    my ($opts) = @_;

    logcomment("Utils::_dense_aggregate_attach called with :" . Dumper($opts)) if($opts->{debug});

    ## Verify whether NOTE Volume object exists, if not attach the volume to NOTE Object
    if(! opts->{aggregate_obj}) {
        eval { $opts->{aggregate_obj} = NOTE::Aggregate->attach(
                        Node => $opts->{Node},
                        name => $opts->{aggregate},
                        );
        };
    }
}


###################################################
########### VSERVER Related Utilities #############


##
## Creates vserver based on nate_hosts files PARAMS
##	Required Parameters
## 			: Filer 
## 			: Vserver Name 
##			: Vserver IP Address 
## 	All Others are optional
##
##
sub dense_vserver_new 
{
    my (%opts) = validate( @_,
                {
                        Node    => {type => SCALAR|HASHREF,
                                    },
                        address    => {type => SCALAR|HASHREF,
                                    },
                        vserver     => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_VSERVER_NAME,
                                    },
                        aggregate => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_AGGRNAME,
                                    },
                        reuse => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => 1,
                                    },
                        debug    => {type => SCALAR,
                                    optional => 1,
                                    default => 0,
                                    },
                } );


    #Local Variables
    my($server,$aggr,$rootvol);

    logcomment("Utils::dense_vserver_new called with :" . Dumper(%opts)) if($opts{debug});

    $server = new Server(nodes => $opts{Node});
    $opts{server} = $server;
    if(dense_vserver_exists(\%opts) && $opts{reuse}) 
    {
	NOTE::logcomment("Vserver exists and so reusing");
    } else {
	
        $aggr = dense_aggregate_new(Node => $opts{Node}, aggregate => $opts{aggregate} );
        $opts{aggregate_obj} = $aggr->{aggregate_obj};
	
        $rootvol = "root_".$opts{vserver};
        #$opts{vserver} = $opts{Node}."_".$opts{vserver};
        eval {
            $server->vserver_new(
           		vserver => $opts{vserver},
                	rootvolume => $rootvol,
                	aggregate => $opts{aggregate},
	        	"ns-switch" => "file",
	        	language => "en_US",
	        	"rootvolume-security-style" => "mixed",
        		);
        };
        if($@) {
            NOTE::logcomment("Failed to create vserver");
        }

        ## Create the data lifs
        @Nodes = $server->servers();
        _configure_interfaces(interface_type => "DATA", onserver => $opts{vserver}, ports => ["e0c", "e0d"], Node => $opts{Node} ,start=> 1, address=> $opts{address});

        NOTE::logcomment("Completed vserver creation and configuring the data lifs");
        NOTE::logcomment("Setup the NFS License and setup for Mount ");
	
        eval { 
            dense_cluster_license(Node=>$opts{Node}, license => 'NFS');
        }; 
        if($@) { 
            dense_vserver_delete(\%opts);
        }

        ##Setup the Export Rule 
        eval {
             $server->protocol_exportruleset_rule_new(
         			vserver => $opts{vserver},
         			rulesetname => "default",
            			clientmatch => CLIENTMATCH,
            			rorule => "any",
            			rwrule => "any",
            			superuser => "any",
            			"allow-suid" => "true",
            			"allow-dev" => "true",
        			);
        };
        if($@) { 
            dense_vserver_delete(\%opts);
        }
	

        ## Setup NFS 
        eval { 
            $server->protocols_nfs_server_new(
        			vserver => $opts{vserver},
	        		access => "true",
	        		"rpcsec-ctx-high" => 30,
	        		"rpcsec-ctx-idle" => 30,
	        		"v3-enable" => "true",
	        		"udp-enable" => "true",
	        		"tcp-enable" => "true",
	        		spinauth => "true",
	    			);
        };
        if($@) { 
            dense_vserver_delete(\%opts);
        }

        ##
        NOTE::logcomment("Completed vserver setup");
	
    } #end else 
	
    return \%opts;
}


##
##  Deletes the Vserver
##	Required Parameters
##			: Filer
##			: Vserver Name
##


sub dense_vserver_delete
{
    my ($opts) = @_;

    #Local Variables
    my($output,$server);

    logcomment("Utils::dense_vserver_delete called with :" . Dumper($opts)) if($opts->{debug});
    
    if(dense_vserver_exists($opts)){
        $output;	
        NOTE::logcomment("Starting to delete the vserver: %s",$opts{vserver});
	$opts->{server} = new Server(nodes => $opts->{Node}) unless (defined $opts->{server});

	$server = $opts->{server};
	_delete_interfaces(interface_type => "DATA", onserver => $opts->{vserver}, ports => ["e0c", "e0d"], Node => $opts->{Node} , start=> 1);
	eval {
	    $output = $server->volume_show( vserver => $opts->{vserver});
	    foreach my $each (@$output){
	        if(exists $each->{volume}) {
		    $server->volume_delete ( 
                    			volume => $each->{volume},
					vserver => $opts->{vserver},
					);
		} #end of if

	    } 	# end of foreach

	};
	eval {
	    $server->vserver_delete(
	    			vserver => $opts->{vserver},
				);
	};
	die "Failed to delete vserver : $@" if($@);

    } else {
        NOTE::logcomment("Vserver Doesn't exist so returning from dense_vserver_delete");
    } # end of if-else

    return TRUE;
	
}



##
## verifies whether vserver exists
## returns TRUE(1) if exists else FALSE(0)
##

sub dense_vserver_exists
{
    my ($opts) = @_;
    
    #Local Variables
    my ($out,$server);

    logcomment("Utils::dense_vserver_exists called with :" . Dumper($opts)) if($opts->{debug});

    $opts->{server} = new Server(nodes => $opts->{Node}) unless $opts->{server};	
    $server = $opts->{server};
    eval {
        $out = $server->vserver_show(vserver => $opts->{vserver});
    };
    foreach my $each (@$out) {
        return TRUE if(exists $each->{vserver});
    }
    return FALSE;
}



###################################################
########### Filer Related Utilities ###############




###################################################
########### CLUSTER Related Utilities #############






##
##	Create Cluster with the given Name. All the Cluster related information will be retrieved from nate_hosts file
##	Required Parameters : 
##				Node or Filer
##	Optional Parameters :
##				Cluster Name
##				
##

sub dense_cluster_new
{
    my (%opts) = validate( @_,
                	{
                        Node    => {type => SCALAR|HASHREF,
                                    },
                        cluster => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_CLUSTER_NAME,
                                    },
                        license => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => DFLT_LICENSE,
                                   }, 
                        reuse => {type => SCALAR|HASHREF,
                                    optional => 1,
                                    default => 1,
                                   }, 
                        debug    => {type => SCALAR,
                                    optional => 1,
                                    default => 0,
                                    },
                              	
			 } );
	my ($ref) = @_; 

	my @create_join_args;
	
	##
	NOTE::logcomment("Trying to check for existing cluster with name : %s",$opts{cluster});
	my $hash = dense_cluster_exists(%opts);
	if(exists $hash->{cluster_obj} ) {
		NOTE::logcomment("Cluster exists so reusing it :%s ",$hash{cluster});
		$opts{cluster} = $hash->{cluster};
	} else {
		my $params = host($opts{Node});

	
		my $server = new Server(nodes => $opts{Node});

		@create_join_args = _clus_create_join_params(Node => $opts{Node}, interface_type => "CLUSTER", ports => ["e0a", "e0b"]);

        	my  %mgmt_values = (
                'mgmt-port'    => $params->{CLUSTER_MGMT_PORT},
                'mgmt-ip'      => $params->{CLUSTER_MGMT_IP},
                'mgmt-netmask'  => $params->{MGMT_MASK},
                'mgmt-gateway' => $params->{MGMT_GATEWAY},
            	);

	  	eval {
        	    $retval = $server->cluster_create(
                	-Timeout => 300,
	                clustername => $opts{cluster},
			license => CLUSTER_LICENSE,
                	%mgmt_values,
	                @create_join_args,
        	    );
        	};
        	if($@) {
			NOTE::logcomment("Failed to create the Cluster :%s", $@);
			return undef;
		} else {	
        		NOTE::logcomment("Created cluster Successfully");
        		NOTE::logcomment("Trying to setup the license :%s ", $opts{license});
			my %hash = dense_cluster_license(%opts);
			$opts{cluster_obj} = $hash{cluster_obj};
		}
	}
	return \%opts;
	
	
}


##
## Filer Name
## Note Cluster Object 
## Reuse the existing cluster  


sub dense_cluster_exists
{
	my (%opts) = @_;

	my ($cluster);
	if($opts{cluster_obj}) {
		$cluster = $opts{cluster_obj};	
	} else {
	    eval { 	
		$cluster = NOTE::GX::Cluster->connect( Node => $opts{Node} );
	    };
	    return FALSE if($@);
	}
	if($cluster->name)
	{
		$opts{cluster_obj} = $cluster;
		NOTE::logcomment("Cluster exists and the Cluster name is : %s",$cluster->name);
		if($cluster->name ne $opts{cluster}){

			NOTE::logcomment("Cluster name is different than the expected cluster name 
				 \n Expected Cluster Name : %s 
				\n  Existing Cluster Name on Filer : %s",$opts{cluster}, $cluster->name);			
			 die"Cluster exists but with different name" if(! $opts{reuse});
		}
		$opts{cluster} = $cluster->name;
		return \%opts;
	}
	
}


##
## Setups up the license on the Cluster 
## Verifies whether Cluster exists if yes, will try to setup the license 
## uses GNOTE::Cluster object to Setup the license
## Supported License options : Base, Mirror, FlexVol_HPO, CIFS, SnapRestore, NFS, SnapRestore, SnapMirror_DP, SnapMirror_LS, Striped_Volume

sub dense_cluster_license
{
	my (%opts) = @_; 
	
	my ($cluster,%hash);
	$opts{reuse} = 1;
	if(exists $opts{cluster_obj}) {
		$cluster = $opts{cluster_obj};
	} else {
		$hash = dense_cluster_exists(%opts);
		if(exists $hash->{cluster_obj})
		{
			$cluster = $hash->{cluster_obj};
		} else {
			NOTE::logcomment("Cluster doesn't exist");
			die "Cluster doesn't exist, try to setup the cluster and rerun\n";
		}
	}
	$opts{cluster_obj} = $hash->{cluster_obj} unless $opts{cluster_obj};
	$cluster->license($opts{license},1);
	#return %opts;
}



####
####
#### PRIVATE SUBROUTINES WHICH WILL NOT BE EXPOSED TO USER
####
####


sub _portsort {
    $a =~ /(\d+)$/;
    my $an = $1;
    $b =~ /(\d+)$/;
    my $bn = $1;

    $an <=> $bn;
}


##
## For configuring data lifs 
##
sub _vifinfo {
    # This sub makes an array of hashes containing vif information for a particular interface type for a single Node.
    my %params = (@_);
    my @required_keys = qw(name port address netmask gateway mtu);
    my @retval;
    my %options;
    my $option_ref = \%options;
    my $Node = $params{Node};
    $option_ref->{$Node} = host($Node);	

    my @keys = grep(/^$params{interface_type}_(IP|CONFIG)\d+$/, keys %{$option_ref->{$Node}});
    # @keys should now have everything that we need to operate over, but do them in order
    foreach my $key (sort _portsort @keys){
        $key =~ /(\d+)$/;
        my $instance = $1;

        my %data = (
            mtu => 1500,
        );
        if ($params{interface_type} =~ /cluster/i){
            $data{name} = "clus$instance",
        } else {
            #$data{name} = $params{Node} . "_".(lc $params{interface_type})."$instance",
	     $data{name} = $params{onserver},
        }
        if ($key =~ /IP/){
            # Old format where everything is in individual keys
            $data{port} = $option_ref->{$params{Node}}->{$params{interface_type}."_PORT$instance"};
            $data{address} = $option_ref->{$params{Node}}->{$params{interface_type}."_IP$instance"};
            $data{netmask} = $option_ref->{$params{Node}}->{$params{interface_type}."_MASK$instance"};
            $data{gateway} = $option_ref->{$params{Node}}->{$params{interface_type}."_GATEWAY$instance"};
            if (exists($option_ref->{$params{Node}}->{$params{interface_type}."_MTU$instance"})
                && defined($option_ref->{$params{Node}}->{$params{interface_type}."_MTU$instance"})){
                $data{mtu} = $option_ref->{$params{Node}}->{$params{interface_type}."_MTU$instance"};
            }
        } elsif ($key =~ /CONFIG/){
            # New format where the config is more like what is used in CFE
            foreach my $pair (split(/\s+/,$option_ref->{$params{Node}}->{$key})){
                my ($k, $v) = split(/=/, $pair);
                $data{$k} = $v;
            }
        } # No else since we matched the keys before

        NOTE::logcomment(" Data gleaned from config is %s", Dumper(\%data));
        my @missing;
        foreach my $required_key (@required_keys){
            if (!exists($data{$required_key}) || !defined($data{$required_key}) || !length($data{$required_key})){
                if (($required_key =~ /port/) && exists($params{ports})){
                    $data{$required_key} = $params{ports}->[$instance % @{$params{ports}}];
                } else {
                    push @missing, $required_key;
                }
            }
        }
        if (@missing){
            NOTE::logcomment( "Missing required keys for %s : %s ", @missing, Dumper(\%params));
        }

        push (@retval, \%data);
    }
    @retval
}


##
## For configuring the Interfaces for Vserver and cluster 
##

sub _configure_interfaces 
{
	my %params = @_;

	my $Node = $params{Node};
	#foreach my $Node (@Nodes) {
        #	next if (defined($params{Node}) && ($Node ne $params{Node}));
	#        $server->set_execution_server(-Server => $Node) if defined($server);
        	my $servername;
	        if (defined($params{onserver})){
        	    $servername = $params{onserver};
	        } else {
        	    $servername = $Node;
        	    #       $servername =~ s/-/_/g;
        	}
        	my $interface_number = 0;
	        # Now some data interfaces
		
		my $vif_config = new VifMgr::VifConfig();
	        my @if_info = _vifinfo(%params, Node => $Node);


	        foreach my $ref (@if_info){
        		$interface_number++;
            		next if ($params{start} && ($interface_number <= $params{start}));
            		my %data = %$ref;
		
			if($params{interface_type} eq "DATA")
			{$data{address} = $params{address};}
			

	
		eval {
			$vif_config->create_vif(
                        server      => $servername,
                        vif         => $data{name}.$interface_number,
                        role        => (lc $params{interface_type}),
                        home_node   => $Node,
                        home_port   => $data{port},
                        address     => $data{address},
                        if_netmask  => $data{netmask},
                        server_args => \@server_add_args,
                        gateway     => $data{gateway},
			);
		};	
		}
	#}
}

##
## For Deleting the Interfaces which were creating using vserver creation
##

sub _delete_interfaces
{
	my (%params) = @_;
	my $Node = $params{Node};

	NOTE::logcomment("Entering into delete_interfaces routine");

                my $servername;
                if (defined($params{onserver})){
                    $servername = $params{onserver};
                } else {
                    $servername = $Node;
                    #       $servername =~ s/-/_/g;
                }
                my $interface_number = 0;
                # Now some data interfaces

                my $vif_config = new VifMgr::VifConfig();
                my @if_info = _vifinfo(%params, Node => $Node);

                foreach my $ref (@if_info){
                        $interface_number++;
                        next if ($params{start} && ($interface_number <= $params{start}));
                        my %data = %$ref;



                	eval {
                        $vif_config->delete_vif(
                        	server      => $servername,
                        	vif         => $data{name}.$interface_number,
                        	);
                	};
                }
	NOTE::logcomment("Completed delete_interfaces routine");

}


##
## To create the parameters needed for cluster creation
##
sub _clus_create_join_params {
    my @int = _vifinfo(@_);
    my @retval;


    for (my $i = 1; $i <= 2; $i++)
    {
       push (@retval, "ipaddr$i" => $int[$i-1]->{address})
       if exists($int[$i-1]->{address}) && defined($int[$i-1]->{address});


         if ($i == 1) {
                push (@retval, "mtu" => $int[$i-1]->{mtu})
                    if exists($int[$i-1]->{mtu}) && defined($int[$i-1]->{mtu});
                push (@retval, "netmask" => $int[$i-1]->{netmask})
                    if exists($int[$i-1]->{netmask}) && defined($int[$i-1]->{netmask});
            }
    }

    @retval;
}























1;
