# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Dense/cmode/api/dense_cmds_cli.pm#1 $

## @summary
## @author arvindks@netapp.com
## @status draft
## @pod here

=head1 NAME

Dense::cmode::api::dense_cmds_cli - Dense commands library for cli

=head1 DESCRIPTION

This library contains the interfaces for dense cli apis. It is a sub-class of dense_cmds

=head1 SEE-ALSO

Dense::cmode::api::dense_cmds.pm and Dense::cmode::api::dense_cmds_zapi.pm

=head1 ROUTINES

dense_on()
dense_start()
dense_status()
dense_stop()
dense_off()

All these methods are implemented using zapi on c-mode RR cluster

=cut

package Dense::cmode::api::dense_cmds_cli;
use base qw(Dense::cmode::api::dense_cmds);

use strict;
use warnings;
use Tharn;
use Params::Validate qw(validate validate_pos OBJECT SCALAR BOOLEAN HASHREF ARRAYREF UNDEF);
use Data::Dumper qw(Dumper);
use constant DENSE_SEPERATOR => "<>";
use TracePkg;
my $trace = new TracePkg;

=head2 new()

 Usage:
 my $dense_cmds = Dense::cmode::api::dense_cmds_cli->new(filer=>$FILER, volume=>$TEST_VOL, vserver=>$VSERVER);
 
 =item filer
 
 filer name
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
 =item aggr
 Optional
 
 aggr name containing the vserver and volume
 
 Return Value:
 Success - return the dense_cmds object
 Failure - die with the message
 
=cut

sub new {
	my $class = shift;
	my $self = $class->SUPER::new(@_);
	
	bless $self, $class;
	$trace->comment(msg=>"Setting seperator for filer commands to ".DENSE_SEPERATOR);
	$self->set_seperator();
	return($self);
}

=head2 dense_on()

 Usage:
 $dense_cmds->dense_on(volume=>$TEST_VOL, vserver=>$VSERVER);
 $dense_cmds->dense_on();
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
	if volume and vserver is not specified, it would take the default, which was specified when dense_cmds object was created. If it is not specified there as well, then this would fail
	
 Return Value:
 Success - returns output from sis on
 Failure - die with the message
 
=cut

sub dense_on {
	my $self=shift;
	my %opt = @_;
	$opt{volume} = $self->{volume} if (!$opt{volume} && $self->{volume});
	$opt{vserver} = $self->{vserver} if (!$opt{vserver} && $self->{vserver});
	my $out;
	
	$self->{server}->sis_on(output=>\$out, %opt);

	$trace->debug(msg=>"dense_on() returning with $out");
	return($out);
#	die("\"sis on\" for $opt{volume} of $opt{vserver} failed with <<$out>>") if($out !~/SIS for Volume \'$opt{volume}\' of Virtual Server \'$opt{vserver}\' is enabled.Already existing data could be processed by running \"sis start -vserver $opt{vserver} -volume $opt{volume} -scan true\"./);
}

=head2 dense_start()

 Usage:
 $dense_cmds->dense_start(volume=>$TEST_VOL, vserver=>$VSERVER);
 $dense_cmds->dense_start();
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
	if volume and vserver is not specified, it would take the default, which was specified when dense_cmds object was created. If it is not specified there as well, then this would fail
	
 Return Value:
 Success - returns output from sis start
 Failure - die with the message
 
=cut

sub dense_start {
	my $self=shift;
	my %opt = @_;
	$opt{volume} = $self->{volume} if (!$opt{volume} && $self->{volume});
	$opt{vserver} = $self->{vserver} if (!$opt{vserver} && $self->{vserver});
	my $out;
	
	$self->{server}->sis_start(output=>\$out, %opt);
	
	$trace->debug(msg=>"dense_start() returning with $out");
	return($out);
#	die("\"sis start\" for $opt{volume} of $opt{vserver} failed with <<$out>>") if($out !~/The SIS operation for Volume \'$opt{volume}\' of Virtual Server \'$opt{vserver}\' has started./);
}

=head2 dense_status()

 Usage:
 $dense_cmds->dense_status(volume=>$TEST_VOL, vserver=>$VSERVER, %opt);
 $dense_cmds->dense_status(%opt);
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
	if volume and vserver is not specified, it would take the default, which was specified when dense_cmds object was created. If it is not specified there as well, then this would get the status of all the volumes/vserver on the cluster
	
 =item %opt
 Optional
 
 sis status options and their values can be specified here. These are like 
    -instance             -l                    -vserver
    -state                -op-status            -progress
    -type                 -schedule             -min-blks-shared
    -blks-skipped-sharing -last-op-begin        -last-op-end
    -last-op-size         -last-op-error        -changelog-usage
    -checkpoint-time      -checkpoint-op-type   -checkpoint-stage
    -checkpoint-sub-stage -checkpoint-progress  -op-stage
    -blk-gathered         -start-gather-time    -fp-sorted
    -dups-found           -start-sort-time      -blks-deduped
    -start-dedup-time     -fp-deleted           -start-check-time
    -checkpoint-info      -fields

 Return Value:
 Success - returns an hash having the following fields as an example. The list would have one more than the number volumes for which sis status has been gathered.
 
            'checkpoint-stage' => [
                                   'Checkpoint Stage',
                                   'Saving_pass1'
                                   ],
           'op-stage' => [
                           'Stage',
                           '-'
                                   ],
           'op-status' => [
                            'status',
                            'Active'
                                   ],
           'blks-skipped-sharing' => [
                                       'Blocks Skipped Sharing',
                                       '0'
                                   ],
           'fp-sorted' => [
                            'Fingerprints Sorted',
                            '-'
                                   ],
           'start-dedup-time' => [
                                   'De-duping Begin',
                                   '-'
                                   ],
           'blks-deduped' => [
                               'Blocks Deduplicated',
                               '-'
                                   ],
           'checkpoint-op-type' => [
                                     'Checkpoint Op Type',
                                     'Start'
                                   ],
           'state' => [
                        'state',
                        'Enabled'
                                   ],
           'last-op-size' => [
                               'Last Operation Size',
                               '"0 KB"'
                                   ],
           'checkpoint-progress' => [
                                      'Checkpoint Progress',
                                      '-'
                                   ],
           'changelog-usage' => [
                                      'Checkpoint Progress',
                                      '-'
                                   ],
           'changelog-usage' => [
                                  'Changelog Usage',
                                  '0%'
                                   ],
           'min-blks-shared' => [
                                  'Min Blocks Shared',
                                  '1'
                                   ],
           'start-gather-time' => [
                                    'Gathering Begin',
                                    '-'
                                   ],
           'checkpoint-sub-stage' => [
                                       'Checkpoint Sub-stage',
                                       '-'
                                   ],
           'volume' => [
                         'volume name',
                         'dense_vol'
                                   ],
           'vserver' => [
                          'vserver name',
                          'dense_vs'
                                   ],
           'progress' => [
                           'Progress',
                           '"0 KB (0%) Done"'
                                   ],
           'blk-gathered' => [
                               'Fingerprints Gathered',
                               '-'
                                   ],
           'start-sort-time' => [
                                  'Sorting Begin',
                                  '-'
                                   ],
           'last-op-end' => [
                              'Last Operation End',
                              '"Sat Nov 21 16:07:49 UTC 2009"'
                                   ],
           'dups-found' => [
                             'Duplicate Blocks Found',
                             '-'
                                   ],
           'schedule' => [
                           'Schedule',
                           'sun-sat@0'
                                   ],
           'checkpoint-time' => [
                                  'Checkpoint Time',
                                  '"Sat Nov 21 16:07:49 UTC 2009"'
                                   ],
           'last-op-begin' => [
                                'Last Operation Begin',
                                '"Sat Nov 21 16:07:40 UTC 2009"'
                                   ],
           'start-check-time' => [
                                   'Checking Begin',
                                   '-'
                                   ],
           'fp-deleted' => [
                             'Fingerprints Deleted',
                             '-'
                                   ],
           'type' => [
                       'Type',
                       'Regular'
                                   ],
           'last-op-error' => [
                                'Last Operation Error',
                                '-'
                                   ],
 Failure - die with the message
 
=cut

sub dense_status {
	my $self=shift;
	my %opt = @_;
	$opt{volume} = $self->{volume} if (!$opt{volume} && $self->{volume});
	$opt{vserver} = $self->{vserver} if (!$opt{vserver} && $self->{vserver});
	my $out;
	
	$self->{server}->sis_status(output=>\$out, %opt);
	
	return(_parse_status($out));	
}

=head2 dense_show()

 Usage:
 $dense_cmds->dense_show(volume=>$TEST_VOL, vserver=>$VSERVER, %opt);
 $dense_cmds->dense_show(%opt);
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
	if volume and vserver is not specified, it would take the default, which was specified when dense_cmds object was created. If it is not specified there as well, then this would get the status of all the volumes/vserver on the cluster
	
 =item %opt
 Optional
 
 Same as dense_status
 
=cut

sub dense_show {
	my $self=shift;
	my %opt = @_;
	$opt{volume} = $self->{volume} if (!$opt{volume} && $self->{volume});
	$opt{vserver} = $self->{vserver} if (!$opt{vserver} && $self->{vserver});
	my $out;
	
	$self->{server}->sis_show(output=>\$out, %opt);
	
	return(_parse_status($out));
}

=head2 dense_stop()

 Usage:
 $dense_cmds->dense_stop(volume=>$TEST_VOL, vserver=>$VSERVER);
 $dense_cmds->dense_stop();
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
	if volume and vserver is not specified, it would take the default, which was specified when dense_cmds object was created. If it is not specified there as well, then this would fail
	
 Return Value:
 Success - returns output from sis stop
 Failure - die with the message
 
=cut

sub dense_stop {
	my $self=shift;
	my %opt = @_;
	$opt{volume} = $self->{volume} if (!$opt{volume} && $self->{volume});
	$opt{vserver} = $self->{vserver} if (!$opt{vserver} && $self->{vserver});
	my $out;
	
	$self->{server}->sis_stop(output=>\$out, %opt);
	$self->dense_wait_for(status=>"Idle");
	$trace->debug(msg=>"dense_stop returned with $out");
	return($out);
#	die("\"sis start\" for $opt{volume} of $opt{vserver} failed with <<$out>>") if($out !~/The SIS operation for Volume \'$opt{volume}\' of Virtual Server \'$opt{vserver}\' is being stopped./);
}

=head2 dense_off()

 Usage:
 $dense_cmds->dense_off(volume=>$TEST_VOL, vserver=>$VSERVER);
 $dense_cmds->dense_off();
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
	if volume and vserver is not specified, it would take the default, which was specified when dense_cmds object was created. If it is not specified there as well, then this would fail
	
 Return Value:
 Success - returns output from sis off
 Failure - die with the message
 
=cut

sub dense_off {
	my $self=shift;
	my %opt = @_;
	$opt{volume} = $self->{volume} if (!$opt{volume} && $self->{volume});
	$opt{vserver} = $self->{vserver} if (!$opt{vserver} && $self->{vserver});
	my $out;
	
	$self->{server}->sis_off(output=>\$out, %opt);
	
	$trace->debug(msg=>"dense_stop returned with $out");
	return($out);
#	die("\"sis off\" for $opt{volume} of $opt{vserver} failed with <<$out>>") if($out !~/SIS for volume \'$opt{volume}\' of virtual server \'$opt{vserver}\' is disabled./);
}

=head2 dense_wait_for()

 Usage:
 $dense_cmds->dense_wait_for(volume=>$TEST_VOL, vserver=>$VSERVER, status=>"Idle");
 $dense_cmds->dense_wait_for(status=>"Idle");
 
 =item status
 
 status of the volume on which to wait for
 
 =item volume
 Optional
 
 volume name
 
 =item vserver
 Optional
 
 vserver name containing volume
 
	if volume and vserver is not specified, it would take the default, which was specified when dense_cmds object was created. If it is not specified there as well, then this would fail
	
 =item gap
 Optional

	time interval to check for the status used for changing the frequency of checking the status

 =item timeout
 Optional

	the amount of time to wait till you get the desired result. If the status has not reached the desired state till this timeout value, then the function dies with timeout.

 Return Value:
 Success - does not return any thing
 Failure - die with the message
 
=cut

sub dense_wait_for {
	my $self=shift;
	my (%opt) = validate(@_, {
								volume		=> { type => SCALAR, optional => 1 },
								vserver		=> { type => SCALAR, optional => 1 },
								status		=> { type => SCALAR, default => "Idle" },
								gap			=> { type => SCALAR, default => 5 },
								timeout		=> { type => SCALAR, default => 3600 },
							});
	$opt{volume} = $self->{volume} if (!$opt{volume} && $self->{volume});
	$opt{vserver} = $self->{vserver} if (!$opt{vserver} && $self->{vserver});
	my $ret;
	my $state;
	my $exp_state = $opt{status};
	delete $opt{status};
	$opt{fields} = "op-status";
	$trace->fatal(msg=>"dense_wait_for requires volume, vserver and state information to wait for the sis state to reach state") if(!$exp_state || !$opt{vserver} || !$opt{volume});
	while(($opt{timeout}>0) && ($state !~/$exp_state/)) {
		$ret = $self->dense_status(%opt);
		$state = $ret->{"op-status"}[1]; # Since volume and vserver are mandatory for this function, the entry would be in index 1 only
		logcomment("Would wait for $opt{timeout} more seconds for the status to reach the desired state");
		sleep($opt{gap});
		$opt{timeout} -= $opt{gap};
	}
	$trace->fatal(msg=>"dense_wait_for() timed out before status could reach the desired state") if($opt{timeout}<=0);
}

sub _get_idx {
	my (%opt) = validate(@_, {
								status_hash	=> { type => HASHREF },
								volume		=> { type => SCALAR },
								vserver		=> { type => SCALAR },
								idx			=> { type => SCALAR, optional => 1 },
							});
	return $opt{idx} if($opt{idx} && ($opt{volume} eq $opt{status_hash}->{volume}[$opt{idx}]) && ($opt{volume} eq $opt{status_hash}->{volume}[$opt{idx}]));
	foreach my $idx (0...@{$opt{status_hash}->{"op-status"}}) {
		return($idx) if(($opt{volume} eq $opt{status_hash}->{volume}[$idx]) && ($opt{vserver} eq $opt{status_hash}->{vserver}[$idx]));
	}
	$trace->fatal(msg=>"Could not find $opt{volume} of $opt{vserver} on the cluster");
}

sub _parse_status {
	my ($out) = @_;
	my %status_hash;
	my $sep = DENSE_SEPERATOR;
	my @lines = split(/\n/, $out);
	my @fields = split(/$sep/, shift(@lines));

	foreach (@lines) {
		my @values = split(/$sep/);
		push(@{$status_hash{$fields[$_]}}, $values[$_]) foreach(0...$#values);
	}
	
	return(\%status_hash);
}

sub set_seperator {
	my $self=shift;
	my $out;
	my $sep = DENSE_SEPERATOR;
	my %opt = ( showallfields=>"true", showseparator=> "\"$sep\"");
	
	$self->{server}->set(output=>\$out, %opt);
	return($out);
}

1;
