# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/Dense/sis_zapi.pm#1 $
#
## @summary
##    Zapi routines for SIS
##
## @usage
##  use Dense::sis_zapi;
##
## @author arvindks@netapp.com
## @status
## @pod here

package Dense::sis_zapi;

use strict;

use ZapiTest;
use ZapiErrno;
use NaElement;
use Tharn;
use Tharn qw(connect host say logcomment logresult param tstlog);
use Dense::Constant::Clone_Command;
use Data::Dumper qw(Dumper);

use Params::Validate qw(validate validate_pos OBJECT SCALAR HASHREF ARRAYREF BOOLEAN UNDEF);

BEGIN {
	Params::Validate::validation_options(
			on_fail => sub  {
					my @msgs    = @_;
					chomp(@msgs);
					Carp::confess(Tharn::script_result(@msgs))
					});
} #BEGIN #1

use base qw(Exporter);

BEGIN {

	my @conf    = qw ();

	my @action  = qw (
		zapi_sis_clone_start
		zapi_sis_clone_stop
	);

	my @status  = qw (
		zapi_get_sis_clone_list
		zapi_get_sis_status
	);

	my @state   = qw ();

	my @util    = qw (
		zapi_sis_clone_clear
		get_blk_range_list
	);


	my @verify  = qw (
		zapi_test_file_cloning
	);

	our %EXPORT_TAGS= ( conf   => [ @conf ],
						action => [ @action ],
						status => [ @status ],
						state  => [ @state ],
						util   => [ @util ],
						verify => [ @verify ]
						);

	$EXPORT_TAGS{'all'} = [@conf, @status, @action, @state, @util, @verify,];

	Exporter::export_ok_tags('all');

} #BEGIN #2

logcomment(__FILE__);

sub _zapi_get_clone_id_info {
    my ($result)=@_;
    my %clone_id_info;
	return () if(!$result);
    my $results=$result->child_get("clone-id");
	return () if(!$results);
    $results=$results->child_get("clone-id-info");
    $clone_id_info{clone_id}=$results->child_get_int("clone-op-id");
    $clone_id_info{vol_uuid}=$results->child_get_string("volume-uuid");
    return(\%clone_id_info);
}

sub _zapi_set_clone_id_info {
    my ($clone_id_hash)=@_;
    my $clone_id=new NaElement("clone-id");
    my $clone_id_info=new NaElement("clone-id-info");
    $clone_id_info->child_add_string("clone-op-id", $clone_id_hash->{clone_id});
    $clone_id_info->child_add_string("volume-uuid", $clone_id_hash->{vol_uuid});
    $clone_id->child_add($clone_id_info);
    return($clone_id);
}

sub _zapi_get_block_ranges {
    my ($naElm_blkranges)=@_;
    my @fields=qw/block-count destination-block-number source-block-number/;
    my @blk_ranges;
    return () if(!$naElm_blkranges);
    foreach(my $i=0; $i<$naElm_blkranges->child_max_number(); $i++) {
        my $blkrange=$naElm_blkranges->child_get_nth($i);
        $blk_ranges[$i]->{$_}=$blkrange->child_get_int($_) foreach(@fields);
    }
    return(\@blk_ranges);
}

sub _zapi_set_block_ranges {
    my @blk_ranges=@_;
    my @fields=qw/block-count destination-block-number source-block-number/;
    my $blkrges=new NaElement("block-ranges");

    foreach my $blk_range (@blk_ranges) {
        my $blkrge=new NaElement("block-range");
        $blkrge->child_add_string($_, $blk_range->{$_}) foreach(@fields);
        $blkrges->child_add($blkrge);
    }

    return $blkrges;
}

sub get_blk_range_list {
	my ($blk_range)=@_;
	my @blk_range_list;
	foreach(split(/\s+/, $blk_range)) {
		my ($src_fbn, $dest_fbn, $fbn_cnt)= split(/:/);
		my $blk_rge_rec={ 	"source-block-number"		=> $src_fbn,
							"destination-block-number"	=> $dest_fbn,
							"block-count"				=> $fbn_cnt };
		push(@blk_range_list, $blk_rge_rec);
	}
	return(\@blk_range_list);
}

########################################################
### Start sis clone operation through zapi
########################################################

=head2 zapi_sis_clone_start(filer=> $filer, source=> $src_file, clone => $clone_file, changelog => 0, nosnap => 0, fail_msg => 0,
	blk_range => [ {"source-block-number"       => $src_fbn,
					"destination-block-number"  => $dest_fbn,
					"block-count"               => $fbn_cnt } ], 
	timeout => 3600)
start clone operation

Return value:

Success : Return the clone ID info structure in the form of hash as follows, if expected to succeed, else if expected to fail with the fail_msg return undef:
	{ clone_id => <clone operation id>,
	  vol_uuid => <volume uuid> }
Fail    : die with the message

=cut

sub zapi_sis_clone_start($@) {
	my (%opts) = validate(@_, 
		{	filer		=> { type => HASHREF },
			source		=> { type => SCALAR },
			clone		=> { type => SCALAR, optional=>1 },
			block_ranges=> { type => ARRAYREF, optional=>1 },
			changelog	=> { type => BOOLEAN, optional=>1 },
			nosnap		=> { type => BOOLEAN, optional=>1 },
			fail_msg	=> { type => SCALAR, optional=>1 },
			timeout		=> { type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1 },
		});
	my $result;
	my $zapi_server=$opts{filer}->{z};

	my $cmd = new NaElement("clone-start");
	$cmd->child_add_string("source-path", $opts{source});
	$cmd->child_add_string("destination-path", $opts{clone}) if($opts{clone});
	$cmd->child_add(new NaElement("change-log", "true")) if($opts{changelog});
	$cmd->child_add(new NaElement("no-snap", "true")) if($opts{nosnap});
	$cmd->child_add(_zapi_set_block_ranges(@{$opts{block_ranges}})) if($opts{block_ranges});

	my $zapi_result = $zapi_server->invoke_elem($cmd);

	if(!$opts{fail_msg}) {
		if($zapi_result->results_ok()) {
			$result = _zapi_get_clone_id_info($zapi_result);
			logcomment("zapi clone started");
			return($result);
		} else {
			die fatal_result($zapi_result->toString());
		}
	} else {
		my $obtained = $zapi_result->results_reason();
		if($obtained =~/$opts{fail_msg}/) {
			logcomment("clone start failed with the expected reason $opts{fail_msg}");
			$result = _zapi_get_clone_id_info($zapi_result);
			return($result);
		} else {
			die fatal_result("clone start failed expected: $opts{fail_msg}, but obtained: $obtained");
		}
	}
}

########################################################
### Get Clone status list
########################################################
=head2 zapi_get_sis_clone_list(conn=>$conn, clone_id_info=>$hashref ({ clone_id => <clone id to be stopped>, vol_uuid => <volume uuid on which to stop>}), timeout => 3600

Get the clone status list on the filer for the clone id mentioned or for all the clone operations on the filer

Return value:

    List of following Status object

    Clone status        : $status_info{'clone-state'}
    Source file         : $status_info{'source-file'}
    Destination file    : $status_info{'destination-file'}
    blocks cloned		: $status_info{'blocks-copied'}
    total blocks		: $status_info{'total-blocks'}
    percent cloned		: $status_info{'percent-done'}
    Clone type          : $status_info{'clone-type'}
    error if found		: $status_info{'error'}
    reason for error	: $status_info{'reason'}

=cut

sub zapi_get_sis_clone_list($@) {
    my (%opts) = validate(@_, 
		{	filer => { type => HASHREF },
			clone_id_info => { type => HASHREF, optional=>1 },
			timeout => { type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1 },
		});
	my @results;
	my @strfields=qw/source-file destination-file clone-state clone-type reason/;
	my @intfields=qw/blocks-copied error total-blocks percent-done/;
	my $zapi_server=$opts{filer}->{z};

	my $cmd = new NaElement("clone-list-status");
	if($opts{clone_id_info}) {
		$cmd->child_add(_zapi_set_clone_id_info($opts{clone_id_info}));
	}

	my $zapi_result = $zapi_server->invoke_elem($cmd);

	if($zapi_result->results_ok()) {
		my $zapi_results = $zapi_result->child_get("status");
		return () if(!$zapi_results);
		foreach(my $i=0; $i<$zapi_results->child_max_number(); $i++) {
			my $clone_result=$zapi_results->child_get_nth($i);
			my $clone_id_info=_zapi_get_clone_id_info($clone_result);
			$results[$i]=$clone_id_info;
			$results[$i]->{$_}=$clone_result->child_get_string($_) foreach(@strfields);
			$results[$i]->{$_}=$clone_result->child_get_int($_) foreach(@intfields);
			$results[$i]->{"block-ranges"}=_zapi_get_block_ranges($clone_result->child_get("block-ranges"));
		}
	} else {
		die fatal_result("clone-list-status ZAPI returns ".$zapi_result->toString());
	}
	return(\@results);
}

########################################################
### Stop sis clone operation through zapi
########################################################

=head2 zapi_sis_clone_stop(filer=>$filer, clone_id_info=>$hashref ({ clone_id => <clone id to be stopped>, vol_uuid => <volume uuid on which to stop>}), timeout => 3600)

Return value:

    Success : 1
    Fail    : 0

=cut

sub zapi_sis_clone_stop($@) {
	my (%opts) = validate(@_, 
		{	filer => { type => HASHREF },
			clone_id_info => { type => HASHREF },
			timeout => { type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1 },
		});

	my $cmd = new NaElement("clone-stop");
	$cmd->child_add(_zapi_set_clone_id_info($opts{clone_id_info}));

	my $result = $opts{filer}->{z}->invoke_elem($cmd);

	logcomment("clone_stop on filer ".$result->results_status());
	logcomment("clone_stop failed because ".$result->results_reason()) if(!$result->results_ok());
	return($result->results_ok());
}

########################################################
### Clear failed clone ID through zapi
########################################################
=head2 zapi_sis_clone_clear(server=>$zapi_server, clone_id_info=>$hashref ({ clone_id => <clone id to be stopped>, vol_uuid => <volume uuid on which to stop>}),timeout => 3600

Clear failed clone operations on the filer with clone id mentioned

Return value:

    Success : 1
    Fail    : 0

=cut

sub zapi_sis_clone_clear($@) {
    my (%opts) = validate(@_, 
		{	filer => { type => HASHREF },
			clone_id_info => { type => HASHREF },
			timeout => { type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1 },
		});

	my $cmd = new NaElement("clone-clear");
	$cmd->child_add(_zapi_set_clone_id_info($opts{clone_id_info}));

	my $result = $opts{filer}->{z}->invoke_elem($cmd);

	logcomment("clone_clear on filer ".$result->results_status());
	logcomment("clone_clear failed because ".$result->results_reason()) if(!$result->results_ok());
	return($result->results_ok());
}

##########################################################
### Get sis status through zapi
##########################################################
=head2 zapi_get_sis_status(filer=>$filer_host, verbose=>$verbose, path=>$path)

Display status of all the dense volumes and de-duplication operations running on them.

Return value:

    List of following Status object

    path		: $status_info{'path'}
    state		: $status_info{'state'}
    status		: $status_info{'status'}
    progress		: $status_info{'progress'}
    type                : $status_info{'type'}
    schedule            : $status_info{'schedule'}
    last operation begin: $status_info{'last-operation-begin-timestamp'}
    last operation end  : $status_info{'last-operation-end-timestamp'}
    last operation error: $status_info{'last-operation-error'}
    last operation size	: $status_info{'last-operation-size'}

=cut

sub zapi_get_sis_status($@) {
    my (%opts) = validate(@_,
                {       filer => { type => HASHREF },
			verbose => { type=>SCALAR, default=>"TRUE", optional=>1, regex=>qr/^(TRUE|FALSE)$/ },
			path => { type=>SCALAR, default=>"", optional=>1 },
                        timeout => { type=>SCALAR, default=>DENSE_CLONE_DEFAULT_TIMEOUT, optional=>1 },
                });

    my @results;
    my @strfields=qw/path state status progress type schedule last-operation-size last-operation-error/;
    my @intfields=qw/last-operation-begin-timestamp last-operation-end-timestamp/;

    my ($verbose) = $opts{verbose};
    my ($path) = $opts{path};
    my $cmd = new NaElement("sis-status");

    $cmd->child_add_string("verbose", $opts{verbose});
    $cmd->child_add_string("path", $opts{path}) if($opts{path});
    logcomment("Before ZAPI call");
    my $zapi_result = $opts{filer}->{z}->invoke_elem($cmd);
    my $out = $zapi_result->toString();

    logcomment("Result string : $out");
    if($zapi_result->results_ok()) {
            my $zapi_results = $zapi_result->child_get("status");
            return () if(!$zapi_results);
            foreach(my $i=0; $i<$zapi_results->child_max_number(); $i++) {
                    my $status_result=$zapi_results->child_get_nth($i);
                    $results[$i]->{$_}=$status_result->child_get_string($_) foreach(@strfields);
                    $results[$i]->{$_}=$status_result->child_get_int($_) foreach(@intfields);
            }
    } else {
            die fatal_result("clone-list-status ZAPI returns ".$zapi_result->toString());
    }
    return(\@results);

}

1;
#### END OF FILE ############
