# -*-perl-*x
# $Id: //depot/prod/test/Rsierranevada/lib/Df_stats.pm#1 $
# 
#
# Copyright (c) 2008 Network Appliance, Inc.
# All rights reserved.
#
# This is C-Mode Library for Space Management
#
## @summary
##     Library is for showing 'df' from node run in ngsh
##
## @description
##
##   calls df with -V(k|m|g) or -A(k|m|g)
##   
##
## @usage
##     use Df_stats;
##
## @author Hardikkumar Darji (darji@netapp.com) 
##
## @status private

package Df_stats;

use Carp qw(croak);
use Tharn;
use Params::Validate qw (validate validate_pos SCALAR OBJECT ARRAYREF);
use Server;

use vars ('@ISA','@EXPORT');
@ISA = qw(Exporter);
@EXPORT = qw(df_show);

## @name ::df_show($server,$type,$instance,$unit)
##
## @summary
#i     Calls df to check the value of total,available and used space as well as capacity for volume and aggregate
## @description
##     
##
## @arg $server Required: filer connection object.
## @arg $type Required: volume or aggregate.
## @arg $instance Required: name of volume oraggregate.
## @arg $unit Required: unit (KB,MB or GB).
##
## @return value
##      Returns errcode = 0 - Success
##                      < 0 - Failure
##              result = On Success, the array containing the values.
##                     = On Failure, the array containing error message.
##                          
sub df_show(%)
{
     my %opts = validate(
	@_,
	{
		server => { type => OBJECT },
		type => {type => SCALAR, default=>'V'},
		instance => {type => SCALAR},
                unit => {type => SCALAR, default=>'k'}
	}
    );
     
     
    my $server = $opts{server};
    my $type= $opts{type}; 
    my $instance = $opts{instance};
    my $unit = $opts{unit};
    my $node = $server->get_execution_server();
    my $msg;
    my $errcode = 0;
    my $err_msg_instance = "df: $instance: No such file or directory";
    my $cmd = "df -$type$unit $instance";
    
    ## check for invalid type of object 
    if($type ne 'V' && $type ne 'A')
     {
          $errcode= -1;
          $result[0]= "Object type is not correct.Accepted object types are 'V' or 'A'";
          return ($errcode, \@result);
     }
     
     ## check for invalid  unit
     if($unit ne 'k' && $unit ne 'm' && $unit ne 'g')
     {
          $errcode= -1;
          $result[0]= "Unit is not correct. Accepted Units are 'k' or 'm' or 'g'";
          return ($errcode, \@result);
     }
     
     
     if($type eq 'V')
     {
          $msg="/vol/$instance/";
     }
     else
     {
          $msg="$instance";
     }
     
    ## set privilege level to "test" 
    $server->set(privilege=>"test");
    
    ## execution of df command
    my $res;
    $server->system_node_run(node    => $node,
                             command =>$cmd,
                             output  => \$res);
     
    my @lines = split /\n/, $res;
    my @result;
   
    ## checking output of 'df' command 
    foreach (@lines)
    {
          ## if there is an error then errcode is < 0 and return the error message
          if($_=~ /$err_msg_instance/)
          {
              $errcode = -1;
              $result[0] = $_;
          }
          ## if no error then parse the output into array for the line that mentions volume's or aggr's space statistics
          if($_=~/$msg\s+(\d+)(KB|MB|GB)\s+(\d+)(KB|MB|GB)\s+(\d+)(KB|MB|GB)\s+(\d+)%/)
          {
               $errcode = 0;
               $result[0]=$1; 
               $result[1]=$3; 
               $result[2]=$5; 
               $result[3]=$7; 
          }
    }
    return ($errcode, \@result);
}

1;



