#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Diagnosis.pm#1 $

#***********************************************************************
#
# NAME
#   Diagnosis.pm - programatic interfaces for Diagnosis automation
#
# SYNOPSIS
#   Enables remote programmatic interfaces of Diagnosis Automation
#
# DESCRIPTION
#   This Perl module is meant to be programmatic way of automating 
#   Diagnosis project
#
# AUTHOR
#       dl-nb-plat-val
#
#***********************************************************************
#RETURN CODES
# 300 - in sufficient params
# 301 - unable to fetch reqd info
#

#use vars qw/$Server $FILER $NOHEALTHCHECK/;

#***********************************************************************
#Function Name	:start
#Paremeters	:NULL
#Return Type	:0 on sucess and Error code on failure
#Description	:It tests whether system is ready to start test.
#***********************************************************************
sub start {
param("NOHEALTHCHECK","-default","0");

if($NOHEALTHCHECK)
{
	tstlog -comment, "NOCHECK Enabled";
} else
{
	if(&harama)
	{
		tstlog -fatal, "Health is not OK. Cannot start test" unless(&wait_for_ok); 

	}
	tstlog -comment, "HEALTH IS OK. TEST STARTED";
}
}

#***********************************************************************
#Function Name	:finish
#Paremeters	:NULL
#Return Type	:NULL
#Description	:This function should be called in END block of every script
#It collects configuration of the filer.
#***********************************************************************
sub finish {
tstlog -comment, "TEST END BLOCK STARTED: Capturing Configuration";
my $cmd='run -node local -command sysconfig -av';
	$Server->cli(cmd => $cmd);
#$cmd='set test;debug mtrace log show -process shm';
#	$Server->cli(cmd => $cmd);
#$cmd='set test;debug mtrace log show -process schm';
#	$Server->cli(cmd => $cmd);
#$cmd='set test;debug mtrace log show -process nchm';
#	$Server->cli(cmd => $cmd);
$cmd='cluster show';
	$Server->cli(cmd => $cmd);
$cmd='system health config show';
	$Server->cli(cmd => $cmd);

tstlog -comment, "TEST END";
}

#***********************************************************************
#Function Name	:harama
#Paremeters	:NULL
#Return Type	:0-OK;1-OK_with_suppressed;2-degraded;3-unreachable;4-other
#Description	:It gives the status of the system health monitor. harama means are you fine? in kannada(Bangalore)
#***********************************************************************

sub harama{
	my $cmd='system health status show';
	my $out = $Server->cli(cmd => $cmd);
	if($out =~ /OK_with_suppressed|ok-with-suppressed/)
	{
		return 1;
	} elsif ($out =~ /degraded/) {
		return 2;
	} elsif ($out =~ /OK|ok/) {
		return 0;
	} elsif ($out =~ /unreachable/) {
		return 3;
	}else {
		return 4;
	}
}

#***********************************************************************
#Function Name	:get_subsystems
#Paremeters	:optional command option and its value
#Return Type	:Reruns available subsystems in the cluster
#Description	:It runs system health config show command and list available subsystems
#***********************************************************************
sub get_subsystems
{
	my ($key,$value)=@_;
	my $looptwice=0;
	my $cmd;
	if($key)
	{
		$cmd='system health config show -'.$key.' '.$value;
	} else {
		$cmd='system health config show';
	}
	my $cmdout = $Server->cli(cmd => $cmd);
	my @sub_systems;
	my @cmdout_array=split(/\n/,$cmdout);
	foreach(@cmdout_array)
	{
		$looptwice++;
		next if($looptwice < 3);
		my @array_out=split(/X;X/);
		push(@sub_systems,$array_out[2]) unless(grep(/$array_out[2]/,@sub_systems));
	}
	return @sub_systems;
}

#***********************************************************************
#Function Name	:get_alertids
#Paremeters	:optional command option and its value
#Return Type	:Reruns available alerts in the cluster
#Description	:It runs system health alerts show command and list available alert ids
#***********************************************************************
sub get_alertids
{
	my ($key,$value)=@_;
	my $looptwice=0;
	my $cmd;
	if($key)
	{
		$cmd='system health alert show -'.$key.' '.$value;
	} else {
		$cmd='system health alert show';
	}
	my $cmdout = $Server->cli(cmd => $cmd);
	my @alertids;
	my @cmdout_array=split(/\n/,$cmdout);
	foreach(@cmdout_array)
	{
		$looptwice++;
		next if($looptwice < 3);
		my @array_out=split(/X;X/);
		push(@alertids,$array_out[2]) unless(grep(/$array_out[2]/,@alertids));
	}
	return @alertids;
}

#***********************************************************************
#Function Name	:get_hms;
#Paremeters	:optional command option and its value
#Return Type	:Reruns available HMs  in the cluster
#Description	:It runs system health config show command and list available health monitors
#***********************************************************************
sub get_hms
{
	my ($key,$value)=@_;
	my $looptwice=0;
	my $cmd;
	if($key)
	{
		$cmd='system health config show -'.$key.' '.$value;
	} else {
		$cmd='system health config show';
	}
	my $cmdout = $Server->cli(cmd => $cmd);
	my @hms;
	my @cmdout_array=split(/\n/,$cmdout);
	foreach(@cmdout_array)
	{
		$looptwice++;
		next if($looptwice < 3);
		my @array_out=split(/X;X/);
		push(@hms,$array_out[1]); 
	}
	return @hms;
}

#***********************************************************************
#Function Name  :get_SCHM_node;
#Paremeters     :optional command option and its value
#Return Type    :Reruns available node where SCHM is running
#Description    :It runs system health config show command and list available health monitors
#***********************************************************************
sub get_SCHM_node
{
        my ($key,$value)=@_;
        my $looptwice=0;
        my $cmd;
        if($key)
        {
                $cmd='system health config show -'.$key.' '.$value;
        } else {
                $cmd='system health config show';
        }
        my $cmdout = $Server->cli(cmd => $cmd);
        my @alertids;
        my @cmdout_array=split(/\n/,$cmdout);
        foreach(@cmdout_array)
        {
                $looptwice++;
                next if($looptwice < 3);
                my @array_out=split(/X;X/);
                if ($array_out[1] =~ /system-connect/)
                {
                        $node_SCHM = $array_out[0];
                }
        }
        return $node_SCHM;
}

#***********************************************************************
#Function Name  :get_SHM_node;
#Paremeters     :optional command option and its value
#Return Type    :Reruns available node where SHM is running
#Description    :It runs system health config show command and list available health monitors
#***********************************************************************
sub get_SHM_node
{
        my ($key,$value)=@_;
        my $looptwice=0;
        my $cmd;
        if($key)
        {
                $cmd='system health config show -'.$key.' '.$value;
        } else {
                $cmd='system health config show';
        }
        my $cmdout = $Server->cli(cmd => $cmd);
        my @alertids;
        my @cmdout_array=split(/\n/,$cmdout);
        foreach(@cmdout_array)
        {
                $looptwice++;
                next if($looptwice < 3);
                my @array_out=split(/X;X/);
                if ($array_out[1] =~ /^system$/)
                {
                        $node_SHM = $array_out[0];
                }
        }
        return $node_SHM;
}

#***********************************************************************
#Function Name	:get_aggrs
#Paremeters	:hm
#Return Type	:Reruns the aggregator of given hm
#Description	:It runs system health config show command and returns the aggrgator
#***********************************************************************
sub get_aggr
{
	my ($hm)=@_;
	return 300 unless($hm);
	my $cmd='system health config show -monitor '.$hm;
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my @array_out=split(/X;X/,$cmdout_array[2]);
	return $array_out[7];
}

#***********************************************************************
#Function Name	:get_hm_state
#Paremeters	: hm
#Return Type	:Reruns state of HM
#Description	:It runs system health config show command and returns the state of requested hm
#***********************************************************************
sub get_hm_state
{
	my ($hm)=@_;
	return 300 unless($hm);
	my $cmd='system health config show -monitor '.$hm;
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my @array_out=split(/X;X/,$cmdout_array[2]);
	return $array_out[3];
}


#***********************************************************************
#Function Name	:get_nchm_adapters
#Paremeters	:node(optional)
#Return Type	:Returns list of adapters listed in nchm
#Description	:It runs system health node-connect adapter show and returns list of adapters
#***********************************************************************
sub get_nchm_adapters
{
	my $node=$_;
	$node = 'local' unless($node);
	my $cmd='system health node-connect adapter show -node '.$node;
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my $looptwice=0;
	my @adapters;
	foreach(@cmdout_array)
	{
		$looptwice++;
		next if($looptwice < 3);
		my @array_out=split(/X;X/);
		push(@adapters,$array_out[1]) unless(grep(/$array_out[1]/,@adapters));
	}
	return @adapters;
}


#***********************************************************************
#Function Name	:get_nchm_isks
#Paremeters	:node(optional)
#Return Type	:Returns list of disks listed in nchm
#Description	:It runs system health node-connect disks show and returns list of disks
#***********************************************************************
sub get_nchm_disks
{
	my $node=$_;
	$node = 'local' unless($node);
	my $cmd='system health node-connect disk show -node '.$node;
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my $looptwice=0;
	my @disks;
	foreach(@cmdout_array)
	{
		$looptwice++;
		next if($looptwice < 3);
		my @array_out=split(/X;X/);
		push(@disks,$array_out[3]) unless(grep(/$array_out[1]/,@disks));
	}
	return @disks;
}


#***********************************************************************
#Function Name	:get_nchm_shelves
#Paremeters	:node(optional)
#Return Type	:Returns list of shelves listed in nchm
#Description	:It runs system health node-connect shelf show and returns list of shelves
#***********************************************************************
sub get_nchm_shelves
{
	my $node=$_;
	$node = 'local' unless($node);
	my $cmd='system health node-connect shelf show -node '.$node;
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my $looptwice=0;
	my @shelves;
	foreach(@cmdout_array)
	{
		$looptwice++;
		next if($looptwice < 3);
		my @array_out=split(/X;X/);
		push(@shelves,$array_out[3]) unless(grep(/$array_out[3]/,@shelves));
	}
	return @shelves;
}

#***********************************************************************
#Function Name	:get_sas_channels
#Paremeters	:NULL
#Return Type	:Returns list of available SAS channels(adapters)
#Description	:It sends sasadmin channels command to dblade cli and returns available channels
#***********************************************************************
sub get_sas_channels
{
	my $cmd='run -node local -command sasadmin channels';
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my @channels;
	foreach(@cmdout_array)
	{
		my @array_out=split(/\s+/);
		push(@channels,$array_out[1]);# unless(grep(/$array_out[1]/,@shelves));
	}
	return @channels;	
}

#***********************************************************************
#Function Name	:get_sas_shelves
#Paremeters	:sas adapter(optional)
#Return Type	:Returns list of available SAS shelves for the given adapter. provides all shelves if not passed adapter
#Description	:It sends sasadmin shelf command to dblade cli and returns available shelves
#***********************************************************************
sub get_sas_shelves
{
	my $adapter = shift;
	my $cmd='run -node local -command sasadmin shelf '.$adapter;
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my @shelves;
	my $adapter;
	my $idflag=0;
	my $opened=0;
	foreach(@cmdout_array)
	{
		next if(/Cannot Complete operation/);
		if(/-------/)
		{
			$idflag=1;
			next;
		}
		if(/Expanders on channel/)
		{
			my @array_out=split(/\s+/);
			chop($array_out[3]);
			$adapter=$array_out[3];	
			next;
		}	
		if($idflag)
		{
			if($opened)
			{
				$opened=0;	
				$idflag=0;
				next;
			} else {
				my @array_out=split(/\|/);
				$array_out[0] =~ /\d+/;
				my $shelfid=$&;
				$idflag=0;
				my $shelf=$adapter.'.shelf'.$shelfid;
				push(@shelves,$shelf);
				$opened=1;
			}
		}
	}
	return @shelves;
}

#***********************************************************************
#Function Name	:get_sas_disks
#Paremeters	:sas adapter(optional)
#Return Type	:Returns list of available SAS disks for the given adapter. provides all disks if not passed adapter
#Description	:It sends sasadmin expander command to dblade cli and returns available shelves
#***********************************************************************
sub get_sas_disks
{
	my $adapter = shift;
	my @adapters;
	if($adapter)
	{
		push(@adapters,$adapter);
	} else {
		@adapters = &get_sas_channels;
	}
	my @disks;
	foreach $adapter(@adapters)
	{
		my $cmd='run -node local -command sasadmin expander '.$adapter;
		my $cmdout = $Server->cli(cmd => $cmd);
		my @cmdout_array=split(/\n/,$cmdout);
		#tstlog -comment, "ADAPTERS:@adapters  C ADAPTER:$adapter";
		foreach(@cmdout_array)
		{
			if(/SAS Device/)
			{
				next if(/Virtual/);
				/\d\w\.\d{2}\.\d+/;
				push(@disks,$&);
			}	
		}
	}
	return @disks;
}

#***********************************************************************
#Function Name	:get_sas_adapter_state
#Paremeters	:sas adapter
#Return Type	:Returns list of available SAS disks for the given adapter. provides all disks if not passed adapter
#Description	:It sends sasadmin expander command to dblade cli and returns available shelves
#***********************************************************************
sub get_sas_adapter_state
{
	my $adapter=shift;
	return 300 unless($adapter);
	my $cmd='run -node local -command sasadmin adapter_state '.$adapter;
	my $cmdout = $Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	foreach (@cmdout_array)
	{
		if(/State for adapter channel/)
		{
			tstlog -comment, "$LINE:$_";
			/[A-Z][A-Z]+/;
			return $&;
		}
	}
	return 301;
}

#***********************************************************************
#Function Name  :disable_sas_adapter
#Paremeters     :sas adapter
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It sends sasadmin adapter_disable <adapter name> to the dblade cli
#***********************************************************************
sub disable_sas_adapter
{
        my $adapter=shift;
        return 300 unless($adapter);
        my $cmd='run -node local -command "priv set test;sasadmin adapter_offline '.$adapter.'"';
        my $cmdout = $Server->cli(cmd => $cmd);
#TBD:Need to verify whether state is changed or not and return 0 if not changed
}


#***********************************************************************
#Function Name  :disk_power_on
#Paremeters     :sas disk
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It finds phy number and power on through sasadmin
#***********************************************************************
sub disk_power_on
{
	my ($shelf_id,$phy)=@_;
	return 300 unless($phy);
	$cmd='run -node local -command "priv set test; sasadmin expander_phy_power_on '."$shelf_id".' '."$phy".'"';
        $cmdout = $Server->cli(cmd => $cmd);
	return 1;
}
#TBD:Need to verify whether state is changed or not and return 0 if not changed

#***********************************************************************
#Function Name  :get_phy_num
#Paremeters     :sas disk
#Return Type    :Returns phy number on success 0 on Failure
#Description    :It finds phy number through sasadmin
#***********************************************************************
sub get_phy_num
{
	my $disk=shift;
	return 300 unless($disk);
	$offset="-3";
	$offset="-2" if (length($disk) == 7);
	my $shelf_id= substr $disk,0,$offset;
        my $cmd='run -node local -command "priv set test;sasadmin expander '.$shelf_id.'"';
        my $cmdout = $Server->cli(cmd => $cmd);
	my @outlines=split(/\n/,$cmdout);
	my $phy=0;
	foreach (@outlines)
	{
		if(/$disk/)
		{
			/\d+\//;
			$phy=$&;
			chop $phy;	
			last;
		}	
	}
	return $phy;
}

#***********************************************************************
#Function Name  :disk_power_off
#Paremeters     :sas disk
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It finds phy number and power off through sasadmin
#***********************************************************************
sub disk_power_off
{
	my $disk=shift;
	return 300 unless($disk);
	$offset="-3";
	$offset="-2" if (length($disk) == 7);
	my $shelf_id= substr $disk,0,$offset;
        my $cmd='run -node local -command "priv set test;sasadmin expander '.$shelf_id.'"';
        my $cmdout = $Server->cli(cmd => $cmd);
	my @outlines=split(/\n/,$cmdout);
	my $phy=0;
	foreach (@outlines)
	{
		if(/$disk/)
		{
			/\d+\//;
			$phy=$&;
			chop $phy;	
			last;
		}	
	}
	return 0 unless($phy);
	$cmd='run -node local -command "priv set test; sasadmin expander_phy_power_off '."$shelf_id".' '."$phy".'"';
        $cmdout = $Server->cli(cmd => $cmd);
	return 1;
}
#TBD:Need to verify whether state is changed or not and return 0 if not changed

#***********************************************************************
#Function Name  :enable_sas_adapter
#Paremeters     :sas adapter
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It sends sasadmin adapter_enable <adapter name> to the dblade cli
#***********************************************************************
sub enable_sas_adapter
{
        my $adapter=shift;
        return 300 unless($adapter);
        my $cmd='run -node local -command "priv set test;sasadmin adapter_online '.$adapter.'"';
        my $cmdout = $Server->cli(cmd => $cmd);
#TBD:Need to verify whether state is changed or not and return 0 if not changed
}

#***********************************************************************
#Function Name  :wait_for_alert;
#Paremeters     :Null
#Return Type    :Returns 1 if alerts arriaved with in 15 mins. 0 otherwise
#Description    :It sends alert show for 30 sec and returns 1 if there is no 'This table is currently empty' message;
#***********************************************************************
sub wait_for_alert
{
my $count=0;
while ($count < 30)
{
	eval {
       	my $cmd='system health alert show';
       	$Server->cli(cmd => $cmd);
	};
	if($@)
	{
		sleep 30;
		$count++;
	} else {
		return 1;
	}
}
return 0;	
}

#***********************************************************************
#Function Name  :wait_for_ok;
#Paremeters     :Null
#Return Type    :Returns 1 if status changed to OK with in 15 minutes. 0 otherwise
#Description    :It sends status show for 30 sec and returns 1 if status is OK. 0 otherwise
#***********************************************************************
sub wait_for_ok
{
my $count=0;
while ($count < 30)
{
	my $cmd='system health status show';
	my $cmdout=$Server->cli(cmd => $cmd);
	if($cmdout =~ /OK|ok/ and $cmdout !~ /OK_with_suppressed|ok-with-suppressed/)
	{
		return 1;
	} else {
	sleep 30;
	$count++;
	}
}
return 0;
}

#***********************************************************************
#Function Name  :query_config
#Paremeters     :field,value,output field or only output field.
#Return Type    :Returns an hash that contains filtered results of health monitor as key and requested field valus as value.
#Example: query_config(context,node,health) returns below hash
#  node_connect = OK
#  node_connect = OK
#Description    :It sends config show -node and -requested filed its value and requested output field.
#Example:above example will make command as system health config show -context node -fields health
#***********************************************************************
sub query_config
{
(my $field, my $value, my $ofield)=@_;
return 300 unless($field);
$ofield=$field unless($ofield);
my $cmd;
if($value)
{
	$cmd="system health config show -$field $value -fields $ofield";
} else {

	$cmd="system health config show -node local -fields $ofield";
}
my $cmdout=$Server->cli(cmd => $cmd);
my @cmdout_array=split(/\n/,$cmdout);
my $looptwice=0;
my %hash1;
foreach(@cmdout_array)
{
	$looptwice++;
	next if($looptwice < 3);
	my @array_out=split(/X;X/);
	#tstlog -comment, "In function:$array_out[1]--$array_out[2]";
	$hash1{$array_out[1]}=$array_out[2];
}
return %hash1;
}
#***********************************************************************
#Function Name  :query_alert :: Under development 01-july-2010
#Paremeters     :field,value,output field or only output field.
#Return Type    :Returns an hash that contains filtered results of health monitor as key and requested field valus as value.
#Example: query_config(context,node,health) returns below hash
#  node_connect = OK
#  node_connect = OK
#Description    :It sends config show -node and -requested filed its value and requested output field.
#Example:above example will make command as system health config show -context node -fields health
#***********************************************************************
sub query_alert
{
(my $field, my $value, my $ofield)=@_;
return 300 unless($field);
$ofield=$field unless($ofield);
my $cmd;
if($value)
{
	$cmd="system health alert show -node local -$field $value -fields $ofield";
} else {

	$cmd="system health alert show -node local -fields $ofield";
}
my $cmdout=$Server->cli(cmd => $cmd);
my @cmdout_array=split(/\n/,$cmdout);
my $looptwice=0;
my @ret_arr;
my $out_field_index;
foreach my $line (@cmdout_array)
{
	$looptwice++;
	my @array_out=split(/X;X/,$line);
	if($line =~ /$ofield/)
	{
		$out_field_index++ while($array_out[$out_field_index] ne $ofield);
	}
	next if($looptwice < 3);
	#tstlog -comment, "In function:$array_out[1]--$array_out[2]";
	$hash1{$array_out[1]}=$array_out[2];
}
return %hash1;
}

#***********************************************************************
#Function Name  :get_alert_instance
#Paremeters     :NULL
#Return Type    :Returns an array of nodes in the cluster
#Description    :It sends cluster show and find out nodes in the cluster
#***********************************************************************
sub get_alert_instance
{
        my ($alert_id,$resource) = @_;
	return 300 unless($resource);
	my $cmd='system health alert show -instance -alert-id '.$alert_id.' -alerting-managed-element '.$resource ;
	my $cmdout=$Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my %alert_hash;
	foreach my $line (@cmdout_array)
	{
		if($line =~ /Node|Monitor|Alert Id|Alerting resource|Subsystem|Indication time|Perceived severity|Probable cause|Description|Service actions|Possible effect|Acknowledge[^r]|Acknowledger|Suppress|Policy Id/) {
			my $hashkey=$&;
			my ($key,$value, $value1)=split(/:/,$line);
			chomp($value);
			chomp($value);
			$value=$value1 if($key =~ /Additional/);
			$alert_hash{$hashkey}= substr $value, 1;
		} elsif($line =~ /Shelf uuid|Shelf id|Shelf Name|Paths|Number of Disks|Adapter connected to IOMA|Adapter connected to IOMB|Uuid|Serial Number|Bay Number|Secondary Disk name|Shelf Id|WWN|In use\?|Is enabled\?|Is dual attached|IOM port to be connected to|^Number of shelves$|Number of Shelves connected to IOMA|Number of Shelves connected to IOMB|Number of Shelves connected to Circle Port|Number of Shelves connected to Square Port|^List of shelf Names$|List of shelfs connected to IOMA|List of shelfs connected to IOMB|List of shelfs connected to Circle Port|List of shelfs connected to Square Port/) {
			my $hashkey=$&;
			my ($key,$value)=split(/:/,$line);
			chomp($value);
			chomp($value);
			$alert_hash{$hashkey}= substr $value, 1;
		}
	}
	return %alert_hash;
}

#***********************************************************************
#Function Name  :delete_alerts
#Paremeters     :alert_id
#Return Type    :Returns success or failure
#Description    :Deletes all alerts if no paramenter passed or deletes alert id if sent
#***********************************************************************
sub delete_alerts
{
        my ($alert_id) = shift;
	$alert_id='*' unless($alert_id);

#Verify that some alert is there
	my $cmd='system health alert show -node local -alert-id '.$alert_id;
	eval {
       	$Server->cli(cmd => $cmd);
	};
	if($@)
	{
		return 0;
	}
	
#Delete them
	$cmd='system health alert delete -node local -alert-id '.$alert_id;
	my $cmdout=$Server->cli(cmd => $cmd);

#Check still exists
	eval {
       	$cmd='system health alert show';
       	$Server->cli(cmd => $cmd);
	};
	if($@)
	{
		return 0;
	} else {
		return 1;
	}
}

#***********************************************************************
#Function Name  :delete_asup
#Paremeters     :alert_id
#Return Type    :Returns success or failure
#Description    :Deletes all asup messages if no paramenter passed or deletes alert id if sent
#***********************************************************************
sub delete_asup
{
        my ($alert_id) = shift;
	$alert_id='*' unless($alert_id);

#Verify that some asup is there
	my $cmd='system health autosupport trigger history show -node local -alert-id '.$alert_id;
	eval {
       	$Server->cli(cmd => $cmd);
	};
	if($@)
	{
		return 0;
	}
	
#Delete them
	$cmd='system health autosupport trigger history delete -node local -alert-id '.$alert_id;
	my $cmdout=$Server->cli(cmd => $cmd);

#Check still exists
	eval {
	$cmd='system health autosupport trigger history show -node local -alert-id '.$alert_id;
       	$Server->cli(cmd => $cmd);
	};
	if($@)
	{
		return 0;
	} else {
		return 1;
	}
}



#***********************************************************************
#Function Name  :get_alert_resources
#Paremeters     :alert_id(optional)
#Return Type    :Returns an array of resources that match with alert id sent.
#Description    :It sends alert show and creates array of resources belongs to given alert id. if alert is not passed, returns all resources of currect alerts
#***********************************************************************
sub get_alert_resources
{
        my $alert_id = shift;
	my $cmd;
	if($alert_id)
	{
		 $cmd='system health alert show -alert-id '.$alert_id;
	} else {
		 $cmd='system health alert show';
	}
	my $cmdout=$Server->cli(cmd => $cmd);
	my @cmdout_array=split(/\n/,$cmdout);
	my @resources;
	my $loop=0;
	foreach my $line (@cmdout_array)
	{
		$loop++;
		next if($loop < 3);
		my @array_out=split(/X;X/,$line);
		push(@resources,$array_out[3]);
	}
	return @resources;
}

#***********************************************************************
#Function Name  :get_nodes
#Paremeters     :NULL
#Return Type    :Returns an array of nodes in the cluster
#Description    :It sends cluster show and find out nodes in the cluster
#***********************************************************************
sub get_nodes
{
my $cmd='cluster show';
my $cmdout=$Server->cli(cmd => $cmd);
my @cmdout_array=split(/\n/,$cmdout);
my $loop=0;
my @nodes;
foreach my $line (@cmdout_array)
{
	$loop++;
	next if($loop < 3);
	my @array_out=split(/X;X/,$line);
	push(@nodes,$array_out[0]);
}
return @nodes;
}

#*******************************************************************************************
#Function Name  :counter_value
#Paremeters     :counter name
#Return Type    :Returns the value for the respective counter and zero if no instance found
#Description    :It sends counter name and find out the value for the respective counter
#*******************************************************************************************

sub counter_value {
        my ($counter_name) = @_;
	tstlog -fatal, "Please provide the appropriate counter name" if(!$counter_name);
        my @list;
        my @value;
        my $cmd="cm_client -p nchm -a object-get-instances -o sys_mgmt_and_diagnosis -c $counter_name";
        my $cmd_out = $Server->shell(cmd => $cmd);
        my @lines = split(/\n/, $cmd_out);
        foreach my $line(@lines) {
                if ( $line =~ /Num Instances/) {
                        @list = split(/Num Instances:/,$line);
                        return 0 if($list[-1] =~ 0);
                }elsif ( $line =~ /$counter_name/) {
                        @value = split(/value:/,$line);
                        last;
                }
        }
        chomp($value[-1]);
        $value[-1] =~ tr/ "//d;
        return $value[-1];
}

sub exec_zapi
{
	my $cmd=shift;
	my $target;
	$target=&get_localnodemgmtip;

	my $h=host($CLIENT);
	my $c=connect("#rlogin");
#	if($FILER_IP)
#	{
#		$target=$FILER_IP;
#	} else {
#		$target=$FILER;	
#	}
	return 301 unless($target);
	return 300 unless($cmd);
	my $zapi_cmd='/usr/software/test/bin/apitest '.$target.' admin netapp1! '.$cmd;
	(undef, undef,my $out,undef)=say($c,$zapi_cmd);
	return $out;
}

sub get_clusmgmtip
{
	my $cmd='network interface show -role cluster-mgmt -fields address';
	my $cmdout=$Server->cli(cmd => $cmd);
        my @lines = split(/\n/, $cmdout);
        foreach my $line(@lines) {
                if ( $line =~ /cluster_mgmt/) {
			$line =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;
                        return $&;
                }
        }
	return 0;
}

sub get_localnodemgmtip
{
	my $cmd='network interface show -role node-mgmt -vserver local -fields address';
	my $cmdout=$Server->cli(cmd => $cmd);
        my @lines = split(/\n/, $cmdout);
        foreach my $line(@lines) {
                if ( $line =~ /mgmt1/) {
			$line =~ /(\d{1,3})\.(\d{1,3})\.(\d{1,3})\.(\d{1,3})/;
                        return $&;
                }
        }
	return 0;
}

sub get_shelf_info
{
	my $shelf_id=shift;
	return 300 unless($shelf_id);
	my $cmd="storage show shelf $shelf_id";
	my $cmdout=$Server->dblade(cmd => $cmd);
	my %shelf_info;
	my $shelf_name='';
	my @lines = split(/\n/, $cmdout);
	foreach my $line(@lines) {
		if($line =~ /Shelf name/)
		{
			$shelf_info{'num-paths'}++ ;
			$shelf_info{'name'}=$shelf_id;
			$line=~/\d[a-d]\.shelf\d\d?/;	
			$shelf_name=$&;
		}
		if($line =~ /Shelf UID/)
		{
			$line =~ /([a-f0-9][a-f0-9]:){7}[a-f0-9][a-f0-9]/;
			$uuid=$&;
			$shelf_info{'uuid'}=$uuid unless($shelf_info{'uuid'});
		}

		$shelf_info{'num-disks'}++ if($line =~ /\[\s\s?\d\d?\s\]\sOK/);

		if($line =~ /Module:/)
		{
			my($temp,$module)=split(/:/,$line);
			$module=~/A|B/;
			$module=$&;
			if($module eq 'A')
			{
				my ($adapter,$temp)=split(/\./,$shelf_name);	
				$adapter=~/\d/;
				my $slot=$&;
				$adapter=~/[a-d]/;
				my $port=$&;
				$shelf_info{'ioma-adapter-slot'}=$slot;
				$shelf_info{'ioma-adapter-port'}=$port;
				$shelf_info{'ioma-adapter'}=$adapter;
			}
			if($module eq 'B')
			{
				my ($adapter,$temp)=split(/\./,$shelf_name);	
				$adapter=~/\d/;
				my $slot=$&;
				$adapter=~/[a-d]/;
				my $port=$&;
				$shelf_info{'iomb-adapter-slot'}=$slot;
				$shelf_info{'iomb-adapter-port'}=$port;
				$shelf_info{'iomb-adapter'}=$adapter;
			}
		}
		if($line =~ /Module state/)
		{
			$shelf_info{'status'}='OK' if($line =~ /OK/);
		}
		if($line =~ /Shelf id:/)
		{
			$line=~/\d+/;
			$shelf_info{'id'}=$&;
		}

	}
	$shelf_info{'num-disks'}=$shelf_info{'num-disks'}/2 if($shelf_info{'num-paths'} == 2);	
	return %shelf_info;
}

sub get_disk_info
{
	my $disk_id=shift;
	return 300 unless($disk_id);
	my $cmd="storage show disk $disk_id";
	my $cmdout=$Server->dblade(cmd => $cmd);
	my %disk_info;
	my @lines = split(/\n/, $cmdout);
	$disk_info{'disk-name'}=$disk_id;
	foreach my $line(@lines) {
		if($line =~ /Serial:/)
		{
			my($temp,$slno)=split(/:/,$line);
			chomp $slno;
			$disk_info{'serial-no'}=$slno;	
		}	
		if($line =~ /Sec Name/)
		{
			$line =~ /\d[a-d]\.\d\d?\.\d\d?/;
			$disk_info{'sec-name'}=$&;	
		}	
		if($line =~ /UID/)
		{
			my($temp,$uuid)=split(/D:/,$line);
			chomp $uuid;
			$disk_info{'uuid'}=$uuid;	
		}	
		if($line =~ /Shelf:/)
		{
			$line =~ /\d\d?/;
			$disk_info{'shelf-id'}=$&;	
		}	
		if($line =~ /Bay:/)
		{
			$line =~ /\d\d?/;
			$disk_info{'bay-no'}=$&;	
		}	
	}
	if($disk_info{'sec-name'})
	{
		$disk_info{'num-paths'}=2;
	} else {
		$disk_info{'num-paths'}=1;
	}
	return %disk_info;
}

sub get_adapter_info
{
	my $adapter_id=shift;
	return 300 unless($adapter_id);
	my $cmd="storage show adapter $adapter_id";
	my $cmdout=$Server->dblade(cmd => $cmd);
	my %adapter_info;
	my @lines = split(/\n/, $cmdout);
	$adapter_info{'adapter-name'}=$adapter_id;
	foreach my $line(@lines) {
		if($line =~ /Base WWN:/)
		{
			my($temp,$wwn)=split(/WWN:/,$line);
			chomp $wwn;
			$adapter_info{'wwn'}=$wwn;
		}
		if($line =~ /State:/)
		{
			if($line=~/Enabled/)
			{
				$adapter_info{'is-enabled'}='true';
			} else {
				$adapter_info{'is-enabled'}='false';
			}
		}
		if($line =~ /In Use:/)
		{
			if($line=~/Yes/)
			{
				$adapter_info{'in-use'}='true';
			} else {
				$adapter_info{'in-use'}='false';
			}
		}
		if($line =~ /Slot:/) 
		{
			$line =~ /\d[a-d]/;
			$adapter_info{'slot-num'}=$&;
		}
	}
	return %adapter_info;
}

sub create_test_alert
{
	my $cmd='set -privilege test -confirmations off';
	my $cmdout=$Server->cli(cmd => $cmd);
	my $cmd='system health test -node local -monitor node-connect -command testAlertUtilAdd';
	my $cmdout=$Server->cli(cmd => $cmd);
}

sub delete_test_alert
{
	my $cmd='set -privilege test -confirmations off';
	my $cmdout=$Server->cli(cmd => $cmd);
	my $cmd='system health test -node local -monitor node-connect -command testAlertUtilDelete';
	my $cmdout=$Server->cli(cmd => $cmd);
}


1
