# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Diskinfo.pm#1 $

## @summary Library of 
## @author dl-setupcleanup-libs
## @status Shared
## @pod here

=head1 NAME

Diskinfo.pm - Methods for retrieving disk information from the filer.

=head1 SYNOPSIS

use Diskinfo;

use vars qw($FILER);

use Diskinfo qw(get_non_aggr0_disks get_disk_channels);

param('FILER', '-mesg', "Filer Name");

my $disks = &get_non_aggr0_disks(filer=>$FILER);

my $channels = &get_disk_channels(disks=>$disks);

=head1 DESCRIPTION

This library contains methods to retrieve disk information of a
filer configured in 7mode.

=cut

package Diskinfo;

use strict;
use warnings;
use Tharn qw($Log script_result say);
use Params::Validate qw(validate SCALAR ARRAYREF);
use Utils qw(make_conn make_host);
BEGIN
{
    Params::Validate::validation_options(
                 on_fail => sub {
                                            my @msgs = @_;
                                            chomp(@msgs);
                                            die script_result([@msgs]);
                 }, );
}

require Exporter;
our @ISA=qw(Exporter);
our @EXPORT_OK=qw(get_disks get_aggr0_disks get_non_aggr0_disks get_disk_channels);

=head1 METHODS

=head2 get_disks

This method takes as input the name of the Filer and it returns an arrayref 
which contains the list of disks owned by the filer

=cut

sub get_disks {
    my (%opts) = validate(@_, {
                               filer => { type => SCALAR }
                              }
                          );
    my $filer = $opts{'filer'};
    my $host = make_host($filer);
    my $conn = make_conn($host);
    my $host_id = $host->id();
    my $out;
    (undef,undef,$out) = say($conn ,"disk show -o $host_id");
    my @lines = split(/\n+/,$out);
    splice(@lines,0,2);
    my $disk = &_parse_disks($out,0);
    my @disk_list = @$disk;
    splice(@disk_list,0,2);
    return \@disk_list;
}

=head2 get_aggr0_disks

This method takes as input the name of the Filer and it returns an arrayref
which contains the list of disks assigned to root aggr

=cut

sub get_aggr0_disks {
    my (%opts) = validate(@_, {
                              filer => { type => SCALAR }
                              }
                         );
    my $filer = $opts{'filer'};
    my $host = make_host($filer);
    my $conn = make_conn($host);
    my $out;
    (undef,undef,$out) = say($conn ,"aggr status -r aggr0");
    my $disk = &_parse_disks($out,1);
    my @aggr0_disk_list = @$disk;
    splice(@aggr0_disk_list,0,5);
    return \@aggr0_disk_list;
}

=head2 get_non_aggr0_disks

This method takes as input the name of the Filer and it returns an arrayref
which contains the list of disks owned by the filer ,other than the disk
assigned to root aggr

=cut

sub get_non_aggr0_disks {
    my (%opts) = validate(@_, {
                              filer => { type => SCALAR }
                              }
                         );
    my $filer = $opts{'filer'};
    my(%hash,@disks);
    my $disk_list = &get_disks(filer=>$filer);
    my $aggr0_disk_list = &get_aggr0_disks(filer=>$filer);
    foreach (@$disk_list) {
        $hash{$_} = 1;
    }
    foreach (@$aggr0_disk_list) {
        if (exists($hash{$_})) {
           delete $hash{$_};
        }
    }
    @disks =  keys(%hash);
    return \@disks;
}

=head2 get_disk_channels

This method takes list of disks stored in an arrayref and it returns the disk
channels

=cut

sub get_disk_channels {
    my (%opts) = validate(@_, {
                               disks => { type => ARRAYREF }
                              }
                          );

    my $disks = $opts{'disks'};
    my %channels;
    foreach (@$disks) {
        my @c = split(/\./,$_);
        $channels{$c[0]} = 1;
    }
    my @channels = keys(%channels);
    return \@channels
}


#
#Private routine which parses a list and returns a list of disk names
#
sub _parse_disks {
    my ($lines,$pos) = @_;
    my @lines = split(/\n+/,$lines);
    my (@words,@disk);
    foreach (@lines) {
        s/^\s+//g;s/\s+$//g;
        if($_) {
          @words = split(/\s+/,$_);
          push(@disk,$words[$pos]);
        }
    }
    return \@disk;
}
1;
=head1 STATUS

    Shared

=head1 AUTHOR

    dl-setupcleanup-libs
    $Id: //depot/prod/test/Rsierranevada/lib/Diskinfo.pm#1 $

=head1 COPYRIGHT

    Copyright 2010 Network Appliance Inc., All Rights Reserved.

=cut
