# $Id:$

#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary SnapshotAutodelete Controller Layer Module
## @author dl-san-automation-core
## @status shared
## @pod here

=head1 NAME

EnGenX::Controller::SnapshotAutodelete

=head1 DESCRIPTION

EnGenX::Controller::SnapshotAutodelete is a derived class of
EnGenX::Controller::Task.  It is used to Create and
Delete ONTAP SnapshotAutodeletes. All methods are static.

=cut

package EnGenX::Controller::SnapshotAutodelete;

# Compiler directives
use strict;
use warnings;

# NATE and Standard Module import
use Tharn qw($Log);
use NATE::Util::DataTypes qw(TRUE FALSE);
use Params::Validate
    qw(validate_with validate SCALAR ARRAYREF BOOLEAN UNDEF SCALARREF HASHREF);

# NACL and EnGenX Libs
use NACL::C::VolumeSnapshotAutodelete;
use EnGenX::SubProcess;
use EnGenX::EnGenXUtil;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);

#Inherit base class
use base qw( EnGenX::Controller::Task );

use vars qw(%DTD_To_NACL_Translate
    %DTD_To_NACL_Ignore
    %Delete_Args);

# Arguments after converting from xml to a
# resource list that need their keys translated to
# keys that NACL methods will accept.
%DTD_To_NACL_Translate = (
);

# Arguments that are used by EnGenX but cannot be
# passed into the NACL methods
%DTD_To_NACL_Ignore = (
    parallel => '',
);

# Arguments that can be passed into the NACL delete
# command
%Delete_Args = (
);

=head1 METHODS

=head2 create

(Static method) Request modify of one or more SnapshotAutodeletes serially
or in parallel and return whether the modify was successfully.
This method passes all user arguments through to EnGenX::Controller::Task
and adds specific arguments used by the EnGenX framework.

  EnGenX::Controller::SnapshotAutodelete->create(
            command_interface => $Node,
            volume            => ( $Volume_name),
            'defer-delete'    => $defer_delete_option,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::SnapshotAutodelete->create(
            command_interface => $Node,
            volume            => ( $Volume_name),
            'defer-delete'    => $defer_delete_option,
            'method-timeout'  => 3600,
            count             => 5
            );

  EnGenX::Controller::SnapshotAutodelete->create(
            command_interfac  => $Node,
            resource_list      => @snapshot_autodelete_options,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::SnapshotAutodelete->create(
            command_interface => $Node,
            resource_list      => @snapshot_autodelete_options,
            );

  EnGenX::Controller::SnapshotAutodelete->create(
            resource_list      => @snapshot_autodelete_options,
            'method-timeout'  => 3600,
            );

  EnGenX::Controller::SnapshotAutodelete->create(
            resource_list      => @snapshot_autodelete_options,
            );

=over

=item Options

Either the resource_list is required or the NACL SnapshotAutodelete modify Arguments
are required but both should NOT be used in the same call.

=over

=item C<< resource_list => @resource_list >>

(Optional)
If this is passed in the SnapshotAutodeletes in the resource list will be created
based on the information that is supplied in the resource list. If
the resource list is passed in the NACL SnapshotAutodelete modify Arguments should
NOT be passed in.

=item C<< NACL SnapshotAutodelete modify Arguments >>

(Optional)
These are arguments that can be passed into NACL::C::VolumeSnapshotAutodelete->modify.
If these arguments are passed in the resource_list argument should NOT
be passed in

=item C<< command_interface => $nacl_command_interface >>

(Optional)
This is the command_interface that will be used during NACL calls.  If a
command_interface is not passed in then one will be created.  Passing in a
command_interface helps performance because for every command_interface created
the version of the Filer must be determined by creating a connection to the filer
and then sending commands to get the version.

=item C<< method_timeout => $NACL_method_timeout >>

(Optional)
This is the method_timeout that will be used during NACL calls. If a
method_timeout is not passed in then the method_timeout will be defaulted.

=back

=item Return

This method returns a True or False based on whether the method succeeds or fails
in the creation of the SnapshotAutodelete(s) based on the Options passed in.

=item Exceptions

This method produces and throws no exceptions of its own but pass through any
uncaught exceptions from other library calls.

=cut

sub create {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;

    # Locals
    my $parent_answer;

    # Globals that will also be set as globals in the base class
    $args{'Name'} = "SnapshotAutodelete";

    # Super::_create is in Task.pm
    $parent_answer = $pkg->SUPER::_create(%args);

    return $parent_answer;
} ## end sub create

=head2 delete

Is currently a noop

=cut

sub delete {
 # noop
 return TRUE;
} ## end sub delete


#==============================================================================
# Method:     _nacl_create_call
# Objective:  Makes the raw NACL calls for modification of SnapshotAutodelete.
# Details:    Makes the NACL::C::VolumeSnapshotAutodelete->modify() call. Also
#             works around some common non-fatal errors that can be return from
#             modification and then has a retry mechanism to retry SnapshotAutodelete
#             creation.
# Inputs:     NACL::C::VolumeSnapshotAutodelete->modify arguments
#             retry => integer, is the current retry the method is on.
#                    This is used to count how many times the _nacl_create_call
#                    has been retried so that the retries can be limited.
# Outputs:    Returns whether the modify was successful or a hash with success
# Exceptions: This method produces and throws no exceptions of its own but
#             passes through any uncaught exceptions from other library calls.
#==============================================================================
sub _nacl_create_call {
    $Log->enter();
    my $pkg  = shift;
    my %args = @_;
    # Locals
    my $duplicated = FALSE;
    my $if_exists
        = exists($args{if_exists}) ? delete($args{if_exists}) : 'reuse';
    my $retry = exists $args{retry} ? delete $args{retry} : 0;
    $args{'method-timeout'} = 3600 unless exists($args{'method-timeout'});

    if (!exists($args{command_interface})) {
        my @Nodes = NACL::C::Node->find();
        $args{command_interface} = $Nodes[0];
    }

    # Execute Modify
    try {
        NACL::C::VolumeSnapshotAutodelete->modify(%args);
        $Log->comment("Comment: SnapshotAutodelete modify succeeded for volume:" . $args{volume});
    }
    catch NACL::APISet::Exceptions::ConnectionFailedException with {
        if ($retry < 5) {
            sleep 30 * ($retry++);
            # recursively calling _nacl_create_call to retry create
            $pkg->_nacl_create_call(
                                %args,
                                retry     => $retry,
                                if_exists => $if_exists,
                            );
        } else {
            my $exception = shift;
            $Log->exit();

            # Rethrowing caught exception
            $exception->throw();
        }
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception = shift;
        if ($exception->text() =~ /Authorization failed/ && $retry < 5) {
            sleep 30 * ($retry++);
            # recursively calling _nacl_create_call to retry create
            $pkg->_nacl_create_call(
                                %args,
                                retry     => $retry,
                                if_exists => $if_exists
                                );

        } else {
            $Log->exit();

            # Rethrowing caught exception
            $exception->throw();
        }
    };

    $Log->exit();
    # Returning either a list (hash) or scalar TRUE
    return (wantarray()) ? ('success'=>TRUE, 'duplicated'=>$duplicated) : TRUE;


} ## end sub _nacl_create_call

#========================================================================================
# Method:     _nacl_delete_call
# Objective:  Noop
# Details:    This is currently a noop
#==============================================================================
sub _nacl_delete_call {
    # Noop
    return TRUE;
} ## end sub _nacl_delete_call

#==============================================================================
# Method:     _nacl_wait_call
# Objective:  Noop
# Details:    This is currently a noop
#==============================================================================
sub _nacl_wait_call {
    # noop
    return TRUE;
}

#==============================================================================
# Method:     _get_dtd_nacl_hashes (translate, ignore and delete)
# Objective:  Getter methods for the variables of the same name.
# Details:    Retrieves the hash or a reference to one of the following:
#             - DTD_To_NACL_Translate.
#             - DTD_To_NACL_Ignore.
#             - Delete_Args.
# Inputs:     none.
# Outputs:    The reference or hash variable.
# Exceptions:
#==============================================================================
sub _get_dtd_nacl_translate {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Translate : \%DTD_To_NACL_Translate;
}
#==============================================================================
sub _get_dtd_nacl_ignore {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %DTD_To_NACL_Ignore : \%DTD_To_NACL_Ignore;
}
#==============================================================================
sub _get_dtd_nacl_delete {
    $Log->enter();
    my $pkg  = shift;
    my %opts = @_;

    $Log->exit();
    return (wantarray()) ? %Delete_Args : \%Delete_Args;
}

1;
