# $Id: //depot/prod/test/main/lib/NACL/Exceptions/SubProcessFailure.pm#1 $
#
# Copyright (c) 2009-2011 NetApp Inc.
# All rights reserved.
#
## @summary Provide an exception class that is a subclass of
##          NATE::BaseException that will be thrown when an instance of
#           Process.pm implemented in SubProcess.pm encounters an error
#           and needs to return the resource list it was working.
##
## @author jcoca@netapp.com, dl-san-automation
## @status Shared
## @pod here

=head1 NAME

EnGenX::Exceptions::SubProcessFailure

=head1 DESCRIPTION

This exception is thrown when an instance of NATE::Process implemented
in EnGenX::SubProcess.pm encounters an error and needs to gracefuly exit
and return the resource list it was working.

=head1 ATTRIBUTES

=head2 resource

In addition to the attributes that exception objects generally contain,
this exception also contains an C<resource_list> attribute. This is
an array-reference that will contain all the resources that were process.

=head1 EXAMPLE

   use EnGenX::Exceptions::SubProcessFailure qw(:try);

   try {
        @answer = $EGX_SubProc_Obj->parallel_execution(
                                        'list'=> \@resource_list,
                                        'on_error'=> "finish",
                                    );
	} catch EnGenX::Exceptions::SubProcessFailure with {
        my $xptn = shift;
        @resource_list = @($xptn->{resource});
	};

=cut

package EnGenX::Exceptions::SubProcessFailure;


use base qw(NATE::BaseException);
# Adding the key name (resource) of type (type) for this exception.
# This will enable us to add the resource list to the thrown exception.
use Class::MethodMaker [ scalar => 'resource', ];

1;
