# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary An Event Module for Unit Testing the EventMan library
## @author hrivera@netapp.com
## @status private
## @pod here

package EventMan::EventModules::LibraryUnitTest;

use Tharn;
use NATE::IPC::Socket;
use San::SanUtil;
use Data::Dumper;
use ParamsMgr;

###############################################################################
#                               Internal Methods
#                     (WARNING: NEVER INVOKE THESE METHODS)
###############################################################################

###############################################################################
# Method:    __register_event_module()
# Objective: Register this Event Module with the EventMan library
###############################################################################
sub __register_event_module {
    my ($self) = @_;

    my @event_modules = (__PACKAGE__,
                         "EventMan::EventModules");

    $self->EventUtil::register_event(-event_name => "libunittest_valid",
                                     -event_modules => \@event_modules,
                                     -description => "This represents a Valid event",
                                     -exec => sub { &valid_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_sleep30",
                                     -event_modules => \@event_modules,
                                     -description => "This will sleep for 30 seconds",
                                     -exec => sub { &sleep30_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_sleep300",
                                     -event_modules => \@event_modules,
                                     -description => "This will sleep for 300 seconds",
                                     -exec => sub { &sleep300_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_fail",
                                     -event_modules => \@event_modules,
                                     -description => "This will fail on purpose",
                                     -exec => sub { &fail_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_paramsmgr_fail",
                                     -event_modules => \@event_modules,
                                     -description => "This will fail on purpose using paramsmgr",
                                     -exec => sub { &fail_event_paramsmgr(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_success",
                                     -event_modules => \@event_modules,
                                     -description => "This will end successfully",
                                     -exec => sub { &success_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_resource_validation",
                                     -event_modules => \@event_modules,
                                     -description => "This will test resource requirements",
                                     -required_resources => ["required_resource1", "required_resource2"],
                                     -optional_resources => ["optional_resource1"],
                                     -require_at_least_one_resource_types => ["require_one_resource1", "require_one_resource2"],
                                     -resource_counts => { "required_resource1" => "2+",
                                                           "required_resource2" => "2",
                                                           "require_one_resource1" => "2-",
                                                           "require_one_resource2" => "1-" },
                                     -exec => sub { },
                                     -validate => sub { print "validate method\n"; } );
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_sync",
                                     -event_modules => \@event_modules,
                                     -description => "This will perform event synchronizations",
                                     -exec => sub { &sync_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_job_management",
                                     -event_modules => \@event_modules,
                                     -description => "This will test all pre/post/cojob/init/uninit functionality",
                                     -prejob => sub { $self->SubtestComm::log_event_message(-event_message => "PREJOB"); },
                                     -postjob => sub { $self->SubtestComm::log_event_message(-event_message => "POSTJOB"); },
                                     -cojob => sub { $self->SubtestComm::log_event_message(-event_message => "COJOB"); },
                                     -init => sub { $self->SubtestComm::log_event_message(-event_message => "INIT"); },
                                     -uninit => sub { $self->SubtestComm::log_event_message(-event_message => "UNINIT"); },
                                     -exec => sub { &job_management_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_validate",
                                     -event_modules => \@event_modules,
                                     -description => "This will perform custom event validations",
                                     -validate => sub { print "i am validating myself\n" },
                                     -exec => sub { &validate_event(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_resmgr_obj_updates",
                                     -event_modules => \@event_modules,
                                     -description => "This will test resource object update propagations",
                                     -exec => sub { &resmgr_obj_updates(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_resmgr_obj_attr_dump",
                                     -event_modules => \@event_modules,
                                     -description => "This will test resource object attr synchronizations",
                                     -exec => sub { &resmgr_obj_attr_dump(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_dump_self",
                                     -event_modules => \@event_modules,
                                     -description => "This will dump the eventman object",
                                     -exec => sub { &dump_self(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }


    $self->EventUtil::register_event(-event_name => "libunittest_syslogging",
                                     -event_modules => \@event_modules,
                                     -description => "This will test the SysLogging Utility Module",
                                     -exec => sub { &syslogging_test(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    $self->EventUtil::register_event(-event_name => "libunittest_toggle_faulted_unfaulted",
                                     -event_modules => \@event_modules,
                                     -required_resources => ["sanfilersCm"],
                                     -description => "This will toggle faulted/unfaulted on all sanfilersCm",
                                     -exec => sub { &toggle_faulted_unfaulted(@_); });
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    # define the default values for Event Parameters
    $self->EventUtil::register_event_module_defaults(-event_module => __PACKAGE__,
                                                     -arg_defaults =>
                                                         { -param1 => "param1",
                                                           -param2 => "param2",
                                                           -param3 => "param3",
                                                           -param4 => "param4",
                                                           -param5 => "param5",
                                                           -param6 => "param6", }
                                                     );
    if ($self->error()) {
        return $EventUtil::FALSE;
    }

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    _init()
# Objective: Initialize this Event Module
###############################################################################
sub __init {
    my ($self) = @_;

    my $obj = {value => "i have not been updated"};

    $self->ResMgr::add_resource(-name => "dummy_resource",
                                -type => "_GLOBALS",
                                -obj => $obj);

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    _uninit()
# Objective: Uninitialize this Event Module
###############################################################################
sub __uninit {
    return $EventUtil::TRUE;
}

###############################################################################
#                               Event Methods
###############################################################################

###############################################################################
# Method:    valid_event()
# Objective: A Valid NOOP event
###############################################################################
sub valid_event {
    my ($self, %args) = @_;

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    sleep30_event()
# Objective: an event that sleeps for 30 seconds
###############################################################################
sub sleep30_event {
    my ($self, %args) = @_;

    print Dumper($self);

    &SanUtil::nate_sleep(30);

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    sleep300_event()
# Objective: an event that sleeps for 300 seconds
###############################################################################
sub sleep300_event {
    my ($self, %args) = @_;

    print Dumper($self);

    &SanUtil::nate_sleep(300);

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    fail_event()
# Objective: an event that issues a failure
###############################################################################
sub fail_event {
    my ($self, %args) = @_;

    print Dumper($self);

    $self->_set_err("Event has failed on purpose");

    return $EventUtil::FALSE;
}

###############################################################################
# Method:    fail_event_paramsmgr()
# Objective: an event that issues a failure
###############################################################################
sub fail_event_paramsmgr {
    my ($self, %args) = @_;

    my $sub = sub {
        my ($subself, %subargs) = @_;
        &ParamsMgr::process_args(%subargs);
        my $p1 = &ParamsMgr::get_arg(-arg => "required_param1",
                                     -required => 1);
        my $p2 = &ParamsMgr::get_arg(-arg => "ref_type_param1",
                                     -default => [],
                                     -ref_type => "ARRAY");
        if (&ParamsMgr::error()) {
            $subself->_set_err(&ParamsMgr::get_error());
            return $EventUtil::FALSE;
        }
    };

    $self->$sub(-required_param1 => "value",
                -ref_type_param1 => ["hello", "hello2"]);
    if ($self->error()) {
        $self->_set_err("Unexpected failure\n");
        return $EventUtil::FALSE;
    }

    $self->$sub(-ref_type_param1 => ["hello", "hello2"]);
    if ($self->error()) {
        $self->_set_err("Asserted failure");
    } else {
        $self->_set_err("Test did not fail properly");
    }

    print Dumper($self);

    return $EventUtil::FALSE;
}

###############################################################################
# Method:    success_event()
# Objective: an event that is successful and quick
###############################################################################
sub success_event {
   my ($self, %args) = @_;

   print Dumper($self);

   return $EventUtil::TRUE;
}

###############################################################################
# Method:    dump_self()
# Objective: an event that is dumps the eventman object
###############################################################################
sub dump_self {
   my ($self, %args) = @_;

   print Dumper($self);

   return $EventUtil::TRUE;
}

###############################################################################
# Method:    sync_event()
# Objective: an event that tests synchronization between event sets
###############################################################################
sub sync_event {
    my ($self, %args) = @_;

    print Dumper($self);

    $self->EventUtil::synchronize();

    $self->EventUtil::synchronize();

    $self->EventUtil::synchronize();

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    job_management_event()
# Objective: an event that tests job management subroutines
###############################################################################
sub job_management_event {
    my ($self, %args) = @_;

    print Dumper($self);

    &SanUtil::nate_sleep(20);

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    validate_event()
# Objective: an event that tests validation methods on events
###############################################################################
sub validate_event {
    my ($self, %args) = @_;

    print Dumper($self);

    &SanUtil::nate_sleep(10);

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    resmgr_obj_updates
# Objective: an event that tests that resmgr updates propagate correctly
###############################################################################
sub resmgr_obj_updates {
    my ($self, %args) = @_;

    my $resource = $self->ResMgr::get_resource(-name => "dummy_resource",
                                               -type => "_GLOBALS");
    print "Dummy resource before update: " . Dumper($resource);

    my $obj = {value => "i have been updated"};

    $self->ResMgr::modify_resource_obj(-name => "dummy_resource",
                                       -type => "_GLOBALS",
                                       -value => $obj);
    if ($self->error()) {
        $self->_set_err("Failed to modify resource obj");
        return $EventUtil::FALSE;
    }

    $self->ResMgr::set_resource_attribute(-name => "dummy_resource",
                                          -type => "_GLOBALS",
                                          -attribute => "hello",
                                          -value => "hi");

    $self->ResMgr::set_faulted_resource(-name => "dummy_resource",
                                        -type => "_GLOBALS");

    $resource = $self->ResMgr::get_resource(-name => "dummy_resource",
                                            -type => "_GLOBALS");
    print "Dummy resource after update: " . Dumper($resource);


    $self->EventUtil::synchronize();
    if ($self->error()) {
        $self->_set_err("Error while sync'ing");
        return $EventUtil::FALSE;
    }

    $resource = $self->ResMgr::get_resource(-name => "dummy_resource",
                                            -type => "_GLOBALS");
    print "Dummy resource after sync: " . Dumper($resource);

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    resmgr_obj_attr_dump
# Objective: an event that tests that resmgr updates propagate correctly
###############################################################################
sub resmgr_obj_attr_dump {
    my ($self, %args) = @_;

    $self->EventUtil::synchronize();
    if ($self->error()) {
        $self->_set_err("Error while sync'ing");
        return $EventUtil::FALSE;
    }

    print Dumper($self);

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    syslogging_test()
# Objective: an event that tests possitively the syslogging mechanism
###############################################################################
sub syslogging_test {
    my ($self, %args) = @_;

    print Dumper($self);

    $self->SysLogging::syslog_msg_all(-message => ["Testing syslogging"]);
    if ($self->error()) {
        $self->_set_err("Syslogging failed");
        return $EventUtil::FALSE;
    }

    return $EventUtil::TRUE;
}

###############################################################################
# Method:    toggle_faulted_unfaulted()
# Objective: an event that tests that faulted/unfaulted toggling behaves properly
###############################################################################
sub toggle_faulted_unfaulted {
    my ($self, %args) = @_;


    foreach my $sanfilerCm ($self->EventUtil::get_sanfilers()) {
        $self->ResMgr::set_faulted_resource(-name => $sanfilerCm,
                                            -type => "sanfilersCm",
                                            -propagate => $EventUtil::TRUE);
        if ($self->error()) {
            $self->_set_err("Failed");
            return $EventUtil::FALSE;
        }
        $self->ResMgr::unset_faulted_resource(-name => $sanfilerCm,
                                              -type => "sanfilersCm",
                                              -propagate => $EventUtil::TRUE);
        if ($self->error()) {
            $self->_set_err("Failed");
            return $EventUtil::FALSE;
        }
    }

    return $EventUtil::TRUE;
}

1;
