#Hey emacs, this is basically -*- perl -*-

#
#
package ExecTC::Action;

use strict;
no strict 'refs';
use Getopt::Long;
use Data::Dumper;

use TracePkg;
use ExecTC::Step;
use Server;

use Tharn;
use Utils;
use SayWrap;


my $t = TracePkg->new();


sub new {

    $t->entry(msg=>"With args: " . join(",", @_) );
    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    #  a hash.
    my ($self, %arg) = @_;

    # Fill the member vars for this instance
    my $object = {
	
	_cmd        => $arg{Cmd},
	_refId      => $arg{RefId},
	_host       => $arg{Host},
	_fields     => $arg{Fields},
	_verify     => $arg{Verify},
	_tc         => $arg{TC},
	_deleteId   => $arg{Delete},
	_cleanUp    => $arg{CleanUp},
	_onError    => $arg{OnError},
	_revert     => $arg{Revert},
	_timeOut    => $arg{TimeOut} || "120",
	_fullCmd    => undef,
	_vFullCmd   => undef,
	_output     => undef,
	_status     => undef,
	_server     => undef,
	_primeObj   => undef,
	_outHash    => undef,
    };

    # This tags the hash pointed to by $object as a 
    #  type of Table
    bless $object, $self;

    # If this is a Delete Action, make it look like a CleanUp Action
    # instead.  This will cause the PreDelete from the Object to get 
    # driven.  But since there's no CleanUp definition in the tc, it
    # won't get used for automatic CleanUp on exit.
    # And, the deleteId is really the same as the refId, just saving the
    # user from some extra type (RefId => .....)
    if ( defined $object->{_deleteId} ) {
	$t->debug(msg=>"Processing a Delete Action");

	# Fisrt setup the _cmd to trigger CleanUp processing in execute
	$object->{_cmd} = "-CLEANUP-";

	# Now set the RefId of the object to delete
	$object->{_cleanUp}{Objects} = $object->{_deleteId};
	$object->{_refId} = $object->{_deleteId};

	# Finally, set the Object's state to maybe, we'll always try to delete
	$object->{_tc}->getObject( $object->{_deleteId} )->setState(state=>"maybe");
	$object->{_tc}->getObject( $object->{_deleteId} )->setTestCreated();
    }

    # Build the action using the RefId obj and Fields.
    if ( defined $object->{_cmd} && $object->{_cmd} ne "-CLEANUP-") {
	_buildCmd( $object );
    }

    # Build the verify action using the RefId obj and Fields.
    if ( defined $object->{_verify} ) {
	_buildvCmd( $object );
    }

    # Build the CleanUp section for this testcase
    if ( defined $object->{_cleanUp} ) {
	_buildCleanUp( $object );
    }

    $object->{_server} = Server->new();

    $t->exit(msg=>"cmd: $object->{_fullCmd}, to object: $object->{_refId}\n");
    # And give the caller the referece to the bless object.
    return $object;
}


sub _buildCmd {
    my ($self) = @_;

    $t->entry(msg=>"Building cmd: $self->{_cmd}\n");

    my ($cmd, $key, $value);

    # Get the object needed for this Action.
    my $refObj = $self->{_tc}->getObject( $self->{_refId} );
	
    # For now, let's just put the AUTOLOADer version of the command
    # in some instance variables
    $self->{_cliMethod} = $refObj->getCmdDir() . " " . $self->{_cmd};
    $self->{_cliMethod} =~ s/[ -]/_/g;

    # Build command defined within this action.
    if ( $self->{_cmd} eq "delete" ) { 
	$cmd = $refObj->getCmdDir() . " " . $self->{_cmd} . " " .
	    _addKeys( $self );
	# Build the list that contains only the key fields and values
	%{$self->{_cliArgs}} = $self->_buildArgList();

    } else {
	$cmd = $refObj->getCmdDir() . " " . $self->{_cmd} . " " .
	    _addKeys( $self ) . _addFields( $self );
	# Build the list that contains all fields and values
	%{$self->{_cliArgs}} = $self->_buildArgList( nonKeys => 1 );
    }    
    $t->debug(msg=>"_cliMeth is:\n" . Dumper( $self->{_cliMethod} ));    
    $t->debug(msg=>"_cliArg is:\n" . Dumper( $self->{_cliArgs} ));    

    $self->{_fullCmd} = $cmd;
    $t->debug(msg=>"Actions will be run on host: $self->{_host}");
    $t->exit(msg=>"Cmd built: $self->{_fullCmd}\n");
}


sub _buildArgList {
    my ($self, %arg) = @_;

    my %cliArgs;

    # Get the object needed for this Action.
    my $refObj = $self->{_tc}->getObject( $self->{_refId} );
    
    # Add the keys for this object
    if ( %cliArgs = %{$refObj->getKeys()} ) {
	$t->debug(msg=>"Added keys to arg list:\n" . Dumper(%cliArgs));
    }

    if ( $arg{nonKeys} ) { 
	# Now add the fields (first from the Action, then from the Object
	if ( defined $self->{_fields} ) {
	    # Grab local fields if defined
	    %cliArgs = ( %cliArgs , %{$self->{_fields}} );
	} elsif ( defined $refObj->getFields() ) { 
	    # No locally defined fields, add the fields from this object
	    %cliArgs = ( %cliArgs , %{$refObj->getFields()} );
	}
	$t->debug(msg=>"Added non-keys to arg list:\n" . Dumper(%cliArgs));
    }
    return %cliArgs;

}


sub _buildvCmd {
    my ($self) = @_;

    $t->entry(msg=>"Building verify:$self->{_verify}{State} cmd for: $self->{_cmd}\n");

    my ($cmd, $key, $value);

    # Get the object needed for this Action.
    my $refObj = $self->{_tc}->getObject( $self->{_refId} );
	
    # Build a verifying "show" command
    $cmd = $refObj->getCmdDir() . " " . "show" . " " .
	_addKeys( $self );
    $self->{_cliVMethod} = $refObj->getCmdDir() . " show";
    $self->{_cliVMethod} =~ s/[ -]/_/g;
    %{$self->{_cliVArgs}} = $self->_buildArgList();
    
    $t->debug(msg=>"_cliVMeth is:\n" . Dumper( $self->{_cliVMethod} ));    
    $t->debug(msg=>"_cliVArg is:\n" . Dumper( %{$self->{_cliVArgs}} ));    

    $self->{_vFullCmd} = $cmd;
    $t->exit(msg=>"Verify cmd built: $self->{_vFullCmd}\n");
}


# Add the Keys 
sub _addKeys {
    my ($self) = @_;
    my ($cmd, $key, $value);

    # Get the object needed for this Action.
    my $refObj = $self->{_tc}->getObject( $self->{_refId} );
	
    # Add the keys for this object
    if ( $refObj->getKeys() ) {
	while ( ($key,$value) = each %{$refObj->getKeys()} ) {
	    $cmd .= " -$key $value";
	}
    }
    return $cmd;
}


# Add the non-Keys 
sub _addFields {
    my ($self) = @_;
    my ($cmd, $key, $value);

    # Get the object needed for this Action.
    my $refObj = $self->{_tc}->getObject( $self->{_refId} );
	
    # Add the fields for this action
    if ( defined $self->{_fields} ) {
	# Grab local fields if defined
	while ( ($key,$value) = each %{$self->{_fields}} ) {
	    if ( $value =~ /^-$/ ) {
		$value = "\"$value\"";
	    }
	    $cmd .= " -$key $value";
	}
    } elsif ( defined $refObj->getFields() ) { 
	# No localll defined fields, add the fields from this object
	while ( ($key,$value) = each %{$refObj->getFields()} ) {
	    $cmd .= " -$key $value";
	}
    }
    return $cmd;
}


# In a CleanUp Action, we need to run the list of createdObjs or the
# list of defined objects in the CleanUp section.  We are looking
# for ones that are in a "exist", and delete them.
sub _buildCleanUp {
    my ($self) = @_;

    $t->entry(msg=>"Build CleanUp action");
    $self->{_cmd} = "-CLEANUP-";
    $self->{_verify}{State} = "missing";
    if ( exists $self->{_cleanUp}{TimeOut} ) {
	$self->{_timeOut} = $self->{_cleanUp}{TimeOut};
	$t->debug(msg=>"TimeOut setting: ".$self->{_timeOut});
    }
    if ( exists $self->{_cleanUp}{Attempts} ) {
	$self->{_verify}{Attempts} = $self->{_cleanUp}{Attempts};
	$t->debug(msg=>"Attempts setting: ".$self->{_verify}{Attempts});
    }
    if ( exists $self->{_cleanUp}{Interval} ) {
	$self->{_verify}{Interval} = $self->{_cleanUp}{Interval};
	$t->debug(msg=>"Interval setting: ".$self->{_verify}{Interval});
    }

    # There isn't a single _fullCmd for this Action, instead
    # the execute() method will loop through the defined Objects
    # or the object that currently "exist".
    if ( defined $self->{_cleanUp}{Objects} ) {
	$self->{_cleanUpObjs} = $self->{_cleanUp}{Objects};
    }
    $t->exit(msg=>"Built CleanUp action");
}    


sub getName {
    my ($self, %arg) = @_;
    return $self->{_cmd};
}


sub setFields {
    my ($self, %arg) = @_;
    $self->{_fields} = $arg{fields};
}


sub execute {
    my ($self, %arg) = @_;
    my $timeOut = $arg{timeOut};
    my $no_save = $arg{no_save};
    my ($output, $outArray, $before, $after);
    my ($attempts, $interval);

    $t->entry(msg=>"Running action: $self->{_fullCmd}, meth: $self->{_cliMethod}, on host: $self->{_host}");

    # If a specific timeout isn't passes, use the value from construction time
    if ( defined $timeOut ) { 
	$t->debug(msg=>"Resetting timeout to: $timeOut");
    } else {
	$timeOut = $self->{_timeOut}; 
	$t->debug(msg=>"Use timeout from init: $timeOut");
    }

    # If this is a CleanUp Action, delete the obj and return
    if ( $self->{_cmd} eq "-CLEANUP-" ) {

	return _cleanUp( $self );
    }

    # If they don't want auto CleanUp set the no_save flag
    if ( $self->{_revert} eq "none" ) {
	$no_save = 1;
    }

    # If this is modify command, save the original state of the object
    # for later reverting
    if ( $self->{_cmd} eq "modify" && !$no_save) {
	
	my $obj = $self->{_tc}->getObject($self->{_refId});
	# To save the state of an object for later revert, we need to
	# know which fields to save.  The locally defined fields and
	# the object fields (if any) are used.
	my $currFields = $obj->saveState( fields => $self->{_fields},
					  state => $obj->getState() );

	# If there are any dupFields, delete them from the _fields Hash
	# and remove them from the already built modify command.
	if ( scalar keys %{$currFields} > 0 ) {
	    $t->debug(msg=>"Only persisted fields: " .
		      join(",", keys %{$currFields}) );
	    foreach my $key ( keys %{$self->{_fields}} ) {
		if ( !defined $currFields->{$key} ) {
		    $t->debug(msg=>"Deleting field $key, since its " .
			      "value hasn't changed");
		    delete $self->{_fields}{$key};

		    # Now adjust the actual command 
		    $self->{_fullCmd} =~ s/-$key \S+//;
		}
	    }
	} else {
	    $self->{_status} = "PASSED";
	    $self->{_output} = "Command not modifying anyhting, NOP";
	    $t->debug(msg=>"No fields are different, skip modify");
	    $t->exit(msg=>"Action complete: $self->{_cmd}, status: $self->{_status}");
	    return $self->{_status};
	}	
    }    

    # Run this step's Action
    eval { 
	$output = undef;
	$outArray = undef;
#	$self->{_server}->cli(cmd => $self->{_fullCmd}, 
#			      output => \$output,
#			      "-Server" => $self->{_host},
#			      "-Timeout" => $timeOut );
	my $meth = $self->{_cliMethod};
	# Skip verification on non-show commands, this is needed since
	# 'verification' is too closely tied with getting a proper
	# array of hash values back
	my $doVerify = ( $meth =~ /show$/ ) ? 1 : 0;
	$outArray = $self->{_server}->$meth( %{$self->{_cliArgs}},
				 output => \$output,
				 "-Server" => $self->{_host},
				 "-Verification" => $doVerify,
				 "-Timeout" => $timeOut );

    };
    my $serverResult = $@;
    if (_cmdErrorCheck( result => $serverResult,
			cmdOut => $output) ) {
	$t->debug(msg=>"Error returned from Server cmd:\n$@");
	$self->{_output} = $output;
	$self->{_output} .= "\n$serverResult";
	$self->{_status} = "FAILED";
	$self->{_errMsg} = $self->{_output};

	# The command timedout, and was a new, mark the object as 'maybe'
	if ( $self->{_cmd} eq "new") {
	#if ( _cmdErrorTimeout(cmdOut=>$output) && $self->{_cmd} eq "new") {
	    # Let's assume the object was created 
	    #(so save that fact for later cleanup)
	    $self->{_tc}->getObject($self->{_refId})->setState(state => "maybe") ;
	    $self->{_tc}->getObject($self->{_refId})->setTestCreated() ;
	    $self->{_tc}->pushCreatedObj( object => $self->{_refId} );
	}
    } else {
	
	# The didn't error, and was a new, mark the object as 'maybe'
	if ( $self->{_cmd} eq "new") {
	    # Let's assume the object was created 
	    #(so save that fact for later cleanup)
	    $self->{_tc}->getObject($self->{_refId})->setState(state => "maybe") ;
	    $self->{_tc}->getObject($self->{_refId})->setTestCreated() ;
	    $self->{_tc}->pushCreatedObj( object => $self->{_refId} );
	}
	# No error, and we expected the obj to exist, grab the output and convert to a hash
	if ( !_cmdErrorCheck( result => $serverResult, cmdOut => $output) && 
	     $self->{_cmd} eq "show" &&
	     $self->{_verify}{State} ne "missing" ) {
	    $self->{_outHash} = $outArray; 
	    $t->debug(msg=>"_outHash is: " . Dumper($self->{_outHash}));
	}
	
      VERIFY: {
	  $t->debug( msg=>"Cmd returned: \n$output\n");
	  
	  # Nothing seemingly bad, now verify the last command.
	  if ( defined $self->{_verify}{State} ) {
	      $t->debug( msg=>"Running verifier: $self->{_verify}{State}\n");
	      
	      # If they want us to wait for the field to change, setup the
	      # loop variables
	      if ( $self->{_verify}{Attempts} ) {
		  if ( $self->{_verify}{Interval} ) {
		      $interval = $self->{_verify}{Interval};
		  } else {
		      $interval = 5;
		  }
	      } else {
		  $self->{_verify}{Attempts} = 1;
	      }
	      $attempts = 1;
	      
	      while ( $attempts <= $self->{_verify}{Attempts} ) {
		  $t->debug(msg=>"Running verifier, attempt $attempts out of " .
			    $self->{_verify}{Attempts} );
		  
		  eval {
		      $output = undef;
		      $outArray = undef;
#		      $self->{_server}->cli(cmd=>$self->{_vFullCmd}, 
#					    output=>\$output,
#					    "-Server"=>$self->{_host},
#					    "-Timeout" => $timeOut );
		      my $meth = $self->{_cliVMethod};
		      $outArray = $self->{_server}->$meth( %{$self->{_cliVArgs}},
					       output => \$output,
					       "-Server" => $self->{_host},
					       "-Verification" => 1,
					       "-Timeout" => $timeOut );

		  };
		  $serverResult = $@;
		  $self->{_output} = $output;
		  if ($serverResult) {
		      $t->debug(msg=>"Error returned from Server cmd:\n$@");
		  }
		  if ( _cmdErrorCheck( result => $serverResult,
				       cmdOut => $self->{_output}) ) {
		      $self->{_status} = "FAILED";
		      $self->{_output} .= "\n$serverResult";
		      $self->{_errMsg} = $self->getOutput();
		      # Consider cmd errors more fatal than field verification
		      # failures, so dump out here.
		      last VERIFY;
		  } 		  
		  
		  if ( !(_verify( $self, outString => $output, outArray => $outArray )) ) {
		      $attempts++;
		      # We didn't verify, more chances left?
		      if ( $attempts > $self->{_verify}{Attempts} ) {
			  # Nope
			  $t->debug(msg=>"No remaining verify attempts left");
			  $self->{_status} = "FAILED";
			  last VERIFY;
		      } else {
			  # We have more chances
			  sleep( $self->{_verify}{Interval} );
		      }
		  } else {
		      # We did verify
		      $self->{_status} = "PASSED";
		      last VERIFY;
		  }
	      }
	  } else {
	      $self->{_status} = "PASSED";
	      $t->debug(msg=>"No verifier, and no 'error' string, status: " . 
			$self->{_status});
	  }
      }    # last VERIFY

	$t->debug(msg=>"Verifier complete, status: " . $self->{_status});
	
	# If this is a "new" command, set the state of the object
	if ( ($self->{_cmd} eq "new" || $self->{_cmd} eq "modify") &&  
	     ($self->{_status} eq "PASSED") ) {

	    # If this is a new command, mark the object as exists
	    if ( $self->{_cmd} eq "new") {
		$self->{_tc}->getObject($self->{_refId})->setState(state => "exists") ;
		$self->{_tc}->getObject($self->{_refId})->setTestCreated() ;
		# Save this now created object
		$self->{_tc}->pushCreatedObj( object => $self->{_refId} );
	    } elsif ( $self->{_cmd} eq "modify" ) {
		$self->{_tc}->getObject($self->{_refId})->setState(state => "modified") ;
		if ( !$no_save &&
		     !$self->{_tc}->getObject($self->{_refId})->wasTestCreated() ) { 
		    $self->{_tc}->pushCreatedObj( object => $self->{_refId} );
		}
	    }

	    $t->debug(msg=>"Set obj: " . $self->{_refId} . ", to state: " .
		      $self->{_tc}->getObject($self->{_refId})->getState() );
	}
    }

    # Don't error out if we don't care about errors
    if ($self->{_onError} eq "continue" &&
	$self->{_status} eq "FAILED" ) {
	$t->warn(msg=>"Continuing on error:\n" . $self->{_output});
	
	# Resett the status since we don't want to fail
	$self->{_status} = "PASSED";
    } 
    
    $t->exit(msg=>"Action complete: $self->{_cmd}, status: $self->{_status}");
    return $self->{_status};
}


sub _cleanUp {
    my ($self, %arg) = @_;

    my ($createdObjRef, $createdObj);
    my ($modifyAction, $oldFields);
    my $result = "PASSED";   # Oh this is so bad, I know.

    $t->debug(msg=>"Processing a CleanUp action");
    # If there is a list of defined objects
    if ( defined $self->{_cleanUp}{Objects} && 
	 $self->{_cleanUp}{Objects} ne "--ALL--" ) {
	
	$t->debug(msg=>"Running a clean up for each object");
	for $createdObjRef ( $self->{_cleanUp}{Objects} ) {
	    $createdObj = $self->{_tc}->getObject( $createdObjRef );

	    # Does the object exist?
	    if ( $createdObj->getState() ne "exists" &&
		 $createdObj->getState() ne "maybe") {
		
		$t->debug(msg=>"Object not in 'exists' or 'maybe' state, " .
			  "skipping: " . $createdObj->getName());
		next;
	    }
	    # Run the Object's delete Action
	    $result = _deleteObj( $self, obj => $createdObj );	    
	    if ( $result eq "FAILED" ) {
		if ( $createdObj->getState() eq "exists" ) {
		    $t->debug(msg=>"Failed trying to delete, set to state " .
			      "'maybe', object: $createdObjRef");
		    $createdObj->setState(state => "maybe");
		}
		last;
	    } 
	}
    } else {
	# There are no defined objects to clean up, run through
	# all the existing ones
	$t->debug(msg=>"Running a clean up for ALL objects");

	while ( $createdObjRef = $self->{_tc}->popCreatedObj() ) {
	    if ( !$createdObjRef ) { next; }
	    $createdObj = $self->{_tc}->getObject( $createdObjRef );
	    
	    if ( $createdObj->getState() ne "exists" &&
		 $createdObj->getState() ne "maybe" && 
		 $createdObj->getState() ne "modified" ) {
		$t->debug(msg=>"Object doesn't exist, skip it: " . 
			  $createdObj->getId()); 
		next;
	    }

	    $t->debug(msg=>"Object exists, clean it up: " . 
		      $createdObj->getId()); 
	    
	    # Modify to all states in the delta stack
	    while ( $oldFields = $createdObj->popDelta() ) {
		if ( !$oldFields ) { next; }

		$t->debug(msg=>"Running a modify-cleanup for: " .
			  $createdObj->getId());
		$t->debug(msg=>"Moving to state: " .
			  Dumper( $oldFields) );

		# Now build the full modify command (this needs to be done
		# at runtime, versus initial parse time.
		$modifyAction = $createdObj->getModifyAction();		    
		$modifyAction->setFields( fields => $oldFields );
		$modifyAction->setFullCmd( fullCmd => 
					   $createdObj->getCmdDir() . " " . 
					   $modifyAction->{_cmd} . " " .
					   $modifyAction->_addKeys() . 
					   $modifyAction->_addFields() );
		$modifyAction->setFullCliCmd( cmdDir => $createdObj->getCmdDir(),
					      cmd    => $modifyAction->{_cmd},
					      nonKeys => 1);
		# Set the Attempts and Interval settings in the ModifyAction to
		# match those of the CleanUp Step
		$modifyAction->setAttempts( $self->{_cleanUp}{Attempts} );
		$modifyAction->setInterval( $self->{_cleanUp}{Interval} );
		$result = $modifyAction->execute( no_save=>1);
		
		if ( $result eq "FAILED" ) {
		    $t->debug(msg=>"Failed trying to modify: " .
			      "$createdObjRef");
		    last;
		}
	    }
	    
	    # Delete the object if we created it.  If we just modified it, leave it at prime.
	    if ( $createdObj->wasTestCreated() ) {	    
		$t->debug(msg=>"We created it, so let's delete it");
		$result = _deleteObj( $self, obj => $createdObj );	    
		if ( $result eq "FAILED" ) {
		    if ( $createdObj->getState() eq "exists" ) {
			$t->debug(msg=>"Failed trying to delete, set to state " .
				  "'maybe', object: $createdObjRef");
			$createdObj->setState(state => "maybe");
			# Try to delete the object again, just once though
			$self->{_tc}->pushCreatedObj( object=>$createdObjRef );
		    }
		    last;
		} else {
		    $createdObj->setState(state => "deleted");
		}
	    } else { 
		$t->debug(msg=>"Not deleting, state is: " . $createdObj->getState());
	    }
	}     ## while ( $createdObjRef = $self->{_tc}->popCreatedObj() ) {
    }
    return $result;
}


sub _deleteObj {
    my ($self, %arg) = @_;
    
    my $createdObj = $arg{obj};
    my ($action, $result);

  Delete: {
      # Run the Object's PreDelete Action
      $action = $createdObj->getPreDelAction();
      if ( $action ) {
	  $t->debug(msg=>"Processing pre-delete action: " . 
		    $createdObj->getId());
	  # Pass the timeout from the current Action into the
	  # execute of the pre-delete action.
	  $result = $action->execute( timeOut => $self->{_timeOut},
				      no_save => 1 );
	  if ( $result eq "FAILED" ) {
	      $self->{_errMsg} = $action->getOutput();
	      last Delete;
	  }
      } else {
	  $t->debug(msg=>"Object has no pre-delete action: " . 
		    $createdObj->getId());
      }
      # Run the Object's delete Action
      $t->debug(msg=>"Processing delete action: " . 
		$createdObj->getId());
      $action = $createdObj->getDelAction();
      # Pass the timeout from the current Action into the
      # execute of the delete action.
      $result = $action->execute( timeOut => $self->{_timeOut} );
      if ( $result eq "FAILED" ) {
	  $self->{_errMsg} = $action->getOutput();
	  last Delete;
      }
  }
    return $result;
}

# Check for some standard, non-handlable errors from the CLI and underlying
# infrastructure, Server, Tharn, ...
sub _cmdErrorCheck {
    my (%arg) = @_;
    
    my $failed = undef;

  SWITCH: {

      # Handle 'errors' that aren't really errors first
      if ( _notExist( cmdOut=> $arg{cmdOut} ) ) {
	  last SWITCH;
      }
      if ( $arg{result} =~ /Err=TIMEOUT/ ) {
	  $t->debug(msg=>$arg{result}); 
	  $failed = 1;
	  last SWITCH;
      }
      # Now look for 'real' errors
      if ( $arg{cmdOut} =~ /ERROR: / ) {
	  $t->debug(msg=>$arg{result}); 
	  $failed = 1;
	  last SWITCH;
      }
      if ( $arg{result} ) {
	  $t->comment(msg=>"Unhandled error:\n$arg{result}"); 
	  $failed = 1;
	  last SWITCH;
      }
  }
    return $failed;
    
}


sub _cmdErrorTimeout {
    my (%arg) = @_;
    my $cmdOut = $arg{cmdOut};
    if ( $cmdOut =~ /Err=TIMEOUT/ ||
	 $cmdOut =~ /Err=IDLE/ ) {
	$t->debug(msg=>"Hit a timeout condition");
	return 1;
    }
    return undef;
}


sub _notExist {
    my (%arg) = @_;
    my $cmdOut = $arg{cmdOut};
    if ( $cmdOut =~ /no\s+entries\s+matching/ ||
	 $cmdOut =~ /does\s+not\s+exist/ ||
	 $cmdOut =~ /entry\s+doesn\'t\s+exist/) {
	$t->debug(msg=>"Not treating 'not found' as errors");
	return 1;
    }
    return undef;
}


sub _verify {
    my ($self, %args) = @_;
    my $output = $args{outString};
    my $outArray = $args{outArray};

    my $refObj = $self->{_tc}->getObject( $self->{_refId} );
    my ($field, %fields, $idx, $match, $found);
    my (@lines, @args, @headers, @values);

    $t->entry(msg=>"Working to verify fields of obj: " . $refObj->getId());

    $t->debug(msg=>"outArray looks like:\n" . Dumper( $outArray ));
    # Easy case, the object should be gone, we're verifying after a delete.
    # Just check for the failed query message.
    if ( $self->{_verify}{State} eq "missing" ) {
	if ( _notExist( cmdOut => $output ) ) {
	    $t->exit( msg=>"Object not found, as it shouldn't be");
	    return 1;
	} else {
	    $t->warn( msg=>"The object was not missing\n$output");
	    $self->{_errMsg} = "The object was not missing\n$output";
	    return undef;
	}
    }

    # SMF CLI output, with -showallfields true, is at least three lines.
    # The fields, the column names, and the field values.  There may be
    # multiple field value lines (if there are more than 1 row in the
    # table and the query was generic enough to return multiples.
    $output =~ s/\r//g;
    @lines = split /\n/, $output; 
    @args = split /X;X/, $lines[0];
    @headers = split /X;X/, $lines[1];
    @values = split /X;X/, $lines[2];

    # Build a hash for each field in the command.  For each field
    # save the location in the output and the desired value

    if ( $self->{_verify}{State} eq "exists" ||
	 $self->{_verify}{State} eq "modified" ) {

	$t->debug(msg=>"Verify type: " . $self->{_verify}{State});
	# Find each key and field in the cmd's output
	# First check the keys defined with the object def
	my %objFields;
	if ( $refObj->getKeys() ) { 

	    %objFields = %{$refObj->getKeys()};
	    
	    $match = 1; $found = 0;
	    for $field (keys  %objFields) {
		# Find the key in the cmds args.
		
		# We have keys, so we only care about the 1st element in the
		# outArray
		if ( defined $$outArray[0]{$field} ) {
		    # OK, found the field name in the list of args.
		    # Verify its value
		    if ( $objFields{$field} ne $$outArray[0]{$field} ) {
			$self->{_errMsg} = "Key: $field, expected: " .
			    $objFields{$field} . ", but found instead: " . $$outArray[0]{$field};
			$t->trace( msg=>$self->{_errMsg} );
			$match = 0;
		    } else {
			$t->debug(msg=>"Found key: $field match for value: " . 
				  $$outArray[0]{$field});
			last;
		    } 
		} else {
		    # Couldn't event find the field.
		    $self->{_errMsg} = "Key: $field, not found in cmd output:\n$output\n";
		    $t->exit( msg=>$self->{_errMsg});
		    return undef;
		}
		
	    }
	    if ( !$match ) {
		$self->{_errMsg} = "Key value(s) don't match.  Check trace for " .
		    "other misses, last miss was:\n" . $self->{_errMsg};
		$t->exit( msg=>$self->{_errMsg});
		return undef;
	    }
	} else {
	    $t->trace(msg=>"There are no keys for this object, check all the returned rows");
	}
	# Now check the fields defined (first in the Verify section, 
	# then in the Action section, and if not in either of those
	# in the Objects definition section
	if ( defined $self->{_verify}{Fields} ) {
	    # Grab Verify  fields if defined
	    $t->debug( msg=>"Grabbing fields from Verify section");
	    %objFields = %{$self->{_verify}{Fields}};
	} elsif ( defined $self->{_fields} ) {
	    # Grab Action fields if defined
	    $t->debug( msg=>"Grabbing fields from Action section");
	    %objFields = %{$self->{_fields}};
	} elsif ( defined $refObj->getFields() ) { 
	    # Try getting them from the object instead.
	    $t->debug( msg=>"Grabbing fields from Object definition");
	    %objFields = %{$refObj->getFields()};
	} else {
	    $t->debug( msg=>"No fields defined to verify");
	}
		
	$match = 1; $found = 0; 
	my $valueLine = 0; my $fieldVal;
	for $field (keys %objFields) {
	    if ( defined $$outArray[0]{$field} ) {
		# OK, found the field name in the list of args.  The value
		# is at the same array index in the values array.
		$found = 1; $valueLine = 0;
	      ValCheck: {
		  if ( ref($$outArray[$valueLine]{$field}) eq "ARRAY" ) {
		      $t->debug(msg=>"Building list from array: " . 
				Dumper($$outArray[$valueLine]{$field}) );
		      $fieldVal = join(",", @{$$outArray[$valueLine]{$field}} );
		      $t->debug(msg=>"Now list looks like: " .
				Dumper($fieldVal));
		  } else {
		      $fieldVal = $$outArray[$valueLine]{$field};
		  }
		  if ( $objFields{$field} ne $fieldVal ) {
		      
		      # If there are more returned rows, try them all
		      if ( defined $$outArray[++$valueLine] ) {
			  redo ValCheck;
		      }
		      $self->{_errMsg} = "Field: $field, expected: " .
			  $objFields{$field} . ", but found instead: " . $fieldVal;
		      $t->trace( msg=>$self->{_errMsg});
		      $match = 0;
		  } else { 
		      $t->debug(msg=>"Found field: $field match for value: " . $$outArray[0]{$field});
		      last;
		  }
	      }
	    } else { 
		# Couldn't even find the field??
		$self->{_errMsg} = "Field: $field  not found in cmd output:\n" .
		    Dumper( $outArray );
		$t->exit( msg=>$self->{_errMsg} );
		return undef;
	    }
	}
	if ( !($match) ) {
	    $self->{_errMsg} = "Field value(s) don't match.  Check trace for " .
		"other misses, last miss was:\n" . $self->{_errMsg};
	    $t->exit( msg=>$self->{_errMsg});
	    return undef;
	}
    }
    $t->exit( msg=>"Done verifying object's keys and fields");
    return 1;
}
    

sub getRefId {
    my ($self, %arg) = @_;
    return $self->{_refId};
}


sub getOutput {
   my ($self, %arg) = @_;
   return "$self->{_errMsg}\n$self->{_output}";
}


sub getOutHash {
   my ($self, %arg) = @_;
   return $self->{_outHash};
}


sub getFullCmd {
   my ($self, %arg) = @_;
   if ( defined $self->{_fullCmd} ) {
       return $self->{_fullCmd};
   } else {
       $t->debug(msg=>"Action: $self->{_cmd} has no full command");
       return undef;
   }
}


sub setFullCmd {
   my ($self, %arg) = @_;
   $self->{_fullCmd} = $arg{fullCmd};
}


sub setAttempts {
   my ($self, $attempts) = @_;
   $self->{_verify}{Attempts} = $attempts;
}


sub setInterval {
   my ($self, $interval) = @_;
   $self->{_verify}{Interval} = $interval;
}


sub setFullCliCmd {
   my ($self, %arg) = @_;
   $self->{_cliMethod} = "$arg{cmdDir} $arg{cmd}";
   $self->{_cliMethod} =~ s/[ -]/_/g;
   %{$self->{_cliArgs}} = $self->_buildArgList( nonKeys => $arg{nonKeys} );
}


1;
