#Hey emacs, this is basically -*- perl -*-

#
#
package ExecTC::Engine;

use strict;
use Data::Dumper;

use TracePkg;
use ExecTC::TestCase;

my $t = TracePkg->new();


use Tharn;
use Utils;
use SayWrap;
use Server;

my $object;

# Create the test environment

# Create the TestCase object
  # Create the test run defaults
  # Create the test steps
  # For each step, create used objects (like "vifs")
sub new {

    $t->entry( msg=>"Initializing Engine and TestCase\n" );

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    #  a hash.
    my ($self, %arg) = @_;

    # Fill the member vars for this instance
    $object = {
	
	tcDef       => $arg{tcDef},
	_tc         => undef,
	_conn       => undef,
	_lastStep   => undef,
	_lastErrMsg => undef,
	_lastRC     => undef,
	_status     => undef,
	_serverObj  => undef,
    };

    # This tags the hash pointed to by $object as a 
    #  type of Table
    bless $object, $self;

    # Initialize the test case objects
    _init();

    $t->exit( msg=>"Initialization complete\n" );
    # And give the caller the referece to the bless object.
    return $object;
}


sub _init {

    $t->entry(msg=>"Building objects for: $object->{tcDef}{Name}\n");

    my ($tc, $host);

    if ( ref($object->{tcDef}{Hosts}) eq "ARRAY" ) {
	foreach my $name (@{$object->{tcDef}{Hosts}}) {
	    $t->debug(msg=>"Will connect to system: $name"); 
	} 
    } elsif ( $object->{tcDef}{Hosts} ne '' ) {
	$t->debug(msg=>"Will connect to system: " . $object->{tcDef}{Hosts}); 
    } else {
	$t->fatal(msg=>"Hosts must be specified at the TestCase level", 
		  result=>"CONFIG");
    }
    
    _connect();

    # Did they define some trace, set it up
    if ( defined $object->{tcDef}{DbladeTrace} ) {
	$object->{_serverObj}->shell( cmd => "dbladecli -c \"" .
				      $object->{tcDef}{DbladeTrace} .
				      "\"");
	$t->debug( msg=>"Set dblade trace to: " . 
		   $object->{tcDef}{DbladeTrace} );
    }
   
    # Create TestCase with global run settings (like error level).
    $tc = ExecTC::TestCase->new( name => $object->{tcDef}{Name},
				 desc => $object->{tcDef}{Description},
				 hosts => $object->{tcDef}{Hosts},
				 errorLevel => $object->{tcDef}{ErrorLevel},
				 connections => $object->{_conn},
				 engine => $object,
				 );

    $object->{_tc} = $tc;

    $t->debug(msg=>"Connection init'd:\n" . Dumper($object->{_conn}));
    # Add the rest of the test case's objects (like steps).
    $tc->addSteps( steps => $object->{tcDef}{Steps} );
    
    $t->exit(msg=>"Objects built for: $object->{tcDef}{Name}\n");
}


sub _connect {
    my (%arg) = @_;

    $t->entry( msg=>"Login to system(s)");
    
    my $server = Server->new();
    $object->{_serverObj} = $server;

    my $conn = $server->get_connection(type=>"cli_connection");

    $server->filer_show("-Timeout"=>120);

    $t->exit( msg=>"Connection complete\n");
    return $conn;
}

# Execute the steps in the TestCase object.
sub run {
    $t->entry( msg=>"Begining test run\n" );
    my ($self, %arg) = @_;
    my ($step, $stepCount, $stepNum, $tmp, $result);

    my $tc = $self->{_tc};
    $self->{_status} = "PASSED";

    $stepCount = $tc->getStepCount();

    # Run all the steps
    $t->debug( msg=>"Running all $stepCount steps\n");

  STEPRUN: {
      for $stepNum (0 .. $stepCount-1) {
	  
	  $step = $tc->getStep($stepNum);
	  $tmp = $step->getName();
	  $t->debug( msg=>"Running step: $tmp\n");
	  
	  $self->{_lastStep} = $step->getName();
	  # Execute the action for this step
	  $result = $step->execute();

	  $self->{_lastResult} = $result ;
	  $self->{_lastErrMsg} = $step->getErrMsg() ;	  

	  if ( $result ne "PASSED" ) {
	      $self->{_status} = "FAILED";

	      # Call the CleanUp step
	      for $stepNum (0 .. $stepCount-1) {
		  $step = $tc->getStep($stepNum);
		  if ( $step->isCleanUp() ) {
		      $t->debug( msg=>"Found a clean up step, run it");
		      $step->execute();
		  }
	      }
	  }	      
	  
	  if ( $result ne "PASSED" ) {
	      last STEPRUN;
	  }
	  
	  # Get the output directly from the command (if any)
	  
	  # 
      }	
  }

    $t->exit( msg=>"Test run complete: $result\n" );

    return $result;
}


sub getLastStep {
    my ($self, %arg) = @_;
    return $self->{_lastStep};
}


sub getLastErrMsg {
    my ($self, %arg) = @_;
    return $self->{_lastErrMsg};
}


sub getLastResult {
    my ($self, %arg) = @_;
    return $self->{_lastResult};
}


sub isSuccessful {
    my ($self, %arg) = @_;

    if ( $self->{_status} eq "PASSED" ) {
	return 1;
    } 

    return 0;
}


sub getConn {
    my ($self, %arg) = @_;
    $t->debug(msg=>"Handing connection for: $arg{host}");
    if ( $arg{host} ) {
	return $self->{_conn}{$arg{host}};
    } else {
	return $self->{_conn};
    }
}


1;
