# -*- perl -*-
#  $Id: //depot/prod/test/main/lib/FPolicy.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here


#############################################################################################################
# Package name :
package FPolicy;
#############################################################################################################

#############################################################################################################
# Standard libraries
#############################################################################################################
use strict;
#use warnings;
use Utils qw(make_host make_conn);
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF);
use Tharn qw(say logcomment logresult);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use setup;
use CIFS::Setup qw(cifs_setup);
use Subtest;
use Tharn qw(host connect);
use Filer::Version qw(get_ontap_version);

#############################################################################################################


#############################################################################################################
# File Policy dependencies
use FPolicy::Utils qw(make_NV make_arrayhash);
use FPolicy::Setup;
#use FPolicy::Common;
use FPolicy::CLI;
use FPolicy::ZAPI;
use FPolicy::VFiler;
use FPolicy::Cluster;
use FPolicy::Client;
use FPolicy::Traffic;
use FPolicy::PServer;
#############################################################################################################


sub new 
{
	my $object = shift;
	my %NV = make_NV(@_);
	my @opt = @_;
	my $class = ref($object) || $object;

	logcomment("opt = @opt");
	$NV{type} = ($NV{type} ne "")? "$NV{type}" : "screen",
	push @opt, "type=$NV{type}";

	my $host = host($NV{filer});
	my $version = get_ontap_version($host);
	
	#get_ontap_version returns 5.0 for IC.0 builds. Untill this is fixed we provide a workaround here. 
	if($version eq "5.0"){			
		$version = "7.3";
	}
	logcomment("Filer Version is : $version");
	push @opt, "version=$version";

	my $pserver = undef;
	my $traffic = undef;

#		config 	=> new FPolicy::Setup	(@opt),
#		cluster => new FPolicy::Cluster	(@opt),
#		vfiler	=> new FPolicy::VFiler	(@opt),
#		client	=> new FPolicy::Client	(@opt),
	$pserver = new FPolicy::PServer (@opt) if(defined $NV{hostname});
	$traffic = new FPolicy::Traffic (@opt) if(defined $NV{hostname});

	my $self = {
		FILER => {
			NAME => $NV{filer}, # mandatory
			IP => $NV{filer_ip}, # mandatory
			VERSION => $version || ""
		},

		POLICY => {
			NAME => $NV{policy}, # mandatory
			TYPE => $NV{type},
			MONITOR => $NV{monitor},
			PROTOCOL => $NV{proto},
			EXT_INC => $NV{ext_inc},
			EXT_EXC => $NV{ext_exc},
			VOLUME => $NV{volume},
			PSERVER_PRI => $NV{pserver_pri},
			PSERVER_SEC => $NV{pserver_sec},
		},

		VFILER => {NAME => $NV{vfiler}},

		CIFS => {
			AD => $NV{cifs_ad},
			ADUSER => $NV{cifs_aduser},
			ADPASS => $NV{cifs_adpass},
			MULTIPROTO => $NV{cifs_multiproto},
			DNS => $NV{cifs_dns},
			SETUP => $NV{cifs_setup}
		},

		INTF 	=> $NV{interface}, # mandatory
		CLIENT	=> $NV{nfsclient},	#holds nfs client
		CIFSCLIENT => $NV{cifsclient}, 
		cli 	=> new FPolicy::CLI	(@opt),
		zapi 	=> new FPolicy::ZAPI	(@opt),
		pserver => $pserver,
		traffic => $traffic
	};

#		config 	=> undef,
#		zapi 	=> undef,
#		cluster => undef,
#		vfiler	=> undef,
#		client	=> undef,
       bless($self, $class); 

	logcomment("File Policy Initialized for 
		FILER = $self->{FILER}{NAME}
		FILER IP = $self->{FILER}{IP}
		VERSION = $self->{FILER}{VERSION}
		INTF = $self->{INTF}
		CLIENT = $self->{CLIENT}
		CIFSCLIENT = $self->{CIFSCLIENT} 
		policy = $self->{POLICY}{NAME}
		policy = $self->{POLICY}{NAME}
		type = $self->{POLICY}{TYPE}
		monitor = $self->{POLICY}{MONITOR}
		volume = $self->{POLICY}{VOLUME}
		ext_inc = $self->{POLICY}{EXT_INC}
		ext_exc = $self->{POLICY}{EXT_EXC}
		pserver_pri = $self->{POLICY}{PSERVER_PRI}
		pserver_sec = $self->{POLICY}{PSERVER_SEC}
		cli = $self->{cli}
		cli object = $self->{cli}
		zapi object = $self->{zapi}
		pserver = $self->{pserver}
		traffic = $self->{traffic}
	");

	return $self;
}



sub config
{
	my $self = shift;
	my @options = @_;

	my $status = eval "$self->{setup}{config}(@options)";

	return $status;
}

#################################################################################################################
#sub make_connection
#{
#	my $self = shift;
#	my @opt = @_;
#
#	logcomment("FPolicy: cifs setup (options = @opt");
#	my $status = FPolicy::Common::make_connection($self,@opt);
#	logcomment("FPolicy: cifs setup (status = $status)");
#
#	return $status;
#}
#
#sub fp_cifs_setup
#{
#	my $self = shift;
#	my @opt = @_;
#
#	logcomment("FPolicy: cifs setup (options = @opt");
#	my $status = FPolicy::Common::fp_cifs_setup($self,@opt);
#	logcomment("FPolicy: cifs setup (status = $status)");
#
#	return $status;
#}
#
#sub configure
#{
#	my $self = shift;
#	my @opt = @_;
#
#	logcomment("FPolicy: configure (options = @opt");
#	my $status = FPolicy::Common::configure($self,@opt);
#	logcomment("FPolicy: configure (status = $status)");
#
#	return $status;
#}
#
#sub addconnection
#{
#	my $self = shift;
#	my @opt = @_;
#
#	logcomment("FPolicy: add connection (options = @opt");
#	my $status = FPolicy::Common::addconnection($self,@opt);
#	logcomment("FPolicy: add connection (status = $status)");
#
#	return $status;
#}
#
#sub add_filer_config($@)
#{
#	my $self = shift;
#	my @opt = @_;
#	my %option = validate(@opt,
#		{
#		FILER => { type => SCALAR},
#		FILER_IP => {type => SCALAR}
#		});
#
#	logcomment("FPolicy: add filer config (options = @opt");
#	my $status = FPolicy::Common::add_filer_config($self,\%option);
#	logcomment("FPolicy: add filer config (status = $status)");
#
#	return $status;
#}
#
#sub add_cifs_config($@)
#{
#	my $self = shift;
#	my @opt = @_;
#	
#	my %opts = validate(@opt,
#		{
#			CIFS_AD => {type=> SCALAR},
#			CIFS_DNS => {type => ARRAYREF},
#			CIFS_AD_PASS => {type => SCALAR},
#			CIFS_AD_DOMAIN_USER => {type => SCALAR},
#			CIFS_AD_AUTH_TYPE => {type => SCALAR, default => ""},
#			CIFS_AD_OU_NAME => {type => SCALAR, default => ""}
#		});
#
#	logcomment("FPolicy: add cifs config (options = @opt");
#	my $status = FPolicy::Common::add_cifs_config($self,\%opts);
#	$status &= FPolicy::Common::configure($self,\%opts);
#	logcomment("FPolicy: add cifs config (status = $status)");
#
#	return $status;
#}
#
#################################################################################################################

=head1 NAME
volume

=head2 DESCRIPTION
placeholder for volume API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

oper = operation type

method = where to perform the operation

volume = name of the volume(s)

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut

sub volume
{
	my $self = shift;
	my %parm = @_;

	if($self->{INTF} eq "" or $self->{INTF} eq "cli") 
	{
		return 
		$self->{cli}->volume
		( 
			conn => $self->{cli}->{CONN},
			policy => $self->{POLICY}{NAME},
			oper => $parm{oper},
			method => $parm{method},
			volume => $parm{volume},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} 
		);

	} 
	else 
	{
		logresult("fail", "invalid interface $self->{INTF}");
	}

}

=head1 NAME
monitor

=head2 DESCRIPTION
placeholder for monitor API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

oper = operation type

opspec = list of operation types

proto = protocol type (nfs/cifs)

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut
sub monitor
{
	my $self = shift;
	my %parm = @_;

	if( $self->{INTF} eq "" or $self->{INTF} eq "cli" ) 
	{
		return 
		$self->{cli}->monitor
		( 
			conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			oper => $parm{oper},
			opspec => $parm{opspec},
			proto => $parm{proto},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} 
		);
	} 
	else 
	{
		logresult("fail", "invalid interface $self->{INTF}");
	}
}

=head1 NAME
show

=head2 DESCRIPTION
placeholder for show API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut

sub show
{
	my $self = shift;
	my %parm = @_;

	if( $self->{INTF} eq "" or $self->{INTF} eq "cli" )
	{
		return 
		$self->{cli}->show
		( 
			conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} 
		);
	} 
	else 
	{
		logresult("fail", "invalid interface $self->{INTF}");
	}
}


=head1 NAME
destroy

=head2 DESCRIPTION
placeholder for destroy API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut

sub destroy
{
	my $self = shift;
	my %parm = @_;

	if( $self->{INTF} eq "" or $self->{INTF} eq "cli" ) 
	{
		return 
		$self->{cli}->destroy
		( 
			conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} 
		);
	} 
	else 
	{
		logresult("fail", "invalid interface $self->{INTF}");
	}
}


=head1 NAME
disable

=head2 DESCRIPTION
placeholder for disable API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut

sub disable
{
	my $self = shift;
	my %parm = @_;

	if( $self->{INTF} eq "" or $self->{INTF} eq "cli" ) 
	{
		return 
		$self->{cli}->disable
		( 
			conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} 
		);
	} 
	else 
	{
		logresult("fail", "invalid interface $self->{INTF}");
	}
}

=head1 NAME
enable

=head2 DESCRIPTION
placeholder for enable API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut

sub enable
{
	my $self = shift;
	my %parm = @_;

	if( $self->{INTF} eq "" or $self->{INTF} eq "cli" ) 
	{
		return 
		$self->{cli}->enable
		( 
			conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} 
		);
	} 
	else 
	{
		logresult("fail", "invalid interface $self->{INTF}");
	}
}

=head1 NAME
create

=head2 DESCRIPTION
placeholder for create API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

type = operation type (screen)

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut

sub create
{
	my $self = shift;
	my %parm = @_;

	if( $self->{INTF} eq "" or $self->{INTF} eq "cli" ) 
	{
		return 
		$self->{cli}->create
		( 
			conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			type => $parm{type},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} 
		);
	} 
	else 
	{
		logresult("fail", "invalid interface $self->{INTF}");
	}
}

=head1 NAME
verify

=head2 DESCRIPTION
placeholder for verify API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

pstate = state of the policy

oper = operation type

method = what to do with the operation

volume = name of the volume(s)

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut
sub verify
{
	my $self = shift;
	my %parm = @_;

	if($self->{INTF} eq "" or $self->{INTF} eq "cli") {
		return $self->{cli}->verify( conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			pstate => $parm{pstate},
			cstate => $parm{cstate},
			monitor => $parm{monitor},
			ext_inc => $parm{ext_inc},
			ext_exc => $parm{ext_exc},
			vol_inc => $parm{vol_inc},
			vol_exc => $parm{vol_exc},
			pserver => $parm{pserver},
			sserver => $parm{sserver},
			vfiler => $parm{vfiler},
			polserv => $parm{polserv}, # policy server = boolean
			servreqd => $parm{servreqd}, # pserver reqd = boolean
			result => $parm{result},
			expected => $parm{expected} );
	} else {
		logresult("fail", "invalid interface $self->{INTF}");
	}
}

=head1 NAME
volume

=head2 DESCRIPTION
placeholder for volume API

=item Input Values
ARRAY of input name value pairs

this includes, others will be ignored
some are optional if already set during initialization

conn = connection object

policy = policy name

oper = operation type

method = what to do with the operation

volume = name of the volume(s)

vfiler = name of the vfiler

result = expected result

expected = hash of success and failure output definitions

=cut
sub verify_system_defaults
{
	my $self = shift;
	my %parm = @_;

	if($self->{INTF} eq "" or $self->{INTF} eq "cli") {
		return $self->{cli}->verify_system_defaults( conn => $self->{cli}->{CONN},
			policy => $parm{policy},
			pstate => $parm{pstate},
			cstate => $parm{cstate},
			monitor => $parm{monitor},
			ext_inc => $parm{ext_inc},
			ext_exc => $parm{ext_exc},
			vol_inc => $parm{vol_inc},
			vol_exc => $parm{vol_exc},
			pserver => $parm{pserver},
			sserver => $parm{sserver},
			vfiler => $parm{vfiler},
			result => $parm{result},
			expected => $parm{expected} );
	} else {
		logresult("fail", "invalid interface $self->{INTF}");
	}
}
#################################################################################################################

sub Policy
{
	my $self = shift;
	my @options = @_;

	my %NV = @options;
	if($NV{cmd_type} ne "") {
		FPolicy::Common::Generic(@options);
	} else {
		logresult("fail", 
			"Invalid command type: $NV{cmd_type}, parameters = @options");
	}
}

sub interface
{
	my $self = shift;
	my $name = shift;

	if($name =~ /[zapi]/i) {
		$self->{$name} = new FPolicy::ZAPI();
		print "\nInterface = $name, type = $self->{$name}->{interface}\n";
	}
}
#################################################################################################################

sub testme
{
	package main;
	our ($FILER, $FILER_CONN, $CLIENT, $POLICY);

	param('FILER',       '-mesg',    "Filer to test");
	param('FILER_CONN','-default',"#rsh");
        param('POLICY',   '-default', 'pol1');
        param('CLIENT',   '-mesg', 'CLIENT');
	use FPolicy;
	my $pol = new FPolicy("policy=$POLICY", "vfiler=vfiler0", "filer=$FILER", 
			"monitor=read,write,open,close", 
			"vol_inc=vol0", "vol_exc=", 
			"ext_inc=txt", "ext_exc=ppt", 
			"pserver_pri=10.10.1.1", "pserver_pri=20.1.1.1",
			"client=$CLIENT"
		);

	$pol->{cli}->create();
	$pol->{cli}->create("RESULT=FAIL");
	$pol->{cli}->enable();
	$pol->{cli}->enable("RESULT=PASS");
	$pol->{cli}->verify("policystate=enabled");
	$pol->{cli}->verify("policystate=enabled", "RESULT=PASS");
	$pol->{cli}->disable();
	$pol->{cli}->disable("RESULT=PASS");
	$pol->{cli}->verify("policystate=Disabled", "RESULT=FAIL");

	$pol->{cli}->monitor("RESULT=FAIL");
	$pol->{cli}->monitor("oper=add", "RESULT=FAIL");
	$pol->{cli}->monitor("oper=add", "RESULT=PASS");
	$pol->{cli}->monitor("oper=add", "opspec=link,symlink");
	$pol->{cli}->monitor("oper=add", "opspec=link,symlink", "protocol=nfs");

	$pol->{cli}->volume("oper=add", "method=include");
	$pol->{cli}->volume("oper=add", "method=include", "volume=vol0");
	$pol->{cli}->volume("oper=add", "method=include", "volume=vol0", "RESULT=PASS");
	$pol->{cli}->volume("oper=add", "method=exclude", "volume=vol1");
	$pol->{cli}->volume("oper=add", "method=exclude", "volume=vol1", "RESULT=PASS");
	
	
	$pol->{cli}->extension("oper=add", "method=include");
	$pol->{cli}->extension("oper=add", "method=include", "extension=ppt");
	$pol->{cli}->extension("oper=add", "method=include", "extension=ppt", "RESULT=PASS");
	$pol->{cli}->extension("oper=add", "method=exclude", "extension=txt");
	$pol->{cli}->extension("oper=add", "method=exclude", "extension=txt", "RESULT=PASS");

	$pol->{cli}->destroy();
	$pol->{cli}->destroy("RESULT=PASS");

}

1;
