# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/FPolicy/Client.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Client;

use strict;

use warnings;
use Utils qw(make_host make_conn);
use Tharn;
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;
#use nfs::NfsOps;
#use nfs::NfsDef;
#use nfs::NfsClient;
use nfs::NfsDef qw(UNCHECKED FILE_SYNC NFLNK);
use mount qw(mount umount);
use setup qw(nfs_setup);
use FS::Util::Setup qw(mount_vol);
use MapShare qw(map_cifs_share unmap_cifs_share);
use FPolicy::Utils qw(make_hash_fromArrayHash);
use FPolicy::Traffic;


sub new ($;%)
{
	my $object = shift;
	my $class = ref($object) || $object;
	my %opt = @_;

	my $self = {
			TYPE => "$opt{TYPE}",
			FILER => "$opt{FILER}",
			IP => "$opt{IP}",
			MOUNTPATH => "$opt{MOUNTPATH}",
			VERSION => "$opt{VERSION}",
			PROTO => $opt{PROTOCOL},
			CLIENT => $opt{CLIENT},
			CLIENT_TYPE => $opt{CLIENT_TYPE},
			VOL => "$opt{VOL}",

			traffic => new FPolicy::Traffic (
				type => $opt{TYPE},
				client => $opt{CLIENT},
				clienttype => $opt{CLIENT_TYPE},
				hostname => $opt{FILER},
				protocol => $opt{PROTOCOL},
				version => $opt{VERSION},
				volume => $opt{VOL},
				resvport => 0,
				mount => $opt{MOUNTPATH}
			)
#			'-runid', "create_client", 
#				'-client', $self->{CLIENT},
#				'-onexit', sub {}, #don't pass result to parent
#				'--',
#				"hostname=>$args{hostname}",
#				"version=>$args{version}||3",
#				"proto=>$args{protocol}||'tcp'",
#				"timeout=>$args{timeout}||'30'"

	};

       bless($self, $class); 

	logcomment("Client object Initialized with
			TYPE => $opt{TYPE}
			FILER => $opt{FILER}
			CLIENT => $opt{CLIENT}
			CLIENT_TYPE => $opt{CLIENT_TYPE}
			IP => $opt{IP}
			MOUNTPATH => $opt{MOUNTPATH}
			VERSION => $opt{VERSION}
			PROTO => $opt{PROTOCOL}
			VOL => $opt{VOL}" );

       return $self;
}

sub setup (@;%)
{
	my $self = shift;
	#my %args = make_hash_fromArrayHash(@_);
	my %args = @_;

	if($self->{TYPE} ne "" and $args{type} ne "")
	{	$args{type} = $self->{TYPE};  }

	my $filer = $args{filer} || $self->{FILER};
	my $client = $args{client} || $self->{CLIENT};
	my $client_type = $args{client} || $self->{CLIENT_TYPE};
	my $timeout = $args{timeout} || $self->{TIMEOUT};
	my $export = $args{export} || $self->{EXPORT};
	my $volume = $args{volume} || $self->{VOL};

	eval {
	if( $args{type} eq "nfs" ) {

#		$self->{MOUNTPATH} = 
#		mount_vol( filer_vol=>$volume, 
#			filer=>$filer, 
#			client=>$client, 
#			timeout=>$timeout ) or logcomment("ERROR while mounting with $volume, more details: $@");

		my $ch = host("$client;hosttype=$client_type");
		my $cc = connect("#rsh");

		$self->{hostobj} = $ch;
		$self->{conn} = $cc;

		logcomment("mount path = $self->{MOUNTPATH}, errors = $@
			client host object = $ch, connection object = $cc");

		if($self->{conn} ne "") 
		{
			my $volume = ($volume eq "root")? "/vol/vol0" : "/vol/$volume";
			logcomment("NFS setup for path=$volume and export=$export"); 
			my $stat = nfs_setup($filer, path=>$volume, export=>$export);
			logcomment("status from nfs_setup : $stat");
		}
		return $self->{MOUNTPATH};
	} elsif( $args{type} eq "cifs" ) {

		my $status = map_cifs_share(
				server       => $args{server},
				share        => $args{share},
				client       => $args{client},
				domain       => $args{domain},
				username     => $args{username},
				password     => $args{password},
				drive        => $args{drive},
			);

		if($status){
			logcomment("Mapping cifs share status = $status");
			$self->{cifsshare} = $status;
		}

	} else {
		logcomment("Invalid type $args{type}");
		return 0;
	}
	};

	if($@) { logresult("fatal", "error - $@"); }
}


sub testme
{

	package main;
	our ($FILER, $FILER_IP, $FILER_CONN, $NFSCLIENT, $NFSCLIENT_TYPE, $CIFSCLIENT, $POLICY, $PROTOCOL, $VERSION, $VOLUME, $CMDTIMEOUT, $MOUNT);

	param('FILER',       '-mesg',    "Filer to test");
	param('FILER_IP',       '-mesg',    "Filer to test_IP");
	param('FILER_CONN','-default',"#rsh");
        param('POLICY',   '-default', 'pol1');
        param('NFSCLIENT',   '-mesg', 'NFSCLIENT');
        param('CIFSCLIENT',   '-mesg', 'CIFSCLIENT');
        param('NFSCLIENT_TYPE',   '-mesg', 'NFSCLIENT_TYPE');
        param('PROTOCOL',   '-mesg', 'PROTOCOL');
        param('VERSION',   '-mesg', 'VERSION');
        param('MOUNT',   '-mesg', 'MOUNT PATH');
        param('VOLUME',   '-mesg', 'VOLUME');
        param('CMDTIMEOUT',   '-default', '30');

	use FPolicy::Client;

	my $client = new FPolicy::Client(
			TYPE => "nfs",
			FILER => $FILER,
			CLIENT => $NFSCLIENT,
			CLIENT_TYPE => $NFSCLIENT_TYPE,
			PROTOCOL => $PROTOCOL,
			IP => $FILER_IP,
			MOUNTPATH => "$MOUNT"||$VOLUME,
			VERSION => "$VERSION",
			VOL => "$VOLUME",
		);

	my $path;

	## NFS client
	#$path = $client->setup(type => "nfs", volume=>$VOLUME, filer=>$FILER, client=>$NFSCLIENT, client_type => $NFSCLIENT_TYPE, timeout=>$CMDTIMEOUT);

#	$client->setup( file=>$FILER,
#			type => "nfs",
#			timeout => $CMDTIMEOUT );
			
			
#	logcomment("mount path = $client->{MOUNTPATH}");
#	say($client->{conn}, "cd $client->{MOUNTPATH};pwd", $client->{MOUNTPATH});

	#system("cp /u/sureshb/data_files/1k_file.txt $client->{mountpath}/2k_file.txt; ls -l $client->{mountpath}/2k_file.txt");
	#use FPolicy::Traffic;

	#$client->{traffic} = new FPolicy::Traffic(
	#			client => $NFSCLIENT,
	#			type => $NFSCLIENT_TYPE,
	#			hostname => $FILER,
	#			protocol => $PROTOCOL,
	#			version => $VERSION,
	#			volume => $VOLUME,
	#			resvport => 0,
	#			mount => $client->{MOUNTPATH} );

	$client->{traffic}->setup();	

	$client->setratio(
			read => 0,
			write => 0, 
			open => 0,
			close => 0,
			create => 1, 
			delete => 0, 
			rename => 0,
			dircreate => 0,
			dirrename => 0,
			dirdelete => 0,
			setattr => 0,
			getattr => 0,
			lookup => 0
		);

	$client->generate(
			create => {
				filename => "2k_file.txt",
				length => 512,
				offset => 512
			},
		);
		#	read => {
		#		filename => "2k_file.txt",
		#		length => 512,
		#		offset => 512
		#	},
		#	write => {
		#		filename => "3k_file.txt",
		#		length => 3096,
			#	buffer => $buf
		#	}
		#);

#	$client->cleanup();

	## CIFS client
#	$path = $client->setup(type => "cifs",
#				server       => "172.17.180.233",
#				share        => "home",
#				client       => $client->{CLIENT},
#				domain       => "cifslab",
#				username     => "Administrator",
#				password     => "BigCheese",
#				drive        => ""
#			);
#
#	$client->cleanup();

	$client->{traffic}->cleanup();	
}

sub cleanup
{
	my $self = shift;

	if($self->{type} eq "nfs") {
		logcomment("Unmounting path [$self->{mountpath}] on client $self->{CLIENT}");
		umount($self->{mountpath}) if(defined $self->{mountpath});
	}elsif($self->{type} eq "cifs") {
		logcomment("Unmounting path [$self->{cifsshare}] on client $self->{CLIENT}");
		unmap_cifs_share( 
			mappath => $self->{cifsshare},
			 client => $self->{CLIENT} 
		);
	}
}


sub setratio ($;%)
{
	my $self = shift;
	my @options = @_;
	my %args = @options;

	my $total = $#options;

	foreach my $opt (@options)
	{
		
	}

	$self->{blocksize} = 1024 * 4;

	$self->{distribution} = {
		read => sprintf("%.2f",$args{read}/$total * 100),
		write => sprintf("%.2f", $args{write} / $total * 100),
		create => sprintf("%.2f", $args{create} / $total * 100),
		delete => sprintf("%.2f", $args{delete} / $total * 100),
		rename => sprintf("%.2f", $args{rename} / $total * 100),
		dircreate => sprintf("%.2f", $args{dircreate} / $total * 100),
		dirrename => sprintf("%.2f", $args{dirrename} / $total * 100),
		dirdelete => sprintf("%.2f", $args{dirdelete} / $total * 100),
		setattr => sprintf("%.2f", $args{setattr} / $total * 100),
		getattr => sprintf("%.2f", $args{getattr} / $total * 100),
		lookup => sprintf("%.2f", $args{lookup} / $total * 100),
		open => sprintf("%.2f", $args{open} / $total * 100),
		close => sprintf("%.2f", $args{close} / $total * 100),
		offline => sprintf("%.2f", $args{offline} / $total * 100)
	};

	my $str = "";
	foreach my $dist (keys %{$self->{distribution}})
	{
		$str .= "$dist = $self->{distribution}{$dist}, ";
	}

	logcomment("Distribution = $str");
}

sub generate ($;%)
{
	my $self = shift;
	my @options = @_;
	my %args = @_;

	#my $readfh;
	#open($readfh, "<$args{filename}") or die "$!";

	#$self->{readfh} = $readfh;
	#$self->Read( $readfh, $args{length}, $args{offset} );
	#$self->Read( $args{filename}, $args{length}, $args{offset} );

	#foreach my $d (keys %distribution) { 
	#	print "\n $d = $distribution{$d}\n"; 
	#	if($d > 0) {
	#		$self->$d($distribution{$d});
	#	}
	#}

	#close(readfh);

	## work for each iteration till everyone is zero
	my %ratio;
	foreach my $key (keys %{$self->{distribution}})
	{
		$ratio{$key} = $self->{distribution}{$key} if($self->{distribution}{$key} > 0);
	}

	#my $setattr = $self->{nfs}->setattr( fhandle => $self->{nfs}{mnt_fh},
	#			attr => {mode => 0777} );

	#foreach my $k (keys %{$setattr})
	#{ 
	#	print "\nsetattr = $k = ${$setattr}{$k} \n";
	#}

	foreach my $r (keys %ratio)
	{
		if($ratio{$r} > 0)
		{
			SWITCH: for ($r) {

			/create/i && do {
				#my $stat = subtest(
				#$self->{traffic}->Create,
				#'-runid', "create_file", 
				#'-client', $self->{CLIENT},
				#'-onexit', sub {}, #don't pass result to parent
				#'--',
				#filename=>"2k_file.txt"
				#);
				$self->{traffic}->Create(filename => "2k_file.txt");
			};

			/read/i && do {
				#$self->Read(filename => "$args{read}{filename}");
				#my $open = $self->Open(filename => "1k_file.txt");
				my $open = $self->{traffic}->Read(fhandle => $self->{nfs}{mnt_fh}, stateid => 0);
				#$self->Close(stateid => $open->{stateid});
			};

			/open/i && do {
				$self->{traffic}->Open(filename => "1k_file.txt");
			};

			/write/i && do {
				my $buf = "this is the trial test";
				$self->{traffic}->Write(offset => 0, 
						data => $buf, 
						stable => FILE_SYNC, 
						stateid => 0 );
			};

			/readdir/i && do {
				$self->{traffic}->ReadDir(dhandle => $self->{nfs}{mnt_fh},
						dircount => 4096,
						maxcount => 4096,
						cookie => 0,
						cookieverf => '');
			};

			} #switch
		} # if
	} # foreach

} # sub 


sub DESTROY
{
}

sub _Client {
	my @options = @_;
	my %NV = FPolicy::Utils::make_NV(@options);

	my $type = (lc($NV{type}) ne "nfs")? lc($NV{type}) : "nfs";

	my $client = $NV{client};
	my $clientip = $NV{clientip};

	my %obj = ($type eq "nfs")? NFSClient( @options ) : CIFSClient( @options );

	return %obj;
}
sub mount_NfsClient ($@)
{
	my $self = shift;
	my @options = @_;

	my %NV = make_hash_fromArrayHash(@options);

	$self->{nfs_obj} = nfs::NfsClient->new(
				hostname => $NV{hostname}, 
				version => $NV{version},
				 proto => $NV{proto} 
				);

	my $path = ($NV{path} ne "" and $NV{vol} eq "")? $NV{path} : "/vol/$NV{vol}";
	logcomment("path = $path, obj = $self->{nfs_obj}, host = $self->{nfs_obj}->{hostname}");
	my $str = "";
	foreach my $p (keys %{$self->{nfs_obj}}) { $str .= "$p = $self->{nfs_obj}->{$p}, "; }
	logcomment("NfsClient obj = { $str }");
	$self->{mountpt} = $self->{nfs_obj}->mount(path => $path);
	$self->{proto} = $NV{protocol};
	$self->{version} = $NV{version};
	$self->{volume} = $NV{volume};
	
	return;
}
#sub CIFSClient
#{
#	my @options = @_;
#
#	my %NV = FPolicy::Utils::make_NV(@options);
#
#	#use lib '/u/sureshb/p4/test/lib/CIFS/Windows/site/lib/';
#
#	#use CIFS::FileOp;
#	#use MapShare;
#
#	my $stat = 
#	MapShare::map_cifs_share( server => $NV{server},
#					share => $NV{share},
#					drive => $NV{drive},
#					client => $NV{client}
#				);
#	
#
#	logcomment("share return = $stat");	
#
#	my %retVal = (
#		object => $stat,
#		client => $NV{client},
#		drive => $NV{drive},
#		share => $NV{share}
#	);
#
#	return %retVal;
#}

#	$nfs->mount_NfsClient(
#			hostname => "$FILER",
#			version => "$VERSION",
#			 proto => "$PROTOCOL",
#			path => "$MOUNT"
#		);

#	$nfs->readdir();

#	my $path = $nfs->cmount(
#			filer => $FILER, 
#			volume => $VOLUME,
#			type => "nfs",
#			proto => $PROTOCOL,
#			vers => $VERSION,
#			mountopts => $MOUNT,
#			client => $CLIENT
#		);


1;
