# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/FPolicy/Common.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Common;

use strict;
use warnings;
use Tharn;
use Utils qw(make_host make_conn);
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF);
use Tharn qw(say logcomment logresult);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use setup;
use CIFS::Setup qw(cifs_setup);
use Subtest;
use FPolicy::Syntax;
use FPolicy::Utils;
use FPolicy::CLI;
use vars qw(%Syntax);

use Aggregate qw(
		aggr_create
		aggr_exists
		);
use Volume qw(
		vol_create_flex
		vol_destroy
		vol_exists
		vol_online
		vol_offline
		);
use install qw(license);
use VFILER::Vfiler qw(
		vfiler_create_non_default_ipspace
		vfiler_dr_configure
		vfiler_dr_delete
		vfiler_dr_status
		vfiler_stop_destroy
		vfiler_run_vfiler_add_default_route
		vfiler_run_vfiler_exportfs
		vfiler_run_vfiler_setup
		vfiler_run_vfiler_cifs_setup
		);

use VFILER::Util qw(
		clean_vfiler_and_resources
		host_connect
		set_timezone
		);

use VFILER::IPspace qw(
		create_ipspace
		assign_ipspace
		assign_default_ipspace
		destroy_ipspace
		);

#######################
## Function Prototypes
#######################

#sub create(@);
#sub destroy(@);
#sub verify(@);
#sub show(@);
#sub enable(@);
#sub disable(@);
#sub monitor(@);
#sub volume(@);
#sub extension(@);
########################

#sub new 
#{
#       my $object = shift;
#       my @options = @_;
#       my %NV = FPolicy::Utils::make_NV(@options);
#       my $policy = $NV{policy};
#
#       my $class = ref($object) || $object;
#       my $self = {
#		FILER => {
#			CONN => "#telnet",
#		},
#		POLICY => {
#			NAME => "$policy",
#			VOLUME => {
#				INCLUSION => ("All"),
#				EXCLUSION => ("None"),
#			},
#			EXTENSION => {
#				INCLUSION => ["\\?\\?\\?"],
#				EXCLUSION => ["Extensions-not-to-screen list is empty"],
#			},
#			SCREENED => 0,
#			FAILURES => 0,
#			STATE => "disabled",
#		},
#       };
#       
#       bless($self, $class); 
#       return $self;
#}

sub volume
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=volume";

	#return Generic($self, @opts);
	
}

sub monitor
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=monitor";

	#return Generic($self, @opts);
	$self->{cli}->{FILER}{NAME} = $self->{FILER}{NAME};
	$self->{cli}->{POLICY}{NAME} = $self->{POLICY}{NAME};

	return $self->{cli}->monitor(@opts);
}

sub extension
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=extension";

	return $self->Generic(@opts);
}

sub show
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=show";

	return $self->Generic(@opts);
}

sub destroy
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=destroy";

	return $self->Generic(@opts);
}

sub disable
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=disable";

	return $self->Generic(@opts);
}


sub enable
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=enable";

	return $self->Generic(@opts);
}


sub create
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=create";

	#return Generic($self, @opts);
	$self->{cli}->{FILER}{NAME} = $self->{FILER}{NAME};
	$self->{cli}->{POLICY}{NAME} = $self->{POLICY}{NAME};

	#return $self->{cli}->create(@opts);
	return $self->Generic(@opts);
}

sub verify
{
	my $self = shift;
	my @opts = @_;

	push @opts, "cmd_type=verify";

	#return Generic($self, @opts);
	$self->{cli}->{FILER}{NAME} = $self->{FILER}{NAME};
	$self->{cli}->{POLICY}{NAME} = $self->{POLICY}{NAME};
	$self->{cli}->{POLICY}{STATE} = $self->{POLICY}{STATE};
	$self->{cli}->{POLICY}{MONITORED} = $self->{POLICY}{MONITORED};
	$self->{cli}->{POLICY}{VOLUME} = $self->{POLICY}{VOLUME};
	$self->{cli}->{POLICY}{EXTENSION}{INC} = $self->{POLICY}{EXTENSION}{INC};
	$self->{cli}->{POLICY}{EXTENSION}{EXC} = $self->{POLICY}{EXTENSION}{EXC};
	$self->{cli}->{POLICY}{PSERVER}{0} = $self->{POLICY}{PSERVER}{0};
	return $self->{cli}->verify(@opts);
}

sub verify_system_defaults
{
	my $self = shift;
	my @opt = @_;
	#logcomment("self = $self, options = @opt");
	my %optvar = FPolicy::Utils::make_NV(@opt);

	my @temp = ();
	foreach my $var (keys %optvar) { push @temp, $var => $optvar{$var}; }
	# default is to verify all the default objects
	if(!defined $optvar{cifs} and 
		!defined $optvar{fpolicy} and 
		!defined $optvar{license} ) 
	{
		push @opt, cifs => "default";
		push @opt, fpolicy => "default";
		push @opt, license => "default";
	} else {
		my %opts = validate(@temp, {
			cifs => {type => SCALAR, default => "default", optional=>1 },
			fpolicy => {type => SCALAR, default => "default", optional=>1 },
			license => {type => SCALAR, default => "default", optional=>1 }
			});
	}

	$self->{INTF} = ($self->{INTF} eq "") ? "cli" : $self->{INTF} ;

	logcomment("FPolicy:Common: verify_system_defaults: self=$self, intf=$self->{INTF}");
	if($self->{INTF} =~ /cli/i)
	{
		$self->{cli}->verify_system_defaults(@opt);
	}elsif($self->{INTERFACE} =~ /zapi/i)
	{
		$self->ZAPI_FPOLICY_VERIFY_SYSTEM_DEFAULTS();
	}
}

sub Generic
{
	my $self = shift;
	my @options = @_;

	my %NV = FPolicy::Utils::make_NV(@options);
	my $cmd_type = $NV{cmd_type};
	delete $NV{cmd_type};
	@options = FPolicy::Utils::array_from_hash(%NV);
	my @parm = FPolicy::Utils::make_arrayhash(@options);

	my %var = validate( @parm, { 
			$Syntax{$NV{cmd_type}} 
		});

	push @options, "cmd_type=$cmd_type";
	my $retVal = 0;
	if($self->{INTF} =~ /cli/i) {
		#$retVal = eval "FPolicy::CLI::$self->{OPER}(@options)" ;
		#$retVal = eval "$self->{cli}->{$cmd_type}(@options)" ;
		$retVal = eval "$self->{cli}->{$cmd_type}(@options)" ;
		print "EVAL = $retVal, ERROR: $@\n";
	} elsif($self->{INTF} =~ /zapi/i)
	{
		$retVal = eval "$self->{zapi}->{$self->{OPER}}(@options)" ;
		logcomment("EVAL = $retVal, ERROR: $@");
	} else {
		logresult("fail", "Uknown interface $self->{INTF}, parameters @options");
#		FPolicy::Utils::DumpData($self);
		$retVal = 0;
	}
	return $retVal;
}

#sub fp_cifs_setup
#{
#	my $self = shift;
#	my @opt = @_;
#	my $filer = $self->{FILER}{NAME};
#
#	#print "\n Configuring cifs setup\n";
#	#push @opt, "FILER=$filer";
#	return FPolicy::Utils::Setup_Cifs($self, @opt);
#}
#
#sub configure
#{
#	my $self = shift;
#
#	#print "\nConfiguring ...\n";
#	fp_cifs_setup();
#
#	#Subtest::subtest("cifs/ccover/smb/smbsetup/cifs_setup.thpl", -nobg, -runid, "setup","FILER=$self->{FILER}{NAME}", "CLIENT=nss-w40;hosttype=windows", -iffatal, -fail, "Donot be panic,check the error and try again:))))))", -ifscript,-fail, "Script Error" );
#
#	#print "\nDone.\n";
#}
#
#sub addconnection
#{
#	my $self = shift;
#	my $handle = shift;
#	my $conn = shift;
#	$self->{FILER}{HOST} = $handle;
#	$self->{FILER}{CONN} = $conn;
#}
#
#sub add_filer_config
#{
#	my $self = shift;
#	my $values = shift;
#
#	#print "\nAdding Filer\n";
#	$self->{FILER}{NAME} = "'". ${$values}{FILER} ."'";
#	#print "NAME = $self->{FILER}{NAME}, ${$values}{FILER}\n ";
#	$self->{FILER}{IP} = ${$values}{FILER_IP};
#	#print "IP = $self->{FILER}{IP},  ${$values}{FILER_IP}\n";
#	$self->{HOSTTYPE} = "filer";
#}
#
#sub add_cifs_config
#{
#	my $self = shift;
#	my @values = @_;
#
#	my %NV = FPolicy::Utils::make_NV(@values);
#
#	foreach my $v (@{$self->{FILER}{CIFS_CONFIG}{DNS}}) {
#		#print "V= $v\n";
#	}
#	$self->{CIFS_CONFIG}{DNS} = $NV{CIFS_DNS}; 
#	$self->{CIFS_CONFIG}{AD}{IP} = $NV{CIFS_AD};
#	#print "CIFS AD = $self->{CIFS_CONFIG}{AD}{IP}  \n";
#	$self->{CIFS_CONFIG}{AD}{PASSWD} = $NV{CIFS_AD_PASS};
#	#print "CIFS AD PASS = $self->{CIFS_CONFIG}{AD}{PASSWD}  \n";
#	$self->{CIFS_CONFIG}{AD}{OU_NAME} = $NV{CIFS_AD_OU_NAME};
#	#print "CIFS AD OU-NAME = $self->{CIFS_CONFIG}{AD}{OU_NAME}  \n";
#	$self->{CIFS_CONFIG}{AD}{AUTH_TYPE} = $NV{CIFS_AD_AUTH_TYPE};
#	#print "CIFS AD AUTH-TYPE = $self->{CIFS_CONFIG}{AD}{AUTH_TYPE}  \n";
#	$self->{CIFS_CONFIG}{AD}{DOMAIN_USER} = $NV{CIFS_AD_DOMAIN_USER};
#	#print "CIFS AD DOMAIN_USER = $self->{CIFS_CONFIG}{AD}{DOMAIN_USER}  \n";
#	
#}
#
#sub add_vfiler_config
#{
#	my $self = shift;
#	my @values = @_;
#
#	my %NV = FPolicy::Utils::make_NV(@values);
#
#	$self->{FILER}{$NV{VFILER}}{NAME} = $NV{VFILER};
#	$self->{FILER}{$NV{VFILER}}{IP} = $NV{VFILER_IP};
#	$self->{FILER}{$NV{VFILER}}{MASK} = $NV{VFILER_SUBNETMASK};
#	$self->{FILER}{$NV{VFILER}}{GATEWAY} = $NV{VFILER_GATEWAY};
#}
#
#

1;
