# -*- perl -*-
# $Id: $
## @summary Library of FPolicy routines
## @author  sureshb,sandeepp (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Constants;
use strict;

use constant VFILER_NAME	=> "vfiler1";
use constant POLICY_NAME	=> "vf1_poltest";
use constant FORCE_SETUP	=> "1";
use constant CIFS_SETUP		=> "1";
use constant FILER_CONN		=> "#rsh-socket";


###########################################################################
## This is used for Generating commands
###########################################################################
				
our %BaseCmdTemplate = (
	fpo => "<command> ",
	hel => "<command> <cmd_type> <command>",
	sho => "<command> <cmd_type> <policy>",
	cre => "<command> <cmd_type> <policy> <type>",
	des => "<command> <cmd_type> <policy>",
	ena => "<command> <cmd_type> <policy>",
	dis => "<command> <cmd_type> <policy>",
	ext => "<command> <cmd_type> <method> <oper> <policy> <extension>",
	opt => "<command> <cmd_type> <policy> <method> <value>",
	ser => "<command> <cmd_type> <method> <policy> <value>",
	mon => "<command> <cmd_type> <oper> <policy> -p <proto> -f <opspec>",
	vol => "<command> <cmd_type> <method> <oper> <policy> <volume>",
	opt_show => "<command> <cmd_type> <policy> <method>",    # Added by prashant
	sta => "priv set advanced; <command> <cmd_type> -V <volume>",  # snap status
	glo => "options <command>.<cmd_type> <value>",
);

our %R7_3CmdTemplate = (
	fpo => "<command> ",
	hel => "<command> <cmd_type> <command>",
	sho => "<command> <cmd_type> <policy>",
	cre => "<command> <cmd_type> <policy> <type>",
	des => "<command> <cmd_type> <policy>",
	ena => "<command> <cmd_type> <policy>",
	dis => "<command> <cmd_type> <policy>",
	ext => "<command> <cmd_type> <method> <oper> <policy> <extension>",
	ext_sho => "<command> <cmd_type> <method> <oper> <policy>",
	opt => "<command> <cmd_type> <policy> <method> <value>",
	ser => "<command> <cmd_type> <method> <policy> <value>",
	mon => "<command> <cmd_type> <oper> <policy> -p <proto> -f <opspec>",
	vol => "<command> <cmd_type> <method> <oper> <policy> <volume>",
	vol_sho => "<command> <cmd_type> <method> <oper> <policy>",
	opt_show => "<command> <cmd_type> <policy> <method>",   # Added by Prashant
	sta => "priv set advanced; <command> <cmd_type> -V <volume>",  # snap status
	glo => "options <command>.<cmd_type> <value>",
);


our %BaseExpectedTemplate = (
	policy => {
		create => {
			success =>  "File policy <policy> created successfully.",
			failure =>  [ "A policy named <policy> already exists.",
				"Usage:"
				],
		},

		enable => {
			success => 'File policy <policy> \(file screening\) is enabled.',
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
		},

		disable => {
			success => 'File policy <policy> \(file screening\) is disabled',
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
		},

		destroy => {
			success => "-blind",
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
		},
		## Added by Prashant
		options => {
			success => "-blind",
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
		},
		
		monitor => {
			success => "-blind",
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
			show => '(\\w+)(,(\\w+))+',
		},

		extension => {
			success => "-blind",
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
			show => '(\\w+)(,(\\w+))+',
		},

		volume => {
			success => "-blind",
#			success => ["-blind",
#				    "Exclude-volume list for policy <policy> nullifies the include-volume list."
#				    ],
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
			show => '(\\w+)(,(\\w+))+',
		},

		show => {
			success => "screen failures",
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
		},
		
		global => {
			success => "fpolicy.enable               <value>",
			failure => [ "No such option",
						"Usage:"
						],
		},
		
		destroyverify => {
			#success => [ "Policy name <policy> not found.",
			#	"Usage:"
			#	],
			success => "Policy name <policy> not found.",
			failure => "-blind",
		},

		default_show => {
			failure => [
				"Policy name <policy> not found.",
				"Usage:"
				],

			success => 'File policy <policy> \(file screening\) is <policystate>

		No file policy servers are registered with the filer\.

		Operations monitored:
		 No operations are monitored.

		List of extensions to screen:
		 ???

		List of extensions not to screen:
		 Extensions-not-to-screen list is empty.

		Number of files screened:   <screened>
		Number of screen failures:  <screenfailures>',

		}
	},
	snap => {
			status => {
				success =>  "complete.",
				failure =>  [ "Volume <volume> does not exist or is not online.",
					"Usage:"
					],
		}
	}
);

our %R7_2ExpectedTemplate = (
policy => {
	disable => {
		success => 'File policy <policy> \(file screening\) has been disabled',
	},
}
);


our %R7_3ExpectedTemplate = (
	policy => {

		disable => {
			success => 'File policy <policy> \(file screening\) has been disabled',
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
		},


		show => {
			success => "requests blocked locally",
			failure => [ "Policy name <policy> not found.",
				"Usage:"
				],
		},

		default_show => {
			failure => [
				"Policy name <policy> not found.",
				"Usage:"
				],

			success => 'File policy <policy> \(file screening\) is <policystate>

		No file policy servers are registered with the filer\.

		Operations monitored:
		 No operations are monitored.

		List of extensions to screen:
		 ???

		List of extensions not to screen:
		 Extensions-not-to-screen list is empty.

		Number of requests screened          :  <screened>
		Number of screen failures            :  <screenfailures>
		Number of requests blocked locally   :  <blocked>'

		}
	}
);


our %BaseOpsTemplate = (

	open => "File open",
	close => "File close",
	read => "File read",
	write => "File write",
	create => "File create",
	delete => "File delete",
	rename => "File rename",
	create_dir => "Directory create",
	delete_dir => "Directory delete",
	rename_dir => "Directory rename",
	getattr => "Getattr",
	setattr => "Setattr",
	lookup => "Lookup",
	link => "Link",
	symlink => "Symlink",
	allCifs => "File open,File create,File rename,File close,File delete,File read,File write,Directory rename,Directory delete,Directory create",
	allNfs => "File open,File create,File rename,File close,File delete,File read,File write,Directory rename,Directory delete,Directory create,Lookup,Setattr,Getattr,Link,Symlink",
	all => "File open,File create,File rename,File close,File delete,File read,File write,Directory rename,Directory delete,Directory create,Lookup,Setattr,Getattr,Link,Symlink"
);

our %R7_3OpsTemplate = (

	open => "File open",
	close => "File close",
	read => "File read",
	write => "File write",
	create => "File create",
	delete => "File delete",
	rename => "File rename",
	create_dir => "Directory create",
	delete_dir => "Directory delete",
	rename_dir => "Directory rename",
	getattr => "Getattr",
	setattr => "Setattr",
	lookup => "Lookup",
	link => "Link",
	symlink => "Symlink",
	allCifs => "File open,File create,File rename,File close,File delete,File read,File write,Setattr,Directory rename,Directory delete,Directory create",
	allNfs => "File open,File create,File rename,File close,File delete,File read,File write,Setattr,Directory rename,Directory delete,Directory create,Lookup,Getattr,Link,Symlink",
	all => "File open,File create,File rename,File close,File delete,File read,File write,Setattr,Directory rename,Directory delete,Directory create,Lookup,Getattr,Link,Symlink",
);


our %CmdTemplate = (

	default => \%BaseCmdTemplate,

	7.2 => \%BaseCmdTemplate,
	
	7.3 => \%R7_3CmdTemplate,
	
	'8.0' => \%R7_3CmdTemplate           
);


our %ExpectedTemplate = (
	default => \%BaseExpectedTemplate,

	7.2 => \%R7_2ExpectedTemplate,
	
	7.3 => \%R7_3ExpectedTemplate,
	
	'8.0' => \%R7_3ExpectedTemplate     
);


our %OpsTemplate = (

	default => \%BaseOpsTemplate,

	7.2 => \%BaseOpsTemplate,
	
	7.3 => \%R7_3OpsTemplate,
	
	'8.0' => \%R7_3OpsTemplate          
);


our %NFS_attributes = (
);

our %NFSParam = (
	attrib => \%NFS_attributes,
);



our $POLICY_SERVER_EVENT_EXEPATH = "c:\\bin";
our $POLICY_SERVER_EVENT_EXE = "nfpc-event.exe";
#our $POLICY_SERVER_EVENT_EXE = "nfpc-transfer.exe";
our $POLICY_SERVER_TRANSFER_EXEPATH = "c:\\bin";
our $POLICY_SERVER_TRANSFER_EXE = "nfpc-transfer1.exe";
our $DELETE_NT_STREAMS_PATH = "c:\\bin";         #Added by Rajaram for ADS support
our $DELETE_NT_STREAMS = "streams.exe";          #Added by Rajaram for ADS support
our $SET_ACL_EXE = "SetACL.exe";
our $SET_SPARSE_EXE = "setsparseattr.exe";
our $CIFS_DOMAIN_NETBIOS = main::param("DOMAIN_NETBIOS","-mesg", "DOMAIN_NETBIOS");
our $CIFS_DOMAIN_USERNAME = main::param("DOMAIN_USERNAME","-mesg", "DOMAIN_USERNAME");
our $CIFS_DOMAIN_PASSWORD = main::param("DOMAIN_PASSWORD","-mesg", "DOMAIN_PASSWORD");

#our $CIFS_DOMAIN_NETBIOS = "";
#our $CIFS_DOMAIN_USERNAME = "";
#our $CIFS_DOMAIN_PASSWORD = "";
our $CIFS_WORKGROUP = "WORKGROUP";
our $CIFS_SHARE = "HOME";
our $MINWIN_SHARE = "\\\\btcinf02.eng.btc.netapp.in\\localtest\\arch\\win32 /u:btcinf02\\minwin qa4testing";

# this may be required when host is not included in host DB and 
# script has to add this
our $RHOSTS_DB = "/x/eng/localtest/noarch/share/nate/nate_hosts";

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(
		%CmdTemplate
		%ExpectedTemplate
		%OpsTemplate

		$POLICY_SERVER_EVENT_EXE
		$POLICY_SERVER_TRANSFER_EXE
		$POLICY_SERVER_EVENT_EXEPATH
		$POLICY_SERVER_TRANSFER_EXEPATH
		$DELETE_NT_STREAMS_PATH
		$DELETE_NT_STREAMS
		$SET_ACL_EXE
		$SET_SPARSE_EXE
		
		$CIFS_DOMAIN_NETBIOS 
		$CIFS_DOMAIN_USERNAME 
		$CIFS_DOMAIN_PASSWORD 
		$CIFS_WORKGROUP
		$CIFS_SHARE
		$MINWIN_SHARE
		
		$RHOSTS_DB
		VFILER_NAME
		POLICY_NAME
		FORCE_SETUP
		CIFS_SETUP
		FILER_CONN
	);

1;
