# -*- perl -*-
#  $Id: //depot/prod/test/main/lib/Execute.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Execute;

use strict;

use warnings;
use Utils qw(make_host make_conn);
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF);
use Tharn qw(say logcomment logresult);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;
use FPolicy::Utils; # qw(trim_str);

require Exporter;
use base qw(Exporter);

our @EXPORT_OK = qw(
		ValidateExpected
		ExecuteCmd
		ExecuteAndExpect
	);


=head1 NAME
ValidateExpected

=head2 DESCRIPTION
Compare the expected string with the received output string

=over

=item ARRAY
output is a reference to array string and 
expected value is a hash  of {success} and {failure} arrays
{failure} will be used only when {success} is null (no expected string)

=item Returns
returns 0 - for pass, 1 for fail

=cut

sub ValidateExpected
{	
	my ($output, $expected, $vfiler) = @_;			### Changed by Rajaram

	my @output = split /\n/, $output;

	#### Added by rajaram
	if($vfiler ne "vfiler0")
	{
		shift(@output);
		$output = join /\n/, @output;
	}

	my @tmp = @output;
	my $status = 0;
	my $noOutput = 0;
	
	
	
	logcomment("Comparing \"$output\" with expected \"${$expected}{success}\"");

	my @arr = ( ${$expected}{success} );

	foreach my $expline ( @arr )
	{
		$_ = pop @tmp;
		if( $expline =~ /-blind/ or $expline =~ /.*\\r/ )
		{
			logcomment("No Output Expected");
			$noOutput = 1;
			last;
		} 
		elsif ( !m/$expline/ )
		{
			$status = 1;
			logcomment("No Match for $expline with $_");
			last;
		}
	}

	if( $noOutput )
	{
#		@tmp = @output;
#		@arr = ${$expected}{failure};
	#TBD: This check is to take care of warning for volume commands.
		if($output =~ /nullifies the include-volume list./i)
		{
			return 0;
		}
		if($output ne "")   #some output recieved when no output was expected is a failure condition
		{
			return 1;
		
		}
		#TBD: Commenting out matching of Failure cases. This can be included if required in future.
#		logcomment("Comparing \"$output\" with expected failure case \"${$expected}{failure}\"");
#	
#		foreach my $expline ( @arr )
#		{
#			logcomment("comparing with \"$expline\" and \"$_\"");
#	
#			$_ = pop @tmp;
#	
#			if ( m/$expline/ )
#			{
#				$status = 1;
#				logcomment("Failure Match for \"$expline\" with \"$_\"");
#				last;
#			}
#		} # for

	} # if


	return $status;
}

=head1 NAME
ExecuteCmd

=head2 DESCRIPTION
Executes the input command

=over

=item handle
connection object to the filer

=item cmd
input command to execute on the filer

=item Result
returns the obtained output

=back

=cut
sub ExecuteCmd
{
	my ($handle, $cmd) = @_;
	logcomment("Cmd = $cmd");

	return sayEOF($handle, $cmd);
}

=head1 NAME
ExecuteAndExpect

=head2 DESCRIPTION
Executes the input string and compares the expected string with the received output

=item handle
connection object to the filer

=item cmd
input command

=item expected
hash of expected values for success array and failure array strings

=item Return value
returns hash of return values

hash with
result => pass/fail or 0/1
output => received output

=back

=cut

sub ExecuteAndExpect
{
	my ($handle, $cmd, $expected, $vfiler) = @_;		### Changed by Rajaram

	logcomment("Handle = $handle
		Cmd = $cmd
		Expected Response: success = ${$expected}{success}
				failure = ${$expected}{failure}");

	my $expstatus = ExecuteCmd($handle, $cmd);
	my $cmdstatus = 0;

	#if($#expected >= 0) {
		$cmdstatus = ValidateExpected(FPolicy::Utils::trim_str($expstatus), $expected, $vfiler); ### Changed by Rajaram
	#}

	logcomment("Received status = $cmdstatus, Output = $expstatus");
	return ( result=>$cmdstatus, output=>"$expstatus" );
}


1;

