# -*- perl -*-
#  $Id: //depot/prod/test/main/lib/Generator.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Generator;

use strict;

#use warnings;
use Utils qw(make_host make_conn);
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF);
use Tharn qw(say logcomment logresult);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;
use FPolicy::Utils qw(make_NV make_hash_fromArrayHash);
use FPolicy::Constants qw(%ExpectedTemplate);

require Exporter;
use base qw(Exporter);
our @EXPORT_OK = qw(
			CLI_make_command
			CLI_Expected_Str
		);

###################################################
# this module is replaced by Grammer::GenerateCmd
###################################################
#my %CLI = (
#	CREATE_CMD_PREFIX => "fpolicy create ",
#	CREATE_CMD_SUFFIX => " screen",
#
#	ENABLE_CMD_PREFIX => "fpolicy enable ",
#
#	DISABLE_CMD_PREFIX => "fpolicy disable ",
#
#	DESTROY_CMD_PREFIX => "fpolicy destroy ",
#
#	SHOW_CMD_PREFIX => "fpolicy show ",
#
#	ADD_MONITOR_PREFIX => "fpolicy monitor add ",
#	REMOVE_MONITOR_PREFIX => "fpolicy monitor remove ",
#	SET_MONITOR_PREFIX => "fpolicy monitor set ",
#
#	VOLUME_PREFIX => "fpolicy volume ",
#	ADD_VOLUME_SUFFIX => " add ",
#	REMOVE_VOLUME_SUFFIX => " remove ",
#	SET_VOLUME_SUFFIX => " set ",
#
#	EXTENSION_PREFIX => "fpolicy extension ",
#	EXTENSION_SUFFIX => "fpolicy extension ",
#);
#
#sub CLI_make_command ($;@)
#{
#	my $cmd_type = shift;
#	my @options = @_;
#	my %NV = @_;
#	my $retVal = "";
#
#
#	SWITCH: for ($cmd_type) {
#	/policy_create/i && do {
#		 $retVal = "$CLI{CREATE_CMD_PREFIX} $NV{policy} $CLI{CREATE_CMD_SUFFIX}";
#		} ;
#	/policy_enable/i && do {
#		$retVal = "$CLI{ENABLE_CMD_PREFIX} $NV{policy} ";
#		$CLI{POLICY}{STATE} = "enable";
#		};
#	/policy_disable/i && do {
#		$retVal = "$CLI{DISABLE_CMD_PREFIX} $NV{policy} ";
#		$CLI{POLICY}{STATE} = "disable";
#		} ;
#	/policy_destroy|policy_destroy_verify/i && do {
#		$retVal = "$CLI{DESTROY_CMD_PREFIX} $NV{policy} ";
#		$CLI{POLICY}{DESTROY} = "true";
#		} ;
#	/[policy_add_volume|policy_remove_volume|policy_set_volume|policy_show_volume|policy_reset_volume]/i && do {
#		$retVal = "$CLI{VOLUME_PREFIX} $NV{type} $NV{method} $NV{oper} $NV{policy}" if(defined $NV{oper} and $NV{oper} =~ /[reset|show|eval]/i);
#		$retVal = "$CLI{VOLUME_PREFIX} $NV{type} $NV{method} $NV{oper} $NV{policy} $NV{volume}" if(defined $NV{oper} and $NV{oper} =~ /[add|remove|set]/i);
#		#$retVal = "$CLI{VOLUME_PREFIX} $NV{type} $NV{method} $NV{oper} $NV{policy} $NV{volume}";
#		};
#	/policy_add_monitor/i && do {
#		$retVal = "$CLI{ADD_MONITOR_PREFIX} $NV{policy} -f $NV{opspec}";
#		$CLI{POLICY}{MONITOR}{OPSPEC} = "$NV{opspec}";
#		$CLI{POLICY}{MONITOR}{PROT} = "$NV{proto}";
#		if(defined $NV{proto} and $NV{proto} ne "") { $retVal .= " -p $NV{proto}"; }
#		} ;
#	/policy_remove_monitor/i && do {
#		$retVal = "$CLI{REMOVE_MONITOR_PREFIX} -f $NV{opspec}";
#		if(defined $NV{proto} and $NV{proto} ne "") {
#			$CLI{POLICY}{MONITOR}{PROTO} = $NV{proto};
#			$retVal .= " -p $NV{proto}";
#		}
#		foreach my $spec (split /\,/, $NV{opspec}) {
#			$CLI{POLICY}{MONITOR}{OPSPEC} =~ s/$spec//g;
#		}
#		$CLI{POLICY}{MONITOR}{OPSPEC} =~ s/\,\,/\,/g;
#		foreach my $spec (split /\,/, $NV{opspec}) {
#			$CLI{POLICY}{MONITOR}{PROT} =~ s/$NV{proto}//g;
#		}
#		$CLI{POLICY}{MONITOR}{PROT} =~ s/\,\,/\,/g;
#		} ;
#	/policy_set_monitor/i && do {
#		$retVal = "$CLI{SET_MONITOR_PREFIX} -f $NV{opspec}";
#		$CLI{POLICY}{MONITOR}{OPSPEC} =~ s/$NV{opspec}//g;
#		if(defined $NV{proto} and $NV{proto} ne "") {
#			$CLI{POLICY}{MONITOR}{PROTO} = $NV{proto};
#			$retVal .= " -p $NV{proto}";
#		}
#		};
#	/policy_show/i && do {
#		$retVal = "$CLI{SHOW_CMD_PREFIX} $NV{policy} ";
#		};
#	/policy_extension/i && do {
#		$retVal = "$CLI{EXTENSION_PREFIX} $NV{method} $NV{oper} $NV{policy} $NV{ext}";
#		};
#		#logresult("fail", "Invalid command type: $cmd_type");
#	}
#
#	#logcomment("Cmd generation = $retVal");
#	return $retVal;
#}

=head1 NAME
CLI_Expected_Str

=head2 DESCRIPTION
This API provides default EXPECTED array of text to match the policy CLI output

Release specific expected output is retrieved based template definition for that release

=over

=item ARRAY
array of input name value pairs.
The name value pairs are used to form the expected output
based on the EXPECTED template predefined output.

=cut

sub CLI_Expected_Str
{
	my %retVal = ();
	my %NV = @_;
	my $cmd_type = $NV{cmd_type};


	my $version = $NV{version};
	logcomment("Looking for Expected value for version $version = [$cmd_type]");


	# use the cmd_type of the form <cmdcode>_<cmdtype> to map the output from template
	my ($cmdcode, $cmdtype) = split /_/, $cmd_type;


	if($version ne "")
	{
		logcomment("$version,$cmdcode,$cmdtype = $ExpectedTemplate{$version}{$cmdcode}{$cmdtype}{success}");

		# get the expected output = success
		$retVal{success} = $ExpectedTemplate{$version}{$cmdcode}{$cmdtype}{success};

		# get the array of not-expected output = failure
		# this will be useful where expected output is null/-blind/.* (basically nothing to expect)
		# so failure cases are checked against the output to decide failure
		$retVal{failure} = $ExpectedTemplate{$version}{$cmdcode}{$cmdtype}{failure};
	}


	# if the $version does NOT have the definition, look for base values
	if($retVal{success} eq "") 
	{
		logcomment("default,$cmdcode,$cmdtype = $ExpectedTemplate{default}{$cmdcode}{$cmdtype}{success}");
		$retVal{success} = $ExpectedTemplate{default}{$cmdcode}{$cmdtype}{success};
	}


	if($retVal{failure} eq "") {
		logcomment("default,$cmdcode,$cmdtype = $ExpectedTemplate{default}{$cmdcode}{$cmdtype}{failure}");
		$retVal{failure} = $ExpectedTemplate{default}{$cmdcode}{$cmdtype}{failure};
	}


	# replacing the actual values for the tagged parameters
	$retVal{success} =~ s/<policy>/$NV{policy}/g;
	$retVal{success} =~ s/<policystate>/$NV{policystate}/g;

	my @out = ();
	if($retVal{failure} ne "-blind"){
	
		foreach my $e ( @{$retVal{failure}} )
		{
			$e =~ s/<policy>/$NV{policy}/g;
			$e =~ s/<policystate>/$NV{policystate}/g;
			push @out, $e;
		}
	}
	
	$retVal{failure} = [@out];

	#logcomment("Expected value for $cmd_type = $logval");
	logcomment("Expected value for $cmd_type, success= $retVal{success}, failure = $retVal{failure}");

	return \%retVal;
}


1;
