# -*- perl -*-
#  $Id: //depot/prod/test/main/lib/Grammer.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Grammer;

use strict;

use warnings;
use Utils qw(make_host make_conn);
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF OBJECT);
use Tharn qw(say logcomment logresult);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;
use FPolicy::Utils qw(make_NV make_hash_fromArrayHash make_arrayhash);
use FPolicy::Constants qw (%CmdTemplate);

require Exporter;
use base qw(Exporter);
our @EXPORT_OK = qw(
			ValidateRuleOfGrammer
			GenerateCmd
			ValidateParams
		);


=head1 NAME
CommandFromTemplate

=head2 DESCRIPTION
Reads the command semantics from the template
and forms the complete command with the supplied values

=over

=item ARRAY
list of name value pairs required for each command to be formed
this varies with each command type

This module tries to map all the name value pairs to the command template and form the complete commmand

If there are additional name value pairs, those will be appended to the command

=cut

sub CommandFromTemplate (@)
{
	my %NV = @_;
	my @options = @_;
	
	
	my $version = $NV{version};
	
	my $code = substr($NV{cmd_type}, 0, 3);
	
	#fpolicy vol and ext commands take different input from IC.0 or 7.3. vol_sho and ext_sho are corresponding entries in 7_3cmdTemplate
	if((($version eq "7.3") or ($version eq "8.0")) and ($code eq "vol" or $code eq "ext") and (($NV{oper} eq "show") or ($NV{oper} eq "reset"))){ 
#		$code .= "_" . substr($NV{oper}, 0, 3);
		$code .= "_sho";
	}

	# version control for command code lookup from template
	my $ref = ( $version ne "" )? 
			$CmdTemplate{$version} :
			$CmdTemplate{default} ;

	# map the command from template
	my $cmd = ${$ref}{$code};

	logcomment("Command from Template for version $version: [$cmd]");
	

#####	### Rajaram - Modified for Vfiler support
	my $tmp = $NV{vfiler};
#####	

	# populate the standard tokens for the command as defined in the template
	foreach my $key (keys %NV) 
	{ 
		my $tmpstr = $NV{$key};
		if( ref($NV{$key}) eq "ARRAY" ) 
		{
			my $str = "";
			foreach my $k ( @{$NV{$key}} )
			{ $str .= "$k,"; }

			chop $str;

			logcomment("Converted opspec $tmpstr from array $NV{$key} 
				to str = $str, arraysize = ". $#$tmpstr);

			$NV{$key} = $str;
		}
		
		# convert the tag <name> to actual value(s) for this instance
		$cmd =~ s/\<$key\>/$NV{$key}/g; delete $NV{$key}; 
	}
	
		
	# handle vfiler later
#	if($NV{vfiler} ne "" and $NV{vfiler} ne "vfiler0") {
#		my $tmp = "vfiler run $NV{vfiler} $cmd";
#		$cmd = $tmp;
#	}

	#append the additional tokens in the command
	foreach my $key (keys %NV) 
	{ $cmd .= " $NV{$key} "; }

	# if command was not found report error
	if($cmd eq "") 
	{ 
		logcomment("null/Invalid command generated at Cmd Grammer for @options"); 
	}
	
	
####### Rajaram - Modified for Vfiler support

	## this need to be handled gracefully
	#if($NV{vfiler} ne "" and $NV{vfiler} ne "vfiler0") 
	#{
	#	my $c = "vfiler run $cmd";
	#	$cmd = $c;
	#}

	# this need to be handled gracefully
	if($tmp ne "" and $tmp ne "vfiler0") 
	{
		my $c = "vfiler run $tmp $cmd";
		$cmd = $c;
	}

######

	# return the formed command
	return $cmd;
}

=head1 NAME
ValidateRuleofGrammer

=head2 DESCRIPTION
This API is used for validating the input parameters 
There are different cases where the number of input parameters differ and need to be validated differently. Which this module does !!

##############################
# 
# MONITOR
##############################
# inclusion
# 	add, remove, set
# exclusion
# 	add, remove, set
# Grammer:
#---------
# <fpolicy:mandatory> <monitor:mandatory> <add,remove,set:mandatory> <policy:mandatory> <-p nfs,cifs:mandatory> <-f opspec:mandatory>
##############################
##############################
# VOLUME
##############################
# inclusion
# 	add, remove, set, show, reset, eval
# exclusion
# 	add, remove, set, show, reset, eval
# Grammer:
#---------
# <fpolicy:mandatory> <volume:mandatory> <inc/exc:mandatory> <add/remove/set:mandatory> <policy:mandatory> <vol_spec:mandatory>
# <fpolicy:mandatory> <volume:mandatory> <inc/exc:mandatory> <reset/show/eval:mandatory> <policy:mandatory>
##############################

=over

=item ARRAY
list of name value pairs required for each command to be formed
this varies with each command type

=cut
sub ValidateRuleOfGrammer (@)
{
	my @options = @_;
	my %NV = @_;
	my $cmd_type = $NV{cmd_type};

	logcomment("ValidateRuleOfGrammer = @options");
	# remove additional/internal parameters
	delete $NV{version};

	# convert to array for validation
	my @arr = make_arrayhash(\%NV);

	print "####**** Validating Grammer for $cmd_type*** $NV{cmd_type}****** \n";

	if( ${cmd_type} =~ /mon([i|it|tor])?/i ) 
	{
		print "@@@@@@ Entered monitored @@@@@";
		# this has couple of diferent possibilities 
		# handling monitor operations seperately
		return ValidateParams(@arr);

	} 
	elsif ( ${cmd_type} =~ /vol([u|um|ume])?/i ) 
	{

		my %parms = validate
		(
			@arr, 
			{
			cmd_type => {	type => SCALAR, 
					regex => qr/(\w+)/
				},

			command => {	type => SCALAR, 
					regex => qr/(\w+)/
				},

			oper => { 	type => SCALAR, 
					regex => qr/[add|remove|set|reset|show|eval]/i 
				},

			method => { 	type => SCALAR, 
					regex => qr/[inc|exc](l)?(u)?(s)?(i)?(o)?(n)?/i 
				},

			volume => { 	type => SCALAR, 
					regex => qr/((\w+)(,(\w)+)*)?/i, 
					optional => 1 
				},

			cmd => { 	type => SCALAR, 
					regex => qr/fpolicy(\s)+vol([u|um|ume])?(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(,(\w+))*/i, 
					optional => 1 
				},

			policy => {	type => SCALAR, 
					regex => qr/(\w+)/i
				},

			vfiler => {	type => SCALAR, 
					regex => qr/(\w+)/i, 
					optional => 1
				}
			}
		);

	} 
	
	# Added by Prashant(prash@netapp.com) to validate fpolicy options command
	
		elsif ( ${cmd_type} =~ /option/ )
				{       
					my %parms = validate
					(
								@arr, 
								{
								cmd_type => {	type => SCALAR, 
										regex => qr/(\w+)/
									},
								command => {	type => SCALAR, 
										regex => qr/(\w+)/
									},
								method => { 	type => SCALAR, 
										regex => qr/[required|secondary_servers]/i 
									},
								value => { 	type => SCALAR, 
								
# commenting regex to accomodate negative test cases for Secondary IP address i.e., incomplete Ip address.
										# regex => qr/[on|off| ]/i
									},
								
								#cmd => { 	type => SCALAR, 
								#		regex => qr/fpolicy(\s)+ext([e|en|ens|ensi|ensio|ension])?(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(,(\w+))*/i, 
								#		optional => 1 
								#	},
								
								policy => {	type => SCALAR, 
										regex => qr/(\w+)/i
									},
								vfiler => {	type => SCALAR, 
										regex => qr/(\w+)/i, 
										optional => 1
									}
								}
							);
					
		} 
	elsif ( ${cmd_type} =~ /status/i ) 
		{
			my %parms = validate
			(
				@arr, 
				{
				cmd_type => {	type => SCALAR, 
						regex => qr/(\w+)/
					},
				command => {	type => SCALAR, 
#						default => "snap", 
						regex => qr/(\w+)/i
					},
				volume => {	type => SCALAR, 
						regex => qr/(\w+)/i
					},
				vfiler => {	type => SCALAR, 
						regex => qr/(\w+)/i, 
						optional => 1
					},
				}
			);
	}		


	elsif ( ${cmd_type} =~ /ext([e|en|ens|ensi|ensio|ension])?/i ) 
	{
		
		my %parms = validate
		(
			@arr, 
			{
			cmd_type => {	type => SCALAR, 
					regex => qr/(\w+)/
				},
			command => {	type => SCALAR, 
					regex => qr/(\w+)/
				},
			oper => { 	type => SCALAR, 
					regex => qr/[add|remove|set|reset|show]/i 
				},
			method => { 	type => SCALAR, 
					regex => qr/[inc|exc](l)?(u)?(d)?(e)?/i 
				},
			extension => { 	type => SCALAR, 
					regex => qr/(\w+)(,(\w)+)*/i
				},
			cmd => { 	type => SCALAR, 
					regex => qr/fpolicy(\s)+ext([e|en|ens|ensi|ensio|ension])?(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(,(\w+))*/i, 
					optional => 1 
				},
			policy => {	type => SCALAR, 
					regex => qr/(\w+)/i
				},
			vfiler => {	type => SCALAR, 
					regex => qr/(\w+)/i, 
					optional => 1
				}
			}
		);

	} 
	 
	elsif ( ${cmd_type} =~ /[create|destroy|show|enable|disable]/i ) 
	{
		my %parms = validate
		(
			@arr, 
			{
			cmd_type => {	type => SCALAR, 
					regex => qr/(\w+)/
				},
			policy => {	type => SCALAR, 
#					regex => qr/(\w+)/i
				},
			type => {	type => SCALAR, 
					default => "screen", 
					regex => qr/(\w+)/i, 
					optional => 1
				},
			vfiler => {	type => SCALAR, 
					regex => qr/(\w+)/i, 
					optional => 1
				},
			command => {	type => SCALAR, 
					regex => qr/(\w+)/, 
					optional => 1
				},
			}
		);
	}
	
	elsif ( ${cmd_type} =~ /status/i ) 
		{
			my %parms = validate
			(
				@arr, 
				{
				cmd_type => {	type => SCALAR, 
						regex => qr/(\w+)/
					},
				command => {	type => SCALAR, 
#						default => "snap", 
						regex => qr/(\w+)/i
					},
				volume => {	type => SCALAR, 
						regex => qr/(\w+)/i
					},
				vfiler => {	type => SCALAR, 
						regex => qr/(\w+)/i, 
						optional => 1
					},
				}
			);
	}
	
		
	else 
	{
		logcomment("Grammer: Invalid command code ${cmd_type}, @options");
		logcomment("ValidateRuleofGrammer = FAIL");
		return 1;
	}

	logcomment("Validate Command Grammer = PASS");
	# return TRUE, since validate has qualified the rule for volume
	return 0;
}

=head1 NAME
GenerateCmd

=head2 DESCRIPTION
This API is used to form the command as required for CLI operations
It uses Validate and command generate from template APIs to accomplish
the task
This also uses the version control mechanism to form the command as per 
the release definitions.


=over

=item ARRAY
list of name value pairs required for each command to be formed
this varies with each command type

=cut

sub GenerateCmd
{
	my @options = @_;
	my $cmd = "";

	logcomment("GenerateCmd = @options");

	## making the command template Generic
	push @options, "command";
	push @options, "fpolicy";

	# validate the input parameters
	if( ValidateRuleOfGrammer ( @options ) == 0 ) 
	{
		# get the command from the template
		$cmd = CommandFromTemplate ( @options );

	} 
	else 
	{
		logcomment("Grammer: Validation Command Grammer failed for parameters @options");
	}

	logcomment("Grammer: Generated Command: [$cmd]");

	return $cmd;
}


=head1 NAME
ValidateParams

=head2 DESCRIPTION
This API is used for validating the input parameters for monitor operations
There are different cases where the number of input parameters differ and need to be validated differently.

=over

=item cmd_type
Command type

=item ARRAY
list of name value pairs required for each command to be formed
this varies with each command type

=cut

sub ValidateParams
{
	my $cmd_type = shift;
	my %NV = @_;

	# this function is currently only for monitor operations
	SWITCH: for ( $cmd_type ) {

	/monitor/ && do {

	if( $NV{oper} =~ /[add|remove|set]/i ) 
	{
		return eval {

		my %parm = validate
		(	
			@_, 
			{
			conn => { 	type => SCALAR|OBJECT, 
					optional => 1 
				},
			policy => { 	type => SCALAR, 
					regex =>  qr/(\w+)/, 
					optional => 1 
				},
			oper => { 	type => SCALAR, 
					regex => qr/[add|remove|show|set|reset]/i 
				},
			opspec => { 	type => SCALAR|ARRAYREF, 
					optional => 1 
				},
			protocol => { 	type => SCALAR, 
					regex => qr/[nfs|cifs]/i, 
					optional => 1 
				},
			vfiler => { 	type => SCALAR, 
					regex => qr/(\w)+/, 
					optional => 1
				},
			result => { 	type => SCALAR, 
					default => "PASS", 
					regex =>  qr/[pass|fail]/i, 
					optional => 1 
				},
			expected => { 	type => SCALAR|ARRAYREF, 
					optional => 1
				}
			}
		);

		}; # eval

	} 
	elsif( $NV{oper} =~ /[show|reset]/i )
	{
		return eval {
		my %parm = validate
		(
			@_, 
			{
			conn => { 	type => SCALAR|OBJECT, 
					optional => 1 
				},
			policy => { 	type => SCALAR, 
					regex =>  qr/(\w+)/, 
					optional => 1 
				},
			oper => { 	type => SCALAR, 
					regex => qr/[add|remove|show|set|reset]/i 
				},
			vfiler => { 	type => SCALAR, 
					regex => qr/(\w)+/, 
					optional => 1
				},
			result => { 	type => SCALAR, 
					default => "PASS", 
					regex =>  qr/[pass|fail]/i, 
					optional => 1 
				},
			expected => { 	type => SCALAR|ARRAYREF, 
					optional => 1
				}
			}
		);

		}; #eval
	}
	} # do
	}; # switch

	return 0;
}



1;

