# -*- perl -*-
#  $Id: //depot/prod/test/main/lib/PServer.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::PServer;

use strict;

#use warnings;
use Switch;
use Utils;
use Carp qw(croak);
use TestEnv::MinWin qw(:all);
use Tharn;
use Carp ;
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF OBJECT);
use Utils qw(make_host make_conn);
use Tharn qw(say logcomment logresult subtest);
use SayWrap qw(sayEOF);
use Subtest;
use config_base;
use filer;
use ModelInfo;
use FPolicy::Execute;
use TestEnv::MinWin;
use FPolicy::Utils qw(make_NV);
use FPolicy::Constants qw($POLICY_SERVER_EVENT_EXE $POLICY_SERVER_TRANSFER_EXE $RHOSTS_DB $POLICY_SERVER_EVENT_EXEPATH $POLICY_SERVER_TRANSFER_EXEPATH);


sub new
{
    my $object = shift;
	my @options = @_;

	my %NV = make_NV(@options);
	my $filer_host = make_host("$NV{filer}");
	my $filer_conn = make_conn($filer_host); 
	

       my $class = ref($object) || $object;
       my $self = { 
		client => $NV{client},
		filer => $NV{filer},
		filer_ip => $NV{filer_ip},
		policy => $NV{policy},
		monitor => $NV{monitor},
		proto => $NV{proto},
		traffic_type => $NV{traffic_type},
		pserver_pri => $NV{pserver_pri},
		pserver_sec => $NV{pserver_sec},
		filer_conn => $filer_conn,
	};

	logcomment("Policy Server Initialized with 
		client = $NV{client}
		filer => $NV{filer}
		filer_ip => $NV{filer_ip}
		policy => $NV{policy}
		monitor => $NV{monitor}
		proto => $NV{proto}
		traffic_type => $NV{traffic_type}
		pserver_pri => $NV{pserver_pri}
		pserver_sec => $NV{pserver_sec}" );
		
       bless($self, $class); 
       return $self;
}

sub CheckClientIn_RhostDB
{
	my $client = validate(@_, 
		{ client => {type => SCALAR, regex => qr/(\w+)/} });

	if(-f "$RHOSTS_DB/$client") 
	{
		return 0;
	}else {
		return 1;
	}
}

sub FindSID
{
	my $self = shift;
	
	my $DomainNetbios = main::param("DOMAIN_NETBIOS","-mesg", "DOMAIN_NETBIOS");
	my $DomainUsername = main::param("DOMAIN_USERNAME","-mesg", "DOMAIN_USERNAME");
	my $DomainAdminSID  = sayEOF($self->{filer_conn}, "cifs lookup $DomainNetbios\\$DomainUsername");
	$DomainAdminSID  =~ s/SID = (.*)/$1/;
	$self->{DomainAdminSID} = $DomainAdminSID;
	
	my $AdminsSID  = sayEOF($self->{filer_conn}, "cifs lookup BUILTIN\\Administrators");
	$AdminsSID  =~ s/SID = (.*)/$1/;
	$self->{AdminsSID} = $AdminsSID;
}



sub launch_pserverevent()
{
	my $self = shift;
	my %args = validate(@_, {
				polObj => {type => OBJECT},
                protocol => {type => SCALAR, default => ""},
				monitor => {type => SCALAR, default => ""},
				optional_param => {type => SCALAR, default => "-T Auto", optional => 1},
				server_count => {type => SCALAR, default => 1, optional => 1},
                result => {type => SCALAR, default => "pass", optional => 1}
				});
	
	
	my $host = make_host("$self->{pserver_pri};hosttype=windows");
	my $conn = make_conn($host);

	my $res = 0;
	
	my $i = 0;
	while ($i < 2) {
		say($conn, "cmd /c $POLICY_SERVER_EVENT_EXEPATH\\$POLICY_SERVER_EVENT_EXE $self->{filer_ip} $self->{policy}  $args{protocol} $args{monitor} $args{optional_param} ", '-blind');
		sleep (5);

		logcomment("Check presence of $POLICY_SERVER_EVENT_EXE");
		my (undef,undef,$status) = say($conn, "cmd /c tasklist /FI \"IMAGENAME eq $POLICY_SERVER_EVENT_EXE\"", "dummy", -ok);
		
		if($args{server_count} == 1)
		{
			if($status =~ /No tasks/i) {   
				#sandeepp No tasks are running
				;
			} else {
				# make sure the app is ready after registeration
				# sleep(30);
				$i = 5;
			}
		}
		else
		{	
			my $count = 0;
			while ($status =~ /$POLICY_SERVER_EVENT_EXE/g)
			{ 
				$count++ ;
			}
			if($count < $args{server_count}) {   
				;
			} else {
				# make sure the app is ready after registeration
				# sleep(30);
				$i = 5;
			}
		}
		$i++;
	}
	
	if($i >= 5 and $args{result} eq "pass") 
	{
		logcomment("PASS: Successfully launched $POLICY_SERVER_EVENT_EXE");
		$res = 0;
	}
	elsif ( $i >= 5 and $args{result} eq "fail")
	{
		logcomment("FAIL: Expected the launching of $POLICY_SERVER_EVENT_EXE to fail but it Passed");
		$res = 1;
	}
	elsif ( $i < 5 and $args{result} eq "pass")
	{
		logcomment("FAIL: Expected the launching of $POLICY_SERVER_EVENT_EXE to PASS but it Failed");
		$res = 1;
	}
	else
	{
		logcomment("PASS: Expected the launching of $POLICY_SERVER_EVENT_EXE to FAIL and it Failed");
		$res = 0;
	}
	
	return $res;
}

sub cleanup_pserverevent()
{
	my $self = shift;
		
	my $host = make_host("$self->{pserver_pri};hosttype=windows");
	my $conn = make_conn($host);
	
	eval {
		logcomment("Check presence of $POLICY_SERVER_EVENT_EXE");
		my(undef,undef,$msgc) = say($conn, "cmd /c tasklist \/FI \"IMAGENAME eq $POLICY_SERVER_EVENT_EXE\"", "dummy", -ok);

		if($msgc =~ /No tasks/i) {   #sandeepp No tasks are running
			;
		}
		else {
			logcomment("Kill the process $POLICY_SERVER_EVENT_EXE");
			(undef,undef,$msgc) = say($conn, "cmd /c taskkill \/F \/IM $POLICY_SERVER_EVENT_EXE", "dummy", -ok);
		}
	};
	
	if($@) {
		logcomment("Error during pserver cleanup: $@");
	}
	
}

sub launch_vscanserver()
{
	my $self = shift;
	my %args = validate(@_, {
				polObj => {type => OBJECT},
				optional_param => {type => SCALAR, default => "", optional => 1},
                result => {type => SCALAR, default => "pass", optional => 1}
				});
	
	
	my $host = make_host("$self->{pserver_pri};hosttype=windows");
	my $conn = make_conn($host);

	my $res = 0;
	my $VSCAN_SERVER = "nvsc.exe";
	
	my $i = 0;
	while ($i < 2) {
		say($conn, "cmd /c $POLICY_SERVER_EVENT_EXEPATH\\$VSCAN_SERVER $self->{filer_ip} ", '-blind');
		sleep (5);

		logcomment("Check presence of $VSCAN_SERVER");
		my (undef,undef,$status) = say($conn, "cmd /c tasklist /FI \"IMAGENAME eq $VSCAN_SERVER\"", "dummy", -ok);
		
			if($status =~ /No tasks/i) {   
				#sandeepp No tasks are running
				;
			} else {
				# make sure the app is ready after registeration
				# sleep(30);
				$i = 5;
			}

		$i++;
	}
	
	if($i >= 5 and $args{result} eq "pass") 
	{
		logcomment("PASS: Successfully launched $VSCAN_SERVER");
		$res = 0;
	}
	elsif ( $i >= 5 and $args{result} eq "fail")
	{
		logcomment("FAIL: Expected the launching of $VSCAN_SERVER to fail but it Passed");
		$res = 1;
	}
	elsif ( $i < 5 and $args{result} eq "pass")
	{
		logcomment("FAIL: Expected the launching of $VSCAN_SERVER to PASS but it Failed");
		$res = 1;
	}
	else
	{
		logcomment("PASS: Expected the launching of $VSCAN_SERVER to FAIL and it Failed");
		$res = 0;
	}
	
	return $res;
}

sub cleanup_vscanserver()
{
	my $self = shift;
	my $VSCAN_SERVER = "nvsc.exe";
		
	my $host = make_host("$self->{pserver_pri};hosttype=windows");
	my $conn = make_conn($host);
	
	eval {
		logcomment("Check presence of $VSCAN_SERVER");
		my(undef,undef,$msgc) = say($conn, "cmd /c tasklist \/FI \"IMAGENAME eq $VSCAN_SERVER\"", "dummy", -ok);

		if($msgc =~ /No tasks/i) {   #sandeepp No tasks are running
			;
		}
		else {
			logcomment("Kill the process $VSCAN_SERVER");
			(undef,undef,$msgc) = say($conn, "cmd /c taskkill \/F \/IM $VSCAN_SERVER", "dummy", -ok);
		}
	};
	
	if($@) {
		logcomment("Error during pserver cleanup: $@");
	}
	
}


sub check_pserverevent()
{
	my $self = shift;
	my %args = validate(@_, {
				polObj => {type => OBJECT},
                protocol => {type => SCALAR, default => ""},
				monitor => {type => SCALAR, default => ""},
				optional_param => {type => SCALAR, default => "-T Auto", optional => 1},
				server_count => {type => SCALAR, default => 1, optional => 1},
				force_launch => {type => SCALAR, default => "false", optional => 1},
                result => {type => SCALAR, default => "pass", optional => 1}
				});
	
	## Added by Rajaram to check if Server is connected to the respective policy
	my $vfiler = main::param("VFILER","-mesg", "VFILER");
	my $status;
	my $pserverStatus = 1;
	my $res = 0;
	
	if($vfiler ne "vfiler0")
	{
		$status  = sayEOF($self->{filer_conn}, "vfiler run $vfiler fpolicy servers show $self->{policy}");
	}
	else
	{
		$status  = sayEOF($self->{filer_conn}, "fpolicy servers show $self->{policy}");
	}
	if(!($status =~ /$self->{pserver_pri}/i))
	{
		$pserverStatus = 0;
		#launch and exit
	}
	
	my $zapi_server = $args{polObj}->{zapi}->fpolicy_server_list_options($self->{policy},$self->{filer_ip},"pass");
	logcomment("$zapi_server");
	
	my $zapi_policy = $args{polObj}->{zapi}->fpolicy_list_info($self->{policy},$self->{filer_ip});
	logcomment("$zapi_policy");
	
	if( ($args{optional_param} =~ /version2=true/ and $zapi_server !~ /<is-version2>true/) or 
	    ($args{optional_param} !~ /version2=true/ and $zapi_server =~ /<is-version2>true/) )
	{
		$pserverStatus = 0;
	}
	if( ($args{optional_param} =~ /size_and_owner=true/ and $zapi_server !~ /<is-size-and-owner-required>true/) or 
	    ($args{optional_param} !~ /size_and_owner=true/ and $zapi_server =~ /<is-size-and-owner-required>true/) )
	{
		$pserverStatus = 0;
	}
	if( ($args{optional_param} =~ /async=true/ and $zapi_server !~ /<is-asynchronous>true/) or 
	    ($args{optional_param} !~ /async=true/ and $zapi_server =~ /<is-asynchronous>true/) )
	{
		$pserverStatus = 0;
	}
	if( ($args{optional_param} =~ /snapid=true/ and $zapi_server !~ /<is-snapid-required>true/) or 
	    ($args{optional_param} !~ /snapid=true/ and $zapi_server =~ /<is-snapid-required>true/) )
	{
		$pserverStatus = 0;
	}
	
	if( ($args{optional_param} =~ /-I true/ and $zapi_policy !~ /<is-i2p-enabled>true/) or 
	    ($args{optional_param} !~ /-I true/ and $zapi_policy =~ /<is-i2p-enabled>true/) )
	{
		$pserverStatus = 0;
	}
	if( ($args{optional_param} =~ /ads=true/ and $zapi_policy !~ /<is-ads-monitored>true/) or 
	    ($args{optional_param} !~ /ads=true/ and $zapi_policy =~ /<is-ads-monitored>true/) )
	{
		$pserverStatus = 0;
	}
	
	if( ($pserverStatus == 0 ) or ($args{force_launch} eq "true") )
	{
		$args{polObj}->{pserver}->cleanup_pserverevent();
		$res |= $args{polObj}->{pserver}->launch_pserverevent(polObj => $args{polObj}, protocol => $args{protocol}, monitor => $args{monitor}, optional_param => $args{optional_param}, server_count => $args{server_count}, result => $args{result} );
	}
	
	return $res;
}



sub launch($;@)
{
	#my $self = $_;
	#my $filer = $self->{filer_ip};
	#my $policy = $self->{policy};
	#my $monitor = $self->{monitor};
	#my $client = $self->{client};
	#my $pserver = $self->{pserver_pri};
	#my $proto = $self->{proto};
	#$proto = ($self->{proto} eq "")? "" : " -p $proto "; 
	my $pserver = shift;
	my $self = shift;
	logcomment("Value passed to launch sub routine is: $self");
	
	#my %opts = validate(@_, {
	#	is_mult => {type => SCALAR, default => "", optional => 1}
    #});
    #logcomment ("Lunch value is : $opts{is_mult} ");
#	logcomment("Check MinWin Complaint");
	#if($self->{host} =~ /windows/) {
#		if(!TestEnv::MinWin::minwin_compatible($pserver) ) {
#			logcomment("MinWin Compatibility failed for client $pserver");
#			return 1;
#		}
	#}

#	logcomment("Connecting to Policy Server $pserver");
#	$self->{host} = make_host("$pserver;hosttype=windows");
#	$self->{CONN} = make_conn($self->{host});

# these below steps are converted to seperate .thpl files.
# retaining it here for future, when launch/relaunch is taken care of
#	logcomment("Check presence of $POLICY_SERVER_EVENT_EXE");
#	my $status = say($self->{CONN}, "cmd /c dir $POLICY_SERVER_EVENT_EXEPATH\\$POLICY_SERVER_EVENT_EXE && CD");
#
#	if($status =~ /file not found/i) {
#		logcomment("Application $POLICY_SERVER_EVENT_EXE not found at the specified path");
#		return 1;
#	}
#
#	logcomment("Check presence of $POLICY_SERVER_TRANSFER_EXE");
#	$status = say($self->{CONN}, "cmd /c dir $POLICY_SERVER_TRANSFER_EXEPATH\\$POLICY_SERVER_TRANSFER_EXE && CD");
#
#	if($status =~ /file not found/i) {
#		logcomment("Application $POLICY_SERVER_TRANSFER_EXE not found at the specified path");
#		return 1;
#	}
#
#	#my $cmd = "cmd /c $POLICY_SERVER_EVENT_EXE  $filer $policy -o $monitor  $proto -l logfile -d debuglog.txt ";
#	my $cmd = "cmd /c $POLICY_SERVER_EVENT_EXEPATH\\$POLICY_SERVER_EVENT_EXE  $filer $policy -o $monitor  $proto";
##	#my $cmd = "cmd /c $POLICY_SERVER_EVENT_EXEPATH\\$POLICY_SERVER_EVENT_EXE 1 1 ";
##
#	logcomment("Launch $POLICY_SERVER_EVENT_EXE using $cmd");
#	my $msg = say($self->{CONN}, $cmd, '-blind');
##	subtest("FPolicy/fft/pserver/launch_pserver_event.thpl", "CLIENT=$self->{client}", "OPTION=PSERVER_EVENT");
##
##	# wait for the command to launch, this is required
#	sleep(15);
#
#	my $msgc = say($self->{CONN}, "tasklist \/FI \"IMAGENAME eq $POLICY_SERVER_EVENT_EXE\"");
#
#	if($msgc =~ /No tasks are running/) {
#		logcomment("Could not launch $POLICY_SERVER_EVENT_EXE");
#		return 1;
#	} 
#	else 
#	{
#		my $tcmd = "cmd /k $POLICY_SERVER_TRANSFER_EXEPATH\\$POLICY_SERVER_TRANSFER_EXE";

#		logcomment("Launch $POLICY_SERVER_TRANSFER_EXE using $tcmd");
#		my $tmsg = say($self->{CONN}, $tcmd, '-blind');

		# wait for command to launch, this is required
#		sleep(20);

#		my $msgc = say($self->{CONN}, "tasklist \/FI \"IMAGENAME eq $POLICY_SERVER_TRANSFER_EXE");

#		if($msgc =~ /No tasks are running/) {
#			logcomment("Could not launch $POLICY_SERVER_TRANSFER_EXE");
#			return 1;
#		} else {
#			if(!defined $self->{server} ) {
#				$self->ConnectToTransferServer();
#			}
#		}
#	}
#} elsif($OPTION eq "PSERVER_TRANSFER") {

        # cleanup
        logcomment("Pserver here is : $pserver");
my $host = make_host("$pserver;hosttype=windows");
my $conn = make_conn($host); 


       logcomment("PServer  Transfer - Cleanup");

# Added By prashant to support Multiple server test cases	   
if($self eq "") {
        eval {
        # close the session

        logcomment("Check presence of $POLICY_SERVER_TRANSFER_EXE");
        my(undef,undef,$msgc) = say($conn, "cmd /c tasklist /FI \"IMAGENAME eq $POLICY_SERVER_TRANSFER_EXE\"", "dummy", -ok);
                logcomment("RELAUNCH_PSERVER RETURNED: $msgc");
        if($msgc =~ /No tasks/i) {   #No tasks are running
                ;
        }
        else
        {
                logcomment("Kill the process $POLICY_SERVER_TRANSFER_EXE");
                (undef,undef,$msgc) = say($conn, "cmd /c taskkill \/F \/IM $POLICY_SERVER_TRANSFER_EXE", "dummy", -ok);
        }

        };

}
        if($@) {
                logcomment("Error during cleanup: $@");
        }

        # say($conn, "cmd /k $POLICY_SERVER_TRANSFER_EXEPATH\\$POLICY_SERVER_TRANSFER_EXE", '-blind');
		if($self eq "") {
        say($conn, "cmd /c $POLICY_SERVER_TRANSFER_EXEPATH\\$POLICY_SERVER_TRANSFER_EXE", '-blind');
		} else {
			say($conn, "cmd /c $POLICY_SERVER_TRANSFER_EXEPATH\\$POLICY_SERVER_TRANSFER_EXE $self", '-blind');
		}
        sleep(10);
        # return success
        return 0;
}


=head1 NAME
ConnectToTransferServer

=head2 DESCRIPTION
Connect to the Event transfer server on the windows client machine

=head2 ROUTINES
	$pol->{pserver}->ConnectToTransferServer();

=over

=item result
Stores the connected session in the object
returns PASS (0) / FAIL (1)

=cut

sub ConnectToTransferServer ($;@)
{
      
	my $self = shift;
	#my %opts = @_;
    #my %opts = validate(@_, {
    #    		is_mult => {type => SCALAR, default => "", optional => 1}
    #            });
	
	my %opts = validate(@_, {
        		is_mult => {type => SCALAR, default => "", optional => 1}
                });
	my $vfiler = main::param("VFILER","-mesg", "VFILER");
	
	logcomment("Check fpserver");
	## Added by Rajaram to check if Server is connected to the respective policy
	my $res;
	if($vfiler ne "vfiler0")
	{
		$res  = sayEOF($self->{filer_conn}, "vfiler run $vfiler fpolicy servers show $self->{policy}");
	}
	else
	{
		$res  = sayEOF($self->{filer_conn}, "fpolicy servers show $self->{policy}");
	}
	if(!($res =~ /$self->{pserver_pri}/i))
	{
		logresult("fatal", "Fpolicy Server is not registered to the policy");
	}
	
	use Net::Telnet;

	my $pserver = $self->{pserver_pri};
	
	## Added By prashant(prash) to support multiple server test cases.
	if($opts{is_mult} eq "") {
	launch($pserver);
	} else	{
		launch($pserver, $opts{is_mult} );
	}
	logcomment("Connecting to Policy server $pserver via $POLICY_SERVER_TRANSFER_EXE");
	logcomment("#### Port to be used for connecting server is $opts{is_mult} ");

	my $obj;
	my $obj_new;
	
	if($opts{is_mult} ne "") {
		    # Added By prashant to support multiple server test cases.
	eval {
		$obj = new Net::Telnet(Host => $pserver,
			Timeout => 60,
#			Timeout => 10,
			Output_record_separator => '', 
			Port => "27020") or logcomment("Die $@");

		};
		logcomment("Port Number 27020 is used\n");
	} else {
	
	eval {
	# later, the port number should be obtained from Constants.pm
	$obj = new Net::Telnet(Host => $pserver,
			Timeout => 60,
#			Timeout => 10,
			Output_record_separator => '', 
			Port => "27019") or logcomment("Die $@");

	};
		logcomment("Port Number 27019 is used\n");
	}
	if($@) {
		logresult("fatal", "Unable to connect to Transfer Server $self->{pserver_pri} via $POLICY_SERVER_TRANSFER_EXE, error = $@");
		#return 1;
	} else {
		logcomment("Connected to Transfer Server $self->{pserver_pri} via $POLICY_SERVER_TRANSFER_EXE = $obj");
		$self->{server} = $obj;
	}

	# Determine the SID values
	$self->FindSID();
	
	return $obj;
}

=head1 NAME
CollectEventsFromServer

=head2 DESCRIPTION
Collect Events from the transfer server on the windows client machine

This routine can be used when extra throttling or event polling mechanism is required 

This routine is currently obselete and the same functionality is incorporated in 
Verify routine

=head2 ROUTINES
	$pol->{pserver}->CollectEventsFromServer();

=over

=item result
Stores the collected events in the object for validation
returns PASS (0) / FAIL (1)

=cut

sub CollectEventsFromServer
{
	my $self = shift;
	my $pserver = $self->{pserver_pri};

	#if(!defined $self->{server} ) {
	#	$self->ConnectToTransferServer();
	#}

	my $obj = $self->{server};
	eval {
		print "Telnet object = $obj\n";
		my $i = 0;

		my $buf = "";

		# Use to throttle the incoming events
		# can be later modified for dynamic variation
		# of this value to adopt to the wait times and stress
		my $next = 2;

		# in case required, the iterations can be increased to 
		# wait for lots of events, which ideally should be avoided
		# increase the $next value to increase the collect time
		while($i++ < $next) {
		$buf .= $obj->get();
		}

		$self->{events} = $buf;
		logcomment("Obtained events = $buf");
	};
	
	logcomment("Encountered Error : $@") if($@);
}

=head1 NAME
Verify

=head2 DESCRIPTION
This API is used to Collect Events and Verify against the type of event generated

Currently, only few parameters are being validated like type of the event
All other parameters need to validated in future

=head2 ROUTINES
	$pol->{pserver}->Verify(filename => "<filename>", type => "<type of the event>");

=over

=item result
Stores the collected events in the object for validation
returns PASS (0) / FAIL (1)

=cut

sub Verify ($;@)
{
	my $self = shift;
#	my %opts = @_;

        my %opts = validate(@_, {
        		filename => {type => SCALAR, default => ""},
        		type => {type => SCALAR},
				traffic_type => {type => SCALAR, default => $self->{traffic_type}},
	            result => {type => SCALAR, default => "pass"},
	            is_sao => {type => SCALAR, default => "", optional => 1},
				is_npc => {type => SCALAR, default => "", optional => 1},
				is_rrd => {type => SCALAR, default => "", optional => 1},
				snapid => {optional => 1}
                });

#	my $filename = $opts{filename};
#	my $type = $opts{type};
#	my $traffic_type = $self->{traffic_type} if(!defined $opts{traffic_type});
#	$opts{result} = "pass" if(!defined $opts{result});

	logcomment("Traffic type is : $opts{traffic_type}");
	my $ret = 1; # start with fail
	my $nfsRes = 1; # start with fail
	my $cifsRes = 1; # start with fail

	my $i=0;
	my $timeout = 10;  # timeout for get()
	
	### Get the SID values
	my $CreaterOwnerSID = 'S-1-3-0';
	my $DomainAdminSID = $self->{DomainAdminSID};
	my $AdminsSID = $self->{AdminsSID};
	
	## Decide on expression matching, depending on the features enabled.
	
	if($opts{result} =~ /fail/i) { $timeout = 4; } # Reducing timeout in case of regative testing
	$self->{events} = "";
	my $obj = $self->{server};
	logcomment("Collecting events from $obj");
	# try twice to collect the events and match the results
	while ($i < 2) {
		$i++;
		#sleep(10);
		#$self->CollectEventsFromServer();
		eval {
			$self->{events} .= $obj->get(Timeout => $timeout);
			# getting data from the nfpc_transfer twice as a work around since the current max is 1024bytes is being tranfered at a time and 
			# in cases where both nfs and cifs traffic are being monitored this buffer only fetches 3 notifications at a time. Hence 
			# getting for the second time to accumulate all requests available in buffer.
			sleep(2);
			$self->{events} .= $obj->get(Timeout => 10);  

		};

		logcomment("Events = \n $self->{events}");
		my $events = $self->{events};
		$events =~  s/\n/ /g;
		## logcomment("^^^^^^^^^@@@@@@@@ After processing = \n $events");
		
	if( $opts{type} =~ /(getattr|setattr|lookup|link|symlink)/){
		$opts{traffic_type} = "nfs";
	}

	# EVENT types: create, delete, open, close, read, write, lookup, getattr, setattr, rename, dircreate, dirdelete, dirrename
	# This can be converted into a template 
	# which is ideal to match all the fields of the event
	# waiting for confirmation of the required fields w.r.t testing
	SWITCH: for (lc($opts{type})) {
	/(create|delete|open|close)/ && do {
		#saperate blocks to check the NFS and CIFS traffic for future use. 
		#any NFS and CIFS specific notifications can be handled. Ex: NFS snapID feature.
		
		if($opts{traffic_type} =~ /nfs/i) {
	               
	              if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /create/i)) {

	              	if($events =~ /protocol NFS Client Op: file $opts{type} Access Path:.+$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } elsif (($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /create/i))  {

	              		if($events =~ /protocol NFS Client Op: file $opts{type} Access Path:.+$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	            			$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      } else {

				if($events =~ /protocol NFS Client Op: file $opts{type} Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      	}
		      
		} #if nfs
		
		
		if($opts{traffic_type} =~ /cifs/i) {
               		if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /create/i)) {

               			if($events =~ /protocol CIFS Client Op: file $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $CreaterOwnerSID.+size: 0/) {
               				$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			} elsif ( ($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /create/i) )  {

				if($events =~ /protocol CIFS Client Op: file $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 11/) {
				        $ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			
               		} else {

               			if($events =~ /protocol CIFS Client Op: file $opts{type}/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
		}#if cifs
		
		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}
	};

	/(read|write)/ && do {
		#saperate blocks to check the NFS and CIFS traffic for future use. 
		#any NFS and CIFS specific notifications can be handled. Ex: NFS snapID feature.		
		if($opts{traffic_type} =~ /nfs/i){
		     if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /write/i)) {

	              	if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } elsif (($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /write/i))  {

	              		if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	            			$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
			} else {

				if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      	}

	# Added for RRD
			if(($opts{is_rrd} =~ /yes/i) && !($opts{type} =~ /write/i))
			{
				if($events =~ /FP_FileData/)
				{
					$ret = 0;
					logcomment("Client served data using FP_FileData with Version2 enabled");
					$i = 5; $nfsRes = 0;
				}
				else
				{	
					$ret = 1;
					logcomment("Data was not serverd using FP_FileData");
					$i = 2; $nfsRes = 1;
				}
			}				
		}#if nfs
		
		
		if($opts{traffic_type} =~ /cifs/i){
               		if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /write/i)) {
					      
               			if($events =~ /protocol CIFS Client Op: $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 0/) {
               				$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			} elsif ( ($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /write/i) )  {

				if($events =~ /protocol CIFS Client Op: $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 11/) {
				        $ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			
               		} else {

               			if($events =~ /protocol CIFS Client Op: $opts{type}/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}

	# Added for RRD
			if(($opts{is_rrd} =~ /yes/i) && !($opts{type} =~ /write/i))
			{
				if($events =~ /FP_FileData/)
				{
					$ret = 0;
					logcomment("Client served data using FP_FileData with Version2 enabled");
					$i = 5; $cifsRes = 0;
				}
				else
				{
					$ret = 1;
					logcomment("Data was not served using FP_FileData");
					$i = 2; $cifsRes = 1;
				}

			}			
		}#if cifs
		
		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}
	};
	
	/(^dircreate|^dirdelete)/ && do {

		my $ttype = $opts{type};
		$ttype =~ s/dir//;
		if($opts{traffic_type} =~ /nfs/i){

		    if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dircreate/i)) {

	              	if($events =~ /protocol NFS Client Op: directory $ttype Access Path: $opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	            } elsif (($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dirdelete/i))  {

	              	if($events =~ /protocol NFS Client Op: directory $ttype Access Path: $opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 4096/) {
	            		$ret = 0;
				logcomment("Matched NFS Op with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
		   } else {
			
			if($events =~ /protocol NFS Client Op: directory $ttype Access Path:.*$opts{filename}/) {
				$ret = 0;
				logcomment("Matched NFS Op with $opts{type}");
				$i = 5; $nfsRes = 0;
		    	}
		   }
		}#if nfs
		
		if($opts{traffic_type} =~ /cifs/i){

			if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dircreate/i)) {

				if($events =~ /protocol CIFS Client Op: directory $ttype.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $CreaterOwnerSID .+size: 0/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
				
			} elsif (($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dirdelete/i))  {

				if($events =~ /protocol CIFS Client Op: directory $ttype.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 4096/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;	
				}
			} else {
				if($events =~ /protocol CIFS Client Op: directory $ttype/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
			
		}#if cifs

		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}		
	};

	/^dirrename/ && do {
		my $ttype = $opts{type};
		$ttype =~ s/dir//;
		if($opts{traffic_type} =~ /nfs/i) {
		    if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dirrename/i)) {

	              	if($events =~ /protocol NFS Client Op: directory $ttype.+Rename Access Path: $opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 4096/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	}
	              	
	            } else {
	             				              			
			if($events =~ /protocol NFS Client Op: directory $ttype.+Rename Access Path:.*$opts{filename}/) {
				$ret = 0;
				logcomment("Matched NFS Op with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
		   }
		      	
		}#if nfs
	
		if($opts{traffic_type} =~ /cifs/i){
			if(($opts{is_sao} =~ /yes/i)) {

				if($events =~ /protocol CIFS Client Op: directory $ttype.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 4096/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
				
			} else {

				if($events =~ /protocol CIFS Client Op: directory $ttype/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
			
		}#if CIFS	

		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}		
	};

	/rename/ && do {
		if($opts{traffic_type} =~ /nfs/i){	
		    if(($opts{is_sao} =~ /yes/i)) {

	              	if($events =~ /protocol NFS Client Op: file $opts{type} .+ Rename Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {

				if($events =~ /protocol NFS Client Op: file $opts{type} .+ Rename Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
		}#if nfs
		
		if($opts{traffic_type} =~ /cifs/i){
			if(($opts{is_sao} =~ /yes/i)) {
			
				if($events =~ /protocol CIFS Client Op: file $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 11/) {
					$ret=0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
				
			} else {

				if($events =~ /protocol CIFS Client Op: file $opts{type}/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
		}#if cifs
		
		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}
	};

	/(link)/ && do {
		if(($opts{is_sao} =~ /yes/i)) {

	        	if($events =~ /protocol NFS Client Op: $opts{type} .+ Rename Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {

				if($events =~ /protocol NFS Client Op: $opts{type} .+ Rename Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
		
		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}
	};
	
	/(symlink)/ && do {
		    if(($opts{is_sao} =~ /yes/i)) {

	              	if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {

				if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}/ ) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
	
		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}
	};	
		


#TBD:sandeepp: split setattr and make a separate case for CIFS setattr feature.

	/(^lookup|^getattr|^setattr)/ && do {
		if(($opts{is_sao} =~ /yes/i)) {

	              	if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {
	
				if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}/ ) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
		
		if(defined($opts{snapid}))
		{
			if($events =~ /SnapID: $opts{snapid}/)
			{
				$ret = 0;
				logcomment("Matched snapid with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
			else
			{
				$ret = 1;
				logcomment("Snapid NOT matching with $opts{type}");
				$i = 2; $nfsRes = 1;
			}
		}
	};
	
	/(^cifs_attr)/ && do {
		if($opts{is_npc} =~ /sacl/i) {
		
			if($events =~ /protocol CIFS Client Op: setattr.+mode.+4/i) {
	              		$ret = 0;
	              		logcomment("Matched CIFS setattr Op with $opts{is_npc}");
	              		$i = 5; $cifsRes = 0;
	              	 }
	         } elsif ($opts{is_npc} =~ /dacl/i) {
	         	logcomment("@@@ Inside the dacl and traffic tyep is @@@ $opts{traffic_type}");
	         	if($events =~ /protocol CIFS Client Op: setattr.+mode.+8/i) {
				$ret = 0;
				logcomment("Matched CIFS setattr Op with $opts{is_npc}");
				$i = 5; $cifsRes = 0;
	              	 }
	              	 
	          } elsif ($opts{is_npc} =~ /owner_chane/i) {
	          	if($events =~ /protocol CIFS Client Op: setattr.+mode.+1/i) {
				$ret = 0;
				logcomment("Matched CIFS setattr Op with $opts{is_npc}");
				$i = 5; $cifsRes = 0;
			}
		} elsif ($opts{is_npc} =~ /group_change/i) {
	          	if($events =~ /protocol CIFS Client Op: setattr.+mode.+2/i) {
				$ret = 0;
				logcomment("Matched CIFS setattr Op with $opts{is_npc}");
				$i = 5; $cifsRes = 0;
			}
		}
	};
	
	
	} # SWITCH
	} # while

	if($i != 5) {
		logcomment("No Matching Event for $opts{type}");
	}
	
	# $ret = 0 => pass
	# $ret = 1 => fail
	my $result = 0;


	if($opts{traffic_type} =~ /nfs/i){
		if($opts{result} =~ /fail/i and $nfsRes) {
			# you expected test to fail and it failed
			#logresult("pass", "Notification $opts{type} for NFS did NOT match. Expected to fail.");
			logcomment("PASS: Notification $opts{type} for NFS did NOT match. Expected to fail.");
			$result |= 0;
		} elsif($opts{result} =~ /fail/i and !$nfsRes) {
			# you expected test to fail but it passed
			#logresult("fail", "Expected NFS $opts{type} to fail BUT notification received");
			logcomment("FAIL: Expected NFS $opts{type} to fail BUT notification received");
			$result |= 1;
		} elsif($opts{result} =~ /pass/i and !$nfsRes) {
			# you expected test to pass and it passed
			#logresult("pass", "Notification matched for NFS $opts{type}");
			logcomment("PASS: Notification matched for NFS $opts{type}");
			$result |= 0;
		} elsif($opts{result} =~ /pass/i and $nfsRes) {
			# you expected test to pass and it failed
			#logcomment("@@@@ Failed Prashant");
			#logresult("fail", "Expected to pass but NFS $opts{type} notification was not received.");
			logcomment("FAIL: Expected to pass but NFS $opts{type} notification was not received.");
			$result |= 1;
		}	
	}#if nfs

	if($opts{traffic_type} =~ /cifs/i){
		if($opts{result} =~ /fail/i and $cifsRes) {
			# you expected test to fail and it failed
			#logresult("pass", "Notification $opts{type} for CIFS did NOT match. Expected to fail.");
			logcomment("PASS: Notification $opts{type} for CIFS did NOT match. Expected to fail.");
			$result |= 0;
		} elsif($opts{result} =~ /fail/i and !$cifsRes) {
			# you expected test to fail but it passed
			#logresult("fail", "Expected CIFS $opts{type} to fail BUT notification received");
			logcomment("FAIL: Expected CIFS $opts{type} to fail BUT notification received");
			$result |= 1;
		} elsif($opts{result} =~ /pass/i and !$cifsRes) {
			# you expected test to pass and it passed
			#logresult("pass", "Notification matched for CIFS $opts{type}");
			logcomment("PASS: Notification matched for CIFS $opts{type}");
			$result |= 0;
		} elsif($opts{result} =~ /pass/i and $cifsRes) {
			# you expected test to pass and it failed
			#logresult("fail", "Expected to pass but CIFS $opts{type} notification was not received.");
			logcomment("FAIL: Expected to pass but CIFS $opts{type} notification was not received.");
			$result |= 1;
		}	
	}#if cifs		

	return (status => $result);
}

sub VerifyAllOps
{
	my $self = shift;
	my %args = validate(@_, {
		polObj => {type => OBJECT},
		events => {type => SCALAR, default => $self->{events}},
		filename => {type => SCALAR, default => "this_file.txt"},
		newfilename => {type => SCALAR, default => "this_file_1.txt"},
		fromdir => {type => SCALAR, default => "./"},
		todir => {type => SCALAR, default => "./"},
		directory => {type => SCALAR, default => "thisDir"},
		newdirectory => {type => SCALAR, default => "thisDir1"},
#		type    => { type => SCALAR, default => NFDIR, optional => 1 },
#		attr => {type => SCALAR, default => 0777, optional => 1},
#		## Added By Prashant(prash@netapp.com) for negative testing
		result => {type => SCALAR, default => "pass", optional => 1},
		traffic_type => {type => SCALAR, default => $self->{traffic_type}, optional => 1},
		## Added prashat to support i2p.
		i2p_path => {type => SCALAR, default => "", optional => 1},
		is_sao => {type => SCALAR, default => "", optional => 1},
		is_npc => {type => SCALAR, default => "", optional => 1},
		is_sparse => {type => SCALAR, default => "", optional => 1},
		nfsver => {type => SCALAR, default => "", optional => 1}
	} );	
	 
       
       my $result = 0;
       
       ## used for new pathname for i2p..
       my $newfilename_1 = "";
       my $newdirectory_1 = "";
       my %res = "";
       
	### Get the SID of root
	my $CreaterOwnerSID = 'S-1-3-0';
	
#-----Create-----	
     
	# Added by prashant(prash) to support i2p.
	if($args{i2p_path} ne "") {
		$newfilename_1 = "$args{i2p_path}\/$args{filename}";
		
		%res = $args{polObj}->{pserver}->Verify_mult( type => "create", filename => $newfilename_1, traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	} else {
		
		%res = $args{polObj}->{pserver}->Verify_mult( type => "create", filename => "", traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}

	
	#$args{polObj}->{pserver}->Verify_mult( type => "create", filename => $args{filename}, traffic_type => $args{traffic_type}, result => $args{result});

#----Open-------
#Open being tested only for CIFS. To be changed to support NFSv4 Open and Close Ops.
	if($args{traffic_type} =~ /cifs/i) { 
		%res = $args{polObj}->{pserver}->Verify_mult( type => "open", filename => $args{filename}, traffic_type => "cifs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});	
		$result |= $res{status};
	}
	
### For Nfsv4 open ops is supported.

	if($args{nfsver} ne "") {
		
		logcomment ("#### Inside the Nfs version 4");
		logcomment("####### version information is $args{nfsver}");
		## Call the nfsv4 open ops.
		#$self->Open(filename => $args{filename}, traffic_type => "nfs", nfsver => $ver);
		
		## Decide on whether i2p has to be verified or not.
		if($args{i2p_path} ne "") {
			$newfilename_1 = "$args{i2p_path}\/$args{filename}";
			%res = $args{polObj}->{pserver}->Verify_mult( type => "open", filename => $newfilename_1, traffic_type => "nfs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
			$result |= $res{status};
		} else {
			%res = $args{polObj}->{pserver}->Verify_mult( type => "open", filename => "", traffic_type => "nfs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
			$result |= $res{status};
		}
	}
	
	
	
#-----Write------

	# Added by prashant(prash) to support i2p.
	if($args{i2p_path} ne "") {
		$newfilename_1 = "$args{i2p_path}\/$args{filename}";
			
		%res = $args{polObj}->{pserver}->Verify_mult( type => "write", filename => $newfilename_1, traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	} else {
		#$args{filename} = "";
		%res = $args{polObj}->{pserver}->Verify_mult( type => "write", filename => "", traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}

	#$args{polObj}->{pserver}->Verify_mult( type => "write", filename => $args{filename}, traffic_type => $args{traffic_type}, result => $args{result});
	
	
	
#-----Read---------
#	$self->Read( filename => $args{filename} );
#	$args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => $args{filename}, result => $args{result});

	## Added by prashant(prash) to support i2p
	if($args{i2p_path} ne "") {
		$newfilename_1 = "$args{i2p_path}\/$args{filename}";
				
		%res = $args{polObj}->{pserver}->Verify_mult( type => "read", filename => $newfilename_1, traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	} else {
		#$args{filename} = "";
		%res = $args{polObj}->{pserver}->Verify_mult( type => "read", filename => "", traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}

	#$args{polObj}->{pserver}->Verify_mult( type => "read", filename => $args{filename}, traffic_type => $args{traffic_type}, result => $args{result} );


	
#-----Close-------
#Close being tested only for CIFS. To be changed to support NFSv4 Open and Close Ops.
	if($args{traffic_type} =~ /cifs/i) { 	
		%res = $args{polObj}->{pserver}->Verify_mult( type => "close", filename => $args{filename} , traffic_type => "cifs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}	
	
#-----getattr and setattr-----
		if($args{traffic_type} =~ /nfs/i) { 
			if($args{i2p_path} ne "") {
				$newfilename_1 = "$args{i2p_path}\/$args{filename}";
				$args{polObj}->{pserver}->Verify_mult( type => "getattr", filename => $newfilename_1, events => $args{events} , result => $args{result}, is_sao => $args{is_sao} );			
				$result |= $res{status};
				
			} else {
				#$args{filename} = "";
				$args{polObj}->{pserver}->Verify_mult( type => "getattr", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
				$result |= $res{status};
			}
	
			## Added by prahant(prash) to support i2p
			
			if($args{i2p_path} ne "") {
				$newfilename_1 = "$args{i2p_path}\/$args{filename}";
				%res = $args{polObj}->{pserver}->Verify_mult( type => "setattr", filename => $newfilename_1, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
				$result |= $res{status};
			} else {
				#$args{filename} = "";
				%res = $args{polObj}->{pserver}->Verify_mult( type => "setattr", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
				$result |= $res{status};
			}
			
			#$args{polObj}->{pserver}->Verify_mult( type => "setattr", filename => $args{filename}, result => $args{result} );
	}	

	#-----Lookup---------
			## Added by prashant(prash) to support i2p
			if($args{traffic_type} =~ /nfs/i) { 
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{filename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => $newfilename_1, result => $args{result}, events => $args{events} ,is_sao => $args{is_sao});
					$result |= $res{status};
				} else {
				
					#$args{filename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => "", result => $args{result}, events => $args{events} ,is_sao => $args{is_sao});
					$result |= $res{status};
				}
				
				
				#$args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => $args{filename}, result => $args{result} );
			}
		#-----Symlink--------
			if($args{traffic_type} =~ /nfs/i) { 
				### Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "symlink", filename => $newfilename_1, result => $args{result},events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "symlink", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "symlink", filename => $args{newfilename}, result => $args{result} );
			}
		
		#----Delete Symlink---
			if($args{traffic_type} =~ /nfs/i) { 
				#res = $self->Delete(filename => $args{newfilename}, traffic_type => "nfs" );
				#result |= $res{status};
				#setting the expected value to fail. But the once the RFE is fixed the symlink delete nitification should also be reported
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => "nfs", result => $args{result}  );	
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => "nfs", result => "fail"  );	
			}
		#----Link------------	
			if($args{traffic_type} =~ /nfs/i) { 
				### Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "link", filename => $newfilename_1, events => $args{events} , result => $args{result}, is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "link", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "link", filename => $args{newfilename}, result => $args{result} );
			}
		#----Delete link---
			if($args{traffic_type} =~ /nfs/i) { 
				
				## Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $newfilename_1, traffic_type => "nfs", events => $args{events} ,result => $args{result}, is_sao => $args{is_sao}  );	
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => "", traffic_type => "nfs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao}  );	
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => "nfs", result => $args{result}  );	
			}
		
		#-----Rename-------
						
				## Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {		
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "rename", filename => $newfilename_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} ,is_sao => $args{is_sao});
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "rename", filename => "", traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao});
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "rename", filename => $args{newfilename}, traffic_type => $args{traffic_type}, result => $args{result});
				
			#-----Delete------
				
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $newfilename_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao});	
					$result |= $res{status};
				} else {
					#$args{newfilename}= "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => "", traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao});	
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => $args{traffic_type}, result => $args{result});	
				
				%res = $args{polObj}->{cli}->show();
				
			#-----Directory Ops----	
				
				## Added by Prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newdirectory_1 = "$args{i2p_path}\/$args{directory}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dircreate", filename => $newdirectory_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dircreate", filename => $args{directory}, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "dircreate", filename => $args{directory}, traffic_type => $args{traffic_type}, result => $args{result} );
			
				
							
				## Added by Prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newdirectory_1 = "$args{i2p_path}\/$args{newdirectory}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirrename", filename => $newdirectory_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirrename", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "dirrename", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result} );
			
				
				## Added by Prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newdirectory_1 = "$args{i2p_path}\/$args{newdirectory}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirdelete", filename => $newdirectory_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirdelete", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );	
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "dirdelete", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result} );	
				return ( status => $result );
}

sub VerifyAllFileOps
{
	my $self = shift;
	my %args = validate(@_, {
		polObj => {type => OBJECT},
		events => {type => SCALAR, default => $self->{events}},
		filename => {type => SCALAR, default => "this_file.txt"},
		newfilename => {type => SCALAR, default => "this_file_1.txt"},
		fromdir => {type => SCALAR, default => "./"},
		todir => {type => SCALAR, default => "./"},
		directory => {type => SCALAR, default => "thisDir"},
		newdirectory => {type => SCALAR, default => "thisDir1"},
#		type    => { type => SCALAR, default => NFDIR, optional => 1 },
#		attr => {type => SCALAR, default => 0777, optional => 1},
#		## Added By Prashant(prash@netapp.com) for negative testing
		result => {type => SCALAR, default => "pass", optional => 1},
		traffic_type => {type => SCALAR, default => $self->{traffic_type}, optional => 1},
		## Added prashat to support i2p.
		i2p_path => {type => SCALAR, default => "", optional => 1},
		is_sao => {type => SCALAR, default => "", optional => 1},
		is_npc => {type => SCALAR, default => "", optional => 1},
		is_sparse => {type => SCALAR, default => "", optional => 1},
		nfsver => {type => SCALAR, default => "", optional => 1}
	} );	
	 
       
       my $result = 0;
       
       ## used for new pathname for i2p..
       my $newfilename_1 = "";
       my $newdirectory_1 = "";
       my %res = "";
              
	### Get the SID of root
	my $CreaterOwnerSID = 'S-1-3-0';
	
#-----Create-----	
     
	# Added by prashant(prash) to support i2p.
	if($args{i2p_path} ne "") {
		$newfilename_1 = "$args{i2p_path}\/$args{filename}";
		
		%res = $args{polObj}->{pserver}->Verify_mult( type => "create", filename => $newfilename_1, traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	} else {
		
		%res = $args{polObj}->{pserver}->Verify_mult( type => "create", filename => "", traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}

	
	#$args{polObj}->{pserver}->Verify_mult( type => "create", filename => $args{filename}, traffic_type => $args{traffic_type}, result => $args{result});

#----Open-------
#Open being tested only for CIFS. To be changed to support NFSv4 Open and Close Ops.
	if($args{traffic_type} =~ /cifs/i) { 
		%res = $args{polObj}->{pserver}->Verify_mult( type => "open", filename => $args{filename}, traffic_type => "cifs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});	
		$result |= $res{status};
	}
	
### For Nfsv4 open ops is supported.

	if($args{nfsver} ne "") {
		
		logcomment ("#### Inside the Nfs version 4");
		logcomment("####### version information is $args{nfsver}");
		## Call the nfsv4 open ops.
		#$self->Open(filename => $args{filename}, traffic_type => "nfs", nfsver => $ver);
		
		## Decide on whether i2p has to be verified or not.
		if($args{i2p_path} ne "") {
			$newfilename_1 = "$args{i2p_path}\/$args{filename}";
			%res = $args{polObj}->{pserver}->Verify_mult( type => "open", filename => $newfilename_1, traffic_type => "nfs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
			$result |= $res{status};
		} else {
			%res = $args{polObj}->{pserver}->Verify_mult( type => "open", filename => "", traffic_type => "nfs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
			$result |= $res{status};
		}
	}
	
	
	
#-----Write------

	# Added by prashant(prash) to support i2p.
	if($args{i2p_path} ne "") {
		$newfilename_1 = "$args{i2p_path}\/$args{filename}";
			
		%res = $args{polObj}->{pserver}->Verify_mult( type => "write", filename => $newfilename_1, traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	} else {
		#$args{filename} = "";
		%res = $args{polObj}->{pserver}->Verify_mult( type => "write", filename => "", traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}

	#$args{polObj}->{pserver}->Verify_mult( type => "write", filename => $args{filename}, traffic_type => $args{traffic_type}, result => $args{result});
	
	
	
#-----Read---------
#	$self->Read( filename => $args{filename} );
#	$args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => $args{filename}, result => $args{result});

	## Added by prashant(prash) to support i2p
	if($args{i2p_path} ne "") {
		$newfilename_1 = "$args{i2p_path}\/$args{filename}";
				
		%res = $args{polObj}->{pserver}->Verify_mult( type => "read", filename => $newfilename_1, traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	} else {
		#$args{filename} = "";
		%res = $args{polObj}->{pserver}->Verify_mult( type => "read", filename => "", traffic_type => $args{traffic_type}, events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}

	#$args{polObj}->{pserver}->Verify_mult( type => "read", filename => $args{filename}, traffic_type => $args{traffic_type}, result => $args{result} );


	
#-----Close-------
#Close being tested only for CIFS. To be changed to support NFSv4 Open and Close Ops.
	if($args{traffic_type} =~ /cifs/i) { 	
		%res = $args{polObj}->{pserver}->Verify_mult( type => "close", filename => $args{filename} , traffic_type => "cifs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao});
		$result |= $res{status};
	}	
	
#-----getattr and setattr-----
		if($args{traffic_type} =~ /nfs/i) { 
			if($args{i2p_path} ne "") {
				$newfilename_1 = "$args{i2p_path}\/$args{filename}";
				$args{polObj}->{pserver}->Verify_mult( type => "getattr", filename => $newfilename_1, events => $args{events} , result => $args{result}, is_sao => $args{is_sao} );			
				$result |= $res{status};
				
			} else {
				#$args{filename} = "";
				$args{polObj}->{pserver}->Verify_mult( type => "getattr", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
				$result |= $res{status};
			}
	
			## Added by prahant(prash) to support i2p
			
			if($args{i2p_path} ne "") {
				$newfilename_1 = "$args{i2p_path}\/$args{filename}";
				%res = $args{polObj}->{pserver}->Verify_mult( type => "setattr", filename => $newfilename_1, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
				$result |= $res{status};
			} else {
				#$args{filename} = "";
				%res = $args{polObj}->{pserver}->Verify_mult( type => "setattr", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
				$result |= $res{status};
			}
			
			#$args{polObj}->{pserver}->Verify_mult( type => "setattr", filename => $args{filename}, result => $args{result} );
	}	

	#-----Lookup---------
			## Added by prashant(prash) to support i2p
			if($args{traffic_type} =~ /nfs/i) { 
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{filename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => $newfilename_1, result => $args{result}, events => $args{events} ,is_sao => $args{is_sao});
					$result |= $res{status};
				} else {
				
					#$args{filename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => "", result => $args{result}, events => $args{events} ,is_sao => $args{is_sao});
					$result |= $res{status};
				}
				
				
				#$args{polObj}->{pserver}->Verify_mult( type => "lookup", filename => $args{filename}, result => $args{result} );
			}
		#-----Symlink--------
			if($args{traffic_type} =~ /nfs/i) { 
				### Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "symlink", filename => $newfilename_1, result => $args{result},events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "symlink", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "symlink", filename => $args{newfilename}, result => $args{result} );
			}
		
		#----Delete Symlink---
			if($args{traffic_type} =~ /nfs/i) { 
				#res = $self->Delete(filename => $args{newfilename}, traffic_type => "nfs" );
				#result |= $res{status};
				#setting the expected value to fail. But the once the RFE is fixed the symlink delete nitification should also be reported
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => "nfs", result => $args{result}  );	
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => "nfs", result => "fail"  );	
			}
		#----Link------------	
			if($args{traffic_type} =~ /nfs/i) { 
				### Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "link", filename => $newfilename_1, events => $args{events} , result => $args{result}, is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "link", filename => "", result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "link", filename => $args{newfilename}, result => $args{result} );
			}
		#----Delete link---
			if($args{traffic_type} =~ /nfs/i) { 
				
				## Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $newfilename_1, traffic_type => "nfs", events => $args{events} ,result => $args{result}, is_sao => $args{is_sao}  );	
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => "", traffic_type => "nfs", events => $args{events} , result => $args{result}, is_sao => $args{is_sao}  );	
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => "nfs", result => $args{result}  );	
			}
		
		#-----Rename-------
						
				## Added by prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {		
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "rename", filename => $newfilename_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} ,is_sao => $args{is_sao});
					$result |= $res{status};
				} else {
					#$args{newfilename} = "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "rename", filename => "", traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao});
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "rename", filename => $args{newfilename}, traffic_type => $args{traffic_type}, result => $args{result});
				
			#-----Delete------
				
				if($args{i2p_path} ne "") {
					$newfilename_1 = "$args{i2p_path}\/$args{newfilename}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $newfilename_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao});	
					$result |= $res{status};
				} else {
					#$args{newfilename}= "";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "delete", filename => "", traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao});	
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "delete", filename => $args{newfilename}, traffic_type => $args{traffic_type}, result => $args{result});	
				
				%res = $args{polObj}->{cli}->show();
				
				#$args{polObj}->{pserver}->Verify_mult( type => "dirdelete", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result} );	
				return ( status => $result );
}

sub VerifyAllDirOps
{
	my $self = shift;
	my %args = validate(@_, {
		polObj => {type => OBJECT},
		events => {type => SCALAR, default => $self->{events}},
		fromdir => {type => SCALAR, default => "./"},
		todir => {type => SCALAR, default => "./"},
		directory => {type => SCALAR, default => "thisDir"},
		newdirectory => {type => SCALAR, default => "thisDir1"},
#		type    => { type => SCALAR, default => NFDIR, optional => 1 },
#		attr => {type => SCALAR, default => 0777, optional => 1},
#		## Added By Prashant(prash@netapp.com) for negative testing
		result => {type => SCALAR, default => "pass", optional => 1},
		traffic_type => {type => SCALAR, default => $self->{traffic_type}, optional => 1},
		## Added prashat to support i2p.
		i2p_path => {type => SCALAR, default => "", optional => 1},
		is_sao => {type => SCALAR, default => "", optional => 1},
		is_npc => {type => SCALAR, default => "", optional => 1},
		is_sparse => {type => SCALAR, default => "", optional => 1},
		nfsver => {type => SCALAR, default => "", optional => 1}
	} );	
	 
       
       my $result = 0;
       
       ## used for new pathname for i2p..
       my $newfilename_1 = "";
       my $newdirectory_1 = "";
       my %res = "";
	   
		### Get the SID of root
		my $CreaterOwnerSID = 'S-1-3-0';
	
		#-----Directory Ops----	
				
				## Added by Prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newdirectory_1 = "$args{i2p_path}\/$args{directory}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dircreate", filename => $newdirectory_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dircreate", filename => $args{directory}, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "dircreate", filename => $args{directory}, traffic_type => $args{traffic_type}, result => $args{result} );
			
				
							
				## Added by Prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newdirectory_1 = "$args{i2p_path}\/$args{newdirectory}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirrename", filename => $newdirectory_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirrename", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "dirrename", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result} );
			
				
				## Added by Prashant(prash) to support i2p.
				if($args{i2p_path} ne "") {
					$newdirectory_1 = "$args{i2p_path}\/$args{newdirectory}";
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirdelete", filename => $newdirectory_1, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );
					$result |= $res{status};
				} else {
					%res = $args{polObj}->{pserver}->Verify_mult( type => "dirdelete", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result}, events => $args{events} , is_sao => $args{is_sao} );	
					$result |= $res{status};
				}
				
				#$args{polObj}->{pserver}->Verify_mult( type => "dirdelete", filename => $args{newdirectory}, traffic_type => $args{traffic_type}, result => $args{result} );	
				return ( status => $result );
}

sub Verify_mult ($;@)
{
	my $self = shift;
#	my %opts = @_;

        my %opts = validate(@_, {
        		filename => {type => SCALAR, default => ""},
        		type => {type => SCALAR},
				traffic_type => {type => SCALAR, default => $self->{traffic_type}},
				events => {type => SCALAR, default => $self->{events}},
	            result => {type => SCALAR, default => "pass"},
	            is_sao => {type => SCALAR, default => "", optional => 1},
				is_npc => {type => SCALAR, default => "", optional => 1},
				is_rrd => {type => SCALAR, default => "", optional => 1}	
                });

#	my $filename = $opts{filename};
#	my $type = $opts{type};
#	my $traffic_type = $self->{traffic_type} if(!defined $opts{traffic_type});
#	$opts{result} = "pass" if(!defined $opts{result});
	
	### Get the SID values
	my $CreaterOwnerSID = 'S-1-3-0';
	my $DomainAdminSID = $self->{DomainAdminSID};
	my $AdminsSID = $self->{AdminsSID};
	
	logcomment("Traffic type is : $opts{traffic_type}");
	
	#logcomment("Events recieved at Verify_mult are : $opts{events}");
	my $ret = 1; # start with fail
	my $nfsRes = 1; # start with fail
	my $cifsRes = 1; # start with fail
	my $i=0;

		my $events = $opts{events};
		$events =~  s/\n/ /g;
		## logcomment("^^^^^^^^^@@@@@@@@ After processing = \n $events");
		
	if( $opts{type} =~ /(getattr|setattr|lookup|link|symlink)/){
		$opts{traffic_type} = "nfs";
	}

	# EVENT types: create, delete, open, close, read, write, lookup, getattr, setattr, rename, dircreate, dirdelete, dirrename

	# This can be converted into a template 
	# which is ideal to match all the fields of the event
	# waiting for confirmation of the required fields w.r.t testing
	SWITCH: for (lc($opts{type})) {
	/(create|delete|open|close)/ && do {
		#saperate blocks to check the NFS and CIFS traffic for future use. 
		#any NFS and CIFS specific notifications can be handled. Ex: NFS snapID feature.

		
		if($opts{traffic_type} =~ /nfs/i) {
	               
	              if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /create/i)) {
	              	if($events =~ /protocol NFS Client Op: file $opts{type} Access Path:.+$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } elsif (($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /create/i))  {
	              		if($events =~ /protocol NFS Client Op: file $opts{type} Access Path:.+$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	            			$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      } else {
				if($events =~ /protocol NFS Client Op: file $opts{type} Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      	}
		      
		} #if nfs
		
		
		if($opts{traffic_type} =~ /cifs/i) {
               		if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /create/i)) {

               			if($events =~ /protocol CIFS Client Op: file $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $CreaterOwnerSID.+size: 0/) {
               				$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			} elsif ( ($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /create/i) )  {

				if($events =~ /protocol CIFS Client Op: file $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 11/) {
				        $ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			
               		} else {

               			if($events =~ /protocol CIFS Client Op: file $opts{type}/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
		}#if cifs
	};

	/(read|write)/ && do {
		#saperate blocks to check the NFS and CIFS traffic for future use. 
		#any NFS and CIFS specific notifications can be handled. Ex: NFS snapID feature.		
		if($opts{traffic_type} =~ /nfs/i){
		    if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /write/i)) {

	              	if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } elsif (($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /write/i))  {

	              		if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	            			$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
			} else {

				if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      	}	
	
	# Added for RRD
			if(($opts{is_rrd} =~ /yes/i) && !($opts{type} =~ /write/i))
			{
				if($events =~ /FP_FileData/)
				{
					$ret = 0;
					logcomment("Client served data using FP_FileData with Version2 enabled");
					$i = 5; $nfsRes = 0;
				}
				else
				{	
					$ret = 1;
					logcomment("Data was not serverd using FP_FileData");
					$i = 2; $nfsRes = 1;
				}
			}
		}#if nfs
		
		
		if($opts{traffic_type} =~ /cifs/i){
               		if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /write/i)) {
					      
               			if($events =~ /protocol CIFS Client Op: $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 0/) {
               				$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			} elsif ( ($opts{is_sao} =~ /yes/i) && !($opts{type} =~ /write/i) )  {

				if($events =~ /protocol CIFS Client Op: $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 11/) {
				        $ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			
               		} else {

               			if($events =~ /protocol CIFS Client Op: $opts{type}/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}

	# Added for RRD
			if(($opts{is_rrd} =~ /yes/i) && !($opts{type} =~ /write/i))
			{
				if($events =~ /FP_FileData/)
				{
					$ret = 0;
					logcomment("Client served data using FP_FileData with Version2 enabled");
					$i = 5; $cifsRes = 0;
				}
				else
				{
					$ret = 1;
					logcomment("Data was not served using FP_FileData");
					$i = 2; $cifsRes = 1;
				}

			}
		}#if cifs
	};
	
	/(^dircreate|^dirdelete)/ && do {

		my $ttype = $opts{type};
		$ttype =~ s/dir//;
		if($opts{traffic_type} =~ /nfs/i){

		    if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dircreate/i)) {

	              	if($events =~ /protocol NFS Client Op: directory $ttype Access Path: $opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	            } elsif (($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dirdelete/i))  {

	              	if($events =~ /protocol NFS Client Op: directory $ttype Access Path: $opts{filename} .+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 4096/) {
	            		$ret = 0;
				logcomment("Matched NFS Op with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
		   } else {
			
			if($events =~ /protocol NFS Client Op: directory $ttype Access Path:.*$opts{filename}/) {
				$ret = 0;
				logcomment("Matched NFS Op with $opts{type}");
				$i = 5; $nfsRes = 0;
		    	}
		   }
		}#if nfs
		
		if($opts{traffic_type} =~ /cifs/i){

			if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dircreate/i)) {

				if($events =~ /protocol CIFS Client Op: directory $ttype.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $CreaterOwnerSID .+size: 0/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
				
			} elsif (($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dirdelete/i))  {

				if($events =~ /protocol CIFS Client Op: directory $ttype.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 4096/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;	
				}
			} else {
				if($events =~ /protocol CIFS Client Op: directory $ttype/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
			
		}#if cifs		
	};

	/^dirrename/ && do {
		my $ttype = $opts{type};
		$ttype =~ s/dir//;
		if($opts{traffic_type} =~ /nfs/i) {
		    if(($opts{is_sao} =~ /yes/i) && ($opts{type} =~ /^dirrename/i)) {

	              	if($events =~ /protocol NFS Client Op: directory $ttype.+Rename Access Path: $opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 4096/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	}
	              	
	            } else {
	             				              			

			if($events =~ /protocol NFS Client Op: directory $ttype.+Rename Access Path:.*$opts{filename}/) {
				$ret = 0;
				logcomment("Matched NFS Op with $opts{type}");
				$i = 5; $nfsRes = 0;
			}
		   }
		      	
		}#if nfs
	
		if($opts{traffic_type} =~ /cifs/i){
			if(($opts{is_sao} =~ /yes/i)) {

				if($events =~ /protocol CIFS Client Op: directory $ttype.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 4096/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
				
			} else {

				if($events =~ /protocol CIFS Client Op: directory $ttype/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
			
		}#if CIFS		
	};

	/rename/ && do {
		if($opts{traffic_type} =~ /nfs/i){	
		    if(($opts{is_sao} =~ /yes/i)) {

	              	if($events =~ /protocol NFS Client Op: file $opts{type} .+ Rename Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {

				if($events =~ /protocol NFS Client Op: file $opts{type} .+ Rename Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
		}#if nfs
		
		if($opts{traffic_type} =~ /cifs/i){
			if(($opts{is_sao} =~ /yes/i)) {
			
				if($events =~ /protocol CIFS Client Op: file $opts{type}.+Usr context :.+SID: $DomainAdminSID Owner :.+SID: $AdminsSID.+size: 11/) {
					$ret=0;
					logcomment("Inside the cifs rename");
					$i = 5; $cifsRes = 0;
				}
				
			} else {

				if($events =~ /protocol CIFS Client Op: file $opts{type}/) {
					$ret = 0;
					logcomment("Matched CIFS Op with $opts{type}");
					$i = 5; $cifsRes = 0;
				}
			}
		}#if cifs
	};

	/(link)/ && do {
		if(($opts{is_sao} =~ /yes/i)) {

	        	if($events =~ /protocol NFS Client Op: $opts{type} .+ Rename Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {

				if($events =~ /protocol NFS Client Op: $opts{type} .+ Rename Access Path:.*$opts{filename}/) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
	};
	
	/(symlink)/ && do {
		    if(($opts{is_sao} =~ /yes/i)) {

	              	if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 0/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {
	
				if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}/ ) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
	};	
		


#TBD:sandeepp: split setattr and make a separate case for CIFS setattr feature.

	/(^lookup|^getattr|^setattr)/ && do {
		if(($opts{is_sao} =~ /yes/i)) {

	              	if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}.+Usr context : UID:0, SID: NULL Owner : UID:0, SID: $CreaterOwnerSID .+size: 200/) {
	              		$ret = 0;
	              		logcomment("Matched NFS Op with $opts{type}");
	              		$i = 5; $nfsRes = 0;
	              	 }
	              	 
	              } else {
	
				if($events =~ /protocol NFS Client Op: $opts{type} Access Path:.*$opts{filename}/ ) {
					$ret = 0;
					logcomment("Matched NFS Op with $opts{type}");
					$i = 5; $nfsRes = 0;
				}
		      }
	};
	
	/(^cifs_attr)/ && do {
		if($opts{is_npc} =~ /sacl/i) {
		
			if($events =~ /protocol CIFS Client Op: setattr.+mode.+4/i) {
	              		$ret = 0;
	              		logcomment("Matched CIFS setattr Op with $opts{is_npc}");
	              		$i = 5; $cifsRes = 0;
	              	 }
	         } elsif ($opts{is_npc} =~ /dacl/i) {
	         	if($events =~ /protocol CIFS Client Op: setattr.+mode.+8/i) {
				$ret = 0;
				logcomment("Matched CIFS setattr Op with $opts{is_npc}");
				$i = 5; $cifsRes = 0;
	              	 }
	              	 
	          } elsif ($opts{is_npc} =~ /owner_chane/i) {
	          	if($events =~ /protocol CIFS Client Op: setattr.+mode.+1/i) {
				$ret = 0;
				logcomment("Matched CIFS setattr Op with $opts{is_npc}");
				$i = 5; $cifsRes = 0;
			}
		} elsif ($opts{is_npc} =~ /group_change/i) {
	          	if($events =~ /protocol CIFS Client Op: setattr.+mode.+2/i) {
				$ret = 0;
				logcomment("Matched CIFS setattr Op with $opts{is_npc}");
				$i = 5; $cifsRes = 0;
			}
		}
	};
	
	
	} # SWITCH
	

	if($i != 5) {
		logcomment("No Matching Event for $opts{type}");
	}
	
	# $ret = 0 => pass
	# $ret = 1 => fail
	my $result = 0;


	if($opts{traffic_type} =~ /nfs/i){
		if($opts{result} =~ /fail/i and $nfsRes) {
			# you expected test to fail and it failed
			#logresult("pass", "Notification $opts{type} for NFS did NOT match. Expected to fail.");
			logcomment("PASS: Notification $opts{type} for NFS did NOT match. Expected to fail.");
			$result |= 0;
		} elsif($opts{result} =~ /fail/i and !$nfsRes) {
			# you expected test to fail but it passed
			#logresult("fail", "Expected NFS $opts{type} to fail BUT notification received");
			logcomment("FAIL: Expected NFS $opts{type} to fail BUT notification received");
			$result |= 1;
		} elsif($opts{result} =~ /pass/i and !$nfsRes) {
			# you expected test to pass and it passed
			#logresult("pass", "Notification matched for NFS $opts{type}");
			logcomment("PASS: Notification matched for NFS $opts{type}");
			$result |= 0;
		} elsif($opts{result} =~ /pass/i and $nfsRes) {
			# you expected test to pass and it failed
			#logresult("fail", "Expected to pass but NFS $opts{type} notification was not received.");
			logcomment("FAIL: Expected to pass but NFS $opts{type} notification was not received.");
			$result |= 1;
		}	
	}#if nfs

	if($opts{traffic_type} =~ /cifs/i){
		if($opts{result} =~ /fail/i and $cifsRes) {
			# you expected test to fail and it failed
			#logresult("pass", "Notification $opts{type} for CIFS did NOT match. Expected to fail.");
			logcomment("PASS: Notification $opts{type} for CIFS did NOT match. Expected to fail.");
			$result |= 0;
		} elsif($opts{result} =~ /fail/i and !$cifsRes) {
			# you expected test to fail but it passed
			#logresult("fail", "Expected CIFS $opts{type} to fail BUT notification received");
			logcomment("FAIL: Expected CIFS $opts{type} to fail BUT notification received");
			$result |= 1;
		} elsif($opts{result} =~ /pass/i and !$cifsRes) {
			# you expected test to pass and it passed
			#logresult("pass", "Notification matched for CIFS $opts{type}");
			logcomment("PASS: Notification matched for CIFS $opts{type}");
			$result |= 0;
		} elsif($opts{result} =~ /pass/i and $cifsRes) {
			# you expected test to pass and it failed
			#logresult("fail", "Expected to pass but CIFS $opts{type} notification was not received.");
			logcomment("FAIL: Expected to pass but CIFS $opts{type} notification was not received.");
			$result |= 1;
		}	
	}#if cifs		
	
	return (status => $result);
}


# this module is obselete
# was initially an alternative to the methods
# to be implemented for event capture
# leaving it here for future use
sub CollectEventsFromServer_usingFTP
{
	use Net::FTP;

	logcomment("Tring to connect to FTP");
	my $ftp = Net::FTP->new("172.17.180.40", Debug => 1) 
		or logcomment("Cannot connect to 172.17.180.40 : $@");

	$ftp->login("Administrator", "BigCheese");
	$ftp->pwd();
	$ftp->get("logfile.txt");
	$ftp->quit;
	logcomment("Done with connect to FTP");
}

# the definitions here should be commented for now.
# cleanup is explicitly called from seperate .thpl
sub DESTROY
{
	my $self = shift;
	# close the FPolicy server
#	say($self->{CONN}, "s", "Total");

	# cleanup
#	logcomment("PServer - Cleanup");
#
#	eval {
#	# close the session
#	$self->{server}->close() if(defined $self->{server} and $self->{server} =~ /Telnet/);
#
#	logcomment("Check presence of $POLICY_SERVER_TRANSFER_EXE");
#	my $msgc = say($self->{CONN}, "cmd /c tasklist /FI \"IMAGENAME eq $POLICY_SERVER_TRANSFER_EXE\"");
#
#	if($msgc =~ /No tasks are running/i) {
#		;
#	}
#	else
#	{
#		logcomment("Kill the process $POLICY_SERVER_TRANSFER_EXE");
#		$msgc = say($self->{CONN}, "cmd /c taskkill \/F \/IM $POLICY_SERVER_TRANSFER_EXE");
#	}
#
#	logcomment("Check presence of $POLICY_SERVER_EVENT_EXE");
#	$msgc = say($self->{CONN}, "cmd /c tasklist \/FI \"IMAGENAME eq $POLICY_SERVER_EVENT_EXE\"");
#
#	if($msgc =~ /No tasks are running/i) {
#		;
#	}
#	else {
#		#logcomment("Kill the process $POLICY_SERVER_EVENT_EXE");
#		#$msgc = say($self->{CONN}, "cmd /c taskkill \/F \/IM $POLICY_SERVER_EVENT_EXE");
#	}
#
#	};
#
#	if($@) {
#		logcomment("Error during cleanup: $@");
#	}
#	# task is killed successfully.
#	# wait for a while for the RPC connections to break on both ends
#	sleep(5);
}

sub testme()
{
	package main;
	our ($FILER, $FILER_CONN, $CLIENT, $PSERVER, $FILER_IP, $NFSCLIENT, $NFSCLIENT_TYPE, $CIFSCLIENT, $POLICY, $PROTOCOL, $NFSVERSION, $VOLUME, $CMDTIMEOUT, $MOUNT, $TRAFFIC_TYPE);

	param('FILER',	'-mesg',	"Filer to test");
	param('FILER_IP','-mesg',	"Filer to test");
	param('FILER_CONN','-default',	"#rsh");
        param('POLICY',   '-default',	'pol1');
        param('CLIENT',   '-mesg',	'CLIENT');
        param('PSERVER',  '-mesg',	'PSERVER');
        param('NFSCLIENT','-mesg',	'CLIENT');
        param('CIFSCLIENT','-mesg',	'CLIENT');
        param('NFSCLIENT_TYPE',	'-mesg',	'CLIENT_TYPE');
        param('PROTOCOL',   '-mesg',	'PROTOCOL');
        param('NFSVERSION', '-mesg',	'NFSVERSION');
        param('MOUNT',   '-mesg',	'MOUNT PATH');
        param('VOLUME',  '-mesg',	'VOLUME');
        param('CMDTIMEOUT','-default',	'30');
        param('TRAFFIC_TYPE','-default',	"nfs");

	use FPolicy;

	my $pol = new FPolicy(
			"policy=$POLICY", 
			"vfiler=vfiler0", 
			"filer=$FILER", 
			"filer_ip=$FILER_IP",
			"monitor=read,write,open,close", 
			"vol_inc=vol0", "vol_exc=", 
			"ext_inc=txt", "ext_exc=ppt", 
			"pserver_pri=$PSERVER",
			"client=$CLIENT",
			"hostname=$FILER_IP",	
			"nfs_version=$NFSVERSION",
			"protocol=$PROTOCOL",
			"timeout=$CMDTIMEOUT",
			"client=$NFSCLIENT",
			"volume=$VOLUME",
			"traffic_type=$TRAFFIC_TYPE"
		);

	$pol->{cli}->connect();
	$pol->{cli}->create();
	$pol->{cli}->enable();
	$pol->{pserver}->launch();
	sleep(30);

	# launch would connect, can be used additionally if required

	$pol->{traffic}->setup();
	$pol->{pserver}->ConnectToTransferServer(); # if(!defined $pol->{server});
	$pol->{traffic}->Create( filename => "this_file.txt" );
	$pol->{pserver}->Verify( type => "create", filename => "this_file.txt" );
	$pol->{traffic}->Read( filename => "this_file.txt" );
	$pol->{pserver}->Verify( type => "read", filename => "this_file.txt" );
	$pol->{traffic}->Delete( filename => "this_file.txt" );
	$pol->{pserver}->Verify( type => "delete", filename => "this_file.txt" );

	#$pol->{cli}->destroy();

}

1;
