# -*- perl -*-
#  $Id: //depot/prod/test/main/lib/Setup.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Setup;

use strict;

use warnings;
use Utils qw(make_host make_conn);
use Tharn qw(say logcomment logresult);
use Params::Validate qw(validate_pos validate OBJECT BOOLEAN SCALAR ARRAYREF HASHREF);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;
use CIFS::Setup qw(is_cifs_setup cifs_setup);

use FPolicy::Utils;
use Admin::License;

BEGIN {
	our @EXPORT = qw (
		setup_cifs
	);
};

sub new
{
       my $object = shift;
       my $class = ref($object) || $object;
       my $self = {};
       bless($self, $class); 
       return $self;

}


sub setup_cifs (@)
{
	my @parms = @_;
	my $retVal = 0;

	my %opt = validate(@parms, {
			host => {type => SCALAR | OBJECT},
			vfiler => {type => SCALAR},
			dns_server_list => {type => SCALAR},
			'auth-type' => {type => SCALAR},
			domain_user => {type => SCALAR},
			domain_pass => {type => SCALAR},
			domain => {type => SCALAR},
			force_setup => {type => SCALAR, default=> 0, regex => qr/(\d)/}
			});

	my $host = make_host($opt{host});
		logcomment("Performing cifs setup");
		$retVal = cifs_setup(
				host => $opt{host},
				vfiler => $opt{vfiler},
				dns_server_list=> $opt{dns_server_list},
				'auth-type' => $opt{'auth-type'},
				domain_user => $opt{domain_user},
				domain_pass => $opt{domain_pass},
				domain => $opt{domain}
			 );

	return $retVal;
}

sub Setup_Filer
{	
	my $VOLUME=main::param("VOLUME","-mesg", "VOLUME");
	my $vfiler=main::param("VFILER","-mesg", "VFILER");
	
    # These options need to be set on the filer
	my %args = @_;

	if ($vfiler ne "vfiler0")
	{
		my(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options nfs.mount_rootonly off");
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options nfs.mount_rootonly");
		if(!($Res =~ /nfs.mount_rootonly/ && $Res =~ /off/))
		{
			logresult("fatal","Unable to set options nfs.mount_rootonly to OFF. Please disable the option");
		}

		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler qtree security /vol/$VOLUME mixed");
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler qtree security /vol/$VOLUME");
		if(!($Res =~ /$VOLUME\/ has mixed security style/))
		{
			logresult("fatal","Unable to set volume security to mixed. Please set security of the volume to mixed");
		}
	
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options httpd.enable on");
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options httpd.enable");
		if(!($Res =~ /httpd.enable/ && $Res =~ /on/))
		{
			logresult("fatal", "Unable to set options httpd.enable to ON. Please enable the option");
		}
	
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options httpd.admin.enable on");
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options httpd.admin.enable");
		if(!($Res =~ /httpd.admin.enable/ && $Res =~ /on/))
		{
			logresult("fatal", "Unable to set options httpd.admin.enable to ON. Please enable the option");
		}
	
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options nfs.v4.enable on");
		(undef,undef,$Res) = say($args{conn}, "vfiler run $vfiler options nfs.v4.enable");
		if(!($Res =~ /nfs.v4.enable/ && $Res =~ /on/))
		{
			logresult("fatal","Unable to set options nfs.v4.enable to ON. Please enable the option");
		}
	}
	else
	{
		my(undef,undef,$Res) = say($args{conn}, "options nfs.mount_rootonly off");
		(undef,undef,$Res) = say($args{conn}, "options nfs.mount_rootonly");
		if(!($Res =~ /nfs.mount_rootonly/ && $Res =~ /off/))
		{
			logresult("fatal","Unable to set options nfs.mount_rootonly to OFF. Please disable the option");
		}

		(undef,undef,$Res) = say($args{conn}, "qtree security /vol/$VOLUME mixed");
		(undef,undef,$Res) = say($args{conn}, "qtree security /vol/$VOLUME");
		if(!($Res =~ /$VOLUME\/ has mixed security style/))
		{
			logresult("fatal","Unable to set volume security to mixed. Please set security of the volume to mixed");
		}
	
		(undef,undef,$Res) = say($args{conn}, "options httpd.enable on");
		(undef,undef,$Res) = say($args{conn}, "options httpd.enable");
		if(!($Res =~ /httpd.enable/ && $Res =~ /on/))
		{
			logresult("fatal", "Unable to set options httpd.enable to ON. Please enable the option");
		}
	
		(undef,undef,$Res) = say($args{conn}, "options httpd.admin.enable on");
		(undef,undef,$Res) = say($args{conn}, "options httpd.admin.enable");
		if(!($Res =~ /httpd.admin.enable/ && $Res =~ /on/))
		{
			logresult("fatal", "Unable to set options httpd.admin.enable to ON. Please enable the option");
		}
	
		(undef,undef,$Res) = say($args{conn}, "options nfs.v4.enable on");
		(undef,undef,$Res) = say($args{conn}, "options nfs.v4.enable");
		if(!($Res =~ /nfs.v4.enable/ && $Res =~ /on/))
		{
			logresult("fatal","Unable to set options nfs.v4.enable to ON. Please enable the option");
		}
	}
}

### this function will sync the filer time to AD time
sub Setup_Filer_Datetime
{}

sub Setup_Vfiler
{}

sub Setup_Cluster
{}

sub Setup_PolicyServer
{}

sub Disconnect_PolicyServer
{}

sub CleanUp_Vfiler
{}

sub CleanUp_Cluster
{}

sub CleanUp_Filer
{}

sub Setup_Licenses
{
    my %args = @_;
	Admin::License::license($args{conn}, "nfs");
	Admin::License::license($args{conn}, "cifs");
	Admin::License::license($args{conn}, "http");
	Admin::License::license($args{conn}, "multistore");
	Admin::License::license($args{conn}, "snapmirror");
}


sub is_vfiler_setup
{}

sub is_policyserver_setup
{}

sub is_policyserver_running
{}

sub is_cluster_setup
{}

sub is_takeover_complete
{}

sub is_giveback_complete
{}

sub is_giveback
{}

sub help
{
	logcomment("Help for Setup");
}


sub testme 
{
	package main;
	our ($FILER, $FILER_IP, $FILER_CONN, $CLIENT, $CLIENT_TYPE, $POLICY, $PROTOCOL, $VERSION, $VOLUME, $CMDTIMEOUT, $MOUNT, $FORCESETUP);

	param('FILER',       '-mesg',    "Filer to test");
	param('FILER_IP',       '-mesg',    "Filer to test_IP");
	param('FILER_CONN','-default',"#rsh");
        param('POLICY',   '-default', 'pol1');
        param('CLIENT',   '-mesg', 'CLIENT');
        param('CLIENT_TYPE',   '-mesg', 'CLIENT_TYPE');
        param('PROTOCOL',   '-mesg', 'PROTOCOL');
        param('VERSION',   '-mesg', 'VERSION');
        param('MOUNT',   '-mesg', 'MOUNT PATH');
        param('VOLUME',   '-mesg', 'VOLUME');
        param('CMDTIMEOUT',   '-default', '30');
        param('FORCESETUP',   '-default', '0');

	use FPolicy::Setup;


	my $stat = FPolicy::Setup::setup_cifs (
		host => $FILER,
		dns_server_list => "172.19.2.30",
		'auth-type' => "ad",
		domain_user => "Administrator",
		domain_pass => "BigCheese",
		domain => "cifs.lab.netapp.com",
		force_setup => $FORCESETUP
	);

	
}


1;
