# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/FPolicy/Syntax.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Syntax;

use strict;
use warnings;
use Utils qw(make_host make_conn);
use Tharn qw(say logcomment logresult);
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;

use vars qw(%Syntax);

my %Syntax = (
	system_defaults => {
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
		cifs => { type => SCALAR, default => "not licensed", optional=>1 },
		fpolicy => { type => SCALAR, default => "not licensed", optional=>1 },
		license => { type => SCALAR, default => "not licensed", optional=>1 }
	},

	create => { oper => { type => SCALAR, regex => qr/create/i}, 
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
	},

	enable => { oper => { type => SCALAR, regex => qr/enable/i },
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
 	},

	disable => { oper => { type => SCALAR, regex => qr/disable/i },
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
 	},

	destroy => { oper => { type => SCALAR, regex => qr/destroy/i },
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
 	},

	cifs => { oper => { type => SCALAR, regex => qr/[restart|terminate|setup]/i }, 
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
 	},

	monitor => {
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
		oper => {type=> SCALAR, regex => qr/[add|remove|set]/i},
		proto => {type => SCALAR, default => "cifs,nfs", regex => qr/[nfs|cifs]/i},
		opspec => {type => SCALAR, 
			default => "create,read,write,delete,rename,open,close,setattr,getattr,lookup,dircreate,dirdelete,dirrename,link,symlink", 
			regex => qr/[create|read|write|delete|rename|open|close|setattr|getattr|lookup|dircreate|dirdelete|dirrename|link|symlink]/i}
	},

	volume => {
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
		oper => { tpye => SCALAR, regex => qr/[add|remove|set|reset|show|eval]/i },
		method => { type => SCALAR, regex => qr/[inc|exc](l)?(u)?(s)?(i)?(o)?(n)?/i },
		vol_spec => { type => SCALAR, regex => qr/(\w+)(,(\w)+)*/i },
		command => { type => SCALAR, regex => qr/fpolicy(\s)+vol([u|um|ume])?(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(,(\w+))*/i, optional => 1 },
		cmdtype => {type => SCALAR, regex => qr/(\w+)/i},
		policy => {type => SCALAR, regex => qr/(\w+)/i}
	},

	extension => {
		cmd_type => {type => SCALAR, reqex => qr/(\w+)/ },
		oper => { tpye => SCALAR, regex => qr/[add|remove|set|reset|show]/i },
		method => { type => SCALAR, regex => qr/[inc|exc](l)?(u)?(d)?(e)?/i },
		extension => { type => SCALAR, regex => qr/(\w+)(,(\w)+)*/i },
		command => { type => SCALAR, regex => qr/fpolicy(\s)+ext([e|en|ens|ensi|ensio|ension])?(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(\s)+(\w+)(,(\w+))*/i, optional => 1 },
		cmdtype => {type => SCALAR, regex => qr/(\w+)/i},
		policy => {type => SCALAR, regex => qr/(\w+)/i}
	},

);

