# -*- perl -*-
#  $Id: //depot/prod/test/main/lib/Utils.pm#1 $


## @summary Library of Utils routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::Utils;

use strict;
use warnings;
use Utils qw(make_host make_conn);
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF OBJECT);
use Tharn qw(host connect);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;
use Tharn;
use FPolicy::Execute qw(ExecuteCmd ExecuteAndExpect ValidateExpected);

require Exporter;
use base qw(Exporter);
our @EXPORT_OK = qw(
			make_connection		
			make_NV	
			make_arrayhash 
			make_hash_fromArrayHash
			Win_Connect
			ReadSetupFile
			PreProcess_InputParams
			DumpData
			Is_policy_exists
			trim_str
			stats_parse
		);
		
sub trim_str($)
{
	my $string = shift;
	$string =~ s/^\s+//;
	$string =~ s/\s+$//;
	return $string;
}

sub make_NV 
{
	my @nvs = @_;
	my %temp_nv = ();
	my @tmp = ();

	foreach my $nv (@nvs) {
		if(ref($nv) eq "HASH") { my ($n, $v) = split /=>/, $nv; $temp_nv{$n} = $v; } 
		else {
		my ($n, $v) = split /=/, $nv;
		$v = $n if(!defined($v) and $v eq "");
		$temp_nv{$n} = $v;
		push @tmp, "$n = $v";
		}
	}
	return %temp_nv;
}

sub make_hash_fromArrayHash
{
	my @options = @_;
	my %parm = ();

	my $n = "";
	foreach my $ar (@options)
	{
		if($n ne "") { $parm{$n} = $ar; $n = ""; }
		else { $n = $ar; }
	}

	return %parm;
}

sub array_from_hash
{
	my $ref = shift;
	my @arr = ();
	foreach my $key (keys %{$ref}) 
	{
		push @arr, "$key=${$ref}{$key}";
	}
	return @arr;
}

sub make_arrayhash 
{
	my $ref = shift;
	my %nvs = %{$ref};
	my @temp_nv = ();

	foreach my $nv (keys %nvs) {
		push @temp_nv, $nv;
		push @temp_nv, $nvs{$nv};
	}
	return @temp_nv;
}

sub PreProcess_InputParams
{
	my $ptr = shift;
	my @opt = @_;
	my %expected = make_NV(@opt);

	if(defined $expected{RESULT}) {
		if($expected{RESULT} eq "")
		 { $ptr->{FLAGS}{RESULT} = "PASS"; }
		else { $ptr->{FLAGS}{RESULT} = $expected{RESULT}; }
		logcomment("Negate Key = $expected{RESULT}, $ptr->{FLAGS}{RESULT}");
		delete $expected{RESULT};
	} else {
		$ptr->{FLAGS}{RESULT} = "PASS";
	}

	@opt = array_from_hash(\%expected);
	return @opt;
}

sub Win_Connect
{
	my %ds = shift;

	require CIFS::Utils_gui;
	import CIFS::Utils_gui qw(vdthrash);

	return vdthrash(filer => "nss-u46",
			share => "y:",
		);
}

sub Connect 
{
	my @params = @_;

	my %ds = make_NV(@params);

	if($ds{type} =~ /policyserver/i) {
		if($ds{plat} =~ /win/i) {
			Win_Connect(%ds);
		} else {
			logcomment("Invalid Platform type ($ds{plat}) for File Policy Server $ds{name}:$ds{ip}");
		}
	}
	$ds{fps}{session} = FPolicy::Utils::Connect("type=policyserver", 
				"plat=win",
				"name=$ds{fps}", 
				"ip=$ds{fps}{IP}", "pipe=$ds{fps}{PIPE}"
			);
}

sub make_connection
{
	my @parms = @_;

	my %NV = make_NV(@parms);
	my $filer = $NV{filer};
	my $conn_type = $NV{conn_type};

	#print "\n Connecting to $filer \n";
	my $conn;
	my $hostp;

	if($NV{conn_type} =~ /[cli|console]/i) {
		# Create the telnet connection for the filer and fill up the
		# root_pass in our args if we don't know it yet.
		my $host = $NV{filer};
		$host =~ s/\'//g;
		logcomment("Connecting to Filer $host");
		$hostp = Tharn::host("$host;hosttype=filer");
		$conn = Tharn::connect("#rsh");
		die($@) if ($@); # remove when the return eval is removed
		if (defined($NV{CMDTIMEOUT})) {
			$conn->timeout($NV{CMDTIMEOUT});
		}
	}else {
		logcomment("Invalid Connection type : $NV{conn_type}");
	}
	logcomment(" {host=>$hostp, conn=>$conn}");
	return {host=>$hostp, conn=>$conn};
}

sub Launch_PolicyServer
{
	my @parms = @_;

	my %opt = make_NV(@parms);

	if(!TestEnv::MinWin::minwin_compatible($opt{client})) {
		ExitConfigError("Client $opt{client} is not MinWin compatible");
	}

	
}

sub fp_make_host
{
	my $self = shift;
	my $filer = $self->{FILER}{NAME};
	my $h;
	if( !defined $self->{FILER}{HOST} ) {
		logcomment("Creating new CLI host for $filer");
		$h = make_host($filer);
		$self->{FILER}{HOST} = $h;
	}
	return $h;
}

sub make_array_to_validate
{
	my @opt = @_;
	my @parm =  ();
	foreach my $key (@opt) {
		my ($n, $v) = split /=/, $key;
		if($v ne "") {
			push @parm, $n;
			push @parm, $v;
		}
	}
	return @parm;
}

sub Setup_Cifs
{
	my $self = shift;
	my @parms = @_;
	my $filer = $self->{FILER}{NAME};
	my $retVal = 0;

	my $host = fp_make_host($self);
	push @parms, "host=$host";
	my %opt = make_NV(@parms);
	my @arr = make_array_to_validate(@parms);

	print "\nArry = @arr\n";
	my %temp = validate(@arr, {
			host => {type => SCALAR | OBJECT},
			dns_server_list => {type => SCALAR},
			'auth-type' => {type => SCALAR},
			domain_user => {type => SCALAR},
			domain_pass => {type => SCALAR},
			domain => {type => SCALAR},
			force_setup => {type => SCALAR, default=> 0, regex => qr/(\d)/}
			});

	if (!CIFS::Setup::is_cifs_setup(host => $host) or $opt{force_setup}) {
		logcomment("CIFS is not setup. Performing cifs setup");
		
		$retVal = CIFS::Setup::cifs_setup(
				host => $host,
				dns_server_list=> $opt{dns_server_list},
				'auth-type' => $opt{'auth-type'},
				domain_user => $opt{domain_user},
				domain_pass => $opt{domain_pass},
				domain => $opt{domain}
			 );
	} else {
		logcomment("CIFS is already setup");
	}

	return $retVal;
}

sub FindReadSetupFile
{
	my $filename = shift;
	my $buf = "";
	my $status = 1;
	my $fh;
	my $STARTBIT = "{";
	my $STOPBIT = "}";
	my %reshash = ();
	my $start = 0;
	my $error = "";
	my $counter = 0; # to catch up all the different policy configurations
	my @toprint = ();

	#logcomment("FindReadSetupFile = $filename");

	open($fh, $filename) or warn "Couldnot read file $filename";
		my $line = "";
		while ( $line = <$fh> ) {
			$buf .= "$line";
			$status = 0;
			if ($line =~ /[$STARTBIT]/) { $start = 1; } 

			# found STARTBIT, store rest of data till STOPBIT
			elsif ( $start ) {
				my ($name, $value, @rest) = split /=/, $line;
				foreach my $r (@rest) { $value .= "=$r"; } # patch up the rest of tokens if the line was of type (name=value1=value2=value3) due to user error 
				$reshash{$counter}{$name} = "$value";
				logcomment("$counter - $name = $value");
				push @toprint, "$counter - $name = $value";
			}

			elsif ($line =~ /[$STOPBIT]/) { $start = 0; $counter++; }
			else {
				# else, skip blank or uknown lines
				print "\nInvalid / blank line\n";
			}
		}
	$error = "Did not find any useful configuration data. Check the format and tags" if (!$counter);
	close $fh;
	#logcomment("FindReadSetupFile - result = $status, buff = $buf");
	#return { result => $status , buffer => $buf };
	#logcomment("ParseStore_SetupData - result = $counter, hash = %reshash");
	#logcomment("ParseStore_setupData - Values = @toprint");
	return { result => $counter, hash => %reshash };
}

sub ReadSetupFile
{
	my $filename = shift;
	my $status = "";
	my $buf = "";
	my %reshash = ();
	my %resh = ();

#	if ( ref( $filename ) eq "HASH" ) {
		# Data is already presented in the hash format. Parse and Store the data
#		$status = ParseStore_SetupData ( $filename );
#		logcomment("SetupFile : Hash value $filename, ParseStore status = $status");
#	} elsif ( ref( $filename ) eq "SCALAR" and -f $filename ) {
		# Data File is presented. Read and then send it to Parse and Store the data
		%reshash  = FindReadSetupFile ( $filename );
		$status = $reshash{result};

		if ( $reshash{result} ) {
			#logcomment("Read_SetupFile: FindReadSetupFile for filename $filename returned error : $status, $reshash{buffer}, $@");
#		} else {
#			%resh = ParseStore_SetupData ( "$reshash{buffer}" );
#			logcomment("Read_SetupFile: ParseStore status = $resh{result}") if( $resh{result} );
		}
		#logcomment("Read_SetupFile: FindReadSetupFile for filename $filename returned error : $@") if( $status );
#	}
	return %resh;
}

sub push_arr
{
	my @ele = @_;
	my @arr = ();
	foreach my $ele (@ele) { push @arr, $ele; }
	return @arr;
}

sub DumpData
{
	my $ptr = shift;

	print "\n REF = ". ref($ptr). " \n";
	if(ref($ptr) eq "HASH") {
		print "\n Working on Hash\n";
		print "\nKeys = ". keys(%{$ptr}) ."\n";
		foreach my $e (keys %{$ptr}) {
			print "\nT = ${$ptr}{$e}\n";
			DumpData(${$ptr}{$e});
		}
	} elsif(ref($ptr eq "ARRAY")) {
		print "\n Working on Array\n";
		foreach my $e (@{$ptr}) {
			print "\nT = ${$ptr}{$e}\n";
			DumpData(${$ptr}{$e});
		}
	} elsif(ref($ptr eq "SCALAR")) {
		print " $ptr \n";
	}
}

sub ConfigureSetup
{
	my (@opt) = @_;

	my %option = validate(@opt, {
		setupfile => { type => SCALAR, regex => qr/(\w+)/ }
		} );
	my %NVt = ReadSetupFile( "$option{setupfile}" );

	if ( $NVt{result} ) { logresult("fail", "Failed during reading setupfile $option{setupfile} with error $NVt{error}, $@"); } 

	my @obj = ();
	my $count = 0;
	foreach my $index (keys %NVt) {
		#my @arr = ();
		#foreach my $ar (keys %{$NVt{$index}}) { 
		#	push @arr, "$ar=$NVt{$index}{$ar}"; 
		#}
		$obj[$count] = new FPolicy("policy=$NVt{$index}{policy}",
			"monitor=$NVt{$index}{monitor}",
			"ext_inc=$NVt{$index}{ext_inc}",
			"ext_exc=$NVt{$index}{ext_exc}",
			"pserver_pri=$NVt{$index}{policy_server_primary}",
			"pserver_sec=$NVt{$index}{policy_server_secondary}",
			"cifs_domain=$NVt{$index}{cifs_domain}",
			"cifs_ad=$NVt{$index}{cifs_ad}",
			"cifs_aduser=$NVt{$index}{cifs_aduser}",
			"cifs_adpass=$NVt{$index}{cifs_adpass}",
			"cifs_multiproto=$NVt{$index}{cifs_multiprotocol}",
			"cifs_dns=$NVt{$index}{cifs_dns}",
			"cifs_setup=$NVt{$index}{cifs_setup}" );
		
		$obj[$count]->{cli}->create();
		$obj[$count]->{cli}->monitor("oper=add", "proto=nfs");
		$obj[$count]->{cli}->extension();
		$obj[$count]->{cli}->volume();
		$count++;
	}
	return @obj;
}

sub Is_policy_exists
{
	my %parms = validate(@_, {
		conn => {type => HASHREF}, 
		intf => {type => SCALAR},
		vfiler => {type => SCALAR, optional => 1},
		policy => { type => SCALAR, regex =>  qr/(\w+)/ }
		});

	my $cmd = FPolicy::Grammer::GenerateCmd(
				vfiler => "$parms{vfiler}",
				cmd_type => "show",
				policy => $parms{policy} );


	my $status = 
	FPolicy::Execute::ExecuteCmd($parms{conn}, "fpolicy show $parms{policy}");

	return ($status =~ /Policy name $parms{policy} not found/)? 0 : 1;
}

sub stats_parse($)
{
	my ($stats) = @_;
	my @lines = split(/\r?\n/, $stats);
	my $line;
	my $ret = {};

	foreach $line (@lines)
	{
		my @elts = split(/:/, $line);
		my $object = $elts[0];
		my $instance = join(":", @elts[1 .. $#elts - 2]);
		my $counter = $elts[$#elts - 1];
		my $value = $elts[$#elts];
		$ret->{$object}{$instance}{$counter} = $value;
		logcomment("object=$object instance=$instance counter=$counter value=$value");
	}
	
	return $ret;
}

1;

