# -*- perl -*-
#  $Id: //depot/prod/test/Rsierranevada/lib/FPolicy/ZAPI.pm#1 $


## @summary Library of FPolicy routines
## @author  sureshb (DRG FPolicy QA)
## @status  private
## @pod     here

package FPolicy::ZAPI;

use strict;

#use warnings;
#use warnings;
use Params::Validate qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF OBJECT);
use Tharn qw(say logcomment logresult host );
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Subtest;
use FPolicy::Utils qw(make_NV PreProcess_InputParams Is_policy_exists make_connection make_arrayhash make_hash_fromArrayHash trim_str);
use FPolicy::Grammer qw(GenerateCmd ValidateParams);
use FPolicy::Parser qw( Get_LocalBlkdPolicyStat Get_Monitored Get_PServer Get_Extension Get_PolicyStat Get_PolicyState Get_Volume Get_CifsState CLI_Execute_Validate_Outputstr get_STATES CLI_Verify_NV_Generic );
use FPolicy::Constants qw(%OpsTemplate);

use FPolicy::Generator qw(CLI_Expected_Str);

use FPolicy::Execute;
use FPolicy::Utils;
use FPolicy::Grammer;
use FPolicy::Parser;

use strict;
use ZapiErrno;
use ZapiTest;
use NaElement;
use misc;

#sub new
#{
#       my $object = shift;
#       my $class = ref($object) || $object;
#       my $self = {};
#       my $interface = "zapi";
#       bless($self, $class); 
#       return $self;
#}

sub new ($;@)
{
	my $object = shift;
	my $class = ref($object) || $object;

	my %NV = make_NV(@_);
    #    logcomment ("@@@@ entered object create");
	
	my $host = host($NV{filer});
	my $default_password = $host->default_password();

	my $ZAPI = {
	       INTF => "zapi",
		FILER => {
			NAME => "$NV{filer}",
			IP => "$NV{filer_ip}",
			VERSION => $NV{version} || ""
			},

		POLICY => {
			NAME => "$NV{policy}",
			TYPE => "$NV{type}" || "screen",
			MONITOR => "$NV{monitor}",
			PROTOCOL => "$NV{proto}",
			EXT_INC => "$NV{ext_inc}",
			EXT_EXC => "$NV{ext_exc}",
			VOL_INC => "$NV{vol_inc}",
			VOL_EXC => "$NV{vol_exc}",
			PSERVER_PRI => "$NV{pserver_pri}",
			PSERVER_SEC => "$NV{pserver_sec}",
			},

		VFILER => {NAME => "$NV{vfiler}" || "vfiler0"},

		CLIENT => "$NV{client}",
		DEFAULT_PASSWORD => "$default_password",
		CONN => undef,
		HOST => undef,

	};

	logcomment("Initializing Zapi Interface
		INTF = zapi
		FILER{NAME} = $NV{filer}
		FILER{VERSION} = $NV{version}
		FILER{IP} = $NV{filer_ip}
		POLICY{NAME} = $NV{policy}
		POLICY{TYPE} = $NV{type}
		POLICY{MONITOR} = $NV{monitor}
		POLICY{PROTOCOL} = $NV{proto}
		POLICY{EXT_INC} = $NV{ext_inc}
		POLICY{EXT_EXC} = $NV{ext_exc}
		POLICY{VOL_INC} = $NV{vol_inc}
		POLICY{VOL_EXC} = $NV{vol_exc}
		POLICY{PSERVER_PRI} = $NV{pserver_pri}
		POLICY{PSERVER_SEC} = $NV{pserver_sec} ");


	bless($ZAPI, $class); 

	return $ZAPI;
}


sub help
{
	logcomment("HELP for ZAPI");
}




sub fpolicy_secondary_servers {

	my $self = shift;
	
	my ($policy_name,$filer_ip,$result,@sec_ip) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});

	my $e = new NaElement("fpolicy-set-secondary-servers");

	my $info = new NaElement("policy-name", $policy_name);
	$e->child_add($info);

	# Define a list of servers
	my $args = new NaElement("secondary-servers");
	$e->child_add($args);

	my $ip;
	foreach $ip (@sec_ip) {
		$info = new NaElement("secondary-server-info");
		$info->child_add_string("server-ip","$ip");
		$args->child_add($info);
	}

	#$info = new NaElement("secondary-server-info");
	#$info->child_add_string("server-ip","3.3.3.3");
	#$args->child_add($info);

	my $o = $server->invoke_elem($e,"passed");
	my $out = $o->toString();

	logcomment ("-comment", $o->sprint_pretty());
	return decideResult($out, $result, "adding the Secondary server IP");

#		my $out = $o->toString();
#		
#		if($out =~ /results status=\"$result\"/i)
#		{
#			logresult("pass","Zapi call for adding the Secondary server IP passed");	
#		}else
#		{
#			logresult("fail","Zapi call for adding the secondary server IP failed");	
#	}
}


sub fpolicy_extensions {
	
	my $self = shift;
	
	my ($policy_name,$filer_ip,$set_name,$command,$result,@extensions) = @_;
	
	#logcomment ("### $policy_name,$filer_ip,$set_name,$extensions,$command,$result");
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		
	my $e = new NaElement("fpolicy-extensions");

	my $info = new NaElement("policy-name", $policy_name);
	$e->child_add($info);

	$info = new NaElement("set-name", $set_name);
	$e->child_add($info);

	$info = new NaElement("command", $command);
	$e->child_add($info);

	# Define a list of extensions
	my $args = new NaElement("extensions");
	$e->child_add($args);
	
	#$info = new NaElement("extension-list-info");
	my $ext;
	
	foreach $ext (@extensions) {
		logcomment ("#### Adding extension $ext");
		$info = new NaElement("extension-list-info");
		$info->child_add_string("name-spec", $ext);
		$args->child_add($info);
	}
	
	#$info = new NaElement("extension-list-info");
	#$info->child_add_string("name-spec",$extensions);
	#$args->child_add($info);
	
	#$info = new NaElement("extension-list-info");
	#$info->child_add_string("name-spec","bbb");
	#$args->child_add($info);


	my $o = $server->invoke_elem($e,$result);

	logcomment ("-comment", $o->sprint_pretty());
	my $out = $o->toString();
	return decideResult($out, $result, "adding the extensions");
	
#	if($out =~ /results status=\"$result\"/i)
#	{
#		logresult("pass","Zapi call for adding the extensions passed");	
#	}else
#	{
#		logresult("fail","Zapi call for adding the extensions failed");	
#	}
	
}


sub fpolicy_global_enable {

	my $self = shift;
	
	my ($filer_ip,$result) = @_;
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
	
	my $e = new NaElement("fpolicy-enable");

	my $o = $server->invoke_elem($e,"passed");
	logcomment("$o->sprint_pretty()");
	my $out = $o->toString();
	return decideResult($out, $result, "Global policy enable");

#	if($out =~ /results status=\"$result\"/i)
#		{
#			#logresult("pass","Zapi call for Global policy enable passed");
#			logcomment("PASS: ZAPI call for Global policy enable passed.");
#			return 0;
#		}else
#		{
#			#logresult("fail","Zapi call for Global policy enable failed");
#			logcomment("FAIL: ZAPI call for Global policy enable failed.");
#			return 1;
#	}
		
}


sub fpolicy_global_disable {

	my $self = shift;
	
	my ($filer_ip,$result) = @_;
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
	
	my $e = new NaElement("fpolicy-disable");

	my $o = $server->invoke_elem($e,"passed");
	#logcomment("$o->sprint_pretty()");
	my $out = $o->toString();
	return decideResult($out, $result, "Global policy disable");

#	if($out =~ /results status=\"$result\"/i)
#		{
#			#logresult("pass","Zapi call for Global policy disable passed");
#			logcomment("PASS: ZAPI call for Global policy disable passed.");
#			return 0;
#		}else
#		{
#			#logresult("fail","Zapi call for Global policy disable failed");
#			logcomment("FAIL: ZAPI call for Global policy disable failed.");
#			return 1;
#	}
	
		
}



sub fpolicy_disable {

	my $self = shift;
	
	my ($policy_name,$filer_ip,$result) = @_;
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
	
	my $e = new NaElement("fpolicy-disable-policy");

	my $info = new NaElement("policy-name", $policy_name);
	$e->child_add($info);

	my $o = $server->invoke_elem($e,"passed");
	logcomment("$o->sprint_pretty()");
	my $out = $o->toString();
	return decideResult($out, $result, "Policy disable");

#	if($out =~ /results status=\"$result\"/i)
#		{
#			logresult("pass","Zapi call for policy disable passed");
#			logcomment("-comment", $o->sprint_pretty() );
#		}else
#		{
#			logresult("fail","Zapi call for policy disable failed");
#			logcomment("-comment", $o->sprint_pretty() );
#	}
	
		
}



sub fpolicy_enable {
	
	my $self = shift;
	
	my ($policy_name,$filer_ip,$result) = @_;
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
	
	my $e = new NaElement("fpolicy-enable-policy");

	my $info = new NaElement("policy-name", $policy_name);
	$e->child_add($info);

	my $o = $server->invoke_elem($e,"passed");
	logcomment("$o->sprint_pretty()");
	my $out = $o->toString();
	return decideResult($out, $result, "Policy enable");

#	if($out =~ /results status=\"$result\"/i)
#		{
#			logresult("pass","Zapi call for policy enable passed");
#			logcomment("-comment", $o->sprint_pretty() );
#		}else
#		{
#			logresult("fail","Zapi call for policy enable failed");
#			logcomment("-comment", $o->sprint_pretty() );
#	}
	
		
}

sub fpolicy_destroy {

	my $self = shift;

	my ($policy_name,$filer_ip,$result) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});


	my $e = new NaElement("fpolicy-destroy-policy");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);

	my $o = $server->invoke_elem($e,$result);

	logcomment("-comment", $o->sprint_pretty());
	
	my $out = $o->toString();
		
	#### Decide the result
	return decideResult($out, $result, "Policy destroy");

#	if($out =~ /results status=\"$result\"/i)
#	{
#		logresult("pass","Zapi call for policy destroy passed");
#		logcomment("-comment", $o->sprint_pretty() );
#	}else
#	{
#		logresult("fail","Zapi call for policy destroy failed");
#		logcomment("-comment", $o->sprint_pretty() );
#	}
}



sub fpolicy_create {

	my $self = shift;
	
	my ($policy_name,$filer_ip,$result) = @_;
	logcomment("$self->{default_password}");
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		
	logcomment("-comment", "Create policy $policy_name");

	my $e = new NaElement("fpolicy-create-policy");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);

	$info = new NaElement("policy-type", "screen");
	$e->child_add($info);

	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,$result);
	
	my $out = $o->toString();
	
	#### Decide the result
	
	return decideResult($out, $result, "Policy create");
#	if($out =~ /results status=\"$result\"/i)
#	{
#		logresult("pass","Zapi call for policy create passed");
#		logcomment("-comment", $o->sprint_pretty() );
#	}else
#	{
#		logresult("fail","Zapi call for policy create failed");
#		logcomment("-comment", $o->sprint_pretty() );
#	}
	
}

sub fpolicy_retrive {

	my $self = shift;
	
	my ($policy_name,$filer_ip, $is_enable, $screened_files, $screen_fails, $screen_local_fails) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});

	my $e = new NaElement("fpolicy-list-info");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);

	
	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	
	my $out = $o->toString();
	
	logcomment("$out");
	if(!($self->{FILER}{VERSION} eq "7.2"))
	{
		if (($out =~ /<policy-name>$policy_name<\/policy-name>/i) && ($out =~ /<is-enabled>$is_enable<\/is-enabled>/i) && ($out =~ /<number-of-screened-files>$screened_files<\/number-of-screened-files>/i) && ($out =~ /<number-of-screen-failures>$screen_fails<\/number-of-screen-failures>/i) && ($out =~ /<number-of-requests-blocked-locally>$screen_local_fails<\/number-of-requests-blocked-locally>/i))
		{
			return decideResult("results status=\"passed\"", "passed", "Policy retrive");
			#logresult("pass","Policy retrive Zapi call passed");
			#logcomment ("####  Output required $out");
		}else
		{
			return decideResult("results status=\"failed\"", "passed", "Policy retrive");
			#logresult("fail","Policy retrive Zapi call Failed");
			#logcomment ("####  Output required $out");
		}
	}
	else
	{
		if (($out =~ /<policy-name>$policy_name<\/policy-name>/i) && ($out =~ /<is-enabled>$is_enable<\/is-enabled>/i) && ($out =~ /<number-of-screened-files>$screened_files<\/number-of-screened-files>/i) && ($out =~ /<number-of-screen-failures>$screen_fails<\/number-of-screen-failures>/i))
		{
			return decideResult("results status=\"passed\"", "passed", "Policy retrive");
			#logresult("pass","Policy retrive Zapi call passed");
			#logcomment ("####  Output required $out");
		}else
		{
			return decideResult("results status=\"failed\"", "passed", "Policy retrive");
			#logresult("fail","Policy retrive Zapi call Failed");
			#logcomment ("####  Output required $out");
		}
	}
}



sub fpolicy_retrive_extensions {
	
	my $self = shift;
	
	my ($policy_name,$filer_ip,$extension_type,$result,@extensions) = @_;
		
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		
	my $e = new NaElement("fpolicy-extensions-list-info");

	my $info = new NaElement("policy-name", $policy_name);
	$e->child_add($info);

	
	my $o = $server->invoke_elem($e,$result);

	logcomment ("-comment", $o->sprint_pretty());
	my $out = $o->toString();
	
	my $ext;
	my $result_flag;
	$result_flag = 0;
	my $ptrn_strt=0;
	my $ptrn_end=0;
	
	if ($extension_type =~ /include/i)
	{
		$ptrn_strt = "<include-extensions>";
		$ptrn_end  = "<\/include-extensions>";
		
	}else
	{
		$ptrn_strt = "<exclude-extensions>";
		$ptrn_end  = "<\/exclude-extensions>";
	}
	
	foreach $ext (@extensions) {
		if ($out =~ /$ptrn_strt.+<name-spec>$ext<\/name-spec>.+$ptrn_end/i)
		{
			logcomment("Extension $ext exist in $extension_type list");
			$result_flag += 0;
		}else
		{
			logcomment ("Extension $ext doesn't exist in the $extension_type list");
			$result_flag += 1;
		}
	}		
	
	if (!$result_flag)
	{
		#logresult("pass","Policy retrive for extension $extension_type passed");
		logcomment("PASS: Policy retrive for extension $extension_type passed");
		return 0;
	}else
	{
		#logresult("fail","Policy retrive for extension $extension_type Failed");
		logcomment("FAIL: Policy retrive for extension $extension_type failed");
		return 1;
	}
	
}


sub retrive_fpolicy_secondary_servers {

	my $self = shift;
	
	my ($policy_name,$filer_ip,$result,@sec_ip) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});

	my $e = new NaElement("fpolicy-get-secondary-servers-info");

	my $info = new NaElement("policy-name", $policy_name);
	$e->child_add($info);
	
	my $o = $server->invoke_elem($e,"passed");
	my $out = $o->toString();
	
	my $ip;
	my $result_flag = 0;
	
	foreach $ip (@sec_ip) {
		if ($out =~ /<server-ip>$ip<\/server-ip>/i) {
			logcomment("IP: $ip exist in secondary server IP list");
			$result_flag += 0;
		}else
		{
			logcomment("IP: $ip doesn't exist in secondary server IP list");
			$result_flag += 1;
		}
	}
	
	if (!$result_flag) {
		#logresult("pass", "Zapi call for retriving secondary server IP list passed");
		logcomment("PASS: Zapi call for retriving secondary server IP list passed");
		return 0;
	}else
	{
		#logresult("fail", "Zapi call for retriving secondary server IP list failed");
		logcomment("FAIL: Zapi call for retriving secondary server IP list failed");
		return 1;
	}
	
}


sub fpolicy_set_required {

	my $self = shift;
	
	my ($policy_name,$filer_ip, $is_enable, $result) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		

	my $e = new NaElement("fpolicy-set-required");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);
	
	$info = new NaElement("required",$is_enable);
	$e->child_add($info);

	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	my $out = $o->toString();
	
	return decideResult($out, $result, "setting Policy Options Required");

}

sub fpolicy_get_required_info {

	my $self = shift;
	my $res = 0;
	
	my ($filer_ip, $is_required, $result) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		

	my $e = new NaElement("fpolicy-get-required-info");

	my $info = new NaElement("policy-name",$self->{POLICY}{NAME});
	$e->child_add($info);
	
	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	my $out = $o->toString();
	
	if($self->{FILER}{VERSION} eq "7.2")
	{
			if (($out =~ /<required>(.+)<\/required>/i ))
		{
			if($1 ne $is_required)
			{
				$res |= 1;
				logcomment("$out" );
				logcomment("FAIL: Zapi call for Get Required Status failed. No matching Status");
			}
			else
			{
				logcomment("$out" );
				logcomment("PASS: Zapi call for Get Required Status passed");
			}
		
		}
		else
		{
			$res |= 1;
			logcomment("$out" );
			logcomment("FAIL: Zapi call for Get Required Status failed");
		}
	}
	else
	{
		if (($out =~ /<is-required>(.+)<\/is-required>/i ))
		{
			if($1 ne $is_required)
			{
				$res |= 1;
				logcomment("$out" );
				logcomment("FAIL: Zapi call for Get Required Status failed. No matching Status");
			}
			else
			{
				logcomment("$out" );
				logcomment("PASS: Zapi call for Get Required Status passed");
			}
		
		}
		else
		{
			$res |= 1;
			logcomment("$out" );
			logcomment("FAIL: Zapi call for Get Required Status failed");
		}
	}
	
	return $res;
	
}

sub fpolicy_server_list_info {

	my $self = shift;
	
	my ($policy_name,$filer_ip, $result, @pserver_ip) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		

	my $e = new NaElement("fpolicy-server-list-info");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);
	

	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	my $out = $o->toString();
	my $ip;
	my $result_flag = 0;
	foreach $ip (@pserver_ip) {
		if ($out =~ /<server-ip>$ip<\/server-ip>/i)
		{
			logcomment("IP: $ip exist in FPserver IP list");
			$result_flag += 0;
		}else
		{
			logcomment("IP: $ip doesn't exist in FPserver IP list");
			$result_flag += 1;
		}
	}
		
	if (!$result_flag)
	{	
		if($result =~ /pass/i)
		{
			logcomment("PASS: Zapi call for retriving FP_server IP list expected to pass and it passed");
			return 0;
		}
		else
		{
			logcomment("FAIL: Zapi call for retriving FP_server IP list expected to pass and it failed");
			return 1;
		}
	}else
	{
		if($result =~ /fail/i)
		{
			logcomment("PASS: Zapi call for retriving FP_server IP list expected to fail and it failed");
			return 0;
		}
		else
		{
			logcomment("FAIL: Zapi call for retriving FP_server IP list expected to fail and it passed");
			return 1;
		}
	}
}

sub fpolicy_list_info {

	my $self = shift;
	
	my ($policy_name,$filer_ip) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		

	my $e = new NaElement("fpolicy-list-info");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);

	
	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	
	my $out = $o->toString();
	
	return $out;
}

sub fpolicy_set_policy_options {

	my $self = shift;
	
	my ($policy_name, $filer_ip, $option, $boolean) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		

	my $e = new NaElement("fpolicy-set-policy-options");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);
	
	$info = new NaElement($option, $boolean);
	$e->child_add($info);

	
	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	### ex: ontapi 10.72.206.9 root "" fpolicy-set-policy-options policy-name p1 is-cifs-setattr 1
	
	my $out = $o->toString();
	
	logcomment ("-comment", $o->sprint_pretty());
		
		
	if($out =~ /results status=\"passed\"/i) {
		logresult("pass","Zapi call for setting the policy options passed");	
		
	} else {
	
		logresult("fail","Zapi call for setting policy options failed");	
	}
}

sub  fpolicy_server_stop {

	my $self = shift;
	
	my ($policy_name,$filer_ip, $pserver_ip,$result) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		

	my $e = new NaElement("fpolicy-server-stop");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);
	
	$info = new NaElement("server-ip",$pserver_ip);
	$e->child_add($info);
	

	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	my $out = $o->toString();
	
	return decideResult($out, $result, "stoppping fpolicy server ");

}

sub fpolicy_get_policy_options {

	my $self = shift;
	
	my ($policy_name,$filer_ip, $option , $is_enabled) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		

	my $e = new NaElement("fpolicy-get-policy-options");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);

	
	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	
	my $out = $o->toString();
	
	if ($is_enabled) {
		if ($out =~ /<$option>true/i) {
			logresult("pass","$option is enabled");
			logcomment ("####  Output required $out");

		} else {
			logresult("fail", "$option is expected to be enabled, but status returned not enabled");
			logcomment ("####  Output required $out");
		}
		
	} else {
		if ($out =~ /<$option>false/i) {
			logresult("pass", "$option is disabled");
			logcomment ("####  Output required $out");
		} else {
			logresult("fail", "$option is expected to be disabled, but status returned wrong value");
			logcomment ("####  Output required $out");
		}
	}		
}


sub  fpolicy_server_list_options {

	my $self = shift;
	
	my ($policy_name,$filer_ip,$result) = @_;

	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});

	my $e = new NaElement("fpolicy-server-list-info");

	my $info = new NaElement("policy-name",$policy_name);
	$e->child_add($info);
	

	## Invoke the Zapi call..
	my $o = $server->invoke_elem($e,"passed");
	my $out = $o->toString();
	# logcomment ("####  Output required $out");
	# if ($out =~ /<$option>$result<\/$option/i) {
		# logresult("pass", "Zapi fpolicy-server-list-info call passed");
			
	# } else {
		# logresult("fail", "Zapi fpolicy-server-list-info call failed");	
	# }
	return $out;
	
}

sub fpolicy_status {

	my $self = shift;
	
	my ($filer_ip,$status,$result) = @_;
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
	
	my $e = new NaElement("fpolicy-status");

	my $o = $server->invoke_elem($e,"passed");
	
	my $out = $o->toString();
	
	if($self->{FILER}{VERSION} eq "7.2")
	{
		if ($out =~ /<enable-value>$status<\/enable-value/i)
		{
			return decideResult($out,$result,"Fpolicy Status")
		}
		else
		{
			return decideResult($out,"fail","Fpolicy Status")
		}
	}
	else
	{
		if ($out =~ /<is-enabled>$status<\/is-enabled/i)
		{
			return decideResult($out,$result,"Fpolicy Status")
		}
		else
		{
			return decideResult($out,"fail","Fpolicy Status")
		}
	}
}

sub fpolicy_volume_list_set {
	
	my $self = shift;
	
	my ($filer_ip,$list_type,$result,@vol_list) = @_;
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
		
	my $e = new NaElement("fpolicy-volume-list-set");

	my $info = new NaElement("policy-name", $self->{POLICY}{NAME});
	$e->child_add($info);

	$info = new NaElement("list-type", $list_type);
	$e->child_add($info);

	#$info = new NaElement("command", $volumes);
	#$e->child_add($info);

	# Define a list of volumes
	my $args = new NaElement("volumes");
	$e->child_add($args);
	
	my $vol;
	
	foreach $vol (@vol_list) {
		logcomment ("#### Adding volume $vol");
		$info = new NaElement("fpolicy-volumes-list-info");
		$info->child_add_string("volume-spec", $vol);
		$args->child_add($info);
	}
	
	my $o = $server->invoke_elem($e,$result);

	logcomment ("-comment", $o->sprint_pretty());
	my $out = $o->toString();
	return decideResult($out, $result, "adding the volumes");
	
}


sub fpolicy_volume_list_info {

	my $self = shift;
	my $res = 0;
	
	my ($filer_ip,$result,@vol_list) = @_;
	
	my $server = new ZapiTest($filer_ip,"root",$self->{DEFAULT_PASSWORD});
	
	my $e = new NaElement("fpolicy-volume-list-info");

	my $info = new NaElement("policy-name", $self->{POLICY}{NAME});
	$e->child_add($info);

	my $o = $server->invoke_elem($e,"passed");
	logcomment("$o->sprint_pretty()");
	my $out = $o->toString();
	
	my $vol;
	foreach $vol (@vol_list) {
		if (!($out =~ /<volume-spec>$vol<\/volume-spec/i))
		{
			$res |= 1;
			logcomment("FAIL: Volume $vol not in list");
		}
	}
	
	return decideResult($out, $result, "Verifying the volumes");
}



sub decideResult
{

	my ($output, $expectedResult, $case) = @_;
	my $retResult = 0;
	
	if($output =~ /results status=\"$expectedResult\"/i)
	{
		logcomment("$output" );
		logcomment("PASS: Zapi call for $case passed");
	}else
	{
		logcomment("$output" );
		logcomment("FAIL: Zapi call for $case failed");
		$retResult = 1;
	}
	
	return $retResult;
}

sub finalResult
{
        shift @_;
        my $res = shift;
#        logcomment ("result value is in ZAPI module is $res");
        if($res) {
                logresult("fail", "Test case failed. Refer to the logs to debug the issue.");
        } else {
                logresult("pass", "Test case passed.");
        }
}



1;
