#Hey emacs, this is basically -*- perl -*-
# $Id$

#this library is intended for C-mode Raid error propagation scripts

=head1 NAME

  FS::Aggr_block_check- Library that supports  aggr block_check operation
                        with different options using NACL APISet

=head1 DESCRIPTION

  The Aggr_block_check library consists of function to perform following tasks:

   aggr_blockcheck() :  runs the aggr block_check  
   aggr_blockcheck_wait(): waits till aggr_block_check completes
   aggr_blockcheck_read_output(): reads output of aggr_block_check
  

=head1 SYNOPSIS

  
    use FS::Aggr_block_check;
    
    ($errcode,$blockcheck_path) = aggr_blockcheck( api_obj => $CLI_Obj,aggr => "wqa_gx9_aggr1" ,pvbn=>\@pvbns, wait=>1);
                           
    if ($errcode != 0) {
        $Log->comment("aggr_block_check FAILED with eroor: $blockcheck_path");
    }
    
         
=head1 STATUS
 @status private
 
=head1 AUTHOR
 @author Hardikkumar.Darji@netapp.com

=head1 DISCLAIMER

 @disclaimer This library does NOT support duplicate vol names in the same aggr. 
             In short no vserver checking is available.
=cut


# Compiler directives.
use strict;
use warnings;

# =============================================================
# Module Import
# =============================================================

require Exporter;

package FS::Aggr_block_check;
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF HASHREF OBJECT BOOLEAN UNDEF);
use NACL::APISet;
use vars qw (@ISA @EXPORT_OK);
use Data::Dumper;
use Tharn qw($Log);
use POSIX;
use NACL::APISet::Exceptions::ResponseException qw(:try);

# =============================================================
# Components Used
# =============================================================

use NACL::C::Node;

@ISA       = qw(Exporter);
@EXPORT_OK = qw(aggr_blockcheck aggr_blockcheck_wait aggr_blockcheck_read_output);

# =============================================================
# Declare Global Variables in BEGIN
# =============================================================

BEGIN {
    use vars qw(
        $Node
        $Node_name
        $Api_Set_Obj
        );
}


#$Node = NACL::C::Node->find();

=head2 aggr_blockcheck(%opt)
This function will start the aggr block_check scanner on the filer aggr 
and wait until it finishes scanning


Returns the aggr block_check command output file in case of wait equal to 1.

Returns the scanid of the block_check scanner and ls output of /etc/log/ 
which has been obtained before executing the aggr block_check command in case of wait equal to 0. 

This scanid and ls output can be passed to the aggr_blockcheck_wait function 
to get the output file of aggr block_check command later.

=item %opt
This is the hash with the following keys

     api_obj => APISet object Required
     
     aggr  => name of the aggregate Required

     pvbn  => SCALAR|ARRAYREF of pvbn numbers Optional.

     disks => SCALAR|ARRAYREF of disk id's Optional.

     wait => 0|1 Optional
     if 1,Wait until the aggr block_check scanner to complete scanning. 
     default is 1.
     if 0,Didn't wait until the block_check scanner to complete.
Usage:
             ($errcode, $path) = aggr_blockcheck(api_obj=>$API_Set_obj, 
                                                 aggr => $aggr, 
                                                 pvbn=>\@pvbn, 
                                                 wait=>1);
             ($scanid, $ls_etc_log_before)=aggr_blockcheck(api_obj=>$API_Set_obj, 
                                                           aggr => $aggr, 
                                                           pvbn=>\@pvbn, 
                                                           wait=>0);

=cut

sub aggr_blockcheck (%)
{
        my %opts=validate(
             @_,
             {       api_obj => { type=>OBJECT,optional => 0},
                     aggr      => {type => SCALAR,,optional => 0},
                     pvbn      => {type => SCALAR|ARRAYREF, optional =>1},
                     disks     => {type => SCALAR|ARRAYREF, optional =>1},
                     wait      => {type => SCALAR, default =>1, optional=>1}
             }

         );

        my $api_obj        = $opts{api_obj};
        my $aggr           = $opts{aggr};
        my $pvbn           = $opts{pvbn};
        my $disks          = $opts{disks};
        my $wait           = $opts{wait};

        my ($resp, $output, $result, $after, $before, $errcode, 
            $scanid, $ls_etc_log_before,$scan_out);
        my $str      = "Starting scan on $aggr";
        my $start_msg= "Starting aggr block_check ";
        my $err_msg5 = "usage: ";
        my $err_msg2 = "aggr block_check: ";
        my $err_msg3 ="block_check:";
        my $err_msg4 ="Invalid .*PVBN";
        my $err_msg1 ="Aggregate $aggr not found";
        my $err_msg6 = "Disk.*not found";
        my $err_msg7 = "Disk.*not part of $aggr"; 
        my $err_msg8 = "Disk.*not a data disk in $aggr";
	my $err_msg9 = "block_check already running";
        
        if(defined($pvbn)) {

           my $cmd = "priv set diag;aggr block_check $aggr -p @$pvbn";
        
           ##Getting the ls /etc/log/ before running block_check command
           ##which will be used to get the block_check output file later
           $ls_etc_log_before=&_get_files($api_obj);

           ##Running aggr block_check command 
           $resp = $api_obj->system_node_run(node => "local",command => "\" $cmd \"");
	   $output=Dumper($resp->get_raw_output());
	   
        }

        if(defined($disks)){
      
          ##Getting the ls /etc/log/ before running block_check command
          ##which will be used to get the block_check output file later
          $ls_etc_log_before=&_get_files($api_obj);

          my $cmd ="priv set diag;aggr block_check $aggr -d @$disks";
	  
          ##Running aggr block_check command
          $resp = $api_obj->system_node_run(node => "local",command => "\" $cmd \"");
	  $output=Dumper($resp->get_raw_output());
        }
        if ($output =~
            /$err_msg1|$err_msg2|$err_msg3|$err_msg4|$err_msg5|$err_msg6|$err_msg7|$err_msg8|$err_msg9/)
        {
            $errcode = 1;
            $result  = $output;
            return ($errcode, $result);
        }
	elsif ($output =~ /$start_msg|$str/)
	{
            ##waiting for aggr block check to start scanning
            sleep(5);
	    my $cmd="priv set -q diag;wafl scan status -A $aggr";
	    $resp = $api_obj->system_node_run(node => "local",command => "\" $cmd \"");
            $scan_out = Dumper($resp->get_raw_output());

            if ($scan_out =~ /(\d+)\s+aggr\sblock_check\s+Aggregate\s+/) {
                $scanid = $1;
            }
        }

        if (defined($scanid)) {

            if ($wait == 1) {
                $result = 
                  aggr_blockcheck_wait(api_obj           => $api_obj,
                                       scanid            => $scanid,
                                       aggr              => $aggr,
                                       ls_etc_log_before => $ls_etc_log_before);
                $errcode = 0;
            } else {

                return ($scanid, $ls_etc_log_before);

            }
        } else {

            if ($wait == 1) {
                $result = 
                  aggr_blockcheck_wait(api_obj             => $api_obj,
                                       aggr              => $aggr,
                                       ls_etc_log_before => $ls_etc_log_before);
                $errcode = 0;

            } else {
                return (undef, $ls_etc_log_before);
            }
        } 

        return ($errcode, $result);
}
=head2 aggr_blockcheck_wait(%opt)
This function will wait for the aggr block_check scanner on the filer aggregate
to complete scanning.

Returns the aggr block_check command output file after completion of scanning.

=item %opt
This is the hash with the following keys

     api_obj             => APISEt Object Required

     aggr              => name of the aggregate Required

     scanid            => scanid of the block_check scanner Optional.
                          (Use scanid obtained from aggr_blockcheck function.)

     ls_etc_log_before => ls output of /etc/log/ before obtained before running aggr block_check.
                          (Use ls_etc_log_before obtained from aggr_blockcheck function.)
Usage:
         $blockcheck_path = aggr_blockcheck_wait(api_obj=>$APISet_obj,
                                                 aggr => $aggr,
                                                 scanid=> $scanid,
                                                 ls_etc_log_before=> $ls_etc_log_before);

=cut


sub aggr_blockcheck_wait(%){

        my %opts=validate(@_,
                          {api_obj               => {type => OBJECT},
                           aggr                  => {type => SCALAR},
                           scanid                => {type => SCALAR, optional=>1},
                           ls_etc_log_before     => {type => ARRAYREF}
                          });

        my $api_obj        = $opts{api_obj};
        my $aggr           = $opts{aggr};
        my $scanid         = $opts{scanid};
        my $ls_etc_log_before = $opts{ls_etc_log_before};
        my ($output_file, $output);
        my $count =1;
        my $resp;
	
        if(defined($scanid)){
           do {
	      my $cmd="priv set -q diag;wafl scan status -A $aggr";
	      $resp = $api_obj->system_node_run(node => "local",command => "\" $cmd \"");
              $output = Dumper($resp->get_raw_output());
              
              ##Instead of checking Wafl scan status many times which will increase rsh count,
              ##putting sleep to reduce the rsh connection count with the filer
              sleep(30);
              if($output !~ /$scanid/){
                 $count =0;
                }
	    
           
           } while (($count == 1)); 
         }
         # finding all files in "/etc/log/" after issuesing aggr block_check
	 my $ls_etc_log_after=&_get_files($api_obj);
	 
         my @union =my @inter = my @diff = ();
         my %count = ();

        foreach my $element (@$ls_etc_log_after, @$ls_etc_log_before) {
                $count{$element}++;
		
        };
        foreach my $element (keys %count) {
                push @union, $element;
                push @{ $count{$element} > 1 ? \@inter : \@diff }, $element;
        };
        foreach my $diff (@diff){
	    
            if ($diff =~ /(aggr_blockcheck.$aggr.\d+)/){
                my $path ="/etc/log/";
                $diff =$1;
                $output_file = "$path" . "$diff";
            }
       }
       $Log->comment("path of file= ".$output_file);
       return $output_file;

}
=head2 aggr_blockcheck_read_output(%opt)
This function will read the aggr block_check output file

When defined pvbn
Returns the hash reference with pvbn and the corresponding file path, level and error code

When defined disks
Returns hash of array of toatl scanned disks,lostblocks vbn,corrupted disks,snapshoot lists 
and the hash of number of lost blocks in each volume on corruption


=item %opt
This is the hash with the following keys

     api_obj => hostspec Required

     path  => path of the block_check output file Required.

     pvbn  => SCALAR|ARRAYREF of pvbn numbers Optional

     disks => 0|1 defaults to 0 Optional.

Usage:
    $pvbn_file = aggr_blockcheck_read_output(api_obj=>$APISet_obj,
                                             path => $file_path,
                                             pvbn=>\@pvbn);
    ($scanned_disk,$vvbns_v,$corrupted_disks_v,$levels_v,$fbn_v,$error_code_v,$snap_list_v,$count)= aggr_blockcheck_read_output(api_obj=>$APISet_obj,
                                                                                                        path => $file_path,
                                                                                                        disks=>1);

=cut

sub aggr_blockcheck_read_output (%)
{
    my %opts = validate(
	@_,
	{
		api_obj => {type => OBJECT },
		path =>  {type => SCALAR },
		pvbn =>  {type => SCALAR|ARRAYREF, optional =>1},
		disks => {type => SCALAR, default=> 0,optional =>1}
	}
    );

    my $api_obj = $opts{api_obj};
    my $path = $opts{path};
    my $pvbn_ref = $opts{pvbn};
    my $disks = $opts{disks};
    my $output;
    my $complete_msg = "Completed aggregate block check";
    my $cmd= "priv set -q diag;rdfile $path";
    my $resp;
    do{
	$resp = $api_obj->system_node_run(node => "local",command => "\" $cmd \"");
	$output = Dumper($resp->get_raw_output());
	sleep(5);
    }until ($output =~ /$complete_msg/s);
   
    my @lines = split /\n/, $output;

    my $pvbn_file = {};
    #my $lost_blocks_msg = "Lost blocks: ";
    my @scanned_disk;
    my (@vvbn_list, @corrupted_disk);
    my $count =0;
    my $snapshot_msg = "The following snapshots contain any of the listed blocks:";
    my $snapshots_corrupt;
    my @snapshot_list ;
    
    my (%level,%fbn,%error_code);
    my(%count,$vbn,%vvbns_v,%corrupted_disks_v,%levels_v,%fbn_v,%error_code_v,%snap_list_v,$new_vvol);
    $snapshots_corrupt =0;
    
    if ($disks == 1)
    {
        my $disk_no_count =0;
        foreach (@lines)
	{
	    # since each  Lost VVBN line has disk name
	    # after first occurence of Completed scan for volume
	    # we will not consider disk for @scanned_disk
	    if($_=~/Completed scan for volume/)
	    {		
		$disk_no_count=1;
	    }
	    
	    if($_ =~ /(\w+\.\w+.*)/ && $disk_no_count==0)
	    {
	       my @disk_list = split / /, $1;               
		foreach my $list (@disk_list)
		{
		    push @scanned_disk, $list;
		}

	    }
                
	    # New format in RR.0
	    # Lost VVBN: 3489435, Disk: 0a.68, Volume: vv_t, Inode: 98, Level: 0, Fbn: 19, References: 0, Error: 119

	    if($_ =~ /Lost\sVVBN:\s(\d+),\s+Disk:\s(.*),\sVolume:\s(.*),\s+Inode:\s+(\d+),\s+Level:\s(.*),\sFbn:\s(.*),\sReferences:\s.*,\sError:\s(.*)/)
	    {
                # creating Hash of Arrays to store valuses according to Volume they relate to
		#getting volume (the Key for the Hash)
                $new_vvol=$3;
                #storing lost vvbn in to Hash
                push @{$vvbns_v{$new_vvol}}, $1;
                #storing corrupted disks in to Hash
                push @{$corrupted_disks_v{$new_vvol}},$2;
                #storing Level in to Hash
                push @{$levels_v{$new_vvol}},$5;
                #storing Fbn in to Hash
                push @{$fbn_v{$new_vvol}},$6;
                #storing Error code in to Hash
                push @{$error_code_v{$new_vvol}},$7;                     
                    
           }
   
	    # Determine if there are any corrupt blocks in the snapshots.
	    if($_ =~ /$snapshot_msg/)
	    {
	       $snapshots_corrupt = 1;
	    }
    
	    # If there are snapshots, then add them to a list.
	    if(($_ =~ /\d+:\d+:\d+/) && ($snapshots_corrupt == 1))
	    {
		 my @line = split / /, $_;
		 
		 push @{$snap_list_v{$new_vvol}},$line[0];
	    }
    
	    # Store the number of lost blocks
	    if($_ =~ /Total\s(\d+)\slost\sblock/)
	    {
		$count{$new_vvol} += $1;
		$snapshots_corrupt =0;
	    }
      
      
    }
    return (\@scanned_disk,\%vvbns_v,\%corrupted_disks_v,\%levels_v,\%fbn_v,\%error_code_v,\%snap_list_v,\%count);
}
    if(defined($pvbn_ref)){

	my @pvbn_nos  = @$pvbn_ref;
	#$Log->comment("in pvbn ref");
	foreach my $temp_no (@pvbn_nos) {
	    #$Log->comment("temp =$temp_no");
	    for (my $i = 0; $i < @lines; $i++) {
		# Str to look for completed block check
		my $mat_str = "Completed aggregate block check ";
		
		# New format in RR.0
		# PVBN: 145854, Disk: 0a.68, Volume: vv_t, Inode: 100, Level: 0 Error: 119
		# Inode 100 has 1 name(s).
		# Volume: testvol_flex, Inode: 100, Pathname: /vol/testvol_flex/file1

		if($lines[$i] =~ m/PVBN:\s($temp_no),\s+.*,\s+Level:\s+(.*).*Error:\s+(.*)/i) {
		    #$Log->comment("inside pvbn regex");
		    $level{$temp_no}=$2;
		    $error_code{$temp_no}=$3;
		    $lines[$i + 2] =~ m/Pathname:\s(.*)/;
		    $pvbn_file->{$temp_no} = $1;
		    
		}
		if($lines[$i] =~ m/$mat_str/) {
		    last;
		}
        
	    }
	}
      
      return ($pvbn_file,\%level,\%error_code);

      }

}

sub _get_files()
{
    my ($api_obj,$resp,$ls_out,@dmp);
    
    $api_obj= shift;
    
    # issuing "ls /etc/log/" to find out which files are available (before and after issuing aggr block_check)
    $resp = $api_obj->system_node_run(node => "local",command => "\" priv set -q test;ls /etc/log/ \"");
    $ls_out=Dumper($resp->get_raw_output());
    @dmp = split(/\n/, $ls_out);
    my @array;
    foreach (@dmp)
    {   #removing element containing  just $VAR1 and ' from array 
	if(!($_ =~/\$VAR1 |^'/))
        {
           #removing element containing just blank space from array
           if(!($_ eq ""))
            {
		push(@array,$_); 
            }            
        }
        
    }
    # returning the raw output of the "ls " command
    # please parse the output so that it has only file names
    return (\@array);
}
1;
