# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/EA/AggregateMemberTask.pm#1 $
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##
##	    This AggregateMemeber task library accomodates all Aggregate Member
##      related function that are reusable by team.
##
##      This library is developed by WAFL-EA team.
##
##	@description
## 	    This Library covers the following functions
##
##	@functions
##
##		 create ()
##		 check_state ()
##
## 	@dependencies
##		NACL::C::AggregateMember;
##		NACL::C::Aggregate;
##		EventlogTask;
##		Data::Dumper;
##		NACL::APISet::Exceptions::ResponseException qw(:try);
##
## 	@notes
##
##	@duration
##
##
## 	@author c-dl-rakesh-automation@netapp.com
##
## 	@status public
##

package AggregateMemberTask;

use NACL::C::AggregateMember;
use base qw(FS::EA::Exceptions::WaflironException);
use Carp qw(confess);
use Tharn qw($Log);
use Params::Validate
	qw(validate validate_with SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use strict;
use warnings;
use Data::Dumper;

#####################################################################
#
#   function new :
#   	Constructor for package AggregateMemberTask.
#
#   usage : $wafliron_obj = AggregateMemberTask->new(command_interface =>
#           $command_interface)
#
#####################################################################

sub new {
	$Log->enter();
	my $pkg   = shift;
	my $class = ref($pkg) || $pkg;
	my %opts  = validate @_,
		{ command_interface => { isa => 'NACL::C::CommandInterface' }, };

	my $self = {@_};
	bless $self, $class;
	$Log->exit();

	# Return the blessed object
	return $self;
}

#####################################################################
#
#   function check_state :
#   	Check the aggregate member for a particuler status(online|offline).
#       Wait until the status is achieved.
#
#   usage : $aggr_mem_obj = AggregateMemberTask->new(command_interface =>
#           $command_interface);
#			$aggr_mem_obj->check_status($params);
#
#
#####################################################################

sub check_state() {

	my $self = shift;
	my $pkg  = ref($self);

	$Log->enter();

	my %opts = validate @_, {
		'member_aggregate' => { type => SCALAR },
		'expected_state'   => { type => SCALAR },
		'wait_time'        => { type => SCALAR, optional => 1 }

	};

	my ( $object, $state );
	my $endtime = time() + $opts{'wait_time'};
	$object = NACL::C::AggregateMember->find(
		command_interface => $self->{'command_interface'},
		filter            => { aggregate => $opts{'member_aggregate'} }
	);
	$state = $object->state()->state();
	while ( $state ne $opts{'expected_state'} ) {
		Tharn::sleep 5;
		$state = $object->state()->state();
		if ( time() > $endtime ) {
			NACL::APISet::Exceptions::ResponseException->throw( text =>
					  "Failure :: Aggregate Member $opts{'member_aggregate'} "
					. " could not reach the state \"$opts{'expected_state'}\" "
					. "before timeout " );
		}
	}    ##While loop end
}

#####################################################################
#
#   function create :
#   	Creates a agggregate member.
#
#   usage : $aggr_mem_obj = AggregateMemberTask->new(command_interface =>
#           $command_interface);
#			$aggr_mem_obj->create($params);
#
#
#####################################################################

sub create {
	my $self = shift;

	my %opts = validate @_,
		{     aggr_member_params => { type => HASHREF },
		'wait_time'        => { type => SCALAR, default => 300 },
		allow_extra => 1,
		};

	#      $self->{aggregate_params} = {$opts{aggregate_params}};
	#Check if there are spare disks.
	#change ownership if spare disks exists but unassigned.
	$self->{aggr_obj} = NACL::C::AggregateMember->create(
		command_interface => $self->{command_interface},
		$opts{aggr_member_params}
	);
	my $aggr_obj_state = $self->{aggr_obj}->state();
	my $endtime        = time() + $opts{'wait_time'};
	while ( $aggr_obj_state->state() ne "online" ) {
		Tharn::sleep 5;
		$aggr_obj_state = $self->{aggr_obj}->state();
		if ( time() > $endtime ) {
			NACL::APISet::Exceptions::ResponseException->throw(
				      text => "Failure :: "
					. "Aggregate Member $opts{aggr_member_params}->{aggrgeate} could not"
					. " reach the status \"Online\" before timeout " );
		}    # if ( !$time_limit) ends here
	}    # while ( $opts{'wait_time'} ) ends here
	return 1;
}

1;
