# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/EA/AggregateTask.pm#1 $
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##
##	    This Aggregate task library accomodates all Aggregate
##      related function that are reusable by team.
##
##      This library is developed by WAFL-EA team.
##
##	@description
## 	    This Library covers the following functions
##
##	@functions
##
##		 create ()
##		 create_aggregate_member ()
##		 check_wafliron_member_state ()
##		 get_error_message ()
##		 check_all_wafliron_member_state ()
##		 wafliron_start ()
##		 wafliron_stop ()
##		 get_aggr_state()
##		 get_aggr_obj()
##		 check_wafliron_status ()
##		 check_wafliron_attributes ()
##		 check_wafliron_state ()
##		 wait_wafliron_status ()
##		 get_wafliron_member_status ()
##		 check_wafliron_member_attributes ()
##		 aggr_exists()
##		 delete()
##		 check_state()
##		 check_aggregate_online ()
##		 check_all_aggr_member_online ()
##		 set_aggr_obj ()
##
## 	@dependencies
##		NACL::C::AggregateMember;
##		NACL::C::Aggregate;
##		Data::Dumper;
##		NACL::APISet::Exceptions::ResponseException qw(:try);
##
## 	@notes
##
##	@duration
##
##
## 	@author c-dl-rakesh-automation@netapp.com
##
## 	@status (private to WAFL-EA)
##

package AggregateTask;
use base qw(FS::EA::Exceptions::WaflironException);
use NACL::C::AggregateMember;
use NACL::C::Aggregate;
use Data::Dumper;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use Tharn qw($Log);
use Params::Validate
	qw(validate validate_with SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use strict;
use warnings;
my $Infinity = 9**9**9;
use Data::Dumper;

#####################################################################
#
#   function new :
#   	Constructor for package AggregateTask.
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#
#####################################################################

sub new {
	my $pkg   = shift;
	my $class = ref($pkg) || $pkg;
	my %opts  = validate @_,
		{ command_interface => { isa => 'NACL::C::CommandInterface' }, };
	my $self = {%opts};
	bless $self, $class;
	$Log->exit();

	# Return the blessed object
	return $self;
}

#####################################################################
#
#   function create :
#   	Creates an aggregate
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->create($create_params);
#
#####################################################################

sub create {
	my $self = shift;
	my %opts = validate_with(
		params => \@_,
		spec   => {
			'aggregate'   => { type => SCALAR },
			'wait_time' => { type => SCALAR, default =>  $Infinity},
			'poll_interval' => { type => SCALAR, default => 120 },
		},
		allow_extra => 1
	);

	my $wait_time = delete( $opts{'wait_time'} );
	my $poll_interval = delete $opts{'poll_interval'};

	if ( !$self->aggr_exists( aggregate => $opts{aggregate} ) ) {

		#Check if there are spare disks.
		#change ownership if spare disks exists but unassigned.
		$self->{'aggr_obj'} = NACL::C::Aggregate->create(
			'command_interface' => $self->{command_interface},
			%opts
		);
		my $end_time = time() + $wait_time ;
		while ( $self->{'aggr_obj'}->state()->state() ne "online" ) {
			if ( time() > $end_time) {
				NACL::APISet::Exceptions::ResponseException->throw( text =>
						"Aggregate $opts{aggregate} failed to be Online" );
			}
			Tharn::sleep $poll_interval;
		}
	}
}

#####################################################################
#
#   function create_aggregate_member :
#   	Creates an aggregate member
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->create_aggregate_member($create_params);
#
#####################################################################

sub create_aggregate_member {
	$Log->enter();
	my $self = shift;
	my %opts = @_;
	if ( !exists( $self->{'aggr_obj'} ) ) {
		%opts = validate_with(
			params => \%opts,
			spec   => {  aggregate   => { type => SCALAR },
				'wait_time' => { type => SCALAR, default =>  $Infinity }
			},
			allow_extra => 1
		);
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}
	my $wait_time         = delete( $opts{'wait_time'} );
	my $aggr_obj_state    = $self->{'aggr_obj'}->state();
	my $command_interface = $self->{command_interface};
	my $aggregate         = delete( $opts{aggregate} );
	$self->{'aggr_obj'}->add_members(%opts);
	my @member_aggregate = NACL::C::AggregateMember->find(
		'command_interface' => $command_interface,
		filter              => { aggregate => $aggregate . "*" }
	);

	if ( $#member_aggregate > 1 ) {
		my $member_aggr_state
			= $member_aggregate[$#member_aggregate]->state();
		while ( $member_aggregate[$#member_aggregate]->state()->state()
			ne "online" )
		{
			if ( $wait_time <= 0 ) {
				NACL::APISet::Exceptions::ResponseException->throw( text =>
						"Aggregate $opts{aggregate} failed to be Online" );
			}
			Tharn::sleep 5;
			$wait_time -= 1;
		}
	}
}

#####################################################################
#
#   function check_wafliron_member_state :
#   	check wafliron member state
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->check_wafliron_member_state($params);
#
#   parameters :
#
# 		 wafl_member_name : wafliron member name
#		 attributes       : hashref containing values to check.
#
#####################################################################

sub check_wafliron_member_state {
	my $self = shift;
	my %opts = @_;
	%opts = validate_with(
		params => \%opts,
		spec   => {
			'wafl_member_name' => { type => SCALAR },
			attributes         => { type => HASHREF },
		},
		allow_extra => 1
	);

	my $vol_member_obj = NACL::C::WaflironMember->find(
		command_interface => $self->{command_interface},
		filter            => { aggregate => $opts{'wafl_member_name'} }
	);
	my $wafliron_status  = $vol_member_obj->state();
	my $mis_match_degree = 0;
	my $error_message    = "";
	foreach my $key ( keys( %{ $opts{attributes} } ) ) {
		my $key1 = $key;
		$key1 =~ s/-/_/g;
		if ( $wafliron_status->$key1() ne $opts{'attributes'}->{$key} ) {
			$mis_match_degree++;
			$error_message = $error_message
				. " Actual Val : ["
				. $wafliron_status->$key1()
				. "] mismatch-with Expected Val : [$opts{'attributes'}->{$key}] \n";
			$self->{'aggr_obj'}->{error_message} = $error_message;
		}
	}
	if ($mis_match_degree) {
		return 0;
	}
	return 1;

}

sub get_error_message {
	my $self = shift;
	return $self->{'aggr_obj'}->{error_message};
}

#####################################################################
#
#   function check_all_wafliron_member_state :
#   	check all wafliron member state
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->check_all_wafliron_member_state($params);
#
#   parameters :
#
#		 attributes       : hashref containing values to check.
#
#####################################################################

sub check_all_wafliron_member_state {
	my $self = shift;
	my %opts = @_;
	my $member_state;
	$Log->enter();
	%opts = validate_with(
		params      => \%opts,
		spec        => { 'attributes' => { type => HASHREF } },
		allow_extra => 1
	);
	if ( !exists( $self->{'aggr_obj'} ) ) {
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}

	my @members_status = $self->get_wafliron_member_status();
	my $status         = 0;
	foreach my $member_state (@members_status) {
		$status = $status || $self->check_wafliron_member_state(
			'wafl_member_name' => $member_state->aggregate(),
			attributes         => $opts{attributes}
		);
	}
	return $status;
}

#####################################################################
#
#   function wafliron_start :
#   	Starts wafliron on an aggragte
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->wafliron_start($params);
#
#
#####################################################################

sub wafliron_start {
	my $self = shift;
	my %opts = @_;
	if ( !exists( $self->{'aggr_obj'} ) ) {
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}
	NACL::C::Wafliron->start(
		command_interface => $self->{command_interface},
		aggregate         => $self->{'aggr_obj'}->aggregate()
	);
}

#####################################################################
#
#   function wafliron_stop :
#   	Stops a running wafliron on an aggragte
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->wafliron_stop($params);
#
#
#####################################################################

sub wafliron_stop {
	my $self = shift;
	my %opts = @_;
	if ( !exists( $self->{'aggr_obj'} ) ) {
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}
	NACL::C::Wafliron->stop(
		command_interface => $self->{command_interface},
		aggregate         => $self->{'aggr_obj'}->aggregate()
	);
}

sub get_aggr_state {
	my $self = shift;
	my %opts = @_;

	if ( !exists( $self->{'aggr_obj'} ) ) {
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}
	return $self->{'aggr_obj'}->state();
}

sub get_aggr_obj {
	my $self = shift;
	return $self->{'aggr_obj'};
}

#####################################################################
#
#   function check_wafliron_status :
#   	check running wafliron status
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->check_wafliron_status($params);
#
#
#####################################################################

sub check_wafliron_status {
	my $self = shift;
	my %opts = @_;

	if ( !exists( $self->{'aggr_obj'} ) ) {
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}
	my @vol_member_obj = NACL::C::Wafliron->find(
		command_interface => $self->{command_interface} );
	my $i = 0;
	my $j = 0;
	my @wafliron_status;
	foreach my $vol_member_obj (@vol_member_obj) {
		$wafliron_status[$i] = $vol_member_obj->state();
		if ( $wafliron_status[$i]->{aggregate}
			eq $self->{'aggr_obj'}->aggregate() )
		{
			$j = $i;
			last;
		}
		$i++;
	}
	return wantarray ? @wafliron_status : $wafliron_status[$j];
}

#####################################################################
#
#   function check_wafliron_attributes :
#   	check running wafliron state attributes
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->check_wafliron_attributes($params);
#
#
#####################################################################

sub check_wafliron_attributes {
	my $self = shift;
	my %opts = @_;
	%opts = validate_with(
		params      => \%opts,
		spec        => { attributes => { type => HASHREF } },
		allow_extra => 1
	);
	my $wafliron_status = $self->check_wafliron_status(%opts);
	$Log->comment( "Dumper = " . Dumper($wafliron_status) );
	my $mis_match_degree = 0;
	my $error_message    = undef;
	foreach my $key ( keys( %{ $opts{'attributes'} } ) ) {
		my $key1 = $key;
		$key1 =~ s/-/_/g;
		if ( $wafliron_status->$key1() ne $opts{'attributes'}->{$key} ) {
			$mis_match_degree++;
			$error_message = $error_message
				. " Actual Val : ["
				. $wafliron_status->$key1()
				. "] mismatch-with Expected Val : [$opts{'attributes'}->{$key}] \n";
		}
	}
	if ($mis_match_degree) {
		NACL::APISet::Exceptions::ResponseException->throw(
			text => $error_message );
	}
	return 1;
}

sub check_wafliron_state {
	my $self = shift;
	my %opts = @_;
	%opts = validate_with(
		params      => \%opts,
		spec        => { attributes => { type => HASHREF } },
		allow_extra => 1
	);
	my $wafliron_status  = $self->check_wafliron_status(%opts);
	my $mis_match_degree = 0;
	my $error_message    = undef;
	foreach my $key ( keys( %{ $opts{'attributes'} } ) ) {
		my $key1 = $key;
		$key1 =~ s/-/_/g;
		if ( $wafliron_status->$key1() ne $opts{'attributes'}->{$key} ) {
			$mis_match_degree++;
			$error_message = $error_message
				. " Actual Val : ["
				. $wafliron_status->$key1()
				. "] mismatch-with Expected Val : [$opts{'attributes'}->{$key}] \n";
		}
	}
	if ($mis_match_degree) {
		return 0;
	}
	return 1;
}

#####################################################################
#
#   function wait_wafliron_status :
#   	wait for a user provided wafliron status
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->wait_wafliron_status($params);
#
#
#####################################################################

sub wait_wafliron_status {
	$Log->enter();
	my $self = shift;
	my %opts = @_;
	%opts = validate_with(
		params => \%opts,
		spec   => {
			'expected_status' => { type => SCALAR },
			'wait_time'       => { type => SCALAR, default =>  $Infinity }
		},
		allow_extra => 1
	);
	if ( !exists( $self->{'aggr_obj'} ) ) {
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}

	my $endtime = time() + $opts{'wait_time'};
	my ( $object, $status);
	$object = NACL::C::Wafliron->find(
		command_interface => $self->{'command_interface'},
		filter            => { aggregate => $self->{'aggr_obj'}->aggregate() }
	);
	$status = $object->state()->iron_status();
	while ( $status ne $opts{'expected_status'}  ) {
		Tharn::sleep 10;
		$status = $object->state()->iron_status();
		if (time() > $endtime ) {
			NACL::APISet::Exceptions::ResponseException->throw( text => "Failure :: ".
			"Wafliron on aggregate$opts{'aggregate'} could not reach the status".
			" \"$opts{'expected_status'}\" before timeout ");
		}
	}    ##While loop end
	$Log->exit();
}    ## sub end

#####################################################################
#
#   function get_wafliron_member_status :
#   	get wafliron staus for user provided aggregate
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->get_wafliron_member_status($params);
#
#
#####################################################################

sub get_wafliron_member_status {
	my $self = shift;
	my %opts = @_;
	if ( !exists( $self->{'aggr_obj'} ) ) {
		$self->{'aggr_obj'} = NACL::C::Aggregate->find(
			command_interface => $self->{command_interface},
			filter            => { aggregate => $opts{aggregate} }
		);
	}
	my @wafliron_obj = NACL::C::WaflironMember->find(
		command_interface => $self->{command_interface} );
	my $i = 0;
	my $j = 0;
	my @wafliron_status;
	foreach my $wafliron_obj (@wafliron_obj) {
		$wafliron_status[$i] = $wafliron_obj->state();
		if ( $wafliron_status[$i]->{aggregate}
			eq $self->{'aggr_obj'}->aggregate() )
		{
			$j = $i;
			last;
		}
		$i++;
	}
	return wantarray ? @wafliron_status : $wafliron_status[$j];
}

#####################################################################
#
#   function check_wafliron_member_attributes :
#   	get wafliron attributes for user provided aggregate
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->check_wafliron_member_attributes($params);
#
#
#####################################################################

sub check_wafliron_member_attributes {
	my $self = shift;
	my %opts = @_;
	%opts = validate_with(
		params      => \%opts,
		spec        => { attributes => { type => HASHREF } },
		allow_extra => 1
	);
	my $wafliron_status  = $self->get_wafliron_member_status(%opts);
	my $mis_match_degree = 0;
	my $error_message    = undef;
	foreach my $key ( keys( %{ $opts{'attributes'} } ) ) {
		my $key1 = $key;
		$key1 =~ s/-/_/g;
		if ( $wafliron_status->$key1() ne $opts{'attributes'}->{$key} ) {
			$mis_match_degree++;
			$error_message = $error_message
				. " Actual Val : [$wafliron_status->$key1()] mismatch-with "
				. "Expected Val : [$opts{'attributes'}->{$key}] \n";
		}
	}
	if ($mis_match_degree) {
		NACL::APISet::Exceptions::ResponseException->throw(
			text => $error_message );
	}
}

#####################################################################
#
#   function aggr_exists :
#   	checks if aggregate exists
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->aggr_exists($params);
#
#
#####################################################################

sub aggr_exists() {
	my $self  = shift;
	my %opts  = validate @_, { aggregate => { type => SCALAR }, };
	my @aggrs = NACL::C::Aggregate->find(
		command_interface => $self->{command_interface} );
	foreach my $aggr (@aggrs) {
		if ( $aggr->aggregate() eq $opts{aggregate} ) {
			return 1;
		}
	}
	return 0;
}

#####################################################################
#
#   function delete :
#   	deletes a aggregate
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->delete($params);
#
#
#####################################################################

sub delete() {
	my $self = shift;
	my %opts = @_;
	$self->set_aggr_obj( 'aggregate' => $opts{'aggregate'} )
		if ( !exists( $self->{'aggr_obj'} ) );
	if ( $self->aggr_exists( 'aggregate' => $self->{'aggr_obj'}->aggregate() )
		)
	{
		$self->{'aggr_obj'}->modify( 'state' => 'offline' );
		$self->{'aggr_obj'}->delete( 'force' => 'true' );
	}
}

#####################################################################
#
#   function check_state :
#   	checks aggregate state
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->check_state($params);
#
#
#####################################################################

sub check_state {
	my $self = shift;
	my $pkg  = ref($self);
	my %opts = validate @_, {
		'aggregate'      => { type => SCALAR },
		'expected_state' => { type => SCALAR },
		'wait_time'      => { type => SCALAR, default =>  $Infinity }

	};
	my ( $object, $state );
	my $endtime = time() + $opts{'wait_time'};
	$object = NACL::C::Aggregate->find(
		command_interface => $self->{'command_interface'},
		filter            => { aggregate => $opts{'aggregate'} }
	);
	$state = $object->state()->state();
	while ( $state ne $opts{'expected_state'} ) {
		Tharn::sleep 5;
		$state = $object->state()->state();
		if ( time() > $endtime ) {
			NACL::APISet::Exceptions::ResponseException->throw( text =>
				"Failure :: Aggregate $opts{'aggregate'}  could not reach the "
					. "state \"$opts{'expected_state'}\" before timeout " );
		}

	}    ##While loop end

}

#####################################################################
#
#   function check_aggregate_online :
#   	checks all aggregate's is online
#
#   usage : $aggr_obj = AggregateTask->new(command_interface => $command_inter
#			face)
#			$aggr_obj->check_aggregate_online($params);
#
#
#####################################################################

sub check_aggregate_online {
	my $self = shift;
	my %opts = @_;
	$self->set_aggr_obj( 'aggregate' => $opts{'aggregate'} )
		if ( !exists( $self->{'aggr_obj'} ) );

	my $state = NACL::C::Aggregate->find(
		command_interface => $self->{'command_interface'},
		filter => { 'aggregate' => $self->{'aggr_obj'}->aggregate(), }
	)->state->state();

	if ( $state ne "online" ) {
		NACL::APISet::Exceptions::ResponseException->throw( text =>
				"Aggregate: $self->{'aggr_obj'}->aggregate() is not online" );
	}
}

sub check_all_aggr_member_online {
	my $self = shift;
	my %opts = @_;
	$self->set_aggr_obj( 'aggregate' => $opts{'aggregate'} )
		if ( !exists( $self->{'aggr_obj'} ) );

	my (@mem_objects,@all_members);
	my $pattern = $self->{'aggr_obj'}->aggregate() .'_'.'*' ;

	@all_members = NACL::C::AggregateMember->find(
		command_interface => $self->{'command_interface'}
		#filter => { 'aggregate' => $self->{'aggr_obj'}->aggregate() . '*', }
	);

	foreach (@all_members) {
	       if ( $_->state->aggregate() =~ /$pattern/){
	          push (@mem_objects , $_);
		   }
	}

	my $mis_match_degree = 0;
	my $error_message;

	foreach my $mem (@mem_objects) {
		my $state = $mem->state();
		if ( $state->state() ne "online" ) {
			$mis_match_degree++;
			$error_message = $error_message
				. "$state->aggregate() is in the state: [$state->state()]\n";
		}
	}
	if ($mis_match_degree) {
		NACL::APISet::Exceptions::ResponseException->throw(
			text => $error_message );
	}

}

sub set_aggr_obj {
	my $self = shift;
	my %opts = @_;
	%opts = validate_with(
		params      => \%opts,
		spec        => { 'aggregate' => { type => SCALAR }, },
		allow_extra => 1
	);
	$self->{'aggr_obj'} = NACL::C::Aggregate->find(
		command_interface => $self->{command_interface},
		filter            => { 'aggregate' => $opts{'aggregate'} }
	);
}

1;
