# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/EA/EventlogTask.pm#1 $
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##
##	    This Eventlog task library accomodates all Eventlog
##      related function that are reusable by team.
##
##      This library is developed by WAFL-EA team.
##
##	@description
## 	    FS::EA::EventlogTask library covers the following functions
##
##	@functions
##
##		get_time_stamp()
##		capture_event_log()
##		_get_time_diff()
##		capture_all_event_logs
##
## 	@dependencies
##		use NACL::C::Eventlog;
##		use Tharn qw($Log);
##		use Date::Calc qw(Delta_YMDHMS);
##		use NACL::APISet;
##	`	use Params::Validate
##		NACL::APISet::Exceptions::ResponseException qw(:try);
##
## 	@notes
##
##	@duration
##
##
## 	@author c-dl-rakesh-automation@netapp.com
##
## 	@status (private to WAFL-EA)
##

package EventlogTask;
use NACL::C::Eventlog;
use base qw(FS::EA::Exceptions::WaflironException);
use Date::Calc qw(Delta_YMDHMS);
use NACL::APISet;
use Tharn qw($Log);
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use strict;
use warnings;
use Data::Dumper;

#####################################################################
#
#   function new :
#   	Constructor for package EventlogTask.
#
#   usage : $event_obj = EventlogTask->new(command_interface => $command_inter
#			face)
#
#####################################################################

sub new {
    $Log->enter();
    $Log->warn( "FS::EA::EventlogTask library is deprecated. Please"
            . " use NACL::MTask::EventLogDetector" );
    my $pkg   = shift;
    my $class = ref($pkg) || $pkg;
    my %opts  = validate @_,
        { command_interface => { isa => 'NACL::C::CommandInterface' } };

    my $self = {%opts};
    $self->{'node-name'} = $opts{command_interface}->name();
    bless $self, $class;

    $self->get_time_stamp( $self->{'node-name'} );

    # Return the blessed object
    $Log->exit();
    return $self;
} ## end sub new

#####################################################################
#
#   function get_time_stamp :
#   	Returns the time stamp of the filer(Date show)
#
#   usage : $event_obj = EventlogTask->new(command_interface => $command_inter
#			face)
#			$event_obj->get_time_stamp($params);
#
#   Note:   This function is deprecated. Please use
#           NACL::MTask::EventLogDetector methods.
#
#####################################################################

sub get_time_stamp {

    # FS::EA::EventlogTask library is deprecated. Please use
    # NACL::MTask::EventLogDetector
    $Log->warn( "FS::EA::EventlogTask library is deprecated. Please"
            . " use NACL::MTask::EventLogDetector" );

    my $self        = shift;
    my $date        = undef;
    my $Api_Set_Obj = NACL::APISet->new(
        name      => $self->{'node-name'},
        category  => "Node",
        interface => "CLI",
        set       => "CMode"
    );

    my $text_buffer = $Api_Set_Obj->execute_raw_command(
        command => "system node date show" );
    if ( $text_buffer =~ m[X"(\S+ \S+)((\s+)(\S+))?"X;X"(\S+ \S+)"X] ) {
        $date = $1;
    }
    $self->{time_stamp} = $date;
    $Log->comment("Date string : $date");
    return $date;
} ## end sub get_time_stamp

#####################################################################
#
#   function capture_event_log :
#       Capture evnt logs generated during run time. It maches for the
#       latest log generated by message id.
#
#   usage : $event_obj = EventlogTask->new(command_interface => $command_inter
#			face)
#			$event_obj->capture_event_log($params);
#
#   params :
#       log_details   => {
#	     	                 node        => <node name>,
#          				     messagename => <msg name>
#       				}
#       match_details => {
#	                         regex => <regular expression>,
#                            match => 0 or 1
#						}
#		 init_time     => "4/14/2010 11:35:41"
#
#  returns:  Boolean or string
#
#   Note:   This function is deprecated. Please use
#           NACL::MTask::EventLogDetector methods.
#
#################################################################

sub capture_event_log {

    # FS::EA::EventlogTask library is deprecated.
    # Please use NACL::MTask::EventLogDetector
    $Log->warn( "FS::EA::EventlogTask library is deprecated."
            . " Please use NACL::MTask::EventLogDetector" );

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'messagename' => { type => SCALAR },
            'regex'       => { type => SCALAR, optional => 1 },
            'match'       => { type => SCALAR, optional => 1 },
            'init_time'   => { type => SCALAR, optional => 1 }
        },
        allow_extra => 1
    );

    my $match = delete( $opts{'match'} );
    my $regex = delete( $opts{'regex'} );

    my $elog_args = {
        command_interface => $self->{command_interface},
        filter            => { messagename => $opts{'messagename'} }
    };

    my $elog_obj   = NACL::C::Eventlog->find(%$elog_args);
    my $final_time = $elog_obj->state()->time();
    $final_time =~ s/"//g;
    $Log->comment( "Final time: " . $final_time );
    my $init_time = $opts{'init_time'} || $self->{time_stamp};
    $Log->comment( "Init time: " . $init_time );
    my $time_diff = $self->_get_time_diff(
        {   final_time => $final_time,
            init_time  => $init_time
        }
    );

    if (   $time_diff->{delta_year} != 0
        or $time_diff->{delta_month} != 0
        or $time_diff->{delta_day} != 0 )
    {
        $Log->comment("Time differnce is much higher than expected");
        return 0;
    } ## end if ( $time_diff->{delta_year...
    if (   $time_diff->{delta_hour} < 0
        or $time_diff->{delta_minute} < 0
        or $time_diff->{delta_second} < 0 )
    {
        $Log->comment("Time differnce is showing negative value");
        $Log->comment( "This log msg and all the mesgs below this log"
                . " will not be considered" );
        return 0;
    } ## end if ( $time_diff->{delta_hour...

    if ( $match == 1 ) {
        $Log->comment( "Event Details: " . $elog_obj->state()->event() );
        if ( $elog_obj->state()->event() !~ m[$regex] ) {
            $Log->comment("Regex mismatch");
            return 0;
        }
        return 1;
    } ## end if ( $match == 1 )

    return $elog_obj->state()->event();
} ## end sub capture_event_log

#####################################################################
#
#   function _get_time_diff :
#          Calculates the time difference to check logs are generated
#          during run time.
#
#   Status : Private
#
#####################################################################

sub _get_time_diff {
    my $self       = shift;
    my $time_calc  = shift;
    my @delta_time = ();
    my %time_diff  = ();

    foreach ( keys %$time_calc ) {
        $time_calc->{$_} =~ s/ /:/g;
        $time_calc->{$_} =~ s/\//:/g;
    }
    $Log->comment( "Initial Time: " . $time_calc->{init_time} );
    $Log->comment( "Final Time: " . $time_calc->{final_time} );

    my @init_time  = split( /:/, $time_calc->{init_time} );
    my @final_time = split( /:/, $time_calc->{final_time} );

    foreach ( 0 .. $#init_time ) {
        if ( $init_time[$_] !~ m[\d{2}] ) {
            $init_time[$_] = "0" . $init_time[$_];
        }
        if ( $final_time[$_] !~ m[\d{2}] ) {
            $final_time[$_] = "0" . $final_time[$_];
        }
    } ## end foreach ( 0 .. $#init_time )

    # $init_time[5] = Seconds
    # $init_time[4]= Minutes
    # $init_time[3] = Hours
    # $init_time[2] = Days
    # $init_time[1] = Months
    # $init_time[0] = Year

    @delta_time = Date::Calc::Delta_YMDHMS(
        $init_time[2],  $init_time[0],  $init_time[1],  $init_time[3],
        $init_time[4],  $init_time[5],
        $final_time[2], $final_time[0], $final_time[1], $final_time[3],
        $final_time[4], $final_time[5]

    );

    # 'delta_year' = $delta_time[0]'
    # 'delta_month'=  $delta_time[1],
    # 'delta_day'= $delta_time[2],
    # 'delta_hour'= $delta_time[3],
    # 'delta_minute'= $delta_time[4]
    # 'delta_second'= $delta_time[5]

    %time_diff = (
        'delta_year',   $delta_time[0], 'delta_month',  $delta_time[1],
        'delta_day',    $delta_time[2], 'delta_hour',   $delta_time[3],
        'delta_minute', $delta_time[4], 'delta_second', $delta_time[5]
    );
    return \%time_diff;
} ## end sub _get_time_diff

#####################################################################
#
#   function capture_all_event_logs :
#		Returns all log messages genrated during run time based on message id.
#
#   usage : $event_obj = EventlogTask->new(command_interface => $command_inter
#			face)
#			$event_obj->capture_all_event_logs($params);
#
#   Note:   This function is deprecated. Please use
#           NACL::MTask::EventLogDetector methods.
#
#####################################################################

sub capture_all_event_logs {

    # FS::EA::EventlogTask library is deprecated.
    # Please use NACL::MTask::EventLogDetector
    $Log->warn( "FS::EA::EventlogTask library is deprecated."
            . " Please use NACL::MTask::EventLogDetector" );

    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'messagename' => { type => SCALAR },
            'init_time'   => { type => SCALAR, optional => 1 }
        },
        allow_extra => 1
    );
    my $buffer = undef;

    my $elog_args = {
        command_interface => $self->{command_interface},
        filter            => { messagename => $opts{'messagename'} }
    };

    my @elog_obj = NACL::C::Eventlog->find(%$elog_args);

    foreach (@elog_obj) {
        my $final_time = $_->state()->time();
        $final_time =~ s/"//g;
        $Log->comment( "Final time: " . $final_time );
        my $init_time = $opts{'init_time'} || $self->{time_stamp};
        $Log->comment( "Init time: " . $init_time );
        my $time_diff = $self->_get_time_diff(
            {   final_time => $final_time,
                init_time  => $init_time
            }
        );

        if (   $time_diff->{delta_hour} < 0
            or $time_diff->{delta_minute} < 0
            or $time_diff->{delta_second} < 0 )
        {
            $Log->comment("Time differnce is showing negative value");
            $Log->comment( "This log msg and all the mesgs below this log"
                    . " will not be considered" );
            last;
        } ## end if ( $time_diff->{delta_hour...

        $buffer .= $_->state()->event() . "\n\n";
    } ## end foreach (@elog_obj)

    return $buffer;
} ## end sub capture_all_event_logs

1;
