# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/EA/SnapmirrorTask.pm#1 $
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##
##	    This Snapmirror task library accomodates all Snapmirror
##      related function that are reusable by team.
##
##      This library is developed by WAFL-EA team.
##
##	@description
## 	    This Library covers the following functions
##
##	@functions
##
##		create()
##		initialize()
##		initialize_ls_set()
##		resync()
##      break()
##      delete()
##      release()
##
##  @dependencies
##		use NACL::C::Snapmirror;
##		use Tharn qw($Log);
##		use Date::Calc qw(Delta_YMDHMS);
##		use NACL::APISet;
##	`	use Params::Validate
##		NACL::APISet::Exceptions::ResponseException qw(:try);
##
## 	@notes
##
##	@duration
##
##
## 	@author c-dl-rakesh-automation@netapp.com
##
## 	@status (private to WAFL-EA)
##

package SnapmirrorTask;
use FS::EA::Exceptions::WaflironException;
use NACL::C::Snapmirror;
use NACL::APISet;
use Tharn qw($Log);
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use strict;
use warnings;
use Data::Dumper;

#####################################################################
#
#   function new :
#       Constructor for package SnapmirrorTask.
#
#   usage : $vol_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#
#####################################################################

sub new {
    $Log->enter();
    my $pkg   = shift;
    my $class = ref($pkg) || $pkg;
    my %opts  = validate @_,
        { command_interface => { isa => 'NACL::C::CommandInterface' } };
    my $self = {%opts};
    bless $self, $class;
    $Log->exit();

    # Return the blessed object
    return $self;
} ## end sub new

#####################################################################
#
#   function create :
#       creates a snapmirror.Waits until it is initialized
#
#   usage : $snapm_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#			$snapm_obj->create($params);
#
#
#####################################################################

sub create {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'source-cluster'      => { type => SCALAR },
            'source-vserver'      => { type => SCALAR },
            'source-volume'       => { type => SCALAR },
            'destination-cluster' => { type => SCALAR },
            'destination-vserver' => { type => SCALAR },
            'destination-volume'  => { type => SCALAR },
            'wait_time'           => { type => SCALAR, default => 300 }
        },
        allow_extra => 1
    );

    my $wait_time = delete( $opts{'wait_time'} );
    my $endtime   = time() + $wait_time;

    # Create snapmirror
    $self->{'snapm_obj'} = NACL::C::Snapmirror->create(
        'command_interface' => $self->{command_interface},
        %opts
    );
    while (
        $self->{'snapm_obj'}->state()->state() !~ m(Uninitialized|Unknown|Broken-Off)i )
    {
        Tharn::sleep 5;
        if ( time() > $endtime ) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "Fail: Snapmirror creation failed" );
        }    # if ( !$time_limit) ends here
    }    # while ($self->{'snapm_obj'}..) ends here

    return 1;
} ## end sub create

#####################################################################
#
#   function delete :
#       deletes a snapmirror.
#
#   usage : $snapm_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#			$snapm_obj->delete($params);
#
#
#####################################################################

sub delete {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'source-cluster'      => { type => SCALAR, optional => 1 },
            'source-vserver'      => { type => SCALAR, optional => 1 },
            'source-volume'       => { type => SCALAR, optional => 1 },
            'destination-cluster' => { type => SCALAR, optional => 1 },
            'destination-vserver' => { type => SCALAR, optional => 1 },
            'destination-volume'  => { type => SCALAR, optional => 1 },
            'wait_time'           => { type => SCALAR, default  => 500 }
        },
        allow_extra => 1
    );

    my $snapm_args = $self->_get_snapmirror_args(%opts);
    NACL::C::Snapmirror->delete(%$snapm_args);
    return 1;
} ## end sub delete


#####################################################################
#
#   function release :
#       releases a snapmirror.
#
#   usage : $snapm_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#			$snapm_obj->release($params);
#
#
#####################################################################

sub release {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'source-vserver'      => { type => SCALAR, optional => 1 },
            'source-volume'       => { type => SCALAR, optional => 1 },
            'destination-vserver' => { type => SCALAR, optional => 1 },
            'destination-volume'  => { type => SCALAR, optional => 1 },
            'wait_time'           => { type => SCALAR, default  => 500 }
        },
        allow_extra => 1
    );

    #the destination-cluster or source-cluster are not passed into
    #the snapmirror release command
    my $snapm_args = $self->_get_snapmirror_args(%opts);
    delete $snapm_args->{'destination-cluster'};
    delete $snapm_args->{'source-cluster'};

    NACL::C::Snapmirror->release(%$snapm_args);

    return 1;
} ## end sub release
#####################################################################
#
#   function initialize :
#       Initialize a broken snapmirror.Waits until it is initialized
#
#   usage : $snapm_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#			$snapm_obj->initialize($params);
#
#
#####################################################################

sub initialize {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'source-cluster'      => { type => SCALAR, optional => 1 },
            'source-vserver'      => { type => SCALAR, optional => 1 },
            'source-volume'       => { type => SCALAR, optional => 1 },
            'destination-cluster' => { type => SCALAR, optional => 1 },
            'destination-vserver' => { type => SCALAR, optional => 1 },
            'destination-volume'  => { type => SCALAR, optional => 1 },
            'wait_time'           => { type => SCALAR, default  => 1200 }
        },
        allow_extra => 1
    );

    my $snapm_args = $self->_get_snapmirror_args(%opts);

    NACL::C::Snapmirror->initialize(%$snapm_args);

    my %find_hash =  ( 'destination-vserver' => $snapm_args->{'destination-vserver'},
                       'destination-volume'  => $snapm_args->{'destination-volume'} );
    if ($snapm_args->{'destination-cluster'}) {
        $find_hash{'destination-cluster'} = $snapm_args->{'destination-cluster'};
    }

    my $snapm_obj = NACL::C::Snapmirror->find(
        command_interface => $self->{command_interface},
        filter            => \%find_hash,
    );

    $Log->comment( "This is the wait time " . $opts{'wait_time'} );
    my $end_time   = time() + $opts{'wait_time'};
    my $snap_state = $snapm_obj->state();

    while ($snap_state->state() !~ m[Snapmirrored]
        || $snap_state->status() !~ m[Idle] )
    {
        Tharn::sleep 5;
        $snap_state = $snapm_obj->state();
        if ( time() > $end_time ) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "Snapmirror failed to come to state Snapmirrored" );
        }
    }    # while ( $time_limit ) ends here

    return 1;
} ## end sub initialize

#####################################################################
#
#   function initialize_ls_set :
#       Initialize a broken snapmirror of type LS.Waits until it is
#       initialized
#
#   usage : $snapm_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#			$snapm_obj->initialize_ls_set($params);
#
#
#####################################################################

sub initialize_ls_set {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'destination-cluster' => { type => SCALAR, optional => 1 },
            'destination-vserver' => { type => SCALAR, optional => 1 },
            'destination-volume'  => { type => SCALAR, optional => 1 },
            'source-cluster'      => { type => SCALAR, optional => 1 },
            'source-vserver'      => { type => SCALAR, optional => 1 },
            'source-volume'       => { type => SCALAR, optional => 1 },
            'wait_time'           => { type => SCALAR, default  => 1500 }
        },
        allow_extra => 1
    );

    my $source_cluster = $opts{'source-cluster'}
        || $self->{snapm_obj}->state()->source_cluster();
    my $source_vserver = $opts{'source-vserver'}
        || $self->{snapm_obj}->state()->source_vserver();
    my $source_volume = $opts{'source-volume'}
        || $self->{snapm_obj}->state()->source_volume();
    my $destination_cluster = $opts{'destination-cluster'}
        || $self->{snapm_obj}->state()->destination_cluster();
    my $destination_vserver = $opts{'destination-vserver'}
        || $self->{snapm_obj}->state()->destination_vserver();
    my $destination_volume = $opts{'destination-volume'}
        || $self->{snapm_obj}->state()->destination_volume();

    my $snapm_args = {
        command_interface => $self->{command_interface},
        'source-cluster'  => $source_cluster,
        'source-vserver'  => $source_vserver,
        'source-volume'   => $source_volume
    };

    NACL::C::Snapmirror->initialize_ls_set(%$snapm_args);

    my $snapm_obj = NACL::C::Snapmirror->find(
        command_interface => $self->{command_interface},
        filter            => {
            'destination-cluster' => $destination_cluster,
            'destination-vserver' => $destination_vserver,
            'destination-volume'  => $destination_volume
        }
    );
    my $end_time   = time() + $opts{'wait_time'};
    my $snap_state = $snapm_obj->state();

    while ($snap_state->state() !~ m[Snapmirrored]
        || $snap_state->status() !~ m[Idle] )
    {
        Tharn::sleep 5;
        $snap_state = $snapm_obj->state();
        if ( time() > $end_time ) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "Snapmirror failed to come to state Snapmirrored" );
        }
    }    # while ( $time_limit ) ends here
    return 1;
} ## end sub initialize_ls_set

#####################################################################
#
#   function break :
#       break a snapmirror. wait until its status becomes 'broken-off'
#
#   usage : $snapm_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#			$snapm_obj->break($params);
#
#
#####################################################################

sub break {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'source-cluster'      => { type => SCALAR, optional => 1 },
            'source-vserver'      => { type => SCALAR, optional => 1 },
            'source-volume'       => { type => SCALAR, optional => 1 },
            'destination-cluster' => { type => SCALAR, optional => 1 },
            'destination-vserver' => { type => SCALAR, optional => 1 },
            'destination-volume'  => { type => SCALAR, optional => 1 },
            'wait_time'           => { type => SCALAR, default  => 500 }
        },
        allow_extra => 1
    );

    my $snapm_args = $self->_get_snapmirror_args(%opts);
    NACL::C::Snapmirror->break(%$snapm_args);
    my $snapm_obj = NACL::C::Snapmirror->find(
        command_interface => $self->{command_interface},
        filter            => {
            'destination-cluster' => $snapm_args->{'destination-cluster'},
            'destination-vserver' => $snapm_args->{'destination-vserver'},
            'destination-volume'  => $snapm_args->{'destination-volume'}
        }
    );
    my $end_time   = time() + $opts{'wait_time'};
    my $snap_state = $snapm_obj->state();

    while ($snap_state->state() !~ m[Broken-off]
        || $snap_state->status() !~ m[Idle|-] )
    {

        Tharn::sleep 5;
        $snap_state = $snapm_obj->state();
        if ( time() > $end_time ) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "Snapmirror failed to come to state Broken-off" );
        }
    }    # while ( $time_limit ) ends here

    return 1;
} ## end sub break

#####################################################################
#
#   function resync :
#       Resync a snapmirror. wait until it is resynced
#
#   usage : $snapm_obj = SnapmirrorTask->new(command_interface => $command_inter
#			face)
#			$snapm_obj->resync($params);
#
#
#####################################################################

sub resync {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'source-cluster'      => { type => SCALAR, optional => 1 },
            'source-vserver'      => { type => SCALAR, optional => 1 },
            'source-volume'       => { type => SCALAR, optional => 1 },
            'destination-cluster' => { type => SCALAR, optional => 1 },
            'destination-vserver' => { type => SCALAR, optional => 1 },
            'destination-volume'  => { type => SCALAR, optional => 1 },
            'wait_time'           => { type => SCALAR, default  => 500 }
        },
        allow_extra => 1
    );

    my $snapm_args = $self->_get_snapmirror_args(%opts);
    NACL::C::Snapmirror->resync(%$snapm_args);
    my $snapm_obj = NACL::C::Snapmirror->find(
        command_interface => $self->{command_interface},
        filter            => {
            'destination-cluster' => $snapm_args->{'destination-cluster'},
            'destination-vserver' => $snapm_args->{'destination-vserver'},
            'destination-volume'  => $snapm_args->{'destination-volume'}
        }
    );
    my $end_time   = time() + $opts{'wait_time'};
    my $snap_state = $snapm_obj->state();

    while ($snap_state->state() !~ m[Snapmirrored]
        || $snap_state->status() !~ m[Idle] )
    {

        Tharn::sleep 5;
        $snap_state = $snapm_obj->state();
        if ( time() > $end_time ) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "Snapmirror failed to come to state Snapmirrored" );
        }
    }    # while ( $time_limit ) ends here

    return 1;
} ## end sub resync

############################################################################
## Name: wait_for_snapmirror_to_finish()
##
## Description:
## Method to wait for the Snapmirror status - "Snapmirrored".
## Waits still the snapmirror to finish. If snapmirror doesn't finishes
## within the given wait_time, throws as exception.
##
## Synopsis:
##         wait_for_snapmirror_to_finish( <source_path>, <wait_time>);
##
## Arguments:
##           <destination_path>   : Destination path as given for the snapmirror
##                                  initialize.
##           <wait_time>          : Wait counter for expected status
##                                  Wait cycle = <wait_time> * 5 Secs
##
#############################################################################

sub wait_for_snapmirror_to_finish {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'destination-path' => { type => SCALAR, optional => 1 },
            'wait_time'        => { type => SCALAR, default  => 900 }
        },
        allow_extra => 1
    );

    my $destination_path = $opts{'destination-path'}
        || $self->{snapm_obj}->state()->destination_path();
    my $wait_time = $opts{'wait_time'};

    my $snapmirror_obj = NACL::C::Snapmirror->find(
        command_interface => $self->{command_interface},
        filter            => { 'destination-path' => $destination_path }
    );

    # ->state();

    my $endtime = time() + $wait_time;
    my $snapmirror_status;
    while ( time() < $endtime ) {
        $snapmirror_status = $snapmirror_obj->state();

        if (   $snapmirror_status->state() =~ /Snapmirrored/
            && $snapmirror_status->status() =~ /Idle/ )
        {
            last;
        }
        Tharn::sleep 5;
    } ## end while ( time() < $endtime)
    if ( $snapmirror_status->state() !~ /Snapmirrored/ ) {
        FS::EA::Exceptions::WaflironException->throw( text =>
                  "Snapmirror status is not \"Snapmirrored\", even after the"
                . " given timeout" );
    }
} ## end sub wait_for_snapmirror_to_finish

############################################################################
## Name: _get_snapmirror_args()
##
## Description:
##         Creates a hashref of required params for different snapmirror
##         operations
##
## Synopsis:
##         _get_snapmirror_args( %opts );
##
#############################################################################

sub _get_snapmirror_args {
    my ( $self, %opts ) = @_;
    my $source_cluster = $opts{'source-cluster'}
        || $self->{snapm_obj}->state()->source_cluster();
    my $source_vserver = $opts{'source-vserver'}
        || $self->{snapm_obj}->state()->source_vserver();
    my $source_volume = $opts{'source-volume'}
        || $self->{snapm_obj}->state()->source_volume();
    my $destination_cluster = $opts{'destination-cluster'}
        || $self->{snapm_obj}->state()->destination_cluster();
    my $destination_vserver = $opts{'destination-vserver'}
        || $self->{snapm_obj}->state()->destination_vserver();
    my $destination_volume = $opts{'destination-volume'}
        || $self->{snapm_obj}->state()->destination_volume();

    my $snapm_args = {
        command_interface     => $self->{command_interface},
        'source-cluster'      => $source_cluster,
        'source-vserver'      => $source_vserver,
        'source-volume'       => $source_volume,
        'destination-cluster' => $destination_cluster,
        'destination-vserver' => $destination_vserver,
        'destination-volume'  => $destination_volume
    };

    delete $snapm_args->{'destination-cluster'} if ($destination_cluster eq "-");

    return $snapm_args;
} ## end sub _get_snapmirror_args

1;
