# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/EA/VolumeTask.pm#1 $
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##
##	    This Aggregate task library accomodates all Aggregate
##      related function that are reusable by team.
##
##      This library is developed by WAFL-EA team.
##
##	@description
## 	    This Library covers the following functions
##
##	@functions
##
##		 sub check_all_volume_member_state ()
##		 get_volume_member_status ()
##		 check_volume_member_state ()
##		 exists ()
##		 explore ()
##		 check_available_space ()
##		 _convert_size ()
##		 _round ()
##		 vol_exists ()
##		 create ()
##		 create_snapshot ()
##		 delete_snapshot ()
##		 create_qtree ()
##		 is_root_volume ()
##		 delete_flexvol ()
##		 delete_striped ()
##		 delete_all_volumes ()
##		 set_vol_obj ()
##		 check_all_vol_member_online ()
##		 get_vol_obj ()
##		 check_vol_attributes ()
##		 get_vol_members ()
##		 check_state ()
##		 gridiron_state ()
##		 check_gridiron_state ()
##		 write_file ()
##		 delete_file ()
##		 get_file_content ()
##		 get_all_files_in_snapshot ()
##		 get_all_snapshots_of_volume ()
##		 get_all_files ()
##               volume_move ()
##
## 	@dependencies
##		NACL::C::Volume;
##		NACL::C::Snapshot;
##		EventlogTask;
##		Data::Dumper;
##		NACL::APISet::Exceptions::ResponseException qw(:try);
##
## 	@notes
##
##	@duration
##
##
## 	@author c-dl-rakesh-automation@netapp.com
##
## 	@status (private to WAFL-EA)
##

package VolumeTask;
use base qw(FS::EA::Exceptions::WaflironException);
use NACL::C::Volume;
use NACL::C::Snapshot;
use NACL::C::SnapshotPolicy;
use Carp qw(confess);
use Tharn qw($Log host);
use Data::Dumper;
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use strict;
use warnings;
use Data::Dumper;

#####################################################################
#
#   function new :
#   	Constructor for package VolumeTask.
#
#   usage : $vol_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#
#####################################################################

sub new {
    $Log->enter();
    my $pkg   = shift;
    my $class = ref($pkg) || $pkg;
    my %opts  = validate @_,
        { command_interface => { isa => 'NACL::C::CommandInterface' }, };

    my $self = {@_};
    bless $self, $class;
    $Log->exit();

    # Return the blessed object
    return $self;
} ## end sub new

#####################################################################
#
#   function check_all_volume_member_state :
#   	Check all the memeber volume state for a given volume
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->check_all_volume_member_state($params);
#
#   input params : attribute => Hashref of all values to verify
#                               each member volume.
#
#   returns :  Boolean
#
#####################################################################

sub check_all_volume_member_state {
    my $self = shift;
    my %opts = @_;
    my $member_state;
    $Log->enter();
    %opts = validate_with(
        params      => \%opts,
        spec        => { 'attributes' => { type => HASHREF } },
        allow_extra => 1
    );
    if ( !exists( $self->{'vol_obj'} ) ) {
        $self->{'vol_obj'} = NACL::C::Volume->find(
            command_interface => $self->{command_interface},
            filter => { volume => $opts{volume}, vserver => $opts{vserver} }
        );
    }

    my @members_status = $self->get_volume_member_status();
    my $status         = 0;
    foreach my $member_state (@members_status) {
        $status = $status
            || $self->check_volume_member_state(
            'vol_member_name' => $member_state->volume(),
            attributes        => $opts{attributes}
            );
    }
    return $status;
} ## end sub check_all_volume_member_state

#####################################################################
#
#   function get_volume_member_status :
#   	Gets all volume member status ( state objects)
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->get_volume_member_status($params);
#
#    return : Array of object.
#
#####################################################################

sub get_volume_member_status {
    my $self = shift;
    my %opts = @_;
    if ( !exists( $self->{'vol_obj'} ) ) {
        $self->{'vol_obj'} = NACL::C::Volume->find(
            command_interface => $self->{command_interface},
            filter => { volume => $opts{volume}, vserver => $opts{vserver} }
        );
    }
    my @vol_member_obj = NACL::C::VolumeMember->find(
        command_interface => $self->{command_interface} );
    my $i = 0;
    my $j = 0;
    my @vol_member_status;
    foreach my $vol_member_obj (@vol_member_obj) {
        $vol_member_status[$i] = $vol_member_obj->state();
        if ($vol_member_status[$i]->{volume} eq $self->{'vol_obj'}->volume() )
        {
            $j = $i;
            last;
        }
        $i++;
    }
    return wantarray ? @vol_member_status : $vol_member_status[$j];
} ## end sub get_volume_member_status

#####################################################################
#
#   function check_volume_member_state :
#   	checks for a single volume member all the user provided
#       attributes are macthing. If any mismatch report error
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->check_volume_member_state($params);
#
#    return : Boolean.
#             Updates error_message field if mismatch found.
#
#####################################################################

sub check_volume_member_state {
    my $self = shift;
    my %opts = @_;
    %opts = validate_with(
        params => \%opts,
        spec   => {
            'vol_member_name' => { type => SCALAR },
            attributes        => { type => HASHREF },
        },
        allow_extra => 1
    );

    my $vol_member_obj = NACL::C::VolumeMember->find(
        command_interface => $self->{command_interface},
        filter            => { volume => $opts{'vol_member_name'} }
    );
    my $vol_member_status = $vol_member_obj->state();
    my $mis_match_degree  = 0;
    my $error_message     = "";
    foreach my $key ( keys( %{ $opts{attributes} } ) ) {
        my $key1 = $key;
        $key1 =~ s/-/_/g;
        if ( $vol_member_status->$key1() ne $opts{'attributes'}->{$key} ) {
            $mis_match_degree++;
            $error_message
                = $error_message
                . " Actual Val : ["
                . $vol_member_status->$key1()
                . "] mismatch-with Expected Val : [$opts{'attributes'}->{$key}] \n";
            $self->{'vol_obj'}->{error_message} = $error_message;
        }
    } ## end foreach my $key (keys(%{$opts...
    if ($mis_match_degree) {
        return 0;
    }
    return 1;

} ## end sub check_volume_member_state

#####################################################################
#
#   function explore :
#         runs volume explore command. Returns a arrayref or reference of
#         volume dump command
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->explore($params);
#
#    return : Returns a arrayref or reference of
#             volume dump command.
#
#####################################################################

sub explore {
    my $self = shift;
    my $i    = 0;
    my ( $file_name, $volume, $inum );
    my %opts = validate_with(
        params => \@_,
        spec   => {
            format => { type => SCALAR },
            name   => { type => SCALAR, optional => 1 },
            filer  => { type => SCALAR },
        },
        allow_extra => 1
    );
    if ( exists $opts{name} ) {
        $file_name = delete $opts{name};
    }
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $volume = $opts{volume} || $self->{'vol_obj'}->volume();
    my $Api_set_obj = NACL::APISet->new(
        name      => $opts{filer},
        category  => "Node",
        interface => "CLI",
        set       => 'CMode',
        connid    => 'ssh'
    );
    if ( exists $opts{volume} ) {
        delete $opts{volume};
    }
    if ( exists $opts{vserver} ) {
        delete $opts{vserver};
    }
    if ( exists $opts{filer} ) {
        delete $opts{filer};
    }
    if ( exists $opts{inum} ) {
        $inum = delete $opts{inum};
    }
    else {
        $inum = "";
    }
    if ( !exists( $opts{'dump'} ) ) {
        $opts{'dump'} = "all";
    }
    my $output;
    my $response;
    try {
        $response = $Api_set_obj->volume_explore( %opts,
            'pass_through' => "$volume.$inum/" );
        $output = $response->get_raw_output();
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->comment( "Task ERROR  is: " . $exception_object->text() );
    };
    if ( $output !~ /\w+=.*/ ) {
        return $output;
    }
    my @space_split = split( /\n/s, $output );
    my %hash;
    $i = 0;
    my @array;
    my $j         = 0;
    my $next_hash = 0;
    my %new_hash;
    my $last_key;

    foreach my $line (@space_split) {
        if ( $line =~ /(\w+)=(.*)/ ) {
            $hash{$1} = $2;
            $last_key = $1;
        }
        else {
            %new_hash = %hash;
            $j        = 0;
            next;
        }
        if ( exists( $new_hash{$last_key} ) ) {
            $next_hash = 1;
            $j++;
        }

        if ( ( $next_hash == 1 ) and ( $j == 1 ) ) {
            my %hash_temp = %new_hash;
            $array[$i] = \%hash_temp;
            $i++;
            $next_hash = 0;
        }
    } ## end foreach my $line (@space_split)
    $i = 0;
    if ( defined $file_name ) {
        foreach my $vol_expl_output (@array) {
            if ( $vol_expl_output->{name} eq $file_name ) {
                last;
            }
            $i++;
        }
    }
    return wantarray ? @array : $array[$i];
} ## end sub explore

#####################################################################
#
#   function check_available_space :
#          check available space for creating a volume
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->check_available_space($params);
#
#####################################################################

sub check_available_space {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            vol_size  => { type => SCALAR },
            aggregate => { type => SCALAR },
        },
        allow_extra => 0
    );
    my $aggr_obj = NACL::C::Aggregate->find(
        command_interface => $self->{command_interface},
        filter            => { aggregate => $opts{aggregate} }
    );
    my $aggr_status = $aggr_obj->state();
    $Log->debug("================= $opts{vol_size} ================");
    my $convertion = $self->_convert_size( $opts{vol_size} );
    $Log->debug(
        "================= " . $convertion->{bytes} . "===================" );
    if ( $aggr_status->availsize() < $convertion->{bytes} ) {
        NACL::APISet::Exceptions::ResponseException->throw( text =>
                "Aggregate $opts{aggregate} doesn't contain available space of [$opts{vol_size}]"
        );
    }
} ## end sub check_available_space

#####################################################################
#
#   function _convert_size :
#          converts a give value into bytes, Kb, Mb, Gb, Tb
#
#   usage :
#			$self->_convert_size($params);
#
#   Status : Private
#
#####################################################################

sub _convert_size {
    my $self = shift;
    my $size = shift;
    my ( %size, $bytes );
    my %hash;
    $bytes = $size;
    if ( $size =~ /(\d+\.{0,1}\d{0,})m/i ) {
        $bytes = $1 * 1048576;
    }
    elsif ( $size =~ /(\d+\.{0,1}\d{0,})g/i ) {
        $bytes = $1 * 1073741824;
    }
    elsif ( $size =~ /(\d+\.{0,1}\d{0,})k/i ) {
        $bytes = $1 * 1024;
    }
    elsif ( $size =~ /(\d+\.{0,1}\d{0,})t/i ) {
        $bytes = $1 * 1099511627776;
    }
    map { $hash{$_} = _round($bytes) and $bytes /= 1024 }
        qw(bytes Kb Mb Gb Tb);
    return \%hash;
} ## end sub _convert_size

#####################################################################
#
#   function _round :
#          rounds decimal no to nearest integer value
#
#   usage :
#			$self->_round($params);
#
#   Status : Private
#
#####################################################################

sub _round {
    my $number = shift;
    return int( $number + .05 * ( $number <=> 0 ) );
}

#####################################################################
#
#   function vol_exists :
#          checks if a volume exists.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->vol_exists($params);
#
#    returns : boolean
#
#####################################################################

sub vol_exists {
    my $self = shift;
    my ( $spec, %args );
    my $mode = $self->{'command_interface'}->mode();
    if ( $mode =~ /CMode/ ) {
        $spec = {
            volume  => { type => SCALAR },
            vserver => { type => SCALAR },
        };
    }
    else {
        delete $args{'vserver'};
        $spec = { volume => { type => SCALAR }, vserver => { type => SCALAR, optional => 1 } };
    }
    my %opts = validate @_, $spec;
    delete $opts{'vserver'} if ($mode =~ /7Mode/);

    my @vols = NACL::C::Volume->find(
        command_interface => $self->{command_interface} );
    if ( $mode =~ /CMode/ ) {
        %args = (
            volume  => $opts{volume},
            vserver => $opts{vserver}
        );
    }
    else {
        %args = ( volume => $opts{volume}, );
    }

    foreach my $vol (@vols) {
        if ( $vol->volume() eq $opts{volume} ) {
            $self->set_vol_obj(%args);
            return 1;
        }
    }
    return 0;
} ## end sub vol_exists
#####################################################################
#
#   function create :
#          creates a volume and waints until it comes online.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->create($params);
#
#   Staus : depricated
#
#####################################################################

sub create {
    my $self = shift;
    $Log->comment("This function is depricated");
    $self->create_wait(@_);
}

#####################################################################
#
#   function create_wait :
#          creates a volume and waints until it comes online.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->create_wait($params);
#
#   Staus : Functional
#
#####################################################################

sub create_wait {
    my $self = shift;
    my %args;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            volume      => { type => SCALAR },
            vserver     => { type => SCALAR, optional => 1 },
            'wait_time' => { type => SCALAR, default => 300 }
        },
        allow_extra => 1
    );
    my $mode = $self->{'command_interface'}->mode();
    if ( $mode =~ /CMode/ ) {
        %args = (
            volume  => $opts{volume},
            vserver => $opts{vserver}
        );
    }
    else {
        %args = ( volume => $opts{volume}, );
    }

    my $wait_time = delete( $opts{'wait_time'} );
    if ( !$self->vol_exists(%args) ) {
        $self->{'vol_obj'} = NACL::C::Volume->create(
            'command_interface' => $self->{command_interface},

            #			'connectrec-timeout' => 600,
            'foreground' => 'true',
            %opts
        );


        while ( $self->{'vol_obj'}->state()->state() ne "online" ) {
            if ( $wait_time <= 0 ) {
                NACL::APISet::Exceptions::ResponseException->throw(
                    text => "Volume $opts{volume} failed to be Online" );
            }
            Tharn::sleep 5;
            $wait_time -= 5;
        }
    } ## end if (!$self->vol_exists...
} ## end sub create_wait

#####################################################################
#
#   function snap_disable_purge :
#          Disables and purges any snapshots on a given volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->snap_disable_purge($params);
#
#   Status : Functional
#
#   DO NOT USE!!! This lib will move once BURT 508215 is resolved.
#   contact: dl-space-qa
#
#####################################################################

sub snap_disable_purge {
    my $self = shift;
    my %opts = validate_with(
        params => \@_,
        spec   => {
            volume      => { type => SCALAR },
        },
        allow_extra => 1
    );
    my $mode = $self->{'command_interface'}->mode();

    # Disable the auto creation of snapshot
    # 7-Mode volume needs to exist before this can be modified.
    # Need to handle case for BURT 505883 as there is a timing
    # reliability issue with creating volumes and disabling snapshots
    # afterwards.
    if ( $mode eq "CMode" ) {
        NACL::C::SnapshotPolicy->modify(
            'command_interface' => $self->{command_interface},
            policy              => "default",
            enabled             => "false"
        );
    } else {
        NACL::C::Volume->modify(
            'command_interface' => $self->{command_interface},
            volume              => $opts{volume},
            'snapshot-policy'   => "none"
        );
    } ## end if/else

    eval {
        NACL::C::Snapshot->delete(
            'command_interface' => $self->{command_interface},
            'volume'            => $opts{'volume'},
            'snapshot'          => '*',
            'all'               => 1,
            'force'             => 'true'
        );
    }; ## end eval
} ## end sub snap_disable_purge


#####################################################################
#
#   function create_snapshot :
#          creates snapshot on volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->create_snapshot($params);
#
#####################################################################

sub create_snapshot {
    my $self = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => { snapshot => { type => SCALAR }, },
        allow_extra => 1
    );
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $opts{volume}  = $opts{volume}  || $self->{'vol_obj'}->volume();
    $opts{vserver} = $opts{vserver} || $self->{'vol_obj'}->vserver();
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $opts{command_interface} = $opts{command_interface}
        || $self->{'command_interface'};
    NACL::C::Snapshot->create(%opts);
} ## end sub create_snapshot

#####################################################################
#
#   function delete_snapshot :
#          creates snapshot on volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->delete_snapshot($params);
#
#
#####################################################################

sub delete_snapshot {
    my $self = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => { snapshot => { type => SCALAR }, },
        allow_extra => 1
    );
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $opts{volume}  = $opts{volume}  || $self->{'vol_obj'}->volume();
    $opts{vserver} = $opts{vserver} || $self->{'vol_obj'}->vserver();
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $opts{command_interface} = $opts{command_interface}
        || $self->{'command_interface'};
    NACL::C::Snapshot->delete(%opts);
} ## end sub delete_snapshot

#####################################################################
#
#   function create_qtree :
#          creates qtree on volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->create_qtree($params);
#
#
#####################################################################

sub create_qtree {
    my $self = shift;
    my %opts = validate_with(
        params      => \@_,
        spec        => { qtree => { type => SCALAR }, },
        allow_extra => 1
    );
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $opts{command_interface} = $opts{command_interface}
        || $self->{'command_interface'};
    $opts{volume}  = $opts{volume}  || $self->{'vol_obj'}->volume();
    $opts{vserver} = $opts{vserver} || $self->{'vol_obj'}->vserver();
    NACL::C::Qtree->create(%opts);
} ## end sub create_qtree

#####################################################################
#
#   function delete_flexvol :
#          deletes a flex volume
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->delete_flexvol($params);
#
#   status : depricated
#
#####################################################################

sub delete_flexvol {
    my $self = shift;
    $self->purge_flexvol(@_);
}

#####################################################################
#
#   function purge_flexvol :
#          deletes a flex volume
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->purge_flexvol($params);
#
#
#####################################################################

sub purge_flexvol {

    my $self = shift;
    my %opts = @_;
    my $mode = $self->{'command_interface'}->mode();
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) && $mode =~ /CMode/ );
    $self->set_vol_obj( volume => $opts{volume} )
        if ( !exists( $self->{'vol_obj'} ) && $mode =~ /7Mode/ );

    my %args;
    if ( $mode =~ /CMode/ ) {
        %args = (
            volume  => $opts{volume} || $self->{'vol_obj'}->{'volume'},
            vserver => $opts{vserver} || $self->{'vol_obj'}->{'vserver'}
        );
    }
    else {
        %args = ( volume => $opts{volume} || $self->{'vol_obj'}->{'volume'},);
    }
    if ( $self->vol_exists(%args) ) {
        $self->{'vol_obj'}->unmount( 'force' => 'true' )
            if ( $mode =~ /CMode/ );
        $self->{'vol_obj'}->modify( state => 'offline' );
        $self->{'vol_obj'}->delete();
    }

} ## end sub purge_flexvol

#####################################################################
#
#   function delete_striped :
#          deletes a stripped volume
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->delete_striped($params);
#
#   status : deprecated
#
#####################################################################

sub delete_striped {
    my $self = shift;
    my %opts = @_;

    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $self->{'vol_obj'}->unmount( 'force' => 'true' );

    $self->{'vol_obj'}->modify( state => 'offline' );
    my $vol_status    = $self->{'vol_obj'}->state();
    my $state_offline = 0;
    if ( $vol_status->volume_style() eq "striped" ) {
        Tharn::sleep(30);
        my $timeout = 100;
        while ($state_offline) {
            if ($self->check_all_volume_member_state(
                    attributes => { state => 'offline' }
                )
                )
            {
                $state_offline = 1;
            }
            Tharn::sleep(5);
            $timeout--;
            if ( $timeout == 0 ) {
                NACL::APISet::Exceptions::ResponseException->throw(
                    text => "all Volume members not set to state=[offline]" );
            }
        } ## end while ($state_offline)
    } ## end if ($vol_status->volume_style...

    $self->{'vol_obj'}->delete();

} ## end sub delete_striped

#####################################################################
#
#   function purge_striped :
#          deletes a striped volume
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->purge_striped($params);
#
#
#####################################################################

sub purge_striped {
    my $self = shift;
    $self->delete_striped(@_);
}

#####################################################################
#
#   function delete_all_volumes :
#          deletes all volumes under a vserver
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->delete_all_volumes($params);
#
#
#####################################################################

sub delete_all_volumes {
    my $self = shift;
    my %opts = @_;
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    my @vol_objs = NACL::C::Volume->find(
        command_interface => $self->{command_interface},
        filter            => { vserver => $self->{'vol_obj'}->vserver() }
    );
    foreach my $vol_obj (@vol_objs) {
        $self->delete(
            vserver => $self->{'vol_obj'}->vserver(),
            volume  => $vol_obj->volume()
        );
    }
} ## end sub delete_all_volumes

sub set_vol_obj {
    my $self = shift;
    my %opts = @_;
    my ( $filter, $spec );
    my $mode = $self->{'command_interface'}->mode();
    if ( $mode =~ /CMode/ ) {
        $spec = {
            volume  => { type => SCALAR },
            vserver => { type => SCALAR, optional => 1 },
        };
    }
    else {
        $spec = { volume => { type => SCALAR }, };
    }
    %opts = validate_with(
        params      => \%opts,
        spec        => $spec,
        allow_extra => 1
    );
    if ( $mode =~ /CMode/ ) {
        $filter = { vserver => $opts{vserver}, volume => $opts{volume} };
    }
    else {
        $filter = { volume => $opts{volume} };
    }

    $self->{'vol_obj'} = NACL::C::Volume->find(
        command_interface => $self->{command_interface},
        filter            => $filter
    );
} ## end sub set_vol_obj
#####################################################################
#
#   function check_all_vol_member_online :
#          checks whether all member volumes associated to volume is
#          online.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->delete_all_volumes($params);
#
#
#####################################################################

sub check_all_vol_member_online {
    my $self = shift;
    my %opts = @_;
    $Log->enter();

    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );

    my $command_interface = $self->{'command_interface'};

    my ( @mem_objects, @all_members );
    my $pattern = $self->{'vol_obj'}->volume() . '_' . '*';

    @all_members = NACL::C::VolumeMember->find(
        command_interface => $self->{'command_interface'},
        filter            => {

            #'volume'  => $self->{'vol_obj'}->volume() . '*',
            'vserver' => $self->{'vol_obj'}->vserver()
        }
    );

    foreach (@all_members) {
        if ( $_->state->volume() =~ /$pattern/ ) {
            push( @mem_objects, $_ );
        }
    }

    my $mis_match_degree = 0;
    my $error_message;

    foreach my $vol_mem (@mem_objects) {
        my $state = $vol_mem->state()->state();
        if ( $state ne "online" ) {
            $mis_match_degree++;
            $error_message = $error_message
                . "$vol_mem->state()->volume() is in the state: [$state]\n";
        }
    }
    if ($mis_match_degree) {
        NACL::APISet::Exceptions::ResponseException->throw(
            text => $error_message );
    }
    $Log->exit();
} ## end sub check_all_vol_member_online

sub get_vol_obj {
    my $self = shift;
    return $self->{'vol_obj'};
}

#####################################################################
#
#   function check_vol_attributes :
#          Verify all volume attribute(vol show). throws exception
#          on mismatch.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->check_vol_attributes($params);
#
#
#####################################################################

sub check_vol_attributes {
    my $self = shift;
    my %opts = @_;
    %opts = validate_with(
        params      => \%opts,
        spec        => { attributes => { type => HASHREF } },
        allow_extra => 1
    );
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    my $vol_status       = $self->{'vol_obj'}->state();
    my $mis_match_degree = 0;
    my $error_message    = undef;
    foreach my $key ( keys( %{ $opts{'attributes'} } ) ) {
        my $key1 = $key;
        $key1 =~ s/-/_/g;
        if ( $vol_status->$key1() ne $opts{'attributes'}->{$key} ) {
            $mis_match_degree++;
            $error_message
                = $error_message
                . " Actual Val : [$vol_status->$key1()] mismatch-with "
                . "Expected Val : [$opts{'attributes'}->{$key}] \n";
        }
    } ## end foreach my $key (keys(%{$opts...
    if ($mis_match_degree) {
        NACL::APISet::Exceptions::ResponseException->throw(
            text => $error_message );
    }
} ## end sub check_vol_attributes

#####################################################################
#
#   function get_vol_members :
#          get all volume member state objects.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->get_vol_members($params);
#
#
#####################################################################

sub get_vol_members {
    my $self = shift;
    my %opts = @_;
    $self->set_vol_obj(
        'volume'  => $opts{'volume'},
        'vserver' => $opts{'vserver'}
    ) if ( !exists( $self->{'vol_obj'} ) );

    my @mem_objects;
    @mem_objects = NACL::C::VolumeMember->find(
        command_interface => $self->{'command_interface'},
        filter            => { 'vserver' => $self->{'vol_obj'}->vserver() }
    );

    ## since volume member find filter is not accepting wildcards when run
    ## in qc filter this line is added

    my $pattern = $self->{'vol_obj'}->volume() . '*';
    my @vol_mem_objs;

    foreach (@mem_objects) {
        if ( $_->state()->volume() =~ /$pattern/ ) {
            push( @vol_mem_objs, $_ );
        }
    }

    return @vol_mem_objs;
} ## end sub get_vol_members

#####################################################################
#
#   function check_state :
#          checks volume state(online/offline/etc). If state not attained then
#          wait until that state is reached.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->check_state($params);
#
#
#####################################################################

sub check_state {
    my $self = shift;
    $Log->enter();
    my $command_interface = $self->{'command_interface'};
    my %opts = validate @_, {
        'volume'         => { type => SCALAR },
        'expected_state' => { type => SCALAR },
        'wait_time'      => { type => SCALAR, default => 600 }

    };
    my ( $time, $timeout );
    my ( $object, $state, $reached );
    $timeout = $opts{'wait_time'};
    $reached = "false";
    $time    = time;
    $timeout += $time;
    while ( $time < $timeout ) {

        Tharn::sleep 60;
        $object = NACL::C::Volume->find(
            command_interface => $self->{'command_interface'},
            filter            => { volume => $opts{'volume'} }
        );
        $state = $object->state()->state();

        if ( $state eq $opts{'expected_state'} ) {

            $Log->step( "Success :: Volume $opts{'volume'} is in"
                    . " state \"$state\"   " );
            $reached = "true";
            last;
        }

        $time = time;

    }    ##While loop end
    if ( ( $reached ne "true" ) && $time >= $timeout ) {
        $Log->step(
            "Failure :: Volume $opts{'volume'}  could not reach the state"
                . " \"$opts{'expected_state'}\" before timeout " );
        $Log->step("Volume state is \"$state\" ");
        return 0;

    }
    return 1;
    $Log->exit();
} ## end sub check_state

#####################################################################
#
#   function gridiron_state :
#          checks and return gridiron output.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->gridiron_state($params);
#
#   BURT: New bug 415090 [3A] - "apiset volume_show with -checking option
#         doesn't give hash format output."
#
#####################################################################

sub gridiron_state {
    my $self = shift;
    $Log->enter();
    my %opts = validate_with(
        params      => \@_,
        spec        => { filer => { type => SCALAR }, },
        allow_extra => 1
    );
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    my $Api_set_obj = NACL::APISet->new(
        name      => $opts{filer},
        category  => "Node",
        interface => "CLI",
        set       => 'CMode',
        connid    => 'ssh'
    );
    my $response = $Api_set_obj->volume_show(
        checking => "",
        volume   => $self->{vol_obj}->volume(),
        vserver  => $self->{vol_obj}->vserver()
    );

#BURT: New bug 415090 [3A] - "apiset volume_show with -checking option doesn't give hash format output."
    return $response->get_raw_output();
} ## end sub gridiron_state

#####################################################################
#
#   function check_gridiron_state :
#          checks gridiron state(scanning/checking/completed). If state not
#          attained then wait until that state is reached.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->check_gridiron_state($params);
#
#
#####################################################################

sub check_gridiron_state {

    my $self = shift;
    $Log->enter();

    $Log->comment("This is new's method ");
    my %opts = validate_with(
        params => \@_,
        spec   => {
            state   => { type => SCALAR },
            filer   => { type => SCALAR },
            timeout => { type => SCALAR, default => 60 },
        },
        allow_extra => 1
    );
    my $time_interval = 0;
    while ( $time_interval < $opts{timeout} ) {
        $time_interval++;
        Tharn::sleep(2);
        if ( $self->gridiron_state( filer => $opts{filer} )
            =~ /$opts{state}/m )

        {
            last;
        }

        $Log->comment( "This is the return value "
                . $self->gridiron_state( filer => $opts{filer} ) );

        if ( $time_interval == $opts{timeout} ) {
            NACL::APISet::Exceptions::ResponseException->throw( text =>
                    "Timeout : Gridiron iron state failed to set to [$opts{state}] "
            );
        }
    } ## end while ($time_interval < $opts...
} ## end sub check_gridiron_state

#####################################################################
#
#   function write_file :
#          create a file on volume.Fills data into it.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->write_file($params);
#
#
#####################################################################

sub write_file {

    my $self = shift;
    $Log->enter();
    $Log->comment("This is new method");
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'client'   => { type => SCALAR },
            'file'     => { type => SCALAR },
            'connid'   => { type => SCALAR, default => 'ssh' },
            'filedata' => { type => SCALAR, optional => 1 }
        },
        allow_extra => 1
    );

    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    $opts{'filedata'} = $opts{'filedata'} || "Sample Data for Test";

    my ( $api_set_obj, $mount_name, $volume_state, $junctionpath );
    $api_set_obj = NACL::APISet->new(
        name      => $opts{'client'},
        category  => "Host",
        interface => "CLI",
        set       => $self->_get_client_hosttype( $opts{'client'} ),
        connid    => $opts{'connid'},
        conntype  => ''
    );

    $Log->comment("reached here");

    $volume_state = $self->{'vol_obj'}->state();
    $junctionpath = $volume_state->junction_path();
    $Log->comment("This is the junc path $junctionpath");
    $Log->comment(
        "This is the juncactive" . $volume_state->junction_active() );
    if (   $volume_state->junction_active() ne 'true'
        || $volume_state->junction_path() !~ /\w+/ )
    {
        NACL::APISet::Exceptions::ResponseException->throw(
            text => "Volume $self->{'vol_obj'}->volume() either"
                . " doesnot have a junction path or its junction is not active"
        );
    }
    $mount_name = "/tmp/" . $volume_state->volume() . $$;
    $api_set_obj->execute_raw_command( 'command' => "mkdir " . $mount_name );
    $api_set_obj->mount(
        'source-path' =>
            $self->{'command_interface'}->{'hostrec'}->{'DATA_IP1'}
            . ":$junctionpath",
        'mount-point' => $mount_name
    );
    $api_set_obj->execute_raw_command(
        'command' => "touch " . $mount_name . "/$opts{'file'}" );
    $api_set_obj->execute_raw_command( 'command' =>
            "/bin/echo $opts{'filedata'} > $mount_name/$opts{'file'}" );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/umount $mount_name" );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/rmdir $mount_name" );
    return 1;
} ## end sub write_file

#####################################################################
#
#   function delete_file :
#          deletes a file on volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->delete_file($params);
#
#
#####################################################################

sub delete_file {
    my $self = shift;
    $Log->enter();
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'client' => { type => SCALAR },
            'file'   => { type => SCALAR }
        },
        allow_extra => 1
    );
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    my ( $api_set_obj, $mount_name, $volume_state, $junctionpath );

    $api_set_obj = NACL::APISet->new(
        name      => $opts{'client'},
        category  => "Host",
        interface => "CLI",
        set       => $self->_get_client_hosttype( $opts{'client'} ),
        connid    => 'ssh',
        conntype  => ''
    );

    $volume_state = $self->{'vol_obj'}->state();
    $junctionpath = $volume_state->junction_path();
    return 0
        if ( $volume_state->junction_active() ne 'true'
        || $volume_state->junction_path() !~ /\w+/ );
    $mount_name = "/tmp/" . $volume_state->volume() . $$;
    $api_set_obj->execute_raw_command( 'command' => "mkdir " . $mount_name );
    $api_set_obj->mount(
        'source-path' =>
            $self->{'command_interface'}->{'hostrec'}->{'DATA_IP1'}
            . ":$junctionpath",
        'mount-point' => $mount_name
    );
    $api_set_obj->execute_raw_command(
        'command' => "rm " . $mount_name . "/$opts{'file'}" );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/umount $mount_name" );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/rmdir $mount_name" );
    return 1;
} ## end sub delete_file

#####################################################################
#
#   function get_file_content :
#          gets the contents of a file on a volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->get_file_content($params);
#
#
#####################################################################

sub get_file_content {

    my $self = shift;
    my $pkg  = ref($self);
    $Log->enter();
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'client' => { type => SCALAR },
            'file'   => { type => SCALAR },
            'connid' => { type => SCALAR, default => 'ssh' }
        },
        allow_extra => 1
    );

    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    my ($api_set_obj,  $mount_name, $filecontent,
        $volume_state, $junctionpath
    );

    $api_set_obj = NACL::APISet->new(
        name      => $opts{'client'},
        category  => "Host",
        interface => "CLI",
        set       => $self->_get_client_hosttype( $opts{'client'} ),
        connid    => $opts{'connid'},
        conntype  => ''
    );

    $volume_state = $self->{'vol_obj'}->state();
    $junctionpath = $volume_state->junction_path();

    if ( $volume_state->volume_style() eq "striped" ) {

        if (   $volume_state->junction_active() ne 'true'
            or $volume_state->junction_path() !~ /\w+/ )
        {
            return 0;
        }

        $mount_name = "/tmp/" . $opts{volume} . $$;
        $api_set_obj->execute_raw_command(
            'command' => "mkdir " . $mount_name );
        $api_set_obj->mount(
            'source-path' =>
                $self->{'command_interface'}->{'hostrec'}->{'DATA_IP1'}
                . ":$junctionpath",
            'mount-point' => $mount_name
        );
        $filecontent = $api_set_obj->execute_raw_command(
            'command' => "cat $mount_name/$opts{'file'}" );
        $api_set_obj->execute_raw_command(
            'command' => "/bin/umount $mount_name" );
        $api_set_obj->execute_raw_command(
            'command' => "/bin/rmdir $mount_name" );

        $self->{'vol_obj'}->{'filecontent'} = $filecontent;

        return $filecontent;
    } ## end if ($volume_state->volume_style...

    else {

        $filecontent = NACL::C::Node->run(
            command_interface => $self->{'command_interface'},
            node => $self->{'command_interface'}->{'hostrec'}->{'hostname'},
            command => "rdfile /vol/$opts{volume}/$opts{file}"
        );
        my $command
            = "system node run .* rdfile /vol/$opts{volume}/$opts{file}";
        $filecontent =~ s/$command//;
        $self->{'vol_obj'}->{'filecontent'} = $filecontent;
        return $filecontent;
    } ## end else [ if ($volume_state->volume_style...

    $Log->exit();

    return 1;

} ## end sub get_file_content

#####################################################################
#
#   function get_all_files_in_snapshot :
#          gets all files in a snapshot on a volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->get_all_files_in_snapshot($params);
#
#
#####################################################################

sub get_all_files_in_snapshot {

    my $self = shift;
    my $pkg  = ref($self);

    $Log->enter();

    my %opts = validate @_,
        {
        'volume'   => { type => SCALAR, optional => 1 },
        'client'   => { type => SCALAR },
        'snapshot' => { type => SCALAR, optional => 1 },
        'vserver'  => { type => SCALAR, optional => 1 }
        };

    $opts{'volume'}
        = $opts{'volume'} ? $opts{'volume'} : $self->{'vol_obj'}->volume();

    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );

    if (!$self->vol_exists(
            volume  => $self->{vol_obj}->volume(),
            vserver => $self->{vol_obj}->vserver(),
        )
        )
    {
        $Log->comment(
            "$pkg:: Failure :: The  specifed volume doesnot exists.");
    }

    my ( $api_set_obj, $mount_name, $filelist, @files, $volume,
        $junctionpath );

    $api_set_obj = NACL::APISet->new(
        name      => $opts{'client'},
        category  => "Host",
        interface => "CLI",
        set       => $self->_get_client_hosttype( $opts{'client'} ),
        connid    => 'ssh',
        conntype  => ''
    );

    $volume = NACL::C::Volume->find(
        command_interface => $self->{'command_interface'},
        filter            => { 'volume' => $opts{'volume'} }
    );

    $junctionpath = $volume->state()->junction_path();

    return 0
        if ( $volume->state()->junction_active() ne 'true'
        || $volume->state()->junction_path() !~ /\w+/ );
    $mount_name = "/tmp/" . $opts{volume} . $$;
    $api_set_obj->execute_raw_command( 'command' => "mkdir " . $mount_name );
    $api_set_obj->mount(
        'source-path' =>
            $self->{'command_interface'}->{'hostrec'}->{'DATA_IP1'}
            . ":$junctionpath",
        'mount-point' => $mount_name
    );

    my $command = "ls  $mount_name/.snapshot/$opts{'snapshot'}";

    $filelist = $api_set_obj->execute_raw_command(
        'command' => "ls  $mount_name/.snapshot/$opts{'snapshot'}" );

    $filelist = $api_set_obj->execute_raw_command( 'command' => $command );

    @files = split( /\s+/, $filelist );

    $api_set_obj->execute_raw_command(
        'command' => "/bin/umount $mount_name" );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/rmdir $mount_name" );
    return \@files;

    $Log->exit();

} ## end sub get_all_files_in_snapshot

#####################################################################
#
#   function get_all_files_in_snapshot :
#          gets all snapshots under a striped volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->get_all_files_in_snapshot($params);
#
#
#####################################################################

sub get_all_snapshots_of_volume {

    my $self = shift;
    $Log->enter();
    my %opts = validate @_, {
        'client' => { type => SCALAR }

    };

    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    my ($api_set_obj, $mount_name,   $filelist,
        @files,       $volume_state, $junctionpath
    );

    $api_set_obj = NACL::APISet->new(
        name      => $opts{'client'},
        category  => "Host",
        interface => "CLI",
        set       => $self->_get_client_hosttype( $opts{'client'} ),
        connid    => 'ssh',
        conntype  => ''
    );

    $volume_state = $self->{'vol_obj'}->state();
    $junctionpath = $volume_state->junction_path();

    if (   $volume_state->junction_active() ne 'true'
        || $volume_state->junction_path() !~ /\w+/ )
    {
        NACL::APISet::Exceptions::ResponseException->throw(
            text => "Volume $self->{'vol_obj'}->volume() either"
                . " doesnot have a junction path or its junction is not active"
        );
    }
    $mount_name = "/tmp/" . $volume_state->volume() . $$;
    $api_set_obj->execute_raw_command( 'command' => "mkdir " . $mount_name );
    $api_set_obj->mount(
        'source-path' =>
            $self->{'command_interface'}->{'hostrec'}->{'DATA_IP1'}
            . ":$junctionpath",
        'mount-point' => $mount_name
    );

    my $command = "ls  $mount_name/.snapshot";
    $filelist = $api_set_obj->execute_raw_command( 'command' => $command );
    @files = split( /\s+/, $filelist );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/umount $mount_name" );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/rmdir $mount_name" );

    $Log->exit();
    return \@files;

} ## end sub get_all_snapshots_of_volume

#####################################################################
#
#   function get_all_files :
#          gets all files on a volume.
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#			face)
#			$volume_obj->get_all_files($params);
#
#
#####################################################################

sub get_all_files {

    my $self = shift;
    $Log->enter();
    my %opts = validate_with(
        params => \@_,
        spec   => {
            'client' => { type => SCALAR },
            'connid' => { type => SCALAR, default => 'ssh' }
        },
        allow_extra => 1
    );
    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );
    my ($api_set_obj, $mount_name,   $filelist,
        @files,       $volume_state, $junctionpath
    );
    $api_set_obj = NACL::APISet->new(
        name      => $opts{'client'},
        category  => "Host",
        interface => "CLI",
        set       => "Linux",
        connid    => $opts{'connid'},
        conntype  => ''
    );

    $volume_state = $self->{'vol_obj'}->state();
    $junctionpath = $volume_state->junction_path();

    return 0
        if ( $volume_state->junction_active() ne 'true'
        || $volume_state->junction_path() !~ /\w+/ );
    $mount_name = "/tmp/" . $volume_state->volume() . int( rand(100) );
    $api_set_obj->execute_raw_command( 'command' => "mkdir " . $mount_name );
    $api_set_obj->mount(
        'source-path' =>
            $self->{'command_interface'}->{'hostrec'}->{'DATA_IP1'}
            . ":$junctionpath",
        'mount-point' => $mount_name
    );
    $filelist
        = $api_set_obj->execute_raw_command( 'command' => "ls $mount_name" );
    @files = split( /\s+/, $filelist );

    $api_set_obj->execute_raw_command(
        'command' => "/bin/umount $mount_name" );
    $api_set_obj->execute_raw_command(
        'command' => "/bin/rmdir $mount_name" );

    $Log->exit();

    return \@files;
} ## end sub get_all_files

#####################################################################
#
#   function volume_move :
#          Moves volume to another aggregate
#
#
#   usage : $volume_obj = VolumeTask->new(command_interface => $command_inter
#                       face)
#                       $volume_obj->volume_move($params);
#
#
#####################################################################

sub volume_move {
    my $self = shift;

    my %opts = validate_with(
        params => \@_,
        spec   => {
            volume      => { type => SCALAR },
            dest_aggr   => { type => SCALAR },
            vserver     => { type => SCALAR },
            'wait'      => { type => BOOLEAN, optional => 1 },
            'wait_time' => { type => SCALAR, optional => 1 }
        }
    );

    my $wait_option = $opts{wait}        || 0;
    my $wait_time   = $opts{'wait-time'} || 300;
    my ( $vol_status, $job );

    $self->set_vol_obj( volume => $opts{volume}, vserver => $opts{vserver} )
        if ( !exists( $self->{'vol_obj'} ) );

    NACL::C::Volume->move(
        command_interface       => $self->{command_interface},
        volume                  => $opts{volume},
        vserver                 => $opts{vserver},
        'destination-aggregate' => $opts{dest_aggr},
        job_component           => \$job
    );

    ##Checks if the vol move job has started or not

    if (   ( $job->state()->{'state'} !~ /Running/ )
        && ( $job->state()->{'progress'} !~ /Transferring snapshot/ ) )
    {
        NATE::BaseException->throw("Volume move job failed to start");

    }

    # Wait for the vol move operation to get over
    if ($wait_option) {

        my $end_time = time() + $wait_time;

        while ( time() < $end_time ) {

            ##Check the vol state continuously.
            $vol_status = $self->{'vol_obj'}->state();

            ##exit when the volume is in the desired aggregate.
            if ( $vol_status->{'aggregate'} eq $opts{dest_aggr} ) {
                last;
            }
            Tharn::sleep 5;
        } ## end while (time() < $end_time)
        if ( $vol_status->{'aggregate'} ne $opts{dest_aggr} ) {
            NATE::BaseException->throw(
                "Failed to move volume $opts{volume} to aggregate $opts{dest_aggr}"
            );
        }
        return 1;
    }
    else {
        return $job->state()->{'id'};
    }
} ## end sub volume_move

##===========================================================================
## Function:    _get_client_hosttype
##
## Description: This is a function to get the conntype for unix client based
##              on host file.
##
## Synopsys:    _get_client_hosttype(<client>);
##
## Returns:     hosttype of the client
##
## Note   :   As of now the NACL APISet has only 'Linux' as conntype for all
##            unix types. When there is a change in APISet, this can be
##            updated.
##
##===========================================================================

sub _get_client_hosttype {
    my $self        = shift;
    my $client_spec = shift;
    my $h           = host($client_spec);
    my $type        = $h->hosttype();

    my %hosttype_map = (
        'unix-solaris'      => 'Linux',
        'unix-linux'        => 'Linux',
        'unix-hpux'         => 'Linux',
        'unix-aix'          => 'Linux',
        'unix-freebsd'      => 'Freebsd',
        'unix-*'            => 'Linux',
        'unix-linux-redhat' => 'Linux'
    );

    return $hosttype_map{$type};

} ## end sub _get_client_hosttype

1;
