# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/EA/WaflironMemberTask.pm#1 $
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##
##	    This WaflironMember task library accomodates all Wafliron
##      related function that are reusable by team.
##       
##      This library is developed by WAFL-EA team. 
##
##	@description
## 	    This Library covers the following functions
##
##	@functions
##
##		 check_status()
##
## 	@dependencies
##		use NACL::C::WaflironMember;
##		use Tharn qw($Log);
##	`	use Params::Validate
##		NACL::APISet::Exceptions::ResponseException qw(:try);
##
## 	@notes
##
##	@duration
##
##
## 	@author c-dl-rakesh-automation@netapp.com
##
## 	@status (private to WAFL-EA)
##


package WaflironMemberTask;
use base qw(FS::EA::Exceptions::WaflironException);
use NACL::C::WaflironMember;
use Carp qw(confess);
use Tharn qw($Log);
use Params::Validate
  qw(validate validate_with SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use strict;
use warnings;
use Data::Dumper;

#####################################################################
#
#   function new :
#   	Constructor for package WaflironMemberTask.
#
#   usage : $wafl_mem_obj = WaflironMemberTask->new(command_interface => $command_inter
#			face)
#	
#####################################################################

sub new {
	$Log->enter();
	my $pkg   = shift;
	my $class = ref($pkg) || $pkg;
	my %opts  = validate @_,
	  { command_interface => { isa => 'NACL::C::CommandInterface' }, };

	my $self = {@_};
	bless $self, $class;
	$Log->exit();

	# Return the blessed object
	return $self;
}

#####################################################################
#
#   function check_status :
#   	Check the wafliron member for a particuler status(online|offline).
#       Wait until the staus is achieved.
#
#   usage : $wafl_mem_obj = WaflironMemberTask->new(command_interface => $command_inter
#			face)
#			$wafl_mem_obj->check_status($params);
#
#
#####################################################################

sub check_status() {

	my $self = shift;
	my $pkg  = ref($self);

	$Log->enter();

	my $command_interface = $self->{'command_interface'};

	my %opts = validate(
		@_,
		{
			"member_aggregate" => { type => SCALAR },
			"expected_status"  => { type => SCALAR },
			"wait_time"        => { type => SCALAR, optional => 1 }

		}
	);

	my ( $object, $state );
	my $endtime = time() + $opts{'wait_time'};
	$object = NACL::C::WaflironMember->find(
			command_interface => $self->{'command_interface'},
			filter            => { aggregate => $opts{'member_aggregate'} }
	);
	$state = $object->state()->iron_status();
	while ( $state ne $opts{'expected_status'} ) {
		Tharn::sleep 2;
		$state = $object->state()->iron_status();
		if ( time() > $endtime ) {
			NACL::APISet::Exceptions::ResponseException->throw( text =>
				"Wafliron on Member aggregate $opts{'member_aggregate'} ".
				" could not reach the status \"$opts{'expected_status'}\" ".
				"before timeout " );
		}

	}    ##While loop end
	
}    ## sub end

1;
