# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/EA/WaflironTask.pm#2 $
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary
##    This Wafliron task library accomodates all Wafliron
##    related function that are reusable by team.
##
##    This library is developed by WAFL-EA team.
##
## @description
##    This Library covers the following functions
##
## @functions
##    check_status()
##
## @dependencies
##    use NACL::C::Wafliron;
##    use Tharn qw($Log);
##    use Params::Validate
##    NACL::APISet::Exceptions::ResponseException qw(:try);
##
## @notes
##
## @duration
##
## @author c-dl-rakesh-automation@netapp.com
##
## @status (private to WAFL-EA)
## @changes 2014-Jan-6 roopeshv added code to make wait_for_commit state. burt774768

package WaflironTask;
use base qw(FS::EA::Exceptions::WaflironException);
use NATE::BaseException qw(:try);
use NACL::C::Wafliron;
use Carp qw(confess);
use Tharn qw($Log);
use Params::Validate
  qw(validate validate_with SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use strict;
use warnings;
use Data::Dumper;
my $Infinity = 9**9**9;

#####################################################################
#
#   function new :
#   	Constructor for package WaflironTask.
#
#   usage : $wafliron_obj = WaflironTask->new(command_interface => $command_inter
#			face)
#
#####################################################################

sub new() {
    $Log->enter();
    my $pkg   = shift;
    my $class = ref($pkg) || $pkg;
    my %opts  = validate @_,
      { command_interface => { isa => 'NACL::C::CommandInterface' }, };

    my $self = {@_};
    bless $self, $class;
    $Log->exit();

    # Return the blessed object
    return $self;
}

#####################################################################
#
#   function check_status :
#   	Check the wafliron for a particuler status(online|offline).
#       Wait until the staus is achieved.
#
#   usage : $wafl_obj = WaflironTask->new(command_interface => $command_inter
#			face)
#			$wafl_obj->check_status($params);
#
#
#####################################################################

# sub check_status {

#     my $self = shift;
#     $Log->enter();
#     my %opts = validate @_, {
#         'aggregate'       => { type => SCALAR },
#         'expected_status' => { type => SCALAR },
#         'wait_time'       => { type => SCALAR, default => 600 }

#     };
#     my $end_time = time() + $opts{'wait_time'};
#     my ( $object, $status );
#     $object = NACL::C::Wafliron->find(
#         command_interface => $self->{'command_interface'},
#         filter            => { aggregate => $opts{'aggregate'} }
#     );
#     $status = $object->state()->iron_status();
#     while ( $status ne $opts{'expected_status'} ) {
#         Tharn::sleep 5;
#         $status = $object->state()->iron_status();
#         if ( time() > $end_time ) {
#             NACL::APISet::Exceptions::ResponseException->throw(
#                 text => "Failure:: Wafliron on aggregate $opts{'aggregate'} "
#                     . "could not reach the status \"$opts{'expected_status'}\""
#                     . " before timeout " );
#         }
#     }
#     $Log->exit();
# }    ## sub end

sub _get_wafliron_status {
    my $pkg       = shift;
    my $aggregate = shift;
    my ( $status, $object );

    if ( $pkg->{'command_interface'}->mode() eq 'CMode' ) {
        try {

            $object = NACL::C::Wafliron->find(
                command_interface => $pkg->{'command_interface'},
                filter            => { aggregate => $aggregate }
            );

            $status = $object->state->iron_status();
        }
        catch NACL::Exceptions::NoElementsFound with {
            my $exception_object = shift;
            $status = "not_running";
        };

        return $status;

    }    # if cmode ends here

    else {
        # burt774768
        # For 7mode, to maintain library compatibility, the wafliron nacl object
        # will return not_running even though the aggregate is
        # waiting_for_commit. The waiting_for_commit state can be deduced by
        # checking the truth value of iron_waiting_for_commit_reject flag on
        # wafliron object
        try {
            my $object = NACL::C::Wafliron->find(
                command_interface => $pkg->{command_interface},
                filter            => {aggregate => $aggregate},
            );
            $status = $object->state()->iron_status();
            my $waiting_for_commit = $object->state->iron_waiting_for_commit_reject;

            if ($waiting_for_commit == 1 && $status eq 'not_running') {
                $status = 'waiting_for_commit';
            }
        } catch NACL::Exceptions::NoElementsFound with {
            my $exception_object = shift;
            $status = "not_running";
        };
        return $status;
    }
}

## Mode Agnostic check_status

sub check_status {

    my $pkg = shift;
    $Log->enter();
    my %opts = validate @_, {
        'aggregate'       => { type => SCALAR },
        'expected_status' => { type => SCALAR },
        'wait_time'       => { type => SCALAR, default => $Infinity }

    };

    my $endtime = time() + $opts{wait_time};
    my $status  = $pkg->_get_wafliron_status( $opts{'aggregate'} );
    while ( $status ne $opts{'expected_status'} ) {

        Tharn::sleep 5;
        $status = $pkg->_get_wafliron_status( $opts{'aggregate'} );
        if ( time() > $endtime ) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "Failure:: Wafliron on aggregate $opts{'aggregate'} "
                  . "could not reach the status \"$opts{'expected_status'}\""
                  . " before timeout " );
        }

    }
    $Log->exit();
}

sub stop {
    my $self = shift;
    my ( $object, $status );
    $Log->enter();
    my %opts = validate @_,
      {
        'aggregate' => { type => SCALAR },
        'wait_time' => { type => SCALAR, default => 600 }
      };

    $object = NACL::C::Wafliron->new(
        command_interface => $self->{'command_interface'},
        aggregate         => $opts{'aggregate'}
    );
    $status = $object->state()->iron_status();
    if ( $status eq 'not_running' ) {
        $Log->comment( "Wafliron is currently not running on the aggregate - "
              . $opts{'aggregate'} );
        return 1;
    }
    else {
        $object->stop();
    }
    Tharn::sleep 5;

    my $end_time = time() + $opts{'wait_time'};

    $status = $object->state()->iron_status();
    while ( $status ne 'not_running' ) {
        Tharn::sleep 20;
        $status = $object->state()->iron_status();
        if ( time() > $end_time ) {
            NACL::APISet::Exceptions::ResponseException->throw(
                text => "Failure:: Wafliron on aggregate $opts{'aggregate'} "
                  . "could not reach the status \"$opts{'expected_status'}\""
                  . " before timeout " );
        }
    }
    $Log->exit();
}    ## sub end
1;
