# $Id: //depot/prod/test/Rsierranevada/lib/FS/File_check.pm#1 $

#this library is intended for C-mode Raid error propagation scripts


=head1 NAME  
                 FS::File_check
                

=head1 DESCRIPTION
                File_check -   Library that supports  file check operation
                with different options using NACL APISet

=head1 ROUTINES
                file_check(): Routine for executing file check on a particular file 
		file_check_read_output(): routine to parse the output file of
                                         file check
=head1 STATUS
 @status public
 
=head1 AUTHOR
 @author Hardikkumar.Darji@netapp.com

=head1 DISCLAIMER

 @disclaimer This library does NOT support duplicate vol names in the same aggr. 
             In short no vserver checking is available.
=cut


# Compiler directives.
use strict;
use warnings;

# =============================================================
# Module Import
# =============================================================

require Exporter;

package FS::File_check;
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF HASHREF OBJECT BOOLEAN UNDEF);
use NACL::APISet;
use vars qw (@ISA @EXPORT_OK);
use Data::Dumper;
use Tharn qw($Log);
use POSIX;
use NACL::APISet::Exceptions::ResponseException qw(:try);

# =============================================================
# Components Used
# =============================================================



@ISA       = qw(Exporter);
@EXPORT_OK = qw(file_check file_check_read_output);

# =============================================================
# Declare Global Variables in BEGIN
# =============================================================

BEGIN {
    use vars qw(
        $Api_Set_Obj
        );
}



=head2 file_check(%opts )
 @summary
     Return the output file by executing file check
     command on particular file.
     
 @description
     file_check - Calls file check on a particular file 

 @arg api_obj Required:
      api_obj => APISet object.

 @arg args Required:
      args is the argument to the file check command
      
      example:
      file check <file path>
      args=>"/vol/testvol_flex/file"
      file check  -vol <vol> -i <inode>
      args=>"-vol testvol_flex -i 96"

@arg mode Optional:
     can be "test", "diag", etc.
 
  
@return value
      Returns errcode = 0 - Success
                      < 0 - Failure
              result = On Success, path of file, where output of file check is stored.
                     = On Failure, error message.
 @usage: 
               (errcode, $path) = file_check( api_obj => $APISet_obj, args => "/vol/vol1/testfile",mode=>"test");
=cut
# =============================================================

sub file_check {
     my %opts = validate(
	@_,
	{
		api_obj => { type => OBJECT },
                args => {type => SCALAR },
                mode => {type => SCALAR, optional =>1},
	}
    );
    
    my $Api_Set_Obj =  $opts{api_obj};
    my $fc = $opts{conn};
    my $filecheck_str = $opts{args};

    
    #Splitting the filecheck_str to get an array of arguments
    my @arr = split('\s+',$filecheck_str);
    my $mode = $opts{mode};

    if (!defined($opts{mode})){
       $mode = "diag";
    }
 
    my ($output,$resp,$result, $inum, $vol, $snap_id, $snap, $Snap, $path, $vol_snap, $private, $msg,
        $errcode, $i, $neg_flag, %parser_fc_args, $invalid_snap_flag, $path_of_file);
    
    #
    #Initializing Values
    #
    $msg = "Output will be saved to file ";
    $errcode = -3;
    $i = 0;
    $invalid_snap_flag = 0;
    $path = ""; 
    $vol = "";
    $vol_snap = "";
    $inum = "";
    $snap = "";
    $snap_id = "";
    $Snap = "";
    $private = "";
  
    #Parsing the file check arguments provided by the user. 
    
    for($i = 0;$i < @arr;$i++) {

        #For negative senario.(Avoid overwriting in to the hash data structure created)

        #E.g. if the user is passing -s snap1 -s snap2 which should not be overwritten.

        if (!defined($parser_fc_args{$arr[$i]})) {  
           #Keys to match in the array inorder to assign the corresponding value.           
 
	   if ($arr[$i] =~ m!(/)|-[v|s|S|i]|(-private)!) { 
              if ($1) { 
                 #Argument starting with a / is nothing but path of a file provided for file check.
                 #Assigning the path provided,to the hash by setting the key as -path. 

                 $parser_fc_args{-path} = $arr[$i]; 
              }
              elsif($2) {  
                 #If -private is provided.hash will have "-private" as a key and value.

                 $parser_fc_args{$arr[$i]} = "-private";
              }
              else {
                 #If the arguments are having -v or -s or -i then the values which
                 #are next to this arguments in the array will have the corresponding
                 #values(E.g. -v <volname>) so,assign that to a hash  as a key ,value pair.
 
                 #Since -v and -vol are treated same in file check, If the argument 
                 #contains -vol, then it is handled by assigning the value next to -vol in 
                 #the array as a value for the key -v in the hash.

		 if ($arr[$i] =~ m!-vol!) {
                    $parser_fc_args{-v} = $arr[$i+1];
                    $i++;
		 } 
                 else {
                    $parser_fc_args{$arr[$i]}= $arr[$i+1]; 
	            $i++;                                 
                 }
              }                                
           }
            else {
              #If the array is having any other keys other than -v etc.,set the flag.  

              $neg_flag =1;
           }
        }
        else {
           #If 2 keys are same set the negative flag.

           $neg_flag =1;
        }
    }
    
    #If the negative flag($neg_flag) is set to one means,the arguments passed by the user 
    #is having some wrong data.So pass it as it is to file check and return the appropriate
    #errorcode to the caller.
 
    if (!defined($neg_flag)) {
    
       #Producing the strings inorder to pass to a file check command.

       if (defined($parser_fc_args{-S})) {
          #Snapshot name. The pattern will be -S <snapshot_name>

          $Snap = "-S $parser_fc_args{-S}";
       }

       if (defined($parser_fc_args{-i})) {
          #Inode number of the file.The pattern will be -i <inode number>
          $inum = "-i $parser_fc_args{-i}";
       }

       if (defined($parser_fc_args{-v})) {
          #Volume name.The pattern will be -v <volname>
          $vol = "-v $parser_fc_args{-v}";
       }

       if (defined($parser_fc_args{-path})) {
          #Path of the file.The pattern will be /vol/<vol>/<file>

          $path = "$parser_fc_args{-path}";
       }

       if (defined($parser_fc_args{-private})) {
          #"-private" option.The pattern is -private.

          $private = "$parser_fc_args{-private}";
       }
       
       if (defined($parser_fc_args{-s})) {
          #Snapshot id of the snapshot.The pattern will be -s <snapid>
	  $snap = "-s $parser_fc_args{-s}";
          
       }
       #Checking the version and generating the input string /argument sting to the file check command.
       $filecheck_str="$path $vol $inum $snap $Snap $private";
             
    }#end of  if(!defined($neg_flag)) {
    
    $resp = $Api_Set_Obj->system_node_run(node => "local", command => "\" priv set $mode; file check $filecheck_str\""); 
    $output= Dumper($resp->get_raw_output());
    
    my @lines = split /\n/, $output;
    
    if($output eq "") {
        $errcode = -3;
        $result = "Bad output string: ".$output;
        return ($errcode,$result);
    }
    
    
    foreach (@lines) {
    # removing extra '\' from the output 
    $_ =~ s/\\//g;;
    
       if($_ =~ m/^(File|Snapshot|Volume|Snapid)\s+(\S+)\s+doesn't\s+exist/i){
          $errcode = -1;
          $result = $_;
          last;
       }

       if($_ =~ m/$msg(.*)/){
           $errcode = 0;
           $result = $1;
           # removing the extra " ' " from  the output 
           $result =~ s/'/ /g;
           
           last;
       }
    }

    return ($errcode,$result);
}

=head2 file_check_read_ouput($fc, $PATH)

 @summary
     Parses output of `file check`
     
 @description
     Returns the parsed output of file check log.

 @arg api_obj Required:
      APISet object.
      
 @arg $path Required:
      path of the log, returned when file_check() is called

 @return value
      Returns  $count = Number of corrupted blocks
               $total = Number of blocks scanned
               @vbnlist = Reference to Array of vbns corrupted.
               @snapshot_list = Reference to Array of snapshots that are possibly corrupt.
               $level = block Error level (Level 0 or Level 1)
               $error_flag = Error number is set to non zero or zero
               @fbns = Reference to Array of corrupted File Block Numbers
               $level = Level 0 or Level 1 error
               $infocode = special info code set or not (-1,-2 or -3)
               $info_string = spacial information
               
 @usage
               my ($count, $total, $vvbn_list, $snapshot_list, $level,
               $error_flag, $fbns,$infocode,$info_string) = file_check_read_output(api_obj=> $APISet_obj, path => $file_check_path);
=cut
sub file_check_read_output(%)
{
    my %opts = validate(
	@_,
	{
		api_obj => {type => OBJECT },
		path => {type => SCALAR },
	}
    );

    my $Api_Set_Obj = $opts{api_obj};
    my $path = $opts{path};

    
    my $snapshot_msg = "The following snapshots contain any of the listed blocks:";
    my $snapshots_corrupt=0;
    my $infocode=0;
    my $info_string="";
    my $count=0;
    my $total =0;
    my $error_flag=0;
    my @vvbn_list;
    my @snapshot_list;
    my ($level, $errno, @fbns);
    $level = 0;
    $errno = 0;
    
    my $resp =  $Api_Set_Obj->system_node_run(node => "local",command =>"rdfile $path");
    my $output = Dumper($resp->get_raw_output());
    my @lines = split /\n/, $output;

    foreach (@lines) {
     
     if($_ =~ m/Lost\sVVBN:\s(\d+),/) {
        # New format from IC.3 onwards
        # Lost VVBN: 4150, Disk: 0a.18, Volume: testvol_flex, Inode: 100

        push @vvbn_list, $1;
      }	 

      #if($_ =~ m/.*Level:\s+(\d+),\s+Fbn:\s+(\d+),\s+References:\s+(\d+),\s+Error:\s+(\d+)/i) {
         if($_ =~ m/.*Level:\s+(\d+),\s+Fbn:\s+(\d+),(.*)+Error:\s+(\d+)/i){
        # New format in BR.1/RR.0 when we enabled compression on the volume.
        # Lost VVBN: 1941867, Disk: 0a.20, Volume: vol1, Inode: 96, Level: 0, Fbn: 0, Error: 117
        # Level and errno are same for all the block in CG since level and errno are
        # assigned to scalar variable.
        $level = $1;
        push @fbns, $2;
        $errno = $4;
        #checking whether "Error:" is > 0 
        if($errno > 0)
        {
              $error_flag=1;
        }
      }

      # Store the number of lost blocks
      if($_ =~ m/Total\s(\d+)\slost\sblock/) {
        $count = $1;
      }

      # Determine if there are any corrupt blocks in the snapshots.
      if($_ =~ /$snapshot_msg/) {
        $snapshots_corrupt = 1;
      }

      # If there are snapshots, then add them to a list.
      if(($_ =~ /\d+:\d+:\d+/) && ($snapshots_corrupt == 1)) {
        my @line = split / /, $_;
        push @snapshot_list, $line[0];
      }

      # Store the total number of blocks scanned
      if($_ =~ m/Total\s(\d+)\sblock\(s\)\sscanned/) {
        $total = $1;
      }
      # if fixable metafile id
      if($_ =~ m/Inode\s+(\d+)\s+is\s+a\s+metafile\s+id/)
      {
        $infocode=-1;
        $info_string=$_; ;
      }
      # if non fixable metafile id
      if($_=~ m/Inode\s+(\d+)\s+-\s+I2P\s+not\s+available\s+for\s+private\s+inode/)
      {
        $infocode=-2;
        $info_string=$_;
      }
       
       # if Failed to load inode
      if($_=~ m/Failed\s+to\s+load\s+inode/)
      {
        $infocode=-3;
        $info_string=$_;
      }
         
    }
    return ($count, $total, \@vvbn_list, \@snapshot_list, $level, $error_flag, \@fbns,$infocode,$info_string);
    
}

1;
