# Hey emacs, this is basically -*- perl -*-
#
# Copyright (c) 2004 Network Appliance, Inc.
# All rights reserved.
#
## @summary  snapmirror library -- now obsolete
##
## @description
## DISCLAIMER - this library is just temporary, until the new SnapMirror.pm
## is delivered.
##
## $Id: //depot/prod/test/Rsierranevada/lib/FS/SnapMirror.pm#1 $
##
## These test functions use various commands to manipulate snapmirrors
## <p>
##
## @usage
##   use FS::SnapMirror;
##   most routines are passed a  valid connection $conn which
##   is created using $conn=FS:Util:Setup::create_vol_and_mount()
##
## @keywords snapmirror
## @dependencies
##     test/snapmirror/sm_init.thpl
##
## @notes  interface changes in SnapMirror library
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
##
## @status obsolete -- use lib/SnapMirror.pm
##
package FS::SnapMirror;

use strict;

use Tharn;
use SPARSE::Util::Logger;
use SnapMirror qw(snapmirror_status);
use Data::Dumper;

use vars ('@ISA','@EXPORT_OK');

require Exporter;
@ISA = ('Exporter');
@EXPORT_OK = qw(
    snapmirror_initialize
    snapmirror_quiesce
    snapmirror_break
);

use constant SLEEPYTIME => 20;
use constant NO_SUCH_DEST => 'the given path is not a snapmirror destination';

# public sub-routines

## @name ::snapmirror_initialize
##
## @description
##  Initialize the given snapmirror
##
## @synopsis
##  snapmirror_initialize($conn)
##
## @arg $conn required: valid connection (see snapmirror usage description)
##
## @example
##  snapmirror_initialize($conn);
##

sub snapmirror_initialize($$$$$$$) {
    my ($conn, $src_filer, $dst_filer, $src_path, $dst_path, $complete, $delay) = @_;

    # initialize the snapmirror with subtest sm_init.thpl
    my $result = subtest(
	"snapmirror/sm_init.thpl",
	"FILER_SRC=$src_filer",
	"FILER_SRC_PATH=$src_path",
	"FILER_DST=$dst_filer",
	"FILER_DST_PATH=$dst_path",
	"COMPLETE=$complete",
	"DELAY=$delay",
	'-onexit', sub {},
    );

    ## @step check snapmirror status
    logStep("check snapmirror status");
    my @status = snapmirror_status(
	$conn,
	"${dst_filer}:${dst_path}",
	source => "${src_filer}:${src_path}",
    ) or die $@;

    return($result, @status);
}

## @name ::snapmirror_quiesce
## @summary Allow in-progress transfers to snapmirror destination to complete
## after which new transfers are not started.
## @description Control is returned after the quiesce has completed 
## @arg $conn Required: The filer connection object for the command line interface
## @arg $dest Required: The destination volume or qtree
sub snapmirror_quiesce($$) {
    my $conn  = shift;
    my $dest  = shift;

    if ($dest eq '') {
	die "A valid volume or qtree must be specified";
    }

    my @status = snapmirror_status($conn, $dest);
    logDbg("1 - snapmirror status:\n" . Dumper(\@status) . "\n");

    if ($status[2] !~ /Snapmirrored/) {
	return @status;
    }

    my $cmd = "snapmirror quiesce $dest"; 
    my (undef, undef,$after) = say($conn, $cmd);
    logDbg("after: $after");
    sleep(5);
    if ($after =~ m/Snapmirror is off|volume is not online|destination is offline, is restricted, or does not exist/) {
	$@ = $after;
	return undef;
    }

    # wait for the mirror to settle
    my $done = 0;
    do {
	@status = snapmirror_status($conn, $dest);
	logDbg("here - snapmirror status:\n" . Dumper(\@status) . "\n");


	$done = (
	    ($after =~ m/Successfully quiesced|has quiesced/) or
	    ($status[2] =~ /Quiesced/)
	);

	sleep(SLEEPYTIME) if (!$done);
    } while (!$done);

    return @status;
}


## @name ::snapmirror_break
## @summary break a snapmirror
## @description Control is returned after the break has completed 
## @arg $conn Required: The filer connection object for the command line interface
## @arg $dest Required: The destination volume or qtree
sub snapmirror_break($$) {
    my $conn  = shift;
    my $dest  = shift;

    if ($dest eq '') {
	die "A valid volume or qtree must be specified";
    }

    my @status = snapmirror_status($conn, $dest);
    logDbg("2 - snapmirror status:\n" . Dumper(\@status) . "\n");

    if ($status[2] !~ /Quiesced/) {
	return @status;
    }

    my $cmd = "snapmirror break $dest"; 

    my (undef, undef,$after) = say($conn, $cmd);
    logDbg("after: $after");
    sleep(5);

    if ($after =~ m/Snapmirror is off|volume is not online|destination is offline, is restricted, or does not exist/) {
	$@ = $after;
	return undef;
    }

    return $after;
}

1;

