
# $Id: //depot/prod/test/Rsierranevada/lib/FS/Util/CIFS_Client_Session.pm#3 $

package FS::Util::CIFS_Client_Session;

use 5.008008;
use strict;
use warnings;

require Exporter;

use Params::Validate qw(validate BOOLEAN SCALAR HASHREF ARRAYREF OBJECT);
use Carp qw(croak);
use File::Basename qw(basename);
use Data::Dumper;

# need to use an import list here because we define a disconnect() method
use Tharn qw();

# avoid cruft
#BEGIN { @INC = grep !ref($_), @INC; }

# these constants map the indices of the components within the list that say() returns.
use constant SAY_MATCH_IDX  => 0;
use constant SAY_BEFORE_IDX => 1;
use constant SAY_AFTER_IDX  => 2;
use constant SAY_PROMPT_IDX => 3;

# change this when our version is incorporated into the shared MinWin.pm (in "phase 2").
# also, note other MinWin package references to be changed below.
use dblade::wafl::cifs::temp::TestEnv::MinWin qw(execute_cmd_interactive execute_cmd_special);
use SayWrap qw(sayEOF);

use FS::Util::CIFS_Session;

BEGIN {
    Params::Validate::validation_options(
        # allow presence of options other than those named in the validation specification
        allow_extra => 1,
   );
}

our @ISA = qw(Exporter FS::Util::CIFS_Session);

# Items to export into callers namespace by default. Note: do not export
# names by default without a very good reason. Use EXPORT_OK instead.
# Do not simply export all your public functions/methods/constants.

# This allows declaration	use vol_db ':all';
# If you do not need this, moving things directly into @EXPORT or @EXPORT_OK
# will save memory.
our %EXPORT_TAGS = (
    'all' => [ qw() ]
);

our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );

our @EXPORT = qw();

our $VERSION = '0.01';

sub new {
    my $class = shift;
    $class = ref($class) || $class;

    # initialize self
    my $self = $class->SUPER::new(@_);

    $self->{capture_sessions} = [];

    # done
    return $self;
}

sub create_file {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    file => {
		type => SCALAR,
	    },
	    contents => {
		type    => SCALAR,
		default => 'just some text',
	    },
	}
    );

    my $cmd = "echo \"$args{contents}\" > \"$args{file}\"";
    my $response = execute_cmd_interactive($self->{session_conn}, $cmd, -swallow);
    return $response->[SAY_AFTER_IDX];
}

sub del_files {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    files => {
		type => ARRAYREF,
	    },
	}
    );

    # build the command
    my $cmd = 'del ';
    foreach my $file (@{$args{files}}) {
	$cmd .= "\"$file\" ";
    }

    my $response = execute_cmd_interactive($self->{session_conn}, $cmd, -swallow);
    return $response->[SAY_AFTER_IDX];
}

sub cacls {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    file => {
		type => SCALAR,
	    },
	    options => {
		type    => SCALAR,
		default => '',
	    },
	}
    );

    my $cmd = "cacls $args{file} $args{options}";
    my $response = execute_cmd_interactive($self->{session_conn}, $cmd, -swallow);
    return $response->[SAY_AFTER_IDX];
}

sub create_sparse_file {
    my $self = shift;

    my (%args) = validate(
        @_,
        {
            file => {
                type => SCALAR,
            },
            data_blk1_num => {
                type => SCALAR,
            },
            hole_blk_num => {
                type => SCALAR,
            },
            data_blk2_num => {
                type => SCALAR,
            },
            file_type => {
                type => SCALAR,
            },
        }
    );

    my $cmd = "\"" .  $self->{minwin_xshare} . "\\WAFL\\bin\\mksparse.exe $args{file} $args{data_blk1_num} $args{hole_blk_num} $args{data_blk2_num} $args{file_type}\"";
    my $response = execute_cmd_interactive($self->{session_conn}, $cmd, -swallow);

    return $response->[SAY_AFTER_IDX];
}

sub copy_file {
    my $self = shift;

    my (%args) = validate(
        @_,
        {
            file1 => {
                type => SCALAR,
            },
            file2 => {
                type => SCALAR,
            },
        }
    );

    my $cmd = "copy $args{file1} $args{file2}";
    my $response = execute_cmd_interactive($self->{session_conn}, $cmd, -swallow);

    return $response->[SAY_AFTER_IDX];
}

sub get_capture_interfaces {
    my $self = shift;

    my %ifc_hash;

    my $cmd = '"c:\Program Files\Wireshark\tshark.exe" -l -D';
    my $response = execute_cmd_interactive($self->{session_conn}, $cmd, -swallow);

    my @lines = split(/\r|\n/, $response->[SAY_AFTER_IDX]);
    foreach my $line (@lines) {
        if ($line =~ /^(\d+)\.\s+(.*)$/) {
	    #$ifc_hash{$1} = $2;
	    $ifc_hash{$2} = $1;
	}
    }

    return \%ifc_hash;
}

sub start_capture {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    interface => {
		type    => SCALAR,
		default => undef,
	    },
	    capture_file => {
		type => SCALAR,
		default => undef,
	    },
	    options => {
		type => SCALAR,
		default => undef,
	    }
	}
    );

    # build the command line
    my $cmd = '"c:\Program Files\Wireshark\tshark.exe" -l -q';
    $cmd .= " -i $args{interface}" if (defined($args{interface}));
    $cmd .= " -w $args{capture_file}"  if (defined($args{capture_file}));

    # tack on any options
    $cmd .= " $args{options}" if (defined($args{options}));

    # get a new capture connection
    my ($capture_conn, $minwin_xshare) = $self->new_session();

    # using -swallow and -blind with a command that doesn't exit, so there's nothing returned from say()
    Tharn::say($capture_conn, $cmd, -swallow, -blind);

    my %capture_session = (
	file          => $args{capture_file},
	conn          => $capture_conn,
	minwin_xshare => $minwin_xshare,
    );

    # save this for self-cleanup
    push(@{$self->{capture_sessions}}, \%capture_session);

    return \%capture_session;
}

sub end_capture {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    capture_session => {
		type => HASHREF,
	    },
	}
    );

    my $capture_session = $args{capture_session};
    my $capture_conn = $capture_session->{conn};
    my $capture_file = $capture_session->{file};

    # send an interrupt to end the capture
    my $cmd = "\cC";
    my @response = Tharn::say($capture_conn, $cmd, -swallow);
    $Tharn::Log->trace("end capture response is \n" . Dumper(\@response) . "\n");

    @response = Tharn::say($capture_conn, "cmd.exe /c dir \"$capture_file\"", -swallow);
    $Tharn::Log->trace("post-capture dir response is \n" . Dumper(\@response) . "\n");

    # disconnect the capture connection
    $self->disconnect(conn => $capture_conn);

    # invalidate the capture session entry
    for (my $i = 0; $i < scalar(@{$self->{capture_sessions}}); $i++) {
	my $s = $self->{capture_sessions}->[$i];
        if (
	    defined($s) &&
	    ($s->{file} eq $capture_session->{file}) &&
	    ($s->{minwin_xshare} eq $capture_session->{minwin_xshare})
	) {
	    $self->{capture_sessions}->[$i] = undef;

	    # there should never be more than one matching entry, so we are done
	    last;
	}
    }

    return 1;
}

sub decode_capture_data {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    capture_file => {
		type => SCALAR,
	    },
	    decode_file => {
		type => SCALAR,
		default => undef,
	    },
	    decode_options => {
		type    => SCALAR,
		default => '',
	    },
	}
    );

    my $decode_file = $args{decode_file} || $args{capture_file} . '.decoded';

    # first, decode the captured packets into an xml file
    my $cmd;
    $cmd  = '"c:\Program Files\Wireshark\tshark.exe" -l';
    $cmd .= " -r $args{capture_file}";
    $cmd .= " -T pdml";
    $cmd .= " $args{decode_options}" if (defined($args{decode_options}));
    $cmd .= " > $decode_file";

    my @response = Tharn::say($self->{session_conn}, $cmd, -swallow, -verbose);
    $Tharn::Log->trace("\n\n\n***************\nafter decode, say response is \n" . Dumper(\@response) . "\n***************\n\n");

    @response = Tharn::say($self->{session_conn}, "cmd.exe /c dir \"$decode_file\"", -swallow);
    $Tharn::Log->trace("post-decode dir response is \n" . Dumper(\@response) . "\n");

    return $decode_file;
}

sub find_ntcreate_andx_responses_in_capture_data {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    decode_file => {
		type => SCALAR,
		default => undef,
	    },
	}
    );

    # scan the xml packets for 'NTCreate AndX Response' events
#    my $exe = "c:\\find_ntcreate_responses_in_pdml.pl"; # for testing use
#    my $exe = $self->{minwin_xshare} . "\\WAFL\\bin\\find_ntcreate_responses_in_pdml.pl"; # legacy way
    my $exe = $self->{minwin_xshare} . "\\noarch\\dev\\dblade\\wafl\\cifs\\bin\\find_ntcreate_responses_in_pdml.pl";
    my $cmd = $self->{minwin_xshare} . "\\perl588\\bin\\perl.exe \"$exe\" \"$args{decode_file}\"";

    return $self->scan_capture_data(
        scanner_cmd => $cmd,
    );
}

sub find_file_info_in_capture_data {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    decode_file => {
		type => SCALAR,
		default => undef,
	    },
	}
    );

    # scan the xml packets for file info events
#    my $exe = "c:\\find_file_info_in_pdml.pl"; # for testing use
#    my $exe = $self->{minwin_xshare} . "\\WAFL\\bin\\find_file_info_in_pdml.pl";
    my $exe = $self->{minwin_xshare} . "\\noarch\\dev\\dblade\\wafl\\cifs\\bin\\find_file_info_in_pdml.pl";
    my $cmd = $self->{minwin_xshare} . "\\perl588\\bin\\perl.exe \"$exe\" \"$args{decode_file}\"";

    return $self->scan_capture_data(
        scanner_cmd => $cmd,
    );
}

sub scan_capture_data {
    my $self = shift;

    my (%args) = validate (
	@_,
	{
	    scanner_cmd => {
		type => SCALAR,
	    },
	    decode_file => {
		type => SCALAR,
		default => undef,
	    },
	}
    );

    # scan the xml packets for the target events
    # sometimes a job can last more than 7 minutes on various windows
    # platforms. Adding a 10 minutes timeout will keep telnet session active.
    my @response = Tharn::say($self->{session_conn}, $args{scanner_cmd}, -swallow, -timeout=>600);

    $Tharn::Log->trace("\n\n\n***************\nsay response is \n" . Dumper(\@response) . "\n***************\n\n");

    my @responses;

    my @lines = split(/\r+/, $response[SAY_AFTER_IDX]);

    $Tharn::Log->trace("\n\n\n***************\nlines is \n" . Dumper(\@lines) . "\n***************\n\n");

    foreach my $line (@lines) {
        $line =~ s/\n//g;
        if ($line =~ /^timestamp=(\S+), file='(.*)', end_of_file=(\S+), alloc_size=(\S+)$/) {
	    push(@responses, { timestamp => $1, file => "$2", end_of_file => $3, alloc_size => $4 });
	} else {
	    $Tharn::Log->trace("FYI - ignoring irrelevant data: '$line'");
	}
    }

    return(@responses);
}

sub say {
    my $self = shift;

    # sometimes a job can last more than 7 minutes on various windows
    # platforms. Adding a 10 minutes timeout will keep telnet session active.
    return Tharn::say($self->{session_conn}, @_, -swallow, -timeout=>600);
}

# make sure to close everything out upon object destruction
sub DESTROY {
    my $self = shift;

    # interesting corner case - if we die during construction, then the
    # object goes out of scope immediately and DESTROY is called before
    # the exception is propagated. this means we need to preserve any eval
    # error before proceeding.
    my $eval_error = $@ if ($@);

    if (defined($self->{capture_sessions})) {
        foreach my $capture_session (@{$self->{capture_sessions}}) {
	    if (defined($capture_session)) {
		# bypass exceptions
		eval {
		    $self->end_capture(
			capture_session =>  $capture_session,
		    );
		};
		# tack on any exceptions thrown above
		$eval_error .= "failed to end leftover capture session during DESTROY: $@\nsession data is -\n" . Dumper($capture_session) . "\n" if ($@);
	    }
	}
    }

    if (defined($self->{session_conn})) {
	eval { $self->disconnect(); };
	$eval_error .= "disconnect error: $@" if ($@);
    }

    # now, reset the eval error
    $@ = $eval_error;
}

1;

__END__

## @summary     FS::Util::CIFS_Client_Session - NATE library for managing CIFS client
##              sessions, with support for manipulating various file attributes and
##              packet tracing.
##
## @status      private (in development)
##
## @author      mcrae@netapp.com
##
## @pod here

=head1 NAME

    FS::Util::CIFS_Client_Session - NATE library for managing CIFS client
    sessions, with support for manipulating various file attributes and
    packet tracing.

=head1 SYNOPSIS

  use FS::Util::CIFS_Client_Session;

  # create a new FS::Util::CIFS_Client_Session object here
  my $utility = FS::Util::CIFS_Client_Session->new(host => $fc);

=head1 STATUS

  Private, in development.

=head1 DESCRIPTION

  This module provides general support for executing various operations on a
  given Windows client, including:
  - creating and deleting files.
  - manipulating CIFS ACLs (Access Control Lists) via the cacls command.
  - capturing, decoding and searching network packets

  This class inherits from the FS::Util::CIFS_Session class, which provides
  support for managing client connections.

=head1 METHODS

=over

=item new() - create a new instance of this class, connected to the given (Windows) host. see
the FS::Util::CIFS_Session parent class doc for usage details.

=item create_file() - uses the current session to dump the given contents into the given file or stream. for more info, see http://wikid.netapp.com/w/User:Ymichael/BURT324637#Streams. returns a string containing the client output.

=over

=item file

[Required] path to the target file.

=item contents

[Optional] text to write into the target file, default is 'just some text'.

=back

=item del_files() - delete one or more files on the client.

=over

=item files

[Required] reference to an array of target file paths.

=back

=item cacls() - uses the current session to execute the cacls command on the given file and with the given options. for details, see http://wikid.netapp.com/w/User:Ymichael/BURT324637#ACLs. returns a string containing the client output.

=over

=item files

[Required] reference to an array of target file paths.

=back

=item get_capture_interfaces() - return a hash mapping interface names to numbers, for use in subsequent start_capture() calls.


=item start_capture() - start a new client session and begin capturing packets on the given interface.
returns a hash containing the following keys:

    file - the capture file path.
    conn - the capture session ConnectRec reference.
    minwin_xshare - the UNC path to the MinWin shared tools repository for the capture session.

=over

=item interface

[Required] the id of the target interface.

=item capture_file

[Required] the destination file path where captured packets are saved.

=item options

[Optional] a string to be passed as options on the tshark.exe command line.

=back

=item end_capture() - end packet capture on the given capture session.

=over

=item capture_session

[Required] A reference to a capture descriptor hash returned by a previous invocation of the start_capture() method.

=back

=item decode_capture_data() - decode previously-captured packet data into XML (PDML) format.

=over

=item capture_file

[Required] the name of a file containing previously-captured packet data.

=item decode_file

[Optional] the name of the file to use for the decoded packet data.

=item decode_options

[Optional] a string to be passed as options on the tshark.exe command line.

=back

=item find_ntcreate_andx_responses_in_capture_data() - use the X:\WAFL\bin\find_ntcreate_responses_in_pdml.pl utility on the MinWin share to scan the given XML (PDML) file of packet data for 'NTCreate AndX Response' data.
returns a list of hashes describing the relevant events discovered in the data, keys include 'timestamp', 'file', 'end_of_file' and 'alloc_size'.

=over

=item decode_file

[Required] the name of the source PDML file.

=back

=item find_file_info_in_capture_data() - use the X:\WAFL\bin\find_file_info_in_pdml.pl utility on the MinWin share to scan the given XML (PDML) file of packet data for file info data.
returns a list of hashes describing the relevant events discovered in the data, keys include 'timestamp', 'file', 'end_of_file' and 'alloc_size'.

=over

=item decode_file

[Required] the name of the source PDML file.

=back

=item scan_capture_data() - lower-level routine used to support the find_ntcreate_andx_responses_in_capture_data() and find_file_info_in_capture_data() methods.
returns a list of hashes describing the relevant events discovered in the data, keys include 'timestamp', 'file', 'end_of_file' and 'alloc_size'.

=over

=item scanner_cmd

[Required] the command line for the PDML scan utility to use.

=item decode_file

[Required] the name of the source PDML file.

=back

=item say() - send input text to the default client session. any args given are passed directly to Tharn::say() and the output is returned.

=back

=head1 SEE ALSO

  BURT 324637
  http://wikid.netapp.com/w/User:Ymichael/BURT324637
  http://wikid.netapp.com/w/User:Raghavc/Test_plan/SMB_file_allocation_size
  dblade/wafl/cifs/temp/TestEnv/MinWin.pm (temporary)
  TestEnv/MinWin.pm
  NATE documentation

=head1 AUTHOR

  Chris McRae (mcrae@netapp.com)
  $Id: //depot/prod/test/Rsierranevada/lib/FS/Util/CIFS_Client_Session.pm#3 $
  Last Revised $Date: 2013/05/24 $

=head1 COPYRIGHT AND LICENSE

  Copyright (c) 2008 Network Appliance, Inc.
  All rights reserved.

=cut

