#Hey emacs, this is basically -*- perl -*-
#$Id: $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.


# THIS IS A COPY OF SPARSE::Util::FileIO
# WE NEED TO TWEAK SOME THINGS SINCE THIS WILL BE 
# A MORE GENERAL VERSION OF THE SPARSE LIB.
# IT LOOKS LIKE ITS EASIER TO MAKE THE SPARSE LIB
# TO CALL INTO THIS LIB IN THE FUTURE, THAN TO EXTEND
# SPARSE LIB TO DEAL WITH MORE GENERAL CASES 



## @summary	Library that defines a FileIO object
## @description
##	Abstract basic file IO operations.
##
## @author dl-johny@netapp.com (FSS WAFL Quality)

package FS::Util::FileIO;

use strict;
use Tharn;
use SPARSE::Util::Logger;
use FS::Util::Utilities qw(:utils);
use POSIX;
use Fcntl;
use Params::Validate
	qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF SCALARREF OBJECT UNDEF);
use Module::Common qw(validate_mixed);
use Time::HiRes qw(gettimeofday);
use Carp qw(croak);
use FS::commonParams;

no warnings("uninitialized");

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&fioOpen
		&fioMkDir
		&fioRmDir
	        &VfyDirAbsent
	        &VfyDirPresent
	        &fioMkFifo

		&fioRCmpTestDirSnaps
		&fioRCmpTestDirs
		fioRCmpDir
		&fioDiffFileContents
	
	        &StatCheck
	        &VfyFileAbsent
	        &VfyFilePresent
		
		&Truncate
		
	        &Chown
	        &Chgrp
		&FileUnlink
		&fioLock
	        &make_random_sequence

		&sysSeek
		&sysTell
		&ReadData
		&WriteData
		&ReadVerifyData
		&WriteVerifyData
		&mkFile
	    );

# ------------------------------------------------------------------------

## @name	::fioOpen
## @description
## 	Construct file handle based on vol and test dir
##
##           Either a "path" arg is required, or else both a "name" and "dir" arg.
## @arg		path		(mounted) Path to a file. Optional.
##				    Default is concatination of: $dir . "/" . $name
## @arg		dir		(mounted) Path to directory containing file. Optional.
## @arg		name		Name of file. Optional.
## @arg		vol required
## @arg		mode optional "+>" trunc+write+read
## @arg         mask optional which umask to use when opening file
## @arg         perms optional. file perms.
## @arg         err optional. expected error
## @return 	file object
sub fioOpen
{
	#my ($dir, $name, $mode, $vol, $mask, $perms, $err)	= @_;
	my (%args) = validate (
			       @_,
			       {
				dir		=> {type=>SCALAR, optional=>1},
				name		=> {type=>SCALAR, optional=>1},
				path		=> {type=>SCALAR, optional=>1},
				vol=>		=> {type=>SCALAR, optional=>1},   #XXX probably dont need $vol

				mode=>{type=>SCALAR,
					optional=>1,
					default=>"+<"
				       },
				mask=> {type=>SCALAR,
					 optional=>1
					},
				perms => {type=>SCALAR,
					   optional=>1,
					   default=>0666
					  },
				err=> {type=>SCALAR,
					optional=>1,
					default=>0
				       }
			       }
			      );
	
	my ($dir, $name, $path, $mode, $vol, $mask, $perms, $err) =
		($args{dir}, $args{name}, $args{path}, $args{mode},
		 $args{vol}, $args{mask}, $args{perms}, $args{err} );

	if ($path) {
	        ($dir, $name) = ($path =~ m%(.*)[\/]([^\/]*)$%);
	} elsif (!$name || !$dir) {
		croak("fioOpen: Either 'path' or both of 'dir' and 'name' arguments must be given");
	} else {
		my ($sep) = ($dir =~ m%([\\/])%);
		$path = $dir . $sep . $name;
	}

	#$err = 0      unless defined($err);
	#$mode = "+<"  unless defined($mode);
	#$perms = 0666 unless defined($perms);

	my $fio			= {};
	$fio->{path} = $path;
	$fio->{name} = $name;
	$fio->{dir} = $dir;

	$fio->{vol} = $vol;		   #XXX probably dont need this
	$fio->{vol_path} = $fio->{path};   #XXX probably dont need this
	$fio->{vol_path} =~ s/\Q$dir\E//;

	$fio->{offset}	= 0;			# current offset
	bless($fio);

	logTrc("open: $fio->{path} mode=$mode mask=$mask "
		. "perms=$perms err=$err");

	# if user wants specific umask to execute for open, do that
	my $umask = umask();
	if (defined($mask)) {
		umask($mask)
			or die("could not change umask to $mask $!");
	}

	# Perl file handle
	$fio->{fh} = new IO::File($fio->{path}, $mode, $perms);

	if (!defined($fio->{fh})) {
		if ($err) {
			_process_error($!, $err, "open file $fio->{path}:");
			return undef;
		} else {
			croak("could not open $fio->{path} $!");
		}
	}

	#if we tweaked with mask, set it back
	umask($umask) if (defined($mask));

	# replacing "setbuf(0)" with "autoflush(1)", since "setbuf"
	# fails on simulators with the introduction of Perl 5.8
	$fio->{fh}->autoflush(1);

	$fio->Stat();

	return ($fio);
}



# ------------------------------------------------------------------------

#
#  Internal subroutines, not exported
#


sub cvt_size_to_bytes
{
	# convert_to_bytes($orig_size);
	my ($orig_size) = @_;
	if ($orig_size !~ /^(\d+)([kmgtp]?)$/) {
		die("cvt_size_to_bytes:  Bad size format: \"$orig_size\"");
	}
	my ($base, $suff) = ($1, $2);
	if (!$suff) {
		# $base *= 1;
	} elsif ($suff =~ /k/i) {
		$base *= 1024;
	} elsif ($suff =~ /m/i) {
		$base *= 1024*1024;
	} elsif ($suff =~ /g/i) {
		$base *= 1024*1024*1024;
	} elsif ($suff =~ /t/i) {
		$base *= 1024*1024*1024*1024;
	} elsif ($suff =~ /p/i) {
		$base *= 1024*1024*1024*1024*1024;
	} else {
		die("cvt_size_to_bytes:  Bad size format: \"$orig_size\"");
	}
	return($base);
}


sub DESTROY($)
{
	my ($fio) = @_;

	my $exc_state = $@;

	logTrc("destroy handle $fio->{name}");
	$fio->Close();

	$@ = $exc_state;
}

# ------------------------------------------------------------------------

## @name	::Error
## @summary	Report error on a file
## @arg		op-name required
## @arg		msg required
## @arg         err  -optional
sub Error($$$)
{
	my ($fio, $op, $err)	= @_;

	_process_error($err, 0,
		       "$op failed, name=$fio->{path}"
		       . " off=$fio->{offset} size=$fio->{Stat}{size}\n");
}

## @name	::Open
## @summary	(Re)Open existing file handle
## @arg		mode optional "+>" trunc+write+read
sub Open($;$)
{
	my ($fio, $mode) = @_;
	$mode = "+<" unless defined($mode);

	$fio->{fh}->open($fio->{path}, $mode)	or $fio->Error("open", $!);
}

## @name	::Close
## @summary	Close file handle

             #XXX NEED TO PORT IT RIGHT!
sub Close($)
{
	my ($fio)	= @_;
=pod
XXX allegedly Solaris hangs on a close of a non-existent file
	my $file = $fio->{path};
	#With ROD/PFS file might not even exist. Surprize!
	if (! (-e $file)) {
	    tstlog(-warn, "trying to close filehandle ".
		   "to unexisting file $file");
	    return;
	}
=cut
	
	return unless $fio->{fh};
	logTrc("close: $fio->{path}");
	$fio->{fh}->close();
}

## @name	::Unlink
## @summary	Unlink file
## @arg         err optional error code
##
## N.B. this may not quite do what you expect - since the file is open
## over NFS this will likely do a rename (see FileUnlink)
##

        #XXX NEED TO PORT IT RIGHT!
sub Unlink($;$)
{
	my ($fio, $err)	= @_;
	$err = 0 unless $err;

	logTrc("unlink: $fio->{path}");
	my $rv = unlink($fio->{path});
	if ($err || ($rv == 0)) {
		_process_error($!, $err, "unlink $fio->{path}");
	}
}

## @name	::Rename
## @summary	Renames file
## @arg         new_name required
## @arg         err optional error code
sub Rename($$;$)
{
	my ($fio, $new_name, $err) = @_;

	$err = 0 unless $err;
	my $new_path = $fio->{path};
	my $vol = $fio->{vol};
	$new_path =~ s/\Q$fio->{name}\E/$new_name/;

	logTrc("rename $fio->{path} to $new_path");

	my $rv = rename("$fio->{path}", $new_path);
	if ($err || ($rv == 0)) {
		_process_error($!, $err, "rename $fio->{path}");
		return;
	}

	$fio->{name} = $new_name;
	$fio->{path} = $new_path;
		
	#XXX TODO assume we are in TEST DIR or its subdirs for now
	#$fio->{vol_path} = $fio->{path};
	#$fio->{vol_path} =~ s/\Q$vol->{mountPoint}\E//;
}

## @name	::Stat
## @summary	Get stats for file
## @notes
##	Set values for $f->{Stat}{dev ino mode nlink uid gid ...}
sub Stat($)
{
	my ($fio)	= @_;

	my @stat = $fio->{fh}->stat()		or $fio->Error("stat", $!);

	my @yuk = qw( dev ino mode nlink uid gid rdev size atime mtime ctime io_size blocks );
	for (0..$#yuk) {
		$fio->{Stat}{$yuk[$_]} = $stat[$_];
	}

	logTrc("stat: name=$fio->{path}"
		. " ino=$fio->{Stat}{ino}"
		. " size=$fio->{Stat}{size}"
		. " nlink=$fio->{Stat}{nlink}");

	return ( { %{$fio->{Stat}} } );
}

## @name     ::StatCheck
## @summary  Check stat data structure for specific fields
## @arg      opts-stat elelments that need to be verified. 
##                required
sub StatCheck($$)
{
	my ($fio, $opts) = @_;
	
	# stat it, just in case
	$fio->Stat();

	foreach my $key (keys(%$opts)) {
		if (!defined($fio->{Stat}{$key})) {
			croak("unknown Stat key $key");
		}
		if ($fio->{Stat}{$key} ne $opts->{$key}) {
			croak("$fio->{path} $key expected $opts->{$key}"
				. " but got $fio->{Stat}{$key}");
		}
	}
}

## @name	::Info
## @summary	Create info about file
## @return	String with info about file
sub Info($)
{
	my ($fio)	= @_;
	my $vol		= $fio->{vol};

	$fio->Stat();
	my $info = "--- File Info"
		. " name=$fio->{path}"
		. " ino=$fio->{Stat}{ino}"
		. " size=$fio->{Stat}{size}"
		. " nlink=$fio->{Stat}{nlink}\n"
		. "VBNs:"
		. $vol->{filer}->filer_say(
			"wafl showvbns /vol/$vol->{name}/$fio->{name}");
	return ($info);
}

## @name	::Seek
## @summary	Seek to offset in file
## @arg		offset required

           #XXX NEED TO PORT IT RIGHT!
sub Seek($$)
{
	my ($fio, $off)	= @_;

	logTrc("seek: name=$fio->{path} off=$off");
	logDbg("fd=" .fileno($fio->{fh}));

	$fio->{fh}->seek($off, 0)		or $fio->Error("seek", $!);
	$fio->{offset} = $off;
}


my $BUFSIZ = 4096;

# Verify content of block with default sequence
sub vfyDefault($$$)
{
	my ($off, $buf, $len) = @_;

	my $rseq = substr($buf, 0, 1);	# first byte
	my $xseq = &uSeqStr($off, 1);	# seq based on offset
	($rseq eq $xseq)
		or croak("seq mismatch \"$rseq\" != \"$xseq\"");
}

## @name	::Read
## @summary	Read and verify general file data
## @arg		length required
## @arg		verify optional sub callback(off, buf, len)
## @example
##	$f->Read(1024, \&mycallback);

             #XXX NEED TO PORT IT RIGHT
sub Read($;$)
{
	my ($fio, $len, $verify)	= @_;

	logTrc("read: name=$fio->{path} len=$len off=$fio->{offset}");
	logDbg("fd=" .fileno($fio->{fh}));

	$verify = \&vfyDefault unless defined($verify);

	while ($len) {
		my $buf;
		$fio->{fh}->read($buf, ($len > $BUFSIZ) ? $BUFSIZ : $len);
		my $rlen = length($buf);

		$fio->Error("read", ($! ? $! : "EOF")) if ($rlen == 0);

		&$verify($fio->{offset}, $buf, $rlen);

		$len -= $rlen;
		$fio->{offset} += $rlen;
	}
}

## @name	::ReadSeq
## @summary	Read and verify sequence file data
## @arg		length required
## @arg		sequence optional offset into sequence

           #XXX NEED TO PORT IT RIGHT!
sub ReadSeq($$;$)
{
	my ($fio, $len, $seq)	= @_;

	logTrc("read seq: name=$fio->{path} len=$len off=$fio->{offset}");

	$seq = 0 unless (defined($seq));
	uReadSeq($fio->{fh}, $seq, $len);
	$fio->{offset} += $len;
}

## @name	::Write
## @summary	Write general file data
## @arg		data required
## @arg		length required


      #XXX NEED TO PORT IT RIGHT!
sub Write($$$)
{
	my ($fio, $buf, $len)	= @_;

	logTrc("write: name=$fio->{path} len=$len off=$fio->{offset}");
	logDbg("fd=" .fileno($fio->{fh}));
	#logDbg("\"" . substr($buf, 0, $len) . "\"");

	$fio->{fh}->write($buf, $len)	or $fio->Error("write", $!);
	$fio->{offset} += $len;
}

## @name	::WriteSeq
## @summary	Write sequence file data
## @arg		length required
## @arg		sequence optional offset into sequence
sub WriteSeq($$;$$)
{
	my ($fio, $len, $seq, $err)	= @_;

	logTrc("write seq: name=$fio->{path} len=$len off=$fio->{offset}");

	$seq = 0 unless (defined($seq));
	my $ret = uSendSeq($seq, $len, $fio->{fh}, $err);
	if ($err) {
		_process_error($ret, $err,
			"write to $fio->{path} $len");
		return;
	}

	$fio->{offset} += $len;
}

## @name	::Truncate
## @summary	Truncate file to size
## @arg		size optional
sub Truncate($;$)
{
	my ($fio, $len)	= @_;

	$len = 0 unless (defined($len));
	logTrc("truncate name=$fio->{path} len=$len");

	$fio->{fh}->truncate($len)	or $fio->Error("truncate", $!);
}

## @name	::Chown
## @summary	Change file's owner
## @arg		owner required
sub Chown($$)
{
	my ($fio, $owner) = @_;
	my $cmd=$main::SUDO_PATH . " chown $owner $fio->{path}";
	uSystem($cmd);
}


## @name	::Chgrp
## @summary	Change file's group
## @arg		gid required
sub Chgrp($$)
{
	my ($fio, $gid) = @_;
	my $cmd=$main::SUDO_PATH . " chgrp $gid $fio->{path}";
	uSystem($cmd);
}

## @name	::Link
## @summary	Create link to a file
## @arg		dest-name required
sub Link($$;$)
{
	my ($fio, $name, $err)	= @_;

	logTrc("link name=$fio->{path} link=$fio->{vol}{testDir}/$name");
	my $rv = link($fio->{path}, "$fio->{vol}{testDir}/$name");
	if ($err || ($rv == 0) )  {
		_process_error($!, $err,
			"link $fio->{vol}{testDir}/$name to $fio->{path}");
	}
}

## @name	::SymLink
## @summary	Create symbolic link to a file
## @arg		dest-name required
## @arg         err-optional
sub SymLink($$;$)
{
	my ($fio, $name, $err)	= @_;
	$err = 0 unless $err;

	logTrc("symlink name=$fio->{path} link=$fio->{path}.$name");
	my $rv = symlink($fio->{path}, "$fio->{path}.$name");
	if ($err || ($rv == 0) )  {
		_process_error($!, $err, "symlink $fio->{path}.$name "
			       ." to $fio->{path}");
	}
}

## @name	::Chmod
## @summary	Change mode of file
## @arg		mode required
sub Chmod($$;$)
{
	my ($fio, $mode, $err)	= @_;
	$err = 0 unless $err; # OS error code, not perl
	
	logTrc("chmod name=$fio->{path} mode=$mode");
	my $rv = chmod($mode, $fio->{path});
	# if expected error or failed, process err
	if ($err || $rv == 0) {
		_process_error($!, $err, "chmod $fio->{path}");
	}
}

## @name	::_process_error
## @summary     internal error processing function. useful when
##              not operating on fio Objects
## @arg		got required
## @arg         expected required
## @arg         op optional.
sub _process_error($$;$)
{
	my ($got, $expected, $op) = @_;
	$op = "" unless $op;
	$expected = 0 unless $expected;

	if ($got && ($got != $expected)) {
		# got unexpected error
		croak("$op got ".sprintf("%d", $got)
			 ." ($got) instead of expected $expected ");
	}
	
	if ($expected && !$got) {
		# expected error, but did not get it
		croak("$op succeeded when expected error $expected");
	}
	
	logTrc("op: $op expected $expected, got $got");
}

# ------------------------------------------------------------------------

## @name	::SameAs
## @summary	Verify similarity of files
## @arg		other-file required
## @arg		optional parts to ignore
sub SameAs($$;$)
{
	my ($f1, $f2, $ignore)	= @_;
	$f1->Stat();
	$f2->Stat();

	for (keys(%{$f1->{Stat}})) {
		next if $_ eq "dev";	# may not match
		next if $_ =~ $ignore;
		$f1->{Stat}{$_} == $f2->{Stat}{$_}
			or croak("files $f1->{path} $f2->{path} differ: $_"
				. " $f1->{Stat}{$_} != $f2->{Stat}{$_}");
	}
}

## @name     ::VfyFileAbsent
## @summary  Verify that the file does not exist
## @arg      path-path to a file. required
## @arg      opts-optional
sub VfyFileAbsent($;$)
{
	my (%args) = validate (
			       @_,
			       {
				path=>{type=>SCALAR},
				err_msg=>{type=>SCALAR,
					  optional=>1,
					  default=>''
					 }
			       }
			      );

	my ($path, $err_msg) = ($args{path}, $args{err_msg} );
	
	if (! -e $path) {
		return;
	}
	croak("file $path exists. $err_msg");
}

## @name     ::VfyFilePresent
## @summary  Verify that the file exists
## @arg      path-path to a file. required
## @arg      opts-optional
sub VfyFilePresent($;$)
{
	my ($path, $opts) = @_;
	
	if (-e $path) {
		return;
	}
	my $extra_info = $opts->{err_msg} || "";
	croak("file $path does not exist. $extra_info");
}


## @name	::fioDiffFileContents
## @summary	compare two files
## @description
##	Read two files into memory and
##	compare them line by line.
## @arg		file1 required
## @arg		file2 required
sub fioDiffFileContents($$)
{
	my ($file1, $file2) = @_;

	my $line = 0;
	my ($f1_contents, $f2_contents);

	# it would be simpler to read files into memory, but they could
	# be quite large. also, lets not depend on local diff utility.
	# we skip checking file size, since even if the sizes are different
	# we want to see first found difference
	open(S_FH, $file1)
		or croak("could not open $file1");
	open(P_FH, $file2)
		or croak("could not open $file2");
	while ((!eof(P_PH)) || (!eof(S_FH))) {
		$line++;
		$f1_contents = <P_FH>;
		$f2_contents = <S_FH>;
		if ($f1_contents ne $f2_contents) {
			close(S_FH);
			close(P_FH);
			croak("fioDiffFiles:files are different at "
				 . " line $line:\n"
				 . "$file1:$line - $f1_contents \n"
				 . "$file2:$line - $f2_contents \n");
		}
	}
	close(S_FH) or croak("could not close $file1: $!");
	close(P_FH) or croak("could not close $file2: $!");
}

## @name	::fioRCmpSnapTestDirs
## @summary 	Recursively compare test directories of two volumes
## @arg 	vol1 required
## @arg         vol2 required
sub fioRCmpTestDirSnaps($$)
{
	my ($vol1, $vol2) = @_;
	return fioRCmpDir("$vol1->{testDir}/.snapshot",
			  "$vol2->{testDir}/.snapshot");
}

## @name	::fioRCmpTestDirs
## @summary 	Recursively compare test directories of two volumes
## @arg 	vol1 required
## @arg         vol2 required
sub fioRCmpTestDirs($$;$)
{
	my ($vol1, $vol2) = @_;

	return fioRCmpDir($vol1->{testDir},
			  $vol2->{testDir});
			
}

## @name	::fioRCmpDir
## @summary	Recursively compare two directories
## @description execute 'ls -lAR' on two directories
##	writing results to outfiles. compare outfiles.
## @arg		dir_src required
## @arg		dir_dst required
## @arg		tmp_dir optional
sub fioRCmpDir($$;$;$)
{
	my ($dir_src, $dir_dst, $tmp_dir) = @_;

	if (!defined $tmp_dir) {
		$tmp_dir = defined $ENV{TMPDIR} ? $ENV{TMPDIR} : "/tmp";
	}

	my $out_src = "$tmp_dir/out_src.$$";
	my $out_dst = "$tmp_dir/out_dst.$$";

	# LINUX ls hack
	# clock skew can result in a different format being reported
	# in the date field
	sleep(1);
	system("cd $dir_src && ls -lAR > $out_src") == 0
		or croak("could not list $dir_src "
			    . "and redirect to $out_src");

	system("cd $dir_dst && ls -lAR > $out_dst") == 0
		or croak("could not list $dir_dst "
			    . "and redirect to $out_dst");

	fioDiffFileContents($out_src, $out_dst);

	unlink($out_src, $out_dst);
}

# ------------------------------------------------------------------------

## @name	::fioMkDir
## @summary 	Create a directory
## @arg 	dir required
## @arg 	mode optional eg 0777
## @arg         err optional error code. (only when mkdir fails)
# XXX iffy - new object? overload file?
sub fioMkDir($$;$$)
{
	my ($vol, $name, $mode, $err)	= @_;
	$mode = 0777	unless $mode;
	$err = 0        unless $err;
	my $full_name =  "$vol->{testDir}/$name";
	
	logTrc("mkdir name=$full_name mode=".sprintf("%o", $mode));
	# this is not as clean as it should be
	# (retval of mkdir() vs $!)
	my $rv = mkdir($full_name);
	if ($err || $rv == 0) {
		_process_error($!, $err, "mkdir $full_name");
		return; # nothing to chmod, so bail
	}

	# mkdir() takes a $mode parameter that gets 'and'ed
	# with umask, so instead of mucking with umask, just chmod it
	chmod($mode, $full_name)
		or croak("could not change permissions of "
			    ."$full_name to ".sprintf("%o", $mode));
}

## @name	::fioRmDir
## @summary 	Remove a subdirectory of volume's test dir
## @arg 	vol required
## @arg 	name required
## @arg         err optional. expected error (only when rmdir fails)
sub fioRmDir($$;$)
{
	my ($vol, $name, $err) = @_;
	my $full_name = "$vol->{testDir}/$name";
	
	logTrc("rmdir $full_name");
	
	my $rv = rmdir("$full_name");
	if ($err || $rv == 0) {
		_process_error($!, $err, "rmdir $full_name");
	}
}

## @name	::VfyDirAbsent
## @summary 	Verify that a directory does not exist 
##              (assumes subdir of test dir)
## @arg 	vol required
## @arg 	name required
sub VfyDirAbsent($$)
{
	my ($vol, $name) = @_;
	if (-d "$vol->{testDir}/$name") {
		croak("directory $vol->{testDir}/$name is presnet");
	}
}

## @name	::VfyDirPresent
## @summary 	Verify that a directory exists (assumes subdir of test dir)
## @arg 	vol required
## @arg 	name required
sub VfyDirPresent($$)
{
	my ($vol, $name) = @_;
	if (! -d "$vol->{testDir}/$name") {
		croak("directory $vol->{testDir}/$name is absent");
	}		

}

## @name	::fioMkFifo
## @summary 	Create a pipe
## @arg 	name required
## @arg 	err optional
## @arg         options optional 
sub fioMkFifo($;$%)
{	
	# XXX REMOVE THIS UNREADABLE GIBBERISH
	# XXX TODO need to change Daisuke's validate_mixed() 
	# so that later we can substitte $optsions HASHREF instead
	# of the %otpions HASH to make things consistent
	my ($name, $err, %options) =
		validate_mixed(@_,
		[
			{ type=>SCALAR },
			{ type=>SCALAR, optional=>1, default=>0 }
		],
		{
			mask=> {type=>SCALAR, optional=>1, default=>undef},
			mode=> {type=>SCALAR, optional=>1, default=>0666}
		}
		);
	my $opts = \%options;
	my $mode = $opts->{mode};
	my $mask;
	my $old_umask;
	
	# tweak mask if need to
	if (defined ($mask = $opts->{mask}))  {
		$old_umask = umask();
		umask($mask) 
			or croak("could not change umask to $mask $!");
	}

	# create fifo, process error if need be
	my $rv = POSIX::mkfifo($name, $mode);
	if (!defined($rv) || $err) {
		_process_error($!, $err, "fioMkFifo $name");
	}
	
	# if we tweaked with mask, set it back
	if (defined($mask)) {
		umask($old_umask) 
			or  croak("could not change umask to $old_umask $!");
	}
}

## @name	::FileUnlink
## @summary	Unlink file
## @arg         err optional error code
sub FileUnlink($;$)
{
	my ($filename, $err)	= @_;
	$err = 0 unless $err;

	logTrc("unlink: $filename");
	my $rv = unlink($filename);
	if ($err || $rv == 0) {
		_process_error($!, $err, "unlink $filename");
	}
}

## @name	::fioLock
## @summary	create a file region lock
## @notes
## Mimics system call, fcntl(), via Perl's Fcntl::fcntl() to
## create a file region lock. Cannot use Perl's Fcntl::flock() as
## that will lock the entire file, rather than a region in the file.
##
## The flock struct must be observed when this method is used on different
## OS. If the information is not packed correctly in $lock, file may
## not be locked properly.
##
## Retry failure for 3 times before giving up.
##
## Lock usage: 
##  General: $fio->fioLock($type, $whence, $start, $len);
##           $type   - F_RDLCK  | F_WRLCK  | F_UNLCK
##           $whence - SEEK_SET | SEEK_CUR | SEEK_END
##           $start  - offset in bytes, relative to $whence
##           $len    - length in bytes
##
##  Write exclusive lock: 
##    $fio->fioLock(F_WRLCK, SEEK_SET, 0, 4096);
##  Read shared lock:
##    $fio->fioLock(F_RDLCK, SEEK_CUR, 0, 1024);
##  Unlock:
##    $fio->fioLock(F_UNLCK, SEEK_END, 0, 2096);
 

               #XXX NEED TO PORT IT RIGHT!

sub fioLock($$;$$$$)
{
	my ($fio, $type, $whence, $start, $len, $err) = @_;
	$whence = SEEK_SET	unless $whence;
	$start = 0		unless $start;
	$len = 0	        unless $len;	# entire file
	$err = 0		unless $err;

	# typical flock struct found on Solaris, Linux, and OSF1
	# do a "grep 'struct flock' /usr/include/*/*" to check
	#
	# struct  flock   {
	#    short   l_type;
	#    short   l_whence;
	#    off_t   l_start;
	#    off_t   l_len;          /* len = 0 means until end of file */
	#    pid_t   l_pid;
	# };
	#
	# only using first 4 fields (no pid)
	my $flock_struct = "ssll";
	my @flock = ($type, $whence, $start, $len);
	my $lock = pack($flock_struct, @flock);

	my $retry = 0;
	my $rv;

	logTrc("fcntl name=$fio->{path} type=$type whence=$whence "
		. "start=$start len=$len err=$err");

	while ($retry < 3) {
		# perl's fcntl returns an undef value if fails
		$rv = fcntl($fio->{fh}, F_SETLKW, $lock) or $rv = -1;
		last if ($rv != -1);
		logWarn("$fio->{path}: Encountered file region locking error. "
			. "The rest of the test may fail due to this error.");
		++$retry;
	}
	if ($err || $rv != 0) {
		_process_error($!, $err, "flock $fio->{path}");
	}
}

# PORTING THE FUNCTION fioLock(). IN PROGRESS
sub fioLock_new($$;$$$$)
{
	my ($fio, %args) = validate_mixed(
			     @_,
			     [ {isa => 'FS::FileIO' } ],
			     {	
			       lock=>{type=>SCALAR}, #XXX make more robust later
			       whence=>{type=>SCALAR,
					optional=>1,
					default=>SEEK_SET
				       },
			       start=>{type=>SCALAR,
				       optional=>1,
				       default=>0
				      },
			       len=>{type=>SCALAR,
				     optional=>1,
				     default=>0
				    }
			      }
			     );

	my ($handle, $path, $type, $whence, $start, $len, $err) =
		($fio->{fh}, $fio->{path}, $args{type}, $args{whence},
		 $args{start}, $args{len}, $args{err});

	$whence = SEEK_SET	unless $whence;
	$start = 0		unless $start;
	$len = 0	        unless $len;	# entire file
	$err = 0		unless $err;

	# typical flock struct found on Solaris, Linux, and OSF1
	# do a "grep 'struct flock' /usr/include/*/*" to check
	#
	# struct  flock   {
	#    short   l_type;
	#    short   l_whence;
	#    off_t   l_start;
	#    off_t   l_len;          /* len = 0 means until end of file */
	#    pid_t   l_pid;
	# };
	#
	# only using first 4 fields (no pid)
	my $flock_struct = "ssll";
	my @flock = ($type, $whence, $start, $len);
	my $lock = pack($flock_struct, @flock);

	my $retry = 0;
	my $rv;

	logTrc("fcntl path=$path handle=$handle type=$type whence=$whence "
		. "start=$start len=$len err=$err");

	while ($retry < 3) {
		# perl's fcntl returns an undef value if fails
		$rv = fcntl($handle, F_SETLKW, $lock) or $rv = -1;
		last if ($rv != -1);
		logWarn("$path: Encountered file region locking error. "
			. "The rest of the test may fail due to this error.");
		++$retry;
	}
	if ($err || $rv != 0) {
		_process_error($!, $err, "flock $path");
	}
}

## @name	::make_random_sequence
## @summary	create random sequence of characters
## @arg         len - how long of a sequence
## @arg         %args Optional. Hash of optional named params.
sub make_random_sequence
{
	my ($len, %args) = 
		validate_mixed(
			       @_,
			       [ {type=>SCALAR }],
			       {
				ascii_offset => { type => SCALAR,
						  default => 0,
						  # got to look like a number
						  regex => qr/^\d+$/
						},
				seed => {type => SCALAR,
					 optional => 1,
					 regex => qr/^\d+$/	
					}
			       }
			      );
	
	my $ascii_table_size = 126;
	my ($ascii_offset, $seed) = ($args{ascii_offset}, $args{seed});
	if (!defined($seed)) {
		my ($sec, $msec) = gettimeofday();
		$seed = $sec + $msec;
	}

	logDbg("make_random_sequence - offset: $ascii_offset, seed: $seed");

	if ($ascii_offset < 0
	    || $ascii_offset > $ascii_table_size) {
		die "illegal ascii offset $ascii_offset"
	}
	
	$seed = $seed % $ascii_table_size;
	
	my $i;
	my $buf = "";
	my $next_char = $ascii_offset;


	# NOTE that we are accessing the characters in the ascii
	# table by their decimal index, not octal
	for (0..($len-1)) {
		
		# append the chozen character to the buffer
		$buf .= chr($next_char);
		
		# chooze every Nth char, wrapping around
		# ascii table
		$next_char += $seed;
		if ($next_char > $ascii_table_size) {
			$next_char -= $ascii_table_size;
		}
	}
	return $buf;
}


## @summary	reads file in chunks and compares each chunk
##              to buffer
## @arg         buffer - buffer that is expected
## @arg         opts - hash of opts. can contain
##                    offset - where to start reading from
##                    times- how many chunks to read
sub read_file_buf
{
	my ($fio, $buffer, %opts) = @_;
	my $times = $opts{times} || 1;
	my $offset = defined($opts{offset}) ? $opts{offset} : $fio->{offset};

	logDbg("read_file_buf offset: $offset, buffer: $buffer");
	$fio->Seek($offset);
	my $read_in = "";
	for (1..$times) {
		$fio->{fh}->read($read_in, length($buffer));
	
		if ($buffer ne $read_in) {
		       die("expected $buffer, \n\n got $read_in");
		}
		$read_in = "";
	}
}



## @name	::sysSeek
## @summary	Seek to offset in file using 'sysseek'
## @arg		offset required
sub sysSeek($$)
{
	my ($fio, $fpos) = @_;

	logDbg("sysSeek: name=$fio->{path}  file_pos=$fpos   fd=".fileno($fio->{fh}));
	sysseek($fio->{fh}, $fpos, 0)
	    or $fio->Error("sysseek", $!);
	$fio->{offset} = $fpos;
	return($fpos);
}


## @name	::sysTell
## @summary	Returns current 'sysseek' compatible offset in file.
sub sysTell($$)
{
	my ($fio) = @_;

	my $fpos = sysseek($fio->{fh}, 0, 1)
	    or $fio->Error("sysseek", $!);
	$fio->{offset} = $fpos;
	logDbg("sys%ell: name=$fio->{path}  file_pos: $fpos  fd=" .fileno($fio->{fh}));
	return($fpos);
}


## @name	::ReadData
## @summary	Read and return arbitrary file data using 'sysread'
##
## @note     All args  are "keyword".
#a		Either an 'fio' or or an 'path' arg is required.
##			-- If 'fio' arg is given, it is an FileIO object ref. It must be
##			   opened for both reading and writing (either "+<" or "+>"). The
##			   fio will remain open after exit
##			-- If 'path' arg is given, it is an absolote path to an existing file,
##			   with permissions for both reading and writing.
##
## @arg		path		Optional (see note above) Absolute path to file.
## @arg		fio		Optional (see note above) FileIO object ref.
## @arg		length		Required.  Length of deata to read.
## @arg         blksz		Optional. I/O block size to use. I/O ops are split into
##				    a sequence of smaller ones, such that all ops have
##				    length blksz or less. If 0 (zero), then I/O ops are
##				    not split.
## @arg         buffer_ref	Optional. SCALAR REF to be used for storage of
##				    data read. (ref to string).
##				    If 'undef', '0', or "", then new storage is used.
## @arg		buffer_offset   Optional. Offset into buffer to begin copying to.
##                          	    Default is zero. Ignored unless "buffer_ref" is specified.
## @arg		file_pos    	Optional. If given, then 'sysseek' will be done to
##				    position the file at the given file_pos before
##				    doing the 'write'. Default is 'undef' -- no
##				    'sysseek is done.

sub ReadData(%)
{
	my (%args) = validate (
			    @_,
			    {
				path		=> {type=>SCALAR, optional=>1},
			        fio		=> {type=>OBJECT, optional=>1},
				length		=> {type=>SCALAR, default=>0, optional=>1},
				blksz		=> {type=>SCALAR, default=>0, optional=>1},
				buffer_ref	=> {type=>SCALARREF|UNDEF, default=>undef, optional=>1},
				buffer_offset	=> {type=>SCALAR, default=>0, optional=>1},
				file_pos	=> {type=>SCALAR, default=>undef, optional=>1},
			    }
		     );

	my ($path, $fio, $length, $blksz, $buffer_ref, $buffer_offset, $file_pos) =
		($args{path}, $args{fio}, $args{length}, $args{blksz},
		    $args{buffer_ref}, $args{buffer_offset}, $args{file_pos});

	$blksz ||= $length;
	my $local_file = 0;
	if ($fio) {
		$path = $fio->{path};
	} else {
		if (!$path) {
			croak("ReadData: One of 'fio' or 'path' args must be given");
		} elsif (! -e $path) {
			croak("ReadData: Path \"$path\" not found");
		}
		$local_file = 1;
        	my $mode = "<";
	        my ($dir, $name) = ($path =~ m%(.*)[\/]([^\/]*)$%);
		$fio = fioOpen( name=>$name,
				dir=>$dir,
				mode=>$mode);
		if (!$fio) {
			croak("ReadData: fioOpen failed: $@");
		}
	}

	if (defined($file_pos)) {
		$fio->sysSeek($file_pos);
	}

	logTrc("ReadData:  name=$fio->{path}  length=$length  buffer_offset=$buffer_offset  ".
		"filepos=$fio->{offset}  blksz=$blksz");

	my $rem_length = $length;
	my $length_read = 0;
	while ($rem_length) {
		my $to_read = ($blksz < $rem_length) ? $blksz : $rem_length;
		my $rdlen = FS::Util::Utilities::uReadData(
						             rd=>$fio->{fh},
						             length=>$to_read,
						             buffer_ref=>$buffer_ref,
						             buffer_offset=>$buffer_offset);
		$fio->{offset} += $rdlen;
		$rem_length -= $rdlen;
		$length_read += $rdlen;
		if ($rdlen == 0) {
			logWarn("ReadData:  Zero bytes read, expected to read: $to_read: $!");
			last;
		} elsif ($rdlen != $to_read) {
			logWarn("ReadData:  Read $rdlen bytes, expected: $to_read");
		}
	}

	if ($local_file) {
		$fio->Close();
	}
	return($length_read);
}


## @name	::ReadVerifyData
## @summary	Read and verify arbitrary file data
##
## @note     All args  are "keyword".
#a		Either an 'fio' or or an 'path' arg is required.
##			-- If 'fio' arg is given, it is an FileIO object ref. It must be
##			   opened for both reading and writing (either "+<" or "+>"). The
##			   fio will remain open after exit
##			-- If 'path' arg is given, it is an absolote path to an existing file,
##			   with permissions for both reading and writing.
##
## @arg		path		Optional (see note above) Absolute path to file.
## @arg		fio		Optional (see note above) FileIO object ref.
## @arg		data		Required. Data to write.
## @arg		length		Optional. Length of deata to write/read. Default is length(data)
## @arg		data_offset	Optional. Offset into 'data'. Default is zero.
## @arg		file_pos    	Optional. If given, then 'sysseek' will be done to
##				    position the file at the given file_pos before
##				    doing the 'write'. Default is 'undef' -- no
##				    'sysseek is done.
## @arg         blksz		Optional. I/O block size to use. I/O ops are split into
##				    a sequence of smaller ones, such that all ops have
##				    length blksz or less. If 0 (zero), then I/O ops are
##				    not split.
## @arg         buffer_ref	Optional. SCALAR REF to be used for temp. storage of
##			            data read. (ref to string).
##				    If 'undef', '0', or "", then new storage is used.
## @arg         croak_on_miscompare     Optional. If '1', then croak on error.
##					    If '0', then return 0 on error. Default is '1'.
## @return			Returns '1' if verification succeeded, '0' if verification failed.
##				'croaks' if 'read' failed, or if verification failed and
##				'croak_on_miscompare' is set.
sub ReadVerifyData(%)
{
	my (%args) = validate (
			    @_,
			    {
				path		=> {type=>SCALAR, optional=>1},
				fio		=> {type=>OBJECT, optional=>1},
				data		=> {type=>SCALAR},
				length		=> {type=>SCALAR, default=>0, optional=>1},
				data_offset	=> {type=>SCALAR, default=>0, optional=>1},
				file_pos	=> {type=>SCALAR, default=>undef, optional=>1},
				blksz		=> {type=>SCALAR, default=>0, optional=>1},
				buffer_ref	=> {type=>SCALARREF|UNDEF, default=>undef, optional=>1},
				croak_on_miscompare => {type=>SCALAR, default=>1, optional=>1},
			    }
		     );

	my ($path, $fio, $data, $length, $data_offset, $file_pos, $blksz,
		$buffer_ref, $croak_on_miscompare) =
		($args{path}, $args{fio}, $args{data}, $args{length},
		     $args{data_offset}, $args{file_pos}, $args{blksz},
		    $args{buffer_ref}, $args{croak_on_miscompare});

	my $data_len = length($data);
	my $max_length = ($data_len - $data_offset);
	$max_length = 0 if ($max_length < 0);
	$length = $max_length if (!$length || ($length > $max_length));
	$blksz ||= $length;

	my $local_file = 0;
	if ($fio) {
		$path = $fio->{path};
	} else {
		if (!$path) {
			croak("ReadVerifyData: One of 'fio' or 'path' args must be given");
		} elsif (! -e $path) {
			croak("ReadVerifyData: Path \"$path\" not found");
		}
		$local_file = 1;
        	my $mode = "<";
		my $perms = 0777;
	        my ($dir, $name) = ($path =~ m%(.*)[\/]([^\/]*)$%);
		$fio = fioOpen( name=>$name,
				dir=>$dir,
				mode=>$mode,
				perms=>$perms);
		if (!$fio) {
			croak("ReadVerifyData: fioOpen failed: $@");
		}
	}

	if (defined($file_pos)) {
		$fio->sysSeek($file_pos);
	}

	logTrc("ReadVerifyData:  name=$fio->{path}  length=$length  ".
		    "data_offset=$data_offset   filepos=$fio->{offset}  ".
		    "blksz=$blksz   croak_on_miscompare=$croak_on_miscompare");

	my $rem_length = $length;
	my $length_read = 0;
	while ($rem_length) {
		my $to_read = ($blksz < $rem_length) ? $blksz : $rem_length;
		my $rdlen;
		my $verify = FS::Util::Utilities::uReadVerifyData(
							            rd=>$fio->{fh},
							            data=>$data,
							            length=>$to_read,
							            data_offset=>$data_offset,
							            buffer_ref=>$buffer_ref,
							            length_read_ref=>\$rdlen,
							            croak_on_miscompare=>$croak_on_miscompare);

		$fio->{offset} += $rdlen;
		$rem_length -= $rdlen;
		$length_read += $rdlen;
		if ($rdlen == 0) {
			logWarn("ReadVerifyData:  Zero bytes read, expected to read: $to_read: $!");
			last;
		} elsif ($rdlen != $to_read) {
			logWarn("ReadVerifyData:  Read $rdlen bytes, expected: $to_read");
		}
		if (!$verify) {
			logWarn("ReadVerifyData: Verify failed");
			if ($croak_on_miscompare) {
				croak("ReadVerifyData: Verify failed");
			}
		}
	}

	if ($local_file) {
		$fio->Close();
	}
	return($length_read);
}


## @name	::WriteData
## @summary	Write arbitrary file data using 'syswrite'
##
## @note     All args  are "keyword".
#a		Either an 'fio' or or an 'path' arg is required.
##			-- If 'fio' arg is given, it is an FileIO object ref. It must be
##			   opened for both reading and writing (either "+<" or "+>"). The
##			   fio will remain open after exit
##			-- If 'path' arg is given, it is an absolote path to an existing file,
##			   with permissions for both reading and writing.
##
## @arg		path		Optional (see note above) Absolute path to file.
## @arg		fio		Optional (see note above) FileIO object ref.
## @arg		data		Required. Data to write.
## @arg		length		Optional. Length of deata to write/read. Default is length(data)
## @arg		data_offset	Optional. Offset into 'data'. Default is zero.
## @arg		file_pos    	Optional. If given, then 'sysseek' will be done to
##				    position the file at the given file_pos before
##				    doing the 'write'. Default is 'undef' -- no
##				    'sysseek is done.
## @arg         blksz		Optional. I/O block size to use. I/O ops are split into
##				    a sequence of smaller ones, such that all ops have
##				    length blksz or less. If 0 (zero), then I/O ops are
##				    not split.
## @return			Returns the length of data written.
sub WriteData(%)
{
	my (%args) = validate (
			    @_,
			    {
				path		=> {type=>SCALAR, optional=>1},
			        #fio		=> {isa => 'FS::FileIO', optional=>1},
				fio		=> {type=>OBJECT, optional=>1},
				data		=> {type=>SCALAR},
				length		=> {type=>SCALAR, default=>0, optional=>1},
				data_offset	=> {type=>SCALAR, default=>0, optional=>1},
				file_pos	=> {type=>SCALAR, default=>undef, optional=>1},
				blksz		=> {type=>SCALAR, default=>0, optional=>1},
			    }
		     );

	my ($path, $fio, $data, $length, $data_offset, $file_pos, $blksz) =
		($args{path}, $args{fio}, $args{data}, $args{length},
		   $args{data_offset}, $args{file_pos}, $args{blksz});

	$length ||= length($data);
	$blksz ||= $length;

	my $local_file = 0;
	if ($fio) {
		$path = $fio->{path};
	} else {
		if (!$path) {
			croak("WriteData: One of 'fio' or 'path' args must be given");
		}
		$local_file = 1;
        	my $mode = "+<";
		my $perms = 0777;
	        my ($dir, $name) = ($path =~ m%(.*)[\/]([^\/]*)$%);
		$fio = fioOpen( name=>$name,
				dir=>$dir,
				mode=>$mode,
				perms=>$perms);
		if (!$fio) {
			croak("WriteData: fioOpen failed: $@");
		}
	}

	if (defined($file_pos)) {
		$fio->sysSeek($file_pos);
	}

	logTrc("WriteData:  name=$fio->{path}  len=$length  data_offset=$data_offset  ".
		   "filepos=$fio->{offset}  blksz=$blksz");

	my $rem_length = $length;
	my $length_written = 0;
	while ($rem_length) {
		my $to_write = ($blksz < $rem_length) ? $blksz : $rem_length;
		my $wlen = FS::Util::Utilities::uSendData(
							   wrt=>$fio->{fh},
							   data=>$data,
							   length=>$to_write,
							   data_offset=>$data_offset);

		$rem_length -= $wlen;
		$length_written += $wlen;
		$fio->{offset} += $wlen;
		if ($wlen == 0) {
			logWarn("WriteData:  Zero bytes written, expected to write: $to_write: $!");
			last;
		} elsif ($wlen != $to_write) {
			logWarn("WriteData:  Wrote $wlen bytes, expected: $to_write");
		}
	}
	if ($local_file) {
		$fio->Close();
	}
	return($length_written);
}


## @name	::WriteVerifyData
## @summary	Write arbitrary file data, read it back, and verify it.
##		File must be open for both reading and writing
##
## @note     All args  are "keyword".
#a		Either an 'fio' or or an 'path' arg is required.
##			-- If 'fio' arg is given, it is an FileIO object ref. It must be
##			   opened for both reading and writing (either "+<" or "+>"). The
##			   fio will remain open after exit
##			-- If 'path' arg is given, it is an absolote path to an existing file,
##			   with permissions for both reading and writing.
##
## @arg		path		Optional (see note above) Absolute path to file.
## @arg		fio		Optional (see note above) FileIO object ref.
## @arg		data		Required. Data to write.
## @arg		length		Optional. Length of deata to write/read. Default is length(data)
## @arg		data_offset	Optional. Offset into 'data'. Default is zero.
## @arg		file_pos    	Optional. If given, then 'sysseek' will be done to
##				    position the file at the given file_pos before
##				    doing the 'write'. Default is 'undef' -- no
##				    'sysseek is done.
## @arg         blksz		Optional. I/O block size to use. I/O ops are split into
##				    a sequence of smaller ones, such that all ops have
##				    length blksz or less. If 0 (zero), then I/O ops are
##				    not split.
## @arg         buffer_ref	Optional. SCALAR REF to be used for temp. storage of
##			            data read. (ref to string).
##				    If 'undef', '0', or "", then new storage is used.
## @arg         croak_on_miscompare    Optional. If '1', then croak on error.
##					    If '0', then return 0 on error. Default is '1'.
## @return			Returns '1' if verification succeeded, '0' if verification failed.
##				'croaks' if 'write' or 'read' failed, or if verification failed and
##				'croak_on_miscompare' is set.
sub WriteVerifyData(%)
{
	my (%args) = validate (
			    @_,
			    {
				fio		=> {type=>OBJECT, optional=>1},
				path		=> {type=>SCALAR, optional=>1},
				data		=> {type=>SCALAR},
				length		=> {type=>SCALAR, default=>0, optional=>1},
				data_offset	=> {type=>SCALAR, default=>0, optional=>1},
				file_pos	=> {type=>SCALAR, default=>undef, optional=>1},
				blksz		=> {type=>SCALAR, default=>0, optional=>1},
				buffer_ref	=> {type=>SCALARREF|UNDEF, default=>undef, optional=>1},
				croak_on_miscompare => {type=>SCALAR, default=>1, optional=>1},
			    }
		     );
	
	my ($fio, $path, $data, $length, $data_offset, $file_pos, $blksz,
		$buffer_ref, $croak_on_miscompare) =
	        ($args{fio}, $args{path}, $args{data}, $args{length},
		    $args{data_offset}, $args{file_pos}, $args{blksz},
		    $args{buffer_ref}, $args{croak_on_miscompare});


	my $data_len = length($data);
	my $max_length = ($data_len - $data_offset);
	$max_length = 0 if ($max_length < 0);
	$length = $max_length if (!$length || ($length > $max_length));
	$blksz ||= $length;
	if (!defined($path) && !defined($fio)) {
		croak("WriteVerifyData: One of 'fio' or 'path' args must be given");
	}
	my $local_file = 0;
	if ($fio) {
		$path = $fio->{path};
	} else {
		if (!$path) {
			croak("WriteVerifyData: One of 'fio' or 'path' args must be given");
		}
		$local_file = 1;
        	my $mode = "+<";
		my $perms = 0777;
	        my ($dir, $name) = ($path =~ m%(.*)[\/]([^\/]*)$%);
		$fio = fioOpen( name=>$name,
				dir=>$dir,
				mode=>$mode,
				perms=>$perms);
		if (!$fio) {
			croak("WriteVerifyData: fioOpen failed: $@");
		}
	}

	if (defined($file_pos)) {
		$fio->sysSeek($file_pos);
	}

	logTrc("WriteVerifyData:  name=$fio->{path}  length=$length  data_offset=$data_offset  ".
		  "fileoffset=$file_pos   filepos=$fio->{offset}  blksz=$blksz  ".
		  "croak_on_miscompare=$croak_on_miscompare");

	my $rem_length = $length;
	my $length_written = 0;
	while ($rem_length) {
		my $to_write = ($blksz < $rem_length) ? $blksz : $rem_length;
		my ($rdlen, $wlen);
		my $verify = FS::Util::Utilities::uSendVerifyData(
							            fh=>$fio->{fh},
							            data=>$data,
							            length=>$to_write,
							            data_offset=>$data_offset,
							            buffer_ref=>$buffer_ref,
							            croak_on_miscompare=>$croak_on_miscompare,
								    length_read_ref=>\$rdlen,
								    length_written_ref=>\$wlen);

		$rem_length -= $wlen;
		$length_written += $wlen;
		$fio->{offset} += $rdlen;
		if ($wlen == 0) {
			logWarn("WriteVerifyData:  Zero bytes written, expected to write: $to_write: $!");
			croak("WriteVerifyData:  Zero bytes written, expected to write: $to_write: $!");
			last;
		} elsif ($wlen != $to_write) {
			logWarn("WriteVerifyData:  Wrote $wlen bytes, expected: $to_write");
			croak("WriteVerifyData:  Wrote $wlen bytes, expected: $to_write");
		}
		if ($rdlen == 0) {
			logWarn("WriteVerifyData:  Zero bytes read, expected to read: $to_write: $!");
			croak("WriteVerifyData:  Zero bytes read, expected to read: $to_write: $!");
		} elsif ($rdlen != $to_write) {
			logWarn("WriteVerifyData:  Read $rdlen bytes, expected: $to_write");
			croak("WriteVerifyData:  Read $rdlen bytes, expected: $to_write");
		}
		if (!$verify) {
			logWarn("WriteVerifyData: Verify failed");
			if ($croak_on_miscompare) {
				croak("WriteVerifyData: Verify failed");
			}
		}
	}

	if ($local_file) {
		$fio->Close();
	}
	return($length_written);
}


## @name     ::mkFile
## @summary  Create a new (or overwrite existing) file
##
## @note     All args are "keyword".
##           Either a "path" arg is required, or else both a "name" and "dir" arg.
##
## @arg	     path		(mounted) Path to a file. Optional.
##				    Default is concatination of: $dir . "/" . $name
## @arg	     dir		(mounted) Path to directory containing file. Optional.
## @arg      name		Name of file. Optional.
## @arg	     vol		Volume storing the file. Optional.
## @arg	     perms		Optional. Permissions to give new file. Default is "0666".
## @arg      dont_overwrite	Optional. If '1' then don't overwrite or modify existing files.
##				    Default is '0' (overwrite in any case).
## @arg	    size		Optional. Default is zero. If non-zero, then file is 'truncated'
##				    to specified size (but not populated). Size may be immediately
##				    followed by one of: k, m, g, t, p, to indicate kilobytes, megabytes, etc.
## @arg	    keep_open		Optional. Default is '0'. If '1', then fio file handle is not closed, and
##				    the fio object is returned after (successful) completion.
sub mkFile(%)
{
	my (%args) = validate (
			   @_,
			   {
				dir		=> {type=>SCALAR, optional=>1},
				name		=> {type=>SCALAR, optional=>1},
				path		=> {type=>SCALAR, optional=>1},
				vol		=> {type=>SCALAR, optional=>1 },
				perms		=> {type=>SCALAR, optional=>1 },
				dont_overwrite	=> {type=>SCALAR, optional=>1, default=>0 },
				size		=> {type=>SCALAR, optional=>1 },
				keep_open	=> {type=>SCALAR, optional=>1, default=>0 },
			   }
		     );
	
	my ($path, $dir, $name, $vol, $dont_overwrite, $perms, $size, $keep_open) =
		( $args{path}, $args{dir}, $args{name}, $args{vol},
		    $args{dont_overwrite}, $args{perms}, $args{size}, $args{keep_open} );

	my $mode = ">";

	if ($path) {
	        ($dir, $name) = ($path =~ m%(.*)[\/]([^\/]*)$%);
	} elsif (!$name || !$dir) {
		croak("mkFile: Either 'path' or both of 'dir' and 'name' arguments must be given");
	} else {
		my ($sep) = ($dir =~ m%([\\/])%);
		$path = $dir . $sep . $name;
	}
	if ($dont_overwrite && -e $path) {
		return(1);
	}
	my $args_hash = { name=>$name, dir=>$dir, mode=>$mode };
	$args_hash->{perms} = $perms if (defined($perms));
	my $file = fioOpen(%$args_hash);
	if (!$file) {
		croak("mkFile: fioOpen failed: $@");
	}
	my $size_bytes;
	if ($size) {
		$size_bytes = cvt_size_to_bytes($size);
		Truncate($file, $size_bytes);
	}
	if ($keep_open) {
		return($file);
	} else {
		$file->Close();
		return(1);
	}
}


# ------------------------------------------------------------------------
1;
# ------------------------------------------------------------------------
