# Hey emacs, this is basically -*- perl -*-
# $Id: aggr_charge_onoffline.thpl,v 1.6 2004/05/19 21:11:55 sidlo Exp $
#
# Copyright (c) 2004 Network Appliance, Inc.
# All rights reserved.
#
#

## @summary
##        Utility routines used to QA NetApp space_calculation.
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
##
##
## @description =========================================
## @description Space Calculation Subroutines:
## @description =========================================


package FS::Util::space_calc;

use strict;
use Aggregate qw(aggr_create aggr_purge aggr_disk_free);
use Volume qw(vol_create_flex vol_disk_free);
use Module::Common qw(validate_mixed);
use Utils::Units qw(UNIT_KB);
use SayWrap qw(sayEOF);
use SPARSE::Util::Logger qw(logWarn logTrc);

use Exporter;

our($Validation, $VERSION);
our(@ISA, @EXPORT);
BEGIN
{
        $Validation = 1;
        $VERSION    = '0.01';

        @ISA    = qw(Exporter);
        @EXPORT = qw(get_aggr_df get_aggr_ss_df get_vol_df roundupto get_vol_ss_df wait_aggr_df wait_vol_df);

}

##
## get_aggr_df()
##
## -   Get the disk available on the aggregate.

sub get_aggr_df ($$$) {

  ## @name ::get_aggr_df
  ## @usage get_aggr_df(<FilerConn>, <AggrName>, <df_Field>);
  ## @synopsis
  ##    Performs the df command against an aggregate and
  ##    returns the field-value of (df_field) from df's output.

  my ($fc, $aggr, $fld) = @_;

  my $a_old_df = aggr_disk_free($fc,
                              path => $aggr,
                              space_type => 'space',
                              report_type => UNIT_KB);

  return $a_old_df->{$fld};
}

sub get_aggr_ss_df ($$$) {

  ## @name ::get_aggr_ss_df
  ## @usage get_aggr_df(<FilerConn>, <VolName>, <df_Field>, <snapshot_flag>);
  ## @synopsis
  ##    Performs a df and command on a given Aggr (VolName).
  ##    Returns the field-value of (df_field) from df's output.

  my ($fc, $aggr, $fld) = @_;

  my $a_ss_df = aggr_disk_free($fc,
                               path => $aggr,
                               snapshot => 1,
                               space_type => 'space',
                               report_type => UNIT_KB);

  return $a_ss_df->{$fld};

}

##
## get_vol_df()
##
## -   Here we gather the volume disk free and
##     the associated snapshot disk free.

sub get_vol_df ($$$$) {

  ## @name ::get_vol_df
  ## @usage get_vol_df(<FilerConn>, <VolName>, <df_Field>, <snapshot_flag>);
  ## @synopsis
  ##    Performs a df and command on a given FlexVol (VolName).
  ##    Returns the field-value of (df_field) from df's output.
  ##    If snapshot_flag is 1, includes (adds) the df_field of
  ##    of the Volume and it's snapshot Reserved Space.

  my ($fc, $filerVol, $fld, $w_snapshot) = @_;

  my $fv_df = vol_disk_free($fc,
                            path => $filerVol,
                            space_type => 'space',
                            report_type => UNIT_KB);

  my $ss_avail;
  my $fv_ss_df;
  if ($w_snapshot) {
    $fv_ss_df = vol_disk_free($fc,
                              path => $filerVol,
                              snapshot => 1,
                              space_type => 'space',
                              report_type => UNIT_KB);
    $ss_avail = $fv_ss_df->{$fld}
  } else {
    $ss_avail = 0;
  }

  my $fv_space_avail = $fv_df->{$fld} + $ss_avail;

  return $fv_space_avail;

}

sub get_vol_ss_df ($$$) {

  ## @name ::get_vol_ss_df
  ## @usage get_vol_df(<FilerConn>, <VolName>, <df_Field>, <snapshot_flag>);
  ## @synopsis
  ##    Performs a df and command on a given FlexVol (VolName).
  ##    Returns the field-value of (df_field) from df's output.
  ##    If snapshot_flag is 1, includes (adds) the df_field of
  ##    of the Volume and it's snapshot Reserved Space.

  my ($fc, $filerVol, $fld) = @_;

  my $fv_ss_df = vol_disk_free($fc,
                               path => $filerVol,
                               snapshot => 1,
                               space_type => 'space',
                               report_type => UNIT_KB);

  return $fv_ss_df->{$fld};

}

sub wait_vol_df ($$$) {

        ## @name ::wait_vol_df
        ## @usage wait_vol_df(<filerConn>, <filerVolPattern>, <timeOutDfWait>)
        ## @synopsis
        ##    Performs "df " continuosly until the returned values
        ##    quiesce. If <filerVolPattern> is '' <nil>, we look for all
        ##    volumes on the filer to quiesce.
        ##
        ##    The <timeOutDfWait> value is a timeout value in seconds.  It
        ##    defaults to 30 seconds.
        ##    If the values do not quiesce by the timeout value, we return 1
        ##    If values quiesce, return 0
        # Wait for the volume's df stats to quiesce.
        #

        my ($fc, $filerVol, $timeOutDfWait) = @_;
        my $df_prior = undef;
        my $waitTime = 5;
        $timeOutDfWait = 30 if !$timeOutDfWait;
        my $currentTime = my $startTime = time();
	my $same;
	while (($currentTime - $startTime) < $timeOutDfWait) {
		
                my $df_now = vol_disk_free($fc,
                                           path => $filerVol,
                                           space_type => 'space',
                                           report_type => UNIT_KB)
                        or die $@;
		if (defined $df_prior) {
			$same =1;
			foreach my $key (keys (%{$df_now})) {
				if ($df_now->{$key} ne $df_prior->{$key}) {
					$same = 0;
				}
			}
			return 1 if ($same);
                }
		$df_prior = $df_now;
		
		sleep($waitTime);
                $currentTime = time();
        }
	return 1 if ($same);
	return 0;
}

sub wait_aggr_df ($$$) {

  ## @name ::wait_aggr_df
  ## @usage wait_aggr_df(<filerConn>, <filerVolPattern>, <timeOutDfWait>)
  ## @synopsis
  ##    Performs "df -A" continuosly until the returned values
  ##    quiesce. If <filerVolPattern> is '' <nil>, we look for all
  ##    volumes on the filer to quiesce.
  ##
  ##    The <timeOutDfWait> value is a timeout value in seconds.
  ##    If the values do not quiesce by the timeout value, we output
  ##    a warning indicating that the filer is too busy.

  #
  # Wait for the volume's df stats to quiesce.
  #

  my ($fc, $filerVol, $timeOutDfWait) = @_;

  my $dfText      = "";
  my $dfTextPrior = "";

  # Timeout variables
  my $timeout      = 0;
  my $startTime    = time();

  my $done = 0;
  do {

    $dfText = sayEOF($fc, "df -A $filerVol");

    if ($dfText ne $dfTextPrior) {
      $dfTextPrior = $dfText;
      sleep 2;
    } else {
      $done = 1;
    }

    # Perform a timeout check.
    $timeout = 1 if ((time() - $startTime) > $timeOutDfWait);
    logWarn("Timed-out waiting df status to quiesce (Filesystem is too busy).") if ($timeout);

  } until ($done or $timeout);

}


##
## roundupto($size,$unit)
##
## - Returns the value of $size rounded up to the nearest
## $unit. This is useful for calculations regarding blocks.
## Units must not be 0, or a div by 0 error will occur.
##
##  $sized_up = roundupto($size_a_units, $new_unit_in_a_units);
##
##       2048 = roundupto(2000, 1024);


# Round a value up to the nearest multiple of n.

sub roundupto ($$) {

  ## @name ::roundupto
  ## @usage roundupto(<Val>, <UnitVal>);
  ## @synopsis
  ##    Returns the value of <Val> rounded up to the nearest
  ##    <UnitVal>. This is useful for calculations regarding blocks
  ##    and other arbitrary units.
  ##
  ##    Units must not be 0, or a div by 0 error will occur.
  ##
  ##  $sized_up = roundupto($size_a_units, $new_unit_in_a_units);
  ##
  ##       2048 = roundupto(2000, 1024);

  my ($val, $n) = @_;

  my $ret;
  my $rup = int($val/$n)*$n;
  if (($val/$n - int ($val/$n)) == 0) {
    $ret = $rup;
  } else {
    $ret = $rup + $n;
  }

  return $ret;

}

1;
