#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/Vol_vbncheck.pm#1 $

## @name Vol_vbncheck.pm
## @summary Vol_vbncheck having the subroutine used by
## REP(Raid Error Propagation) Tests
## @status Development
## @author Hardikkumar.Darji@netapp.com
## @pod here

=head1 NAME  
                 Vol_vbncheck
                

=head1 DESCRIPTION
                Vol_vbncheck -   Library that supports  vbncheck operation
                with different options using NACL APISet

=head1 ROUTINES
                vol_vbncheck
                vol_vbncheck_read_output

=cut


# Compiler directives.
use strict;
use warnings;

# =============================================================
# Module Import
# =============================================================

require Exporter;

package FS::Vol_vbncheck;
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF HASHREF OBJECT BOOLEAN UNDEF);
use NACL::APISet;
use vars qw (@ISA @EXPORT_OK);
use Data::Dumper;
use Tharn qw($Log);
use POSIX;

# =============================================================
# Components Used
# =============================================================

use NACL::C::Node;

@ISA       = qw(Exporter);
@EXPORT_OK = qw(vol_vbncheck vol_vbncheck_read_output);

# =============================================================
# Declare Global Variables in BEGIN
# =============================================================

BEGIN {
    use vars qw(
        $Api_Set_Obj
        );
}



# =============================================================
## @name ::vol_vbncheck($PATH)
##
## @summary
##    Calls vbncheck on an array of VBNs
## @description
##     
##
## @arg $fc Required: filer connection object.
## @arg $vol Required: Name of the volume.
## @arg $@vbn Required: Array  of VBNs used a input for vbncheck.

##
## @return value
##      Returns errcode = 0 - Success
##                      < 0 - Failure
##              result = On Success, path of file, where output of vbncheck is stored.
##                     = On Failure, error message.
##        
# =============================================================

sub vol_vbncheck {
     my %opts = validate(
	@_,
	{
		api_obj => { type => OBJECT },
                vol => {type => SCALAR },
		vbn => {type => ARRAYREF , optional => 1},
                mode => {type => SCALAR, optional => 1},
                maps => {type => SCALAR, optional => 1},
	}
    );
    
    my $Api_Set_Obj =  $opts{api_obj};
    my $vol = $opts{vol};
    my $vbn = $opts{vbn};
    my $mode = $opts{mode};
    my $maps = $opts{maps};
	
    my $result;
    my $msg = "Starting scan on ".$vol.", logfile ";
    my $errcode = 0;
    my $err_msg = "vol vbncheck: ";
	
    if(!defined($opts{mode})){
        $mode = "-q test"; 
    }
     
    my $output;
    my $resp={};
    
    if(!defined($opts{maps}))
     {
        $resp = $Api_Set_Obj->system_node_run(node => "local", command => "\"priv set $mode; vol vbncheck $vol @$vbn\""); 
     }
    else
     {
        $resp = $Api_Set_Obj->system_node_run(node => "local", command => "\"priv set $mode; vol vbncheck $vol -maps @$vbn\"");
     }
    $output= Dumper($resp->get_raw_output());
    
    my @lines = split /\n/, $output;

    foreach (@lines) {

       if($_ =~ /$err_msg/) {
           $errcode = -1;
           $result = $_;
       }
       if($_ =~ m/$msg(.*)/){
           $errcode = 0;
           $result = $1;
       }
    }

    return ($errcode, $result);
}

##@name ::vol_vbncheck_read_output($Api_set_obj, $PATH)
##
## @summary
##     Parses output of `vbncheck`
## @description
##     Returns the parsed output of vbncheck log.

## ##
## @arg $fc Required: filer connection object.
## @arg $path Required: path of the log, returned when vol_vbncheck() is called
## @arg $maps Optional: to enable -maps option 
##
## @return value
##      Returns  
##               %result = A hash where VBNs are the key, and the pathname is a value.
##                                pathname is set to 'in active file system' if present in afs.
##                                pathname is set to 'in snapshot' if present in snapshot.
##                                pathname is set to 'not in use' if VBN is not used.                 
##               @snapshot_list = Reference to Array of snapshots that possibly contain this VBN.

sub vol_vbncheck_read_output(%)
{
    my %opts = validate(
        @_,
        {
                api_obj => { type => OBJECT },
                path => {type => SCALAR },
                
        }
    );
    sleep(50);
    my $Api_Set_Obj =  $opts{api_obj};
    my $path = $opts{path};
    my $in_afs = "in active file system";
    my $notinuse_msg = "not in use";
    my $pathname_msg = "pathname = ";
    my $in_snapshot =  "in snapshot";
    my $snapshot_msg = "The following snapshots contain any of the listed blocks:";
    my $snapshots_exist = 0;

    my %result = ();
    my %error = ();
    my %level = ();
    my $vvbn;

    my @snapshot_list;
    
    my $resp =  $Api_Set_Obj->system_node_run(node => "local",command =>"rdfile $path");
    
    my $output= Dumper($resp->get_raw_output());
    my @lines = split /\n/, $output;
    
    my $new_format = 0;
    foreach (@lines) {

      # Store the corrupt blocks in a list.
      if($_ =~ /VVBN\s(\d+)/) {
        $vvbn = $1;
        $result{$1} = $notinuse_msg;
        $new_format = 0;
     
        } elsif ($_ =~ /VVBN:\s(\d+)(.*),\sLevel:\s+(\d+),\sError:\s+(\d+)/) {
        # New format in RR.0 (Level and Error appended)
        # VVBN: 4150, Disk: 0a.18, Volume: testvol_flex, Inode: 100, Level: 0 Error: 119
        # Inode 100 has 1 name(s).
        # Volume: testvol_flex, Inode: 100, Pathname: /vol/testvol_flex/file1

        $vvbn = $1;
        $result{$1} = $notinuse_msg;
        $level{$1}= $3;
        $error{$1}=$4;
        $new_format = 1;
      } 
     
         # New format in IC.3 'vbncheck with -maps option'
         # Whenever the vvbn is in Active file system the output will be
         # VVBN 4150  in active file system
      if($_ =~ /VVBN\s(\d+)\s+$in_afs/)
      {
        
        $vvbn = $1;
        $result{$1} = $in_afs;
      }
      
      if($_ =~ /VVBN\s(\d+)\s+$in_snapshot/)
      {
        $vvbn = $1;
        $result{$1} = $in_snapshot;
      }

      if($new_format) {
        if($_ =~ /Pathname:\s(.*)/) {
           $result{$vvbn} = $1;
        }
      } else {
        if($_ =~ /$pathname_msg(.*)/) {
           $result{$vvbn} = $1;
        }
      }

      # Determine if there are any corrupt blocks in the snapshots.
      if($_ =~ /$snapshot_msg/) {
        $Log->comment("snapshot_exist assigned");
        $snapshots_exist = 1;
        
      }

      # If there are snapshots, then add them to a list.
      if(($_ =~ /\d+:\d+:\d+/) && ($snapshots_exist == 1)) {
        my @line = split / /, $_;
        push @snapshot_list, $line[0];
      }

    }
    
    return (\%result,\@snapshot_list,\%level,\%error);
    
}

1;
