#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/FS/Wreck.pm#1 $

#this library is intended for C-mode Raid error injection

=head1 NAME

  FS::Wreck- This library helps you use the 'wreck' command which
             corrupts filesystems for testing purposes

=head1 DESCRIPTION

  The Wreck library consists of function to perform following tasks:

  wreck() : Routines for using 'wreck' corruption command

=head1 SYNOPSIS

  
    use FS::Wreck;
    
    my ($errcode,$result) = wreck(api_obj =>$APISet_obj,
                           type =>'aggr', vol =>'myaggr',
                           inject_script =>'inode_type inum=64 type=1'
                           error_type =>"unverified");
                           
    if ($errcode < 0) {
        $Log->comment("wreck failed with eroor: $result");
    }
    
         
=head1 STATUS
 @status private
 
=head1 AUTHOR
 @author Hardikkumar.Darji@netapp.com

=head1 DISCLAIMER

 @disclaimer This library does NOT support duplicate vol names in the same aggr. 
             In short no vserver checking is available.
=cut


# Compiler directives.
use strict;
use warnings;

# =============================================================
# Module Import
# =============================================================

require Exporter;

package FS::Wreck;
use Params::Validate
    qw(validate validate_with SCALAR ARRAYREF HASHREF OBJECT BOOLEAN UNDEF);
use NACL::APISet;
use vars qw (@ISA @EXPORT_OK);
use Data::Dumper;
use Tharn qw($Log);
use POSIX;
use Carp qw(croak);
use NACL::Exceptions::NoElementsFound qw(:try);

# =============================================================
# Components Used
# =============================================================

use NACL::C::Node;
use NACL::C::Aggregate;
use NACL::C::Volume;
@ISA       = qw(Exporter);
@EXPORT_OK = qw(wreck);

# =============================================================
# Declare Global Variables in BEGIN
# =============================================================

BEGIN {
    use vars qw(
        $Api_Set_Obj
        );
    
}


# ===============================================================
=head2 wreck(api_obj=> $APISet_obj, vol => $vol [, %opts])

 @summery Routines for using 'wreck' corruption command
 
 @params
 
    api_obj        - APISet object (Required)

    type           - 'aggr' or 'flex' (Required)

    vol            - name of target volume/aggr
    
    vserver        - name of vserver where vol is created

    inject_script  - text of inject script to apply

    timeout        - seconds for 'wreck' command (optional, default 600)

    error_type     - type of error to be injected(pseudo_bad OR unverified)
           
    Note: The pseudo_bad /unverified option should be used only with initialized parity disks

    

 @return value
      Returns $errcode = 0 if command successful
                       < 0 if command fail
                       
              $result = "wreck command was successful" if command successful
                      = error string if command fail
                          
 @example my ($errcode,$result) = wreck(api_obj =>$APISet_obj,
                           type =>'aggr', vol =>'myaggr',
                           inject_script =>'inode_type inum=64 type=1'
                           error_type =>"unverified");

=cut
# =============================================================
sub wreck(%) 
{
    my %opts = validate( 
	@_,
	{
		api_obj       => { type => OBJECT,optional => 0},
                vol           => { type => SCALAR,optional => 0},
                vserver       => { type => SCALAR,optional => 1,default => "vs0"},
		type          => { type => SCALAR, optional => 1},
		inject_script => { type=> SCALAR, optional => 0 },
		timeout       => { type => SCALAR, optional => 1,default => 600 },
	        error_type    => { type => SCALAR, optional => 1,regexp => qr/pseudo_bad|unverified/i},
		
		
	}
    );

    my ($cmd,$errcode,$result,$status,$volume,$aggr,$aggregate,$vserver,$error_type,$parity_flag);
    my $Api_Set_Obj = $opts{api_obj};
    my $vol = $opts{vol};
    my $inject_script= $opts{inject_script};
    
    ## $no_element_flag is to track whether the exception was thrown by the try block or not
    my $no_element_flag =0;
    if ($opts{type} =~ /aggr/) {
            $cmd = "wreck -A $vol '".$inject_script."'";
    } elsif ($opts{type} =~ /flex/i) {
        $vserver= $opts{vserver};
        $cmd = "wreck $vol '".$inject_script."'";
    } else {
            # We shouldn't ever reach here. the
            # type is being validated at the top
            $errcode =-1;
            $result="Illegal wreck type: $opts{type}";
            return ($errcode,$result);
    }
    ## checking parity initialized when
    ## error_type is defined
    if(defined($opts{error_type}))
    {
        if ($opts{error_type} =~ /pseudo_bad|unverified/i )
        {
                
            $Log->comment("Inside parity checking");
            
            $parity_flag=0;
            $status =&_get_raid_status(type=>$opts{type},vol=>$vol,vserver=>$vserver);
            
            $parity_flag = -1 if $status =~ /parity uninit'd!/i;

            if($parity_flag == -1)
            {
                $errcode =-1;
                $result="pseudo_bad and unverified options should be exercised only "
                      . "on containers which have the parity disk initialized";
                return ($errcode,$result); 
            }
            else
            {
                
                $Log->comment("Containing aggregate has parity initialized");
                $error_type= $opts{error_type};
                $cmd = $cmd . " -".$error_type;
            }
            
                
        }
        else
        {
            $errcode =-1;
            $result="Illegal error_type value provided".
                    "allowd values are (pseudo_bad|unverified)";
            return ($errcode,$result);
                  
        }
    }
    ## checking parity initialized when
    ## error_type is not defined
    else
    {
        if ($inject_script =~ /pseudo_bad|unverified/i )
        {
                
            $Log->comment("Inside parity checking");       
            
            $parity_flag=0;
            $status =&_get_raid_status(type=>$opts{type},vol=>$vol,vserver=>$vserver);
            
            $parity_flag = -1 if $status =~ /parity uninit'd!/i; 
            
            if($parity_flag == -1)
            {
                $errcode =-1;
                $result="pseudo_bad and unverified options should be exercised only "
                      . "on containers which have the parity disk initialized";
                return ($errcode,$result);
               
            }
            else
            {
                $Log->comment("Containing aggregate has parity initialized");
            }
            
        }
    }
    
    # making APISet call to run the wreck comamnd
    my $resp=$Api_Set_Obj->system_node_run(node => "local", command => "\" priv set test; $cmd \"");
    my $output=Dumper($resp->get_raw_output());
   
    # checking whether wreck injected corruption correctly or not
    # "Wrote vbn" and "Finished"
    # should be present in the wreck output
    
    if (   $output !~ /.*Wrote vbn \d+.*/i
        || $output !~ /.*Finished.*/ )
    {
        $errcode = -1;
        $result="wreck command for injection string $opts{inject_script} failed, check logs.";
        return ($errcode,$result);      
    }
    
    $errcode =0;
    $result="wreck command was successful";    
    return ($errcode,$result); 
}


### PRIVATE FUNCTION #######################
sub _get_raid_status()
{
    my %opts = validate( 
	@_,
	{
            vol           => { type => SCALAR,optional => 0},
            vserver       => { type => SCALAR,optional => 1,default => "vs0"},
            type          => { type => SCALAR, optional => 0}
	}
    );
    my ($type,$vol,$vserver,$volume,$aggr,$aggregate,$status,$node);
    $node = NACL::C::Node->find();
    $type= $opts{type};
    $vol =$opts{vol};
    $vserver = $opts{vserver};
    
    # Checking if parity disk is initialized.
    # pseudo_bad option should not be used with 
    # uninitialized disks
    
    # if type==aggr then find out the raidstatus by providing
    # aggregate name which is stored in $vol          
    if($type =~ /aggr/i) {
        $aggregate = NACL::C::Aggregate->find( 
        command_interface => $node, 
        filter            => {aggregate => $vol}, 
        );
    $status=$aggregate->state()->raidstatus();
    }
    # else type==flex then find out the volume's 
    # containing aggregate by using volume and vserver
    # and then find out containing aggregate's raidstatus     
    else
    {
        $volume = NACL::C::Volume->find( 
        command_interface => $node, 
        filter            => {volume => $vol, vserver => $vserver}, 
        );
        
        $aggr=$volume->state()->aggregate();
        $aggregate = NACL::C::Aggregate->find( 
        command_interface => $node, 
        filter            => {aggregate => $aggr}, 
        );
        $status=$aggregate->state()->raidstatus();
    $Log->comment("Raid status: $status"); 
    }
    
    return $status;
}
1;
