# -*- perl -*-

## @summary Library for managing commonly-used iron parameters
## @author prasank@netapp.com, dl-waflqa@netapp.com
## @status public
## @pod here

=head1 NAME

FS::commonLDCParams.pm - provide support for managing commonly-used iron parameters

=head1 SYNOPSIS

  use Tharn;

  BEGIN {
     # set param values here, before loading FS::commonLDCParams
     param('EXAMPLE_PARAM', -default, 'example_value');
  }

  # load the module
  use FS::commonLDCParams;

  # this declares the common parameters in the 'main' package, and sets default
  # values for any of them which are undefined.
  my $clp = FS::commonLDCParams->new();

=head1 DESCRIPTION

commonLDCParams - this module provides support for managing commonly-used iron parameters.

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-waflqa-automation@netapp.com
  $Id: //depot/prod/test/main/lib/FS/commonLDCParams.pm#11 $
  Last revised $Date: 2010/04/13 $

=head1 COPYRIGHT

  Copyright 2010, Network Appliance Inc., All Rights Reserved.

=cut

package main;

use strict;
use Data::Dumper;

# declare PERL vars in 'main' - please keep the list in alphabetical order
use vars qw(
	    $LDC_CONTAINER_NAME
	    $LDC_CONTAINER_SIZE
	    $LDC_CONTAINER_RAID_TYPE
	    $LDC_LARGER_AGGR
	    $LDC_VOL_PREFIX
	    $LDC_VOL_COUNT
	    $LDC_VOL_SIZE
	    $LDC_NUMOF_FILES
	    $LDC_VOL_COMPRESSION
	    $LDC_VOL_DENSE
	    $LDC_VOL_PCT_FILL
            $LDC_VOL_SNAP_COUNT
            $LDC_VSERVER
            $LDC_USE_VOLS_OR_CLONES
	   );

package FS::commonLDCParams;

use strict;
use warnings;

use Carp qw(croak);

use Tharn;

use Params::Validate qw(validate SCALAR);

sub new {
    my $class = shift;
    $class = ref($class) || $class;

    my $self = bless {}, $class;
    $self->_set_defaults();

    #
    $self->_set_ldc_vars();

    return $self;
}

# this method sets default values
sub _set_defaults {
    my $self = shift;

    # set params in package 'main', so the param refs below can be resolved properly
    package main;


    param('LDC_CONTAINER_NAME',                  '-default', 'ldcaggr');
    param('LDC_CONTAINER_SIZE',                  '-default', 3);
    param('LDC_CONTAINER_RAID_TYPE',             '-default', 'raid_dp');
    param('LDC_LARGER_AGGR',                     '-default', 0);
    param('LDC_VOL_PREFIX',                      '-default', 'ldc');
    param('LDC_VOL_COUNT',                       '-default', 2);
    param('LDC_VOL_SIZE',                        '-default', '10g');
    param('LDC_NUMOF_FILES',                     '-default', '3');
    param('LDC_VOL_COMPRESSION',                 '-default', 1);
    param('LDC_VOL_DENSE',                       '-default', 1);
    param('LDC_VOL_SNAP_COUNT',                  '-default', 3);
    param('LDC_VOL_PCT_FILL',                    '-default', 10);
    param('LDC_VSERVER',                         '-default', 'vs0');
    param('LDC_USE_VOLS_OR_CLONES',              '-default', 'vols');
}

sub _set_ldc_vars {
    my $self = shift;
    my ($key, $value);

    # set params in package 'main', so the param refs below can be resolved properly
    package main;

    my ( %ldc_params );

    # CONTAINER opts
    $ldc_params{LDC_CONTAINER_NAME} = $LDC_CONTAINER_NAME;
    $ldc_params{LDC_CONTAINER_SIZE} = $LDC_CONTAINER_SIZE;
    $ldc_params{LDC_CONTAINER_RAID_TYPE} = $LDC_CONTAINER_RAID_TYPE;
    $ldc_params{LDC_LARGER_AGGR}    = $LDC_LARGER_AGGR;
    $ldc_params{LDC_VOL_PREFIX}     = $LDC_VOL_PREFIX;
    $ldc_params{LDC_VOL_COUNT}      = $LDC_VOL_COUNT;
    $ldc_params{LDC_VOL_SIZE}       = $LDC_VOL_SIZE;
    $ldc_params{LDC_NUMOF_FILES}    = $LDC_NUMOF_FILES;
    $ldc_params{LDC_VOL_COMPRESSION}= $LDC_VOL_COMPRESSION;
    $ldc_params{LDC_VOL_DENSE}      = $LDC_VOL_DENSE;
    $ldc_params{LDC_VOL_PCT_FILL}   = $LDC_VOL_PCT_FILL;
    $ldc_params{LDC_VOL_SNAP_COUNT} = $LDC_VOL_SNAP_COUNT;
    $ldc_params{LDC_VSERVER}        = $LDC_VSERVER;
    $ldc_params{LDC_USE_VOLS_OR_CLONES}= $LDC_USE_VOLS_OR_CLONES;
    $self->{LDC_PARAMS} = \%ldc_params;
}

1;
