#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/FSH/CifsCopy.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
## @summary
##     cifs copy routing
##
## @description
##     This library contains functions which perform tasks common to the
##     fs/write_correctness/bad_disk tests.
##
## @usage
##     use FSH::CifsCopy;
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
## @status public
package CifsCopy;
use Tharn;
use FSH::CifsServer;
#use Data::Dumper;

# Configure CIFS copy job.
sub new( $$$$;$$ ) {
    my $class = shift;
    my $fen = shift;
    my $timestamp = shift;
    my $mnt_path = shift;
    my $client = shift || $main::CIFS_CLIENT;
    my $last = shift || 21;
    my $source = sprintf('my_source_cifs_%s', $timestamp);
    my $self = {};
    $self->{fen} = $fen;
    $self->{server} = CifsServer->new($client);
    $self->{source_path} = $mnt_path . '\\' . $source;
    $self->{target_prefix} = sprintf('%s\\my_target_cifs_%s', $mnt_path, $timestamp);
    $self->{first} = 11;
    $self->{last} = $last;
    $self->{size} = 8000;
    bless $self, $class;
    $self->log_event('cifs copy', 'new');
    return $self;
}

# Create the source file that will be repeatedly copied during
# reconstruction.
sub setup( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    my $source_path = $self->{source_path};
    $self->log_event('cifs copy', 'setup begin');
    $server->setup_rpc_server();
    $server->send_create_file($source_path, $self->{size});
    $self->log_event('cifs copy', "cp # $source_path");
    $server->rpc_finish();
    $self->log_event('cifs copy', 'setup end');
}

# Copy file and repeat to generate volume write activity.
sub activity( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    my $source_path = $self->{source_path};
    my $target_prefix = $self->{target_prefix};
    $server->setup_rpc_server();
    $self->log_event('cifs copy', 'activity start');
    for (my $i=$self->{first};$i<$self->{last};$i++) {
        my $target_path = $target_prefix ."_$i";
        $server->send_copy_file($source_path, $target_path);
        $self->log_event('cifs copy', 'cp #' . "$source_path --> $target_path");
    }
    $self->{first} = $self->{last}+1;
    $self->{last} = $self->{first} + 15;

    $self->log_event('cifs copy', 'activity end');
    #Note that the above activities are still running in the
    #background.
}

# Wait for all background copy tasks to complete.
sub finish( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    $server->rpc_finish();
    $self->log_event('cifs copy', 'finish');
}

sub log_event( $ ) {
    my ($self, $event, $message) = @_;
    logcomment("$event : $message");
    $self->{fen}->write_event($event, $message);
}

1;
