#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/FSH/CifsServer.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
## @summary
##     CifsServer - Creates a CifsServer object (parent class is CmdServer).
##
## @description
##     This library contains functions which perform tasks common to the
##     fs/write_correctness/bad_disk tests.
##
## @usage
##     use FSH::CifsServer;
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
## @status publicuse strict;
use FSH::CmdServer;

package CifsServer;
use vars qw(@ISA);
@ISA = qw(CmdServer);

use Tharn;
use Utils qw(make_conn);
use TestEnv::MinWin qw(minwin_compatible);
use SPARSE::Util::Filer_Utils qw(setup_cifs);
use MapShare qw(map_cifs_share);
use ExportFS qw(:nfs share_add);
use FS::Util::Setup qw(create_and_map_cifs_share);
use SPARSE::Util::Logger qw( logTrc logWarn logDbg);
use Data::Dumper;


sub new( $$$;$ ) {
    my $class = shift;
    my $self = $class->SUPER::new(@_);
    $self->{MAP_DRIVE} = undef;
    return $self;
}

sub create_cifs_share ( $$$$$$$$ ) {
    my ($self, $filer, $filerconn, $vol_name, $cifs_share, $cifs_client,
        $cifs_drive, $user,$passwd) = @_;

    logcomment("cifs_client = $cifs_client");
    minwin_compatible($cifs_client)
        or die($@);

    logTrc("attempting to do cifs setup.. ");
    setup_cifs($filer, $passwd);

    logTrc("cifs setup complete .. ");

    my $fh = host($filer);
    my $fc = connect($filerconn, -timeout, $::TIMEOUT);
    $fc->timeout($::CMDTIMEOUT);

    logTrc("add new share $cifs_share on $vol_name");
    share_add($fc, $cifs_share, "/vol/$vol_name", verify=>1, umask=>0)
        or die("failed to export share $cifs_share");

    my $used_drives = {};

    my $cc = make_conn($cifs_client);
    my (undef, undef, $net_use_status) = say($cc, "net use", "-eval_error");
    if ($@ && $@ =~ /Err=TIMEOUT/i) {
        logcomment("minwin_compatible warning: TIMEOUT during net use");
    }
    foreach my $line (split(/\n/, $net_use_status)) {
         if ($line =~ /(Disconnected|OK)\s+(\w)\:/) {
             $used_drives->{$2} = 1;
         }
    }

    my $drive = "G";
    while (1)  {
        last if (!$used_drives->{$drive});
        $drive = chr(ord($drive)+1);
        die ("no drive letters available on cifs client")
           if (ord($drive) == (ord('X')+1));
    }

    $cifs_drive = $drive;

    logDbg("added share /vol/$vol_name");


    my $host = $fc->connhost();
    $cifs_drive = map_cifs_share(
                                 server   => $host,
                                 share    => $cifs_share,
                                 client   => $cifs_client,
                                 drive    => $cifs_drive,
                                 username => $user,
                                 password => $passwd
                                )
        or die($@);
    logDbg("mapped drive $cifs_drive");
    if (!$cifs_drive) {
        die("could not map the drive");
    }

    $cifs_drive .= ":" unless $cifs_drive =~ /:$/;

    $self->{MAP_DRIVE} = $cifs_drive;
}

sub unmap_cifs_share ( $ ) {
    my $self = shift;
    MapShare::unmap_cifs_share(client=>$self->my_client(),
                               drive=>$self->map_drive())
          or die "error($@)";
}

sub map_drive( $ ) {
    my $self = shift;
    return $self->{MAP_DRIVE};
}

1;
