#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/FSH/FenceLog.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
## @summary
##     FenceLog -
## Create a file-like object for sequential output.  The output is
## formatted into columns, and writes specify which column the output
## goes to.  Think of each column as a sequential process, and the
## entries in its column to be a trace of the events.  Other columns
## are other processes running concurrently.  Each write advances a
## whole line, but only one event column has an entry.
##
## @description
##     This library contains functions which perform tasks common to the
##     fs/write_correctness/bad_disk tests.
##
## @usage
##     use FSH::FenceLog;
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
## @status public

#package fs::write_correctness::bad_disk::FenceLog;
package FenceLog;
use Data::Dumper;
use IO::File;
use IO::Handle;

# Several scripts can write to the named file, and it up to them to
# write compatiable data.  Only the first script can start writing,
# however, since the heading need to be written once.

sub new ( $$$$;$$ ) {
    my $class = shift;
    my $create = shift;
    my $path = shift;
    my $col_info = shift;
    my $no_write = shift || 0;
    my $field_width = shift || 25;
    my $self = {};
    bless $self, $class;
    $self->{PATH} = $path;
    $self->{COL} = $col_info;
    $self->{FW} = $field_width;
    $self->{no_write} = $no_write;
    return $self if $no_write; # TBD use null log class.
    $self->{FILE} = IO::File->new(">>$path") or die $!;
    $self->{FILE}->autoflush(1);
    if ($create) {
        foreach my $k ( $self->col_names() ) {
            my $head = center('(' . $k . ')', $self->{FW});
            $self->{FILE}->print($head);
        }
        $self->{FILE}->print("<\n");
    }
    return $self;
}

sub col_names( $ ) {
    my $self = shift;
    return @{ $self->{COL} };
}

sub write_event( $$$ ) {
    my $self = shift;
    my $proc = shift;
    my $event = shift;
    return if $self->{no_write};
    my $spacer = center('|', $self->{FW});
    foreach my $k ( $self->col_names() ) {
        if ($k eq $proc) {
            $event = center($event, $self->{FW});
            $self->{FILE}->print($event);
        } else {
            $self->{FILE}->print($spacer);
        }
    }
    $self->{FILE}->print("<\n");
}

# len(lead_sp) + len(word1) + len(word2) + len(trail_sp) == len(width)
#     where word1 ++ word2 == word
sub center( $$ ) {
    use POSIX;
    my $word = shift;
    my $width = shift;
    my @w = (floor(length($word)/2),  floor(length($word)/2) + floor(length($word)% 2));
    my $sp = $width - length($word);
    if ($sp <= 0) {
        return substr($word,0,$width); # truncate;
    }
    my @s = (floor($sp/2),  floor($sp/2) + floor($sp% 2));
    return (' ' x $s[1]) . substr($word,0,$w[0]) . substr($word,$w[0]) .(' ' x $s[0]);

}

1;
