#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/FSH/NfsCopy.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
## @summary
##     NfsCopy - Do background writes from NFS to the filer.
##
## @description
##     This library contains functions which perform tasks common to the
##     fs/write_correctness/bad_disk tests.
##
## @usage
##     use FSH::NfsCopy;
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
## @status publicuse strict;
package NfsCopy;
use Tharn;
use FSH::NfsServer;
#use Data::Dumper;

# Configure NFS copy job.
sub new( $$$$;$ ) {
    my ($class, $fen, $timestamp, $mnt_path, $last);
    $class = shift;
    $fen = shift;
    $timestamp = shift;
    $mnt_path = shift;
    $last = shift || 21;
    my $source = sprintf('my_source_%s', $timestamp);
    my $self = {};
    $self->{fen} = $fen;
    $self->{server} = NfsServer->new($main::CLIENT);
    $self->{source_path} = $mnt_path . '/' . $source;
    $self->{target_prefix} = sprintf("$mnt_path/my_target_%s", $timestamp);
    $self->{first} = 11;
    $self->{last} = $last;
    $self->{size} = 8000;
    bless $self, $class;
    $self->log_event('nfs copy', 'new');
    return $self;
}

# Create the source file that will be repeatedly copied during
# reconstruction.
sub setup( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    my $source_path = $self->{source_path};
    $self->log_event('nfs copy', 'setup begin');
    $server->setup_rpc_server();
    $server->send_create_file($source_path, $self->{size});
    $self->log_event('nfs copy', "cp # $source_path");
    $server->rpc_finish();
    $self->log_event('nfs copy', 'setup end');
}

# Copy file and repeat to generate volume write activity.
sub activity( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    my $source_path = $self->{source_path};
    my $target_prefix = $self->{target_prefix};
    $server->setup_rpc_server() if (!defined $self->{SUBTEST});
    $self->log_event('nfs copy', 'activity start');
    for (my $i=$self->{first};$i<$self->{last};$i++) {
        my $target_path = $target_prefix ."_$i";
        $server->send_copy_file($source_path, $target_path);
        $self->log_event('nfs copy', 'cp #' . "$source_path --> $target_path");
    }
    $self->{first} = $self->{last}+1;
    $self->{last} = $self->{first} + 15;

    $self->log_event('nfs copy', 'activity end');
    #Note that the above activities are still running in the
    #background.
}


# Wait for all background copy tasks to complete.
sub finish( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    $server->rpc_finish();
    $self->log_event('nfs copy', 'finish');
}

sub log_event( $ ) {
    my ($self, $event, $message) = @_;
    logcomment("$event : $message");
    $self->{fen}->write_event($event, $message);
}

1;
