#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/FSH/NfsTrunc.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
## @summary
##     NfsTrunc - Nfs truncation job to perform background truncation.
##
## @description
##     This library contains functions which perform tasks common to the
##     fs/write_correctness/bad_disk tests.
##
## @usage
##     use FSH::NfsTrunc;
##
## @author dl-johny@netapp.com (FSS WAFL Quality)
## @status public
use strict;
package NfsTrunc;
use Tharn;
use FSH::NfsServer;
#use Data::Dumper;

# Configure NFS truncation job.
sub new( $$$$ ) {
    my ($class, $fen, $timestamp, $mnt_path) = @_;
    my $source = sprintf('my_trunc_source_%s', $timestamp);
    my $self = {};
    $self->{fen} = $fen;
    $self->{server} = NfsServer->new($main::CLIENT);
    $self->{source_dir} = $mnt_path . '/' . $source;
    $self->{first} = 11;
    $self->{last} = 15;
    $self->{size} = 8000;
    bless $self, $class;
    $self->log_event('nfs trunc', 'new');
    return $self;
}

# Create the source files that will be truncated during reconstruction.
sub setup( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    my $source_dir = $self->{source_dir};
    $self->log_event('nfs trunc', 'setup begin');
    $server->setup_rpc_server();
    for (my $i=$self->{first};$i<$self->{last};$i++) {
        my $path = $source_dir . "_$i";
        $server->send_create_file($path, $self->{size});
        $self->log_event('nfs trunc', "tr# $path");
    }
    $server->rpc_finish();
    $self->log_event('nfs trunc', 'setup end');
}

# Truncate files to generate volume I/O activity.
sub activity( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    my $source_dir = $self->{source_dir};
    $server->setup_rpc_server();
    $self->log_event('nfs trunc', 'activity start');
    for (my $i=$self->{first};$i<$self->{last};$i++) {
        my $path = $source_dir ."_$i";
        $server->send_truncate_file($path);
        $self->log_event('nfs trunc', "tr # $path");
    }
    $self->log_event('nfs trunc', 'activity done');
}

# Wait for background tasks to complete.
sub finish( $ ) {
    my ($self) = @_;
    my $server = $self->{server};
    $server->rpc_finish();
    $self->log_event('nfs trunc', 'finish');
}

sub log_event( $ ) {
    my ($self, $event, $message) = @_;
    logcomment("$event : $message");
    $self->{fen}->write_event($event, $message);
}

1;
