#
# $Id: //depot/prod/test/Rsierranevada/lib/FijiParser.pm#1 $
#
# Copyright (c) 2007 Network Appliance, Inc.
#   All rights reserved.

package FijiParser;

use Exporter;
@ISA = qw(Exporter);
@EXPORT_OK = qw(fijiFileParse);

use strict;

=head1 NAME

FijiParser - Parse .fiji files

=head1 SYNOPSYS

use FijiParser qw(fijiFileParse);

my (@points) = fijiFileParse($path_to_dot_fiji_file);

=head1 DESCRIPTION

This module provides a mechanism to read and parse .fiji files, to
automatically discover a set of fiji points which can be triggered.

=head1 FUNCTIONS

=head2 fijiFileParse

fijiFileParse() accepts a list of .fiji files to parse.
It returns a list of fiji points discovered within those files.

=cut

sub fijiFileParse {
    my (@ret);
    foreach my $file (@_) {
	push @ret, _parseOneFile($file);
    }
    
    return @ret;
}

sub _parseOneFile {
    my $filename = shift;

    my $fijifile;
    my @ret;

    # slurp in the contents of the file...
    open(FIJIFILE, "<$filename") || die "Can't open $filename";
    while (<FIJIFILE>) {
	# ...and discard everything outside of the fiji pragmas
	next unless m/^#pragma fijiStart/ .. m/^#pragma fijiDone/;

	$fijifile .= $_;
    }
    close FIJIFILE;

    # discard all C style comments
    $fijifile =~ s|
		\/\*	# it begins with /*
		.*?	# ...contains the minimal number of characters
		\*\/	# ...up to the next */
		||sgx; 


    # find anything remaining that looks like a FIJI point definition
    while ($fijifile =~ m/^extern\s*\(\w+\)\s+(.*?)\(.*?\)/msg) {
	my $name = $1;

	push @ret, $name;
    }

    return @ret;
}

# return value so perl doesn't have a cow
1;
