# Hey emacs, this is basically -*- perl -*-
# vim: set filetype=perl tabstop=4 shiftwidth=4 shiftround expandtab:
# $Id: //depot/prod/test/Rsierranevada/lib/Filelock/Unix.pm#1 $
## @summary     Class for basic unix file locking
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here
###############################################
##   NOT READY FOR PUBLIC CONSUMPTION
###############################################
package Filelock::Unix;

use strict;
use Carp qw(croak);
use Data::Dumper;
use Exporter;
use File::FcntlLock;
use File::stat;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use POSIX;
use Tharn qw ($Log);

my %MapOf;
my %File_Handle_Map;
my $FH_ID = 1;

###############################################################################
# Method:    close_file()
# Objective: Close file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub close_file {
    my (%args)=@_;
    my (
        $fh,
        $fh_map_key,
        $file,
    );
    
    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
        }
    );
    
    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    ### Close the file
    close($fh) or 
        croak "Could not close file $file: $!"; 
   
    ### Delete the correcponding entries from the file handle and file maps
    delete $File_Handle_Map{$fh};
    @{$MapOf{$file}} = grep {!($_ eq $fh_map_key)} @{$MapOf{$file}};
    if (!$MapOf{$file}) {
        delete $MapOf{$file};
    } 
   
    return 1;
}

###############################################################################
# Method:    delete_file()
# Objective: Close file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub delete_file {
    my (
        $fh,
        $file,
    );
    
    ### Validate the input arguments
    my (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
            },
        }
    );

    $file = $args{file};

    ### Delete the file
    unlink($file) or 
        croak "Could not delete file $file: $!"; 
   
    ### Delete the corresponding entries from the file handle and file maps
    foreach my $fh_map_key (@{$MapOf{$file}}) {
        delete $File_Handle_Map{$fh_map_key};
    }
    delete $MapOf{$file};

    return 1;
}

###############################################################################
# Method:    lock_file()
# Objective: Lock file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub lock_file {
    my (%args)=@_;
    my (
        $fh,
        $fh_map_key,
        $file,
        $fl,
        $lcmd,
        $length,
        $ltype,
        $offset,
    );
    
    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            ltype => {
                type => SCALAR,
                default => "F_WRLCK",
                callbacks => {
                    'legal values' => sub{shift() =~ /F_WRLCK|F_RDLCK/i},
                },
            },
            lcmd => {
                type => SCALAR,
                default => "F_SETLK",
                callbacks => {
                    'legal values' => sub{shift() =~ /F_SETLK|F_SETLKW/i},
                },
            },
            offset => {
                type => SCALAR,
                default => 0,
                regex => qr/^\d+$/,
            },
            length => {
                type => SCALAR,
                default => 0,
                regex => qr/^\d+$/,
            },
        }
    );

    $offset = $args{offset};
    $length = $args{length};
    $lcmd = $args{lcmd};

    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    ### Determine the lock type
    $ltype |= eval "$args{ltype}" if $args{ltype};
    $lcmd |= eval "$args{lcmd}" if $args{lcmd};
    $Log->comment("LTYPE: ".$ltype);
    $Log->comment("LCMD: ".$lcmd);

    ### Create a new lock instance on the file  
    $fl = new File::FcntlLock;
    $fl->l_type($ltype);
    $fl->l_whence(SEEK_SET);
    $fl->l_start($offset);
    $fl->l_len($length);
    $fl->l_pid($$);
    
    ### Lock the file
    eval {
        $fl->lock($fh, $lcmd);
    };
    if ($fl->error) {
        $Log->comment("errnbr> ".$fl->lock_errno());
        $Log->comment("error:  ".$fl->error()); 
        croak "Locking failed: ".$fl->error;   
    };
    
    ### Add the lock information to the array of locks stored in the map
    push @{$File_Handle_Map{$fh_map_key}{lock_list}}, 
        {
            offset => $offset, 
            length => $length, 
            ltype => $ltype,
            lcmd => $lcmd,
            lock_obj => $fl,
        };            

    return 1;
}

###############################################################################
# Method:    open_file()
# Objective: Open file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub open_file {
    my (%args)=@_;
    my (
        $fh,
        $fh_map_key,
        $file,
        $legal,
        $oflags,
        $cflags,
        @inputs,
    );
    
    my @oflags=qw(
        O_RDONLY
        O_WRONLY
        O_RDWR
    );
        
    my @cflags=qw(
        O_CREAT
        O_TRUNC
        O_EXCL
        O_APPEND
    );
    
    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
            },
            oflags => {
                type => SCALAR,
                callbacks => {
                    'Legal values' => sub{
                        @inputs = split(/\|/, shift());
                        $legal = join("|", @oflags);
                        not scalar(grep{$_ !~ /^$legal$/} @inputs);
                    },
                },
            },
            cflags => {
                type => SCALAR,
                optional => 1,
                callbacks => {
                    'Legal values' => sub{
                        @inputs = split(/\|/, shift());
                        $legal = join("|", @cflags);
                        not scalar(grep{$_ !~ /^$legal$/} @inputs);
                    },
                },
            },
            perms => {
                type => SCALAR,
                optional => 1,
                default => 0777,
                regex => qr/^\d+$/,
            },
        }
    );
  
    $file = $args{file};
    
    ### Translate flags to integer equivalent and OR together
    $oflags |= eval "$args{oflags}" if ($args{oflags});
    
    ### Sysopen doesn't support separate create options,
    ### so append to open flags mask
    $oflags |= eval "$args{cflags}" if ($args{cflags});
    
    ### Open up the file and assign user file handle ID 
    sysopen($fh, $file, $oflags, $args{perms})
        or croak "Could not open file: $file ($!)";
    $fh_map_key = $FH_ID;
    $FH_ID++;
 
    ### Store information in file handle and file maps 
    push @{$MapOf{$file}}, $fh_map_key;
    $File_Handle_Map{$fh_map_key} = {
        file => $file,
        file_handle => $fh,
        offset => 0,
        lock_list => [],
    };
    
    return $fh_map_key;
}


###############################################################################
# Method:    read_file()
# Objective: Read file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub read_file {
    my (%args)=@_;
    my (
        $auto_create_lock,
        $buffer,
        $bytes,
        $err,
        $fh,
        $fh_map_key,
        $file,
        $fl,
    );
    
    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            offset => {
                type => SCALAR,
                optional => 1,
                regex => qr/^\d+$/,
            },
            length => {
                type => SCALAR, 
                default => 0, 
                regex => qr/^\d+$/,
            },
            auto_create_lock => {
                type => SCALAR,
                default => 1,
            },                
        }
    );

    $auto_create_lock = $args{auto_create_lock};

    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    ### Set a default offset if none is specified
    if (!exists $args{offset}) {
        $args{offset} = $File_Handle_Map{$fh_map_key}{offset};
    } else {
        $File_Handle_Map{$fh_map_key}{offset} = $args{offset};
    }

    $Log->comment("OFFSET: ".$args{offset});
    $Log->comment("LENGTH: ".$args{length});
   
    # Create and check the lock before reading (optionally skip
    # this if desired)

    if ($auto_create_lock) {
        ### Create lock before reading
        $fl = new File::FcntlLock;
        $fl->l_type(F_RDLCK);
        $fl->l_whence(SEEK_SET);
        $fl->l_start($args{offset});
        $fl->l_len($args{length});
        $fl->l_pid($$);

        ### Check if lock exists
        $fl->lock($fh, F_SETLK );
        if ($fl->lock_errno == EACCES || $fl->lock_errno == EAGAIN) {
            croak "Can't read due to lock by another process ";  
        }
        $Log->comment("l_start:".$fl->l_start);
    }

    ### Set seek position to requested offset  
    $Log->comment("Performing Seek to ".$args{offset});
    my $rc=sysseek($fh, $args{offset}, SEEK_SET)
        or croak "Could not seek on file $file: $!";
    $Log->comment("seek return:". $rc);
   
    ### Performing file read
    $Log->comment("Performing Read");
    $bytes=sysread($fh, $buffer,$args{length});
    $Log->comment("Bytes Read: ".$bytes);    
    if (not defined $bytes) {
        if ($auto_create_lock) {
            $fl->l_type(F_UNLCK);
            $fl->lock($fh, F_SETLK );
        }            
        croak "Could not read from file $file: $^E";
    }
    if ($auto_create_lock) {
        $fl->l_type(F_UNLCK);
        $fl->lock($fh, F_SETLK ) or
            croak "Unlocking failed: ".$fl->error;
    }            

    ### Increment offset pointer by number of bytes read.
    $File_Handle_Map{$fh_map_key}{offset} += $bytes;
    $Log->comment("BYTES: ".$bytes." read");
    $Log->comment("CRRT OFFSET: ".$File_Handle_Map{$fh_map_key}{offset});
    
    ### Return output buffer
    return $buffer;

}

###############################################################################
# Method:    unlock_file()
# Objective: Unlock file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub unlock_file {
    my (%args)=@_;
    my (
        $err,
        $fh,
        $fh_map_key,
        $file,
        $lcmd,
        $length,
        $offset,
    );
    
    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            lcmd => {
                type => SCALAR,
                default => "F_SETLK",
                callbacks => {
                    'legal values' => sub{shift() =~ /F_SETLK|F_SETLKW/i},
                },
            },
            offset => {
                type => SCALAR, 
                regex => qr/^\d+$/,
            },
            length => {
                type => SCALAR,
                regex => qr/^\d+$/,
            },          
        }
    );

    $offset = $args{offset};
    $length = $args{length};
    $lcmd = $args{lcmd};

    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};
 
    $lcmd |= eval "$args{lcmd}" if $args{lcmd};
    $Log->comment("LCMD: ".$lcmd);

    ### Check that the lock exists and get the lock object
    my @found_locks = grep
        {
            (
                $_->{offset} eq $offset &&
                $_->{length} eq $length &&
                $_->{lcmd} eq $lcmd
            )
        }
        @{$File_Handle_Map{$fh_map_key}{lock_list}};
    if (@found_locks) {
        if (@found_locks > 1) {
            $Log->comment("FOUND LOCKS: ". @found_locks);
            croak "More than one lock record for file $file at offset ".
                "$offset and length $length with lcmd $lcmd found.";
        }
    } else {
        croak "No lock record for file $file at offset $offset ".
            "and length $length with lcmd $lcmd found.";
    }

    my $lockObj = $found_locks[0]->{lock_obj};
    $lockObj->l_type(F_UNLCK);
    $lockObj->lock($fh, $lcmd) or die "Can't unlock file $file\n";

    ### Remove the matching lock from the lock list in the map
    @{$File_Handle_Map{$fh_map_key}{lock_list}} = grep
        {
            !(
                $_->{offset} eq $offset &&
                $_->{length} eq $length &&
                $_->{lcmd} eq $lcmd
            )
        }
        @{$File_Handle_Map{$fh_map_key}{lock_list}};

    return 1;
}

###############################################################################
# Method:    write_file()
# Objective: Write file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub write_file {
    my (%args)=@_;
    my (
        $auto_create_lock,
        $bytes,
        $data,
        $err,
        $fh,
        $fh_map_key,
        $file,
        $fl,
    );
    
    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            contents => {
                type => SCALAR,
            },
            offset => {
                type => SCALAR,
                optional => 1,
                regex => qr/^\d+$/,
            },
            auto_create_lock => {
                type => SCALAR,
                optional => 1,
                default => 1,
            },                
        }
    );

    $auto_create_lock = $args{auto_create_lock};

    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    ### Set a default offset if none is specified
    if (!exists $args{offset}) {
        $args{offset} = $File_Handle_Map{$fh}{offset};
    } else {
        $File_Handle_Map{$fh_map_key}{offset} = $args{offset};
    }

    # Create and check the lock before writing (optionally skip
    # this if desired)
    if ($auto_create_lock) {
        ### Lock area that is to be written
        $fl = new File::FcntlLock;
        $fl->l_type(F_WRLCK);
        $fl->l_whence(SEEK_SET);
        $fl->l_start($args{offset});
        $fl->l_len(length($args{contents}));
        $fl->l_pid($$);

        ### Check if lock exists
        $fl->lock($fh, F_SETLK );
        if ($fl->lock_errno == EACCES || $fl->lock_errno == EAGAIN) {
            croak "Can't write due to lock by another process ";  
        }
    }

    ### Set seek position to requested offset  
    $Log->comment("Performing 'seek' to offset ".$args{offset});
    sysseek($fh, $args{offset}, SEEK_SET)
        or croak "Could not seek on file $file: $!";


    ### Write data       
    $Log->comment("Performing 'write'");
    if ( defined($args{contents}) ) {
        $bytes = syswrite($fh, $args{contents}) or do {
            if ($auto_create_lock) {
                $fl->l_type(F_UNLCK);
                $fl->lock($fh, F_SETLK );
            }                
            croak "Could not write to file $file: $!"; 
        }
    }
    if ($auto_create_lock) {
        $fl->l_type(F_UNLCK);
        $fl->lock($fh, F_SETLK ) or
            croak "Unlocking failed: ".$fl->error;
    }

    ### Increment offset pointer by number of bytes written
    
    $File_Handle_Map{$fh_map_key}{offset} += $bytes;
    $Log->comment("BYTES: ".$bytes." written");
    $Log->comment("CRRT OFFSET: ".$File_Handle_Map{$fh_map_key}{offset});
    
    return 1;

}

###############################################################################
# Method:    _get_fh_map_key()
# Objective: Get the file handle map key based on the input parameters specified
############################################################################### 

sub _get_fh_map_key {
    
    my (%args) = @_;
    my $fh_map_key; 

    # The file or file_handle parameter (but not both) must be specified

    if (!(exists $args{file} xor exists $args{file_handle})) { 
        croak "Either the 'file' or 'file_handle' parameter must be specified.";
    }

    # Determine the file handle to use based on the parameter specified

    if (exists $args{file_handle}) {
        $fh_map_key = $args{file_handle};
        if (!exists $File_Handle_Map{$fh_map_key}) {
            croak "Specified file handle ($args{file_handle}) does not exist.";
        }
    } else {
        if (!exists $MapOf{$args{file}}) {
            croak "File $args{file} is not currently open.";
        }
        $fh_map_key = $MapOf{$args{file}}[0];
    }

    return $fh_map_key;

}    

###############################################################################
# Method:    get_file_locks()
# Objective: Get the current list of successfully requtested locks
# Details:   See POD documentation at the end of this file.
############################################################################### 

sub get_file_locks {
    
    my (%args) = @_;
    my $fh_map_key;
    my @lock_list;

    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
        }
    );
    
    ### Return the list of locks for the file or file handle 
    $fh_map_key = _get_fh_map_key(%args);
    @lock_list = @{$File_Handle_Map{$fh_map_key}{lock_list}};

    $Log->comment("Total Number of Locks = ". @lock_list);
    foreach my $lhash (@lock_list) {
        $Log->comment("Lock Definition:");
        foreach my $key (keys %$lhash) {
            $Log->comment("->  $key = $lhash->{$key}");
        }
    }

    return \@lock_list;

}

1;

############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

Filelock::Unix

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

Filelock::Unix.pm provides an API interface for opening files for exclusive
or shared access. It includes support for byte range locking as well as support
for reading and writing of files.

=head1 See Also

This package is accessible through instantiation of ClientRPC::Interface 
enabling easy execution of locking methods on remote host via RPC 

B<ClientRPC::Interface>

=head1 Public Methods

=head2 B<close_file( B<file>, B<file_handle> )>

Closes an open file handle  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened or the close_file
operation fails, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<file_handle>    

File handle to a file that was earlier opened through use of the I<open_file()> 
method.  If the file_handle was not previously opened or the close_file
operation fails, then an error is thown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Close open filehandle 
    
eval {
   $rc = close_file (file=>"/t/vol1/f1.txt");
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   

=back

=head2 B<delete_file( B<file> )>

Deletes file  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file.  If the operation fails than an error is thrown.

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Delete file 
    
eval {
   $rc = delete_file (file=>"/t/vol1/f1.txt");
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   


=back

=head2 B<lock_file( B<file>, B<file_handle>, I<ltype>, I<lcmd>, I<offset>, I<length> )>

Places a byte range lock on a file that is currently opened. 

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * I<ltype>  default ("F_WRLCK")    

The type of lock to request, write(exclusive) or read(shared).   A single
process can hold only one type of  lock on a file region; if a new lock is
applied to an already-locked region, then the existing lock is converted to
the new lock type.

=begin html
<small>
<ul>
<li><b>F_WRLCK</b>: Write Lock (exclusive lock) - Only one process may hold
       a write lock.  An exclusive lock excludes all other locks, both
       shared and exclusive. 
<li><b>F_RDLCK</b>:  Read Lock (shared lock) may be held by one or more
       processes on a file region.
</ul>
</small><p>

=end html

=item * I<lcmd>  default ("F_SETLK")    

The type of lock request to be made:  waiting or non-waiting

=begin html
<small>
<ul>
<li><b>F_SETLK</b>: If the requested lock cannot be set immediately, the
       call will fail.
<li><b>F_SETLKW</b>: If the requested lock is blocked by other locks, the
       call will block until the lock can be satisfied.
</small><p>

=end html

=item * I<offset>  (default: 0)

Byte offset from start of file to start byte range lock 

=item * I<length>  (default: 0) 

Length of the byte range lock

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Create byte range lock from bytes 10 to 30 
    
eval {
   $rc = lock_file (
            file=>"/t/vol1/f1.txt",
            offset=>10,
            length=>20,
        );
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   

=back

=head2 B<open_file( B<file>, B<oflags>, I<cflags>, I<perms> )>

Creates or opens a file with the requested access including whether the file will
be shared. 

=over 4

=item Inputs

=over 4

=item * B<file>    

The name of the file or device to be created or opened.  If the file path 
is already currently opened than an error is thrown.

=item * B<oflags>    

The requested access to the file which can be summarized as read, write, both.  Access 
rights can be OR'd together.  Value must be passed in as a scalar. 

=begin html
<small>
<ul>
<li><b>O_RDONLY</b>: Read only access to file
<li><b>O_WRONLY</b>: Write only access to file
<li><b>O_RDWR</b>: Read and Write acces to file
</ul>
</small><p>

=end html

=item * I<cflags> 

An action to take on a file that exists or does not exist.  Creation rights
can be OR'd together.  Value must be passed in as a scalar.  

=begin html

<small>
<ul>
<li><b>O_CREAT</b>: If the file does not exist it will be created.
<li><b>O_TRUNC</b>: If the file already exists and is a regular file and
the open  mode allows  writing  (i.e., is O_RDWR or O_WRONLY) it will be
truncated to length 0.
<li><b>O_EXCL</b>: When used with O_CREAT, if the file already exists it is
an  error and  the open() will fail
<li><b>O_APPEND</b>: Append to the file
</ul>
</small><p>

=end html

=item * I<perms> 

The permission on the file in octal form (ex: 0666);

=back

=item Outputs

Returns the file handle on success, throws error on failure

=item Examples

# Example 1: Open up a file for Read Write access. 
    
eval {
   $fh = open_file (
            file=>"/t/vol1/f1.txt",
            oflags=>"O_RDWR",
        );
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   

=back

=head2 B<read_file( B<file>, B<file_handle>, I<offset>, I<length>,
                    I<auto_create_lock> )>

Read data from an open file  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * I<offset>  (default: 0|'end of last read')

Byte offset to start from when reading from file.  If an offset is not provided, 
subsequent reads will start from where the last read ended.

=item * I<length>  (default: 0) 

Length of bytes to read starting at file offset.  If the length extends 
the file size than only the data between the offset to the EOF is returned.

=item * I<auto_create_lock>  (default: 1)

Optionally create a lock and check the status before attempting to read
the file and then remove the lock after the read operation.

=back

=item Outputs

Returns data read from file. Throws error on failure

=item Examples

 # Example 1: Read in chunks of 10 bytes to EOF starting at offset 50 

 eval {
    # Prime where to start reading in file
    $buffer = read_file (
       file=>"/t/vol1/f1.txt",
       offset=>50,
       length=>0);
    # Loop through file until all data has
    # been read.
    while ($data) {
       $data = read_file (
       file=>"/t/vol1/f1.txt",
       length=>10);
	   # Append data to 
	   # buffer
       $buffer.=$data;
    }
    $Test->nlog("Contents: ",$buffer);
 }; 
 if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
 }

=back

=head2 B<unlock_file( B<file>, B<file_handle>, I<lcmd>, I<offset>, I<length> )>

Removes a byte range lock that was placed on an open file.  A lock with the 
same offset and length must exist for the method to work

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.
 
Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * I<lcmd>  default ("F_SETLK")    

The type of unlock request to be made:  waiting or non-waiting

=begin html
<small>
<ul>
<li><b>F_SETLK</b>: If the requested unlock cannot be set immediately, the
       call will fail.
<li><b>F_SETLKW</b>: If the requested unlock is blocked by other locks, the
       call will block until the lock can be satisfied.
</small><p>

=end html

=item * I<offset>  (default: 0)

Byte offset from start of file where byte range lock starts 

=item * I<length>  (default: 0) 

Length of the byte range lock

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

   # Example 1: Remove byte range lock from bytes 10 to 30 
    
   eval {
      unlock_file (
		file=>"/t/vol1/f1.txt",
		offset=>10,
		length=>20,
      );
   );
   if ($@) {
      $Test->nlog($@);
      return $TCD:FAIL;
   }   

=back

=head2 B<write_file( B<file>, B<file_handle>, B<contents>, I<length>,
                     I<auto_create_lock> )>

Write data to an open file  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<contents> 

Data to be written to file

=item * I<length>  (default: 0) 

File offset to start writing contents.  

=item * I<auto_create_lock>  (default: 1)

Optionally create a lock and check the status before attempting to write 
the file and then remove the lock after the write operation.

=back

=item Outputs

Returns 1 on successful write. Throws error on failure

=item Examples

 # Example 1: Write a string of 50 characters starting at offset 500 

 eval {
   # Prime where to start reading in file
   write_file (
     file=>"/t/vol1/f1.txt",
     offset=>50,
     contents=>"A"x50);
 };
 if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
 }   

=back

=head2 B<get_file_locks ( B<file>, B<file_handle> )>

Gets a reference to an array of the current succesfully requested locks.  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=back

=item Outputs

Returns a reference to an array of hashes where each hash contains the 
information for a lock (hash keys are offset, length, ltype, lcmd, and
lock_obj).  Throws an error on failure.

=item Examples

 # Example 1: Get a reference to the array of requested locks 

 eval {
   $lock_ref = get_file_locks(file=>"/t/vol1/f1.txt");
 };
 if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
 }   

=back

