# Hey emacs, this is basically -*- perl -*-
# vim: set filetype=perl tabstop=4 shiftwidth=4 shiftround expandtab:
# $Id: //depot/prod/test/Rsierranevada/lib/Filelock/Win32.pm#1 $

## @summary     Class for basic win32 file locking
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here
###############################################
##   NOT READY FOR PUBLIC CONSUMPTION
###############################################
package Filelock::Win32;

use strict;
use Carp qw(croak);
use Data::Dumper;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use Tharn qw ($Log);
use Win32::API;
use Win32API::File qw(
   :Func :Misc :FILE_ :FILE_FLAG_ :FILE_SHARE_ :FILE_ATTRIBUTE_ :GENERIC_ );
use constant KERNEL32 => 'C:\WINDOWS\system32\kernel32.dll';

my (%MapOf,$LockObj,$UnlockObj,%File_Handle_Map);
my $FH_ID = 1;

###############################################################################
# Method:    close_file()
# Objective: Close file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub close_file {
    my (%args)=@_;
    my (
        $fh,
        $fh_map_key,
        $file,
    );
    
    ### Validate input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },              
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
        }
    );

    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    ### Close the file 
    Win32API::File::CloseHandle($fh) or
        croak "Could not close file $file: $!";
  
    ### Delete the corresponding entries from the file handle and file maps
    delete $File_Handle_Map{$fh};
    @{$MapOf{$file}} = grep {!($_ eq $fh_map_key)} @{$MapOf{$file}};
    if (!$MapOf{$file}) {
        delete $MapOf{$file};
    } 

    return 1;
}

###############################################################################
# Method:    delete_file()
# Objective: Delete file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub delete_file {
    my (
        $fh,
        $file,
    );
    
    ### Validate the input arguments
    my (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
            },
        }
    );

    $file = $args{file};

    ### Delete the file
    unlink($file) or croak "Could not delete file $file: $!";

    ### Delete all corresponding entries in the file handle and file maps
    foreach my $fh_map_key (@{$MapOf{$file}}) {
        delete $File_Handle_Map{$fh_map_key};
    }
    delete $MapOf{$file};
   
    return 1;
}
###############################################################################
# Method:    lock_file()
# Objective: Lock file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub lock_file {
    my (%args)=@_;
    my (
        $fh,
        $fh_map_key,
        $file,
        $legal,
        $length,
        $ltype,
        $offset,
        @inputs,
    );

    use constant LOCKFILE_FAIL_IMMEDIATELY => 1;
    use constant LOCKFILE_EXCLUSIVE_LOCK   => 2; 
    
    my @ltypes=qw(
        LOCKFILE_FAIL_IMMEDIATELY
        LOCKFILE_EXCLUSIVE_LOCK
    );
 

    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },                
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            ltype => {
                type => SCALAR,
                default => "LOCKFILE_FAIL_IMMEDIATELY",
                callbacks => {
                    'Legal values' => sub{
                        @inputs = split(/\|/,shift());
                        $legal = join("|",@ltypes);
                        not scalar(grep{$_ !~ /^$legal$/} @inputs);
                    },
                },
            }, 
            offset => {
                type => SCALAR,
                default => 0,
                regex => qr/^\d+$/,
            },
            length => {
                type => SCALAR,
                default => 0,
                regex => qr/^\d+$/,
            },
        }
    );

    $offset = $args{offset};
    $length = $args{length};

    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};
    
    ### Determine the lock type 
    $ltype |= eval "$args{ltype}" if ($args{ltype});
     
    ### Create function prototype for creating locks    
    unless (defined $LockObj){
        $Log->comment("Creating LockObj");
        $LockObj = new Win32::API(KERNEL32, 'LockFileEx','NNNNNP','N') or 
            croak "LockFileEx: ".$^E;
    }
    
    ### Call function prototype in eval block since prototypes
    ### throw die on error.
    eval {
        
        ### Set file pointer to correct offset
        $Log->comment("Performing SEEK to offset ".$offset);  
        Win32API::File::setFilePointer($fh, $offset, FILE_BEGIN)
            or croak "Could not seek on file $file: ".$^E;
        
        my $status=$LockObj->Call($fh,$ltype,0,$length,0,
            pack('PPLLL', 0, 0, $offset, 0, 0)
        );
        
        $Log->comment("status = ".$status);
       
    };
    if ($@) {
        chomp $@;
        croak "".(caller(0))[3].": ".$^E." (".$@.")";
    }
    
    ### Add the lock information to the array of locks stored in the map
    push @{$File_Handle_Map{$fh_map_key}{lock_list}},
        {
            offset => $offset,
            length => $length,
            ltype => $ltype,
            lock_obj => $LockObj,
        };

    return 1;
}

###############################################################################
# Method:    open_file()
# Objective: Open file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub open_file {
    my (%args)=@_;
    my (
        $fh,
        $fh_map_key,
        $file,
        $legal,
        $cflags,
        $oflags,
        $sflags,
        @inputs,
    );
  
    my @oflags=qw(
        GENERIC_WRITE
        GENERIC_READ
        GENERIC_EXECUTE
        GENERIC_ALL
    );
        
    my @sflags=qw(
        FILE_SHARE_READ
        FILE_SHARE_WRITE
        FILE_SHARE_DELETE
        FILE_SHARE_EXECUTE
    );
        
    my @cflags=qw(
        OPEN_ALWAYS
        OPEN_EXISTING
        TRUNCATE_EXISTING
        CREATE_ALWAYS
        CREATE_NEW
    );

    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
            },
            oflags => {
                type => SCALAR,
                callbacks => {
                    'Legal values' => sub{
                        @inputs = split(/\|/, shift());
                        $legal = join("|", @oflags);
                        not scalar(grep{$_ !~ /^$legal$/} @inputs);
                    },
                },
            },
            sflags => {
                type => SCALAR,
                optional => 1,
                callbacks => {
                    'Legal values' => sub{
                        @inputs = split(/\|/, shift());
                        $legal = join("|", @sflags);
                        not scalar(grep{$_ !~ /^$legal$/} @inputs);
                    },
                },
            },
            cflags => {
                type => SCALAR,
                default => "OPEN_ALWAYS",
                callbacks => {
                    'Legal values' => sub{
                        @inputs = split(/\|/, shift());
                        $legal = join("|", @cflags);
                        not scalar(grep{$_ !~ /^$legal$/} @inputs);
                    },
                },
            },
        }
    );                      
    
    $file = $args{file};
    
    ### Append and transform flag back to constant form
    $oflags |= eval "$args{oflags}" if ($args{oflags});
    $sflags |= eval "$args{sflags}" if ($args{sflags});
    $cflags |= eval "$args{cflags}" if ($args{cflags});
   
    $Log->comment(
        "oflags=".$args{oflags}." (".$oflags.")\n".
        "sflags=".$args{sflags}." (".$sflags.")\n".
        "cflags=".$args{cflags}." (".$cflags.")"
    );
    
    ### Open up the file and assign a user file handle
    $fh_map_key = $FH_ID;
    $FH_ID++;
    $fh=Win32API::File::CreateFile($file,$oflags,$sflags,[],
                                   $cflags,[],[]) or 
        croak "Could not open file: $file ($^E)";

    ### Store filehandle and set offset to 0
    push @{$MapOf{$file}}, $fh_map_key;
    $File_Handle_Map{$fh_map_key} = {
        file => $file,
        file_handle => $fh,
        offset => 0,
        lock_list => [],
    };
                                               
    return $fh_map_key;

}

###############################################################################
# Method:    read_file()
# Objective: Read file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub read_file {
    my (%args)=@_;
    my (
        $buffer,
        $bytes,
        $err,
        $fh,
        $fh_map_key,
        $file,
        $rc,
    );
    
    ### Validate input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },                
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            offset => {
                type => SCALAR,
                optional => 1,
                regex => qr/^\d+$/,
            },                
            length => {
                type => SCALAR,
                default => 0,
                regex => qr/^\d+$/,
            },          
        }
    );
    
    ### Determine the file handle to use 
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    ### Set a default offset if none is specified
    if (!exists $args{offset}) {
        $args{offset} = $File_Handle_Map{$fh_map_key}{offset};
    } else {
        $File_Handle_Map{$fh_map_key}{offset} = $args{offset};
    }

    ### Set file pointer to correct offset
    $Log->comment("Performing SEEK to offset ".$args{offset});  
    Win32API::File::setFilePointer($fh, $args{offset}, FILE_BEGIN) 
        or croak "Could not seek on file $file: ".$^E;
    
    ### Performing file read      
    $Log->comment("Performing READ");
    Win32API::File::ReadFile($fh,$buffer,$args{length},$bytes,[])
        or croak "Could not read from file $file: ".$^E;
   
    ### Increment offset pointer by number of bytes read.
    $Log->comment("BYTES: ".$bytes." read");
    $File_Handle_Map{$fh_map_key}{offset} += $bytes;
    $Log->comment("CRRT OFFSET: ".$File_Handle_Map{$fh_map_key}{offset});

    ### Return output buffer
    return $buffer;

}

###############################################################################
# Method:    unlock_file()
# Objective: Unlock file
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub unlock_file{
    my (%args)=@_;
    my (
        $fh,
        $fh_map_key,
        $file,
        $length,
        $offset,
    );

    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            offset => {
                type => SCALAR,
                default => 0,
                regex => qr/^\d+$/,
            },
            length => {
                type => SCALAR,
                default => 0,
                regex => qr/^\d+$/,
            },
        }
    );

    $length = $args{length};
    $offset = $args{offset};

    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    ### Create function prototype for creating locks    
    unless (defined $UnlockObj){ 
        $Log->comment("Creating UnlockObj");
        $UnlockObj= new Win32::API(KERNEL32,'UnlockFileEx','NNNNP','N');
        croak "UnlockfileEx: ".$^E unless ($UnlockObj);
    }

    ### Check that the lock exists
    my @found_locks = grep
        {
            (
                $_->{offset} eq $offset &&
                $_->{length} eq $length
            )
        }
        @{$File_Handle_Map{$fh_map_key}{lock_list}};
    if (@found_locks) {
        if (@found_locks > 1) {
            $Log->comment("FOUND LOCKS: ". @found_locks);
            croak "More than one lock record for file $file at offset ".
                "$offset and length $length found.";
        }
    } else {        
        croak "No lock record for file $file at offset $offset ".
            "and length $length found.";
    }

    ### Call function prototype in eval block since prototypes
    ### throw die on error.
   
    eval {
        $UnlockObj->Call($fh,0,$length,0,
            pack('PPLLL', 0, 0, $offset, 0, 0));
    };
    if ($@) {
        chomp $@;
        croak "".(caller(0))[3].": ".$^E." (".$@.")";
    }

    ### Remove the matching lock from the ock list in the map
    @{$File_Handle_Map{$fh_map_key}{lock_list}} = grep
        {
            !(
                $_->{offset} eq $offset &&
                $_->{length} eq $length
            )
        }
        @{$File_Handle_Map{$fh_map_key}{lock_list}};
   
    return 1;
}

###############################################################################
# Method:    write_file()
# Objective: Write file
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub write_file {
    my (%args)=@_;
    my (
        $bytes,
        $err,
        $fh,
        $fh_map_key,
        $file,
    );

    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR ,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
            contents => {
                type => SCALAR,
            },
            offset => {
                type => SCALAR,
                optional => 1,
                regex => qr/^\d+$/,
            },
        }
    );
     
    ### Determine the file handle to use
    $fh_map_key = _get_fh_map_key(%args);
    $fh = $File_Handle_Map{$fh_map_key}{file_handle};
    $file = $File_Handle_Map{$fh_map_key}{file};

    # Set a default offset if none is specified
    if (!exists $args{offset}) {
        $args{offset} = $File_Handle_Map{$fh_map_key}{offset};
    } else {
        $File_Handle_Map{$fh_map_key}{offset} = $args{offset};
    }

    # Set file pointer to correct offset
    $Log->comment("Performing SEEK to offset ".$args{offset});  
    Win32API::File::setFilePointer($fh, $args{offset}, FILE_BEGIN) or 
        croak "Could not seek on file $file: ".$^E;
    $Log->comment("SEEK Done");  
    
    $Log->comment("Performing WRITE");
    Win32API::File::WriteFile($fh, $args{contents}, 0, $bytes, []) or 
        croak "Could not write to file $file: ".$^E;
    $Log->comment("WRITE Done");  

    ### Increment offset pointer by number of bytes written
    $Log->comment("BYTES: ".$bytes." written");
    $File_Handle_Map{$fh_map_key}{offset} += $bytes;
    $Log->comment("CRRT OFFSET: ".$File_Handle_Map{$fh_map_key}{offset});

    return 1;

}
###############################################################################
# Method:    _get_fh_map_key()
# Objective: Get the file handle map key based on the input parameters specified
###############################################################################

sub _get_fh_map_key {

    my (%args) = @_;
    my $fh_map_key;

    # The file or file_handle parameter (but not both) must be specified

    if (!(exists $args{file} xor exists $args{file_handle})) {
        croak "Either the 'file' or 'file_handle' parameter must be specified.";
    }

    # Determine the file handle to use based on the parameter specified

    if (exists $args{file_handle}) {
        $fh_map_key = $args{file_handle};
        if (!exists $File_Handle_Map{$fh_map_key}) {
            croak "Specified file handle ($args{file_handle}) does not exist.";
        }
    } else {
        if (!exists $MapOf{$args{file}}) {
            croak "File $args{file} is not currently open.";
        }
        $fh_map_key = $MapOf{$args{file}}[0];
    }        
    
    return $fh_map_key;
    
}

###############################################################################
# Method:    get_file_locks()
# Objective: Get the current list of successfully requtested locks
# Details:   See POD documentation at the end of this file.
###############################################################################

sub get_file_locks {

    my (%args) = @_;
    my $fh_map_key;
    my @lock_list;

    ### Validate the input arguments
    (%args) = validate (
        @_,
        {
            file => {
                type => SCALAR,
                optional => 1,
            },
            file_handle => {
                type => SCALAR,
                optional => 1,
            },
        }
    );

    ### Return the list of locks for the file or file handle
    $fh_map_key = _get_fh_map_key(%args);
    @lock_list = @{$File_Handle_Map{$fh_map_key}{lock_list}};

    $Log->comment("Total Number of Locks = ". @lock_list);
    foreach my $lhash (@lock_list) {
        $Log->comment("Lock Definition:");
        foreach my $key (keys %$lhash) {
            $Log->comment("->   $key = $lhash->{$key}");
        }
    }

    return \@lock_list;

}

1;

############################################################################
#  P O D
############################################################################
=pod

=head1 Package Name

Filelock::Win32

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

Filelock::Win32.pm provides an API interface for opening files for exclusive
or shared access. It includes support for byte range locking as well as support
for reading and writing of files.

=head1 See Also

This package is accessible through instantiation of ClientRPC::Interface 
enabling easy execution of locking methods on remote host via RPC  

B<ClientRPC::Interface>

=head1 Public Methods

=head2 B<close_file( B<file>, B<file_handle> )>

Closes an open file handle  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened or the close_file operation
fails, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened or the close_file operation
fails, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Close open filehandle 
    
eval {
   $rc = close_file (file=>"/t/vol1/f1.txt");
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   


=back

=head2 B<delete_file( B<file> )>

Deletes file  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file.  If the operation fails than an error is thrown.

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Delete file 
    
eval {
   $rc = delete_file (file=>"/t/vol1/f1.txt");
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   


=back

=head2 B<lock_file( B<file>, B<file_handle>, I<ltype>, I<offset>, I<length> )>

Places a byte range lock on a file that is currently opened. 

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * I<ltype>  default ("LOCKFILE_FAIL_IMMEDIATELY")    

The type of lock to request, exclusive or shared, including whether to wait
or return on failure if a lock is unable to be acquired

=begin html
<small>
<ul>
<li><b>LOCKFILE_EXCLUSIVE_LOCK</b>: The function requests an exclusive lock.
Otherwise, it requests a shared lock.
<li><b>LOCKFILE_FAIL_IMMEDIATELY</b>: The function returns immediately if it is
unable to acquire the requested lock. Otherwise, it waits.
</ul>
</small><p>

=end html

=item * I<offset>  (default: 0)

Byte offset from start of file to start byte range lock 

=item * I<length>  (default: 0) 

Length of the byte range lock

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Create byte range lock from bytes 10 to 30 
    
eval {
   $rc = lock_file (
            file=>"/t/vol1/f1.txt",
            offset=>10,
            length=>20,
        );
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   

=back

=head2 B<open_file( B<file>, B<oflags>, I<sflags>, I<cflags> )>

Creates or opens a file with the requested access including whether the file will
be shared. 

=over 4

=item Inputs

=over 4

=item * B<file>    

The name of the file or device to be created or opened.  If the file path 
is already currently opened than an error is thrown.

=item * B<oflags>    

The requested access to the file which can be summarized as read, write, both.  Access 
rights can be OR'd together.  Value must be passed in as a scalar. 

=begin html
<small>
<ul>
<li><b>GENERIC_ALL</b>:	Read, write, and execute access
<li><b>GENERIC_EXECUTE</b>:	Execute access
<li><b>GENERIC_READ</b>: Read access
<li><b>GENERIC_WRITE</b>: Write access
</ul>
</small><p>

=end html

=item * I<sflags> 

The requested sharing mode of the file, which can be read, write, 
both, delete, all of these. Share rights can be OR'd together.  Value must 
be passed in as a scalar.

=begin html

<small>
<ul>
<li><b>FILE_SHARE_DELETE</b>: Enables subsequent open operations on a file or device to request delete access.
<li><b>FILE_SHARE_READ</b>: Enables subsequent open operations on a file or device to request read access.
<li><b>FILE_SHARE_WRITE</b>: Enables subsequent open operations on a file or device to request write access.
</ul>
</small><p>

=end html


=item * I<cflags>  (default: "OPEN_ALWAYS") 

An action to take on a file that exists or does not exist.  Share rights can be OR'd 
together.  Value must be passed in as a scalar.  

=begin html

<small>
<ul>
<li><b>CREATE_ALWAYS</b>: If the specified file exists and is writable, the function 
overwrites the file, the function succeeds, and last-error code is set to 
ERROR_ALREADY_EXISTS (183). If the specified file does not exist and is a valid path,
a new file is created, the function succeeds, and the last-error code is set to zero.
<li><b>CREATE_NEW</b>: Creates a new file, only if it does not already exist. If the 
specified file exists, the function fails and the last-error code is set to 
ERROR_FILE_EXISTS (80). If the specified file does not exist and is a valid path to a 
writable location, a new file is created.
<li><b>OPEN_ALWAYS</b>: Opens a file, always. If the specified file exists, the function
succeeds and the last-error code is set to ERROR_ALREADY_EXISTS (183).If the specified 
file does not exist and is a valid path to a writable location, the function creates a
file and the last-error code is set to zero.
<li><b>OPEN_EXISTING</b>: Opens a file or device, only if it exists. If the specified
file or device does not exist, the function fails and the last-error code is set to 
ERROR_FILE_NOT_FOUND (2).
<li><b>TRUNCATE_EXISTING</b>: Opens a file and truncates it so that its size is zero 
bytes, only if it exists.
</ul>
</small><p>

=end html

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

# Example 1: Open up a file for Read Write access with share mode Read. 
    
eval {
   $rc = open_file (
            file=>"/t/vol1/f1.txt",
            oflags=>"GENERIC_READ|GENERIC_WRITE",
            sflags=>"FILE_SHARE_READ"
        );
);
if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
}   

=back

=head2 B<read_file( B<file>, B<file_handle>, I<offset>, I<length> )>

Read data from an open file  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * I<offset>  (default: 0|'end of last read')

Byte offset to start from when reading from file.  If an offset is not provided, 
subsequent reads will start from where the last read ended.

=item * I<length>  (default: 0) 

Length of bytes to read starting at file offset.  If the length extends 
the file size than only the data between the offset to the EOF is returned.

=back

=item Outputs

Returns data read from file. Throws error on failure

=item Examples

 # Example 1: Read in chunks of 10 bytes to EOF starting at offset 50 

 eval {
    # Prime where to start reading in file
    $buffer = read_file (
       file=>"/t/vol1/f1.txt",
       offset=>50,
       length=>0);
    # Loop through file until all data has
    # been read.
    while ($data) {
       $data = read_file (
       file=>"/t/vol1/f1.txt",
       length=>10);
	   # Append data to 
	   # buffer
       $buffer.=$data;
    }
    $Test->nlog("Contents: ",$buffer);
 }; 
 if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
 }

=back

=head2 B<unlock_file( B<file>, B<file_handle>, I<offset>, I<length> )>

Removes a byte range lock that was placed on an open file.  A lock with the 
same offset and length must exist for the method to work

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * I<offset>  (default: 0)

Byte offset from start of file where byte range lock starts 

=item * I<length>  (default: 0) 

Length of the byte range lock

=back

=item Outputs

Returns 1 on success, throws error on failure

=item Examples

   # Example 1: Remove byte range lock from bytes 10 to 30 
    
   eval {
      unlock_file (
		file=>"/t/vol1/f1.txt",
		offset=>10,
		length=>20,
      );
   );
   if ($@) {
      $Test->nlog($@);
      return $TCD:FAIL;
   }   

=back

=head2 B<write_file( B<file>, B<file_handle>, B<contents>, I<length> )>

Read data from an open file  

=over 4

=item Inputs

=over 4

=item * B<file>    

Full path to file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * B<file_handle>    

File handle for a file that was earlier opened through use of the I<open_file()> 
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be specified.

=item * B<contents> 

Data to be written to file

=item * I<length>  (default: 0) 

File offset to start writing contents.  

=back

=item Outputs

Returns 1 on successful write. Throws error on failure

=item Examples

 # Example 1: Write a string of 50 characters starting at offset 500 

 eval {
   # Prime where to start reading in file
   write_file (
     file=>"/t/vol1/f1.txt",
     offset=>50,
     contents=>"A"x50);
 };
 if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
 }   

=back

=head2 B<get_file_locks ( B<file>, B<file_handle> )>

Returns a reference to an array of the current succesfully requested locks.

=over 4

=item Inputs

=over 4

=item * B<file>

Full path to file that was earlier opened through use of the I<open_file()>
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=item * B<file_handle>

File handle for a file that was earlier opened through use of the I<open_file()>
method.  If the file was not previously opened, then an error is thrown.

Either the B<file> or B<file_handle> parameter (but not both) must be
specified.

=back

=item Outputs

Returns a reference to an array of hashes where each hash contains the 
information for a lock (hash keys are offset, length, ltype and lock_obj).
Throws an error on failure.

=item Examples

 # Example 1: Get a reference to the array of requested locks
 
 eval {
   $lock_ref = get_file_locks(file=>"/t/vol1/f1.txt");
 };
 if ($@) {
   $Test->nlog($@);
   return $TCD:FAIL;
 }

=back

=cut
