# Hey, emacs, this is basically -*-perl-*-
#$Id: //depot/prod/test/Rsierranevada/lib/Filer/Upgrade.pm#1 $
#
#Copyright (c) 2004 Network Appliance, Inc.
#All rights reserved.
#
## @summary     Upgrade library
##
## @examples
##
##    my $up = Filer::Upgrade->new($conn,
##                                 to_version => $TO_VERSION,
##                                 from_version => $FROM_VERSION,
##                                 FILER => $FILER,
##                                 rootvol_mount=>$rootvol_mount)
##                   || die ("no upgrade object");
##
##   die config_result("upgrade not allowed") unless $up->allowed();
##
##   my $result=$up->do_upgrade();
##
## @status public
##
## @author   shenry@netapp.com (originally amandag, conor)

package Filer::Upgrade;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use Filer::Transform;

use Tharn;
use SPARSE::Util::Logger;
use Carp;
use boot; # do we need this? FIXME
use Filer::Version qw(version_compare);



@ISA = qw(Filer::Transform);

# ---------------------
# sub new()
# Constructor requires a connection record as the first argument
# All optional parameters are supplied in a named fashion. If not
# supplied, the "to_version" argument will default to undef
# ---------------------
sub new {
  my ($class, @args) = @_;
  my $self = $class->SUPER::new(@args);
  return $self;
}


# --------------------
# sub _version_allows
# _version_allows checks whether the "to_version" that's been
# requested is compatible with the acceptable previous versions
# returns 1 if allowed 0 if not
# --------------------
sub _version_allows {
    my $self = shift;

    # first test that that we are actually upgrading, that the to_version is a
    # bigger number than the from_version
    # FIXME #
    # This is probably somewhat obsolete, refer to Filer::Transform and 
    # Filer::Version for more information about version testing
    if (version_compare($self->{_to_version}, $self->{_from_version}) != 1) {
	logWarn("from_version ($self->{_from_version}) is >= to_version ($self->{_to_version}). Not an upgrade!");
	return 0;
    }

    # to quote Mary Vetter
    # systems with PC cards had to go through another release, before they
    # could be upgraded to Anchorsteam.  The release that the systems have
    # to go through is 6.4.2.  The firmware was upgraded that release to FAT
    # format and the system has to be running firmware 4.2.2 or later to be
    # able to upgrade beyond that point.
    # FIXME this also seems to be buggy -- I am not certain if it is
    # checking correctly

    if ( (version_compare("6.4.2", $self->{_from_version}) == 1) &&
	 (version_compare($self->{_to_version}, "6.4.2") == 1) ) {
	my ($match, undef, $firmrev) = say($self->conn(),
					 "sysconfig -v 0",
					 "Firmware release:"
					 ,'-eval_error');
	$firmrev =~ s/.*?\s+(.?\d+\.\d+\.\d+).*/$1/;
        logDbg("Firmware rev = $firmrev");
	if (version_compare($firmrev, "4.2.2") == -1) {
	    logWarn("For this upgrade you must upgrade to release 6.4.2 first");
	    return 0;
	}
    }
    # otherwise all is well for upgrade
    return 1;
}

# --------------------
# sub _running_status_allows
# inherited
# --------------------

# --------------------
# sub _min_max_allows
# inherited
# --------------------

# --------------------
# sub _hardware_allows
# inherited
# --------------------

# --------------------
# sub _config_allows
# inherited
# --------------------


# --------------------
# sub max_snaps
# find out the max number of snapshots allowed for this filer version
# --------------------
sub max_snaps {
    my $self = shift if ref $_[0];
    # current version greater than or equal to 6.4
    if ($self->v_compare($self->{_from_version}, "6.4") >= 0) {
	return ("255");
    }
    # current version equal to 6.3
    if ($self->v_compare($self->{_from_version}, "6.3") == 1) {
	return ("32");
    }
    # else return unknown or zero
    return 0;
}

# --------------------
# sub do_upgrade
# Optional params
#  REBOOT => [0|NO] 
#  releases => [releases|releases_pbo|releases_dbg]
# --------------------
sub do_upgrade {
    my $self = shift;
    my %parms = @_;
    unless ($ENV{DONOTREALLYUPGRADE} == 1 || 
            $::DONOTREALLYUPGRADE == 1) {
      $self->install_release(@_) or croak "Could not install"; 
      $self->download_kernel(@_) or croak "Could not download kernel"; 
      $self->do_reboot(@_) or croak "Could not reboot"; 
    }
    return 1;
}

# --------------------
# sub do
# This isn't working I don't think so it is a FIXME
# was just a stub so that Upgrade/Revert do() would be same-named
# --------------------
sub do { shift->(do_upgrade(@_)) }

1;
