#!perl -w

package Finisar::Frunner;

# Main package for the Finisar test utilities
# This package calls the other packages and set some default
# values to minimize the amount of code the users have to put
# into their test cases.

use Exporter;

use vars qw(@ISA @EXPORT); 

@ISA = qw(Exporter);
@EXPORT = qw(Endtest %OPT);

push (@main::options,"debug","sim");

use Finisar::Usage;

newGetOptions;

use Finisar::Extern;
use Finisar::Utils;

# Module for the Physical Tester
use Finisar::PT;

# Module for the Trace Controller
use Finisar::TC;

# Module for the XgigJM
use Finisar::XJ;

use strict;
use warnings;


# Initialize code

sub Endtest($) {
    my($exitCode) = @_;

    Utils::CloseSockets();

    exit($exitCode);
}

sub New {
    my $self = {} ;
    $self->{'name'} = shift;
    my $host = shift;
    my $port = shift;
    my $location = shift;

    if ( !defined($port) || $port eq "" ) {
	if ( $ENV{'FRUNNER_REQUIRE_PORT'} ) {
	    print("ERROR - You must explicitely give the port number to create an Frunner object\n");
	    exit(1);
	} else {
	    $port = 1001;
	}
    }

    $self->{'host'} = $host . ":" . $port;

    bless $self;

    # If a location is provided, we have to try talking to it,
    # if it does not answer, we try to start it.
    if ( defined($location) && $location ne "" ) {
	$ENV{'FRUNNER_LOCATION'} = $location;
	SendFrunner($self,"Frunner New");
    }

    return($self);
}


sub Clean($) {
    my($self) = @_;

    SendFrunner($self, "Frunner Clean");
}


sub Clear($) {
    my($self) = @_;

    SendFrunner($self, "Frunner Clear");
}


sub Configdir($$) {
    my($self, $configDir) = @_;

    $configDir = QuoteFile($configDir);
    SendFrunner($self, "Frunner Configdir $configDir");
}


sub DestroyDomain($$) {
    my($self, $domainId) = @_;

    SendFrunner($self, "Frunner DestroyDomain $domainId");
}


sub Exit($) {
    my($self) = @_;

    SendFrunner($self, "Frunner Exit");
}


sub GetReturnCode($) {
    my ($self) = @_;

    return($self->{'returnCode'});
}


sub Getreturncode($) {
    my ($self) = @_;

    return($self->{'returnCode'});
}


sub Log($$) {
    my($self, $logLevel) = @_;

    SendFrunner($self, "Frunner Log $logLevel");
}

sub LogToFile($$$) {
    my($self, $logLevel, $logFile) = @_;

    $logFile = QuoteFile($logFile);
    SendFrunner($self, "Frunner LogToFile $logLevel $logFile");
}


sub Resultdir($$) {
    my($self, $resultDir) = @_;

    $resultDir = QuoteFile($resultDir);
    SendFrunner($self, "Frunner Resultdir $resultDir");
}


sub Reuseobj($$) {
    my($self, $value) = @_;

    SendFrunner($self, "Frunner Reuseobj $value");
}


sub Set($$$) {
    my($self, $key, $value) = @_;

    SendFrunner($self, "Frunner Set $key $value");
    return(0);
}


sub SetAutoDomain($$) {
    my($self,$value) = @_;

    Finisar::Utils::SetAutoDomain($value);
    # TODO - Should we return something else?
    return(0);
}


sub SetWaitOnSocket($$) {
    my($self, $newWait) = @_;

    Finisar::Utils::WaitOnSocket($newWait);
    # TODO - Should we return something else?
    return(0);
}


sub Shell($$) {
    my($self, $shellCmd) = @_;

    SendFrunner($self, "Frunner Shell $shellCmd");
}


sub StepMode($) {
    my($self) = @_;

    SendFrunner($self, "Frunner StepMode");
}


sub Ver($) {
    my($self) = @_;

    SendFrunner($self, "Frunner Ver");
    return($main::returnValue);
}


sub Visible($$) {
    my($self,$state) = @_;

    SendFrunner($self, "Frunner Visible $state");
}


1;
