#!perl -w

package Finisar::PT;

# Module to handle a physical tester
# This module contains all the classes for the object
# "physical tester".

use Exporter;

use Finisar::Extern;
use Finisar::Utils;

use strict;
use warnings;

use vars qw(@ISA @EXPORT);

@ISA = qw(Exporter);

sub New {
    my $self = {} ;
    $self->{'name'} = shift;
    
    my $host = shift;
    my $port = shift;
    if ( !defined($port) || $port eq "" ) {
	if ( $ENV{'FRUNNER_REQUIRE_PORT'} ) {
	    print("ERROR - You must explicitely give the port number to create a PT object\n");
	    exit(1);
	} else {
	    $port = 1001;
	}
    }

    $self->{'host'} = $host . ":" . $port;

    $self->{'configDir'}   = $configDir;
    $self->{'configFile'}  = undef;   
    $self->{'logDir'}      = $logDir;
    $self->{'logFile'}     = "ptscript.log"; 
    $self->{'password'}    = "";
    $self->{'returnCode'}  = "";
    $self->{'returnValue'} = "";
    $self->{'running'}     = 0;  
    $self->{'startTime'}   = 5;
    $self->{'user'}        = "";

    bless $self;

    SendFrunner($self,"$self->{'name'} New PhysicalTester");

    return $self ;
}

sub Configdir($$) {
    my($self, $configDir) = @_;

    $configDir = QuoteFile($configDir);
    SendFrunner($self, "$self->{'name'} Configdir $configDir");
}

sub Exit($) {
    my($self) = @_;

    SendFrunner($self, "$self->{'name'} Exit");
}

sub GetReturnCode($) {
    my($self) = @_;

    return($self->{'returnCode'});
}

sub Load($$) {
    my($self, $configFile) = @_;

    $configFile = QuoteFile($configFile);
    SendFrunner($self, "$self->{'name'} Load $configFile");
}

sub Metric($$) {
    my($self, $metricName) = @_;
    
    SendFrunner($self, "$self->{'name'} Metric $metricName");
    $self->{'returnValue'} = $main::returnValue;
    return($main::returnValue);
}

sub Personality($$) {
    my ($self, $personalityType) = @_;

    SendFrunner($self, "$self->{'name'} Personality $personalityType");
}

sub Port($$$$$$) {
    my ($self, $chassisIP, $ordinal, $blade, $firstPort, $nbPort) = @_;

    SendFrunner($self, "$self->{'name'} Port $chassisIP $ordinal $blade $firstPort $nbPort");
}

sub Speed($$) {
    my ($self, $speed) = @_;

    SendFrunner($self, "$self->{'name'} Speed $speed");
}

sub Start($) {
    my ($self) = @_;

    SendFrunner($self, "$self->{'name'} Start");
}

sub State($) {
    my ($self) = @_;

    SendFrunner($self,"$self->{'name'} State");
    $self->{'returnValue'} = $main::returnValue;
    return($main::returnValue);
}

sub Stop($) {
    my ($self) = @_;

    SendFrunner($self, "$self->{'name'} Stop");
}

sub TTL {
    my ($self, $ttlSelection, $ordinal) = @_;

    if ( !defined($ordinal) ) {
	$ordinal = '';
    }
    SendFrunner($self, "$self->{'name'} TTL $ttlSelection $ordinal");
}

sub Waitstate {
    my ($self, $state, $seconds, $logInterval) = @_;

    if ( !defined($logInterval) ) {
	$logInterval = '';
    }
    SendFrunner($self, "$self->{'name'} Waitstate $state $seconds $logInterval");
}


1;
