#!perl -w

package Finisar::TC;

# Module to handle a Test Controller object
# This module contains all the classes for the object
# "test controller".

use Exporter;

use Finisar::Extern;
use Finisar::Utils;

use strict;
use warnings;

use vars qw(@ISA @EXPORT); 

@ISA = qw(Exporter);

sub New {
    my $self = {} ;
    $self->{'name'} = shift;
    my $host = shift;
    my $port = shift;
    if ( !defined($port) || $port eq "" ) {
	if ( $ENV{'FRUNNER_REQUIRE_PORT'} ) {
	    print("ERROR - You must explicitely give the port number to create an TC object\n");
	    exit(1);
	} else {
	    $port = 1001;
	}
    }

    $self->{'host'} = $host . ":" . $port;

    $self->{'configDir'}   = $configDir;
    $self->{'configFile'}  = undef;  
    $self->{'logDir'}      = $logDir;
    $self->{'logFile'}     = "tcscript.log"; 
    $self->{'password'}    = "";
    $self->{'resultDir'}   = $logDir;
    $self->{'resultFile'}  = "tcscript.tgp";
    $self->{'returnCode'}  = "";
    $self->{'returnValue'} = "";
    $self->{'running'}     = 0;  
    $self->{'startTime'}   = 30;
    $self->{'user'}        = "";
    
    bless $self;

    SendFrunner($self,"$self->{'name'} New TraceController");

    return($self);
}

sub Cards($$$) {
    my($self, $cardStart, $numberOfCards) = @_;

    SendFrunner($self, "$self->{'name'} Cards $cardStart $numberOfCards");
}

sub Chassis($$) {
    my($self, $chassisIP) = @_;

    SendFrunner($self, "$self->{'name'} Chassis $chassisIP");
}

sub Configdir($$) {
    my($self, $configDir) = @_;

    $configDir = QuoteFile($configDir);
    SendFrunner($self, "$self->{'name'} Configdir $configDir");
}

sub Exit($) {
    my($self) = @_;

    SendFrunner($self, "$self->{'name'} Exit");
}

sub GetReturnCode($) {
    my($self) = @_;

    return($self->{'returnCode'});
}

sub Load($$) {
    my($self, $configFile) = @_;

    $configFile = QuoteFile($configFile);
    SendFrunner($self, "$self->{'name'} Load $configFile");
}

sub Personality($$) {
    my($self, $personalityType) = @_;

    SendFrunner($self, "$self->{'name'} Personality $personalityType");
}

sub Port {
    my($self, $chassisIP, $ordinal, $blade, $firstPort, $nbPort, $mode, $pair) = @_;

    if ( !defined($mode) || $mode =~ /^\s*$/ ) { $mode = 0; }
    if ( !defined($pair) || $pair =~ /^\s*$/ ) { $pair = 1; }
    SendFrunner($self, "$self->{'name'} Port $chassisIP $ordinal $blade $firstPort $nbPort $mode $pair");
}

sub Resultdir($$) {
    my($self, $resultDir) = @_;

    $resultDir = QuoteFile($resultDir);
    SendFrunner($self, "$self->{'name'} Resultdir $resultDir");
}

sub Save($$) {
    my($self, $resultFile) = @_;

    SendFrunner($self, "$self->{'name'} Save $resultFile");
}

sub SetCaptureOption($$$) {
    my($self, $captureOptionName, $captureOptionValue) = @_;

    SendFrunner($self, "$self->{'name'} SetCaptureOption $captureOptionName $captureOptionValue");
}

sub Speed($$) {
    my ($self, $speed) = @_;

    SendFrunner($self, "$self->{'name'} Speed $speed");
}

sub Start($) {
    my($self) = @_;

    SendFrunner($self, "$self->{'name'} Start");
}

sub State($) {
    my($self) = @_;

    SendFrunner($self, "$self->{'name'} State");
    $self->{'returnValue'} = $main::returnValue;
    return $main::returnValue;
}

sub Stop($) {
    my($self) = @_;

    SendFrunner($self, "$self->{'name'} Stop");
}

sub TTL {
    my($self,$ttlSelection,$ordinal) = @_;

    if ( !defined($ordinal) ) {
	$ordinal = '';
    }
    SendFrunner($self, "$self->{'name'} TTL $ttlSelection $ordinal");
}

sub Waitstate {
    my($self, $state, $seconds, $logInterval) = @_;

    if ( !defined($logInterval) ) {
	$logInterval = '';
    }
    SendFrunner($self, "$self->{'name'} Waitstate $state $seconds $logInterval");
}

sub Waittrigger {
    my($self, $seconds, $logInterval) = @_;

    if ( !defined($logInterval) ) {
	$logInterval = '';
    }
    SendFrunner($self, "$self->{'name'} Waittrigger $seconds $logInterval");
}

   
1;
