#!perl -w

package Finisar::Utils;

use Exporter;
use IO::Socket;

use Finisar::Extern;

use strict;
use warnings;

use vars qw(@ISA @EXPORT %hosts);

@ISA = qw(Exporter);
@EXPORT = qw(CloseSockets PingHost QuoteFile SendFrunner);

# TODO
#      - Test ping on Solaris or other Unix

# Init Code
%hosts = ();        # Initialize the list of Hosts as empty
                    # Keep it in main to ensure it is not initialized
                    # many times and seen by all the objects

my $WaitOnSocket = 1; # In seconds

# Subroutines

sub CloseSockets() {

    my $oneSocket;
    foreach $oneSocket ( values(%hosts) ) {
        close($oneSocket);
    }
    return();
}


sub PingHost($) {
   my $hostname = shift;

   my $cmd = "ping -c 1 $hostname";  # Tested on Linux
   system($cmd);
   
}

sub QuoteFile($) {
    my($filename) = @_;

    if ( $filename =~ / / &&
         $filename !~ /^".*"$/ ) {

        $filename =~ s/(.*)/"$1"/;
    }
    return($filename);
}

sub SendFrunner($$) {
    my($obj, $cmd) = @_;

    my $socket;

    my $hostname = $obj->{'host'};

    if ( 1 || ! $hosts{$hostname} ) {
        # This host has never been contacted, we have to open the
        # connection first

        # $hostname contains host:port
        # due to a limitation of Trace Controller, we can not
        # create 2 objects on in the same process, so we use
        # 2 Frunner on the same machine with different ports
        my($host,$port) = split(/:/, $hostname);
        $socket = IO::Socket::INET->new(PeerAddr => $host,
                                        PeerPort => $port,
                                        Proto    => "tcp",
                                        Type     => SOCK_STREAM);

        # Should we try to start it remotely?
        # Only allow for the "Frunner New" command if
        # the user provide the location where FRunner is.
        if ( ! $socket && $ENV{'FRUNNER_LOCATION'} && $cmd =~ /Frunner New/i ) {

            StartRemoteFrunner($host,$port,$ENV{'FRUNNER_LOCATION'});

            # Give another try...
            $socket = IO::Socket::INET->new(PeerAddr => $host,
                                            PeerPort => $port,
                                            Proto    => "tcp",
                                            Type     => SOCK_STREAM);

        }

        if ( ! $socket ) {
            die "FATAL - Could not connect to FRunner on $hostname\n";
        }

        $hosts{$hostname} = $socket;
    } else {
        $socket = $hosts{$hostname};
    }

    # Now send the message
    print $socket ("$cmd");
    my $answer = <$socket>;

    # TODO - How can we avoid closing the socket?
    close($socket);   

    # Separate the LOG part from the returnCode/returnValue
    while ( $answer =~ s/<LOG>(.*?)<\\LOG>// ) {
        #print("LOG - $1\n");
        &main::tstlog('-comment', "$1");
    }

    $main::returnCode = "";
    $main::returnValue = "";
    chomp($answer);
    ($main::returnCode,$main::returnValue) = split(/::/,$answer);
    ($obj->{returnCode},$obj->{returnValue}) = split(/::/,$answer);

    if ( !defined($main::returnValue) ) {
	$main::returnValue = '';
    }
    if ( !defined($obj->{returnValue}) ) {
	$obj->{returnValue} = '';
    }

    if ($main::OPT{'debug'}) {
        print "RETURN CODE:$answer\n";
    }

    # TODO - How can we sleep for less if we can not keep the socket open?
    sleep($WaitOnSocket);

}


sub StartRemoteFrunner($$$) {
    my($host, $port, $location) = @_;

    my $cmd;
    my $cmdOut;
    my $ctime;
    my $frunnerCmd;
    my $hours;
    my $jobTime;
    my $minutes;
    my $minutesDelay = 1;  #TODO - should use 2 minutes
    my $now;

    $now = localtime();
    $now =~ /(\d+):(\d+)/;
    $hours = $1;

    # TODO - can not increment 60 !!!
    $minutes = $2 + $minutesDelay;
    $jobTime = "$hours:$minutes";

    # Create a task remotely

    print("Will take $minutesDelay minutes to try to start remotely Frunner\n");
    print(" on host: $host, port: $port, at: $jobTime\n");

    $frunnerCmd = "$location\\Frunner.exe -port $port -icon";

    $cmd = "at \\\\$host $jobTime /interactive $frunnerCmd ";
    $cmdOut = `$cmd 2>&1`;
    if ( $? ) {
        print $cmdOut;
    }

    sleep($minutesDelay * 60);

    # Verify if the task is still there...
    # If the clocks are not in sync, the task could be set up for tomorrow
    $cmd = "at \\\\$host";
    $cmdOut = `$cmd 2>&1`;

    # Look if there is a task left for Frunner on that port
    # With the clock of the server behind, the task would appear as tomorrow.
    $frunnerCmd =~ s/\\/\\\\/g;  # Double the slashes to used in regular expression.
    if ( $cmdOut =~ /\n\s*(\d+)[^\n]+$frunnerCmd/s ) {
        # The task was still there, remove it.
        print("WARNING - Your clocks between computer seems off, could not start FRunner\n");

        $cmd = "at \\\\$host $1 /delete";
        $cmdOut = `$cmd 2>&1`;
        if ( $? ) {
            print("ERROR - Unable to remove temporary task to start Frunner remotely\n");
        }
    }
    return();
}


sub WaitOnSocket($) {
    my($newWait) = @_;

    $WaitOnSocket = $newWait;

    return();
}


1;
