#
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/7G.pm#1 $
#
## @summary  GNOTE 7G module
## @author   dl-dpgqa-automation@netapp.com
## @status   Private
## @pod      here
#
package NOTE::7G;
use Carp qw( croak );

use GNOTE::7G::Volume;
use GNOTE::7G::Filer;
use GNOTE::7G::Aggregate;
use GNOTE::7G::Aggregate::Copy;
use GNOTE::7G::Volume::Copy;
use GNOTE::7G::Qtree;
use GNOTE::7G::SnapMirror;
use GNOTE::7G::SnapVault;
use GNOTE::7G::LockVault;
use GNOTE::7G::Fiji;
use GNOTE::7G::Registry;
use GNOTE::7G::Registry::Slice;
use filer;  # this is needed for filer_cli_say

use Aggregate qw();
our @ISA = qw(NOTE);

# Returns TRUE if an object *looks* anonymous according to
# its name. This is used by 'rename' methods to determine
# whether a renamed object should be spared or not.
sub _looksAnonymous {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $id;

    if ( $self->isa( "NOTE::Aggregate" ) ) {
	$id = "ag";
    } elsif ( $self->isa( "NOTE::Volume" ) ) {
	$id = "vl";
    } elsif ( $self->isa( "NOTE::Qtree" ) ) {
	$id = "qt";
    }
    return ( $self->name =~ /^(_|)$id(.....)_(....)_(........)/i );
}

# Returns TRUE if an object *looks* reusable according to
# its name.
sub looksReusable {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    return $self->_looksAnonymous && (($self->name =~ /^_/) || ($self->name =~ /^AG/));
}


# If string is passed, it is tested for whether it
# looks like something shareable. If string is NOT passed
# the name of the object is tested.
#
# The only currently shareable things are aggregates.
sub looks_shareable {
    my ($self,$string) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $name;

    if ( scalar @_ > 1 ) {
	$name = $string;
    } else { # Test the name of the object
	$name = $self->name;
    }
    return $name =~ /^AG/;
}

# If string is passed, it is tested for whether it
# looks like something this is free. If string is NOT passed
# the name of the object is tested.
#
# Free 7X aggregate and volume names start with an '_'.
sub looks_free {
    my ($self,$string) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $name;

    if ( scalar @_ > 1 ) {
	$name = $string;
    } else { # Test the name of the object
	$name = $self->name;
    }
    return $name =~ /^_(ag|vl)/;
}

# If string is passed, it is returned with an a prefix that makes
# it look like it's free.
# If string is NOT passed the name of the object is returned
# with the free prefix.
#
# Free 7X aggregate and volume names start with an '_'.
sub _make_free {
    my ($self,$string) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $name;

    if ( scalar @_ > 1 ) {
	$name = $string;
    } else { # Test the name of the object
	$name = $self->name;
    }
    return '_'.$name;
}

sub save_logs {
    my ( $filer ) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $root_vol  = $filer->vol_root;
    return unless( $root_vol );

    # wait for core files to be saved if in progress, so cores will be collected as well
    my $tried=0;
    while( $tried++ < 3 ) {
	# do not rely on NFS, NFS has cache and may have delays, we do not want to miss anythihng
	my $cores = filer_cli_say($filer->conn, "priv set -q advanced;ls /vol/$root_vol/etc/crash");
	if( $cores =~ /saving/ ) {
	    NOTE::logcomment( "NOTE: waiting for core to be saved:\n$cores");
	    $tried=0;
	    sleep 5;
	}
    }

    my ($presnap,$postsnap) = ( '_pretest_'.$filer->{_root_snapshots}, '_posttest_'.$filer->{_root_snapshots} );
    my $root = NOTE::Volume->attach( Filer => $filer, name => $root_vol );

    my $presnapflag = 0;    # we need to check if pretest snapshot exists
    my $snapretries = 5;    # and we need to retry, see BURT 231315
    do {
	sleep 10 if( $snapretries < 5 );
	eval{ $presnapflag = $root->snapshot_exists($presnap) };
    } while( $@ && $snapretries-- );

    return unless( $presnapflag );  # If we can not get hold of snapshots then there is nothing we can do

    my $LOGDIR = param('LOGDIR','-default',undef);
    my $saveto = $filer->host . "_filerlogs";			    # where WE actually want to save files
    $saveto = "$_SAVE_LOGS/$saveto" if( $_SAVE_LOGS =~ /[a-z]/i );  # use directory given by SAVE_LOGS
    $saveto = "$LOGDIR/$saveto" if( $LOGDIR =~ /\w/ );		    # but add/use LOGDIR also if exist

    # and finally, create posttest snapshot only after all cores are saved

    if( $root->snapshot_create($postsnap) ) {
	my $srcpath = "/vol/$root_vol/.snapshot/$presnap";
	my $dstpath = "/vol/$root_vol/.snapshot/$postsnap";

	my $out ='';
	eval { $out = filer_cli_say(    # list of changed files in '$out'
		$filer->conn,
		"priv set -q test; treecompare start -V -c -V $srcpath $dstpath",
		{ timeout => 7200 }
	    );
	};

	my %files; # use hash of names to prevent multiple entries due to multiple mismatch like mtime and data

	foreach( split('[\n\r]',$out )) {
	    s/(^\s+|\s+$)//g;
	    if( /(Found tree2 only directory entry|size mismatch|data mismatch)/ ) {	# pay attention to this miscompare only
		s/.*\s+//; s/^\.//; # clean up - leave filename only
		$files{$_} = '';    # and register name in our hash
	    }
	}

	if( keys %files ) {
	    my $mount = undef;
	    eval { $mount = $root->mount; };	# if mount will fail for some reason then we just exit
	    unless( $@ ) {
		`mkdir -p $saveto`;
		if( -d $saveto && $root->mounted ) {
		    NOTE::logcomment("Saving logs from filer " . $filer->host . " to $saveto directory");
		    foreach my $file (keys %files) {
			my $newname = $file;
			$newname =~ s/\//_/g;
			`/usr/local/bin/sudo cp $mount/$file $saveto/$newname` if( -f "$mount/$file" );
		    }
		    `/usr/local/bin/sudo chmod 666 $saveto/*`;
		} else {
		    logcomment("Unable to save log files for filer " . $filer->host . ". Unable to mount or $saveto is not created");
		}
	    }
	}

	unless( NOTE->ALL_SPARED ) {	# do not delete snapshots if we want to spare rest of stuff.
	    $root->snapshot_delete($presnap);
	    $root->snapshot_delete($postsnap);
	}
    } else {
	NOTE::logcomment("Unable to save filer logs for filer ".$filer->host.", unable to create post test snapshot $postsnap");
    }
}

