#
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/7G/Aggregate.pm#1 $
#
## @summary  GNOTE 7G Aggregate module
## @author   tmichael@netapp.com
## @status   Private
## @pod      here
package NOTE::7G::Aggregate;
use GNOTE::7G;
use GNOTE::Aggregate;
use Carp qw(croak);
our @ISA = qw(NOTE::Aggregate NOTE::7G);

=head2 ::create

Constructor for NOTE::7G::Aggregate object

=over

=item Arguments

=item Examples

=back

=cut

sub create {
    my ($class,%args) = @_;
    my $self = {};
    my $args = \%args;

    $self->{_debug} = $args{debug};
    $self->{_owner} = $$;
    $self->{_args} = $args;

    exists $args->{name}      and $args->{name}      = NOTE::option( $args->{name} );
    exists $args->{snaplock}  and $args->{snaplock}  = NOTE::option( $args->{snaplock} );
    exists $args->{raidtype}  and $args->{raidtype}  = NOTE::option( $args->{raidtype} );
    exists $args->{rgsize}    and $args->{rgsize}    = NOTE::option( $args->{rgsize} );
    exists $args->{disks}     and $args->{disks}     = NOTE::option( $args->{disks} );
    exists $args->{anonymous} and $args->{anonymous} = NOTE::option( $args->{anonymous} );
    exists $args->{dataset}   and $args->{dataset}   = NOTE::option( $args->{dataset} );
    exists $args->{recycle}   and $args->{recycle}   = NOTE::option( $args->{recycle} );

    # if the $args{Filer} is a filer object, use it,
    # otherwise try to create one by calling the NOTE::Filer constructor.
    if ( !defined($args{Filer}) ) {
	# In this case, there are no arguments (i.e. no hostname) for the
        # filer. I call the filer method anyway and let it decide to
        # handle it or croak.
        $self->{_Filer} = NOTE::Filer->connect();
    } elsif ( ref $args{Filer} eq "HASH" ) {
	$self->{_Filer} = NOTE::Filer->connect( %{$args{Filer}} );
    } elsif ( ref $args{Filer} eq "NOTE::Filer" ) {
	$self->{_Filer} = $args{Filer}; # This is already an object
    } elsif ( ref \$args{Filer} eq "SCALAR" ) {
	# Is just a hostspec
	$self->{_Filer} = NOTE::Filer->connect( host => $args{Filer} );
    } else {
	croak "Invalid filer specification: ".ref \$args{Filer}."\n";
    }
    delete $args{Filer};

    if ( my $snaptype = $args{snaplock} ) {
	# Make sure proper licences are installed.
	# Fix this up the way Aggregate::aggr_create wants them:
	delete $args{snaplock};
	if ( $snaptype eq 'compliance' ) {
	    $args{snaplock} = '1';
	    $self->{_Filer}->check_license( 'snaplock' );
	} elsif ( $snaptype eq 'enterprise' ) {
	    $self->{_Filer}->check_license( 'snaplock_enterprise' );
	    $args{snaplock_enterprise} = '1';
	}
    }

    $self->{_destroyed} = 1; # Do this because the aggr doesn't physically exist yet.
    bless $self, $class;
    $self->SUPER::create( %args );
    $self->{_destroyed} = 0; # Now it exists

    if ( $self->is_anonymous && !$self->is_shareable ) {
	eval{ $self->snapshot_create( "__empty__" ) };
	if ( $@ ) {
	    croak $@ unless ( $@ =~ /Snapshot already exists/ );
	}
    }

    if ( $self->{_dataset} ) {
	NOTE::logcomment( "Initializing dataset of ".$self->description );
	my $func = eval 'sub {'.$self->{_dataset}.'}';
	croak $@ if $@;
	eval { $self->$func };
	croak $@ if $@;
	if ( $self->_looksAnonymous ) {
	    $self->_rename( NOTE::datasetEncode( $self->{_name}, $self->{_dataset} ) );
	}
	# We need to turn this off, otherwise the __clean__ snapshot
	# might be auto-deleted.
	$self->options( "snapshot_autodelete", "off" );
    }
    # Only save the state of private aggregates
    if ( $self->is_anonymous && !$self->is_shareable ) {
	eval{ $self->snapshot_delete( "__clean__" ) }; # Don't care if this fails
	$self->snapshot_create( "__clean__" );
    }
    return $self;
}

sub attach {
    my ($class,%args) = @_;
    my $self = {};
    my $args = \%args;

    $self->{_debug} = $args->{debug};
    $self->{_owner} = $$;
    $self->{_args}  = $args;

    exists $args->{name} and $args->{name} = NOTE::option( $args->{name} );
    exists $args->{type} and $args->{type} = NOTE::option( $args->{type} );

    # if the $args->{Filer} is a filer object, use it,
    # otherwise try to create one by calling the NOTE::Filer constructor.
    if ( !defined($args->{Filer}) ) {
	# In this case, there are no arguments (i.e. no hostname) for the
        # filer. I call the filer method anyway and let it decide to
        # handle it or croak.
        $self->{_Filer} = NOTE::Filer->connect();
    } elsif ( ref $args->{Filer} eq "HASH" ) {
	$self->{_Filer} = NOTE::Filer->connect( %{$args->{Filer}} );
    } elsif ( ref $args->{Filer} eq "NOTE::Filer" ) {
	$self->{_Filer} = $args->{Filer}; # This is already an object
    } elsif ( ref \$args->{Filer} eq "SCALAR" ) {
	# Is just a hostspec
	$self->{_Filer} = NOTE::Filer->connect( host => $args->{Filer} );
    } else {
	croak "Invalid filer specification: ".ref \$args->{Filer}."\n";
    }
    delete $args->{Filer};

    bless $self, $class;
    return $self->SUPER::attach( %args );
}


=head2 ::options

Sets or gets the options of an aggregate. Wrapper around
L<C<Aggregate::aggr_options>|engineering::ontap_tests::agnostic::docs::lib-Aggregate-pm/aggr_options($conn,$aggr[, %opts])>.

=over

=item Examples

Reading example:

    my $time = $aggr->options('resyncsnaptime');

Setting example:

    $aggr->options(resyncsnaptime => 100);

The options "root, ignore_inconsistent" can be changed in 
maintainance mode only.

=back

=cut

sub options {
    my ($self,$which_one,$value) = @_;
    my ($options,$ret);
    if ( @_ == 3 ) {
	$ret = Aggregate::aggr_options (
	    $self->Filer->conn,
	    $self->{_name},
	    $which_one => $value,
	);
	croak $@ if $@;
	return $ret;
    } else {
	$options = Aggregate::aggr_options (
	    $self->Filer->conn,
	    $self->{_name},
	);
	croak $@ if $@;
	if ( $which_one ) {
	    return $options->{ $which_one };
	} else {
	    return $options;
	}
    }
}


=head2 ::status

Returns the state, status, host aggregate name, and a hash of aggregate options
of the given aggregate. Wrapper around 
L<C<Aggregate::aggr_status>|engineering::ontap_tests::agnostic::docs::lib-Aggregate-pm/aggr_status($conn,$aggr[,$timeout])>.

=over

=item Examples

  my $status_obj = $aggr->status;

  my $type      = $status_obj->type();
  my $state     = $status_obj->state();
  my $status    = $status_obj->status();
  my %options   = $status_obj->options();
  my $vols_list = $status_obj->volumes(); # returns reference to an array

=back

=cut

sub status {
    my $self = shift;
    
    if ( wantarray ) {
        my ($state, $status, $aggr, $options) = Aggregate::aggr_status(
	    $self->{_Filer}->conn, $self->{_name},
        );
	return ( $state, $status, $aggr, $options );
    } else {
        return Aggregate::aggr_status(
	    $self->{_Filer}->conn, $self->{_name},
        );
    }
}

=head2 ::hostpath

Returns the name of the aggregate
prepended by its filer's name.

=over

=item Arguments

None.

=item Example

=back

     $hp = $aggr->hostpath;

=cut

sub hostpath {
    my $self = shift;
    return $self->{_Filer}->host.":".$self->{_name};
}

=head2 ::fsid

Returns the FSID (in hexadecimal) of an aggregate or rewrites the FSID of the desired aggregate.
When rewrites FSID it returns the *original* FSID of the aggregate prior to the FSID rewrite.

Do not force a rewrite of FSID of an aggregate if you have no idea what
you're doing. Note: You need to restrict the volume(s) first prior to calling this API.

When setting new fsid, additional parameters are passed to
L<C<Aggregate::aggr_rewrite_fsid>|engineering::ontap_tests::agnostic::docs::lib-Aggregate-pm/aggr_rewrite_fsid($conn,$aggr,[%opts])>.

=over

=item Arguments

B<fsid> = id

I<Otional.> Sets FSID to one provided

B<force> = 1|0

I<Otional.> Force rewriting of the FSID even though an existing of the
same FSID exists on the system.

=item Examples

B<Read FSID>

    $fsid = $aggr->fsid;

B<Set FSID>

    $aggr->fsid( $newfsid );

    $aggr->fsid( $newfsid, force => 1 );

=back

=cut

sub fsid {
    my ($self,$newfsid,%args) = @_;
    my $oldfsid;
    $oldfsid = Aggregate::aggr_read_fsid(
        $self->{_Filer}->conn,
	$self->{_name}
    );
    croak $@ if $@;
    if ( $newfsid ) {
	Aggregate::aggr_rewrite_fsid(
	    $self->{_Filer}->conn,
	    $self->{_name},
	    fsid => $newfsid,
	    %args
	);
    }
    croak $@ if $@;
    return $oldfsid;
}

=head2 ::split

Split a aggregate. After the split of the syncmirrored aggregate, a
NOTE::Aggregate object would be returned to caller for the split-off
aggregate.

=over

=item Syntax

$aggr->split( $plex, $newaggr, %args );

=item Argments

B<plex> = name 

I<Optional> Plex to use. If no plex given then automatically grab the last plex as the one to be split off

B<newaggr> = name

I<Optional> New name for splitted half

B<debug> = 1|0

I<Optional> Print extra debugging info

B<recycle> = 1|0

I<Optional> Set or clear recycle flag

B<anonymous> = 1|0

I<Optional> Generate anonymous name for new aggregate. This is default behavior

=item Example

my $aggr_b = $aggr_a->split();

=back

=cut

sub split {
    my ($parent,$plex,$newaggr,%args) = @_;
    my $self = {};

    $parent->Filer->check_license( "syncmirror_local" );
    $self->{_debug}    = $args{debug};
    $self->{_owner}    = $$;
    $self->{_dataset}  = $parent->{_dataset};
    $self->{_tag}      = "SyncMirrored of ".$parent->description;
    $self->{_recycle}  = $args{recycle};
    delete $args{recycle};

    NOTE::logcomment( "Splitting ".$parent->description );
    if ( $newaggr ) {
	$self->{_name} = $newaggr;
	$self->{_anonymous} = $args{anonymous};
	delete $args{newaggr};
	delete $args{anonymous};
    } else {
	# Create a unique name
	$self->{_name} = NOTE::datasetEncode(
	    NOTE::Aggregate->newName,
	    $self->{_dataset}
	);
	$self->{_anonymous} = 1;
    }

    $self->{_Filer} = $parent->Filer;

    # automatically grab the last plex as the one to be split off
    if ( !$plex ) {
	my $status = Aggregate::aggr_raidgroup_status( 
            $parent->Filer->conn,
	    $parent->name 
        ) or croak $@;

	foreach my $plex_info ( @{$status->{plex}} ) {
	    $plex = $plex_info->{name};
	}
    }

    Aggregate::aggr_split( $parent->Filer->conn, 
			   $plex,
			   $self->{_name} );
    croak $@ if $@;

    bless $self, ref($parent);
    NOTE::reference( $self );
    $self->resource( 'allocate' );

    return $self;
}

=head2 ::restricted

Returns true if aggregate is restricted

=cut

sub restricted {
    my $self = shift;
    my ($state, $status, $aggr, $options) = Aggregate::aggr_status(
        $self->Filer->conn, $self->name
    );
    croak $@ if $@;
    return $state =~ /restricted/;
}


=head2 ::volumes

Returns array of volume names found in aggregate. Array is simply array of strings.

=cut

sub volumes { return @{shift->status->volumes} }

=head2 ::Volumes

Returns array of
L<C<NOTE::Volume>|engineering::ontap_tests::agnostic::docs::lib-NOTE-Volume-pm>
objects found in aggrgate.

=over

=item Arguments

B<spare_only> = 0|1

I<Optional> If set to 1, then it will return list of
L<C<NOTE::Volume>|engineering::ontap_tests::agnostic::docs::lib-NOTE-Volume-pm>
objects of recycled type which have not yet been reused. By default all volumes are retured.

=back

=cut

sub Volumes {
    my ($self,%args) = @_;
    my @objList = ();
    for my $vol ( $self->volumes ) {
	next if ( $args{'spare_only'} && $vol !~ /^_/ );
	push @objList, NOTE::Volume->attach(
	    Filer => $self->Filer,
	    name => $vol,
	);
    }
    return @objList;
}

=head2 ::copy

Execute aggr copy command from source to self. Brings aggregate online
after aggr copy command is completed. If aggr online failed then
command is retried up to 5 times before failing

=over

=item Arguments

B<src> aggregate object

I<Required> Source aggregate object. Object is the NOTE's object.

B<background> = 1|0

I<Optional> Background or foreground execution flag

=item Examples

    $dst_aggr->copy( src => $src_aggr );

=back

=cut

sub copy {
    my ($self,%args) = @_;
    my $background = $args{background};
    delete $args{background};
    my $src = $args{src};
    delete $args{src};
    my %ARGS = ();
    my $was_restricted = $self->restricted;

    croak "src must be specified for copy" unless ( $src );
    
    NOTE::logcomment( "Copying ".$src->description." -> ".$self->description );
    $ARGS{uc $_}    = $args{$_} for ( keys %args );
    $ARGS{COMPLETE} = $background ? 0 : 1;
    $ARGS{SOURCE}   = $src->name;

    $self->restrict unless ( $was_restricted );
    Aggregate::Copy::aggrcopy_start(
	$self->Filer->conn,
	$self->name,
	%ARGS,
    );
    croak $@ if $@;

    unless ( $was_restricted ) {
	my $try = 0;
	for $try ( 1..5 ) {
	    eval { $self->online };
	    last unless ( $@ );
	    sleep 1;
	}
	if ( $try >= 5 ) {
	    croak "Couldn't bring ".$self->name." online after aggrcopy: $@";
	}
    }
    return $self;
}


=head2 snapshot_create

Create a snapshot on the aggregate

=over

=item Argument

B<snapname> - I<name>

Optional

If not set then a anonymous name is used to create the snapshot with

=item Examples

=back

     $aggr->snapshot_create;

     $aggr->snapshot("special");

=cut

sub snapshot_create {
    my ($self,$snapname) = @_;

    $snapname = $self->SUPER::newName( "sn" ) unless( $snapname );

    NOTE::logcomment( "Creating snapshot ".$snapname." of ".$self->description );
    SnapCmds::snapshot_create(
        $self->Filer->conn,
        "-A ".$self->name,
	$snapname,
    );
    croak $@ if ($@);
    return $snapname;
}

=head2 ::snapshot_restore

Restored a named snapshot

=over

=item Arguments

B<snapname> - I<name>

Name of the snapshot to restore

=item Example

=back

     $aggr->snapshot_restore("special");

=cut

sub snapshot_restore {
    my ($self,$snapname,%opts) = @_;

    croak "Need snapshot name" unless ( $snapname );
    NOTE::logcomment( "Restoring ".$self->description." using snapshot ".$snapname );

    $self->Filer->check_license( "snaprestore" );
    SnapCmds::snaprestore_volume(
        $self->Filer->conn,
        "-A ".$self->name,
	$snapname,
	%opts,
    );
    croak $@ if ($@);
    return $self;
}

=head2 ::snapshot_list

Returns an array of the snapshots that exists on this aggregate

=over

=item Arguments

None.

=item Example

=back

     $aggr->snapshot_list;

=cut

sub snapshot_list {
    my $self = shift;

    my @list = SnapCmds::snapshot_list(
        $self->Filer->conn,
        "-A ".$self->name
    );
    croak $@ if ($@);
    return @list;
}

=head2 ::snapshot_status

Returns a reference to a hash of the status with the
snapshot names as keys.

=over

=item Arguments

None

=item Example

=back

     $aggr->snapshot_status;

=cut

sub snapshot_status {
    my $self = shift;
    my %status = ();
    my @status;

    @status = SnapCmds::snapshot_status(
	$self->Filer->conn,
	"-A ".$self->name
    );
    croak $@ if ($@);

    for ( @status ) {
	$status{$_->{name}} = $_;
    }
    return \%status;
}

=head2 ::snapshot_exists

Returns a boolean to indicate whether a given named snapshot exists or not

=over

=item Arguments

B<snapshot> - I<name>

Name of the snapshot to check

=item Example

=back

     $aggr->snapshot_exists('special');

=cut

sub snapshot_exists {
    my ($self,$snapshot) = @_;
    my $ret;

    $ret = SnapCmds::snapshot_exists(
	$self->Filer->conn,
	"-A ".$self->name,
	$snapshot
    );
    croak $@ if ($@);
    return $ret;
}

=head2 ::snapshot_delete

Delete a named snapshot

=over

=item Arguments

B<snapshot> - I<name>

Name of the snapshot to delete

=item Example

=back

      $vol->snapshot_delete('special');

=cut

sub snapshot_delete {
    my ($self,$snapshot,%args) = @_;

    NOTE::logcomment( "Deleting snapshot ".$snapshot );

    SnapCmds::snapshot_delete(
	$self->Filer->conn,
	"-A ".$self->name,
	$snapshot,
	# %args
    );
    croak $@ if ($@);
    return $self;
}

=head2 ::snaplock

Returns snaplock type if snaplock is licensed and enabled for given aggregate. Snaplock
returned types are compliance or enterprise. Returns zero if no snaplock present.

=over

=item Example

      my $snaplock_type = $aggr->snaplock;

=back

=cut

sub snaplock {
    my $self = shift;
    my $opts = $self->options;
    return 'compliance' if ( $opts->{'snaplock_compliance'} );
    return 'enterprise' if ( $opts->{'snaplock_enterprise'} );
    return 0;
}

=head2 ::writeable

Returns 1 if aggregate is writable. Aggregate is 
writable if state is 'online' and it's not 'read-only'
and it's not 'restricted'

=cut


sub writeable {
    my $self = shift;

    my $state = $self->status->{'state'};

    return (
    	$state =~ /online/ &&
    	$state !~ /read-only/ &&
    	$state !~ /restricted/
    );
}

=head2 ::blocks

Returns total nummber of blocks in aggregate. Output is taken from 'aggr status -b' command

=cut

sub blocks {
    my $self = shift;
    my $blocks;
    eval {
	$blocks = Aggregate::aggr_blocks( $self->{_Filer}->conn, 
					  $self->{_name} );
    };
    croak $@ if $@;
    return $blocks->{blocks};
}


=head2 ::block_size

Returns number of bytes per block. Output is taken from 'aggr status -b' command

=cut

sub block_size {
    my $self = shift;
    my $blocks;
    eval {
	$blocks = Aggregate::aggr_blocks( $self->{_Filer}->conn, 
					  $self->{_name} );
    };
    croak $@ if $@;
    return $blocks->{block_size};
}

=head2 ::fs_size

Returns aggregate size in blocks. Output is taken from 'aggr status -b' command

=cut

sub fs_size {
    my $self = shift;
    my $blocks;
    eval {
	$blocks = Aggregate::aggr_blocks( $self->{_Filer}->conn, 
					  $self->{_name} );
    };
    croak $@ if $@;
    return $blocks->{fs_size};
}

=head2 ::disk_free

Returns values of 'total', 'used', 'avail', 'capacity' as hash for self or given path.
Output is taken from 'df -A -k' command.

=over

=item Syntax

$aggr->disk_free( path=>'/vol/vol0/etc' );

=item Arguments

B<path> string

I<Optional> Path for 'df -A -k' command. Default is plain aggregate name

=back

=cut

sub disk_free {
    my ($self, %args) = @_;

    if ( defined $args{path} ) {
        croak "\"path\" may not be a key in the options hash";
    }
    $args{path} = $self->name;
    my $out = Aggregate::aggr_disk_free(
        $self->Filer->conn,
        %args
    );
    croak $@ if $@;
    return wantarray ? ($out->{total}, $out->{used}, $out->{avail}, $out->{capacity}) : $out;
}

=head2 ::purge

Delete all volumes within aggregate and then destroy aggregate itself. Uses
L<C<aggr_purge>|engineering::ontap_tests::agnostic::docs::lib-Aggregate-pm/aggr_purge($conn,$aggr,[%opts])>
function from lib/Aggregate.pm

=over

=item Arguments

B<vol_only> = 1|0

I<Optional> Delete all volumes within aggregate but do not destroy aggregate itself. Default is 0.

=item Examples

    $aggr->purge( vol_only => 1 );

=back

=cut

sub purge {
    my ($self,%args) = @_;
    if ( $args{vol_only} == 1 ) {
	NOTE::logcomment( "Deleting contents of aggregate ".$self->description );
    } else {
	NOTE::logcomment( "Purging Aggregate ".$self->description );
    }
    Aggregate::aggr_purge( $self->{_Filer}->conn, $self->{_name}, %args );
    croak $@ if $@;
    # if vol_only => 1 was specified, only the volumes will be destroyed
    # NOT the aggregate
    unless ( $self->exists ) {
	$self->{_destroyed} = 1;
	$self->resource( 'destroy' );
	$self->{_Filer}->zero_spares; # Zero spares on teardown
    }
    return $self;
}


=head2 ::destroy

Destroy aggregate. This method uses 
L<C<aggr_destroy>|engineering::ontap_tests::agnostic::docs::lib-Aggregate-pm/aggr_destroy($conn,$aggr)>
function from lib/Aggregate.pm

=cut

sub destroy {
    my $self = shift;
    NOTE::logcomment( "Destroying Aggregate ".$self->description );
    Aggregate::aggr_destroy( $self->{_Filer}->conn, $self->{_name} );
    croak $@ if $@;
    $self->{_destroyed} = 1;
    $self->resource( 'destroy' );
    $self->{_Filer}->zero_spares; # Zero spares on teardown
    return $self;
}

=head2 ::wafliron_start

Start WAFL iron on aggregate. By default it runs in the background. Set I<wait> arguement
to 1 if foreground execution is required. If executed as a background process then 
the subtest object is saved in $self->{_wafliron_wait} private variable.

This method will croak if the aggregate is shared, because snapmirror activity by other
scripts will cause this to fail. Wafliron can only reliably be run in private anonymous
aggregates.

=over

=item Arguments

B<wait> = 1|0

I<Optional> Wait for wafliron to finish and exit. By default it start wafliron as background process.

=item Examples

    $aggr->wafliron_start( wait => 1 );

=back

=cut

sub wafliron_start {
    my ($self,%args) = @_;

    # Don't allow wafliron to run on a shared aggregate. This is
    # unreliable.
    croak "Cannot start wafliron on a shared aggregate ".$self->description
	if ( $self->is_shareable );
    NOTE::logcomment( "Starting Wafliron on ".$self->description );
    NOTE::Volume->attach( Filer => $self->Filer, name => $self->Filer->vol_root );
    my @out = Wafliron::wafliron_start( $self->Filer->conn, 
					$self->name, 
					%args );

    # user request not to wait for wafliron to complete
    # keep the $id returned to be used for wafliron_wait() method
    if ( !$args{wait} && ref( $out[0] ) ne 'SCALAR' ) {
	$self->{_wafliron_wait} = $out[0];
	# Cause it to be stopped on teardown, if still running
	NOTE::reference( sub { $self->wafliron_stop if ( $self->{_wafliron_wait} ) } );
    }
    return @out;
}


=head2 ::wafliron_stop

Termintate WAFL iron process.

=cut

sub wafliron_stop {
    my ($self,%args) = @_;

    my $out;
    delete $args{id}; # use $self->{_wafliron_wait} instead

    NOTE::logcomment( "Stopping Wafliron on ".$self->description );
    if ( exists $self->{_wafliron_wait} ) {
	$out = Wafliron::wafliron_stop( $self->Filer->conn, 
					$self->name, 
				        id => $self->{_wafliron_wait},
				        %args );
	delete $self->{_wafliron_wait};

    } else {
	$out = Wafliron::wafliron_stop( $self->Filer->conn, 
					$self->name, 
				        %args );
    }    
    return $out;
}


=head2 ::wafliron_status

Return WAFL iron status string

=cut

sub wafliron_status {
    my $self = shift;
    return Wafliron::wafliron_status( $self->Filer->conn, $self->name );
}

=head2 ::wafliron_wait

Wait for WAFL iron to finish or exit with error "I<Wafliron doesn't seem to be running on this aggregate>"

=cut

sub wafliron_wait {
    my $self = shift;

    if ( exists $self->{_wafliron_wait} ) {
	NOTE::logcomment( "Waiting for Wafliron on ".$self->description );
	my @out = Wafliron::wafliron_wait( $self->{_wafliron_wait} );
	delete $self->{_wafliron_wait};
	return @out;
    } else {
	croak "Wafliron doesn't seem to be running on this aggregate";
    }
}

=head2 ::preserve

Create a snapshot that represents the aggregate in its current
state.  See also L<C<::restore>|/::restore>

=over

=item Arguments

None.

=item Example

=back

     $aggr->preserve;

=cut

sub preserve {
    my $self = shift;

    if ( $self->snapshot_exists( "__preserve__" ) ) {
	$self->snapshot_delete( "__preserve__" );
    }
    $self->snapshot_create( "__preserve__" );
    return $self;
}

=head2 ::restore

Restore aggregate to the state preserved with L<C<::preserve>|/::preserve>.
Croaks if an error occurred during the restore.
Returns $self. See also L<C<::preserve>|/::preserve>

=over

=item Arguments

None.

=item Example

=back

     $aggr->restore;

=cut

sub restore {
    my $self = shift;

    $self->snapshot_restore( "__preserve__" );
    return $self;
}

=head2 ::empty

If an __empty__ snapshot exists, the aggregate will be restored
using this snapshot. 

Croaks on error.
Returns $self.

=over

=item Examples

=back

     $aggr->empty;

=cut

sub empty {
    my ($self,%args) = @_;

    unless ( $self->is_anonymous && !$self->is_shareable ) {
	croak "Cannot empty a non-anonymous or shareable aggregate: ".$self->description;
    }
    if ( $self->snapshot_exists( "__empty__" ) ) {
	$self->snapshot_restore( "__empty__" );
	if ( $self->{_dataset} ) {
	    delete $self->{_dataset};
	    $self->rename if ( $self->_looksAnonymous );
	}
	eval{ $self->snapshot_delete( "__clean__" ) }; # Don't care if this fails
	$self->snapshot_create( "__clean__" );
    } else {
	croak "Could not restore aggregate: __empty__ snapshot missing: ".$self->description;
    }
    return $self;
}

=head2 ::clean

If a __clean__ snapshot exists, the aggregate will be restored
using this snapshot. 

Croaks on error.
Returns $self.

=over

=item Examples

=back

     $aggr->clean;

=cut

sub clean {
    my ($self,%args) = @_;

    unless ( $self->is_anonymous && !$self->is_shareable ) {
	croak "Cannot clean a non-anonymous or shareable aggregate: ".$self->description;
    }
    if ( $self->snapshot_exists( "__clean__" ) ) {
	$self->snapshot_restore( "__clean__" );
    } else {
	croak "Could not restore aggregate: __clean__ snapshot missing: ".$self->description;
    }
    return $self;
}

# Below are the interfaces to the core libraries. There are parallel methods
# on the GX side that call the Server.pm library. These are meant to be
# called from the common Aggregate code in NOTE::Aggregate.
#
sub _aggr_create {
    my ($self, $diskspec, %args) = @_;
    return Aggregate::aggr_create( $self->Filer->conn, $self->name, $diskspec, %args );
}

sub _aggr_rename {
    my ($self,$name) = @_;
    Aggregate::aggr_rename( $self->Filer->conn, $self->name, $name );
    croak $@ if $@;
    $self->{_name} = $name;
}

sub _aggr_offline {
    my $self = shift;
    return Aggregate::aggr_offline( $self->{_Filer}->conn, $self->{_name}, @_ );
}

sub _aggr_online {
    my $self = shift;
    return Aggregate::aggr_online( $self->{_Filer}->conn, $self->{_name}, @_ );
}

sub _aggr_restrict {
    my $self = shift;
    return Aggregate::aggr_restrict( $self->{_Filer}->conn, $self->{_name}, @_ );
}

sub _aggr_add {
    my ($self,$diskspec,%args) = @_;

    return Aggregate::aggr_add(
	$self->{_Filer}->conn,
	$self->{_name},
	$diskspec,
	%args
    );
}

sub _aggr_exists {
    my ($self,$name) = @_;
    my $ret;
    my $name ||= $self->{_name};
    $ret = Aggregate::aggr_exists( $self->{_Filer}->conn, $name );
    if ( $@ ) {
	# aggr_exists sets $@ if the volume doesn't exist
	# Croak only if that's not the reason.
	croak $@ unless ( $@ =~ /No .* named '$name' exists/ );
    }
    return $ret;
}

sub _aggr_disks {
    my $self = shift;
    if ( wantarray ) {
	my @disks = Aggregate::aggr_disks( $self->{_Filer}->conn, $self->{_name} );
	croak $@ if $@;
	if ( scalar @disks == 0 ) {
	    croak "Aggregate ".$self->description." reports ZERO disks";
	}
	$self->{_disks} = scalar @disks;
	return @disks;
    } else {
	unless ( $self->{_destroyed} ) {
	    # Unfortunately, buggy formating code in ONTAP sometimes generates
	    # unparsable output for the aggr status <name> -d command, so if I just
	    # want a count of disks, this is a much more reliable method:
	    $self->{_disks} = scalar grep /^\s*[v]?\d/, split( /\n/,
		filer::filer_cli_say( $self->Filer->conn, "aggr status ".$self->name." -d" )
	    );
	}
	return $self->{_disks}; # return last known number of disks
    }
}


=head2 verify_aggregate_attributes

Verify that the aggregate's attributes match the caller's request. Returns a
value of 0 (zero) if the attributes match, or a non-zero error code. The error
codes are:

    1 - aggregate is offline
    2 - fake_large is not correct
    3 - local_storage is not correct
    4 - the aggregate is on the wrong nodes
    5 - the aggregate uses fewer disks than requested
    6 - dataset is not correct
    7 - snaplock is not correct
    8 - one of the other attributes is not as requested

=over

=item Arguments

=over

=item B<fake_large> = boolean

Aggregate should be a fake large aggregate

=item B<local_storage> = boolean

Aggregate should be on local storage

=item B<nodes> = string

Sorted, comma separated list of nodes the aggregate should be on

=item B<disk> = integer

The number of disks required for the aggregate

=item B<snaplock> = boolean

Aggregate should be a snaplock aggregate

=item B<name> = string

Should match the aggregate name

=item B<indirect_type> = string

Aggregate block-type, one of C<32-bit> or C<64-bit>

=item B<raidtype> = string

Aggregate raid type, one of C<raid_dp> or C<raid4>

=item B<rgsize> = integer

Aggregate raid group size

=item B<type> = string

Aggregate type, one of C<flex> or C<striped>

=back

=item Examples

Check that C<$aggr> is a 64-bit aggregate of 3 disks on local storage:

    my $code = $aggr->verify_aggregate_attributes(
        local_storage => 1,
        disk          => 3,
        indirect_type => '64-bit'
    );
    if ($code) {
        # Remember, any value of $code > 0 is an error!
        ...
    }

=back

=cut

###############################################################################
#
#   Sub Name: verify_aggregate_attributes
#
#   Description:
#       Called with $server->{_args} and diskspec created by
#       GNOTE::Aggregate::create and GNOTE::GX::Aggregate:create to verify
#       the aggregate attributes match the callers request.
#
#       NOTE: this method needs to support both NOTE and GNOTE argument
#       values (the compression group using scripts that run in both).
#       so far this has only been a problem for indirect_type but others
#       may eventually show up.
#
#       Finally, this method exists in both 7G and GX. The GX version is
#       fully functional. The 7G version is a stub. The method used to
#       reside in GNOTE::Aggregate.pm but the differences between the 7G
#       and GX versions of the status() method could not be rectified.
#       More information about the problem can be found in burt 492987.
#
#   Arguments:
#       This is the current list of arguments. Any additional arguments
#       are noted in the log and ignored. If other arguments are needed
#       a burt should be opened to have them added.
#
#               NAME        IN/OUT  TYPE    DESCRIPTION
#               fake_large  in      boolean should be a fake large aggregate
#               local_storage
#                           in      boolean should be on local storage
#               nodes       in      scalar  sorted comma separated list
#               disk        in      scalar  the number of disks required for
#                                           this aggregate
#               snaplock    in      boolean should be a snaplock aggregate
#               name        in      scalar  aggregate name
#               indirect_type
#                           in      scalar  block-type, '32-bit'|'64-bit'
#               raidtype    in      scalar  raid type, 'raid_dp'|'raid4'
#               rgsize      in      scalar  raid group size, integer
#               type        in      scalar  aggregate type, 'flex'|'striped'
#
#   Globals:
#
#   Environment:
#
#   Returns:    Success:    0 - aggregate matches requirements
#               Failure:    1 - aggregate is offline
#                           2 - fake_large is not correct
#                           3 - local_storage is not correct
#                           4 - the aggregate is on the wrong nodes
#                           5 - the aggregate uses fewer disks than requested
#                           6 - dataset is not correct
#                           7 - snaplock is not correct
#                           8 - one of the other attributes is not as requested
#                           9 - indirect_type/block-type do not match
#
###############################################################################
sub verify_aggregate_attributes {
    my ($self, %args) = @_;

    # this method is to solve burt 492987, more details can be found there
    return 0;
}


=head1 SEE ALSO

L<C<NOTE>|engineering::ontap_tests::agnostic::docs::lib-NOTE-pm/>

L<C<NOTE::Filer>|engineering::ontap_tests::agnostic::docs::lib-NOTE-Filer-pm/>

L<C<Aggregate>|engineering::ontap_tests::agnostic::docs::lib-Aggregate-pm>

L<C<Aggregate::Copy>|engineering::ontap_tests::agnostic::docs::lib-Aggregate-Copy-pm/>

L<C<Wafliron>|engineering::ontap_tests::agnostic::docs::lib-Wafliron-pm/>

L<C<disk>|engineering::ontap_tests::agnostic::docs::lib-disk-pm/>

=head1 STATUS

Public.

$Date: 2013/01/22 $

=cut


1;
