#
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/7G/Registry.pm#1 $
#
## @summary NOTE 7G Registry module
##
## @description
##    An internal module implementating NOTE::Registry object using Perl's tie hash
##    feature. Module is meant for internal use within NOTE modules, and are not to
##    be called directly
##
## @status  Private
##
## @author  dl-dpgqa-automation@netapp.com
##

# Implements the filer registry as a tied hash
#
# tie %reg, 'NOTE::Registry', $filer;
# $reg{'xyx.abc'} = 'off';
# delete $reg{'xyz.abc'};
# delete $reg{'xyz.*'};  # Does a deleteTree
#
# Note: the NOTE::Filer class stores one of these in the
# filer object, and is accessable via the NOTE::Filer->registry method.
# (e.g.  $reg = $filer->registry; print $reg->{'file.contents.dumpdates'}; )
#
package NOTE::Registry;
use strict;
use Carp qw(croak);
use GNOTE::7G;
our @ISA = qw(NOTE::7G);

# NOTE: Do *NOT* import any external functions into this module!
# This is to avoid namespace collisions namely prototype mismatches and
# all that. Call the external function directly by prepending the package 
# name.
use RegistryCmds qw();

sub TIEHASH {
    my ($pkg, $filer) = @_;
    my $self = {};
    # if $filer is a filer object, use it,
    # otherwise try to create one by calling the NOTE::Filer constructor.
    croak "Filer not specified" unless ( defined($filer) );
    if ( ref $filer eq "HASH" ) {
	$self->{_Filer} = NOTE::Filer->connect( %{$filer} );
    } else {
	$self->{_Filer} = $filer; # This must be an object
    }
    $self->{_registry} = {};
    return (bless $self, $pkg);
}

sub CLEAR {
    my ($self) = @_;
}

sub FIRSTKEY {
    my $self = shift;
    eval {
        $self->{_registry} = RegistryCmds::registryWalk(
	   $self->{_Filer}->conn,
	   ""
        );
    };
    croak $@ if $@;
    my $a = keys %{$self->{_registry}};          # reset each() iterator
    each %{$self->{_registry}};
}

sub NEXTKEY  {
    my $self = shift;
    return each %{$self->{_registry}};
}

sub EXISTS {
    my ($self,$key) = @_;
    return defined RegistryCmds::registryGet($self->{_Filer}->conn, $key);
}

sub FETCH {
    my ($self, $key) = @_;
    if ( $key =~ /(.*)\.\*$/ ) {
	# If the key looks like a registry tree ( ends in .* ),
	# return a registry slice that represents this tree.
	my $prefix = $1;
        my $slice = RegistryCmds::registryWalk( $self->{_Filer}->conn, $prefix );
	return undef unless $slice;
	# Now construct an anonymous slice description and a hash,
	# and tie them together. Return a reference to this hash.
	my $regSlice = {};
        for my $fullkey ( keys %{$slice}) {
	    my $shortkey = $fullkey;
	    $shortkey =~ s/^$prefix\.//;
	    $regSlice->{$shortkey} = sub { $fullkey };
	}
	tie my %virtSlice, 'NOTE::Registry::Slice', $self, $regSlice;
	return \%virtSlice;
    } else {
	return RegistryCmds::registryGet($self->{_Filer}->conn, $key);
    }
}

sub STORE {
    my ($self, $key, $value) = @_;
    if ( ref $value eq 'ARRAY' ) {
	$value = join( ' ', @{$value} );
    }
    RegistryCmds::registrySet($self->{_Filer}->conn, $key, $value) or croak $@;
}

sub DELETE {
    my ($self, $key) = @_;
    if ( $key =~ /\.\*$/ ) {
	$key =~ s/\.\*$//;
        RegistryCmds::registryDeltree($self->{_Filer}->conn, $key) or croak $@;
    } else {
        RegistryCmds::registryDelete($self->{_Filer}->conn, $key) or croak $@;
    }
}

sub DESTROY {
    my ($self) = @_;
}

1;
