#
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/7G/Volume.pm#1 $
#
## @summary  GNOTE 7G Volume module
## @author   tmichael@netapp.com
## @status   Private
## @pod      here
package NOTE::7G::Volume;
use Carp qw( croak );
use GNOTE::7G;
use GNOTE::Volume;
our @ISA = qw(NOTE::Volume NOTE::7G);

use Tharn qw();
use Volume qw();
use Volume::Clone qw();
use Volume::Copy qw();
use Aggregate qw();
use Aggregate::ImplTemplate::RaidInfo_new qw();
use ExportFS qw();
use Wafliron qw();
use disk qw();
use SnapCmds qw();
use SnapMirror qw();
use GenericMount qw();
use fileSystem qw();
use Utils::Units qw();
use VFILER::Util qw();
use RichData qw();

my $regSlice = {
    "options.convert_ucode" 	      => sub { "options.vols.".shift->name.".convert_ucode" },
    "options.create_ucode"  	      => sub { "options.vols.".shift->name.".convert_ucode" },
    "options.lang"	    	      => sub { "options.vols.".shift->name.".lang" },
    "options.maxdirsize"    	      => sub { "options.vols.".shift->name.".maxdirsize" },
    "options.maxfiles"	    	      => sub { "options.vols.".shift->name.".maxfiles" },
    "options.minra"	    	      => sub { "options.vols.".shift->name.".minra" },
    "options.no_i2p"	    	      => sub { "options.vols.".shift->name.".no_i2p" },
    "options.nosnap"	    	      => sub { "options.vols.".shift->name.".nosnap" },
    "options.nosnapdir"	    	      => sub { "options.vols.".shift->name.".nosnapdir" },
    "options.nvfail"	    	      => sub { "options.vols.".shift->name.".nvfail" },
    "options.quotas"	    	      => sub { "options.vols.".shift->name.".quotas" },
    "options.raidsize"	    	      => sub { "options.vols.".shift->name.".raidsize" },
    "options.resyncsnaptime"	      => sub { "options.vols.".shift->name.".resyncsnaptime" },
    "options.no_atime_update"	      => sub { "options.vols.".shift->name.".no_atime_update" },
    "options.snaplock_default_period" => {
	get => sub { shift->options( 'snaplock_default_period' ) },
        set => sub { shift->options( 'snaplock_default_period', shift ) },
    },
    "options.snaplock_maximum_period" => {
	get => sub { shift->options( 'snaplock_maximum_period' ) },
        set => sub { shift->options( 'snaplock_maximum_period', shift ) },
    },
    "options.snaplock_minimum_period" => {
	get => sub { shift->options( 'snaplock_minimum_period' ) },
        set => sub { shift->options( 'snaplock_minimum_period', shift ) },
    },
    "options.snapmirrored"            => sub { "options.vols.".shift->name.".snapmirrored" },
    "status.checksum.enabled"	      => sub { "status.vols.".shift->name.".checksum.enabled" },
    "status.checksum.style"	      => sub { "status.vols.".shift->name.".checksum.style" },
    "status.diskcount"	      	      => sub { "status.vols.".shift->name.".diskcount" },
    "status.filestotal"	      	      => sub { "status.vols.".shift->name.".filestotal" },
    "status.filesused"	      	      => sub { "status.vols.".shift->name.".filesused" },
    "status.inconsistent"	      => sub { "status.vols.".shift->name.".inconsistent" },
    "status.mirror_status"	      => sub { "status.vols.".shift->name.".mirror_status" },
    "status.name"	      	      => sub { "status.vols.".shift->name.".name" },
    "status.plex_count"	      	      => sub { "status.vols.".shift->name.".plex_count" },
    "status.quotainit"	      	      => sub { "status.vols.".shift->name.".quotainit" },
    "status.raidsize"	      	      => sub { "status.vols.".shift->name.".raidsize" },
    "status.raidstatus"	      	      => sub { "status.vols.".shift->name.".raidstatus" },
    "status.sizeavail.bytes"	      => sub { "status.vols.".shift->name.".sizeavail.bytes" },
    "status.sizetotal.bytes"	      => sub { "status.vols.".shift->name.".sizetotal.bytes" },
    "status.sizeused.bytes"	      => sub { "status.vols.".shift->name.".sizeused.bytes" },
    "status.sizeused.percent"	      => sub { "status.vols.".shift->name.".sizeused.percent" },
    "status.snaplock_compliance"      => sub { "status.vols.".shift->name.".snaplock_compliance" },
    "status.snaplock_enterprise"      => sub { "status.vols.".shift->name.".snaplock_enterprise" },
    "status.space_reservations"       => sub { "status.vols.".shift->name.".space_reservations" },
    "status.state"      	      => sub { "status.vols.".shift->name.".state" },
    "state.snapmirror.restart"	      => sub { "state.snapmirror.restart.vol.data.".shift->name },
    "snapmirror.contents"	      => sub { "status.snapmirror.active.".shift->fullname.".contents" },
    "snapmirror.current_size"	      => sub { "status.snapmirror.active.".shift->fullname.".current_size" },
    "snapmirror.current_transfer_error" =>
	sub { "status.snapmirror.active.".shift->fullname.".current_transfer_error" },
    "snapmirror.current_transfer_type" =>
	sub { "status.snapmirror.active.".shift->fullname.".current_transfer_type" },
    "snapmirror.lag" =>
	sub { "status.snapmirror.active.".shift->fullname.".lag" },
    "snapmirror.last_transfer_duration" =>
	sub { "status.snapmirror.active.".shift->fullname.".last_transfer_duration" },
    "snapmirror.last_transfer_from" =>
	sub { "status.snapmirror.active.".shift->fullname.".last_transfer_from" },
    "snapmirror.last_transfer_size" =>
	sub { "status.snapmirror.active.".shift->fullname.".last_transfer_size" },
    "snapmirror.last_transfer_type" =>
	sub { "status.snapmirror.active.".shift->fullname.".last_transfer_type" },
    "snapmirror.snapshot" =>
	sub { "status.snapmirror.active.".shift->fullname.".snapshot" },
    "snapmirror.source"	=>
	sub { "status.snapmirror.active.".shift->fullname.".source" },
    "snapmirror.state" =>
	sub { "status.snapmirror.active.".shift->fullname.".state" },
    "snapmirror.status"	=>
	sub { "status.snapmirror.active.".shift->fullname.".status" },
    "snapmirror.timestamp" =>
	sub { "status.snapmirror.active.".shift->fullname.".timestamp" },
};

sub _connect {
    my $self = shift;

    my $args = $self->{_args};

    # if the $args->{Filer} is a filer object, use it,
    # otherwise try to create one by calling the NOTE::Filer constructor.
    if ( !defined($args->{Filer}) ) {
        # In this case, there are no arguments (i.e. no hostname) for the
        # filer. I call the filer method anyway and let it decide to
        # handle it or croak.
        $self->{_Filer} = NOTE::Filer->connect();
    } elsif ( ref $args->{Filer} eq "HASH" ) {
	$self->{_Filer} = NOTE::Filer->connect( %{$args->{Filer}} );
    } elsif ( ref $args->{Filer} eq "NOTE::Filer" ) {
	$self->{_Filer} = $args->{Filer}; # This is already an object
    } elsif ( ref \$args->{Filer} eq "SCALAR" ) {
	# Is just a hostspec
	$self->{_Filer} = NOTE::Filer->connect( host => $args->{Filer} );
    } else {
	croak "Invalid filer specification: ".ref \$args->{Filer}."\n";
    }
    delete $args->{Filer};
}

sub new {
    my ($class,%args) = @_;
    my $self = {};
    my $args;

    $self->{_debug} = $args{debug};
    $self->{_owner} = $$;

    $self->{_args} = $args = \%args;

    exists $args->{type}      and $args->{type}      = NOTE::option( $args->{type} );
    exists $args->{name}      and $args->{name}      = NOTE::option( $args->{name} );
    exists $args->{disks}     and $args->{disks}     = NOTE::option( $args->{disks} );
    exists $args->{recycle}   and $args->{recycle}   = NOTE::option( $args->{recycle} );
    exists $args->{anonymous} and $args->{anonymous} = NOTE::option( $args->{anonymous} );
    exists $args->{snaplock}  and $args->{snaplock}  = NOTE::option( $args->{snaplock} );
    exists $args->{richdata}  and $args->{richdata}  = NOTE::option( $args->{richdata} );
    exists $args->{dataset}   and $args->{dataset}   = NOTE::option( $args->{dataset} );
    exists $args->{raidtype}  and $args->{raidtype}  = NOTE::option( $args->{raidtype} );
    exists $args->{rgsize}    and $args->{rgsize}    = NOTE::option( $args->{rgsize} );

    $args->{type} ||= 'flex';
    unless ( $args->{type} =~ /(trad|flex)/ ) {
	croak "Unknown type: ".$args->{type};
    }
    $self->{_type} = $args{type};
    %{$args} = %{NOTE::_hashOverlay( target => $args, source => $args->{$self->{_type}}, overwrite => 0 )};

    exists $args->{size}      and $args->{size}      = NOTE::option( $args->{size} );

    delete $args->{flex};
    delete $args->{trad};

    bless $self, $class;

    # This is needed because $self is already an object, but
    # hasn't been physically created yet.
    # This flag will be cleared when the volume is recycled
    # or created.
    $self->is_destroyed = 1;

    $self->_connect;
    $self->SUPER::new(%{$args});
    if ( !defined $args->{size} ) {
	if ( $args->{dataset} ) {
	    # In this case, don't allow volume size to be undefined,
	    # otherwise, the volume chosen might be bigger than the minimum.
	    # If this is a source volume, the destination might then
	    # end up being smaller, if it also was created without
	    # specifying a size. Setting it here insures that it
	    # will be exactly the minimum allowed size.
	    $args->{size} = "20m";
	}
    }
    tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;
    return $self;
}

sub create {
    my ($class,%args) = @_;
    my $exportfs_opts = (delete $args{exportfs}) || "rw,anon=0";
    my $self = __PACKAGE__->new( %args );
    # user can't decide if they want trad or flex
    if ( ($self->{_type} eq 'trad') && defined $self->{_args}->{Aggregate} ) {
	croak "Tradvol can't be created in an aggregate";
    }
    unless ($self->{_type} eq 'trad') {
        $self->{_args}->{size} = NOTE::units( $self->{_args}->{size} || "20m" );
    }
    $self->SUPER::create;
    $self->{_destroyed} = 0;
    $self->exportfs( $exportfs_opts ) if ( $exportfs_opts );
    return $self;
}

sub attach {
    my ($class,%args) = @_;
    my $self = {};
    $self->{_args} = $args = \%args;
    my $exportfs_opts = (delete $args->{exportfs}) || "rw,anon=0";
    $self->{_debug}    = $args{debug};
    $self->{_owner}    = $$;

    exists $args->{type}      and $args->{type}      = NOTE::option( $args->{type} );
    exists $args->{name}      and $args->{name}      = NOTE::option( $args->{name} );

    bless $self, $class;
    $self->SUPER::attach( %args );
    tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;

    if ( exists $args{mntpoint} ) {
	$self->{_mntpoint} = $args{mntpoint};
    }

    # It's possible this'll fail if volume is offline:
    eval { $self->exportfs( $exportfs_opts ) };
    return $self;
}

=head2 B<Disk Space>

=cut

=head2 ::size

Returns the size of the volume in KB.

=over

=item Arguments

I<size spec>

I<Optional.>
A new size to make the volume.

=item Examples

=back

     $size = $vol->size;
     $size = $vol->size( '500m' );   # 500MB returned as kilobytes
     $size = $vol->size( '+100m' );  # Increase volume size

=cut

sub size {
    my ($self,$newsize) = @_;
    my $ret;

    if ( defined $newsize ) {
        # If there are no units specified, default to 'k'
        $newsize .= "k" unless ( $newsize =~ /[tbkmg]$/i );
    }

    if ( $self->type eq 'flex' ) {
	if ( defined($newsize) ) {
	    $ret = Volume::vol_size( $self->Filer->conn, 
				     $self->{_name}, 
				     $newsize );
	    croak "Unable to resize ".$self->description.": $@" if ( $@ );
	} else {
	    $ret = Volume::vol_size( $self->Filer->conn, 
				     $self->{_name} );
	    croak "Unable to get size of ".$self->description.": $@" if ( $@ );
	}
    } else {
	if ( defined($newsize) ) {
	    croak "Cannot resize traditional volume. Add disks to it instead";
	} else {
	    $ret = $self->disk_free;
	    croak "Unable to get size of ".$self->description.": $@" unless( $ret );
	}
    }

    croak $@ if $@;
    # Caching the size helps performance when sorting arrays of volumes by size
    $self->{_size} = $ret;
    return $ret;
}

=head2 ::disk_free

Returns a list of items about the disk usage
of the volume.
The items returned,
and the order in which they are returned are:

B<total> -- the total disk space extant in the volume

B<used> -- the total disk space currently used in the volume

B<avail> -- the total disk space currently available in the volume

B<capacity> -- the percentage of the total disk space
in the volume that is currently being used

There are also methods by each of the item names
above that return only the value for that particular item.

=over

=item Arguments

None.

=item Example

=back

     ($total $used $avail $capacity) = $vol->disk_free;

=cut

sub disk_free {
    my ($self,%args) = @_;
    my $vol;

    exists $args{'type'} and $args{'space_type'} = delete $args{'type'};

    if ( $self->isa( "NOTE::Qtree" ) ) {
	# Do this until I figure out how to get the
	# stats of a qtree:
	$vol = $self->Volume;
    } else {
	$vol = $self;
    }
    my ($total,$used,$avail,$capacity) = Volume::vol_disk_free(
	$vol->Filer->conn,
	path => $vol->mount_path,
	%args
    );
    croak $@ if $@;
    return wantarray ? ($total,$used,$avail,$capacity) : $total;
}

=head2 ::total

Returns the total disk space extant in the volume.

=over

=item Arguments

None.

=item Examples

=back

     $total = $vol->total;

=head2 ::used

Returns the total disk space currently used in the volume.

=over

=item Arguments

None.

=item Examples

=back

     $used = $vol->used;

=head2 ::avail

Returns the total disk space currently available in the volume.

=over

=item Arguments

None.

=item Examples

=back

     $total = $vol->avail;

=head2 ::capacity

Returns the percentage of the total disk space
in the volume that is currently being used.

=over

=item Arguments

None.

=item Examples

=back

     $capacity = $vol->capacity;

=cut

sub total    { return (shift->disk_free( @_ ))[0] }
sub used     { return (shift->disk_free( @_ ))[1] }
sub avail    { return (shift->disk_free( @_ ))[2] }
sub capacity { return (shift->disk_free( @_ ))[3] }

=head2 ::add

Adds disks to the volume.
Only valid for traditional volumes.

If both the B<disks> and B<disklist> arguments are given
the B<disks> argument will be used and B<disklist> ignored.

If neither option is given the default is C<disks =E<gt> 1>.

=over

=item Arguments

B<disks> => I<positive integer>

I<Optional.>
The number of disks to add.
The specific disks will be picked by ONTAP.

B<disklist> => C<(> I<disk name> ... C<)>

I<Optional.>
A list of disks to add.

=item Examples

=back

     $vol->add;
     $vol->add( disks => 2 );
     $vol->add( disklist => qw(0b.13, 0b.16) );

=cut

sub add {
    my ($self,%args) = @_;

    my $diskspec = $args{disks} || $args{disklist};
    delete $args{disks};
    delete $args{disklist};
    $diskspec = "1" unless ( $diskspec );

    if ( ref $diskspec eq "ARRAY" ) {
        my $howmany = scalar @{$diskspec};
        my $what = $howmany == 1 ? "disk" : "disks";
        NOTE::logcomment( "Adding $howmany $what to ".$self->description );
    } else {
        my $what = $diskspec == 1 ? "disk" : "disks";
        NOTE::logcomment( "Adding $diskspec $what to ".$self->description );
    }
    # note: disks can't be added to a flexvol.
    # let ONTAP do the error checking for flexvol
    Volume::vol_add(
	$self->Filer->conn,
	$self->name,
	$diskspec,
	%args
    );
    croak $@ if $@;
    if ( $self->type eq 'trad' && $self->{_anonymous} && $self->{_recycle} ) {
	NOTE::logcomment(
	    "Added disk to anonymous, recyclable tradvol.\n".
	    "    Turning recycle flag off so volume will be destroyed at teardown.\n".
	    "    Name the volume if you want to get around this."
	);
	$self->{_recycle} = 0;
    }
    return $self;
}

=head2 ::disks

Returns a list of the disks
comprising a traditional volume
or the array of disks used by
the underlying aggregate
of a flexible volume.

=over

=item Arguments

None.

=item Example

=back

     @disks = $vol->disks;

=cut

sub disks {
    my $self = shift;
    my @disks = Volume::vol_disks( $self->{_Filer}->conn, $self->{_name} );
    croak $@ if $@;
    if ( scalar @disks == 0 ) {
        croak "Volume ".$self->description." reports ZERO disks";
    }
    return @disks;
}

=head2 B<Mounting>

=cut

=head2 ::exportfs

Gets or sets NFS export options on the volume.

If no option string is specified, returns the current exportfs
value for the volume.

If called multiple times, this function will not re-issue the
exportfs command to the filer unless the exportfs option string
changes or a call to
L<C<NOTE::Volume::unexportfs>|engineering::ontap_tests::main::docs::lib-NOTE-Volume-pm/::unexportfs>
is made.

=over

=item Arguments

Optional: A string containing NFS exportfs options.

=item Example

=back

     $vol->exportfs( 'rw,nosuid,anon=0' );   # No spaces allowed within the list

     printf "Volume export options are ", $vol->exportfs, "\n";

=cut

sub exportfs {
    my ($self,$exportfs_opts) = @_;

    $self->Filer->check_license( "nfs" );
    if ( defined $exportfs_opts ) {
	if ( $exportfs_opts ne $self->{_exportfs_opts} ) {
	    ExportFS::export_path( $self->Filer->conn, $self->mount_path, OPTIONS => $exportfs_opts );
	    croak $@ if $@;
	    $self->{_exportfs_opts} = $exportfs_opts;
	}
	return $self;
    } else {
	return ExportFS::list_exports( $self->Filer->conn )->{$self->mount_path};
    }
}

=head2 ::unexportfs

Unexport a path on the filer

=over

=item Arguments

None

=item Example

=back

     $vol->unexportfs;

=cut

sub unexportfs {
    my ($self) = @_;

    $self->Filer->check_license( "nfs" );
    ExportFS::unexport_path( $self->Filer->conn, $self->mount_path, EXISTENT => 0 );
    croak $@ if $@;
    delete $self->{_exportfs_opts};
    return $self;
}

=head2 ::mount

NFS mounts the volume.

=over

=item Arguments

I<string>

I<Optional.>
A list of mount options.
Valid options are:

XXX

=item Example

=back

     $vol->mount( chmod => '777' );

=cut

sub mount {
    my ($self,%opts) = @_;

    $self->Filer->check_license( "nfs" );
    my $reply = SayWrap::sayEOF( $self->Filer->conn, "nfs on");
    if( $reply !~ /NFS server is running/ ) {
	croak("Unable to bring NFS server up using 'nfs on' command.\nERROR: $reply");
    }
    NOTE::logcomment("NFS MOUNT: $reply");
    if ( exists $opts{'exportfs'} ) {
	$self->exportfs( $opts{'exportfs'} );
	delete $opts{'exportfs'};
    }
    unless ( $self->{_mntpoint} ) {
	$self->{_client} = exists $opts{client} ? $opts{client} : "localhost";
	NOTE::logcomment( "Mounting ".$self->description." on ".$self->{_client} );
        $self->{_mntpoint} = GenericMount::mount(
	    $self->Filer->host,
	    $self->mount_path,
	    %opts,
        );
    }
    croak $@ if $@;
    return $self->{_mntpoint};
}

=head2 ::umount

Unmounts the volume.

=over

=item Arguments

I<string>

I<Optional.>
A list of unmount options.
Valid options are:

XXX

=item Example

=back

     $vol->umount;

=cut

sub umount {
    my ($self, %opts) = @_;

    if ( $self->{_mntpoint} ) {
	$opts{client} = $self->{_client} unless $opts{client};
	NOTE::logcomment( "Unmounting ".$self->description." from ".$opts{client} );
	if ( GenericMount::umount( $self->{_mntpoint}, %opts ) ) {
	    delete $self->{_mntpoint};
	    return 1;
	}
	croak "NOTE::Volume::umount: $@" if $@;
    }
    return 0;
}

=head2 ::mount_path

Returns the full path name of the volume,
i.e. C</vol/><I<volume name>>

=over

=item Arguments

None.

=item Example

=back

     $mntpt = $vol->mount_path;

=cut

sub mount_path { return "/vol/".shift->{_name} }

=head2 ::hostpath

Returns the name of the volume
prepended by its filer's name.

=over

=item Arguments

None.

=item Example

=back

     $hp = $vol->hostpath;

=cut

sub hostpath {
    my $self = shift;
    return $self->{_Filer}->host.":".$self->{_name};
}

=head2 ::fsid

Sets or returns Volume's fsid

=over

=item new_fsid

I<Optional.>

If supplied will set the volume's fsid

=item Examples

	$id = $vol->fsid;

	$new_id = $vol->fsid($new_fsid);

=back

=cut

sub fsid {
    my ($self, $new_fsid, %args) = @_;
    my $fsid;

    if ( defined($new_fsid) ) {
	NOTE::logcomment( "Setting FSID of ".$self->description." to 0x%08x", $new_fsid );
	$fsid = Volume::vol_rewrite_fsid(
	    $self->Filer->conn, 
	    $self->name,
	    fsid => sprintf( "0x%x", $new_fsid ),
	    %args, 
	);
    } else {
        $fsid = Volume::vol_read_fsid( $self->Filer->conn, $self->name );
    }
    croak $@ if $@;
    return hex $fsid;
}

=head2 ::uuid

Returns volume's UUID

=over

=item Arguments

None

=item Example

=back

     $id = $vol->uuid;

=cut

sub uuid {
    my ($self) = @_;

    my $vinfo = Aggregate::ImplTemplate::RaidInfo_new::get_vol_raid_info($self->Filer->conn, $self->name);
    return $vinfo->{uuid};
}

=head2 ::offline, ::online, ::restrict

Sets volume offline, online and restrict respectively

=over

=item Arguments

None

=item Examples

=back

     $vol->offline;

     $vol->online;

     $vol->restrict;

=cut

sub offline {
    my $self = shift;
    Volume::vol_offline( $self->Filer->conn, $self->{_name}, @_ );
    croak $@ if $@;
    return $self;
}

sub online {
    my $self = shift;
    Volume::vol_online( $self->Filer->conn, $self->{_name}, @_ );
    croak $@ if $@;
    return $self;
}

sub restrict {
    my $self = shift;
    Volume::vol_restrict( $self->Filer->conn, $self->{_name}, @_ );
    croak $@ if $@;
    return $self;
}

=head2 ::status

Get the status of the volume.
The return value is a hash reference.

=over

=item Arguments

None.

=item Examples

=back

     $status = $vol->status;

=cut

sub status {
    my $self = shift;
    Volume::vol_status( $self->Filer->conn, $self->name );
}

=head2 ::restricted

Returns a boolean indicating whether a volume is restricted or not

=over

=item Arguments

None.

=item Examples

=back

      $vol->restricted;

=cut

sub restricted {
    my $self = shift;
    my ($state, $status, $aggr, $options) = Volume::vol_status(
        $self->Filer->conn, $self->name
    );
    croak $@ if $@;
    return $state =~ /restricted/;
}

=head2 ::snaplock

If a volume is a lockvault volume then returns it's lockvault type,
whether compliance or enterprise. Otherwise returns 0

=over

=item Arguments

None.

=item Example

=back

     $vol->snaplock;

=cut

sub snaplock {
    my $self = shift;
    # use cached value if available
    unless ( defined $self->{_snaplock} ) {
	if ( $self->type eq 'flex' ) {
	    # Snaplock type is inherited from containing aggregate
	    $self->{_snaplock} = $self->{_Aggregate}->snaplock;
	} else {
	    my $opts = $self->options;
	    if ( $opts->{'snaplock_compliance'} ) {
		$self->{_snaplock} = 'compliance';
	    } elsif ( $opts->{'snaplock_enterprise'} ) {
		$self->{_snaplock} = 'enterprise';
	    } else {
		$self->{_snaplock} = 0;
	    }
	}
    }
    return $self->{_snaplock};
}

=head2 ::snapmirrored

Returned a boolean indicating whether the volume state is snapmirrored or not

=over

=item Arguments

None.

=item Examples

=back

     $vol->snapmirrored

=cut

sub snapmirrored {
    my $self = shift;
    my ($state, $status, $aggr, $options) = Volume::vol_status(
        $self->Filer->conn, $self->name
    );
    croak $@ if $@;
    return $state =~ /snapmirrored/;
}

=head2 ::treecompare

Perform treecompare to perform treecompare between this object
with another NOTE object. Treecompare is done on the filer that the other
object resides on.

Returns 1 if match and croaks if any mismatches found.

=over

=item Arguments

B<other> - I<NOTE object>

The other object to perform treecompare with.

B<%args> - Hash of parameters that filerSystem::treecompare_vol accepts.

=item Examples

=back

     $vol_src->treecompare($vol_dst);

     $vol_src->treecompare($vol_dst, args=>"-v");

=cut

sub treecompare {
    my ($self,$other,%args) = @_;
    my $ret;

    $args{timeout} = ($self->Filer->is_a_simulator ? 900 : 300)
	unless $args{timeout};

    $ret = fileSystem::treecompare_vol(
	$self->Filer->conn,
	$self->name,
	$other->name,
	filer_dst => $other->Filer->conn,
	%args,
    );
    croak $@ if $@;
    return $ret;
}

=head2 ::writeable

Returns a boolean indicating whether the volume is writeable

=over

=item Arguments

None.

=item Examples

=back

      $vol->writeable

=cut

sub writeable {
    my $self = shift;
    my ($state, $status, $aggr, $options) = Volume::vol_status(
        $self->Filer->conn, $self->name
    );
    croak $@ if $@;
    return (
    	$state =~ /online/ &&
    	$status !~ /read-only/ &&
    	$state !~ /restricted/
    );
}

=head2 ::readable

Returns a boolean indicating whether a volume is readable,
ie online and not restricted

=over

=item Arguments

None.

=item Examples

=back

      $vol->readable

=cut

sub readable {
    my $self = shift;
    my ($state, $status, $aggr, $options) = Volume::vol_status(
        $self->Filer->conn, $self->name
    );
    croak $@ if $@;

    return (
    	$state =~ /online/ &&
    	$state !~ /restricted/
    );
}

=head2 ::language

Sets or gets the volume's language setting

=over

=item Argument

None - Returns the current language setting for the volume

1 parameter - Sets the volume language to the supplied language

=item Examples

=back

      $lang = $vol->language;

      $vol->language("fr");

=cut

sub language {
    my ($self, $language) = @_;

    if ( defined $language ) {
	croak "Failed to set new language $language: $@"
		unless Volume::vol_lang($self->Filer->conn, $self->name,
					language=>$language);
	return 1;
    } else {
	my @result = Volume::vol_lang($self->Filer->conn, $self->name);
	croak $@ if $@;
	return wantarray ? @result : $result[0];
    }
}

=head2 ::options

Set or get options of a volume

=over

=item Arguments

No param - Will return a hash of volume's options.

1 param - Returns the volume of the options indicated by the supplied key

2 param - Sets the options indicated by the first parameter to the value indicated by the second, and returns a hash of all the options.

=item Example

=back

      $opts = $vol->options;

      $is_fixed = $vol->options("fs_fixed_size");

      $vol->options("maxdirsize"=>512);

=cut

sub options {
    my ($self,@args) = @_;
    if ( scalar(@args) == 0 ) { 
	my $options = Volume::vol_options( $self->Filer->conn, $self->name );
	croak $@ if $@;
	return $options;

    } elsif ( scalar(@args) == 1 ) {
	my $options = Volume::vol_options(
	    $self->Filer->conn,
	    $self->name
	);
	croak $@ if $@;
	return $options->{ $args[0] };

    } else {
	my $options = Volume::vol_options(
	    $self->Filer->conn,
	    $self->name,
	    @args
	);
	croak $@ if $@;
	return $options;
    }
}

=head2 snapshot_create

Create a snapshot on the volume

=over

=item Argument

B<snapname> - I<name>

Optional

If not set then a anonymous name is used to create the snapshot with

=item Examples

=back

     $vol->snapshot_create;

     $vol->snapshot("special");

=cut

sub snapshot_create {
    my ($self,$snapname) = @_;

    $snapname = $self->newName( "sn" ) unless( $snapname );

    NOTE::logcomment( "Creating snapshot ".$snapname." of ".$self->description );
    SnapCmds::snapshot_create(
        $self->Filer->conn,
        $self->name,
	$snapname,
    );
    croak $@ if ($@);
    return $snapname;
}

=head2 ::snapshot_restore

Restored a named snapshot

=over

=item Arguments

B<snapname> - I<name>

Name of the snapshot to restore

=item Example

=back

     $vol_snapshot_restore("special");

=cut

sub snapshot_restore {
    my ($self,$snapname,%opts) = @_;

    croak "Need snapshot name" unless ( $snapname );
    NOTE::logcomment( "Restoring ".$self->description." using snapshot ".$snapname );

    $self->Filer->check_license( "snaprestore" );
    SnapCmds::snaprestore_volume(
        $self->Filer->conn,
        $self->name,
	$snapname,
	%opts,
    );
    croak $@ if ($@);
    return $self;
}

=head2 ::snapshot_list

Returns an array of the snapshots that exists on this volume

=over

=item Arguments

None.

=item Example

=back

     $vol->snapshot_list;

=cut

sub snapshot_list {
    my $self = shift;

    my @list = SnapCmds::snapshot_list(
        $self->Filer->conn,
        $self->name
    );
    croak $@ if ($@);
    return @list;
}

=head2 ::snapshot_status

Returns a reference to a hash of the status with the
snapshot names as keys.

=over

=item Arguments

None

=item Example

=back

     $vol-snapshot_status;

=cut

sub snapshot_status {
    my $self = shift;
    my %status = ();
    my @status;

    @status = SnapCmds::snapshot_status(
	$self->Filer->conn,
	$self->name
    );
    croak $@ if ($@);

    for ( @status ) {
	$status{$_->{name}} = $_;
    }
    return \%status;
}

=head2 ::snapshot_exists

Returns a boolean to indicate whether a given named snapshot exists or not

=over

=item Arguments

B<snapshot> - I<name>

Name of the snapshot to check

=item Example

=back

     $vol->snapshot_exists('special');

=cut

sub snapshot_exists {
    my ($self,$snapshot) = @_;
    my $ret;

    $ret = SnapCmds::snapshot_exists(
	$self->Filer->conn,
	$self->name,
	$snapshot
    );
    croak $@ if ($@);
    return $ret;
}

=head2 ::snapshot_delete

Delete a named snapshot

=over

=item Arguments

B<snapshot> - I<name>

Name of the snapshot to delete

=item Example

=back

      $vol->snapshot_delete('special');

=cut

sub snapshot_delete {
    my ($self,$snapshot,%args) = @_;

    NOTE::logcomment( "Deleting snapshot ".$snapshot );

    SnapCmds::snapshot_delete(
	$self->Filer->conn,
	$self->name,
	$snapshot,
	# %args
    );
    croak $@ if ($@);
    return $self;
}

=head2 ::clone

Create a clone of this volume

=over

=item Arguments

All parameters are passed to NOTE::Volume::Clone->create;

=item Example

=back

     $vol->clone;

=cut

sub clone {
    my $self = shift;
    return NOTE::Volume::Clone->create( $self, @_ );
}

=head2 ::clones

Returns clones for this volume

=over

=item Arguments

None.

=item Example

=back

     @list = $vol->clones;

=cut

sub clones {
    my $self = shift;
    my @list = Volume::Clone::get_clones(
	conn => $self->Filer->conn,
	vol => $self->name,
    );
}

=head2 ::Clones

Returns a list of NOTE::Volume objects that are clones of this volume

=over

=item Argument

None.

=item Example

=back

     @vol_objs = vol->Clones;

=cut

sub Clones {
    my $self = shift;
    my @objList = ();

    for my $name ( $self->clones ) {
	my $clone = NOTE::Volume::Clone->attach(
	    Filer => $self->Filer,
	    name => $name,
	);
	push @objList, $clone;
    }
    return @objList;
}

=head2 ::wafliron_start

Initiates wafliron on this volume if it is a trad vol, and on the containing
aggregate if a flex volumes.

=over

=item Arguments

B<%args> - I<hash>

Hash of arguments for Wafliron::wafliron_start

=item Example

=back

      $vol->wafliron_start;

=cut

sub wafliron_start {
    my ($self,%args) = @_;
    my @out;

    if ( $self->type eq 'flex' ) {
	@out = $self->Aggregate->wafliron_start( %args );
    } else {
        NOTE::logcomment( "Starting Wafliron on ".$self->description );
	# Make sure root_vol is exportfs'd properly:
	NOTE::Volume->attach( Filer => $self->Filer, name => $self->Filer->vol_root );
	@out = Wafliron::wafliron_start( $self->Filer->conn, 
					 $self->name, 
					 %args );
	
	# user request not to wait for wafliron to complete
	# keep the $id returned to be used for wafliron_wait() method
	if ( !$args{wait} && ref( $out[0] ) ne 'SCALAR' ) {
	    $self->{_wafliron_wait} = $out[0];
	    # Cause it to be stopped on teardown, if still running
	    NOTE::reference( sub { $self->wafliron_stop if ( $self->{_wafliron_wait} ) } );
	}
    }
    return @out;
}

=head2 ::wafliron_stop

Aborts a wafliron running for this trad volume, or the containing
aggregate for a flex volume.

=over

=item Arguments

B<%args> - I<hash>

Hash of parameters for Wafliron::wafliron_stop

=item Example

=back

     $vol->wafliron_stop;

=cut

sub wafliron_stop {
    my ($self,%args) = @_;

    delete $args{id}; # use $self->{_wafliron_wait} instead

    if ( $self->type eq 'flex' ) {
	return $self->Aggregate->wafliron_stop(%args);

    } else {
	my $out;
        NOTE::logcomment( "Stopping Wafliron on ".$self->description );
	if ( exists $self->{_wafliron_wait} ) {
	    $out = Wafliron::wafliron_stop( $self->Filer->conn, 
					    $self->name, 
					    id => $self->{_wafliron_wait},
					    %args );
	    delete $self->{_wafliron_wait};

	} else {
	    $out = Wafliron::wafliron_stop( $self->Filer->conn, 
					    $self->name, 
					    %args );
	}    
	return $out;
    }
}

=head2 ::wafliron_status

Returns the current status of the wafliron running on this trad vol
or the containing aggregate for a flex vol

=over

=item Arguments

None.

=item Example

=back

     $status = $vol->wafliron_status;

=cut

sub wafliron_status {
    my $self = shift;

    if ( $self->type eq 'flex' ) {
	return $self->Aggregate->wafliron_status();
    } else {
	return Wafliron::wafliron_status( $self->Filer->conn, $self->name );
    }
}

=head2 ::wafliron_wait

Wait for wafliron to complete

=over

=item Arguements

None.

=item Examples

=back

     $vol->wafliron->wait;

=cut

sub wafliron_wait {
    my $self = shift;

    my @out;
    if ( $self->type eq 'flex' ) {
	@out = $self->Aggregate->wafliron_wait();

    } else {
	if ( exists $self->{_wafliron_wait} ) {
	    NOTE::logcomment( "Waiting for Wafliron on ".$self->description );
	    @out = Wafliron::wafliron_wait( $self->{_wafliron_wait} );
	    delete $self->{_wafliron_wait};
	} else {
	    croak "Wafliron doesn't seem to be running on this volume";
	}
    }
    return @out;
}

=head2 ::copy

Perform copy data from the supplied object to this object

=over

=item Arguments

B<src> - I<NOTE object>

Src of data to be copied

B<background> - I<Boolean>

Optional

If set will perform copying in the background. Default 0

=item Examples

=back

     $vol->copy(src=>$vol_src);

=cut

sub copy {
    my ($self,%args) = @_;
    my $background = delete $args{background};
    my $src = delete $args{src};
    my %ARGS = ();
    my $was_restricted = $self->restricted;

    croak "src must be specified for copy" unless ( $src );
    
    my $op = "Copying\n" .
             "    +---" . $src->description . "\n" .
             "    +-> " . $self->description;
    NOTE::logcomment( $op );
    $ARGS{uc $_}    = $args{$_} for ( keys %args );
    $ARGS{COMPLETE} = $background ? 0 : 1;
    $ARGS{SOURCE}   = $src->name;

    $self->restrict unless ( $was_restricted );
    my ($ret, $op_num) = Volume::Copy::volcopy_start(
	$self->Filer->conn,
	$self->name,
	%ARGS,
    );
    croak $@ if $@;

    if ( ref $ret eq 'Subtest' ) {
        push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
    } else {
	$ret = $self;
    }

    unless ( $was_restricted ) {
	my $try = 0;
	for $try ( 1..5 ) {
	    eval { $self->online };
	    last unless ( $@ );
	    sleep 1;
	}
	if ( $try >= 5 ) {
	    croak "Couldn't bring ".$self->name." online after volcopy: $@";
	}
    }

    if ( wantarray ) {
        return ($ret, $op_num);
    } else {
        return $ret;
    }
}

=head2 ::store

Invoke 'snapmirror store' to backup volume data to tape

Returns a string representing the backup reference number.

=over

=item Arguments

B<device> - I<name>

Name of tape device to dump data to.

B<background> - I<Boolean>

Optional

If set returns immediately after starting backup.

=item Examples

=back

     $vol->store(device=>"rst01");

=cut

sub store {
    my ($self,%args) = @_;
    my $ref;
    my $device = $args{device};
    my $background = $args{background} || 0;
    delete $args{background};
    delete $args{device};

    unless ( $device ) {
	croak "snapmirror store requires device name";
    }
    NOTE::logcomment( "Storing ".$self->description." to ".$device );

    $ref = SnapMirror::snapmirror_store(
	host     => $self->Filer->hostrec,
	vol      => $self->name,
	dumpfile => $device,
	%args,
    );
    croak $@ if ( $@ );
    if ( $background == 0 ) {
	SnapMirror::snapmirror_tape_status(
	    host => $self->Filer->hostrec,
	    vol  => $self->name,
	    type => "store",
	    wait => 1,
	    timeout => $args{timeout} || 900,
	);
	NOTE::logcomment( "Store complete; reference is $ref" );
    } else {
	NOTE::logcomment( "Store started; reference is $ref" );
    }
    return $ref;
}

=head2 ::retrieve

Invokes 'snapmirror retrieve' to restore data from a tape device

=over

=item Argument

B<device> - I<name>

Name of tape device holding data to be restore

B<background> - I<Boolean>

If set returns immediately upon starting the restore.
When ran in background mode, use SnapMirror::snapmirror_tape_status to
monitor restore's progress.

=item Examples

=back

     $vol->retrieve(device=>"rst01");

=cut

sub retrieve {
    my ($self,%args) = @_;
    my $device  = $args{device};
    my $timeout = $args{timeout} || 900;
    my $background = $args{background} || 0;
    delete $args{device};
    delete $args{timeout};
    delete $args{background};

    unless ( $device ) {
	croak "snapmirror retrieve requires device name";
    }

    NOTE::logcomment( "Restoring ".$self->description." from ".$device );

    $self->restrict;
    SnapMirror::snapmirror_retrieve(
	host        => $self->Filer->hostrec,
	vol         => $self->name,
	restorefile => $device,
	%args,
    );
    croak $@ if ( $@ );
    if ( ! $background ) {
	SnapMirror::snapmirror_tape_status(
	    host    => $self->Filer->hostrec,
	    vol     => $self->name,
	    type    => "retrieve",
	    wait    => 1,
	    timeout => $timeout,
	);
	NOTE::logcomment( "Restore complete" );
    } else {
	NOTE::logcomment( "Restore started" );
    }
    return $self;
}

=head2 ::_vol_exists

Returns a boolean indicating is this volume exists on the filer

=over

=item Arguments

None.

=item Example

=back

     $vol->_vol_exists;

=cut

sub _vol_exists {
    my $self = shift;
    return Volume::vol_exists( $self->Filer->conn, $self->name );
}

sub _vol_aggr {
    my $self = shift;

    my ($state, $status, $aggr, $options) = Volume::vol_status(
	$self->Filer->conn, $self->name,
    );
    return $aggr;
}

sub _vol_size {
}

sub _vol_type {
    my $self = shift;

    my ($state, $status, $aggr, $options) = Volume::vol_status(
	$self->Filer->conn, $self->name,
    );
    return "trad" if ( $status =~ /trad/ );
    return "flex" if ( $status =~ /flex/ );
    return undef;
}

sub _vol_create {
    my ($self,%args) = @_;

    my $size = delete $args{size};
    Volume::vol_create_flex(
	$self->{_Filer}->conn,
	$self->{_name},
	$self->{_Aggregate}->name,
	$size . 'k',
	%args
    );
    if ( $@ ) {
	if ( $@ =~ /space cannot be guaranteed for the volume/ ) {
	    croak "Not enough space";
	}
	croak $@;
    }
}

sub _vol_create_trad {
    my ($self, %args) = @_;
    my $diskspec = delete $args{disks};

    # Fix %args up the way Volume::create wants them:
    if ( $self->{_snaplock} eq 'compliance' ) {
	$args{snaplock} = '1';
    } elsif ( $self->{_snaplock} eq 'enterprise' ) {
	$args{snaplock_enterprise} = '1';
    }
    Volume::vol_create(
	$self->{_Filer}->conn,
	$self->{_name},
	$diskspec,
	%args
    );
    if ( $@ ) {
	if ( $@ =~ /disks needed, but only|no disks are available/ ) {
	    croak "Not enough disks";
	}
	croak $@;
    }
}

sub _vol_rename {
    my ($self,$newname) = @_;
    Volume::vol_rename( $self->{_Filer}->conn, $self->{_name}, $newname );
    croak $@ if $@;
    $self->{_name} = $newname;
}

sub _vol_destroy {
    my ($self,%args) = @_;
    my $force = delete $args{'force'};

    Volume::vol_destroy( $self->Filer->conn, $self->{_name}, destroy_corrupt_volumes => $force, %args );
    croak $@ if $@;

    # zero spares on teardown:
    $self->Filer->zero_spares if ( $self->{_type} eq 'trad' );
}

1;
