#
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/GX/LSMirror.pm#1 $
#
## @summary  NOTE GX LSMirror module
## @author   dl-dpgqa-automation@netapp.com
## @status   Private
## @pod      here
## @author   subrad@netapp.com dl-dpgqa-automation@netapp.com
=head1 NAME

NOTE::GX::LSMirror

=head1 DESCRIPTION

This is the NOTE::GX class for load sharing mirrors.

= head1 METHODS

=cut

package NOTE::GX::LSMirror;
use GNOTE;
use GNOTE::GX;
use GNOTE::GX::Mirror;
our @ISA = qw(NOTE::GX);
use Carp qw(croak);
use Data::Dumper;
use TracePkg;

my $Trace = new TracePkg;
sub _set_src {
    my (%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $Trace->entry(msg=>">>entry");
    my $self = {};
    my $args;
    my $opts;
    $args{dst} ||= {};
    $args{src} ||= {};

    $self->{_args} = $args = \%args;

    my $src_opts = {
        Node    => q/$NODES_SRC:$NODES:$FILER_SRC:$FILER/,
	Vserver => q/$VSERVER_SRC:$VSERVER/,
        name    => q/$SRC_VOL:$FILER_SRC_VOL/,
	tag     => 'Source volume',
        type    => q/$VOL_TYPE/,
        recycle => 1,
	size    => q/$VOL_SIZE/,
	Aggregate => {
	    name    => q/$SRC_AGGR:$FILER_SRC_AGGR/,
	    tag     => 'Source aggregate',
	    share   => 1,
	},
	# '*' means use all nodes in the cluster
	striped => { Aggregate => { nodes => '*' }, },
    };

    if ( ref $args->{src} eq "HASH" ) {
	if ( ref \$args->{src} eq 'SCALAR' && $args->{src} ne 'NOTE::Volume' ) {
	    croak "Invalid src specification: ".$args->{src};
	}
	$opts = $src_opts;
	if ( ref $args->{src} eq 'HASH' ) {
	    $opts = NOTE::_hashOverlay( target => $opts, source => $args->{src}, overwrite => 1 );
	}
	$self->{_src} = NOTE::Volume->create( %{$opts} );
    } elsif ( (ref $args->{src} eq "NOTE::GX::Volume") ) {
        $self->{_src} = $args->{src}; # already an object
    } else {
        croak "Invalid src specification: ".ref $args->{src};
    }
    $self->{_src_cluster} = $args->{src_cluster};

    $Trace->exit(msg=>"<<exit");
    return $self;
}

=head2 B<Relationship Creation>

=head2 ::new

Creates an object representing a LSMirror relationship
but does B<not> cause the creation
of a snapmirror load-share relationship on the GX cluster.
The L<C<::create>|/::create> or L<C<::initialize>|/::initialize> methods of the return
object may be used to create the physical relationship on the cluster.

This method should not normally be used;
use C<::create> or C<::initialize> directly instead.

=over

=item Arguments

B<src> => I<Volume object> | I<Volume spec> | 'NOTE::Volume' | 'NOTE::GX::Volume'

I<Optional.>
A a volume spec or object. May also be the string or NOTE::Volume
to indicate directly that a Volume with default parameters is to be created.
The default is NOTE::Volume'.


=cut

sub new {
    my ($class, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $Trace->entry(msg=>">>entry");

    my $self = _set_src(%args);

    $self->{_owner} = $$;
    $self->{_destroyed} = 1;

    bless $self, $class;
    NOTE::reference( $self );
    $Trace->exit(msg=>"<<exit");

    return $self;
}

=head2 ::create

Creates a LSMirror relationship
and returns an object representing the relationship.

== over

== item Arguments

B<src> => I<Volume object> | I<Volume spec> | 'NOTE::Volume' | 'NOTE::GX::Volume'

I<Optional.>
A a volume spec or object. May also be the string or NOTE::Volume
to indicate directly that a Volume with default parameters is to be created.
The default is NOTE::Volume'.

B<src_cluster> => I<string>

I<Required.>
The name of the cluster where this load-share mirror is going to be created.

B<foreground> = 'true' | 'false'

I<Optional.>
Puts the create operation into foreground if the value is 'true'.


=cut

sub create {
    my ($class, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $Trace->entry(msg=>">>entry");
    my $self = __PACKAGE__->new( %args );
    my %opts = {};
    my $was_mounted;

    $opts{'source-volume'}  = $self->{_src}->name;
    $opts{'source-vserver'} = $self->{_src}->Vserver->name;
    $opts{'source-cluster'} = $self->{_src_cluster}; 
    $opts{comment}       = $args{comment} if exists $args{comment};
    $opts{foreground}    = "true";

    check_and_add_license($self);
 
    # If the volume is mounted prior to creating a load shring 
    # mirror relationship, this need to be unmounted and 
    # mounted at ./admin/<volumename>.
    # CAVEAT: User may need to be aware of this action as he/she
    # may need to get a new mount object.
    if ( $was_mounted = $self->{_src}->mounted ) {
        $self->{_src}->umount;
    }

    NOTE::logcomment("Creating LSMirror: ".$self->description);

    eval {
	$self->{_src}->server->snapmirror_load_share_create(%opts);
    };
    croak "$@" if $@;

    # If the load share mirror relationship is successfully created, 
    # mark the load share source as load sharing source so that we can
    # avoid a call to snapmirror show during mounting.
    $self->{_src}->set_loadsharing_src_flag;
    # If the source was mounted, then remount it
    $self->{_src}->mount if $was_mounted;
    $self->{_destroyed} = 0;
    $Trace->exit(msg=>"<<exit");

    return $self;
}

=head2 ::initialize

Performs a 'snapmirror load-share initialize' to perform the baseline transfer of data.
If the relationship has not been created before, it will call the C<::create>
method to create the relationship.

=over

=item Arguments

== item Arguments

B<src> => I<Volume object> | I<Volume spec> | 'NOTE::Volume' | 'NOTE::GX::Volume'

I<Optional.>
A a volume spec or object. May also be the string or NOTE::Volume
to indicate directly that a Volume with default parameters is to be created.
The default is NOTE::Volume'.

B<src_cluster> => I<string>

I<Required.>
The name of the cluster where this load-share mirror is going to be created.

B<foreground> = 'true' | 'false'

I<Optional.>
Puts the create operation into foreground if the value is 'true'.

=item Examples

=back


    my $vol_src = NOTE::Volume->create();
    my $vol_dst = NOTE::Volume->create();
    my $lsm = NOTE::GX::LSMirror::initialize(src=>$vol_src);

=cut

sub initialize {
    my ($class, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 
    $Trace->entry(msg=>">>entry");
    my $self = {};
    if ( ref $class eq "NOTE::GX::LSMirror") {
        $self = $class;
    } else {
        $self = __PACKAGE__->create(%args);
    }

    # Run initialize command in background mode, and use status to monitor progress
    my %opts;
    $opts{'source-volume'}  = $self->{_src}->name;
    $opts{'source-vserver'} = $self->{_src}->Vserver->name;
    $opts{'source-cluster'} =  $self->{_src_cluster};
    $opts{'foreground'} = (exists $args{'foreground'}) ? $args{'foreground'} : "true";
    $opts{'-Timeout'}   = 108000;

    NOTE::logcomment("Initialize LSMirror: ".$self->description);
    eval {
	$self->{_src}->server->snapmirror_load_share_initialize(%opts);
    };
    croak "$@" if $@;
    $Trace->exit(msg=>"<<exit");

    return $self;
}


=head2 ::update

Performs a 'snapmirror load-share update' to perform the incremental transfer of data. 

=over

=item Arguments

B<foreground> => I<string>

I<Optional.>
true - if the lsmirror update job will be executed in foreground
false - if the lsmirror update job will be executed in background

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 300 seconds.

=item Examples

	my $lsm = NOTE::GXP::LSMirror->initialize(src => $vol_src );
        $lsm->update();

=back

=cut

sub update {
    my ($self, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $Trace->entry(msg=>">>entry");
    # Run update command in background mode, and use status to monitor progress
    my %opts;
    $opts{'source-volume'}         = $self->{_src}->name;
    $opts{'source-vserver'} = $self->{_src}->Vserver->name;
    $opts{'source-cluster'} =  $self->{_src_cluster};
    $opts{'foreground'} = (exists $args{'foreground'}) ? $args{'foreground'} : "true";
    $opts{'-Timeout'}   = $args{'timeout'} if exists $args{'timeout'};

    NOTE::logcomment("Update snapmirror: ".$self->description);
    eval {
        $self->{_src}->server->snapmirror_load_share_update(%opts);
    };
    croak "$@" if $@;

    $Trace->exit(msg=>"<<exit");
}

#############################################
=head2 ::abort

Performs a 'snapmirror load-share abort' to abort the load-share mirror transfer. 

=over

=item Arguments

B<hard> => I<string>

I<Optional.>
true -  if true, check point will be deleted when the transfer aborts.
false - false is the default.

B<check> => I<number>

I<Optional.>
true - if true, "snapmirror abort" only aborts any "snapmirror check" operation.
false - false is the default.

=item Examples

	my $lsm = NOTE::GXP::LSMirror->initialize(src => $vol_src );
        $lsm->abort();

=back

=cut

sub abort {
    my ($self, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $Trace->entry(msg=>">>Entering LSMirror::abort");

    my %opts;
    $opts{'source-volume'}  = $self->{_src}->name;
    $opts{'source-vserver'} = $self->{_src}->Vserver->name;
    $opts{'source-cluster'} =  $self->{_src_cluster};
    $opts{'hard'}   = $args{'hard'} if exists $args{'hard'};
    $opts{'check-only'}   = $args{'check'} if exists $args{'check'};

    eval {
        $self->{_src}->server->snapmirror_load_share_abort(%opts);
    };
    croak "$@" if $@;

    $Trace->exit(msg=>"<<Exiting LSMirror::abort");
}


=head2 ::status

Returns a hash reference to the show of the relationship.

=over

=item Arguments

B<element> = I<status element>

If defined returns just the named status element.

=item Examples

=back

    my $stat = $lsm->status;
    my $state = $lsm->status('Mirror Status');

=cut

sub status {
    my ($self, $entry) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $Trace->entry(msg=>">>Entering LSMirror::status");

    my %opts, $show;
    $opts{'source-volume'}  = $self->{_src}->name;
    $opts{'source-vserver'} = $self->{_src}->Vserver->name;
    $opts{'source-cluster'} = $self->{_src_cluster};
    
    eval {
        $show = $self->{_src}->server->snapmirror_load_share_show(%opts);
    };
    croak "$@" if $@;

    $Trace->exit(msg=>"<<Exiting LSMirror::status");
    if( defined $entry) {
         return $show->[0]{$entry};
    } else {
         return $show->[0];
    }
}

=head2 ::check

Performs a "snapmirror check" to verify the integrity of
transferred data.  If the checker detects an inconsistent
mirror, anomalies are recorded in a consolidated log file
that is specified as a node and absolute pathname. Possible
return values:

       0 - Checker ran successfully and found consistent mirrors

$hashref - Checker ran successfully and found inconsistent mirrors
           where:

               node => $nodename
               path => $logfile_abs_path

=over

=item Arguments

B<target-snapshot> => I<string>

I<Optional.>
target-snapshot name for a specific snapshot to be
used to the snapmirror check.

B<timeout> => I<timeout>

I<Optional>

Timeout in seconds. Default: 30 mins

=item Examples

        $lsm->check('target-snapshot' => "snapshot");

        $chk_log = $lsm->check();
        if ( $chk_log ) { ... }

=back

=cut

sub check{

    $Trace->entry(msg=>">>Entering LSMirror::check()");

    my ($self, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    croak "Missing LSM dest volume object" unless ($args{dst});

    my %opts = ();
    my ( $chk_result, @log_loc );

    $opts{'foreground'} = "true";

    my $dst_name = $args{dst}->name;
    my $dst_vserver_name = $args{dst}->Vserver->name;
    my $dst_cluster_name = $args{dst}->Cluster->name;

    $opts{'destination-volume'} = $dst_name;
    $opts{'destination-vserver'} = $dst_vserver_name;
    $opts{'destination-cluster'} = $dst_cluster_name;
    $opts{'source-volume'} = $self->{_src}->name;
    $opts{'source-vserver'} = $self->{_src}->Vserver->name;
    $opts{'source-cluster'} = $self->{_src_cluster};

    exists $args{'target-snapshot'} ?
    $opts{'target-snapshot'} = $args{'target-snapshot'} :
        delete $opts{'target-snapshot'};
    exists $args{'timeout'} ?
    $opts{'-Timeout'} = $args{'timeout'} :
    $opts{'-Timeout'} = 3600; # One hour

    eval {
        $chk_result = $self->{_src}->server->snapmirror_check(%opts);
    };

    if ( $@ ) {

        NOTE::logcomment("error during snapmirror check");
        croak "$@";

    } elsif ( $chk_result =~ /NOTE: Found inconsistent mirror/ ) {

        NOTE::logcomment("snapmirror check - Found inconsistent mirror");
        @log_loc = split( ' ', $chk_result );   # extract log node and path

        return { node => $log_loc[-4], path => $log_loc[-1] };
    }
    else {

        NOTE::logcomment("snapmirror check - mirror OK");
        return 0;
    }
}
    
############################################
=head2 B<Component Objects>

=head2 B<Names and Paths>

=head2 ::src_name

Returns the name of the source volume.

=over

=item Arguments

None.

=item Example

=back

     $srcname = $lsm->src_name;

=cut

sub src_name {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return shift->{_src}->name;
}

=head2 ::src_path

Returns the vserver-qualified path name
of the source volume.

=over

=item Arguments

None.

=item Example

=back

     $srcpath = $lsm->src_path;

=cut

sub src_path {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    return $self->{_src}->Vserver->name.":".$self->src_name;
}

sub DESTROY {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    NOTE::logcomment("Destroying snapmirror ".$self->description);
    $self->{_destroyed} = 1;
    # This is a no-op for now
    return $self;
}

=head2 ::check_and_add_license 

This is a quick fix solution to the problem where deux_init is not adding the licenses for snapmirror on the filers .
This routine will be removed do not use this for normal execution
=over

=item Arguments

B<force> => I<string>

I<Optional.>
=item Examples

=back

=cut

sub check_and_add_license {
   $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

   $Trace->entry();
   NOTE::logcomment("Adding snapmirror license");
   eval {
      # $self->{_src}->server->system_license_add('license-code'=>"FEDPEKPJYBFDAA");	# leave old code here
      # $self->{_src}->server->system_license_add('license-code'=>"RYAEFKPJYBFDAA");	# this is pre V2 hardcoding
      $self->src->Cluster->license('SnapMirror_LS',1);					# see BURT542699
   };
   croak "$@" if $@;
   $Trace->exit(msg=>"<<exit"); 
}

=head2 B<Object Identification>

=head2 ::description

Returns a description of the LSMirror relationship
including a tag for the source.

=over

=item Arguments

None.

=item Example

=back

     $desc = $lsm->description;

=cut

sub description {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    return $self->src_path . ($self->src->tag ? " (".$self->src->tag.") " : "");
}

=head2 ::src

Returns the object representing the current source.

=over

=item Arguments

I<volume object>>

I<Optional.>
If present, replaces the current source object.

=item Examples

=back

     $src = $lsm->src;

=cut

sub src {
    my ($self,$newsrc) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    if ( defined $newsrc ) {
        my $oldsrc = $self->{_src};
        $self->{_src} = $newsrc;
        return $oldsrc;
    } else {
        return $self->{_src};
    }
}
