#
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/GX/Port.pm#1 $
#
## @summary  GNOTE GX port module
## @author   tmichael@netapp.com
## @status   Private
## @pod      here
package NOTE::GX::Port;
our @ISA = qw(NOTE::GX);

sub new {
    my ($class,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = {};

    $self->{_debug}    = $args{debug};
    $self->{_owner}    = $$;

    $self->{_container} = $args{container};
    $self->{_name}      = $args{name};

    bless $self, $class;

    return $self;
}

sub attach {
    my ($class,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = $class->new( %args );

    $self->status; # make sure this port exists

    return $self;
}

sub container {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    shift->{_container}
}

sub server {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    shift->{_container}->server
}

sub name {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return shift->{_name}
}

sub status {
    my ($self, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $r = $self->server->network_port_show(
	node => $self->container->name,
	port   => $self->name,
	%args,
    )->[0];
    # Fix up the keys.. don't need this '-admin' stuff.
    $r->{'autonegotiate'} = delete $r->{'autonegotiate-admin'};
    $r->{'duplex'}        = delete $r->{'duplex-admin'};
    $r->{'speed'}         = delete $r->{'speed-admin'};
    return $r;
}

=head2 ::modify

Modify the properties of a network port.

=over

=item Arguments

B<role> => I<cluster|data|mgmt|intercluster>

Role.

B<MTU> => I<integer>

MTU.

B<autonegotiate> => I<true|false>

Auto-negotiate.

B<duplex> => I<half|full>

Duplex mode.

B<speed>  => I<auto|10|100|1000|10000>

Speed.

=item Example

=back

     $node->Port( name => 'e1b' )->modify(
         speed => '100',
     );

=cut

sub modify {
    my ($self, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $self->server->network_port_modify(
	server => $self->container->name,
	port   => $self->name,
	%args,
    );
}

sub DESTROY {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    # Nothing to do
}

1;
