# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/GX/San/Lun.pm#1 $

## @summary  NOTE Volume/Qtree extended san methods
## @status   Public
## @pod      here
## @author   dpg-qa-automation@netapp.com

package NOTE::GX::San::Lun;

use strict;
use warnings;
no warnings 'redefine';

use Carp qw(croak confess);
use GNOTE;

use San::SanMgr;
use San::San;		    #for SIO
use San::SanClient::SanIo;  #for old sio
use IOMan;
use Data::Dumper;
use MIME::Base64;

# I hate to define global variables, but san manager will fail if they are not set.
use vars qw( $DEV_TYPE $MP_TYPE $CONFIG_ISCSI_SUBNET_MATCHING $CONFIG_ISCSI_DISCOVERY_PRECLEANUP $CONFIG_ISCSI_WIN_MPIO );

BEGIN {
    my $cdefs = Tharn::param("CDEFS_DIRECTORY", -default, undef);	    # propagate CDEFS_DIRECTORY parameter into
    if( $cdefs ) {							    # CDEF_PATH if exists. CDEF_PATH is used by
	my $res = Rslt->new( type=>'PARAM', msg=>[ 'CDEF_PATH='.$cdefs ] ); # San::SanMgr, San::San, San::SanClient::SanIo
	Tharn::logresult($res);						    # and CDEFS_DIRECTORY is used by rest of the
	$ENV{CDEF_PATH} = $cdefs;					    # NATE world.
    }

    Tharn::param('DEV_TYPE', '-default', "raw");			    # parametrs for SAN client
    Tharn::param('MP_TYPE', '-default', "none");			    # library only.
    Tharn::param('CONFIG_ISCSI_SUBNET_MATCHING', '-default', "0");	    # really not sure if we have
    Tharn::param('CONFIG_ISCSI_DISCOVERY_PRECLEANUP', '-default', "0");	    # to keep these variable
    Tharn::param('CONFIG_ISCSI_WIN_MPIO', '-default', "0");		    # here all the time. Contact korry@netapp.com for info
};

# private... check object and return hash like { volume=>'whatever', vserver => 'whatever, qtree=>'whatever' };
# according to what we are, like GNOTE::GX::Qtree or GNOTE::Qtree or GNOTE::GX::Volume or GNOTE::Volume.
sub _vs_vl_qt {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my %args;
    $args{volume}   = ref($self) =~ /Qtree/ ? $self->Volume->name : $self->name;
    $args{qtree}    = $self->name if ref($self) =~ /Qtree/;
    $args{vserver}  = $self->Vserver->name;
    return %args;
}

# private... returns client type in format acceptable by filer's SAN commands
# arguments: Uses $args{client}, if its not set, then checks environment for
# CLIENT variable, and falls to 'localhost' if nothing was set
# Calls NATE to pull host type and then uses RegEx to convert it to somthing
# suitable for SAN. If its impossible to convert, then undef is returned
sub _get_san_client_type {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $hostname = $args{client} || Tharn::param('CLIENT') || 'localhost';
    if( Tharn::host($hostname)->hosttype() =~ /^(?:unix-)?(vmware|hyper_v|windows_2008|windows_gpt|windows|linux|xen|solaris|solaris_efi|hpux|aix|netware|openvms)-?/ ) {
	return $1;
    }
    return undef;
}

=head2 ::lun_create

Creates luns on given volume and returns array of hashes containing its attributes. This method also checks if iscsi service is running or not
and attempts to start it if service is not available.

=over

=item Arguments

B<size>	  => I<string> Line size. Format include digits followed by type suffix. Warning, windows luns are always larger then specified by 4 - 6 Mb.
Default size is 40Mb. If size is not given method will check CLI for LUN_SIZE and will take it. If size is missing and LUN_SIZE is missing then default
of 40Mb is used.

B<client> => I<string> Client's host name, used for B<ostype> autodetect. Optional parameter. CLI or environment variable CLIENT can be used as well
B<ostype> => I<string> One of solaris,windows,windows_gpt,windows_lhs,hpux,aix,linux,netware,vmware or openvms. Default is taken from client, and if
attempt to get if from client host fails then default is set to 'windows'

B<lun>    => I<string> Lun name. If not given then unique name is generated

B<count>  => I<number> Number of luns to create. Default is 1. If count is greater then 1 and lun name is also given, then
names are generated using following format: sprintf "%s\_%03d", $name, $count. If only count is given then all lun names
are uniqly generated. You need this argument for builk lun create only.

B<igroup> => I<string> Optional igroup name. If igroup is given then lun will be added to igroup. If igroup name is given but not
created then igroup will be created first.

B<space-resv> => I<string> Pass through as is. Default is filer's default

B<state> => I<string> online/offline. Default is filer's default

=item Examples

=back

Using volumes:

    my $util = NOTE::SnapMirrorUtil->new();
    my ($src) = $util->create_volume(tag=>'src');
    my $luns = $src->lun_create(size=>'60MB',ostype=>'windows');  # output is reference to an array of hashes.

Create tree luns, create 3 anonymous igroups and map luns to such igroups

    my $util = NOTE::SnapMirrorUtil->new();
    my $vol  = $util->create_volume(tag=>'src')
    my $luns = $vol->lun_create( count=>3, igroup=>1 );
    my @lunnames = map { $_->{lun} } @$luns;


Using qtrees:

    my $ut = NOTE::SnapMirrorUtil->new();		    ## @step Process CLI parameters, see lib/GNOTE/GX/SnapMirrorUtil.pm
    my $vl = $ut->create_volume(tag=>'src');		    ## @step Create single RW volume of default size. Vol name is autogenerated
    my $qt = NOTE::GX::Qtree->create( Volume => $vl );	    ## @step Create Qtree on same volume. Qtree name is autogenerated
    my ($lun) = $qt->lun_create(igroup=>1, size =>'5g');    ## @step Create igroup with autogenerated name, create lun with autogenerated name, map lun to igroup
							    # lun is returned as a hash ref, not an array ref because of "my ($lun) =" instead of "my $lun = "

here is what's inside the $lun:

 # $lun = {
 #          'device-text-id' => '-',
 #          'block-size' => '512',
 #          'creation-timestamp' => '"11/16/2010 09:16:11"',
 #          'device-binary-id' => '-',
 #          'uuid' => '777ad3ba-f337-11df-905d-00a098123986',
 #          'state' => 'online',
 #          'size' => '49351680',
 #          'device-legacy-id' => '-',
 #          'ostype' => 'windows',
 #          'lun' => 'qt23982_area_replying',
 #          'space-allocation' => 'disable',
 #          'qtree' => 'qt23982_Lucy_assuming',
 #          'vdiskId' => '800004190000000000000000000018721b997534',
 #          'size-used' => '0',
 #          'space-reserve-honored' => 'true',
 #          'extent-size' => '0',
 #          'volume' => 'vl23982_sock_Elsevier',
 #          'vserver' => 'vs0',
 #          'suffix-size' => '0',
 #          'serial' => '2Fgsz+9IXOvF',
 #          'path' => '/vol/vl23982_sock_Elsevier/qt23982_Lucy_assuming/qt23982_area_replying',
 #          'prefix-size' => '32256',
 #          'space-reserve' => 'enable',
 #          'comment' => '""',
 #          'read-only' => 'false',
 #          'mapped' => 'mapped'
 #        };

Out of the box working example with qtrees:

      1 use strict;
      2 use Data::Dumper;
      3 use GNOTE;
      4 eval {
      5     my $ut = NOTE::SnapMirrorUtil->new();              ## @step Process CLI and connect to cluster
      6     my $vl = $ut->create_volume(tag=>'src');           ## @step Create volume based on CLI parameters
      7     my $qt = NOTE::GX::Qtree->create( Volume => $vl ); ## @step Create Qtree
      8     my $luns = $qt->lun_create( count=>3, igroup=>1);  ## @step Set up iSCSI. Create 3 luns and map to 3 new igroups
      9     print "All luns:".Dumper($luns);
     10
     11     my @lunnames = map{ $_->{lun} } @$luns;            # extract names from reference of array of hashes and return as
     12     print "Lun names only: ".join(' ',@lunnames)."\n"; # array of strings. Print them out.
     13
     14     my @lunnames = $qt->lun_show_names;                # easier way to do same printing
     15     print Dumper(\@lunnames);
     16 };
     17 logresult('FAIL',$@) if $@;                            ## @step Print errors if found
     18 NOTE::teardown;                                        ## @step Clean up

Out of the box working example with snapmirror:

 sims4:~$ ntest lun_compare.thpl FILER_SRC=dpg-v-spectre9a FILER_DST=dpg-v-spectre9b VOL_SIZE=50g LUN_SIZE=1g
      1 use strict;
      2 use Data::Dumper;
      3 use GNOTE;
      4 eval {
      5     my $ut = NOTE::SnapMirrorUtil->new();                                            ## @step process CLI parameters and connect to cluster
      6     my $sm = NOTE::GX::SnapMirror->initialize(                                       ## @step Create source and destination volumes
      7                                                 src=>$ut->create_volume(tag=>'src'), ##       initialize snapmirror and perform initial
      8                                                 dst=>$ut->create_volume(tag=>'dst')  ##       data transfer
      9                                              );
     10
     11     $sm->src->lun_create( count=>2, igroup=>1);                                      ## @step create two igroups. Create two luns on source and map them
     12     $sm->update;                                                                     ## @step Update snapmirror, which should move luns to destination
     13
     14     if( my $mismatch = $sm->lun_compare ) {                                          ## @step copmare all lun properties
     15         die "Lun compare failed:\n".Dumper($mismatch);                               ##       between source and destination
     16     }
     17 };
     18 logresult('FAIL',$@) if $@;                                                          # check for errors
     19 NOTE::teardown;                                                                      # clean up

=cut

#   name    count   action
#    -	      -	     generate unique name
#    X	      -	     use name as is
#    -	      X	     generate unique names
#    X	      X	     use name as prefix and count index as suffix

sub lun_create {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $results = undef;	# return reference to array of luns

    my $count = $args{count} ? delete $args{count} : 1;
    my $name  = delete $args{lun};

    my $protocol = $args{service} ? delete $args{service} : 'iscsi';	# this code needs modification for FCP
    $self->Vserver->iscsi_start if $protocol =~ /iscsi/;		# and its called service, not protocol

    for( 1 .. $count ) {
	$args{lun} = $name;
	$args{lun} = sprintf "%s\_%03d", $name, $_ if $name && $count > 1;
	push @$results, $self->_lun_create(%args);
    }
    return $results;
}

sub _lun_create {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $args{ostype} ||= $self->_get_san_client_type(%args) || 'windows';  # Set ostype from client if not given, if client is unknown
    delete $args{client};						# then set it to windows and then remove client parameter

    $args{size} ||= Tharn::param('LUN_SIZE') ? Tharn::param('LUN_SIZE') : '40MB'; # lun size of 40Mb is a minimum requirements by microsoft
    $args{lun}  ||= $self->newName("lun_");					  # generate new lun name if lun name is not given at all

    my $mapto     = exists $args{igroup} ? delete $args{igroup} || 1 : undef;	  # make sure we still can call it like lun_create( igroup => undef )

    eval { $self->server->lun_create( ( $self->_vs_vl_qt, %args) ) };
    confess $@ if( $@ && $@ !~ /(There are no entries matching your query.|Verification failure for field 'size')/ );

    delete $args{size};	# size is deleted before prior to verification because it may be larger due to OS type

    my $output = ($self->server->lun_show( ($self->_vs_vl_qt, %args) ))[0];  #returning a hash instead of an array only one lun will be created therefore taking the first index
    confess "Lun create has failed, there is no output from lun show" unless($output);
    if( $mapto ) {												   # IF lun is created and $mapto has any value
	$self->lun_map( lun=>$$output{lun}, ostype=>$$output{'ostype'}, igroup => $mapto=~/\D/ ? $mapto : undef ); # THEN { add lun to igroup $mapto or generate
	$output = ($self->server->lun_show( ($self->_vs_vl_qt, %args) ))[0];					   # new igroup first if $mapto !~ /[a-z]/ }
    }														   # and reload lun_show to reflect a fact that lun is mapped
    NOTE::logcomment("LUN: $args{lun} was created successfully! \n");

    return $output;
}

=head2 ::lun_change_serial

Sends "lun modify" command to change the serial number of a provided lun,
It offlines the lun and then modifies the serial number. It then onlines the lun.
It returns the lun show of the provided lun.
New Serial number is composed of 12 alphanumeric type characters.
It expects lun to be online.

=over

=item Arguments

B<lun> I<Required> Lun name.

B<serial> I<Optional> New serial number. String of 12 characters. If non is given then
number will be generated using timestamp, random number and pid.

=item Examples

=back

    $vol->lun_change_serial(lun=> '01145_Cain_chiseled' );

    Following is the output returned by lun_change_serial:

    $VAR1 =
       {
       'device-text-id' => '-',
       'block-size' => '512',
       'isMapped' => 'true',
       'device-binary-id' => '-',
       'uuid' => '075f6a4e-c69d-11df-b340-00a098123986',
       'size' => '22553717760',
       'device-legacy-id' => '-',
       'ostype' => 'windows',
       'lun' => '09295_away_famously',
       'space-resv-honored' => 'true',
       'qtree' => '""',
       'vdiskId' => '800004110000000000000000000000651a8603bb',
       'size-used' => '0',
       'extent-size' => '0',
       'volume' => 'vl09295_sway_britches',
       'vserver' => 'vs0',
       'suffix-size' => '0',
       'serial' => '2Fgsz+92Jr8l',
       'path' => '/vol/vl09295_sway_britches/09295_away_famously',
       'space-resv' => 'enable',
       'prefix-size' => '32256',
       'space-alloc' => 'disable',
       'comment' => '""',
       'read-only' => 'false',
       'isOnline' => 'true'
     }

Working example:

      1 use strict;
      2 use Data::Dumper;
      3 use GNOTE;
      4 eval {
      5     my $ut = NOTE::SnapMirrorUtil->new();               ## @step process CLI
      6     my $vl = $ut->create_volume(tag=>'src');            ## @step create RW volume
      7     my $ln = ($vl->lun_create())->[0];                  ## @step create one lun and return lun show as hash
      8
      9     my $sn = $vl->lun_change_serial(lun=>$ln->{lun});   ## @step change serial and return new lun show as hash
     10
     11     print "Old lun_show:\n".Dumper($ln);                ## @step Print old lun show hash
     12     print "New lun_show:\n".Dumper($sn);                ## @step print lun show hash with new serial number
     13 };
     14 logresult('FAIL',$@) if $@;                             ## @step Report errors if any
     15 NOTE::teardown;                                         ## @step Clean up after yourself

Here is what you should see in the log from script above:

   2458 20101119 152128 [0001_lun_serial] ........................................
   2459 20101119 152128 [0001_lun_serial] . Comment: %%
   2460 20101119 152128 [0001_lun_serial] .     ::GX::San::Lun::lun_change_serial [290]
   2461 20101119 152128 [0001_lun_serial] .         Changing old serial number [2Fgsz+9KQsnn] to new one [NTAuNzY0NTgz] for lun lun_09671_days_dreading
   ....
   2586 20101119 152133 [0001_lun_serial] .         Old lun_show:
   2587 20101119 152133 [0001_lun_serial] .         $VAR1 = {
   2588 20101119 152133 [0001_lun_serial] .                   'device-text-id' => '-',
   2589 20101119 152133 [0001_lun_serial] .                   'block-size' => '512',
   2590 20101119 152133 [0001_lun_serial] .                   'creation-timestamp' => '"11/19/2010 15:21:21"',
   2591 20101119 152133 [0001_lun_serial] .                   'device-binary-id' => '-',
   2592 20101119 152133 [0001_lun_serial] .                   'uuid' => '082a76dc-f5c6-11df-b22a-00a098123986',
   2593 20101119 152133 [0001_lun_serial] .                   'state' => 'online',
   2594 20101119 152133 [0001_lun_serial] .                   'size' => '49351680',
   2595 20101119 152133 [0001_lun_serial] .                   'device-legacy-id' => '-',
   2596 20101119 152133 [0001_lun_serial] .                   'ostype' => 'windows',
   2597 20101119 152133 [0001_lun_serial] .                   'lun' => 'lun_09671_days_dreading',
   2598 20101119 152133 [0001_lun_serial] .                   'space-allocation' => 'disable',
   2599 20101119 152133 [0001_lun_serial] .                   'qtree' => '""',
   2600 20101119 152133 [0001_lun_serial] .                   'vdiskId' => '8000041c0000000000000000000000601a19c80f',
   2601 20101119 152133 [0001_lun_serial] .                   'size-used' => '0',
   2602 20101119 152133 [0001_lun_serial] .                   'space-reserve-honored' => 'true',
   2603 20101119 152133 [0001_lun_serial] .                   'extent-size' => '0',
   2604 20101119 152133 [0001_lun_serial] .                   'volume' => 'vl09671_sale_selector',
   2605 20101119 152133 [0001_lun_serial] .                   'vserver' => 'vs0',
   2606 20101119 152133 [0001_lun_serial] .                   'suffix-size' => '0',
   2607 20101119 152133 [0001_lun_serial] .                   'serial' => '2Fgsz+9KQsnn',
   2608 20101119 152133 [0001_lun_serial] .                   'path' => '/vol/vl09671_sale_selector/lun_09671_days_dreading',
   2609 20101119 152133 [0001_lun_serial] .                   'prefix-size' => '32256',
   2610 20101119 152133 [0001_lun_serial] .                   'space-reserve' => 'enable',
   2611 20101119 152133 [0001_lun_serial] .                   'comment' => '""',
   2612 20101119 152133 [0001_lun_serial] .                   'read-only' => 'false',
   2613 20101119 152133 [0001_lun_serial] .                   'mapped' => 'unmapped'
   2614 20101119 152133 [0001_lun_serial] .                 };
   ....
   2620 20101119 152133 [0001_lun_serial] .         New lun_show:
   2621 20101119 152133 [0001_lun_serial] .         $VAR1 = {
   2622 20101119 152133 [0001_lun_serial] .                   'device-text-id' => '-',
   2623 20101119 152133 [0001_lun_serial] .                   'block-size' => '512',
   2624 20101119 152133 [0001_lun_serial] .                   'creation-timestamp' => '"11/19/2010 15:21:21"',
   2625 20101119 152133 [0001_lun_serial] .                   'device-binary-id' => '-',
   2626 20101119 152133 [0001_lun_serial] .                   'uuid' => '082a76dc-f5c6-11df-b22a-00a098123986',
   2627 20101119 152133 [0001_lun_serial] .                   'size' => '49351680',
   2628 20101119 152133 [0001_lun_serial] .                   'state' => 'online',
   2629 20101119 152133 [0001_lun_serial] .                   'lun' => 'lun_09671_days_dreading',
   2630 20101119 152133 [0001_lun_serial] .                   'ostype' => 'windows',
   2631 20101119 152133 [0001_lun_serial] .                   'device-legacy-id' => '-',
   2632 20101119 152133 [0001_lun_serial] .                   'qtree' => '""',
   2633 20101119 152133 [0001_lun_serial] .                   'space-allocation' => 'disable',
   2634 20101119 152133 [0001_lun_serial] .                   'vdiskId' => '8000041c0000000000000000000000601a19c80f',
   2635 20101119 152133 [0001_lun_serial] .                   'size-used' => '0',
   2636 20101119 152133 [0001_lun_serial] .                   'space-reserve-honored' => 'true',
   2637 20101119 152133 [0001_lun_serial] .                   'extent-size' => '0',
   2638 20101119 152133 [0001_lun_serial] .                   'volume' => 'vl09671_sale_selector',
   2639 20101119 152133 [0001_lun_serial] .                   'serial' => 'NTAuNzY0NTgz',
   2640 20101119 152133 [0001_lun_serial] .                   'suffix-size' => '0',
   2641 20101119 152133 [0001_lun_serial] .                   'vserver' => 'vs0',
   2642 20101119 152133 [0001_lun_serial] .                   'path' => '/vol/vl09671_sale_selector/lun_09671_days_dreading',
   2643 20101119 152133 [0001_lun_serial] .                   'prefix-size' => '32256',
   2644 20101119 152133 [0001_lun_serial] .                   'space-reserve' => 'enable',
   2645 20101119 152133 [0001_lun_serial] .                   'comment' => '""',
   2646 20101119 152133 [0001_lun_serial] .                   'mapped' => 'unmapped',
   2647 20101119 152133 [0001_lun_serial] .                   'read-only' => 'false'
   2648 20101119 152133 [0001_lun_serial] .                 };

=cut

sub lun_change_serial {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess( "Missing lun name. Argument 'lun' must exist" ) unless $args{lun} =~ /\w/;
    ( my $newsn = $args{serial} ? delete $args{serial} : encode_base64(time().rand(1).$$) ) =~ s/^\S{12}(\S{12}).*/$1/s;# take next 12 characters from time+random+pid
    my $oldsn = ($self->lun_offline(%args))->{'serial'};								# take lun offline and note old serial as well
    NOTE::logcomment("Changing old serial number [$oldsn] to new one [$newsn] for lun $args{lun}");			# inform world how dare we are to change serial
    my $chksn = ($self->lun_modify(%args,serial=>$newsn))->[0]->{'serial'};						# change serial and note actual serial number
    confess "Serial number was not changed from [$oldsn] to [$newsn], actual returned is [$chksn]" if $newsn ne $chksn;	# confess if new serial is not changed at all
    return $self->lun_online(%args);											# bring lun online and return lun_show()
}


#This is an internal function
###############################################################################
#
#   Sub Name:  _mk_lun_iscsi_xml
#
#   Description:   Generate the required xml content to be passed to SanMgr
#                  module, for configuring and mapping the luns to the clinet.
#                  The generated xml content is written to the provided file
#                  and is returned.
#
#   Arguments:      NAME      IN/OUT  TYPE      DESCRIPTION
#                   client    IN      scalar    clinet host name
#                   client_config IN  scalar    0/1. 1 means configuration to be
#                                               done for all luns. 0 means, only
#                                               for this lun.
#                   dev_type  IN      scalar    Dev type on client, raw or fs
#                                               Default: CLI's DEV_TYPE, if not
#                                               provided 'raw'
#                   mp_type   IN      scalar    Multipath type. Default: CLI's
#                                               MP_TYPE , if not provided 'none'
#                   lun       IN      arrayref  names of luns created in volume
#                   file      IN      scalar    file to which xml content to be
#                                               wriiten, if provided.
#                                               Default: xml content not written
#                                               any where.
#                   qtree_luns_href IN  hashref reference to hash of luns created
#                                               in qtrees of the volume. see
#                                               POD of lun_map_to_client, for more
#                                               details.
#
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success: returns xml content
#                   Failure: confesses if error in generating xml content
#
###############################################################################

sub _mk_lun_iscsi_xml {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $args{dev_type} ||= $DEV_TYPE ? $DEV_TYPE : 'raw';			# can not be empty or
    $args{mp_type}  ||= $MP_TYPE ? $MP_TYPE : 'none';           # San manager will fail

    my $vlobj = ref($self) =~ /Qtree/ ? $self->Volume : $self;		# need real volume for XML

    my ($iscsi) = @{$self->Vserver->iscsi_show()};			# first things first
    confess "iSCSI service is not running" unless $iscsi;		# no money no honey

    my $_lns = $self->lun_show;						# use ver private array ref for lun show
    confess "Didn't find any luns on ".$self->name unless $_lns;	# check if we have any luns at all
    my %lnsh = map{ $$_{'lun'}, $_  } @{$_lns};				# create hash of lun shows by name

    #If user do not provide any luns which are created directly in volume
    #or luns created in qtrees of the volume, map all the luns in the volume
    my @luns;
    my @qtree_luns;
    if($args{lun}){
        @luns = ref $args{lun} eq 'ARRAY' ? @{$args{lun}} : ($args{lun});
    }
    else {
        if( $args{'qtree_luns_href'} ) {
            #get the qtree luns into an array
            foreach my $qtree_tmp ( keys %{ $args{'qtree_luns_href'} } ) {
                push @qtree_luns, @{ $args{'qtree_luns_href'}->{$qtree_tmp} };
            }
        }
        else {
            #user did not provide both luns in volume and luns in qtrees
            #map all the luns in the volume
            @luns = keys %lnsh unless $args{lun};
        }
    }

    #check all the luns are present
    my @all_luns = (@luns,@qtree_luns);
    foreach my $lun_tmp (@all_luns) {
        if( !$lnsh{$lun_tmp} ) {
            confess "Requested lun \"$lun_tmp\" was not found on " .
                     $self->name . ' container';
        }
    }

    # pull iSCSI interface information or confess, if no iscsi lif
    my ($lif) = $self->Vserver->get_iscsi_lif();
    confess 'No iscsi LIFs found' unless $lif;
    # split 4-bytes IP address using the first 3 octals to create gateway
    my ($first, $secand, $third, $fourth) = split(/[.]/, $lif->{'address'});
    my $gateway = "$first.$secand.$third.1";

    # try to use igroup from parameters first
    # if not provided then try to find any available from luns itself make sure
    # to check each and every listed lun and then take first available igroup
    my $igroup = delete $args{igroup};
    if(! $igroup ) {
        my %igroups;
        foreach my $name ( @all_luns ) {
            foreach (@{ $self->is_lun_mapped(lun=>$name) } ) {
                $igroups{ $_->{'igroup'} }++ ;
            }
        }
        NOTE::logcomment('igroups hash: ' . Dumper(\%igroups));
        #Get the igroup to which most of the given luns are mapped to
        my @rev_sorted_igrps = reverse sort { $igroups{$a} <=> $igroups{$b} } keys %igroups;
        $igroup = $rev_sorted_igrps[0];

        if(!$igroup){
            confess 'No igroup given or detected!';
        }
    }
    #Assign all luns to same group
    foreach my $lun_tmp ( @all_luns ) {
        $self->lun_map(lun=>$lun_tmp, igroup=>$igroup );
    }

    # and get igroup as complete hash info
    my $igroup_hr;
    ($igroup_hr) = @{$self->is_lun_mapped(lun=>$all_luns[0], igroup=>$igroup)};


    my $lun_name = $luns[0];
    my $vserver_aggr    = $self->Vserver->status()->{'aggregate'};
    my $vl_nodename     = $vlobj->Node->name;
    my $vl_clustername     = $vlobj->Node->Cluster->name;
    my $vl_aggrname     = $vlobj->Aggregate->name;
    my $vl_aggrnumdisks = scalar( $vlobj->Aggregate->_aggr_disks );
    my $vserver_name    = $vlobj->Vserver->name;
    my $vl_name         = $vlobj->name;
    my $vl_size         = $vlobj->size;
    my $igrp_name       = $igroup_hr->{'igroup'};
    my $lif_protocol    = $lif->{'data-protocol'};
    my $lif_name = $lif->{'lif'};
    my $lif_currport    = $lif->{'curr-port'};
    my $lif_currnode    = $lif->{'curr-node'};
    my $lif_addr        = $lif->{'address'};
    my $lif_netmask     = $lif->{'netmask'};
    my $igrp_os         = $igroup_hr->{os} || $igroup_hr->{ostype};
    my $client_name     = $args{client};
    my $client_iotype   = $args{dev_type};
    my $client_mptype   = $args{mp_type};

    #let us print some information to log that might help in debugging
    NOTE::logcomment('igroup hash: ' . Dumper($igroup_hr) . "\nlif hash: " . Dumper($lif));

    my $sanxml;
    $sanxml  = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
    $sanxml .= "<scenario>\n";
    $sanxml .=     "<environment debug_level=\"1\" pnate=\"0\" oem_rebrand=\"netapp\">\n";
    $sanxml .=         "<global_iscsi_options subnet_matching=\"0\" use_mgmt_port=\"1\" discovery_precleanup=\"1\"/>\n";
    $sanxml .=     "</environment>\n";
    $sanxml .=     "<actions cleanup=\"1\" config=\"1\" setup=\"1\" validate=\"0\"/>\n";
    $sanxml .=     "<reuse_policy setup=\"reuse\" cleanup=\"new\"/>\n";
    $sanxml .=     "<clusters>\n";
    # Due to changes incorporated by burt:505603, correct cluster name is 
    # to be provided, along with new attribute "nodes"
    $sanxml .=         "<cluster name=\"$vl_clustername\" " .
                       "nodes=\"$vl_nodename\" " .
                       "precleanup=\"0\" config=\"1\" " .
                       "setup=\"1\" validate=\"0\">\n";
    $sanxml .=             "<aggr name=\"$vl_aggrname\" num_disks=\"$vl_aggrnumdisks\" disk_owner=\"$vl_nodename\" state=\"spare\"/>\n";
    $sanxml .=             "<vserver parallel=\"0\" name=\"$vserver_name\" count=\"1\" " .
                                     "fcp_enabled=\"0\" iscsi_enabled=\"1\" aggr=\"$vserver_aggr\">\n";
    $sanxml .=                 "<lif name=\"$lif_name\" count=\"1\" data-protocol=\"$lif_protocol\" " .
                                    "home-port=\"$lif_currport\" home-node=\"$lif_currnode\" " .
                                    "address=\"$lif_addr\" netmask=\"$lif_netmask\" " .
                                    "gateway=\"$gateway\" role=\"data\">\n";
    $sanxml .=                 "</lif>\n";
    $sanxml .=                 "<vol name=\"$vl_name\" size=\"$vl_size\" aggr=\"$vl_aggrname\">\n";
                               #Add luns created directly in volume to the
                               #volume element of xml content
                               foreach my $lun_tmp (@luns ) {
    $sanxml .=                     "<lun name=\"$lun_tmp\" count=\"1\" size=\"".
                                   $lnsh{$lun_tmp}->{'size'}.'" ' . 'ostype="' .
                                   $lnsh{$lun_tmp}->{'ostype'}."\">\n" .
                                       "<lun_map igroup=\"$igrp_name\"/>\n" .
                                   "</lun>\n";
                               }

                               #Add igroup luns, to the xml content
                               foreach my $qt_tmp ( keys %{ $args{'qtree_luns_href'} } ) {
    $sanxml .=                     "<qtree name=\"$qt_tmp\" count=\"1\">\n";
                                   foreach my $lun_tmp ( @{ $args{'qtree_luns_href'}->{$qt_tmp} } ) {
    $sanxml .=                          "<lun name=\"$lun_tmp\" count=\"1\" size=\"" .
                                        $lnsh{$lun_tmp}->{'size'}.'" ' .
                                        'ostype="' . $lnsh{$lun_tmp}->{'ostype'}."\">\n" .
                                            "<lun_map igroup=\"$igrp_name\"/>\n" .
                                        "</lun>\n";
                                   }
    $sanxml .=                     "</qtree>\n";
                               }

    $sanxml .=                 "</vol>\n";
    $sanxml .=                 "<igroup name=\"$igrp_name\" count=\"1\" protocol=\"$lif_protocol\" ostype=\"$igrp_os\">\n";
    $sanxml .=                     "<initiator name=\"$client_name\"/>\n";
    $sanxml .=                 "</igroup>\n";
    $sanxml .=             "</vserver>\n";
    $sanxml .=         "</cluster>\n";
    $sanxml .=     "</clusters>\n";
    $sanxml .=     "<clients setup=\"1\" config=\"1\">\n";

    #Based on client_config value, the xml content differ
    $sanxml .=         "<client name=\"$client_name\" io_type=\"$client_iotype\" protocol=\"iscsi\" " .
                                "mp_type=\"$client_mptype\" conn_type=\"THARNHOST\" config=\"1\" " .
                                "setup=\"1\" cleanup=\"1\" kill_io_processes=\"0\" validate=\"0\"/>\n" if($args{client_config} == 1);
    $sanxml .=         "<client name=\"$client_name\" io_type=\"$client_iotype\" protocol=\"iscsi\" " .
                                "mp_type=\"$client_mptype\" conn_type=\"THARNHOST\" config=\"0\" " .
                                "setup=\"1\" cleanup=\"1\" kill_io_processes=\"0\" validate=\"0\" lun_path_regex=\"qr\/$lun_name\/\"/>\n" if($args{client_config} == 0);

    $sanxml .=     "</clients>\n";
    $sanxml .= "</scenario>\n";

    NOTE::logcomment("\nCreated XML File and Content: $args{file} :\n$sanxml");

    if( $args{file} ) {
        my $FH;
        open ($FH,'>', $args{file}) || confess "Can not open file $args{file} for writing. $!";
        push @{$self->{_san_xml_files}}, $args{file};
        print $FH $sanxml;
        close $FH || confess "Unable to close: $!";
    }

    return $sanxml;
}

=head2 ::san_manager

Attempt to create SAN managers if not already created for given client.

Returns San::SanMgr object

=over

=item Arguments

B<client> hostname of client

Following CLI defaults are set for SanMgr as suggested by Korry.

    DEV_TYPE => "raw"
    MP_TYPE => "none"
    CONFIG_ISCSI_SUBNET_MATCHING => "0"
    CONFIG_ISCSI_DISCOVERY_PRECLEANUP => "0"
    CONFIG_ISCSI_WIN_MPIO => "0";

=back

=cut

sub san_manager {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "Missing client name" unless $args{client};
    if( ! $self->{_san_manager}->{ $args{client} } ) {
	$self->{_san_manager}->{ $args{client} } = San::SanMgr->new(-file => $args{file});
	confess $self->{_san_manager}->{ $args{client} }->errmsg() if $self->{_san_manager}->{ $args{client} }->error();
    }
    return $self->{_san_manager}->{ $args{client} };
}

###########################################################################
#Currently takes an array of luns and the client name as a required param
###########################################################################
#data_pattern is optional default is set to 'a'
#pass_thru is optional default is set to -verify
#num_thread is optional default is set to 10
#run_time is optional default is set to 60 secs
#last_byte is optional default is set to 1048576 * 50 or filesize
#first_byte is optional default is set to 8192
#block_size is optional default set to 4096
#random_pct is optional default set to 0
#read_pct is optional default set to 50
#wait_flag is optional default set to 0
#sio_subtest_id is optional default set to sio_write
#io_tool is options default set to sio
#filesize is optional default is set to 1048576 * 50
#client_io_dev is required i.e. raw/fs device name e.g.  -client_io_dev  => '\\\\.\\PhysicalDrive1',
#client name is required
#protocol is optional

sub SanIo {
    my ($self, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "Missing client name or client object" unless $args{client};
    confess "Missing lun names" unless $args{luns};
    $args{protocol}    ||= 'iscsi';
    $args{io_tool}     ||= 'sio';
    $args{sio_subtest_id} ||= 'sio_write';
    $args{wait_flag}   ||= 0;
    $args{read_pct}    ||= 50;
    $args{random_pct}  ||= 50;
    $args{block_size}  ||= 4096;
    $args{first_byte}  ||= 8192;
    $args{run_time}    ||= 60;
    $args{num_threads} ||= 10;
    $args{pass_thru} ||= "-verify";
    $args{data_pattern} ||= 'a';


    if(!(ref($args{client}))) {   #if pass obj thats fine
	$self->{_sanclient}->{client_name} = $args{client};
	$args{client} = new SanClient(-host => $args{client},
			      	     -protocol     => $args{protocol},
	      			     -conn_type    => "THARNHOST");

	$self->{_sanclient}->{client_obj} = $args{client};  #setting client object into vol object
    }

    my @luns = @{$args{luns}};
    print Dumper(@luns);

    my @rawNames;
    my $min_lun_size = -1;
    my $lun_show = $self->lun_show;
    foreach my $lun (@luns) {
	foreach (@{$lun_show}) {
	    if ($_->{lun} eq $lun) {
		NOTE::logcomment("lun = $_->{lun}\nraw = $_->{rawName}\n");
		push @rawNames , $_->{rawName};
		if (($min_lun_size == -1) || ($min_lun_size > $_->{size})) {
		    $min_lun_size = $_->{size};
		}
	    }
	}
    }
    $min_lun_size -= 1048576 if ($min_lun_size > 1048576);
    $args{last_byte} ||= $min_lun_size;
    NOTE::logcomment("sio devices last_byte = $args{last_byte}\n");

    my $client_io_dev .= join " ", @rawNames;
    NOTE::logcomment("devs are $client_io_dev\n");

    $args{client_io_dev} = $client_io_dev;

    NOTE::logcomment("before running IO dumping vol sanclient obj\n");

    my $sanIoObj= new SanIo(
			     -io_tool        => $args{io_tool},
			     -sio_subtest_id => $args{sio_subtest_id},
			     -san_client_obj => $args{client},		# Client to run I/O
			     -wait_flag      => $args{wait_flag},	# wait for io to complete
			     -client_io_dev  => $client_io_dev,		# e.g \\\\.\\PhysicalDrive1  client_io_ dev  for raw device
			     -read_pct       => $args{read_pct},	# read_pct
			     -random_pct     => $args{random_pct},	# random_pct
			     -block_size     => $args{block_size},	# block_size $CLIENT_IO_BLOCK_SIZES      block_size
			     -first_byte     => $args{first_byte},	#CLIENT_IO_DEV_OFFSET_BEGIN first_byte
			     -last_byte      => $args{last_byte},	# last_byte
			     -run_time       => $args{run_time},	# time in seconds
			     -num_threads    => $args{num_threads},	#$CLIENT_IO_NUM_THREADS  num_threads
			     -pass_thru      => $args{pass_thru}	#-verify  sio_switches  -pass_thru=> -fillonce -fixedfill -$pattern
			   );

    NOTE::logcomment("--writing to $client_io_dev for $args{run_time} \n");
    $sanIoObj->san_io_run();

    if (! $sanIoObj->san_io_is_alive()) {
	# let dump the object
	print Data::Dumper->Dump([$sanIoObj], [qw(sanIoObj)]);
	confess "Fail: write IO failed";
    }

    NOTE::logcomment("wait for sio to complete");

    my $elapsed_time = 0;

    while($sanIoObj->san_io_is_alive()) {
    	sleep(5);
	$elapsed_time += 5;
	NOTE::logcomment("wait for sio to complete: elapsed time $elapsed_time");
    }

    my ($status,@subtest_result);
    if ($sanIoObj->{io_tool_obj}->{sub_test_obj}) {
	if ($status = $sanIoObj->{io_tool_obj}->{sub_test_obj}->exit_status()) {
	    # check the result
	    @subtest_result = $sanIoObj->{io_tool_obj}->{sub_test_obj}->results();
	    print Dumper(@subtest_result);
	    confess " Fail";
	}
    }
    NOTE::logcomment("Write completed successfully\n");
}

sub _lun_names_to_lunshow {							# private function which takes $args{lun}
    my ($self,%args) = @_;							# containg lun names as string or array of
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my @lunnames = ref $args{lun} eq 'ARRAY' ? @{$args{lun}} : ($args{lun});	# strings and returns "lun_show" output as
    if( @lunnames ) {								# array of hashes. if $args{lun} is not
	my %lunshow = map{ ($$_{lun}, $_ ) } @{$self->lun_show()};		# given then complete lun_show output is
	return map{ $lunshow{$_} } @lunnames;					# returned as is.
    } else {
	return $self->lun_show();
    }
}

=head2 ::SanIOman

Wrapper around san's IOMan call. Returns IOMan object.

=over

=item Arguments

B<client> I<required> hostname of client

B<client_automap> I<optional> Boolean parameter. Call lun_map_to_client prior to IOMan if parameter is given

B<lun> I<optional> Lun name(s) to map. Can be an array of strings. If no luns given then all luns are taken

See http://wikid.netapp.com/w/Scalable_SAN_Test_Engineering/Automation/IOMan/user_guide#Create_io_objs for other arguments.

=item Example

     32 my $vol = NOTE::Volume->create( 'mirror-type' => "RW", Node=>$FILER, Vserver => $VSERVER, size=>$VOL_SIZE, 'snapshot-policy' => 'none'  );
     33
     34 my $luns = $vol->lun_create( count=>$LUN_COUNT,size=>$LUN_SIZE, ostype=>Tharn::host($CLIENT)->hosttype, igroup => undef );
     35
     36 my @lunnms = $vol->lun_show_names();
     37 print Dumper(\@lunnms);
     38
     39 $vol->lun_map_to_client( client=>$CLIENT, lun =>\@lunnms );
     40
     41 my @luns = $vol->lun_show();
     42
     43 print "Hello loonz:\n".Dumper(\@luns);
     44
     45 my $ioman = $vol->SanIOman(
     46                      client         => $CLIENT,
     47                      lun            => \@lunnms,
     48                      io_tool        => "sio",
     49                      limit          => "50m",
     50                      iotype         => "random",
     51                      runtime        => 180,
     52                      read_perc      => 50,
     53                      bs             => "4k"
     54                  );
     55
     56 $ioman->wait_finish();

=back

=cut

sub SanIOman {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "Missing client name or client object" unless $args{client};							# bail out if client name is not given

    $self->lun_map_to_client( client=>$args{client}, lun=> $args{lun} ) if( delete $args{client_automap} );			# if automap is requested, then

    my @lunshow = $self->_lun_names_to_lunshow(%args);										# need to pull lun show for selected list of luns only
    confess "No suitable luns found. Requested luns do not exist." unless @lunshow;

    my @devices = grep { defined $_ } map { $$_{'rawName'} } @lunshow;								# pull device names from client (raw only for now)
    confess "No suitable devices found on client.\n".Dumper(\@lunshow) unless @devices;

    $args{'-devices'} = \@devices;												# save devices in %args and
    delete $args{lun};														# remove lun argument kz its not needed for IOMan
    foreach(keys %args) { $args{'-'.$_} = delete $args{$_} unless /^\-/ }							# make sure we got '-' set for every key

    my $IOMan_obj = new IOMan();												# create IOMan object

    #Keep track of all the IOMan objs, to clean up in the end
    push @{$self->{_ioman_objs_array_ref} }, $IOMan_obj;

    my @io_records = $IOMan_obj->create_io_objs(%args);												# prepare client for traffic
    confess $IOMan_obj->errmsg() if $IOMan_obj->error();
    $IOMan_obj->start(-io_records=>\@io_records);													# start traffic on background and
    confess $IOMan_obj->errmsg() if $IOMan_obj->error();									# check for errors

    return $IOMan_obj;														# and return IOMan object as IS
}

=head2 ::lun_map_to_client

Map lun on client and return array of devices objects from
San::SanMgr->get_devices.
Calls GNOTE::GX::San::Lun::_mk_lun_iscsi_xml prior to mapping.
Returns array of lun show info with client's lun names.

=over

=item Arguments

B<client> I<required>, hostname of client. If client name is not given then CLI is being checked for CLIENT argument.
method will exit with error if no client found.

I<Optional Arguments>

B<lun> names of Luns(created directly in volume) to map. Can be an array of strings.

B<qt_luns_href> hash reference to the luns that are part of qtree

The hash would be like this:
$qt_luns_href = {$qt1_name => [$lun1_name, $lun2_name],
                 $qt2_name => [$lun3_name, $lun4_name],
                }

B<dev_type> Dev type on client, raw or fs (this needs correction from SAN team),
default is CLI's DEV_TYPE and if CLI is empty then raw device is assumed.

B<mp_type>  Multipath type. Default is falling to CLI's MP_TYPE,
and if CLI is empty then multi-pathing is set to 'none'

B<file> => I<optional> Filename to write data to.
Default is not to write anywhere. Will attempt to write file if given.

=over

=item B<client_config> => I<0|1>

Default value: 1
If 1, configuraion(Setting up iSCSI sessions) is done for all luns(including previously created luns)
      on client side.
If 0, configuration is done only for the present lun.
If it is choosen to configure all the luns (including previously created luns),
the client configuration might fail, if there is an IO running on previous configured luns.

=back

=item Examples

=back

     #To map a lun to client
     my @vol_luns = ($lun1_name, $lun2_name);
     my @devs = $src_vol->lun_map_to_client( client => $CLIENT,
                                            dev_type => $DEV_TYPE,
                                            mp_type => $MP_TYPE,
                                            igroup=>$IGROUP_SRC,
                                            lun=>\@vol_luns,
                                          );

     #To map 2 luns created in qtree named as qtree1,
     #2 other luns created another qtree qtree2
     # and 1 lun created directly in the volume
     my @vol_luns = ($lun1_name);
     my $qt_luns_href = {'qtree1' => [$lun2_name, $lun3_name],
                         'qtree2' => [$lun4_name, $lun5_name],
                        };
     my @devs = $src_vol->lun_map_to_client( client => $CLIENT,
                                            dev_type => $DEV_TYPE,
                                            mp_type => $MP_TYPE,
                                            igroup=>$IGROUP_SRC,
                                            lun=>\@vol_luns,
                                            'qtree_luns_href' => $qt_luns_href,
                                          );


=cut

sub lun_map_to_client {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    if (defined $args{client_config}) {
        if($args{client_config} !~ /^[0-1]$/) {
           $args{client_config} = 1;
        }
    }
    else {
        $args{client_config} = 1;
    }

    unless( $args{client} ) {												# check if we have client or
	$args{client} = Tharn::param('CLIENT', -default, undef );               # if we can pull it from CLI, and
	confess 'Client name is not provided' unless $args{client};             # confess if none available
    }

    $args{file} ||=  '/tmp/' . NOTE::newName('iscsi_sanmngr_dpg_xml_');     # define location for XML file. It's not created yet
    # always ensure that a file ends with .xml because SanMgr needs it
    if($args{file} !~ /[.]xml$/) {
        $args{file} .= '.xml';
    }

    # create XML. It will also call igroup create
    $self->_mk_lun_iscsi_xml( %args );
    delete $args{igroup};

    # igroup is needed by XML but not san manager
    $self->san_manager( %args );                                            # store San SanManager constructor inside Volume
    #Need to load the just created xml file
    $self->{_san_manager}->{ $args{client} }->load_xml(xml_files => [$args{file}]);
    $self->{_san_manager}->{ $args{client} }->setup(-debug_level => 1);     # configure SanManager with XML data
    confess $self->{_san_manager}->{ $args{client} }->errmsg() if $self->{_san_manager}->{ $args{client} }->error();

    #adding two agruments for more device information..-device_type and -protocol.
    #-path_only=>1 will give you the dev type only instead of the whole hash

    # get device list as is from Client
    my $Devices = $self->{_san_manager}->{ $args{client} }->get_client_dev_list(-client => $args{client});
    #Let us print devices, as this structure is changed, helps in debugging
    NOTE::logcomment('Devices returned by get_client_dev_list:' . Dumper($Devices));
    confess $self->{_san_manager}->{ $args{client} }->errmsg() if $self->{_san_manager}->{ $args{client} }->error();

    # extract devices only
    my @sandisks;
    if( defined $Devices->{$args{client}} ) {
        foreach ( @{$Devices->{$args{client}} } ) {
            push @sandisks, $_;
        }
    }

    # print the returned devices from SanMgr::get_client_dev_list
    NOTE::logcomment('sandisks: ' . Dumper(@sandisks) );

    #This part is for finding duplicate serial numbers
    #This piece of code iterates over the disks structure inside $Devices structure which is returned by get_devices
    #It then maintains a hash which acts as a counter for duplicate serial numbers
    #as it finds duplicate serial number it increases the counter and $ser_no_err gets incremented
    my ( $ser_no_err, %ser_no_cnt );
    $ser_no_err = 0;
    map{ ++$ser_no_cnt{ $_->{'serial'} } > 1 && $ser_no_err++ } @sandisks;
    NOTE::logcomment( "Found duplicate serial numbers" ) if $ser_no_err;

    # Earlier (prior to 8/1/2011) , the internal data staructure
    # $self->{_luns} is being updated with returned disks rawName,
    # matching the 'serial' number. However, for linux client, the
    # SanMgr::get_client_dev_list does not return serial number as
    # part of the data strutre returned. Refer burt# 518402.
    # So update the internal data structure $self->{_luns}, matching
    # the "volume", "lun" and "vserver" combination.
    my @dev;
    foreach my $san_disk (@sandisks) {
        my $updated_sandisk = $self->_lun_update_by_lun_volume_vserver_names(
                               'lun_name' => $san_disk->{'lun'}->[0],
                               'volume_name' => $san_disk->{'volume'}->[0],
                               'vserver_name' => $san_disk->{'vserver'}->[0],
                               'rawName'=> $san_disk->{'path'},
                               );
        if(defined $updated_sandisk){
            push @dev, $updated_sandisk;
        }
    }
    return @dev;
}

=head2 ::lun_show

Return a lun show output in a hash

=over

=item Arguments

B<lun-path> I<Optional> Include filtering by lun-path.
B<uuid> I<Optional> Include filtering by uuid.
B<space-resv> I<Optional> Include filtering by space-resv.
B<state> I<Optional> Include filtering by state.
B<size> I<Optional> Include filtering by size.
B<lun> I<Optional> Include filtering by lun.
B<ostype> I<Optional> Include filtering by ostype.
B<fields> I<Optional> Comma separated list of fields to be returned. List may not have spaces.

=item Examples

=back

    my $util = NOTE::SnapMirrorUtil->new(FILER=>$FILER);
    my $src = $util->create_volume(tag=>'src',count=>1,populate=>0,name=>'lun0',VOL_SIZE=>'50MB');
    my $output = $src->lun_show;
    print Dumper($output);

    Following is the output returned by lun_show :

    $VAR1 = [
	     {
		 'volume' => 'lun0_vol',
		 'serial' => 'idWqJ4WB20Va',
		 'vserver' => 'vs0',
		 'lun-path' => '/vol/lun0_vol/01145_Cain_chiseled',
		 'uuid' => '181527db-4d57-11df-8ad6-123478563412',
		 'space-resv' => '-',
		 'state' => 'online',
		 'size' => '65802240',
		 'lun' => '01145_Cain_chiseled',
		 'ostype' => 'windows'
		 }
	     ];

Possible code to get first element as hash ref, instead of reference to array of hashes

    my ($lun) = @{ $vol->lun_show( ostype=>'linux' ) };	# get first Linux lun

=over

=item Known Issies

GNOTE relies on lun's UUID. At some point UID became hidden, see BURT 440258
We currently force to return following fields: B<volume>, B<serial>, B<vserver>,
B<lun-path>, B<uuid>, B<space-resv>, B<state>, B<size>, B<lun>, B<ostype>. Please
feel free to add new fields if required or use B<fields>' option to provide your
own comma separated list.

=back

=cut

sub lun_show {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $lunshow = undef;
    my $output = undef;								# default output will be undef

    $args{fields} ||= 'volume,serial,vserver,lun-path,uuid,space-resv,state,size,lun,ostype'; # this is work around BURT 440258

    # check luns in eval to prevent failure on "not found"
    my $eval_ret = eval {
           $output = $self->server->lun_show(( $self->_vs_vl_qt, %args ));
           1;
        };
    # if error states not found then dont fail and return nothing instead
    if(!$eval_ret) {
      if($@ && $@ !~ /There\s+are\s+no\s+entries\s+matching\s+your\s+query./ix){
         confess $@;
      }
    }

    # go through each and every lun and update each
    foreach my $ls ( @{ $output } ) {
        # individual element instead of assining whole set
        foreach( keys %{ $ls } ) {
            # this is done because we also push client data later
            $self->{_luns}->{ $ls->{'uuid'} }->{$_} = $ls->{$_};
        }   # and we do not want to overwrite client mount points
        # however we return results as array reference for #
        # overall compatibility.
	    push @{ $lunshow }, $self->{_luns}->{ $ls->{'uuid'} };
    }
    return $lunshow;
}

=head2 ::lun_move

Moves lun to new-lun, acting as rename or move between qtrees. Returns reference to
lun show array for new location. Method doesn't check if lun already exists or new
path is valid for sake of negative testing.

=over

=item Arguments

B<lun> I<Required> Original lun name
B<new-lun> I<Required> New lun name

B<new-path> I<Optional> Optional new path
B<new-qtree> I<Optional> Optional new qtree

=item Examples

=back

    my ($state) = @{ $src->lun_move( lun=>'HelloThere', 'new-lun'=>'01145_Cain_chiseled' ) };

    Following is the output returned by lun_show :

    $VAR1 = [
	     {
		 'volume' => 'lun0_vol',
		 'serial' => 'idWqJ4WB20Va',
		 'vserver' => 'vs0',
		 'lun-path' => '/vol/lun0_vol/01145_Cain_chiseled',
		 'uuid' => '181527db-4d57-11df-8ad6-123478563412',
		 'space-resv' => '-',
		 'state' => 'online',
		 'size' => '65802240',
		 'lun' => '01145_Cain_chiseled',
		 'ostype' => 'windows'
		 }
	     ];

=cut

sub lun_move {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "Missing original lun name. ('lun')" unless $args{lun} =~ /\w/;
    confess "Missing new lun name. ('new-lun')" unless $args{'new-lun'} =~ /\w/;
    $self->server->lun_move( ($self->_vs_vl_qt, %args) );
    return $self->lun_show( lun=>$args{'new-lun'} );
}

=head2 ::lun_show_names

Returns array of lun names. All arguments are same as for GNOTE::GX::Volume::lun_show method.

=cut

sub lun_show_names { 
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    map{$$_{'lun'}} @{&lun_show} 
};

#This is an internal function
###############################################################################
#
#   Sub Name:  _lun_update_by_lun_volume_vserver_names
#
#   Description:   Update the internal data structre $self->{_luns} with
#                  values of "rawName" returned by SanMgr::get_client_dev_list.
#                  This is done, by matching the "volume", "lun" and "vserver"
#                  combination.
#
#   Arguments:      NAME        IN/OUT  TYPE      DESCRIPTION
#                   volume_name IN      scalar    volume name
#                   lun_name    IN      scalar    lun name
#                   vserver_name  IN    scalar    vserver name
#                   rawName     IN      scalar    path of the mapped device
#
#
#   Globals:
#
#   Environment:
#
#   Returns:        Success: returns reference to updated hash of
#                   lun_show output
#                   Failure: undef
#
###############################################################################

sub _lun_update_by_lun_volume_vserver_names {
    my ($self,%args) = @_;
    # log internal metrics
    my $trace = $NOTE::TRACELOG->start_timer();

    my $out = undef;
    my $volume_name = delete $args{volume_name};
    my $lun_name = delete $args{lun_name};
    my $vserver_name = delete $args{vserver_name};
    if( !defined($volume_name) ||
        !defined($lun_name) ||
        !defined($vserver_name) ){
        confess 'Need to provide all three arguments ' .
                'volume_name,lun_name and vserver_name.';
    }

    # Check the internal $self->{_luns} data structre, until
    # the provided lun, volume, vserver names matched.
    # and then add the provided key value pairs
    CHECK_LUNS:
    foreach my $uuid ( keys %{ $self->{_luns} } ) {
        if ( ($self->{_luns}->{$uuid}->{lun} eq $lun_name) and
             ($self->{_luns}->{$uuid}->{volume} eq $volume_name) and
             ($self->{_luns}->{$uuid}->{vserver} eq $vserver_name) )
        {
            foreach  (keys %args) {
                $self->{_luns}->{$uuid}->{$_} = $args{$_};
            }
            $out = $self->{_luns}->{$uuid};
            last CHECK_LUNS;
        }
    }
    return $out;
}

=head2 ::lun_online

Sends "lun online" command and returns results from lun show for given lun.
If returned state is not 'online' then method will fail.

There is no checks done if lun actually exists or is it already online
which is intentionally done for sake of negative tests

=over

=item Arguments

B<lun> I<Required> Lun name.

=item Examples

=back

    $vol->lun_online( lun=> '01145_Cain_chiseled' );

    Following is the output returned by lun_online:

    $VAR1 = {
		 'volume' => 'lun0_vol',
		 'serial' => 'idWqJ4WB20Va',
		 'vserver' => 'vs0',
		 'lun-path' => '/vol/lun0_vol/01145_Cain_chiseled',
		 'uuid' => '181527db-4d57-11df-8ad6-123478563412',
		 'space-resv' => '-',
		 'state' => 'online',
		 'size' => '65802240',
		 'lun' => '01145_Cain_chiseled',
		 'ostype' => 'windows'
	     };

Other UI also produces code like this:

     {
       'device-text-id' => '-',
       'block-size' => '512',
       'isMapped' => 'true',
       'device-binary-id' => '-',
       'uuid' => '075f6a4e-c69d-11df-b340-00a098123986',
       'size' => '22553717760',
       'device-legacy-id' => '-',
       'ostype' => 'windows',
       'lun' => '09295_away_famously',
       'space-resv-honored' => 'true',
       'qtree' => '""',
       'vdiskId' => '800004110000000000000000000000651a8603bb',
       'size-used' => '0',
       'extent-size' => '0',
       'volume' => 'vl09295_sway_britches',
       'vserver' => 'vs0',
       'suffix-size' => '0',
       'serial' => '2Fgsz+92Jr8l',
       'path' => '/vol/vl09295_sway_britches/09295_away_famously',
       'space-resv' => 'enable',
       'prefix-size' => '32256',
       'space-alloc' => 'disable',
       'comment' => '""',
       'read-only' => 'false',
       'isOnline' => 'true'
     }

=cut

sub lun_online {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess( "Missing lun name. Argument 'lun' must exist" ) unless $args{lun} =~ /\w/;
    my $err = eval { $self->server->lun_online( ($self->_vs_vl_qt, %args) ) };
    confess $err unless $err =~ /(LUN is not currently offline|\d+ entry was acted on.|^\s*$)/s;
    my ($state) = @{ $self->lun_show( lun=>$args{lun} ) };
    confess( "Lun is not online:".Dumper($state) ) unless( $state->{'state'} eq 'online' || $state->{'isOnline'} eq 'true' );
    return $state;
}

=head2 ::lun_offline

Sends "lun offline" command and returns results from lun show for given lun.
If returned state is not 'offline' then method will fail.

There is no checks done if lun actually exists or is it already offline
which is intentionally done for sake of negative tests

=over

=item Arguments

B<lun> I<Required> Lun name.

B<force> I<optional> Boolean value for forcing lun offline anyway

=item Examples

=back

    $vol->lun_offline( lun=> '01145_Cain_chiseled' );

    Following is the output returned by lun_offline:

    $VAR1 = {
		 'volume' => 'lun0_vol',
		 'serial' => 'idWqJ4WB20Va',
		 'vserver' => 'vs0',
		 'lun-path' => '/vol/lun0_vol/01145_Cain_chiseled',
		 'uuid' => '181527db-4d57-11df-8ad6-123478563412',
		 'space-resv' => '-',
		 'state' => 'offline',
		 'size' => '65802240',
		 'lun' => '01145_Cain_chiseled',
		 'ostype' => 'windows'
	     };

Other UI also produces code like this:

     {
       'device-text-id' => '-',
       'block-size' => '512',
       'isMapped' => 'true',
       'device-binary-id' => '-',
       'uuid' => '075f6a4e-c69d-11df-b340-00a098123986',
       'size' => '22553717760',
       'device-legacy-id' => '-',
       'ostype' => 'windows',
       'lun' => '09295_away_famously',
       'space-resv-honored' => 'true',
       'qtree' => '""',
       'vdiskId' => '800004110000000000000000000000651a8603bb',
       'size-used' => '0',
       'extent-size' => '0',
       'volume' => 'vl09295_sway_britches',
       'vserver' => 'vs0',
       'suffix-size' => '0',
       'serial' => '2Fgsz+92Jr8l',
       'path' => '/vol/vl09295_sway_britches/09295_away_famously',
       'space-resv' => 'enable',
       'prefix-size' => '32256',
       'space-alloc' => 'disable',
       'comment' => '""',
       'read-only' => 'false',
       'isOnline' => 'false'
     }

=cut

sub lun_offline {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess( "Missing lun name. Argument 'lun' must exist" ) unless $args{lun} =~ /\w/;
    my $err = eval { $self->server->lun_offline( ($self->_vs_vl_qt, %args) ) };
    confess $err unless $err =~ /(LUN is not currently online|\d+ entry was acted on.|^\s*$)/s;
    my ($state) = @{ $self->lun_show( lun=>$args{lun} ) };
    confess( "Lun is not offline:".Dumper($state) ) unless( $state->{'state'} eq 'offline' || $state->{'isOnline'} eq 'false' );
    return $state;
}

=head2 ::lun_resize

Sends "lun resize" command and returns results from lun show for given lun.
if size is passed then it checks if lun size matches after resizing.

=over

=item Arguments

B<lun> I<Required> Lun name.

B<size> I<optional> size of the lun

=item Examples

=back


    $vol->lun_resize( lun=> '01145_Cain_chiseled' );

    Following is the output returned by lun_offline:

    $VAR1 = {
		 'volume' => 'lun0_vol',
		 'serial' => 'idWqJ4WB20Va',
		 'vserver' => 'vs0',
		 'lun-path' => '/vol/lun0_vol/01145_Cain_chiseled',
		 'uuid' => '181527db-4d57-11df-8ad6-123478563412',
		 'space-resv' => '-',
		 'state' => 'offline',
		 'size' => '65802240',
		 'lun' => '01145_Cain_chiseled',
		 'ostype' => 'windows'
	     };

=cut

sub lun_resize {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess( "Missing lun name. Argument 'lun' must exist" ) unless $args{lun} =~ /\w/;
    #populating the required fields
    my $result = eval {$self->server->lun_resize( ($self->_vs_vl_qt, %args) ); 1;};
    confess "Lun resize failed:". $@ if(! $result);
    my ($lun_show) = @{ $self->lun_show( lun=>$args{lun} ) };
    return $lun_show;
}


#   Failed to delete Lun in DBlade  This error is generated if you try to destroy the lun on the
#   destination side, as per talking to hari we need to put ignore this, therefore i m going to putg
#   this work around for right now and file a burtg


=head2 ::lun_destroy

This method destroys the provided lun, it also unmaps the lun if it is already mapped.

=over

=item Arguments

B<lun>   = I<string_lun_name>

=item Examples

=back

    my $util = NOTE::SnapMirrorUtil->new(FILER=>$FILER);
    my $src = $util->create_volume(tag=>'src',count=>1,populate=>0,name=>'lun0',VOL_SIZE=>'50MB');
    $src->lun_destroy(lun=><lun_name>);

=cut

sub lun_destroy {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "lun name is not provided\n" unless($args{lun});
    $self->lun_unmap(%args);

    my $output = undef;
    eval { $output = $self->server->lun_destroy( ($self->_vs_vl_qt, %args) ) };   #then destroyg
    if($@ =~ /Failed to delete Lun in DBlade /) {
	NOTE::logcomment(" Error: $@ is generated when trying to destroy a lun on the destination side after discussing this with".
			 " Hari (harihara\@netapp.com) - ignoring this error for right now\n");
	return;
    }
    NOTE::logcomment("Lun: $args{lun} was destroyed successfully! \n ") unless($@);
}

=head2 ::lun_delete

This method deletes the provided lun, it also unmaps the lun if it is already mapped.

=over

=item Arguments

B<lun>   = I<string_lun_name>

=item Examples

=back

    my $util = NOTE::SnapMirrorUtil->new(FILER=>$FILER);
    my $src = $util->create_volume(tag=>'src',count=>1,populate=>0,name=>'lun0',VOL_SIZE=>'50MB');
    $src->lun_delete(lun=><lun_name>);

=cut

sub lun_delete {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "lun name is not provided\n" unless($args{lun});
    my $output = undef;
    $self->lun_unmap(%args);
    eval { $output = $self->server->lun_delete(($self->_vs_vl_qt, %args)) };   #then destroyg
    confess "lun delete error: $@ \n" if ($@);
    NOTE::logcomment("Lun: $args{lun} was deleted successfully! \n ");
}

#lun name and igroup is required

=head2 ::lun_map

This method maps the lun to the provided igroup.

Returns array of hashes for each igroup where lun is mapped. See GNOTE::GX::Vserver::is_lun_mapped for output details

=over

=item Arguments

B<lun>   => I<string> Lun name

B<igroup>   => I<string> Optional igroup name. If igroup is not given then appropriate igroup is searched
and reused. New igroup is created if no available igoups found

=item Examples

=back

    my $util = NOTE::SnapMirrorUtil->new(FILER=>$FILER);
    my $src = $util->create_volume(tag=>'src',count=>1,populate=>0,name=>'lun0',VOL_SIZE=>'50MB');
    $src->lun_map(lun=> <lun_name>,igroup=> <igroup_name>));

=cut

sub lun_map {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "lun name is not provided\n" unless($args{lun});

    my ($lun) = @{$self->lun_show( ($self->_vs_vl_qt, lun=>$args{lun}) )};
    confess "Lun \"$args{lun}\" not found" unless $lun;
    map{ $args{$_} = $$lun{$_} } qw( volume vserver ostype );										# get all parameters from lun itself
    $args{igroup} = $self->Vserver->igroup_create( vserver=>$$lun{vserver}, ostype=>$$lun{ostype}, igroup=>$args{igroup} )->{igroup};	# create if not exists ( Vserver.pm does IF )
    $self->server->lun_map(($self->_vs_vl_qt, %args)) unless $self->is_lun_mapped(%args);
    my $mapped = $self->is_lun_mapped(%args);
    confess "Lun \"$args{lun}\" is not mapped to igroup \"$args{igroup}\"" unless $mapped;
    return $mapped;
}

=head2 ::lun_unmap

This method unmaps the lun from the provided igroup. If group is not given then lun is removed from all igroups

=over

=item Arguments

B<lun>   = I<string_lun_name>

B<igroup>   = I<igroup_name>

=item Examples

=back

    my $util = NOTE::SnapMirrorUtil->new(FILER=>$FILER);
    my $src = $util->create_volume(tag=>'src',count=>1,populate=>0,name=>'lun0',VOL_SIZE=>'50MB');
    $src->lun_unmap(lun=> <lun_name>,igroup=> <igroup_name>));

=cut

sub lun_unmap {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "lun name is not provided\n" unless($args{lun});
    my ($lun) = @{$self->lun_show( lun=>$args{lun} )};
    confess "Lun \"$args{lun}\" not found" unless $lun;

    my $maps = $self->is_lun_mapped(%args);
    if( $maps ) {
	if( $args{igroup} ) {
	    $self->server->lun_unmap( ($self->_vs_vl_qt, %args) );
	} else {
	    $self->server->lun_unmap( ($self->_vs_vl_qt, %args), igroup => $_) foreach( map{ $$_{'igroup'} } @$maps );
	}
    }
}

=head2 ::is_lun_mapped

Returns referense to an array of hashes if lun is mapped to at least one igroup or returns undef.

=over

=item Arguments

B<lun>   = I<string_lun_name>

=item Examples

=back

    my $util = NOTE::SnapMirrorUtil->new(FILER=>$FILER);
    my $src = $util->create_volume(tag=>'src',count=>1,populate=>0,name=>'lun0',VOL_SIZE=>'50MB');
    my $maps = $src->is_lun_mapped(lun=> <lun_name>);

    my @igroup_names_only = map{ $$_{'igroup'} } @{ $src->is_lun_mapped(lun=> <lun_name>) };

    # map looks like this if mapped to two igroups without initiators
    # $maps = [
    #		  {
    #		    'initiators' => '-',
    #		    'volume' => 'vl06423_bump_branding',
    #		    'throttle-borrow' => '-',
    #		    'vserver' => 'vs0',
    #		    'lun-path' => '/vol/vl06423_bump_branding/06423_Luis_Khartoum',
    #		    'throttle-resv' => '-',
    #		    'portset' => '-',
    #		    'os' => 'windows',
    #		    'lun' => '06423_Luis_Khartoum',
    #		    'igroup' => 'igroup06423_Indo_uniquely',
    #		    'alua' => '-',
    #		    'luno' => '0',
    #		    'type' => 'iscsi'
    #		  },
    #		  {
    #		    'initiators' => '-',
    #		    'volume' => 'vl06423_bump_branding',
    #		    'throttle-borrow' => '-',
    #		    'vserver' => 'vs0',
    #		    'lun-path' => '/vol/vl06423_bump_branding/06423_Luis_Khartoum',
    #		    'throttle-resv' => '-',
    #		    'portset' => '-',
    #		    'os' => 'windows',
    #		    'lun' => '06423_Luis_Khartoum',
    #		    'igroup' => 'igroup06423_jest_filename',
    #		    'alua' => '-',
    #		    'luno' => '0',
    #		    'type' => 'iscsi'
    #		  }
    #	      ];

=cut

sub is_lun_mapped {
    my ($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    confess "lun name is not provided\n" unless($args{lun});

    #ostype param is not a valid parameter for cli lun mapped show
    my $ostype = delete $args{ostype};

    my $output = undef;
    eval { $output = $self->server->lun_mapped_show(($self->_vs_vl_qt, %args)) }; # see BURT 439477, UI change after fsb_rrN_100812_0200
    if( $@ ) {
	return $output if $@ =~ /There are no entries matching your query/;	  # return if error is "no mapping"
	confess $@ unless $@ =~ /ERROR:.*is not a recognized command/;		  # return actual error unless its "not a recognized command"
	eval { $output = $self->server->lun_show_map(($self->_vs_vl_qt,%args)) }; # or try old style command as well and procss error if any
	confess $@ if $@ && $@ !~ /There are no entries matching your query/;	  # skip on "no entries" or this "methodio" must die
    }
    return $output;
}

=head2 ::lun_clone

Creates clone for given lun. Returns lun_show() output for created clone.
Method uses GNOTE::GX::Volume->file_clone_create() method to create clone.

=over

=item Arguments

B<lun>   = I<string_lun_name>, required if Volume or Qtree has more then one lun.
Original name is used as 'source-path' for GNOTE::GX::Volume->file_clone_create()

B<destination-path> = string

The path, relative to the root of the volume or qtree, of the new lun to create.
If path is missing then new name is generated using original name as prefix followed
by '_clnd_' key.

B<snapshot-name> = string

Optional. The name of a snapshot on the volume to use as the source of the
lun, rather than the volume or qtree itself.

=item Examples

      1 use GNOTE;
      2 eval {
      3     my $ut = NOTE::SnapMirrorUtil->new();               ## @step Process CLI and connect to cluster
      4     my $vl = $ut->create_volume(tag=>'src');            ## @step Create volume based on CLI parameters
      5     my $qt = NOTE::GX::Qtree->create(Volume => $vl);    ## @step Create Qtree
      6     $qt->lun_create;                                    ## @step Setup iSCSI if needed and create single lun
      7     my ($clone) = $qt->lun_clone;                       ## @step Clone the lun and return lun's properties
      8 };
      9 logresult('FAIL',$@) if $@;                             ## @step Print errors if found
     10 NOTE::teardown;                                         ## @step clean up

      1 use GNOTE;
      2 eval {
      3     my $ut = NOTE::SnapMirrorUtil->new();                           ## @step Process CLI and connect to cluster
      4     my $vl = $ut->create_volume(tag=>'src');                        ## @step Create volume based on CLI parameters
      5     my $luns = $vl->lun_create( count=>5, igroup=>1);               ## @step Create igroup. Create 5 luns and map to igroup
      6     my @clones = map { $vl->lun_clone( lun=>$_->{'lun'} ) } @$luns; ## @step Create clone for each created lun
      7 };
      8 logresult('FAIL',$@) if $@;                                         ## @step Print errors if found
      9 NOTE::teardown;                                                     ## @step Clean up

Original lun show:

   {
     'device-text-id' => '-',
     'block-size' => '512',
     'isMapped' => 'true',
     'device-binary-id' => '-',
     'uuid' => '3781558e-eabf-11df-bff4-00a098123986',
     'size' => '49351680',
     'device-legacy-id' => '-',
     'ostype' => 'windows',
     'lun' => 'lun_01326_area_Maguires',
     'space-resv-honored' => 'true',
     'qtree' => '""',
     'vdiskId' => '800004610000000000000000000000651f553acf',
     'size-used' => '0',
     'extent-size' => '0',
     'volume' => 'vl01326_mats_Adolphus',
     'vserver' => 'vs0',
     'suffix-size' => '0',
     'serial' => '2Fgsz+9CpT1u',
     'path' => '/vol/vl01326_mats_Adolphus/lun_01326_area_Maguires',
     'space-resv' => 'enable',
     'prefix-size' => '32256',
     'space-alloc' => 'disable',
     'comment' => '""',
     'read-only' => 'false',
     'isOnline' => 'true'
   }

Lun show of clone:

   {
     'device-text-id' => '-',
     'block-size' => '512',
     'isMapped' => 'false',
     'device-binary-id' => '-',
     'uuid' => '46eec43e-eabf-11df-bff4-00a098123986',
     'size' => '49351680',
     'device-legacy-id' => '-',
     'ostype' => 'windows',
     'lun' => 'lun_01326_area_Maguires_clnd_01326_gill_envelops',
     'space-resv-honored' => 'true',
     'qtree' => '""',
     'vdiskId' => '800004610000000000000000000025971f5544eb',
     'size-used' => '200704',
     'extent-size' => '0',
     'volume' => 'vl01326_mats_Adolphus',
     'vserver' => 'vs0',
     'suffix-size' => '0',
     'serial' => '2Fgsz+9CpT1w',
     'path' => '/vol/vl01326_mats_Adolphus/lun_01326_area_Maguires_clnd_01326_gill_envelops',
     'space-resv' => 'enable',
     'prefix-size' => '32256',
     'space-alloc' => 'disable',
     'comment' => '""',
     'read-only' => 'false',
     'isOnline' => 'true'
   }

=back

=cut

sub lun_clone {
    my($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    unless( $args{lun} ) {					# lun name is required parameter but only if
	my @luns = $self->lun_show_names;			# there more then one lun defined within object
	confess "Missing lun name" unless scalar(@luns) == 1;	# there is no need to provide lun name if volume or
	$args{lun} = shift @luns;				# qtree hold just a single lun
    }

    my $vol = ref($self) =~ /Qtree/ ? $self->Volume : $self;							   # need vol object for clone method
    my $container = $vol->name;											   # need name without volume, but with qtree for 'source-path'
    ( $args{'source-path'} = ( $self->lun_show( lun => delete $args{lun} ) )->[0]->{'path'} ) =~ s/.*$container//; # use full path including qtree, but not volume name
    $args{'destination-path'} = $self->newName( $args{'source-path'} . '_clnd_') unless $args{'destination-path'}; # create destination if not exist. Include old name as prefix

    $self->file_clone_create( %args );
    $args{'destination-path'} =~ s|.*/||g; # use base name only for lun show, strip qtree if present.
    return $self->lun_show( lun=>$args{'destination-path'} ) || confess "Lun $args{'source-path'} was not cloned! No lun clone with name $args{'destination-path'} found";
}

=head2 ::lun_modify

Wrapper around filer's lun modify command followed by lun show. All arguments are passed
through as is with exception of vserver, volume and qtree. They are taken from volume or
qtree object itself. Please do not provide volume, or qtree or vserver parameters.

B<NOTE:> Its up to end user to decide which parameters are mandatory and which are optional.

=over

=item Arguments

All arguments are the same as filer's lun modify arguments. Set can be changed by
ONTAP, so refer to actual ONTAP documentation. At time of this writing we had
following arguments present in ONTAP:

B<vserver> I<vserver name> Vserver Name (default: vs0)

B<path> I<path> LUN Path

B<qtree> I<text (size 0..255)> Qtree Name (default: "")

B<lun> I<text (size 1..255)> LUN Name

B<space-reserve> I<enable|disable> Space Reservation

B<serial> I<text> Serial Number

B<comment> I<text (size 0..255)> Comment

B<space-allocation> I<enable|disable> Space Allocation

B<state> I<online|offline> State

B<device-legacy-id> I<integer> Device Legacy ID

B<device-binary-id> I<text (size 0..128)> Device Binary ID

B<clear-binary-id> I<true> Clear Device Binary ID

B<device-text-id> I<text (size 0..255)> Device Text ID

B<clear-text-id> I<true> Clear Device Text ID

=item Examples

=back

      1 use strict;
      2 use Data::Dumper;
      3 use GNOTE;
      4 eval {
      5     my $ut = NOTE::SnapMirrorUtil->new();                  ## @step process CLI
      6     my $vl = $ut->create_volume(tag=>'src');               ## @step create RW volume
      7     my $ln = ($vl->lun_create())->[0]->{lun};              ## @step create two luns
      8     my $md = $vl->lun_modify(state=>'offline', lun=>$ln);  ## @step change lun state to 'offline'
      9     print Dumper($md);                                     ## @step display lun status
     10 };
     11 logresult('FAIL',$@) if $@;                       ## @step Report errors if any
     12 NOTE::teardown;                                   ## @step Clean up after yourself

=cut

sub lun_modify {
    my($self,%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $self->server->lun_modify( ( $self->_vs_vl_qt, %args ) );
    return $self->lun_show(%args);
}


# Best Effort Clean Up. This method is called by Volume->DESTROY and Qtree->DESTROY
# code was moved out from Volume->DESTROY and Qtree->DESTROY.
sub _san_cleanup {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    eval {								# client clean up. Such as unmount/unmap UNIX/Windows
        if( $self->{_san_manager} ) {                                   # iterating over the san san_manager object
            $_->cleanup() foreach( values %{ $self->{_san_manager} } ); # and finding the clients and calling san_manager->cleanup method
        }                                                               # see sub san_manager for more informtation
    };

    #IOMan objects clean up
    my $ret_val = eval {
                      if( $self->{_ioman_objs_array_ref} ) {
                          foreach ( @{ $self->{_ioman_objs_array_ref} } ) {
                              NOTE::logcomment('Cleaning up IOMan_obj...');
                              $_->cleanup;
                          }
                      }
                      1;
                  };
    #If error just print, no need to do anything
    if(!$ret_val) { NOTE::logcomment("Error in cleaning SANIOman objects. Error: $@") }

    eval {
        if( defined($self->lun_show()) ) {									  # remove all luns as well
            map {$self->lun_destroy(lun=>$$_{'lun'},force=>"true")} @{$self->lun_show()} unless NOTE->ALL_SPARED; # don't leave them around
        }													  # this includes removing from
    };														  # igroup as well

    foreach( @{$self->{_san_xml_files}} ) { unlink $_ if -f $_ } # removing temorary XML files for san manager if have been created
}

1;
