# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/GX/SnapVaultSet.pm#1 $
#
## @summary  GNOTE GX SnapVault Set module
## @author   dl-dpgqa-automation@netapp.com, cpeter@netapp.com
## @status   private
## @pod      here

=head1 NAME

NOTE::GX::SnapVaultSet

=head1 DESCRIPTION

SnapVault is subset of SnapMirror class.

STATE IS STILL PRIVATE!!! Use at your own risk


=head1 METHODS

Same as NOTE::GX::SnapMirror. With only difference that call is made for each member of the set. See NOTE::GX::SnapMirror for rest of the methods.

=cut

package NOTE::GX::SnapVaultSet;

use strict;
use warnings;
no warnings 'redefine';
use re 'eval';

use GNOTE;
use GNOTE::GX;
use GNOTE::GX::SnapMirror;
use Carp qw(confess);
use TracePkg;

our @ISA = qw(NOTE::GX);
our $AUTOLOAD;

my $Trace = TracePkg->new();

sub AUTOLOAD {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my $name = $AUTOLOAD;

    $name =~ s/.*://g;			# strip fully-qualified portion
    return undef if $name eq 'DESTROY'; # each SM is destroyed individually, and not as part of the set
    confess "Unapplicable method \"NOTE::GX::SnapMirror->$name\" called..." unless( NOTE::GX::SnapMirror->can($name) );
    return map{ $_->$name(%args) } @{ $self->members };
}

=head2 create

Creates a SnapVaultSet object only. Returns object representing the relationships.

=over

=item Arguments

See NOTE::GX::SnapMirror::create for arguments and examples

=item Example

      1 # -*- perl -*-
      2 # $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/GX/SnapVaultSet.pm#1 $
      3
      4 use strict;
      5 use GNOTE;
      6 use Data::Dumper;
      7 use dp::TestWrapper qw( run_test );
      8
      9 my ( $svset, $src );
     10
     11 run_test();
     12
     13 sub do_setup {
     14     my $util = NOTE::SnapMirrorUtil->new();
     15     my ($src) = $util->create_volume(tag=>"src");
     16     my @dst = $util->create_volume(tag=>"dst", 'mirror-type'=>"DP", count => 2);
     17
     18     $svset = NOTE::GX::SnapVaultSet->create( src => $src, dst => \@dst );
     19     my @show = $svset->status;
     20     print "Status of all SM relationships:\n" . Dumper(\@show);
     21 }
     22
     23 sub do_test {
     24     $svset->members->[0]->src->fill( proto => 'local', type => 'filersio', max => '100m' ); # add data to source
     25     $svset->initialize;
     26     $svset->members->[0]->src->fill( proto => 'local', type => 'filersio', max => '500m' ); # add more data
     27     $svset->update;
     28     $svset->check;
     29 }
     30
     31 sub do_teardown { NOTE::teardown }

=back

=cut

sub create {
    my ( $class, %args ) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my $self = {};
    bless $self, $class;

    if( ref($args{dst}) ne 'ARRAY' ) {
	confess "Destination must be reference to an array of volume objects!";
    }

    my @dst = @{ delete $args{dst} };
    $args{type} ||= 'XDP';
    @{ $self->{_members} } = map{ NOTE::GX::SnapMirror->create(%args, dst => $_) }  @dst;
    return $self;
}

=head2 members

Returns array reference of SnapMirror objects from SnapMirror set

=cut

sub members {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();
    return $self->{_members};
}


=head2 initialize

Wrapper around NOTE::GX::SnapMirror->initialize. Calls NOTE::GX::SnapMirror->initialize for each member of SnapVault set

=cut

sub initialize {
    my ( $self, %args ) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();
    if ( ref($self) ne 'NOTE::GX::SnapVaultSet' ) {
	$self = NOTE::GX::SnapVaultSet->create(%args);
    }
    delete $args{src};
    delete $args{dst};
    $_->initialize( %args ) foreach( @{ $self->members } );
    return $self;
}

1;
