#
# $Id: //depot/prod/test/Rsierranevada/lib/GNOTE/SnapMirror.pm#1 $
#
## @summary  NOTE Volume class
## @status   Public
## @pod      here
## @author   tmichael@netapp.com dl-dpgqa-automation@netapp.com

=head1 NAME

NOTE::SnapMirror

=head1 DESCRIPTION

This is the GNOTE base class for SnapMirror relationships.

=head1 METHODS

=cut

package NOTE::SnapMirror;
use GNOTE;
our @ISA = qw(NOTE);
use Carp qw(croak);

use Tharn qw();

# Checks the arguments for clues to determine if this
# is a GX style call or a 7G style call.
# Returns TRUE if:
#   1) either source or destination are GX volumes
#   2) the Node argument exists or
#   3) the Vserver argument exists or
#   4) the Filer argument exists and is in a NOTE::GX class or
#   5) the VSERVER option is defined
# Otherwise, returns FALSE.

sub _is_GX {
    my (%args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    return 1 if defined NOTE::option( q/$VSERVER/ );
    for my $v ( 'src', 'dst' ) {
	return 1 if $args{$v} eq 'NOTE::GX::Volume';
	return 1 if ref $args{$v} eq 'NOTE::GX::Volume';
	if ( ref $args{$v} eq 'HASH' ) {
	    return 1 if exists $args{$v}->{Cluster};
	    return 1 if exists $args{$v}->{Node};
	    return 1 if exists $args{$v}->{Vserver};
	    return 1 if exists $args{$v}->{Nodes};
	    return 1 if exists $args{$v}->{nodes};
	    return 1 if exists $args{$v}->{Filer}
		    && ref($args{$v}->{Filer}) =~ /^NOTE::GX/;
	    if ( exists $args{$v}->{Aggregate} ) {
		# Check whether this is (or will be) a GX aggregate
		if ( ref $args{$v}->{Aggregate} eq 'HASH' ) {
		    return NOTE::Aggregate::_is_GX( %{$args{$v}->{Aggregate}} );
		}
		return 1 if ( ref $args{$v}->{Aggregate} =~ /^NOTE::GX/ );
	    }
	}
    }
    return 0;
}

sub _do_method {
    my ($method, $self, %args) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    if ( ! ref $self ) {
	if ( _is_GX( %args ) ) {
	    return NOTE::GX::SnapMirror->$method( %args );
	} else {
	    return NOTE::7G::SnapMirror->$method( %args );
	}
    }
    # At this point, $self is an object
    return $self;
}

sub new {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return _do_method("new", @_);
}

sub initialize {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return _do_method("initialize", @_);
}

=head2 B<Component Objects>

=head2 ::src

Returns the object representing the current source.

=over

=item Arguments

I<volume object> | I<Qtree object>

I<Optional.>
If present, replaces the current source object.

=item Examples

=back

     $src = $sm->src;

=cut

sub src {
    my ($self,$newsrc) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    if ( defined $newsrc ) {
	my $oldsrc = $self->{_src};
	$self->{_src} = $newsrc;
	return $oldsrc;
    } else {
	return $self->{_src};
    }
}

=head2 ::dst

Returns the object representing the current destination.

=over

=item Arguments

I<volume object> | I<Qtree object>

I<Optional.>
If present, replaces the current destination object.

=item Examples

=back

     $dst = $sm->dst;

=cut

sub dst {
    my ($self,$newdst) = @_;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    if ( defined $newdst ) {
	my $olddst = $self->{_dst};
	$self->{_dst} = $newdst;
	return $olddst;
    } else {
	return $self->{_dst};
    }
}

=head2 B<Object Identification>

=head2 ::description

Returns a description of the SnapMirror relationship
including a tag for the source
and a tag for the destination if set.

=over

=item Arguments

None.

=item Example

=back

     $desc = $sm->description;

=cut

sub description {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    return $self->src_path . ($self->src->tag ? " (".$self->src->tag.") " : "")
	.  " -> "
	.  $self->dst_path . ($self->dst->tag ? " (".$self->dst->tag.") " : "")
    ;
}


sub DESTROY {
    my $self = shift;
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    return if ( $$ != $self->{_owner} ); # Only the creator can destroy an object
    return if ( $self->{_spare_me} );	 # Means don't destroy
    return if ( $self->{_destroyed} );	 # Already dead
    return if ( NOTE->ALL_SPARED );
    $self->destroy;
}

