#
# Copyright (c) 2011 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Traffic setup in HA systems
## @author anusuyak@netapp.com, pragyan@netapp.com
## @status review
## @pod here

##################################################################
####  THIS LIBRARY IS NOT YET COMPLETE WITH SOME FUNCTIONALITY YET.
####  ALSO LOT OF OPTIMISATIONS NEED TO BE DONE AND THIS WILL
####  BE DONE IN THE NEAR FUTURE 
##################################################################

package HAiosetup;

use strict;
use warnings;

use Tharn qw($Log subtest host);
use Params::Validate qw(:all);
use Data::Dumper;

use NACL::APISet;
use NATE::Exceptions::Argument qw(:try);
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::Exceptions::VerifyFailure qw(:try);
use NACL::C::Vserver;
use NACL::C::License;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::STask::NetworkInterface;
use NACL::STask::Volume;
use NACL::C::ExportPolicy;
use NACL::C::ExportPolicyRule;
use NACL::STask::VserverNFS;
use NACL::STask::Client::Mount;
use NACL::C::Client;
use NACL::STask::Cluster;
use NACL::C::NetworkRoutingGroupsRoute;
use NACL::C::Statistics;
use NACL::C::StorageDisk;
use NACL::STask::VolumeEfficiency;
use Switch;
use HAconstants;
use NATE::ProcManager;
use NATE::STAF;
use HAutils qw(get_client);

use Class::MethodMaker [
    new    => [ '-hash', 'new' ],
    scalar => [ { -type => 'NACL::C::Node'}, 'local' ],
    scalar => [ { -type => 'NACL::C::Node'}, 'partner' ],
    scalar => [ { -type => 'NATE::ProcManager'}, 'proc_manager'],
    ## Note: more fields will be added to accomodate future enhancements.
];

=head1 NAME

HAiosetup

=head1 DESCRIPTION

C<HAiosetup> sets up traffic on HA pair, Single node or MCC configuration. 
It sets various kinds of traffic like NFS, CIFS, PFS and also gives the option 
to the caller of the methods to control the traffic, to setup aggregate and 
volume configurations based on caller inputs as well as allows choosing data 
traffic types to be run which will be discued in the methods below.

This module will be used in tests and utility scripts needed to test HA related
features, such as those being tested by the HA QA team and WAFL HA QA team.

=head1 METHODS

=head2 new
    my $HAio_obj = HAiosetup->new(
        local => $command_interface_of_local_node,
        partner => $command_interface_of_partner_node,
    );
Return a new HAiosetup object with these attributes.

=head2 setup
    $HAio_obj->setup();

This method is used to populate the HAiosetup object further with the Proc Manager
object which will be helpful in starting the traffic in parallel as subtests
which uses the NATE::ProcManager.

This method creates vservers, export polices and data interfaces if required.
It also takes a parameter which says that the deux_init is already run and there
is no need to create the interfaces, vservers etc. By default if no parameters
is passed it assumes that deux_init is run and there is no need to create 
vservers, interfaces etc.

The main job of this method is to create aggregates and volumes according to
the input parameters and create the ProcManager object with the traffic type
that needs to run on these aggregates/volumes. It also mounts the volumes onto
the client list specified.

=over

=item Options

=over

=item C<< deux_init_interface => $one_or_zero >>

(optional) This accpets either 1 or 0. 1 means the deux_init is run and no need
to setup vserver and data interfaces.
(default) 1.

=item C<< aggregate => $aggregate_name >>

(optional) The name of the aggregate where the volumes will be created.
If multiple aggregates get created they will be of the style aggr_name<i> where
i is the count.By default it will pic sfo_aggr as the name. Aggregates will
be reused if they are already present with the name.

=item C<< vserver => $vserver_name >>

(optional) Vserver name. If deux_init_interface is set to 1 the default will be
taken as "vs0". Else the default will be sfo_vserver.

=item C<< volume => $volume >>

(optional) Name of the volume to be created. Default name is sfo_vol.

=item C<< number_of_data_aggrs => $integer_or_max >>

(optional) Number of aggregates that needs to be created. By default 1 aggregate.
It also takes a value "max" which will calculate the max number of aggrs
that can be created on that system by looking for the number of spare disks
available in that system.

=item C<< number_of_data_vols => $integer >>

(optional) Number of volumes that need to be created on each aggr. By default 1. 

=item C<< number_of_data_disks => $integer >>

(optional) Number of disks required for each aggr. By default 2(raid4 config)

=item C<< raid_type_of_aggr => $raidtype >>

(optional) Raid type of the aggregates. By default raid4.

=item C<< vol_size => $volsize >>

(optional) Volume size. Default 300g

=item C<< io_method => $iomethod >>

(optional) Below is the list of io_methods supported which will be the kind of 
data traffic. Default "hammer_mixed"
hammer_mixed
hammer_overwrites
sio_sequential_4k
sio_sequential_64k
sio_random_4k
sio_random_64k
sio_partial_sequential
sio_random_partial
More types will be supported soon.

=item C<< io_duration => $ioduration >>
(optional) io duration in seconds. Default 80000 seconds.

Documentation is incomplete, will be done soon.

=cut

sub setup {

    my $self  = shift;
    my %opts = validate(
        @_, 
        {
            deux_init_interface => {type => SCALAR,optional => 1,default =>1}, ## By default we assume deux_init is run and vs0 and data interfaces are created
            vserver => { type => SCALAR , optional=> 1 , default => SFO_VSERVER },
            aggregate => { type => SCALAR , optional=> 1 , default => SFO_AGGR }, 
            volume => {type => SCALAR , optional=> 1 , default => SFO_VOL },
            data_lif => { type => SCALAR , optional=> 1 , default => SFO_DATALIF },
            'ns-switch' => { type => SCALAR , optional=> 1 , default => "file" }, ## Need to change to ARRAYREF	
            security => { type => SCALAR,optional=> 1,  default => "unix" },
            number_of_data_aggrs => { type => SCALAR,optional=> 1,  default => NUM_AGGR },  ## One aggr on each node, also suuports a hidden value "max" which creates maximum number of aggreagtes supported by the configuration.
            number_of_data_disks => { type => SCALAR,optional=> 1,  default => DISKCOUNT}, ## 2 disks by default as the default config is raid4
            raid_type_of_aggr => { type => SCALAR,optional => 1, default => RAIDTYPE }, ## raid4 by default
            number_of_data_vols =>  { type => SCALAR,optional=> 1,  default =>  NUM_VOL }, ## One volume on each aggregate is the default
            vol_size => {type => SCALAR,optional=> 1, default => "300G" }, 
            io_method => {type => SCALAR,optional=> 1, default => "hammer_mixed" },
            io_duration => {type => SCALAR,optional=> 1, default => 80000},
            enable_sis => { type => BOOLEAN,optional=> 1, default => 0 },
        },
    );	

    my $node = $self->local();
    my $partner = $self->partner();
    my $sfo_vserver_aggr;
    my @sfo_data_aggrs;
    my @sfo_data_vols;
    my $sfo_vserver;
    my @sfo_data_lifs;
    my $sfo_vserver_nfs;
    my $server_gateway;
    my @sfo_export_rule;
    my @sfo_export_policy_rule;
    my @filers;
    my $aggr_count = delete $opts{number_of_data_aggrs};
    my $vol_count = delete $opts{number_of_data_vols};
    my $vol_size = delete $opts{vol_size};
    my $enable_sis = delete $opts{enable_sis};
    push ( @filers, $node);
    push ( @filers, $partner);
    
    # If deux_init is run then dont set up vserver and interfaces.
    
    my $int_created = delete $opts{deux_init_interface};
    if (!$int_created) {
    ########################################################################
    # Ensure aggregate  exists. Create the aggregate if needed
    ########################################################################
        my $root_aggr_name = $opts{aggregate} ."_vserver";        
        try {
            my @aggr_nonroot_existing = NACL::C::Aggregate->find(
                command_interface => $node,
                filter => {'home-name' => $node, root => "false", 'ha-policy' => 'sfo'}
            );
            $sfo_vserver_aggr = $aggr_nonroot_existing[0];
        } 
        catch NACL::Exceptions::NoElementsFound with {
            $Log->comment(
                "Caught ResponseException: no non-root aggr found, so creating it \n"
            );
            $sfo_vserver_aggr = NACL::STask::Aggregate->create(
                command_interface => $node,
                aggregate         => $root_aggr_name,  
                if_exists         => 'reuse',
                diskcount         => DISKCOUNT,
                raidtype          => RAIDTYPE,
            );

        };
        ########################################################################
        # Ensure VS exists. Create the vserver if needed
        ########################################################################
        try {
            $sfo_vserver = NACL::C::Vserver->find(command_interface => $node, 
            filter => { vserver => $opts{vserver} },
        );
        } catch NACL::Exceptions::NoElementsFound with {
            my $vserver_root_volume = $opts{volume} ."_vs_root";
            $sfo_vserver = NACL::C::Vserver->create(
                command_interface           => $node,
                vserver                     => $opts{vserver},
                 rootvolume                 => $vserver_root_volume,
                aggregate                   => $sfo_vserver_aggr,
                "ns-switch"                 => $opts{"ns-switch"},
                "rootvolume-security-style" => $opts{security},
            );
        };
   
        ########################################################################
        # Ensure at least one data LIF exists from each Filer is bound to this
        # VS. Create the LIF if needed using optionally passed in params. Let
        # ngsh error out if some of the required params to create LIF are not
        # specified
        ########################################################################\
        my $j = 1;
        foreach my $filer ( @filers ) {

            my $lifinfo =  NACL::STask::Node->get_hostrec_lif_info(
                command_interface    => $filer,
                default_mtu          => DEFAULT_MTU,
                required_fields      => [qw( NAME IP MASK GATEWAY PORT )],
                verbose_checks       => 1,
            );

            my $data_lif = $lifinfo->{DATA};
            
            for ( my $i = 0 ; $i < (scalar (@$data_lif)) ; $i++ ) {
                # Create new Lif on the vserver if not exists earlier.
                my $info = $data_lif->[$i];
                $server_gateway= $info->{GATEWAY} ;
                my $data_lif_name = $opts{data_lif} . $j;
                $j++;
                my $node_name = $filer->node();
                $Log->comment("Creating Lif $data_lif_name on node $node_name");
                try {
                    my $sfo_data_lif = NACL::STask::NetworkInterface->create(
                        command_interface => $filer,
                        server            => $sfo_vserver,
                        address           => $info->{IP},
                        lif               => $data_lif_name,
                        role              => 'data',
                        'home-node'       => $filer,
                        'home-port'       => $info->{PORT},
                        netmask           => $info->{MASK}, 
                        nacltask_if_exists => 'reuse',
                    );
                    push(@sfo_data_lifs, $sfo_data_lif);
                } catch NACL::APISet::Exceptions::ResponseException with {
                    my $exception_object = shift;
                    $Log->comment( "Caught ResponseException in partner with"
                    . $exception_object->text() );
                };

            }
        }
        if ( scalar ( @sfo_data_lifs ) == 0 ) {
            $Log->exit();
            NATE::BaseException->throw("Failed to create data lifs");
        }
        ########################################################################
        ## Ensure NFS is created for our VS. Create NFS if needed
        ########################################################################
        $sfo_vserver_nfs = NACL::STask::VserverNFS->create(
            vserver            => $sfo_vserver,
            command_interface  => $node, 
            if_exists	       => 'reuse',
            access             => "true",
        );

        # Find Routing Group for data lif
        my $route;
        my $default_routing_group = $sfo_data_lifs[0]->state()->routing_group();

        ## Create New route for Vserver
        try {
            $route = NACL::C::NetworkRoutingGroupsRoute->find(
                command_interface => $node,
                filter            => {
                    server          => $sfo_vserver,
                    'routing-group' => $default_routing_group, 
                }
            );
        } catch NACL::Exceptions::NoElementsFound with {	
            NACL::C::NetworkRoutingGroupsRoute->create(
                command_interface => $node,
                server            => $sfo_vserver,
                gateway           => $server_gateway,
                'routing-group'   => $default_routing_group,
                destination       => '0.0.0.0/0'
            );
        };

        ########################################################################
        # Ensure ruleset exists for VS. Create exportruleset is needed
        ########################################################################
        try {
            @sfo_export_rule = NACL::C::ExportPolicy->find(
                command_interface => $node,
                filter            => {
                    vserver           => $sfo_vserver,
                    policyname        => 'default',
                }
            );   
        } catch NACL::Exceptions::NoElementsFound with {
            $sfo_export_rule[0] = NACL::C::ExportPolicy->create(
                command_interface => $node,
                 vserver           => $sfo_vserver, 
                 policyname        => 'default',
            );
        };
        ########################################################################
        # Ensure export rule within the ruleset. Create new export rule if needed
        ########################################################################
        try {
            @sfo_export_policy_rule = NACL::C::ExportPolicyRule->find(
                command_interface => $node,
                filter            => {
                    vserver           => $sfo_vserver,
                    policyname        => 'default',
                    rwrule            => ["any"],
                    rorule            => ["any"], 
                }
            );
        } catch NACL::Exceptions::NoElementsFound with {        
            $sfo_export_policy_rule[0] = NACL::C::ExportPolicyRule->create(
                command_interface => $node,
                vserver           => $sfo_vserver,
                policyname        => 'default',
                clientmatch       => "0.0.0.0/0",
                rorule            => ["any"],
                rwrule            => ["any"],
                protocol          => ["any"],
                anon		      => '0',
                superuser	      => ["any"],
                'chown-mode'	  => 'unrestricted',
           );
        };
    } else {
        $sfo_vserver = "vs0";
        @sfo_data_lifs = NACL::C::NetworkInterface->find(
            command_interface => $node,
            filter => { lif => "*data1" },
        );
    }
    ########################################################################
    # Ensure Aggregates for data population  exists on each node, otherwise 
    # Create given number of aggregates on each node. Also compute the max 
    # aggrs supported by the system if "max" is supplied as the parameter. 
    ########################################################################
    our ($local_aggr_count,$partner_aggr_count,$count) = (0) x 3;
    $local_aggr_count = $aggr_count;
    if (defined ($partner))
    {
        $partner_aggr_count = $aggr_count;
    }
    if ($aggr_count eq "max") {
        $Log->comment("Computing max data aggrregates supported based on spare disk availability");
        my @temp = NACL::C::StorageDisk->find(
            command_interface => $node,
			filter => {
				owner => $node->node,
				state => "spare",
				pool => "Pool0", # By default Pool0 because MCC local site uses Pool0 during aggr create
			}
        );
        print ("No of spare disks on local node: ". @temp. "\n");
        $local_aggr_count = int((@temp - 1)/$opts{number_of_data_disks});
        print ("No of Aggregates possible on Local Node: ".$local_aggr_count. "\n");
		if (!$local_aggr_count) {
			my @temp_aggr = NACL::C::Aggregate->find(
				command_interface => $node,
				filter => {
					aggregate => "$opts{aggregate}*",
					nodes => $node->node,
				}
			);
			$local_aggr_count = @temp_aggr;
		}
        if ( defined ( $partner ) ) {
            my @temp = NACL::C::StorageDisk->find(
                command_interface => $partner,
				filter => {
					owner => $node->node,
					state => "spare",
					pool => "Pool0", # By default Pool0 because MCC local site uses Pool0 during aggr create,
				}
			);
            print ("No of spare disks on partner node: ". @temp. "\n");
            $partner_aggr_count = int((@temp - 1)/$opts{number_of_data_disks});
            print ("No of Aggregates possible on Partner Node: ".$partner_aggr_count. "\n");
        }
		if (!$partner_aggr_count) {
			my @temp_aggr = NACL::C::Aggregate->find(
				command_interface => $partner,
				filter => {
					aggregate => "$opts{aggregate}*",
					nodes => $partner->node,
				}
			);
			$partner_aggr_count = @temp_aggr;
		}
    }
	my $diskcount = $opts{number_of_data_disks};
    while ($local_aggr_count) {
        $Log->comment("Creating Data Aggregate..."); 
        $sfo_data_aggrs[$count] =  NACL::STask::Aggregate->create(
            command_interface => $node,
            aggregate         => $opts{aggregate} . $count,
            if_exists         => 'reuse',
            diskcount         => $diskcount,
            raidtype          => $opts{raid_type_of_aggr},
        );
        $count++;
        $local_aggr_count--;
    }
    if ( defined ( $partner ) ) {
		while ($partner_aggr_count) {
			$Log->comment("Creating Data Aggregate...");
			$sfo_data_aggrs[$count] =  NACL::STask::Aggregate->create(
				command_interface => $partner, 
				aggregate         => $opts{aggregate} . $count,
				if_exists         => 'reuse',
				diskcount         => $diskcount,
				raidtype          => $opts{raid_type_of_aggr},
			);
			$count++;
			$partner_aggr_count--;
		}
    }
    #########################################################################
    # Ensure that volumes exists on each node, otherwise create given number 
    # of volumes on ecah aggregate.
    ##########################################################################
    ## If exists will be chaged to purge the volume
    my $vol_index = 0;
    foreach my $aggr ( @sfo_data_aggrs ) { 
        my $count = $vol_count;
        while ($count != 0 ) {
            my $vol_name = $opts{volume} . $vol_index;
            my $exception_object;
            LOOP: {
                try {
                    $sfo_data_vols[$vol_index] = NACL::STask::Volume->create(
                        command_interface => $node,
                        volume            => $vol_name,
                        aggregate         => $aggr,
                        vserver           => $sfo_vserver,
                        size              => $vol_size,
                        if_exists         => "reuse",
                        'junction-path'   => "/$vol_name",
                        'unix-permissions' => "777",
                    );
                } catch NACL::APISet::Exceptions::ResponseException with {
                    $exception_object = shift;
                    if ($exception_object->text() =~ 
                /Request to create volume failed because there is not enough space in the given aggregate/ ) 
                    {
                        ###  Add another aggregate and recreate the volume
                        try {
                            $aggr->add_disks(
                                diskcount          => "1",
                            );
                            redo LOOP;
                        } catch NACL::APISet::Exceptions::ResponseException with {
                            $exception_object = shift;
                            $Log->comment("Failed to add extra disk to aggregate, Cannot create new volume" 
                            . $exception_object->text() );
                            $vol_index--;
                        };
                    } 
                    else { 
                       $Log->exit();
                       $exception_object->throw(); 
                    }
                };
                ## Enable sis on all the volumes
                my $sis;
                if ( $enable_sis ) {
                    $sis =   NACL::STask::VolumeEfficiency->on( 
                        command_interface => $node,
                        volume            => $vol_name,
                        vserver           => $sfo_vserver,
                    );
                    $sis->start(nacltask_wait => '0');
               } 
            }
            $count--;
            $vol_index++;
        }
    }

    my $clients = get_client();
    my $num_client = scalar(@{$clients});
    my $num_vols = ( scalar ( @sfo_data_vols ));
    my $client_iterator = 0;
    my $lif_iterator = 0;
    my @mount_points;
    my $mount_status = 1;
    my $io_status = 0;
    my @proc_info;
    ## Using round robin method to mount volumes on data lifs and multiple clients
    for(my $index = 0 ; $index < $num_vols ; $index++) {

        $lif_iterator = 0 if ( $lif_iterator == scalar @sfo_data_lifs )  ;
        $client_iterator = 0 if ($client_iterator == $num_client );

        ## Get the IP adress of the lif.
        
        my $interface_state = $sfo_data_lifs[$lif_iterator]->state();
        my $data_ip = $interface_state->address();
        my $mount; 
		
		## Delete the stale mount point if it exists to make sure we will be able to mount.
		try { 
			my $vol = $sfo_data_vols[$index]->volume;
			my $mnt_pt = "/t/$data_ip/$vol";
			NACL::STask::Client::Mount->delete(
				command_interface=>$clients->[$client_iterator],
				mount_point => $mnt_pt,
			);
		};	
        try {
            $mount = NACL::STask::Client::Mount->create(
                command_interface=>$clients->[$client_iterator],
                server=>$data_ip,
                vserver => $sfo_vserver, 
                volume=> $sfo_data_vols[$index],
             );
        } catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $response         = $exception_object->get_response_object();
            $Log->comment( "Caught ResponseException in partner with"
                . $exception_object->text() );
            $mount_status = 0;
        };
        if ( $mount_status ) { 
            push(@mount_points , $mount);  
            my $mount_point = $mount->{mount_point};
            my $lif_mnt=
               $sfo_data_lifs[$lif_iterator].":"."/".$sfo_data_vols[$index];
            my $hostname = 
               $clients->[$client_iterator]->hostrec()->{hostname};
            if ( $opts{io_method} eq "hammer_mixed"){
            ###### ProcManager controls the execution of IO scripts. 
            ###### independent argument decides whether sub-units are independent
            ###### This module starts IO on all the volumes and exits the execution
            ###### Default IO duration is set to 80000, 1200 is used only for testing.
                push( @proc_info, 
                    { 
                        codespec => HAMMER ,
                        params => { 
                            CLIENT => $hostname , 
                            MOUNT_POINT => $mount_point , 
                            RUNTIME => $opts{io_duration},
							MODE => "mixed",
                        }
                    }
                );
            } ## End of hammer
			elsif( $opts{io_method} eq "hammer_overwrites"){
				push( @proc_info,
					{
						codespec => HAMMER ,
						params => {
							CLIENT => $hostname ,
							MOUNT_POINT => $mount_point ,
							RUNTIME => $opts{io_duration},
							MODE => "overwrites",
						}
					}
				);
			}
            elsif($opts{io_method} eq "sio_sequential_4k") {
                push( @proc_info, 
                    { 
                        codespec => SIO ,
                        params => { 
                            CLIENT => $hostname , 
                            MOUNT_POINT => $mount_point , 
                            RUNTIME => $opts{io_duration},
                            RAND => 0,
                            BLOCK_SIZE => "4k",
                        }
                    }
                );
            }
            elsif($opts{io_method} eq "sio_sequential_64k") {
                push( @proc_info, 
                    { 
                        codespec => SIO ,
                        params => { 
                            CLIENT => $hostname , 
                            MOUNT_POINT => $mount_point , 
                            RUNTIME => $opts{io_duration},
                            RAND => 0,
                            BLOCK_SIZE => "64k",
                        }
                    }
                );
            }
            elsif($opts{io_method} eq "sio_random_4k") {
                push( @proc_info, 
                    { 
                        codespec => SIO ,
                        params => { 
                            CLIENT => $hostname , 
                            MOUNT_POINT => $mount_point , 
                            RUNTIME => $opts{io_duration},
                            RAND => 100,
                            BLOCK_SIZE => "4k",
                        }
                    }
                );
            }
            elsif($opts{io_method} eq "sio_random_64k") {
                push( @proc_info, 
                    { 
                        codespec => SIO ,
                        params => { 
                            CLIENT => $hostname , 
                            MOUNT_POINT => $mount_point , 
                            RUNTIME => $opts{io_duration},
                            RAND => 100,
                            BLOCK_SIZE => "64k",
                        }
                    }
                );
            }
            elsif($opts{io_method} eq "sio_partial_sequential") {
                push( @proc_info, 
                    { 
                        codespec => SIO ,
                        params => { 
                            CLIENT => $hostname , 
                            MOUNT_POINT => $mount_point , 
                            RUNTIME => $opts{io_duration},
                            RAND => 0,
                            BLOCK_SIZE => "1k",
                        }
                    }
                );
            }
            elsif($opts{io_method} eq "sio_partial_random") {
                push( @proc_info, 
                    { 
                        codespec => SIO ,
                        params => { 
                            CLIENT => $hostname , 
                            MOUNT_POINT => $mount_point , 
                            RUNTIME => $opts{io_duration},
                            RAND => 100,
                            BLOCK_SIZE => "1k",
                        }
                    }
                );
            }
            elsif( $opts{io_method} eq "touch"){
				push( @proc_info,
					{
						codespec => CUSTOM ,
						params => {
							CLIENT => $hostname ,
							MOUNT_POINT => $mount_point ,
							RUNTIME => $opts{io_duration},
							TYPE => "touch",
						}
					}
				);
			}
            elsif( $opts{io_method} eq "truncate"){
				push( @proc_info,
					{
						codespec => CUSTOM ,
						params => {
							CLIENT => $hostname ,
							MOUNT_POINT => $mount_point ,
							RUNTIME => $opts{io_duration},
							TYPE => "truncate",
						}
					}
				);
			}
            elsif ($opts{io_method} eq "dt") {
                
            }### End of dt
            elsif ($opts{io_method} eq "populate"){
               $Log->comment("Not yet implemented");
           }### End of populate
           else {
                $Log->exit();
                NACL::Exceptions::InvalidChoice->throw("Invalid Takeover type");
           }### Invalid choice

        } ### end of if mount status
        $client_iterator++;
        $lif_iterator++;
    }## end of for
    
	# Create the ProcManager object finally.
    
    my $proc_manager = NATE::ProcManager->new( 
        proc_info => \@proc_info, 
        independent => 1 
    );
	$self->proc_manager($proc_manager);
}

sub start_traffic {
    my $self = shift;
    my $proc_manager = $self->proc_manager;
	$proc_manager->start;
}

sub stop_traffic {
    my $self = shift;
    my $proc_manager = $self->proc_manager;
	$proc_manager->stop;
    #$proc_manager->destroy;
    #$self->proc_manager(undef);
}

sub wait_traffic {
    my $self = shift;
    my $proc_manager = $self->proc_manager;
    $proc_manager->wait;
}

sub verify_traffic {
    my $self = shift;
    my $local = $self->local;
    my $partner = $self->partner;
    my $op1 = NACL::C::Statistics->show_periodic(
        command_interface => $local,
        iterations => 2,
    );
    my $op2 = NACL::C::Statistics->show_periodic(
        command_interface => $partner,
        iterations => 2,
    );
    if (!$op1->averages->nfs_ops || !$op2->averages->nfs_ops) {
        return 0;
    } else { 
        return 1;
    }
}

sub cleanup() {

    my $self  = shift;
    my %opts = validate(
        @_,
        {
            vserver => { type => SCALAR , optional=> 1 , default => SFO_VSERVER },
            aggregate => { type => SCALAR , optional=> 1 , default => SFO_AGGR },
            dont_cleanup_interface => { type => SCALAR, optional => 1, default => 1 },
        }
    );     
    my $sfo_vserver;
    my $sfo_vserver_root_vol;
    my @sfo_data_vols;
    my @sfo_aggrs;
    my @sfo_data_lifs;
    if ($opts{dont_cleanup_interface}) {
		$sfo_vserver = "vs0";
	} else {
		$sfo_vserver = SFO_VSERVER;
	}
    my $node = $self->local();
    my $clients = get_client();
    my $staf = NATE::STAF->new(global=>1);

    ### Get The vserver object
    $sfo_vserver = NACL::C::Vserver->find(
        command_interface => $node,
        filter => { vserver => $sfo_vserver },
    );
    my $volumes_exists = 1;
    ### Get the list of volumes in vserver
    try {
        @sfo_data_vols = NACL::STask::Volume->find(
            command_interface   => $node,
            filter              => {vsroot => 'false' ,vserver => $sfo_vserver},
        );
    } catch NACL::Exceptions::NoElementsFound with {
        $volumes_exists = 0;
        $Log->comment("Caught ResponseException in cleanup_funciton when getting volumes : No elements found \n ");
    };

    if ( $volumes_exists ) {
        foreach my $client ( @{$clients} ) {
            ## check if IO is running on given clients
            my $endpoint = $clients->[0]->hostrec()->{default_ip};
            my $client_apiset = $client->apiset( 
                                    category  => "Host",
                                    interface => "CLI",  
                                    set       => "Linux", 
            );
            my @IO_handles = @{$staf->handle_list(endpoint => $endpoint )}; 
            print Dumper(\@IO_handles);
            foreach my $handle (@IO_handles) { 
                if ($handle->{name} =~ /PM\d+\/\d+\_\s*/ ) {
                    my $io_name = $handle->{name};
                    $Log->comment("$io_name IO is still on progress, stop the IO");
                    my $pid = $handle->{pid};
                    $Log->comment("Killing the pid $pid ...........");
                    my $cmdstring = "/usr/software/bin/sudo kill -9 $pid";
                    $client_apiset->execute_raw_command(command=>$cmdstring);
                }
            } ## End for loop IO_handles
            ## Get the list of mount points mounted on the client.    
            ## Unmount all the volumes mounted on different clients
            my @mount_objs = NACL::STask::Client::Mount->find(command_interface => $client ); 
            foreach my $mount_obj (@mount_objs) {
                foreach my $vol ( @sfo_data_vols ) {
                    my $volume = $vol->volume();
	            if ( $mount_obj->mount_point() =~ /$volume/ ) {
	                $Log->comment("Unmounting the mount point" . $mount_obj->mount_point());
                    try {
                        $mount_obj->purge( nacltask_verify => 1, lazy => 1 );	
                        } #catch NACL::APISet::Exceptions::ResponseException with {
                         catch NACL::Exceptions::VerifyFailure with {
                             my $exception_object = shift;
                            #$Log->comment("Caught ResponseException in cleanup_funciton when deleting mount points on client: \n ".$exception_object->text() );
                            $Log->comment("Caught VerifyFailure Exception in cleanup_funciton when deleting mount points on client: \n ".$exception_object->text() );
                            #Execute raw command to unmount the mount point lazily..
                            my $umount_cmd = "/usr/software/bin/sudo umount -l $mount_obj->mount_point()";
                            $client_apiset->execute_raw_command(command=>$umount_cmd);
                        };
                    } ## End of If
                }## End of for each @sfo_data_vols
            }## End of for each @mount_objects
        } ## End of for each @clients
      
        ## Delete all data volumes      
        foreach my $vol ( @sfo_data_vols ) {
            my $name = $vol->volume();
            $Log->comment("Deleting volume $name");
            NACL::STask::Volume->purge(
                command_interface => $node,
                volume            => $vol,
                vserver           => $sfo_vserver,
            );
        } ## End of for loop, deleting SFO volumes
    } ## End if $volume_exists
    
    if (!($opts{dont_cleanup_interface})) {
        ## Get the list  of DATA lifs created for vserver
        my $data_lifs_exists = 1;
        try {
            @sfo_data_lifs = NACL::STask::NetworkInterface->find(
                command_interface => $node,
                filter => {vserver => $sfo_vserver, },
            );
        } catch NACL::Exceptions::NoElementsFound with {
            my $exception_object = shift;
            $data_lifs_exists = 0;
            $Log->comment("Caught ResponseException in cleanup_funciton when identyfying data lifs on filer: \n ". $exception_object->text());
        };

        if ( $data_lifs_exists ) {
            ## Get the default routing group created.
            my $routing_group = $sfo_data_lifs[0]->state()->routing_group();
            my $routing_group_obj;
            my $route_exists=1 ;
            $Log->comment("Deleting the default routing group of the interfaces $routing_group " );
            try {
                $routing_group_obj = NACL::C::NetworkRoutingGroupsRoute->find(
                    command_interface => $node,
                    filter            => {
                        server          => $sfo_vserver,
                        'routing-group' => $routing_group,
                    }
                );
            } catch NACL::Exceptions::NoElementsFound with {
                $route_exists = 0;
                $Log->comment(" Routing group $routing_group doesnot exists ");
            };
     
            $routing_group_obj->delete() if ( $route_exists );

            foreach my $lif (@sfo_data_lifs) {
                my $interface = $lif->lif(); 
                $Log->comment("Deleting the interface $interface");
                NACL::STask::NetworkInterface->purge(
                    command_interface => $node ,
                    lif               => $lif,
                    vserver           => $sfo_vserver,
                    nacltask_verify   => 1, # default is 0
                );
            } ## End of for each
        }

        #my $root_volume_name = $sfo_vserver_root_vol->volume();
        #$Log->comment("All the data volumes are deleted, Now deleting the vserver root volume $root_volume_name ");
       
        $sfo_vserver_root_vol  = NACL::STask::Volume->find(
            command_interface   => $node,
            filter              => {vsroot => 'true' ,vserver => $sfo_vserver},
        ); 
     
        NACL::STask::Volume->modify(
            command_interface=>$node, 
            volume  => $sfo_vserver_root_vol,
            vserver => $sfo_vserver,
            state => "offline",
        );

        NACL::STask::Volume->delete(
            command_interface=>$node,
            volume  => $sfo_vserver_root_vol,
            vserver => $sfo_vserver,
        );

        my $vserver = $sfo_vserver->vserver();
        $Log->comment("Deleting the vserver $vserver");
        NACL::C::Vserver->delete(
            command_interface => $node,
            vserver           => $sfo_vserver,
        );

    }
    ## Get the lists of  aggregate present on the filer 
    my $aggrs_exists = 1;
    try {
        @sfo_aggrs = NACL::C::Aggregate->find(
           command_interface => $node, 
           filter => { aggregate => $opts{aggregate} . "*" } , 
        );
    } catch NACL::Exceptions::NoElementsFound with {
        $aggrs_exists = 0;
        $Log->comment("No aggregate starting with name $opts{aggregate} found");
    };

    if ( $aggrs_exists ) {
        foreach my $aggr (@sfo_aggrs ) {
            my $aggr_name = $aggr->aggregate();
            $Log->comment(" Deleting Aggregate $aggr_name ..... ");
            try {
                NACL::STask::Aggregate->purge(
                    command_interface => $node,
                    aggregate         => $aggr,
                );
            }catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
            $Log->comment(
"Caught ResponseException in cleanup_funciton when deleting aggregate $aggr_name on filer: \n "
                  . $exception_object->text() );
            };
        }
    }

}

1;	
