#!/usr/bin/perl 
# Hey emacs, this is basically -*- perl -*-
#
#Copyright (c) 2011 NetApp Inc.
#All rights reserved.
#
## @summary HA compliance library
##
## @description
##
## $Id: //depot/prod/HA/HA_SFO/lib/ $
##
## These test functions use various commands to verify takeover and giveback.
##
##
## @usage
##   use HApackage;
##
## @keywords HApackage takeover giveback
## @dependencies
##
##
## @author dl-ha-qa
##
## @status public
##
package HApackage;

use strict;
use warnings;
use Params::Validate qw(validate SCALAR UNDEF BOOLEAN);
use NACL::ComponentUtils qw(_dump_one Dumper);
use Tharn qw($Log);
use NACL::CS::StorageFailover;
use NACL::STask::StorageFailover;
use NACL::STask::StorageDisk;
use NACL::Exceptions::NoElementsFound qw(:try);
use NACL::Exceptions::EventCheckFailure qw(:try);
use NACL::MTask::EventLogDetector;
use NACL::C::Node;
use NACL::STask::Aggregate;
use NACL::C::SystemNodeAutosupport;
use NACL::CS::SystemNodeAutosupport;
use NACL::C::Vserver;
use NACL::C::Client::Mount;
use NACL::STask::NetworkInterface;
use NACL::CS::Cluster;
use NACL::C::Client;
use NACL::CS::Volume;
use NACL::C::Volume;
use HAconstants;
use Switch;
use HAutils;
use NACL::Transit;



# Global Variables.
use vars qw( $result 
			@aggr_nonroot_existing
			@aggr_nonroot_created
			@aggr_list
			@vol_list
			$number_of_nodes
);

#exporting the variables
require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
			@aggr_nonroot_existing
			@aggr_nonroot_created
			@aggr_list
			@vol_list
			$number_of_nodes
);


#
# The constructor of  HApackage
#

sub new {
    my $class = shift;
    my $self  = {
        _local               => shift,
        _partner                 => undef,
        _local_aggr              => undef,
        _partner_aggr            => undef,
        _cluster_nodes           => undef,
        _auto_giveback           => undef,
        _takeover_detection_time => undef,
        _check_partner           => undef,
        _takeover_on_failure     => undef,
        _takeover_on_panic       => undef,
        _takeover_on_reboot      => undef,
        _autogiveback_delay      => undef,
        _hwassist_enabled        => undef,
        _abort_operations       => undef,
        _start_io                => "false",
        _type_of_takeover        => undef,
    };

    bless $self, $class;

    $self->{_local}     = NACL::C::Node->new( name => $self->{_local} );
    $self->{_partner}       = $self->partner();
    if($self->{_partner} == FAIL) {
        NATE::BaseException->throw("Unable to get partner node information");
    }

    $self->{_cluster_nodes} = $self->number_of_nodes_in_cluster();
    if($self->{_cluster_nodes} == FAIL) {
        NATE::BaseException->throw("Could not determine the number of nodes in the cluster");
    }

    ## Check if disk shelves are cross-connected to the filers
    my $check_connect_connect = $self->check_cross_connect();
    if($check_connect_connect == FAIL) {
        NATE::BaseException->throw("Disk shelves are NOT cross-connected");
    }
    ## Enable storage failover
    my $enable_sfo  = $self->enable_SFO(); 
    if($enable_sfo == FAIL) {
        NATE::BaseException->throw("Enabling storage failover failed");
    }
    ## Populate the list of aggr names into the HApackage object
    my $populate_aggr_list = $self->populate_aggr_list(); 
    if($populate_aggr_list == FAIL) {
        NATE::BaseException->throw("Unable to populate the list of aggrs");
    }

    return $self;
}

#This method will find the partner node of local instance.

sub partner() {
    my $self = shift;

    my $local_obj = $self->{_local};
    my $cs;
    $result = PASS;
    try {
        $cs = NACL::CS::StorageFailover->fetch(
            command_interface => $local_obj,
            requested_fields  => ['partner-name'],
            filter            => { node => $local_obj->node() }
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in partner with"
              . $exception_object->text() );
    };
    my $partner_name = $cs->partner_name();
    my $partner_node_obj = NACL::C::Node->new( name => $partner_name );
    return $partner_node_obj;
}

sub number_of_nodes_in_cluster() {

    my $self          = shift;
    my $local_obj = $self->{_local};
    my @nodes_in_cluster;
    try {
        @nodes_in_cluster =
          NACL::C::Node->find( command_interface => $local_obj );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $Log->comment(
            "Caught ResponseException in number_of_nodes_in_a_cluster with"
              . $exception_object->text() );
    };

    return scalar(@nodes_in_cluster);
}

sub reset_HA_flags {
    $Log->comment( "resetting HA flags using HApackage \n" );
    my $self = shift;
    if (@_) { $self->{_local} = shift }
    my $node = $self->{_local};
    $result = PASS;

    try {
        NACL::C::StorageFailover->modify(
            command_interface => $node,
            node              => $node,
            'auto-giveback'   => "false",
            'check-partner'  => "true", 
            'detection-time' => "15",
            'onfailure'      => "true",
            'onpanic'        => "false",
            'onreboot'       => "true",
            'abort-operations' => "false",
            'delay-seconds' => "300",
            #all hwassit flags are not yet implemeted in NACL::C::StorageFailover burt508171
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in reset_HA_flags with"
              . $exception_object->text() );

    };
    return $result;
}

sub enable_SFO() {
    $Log->comment( "enable SFO using HApackage \n" );
    my $self = shift;
    if (@_) { $self->{_local} = shift }
    my $node = $self->{_local};
    $result = PASS;
    try {
        NACL::STask::StorageFailover->enable(
            command_interface => $node,
            node              => $node,
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in enable_SFO with"
              . $exception_object->text() );

    };
    return $result;
}

sub disable_SFO() {
    $Log->comment( "disable SFO using HApackage \n" );
    my $self = shift;
    if (@_) { $self->{_local} = shift }
    my $node = $self->{_local};
    $result = PASS;
    try {
        NACL::STask::StorageFailover->disable(
            command_interface => $node,
            node              => $node,
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in disable_SFO with"
              . $exception_object->text() );

    };
    return $result;
}

sub check_cross_connect() {
    $Log->comment(
        "check cross connection of the disk shelves using HApackage \n" );
    my $self = shift;
    if (@_) { $self->{_local} = shift }
    my $ci      = $self->{_local};
    my $node    = $self->{_local}->node();
    my $partner = $self->{_partner}->node();
    $result = PASS;
    try {
        NACL::STask::StorageDisk->check_cross_connect(
            command_interface => $ci,
            node1             => $node,
            node2             => $partner,
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in check_cross_connect with"
              . $exception_object->text() );

    };
    return $result;
}

sub takeover() {
    $Log->comment( "Hapackage: Takeover routine start\n" );
    my $self    = shift;
    my $node    = $self->{_local};
    my $partner = $self->{_partner};

    my %opts = validate(
        @_,
        {
            takeovertype  => {type =>SCALAR, default => "takeover_command" },
            verify => {type =>SCALAR, default => YES },
            result => {type =>SCALAR, default => "pass"},
            option => {type =>SCALAR, default => "normal" },
        }
    );

    $result = PASS;
    my $state = NACL::CS::StorageFailoverInternalOptions->fetch(
             command_interface => $node,
             filter            => { 'node' => $node },
             requested_fields  => [qw(auto-giveback)],
        );
    my $auto_giveback = $state->auto_giveback();
    my $node_state;
    if ($auto_giveback eq 'false' ) {
        $node_state = "takeover";
    }else {
        $node_state = "takeover_autogiveback_scheduled";
    }
    my $partner_state = "nocheck";
    my $takeover_timeout = MAX_TAKEOVER_TIME_NORMAL; 
    my $owner_sfo_aggr = $node->node();
    my $owner_cfo_aggr = $node->node();
    my $ems_check = YES;

    # Constructing the EventLogDetector object
    my $detector = NACL::MTask::EventLogDetector->new(
        command_interface      => $node,
        check_for_all_presence => [
            'cf.fm.takeoverStarted', 'cf.fm.takeoverComplete',
            'cf.fm.takeoverDuration' 
        ]
    );

    # Calling start on EventLogDetector object
    $detector->start();

    if ( $opts{takeovertype} eq "takeover_command"){
                              if($opts{option} eq "normal" || 
                              $opts{option} eq "allow-version-missmatch" || 
                              $opts{option} eq "allow-disk-inventory-mismatch") {
                                  $takeover_timeout = MAX_TAKEOVER_TIME_NORMAL;
                              } else {
                                  $takeover_timeout = MAX_TAKEOVER_TIME_FORCED;
                              }
                              #perform negotiated takeover using NACL::STask::StorageFailover->takeover
                              try {
                                  NACL::C::StorageFailover->takeover(
                                                  command_interface => $node,
                                                  bynode            => $node,
                                                  option            => $opts{option},
                                  );
                              }
                              catch NACL::APISet::Exceptions::ResponseException with {
                                  my $exception_object = shift;
                                  my $response         = $exception_object->get_response_object();
                                  $result = FAIL;
                                  $Log->comment( "Caught ResponseException in takeover with"
                                     . $exception_object->text() );
                                  return $result;
                              };
    } ## end of negotiated takeover
    elsif ($opts{takeovertype} eq "halt")     { 
                              my $haltcmd;
                              ## Cmode halt command will be user after burt#518836 is resolved.
                              if ( $opts{option} eq "inhibit" ){
                                  $haltcmd = "set  -confirmations off ;system node run -node local -command halt -f -t 0 ";
                              }else{
	                          $haltcmd = "set  -confirmations off ;system node run -node local -command halt -t 0";
	                      }
                              my $transit_obj = NACL::Transit->new( name => $partner->hostrec() );
                              $transit_obj->change_state(
                                   to      => "FIRMWARE",
		                   haltcmd => $haltcmd,
                                   timeout => MAX_HALT_TIME,
                              );

                              if ( $opts{option} eq "inhibit") {
                                  $node_state = "partner_inhibit_takeover";
                                  $takeover_timeout = ONE_MIN;
                                  $owner_sfo_aggr = $partner->node();
                                  $owner_cfo_aggr = $partner->node();
                                  $ems_check = NO;
                              }
    }## end of takeovertype = halt
    elsif ($opts{takeovertype} eq "reboot")   { 
        if ( $opts{option} eq "inhibit" ){
            try {
                NACL::STask::StorageFailover->takeover(
                    command_interface => $node,
                    node              => $node,
                    partner           => $partner,
                    takeovertype      => "reboot",
                    nacltask_inhibit_takeover => YES,
                    nacltask_expect_takeover => NO,
                    'method-timeout'  => TAKEOVER_COMMAND_TIMEOUT,
                );
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
                my $response         = $exception_object->get_response_object();
                $Log->comment("Caught exception in reboot inhibit Takeover : "
                 . $exception_object->text() );
                return FAIL;
           };
           ## Verification details
           $node_state = "up";
           $partner_state = "up";
           $owner_sfo_aggr = $partner->node();
           $owner_cfo_aggr = $partner->node();
           $takeover_timeout = MAX_REBOOT_TIME ;
           $ems_check = NO;

       } else {
           try {
               NACL::STask::StorageFailover->takeover(
                   command_interface => $node,
                   node              => $node,
                   partner           => $partner,
                   takeovertype      => "reboot",
                   nacltask_inhibit_takeover => NO,
                   nacltask_expect_takeover => YES,
                   'method-timeout'  => TAKEOVER_COMMAND_TIMEOUT,
               );
           }
           catch NACL::APISet::Exceptions::ResponseException with {
               my $exception_object = shift;
               my $response         = $exception_object->get_response_object();
               $Log->comment("Caught exception in reboot inhibit Takeover : "
               .$exception_object->text() );
               return FAIL;
           };
           ## Verification details
           $node_state = "takeover_autogiveback_scheduled";
       }
    }## end of takeovertype = reboot
    elsif ($opts{takeovertype} eq "panic")    { 
        my $failover_options = NACL::CS::StorageFailoverInternalOptions->fetch(
             command_interface => $node,
             filter            => { 'node' => $node },
             requested_fields  => [qw(onpanic)],
        );
        try{ 
             NACL::STask::StorageFailover->takeover(
                 command_interface => $node,
                 node              => $node,
                 partner           => $partner,
                 'takeovertype'    => "panic",
                 nacltask_verify   => 0,
                 'method-timeout'  => MAX_PANIC_HALT_TIME,
             );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $response         = $exception_object->get_response_object();
            $result = FAIL;
            $Log->comment( "Caught ResponseException in takeover due to panic with "
            . $exception_object->text() );
            return $result;
        };
        if ( $failover_options->onpanic() =~ /false/i ){
            $node_state = "up";
            $partner_state = "up";
            $owner_sfo_aggr = $partner->node();
            $owner_cfo_aggr = $partner->node();
            $takeover_timeout = MAX_REBOOT_TIME ;
            $ems_check = NO;
        }
    } ## end of takeovertype = panic
    else {
        $Log->comment("$opts{takeover_type} is Not yet implemented"); 
        return FAIL;
    }
    ## verify takeover has completed on node
    if ($opts{verify} == YES) {
        $result &= $self->wait_for_state(
                  node_state => $node_state,
                  partner_state => $partner_state,
                  polling_interval => POLLING_INTERVAL,
                  'method-timeout' => $takeover_timeout,
        );
        ## Wait for 30 seconds before we check aggregate ownership.
        sleep 30;
        $result &= $self->verify_aggr_owner(node => $partner->node(), owner_sfo_aggr => $owner_sfo_aggr,
                                                owner_cfo_aggr => $owner_cfo_aggr ); 
      
        # Stop and check for presence of all the event messagenames
        my $ems_status = PASS;
        if ( $ems_check ) {
            try {
                $detector->stop( 'check' => 1 );
            }
            catch NACL::Exceptions::EventCheckFailure with {
                my $exception_obj = shift;
                $ems_status = FAIL;
                $Log->comment(
                   "Test failed with the error: \n" . $exception_obj->text() );
                $Log->comment( "Unmatched events are :\n"
                   . @{ $exception_obj->unmatched_events() } );
            };  
            $result &= $ems_status ; 
            $Log->comment( "Print the event messages");
            $result &= print_ems_messages(node => $node , 
                                          event_object => $detector ,  
                                          event =>  "cf.fm.takeover*" );

            $Log->comment("Verify the takeover duration");
            $result &= verify_takeover_giveback_duration(event_object => $detector );
   
         } else {
	    ## EMS verification is not required, just stop the detector
            $detector->stop('check' => 0 );
         }### Enf if $ems_check
    }else {
        ## Stop the event detector here , if verification is not required
	$detector->stop('check' => 0 );
    }
    return $result;
}

                                                                                                                                                              
##    HApackage_obj->giveback(
##          command_interface     => $command_interface,
##          post_giveback_action       => $post_giveback_action,
##          post_giveback_action_delay => $post_giveback_action_delay,
##      );
##
##    HApackage_obj->giveback();
##
##    (Class method) Initiate giveback. This method provides additional services that are inherited by using NACL::STask::StorageFailover
##    to perform giveback with post giveback action. The post giveback action will be passed on to the method by the user.  The user also passes the post giveback action delay that will be the delay between giveback and  post giveback action.
##    If post giveback action is provided, the action is performed after post giveback action delay time else perform giveback with no post giveback action. If  the post giveback action delay is not given, perform post giveback action after default delay time of 300 seconds.
##
##    Currently will support only C-mode.
##
##    Options
##    node=>$node
##    (Optional) Node initiating giveback. Specifies the node that currently holds the storage that is to be returned to the partner node.
##    partner=>$partner
##    (Optional) Node to which control is givenback. Specifies the node whose storage is currently taken over by its partner and will be given back by this giveback operation.
##    This can be discovered if not provided.
##    post_giveback_action => $post_giveback_action
##    (Optional) Specify the post giveback action
##    post_giveback_action_delay => $post_giveback_action_delay
##    (Optional) Specify the post giveback action delay time in seconds, wait time to perform post giveback action after giveback. Default: 300
##
##    Assumptions
##    All assumptions will be related to NACL::STask::StorageFailover

sub giveback() {
    $Log->comment( "perform giveback using HApackage \n" );
    my $self    = shift;
    my $node    = $self->{_local};
    my $partner = $self->{_partner};

    my %opts = validate(
        @_,
        {
            post_giveback_action => { type => SCALAR | UNDEF, optional => YES },
            post_giveback_action_delay =>
              { type => SCALAR | UNDEF, optional => YES },
            'require-partner-waiting' =>
              { type => SCALAR | UNDEF, optional => 1 },
            'override-vetoes'     => { type => SCALAR | UNDEF, optional => 1 },
            'only-cfo-aggregates' => { type => SCALAR | UNDEF, optional => 1 },
            nacltask_verify       => { type => SCALAR, default  => 1 },
            verify_ems            => { type => SCALAR, default  => 1 },
            'wait-for-asup-generation' =>
              { type => SCALAR | UNDEF, optional => 1, default => 0 },
            'failure-reason' => { type => SCALAR, optional => 1 },
            auto_giveback_expected => { type => BOOLEAN, default => 0 },
            wait_for_partner => {type => BOOLEAN, default => 1 },
        }
    );

    $result = PASS;
    my $owner_sfo_aggr;
    my $owner_cfo_aggr;
    my $verify_ems = delete $opts{verify_ems};
    

    # Constructing the EventLogDetector object
    my $detector = NACL::MTask::EventLogDetector->new(
        command_interface      => $node,
        check_for_any_presence => [
            'cf.fm.givebackStarted', 'cf.fm.givebackComplete',
            'cf.fm.givebackDuration'
        ]
    );

    # Calling start on EventLogDetector object
    $detector->start();

    my $retry_giveback_waiting_for_asup = 0;

    #perform giveback using NACL::STask::StorageFailover->giveback

    try {
        NACL::STask::StorageFailover->giveback(
            command_interface => $node,
            node              => $node,
            partner           => $partner,
            %opts,
        );
    } catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
	    $Log->comment( "Caught ResponseException in HApackage giveback with "
              . $exception_object->text() );
    }; 
    if ($opts{nacltask_verify} == 1) {
        if ( $verify_ems == 1 ) {
            my $ems_status = PASS;
            try {
                $detector->stop( 'check' => 1 );
            }
            catch NACL::Exceptions::EventCheckFailure with {
                my $exception_obj = shift;
                $ems_status = FAIL;
                $Log->comment(
                   "Test failed with the error: \n" . $exception_obj->text() );
                $Log->comment( "Unmatched events are :\n"
                   . @{ $exception_obj->unmatched_events() } );
            };
            $result &= $ems_status ;
            $Log->comment( "Print the event messages");
            $result &= print_ems_messages(node => $node ,
                              event_object => $detector ,
                              event =>  "cf.fm.giveback*" );
            $Log->comment("Verify the takeover duration");
            $result &= verify_takeover_giveback_duration(event_object => $detector );
        }
        else {
	    $detector->stop( 'check' => 0 );
        }
        ## Sleep 30 seconds before verifying aggregate ownerships.
        sleep 30;
        if ( defined $opts{'only-cfo-aggregates'} ) {
            $owner_sfo_aggr = $node->node();
            $owner_cfo_aggr = $partner->node();
            $result &= $self->verify_aggr_owner(node => $partner->node(), owner_sfo_aggr => $owner_sfo_aggr,
                                                owner_cfo_aggr => $owner_cfo_aggr );
        }else {
            $owner_sfo_aggr = $partner->node();
            $owner_cfo_aggr = $partner->node();
            $result &= $self->verify_aggr_owner(node => $partner->node(), owner_sfo_aggr => $owner_sfo_aggr,
                                                owner_cfo_aggr => $owner_cfo_aggr );
        }
    }
    else {
        $detector->stop( 'check' => 0 );
    }
    if ( $opts{post_giveback_action} ) {
        my $msg = 'post giveback action is currently not supported';
        $Log->exit();
        NACL::Exceptions::InvalidChoice->throw($msg);
    }
    return $result;
}

## Wait for the filers to reach particular state
sub wait_for_state { 
    my $self    = shift;
    my $node    = $self->{_local};
    my $partner = $self->{_partner};
    my $nodename = $node->node();
    my $partnername = $partner->node();

    my %opts = validate(
        @_,
        {
            node_state => { type => SCALAR },
            partner_state => { type => SCALAR },
	    polling_interval => {type =>SCALAR , OPTIONAL => 'yes', default => 10 },
	    'method-timeout' => {type =>SCALAR },
        }
    );

    $result = PASS;
    my %state_hash ;
    if ($opts{partner_state} eq "nocheck" ) {
       $state_hash{node_state} = $opts{node_state} ;
    }else {
        $state_hash{node_state} = $opts{node_state} ;
        $state_hash{partner_state} = $opts{partner_state};
    }
    ## call wait_for_state from STask::StorageFailover
    try {
        NACL::STask::StorageFailover->wait_for_state(
                command_interface     => $node,
                node                  => $node,
                partner               => $partner,
                'method-timeout' => $opts{'method-timeout'},
                %state_hash,
        );
    }catch NACL::APISet::Exceptions::TimeoutException with {
        my $exception_object = shift;
        $Log->comment( "Caught ResponseException in wait_for_state with "
        . $exception_object->text() );
        $result = FAIL;
     };

    return $result;

}## end wait_for_state


sub set_HA_flags{
    $Log->comment("setting HA flags using HApackage \n");
    my $self = shift;
    my $node = $self->{_local};

    my %opts = validate(
        @_,
        {
            'auto-giveback'  => { type => SCALAR, optional => 1 },
            'check-partner'  => { type => SCALAR, optional => 1 }, 
            'detection-time' => { type => SCALAR, optional => 1 },
            'onfailure'      => { type => SCALAR, optional => 1 },
            'onpanic'        => { type => SCALAR, optional => 1 },
            'onreboot'       => { type => SCALAR, optional => 1 },
            'delay-seconds'  => { type => SCALAR, optional => 1 },
            'hwassist'       => {type  => SCALAR, optional => 1 }, 
            'abort-operations' => { type => SCALAR, optional => 1 },
        }
    );

    $result = PASS;
    try {
        NACL::C::StorageFailover->modify(
            command_interface => $node,
            node              => $node,
            %opts,
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in set_HA_flags with"
              . $exception_object->text() );
    };
    return $result;

}

sub get_HA_flags {
    $Log->comment("Get HA flags using HApackage \n");
    my $self = shift;
    if (@_) { $self->{_local} = shift }
    my $node = $self->{_local};
    my @flags;
    $result = PASS;

    try {
        @flags = NACL::CS::StorageFailover->fetch(
            command_interface => $node,
            requested_fields  => [
                qw(auto-giveback check-partner detection-time onfailure onpanic onreboot abort-operations delay-seconds)
            ],

            #TODO: add hwassit flag when burt resolved, burt518400

        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in reset_HA_flags with"
              . $exception_object->text() );
    };
    if ( $result != 1 ) {
        return $result;
    }
    else { return @flags; }
}



sub confirm_HA_config {
    $Log->comment("Confirm that the nodes are in HA config \n");
    my $self = shift;
    if (@_) { $self->{_local} = shift }
    my $node = $self->{_local};
    $result = PASS;

    try {
        my $storage_failover_connected = NACL::CS::StorageFailover->fetch(
            command_interface => $node,
            filter            => { 'node' => $node },
            requested_fields  => [qw(state)],
        );
        if ( $storage_failover_connected->state() =~ m/connected/ ) {
            $Log->comment("The nodes are connected to their partner\n ");
        }
        else {
            $result = FAIL;
            $Log->comment("The nodes are not in connected state \n ");
        }
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment(
"Caught ResponseException in confirm_HA_config when finding the connectivity of the node with its  partner with: \n "
              . $exception_object->text() );
    };
    return $result;
}

#Used to compare HA flags before and after TOGB or any tests.
sub compare_HA_flags {	
	shift;
	my ($HA_flag_before_test, $HA_flag_after_test) = @_;
    $Log->comment("comparing HA flags using HApackage \n");
	my @diffFlags = ();
	for my $key (keys %{$HA_flag_after_test}) {
		if (${$HA_flag_before_test}{$key} ne ${$HA_flag_after_test}{$key}) {
			print "Keys not equal";
			push(@diffFlags, $key)
		}
	}
	return @diffFlags;
}

###############################################################################
# Method: HA_cleanup
# Objective: To bring the nodes to up state, when it is in wfg or loader prompt
# Details:   This method checks the state of both the nodes.
# If the nodeB is in wfg, issue giveback from nodeA
# If the nodeB is in loader prompt, issues bootontap from nodeA
# Resetting all the HA flags
# If aggregate was created during setup, deleting them
# Verifying the number of nodes in the cluster is same as it was during setup
# Checking the health of the nodes
# Enabling SFO before exit
###############################################################################

sub HA_cleanup() {

    $Log->comment("In Cleanup Function \n");
    my $self = shift;
    my $status_local_node;
    my $status_partner_node;
    my $return_val;
    my $giveback_val;
    my $nodename;

    #---------------------------------------------
    # Checking the state of local and partner node
    #---------------------------------------------
    my @sfo = NACL::CS::StorageFailover->fetch(
                                      command_interface => $self->{_local},
                                      requested_fields => [qw(node partner-name state-description)],
    );
    my $local_node   = $self->{_local}->name();
    my $partner_node = $self->{_partner}->name();
    my $node         = $self->{_local};
    $result = PASS;

    foreach my $sf (@sfo) {
        $nodename = $sf->node();
        if ($nodename eq $local_node) {
            $status_local_node = $sf->state_description();
        } else {
            $status_partner_node = $sf->state_description();
        }
    }
    $Log->comment(" Status of the local node is $status_local_node ");
    $Log->comment(" Status of the partner node is $status_partner_node");

    #---------------------------------------------------
    # Check the status of the nodeA is in takeover mode
    #---------------------------------------------------
    if ($status_local_node =~ /Auto giveback will be initiated/) {

       $return_val = $self->wait_for_state(node_state       => "up",
                              partner_state    => "up",
                              'method-timeout' => MAX_REBOOT_TIME);
       $result = FAIL if($return_val == FAIL);

    } elsif (   $status_local_node =~ /In takeover/
        or $status_local_node =~ /Previous giveback failed in module/) {
        $Log->comment(" Filer $nodename is in takeover mode ");
        $Log->comment(" Check the partner state ");
        $self->set_HA_flags('auto-giveback' => "false");
        $self->wait_for_state(node_state       => "takeover",
                              partner_state    => "nocheck",
                              'method-timeout' => MAX_WFG_TIME);

        #-------------------------------------------------------
        #Check the status of nodeB is in Loader prompt or in WFG
        #-------------------------------------------------------
        if ($status_partner_node =~ /Unknown/) {
            $Log->comment("Filer $partner_node is not in WGF, bring the filer to wfg");
            boot_ontap(node => $self->{_partner});
            $return_val = $self->wait_for_state(node_state       => "takeover",
                                                partner_state    => "wfg",
                                                'method-timeout' => MAX_WFG_TIME);
        }
        $giveback_val = $self->giveback();
        $result = FAIL if ($giveback_val == FAIL);

    } elsif ($status_local_node =~ /Connected/) {
        $result = PASS;
    } else {
        return FAIL;
    }

    #---------------------------
    #resetting all the HA flags
    #---------------------------
    $Log->comment("Resetting all HA flags during cleanup \n ");
    $self->reset_HA_flags();

    #-------------------------------------------------------------------------------
    #if aggregate was created during setup, deleting them only when AGGR_CLEANUP = 1
    #-------------------------------------------------------------------------------
    my @sfo_aggr_list = ();
    try {
        @sfo_aggr_list =
          NACL::CS::StorageAggregate->fetch(
                           command_interface => $self->{_local},
                           filter => {'home-name' => $self->{_local}, 'ha-policy' => 'sfo', state => 'online'}
          );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment(
                     "Caught ResponseException in cleanup_function when fetching aggregate with: \n"
                       . $exception_object->text());
    };
    if ($::AGGR_CLEANUP eq '1') {
        foreach my $aggr (@sfo_aggr_list) {
            if ($aggr =~ /^(SFO_AGGR).*/) {
                $Log->comment("Deleting aggregate: $aggr");
                try {
                    NACL::STask::Aggregate->purge(command_interface => $node,
                                                  aggregate         => $aggr,);
                }
                catch NACL::APISet::Exceptions::ResponseException with {
                    my $exception_object = shift;
                    my $response         = $exception_object->get_response_object();
                    $result = FAIL;
                    $Log->comment(
                        "Caught ResponseException in cleanup_funciton when purging aggregate with: \n"
                          . $exception_object->text());
                };
            } ## end if ($aggr =~ /^(SFO_AGGR).*/)
        } ## end foreach my $aggr (@sfo_aggr_list)
    } ## end if ($::AGGR_CLEANUP eq...

    #---------------------------------------------------------------------------
    #verifying the number of nodes in the cluster is same as it was during setup
    #---------------------------------------------------------------------------
    $Log->comment("Checking the number of nodes in the cluster as part of cleanup \n ");

    my $nodes_in_cluster = $self->number_of_nodes_in_cluster();
    if ($nodes_in_cluster != $number_of_nodes) {
        $Log->comment(
            "There are $nodes_in_cluster nodes in the cluster but $number_of_nodes nodes were found during setup"
        );
        $result = FAIL;
    } else {
        $Log->comment("Verified that the number of nodes in the cluster is: $nodes_in_cluster \n");
    }

    #---------------------------------
    #checking the health of the nodes
    #---------------------------------
    $Log->comment("Checking the health of all the nodes in the cluster as part of cleanup \n ");
    try {
        my $cluster_health_flag =
          NACL::CS::Cluster->fetch(command_interface => $node,
                                   requested_fields  => [qw(health)],);
        if ($cluster_health_flag->health() =~ /false/) {
            $Log->comment("The nodes look unhealthy \n ");
            $result = FAIL;
        } else {
            $Log->comment("Verified healthy nodes as part of cleanup\n ");
        }
    } ## end try

    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment(
            "Caught ResponseException in cleanup_funciton when finding the health of the cluster with: \n "
              . $exception_object->text());
    };

    #------------------------
    #enabling SFO before exit
    #------------------------
    $Log->comment("Enable Storage Failover as part of cleanup \n ");
    $self->enable_SFO();
    return $result;
} ## end sub HA_cleanup()


sub create_aggr() {
    $Log->comment("creating aggregate using HApackage \n");
    my $self = shift;
    if (@_) { $self->{_local} = shift }
    my $node = $self->{_local};
    $result = PASS;
    my $rand = int( rand(RANGE) );

    try {
        NACL::STask::Aggregate->create(
            command_interface => $node,
            aggregate         => SFO_AGGR . "$rand",
            if_exists         => 'reuse',
            diskcount         => DISKCOUNT,
        );

    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in create_aggr with: \n"
              . $exception_object->text() );

    };
    return $result;
}

sub delete_aggr() {
    $Log->comment("deleting aggregate using HApackage \n");
    my $self = shift;
    my $node = $self->{_local};
    my %opts = validate( @_, { aggregate => { type => SCALAR }, } );
    $result = PASS;
    try {
        NACL::STask::Aggregate->purge(
            command_interface => $node,
            aggregate         => $opts{aggregate},
        );

    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in delete_aggr with: \n"
              . $exception_object->text() );
    };
    return $result;
}

sub enable_disable_ausp {
    my $self    = shift;
    my $node    = $self->{_local};

    my %opts =
      validate( @_,
        { autosupport => { type => SCALAR, default => "enable" }, } );
    $result = PASS;
    if ( $opts{autosupport} eq "enable" ) {
        $Log->comment("Trying to enable autosupport\n");
    }
    else {
        $Log->comment("Trying to disable autosupport\n");
    }
    try {
        NACL::C::SystemNodeAutosupport->modify(
            command_interface => $node,
            node              => $node,
            state             => $opts{autosupport},
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment(
            "Caught ResponseException in enable_disable_asup for $node with: \n"
              . $exception_object->text() );
    };
    if ( $result =~ m/FAIL/) {
        return $result;
    }
    return $result;
}


#get all the non root aggr
#get all the non root vols
#get the number of nodes in the cluster
#confirm it is a HA_config
#turn AUTOBOOT as false

sub HA_setup() {
    $Log->comment("In Setup Funtion \n");

    my $self = shift;
    my $node = $self->{_local};
    my %opts =
      validate( @_,
        { autoboot_value => { type => SCALAR, default => "true" }, } );
    $result = PASS;

	# get all the non-root aggr
    # if non exist, create
    try {
        @aggr_nonroot_existing = NACL::C::Aggregate->find(
            command_interface => $node,
            filter            => { root => "false" },
        );
    }
    catch NACL::Exceptions::NoElementsFound with {
        $Log->comment(
"Caught ResponseException: no non-root aggr found, so creating it \n"
        );
        $self->create_aggr();
        @aggr_nonroot_created = NACL::C::Aggregate->find(
            command_interface => $node,
            filter            => { root => "false" },
        );
    };

    #get the aggr list to be used for verification after takeover and giveback
    try {
        $Log->comment("Getting the aggregate list\n");
        @aggr_list = NACL::C::Aggregate->find( command_interface => $node, );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment(
            "Caught ResponseException when getting aggregate list: \n "
              . $exception_object->text() );
    };
    if ( $result != 1 ) {
        return $result;
    }

    #get the vol list
    try {
        $Log->comment("Getting the volume list\n");
        @vol_list = NACL::C::Volume->find( command_interface => $node, );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException when getting volume list: \n "
              . $exception_object->text() );
    };
    if ( $result != 1 ) {
        return $result;
    }

    #find the number of nodes in the cluster during setup
    $Log->comment("Finding the number of nodes in the cluster in HA_setup\n");
    $number_of_nodes = $self->{_cluster_nodes};
    $Log->comment("There are $number_of_nodes nodes in the cluster \n");

    #confiming the cluster as HA config
    $Log->comment("Confirming that the cluster is a HA config in HA_setup\n");
    $result = $self->confirm_HA_config();
    if ( $result != 1 ) {
        $Log->comment("The cluster is not a HA config \n");
        return $result;
    }
    else {
        $Log->comment("Confirmed that the cluster is a HA config\n");
    }

    #set AUTOBOOT during setup,
    $Log->comment("Setting AUTOBOOT in HA_setup\n");
    if ( $opts{autoboot_value} ) {
        $result = set_bootarg_autoboot(
            node           => $node,
            autoboot_value => $opts{autoboot_value}
        );
    }
    if ( $result != 1 ) {
        $Log->comment("Unable to set AUTOBOOT to $opts{autoboot_value} \n");
        return $result;
    }
    return $result;
}

sub verify_aggr_owner()
{
    my $self = shift;
    my %opts = validate(
        @_,
        {
            node             => {type => SCALAR},
            owner_sfo_aggr   => {type => SCALAR},
            owner_cfo_aggr   => {type => SCALAR},
        }
    );

    my @aggr_list;
    my $result = PASS;

    ## assign aggrs to list based on whether they we need local or partner nodes aggrs
    if ($opts{node} eq $self->{_local}->node()) {
        @aggr_list = @{ $self->{_local_aggr} };
    } elsif ($opts{node} eq $self->{_partner}->node()) {
        @aggr_list = @{ $self->{_partner_aggr} }
    } else {
        $Log->comment("Invalid node");
        return FAIL;
    }

    ## iterate through all aggr in the list
    foreach my $aggr (@aggr_list) {
        my $aggr_obj;
        try { 
           $aggr_obj = NACL::CS::Aggregate->fetch(command_interface => $self->{_local},
                       filter => { aggregate => $aggr},
                       requested_fields => [ qw(aggregate nodes ha-policy state)]);
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $response         = $exception_object->get_response_object();
            $result &= FAIL;
            $Log->comment( "Caught ResponseException when getting aggr list: \n "
                  . $exception_object->text() );
        };

        my $state = $aggr_obj->state();
        my $ha_policy = $aggr_obj->ha_policy();
        my @nodes = $aggr_obj->nodes();

        ## If it is an sfo aggr
        if ($ha_policy eq "sfo") {
            ## verify ownership of aggr
            if ($nodes[0] ne $opts{owner_sfo_aggr}) {
                    $Log->comment("Owner of aggr $aggr not as expected");
                    $Log->comment("Expected value = $opts{owner_sfo_aggr}, actual value = $nodes[0]");
                    return FAIL;
            }
        }
        ## If it is an cfo aggr
        if ($ha_policy eq "cfo") {
            ## verify ownership of aggr
            if ($nodes[0] ne $opts{owner_cfo_aggr}) {
                    $Log->comment("Owner of aggr $aggr not as expected");
                    $Log->comment("Expected value = $opts{owner_cfo_aggr}, actual value = $nodes[0]");
                    return FAIL;
            }
        }

        ## verify whether aggr is online
        if ($state ne "online") {
            $Log->comment("Aggr $aggr not online");
            return FAIL;
        }

    } ##end of foreach

    ## verify whether the volumes are online
    my @volumes;
    try {
        @volumes = NACL::CS::Volume->fetch(command_interface => $self->{_local},
                      requested_fields => [ qw(volume state)]);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result &= FAIL;
        $Log->comment( "Caught ResponseException when getting aggr list: \n "
                  . $exception_object->text() );
    };

    foreach my $volume (@volumes) {
        my $vol_state = $volume->state();
        my $vol_name = $volume->volume();
        if ($vol_state ne "online") {
            $Log->comment("Volume $vol_name not online");
            return FAIL;
        }
    }

    return $result;
}


sub populate_aggr_list()
{
    my $self = shift;
    my @agr_list;
    try {
        @aggr_list = NACL::CS::StorageAggregate->fetch(command_interface => $self->{_local},
                    requested_fields => [ qw(aggregate nodes)]);
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result &= FAIL;
        $Log->comment( "Caught ResponseException when getting aggr list: \n "
                  . $exception_object->text() );
    };

    my @local_aggr = ();
    my @partner_aggr = ();
    ## iterate through all aggrs in the list
    foreach my $aggr (@aggr_list) {
        my @nodes = $aggr->nodes();
        my $aggr_name = $aggr->aggregate();
        ## add aggrs to local aggr list
        if($nodes[0] eq $self->{_local}->node()) {
            push(@local_aggr,$aggr_name);
        }
        ## add aggrs to partner aggr list
        if($nodes[0] eq $self->{_partner}->node()) {
            push(@partner_aggr,$aggr_name);
        }
    }
    $self->{_local_aggr} = \@local_aggr ;
    $self->{_partner_aggr} = \@partner_aggr ;

    return $result;
}


1;

