#!/usr/bin/perl
# Hey emacs, this is basically -*- perl -*-
#
#Copyright (c) 2011 NetApp Inc.
#All rights reserved.
#
## @summary HA compliance library
##
## @description
##
## $Id: //depot/prod/test/main/HA/HA_SFO/Sandbox/lib/HAutils.pm $
##
## These test functions use various commands to verify takeover and giveback.
##
##
## @usage
##   use HAutils;
##
## @keywords HA utily methods
## @dependencies
##
##
## @author dl-ha-qa
##
## @status public
##

package HAutils;

use vars qw( $result
                        $number_of_nodes

);

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
              set_bootarg_autoboot
              system_halt
              boot_ontap
              system_reboot
              get_client 
              get_bootarg_autoboot
              print_ems_messages
              verify_takeover_giveback_duration
              disk_fail
              disk_unfail
              get_aggr_vol
              set_loader_variable
	      get_system_id
              disk_remove_ownership
              convert_to_ha_mode
	      convert_to_non_ha_mode
              HA_setup
              HA_cleanup
              number_of_nodes_in_cluster
              confirm_HA_config
              set_HA_flags
              ha_config_show
              ha_config_modify
	      compare_HA_flags
              restore_to_pre_setup
              HA_Timesync
              HWassist_setup
              Verify_HWassist
              arl_cleanup
              get_aggr_list_to_relocate
);

use strict;
use warnings;
use NACL::APISet;
use Tharn qw($Log);
use Params::Validate qw(:all);
use NACL::C::SystemNodeAutosupport;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::Transit;
use Data::Dumper;
use HAconstants;
use NACL::STask::StorageDisk;
use NACL::CS::StorageAggregate;
use NACL::CS::Eventlog;
use NACL::CS::SystemNodeServiceProcessorNetwork;
use NACL::CS::StorageFailoverHwassist;
use NACL::STask::Node;
use NACL::STask::StorageFailover;
use NACL::MTask::HA;
use NACL::MTask::IPv6;
use NACL::MTask::StorageAggregateRelocation;
use NACL::STask::Aggregate;
use NACL::C::StorageFailoverHwassistStats;
use NACL::C::StorageFailoverHwassist;
use NACL::C::SystemNodeServiceProcessorNetwork;
#get all the non root aggr
#get all the non root vols
#get the number of nodes in the cluster
#confirm it is a HA_config
#turn AUTOBOOT as false

sub HA_setup {
    $Log->comment("In Setup Funtion \n");

    my %opts = validate(
        @_,
        {   ha_obj => {type => OBJECT , isa => "NACL::MTask::HA" },
            autoboot_value  => {type =>SCALAR , default => "true" },
        }
    );

    my $node = $opts{ha_obj}->node();
    my $partner      = NACL::STask::Node->get_partner_obj(
                           command_interface => $node,
                           node => $node);
    my $local_node   = $node->name();
    my $partner_node = $partner->name();
 
    $result = PASS;

    # find the number of nodes in the cluster during setup
    $Log->comment("Finding the number of nodes in the cluster in HA_setup\n");
    $number_of_nodes = number_of_nodes_in_cluster(node => $node);
    $Log->comment("There are $number_of_nodes nodes in the cluster \n");


    foreach $node ($partner,$node) {
        # get all the non-root aggr
        # if non exist, create
        try {
            my @aggr_nonroot_existing = NACL::C::Aggregate->find(
                command_interface => $node,
                filter => {'home-name' => $node, root => "false", 'ha-policy' => 'sfo'}
            );
        }
        catch NACL::Exceptions::NoElementsFound with {
            $Log->comment(
            "Caught ResponseException: no non-root aggr found, so creating it \n"
            );
            my $rand = int( rand(RANGE) );
            try {
                NACL::STask::Aggregate->create(
                    command_interface => $node,
                    aggregate         => SFO_AGGR . "$rand",
                    if_exists         => 'reuse',
                    diskcount         => DISKCOUNT,
                    "method-timeout" => 7200
                );
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
                my $response         = $exception_object->get_response_object();
                $result = FAIL;
                $Log->comment( "Caught ResponseException in create_aggr with: \n"
                    . $exception_object->text() );
            };
            my @aggr_nonroot_created = NACL::C::Aggregate->find(
                command_interface => $node,
                filter            => { root => "false" },
            );
        };
        if ( $result != 1 ) {
            return $result;
        }

        # get the vol list
        try {
            $Log->comment("Getting the volume list\n");
            my @vol_list = NACL::C::Volume->find( command_interface => $node, );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $response         = $exception_object->get_response_object();
            $result = FAIL;
            $Log->comment( "Caught ResponseException when getting volume list: \n "
              . $exception_object->text() );
        };
        if ( $result != 1 ) {
            return $result;
        }

        # Check if Nodes are in connected state
        $Log->comment("Confirming that the cluster is a HA config in HA_setup\n");
        $result = confirm_HA_config(node => $node);
        if ( $result != 1 ) {
            $Log->comment("The cluster is not a HA config \n");
            return $result;
        }
        else {
            $Log->comment("Confirmed that the cluster is a HA config\n");
        }
        #------------------------------------------------------------------------------
        # Setting the auto giveback attempts to 6 within 60min if NO_OF_ATTEMPTS eq 1
        #------------------------------------------------------------------------------
        if ($::NO_OF_ATTEMPTS eq '1') {
            try {
                $Log->comment(
                           "Setting the auto giveback attempts 4 within 60min");
                NACL::STask::StorageFailover->modify(command_interface => $node,
                                                     node     => $local_node,
                                                     attempts => "6",);
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
                my $response         = $exception_object->get_response_object();
                $result = FAIL;
                $Log->comment("Caught ResponseException in create_aggr with: \n"
                              . $exception_object->text());
            };
            if ($result != 1) {
                return $result;
            }
        } ## end if ($::NO_OF_ATTEMPTS ...

        #set AUTOBOOT during setup,
        $Log->comment("Setting AUTOBOOT in HA_setup\n");
        if ( $opts{autoboot_value} ) {
            $result = set_bootarg_autoboot(
                node           => $node,
                autoboot_value => $opts{autoboot_value}
            );
        }
        if ( $result != 1 ) {
            $Log->comment("Unable to set AUTOBOOT to $opts{autoboot_value} \n");
            return $result;
        }

    } # end of foreach 

    return $result;
}

sub HWassist_setup {
    $Log->comment("In HWassist Setup Function \n");

    my %opts = validate(
        @_,
        {
            ha_obj         => { type => OBJECT, isa     => "NACL::MTask::HA" },
            autoboot_value => { type => SCALAR, default => "true" },
            address_family => { type => SCALAR, default => "ipv4" },
        }
    );
    my $node           = $opts{ha_obj}->node();
    my $address_family = $opts{address_family};
    my $partner =
      NACL::STask::Node->get_partner_obj( command_interface => $node, );

    my $result = PASS;
    if ($address_family eq 'ipv4') {
        foreach my $source_object ($node, $partner) {
            my $response =
              $source_object->get_7m_or_nodescope_apiset()
              ->execute_command('command' => "options sp.setup");
            $Log->comment("The output of the response is : $response \n");

            my ($IPv4_IP, $IPv4_gateway, $IPv4_mask, $filer);
            if ($response =~ /off/i) {
                $IPv4_IP      = $source_object->hostrec()->RLM_IP();
                $IPv4_gateway = $source_object->hostrec()->RLM_GATEWAY();
                $IPv4_mask    = $source_object->hostrec()->RLM_MASK;
                NACL::C::SystemNodeServiceProcessorNetwork->modify(
                                                              'command_interface' => $source_object,
                                                              'node'              => $source_object,
                                                              'address-type'      => "IPv4",
                                                              'enable'            => "true",
                                                              'dhcp'              => "none",
                                                              'ip-address'        => $IPv4_IP,
                                                              'gateway'           => $IPv4_gateway,
                                                              'netmask'           => $IPv4_mask);
                my $response =
                  $source_object->get_7m_or_nodescope_apiset()
                  ->execute_command('command' => "options sp.setup");
                $Log->comment("The output of the response is : $response \n");
                if ($response =~ /off/i) {
                   $result = FAIL;
                   return $result;
                } 
            } ## end if ($response =~ /off/i)
        } ## end foreach my $source_object (...
    } ## end if ($address_family eq...

    elsif ($address_family eq 'ipv6') {
        foreach my $source_object ( $node, $partner ) {
            my ( $IPv6_mgmt, $IPv6_Prefix, $IPv6_Gateway, $Vserver,
                $sp_ipv6_ip );
            my $filer  = $source_object->name();
            my $Port_C = NACL::C::NetworkPort->find(
                command_interface => $source_object,
                filter            => { role => 'node-mgmt' }
            );
            my $port_mgmt = $Port_C->port;
            $IPv6_mgmt    = $source_object->hostrec()->IPV6_ADDRESS1();
            $IPv6_Prefix  = $source_object->hostrec()->NETMASK_LENGTH();
            $IPv6_Gateway = $source_object->hostrec()->NODE_MGMT_GATEWAY();
            $sp_ipv6_ip   = $source_object->hostrec()->RLM_IPV6();
            $Vserver      = $filer;
            try {
                NACL::CS::NetworkInterface->fetch(
                    command_interface => $source_object,
                    filter            => {
                        'vserver'        => $filer,
                        'address-family' => $address_family,
                        'curr-port'      => $port_mgmt
                    }
                );
            }
            catch NACL::Exceptions::NoElementsFound with {
                $Log->comment("IPv6 lifs does not exists Create IPv6 setup");
                if (   !defined $IPv6_mgmt
                    || !defined $IPv6_Prefix
                    || !defined $IPv6_Gateway )
                {
                    NATE::Exceptions::Argument->throw( "IPV6_ADDRESS1, "
                          . "NETMASK_LENGTH and DATA_GATEWAY "
                          . "are the required params" );
                }
                my $Lif = 'ipv6_lif';
                $Log->comment("IPv6 setup for setup for $filer");
                NACL::MTask::IPv6->ipv6_setup(
                    command_interface => $source_object,
                    interfaces        => [
                        {
                            lif              => $Lif,
                            vserver          => $Vserver,
                            role             => 'node-mgmt',
                            'home-node'      => $filer,
                            'home-port'      => $port_mgmt,
                            'address'        => $IPv6_mgmt,
                            'netmask-length' => $IPv6_Prefix
                        }
                    ],
                    mgmt_gateway => $IPv6_Gateway,
                );
            };    #End of catch

            my $SP_Network = NACL::CS::SystemNodeServiceProcessorNetwork->fetch(
                command_interface => $source_object,
                filter            => {
                    'node'         => $filer,
                    'address-type' => "IPv6"
                },
            );

            if ( $SP_Network->ip_address() eq '-' ) {
                NACL::C::SystemNodeServiceProcessorNetwork->modify(
                    command_interface => $source_object,
                    'node'            => $filer,
                    'address-type'    => $address_family,
                    'enable'          => "true",
                    'gateway'         => $IPv6_Gateway,
                    'prefix-length'   => $IPv6_Prefix,
                    'ip-address'      => $sp_ipv6_ip,
                );
               my $response =
               $source_object->get_7m_or_nodescope_apiset()->execute_command('command' => "options sp.setup");
               $Log->comment("The output of the response is : $response \n");
               if ($response =~ /off/i) {
               $result = FAIL;
               return $result;
              }

             }
        }    #end of foreach
    }    #end of if($address_family eq 'ipv6')

    foreach my $source_object ( $partner, $node ) {
        my $node_name = $source_object->name();
        my $partner_object =
          NACL::STask::Node->get_partner_obj(
            command_interface => $source_object, );

        my $local_node   = $source_object->name();
        my $partner_node = $partner_object->name();
        my ( $partner_status, $hwassist_enable );

        my $hwassist_show = NACL::CS::StorageFailoverHwassist->fetch(
            command_interface => $source_object,
            filter            => { node => $local_node }
        );
        $hwassist_enable = $hwassist_show->enabled;
        $Log->comment("hwassist enabled on $local_node : $hwassist_enable");

        if ( $hwassist_enable ne 'true' ) {
            $Log->comment("Modify hwassist value");
            NACL::C::StorageFailover->modify(
                command_interface => $source_object,
                node              => $local_node,
                'hwassist'        => "true",
            );
        }
        my @local_hwassist_show =
          NACL::CS::StorageFailoverHwassist->fetch(
            command_interface => $source_object );
        foreach my $local_hwassist_show (@local_hwassist_show) {
            my $name = $local_hwassist_show->node;
            if ( $name eq $local_node ) {
                $hwassist_enable = $local_hwassist_show->enabled;
                $Log->comment(
                    "hwassist enabled on $local_node : $hwassist_enable");
            }
            elsif ( $name eq $partner_node ) {
                $partner_status = $local_hwassist_show->local_status;
                $Log->comment(
                    "Monitor status on $partner_node : $partner_status");
            }
        }
        if ( ( $hwassist_enable ne 'true' ) ) {
            $Log->comment(
" hwassist is not enabled on the filer $local_node even after enabling : $hwassist_enable "
            );
            $result = FAIL;
            return $result;
        }
        elsif (( $hwassist_enable eq 'true' )
            && ( $partner_status ne 'active' ) )
        {
            $Log->comment(
" Monitor status is not active on the filer $partner_node even after enabling hwassist on $local_node : $partner_status "
            );
            $result = FAIL;
            return $result;
        }

        my @net_int_show = NACL::CS::NetworkInterface->fetch(
            command_interface => $source_object,
            filter            => { 'address-family' => $address_family, }
        );

        foreach my $intshow (@net_int_show) {
            my $vserver = $intshow->vserver;
            my $role    = $intshow->role;

            if ( ( $vserver eq $partner_node ) and ( $role eq 'node-mgmt' ) ) {
                my $partner_ip = $intshow->address;
                NACL::C::StorageFailover->modify(
                    command_interface     => $source_object,
                    node                  => $node_name,
                    'hwassist-partner-ip' => $partner_ip,
                );
            }
        }    #end of foreach my $intshow

        try {
            NACL::C::StorageFailoverHwassist->test(
                command_interface => $source_object,
                node              => $node_name
            );
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Caught ResponseException in partner with"
                  . $exception_object->text() );
            $Log->err(
"HWassist is not configured properly, Please configure manually and rerun the test again"
            );
        };    #end of catch block...
        $Log->comment("Clear all the stats output");
        NACL::C::StorageFailoverHwassistStats->clear(
            command_interface => $source_object,
            node              => $node_name,
        );

    }    #end of foreach my $source_object..

    return $result;

} ## end sub HWassist_setup

sub Verify_HWassist {
    $Log->comment("Verify HWassist stats after takeover \n");
    my %opts = validate(
        @_,
        {
            ha_obj     => { type => OBJECT, isa => "NACL::MTask::HA" },
            event_type => { type => SCALAR, },
        }
    );
    my $result     = PASS;
    my $node       = $opts{ha_obj}->node();
    my $event_type = $opts{event_type};
    my $node_name = $node->name;
    $node->apiset( interface => 'CLI' )
      ->execute_raw_command( command =>
"set -showallfields false -showseparator \"X;X\" ; system timeout modify -timeout 0 ; rows 0"
      );
    if ( $::USE_ZAPI eq 1 ) {
         my $zapi_interface_object = $node->apiset(
                                     category  => 'Node',
                                     interface => 'ZAPI',
                                      set      => 'CMode'
         );
    my %hw_args;
    my $count = 0;
    $hw_args{'node'} = $node;

    my $hwassist_response = $zapi_interface_object->cf_hwassist_stats(node => $node_name); 
    my $hwassist_output = $hwassist_response->get_parsed_output();
        print Dumper($hwassist_output);
       $count++ if(grep(/$event_type\s+2/,values %{$$hwassist_output[0]}));
       if ( $count != 1 ) {
        $Log->error("Did not generate expected hwassist stats count for event $event_type");
        $result = FAIL;
       }
    } else {
    my @StorageFailoverHwassistStats =
      NACL::CS::StorageFailoverHwassistStats->fetch(
        command_interface => $node,
      );

    my ( $event, $count, );
    foreach my $val (@StorageFailoverHwassistStats) {
        $event = $val->alert_event();
        if ( $event eq $event_type ) {
            $count = $val->count();
            $Log->comment("hwassist stats count for event $event is : $count");
            last;
        }
    }
    if ( $count == 2 ) {
        $Log->comment(
            "hwassist stats count is as expected for event $event : $count");
    }
    else {
        $Log->error(
            "Did not generate expected hwassist stats count for event $event");
        $result = FAIL;
    }
    }
    $node->apiset( interface => 'CLI' )
      ->execute_raw_command( command =>
"set -showallfields true -showseparator \"X;X\" ; system timeout modify -timeout 0 ; rows 0"
      );

    return $result;

}

###############################################################################
# Method: HA_cleanup
# Objective: To bring the nodes to up state, when it is in wfg or loader prompt
# Details:   This method checks the state of both the nodes.
# If the nodeB is in wfg, issue giveback from nodeA
# If the nodeB is in loader prompt, issues bootontap from nodeA
# Resetting all the HA flags
# If aggregate was created during setup, deleting them
# Verifying the number of nodes in the cluster is same as it was during setup
# Checking the health of the nodes
# Enabling SFO before exit
###############################################################################

sub HA_cleanup {

    $Log->comment("In Cleanup Function \n");

    my %opts = validate(
        @_,
        { 
            ha_obj => {type => OBJECT , isa => "NACL::MTask::HA" },
        }
    );
    my $local_ha_object = $opts{ha_obj};
    my $node = $local_ha_object->node();
    my $local_node_name = $node->name();
    my $transit_to_state = "CLI";
    #Try to bring up NodeA.
    $Log->comment("Bring up $local_node_name if its down");
    my  $local_transit_obj = NACL::Transit->new( name => $local_node_name );
    $local_transit_obj->change_state(to => $transit_to_state) if $local_transit_obj->get_state(timeout => MAX_REBOOT_TIME) ne $transit_to_state;

    # Do a blind giveback
    $node->refresh_command_interface();
    StorageFailoverCommands_For_Debug(node => $node);
    $Log->comment("Issue a giveback if the node is stuck in takeover/partial giveback");
    my $partner = NACL::STask::Node->get_partner_obj(
        command_interface => $node,
        node              => $node
    );
    my $partner_node_name = $partner->name();
    my  $partner_transit_obj = NACL::Transit->new( name => $partner_node_name );
    try {
      NACL::STask::StorageFailover->giveback(
        command_interface  => $node,
        node               => $node,        
        partner            => $partner,
        'override-vetoes' => 'true',
        'nacltask_skip_auto_giveback' => 1,
        'wait_for_partner' => 0,
        'nacltask_verify'    => 0,
        'require-partner-waiting' => "false",
    );
    } catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment("We dont care about the error since we are doing a blind giveback");
    };
    
    #Try to bring up NodeB
    $Log->comment("Bring up $partner_node_name if its down"); 
    $partner_transit_obj->change_state(to => $transit_to_state, iftakenover => "wait") if $partner_transit_obj->get_state(timeout => MAX_REBOOT_TIME) ne $transit_to_state;    
    $partner->refresh_command_interface(); 
    
    # Wait until both nodes show connected
     NACL::STask::StorageFailover->wait_for_state(command_interface => $node,
                              node             => $node,
                              partner          => $partner,
                              node_state       => "up",
                              partner_state    => "up",
                              'method-timeout' => MAX_REBOOT_TIME); 
                              
      # Enable storage failover if its disabled     
     $Log->comment("Enable storage failover if its disabled");
     NACL::STask::StorageFailover->enable(
        command_interface => $node,
        node => $node);

     # Reset HA flags
     $Log->comment("Resetting all HA flags during cleanup \n ");
     $local_ha_object->reset_HA_flags(node => $local_node_name);  


    #-------------------------------------------------------------------------------
    #if aggregate was created during setup, deleting them only when AGGR_CLEANUP = 1
    #-------------------------------------------------------------------------------
    my @sfo_aggr_list = ();

    foreach $node ($partner,$node) {
        if ($::AGGR_CLEANUP eq '1') {
            try{
                @sfo_aggr_list =
                 NACL::CS::StorageAggregate->fetch(
                             command_interface => $node,
                             filter => {'home-name' => $node, root => "false", 'ha-policy' => 'sfo'}
              );
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                 my $exception_object = shift;
                        my $response         = $exception_object->get_response_object();
                        $result = FAIL;
                        $Log->comment(
                        "Caught ResponseException in cleanup_function when fetching aggregate with: \n"
                          . $exception_object->text());
            };
            foreach my $aggr (@sfo_aggr_list) {
                if ($aggr =~ /^(SFO_AGGR).*/) {
                    $Log->comment("Deleting aggregate: $aggr");
                    try {
                        NACL::STask::Aggregate->purge(command_interface => $node,
                                                  aggregate         => $aggr,);
                    }
                    catch NACL::APISet::Exceptions::ResponseException with {
                        my $exception_object = shift;
                        my $response         = $exception_object->get_response_object();
                        $result = FAIL;
                        $Log->comment(
                        "Caught ResponseException in cleanup_funciton when purging aggregate with: \n"
                          . $exception_object->text());
                    };
                } ## end if ($aggr =~ /^(SFO_AGGR).*/)
            } ## end foreach my $aggr (@sfo_aggr_list)
        } ## end if ($::AGGR_CLEANUP eq...
        #------------------------------------------------------------------------------
        # Resetting the auto giveback attempts to 2 within 60min if NO_OF_ATTEMPTS eq 1
        #------------------------------------------------------------------------------
        if ($::NO_OF_ATTEMPTS eq '1') {
            try {
                $Log->comment(
                           "Setting the auto giveback attempts 4 within 60min");
                NACL::STask::StorageFailover->modify(command_interface => $node,
                                                     node     => $node,
                                                     attempts => "2",);
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
                my $response         = $exception_object->get_response_object();
                $result = FAIL;
                $Log->comment("Caught ResponseException in create_aggr with: \n"
                              . $exception_object->text());
            };
            if ($result != 1) {
                return $result;
            }
        } ## end if ($::NO_OF_ATTEMPTS ...

        #---------------------------------
        #checking the health of the nodes
        #---------------------------------
        $Log->comment("Checking the health of all the nodes in the cluster as part of cleanup \n ");
        try {
            my $cluster_health_flag =
              NACL::CS::Cluster->fetch(command_interface => $node,
                                   requested_fields  => [qw(health)],);
            if ($cluster_health_flag->health() =~ /false/) {
                $Log->comment("The nodes look unhealthy \n ");
                $result = FAIL;
            } else {
                $Log->comment("Verified healthy nodes as part of cleanup\n ");
            }
        } ## end try
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $response         = $exception_object->get_response_object();
            $result = FAIL;
            $Log->comment(
            "Caught ResponseException in cleanup_funciton when finding the health of the cluster with: \n "
              . $exception_object->text());
        };

    } # end of foreach

    #---------------------------------------------------------------------------
    #verifying the number of nodes in the cluster is same as it was during setup
    #---------------------------------------------------------------------------
    $Log->comment("Checking the number of nodes in the cluster as part of cleanup \n ");

    my $nodes_in_cluster = number_of_nodes_in_cluster(node => $node);
    if ($nodes_in_cluster != $number_of_nodes) {
        $Log->comment(
            "There are $nodes_in_cluster nodes in the cluster but $number_of_nodes nodes were found during setup"
        );
        $result = FAIL;
    } else {
        $Log->comment("Verified that the number of nodes in the cluster is: $nodes_in_cluster \n");
    }  

    return $result;

} ## end sub HA_cleanup()

##################################################################
# Method : arl_cleanup
# Objective: To relocate the aggregates to their appropriate nodes
##################################################################
sub arl_cleanup {
    my %opts = validate(
        @_,
        {
            ha_obj           => { type => OBJECT, isa => "NACL::MTask::HA" },
            source_aggr_list => { type => ARRAYREF },
            dest_aggr_list   => { type => ARRAYREF },
        }
    );

    my $source_object      = $opts{ha_obj}->node();
    my $destination_object = NACL::STask::Node->get_partner_obj(
        command_interface => $source_object,
        node              => $source_object
    );
    my $source      = $source_object->name();
    my $destination = $destination_object->name();

    my $source_aggr_list = $opts{'source_aggr_list'};
    my $dest_aggr_list   = $opts{'dest_aggr_list'};
    my @curr_aggr_list;
    StorageFailoverCommands_For_Debug(node => $source_object);#Fix for burt 602341
    # Making sure that local aggrs are intact.
    try {
        @curr_aggr_list = NACL::CS::StorageAggregate->fetch(
            command_interface => $source_object,
            filter            => {
                'owner-name' => $source,
                'ha-policy'  => 'sfo',
                state        => 'online'
            }
        );
    }
    catch NACL::Exceptions::NoElementsFound with {
        $Log->comment(
            "Caught ResponseException in : no entries matching your query\n" );
    };

    my @aggr_to_be_relocated = get_aggr_list_to_relocate(
        initial_aggr_list => $source_aggr_list,
        curr_aggr_list    => \@curr_aggr_list
    );
    if ( scalar(@aggr_to_be_relocated) ) {
        NACL::MTask::StorageAggregateRelocation->start(
            command_interface => $source_object,
            node              => $destination,
            destination       => $source,
            'aggregate-list'  => \@aggr_to_be_relocated
        );
    }

    @curr_aggr_list       = ();
    @aggr_to_be_relocated = ();

    # Making sure that partner aggrs are intact.
    try {
        @curr_aggr_list = NACL::CS::StorageAggregate->fetch(
            command_interface => $source_object,
            filter            => {
                'owner-name' => $destination,
                'ha-policy'  => 'sfo',
                state        => 'online'
            }
        );
    }
    catch NACL::Exceptions::NoElementsFound with {
        $Log->comment(
            "Caught ResponseException in : no entries matching your query\n");
    };
    @aggr_to_be_relocated = get_aggr_list_to_relocate(
        initial_aggr_list => $dest_aggr_list,
        curr_aggr_list    => \@curr_aggr_list
    );
    if ( scalar(@aggr_to_be_relocated) ) {
        NACL::MTask::StorageAggregateRelocation->start(
            command_interface => $source_object,
            node              => $source,
            destination       => $destination,
            'aggregate-list'  => \@aggr_to_be_relocated
        );
    }

}
###############################################################################################
# Method : get_aggr_list_to_relocate()
# Objective: Compare two aggregate objects list and
#            give the names of the aggregates which are not avaliable in current aggregate list
################################################################################################
sub get_aggr_list_to_relocate {
    my %opts = validate(
        @_,
        {
            initial_aggr_list => { type => ARRAYREF },
            curr_aggr_list    => { type => ARRAYREF },
        }
    );

    my @initial_aggr_list = @{ $opts{'initial_aggr_list'} };
    my @curr_aggr_list    = @{ $opts{'curr_aggr_list'} };
    my %curr_aggr_list    = map { $_->aggregate(), 1 } @curr_aggr_list;
    my @aggr_to_be_relocated =
      map { $_->aggregate() }
      grep( !defined $curr_aggr_list{ $_->aggregate() }, @initial_aggr_list );
    return @aggr_to_be_relocated;
}

#set boot arg AUTOBOOT to true or false.
#
sub set_bootarg_autoboot {

    $Log->enter();
    my %opts = validate(
        @_,
        {   node => {type => OBJECT , isa => "NACL::C::Node" },
            autoboot_value  => {type =>SCALAR , default => "false" },
        }
    );

    my $node = $opts{node}; 
    my $local_apiset = $node->get_7m_or_nodescope_apiset();
    my $node_name = $node->node();
    my $result = 1;
    $Log->comment("Setting AUTOBOOT to $opts{autoboot_value} on node $node_name");
    try {
        my $response = $local_apiset->bootargs_set(arg => 'AUTOBOOT', value => $opts{autoboot_value} );
        $Log->debug( "Response get_parsed_output:\n" . 
            Dumper($response->get_parsed_output()));
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->debug("Error is: " . $exception_object->text());
        my $resp = $exception_object->get_response_object();
        $Log->debug("Raw output was:\n" . $resp->get_raw_output());
        $result = 0;
    };

    return $result;

}

## Halt the filer and wait for the system to enter Loader prompt.
## Usage:
##      HAutils::system_halt(node => $node );
##      HAutils::system_halt(node => $node, 'inhibit-takeover'  => "true" );
##      HAutils::system_halt(node => $node, dump => "true");

sub system_halt {
    $Log->enter();
    my %opts = validate(
        @_,
        {   node => {type => OBJECT , isa => "NACL::C::Node" },
            'inhibit-takeover' => {type =>SCALAR , default => "false" },
             dump => {type => SCALAR , default => "false" },
        }
    );
    my $node = delete $opts{node};
    my $result = TRUE;
    my $haltcmd ;
    my $timeout = ( $opts{dump} eq "true" ) ? MAX_PANIC_HALT_TIME : MAX_HALT_TIME;

     ## Transit.pm doesnt handle confirmations, so confirmations are set to off before execution
    if ( ( $opts{'inhibit-takeover'} eq "true" ) ) {
        $haltcmd = "set  -confirmations off ; system node halt -node local -inhibit-takeover true";
    }elsif ( $opts{dump} eq "true" ) {
        $haltcmd = "set  -confirmations off ; system node halt -node local -dump true";
    }else {
        $haltcmd = "set  -confirmations off ; system node halt -node local";
    }

    my $transit_obj = NACL::Transit->new( name => $node->hostrec() );
    try {
        $transit_obj->change_state(
                                   to      => "FIRMWARE",
                                   haltcmd => $haltcmd,
                                   timeout => $timeout,
        );
    }catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in inhibit reboot with "
            . $exception_object->text() );
    };
    return $result;

}

## Boot the filer, if it is in Loader prompt
## Usage:
##         HAutils::boot_ontap(node => $node );

sub boot_ontap {
    $Log->enter();
    my %opts = validate(
        @_,
        {   node => {type => OBJECT , isa => "NACL::C::Node" },
        }
    );
    my $result = 1;
    my $transit_obj = NACL::Transit->new(name => $opts{node}->hostrec() );
    if ( $transit_obj->get_state() eq 'FIRMWARE' ) {
	$transit_obj->{conn}->send("boot_ontap\r");
    }else {
        $Log->comment( "Filer state is ont equal to FIRMWARE");
        $result = 0;
    }

    return $result;
}

    
## Reboot the given node
## Usage:
##      HAutils::system_reboot(node => $node );
##      HAutils::system_reboot(node => $node, 'inhibit-takeover' => "true" );
##      HAutils::system_reboot(node => $node, dump => "true" );
##      HAutils::system_reboot(node => $node, 'skip-lif-migration' => "true" );

sub system_reboot {
    my %opts = validate(
        @_,
        {   node => {type => OBJECT , isa => "NACL::C::Node" },
            'inhibit-takeover' => {type =>SCALAR|UNDEF , optional => 1 },
             dump => {type => SCALAR|UNDEF , optional => 1 },
            'skip-lif-migration' => {type => SCALAR , default => "false" },
        }
    );
    my $node = delete $opts{node};
    my $result = 1;
    my $rebootcmd;
    my $timeout = defined $opts{dump} ? MAX_PANIC_HALT_TIME : MAX_REBOOT_TIME;
    my $snooz_timeout = ONE_MIN;
   
    if ( ( $opts{'skip-lif-migration'} eq "true" ) ) {
       $rebootcmd = "set  -confirmations off ; system node reboot -node local -skip-lif-migration true";
    } elsif ($opts{'inhibit-takeover'} eq "true" ) {
       $rebootcmd = "set  -confirmations off ; system node reboot -node local -inhibit-takeover true" ;
    } else {
       $rebootcmd = "set  -confirmations off ; system node reboot -node local";
    }

    my $transit_obj = NACL::Transit->new( name => $node->hostrec() );
   
    try {
        $transit_obj->change_state(
                 to      => "FIRMWARE",
                 haltcmd => $rebootcmd,
                 timeout => $timeout,
         );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = 0;
        Log->comment( "Caught ResponseException in inhibit reboot with "
            . $exception_object->text() );
        return $result;
    };
    $Log->comment("Boot the filer");
    my $boot_status = HAutils::boot_ontap(node=> $node );
    $result = 0 if ( !$boot_status );
 
    return $result; 

}

#Get the client/clients information from the ntest command and populate @clients to be used during IO
sub get_client {
    $Log->comment("Getting client information \n");
    $result = 1;
    my @clients;
    try {
        @clients = NACL::C::Client->find();
        foreach my $client (@clients) {
            my $client_name = $client->client();
            $Log->comment("Client: $client_name \n");
        }
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = 0;
        $Log->comment(
            "Caught ResponseException when getting client information with: \n "
              . $exception_object->text() );
    };
    if ( $result != 1 ) {
        return $result;
    }
    else {
        return \@clients;
    }

}
#get boot arg AUTOBOOT value
#
sub get_bootarg_autoboot {
    $Log->enter();
    my %opts =
      validate( @_, { node => { type => OBJECT, isa => "NACL::C::Node" }, } );

    my $node         = $opts{node};
    my $node_name    = $node->node();
    my $local_apiset = $node->get_7m_or_nodescope_apiset();
    my $result       = 1;
    my $bootarg_autoboot;
    $Log->comment("Getting AUTOBOOT from node $node_name");
    try {
        my $response        = $local_apiset->bootargs_get( arg => 'AUTOBOOT' );
        my $resp_autoboot   = $response->get_raw_output();
        my @result_autoboot = split( /==/, $resp_autoboot );
        $bootarg_autoboot = $result_autoboot[1];
        $Log->debug( "Response get_raw_output:\n" . Dumper($bootarg_autoboot) );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        $Log->debug( "Error is: " . $exception_object->text() );
        $result = 0;
    };
    return $bootarg_autoboot;
}

=head2 :: print_ems_messages

Print the obtained ems messages
  HAutils::print_ems_messages(
                     node => $node,
		     event_object => $event_detector,
		     event => "cf.fm.takeover*",
          );

=over

=item C<< node=>$node >>

(Required) STask Node object

=item C<< event_object => $event_detector >>

(Required) MTask EventLogDetector object

=item C<< event => $event >>

(Required) MTask event name/name* to search and print

=back

=back

=cut

sub print_ems_messages {
    $Log->enter();
    my %opts =
      validate( @_, { node => { type => OBJECT, isa => "NACL::C::Node" }, 
                      event_object => { type => OBJECT, isa => "NACL::MTask::EventLogDetector"}, 
		      event  => {type => SCALAR }});

    my $result = PASS;
    my $node       = $opts{node};
    my $detector   = $opts{event_object};
    my $begin_time =  $detector->event_begin_time();
    my $end_time   =  $detector->event_end_time();
    my $date_range = $begin_time . '..' . $end_time;
    my %filter_opts;
    $filter_opts{'time'} = $date_range;
    $filter_opts{'node'} = $node;
    $filter_opts{'event'} =  $opts{event};
     
    my @event_state_objs = NACL::CS::Eventlog->fetch(
            'command_interface' => $node,
            'filter'            => \%filter_opts,
            'requested_fields'  => [qw(event)],
            );

    $detector->events( \@event_state_objs );

    $result = FAIL  if ( scalar @event_state_objs == 0 ) ;
    foreach my $event ( @event_state_objs ) {
        my $ems = $event->event();
        $Log->comment("EMS is : $ems");  
    }
    
    return $result ;
}

=head2 :: verify_takeover_giveback_duration 

Verify the takeover duration time. 
If duration is > 60 seconds, print warning message.

  HAutils::verify_takeover_giveback_duration(
                     event_object => $event_detector,
          );

=over

=item C<< event_object => $event_detector >>

(Required) MTask EventLogDetector object

=back

=back

=cut

sub verify_takeover_giveback_duration {
    $Log->enter();
    my %opts =
      validate( @_, { event_object => { type => OBJECT, isa => "NACL::MTask::EventLogDetector"}} );
    my $result = PASS;
    my $verify = FALSE;
    my $detector   = $opts{event_object};
    my $events = $detector->events();
    $result = FAIL  if ( scalar @{$events} == 0 );
    foreach my $event ( @{$events} ) {
        my $event_name = $event->event();
        if ( $event_name =~ /Duration/ ) {
            $verify = TRUE;
            my @tmp = split(/\s+/, $event_name );
            my $duration = $tmp[7];
	    if ( $duration  > MAX_TO_GB_DURATION ) { 
                $Log->warning(" $tmp[3] duration is greater than MAX_TO_GB_DURATION seconds : Actual Duration = $duration" );
            } else {
                $Log->comment("$tmp[3] has completed in $duration seconds");
            }
        }
    } 
    $result = FAIL if ( !$verify ); 
    return $result;
}

#fail a disk from the non-mroot containing aggregate
sub disk_fail {
    $Log->enter();
    my %opts =
      validate( @_, { node => { type => OBJECT, isa => "NACL::C::Node" }, } );
    my $node   = $opts{node};
    my $result = PASS;

    my @aggr_list;
    my @disk_list;

    #get all the aggregates
    try {
        @aggr_list = NACL::CS::StorageAggregate->fetch(
            command_interface => $node,
            requested_fields  => [qw(has-mroot has-partner-mroot disklist)]
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException when getting aggr list: \n "
              . $exception_object->text() );
    };

    #get a list of aggr that does not have the node or partner mroot vol
    #get all disk that are in those aggr
    #fail the disk that is on the top of the list
    foreach my $aggr (@aggr_list) {
        if (   ( $aggr->has_mroot() =~ m/false/ )
            && ( $aggr->has_partner_mroot() =~ m/false/ ) )
        {
            my @disk = $aggr->disklist();
            push @disk_list, @disk;
        }
    }

    $Log->comment("Failing disk $disk_list[0]");
    try {
        NACL::STask::StorageDisk->fail(
            command_interface => $node,
            disk              => $disk_list[0],
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException when failing the disk: \n "
              . $exception_object->text() );
    };
    return $result;
}

#unfail a disk if exist.
sub disk_unfail {
    $Log->enter();
    my %opts =
      validate( @_, { node => { type => OBJECT, isa => "NACL::C::Node" }, } );
    my $node   = $opts{node};
    my $result = PASS;
#get list of broken disk
    my @broken_disk_list;
    @broken_disk_list = NACL::CS::StorageDisk->fetch(
        command_interface => $node,
        filter            => { state => "broken", }
    );
#unfail the broken disk
    foreach my $broken_disk (@broken_disk_list) {
        my $broken_disk_name = $broken_disk->{name};
        $Log->comment("Unfailing disk $broken_disk_name");
        try {
            NACL::C::StorageDisk->unfail(
                command_interface => $node,
                disk              => $broken_disk->disk(),
                "spare"           => "true",
            );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $response         = $exception_object->get_response_object();
            $result = FAIL;
            $Log->comment( "Caught ResponseException when failing the disk: \n "
                  . $exception_object->text() );
        };

    }
    return $result;
}


=head2 :: get_aggr_vol

return a hash of aggr and associated vols
HAutils::get_aggr_vol(node => $node);

=over

=item C<< node=>$node >>

(Required) Node object

=back

=back

=cut
sub get_aggr_vol {
    $Log->enter();
    my %opts =
      validate( @_, { node => { type => OBJECT, isa => "NACL::C::Node" }, } );
    my $node   = $opts{node};
    my $result = PASS;
    my $aggr_name;
    my $vol_name;
    my @aggr_list;
    my @aggr_names;
    my @vol_names;
    my @vol;
    my %aggr_vol = ();
    my $aggr;
    #get all the aggr the exist in the cluster
    try {
        @aggr_list = NACL::CS::Aggregate->fetch( command_interface => $node, filter => {'owner-name' => $node->node(), 'ha-policy' => 'sfo', state => 'online' });
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in get_aggr_vol with "
              . $exception_object->text() );
    };
# for all the aggr, get the names of the vols associated with it and populate a hash
    foreach $aggr (@aggr_list) {
     $aggr_name = $aggr->aggregate();
     my $volcount = $aggr->volcount();
     $Log->comment("volcount in utils of $aggr_name is $volcount");
      if ($aggr->volcount() > 0){
        try {
            @vol = NACL::CS::Volume->fetch(
                command_interface => $node,
                filter            => { aggregate => $aggr_name, },
            );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            my $response         = $exception_object->get_response_object();
            $result = FAIL;
            $Log->comment( "Caught ResponseException in get_aggr_vol with "
                  . $exception_object->text() );
        };
        $vol_name  = "";
        @vol_names = ();
        foreach my $v (@vol) {
            $vol_name = $v->volume();
            push( @vol_names, $vol_name );
        }
        push( @{ $aggr_vol{"$aggr_name"} }, "@vol_names" );
      }else {
        push( @{ $aggr_vol{"$aggr_name"} }, "");
      }
    }
    if ( $result =~ m/FAIL/ ) {
        return $result;
    }
    else { return %aggr_vol; }
}

## Set the given environment variable at Loader prompt
## Usage:
##      HAutils::set_loader_variable( node => $node , cmd = $cmd );
## eg: set_loader_variable ( node => $node , cmd => "setenv bootarg.new.sysid 1573902733" );
##
sub set_loader_variable {
    $Log->enter();
    my %opts = validate(
        @_,
        {   node => {type => OBJECT , isa => "NACL::C::Node" },
            cmd  => {type => SCALAR },
        }
    );
    my $result = 1;
    my $cmd = $opts{cmd} . " \n" ;

    my $transit_obj = NACL::Transit->new(name => $opts{node}->hostrec() );
    if ( $transit_obj->get_state() eq 'FIRMWARE' ) {
        $transit_obj->{conn}->send("$cmd");
    }else {
        $Log->comment( "Filer state is not equal to FIRMWARE");
        $result = 0;
    }

    return $result;
}

## Get the system ID of both local filer and partner.

sub get_system_id {
    $Log->enter();
    my %opts =
    validate( @_,
              { node => { type => OBJECT, isa => "NACL::C::Node" },
               }
      );

     my $sys_info  = NACL::CS::StorageFailover->fetch(
                            command_interface => $opts{node},
                            filter => { node => $opts{node}, },
                  );
    my %sys_hash;
    $sys_hash{local_sysid} = $sys_info->nvramid();
    $sys_hash{partner_sysid} = $sys_info->partner_nvramid();

    return \%sys_hash;

}
## Remove the ownership of given disks.
## Usage:
##    HAutils::disk_remove_ownership( node => $node , disk = $disk );
## disk => Disk could be single disk or camma seperated disk list.
## eg: 0a.01
##     0a.01,0b.01
##
sub disk_remove_ownership {

    my %opts = validate(
        @_,
        {
            node             => {type => OBJECT, isa => "NACL::C::Node" },
            disk             => {type => SCALAR},
        }
    );
    my $unassigned_count=0;
    $Log->comment("Removing the ownership of a $opts{disk} disks");
    try {
        NACL::C::StorageDisk->removeowner(
            command_interface => $opts{node},
            disk              => $opts{disk},
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $Log->comment( "Could not remove the ownership of the disk"
              . $exception_object->text() );
                return FAIL;
    };

    $Log->comment("Verify that all disks ownership has been removed");
    sleep 20;
    my @unassigned_disks = NACL::CS::StorageDisk->fetch (
        command_interface => $opts{node},
        filter => { disk => $opts{disk} } ,
    );
    foreach my $unassigned_disks (@unassigned_disks) {
        if ( $unassigned_disks->container_type() ne 'unassigned' ) {
            $Log->comment("Ownership has not changed for the disk : " . $unassigned_disks->disk());
            $unassigned_count++;
        }
    }
    if ( $unassigned_count ){
        return FAIL;
    } else {
        return PASS;
    }

}

## Routine to convert the node to non_ha
## Usage:
##    HAutils::convert_to_ha_mode ( node => $node );
##
sub convert_to_ha_mode {

    my %opts = validate(
        @_,
        {
            node             => {type => OBJECT, isa => "NACL::C::Node" },
        }
    );
    my $node = $opts{node};
    $Log->comment("Currently filer is in non_ha mode,Converting the filer to ha mode");
    NACL::C::StorageFailover->modify(
                       command_interface => $node,
                       node              => $node,
                       mode              => 'ha',
    );
    my @sfo_mode = NACL::CS::StorageFailover->fetch(
                    command_interface => $node,
    );
    my $node_name = $node->node();
    foreach my $sf ( @sfo_mode ) {
        if ( $sf->node() eq $node_name ) {
            my $sf_status = $sf->state_description();
            if ( $sf_status !~ /Non-HA mode, Reboot node to activate HA/ ) {
                $Log->warn("SFO status has not changed");
            }
            my $mode = $sf->mode();
            if ( $mode ne 'ha' ) {
                $Log->warn("SFO mode has not changed to HA");
            }
        }
    }
    $Log->comment("Reboot the node after changing the mode to ha " . $node_name );
    my $Transit_obj = NACL::Transit->new(name => $node_name);
    $Transit_obj->reboot();
    $node->refresh_command_interface();
}

## Routine to convert the node to non_ha
## Usage:
##    HAutils::convert_to_non_ha_mode ( node => $node );
##
sub convert_to_non_ha_mode {
    my %opts = validate(
        @_,
        {
            node             => {type => OBJECT, isa => "NACL::C::Node" },
        }
    );
    my $node = $opts{node};
    $Log->comment("Currently filer is in ha mode,Converting the filer to non_ha mode");
    NACL::C::StorageFailover->modify(
                       command_interface => $node,
                       node              => $node,
                       mode              => 'non_ha',
    );
    sleep 10;
    my @sfo_mode = NACL::CS::StorageFailover->fetch(
                    command_interface => $node,
    );
    my $node_name = $node->node();
    # "Non-HA mode" Should be changed after burt 559772 gets fixed in sn_warm_qa
    foreach my $sf ( @sfo_mode ) {
        my $sf_status = $sf->state_description();
        my $mode = $sf->mode();
        if ( $sf->node() eq $node_name ) {
            if ( $mode !~ /non_ha/ ) {
                $Log->warn("SFO mode has not changed after changing the mode to non_ha");
            }
            if ( ( $sf_status =~ /Non-HA mode, Reboot node to use full NVRAM/ ) 
               || (  $sf_status =~ /Non-HA mode, Remove HA interconnect adapter to use full NVRAM/ ) 
               || ( $sf_status =~ /Non-HA mode, Remove partner unit to use NVRAM/ ) ) {
                $Log->comment("SFO status has changed properly after changing the mode to non_ha");
            } else {
                $Log->warn("SFO status has not changed  properly after changing the mode to non_ha");
            }
        }
    }
    $Log->comment("Reboot the node after changing the mode non_ha " . $node_name );
    my $Transit_obj = NACL::Transit->new(name => $node_name);
    $Transit_obj->reboot();
    $node->refresh_command_interface();
}


sub number_of_nodes_in_cluster {
    my %opts =
        validate( @_,
        {
            node => {type => OBJECT , isa => "NACL::C::Node" },
        } );

    my @nodes_in_cluster;
    try {
        @nodes_in_cluster =
          NACL::C::Node->find( command_interface => $opts{node} );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $Log->comment(
            "Caught ResponseException in number_of_nodes_in_a_cluster with"
              . $exception_object->text() );
    };

    return scalar(@nodes_in_cluster);
}


sub confirm_HA_config {
    $Log->comment("Confirm that the nodes are in HA config \n");
    my %opts =
        validate( @_,
        {
            node => {type => OBJECT , isa => "NACL::C::Node" },
        } );
    
    $result = PASS;

    try {
        my $storage_failover_connected = NACL::CS::StorageFailover->fetch(
            command_interface => $opts{node},
            filter            => { 'node' => $opts{node} },
            requested_fields  => [qw(state)],
        );
        if ( $storage_failover_connected->state() =~ /connected/i ) {
            $Log->comment("The nodes are connected to their partner\n ");
        }
        else {
            $result = FAIL;
            $Log->comment("The nodes are not in connected state \n ");
        }
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment(
"Caught ResponseException in confirm_HA_config when finding the connectivity of the node with its  partner with: \n "
              . $exception_object->text() );
    };
    return $result;
}

sub set_HA_flags{
    $Log->comment("setting HA flags \n");

    my %opts = validate(
        @_,
        {   
            node => {type => OBJECT , isa => "NACL::C::Node" },
            'auto-giveback'  => { type => SCALAR, optional => 1 },
            'check-partner'  => { type => SCALAR, optional => 1 },
            'detection-time' => { type => SCALAR, optional => 1 },
            'onfailure'      => { type => SCALAR, optional => 1 },
            'onpanic'        => { type => SCALAR, optional => 1 },
            'onreboot'       => { type => SCALAR, optional => 1 },
            'delay-seconds'  => { type => SCALAR, optional => 1 },
            'hwassist'       => { type  => SCALAR, optional => 1 },
            'abort-operations' => { type => SCALAR, optional => 1 },
        }
    );

    my $node = $opts{node};
    $result = PASS;
    try {
        NACL::C::StorageFailover->modify(
            command_interface => $node,
            node              => $node,
            %opts,
        );
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        my $response         = $exception_object->get_response_object();
        $result = FAIL;
        $Log->comment( "Caught ResponseException in set_HA_flags with"
              . $exception_object->text() );
    };
    return $result;
}
## Routine to get ha_config show output
## Usage:
##    HAutils::ha_config_show ( mode => maint , console => $console_conn );
##
sub ha_config_show {
     my %opts = validate(
        @_,
        {
            node             => {type => OBJECT ,isa => "NACL::C::Node" },
            mode             => { type => SCALAR , default => "normal" },
            console          => { type => OBJECT , optional => 1 } ## required only for Maint mode
        }
    );

    my $cmd;
    my $apiset_object;
    ## Command should be modified after burt 561259 is fixed
    if ( $opts{mode} eq 'normal' ) {
        $cmd = 'priv set test; ha-config show';
        $apiset_object = $opts{node}->get_7m_or_nodescope_apiset();
    } elsif( $opts{mode} eq 'maint' ) {
        $cmd = 'ha-config show';
        $apiset_object = NACL::APISet->new( connection => $opts{console},
                                  category => "Node",
                                  interface => "CLI",
                                  set       => "Maintenance",
                                );
    }
    sleep (10);
    my $output = $apiset_object->execute_raw_command(command=> $cmd );
    my @lines = split ('\n', $output);
    my %output_hash ;
    foreach my $line ( @lines) {
        if ( $line =~ /Chassis HA configuration/ ) {
            my @configuration = split( ':', $line);
            my $string = $configuration[1];
            $string =~ s/^\s+//;
            $string =~ s/\s+$//;
            $output_hash{chassis} = $string;
        } elsif($line =~ /Controller HA configuration/ ){
            my @configuration = split( ':', $line);
            my $string = $configuration[1];
            $string =~ s/^\s+//;
            $string =~ s/\s+$//;
            $output_hash{controller} = $string;
        }
    }
    print Dumper(\%output_hash);
    return \%output_hash;
}

## Routine to modify  ha_config values
## Usage:
##    HAutils::ha_config_modify ( node => $node, mode => maint , console => $console_conn );
##
sub ha_config_modify {
     my %opts = validate(
        @_,
        {
            node             => {type => OBJECT ,isa => "NACL::C::Node" },
            mode             => { type => SCALAR , default => "normal" },
            console          => { type => OBJECT , optional => 1 }, ## required only for Maint mode
            chassis          => { type => BOOLEAN , optional => 1 },
            chassis_value    => { type => SCALAR , optional => 1 },
            controller       => { type => BOOLEAN , optional => 1 },
            controller_value => { type => SCALAR , optional => 1 },
        }
    );
    if ( defined $opts{chassis} && !defined $opts{chassis_value}) {
        $Log->comment("Please provide chassis value to configure");
        return FAIL;
    }
    if ( defined $opts{controller} && !defined $opts{controller_value} ) {
        $Log->comment("Please provide controller value to configure");
        return FAIL;
    }
    my $apiset_object;
    ## Command should be modified after burt 561259 is fixed
    if ( $opts{mode} eq 'normal' ) {
        $apiset_object = $opts{node}->get_7m_or_nodescope_apiset();
    } elsif( $opts{mode} eq 'maint' ) {
        $apiset_object = NACL::APISet->new( connection => $opts{console},
                                  category => "Node",
                                  interface => "CLI",
                                  set       => "Maintenance",
                                );
    }
    my $cmd;
    my $config;
    my $config_value;
    if ( $opts{chassis} == 1 ) {
        $cmd = 'ha-config modify chassis ' . $opts{chassis_value};
        $config = "chassis";
        $config_value = $opts{chassis_value};
    }elsif($opts{controller} == 1 ) {
        $cmd = 'ha-config modify controller ' . $opts{controller_value};
        $config = "controller";
        $config_value = $opts{controller_value};
    }else{
        $Log->comment("Please specify either controller or chassis to modify");
        return FAIL;
    }
    sleep 10;
    my @prompts_answers = ( '.*Do you want to continue.*' => 'yes' );
    my $output = $apiset_object->execute_raw_command(command=> $cmd ,
                 'connectrec-match_table' => \@prompts_answers,
                 'connectrec-timeout' => 20,
        );
    if ( $output =~ /ERROR/i )  {
        $Log->comment("Failed to modify $config HA configuration to $config_value");
        return FAIL;
    }

    return PASS;
}

##Routine to compare given HA flags.
##Usage:
##      compare_HA_flags($hash_before_test , $hash_after_test)
##
sub compare_HA_flags {
        my ($HA_flag_before_test, $HA_flag_after_test) = @_;
        my @diffFlags = ();
        foreach my $key (keys %{$HA_flag_after_test}) {
            if (${$HA_flag_before_test}{$key} ne ${$HA_flag_after_test}{$key}) {
                    $Log->comment("$key is not same\n");
                    push(@diffFlags, $key)
            }
        }
        return \@diffFlags;
}

##Set HA flags to pre-setup state
## Usage:
##   restore_to_pre_set($node,$values)
##
sub restore_to_pre_setup {
    my ($node, $hash_default) = @_;
    my $result = PASS;
    $node->refresh_command_interface();
    while (my ($key, $value) = each(%$hash_default)) {
        if ($key =~ s/_/-/) {
            try {
                NACL::C::StorageFailover->modify(command_interface => $node,
                                                 node              => $node,
                                                 $key              => $value);

            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
                my $response         = $exception_object->get_response_object();
                $Log->comment("Not ableto modify the storageFailover $key value"
                              . $exception_object->text());
                $result = FAIL;
            };
        } elsif ($key =~ /node/) {
            $Log->comment("Skip the node parameter :$key");
            next;
        } elsif (($key =~ /enabled/) && ($value =~ /false/)) {
            try {
                NACL::STask::StorageFailover->disable(command_interface => $node,
                                                      node              => $node,);
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;

                my $response = $exception_object->get_response_object();
                $Log->comment("Not ableto modify the storageFailover $key value"
                              . $exception_object->text());
                $result = FAIL;
            };
        } elsif (($key =~ /enabled/) && ($value =~ /true/)) {
            try {
                NACL::STask::StorageFailover->enable(command_interface => $node,
                                                      node              => $node,);
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;

                my $response = $exception_object->get_response_object();
                $Log->comment("Not ableto modify the storageFailover $key value"
                              . $exception_object->text());
                $result = FAIL;
            };
        } else {
            try {
                NACL::C::StorageFailover->modify(command_interface => $node,
                                                 node              => $node,
                                                 $key              => $value);
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $exception_object = shift;
                my $response         = $exception_object->get_response_object();
                $Log->comment("Not ableto modify the storageFailover $key value"
                              . $exception_object->text());
                $result = FAIL;
            };
        } ## end else [ if ($key =~ s/_/-/)
    } ## end while (my ($key, $value) ...
    return $result;

} ## end sub restore_to_pre_setup

##
## Subroutine for Timesync
##
##

sub HA_Timesync {

    my ($local_node_object) = @_;
    my $SystemNodeDate = NACL::C::SystemNodeDate->find( command_interface => $local_node_object );
    my $Date = $SystemNodeDate->get_one_state_attribute('date');
    $Log->debug('Date     : $Date');
    my @dateandtime = split /\"|\/| |\:/, $Date;
    my $Zero = 0;
    my $month;
    my $day;
    if($dateandtime[1] <= 9){
    $month = $Zero . $dateandtime[1];
    } else{
    $month = $dateandtime[1];
    }
    if($dateandtime[2] <= 9){
        $day = $Zero . $dateandtime[2];
    } else{
        $day = $dateandtime[2];
    }

      NACL::C::SystemNodeDate->modify(
                     command_interface => $local_node_object,
                     node              => '*',
                     dateandtime       => "$dateandtime[3]$month$day$dateandtime[4]$dateandtime[5].$dateandtime[6]",
    );
    }##end
## end of Subroutine for Timesync

###############################################################################
# Method: StorageFailoverCommands_For_Debug
# Objective: To capture output of below commands in case of failure in NACL scripts.
#This will help us to triage script/product failures easily
#
#1. storage failover show
#2. storage failover show-takeover
#3. storage failover show-giveback
#4. storage aggregate relocation show
#5. storage failover interconnect show
# Usage:
#         HAutils::StorageFailoverCommands_For_Debug(node => $node );
#
###############################################################################

sub StorageFailoverCommands_For_Debug {
    my %opts = validate(
        @_,
        {   node => {type => OBJECT , isa => "NACL::C::Node" },
        }
    );

    my $node = $opts{node};
    my $local_node_name = $node->name();
    $Log->comment(
        "Storage Failover debug commands executed on $local_node_name");
    my @commands=("set -showallfields false",
                  "storage failover show",
                  "storage failover show-takeover",
                  "storage failover show-giveback",
                  "storage aggregate relocation show",
                  "storage failover interconnect show",
                  "set -showallfields true");
   foreach my $commands(@commands){
      $node->apiset( interface => 'CLI' )->execute_raw_command( command =>$commands);
   }
}##end
## end of Subroutine for StorageFailoverCommands_For_Debug

1;
