#this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/HTTPS_OnDemand_Server.pm#1

# Don't buffer output
$| = 1;

#***********************************************************************
#
# NAME
#   HTTPS_OnDemand_Server.pm - 
#
# SYNOPSIS
#   An HTTP server implemented via PERL to simulate ASUP OnDemand server.
#
# DESCRIPTION
#   This library provides a basic HTTP server under Perl.  It's 
#       catered to ASUP Ondemand.
#
# AUTHOR
#   dl-cluster-qa
#
# HISTORY
#  
#***********************************************************************

## @summary     HTTPS_Ondemand_Server.pm

package HTTPS_OnDemand_Server;

use strict;

use Options;
use Tharn qw(host snooze subtest $Parent $Log fatal_result);

use NATE::Result;
use Data::Dumper;
use IO::Socket;
use IO::Select;  #for http server
use Sys::Hostname;   #for http server
use HTTPS_Daemon; #for http server
use HTTP::Status; #for http server
use HTTP::Response;
use HTTP::Request;
use HTTP::Headers;
use Carp;
use MIME::Parser;
use Data::Dumper;
use NATE::Inc qw(find_file);

my $opts = new Options;

$opts->parameters(definitions=>
    [
       {
        name => "DELIVER",
        description => "Should I accept or defy ASUP delivery. Default is yes.",
        default => 1,
       },
       {
        name => "POLLING_RATE_CHANGE",
        description => "Do I have any actions qued up. Default is no",
        default => 0,
       },
       {
        name => "RETRANSMISSION_REQUIRED",
        description => "Do I have any actions qued up. Default is no",
        default => 0,
       },
       {
        name => "METHOD_NOT_SUPPORTED",
        description => "What HTTP equate to return if method is not supported. Default is 405.",
        default => 405,
       },
    ]);

my $opt_href = $opts->get_hash();

# new method - Create a new HTTP_Server object
#  Parameter hash (* means required):
#   *client               - Client on which to run the server
#    port                 - Port on which to run the server (default:80)
#    deliver_uri          - ASUP Delivery URI
#    retransmit_seq_no    - ASUP sequence number to be retranmitted
#    retransmit_uri       - ASUP retransmit URI
#    retransmit_size      - Size budget for the ASUP to be retransmitted
#    polling_rate         - New polling rate value
#	 invoke_type		  - all or test or performance or none
#	 invoke_message	      - user defined message
#    invoke_uri			  - ASUP delivery URI for Ondemand invokes
#    method_not_supported - Return code to use when method is not supported? (default: 405)
#    ondemand_dir         - Path to store the ASUP history table in xml format
#	 malformed_response   - Send a malformed XML response to the filer 0 0r 1(Boolean default:0)
#    malformed_resp_type  - Type of malformed response to be sent 1 to 12 
# 		malformed_response_type options
# 		1 - Remove the final </ASUPOnDemand> tag
# 		2 - Remove the entire Actions tag with all its contents
# 		3 - Remove seq-no and size options from the <retransmit/> tag
# 		4 - Remove all options from retransmit tag
# 		5 - Remove all tags except the main tag <ASUPOnDemand></ASUPOnDemand>
# 		6 - Mispell <deliver_options/> tag
# 		7 - Mispell <retransmit/> tag
# 		8 - Mispell <actions></actions> tag
# 		9 - Remove "/>" from <polling_rate/> tag
# 		10 - Send a response exceeding 4500 Bytes
# 		11 - Set Force option in invoke Tag
#    read_bytes           - How many bytes to read of request before terminating server. (default: all)
#
sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my %options = (
        port=>8443,
        delivery_required=>$opt_href->{DELIVER},
        polling_rate_change=>$opt_href->{POLLING_RATE_CHANGE},
        retransmit_required=>$opt_href->{RETRANSMISSION_REQUIRED},
		multiple_retransmits=>0,
		multiple_invokes=>0,
		deliver_uri=>undef,
		retransmit_uri=>undef,
		retransmit_seq_no=>undef,
		retransmit_size=>undef,
		polling_rate=>-1,
		invoke_type=>undef,
		invoke_message=>undef,
		invoke_uri=>undef,
        method_not_supported=>$opt_href->{METHOD_NOT_SUPPORTED},
		ondemand_dir=>undef,
		malformed_response=>0,
		malformed_resp_type=>0,
		ipv6=>undef,
        read_bytes=>"all",
		@_
    );

    my @subArgs;
    if (!$options{client}) {
        $Log->error("HTTPS Server requires a client parameter \n");
        return 0;
		
    }

# change deliver_uri to optionial
    if (!$options{deliver_uri}) {
        $Log->comment("Set the ASUP delivery URL to null\n");
        $options{deliver_uri}='\"\"';
    }

# change retransmit_seq_no to accept CHAR also 
    if ($options{retransmit_seq_no}) {    
        $options{retransmit_required}=1;
    }
    
    if ($options{polling_rate} > 0) {
        $options{polling_rate_change}=1;
    }

    if ($options{method_not_supported} < 0) {
        $options{method_not_supported}=405;
    }
	
	if (!$options{ondemand_dir}) {
        $Log->error("Please provide a dir to save the autosupport_history.xml \n");
        return 0;
	}

    @subArgs = ('{use HTTPS_OnDemand_Server; HTTPS_OnDemand_Server::run(port=>"'.$options{port}.'"'.
                                                 ',client=>"'.$options{client}.
                                                 '",delivery_required=>'.$options{delivery_required}.
                                                 ',retransmit_required=>'.$options{retransmit_required}.
                                                 ',polling_rate_change=>'.$options{polling_rate_change}.
												 ',multiple_retransmits=>'.$options{multiple_retransmits}.
												 ',multiple_invokes=>'.$options{multiple_invokes}.
                                                 ',deliver_uri=>"'.$options{deliver_uri}.
                                                 '",retransmit_uri=>"'.$options{retransmit_uri}.
                                                 '",retransmit_seq_no=>"'.$options{retransmit_seq_no}.
                                                 '",retransmit_size=>"'.$options{retransmit_size}.
                                                 '",polling_rate=>'.$options{polling_rate}.
												 ',invoke_type=>"'.$options{invoke_type}.
												 '",invoke_message=>"'.$options{invoke_message}.
												 '",invoke_uri=>"'.$options{invoke_uri}.
                                                 '",method_not_supported=>'.$options{method_not_supported}.
												 ',ondemand_dir=>"'.$options{ondemand_dir}.
												 '",malformed_response=>'.$options{malformed_response}.
												 ',malformed_resp_type=>'.$options{malformed_resp_type}.
                                                 ',ipv6=>"'.$options{ipv6}.
                                                 '",read_bytes=>"'.$options{read_bytes}.'"); }',
                 '-bg', '-username', 'root', '-client', "$options{client}");

    my $subtest = subtest @subArgs;
    my $self = {
        "delivery_required"    => $options{delivery_required},
        "retransmit_required"  => $options{retransmit_required},
        "polling_rate_change"  => $options{polling_rate_change},
		"multiple_retransmits" => $options{multiple_retransmits},
		"multiple_invokes"     => $options{multiple_invokes},
        "deliver_uri"          => $options{deliver_uri},
        "retransmit_uri"       => $options{retransmit_uri},
        "retransmit_seq_no"    => $options{retransmit_seq_no},
        "retransmit_size"      => $options{retransmit_size},
        "polling_rate"         => $options{polling_rate},
		"invoke_type"		   => $options{invoke_type},
		"invoke_message"       => $options{invoke_message},
		"invoke_uri"           => $options{invoke_uri},
        "method_not_supported" => $options{method_not_supported},
        "client"               => $options{client},
        "port"                 => $options{port},
		"ondemand_dir"         => $options{ondemand_dir},
		"malformed_response"   => $options{malformed_response},
		"malformed_resp_type"  => $options{malformed_resp_type},
        "ipv6"                 => $options{ipv6},
        "read_bytes"           => $options{read_bytes},
		"subtest"              => $subtest,
        @_,
    };
    bless($self, $class);
    return $self;
}

sub is_ready {
    my $me = (caller(0))[3];
    my $self = shift;
    my %opts = (
        timeout => 60,
        sleep   => 1,
        check   => 'Server now ready for connections',
        @_
    );

    my $subtest = $self->{subtest};
    my $runid   = $subtest->runid();
    my $server_file = $subtest->logfile();

    my $timeout = $opts{timeout};
    my $sleep   = $opts{sleep};
    my $check   = $opts{check};

    my $end = time() + $timeout;
    while ( time() < $end ) {
        if ( -e $server_file && -f $server_file ) {
            my $result = `grep "$check" $server_file`;
            chomp( $result );

            if ( $result =~ /$check/ ) {
                $Log->comment("$me: Server $runid is ready");
                return 1;
            }
        }
        Tharn::sleep( $sleep );
    }

    die "$me: Server with runid $runid did not become ready within $timeout seconds";
}

sub restart {
    my $self = shift;
    my %options = (
        client=>$self->{client},
        port=>$self->{port},
        delivery_required=>$self->{delivery_required},
        retransmit_required=>$self->{retransmit_required},
        polling_rate_change=>$self->{polling_rate_change},
		multiple_retransmits=>$self->{multiple_retransmits},
		multiple_invokes=>$self->{multiple_invokes},
        deliver_uri=>$self->{deliver_uri},
        retransmit_uri=>$self->{retransmit_uri},
        retransmit_seq_no=>$self->{retransmit_seq_no},
        retransmit_size=>$self->{retransmit_size},
        polling_rate=>$self->{polling_rate},
		invoke_type=>$self->{invoke_type},
		invoke_message=>$self->{invoke_message},
		invoke_uri=>$self->{invoke_uri},
        method_not_supported=>$self->{method_not_supported},
		ondemnad_dir=>$self->{ondemand_dir},
		malformed_response=>$self->{malformed_response},
		malformed_resp_type=>$self->{malformed_resp_type},
        ipv6=>$self->{ipv6},
        read_bytes=>$self->{read_bytes},
        @_
    );
    my @subArgs = ('{use HTTPS_OnDemand_Server; HTTPS_OnDemand_Server::run(port=>"'.$options{port}.'"'.
                                                 ',client=>"'.$options{client}.
                                                 '",delivery_required=>'.$options{delivery_required}.
                                                 ',retransmit_required=>'.$options{retransmit_required}.
                                                 ',polling_rate_change=>'.$options{polling_rate_change}.
												 ',multiple_retransmits=>'.$options{multiple_retransmits}.
												 ',multiple_invokes=>'.$options{multiple_invokes}.
                                                 ',deliver_uri=>"'.$options{deliver_uri}.
                                                 '",retransmit_uri=>"'.$options{retransmit_uri}.
                                                 '",retransmit_seq_no=>"'.$options{retransmit_seq_no}.
                                                 '",retransmit_size=>"'.$options{retransmit_size}.
                                                 '",polling_rate=>'.$options{polling_rate}.
												 'invoke_type=>"'.$options{invoke_type}.
												 '"invoke_message=>"'.$options{invoke_message}.
												 '"invoke_uri=>"'.$options{invoke_uri}.
                                                 '",method_not_supported=>'.$options{method_not_supported}.
												 ',ondemand_dir=>"'.$options{ondemand_dir}.
												 '",malformed_response=>'.$options{malformed_response}.
												 ',malformed_resp_type=>'.$options{malformed_resp_type}.
                                                 ',ipv6=>"'.$options{ipv6}.
                                                 '",read_bytes=>"'.$options{read_bytes}.'"); }',
                 '-bg', '-username', 'root', '-client', "$options{client}");
    my $subtest = subtest @subArgs;
    $self->{"subtest"} = $subtest;
    $self->{client}=$options{client};
    $self->{port}=$options{port};
    $self->{delivery_required}=$options{delivery_required};
    $self->{retransmit_required}=$options{retransmit_required};
    $self->{polling_rate_change}=$options{polling_rate_change};
	$self->{multiple_retransmits}=$options{multiple_retransmits};
	$self->{multiple_invokes}=$options{multiple_invokes};
    $self->{deliver_uri}=$options{deliver_uri};
    $self->{retransmit_uri}=$options{retransmit_uri};
    $self->{retransmit_seq_no}=$options{retransmit_seq_no};
    $self->{retransmit_size}=$options{retransmit_size};
    $self->{polling_rate}=$options{polling_rate};
	$self->{invoke_type}=$options{invoke_type};
	$self->{invoke_message}=$options{invoke_message};
	$self->{invoke_uri}=$options{invoke_uri};
    $self->{method_not_supported}=$options{method_not_supported};
	$self->{ondemand_dir}=$options{ondemand_dir};
	$self->{malformed_response}=$options{malformed_response};
	$self->{malformed_resp_type}=$options{malformed_resp_type};
    $self->{ipv6}=$options{ipv6};
    $self->{read_bytes}=$options{read_bytes};
    return;
}

sub stop {
    my $self = shift;
		my $logfile = $self->{subtest}->logfile();
		my $F;
		my $c = undef;
    $Log->trace("Stopping the server...\n");
		$self->{"subtest"}->kill();
		open $F, "$logfile" or die "ondemand server: $!";
        my $end = time() + 300;
		while (my $line = <$F>) {
			if($line =~ m/HTTP POST request received/) {
				$c = 1;
				last;
			}
            if ( time() > $end ) {
                close( $F );
                die "HTTPS_OnDemand_Server::stop: Did not receive \"HTTP POST request received\" after 5 minutes";
            }
		}
        close( $F );
    return $c;
}

sub get_entry {
    my $self = shift;
    my $message=undef;

    #get the single message from the queue
    #$message=$self->{"subtest"}->message_get(filter => sub{$_[0]->{message} =~ qr/HTTP/}, type => "HTTP", timeout => 3);
    $message=$self->{"subtest"}->message_get(type => "HTTPS", timeout => 3);
    $Log->debug("Found raw message:  ".Dumper($message)."\n");
    return $message;
}

sub get_all {
    my @msgConcat = ();
    my $msgTmp = undef;

    while (1) {
        $msgTmp=undef;
        #get an entry that is on the queue, a message reported from the server
        eval {
            $msgTmp = get_entry(@_);
        };
        #if no message available, then exit
        if (!defined $msgTmp) {
            last;
        }
        #else add the message to the array
        else {
            $Log->debug("Adding message to array: ".Dumper($msgTmp));
            push @msgConcat, $msgTmp;
        }
    }
    #return the array of messages
    return \@msgConcat;
}

sub _malformed_content
{
    my ($t,$c) = @_;
	if($t == 1) {
		$c =~ s/<\/ASUPOnDemand>$//;
		return $c;
	}
	if($t == 2) {
		$c =~ s/<actions.*?<\/actions>//;
		return $c;
	}
	if($t == 3) {
		$c =~ s/seq\-.*?" uri/uri/;
		return $c;
	}
	if($t == 4) {
		$c =~ s/seq\-.*?" \/>/\/>/;
		return $c;
	}
	if($t == 5) {
		$c = '<ASUPOnDemand></ASUPOnDemand>';
		return $c;
	}
	if($t == 6) {
		$c =~ s/delivery_options/delivery_oons/;
		return $c;
	}
	if($t == 7) {
		$c =~ s/retransmit/retmit/;
		return $c;
	}
	if($t == 8) {
		$c =~ s/actions/acns/g;
		return $c;
	}
	if($t == 9) {
		$c =~ s/\/><\/actions><\/ASUPOnDemand>$/<\/actions><\/ASUPOnDemand>/;
		return $c;
	}
	if($t == 10) {
		$c = '<ASUPODemand><delivery_options deliver="yes" uri="http://asup.netapp.com/xyz" /><actions><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><retransmit seq-num="42" size="0" uri="https://asup.netapp.com/xyz" /><polling_rate value="300" /></actions></ASUPOnDemand>';
		return $c;
	}
		if($t == 11) {
		$c =~ s/message/force=\"true\" message/;
		return $c;
	}
	else {
		return $c;
	}
}

sub run {
    my %params = (
            client=>undef,
            port=>8443,
            delivery_required=>1,
            retransmit_required=>0,
            polling_rate_change=>0,
			multiple_retransmits=>0,
			multiple_retransmits=>0,
			deliver_uri=>undef,
			retransmit_uri=>undef,
			retransmit_seq_no=>undef,
			retransmit_size=>undef,
			polling_rate=>-1,
			invoke_type=>undef,
			invoke_message=>undef,
			invoke_uri=>undef,
            method_not_supported=>405,
			ondemand_dir=>undef,
			malformed_response=>0,
			malformed_resp_type=>0,
			read_bytes=>"all",
            ipv6=>undef,
			@_,
    );

# JDS
$Log->comment("Passed in parameters: ".Dumper(\%params));

    if (!$params{client}) {
        return fatal_result("HTTPS Server run requires a client parameter \n");
    }

# change deliver_uri to optionial    
    if (!$params{deliver_uri}) {
        $Log->comment("Set the ASUP delivery URL to null\n");
        $params{deliver_uri}='\"\"';
    }

# change retransmit_seq_no to accept CHAR also 
    if ($params{retransmit_seq_no}) {
        $params{retransmit_required}=1;
    }

    if ($params{polling_rate} > 0) {
        $params{polling_rate_change}=1;
    }
	
	my (@retransmit_seq_nos, @retransmit_uris, @retransmit_sizes);
	my $retransmit_tag = "";
	my (@invoke_types, @invoke_uris, @invoke_messages);
	my $invoke_tag = "";
	if($params{'multiple_retransmits'}) {
		$Log->comment("Multiple retransmits set to true: ".$params{retransmit_seq_no});
		@retransmit_seq_nos = split(/,/, $params{retransmit_seq_no});
		@retransmit_uris = split(/,/, $params{retransmit_uri});
		@retransmit_sizes = split(/,/, $params{retransmit_size});
		my $retransmits = @retransmit_seq_nos;
		$Log->comment("no of retransmits: ".$retransmits);
		my $i = 0;
		while ( $i < $retransmits){
			$retransmit_tag=$retransmit_tag.'<retransmit seq-num="'.$retransmit_seq_nos[$i].'" size="'.$retransmit_sizes[$i].'" uri="'.$retransmit_uris[$i].'" />';
			$Log->comment($i);
			$i++;
		}
		$Log->comment("Retransmit Tag: ".$retransmit_tag);
	}
	
	if($params{'multiple_invokes'}) {
		@invoke_types = split(/,/, $params{invoke_type});
		@invoke_uris = split(/,/, $params{invoke_uri});
		@invoke_messages = split(/,/, $params{invoke_message});
		my $invokes = @invoke_types;
		$Log->comment("no of invokes: ".$invokes);
		my $i = 0;
		while ( $i < $invokes){
			$invoke_tag=$invoke_tag.'<invoke type="'.$invoke_types[$i].'" message="'.$invoke_messages[$i].'" uri="'.$invoke_uris[$i].'" />';
			$i++;
		}
		$Log->comment("Invoke Tag: ".$invoke_tag);
	}
	
	my $invoke_required;
	if(($params{'invoke_type'} =~ /all/) or ($params{'invoke_type'} =~ /test/) or ($params{'invoke_type'} =~ /performance/)) {
			$invoke_required = 1;
	}
	else {
			$invoke_required = 0;
	}

    my $counter=0;
	my $content1 = '<?xml version="1.0" encoding="UTF-8" standalone="yes" ?><!DOCTYPE ASUPOnDemand[<!ELEMENT ASUPOnDemand (delivery_options?,actions?)><!ATTLIST ASUPOnDemand version CDATA #REQUIRED><!ELEMENT delivery_options EMPTY><!ATTLIST delivery_options deliver (yes|no) #REQUIRED><!ATTLIST delivery_options uri CDATA #IMPLIED><!ELEMENT actions (invoke*,retransmit*,polling_rate?)><!ELEMENT invoke EMPTY><!ATTLIST invoke type (test|all|performance) #REQUIRED><!ATTLIST invoke uri CDATA #IMPLIED><!ATTLIST invoke message CDATA #IMPLIED><!ELEMENT retransmit EMPTY><!ATTLIST retransmit seq-num CDATA #REQUIRED><!ATTLIST retransmit size CDATA "0"><!ATTLIST retransmit uri CDATA #REQUIRED><!ELEMENT polling_rate EMPTY><!ATTLIST polling_rate value CDATA #REQUIRED>]>';
    my $max_clients=5;
    
    if ($params{port} <= 0) {
        $params{port}=8443;
    }

    if ($params{method_not_supported} < 0) {
        $params{method_not_supported}=405;
    }
	
	if (!$params{ondemand_dir}) {
        $Log->error("Please provide a dir to save the autosupport_history.xml \n");
        return 0;
	}

    my $addr;
    if ( $params{ipv6} ) {
        $addr = $params{ipv6};
    } else {
        my $clientobj = host($params{client});
        $addr = $clientobj->default_ip();
    }

    $Log->comment("Creating HTTPS Ondemand Server with Address: https:\/\/$addr:$params{port}");
    $Log->comment("Protocol Support - POST");
    $Log->comment("Bytes to read for request: $params{read_bytes}");
    
     my $NATE_LIB = Tharn::param("NATE_LIB");
     my @nate_lib_path = split(';', $NATE_LIB );
     my $priv_key_1 = 'supportability/asup/privkey.pem';
     my $full_path_to_priv_key  = find_file( $priv_key_1, search => [@nate_lib_path] );
     my $priv_cacert_2 = 'supportability/asup/cacert.pem';
     my $full_path_to_cert = find_file( $priv_cacert_2, search => [@nate_lib_path] );

    my $d = HTTPS_Daemon->new(LocalAddr=>$addr, LocalPort=>$params{port}, SSL_key_file=>$full_path_to_priv_key , SSL_cert_file=>$full_path_to_cert)|| undef;
    if (!defined($d)) {
        for (my $retries = 1; (!defined ($d) && $retries < 11); $retries++) {
            $Log->comment("Connection is ".Dumper($d)."\n");
             $d = HTTPS_Daemon->new(LocalAddr=>$addr, LocalPort=>$params{port}, SSL_key_file=>$full_path_to_priv_key, SSL_cert_file=> $full_path_to_cert)|| undef;
            $Log->comment("Try $retries: Waiting 6 seconds for HTTPS server to start on port $params{port}...Error: $!\n");
            snooze(6);
        }

        if (!defined($d)) {
            $Log->error("Could not bind to port or server failed to start...\n");
            return fatal_result("No server, unable to handle connetions");
        }
    }
    $Log->comment("Ondemand Server now ready for connections \n");
    while (my $connection = $d->accept) { #$connection is ClientConn returned by the accept call.  Accept waits until a connection is opened.
        $counter++;
        
        while (my $request = $connection->get_request) 
            {     
                my $method = $request->method();
                $Log->comment("HTTPS $method request received...\n");
				if ( $method eq "POST" ) 
                {
                    $Log->comment("HTTPS $method request is supported...\n");
					
					my $request_string = $request->as_string;
					$Log->comment("Dumping the HTTP request");
					my $dir = $params{ondemand_dir};
                    system (mkdir $dir);
                    my $request_log = $dir.'/ondemand_request.log';
                    open (FILE,">>$request_log") || die $!;
					print FILE "Request No: $counter \n";
                    print FILE $request_string."\n";
                    close (FILE);
					my $asup_history;
					$Log->comment("ASUP directory: ".$dir);
					if($request->header('Content-Type') =~ /x-7z-compressed/) {
						$asup_history = $dir.'/asup_history_'.$counter.'.7z';
					}
					elsif($request->header('Content-Type') =~ /x-compressed/) {
						$asup_history = $dir.'/asup_history_'.$counter.'.tgz';
					}
					else { # this block will be replaced with the contents commented out after ONTAP is ready to respond with correct content type 
						$asup_history = $dir.'/asup_history_'.$counter.'.7z';
						#$Log->error("Invalid content type for autosupport history payload");
						#return 0;
					}
					open (FILE, ">$asup_history");
					print FILE $request->content();
					close(FILE);
					
                    my $content;
					
                    if ($request->header('X-Netapp-asup-aod-message-type') eq "trigger-query")
                    {
                        if ( $invoke_required && ($params{'delivery_required'}) && ($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && ($params{'delivery_required'}) && ($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && ($params{'delivery_required'}) && !($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && ($params{'delivery_required'}) && !($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && !($params{'delivery_required'}) && ($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && !($params{'delivery_required'}) && ($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && !($params{'delivery_required'}) && !($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && !($params{'delivery_required'}) && !($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif ( !$invoke_required && ($params{'delivery_required'}) && ($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && ($params{'delivery_required'}) && ($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && ($params{'delivery_required'}) && !($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && ($params{'delivery_required'}) && !($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="yes" uri="'.$params{'deliver_uri'}.'" /><actions></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && !($params{'delivery_required'}) && ($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && !($params{'delivery_required'}) && ($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && !($params{'delivery_required'}) && !($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && !($params{'delivery_required'}) && !($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><delivery_options deliver="no"/><actions></actions></ASUPOnDemand>';
                        }
						if($params{multiple_retransmits}) {
							$content =~ s/<retransmit.*?" \/>/$retransmit_tag/;
						}
						if($params{'multiple_invokes'}) {
							$content =~ s/<invoke.*?" \/>/$invoke_tag/;
						}
						if($params{malformed_response}) 
						{
							$Log->comment("Actual content is : $content");
							$content = _malformed_content($params{malformed_resp_type},$content);
							$Log->comment("malformed_content is : $content");
						}
						$content = $content1.$content;
						my $code = 200;
						my $message = "Request succeeded";
						my $header = HTTP::Headers->new(Content_Type => 'text/xml');
						my $res = HTTP::Response->new( $code,$message,$header,$content);
						$connection->send_response($res);
                                                $Log->comment("Response: ".Dumper($res));
					}
					elsif($request->header('X-Netapp-asup-aod-message-type') eq "poll")
					{
					    if( $invoke_required && ($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && ($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && !($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( $invoke_required && !($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions><invoke type="'.$params{'invoke_type'}.'" message="'.$params{'invoke_message'}.'" uri="'.$params{'invoke_uri'}.'" /></actions></ASUPOnDemand>';
                        }
						elsif( !$invoke_required && ($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && ($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions><retransmit seq-num="'.$params{'retransmit_seq_no'}.'" size="'.$params{'retransmit_size'}.'" uri="'.$params{'retransmit_uri'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && !($params{'retransmit_required'}) && ($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions><polling_rate value="'.$params{'polling_rate'}.'" /></actions></ASUPOnDemand>';
                        }
                        elsif( !$invoke_required && !($params{'retransmit_required'}) && !($params{'polling_rate_change'}))
                        {
                            $content = '<ASUPOnDemand version="1.1"><actions></actions></ASUPOnDemand>';
                        }
						if($params{multiple_retransmits}) {
							$content =~ s/<retransmit.*?" \/>/$retransmit_tag/;
						}
						if($params{'multiple_invokes'}) {
							$content =~ s/<invoke.*?" \/>/$invoke_tag/;
						}
						if($params{malformed_response}) 
						{
							$Log->comment("Actual content is : $content");
							$content = _malformed_content($params{malformed_resp_type},$content);
							$Log->comment("malformed_content is : $content");
						}
						$content = $content1.$content;
						my $code = 200;
						my $message = "Request succeeded";
						my $header = HTTP::Headers->new(Content_Type => 'text/xml');
						my $res = HTTP::Response->new( $code,$message,$header,$content);
						$connection->send_response($res);
                                                $Log->comment("Response: ".Dumper($res));
					}
					else
					{
						$Log->comment("NOT a valid X-Netapp-asup-aod-message-type");
						$connection->send_status_line($params{method_not_supported});
					}
				}	
                else {
                    $Log->comment("HTTP $method request is NOT supported");
                    $connection->send_status_line($params{method_not_supported}); #send status code back to the client
                }
            
            }
        $connection->close;
    }
}

