#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/HTTPS_Server.pm#1

# Don't buffer output
$| = 1;

#***********************************************************************
#
# NAME
#   HTTPS_Server.pm - 
#
# SYNOPSIS
#   An HTTPS server implemented via PERL.
#
# DESCRIPTION
#   This library provides a basic HTTPS server under Perl.  It's currently
#       catered to ASUP, but can be easily modified for other needs.  Note that
#       this library does require a privkey.pem and a cacert.pem file to be
#       available in the local directory where the thpl will be run from.
#
# AUTHOR
#   dl-cluster-qa
#
# HISTORY
# 7/13/2009 Initial revision
# 8/19/2009 Added retry logic when creating server/binding to port
# 6/1/2010 Support for put/resume
#***********************************************************************

## @summary     HTTPS_Server.pm

package HTTPS_Server;

use strict;

use Options;
use Tharn qw(host snooze subtest $Parent $Log fatal_result);
use vars qw($LOGDIR $RUNID);

use NATE::Result;
use Data::Dumper;
use IO::Socket;
use IO::Select;  #for http server
use Sys::Hostname;   #for http server
use HTTPS_Daemon; #for http server
use HTTP::Status; #for http server
use Carp;
use MIME::Parser;
use Data::Dumper;
use NATE::Inc qw(find_file);
my $opts = new Options;

$opts->parameters(definitions=>
    [
       {
        name => "HTTPS_PUT_SUPPORT",
        description => "Does this Server support PUTs. Default is yes.",
        default => 1,
       },
       {
        name => "HTTPS_POST_SUPPORT",
        description => "Does this Server support POSTs. Default is yes.",
        default => 1,
       },
       {
        name => "HTTPS_METHOD_NOT_SUPPORTED",
        description => "What HTTP equate to return if method is not supported. Default is 405.",
        default => 405,
       },
    ]);

my $opt_href = $opts->get_hash();


# new method - Create a new HTTPS_Server object
#  Parameter hash (* means required):
#   *client               - Client on which to run the server
#    port                 - Port on which to run the server (default:443)
#    asup_dir             - Directory in which to store the asups
#                           (default: undef - don't store)
#    put_supported        - Should server support PUTs?  (default: 1)
#    post_supported       - Should server support POSTs? (default: 1)
#    method_not_supported - Return code to use when method is not supported? (default: 405)
#
sub new {
    my $this = shift;
    my $class = ref($this) || $this;

    my %options = (
        asup_dir=>undef,
        port=>443,
        put_supported=>$opt_href->{HTTPS_PUT_SUPPORT},
        post_supported=>$opt_href->{HTTPS_POST_SUPPORT},
        method_not_supported=>$opt_href->{HTTPS_METHOD_NOT_SUPPORTED},
        ipv6=>undef,
        read_bytes=>"all",
        @_
    );

    my @subArgs;
    if (!$options{client}) {
        $Log->error("HTTPS Server requires a client parameter \n");
        return 0;
    }

    if ($options{put_supported} < 0) {
        $options{put_supported}=$opt_href->{HTTP_PUT_SUPPORT};
    }

    if ($options{post_supported} < 0) {
        $options{post_supported}=$opt_href->{HTTP_POST_SUPPORT};
    }

    if ($options{method_not_supported} < 0) {
        $options{method_not_supported}=$opt_href->{HTTP_METHOD_NOT_SUPPORTED};
    }

    @subArgs = ('{use HTTPS_Server; HTTPS_Server::run(port=>"'.$options{port}.'",asup_dir=>"'.$options{asup_dir}.'"'.
                                                 ',client=>"'.$options{client}.
                                                 '",put_supported=>'.$options{put_supported}.
                                                 ',post_supported=>'.$options{post_supported}.
                                                 ',method_not_supported=>'.$options{method_not_supported}.
                                                 ',ipv6=>"'.$options{ipv6}.
                                                 '",read_bytes=>"'.$options{read_bytes}.'"); }',
                 '-bg', '-username', 'root', '-client', "$options{client}");
                 #'-bg', '-username', 'root', '-client', "$options{client}", '-runid', $runid  );

    my $subtest = subtest @subArgs;

    my $self = {
        "put_supported"        => $options{put_supported},
        "post_supported"       => $options{post_supported},
        "method_not_supported" => $options{method_not_supported},
        "client"               => $options{client},
        "port"                 => $options{port},
        "read_bytes"           => $options{read_bytes},
        "ipv6"                 => $options{ipv6},
        "subtest"              => $subtest,
        @_,
    };
    bless($self, $class);
    return $self;
}

sub is_ready {
    my $me = (caller(0))[3];
    my $self = shift;
    my %opts = (
        timeout => 60,
        sleep   => 1,
        check   => 'Server now ready for connections',
        @_
    );

    my $subtest = $self->{subtest};
    my $runid   = $subtest->runid();
    my $server_file = $subtest->logfile();

    my $timeout = $opts{timeout};
    my $sleep   = $opts{sleep};
    my $check   = $opts{check};

    my $end = time() + $timeout;
    while ( time() < $end ) {
        if ( -e $server_file && -f $server_file ) {
            my $result = `grep "$check" $server_file`;
            chomp( $result );

            if ( $result =~ /$check/ ) {
                $Log->comment("$me: Server $runid is ready");
                return 1;
            }
        }
        Tharn::sleep( $sleep );
    }

    die "$me: Server with runid $runid did not become ready within $timeout seconds";
}

sub restart {
    my $self = shift;
    my %options = (
        asup_dir=>undef,
        client=>$self->{client},
        port=>$self->{port},
        put_supported=>$self->{put_supported},
        post_supported=>$self->{post_supported},
        method_not_supported=>$self->{method_not_supported},
        ipv6=>$self->{ipv6},
        read_bytes=>$self->{read_bytes},
        @_
    );
    my @subArgs = ('{use HTTPS_Server; HTTPS_Server::run(port=>"'.$options{port}.'",asup_dir=>"'.$options{asup_dir}.'"'.
                                                 ',client=>"'.$options{client}.
                                                 '",put_supported=>'.$options{put_supported}.
                                                 ',post_supported=>'.$options{post_supported}.
                                                 ',method_not_supported=>'.$options{method_not_supported}.
                                                 ',ipv6=>"'.$options{ipv6}.
                                                 '",read_bytes=>"'.$options{read_bytes}.'"); }',
                 '-bg', '-username', 'root', '-client', "$options{client}");
    my $subtest = subtest @subArgs;
    $self->{"subtest"} = $subtest;
    $self->{client}=$options{client};
    $self->{asup_dir}=$options{asup_dir};
    $self->{port}=$options{port};
    $self->{put_supported}=$options{put_supported};
    $self->{post_supported}=$options{post_supported};
    $self->{method_not_supported}=$options{method_not_supported};
    $self->{ipv6}=$options{ipv6};
    $self->{read_bytes}=$options{read_bytes};
    return;
}

sub stop {
    my $self = shift;
    $Log->trace("Stopping the server...\n");
    $self->{"subtest"}->kill();
    return;
}

sub get_entry {
    my $self = shift;
    my $message=undef;

    #get the single message from the queue
    #$message=$self->{"subtest"}->message_get(filter => sub{$_[0]->{message} =~ qr/HTTPS/}, type => "HTTPS", timeout => 3);
    $message=$self->{"subtest"}->message_get(type => "HTTPS", timeout => 3);
    $Log->debug("Found raw message:  ".Dumper($message)."\n");
    return $message;
}

sub get_all {
    my @msgConcat = ();
    my $msgTmp = undef;

    while (1) {
        $msgTmp=undef;
        #get an entry that is on the queue, a message reported from the server
        eval {
            $msgTmp = get_entry(@_);
        };
        #if no message available, then exit
        if (!defined $msgTmp) {
            last;
        }
        #else add the message to the array
        else {
            $Log->debug("Adding message to array: ".Dumper($msgTmp));
            push @msgConcat, $msgTmp;
        }
    }
    #return the array of messages
    return \@msgConcat;
}

sub run {
    my %params = (
            client=>undef,   
            asup_dir=>undef,   #the asup to generate
            port=>"443",
            put_supported=>1,
            post_supported=>1,
            method_not_supported=>405,
            ipv6=>undef,
            read_bytes=>"all",
            @_,
    );

    if (!$params{client}) {
        return fatal_result("HTTP Server run requires a client parameter \n");
    }

    my $addr;

    if ( $params{ipv6} ) {
        $addr = $params{ipv6};
    } else {
        my $clientobj = host($params{client});
        $addr = $clientobj->default_ip();
    }
    
    my $counter=0;

    # per message type counter
    my %counter= ( PUT => 0, POST => 0 );

    my $max_clients=5;
    
    if ($params{port} <= 0) {
        $params{port}=443;
    }

    if ($params{put_supported} < 0) {
        $params{put_supported}=1;
    }

    if ($params{post_supported} < 0) {
        $params{post_supported}=1;
    }

    if ($params{method_not_supported} < 0) {
        $params{method_not_supported}=405;
    }

    $Log->comment("Creating HTTPS Server with Address: https:\\\\$addr:$params{port} \n");
    $Log->comment("ASUP Directory = $params{asup_dir}");


      
     my $NATE_LIB = Tharn::param("NATE_LIB");
     my @nate_lib_path = split(';', $NATE_LIB );
     my $priv_key_1 = 'supportability/asup/privkey.pem';
     my $full_path_to_priv_key  = find_file( $priv_key_1, search => [@nate_lib_path] );

     my $priv_cacert_2 = 'supportability/asup/cacert.pem';
     my $full_path_to_cert = find_file( $priv_cacert_2, search => [@nate_lib_path] );

    my $d = HTTPS_Daemon->new(LocalAddr=>$addr, LocalPort=>$params{port}, SSL_key_file=>$full_path_to_priv_key, SSL_cert_file=>$full_path_to_cert)|| undef;
    #my $d = HTTPS_Daemon->new(LocalAddr=>$addr, LocalPort=>$params{port}, SSL_key_file=>'/x/eng/kc-users/weiss/p4/test/supportability/asup/privkey.pem', SSL_cert_file=>'/x/eng/kc-users/weiss/p4/test/supportability/asup/cacert.pem')|| croak("Unable to handle client connections: $!\n");
    if (!defined($d)) {
        for (my $retries = 1; (!defined ($d) && $retries < 11); $retries++) {
            $d = HTTPS_Daemon->new(LocalAddr=>$addr, LocalPort=>$params{port}, SSL_key_file=>$full_path_to_priv_key, SSL_cert_file=>$full_path_to_cert)|| undef;
            $Log->comment("Try $retries: Waiting 6 seconds for HTTPS server to start on port $params{port}...Error: $!\n");
            snooze(6);
        }

        if (!defined($d)) {
            $Log->error("Could not bind to port or server failed to start...\n");
            return fatal_result("No server, unable to handle connetions");
        }
    }
    $Log->comment("HTTPS Server now ready for connections \n");
    while (my $connection = $d->accept) { #$connection is ClientConn returned by the accept call.  Accept waits until a connection is opened.

        $counter++;
        
        #If the server should halt before reading in the entire request
        if ($params{read_bytes} ne "all") { #if read_bytes is configured, we should terminate pre-maturely
            my $buffer;
            my $filename;
            my $header=$connection->get_request(1);
            $Log->comment("HEADER of request: ".$header->as_string()."\n");
            $connection->read($buffer,$params{read_bytes});
            $Log->comment("URI is: ".$header->uri()."\n");
            my $dir=$params{asup_dir}.$header->uri();
            $Log->comment("Directory to use is: $dir \n");
            system ("mkdir $params{asup_dir}");
            system ("mkdir $dir");
            $Log->comment("HTTP BYTES: \n".$buffer."\n");
            if ($header->header('X-Netapp-asup-version') == 3) {
                if ($header->header('Content-Type') =~ /x-7z-compressed/) { #check for compression type
                    open (datafile2,">$dir/body.7z"); #pull out content of put/post, which is a 7z file
                }
                else {
                    open (datafile2,">$dir/body.tgz"); #pull out content of put/post, which is a tgz file
                }
                print datafile2 $buffer;
                close (datafile2);
            }
            return;
        }
        else { #read the full request
            while (my $request = $connection->get_request) { #get the request from the client connection, returns an http request object
    
                my $method = $request->method();
    
                if ( exists $counter{$method} ) {
                    $counter{$method}++;
                } else {
                    $counter{$method} = 0;
                }
    
                $Log->comment("HTTPS $method request received...\n");
    
                if ( ($params{put_supported}  && ($method eq "PUT") ) ||
                     ($params{post_supported} && ($method eq "POST") ) ||
                     ($params{put_supported}  && ($method eq "HEAD") ) ) {
    
                    $Log->comment("HTTPS $method request is supported...\n");
                    $Log->comment("HTTPS Header: \n".$request->headers_as_string()."\n");
                    
                    if ($request->method() eq 'HEAD') { #if this is a resume, check for file and return appropriate codes
                        my $dir=$params{asup_dir}.$request->uri();
                        my @file_info=split(" ",`du -s -b $dir/body.*`);
                        my $rec_bytes=$file_info[0];
                        if (!defined($rec_bytes)) {
                            $rec_bytes=0;
                        }
                        my $res = HTTP::Response->new(200,"OK");
                        $res->header("Content-Length"=>$rec_bytes);
                        $connection->send_response($res);          
                    }
                    else {
                        my $dir=undef;
                        if ($params{asup_dir}) {
                            system ("mkdir $params{asup_dir}");
                            if ($request->method() eq "PUT") {
                                $dir=$params{asup_dir}.$request->uri();
                            }
                            else {
                                $dir=$params{asup_dir}."/HTTPasup-".$counter;
                            }
                            system ("mkdir $dir");
                            my $filename="$dir/httpspost";
                            $Log->comment("SAVING HTTPS REQUEST TO: $filename\n");
                            open (datafile,">$filename.log"); #save full post request
                            print datafile $request->as_string."\n";
                            close (datafile);
                            #if this is a RR filer, save off the body.tgz or body.7z attachment
                            if ($request->header('X-Netapp-asup-version') == 3) {
                                if ($request->header('Content-Type') =~ /x-7z-compressed/) { #check for compression type
                                    if ($request->header('Content-Range') =~ /bytes/) {#this is a resume, so lets append to the file
                                        open (datafile2,">>$dir/body.7z");
                                    }
                                    else {
                                        open (datafile2,">$dir/body.7z"); #pull out content of put/post, which is a 7z file
                                    }
                                }
                                else {
                                    if ($request->header('Content-Range') =~ /bytes/) {#this is a resume, so lets append to the file
                                        open (datafile2,">>$dir/body.tgz");
                                    }
                                    else {
                                        open (datafile2,">$dir/body.tgz"); #pull out content of put/post, which is a tgz file
                                    }
                                }
                                print datafile2 $request->content();
                                close (datafile2);
                            }
                            #otherwise, we need to unzip the attachment and mime decode the content
                            else {
                               open (datafile2,">$filename.gz"); #pull out content of post, which is a gzip file
                               print datafile2 $request->content();
                               close (datafile2);
                               system("gzip -df $filename.gz"); #unzip the file
                               my $parser= new MIME::Parser;
                               $parser->output_dir($dir);
                               $parser->output_to_core; 
                               open  (INPUT, "$filename"); #parse the attachments and save them off
                               $parser->read(\ *INPUT);
                               close (INPUT);
                            }
                        }
                        $Parent->message_put(type => "HTTPS", 
                                         message => "HTTPS: HTTP $method of ".$request->header('X-Netapp-Asup-Subject').
                                         " received from node ".$request->header('X-Netapp-Asup-Hostname').
                                         ". Saved to directory $dir");
        
                        $Log->comment("***I've received $counter{$method} total HTTPS $method requests***\n");
        
                        my $rc = 200; # OK
                        if ( $method eq "PUT" ) {
                            $rc = 201; # created
                        }
        
                        $Log->comment("***Returning code $rc in response to HTTPS $method request***\n");
                        $connection->send_status_line($rc); #send status code back to the client
                    }
                } else {
                    $Log->comment("***HTTPS $method request is NOT supported***\n");
                    $Log->comment("***Returning code $params{method_not_supported} in response to HTTPS $method request***\n");
                    $connection->send_status_line($params{method_not_supported}); #send status code back to the client
                }
            }
        }
        $connection->close;
    }
}

1;

=pod

=head1 NAME

HTTPS_Server.pm - Basic HTTPS server.


=head1 SYNOPSIS

    use HTTPS_Server;
    my $https_server = new HTTPS_Server(client => $params{client}, asup_dir=>$params{asup_save_dir});
    my $single_post = $https_server->get_entry();
    my $posts = $https_server->get_all();
    $https_server->stop();

=head1 DESCRIPTION

This library provides a basic HTTPS server in perl.
  
    
=head1 METHODS

new(client=>CLIENT);
        Description:
            Creates a new instance of the HTTPS_Server object.  It will bind to
                port 443 on the specified client.
        Inputs:
            client => the client to run this HTTPS_Server on.
            asup_dir => directory to save asups to
            port => the port to use for the server
        Outputs:
            HTTPS_Server object instance
        Examples:
            <see synopsis above>
            
stop();
        Description:
            Stops an HTTPS_Server instance
        Inputs:
            <none>
        Outputs:
            <none>
        Examples:
            <see synopsis above>
            
get_entry();
        Description:
            Gets a single entry that was posted to the HTTPS Server
        Inputs:
            <none>
        Outputs:
            $post => A single entry that was posted to the server.
        Examples:
            <see synopsis above>

get_all();
        Description:
            Get all entries that were posted to the HTTPS Server
        Inputs:
            <none>
        Outputs:
            $posts => All entries that were posted to the server.
        Examples:
            <see synopsis above>
            
run();
        Description:
            The main HTTPS server method.  This method will bind to a specific
                port on the specified client and listen for incoming request,
                like an HTTPS server.  When a post is received, it will parse
                the post for header information (subject, from, etc.) and will
                place certain post information on the HTTPS server queue to be
                read by a client application.  Also if requested, the server
                method will parse off the post request and save the full
                request to a location (decoding mime attachments, etc.)
        Inputs:
            $logdir => the directory to save off the post request
        Outputs:
            <none>
        Examples:
            <see synopsis above>
=cut

