#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/HTTP_Proxy.pm#1

#***********************************************************************
#
# NAME
#   HTTP_Proxy.pm - 
#
# SYNOPSIS
#   An HTTP server implemented via PERL.
#
# DESCRIPTION
#   This library provides a basic HTTP server under Perl.  It's currently
#       catered to ASUP, but can be easily modified for other needs.
#
# AUTHOR
#   dl-cluster-qa
#
# HISTORY
# 7/13/2009 Initial revision
# 8/19/2009 Added retry logic when creating server/binding to port
# 6/1/2010 Support for put/resume 
#***********************************************************************

## @summary     HTTP_Proxy.pm

package HTTP_Proxy;

use strict;

#use lib '/x/eng/kc-users/weiss/perl';
#required for Archive::Extract, which also needs IPC::Cmd and Module::Load::Conditional

use Options;
use Tharn qw(host snooze subtest $Parent $Log fatal_result);

use NATE::Result;
use Data::Dumper;
use IO::Socket;
use IO::Select;  #for http server
use Sys::Hostname;   #for http server
use HTTP_Daemon; #for http server
use HTTP::Status; #for http server
use Carp;
use MIME::Parser;
use Data::Dumper;
use HTTP::Proxy qw( :log );
use HTTP_Proxy_IPv6;

# new method - Create a new HTTP_Server object
#  Parameter hash (* means required):
#   *client               - Client on which to run the server
#    port                 - Port on which to run the server (default:3128)
#
sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my %options = (
        port=>3128,
        method=>'OPTIONS,GET,HEAD,POST,PUT,DELETE,TRACE,CONNECT',
        @_
    );

    my @subArgs;
    if (!$options{client}) {
        $Log->error("HTTP Proxy requires a client parameter \n");
        return 0;
    }

    $Log->comment("Starting proxy on ".$options{client}.":".$options{port});

    @subArgs = ('{use HTTP_Proxy;
    HTTP_Proxy::run(method=>"'.$options{method}.'",client=>"'.$options{client}.'",ipv6=>"'.$options{ipv6}.'", port=>"'.$options{port}. '");}', 
                    '-bg', '-username', 'root', '-client', "$options{client}");


    my $subtest = subtest @subArgs;
    my $self = {
        "client"               => $options{client},
        "port"                 => $options{port},
        "subtest"              => $subtest,
        @_,
    };

    sleep(30); #sleep to ensure the proxy starts up

    bless($self, $class);
    return $self;
}

sub stop {
    my $self = shift;
    $Log->trace("Stopping the server...\n");
    $self->{"subtest"}->kill();
    sleep(10); #wait a few seconds to ensure the process dies and the port is free
    return;
}


sub run {
    my %params = (
            client=>undef,
            port=>3128,
            method => 'OPTIONS,GET,HEAD,POST,PUT,DELETE,TRACE,CONNECT',
            @_,
    );

    if (!$params{client}) {
        return fatal_result("HTTP Proxy run requires a client parameter \n");
    }

    my $counter=0;

    # per message type counter
    if ($params{port} <= 0) {
        $params{port}=3128;
    }
    my $addr;
    if ( $params{ipv6} ) {
        $addr = $params{ipv6};
    } else {
        my $clientobj = host($params{client});
        $addr = $clientobj->default_ip();
    }

    my @http_test = `ps -aux | egrep "HTTP_Server|HTTPS_Server|HTTP_Proxy" | grep -v "grep"`;
    my $http_count = 0;

    foreach(@http_test) {
        $http_count++;
    }
    
    if($http_count > 1 ) {
        $Log->error("To many ports in use, found either another HTTP_Server, HTTPS_Server, or HTTP_Proxy\n");
        foreach(@http_test) {
            $Log->comment("$_ - $#http_test\n");
        }
        return;
    }

    $Log->comment("Creating HTTP Proxy with Address: $addr:$params{port} \n");

    my $d = HTTP_Proxy_IPv6->new(method=>$params{method},port=>$params{port}, host=>$addr);
    $d->push_filter(method => $params{method});
    $d->logmask(ALL);
    $d->start;
    $Log->comment("Completed proxy run");
}

sub restart {
    my $self = shift;
    my %options = (
        client=>$self->{client},
        port=>$self->{port},
        @_
    );
    my @subArgs = ('{use HTTP_Proxy; HTTP_Proxy::run(method=>"'.$options{method}.'",client=>"'.$options{client}.'", port=>"'.$options{port}. '");}', 
                    '-bg', '-username', 'root', '-client', "$options{client}");
    my $subtest = subtest @subArgs;

    $self->{"subtest"} = $subtest;
    $self->{client}=$options{client};
    $self->{port}=$options{port};

    sleep(30);
    return;
}


=pod

=head1 NAME

HTTP_Proxy.pm - Basic HTTP proxy.


=head1 SYNOPSIS

    use HTTP_Proxy;
    my $http_proxy = new HTTP_Proxy(client => $params{client}, port => 3128);
    $http_proxy->stop();

=head1 DESCRIPTION

This library provides a basic HTTP proxy server in perl.
  
    
=head1 METHODS

new(client=>CLIENT);
        Description:
            Creates a new instance of the HTTP_Proxy object.  It will bind to
                port 3128 on the specified client.
        Inputs:
            client => the client to run this HTTP_Proxy on.
            port => the port to use for the server
        Outputs:
            HTTP_Proxy object instance
        Examples:
            <see synopsis above>
            
stop();
        Description:
            Stops an HTTP_Proxy instance
        Inputs:
            <none>
        Outputs:
            <none>
        Examples:
            <see synopsis above>
            
run();
        Description:
            The main HTTP proxy method.  This method will bind to a specific
                port on the specified client and listen for incoming request,
                and allow it to move on.
        Inputs:
            <none>
        Outputs:
            <none>
        Examples:
            <see synopsis above>
=cut

1;
