# $Id: //depot/prod/test/Rsierranevada/lib/IFT/CFO_Ops.pm#1 $

## @summary The set of functional CFO tests
##
## @synopsis
##   use cfo_tests1;
##
## @author      bobj
## @cf          true
## @status private

package CFO_Ops;

use strict;

use vars ('@ISA','@EXPORT');
require Exporter;
@ISA = ('Exporter');
@EXPORT = ('do_cfo_tests1', 'setup_CFO');


use Tharn;
use IFT::CFO_Utils;
use Data::Dumper;
our $NA = "NA";
our $FAIL = "FAIL";
our $ERROR = "ERROR";
our $FATAL = "FATAL";
our $CONFIG = "CONFIG";
our $PASS = "PASS";
our $ACTION = "ACTION";
our $INFO = "INFO";
our $WAIT    = "wait";
our $NOWAIT    = "nowait";
our $ACK     = "ACK";
our $NCK     = "NCK";
our $QUIT    = "quit";
our $PAUSE   = "pause ";
our $RESUME  = "resume";


sub setup_CFO {
    my $class = shift;
    my $ts = shift;
    my $testid = shift;

    my $self = ();
    my $cfoinfo = $ts->{$testid};

    my @filer_keys = keys(%{$ts->{filers}});
    my @filers = ();
    my $mykey = "filerA";
    while (my ($key, $filer) = each(%{$ts->{filers}})) {
        $self->{$mykey}{host} = $filer->{fh};
        $self->{$mykey}{name} = $filer->{fname};
        $self->{$mykey}{conn} = $filer->{fc};
        $mykey = "filerB";
    }
    $self->{testid} = $testid;
    $self->{msgno} = 1;
    $self->{testno} = 1;
    $self->{interval} = $cfoinfo->{cf_interval};
    $self->{logfile} = $ts->{logfile} if (exists $ts->{logfile});
    $self->{duration} = $cfoinfo->{duration};
    ### my @CLUSTER = ($self);
    my $result = cfo_util::normalize_cluster ($self);
#    die config_result($result) if $result;
#    $result = cfo_util::unset_all_testpoints ($self);
#    die config_result($result) if $result;
#    $result = cfo_util::restore_cfo_options ($self);
#    die config_result($result) if $result;
#    $result = cfo_util::check_up_up ($self);
#    die config_result($result) if ($result ne 'UP_UP');
    bless $self, $class;
    $self->{subt} = subtest( \&_start_tests, -bg, -ipc, -runid,
                "CFO test $testid",
#                '-onexit',\&_subtest_exit_handler,
                "--", $self );
    $cfoinfo->{subt} = $self->{subt};

    return ($self);
}



use constant GIVEBACK_TIMEOUT => 600;

# all takeover/gb tests
sub _start_tests  {
    my $self = shift;
    $self->{parent} = Subtest::ipc();
    $self->{starttime} = time();
    my $testid = $self->{testid};
    print "DBGCEB:CFO:_start_tests: testid - $testid\n";

    my $tcount = 1;
    while (!$self->_check_duration()) {
      my $result = cfo_util::normalize_cluster ($self);
      $self->{testid} = "$testid:$tcount";
      $self->cfo_test();
      logcomment("$self->{testid}: snoozing for $self->{interval} seconds");
      sleep($self->{interval});
    }
}

sub _check_duration {
    my $self = shift;
    my $current = time();
    my $diff    = $current - $self->{starttime};
      if ( $diff >= $self->{duration} ) {
           logcomment("Test time is up!");
           return 1;
      }
    return;
}


sub cfo_test {
    my $self = shift;
    my $hostA = $self->{filerA}{host};
    my $hostB = $self->{filerB}{host};
    my $connA = $self->{filerA}{conn};
    my $connB = $self->{filerB}{conn};
    my $nameA = $self->{filerA}{name};
    my $nameB = $self->{filerB}{name};
    my $cluster_nodes_stat = '';

#        NOTE: it's very hard to test plain giveback, since it may
#         randomly fail due to reconstructs etc from other tests.
#        So we only use -f here.

    my $ownname = "NFO takeover on $nameA";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> NFO takeover on $nameA'");
    say ($connA,"priv set -q test ; cf takeover ");
    $cluster_nodes_stat =
        cfo_util::check_takeover_dead ($self);
    if ($cluster_nodes_stat !~ 'TAKEOVER_DEAD') {
         return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "giveback -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> giveback -f on $nameA'");
    say ($connA,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "NFO takeover on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> NFO takeover on $nameB'");
    say ($connB,"priv set -q test ; cf takeover ");
    $cluster_nodes_stat = cfo_util::check_dead_takeover
         ($self);
    if ($cluster_nodes_stat !~ 'DEAD_TAKEOVER') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "giveback -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> giveback -f on $nameB'");
    say ($connB,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "takeover -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> takeover -f on $nameA'");
    say ($connA,"priv set -q test ; cf takeover -f");
    $cluster_nodes_stat = cfo_util::check_takeover_dead ($self);
    if ($cluster_nodes_stat !~ /TAKEOVER_DEAD/) {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "giveback -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> giveback -f on $nameA'");
    say ($connA,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "takeover -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> takeover -f on $nameB'");
    say ($connB,"priv set -q test ; cf takeover -f");
    $cluster_nodes_stat = cfo_util::check_dead_takeover ($self);
    if ($cluster_nodes_stat !~ 'DEAD_TAKEOVER') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "giveback -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> giveback -f on $nameB'");
    say ($connB,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "forcetakeover -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> forcetakeover -f on $nameA'");
    say ($connA,"priv set -q test ; cf forcetakeover -f");
    $cluster_nodes_stat = cfo_util::check_takeover_dead ($self);
    if ($cluster_nodes_stat !~ 'TAKEOVER_DEAD') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "giveback -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> giveback -f on $nameA'");
    say ($connA,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "forcetakeover -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> forcetakeover -f on $nameB'");
    say ($connB,"priv set -q test ; cf forcetakeover -f");
    $cluster_nodes_stat = cfo_util::check_dead_takeover ($self);
    if ($cluster_nodes_stat !~ 'DEAD_TAKEOVER') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "giveback -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> giveback -f on $nameB'");
    say ($connB,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    goto skipcfodtests;
    # this cfod won't be entirely real (can still see both plexes to have MBs),
    # but it's better than nothing
    $ownname = "cfod on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> cfod on $nameA'");
    say ($connA,"priv set -q test ; cf forcetakeover -df");
    $cluster_nodes_stat = cfo_util::check_takeover_dead ($self);
    if ($cluster_nodes_stat !~ 'TAKEOVER_DEAD') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    $self->_send_end_msg($tid);
    sleep(180);

    $ownname = "giveback -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> giveback -f on $nameA'");
    say ($connA,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);

    $ownname = "cfod on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> cfod -f on '");
    say ($connB,"priv set -q test ; cf forcetakeover -df");
    $cluster_nodes_stat = cfo_util::check_dead_takeover ($self);
    if ($cluster_nodes_stat !~ 'DEAD_TAKEOVER') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);


    $ownname = "giveback -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> giveback -f on $nameB'");
    say ($connB,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);

  skipcfodtests:

    $ownname = "takeover -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> takeover -f on $nameA'");
    say ($connA,"priv set -q test ; cf takeover -f");
    $cluster_nodes_stat = cfo_util::check_takeover_dead ($self);
    if ($cluster_nodes_stat !~ 'TAKEOVER_DEAD') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);

    # this reboot is a clean shutdown
    $ownname = "reboot in takeover state in $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; ".
         "log 'NATE -----> reboot in takeover state in $hostA'");
    say ($connA,"priv set -q test ; reboot -t 0",
        '-timeout', '600', '-eval_error');
    # reboot should be back into TO state
    $cluster_nodes_stat = cfo_util::check_takeover_dead ($self);
    if ($cluster_nodes_stat !~ 'TAKEOVER_DEAD') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);

    $ownname = "giveback -f on $nameA";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connA,"priv set -q test ; log 'NATE -----> giveback -f on $nameA'");
    say ($connA,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);



    $ownname = "takeover -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> takeover -f on $nameB'");
    say ($connB,"priv set -q test ; cf takeover -f");
    $cluster_nodes_stat = cfo_util::check_dead_takeover ($self);
    if ($cluster_nodes_stat !~ 'DEAD_TAKEOVER') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);

    # this "reboot" is a dirty shutdown, but not an actual panic (don't need a core)
    $ownname = "panic in takeover state on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> panic in takeover state on $nameB'");
    say ($connB,"priv set -q test ; tp reboot fm fasttimeout 1");
    $cluster_nodes_stat = cfo_util::check_dead_takeover ($self);
    if ($cluster_nodes_stat !~ 'DEAD_TAKEOVER') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);

    $ownname = "giveback -f on $nameB";
    ($tid, $rc) = $self->_send_start_msg($ownname);
    $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    say ($connB,"priv set -q test ; log 'NATE -----> giveback -f on $nameB'");
    say ($connB,"priv set -q test ; cf giveback -f");
    $cluster_nodes_stat = cfo_util::check_up_up ($self);
    if ($cluster_nodes_stat !~ 'UP_UP') {
          return($self->_send_end_msg($tid,"FAIL","cluster_nodes_stat=".
                "$cluster_nodes_stat"));
    }
    sleep(180);

}

sub _get_testid {
    my $self = shift;
    my $tid = "$self->{testid}:$self->{testno}";
    $self->{testid}++;
    return $tid;
}

sub _get_msgno {
    my $self = shift;
    return $self->{msgno}++;
} #end _get_msgno

sub _send_end_msg {
    my $self   = shift;
    my $tid    = shift;
    my $result = shift;
    my $msg = shift;

    my $mode   = undef;
    my $tinfo = $self->{tests}{$tid};
    if (!$tinfo) {
        logcomment("_send_end_msg: test $tid does not exist");
        return;
    }
    $tinfo->{result} = $result if ($result);
    $tinfo->{text} = $msg if ($msg);
    my $header = _create_msg_header( $self, $tinfo );
    $tinfo->{duration} = (time() - $tinfo->{starttime});

    print
"DBGCEB:_send_end_msg: got a RESULT message to send from $tinfo->{testname}\n";
    my $msg =
      sprintf "%s;ENDTEST;RESULT=%s;RESULTSTR=%s;DURATION=%s;EMSG",
      $header, $tinfo->{result}, $tinfo->{text},$tinfo->{duration};
    print "DBGCEB:$self->{testid}:_send_end_msg: sending msg to parent\n$msg\n";
#   $self->_logmessage($msg);
    my $resp = _send_parent( $self, $msg, $mode );
    delete $self->{tests}{$tid};
    return $resp;
} #end _send_end_msg

sub _send_start_msg {
    my $self   = shift;
    my $testname = shift;
    my $cont = shift;
    my $tinfo = {};
    $tinfo->{testname} = $testname;
    $tinfo->{filer} = $self->{filer}{fname};
    $tinfo->{tid} = $self->_get_testid();
    $tinfo->{starttime} = time();
    $tinfo->{cont} = $cont;
    my $header = _create_msg_header( $self, $tinfo );
    $self->{omsginfo} = $tinfo;
    my $starttime = time();
    my $msg = sprintf "%s;STARTTIME=%s;CONTAINER=%s;EMSG", $header,
      $tinfo->{starttime}, $tinfo->{cont}  ;
#    $self->_logmessage($msg);
    my $resp = _send_parent( $self, $msg, undef );
    $self->{tests}{$tinfo->{tid}} = $tinfo;
    return ($tinfo->{tid},$resp);
} #end _send_start_msg

sub _create_msg_header {
    my $self  = shift;
    my $tinfo = shift;
    my $tid   = undef;
    $tinfo->{msgno} = $self->_get_msgno();
    my $msg = sprintf "ID=%s;MSG=%d;TESTNAME=%s;FILER=%s", $tinfo->{tid},
                      $tinfo->{msgno}, $tinfo->{testname},$tinfo->{filer};
    return $msg;
}

sub _send_noend_msg {
    my $self   = shift;
    my $tid  = shift;
    my $type = shift;
    my $text = shift;

    my $tinfo = $self->{tests}{$tid};
    if (!$tinfo) {
        logcomment("_send_end_msg: test $tid does not exist");
        return;
    }
    $tinfo->{type} = $type if $type;
    $tinfo->{text} = $text if $text;
    my $header = _create_msg_header( $self, $tinfo );
    my $msg    = undef;
    my $mode   = undef;
    $self->{omsginfo} = $tinfo;

    if ( $tinfo->{type} eq $ACTION ) {
        $msg = sprintf "%s;ACTION=%s;ITYPE=%s;ITEM=%s;EMSG", $header,
          $tinfo->{action}, $tinfo->{itype}, $tinfo->{item};
        $mode = $WAIT;
    } else {
        $msg = sprintf "%s;%s;TEXT=%s;EMSG" . $header, $tinfo->{type},
          $tinfo->{text};
    }
    print "DBGCEB:$self->{testid}:_send_noend_msg: sending msg to parent\n$msg\n";
#    $self->_logmessage($msg);
    my $resp = _send_parent( $self, $msg, $mode );
    $tinfo->{text} = "";
    $tinfo->{type} = "";
    return $resp;
} #end _send_noend_msg





sub _logmessage {
    my $self = shift;
    my $text = shift;
    my $status = lc shift;
    my $result = undef;
    my $error = undef;
    logcomment($text);

    if (exists $self->{logfile}) {
        open LOGFILE,"+>$self->{logfile}"
             or $error = "cannot open $self->{logfile}:$!";
        if ($error) {
            _logmessage($error);
            return;
        }
        print LOGFILE $text;
    }
    close LOGFILE;
    return if (!$status);
    if ($status eq "pass") {
        pass_result($text);
    } elsif ($status eq "fail") {
        $result = fail_result($text);
    } elsif ($status eq "warn") {
        $result = warn_result($text);
    } elsif ($status eq "config") {
        $result = config_result($text);
    } elsif ($status eq "info") {
        $result = logresult("Info",$text);
    } elsif ($status eq "na") {
        $result = na_result($text);
    } elsif ($status eq "fatal") {
        $result = logresult("FATAL",$text);
    }
    return $result;
}

sub read_parent {
    my $self                     = shift;
    my $exp                      = shift;
    my $mode                     = $self->{comm}{mode};
    my $omsg                     = $self->{comm}{outmsg};
    my $omsg_info                = $self->{comm}{omsginfo};
    my $sock                     = Subtest::ipc();
    my $ack                      = undef;
    my $resp                     = undef;
    my $tid = undef;
    my $msgno = undef;
    if ( $exp eq $ACK ) {
        $ack   = 1;
        $tid   = $omsg_info->{tid};
        $msgno = $omsg_info->{msgno};
    }
    my $retries = 5;
    while (1) {
        while ( $sock->can_read() ) {
            my $msg = $sock->read_message();
            chomp $msg;
            if ($ack) {
                $resp = $1
                  if ( $msg =~ /^$tid\;MSG=$msgno\;.*;ACK=(\w*)\;EMSG$/ );
            }
            if ( $msg =~ /$PAUSE/ ) {
                $self->{teststatus} = $PAUSE;
            } elsif ( $msg =~ /$RESUME/ ) {
                $self->{teststatus} = $RESUME;
            } elsif ( $msg =~ /$QUIT/ ) {
                exit(0);
#                $self->{teststatus} = $QUIT;
            } elsif ($msg eq "LIVETESTS") {
                my @livetests = ();
                foreach my $key (keys %{$self->{tests}}) {
                   push @livetests,$self->{tests}{$key}{testname};
                }
                return @livetests;
            } else {
                _logcomment(
                    "$self->{hname} received unknown msg from parent: $msg");
            }
        }
        $retries--;
 #       last if ( $self->{teststatus} eq $QUIT );
        last if ((!$retries) && ( $self->{teststatus} ne $PAUSE ));
        next if (($mode eq $WAIT) && (!$resp));
        last if ( $self->{teststatus} ne $PAUSE );
    }
    return $resp;
} #end read_parent

sub _send_parent {
    my $self = shift;
    my $msg  = shift;
    my $mode = shift;
    $self->{comm}{mode}   = $mode;
    $self->{comm}{outmsg} = $msg;
    my $sock = Subtest::ipc();
    my $ack  = undef;

    #add subfunction id if subfunction is zero.
    _logmessage();
    eval { $sock->write_message(); };
    if ($@) {
        logWarn("$self->{testid}:unable to write message to parent: $@");
        return 1;
    }
    my $resp = read_parent( $self, $ACK );
    logcomment("$self->{testid}:did not receive parent ack") if ($resp ne $ACK);
    logcomment($msg);
    return;
} #end _send_parent

1;
