# $Id: //depot/prod/test/Rsierranevada/lib/IFT/CFO_Utils.pm#1 $

## @summary Library of subroutines for cfo functional tests cfo_func*
##          To be used with the Integrated Functional Test Suite
## @synopsis
##   use cfo_util;
##
##   log_cf_timers ($hostA, $connA, $hostB, $connB);
##   check_up_up ($hostA, $connA, $hostB, $connB);
##   check_takeover_dead ($hostA, $connA, $hostB, $connB);
##   check_dead_takeover ($hostA, $connA, $hostB, $connB);
##   normalize_cluster ($hostA, $connA, $hostB, $connB);
##   unset_all_testpoints ($hostA, $connA, $hostB, $connB);
##   restore_cfo_options ($hostA, $connA, $hostB, $connB);
##
##
## @status private
## @author bundy

package cfo_util;

use strict;

use vars ('@ISA','@EXPORT');
require Exporter;
@ISA = ('Exporter');
@EXPORT = ('log_cf_timers','check_up_up','check_takeover_dead',
'check_dead_takeover','normalize_cluster','unset_all_testpoints',
'restore_cfo_options');

use Tharn;
use cluster;

use constant GIVEBACK_TIMEOUT => 600;
use constant TAKEOVER_TIMEOUT => 800;
use constant LONG_TIMEOUT_TIMER => 300;
use constant SHORT_TIMEOUT_TIMER => 10;

# helper to do some useful stuff to aid debugging
sub log_cf_timers  {
    my $self = shift;
    my $connA = $self->{filerA}{conn};
    my $connB = $self->{filerB}{conn};
    logcomment("Log timers for " . $self->{filerA}{name});
    say ($connA,"priv set -q test ; cf timers  ");
    logcomment("Log timers for " . $self->{filerB}{name});
    say ($connB,"priv set -q test ; cf timers  ");
}

# helpers to check state of cluster {A,B} is as expected
sub check_up_up {
    my $self = shift;
    my $hostA = $self->{filerA}{host};
    my $hostB = $self->{filerB}{host};
    my $connA = $self->{filerA}{conn};
    my $connB = $self->{filerB}{conn};

    logcomment("Check if nodes are UP and UP");
#        when we do a TO or a GB, the command can take time to get started.
# We want to capture the new
# state, not the old one, so a delay here.
    sleep (2);

# this assumes that we might have just done a giveback or reboot on either
# node, so we use GB timeout, which is the worst case.
    wait_state ($hostA, "UP", GIVEBACK_TIMEOUT) or
            return ($@);
    wait_state ($hostB, "UP", GIVEBACK_TIMEOUT) or
            return ($@);

# include the time between tests here
    sleep (SHORT_TIMEOUT_TIMER);
    log_cf_timers ($self);
    return 'UP_UP';
}

sub check_takeover_dead {
    my $self = shift;
    my $hostA = $self->{filerA}{host};
    my $hostB = $self->{filerB}{host};
    my $connA = $self->{filerA}{conn};
    my $connB = $self->{filerB}{conn};

    logcomment("Check if nodes are TAKEOVER and DEAD");
    sleep(2);
    wait_state ($hostA, "TAKEOVER", TAKEOVER_TIMEOUT) or
            return ($@);

    wait_state ($hostB, "DEAD", TAKEOVER_TIMEOUT) or
            return ($@);
    log_cf_timers ($self);
    return 'TAKEOVER_DEAD';
}

sub check_dead_takeover  {
    my $self = shift;
    my $hostA = $self->{filerA}{host};
    my $hostB = $self->{filerB}{host};
    my $connA = $self->{filerA}{conn};
    my $connB = $self->{filerB}{conn};

    logcomment("Check if nodes are DEAD and TAKEOVER ");
    sleep(2);
    wait_state ($hostA, "DEAD", TAKEOVER_TIMEOUT) or
            return ($@);

    wait_state ($hostB, "TAKEOVER", TAKEOVER_TIMEOUT) or
            return ($@);

    log_cf_timers ($self);
    return 'DEAD_TAKEOVER';
}

sub normalize_cluster  {
    my $self = shift;

    logcomment("Normalize cluster");
    say ($self->{filerA}{conn}, "priv set -q test ; cf giveback -f");
    say ($self->{filerB}{conn}, "priv set -q test ; cf giveback -f");
    say ($self->{filerA}{conn}, "cf enable");
    check_up_up ($self);

    say ($self->{filerA}{conn}, "cf enable");
    print "DBGCEB:normalize_cluster: returning\n";
    return;
}

sub unset_all_testpoints  {
    my $self = shift;
    my $hostA = $self->{filerA}{host};
    my $hostB = $self->{filerB}{host};
    my $connA = $self->{filerA}{conn};
    my $connB = $self->{filerB}{conn};

# clean up any old tps used in previous iterations
    logcomment("Unset testpoints");
    say ($connA, "priv set -q test ; tp reboot fm fasttimeout 0");
    say ($connB, "priv set -q test ; tp reboot fm fasttimeout 0");

    say ($connA, "priv set -q test ; tp error fm takeover1 0 0");
    say ($connB, "priv set -q test ; tp error fm takeover1 0 0");

    say ($connA, "priv set -q test ; tp error fm takeover2 0 0");
    say ($connB, "priv set -q test ; tp error fm takeover2 0 0");

    say ($connA, "priv set -q test ; tp error fm giveback1 0 0");
    say ($connB, "priv set -q test ; tp error fm giveback1 0 0");

    say ($connA, "priv set -q test ; tp error fm giveback 0 0");
    say ($connB, "priv set -q test ; tp error fm giveback 0 0");
}

sub restore_cfo_options  {
    my $self = shift;
    my $hostA = $self->{filerA}{host};
    my $hostB = $self->{filerB}{host};
    my $connA = $self->{filerA}{conn};
    my $connB = $self->{filerB}{conn};

    logcomment("Restore cfo options for reservations and killpackets ".
               "back to enabled");
    say ($connA,"priv set -q test ; cf enable reservedisks");
    say ($connB,"priv set -q test ; cf enable reservedisks");
    say ($connA,"priv set -q test ; cf enable killpackets");
    say ($connB,"priv set -q test ; cf enable killpackets");

}

1;
