#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/CmdServer.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
## @summary
##     CmdServer - CmdServer class to invoke rpc server commands on the
##     remote host.  This is the parent class for NfsServer and CifsServer
##     classes
##
## @description
##     This library contains functions which perform tasks common to the
##     integrated functional test
##
## @usage
##     use IFT::CmdServer;
##
## @author bundy@netapp.com
## @status public
use strict;
package CmdServer;
use Tharn;
use Data::Dumper;
use Carp;
use File::Find;
use SPARSE::Util::Logger qw(logDbg logError logTrc logStep logWarn);
use File::Path;
use Cwd;
use File::Compare;

our $ACK = "ACK";
our $NCK = "NCK";
our $DBGCEB = 0;
our $DBGTMP = 0;

#use MapShare;
#use FS::Util::Setup qw(create_and_map_cifs_share);
#use SPARSE::Util::Logger qw( logTrc logWarn );
#use Data::Dumper;

# The literal code in $CMD are the commands that the rpc server can
# invoke on the remote host.  These commands are written in portable
# perl so that they may be identically use on either *nix or windows.

my $CMD =<<'EOF';
{
use IFT::IFTCmd;
my $cmd_counter = 0;
Subtest::ipc()->serve();
}
EOF


# All of the commands issued run in the background. Only the
# rpc_finish method waits for command to finish before returning.  To
# run command in strict sequence requires that they be bracketed with
# calls to setup_rpc_server and rpc_finish.
sub new( $$ ) {
    my ($class, $client, $subcount, $sserv) = @_;
    my $self = {};
    $self->{CLIENT} = $client;
    $self->{CMD_BLOCK} = $CMD;
    $self->{SUBTEST} = undef;
    $self->{result} = ();
    $self->{tid} = "$client->{hname}:$subcount";
    $self->{subcount} = $subcount;
    $self->{sserv} = $sserv;
    bless $self, $class;
    return $self;
}

# Should be called as the first step on the new instance.
sub setup_rpc_server( $ ) {
    my $self = shift;
    my $client = $self->my_client();
logcomment("Enter setup_rpc_server");
logcomment("setup_rpc_server  Calling subtest");
    my $subtest_obj = subtest($self->{CMD_BLOCK},
                              '-bg',
                              '-ipc',
                              '-runid', "subtest on $client->{hname}_$self->{subcount}",
                              '-onexit',\&_subtest_exit_handler,
                              '-client', $client->{fullname},
                              '--', $self->{sserv});
logcomment("setup_rpc_server  Returned from subtest");
    my $timer_on = time();
    while ( ! $subtest_obj->ipc()->eval('return 1;') ) {
        sleep 1;
        if ( (time() - $timer_on) > 400 ) {
            logWarn("setup_rpc_server: rpc server is not running");
            die " rpc server is not running ";
        }
    }
    my $pid = $subtest_obj->{child}{pid};
    $self->{pid} = $pid;
    $self->{SUBTEST} = $subtest_obj;
logcomment("Exit setup_rpc_server  PID: $self->{pid}");
}

sub _subtest_exit_handler {
    my $subtest = shift;
    my @results = $subtest->rslts();
    print "DBGCEB:_subtest_exit_handler: $subtest->{runid} EXITING!!!!\n" if $DBGTMP;
    foreach my $result (@results) {
       my $message = sprintf($result);
       if (ref($result) =~ /Fatal/) {
         logcomment("FAIL: $subtest->{runid}:$message");
         logresult("FAIL","$subtest->{runid}:$message",abort=>1);
       } else {
         logcomment("$subtest->{runid} EXITING:  $message");
         logresult($result);
       }
    }
}


sub _call_subtest {
    my ($self, $function, @args) = @_;
#    my ($self, @args) = @_;
logcomment("Exit _call_subtest");
    my $client = $self->subtestx()->ipc();
    $self->{error} = '';
#    my $function = "test_control";
    my @frozen_args = map { NATE::RPC::Freeze::freeze($_) } @args;
    my $function_call = $function . "(" . join(",", @frozen_args) . ")";
    print "DBGCEB:_call_subtest: function = $function\n" if $DBGCEB;
logcomment("_call_subtest  --  Function call:\n\n$function_call");
    $client->eval_request($function_call) or
        $self->{error} = "eval_request failed to invoke the function: "
                        . "$function_call on client = $client";
logcomment("Exit _call_subtest");
    return;
}


sub _call_subtest_and_wait_for_reply {
    my ($self, $function, @args) = @_;
#    my ($self, @args) = @_;
logcomment("Enter _call_subtest_and_wait_for_reply");
    my $client = $self->subtestx()->ipc();
    $self->{error} = '';
#    my $function = "test_control";
    my @frozen_args = map { NATE::RPC::Freeze::freeze($_) } @args;
    my $function_call = $function . "(" . join(",", @frozen_args) . ")";
    print "DBGCEB:_call_subtest: function = $function\n" if $DBGCEB;
logcomment("_call_subtest_and_wait_for_reply  --  Function call:\n\n$function_call");
    my $rslt = $client->eval_request($function_call);
    if ($rslt) {
logcomment("_call_subtest_and_wait_for_reply  --  Function returned:\n\n".Dumper($rslt));
    } else {
        $self->{error} = "eval_request failed to invoke the function: "
                        . "$function_call on client = $client";
        logWarn("_call_subtest_and_wait_for_reply: $self->{error}");
    }
logcomment("_call_subtest_and_wait_for_reply  --  Calling read_reply");
    my $rval = $client->read_reply();
logcomment("_call_subtest_and_wait_for_reply  --  read_reply Returned:\n\n".Dumper($rval));
    die("_call_subtest_and_wait_for_reply:  RVAL is undefined") unless defined ($rval);
logcomment("Exit _call_subtest_and_wait_for_reply");
    return;
}


sub _check_client {
    my $self = shift;
    my $client = $self->subtestx()->ipc();
    my $subtest = $self->subtestx();
logcomment("Enter _check_client  TID: $self->{tid}");
    my $id = undef;
    $self->{result} = ();
    $self->{testcount} = 0 if (!exists $self->{testcount});
    my $lastmessage = undef;
#    return;  #DBGCEB take this out when NATE is fixed

logcomment("CmdServer:_check_client:checking on client");
    my @results = ();
    eval {
    my $endtest = 0;
    while ($client->can_read()) {
logcomment("_check_client:  Client can_read");
       print "DBGCEB: can_read\n" if $DBGTMP;
       my $message = $client->read_message();
logcomment("CmdServer:_check_client: message: $message");
       my $testinfo = _parse_message($message);
       ### next if ($message !~ /^ID\=.*EMSG$/);
       next unless (defined($testinfo->{type}));
logcomment("CmdServer:_check_client: we want this msg:\n\n$message");
       my $endtest = $testinfo->{endtest};
logcomment("_check_client  Line: ".__LINE__."   testinfo:\n\n".Dumper($testinfo));
logcomment("_check_client  Line: ".__LINE__);
       push(@results, $testinfo);
       push(@{$self->{result}}, $testinfo);
logcomment("_check_client  Line: ".__LINE__);
       if ($endtest) {
             logcomment("_check_client  Line: ".__LINE__."   ENDTEST reached");
             last;
       }
    }
logcomment("CmdServer:_check_client: done reading socket");
    };

logcomment("_check_client  Line: ".__LINE__);
    if ($@) {
logcomment("_check_client  Line: ".__LINE__);
        $self->{error} = "$self->{subtest}{runid} socket failed: $@";
logcomment("_check_client:ERROR in reading child socket: $self->{error}");
        logWarn($self->{error});
logcomment("_check_client  Line: ".__LINE__);
        return;
    }
    my $nresults = scalar(@results);
logcomment("_check_client  Line: ".__LINE__);
logcomment("CmdServer:_check_client: Returning [$nresults]:\n\n".Dumper(@results));
#    print "DBGCEB:CmdServer:_check_client: returning\n\n";
    return(@results);
}


sub _wait_for_client {
    my $self = shift;
    my $client = $self->subtestx()->ipc();
    my $subtest = $self->subtestx();
logcomment("Enter _check_client  TID: $self->{tid}");
    my $id = undef;
    $self->{result} = ();
    $self->{testcount} = 0 if (!exists $self->{testcount});
    my $lastmessage = undef;
#    return;  #DBGCEB take this out when NATE is fixed

logcomment("CmdServer:_check_client:checking on client");
    my @results = ();
    eval {
    my $endtest = 0;
    while (1) {
logcomment("_check_client:  Client can_read");
       print "DBGCEB: can_read\n" if $DBGTMP;
       my $message = $client->read_message();
logcomment("CmdServer:_check_client: message: $message\n\n".Dumper($message));
       print "DBGCEB:CmdServer:_check_client: message: $message\n" if $DBGCEB;
       next if ($message !~ /^ID\=.*EMSG$/);
logcomment("CmdServer:_check_client: we want this msg:\n\n$message");
       print "DBGCEB:CmdServer:_check_client: we want this one \nmessage: $message\n" if $DBGTMP;
       my $testinfo = _parse_message($message);
       my $endtest = $testinfo->{endtest};
logcomment("_check_client  Line: ".__LINE__."   testinfo:\n\n".Dumper($testinfo));
       push(@results, $testinfo);
       push(@{$self->{result}}, $testinfo);
       if ($endtest) {
             logcomment("_check_client  Line: ".__LINE__."   ENDTEST reached");
             last;
       }
    }
logcomment("CmdServer:_check_client: done reading socket");
    };

    if ($@) {
        $self->{error} = "$self->{subtest}{runid} socket failed: $@";
logcomment("_check_client:ERROR in reading child socket: $self->{error}");
        print "\n\nDBGCEB:_check_client:ERROR in reading child socket: $self->{error}\n\n" if $DBGTMP;
        logWarn($self->{error});
        return;
    }
logcomment("CmdServer:_wait_for_client: Returning:\n\n".Dumper(@results));
#    print "DBGCEB:CmdServer:_check_client: returning\n\n";
    return(@results);
}


sub _OLD_check_client {
    my $self = shift;
logcomment("Enter _check_client");
    my $client = $self->subtestx()->ipc();
    my $subtest = $self->subtestx();
    my $id = undef;
    $self->{result} = ();
    $self->{testcount} = 0 if (!exists $self->{testcount});
    my $lastmessage = undef;
#    return;  #DBGCEB take this out when NATE is fixed

logcomment("CmdServer:_check_client:checking on client");
    eval {
    while ($client->can_read()) {
logcomment("_check_client:  Client can_read");
       print "DBGCEB: can_read\n" if $DBGTMP;
       my $message = $client->read_message();
logcomment("CmdServer:_check_client: message: $message");
       print "DBGCEB:CmdServer:_check_client: message: $message\n" if $DBGCEB;
       next if ($message !~ /^ID\=.*EMSG$/);
logcomment("CmdServer:_check_client: we want this msg:\n\n$message");
       print "DBGCEB:CmdServer:_check_client: we want this one \nmessage: $message\n" if $DBGTMP;
       my $testinfo = _parse_message($message);
logcomment("_check_client:  testinfo:\n\n".Dumper($testinfo));
 #      getc; #DBGCEB
 #      print "DBGCEB:CmdServer:_check_client: message: $message\n";
 #      next if ($message !~ /^ID\=.*EMSG$/);
 #      print "DBGCEB:CmdServer:_check_client: we want this one message: $message\n";
#       my $message = $client->read_obj();
#       next if (!exists $message->{testname});
#       if ($message->{id} == 0) {
#           $self->{testcount}++;
#           $message->{id} = $self->{testcount};
#       }
#       $message->{ack} = "ACK";
#       $client->write_obj($message);
       push @{$self->{result}},$testinfo;
#       last if ($message =~ /EMSG/);
    }
#    print "DBGCEB:CmdServer:_check_client: done_reading socket\n";
logcomment("CmdServer:_check_client: done reading socket");
    };

    if ($@) {
        $self->{error} = "$self->{subtest}{runid} socket failed: $@";
logcomment("_check_client:ERROR in reading child socket: $self->{error}");
        print "\n\nDBGCEB:_check_client:ERROR in reading child socket: $self->{error}\n\n" if $DBGTMP;
        logWarn($self->{error});
        return;
    }
logcomment("CmdServer:_check_client: returning:\n\n".Dumper($self->{result}));
#    print "DBGCEB:CmdServer:_check_client: returning\n\n";
    return;
}


sub _parse_message {
    my $msg = shift;
logcomment("Enter _parse_message");
    return({}) unless ($msg);
    my $tinfo == {};
    my @fields = split/;/,$msg;
    print "DBGCEB:_parse_message: $msg\n" if $DBGTMP;
    foreach my $f (@fields) {
       my ($k, $v);
       if ($f =~ /(\w+)=(.*)/) {
            $k = $1;
            $v = $2;
       }
#      print "DBGCEB:_parse_message: field = $f\n";
      if ($f =~ /HB\=(.*)/) {
          $tinfo->{hb} = time();
      } elsif ($k) {
          $tinfo->{lc($k)} = $v;
      }
  }
  if (!$tinfo->{text}) {
       my $text = ($tinfo->{error} || $tinfo->{warn} || $tinfo->{resultstr} ||
                             $tinfo->{info});
       $tinfo->{text} = $text if (defined($text));
  }
logcomment("_parse_message: msg:\n\n$msg\n\ntinfo =\n\n ".Dumper($tinfo));
      return $tinfo;
}


sub _send_ack {
    my $self = shift;
    my $type = shift;
    my $msg = shift;
    my $id = shift;
    my $sock = $self->subtestx()->ipc();

    my ($msgno, $testname) = ($msg =~ /MSG\=(\d*).*TESTNAME\=(\w*):/);

    my $resp = sprintf "ID=%s:MSG=%d:TESTNAME=%s:%s:EMSG", $id, $msgno,
                       $testname, $type;
    print "DBGCEB:_send_ack: sending $type  $resp\n" if $DBGCEB;
    $sock->write_message($resp);
    return;
}

sub rpc_finish ( $ ) {
    my $self = shift;
    my $begin = time();
    if ( defined($self->{SUBTEST}) ) {
        $self->{SUBTEST}->ipc()->quit();
        $self->{SUBTEST}->wait_finish();
    }
    logcomment(sprintf("Waited %d secs for subtests to finish.",
                        time()-$begin));
}

sub kill_subtest ( $ ) {
    my $self = shift;
    if ( defined($self->{SUBTEST}) ) {
        $self->{SUBTEST}->kill("subtest did not respond within 10 minutes");
    }
}

sub my_client ( $ ) {
    my $self = shift;
    return $self->{CLIENT};
}

sub subtestx ( $ ) {
    my $self = shift;
    return $self->{SUBTEST};
}

1;
