#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTClient.pm#1 $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary        Library that defines useful client-related routines
## @description
##             Client queries, stats
##
## @author bundy
package IFTClient;
use strict;
use Tharn;
use mount qw(mount umount);
use mount_old qw(mount umount);
use Data::Dumper;
use Utils qw(make_conn make_host);
use FS::Util::Utilities qw(uSystem);
use SPARSE::Util::Logger qw(logError logTrc logStep logWarn);
use IFT::NfsFuncs;
#use IFT::IFTFDOTests;
use SayWrap qw(sayEOF);
use SanClient;
use blocks_misc;
use SAN qw(create_lun);


our $SUSPEND = "suspend";
our $QUIT    = "quit";
our $PAUSE   = "pause ";
our $RESUME  = "resume";
our $MOUNTED = "mounted";
our $DELETED = "deleted";
our $UMOUNT  = "umount";
our $MOUNT   = "mount";
our $RMDIR   = "rmdir";
our $MOUNTS  = "mounts";
our $NOT_OK  = "NK";
our $OK      = "OK";
our $ERROR   = "error";
our $RESULTS = "results";
our $CLASS   = 'Client';
our $AGGR    = "aggr";
our $FLEX    = "flex";
our $TRAD    = "trad";
our $LUNS    = "LUNs";
our $QTREES  = "qtrees";
our $DISABLEMOUNT = "DISABLEMOUNT";
our $ENABLEMOUNT = "ENABLEMOUNT";
our $DELETEMOUNT = "DELETEMOUNT";
our $ADDMOUNT = "ADDMOUNT";
our $RETRIES = 3;
our $ACK = "ACK";
our $NCK = "NCK";


our $DBGCEB = 1;

# ------------------------------------------------------------------------

#================================================================================
# Client setup Routines
#================================================================================

sub new {
    my $class      = shift;
    my $clientinfo = shift;
    my $testinfo   = shift;
    my $type       = shift;
    my (%opts) = @_;
    $opts{type} = $type if ($type);
    my $self       = {};
    my @client_list = ();
    $self->{clients} = {};
    $self->{sserv} = $testinfo->{sserv};
    logcomment("Client:new");
    if (!$clientinfo) {
        $self->{error} = "No clients defined";
        return $self;
    }
    if (ref($clientinfo) eq 'Client') {
        $self = $clientinfo;
        return( $self );
    } elsif (ref($clientinfo) eq 'ARRAY') {
        @client_list = @$clientinfo;
    } else {
        @client_list = ( $clientinfo );
    }

    foreach my $cl (@client_list) {
        print "DBGCEB:Client:new: client = $cl\n";
        my $count = _parse_client_list( $self, $cl, %opts );
        $self->{error} = " No clients were specified " unless $count;
    }
#    $self->{quotas} = (exists($opts{quotas}) ? $opts{quotas} : $testinfo->{quotas});
    $self->{quotas} = (exists $testinfo->{quotas}) ? $testinfo->{quotas} : 0;
    $self->{unique_cont} = (exists $testinfo->{unique_cont}) ?
                           $testinfo->{unique_cont} : 0;
    bless $self, $class;
    $self->_initialize_clients(%opts);
    return $self;
}

sub add_client {
    my $self      = shift;
    my ($clientstr, %opts) = @_;
    my $count;
    my $type = $opts{type};
    if ($type) {
        $count = _parse_client_list($self, $clientstr, %opts);
    }
    $self->{error} = " No clients were specified " unless $count;
    return $self;
}

sub _parse_client_list {
    my $self      = shift;
    my $clientstr = shift;
    print "DBGCEB:_parse_client_list:clientstr = $clientstr\n";
    my (%opts) = @_;
    my $type      = $opts{type};
## @step test setup: parse clients
    my $howto = "<client1>;<hosttype>;<type>[;<type>];<OS>[,<client2>;...]";
    my $count = 0;
    my @types = ( 'nfs', 'cifs', 'iscsi', 'fcp' );
    for my $field ( split /,/, $clientstr ) {

        # Now we are looking for client definitions with protocol and
        # operation system specifications
        my ( $client, @fields ) = split /;/, $field;
        $client =~ s/\..*//;
        #### my  $clinfo = $self->{clients}{$client};
        my  $clinfo = {};
        $self->{clients}{$client} = $clinfo;
        print "$client @fields \n ";
        return if ( $self->{error} );
        $clinfo->{hname}   = $client;
        $clinfo->{hostname}   = $client;
        $clinfo->{subtest} = undef;
        $clinfo->{type} = {};

        foreach my $f (@fields) {
            print "DBGCEB: f = $f\n";
            $f = lc $f;
            if ( $f =~ /nfs|fc|cifs|iscsi/) {
                $f = 'fcp' if ($f =~ /fc/);
                $clinfo->{type}{$f} = {};
            } elsif ( $f =~ /hosttype\=([\w\-]+)/ ) {
                $clinfo->{hosttype} = $1;
                $clinfo->{fullname} = sprintf "%s;%s", $client,$f;

            }
        }
        if ($type) {
            my (@tlist) = split(/\W\s*/, $type);
            foreach (@tlist) {
                $clinfo->{type}{$_} = {};
            }
        }
        if ( !$clinfo->{hosttype} ) {
            my $fh = host($client);
            if (!$fh) {
                $self->{error} =
                  sprintf "Invalid CLIENT parameters no hosttype:\n%s\n%s",
                  return;
            }
            $clinfo->{hosttype} = $fh->hosttype();
            $clinfo->{default_ip} = $fh->default_ip();
            if ( !$clinfo->{hosttype} ) {
                $self->{error} =
                  sprintf "Invalid CLIENT parameters no hosttype:\n%s\n%s",
                $clientstr, $howto;
                return;
            }
        }
        $clinfo->{fullname} = "$client;hosttype=$clinfo->{hosttype}" unless ($clinfo->{fullname});
        if ( !keys(%{$clinfo->{type}}) ) {
            logresult('WARN', "No 'type' specified for client $client -- assigning type \'nfs\'");
            $clinfo->{type}{nfs} = 1;
        }

        $clinfo->{conn} = $opts{conn} || $main::CLIENT_CONN;
        $count++;
    }
    return $count;
}

sub _initialize_client_connection {
    my $self  = shift;
    my $cl_name = shift;
    my $cl_info = $self->{clients}{$cl_name};
    if ($cl_info->{cc}) {
        $cl_info->{cc} = make_conn($cl_info->{cc});
    } else {
        my $clconn = $cl_info->{conn};
        my $fullname = $cl_info->{fullname};
        my $cc;
        if ($clconn) {
            $cc = make_conn($fullname, $clconn);
        } else {
            $cc = make_conn($fullname, $clconn);
        }
        if (!$cc) {
            $self->{error} = "initialize_client: make_conn failed: $@";
            return 0;
        }
        my $ch = $cc->hostp();
        $cl_info->{cc} = $cc;
        $cl_info->{ch} = $ch;
        $cl_info->{hosttype} = $ch->hosttype();
    }
}


sub _initialize_clients {
    my $self  = shift;
    my (%opts) = @_;
    my $timeout = $opts{timeout} || $::CMDTIMEOUT;
    my $client_names = keys(%{ $self->{clients} });
    while ( my ($clnm, $clinfo) = each(%{ $self->{clients} }) ) {
        $self->_initialize_client_connection($clnm);
        $clinfo->{cc}->timeout($timeout) if (defined($timeout) && $timeout ne "");
    }
}

sub mount_clients {
    my $self  = shift;
    my $tinfo = shift;
    my (%opts) = @_;
    my $max_mounts = $opts{max_mounts};
    logcomment("CLient::mount_clients");
    foreach my $filer ( keys %{ $tinfo->{filers} } ) {
        $self->_mount_clients( $tinfo->{filers}{$filer}, $max_mounts);
    }
}

sub _mount_clients {
    my $self  = shift;
    my $filer = shift;
    my $maxmounts = shift;
    my $ckey = undef;
    logcomment("CLient::_mount_clients");
    print "DBGCEB:_mount_clients\n" if $DBGCEB;
    my $mntinfo = {
        'nfs'   => { 'mount' => \&_nfsmount },
#        'cifs'  => { 'mount' => \&_cifsmount },
        'fcp'    => { 'mount' => \&_fcmount },
        'iscsi' => { 'mount' => \&_iscsimount },
    };

    # we assign client ids if the clients need to have unique
    # containers.
#    if (($self->{unique_cont}) || ($self->{quotas}))

      if ($self->{quotas}) {
          $ckey = $QTREES;
      } else {
          $ckey = "volume";
      }

      my @clis = (keys %{$self->{clients}});
      my $ccount = (keys %{$self->{clients}});
      my $cntr = $ccount;
      foreach my $vol (keys(%{$filer->{containers}{$ckey}})) {
         $cntr--;
         if ($maxmounts) {
            $self->{clients}{$clis[$cntr]}{mcount} = 0
                if (!exists $self->{clients}{$clis[$cntr]}{mcount});
            next if ($self->{clients}{$clis[$cntr]}{mcount} >= $maxmounts);
            $self->{clients}{$clis[$cntr]}{mcount}++;
         }
         $filer->{containers}{$ckey}{$vol}{client} = $clis[$cntr];
         $cntr = $ccount if ($cntr == 0);
      }

    _initialize_clients();
    foreach my $client ( keys %{ $self->{clients} } ) {
        print "DBGCEB:_mount_clients:client = $client\n" if $DBGCEB;
        foreach my $type ( keys %{ $self->{clients}{$client}{type}} ) {
            print "DBGCEB:_mount_clients: $client type = $type\n" if $DBGCEB;
            $mntinfo->{$type}{mount}( $self, $client, $filer);
            return if $self->{error};
        }
    }
}


sub _nfsmount {
    my $self    = shift;
    my $client  = shift;
    my $filer   = shift;
    $self->_nfsmount_qtrees($client, $filer);
    $self->_nfsmount_vols($client, $filer);
#    my ${mount_qtrees} = defined($opts{mount_qtrees}) ? $opts{mount_qtrees} : $self->{quotas};
#    $self->_nfsmount_qtrees($client, $filer)
#        if ($mount_qtrees);
}


sub _nfsmount_container {
    my $self    = shift;
    my (%opts) = @_;
    my $client  = $opts{client};
    my $filer   = $opts{filer};
    my $sudo = "sudo";
    my $mp_type = $opts{mp_type};
    my $cli_info = $self->{clients}{$client};
    my $error   = undef;
    my $ipaddr = $filer->{ipaddr};
    my $local_path = $opts{path};
    my $con_hash = $opts{container};
    my $con_name = $con_hash->{name};
    my $mount_path = $opts{mount_dir} || "/t/$ipaddr/$con_name";
    my $vol = $opts{vol};
    my $mountopts = (exists($opts{mountopts})) ? $opts{mountopts} : 'actimeo=0,noac,hard,intr';
    my $use_old_mount_lib = ($> == 0);
    my $mount_fun;
    my $umount_fun;
    if ($use_old_mount_lib) {
        $mount_fun = \&mount_old::mount;
        $umount_fun = \&mount_old::mount;
    } else {
        $mount_fun = \&mount::mount;
        $umount_fun = \&mount::mount;
    }

    if ( !$cli_info->{cc} ) {
        $self->_initialize_client_connection($client);
    }
    if (!$mount_path) {
        logWarn("_nfsmount_container: No mount_path given");
        next;
    }
    if (!$local_path) {
        logWarn("_nfsmount_container: No local_path given");
        next;
    }
    if (!$vol) {
        logWarn("_nfsmount_container: No volume given");
        next;
    }

    $cli_info->{mounts} = {}
        unless (defined($cli_info->{mounts}));
    $con_hash->{mounts} = {}
        unless ($con_hash->{mounts});

    if ( $con_hash->{status} ne $DELETED ) {
        next if ( $cli_info->{mounts}{$mount_path}{status} eq $MOUNTED );
        next if ( !$con_hash->{export} );
        next if ( $mp_type && $con_hash->{mp_type} && $mp_type ne $con_hash->{mp_type} );
        logcomment("$client: adding mnt point $mount_path");
        my $mountopts = 'actimeo=0,noac,hard,intr';
        print "\n\nDBGCEB:_nfsmount:mounting filer $ipaddr \n\n",
          "path = $local_path mntpt = $mount_path on $client\n"
          if $DBGCEB;
        my $retries = $RETRIES;
        RETRYIT:
        my $mount_dir = eval { &$mount_fun(
                                	$ipaddr, $local_path,
                                	mountpoint => $mount_path,
#                                	mountopts  => $mountopts,
                                	client     => $client)
          		};
          if (!$mount_dir || $@) {
             $error = warn_result(
                  "$filer->{fname}:cannot create mountpoint " . "for $local_path: $@"
            );
          }
        if ($error) {
	    my $mount;
	    my @bin_dirs = qw( /usr/sbin /bin /usr/bin /sbin );
            my $find_output = sayEOF($cli_info->{cc}, "find @bin_dirs -name mount");
	    my @find_lines = split(/\n/, $find_output);
	    my $mount_binary;
            foreach (@find_lines) {
		s/^\s*//;
		s/\s*$//;
		if (/mount/) {
			$mount_binary = $_;
			last;
		}
	    }
	    die("_nfsmount_container: Could not find 'mount' binary") unless ($mount_binary);
            print "DBGCEB:_nfsmount:retrying mount via say\n";
            my $cmd = "$sudo $mount $ipaddr:$local_path $mount_path";
            my $output = sayEOF($cli_info->{cc},$cmd);
            print "DBGCEB:_nfsmount:retry mount with say:$output\n";
            $error = undef;
        }

        my $cmd = "$sudo chmod 0777 $mount_dir";
        my $output = eval { sayEOF($cli_info->{cc},$cmd) };
        warn_result("Error from 'chmod' on $client $mount_dir: $@");

        $cli_info->{mounts}{$mount_dir} = {}
            unless (defined($cli_info->{mounts}{$mount_dir}));
        $con_hash->{mounts}{$client} = {}
            unless ($con_hash->{mounts}{$client});
        my $mnt_info = $cli_info->{mounts}{$mount_dir} || {};
        $con_hash->{mounts}{$client} = $mnt_info;

        $mnt_info->{status}           = $MOUNTED;
        ### $mnt_info->{$filer->{fname}}  = $filer;
        $mnt_info->{filer}            = $filer->{fname};
        $mnt_info->{filer_info}       = $filer;
        $mnt_info->{client}           = $client;
        $mnt_info->{client_info}      = $cli_info;
        $mnt_info->{path}             = $local_path;
        $mnt_info->{type}             = "nfs";
        $mnt_info->{vol}              = $vol;
        $mnt_info->{mount_dir}        = $mount_dir;
        $mnt_info->{container}        = $con_hash;
        $mnt_info->{mp_type}          = $mp_type
            if ($mp_type);


        sleep(1);    #put back DBGCEB
        return ($mount_dir, $mnt_info);
    } else {
        my $mnt_info = $con_hash->{mounts}{$client}{mount_dir};
        my $mount_dir = $mnt_info->{mount_dir};
        next unless ($mount_dir);
        logcomment("Deleting mountpoint for $client:$mount_dir");
        &$umount_fun( $mount_dir, client => $client )
          or my $error = "WARN: $@";
        push @{ $self->{error} }, $@ if ($error);
        say( $cli_info->{cc}, "$sudo rmdir $mount_dir", '-eval_error' );
        $error = "WARN: $@" if $@;
        $mnt_info->{status} = $DELETED;

        push @{ $self->{error} }, $@ if ($error);
        delete ($con_hash->{mounts}{$client});
        return (undef, undef);
    }
}


sub _nfsmount_vols {
    my $self    = shift;
    my $client  = shift;
    my $filer   = shift;
    my $changed = 0;
    my $error   = undef;
    print "DBGCEB:_nfsmount_vols\n"; # if $DBGCEB;
    my $cli_info = $self->{clients}{$client};
    $cli_info->{filers}{$filer->{fname}}{ipaddr} = $filer->{ipaddr};
    my $vinfo = undef;
    $vinfo    = $filer->{containers}{volume};
    if ( !$cli_info->{cc} ) {
        print "DBGCEB: creating host obj $cli_info->{fullname}\n";
        my $ch = host( $cli_info->{fullname} );
        print "DBGCEB: connecting to client $cli_info->{fullname}\n"; # if $DBGCEB;
        my $cc = connect( $cli_info->{conn} );

        $cli_info->{cc} = $cc;
        $cli_info->{ch} = $ch;
    }
    my $count = 0;
    while (my ($vol, $volhash) = each(%$vinfo) ) {
         next if ((exists $volhash->{client}) &&
                 ($volhash->{client} ne $cli_info->{hname}));

        print "DBGCEB:_nfsmount: mounting $vol on $client\n"; # if $DBGCEB;
        my $path    = $volhash->{path};
        if (!$path) {
            logWarn("$vol has no exported path");
            next;
        }
        my $mntpt = "/t/$filer->{ipaddr}/$vol";
        my ($mount_dir, $mnt_info) = $self->_nfsmount_container(client=>$client,
                                                    filer=>$filer,
                                                    path=>$path,
                                                    mount_dir=>$mntpt,
                                                    container=>$volhash,
                                                    vol=>$vol,
                                                    mp_type=>'vol');
        if (!$mount_dir) {
logWarn("nfsmount_vol: Could not mount volume: $vol");
            next;
        }
        $mnt_info->{mp_type}  = 'vol';
        $count++;
   }
}


sub _nfsmount_qtrees {
    my $self    = shift;
    my $client  = shift;
    my $filer   = shift;
    my $changed = 0;
    my $error   = undef;
    print "DBGCEB:_nfs_mount\n" if $DBGCEB;
    my $cli_info = $self->{clients}{$client};
    my $qtreeinfo    = $filer->{containers}{qtrees};

    my $count = 0;

    while (my ($qtname, $qthash) = each(%$qtreeinfo) ) {
logresult('INFO', "nfsmount_qtree  QTREE: $qtname");
        my $mntpt = "/t/$filer->{ipaddr}/$qtname";
        my $path = $qthash->{path};
        my ($mount_dir, $mnt_info) = $self->_nfsmount_container(client=>$client,
                                                    filer=>$filer,
                                                    path=>$path,
                                                    mount_dir=>$mntpt,
                                                    container=>$qthash,
                                                    vol=>$qthash->{volume},
                                                    mp_type=>'qtree');
        if (!$mount_dir) {
logWarn("nfsmount_vol: Could not mount qtree: $qthash->{path}");
            next;
        }
        $mnt_info->{mp_type}  = 'qtree';
        $count++;

        if ($self->{quotas}) {
            $mnt_info->{uid} = $qthash->{uid};
            $mnt_info->{gid} = $qthash->{gid};
            my $quota_info = quota_info_for_mp($mnt_info);
            $mnt_info->{curr_size} = $quota_info->{curr_size};
            $mnt_info->{max_size} = $quota_info->{max_size};
            $mnt_info->{curr_files} = $quota_info->{curr_files};
            $mnt_info->{max_files} = $quota_info->{max_files};
        }
    }
}


sub quota_info_for_mp {
     my $mp = shift;
     my $qtree = $mp->{container};
     my $filer = $mp->{filer_info};
     my $path = $mp->{mount_dir};
     if ($mp->{mp_type} ne 'qtree') {
         logWarn("Client::quota_info_for_qtree: mount_point \"$path\" is not a qtree");
         return({});
     }
     return ($filer->quota_info_for_qtree($qtree));
}



#================================================================================
#  LUN stuff
#================================================================================


sub _linux_get_san_devices {
    my $self   = shift;
    my $client = shift;
    my $type = shift;
    my $cli_info = $self->{clients}{$client};
    my $cmd = "/usr/local/bin/sudo /sbin/devlabel status";
    my $out = sayEOF($cli_info->{cc}, $cmd);
    my @devs = grep(m%/$type/%, split(/\n/, $out));
    my @san_devs = ();
    foreach my $dev (@devs) {
        next unless ($dev =~ /([^\s]+)\s*->/);
        push(@san_devs, $1);
    }
    return(@san_devs);
}


sub _get_san_devices {
    my $self = shift;
    my $client = shift;
    my $type = shift;
    my $cli_info = $self->{clients}{$client};
    my $hosttype = $cli_info->{hosttype};
    if ($hosttype =~ /linux/) {
        return ($self->_linux_get_san_devices($client, $type) );
    } elsif ($hosttype =~ /solaris/) {
#        return ($self->_solaris_get_san_devices($client, $type) );
    }
}


sub _remove_san_devices {
    my $self   = shift;
    my $client = shift;
    my $type = shift;
    my $cli_info = $self->{clients}{$client};
    my @devs = $self->_get_san_devices($client, $type);
    foreach (@devs) {
        my $cmd = "/usr/local/bin/sudo /sbin/devlabel remove -s $_";
        my $out = sayEOF($cli_info->{cc}, $cmd);
    }
}


sub _initialize_iscsi {
    my $self   = shift;
    my $client = shift;
    my $cli_info = $self->{clients}{$client};
    return unless ($cli_info->{type}{iscsi});

    sayEOF($cli_info->{cc}, "/usr/local/bin/sudo /etc/rc.d/init.d/iscsi stop");
    sayEOF($cli_info->{cc}, "/usr/local/bin/sudo /sbin/chkconfig iscsi off");
    $self->_umount_san_luns($client, 'iscsi');
    $self->_remove_san_devices($client, 'iscsi');
    sayEOF($cli_info->{cc}, "/usr/local/bin/sudo /sbin/chkconfig iscsi on");
    sayEOF($cli_info->{cc}, "/usr/local/bin/sudo /etc/rc.d/init.d/iscsi restart");
    sayEOF($cli_info->{cc}, "/usr/local/bin/sudo /sbin/service iscsi reload");
    sleep(10);
}


sub _initialize_fcp {
    my $self   = shift;
    my $client = shift;
    my $cli_info = $self->{clients}{$client};
    return unless ($cli_info->{type}{fcp});

    $self->_umount_san_luns($client, 'fcp');
    $self->_remove_san_devices($client, 'fcp');

#######    More FCP Stuff Here #######

}


sub _configure_iscsi {
    my $self   = shift;
    my $client = shift;
    my $filer  = shift;

    my $tmp;
    my $cli_info = $self->{clients}{$client};
    if ( !$cli_info->{cc} ) {
        $self->_initialize_client_connection($client);
    }
    return unless ($cli_info->{type}{iscsi});

    $cli_info->{iscsi} = {} unless $cli_info->{iscsi};
    $self->_initialize_iscsi($client);

    my $initiatorname = $self->_get_iscsi_initiator_nodename($client);
    $cli_info->{iscsi}{initiator_nodename} = $initiatorname;

    print "DBGCEB:iSCSI initatiorname is $initiatorname\n";

    # Add initiator name to igroup
    ### $filer->add_igroup_client($self->{igroup}{name}, $cli_info->{hname}, $cli_info);

    sayEOF($cli_info->{cc}, "/usr/local/bin/sudo /etc/rc.d/init.d/iscsi restart");
    sayEOF($cli_info->{cc}, "/usr/local/bin/sudo /sbin/service iscsi reload");
    sleep(10);

    # Probe for iSCSI LUNs
    $self->_discover_iscsi_luns($client, $filer);
logresult('INFO', "Exit configure_iscsi");
    $cli_info->{iscsi}{configured} = 1;
}


sub _configure_fcp {
    my $self   = shift;
    my $client = shift;
    my $filer  = shift;

    my $tmp;
    my $cli_info = $self->{clients}{$client};
    if ( !$cli_info->{cc} ) {
        $self->_initialize_client_connection($client);
    }
    return unless ($cli_info->{type}{fcp});

    $self->_initialize_fcp($client);

    $cli_info->{fcp} = {} unless ($cli_info->{fcp});

    $cli_info->{fcp}{bindings} = $self->_get_fcp_bindings($client, $filer);

    $cli_info->{fcp}{initiators} = $self->_get_fcp_initiators($client);

    # Add initiator name to igroup
    ### $filer->add_igroup_client($self->{igroup}{name}, $cli_info->{hname}, $cli_info);


#######    More FCP Stuff Here #######

    sleep(10);



logresult('FATAL', "CONFIGURE_FCP");
    # Probe for FCP LUNs
    $self->_discover_fcp_luns($client, $filer);
    $cli_info->{fcp}{configured} = 1;
}


sub _iscsimount {
    my $self   = shift;
    my $client = shift;
    my $filer  = shift;

    my $cli_info = $self->{clients}{$client};
    $self->_configure_iscsi($client, $filer) unless ($cli_info->{iscsi}{configured});
    $self->_umount_san_luns($client, 'iscsi', $filer);
    $self->_mount_san_luns($client, 'iscsi', $filer);
}


sub _fcmount {
    my $self   = shift;
    my $client = shift;
    my $filer  = shift;
    ## my $op     = shift;

    my $cli_info = $self->{clients}{$client};
    $self->_configure_fcp($client, $filer) unless ($cli_info->{fcp}{configured});
    $self->_umount_san_luns($client, 'fcp', $filer);
    $self->_mount_san_luns($client, 'fcp', $filer);
}


sub _discover_iscsi_luns {
    my $self = shift;
    my $client = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $linfo = $cli_info->{luns};
    my $iinfo = $linfo->{iscsi};
    my $dinfo = $iinfo->{disks};
    my @lun_dev;
    my $count = 0;

    # Waste some time so the luns come up
    print "DBGCEB: Waiting for discovering iscsi luns\n\n";
    sayEOF($cli_info->{cc},
                "/usr/local/bin/sudo /usr/sbin/sanlun lun show $filer->{fname}", timeout=>600);
    sleep(5);
    say($cli_info->{cc},
                "/usr/local/bin/sudo /usr/sbin/sanlun lun show $filer->{fname}",'-blind');
    sleep(10);

    my $out = sayEOF($cli_info->{cc},
                "/usr/local/bin/sudo /usr/sbin/sanlun lun show $filer->{fname}", timeout=>600);

    my $hostfiler = $filer->{fname};
    foreach my $line (split(/\n/, $out)) {
        my ($hostpath, $disk_name, undef, $state) =
           ($out =~ /$filer->{fname}:\s+(\S+)\s+(\/dev\/\S+)\s+.*\s+iSCSI\s+(\w+\s+\(\w+\))\s+(.+)/gm);
        next unless ($hostpath);
        push(@lun_dev, $hostpath);
        if ($disk_name =~ /^.*\.1$/) {
                chop($disk_name);
        }
        my $disk = {};
        $disk->{name} = $disk_name;
        $disk->{device} = $disk_name;
        $disk->{hostpath} = $hostpath;
        $disk->{hostfiler} = $hostfiler;
        print "DBGCEB: Detected disk $disk_name\n";
        my $dout = sayEOF($cli_info->{cc},
                "/usr/local/bin/sudo /sbin/iscsi-device $disk_name");
        print "iscsi-device reports: $dout\n";
        if ($dout =~ /^Warning/) {
                my @lines = split(/\n/, $dout);
                $dout = $lines[1];
        }
        my $bus;
        my $target;
        my $lunid;
        my @rest;
        my $info;
        my $ip;
        (undef, $info, @rest) = split(/: /, $dout);
        (undef, undef, undef, undef, $ip, @rest) = split(/\s+/, $info);
        if ($ip =~ $filer->{ipaddr}) {
                (undef, $bus, $target, $lunid, @rest) = split(/\s+/, $info);
        } else {
                ($bus, $target, $lunid, @rest) = split(/\s+/, $info);
        }
        $disk->{bus} = $bus;
        $disk->{target} = $target;
        $disk->{lunid} = $lunid;
        my $undefined = 0;
        foreach (qw(bus target lunid name hostpath hostfiler)) {
            if (!defined($disk->{$_}) || $disk->{$_} eq "") {
                $undefined = 1;
                logresult('WARN', "discover_iscsi_luns:  Incomplete information for disk: \"$disk_name\"\n\n".Dumper($disk));
                # Just press on with the LUNs you've got #
            }
        };
        next if ($undefined);
        push(@lun_dev,$1);
        $disk->{path} = "/dev/iscsi/bus".$bus."/target".$target."/lun".$lunid;
        print "Discover adding disk path: $disk->{path}\n";
        $dinfo->{$disk_name} = $disk;
    }
    $iinfo->{disks} = $dinfo;
    $linfo->{iscsi} = $iinfo;
    $cli_info->{luns} = $linfo;

    print "Found iscsi luns: @lun_dev\n";
    $iinfo->{count} = @lun_dev;

    if (keys(%$dinfo) == 0) {
        $self->{error} = "discover_iscsi_luns: No valid LUNs found";
logresult('FATAL', "discover_iscsi_luns: No valid LUNs found -- 'sanlun' output:\n\n$out");
    }
logresult('FATAL', "LINFO:\n\n".Dumper($linfo));
}


sub _get_fcp_bindings {
    my $self = shift;
    my $client = shift;
    my $filer = shift;
    my $hash_by_wwpn = {};
    my $cli_info = $self->{clients}{$client};
    my $out = sayEOF($cli_info->{cc}, "grep -i wwpn /kernel/drv/lpfc.conf");
    foreach my $line (split(/\n/, $out)) {
        my ($wwpn, $adapter) = ($line =~ /^\s*fcp-bind-WWPN=\"([^:]+):(\w+)\"/i);
        next unless ($wwpn);
logresult('WARN', "GET_FCP_BINDINGS:  Line: \"$line\"");
        $hash_by_wwpn->{$wwpn} = $adapter;
    }
logresult('WARN', "GET_FCP_BINDINGS   HASH_BY_WWPN:\n\n".Dumper($hash_by_wwpn));
    return ($hash_by_wwpn);
}


sub _discover_fcp_luns {
    my $self = shift;
    my $client = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $linfo = $cli_info->{luns};
    my $iinfo = $linfo->{fcp};
    my $dinfo = $iinfo->{disks};
    my @lun_dev;
    my $count = 0;

    # Waste some time so the luns come up
    print "DBGCEB: Waiting for discovering fcp luns\n\n";
    sayEOF($cli_info->{cc},
                "/usr/local/bin/sudo /usr/sbin/sanlun lun show $filer->{fname}", timeout=>600);
    sleep(5);
    say($cli_info->{cc},
                "/usr/local/bin/sudo /usr/sbin/sanlun lun show $filer->{fname}",'-blind');
    sleep(10);

# Detect FCP luns. Add code for detecting FCP luns
#    my $out = sayEOF($cli_info->{cc},
#                "/usr/local/bin/sudo /usr/sbin/sanlun lun show $filer->{fname}");
#    while ($out =~ /$filer_id:\s+.*+(\/dev\/rdsk\/c\dt\dd\ds\d)\s+(lpfc|td)\d+\s+.*GOOD/gm) {
#        push(@lun_dev,$1) if ($1);
#    }
#    print "Found FCP luns: @lun_dev\n\n";
#    $client->{fcp}{luns} = @lun_dev;


#######    More FCP Stuff Here #######

}


sub _get_iscsi_initiator_nodename {
    my $self = shift;
    my $client = shift;
    my $cli_info = $self->{clients}{$client};

    my $output = sayEOF($cli_info->{cc}, "/usr/local/bin/sudo cat /etc/initiatorname.iscsi | grep InitiatorName=");
    my ($tmp, $initiatorname) = split(/=/, $output);
    if ($initiatorname eq "") {
        $self->{error} = config_result("Could not find iscsi initiator name of client\n");
        logcomment("Unknown client iscsi initatorname\n");
        return;
    }
}


sub _get_fcp_initiators {
    my $self = shift;
    my $client = shift;
    my $cli_info = $self->{clients}{$client};
    my @wwpn_list = ();
    my $out = sayEOF($cli_info->{cc},
                "/usr/local/bin/sudo /usr/sbin/sanlun fcp show adapter all", timeout=>600);
    my @lines = split(/\n/, $out);
    my $hash_by_initiator = {};
    foreach my $line (@lines) {
        my ($initiator, $wwpn) = ($line =~ /(\w+)\s+WWPN:\s*(\S+)/);
        next unless ($initiator);
        $hash_by_initiator->{$initiator} = $wwpn;
    }
logresult('WARN', "FCP_SHOW_INITIATORS:\n\n".Dumper($hash_by_initiator));
    return($hash_by_initiator);
}


sub _linux_umount_san_luns {
    my $self = shift;
    my $client = shift;
    my $type = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $devices;
    my $count;
    $count = $cli_info->{luns}{$type}{count};
    $devices = $cli_info->{luns}{$type}{disks};

    my $cnt = 0;

    my $out = sayEOF($cli_info->{cc}, "ls -1 /$type");
    my @remaining_dirs = split(/\n/, $out);
    foreach (@remaining_dirs) {
        my $dirpath = "/$type/$_";
        my $cmd = "/usr/local/bin/sudo /bin/umount $dirpath";
        sayEOF($cli_info->{cc}, $cmd);
        $cmd = "/usr/local/bin/sudo rm -rf $dirpath";
        sayEOF($cli_info->{cc}, $cmd);
    }

    $cli_info->{mounts} = {}
        unless (defined($cli_info->{mounts}));

    foreach my $device ( keys %{ $devices } ) {
            my $disk = $devices->{$device};
            my $diskpath = $disk->{path}."/disk";
            my $partition =$disk->{path}."/part1";
            my $mountpath = "/$type/".$disk->{bus}.$disk->{target}.$disk->{lunid};
            my $cmd;
            $cli_info->{mounts}{$mountpath} = {}
                unless (defined($cli_info->{mounts}{$mountpath}));
            my $mnt_info = $cli_info->{mounts}{$mountpath};
            next if ($filer && $filer->{fname} ne $mnt_info->{filer});

            $cmd = "/usr/local/bin/sudo /bin/umount $mountpath";
            sayEOF($cli_info->{cc}, $cmd);
            $cmd = "/usr/local/bin/sudo /sbin/devlabel remove -s $diskpath";
            sayEOF($cli_info->{cc}, $cmd);
            $cmd = "/usr/local/bin/sudo /bin/rmdir $mountpath";
            sayEOF($cli_info->{cc}, $cmd);
            $mnt_info->{status} = $UMOUNT;
            $mnt_info->{path}   = $mountpath;
            $mnt_info->{type}   = $type;
            $mnt_info->{mount_dir}    = $mountpath;
            $mnt_info->{mp_type}  = 'lun';
            $mnt_info->{filer} = $filer->{fname};
            $mnt_info->{filer_info} = $filer;
            $mnt_info->{client} = $client;
            $mnt_info->{client_info} = $cli_info;
            $cnt = $cnt+1;
    }
}


sub _solaris_umount_san_luns {
    my $self = shift;
    my $client = shift;
    my $type = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $devices;
    my $count;
    $count = $cli_info->{luns}{$type}{count};
    $devices = $cli_info->{luns}{$type}{disks};

    my $cnt = 0;

    my $out = sayEOF($cli_info->{cc}, "ls -1 /$type");
    my @remaining_dirs = split(/\n/, $out);
    foreach (@remaining_dirs) {
        my $dirpath = "/$type/$_";
        my $cmd = "/usr/local/bin/sudo /usr/sbin/umount $dirpath";
        sayEOF($cli_info->{cc}, $cmd);
        $cmd = "/usr/local/bin/sudo rm -rf $dirpath";
        sayEOF($cli_info->{cc}, $cmd);
    }

    foreach my $device ( keys %{ $devices } ) {
            my $disk = $devices->{$device};
            my $diskpath = $disk->{path}."/disk";
            my $partition =$disk->{path}."/part1";
            my $mountpath = "/$type/".$disk->{bus}.$disk->{target}.$disk->{lunid};
            my $cmd;
            $cli_info->{mounts}{$mountpath} = {}
                unless (defined($cli_info->{mounts}{$mountpath}));
            my $mnt_info = $cli_info->{mounts}{$mountpath};
            next if ($filer && $filer->{fname} ne $mnt_info->{filer});

            $cmd = "/usr/local/bin/sudo /usr/sbin/umount $mountpath";
            sayEOF($cli_info->{cc}, $cmd);
            $cmd = "/usr/local/bin/sudo /bin/rmdir $mountpath";
            sayEOF($cli_info->{cc}, $cmd);
            $mnt_info->{status} = $UMOUNT;
            $mnt_info->{path}   = $mountpath;
            $mnt_info->{type}   = $type;
            $mnt_info->{mount_dir}    = $mountpath;
            $mnt_info->{mp_type}  = 'lun';
            $mnt_info->{filer} = $filer->{fname};
            $mnt_info->{filer_info} = $filer;
            $mnt_info->{client} = $client;
            $mnt_info->{client_info} = $cli_info;
            $cnt = $cnt+1;
    }
}


sub _umount_san_luns {
    my $self = shift;
    my $client = shift;
    my $type = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $hosttype = $cli_info->{hosttype};
    if ($hosttype =~ /linux/) {
        return ($self->_linux_umount_san_luns($client, $type, $filer) );
    } elsif ($hosttype =~ /solaris/) {
        return ($self->_solaris_umount_san_luns($client, $type, $filer) );
    }
}


sub _linux_format_lun {
    my $self = shift;
    my $client = shift;
    my $partition = shift;
    my $cli_info = $self->{clients}{$client};
    my $cmd;
    $cmd = "/usr/local/bin/sudo /sbin/mkfs -t ext3 $partition";
    my $out = sayEOF($cli_info->{cc}, $cmd, timeout=>600);
    return( $out );
}


sub _solaris_format_lun {
    my $self = shift;
    my $client = shift;
    my $partition = shift;
    my $cli_info = $self->{clients}{$client};
    my $cmd;
    $cmd = "/usr/local/bin/sudo /usr/sbin/newfs $partition";
    my $out = sayEOF($cli_info->{cc}, $cmd, timeout=>600);
    return( $out );
}


sub _format_lun {
    my $self = shift;
    my $client = shift;
    my $partition = shift;
    my $cli_info = $self->{clients}{$client};
    my $hosttype = $cli_info->{hosttype};
    if ($hosttype =~ /linux/) {
        return ($self->_linux_format_lun($client, $partition) );
    } elsif ($hosttype =~ /solaris/) {
        return( $self->_solaris_format_lun($client, $partition) );
    }
}


sub _linux_partition_lun {
    my $self = shift;
    my $client = shift;
    my $diskpath = shift;
    my $cli_info = $self->{clients}{$client};
    my $cmd;

    my $fdisk_cmd_str = "d\\n1\\nn\\np\\n1\\n\\n\\np\\nw\\n";
    $cmd = "printf \"$fdisk_cmd_str\" |  /usr/local/bin/sudo /sbin/fdisk $diskpath";
    my $out = sayEOF($cli_info->{cc}, $cmd, timeout=>60);
    return($out);
}


sub _solaris_partition_lun {
    my $self = shift;
    my $client = shift;
    my $diskpath = shift;
    my $cli_info = $self->{clients}{$client};
    my $cmd;

    my $fdisk_cmd_str;  ###### <=== need to determine this

    $cmd = "printf \"$fdisk_cmd_str\" |  /usr/local/bin/sudo /usr/sbin/fdisk $diskpath";
    my $out = sayEOF($cli_info->{cc}, $cmd, timeout=>60);
    return($out);
}


sub _partition_lun {
    my $self = shift;
    my $client = shift;
    my $diskpath = shift;
    my $cli_info = $self->{clients}{$client};
    my $hosttype = $cli_info->{hosttype};
    if ($hosttype =~ /linux/) {
        return( $self->_linux_partition_lun($client, $diskpath) );
    } elsif ($hosttype =~ /solaris/) {
        return( $self->_solaris_partition_lun($client, $diskpath) );
    }
}


sub _linux_mount_san_luns {
    my $self = shift;
    my $client = shift;
    my $type = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $devices;
    my $count;
    $count = $cli_info->{luns}{$type}{count};
    $devices = $cli_info->{luns}{$type}{disks};

    my $cnt = 0;
    print "Mounting luns\n\n";
    while ( my ($disk_device, $disk) = each(%$devices) ) {
            my $lunpath = $disk->{hostpath};
            my $disk_symlink = $disk->{path}."/disk";
            my $partition_symlink = $disk->{path}."/part1";
            my $mountpath = "/$type/".$disk->{bus}.$disk->{target}.$disk->{lunid};
            my $device_path = "/dev/$type/bus".$disk->{bus}."/target".$disk->{target}."/lun".$disk->{lunid};
            my $partition_device = $disk_device . "1";
            my ($cmd, $out, $cmd_out);

            $cmd = "/usr/local/bin/sudo /bin/mkdir -p $mountpath";
            $out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "/usr/local/bin/sudo /bin/mkdir -p $device_path";
            $out = sayEOF($cli_info->{cc}, $cmd);

            $cmd = "ls $device_path >/dev/null";
            $out = sayEOF($cli_info->{cc}, $cmd);
            if ($out) {
                $self->{error} = "_linux_mount_san_luns: Could not create device path \"$device_path\" on client $client";
                return($self->{error});
            }

            # Remove partition symlink/devlabel
            $cmd = "/usr/local/bin/sudo /sbin/devlabel remove -s $partition_symlink";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "ls $partition_symlink >/dev/null";
            $out = sayEOF($cli_info->{cc}, $cmd);
            if (!$out) {
                $self->{error} = "_linux_mount_san_luns: devlabel remove -s $partition_symlink failed on client $client";
                logresult('FATAL', $self->{error});
                return($self->{error});
            }

            # Remove disk symlink/devlabel
            $cmd = "/usr/local/bin/sudo /sbin/devlabel remove -s $disk_symlink";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "ls $disk_symlink >/dev/null";
            $out = sayEOF($cli_info->{cc}, $cmd);
            if (!$out) {
                $self->{error} = "_linux_mount_san_luns: devlabel remove -s $disk_symlink failed on client $client";
                logresult('FATAL', $self->{error});
                return($self->{error});
            }
            $cmd = "/usr/local/bin/sudo /sbin/devlabel reload";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);

            $cmd_out = $self->_partition_lun($client, $disk_device);

            $cmd_out = $self->_format_lun($client, $partition_device);

            # Add disk symlink/devlabel
            $cmd = "/usr/local/bin/sudo /sbin/devlabel add -d $disk_device -s $disk_symlink";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "ls $disk_symlink >/dev/null";
            $out = sayEOF($cli_info->{cc}, $cmd);
            if ($out) {
                $self->{error} = "_linux_mount_san_luns: devlabel add -d $disk_device -s $disk_symlink failed on client $client: \"$out\"";
                logresult('FATAL', $self->{error});
                return($self->{error});
            }

            # Add partition symlink/devlabel
            $cmd = "/usr/local/bin/sudo /sbin/devlabel add -d $partition_device -s $partition_symlink";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "ls $partition_symlink >/dev/null";
            $out = sayEOF($cli_info->{cc}, $cmd);
            if ($out) {
                $self->{error} = "_linux_mount_san_luns: devlabel add -d $partition_device -s $partition_symlink failed on client $client";
                logresult('FATAL', $self->{error});
                return($self->{error});
            }

            $cmd = "/usr/local/bin/sudo /bin/mount $partition_symlink $mountpath";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            if ($cmd_out) {
                $self->{error} = "_linux_mount_san_luns: Cannot mount $partition_symlink to $mountpath via $type: $cmd_out";
                logcomment($self->{error});
                logresult('FATAL', $self->{error});
                return;
            }

            $cmd = "/usr/local/bin/sudo chmod 0777 $mountpath";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "ls -ld $partition_symlink";
            $out = sayEOF($cli_info->{cc}, $cmd);
            logresult('INFO', "_linux_mount_san_luns: mountdir:\n\n $out");

            if (-d "$mountpath/lost+found") {
                    $cmd = "/usr/local/bin/sudo /bin/rmdir $mountpath/lost+found";
                    sayEOF($cli_info->{cc}, $cmd);
            }
            $cli_info->{mounts}{$mountpath} = {}
                unless (defined($cli_info->{mounts}{$mountpath}));
            my $mnt_info = $cli_info->{mounts}{$mountpath};
            $mnt_info->{status} = $MOUNTED;
            $mnt_info->{filer}  = $filer->{fname};
            $mnt_info->{filer_info} = $filer;
            $mnt_info->{client} = $client;
            $mnt_info->{client_info} = $cli_info;
            $mnt_info->{path}   = $mountpath;
            $mnt_info->{type}   = $type;
            $mnt_info->{mount_dir}    = $mountpath;
            $mnt_info->{mp_type}  = 'lun';
            $mnt_info->{lun}  = $lunpath;
            $mnt_info->{disk}  = $disk->{name};
            $cnt = $cnt+1;
     }
}


sub _solaris_mount_san_luns {
    my $self = shift;
    my $client = shift;
    my $type = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $devices;
    my $count;
    $count = $cli_info->{luns}{$type}{count};
    $devices = $cli_info->{luns}{$type}{disks};

    my $cnt = 0;
    print "Mounting luns\n\n";
    while ( my ($disk_device, $disk) = each(%$devices) ) {
            my $lunpath = $disk->{hostpath};
            my $disk_symlink = $disk->{path}."/disk";
            my $partition_symlink = $disk->{path}."/part1";
            my $mountpath = "/$type/".$disk->{bus}.$disk->{target}.$disk->{lunid};
            my $device_path = "/dev/$type/bus".$disk->{bus}."/target".$disk->{target}."/lun".$disk->{lunid};
            my $partition_device = $disk_device . "1";
            my ($cmd, $out, $cmd_out);

            $cmd = "/usr/local/bin/sudo /bin/mkdir -p $mountpath";
            $out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "/usr/local/bin/sudo /bin/mkdir -p $device_path";
            $out = sayEOF($cli_info->{cc}, $cmd);

            $cmd = "ls $device_path >/dev/null";
            $out = sayEOF($cli_info->{cc}, $cmd);
            if ($out) {
                $self->{error} = "_solaris_mount_san_luns: Could not create device path \"$device_path\" on client $client";
                return($self->{error});
            }

            $cmd_out = $self->_partition_lun($client, $disk_device);

            $cmd_out = $self->_format_lun($client, $partition_device);

            $cmd = "/usr/local/bin/sudo /usr/sbin/mount $partition_symlink $mountpath";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            if ($cmd_out) {
                $self->{error} = "_solaris_mount_san_luns: Cannot mount $partition_symlink to $mountpath via $type: $cmd_out";
                logcomment($self->{error});
                logresult('FATAL', $self->{error});
                return;
            }

            $cmd = "/usr/local/bin/sudo chmod 0777 $mountpath";
            $cmd_out = sayEOF($cli_info->{cc}, $cmd);
            $cmd = "ls -ld $partition_symlink";
            $out = sayEOF($cli_info->{cc}, $cmd);
            logresult('INFO', "_solaris_mount_san_luns: mountdir:\n\n $out");

            if (-d "$mountpath/lost+found") {
                    $cmd = "/usr/local/bin/sudo /bin/rmdir $mountpath/lost+found";
                    sayEOF($cli_info->{cc}, $cmd);
            }
            $cli_info->{mounts}{$mountpath} = {}
                unless (defined($cli_info->{mounts}{$mountpath}));
            my $mnt_info = $cli_info->{mounts}{$mountpath};
            $mnt_info->{status} = $MOUNTED;
            $mnt_info->{filer}  = $filer->{fname};
            $mnt_info->{filer_info} = $filer;
            $mnt_info->{client} = $client;
            $mnt_info->{client_info} = $cli_info;
            $mnt_info->{path}   = $mountpath;
            $mnt_info->{type}   = $type;
            $mnt_info->{mount_dir}    = $mountpath;
            $mnt_info->{mp_type}  = 'lun';
            $mnt_info->{lun}  = $lunpath;
            $mnt_info->{disk}  = $disk->{name};
            $cnt = $cnt+1;
     }
}


sub _mount_san_luns {
    my $self = shift;
    my $client = shift;
    my $type = shift;
    my $filer = shift;
    my $cli_info = $self->{clients}{$client};
    my $hosttype = $cli_info->{hosttype};
    if ($hosttype =~ /linux/) {
        return ($self->_linux_mount_san_luns($client, $type, $filer) );
    } elsif ($hosttype =~ /solaris/) {
        return ($self->_solaris_mount_san_luns($client, $type, $filer) );
    }
}



#================================================================================
# main Clients subtest
#================================================================================

sub start_client_tests {
    my $self  = shift;
    my $tinfo = shift;
    my $testid = shift;

        my $count = 0;
        foreach my $client ( keys %{ $self->{clients} } ) {
            my $cliname = $self->{clients}{$client}{hname};
            print
"DBGCEB:nfs_clients_tests: starting client; $cliname\n";
#            _client_subtest($self->{clients}{$client},$tinfo;
            next if ( $self->{clients}{$client}{subtest} );
            $self->{clients}{$client}{testid} = "$testid:" . scalar($count+1);
            $self->{clients}{$client}{subtest} =
              subtest( \&_client_subtest, -bg, -ipc, -runid,
                "$cliname nfs subtest",
                '-onexit',\&_subtest_exit_handler,
                "--", $self->{clients}{$client}, $tinfo );
            $count++;
        }

        print "DBGCEB:_nfs_clients_subtest: no client subtests were started\n"
          if ( !$count );
}

sub _check_containers {
    my $self      = shift;
    my $tinfo     = shift;
    my $result    = undef;
    my $vols      = 0;
    my $qtrees    = 0;
    my $luns      = 0;
    my $volresult = 0;
    my $qtresult = 0;
    my $lunresult = 0;

    print "DBGCEB:_check_containers\n" if $DBGCEB;
    foreach my $type ( keys %{ $self->{types} } ) {
        if ( ( $type eq 'nfs' ) || ( $type eq "cifs" ) ) {
            $qtresult = $self->_check_qtrees( $tinfo, $type ) if ( !$qtrees );
            $volresult = $self->_check_volumes($tinfo) if ( !$vols );
            $self->_refresh_client_mounts('vol') if ($volresult || $qtresult);
            $vols = 1;
            $qtrees = 1;
        } elsif ( $type =~ /fcp|iscsi/ ) {
            $lunresult = $self->_check_luns( $tinfo, $type )
              if ( !$luns );
            $self->_refresh_client_mounts("lun") if $lunresult;
            $luns = 1;
        }
    }
    return ( $volresult + $lunresult );
}

sub log_messages {
    my $self = shift;
    my $msgs = shift;
    foreach my $m (@{$msgs}) {
       logcomment($m);
    }
    return;
}

sub _refresh_client_mounts {
    my $self   = shift;
    my $type   = shift;
    my @protos = ();
    if ( $type eq 'vol' ) {
        push @protos, "nfs", "cifs";
    } else {
        push @protos, "fcp", "iscsi";
    }

    foreach my $client ( keys %{ $self->{clients} } ) {
        foreach my $proto (@protos) {
            next if ( !exists $self->{clients}{$client}{type}{$proto} );
            my $subt = $self->{clients}{$client}{subtest};

            # bundy - NATE's ipc sockets keep crashing, so use a hammer!
            $subt->kill() if ($subt);
            $self->{client}{$client}{subtest} = undef;
        }

        # bundy - put this back my NATE's ipc sockets are working better
        #        next if !$subt;
        #        my $sock  = $subt->ipc();
        #        my $error = 0;
        #        $error = 1 if ( send_msg( $sock, $MOUNTS ) );
        #        $error = send_obj( $sock, $self->{clients}{$client}{mounts} )
        #          if !$error;
        #        if ($error) {
        #            my $msg = "unable to send mount info to $client";
        #            $self->_tell_mama($self->{parent},$msg);
        #        }

#        foreach my $mp ( keys %{ $self->{clients}{$client}{mounts} } ) {
#            delete $self->{clients}{clients}{mounts}{$mp}
#              if ( $self->{clients}{clients}{mounts}{$mp}{status} eq $DELETED );
#        }

    }
}

sub _add_client_mounts {
    my $self   = shift;
    my $type   = shift;
    my @protos = ();
    if ( $type eq 'vol' ) {
        push @protos, "nfs", "cifs";
    } else {
        push @protos, "fcp", "iscsi";
    }

    foreach my $client ( keys %{ $self->{clients} } ) {
        foreach my $proto (@protos) {
            next if ( !exists $self->{clients}{$client}{type}{$proto} );
            my $subt = $self->{clients}{$client}{subtest};

            # bundy - NATE's ipc sockets keep crashing, so use a hammer!
            $subt->kill() if ($subt);
            $self->{client}{$client}{subtest} = undef;
        }

        # bundy - put this back my NATE's ipc sockets are working better
        #        next if !$subt;
        #        my $sock  = $subt->ipc();
        #        my $error = 0;
        #        $error = 1 if ( send_msg( $sock, $MOUNTS ) );
        #        $error = send_obj( $sock, $self->{clients}{$client}{mounts} )
        #          if !$error;
        #        if ($error) {
        #            my $msg = "unable to send mount info to $client";
        #            $self->_tell_mama($self->{parent},$msg);
        #        }

#        foreach my $mp ( keys %{ $self->{clients}{$client}{mounts} } ) {
#            delete $self->{clients}{clients}{mounts}{$mp}
#              if ( $self->{clients}{clients}{mounts}{$mp}{status} eq $DELETED );
#        }

    }
}

sub _check_kids {
    return;    #DBGCEB just until we get this working
    my $self = shift;
    my $kids = shift;
    my $msg  = undef;
    my $obj  = undef;
    print "DBGCEB:_check_kids\n" if $DBGCEB;
    return                       if ( !defined $kids );
    foreach my $kid ( keys %{$kids} ) {
        print "DBGCEB:_check_kids: checking $kid\n" if $DBGCEB;
        next if ( !defined $kids->{$kid} );
        my $subt = $kids->{$kid}{subtest};
        next if ( !$subt );
        if ( !$subt->isAlive() ) {
            print
"\n\n**************** DBGCEB: subtest for $kid died! **************\n\n";
        }

#       $Data::Dumper::Maxdepth = 2;
#       print "DBGCEB: subt = ", Dumper($subt),"\n_check_kids: that was subt\n";;
#       getc;
        next if ( !defined $subt );
        next if ( !$subt->ipc()->can_read() );
        $msg = $subt->ipc()->read_message();
        print "DBGCEB:_check_kids: got msg: $msg\n";
        $subt->ipc()->write_message($OK);
        if ( $msg eq $RESULTS ) {
            print "DBGCEB:_check_kids: got results\n";
            $obj = $subt->ipc()->read_obj();
            print "DBGCEB:_check_kids: result obj = ", Dumper($obj),
              "_check_kids: that was result obj\n";
            my $msg = "$RESULTS $OK";
            subt->ipc()->write_message($msg);
        }
        _tell_mama( $self, $msg, $obj );
    }
}

sub _check_volumes {
    my $self  = shift;
    my $tinfo = shift;

    #    my $parent = Subtest::ipc();
    print "DBGCEB:_check_volumes\n" if $DBGCEB;
    my $changed = 0;
    foreach my $filer ( keys %{ $tinfo->{filers} } ) {
        $Data::Dumper::Purity = 1;
        my $ourvols = eval( Dumper( $filer->{containers}{volume} ) );
        $Data::Dumper::Purity = 0;

        #        my $fvols =  $filer->{containers}{volume};
        $filer->get_existing_containers("volumes");
        logWarn($filer->{error}) if (defined $filer->{error});
        my $filervolumes = $filer->{containers}{volume};

        #checking known volumes against the volumes on the filer
        foreach my $vol ( keys %{$ourvols} ) {
            if ( $filervolumes->{$vol} ) {

                #put our data back in the hash
                $filer->{containers}{volume}{$vol} = $ourvols->{$vol};
                next;
            }

            # this one isn't on the filer anymore, put it back in the has
            # and mark it as deleted.11111
            $filer->{containers}{volume}{$vol}         = $ourvols->{$vol};
            $filer->{containers}{volume}{$vol}{status} = $DELETED;
            $filer->{containers}{volume}{$vol}{filer}  =
              $tinfo->{filers}{$filer}{fname};
            $changed = 1;
            print "DBGCEB:$tinfo->{filers}{$filer}{fname}:$vol is deleted\n";
        }

        #        if ( !$changed ) {
        #now just for giggles find out if there are new volumes
        foreach my $vol ( keys %{ $filer->{containers}{volume} } ) {
            if ( !$ourvols->{$vol} ) {
                $changed = 1;
                print "DBGCEB:$tinfo->{filers}{$filer}{fname}:$vol is added\n";
            }
        }

        #       }

    }
    return if ( !$changed );
    logcomment("Clients:filer volumes changed");
    $self->mount_clients($tinfo);
    $self->_refresh_client_mounts();
    return $changed;

}

sub _check_qtrees {
    my $self  = shift;
    my $tinfo = shift;

    #    my $parent = Subtest::ipc();
    print "DBGCEB:_check_qtrees\n" if $DBGCEB;
    my $changed = 0;
    foreach my $filer ( keys %{ $tinfo->{filers} } ) {
        $Data::Dumper::Purity = 1;
        ### my $ourqtrees = eval( Dumper( $filer->{containers}{qtrees} ) );
        my $prevqtrees = { $filer->{containers}{qtrees} };
        $Data::Dumper::Purity = 0;

        $filer->get_existing_containers("qtrees");
        logWarn($filer->{error}) if (defined $filer->{error});
        my $currqtrees = { $filer->{containers}{qtrees} };

        #checking known (old) qtrees against the current qtrees on the filer
        foreach my $qtx ( keys %{$prevqtrees} ) {
            if ( $currqtrees->{$qtx} ) {
                next;
            }
            my $qthash = $prevqtrees->{$qtx};
            # this one isn't on the filer anymore, put it back in the has
            # and mark it as deleted.11111
            $filer->{containers}{qtrees}{$qtx}         = $qthash;
            $qthash->{status} = $DELETED;
            $qthash->{filer}  = $tinfo->{filers}{$filer}{fname};
            $changed = 1;
            print "DBGCEB:$tinfo->{filers}{$filer}{fname}:$qtx is deleted\n";
        }

        #        if ( !$changed )
        #now just for giggles find out if there are new qtrees
        foreach my $qtx ( keys %{$currqtrees} ) {
            if ( $prevqtrees->{$qtx} ) {
                next;
            }
            $changed = 1;
            print "DBGCEB:$tinfo->{filers}{$filer}{fname}:$qtx is added\n";
        }

    }
    return if ( !$changed );
    logcomment("Clients:filer qtrees changed");
    $self->mount_clients($tinfo);
    $self->_refresh_client_mounts();
    return $changed;
}

sub call_home {
    my $self = shift;
    my $msg  = shift;
    my $id   = shift;
    my $sock = Subtest::ipc();
    if ( ref($msg) ) {
        if ( send_msg( $sock, $RESULTS ) ) {
            logresult( "WARN", "Clients: Cannot phone home" );
        } elsif ( send_obj( $sock, $msg ) ) {
            logresult( "WARN", "Clients: Cannot phone home" );
        }
    }
}

sub _log_results {
    my $err = shift;
    my @errors = ref($err) ? [$err] : ($err);
    foreach my $e (@errors) {
        if ( $e =~ /FAIL/ ) {
            logresult( "FAIL", $e );
        } elsif ( $e =~ /WARN/ ) {
            logresult( "WARN", $e );
        } else {
            logcomment($e);
        }
    }
}

#===============================================================================
# per client subtests - these create per protocol subtests
#===============================================================================

sub _check_parent {
    return;    #DBGCEB just until we get this working
    my $self    = shift;
    my $kids    = shift;
    my $parent  = $self->{parent};
    my $message = undef;
    my $mounts  = undef;
    my $caller  = undef;
    print "DBGCEB:_check_parent \n" if $DBGCEB;

    while (1) {
        return if ( ( !$parent->can_read() ) && ( $message ne $SUSPEND ) );
        $message = $parent->read_message();
        print "DBGCEB: got message $message\n"     if $DBGCEB;
        print "DBGCEB:_check_parent: writing OK\n" if $DBGCEB;

        $parent->write_message($OK);

        #DBGCEB
        print "DBGCEB:check_parent message $message\n" if $DBGCEB;
        print "DBGCEB:check_parent message to kids\n"  if $DBGCEB;
        my $errors = _call_kids( $kids, $message );
        return;

        #DBGCEB end
        if ( $message eq $MOUNTS ) {
            while ( !$parent->can_read() ) { sleep 1 }
            $mounts = $parent->read_obj();
            $parent->write_message($OK);
        }
        $errors = _call_kids( $self->{type}, $message, $mounts );
        process_result_error( $self, $errors, $message ) if ($errors);
        return $message if ( $message eq $QUIT );
        return if ( $message ne $SUSPEND );
    }
}

sub _tell_mama {
    return;    #DBGCEB just until we get this working
    my $sock    = shift;
    my $msg     = shift;
    my $obj     = shift;
    my $retries = 3;

    #    return if (!_can_write($sock,$msg));
    $sock->write_message($msg);
    return if ( !_getack( $sock, $msg ) );
    return if ( !$obj );

    #    return if (!_can_write($sock,$msg));
    $sock->write_obj($obj);
    _getack( $sock, $msg );
    return;
}

sub _getack {
    my $sock    = shift;
#    my $msg     = shift;
    my $retries = 3;
    my $result  = 0;
    my $msg     = undef;
    for ( 1 .. $retries ) {
        sleep 2;
        next if ( !$sock->can_read() );
        $msg = $sock->read_message();
        last if ( $msg !~ /$msg $OK/ );
        $result = 1;
    }
    logWarn(
        "NfsFunc:_getack: sending $msg or object: no ack from " . "parent" )
      if ( !$result );

    return $result;
}

#================================================================================
# protocol based subtests
#================================================================================

sub _client_subtest {
    my $self  = shift;
    my $tinfo = shift;
    bless $self, $CLASS;
    print "DBGCEB:Client:client_subtest $self->{hname}:\n";


    foreach my $filer (keys %{$tinfo->{filers}}) {
        print "DBGCEB:_client_subtest: filer = $filer\n";
        $self->{filers}{$filer}{fc} = $tinfo->{filers}{$filer}{fc};
        $self->{filers}{$filer}{fh} = $tinfo->{filers}{$filer}{fh};
        $self->{filers}{$filer}{ipaddr} = $tinfo->{filers}{$filer}{ipaddr};
    }
    print "DBGCEB:_client_subtest: calling NfsFuncs\n";
    my $nfsfuncs = NfsFuncs->new($self, "nfs");
#    my $nfsfuncs = IFTFDOTests->new($self, "nfs");
    $self->{funcs} = $nfsfuncs;
    print "DBGCEB:_start_nfs_tests: starting ops\n";
    $nfsfuncs->start_ops($self);
    $self->{tests}{$nfsfuncs}{subtest} = $nfsfuncs->{dfops}{subtest};
    $self->{error} = $nfsfuncs->{error};
    print "DBGCEB: ***** Client subtest ends: $self->{hname} *****\n";
    return;
}

sub _check_test_status {
    my $self;
    my $result  = undef;
    my $subinfo = shift;
    my $subt    = $subinfo->{subtests};
    my $op      = undef;
    while (1) {

        if ( !Subtest::ipc()->can_read() ) {
            return if ( $op ne $SUSPEND );
        }
        $op = Subtest::ipc()->read_message();

        if (   ( $op eq $SUSPEND )
            || ( $op eq $RESUME ) )
        {
            $result = $subt->send_ops($op);
        } elsif ( $op eq $QUIT ) {
            $result = $subt->stop_ops();
            push @{ $self->{error} }, [ @{ $subt->{error} } ];
        } elsif ( $op eq $MOUNT ) {
            $result = $subt->change_mounts( $self->{mounts} );
        } else {
            Subtest::ipc()->write_message($NOT_OK);
            return;
        }
        return $result;
    }
}

sub _iscsisubtst {
    my $self   = shift;
    my $result = undef;
    bless $self, $CLASS;
    print "DBGCEB:_iscsisubtst: started\n";
    $self->{tests}  = undef;
    $self->{parent} = Subtest::ipc();

#    print  "DBGCEB:_iscsisubtst: parent = ", Dumper($self->{parent}),"\n_iscsisubtst: that was parent\n";
#    print "DBGCEB:Client:_iscsisubtst self = ",Dumper($self);
#    print "DBGCEB:Client:_iscsisubtst that was self\n";
#    getc; #DBGCEB

    # get the iscsi mountpoint for this client
    #   print "DBGCEB:_iscsisubtst: starting iscsi tests\n";
    $self->_start_iscsi_tests();

  #    if ($self->{error}) {
  #        my $msg = "CONFIG: cannot start iscsi client subtests: $self->{error}";
  #        $self->_tell_mama($msg);
  #        return;
  #    }

    while (1) {
        print "\nDBGCEB:_iscsisubtst: checking parent\n" if $DBGCEB;
        $self->_check_parent( $self->{tests} );
        print "\nDBGCEB:_iscsisubtst: checking kids\n" if $DBGCEB;
        $self->_check_kids( $self->{tests} );
        sleep 3;
    }
}

sub _start_iscsi_tests {
    my $self = shift;

    print "DBGCEB:Client:_start_iscsi_tests\n";

    #    getc; #DBGCEB
    #    foreach my $mp (keys %{$self->{mounts}}) {
    #       next if ($self->{mounts}{$mp}{type} ne "iscsi");
    #       $subinfo->{mounts}{$mp} = $self->{mounts}{$mp};
    #    }
    print "DBGCEB:_start_iscsi_tests: calling NfsFuncs\n";
    my $nfsfuncs = NfsFuncs->new($self, "iscsi");
#    my $nfsfuncs = IFTFDOTests->new($self, "iscsi");
    $self->{funcs} = $nfsfuncs;
    print "DBGCEB:_start_iscsi_tests: starting ops\n";
    $nfsfuncs->start_ops($self);
    $self->{tests}{$nfsfuncs}{subtest} = $nfsfuncs->{dfops}{subtest};
    $self->{error} = $nfsfuncs->{error};
    return;
}

sub _fcsubtst {
    my $self   = shift;
    my $result = undef;
    bless $self, $CLASS;
    print "DBGCEB:_fpsubtst: started\n";
    $self->{tests}  = undef;
    $self->{parent} = Subtest::ipc();

#    print  "DBGCEB:_fcsubtst: parent = ", Dumper($self->{parent}),"\n_fcsubtst: that was parent\n";
#    print "DBGCEB:Client:_fcsubtst self = ",Dumper($self);
#    print "DBGCEB:Client:_fcsubtst that was self\n";
#    getc; #DBGCEB

    # get the fcp mountpoint for this client
    #   print "DBGCEB:_fcsubtst: starting fcp tests\n";
    $self->_start_fcp_tests();

  #    if ($self->{error}) {
  #        my $msg = "CONFIG: cannot start fcp client subtests: $self->{error}";
  #        $self->_tell_mama($msg);
  #        return;
  #    }

    while (1) {
        print "\nDBGCEB:_fcsubtst: checking parent\n" if $DBGCEB;
        $self->_check_parent( $self->{tests} );
        print "\nDBGCEB:_fcsubtst: checking kids\n" if $DBGCEB;
        $self->_check_kids( $self->{tests} );
        sleep 3;
    }
}

sub _start_fcp_tests {
    my $self   = shift;

    print "DBGCEB:Client:_start_fcp_tests\n";

    #    getc; #DBGCEB
    #    foreach my $mp (keys %{$self->{mounts}}) {
    #       next if ($self->{mounts}{$mp}{type} ne "fcp");
    #       $subinfo->{mounts}{$mp} = $self->{mounts}{$mp};
    #    }
    print "DBGCEB:_start_fcp_tests: calling NfsFuncs\n";
    my $nfsfuncs = NfsFuncs->new($self, "fcp");
#    my $nfsfuncs = IFTFDOTests->new($self, "fcp");
    $self->{funcs} = $nfsfuncs;
    print "DBGCEB:_start_fcp_tests: starting ops\n";
    $nfsfuncs->start_ops($self);
    $self->{tests}{$nfsfuncs}{subtest} = $nfsfuncs->{dfops}{subtest};
    $self->{error} = $nfsfuncs->{error};
    return;
}

#==============================================================================
# common utilities
#==============================================================================

sub _call_kids {
    return;    #DBGCEB just until we get this working
    my $kids    = shift;
    my $msg     = shift;
    my $object  = shift;
    my $retries = 0;
    my @errs    = ();
    print "DBGCEB:_call_kids: msg = $msg\n" if $DBGCEB;
    foreach my $kid ( keys %{$kids} ) {

        #        $Data::Dumper::Maxdepth = 2;
        #        print "DBGCEB:_call_kids: kid = $kid\n";
        #        print "DBGCEB:_call_kids = kid hash = ",Dumper($kids->{$kid}),
        #              "\n that was _call_kids:kid hash\n";

        #        getc;
        next if ( !ref( $kids->{$kid} ) );
        next if ( !defined $kids->{$kid}{subtest} );
        my $subt = $kids->{$kid}{subtest};
        next if !$subt;
        my $sock = $subt->ipc();
        print "DBGCEB:_call_kids:sending msg $msg\n" if $DBGCEB;
        my $result = send_msg( $sock, $msg );
        print "DBGCEB:_call_kids:result from send_msg: $result\n" if $DBGCEB;

        if ($result) {
            push @errs, $kid;
            next;
        }
        $result = send_obj( $sock, $object ) if ($object);
        push @errs, $kid if $result;
    }
    return @errs;
}


sub send_obj {
    my $sock = shift;
    my $obj  = shift;

    foreach ( 1 .. 4 ) {
        $sock->write_obj($obj);
        return if ( !get_ack($sock) );
    }
    return 1;
}

sub send_msg {
    my $sock = shift;
    my $msg  = shift;
    print "DBGCEB:send_msg: $msg\n" if $DBGCEB;
    foreach ( 1 .. 4 ) {
        print "DBGCEB:send_msg:writing message\n" if $DBGCEB;
        $sock->write_message($msg);
        return if ( !get_ack($sock) );
    }
    return 1;
}

sub get_ack {
    my $sock = shift;
    print "DBGCEB:get_ack:\n" if $DBGCEB;
    foreach ( 1 .. 4 ) {
        sleep 5;
        next if ( !$sock->can_read() );
        my $ack = $sock->read_message();
        return $ack if ( $ack ne $OK );
        return;
    }
    print "DBGCEB:get_ack: no ACK from socket\n" if $DBGCEB;
    return 1;
}

sub process_result_error {
    my $self    = shift;
    my $errors  = shift;
    my $msg     = shift;
    my @errmsgs = ();
    my $sock    = $self->{parent};

    foreach my $kid (@$errors) {
        my $warn = "WARNING:Clients:cannot send $msg to $kid";
        push @errmsgs, $warn;
    }
    my $result = send_msg( $sock, $RESULTS );
    $result = send_obj( $sock, @errmsgs ) if ( !$result );
    if ($result) {
        my $warn = "WARNING:Clients: cannot phone home";
        push @errmsgs, $warn;
        _log_results(@errmsgs);
    }
}

sub _subtest_exit_handler {
    my ($subtest) = @_;
    print "DBGCEB: Client.pm exit_handler\n",Dumper($subtest);
    my @results = $subtest->rslts();

    foreach my $result (@results) {
       if (ref($result) =~ /Fatal/) {
          my $message = sprintf($result);
         logresult("FAIL",$message,abort=>1);
       } else {
         logresult($result);
       }
    }
}

#sub send_subs {
#    my $self    = shift;
#    my $message = shift;
#    my $object  = shift;
#    my $subt    = undef;
#    my @errs    = ();
#    foreach my $type ( @{ $self->{type} } ) {
#        my $sock = $self->{type}{$type}{subtest}->ipc();
#        my $result = send_msg( $sock, $message );
#        if ($result) {
#            push @errs, $type;
#            next;
#        }
#        $result = send_obj( $sock, $object ) if ($object);
#        push @errs, $type if $result;
#    }
#    return @errs;
#}

#sub change_mounts {
#    my $self = shift;

#    foreach my $client ( keys %{ $self->{clients} } ) {
#    }

#}

#sub _cifsmount {
#    my $self   = shift;
#    my $client = shift;
#    my $op     = shift;

#}

#sub _initialize_clients {
#    my $self  = shift;
#    my $client_names = keys(%{ $self->{clients} });
#    while ( my ($clnm, $clinfo) = each(%{ $self->{clients} }) ) {
#        if ($clinfo->{cc}) {
#            $clinfo->{cc} = make_conn($clinfo->{cc});
#        } else {
#            my $clconn = $clinfo->{conn};
#            my $fullname = $clinfo->{fullname};
#            my $cc;
#            if ($clconn) {
#                $cc = make_conn($fullname, $clconn);
#            } else {
#                $cc = make_conn($fullname, $clconn);
#            }
#            if (!$cc) {
#                $self->{error} = "initialize_client: make_conn failed: $@";
#                return 0;
#            }
#            my $ch = $cc->hostp();
#            $clinfo->{cc} = $cc;
#            $clinfo->{ch} = $ch;
#            $clinfo->{hosttype} = $ch->hosttype();
#        }
#    }
#}

#sub _nfsmount {
#    my $self    = shift;
#    my $client  = shift;
#    my $filer   = shift;
#    $self->_nfsmount_qtrees($client, $filer) if (exists $self->{qtrees});
#    $self->_nfsmount_vols($client, $filer);
#}

#sub _fcmount {
#    my $self   = shift;
#    my $client = shift;
#    my $op     = shift;

#}

# input  individual client object and mountpoint
#sub _delete_client_mount {
#    my $self = shift;
#    my $client = shift;
#    my $mp = shift;
#    my $cli_info = $self->{clients}{$client};
#    my $sock = $cli_info->{subtest}->ipc();
#    my $msg = "$DELETEMOUNT=$mp;EMSG";
#    my $resp = undef;
#    my @msgs = ();
#    my $ack = undef;
#    my $retries = $RETRIES;
#    my $error = undef;
#    while ($retries) {
#    $sock->write_message($msg);
#    sleep(5);
#    while ($sock->can_read()) {
#       $resp = $sock->read_message();
#       next if (!$resp);
#       next if ($resp eq "");
#       if ($resp =~ /ACK\=(\w)\;$DELETEMOUNT\=$mp\;EMSG/) {
#           $ack = $1;
#       }
#       push @msgs, $resp;
#    }
#       last if ($ack eq $ACK);
#       sleep(5);
#       $retries--;
#    }
#    if ($ack eq $ACK) {
#    mount::umount( $mp, client => $client )
#      or $error = "WARN: unable to delete mountpoint $mp $self->{hname}: $@";
#    logWarn($error) if ($error);
#    }
#    log_messages($self,/@msgs)
#    return $ack;
#}
#sub start_protocol_subtests {
#    my $self  = shift;
##    my $tinfo = shift;    # test info
#    print "DBGCEB:Client:start_file_dir_subtests:\n";
#    my $subtinfo = {
#        'nfs'   => { 'subtst' => \&_nfssubtst },
#        'cifs'  => { 'subtst' => \&_cifssubtst },
#        'fcp'    => { 'subtst' => \&_fcsubtst },
#        'iscsi' => { 'subtst' => \&_iscsisubtst },
#    };

#    foreach my $type ( keys %{ $self->{type} } ) {
#         $subtinfo->{$type}{subtst}($self);
##        my $test = $subtinfo->{$type}{subtst};
##        $self->{type}{$type}{subtest} =
##          subtest( $test, -bg, -ipc, -runid, "$self->{hname} $type subtest",
##                  '-onexit',\&_subtest_exit_handler,
##                  "--", $self );
#        return if $self->{error};
#    }

#}

#sub _nfssubtst {
#    my $self   = shift;
#    my $result = undef;
#    bless $self, $CLASS;
#    print "DBGCEB:_nfssubtst: started\n";
#    $self->{tests}  = undef;
#    $self->{parent} = Subtest::ipc();

##    print  "DBGCEB:_nfssubtst: parent = ", Dumper($self->{parent}),"\n_nfssubtst: that was parent\n";
##    print "DBGCEB:Client:_nfcsubtst self = ",Dumper($self);
##    print "DBGCEB:Client:_nfcsubtst that was self\n";
##    getc; #DBGCEB

#    # get the nfs mountpoint for this client
#    #   print "DBGCEB:_nfssubtst: starting nfs tests\n";
#    _start_nfs_tests($self);

#  #    if ($self->{error}) {
#  #        my $msg = "CONFIG: cannot start nfs client subtests: $self->{error}";
#  #        $self->_tell_mama($msg);
#  #        return;
#  #    }

##    while (1) {
##        print "\nDBGCEB:_nfssubtst: checking parent\n" if $DBGCEB;
##        $self->_check_parent( $self->{tests} );
##        print "\nDBGCEB:_nfssubtst: checking kids\n" if $DBGCEB;
##        $self->_check_kids( $self->{tests} );
##        sleep 3;
##    }
#}


# ------------------------------------------------------------------------
1;

# ------------------------------------------------------------------------
