# -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTComm.pm#1 $:

package IFTComm;
use IO::Socket;

#use IO::Handle;
use Net::hostent;    # for OO version of gethostbyaddr
use Data::Dumper;
use Storable qw(nstore store_fd nstore_fd freeze thaw dclone);


our $PORT      = 6000;            # pick something not in use
our $HOST      = "localhost";
our $GETSTATUS = "get status";
our $SETSTATUS = "set status ";
our $EXIT      = "exit";
our $testinfo  = {};
our $filerinfo = {};
our $self;
our $hostname = undef;


sub new {
    my $class = shift;
    my %args  = @_;
    return if ( !exists $args{op} );
#    return if ( !exists $args{filers} );
    my $self = {};
    $self->{op}     = $args{op};
    $self->{client} = $args{client};
    if (exists $args{filers}) {
    $self->{filers} = $args{filers} if (exists $args{filers});

      foreach my $f ( keys %{ $self->{filers} } ) {
          $self->{filers}{ $self->{filers}{$f}{fname} }{status} = undef;
      }
    }
    bless $self, $class;
    if ( $self->{op} eq "start" ) {
        return $self if (exists $self->{hostname}); #we already started it
        $hostname = `hostname`;
        $self->{hostname} = $hostname;
        my $pid = fork();
        die "status server not up (cannot fork)" if (!defined $pid);
        if ( !$pid ) {
            $self->start_status_server();
            return;
        } else {
            $self->{pid} = $pid;
            return $self;
        }
    } elsif ( $self->{op} eq "conn" ) {
        $self->connect_to_status_server(%args);
        return $self;
    } else {
        return;
    }
}

sub get_sserver {
    return $self;
}

sub start_status_server {
    my $self   = shift;
    my $server = IO::Socket::INET->new(
        Proto     => 'tcp',
        LocalPort => $PORT,
        Listen    => SOMAXCONN,
        Reuse     => 1
    );
    die "Cannot connect $!" if ( !$server );

    IO::Socket->output_record_separator("\n");

    my $sread  = new IO::Select();
    my $swrite = new IO::Select();
    $self->{sread} = $sread;
    $self->{swrite} = $swrite;
    $swrite->add($server);
    $sread->add($server);
    my $testlog = "/u/bundy/logs/tc1".time();
    $self->{logfile} = $testlog;
    open TESTLOG,"+>$testlog" or
         return ($self->{error} = "WARNING: can't open testlog\n");
    while (1) {
#        print "waiting on a client to connect or ready to read...\n";
        my ($rdsock) = IO::Select->select($sread);
        my $monitor = {};

        foreach my $sock (@$rdsock) {
            if ( $sock == $server ) {
                my $news = $sock->accept();
                $sread->add($news);
#                $swrite->add($news);
                print "got a new one!\n";
                next;
            }
#            print "reading socket...\n";
            my $tinfo =  <$sock>;
            print "DBGCEB:start_status_server: tinfo = $tinfo\n";
            print $sock "ACK";
            my $thash =  _parse_data_in($tinfo);
            _process_data_in($thash,$sock);
            next;
       }
    }
}

sub _stop_server {
        my $sread = $self->{sread};
        my $socks = $sread->handles;
        print "closing down the sockets...\n";
        foreach my $s (@$socks) {
                close $s;
        }
        print "adios!\n";
        exit(0);
}

sub _parse_data_in {
    my $tinfo = shift;
    my $key = undef;
    my $value = undef;
    my $hash = undef;
    if ($tinfo =~ /EXIT/) {
        _stop_server();
    }
    my @fields = (split /,/,$tinfo);
    foreach my $field (@fields) {
      my ($key, $value) = split /=>/,$field;
      print "DBGCEB: _parse_data_in: key = $key  value = $value\n";
      $hash->{$key} = $value;
    }
    if (exists $hash->{id}) {
       my $tid = $hash->{id};
       $self->{tests}{$tid} = $hash;
       print "DBGCEB:_parse_data_in: hash = ",Dumper($self->{tests}{$tid});
    }
    return $hash;
}


sub _process_data_in {
    my $tinfo = shift;
    my $sock = shift;
    if ($tinfo->{type} eq "START") {
        $self->{tests}{$tinfo->{id}} = {};
        $self->{tests}{$tinfo->{id}}{sock} = $sock;
        _process_start_msg($self->{tests}{$tinfo->{id}});
    } elsif ($tinfo->{type} eq "RESULT") {
        _process_result_msg($tinfo);
        delete $self->{tests}{$tinfo->{id}}  if (exists $tinfo->{endtest});
    } elsif ($tinfo->{type} eq "FILER") {
        _process_filer_msg($tinfo);
    } elsif ($tinfo->{type} eq "NOTIFY") {
        _process_notify_msg($tinfo);
    } elsif ($tinfo->{type} eq "STATUS") {
        _process_status_msg($tinfo);
    } else {
        _process_misc_msg($ti);
    }
}

sub _process_start_msg {
    my $tinfo = shift;
    $tinfo->{starttime} = time();
    my $header = sprintf "ID=%s;MSGNO=%s;TESTNAME=%;STARTTEST;STARTTIME=%s;",
                          $tinfo->{id},$tinfo->{msg},$tinfo->{testname},
                          $tinfo->{starttime};
    my $logmsg = $header;
    foreach my $key (keys %{$tinfo}) {
        next if (
           ($key eq 'id') ||
           ($key eq 'msg') ||
           ($key eq 'testname') ||
           ($key eq 'starttime'));
        $logmsg .= sprintf "%s=%s", uc $key, $tinfo->{$key};
    }
    _logit($logmsg);
}

sub _process_result_msg {
    my $ti = shift;
    my $tid = $tinfo->{id};
    my $tinfo = {};
    foreach my $key (keys %{$ti}) {
        $tinfo->{$key} = $ti->{$key};
    }

    my $logmsg =
          sprintf "ID=%s;MSGNO=%s;TESTNAME=%s;",
                          $tinfo->{id},$tinfo->{msg},$tinfo->{testname};
          delete $tinfo->{id};
          delete $tinfo->{msg};
          delete $tinfo->{testname};
    if (exists $tinfo->{endtest}) {
        my $endtime = time();
        $logmsg .= "ENDTEST;RESULT=$tinfo->{result};ENDTIME=$endtime;",
          my $duration = $endtime - $tinfo->{starttime};
          $logmsg .= "DURATION=$duration";
          delete $tinfo->{result};
          delete $tinfo->{starttime};
          delete $tinfo->{endtest};
    } else {
        $logmsg .= "RESULT=$tinfo->{result}";
          delete $tinfo->{result};
    }

    foreach $key (keys %{$tinfo}) {
        $logmsg .= sprintf "%s=%s;", uc $key, $key->{$tinfo};
    }

    _logit($logmsg);
    delete $self->{tests}{$tid} if (exists $ti->{endtest});
    return;
}


sub _process_misc_msg {
    my $ti = shift;
    my $tinfo = {};
    foreach my $key (keys %{$ti}) {
        $tinfo->{$key} = $ti->{$key};
    }
    my $logmsg =
          sprintf "ID=%s;MSGNO=%s;TESTNAME=%s;%s;",
                  $tinfo->{id},$tinfo->{msg},$tinfo->{testname},$tinfo->{type};
    delete $tinfo->{id};
    delete $tinfo->{msg};
    delete $tinfo->{testname};
    delete $tinfo->{type};
    foreach my $key (keys %{$tinfo}) {
        $logmsg .= sprintf "%s=%s;", uc $key, $key->{$tinfo};
    }
    _logit($logmsg);
    return;
}

sub _process_filer_msg {
    my $tinfo = shift;

}

sub _process_notify_msg {
    my $tinfo = shift;
}

sub _process_status_msg {
    my $ti = shift;
}

sub _logit {
    my $logmsg = shift;
    open LOGFILE, "+>$self->{logfile}"
        or return $self->{error} = "Unable to open $self->{logfile}";
    print LOGFILE $logmsg;
    close LOGFILE;
    return;
}

################################################################################
# Client side routines
################################################################################

sub connect_to_status_server {
    my $self   = shift;
    my %args = @_;

    my $host   = undef;
    if ((exists $args{client}) && $args{client} ne $self->{hostname}) {
         $host = $self->{hostname};
    } else {
         $host = "localhost";
    }
    my $port   = $PORT;
    my $handle = undef;
    print "DBGCEB:connect_to_status_server: hostname = $hostname\n";
    getc; #DBGCEB
    print "DBGCEB:connect_to_status_server: opening socket";

    $handle = IO::Socket::INET->new(
        Proto    => "tcp",
        PeerAddr => $host,
        PeerPort => $port
      )
      or die "can't connect to port $port on $host: $!";
#    print "DBGCEB:connect_to_status_server: handle = ",Dumper($handle);
    $handle->autoflush(1);    # so output gets there right away
    IO::Socket->output_record_separator("\n");

    if ( !defined $handle ) {
        $self->{error} = "Cannot connect to status server: $!";
    }
    $self->{server} = $handle;
 #   $Data::Dumper::Maxdepth = 2;
 #   print "DBGCEB:connect_to_status_server: self = ",Dumper($self);

 #   print $handle "Hello There!";
    return;
}

    sub _ift_send_endtest {
        my $self = shift;
        my $tid = shift;
        my $result = shift;
        my $text = shift;
        print "DBGCEB:_send_endtest: test=$tid result=$result text=$text\n" if $DBGCEB;
        $tests->{$tid}{type} = "RESULT";
        $tests->{$tid}{result} = $result;
        $tests->{$tid}{endtest} = "ENDTEST";
        $tests->{$tid}{endtime} = time();
        $tests->{$tid}{duration} = $tests->{$tid}{endtime} -
                                   $tests->{$tid}{starttime};
        $tests->{$tid}{text} = $text;
        print "DBGCEB:_send_endtest: tinfo = ",Dumper($tests->{$tid});
        _send_msg($tid);
        delete $tests->{$tid};
        return;
    }

    sub _ift_send_starttest {
        my $self = shift;
        my $args =
        my $tid = shift;
        my $test = shift;
        my $client = shift;
        my $mp = shift;
        my $text = shift;
        $tests->{$tid} = {};
        $tests->{$tid}{type} = "START";
        $tests->{$tid}{id} = $tid;
        $tests->{$tid}{client} = $client;
        $tests->{$tid}{mp} = $mp;
        $tests->{$tid}{text} = $text;
        $tests->{$tid}{testname} = $test;
        $tests->{$tid}{starttime} = time();
        print "DBGCEB:_send_starttest: tinfo = ",Dumper($tests->{$tid});
        _send_msg($tid);
        return $tid;
    }

    sub _ift_send_noend_msg {
        my $self = shift;
        my $tid = shift;
        my $type = shift;
        my $text = shift;
        my $result = shift;
        $tests->{$tid}{type} = $type;
        $tests->{$tid}{text} = $text;
        print "DBGCEB:_send_noend_msg: tinfo = ",Dumper($tests->{$tid});
        _send_msg($tid);
        return;
    }




sub get_status {
    my $self  = shift;
    my %args  = @_;
    my $filer = $args{filer};
    $filer = "all" if ( !$filer );
    my $cmd    = "get status $filer";
    my $handle = $self->{server};
    print $handle $cmd;
    my $line = undef;
    my $stat = undef;

    while ( $line = <$handle> ) {
        next if ( $line !~ /status/ );
        $stat = 1;
        if ( $filer eq "all" ) {
            $line =~ /~status\s(\w+)\s(\w+)\s(\w+)\s(\w+)$/;
            $self->{filers}{$1}{status} = $2;
            $self->{filers}{$3}{status} = $4;
        } else {
            $line =~ /status\s\w+\s(\w+)/;
            $self->{filers}{$filer}{status} = $1;
        }
    }
    $self->{error} = "could not retrieve status" if ( !$stat );
    return;
}

sub set_status {
    my $self   = shift;
    my %args   = @_;
    my $filer  = $args{filer};
    my $status = $args{status};
    my $handle = $self->{server};
    $filer = "all" if ( !$filer );
    my $cmd = "set status $filer $status";

    print $handle $cmd;
    my $line = undef;
    my $stat = undef;
    while ( $line = <$handle> ) {
        next if ( $line !~ /OK/ );
        $stat = 1;
    }
    if ( $filer eq "all" ) {
        foreach my $f ( keys %{ $self->{filers} } ) {
            $self->{filers}{$f}{status} = $status;
        }
    } else {
        $self->{filers}{$filer}{status} = $status;
    }
    $self->{error} = "could not retrieve status" if ( !$stat );
    return;
}

sub do_test_info {
    my $sock    = shift;
    my $cmd     = shift;
    my $cmdinfo = {};
    my $end     = 0;
    while ($cmd) {
        my ( $k, $v );
        for my $field ( split /;/, $cmd ) {
            if ( $field eq "END" ) {
                $end = 1;
                next;
            }
            my ( $k, $v ) = split /:/, $field;
            $cmdinfo->{$k} = $v;
        }
        last if ($end);
        $cmd = <$sock>;
    }

    if ( !exists $cmdinfo->{TEST} ) {
        print $sock "NAK";
        return;
    }
    my $test = $cmdinfo->{TEST};
    delete $cmdinfo->{TEST};
    foreach my $key ( keys %{$cmdinfo} ) {
        $testinfo->{$test}{$key} = $cmdinfo->{$key};
    }
    log_test_info($test);

}

sub log_test_info {
    my $test = shift;
}

sub start_monitor {
    my $self = shift;

    foreach my $key ( keys %{ $self->{filers} } ) {
        $self->{monitor}{$key}{pid} = fork() or die "Unable to start monitor";
        if ( !exists $self->{monitor}{$key}{pid} ) {
            $self->monitor_filer($key);
        }
    }
}

sub monitor_filer {
    my $filer = shift;
    return;
}

# ------------------------------------------------------------------------
1;

# ------------------------------------------------------------------------
