#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTFDOTests.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
## @summary
##     IFTFDOTests - perform  directory and file operations tests across
##                   multiple clients
##
## @description
##
## @usage
##     use IFT::IFTFDOTests;
##
## @author dl-johny@netapp.com (FSS WAFL Quality) bundy
## @status public
package IFTFDOTests;

$Data::Dumper::Indent = 1;

use Tharn;
use SPARSE::Util::Logger qw(logDbg logError logTrc logStep logWarn);
use IFT::NfsServer;

use IFT::IFTCmd;
our $SUSPEND = "suspend";
our $QUIT    = "quit";
our $PAUSE   = "pause ";
our $RESUME  = "resume";
our $MOUNTED = "mounted";
our $DELETED = "deleted";
our $UMOUNT  = "umount";
our $MOUNT   = "mount";
our $RMDIR   = "rmdir";
our $MOUNTS  = "mounts";
our $NOT_OK  = "NK";
our $OK      = "OK";
our $ERROR   = "error";
our $RESULTS = "results";
our $DISABLEMOUNT = "DISABLEMOUNT";
our $ENABLEMOUNT = "ENABLEMOUNT";
our $DELETEMOUNT = "DELETEMOUNT";
our $ADDMOUNT = "ADDMOUNT";
our $ACK = "ACK";
our $NCK = "NCK";
our $DBGCEB  = 1;
use Data::Dumper;

$DBGCEB  = 0; ###################################


sub new($$) {
    my $class         = shift;
    my $testinfo      = shift;
    my $test_manager  = $testinfo->{test_manager};
    my $testcfg       = $testinfo->{cfg};
    my $type          = $testcfg->{type};
    my $client        = $testcfg->{client};
    my $clients       = $testcfg->{clients};
    my $mp_type       = $testinfo->{mp_type};
    my $test          = $testinfo->{test};
    my $tid           = $testinfo->{tid};
    my $mounts        = $client->{mounts};
    my $self = {};

    foreach my $mp ( keys %{$mounts} ) {
        next if ( $mp_type && $mp_type ne $mounts->{$mp}{type} );
        $self->{mounts}{$mp} = $mounts->{$mp};
    }

    #    $self->{server} = NfsServer->new($client);
    #    $server->rpc_finish();
    $Data::Dumper::Maxdepth   = 2;
    $self->{test_manager}     = $test_manager;
    $self->{test}             = $test;
    $self->{tid}              = $tid;
    $self->{client}           = {};
    $self->{client}{fullname} = $client->{fullname};
    $self->{client}{hname}    = $client->{hname};
    $self->{client}{hosttype} = $client->{hosttype};
    $self->{filers}           = $client->{filers};
    $self->{quotas}           = $client->{quotas};
    $self->{mp_type}          = $mp_type;
    if ( $self->{quotas} ) {
        $self->{testname} = "quotas_nfs_directory_and_file_ops";
    } else {
        $self->{testname} = "nfs_directory_and_file_ops";
    }

    $self->{sserv}            = $client->{sserv};
    bless $self, $class;
    print "DBGCEB:IFTFDOTests:new client = ", Dumper($client) if ($DBGCEB);
    return $self;
}


sub start_tests {
    my $self   = shift;
logresult('INFO', "Enter IFTFDOTests::start_tests Line: ".__LINE__);
    $Data::Dumper::Maxdepth = 1;
logcomment("start_tests  Line: ".__LINE__);
logresult('INFO', "IFTFDOTests::start_tests Calling test_ops  Line: ".__LINE__);
    $self->test_ops();
logcomment("start_tests  Line: ".__LINE__);
logresult('INFO', "IFTFDOTests::start_tests Returned from test_ops");
logresult('INFO', "Exit IFTFDOTests::start_tests Line: ".__LINE__);
    return;
}


# Create the source file that will be repeatedly copied during
# reconstruction.
sub subtest_setup ( $ ) {
logcomment("Enter subtest_setup");
    my $self = shift;
    $self->{subcount} = 0 if ( !exists $self->{subcount} );
    $self->{subcount}++;
    my $server =
      NfsServer->new( $self->{client}, $self->{subcount}, $self->{sserv}  );

    print "\nDBGCEB:   IFTFDO::subtest_setup: starting nfs subtest $self->{subcount}\n";
    print "\nDBGCEB:   IFTFDO::subtest_setup: Calling setup_rpc_server\n";
    $server->setup_rpc_server();
    my $pid = $server->{pid};
logcomment("subtest_setup:  PID: $pid");

logcomment("IFTFDO::subtest_setup: Returned from setup_rpc_server");
logcomment("subtest_setup  Returning server:\n\n"._print_server($server));
    return $server;
}


#####################################################################
#
#  Subtests
#
####################################################################
sub test_ops {
    my ( $self, %opts ) = @_;
logcomment("Enter test_ops");
logcomment("test_ops  Line: ".__LINE__);
    my $ownname = $self->{testname};
    $self->{parent} = Subtest::ipc();
    print "\nDBGCEB:$ownname has started\n" if ($DBGCEB);
    my $old_Maxdepth = $Data::Dumper::Maxdepth;
    $Data::Dumper::Maxdepth   = 2;
logcomment("test_ops  Line: ".__LINE__);
    my $subtestid = 0;

    my @tests   = directory_and_file_ops_data( $self, %opts );
logcomment("test_ops  Line: ".__LINE__);

    my $server  = undef;
    my @servers = ();
    # setup mount points
    my $firstmp = _llmounts($self);
    my $mntcnt = (keys %{$self->{mounts}});
    # start rpc servers
    my $servno = ($mntcnt < 5) ? $mntcnt : 5;
logcomment("test_ops  Line: ".__LINE__);
    print "DBGCEB:IFTFDOTestOps:number of servers started $servno\n"
            if ($DBGCEB);
    for ( 1 .. $servno  ) {    #DBGCEB put this back
        $server = $self->subtest_setup();
        last if ( !$server );
        push @servers, $server;
    }
logcomment("test_ops  Line: ".__LINE__);
    die config_result("Unable to start servers") if ( @servers eq () );
logcomment("test_ops  Line: ".__LINE__);
logcomment("IFTFDOTests::test_ops: rpc servers are started");

logcomment("test_ops  Line: ".__LINE__);
    my $mp      = $firstmp;
    $Data::Dumper::Maxdepth = 2;

    while (1) {
$Data::Dumper::Maxdepth = 2;
logcomment("test_ops -- top of loop");
        $mp = $self->{mounts}{$mp}{nxt};
logcomment("test_ops  Line: ".__LINE__."    MP:\n\n".Dumper($mp));
        next if ($self->{mounts}{$mp}{status} eq $DISABLEMOUNT);
        $self->{params}{path} = $mp;
        logcomment("IFTFDOTestOps: MP: $mp");
        my $mpobj = $self->{mounts}{$mp};
logcomment("test_ops  Line: ".__LINE__."    MPOBJ:\n\n"._print_mpobj($mpobj));

        if (defined($mpobj->{test})) {
logcomment("test_ops  Test in progress for MP: $mp  .....");
            # A test for this mp is running, see if it's finished.
            ### $mpobj->{endtime} = time() + 600;
    logcomment("test_ops  Line: ".__LINE__."  Calling _check_result");
            my ( $result ) = _check_result( $self, $mpobj );
    logcomment("test_ops  Line: ".__LINE__."  Returned from _check_result: $result");
            if ( !$result ) {
logcomment(".... test_ops  Test NOT completed for MP: $mp  --  goto next mp ....");
                # Test for this mp is still runnning, go to next $mp
                my $test = $mpobj->{test}{name};
                my $start_time = $mpobj->{test}{start_time};
                my $elapsed = time() - $start_time;
logcomment("test_ops  Elapsed time for Test: $test on MP: $mp:  $elapsed");
                if ($elapsed > 600) {
                    logWarn("test_ops  Elapsed time for Test: $test on MP: $mp:  $elapsed  Exceeded limit (600 sec)");
                }
                next;
            }
            # Test completed for this mp. Mark this mp as ready for new test.
logcomment(".... test_ops  Test completed for MP: $mp");
            delete($mpobj->{test});
            $mpobj->{params}{pid} = undef;
            $self->{params}{test} =  undef;
            $mpobj->{params}{idx} = undef;
        } else {

logcomment("No test running for MP: $mp  --  start new one");
            # There is no test running for this mp, grab a server and start one
    logcomment("test_ops  Line: ".__LINE__);
            if ( ( !$mpobj->{server} ) || ( !exists($mpobj->{server}) ) ) {
    logcomment("test_ops  Line: ".__LINE__);
                $server = $servers[ $subtestid % scalar(@servers) ];
    logcomment("test_ops  Line: ".__LINE__."   SERVER:\n\n"._print_server($server));
    
                if (!$server->{SUBTEST}->is_alive()) {
    logcomment("test_ops  Line: ".__LINE__."  Calling subtest_setup");
                     $server = $self->subtest_setup();
    logcomment("test_ops  Line: ".__LINE__."  Returned from subtest_setup");
                }
    logcomment("test_ops  Line: ".__LINE__);
    
               logcomment("IFTFDOTestOps: yes we do have a server! ".
                     "starting tests on $mp");
                $mpobj->{server} = $server;
            } else {
    logcomment("test_ops  Line: ".__LINE__);
    
                # if there is a test running check if it's done
                $server = $mpobj->{server};
    logcomment("test_ops  Line: ".__LINE__."    MPOBJ:\n\n"._print_mpobj($mpobj));
            }

    logcomment("test_ops  Line: ".__LINE__);
            while (!$server->{SUBTEST}->is_alive()) {
    logcomment("test_ops  Line: ".__LINE__."   Server $server->{SUBTEST} not alive, calling subtest_setup");
                 $mpobj->{server} = $self->subtest_setup();
            }
    logcomment("test_ops  Line: ".__LINE__."   Server $server->{SUBTEST} is alive");
    
            # start a new test
            $mpobj->{test} = {};
            $mpobj->{test}{pid} = $server->{pid};
            $mpobj->{params}{pid} = $server->{pid};
    
            my $test;
            if ($mpobj->{populate_dirs_completed}) {
logcomment("test_ops  Line: ".__LINE__."   'populate_dirs_completed' for MP $mp");
                $test = $tests[ $subtestid % scalar(@tests) ];
            } else {
logcomment("test_ops  Line: ".__LINE__."   Starting 'my_populate_directories' for MP $mp");
                $test = "my_populate_directories";
                $mpobj->{populate_dirs_completed} = 1;
            }
logcomment("test_ops  Line: ".__LINE__);
            $mpobj->{test}{name} = $test;
            $self->{params}{test} = $test;
            $mpobj->{params} = $self->{params};
            if ($self->{quotas}) {
    logcomment("test_ops  Line: ".__LINE__."   'quotas' is set");
                $mpobj->{params}{uid} = $mpobj->{uid};
                $mpobj->{params}{gid} = $mpobj->{gid};
            }
            $mpobj->{test}{idx} = $subtestid;
            $mpobj->{params}{idx} = $subtestid;
            my $tid = "$mpobj->{server}{tid}:$subtestid";
    logcomment("test_ops  Line: ".__LINE__."   New test for MP: $mp  IDX: $subtestid  TID: $tid  Test: $test");
            $mpobj->{test}{tid} = $tid;
            $mpobj->{params}{tid} = $tid;
            $mpobj->{test}{start_time} = time();
    logcomment("test_ops  Line: ".__LINE__."    MPOBJ:\n\n"._print_mpobj($mpobj));
    logcomment("test_ops  Line: ".__LINE__."    Call _call_subtest\n\nMPOBJ_PARAMS:\n\n".Dumper($mpobj->{params}));
            # Run the test
            $mpobj->{server}->_call_subtest( $test,$mpobj->{params} );
logcomment("test_ops  Line: ".__LINE__."    Returned from _call_subtest");
            $subtestid++;
        }

    }
    $Data::Dumper::Maxdepth = my $old_Maxdepth;
}


sub directory_and_file_ops_data {
    my $self = shift;
    my %opts = @_;
    $self->{params}{width}      = $opts{dir_width}  ? $opts{dir_width}  : 5;
    $self->{params}{depth}      = $opts{dir_depth}  ? $opts{dir_depth}  : 3;
    $self->{params}{dircount}   = $opts{dir_count}  ? $opts{dir_count}  : 2;
    $self->{params}{rdcount}    = $opts{rdcount}    ? $opts{rdcount}    : 50;
    $self->{params}{iterations} = $opts{iterations} ? $opts{iterations} : 5;
    $self->{params}{filecount}  = $opts{file_count} ? $opts{file_count} : 2;
    $self->{params}{client}      = $self->{client}{hname};
    $self->{params}{fsize}       = $opts{file_size} ? $opts{file_size} : 1024;
    $self->{params}{file_prefix} =
      $opts{file_prefix} ? $opts{file_prefix} : "tfile";
    $self->{params}{dir_prefix} =
      $opts{dir_prefix} ? $opts{dir_prefix} : "testdir";
    $self->{params}{tree}   = 1;
    $self->{params}{quotas} = $self->{quotas};

    my @tests = ();
    if ( $self->{quotas} ) {
logcomment("XXXXXXXX: quotas ops ");
        @tests = ( "my_write_files", "my_delete_files" );
    } else {
        @tests = (
###            "my_delete_directory_tree", "my_create_directory_tree",
            "my_write_files",           "my_readdir",
            "my_copydir",               "my_truncate_files",
            "my_delete_files",
            "my_file_attributes",
        );
    }

# @tests = qw(my_check_path);  ##################################
# @tests = qw(my_echo_params);  ##################################
# @tests = qw(my_echo_error);  ##################################

    return @tests;
}


sub _llmounts {
    my $self = shift;

    # make mountpoint into a linked list
    my @mps   = ( keys %{ $self->{mounts} } );
    print "DBGCEB:IFTFDOTests:mountpoints for $self->{hname}\n";
    foreach $mp (@mps) {
       print "   $mp\n";
    }
    my $lstmp = $#mps;
    my $idx   = 0;
    while ( $idx < $lstmp ) {
#    while ( $idx < 4 ) {
        if ( $idx == 0 ) {
            $self->{mounts}{ $mps[$idx] }{prv} = $mps[$lstmp];
        } else {
            $self->{mounts}{ $mps[$idx] }{prv} = $mps[ $idx - 1 ];
        }
        $self->{mounts}{ $mps[$idx] }{nxt} = $mps[ $idx + 1 ];
        $self->{mounts}{ $mps[$idx] }{mp}  = $mps[$idx];
        $idx++;
    }
    $self->{mounts}{ $mps[$idx] }{nxt} = $mps[0];
    $self->{mounts}{ $mps[$idx] }{prv} = $mps[ $idx - 1 ];
    $self->{mounts}{ $mps[$idx] }{mp} = $mps[$idx];
    if ($DBGCEB) {
        print "DBGCEB:IFTFDOTests:_llmounts:self mounts = ",
               Dumper( $self->{mounts} )
               if ($DBGCEB);
        print "DBGCEB:IFTFDOTests:_llmounts:that was self mounts\n"
               if ($DBGCEB);

        #    getc;
    }
    my $firstmp = $mps[0];
    return $firstmp;
}


sub _check_result {
    my $self   = shift;
    my $mpobj  = shift;
logcomment("Enter _check_result  Test:$server->{SUBTEST}{runid}:$mpobj->{test}{name}");
    my @results   = ();
    my @logmsgs   = ();
    my $status = undef;
    my $server = $mpobj->{server};
    my $parent = $self->{parent};

logcomment("_check_result  Line: ".__LINE__."  Calling check_client");
    my @results = $server->_check_client();
logcomment("_check_result  Line: ".__LINE__."  Returned from check_client");
    my $nresults = scalar(@results);
logcomment("_check_result  Line: ".__LINE__."   Results[$nresults]:\n\n".Dumper(@results));
    return if ( !@results );
    my $logmsg = undef;
logcomment("_check_result  Line: ".__LINE__);
    my $cnt = 0;
    foreach my $tinfo (@results) {
logcomment("_check_result  Line: ".__LINE__."\ntinfo[$cnt]: \n\n".Dumper($tinfo));
        $cnt++;
logcomment("_check_result  Line: ".__LINE__);
        my $msgheader =
           "ID=$tinfo->{id} TESTNAME=$tinfo->{testname} TYPE= $tinfo->{type} MSG=$tinfo->{msg} ";
        if ( $tinfo->{type} eq "START" ) {
logcomment("_check_result  Line: ".__LINE__);
          $logmsg = sprintf "%s:START CLIENT=%s STARTTIME=%s MOUNTPOINT=%s %s",
              $msgheader, $tinfo->{client}, $tinfo->{starttime}, $tinfo->{mp},
              $tinfo->{text};
        } elsif ( $tinfo->{endtest} eq "ENDTEST" ) {
            $status = $tinfo->{endtest} || "END";
logcomment("_check_result  Line: ".__LINE__);
logcomment("GOT ENDTEST, QUOTAS: $self->{quotas}");
            my $msg = $self->_process_quota_limits($mpobj, $tinfo) if ($self->{quotas});
logcomment("_check_result  Line: ".__LINE__);
            $logmsg = sprintf "%s:ENDTEST RESULT=%s ENDTIME=%s DURATION=%s %s",
              $msgheader, $msg->{result}, $msg->{endtime}, $msg->{duration},
              $msg->{text};
logcomment("_check_result  Line: ".__LINE__);
        } else {
logcomment("_check_result  Line: ".__LINE__);
              $logmsg = sprintf "%s:%s:%s", $msgheader, $msg->{type}, $msg->{text};
logcomment("check result got other message:  $logmsg");
        }
logcomment("_check_result  Line: ".__LINE__);

        if ($logmsg) {
logcomment("_check_result  Line: ".__LINE__);
           $self->_logcomment($logmsg);
           push(@logmsgs, $logmsg);
        }
logcomment("_check_result  Line: ".__LINE__);
    }
logcomment("_check_result  Line: ".__LINE__);
    $server->{results} = ();

logcomment("_check_result  Line: ".__LINE__);
logcomment("Exit  _check_result:  $server->{SUBTEST}{runid}:$mpobj->{test}{name}\n\nresults:\n\n".Dumper(\@results)."\n\nlogmsgs:\n\n".Dumper(\@logmsgs)."\n\nReturning: Result: $status");
logcomment("_check_result  Line: ".__LINE__);
    return $status;
}


sub _process_quota_limits {
    my $self = shift;
    my $mp  = shift;  #self->{mounts}{$mp}
    my $tinfo   = shift; #msg

    my $qhash = Client::quota_info_for_mp($mp);
    if ((!exists $tinfo->{write_size}) && (!exists $tinfo->{file_count})) {
         # we errored out, we may have or have not created files and written
         # to files, but it's too complicated to figure out what was written
         # successfully and what wasn't
         #get quota report and update $mpobj with current info;
         #
         if ($tinfo->{text} =~ /exceed/i) {
logcomment("MMMMMMMMMMMMMMMMM: fail but message is excceed");
            if ($qhash->{curr_files} >= $qhash->{max_files} ||
                $qhash->{curr_size} >= $qhash->{max_size}) {
                $tinfo->{result} = "PASS";
logcomment("MMMMMMMMMMMMMMMMM: fail but message is excceed, quota limit");
            } 
         }
         $mp->{curr_size} = $qhash->{curr_size};
         $mp->{max_size} = $qhash->{max_size};
         $mp->{curr_files} = $qhash->{curr_files};
         $mp->{max_files} = $qhash->{max_files};

         return;
    }
    my $write_size  = $tinfo->{write_size};
    my $file_count = $tinfo->{file_count};
    my $result = $tinfo->{result};
    if ($write_size) {
logcomment("MMMMMMMMMMMMMMMMM: write size : $write_size");
      my $new_size = $write_size+$mp->{curr_size};
      if ($result eq "PASS") {
        if ($qhash->{curr_size} > $qhash->{max_size}) {
logcomment("MMMMMMMMMMMMMMMMM: passed but curr>max: $qhash->{curr_size},  $qhash->{max_size}");
             $tinfo->{result} = "FAIL";
             $tinfo->{text} = "quota(size) exceed at $mp->{filer_info} ".
                 "$mp->{mount_dir}, max_size: $qhash->{max_size}, but ".
                 "current size : $qhash->{curr_size}"; 
        }
        elsif ($new_size > $qhash->{max_size}) {
logcomment("MMMMMMMMMMMMMMMMM: passed but new>max: $new_size,  $qhash->{max_size}");
             $tinfo->{result} = "FAIL";
             $tinfo->{text} = "writed size exceed quota size limit at ".
                "$mp->{filer_info}, $mp->{mount_dir}, max_size=".
                "$qhash->{max_size}, previous size=$mp->{curr_size},".
                "writed size=$write_size"; 
        }
        elsif (
         (($qhash->{curr_size} - $new_size)/$qhash->{curr_size}) > 0.2) {
logcomment("MMMMMMMMMMMMMMMMM: passed but new/curr: $new_size,  $qhash->{curr_size}");
             $tinfo->{result} = "FAIL";
             $tinfo->{text} = "writed size and the real size has more ".
                "than 20% difference: writed size=$new_size,".
                " current size: $qhash->{curr_size}";             
        }; 
      }
      else # $result eq "FAIL"
      {
logcomment("MMMMMMMMMMMMMMMMM: message return failed ");
        if ($new_size >= $qhash->{max_size} ) {
logcomment("MMMMMMMMMMMMMMMMM: message return failed, new>max:$new_size, $qhash->{max_size} ");
            $tinfo->{result} = "PASS";
        }
        elsif ($qhash->{curr_size} >= $qhash->{max_size}) {
logcomment("MMMMMMMMMMMMMMMMM: message return failed, curr>max:$qhash->{curr_size}, $qhash->{max_size} ");
            $tinfo->{result} = "PASS";
        }  
      }
    } 

    if ($file_count) {
logcomment("CCCCCCCCCCCCC: got file_count $file_count ");
      my $new_count = $file_count+$mp->{curr_files};
      if ($result eq "PASS") {
        if ($qhash->{curr_files} > $qhash->{max_files}) {
logcomment("CCCCCCCCCCCCC: passed but cur>max: $qhash->{curr_files} > $qhash->{max_files} ");
             $tinfo->{result} = "FAIL";
             $tinfo->{text} = "quota(files) exceed at $mp->{filer_info} ".
                 "$mp->{mount_dir}, max_files: $qhash->{max_files}, but ".
                 "current files count : $qhash->{curr_files}"; 
        }
        elsif ($new_count > $qhash->{max_files}) {
logcomment("CCCCCCCCCCCCC: passed but new>max: $new_count > $qhash->{max_files} ");
             $tinfo->{result} = "FAIL";
             $tinfo->{text} = "writed file count exceed quota limit at ".
                "$mp->{filer_info}, $mp->{mount_dir}, max_files=".
                "$qhash->{max_files}, previous files=$mp->{curr_files},".
                "writed file count=$file_count"; 
        }
        elsif ($new_count != $qhash->{curr_files}) {
logcomment("CCCCCCCCCCCCC: passed but new!= curr: $new_count != $qhash->{curr_files} ");
             $tinfo->{result} = "FAIL";
             $tinfo->{text} = "writed file count is not the same as the ".
                "real file count: writed count=$new_count,".
                " current file count: $qhash->{curr_files}";             
        } 
     }
     else {  # fail
logcomment("CCCCCCCCCCCCC: message says failed");
        if ($new_count >= $qhash->{max_files} ) {
logcomment("CCCCCCCCCCCCC: message says failed, new>max: $new_count >= $qhash->{max_files}");
            $tinfo->{result} = "PASS";
        }
        elsif ($qhash->{curr_size} >= $qhash->{max_size}) {
logcomment("CCCCCCCCCCCCC: message says failed, cur>max: $qhash->{curr_size}>= $qhash->{max_files}");
            $tinfo->{result} = "PASS";
        }
     }
   }

   $mp->{curr_size} = $qhash->{curr_size};
   $mp->{max_size} = $qhash->{max_size};
   $mp->{curr_files} = $qhash->{curr_files};
   $mp->{max_files} = $qhash->{max_files};

   return;
}


sub _getack {
    my $sock    = shift;
    my $msg     = shift;
    my $retries = 3;
    my $result  = 0;
    for ( 1 .. $retries ) {
        sleep 2;
        next if ( !$sock->can_read() );
        my $message = $sock->read_message();
        if ( $message !~ /$msg OK/ ) {
            print "DBGCEB:IFTFDOTests:_getack: message from parent (not ack) ".
                   "$message\n"
                   if ($DBGCEB);
            $self->{lastmsgfromparent} = $message;
            last;
        }
        return 1;
    }
    logWarn(
        "IFTFDOTests:IFTFDOTests:_getack: sending $msg or object: no ack from "
        . "parent" )
        if ( !$result );

    return $result;
}

sub _logcomment {
    my $self = shift;
    my $msg  = shift;
    print "DBGCEB:IFTFDOTests:_logcomment $self->{client}{hname}\n"
           if ($DBGCEB);
    logcomment($msg);
    my $logdir = "/u/bundy/logs/$self->{client}{hname}/";
    mkdir $logdir if ( !-e $logdir );

    #    if ($!) {
    #        logcomment("cannot create dir $logdir");
    #        return;
    #    }
    my $logfile = undef;
    if ( !$self->{logfile} ) {
        $logfile = $logdir . $self->{client}{hname} . time();
        $self->{logfile} = $logfile;
    } else {
        $logfile = $self->{logfile};
    }
    open LOGFILE, ">>$logfile";
    if ( !LOGFILE ) {
        logcomment("IFTFDOTests:_logcomment:cannot open $logfile");
        return;
    }
    print LOGFILE "$msg\n";
    close LOGFILE;
    return;
}


sub _print_mpobj {
	my ($mpobj) = @_;
	my $mpcopy = { %$mpobj };
	$mpcopy->{filer_info} = $mpcopy->{filer_info}{hostname};
	$mpcopy->{filer} = $mpcopy->{filer_info};
	$mpcopy->{client_info} = $mpcopy->{client_info}{hname};
	delete($mpcopy->{container});
	if ($mpcopy->{server}) {
		$mpcopy->{server} =  _print_server($mpcopy->{server});
		### delete($mpcopy->{server});
	}
logcomment("_print_mpobj:   KEYS:\n\n".Dumper(keys(%$mpcopy)));
delete($mpcopy->{"wqa-f56"}); ##################
# while (my ($k,$v) = each(%$mpcopy)) { logcomment("MPOBJ{\"$k\"}:\n\n".Dumper($v)); }
	return ( Dumper($mpcopy) );
}


sub _print_server {
	my ($srv) = @_;
	my $srvcopy = { %$srv };
	if ($srvcopy->{SUBTEST}) {
		$srvcopy->{SUBTEST} = _print_subtest($srvcopy->{SUBTEST});
	}
$srvcopy->{KEYS} = [ keys(%$srvcopy) ];
# while ( my ($k,$v) = each(%$srvcopy)) {  logcomment("server->{ \"$k\" }\n\n".Dumper($v)); }
	return ( Dumper($srvcopy) );
}

sub _print_subtest {
	my ($sub) = @_;
	my $subcopy = { %$sub };
	my $subsubcopy = { %{$subcopy->{SUBTEST}} };
	$subcopy->{SUBTEST} = $subsubcopy;
	delete($subcopy->{params});
	my $child = $subcopy->{child};
	my $childcopy = { %$child };
	$subcopy->{child} = $childcopy;
	delete($childcopy->{on_termination});
	delete($childcopy->{child_spec});
	delete($childcopy->{client});
	delete($childcopy->{iffatal});
	delete($childcopy->{ifscript});
	delete($subcopy->{iffatal});
	delete($subcopy->{ifscript});
	delete($subcopy->{iftimeout});
# while ( my ($k,$v) = each(%$subcopy)) {  next if ($k eq "child"); logcomment("subtest->{ \"$k\" }\n\n".Dumper($v)); }
# while ( my ($k,$v) = each(%{$subcopy->{child}})) {  logcomment("subtest->{ child }{ \"$k\" }\n\n".Dumper($v)); }
my $pid = $sub->{pid};
# logcomment("PRINT_SUBTEST  PID: $pid");
# logcomment("PRINT_SUBTEST\n\n".Dumper($subcopy));
	return ( Dumper($subcopy) );
}


1;
