#!/usr/local/bin/perl -d
# -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTSIOTests.pm#1 $

package IFTSIOTests;
use strict;
use File::Basename;
use Tharn;
use Data::Dumper;
use SPARSE::Util::Logger qw(logError logTrc logStep logWarn);
use FS::Util::Utilities qw(util_mkfile util_filesize util_stat delfile util_chmod);
use Utils::Units qw(convert_to_bytes);
use SayWrap qw(sayEOF);


my $OK         = "OK";
my $LOGGING    = 0;
my $IPC = 0;
my $AGGR  = "aggr";
my $FLEX  = "flex";
my $TRAD  = "trad";
my $LUNS  = "LUNs";
my $QTREES = "qtrees";
my $DBGCEB = 0;

# for development only
use constant RSH_TIMEOUT => 60;


sub new($$) {
    my $class         = shift;
    my $testinfo      = shift;
    my $testname      = "SIO operations";
    my $test_manager  = $testinfo->{test_manager};
    my $test          = $testinfo->{test};
    my $tid           = $testinfo->{tid};
    my $endtime       = $testinfo->{endtime};
    my $testcfg       = $testinfo->{cfg};
    my $duration      = $testcfg->{duration};
    my $filesize      = convert_to_bytes($testcfg->{filesize});

    my $self = {};

    my $mp_type = "vol";

    my $vol_list = [];
    my $mounts = $testcfg->{mounts};
    my $own_mounts = {};
    foreach my $mp ( keys(%$mounts) ) {
        next if ( $mp_type && $mp_type ne $mounts->{$mp}{mp_type} );
        $own_mounts->{$mp} = $mounts->{$mp};
        push(@$vol_list, $mp);
    }
    $self->{mounts} = $own_mounts;
    $self->{vol_list} = $vol_list;

    $Data::Dumper::Maxdepth   = 2;
    $self->{test_manager}     = $test_manager;
    $self->{test}             = $test;
    $self->{tid}              = $tid;
    $self->{type}             = $tid;
    $self->{duration}         = $duration;
    $self->{endtime}          = $endtime;
    $self->{mp_type}          = $mp_type;
    $self->{type}             = $testcfg->{type};
    $self->{client}           = $testcfg->{client};
    $self->{clname}           = $self->{client}{hname};
    $self->{clients}          = $testcfg->{clients};
    $self->{numthread}        = $testcfg->{numthread};
    $self->{blksize}          = $testcfg->{blksize};
    $self->{max_open}         = $testcfg->{max_open};
    $self->{sio_options}      = $testcfg->{sio_options};
    $self->{filesize}         = $filesize;
    $self->{filename}         = $testcfg->{filename};
    $self->{read_pct}         = $testcfg->{read_pct};
    $self->{rand_pct}         = $testcfg->{rand_pct};
    $self->{reboot_clients}   = $testcfg->{reboot_clients};

    bless $self, $class;
    print "DBGCEB:IFTSIOTests:new client = ", Dumper($self->{client}) if ($DBGCEB);

#    $self->_make_sio_files();
     $self->kill_existing_sio_procs();
sleep(60);

    return $self;
}


sub start_tests {
    my $self   = shift;
logresult('INFO', "Enter IFTSIOTests::start_tests Line: ".__LINE__);
    my $ownname = $self->{testname};
    $Data::Dumper::Maxdepth = 1;
    my $this_host = system("hostname");
    chomp($this_host);
    $this_host =~ s/\..*//;
    print "DBGCEB:IFTAVOTests:start_tests\n" if ($DBGCEB);
    $self->{parent} = Subtest::ipc();
    print "\nDBGCEB:$ownname has started\n" if ($DBGCEB);

    $self->{subt} =
      subtest( \&_sio_test_main,
               '-bg',
               '-ipc',
               '-runid',
               $self->{tid},
###               "sio_ops",
               "--",
               $self );
    $self->{pid} = $self->{subt}->pid();
    $self->{host} = $this_host;
    sleep(4);
logresult('INFO', "Exit IFTSIOTests::start_tests  Line: ".__LINE__);
    return;
}


sub _sio_test_main {
    my $self = shift;
    $self->_make_sio_files();
    return if ($self->{error});
    $self->_start_sio_ops();
    $self->wait_for_children();
}


sub _make_sio_files {
    my $self      = shift;
    my $client    = shift;
    my $filename  = $self->{filename};
    my $client    = $self->{client};
    my $clname    = $self->{clname};
    my $endtime   = $self->{endtime};
    my $duration  = $self->{duration};
    my $max_open  = $self->{max_open};
    my $own_mounts = $self->{mounts};
    my $vol_count = 0;
    my $cc        = $client->{cc};
    my $fname     = $filename."_".$clname;
    my $filesize  = $self->{filesize};
    my $size      = $self->{filesize};
    my $timeout   = 60 + $size/10000;
    my $sio_file_paths = {};
logresult('INFO', "Enter IFTSIOTests::_make_sio_files  Line: ".__LINE__."");

    $self->{subtest_count} = 0;
    ### for my $mp (@$vol_list) {
    while ( my ($mp, $mount) = each(%$own_mounts)) {
        if ($endtime && time() > $endtime) {
             $self->{error} = "setup used longer time than duration ($duration). Give a larger duration time";
             return;
        }
        if ($vol_count > $max_open) {
            logWarn("SIO test max_open ($max_open) limit reached");
            last;
        }
        print "DBGCEB:IFTsio:sio_setup:\n$mp\n$mp\n";
        my $fpath = "$mp/$fname";
        my @fstat = util_stat($fpath, conn=>$cc);
	if (@fstat) {
	    util_chmod(0666, $fpath, conn=>$cc);
	    delfile($fpath, conn=>$cc);
	}
        logcomment("Creating SIO file $fpath");
        my $result = util_mkfile($fpath, conn=>$cc, size=>$size,
                                   timeout=>$timeout);

        if ($result) {
            ### push(@$sio_file_paths, $fpath);
	    my $sio_fpath_info =
			{
				fpath => $fpath,
				mountpoint => $mp,
				fname => $fname,
				filer => $mount->{filer},
				client => $mount->{client},
				vol => $mount->{vol},
			};
            $sio_file_paths->{$fpath} = $sio_fpath_info;
        } else {
            logWarn("Unable to create $fpath on $clname: " .
                    "removing $mp from vol list\n$@");
            next;
        }
logcomment("make_sio_files:  File Stat(4) after 'mkfile': \"$fpath\":\n\n".Dumper(\@fstat));
        $result = util_chmod(0666, $fpath, conn=>$cc);
logcomment("make_sio_files:  File Stat(5) after 'mkfile': \"$fpath\":\n\n".Dumper(\@fstat));
        $vol_count++;
    }
    $self->{sio_file_paths} = $sio_file_paths;
logresult('INFO', "Exit IFTSIOTests::_make_sio_files  Line: ".__LINE__."   SIO_FILE_PATHS:\n\n".Dumper($self->{sio_file_paths}));
}


sub _start_sio_ops {
    my $self      = shift;
    my $filename  = $self->{filename};
    my $client    = $self->{client};
    my $clname    = $self->{clname};
    my $endtime   = $self->{endtime};
    my $duration  = $self->{duration};
    my $filesize  = $self->{filesize};
    my $size      = $self->{filesize};
    my $sio_options = $self->{sio_options};
    my $numthread = $self->{numthread};
    my $sio_file_paths = $self->{sio_file_paths};
    my $sio_options = $self->{sio_options};
    $self->{subtests} = {};
    my $subtests = $self->{subtests};
    my $subtest_count = 1;

    if ($endtime && time() > $endtime) {
        $self->{error} = "setup used longer time than duration ($duration). Give a larger duration time";
        return;
    }
    my %fpath_hash = ();
    # Subtest commands (via rsh) can fail if the command is too long.
    # Split up @sio_file_paths into groups by hostname. Each
    # 'group' will have its own subtest.

    while (my ($fp, $sio_fpath_info) = each(%$sio_file_paths)) {
    ### foreach my $fp (@$sio_file_paths) {
        my ($fpath, $fdir);
        if ($fp =~ m%^(/t/[^/]+)/(.*)%) {
            $fdir = $1;
            $fpath = $2;
        } else {
            $fdir = "";
            $fpath = $fp;
        }
	if (!exists($fpath_hash{$fdir})) {
		$fpath_hash{$fdir} = [];
	}
        push(@{$fpath_hash{$fdir}}, $fpath);
    }
    ### while ( my($fdir, $fplist) = each(%fpath_hash) ) {
    while ( my($fdir, $fplist) = each(%fpath_hash) ) {
logresult('INFO', "IFTSIOTests::_start_sio_ops  Line: ".__LINE__."    CLNAME: \"$clname\"   FDIR: \"$fdir\"  FPLIST:\n\n".Dumper($fplist));
sleep(30);

        my $cmd = "";
        my $runid = "sio_subtest_on_$clname" . "_" . $subtest_count;
        $cmd .= "cd $fdir; "
            if ($fdir);

        $cmd .= "/usr/local/test/bin/sio ".
                    "$self->{read_pct} ".
                    "$self->{rand_pct} ".
                    "$self->{blksize} ".
                    "0 ".
                    "$size ".
                    "$duration ".
                    "$self->{numthread} ";

        foreach my $fpath (@$fplist) {
            $cmd .= "$fpath ";
        }

        $cmd .= " $sio_options ";

        my $command = "(ulimit -n 2000; $cmd; echo Exiting subtest on client $client->{hname})";
#        my $command = "/bin/sh -c \"($cmd)\"";
        logresult('INFO', "sio_test: starting sio on $clname\n cmd:\n\n$command");

         my $subt = subtest(  \&launch_subtest_as_root,
                              -bg,
###                              -client, $client->{fullname},
#                              -ipc,
                              -runid, $runid,
			      "--",
                              $client, $command);

         logcomment("launched a subtest on $client->{fullname}");
	 my $subt_info = {
         			runid	=> $subt->runid(),
         			cmd	=> $cmd,
         			host	=> $clname,
         			pid	=> $subt->pid(),
         			index	=> $subtest_count,
         			subt	=> $subt,
         			subtest_count => $subtest_count,
			};
	 $subtests->{$runid} = $subt_info;
	 $subtest_count++;
         sleep(2);
    }
    return;
}


########################################################################


sub launch_subtest_as_root {
        my $client = shift;
        my $cmmd = shift;
        my $su_cmmd = "/usr/local/bin/sudo su root -c \"$cmmd\"";
        my $subtest_out = sayEOF($client->{cc}, $su_cmmd, timeout=>-1);
        logresult('WARN', "subtest command exited with message: $subtest_out");
}


sub kill_existing_sio_procs {
	my $self = shift;
	my $client = $self->{client};
        my $cl_name = $client->{hname};
        logcomment("Killing sio processes on client $cl_name");
        my $output = eval { sayEOF($client->{cc}, "/bin/ps -ef") };
        return (0)
            if ($@);

        my @lines = split(/\n/, $output);
        my @procs = ();
        foreach (@lines) {
                next unless (/\S+/);
		$_ =~ s/^\s*//;
		chomp($_);
                my (undef, $pid, $ppid, undef, undef, undef, undef, $cmd) = split(/\s+/, $_);
		my ($cmd_path, $args) = split(/\s+/, $cmd, 2);
		my $cmd_name = basename($cmd_path);
		next unless ($cmd_name eq "sio");
logcomment("kill_existing_sio_procs  Line: ".__LINE__."   PID: $pid   PPID: $ppid   CMD_NAME: $cmd_name   CMD_PATH: $cmd_path  CMD: \"$cmd\"\n\nPS: $_");
                eval { sayEOF($client->{cc}, "/usr/local/bin/sudo kill -9 $pid", timeout=>300); };
                return (0) if ($@);
        }
        return (1);
}


sub reboot_client {
        my $client = shift;
        my $cl_name = $client->{hname};
###     my $output = say $f->{cc}, "reboot -t 0", -blind);
        logcomment("rebooting client $cl_name");
        system("/usr/local/bin/sudo /usr/local/bin/power_cycle $cl_name reboot")
            or die("power_cycle failed for $cl_name");
        my $max_wait = 600;
        my $start_time = time();
        my $up = 0;
        logcomment("waiting for $cl_name to come back up");
        while ( (time() - $start_time) <= $max_wait && !$up ) {
                sleep(30);
                my $output = `( ping -c 1 wqc8 > /dev/null ) ; echo $?`;
                $up = 1
                    if ($output =~ /0/);
        }
        if ($up) {
                logcomment("$cl_name is back up");
                return (1);
        } else {
#               die("Client $cl_name did not come back up after reboot");
                return (0);
        }
}


sub wait_for_children {
    logTrc("Enter IFTSIOTests::wait_for_children");
    my $self = shift;
    my $endtime = $self->{endtime};
    my $subtests = $self->{subtests};
    my $live_tests = 1;

    while ( time() < $endtime ) {
        sleep 30;
         $live_tests = 0; 
         foreach my $subtinfo (values(%$subtests)) {
             my $subt = $subtinfo->{subt};
             my $runid = $subtinfo->{runid};
             my $cmd = $subtinfo->{cmd};
             next if ($subtinfo->{exited});
             $live_tests = 1;
#             logTrc("IFTSIOTests::wait_for_children  Checking subtest $runid");
             my $error = $self->_check_subtest($subt, $subtinfo);
             my $is_alive;
             if (!$subt) {
                 $is_alive = 0;
             } else {
                 $is_alive = eval { $subt->isAlive() };
                 if ($@) {
                      logWarn("Error returned from isAlive for subtest $runid: $@");
                 } elsif (!$is_alive) {
                     logWarn("IFTSIOTests::wait_for_children: ".
                            "Subtest $runid has apparently exited -- isAlive returned 0");
                    $subtinfo->{exited} = 1;
                 }
             }
         }
    }
    logWarn("IFTSIOTests::wait_for_children: All children have exited");
    logTrc("IFTSIOTests::wait_for_children: Stopping child tests");
    $self->stop_children();
    return;
}


sub stop_children {
    my $self = shift;
    logcomment("Stopping all subtests");
    my $subtests = $self->{subtests};
    foreach my $subtinfo (values(%$subtests)) {
       my $subt = $subtinfo->{subt};
       $subt->kill();
    }
}

sub _check_subtest {
    my $self = shift;
    my $subtest = shift;
    my $subtinfo = shift;
    my $runid = $subtinfo->{runid};
    return unless ($subtest);
    my @results = $subtest->results();
    return unless (@results);
    my $worst = Rslt::worst(@results);
    return unless ($worst);
    return if ($subtinfo->{results}{$worst});
    $subtinfo->{results}{$worst} = 1;
    my $runid = $worst->runid();
    my @messages = $worst->messages();
    my $msgs =  join("\n", @messages);
    $subtinfo->{last_message} = $msgs;
    my $sev = $worst->severity();
    my $rtype = $worst->type();
    if ($sev >= 4) {
         my $error_message = "Subtest Error in $runid \n" . join("\n", @messages);
         $subtinfo->{error} = $error_message;
         logWarn("Error from subtest $runid:\n" . $msgs);
         return ( $error_message );
    } elsif ($sev == 3) {
         logWarn("Warning from subtest $runid:\n" . $msgs);
    } else {
         logcomment("Message from subtest $runid:\n" . $msgs);
    }
    return;
}


sub logmessage {
    # TBD: Add logging to 'logfile'
    my $msg = shift;
    logcomment($msg);
}




##############################################################################



1;
