package IFTSMOTests;
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTSMOTests.pm#1 $

use strict;
use Tharn;
use IFT::IFTSetup;
use IFT::Client;
use IFT::IFTComm;
use Data::Dumper;
use SPARSE::Util::Logger qw(logError logTrc logStep logWarn);
use SnapMirror::Helper qw(sm_init sm_cleanup);
use QtreeCmds qw(qtreeCreate qtreeList);
use filer qw(filer_file_make);
use Utils::Units qw(UNIT_MB);
use SnapMirror
  qw(snapmirror_status snapmirror_quiesce snapmirror_break
     snapmirror_update snapmirror_resync);
use FS::Util::Utilities qw(util_mkfile);
use Subtest;
use Tharn;
use Aggregate qw(aggr_create aggr_purge aggr_exists aggr_status aggr_disk_free);
use Volume qw(vol_create_flex vol_disk_free);
use Aggregate qw(:df);
use Utils::Units qw(:constants);
use mount qw(mount umount);
use ExportFS qw(export_path unexport_path list_exports);

our $AGGR       = "aggr";
our $FLEX       = "flex";
our $QTREES     = "qtrees";
our $SRC        = "SRC";
our $DST        = "DST";
our $AGGR_SPACE = "1000M";
our $QSM        = "QSM";
our $RQSM       = "RQSM";
our $VSM        = "VSM";
our $RVSM       = "RVSM";
our $SLEEP_TIME = 120;
our $DBGCEB     = 1;
our $too_many_updates = "too many active transfers";

#######################################################################
#
#  Test Setup Routines
#
#######################################################################

sub new {
    my $class = shift;
    my $tinfo = shift;
    my $tid   = shift;
    print "DBGCEB:IFTSMOTests:_new\n" if ($DBGCEB);

    my $self = {};
    bless $self, $class;
    $self->{tid} = $tid;
    $self->_initialize_sm_config($tinfo);
    return if ( $self->{error} );
    $self->_setup_filers($tinfo);
    return if ( $self->{error} );
    $self->_setup_sm();
    return $self;
}

sub _initialize_sm_config {
    my $self  = shift;
    print "DBGCEB:IFTSMOTests:_initialize_sm_config\n" if ($DBGCEB);

    my $tinfo = shift;
    $self->{ownname}       = "SnapMirror Tests";
    $self->{filers}        = $tinfo->{filers};
    $self->{sserver}       = $tinfo->{sserver} if ( exists $tinfo->{sserver} );
    if (exists $tinfo->{tests}{$self->{tid}}{cfg}) {
        $self->{cfg} = $tinfo->{testcfg}{SMO};
    } else {
      $self->_get_config_vars();
    }
    $self->{cfg}{src_count} = $self->{cfg}{maxxfers} / 2;
    $self->{cfg}{dst_count} = $self->{cfg}{maxxfers} / 4;

    return ( $self->{error} = "CONFIG ERROR: No Client Configured" )
         if ( !$self->{cfg}{client} );
    $self->{error} = 0;
    print "DBGCEB:IFTSMOTests:_initialize_sm_config: end\n self = ",
          Dumper($self) if ($DBGCEB);
    getc;
    return;
}

sub _get_config_vars {
   my $self = shift;
    print "DBGCEB:IFTSMOTests:_get_config_vars\n" if ($DBGCEB);
   $self->{cfg}{$AGGR}{prefix} =
             ($main::SM_AGGR_PREFIX)
            ? $main::SM_AGGR_PREFIX
            : "testag";
   $self->{cfg}{$AGGR}{size} =
             ($main::SM_AGGR_SIZE)
            ? $main::SM_AGGR_SIZE
            : 3;
   $self->{cfg}{$AGGR}{raid} =
             ($main::SM_AGGR_RAID)
            ? $main::SM_AGGR_RAID
            : "raid4";
   $self->{cfg}{$FLEX}{prefix} =
             ($main::SM_VOL_PREFIX)
            ? $main::SM_VOL_PREFIX
            : "sm_vol_";
   $self->{cfg}{$FLEX}{size} =
             ($main::SM_VOL_SIZE)
            ? $main::SM_VOL_SIZE
            : "300M";
   $self->{cfg}{$QTREES}{prefix} =
             ($main::SM_QTREE_PREFIX)
            ? $main::SM_QTREE_PREFIX
            : "sm_qt_";
   $self->{cfg}{client} =
             ($main::SM_CLIENT)
            ? $main::SM_CLIENT
            :"localhost";
   $self->{cfg}{maxxfers} =
             ($main::SM_MAXTRANSFERS)
            ? $main::SM_MAXTRANSFERS
            : 32;
   return;
}

# _setup_filers - if the test manager has not done the initial filer setup,
# do that, if it has, do the snapmirror setup.
sub _setup_filers {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_setup_filers\n" if ($DBGCEB);
    return ( $self->{error} = "CONFIG ERROR:no filers specified" )
      if ( !exists $self->{filers} );
    if ( ref( $self->{filers} ) eq "ARRAY" ) {
        my @filers = [ $self->{filers} ];
        $self->{filers} = {};
        foreach my $filer ( keys %{ $self->{filers} } ) {
            my $f = IFTFilerSetup->new($filer);
            $f->setup_licenses( $self->{licenses} );
            $self->{filers}{ $f->{fname} } = $f;
            $f->get_existing_containers($AGGR);
        }
    } elsif ( ref( $self->{filers} ) ne "HASH" ) {
        return ( $self->{error} = "CONFIG: No filers were specified" );
    } else {
        print "DBGCEB:IFTSMOTests:_setup_filers: filers already setup\n"
             if ($DBGCEB);
    }
    return;
}

sub _setup_sm {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_setup_sm\n" if ($DBGCEB);
    $self->_setup_containers();
    return if ( $self->{error} );
    $self->_populate_source;
    return if ( $self->{error} );
    $self->_config_sm();
    print "DBGCEB:IFTSMOTests:_setup_sm: end\n" if ($DBGCEB);
    return;
}

# _setup_containers -
#   1) Select an existing aggr, if no aggr, create one.
#   2) Create volumes
#   3) create qtrees
#   The result will be a set of snapmirror source and local and remote
#   destination structures.
# the colum
sub _setup_containers {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_setup_containers\n" if ($DBGCEB);

    # start with source info
    $self->{type} = $SRC;
    my $rc = undef;
    foreach my $f ( keys %{ $self->{filers} } ) {
        my $filer = $self->{filers}{$f};
        if ( $self->{type} eq $SRC ) {
            $self->{src_filer} = $filer;
        } else {
            $self->{dst_filer} = $filer;
        }

        $self->_select_aggrs($f) if ( $filer->{containers}{count}{$AGGR} );
        $self->_create_aggr($f)
                        if ( !exists $self->{ $self->{type} }{aggr} );
        last                   if ( $self->{error} );
        $self->_setup_volumes($f);
        last if ( $self->{error} );
        $self->_setup_qtrees($f);
        last if ( $self->{error} );
        $self->{type} = $DST;
        print "DBGCEB: now we are doing DST\n" if ($DBGCEB);
        getc if ($DBGCEB);
    }
    return if ($self->{error});
    #load src volumes/qtrees in test data structure
    my $count = 1;
    foreach my $v ( keys %{ $self->{vols} } ) {
        my $vol = $self->{vols}{$v};
        next if ($vol->{role} ne $SRC);
        my $test = "test" . scalar($count);
        my $type = $vol->{type};
        $self->{tests}{$test}{type} = $type;
        if ($type eq $QSM) {
            $self->{tests}{$test}{src} = $vol->{qtree}{path};
            $self->{tests}{$test}{srcname} = $vol->{qtree}{name};
        } else {
            $self->{tests}{$test}{src} = $vol->{path};
            $self->{tests}{$test}{srcname} = $vol->{name};
        }
        $self->{tests}{$test}{srchost} = $vol->{filer}{name};
        $self->{tests}{$test}{srcvol} = $vol->{name};
        delete $self->{vols}{$v};
        $count++;
    }

    # load dst volumes/qtrees in test data structure
    foreach my $v ( keys %{ $self->{vols} } ) {
        my $vol = $self->{vols}{$v};
#        next if ($vol->{role} ne $DST);
        foreach my $test ( keys %{ $self->{tests} } ) {
            next if ( exists $self->{tests}{$test}{smname} );
            print "DBGCEB: test type = $self->{tests}{$test}{type} ".
                  "vol type = $vol->{type}\n";
            if ($vol->{type} =~ /QSM/ ) {
                next if ($self->{tests}{$test}{type} !~ /QSM/ );
            } elsif ($vol->{type} =~ /VSM/ ) {
                next if ($self->{tests}{$test}{type} !~ /VSM/ );
            }

#            next if ( ( $self->{tests}{$test}{type} !~ /QSM/ )
#                         && ( $vol->{type} =~ /QSM/ ));
#            next if (( !$self->{tests}{$test}{type} !~ /VSM/ )
#                       && ($vol->{type} =~ /VSM/));
            print "DBGCEB: we have a match\n";
            if ($self->{tests}{$test}{type} =~ /QSM/) {
                $self->{tests}{$test}{dstname} = $vol->{qtree}{name};
                $self->{tests}{$test}{dst} = $vol->{qtree}{path};
                $self->{tests}{$test}{smname} = $vol->{qtree}{name};
            } else {
                $self->{tests}{$test}{dstname} = $vol->{name};
                $self->{tests}{$test}{smname} = $vol->{name};
                $self->{tests}{$test}{dst} = $vol->{path};
            }
            $self->{tests}{$test}{dsthost} = $vol->{filer}{name};
            $self->{tests}{$test}{dstvol} = $vol->{name};
            $self->{tests}{$test}{type} = $vol->{type};
            print "\n\n\nDBGCEB:IFTSMOTests:_setup_containers: test struct $test = ",
              Dumper($self->{tests}{$test}) if ($DBGCEB);
            last;

        }
    }
    print "DBGCEB: sm tests = ", Dumper( $self->{tests} );
    getc;    #DBGCEB
    return;
}

sub _setup_volumes {
    my $self    = shift;
    my $filer   = shift;
    my @volumes = undef;
    my $fc      = undef;
    $self->{vols} = {};
    my $vinfo   = $self->{vols};
    my $filer = undef;

    print "DBGCEB:IFTSMOTests:_setup_volumes\n" if ($DBGCEB);
    # if source we create the source and local destination volumes for vsm
    # and qsm
    if ( $self->{type} = $SRC ) {
        @volumes = (
            {
                'prefix' => $self->{cfg}{$FLEX}{prefix} . "vsm_src",
                'count'  => $self->{cfg}{src_count},
                'type'   => $VSM,
                'role' => $SRC,
            },
            {
                'prefix' => $self->{cfg}{$FLEX}{prefix} . "qsm_src",
                'count'  => $self->{cfg}{src_count},
                'type'   => $QSM,
                'role' => $SRC,
            },
            {
                'prefix' => $self->{cfg}{$FLEX}{prefix} . "qsm_local_dst",
                'count'  => $self->{cfg}{dst_count},
                'type'   => $QSM,
                'role' => $DST,
            },
            {
                'prefix' => $self->{cfg}{$FLEX}{prefix} . "vsm_local_dst",
                'count'  => $self->{cfg}{dst_count},
                'type'   => $VSM,
                'role' => $DST,
            },
        );
        $filer = $self->{src_filer};
    } else {
        @volumes = (
            {
                'prefix' => $self->{cfg}{$FLEX}{prefix} . "vsm_remote_dst",
                'count'  => $self->{cfg}{dst_count},
                'type'   => $RVSM,
                'role' => $DST,
            },
            {
                'prefix' => $self->{cfg}{$FLEX}{prefix} . "qsm_remote_dst",
                'count'  => $self->{cfg}{dst_count},
                'type'   => $RQSM,
                'role' => $DST,
            },
        );
        $filer = $self->{dst_filer};
    }

    my ($size, $name, $size, $guarantee, $aggr, $smvol, $path);
    my $fvolumes = $filer->{containers}{volume};
    my @types = ($FLEX, $QTREES);
    $filer->get_existing_containers(\@types,
                 flex_prefix => $self->{cfg}{$FLEX}{prefix},
                 qtree_prefix => $self->{cfg}{$QTREES}{prefix});


    foreach my $vol (@volumes) {
        my $name = undef;
        my $smvol = undef;
        for ( 1 ... $vol->{count} ) {
            $name = $vol->{prefix};
            print "\nDBGCEB:IFTSMOTests:_setup_volumes looking for $name\n";
            getc if ($self->{type} eq $DST);
            # check if a sm volume with this prefix already exists
            foreach my $fvol (keys %{$fvolumes}) {
                next if (exists $fvolumes->{$fvol}{sm});
                print "DBGCEB:IFTSMOTests:_setup_volumes found $fvol\n"
                     if (($fvol =~ /sm/) && ($DBGCEB));
                next if ($fvol !~ /$name/);
                $smvol = $fvol;
                $fvolumes->{$fvol}{sm} = 1;
                last;
            }
            if (!$smvol) {
               # we need to create the volume
               $name .= time();
               print "DBGCEB:IFTSMOTests:_setup_volumes: creating a new ".
                     " volume $name\n" if ($DBGCEB);
               $size      = $self->{cfg}{$FLEX}{size};
               $guarantee = "none";
               $aggr      = $self->{ $self->{type} }{aggr};
               print "DBGCEB:IFTSMOTests:_setup_volumes vol = $name aggr = ".
                     "$aggr size = $size guarantee = $guarantee end\n"
                      if ($DBGCEB);
   #                  getc if (DBGCEB);

               eval {
                   vol_create_flex( $filer->{fc}, $name, $aggr, $size,
                       guarantee => $guarantee );
               };
               $self->{error} = sprintf "%s: cannot create %s :$@", $filer,
                    $name  if $@;
               return if ( $self->{error} );
               $path = "/vol/$name";
               export_path( $filer->{fc}, $path )
                  or logWarn(
                "SMO_Setup:_setup_volumes: $filer->{fname}: cannot export "
                  . "$path" );
               $vinfo->{$name}{name}      = $name;
               $vinfo->{$name}{size}      = $size;
               $vinfo->{$name}{guarantee} = $guarantee;
               $vinfo->{$name}{path}      = $path;
            }  else {
               print "DBGCEB:IFTSMOTests:_setup_volumes: grabbing an existing".
                     " volume $name\n" if ($DBGCEB);
               $name = $smvol;
               $vinfo->{$name}{name} = $smvol;
               $vinfo->{$name}{path} = $fvolumes->{$name}{path}
            }
            $vinfo->{$name}{filer}{ipaddr}     = $filer->{ipaddr};
            $vinfo->{$name}{filer}{name}  = $filer->{fname};
            $vinfo->{$name}{filer}{fc}        = $filer->{fc};
            $vinfo->{$name}{type} = $vol->{type};
            $vinfo->{$name}{role} = $vol->{role};
        }
    }
    return;
}

sub _setup_qtrees {
    my $self  = shift;
    my $filer = shift;
    print "DBGCEB:IFTSMOTests:_setup_qtrees\n" if ($DBGCEB);
    foreach my $v ( keys %{ $self->{vols}} ) {
        my $vol = $self->{vols}{$v};
        print "DBGCEB:IFTSMOTests:_setup_qtrees: $v = ",
               Dumper($vol) if ($DBGCEB);
        next if ($vol->{type} !~ /QSM/);
        $self->_create_qtree($vol);
        last if $self->{error};
    }
    return;
}

sub _create_qtree {
    my $self = shift;
    my $vol  = shift;
    print "DBGCEB:IFTSMOTests:_create_qtree\n" if ($DBGCEB);

    my $filer  = $vol->{filer}{fc};
    my $name   = $self->{cfg}{$QTREES}{prefix} . time();
    my $qtpath = "/vol/$vol->{name}/$name";
    qtreeCreate( $filer, $qtpath )
      or $self->{error} = "Unable to create qtree $qtpath on $filer:$@";

    print "DBGCEB:IFTSMOTests:_create_qtree: error = $self->{error}\n"
         if (($self->{error}) && ($DBGCEB));
    return if ( $self->{error} );
    $vol->{qtree}{name} = $name;
    $vol->{qtree}{path} = $qtpath;
    export_path( $filer, $qtpath );

    return;
}

sub _select_aggrs {
    my $self  = shift;
    my $filer = shift;
    print "DBGCEB:IFTSMOTests:_select_aggrs filer=$filer\n" if ($DBGCEB);
    foreach my $aggr ( keys %{ $self->{filers}{$filer}{containers}{$AGGR} } ) {
        my ( $total, $used, $avail, $capacity ) = aggr_disk_free(
            $self->{filers}{$filer}{fc},
            path        => $aggr,
            space_type  => DF_SPACE,
            report_type => UNIT_MB
          )
          or return ( $self->{error} =
              "Unable to get free space from $aggr " . " on $filer" );
        if ( $avail >= $AGGR_SPACE ) {
            $self->{ $self->{type} }{aggr} = $aggr;
            last;
        }
    }
    print "DBGCEB:IFTSMOTests:_select_aggrs: end\n $self->{type} = ",
          Dumper($self->{$self->{type}})
              if ($DBGCEB);
#    getc if ($DBGCEB);

    return;
}

sub _create_aggr {
    my $self  = shift;
    my $filer = shift;
    print "DBGCEB:IFTSMOTests:_create_aggr\n" if ($DBGCEB);
    my $size  = $self->{cfg}{$AGGR}{size};
    my $name  = $self->{cfg}{$AGGR}{prefix};
    my $raidtype = $self->{cfg}{$AGGR}{raid};
    $name .= time();
    eval {
        aggr_create( $self->{filers}{$filer}{fc},
            $name, $size, raidtype => $raidtype );
    };
    $self->{error} = "$filer: cannot create $name :$@" if $@;
    $self->{ $self->{type} }{aggr} = $name if ( !$self->{error} );
    return;
}

sub _config_sm {
    print "DBGCEB:IFTSMOTests:_config_sm\n" if ($DBGCEB);
    return;
}


sub _populate_source {
    my $self     = shift;
    my $path     = undef;
    my $name     = undef;
    my $vol_name = undef;
    print "DBGCEB:IFTSMOTests:_populate_source\n" if ($DBGCEB);
    my $client = "localhost";
    logcomment("Snapmirror Test: populating source containers");
    getc; #DBGCEB
    $self->_mount_sources();
    foreach my $test ( keys %{ $self->{tests} } ) {
        my $mntpt = $self->{tests}{$test}{mp};
        my $file = "$mntpt/file." . ( time() );
        my $vol_name = $self->{tests}{$test}{srcvol};
        my $fc = $self->{src_filer}{fc};

        my $vol_avail = vol_disk_free(
                    $fc,
                    path        => $vol_name,
                    report_type => UNIT_MB
        );

         my $file_size = ( $vol_avail->{avail} ) / 8;
         $file_size = int( $file_size + 1 ) . "m";
         filer_file_make( $fc, $file, $file_size )
                  or return ($self->{error} =
                  "could not populate ".
                  "$self->{tests}{$test}{srcvol}{filer}{name}:$path");
    }
    return;
}



#######################################################################
#
#  Test Execution Routines
#
#######################################################################
my $CMD =<<'EOF';
{
    use strict;
    use Data::Dumper;
    use Fcntl qw(:DEFAULT);
    use File::Copy;
    use File::Compare;
    use File::Path;
    use File::Find;
    use File::Basename;
    use IO::Handle;
    use IFT::IFTSMOTests;
    use Cwd;
    use Tharn;
    use SPARSE::Util::Logger qw(logError logTrc logStep logWarn);
    my $self = shift;
    my $hostname = `hostname`;
    my $tests = {};
    my $RETRIES = 3;
    my $DBGCEB = 1;

    _client_traffic($self);
}
EOF



sub start_tests {
    my $self = shift;
    my $tid  = shift;
    print "DBGCEB:IFTSMOTests:start_tests\n" if ($DBGCEB);
    $self->_start_traffic();
    my $tcount = 0;
    foreach my $test ( keys %{ $self->{tests} } ) {
        my $smtest = $self->{tests}{$test};
        $smtest->{tid} = $self->{tid} . ":" . $tcount;
        $smtest->{runid}   = "$smtest->{tid}:$smtest->{type}";
        $smtest->{subtest} =
          subtest( \&_sm_test_loop, -bg, -ipc, -runid, $smtest->{runid}, "--",
            $smtest );
        return ( $self->{error} = "Unable to start $smtest->{runid}" )
          if ( !$smtest->{subtest} );
        $tcount++;
    }
}

sub _sm_test_loop {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_sm_test_loop\n" if ($DBGCEB);
    $self->{starttime} = time();
    _init_stats($self);
    _init_sm($self);
    while (1) {
        if ( !_do_sm_test($self) ) {
            last;
        }
        if ( check_duration() ) {
            logresult( 'PASS', "Subtest was successful" );
            last;
        }
        sleep( int( rand($SLEEP_TIME) ) );
    }
    print_stats($self);
}

sub _do_sm_test {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_do_sm_test\n" if ($DBGCEB);
    return 0 if !update_sm($self);
    return 0 if !break_sm($self);
    sleep( int( rand($SLEEP_TIME) ) );
    return 0 if !resync_sm($self);
    return 1;
}

sub _init_sm {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_init_sm\n" if ($DBGCEB);
    my ( $src, $dst, $srchost, $dsthost );
    my $sm_obj =  eval {
        sm_init(
            srchost    => $self->{srchost},
            dsthost    => $self->{dsthost},
            src        => $self->{src},
            dst        => $self->{dst},
            background => 0,
            timeout    => 9600
        );
    };

    if ($@) {
        logWarn("init_sm: $@");
        return 0;
    } else {
        log_transfer_info($self);
    }
    $self->{status} = "init";
    return;
}

sub _init_stats {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_init_stats\n" if ($DBGCEB);
    $self->{stats}{update}{passed}  = 0;
    $self->{stats}{update}{failed}  = 0;
    $self->{stats}{quiesce}{passed} = 0;
    $self->{stats}{quiesce}{failed} = 0;
    $self->{stats}{break}{passed}   = 0;
    $self->{stats}{break}{failed}   = 0;
    $self->{stats}{resync}{failed}  = 0;
    $self->{stats}{resync}{failed}  = 0;
}

sub print_stats {
    my $self = shift;
    my @msgs = ();
    print "DBGCEB:IFTSMOTests:print_stats\n" if ($DBGCEB);

    push @msgs, "sm stats";
    push @msgs, "update: passed $self->{stats}{update}{passed}";
    push @msgs, "update: failed $self->{stats}{update}{failed}";
    push @msgs, "quiesce: passed $self->{stats}{quiesce}{passed}";
    push @msgs, "quiesce: failed $self->{stats}{quiesce}{failed}";
    push @msgs, "break:        passed $self->{stats}{break}{passed}";
    push @msgs, "break: failed $self->{stats}{break}{failed}";
    push @msgs, "resync: passed $self->{stats}{resync}{passed}";
    push @msgs, "resync: failed $self->{stats}{resync}{failed}";
    logcomment(@msgs);
}

sub _mount_sources {
    my $self = shift;
    my $client = shift;

    $client = `hostname` if (!$client);
    $client =~ s/\..*//;
    print "DBGCEB:IFTSMOTests:_mount_sources $client\n" if ($DBGCEB);
    my @mounts = ();

    foreach my $test (keys %{$self->{tests}}) {
       my $ti = $self->{tests}{$test};

       print "DBGCEB:IFTSMOTests:_mount_sources: test=$test, ti = ",
             Dumper($ti) if ($DBGCEB);
       getc if ($DBGCEB);
       my $ip = $self->{src_filer}{ipaddr};
       my $mntpt = "/t/$ip/".$ti->{srcname};
       print "DBGCEB:IFTSMOTests:_mount_sources $test mounting $mntpt ".
             "ip=$ip src=$ti->{src} client = $client\n"
          if ($DBGCEB);

       my $mount_dir = eval { mount(
                                    $ip,
                                    $ti->{src},
                                    mountpoint => $mntpt,
#                                   mountopts  => $mountopts,
                                    client     => $client
          ) };

       logWarn("$self->{tid}:cannot create mountpoint " .
                    "for $mntpt: $@") if (!$mount_dir || $@);
       if ($mount_dir) {
          print "DBGCEB:IFTSMOTests:_mount_sources: mount worked\n"
               if ($DBGCEB);
          $ti->{mp} = $mount_dir;
          push @mounts, $mount_dir;
       }
    }
    $self->{mounts} = [@mounts];

    print "DBGCEB:IFTSMOTests:_mount_sources end\n" if ($DBGCEB);
    getc if ($DBGCEB);
    return;
}

sub _start_traffic {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_start_traffic\n" if ($DBGCEB);
    $self->{client_subtest} = subtest($CMD,
                           '-bg',
                           '-ipc',
                           '-runid', "snapmirror traffic $self->{cfg}{client}",
                           '-client', $self->{cfg}{client},
                           '--', $self);

    return;
}

sub stop_traffic {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_stop_traffic\n" if ($DBGCEB);
    foreach my $subtest ( @{ $self->{subtests} } ) {
        next unless defined($subtest);
        $subtest->kill();
    }
}


sub _get_sm_status {
    my $self   = shift;
    print "DBGCEB:IFTSMOTests:get_sm_status\n" if ($DBGCEB);
    my $status = undef;
    eval {
        $status = snapmirror_status(
            host => $self->{dsthost},
            dst  => $self->{dst}
        );
        print Dumper($status) if $DBGCEB;
    };
    if ($@) {
        logWarn("get_sm_status: $@");
        return 0;
    } else {
        return $status;
    }
}

sub log_transfer_info {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:log_transfer_info\n" if ($DBGCEB);
    if ( my $status = _get_sm_status($self) ) {
        my $text = sprintf "type %s, size %sk, duration %ss",
          $status->{'last-transfer-type'}, $status->{'last-transfer-size'},
          $status->{'last-transfer-duration'};
        logcomment("transfer info: $text");
    }
    return;
}

sub update_sm {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:update_sm\n" if ($DBGCEB);

    while (1) {
        eval {
            snapmirror_update(
                host => $self->{dsthost},
                src  => $self->{src},
                dst  => $self->{dst}
            );
        };
        if ($@) {
            if ( $@ =~ /$too_many_updates/ ) {
                sleep( int( rand($SLEEP_TIME) ) );
                print "DBGCEB: update_sm: retrying\n" if $DBGCEB;
                next;
            } else {
                logWarn("update_sm: $@");
                $self->{stats}{update}{failed}++;
                return 0;
            }
        } else {
            $self->{stats}{update}{passed}++;
            log_transfer_info($self);
            return 1;
        }
    }
}

sub break_sm {
    my $self    = shift;
    print "DBGCEB:IFTSMOTests:break_sm\n" if ($DBGCEB);

    my $dstpath = ( split ( /:/, $self->{dst} ) )[1];

    if ( $self->{is_qsm} ) {
        eval {
            snapmirror_quiesce( host => $self->{dsthost}, dstpath => $dstpath );
        };
        if ($@) {
            logWarn("break_sm: $@");
            $self->{stats}{quiesce}{failed}++;
            return 0;
        } else {
            $self->{stats}{quiesce}{passed}++;
        }
    }

    eval { snapmirror_break( host => $self->{dsthost}, dstpath => $dstpath ); };
    if ($@) {
        logWarn("break_sm: $@");
        $self->{stats}{break}{failed}++;
        return 0;
    } else {
        $self->{stats}{break}{passed}++;
        return 1;
    }
}

sub resync_sm {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:resync_sm\n" if ($DBGCEB);

    while (1) {
        logcomment("resync_sm: $self->{src}, $self->{dst}");
        eval {
            snapmirror_resync(
                host => $self->{dsthost},
                src  => $self->{src},
                dst  => $self->{dst}
            );
        };
        if ($@) {
            if ( $@ =~ /$too_many_updates/ ) {
                sleep( int( rand($SLEEP_TIME) ) );
                print "DBGCEB: resync_sm: retrying\n" if $DBGCEB;
                next;
            } else {
                logWarn("resync_sm: $@");
                $self->{stats}{resync}{failed}++;
                return 0;
            }
        } else {
            $self->{stats}{resync}{passed}++;
            return 1;
        }
    }
}

################################################################################
# Client routines
################################################################################

sub _client_traffic {
    my $self = shift;
    print "DBGCEB:IFTSMOTests:_client_traffic\n" if ($DBGCEB);
    my $mounts = _mount_sources($self);
    foreach my $mount (@$mounts) {
        _create_directory_tree($self,$mount);
    }
    while (1) {
    foreach my $mount (@$mounts) {
        _write_files($self,$mount);
        _delete_files($self,$mount);
    }
    }
    return;
}

sub _create_directory_tree {
    print "DBGCEB: in _create_directory_tree";
    return;
}

sub _write_files {
    print "DBGCEB: in _write_files";
    return;
}

sub _delete_files {
    print "DBGCEB: in _delete_files";
    return;
}

1;
