#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTTestManager.pm#1 $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary        IFT general utilities library for WICO QA Integrated
##                 functional test
## Functional test suite.
## @description
##             General IFT utilities
##
## @author bundy

use strict;

package IFTTestManager;
use Tharn;
#use IFT::IFTComm;
use IFT::IFTClient;
use IFT::IFTSMOTests;
use IFT::IFTSetup;
use IFT::IFTFDOTests;
use IFT::IFTAVOTests;
use IFT::IFTWICOTests;
use IFT::IFTSIOTests;
#use IFT::IFTQQOTests;

use SayWrap qw(sayEOF);
use Data::Dumper;
use SPARSE::Util::Logger qw(logStep logWarn logDbg logTrc);

our $QUIT      = "quit";
our $SUSPEND   = "suspend";
our $RESUME    = "resume";
our $GETSTATUS = "get status *";
our $RESULTS   = "results";
our $OK        = "OK";
our $RETRIES   = 30;
our $AGGR      = "aggr";
our $FLEX      = "flex";
our $QTREES  = "qtrees";
our $TRAD      = "trad";
our $LUNS      = "LUNs";
our $QTREES    = "qtrees";
our $ACK       = "ACK";
our $NCK       = "NCK";
our $PURGE     = "purge";
our $DBGCEB    = 1;

our($self);

our %test_info = (
    "CFO" => {
        'setup' => \&CFO_Setup,
    },
    "AVO" => {
        'setup' => \&AVO_Setup,
    },
    "SIO" => {
        'setup' => \&SIO_Setup,
    },
    "SMO" => {
        'setup' => \&SMO_Setup,
    },
    "WCO" => {
        'setup' => \&WICO_Setup,
    },
    "WIO" => {
        'setup' => \&WICO_Setup,
    },
    "FDO" => {
        'setup' => \&FDO_Setup,
    },
    "QQO" => {
        'setup' => \&QQO_Setup,
    },
);

sub new {
    my $class    = shift;
    my $testname = shift;
    my $testid   = shift;
    $self = {} unless (ref($self));
    bless $self, $class;

    print "DBGCEB:IFTTestManager:new\n" if ($DBGCEB);
    if ($main::CONFIG_FILE) {
        _process_config_file($self,$testname,$testid);
    } else {
        _process_config_vars($self,$testname,$testid);
    }

    # get filer info
    # get tests
    $self->_comm_server_setup() if ( $self->{cfg}{sserver} );
    die config_result("test setup failure: $self->{error}")
      if ( $self->{error} );
    $self->{cfg}{duration} = $self->_get_duration( $main::DURATION );
    $self->{endtime} = $self->{cfg}{duration} + time();
    die config_result("test setup failure: $self->{error}")
      if ( $self->{error} );

    # setup filer
    $self->Filer_setup();

    # setup individual tests
    $self->_test_setup();
    die config_result("test setup failure: $self->{error}")
      if ( $self->{error} );

    return($self);
}

sub _process_config_file {
    print "DBGCEB:IFTTestManager:_process_config_file\n" if ($DBGCEB);
    my $testname = shift;
    my $testid = shift;
    return;
}

sub _process_config_vars {
    print "DBGCEB:IFTTestManager:_process_config_vars\n" if ($DBGCEB);
    my $self = shift;
    my $testname = shift;
    my $testid = shift;
    my $value = undef;
    my $config_hash =
      {
        'testname' => $testname,
        'tid'      => $testid,
        'cfg'      => {
            'filers'   => undef,
            'cleanup'  => ($main::CLEANUP) ? $main::CLEANUP : "none",
            'duration' => ($main::DURATION) ? $main::DURATION : '72H',
            'sserver'  => ($main::STATUS_SERVER)
            ? $main::STATUS_SERVER
            : undef,
            'cmdtimeout' => ($main::CMDTIMEOUT) ? $main::CMDTIMEOUT : 600,
            'tests'    => $main::TESTS,
            'iftlogfile' => ($main::LOGFILE)    ? $main::LOGFILE    : undef,
        },
        ' clientcfg' => {
            'conn' => ($main::CLIENT_CONN) ? $main::CLIENT_CONN : "#rsh-persay",
            'max_mounts' => ($main::MAX_MOUNT_POINTS) ? $main::MAX_MOUNT_POINTS
                           : 0,
        },
        ' config_file ' => ($main::CONFIG_FILE) ? $main::CONFIG_FILE : undef,
        'filercfg' => {
            'licenses' => ($main::LICENSES)
            ? $main::LICENSES
            : (
                'nfs',             'http',
                'cifs',            'fcp',
                'cluster',         'flex_clone',
                'iscsi',           'snapmirror',
                'snapmirror_sync', 'snaprestore'
            ),
            "aggr" => {
                'count'  => ($main::AGGR_COUNT) ? $main::AGGR_COUNT : -1,
                'size'   => ($main::AGGR_SIZE)  ? $main::AGGR_SIZE  : 3,
                'raid' => ($main::AGGR_RAID)  ? $main::AGGR_RAID  : "raiddp",
                'prefix' => ($main::AGGR_PREFIX) ? $main::AGGR_PREFIX
                             : "testag",
            },
            "flex" => {
                'count' => ($main::FLEX_COUNT) ? $main::FLEX_COUNT : 0,
                'size' => ($main::FLEX_SIZE) ? $main::FLEX_SIZE : "300M",
                'prefix'    => ($main::FLEX_PREFIX) ? $main::FLEX_PREFIX :
                               "testflex",
                'guarantee' => ($main::FLEX_GUARANTEE)
                               ? $main::FLEX_GUARANTEE : "none",
            },
            "trad" => {
                'count'  => ($main::TRAD_COUNT) ? $main::TRAD_COUNT : -1,
                'size'   => ($main::TRAD_SIZE)  ? $main::TRAD_SIZE  : 2,
                'raid' => ($main::TRAD_RAID)  ? $main::TRAD_RAID  : "raid4",
                'prefix' => ($main::TRAD_PREFIX)
                ? $main::TRAD_PREFIX
                : "testtrad",
            },
            "lun" => {
                'count' => ($main::LUN_COUNT) ? $main::LUN_COUNT : -1,
                'size' => ($main::LUN_SIZE) ? $main::LUN_SIZE : "50m",
                'prefix' => ($main::LUN_PREFIX) ? $main::LUN_PREFIX : "testlun",
            },
            'qtree' => {
                'count' => ($main::QTREE_COUNT) ? $QTREE::LUN_COUNT : -1,
                'prefix' => ($main::QTREE_PREFIX) ? $main::QTREE_PREFIX
                : "testqt",
            },
        },
    };
    while ( my($key,$val) = each(%$config_hash) ) {
          $self->{$key} = $val;
    }

    $self->{error} = _parse_test_list( $self, $self->{cfg}{tests} );
    return if ( $self->{error} );
    foreach my $key ( @{ $self->{cfg}{tests} } ) {
          if ( $key eq "FDO" ) {

              # FDO variables
              $self->{testcfg}{FDO}{clients} =
                ($main::FDO_CLIENTS)
                ? $main::FDO_CLIENTS
                : undef;
              $self->{testcfg}{FDO}{maxmounts} =
                ($main::MAX_MOUNTPOINTS)
                ? $main::MAX_MOUNTPOINTS
                : 300;
          } elsif ( $key eq "AVO" ) {

              # AVO variables
              $self->{testcfg}{AVO}{snapshot_flag} = $main::AVO_SNAPSHOTS;
              $self->{testcfg}{AVO}{qtree_flag}    = $main::AVO_QTREES;
          } elsif ( $key eq "CFO" ) {

              # CFO variables
              $self->{testcfg}{CFO}{interval} =
                  _get_interval($main::CFO_INTERVAL)
                      if ( defined $main::CFO_INTERVAL );
          } elsif ( $key eq "RBO" ) {

              # RBO variables
              $value = ($main::REBOOT_INTERVAL) ? $main::REBOOT_INTERVAL : "1H";
              $self->{testcfg}{CFO}{interval} = _get_interval($value);
              $value = ($main::REBOOT_TIMEOUT) ? $main::REBOOT_TIMEOUT : "1H";
              $self->{testcfg}{CFO}{timeout} = _get_interval($value);
          } elsif ( $key eq "SMO" ) {

              # SMO variables
              $self->{testcfg}{SMO}{$AGGR}{prefix} =
                ($main::SM_AGGR_PREFIX)
                ? $main::SM_AGGR_PREFIX
                : "testag";
              $self->{testcfg}{SMO}{$AGGR}{size} =
                ($main::SM_AGGR_SIZE)
                ? $main::SM_AGGR_SIZE
                : 3;
              $self->{testcfg}{SMO}{$AGGR}{raid} =
                ($main::SM_AGGR_RAID)
                ? $main::SM_AGGR_RAID
                : "raid4";
              $self->{testcfg}{SMO}{$FLEX}{prefix} =
                ($main::SM_VOL_PREFIX)
                ? $main::SM_VOL_PREFIX
                : "sm_vol_";
              $self->{testcfg}{SMO}{$FLEX}{size} =
                ($main::SM_VOL_SIZE)
                ? $main::SM_VOL_SIZE
                : "300M";
              $self->{testcfg}{SMO}{$QTREES}{prefix} =
                ($main::SM_QTREE_PREFIX)
                ? $main::SM_QTREE_PREFIX
                : "sm_qt_";
              $self->{testcfg}{SMO}{client} =
                ($main::SM_CLIENT)
                ? $main::SM_CLIENT
                : "localhost";
              $self->{testcfg}{SMO}{maxxfers} =
                ($main::SM_MAXTRANSFERS)
                ? $main::SM_MAXTRANSFERS
                : 32;
          } elsif ( $key eq "SIO" ) {

              # SIO Variables
              $self->{testcfg}{SIO}{clients}        = $main::SIO_CLIENTS;
              $self->{testcfg}{SIO}{read_pct}       = $main::SIO_READ_PCT;
              $self->{testcfg}{SIO}{rand_pct}       = $main::SIO_RAND_PCT;
              $self->{testcfg}{SIO}{blksize}        = $main::SIO_BLKSIZE;
              $self->{testcfg}{SIO}{numthread}      = $main::SIO_NUMTHREAD;
              $self->{testcfg}{SIO}{filesize}       = $main::SIO_FILESIZE;
              $self->{testcfg}{SIO}{filename}       = $main::SIO_FILENAME;
              $self->{testcfg}{SIO}{max_open}       = $main::SIO_MAX_OPEN;
              $self->{testcfg}{SIO}{reboot_clients} = $main::SIO_REBOOT_CLIENTS;
              $self->{testcfg}{SIO}{sio_options}    = $main::SIO_OPTIONS;

          } elsif ( $key eq "WIO" || $key eq 'WCO') {

              # WIO/WCO Variables
              $value =
                ($main::WICO_INTERVAL) ? $main::WICO_INTERVAL : "2h";
              $self->{testcfg}{WICO}{interval} = $self->_get_duration($value);
              $value =
                ($main::WICO_TIMEOUT) ? $main::WICO_TIMEOUT : "2h";
              $self->{testcfg}{WICO}{timeout} = $self->_get_duration($value);
              $self->{testcfg}{WICO}{wackall} =
                ($main::WICO_WACK_ALL)
                ? $main::WICO_WACK_ALL
                : undef;

          } elsif ( $key eq "QQO" ) {

              # QQO Variables
              $self->{testcfg}{QQO}{quotas} =
                ($main::QUOTAS) ? $main::QUOTAS : undef;
          } else {
              return ( $self->{error} = "Invalid test: $key" );
          }
    }

    my @filers = ();
    if (!$main::FILERB) {
          push @filers, $main::FILER;
    } else {
          @filers = ( $main::FILERA, $main::FILERB );
    }
    foreach (@filers) { $self->{filers}{$_}{fname} = $_; }

}

sub _parse_test_list {
      $self = shift;
      my $teststr = shift;
      print "DBGCEB:IFTTestManager:_parse_test_list $teststr\n" if ($DBGCEB);
      return ( $self->{error} = "No tests specified" ) if ( !$teststr );
      my @tests = split /,/, $teststr;
      $self->{cfg}{tests} = [@tests];
      return 0;
}

sub _test_setup {
      $self = shift;
      print "DBGCEB:IFTTestManager:_test_setup\n" if ($DBGCEB);

      foreach my $test ( @{ $self->{cfg}{tests} } ) {
          if ( !exists $test_info{$test} ) {
              return ( $self->{error} = "$test does not exist" );
          }
          my $rc = &{ $test_info{$test}{setup} }( $self, $test );
          return if ( $self->{error} );
      }
      return;
}

# File and Directory Operations test setup
sub FDO_Setup {
      logStep("step: FDO setup");
      my $self       = shift;
      my $test       = "FDO";
      my $type       = "nfs";
      my $testcfg    = $self->{testcfg}{$test};
      my $cl_names   = $testcfg->{clients};
      my $clients    = IFTClient->new( $cl_names, $self, $type, $test );
      $testcfg->{clients} = $clients;
      return ( $self->{error} = $clients->{error} )
        if ( $clients->{error} );

      $clients->mount_clients( $self,
                 max_mounts => $testcfg->{maxmounts} );
      $self->{error} = $clients->{error}
        if ( $clients->{error} );

      while ( my($clname, $clinfo) = each(%{$clients->{clients}}) ) {
          my $testinfo                = {};
          my $tid                     = $self->_get_tid($test);
          $self->{tests}{$tid}        = $testinfo;
          my $mounts                  = $clinfo->{mounts} || {};

          $testinfo->{test_manager}   = $self;
          $testinfo->{test}           = $test;
          $testinfo->{tid}            = $tid;
          $testinfo->{endtime}        = $self->{endtime};
          $testinfo->{cfg}            = { %$testcfg };
          $testinfo->{cfg}{duration}  = $self->{cfg}{duration};
          $testinfo->{cfg}{type}      = $type;
          $testinfo->{cfg}{client}    = $clinfo;
          $testinfo->{cfg}{clients}   = $clients;
          $testinfo->{cfg}{mounts}    = { %$mounts };
          $testinfo->{obj}            = IFTFDOTests->new( $testinfo );

          return ( $self->{error} = $testinfo->{obj}{error} )
            if ( $testinfo->{obj}{error} );
      }

      return;
}    # end FDO_Setup

# Aggregate and Volume Operations test setup
sub AVO_Setup {
      logStep("step: AVO setup");
      my $self           = shift;
      my $test           = "AVO";
      my $testcfg        = $self->{testcfg}{$test};
      my $snapshot_flag  = $testcfg->{snapshot_flag};
      my $qtree_flag     = $testcfg->{qtree_flag};
      foreach my $filer ( values %{ $self->{filers} } ) {
          my $testinfo                 = {};
          my $tid                      = $self->_get_tid($test);
          $self->{tests}{$tid}         = $testinfo;
          $testinfo->{test_manager}    = $self;
          $testinfo->{test}            = $test;
          $testinfo->{tid}             = $tid;
          $testinfo->{endtime}         = $self->{endtime};
          $testinfo->{cfg}             = { %$testcfg };
          $testinfo->{cfg}{duration}   = $self->{cfg}{duration};
          $testinfo->{cfg}{filer}      = $filer;
          $testinfo->{obj}             = IFTAVOTests->new( $testinfo );

logresult('FATAL', "Line: ".__LINE__."   TID: \"$tid\"   ERROR: \"$testinfo->{obj}{error}\"")  if ( $testinfo->{obj}{error} );

          return ( $self->{error} = $testinfo->{obj}{error} )
            if ( $testinfo->{obj}{error} );
      }
      return;
}    # end AVO_Setup

# SIO Operations test setup
sub SIO_Setup {
      logStep("step: SIO setup");
      my $self           = shift;
      my $test           = "SIO";
      my $type           = "nfs";
      my $testcfg        = $self->{testcfg}{$test};
      my $cl_names       = $testcfg->{clients};
      my $clients        = IFTClient->new( $cl_names, $self, $type, $test );
      $testcfg->{clients} = $clients;
      return ( $self->{error} = $clients->{error} )
        if ( $clients->{error} );

      $clients->mount_clients( $self,
                 max_mounts => $testcfg->{maxmounts} );
      $self->{error} = $clients->{error}
        if ( $clients->{error} );

      while ( my($clname, $clinfo) = each(%{$clients->{clients}}) ) {
          my $testinfo                 = {};
          my $tid                      = $self->_get_tid($test);
          $self->{tests}{$tid}         = $testinfo;
          my $mounts                   = $clinfo->{mounts} || {};
          $testinfo->{test_manager}    = $self;
          $testinfo->{test}            = $test;
          $testinfo->{tid}             = $tid;
          $testinfo->{endtime}         = $self->{endtime};
          $testinfo->{cfg}             = { %$testcfg };
          $testinfo->{cfg}{duration}   = $self->{cfg}{duration};
          $testinfo->{cfg}{type}       = $type;
          $testinfo->{cfg}{client}     = $clinfo;
          $testinfo->{cfg}{clients}    = $clients;
          $testinfo->{cfg}{mounts}     = { %$mounts };
          $testinfo->{obj}             = IFTSIOTests->new( $testinfo );

          return ( $self->{error} = $testinfo->{obj}{error} )
            if ( $testinfo->{obj}{error} );

      }
      return;
}    # end SIO_Setup

# Clustered Failover Operations test setup
sub CFO_Setup {
      logStep("step: CFO setup");
      my $self     = shift;
      my $test     = "CFO";
      my $tid      = $self->_get_tid($test);
      my $testcfg  = $self->{testcfg}{$test};
      my $testinfo                = {};
      $self->{tests}{$tid}        = $testinfo;
      $testinfo->{test_manager}   = $self;
      $testinfo->{test}           = $test;
      $testinfo->{tid}            = $tid;
      $testinfo->{endtime}        = $self->{endtime};
      $testinfo->{cfg}            = { %$testcfg };
      $testinfo->{cfg}{duration}  = $self->{cfg}{duration};
      $testinfo->{cfg}{filers}    = $self->{filers};
      $testinfo->{obj}            = IFTCFOTests->new( $testinfo );
      return ( $self->{error} = $testinfo->{obj}{error} )
        if ( $testinfo->{obj}{error} );
      return;
}    # end CFO_Setup

# Reboot Operations testsetup
sub RBO_Setup {
      my $self = shift;
      logStep("step: RBO setup");
      my $test = "RBO";
      my $tid  = $self->_get_tid($test);
      my $testcfg        = $self->{testcfg}{$test};
      foreach my $filer ( values %{ $self->{filers} } ) {
          my $testinfo                = {};
          $self->{tests}{$tid}        = $testinfo;
          $testinfo->{test_manager}   = $self;
          $testinfo->{test}           = $test;
          $testinfo->{tid}            = $tid;
          $testinfo->{endtime}        = $self->{endtime};
          $testinfo->{cfg}            = { %$testcfg };
          $testinfo->{cfg}{duration}  = $self->{cfg}{duration};
          $testinfo->{cfg}{filer}     = $filer;
          $testinfo->{obj}            = IFTRBOTests->new( $testinfo );
      }
      $self->{error}             = $self->{tests}{$tid}{obj}{error}
        if ( $self->{tests}{$tid}{obj}{error} );
      return;
}    # end RBO_Setup

# SnapMirror Operations testsetup
sub SMO_Setup {
      my $self = shift;
      logStep("step: SMO setup");
      my $test = "SMO";
      my $tid = $self->_get_tid($test);
      $self->{tests}{$tid}{test} = $test;
      $self->{tests}{$tid}{cfg} = $self->{testcfg}{$test};
      $self->{tests}{$tid}{obj} = IFTSMOTests->new( $self, $tid );
      $self->{error}            = $self->{tests}{$tid}{obj}{error}
        if ( $self->{tests}{$tid}{obj}{error} );
      return;
}    # end SMO_Setup

# Common Wafliron/Waflcheck operations test setup
sub WICO_Setup {
      my $self    = shift;
      my $test    = "WICO";
      my $testcfg = $self->{testcfg}{$test};

      # Only do setup once, even if both 'WIO' and 'WCO' tests are specified
      return
          if ($self->{wafl_tests_setup});
      $self->{wafl_tests_setup} = 1;

      logStep("step: WICO (WIO/WCO) setup");
      my $tests = $self->{cfg}{tests};
      my $wafl_tests = [];
      push(@$wafl_tests, 'WCO')
          if (grep(/^WCO$/, @$tests));
      push(@$wafl_tests, 'WIO')
          if (grep(/^WIO$/, @$tests));

      my $wackall = $testcfg->{wackall};
      my $interval = $testcfg->{wackall};
      my $timeout = $testcfg->{wackall};

      foreach my $filer ( values %{ $self->{filers} } ) {
          my $testinfo                     = {};
          my $tid                          = $self->_get_tid($test);
          $self->{tests}{$tid}             = $testinfo;
          $testinfo->{test_manager}        = $self;
          $testinfo->{test}                = $test;
          $testinfo->{tid}                 = $tid;
          $testinfo->{endtime}             = $self->{endtime};
          $testinfo->{cfg}                 = { %$testcfg };
          $testinfo->{cfg}{wafl_tests}     = $wafl_tests;
          $self->{tests}{$tid}{cfg}{filer} = $filer;
          $testinfo->{cfg}{duration}       = $self->{cfg}{duration};
          $testinfo->{obj}                 = IFTWICOTests->new( $testinfo );
          return ($self->{error} = $testinfo->{obj}{error})
                if ( $testinfo->{obj}{error} );
      }
      return;
}    # end WICO_Setup


# Qtree Quota Operations test setup
sub QQO_Setup {
      $self = shift;
      logStep("step: QQO setup");
      my $test = "QQO";
      my $tid  = $self->_get_tid($test);
      $self->{tests}{$tid}{test} = $test;
      $self->{tests}{$tid}{cfg}  = $self->{testcfg}{$test};
      $self->{tests}{$tid}{obj}  = IFTQQOTests->new( $self, $tid );
      $self->{error}             = $self->{tests}{$tid}{obj}{error}
        if ( $self->{tests}{$tid}{obj}{error} );
      return;
}    #end QQO_Setup

sub Filer_setup {
      $self = shift;
      logStep("step:IFTTestManager:Filer setup");
      foreach my $flr ( keys %{ $self->{filers} } ) {
          my $filer = IFTFilerSetup->new($flr);
          return ( $self->{error} = $filer->{error} )
            if ( $filer->{error} );
          $filer->setup_licenses();
          return ( $self->{error} = $filer->{error} )
            if ( $filer->{error} );
          $filer->get_existing_containers();
          return ( $self->{error} = $filer->{error} )
            if ( $filer->{error} );
          delete $self->{filers}{$flr};
          $self->{filers}{ $filer->{fname} } = $filer;
      }
      return;
}    #end Filer_Setup

sub _comm_server_setup {
      $self = shift;
      $self->{hostname} = `hostname`;
      logStep("step: Comm Server setup");
      my $sserver = IFTComm::get_server();
      if ( !$sserver ) {
          my $sserver = IFTComm::new->( "start" );
          return ( $self->{error} = $sserver->{error} )
            if ( $sserver->{error} );
      }
      $sserver->connect_to_status_server( client => $self->{hostname} );
      return ( $self->{error} = $sserver->{error} )
        if ( $sserver->{error} );
      $self->{sserver}    = $sserver;
      $self->{statusconn} = $sserver->{server};
      return;
}


sub start_tests {
      $self = shift;
      $self->{starttime} = time();
#logresult('FATAL', "Line: ".__LINE__."   SELF->{TESTS}:\n\n".Dumper($self->{tests}));
logresult('INFO', "Enter IFTTestManager::start_tests");

      foreach my $tid ( keys %{ $self->{tests} } ) {
          my $testinfo = $self->{tests}{$tid};
          my $testobj = $testinfo->{obj};
	  my $testname = $testinfo->{test};
	  my $testtype = ref($testobj);
logresult('INFO', "IFTTestManager::start_tests  Calling ${testtype}::start_tests  TID: $tid");
logresult('INFO', "Line: ".__LINE__."   TID: \"$tid\"   ERROR REPORTED: \"$testobj->{error}\"")  if ( $testobj->{error} );
          eval { $testobj->start_tests($self) };
          if ($@) {
              logWarn("IFTTestManager::start_tests: Error returned from ${testtype}::start_test method for test (TID): $tid");
          }
          my $subt = $testobj->{subt};
          my $host = $testobj->{host};
          my $pid = $testobj->{pid};
          $testinfo->{subt} = $subt;
          $testinfo->{host} = $host if ($host);
          $testinfo->{pid} = $pid if ($pid);
logresult('INFO', "IFTTestManager::start_tests Line: ".__LINE__."   TID: \"$tid\"   Returned from ${testtype}::start_tests");
logresult('INFO', "${testtype}  TID: \"$tid\"   ERROR REPORTED: \"$testobj->{error}\"")  if ( $testobj->{error} );
          return ( $self->{error} = $testobj->{error} )
            if ( $testobj->{error} );
          sleep(5);
      }
logresult('INFO', "Exit IFTTestManager::start_tests");
      return;
}


sub check_duration {
      $self = shift;
      my $current = time();
      my $diff    = $current - $self->{starttime};

      if ( $diff >= $self->{cfg}{duration} ) {
          logcomment("Test time is up!");
          return 1;
      }
      return;
}

sub _get_duration {
      $self = shift;
      my $timevalue = shift;

      return ( $self->{error} = "No duration was specified" )
        if ( !$self->{cfg}{duration} );
      my ( $timecode, $value, $multiplier );
      my $errormsg =
        "Invalid duration: must be nH or nM\nwhere n - is number of\n"
        . "       H - is hours\n  M - is minutes";

      ( $value, $timecode ) = ( $timevalue =~ /(\d+)(\w)/ );
      die config_result($errormsg)
        if ( !$value );
      $timecode = "m"
        if ( !$timecode );
      $timecode = lc $timecode;
      if ( $timecode eq "h" ) {
          $multiplier = 3600;
      } elsif ( $timecode eq "m" ) {
          $multiplier = 60;
      } else {
          $self->{error} = $errormsg;
          return;
      }
      my $duration = $value * $multiplier;

#    print "DBGCEB:get_duration: duration=$self->{duration} starttime=$self->{starttime}\n";
#    getc;
      return $duration;
}

sub cleanup {
      my $self = shift;
      $self->stop_children();
      return;
}

sub _get_tid {
      my $self = shift;
      my $test = shift;
      $self->{testcount} = 0
        if ( !exists $self->{testcount} );
      $self->{testcount}++;
      my $tid = $self->{tid} . ":" . $test . ":" . $self->{testcount};
      return $tid;
}


sub watch_children {
    my $live_tests = 1;
    logTrc("Enter IFTTestManager::watch_children");
    while (!$self->check_duration() && $live_tests) {
         sleep(30);
         $live_tests = 0; 
         while ( my($tid,$testinfo) = each(%{$self->{tests}}) ) {
             my $subt = $testinfo->{subt};
             next if ($testinfo->{exited});
             if (!$subt) {
                 logWarn("No subtest defined for TID: $tid");
                 next;
             }
             $live_tests = 1;
#             logTrc("IFTTestManager: Checking test $tid");
             my $error = eval { $self->_check_subtest($subt, $testinfo) };
             if ($@) {
                  logWarn("Error returned from _check_subtest for subtest $tid: $@");
             } elsif ($error) {
                  logWarn("_check_subtest reports problem with subtest $tid: $error");
             }
             my $is_alive;
             if (!$subt) {
                 $is_alive = 0;
             } else {
                 $is_alive = eval { $subt->isAlive() };
                 if ($@) {
                      logWarn("Error returned from isAlive for subtest $tid: $@");
                 } elsif (!$is_alive) {
                     logWarn("IFTTestManager: ".
                             "Test $tid has apparently exited -- isAlive returned 0");
                     $testinfo->{exited} = 1;
                 }
             }
        }
    }
    logWarn("IFTTestManager: All tests have exited");
    logTrc("IFTTestManager: Stopping children");
    $self->stop_children();
    return;
}


sub stop_children {
    my $self = shift;
    logcomment("Stopping all subtests");
    while ( my($tid,$testinfo) = each(%{$self->{tests}}) ) {
       my $subt = $testinfo->{subt};
       $subt->kill()
           if ($subt);
    }
}


sub _get_subtest_message {
    my $self = shift;
    my $subt = shift;
    my $testinfo = shift;
logcomment("Enter _get_subtest_message");
    my $tid = $testinfo->{tid};
    my $msg = "";
    my $sock = $subt->ipc();
    if (!$sock) {
        print "DBGCEB:ift_tc3: could not find socket for $tid\n";
        return;
    }
    while ($sock->can_read()) {
        eval {
            $msg .= $sock->read_message();
        };
        if ($@) {
          my $exception = $@;
          logcomment("ift_tc3:check_results: unable to read $tid child ".
                         "socket: exception: $@");
             if ($exception =~ /Unexpected EOF/) {
                 logcomment("ift_tc3:test $tid is dead");
             }
             return($msg, $exception);
        }
        next if ($msg !~ /^ID\=/);
        logmessage($msg);
        my $header = ($msg =~ /^(.*MSG\=\d+)/);
        my $msg = "$header;$ACK=$ACK";
        eval {
           $sock->write_message($msg);
        };
        if ($@) {
            logcomment("ift_tc3:check_results: unable to write to $tid ".
                       "child socket:  exception: $@");
        }
        logmessage($msg);
    }
logcomment("Exit _get_subtest_message");
    return($msg,0);
}

sub _check_subtest {
    my $self = shift;
    my $subtest = shift;
    my $testinfo = shift;
    my $tid = $testinfo->{tid};
    return unless ($subtest);
    my @results = $subtest->results();
    return unless (@results);
    my $worst = Rslt::worst(@results);
    return unless ($worst);
    return if ($testinfo->{results}{$worst});
    $testinfo->{results}{$worst} = 1;
    my $runid = $worst->runid();
    my @messages = $worst->messages();
    my $msgs =  join("\n", @messages);
    $testinfo->{last_message} = $msgs;
    my $sev = $worst->severity();
    my $rtype = $worst->type();
    if ($sev >= 4) {
         my $error_message = "Subtest Error in $runid \n" . join("\n", @messages);
         $testinfo->{error} = $error_message;
         logWarn("Error from subtest $runid:\n" . $msgs);
         return ( $error_message );
    } elsif ($sev == 3) {
         logWarn("Warning from subtest $runid:\n" . $msgs);
    } else {
         logcomment("Message from subtest $runid:\n" . $msgs);
    }
    return;
}


sub logmessage {
    # TBD: Add logging to 'logfile'
    my $msg = shift;
    logcomment($msg);
}


# ------------------------------------------------------------------------
1;

# ------------------------------------------------------------------------
