#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/IFTWICOTests.pm#1 $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary        Library that defines useful wafliron/WACK routines
## @description
##
##
## @author bundy

package IFTWICOTests;

use strict;
use Tharn;
use Data::Dumper;
use SPARSE::Util::Logger qw(logDbg logWarn logStep);
use Utils qw(make_conn);
use boot qw(transit_to);
use wack;
use IFT::IFTSetup;
use Volume qw(vol_exists);
use Aggregate qw(aggr_exists);
use FS::Util::Utilities qw(uSystem);

our $DBGCEB;


sub new($$) {
    my $class         = shift;
    my $testname      = "wafl check/iron ops";
    my $testinfo      = shift;
    my $test_manager  = $testinfo->{test_manager};
    my $test          = $testinfo->{test};
    my $tid           = $testinfo->{tid};
    my $endtime       = $testinfo->{endtime};
    my $testcfg       = $testinfo->{cfg};
    my $filer         = $testcfg->{filer};
    my $wafl_tests    = $testcfg->{wafl_tests};
    my $timeout       = $testcfg->{timeout};
    my $interval      = $testcfg->{interval};
    my $wackall       = $testcfg->{wackall};
    my $self = {};

    if (scalar(@$wafl_tests) == 0) {
        $self->{error} = "No tests specified (empty wafl_tests)";
    } elsif (grep(!/^(WIO|WCO)$/, @$wafl_tests)) {
        $self->{error} = "Unknown wafl tests in wafl_tests: @$wafl_tests";
    } else {
        $self->{wafl_tests} = $wafl_tests;
    }

    $self->{test_manager}  = $test_manager;
    $self->{testname}      = $testname;
    $self->{test}          = $test;
    $self->{tid}           = $tid;
    $self->{wafl_tests}    = $wafl_tests;
    $self->{filer}         = $filer;
    $self->{timeout}       = $filer;
    $self->{interval}      = $interval;
    $self->{endtime}       = $endtime;
    $self->{wackall}       = $wackall;
    bless $self, $class;

    return ($self);
}


sub start_tests {
    my $self   = shift;
logresult('INFO', "Enter IFTWICOTests::start_tests Line: ".__LINE__);
    my $ownname = $self->{testname};
    $Data::Dumper::Maxdepth = 1;
    print "DBGCEB:IFTAVOTests:start_tests\n" if ($DBGCEB);
    $self->{parent} = Subtest::ipc();
    print "\nDBGCEB:$ownname has started\n" if ($DBGCEB);

    $self->{subt} =
      subtest( \&test_ops,
               '-bg',
               '-ipc',
               '-runid',
               $self->{tid},
###               "wafl_check_iron",
               "--",
               $self );
    sleep(4);
logresult('INFO', "Exit IFTWICOTests::start_tests Line: ".__LINE__);
    return;
}

#############################################################################################
#############################################################################################



sub test_ops
{
    my $self       = shift;
    my $filer      = $self->{filer};
    my $endtime    = $self->{endtime};
    my $wafl_tests = $self->{wafl_tests};
    my $interval   = $self->{interval};
    my $timeout    = $self->{timeout};
    my $wackall    = $self->{timeout};
    my $fh         = $filer->{fh};
    my $fname      = $filer->{fname};
    my $fullname   = $filer->{fullname};
    my $next_test;
    system("/usr/local/bin/cons_reset $fname");
    logcomment("Connecting to filer $fname ....");
    my $fconsole = connect("#console", -timeout, $timeout);
    logcomment(".... Finished connecting to filer $fname");
  
    # Need to boot first if running on a simulator and doing 'wafl_check'
    if (grep(/check/, @$wafl_tests) && $fh->hosttype() eq 'filer-simulator') {
        logStep("Booting the simulator");
        transit_to($fconsole, 'UP') or die config_result("Unable to boot: $@");
    }

    while (time() < $endtime) {
            my $container_pick;
        
            logcomment("Sleeping $interval ....");
            sleep($interval);
            logcomment(".... Finished sleeping");
            my $new_filer = $filer;
        
            if ($wackall) {
                $container_pick = "-all";
            } else {     
                # get existing container
                $new_filer = IFTFilerSetup->new($fullname);
                $new_filer->get_existing_containers();
        
                my $container = $new_filer->{containers};
                my $volinfo = $container->{volumes};
                my $aginfo = $container->{aggr};
        
                my $rand = rand(2);
                if ($rand) {  # pick up aggregate
                    my $aggr_count = keys %{ $container->{aggr} };
                    my $the_aggr = int(rand($aggr_count))+1;
logcomment("pick up number $the_aggr aggregate from $aggr_count aggregates");
                    my $scount = 1;
                    foreach my $aggr ( keys %{ $container->{aggr}} ) {
                        if ($scount++ == $the_aggr) {
                           if (!aggr_exists($fconsole, $aggr)) {
logcomment("$aggr not exists");
                               $the_aggr +=1;
                               next;
                           }
                           $container_pick = $aggr;
logcomment("$aggr / $container_pick is picked up ===== ");
                           last;
                        }
                    }
        
                }
                else {  # pick up trad volume
                    my $vol_count = keys %{ $container->{volumes} };
        
                    my $the_volume = int(rand($vol_count))+1;
logcomment("pick up number $the_volume volume from $vol_count volume");
                    my $scount = 1;
                    foreach my $volume ( keys %{ $volinfo }) {
                        if ($scount++ == $the_volume) {
        
                            if (!vol_exists($fconsole, $volume)) {
logcomment("$volume not exists");
                                $the_volume += 1;
                                next;
                            }
                            $container_pick = $volume;
logcomment("$volume / $container_pick is picked up ===== ");
                            last;
                        }
                    }
                }
            }
                    
            if (!$container_pick) {
                logcomment("no container was picked up");
                next;
            }
        
logresult('INFO', "container_pick: \"$container_pick\"");
        
            $next_test = wafl_check_or_iron($wafl_tests,
                                              $next_test,
                                              $fconsole,
                                              $container_pick,
                                              $new_filer,
                                              $fname,
                                              $timeout);
     }
}


sub wafl_check_or_iron {
    my $wafl_tests        = shift;
    my $next_test         = shift;
    my $fconsole                = shift;
    my $container_pick    = shift;
    my $filer             = shift;
    my $fname             = shift;
    my $timeout           = shift;

    if (!$next_test) {
        $next_test = $$wafl_tests[0];
    }

logcomment("Enter wafl_check_or_iron  Next Test: \"$next_test\"   WAFL Tests: \"@$wafl_tests\"");

    if ($next_test eq 'WIO') {
        ## @step waflcheck
        logStep("start waflcheck on $fname/$container_pick");
        my $output;
        my $ret;
        eval {
           $ret = wack($fconsole,
                        args=>$container_pick,
                        commit=>0, 
                        timeout=>$timeout,
                        output=>\$output);
        };
    	    
        transit_to($fconsole, 'UP') or die("transit_to: $@");
    
        if ($ret eq WACK_DIRTY) {
                die("Wack found corruption:$output"); 
        }   
    } else {
        ## @step wafliron
        logStep("start wafliron on $fname/$container_pick");
        my $found;
        my $output;
        ($found, $output) = wafliron_start($fconsole,
                                             $container_pick,
                                             wait=>1,
                                             timeout=>$timeout);
    
        if ($found) {
                die("wafliron found corruption:$output");
        }
    }

    if (@$wafl_tests > 1) {
        if ($next_test eq 'WCO') {
            $next_test = 'WIO';
        } else {
            $next_test = 'WCO';
        }
    }
    return($next_test);
}


1;

