#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IFT/NfsHammer.pm#1 $
#
#Copyright (c) 2005 Network Appliance, Inc.
#All rights reserved.
#
##
## @summary - This test runs the NFS 'hammer' test on multiple
##            clients. For use with the Integrated Functional Test Suite
##
## @description
##   simultaneously perform continuous NFS 'hammer' tests
##   on multiple clients.  Each client will run a file subtest and a
##   directory subtest for each mountpoint to perform simulataneous
##   operations against multiple volumes
##
## @usage
##     use FSH::NfsFuncs;
##
## @author dl-johny@netapp.com (FSS WAFL Quality) bundy
## @status publicuse strict;

package NfsHammer;

use Tharn;
use SPARSE::Util::Logger qw(logError logTrc logStep logWarn logDbg);
use Data::Dumper;
use Utils qw(make_conn);
use SayWrap qw(sayEOF);
use strict;


sub new( $$$$;$ ) {
    my $class      = shift;
    my $clientinfo = shift;
    my (%opts)     = @_;
    my $mounts = $opts{mounts};
    my $self   = {};

    $self->{tests} => {};
    $self->{mounts} = {};
    $self->{clientinfo} = $clientinfo;
    $self->{duration} = (defined($opts{duration})) ? $opts{duration} : -1;
    $self->{iterations} = (defined($opts{iterations})) ? $opts{iterations} : -1;
    $self->{hammer_opts} = (defined($opts{hammer_opts})) ? $opts{duration} :
                                                           "-mode mixed -fill";
    bless($self, $class);
    return ($self);
}

sub _get_existing_mounts {
    my $self = shift;
    my $clientinfo = $self->{clientinfo};
    my $mounts = {};
    while ( my($client, $cli_info) = each(%{$clientinfo->{clients}}) ) {
        my $cli_info = $clientinfo->{clients}{$client};
        while ( my ($mnt, $mntinfo) = each(%{$cli_info->{mounts}}) ) {
            next unless ($mntinfo->{type} eq 'nfs');
            next unless ($mntinfo->{mp_type} eq 'vol');
            $mounts->{$mnt} = $mntinfo;
        }
    }
    return ($mounts);
}

sub _get_list {
    my $x = shift;
    my @list = ();
    if (!$x) {
    } elsif (!ref($x)) {
       @list = split(/,/, $x);
    } elsif (ref($x) =~ /HASH/) {
       @list = keys(%$x);
    } elsif (ref($x) =~ /ARRAY/) {
       @list = @$x;
    }
    return (@list);
}

sub _get_matching_mounts_aux {
    my $self = shift;
    my $mount_pool = shift;
    my (%opts) = @_;
    return unless ( exists($opts{mounts}) || exists($opts{filers}) ||
        exists($opts{clients}) || exists($opts{vols}) );
    my @vols = _get_list($opts{vols});
    my @clients = _get_list($opts{clients});
    my @mounts = _get_list($opts{mounts});
    my @filers_tmp = _get_list($opts{filers});
    my @filers = ();
    foreach (@filers_tmp) {
        if (ref($_)) {
            push(@filers, $_->{fname});
        } else {
            push(@filers, $_);
        }
    }
    my $rtn_mounts = {};
    if (@mounts) {
        foreach my $mntinfo (@mounts) {
            next unless ($mntinfo->{type} eq 'nfs');
            next unless ($mntinfo->{mp_type} eq 'vol');
            my $client = $mntinfo->{client};
            my $filer = $mntinfo->{filer};
            my $vol = $mntinfo->{vol};
            my $mount_dir = $mntinfo->{mount_dir};
            my $key = "$client:$filer:$vol:$mount_dir";
            $rtn_mounts->{$key} = $mntinfo;
        }
    } else {
        my $current_mounts;
        if (defined($mount_pool)) {
            $current_mounts = $mount_pool;
        } else {
            $current_mounts = $self->_get_existing_mounts();
        } 
        while ( my ($mnt, $mntinfo) = each(%$current_mounts) ) {
            next unless ($mntinfo->{type} eq 'nfs');
            next unless ($mntinfo->{mp_type} eq 'vol');
            my $client = $mntinfo->{client};
            my $filer = $mntinfo->{filer};
            my $vol = $mntinfo->{vol};
            next unless (!@vols ||     grep(/^$vol$/,     @vols));
            next unless (!@clients ||  grep(/^$client$/,  @clients));
            next unless (!@filers ||   grep(/^$filer$/,   @filers));
            my $mount_dir = $mntinfo->{mount_dir};
            my $key = "$client:$filer:$vol:$mount_dir";
            $rtn_mounts->{$key} = $mntinfo;
        }
    }
    return ($rtn_mounts);
}


sub _get_my_matching_mounts {
    my $self = shift;
    my (%opts) = @_;
    return ( $self->_get_matching_mounts_aux($self->{mounts}, %opts) );
}


sub _get_all_matching_mounts {
    my $self = shift;
    my (%opts) = @_;
    return ( $self->_get_matching_mounts_aux(undef, %opts) );
}


sub start_tests {
    my $self = shift;
    my (%opts) = @_;
    my $matching_mounts;
    if (exists($opts{mounts}) || exists($opts{filers}) ||
        exists($opts{clients}) || exists($opts{vols}) ) {
        $matching_mounts = $self->_get_all_matching_mounts(%opts);
    } else {
        $matching_mounts = $self->_get_existing_mounts($self->{clientinfo});
    }
    my $max_vols = $opts{max_vols};
    my $cntr = 0;
logWarn("Enter start_tests  Matching mounts:\n\n".Dumper(keys(%$matching_mounts)));
    foreach my $mntinfo  (values(%$matching_mounts)) {
        last if ($max_vols && $max_vols > 0 && $cntr >= $max_vols);
        $cntr++;
        next unless ($mntinfo->{type} eq 'nfs');
        next unless ($mntinfo->{mp_type} eq 'vol');
        my $filer = $mntinfo->{filer};
        my $client = $mntinfo->{client};
        my $vol = $mntinfo->{vol};
        my $mount_dir = $mntinfo->{mount_dir};
        my $key = "$client:$filer:$vol:$mount_dir";
        my $old_test = $self->{mounts}{$key};
        $self->_stop_test_for_mount($old_test);
        $self->{mounts}{$key} = $mntinfo;
        $mntinfo->{hammer} = {};
        $mntinfo->{hammer}{self} = $self;
        $mntinfo->{hammer}{key} = $key;
        $mntinfo->{test} = {};
        $self->_start_test_for_mount($mntinfo);
    }
logWarn("Exit start_tests:\n\n".Dumper(keys(%{$self->{mounts}})));
}


sub delete_tests {
    my $self = shift;
    my (%opts) = @_;
    my $matching_mounts;
    if (exists($opts{mounts}) || exists($opts{filers}) ||
        exists($opts{clients}) || exists($opts{vols}) ) {
        $matching_mounts = $self->_get_all_matching_mounts(%opts);
    } else {
        $matching_mounts = $self->_get_existing_mounts($self->{clientinfo});
    }
logWarn("Enter delete_tests  Matching mounts:\n\n".Dumper(keys(%$matching_mounts)));
    foreach my $mntinfo (values(%$matching_mounts)) {
        $self->_stop_test_for_mount($mntinfo);
        my $filer = $mntinfo->{filer};
        my $client = $mntinfo->{client};
        my $vol = $mntinfo->{vol};
        my $mount_dir = $mntinfo->{mount_dir};
        my $key = "$client:$filer:$vol:$mount_dir";
        delete($self->{mounts}{$key});
    }
}


sub _sanity_check_mount {
    my $self = shift;
    my $mount = shift;
    if (!$mount->{hammer} || $mount->{hammer}{self} != $self ||
          !$mount->{hammer}{key} ||
          $self->{mounts}{ $mount->{hammer}{key} } != $mount) {
        return (0);
    }
    return (1);
}


sub _start_test_for_mount {
    my $self = shift;
    my $mntinfo = shift;
    my (%opts) = @_;

    my $client = $mntinfo->{client};
    my $cli_info = $mntinfo->{client_info};
    my $filer = $mntinfo->{filer};
    my $vol = $mntinfo->{vol};
    my $mount_dir = $mntinfo->{mount_dir};

    return unless ($mntinfo->{type} eq 'nfs' && $mntinfo->{mp_type} eq 'vol');

    my $duration = (defined($opts{duration})) ? $opts{duration}
                                              : $self->{duration};

    my $iterations = (defined($opts{iterations})) ? $opts{iterations}
                                              : $self->{iterations};

    my $hammer_opts = (defined($opts{hammer_opts})) ? $opts{hammer_opts}
                                                    : $self->{hammer_opts};

    if ( !$self->_sanity_check_mount($mntinfo) ) {
    }

    $self->_stop_test_for_mount($mntinfo);
    my ($subtest, $runid) = $self->_start_hammer($cli_info,
                                                 $filer,
                                                 $vol,
                                                 $mount_dir,
                                                 $duration,
                                                 $iterations,
                                                 $hammer_opts);

    $mntinfo->{hammer}{test}{subtest} = $subtest;
    $mntinfo->{hammer}{test}{runid} = $runid;
    my $date = `date`;
    chomp($date);
    $mntinfo->{hammer}{test}{start_date} = $date;
    $mntinfo->{hammer}{test}{start_time} = time();
    $mntinfo->{hammer}{test}{duration} = $duration;
    $mntinfo->{hammer}{test}{iterations} = $iterations;
    $mntinfo->{hammer}{test}{hammer_opts} = $hammer_opts;
}


sub stop_tests {
    my $self = shift;
    my (%opts) = @_;
    my $matching_mounts;
    if (exists($opts{mounts}) || exists($opts{filers}) ||
        exists($opts{clients}) || exists($opts{vols}) ) {
        $matching_mounts = $self->_get_all_matching_mounts(%opts);
    } else {
        $matching_mounts = $self->_get_existing_mounts($self->{clientinfo});
    }
logWarn("Enter stop_tests  Matching mounts:\n\n".Dumper(keys(%$matching_mounts)));
    foreach my $mntinfo (values(%$matching_mounts)) {
logWarn("Calling stop_test for runid: $mntinfo->{hammer}{test}{runid}\n\n");
        $self->_stop_test_for_mount($mntinfo);
        my $key = $mntinfo->{hammer}{key};
###        delete($mntinfo->{hammer}{test});
###        delete($self->{mounts}{$key});
    }
}


sub _stop_test_for_mount {
    my $self = shift;
    my $mntinfo = shift;
    $mntinfo->{hammer} = {} unless (defined($mntinfo->{hammer}));
    my $test = $mntinfo->{hammer}{test};
    my $subtest = $test->{subtest};
    my $runid = $test->{runid};
    return unless (defined($subtest));
    logresult('INFO', "Stopping hammer subtest $runid");
    $subtest->kill();
###    delete($mntinfo->{hammer}{test});
}


sub show_subtests {
    my $self = shift;
logWarn("Enter show_subtests");
    print "\n---------------------------------------------\n";
    foreach my $mount (values(%{$self->{mounts}})) {
        my $test = $mount->{hammer}{test};
##        next unless ($test);
        my $client = $mount->{client};
        my $filer = $mount->{filer};
        my $vol = $mount->{vol};
        my $mount_dir = $mount->{mount_dir};
        my $subt = $test->{subtest};
        my $start_date =  $test->{start_date};
        my $start_time = $test->{start_time};
        my $duration = $test->{duration};
        my $duration = $test->{iterations};
        my $hammer_opts = $test->{hammer_opts};
        my $runid = $test->{runid};
        my $is_alive = $subt->isAlive();
        my (@results, $worst, $result_type, $messages);
        @results = $subt->results();
        $worst = Rslt::worst(@results) if (@results);
        $result_type = $worst->type() if ($worst);
###        $messages = join("\n", $worst->messages()) if ($worst);
        $messages = "";
        foreach ($worst->messages()) {
             $messages .= "\n$_\n";
        }
        print "\trunid:\t\t$runid\n";
        print "\tfiler:\t\t$filer\n";
        print "\tvol:\t\t$vol\n";
        print "\tclient:\t\t$client\n";
        print "\tstart date:\t$start_date\n";
        print "\thammer opts:\t$hammer_opts\n";
        print "\n\tis alive:\t$is_alive\n";
        print "\tresult type:\t$result_type\n";
        print "messages:\n$messages\n";
    print "---------------------------------------------\n";
        my ($testinfo) = {
               filer => $filer,
               client => $client,
               vol => $vol,
               runid => $runid,
               running => $is_alive,
               result_type => $result_type,
               duration => $duration,
               start_time => $start_time,
               start_date => $start_date,
               hammer_opts => $hammer_opts,
               messages => $messages
         };
#         logresult('INFO', "show_subtests:\n\n".Dumper($testinfo));
     }
     print "\n";
}


sub _start_hammer {
     my $self = shift;
     my $cli_info = shift;
     my $filer = shift;
     my $vol = shift;
     my $mount_dir = shift;
     my $duration = shift;
     my $iterations = shift;
     my $hammer_opts = shift;
     my $err_msg;
     my $cname = $cli_info->{hname};
     my $client_hosttype = $cli_info->{hosttype};
     my $client_fullname = "$cname;hosttype=$client_hosttype";
     my $timeout = (defined($duration) && $duration > 0) ? ($duration + 30) : -1;

     if ( !$mount_dir ) {
         $err_msg = "Volume $vol on filer $filer is not mounted on client $cname";
         logError($err_msg);
         return ( $err_msg );
     }
     my $runid = "hammer-${filer}-${vol}-${cname}";

###     my $subtest = subtest("NFS/hammer.thpl", -bg,
     my $subtest = subtest(\&exec_hammer_on_client, -bg,
                       -runid, $runid,
                       -onexit,\&subtest_exit_handler,
                       -timeout, $timeout,
#                        "-client", "$client_fullname",
"--",
                       "CMDTIMEOUT=$timeout",
                       "DURATION=$duration",
                       "FILER=$filer",
                       "CLIENT=$client_fullname",
                       "HAMMER_TARGET_DIR=$mount_dir",
                       "FILER_MOUNT_DIR=/vol/$vol",
                       "ITERATIONS=$iterations",
                       "RUNID=$runid",
                       "HAMMER_OPTS=$hammer_opts"
                       . " -filercore $filer");

    return ( $subtest, $runid );
}

sub exec_hammer_on_client {
    my (@args) = @_;
    my %opts = ();
    foreach (@args) {
        my ($key, $val) = split(/=/, $_, 2);
        $opts{$key} = $val;
    }
    my $client = $opts{CLIENT};
    my $cname = $client;
    $cname =~ s/;.*//;
    my $fname = $opts{FILER};
    my $timeout = $opts{CMDTIMEOUT};
    my $duration = $opts{DURATION};
    my $target_dir = $opts{HAMMER_TARGET_DIR};
    $fname =~ s/;.*//;
    my $cc = make_conn($client);
    logError("Error from make_conn for CLIENT \"$client\"") unless ($cc);
    my $cmmd_args = $target_dir;
    $cmmd_args .= " -iterations $opts{ITERATIONS}" if (exists($opts{ITERATIONS}));
    $cmmd_args .= " -runtime $opts{DURATION}" if (exists($opts{DURATION}));
    $cmmd_args .= " $opts{HAMMER_OPTS}" if (exists($opts{HAMMER_OPTS}));
    logresult('INFO', "exec_hammer_on_client  FILER: $fname  CLIENT: $client  PATH: $target_dir  RUNID: $opts{RUNID}  COMMAND: \"hammer $cmmd_args\"");
    my $output;
    if ($timeout) {
        $output = sayEOF($cc, "/usr/local/test/bin/hammer $cmmd_args", timeout=>$timeout);
    } else {
        $output = sayEOF($cc, "/usr/local/test/bin/hammer $cmmd_args");
    }
    logWarn("exec_hammer_on_client  Returned: $output");
}


sub _check_subtest {
    my $self = shift;
    my $mount = shift;
    my $testinfo = $mount->{hammer};
    my $subtest = $testinfo->{subtest};
    return unless ($subtest);
    my @results = $subtest->results();
    return unless (@results);
    my $worst = Rslt::worst(@results);
    return unless ($worst);
    my $runid = $worst->runid();
    my @messages = $worst->messages();
    my $sev = $worst->severity();
    if ($sev >= 4) {
         my $error_message = "Subtest Error in $runid \n" . join("\n", @messages);
         $subtest->kill();
         return ( $error_message );
    } elsif ($sev == 3) {
         logWarn("Warning from subtest $runid\n" . join("\n", @messages));
    } else {
         logcomment("Message from subtest $runid\n" . join("\n", @messages));
    }
    return;
}


sub check_for_errors {
    my $self = shift;
    my $error;
    foreach my $mntinfo (values(%{$self->{mounts}})) {
        $error = $self->_check_subtest($mntinfo);
        return ($error) if ($error);
    }
    return;
}


sub subtest_exit_handler
{
        my ($sub) = @_;
        check_for_errors(undef, $sub);
}

1;
