# $Id: //depot/prod/test/Rsierranevada/lib/IFT/Reboot_Ops.pm#1 $

## @summary The set of functional Reboot tests
##
##
## @author      bundy
## @cf          true
## @status private

package Reboot_Ops;

use strict;

use vars ('@ISA','@EXPORT');
require Exporter;
@ISA = ('Exporter');
@EXPORT = ('setup_reboot');


use Tharn;
use IFT::CFO_Utils;
use Data::Dumper;
use SayWrap qw(sayEOF);

our $NA = "NA";
our $FAIL = "FAIL";
our $ERROR = "ERROR";
our $FATAL = "FATAL";
our $CONFIG = "CONFIG";
our $PASS = "PASS";
our $ACTION = "ACTION";
our $INFO = "INFO";
our $WAIT    = "wait";
our $NOWAIT    = "nowait";
our $ACK     = "ACK";
our $NCK     = "NCK";
our $QUIT    = "quit";
our $PAUSE   = "pause ";
our $RESUME  = "resume";


sub setup_reboot {
    my $class = shift;
    my $ts = shift;
    my $testid = shift;
    my $self = ();

    my @filer_keys = keys(%{$ts->{filers}});
    $self->{testid} = $testid;
    $self->{msgno} = 1;
    $self->{testno} = 1;
    $self->{interval} = $ts->{reboot}{interval};
    $self->{timeout} = $ts->{reboot}{timeout};
    $self->{logfile} = $ts->{logfile} if (exists $ts->{logfile});
    $self->{duration} = $ts->{duration};
logresult('INFO', "setup_reboot:\n\n".Dumper($self));
    my $mykey = "filerA";
    while (my ($key, $filer) = each(%{$ts->{filers}})) {
        $self->{$mykey}{host} = $filer->{fh};
        $self->{$mykey}{name} = $filer->{fname};
        $self->{$mykey}{conn} = $filer->{fc};
        $mykey = "filerB";
    }
    $self->{ts} = $ts;
    $self->{filers} = $ts->{filers};
    bless $self, $class;
   my @filers = values( %{$self->{filers}} );
   if ( (@filers > 1) && !$self->cf_enable_disable("disable") ) {
       die("setup_reboot: Can't disable CFO on filers");
   }
   $self->{subt} = subtest( \&_start_tests, -bg, -ipc, -runid,
                "reboot test $testid",
#                '-onexit',\&_subtest_exit_handler,
                "--", $self );

    return ($self);
}


sub _subtest_exit_handler {
    my $subtest = shift;
    my @results = $subtest->rslts();

    foreach my $result (@results) {
        if ( ref($result) =~ /Fatal/ ) {
            my $message = sprintf($result);
            logresult( "FAIL", $message, abort => 1 );
        } else {
            logresult($result);
        }
    }
}


sub _start_tests  {
    my $self = shift;
    $self->{parent} = Subtest::ipc();
    $self->{starttime} = $self->{ts}{starttime} || time();
    my $testid = $self->{testid};
    print "DBGCEB:REBOOT:_start_tests: testid - $testid\n";

    logcomment("$self->{testid}: snoozing for $self->{interval} seconds");
    sleep($self->{interval});

    my $tcount = 1;
    my @filers = values( %{$self->{filers}} );
    while (!$self->_check_duration()) {
      $self->{testid} = "$testid:$tcount";
      $self->reboot_filers(@filers);
      logcomment("$self->{testid}: snoozing for $self->{interval} seconds");
      sleep($self->{interval});
    }
}

sub reboot_filers {
    my $self = shift;
    my (@filers) = @_;

   if ( (@filers > 1) && !$self->cf_enable_disable("disable") ) {
       die("reboot_filers: Can't disable CFO on filers");
   }

    # reboot each filer and verify nfs data continues
    my $timeout = $self->{timeout};
    foreach my $filer ( values %{ $self->{filers} } ) {
        logcomment("rebooting $filer->{fname}");

        die fail_result("unable to reboot $filer: $@ ")
          unless ( reboot_filer($filer, $timeout) );
        sleep(10);
    }
    return;
}

sub reboot_filer {
    my $filer  = shift;
    my $timeout = shift;
    my $fname  = $filer->{fname};
    logcomment("Rebooting filer: $fname");
    my $cmd    = 'reboot -t 0';
    my $output = say( $filer->{fc}, $cmd, -blind );
    my $saved_timeout = $filer->{fc}->timeout();
    $filer->{fc}->timeout(30);
    my $reboot_time = 0;
    my $start_time = time();
    my $text = undef;
    my $filer_up = 0;
    # snooze a bit, then check if we can connect to the filer.
    logcomment("Wait for filer back up: $fname");
    while ( $reboot_time <= $timeout && !$filer_up) {
        sleep(120);
        $filer->{fh} = host( $filer->{fullname} );
        $filer->{fc} = connect( $filer->{conn} );
        eval {$text = sayEOF($filer->{fc},"hostname");};
        print "DBGCEB:reboot_filer: exception = $@\n" if $@;
        $filer_up = ($text =~ $filer->{fname});
        $reboot_time = time() - $start_time;
    }
    $filer->{fc}->timeout($saved_timeout);
    if ($filer_up) {
        logcomment("$filer->{fname} is back up time= $reboot_time");
    } else {
        logcomment("$filer->{fname} is not back up -- timeout exceeded ($timeout)");
    }
    return 1;
}


sub _check_duration {
    my $self = shift;
    my $current = time();
    my $diff    = $current - $self->{starttime};
      if ( $diff >= $self->{duration} ) {
           logcomment("Test time is up!");
           return 1;
      }
    return;
}


sub cf_enable_disable {
    my $self = shift;
    my $op = shift;
    my @filers = values( %{$self->{filers}} );
    my $cfstat = "cf status";
    foreach my $filer (@filers) {
    my $text = sayEOF( $filer->{fc}, $cfstat );
#    return 0 if ($text =~ /Takeover/);
    if ($op eq "disable") {
        return 1 if ( $text =~ /Cluster disabled/ );
        my $cmd = "cf disable";
        $text = sayEOF( $filer->{fc}, $cmd );
        $text = sayEOF( $filer->{fc}, $cfstat );
        if ( $text !~ /Cluster disabled/ ) {
            die config_result( "Unable to disable clustering on "
                  . "$filer->{fname}: $text" );
        }
    } else {
        return 1 if ( $text =~ /Cluster enabled/ );
        my $cmd = "cf enable";
        $text = sayEOF( $filer->{fc}, $cmd );
        $text = sayEOF( $filer->{fc}, $cfstat );
        if ( $text !~ /Cluster enabled/ ) {
            die config_result( "Unable to enable clustering on "
                  . "$filer->{fname}: $text" );
        }
     }
     }
     return 1
}


sub _get_testid {
    my $self = shift;
    my $tid = "$self->{testid}:$self->{testno}";
    $self->{testid}++;
    return $tid;
}

sub _get_msgno {
    my $self = shift;
    return $self->{msgno}++;
} #end _get_msgno

sub _send_end_msg {
    my $self   = shift;
    my $tid    = shift;
    my $result = shift;
    my $msg = shift;

    my $mode   = undef;
    my $tinfo = $self->{tests}{$tid};
    if (!$tinfo) {
        logcomment("_send_end_msg: test $tid does not exist");
        return;
    }
    $tinfo->{result} = $result if ($result);
    $tinfo->{text} = $msg if ($msg);
    my $header = _create_msg_header( $self, $tinfo );
    $tinfo->{duration} = (time() - $tinfo->{starttime});

    print
"DBGCEB:_send_end_msg: got a RESULT message to send from $tinfo->{testname}\n";
    my $msg =
      sprintf "%s;ENDTEST;RESULT=%s;RESULTSTR=%s;DURATION=%s;EMSG",
      $header, $tinfo->{result}, $tinfo->{text},$tinfo->{duration};
    print "DBGCEB:$self->{testid}:_send_end_msg: sending msg to parent\n$msg\n";
#   $self->_logmessage($msg);
    my $resp = _send_parent( $self, $msg, $mode );
    delete $self->{tests}{$tid};
    return $resp;
} #end _send_end_msg

sub _send_start_msg {
    my $self   = shift;
    my $testname = shift;
    my $cont = shift;
    my $tinfo = {};
    $tinfo->{testname} = $testname;
    $tinfo->{filer} = $self->{filer}{fname};
    $tinfo->{tid} = $self->_get_testid();
    $tinfo->{starttime} = time();
    $tinfo->{cont} = $cont;
    my $header = _create_msg_header( $self, $tinfo );
    $self->{omsginfo} = $tinfo;
    my $starttime = time();
    my $msg = sprintf "%s;STARTTIME=%s;CONTAINER=%s;EMSG", $header,
      $tinfo->{starttime}, $tinfo->{cont}  ;
#    $self->_logmessage($msg);
    my $resp = _send_parent( $self, $msg, undef );
    $self->{tests}{$tinfo->{tid}} = $tinfo;
    return ($tinfo->{tid},$resp);
} #end _send_start_msg

sub _create_msg_header {
    my $self  = shift;
    my $tinfo = shift;
    my $tid   = undef;
    $tinfo->{msgno} = $self->_get_msgno();
    my $msg = sprintf "ID=%s;MSG=%d;TESTNAME=%s;FILER=%s", $tinfo->{tid},
                      $tinfo->{msgno}, $tinfo->{testname},$tinfo->{filer};
    return $msg;
}

sub _send_noend_msg {
    my $self   = shift;
    my $tid  = shift;
    my $type = shift;
    my $text = shift;

    my $tinfo = $self->{tests}{$tid};
    if (!$tinfo) {
        logcomment("_send_end_msg: test $tid does not exist");
        return;
    }
    $tinfo->{type} = $type if $type;
    $tinfo->{text} = $text if $text;
    my $header = _create_msg_header( $self, $tinfo );
    my $msg    = undef;
    my $mode   = undef;
    $self->{omsginfo} = $tinfo;

    if ( $tinfo->{type} eq $ACTION ) {
        $msg = sprintf "%s;ACTION=%s;ITYPE=%s;ITEM=%s;EMSG", $header,
          $tinfo->{action}, $tinfo->{itype}, $tinfo->{item};
        $mode = $WAIT;
    } else {
        $msg = sprintf "%s;%s;TEXT=%s;EMSG" . $header, $tinfo->{type},
          $tinfo->{text};
    }
    print "DBGCEB:$self->{testid}:_send_noend_msg: sending msg to parent\n$msg\n";
#    $self->_logmessage($msg);
    my $resp = _send_parent( $self, $msg, $mode );
    $tinfo->{text} = "";
    $tinfo->{type} = "";
    return $resp;
} #end _send_noend_msg





sub _logmessage {
    my $self = shift;
    my $text = shift;
    my $status = lc shift;
    my $result = undef;
    my $error = undef;
    logcomment($text);

    if (exists $self->{logfile}) {
        open LOGFILE,"+>$self->{logfile}"
             or $error = "cannot open $self->{logfile}:$!";
        if ($error) {
            _logmessage($error);
            return;
        }
        print LOGFILE $text;
    }
    close LOGFILE;
    return if (!$status);
    if ($status eq "pass") {
        pass_result($text);
    } elsif ($status eq "fail") {
        $result = fail_result($text);
    } elsif ($status eq "warn") {
        $result = warn_result($text);
    } elsif ($status eq "config") {
        $result = config_result($text);
    } elsif ($status eq "info") {
        $result = logresult("Info",$text);
    } elsif ($status eq "na") {
        $result = na_result($text);
    } elsif ($status eq "fatal") {
        $result = logresult("FATAL",$text);
    }
    return $result;
}

sub read_parent {
    my $self                     = shift;
    my $exp                      = shift;
    my $mode                     = $self->{comm}{mode};
    my $omsg                     = $self->{comm}{outmsg};
    my $omsg_info                = $self->{comm}{omsginfo};
    my $sock                     = Subtest::ipc();
    my $ack                      = undef;
    my $resp                     = undef;
    my $tid = undef;
    my $msgno = undef;
    if ( $exp eq $ACK ) {
        $ack   = 1;
        $tid   = $omsg_info->{tid};
        $msgno = $omsg_info->{msgno};
    }
    my $retries = 5;
    while (1) {
        while ( $sock->can_read() ) {
            my $msg = $sock->read_message();
            chomp $msg;
            if ($ack) {
                $resp = $1
                  if ( $msg =~ /^$tid\;MSG=$msgno\;.*;ACK=(\w*)\;EMSG$/ );
            }
            if ( $msg =~ /$PAUSE/ ) {
                $self->{teststatus} = $PAUSE;
            } elsif ( $msg =~ /$RESUME/ ) {
                $self->{teststatus} = $RESUME;
            } elsif ( $msg =~ /$QUIT/ ) {
                exit(0);
#                $self->{teststatus} = $QUIT;
            } elsif ($msg eq "LIVETESTS") {
                my @livetests = ();
                foreach my $key (keys %{$self->{tests}}) {
                   push @livetests,$self->{tests}{$key}{testname};
                }
                return @livetests;
            } else {
                _logcomment(
                    "$self->{hname} received unknown msg from parent: $msg");
            }
        }
        $retries--;
 #       last if ( $self->{teststatus} eq $QUIT );
        last if ((!$retries) && ( $self->{teststatus} ne $PAUSE ));
        next if (($mode eq $WAIT) && (!$resp));
        last if ( $self->{teststatus} ne $PAUSE );
    }
    return $resp;
} #end read_parent

sub _send_parent {
    my $self = shift;
    my $msg  = shift;
    my $mode = shift;
    $self->{comm}{mode}   = $mode;
    $self->{comm}{outmsg} = $msg;
    my $sock = Subtest::ipc();
    my $ack  = undef;

    #add subfunction id if subfunction is zero.
    _logmessage();
    eval { $sock->write_message(); };
    if ($@) {
        logWarn("$self->{testid}:unable to write message to parent: $@");
        return 1;
    }
    my $resp = read_parent( $self, $ACK );
    logcomment("$self->{testid}:did not receive parent ack") if ($resp ne $ACK);
    logcomment($msg);
    return;
} #end _send_parent

1;
