#$Id: //depot/prod/test/Rsierranevada/lib/IFT/VolOps.pm#1 $
#
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.

## @summary        Library that defines useful aggr and vol operations
## @description
##             Filer aggregates and volume operations
##
## @author dl-johny

package VolOps;

use strict;
use Tharn;

use constant QUIT      => "quit";
use constant SUSPEND   => "suspend";
use constant RESUME    => "resume";
use constant MAX_AGGRS => 100;
use constant MAX_VVOLS => 500;
use constant MAX_TVOLS => 100;

use constant AGGR_NUM         => 4;
use constant VVOL_NUM         => 10;
use constant TVOL_NUM         => 4;
use constant VOLCOPY_TIMEOUT  => 2400;
use constant AGGRCOPY_TIMEOUT => 3600;
use Utils::Units qw( UNIT_KB );

use Aggregate qw(aggr_create
  aggr_purge
  aggr_exists
  aggr_add
  aggr_status
  aggr_spare_status
  aggr_offline
  aggr_online
  aggr_restrict);
use Volume qw(vol_create
  vol_create_flex
  vol_purge
  vol_add
  vol_size
  vol_online
  vol_offline
  vol_exists
  vol_container
  vol_size
  vol_disk_free
  vol_status);
use Volume qw(:df);
use QtreeCmds qw(qtreePresent
  qtreeList);

use SayWrap qw(sayEOF);
use ExportFS qw(:nfs);
use Data::Dumper;
use SPARSE::Util::Logger qw(logStep logWarn logDbg);

#use IFT::IFTSetup;
use VolCopy qw(volcopy_status);
use Admin::License qw(license is_licensed);
use Volume::Clone qw(vol_clone_create
  vol_clone_split_start
  vol_clone_split_status);
use Utils::Units qw(convert_to_bytes);
use VolCmds qw(volRestrict);
use Utils qw(make_conn);
use filer qw(filer_file_make filer_file_destroy filer_file_exists);
use SnapCmds qw(snapshot_create snapshot_delete
  snaprestore_volume snaprestore_file
  snapshot_list);
use CLI::FileIO qw(rdfile wrfile);
our $AGGR    = "aggr";
our $FLEX    = "flex";
our $TRAD    = "trad";
our $LUNS    = "LUNs";
our $QTREES  = "qtrees";
our $ONLINE  = "online";
our $OFFLINE = "offline";
our $WAIT    = "wait";
our $NOWAIT    = "nowait";
our $DELETE  = "delete";
our $ADD     = "add";
our $ENABLE  = "enable";
our $DISABLE = "disable";
our $ACK     = "ACK";
our $NCK     = "NCK";
our $QUIT    = "quit";
our $PAUSE   = "pause ";
our $RESUME  = "resume";
our $CANDESTROY = "candestroy";
our $ACTION  = "action";
our $NA = "NA";
our $FAIL = "FAIL";
our $ERROR = "ERROR";
our $FATAL = "FATAL";
our $CONFIG = "CONFIG";
our $PASS = "PASS";
our $ACTION = "ACTION";
our $INFO = "INFO";
our $MIN_FLEXVOLS = 100;
our $MIN_AGGRS = 5;
our $MIN_TRADVOLS = 5;
## VolOps->new($filer)
## $filer->{vvol_prefix}, {tvol_prefix}, {aggr_prefix}
##         {vvol_size}, {tvol_size}, {aggr_size}
##         {tvol_raid}, {aggr_raid}, {fc}, {fullname}, {fname}
## new() will get back the $obj-> $filer and $subt
## use $obj->{subt}->ipc()->write_message(SUSPEND) to have it suspend
## use $obj->{subt}->ipc()->write_message(RESUME) to have it continue
## use $obj->{subt}->ipc()->write_message(QUIT) to have it stop

# for testing, fill in filer with some pre-created volumes, aggrs, etc

our($self, $filer);

sub new {
    my $class         = shift;
    my $the_filer     = shift;
    my $testid        = shift;
    my $snapshot_flag = shift;
    my $endtime       = shift;
    my $qtree_flag    = shift;
    $self          = {};

    # get existing aggrs and volumes with IFTSetup
    # pass in prefix of aggr, vvol, tvol and size, raidtype
    #    my $f = setup($filer);
    $self->{filer}         = $the_filer;
    $self->{endtime}       = $endtime;
    $self->{testid}        = $testid;
    $self->{testno}        = 1;
    $self->{msgno}         = 1;
    $self->{snapshot_flag} = $snapshot_flag;
    $self->{qtree_flag}    = $qtree_flag;
    $self->{fname}         = $the_filer->{fname};
    bless $self, $class;
    my $subt =
      subtest( \&aggr_vol_ops, '-bg', '-ipc', '-runid', "aggr_vol_ops", "--",
        $self );
    $self->{subt} = $subt;
    return $self;
}

## main loop
# $filer will be updated during each run, it is consistent with
# the filer as long as there is no other aggr,vol operations
sub aggr_vol_ops {
    my $self    = shift;
    my $ownname = "aggr_vol_ops";
    $filer = $self->{filer};
    my ( @op_funcs, @qtree_funcs, @snapshot_funcs );
    $self->{id}    = 0;
    $self->{subid} = 1;
    $self->{msgno} = 1;

    @op_funcs = (
        \&_vvol_create, \&_aggr_create,  \&_aggr_create,
        \&_vvol_create,  \&_tvol_create, \&_vol_destroy,
        \&_vol_copy,    \&_aggr_copy, \&_aggr_destroy,
        \&_vol_offline, \&_aggr_online,
        \&_aggr_offline, \&_vol_resize,  \&_tvol_add,
        \&_aggr_add,     \&_vvol_clone,  \&_vvol_clone_split
    );

    if ( $self->{snapshot_flag} ) {
        @snapshot_funcs = (
            \&_vol_snapshot_create,  \&_vol_snapshot_restore,
            \&_vol_snapshot_sfsr,    \&_vol_snapshot_delete,
            \&_aggr_snapshot_create, \&_aggr_snapshot_restore,
            \&_aggr_snapshot_delete
        );
        @op_funcs = ( @op_funcs, @snapshot_funcs );
    }
    if ( $self->{qtree_flag} ) {
        @qtree_funcs = ( \&_qtree_create, \&_qtree_destroy );
        @op_funcs = ( @op_funcs, @qtree_funcs );
    }
    if ( $self->{snapshot_flag} && $self->{qtree_flag} ) {
        @op_funcs = ( @op_funcs, @qtree_funcs, @snapshot_funcs );
    }

###############################################################################
    # @op_funcs = ( \&_qtree_create, \&_qtree_destroy );
###############################################################################

    my $msg;
    my $endtime = ( $self->{endtime} ) ? $self->{endtime} : -1;
    print "DBGCEB: $filer->{fname}:number of trad vols = "
      . "$filer->{containers}{count}{$TRAD}\n";
    print "DBGCEB: $filer->{fname}:number of flex vols = "
      . "$filer->{containers}{count}{$FLEX}\n";
    print "DBGCEB: $filer->{fname}:number of aggrs     = "
      . "$filer->{containers}{count}{$AGGR}\n";
    while (1) {
        last if ( ( $endtime != -1 ) && ( time() > $endtime ) );
        my $op = $op_funcs[ rand(@op_funcs) ];
#        my $op = $op_funcs[0];
        my $msg = &$op();
#        return if ($self->{teststatus} eq $QUIT);
        print "*\n*\n* DBGCEB:_aggr_vol_ops: snoozing for 600 seconds ***\n*\n";
        ##x sleep(600);
        _check_volumes($filer);
        sleep(300);
    }
}


sub _check_volumes {
    my $filer = shift;
    my $vols = $filer->{containers}{volume};

    foreach my $v (keys %{$vols}) {
        my $vol = $vols->{$v};
        next if ($vol->{type} ne $FLEX);
        ##x vol_disk_free($conn, %options);
        my ($total, $used, $avail, $capacity) = vol_disk_free($filer->{fc},
                path => $v, space_type => DF_SPACE, report_type => UNIT_KB);
        ##x printf("filer=%s vol=$v total=$total used=$used avail=$avail ".
        ##x "capacity=$capacity\n",$filer->{fname});
        if (!$total) {
            print "Unable to get space info on $filer->{fname}:$v $@\n";
            next;
        }
        if($avail < 1) {
           my $size_kb_old = vol_size($filer->{fc}, $v);
           my $size_kb_new = vol_size($filer->{fc}, $v, '+50M');
           logcomment("Filer=$filer->{fname} Vol[$v] increased from ".
                       "$size_kb_old KB to $size_kb_new KB.\n");
           ##x printf("Filer=$filer->{fname} Vol[$v] increased from ".
           ##x        "$size_kb_old KB to $size_kb_new KB.\n");
        }
    }
}


## sub operations

sub _aggr_create {
    my %opts = @_;
    my $ownname = "_aggr_create";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $fc         = $filer->{fc};
    my $aginfo     = $filer->{containers}{$AGGR};
    my $aggr_count = $filer->{containers}{count}{$AGGR};
    return($self->_send_end_msg($tid,$NA,"max aggrs exist"))
       if ($aggr_count == MAX_AGGRS);
    my $count = $opts{count} ? $opts{count} : 1;
    my $acount = undef;
    if (($aggr_count + $count) > MAX_AGGRS) {
         $acount = ($aggr_count + $count) - MAX_AGGRS;
         return($self->_send_end_msg($tid,$NA,"max aggrs exists"));
    } else {
         $acount = $count;
    }



    logcomment("$ownname:$filer->{fname}:START");

    my $prefix  = $opts{prefix} ? $opts{prefix}
                  : $filer->{cfginfo}{$AGGR}{prefix};
    $prefix = "testag" if (!$prefix);
    my $raid = $opts{raid} ? $opts{raid} : $filer->{cfginfo}{$AGGR}{raid};
    $raid = "raid4" if !$raid;
    my $size = $opts{size} ? $opts{size}
                           : $filer->{cfginfo}{$AGGR}{size};
    if ((!$size) && ($raid eq "raid4")) {
         $size = 2;
    } else {
         $size = 3;
    }
    my $error = undef;

    logcomment("$ownname:$filer->{fname}:filer has $aggr_count aggregates");
    ## calculate how many aggregates will be created
    logcomment( "$ownname:$filer->{fname}:Creating $acount Aggregates, "
          . "filer already has $aggr_count aggrs" );


    ## get spare disks number
    my $spare_disks = aggr_spare_status($fc);
    my $spare_num   = 0;
    foreach my $pool ( keys %$spare_disks ) {
        my $num = @{ $spare_disks->{$pool} };
        logcomment("$ownname:$filer->{fname}:spare disk at $pool: $num");
        if ( $num > $spare_num ) {
            $spare_num = $num;
        }
    }
    logcomment("$ownname:$filer->{fname}:spare disks: $spare_num");

        return($self->_send_end_msg($tid,$NA,"Not enough spares to create ".
               "$acount aggrs" )) if ( $spare_num < $size );

    if ( ( $size * $acount ) > $spare_num ) {
        $acount = int( $spare_num / $size );
        logcomment( "$ownname:$filer->{fname}:number of aggregates to be "
              . "created: $acount" );
    }

    my $cntr = 0;
    my @new_aggrs;
    for ( 1 .. $acount ) {
        my $name = $prefix . ( time() );
        if ( !aggr_create( $filer->{fc}, $name, $size, raidtype => $raid ) ) {
           return($self->_send_end_msg($tid,$FAIL,
                  "cannot create aggr $name:$@" ));
        }
        $cntr++;
        logcomment(
            "$ownname:$filer->{fname}:created aggr $name: $cntr " . "aggrs "
              . "created" );
        ## after one aggr created, add in the structure
        $filer->{containers}{$AGGR}{$name}{size} = $size;
        $filer->{containers}{$AGGR}{$name}{raid} = $raid;
        $filer->{containers}{count}{$AGGR} += 1;
        $filer->{containers}{$AGGR}{$name}{tstatus} = $CANDESTROY;
        push @new_aggrs, $name;
    }

    return (@new_aggrs,$self->_send_end_msg($tid)) if ((defined %opts)
             && (@new_aggrs != ()));
    return($self->_send_end_msg($tid));

} #end _aggr_create

# temporarily create guarantee==none volumes
sub _vvol_create {
    my %opts    = @_;
    my $ownname = "_vvol_create";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $tcount = $filer->{containers}{count}{$FLEX};
    return($self->_send_end_msg($tid,$NA,"max flex vols exist"))
        if ($tcount == MAX_VVOLS);

    my $avail = MAX_VVOLS - $tcount;
    my $vcount = (exists $opts{count}) ? $opts{count} :  int( rand(10) ) + 1;
    $vcount = $avail if ($vcount > $avail);

    my $guarantee = (exists $opts{guarantee}) ? $opts{guarantee}
                                              : $filer->{guarantee};
    $guarantee = "none" if (!$guarantee);

    my $prefix  = (exists $opts{prefix}) ? $opts{prefix}
                  : $filer->{cfginfo}{$FLEX}{prefix};
    $prefix = "testflex" if (!defined $prefix);
    my $size = (exists $opts{size}) ? $opts{size}
                           : $filer->{cfginfo}{$FLEX}{size};
    $size = "100m" if (!$size);
    my $error = undef;
    logcomment("$ownname:$filer->{fname}: calling _get_aggr")
      if ( !exists $opts{aggr} );
    my $the_aggr_name = (exists $opts{aggr}) ? $opts{aggr}
                        : _get_aggr( status => $ONLINE );

    ## pick up the aggregate which the volume will be in
    return($self->_send_end_msg($tid,$NA,"didn't find the aggregate"))
         if ( !$the_aggr_name );
    logcomment("$ownname:$filer->{fname}:$the_aggr_name is the aggr");


    # maximum VVOL_NUM vvols can be created at one time

    logcomment( "$ownname:$filer->{fname}:Creating $vcount flex vols at "
          . "aggr $the_aggr_name" );

    my $cntr = 0;
    my @new_vvols;
    for ( 1 .. $vcount ) {
        my $name = (exists $opts{volname}) ? $opts{volname} : $prefix . ( time() ) ;
        vol_create_flex( $filer->{fc}, $name, $the_aggr_name, $size,
            guarantee => $guarantee )
          or return($self->_send_end_msg($tid,$FAIL,"cannot create $name :$@"));
        my $vol_path = "/vol/$name";
        export_path( $filer->{fc}, "$vol_path" )
          or return($self->_send_end_msg($tid,$FAIL,
                             "Cannot export $vol_path: $@"));
          $filer->{fname}, $vol_path;
        $cntr++;
        logcomment( "$ownname:$filer->{fname}:created flex vol $name: $cntr "
              . "volumes "
              . "created" );

        ## add the vvol in volumes structure
        $filer->{containers}{volume}{$name}{size}      = $size;
        $filer->{containers}{volume}{$name}{$AGGR}     = $the_aggr_name;
        $filer->{containers}{volume}{$name}{guarantee} = "none";
        $filer->{containers}{volume}{$name}{type}      = "flex";
        $filer->{containers}{count}{$FLEX} += 1;
        $filer->{containers}{volume}{$name}{tstatus} = $CANDESTROY;
        push @new_vvols, $name;

    }

    return (@new_vvols, $self->_send_end_msg($tid)) if ((defined %opts) &&
            (@new_vvols != ()));
    return($self->_send_end_msg($tid));

} #end _vvol_create


sub _tvol_create {

    my $ownname = "_tvol_create";
    my $caller = shift;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo   = $filer->{containers}{volume};
    my $fc      = $filer->{fc};
    my $volinfo = $filer->{containers}{volume};
    my $prefix  =
      $filer->{cfginfo}{$TRAD}{prefix}
      ? $filer->{cfginfo}{$TRAD}{prefix}
      : 'tradvol';

    # count existing tvols
    my $volume_num = 0;
    $volume_num = $filer->{containers}{count}{$TRAD};
    $volume_num = $filer->{containers}{count}{$AGGR} + $volume_num;
    logcomment( "$ownname:$filer->{fname}:filer has $volume_num "
          . "traditional volumes and aggrs" );

    return($self->_send_end_msg($tid,$FAIL,"max trad vols exist"))
           if ( $volume_num > MAX_TVOLS );

    # maximum TVOL_NUM trad volumes created at one time
    my $count = int( rand(TVOL_NUM) ) + 1;
    logcomment("$ownname:$filer->{fname}:Creating $count tvol");
    my $raid =
      $filer->{cfginfo}{$TRAD}{raid}
      ? $filer->{cfginfo}{$AGGR}{raid}
      : 'raid_dp';
    my $size =
      $filer->{cfginfo}{$TRAD}{size} ? $filer->{cfginfo}{$TRAD}{size} : 3;

    # get spare disks number
    my $spare_disks = aggr_spare_status($fc);
    my $spare_num   = 0;
    foreach my $pool ( keys %$spare_disks ) {
        my $num = @{ $spare_disks->{$pool} };
        logcomment("$ownname:$filer->{fname}:spare disk at $pool: $num");
        if ( $num > $spare_num ) {
            $spare_num = $num;
        }
    }
    logcomment("$ownname:$filer->{fname}:spare disks: $spare_num");

    return($self->_send_end_msg($tid,$NA,
          "cannot create tvols not enough spares"))
          if ( $spare_num < $size );

    if ( ( $size * $count ) > $spare_num ) {
        $count = int( $spare_num / $size );
        logcomment( "$ownname:$filer->{fname}:number of tvols to be "
              . "created: $count" );
    }

    my $cntr = 0;
    my @new_tvols;
    my $error = undef;
    for ( 1 .. $count ) {
      my $name = $prefix . ( time() );
      vol_create( $filer->{fc}, $name, $size, raidtype => $raid )
        or return($self->_send_end_msg($tid,$FAIL,"cannot create $name: $@"));
      my $vol_path = "/vol/$name";
      export_path( $filer->{fc}, "$vol_path" )
          or return($self->_send_end_msg($tid,$FAIL,
                    "Cannot export $vol_path:$@"));
      $cntr++;
      logcomment( "$ownname:$filer->{fname}:created tvol $name: $cntr tvols"
              . "created" );

      ## add the vvol in volumes structure
      $filer->{containers}{volume}{$name}{size} = $size;
      $filer->{containers}{volume}{$name}{type} = "trad";
      $filer->{containers}{volume}{$name}{raid} = $raid;
      $filer->{containers}{count}{$TRAD} += 1;
      $filer->{containers}{volume}{$name}{tstatus} = $CANDESTROY;
      push @new_tvols, $name;

    }
    return(@new_tvols, $self->_send_end_msg($tid))
            if ($caller && (@new_tvols != ()));
    return($self->_send_end_msg($tid));

} #end _tvol_create

sub _aggr_destroy {

    my $ownname = "_aggr_destroy";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo   = $filer->{containers}{volume};
    my $vol_count = keys %{$volinfo};
    my $aginfo  = $filer->{containers}{$AGGR};
    my $volinfo = $filer->{containers}{volume};
    return($self->_send_end_msg($tid,$NA,
           "too few flex vols to destroy an aggr"))
           if ( $filer->{containers}{count}{$FLEX} <= $MIN_FLEXVOLS );
    return($self->_send_end_msg($tid,$NA,"too few aggr to destroy an aggr"))
           if ( $filer->{containers}{count}{$AGGR} <= $MIN_AGGRS );

    ## pick up the aggregate which will be destroyed
    logcomment("$ownname:$filer->{fname}: calling _get_aggr");
    my $the_aggr_name = _get_aggr( tstatus => $CANDESTROY,  status => $ONLINE,
                                   cloned => 1 );
    return($self->_send_end_msg($tid,$NA,"did not select aggr"))
        if ( !$the_aggr_name );
    logcomment("the aggr $the_aggr_name will be destroyed");

    return($self->_send_end_msg($tid,$FAIL,
           "Cannot purge $the_aggr_name:$@"))
           if ( !aggr_purge( $filer->{fc}, $the_aggr_name ) );

    ## delete the aggr and volumes of the aggr from the structure
    foreach my $volume ( keys %{$volinfo} ) {
        if ( $volinfo->{$volume}{$AGGR} eq $the_aggr_name ) {
            delete $volinfo->{$volume};
            $filer->{containers}{count}{$FLEX} -= 1;
            unexport_path( $filer->{fc}, "/vol/$volume" );
        }
    }
    delete $aginfo->{$the_aggr_name};
    $filer->{containers}{count}{$AGGR} -= 1;

    return($self->_send_end_msg($tid));

} #end _aggr_destroy

sub _vol_destroy {
    my $ownname = "_vol_destroy";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo   = $filer->{containers}{volume};
    my $vol_count = keys %{$volinfo};
    my $error     = undef;


    my $the_volume_name = _get_volume(tstatus => $CANDESTROY);
    if ( !$the_volume_name ) {
        $error = "did not pick a volume";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    if ( $volinfo->{$the_volume_name}{type} eq $TRAD ) {
        return($self->_send_end_msg($tid,$NA,"too few trad vols to destroy"))
             if ( $filer->{containers}{count}{$TRAD} <= $MIN_TRADVOLS );
    } else {
        return($self->_send_end_msg($tid,$NA,"too few flex vols to destroy"))
             if ( $filer->{containers}{count}{$TRAD} <= $MIN_FLEXVOLS );
    }
    logcomment( "$ownname:$filer->{fname}:the volume $the_volume_name will "
          . "be destroyed" );

    vol_purge( $filer->{fc}, $the_volume_name )
      or
      return($self->_send_end_msg($tid,$FAIL,"Cannot purge $the_volume_name"));
    _check_clone_aggr( $filer, $the_volume_name );

    ## delete the volume from the structure
    if ( $volinfo->{$the_volume_name}{type} eq $TRAD ) {
        $filer->{containers}{count}{$TRAD} -= 1;
    } else {
        $filer->{containers}{count}{$FLEX} -= 1;
    }

    delete $volinfo->{$the_volume_name};
    unexport_path( $filer->{fc}, "/vol/$the_volume_name" );
    return($self->_send_end_msg($tid));
} #end _vol_destroy

# sub _vol_online: online all offlined volumes
sub _vol_online {

    my $ownname = "_vol_online";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo = $filer->{containers}{volume};
    my $error   = undef;

    ## pick up a volume to be online
    my $count = 0;
    while (1) {
        my $the_volume_name = _get_volume( status => $OFFLINE, indx => 0 );
        last if ( !$the_volume_name );
        logcomment( "$ownname:$filer->{fname}:the volume $the_volume_name will "
              . "be online" );

        vol_online( $filer->{fc}, $the_volume_name )
          or $error = "online $the_volume_name fail: $@";
        if ($error) {
            $tinfo->{result} = $FAIL;
            $self->_send_noend_msg($tid,$ERROR,$error);
            $error = "";
        } else {
            $volinfo->{$the_volume_name}{status} = $ONLINE;
        }
        $count++;

    }
    return($self->_send_end_msg($tid,$NA,"did not pick a volume")) if (!$count);
    return($self->_send_end_msg($tid));
} #end _vol_online

# sub aggr_online:  online all offlined aggrs
sub _aggr_online {
    my $ownname = "_aggr_online";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $aginfo = $filer->{containers}{$AGGR};
    my $error  = undef;

    ## pick up an aggr to be online
    my $count = 0;
    while (1) {
        logcomment("$ownname:$filer->{fname}: calling _get_aggr");
        my $the_aggr_name = _get_aggr( status => $OFFLINE, indx => 0 );
        last if ( !$the_aggr_name );

        logcomment( "$ownname:$filer->{fname}:the aggr $the_aggr_name "
              . "will be online" );

        aggr_online( $filer->{fc}, $the_aggr_name )
          or $error = "aggr online $the_aggr_name fail: $@";


        if ($error) {
           $tinfo->{result} = $FAIL;
           $self->_send_noend_msg($tid,$ERROR,
                       "unable to online $the_aggr_name");
        } else {
            $aginfo->{$the_aggr_name}{status} = $ONLINE;
        }
        $count++;

    }
    return($self->_send_end_msg($tid,$NA,"did not pick a volume")) if (!$count);
    return($self->_send_end_msg($tid));
} #end _aggr_online

# sub vol_offline: offline one trad or flex vol
sub _vol_offline {
    my $ownname = "_vol_offline";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo = $filer->{containers}{volume};
    my $error   = undef;
    my $result = "PASS";

    ## pick up a volume to be offline
    my $the_volume_name = _get_volume( status => $OFFLINE );
    if ( !$the_volume_name ) {
        $error = "did not pick a volume";
        return($self->_send_end_msg($tid,$NA,$error));
    }
   logcomment( "$ownname:$filer->{fname}:the volume $the_volume_name will "
          . "be offline" );

    vol_offline( $filer->{fc}, $the_volume_name )
      or $error = "vol offline $the_volume_name fail: $@";

    if ($error) {
        return($self->_send_end_msg($tid,$FAIL,$error));
    } else {
        $volinfo->{$the_volume_name}{status} = $OFFLINE;
    }

    sleep 300;
    vol_online($filer->{fc}, $the_volume_name)
      or $error = "unable to online $the_volume_name: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    return($self->_send_end_msg($tid));
} #end _vol_offline

sub _aggr_offline {

    ## can't offline aggr if it contains vvols
    ## if the volume is backing clones, it cann't be destroyed?
    ## pick up an aggr to be offline

    my $ownname = "_aggr_offline";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo = $filer->{containers}{volume};
    return if ( $filer->{containers}{count}{$FLEX} <= 200 );
    my $aginfo = $filer->{containers}{$AGGR};
    return($self->_send_end_msg($tid,$NA,"too few volumes to offline aggr"))
                if ((keys %{$volinfo}) <= $MIN_FLEXVOLS);
    my $error  = undef;
    logcomment("$ownname:$filer->{fname}: calling _get_aggr");
    my $the_aggr_name =
        _get_aggr( tstatus=>$CANDESTROY, status => $ONLINE, indx => 0 );

    if ( !$the_aggr_name ) {
        $error = "did not pick an aggr";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    logcomment( "$ownname:$filer->{fname}:the aggr $the_aggr_name will "
          . "be offline" );

    ## purge all the volumes in the aggr
    my $aggr_s = aggr_status( $filer->{fc}, $the_aggr_name );
    my $vols = $aggr_s->volumes();
    foreach my $vol ( @{$vols} ) {
        return($self->_send_end_msg($tid,$NA,"not allowed to destroy $vol in ".
                                   "$the_aggr_name"));
        logcomment("$ownname:$filer->{fname}:Purging volume $vol");
        vol_purge( $filer->{fc}, $vol )
            or $error = "unable to purge $vol in $the_aggr_name: $@";
        return($self->_send_end_msg($tid,$FAIL,$error));
        delete $volinfo->{$vol};
        unexport_path( $filer->{fc}, "/vol/$vol" );
        $filer->{containers}{count}{$FLEX} -= 1;
    }

    aggr_offline( $filer->{fc}, $the_aggr_name )
      or $error = "aggr offline $the_aggr_name fail: $@";

    if ($error) {
        $tinfo->{result} = $FAIL;
        $tinfo->{text} = $error;
    } else {
        $aginfo->{$the_aggr_name}{status} = $OFFLINE;
    }

    return($self->_send_end_msg($tid));

} #end _aggr_offline

# sub _vol_resize  shrink or increase the size of one flex vol
sub _vol_resize {
    my $ownname = "_vol_resize";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo   = $filer->{containers}{volume};
    my $vol_count = keys %{$volinfo};
    return if ( !$vol_count );
    my $error = undef;

    ## pick up a volume to be resize
    my $the_volume_name = _get_volume( type => "flex", status => $ONLINE );
    if ( !$the_volume_name ) {
        $error = "did not pick a volume";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    logcomment( "$ownname:$filer->{fname}:the volume $the_volume_name will "
          . "be resized" );

    ## increase or decrease
    my $randpick = int( rand(2) );
    my $increase = "+";
    if ( $randpick == 0 ) {
        $increase = "-";
    }

    ## do the resize
    my $size = vol_size( $filer->{fc}, $the_volume_name );
    logcomment("$ownname:$filer->{fname}:_vol_resize: size = $size");
    if ( convert_to_bytes( $size . "k" ) <= convert_to_bytes("20m") ) {
        $increase = "+";    # volume is too small to shrink
    }
    my $new_size =
      vol_size( $filer->{fc}, $the_volume_name, $increase . "10m" );
    if ($@) {
        $error = "resize error for $the_volume_name: $@";
        $tinfo->{result} = $FAIL;
        $self->_send_noend_msg($tid,$ERROR,$error);
    }
    $filer->{containers}{volume}{$the_volume_name}{size} = $new_size;
    return($self->_send_end_msg($tid));
} # end _vol_resize

#aggr_add: add disks to one aggr
sub _aggr_add {
    my $ownname = "_aggr_add";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $fc     = $filer->{fc};
    my $aginfo = $filer->{containers}{$AGGR};
    my $error  = undef;
    logcomment("$ownname:$filer->{fname}: calling _get_aggr");
    my $the_aggr_name = _get_aggr( status => $ONLINE );
    if ( !$the_aggr_name ) {
        $error = "did not pick an aggr";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    logcomment("$ownname:$filer->{fname}:disk will be added to $the_aggr_name");

    # get spare disks number
    my $spare_disks = aggr_spare_status($fc);
    my $spare_num   = 0;
    foreach my $pool ( keys %$spare_disks ) {
        my $num = @{ $spare_disks->{$pool} };
        logcomment("$ownname:$filer->{fname}:spare disk at $pool: $num");
        if ( $num > $spare_num ) {
            $spare_num = $num;
        }
    }

    if ( $spare_num > 0 ) {
        aggr_add( $fc, $the_aggr_name, 1 )
          or $error = "fail at aggr add $the_aggr_name: $@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
        $aginfo->{$the_aggr_name}{size} += 1;
    }
    return($self->_send_end_msg($tid));
} #end _aggr_add

sub _tvol_add {
    my $ownname = "_tvol_add";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $error = undef;
    my $fc      = $filer->{fc};
    my $volinfo = $filer->{containers}{volume};
    my $error   = undef;

    my $the_volume_name = _get_volume( type => "$TRAD" );
    if ( !$the_volume_name ) {
        $error = "did not pick a volume";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    logcomment( "$ownname:$filer->{fname}:adding disks to trad volume "
          . "$the_volume_name" );

    # get spare disks number
    my $spare_disks = aggr_spare_status($fc);
    my $spare_num   = 0;
    foreach my $pool ( keys %$spare_disks ) {
        my $num = @{ $spare_disks->{$pool} };
        logcomment("$ownname:$filer->{fname}:spare disk at $pool: $num");
        if ( $num > $spare_num ) {
            $spare_num = $num;
        }
    }

    if ( $spare_num > 0 ) {
        vol_add( $fc, $the_volume_name, 1 )
          or $error = "fail at vol add $the_volume_name: $@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
        $volinfo->{$the_volume_name}{size} += 1;
    }
    return($self->_send_end_msg($tid));
} #end _tvol_add

# to run vol copy start parallely, each subtest has to have their own connection
sub _aggr_copy_start {
    my ( $fc, $spec, $dstaggr, %param ) = @_;
    my $ownname = "_aggr_copy_start";

    logcomment("at $ownname: spec = $spec");
    die "$ownname SOURCE parameter is required\n" unless ( $param{'SOURCE'} );
    my $src = $param{'SOURCE'};

    # Make sure the destination is restricted (if necessary).
    if ( $param{'RESTRICT'} ) {
        aggr_restrict( $fc, $dstaggr ) or return;
    }

    my $timeout = $param{'TIMEOUT'} ? $param{'TIMEOUT'} : 60;

    my $obj = subtest(
        sub {
            my $conn  = make_conn( $spec, "#rsh" );
            my $after = sayEOF( $conn,    "aggr copy start $src $dstaggr" );
            if ( $after !~ /100\%\s+done/ ) {
                logresult( 'FAIL',
                    "$ownname:$filer->{fname}:aggr copy start return: $after" );
            }

        },
        '-bg',
        '-onexit',
        sub {
        },    # don't pass results to parent
    );

    return $obj;

} #end aggr_copy_start

sub _aggr_copy {
    my $ownname = "_aggr_copy";
    logcomment("$ownname:$filer->{fname}:START");
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $error = undef;

    ## pick up an aggr to be base aggr

    my $fc = $filer->{fc};

    my $aginfo = $filer->{containers}{$AGGR};
    if (
        (
            $filer->{containers}{count}{$AGGR} +
            $filer->{containers}{count}{$TRAD}
        ) > MAX_TVOLS
      )
    {
        $error = "maximum trads and aggrs have already been created";
        return($self->_send_end_msg($tid,$NA,$error));
    }

    logcomment("$ownname:$filer->{fname}: calling _get_aggr");
    my $the_aggr_name = _get_aggr( status => $ONLINE );
    if ( !$the_aggr_name ) {
        $error = "didn't pick an aggr";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    logcomment( "$ownname:$filer->{fname}:the aggr $the_aggr_name "
          . "will be copied" );

    my @aggr_list = _aggr_create($filer);
    if ( @aggr_list || @aggr_list == () ) {
        $error = "didn't pick any dst aggr list";
        return($self->_send_end_msg($tid,$NA,$error));
    }

    my @copy_subs = ();
    ## start aggr copy in subtest with different rsh connection
    ## only two aggr copy can happen simultaneously (if to other filer, four)
    foreach my $copy_aggr (@aggr_list) {
        my $subt = _aggr_copy_start(
            $filer->{fc}, $filer->{fullname},
            $copy_aggr,
            SOURCE   => $the_aggr_name,
            RESTRICT => 1
        );
        if ($subt) {
            push @copy_subs, $subt;
        } else {
            $error = "aggr copy start for $the_aggr_name subtest failed ".
                     "to start";
            $self->_send_noend_msg($tid,$CONFIG,$error);
        }

    }

    foreach my $subt (@copy_subs) {
        $subt->waitFinish();
    }

    ## call copy_status until all done
    my $endtime = time() + AGGRCOPY_TIMEOUT;

    while ( time() < $endtime ) {
        my ( undef, undef, $res ) = say( $filer->{fc}, "aggr copy status" );
        if ( $res =~ /No\s+operations\s+in\s+progress/i ) {
            last;
        }

        sleep(5);
    }

    # online the volumes
    foreach my $copy_aggr (@aggr_list) {
        aggr_online( $filer->{fc}, $copy_aggr )
          or $error = "aggr online $copy_aggr returned error: $@";
        if ($error) {
            $self->_send_noend_msg($tid,$CONFIG,$error);
            $tinfo->{result} = $FAIL;
        }

    }
    return($self->_send_end_msg($tid));
} #end _aggr_copy

# to run vol copy start parallely, each subtest has to have their own connection
sub _vol_copy_start {
    my ( $fc, $spec, $dstvol, %param ) = @_;
    my $ownname = "_vol_copy_start";

    logcomment("$ownname:$filer->{fname}:at _vol_copy_start: spec = $spec");
    die "SOURCE parameter is required\n" unless ( $param{'SOURCE'} );
    my $src = $param{'SOURCE'};

    # Make sure the destination is restricted (if necessary).
    if ( $param{'RESTRICT'} ) {
        volRestrict( $fc, $dstvol ) or return;
    }

    my $timeout = $param{'TIMEOUT'} ? $param{'TIMEOUT'} : 60;

    my $obj = subtest(
        sub {
            my $conn  = make_conn( $spec, "#rsh" );
            my $after = sayEOF( $conn,    "vol copy start $src $dstvol" );
            if ( $after !~ /100\%\s+done/ ) {
                logresult( 'FAIL', "vol copy start return: $after" );
            }

        },
        '-bg',
        '-onexit',
        sub {
        },    # don't pass results to parent
    );

    return $obj;

} #end _vol_copy_start

## for both vvol and tvol
sub _vol_copy {
    my $ownname = "_vol_copy";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    ## pick up a volume to be base volume
    my $fc              = $filer->{fc};
    my $volinfo         = $filer->{containers}{volume};
    my $error           = undef;
    my $the_volume_name = _get_volume();
    if ( !$the_volume_name ) {
        $error = "did not pick a volume";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    logcomment( "$ownname:$filer->{fname}:the volume $the_volume_name"
          . " will be copied" );
    if ( $volinfo->{$the_volume_name}{status} eq $OFFLINE ) {
        logcomment( "$ownname:$filer->{fname}:$the_volume_name is offline, "
              . "onlining it" );
        vol_online( $filer->{fc}, $the_volume_name );
    }

    ## if it's flex vol, call _vvol_create to create a list of volumes
    ## else (trad) call _tvol_create to create a list of volumes
    my $volume_list = undef;
    if ( $volinfo->{$the_volume_name}{type} eq "flex" ) {
        if ( $filer->{containers}{count}{flex} == MAX_VVOLS ) {
            $error = "the maximum flex vols have already been created";
        return($self->_send_end_msg($tid,$NA,$error));
        }
        $volume_list = _vvol_create(count => 1);
        return($self->_send_end_msg($tid,$NA,"cannot create dst volume"))
                if (ref($volume_list) ne "ARRAY");
        $filer->{containers}{count}{$FLEX} += (@{$volume_list});
    } else {
        if (
            (
                $filer->{containers}{count}{$TRAD} +
                $filer->{containers}{count}{$AGGR}
            ) == MAX_TVOLS
          )
        {
            $error = "the maximum trad vols "
                  . "and aggrs have already been created";
             return($self->_send_end_msg($tid,$NA,$error));
        }
        $volume_list = _tvol_create($filer);
        return($self->_send_end_msg($tid,$FAIL,"cannot create trad vols"))
               if (ref($volume_list) ne "ARRAY");
        $filer->{containers}{count}{$TRAD} += (@{$volume_list});
    }

    my @copy_subs = ();
    ## start vol copy in subtest with different rsh connection
    ## only two vol copy can happen simultaneously (if to other filer, four)
    foreach my $copy_vol (@$volume_list) {
        my $subt = _vol_copy_start(
            $filer->{fc}, $filer->{fullname},
            $copy_vol,
            SOURCE   => $the_volume_name,
            RESTRICT => 1
        );
        if ($subt) {
            push @copy_subs, $subt;
        }
    }

    foreach my $subt (@copy_subs) {
        $subt->waitFinish();
    }

    ## call volcopy_status until all done
    my $endtime = time() + VOLCOPY_TIMEOUT;

    while ( time() < $endtime ) {
        my $res = volcopy_status( $filer->{fc}, "" );
        if ( $@ || $res ) {
            logcomment("$ownname:$filer->{fname}:volcopy return : $@ or $res");
            sleep(5);
            next;
        }
        if ( !defined($res) ) {    # no vol copy in progress
            last;
        }

        sleep(5);
    }

    # online the volumes
    foreach my $copy_vol (@$volume_list) {
        vol_online( $filer->{fc}, $copy_vol )
          or $error ="vol online $copy_vol " . "return error: $@";

        if ($error) {
            $tinfo->{result} = $FAIL;
            $self->_send_noend_msg($tid,$FAIL,$error);
        }
    }
    return($self->_send_end_msg($tid));
} #end _vol_copy

sub _vvol_clone {
    my $ownname = "_vvol_clone";
    my $fc      = $filer->{fc};
    my $volinfo = $filer->{containers}{volume};
    my $error   = undef;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    ##license
    if ( !_is_licensed( license => 'flex_clone' ) ) {
       $error = "license flex_clone fail:$@";
       return($self->_send_end_msg($tid,$CONFIG,$error));
    }

    ## make sure volume number not reached the uplimit
    if ( $filer->{containers}{count}{$FLEX} >= MAX_VVOLS ) {
        $error = "hit the MAX_VVOLS";
        return($self->_send_end_msg($tid,$NA,$error));
    }

    ##pick up an onlined non-cloned flex volume
    my $the_volume_name = _get_volume( type => $FLEX, cloned => 1 );
    if ( !$the_volume_name ) {
        $error = "did not find a non-cloned flex volume";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    my $msg = "the volume $the_volume_name will be cloned";
    if ( $volinfo->{$the_volume_name}{status} eq $OFFLINE ) {
        logcomment( "$ownname:$filer->{fname}:$the_volume_name is offline, "
              . "onlining it" );
        vol_online( $filer->{fc}, $the_volume_name );
    }

    ## name of the clone volume
    my $prefix =
      $filer->{cfginfo}{$FLEX}{prefix}
      ? $filer->{cfginfo}{$FLEX}{prefix}
      : 'flexvol';
    my $dst_name = $prefix . time();

    #vol_clone_create (Volume::Clone qw(vol_clone_create)
    vol_clone_create(
        conn   => $fc,
        parent => $the_volume_name,
        clone  => $dst_name
      )
      or $error = "error clone create $the_volume_name $dst_name:$@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

    export_path( $fc, "/vol/$dst_name" );

    #mark parent volume {clone} = newvolume
    $volinfo->{$the_volume_name}{clone}{dest} = $dst_name;

    #add newvolume in {volume}, mark {clone}=parentvolume, {$AGGR} etc
    $volinfo->{$dst_name}{size}      = $volinfo->{$the_volume_name}{size};
    $volinfo->{$dst_name}{$AGGR}     = $volinfo->{$the_volume_name}{$AGGR};
    $volinfo->{$dst_name}{guarantee} = $volinfo->{$the_volume_name}{guarantee};
    $volinfo->{$dst_name}{type}      = "flex";
    $volinfo->{$dst_name}{clone}{parent} = $the_volume_name;
    $filer->{containers}{count}{$FLEX} += 1;

    #mark aggr to be cloned
    my $paggr = $volinfo->{$the_volume_name}{$AGGR};
    logcomment("$ownname:$filer->{fname}:at vol_clone: parent aggr is $paggr");
    $filer->{containers}{$AGGR}{$paggr}{clone} = 1;
    return($self->_send_end_msg($tid));
} #end _vol_clone

sub _vvol_clone_split {
    my $ownname = "_vvol_clone_split";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    my $fc      = $filer->{fc};
    my $volinfo = $filer->{containers}{volume};
    my $error   = undef;
    #license
    if ( !_is_licensed( license => 'flex_clone' ) ) {
        $error = "license flex_clone fail:$@";
        return($self->_send_end_msg($tid,$CONFIG,$error));
    }

    #pick up an online cloned vvol
    my $svol =
      _get_volume( type => $FLEX, status => $ONLINE, cloned => "parent" );
    if ( !$svol ) {
       $error = "unable to select source volume";
       return($self->_send_end_msg($tid,$NA,$error));
    }
    if ( $volinfo->{$svol}{status} ne $ONLINE ) {
        logcomment("$ownname:$filer->{fname}:$svol is offline, onlining");
        vol_online( $filer->{fc}, $svol )
          or $error = "unable to online $svol";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    }

    my $dvol = $volinfo->{$svol}{clone}{dest};

    #vol_clone_split_start(dvolume) to complete
    vol_clone_split_start( conn => $fc, clone => $dvol, wait_to_complete => 1 )
      or $error = "fail when clone split $svol, $dvol:$@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

    # _check_clone_aggr (so parent volume also not cloned)
    _check_clone_aggr( $filer, $dvol );
    return($self->_send_end_msg($tid));
} #end _vvol_clone_split

sub _check_clone_aggr {
    my $ownname = "_check_clone_aggr";

    # $volume is potentially the clone(which will be destroyed or split
    my ( $filer, $volume ) = @_;
    my $volinfo = $filer->{containers}{volume};
    my $aginfo  = $filer->{containers}{$AGGR};

    if ( defined( $volinfo->{$volume}{clone} ) ) {
        my $parent = $volinfo->{$volume}{clone}{parent};
        logcomment( "$ownname:$filer->{fname}the parent of the destroyed/split "
              . "volume is $parent" );
        delete $volinfo->{$parent}{clone};
        delete $volinfo->{$volume}{clone};
        my $parent_aggr = $volinfo->{$parent}{$AGGR};
        logcomment( "$ownname:$filer->{fname}at _check_clone_aggr, the parent "
              . "aggr is $parent_aggr" );
        my $cloned = 0;
        foreach my $volume ( keys %{$volinfo} ) {

            if ( $volinfo->{$volume}{$AGGR} eq $parent_aggr
                && defined( $volinfo->{$volume}{clone} ) )
            {
                $cloned = 1;
                last;
            }
        }
        if ( !$cloned ) {
            delete $aginfo->{$parent_aggr}{clone};
        }

    }
    return;

} #end _check_clone_aggr

# calculate flex or trad volume numbers
sub _vol_num {
    my ( $volinfo, $type ) = @_;

    my $num = 0;
    foreach my $volume ( keys %{$volinfo} ) {
        if ( $volinfo->{$volume}{type} eq $type ) {
            $num++;
        }
    }
    return $num;
} #end _vol_name

sub _vol_snapshot_create {
    my $ownname = "_vol_snapshot_create";
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;
    my $volinfo = $filer->{containers}{volume};
    my $fc      = $filer->{fc};
    my $error   = undef;

    #pick up a volume to create snapshots
    my $the_volume_name = _get_volume();
    if ( !$the_volume_name ) {
        $error = "didn't pick up any volume to create snapshot";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    if ( $volinfo->{$the_volume_name}{status} eq $OFFLINE ) {
        logcomment( "$ownname:$filer->{fname}:$the_volume_name is offline, "
              . "onlining..." );
        vol_online( $filer->{fc}, $the_volume_name )
          or $error = "cannot online $the_volume_name";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    }
    logcomment( "$ownname:$filer->{fname}:the volume $the_volume_name will "
          . "create snapshots" );

    #create several snapshots(1-3) with prefix (int_testsnap.time())
    my $prefix =
      $filer->{cfginfo}{snapshot}{prefix}
      ? $filer->{cfginfo}{snapshot}{prefix}
      : 'int_testsnap';

    my $count = int( rand(3) ) + 1;
    for ( 1 .. $count ) {
        my $snap_name = $prefix . time();
        snapshot_create( $fc, $the_volume_name, $snap_name )
          or $error = "fail when creating snapshot $snap_name:$@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

        #add in {volume}->{$volname}{snapshots}
        $volinfo->{$the_volume_name}{snapshots} += 1;
    }
    return($self->_send_end_msg($tid));
} #end _vol_snapshot_create

sub _vol_snapshot_restore {
    my $ownname = "_vol_snapshot_restore";
    logcomment("$ownname:$filer->{fname}:START");
    my $volinfo   = $filer->{containers}{volume};
    my $fc        = $filer->{fc};
    my $error     = undef;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    ##license
    if ( !_is_licensed( license => 'snaprestore' ) ) {
        $error = "license snaprestore fail:$@";
        return($self->_send_end_msg($tid,$CONFIG,$error));
    }

    #pick up a volume to do snaprestore
    my $the_volume_name = _get_volume();
    if ( !$the_volume_name ) {
        $error = "didn't pick up any volume restore snapshot";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    logcomment( "$ownname:$filer->{fname}:Will do snapshot restore at the "
          . "volume $the_volume_name" );

    #create a file with a filename in it
    my $file_name = "$ownname:$filer->{fname}:restorefile" . time();
    filer_file_make( $fc, "/vol/$the_volume_name/$file_name" )
      or $error = "mkfile fail: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    wrfile( $fc, "/vol/$the_volume_name/$file_name", $file_name );

    #take a snapshot
    my $prefix =
      $filer->{cfginfo}{snapshot}{prefix}
      ? $filer->{cfginfo}{snapshot}{prefix}
      : 'int_testsnap';

    my $snap_name = $prefix . time();
    snapshot_create( $fc, $the_volume_name, $snap_name )
      or $error = "snap create fail: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

    #add in {volume}->{$vol}{snapshots}
    $volinfo->{$the_volume_name}{snapshots} += 1;

    #rm the file
    filer_file_destroy( $fc, "/vol/$the_volume_name/$file_name" )
      or $error = "file destroy fail: $@";
    if ($error) {
        $self->{result} = $FAIL;
        $self->_send_noend_msg($tid,$INFO,$error);
    }
    $error = "";

    #do a snaprestore on volume
    snaprestore_volume( $fc, $the_volume_name, $snap_name )
      or $error = "snaprestore fail for $the_volume_name: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    #verify the file is restored correctly
    if ( !filer_file_exists( $fc, "/vol/$the_volume_name/$file_name" ) ) {
        $error = "$file_name is not restored";
        return($self->_send_end_msg($tid,$FAIL,$error));

    }
    my $output = rdfile( $fc, "/vol/$the_volume_name/$file_name" );
    if ( $output !~ /$file_name/ ) {
        $tinfo->{text} = "file $file_name is not restored correctly: $output";
        $tinfo->{result} = $FAIL;
    }
    return($self->_send_end_msg($tid));
} #end _vol_snapshot_restore

sub _vol_snapshot_sfsr {
    my $ownname = "_vol_snapshot_sfsr";
    my $volinfo = $filer->{containers}{volume};
    my $fc      = $filer->{fc};
    my $error   = undef;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    ##license
    if ( !_is_licensed( license => 'snaprestore' ) ) {
        $error = "snaprestore is not licensed";
        return($self->_send_end_msg($tid,$CONFIG,$error));
    }

    #pick up a volume to do snaprestore
    my $the_volume_name = _get_volume( status => $ONLINE );
    if ( !$the_volume_name ) {
        $error = "didn't pick up any volume to do sfsr";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    my $msg = "Will do sfsr at the volume $the_volume_name";
    $self->_send_noend_msg($tid,$INFO,$msg);
    #create a file with a filename in it
    my $file_name = "restorefile" . time();
    filer_file_make( $fc, "/vol/$the_volume_name/$file_name" )
      or $error = "mkfile fail: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    wrfile( $fc, "/vol/$the_volume_name/$file_name", $file_name );

    #take a snapshot
    my $prefix =
      $filer->{cfginfo}{snapshot}{prefix}
      ? $filer->{cfginfo}{snapshot}{prefix}
      : 'int_testsnap';

    my $snap_name = $prefix . time();
    snapshot_create( $fc, $the_volume_name, $snap_name )
      or $error = "snap create fail: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

    #add in {volume}->{$vol}{snapshots}
    $volinfo->{$the_volume_name}{snapshots} += 1;

    #rm the file
    filer_file_destroy( $fc, "/vol/$the_volume_name/$file_name" )
      or $error = "file destroy fail: $@";
    if ($error) {
       $self->_send_noend_msg($tid,$ERROR,$error) if ($error);
       $tinfo->{result} = $FAIL;
       $error = "";
    }

    #do a sfsr snaprestore on volume
    snaprestore_file( $fc, "/vol/$the_volume_name/$file_name", $snap_name )
      or $error =  "snaprestore fail for /vol/$the_volume_name/$file_name: $@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

    #verify the file is restored correctly
    if ( !filer_file_exists( $fc, "/vol/$the_volume_name/$file_name" ) ) {
        $error = "$file_name is not restored";
        return($self->_send_end_msg($tid,$FAIL,$error));
    }
    my $output = rdfile( $fc, "/vol/$the_volume_name/$file_name" );
    if ( $output !~ /$file_name/ ) {
        $tinfo->{result} = $FAIL;
        $tinfo->{text} = "file $file_name is not restored correctly: $output";
    }
    return($self->_send_end_msg($tid));
} #end _vol_snapshot_sfsr

sub _vol_snapshot_delete {
    my $ownname = "_vol_snapshot_delete";
    my $volinfo = $filer->{containers}{volume};
    my $fc      = $filer->{fc};
    my $error   = undef;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    #pick up a volume which has the test snapshots
    my $the_volume_name = _get_volume( status => $ONLINE );
    if ( !$the_volume_name ) {
        $error = "didn't pick up any volume to delete snapshots";
        return($self->_send_end_msg($tid,$NA,$error));
    }
    my $msg = "Will do snapshot delete at the volume $the_volume_name";
    $self->_send_noend_msg($tid,$INFO,$msg);

    #delete 1-n of them
    my $num      = $volinfo->{$the_volume_name}{snapshots};
    my $count    = int( rand($num) ) + 1;
    my @snaplist = snapshot_list( $fc, $the_volume_name );
    my $dcount   = 0;
    my $prefix   =
      $filer->{cfginfo}{snapshot}{prefix}
      ? $filer->{cfginfo}{snapshot}{prefix}
      : 'int_testsnap';
    for ( 1 .. $count ) {
        if ( $snaplist[$dcount] =~ /$prefix/ ) {
            snapshot_delete( $fc, $the_volume_name, $snaplist[$dcount] )
              or $error = "fail when deleting $snaplist[$dcount]:$@";
            return($self->_send_end_msg($tid,$FAIL,$error));
            $volinfo->{$the_volume_name}{snapshots} -= 1;
        }
        if ( ++$dcount >= @snaplist ) {
            $tinfo->{result} = $FAIL;
            $tinfo->{text} = "snapshot count is not correct";
            last;
        }
    }
    return($self->_send_end_msg($tid));
} #end _vol_snapshot_delete

sub _aggr_snapshot_create {
    my $ownname = "_aggr_snapshot_create";
    my $aginfo = $filer->{containers}{$AGGR};
    my $fc     = $filer->{fc};
    my $error  = undef;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    #pick up an aggr to create snapshots
    logcomment("$ownname:$filer->{fname}: calling _get_aggr");
    my $the_aggr_name = _get_aggr( status => $ONLINE );
    if ( !$the_aggr_name ) {
        $error = "didn't pick up any aggr to create snapshot";
        return($self->_send_end_msg($tid,$FAIL,$error));
    }
    my $msg = "the aggr $the_aggr_name will create snapshots";
    $self->_send_noend_msg($tid,$INFO,$msg);

    #create several snapshots(1-3) with prefix (int_testsnap.time())
    my $prefix =
      $filer->{cfginfo}{snapshot}{prefix}
      ? $filer->{cfginfo}{snapshot}{prefix}
      : 'int_testsnap';

    my $count = int( rand(3) ) + 1;
    for ( 1 .. $count ) {
        my $snap_name = $prefix . time();
        snapshot_create( $fc, "-A $the_aggr_name", $snap_name )
          or $error =
          "fail when creating snapshot $snap_name at aggr $the_aggr_name:$@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

        #add in {$AGGR}->{$the_aggr_name}{snapshots}
        $aginfo->{$the_aggr_name}{snapshots} += 1;
    }

    return($self->_send_end_msg($tid));

} #end _aggr_snapshot_create

sub _aggr_snapshot_restore {
    my $ownname = "_aggr_snapshot_restore";
    my $aginfo  = $filer->{containers}{$AGGR};
    my $volinfo = $filer->{containers}{volume};
    my $fc      = $filer->{fc};
    my $error   = undef;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;


    ##license
    if ( !_is_licensed( $filer, license => "snaprestore" ) ) {
        $tinfo->{result} = "CONFIG";
        my $text = "no snaprestore license";
        return($self->_send_end_msg($tid,$CONFIG,$text));
    }

    #pick up an volume/aggr to do snaprestore
    my $the_volume_name = _get_volume( status => $ONLINE, type => "flex" );
    if ( !$the_volume_name ) {
        my $msg =  "didn't pick any volume/aggr to do sfsr" ;
        return($self->_send_end_msg($tid,$NA,$msg));
    }
    my $the_aggr_name = $volinfo->{$the_volume_name}{$AGGR};
    my $msg = "Will do snapshot restore at $the_volume_name/$the_aggr_name";
    $self->_send_noend_msg($tid,$INFO,$msg);

    #create a file with a filename in it
    my $file_name = "restorefile" . time();
    filer_file_make( $fc, "/vol/$the_volume_name/$file_name" )
      or $error = "mkfile fail: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

    wrfile( $fc, "/vol/$the_volume_name/$file_name", $file_name );

    #take a snapshot
    my $prefix =
      $filer->{cfginfo}{snapshot}{prefix}
      ? $filer->{cfginfo}{snapshot}{prefix}
      : 'int_testsnap';

    my $snap_name = $prefix . time();
    snapshot_create( $fc, "-A $the_aggr_name", $snap_name )
      or $error = "aggr snap create fail: $@";
    return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);

    #add in {volume}->{$vol}{snapshots}
    $aginfo->{$the_aggr_name}{snapshots} += 1;

    #rm the file
    filer_file_destroy( $fc, "/vol/$the_volume_name/$file_name" )
      or $error = "file destroy fail: $@";
    if ($error) {
       $self->_send_noend_msg($tid,$ERROR,$error);
       $tinfo->{result} = $FAIL;
    }
    $error = "";

    #do a snaprestore on aggr
    snaprestore_volume( $fc, "-A $the_aggr_name", $snap_name );
    if ( $@ and $@ !~ /revert successful/ ) {
        $error = "snaprestore fail for $the_aggr_name: $@";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    }

    #verify the file is restored correctly
    if ( !filer_file_exists( $fc, "/vol/$the_volume_name/$file_name" ) ) {
        $error = "$file_name is not restored";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    }
    my $output = rdfile( $fc, "/vol/$the_volume_name/$file_name" );
    if ( $output !~ /$file_name/ ) {
        $error = "file $file_name is not restored correctly: $output";
        return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
    }

    return($self->_send_end_msg($tid));
} #end _aggr_snapshot_restore

sub _aggr_snapshot_delete {
    my $ownname = "_aggr_snapshot_delete";
    my $aginfo = $filer->{containers}{$AGGR};
    my $fc     = $filer->{fc};
    my $error  = undef;
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    logcomment("$ownname:$filer->{fname}: calling _get_aggr");
    my $the_aggr_name = _get_aggr( status => $ONLINE, snaps => 1 );
    if ( !$the_aggr_name ) {
         my $text = "didn't pick up any aggr to delete snapshots" ;
        return($self->_send_end_msg($tid,$NA,$text));
    }
    my $msg = "Will do snapshot delete at the aggr $the_aggr_name";
    $self->_send_noend_msg($tid,$INFO,$msg);

    #delete 1-n of them
    my $num      = $aginfo->{$the_aggr_name}{snapshots};
    my $count    = int( rand($num) ) + 1;
    my @snaplist = snapshot_list( $fc, "-A $the_aggr_name" );
    my $dcount   = 0;
    my $prefix   =
      $filer->{cfginfo}{snapshot}{prefix}
      ? $filer->{cfginfo}{snapshot}{prefix}
      : 'int_testsnap';
    for ( 1 .. $count ) {
        if ( $snaplist[$dcount] =~ /$prefix/ ) {
            snapshot_delete( $fc, $the_aggr_name, $snaplist[$dcount], "-A" )
              or $error = "fail when deleting $snaplist[$dcount]:$@";
            return($self->_send_end_msg($tid,$FAIL,$error)) if ($error);
            $aginfo->{$the_aggr_name}{snapshots} -= 1;
        }
        if ( ++$dcount >= @snaplist ) {
            $tinfo->{result} = "FAIL";
            $tinfo->{text} = "snapshot count is not correct";
            last;
        }
    }
    return($self->_send_end_msg($tid));
} #end _aggr_snapshot_delete

###############################################################################
#
#      qtree subroutines
#

sub _check_qtree_status {
    my $ownname = "_check_qtree_status";
    my $qtinfo = $filer->{containers}{$QTREES};
    foreach my $qtpath ( keys(%$qtinfo) ) {
        if ( !qtreePresent( $filer->{fc}, $qtpath ) ) {
            my $error =
              "$ownname:$filer->{fname}:Qtree \"$qtpath\" does " . "not exist";
            logcomment("WARNING:: $error");
        }
    }
    foreach my $qtpath ( qtreeList( $filer->{fc} ) ) {
        if ( !$qtinfo->{$qtpath} ) {
            my $error =
              "$ownname:$filer->{fname}:Qtree \"$qtpath\" not "
              . "in hashtable";
            logcomment("WARNING:: $error");
        }
    }
    logcomment("$ownname:$filer->{fname}:DONE ");
} #end _check_qtree_status

sub _qtree_create {
    my $ownname = "_qtree_create";
    my $count     = 1;
    my $qtinfo    = $filer->{containers}{qtrees};
    my $error     = undef;
    my $prefix    = $qtinfo->{prefix} ? $qtinfo->{prefix} : 'qtree';
    my ($vol_name);
    my ($tid, $rc) = $self->_send_start_msg($ownname);
    my $tinfo = $self->{tests}{$tid};
    $tinfo->{result} = $PASS;

    my $vol_name = _get_volume(status=>$ONLINE);
    if (!$vol_name) {
            $error = "No volumes available";
            return($self->_send_end_msg($tid,$NA,$error));
    }

    my $cntr       = 0;
    my @new_qtrees = ();

    my $msg = "Creating $count qtrees on volume $vol_name";
    $self->_send_noend_msg($tid,$INFO,$msg);
    $filer->_create_qtrees_for_volume(
        $vol_name, $count,
        prefix => $prefix,
        export => 1
    );
    if ( $filer->{error} ) {
        $error =
          "cannot $count qtrees on volume $vol_name:$filer->{error}";
        $tinfo->{result} = "FAIL";
        $tinfo->{text} = $error;
    }
    return($self->_send_end_msg($tid));
} #end _qtree_create

sub _qtree_destroy {
    my $ownname = "_qtree_destroy";
    my $qtinfo   = $filer->{containers}{$QTREES};
    my $error    = undef;
    my @qt_list  = keys(%$qtinfo);
    my $qt_count = @qt_list;
    return unless ( $qt_count > 0 );
    my $result = "PASS";
    my $qtnum  = int( rand($qt_count) );
    my $qtpath = $qt_list[$qtnum];
    my ($tid, $rc) = $self->_send_start_msg($ownname,$qtpath);
    my $tinfo = $self->{tests}{$tid};
    logcomment("_qtree_destroy:  Deleting qtree: \"$qtpath\"");
    $filer->_purge_qtree($qtpath);
    if ( $filer->{error} ) {
        $error = "Cannot purge qtree %s: $filer->{error}";
        return($self->_send_end_msg($tid,"FAIL",$error));
    }
    delete( $qtinfo->{qtpath} );
    $tinfo->{result} = $result;
    $tinfo->{text} = "deleted $qtpath";
    return($self->_send_end_msg($tid));
} #end _qtree_destroy
#
#      end qtree subroutines
#
###############################################################################
###############################################################################
#
#      utility subroutines
#

sub _get_aggr {
    my %opts         = @_;
    my $ownname       = "_get_aggr";
    my $aggr2         = ( exists $opts{'AGGR2'} ) ? $opts{'AGGR2'} : undef;
    my $status        = ( exists $opts{status} ) ? $opts{status} : undef;
    my $index         = (exists $opts{indx}) ? $opts{indx} : undef;
    my $snaps         = (exists $opts{'SNAPSHOTS'}) ? $opts{'SNAPSHOTS'} :undef;
    my $candestroy    = (exists $opts{tstatus}) ? $opts{tstatus} : undef;
    my $scount        = 0;
    my $the_aggr_name = undef;
    my $aggr_count    = undef;
    my $aggrs         = $filer->{containers}{$AGGR};
    if ( !$index ) {
        $aggr_count = keys %{$aggrs};
        $index      = int( rand($aggr_count) ) + 1;
    }

    foreach my $aggr ( keys %{$aggrs} ) {
        $scount++;
        next if ( $scount < $index );
        next if ( ($status) && ( $aggrs->{$aggr}{status} ne $status ) );
        next if ( ($aggr2) && ( $aggr eq $aggr2 ) );
        next
          if (
            ($snaps)
            && (   ( !exists $aggrs->{$AGGR}{$aggr}{snapshots} )
                || ( !$aggrs->{$AGGR}{$aggr}{snapshots} ) )
          );
        next if (($candestroy) &&
             ($aggrs->{$AGGR}{$aggr}{tstatus} ne $CANDESTROY));
        logcomment("$ownname:$filer->{fname}:num $index aggregate is $aggr");
        $the_aggr_name = $aggr;
        last;
    }
    return $the_aggr_name;
}

sub _get_volume {
    my %opts      = @_;
    my $ownname   = "_get_volume";
    my $status    = ( exists $opts{status} ) ? $opts{status} : undef;
    my $type      = ( exists $opts{type} ) ? $opts{type} : undef;
    my $vol2      = ( exists $opts{vol2} ) ? $opts{vol2} : undef;
    my $index     = ( exists $opts{indx} ) ? $opts{indx} : undef;
    my $cloned    = ( exists $opts{clone} ) ? $opts{clone} : undef;
    my $candestroy    = (exists $opts{tstatus}) ? $opts{tstatus} : undef;
    my $vol_count = ( keys %{ $filer->{containers}{volume} } );
    if ( !$vol_count ) {
        logWarn("$ownname:$filer->{fname}: no volumes exist");
        return;
    }
    my $the_vol = undef;

    $index = rand($vol_count) if !$index;
    my $lcount = 0;
    foreach my $key ( keys %{ $filer->{containers}{volume} } ) {
        my $vol = $filer->{containers}{volume}{$key};
        $lcount++;
        next unless ( $lcount >= $index );
        next if ( ($status) && ( $vol->{status} ne $status ) );
        next if ( ($type)   && ( $vol->{type}   ne $type ) );
        next if ( ($vol2) && ( $vol eq $vol2 ) );
        next if ( ( $cloned == 'none' ) && ( exists $vol->{clone} ) );
        next if ( ( $cloned eq "dest" ) && ( !exists $vol->{clone}{parent} ) );
        next if ( ( $cloned eq "parent" ) && ( !exists $vol->{clone}{dest} ) );
        next if (($candestroy) && ($vol->{tstatus} ne $CANDESTROY));
        $the_vol = $key;
        last;
    }
    return $the_vol;
}

sub _is_licensed {
    my $filer = shift;
    my %opts  = @_;
    my $error = undef;
    return if ( !exists $opts{license} );
    if ( !is_licensed( $filer->{fc}, $opts{license} ) ) {
        license( $filer->{fc}, 'snaprestore' )
          or $error = "$filer->{fc}license %opts{license} fail:$@";
        if ($error) {
            logcomment("WARNING::$error");
            return 0;
        }
    }
    return 1;
} #end _is_licensed

sub _send_start_msg {
    my $self   = shift;
    my $testname = shift;
    my $cont = shift;
    my $tinfo = {};
    $tinfo->{testname} = $testname;
    $tinfo->{filer} = $self->{filer}{fname};
    $tinfo->{tid} = $self->_get_testid();
    $tinfo->{starttime} = time();
    $tinfo->{cont} = $cont;
    my $header = _create_msg_header( $self, $tinfo );
    $self->{omsginfo} = $tinfo;
    my $starttime = time();
    my $msg = sprintf "%s;STARTTIME=%s;CONTAINER=%s;EMSG", $header,
      $tinfo->{starttime}, $tinfo->{cont}  ;
    my $resp = _send_parent( $self, $msg, undef );
    $self->{tests}{$tinfo->{tid}} = $tinfo;
    return ($tinfo->{tid},$resp);
} #end _send_start_msg

sub _create_msg_header {
    my $self  = shift;
    my $tinfo = shift;
    my $tid   = undef;
    $tinfo->{msgno} = $self->_get_msgno();
    my $msg = sprintf "ID=%s;MSG=%d;TESTNAME=%s;FILER=%s", $tinfo->{tid},
                      $tinfo->{msgno}, $tinfo->{testname},$tinfo->{filer};
    return $msg;
}

sub _get_testid {
    my $self = shift;
    my $tid = "$self->{testid}:$self->{testno}";
    $self->{testno}++;
    return $tid;
}

sub _get_msgno {
    my $self = shift;
    return $self->{msgno}++;
} #end _get_msgno

sub _send_end_msg {
    my $self   = shift;
    my $tid    = shift;
    my $result = shift;
    my $msg = shift;

    my $mode   = undef;
    my $tinfo = $self->{tests}{$tid};
    if (!$tinfo) {
        logcomment("_send_end_msg: test $tid does not exist");
        return;
    }
    $tinfo->{result} = $result if ($result);
    $tinfo->{text} = $msg if ($msg);
    my $header = _create_msg_header( $self, $tinfo );
    $tinfo->{duration} = (time() - $tinfo->{starttime});

    print
"DBGCEB:_send_msg: got a RESULT message to send from $tinfo->{testname}\n";
    my $msg =
      sprintf "%s;ENDTEST;RESULT=%s;RESULTSTR=%s;DURATION=%s;EMSG",
      $header, $tinfo->{result}, $tinfo->{text},$tinfo->{duration};
    print "DBGCEB:$self->{testid}:_send_end_msg: sending msg to parent\n$msg\n";
    my $resp = _send_parent( $self, $msg, $mode );
    delete $self->{tests}{$tid};
    return $resp;
} #end _send_end_msg

sub _send_noend_msg {
    my $self   = shift;
    my $tid  = shift;
    my $type = shift;
    my $text = shift;

    my $tinfo = $self->{tests}{$tid};
    if (!$tinfo) {
        logcomment("_send_end_msg: test $tid does not exist");
        return;
    }
    $tinfo->{type} = $type if $type;
    $tinfo->{text} = $text if $text;
    my $header = _create_msg_header( $self, $tinfo );
    my $msg    = undef;
    my $mode   = undef;
    $self->{omsginfo} = $tinfo;

    if ( $tinfo->{type} eq $ACTION ) {
        $msg = sprintf "%s;ACTION=%s;ITYPE=%s;ITEM=%s;EMSG", $header,
          $tinfo->{action}, $tinfo->{itype}, $tinfo->{item};
        $mode = $WAIT;
    } else {
        $msg = sprintf "%s;%s;TEXT=%s;EMSG" . $header, $tinfo->{type},
          $tinfo->{text};
    }
    print "DBGCEB:$self->{testid}:_send_noend_msg: sending msg to parent\n$msg\n";

    my $resp = _send_parent( $self, $msg, $mode );
    $tinfo->{text} = "";
    $tinfo->{type} = "";
    return $resp;
} #end _send_noend_msg

sub _send_parent {
    my $self = shift;
    my $msg  = shift;
    my $mode = shift;
    logcomment($msg);
    return;
    $self->{comm}{mode}   = $mode;
    $self->{comm}{outmsg} = $msg;
    my $sock = Subtest::ipc();
    my $ack  = undef;

    #add subfunction id if subfunction is zero.

    eval { $sock->write_message(); };
    if ($@) {
        logWarn("$self->{testid}:unable to write message to parent: $@");
        return 1;
    }
    my $resp = read_parent( $self, $ACK );
    logcomment("$self->{testid}:did not receive parent ack") if ($resp ne $ACK);
    logcomment($msg);
    return;
} #end _send_parent

sub read_parent {
    my $self                     = shift;
    my $exp                      = shift;
    my $mode                     = $self->{comm}{mode};
    my $omsg                     = $self->{comm}{outmsg};
    my $omsg_info                = $self->{comm}{omsginfo};
    my $sock                     = Subtest::ipc();
    my $ack                      = undef;
    my $resp                     = undef;
    my $tid = undef;
    my $msgno = undef;
    if ( $exp eq $ACK ) {
        $ack   = 1;
        $tid   = $omsg_info->{tid};
        $msgno = $omsg_info->{msgno};
    }
    my $retries = 5;
    while (1) {
        while ( $sock->can_read() ) {
            my $msg = $sock->read_message();
            chomp $msg;
            if ($ack) {
                $resp = $1
                  if ( $msg =~ /^$tid\;MSG=$msgno\;.*;ACK=(\w*)\;EMSG$/ );
            }
            if ( $msg =~ /$PAUSE/ ) {
                $self->{teststatus} = $PAUSE;
            } elsif ( $msg =~ /$RESUME/ ) {
                $self->{teststatus} = $RESUME;
            } elsif ( $msg =~ /$QUIT/ ) {
                exit(0);
#                $self->{teststatus} = $QUIT;
            } elsif ($msg eq "LIVETESTS") {
                my @livetests = ();
                foreach my $key (keys %{$self->{tests}}) {
                   push @livetests,$self->{tests}{$key}{testname};
                }
                return @livetests;
            } else {
                _logcomment(
                    "$self->{hname} received unknown msg from parent: $msg");
            }
        }
        $retries--;
 #       last if ( $self->{teststatus} eq $QUIT );
        last if ((!$retries) && ( $self->{teststatus} ne $PAUSE ));
        next if (($mode eq $WAIT) && (!$resp));
        last if ( $self->{teststatus} ne $PAUSE );
    }
    return $resp;
} #end read_parent


# get existing container information
sub setup {
    my $filer = shift;

    # $filer->{fname} has a filer spec
    my $f = IFTSetup->new($filer);

    $f->get_existing_containers();

    # temporary put raid_type here
    $f->{cfginfo}{$AGGR}{raid} = $main::AGGR_RAID
      if ( !$filer->{cfginfo}{$AGGR}{raid} );
    $f->{cfginfo}{$TRAD}{raid} = $main::TVOL_RAID
      if ( !$filer->{cfginfo}{$TRAD}{raid} );

    return;
}

sub stop_test {
    my $self = shift;
    logcomment("subtest got killed");
    $self->{subt}->kill();
}



#
#      end utility subroutines
#
####################################################################

1;
