#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/IOMgr/IO/Dt.pm#1 $

## @summary Provide an interface to the DataTest utility
## @author knopp@netapp.com
## @status private
## @pod here

# Package Name
##############
package Dt;

use strict;
use POSIX qw(strftime); 
use File::Basename;
use Time::localtime; 
use Time::Local;
use Utils qw(make_host make_conn);

use IOMgr::IO;

use Error;
use Data::Dumper;
use POSIX ":sys_wait_h";
use vars qw(
	    $_CLIENT_CNT    
	    @ISA
	    $JOB_CNT
	    );

@ISA = qw(IOMgr::IO);

################################################################################
# Subroutine Name: display()
# Objective:       Display contents of Dt
# Details:         See POD documentation at the end of this file
################################################################################
sub display {
    my ($self)=@_;
    my (
        @buffer,
        );
    
    $self->_clear_err();
    
    unless (exists $self->{output} and length $self->{output}) {
	$self->_set_err("Error: Dt::display() No Datatest information");
        return "";
    }

    foreach my $line (@{$self->{output}}){
        push @buffer, "   " . $line;
    }

    return (wantarray()) ? @buffer : \@buffer;         
}

###############################################################################
# Method:    get_files
# Objective: retrieve files on volumes
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_files {
    my ($self,$filter)=@_;   
    my (
        $cmd,
        $status,
        @buffer,
        @output,
        );
    
    $self->_clear_err();        

    if ($self->{fs} =~ /raw/i){
	$self->_set_err("Error: Dt::get_files() Not a filesystem");
	return (wantarray()) ? () : [];     
    } else {
        if (not $self->{mntobj}->is_mounted()){
            $self->_set_err("Error: Dt::get_files() filesystem is not currently mounted");
            return (wantarray()) ? () : [];
        }
        
        # Get a list of the files produced by "this" dt session.
        $cmd = "ls -1 $self->{path}/" . $self->{name} . "-\* ";
    }

    ($status, @output) = &ClientUtil::execute($self->{parent}->{conn}, $cmd);

    if ($status gt 0) { 
        if ($status eq 1) {
	    $self->_set_err( "Error: Dt::get_files() No files exist to display ", @output);
        } else {
            $self->_set_err("Error: Dt::get_files() Unable to execute command on filesystem \'$self->{path}\'",
			    @output);
        }
        return (wantarray()) ? () : [];       
    } 
    
    foreach my $fullFilePath (@output){
        push @buffer, basename($fullFilePath);
    }

    return (wantarray()) ? @buffer : \@buffer;  

}

################################################################################
# Subroutine Name: get_name
# Objective:       Return Dt object name
# Details:         See POD documentation at the end of this file
################################################################################
sub get_name {
    my ($self) = @_;

    $self->_clear_err();
    
    
    if (defined $self->{name}){
        return $self->{name};
    } else {
        $self->_set_err("Error: Dt::get_name() Dt object is missing name");
        return "";
    }
}

################################################################################
# Subroutine Name: get_pattern
# Objective:       Return pattern used by datatest program.
# Details:         See POD documentation at the end of this file
################################################################################
sub get_pattern {
    my ($self) = @_;

    $self->_clear_err();
    
    if (defined $self->{dt_pattern} and length $self->{dt_pattern}){
        return $self->{dt_pattern};
    } else {
        $self->_set_err("Error: Dt::get_pattern() No pattern set for file creation.");
        return "";
    }
}

###############################################################################
# Method:    io_error()
# Objective: Check if IO error occured
# Details:   See POD documentation at the end of this file.
###############################################################################
sub io_error {
    my ($self)=@_;
    my (@msg,
	$subtest);   

    unless (defined $self->{subtest} and ref($self->{subtest}) eq "Subtest"){
	$self->_set_err("Error: Dt::io_error():  Invalid or missing nate subtest object");
	return $TCase::TRUE;
    }

    $subtest=$self->{subtest};
    
    push @msg, $subtest->results();

    if (grep /error/i, @msg){
	$self->_set_err(@msg);
	return $ClientUtil::TRUE;
    }
    
    return $ClientUtil::FALSE;
}

################################################################################
# Subroutine Name: is_fs
# Objective:       Determine if datatest is running against filesystem
# Details:         See POD documentation at the end of this file
################################################################################
sub is_fs {
    my ($self) = @_;

    $self->_clear_err();

    if (defined  $self->{fs} and $self->{fs} !~ /raw/i){
        return $ClientUtil::TRUE;
    } else {
        return $ClientUtil::FALSE;
    }
}

################################################################################
# Subroutine Name: _new ()
# Objective:       Create a new datatest object 
# Details:         See POD documentation at the end of this file
################################################################################
sub _new {
    my ($class,%args)=@_;
    my (
	$alias,
	$block,
	$brief,
	$cmd,
	$compare,
	$connect,
	$disable,
	$dispose,
	$errors,
	$file,
	$limit,
	$log,
	$mgr,
	$mntObj,
	$mntProp,
	$name,
	$parent,
	$pattern,
	$procs,
	$runid,
	$runtime,
	$seconds,
	$status,
	$time,
	$tm,
	$verbose,
	@buffer,
	@output,
	);
    
    my $self = {};

    bless $self, $class;  

    # This methods will be forked into the background.  Make sure
    # no logging to STDOUT is done in this method.  Otherwise a 2nd
    # set of messages will be duplicated when the fork finishes if 
    # program streamed to STDOUT.  All messaging should be buffered
    # and pass back to the parent process for displaying.

    # defaults
    #$runtime="5s";
    $verbose=0;
    $procs="1";
    $limit="2m";
    $block="128k";
    $errors="1";
    $compare="1";
    $disable="";
    $dispose=1;
    $file="";
    $pattern="0x39c39c39c";

    # parse arguments
    foreach (keys %args) {
	
	if (/-?runtime$/) {
	    if (not length($runtime = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-runtime'");
		return $self;
	    }        
	    next;
	}
	elsif (/-?procs$/) {
	    if (not length($procs = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-procs'");
		return $self;
	    }
	    next;
	}
	elsif (/-?iomgr$/) {
	    if (not length($mgr = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-iomgr'");
		return $self;
	    }
	    next;
	}
	elsif (/-?limit$/) {
	    if (not length($limit = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-limit'");
		return $self;
	    }
	    next;
	}
	elsif (/-?dispose$/) {
	    if (not length($dispose = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-dispose'");
		return $self;
	    }
	    next;
	}
	elsif (/-?block$/) {
	    if (not length($block = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-block'");
		return $self;
	    }
	    next;
	}
	elsif (/-?errors$/) {
	    if (not length($errors = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-errors'");
		return $self;
	    }
	    next;
	}
	elsif (/-?pattern$/) {
	    if (not length($pattern = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-pattern'");
		return $self;
	    }
	    $self->_set_pattern($pattern);
	    next;
	}
	elsif (/-?mntobj$/) {
	    if (not length($mntObj = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-mntobj'");
		return $self;
	    }
	    next;
	}
	elsif (/-?file$/) {
	    if (not length($file = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-file'");
		return $self;
	    }
	    next;
	}
	elsif (/-?name$/) {
	    if (not length($name = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-name'");
		return $self;
	    }
	    next;
	}
	elsif (/-?verbose$/) {
	    if (not length($verbose = $args{$_})) {
		$self->_set_err("Error: Dt::_new() Invalid value for argument '-verbose'");
		return $self;
	    }
	    next;
	}
	elsif (/-?loghdl$/) {
	    $log=$args{$_} if (defined $args{$_} and length($args{$_}));                  
	    next;
	}
	else {
	    $self->_set_err("Error: Dt::_new() Invalid parameter specified \'$_\'");
	    return $self;                            
	}
    }  

    $self->_set_pattern($pattern);

    $self->{log} = $log if (defined $log);

    unless ((defined $mntObj)or(not ref($mntObj))) {
	$self->_set_err("Error: Dt::_new() Missing or invalid required argument '-mntobj'");
	return $self;
    }

    unless (defined $name and length $name) {
	$self->_set_err("Error: Dt::_new() Missing required argument '-name'");
	return $self;
    }

    unless (defined $mgr) {
	$self->_set_err("Error: Dt::_new() Missing required argument '-iomgr'");
	return $self;
    }

    $self->{mntobj} = $mntObj;
    $self->{parent} = $mntObj->{parent};
    $self->{iomgr} = $mgr;

    $mntProp = $mntObj->get_properties();

    $self->{path} = $mntProp->{path};
    $self->{name} = $name;
    $self->{type} = "datatest";

    $self->{hosttype} = $self->{parent}->{conn}->{hostp}->{hosttype};
    $self->{hostname} = $self->{parent}->{conn}->{hostp}->{hostname};

    # create a seperate connection to manage I/O activity  
    #$self->{connect} = $self->{parent}->{conn};

    $self->{fs} = $mntProp->{type};

    $self->{outfile} = ($file eq "") ? $self->{path} : $self->{path} . "/" . $file;

    # Lower the priority of io generation so that other telnet sessions have a 
    # chance to log on.  Otherwise, you will recieve telnet session timeouts
    # if the system is heavily loaded.

    # initialize path to dt based on architecture.
    # Note: DT is issued as a forked process.  Lowering the priority on this
    # may result in expected data not being available to the parent process 
    # when expected.  For example, DT is asked to create a file.  Normally,
    # this take seconds.  However, if the priority is lowered it may start 
    # up while parent is looking for results.  A fix is needed where parent
    # can wait on dt xaction.   

    if ($self->{hosttype} =~ /unix-solaris/i) {
        $self->{datatest} = "/x/eng/localtest/arch/sparc-sun-solaris8/bin/dt ";
    } 
    elsif ($self->{hosttype} =~ /unix-hpux/i) {
        $self->{datatest} = "/x/eng/localtest/arch/hppa-hp-hpux11/bin/dt ";
    }
    elsif ($self->{hosttype} =~ /unix-aix/i) {
	$self->{datatest} = "/x/eng/localtest/arch/powerpc-ibm-aix5.1/bin/dt ";
    }
    elsif ($self->{hosttype} =~ /unix-linux/i) {        
        $self->{datatest} = "nice -n 15 /x/eng/localtest/arch/i386-redhat-linux7/bin/dt ";
    }
    else {
	$self->_set_err("Error: Dt::_new() Hosttype \'$self->{hosttype}\' unknown");
	return $self;
    }

    # Add a 1 second delay between opening and closing device
    $self->{datatest} .= "edelay=1";

    # determine if read/write comparison are to be performed
    $disable="compare,verify" if ($compare=~/0|disable|off/i);

    $dispose=($dispose =~ /1|delete|d|t|true|y|yes/i) ? "delete" : "keep";
    
    if ($self->{fs} =~ /raw/i) {
	# IMPORTANT: Need to save the write limit in the object for 
	# a raw device.  Needed by the verify() method.  Otherwise 
	# the compare will be performed across whole device and fail.
	$self->{raw_limit} = $limit; 
	push @output,("starting i/o on raw volume $self->{path}") if ($verbose);        
    }
    else {
	push @output,("starting i/o on filesystem $self->{path}") if ($verbose);    
    }
    
    $cmd  =  $self->{datatest}." ".
	"of=".$self->{outfile}." ".
	    "limit=".$limit." ".
		"bs=".$block." ".
		    "procs=".$procs." ".
			"oncerr=abort ".
			    "disable=pstats ".
				"errors=".$errors." ".
				    "dispose=".$dispose." ".
					"disable=".$disable." ".
					    "pattern=\"".$pattern."\" ";    

    $cmd .= "runtime=$runtime" if (defined $runtime and length($runtime));
    
    # Add a return at end of cmd so datatest is executed and is not effected
    # by additional cmds tacked on the end.  
    $cmd .= "\\\n";

    if ($verbose) { 
	push @output,
	"DataTest I/O:",
	"name:     ".$name,
	"fs:       ".$self->{fs},
	"procs:    ".$procs,
	"limit:    ".$limit,    
	"block:    ".$block,
	"errors:   ".$errors,
	"dispose:  ".$dispose,
	"disable:  ".$disable,
	"pattern:  \"".$pattern."\"";

	push @output,"runtime:  ".$runtime if (defined $runtime and length($runtime));  
    }
    else {
	$brief = (defined $runtime and length $runtime)? "BG_IO(" : "IO(";
	$brief .= $self->{fs}.") NAME:$name  PROCS:$procs  LIMIT:$limit  DISPOSE:$dispose ";
	$brief .= "RUNTIME:$runtime " if (defined $runtime and length($runtime));
    }

    # provide time estimate when subtest should finish
    if (defined $runtime and length $runtime ){
	$seconds = $self->_runtime_to_seconds($runtime);
	return $self if ($self->error);
	$tm = localtime();
	$time = timegm($tm->sec, $tm->min, $tm->hour,$tm->mday , $tm->mon, $tm->year );
	if ($verbose){
	    push @output, strftime("io_start time:     %Y-%b-%d %H:%M:%S", gmtime($time));
	    push @output, strftime("io_end (expected): %Y-%b-%d %H:%M:%S", gmtime($time+$seconds));
	}
	else {
	    $brief.=strftime("END: %Y-%b-%d %H:%M:%S",gmtime($time+$seconds));
	}
    }

    if ($verbose){
        push (@output, "");
    } else {
        push @output, $brief;
    }

    $self->{subtest}= $self->_execute_background(
						 -name=>$self->{name},
						 -cmd=>$cmd
						 );
    
    # Store PID of subtest
    $self->{pid} = $self->{subtest}->pid();
    
    $self->{output}=\@output;

    return $self;
    
}

################################################################################
# Subroutine Name: is_alive()
# Objective:       check to see if volume has acivity.
# Details:         See POD documentation at the end of this file
################################################################################
sub is_alive {
    my ($self, %args) = @_;

    if (not defined $self->{subtest}){
        return $ClientUtil::FALSE;              
    }

    return $self->{subtest}->isAlive();
}

################################################################################
# Subroutine Name: verify()
# Objective:       Verify file content on volume
# Details:         See POD documentation at the end of this file
################################################################################
sub verify {
    my ($self,%args)=@_;
    my (
	$cmd,
	$file,
	$fs,
	$limit,
	$mount,
	$name,
	$pattern,
	$size,
	$status,
	@output,
	);

    $self->_clear_err();

    # Get hex pattern used as content for file writes.
    $pattern=$self->get_pattern();
    return if ($self->error);

    # parse arguments
    foreach (keys %args) {
	if (/-?file$/) {
	    if (not length($file = $args{$_})) {
		$self->_set_err("Error: Dt::vserify() Invalid value for argument '-file'");
		return $ClientUtil::FALSE;
	    }        
	    next;
	}
	else {
	    $self->_set_err("Error: Dt::verify() Invalid parameter specified \'$_\'");
	    return $ClientUtil::FALSE;                            
	}
    }  

    # verify that dt has a device path
    unless (defined $self->{path}){
        $self->_set_err("Error: Dt::verify() No path to filesystem or raw device exists");
        return $ClientUtil::FALSE;
    }

    # Compare file patterns using proper datatest 
    # format depending on whether volume is raw or 
    # contains a filesystem.
    $cmd  =  $self->{datatest} . " ";

    if ($self->{fs} !~ /raw/i) {
        unless (defined $file and length $file){
	    $self->_set_err("Error: Dt::verify()",
			    "Required parameter missing or invalid for a volume",
			    "with a filesystem '-file'");
	    return $ClientUtil::FALSE;
        }

        unless ($self->{mntobj}->is_mounted){
	    $self->_set_err("Error: Dt::verify() File system is currently not mounted");                                      
	    return $ClientUtil::FALSE;
        }

        $cmd .= "if=".$self->{path} . "/" . $file . " pattern=\"$pattern\"";
    } else {
	$limit = $self->{raw_limit};
	$cmd .= "if=" . $self->{path} . " procs=1 bs=128k limit=$limit pattern=\"$pattern\"";   
    }

    ($status, @output) = &ClientUtil::execute($self->{parent}->{conn}, $cmd); 

    if ($status ne 0 and $status ne 254) {
        $self->_set_err( "Error: Dt::verify() Data pattern mismatch ".$self->{name},
                         "Status: ".$status. " ",
                         @output);
	
        return $ClientUtil::FALSE;                       
    } 

    return $ClientUtil::TRUE;
    
}

################################################################################
##
## P R I V A T E   S U B R O U T I N E S
##
################################################################################

################################################################################
# Method:    _clean
# Objective: Remove content generated by datatest processes
# Details:   See POD documentation at the end of this file
################################################################################
sub _clean {
    my ($self)=@_;   
    my ($cmd,
        $mount_dir,
        $status,        
        @output);
    
    $self->_clear_err();   

    if ($self->{fs} =~ /raw/i){
	# For raw devices write the word 'clean' to the first block
        if ($self->{hosttype} =~ /unix-solaris/i) { 
	    $cmd = $self->{datatest}." of=$self->{path} limit=512 procs=1 pattern='clean'";
        } 
	elsif ($self->{hosttype} =~ /unix-hpux/i) {
	    $cmd = $self->{datatest}." of=$self->{path} limit=1024 bs=1024 procs=1 pattern='clean'";
        }
	elsif ($self->{hosttype} =~ /unix-aix/i) {
	    $cmd = $self->{datatest}." of=$self->{path} limit=512 procs=1 pattern='clean'";
        }
	elsif ($self->{hosttype} =~ /unix-linux/i) {
	    $cmd = $self->{datatest}." of=$self->{path} limit=512 procs=1 pattern='clean'";
        }

    } else {
	if (not $self->{mntobj}->is_mounted()){
	    $self->_set_err("Error: Dt::clean() Filesystem is not currently mounted");
	    return (wantarray()) ? () : [];
	}
	
	$cmd="rm ".$self->{outfile}."-\*";
    }

    ($status, @output) = &ClientUtil::execute($self->{parent}->{conn}, $cmd);

    # Verify that files have been successfully removed
    $cmd = "ls $self->{outfile}"."-\*";

    ($status, @output) = &ClientUtil::execute($self->{parent}->{conn}, $cmd);

    unless ($output[0] =~ /no such file/i) {
        $self->_set_err("Error: Dt::clean() Unable to remove files from filesystem \'$self->{path}\'", $cmd,
			@output);
	
	return $ClientUtil::FALSE;      
    } 
    
    return $ClientUtil::TRUE;  
}

################################################################################
# Subroutine Name: _connect()
# Objective:       Create new telent connection
# Details:         See POD documentation at the end of this file
################################################################################
sub _connect {
    my($self, $hostname) = @_;
    my($connect,
       $conn_str,
       $errmsg,
       $host,
       $id);

    # Variable init.
    $self->{connected} = 0;

    $host = Utils::make_host($hostname); 
    
    $id = $Dt::_CLIENT_CNT++;

    $conn_str=$hostname."#dt_".$id.";conntype=ssh-persist;".
        "username=".$host->{connection_fields}{ssh}{username}.";".
            "password=".$host->{connection_fields}{ssh}{password}.";".
                "timeout=60;"; 

    $connect = Tharn::connect($conn_str,
			      -eval=>sub{
				  if (@_) {
				      my $errmsg = join("",@_);
				      $errmsg =~ s/\n//g;
				      $self->_set_err($errmsg);

				      # Display an error in case user doesn't include error 
				      # handler on object instantiation.  
				      $self->_log("ERROR: Dt::_connect() unable to connect to client '".$hostname."'");

				      Tharn::logresult("CONFIG",abort=>0,"\n@_");    

				      return $self;
				  }
			      });


   

    $self->{connected} = 1;

    return $connect;
}

################################################################################
# Subroutine Name: _convert_to_bytes()
# Objective:       convert size strings to bytes.
# Details:         See POD documentation at the end of this file
################################################################################
sub _convert_to_bytes {
    my ($self,$size)=@_;
    my (
        $bytes,
        $nbr,
        $unit
	);
    # If no units then mb 
    $size .= "m" if ($size=~/^\d+$/);

    ($nbr,$unit)=$size=~/(\d+)(\w+)/;
    
    if ($unit =~/k/i) { 
	$bytes = $nbr * 1024;
    } 
    elsif ($unit =~/m/i){
	$bytes = $nbr * 1024 * 1024;
    }
    elsif ($unit=~/g/i){
	$bytes = $nbr * 1024 * 1024 * 1024;
    }
    
    return $bytes;
}

################################################################################
# Subroutine Name: _execute_background()
# Objective:       execute in background
# Details:         See POD documentation at the end of this file
################################################################################
sub _execute_background {
    my ($self,%args)=@_;
    my (
	$cmd,
	$name,
	$subtest,
	);

    foreach (keys %args) {
	if (/-?cmd$/) {
	    if (not length($cmd = $args{$_})) {
		$self->_set_err("Error: Dt::_execute_background() Invalid value for argument '-cmd'");
		return $self;
	    }
	    next;
	}
	elsif (/-?name$/) {
	    if (not length($name = $args{$_})) {
		$self->_set_err("Error: Dt::_execute_background() Invalid value for argument '-name'");
		return $self;
	    }
	    next;
	}
	else {
	    $self->_set_err("Error: Dt::_execute_background() Invalid parameter specified \'$_\'");
	    return $self;                            
	}
    }  

    $subtest = Tharn::subtest( 
			       sub {
				   my (
				       $connect,
				       $status,
				       @output,
				       @errors,
				       );
				   
				   $self->{log} = undef;
				   $status = 0;
				   $connect = $self->_connect($self->{hostname});
				   if (not defined $status or $status gt 0) {
				       $self->_set_err( "Error: Dt::_execute_background() Error creating connection handle");
				       Tharn::tstlog(-fail, $self->errmsg());                           
				   }
				   ($status, @output) = &ClientUtil::execute($connect, "$cmd"); 
				   if (not defined $status or $status gt 0) {
				       @errors = grep{$_ =~ /errno/} @output;
				       $self->_set_err( "Error: Dt::_execute_background() ($status) ".
							"Background process '$name' failed", @errors);
				     Tharn::tstlog(-fail, $self->errmsg());                           
				   }
			       },
                               -runid=>$name,
                               -bg,
                               -onexit=>sub{}
                          );      
    
    return $subtest;
}

###############################################################################
# Method:    _log
# Objective: Log test output to stdout and file
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _log {
    my ($self,@line) = @_;
    my ($bFirst,
	$dash);
    
    $bFirst=1;
    
    if (defined $self->{log}){
	
	if ($line[0] eq "") {
	    print {${$self->{log}}} "\n";
	return;
	}
	
	foreach my $line (@line){
	    $dash = ($bFirst == 1) ? "- " : "  ";
	    $bFirst=0;
	    
	    print {${$self->{log}}} "   ".$dash.$line."\n";
	}
	
	Tharn::tstlog(-comment,@line);
    }
}

################################################################################
# Subroutine Name: _runtime_to_seconds
# Objective:      Convert runtime string to seconds
# Details:        See POD documentation at the end of this file
################################################################################
sub _runtime_to_seconds {
    my ($self,$runtime)=@_;
    my ($nbr,
	$seconds,
	$unit);

    $seconds=0;

    ($nbr,$unit)=$runtime=~/^(\d+)(\w+)/;

    unless (defined $nbr and defined $unit and $unit =~ /^s|^m|^h|^d/i){
	$self->_set_err("Error: Dt::_runtime_to_seconds() Invalid runtime format '$runtime'");
	return $ClientUtil::FALSE;
    }

    if ($unit =~ /^s/i) {
	$seconds=$nbr;
    }
    elsif ($unit =~ /^m/i) {
	$seconds=$nbr*60; 
    }
    elsif ($unit =~ /^h/i){
	$seconds=$nbr*3600; 
    }
    elsif ($unit =~ /^d/i){
	$seconds= $nbr*3600*24;
    }
    
    return $seconds;
}

################################################################################
# Subroutine Name: _set_pattern()
# Objective:       Pattern used to generate content
# Details:         See POD documentation at the end of this file
################################################################################
sub _set_pattern {
    my ($self,$pattern) = @_;

    if (defined $pattern and length $pattern){
        $self->{dt_pattern} = $pattern;
    }
    else {
        $self->_set_err("Error: Dt::_set_pattern() No pattern was supplied.");
        return 0;
    }

    return 1;
}

################################################################################
# Subroutine Name: _stop()
# Objective:       stop IO on volume
# Details:         See POD documentation at the end of this file
################################################################################
sub _stop {
    my ($self) = @_;
    my (
	$subtest,
	);

    $self->_clear_err();

    $self->_log("Stopping I/O on subtest ".$self->{subtest}->runid());  
    
    $self->{subtest}->kill(complain=>["-ok"]);

    sleep(3);

    if ($self->io_error()) {
	$self->_log("IO ERROR OCCURRED - DISCONNECTING");

	return $ClientUtil::FALSE; 
    }
    
    return $ClientUtil::TRUE;
}

1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

Dt.pm

=head1 Authors

=begin html 

Gary Knopp

=end html

=head1 Description

Dt provides an API interface to the Datatest utility through the use of the Telnet library.  

=head1 See Also

Package inherits from the following modules. See modules for interface detail

B<Error.pm>

=head1 Public Methods

=head2 B<clean()>

Remove file content produced by DataTest session.

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set,
errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Examples

    $Test->log("Remove volume content created by DataTest object");
    if (not $dtObj->clean()){
        $Test->log($dtObj->errmsg);
    }   

=back

=head2 B<display()>

Display information about the DataTest job

=over 4

=item Inputs

none

=item Outputs

Returns an array or array reference of DataTest job information if successful.  If an error occured the error 
flag is set, errmsg is set to the failure reason, and 0 is passed back as the return value.


=item Examples

    $Test->log("-- Retrieving volumeinformation");

    $Test->log($dtObj->display());
    if ($dtObj->error){
        $Test->log($dtObj->errmsg());
        return $TCase::FAIL;
    }

=back

=head2 B<error()>

This method allows you to determine if any errors occured in the
previous method call.  Note that the error flag is reset at the
beginning of each method, so it should be checked after B<every>
major method call.  If you do not check for errors, your calling
program may fail at some point and you will not know why.

=over 4 

=item Inputs

None

=item Outputs

Returns 1 if an error occured in the previous call, 0 otherwise.

=item Examples

 # Determine if error occurred...
 if ($obj->error) {
    print "Error encountered in previous method call\n";
 }

=back

=head2 B<errmsg()>

This method allows you to get the text of the error message if an error
occurred on the previous method call.

=over 4

=item Inputs

None

=item Outputs

Returns the text of the last error message set.  If no error message is
set, an empty string will be returned.

=item Examples

 # Check for errors.
 if ($obj->error) {
    $errmsg = $obj->errmsg;
    print "An error occurred: $errmsg\n";
 }

=back

=head2 B<get_files()>

Retrieves the list of files created by the DataTest object.  An error will occur if this method 
is called on a raw device.

=over 4

=item Inputs

none

=item Outputs

Returns an array or array reference of file names created by DataTest if successful. 
If an error occurs the error flag is set, errmsg is set to the failure reason, and an 
empty array is returned.        

=item Examples

    $Test->log("-- Retrieving files");
    @files=$dtObj->get_files();
    if ($dtObj->error){
        $Test->log($dtObj->errmsg());
        return $TCase::FAIL;
    }  
    

=back

=head2 B<get_name()>

Name of DataTest job.   This name is unique for each DataTest object created.

=over 4

=item Inputs

none

=item Outputs

Returns the name of the DataTest job.  If an error occurs the error flag 
is set, errmsg is set to the failure reason, and "" is returned.        


=item Examples

    print "Datatest job name is '".$dtObj->get_name()."'";

=back

=head2 B<get_pattern()>

Retrieve pattern that will be used to generate file or raw content 

=over 4

=item Inputs

none

=item Outputs

Returns a scalar containing the pattern if successful.  If an error occurs the error flag 
is set, errmsg is set to the failure reason, and "" is returned.        


=item Examples

    print "Pattern used to generate data content is   '".$dtObj->get_pattern()."'";

=back

=head2 B<io_error()>

If the I/O DataTest program is allowed to complete to input runtime specification
without the user issuing io_stop() this method would need to be called to check 
for errors.

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if io errors occured, otherwise 0. If an error occurred the 
errmsg is set to the failure reason.  

=item Examples

        $Test->log("-- Stopping I/O");                                          
        
        sleep 5 while ($dtObj->is_alive());

        if ($dtObj->io_error) {
            $Test->log($dtObj->errmsg());
            return $TCase::FAIL;
        }

=back

=head2 B<io_stop()>

Stop I/O on filesystem or raw device that is currently running

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if the program was stopped sucessfully, otherwise 0. If an error occurs the
error flag is set, errmsg is set to the failure reason.  Its important to note that stopping I/O will 
terminate the Dt objects connection to the remote host.  Using Dt methods that involve requests
of the remote host after an io_stop(), such as get_files(), will fail.

=item Examples

        $Test->log("-- Stopping I/O");                                          
        $dtObj->io_stop();
        if ($dtObj->error) {
            $Test->log($dtObj->errmsg());
            return $TCase::FAIL;
        }

=back

=head2 B<is_alive()>

Determine if DataTest i/o si running in the background.

=over 4

=item Inputs

None

=item Outputs

Returns a value of 1 if DataTest i/o program is still running, otherwise 0.  If an error occured the error flag is set, errmsg
is set to the failure reason.

=item Examples

    # Check if DataTest I/O has completed on the volume 
    # every 5 seconds.
    sleep 5 while($dtObj->is_alive());
   

=back

=head2 B<is_fs()>

Determine if DataTest is using a filesystem 

=over 4

=item Inputs

None

=item Outputs

Returns a value of 1 if a filesystem exists, otherwise 0.  If an error occured the error flag is set, errmsg
is set to the failure reason.

=item Examples

    if(not $dtObj->is_fs()){
        ...
        <error handler>
        ...
    }
   

=back

=head2 B<verify(%args)>

Verify that filecontent is in tact and not corrupted after a restore.

=over 4

=item Inputs

=item * -file 

Filename produced by DataTest 

=item Outputs

Returns a value of 1 if the file content matches generated pattern, otherwise 0.  If an error occured 
the error flag is set, errmsg is set to the failure reason.

=item Examples

   
   foreach my $file (@files){
       $dtObj->verify(-file=>$file);
       if ($dtObj->error){
           $Test->log($dtObj->errmsg());
           $fail_cnt++;
       }  
   }

=back


