package Ini;

use strict;
require      Exporter;
my @ISA       = qw(Exporter);
my $status;

sub values {
	my %p = (
		warn_missing_file => 0,
		@_
	);
	#
	# I need these parms 
	# file      -- file to open
	# delimiter -- string to split on
	# vars      -- array reference of variables to find and return
	#
	#Here is an example of how I call it:
	#
	# my ($dogpound, $name, $phone) = inivalues(file=>'/somefile',delimiter=>'=',vars=>[qw/dogpound name phone/]);

	my %found; #keep the values we collect

	map { $found{$_}=undef; } @{ $p{vars} };

	if (-e $p{file}) {
		open(IN,"$p{file}") or warn "Error: can't open '$p{file}' for reading. ($!)\n";
		while (my $oneline=<IN>) {
			my ($var,$val) = split /$p{delimiter}/, $oneline;

			#remove any spaces from the front of the variable and value
			$val=~s/^\s+//;
			$var=~s/^\s+//;

			#remove any spaces from the end of the variable and value
			$val=~s/\s+$//;
			$var=~s/\s+$//;

			#Remove any quotes from the var/val
			$val=~s/['|"]//g;
			$var=~s/['|"]//g;


			if (exists($found{$var})) { #its one we were looking for
				chomp $val;
				$found{$var}=$val;
			}

		}
		close(IN);
	} else {
		warn "$p{file} is missing." if $p{warn_missing_file};
	}

	my @toreturn=();
	foreach my $onevar ( @{ $p{vars} } ) {
		push(@toreturn,$found{$onevar});
	}

	return @toreturn;
}
1; #all modules must return a true value (so this line is required)


###############################################################################
#####################End of program/Start of Documentation#####################
###############################################################################


=pod

=head1 NAME

Ini.pm - A simple library to parse var=val text from ini/rc files.

=head1 METHODS

=over

=item values

Get some number of values from an ini/rc file

=back

=head1 EXAMPLES

=over

use strict;
use Data::Dumper;

use Ini;

my @return = Ini::values(file=>'/u/mgarrett/vsims/mgarrett_vsim_2/DataONTAP.vmx',delimiter=>'=',vars=>[qw/serial0.fileName/]);
print Dumper \@return;

my @return = Ini::values(file=>'/u/mgarrett/vsims/mgarrett_vsim_2/SimBindings',delimiter=>':',vars=>[qw/pconsport constype/]);
print Dumper \@return;

=head1 AUTHORS

The author is Mike Garrett.

=cut
