# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/IronUtil/MPT/commonParams.pm#2 $
#
# $Change: 1927553 $
#
# $DateTime: 2013/01/31 02:12:21 $
#
# Copyright (c) 2012 Network Appliance, Inc.
# All rights reserved.
#
#
## NAME
#      IronUtil::MPT::commonParams.pm - provide support for commonly-used parameters
#
## DESCRIPTION
#       This perl module provide support for common test parameters
#       Which are used by all MPT test scripts.
#
## AUTHOR
#       tarung@netapp.com
#
##
## @status public
## @pod here

=head1 NAME

IronUtil::MPT::commonParams.pm - provide support for managing commonly-used parameters

=head1 SYNOPSIS

  use Tharn;

  BEGIN {
     # set param values here, before loading IronUtil::MPT::commonParams
     param('EXAMPLE_PARAM', -default, 'example_value');
  }

  # load the module
  use IronUtil::MPT::commonParams;

  # this declares the common parameters in the 'main' package, and sets default
  # values for any of them which are undefined.
  my $cp = IronUtil::MPT::commonParams->new(set_mpt_defaults => 1);

=head1 DESCRIPTION

commonParams - this module provides support for managing commonly-used parameters.

=cut

package main;

use strict;

# declare PERL vars in 'main' - please keep the list in alphabetical order
use vars qw(
        $CLEANUP
        $VSERVER
        $TEST_AGGR
        $START_OPS
        $DEBUG_MSG
        $RAID_TYPE
        $AGGR_SIZE
        $VOL_REUSE
        $AGGR_REUSE
        $LARGE_AGGR
        $TEST_METHOD
        $CMD_TIMEOUT
        $IRON_TIMEOUT
        $WAIT_TIMEOUT
        $TEST_CONTINUE
        $APISET_TIMEOUT
        $MAX_SCAN_SPEED
        $MARK_PSEUDOBAD
        $MARK_UNVERIFIED
        $LDC_DISABLED
        $START_IO_OPS
        $WRECK_TIMEOUT
        $MPT_DISABLED
        $DSC_DISABLED
        $SIS_ON
        $COMPR_ON
	    );

package IronUtil::MPT::commonParams;

use strict;
use warnings;

use Carp qw(croak);

use Tharn;

use Params::Validate qw(validate SCALAR);

sub new {
    my $class = shift;
    $class = ref($class) || $class;

    # validate args
    my (%args) = validate (
	@_,
	{
	    set_mpt_defaults => {
		type    => SCALAR,
		default => 0,
	    },
	}
    );

    my $self = bless {}, $class;

    if ( $args{set_mpt_defaults} ) {
        $self->_set_mpt_defaults();
    }

    return $self;
}

sub _set_mpt_defaults {
    my $self = shift;

    # set params in package 'main', so the param refs below can be resolved properly
    package main;

    param('VSERVER',               '-default', 'vs0');
    param('CLEANUP',               '-default', 'test');
    param('RAID_TYPE',             '-default', 'raid_dp');
    param('TEST_AGGR',             '-default', 'MptAggr');
    param('START_OPS',             '-default', 1);
    param('DEBUG_MSG',             '-default', 0);
    param('VOL_REUSE',             '-default', 0);
    param('AGGR_REUSE',            '-default', 1);
    param('LARGE_AGGR',            '-default', 1);
    param('TEST_METHOD',           '-default', 'IRON');
    param('CMD_TIMEOUT',           '-default', 600);
    param('WAIT_TIMEOUT',          '-default', 600);
    param('IRON_TIMEOUT',          '-default', 3600);
    param('TEST_CONTINUE',         '-default', 1);
    param('MAX_SCAN_SPEED',        '-default', 10000);
    param('APISET_TIMEOUT',        '-default', 300);
    param('MARK_PSEUDOBAD',        '-default', 0);
    param('MARK_UNVERIFIED',       '-default', 0);
    param('LDC_DISABLED',          '-default', 0);
    param('MPT_DISABLED',          '-default', 0);
    param('DSC_DISABLED',          '-default', 0);
    param('START_IO_OPS',          '-default', 1);
    param('WRECK_TIMEOUT',         '-default', 3600);
    param('SIS_ON',                '-default', 0 );
    param('COMPR_ON',              '-default', 0 );

}

1;

=head1 SEE ALSO

    None

=head1 STATUS

    Shared

=head1 AUTHOR

    Tarun Garg (tarung@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/IronUtil/MPT/commonParams.pm#2 $
    Last $Change: 1927553 $
    Last Revised $DateTime: 2013/01/31 02:12:21 $

=head1 HISTORY

    2012-01-20 author: tarung      original lib created

=head1 COPYRIGHT

Copyright 2012, Network Appliance., All Rights Reserved

=cut
