# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/JobHelper.pm $
#
# Copyright (c) 2010 NetApp, Inc.
# All rights reserved.
#
#
# NAME
#      JobHelper.pm - provide interfaces to C-Mode job cli command 
#
# DESCRIPTION
#      This perl module provide wrapper interfaces to c-mode job cli command
#
#
# AUTHOR
#       yatin@netapp.com
## @pod here
#
#

our ($Validation, $VERSION);
use warnings;
use strict qw(vars subs refs);
use Tharn;
use Data::Dumper;
use Params::Validate     qw(validate validate_pos SCALAR ARRAYREF
                            SCALARREF OBJECT HASHREF CODEREF);

use vars qw(@ISA );

=head2 job_history_show

show the jobs histroy

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub job_history_show {
 
    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1, 
        );

    my %args_validn_spec = 
        (
            server      => {type => OBJECT},
            node        => {type => SCALAR, optional => 1},
            record      => {
                            type => SCALAR,
                            optional => 1,
                            regex => qr/^(\d+)$/,
                           },
            id          => {type => SCALAR, optional => 1},
            endtime     => {type => SCALAR, optional => 1},
            starttime   => {type => SCALAR, optional => 1},
            name        => {type => SCALAR, optional => 1},
            description => {type => SCALAR, optional => 1},
            code        => {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(\d+)$/,
                           },
            completion  => {type => SCALAR, optional => 1},
            cli_op      =>  "abc",
            timeout     => {
                            type => SCALAR,
                            optional =>1,
                            default  => 300,
                           },
        );
                        

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    my @out;
    my $output;
    
    eval {
        @out=$server->job_history_show( %input_arg );
        $output = Dumper(@out);
        $Log->log("Raw output for job history show : $output");
    };

    if($@) {
        my $err_ref = "job_history_show() failed, error: $@";
        return undef;
    }
    return $output;
}


sub job_schedule_cron_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            Timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR, optional => 1},
            month       => {type => SCALAR, optional => 1},
            dayofweek   => {type => SCALAR, optional => 1},
            day         => {type => SCALAR, optional => 1},
            hour        => {type => SCALAR, optional => 1},
            minute      => {type => SCALAR, optional => 1},
            description => {type => SCALAR, optional => 1},
            Timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'Timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    my $out;
    
    eval {
        $out=$server->job_schedule_cron_show( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        $Log->log("job_schedule_cron_show() failed, error: $@");
        return undef;
    }
    return 1;
}

=head2 job_schedule_cron_create

create the cron job schedule 

=over 4

=item Arguments:

=over 4

takes a hash as input (see %args_validn_spec for valid keys)

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub job_schedule_cron_create {

    my %invalid_cmd_keys =
        (
            server  => 1,
            Timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR},
            month       => {type => SCALAR, optional => 1},
            dayofweek   => {type => SCALAR, optional => 1},
            day         => {type => SCALAR, optional => 1},
            hour        => {type => SCALAR, optional => 1},
            minute      => {type => SCALAR},
            Timeout     => {
                            type => SCALAR,
                            optional =>1,
                            default  => 60,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'Timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    my $out;
    
    eval {
        $out=$server->job_schedule_cron_create( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        $Log->log("job_schedule_cron_create() failed, error: $@");
        return undef;
    }
    return 1;
}

=head2 job_schedule_cron_delete

deletes the cron job schedule by name

=over 4

=item Arguments:

=over 4

takes a hash as input with following key
    server  => server object
    name    => cron job name
    errmsg => referece to error variable
    timeout => timeout in seconds [60]

=back

=item Returns:

Returns 1 if successful, otherwise 0

=back

=cut

sub job_schedule_cron_delete {
    my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );
    
    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR},
            timeout     => {
                            type => SCALAR,
                            optional =>1,
                            default  => 60,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
     
    my $out;
    
    eval {
        $out=$server->job_schedule_cron_delete( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        my $err_ref = "job_schedule_cron_delete() failed, error: $@";
        $Log->log($err_ref);
        return undef;
    }
    return 1;
}


sub job_schedule_cron_modify {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR},
            month       => {type => SCALAR, optional => 1},
            dayofweek   => {type => SCALAR, optional => 1},
            day         => {type => SCALAR, optional => 1},
            hour        => {type => SCALAR, optional => 1},
            minute      => {type => SCALAR, optional => 1},		
            timeout     => {
                            type => SCALAR,
                            optional =>1,
                            default  => 60,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    my $out;
    
    eval {
        $out=$server->job_schedule_cron_modify( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        my $err_ref = "job_schedule_cron_modify() failed, error: $@";
        $Log->log($err_ref);
        return undef;
    }
    return 1;
}


sub job_schedule_interval_create {

    my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR},
            days        => {type => SCALAR, optional => 1},
            hours       => {type => SCALAR, optional => 1},
            minutes     => {type => SCALAR, optional => 1},
            seconds     => {type => SCALAR, optional => 1},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    
    my $out;
    
    eval {
        $out=$server->job_schedule_interval_create( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        my $err_ref = "job_schedule_interval_create() failed, error: $@";
        $Log->log($err_ref);
        return undef;
    }
    return 1;
}

sub job_schedule_interval_delete {
    
    my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );
        
    my %options = validate(@_,\%args_validn_spec);
    
    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
     
    my $out;
    
    eval {
        $out=$server->job_schedule_interval_delete( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        my $err_ref = "job_schedule_interval_delete() failed, error: $@";
        $Log->log($err_ref);
        return undef;
    }

    return 1;
}



sub job_schedule_interval_modify {

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR},
            days        => {type => SCALAR, optional => 1},
            hours       => {type => SCALAR, optional => 1},
            minutes     => {type => SCALAR, optional => 1},
            seconds     => {type => SCALAR, optional => 1},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }

    my $out;
    
    eval {
        $out=$server->job_schedule_interval_modify( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        my $err_ref = "job_schedule_interval_modify() failed, error: $@";
        $Log->log($err_ref);
        return undef;
    }
    return 1;
}


sub job_schedule_interval_show {

    my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR, optional => 1},
            instance    => {type => SCALAR, optional => 1},
            days         => {type => SCALAR, optional => 1},
            hours        => {type => SCALAR, optional => 1},
            minutes      => {type => SCALAR, optional => 1},
            seconds     => {type => SCALAR, optional => 1},
            description => {type => SCALAR, optional => 1},
            fields      => {type => SCALAR, optional => 1},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    my $out;

    eval {
        $out = $server->job_schedule_interval_show( %input_arg );
        $Log->log("Job schedule cron show:". Dumper($out));
    };

    if($@) {
        my $err_ref = "job_schedule_interval_show() failed, error: $@";
        return undef;
    }
    return 1;
}

sub job_delete {
    my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            id          => {type => SCALAR},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );
        
    my %options = validate(@_,\%args_validn_spec);
    
    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
     
    my $out;
    
    eval {
        $out=$server->job_delete( %input_arg );
        $Log->log("Job delete" . Dumper($out));
    };

    if($@) {
        my $err_ref = "job_pause() failed, error: $@";
        return undef;
    }
    return 1;
}


sub job_pause {
    
    my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            id          => {type => SCALAR},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );
        
    my %options = validate(@_,\%args_validn_spec);
    
    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
     
    my $out;
    
    eval {
        $out=$server->job_pause( %input_arg );
        $Log->log("Job pause" . Dumper($out));
    };

    if($@) {
        $Log->log("Failed to pause a Job");
        return undef;
    }
    return 1;
}


sub job_resume {
        my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            id          => {type => SCALAR},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );
        
    my %options = validate(@_,\%args_validn_spec);
    
    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
     
    my $out;
    
    eval {
        $out=$server->job_resume( %input_arg );
        $Log->log("Job resume" . Dumper($out));
    };

    if($@) {
        my $err_ref = "job_resume() failed, error: $@";
        return undef;
    }
    return 1;
}


sub job_stop {
        my %invalid_cmd_keys =
        (
            server  => 1,
            timeout => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            id          => {type => SCALAR},
            timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 60,
                           },
        );
        
    my %options = validate(@_,\%args_validn_spec);
    
    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
     
    my $out;
    
    eval {
        $out=$server->job_stop( %input_arg );
        $Log->log("Job stop" . Dumper($out));
    };

    if($@) {
        my $err_ref = "job_resume() failed, error: $@";
        return undef;
    }
    return 1;
}

sub job_watch_progress {
    my %options = validate(@_,
                    {
                        server      => {type => OBJECT},
                        id          => {type => SCALAR},
                        interval    => {
                                        type => SCALAR,
                                        optional => 1,
                                        default  => 60,
                                       },
                        timeout     => {
                                        type => SCALAR,
                                        optional => 1,
                                        default  => 60,
                                       },
                    });

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'timeout'};
    $input_arg{'id'}       = $options{'id'};
    $input_arg{'interval'} = $options{'interval'};


    eval {
        $server->job_watch_progress( %input_arg );
    };

    if($@) {
        my $err_ref = "job_watch_progress() failed, error: $@";
        return undef;
    }

    return 1;
}


sub job_show {

    my @state_list = qw( Initial Queued Running Waiting Pausing
                        Paused Quitting Success Failure Reschedule
                        Error Quit Dead Unknown Restart Dormant );

    my $states = join('|', @state_list);

    my $rgexp_valid_states = qr/^($states)$/;

    my %invalid_cmd_keys =
        (
            server  => 1,
            errmsg  => 1,
            timeout => 1,
            cli_op  => 1, 
        );

    my %args_validn_spec = 
        (
            server      => {type => OBJECT},
            name        => {type => SCALAR, optional => 1},
            id          => {type => SCALAR, optional => 1},
            description => {type => SCALAR, optional => 1},
            priority    => {type => SCALAR, optional => 1},
            node        => {type => SCALAR, optional => 1},
            affinity    => {
                            type => SCALAR,
                            optional =>1,
                            regex => qr/^(Cluster|Node)$/,
                           },

            schedule    => {type => SCALAR, optional => 1},
            queuetime   => {type => SCALAR, optional => 1},
            starttime   => {type => SCALAR, optional => 1},
            endtime     => {type => SCALAR, optional => 1},
            deadtime    => {type => SCALAR, optional => 1},
            restarted   => {type => SCALAR, optional => 1},
            state       => {type => SCALAR, optional => 1},
            code        => {type => SCALAR, optional => 1},
            completion  => {type => SCALAR, optional => 1},
            jobtype     => {type => SCALAR, optional => 1},
            uuid        => {type => SCALAR, optional => 1},
            category    => {type => SCALAR, optional => 1},
            progress    => {type => SCALAR, optional => 1},
	    vserver     => {type => SCALAR, optional => 1},
            username    => {type => SCALAR, optional => 1},	
            Timeout     => {
                            type => SCALAR,
                            optional =>1,
                            default  => 300,
                           },
        );
                        

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object
    my $err_ref  = $options{'errmsg'}; # refernce to error msg variable
    
    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'Timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    
    my $out;    
 my $ret_val;

    eval {
        $server->set(privilege=>"advanced");
        $out = $server->job_show( %input_arg );
        $ret_val=scalar(@{$out});
        $Log->log("Output of job show:". Dumper($out));
        $Log->log("Number of threadtest job running:$ret_val");
    };

    if($@) {
        $Log->log("debug_job_threadtest() failed, error: $@");
        return undef;
    }
    return $ret_val;
}


return 1;


sub debug_job_threadtest {
    
    my %invalid_cmd_keys =
        (
            server  => 1,
        );

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            job_length  => {type => SCALAR, optional => 1},
            local       => {type => SCALAR, optional => 1},
            ex_pri_cnt  => {type => SCALAR, optional => 1},
            mp_pri_cnt  => {type => SCALAR, optional => 1},
            lp_pri_cnt  => {type => SCALAR, optional => 1},
            hp_pri_cnt  => {type => SCALAR, optional => 1},
            comingle    => {type => SCALAR, optional => 1},
            schedule    => {type => SCALAR, optional => 1},
            Timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 300,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'Timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    my $out;
    my $output;
    eval {
        $server->set(privilege=>"test");
        $out = $server->debug_job_threadtest( %input_arg );
        $output = Dumper ($out);
    };

    if($@) {
        my $err_ref = "debug_job_threadtest() failed, error: $@";
        return undef;
    }

    return $output;
}



sub job_private_show {
    my %invalid_cmd_keys =
        (
            server  => 1,
        );
  #yatin-vsim1::*> job private show ?
  #[ -inprogress | -instance | -jobstate | -jobuuid | -sched | -times | -type | -fields <fieldname>, ... ]
  

    my %args_validn_spec =
        (
            server      => {type => OBJECT},
            node        => {type => SCALAR, optional => 1},
            id          => {type => SCALAR, optional => 1},
            name        => {type => SCALAR, optional => 1},
            description => {type => SCALAR, optional => 1},
            priority    => {type => SCALAR, optional => 1},
            schedule    => {type => SCALAR, optional => 1},
            queuetime   => {type => SCALAR, optional => 1},
            starttime   => {type => SCALAR, optional => 1},
            endtime     => {type => SCALAR, optional => 1},
            dropdeadtime => {type => SCALAR, optional => 1}, 
            restarted   => {type => SCALAR, optional => 1},
            state       => {type => SCALAR, optional => 1},
            code        => {type => SCALAR, optional => 1}, 
            completion  => {type => SCALAR, optional => 1},
            jobtype     => {type => SCALAR, optional => 1},
            category    => {type => SCALAR, optional => 1},
            uuid        => {type => SCALAR, optional => 1},
            progress    => {type => SCALAR, optional => 1},
            vserver     => {type => SCALAR, optional => 1},
            username    => {type => SCALAR, optional => 1},
            Timeout     => {
                            type => SCALAR,
                            optional => 1,
                            default  => 300,
                           },
        );

    my %options = validate(@_, \%args_validn_spec);

    my %input_arg; # hash of input argument to cli command

    my $server   = $options{'server'}; # server object

    # set the input arguments to cli command
    $input_arg{'-Timeout'} = $options{'Timeout'};

    foreach my $key ( keys %args_validn_spec ) {
        if ( not defined($invalid_cmd_keys{$key}) ) {
            if ( defined($options{$key}) ) {
                $input_arg{$key} = $options{$key};
            }
        }
    }
    my $out;
    my $ret_val;
    
    eval {
        $server->set(privilege=>"advanced");
        $out = $server->job_private_show( %input_arg );
        $ret_val=scalar(@{$out});
        $Log->log("Output of private show:". Dumper($out));
        $Log->log("Number of threadtest job running:$ret_val");
    };

    if($@) {
        $Log->log("debug_job_threadtest() failed, error: $@");
        return undef;
    }
    return $ret_val;
}


return 1;
