# this is basically -*- perl -*-
# $Id$
#
## @summary Kerberos Host Linux Class
## @author	dl-ssqa,gvg
## @status	public
## @pod here

=head1 NAME

Krb::Host::Linux - Kerberos Host Interface for Linux

=head1 SYNOPSIS

  use Krb::Host;

  # Create Kerberos Host Object
  $khost = new Krb::Host(host   => $linuxhost,
                         domain => $domain,
                         realm  => $DOMAIN,);

  Refer Krb::Host

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-ssqa,gvg
  $Id$
  Last revised $Date$

=head1 COPYRIGHT

  Copyright 2008, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Krb::Host::Linux;
use base qw(Krb::Host);

use strict;
use warnings;

use Tharn qw($Log);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Data::Dumper;

#---------------------------------------------------------------------

sub _new {
    my $class = shift;
    my $self = bless {@_,
                      sudo        => "/usr/software/bin/sudo",
                      retry       => 5,
                      kinit       => "/usr/kerberos/bin/kinit",
                      kdestroy    => "/usr/kerberos/bin/kdestroy",
                      krbconf     => "/etc/krb5.conf",
                      keytab      => "/etc/krb5.keytab",
                      krbhostconf => "/etc/sysconfig/nfs",}, $class;

    return $self;
}

#---------------------------------------------------------------------
sub setup {
    my $self = shift;
    my (%opts) =
      validate(@_,
               {krbconf => {type => SCALAR,},
                keytab  => {type => SCALAR,},});
    $Log->enter();

    # Copy krb conf
    my $cmd =
      $self->{sudo} . " cp -f " . $opts{krbconf} . " " . $self->{krbconf};
    my $out = sayEOF($self->{conn}, $cmd);
    croak "$cmd failed: $out" if ($out =~ /fail|error/i);

    # Copy keytab file
    $cmd = $self->{sudo} . " mv " . $opts{keytab} . " " . $self->{keytab};
    $out = sayEOF($self->{conn}, $cmd);
    croak "$cmd failed: $out" if ($out =~ /fail|error/i);

    $self->_sysconfig_conf();

    # Disable may fail if it is the first time to be configured.
    eval { $self->krb_disable(); };
    $Log->comment("krb_disable failed: $@") if $@;

    $self->krb_enable();

    $Log->exit();
}

#---------------------------------------------------------------------
sub krb_enable {
    my $self = shift;

    my $cmd = $self->{sudo} . " /etc/init.d/rpcgssd start";
    my $out = sayEOF($self->{conn}, $cmd);

    croak "$cmd failed: $out" if ($out =~ /fail|error/i);

    # Fedora doesn't support this
    #croak "$cmd failed: $out" if ($out !~ /Starting RPC gssd:(\s+\S+\s+OK\.*)/i );
}

#---------------------------------------------------------------------
sub krb_disable {
    my $self = shift;

    my $cmd = $self->{sudo} . " /etc/init.d/rpcgssd stop";
    my $out = sayEOF($self->{conn}, $cmd);

    croak "$cmd failed: $out" if ($out =~ /fail|error/i);

    # Fedora doesn't support this
    #croak "$cmd failed: $out" if ( ($out !~ /Shutting down RPC gssd:(\s+\S+\s+OK\.*)/i) or ($out =~ /Stopping RPC gssd:(\s\S+\s+OK\.*)/i) );
}

#---------------------------------------------------------------------
sub _sysconfig_conf {
    $Log->enter();
    my $self          = shift;
    my $nfsConfigFile = '/etc/sysconfig/nfs';

    # I tried to improve this function, but the sysconfig/nfs file reads in a retarded format...
    # go ahead, give it a shot. See what I mean.

    # Make sure the nfs config file has the value SECURE_NFS="yes"
    my $needsChange = 0;
    if (-e $nfsConfigFile) {
        my $optionRequired = 'SECURE_NFS="yes"';

        # Just open for reading, its all you will have permission for
        open(CONFIGFILE, '<', $nfsConfigFile)
          or croak "Unable to open NFS config file $nfsConfigFile";
        my @file = <CONFIGFILE>;
        close CONFIGFILE;
        foreach my $line (@file) {
            if ($line =~ /^$optionRequired$/) {
                last;
            } elsif ($line =~ /^#$optionRequired/) {

                # Just use perl to do the substitute
                my $perlMagic =
                    '/usr/software/bin/perl -p -i.bak -e \'s/#'
                  . $optionRequired . "/"
                  . $optionRequired . '/\' '
                  . $nfsConfigFile;
                my $cmd = $self->{sudo} . " " . $perlMagic;
                my $out = sayEOF($self->{conn}, $cmd);
                croak "$cmd failed: $out" if ($out =~ /fail|error/i);
            }
        }
    }

    return;
}

#---------------------------------------------------------------------

1;
