# this is basically -*- perl -*-
# $Id$
#
## @summary Kerberos Host Solaris Class
## @author	dl-ssqa,gvg
## @status	public
## @pod here

=head1 NAME

Krb::Host::Solaris - Kerberos Host Interface for Solaris

=head1 SYNOPSIS

  use Krb::Host;

  # Create Kerberos Host Object
  $khost = new Krb::Host(host   => $solarishost,
                         domain => $domain,
                         realm  => $DOMAIN,);

  Refer Krb::Host

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-ssqa,gvg
  $Id$
  Last revised $Date$

=head1 COPYRIGHT

  Copyright 2008, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Krb::Host::Solaris;
use base qw(Krb::Host);

use strict;
use warnings;

use Tharn qw($Log);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Data::Dumper;

#---------------------------------------------------------------------

sub _new {
    my $class = shift;
    my $self = bless { 
        @_,
        sudo        => "/usr/software/bin/sudo",
        retry       => 5,
        kinit       => "/bin/kinit",
	kdestroy    => "/bin/kdestroy",
        krbconf     => "/etc/krb5/krb5.conf",
        keytab      => "/etc/krb5/krb5.keytab",
        krbhostconf => "/etc/nfssec.conf",
    }, $class;

    return $self;
}

#---------------------------------------------------------------------
sub setup {
    my $self = shift;
    my (%opts) = validate (
        @_,
        {
            krbconf => { type => SCALAR, },
            keytab  => { type => SCALAR, },
        }
    );
    $Log->enter();

    # Copy krb conf
    my $cmd = $self->{sudo}." cp -f ".$opts{krbconf}." ".$self->{krbconf};
    my $out = sayEOF($self->{conn}, $cmd);
    croak "$cmd failed: $out" if($out =~ /fail|error/i);

    # Copy keytab file
    $cmd = $self->{sudo}." mv ".$opts{keytab}." ".$self->{keytab};
    $out = sayEOF($self->{conn}, $cmd);
    croak "$cmd failed: $out" if($out =~ /fail|error/i);

    $self->_editnfssec_conf();

    $self->krb_disable();
    $self->krb_enable();

    $Log->exit();
}

#---------------------------------------------------------------------
sub krb_enable {
    my $self = shift;

    my $cmd = $self->{sudo}." svcadm -v enable svc:/network/rpc/gss:default";
    my $out = sayEOF($self->{conn}, $cmd);

    croak "$cmd failed: $out" if ($out !~ /svc:\/network\/rpc\/gss:default enabled./);
}

#---------------------------------------------------------------------
sub krb_disable {
    my $self = shift;

    my $cmd = $self->{sudo}." svcadm -v disable svc:/network/rpc/gss:default";
    my $out = sayEOF($self->{conn}, $cmd);

    croak "$cmd failed: $out" if ($out !~ /svc:\/network\/rpc\/gss:default disabled./);
}

#---------------------------------------------------------------------
sub _editnfssec_conf {
    my $self = shift;
    my $tmpconf = "/tmp/nfssecconf.$$";

    my $cmd = $self->{sudo}." cat ".$self->{krbhostconf}." | sed -e 's\/\#\\s\*krb\/krb\/' > ".$tmpconf;
    my $out = sayEOF($self->{conn}, $cmd);
    croak "$cmd failed: $out" if($out =~ /fail|error/i);

    $cmd = $self->{sudo}." mv ".$tmpconf." ".$self->{krbhostconf};
    $out = sayEOF($self->{conn}, $cmd);
    croak "$cmd failed: $out" if($out =~ /fail|error/i);

    return;
}

#---------------------------------------------------------------------

1;
