package Leak;

## @summary Leak detection library
##
## @synopsis
##   require Leak;
##
##   Leak::mem_monitor();
##   <run tests>
##   Leak::mem_check();
##
## @description
##   The Leak library can be used by test scripts to monitor and
##   report on possible memory leaks.  The library consists of the
##   following functions:
##
##       Leak::mem_monitor() - start monitoring appliance memory usage
##
##       Leak::mem_check() - report on possible memory leaks
##
##   The Leak::mem_monitor() function will start a background thread
##   that will periodically check the memory status of the appliance.
##   The Leak::mem_check() function will stop the background thread
##   and then analyze the collected data to see if there appear to be
##   any memory leaks.  A test failure will be generated for each
##   identified leak.
##
##   There can be only one outstanding monitoring request at one
##   time.
##
## @param TOP_LOGDIR Required: path to log file directory.
##
## @param FILER Required: hostspec of appliance to monitor.
##
## @param FILER_CONN Optional: Connection ID to use for filer.
##   Default is "#rsh".
##
## @param LEAK_INTERVAL Optional: Number of seconds to delay between
##   each collection of memory statistics.
##   Default is 300 seconds (5 minutes).
##
## @param LEAK_DELAY Optional: Number of seconds to wait before the
##   first collection of memory statistics.
##   Default is 3 seconds.
##
## @param LEAK_COUNT Optional: Number of times to collect memory
##   statistics.
##   Default is 0, which means collect indefinitely.
##
## @example
##   require Leak;
##
##   leak::mem_monitor();
##   <run tests>
##   leak::mem_check();
## @author kerndev
## @status public

require Exporter;

use strict; 
use Tharn;

use vars('@ISA','@EXPORT_OK');
@ISA = ('Exporter');
@EXPORT_OK = ('mem_monitor', 'mem_check');

my $monitor;

sub mem_monitor
{
    if ($monitor) {
	logresult('SCRIPT', "Memory monitoring already started.");
	return;
    }
    $monitor = subtest("Leak/mem_monitor.thpl", '-bg', '-ipc');
}

sub mem_check
{
    if (!$monitor) {
	logresult('SCRIPT', "Memory monitoring not started.");
	return;
    }

    $monitor->ipc()->print("exit\n");
    $monitor->wait_finish();

    subtest("Leak/mem_check.thpl");

    $monitor = undef;
}

1;
