#***********************************************************************
#
# NAME
#   MOUNT_SVC.pm - 
#
# SYNOPSIS
#   Mount_Svc Library for the invocation and verfication of CM SN Mount Service.
#
# DESCRIPTION
#   This library is a wrapper for the typical MOUNT_SVC commands that an end user
#   may desire for the creation and deletion of mounts.  The current library was designed to use NACL CM SN filer.
#
# AUTHOR
#   dl-cluster-qa
#
# HISTORY
# 07/06/11 Initial revision
#***********************************************************************

## @summary     MOUNT_SVC library    

package MOUNT_SVC;

#use statements
use strict;
use Data::Dumper;
use Server;
use Tharn qw($Log snooze host connect script_result fatal_result);
use Options;
use Params::Common;
use Params::Validate qw(:all);

use Carp;
use Sys::Hostname;

use File::Path;
use File::Basename;
use NACL::APISet;
use NACL::C::Node;
use NACL::C::CommandInterface;
use NACL::C::StorageFailover;
use NACL::Transit::Exceptions::TransitException qw(:try);
use Clone;


#Global variables
our $ontap_version;
my $Server = new Server();

my @Nodes=$Server->servers();

#maybe we can load the new the cdefs on declaration
our $Api_Set_ObjS;
our $Api_Set_ObjC;
my $opts = new Options;

my $opt_href = $opts->get_hash();

my $enable_regex = qr/1|on|ON|yes|YES|true|TRUE|0|off|OFF|no|NO|false|FALSE|enable|disable/;
my $id_regex = qr/hostname|systemid|true|false/i;



############################################################################
# TestCase: check_log
# Parameters: None
# Returns: 0 if the string is not found, 1 if string is found
# Purpose: Check for a string within the logs
#############################################################################

sub check_log {
    my %params = (
            node=>"local",
            log => undef,
            message=>undef,
            method => "log",
            @_
    );
    if ( !$params{log} && !$params{message} ){
        $Log->comment("Log file and string to match not specified, exiting");
        return 0;
    }
    my $host = host("$Nodes[0]");
    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     
                                  );
 
    if ( $params{method} eq "log"){
        my $response=$Api_Set_ObjC->system_node_date_show(node=>$params{node});
        my @parsed_output = @{$response->get_parsed_output()};
        $Log->comment("Parsed output: @parsed_output, size: ".scalar(@parsed_output));
        #my $date = "<=".$parsed_output[0]->{'utcdateandtime'};
        my $date=">5m";
        #$Log->error("date: $date");
        $Api_Set_ObjC->debug_log_files_modify("incl-files"=>$params{log});
        my $logfile=$Api_Set_ObjC->debug_log_show(time=>$date,node=>$params{node});
        if ($logfile->get_raw_output() =~ /$params{message}/) {
            $Log->comment("Found message $params{message} in log file $params{log}");    
            return 1;
        }
        
    }
    else{
        my $logfile=$Api_Set_ObjC->debug_mtrace_log_show(node=>$params{node},fields=>'message',buffer=>$params{log});
        if ($logfile->get_raw_output() =~ /$params{message}/) {
            $Log->comment("Found message $params{message} in log file $params{log}");    
            return 1;
        }        
    }
    return 0;
} # end sub  _check_log

############################################################################
# TestCase: spmctl_exec
# Parameters: None
# Returns: result of execution of spmctl
# Purpose: Change the service state using spmctl
#############################################################################
sub spmctl_exec{
    my %params = (
            service => undef,
            command => undef,
            @_
    );    
    if ( !$params{service} && !$params{command} ){
        $Log->error("Service name and command not specified, exiting");
        return 0;
    }
    my $host = host("$Nodes[0]");
    $Api_Set_ObjS=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set       => "Systemshell",
                                     #shell_type => "CMode",
                                     context_definition => {username => "diag", password => "netapp1!"}
                                     );
   

    my $response;
    my @processes;
    my $service = $params{service};
   
    if ( $params{command} eq "start" ){
        $response = $Api_Set_ObjS->spmctl(handle=>$params{service},exec=>1);
    }
    elsif ( $params{command} eq "stop" ){
        $response = $Api_Set_ObjS->spmctl(handle=>$params{service},stop=>1);
    }
    elsif ( $params{command} eq "list" ){
        @processes = @{$Api_Set_ObjS->spmctl(handle=>$params{service},list=>1)->get_parsed_output()};
        foreach my $hash (@processes){        
            foreach my $hvalue (keys %{$hash}){
                #$Log->comment("Process in spmctl list: $hvalue, ".$hash->{$hvalue});
                if (  $hvalue eq "Exec" ){
                    #$Log->comment("Exec in spmctl list: $hvalue, ".$hash->{$hvalue});
                    if ( $hash->{$hvalue} =~ /$service/  ){
                        $Log->comment("Found process $params{service} in spmctl");
                        $response =  $hash->{$hvalue};
                    }
                }
            }
        }
    }
    sleep(20);
    $Log->comment("Return spmctl response: $response");
    return  $response; 

}


############################################################################
# TestCase: counter_exec
# Parameters: None
# Returns: result of execution of cm_client
# Purpose: Change the service state using cm_client
#############################################################################
sub counter_exec{
    my %params = (
            service => undef,
            command => undef,
            @_
    );    
    if ( !$params{service} && !$params{command} ){
        $Log->error("Service name and command not specified, exiting");
        return 0;
    }
    my $host = host("$Nodes[0]");
    if (!$Api_Set_ObjS){
        $Api_Set_ObjS=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set       => "Systemshell",
                                     shell_type => "CMode",
                                     context_definition => {username => "diag", password => "netapp1!"}
      
                                     );
    }
    my $response = $Api_Set_ObjS->execute_raw_command(command=>"cm_client -p $params{service} $params{command}");
    my @parsed_output = split("\n", $response);
    $Log->comment("parsed output: ".Dumper(@parsed_output));
    return $parsed_output[0];

}

############################################################################
# TestCase: system_node_exec
# Parameters: None
# Returns: result of execution of system menu
# Purpose: Change the service state using system menu
#############################################################################
sub system_node_exec{
    my %params = (
            command => undef,
            @_
    );  
    my $ret;
    my $host = host("$Nodes[0]");
    if (!$Api_Set_ObjC){
        $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     
                                  );
    }
    #couldn't get system_node(pass_through) to work
    eval {$ret = $Api_Set_ObjC->execute_raw_command(command=>$params{command});};
    if ($@) { return []; }
    return $ret; 
}


###########################################################################
# TestCase: create_mount
# Parameters: None
# Returns: result of execution of system menu
# Purpose: create a mount
#############################################################################
sub create_mount{
    my %params = (
            node=>"local",
            root_node => undef,
            @_
    );    
    my $ret;
    my $host = host("$Nodes[0]");
    if (!$Api_Set_ObjC){
        $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     
                                  );
    }
    eval {$ret = $Api_Set_ObjC->system_node_root_mount_create(node=>$params{node},"root-node"=>$params{root_node})->get_parsed_output();};
    if ($@) { return "Error"; }
    #$Log->comment("Create Mount output: ".Dumper($ret));
    sleep(10);
    return $ret;  
}

###########################################################################
# TestCase: delete_mount
# Parameters: None
# Returns: result of execution of system menu
# Purpose: delete the mount
#############################################################################
sub delete_mount{
    my %params = (
            node=>"local",
            root_node => undef,
            @_
    );    
    my $ret;
    my $host = host("$Nodes[0]");
    if (!$Api_Set_ObjC){
        $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     
                                  );
    }
    my $result = show_mount();
    $Log->comment("Show mount result: $result");
    if ( $result =~ /Error/ ){
        $Log->comment("No mounts exists, skipping delete");
        return 1;
    }    
    eval {
          $ret = $Api_Set_ObjC->system_node_root_mount_delete(node=>$params{node},"root-node"=>$params{root_node})->get_parsed_output();
          
        };
    if ($@) { return "Error"; }
    sleep(10);    
    return $ret;    

}
###########################################################################
# TestCase: halt_node
# Parameters: None
# Returns: result of execution of system menu
# Purpose: halt the node using the apiset
#############################################################################
sub halt_node {
    my %params = (
            node=>"local",
            exec_node=>undef,
            wait_time=>120,
            command1=>undef,
            command2=>undef,
            @_
    );    
    my $ret;
    my $host = host($params{exec_node});
    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     
                                  );
    #turn off asup before reboot
    #$Api_Set_ObjC->system_node_autosupport_modify(node=>"*",state=>"disable");    
    # Halt node via the partner
    $Api_Set_ObjC->system_node_reboot(node=>$params{node});
    
    #$Log->error("Graceful reboot issued! Sleeping for two minutes...");
    sleep($params{wait_time});
    #execute a command while the node is halted
    my $result;
    if ($params{command1}){
        $result=$Api_Set_ObjC->execute_raw_command(command=>$params{command1});
    }
    # Use NACL::Transit to try to bring the filer back to login prompt
    # If stuck waiting for giveback, issue giveback from partner node
    my $Transit_obj = NACL::Transit->new(name =>$params{node});
    try {
        #$Log->error("Attempting to bring the filer back up...");
        $Log->comment("In the try block for trans...");
        $Transit_obj->change_state(to=>'USERNAME');
    } catch NACL::Transit::Exceptions::TransitException with {
        #$Log->comment("In the try catch for trans...");       
        my $exception_object = shift;
        $Log->comment("Filer found in state: " . $exception_object->text());
        if ($exception_object->text()) {
               #$Log->error("Trying to force giveback...");
               #execute a command during giveback
               if ( $params{command2}){
                    $result = $Api_Set_ObjC->execute_raw_command(command=>$params{command2});
                    
               }
               $Api_Set_ObjC->storage_failover_giveback(fromnode=>$params{exec_node},'override-vetoes'=>"true");
               sleep(60);
        }
        else {
            $Log->error("Failed to bring the filer back up.");
        }
    };
    return $result
        
}

###########################################################################
# TestCase: reboot_node
# Parameters: None
# Returns: result of execution of system menu
# Purpose: reboot the node using the apiset
#############################################################################
sub reboot_node {
    my %params = (
            node=>"local",
            exec_node=>undef,
            @_
    );    
    my $ret;
    my $had_error=0;
    $Log->comment("Enable auto-giveback on all nodes...");
     NACL::C::StorageFailover->modify(command_interface=>
                                        NACL::C::Node->new(name=>$params{exec_node}),
                                        node=>'*',
                                        'auto-giveback'=>'true');

     $Log->comment("Reboot the filer $params{node} ...");
     try {
          my $Transit_obj = NACL::Transit->new(name => $params{node});
          $Transit_obj->reboot();
     } catch NACL::Transit::Exceptions::TransitException with {
          my $exception_object = shift;
          $Log->comment("Failed to bring the filer back up.");
          $Log->comment("Error is: " . $exception_object->text());
          $had_error = 1;
     };

     if ($had_error) {
         return $TCD::FAIL;
     }
        
}

###########################################################################
# TestCase: restart_mdb_process   
# Parameters: None
# Returns: 
# Purpose: restart the mgwd process and re-establish connections
#############################################################################
sub restart_mdb_process {
    my %params = (
            node=>"local",
            exec_node=>undef,
            wait_time=>60,
            command=>undef,
            process=>undef,
            @_
    );
    my $Host = host("$params{exec_node}");     
    my $conn = Connectrec->new(
                    host=>$Host,
                    username=>'diag',
                    password=>'netapp1!',
                    template=>'ssh-persist',
                    login=>1,
                    );
    my $Api_Set_Obj_s = NACL::APISet->new(
                    connection =>$conn,
                    category=>"Node",
                    interface=>"CLI", 
                    set => "Systemshell",);
    my $ngsh_host=host("$params{node}");
    my $ngsh = NACL::APISet->new(hostobj   => $ngsh_host,
                    category=>"Node",
                    interface=>"CLI", 
                    set => "CMode",);    

    $Api_Set_Obj_s->spmctl(handle =>$params{process},stop=>1);
    my $result;
    if ( $params{command}){
        $result=$ngsh->execute_raw_command(command=>$params{command});
    }
    $Api_Set_Obj_s->spmctl(handle =>$params{process},exec=>1);
    $Log->debug("\nSleep $params{wait_time} seconds to wait for the $params{process} fully functional\n"); 
    sleep($params{wait_time});
    return $result;  
}


###########################################################################
# TestCase: show_mount
# Parameters: None
# Returns: result of execution of system menu
# Purpose: Show the mounts
#############################################################################
sub show_mount{
    my %params = (
            node=>"*",
            exec_node=>undef,
            fields=>undef,
            @_
    );
    my $ret;

    my $host = host("$Nodes[0]");
    if ( $params{exec_node}){
        $host = host("$params{exec_node}");
    }
    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     
                                  );
    if ($params{fields}){
      eval {$ret = $Api_Set_ObjC->system_node_root_mount_show(node=>$params{node},fields=>$params{fields})->get_parsed_output();};
    }
    else{
      eval {$ret = $Api_Set_ObjC->system_node_root_mount_show(node=>$params{node})->get_parsed_output();};
    }
    if ($@) { return "Error"; }
    #$Log->comment("Create Mount output: ".Dumper($ret));
    if (scalar(@$ret) == 0 ){
        $ret =  $Api_Set_ObjC->system_node_root_mount_show(node=>$params{node})->get_raw_output();
    }
    return $ret;

}

###########################################################################
# Subroutine: convert_to_filer_time_format()
# Parameters: None
# Returns: TCD Execution Result (passing)
# Purpose: Convert a stat command version of time to match filer format
###########################################################################
sub convert_to_filer_time_format {
    my %params = (
            stime=>undef,
            @_
    );
    my %month = ( JAN=>'1', FEB=>'2', MAR=>'3',
              APR=>'4', MAY=>'5', JUN=>'6',
              JUL=>'7', AUG=>'8', SEP=>'9',
              OCT=>'10', NOV=>'11', DEC=>'12' );
    my @vals = split(/\s/,$params{stime});
    my $date;
    my @minhour;
    foreach my $val (@vals){
        next if (length($val) < 1);
        #$Log->comment("date vals: $val");
        if ($val =~ /\d\d:\d\d:/){
            @minhour = split(":",$val);
            next;
        }
        if ($val =~ /[a-z]|[A-z]/ ){
            foreach my $name (keys %month){
                if (  $name eq uc($val) ){
                    $date = $month{$name};
                    #$Log->comment("date begin: $date");
                }
            }
        }
        elsif ($val =~ /\d\d$/ || $val =~ /\d$/){
            $date = $date."/".$val;
            $Log->comment("date mid: $date");
        }
        elsif ($val =~ /\d\d\d\d$/){
            $date = $date."/".$val;
            #$Log->comment("date last: $date");
        }
    }
    $date = $date." $minhour[0]:$minhour[1]";
    #$Log->comment("date end: $date");    
    #convert a date of Jun 29 08:36:03 2011 to 6/28/2011 15:46:
    #$Log->comment("Returning stat date: $date");
    return $date;
}

###########################################################################
# Subroutine: get_node_fields_value()
# Parameters: None
# Returns: TCD Execution Result (passing)
# Purpose: Return the serial, uuid, or system id from node(s)
###########################################################################
sub get_node_fields_value{
    my %params = (
            fields => "serialnumber",
            node=>undef,
            @_,
    );    
    my @values;
    my $host = host("$Nodes[0]");
    if (!$Api_Set_ObjC){
        $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     
                                  );
    }
    my $response; 
    my @parsed_output;     
    if ( $params{fields} eq "node-uuid" ){
        if ($params{node}){
            $response = $Api_Set_ObjC->cluster_show(node=>$params{node},fields=>$params{fields});
        }
        else{
            $response = $Api_Set_ObjC->cluster_show(fields=>$params{fields});
        }
    }
    else{
        $response = $Api_Set_ObjC->system_node_show(fields=>$params{fields});
    }
    @parsed_output = @{$response->get_parsed_output()}; 
    foreach my $hash (@parsed_output){
        foreach my $hvalue (keys %{$hash}){
            ##$Log->comment("Hash Key for Cluster Show: $hvalue");
            if ( $hvalue eq $params{fields}){
                push(@values,$hash->{$hvalue});
               ##$Log->comment("Adding Execution Node Name $node"); 
            }
        }
    }
    return @values;
}

=pod

=head1 NAME

UNIFIED_LOG_TRACE.pm - UNIFIED_LOG_TRACE configuration, generation, and verification library for 7mode and Cmode SN.
=head1 SYNOPSIS

    use UNIFIED_LOG_TRACE;
    ASUP::show_utl_log(
            node       => undef,
            source => undef,
            time => undef,
            seq => undef,
            message => undef,
            );


=head1 DESCRIPTION

The methods included in this library were developed in order to assist
engineers in generation and verification of Mount Service commands.  The current library
supports  Cmode filers and invokes communication to the filer via the
NACL library.  The MOUNT_SVC.pm library allows the user to configure
and verify Mount states and log data.
  
    
=head1 METHODS
            
    check_log();
        Description:
            This method will return a 1 if a string is found, 0 if the string is not found in the specified log
        Inputs:
            node=>"local",
            log => undef,
            message=>undef,
            method => "log",
        Outputs:
            0 or 1.

    spmctl_exec();
        Description:
            This method executes the spmctl command 
          Inputs:
            service => undef,
            command => undef,
        Outputs:
            The raw output of the command.

    counter_exec();
        Description:
            This method executes the counter command 
          Inputs:
            service => undef,
            command => undef,
        Outputs:
            The raw output of the command.            

    system_node_exec();
        Description:
            This method executes the system->node command 
          Inputs:
            command => undef,
        Outputs:
            The raw output of the command.
            
    create_mount();
        Description:
            This method will create a mount using the system->node->root-mount menu
                
        Inputs:
            node=>"local",
            root_node => undef,
        Outputs:
            The raw output of the comand.
           
    delete_mount();
        Description:
            This method will delete a mount using the system->node->root-mount menu
                
        Inputs:
            node=>"local",
            root_node => undef,
        Outputs:
            The raw output of the comand.
            
    show_mount();
        Description:
            This method will show a mount using the system->node->root-mount menu
                
        Inputs:
            node=>"local",
            root_node => undef,
        Outputs:
            The raw output of the comand.               
           
    halt_node();
        Description:
            This method will halt a node and run a command on the pair node during halt and giveback
                
        Inputs:
            node=>"local",
            exec_node=>undef,
            wait_time=>120,
            command1=>undef,
            command2=>undef,
        Outputs:
            The result of command1/2. 

    reboot_node();
        Description:
            This method will reboot a node and run a command on the pair node during halt and giveback
                
        Inputs:
            node=>"local",
            exec_node=>undef,
            wait_time=>120,
            command1=>undef,
            command2=>undef,
        Outputs:
            The result of command1/2.

    restart_mdb_process();
        Description:
            This method will restart mdb process, i.e. mgwd and run a command on the pair node during stop            
            node=>"local",
            exec_node=>undef,
            wait_time=>60,
            command=>undef,
            process=>undef,
        Outputs:
            The result of command.    

    convert_to_filer_time_format();
        Description:
            This method will convert a 'stat' version of time to a filer version of time
                
        Inputs:
            stime=>undef,
        Outputs:
            The converted time w/o seconds
            
    get_node_fields_value();
        Description:
            This method will return the node-uuid, serial number, etc for node(s) in a cluster
               
        Inputs:
            fields => "serialnumber",
            node=>undef,
        Outputs:
            An array of the values for each node           
        
=head1 EXAMPLES

Typical example:

    use UNIFIED_LOG_TRACE.pm;
    use vars qw($LOGDIR $LOGDIR_UNIX $CLIENT);
    my $Server  = Server->new();
    my @Nodes  = $Server->servers();
    $Server->set_execution_server(-Server => $Nodes[0]);
    
    my $log_output = UNIFIED_LOG_TRACE::show_utl_log();
    if ( $log_output =~ /This table is currently empty\./){
        $Log->error("No outuput returned for log show of the local node(s) in the cluster");
        return $TCD::FAIL;
    }
=cut

