#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/MailServer.pm#1 $

## @summary Create a mail server daemon on host
## @author knopp@netapp.com
## @status private
## @pod here

# Package Name
##############
package MailServer;

use strict;
use Tharn;
use Error;

our @ISA = qw(Error);

sub new {
    my ($class,%args)=@_;
    my (
        $port,
	$hostname,
	$is_ready,
	$self,
	$subtest,
	);

    $self = {};

    bless $self, $class;

    ### default smtp port   
    $port=25;

    ### parse arguments
    foreach (keys %args) {
      if (/-?hostname/i) {
	if (not length($hostname=$args{$_})) {
	    $self->_set_err("Error:MailServer::new() Invalid value for argument '-hostname'");
	    return $self;
	} 	
	next;
      }
      elsif (/-?port/i) {
	if (not length($port=$args{$_})) {
	    $self->_set_err("Error:MailServer::new() Invalid value for argument '-port'");
	    return $self;
	} 	
	next;
      }
      else {
	$self->_set_err("Error: MailServer::new() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    }     

    unless (defined $hostname){
	$self->_set_err("ERROR: MailServer::new() -hostname is a required parameter");
        return $self;
    }
    $self->{hostname}=$hostname;


    $subtest=subtest("{".$self->_get_script()."}",
	         -client=>$hostname, 
                 -bg,
		 -runid=>$hostname."_mail_port".$port,
	         -username=>'root',
	         "--",
	         -hostname=>$hostname,
	         -port=>$port,
                 );
   
     
    ### Lets not return to the calling parent until the subtest is 
    ### ready.  
    $is_ready=0;
    while (!$is_ready){
	my @results = $subtest->results();
	foreach (@results) {
	   if ($_->type()=~/INFO/i) {
		if ("$_->messages()"=~/error/i){
		    $self->_set_err($_->messages());
		    return $self;		
		}
		$is_ready=1;
	   }
	} 
    }
    
    ### Save the subtest in the object
    $self->{subtest}=$subtest;

    return $self;

}

sub get_hostname {
    my ($self)=@_;
    $self->_clear_err();

    unless (defined $self->{hostname}){
	$self->_set_err("ERROR: MailServer::get_hostname()  Unable to retrieve host mailserver is running on");
	return "";
    }

    return $self->{hostname};
}

sub get_message {
    my ($self,$contains)=@_;
    my (
	$found_cnt,
	$index_high,
	$index_low,
	$tot,
	@messages,
	@results,
	@tmps,
	);

    $self->_clear_err();
    
    $self->{date}=undef;

    ### Number of mail messages to search for event.
    $tot=10; 
    $found_cnt=0;

    unless (defined $contains){
	$self->_set_err("ERROR: MailServer::get_message() Parameter $contains is required");
	return (wantarray()) ? () : [];
    }

    ### Return all resulut messages
    @results = $self->{subtest}->results();

    ### Search the last 10 results for the event message
    $index_high=scalar(@results)-1;
    $index_low=($index_high > $tot) ? $index_high-$tot : 0;  
    foreach my $result (@results[$index_low..$index_high]) {
	my @tmps=split("\cM\cJ",($result->messages())[0]);
	if ("@tmps"=~/$contains/){
	    push @messages,@tmps;
	    last;
	}
    }
    
    #push @messages,$found_cnt." event messages found.",@tmps;

    ### If message was not found inform user of the search query.
    if (not @messages){
	$self->_set_err("ERROR: MailServer::get_message() mail message containing content",
	                "'$contains' was not found in last ".$tot." mail",
	                "messages recieved");

	return (wantarray()) ? () : [];
    }
    else {
	foreach my $line (@messages){
	    if ($line=~/Date:/){
		my ($date_str)=$line=~/Date: (.*)/;
		$self->{date}=$date_str;
	    }
	}

    }

    return @messages;
}

sub get_date {
    my ($self)=@_;
    $self->_clear_err();

    return (defined $self->{date}) ? $self->{date} : "";
}

sub _get_script {
    my ($self)=@_;

    return << 'SCRIPT';

    use Carp;
    use Net::SMTP::Server;
    use Net::SMTP::Server::Client;

    my (%args)=@_;

    my (
	@buffer,
	);

    chdir($ENV{LOGDIR});

    my $resObj;

    my $server = new Net::SMTP::Server($args{-hostname},$args{-port});
    if (not defined $server){
	$resObj=NATE::Result::Info->new("ERROR: MailServer::new() Unable to handle client connection for ".
	                                $args{-hostname}." (port:".$args{-port}.").");
	$resObj->append_messages("       ".$!);
	$resObj->store();
	return 0;
    }
    else {
	$resObj=NATE::Result::Info->new("Success");
	$resObj->store();
    }

    while(my $conn = $server->accept()) {

	my $client = new Net::SMTP::Server::Client($conn) ||
	    croak("Unable to handle client connection: $!\n");
	
	# Process the client.  This command will block until
	# the connecting client completes the SMTP transaction.

	$client->process || next;

	my $resObj=NATE::Result::Info->new();
       
	push my @buffer,$client->{MSG},"\n";   	           
	    
	$resObj->messages(\@buffer);

	$resObj->log();
		      
    }
SCRIPT
}


1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

MailServer.pm

=head1 Author

Gary Knopp

=head1 Description

Allows you to create and runt a temporary mailserver on a remote client

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error>

=head1 Public Methods

=head2 B<new(-hostname,I<-port>)>

Create a new EventMgr object to manage events

=over 4

=item Inputs

=item * B<-hostname> 

Remote hostname to create mailserver on

=item * I<-port> (default: 25)

hostname port to listen on for incoming messages

=item Outputs

Returns an MailServer object handle.  If an error occured the error flag is set, 
errmsg is set to the failure reason.

=item Examples

    foreach my $client_name (@Client_Names){
	$Test->nlog("-- Starting up mailserver on client ".$client_name);
	my $ms_obj=MailServer->new(-hostname=>$client_name);
	if ($ms_obj->error()){
	    $Test->nlog($ms_obj->errmsg());
	    return $TCD::FAIL;
	}
	push @ms_objs,$ms_obj;
    }

=back

=head2 B<get_date()>

Returns the date of the message recieved by issuing the MailServer::get_message() function. 
If a message was not retrieved it will retrun NULL.

=over 4

=item Inputs

None 

=item Outputs

Returns event name.

=item Examples

     #retrieve cgid value
     $message=$ms_obj->get_date();
    

=back

=head2 B<get_hostname()>

Retrieve hostname mailserver is running on

=over 4

=item Inputs

None 

=item Outputs

Returns event name.

=item Examples

     #retrieve cgid value
     $hostname=$ms_obj->get_hostname();
    

=back

=head2 B<get_message($contains)>

This method searches the last 10 messages (latest to earliest) for the specified message content.  It returns 
the first message that matches content.

=over 4

=item Inputs

=item * B<$contains>

A scalar containing the string to search for in the last 10 messages recieved by the mail server

=item Outputs

Returns message matching the contain string as an array. If a message was not found or an error
occured during the call the error flag is set, errmsg is set to the failure reason.

=item Examples

    my @message=$ms_obj->get_message("app.log.crit");

=back

=cut
