#-------------------------------------------------------------------------------
# Author: Mauricio Sanchez
#   Date: 2012-Mar-28
#-------------------------------------------------------------------------------
# Description:
#   Various environment probing funcs
#------------------------------------------------------------------------------

package Mercury::Build::ENV;

use strict;
use warnings;

use Mercury::File::IO; 

#-------------------------------------------------------------------------------
# Static call to dump the environment
#-------------------------------------------------------------------------------
sub env_dump {

    # Get the hashed arguments
    my %args = @_;

    # Parse the hashed arguments
    my $output_file = $args{output_file};

    my $fh;

    # See if an output file was passed
    if ( $output_file ) {

        # Capture the current output handle
	$fh = select;

        # See if we can append to the output file
        open(OUTPUT, ">>$output_file") || die("Could not open $output_file");
        select OUTPUT;
    }

    # Show the environment
    print "----------------- ENV variables -----------------\n";
    for my $key (sort(keys(%ENV))) {
        print "\$ENV{$key} = '$ENV{$key}'\n";
    }

    if ( $output_file ) {
        # Close the output file
        close OUTPUT;

	select $fh;
    }
}

#-------------------------------------------------------------------------------
# Static call to read a variable from last dump
#-------------------------------------------------------------------------------
sub env_read_dump {

    # Get the hashed arguments
    my %args = @_;

    # Parse the hashed arguments
    my $input_file = $args{input_file};
    my $env_var    = $args{env_var};

    my @file_contents = file_get_contents(file => $input_file);

    my @matches = grep(/ENV\{$env_var\} = /, @file_contents);

    # Get last match
    my $last_match = $matches[$#matches];

    $last_match =~ s/\$ENV\{$env_var\} = '(.*)'/$1/;

    return $last_match;
}

1;
