#-------------------------------------------------------------------------------
# Author: Mauricio Sanchez
#   Date: 2012-May-30
#-------------------------------------------------------------------------------
# Description:
#   Update the version in hgadmin.PS1 file.
#------------------------------------------------------------------------------

package Mercury::Build::ESXVersion;

use strict;
use warnings;

use Mercury::File::IO qw(file_get_contents file_put_contents);

#-------------------------------------------------------------------------------
#   Instatiate an object with the initial .ps1 and resc file
#------------------------------------------------------------------------------
sub new {

    # Get the hashed arguments
    my ($class, %args) = @_;

    # Create the actual object
    my $this = bless({}, $class);

    # Parse the hashed arguments
    $this->{ps1_file_in}  = $args{ps1_file_in} if ( $args{ps1_file_in} );

    # Read the entire ps1 file into an array
    @{$this->{ps1_file_in_array}} = file_get_contents(file => $this->{ps1_file_in}) if ( $this->{ps1_file_in} );

    return $this;
}

#-------------------------------------------------------------------------------
#   Update the Version info in the arrayis; reflecting current time-stamp
#------------------------------------------------------------------------------
sub set_ver {

    # Get the hashed arguments
    my ($this, %args) = @_;

    my $version = $args{version};

    # Check that we have a ps1 file
    if ( $this->{ps1_file_in} ) {

        # Iterate through the entire ps1 file
        for (@{$this->{ps1_file_in_array}}) {

            # Look for version string definitiopn and change in ps1 array
            s#(^\$HGADMIN_VERSION\s*=\s*").*(".*)$#$1$version$2#; 
        }
    }
}

#-------------------------------------------------------------------------------
#   Save the corresponding file from the array
#------------------------------------------------------------------------------
sub save {

    my $file_out = '';
    my $output_array;

    # Get the hashed arguments
    my ($this, %args) = @_;

    # See if an 'ps1' file was specified
    if ( $args{ps1_file_out} ) {

        # Set the output_file and the corresponding array
        $file_out = $args{ps1_file_out};
        $output_array = $this->{ps1_file_in_array};

    # See if an 'resc' file was specified
    } else {
        die "No output-file specified: $!\n";
    }

    # Send the array to the file
    file_put_contents(file => $file_out, array => $output_array);
}

1;
