#-------------------------------------------------------------------------------
# Author: Mauricio Sanchez
# Date:   2012-Mar-12
#-------------------------------------------------------------------------------
# Description: Get back a valid, connected, ssh handle
#              Didn't want to polute the main script with this logic.
#-------------------------------------------------------------------------------

package Mercury::Build::SSH;

use strict;
use warnings;

use Net::SSH::Expect;

our @ISA = qw(Net::SSH::Expect);

#-------------------------------------------------------------------------------
# Create an object for SSH intgeractions
#-------------------------------------------------------------------------------
sub new {
    # Get the class name and the rest of the hashed arguments
    my ($class, %args) = @_;

    # Parse the hashed arguments
    my $host = $args{host};
    my $user = $args{user};

    # Create the ssh object
    my $ssh = Net::SSH::Expect->new(host => $host, user => $user, ssh_option => '-v 2>&1');
    $ssh->run_ssh();

    # Try an echo command
    my $ssh_test = $ssh->exec('echo hello');

    # See if it worked (if yes/no is in the output, it didn't)
    if ( $ssh_test =~ m#yes/no# ) {

        # Since the command failed we need to run the login() method
        # Unfortunately, the login() method requires a password
        # So we'll create a new ssh object with a bogus passwd
        my $ssh2 = Net::SSH::Expect->new (
            host => $host,
            user => $user,
            password => '1',
        );
        $ssh2->run_ssh();

        # The login method get's us passed 'yes/no', but will choke because of
        # the bogus passwd... So we 'eval' it
        eval {$ssh2->login();};

        # Now our original $ssh object has to be abandoned, we need a new one
        $ssh = Net::SSH::Expect->new(host => $host, user => $user);
        $ssh->run_ssh();
    }

    # Bless this object as an SSH object
    my $this = bless($ssh, $class);

    return $this;
}

1;
