#
# $Id: //depot/prod/test/main/lib/Mercury/SSD.pm $
#
# Copyright (c) 2009 NetApp, Inc., All Rights Reserved
# Permission is granted to use this script on a NetApp Product.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc
#
## @summary      Library used to track SSD properties which will be used by ESXi.pm
##
## @author       nkrajara@netapp.com
##
## @pod          here
##

=head1 NAME

SSD.pm

=head1 AUTHOR

Narayan Rajaram

=head1 DESCRIPTION

C<ESXi> will maintain the properties of the ESXi machine.

1. Will maintain the properties of a SSD device

=head1 SYNOPSIS

Library used to maintain the state and the properties of the SSD device

=cut

package Mercury::SSD;

# Standard Libs

use strict;
use warnings;
use Data::Dumper;
use Switch;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use Tharn;
use Error;
use San::SanLog;
use vars qw(@ISA);
@ISA = qw(Error SanLog);

# Netapp Specific Libs

use NACL::APISet;
use TracePkg;

# Global Variable
my $log = $Tharn::Log;

=head2 new

To instantiate and configure an SSD onject

=cut

#############################################################################
# Subroutine: new
# Parameters: None
# Returns:    Reference to this object
#############################################################################

sub new {

    my $class = shift(@_);
    my $self;
    my %params = validate(
        @_,
        {   id => {
                type     => SCALAR,
                optional => 0,
                default  => "*",
            },
            shell => {
                type     => OBJECT,
                optional => 1,
                default  => "*",
            },
        }
    );

    # Copy specified parameters globally into object

    $self->{_naa_id} = $params{id};
    $self->{_shell}  = $params{shell};

    bless($self, $class);
    $log->comment("Created a ssd object\n");
    return $self;

} ## end sub new

=head set_ssd

Set the parameters required for SSD

=cut

#############################################################################
# Subroutine: set_ssd
# Parameters: NAA ID
#############################################################################

sub set_ssd {
    my ($self, %args) = @_;
    $self->_clear_err();
    my $command
        = "esxcli storage core device list --device=" . $self->{_naa_id};
    my $output = $self->{_shell}->execute_raw_command(command => $command);
    my @results = split(/\n/, $output);
    foreach my $result (@results) {

        # Regex to get the key:value pairs
        if ($result =~ /^\s+(.*)\:\s+(.*)(\s*)$/) {
            my $key   = $1;
            my $value = $2;
            $self->{$key} = $value;
        }
    }
} ## end sub set_ssd

#############################################################################
# Subroutine: get_ssd_property
# Parameters: property to get
#############################################################################

sub get_ssd_property {
    my ($self, %args) = @_;
    $self->_clear_err();
    my $prop = $args{property};
    return $self->{$prop};
} ## end sub get_ssd_property

1;

