#
# $Id: //depot/prod/test/main/lib/Mercury/Vlun.pm $
#
# Copyright (c) 2009 NetApp, Inc., All Rights Reserved
# Permission is granted to use this script on a NetApp Product.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc
#
## @summary      Library used to track Vlun properties which will be used by ESXi.pm
##
## @author       nkrajara@netapp.com
##
## @pod          here
##

=head1 NAME

Vlun.pm

=head1 AUTHOR

Narayan Rajaram

=head1 DESCRIPTION

C<ESXi> will maintain the properties of an Vlun

1. Will maintain the properties of a Vlun device

=head1 SYNOPSIS

Library used to maintain the state and the properties of the Vlun device

=cut

package Mercury::Vlun;

# Standard Libs

use strict;
use warnings;
use Data::Dumper;
use Switch;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use Tharn;
use Error;
use San::SanLog;
use vars qw(@ISA);
@ISA = qw(Error SanLog Mercury::SSD);

#our @ISA = qw(Mercury::SSD);

# Netapp Specific Libs

use NACL::APISet;
use Mercury::SSD;
use TracePkg;

# Global Variable
my $log = $Tharn::Log;

=head2 new

To instantiate and configure an SSD onject

=cut

#############################################################################
# Subroutine: new
# Parameters: None
# Returns:    Reference to this object
#############################################################################

sub new {

    my $class  = shift(@_);
    my %params = validate(
        @_,
        {   id => {
                type     => SCALAR,
                optional => 0,
                default  => "*",
            },
            shell => {
                type     => OBJECT,
                optional => 1,
                default  => "*",
            },
        }
    );

    # Copy specified parameters globally into object

    my $self = $class->SUPER::new(
        id    => $params{id},
        shell => $params{shell}
    );

    $self->{_vm} = "";

    bless($self, $class);
    $log->comment("Created a vlun object\n");
    return $self;

} ## end sub new

=head set_vm

Set the vm to whihc this vlun is getting assigned to.

=cut

#############################################################################
# Subroutine: set_vm
# Parameters: vm_name
#############################################################################

sub set_vm {
    my ($self, %args) = @_;
    $self->_clear_err();
    $self->{_vm} = $args{vm};
}

1;

