#
# Copyright (c) 2001-2012 NetApp, Inc., All Rights Reserved
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#

## @summary Use WCFClient to interface with the FlashAccel WebService
## @author bhanup@netapp.com
## @status private
## @pod not here

package Mercury::Windows::hgMgmt;

#
# Compiler Directives
#
use strict;
use warnings;

# NATE includes
use Tharn qw($Log);

use Mercury::Windows::hgWebService;
use Mercury::Windows::hgCmdLets;

################################################################################
# Method: new
# Objective: create a new hg_cmdlets oject
# Details:
################################################################################
sub new {
    $Log->enter();

    my ($class, %args) = @_;

    my ($mgmt_IF);

    foreach my $param (keys %args) {
        if ($param =~ /^-?interface$/i) {
            $mgmt_IF = $args{$param};
        }
    }

    my $self = {};
    bless($self, $class);

    if($mgmt_IF =~ /^WebService$/i){
        require Mercury::Windows::hgWebService;
        $self = new Mercury::Windows::hgWebService(%args);
    } else {
        require Mercury::Windows::hgCmdLets;
        $self = new Mercury::Windows::hgCmdLets(%args);
    }

    $Log->exit();

    return $self;
}

################################################################################
# Method: normalize_keys_to_cmdlets 
# Objective: maintain the same names for the keys returned by either teh 
#            cmdlets or the webservice module   
# Details: In order to maintain the same keys between the cmdlets and webservice
#          we will convert any webservice specifics to be cmdlets based 
################################################################################
sub normalize_keys_to_cmdlets {
    $Log->enter();

    my ($self, %args) = @_;

    my ($ref);

    foreach my $param (keys %args) {
        if ($param =~ /^-?cacheAndDeviceInfo$/i) {
            $ref = $args{$param};
        }
    }

    unless ($ref) {
        $Log->comment("Have to specify a valid cacheAndDeviceInfo param");
        return $TCD::FAIL;
    }

    # normalize the keys returned by the webservice to that of the cmdlets
    my %normalizing_h = ( 
                CacheOperationalState   => "OperationalState",
                OpStateMsg              => "OperStateMessage",
                CacheNaaId              => "NaaId",
                Identifier              => "NaaId",
                );

    # walk thru the hash keys and change some of the keys
    # to confirm to the cmdlets naming convention
    for my $cache_and_devices (keys %$ref) {
        # each of the keys can be either the cache or devices 
        for my $c_d_details (keys %$cache_and_devices) {
            # each can be properties or metrics or options 
            for my $props_or_metrics (keys %$c_d_details) {
                # props_or_metrics hash details
                for my $n_k (keys %normalizing_h) {
                    if(exists $props_or_metrics->{$n_k}) {
                        $props_or_metrics->{$normalizing_h{$n_k}} = $props_or_metrics->{$n_k};
                        delete $props_or_metrics->{$n_k};
                    }
                }
            }
        }
    }

    return $ref;
}


1;
