# fa_cmdlets.pm
package Mercury::fa_cmdlets;

use NACL::APISet;
use Data::Dumper;
use San::SanClient;
use San::SanUtil;
use San::SanClient::ClientUtil;
use NATE::Log qw(log_global);
my $Log = log_global();

###########################################################################################
#
#
###########################################################################################
# Global variables continued
our $powershell_cmd = "echo | powershell.exe";

################################################################################
# Method: new
# Objective: create a new hg_cmdlets oject
# Details:
################################################################################
sub new {
    my ($class, %args) = @_;
    my $self = {};
    bless($self, $class);
    foreach my $param (keys %args) {
        if ($param =~ /^-?fa_VMs$/) {
            $Clients = $args{$param};
        }
    }
    foreach my $client (@$Clients) {
        my ($Host, $Api_Set_Obj);
        $Log->comment("Create an APISet object.");
        $Host = Tharn::host($client);

        unless ($Host) {
            $Log->comment("host() call failed!!");
        }
        $Api_Set_Obj = NACL::APISet->new(hostobj   => $Host,
                                         category  => "Host",
                                         interface => "CLI",
                                         set       => "PowerShell",);
        $self->{client} = $Api_Set_Obj;
    } ## end foreach my $client (@$Clients)
    return $self;
} ## end sub new

################################################################################
# The command methods are located below
#
################################################################################
# Method: getFaCache
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################

sub getFaCache() {

    my ($self, %args) = @_;

    my $results;

    # start of the arguements string for the test
    my $params = "get-facache";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if Metrics
        if ($param =~ /^-?Metrics$/i) {
            $metrics = $args{$param};
            $params  .= " -Metrics " . $metrics;
            # if Detailed
        } elsif ($param =~ /^-?Detailed$/i) {
            $detailed = $args{$param};
            $params .= " -Detailed " . $detailed;
             # if FormatInfo
        } elsif ($param =~ /^-?FormatInfo$/i) {
            $format_info = $args{$param};
            $params .= " -FormatInfo " . $format_info;
            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
             # if Timeout
        } elsif ($param =~ /^-?Timeout$/i) {
            $timeout = $args{$param};
            $params .= " -Timeout " . $timeout;
            # if Verbose
        } elsif ($param =~ /^-?Verbose$/i) {
            $verbose = $args{$param};
            $params .= " -Verbose " . $verbose;
              # if Debug
        } elsif ($param =~ /^-?Debug$/i) {
            $debug = $args{$param};
            $params .= " -Debug " . $debug;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

    # execute the command on the VM
    my $output =
      $self->{client}
      ->execute_command(command => "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub getFaCache()

################################################################################
# Method: setFaCache
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################
sub setFaCache() {

    my ($self, %args) = @_;
    my %results;
    my $Clients;
    my $results;

    my $params = "set-facache";
    foreach my $param (keys %args) {
        # if CacheMode
        if ($param =~ /^-?CacheMode$/i) {
            $cache_mode = $args{$param};
            $params .= " -CacheMode " . $cache_mode;

            # if AdminState
        } elsif ($param =~ /^-?AdminState$/i) {
            $cache_admin_state = $args{$param};
            $params .= " -AdminState " . $cache_admin_state;

            # if PerfState
        } elsif ($param =~ /^-?PerfState$/i) {
            $perf_state = $args{$param};
            $params .= " -PerfState " . $perf_state;

            # if CacheDeviceId
        } elsif ($param =~ /^-?CacheDeviceId$/i) {
            $cache_device_id = $args{$param};
            $params .= " -CacheDeviceId " . $cache_device_id;

            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;

            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;

             # if Timeout
        } elsif ($param =~ /^-?Timeout$/i) {
            $timeout = $args{$param};
            $params .= " -Timeout " . $timeout;

            # if Verbose
        } elsif ($param =~ /^-?Verbose$/i) {
            $verbose = $args{$param};
            $params .= " -Verbose " . $verbose;

              # if Debug
        } elsif ($param =~ /^-?Debug$/i) {
            $debug = $args{$param};
            $params .= " -Debug " . $debug;

            # else
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
    $params .= " -force";
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

     my $output =
      $self->{client}
      ->execute_command(command => "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    my @myarray = split /\r\n/, $output;
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub setFaCache()
################################################################################
# Method: setFaDevice
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################
sub setFaDevice() {
    my ($self, %args) = @_;
    my $Clients;
    my $results;

    my $params = "set-fadevice";
    foreach my $param (keys %args) {
        # if Disk
        if ($param =~ /^-?Disks$/i) {
            $disk   = $args{$param};
            $params .= " -Disks " . $disk;
            # if NaaIds
        } elsif ($param =~ /^-?NaaIds$/i) {
            $naaids = $args{$param};
            $params .=  " -NaaIds " . $naaids;
            # if Admin State
        } elsif ($param =~ /^-?AdminState$/i) {
            $device_admin_state = $args{$param};
            $params .= " -AdminState " . $device_admin_state;
             # if CacheMode
        } elsif ($param =~ /^-?CacheMode$/i) {
            $device_cache_mode = $args{$param};
            $params .= " -CacheMode " . $device_cache_mode;
            # if PerfState
        } elsif ($param =~ /^-?PerfState$/i) {
            $perf_state = $args{$param};
            $params .= " -PerfState " . $perf_state;
            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params  .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
            # if Timeout
        } elsif ($param =~ /^-?Timeout$/i) {
            $timeout = $args{$param};
            $params .= " -Timeout " . $timeout;
           # if Timeout
        } elsif ($param =~ /^-?Persistence$/i) {
            $persistence = $args{$param};
            $params .= " -Persistence " . $persistence;
           # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
    $params .= " -force";
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");
    
    my $output =
      $self->{client}
      ->execute_command(command => " C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    my @myarray = split /\r\n/, $output;

    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub setFaDevice()
################################################################################
# Method: getFaDevice
# Objective: create a new fa_cmdlets object
# Details:
################################################################################
sub getFaDevice() {
    my ($self, %args) = @_;
    my $results;

    # start of the arguements string for the test
    my $params = "get-fadevice";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if Disk
        if ($param =~ /^-?Disks$/i) {
            $disk   = $args{$param};
            $params .= " -Disks " . $disk;
            # if NaaIds
        } elsif ($param =~ /^-?NaaIds$/i) {
            $naaids = $args{$param};
            $params .= " -NaaIds " . $naaids;
            # if Summarize
        } elsif ($param =~ /^-?Summarize$/i) {
            $summarize = $args{$param};
            $params .= " -Summarize " . $summarize;
            # if Metrics
        } elsif ($param =~ /^-?Metrics$/i) {
            $metrics = $args{$param};
            $params .= " -Metrics " . $metrics;
            # if Detailed
        } elsif ($param =~ /^-?Detailed$/i) {
            $detailed = $args{$param};
            $params .= " -Detailed " . $detailed;
            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

    # execute the command on the VM
    my $output =
      $self->{client}
      ->execute_command(command => " C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub getFaDevice()
################################################################################
# Method: getFaOptions
# Objective: create a new fa_cmdlets object
# Details:
################################################################################
sub getFaOptions() {
    my ($self, %args) = @_;
    my $results;

    # start of the arguements string for the test
    my $params = "get-faoptions";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if ComputerName
        if ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params     .= " -Credential " . $credential;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

    # execute the command on the VM
    my $output =
       $self->{client}
       ->execute_command(command => 
       "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub getFaOptions()
################################################################################
# Method: setFaOptions
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################
sub setFaOptions() {
    my ($self, %args) = @_;
    my $results;

    # start of the arguements string for the test
    my $params = "set-faoptions";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if DefaultDeviceState
        if ($param =~ /^-?DefaultDeviceState$/i) {
            $default_device_state = $args{$param};
            $params .= " -DefaultDeviceState " . $default_device_state;
            # if DefaultDeviceCacheMode
        } elsif ($param =~ /^-?DefaultDeviceCacheMode$/i) {
            $default_device_cache_mode = $args{$param};
            $params .= " -DefaultDeviceCacheMode " . $default_device_cache_mode;
            # if EventLogLevel
        } elsif ($param =~ /^-?EventLogLevel$/i) {
            $event_log_level = $args{$param};
            $params .= " -EventLogLevel " . $event_log_level;
            # if ChecksumEnabled
        } elsif ($param =~ /^-?ChecksumEnabled$/i) {
            $checksum_enabled = $args{$param};
            $params .= " -ChecksumEnabled " . $checksum_enabled;
            # if MetricsResetInterval
        } elsif ($param =~ /^-?MetricsResetInterval$/i) {
            $metrics_reset_interval = $args{$param};
            $params .= " -MetricsResetInterval " . $metrics_reset_interval;
            # if MinReadSize
        } elsif ($param =~ /^-?MinReadSize$/i) {
            $min_read_size = $args{$param};
            $params .= " -MinReadSize " . $min_read_size;
            # if MaxReadSize
        } elsif ($param =~ /^-?MaxReadSize$/i) {
            $max_read_size = $args{$param};
            $params .= " -MaxReadSize " . $max_read_size;
            # if MinWriteSize
        } elsif ($param =~ /^-?MinWriteSize$/i) {
            $min_write_size = $args{$param};
            $params .= " -MinWriteSize " . $min_write_size;
            # if MaxWriteSize
        } elsif ($param =~ /^-?MaxWriteSize$/i) {
            $max_write_size = $args{$param};
            $params .= " -MaxWriteSize " . $max_write_size;
            # if WriteQLimit
        } elsif ($param =~ /^-?WriteQLimit$/i) {
            $write_Q_limit = $args{$param};
            $params .= " -WriteQLimit " . $write_Q_limit;
            # if ChunkMinBlockThreshold
        } elsif ($param =~ /^-?ChunkMinBlockThreshold$/i) {
            $chunk_min_block_threshold = $args{$param};
            $params .= " -ChunkMinBlockThreshold " . $chunk_min_block_threshold;
            # if ChunkMaxBlockThreshold
        } elsif ($param =~ /^-?ChunkMaxBlockThreshold$/i) {
            $chunk_max_block_threshold = $args{$param};
            $params .= " -ChunkMaxBlockThreshold " . $chunk_max_block_threshold;
            # if GenIdInterval
        } elsif ($param =~ /^-?GenIdInterval$/i) {
            $gen_id_interval = $args{$param};
            $params .= " -GenIdInterval " . $gen_id_interval;
            # if MetricsResetInterval
        } elsif ($param =~ /^-?MetricsResetInterval$/i) {
            $metrics_reset_interval = $args{$param};
            $params .= " -MetricsResetInterval " . $metrics_reset_interval;
            # if BlockUsageThreshold
        } elsif ($param =~ /^-?BlockUsageThreshold$/i) {
            $block_usage_threshold = $args{$param};
            $params .= " -BlockUsageThreshold " . $block_usage_threshold;
            # if IoTimeout
        } elsif ($param =~ /^-?IoTimeout$/i) {
            $io_timeout = $args{$param};
            $params .= " -IoTimeout " . $io_timeout;
            # if IoRetryInterval
        } elsif ($param =~ /^-?IoRetryInterval$/i) {
            $io_retry_interval = $args{$param};
            $params .= " -IoRetryInterval " . $io_retry_interval;
            # if IoRetryCount
        } elsif ($param =~ /^-?IoRetryCount$/i) {
            $io_retry_count = $args{$param};
            $params .= " -IoRetryCount " . $io_retry_count;
            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
    $params .= " -force";
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

    # execute the command on the VM
    my $output =
      $self->{client}
      ->execute_command(command =>
      "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub setFaOptions()
################################################################################
# Method: clearFaMetrics
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################
sub clearFaMetrics() {
    my ($self, %args) = @_;
    my $results;
    # start of the arguements string for the test
    my $params = "clear-fametrics";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if Disk
        if ($param =~ /^-?Disks$/i) {
            $disk   = $args{$param};
            $params .= " -Disks " . $disk;
            # if SerialNumber
        } elsif ($param =~ /^-?NaaIds$/i) {
            $naaid = $args{$param};
            $params .= " -NaaIds " . $naaid;
            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
    $params .= " -force";
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

    # execute the command on the VM
    my $output =
      $self->{client}
      ->execute_command(command =>
      "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub clearFaMetrics()
################################################################################
# Method: resetFaDevice
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################
sub resetFaDevice() {
    my ($self, %args) = @_;
    my $results;

    # start of the arguements string for the test
    my $params = "reset-fadevice";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if Disk
        if ($param =~ /^-?Disks$/i) {
            $disk   = $args{$param};
            $params .= " -Disks " . $disk;
            # if SerialNumber
        } elsif ($param =~ /^-?NaaIds$/i) {
            $naaid = $args{$param};
            $params .= " -NaaIds " . $naaid;
            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
    $params .= " -force";
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

    # execute the command on the VM
    my $output =
      $self->{client}
      ->execute_command(command =>
      "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub resetFaDevice()
################################################################################
# Method: formatFaCache
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################
sub formatFaCache() {
    my ($self, %args) = @_;
    my $results;

    # start of the arguements string for the test
    my $params = "format-facache";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if ReplacementPolicy
        if ($param =~ /^-?ReplacementPolicy$/i) {
            $replacement_policy = $args{$param};
            $params .= " -ReplacementPolicy " . $replacement_policy;
            # if BlockSize
        } elsif ($param =~ /^-?BlockSize$/i) {
            $block_size = $args{$param};
            $params .= " -BlockSize " . $block_size;
            # if BlocksPerChunck
        } elsif ($param =~ /^-?BlocksPerChunk$/i) {
            $blocks_per_chunk = $args{$param};
            $params .= " -BlocksPerChunk " . $blocks_per_chunk;
            # if sizeLimitGb
        } elsif ($param =~ /^-?sizeLimitGb$/i) {
            $size_limit_gb = $args{$param};
            $params .= " -sizeLimitGb " . $size_limit_gb;
            # if ComputerName
        } elsif ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
    $params .= " -force";
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");
    
    # execute the command on the VM
    my $output =
      $self->{client}
      ->execute_command(command =>
      "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub formatFaCache()


################################################################################
# Method: stopFaCacheFormat
# Objective: create a new fa_cmdlets oject
# Details:
################################################################################
sub stopFaCacheFormat() {
    my ($self, %args) = @_;
    my $results;

    # start of the arguements string for the test
    my $params = "clear-fametrics";

    # determine which other intputs to add to the arguements
    # and add them to the string
    foreach my $param (keys %args) {
        # if Disk
        if ($param =~ /^-?ComputerName$/i) {
            $computer_name = $args{$param};
            $params .= " -ComputerName " . $computer_name;
            # if Credential
        } elsif ($param =~ /^-?Credential$/i) {
            $credential = $args{$param};
            $params .= " -Credential " . $credential;
            # if none of the above, then it is not a valid input
        } else {
            $Log->comment("***********************************************");
            $Log->comment("* Invalid Param : $param\n");
            $Log->comment("***********************************************");
        }
    } ## end foreach my $param (keys %args)
    $params .= " -force";
 
    $Log->comment("***********************************************");
    $Log->comment("* Command: $params");
    $Log->comment("***********************************************");

    # execute the command on the VM
    my $output =
      $self->{client}
      ->execute_command(command =>
      "C:\\FlashAccel_test\\fa_wrapper.ps1 $params");

    # Since the returned output is in the form of a string
    # we need to split the string by new line characters
    # and put the output into the form of an array.
    my @myarray = split /\r\n/, $output;

    # Trim off any excess new line characters
    chomp @myarray;

    foreach my $record (@myarray) {
        my ($key, $value) = split /:/, $record;
        $key   =~ s/\s//g;
        $value =~ s/\s//g;
        $results{$key} = $value;
    }

    return \%results;
} ## end sub clearFaMetrics()

1;

=pod
d

=head1 Package Name

hg_cmdlets.pm

=head1 Author

Natalie Kerby
knatalie@netapp.com

=head1 Description

This module verifies that input parameters are valid and will then create the 
command to be sent to the script file.

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

B<Error.pm>

B<San/SanClient.pm>

=head1 B<Public Methods>

=head2 B<new(%args)>

=head1 Package Name

hg_cmdlets.pm

=head1 Author

Natalie Kerby
knatalie@netapp.com

=head1 Description

This module verifies that input parameters are valid and will then create the 
command to be sent to the script file.

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

B<Error.pm>

B<San/SanClient.pm>

=head1 B<Public Methods>

=head2 B<new(%args)>

Creates a NACL API object

=over 4

=item Inputs

=item Outputs

Returns a NACL API object

=item Examples

=back

=head2 B<getHgCache()>

This method will typically not take in any inputs however it can take in anywhere from zero to multiple key, value pairs. 

=over 4

=item Inputs

=item *I<-Metrics>  

Displays performance metrics of the cache device.

=item *I<-Detailed>  

Displays all variable attributes of the cache, including performance metrics.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run the command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test. For getHgCache, the hash table will conatin information concerning the CacheMode, AdminState, and DeviceIDs.

=item Examples

=back

=head2 B<setHgCache()>

This method can take in anywhere from zero to multiple key, value pairs and will set the options to their desired values. 

=over 4

=item Inputs

=item *I<-CacheMode>  

The global cache mode for the devices.

=item *I<-CacheAdminState>  

The global administrative state for the devices.

=item *I<-PerfState>  

Global performance statistics gathering for the devices.

=item *I<-CacheDeviceID>  

Set the ID for the cache.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test. For setHgCache, the hash table will typically be empty however an error can sometimes to returned.

=item Examples

=back

=head2 B<getHgDevice()>

This method will typically not take in any inputs however it can take in anywhere from zero to multiple key, value pairs. 

=over 4

=item Inputs

=item *I<-Metrics>  

Displays performance metrics of the cache device.

=item *I<-Detailed>  

Displays all variable attributes of the cache, including performance metrics.

item *I<-Disks>  

List of data disks for which attributes need to be set.

=item *I<-SerialNumbers>  

List of serial numbers of the data disks for which attributes need to be set.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test.

=item Examples

=back

=head2 B<setHgDevice()>

This method can take in anywhere from zero to multiple key, value pairs and will set the options to their desired values. 

=over 4

=item Inputs

=item *I<-DeviceCacheMode>  

The cache mode for the devices.

=item *I<-DeviceAdminState>  

The administrative state for the devices.

=item *I<-PerfState>  

Performance statistics gathering for the devices.

=item *I<-Disks>  

List of data disks for which attributes need to be set.

=item *I<-SerialNumbers>  

List of serial numbers of the data disks for which attributes need to be set.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test.

=item Examples

=back

=head2 B<getHgOptions()>

This method will typically not take in any inputs however it can take in anywhere from zero to multiple key, value pairs. 

=over 4

=item Inputs

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test.

=item Examples

=back

=head2 B<setHgOptions()>

This method can take in anywhere from zero to multiple key, value pairs and will set the options to their desired values. 

=over 4

=item Inputs

=item *I<-DefaultDeviceState>  

The default admin state of cache for newly enumerated devices.

=item *I<-DefaultDeviceCacheMode>  

The default cache mode for newly enumerated devices.

=item *I<-EventLogLevel>  

Verbosity of windows event logging.

=item *I<-ChecksumEnabled>  

Checksum calculation for the cache device block.

=item *I<-MetricsResetInterval>  

The interval to reset device metrics.

=item *I<-InternalTimeout>  

The timeout (in ms) for internal commands.

=item *I<-InternalRetryInterval>  

The retry interval (in ms) for internal commands.

=item *I<-InternalRetryCount>  

The retry count for cache requests.

=item *I<-CacheTimeout>  

The timeout (in us) for cache requests.

=item *I<-CacheRetryCount> 

The retry count for cache requests.

=item *I<-MinIoSize>  

The minimum IO size to cache.

=item *I<-MaxIoSize> 

The maximum IO size to cache.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test.

=item Examples

=back

=head2 B<clearHgMetrics()>

Load additional XML files into SanMgr.

=over 4

=item Inputs

item *I<-Disks>  

List of data disks for which attributes need to be set.

=item *I<-SerialNumbers>  

List of serial numbers of the data disks for which attributes need to be set.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test.

=item Examples

=back

=head2 B<resetHgDevice()>

Initialize an EventMan object based on the <eventman> tags specified in the
XML configuration file.

=over 4

=item Inputs

item *I<-Disks>  

List of data disks for which attributes need to be set.

=item *I<-SerialNumbers>  

List of serial numbers of the data disks for which attributes need to be set.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test.

=item Examples

=back

=head2 B<formatHgCache()>
 
Access the Controller module

=over 4

=item Inputs

item *I<-ReplacementPolicy>  

Cache block replacement policy

=item *I<-BlockSize>  

Cache blocks size in bytes.

item *I<-BlocksPerChunck>  

Cache blocks per chunk (including overhead).

=item *I<-sizeLimitGb>  

Cache size limit in GB.

=item *I<-Async>  

Perform the format asynchronously.

=item *I<-ComputerName>  

Specifies a remote Windows computer on which you want to run teh command.

=item *I<-Credentials> 

Specifies a user account that you want to use to run this command.

=item Outputs

A hash that the .thpl file will use to check the test.

=back

=cut

