# -*- perl -*-
# $Id$

## @summary Mhost::UnixUtils
## @author  dl-coati-core
## @cf     false
## @status public
## @pod here

=head1 NAME

Mhost::UnixUtils - Routines to assist in working with unix hosts.

=head1 SYNOPSIS

  use Mhost::UnixUtils;

=head1 DESCRIPTION

This module provides routines to work with unix hosts.

=cut

package Mhost::UnixUtils;

use strict;
use Tharn;
use Params::Validate
    qw(validate validate_with BOOLEAN SCALAR ARRAYREF HASHREF OBJECT SCALARREF CODEREF);

###############################################################################
#Sub: Routine to run a unix command while handling error codes better than backticks
#Return values: array of lines in array context or string in scalar context
###############################################################################
sub shell_unix_cmd {
    my $me = (caller(0))[3];

    my %opt = validate_with(
        params => \@_,
        spec   => {
            cmd => { type => SCALAR },
        },
    );

    my $cmd             = $opt{cmd};

    # Only print out relatively short commands to the logs unless debug is enabled
    if ( length($cmd) < 100 ) {
        $Log->comment("$me: $cmd 2>&1");
    } else {
        $Log->debug(sub {"$me: $cmd 2>&1"});
    }

    # Redirect STDERR to STDOUT
    open( CMD_OUT, "$cmd 2>&1 |" ) || die "$me: during open on $cmd encountered error: $!";
    my @lines = <CMD_OUT>;
    close( CMD_OUT );
    my $rc = $?;

    chomp( @lines );
    @lines = map { $_ =~ s/\r//g; $_ } @lines;

    # Assumption is that all unix commands return 0 for good
    if ( $rc ) {
        die "$me: $cmd returned non-zero return code: $rc, \noutput: ".join("\n", @lines);
    }

    return ( wantarray ? @lines : join("\n", @lines) );
}

1;
