# $Id: //depot/prod/test/Rsierranevada/lib/MiniXFUN/Config.pm#1 $
#
# TODO: Improve path handling. The code uses splitpath inconsistently.
# TODO: Guess better defaults based on the user's site.

package MiniXFUN::Config;

require Exporter;
@ISA       = qw(Exporter);
@EXPORT_OK = qw($default_testbed testbed_config);

use strict;
use warnings;

use File::Spec::Functions qw(rel2abs catfile);

our $default_testbed = '/x/eng/testbeds/wafl/svl_minixfun';
our $config_filename = 'mini-xfun.config';
our @required_params = qw(id CONVERGED_ARCH MAYTAG_SUFFIX NDISP_ADDR);

sub testbed_config(;$) {
    my %config;

    # Verify existence of testbed.
    my $testbed_path = rel2abs(@_ ? $_[0] : $default_testbed);
    -d $testbed_path or die "$testbed_path: Unknown testbed\n";

    # Verify that it is a testbed.
    my $info_path = catfile($testbed_path, 'testbed.info');
    -d $testbed_path or die "$testbed_path: Not a testbed\n";

    # Verify presence of mini-xfun.config file.
    my $config_path = catfile($testbed_path, $config_filename);
    -f $config_path or die "$testbed_path: Missing $config_filename\n";

    # Read parameters from mini-xfun config file in testbed.
    open(CONFIG, $config_path) or die "$config_path: $!\n";
    foreach my $line (<CONFIG>) {
        chomp($line);

        # Skip blank lines.
        next if $line =~ m/^\s*(?:#.*)?$/;

        # Parse parameter assignments.
        my ($param, $value) = $line =~ m/^ \s*
            (\w+) \s*=\s*                               # parameter
            ((?:"[^"]*" | '[^']*' | [^[:space:]"']*)*)  # value
            (?:\s*|\s+\#.*)                             # optional comment
            $/x;
        defined($param) or die "$config_path: Syntax error\n";

        # Strip quotes.
        $value =~ s/"([^"]*)" | '([^']*)'/defined($1)?$1:$2/egx;

        # Record assignment.
        $config{$param} = $value;
    } ## end foreach my $line (<CONFIG>)

    # Check mini-xfun.config for required parameters.
    foreach my $param (@required_params) {
        exists($config{$param})
          or die "$config_path: Missing required parameter $param\n";
    }

    return %config;
} ## end sub testbed_config(;$)
