#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Mount/MountClass.pm#1 $
#
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.
#

package Mount::MountClass;

## @summary  
##   Provide an OS independent mount class
##
## @description  
##   Provides an interface for OS dependent mounts.
##
## @usage
##   use Mount::MountClass;
##
## @status	review
##
## @author  dl-maryv (FSS Protocols QA)
## @notes	Created by EP Network Storage Performance Lab, Inc.

use strict;
use Carp qw( croak );
use Tharn;
use Mount::Aix;
use Mount::Hpux;
use Mount::Linux;
use Mount::Solaris;
use vars qw( @ISA );

#-------------------------------------------------------
# Exported global variables
#-------------------------------------------------------
$MountClass::novalue = '_-NO_VALUE-_';

## @name ::new
## @description
##  Creates a new Mount Object. This routine will create a new
##  object by calling the platform specific 'new' constructor 
##  (in the platform specific mount package (i.e. Solaris.pm)). This way
##  the user uses a single API to do mounts and unmounts and the code
##  underneath creates the appropriate object based on the platform type.
##
## @synopsis 
##  $obj = MountClass->new($cc);
##
## @arg
##  cc		Required: The Connectrec object for the client.
##
## @return
##  Platform specific mount object

sub new ( $$ ) {
    my $class = shift;
    my $cc = shift;

    # Check for correct connection object
    croak "Not a 'Connectrec' object"
      if ! UNIVERSAL::isa($cc, "Connectrec");

    # Determine the right platform
    # and inherit correct OS dependent class.
    my $type = $cc->hostp()->hosttype();
    if ( $type =~ /unix-solaris/i ) {
        @ISA = qw(Mount::Solaris);
    } elsif ( $type =~ /unix-hpux/i ) {
        @ISA = qw(Mount::Hpux);
    } elsif ( $type =~ /unix-aix/i ) {
        @ISA = qw(Mount::Aix);
    } elsif ( $type =~ /unix-linux/i ) {
        @ISA = qw(Mount::Linux);
    } else {
        croak "MountClass::new does not support hosttype: $type";
    }

    # Call the constructor for the OS dependent class
   return ( SUPER::new $class );
}

1;
