#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/NASClient/NFS/Synth/Fedora.pm#1 $

## @summary Synthetic FedoraCore3 Linux client
## @author snider@netapp.com
## @status in progress
## @pod here

#
# This code is not yet ready. The library should also be renamed to be specific for version 3
#

## Fedora Core 3
package NASClient::NFS::Synth::Fedora;
use strict;
use warnings;
use base qw(NASClient::NFS::Synth);
use Data::Dumper;
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use TracePkg;
my $Trace = new TracePkg;

sub mount {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => OBJECT } ],
					 {
					     export => {type => SCALAR},
					 }
					 );
    $Trace->entry();

    $self->_nfs_null();
#    $self->mount_null();

    my $out = $self->_nfs_mount(path=>$params{export});
    if (!$out) {
        $Trace->fatal(msg=>"Could not mount $params{export}");
    }
    $self->_nfs_fsinfo(fhandle=>$out->{fhandle}) if ($self->{version} == 3);
    $Trace->exit(msg=>);
    return $out->{fhandle};
}

sub umount {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => OBJECT } ],
					 {
                                          # what should these be?
					     export => {type => SCALAR},
					     mntpnt => {type => SCALAR, optional => 1},
					 }
					 );
    $Trace->entry();

    my $out = $self->_nfs_umount(path=>$params{export});
    if (!$out) {
        $Trace->fatal(msg=>"Could not mount $params{export}");
    }
    $Trace->exit(msg=>);
}

sub stat {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => OBJECT,  } ],
					 {
					     handle => {type => SCALAR},
					 }
					 );
    $Trace->entry();
    if (0) { # newer linux
	my $access = $self->_nfs_access(handle=>$params{handle}, access=>0x1f);
	if ($access !~ /read/) {
	    $Trace->fatal(msg=>"Access not allowed on the dir: $access\n");
	}
    }
    my $out = $self->_nfs_getattr(fhandle=>$params{handle});
    my %stat = {$out->{fattr}};
    
    $Trace->exit();
    return %stat;
}

sub read {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => OBJECT,  } ],
					 {
					     handle => {type => SCALAR},
					     file_handle => {type => SCALAR | UNDEF, optional => 1 },
					     file => {type => SCALAR | UNDEF, optional => 1 },
					 }
					 );
    $Trace->entry();

    my $result = $self->_nfs_access(fhandle=>$params{handle}, access=>0x2d);
    if (!$result->{accessbits}) { # FIGURE THIS OUT!!!
	$Trace->fatal(msg=>"Read access not allowed on the dir: $result->{accessbits}\n");
    }
    
    my $fh = $params{file_handle};

    if (!$fh) {
	if (!$params{file}) {
	    $Trace->fatal(result=>"SCRIPT", msg=>"'file_handle' or 'file' must be passed in\n");
	}
        $result = $self->_nfs_lookup(dhandle => $params{handle},name => $params{file});
        if ($result->{status}) {
	    $Trace->fatal(msg=>"file '$params{file}' does not exist: $result->{status}\n");
	}
        $fh = $result->{fhandle};
    }
    $result = $self->_nfs_getattr(fhandle => $fh);
    my $attr = $result->{fattr};

    my $size = 0;
    my $eof = 0;
    while (!$eof) {
        my $count = $attr->{size} - $size;
        if ($count > $self->{rsize}) {
            $count = $self->{rsize};
        }
        $result = $self->_nfs_read(fhandle=>$fh, count=>$count, offset=>$size);
        $size += $result->{count};
        $eof = $result->{eof};
    }
    $Trace->exit();
    return $size;
}

sub readdir {
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => OBJECT,  } ],
					 {
					     handle => {type => SCALAR},
					 }
					 );
    # cached ls will simply perfrom the getattrs rather than a readdirplus again
    $Trace->entry();
    my @entries;
    my $result = $self->_nfs_access(fhandle=>$params{handle}, access=>0x1f);
    my $eof = 0;
    my $cookie = 0;
    my $verf = 0;
    if ($result->{accessbits}) { # FIGURE THIS OUT!!!
	$self->_nfs_getattr(fhandle=>$params{handle});
        while (!$eof) {
            $result = $self->_nfs_readdirplus(dhandle=>$params{handle}, cookie=>$cookie, cookieverf => $verf);
            $eof = $result->{eof};
            my $lentries = $result->{list};

            $verf = $result->{cookieverf};
            foreach my $entry (@{$lentries}) {
                if (!$entry->{fhandle}) {
                    $result = $self->_nfs_lookup(dhandle=>$params{handle}, name=>$entry->{name});
                    $entry->{fhandle} = $result->{fhandle};
                    $entry->{name_attr} = $result->{fattr};
                }
                push(@entries, $entry);
                $cookie = $entry->{cookie};
            }
        }
	# should handle NULL attributes
    } else {
	$Trace->fatal(msg=>"Access not allowed on the dir: $result->{accessbits}\n");
    }
    $Trace->exit();
    return @entries;
}


# TCP sends one MNT request

# GetPort MOUNT
# Mount EXPORT

# foreach export
#v2
# GetPort NFS
# NFS Null
# GetPort MOUNT
# Mount NULL
# Mount MNT
# NFS Getattr
# NFS Statfs

1;
