# this is basically -*- perl -*-
# $Id: $
#
## @summary Synthetic Linux Client
## @author	gvg@netapp.com
## @status	in progress
## @pod here

=head1 NAME

NASClient::NFS::Synth::Linux - Class definition for a Synthetic Linux NFS clients

=head1 DESCRIPTION

Please see the documentation in C<NASClient> which describes the API for all
of the NASClient objects. This class file defines methods that support all
of the synthetic Linux NFS clients.

=cut

# Linux
package NASClient::NFS::Synth::Linux;
use strict;
use warnings;
use base qw(NASClient::NFS::Synth);
use Data::Dumper;
use Module::Common qw(validate_mixed);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use nfs::NfsDef qw(:NFSCONSTANTS);
#use nfs::NfsDef qw(FATTR4_TYPE FATTR4_SIZE FATTR4_FSID FATTR4_FILEID
#  FATTR4_MODE FATTR4_NUMLINKS FATTR4_OWNER FATTR4_OWNER_GROUP FATTR4_RAWDEV
#  FATTR4_TIME_ACCESS FATTR4_TIME_METADATA FATTR4_TIME_MODIFY FATTR4_FILEHANDLE);
use TracePkg;
my $Trace = new TracePkg;

#-------------------------------------------------------------------
sub mount {
    my ($self, %params) = 
        validate_mixed(
                       @_,
                       [ { type => OBJECT } ],
                       {
                           export => {type => SCALAR},
                           mntpnt => {type => SCALAR | UNDEF, optional => 1,
                                      default => "/t/mnt"},
                           chmod  => {type => SCALAR, optional => 1 },
                       }
                       );
    my $out;
    my $nfs = $self->getnfs();
    $Trace->entry();

    # XXX - GV - Implement me
    # getport NFS and nfs->null happens by default with the call to null
    # getport mount
#    $self->mount_null();

    # XXX - GV
    # Should be updated to support all mount options

    # Clear out the saved mount port
    # to make sure portmapper is used
    $nfs->{mnt_port} = 0;
    
    $Trace->trace(msg=>"mount $self->{server}:$params{export} $params{mntpnt}");
    eval { $out = $nfs->mount(path => $params{export}); };
    if ($@) {
        $Trace->fatal(msg=>"Failed to mount $self->{server}:$params{export}: $@");
    }
    $self->{mountpoint} = [$params{mntpnt},$params{export}];
    if ($self->{version} == 3) {
        my $result = $self->_exe(_cmd => "fsinfo",
                                 fhandle=>$out->{fhandle});
        $self->_add_to_cache(path=>$params{mntpnt}, 
                             handle=>$out->{fhandle},
                             attr=>$result->{fattr}
                            );
    } else {
        $self->_add_to_cache(path=>$params{mntpnt}, 
                             handle=>$out->{fhandle},
                             attr=>undef,
                            );
    }

    # SHOULD THERE BE ACCESS HERE?
    
    if($params{chmod}) {
        if ($params{chmod} =~ /^0/) {
            $params{chmod} = oct($params{chmod});
        }
        $Trace->debug(msg=>"Chmod $params{chmod} on $params{export}");
        my %mode;
        if ( $self->{version} == 4 ) {
            $mode{FATTR4_MODE()} = $params{chmod};
        } else {
            $mode{mode} = $params{chmod};
        }
        $nfs->rpc()->auth_sys_set( uid => 0, gid => 0 );
        $self->_exe(_cmd => "setattr", 
                    fhandle => $out->{fhandle},
                    attr    => \%mode );
        $nfs->rpc()->auth_sys_set();
    }

    $Trace->exit(msg=>"return $params{mntpnt}");
    return $params{mntpnt};
}

#-------------------------------------------------------------------
sub umount {
    my ($self, %params) = 
        validate_mixed(
                       @_,
                       [ { type => OBJECT } ],
                       {
                           mntpnt => {type => SCALAR | UNDEF},
                       }
                       );
    my $nfs = $self->getnfs();
    $Trace->entry();

    if (!$self->{mountpoint}) {
        $Trace->fatal(msg=>"Did not find a cached mntpnt", result=>"SCRIPT");
    }
    my ($mount, $export) = @{$self->{mountpoint}};
    if ($mount ne $params{mntpnt}) {
        $Trace->fatal(msg=>"cached mntpnt '$mount' does not equal mntpnt '$params{mntpnt}'", result=>"SCRIPT");
    }

    # XXX - GV
    # Should be updated to support all umount options

    # Clear out the saved mount port
    # to make sure portmapper is used
    $nfs->{mnt_port} = 0;
    
    # getport mount
#    $self->_mount_null();

    eval { $self->_exe(_cmd => "umount", 
                       path => $export); };
    if ($@) {
        $Trace->fatal(msg=>"Failed to umount $self->{server}:$params{export}: $@");
    }
    $self->_clear_cache();
    delete $self->{mountpoint};

    $Trace->exit(msg=>);
    return;
}

#-------------------------------------------------------------------
sub _v3_open_for_read {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
               fhandle => {type => SCALAR},
              });
    $Trace->entry();
    # if the file is being opened for read, then the client checks the read access
    $self->_exe(_cmd => "access",
                fhandle => $params{fhandle}, 
                access => ACCESS_READ | ACCESS_MODIFY | ACCESS_EXTEND | ACCESS_EXECUTE);
    # TODO: verify the access or fail
    $Trace->exit();
}

#-------------------------------------------------------------------
sub _v3_open_for_write {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
               fhandle => {type => SCALAR},
              });
    $Trace->entry();
    # if the file is being opened for write, then the client checks the write access
    #  then truncates the file by setting the size to 0 and performs another getattr
    $self->_exe(_cmd => "access",
                fhandle => $params{fhandle}, 
                access => ACCESS_READ | ACCESS_MODIFY | ACCESS_EXTEND | ACCESS_EXECUTE);
    # TODO: verify the access or fail
    $self->_exe(_cmd => "setattr",
                attr => {
                         size => 0,
                        }
               );
    $self->_exe(_cmd => "getattr",
                fhandle => $params{fhandle});
    # TODO: verify the size was set to 0 and store the results in the cache

    $Trace->exit();
}

#-------------------------------------------------------------------
sub _v3_create {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
               dhandle => {type => SCALAR},
               name => {type => SCALAR}, 
              });
    $Trace->entry();
    
    # Create the new file with the proper attributes
    my (%mode);
    $mode{mode} = 0100666;
    my $out = $self->_exe(_cmd => "create",
                       name => $params{name},
                       dhandle => $params{dhandle},
                       cr_mode => UNCHECKED,
                       attr    => \%mode,);
    my $handle = $out->{fhandle};
    # XXX - GV
    # Hmm... NFSv3 GETACL was sent and received.
    # Wondering what to do with it.

    $Trace->exit();
    return $handle;
}

#-------------------------------------------------------------------
sub _v4_open {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
               fhandle => {type => SCALAR|UNDEF},
               dhandle => {type => SCALAR},
               file => {type => SCALAR}, 
              });
    $Trace->entry();

    # Access call on dir
    $nfs->access(fhandle => $params{dhandle}, 
                 access => 0x1f );
    ## 0x1f => read|lookup|modify|extend|delete|not-execute

    # GetAttr call on dir
    $nfs->getattr(fhandle => $params{dhandle});
    
    # These two calls are confirmed *not* being sent during open,
    # hence commenting them out.
    # setclientid
    # $nfs->setclientid();    
    # setclientid_confirm
    ## XXX - GV
    ## Rhel5 seems to be sending 3 ops, setclientid_confirm, PUTROOTFH and
    ## GETATTR. But the following function does only the first one.
    # $nfs->setclientid_confirm();
    
    ## XXX - GV
    # Underlying open sends only 3 OPs PUTFH, OPEN, GETFH
    # But Rhel5 sends 7 OPs - PUTFH, SAVEFH, OPEN, GETFH, GETATTR, 
    # RESTOREFH and GETATTR        
    my %mode;
    $mode{FATTR4_MODE()} = 0777;
    my $out = $nfs->open(name    => $params{file},
                         dhandle => $params{dhandle},
                         cr_mode => UNCHECKED,
                         access  => OPEN4_SHARE_ACCESS_BOTH,
                         attr    => \%mode,);
    my $fhandle = $out->{results}[2]{result}{fhandle};
    $nfs->{stateid} = $out->{results}[1]{result}{stateid};
    $self->set_stateid(fhandle => $fhandle,
                       stateid => $nfs->{stateid});
    
    ## XXX - GV
    # Send PUTFH, SETATTR, GETATTR
    $Trace->exit();
    return $fhandle;
}

#-------------------------------------------------------------------
# The packet sequence in this routine matches what a real client
#   sends when the open() call is made from perl.
sub open {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
                  file    => {type => SCALAR,},
                  mode    => {type => SCALAR,},
                  dhandle => {type => SCALAR, default => $nfs->mnt_fh()},
              });
    $Trace->entry(msg=>"$params{file}");

    my ($out);

    if (!grep($params{mode} eq $_, qw(r w))) {
        $Trace->fatal(msg=>"The specified mode '$params{mode}' must either be 'r' or 'w'", result=>"SCRIPT");
    }

    # find the filehandle for the given file
    my $handle = $self->_get_handle(object=>$params{file});

    # strip off any leading path information so that we just have the name of file
    my $object = $params{file};
    $object =~ s!.+/!!;

    if ($handle) {
        if ($params{mode} eq "r") {
            if ($self->{version} == 3) {
                $self->_v3_open_for_read(fhandle => $handle);
            } elsif ($self->{version} == 4) {
                $handle = $self->_v4_open(file=>$object, fhandle => $handle, dhandle => $params{dhandle});
            }
        } elsif ($params{mode} eq "w") {
            if ($self->{version} == 3) {
                $self->_v3_open_for_write(fhandle => $handle);
            } elsif ($self->{version} == 4) {
                $self->_v4_open(file=>$object, fhandle => $handle, dhandle => $params{dhandle});
            }
        }                
    } else {
        # The file did not previously exist
        if($params{mode} eq "w") {
            if($self->{version} == 3) {
                $handle = $self->_v3_create(dhandle => $params{dhandle}, name => $object);
            } elsif ($self->{version} == 4) {
                $handle = $self->_v4_open(file=>$object, fhandle => $handle, dhandle => $params{dhandle});
            }
        } else {
            $Trace->fatal(msg=>"$class->open() failed: $@");
        }        
    }

    $Trace->exit();
    # return the filehandle for the given file
    return $handle;
}

#-------------------------------------------------------------------
# the client does nothing when the close is called
sub close {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
               fhandle => {type => SCALAR,},
              });
    $Trace->entry();

    if($self->{version} == 4) {
        # XXX - GV
        # Rhel5 sends 3 OPs PUTFH, CLOSE, GETATTR, but this
        # function sends only the first two OPs.
        my $stateid = $self->get_stateid(fhandle => $params{fhandle},);
        $nfs->close(fhandle => $params{fhandle},
                    stateid => $stateid);
        # Close must be successful, so delete the stateid saved during open
        $self->delete_stateid(fhandle => $params{fhandle});
    } else {
        # Do nothing - NO Op
    }
    $Trace->exit();
}

#-------------------------------------------------------------------
# The packet sequence in this routine matches what a real client
#   sends when the opendir() call is made from perl.
sub opendir {
    my $self = shift;
    my (%params) = validate(
			    @_,
			    {
                             directory => {type => SCALAR },
			    }
			    );
    my $nfs = $self->getnfs();
    $Trace->entry(msg=>"directory=$params{directory}");
    
    # find the filehandle for the given direcory
    my $handle = $self->_get_handle(object=>$params{directory});

    if ($self->{version} != 2) {
        # call access on the directory
        my $results = $self->_exe(_cmd => "access",
                                  fhandle => $handle, 
                                  access => ACCESS_READ | ACCESS_LOOKUP | ACCESS_MODIFY |ACCESS_DELETE | ACCESS_EXTEND);
    }
    # TODO: verify the access
    
    $Trace->exit();
    return $handle;
}

#-------------------------------------------------------------------
# the client does nothing when the close is called
sub closedir {
    my $self = shift;
    my (%params) = validate(
			    @_,
			    {
				handle => {type => SCALAR},
			    }
			    );
    $Trace->entry();

    # is there anything to do here?
    
    $Trace->exit();
}

#-------------------------------------------------------------------
# The packet sequence in this routine matches what a real client
#   sends when the same read loop is executed from a perl script.
# TODO:
#  * support reading from parts of the file rather than the whole thing
#  * support multiple read requests in flight
#    the client sent 4 requests, waited on all, sent 5 requests, waited,
#    then 6 requests, waited
sub read {
    my $self = shift;
    my (%params) = validate(
	      @_,
	      {
                  fhandle => {type => SCALAR}, 
                  offset  => {type => SCALAR, default => 0},
                  count   => {type => SCALAR | UNDEF, optional => 1},
              });
    $Trace->entry();
    my $nfs = $self->getnfs();

    # This code expects that is already has a valid filehandle provided
    my $fhandle = $params{fhandle};

#    my $remaining = $self->{attrs}{$fhandle}{size};

    # Start reading in the file at the provided offset
    my $offset = $params{offset};
#    my $count = ($remaining > $self->{rsize}) ? $self->{rsize} : $remaining - $offset;

    # 'rsize' defines how much data to read in each NFS read call
    my $count = $self->{rsize};

    # convenience variables
    my $eof = 0;
    my $total = 0;
    my $data = "";

    # Keep reading from the file until the server returns eof.
    # After each read, the offset for the next read call is 
    #  incremented by the amount of data we just read. The 
    #  running total of bytes read from the far is also kept
    #  so that it may be passed back to the caller. The actual
    #  data read is also stored for return.
    eval {
        while (!$eof) {
            my $out = $self->_exe(_cmd=>"read",
                                  fhandle => $fhandle,
                                  offset  => $offset,
                                  count   => $count,
                                  stateid => 0);
            my $rcount;
            if ($self->{version} == 4) {
                $rcount = length($out->{data});
            } else {
                $rcount = $out->{count};
            }
            $eof = $out->{eof};
            $offset += $rcount;
            $total += $rcount;
            $data .= $out->{data};
        }
    };
    $Trace->fatal(msg=>"read() failed with \"$@\" for args ".Dumper(\%params)) if $@;
    
    $Trace->exit(msg=>"read $total bytes");

    # return either the total bytes read and the actual data or just the total
    return (wantarray) ? ($total, $data) : $total;
}

#-------------------------------------------------------------------
# The packet sequence in this routine matches what a real client
#   sends when the same write loop is executed from a perl script.
# TODO:
#  * support writing to parts of the file rather than the whole thing
#  * support multiple write requests in flight
#    the client sent 2 requests, waited on all, then went to 4
sub write {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
                  fhandle => {type => SCALAR,},
                  offset  => {type => SCALAR,},
                  data    => {type => SCALAR,},
                  pre_getattr => {type => SCALAR | UNDEF, optional => 1, default => 0},
                  post_getattr => {type => SCALAR | UNDEF, optional => 1, default => 1},
              });
    $Trace->entry();
    
    # Start reading in the file at the provided offset
    my $offset = 0;

    # 'wsize' defines how much data to write in each NFS read call
    my $wsize = $self->{wsize};

    # convenience variables
    my $out;
    my $buf;

    # find out how much data is intended to be written
    my $remaining = length($params{data});

    if ($params{pre_getattr}) {
        # the client performs a getattr after the writed are finished
        $self->_exe(_cmd=>"getattr",
                    fhandle => $params{fhandle});
    }

    # Keep writing to the file until all of the data has been sent.
    # After each write, the offset for the next write call is 
    #  incremented by the amount of data we just wrote. The 
    #  running total of bytes left to be written is decremented
    #  by the amount just written.
    eval {
        while ($remaining > 0) {
            my $count;
            my $buf = substr($params{data}, $offset, $wsize);
            $Trace->trace(msg=>" write($offset, ".length($buf).")");
            if($self->{version} == 4) {    
                $out = $self->_exe(_cmd=>"write",
                                   fhandle => $params{fhandle},
                                   data    => $buf,
                                   stable => UNSTABLE,
                                   stateid => $nfs->{stateid},
                                   offset  => $offset);
                foreach my $result (@{$out->{results}}) {
                    # is this the write op? 
                    if ($result->{op} == 38) {
                        # a failure might be caught in _exe but not sure
                        if ($result->{status}) {
                            $Trace->fatal(msg=>"write failed with status $result->{status}");
                        }
                        $count = $result->{result}{count};
                    }
                }
            } else {
                $out = $self->_exe(_cmd=>"write",
                                   fhandle => $params{fhandle},
                                   data    => $buf,
                                   stable => UNSTABLE,
                                   offset  => $offset);
                $count = $out->{count};
            }
            $offset += $count;
            $remaining -= $count;
            $Trace->debug(msg=>" $remaining bytes remaining");
        }

        if ($params{post_getattr}) {
            # the client performs a getattr after the writed are finished
            $self->_exe(_cmd=>"getattr",
                        fhandle => $params{fhandle});
        }
        # TODO: cache the file and attributes
    };
    $Trace->fatal(msg=>"write() failed with \"$@\"") if $@;
    $Trace->exit();
}

#-------------------------------------------------------------------
# The packet sequence in this routine matches what a real client
#   sends when the unlink() call is made from perl.
# TODO: this code probably does not work if a full path is not given
sub unlink {
    my $self = shift;
    my $class = ref($self);
    my $nfs = $self->getnfs();
    my (%params) = validate(
	      @_,
	      {
                  file    => {type => SCALAR,},
                  dhandle => {type => SCALAR, default => $nfs->mnt_fh()},
              });
    $Trace->entry();

    if($self->{version} == 3 || $self->{version} == 4) {
        ## XXX - GV
        ## for NFSv4, underlying remove does not send GETATTR as part of the
        ## remove compound request. Need to be seen how to implement it

        # find the name of the file in the provided path
        my $file = $params{file};
        $file =~ s!.+/!!;

        # find the parent directory of the given file
        my $dir = $params{file};
        $dir =~ s!/$file!!;

        # find the filehandle for the parent directory
        my $dhandle = $self->_get_handle(object=>$dir);

        # the client verifies that the file exists
        $self->_exe(_cmd=>"lookup",
                    name => $file, dhandle => $dhandle);
        # TODO: what happens if it fails?

        # the client calls remove on the given file
        $self->_exe(_cmd=>"remove",
                    name => $file, dhandle => $params{dhandle});

    } else {
        $Trace->fatal(msg=>"$class->unlink() has not been implemented yet for $self->{version}");
    }
    
    $Trace->exit();
}

#-------------------------------------------------------------------
# The packet sequence in this routine matches what a real client
#   sends when the same readdir loop is executed from a perl script.
# Snider: not sure if the stat param is needed any more
sub readdir {
    my $self = shift;
    my (%params) = validate(
			    @_,
			    {
				handle => SCALAR,
                                directory => {type => SCALAR},
                                stat => {type => SCALAR | UNDEF, optional => 1 },
			    }
			    );
    $Trace->entry();
    my @readdir;

    my $result;
    my $eof = 0;
    my $cookie = 0;
    my $verf = 0;
    my $nfs = $self->getnfs();

    # repeatedly call the readdirplus routine, passing the appropriate cookie and verf,
    #  until the filer replies with eof
    while (!$eof) {
        # call readdirplus with the last known cookie (starts at 0)
        if ($self->{version} == 3) {
            $result = $self->_exe(_cmd=>"readdirplus",
                                  dhandle=>$params{handle}, cookie=>$cookie, 
                                  cookieverf => $verf, dircount=>512, maxcount=>4096);
            # save the verifier and eof flag which is used to control our loop
            $eof = $result->{eof};
            $verf = $result->{cookieverf};
            
            # grab the current entries. Add them to our list and keep track of the latest
            #  cookie we got (our cookie variable will contain the last cookie received)
            my $lentries = $result->{list};
            foreach my $entry (@{$lentries}) {
                push(@readdir, $entry);
                $cookie = $entry->{cookie};
            }
        } elsif ($self->{version} == 4) {
            my @attr = (
                        FATTR4_TYPE(),
                        FATTR4_SIZE(),
                        FATTR4_FSID(),
                        FATTR4_FILEID(),
                        FATTR4_MODE(),
                        FATTR4_NUMLINKS(),
                        FATTR4_OWNER(),
                        FATTR4_OWNER_GROUP(),
                        FATTR4_RAWDEV(),
                        FATTR4_TIME_ACCESS(),
                        FATTR4_TIME_METADATA(),
                        FATTR4_TIME_MODIFY(),
                        FATTR4_FILEHANDLE(),
                       );

            my $out = $nfs->readdir(dhandle=>$params{handle}, cookie => $cookie,
                                    cookieverf => $verf, attr=> \@attr);
            foreach my $result (@{$out->{results}}) {
                # is this the right op? 
                if ($result->{op} == 26) {
                    # a failure might be caught in _exe but not sure
                    if ($result->{status}) {
                        $Trace->fatal(msg=>"readdir failed with status $result->{status}");
                    }
                    $eof = $result->{result}{eof};
                    $verf = $result->{result}{cookieverf};
                    # grab the current entries. Add them to our list and keep track of the latest
                    #  cookie we got (our cookie variable will contain the last cookie received)
                    my $lentries = $result->{result}{list};
                    my $files = "";
                    foreach my $entry (@{$lentries}) {
                        $files .= " $entry->{name}";
                        push(@readdir, $entry);
                        $cookie = $entry->{cookie};
                        # HACK HACK HACK
                        # convert the attributes from numbers to names. This could use
                        # some work to better match v3 and local attributes. We need
                        # a better way to standardize this
                        foreach my $attr (keys %{$entry->{fattr}{attr_vals}}) {
                            if ($attr == FATTR4_TYPE) {
                                $entry->{name_attr}{type} = $entry->{fattr}{attr_vals}{$attr};
                            }
                            if ($attr == FATTR4_FILEHANDLE) {
                                $entry->{fhandle} = $entry->{fattr}{attr_vals}{$attr};
                            }
                        }
                    }
                    $Trace->debug(msg=>"eof=$eof,cookie=$cookie,$files\n");
                }
            }
        }
    }
    # should handle NULL attributes

    # go through each entry in our final list and verify that we got a handle
    #  and attributes. Go back and perform a lookup on an entry that did not
    #  have them and add them to our list. GX does not always return attributes
    #  on non-cached junctions.
    my @entries;
    foreach my $entry (@readdir) {
        if ($params{stat} || !$entry->{fhandle} || ! keys %{$entry->{name_attr}}) {
            $result = $self->_exe(_cmd=>"lookup",
                                  dhandle=>$params{handle}, name=>$entry->{name});
            $entry->{fhandle} = $result->{fhandle};
            $entry->{name_attr} = $result->{fattr};
        }
        push(@entries, $entry); # push to our final list

        # skip adding . and .. to the internal cache
        next if (grep($entry->{name} eq $_, qw(. ..)));

        # cache the info on each entry
        $self->_add_to_cache(path=>"$params{directory}/$entry->{name}",
                             handle => $entry->{fhandle},
                             attr => $entry->{name_attr},
                             );
    }
    $Trace->exit();

    # return a reference to the list of entries
    return \@entries;
}

#-------------------------------------------------------------------
# This is a convenience routine to open a file, read it, and then close it
# buf_size is used by a real client but not in this code path
sub read_file {
    my $self = shift;
    my (%params) = validate(
	      @_,
	      {
                  file    => {type => SCALAR},
                  offset  => {type => SCALAR, default => 0},
                  count   => {type => SCALAR | UNDEF, optional => 1},
                  buf_size => {type => SCALAR | UNDEF, optional => 1}, # unused
              });
    $Trace->entry(msg=>"$params{file}");

    # open the file for reading
    my $handle = $self->open(file=>$params{file}, mode=>"r");

    # call the read routine to read the given file
    my ($bytes, $contents) = $self->read(fhandle=>$handle, offset=>$params{offset},
                                         count=>$params{count});
    
    # now close the file
    $self->close(fhandle=>$handle);
    $Trace->exit(msg=>"return $bytes bytes");

    # return either the bytes read and the contents or just the bytes read
    return (wantarray) ? ($bytes, $contents) : $bytes;
}

#-------------------------------------------------------------------
# This is a convenience routine to open a file, write to it, and then close it
# buf_size is used by a real client but not in this code path
sub write_file {
    my $self = shift;
    my (%params) = validate(
	      @_,
	      {
                  file    => {type => SCALAR},
                  offset  => {type => SCALAR, default => 0},
                  count   => {type => SCALAR | UNDEF, optional => 1},
                  buf_size => {type => SCALAR | UNDEF, default => 8192},
                  data => {type => SCALAR | UNDEF, optional => 1},
              });
    $Trace->entry();
    $Trace->trace(msg=>"write($params{file}, $params{offset}, ".length($params{data}).")");

    # open the file for writing
    my $handle = $self->open(file=>$params{file}, mode=>"w");

    # call the write routine to write to the given file
    $self->write(fhandle=>$handle, offset=>$params{offset}, data=>$params{data});

    # now close the file
    $self->close(fhandle=>$handle);
    $Trace->exit();
}

#-------------------------------------------------------------------
# This is a convenience routine to open a directory, read the contents, 
# and then close it and return the entries
sub ls {
    my $self = shift;
    my (%params) = validate(
			    @_,
			    {
                             directory => {type => SCALAR },
                             stat => {type => SCALAR | UNDEF, optional => 1 },
			    }
			    );
    $Trace->entry();

    my $dir_handle = $self->opendir(directory => $params{directory});
    my $entries = $self->readdir(handle=>$dir_handle, %params);
    $self->closedir(handle=>$dir_handle);
    
    $Trace->exit(msg=>"return ".scalar @$entries." entries");
    return $entries;
}

#-------------------------------------------------------------------
sub fpolicy_read {
    my $self = shift;
    my (%params) = validate(
			    @_,
			    {
                                dhandle => {type => SCALAR, default => $self->{curr_fh}},
                                name    => {type => SCALAR},
                                offset  => {type => SCALAR, default => 0},
                                count   => {type => SCALAR, default => 1024},
			    }
			    );
    my $nfs = $self->getnfs();
    $Trace->entry();

    my ($out, $fsize, $count, $fhandle);

    # Lookup
    $out = $self->_exe(_cmd=>"lookup",
                       name    => $params{name},
                       dhandle => $params{dhandle});
    $fhandle = $self->{curr_fh};

    # GetAttr
    $out = $self->_exe(_cmd=>"getattr", fhandle => $fhandle);
    $fsize = $out->{fattr}->{size};
    
    # Access
    $out = $self->_exe(_cmd=>"access", 
                       fhandle => $fhandle, 
                       access => ACCESS_READ | ACCESS_LOOKUP );
    
    # Read
    $count = ($fsize < $params{count}) ? $fsize : $params{count};
    $out = $self->_exe(_cmd=>"read",
                       fhandle => $fhandle,
                       offset  => $params{offset},
                       count   => $count);

    if($out->{data} =~ /migrtime=/i) {
        $Trace->warn(msg=>"Looks like stub file ".$out->{name}." was given by server. Data found was:\n ".$out->{data});
    }
    
    $Trace->exit();
}


1;
