# this is basically -*- perl -*-
# $Id: $
#
## @summary Synthetic RHEL3 Linux Client
## @description This is not the exact Rhel3 client yet, but close.
## @author	gvg@netapp.com,snider@netapp.com
## @status	in progress
## @pod here

## RHEL 3
package NASClient::NFS::Synth::Rhel3;
use strict;
use warnings;
use base qw(NASClient::NFS::Synth::Linux);
use Data::Dumper;
use Module::Common qw(validate_mixed);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use nfs::NfsDef qw(:NFSCONSTANTS);
use TracePkg;
my $Trace = new TracePkg;

=head1 NAME

NASClient::NFS::Synth::Rhel4 - Class definition for a Redhat 4 NAS client

=head1 DESCRIPTION

Please see the documentation in C<NASClient> which describes the API for all
of the NASClient objects.

This class creates an object which maps the NASClient API to the actual
implementation for a Redhat Enterprise Linux version 4 client. For 
example, the NASClient API defines a method called ls() which is 
supposed to run the equivalent of an 'ls' on the given filesystem. The
ls() implemented in this class will send the proper sequence of NFS
calls to the filer based upon the observed sequence on a real Rhel4 client.

=cut

#-------------------------------------------------------------------
# Snider notes: my testing of this package has been primarily for v3
#-------------------------------------------------------------------


#-------------------------------------------------------------------
sub mount {
    my ($self, %params) = 
        validate_mixed(
                       @_,
                       [ { type => OBJECT } ],
                       {
                           export => {type => SCALAR},
                           mntpnt => {type => SCALAR | UNDEF, optional => 1,
                                      default => "/t/mnt"},
                           chmod  => {type => SCALAR, optional => 1 },
                       }
                      );
    my $out;
    my $nfs = $self->getnfs();
    $Trace->entry();

    # XXX - GV
    # Portmap V2 DUMP call

    # XXX - GV
    # Should be updated to support all mount options

    # Clear out the saved mount port
    # to make sure portmapper is used
    $nfs->{mnt_port} = 0;
    
    $Trace->trace(msg=>"Sending mount request to $self->{server} for $params{export}");
    eval { $out = $self->_exe(_cmd => "mount",
                              path => $params{export}); };
    if ($@) {
        $Trace->fatal(msg=>"Failed to mount $self->{server}:$params{export}: $@");
    }
    
    # XXX - GV
    # Portmap V2 GETPORT call
    $self->{mountpoint} = [$params{mntpnt},$params{export}];
    
    if ($self->{version} == 3) {
        $self->_exe(_cmd => "getattr");
        my $result = $self->_exe(_cmd => "fsinfo",
                                 fhandle=>$out->{fhandle});
        $self->_add_to_cache(path=>$params{mntpnt}, 
                             handle=>$out->{fhandle},
                             attr=>$result->{fattr}
                            );
    } else {
        $self->_add_to_cache(path=>$params{mntpnt}, 
                             handle=>$out->{fhandle},
                             attr=>undef,
                            );
    }
    
    if($params{chmod}) {
        if ($params{chmod} =~ /^0/) {
            $params{chmod} = oct($params{chmod});
        }
        $Trace->debug(msg=>"Chmod $params{chmod} on $params{export}");
        my %mode;
        if ( $self->{version} == 4 ) {
            $mode{FATTR4_MODE()} = $params{chmod};
        } else {
            $mode{mode} = $params{chmod};
        }
        $nfs->{rpc}->auth_sys_set( uid => 0, gid => 0 );
        $self->_exe(_cmd => "setattr", 
                    fhandle => $out->{fhandle},
                    attr    => \%mode );
        $nfs->{rpc}->auth_sys_set();
    }

    $Trace->exit(msg=>);
    return $params{mntpnt};
}

#-------------------------------------------------------------------
sub getfacl {
    my $self = shift;
    my $class = ref($self);
    $Trace->fatal(msg=>"$class->getfacl() is not supported by Linux Rhel3.", result=>"SCRIPT");
}

#-------------------------------------------------------------------
sub setfacl {
    my $self = shift;
    my $class = ref($self);
    $Trace->fatal(msg=>"$class->setfacl() is not supported by Linux Rhel3.", result=>"SCRIPT");
}

sub _set_defaults {
    my $self = shift;
    $self->{jukebox_sleep} = 15;
    $self->{jukebox} = 1;
}
1;
