# this is basically -*- perl -*-
# $Id: $
#
## @summary Synthetic RHEL5 Linux Client
## @description This is not the exact Rhel5 client yet, but close.
## @author	gvg@netapp.com
## @status	in progress
## @pod here

# after the ls(), rhel5 does a fsinfo and pathconf calls before the stats. 
# It only does this when the ls is on the mountpoint.
# The ls() uses readdir on really big directories...hmm...

## RHEL 5
package NASClient::NFS::Synth::Rhel5;
use strict;
use warnings;
use base qw(NASClient::NFS::Synth::Linux);
use Data::Dumper;
use Module::Common qw(validate_mixed);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use nfs::NfsDef qw(:NFSCONSTANTS);
use TracePkg;
my $Trace = new TracePkg;

=head1 NAME

NASClient::NFS::Synth::Rhel5 - Class definition for a Redhat 5 NAS client

=head1 DESCRIPTION

Please see the documentation in C<NASClient> which describes the API for all
of the NASClient objects.

This class creates an object which maps the NASClient API to the actual
implementation for a Redhat Enterprise Linux version 5 client. For 
example, the NASClient API defines a method called ls() which is 
supposed to run the equivalent of an 'ls' on the given filesystem. The
ls() implemented in this class will send the proper sequence of NFS
calls to the filer based upon the observed sequence on a real Rhel5 client.

=cut

#-------------------------------------------------------------------
sub mount {
    my ($self, %params) = 
        validate_mixed(
                       @_,
                       [ { type => OBJECT } ],
                       {
                           export => {type => SCALAR},
                           mntpnt => {type => SCALAR | UNDEF, optional => 1,
                                      default => "/t/mnt"},
                           chmod  => {type => SCALAR, optional => 1 },
                       }
                      );
    my $out;
    my $nfs = $self->getnfs();
    $Trace->entry();

    # XXX - GV
    # Portmap V2 DUMP call

    # XXX - GV
    # Should be updated to support all mount options

    # Clear out the saved mount port
    # to make sure portmapper is used
    $nfs->{mnt_port} = 0;
    
    $Trace->trace(msg=>"Sending mount request to $self->{server} for $params{export}");
    eval { $out = $nfs->mount(path => $params{export}); };
    if ($@) {
        $Trace->fatal(msg=>"Failed to mount $self->{server}:$params{export}: $@");
    }
    
    # XXX - GV
    # Portmap V2 GETPORT call
    $self->{mountpoint} = [$params{mntpnt},$params{export}];
    
    if ($self->{version} == 3) {
        $self->_exe(_cmd => "fsinfo",
                    fhandle=>$out->{fhandle});
        my $result = $self->_exe(_cmd => "fsinfo",
                                 fhandle=>$out->{fhandle});
        $self->_add_to_cache(path=>$params{mntpnt}, 
                             handle=>$out->{fhandle},
                             attr=>$result->{fattr}
                            );
        $self->_exe(_cmd => "access",
                    fhandle => $out->{fhandle},
                    access => 0x1f );
    } else {
        $self->_add_to_cache(path=>$params{mntpnt}, 
                             handle=>$out->{fhandle},
                             attr=>undef,
                            );
    }
    
    if($params{chmod}) {
        if ($params{chmod} =~ /^0/) {
            $params{chmod} = oct($params{chmod});
        }
        $Trace->debug(msg=>"Chmod $params{chmod} on $params{export}");
        my %mode;
        if ( $self->{version} == 4 ) {
            $mode{FATTR4_MODE()} = $params{chmod};
        } else {
            $mode{mode} = $params{chmod};
        }
        $nfs->rpc()->auth_sys_set( uid => 0, gid => 0 );
        $self->_exe(_cmd => "setattr", 
                    fhandle => $out->{fhandle},
                    attr    => \%mode );
        $nfs->rpc()->auth_sys_set();
    }

    $Trace->exit(msg=>);
    return $params{mntpnt};
}

#---------------------------------------------------------------------
sub getfacl {
    my $self = shift;
    my $nfs = $self->getnfs();
    my (%params) = 
        validate(
                 @_,
                 {
                     dhandle => {type => SCALAR, default => $nfs->mnt_fh()},
                     file    => {type => SCALAR},
                 }
                 );
    $Trace->entry();
    
    my ($out, $num_retries, $i, $acl);
    $num_retries = 3;
    $i = $num_retries;

    # find the filehandle for the given file
    my $fhandle = $self->_get_handle(object=>$params{file});
    
    # Retry the command if the filer is returning NFS4ERR_DELAY
    do {
        if ($i != $num_retries) {
            $Trace->debug(msg=>"retrying getacl command due to NFS4ERR_DELAY");
            sleep (5);
        }
        eval { $out = $nfs->getattr(fhandle => $fhandle,
                                    attr    => [ FATTR4_ACL() ]) };
    } while ($i-- > 0 && $@ =~ /NFS4ERR_DELAY/);
    $Trace->fatal(msg=>"getfacl: synthetic client could not get attributes: $@") if $@;
    
    # Put the ACL on the object
    $acl->{ace} = $out->{fattr}{FATTR4_ACL()};
    $acl = $self->_synth_to_acl($acl);

    $Trace->exit(msg=>);
    return $acl;
}

#---------------------------------------------------------------------
sub setfacl {
    my $self = shift;
    my $nfs = $self->getnfs();
    my (%params) = 
        validate(
                 @_,
                 {
                     dhandle => {type => SCALAR, default => $nfs->mnt_fh()},
                     file    => {type => SCALAR},
                     acl     => {type => HASHREF,},
                 }
                 );
    $Trace->entry();

    my ($num_retries, $i, $attrs);
    $num_retries = 3;
    $i = $num_retries;

    # find the filehandle for the given file
    my $fhandle = $self->_get_handle(object=>$params{file});
    
    my $acl = $self->_acl_to_synth($params{acl});
    $attrs->{FATTR4_ACL()} = $acl->{ace};
    # Retry the command if the filer is returning NFS4ERR_DELAY
    do {
        if ($i != $num_retries) {
            $Trace->debug(msg=>"retrying setacl command due to NFS4ERR_DELAY");
            sleep (5);
        }
        eval { $nfs->setattr(fhandle => $fhandle,
                             attr    => $attrs ) };
    } while ($i-- > 0 && $@ =~ /NFS4ERR_DELAY/);
    die "synthetic client could not set attributes: $@" if $@;

    $Trace->exit(msg=>);
    return;
}

#-------------------------------------------------------------------
sub write {
    my $self = shift;
    $Trace->entry();
    $self->NASClient::NFS::Synth::Linux::write(@_, 
                                               post_getattr => undef,
                                               pre_getattr => undef);
    $Trace->exit();
}
1;
