package NBU::Client;

use NBU::Command;

@ISA = qw(NBU::Command);

use Data::Dumper;
use strict;

############################# Accessors ##############################

sub set_policy {
	my ( $self, $p ) = @_;
	die "policy must be an instance of NBU::Policy" unless ref( $p ) eq 'NBU::Policy';
	$self->{policy} = $p;
}
sub get_policy { my ( $self ) = @_; return $self->{policy} }

sub set_name {
	my ( $self, $name ) = @_;
	die "name must have a length" unless length $name;
	$self->{name} = $name;
}
sub get_name { my ( $self ) = @_; return $self->{name} }

#sub set_Solaris {
#	my ( $self, $solaris ) = @_;
#	die "Solaris must take the form: 'Solaris<n>'" unless $solaris =~ /^Solaris\d+/;
#	$self->{Solaris} = $solaris;
#}
#sub get_Solaris { my ( $self ) = @_; return $self->{Solaris} }

sub set_os {
	my ( $self, $host_config ) = @_;
	
	die "os must be /^(Solaris|Windows)/" unless $host_config =~ /^(Solaris|Windows)/;
	
	if ( $host_config =~ /^Solaris/ ) {
		$self->{hardware} = "Solaris";
	} elsif ( $host_config =~ /^Windows/ ) {
		$self->{hardware} = "PC";
	}
 
	$self->{os} = $host_config;
}
sub get_os { my ( $self ) = @_; return $self->{os} }
sub get_hardware { my ( $self ) = @_; return $self->{hardware} }

sub to_create_string {

	my ( $self ) = @_;

	my $string .= $NBU::Command::ROOT . 'bin/admincmd/bpplclients';
	$string .= " $self->{policy}{name}";
	$string .= " -M $self->{policy}{storage_unit}{M}";
	$string .= " -add $self->{name}";
	$string .= " $self->{hardware} $self->{os}" if defined $self->{os};
	
	return $string;
}

sub to_delete_string {

	my ( $self ) = @_;
	
	my $string .= $NBU::Command::ROOT . 'bin/admincmd/bpplclients';
	$string .= " $self->{policy}{name}";
	$string .= " -M $self->{policy}{storage_unit}{M}";
	$string .= " -delete $self->{name}";
	$string .= " Solaris $self->{Solaris}" if defined $self->{Solaris};
	
	return $string;
}

sub create_results {

	my ( $self ) = @_;
	
	return 0 if $NBU::Command::NOOP;
	
	return 0;
}

sub delete_results {

	my ( $self ) = @_;
	
	return 0 if $NBU::Command::NOOP;
		
	return 0;
	
}

1;

=head1 NAME

  NBU::Client - A Client to Backup with NetBackup
  
=head1 DESCRIPTION

  This is a wrapper around the bpplclients command.  The object caches command 
  results internally.  For the most part, the variable names are the same 
  as the NBU command switches.
  
=head1 SYNOPSIS

  use NBU::Command;
  use NBU::StorageUnit;
  use NBU::Policy;
  use NBU::Client;
  
  $NBU::Command::VERBOSE = 1;

  my $s = NBU::StorageUnit->new;
  ... set values

  my $p = NBU::Policy->new;
  ... set values

  # For p2 and p3

  my $t = NBU::Client->new;
  $t->set_policy( $p );                         # The policy to which to add the client
  $t->set_name( 'perf280er.lab.netapp.com' );   # The client host name
  $t->set_Solaris( 'Solaris8' );                # The Solaris version (not needed elsewhere)

  # Issue the create command
  $s->execute_create;

  # Issue the delete command
  $s->execute_delete;

=head1 TODO

  Make a TODO list.
  
=head1 DEPENDENCIES

  NBU::Command.pm
  Data::Dumper.pm
  
=head1 AUTHOR

  Todd Shoenfelt (todd@netapp.com)
  
=cut

__END__

/usr/openv/netbackup/bin/admincmd/bpplclients djohnP \
	-M perf280er.lab.netapp.com \
	-add "perf280er.lab.netapp.com" \
	Solaris Solaris8

djohnP		Policy Name
-M		Master Server
-add		Client to Add
Solaris ...	Client OS Version

/usr/openv/netbackup/bin/admincmd/bpplclients djohnP-W \
	-M perf280er.lab.netapp.com \
	-add "nam2.lab.netapp.com" \
	PC WindowsNET

djohnP-W	Policy Name
-M		Master Server
-add		Client to Add
PC WindowsNET	Client OS Version
