package NBU::Include;

use NBU::Command;

@ISA = qw(NBU::Command);

use Data::Dumper;
use strict;

############################# Accessors ##############################

sub set_path {
	my ( $self, $path ) = @_;
	die "path must have a length" unless length $path;
	$self->{path} = $path;
}
sub get_path { my ( $self ) = @_; return $self->{path} }

sub set_policy {
	my ( $self, $p ) = @_;
	die "policy must be an instance of NBU::Policy" unless ref( $p ) eq 'NBU::Policy';
	$self->{policy} = $p;
}
sub get_policy { my ( $self ) = @_; return $self->{policy} }

sub to_create_string {

	my ( $self ) = @_;

	my $string .= $NBU::Command::ROOT . 'bin/admincmd/bpplinclude';
	$string .= " $self->{policy}{name}";
	$string .= " -M $self->{policy}{storage_unit}{M}";
	$string .= " -add $self->{path}";
	
	return $string;
}

sub to_delete_string {

	my ( $self ) = @_;

	my $string .= $NBU::Command::ROOT . 'bin/admincmd/bpplinclude';
	$string .= " $self->{policy}{name}";
	$string .= " -M $self->{policy}{storage_unit}{M}";
	$string .= " -delete $self->{path}";
	
	return $string;
}

sub create_results {

	my ( $self ) = @_;
	
	return 0 if $NBU::Command::NOOP;

	return 0;
}

sub delete_results {

	my ( $self ) = @_;
	
	return 0 if $NBU::Command::NOOP;

	return 0;
}

1;

=head1 NAME

  NBU::Include - Include Directories/Files NetBackup Backups
  
=head1 DESCRIPTION

  This is a wrapper around the bpplinclude command.  The object caches command 
  results internally.  For the most part, the variable names are the same 
  as the NBU command switches.
  
=head1 SYNOPSIS

  use NBU::Command;
  use NBU::StorageUnit;
  use NBU::Policy;
  use NBU::Include;
  
  $NBU::Command::VERBOSE = 1;

  my $s = NBU::StorageUnit->new;
  ... set values

  my $p = NBU::Policy->new;
  ... set values

  # For p2 and p3

  my $i = NBU::Include->new;
  $i->set_path( '/u/todd/scripts/Logfile' );	# The path to files/directories to backup
  $i->set_policy( $p );						    # The policy to which to include the path
 
  # Issue the create command
  $c->execute_create;

  # Issue the delete command
  $c->execute_delete;

=head1 TODO

  Make a TODO list.
  
=head1 DEPENDENCIES

  NBU::Command.pm
  Data::Dumper.pm
  
=head1 AUTHOR

  Todd Shoenfelt (todd@netapp.com)
  
=cut

__END__

/usr/openv/netbackup/bin/admincmd/bpplinclude djohnP \
	-M perf280er.lab.netapp.com \
	-add "/vijayd/solaris_sample"

djohnP		Policy Name
-M		Master Server
-add		Path to Add
-f		Add the Paths Listed in File


/usr/openv/netbackup/bin/admincmd/bpplinclude djohnP-W \
	-M perf280er.lab.netapp.com \
	-add '/\\FSR-U25/C$/etc/boot'

djohnP-W	Policy Name
-M		Master Server
-add		Path to Add
-f		Add the Paths Listed in File

