package Job;

=head1 NAME

  NBU::Job - NBU Backup or Restore Details
  
=head1 DESCRIPTION

  This contains details about a NBU Backup or Restore job.  This class
  works differently than the rest of the NBU classes.  It does NOT 
  inherit from NBU::Command.

=head1 SYNOPSIS

  use NBU::Job;

  # You won't normally create Job objects, but will get them from the JobMonitor.
  my $j = Job->new;
  $j->set_JobID( $JobID );
  $j->set_Type( $Type );
  $j->set_State( $State );
  $j->set_Status( $State );

  get_on_with_the_test() if $j->get_State eq 'Done';
  
  report_no_error() if $j->get_Status == 0;
  
  if ( $j->done_success ) {
     get_on_with_the_test();
	 report_no_error();
  }
  
=head2 new()

  Constructor.
  
  Takes no arguments.  Returns the object reference.
  
=cut

sub new {	
	my ( $class ) = @_;
	return bless {}, $class;
}

sub set_JobID {
	my ( $self, $id ) = @_;
	$self->{JobID} = $id;
}
sub get_JobID { my ( $self ) = @_; return $self->{JobID} }

sub set_Client {
	my ( $self, $client ) = @_;
	$self->{Client} = $client;
}
sub get_Client { my ( $self ) = @_; return $self->{Client} }

sub set_Dest {
	my ( $self, $Dest ) = @_;
	$self->{Dest} = $Dest;
}
sub get_Dest { my ( $self ) = @_; return $self->{Dest} }

sub set_Status {
	my ( $self, $Status ) = @_;
	$self->{Status} = $Status;
}
sub get_Status { my ( $self ) = @_; return $self->{Status} }

sub set_Policy {
	my ( $self, $Policy ) = @_;
	$self->{Policy} = $Policy;
}
sub get_Policy { my ( $self ) = @_; return $self->{Policy} }

sub set_Schedule {
	my ( $self, $Schedule ) = @_;
	$self->{Schedule} = $Schedule;
}
sub get_Schedule { my ( $self ) = @_; return $self->{Schedule} }

sub set_PID {
	my ( $self, $PID ) = @_;
	$self->{PID} = $PID;
}
sub get_PID { my ( $self ) = @_; return $self->{PID} }

sub set_Type {
	my ( $self, $Type ) = @_;
	$self->{Type} = $Type;
}
sub get_Type { my ( $self ) = @_; return $self->{Type} }

sub set_State {
	my ( $self, $State ) = @_;
	$self->{State} = $State;
}
sub get_State { my ( $self ) = @_; return $self->{State} }

=head2 done_success()

  Instance Method.

  Takes no arguments.  Checks whether the job is finished running and was successful.
  Returns 1 if done and successful.  Returns 0 if not done or not successful.

=cut

sub done_success {
	my ( $self ) = @_;
	return 1 if ( $self->{State} eq 'Done' && $self->{Status} == 0 );
	return 0;
}

=head2 to_string()

  Instance Method.
  
  Takes no arguments.  Builds a string representation of the instance and returns it.

=cut

sub to_string {
	my ( $self ) = @_;
	
	my $string = '';
	$string .= "JobId:  $self->{JobID}\n";
	$string .= "Type:   $self->{Type}\n";
	$string .= "State:  $self->{State}\n";
	$string .= "Status: $self->{Status}\n";
	
	return $string;
}

=head1 TODO

  Make a TODO list.
  
=head1 DEPENDENCIES

  None
  
=head1 AUTHOR

  Todd Shoenfelt (todd@netapp.com)
  
=cut

1;
