package NDMPCredentials;

use NBU::Command;
@ISA = qw(NBU::Command);

use Data::Dumper;
use strict;

############################# Accessors ##############################

sub set_hostname {	
	my ( $self, $hostname ) = @_;
	die "set_hostname() requires a string" unless length $hostname;
	$self->{hostname} = $hostname;
}
sub get_hostname { my ( $self ) = @_; return $self->{hostname} }

sub set_username {
	my ( $self, $username ) = @_;
	die "set_username() requires a string" unless length $username;
	$self->{username} = $username;
}
sub get_username { my ( $self ) = @_; return $self->{username} }

sub set_password {
	my ( $self, $password ) = @_;
	die "set_password() requires a string" unless length $password;
	$self->{password} = $password;
}
sub get_password { my ( $self ) = @_; return $self->{password} }

sub set_snapvault_filer {
	my ( $self, $in ) = @_;
	die "set_snapvault_filer() must be 1 or 0" unless ( $in == 1 || $in == 0 );
	$self->{snapvault_filer} = $in;
}
sub get_snapvault_filer { my ( $self ) = @_; return $self->{snapvault_filer} }

sub to_create_string {

	my ( $self ) = @_;
	
	my $string = $NBU::Command::STEM . '../volmgr/bin/tpconfig';
	$string .= " -add";
	$string .= " -snap_vault_filer" if $self->{snapvault_filer};
	$string .= " -nh $self->{hostname}";
	$string .= " -user_id $self->{username}";
	$string .= " -password $self->{password}";

	return $string;
}

sub to_delete_string {

	my ( $self ) = @_;

	my $string = $NBU::Command::STEM . '../volmgr/bin/tpconfig';
	$string .= " -delete";
	$string .= " -nh $self->{hostname}";
	$string .= " -user_id $self->{username}";
	$string .= " -password $self->{password}";
	
	#return $string;
	return undef;	# tpconfig -delete is not currently working.
}

# This is still the right way to do this.  So I'm leaving it commented.
# This verifies that the credentials actually work.  The replacement
# code only verifies that NBU has some credentials set for the filer,
# not that they actually work.

#sub create_results {
#
#	my ( $self ) = @_;
#	
#	return 0 if $NBU::Command::NOOP;
#	
#	my $string = $NBU::Command::STEM . '../volmgr/bin/tpautoconf';
#	$string .= " -verify $self->{hostname}";
#	
#	my $out = `$string`;
#	
#	if ( $out =~ /Login was successful/g ) {
#		return 0;
#	}
#		
#	return 1;
#}

sub create_results {

	my ( $self ) = @_;
	
	return 0 if $NBU::Command::NOOP;
	
	my $string = $NBU::Command::STEM . '../volmgr/bin/tpconfig';
	$string .= " -dnh -all_hosts";
	
	my $out = `$string`;
	
	if ( $out =~ /NDMP Host Name:\s+$self->{hostname}/g ) {
		return 0;
	}
	
	return 1;
}

# This is still the right way to do this.  So I'm leaving it commented.
# This verifies that the credentials actually work.  The replacement
# code only verifies that NBU has some credentials set for the filer,
# not that they actually work.

#sub delete_results {
#
#	my ( $self ) = @_;
#	
#	return 0 if $NBU::Command::NOOP;
#	
#	my $string = $NBU::Command::STEM . '../volmgr/bin/tpautoconf';
#	$string .= " -verify $self->{hostname}";
#	
#	my $out = `$string`;
#	
#	if ( $out =~ /Login was successful/g ) {
#		return 1;
#	}
#		
#	return 0;
#}

sub delete_results {

	my ( $self ) = @_;
	
	return 0 if $NBU::Command::NOOP;
	
	my $string = $NBU::Command::STEM . '../volmgr/bin/tpconfig';
	$string .= " -dnh -all_hosts";
	
	my $out = `$string`;
	
	if ( $out =~ /NDMP Host Name:\s+$self->{hostname}/g ) {
		return 1;
	}
	
	return 0;	
}

1;

=head1 NAME

  NDMPCredentials - Create Host NDMP Credentials
  
=head1 DESCRIPTION

  This is a wrapper around the tpconfig and tpautoconf commands.  The object 
  caches command results internally.  For the most part, the variable names 
  are the same as the NBU command switches.
  
=head1 SYNOPSIS

  use NDMPCredentials;
  
  my $n = NDMPCredentials->new;
  $n->set_hostname( $FILER1 );
  $n->set_username( 'root' );
  $n->set_password( 'foo' );		
  # For p3
  $n->set_snapvault_filer( 1 );

  # Issue the create command
  $s->execute_create;
  print $s->get_results unless $s->passed;

  # Issue the delete command
  $s->execute_delete;
  print $s->get_results unless $s->passed;

=head1 METHODS

=cut

=head1 TODO

  Make a TODO list.
  
=head1 DEPENDENCIES

  Data::Dumper.pm

=head1 AUTHOR

  Todd Shoenfelt (todd@netapp.com)
  
=cut
















