package NBU::Restore;

use NBU::Command;

@ISA = qw(NBU::Command);

use Data::Dumper;
use strict;

############################# Accessors ##############################

sub set_policy {
	my ( $self, $p ) = @_;
	die "policy must be an instance of NBU::Policy" unless ref( $p ) eq 'NBU::Policy';
	$self->{policy} = $p;
}
sub get_policy { my ( $self ) = @_; return $self->{policy} }

sub set_l {
	my ( $self, $l ) = @_;
	die "l must be 1 or 0" unless ( $l == 1 || $l == 0 );
	$self->{l} = $l;
}
sub get_l { my ( $self ) = @_; return $self->{l} }

sub set_S {
	my ( $self, $S ) = @_;
	die "S must have a length" unless length $S;
	$self->{S} = $S;
}
sub get_S { my ( $self ) = @_; return $self->{S} }

sub set_C {
	my ( $self, $C ) = @_;
	die "C must have a length" unless length $C;
	$self->{C} = $C;
}
sub get_C { my ( $self ) = @_; return $self->{C} }

sub set_D {
	my ( $self, $D ) = @_;
	die "D must have a length" unless length $D;
	$self->{D} = $D;
}
sub get_D { my ( $self ) = @_; return $self->{D} }

sub set_R {
	my ( $self, $R ) = @_;
	die "R must have a length" unless length $R;
	$self->{R} = $R;
}
sub get_R { my ( $self ) = @_; return $self->{R} }

sub set_t {
	my ( $self, $t ) = @_;
	die "t must be 0 (Standard/UNIX) or 13 (Windows)" unless ( $t == 0 || $t == 13 );
	$self->{t} = $t;
}
sub get_t { my ( $self ) = @_; return $self->{t} }

sub set_T {
	my ( $self, $T ) = @_;
	die "T must be 1 (TIR enabled) or 0 (TIR disabled)" unless ( $T == 0 || $T == 1 );
	$self->{T} = $T;
}
sub get_T { my ( $self ) = @_; return $self->{T} }

sub set_K {
	my ( $self, $K ) = @_;
	die "K must be 1 or 0" unless ( $K == 1 || $K == 0 );
	$self->{K} = $K;
}
sub get_K { my ( $self ) = @_; return $self->{K} }

sub set_w {
	my ( $self, $w ) = @_;
	die "w must be hh:mm:ss" unless ( $w =~ /\d\d\:\d\d:\d\d/ );
	$self->{w} = $w;
}
sub get_w { my ( $self ) = @_; return $self->{w} }

sub set_k {
	my ( $self, $k ) = @_;
	die "w must have a length" unless length $k;
	$self->{k} = $k;
}
sub get_k { my ( $self ) = @_; return $self->{k} }

sub set_s {
	my ( $self, $s ) = @_;
	die "s must be of the form: mm/dd/yyyy" unless $s =~ /^\d\d\/\d\d\/\d\d\d\d$/;
	$self->{s} = $s;
}
sub get_s { my ( $self ) = @_; return $self->{s} }

sub set_e {
	my ( $self, $e ) = @_;
	die "e must be of the form: mm/dd/yyyy" unless $e =~ /^\d\d\/\d\d\/\d\d\d\d$/;
	$self->{e} = $e;
}
sub get_e { my ( $self ) = @_; return $self->{e} }

sub set_L {
	my ( $self, $L ) = @_;
	die "L must have a length" unless length $L;
	die "Log path (L) must be absolute, note relative" unless $L =~ /^\//;
	$self->{L} = $L;
}
sub get_L { my ( $self ) = @_; return $self->{L} }

sub set_f {
	my ( $self, $f ) = @_;
	die "f must have a lentth" unless length $f;
	die "Restore file path (f) must be absolute, not relative" unless $f =~ /^\//;
	$self->{f} = $f;
}

sub set_filename {
	my ( $self, $filename ) = @_;
	die "filename must have a length" unless length $filename;
	$self->{filename} = $filename;
}
sub get_filename { my ( $self ) = @_; return $self->{filename}; }

sub set_job {
	my ( $self, $job ) = @_;
	die "job must be an instance of Job" unless ref($job) eq 'Job';
	$self->{job} = $job;
}
sub get_job { my ( $self ) = @_; return $self->{job} }

sub to_create_string {

	my ( $self ) = @_;

	my $string .= $NBU::Command::ROOT . 'bin/bprestore';
	$string .= " -l" if defined $self->{l};
	$string .= " -S $self->{S}" if defined $self->{S};
	$string .= " -w $self->{w}" if defined $self->{w};
	$string .= " -L $self->{L}" if defined $self->{L};
	$string .= " -s $self->{s}" if defined $self->{s};
	$string .= " -e $self->{e}" if defined $self->{e};
	$string .= " -p $self->{policy}{name}";

	$string .= " -C $self->{C}" if defined $self->{C};
	$string .= " -D $self->{D}" if defined $self->{D};
	$string .= " -R $self->{R}" if defined $self->{R};
	$string .= " -t $self->{t}" if defined $self->{t};
	$string .= " -T" if $self->{T};
	$string .= " -K" if defined $self->{K};
	$string .= " -k \"$self->{k}\"" if defined $self->{k};
	$string .= " -f $self->{f}" if defined $self->{f};
	$string .= " $self->{filename}";


	return $string;
}

# No-op to preserve the Command.pm interface.
sub to_delete_string {

	my ( $self ) = @_;

	return undef;
}

sub create_results {

	my ( $self ) = @_;
	
	return 0 if $NBU::Command::NOOP;
	
	if ( $self->{results} !~ /EXIT STATUS/ ) {
		return 0;
	}
	
	return 1;
}

sub delete_results {
	my ( $self ) = @_;
	
	return 0;
}
sub create_rename_file {
	my ($file, $source, $dest) = @_;
	my $renamefile = "/var/tmp/nbu-rename$$";
	`/usr/bin/echo change $source to $dest >$renamefile`;
	return $renamefile;
}
1;


=head1 NAME

  NBU::Restore - Restore Data with NetBackup
  
=head1 DESCRIPTION

  This is a wrapper around the bprestore command.  The object caches command 
  results internally.  For the most part, the variable names are the same as 
  the NBU command switches
  
=head1 SYNOPSIS

  use NBU::Command;
  use NBU::StorageUnit;
  use NBU::Policy; 
  use NBU::Restore;

  my $s = NBU::StorageUnit->new;
  ... set values

  my $p = NBU::Policy->new;
  ... set values

  my $r = NBU::Restore->new;					
  $r->set_policy( $p );                        # The policy to which to apply this restore
  $r->set_s( '01/01/1970' );                   # The start of the restore window
  $r->set_e( '07/27/2005' );                   # The end of the restore window
  $r->set_C( 'perf280er.lab.netapp.com' );     # The source client
  $r->set_D( 'perf280er.lab.netapp.com' );     # The destination client
  $r->set_R( '/tmp' );                         # Rename to
  $r->set_L( '/u/todd/nbu/log' );              # The path to the restore log
  $r->set_w( '00:10:00' );                     # Wait this long for the restore to complete
                                               # The file to restore  
  $r->set_filename( '/u/todd/scripts/Logfile/1/test/logfile_test.pl' );

  # Issue the create command
  $s->execute_create;
  print $s->get_results unless $s->passed;

=head1 TODO

  Make a TODO list.
  
=head1 DEPENDENCIES

  NBU::Command.pm
  
=head1 AUTHOR

  Todd Shoenfelt (todd@netapp.com)
  
=cut


__END__



Restore for UNIX Client

/usr/openv/netbackup/bin/bprestore \
	-l \
	-S perf280er.lab.netapp.com \
	-C perf280er.lab.netapp.com \
	-D perf280er.lab.netapp.com \
	-R /tmp/renamefile \
	-p djohnP \
	-t 0 \
	-K \
	-w \
	"/vijayd/solaris_sample"

-l		Rename Links Just Like Rename Files
-S		Master Server
-C		Client Name as it Appears in Catalog
-D		Destinaton Client Name as it Appears in Catalog
-R		Rename File (see below for example)
-p		Policy
-t		Type (0 = Standard/UNIX; 13 = Windows)
-K		Keep Existing Files; Default is to Overwrite
-k		Keyword Phrase
-L <file>	Generate Progress Log (stored on client)
-f <file>	File of Files to Restore (or you can just specify a filename)

Note: All paths should be absolute (begin with a "/").

The Rename File should look like...

change /vijayd/solaris_sample to /djohn/vijayd/solaris_sample


Restore for Windows Client

/usr/openv/netbackup/bin/bprestore \
	-S perf280er.lab.netapp.com \
	-C nam2 \
	-D nam2 \
	-R /tmp/renamefile \
	-p djohnP-W \
	-t 13 \
	-K \
	-w \
	'/\\FSR-U25/C$/etc/boot'

-l		Rename Links Just Like Rename Files
-S		Master Server
-C		Client Name as it Appears in Catalog
-D		Destinaton Client Name as it Appears in Catalog
-R		Rename File (see below for example)
-p		Policy
-t		Type (0 = Standard/UNIX; 13 = Windows)
-K		Keep Existing Files; Default is to Overwrite
-k		Keyword Phrase
-L <file>	Generate Progress Log (stored on client)
-f <file>	File of Files to Restore (or you can just specify a filename)

Note: All paths should be absolute (begin with a "/").

change /\\FSR-U25/C$/etc/boot to /\\FSR-U25/C$/home/restored_data/boot
