package NBU::Schedule;

use NBU::Command;

@ISA = qw(NBU::Command);

use Data::Dumper;
use strict;

############################# Accessors ##############################

sub set_policy {
	my ( $self, $p ) = @_;
	die "policy must be an instance of NBU::Policy" unless ref( $p ) eq 'NBU::Policy';
	$self->{policy} = $p;
}
sub get_policy { my ( $self ) = @_; return $self->{policy} }

sub set_name {
	my ( $self, $name ) = @_;
	die "name must have a length" unless length $name;
	$self->{name} = $name;
}
sub get_name { my ( $self ) = @_; return $self->{name} }

sub set_type {
	my ( $self, $type ) = @_;
	die "type must be INCR or FULL" unless ( $type =~ /^(FULL|INCR)$/ );
	$self->{type} = $type;
}
sub get_type { my ( $self ) = @_; return $self->{type} }

sub set_cal {
	my ( $self, $cal ) = @_;
	die "cal must be defined" unless defined $cal;
	$self->{cal} = $cal;
}
sub get_cal { my ( $self ) = @_; return $self->{cal} }

sub set_rl {
	my ( $self, $rl ) = @_;
	die "rl must be 1, 2, or 3" unless ( $rl == 1 || $rl == 2 || $rl == 3 );
	$self->{rl} = $rl;
}
sub get_rl { my ( $self ) = @_; return $self->{rl} }

sub set_synthetic {
	my ( $self, $synthetic ) = @_;
	die "sythetic must be 1 or 0" unless ( $synthetic == 0 || $synthetic == 1 );
	$self->{synthetic} = $synthetic;
}
sub get_synthetic { my ( $self ) = @_; return $self->{synthetic} }

sub set_pfi_fast_recovery {
	my ( $self, $fast ) = @_;
	die "pfi_fast_recovery must be 1 or 0" unless ( $fast == 0 || $fast == 1 );
	$self->{pfi_fast_recovery} = $fast;
}
sub get_pfi_fast_recovery { my ( $self ) = @_; return $self->{pfi_fast_recovery} };

sub set_st {
	my ( $self, $st ) = @_;
	die "st must be INCR or FULL" unless ( $st =~ /^(FULL|INCR)$/ );
	$self->{st} = $st;
}
sub get_st { my ( $self ) = @_; return $self->{st} }

sub to_create_string {

	my ( $self ) = @_;

	my $string .= $NBU::Command::ROOT . 'bin/admincmd/bpplsched';
	$string .= " $self->{policy}{name}";
	$string .= " -M $self->{policy}{storage_unit}{M}";
	$string .= " -add $self->{name}";
	$string .= " -type $self->{type}" if defined $self->{type};
	$string .= " -st $self->{st}" if defined $self->{st};
	$string .= " -cal $self->{cal}" if defined $self->{cal};
	$string .= " -rl $self->{rl}" if defined $self->{rl};
	$string .= " -synthetic $self->{synthetic}" if defined $self->{synthetic};
	$string .= " -pfi_fast_recovery $self->{pfi_fast_recovery}" if defined $self->{pfi_fast_recovery};
		
	return $string;
}

sub to_delete_string {

	my ( $self ) = @_;

	my $string .= $NBU::Command::ROOT . 'bin/admincmd/bpplsched';
	$string .= " $self->{policy}{name}";
	$string .= " -M $self->{policy}{storage_unit}{M}";	
	$string .= " -delete $self->{name}";
	$string .= " -type $self->{type}" if defined $self->{type};
	$string .= " -st $self->{st}" if defined $self->{st};
	$string .= " -cal $self->{cal}" if defined $self->{cal};
	$string .= " -rl $self->{rl}" if defined $self->{rl};
	$string .= " -synthetic $self->{synthetic}" if defined $self->{synthetic};
	$string .= " -pfi_fast_recovery $self->{pfi_fast_recovery}" if defined $self->{pfi_fast_recovery};
		
	return $string;
}

sub create_results {

	my ( $self ) = @_;
			
	return 0 if $NBU::Command::NOOP;
	
	return 0;
}

sub delete_results {

	my ( $self ) = @_;
			
	return 0 if $NBU::Command::NOOP;
	
	return 0;
}

1;


=head1 NAME

  NBU::Schedule - A NetBackup Schedule
  
=head1 DESCRIPTION

  This is a wrapper around the bpplsched command.  The object
  caches command results internally.
  
=head1 SYNOPSIS

  use NBU::Command;
  use NBU::Schedule;
  use NBU::Policy;
  use NBU::StorageUnit;

  my $s = NBU::StorageUnit->new;
  ... set values

  my $p = NBU::Policy->new;
  ... set values

  my $c = NBU::Schedule->new;
  # The schedule name
  $c->set_name( 'perf280er.lab.netapp.com' );
  # The type of backup
  $c->set_type( 'FULL' );
  # The frequency, for frequency-based schedules
  $c->set_cal( 0 );
  # The policy to which to apply the schedule
  $c->set_policy( $p );

  # Return the create command string
  my $create = $s->to_create_string;

  # Return the delete command string (Not implemented)
  # my $delete = $s->to_delete_string;

  # Issue the create command
  $c->execute_create;

  # Issue the delete command (Not implemented)
  $c->execute_delete;

=head1 TODO

  Make a TODO list.
  
=head1 DEPENDENCIES

  NBU::Command.pm
  
=head1 AUTHOR

  Todd Shoenfelt (todd@netapp.com)
  
=cut


__END__

/usr/openv/netbackup/bin/admincmd/bpplsched djohnP \
	-add Full \
	-type FULL \
	-cal 0 \
	-M perf280er.lab.netapp.com

/usr/openv/netbackup/bin/admincmd/bpplsched djohnP \
	-add Differential-Inc \
	-st INCR \
	-cal 0 \
	-M perf280er.lab.netapp.com

djohnP		Policy Name
-add		Schedule Name to Add
-type		Type of Backup
-cal		0 = Frequency Based Schedules
-M		Master Server

The documentation mentions a cumulative incremental which backs up all
changes since the last full. It is "-st CINC". I have heard nobody 
mention it, but my guess is we should test with it.


/usr/openv/netbackup/bin/admincmd/bpplsched djohnP-W \
	-add Full \
	-type FULL \
	-cal 0 \
	-M perf280er.lab.netapp.com

/usr/openv/netbackup/bin/admincmd/bpplsched djohnP \
	-add Differential-Inc \
	-st INCR \
	-cal 0 \
	-M perf280er.lab.netapp.com

djohnP-W	Policy Name
-add		Schedule Name to Add
-type		Type of Backup
-cal		0 = Frequency Based Schedules
-M		Master Server

------------------------------------------------------------------------------

/usr/openv/netbackup/bin/admincmd/bpplsched 
	<name of policy> 
	-add full 
	-st FULL 
	-rl 3  						<
	-synthetic 0 
	-pfi_fast_recovery 1 		<
