# $Id: //depot/prod/test/Rsierranevada/lib/NBU/nbuUtils.pm#1 $
#
# Copyright (c) 2004 Network Appliance, Inc.
# All rights reserved.
#
## @summary Vertitas NBU P3 related library routines.
## @pod        here
## @author  dl-dpgqa-automation@netapp.com, dussa@netapp.com, weili@netapp.com, cpeter@netapp.com, htu@netapp.com, sydneyw@netapp.com
## @dependencies
##   lib/SayWrap.pm
##   FSR/FSRutils.pm

=head1 NAME

NBU::nbuUtils

=head1 DESCRIPTION

Vertitas NBU P3 related library routines. Mostly a wrapper around netbackup binaries
under /usr/openv/netbackup. Veritas man pages can be found under
/usr/openv/man and /usr/openv/netbackup/bin/goodies/man.

Veritas is currently installed on nam9.lab.netapp.com machine.

=head1 METHODS

=cut


package NBU::nbuUtils;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(
    bpplinclude bpplscheddel bpplsched bpplsched_p2 bpplclients bpplclients_tape
    bpplinfo bpplinfo_p2_set_snapshotbackups bpplinfo_p2_set_datamovertype
    bpplinfo_p2_set_snapshotmethod bpplinfo_tape bppolicynew bpstuadd bpstuadd_p2 bpstuadd_tape
    bpstulist bpstudel bppldelete bpbackup bprestore add_tpconfig add_tpconfig_list cln_tpconfig_list
    currentDate createRenameFile checkForSuccess compareData backupInfo
    getOStype getHardwareInfo populate modify bounce_pbx bpsturep addbpconfVSM
    rmbpconfVSM bpdbjobs file_compare backupIDs exprImage 
    bpdbjobs_cancel
);

use strict;
use Data::Dumper;
use Carp qw(croak);
use Tharn;
use FSR::FSRutils;
use FSR::DR::DRutils;
#use File::Remote qw(:replace);

use vars qw( $sudo $cfgpath $admpath $nbpath $TESTBIN $NBU_MASTER $localhost_fc );


# check for dependency
BEGIN {
    $sudo = "/usr/software/bin/sudo";
    $cfgpath = "/usr/openv/volmgr/bin/";
    $admpath = "/usr/openv/netbackup/bin/admincmd";
    $nbpath = "/usr/openv/netbackup/bin";
    $TESTBIN = "/usr/local/test/bin";

    (my $host = `hostname`) =~ s/\..*//;
    #my $host = `hostname` ;
    $NBU_MASTER = Tharn::param('NBU_MASTER', '-default', $host);
}

sub client_say_cli {
    my (%args) = @_;
    #my $cmd = delete $args{cmd};
    my $cmd = $args{cmd};

	unless (defined $localhost_fc) {
    	my $fh = host($NBU_MASTER) || die $@;
	    $localhost_fc = connect("#rsh") || die $@; 
	}

    my $caller = (caller(1))[3];
    croak "Command can not be empty. Called by $caller" unless( $cmd =~ /\w/ );

    my ($result,$ShellExitStatus) = ( undef, -1 );

    my $timeout = $args{timeout};
    if(!defined $timeout){
       #(undef,undef,$result,undef) = say($localhost_fc,"/bin/sh -c '$cmd;echo ShellExitStatus=\$?'", %args);
       (undef,undef,$result,undef) = say($localhost_fc,"/bin/sh -c '$cmd;echo ShellExitStatus=\$?'");
    } else {
       #(undef,undef,$result,undef) = say($localhost_fc,"/bin/sh -c '$cmd;echo ShellExitStatus=\$?'", %args, -timeout, $timeout );
       (undef,undef,$result,undef) = say($localhost_fc,"/bin/sh -c '$cmd;echo ShellExitStatus=\$?'",  -timeout, $timeout );
    }
    if( $result =~ /ShellExitStatus=(\d+)/ ) {
    $ShellExitStatus = $1;
    eval { croak "Failed to execute command \"$cmd\", no status returned for [$caller]: $result\n" } if $ShellExitStatus;
    } else {
    eval { croak "Failed to execute command \"$cmd\", no status returned for [$caller]: $result\n" };
    }

    $result =~ s/ShellExitStatus=(\d+).*//si;
    $ShellExitStatus = $1;
    return( $result, $ShellExitStatus );
}



=head2 ::bpplinclude

Adds/Deletes the specified source pathname to the policy file list. The pathname can
be a directory, a filename, a script, or a directive.

Executes following command: B<bpplinclude $class -M $master $operation "$fpath">

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplinclude($class,$master,$operation,$fpath);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<operation> => C<string> (I<Required>) "-add/-delete" operation

B<options> => {<name=value>] Options list of name,value pair

B<fpath> => C<string> (I<Required>) Path to backup. Space separated list of
directories. Path must be a full path starting with C<'/'>!

=back

=cut

sub bpplinclude {
    my ($class,$master,$operation,$fpath,@options) = @_;
    $operation = '-'.$operation unless( $operation =~ /^-/ );
    my $option_list = (join " ", @options)
    	if @options;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplinclude $class -M $master $operation $option_list \"$fpath\"" );
    return undef if $exit;
    return $result;
}
#----------------------------------------------------------------------

=head2 ::bpplscheddel

Delete the listed schedules from the named policy.

Executes following command: B<bpplsched $class -delete $sched>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 bpplscheddel($class,$sched);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<sched> => C<string> (I<Required>) Schedule name(s).
Separate the elements of the schedule label list with
spaces. There can be up to 25 labels in the list.

=back

=cut

sub bpplscheddel {
    my ($class,$sched) = @_;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplsched $class -delete $sched");
    return undef if $exit;
    return $result;
}
#----------------------------------------------------------------------

=head2 ::bpplsched

Add a single frequency-based schedule to the named policy.

Executes following command: B<bpplsched $class -M $master -add $sched -st $type -cal 0>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplsched($class,$sched,$master,$type, [$residence] );
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<sched> => C<string> (I<Required>) Schedule name (label)

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<type> => C<string> (I<Required>) The type of the  schedule.  The  default  schedule
type is FULL. The following list contains the possible values for this attribute with
their meanings:

    FULL - full
    INCR - differential incremental
    CINC - cumulative incremental
    UBAK - user backup
    UARC - user archive

B<residence> => C<string> (I<Optional>) Storage unit name.

=back

=cut

sub bpplsched {
    my ($class,$sched,$master,$type, $stu) = @_;
    my $residence = undef;
    if( $stu ) {
    $residence = "-residence $stu";
    }
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplsched $class -M $master -add $sched -st $type $residence -cal 0");
    return undef if $exit;
    return $result;
}
#----------------------------------------------------------------------

=head2 ::bpplsched_p2

Adds a schedule for a particular class (policy) for p2.

Executes following command:
B<bpplsched $class -M $master -add $sched -st $type -pfi_fast_recovery 1 -retlevel 1 -freq 604800 -residence "*NULL*" -pool " *NULL*" -sg "*NULL*" -res_is_stl 1 -ut -cal 0 -tzo -25200>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplsched($class,$sched,$master,$type);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<sched> => C<string> (I<Required>) Schedule name (label)

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<type> => C<string> (I<Required>) The type of the  schedule.  The  default  schedule
type is FULL. The following list contains the possible values for this attribute with
their meanings:

    FULL - full
    INCR - differential incremental
    CINC - cumulative incremental
    UBAK - user backup
    UARC - user archive

=back

=cut

sub bpplsched_p2 {
    my ($class,$sched,$master,$type) = @_;
    my $cmd = "$admpath/bpplsched $class -M $master -add $sched -st $type -pfi_fast_recovery 1 -retlevel 1 -freq 604800 -residence \"*NULL*\" -pool \"*NULL*\" -sg \"*NULL*\" -res_is_stl 1 -ut -cal 0 -tzo -25200";
    my ($result,$exit) = client_say_cli(cmd=>$cmd);
    return undef if $exit;
    return $result;
}
#----------------------------------------------------------------------

=head2 ::bpplclients

Add a new client to a policy. Delete a list of clients from a policy.
For the -add, -delete,  bpplclients
returns to the system prompt immediately after it submits
the client change request to NetBackup.

Executes following command:
B<bpplclients $class -M $master $option $client $hardware $os>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplclients($class,$option,$client,$master,$hardware,$os);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<option> => C<string> (I<Required>) "-add/-delete"

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<client> => C<string> (I<Required>) Client Server. Fully qualified host name.
The clients are provided as a space-delimited list of
host names.

If the local system has defined the maximum number of clients already,
an error is returned for -add option. The maximum number of
clients is unlimited (the installation default)
for NetBackup Enterprise Server and 4 for NetBackup Server.

Up to twenty clients can be deleted at a time.

B<hardware> => C<string> (I<Required>) Hardware type of the client server (eg., Soaris,or windows)
The hardware type of this client. In the dialog box for adding clients to a policy with the Backup
Policy Management utility, select one of the hardware types.

B<os> => C<string> (I<Required>) Client operating system (eg., "solaris9", or "windows")
The operating system of this client.  The values that you choose
for the hardware and the os options must form a valid combination.

=back

=cut

sub bpplclients {
    my ($class,$option,$client,$master,$hardware,$os) = @_;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplclients $class -M $master $option $client $hardware $os");
    eval{ croak($result) } if $exit;
    return $result;
}

#----------------------------------------------------------------------

=head2 ::bpplinfo

Set, modify or display policy attributes for NetBackup

Executes following commands: if $tir=1 then
B<bpplinfo $class $option -M $master -follownfs 1 -collect_tir_info 1 -residence $storageunit>
if $tir=2 then
B<bpplinfo $class $option -M $master -follownfs 1 -collect_tir_info 2 -residence $storageunit>
or B<bpplinfo $class $option -M $master -follownfs 1 -collect_bmr_info 0 -residence $storageunit> if $tir
is not defined.

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplinfo($class,$master,$option,$storageunit,$tir);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<option> => C<string> (I<Required>) "-modify/-set" (type of operation)

B<storageunit> => C<string> (I<Required>) Nearstore Disk Storage Unit Label.
Specifies the label of the storage unit for storing the backups that were created according to
this schedule. The default is Any_available, which allows the policy to use any storage unit that
where the attribute On Demand Only? is set to No.  If the policy needs to use a specific storage unit
or the storage unit that is wanted has the attribute On Demand Only? set to Yes, then specify the
storage unit. If "*NULL*" is specified,  the residence for the schedule is set (or reset) to
Any_available. The policy residence determines the residence for the policy schedules, unless the
Override Policy Storage Unit setting on an individual schedule specifies a residence. Run bpstulist
to display the set of defined storage units.

B<tir> => C<string> (I<Optional>) If defined, then collects true-image-recovery (TIR) information.

=back

=cut

sub bpplinfo ($$$$;$) {
    my ($class,$master,$option,$storageunit,$tir) = @_;

    my ($result,$exit);
    if (!defined($tir)) {
    ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplinfo $class $option -M $master -follownfs 1 -collect_bmr_info 0 -residence $storageunit");
    } elsif ($tir eq "1") {
    ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplinfo $class $option -M $master -follownfs 1 -collect_tir_info 1 -residence $storageunit");
    } else {
    ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplinfo $class $option -M $master -follownfs 1 -collect_tir_info 2 -residence $storageunit");
    } 
    return undef if $exit;
    return $result;
}

=head2 bpplinfo_tape

Create a policy attribute for a NetBackup tape device. The parameters are a subset of the bpplinfo routine above and have the same format and usage.

=over

=back

=cut

sub bpplinfo_tape {
    my ($class,$master,$storageunit,$type) = @_;
    $type = "NDMP" unless defined $type;
    return _dma_exec("add tape schedule $storageunit, type $type", $admpath."/bpplinfo", "$class -set -M $master -residence $storageunit -pt NDMP");
}

#----------------------------------------------------------------------

=head2 ::bpplinfo_p2_set_snapshotbackups

Sets p2 Perform snapshot backups

Executes following command:
B<bpplinfo $class -snapshot_list_policy -clienttype Standard -residence $storageunit -pool "NetBackup" -priority 0 -keyword "*NULL*" -data_class *NULL* -res_is_stl 0 -sg "*ANY*" -active -compress 0 -follownfs 0 -crossmp 0 -collect_tir_info 0 -rfile 0 -encrypt 0 -blkincr 0 -collect_bmr_info 0 -granular_restore_info 0 -fi 1 -enable_pfi 1 -offhost_backup 1 -use_data_mover 1 -tzo -25200 -M $master -verbose>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplinfo_p2_set_snapshotbackups($class,$master,$storageunit);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<storageunit> => C<string> (I<Required>) Nearstore Disk Storage Unit Label.
Specifies the label of the storage unit for storing the backups that were created according to
this schedule. The default is Any_available, which allows the policy to use any storage unit that
where the attribute On Demand Only? is set to No.  If the policy needs to use a specific storage unit
or the storage unit that is wanted has the attribute On Demand Only? set to Yes, then specify the
storage unit. If "*NULL*" is specified,  the residence for the schedule is set (or reset) to
Any_available. The policy residence determines the residence for the policy schedules, unless the
Override Policy Storage Unit setting on an individual schedule specifies a residence. Run bpstulist
to display the set of defined storage units.

=back

=cut

sub bpplinfo_p2_set_snapshotbackups() {
    my ($class,$master,$storageunit) = @_;

    my $result;
    logcomment("storage unit1: $storageunit");

    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplinfo $class -snapshot_list_policy -clienttype Standard -residence $storageunit -pool \"NetBackup\" -priority 0 -keyword \"*NULL*\" -data_class *NULL* -res_is_stl 0 -sg \"*ANY*\" -active -compress 0 -follownfs 0 -crossmp 0 -collect_tir_info 0 -rfile 0 -encrypt 0 -blkincr 0 -collect_bmr_info 0 -granular_restore_info 0 -fi 1 -enable_pfi 1 -offhost_backup 1 -use_data_mover 1 -tzo -25200 -M $master -verbose");
    return undef if $exit;
    return $result;
}    
        

=head2 ::bpplinfo_p2_set_datamovertype

Sets p2 Use Data Mover

Executes following command:
B<bpplinfo $class -snapshot_list_policy -clienttype Standard -residence $storageunit -pool "NetBackup" -priority 0 -keyword "*NULL*" -data_class *NULL* -res_is_stl 0 -sg "*ANY*" -active -compress 0 -follownfs 0 -crossmp 0 -collect_tir_info 0 -rfile 0 -encrypt 0 -blkincr 0 -collect_bmr_info 0 -granular_restore_info 0 -concurrent_data 1 -fi 1 -enable_pfi 1 -offhost_backup 1 -use_data_mover 1 -data_mover_type 3 -tzo -25200 -M $master -verbose>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplinfo_p2_set_datamovertype($class,$master,$storageunit);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<storageunit> => C<string> (I<Required>) Nearstore Disk Storage Unit Label.
Specifies the label of the storage unit for storing the backups that were created according to
this schedule. The default is Any_available, which allows the policy to use any storage unit that
where the attribute On Demand Only? is set to No.  If the policy needs to use a specific storage unit
or the storage unit that is wanted has the attribute On Demand Only? set to Yes, then specify the
storage unit. If "*NULL*" is specified,  the residence for the schedule is set (or reset) to
Any_available. The policy residence determines the residence for the policy schedules, unless the
Override Policy Storage Unit setting on an individual schedule specifies a residence. Run bpstulist
to display the set of defined storage units.

=back

=cut

sub bpplinfo_p2_set_datamovertype(){
    my ($class,$master,$storageunit) = @_;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplinfo $class -snapshot_list_policy -clienttype Standard -residence $storageunit -pool \"NetBackup\" -priority 0 -keyword \"*NULL*\" -data_class *NULL* -res_is_stl 0 -sg \"*ANY*\" -active -compress 0 -follownfs 0 -crossmp 0 -collect_tir_info 0 -rfile 0 -encrypt 0 -blkincr 0 -collect_bmr_info 0 -granular_restore_info 0 -concurrent_data 1 -fi 1 -enable_pfi 1 -offhost_backup 1 -use_data_mover 1 -data_mover_type 3 -tzo -25200 -M $master -verbose");
    return undef if $exit;
    return $result;
}
#----------------------------------------------------------------------

=head2 ::bpplinfo_p2_set_snapshotmethod

Sets p2 Snapshot Method, and add everything to Policy

Executes following command:
B<bpplinfo $class -modify -clienttype Standard -residence $storageunit -pool "NetBackup" -priority 0 -keyword "*NULL*" -data_class *NULL* -res_is_stl 0 -sg "*ANY*" -active -compress 0 -follownfs 0 -crossmp 0 -collect_tir_info 0 -rfile 0 -encrypt 0 -blkincr 0 -collect_bmr_info 0 -granular_restore_info 0 -concurrent_data 1 -fi 1 -enable_pfi 1 -offhost_backup 1 -use_data_mover 1 -data_mover_type 3 -snapshot_method "NAS_Snapshot" -snapshot_method_args "max_snapshots=2" -M $master -tzo -25200>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpplinfo_p2_set_snapshotmethod($class,$master,$storageunit);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<storageunit> => C<string> (I<Required>) Nearstore Disk Storage Unit Label.
Specifies the label of the storage unit for storing the backups that were created according to
this schedule. The default is Any_available, which allows the policy to use any storage unit that
where the attribute On Demand Only? is set to No.  If the policy needs to use a specific storage unit
or the storage unit that is wanted has the attribute On Demand Only? set to Yes, then specify the
storage unit. If "*NULL*" is specified,  the residence for the schedule is set (or reset) to
Any_available. The policy residence determines the residence for the policy schedules, unless the
Override Policy Storage Unit setting on an individual schedule specifies a residence. Run bpstulist
to display the set of defined storage units.

=back

=cut

sub bpplinfo_p2_set_snapshotmethod(){
    my ($class,$master,$storageunit) = @_;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpplinfo $class -modify -clienttype Standard -residence $storageunit -pool \"NetBackup\" -priority 0 -keyword \"*NULL*\" -data_class *NULL* -res_is_stl 0 -sg \"*ANY*\" -active -compress 0 -follownfs 0 -crossmp 0 -collect_tir_info 0 -rfile 0 -encrypt 0 -blkincr 0 -collect_bmr_info 0 -granular_restore_info 0 -concurrent_data 1 -fi 1 -enable_pfi 1 -offhost_backup 1 -use_data_mover 1 -data_mover_type 3 -snapshot_method \"NAS_Snapshot\" -snapshot_method_args \"max_snapshots=2\" -M $master -tzo -25200");
    return undef if $exit;
    return $result;
}
#-----------------------------------------------------------------------------------------

=head2 ::bppolicynew

Creates a new empty policy

Executes following command:
B<bppolicynew $class -M $master>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bppolicynew($class,$master);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames
running NetBackup. Command will run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

=back

=cut

sub bppolicynew {
    my ($class,$master) = @_;
    my $result;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bppolicynew $class -M $master");
    return undef if $exit;
    return $result;
}
#----------------------------------------------------------------------
sub bppllist {
    my ($class) = @_;
    my %pllist = undef;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bppllist $class -U");
    return undef if $exit;

    my $plname = undef;
    foreach(split('\n',$result)) {
    if( /^Policy.*:\s+(.*)/ ) {
        $plname = $1;
        $pllist{$plname} = undef;
    } else {
        if( /(\S+.*):\s+(\S+.*)/ ) {
        my( $name, $val ) = ( $1, $2 );
        $name =~ s/\s+/_/g;
        $val  =~ s/\s+/ /g;
        $pllist{$plname}{$name} = $val;
        }
    }
    }
    return %pllist;
}

#----------------------------------------------------------------------

=head2 ::bpstuadd

creates NetBackup storage unit group or storage
unit

Executes following command:
B<bpstuadd -M $master -label $class -path $path -cj $noOfJobs -odo 1 -mfs $fragSize -cf 1 -maxmpx 1 -host $client -dt 2 -nh $nearstore -verbose>

=over

=item Syntax

 $result = bpstuadd($class,$master,$path,$client,$noOfJobs,$nearstore,$fragSize,$clearfile);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) Storage unit label


B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of hostnames.
If this option is present, the command is run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<path> => C<string> (I<Required>) Nearstore (FILER) Volume path name (eg., /vol/p2vol)

B<client> => C<string> (I<Required>)
The client from which the data needs to be backed up.


B<noOfJobs> => C<string> (I<Required>)
The maximum number of concurrent jobs that are
permitted for this storage unit.

B<nearstore> => C<string> (I<Required>) The filer to which the data has to be backed up on.

B<fragSize> => C<string> (I<Required>)
Specifies the maximum fragment size in megabytes,
or how large a fragment a NetBackup image can be.
NetBackup supports a maximum fragment size of
1,000,000 megabytes (1 terabyte).


B<clarFile> => C<string> (I<optional>)
Enables the Nearstore P3 feature to share at the
block-level and reduces the total amount of disk
space being used. This value is valid for disk
storage units only. Possible value: 0|1|2
0 for image backup
1 for clearfs backup
2 for soi backup

=back

=cut

sub bpstuadd ($$$$$$$;$) {
    my ($class,$master,$path,$client,$noOfJobs,$nearstore,$fragSize,$clearfile) = @_;
    unless (defined $clearfile) {
        $clearfile = 1;
    }
    my $cmd = "$admpath/bpstuadd -M $master -label $class -path $path -cj $noOfJobs -odo 1 -mfs $fragSize -maxmpx 1 -host $client -dt 2 -nh $nearstore -cf $clearfile  -verbose";
    #$cmd .= " -cf 1" if(defined($clearfile));
    my ($result,$exit) = client_say_cli(cmd=>$cmd);
    return undef if $exit;
    return 0 unless ($result =~ /EXIT status = 0/);
    return 1;
}

=head2 ::bpstuadd_p2

creates NetBackup storage unit group or storage unit for P2

Executes following command:
If C<$clearfile> is enabled, then
C<bpstuadd -M $master -label $class -path $path -cj $noOfJobs -odo 1 -mfs $fragSize -cf 1 -maxmpx 1 -host $client -dt 3 -nh $nearstore -verbose>

If C<$clearfile> is not enabled:
C<bpstuadd -M $master -label $class -path $path -cj $noOfJobs -odo 1 -mfs $fragSize -maxmpx 1 -host $client -dt 3 -nh $nearstore -verbose>

=over

=item Syntax

 $result = bpplinclude(e($class,$master,$path,$client,$noOfJobs,$nearstore,$fragSize,$clearfile);
 croak($@) if($@);

=item Arguments

B<class> => C<string> (I<Required>) Storage unit label

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of hostnames.
If this option is present, the command is run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<path> => C<string> (I<Required>) Nearstore (FILER) Volume path name (eg., /vol/p2vol)

B<client> => C<string> (I<Required>)
The client from which the data needs to be backed up.

B<noOfJobs> => C<string> (I<Required>)
The maximum number of concurrent jobs that are
permitted for this storage unit.

B<nearstore> => C<string> (I<Required>) The filer to which the data has to be backed up on.

B<fragSize> => C<string> (I<Required>)
Specifies the maximum fragment size in megabytes,
or how large a fragment a NetBackup image can be.
NetBackup supports a maximum fragment size of
1,000,000 megabytes (1 terabyte).


B<clarFile> => C<string> (I<optional>)
Enables the Nearstore P3 feature to share at the
block-level and reduces the total amount of disk
space being used. This value is valid for disk
storage units only.

=back

=cut

sub bpstuadd_p2 ($$$$$$$;$) {
    my ($class,$master,$path,$client,$noOfJobs,$nearstore,$fragSize,$clearfile) = @_;
    my $cmd = "$admpath/bpstuadd -M $master -label $class -path $path -cj $noOfJobs -odo 1 -mfs $fragSize -maxmpx 1 -host $client -dt 3 -nh $nearstore -verbose";
    $cmd .= " -cf 1" if (defined($clearfile));
    my ($result,$exit) = client_say_cli(cmd=>$cmd);
    return undef if $exit;
    return 0 unless ($result =~ /EXIT status = 0/);
    return 1;
}

sub bpstuadd_tape ($$$$$$$) {
    my ($class,$filer,$dma,$robot_num,$robot_type,$device_type,$no_of_jobs) = @_;

	bpstudel($class)
		if bpstulist(label=>$class);
    return _dma_exec("stu add for robot $robot_num", $admpath."/bpstuadd",
			"-label $class -density $device_type -cj $no_of_jobs -nh $filer -rt $robot_type -rn $robot_num -host $dma");
}

=head2 ::bpstulist

Displays one or all of NetBackup storage units or storage unit groups

Returns array of storage units. In case of error $@ will
hold error message and returns empty array.

Executes C<bpstulist>, if option C<long> is set, then executes C<bpstulist -L>, if C<label>
is provided, then executes C<bpstulist -label $label>

=over

=item Syntax

 $result = bpstulist( long=>0, label => 'whatever');

=item Arguments

B<long> => C<1|0> (I<optional>) Display output in long format, default is short

B<label> => C<string> (I<optional>) Display info for given label only

=back

=cut

sub bpstulist {
    my (%args) = @_;

    my @storageUnits = ();

    my $cmd = "$admpath/bpstulist";
    $cmd .= " -L" if($args{long});
    $cmd .= " -label $args{label}" if($args{label});

    my ($result, $exit) = client_say_cli(cmd=>$cmd);
    return undef if $exit;

    if ($result =~ /error\n/) {
    eval{ croak($result) };
    return @storageUnits;
    }

    foreach my $stor (split('[\r\n]',$result)) {
    if( $args{long} ) {
        push @storageUnits,$1 if( $stor =~ /^Label:\s+(.*)/);
    } else {
        push @storageUnits,$1 if ($stor =~ /^(\S+).+/);
    }
    }
    return @storageUnits;
}

=head2 ::bpstudel

The bpstudel command deletes a  NetBackup  storage  unit  or
storage  unit group. The command must include either a label
name for the storage unit or a group name  for  the  storage
unit group, but not both.

executes B<bpstudel -label $storLabel -verbose>

Returns 1 if success or 0 if error. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.


=over

=item Syntax

 $result = bpstudel($storLabel);

=item Arguments

B<storLabel> => C<string> (I<Required>) Storage label to be deleted


=back

=cut

sub bpstudel {
    my $storLabel = shift;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpstudel -label $storLabel -verbose");
    return 1 if($result =~ /EXIT status = 0/) ;
    eval { croak $result };
    return 0;
}

=head2 ::bppldelete

delete policies from the NetBackup database

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

Executes following command:
C<bppldelete $class>

=over

=item Syntax

 $result = bpplelete($class);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name


=back

=cut

sub bppldelete {
    my $class = shift;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bppldelete $class");
    return undef if $exit;
    return $result;
}

#----------------------------------------------------------------------
# bpdbjobs - Cleans up job reports
#----------------------------------------------------------------------
#sub bpdbjobs {
#    logcomment"Executing bpdbjobs\n";
#    open(JOBREP, "$admpath/bpdbjobs -report|");
#    while($line = <JOBREP>) {
#    $line =~ /^\s*(.*?)\s/;
#    my($status) = `$sudo $admpath/bpdbjobs -delete $1`;
#    }
#    close(JOBREP);
#    my($status) = `$sudo $admpath/bpdbjobs -clean`;
#}

=head2 ::bpbackup

Back up files to the NetBackup server

Executes following command: C<bpbackup -i -p $class -s $sched -S $master -w>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = bpbackup($class,$log,$sched,$master);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<log> => C<string> (I<Optional>) Specifies the name of a file in  which to  write progress  information.  NetBackup creates the file if it does not exist.

B<sched> => C<string> (I<Required>) The schedule to use for the backup.
If it is not specified, the NetBackup server uses the first user backup schedule it finds for  the  client  in the policy currently in use.

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of client hostnames running NetBackup. Command will run on each of the master servers in this list. The master servers must allow access by the system that issues the command. If an error occurs for any master server, the process stops at that point in the list. The default is the master
server for the system where the command is entered.

=back

=cut

sub bpbackup {
    my ($class,$log,$sched,$master) = @_;
    my ($result,$exit) = client_say_cli(cmd=>"$nbpath/bpbackup -i -p $class -s $sched -S $master -L $log -w ", timeout=>"-1");
    #if( $result =~ /the requested operation.*\bsuccessful\b/i ) {
#    $@=undef;   # masking "EXIT STATUS 1: the requested operation was partially successful"
#    return 1;
#    }
    return undef if $exit;
    return $result;
}

=head2 ::bpbackup_tape

Initiate a tape backup using the given policy

=over

=item Argument

B<class> => I<string>

Policy name to perform backup with.

=back

=item Example

	backup_tape("device_rst1a_policy");

=cut

sub bpbackup_tape {
    my ($class) = @_;
    return _dma_exec("Perform tape backup of $class", "$nbpath/bpbackup", "-p $class -i");
}

=head2 ::bprestore_tape

Initiate a restore from tape via NDMP.

=over

=item Arguments

B<class> = I<string>

The backup policy name.

B<master> = I<dma master>

The host name running NBU application.

B<src_filer> = I<filer name>

The backup filerup.

B<dst_filer> = I<filer name>

The restore filer name

B<starttime> = I<datetime>

The starttime to search for data to restore. Format 'mm/dd/yyyy [hh:mm:ss]'

B<endttime> = I<datetime>

The endtime to search for data to restore. Format 'mm/dd/yyyy [hh:mm:ss]'

B<srcpath> = I<backup path>

The backup path to search for data to restore.

B<dstpath> = I<restore path>

The restore path to put data to.

=back

=item Example:

=cut

sub bprestore_tape {
    my ($class, $master, $src_filer, $dst_filer, $starttime, $endtime, $srcpath, $dstpath) = @_;
	$dst_filer = $src_filer
		unless $dst_filer;
	my $type = 19; # NBU's NDNP type
	my $renameFile = "/usr/tmp/renameFile".$$;
	createRenameFile($renameFile, $srcpath, $dstpath);
    return _dma_exec("perform tape restore of $class, src $srcpath to dst $dstpath",
				"$nbpath/bprestore", "-S $master -t $type -p $class -s $starttime -e $endtime -C $src_filer -D $dst_filer -R $renameFile $srcpath");
}

=head2 ::bpdbjobs_cancel

Cancel or abort an NBU job.

=item Arguments

B<jobid> = I<integer>

NBU job id.

=back

=item Example

	bpdbjobs_cancel(5);

=cut

sub bpdbjobs_cancel {
	my ($jobid) = @_;

    return _dma_exec("abort/abort job $jobid", $admpath.".bpdbjobs", "-cancel $jobid");
}

=head2 ::bprestore

restore files from the NetBackup server

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

Executes following command:
if C<$dstpath> then
C<bprestore -w 00:10:00 -L $logfile -p $class -s $starttime -e $endtime -C $sourceclient -D $destclient -R $renameFile $srcpath>
and if not then
C<bprestore -w 00:10:00 -L $logfile -p $class -s $starttime -e $endtime -C $sourceclient -D $destclient $srcpath>

=over

=item Syntax

 $result = bprestore($class,$starttime,$endtime,$sourceclient,$destclient,$logfile,$srcpath,$dstpath);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<starttime> => C<string> (I<Required>)
Specifies the start date for the listing. The bprestore command restores only
files from backups or the archives that occurred within the specified start and end date range.

B<endtime> => C<string> (I<Required>)
Specifies the end date for the listing.

B<srcClient> => C<string> (I<Required>)
Specifies a client name to use for finding backups or archives from which to restore
files. This name must be as it appears in the NetBackup catalog.
The default is the current client

B<dstclient> => C<string> (I<Optional>)
Specifies a destination client. The master server root user can use this option to direct
restored files to a machine other than the client
that the srcClient. The default is the current client name.

B<log> => C<string> (I<Required>)
Specifies the name of an existing file in which to write progress information.
The default is not to use a progress log.

B<srcPath> => C<string> (I<Required>) The source path of the data to be backed from

B<destPath> => C<string> (I<Optional>) The dest path of the data to be backed up.

=back

=cut

sub bprestore($$$$$$$;$) {
    my($class,$starttime,$endtime,$sourceclient,$destclient,$logfile,$srcpath,$dstpath) = @_;
    my $renameFile = "/usr/tmp/renameFile".$$;

    logcomment("Executing bprestore $class,$starttime,$endtime,$logfile,$srcpath,$dstpath");
    my ($result,$exit);
    if (defined($dstpath)) {
    &createRenameFile($renameFile,$srcpath,$dstpath);
    ($result,$exit) = client_say_cli(cmd=>"$nbpath/bprestore -w 00:10:00 -L $logfile -p $class -s $starttime -e $endtime -C $sourceclient -D $destclient -R $renameFile $srcpath", timeout=>"-1");
    } else {
    ($result,$exit) = client_say_cli(cmd=>"$nbpath/bprestore -w 00:10:00 -L $logfile -p $class -s $starttime -e $endtime -C $sourceclient -D $destclient $srcpath", timeout=>"-1");
    }
    return undef if $exit;
    return $result;
}

=head2 ::add_tpconfig

Adds new filer to NBU server's configuration

Executes following command:
B<tpconfig -add -nh $filer -user_id $user -password $password -snap_vault_filer>

Returns output from command including output from STDERR. Checks for exit code and
places error in $@ if any. Script may check for $? and $@.

=over

=item Syntax

 $result = add_tpconfig( filer => 'hostname', user => 'username', password=>'incognito' );

=item Arguments

B<filer> => C<string> (I<Required>) Filer's host name

B<user> => C<string> (I<Required>) NetBackup user name, default is C<root>

B<password> => C<string> (I<Required>) NetBackup user's password, default is netapp

=back

=cut

# /usr/openv/volmgr/bin/tpconfig -add -nh <NAS HOST> -user_id <userid> -password <password>
sub add_tpconfig {
    my($filer,$user,$password) = @_;
    $user ||= 'root';
    $password ||= 'netapp';
    my ($result,$exit) = client_say_cli(cmd=>"/usr/openv/volmgr/bin/tpconfig -add -nh $filer -user_id $user -password $password -snap_vault_filer");
    return undef if $exit;
    return $result;
}

=head2 ::add_tpconfig_list

Adds a list of tape devices to NBU server's configuration. The routine first calls cln_tpconfig_list routine
to remove any configuration for the named filer/storage unit before add the new configuration.

=over

=item Arguments

B<filer> => I<string>

Name of filer to use.

B<stu_name> = I<string>

Storage Unit name to use in the configuration.

B<user> => I<string>

User name to login to filer. Default: root

B<password> => I<string>

Paswword to use in login. Default: netapp1!

B<dma> = I<string>

Name of client running NBU application.

B<robot_name> = I<string>

Name of the robot on the filer to use.

B<rotbot_type> = I<string>

Robot type.

B<device_type> = I<string>

tape device type.

=back

=cut

sub add_tpconfig_list {
	my ($filer, $stu_name, $user, $password, $dma, $robot_name, $robot_type, $device_type, @device_list) = @_;

	my $user = $user || "root";
	my $pwd = $password || "netapp1!";

	my ($robot_num, $lun_num) = cln_tpconfig_list($filer, $stu_name);
	# Add filer
	my $host_param = "-nh $filer";
	my $result = _dma_exec ("filer $filer", $cfgpath."tpconfig", "-add $host_param -user_id $user -password $pwd");

	# Add robot
	my $rob_param = "-robot $robot_num -robtype $robot_type";
	eval {
		$result = _dma_exec("robot $robot_name", $cfgpath."tpconfig", "-add $rob_param -robpath $robot_name -nh $filer");
		$result = _dma_exec("update robot $robot_name", $cfgpath."tpconfig", "-update $rob_param -bus 0 -target 0 -lun $lun_num");
	};
	croak "Failed to add robot $robot_name: $@"
		if $@ && ! ($@ =~ /serial numbers match/i);

	# Add tape device(s)
	croak "Missing device list."
		unless @device_list;
	my $dev_param = "-type $device_type -robdrnum ".($robot_num+1);
	foreach  my $device (@device_list) {
		$result = _dma_exec("drive $device", $cfgpath."tpconfig",
							"-add -drive $dev_param -path $device $rob_param $host_param"); # ? may need robdrnum
	}

	_restart_media_mgr();
	$result = _dma_exec("Robot DB Unpdate for $robot_name", $cfgpath."vmupdate", "-rt $robot_type -rn $robot_num");

	return $robot_num;
}

sub _restart_media_mgr {
	_dma_exec("Stop media manager", $cfgpath."stopltid");
	eval {
		_dma_exec("Start media manager", $cfgpath."ltid", "", timeout=>5);
	};
}

=head2 ::cln_config_list

Remove all tape device configuration associated with a filer.

=over

=item Arguments

B<filer> = I<string>

Name of filer.

B<stu_name> = I<string>

Storage Unit to revmove.

B<user> = I<string>

User name tp login to filer. Default: root

B<password> = I<string>

Password to use in the login.

=back

=cut

sub cln_tpconfig_list {
	my ($filer, $stu_name, $user, $password) = @_;

	my $user = $user || "root";
	my $pwd = $password || "netapp";

	bpstudel($stu_name)
		if bpstulist(label=>$stu_name);
	my $list = _dma_exec("tape config list", $cfgpath."tpconfig", "-emm_dev_list");
	my @cfg_list = _get_emm_blk($list);
	my ($robot_num, $lun_num) = (0, 0);
	foreach my $cfg (@cfg_list) {
		if ( exists $cfg->{robot} ) {
			if ( $cfg->{'ndmp attach host'} =~ /$filer/i ) {
				_dma_exec("Remove Robot cfg", $cfgpath."tpconfig", "-delete -robot ".$cfg->{'robot number'});
			} else {
				my $this_rnum = $cfg->{'robot number'};
				$robot_num = $this_rnum + 1
					if $this_rnum >= $robot_num;
				my $this_lun = $cfg->{'lun'};
				$lun_num = $this_lun + 1
					if $this_lun >= $lun_num;
			}
		}
		if ( exists $cfg->{drive} && $cfg->{'ndmp host'} =~ /$filer/i ) {
			_dma_exec("Remove Drive cfg", $cfgpath."tpconfig", "-delete -drive ".$cfg->{'index'});
		}
	}
	eval {
		_dma_exec("Remove Filer $filer cfg", $cfgpath."tpconfig", "-delete -nh $filer -user_id $user");
	};
	logcomment("Next Robot #: $robot_num, LUN #: $lun_num");
	return ($robot_num, $lun_num);
}

sub _get_emm_blk {
	my ($list) = @_;

	my @lines = split "\n", $list;
	shift @lines; # skip first "=" line
	my (@list, $cfg);
	while ( my $line = shift @lines ) {
		if ( $line =~ /^=/ ) {
			push @list, $cfg;
			$cfg = undef;
			shift @lines
				if $lines[0] =~ /^=/; # Skip second "=" line
		} else {
			my ($k, $v) = split /:/, lc($line);
			$v =~ s/\s+(\w+)\s*/$1/;
			$cfg->{$k} = $v;
		}
	}
	push @list, $cfg
		if defined $cfg;

	return @list;
}

sub _dma_exec {
	my ($title, $exec, $params, %args) = @_;

#    my ($result,$exit) = client_say_cli(cmd=>"rsh sdqa-c68 /usr/software/bin/sudo $exec $params");
    my ($result,$exit) = client_say_cli(cmd=>"$exec $params", %args);
    if ( $exit ) {
		croak "$exec for $title failed. Params: $params. Exit: $exit. Result: $result";
	} else {
		return $result;
	}
}

=head2 ::currentDate

Returns the current date

=over

=item Syntax

 $result = currentDate();

=back

=cut

sub currentDate {
    my $temp;
    open(DATE, "date '+%m/%d/%y %H:%M:%S'|");
    chomp($temp = <DATE>);
    close(DATE);
    return $temp;
}

=head2 ::createRenameFile

=over

=item Syntax

 $result = createRenamedFile($file,$source,$dest);

=back

=cut




sub createRenameFile {
    my ($file,$source,$dest) = @_;
    #my $status = system("$sudo /usr/bin/echo change $source to $dest >$file");
    #my $status = system("$sudo echo change $source to $dest >$file");
    client_say_cli(cmd=>"$sudo echo change $source to $dest >$file");
}

=head2 ::checkForsSuccess

=over

=item Syntax

 $result = checkForsSuccess

=back

=cut

sub checkForSuccess {
        my $log = shift;
        my $success = "Status = the restore successful";
        my $failure = "Status = the restore failed to recover";

        open(LOGFILE,"<$log") || die "Can't open $log for reading:$!";

        while (<LOGFILE>) {
            logresult("FAIL","NBU RESTORE: FAILED, $_")
                 if ($_ =~ /$failure/) ;
    
            logcomment("NBU RESTORE: SUCCESSFUL")
                 if ($_ =~ /$success/) ;
        }
    
}
# Cleanup shouldn't be part of library as it could be more specific to test case.
# i.g., Some scripts may want to cleanup a portion of this cleanup does.
#sub cleanUp {
#        my $log = shift;
#
#        if (-f $log) {
#            `$sudo /usr/bin/rm $log`;
#        }
#      &bpplinclude($classname, $masterserver,"-delete", $SOURCE_PATH);
#      &bpplscheddel($classname, $schedname);
#      &bpplclients($classname, "-delete", $clientname,$masterserver,$hardware,$os);
#        &bppldelete($classname);
#}

sub compareData {
  my ($tree1, $tree2) = @_;
  my $diff = 1; # no diffs at this time.

  if (-e "$TESTBIN/compare") {
    #my $results = `$sudo $TESTBIN/compare $tree1 $tree2 2>&1`;
    my ($results, $exit) = client_say_cli(cmd=>"$sudo $TESTBIN/compare $tree1 $tree2", timeout=>"-1");

    if ($results =~ /Comparison of $tree1 to $tree2 failed/) {
      logresult("FATAL", "Compare failed ($results)");
      return 0;
    }
    logcomment( "Compare ($results)");
    my @lines = split(/\n/,$results);
    foreach my $text (@lines) {
      if ($text =~ /^(\d+).*/) {
        $diff = 0
          if $1;
        last
          unless $diff;
      }
    }
    return $diff ? 1 : 0;
  } else {
      logresult("FATAL", "$TESTBIN/compare utility not found");
      return 0;
  }
}


#
#----------------------------------------------------------------------
# bpimagelist - returns image information based on the client and policy
#----------------------------------------------------------------------

=head2 ::backupInfo

returns image information based on the client and policy. Function is incomplete! Do not use.

=over

=item Syntax

 $result = backupInfo($class,$option);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<option> => C<string> (I<Optional>) Options, see bpimagelist
man page for details

=back

=cut

sub backupInfo {
    my ($class,$option) = @_;
    my $NO_ENTITY = "no entity was found";
    my $SOCK_ERROR = "cannot connect on socket";
    
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpimagelist $option -policy $class", '-timeout'=>3600);
    return undef if $exit;

    if ( (lc($result) =~ $NO_ENTITY ) || (lc($result) =~ $SOCK_ERROR ) ) {
    return 0;
    }
    my @backups = split(/\n/,$result);

    foreach my $backup (@backups) {
    if ($backup =~ /.+ID:\s*(\S+)\s*(\S+)/) {
    }
    }

}

=head2 ::backupIDs

returns a list of backup id of a given policy and schedule type.

=over

=item Syntax

 @backup_ids = backupIDs($class,$schd_type);

=item Arguments

B<class> => C<string> (I<Required>) NetBackup policy name

B<schd_type> => C<string> (I<Optional>) schedule type, see bpimagelist
man page for details FULL|INCR|CINC|UBAK|UARC|NOT_ARCHIVE, default is FULL

=back

=cut

sub backupIDs {
    my ($class,$schd_type) = @_;
    my $NO_ENTITY = "no entity was found";
    my $SOCK_ERROR = "cannot connect on socket";
    my $type = $schd_type ? $schd_type : "FULL";
    my @backup_ids;
    
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpimagelist -L -policy $class -st $type", '-timeout'=>3600);
    return undef if $exit;

    if ( (lc($result) =~ $NO_ENTITY ) || (lc($result) =~ $SOCK_ERROR ) ) {
    return 0;
    }
    
    my @backups = split(/\r?\n/,$result);
    foreach my $line (@backups) {
        if ($line =~ /^Backup ID:(\s*)(.*)$/ ) {
            my $backup_id = $2;
            push(@backup_ids, $backup_id);
        }
    }
    return @backup_ids;
}


=head2 ::exprImage

 expire a backup image

=over

=item Syntax

 $result = exprImage($backupID, $expr_time, $nodelete);

=item Arguments

B<backupID> => C<string> (I<Required>) Backup ID 

B<expr_time> => C<string> (I<Optional>) expiration time, see bpexprdate
man page for details , default is 0 

B<$nodelete> => C<0|1> (I<Optional>) To delete the image from storage disk or not. default is 0, that image will be deleted from catalog as well as storage disk, if set to 1, image will be only deleted from catalog but not from storage disk

=back

=cut

sub exprImage {
    my ($backupid,$expr_time,$nodelete) = @_;
    my $time = $expr_time ? $expr_time : 0;
    my ($result, $exit);

    if(defined $nodelete) {
         ($result,$exit) = client_say_cli(cmd=>"$admpath/bpexpdate -backupid $backupid -d $time -force -nodelete", '-timeout'=>200);
    } else {
         ($result,$exit) = client_say_cli(cmd=>"$admpath/bpexpdate -backupid $backupid -d $time -force  ", '-timeout'=>3600);
    }
    return undef if $exit;
    return 1;
}


=head2 ::getOSType

 Returns OS type of the hostname

=over

=item Syntax

$result = getOSType();

=back

=cut

sub getOStype {

    my $os = `uname -s`;
    chomp($os);
    
    if ($os =~ /SunOS/) {
                    my $version = `uname -r`;
                    chomp($version);
        
                    my (undef, $ver) = split(/\./,$version);
                    return "Solaris"."$ver";
     }
     elsif ($os =~ /Linux/) {
                    return "TBD";
   }

}


=head2 ::getHardwareInfo

Returns Hardware type of the hostname

=over

=item Syntax

$result = getHardwareInfo();

=back

=cut

sub getHardwareInfo {

    my $hardware = `uname -p`;
    chomp($hardware);
    
  return "Solaris"
             if ($hardware =~    /sparc/);

    # This must be Linux, -m gives iXXX  (or ??)
    if ($hardware =~ /unknown/ || $hardware =~ /i686/)  {
            #        my $lin = `uname -m`;
            #        chomp($lin);
            #   return "Linux"  # should it be something else TBD.
            return "Linux";
  }

}

=head2 ::Populate

=over

=item Arguments

none

=back

=cut


sub populate {
  my $path = shift;
  system("$sudo mkdir -p $path")
    unless (-e $path);
  my $results = `$sudo $TESTBIN/populate $path 2>&1`;

  logcomment("$results");

   if ($results =~ /Population of $path failed/) {


      return 0;
  }


  return 1;
}

=head2 ::Modify

=over

=item Arguments

B<path>    UNIX Path to modify

=back

=cut

sub modify {
  my $path = shift;
  my $results = `$sudo $TESTBIN/modify $path 2>&1`;

  logcomment("$results");
  if($results =~ /Modification of $path failed/) {
            eval { croak $results; };
             return 0;
        }


  return 1;
}

=head2 ::bounce_pbx

=over

=item Arguments

=back

=cut

sub bounce_pbx ($){
    my $option = shift;
  my $list = `ps -aef | grep pbx`;

  my @procs = split(/\n/,$list);
  my $found;
  foreach my $proc (@procs) {
    if ($proc =~ /pbx_exchange/) {
      $found = $proc;
      last;
    }
  }
  if ($found =~ /\s+\w+\s*(\d+)\s+.*/) {
    my $pbx = $1;
    `$sudo kill -9 $pbx`;
  }

  # Start pbx if $option set to "start"
  # Otherwise do not start them.
  #
    if (lc($option) eq "start") {
                    system("$sudo /opt/VRTSpbx/bin/start_pbx.sh");
    }
}


=head2 ::bpsturep

replace selected NetBackup storage unit attributes

=over

=item Syntax

 $result = bpsturep($class,$master,$path,$client,$noOfJobs,$nearstore);

=item Arguments

B<class> => C<string> (I<Required>) Storage Unit label.

B<master> => C<string> (I<Required>)
A list of master servers. This list is a comma - separated list of hostnames.
If this option is present, the command is run on each of the master
servers in this list. The master servers must allow access by the system
that issues the command. If an error occurs for any master server,
the process stops at that point in the list. The default is the master
server for the system where the command is entered.

B<path> => C<string> (I<Required>)
The path to a disk file system,  expressed  as  an
absolute  pathname, the data storage area for this
storage unit.

B<client> => C<string> (I<Required>)
The client from which the data needs to be backed up.

B<noOfJobs> => C<string> (I<Required>)
The maximum number of  concurrent  jobs  that  are
permitted  for  this  storage  unit.

B<nearstore> => C<string> (I<Required>)  The filer to which the data has to be backed up on.

=back

=cut



sub bpsturep {
    my ($class,$master,$path,$client,$nearstore) = @_;
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpsturep -M $master -label $class -path $path -host $client -nh $nearstore -verbose");
    return 0 unless ($result =~ /EXIT status = 0/);
    return 1;
}


=head2 ::addbpconfVSM

Add NEARSTORE_FAILOVER_SERVER entry to NBU_MASTER:/usr/openv/netbackup/bp.conf
the format is:
NEARSTORE_FAILOVER_SERVER = [vsm-src-filer]:/vol/[vsm-src-vol] [vsm-dst-filer]:/vol/[vsm-dst-vol]
Also add the comment marker before this line for later to remove the entry.

=over

=item Syntax

 addbpconfVSM($marker,$srcfiler,$srcvol,$dstfiler,$dstvol);

=item Arguments

B<srcfiler> => C<string> (I<Required>)VSM source filer hostname. 

B<srcvol> => C<string> (I<Required>) VSM source volume name.

B<dstfiler> => C<string> (I<Required>)VSM destination filer hostname. 

B<dstvol> => C<string> (I<Required>) VSM destination volume name.

B<marker> => C<string> (I<Optional>) Marker to set in the file.

=back

=cut

sub addbpconfVSM($$$$;$) {
    my ($srcfiler, $srcvol, $dstfiler, $dstvol, $marker) = @_;
	my $bpconfile = "/usr/openv/netbackup/bp.conf";

    $marker = $marker ?  $marker : $$;
    my $stream = "MARKER FOR JOBID $marker";
    client_say_cli(cmd=>"$sudo echo \\# $stream  >>$bpconfile"); 

    $stream = "NEARSTORE_FAILOVER_SERVER = $srcfiler:/vol/$srcvol $dstfiler:/vol/$dstvol";
    my ($result,$exit) = client_say_cli(cmd=>"$sudo echo $stream  >>$bpconfile"); 

    return 0 unless ($result =~ /EXIT status = 0/);
    return 1;

}


=head2 ::rmbpconfVSM

Delete entry created by addbpconfVSM.
search for the given marker and delete from the line to the bottom of the file.

=over

=item Syntax

 rmbpconfVSM();

=item Arguments

B<marker> => C<string> (I<Optional>) Search marker.

=back

=cut

sub rmbpconfVSM() {
    my ($marker) = @_;
    $marker = $marker ?  $marker : $$;
	my $bpconfile = "/usr/openv/netbackup/bp.conf";
	my $cp = "/usr/openv/netbackup/bp.conf.old";

    my ($result,$exit) = client_say_cli(cmd=>"$sudo mv $bpconfile $cp "); 
    my ($result,$exit) = client_say_cli(cmd=>"$sudo sed -e \"/^\# MARKER FOR JOBID $marker/, \$ d\" < $cp > $bpconfile"); 

    return 0 unless ($result =~ /EXIT status = 0/);
    return 1;
}


=head2 ::bpbjobs

bpdbjobs interacts with the jobs database and is  useful  in
scripts  or  as  a  command  line  administration  tool. Use
bpdbjobs to print the entire jobs database, print a  summary
of  the database, delete done jobs, cancel uncompleted jobs,
and clean old jobs.

Returns a hash of all the JobIds like this for (jobid = 578):

    '578' => {
                      'profile' => '08/15/07 02:08:25 - Error bpdm(pid=5592) storage server not specified',
              ...
                      'pathlastwritten' => '',
                      'try' => '1',
                      'started' => '1187168903'
                    }

In case of error $? will hold error message and function returns C<Undef>.

=over

=item Syntax

 $result = bpdbjobs();

=back

=cut

sub bpdbjobs {
    my %ret = ();
    my ($result,$exit) = client_say_cli(cmd=>"$admpath/bpdbjobs -report -all_columns");
    return undef if $exit;

    foreach my $jtxt (split('\n',$result))
    {
    my @elm = split(',',$jtxt);
    my $jobid = shift @elm;

    $ret{$jobid}{$_} = shift @elm foreach(qw( jobtype state status policy schedule client server started elapsed ended #stunit try operation kbytes files pathlastwritten percent jobpid owner subtype class type schedule_type priority group masterserver retentionunits retentionperiod compression kbyteslastwritten fileslastwritten filelistcount files trycount trypid trystunit tryserver trystarted tryelapsed tryended trystatus trystatusdescription trystatuscount trystatuslines trybyteswritten tryfileswritte nparentjob kbpersec copy robot vault profile session ) );
    }
    return %ret;
}

use constant BLOCKSIZE => 4096;
sub file_compare ($$) {

    my ($src, $dest) = @_;
    open(SRC, "<$src") || die "Can't open $src for reading\n";
    open(DEST, "<$dest") || die "Can't open $dest for reading\n";
    my $counter = 0;
    my ( $src_bytes_read, $dst_bytes_read, $srcbuf, $dstbuf) = (undef);
    while ($src_bytes_read = sysread(SRC,$srcbuf,BLOCKSIZE)) {

        $dst_bytes_read =  sysread(DEST, $dstbuf,BLOCKSIZE);
        die "Data mismatch found\n" if ($src_bytes_read != $dst_bytes_read) ;
        die "Data mismatches found after ".$counter*BLOCKSIZE."bytes\n"
        unless ($srcbuf eq $dstbuf);
        $counter++;
    }
        die "File compare: $dest is longer than $src\n" if (read(DEST,$dstbuf,BLOCKSIZE));
}

