package NISST::Mjolnir::MjolnirUtils;

# Compiler directives.
use strict;
use warnings;
use Tharn qw($Log);
use NATE::BaseException qw(:try);
use NATE::Inc qw(find_file);

################################################################################
# METHODS
################################################################################

sub random_sleep {
    my %args          = @_;
    my $minimum_delay = $args{minimum_delay};
    my $delay_range   = $args{delay_range};

    # Sleep somewhere between Minimum_Delay and Minimum_Delay+Delay_Range
    # secs
    my $sleep_duration = int( rand($delay_range) ) + $minimum_delay;

    $Log->comment( "Sleeping for " . $sleep_duration . " secs..." );
    sleep($sleep_duration);
    $Log->comment("Resuming from sleep...");
}

sub ignore_exception {
    my $exception = shift;
    $Log->comment(
        "Bad happened, caught an exception following are the details ");
    $Log->comment( "Exception srtingfied [" . $exception->stringify() . "]" );
    $Log->comment( "Exception stacktraced [" . $exception->stacktrace() . "]" );
    $Log->comment( "Exception file [" . $exception->file() . "]" );
    $Log->comment( "Exception line [" . $exception->line() . "]" );
    $Log->comment("With the information logged we will continue...");
}

sub validate_task_scripts {
    my ( $task_map ) = @_;
    
    $Log->comment("Verifying the task performing scripts' path");
    #---------------------------------------------------------------------------
    # Verify for the presence of the scripts that will perform the task
    # Failing the verification abort tasks
    #---------------------------------------------------------------------------

    while ( my ( $task_name, $script_path ) = each(%$task_map) ) {
        $Log->comment( "Finding the absolute path of script ["
              . $script_path
              . "] to perform ["
              . $task_name
              . "]" );
        my $abs_file_path = find_file($script_path)
          or die "Unable to find the script in path for ["
          . $task_name
          . "]:\n Reason ["
          . $@ . "]";
        $task_map->{$task_name} = $abs_file_path;
        $Log->comment( "Absolute path of script ["
              . $task_map->{$task_name}
              . "] will perform ["
              . $task_name
              . "]" );
    }
    
    return $task_map;
}

1;
################################################################################
# Lib Ends
################################################################################
