#
# $Id: //depot/prod/test/main/lib/NOTE/DataIntegrity.pm $
#
## @summary  Integrity Check Functions
## @status   Review
## @pod      here
## @author   Shailendra.CH@netapp.com

=head1 NAME

NOTE::DataIntegrity

=head1 DESCRIPTION

This is a simple integrity check library works on couple works at directory level
Verifies the source is similar to destination.
Uses File Find Module, File Stat Module, POSIX compliant fcntl for acls


=head1 METHODS

=cut


package NOTE::DataIntegrity;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
require Exporter;
use base 'Exporter';
our @EXPORT = qw(IntegrityCheck);
use strict;
use IO::File;
use File::Find;
use File::stat;
use File::Basename;
use POSIX;
use POSIX qw(setsid);
use POSIX qw(:errno_h :fcntl_h);
use Carp qw(croak);

my $MYFILE = '';
my @files  = ();
my $dir    = '';

#Constructor for the DataIntegrity Lib
sub new {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $package = shift;
 return bless( {}, $package );
}

#Call back function to defined for file find accumilates filenames
sub _mySub() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 push @files, $File::Find::name;
}

#Enumeration Routine called from seach dir
sub _loadFiles($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 find( \&_mySub, "$dir" );
}

#File Find Routine
sub _searchdir($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 $dir = shift || croak( "Search Iteration Failure\n");
 _loadFiles($dir);
}

#Removes whitespace in a string
sub _trim($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $string = shift;
 $string = _rtrim(_ltrim($string));
 return $string;
}

#Removes whitespace in a string from left
sub _ltrim($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $string = shift;
 $string =~ s/^\s+//;
 return $string;
}

#Removes whitespace in a string from right
sub _rtrim($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $string = shift;
 $string =~ s/\s+$//;
 return $string;
}

#provides timestamp
sub _timestamp() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my @months   = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec);
 my @weekDays = qw(Sun Mon Tue Wed Thu Fri Sat Sun);
 my (
  $second,     $minute,    $hour,
  $dayOfMonth, $month,     $yearOffset,
  $dayOfWeek,  $dayOfYear, $daylightSavings
   )
   = localtime();
 my $year    = 1900 + $yearOffset;
 my $theTime =
"$hour:$minute:$second, $weekDays[$dayOfWeek] $months[$month] $dayOfMonth, $year";
 return $theTime;
}

#gets acl for a give file using POSIX norms
sub _getacl($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $fname = shift || croak('no filename _getacl cannot derive fstat');
 my $fd = POSIX::open( $fname, &POSIX::O_RDONLY );
 my @stts = POSIX::fstat($fd);
 POSIX::close($fd);
 delete $stts[0];
 return @stts;
}

#gets the content of a file at given offset
sub _readatoffset($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $fileHandle = $_[0];
 my $offst = $_[1] || croak('no offset provided');
 seek $fileHandle, $offst, 0;
 my ( $buf, $data, $n );
 if ( ( $n = read $fileHandle, $data, 10 ) != 0 ) {
  $buf .= $data;
 }
 return $buf;
}

#verifies the integrity of a files from previously accumilated data
sub _verify_dataintegrity_metadata() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $aclcounter  = -1;
 my %failurelist = ();
 my %missingfileslist = ();
 my $fnm         = '';
 my $fnsz        = '';
 my @acl         = ();
 my $line        = '';
 my $fndelmit    = '';
 my $fdtdelmit   = '';
 my $fOfst       = '';
 my $fOfstDt     = '';
 my $vdata       = '';
 my $FILEHANDLE;
 my $tmpfilename = '';
 my $flecounter  = 0;
 my $flrcounter  = 0;
 my $dstdir      = shift
   || croak(
  "_verify_dataintegrity_metadata requires a source and destination directory to verify ....");
 my $srcdir = shift
   || croak(
  "_verify_dataintegrity_metadata requires a source and destination directory to verify ....");
 open( MYINPUTFILE, "<$ENV{LOGDIR}/dataIntegrity.txt");

 while (<MYINPUTFILE>) {
  $line = $_;
  chomp($line);
  $fndelmit = index( $line, '<->' );
  if ( $fndelmit > 0 ) {
   $fnm = substr( $line, 0, $fndelmit );
   $fnsz = substr( $line, $fndelmit + 3 );
   @acl        = &_getacl($fnm);    # // next 11-12 lines contain acl
   $flecounter = $flecounter + 1;
   $fnm =~ s/$srcdir/$dstdir/;
    unless (-e $fnm) {
 $missingfileslist{$fnm} = $fnm . ' <file does not exist> ';
 } 
   open( $FILEHANDLE, "< " . $fnm );
  }
  if (index( $line, '<+>' ) == -1
   && index( $line, '<->' ) == -1
   && $line =~ m/^\d+$/ )
  {
   foreach (@acl) {
    my $tmp = $_;
    $tmp  = &_trim($tmp);
    $line = &_trim($line);
    if ( $_ eq $line ) {
     goto cntloop;
    }
   }
   if ( exists( $failurelist{$fnm} ) ) { goto cntloop; }
   else {
    my $ftmp = '';
    foreach (@acl) {
     $ftmp = $ftmp . '-' . $_;
    }
    $failurelist{$fnm} = $line . ' <acl mismatch> ' . $ftmp;
   }
  }
  $fdtdelmit = index( $line, qw('<+>'));
  if ( $fdtdelmit > 0 ) {
   $fOfst   = substr( $line,               0, $fdtdelmit );
   $fOfstDt = substr( $line,               $fdtdelmit + 3 );
   $vdata   = &_readatoffset( $FILEHANDLE, $fOfst );
   $vdata   = _trim($vdata);
   $fOfstDt = _trim($fOfstDt);

   # binary str compare logic would be good & faster but this works too......
   if ( $vdata ne $fOfstDt && length($fOfstDt) == length($vdata) ) {
    if ( exists( $failurelist{$fnm} ) ) { goto cntloop; }
    $failurelist{$fnm} = $fOfstDt . ' <data mismatch> ' . $vdata;
   }
   else {
    if ( exists( $failurelist{$fnm} )) {
     delete $failurelist{$fnm};
    }
   }
  }
cntloop:
 }
 close(MYINPUTFILE);

foreach my $key ( keys %missingfileslist )
{
   if ( exists( $failurelist{$key} ) )
   {
     delete $failurelist{$key};
   } 
}
 open( MYFILE, ">$ENV{LOGDIR}/mismatch.txt" );
 while ( my ( $key, $value ) = each(%failurelist) ) {
  print MYFILE "$key => $value\n";
 }
 close(MYFILE);
 
open( MYFILE, ">>$ENV{LOGDIR}/mismatch.txt" );
 while ( my ( $key, $value ) = each(%missingfileslist) ) {
  print MYFILE "$key => $value\n";
 }
 close(MYFILE);
 my $problems = keys(%failurelist) + keys(%missingfileslist);
 return "total files checked for integrity $flecounter anomalies found in " .
  $problems. "\n";
}

#creates integrity data for verification
sub _create_dataintegrity_metadata($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $srcDir = shift
   || croak(
"_create_dataintegrity_metadata requires a source directory path ...."
   );

 _searchdir($srcDir) && _openlog();
 foreach (@files) {
  my ($line) = $_;
  my $filesize = -s $line;
  my $minsize = sprintf( "%d", $filesize );
  if ( ( -d $line ) || ( $minsize < 100 ) ) {
   goto eol;
  }
  else {
   _logoutput( $line . "<->" . $minsize );
   if ( -e $line ) {
    my @stts = &_getacl($line);
    foreach my $elem (@stts) { _logoutput($elem); }
    &_getrandomreads( $filesize, $line );
   }
  }
eol:
 }
 &_closelog();
}

#open file for accumilating integrity data of a given src
sub _openlog() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 open( MYFILE, ">$ENV{LOGDIR}/dataIntegrity.txt" );
}

#close file for accumilating integrity data of a given src
sub _closelog() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


 close(MYFILE);
}

#log data integrity data
sub _logoutput() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my $m = shift or return;
 print MYFILE "$m\n";

}

#filters directories from files
sub _wanted {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 if (-d) {
  return 1;
 }
}

#generate random offsets within a given limit
sub _getrandomreads($) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 my @nums    = ();
 my @unique  = ();
 my %Seen    = ();
 my $range   = $_[0];
 my $fname   = $_[1];
 my $counter = 100;
 my $data    = '';
 srand( time() ^ ( $$ + ( $$ << 15 ) ) );

 for ( my $x = 0 ; $x < $counter ; $x++ ) {
  push @nums, ceil( rand($range) );
 }
 foreach my $elem (@nums) {
  next if $Seen{$elem}++;
  push @unique, $elem;
 }
 open( my $FILEHANDLE, "< " . $fname );
 foreach my $elem (@unique) {
  $data = &_readatoffset( $FILEHANDLE, $elem );
  &_logoutput( $elem . "<+>" . $data );
 }
 close($FILEHANDLE);
}

=head2 ::integrity_check

This function uses the adaptive integrity check algorithm to verify the integrity of the file, 
from a given source directory recursive search of available files are performed, followed by  
data retrival at random offsets, this information along with the access control list is stored 
tentatively.
the above step is followed by verification of this information the src directory is replaced with 
destination directory and relative files are  searched and data is seeked again from relative offsets
stored in tentaive file and verified.
the function yeilds mismatch.txt file & returns the no of mismatches found as output. 

=over

=item Arguments

B<srcFolder> => I<mounted path to src directory>

I<Required.>
Path to a mounted volume of files for integrity check

B<dstFolder> => I<mounted path to destination directory>

I<Required.>
Path to a mounted volume of files mirrored from source for integrity check

=item Examples

=back

     #$obj->integrity_check(srcFolder=>'/u/shail/FirstLib',dstFolder=>'/u/shail/SecondLib');

=cut



sub integrity_check($,%) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self, %args) = @_;
    my ($srcFolder,$dstFolder) = ('','');
    $srcFolder = exists($args{srcFolder}) ? $args{srcFolder} : croak("integrity_check requires source folder parameter");
    $dstFolder = exists($args{dstFolder}) ? $args{dstFolder} : croak("integrity_check requires destination folder parameter");
    my $startTime = &_timestamp();
    &_create_dataintegrity_metadata($srcFolder);
    my $retVal = &_verify_dataintegrity_metadata( $dstFolder, $srcFolder);
    my $endTime = &_timestamp();
    $retVal = $retVal . "Start Time " . $startTime . "\nEnd Time " . $endTime;
    return $retVal . "\n";
}
1;
__END__
