# $Id: //depot/prod/test/Rsierranevada/lib/NOTE/FcMetrocluster.pm#8 $

## @summary   NOTE Fabric/Stretch MetroCluster class
## @status    Public
## @pod       here
## @author    jawwad@netapp.com

package NOTE::FcMetrocluster;
our @ISA = qw(NOTE);
use NOTE;
use strict;
use Data::Dumper;
use Aggregate qw(aggr_scrub);
use NOTE::Registry;
use NOTE::CFO;
use NOTE::FcSwitch;
use Carp qw(croak);
use cluster;
use boot;
use Tharn qw();
use Net::Ping qw();
use Date::Manip qw();
use NOTE::Aggregate;
use filer qw(filer_sysstat_get);
use LogReader;
use Network::Utils qw(get_ifconfig_info);
use CLI::FileIO qw(rdfile wrfile);
use Params::Validate qw( validate validation_options SCALAR UNDEF BOOLEAN HASHREF );
use SayWrap qw(sayEOF);
use Tharn ;
use Utils qw( make_conn);
use NOTE::Vfiler ;
use dp::fmc::FcMetroclusterUtils;

#use San::SanFiler7M; 
##burt389776
#  Author: secrest  Date: Tue Dec 01 10:24:04 PST 2009---------------
#In the change, on line 24 of lib/NOTE/FcMetrocluster.pm:
#
#  use San::SanFiler7M;
#
#was added.  If the function added in the change doesn't depend on this
#library, can it be removed?  lib/San/SanFiler7M.pm pulls in
#San::SanFiler::Volume on line 31.  lib/San/SanFiler/Volume.pm lives in
#package Volume and manipulates @ISA, thereby breaking the Exporter of
#lib/Volume.pm.


BEGIN {
    $Carp::Verbose = 1;
    $Data::Dumper::Deepcopy = 1;
    $Data::Dumper::Indent = 1;
}

# hash populated by new for FMC
#$VAR1 = {
#    'partner_filer' => {
#           'switch2' => 'mcsw12',
#           'switch1' => 'mcsw10',
#           'name' => 'mc-u31'
#           },
#    'me_filer' => {
#           'switch2' => 'mcsw11',
#           'switch1' => 'mcsw9',
#           'name' => 'mc-u29'
#       },
#    'SMC_CONFIGURATION' => 0,
#    'IB_HA_INTERCONNECT' = 0
#};

# hash populated by new for SMC
#$VAR1 = {
#    'partner_filer' => {
#           'initiator_ports' => [0c,0d,3a,3b],
#           'fcvi_ports' => [2a,2b],
#           'name' => 'sti3240mc-001'
#           },
#    'me_filer' => {
#           'initiator_ports' => [0c,0d,3a,3b],
#           'fcvi_ports' => [2a,2b],
#           'name' => 'sti3240mc-002'
#       },
#    'SMC_CONFIGURATION' => 1,
#    'IB_HA_INTERCONNECT' = 0,
#};
=head1 NAME

NOTE::FcMetrocluster

=head1 DESCRIPTION

This is a NOTE class for Fabric Metrocluster

=head1 METHODS
=cut

#discover expects a Me Filer  object as an argument, and runs sysconfig -va on both filers to get switch info
# Example :  my ($data,$pair_Partner) = $class->discover($pair_ME);

#=head2 ::discover

#Discovers both filers switches (for FMC) or HBA ports (for SMC) by running sysconfig -va on them, and puts them into a hash. 
#Following is the structure of the hash populated by discover():

# hash populated by new for FMC
#$VAR1 = {
#    'partner_filer' => {
#           'switch2' => 'mcsw12',
#           'switch1' => 'mcsw10',
#           'name' => 'mc-u31',
#            'sysid' => 0118043871
#           },
#    'me_filer' => {
#           'switch2' => 'mcsw11',
#           'switch1' => 'mcsw9',
#           'name' => 'mc-u29',
#            'sysid' => 0118048030
#       },
#    'SMC_CONFIGURATION' => 0,
#    'IB_HA_INTERCONNECT' = 0
#};

# hash populated by new for SMC
#$VAR1 = {
#    'partner_filer' => {
#           'initiator_ports' => [0c,0d,3a,3b],
#           'fcvi_ports' => [2a,2b],
#           'name' => 'sti3240mc-002'
#           'switch2' => 'sti3240mc-002',       #Dummy value
#           'switch1' => 'SMC_CONFIGURATION',       #Dummy value
#           },
#    'me_filer' => {
#           'initiator_ports' => [0c,0d,3a,3b],
#           'fcvi_ports' => [2a,2b],
#           'name' => 'sti3240mc-001'
#           'switch2' => 'sti3240mc-001',       #Dummy value
#           'switch1' => 'SMC_CONFIGURATION',       #Dummy value
#       },
#    'SMC_CONFIGURATION' => 1,
#    'IB_HA_INTERCONNECT' = 0
#};

#The return_ value is the hash reference of the populated structure, and both filers object.

#=over

#=item Arguments

#  Required: Filer object

#This argument is required in order for discover to connect to the filer and pull the required information.

#=item Example

#=back

#my ($mc,$pair_Partner) = $NOTE::FcMetrocluster->discover($pair_ME); 

#In this example the return value is the hash reference and parter filers conn object respectively.

#=cut

#Author: jawwad@natpp.com
#status: private

sub discover{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,$pair_ME,%args) =  @_;
    my $Metro_data ={};
    $Metro_data->{SMC_CONFIGURATION} = 1;
    $Metro_data->{IB_HA_INTERCONNECT} = 0;

    $Metro_data->{me_filer}->{name} = $pair_ME->host;                  #populating the me_filer name 

    my $sysconfig_me = $pair_ME->say("sysconfig -va");
    
    #pulling partner id by running sysconfig -va
    foreach(split(/\n/,$sysconfig_me)) {
	    if(/System ID:\s+(\S+)\s+\(\S+\);\s+partner ID:\s+(\S+)\s+\((\S+)\)/ ) {
$Metro_data->{partner_filer}->{name} = $3;
$Metro_data->{me_filer}->{sysid} = $1;
$Metro_data->{partner_filer}->{sysid} = $2;
}
#	$Metro_data->{partner_filer}->{name} = $1 if( /partner ID:\s+\S+\s+\((\S+)\)/ );
    }
    my $pair_Partner = NOTE::Filer->connect(host => $Metro_data->{partner_filer}->{name},%args);
    my $sysconfig_partner = $pair_Partner->say("sysconfig -va");

    my @slots_me = split( /slot/,$sysconfig_me);
    my @slots_partner = split( /slot/,$sysconfig_partner);

    ###### Assuming a FMC config first, let's pull filers' switches detail ######
    #pulling me_filer's switches    
    my $i=1;
    foreach(@slots_me) {
	my $switch ;
	if( /(\w+)\s+Host Adapter.*FC Node Name:\s+(\S+).*Switch Port\s*:\s+(\S+):(\d+)/si ) {
	    $Metro_data->{me_filer}->{'switch'.$i}=$3; 
	    $i++ if ($3 ne undef);  #checking if values are populated in the $3 container
	    $Metro_data->{SMC_CONFIGURATION} = 0;
	}
	last if ($i>2);
    }
    
    #pulling partner_filer's switches    
    $i=1;
    foreach(@slots_partner) {
	my $switch ;
	if( /(\w+)\s+Host Adapter.*FC Node Name:\s+(\S+).*Switch Port\s*:\s+(\S+):(\d+)/si ) {
	    $Metro_data->{partner_filer}->{'switch'.$i}=$3; 
	    $i++ if ($3 ne undef);  #checking if values are populated in the $3 container
	    $Metro_data->{SMC_CONFIGURATION} = 0;
	}
	last if ($i>2);
    }
    
    ###### If its a SMC Configuration, pull filers' initiators & FCVI ports detail ######
    if ($Metro_data->{SMC_CONFIGURATION}){ #looks like a SMC configuration!
        #pulling local FC initiator & FCVI port information
        my (@initiator_ports_me, @fcvi_ports_me);
        foreach (@slots_me) {
            if( $_ =~ /(\w+):\s+(FC|Fibre Channel)\s+Host Adapter\s+(\w+)\s+.*L-port/si ) {
               push(@initiator_ports_me,$3);
            }
            if( $_ =~ /(\w+):\s+FCVI Host Adapter\s+(\w+)\s+.*N-port/si ) {
                push(@fcvi_ports_me,$2);
            }
        }
        $Metro_data->{me_filer}->{initiator_ports} = \@initiator_ports_me;
        $Metro_data->{me_filer}->{fcvi_ports} = \@fcvi_ports_me;

        #pulling partner FC initiator & FCVI ports information
        my (@initiator_ports_partner, @fcvi_ports_partner);
        foreach (@slots_partner) {
            if( $_ =~ /(\w+):\s+(FC|Fibre Channel)\s+Host Adapter\s+(\w+)\s+.*L-port/si ) {
                push(@initiator_ports_partner,$3);
            }
            if( $_ =~ /(\w+):\s+FCVI Host Adapter\s+(\w+)\s+.*N-port/si ) {
                push(@fcvi_ports_partner,$2);
            }
        }
        $Metro_data->{partner_filer}->{initiator_ports} = \@initiator_ports_partner;
        $Metro_data->{partner_filer}->{fcvi_ports} = \@fcvi_ports_partner;  
        
        #Workaround for existing/old FMC scripts which call NOTE::FcSwitch->connect for
        #creating switch objects just after creating MetroCluster 
        #object. Adding the filer name as a dummy value in switch name parameter
        #so that when "connect" subroutine is called from Fcswitch.pm
        #Fcswitch can check & return gracefully. 
        #Note that this is a workaround only for existing FMC scripts to work in SMC too.
        
        $Metro_data->{me_filer}->{'switch1'} = $Metro_data->{me_filer}->{name};
        $Metro_data->{me_filer}->{'switch2'} = 'SMC_CONFIGURATION';
        $Metro_data->{partner_filer}->{'switch1'} = $Metro_data->{partner_filer}->{name};
        $Metro_data->{partner_filer}->{'switch2'} = 'SMC_CONFIGURATION';
        
        ###Let's handle SMC with Infiniband based interconnects
        if($sysconfig_me !~ /FCVI/i){
            #No FCVI found! Lets check for infiniband
            my $cf_monitor_all_me =  $pair_ME->say("priv set -q diag; cf monitor all"); 
            if ($cf_monitor_all_me =~ /HA Interconnect Device is Infiniband/i){
                $Metro_data->{IB_HA_INTERCONNECT} = 1;
            }
            elsif ($sysconfig_me =~ /Cluster Interconnect Port \d+:\w+copper/i ){
                $Metro_data->{IB_HA_INTERCONNECT} = 1;
            }
            else{                
                $Metro_data->{IB_HA_INTERCONNECT} = 0;
            }
        }
              
    }
	#for SN.0 onwards set the option for minutely scheule for asis for existing scripts of older versions to work
	if ($pair_ME->wafl_version >= Filer::Version::WV_SIERRANEVADA_MIN ){
		$pair_ME->say("priv set -q diag; options sis.minutely_schedule on");
		$pair_Partner->say("priv set -q diag; options sis.minutely_schedule on");
	}
    #Workaround for login prompt issues with Console connection on filer boot
    Tharn::host($pair_ME->host); 
    Tharn::connect('#console', -login);
    Tharn::host($pair_Partner->host);
    Tharn::connect('#console', -login);

    return ($Metro_data,$pair_Partner);
};

=head2 ::new

It is a constructor for the library which calls discover() to populate the hash. It returns a hash reference containing the following structure:

Hash structure for FMC:
    $VAR1 = {
    partner => $partner_filer_object,
    'partner_fabrics' => {
        'switch2' => 'mcsw12',
        'switch1' => 'mcsw10',
        'name' => 'mc-u31'
        },
     me => $me_filer_object,
         'me_fabrics' => {
         'switch2' => 'mcsw11',
         'switch1' => 'mcsw9',
         'name' => 'mc-u29'
         },
     'SMC_CONFIGURATION' => 0,
     'IB_HA_INTERCONNECT' = 0,
    };

Hash structure for SMC:
    $VAR1 = {
    partner => $partner_filer_object,
    'partner_fabrics' => {
           'initiator_ports' => [0c,0d,3a,3b],
           'fcvi_ports' => [2a,2b],
           'name' => 'sti3240mc-002'
           'switch1' => 'sti3240mc-002',        #Dummy value
           'switch2' => 'SMC_CONFIGURATION',        #Dummy value
           },
     me => $me_filer_object,
         'me_fabrics' => {
           'initiator_ports' => [0c,0d,3a,3b],
           'fcvi_ports' => [2a,2b],
           'name' => 'sti3240mc-001'
           'switch1' => 'sti3240mc-001',        #Dummy value
           'switch2' => 'SMC_CONFIGURATION',        #Dummy value
       },
    'SMC_CONFIGURATION' => 1,
    'IB_HA_INTERCONNECT' = 0,
};
    

=over

=item Arguments

  Required: Me Filer name e.g $FILER=mc-u29
  Required: mode

=item Example

=back

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);

Filer value can be of object type as well.

How to Extract from the FMC hash:
    
    $me_filer_name = $mc->{me_fabrics}->{name};
    $me_switch1 =  $mc->{me_fabrics}->{switch1};
    
    $me_switch2 = $mc->{me_fabrics}->{switch2};
    $partner_filer_name = $mc->{partner_fabrics}->{name};
    $partner_switch1 =  $mc->{partner_fabrics}->{switch1};
    $parter_switch2 = $mc->{partner_fabrics}->{switch2};

How to Extract from the SMC hash:
    
    $me_filer_name = $mc->{me_fabrics}->{name};
    @me_initiator_ports =  @{$mc->{me_fabrics}->{initiator_ports}};
    @me_fcvi_ports =  @{$mc->{me_fabrics}->{fcvi_ports}};
    $partner_filer_name = $mc->{partner_fabrics}->{name};
    @partner_initiator_ports =  @{$mc->{partner_fabrics}->{initiator_ports}};
    @partner_fcvi_ports =  @{$mc->{partner_fabrics}->{fcvi_ports}};
    $is_smc = $mc->{SMC_CONFIGURATION};
    $is_IB_based_smc = $mc->{IB_HA_INTERCONNECT};
    
Calling me filer object

    $mc->me->version;
    $mc->partner->version;

=cut

sub new{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %args) = @_;    
    if (ref $class) {
	%args = $class;
	$class = 'FcMetrocluster';
    }

    my $self = {};
    $self->{_owner}    = $$;
    my $pair_ME;
    $Carp::Verbose = 1;
    my $ME_FILERA = delete $args{Filer};                          #extracting filerA name
    if(ref ($ME_FILERA) eq "NOTE::Filer") {
	$pair_ME = $ME_FILERA;
    }
    else {
	$pair_ME = NOTE::Filer->connect(host => $ME_FILERA,%args);
    }
    my ($data,$pair_Partner) = $class->discover($pair_ME,%args);                       #calling private method to get switch names    
    $self = {
	    _me => $pair_ME,
	    me_fabrics => $data->{me_filer},
	    _partner => $pair_Partner,
	    partner_fabrics => $data->{partner_filer},
	    SMC_CONFIGURATION => $data->{SMC_CONFIGURATION},
	    IB_HA_INTERCONNECT => $data->{IB_HA_INTERCONNECT}
	    };
    

    bless $self,$class;
    NOTE::reference( $self );

    return ($self);
}

=head2 ::me

Returns the name of the <filer> passed to CFO::new.

=over

=item Examples

   $mc->me;

Will return the filer object for "ME", the filer used with the CFO::new method.

=back

=cut 

sub me {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return (shift)->{_me}; 
}

=head2 ::partner

Returns the name of the <filer> passed to CFO::new.

=over

=item Examples

   $mc->partner;

Will return the filer object for "partner", the filer used with the CFO::new method.

=back

=cut 

sub partner {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return (shift)->{_partner}; 
}


=head2 ::is_fmc

Returns 1 if testbed is a Fabric MetroCluster configuration (and not a Stretch MC configuration).

=over

=item Examples

   $mc->is_fmc;

Will return 1 if testbed is Fabric MetroCluster configuration else returns 0.

=back

=cut 

sub is_fmc {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    if ((shift)->{SMC_CONFIGURATION} == 0){
        return 1;
    }
    else{
        return 0;
    }
}

=head2 ::is_smc

Returns 1 if testbed is a Stretch MetroCluster configuration (and not a Fabric MC configuration).

=over

=item Examples

   $mc->is_smc;

Will return 1 if testbed is Stretch MetroCluster configuration else returns 0.

=back

=cut 

sub is_smc {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return (shift)->{SMC_CONFIGURATION};
}  


=head2 ::is_infiniband_interconnect

Returns 1 if testbed is a infiniband based Stretch MetroCluster configuration.

=over

=item Examples

   $mc->is_infiniband_interconnect;

Will return 1 if testbed is IB based Stretch MetroCluster configuration else returns 0.

=back

=cut 

sub is_infiniband_interconnect {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    return (shift)->{IB_HA_INTERCONNECT};
}  

  
=head2 ::cfod_takeover

simple CFOD takeover which initiate takeover command. 

=over

=item Arguments

  Required: me=>1 or partner=>1
  Optional: command string e.g 'cf forcetakeover -f -d'

=item Example

=back

 $mc->cfod_takeover(me=>1);
 in this case $mc->me filer will takeover the victim filer

 $mc->cfod_takeover(partner=>1);
 in this case $mc->parnter filer will takeover the victim filer
 see /usr/software/test/agnostic/dp/fmc/Basic_CFOD.thpl for usage.

=cut

#Author: jawwad@netapp.com
#cf: forcetakeover -d seems unnecessary, please try "cf forcetakeover [-f]" instead
sub cfod_takeover {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
                   
    my ($class,%options) = @_;
    my (@output,$return_val);
    $return_val = undef;
    $options{cmd} = "cf forcetakeover -f -d";
    if(exists($options{me})) {
	@output = $class->me->say($options{cmd});
	if($output[0] =~ /cf: forcetakeover -d initiated by operator/i) {
	  NOTE::logcomment("cf: forcetakeover -d initiated by operator\n");
	    $return_val = 1;
	}
	else {
	    $return_val = 0;
	}
    }
    elsif(exists($options{partner})) {
	@output = $class->partner->say($options{cmd});
      	if($output[0] =~ /cf: forcetakeover -d initiated by operator/i) {
	  NOTE::logcomment("cf: forcetakeover -d initiated by operator\n");
	    $return_val = 1;
	}
	else {
	    $return_val = 0;
	}
    }
    else {
	$return_val = 0;
	croak "Please provide with the required parameters\n e.g. : cfod_takeover(me=>1)";
    }
    return $return_val;
}

=head2 ::cfod_giveback

simple CFOD giveback which initiate giveback command. 
Note: This function can be used to do giveback for negotiated CFOs also.

=over

=item Arguments


  Required: conn=>$conn_type (console)
  optional: cmd=>'cf giveback -f'

=item Example

=back

    $filer = NOTE::Filer->connect(host=>$filer,mode=>'#console')
    $mc->cfod_giveback(conn=>$filer,cmd=>'cf giveback -f');
     see /usr/software/test/agnostic/dp/fmc/Basic_CFOD.thpl for usage.

=cut

sub cfod_giveback {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $con_obj = $options{conn};
    my (@output,$temp);

    if(!exists($options{cmd})){
	$options{cmd} = "cf forcegiveback -f";
    }
    elsif(exists($options{cmd})) {
        if($options{cmd} eq "cf giveback -f" || $options{cmd} eq "cf forcegiveback -f" || $options{cmd} eq "cf forcegiveback") {
	  NOTE::logcomment("Fine let us do the giveback\n");
	}
    }
    else {
	croak("$options{cmd} is not a valid command for giveback");
    }
    @output =   Tharn::say($con_obj->conn,$options{cmd},'please make sure you have rejoined your aggregates before giveback','givebackStarted:notice]: Failover monitor: giveback started','givebackCancelled:warning]: Failover monitor: giveback cancelled','cf: not in takeover state!',-noprompt,-timeout,20);
    if($output[0] =~ /please make sure you have rejoined your aggregates before giveback/i) {
      Tharn::say($con_obj->conn,'y',-noprompt,-timeout,180);
    }
    if($output[0] =~ /givebackCancelled:warning]: Failover monitor: giveback cancelled/){
       croak("FAIL: Giveback was cancelled by Failover Monitor!\n Giveback Command Output: $output[1]");     
    }
    if($output[0] =~ /cf: not in takeover state!/){
        NOTE::logcomment("Filer is not in TAKEOVER State!\n");
    }
    
}

=head2 ::mirror_out_of_date_aggr

Mirrors victim aggregates from surviving nodes aggregate, Returns 1 if aggregates start to resync or done mirroring, 0 otherwise.
It also expects the filer to be in a partner state where the aggr mirrors are broken or in a 'aggr(1)' form.

=over

=item Arguments

  Required: me=>1|Partner=>1
  Required: array of aggregates from the takeover(partner) mode

=item Example

=back 

  NOTE::FcMetrocluster->mirror_out_of_date_aggr($conn_object,@aggr_present);
  see /usr/software/test/agnostic/dp/fmc/Basic_CFOD.thpl for usage.

=cut

#Author: jawwad@netapp.com
    
sub mirror_out_of_date_aggr {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($self,$pair_object,@aggr_present) = @_;
    my $iter;
    my $return_value=0;
    foreach $iter (@aggr_present) {
	if($iter =~ /(\w+)(\(\d+\))/){
	    my $one = $1;
	    my $two = $2;
	    my $aggr_att = NOTE::Aggregate->attach(Filer=>$pair_object,name=>$iter);
	    my $status_obj = $aggr_att->status;
	    my $type = $status_obj->type();
	    my $state = $status_obj->state();
	    my $status = $status_obj->status();
	    my %options = $status_obj->options();
	    #if status is out-of-date mirror is broken lets go mirror it
	    if($status =~ /out-of-date/i){         
		my $aggr_obj;
		# further check if the survivor aggr is online or not ; if not, then make it online
		$aggr_obj = NOTE::Aggregate->attach(Filer=>$pair_object,name=> $one);
		if(($aggr_obj->status)->state() =~ /failed/i) {
		    $aggr_obj->online(force=>1);
		}
		sleep(5);
		#now everything is good and online lets mirror it
		if(($aggr_obj->status)->state() =~ /online/i) {   # $aggr_obj is surviving aggr's object
		    if($status_obj->state() =~ /online/i) {   # $status_obj is broken aggr's object
			croak "Victim Aggregate is online, can't be mirrored please see the log\n";
		    }
		    #getting a state out of the online aggregate
		    #$one is aggr1 part and $two is (1) part of aggr1(1)
		    my $three = $one.$two;
		  NOTE::logcomment("aggr mirror $one -f -v $one$two\n");
		    $pair_object->say("aggr mirror $one -f -v $three"); 
		    sleep(5);
		    #after mirror command checking if started resynching or not
		    if(($aggr_obj->status)->status() =~/resyncing/i) {  #jawwad changed state to status 
		      NOTE::logcomment("\n$one: Resyncing \nLive status: ".($aggr_obj->status)->state());
			$return_value =1;
		    }
		     if(($aggr_obj->status)->status() =~/mirrored/i) {  #jawwad changed from state to status
		      NOTE::logcomment("\n$one: Mirrored \nLive status: ".($aggr_obj->status)->state());
			$return_value =1;
		    }
		}
	    }
	}	
    }
    #concluding check-post
  NOTE::logcomment("Checking to see if everything is mirrored\n");
    #going through the aggrs that we just mirrored to see if anything left broken or cannot be mirrored
    my @aggr_present1 = $pair_object->aggregates;
    foreach $iter (@aggr_present1){
	my $aggr_att = NOTE::Aggregate->attach(Filer=>$pair_object,name=>$iter);
	my $status_obj = $aggr_att->status;
	my $type = $status_obj->type();
	my $state = $status_obj->state();
	my $status = $status_obj->status();
	my %options = $status_obj->options();
	if(($iter =~ /(\w+)(\(\d+\))/) || ($status =~ /out-of-date/i)) {
	    croak "Cannot be mirroed\n";
	    $return_value =0;
	}
    }
  NOTE::logcomment("No broken mirrors found, Good to go!\n");

  
    $return_value;
}

=head2 ::wait_state_for_broken_aggrs

waits till it finds the unmirrored aggregate. Used for performing CFOD before mirror the aggregates.

=over

=item Arguments

  Required: timeout 

  Required: mc=>1 or partner=>1
    
=item Example

=back 

  NOTE::FcMetrocluster->mirror_out_of_date_aggr(me=>1, timeout=>300);
  see /usr/software/test/agnostic/dp/fmc/Basic_CFOD.thpl for usage.

=cut

sub wait_state_for_broken_aggrs {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $timeout = $options{timeout};
    my ($iter,@aggr_name,$temp);
    my $end_time = $timeout ? Tharn::timeout2time($timeout) : Tharn::timeout2time(-1);
  NOTE::logcomment("waiting to find an aggregate with a broken mirror..");
    while(time() <= $end_time) {
	my $time_remaining = $end_time - time();
	if(exists($options{me})){
	    $temp = 'me';
	}
    elsif(exists($options{partner})){
        $temp = 'partner';
    }
	@aggr_name = $class->$temp->aggregates;
	
      NOTE::logcomment("Timeout remaining: $time_remaining secs");
	foreach $iter (@aggr_name) {
	    if($iter =~ /(\w+)(\(\d+\))/){
	      NOTE::logcomment("aggregate = $iter\n");
		my $aggr_att = NOTE::Aggregate->attach(Filer=>$class->$temp,name=>$iter);
		my $status_obj = $aggr_att->status;
		my $type = $status_obj->type();
		    my $state = $status_obj->state();
		my $status = $status_obj->status();
		my %options = $status_obj->options();
		  NOTE::logcomment("status = $status\n");
		#if status is out-of-date mirror is broken lets go mirror it
		if($status =~ /out-of-date/i) {
		  NOTE::logcomment("Found broken mirrors with status: $status \n first broken aggr found was : $iter\n");
		    return;
		}
	    }
	}
	sleep(5);
    }
    die "Timed out waiting for finding the broken mirrors\n";
}

=head2 ::wait_for_giveback

Runs cf status on the given filer and looks for "is ready for giveback" string

=over

=item Arguments

  Required: timeout 

  Required: mc=>1 or partner=>1
    
=item Example

=back 

    $mc->wait_for_giveback(me=>1);
    see /usr/software/test/agnostic/dp/fmc/Basic_CFOD.thpl for usage.

=cut

sub wait_for_giveback {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %options) = @_;
    my $temp;
    my $timeout = 3000;    #setting default timeout to 3000seconds
    if(exists($options{timeout})) {
	$timeout = $options{timeout};
    }
    my $end_time = $timeout ? Tharn::timeout2time($timeout) : Tharn::timeout2time(-1);
    while(time() <= $end_time) {
	if(exists($options{me})){
	    $temp = 'me';
	}
	elsif(exists($options{partner})) {
	    $temp = 'partner';
	}
	my $output = $class->cf_status($temp=>1);
	if($output =~ /is ready for giveback/) {
	    return ;
	}
	sleep(10);
    }
    die "Timed out waiting for giveback\n";
}

=cut


=head2 ::cf_status_wait

This method calls cf_ic_status_check and wait till cf gets enable

=over

=item Arguments

B<timeout> => I<timeout in seconds>

I<Optional>
Timeout in seconds. default is set to 300 seconds

=item Return Value

1 on success, 0 otherwise

=item Examples

=back

        $mc->cf_status_wait ;

=cut

sub cf_status_wait {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


   my ( $class, %options ) = @_;
   my $timeout;
   my $status = 0;

   if ( exists( $options{timeout} ) ) {
      $timeout = $options{timeout};
   } else {
      $timeout = 300;
   }
   my $end_time =
     $timeout ? Tharn::timeout2time($timeout) : Tharn::timeout2time(-1);

   while ( time() < $end_time ) {
      my $time_remaining = $end_time - time();
      NOTE::logcomment("Timeout remaining : $time_remaining secs\n");
      eval { $class->cf_ic_status_check( me => 1 ); };
      if ($@) {
         NOTE::logcomment("Checking status again..\n");
      } else {
         NOTE::logcomment("Cluster is enabled\n");
         $status = 1;
         last;
      }
      sleep(15);
   }

   if ( !$status ) {
      croak "Unable to recover from unsysnchronized state\n";
   } else {
      return 1;
   }

}



=head2 ::redundancy_check_old  <deprecated>

Issues 'storage show disk -p' and look for redundant path. Returns 1 if redundancy is present, 0 otherwise.

=over

=item Arguments

  Required: me or partner options

=item Example

=back

    my $redun = NOTE::FcMetrocluster->redundancy_check(me=>1);
 if me=>1 then this will run on me filer, else if partner=>1, it will run on partner filer

=cut

#Author: jawwad@netapp.com

sub redundancy_check_old {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($portA, $portB,$redundant,$primary_disk,$secondary_disk);
    $redundant = undef ;
    if(exists($options{me})) {
	foreach(split('[\n\r]',$class->me->say("storage show disk -p"))){
	    s/\s+$//g;
	    next if($_ =~ /PRIMARY .*/);   #take care of the first line
	    if(/\s*(\S+)\s*([AB])\s*(.*)/i){  # $1 contains the primary disk , $2 contains the primary port $3 contains the rest
		if($3 =~ /\s+[AB]{1}\s+/i) { #all i am saying here is if $3 contains any port that means the disk is present
		    $redundant = 1; #redundancy present
		}
		else {
		$redundant = 0; #redundancy  not present
	      NOTE::logcomment("Crashed at $1 disk\n");
		return;
	    }
	}
    }
    return $redundant;
    }
    elsif (exists($options{partner})) {
	foreach(split('[\n\r]',$class->partner->say("storage show disk -p"))){
	    s/\s+$//g;
	    next if($_ =~ /PRIMARY .*/);
	    if(/\s*(\S+)\s*([AB])(.*)/i){
		if($3 =~ /\s+[AB]{1}\s+/i) {
		    $redundant = 1; #redundancy present
		}
		else {
		    $redundant = 0; #redundancy  not present
		  NOTE::logcomment("Crashed at $1 disk\n");
		    return;
		}
	    }
	}
	return $redundant;
    }
}
=head2 :: check_nfs_traffic

This method calls systat on the filer to check if the system is loaded properly and nfs traffic is flowing
Fails if avg_nfs_ops is less than 10 .
=over

=item Arguments
Required: me=>1 or partner=>1



=item Example

=back
 $mc->check_nfs_traffic(me=>1);
 $mc->check_nfs_traffic(partner=>1);

=cut

sub check_nfs_traffic {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options)=@_;
    my $filer_string;
    if(exists($options{partner}))
    {
        $filer_string='partner';
    }
    else { $filer_string ='me'; }
    my $filer=$class->$filer_string;
    $filer->say("uptime", 'timeout',180);
    my %sysstat = filer_sysstat_get(
         $filer->conn,
         {
            'FILER_SYSSTAT_COUNT'    => 10,
            'FILER_SYSSTAT_INTERVAL' => 2,
         }
      );
      my $avg_nfs_ops = $sysstat{'NFS'}{'Avg'};

      if ( $avg_nfs_ops < 10 ) {

         NOTE::logresult( "FAIL",
            "NFS traffic is not running after traffic duration \n" );

      } else {
         NOTE::logcomment("NFS traffic AVG : $avg_nfs_ops \n");
      }

      return 1;
}



=head2 ::Check_Wafl_Scrub

Runs wafl_nvlog_chk and aggr scrub on given filer and check it should not report any error

=over

=item Arguments

  Required:  me=>1|partner=>1, vol_name

=item Example

=back
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->Check_Wafl_Scrub(partner=>1, vol=>$vol);
    $mc->Check_Wafl_Scrub(vol=>$vol);

=cut

sub Check_Wafl_Scrub {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

   my ($class,%args) = @_;
   my $filer_string;
   my $vol=$args{vol};
   if(exists($args{partner})) {
       $filer_string='partner';}
   else {$filer_string='me'}
   $class->$filer_string->reconnect(mode=>'#console');
   my $text;
   eval { $text = $class->$filer_string->say("version"); };
    if( $text =~ /<\d+d/ ) {
        NOTE::logcomment("Starting wafl_nvlog_chk on $filer_string \n");
        my $str= $class->$filer_string->say("priv set test; wafl_nvlog_chk");
        if($str =~/local\s+wafl\s+log/ && $str =~/partner\s+wafl\s+log/)
        {
            NOTE::logcomment("Verified local and partner wafl logs\n");
        }
         else {
             NOTE::logcomment("Unable to verify local and partner logs\n");
         }
     }
     else
     {
            NOTE::logcomment("Unable to start wafl_nvlog_chk on Non debug build\n");
     }
    $class->$filer_string->reconnect(mode=>'#rsh');
    NOTE::logcomment("Starting aggr scrub on partner\n");
    my $aggr = $vol->Aggregate;
    my $aggr_name = $aggr->name;
    aggr_scrub($class->$filer_string->conn,$aggr_name,'stop');
	sleep 2;
    my $status = aggr_scrub($class->$filer_string->conn,$aggr_name,'start');
    if(!$status) {
    croak("Unable to start aggr_scrub on aggregate $aggr_name \n");
    }
         else {
        NOTE::logcomment("Started aggr_scrub on aggregate $aggr_name sucessfully");
    }

}




=head2 ::redundancy_check

Issues 'storage show disk -p' and look for redundant path. Returns 1 if redundancy is present, 0 otherwise.

=over

=item Arguments

  Required: me or partner options

=item Example

=back

    my $redun = NOTE::FcMetrocluster->redundancy_check_old(me=>1);
 if me=>1 then this will run on me filer, else if partner=>1, it will run on partner filer

=cut

sub redundancy_check {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    #my $self = shift @_;
    #my $filer = shift @_;
    my ($portA, $portB,$redundant);
    $redundant = undef ;
    if(exists($options{me})) {
	foreach(split('[\n\r]',$class->me->say("storage show disk -p"))){
	    s/\s+$//g;
        if(/PRIMARY\s+PORT\s+SECONDARY\s+PORT\s+SHELF\s+BAY/ || /----/){
            next;
        }
	    if(/\s*(.*)\s+([AB])\s*(.*)\s+([AB]).*/){
		  $portA = $2;  #$portA = $2
		  $portB = $4   #$portB = $4
	    }
	    #old comparison: if(($portA eq $portB) && ($portA ne undef) && ($portB ne undef))
	    if(($portA eq $portB) || ($portA eq undef) || ($portB eq undef)){
		  $redundant = 0;      # redundancy not present
	      NOTE::logcomment("Line with first occurance of non-redundant path: \"$_\"\n");
		  return $redundant;
	    }
	    if (($portA ne $portB) && ($portA ne undef) && ($portB ne undef)) {
		  $redundant = 1;      #redundancy present
	    }
	    #set $portA & $portB to undef for next loop iteration
	    $portA = undef;
	    $portB = undef;
	}
	return $redundant;
    }
    elsif (exists($options{partner})) {
	foreach(split('[\n\r]',$class->partner->say("storage show disk -p"))){
	    s/\s+$//g;
	    if(/PRIMARY    PORT  SECONDARY  PORT SHELF BAY/ || /----------/){
	        next;
	    }
	    if(/\s*(.*)\s+([AB])\s*(.*)\s+([AB]).*/){
		$portA = $2;
		$portB = $4; 
	    }
	    if(($portA eq $portB) && ($portA ne undef) && ($portB ne undef)){
		  $redundant = 0;      # redundancy not present
	      NOTE::logcomment("Line with first occurance of non-redundant path: \"$_\"\n");
		  return $redundant;
	    }
	    if (($portA ne $portB) && ($portA ne undef) && ($portB ne undef)) {
		  $redundant = 1;      #redundancy present
	    }
        #set $portA & $portB to undef for next loop iteration
        $portA = undef;
        $portB = undef;	    
	}
	return $redundant;
    }
}
	

=head2 ::fsid_check

fsid_check subroutine.Reads the fsid values for all the volumes and aggregates on the filer and returns a hash containing the FSID Value.


=over

=item Arguments

Required: me or partner options


=item Example

=back

my $fsid =  $mc->fsid_check(me =>1);
or
my $fsid =  $mc->fsid_check(partner =>1);
$fsid_volume = $fsid->{'volume'}->{<vol_name>};
$fsid_aggr= $fsid->{'aggr'}->{<aggr_name>};

=cut

sub fsid_check {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

   my ( $class, %options ) = @_;
   my ( $filer_string, $str );
   my ( @volume_list, @vol, $volume );
   my ( $aggregate, @aggr_list );
   our %fsid_table;

   if ( exists( $options{partner} ) ) {
      $filer_string = 'partner';
   } else {
      $filer_string = 'me';
   }
   @aggr_list = $class->$filer_string->Aggregates;

   for ( my $count = 0 ; $count <= $#aggr_list ; $count++ ) {
      $aggregate=$aggr_list[$count]->name;
      $fsid_table{'aggr'}{ $aggregate} = $aggr_list[$count]->fsid;
   }
   @volume_list = $class->$filer_string->Volumes;
   for ( my $count = 0 ; $count <= $#volume_list ; $count++ ) {
      $volume=$volume_list[$count]->name;
      $fsid_table{'volume'}{ $volume} = $volume_list[$count]->fsid;
   }

   return \%fsid_table;

}

	
#mc-sw16:7.35   A    mc-sw15:7.35   B     2    3
#rtpmet-browho-3:3.33  A    rtpmet-browho-4:3.33  B     2    1 


#mc-u34 has taken over mc-u35.
#mc-u35 has been taken over by mc-u34.

=head2 ::cf_status
    
Runs cf status on the filer and returns the output string.

=over

=item Arguments

  Required: me=>1, partner=>1 (me will run on the local filer that you invoked through ntest)

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    my $output = $mc->cf_status(me=>1);

output will contain "mc-u34 has taken over mc-u35" or "mc-u35 has been taken over by mc-u34" or other
strings returned by cf_status in different modes.

=cut

sub cf_status {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %options) = @_;
    my $output;
    if(exists($options{me})) {
	$output = $class->me->say("cf status");
	return $output;
    }
    elsif(exists($options{partner})){
	$output = $class->partner->say("cf status");
	return $output;
    }
}
    
=head2 ::cf_enable
    
Enables cluster on the "me" filer, and makes sure that it is actually enabled by initiating 'cf status' command.

=over

=item Arguments

  Required: None

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->cf_enable(me=>1);
OR
    $pair_ME = NOTE::Filer->connect(host=>$FILERA);
    NOTE::FcMetrocluster->cf_enable;

=cut

#Author : jawwad@netapp.com

sub cf_enable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    if(exists($options{me})) {
        my $output=$class->cf_status( me => 1 );
        if ( $output =~ /(Cluster enabled) | (Controller Failover enabled)/i ) {
         NOTE::logcomment("Cluster is enabled and in Good condition");
      } else {

	my $enable = $class->me->say("cf enable",'-eval_error');
	sleep(4);
	my $status = $class->me->say('cf status');
	if ($status = ~ /(Cluster enabled)|(Controller Failover enabled)/i) {
	  NOTE::logcomment("Cluster Enabled ! \n");
	    return;
	}
        
	else {
	    croak "cf enable returned with an error : $enable\n";
	}
     }
    }
    elsif(exists($options{partner})) {
         my $output=$class->cf_status( partner => 1 );
        if ( $output =~ /(Cluster enabled) | (Controller Failover enabled)/i ) {
         NOTE::logcomment("Cluster is enabled and in Good condition");
      } else {

	my $enable = $class->partner->say("cf enable",'-eval_error');
	sleep(4);
	my $status = $class->partner->say('cf status');
	if ($status = ~ /(Cluster enabled)|(Controller Failover enabled)/i) {
	  NOTE::logcomment("Cluster Enabled ! \n");
	    return;
	}
	else {
	    croak "cf enable returned with an error : $enable\n";
	}
      }
    }
    else {
	croak "No option was provided\n";
    }
}

=head2 ::cf_disable

Disables cluster on the "me" filer, and makes sure that it is actually disabled by initiating 'cf status' command.

=over

=item Arguments

  Required: None

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->cf_disable(me=>1);

=cut

#Author : jawwad@netapp.com.

sub cf_disable	{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $class = shift;
    my $disable = $class->me->say("cf disable",'-eval_error');
    sleep(4);
    my $status = $class->me->say('cf status');
    if ($status =~ /(Cluster disabled)|(Controller Failover disabled)/i) {
      NOTE::logcomment("Cluster Disabled ! \n");
    }
    else {
	croak "cf disable returned with an error : $disable";
    }
}

#Check all the necessary licenses required for MC

=head2 ::check_licenses

Checks the neccesary licenses for MetroCluster on both the filers

=over

=item Arguments

  Required: None

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->check_licenses(me=>1);
OR
    NOTE::FcMetrocluster->check_licenses(partner=>1 or me=>1);

=cut

#author: jawwad@netapp.com
# cf_remote  WBOPNOJ
sub check_licenses {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $filer;
    if(exists($options{me})) {
	$filer = $class->me;
    }
    elsif(exists($options{partner})) {
	$filer = $class->partner;
    }
    my $cf='cf';
    my $cfr='cf_remote';
    my $syn_lic = filer::filer_service_is_licensed($filer->conn,'syncmirror_local');
    my $version =$filer->version;
    if($version<'7'){
        $cf='cluster';
        $cfr='cluster_remote';
    }
    my $cf_lic= filer::filer_service_is_licensed($filer->conn,$cf);
    my $cfr_lic= filer::filer_service_is_licensed($filer->conn,$cfr);

    $filer->license($cf) unless($cf_lic);
    $filer->license($cfr) unless($cfr_lic);
    $cfr_lic = filer::filer_service_is_licensed($filer->conn,$cf);
    $cf_lic = filer::filer_service_is_licensed($filer->conn,$cfr);
    unless($syn_lic){
        $filer->license("syncmirror_local"); 
        sleep(5);
        $filer->reboot;
    }
    
    $syn_lic = filer::filer_service_is_licensed($filer->conn,'syncmirror_local');
    NOTE::logcomment("cfr_lic = $cfr_lic ||  syn_lic = $syn_lic || cf_lic = $cf_lic\n");
    
    if(($cfr_lic) && ($syn_lic) && ($cf_lic)) {
      NOTE::logcomment("cf, cf_remote and syncmirror_local is licensed successfully\n");
    }
    else {
	croak "could not license properly\n";
    }
}

=head2 ::check_filer_status

Checks the status of both the filers using get_cf_state function in cluster.pm
returns two values which contain the state of each filer

=over

=item Arguments

  Required: None

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    my ($me_state,$p_state) = $mc->check_filer_status;

=cut

#author: jawwad@netapp.com

sub check_filer_status {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $class = shift @_;
    my $stateA = cluster::get_cf_state($class->me->conn);
    my $stateB = cluster::get_cf_state($class->partner->conn);
    return ($stateA,$stateB);   
}

=head2 ::disk_count

Issues disk show command and counts the total number of disks.
Returns the total number of disks.

=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $count = $mc->disk_count(me=>1);

Above will run the failed_disk_count method on 'me' filer

    $count = $mc->disk_count(partner=>1);

Above will run the failed_disk_count method on 'partner' filer

OR    
    $pair_ME = NOTE::Filer->connect(host=>$FILERA,mode=>'#console');
    NOTE::FcMetrocluster->disk_count(me=>1);

=cut

#author: jawwad@netapp.com

sub disk_count {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $i = 0;
    if(exists($options{me})) {
	foreach(split('[\n\r]',$class->me->say("disk show"))) {
	    if($_=~ /pool/i){
		$i++;
	    }
	}
	$i = $i-1;
	return $i;
    }
    elsif(exists($options{partner})) {
	foreach(split('[\n\r]',$class->partner->say("disk show"))) {
	    if($_=~ /pool/i){
		$i++;
	    }
	}
	$i = $i-1;
	return $i;
    }
}

=head2 ::get_fcvi_hba

Runs sysconfig -va and pulls out all the fcvi and hba ports connected to fabric switches on the filer
me=>1 will pull info from the provided filer and vice versa for partner=>1. It returns
a hash which contains all the information. 

Note: This is a FMC specific function. For SMC make use get_fcvi_ports or get_initiator_ports

Following is the structure of the hash returned:
 
        $VAR1 = {
	    'FC' => {
		'0b' => {
		    'mc-sw14' => '2'
		    },
		 '3a' => {
		     'mc-sw13' => '9'
		     },
		  '0a' => {
		     'mc-sw13' => '2'
		     },
		  '3b' => {
		     'mc-sw14' => '9'
		     }
	    },
	    'FCVI' => {
		  '2b' => {
		     'mc-sw14' => '3'
		     },
		  '2a' => {
		     'mc-sw13' => '3'
		     }
	      }
	};

0a,3a,0b etc. are the filer ports, followed by the switch name and the port on the switch.
It also return an array which contains all the filer ports i.e. 0a,3a,0b etc.


=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    ($hba,@filer_ports) = $mc->get_fcvi_hba(me=>1);

Above will run the get_fcvi_hba method on 'me' filer

    ($hba,@filer_ports) = $mc->get_fcvi_hba(partner=>1);

Above will run the get_fcvi_hba method on 'partner' filer

OR    

    NOTE::FcMetrocluster->get_fcvi_hba(me=>1);

An example of printing all the HBA and FCVI ports connected to mc-sw13
    
        foreach my $iter (@filer_ports){
	if($hba->{FC}->{$iter}->{$switch}){
	    my $port = $hba->{FC}->{$iter}->{$switch};
	  NOTE::logcomment("HBA ports on $switch : $port\n");
	}
    }
    
    foreach my $iter (@filer_ports){
	if($hba->{FCVI}->{$iter}->{$switch}){
	    my $port = $hba->{FCVI}->{$iter}->{$switch};
	  NOTE::logcomment("FCVI ports on $switch : $port\n");
	}
    }

=cut

sub get_fcvi_hba {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($hba,@filer_ports,$i);
    $i=0;
    if ($class->is_fmc){
        if (exists($options{me})) {
            my @sl = split(/[\n\r]\s+slot\s+\d+:/,$class->me->say('sysconfig -va'));
            foreach(@sl) {                      # $1 = Fc or FCVI  $2 = port on filer i.e 0a, 0b .. $3 = switch host and $4 = port on switch
                if(/(\w+) Host Adapter\s+(\S+)\s+.*Switch Port\s*:\s+(\S+):(\d+)\-(\d+)/gs) {
                    $hba->{$1}->{$2}->{$3}="fc$4/$5"; #populating for Cisco switch
                    $filer_ports[$i] = $2;  #populating filer ports in an array (this is done so that script writer can easily iterate over the hash
                    $i++;
                }
                elsif(/(\w+) Host Adapter\s+(\S+)\s+.*Switch Port\s*:\s+(\S+):(\d+)/gs) {
                    $hba->{$1}->{$2}->{$3}=$4;
                    $filer_ports[$i] = $2;  #populating filer ports in an array (this is done so that script writer can easily iterate over the hash
                    $i++;
                }
           }
            return ($hba,@filer_ports);
        }        
        elsif(exists($options{partner})) {
            my @sl = split(/[\n\r]\s+slot\s+\d+:/,$class->partner->say('sysconfig -va'));
            foreach(@sl) {
                if(/(\w+) Host Adapter\s+(\S+)\s+.*Switch Port\s*:\s+(\S+):(\d+)\-(\d+)/gs) {
                    $hba->{$1}->{$2}->{$3}="fc$4/$5"; #populating for Cisco switch
                    $filer_ports[$i] = $2;  #populating filer ports in an array (this is done so that script writer can easily iterate over the hash
                    $i++;
                }
                elsif(/(\w+) Host Adapter\s+(\S+)\s+.*Switch Port\s*:\s+(\S+):(\d+)/gs) {
                    $hba->{$1}->{$2}->{$3}=$4;
                    $filer_ports[$i] = $2;
                    $i++;
                }
           }
            return ($hba,@filer_ports);
        }
        else {
            croak "No options were provided please provide me=>1 or partner=>1\n";
        }        
    }
    else{
        croak("get_fcvi_hba() routine is supported for FMC configuration only! Not Suported for SMC");
    }
}

=head2 ::ic_status
    
Runs "ic status" on the given filer and populate all the information into a hash

$VAR1 = { 

 'Link_0' => {
              'state' => 'up'
  },
 'cfo_rv' => {
               'role' => 'dest_rv',
               'state' => 'CONNECTED'
  },
 'cfo_rv2_nic_used' => {
                         'state' => '1'
  },
 'Link_1' => {
              'state' => 'up'
  },
 'cfo_rv1_nic_used' => {
                        'state' => '1'
  },
 'cfo_rv_nic_used' => {
                       'state' => '0'
  },
 'cfo_rv2' => {
               'role' => 'not applicable',
               'state' => 'CONNECTED'
  },
 'cfo_rv1'=>  {
               'role' => 'source_rv',
               'state' => 'CONNECTED'
  }
};

=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    my $hash = $mc->ic_status(me=>1);

=cut

sub ic_status {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($hash,$ic_status_output);
    if(exists($options{me})) {
	$ic_status_output = $class->me->say("priv set -q advanced ; ic status");
	foreach(split('[\n\r]',$ic_status_output)) {
	    $hash->{"Link_$1"}->{state} = $2 if($_ =~ /\s*Link\s*(\d+)\s*:\s*(\w+)/i) ;
	    $hash->{$1}->{state} = $2        if($_ =~ /\s*(cfo_rv[0-9]|cfo_rv).*connection\s*state\s*:\s*(\w+[\s\w]*)/i);
            $hash->{$1}->{role} =  $2        if($_ =~ /\s*(cfo_rv[0-9]|cfo_rv).*connection\s*role\s*:\s*(\w+[\s\w]*)/i);
	    $hash->{"$1_nic_used"}->{state}= $2 if($_ =~ /\s*(cfo_rv[0-9]|cfo_rv).*nic\s*used\s*:\s*(\d+)/i);
	}
	return $hash;
    }
    elsif(exists($options{partner})) {
	$ic_status_output = $class->partner->say("priv set -q advanced ; ic status");
	foreach(split('[\n\r]',$ic_status_output)) {
	    $hash->{"Link_$1"}->{state} = $2 if($_ =~ /\s*Link\s*(\d+)\s*:\s*(\w+)/i) ;
	    $hash->{$1}->{state} = $2        if($_ =~ /\s*(cfo_rv[0-9]|cfo_rv).*connection\s*state\s*:\s*(\w+[\s\w]*)/i);
            $hash->{$1}->{role} =  $2        if($_ =~ /\s*(cfo_rv[0-9]|cfo_rv).*connection\s*role\s*:\s*(\w+[\s\w]*)/i);
	    $hash->{"$1_nic_used"}->{state}= $2 if($_ =~ /\s*(cfo_rv[0-9]|cfo_rv).*nic\s*used\s*:\s*(\d+)/i);
	}
	return $hash;
    }
    else {
	croak "No option was provided please provide me=>1 or partner=>1\n";
    }
}

=head2 ::ic_reset_nic
    
Runs "ic reset nic" on the given filer's nic port (0|1).

=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back
To do ic reset nic on nic port 0 of me filer    
    $mc = NOTE::FcMetrocluster->new(Filer => $FILERA);
    $my $ic_reset_nic = $mc->ic_reset_nic( me => 1, nic => 0);
OR
    $my $ic_reset_nic = $mc->ic_reset_nic( me => 1); 

To do ic reset nic on nic port 1 of me filer    
    $mc = NOTE::FcMetrocluster->new(Filer => $FILERA);
    $my $ic_reset_nic = $mc->ic_reset_nic( me => 1, nic => 1);

=cut

sub ic_reset_nic {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %options) = @_;
    my ($ic_reset_nic, $nic);
    if ( $options{nic} == 1){
        $nic = 1;
    }
    else{
        $nic = 0;
    }
    if(exists($options{me})) {
	$ic_reset_nic = $class->me->say("priv set -q  diag; ic reset nic $nic");
	sleep(20);
	return $ic_reset_nic;
    }
    elsif(exists($options{partner})) {
	$ic_reset_nic = $class->partner->say("priv set -q diag ; ic reset nic $nic");
	sleep(20);
	return $ic_reset_nic;
    }
}
   
=head2 ::check_ood_unsync_log

This method will check for any VI/NIC reset after enabling OOD
    
=over
    
=item Arguments

Required: me=>1 or partner=>1

B<timeout> => I<timeout in seconds>

I<Optional>
Timeout in seconds. default is set to 300 seconds

=item Return Value

1 on success, 0 otherwise

=item Examples

=back

        $mc->check_ood_unsync_log(me=>1) ;
        $mc->check_ood_unsync_log(partner=>1);

=cut

sub check_ood_unsync_log {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


   my ( $class, %options ) = @_;
   my ($timeout,$filer);

   if ( exists( $options{timeout} ) ) {
      $timeout = $options{timeout};
   } else {
      $timeout = 300;
   }
    if(exists($options{me})) {
        $filer = $class->me;
    }
    elsif(exists($options{partner})) {
        $filer = $class->partner;
    }

    NOTE::logcomment("Starting LogReader\n");
    my $log = LogReader->new( $filer->conn );
    $log->start( [ 'messages' ] );

     my $end_time =
        $timeout
        ? Tharn::timeout2time($timeout)
        : Tharn::timeout2time(-1);

    while ( time() < $end_time ) {
         my $time_remaining = $end_time - time();
         NOTE::logcomment("Timeout remaining: $time_remaining secs");
         sleep(60);
      }
    $log->stop;
    $log->logcomment;
    if ( $log->{logs}{messages}->to_string =~ /unsynchronized log/gs ) {
        croak "Unsynchronized logs found\n";
      } else {
         NOTE::logcomment("No NIC/VI reset happend. Good to go\n");
      }
}


=head2 ::ic_ood_status

Runs "ic ood status" on the given filer and populate all the information into a hash.

 $VAR1 = {
  'NIC_1 Statistics' => {
    'OOD Status' => 'Enabled',
    'Coalescing' => 'Enabled',
    'Source' => 'No'
  },
  'NIC_0 Statistics' => {
    'OOD Status' => 'Enabled',
    'Coalescing' => 'Enabled',
    'Source' => 'Yes'
   }
 };

=over

=item Arguments
    
  Required: me=>1|partner=>1

=item Example

=back

 $mc = NOTE::FcMetrocluster->new(Filer =>$FILER);
 my $hash = $mc->ic_ood_status(me=>1);

=cut

sub ic_ood_status {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %options) = @_;
    my ($hash,$ic_ood_output,$temp);
    if(exists($options{me})) {
	$temp = 'me';
    }
    elsif(exists($options{partner})) {
	$temp = 'partner';
    }
    else {
      croak "Please provide the required parameters. e.g me=>|partner=>1\n";
    }
    $ic_ood_output = $class->$temp->say("priv set -q diag;ic ood status");
    my ($nic,$source);
    foreach(split('[\n\r]',$ic_ood_output)) {   #split line by line
	if($_ =~ /\s*NIC\s*(\d+)\s*Statistics\s*(\(Source NIC\))*/i) {  #find nic number and source NIC
	    $nic = $1; #store nic number
	    $source = $2;#store source NIC
	    }
	#populating hash with OOD status Enable or Disabled
	$hash->{"NIC_$nic Statistics"}->{"OOD Status"} = $1 if($_ =~ /\s*OOD\s*Status\s*(\w+)/); 
	#populating hash with Coalescing status Enable or Disabled
	$hash->{"NIC_$nic Statistics"}->{"Coalescing"} = $1 if($_ =~ /\s*Coalescing\s*(\w+)/);
	if($source =~ /Source NIC/){ #if source nic is present
	    #setting the right nic to source
	    $hash->{"NIC_$nic Statistics"}->{"Source"} = 'Yes';
	}
	else {
	    $hash->{"NIC_$nic Statistics"}->{"Source"} = 'No';
	}
    }
    return $hash;
}
 

=head2 ::check_ood_enabled

This method calls ic_ood_status and checks OOD and Colescing is enabled on nic 0 and nic 1.Croaks if it is not enabled

=over

=item Arguments


=item Example

=back

    $mc->check_ood_enabled;
    $mc->check_ood_enabled( partner => 1 );
This will run the ic ood status on filer and make sure OOD and Coalescing is enabled on nic 0 & nic 1, else it will croak. 

=cut

sub check_ood_enabled{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($filer_string, $nic );
    if(exists($options{partner}))
    {
        $filer_string='partner';
    }
    else { $filer_string ='me'; }

      my $hash = $class->ic_ood_status( $filer_string => 1 );
      for ( $nic = 0 ; $nic <= 1 ; $nic++ ) {
         if ( $hash->{"NIC_$nic Statistics"}->{"OOD Status"} =~ /Enabled/i ) {
            NOTE::logcomment("Nic $nic OOD status : Enabled\n");
         } else {
            croak "Could not Enable OOD on nic $nic: $@ \n";
         }
         if ( $hash->{"NIC_$nic Statistics"}->{"Coalescing"} =~ /Enabled/i ) {
            NOTE::logcomment("Nic $nic Coalescing  : Enabled\n");
         } else {
            croak "Could not Enable Coalescing  on nic $nic: $@ \n";
         }
      }
   }





=head2 ::ic_ood_on

Runs "ic ood on <nic_number>" on the given filer and runs "ic ood status" to make sure 
that OOD on the given nic is Enabled.

=over

=item Arguments

  Required: me=>1|partner=>1, nic=>0|1 (nic number)

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_ood_on(me=>1,nic=>0);

=cut
   
sub ic_ood_on {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
    my ($temp,$nic);
    if(exists($options{me})) {
	$temp = 'me';
    }
    elsif(exists($options{partner})) {
	$temp = 'partner';
    }
    else {
	croak "Please provide the required parameters. e.g me=>|partner=>1\n";
    }
    $nic = $options{nic} if(exists($options{nic}));
    $class->$temp->say("priv set -q diag; ic ood_support on");
    sleep 1;
    $class->$temp->say("priv set -q diag; ic ood on $nic");
    sleep(40); # increased timeout from 20 seconds to 40 secods for burt#582824
    my $hash = $class->ic_ood_status($temp=>1);
    if($hash->{"NIC_$nic Statistics"}->{"OOD Status"} =~ /Enabled/i){
      NOTE::logcomment("Nic $nic OOD status : Enabled\n");
    }
    else{
    $class->me->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-0?");  
    $class->me->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-1?");   
    $class->partner->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-0?");  
    $class->partner->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-1?");   
    $class->me->say("priv set -q diag; ic status");   
    $class->partner->say("priv set -q diag; ic status");  
	croak "Could not Enable OOD on nic $nic: $@ \n";
    }
}

=head2 ::ic_ood_off

Runs "ic ood off <nic_number>" on the given filer and runs "ic ood status" to make sure 
that OOD on the given nic is Disabled.

=over

=item Arguments

  Required: me=>1|partner=>1, nic=>0|1 (nic number)

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_ood_off(me=>1,nic=>0);

=cut

sub ic_ood_off {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
    my ($temp,$nic);
    if(exists($options{me})) {
	$temp = 'me';
    }
    elsif(exists($options{partner})) {
	$temp = 'partner';
    }
    else {
	croak "Please provide the required parameters. e.g me=>|partner=>1\n";
    }
    $nic = $options{nic} if(exists($options{nic}));
    $class->$temp->say("priv set -q diag; ic ood off $nic");
    sleep(40); # increased timeout from 20 seconds to 40 secods for burt#582824
    my $hash = $class->ic_ood_status($temp=>1);
    if($hash->{"NIC_$nic Statistics"}->{"OOD Status"} =~ /Disabled/i){
      NOTE::logcomment("Nic $nic OOD status : Disabled\n");
    }
    else{
    $class->me->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-0?");  
    $class->me->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-1?");   
    $class->partner->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-0?");  
    $class->partner->say("priv set -q diag; bootargs print ql-enable-ood-on-nic-1?");   
    $class->me->say("priv set -q diag; ic status");   
    $class->partner->say("priv set -q diag; ic status");  
	croak "Could not Disable OOD on nic $nic : $@\n";
    }
}

=head2 ::set_mcrc

This method is only supported for multi-subnet feature. It runs
options cd.takeover.use_mcrc_file on/off.

=over

=item Arguments

  Required: me=>1 | partner=>1
  Required: flag=>'on' or flag=>'off'

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->set_mcrc(me=>1,flag=>'on');

=cut

sub set_mcrc {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $filer = $class->partner;
    $filer = $class->me if($options{me});
    $filer->registry->{"options.cf.takeover.use_mcrc_file"} = $options{flag}; 
}

=head2 ::get_mcrc

This method is only supported for multi-subnet feature. It runs
options cd.takeover.use_mcrc_file and return a boolean value, 0 means
off 1 means on.

=over

=item Arguments

  Required: me=>1 | partner=>1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    my $val = $mc->is_mcrc_on(me=>1);
OR
    NOTE::FcMetrocluster->is_mcrc_set(partner=>1 or me=>1);

=cut

sub get_mcrc {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $filer = $class->partner;
    $filer = $class->me if($options{me});
    return $filer->registry->{"options.cf.takeover.use_mcrc_file"} ;
}
    
=head2 ::vol_status_f

Issues vol status -f command and counts the total number of failed disks.
Returns the total number of failed disks with thier names.

=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    ($count,@disk_names) = $mc->vol_status_f(me=>1);

This will run the vol status -f method on 'me' filer

    $count = $mc->vol_status_f(partner=>1);

This will run the vol status -f method on 'partner' filer

OR    

    $pair_ME = NOTE::Filer->connect(host=>$FILERA,mode=>'#console');

    NOTE::FcMetrocluster->vol_status_f(me=>1);

=cut

#mc-u37*> vol status -f
#Broken disks
#RAID Disk       Device          HA  SHELF BAY CHAN Pool Type  RPM  Used (MB/blks)    Phys (MB/blks)
#---------       ------          ------------- ---- ---- ---- ----- --------------    --------------
#failed          mc-sw19:5.102   5a    6   6   FC:B   0  FCAL 10000 136000/278528000  137104/280790184 
#mc-u34> vol status -f
#
#Broken disks (empty)

#author: jawwad@netapp.com

sub vol_status_f {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($failed_disk_count,@disk_name,$count,$i);
    $i=0;
    $count=0;
    if(exists($options{me})) {        #if option is me=>1 then execute it on me filer
	my $vol_status_f = $class->me->say("vol status -f");
	if($vol_status_f =~/Broken disks \(empty\)/i) {   #match emptiness..
	    $failed_disk_count = 0;
	    NOTE::logcomment("No Failed Disks found\n");
	}
	else{
	    foreach(split('[\n\r]',$vol_status_f)) {
    	    #old regex =   /failed.*(\S+:\d+\d+.\d+).*/i 
    		if($_=~ /failed\s+(\S*)\s+.*/i) {  #if failed disks found match em and store into $1
    		    $disk_name[$count] = $1;
    		    $count++;
    		    $failed_disk_count++;
    		}
	    }
	}
	return($failed_disk_count,@disk_name);
    }
    elsif(exists($options{partner})) {
	my $vol_status_f = $class->partner->say("vol status -f");
	if($vol_status_f =~/Broken disks (empty)/i) {
	    $failed_disk_count = 0;
	  NOTE::logcomment("No Failed Disks found\n");
	}
	else{
	    foreach(split('[\n\r]',$vol_status_f)) {
		if($_=~ /failed\s+(\S*)\s+.*/i) {
		    $disk_name[$count] = $1;
		    $count++;
		    $failed_disk_count++;
		}
	    }
	}
	return($failed_disk_count,@disk_name);
    }
}

=head2 ::failed_disk_count

Issues disk show -v command and counts the total number of failed disks.
Returns the total number of failed disks with thier names.

=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    ($count,@disk_names) = $mc->failed_disk_count(me=>1);

This will run the failed_disk_count method on 'me' filer

    ($count,@disk_names) = $mc->failed_disk_count(partner=>1);

This will run the disk_count method on 'partner' filer

OR    

    $pair_ME = NOTE::Filer->connect(host=>$FILERA,mode=>'#console');

    NOTE::FcMetrocluster->failed_disk_count(me=>1);

=cut

sub failed_disk_count {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $failed_disk_count =0;
    my $count = 0;
    my @disk_name;
    if(exists($options{me})) {
	foreach(split('[\n\r]',$class->me->say("disk show -v"))) {
	    if($_=~ /(\S+:\d+\d+.\d+).*failed/i){  #anything:14.45 (disk name format)
		$disk_name[$count] = $1;
		$count++;
		$failed_disk_count++;
	    }
	}
	return ($failed_disk_count,@disk_name);
    }
    elsif(exists($options{partner})) {
	foreach(split('[\n\r]',$class->partner->say("disk show -v"))) {
	    #old regex = /(\S+:\d+\d+.\d+).*failed/i
	    if($_=~ /(.*)\s+.*failed/i){  #anything:14.45
		$disk_name[$count] = $1;
		$count++;
		$failed_disk_count++;
	    }
	}
	return ($failed_disk_count,@disk_name);
    }
}
=head2 ::pre_post_test_checks

This method calls failed_disk_count and redundance_check and checks for all the failed disks
and redundancy, croaks if redundancy is not found or failed disks are present.


=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    ($failed_disk_count,@failed_disk_names,$redun) = $mc->pre_post_test_checks(me=>1);

This will run the pre_post_test_checks method on 'me' filer

    ($failed_disk_count,@failed_disk_names,$redun) = $mc->pre_post_test_checks(partner=>1);

This will run the above method on 'partner' filer

=cut

sub pre_post_test_checks {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($failed_disk_count,@failed_disk_names,$redun); 
     $class->aggr_mirror_check(%options);
     $class->cf_ic_status_check(%options);
    ($failed_disk_count,@failed_disk_names) = $class->failed_disk_count(%options);
    $redun = $class->redundancy_check(%options);
    if($redun == 1) {
      NOTE::logcomment("Redundancy is present\n");
    }
    elsif($redun == 0) {
    NOTE::logcomment("Redundancy is not present\n");
    croak "Redundancy is not present\n";
    }
    if($failed_disk_count) {
	 NOTE::logcomment("WARNING : FAILED DISKS FOUND \n");
	 NOTE::logcomment("Total Failed disks are: $failed_disk_count\n");
    }
    elsif($failed_disk_count == 0) {
      NOTE::logcomment("No Failed disks found! \n");
    }
    return ($failed_disk_count,@failed_disk_names,$redun);
}

=head2 ::aggr_mirror_check

This method checks if all the aggrs on the given filer are mirrored

=over

=item Arguments


=item Example

=back

    $mc->aggr_mirror_check(me=>1);
    $mc->aggr_mirror_check(partner=>1);
This method checks if all the aggrs on the given filer are mirrored. Test will croak if all the aggrs are not mirrored.

=cut

sub aggr_mirror_check
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($class,%options) = @_;
    my @aggregates;
    my ($aggr,$aggr_name, $filer_name);
    if(exists($options{partner})) {
        @aggregates = $class->partner->Aggregates;}
    else {
        @aggregates = $class->me->Aggregates;
            }
    foreach $aggr(@aggregates){
        $aggr_name=$aggr->name;
        $filer_name=$aggr->Filer->host;
        while(($aggr->status)->status() =~/resyncing/i){
            NOTE::logcomment("\nResyncing aggregate. Sleeping for 1 minute \n");
            sleep 60;
	}
        croak "aggr $aggr_name on filer $filer_name is not mirrorred  " unless($aggr->is_mirrored);
    }
}

=head2 ::cf_ic_status_check

This method calls cf_status and ic_status check and checks if cf is up and links are enabled

=over

=item Arguments


=item Example

=back

    $mc->cf_ic_status_check(me=>1);
    $mc->cf_ic_status_check(partner=>1);
This will run the cf status check and ic status check on both the filers and make sure everything is enabled, if it is not enabled it will croak. 

=cut


sub cf_ic_status_check{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($Link_state,$state,$filer_string);

    if(exists($options{partner}))
    { 
        $filer_string='partner';
    }
    else { $filer_string ='me'; }
    my $output = $class->cf_status($filer_string=>1);
    croak "Cluster is not enabled on the filer " unless ($output =~ /(Cluster enabled)|(Controller Failover enabled)/);
    my $ic_status_output = $class->$filer_string->say("priv set -q advanced ; ic status");
    foreach ( split( '[\n\r]', $ic_status_output ) ) {
        if ( $_ =~ /\s*Link\s*(\d+)\s*:\s*(\w+)/i ) {
            $Link_state = $2;
            $state      = $1;
            NOTE::logcomment("Link_$state --> $Link_state");
            if ( $Link_state =~ /up/i ) {
                NOTE::logcomment("Link_$state is up ");
            } 
            else {
                croak "Test Failed:Link is Down";
            }
        }
    }
} 
     
=head2 :: disk_status_check

This method calls failed_disk_count and redundancy_check and checks for all the failed disks
and redundancy, croaks if redundancy is not found , just returns a warning if  failed disks are present.
Also returns list of failed disks.
Option to pass me=>1 or partner=>1

=over

=item Arguments


=item Example

=back

($failed_disk_count,@failed_disk_names,$redun) = $mc->disk_status_check(me=>1);
($failed_disk_count,@failed_disk_names,$redun) = $mc->disk_status_check(partner=>1);



=cut


sub disk_status_check{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($failed_disk_count,@failed_disk_names,$redun);
    ($failed_disk_count,@failed_disk_names) = $class->failed_disk_count(%options);
    $redun = $class->redundancy_check(%options);
    if($redun == 1) {
      NOTE::logcomment("Redundancy is present\n");
    }
    elsif($redun == 0) {
    NOTE::logcomment("Redundancy is not present\n");
    croak "Redundancy is not present\n";
    }
    if($failed_disk_count) {
         NOTE::logcomment("WARNING : FAILED DISKS FOUND \n");
         NOTE::logcomment("Total Failed disks are: $failed_disk_count\n");
    }
    elsif($failed_disk_count == 0) {
      NOTE::logcomment("No Failed disks found! \n");
    }
    return ($failed_disk_count,@failed_disk_names,$redun);
}



=head2 ::Basic_CFOD

Performs a basic cfod on the filer

=over

=item Arguments

  Required: None 

B<device_rpower_names> - I<string>

I<Optional.>
SMC specific argument. String of comma separated rpower names for devices on SMC partner site to do rpower ON. 
if this argument is not provided, subroutine tries to fetch it from nate_host file of partner filer through SHELF_RPOWER_NAMES & FIlER_RPOWER_NAME parameters

=item Example

=back
    
    $mc->Basic_CFOD

=cut

sub Basic_CFOD {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $me_name = $class->me->host;
    my $p_name = $class->partner->host;
    my ($sw1, $sw2);
    
    if($class->is_fmc){        #FMC specific operations
        $sw1 = NOTE::FcSwitch->connect(host => $class->{partner_fabrics}->{switch1});
        $sw2 = NOTE::FcSwitch->connect(host => $class->{partner_fabrics}->{switch2});
    
        NOTE::logcomment("Put the Partner $p_name to Loader prompt:\n");
    
        $class->partner->transit_to("FIRMWARE");
        $sw1->switch_disable;
        $sw2->switch_disable;
        sleep(100);        
    }
    else{                      #SMC specific operations
        my ($device_rpower_names, $filer_rpower_name, @rpower_list, @subtest_obj);
        
        #Gather shelf rpower names
        if($options{device_rpower_names}){
            $device_rpower_names = $options{device_rpower_names};
        }
        else{
            my $hostrec = Hostrec->new( id => $p_name);
            my %hostrec_fields = $hostrec->param_fields(); 
            $device_rpower_names = $hostrec_fields{SHELF_RPOWER_NAMES};
            $filer_rpower_name = $hostrec_fields{FILER_RPOWER_NAME};
            $device_rpower_names = "$device_rpower_names".",$filer_rpower_name";
        }
        croak ("Unable to fetch devices' rpower information") unless ($device_rpower_names);
        @rpower_list = split( /,/, $device_rpower_names );
        NOTE::logcomment("Devices on remote site to be powered OFF (to simulate disaster): @rpower_list ");                
        NOTE::logcomment("Powering OFF the partner filer $p_name and its local shelves/devices to simulate site disaster.\n");
        FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_list, off=>1);
        sleep(100);
    }
    #Let's issue CFOD command 
    my $return_value;
    for(my $i = 0; $i <= 10; $i++) {
        $return_value = $class->cfod_takeover( me => 1);
        last if($return_value);
        sleep(15);
    }
    if(!$return_value) {
        croak "Couldn't takeover the partner filer\n";
    }
    
    #Let's wait for takeover to complete
    cluster::wait_state($class->partner->hostspec,"DEAD",350);
    cluster::wait_state($class->me->hostspec,"TAKEOVER",350);
#check cf monitor after cfod

SayWrap::sayEOF( $class->me->conn, "priv set -q diag; cf monitor all");

    #Recovery & Giveback Process
    if($class->is_fmc){        #FMC specific operations   
        $sw1->switch_enable;
        $sw2->switch_enable;
        sleep(10);
    }
    else{                      #SMC specific operations
        my ($device_rpower_names, $filer_rpower_name, @rpower_list, @subtest_obj);
        
        #Gather shelf rpower names
        if($options{device_rpower_names}){
            $device_rpower_names = $options{device_rpower_names};
        }
        else{
            my $hostrec = Hostrec->new( id => $p_name);
            my %hostrec_fields = $hostrec->param_fields(); 
            $device_rpower_names = $hostrec_fields{SHELF_RPOWER_NAMES};
            $filer_rpower_name = $hostrec_fields{FILER_RPOWER_NAME};
            $device_rpower_names = "$device_rpower_names".",$filer_rpower_name";
        }
        croak ("Unable to fetch devices' rpower information") unless ($device_rpower_names);
        @rpower_list = split( /,/, $device_rpower_names );
        NOTE::logcomment("Devices on remote site to be powered on: @rpower_list ");       
        NOTE::logcomment("Powering ON the filer & other hardware at partner ($p_name) site.\n");
        FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_list); 
        #Sleeping for 120 seconds for victim filer to boot to loader prompt
        sleep(120);               
    }
    $class->wait_state_for_broken_aggrs( partner => 1, timeout => 2000);
    NOTE::logcomment("Entering Mirroring Phase!!\n");
    my @aggr_name = $class->partner->aggregates;
    $class->mirror_out_of_date_aggr($class->partner, @aggr_name);
    sleep(5);
    NOTE::logcomment("Done Mirroring. Now booting partner to \"Waiting for giveback\" state\n");
    my $host = $class->partner->host;
    my $p_F = NOTE::Filer->connect(host=>$host, mode=>'#console');

    #very important piece: it pushes waiting for giveback to the prompts of the filer
    #so it can stop when it sees waiting for giveback prompt
    push @{ $p_F->{_conn}{'prompts'} }, "Waiting for giveback|Waiting for cluster give|Waiting for clus|Press Ctrl\-C for Maintenance menu to release disks";
    eval {
      Tharn::say($p_F->conn,"boot_ontap", -timeout,420);
    };
    if($@) {
      NOTE::logcomment("Timing out after Waiting for giveback, looks good!: $@ \n");
    }
    $class->wait_for_giveback( me => 1 );
    sleep(20);
    #Workaround for login prompt issues with Console connection on filer boot
    Tharn::host($me_name);
    Tharn::connect('#console', -login);
 
    my $filer_con = NOTE::Filer->connect( host => $me_name, mode => '#console');
    for(my $i = 0; $i <= 10; $i++) {
    eval {
        $class->cfod_giveback(cmd => "cf giveback -f", conn => $filer_con);
        sleep 25;
        cluster::wait_state($class->partner->hostspec,"UP",100);
        };
    if($@){
          NOTE::logcomment("Timed out...Keep trying to giveback. Error Message: $@  \n");
        }
    my $output = $class->cf_status( me => 1);
        my $phost = $class->partner->host;
        last if($output =~ /(Cluster enabled, $phost is up)|(Controller Failover enabled, $phost is up)/);
        sleep(15);
    }
    cluster::wait_state($class->me->hostspec,"UP",300) or die($@);
    #Workaround for login prompt issues with Console connection on filer boot
    Tharn::host($p_name);
    Tharn::connect('#console', -login);
}


=head2 ::TakeOver

TakeOver subroutine to takeover partner filer after simulating site disaster on partner filer's site.
For Fabric MetroCluster configurations, disables the partner switches, halts the partner to loader prompt without failover and does a Takeover from ME filer. 
For Stretch MetroCluster configurations, performs rpower off of partner site equipments such as filer & switches and does a Takeover from ME filer.
This subroutine divides the Basic_CFOD subroutine in TakeOver and GiveBack part. This can be used in scripts where after taking over you have to perform some functions on the filers and then do a giveback. Used with GiveBack subroutine

=over

=item Arguments

  Required: None 

B<device_rpower_names> - I<string>

I<Optional.>
SMC specific argument. String of comma separated rpower names for devices on SMC partner site to do rpower off (for simulating site disaster). 
if this argument is not provided, subroutine tries to fetch it from nate_host file of partner filer through SHELF_RPOWER_NAMES & FIlER_RPOWER_NAME parameters 

=item Example

=back

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->TakeOver;
    
    To run a takeover from the partner filer, create a new mc object with filer as partner filer
    $mc_partner =  NOTE::FcMetrocluster->new(Filer =>$FILERB);
    $mc_partner->TakeOver;

=cut

sub TakeOver{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $me_name = $class->me->host;
    my $p_name = $class->partner->host;
    my ($sw1, $sw2, $filer_rpower_name);
    
    if($class->is_fmc){        #FMC specific operations
        $sw1 = NOTE::FcSwitch->connect(host => $class->{partner_fabrics}->{switch1});
        $sw2 = NOTE::FcSwitch->connect(host => $class->{partner_fabrics}->{switch2});
    
        NOTE::logcomment("Put the Partner $p_name to Loader prompt:\n");
    
        $class->partner->transit_to("FIRMWARE");
        $sw1->switch_disable;
        $sw2->switch_disable;
        sleep(100);        
    }
    else{                      #SMC specific operations
        my ($device_rpower_names, @rpower_list, @subtest_obj);               
        
        #Gather shelf rpower names
        if($options{device_rpower_names}){
            $device_rpower_names = $options{device_rpower_names};
        }
        else{
            my $hostrec = Hostrec->new( id => $p_name);
            my %hostrec_fields = $hostrec->param_fields(); 
            $device_rpower_names = $hostrec_fields{SHELF_RPOWER_NAMES};
            $filer_rpower_name = $hostrec_fields{FILER_RPOWER_NAME};
            $device_rpower_names = "$device_rpower_names".",$filer_rpower_name";
        }
        croak ("Unable to fetch devices' rpower information") unless ($device_rpower_names);
        @rpower_list = split( /,/, $device_rpower_names );
        NOTE::logcomment("Setting AUTOBOOT bootarg variable to FALSE\n");
        $class->partner->say("priv set -q diag; bootargs set AUTOBOOT false");
        NOTE::logcomment("Devices on remote site to be powered OFF (to simulate disaster): @rpower_list ");                
        NOTE::logcomment("Powering OFF the partner filer $p_name and its local shelves/devices to simulate site disaster.\n");
        FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_list, off=>1);
        sleep(100);
    }
    #Let's issue CFOD command 
    my $return_value;
    for(my $i = 0; $i <= 10; $i++) {
        $return_value = $class->cfod_takeover( me => 1);
        last if($return_value);
        sleep(15);
    }
    if(!$return_value) {
        croak "Couldn't takeover the partner filer\n";
    }
    
    #Let's wait for takeover to complete
    cluster::wait_state($class->partner->hostspec,"DEAD",350);
    cluster::wait_state($class->me->hostspec,"TAKEOVER",350);   
    if($class->is_smc && defined($filer_rpower_name)){
        NOTE::logcomment("Takeover complete. Rpowering ON filer to boot to LOADER prompt\n");
        my @temp_array; # temporary array to hold filer name for rpower list
        push(@temp_array, $filer_rpower_name );
        FcMetroclusterUtils::rpower_devices( rpower_list => \@temp_array);
    }
}


=head2 ::GiveBack

GiveBack subroutine to recover partner after CFOD and perform giveback.
For Fabric Metrocluster, enables the switches connected to the partner, mirrors broken aggregates, boots up the partner (taken over) filer and then issues a give back from the me filer. 
For Stretch MetroCluster, performs ropower ON of partner site equipments (such as filer, disk shelves), mirrors broken aggregates, boots up the partner (taken over) filer and then issues a give back from the me filer.

=over

=item Arguments

  Required: None 

B<device_rpower_names> - I<string>

I<Optional.>
SMC specific argument. String of comma separated rpower names for devices on SMC partner site to do rpower ON. 
if this argument is not provided, subroutine tries to fetch it from nate_host file of partner filer through SHELF_RPOWER_NAMES & FIlER_RPOWER_NAME parameters
 
=item Example

=back

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->GiveBack;

    To run a takeover from the partner filer, create a new mc object with filer as partner filer
    $mc_partner =  NOTE::FcMetrocluster->new(Filer =>$FILERB);
    $mc_partner->GiveBack;

=cut

sub GiveBack {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my $me_name = $class->me->host;
    my $p_name = $class->partner->host;
    my ($sw1, $sw2);
    
    NOTE::logcomment("Starting CFOD recovery & giveback process.\n");
    
    if($class->is_fmc){        #FMC specific operations
        $sw1 = NOTE::FcSwitch->connect(host => $class->{partner_fabrics}->{switch1});
        $sw2 = NOTE::FcSwitch->connect(host => $class->{partner_fabrics}->{switch2});
    
        $sw1->switch_enable;
        $sw2->switch_enable;
        sleep(10);
    }
    else{                      #SMC specific operations
        my ($device_rpower_names, $filer_rpower_name, @rpower_list, @subtest_obj);
        
        #Gather shelf rpower names
        if($options{device_rpower_names}){
            $device_rpower_names = $options{device_rpower_names};
        }
        else{
            my $hostrec = Hostrec->new( id => $p_name);
            my %hostrec_fields = $hostrec->param_fields(); 
            $device_rpower_names = $hostrec_fields{SHELF_RPOWER_NAMES};
            $filer_rpower_name = $hostrec_fields{FILER_RPOWER_NAME};
            $device_rpower_names = "$device_rpower_names".",$filer_rpower_name";
        }
        croak ("Unable to fetch devices' rpower information") unless ($device_rpower_names);
        @rpower_list = split( /,/, $device_rpower_names );
        NOTE::logcomment("Devices on remote site to be powered on: @rpower_list ");       
        NOTE::logcomment("Powering ON the filer & other hardware at partner ($p_name) site.\n");
        FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_list); 
        #Sleeping for 120 seconds for victim filer to boot to loader prompt
        sleep(120);               
    }
    $class->wait_state_for_broken_aggrs( partner => 1, timeout => 2000);
    NOTE::logcomment("Entering Mirroring Phase!!\n");
    my @aggr_name = $class->partner->aggregates;
    $class->mirror_out_of_date_aggr($class->partner, @aggr_name);
    sleep(5);
    NOTE::logcomment("Done Mirroring. Now booting partner to \"Waiting for giveback\" state\n");
    my $host = $class->partner->host;
    my $p_F = NOTE::Filer->connect(host=>$host, mode=>'#console');

    #very important piece: it pushes waiting for giveback to the prompts of the filer
    #so it can stop when it sees waiting for giveback prompt
    push @{ $p_F->{_conn}{'prompts'} }, "Waiting for giveback|Waiting for cluster give|Waiting for clus|Press Ctrl\-C for Maintenance menu to release disks";
    eval {
      Tharn::say($p_F->conn,"boot_ontap", -timeout,420);
    };
    if($@) {
      NOTE::logcomment("Timing out after Waiting for giveback, looks good!: $@ \n");
    }
    $class->wait_for_giveback( me => 1 );
    sleep(20);
    #Workaround for login prompt issues with Console connection on filer boot
    Tharn::host($me_name);
    Tharn::connect('#console', -login);
 
    my $filer_con = NOTE::Filer->connect( host => $me_name, mode => '#console');
    for(my $i = 0; $i <= 10; $i++) {
    eval {
        $class->cfod_giveback(cmd => "cf giveback -f", conn => $filer_con);
        sleep 25;
        cluster::wait_state($class->partner->hostspec,"UP",100);
        };
    if($@){
          NOTE::logcomment("Timed out...Keep trying to giveback. Error Message: $@  \n");
        }
    my $output = $class->cf_status( me => 1);
        my $phost = $class->partner->host;
        last if($output =~ /(Cluster enabled, $phost is up)|(Controller Failover enabled, $phost is up)/);
        sleep(15);
    }
    cluster::wait_state($class->me->hostspec,"UP",300) or die($@);
    #Workaround for login prompt issues with Console connection on filer boot
    Tharn::host($p_name);
    Tharn::connect('#console', -login);
     
}


=head2 ::cfo_giveback

Giveback subroutine to recover partner after CFO takeover and perform giveback.
It boots up the takenover filer specified in argument and then issues giveback from the victor filer. 

=over

=item Arguments

  Required: me => 1 or partner => 1

B<force> - I<string>

I<Optional.>
Set to 1 for forced giveback using "cf giveback -f".
 
=item Example

=back

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->cfo_giveback ( partner => 1, force =>1 );

=cut

sub cfo_giveback {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %options) = @_;
    my ( $host, $giveback_filer, $filer_conn);
    my $CF_ENABLED = 0; #flag to identify cf is enabled after giveback
    my $giveback_string = 'cf giveback';

    if($options{force}) {
        $giveback_string = 'cf giveback -f';
    }
    
    if(exists($options{me})){
        $giveback_filer = 'partner';
        $host = $class->me->host;
        $filer_conn = NOTE::Filer->connect( host => $host, mode => '#console' );        
    }
    else{
         $giveback_filer = 'me';       
         $host = $class->partner->host;
        $filer_conn = NOTE::Filer->connect( host => $host, mode => '#console' );  
    }
    
    NOTE::logcomment("Checking filer $host is at loader prompt");
    if ( "FIRMWARE" eq boot_state( $filer_conn->conn, timeout => 150 ) ) {
        NOTE::logcomment("Filer $host is at loader prompt\n");
    } 
    else {
     croak ("Filer $host is not at loader prompt\n. AUTOBOOT options should be set to false\n");
    }

    ##Let's proceed with recovery & giveback
    NOTE::logcomment("Boot $host to Waiting for giveback prompt\n");
    push @{ $filer_conn->{_conn}{'prompts'} },
    "Waiting for giveback|Waiting for cluster give|Waiting for clus|Press Ctrl\-C for Maintenance menu to release disks";
    eval { Tharn::say( $filer_conn->conn, "boot_ontap", -timeout, 420 ); };
    if ($@) {
     NOTE::logcomment(
        "Timing out after Waiting for giveback, looks good! : $@ \n");
      }
    
      #Check from Victor filer that partner is ready for giveback
      $class->wait_for_giveback( $giveback_filer => 1 );
      sleep(20);
      
      #Perform giveback from victor filer
      my $temp_filer = $class->$giveback_filer->host;
      my $giveback_filer_conn =
        NOTE::Filer->connect( host => $temp_filer, mode => '#console' );
      
      #Workaround for login prompt issues with Console connection on filer boot
      Tharn::host($temp_filer);
      Tharn::connect('#console', -login);        
      
      #Using a loop for giveback as a workaround to avoid interleaved console messages 
      #as we are passing giveback command over console connection  
      for ( my $i = 0; $i <= 3; $i++ ) {
         eval {
            $class->cfod_giveback(
               cmd  => $giveback_string,
               conn => $giveback_filer_conn
            );
            cluster::wait_state( $host, "UP", 120 );
         };
         if ($@) {
            NOTE::logcomment(
               "Timed out.. keep trying to giveback :--> $@  <--\n");
         }
         my $output = $class->cf_status( $giveback_filer => 1 );
         if ( $output =~ /(Cluster enabled, $host is up)|(Controller Failover enabled, $host is up)/ ){
             $CF_ENABLED = 1; #set the flag that CF is enabled
             last;
         }
         sleep(60);
      }
      if(!$CF_ENABLED){
          croak("Controller Failover is not enabled!");
      }    
}

=head2 ::check_vfiler_interface

This method checks /etc/rc file and verifies interface assigned to vfiler is properly configured for takeover.This will also checks vfiler interface is active on both nodes

=over

=item Arguments

One argument is needed

B<interface> => I<interface name>

I<Required>
Name of the interface assigned to the vfiler

=item Return Value

1 on success, 0 otherwise

=item Examples

=back

        $mc->check_vfiler_interface(interface => 'e0a') ;

This will checks vfiler interface is active on both nodes. This will also verify /etc/rc file of partner node and checks vfiler interface is configured properly for takeover

=cut

sub check_vfiler_interface {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


   my ( $class, %options ) = @_;
   my ( $vfiler_ifc, $nw_infoA, $nw_infoB, $rc_line, $filer_string );

   if ( exists( $options{interface} ) ) {
      $vfiler_ifc = $options{interface};
   } else {
      croak "Vfiler Interface Name is not passed\n";
   }
   if ( exists( $options{me} ) ) {
      $filer_string = 'me';
   } else {
      $filer_string = 'partner';
   }

   $nw_infoA = get_ifconfig_info( $class->me->conn );
   $nw_infoB = get_ifconfig_info( $class->partner->conn );

   my $ifc_status = "UP,BROADCAST,RUNNING";
   if (  ${$nw_infoA}{$vfiler_ifc}->{'flags'} =~ m/$ifc_status/
      && ${$nw_infoB}{$vfiler_ifc}->{'flags'} =~ m/$ifc_status/ )
   {
      NOTE::logcomment("Vfiler Interface is  active on both nodes\n");
   } else {
      croak "Vfiler Interface is not active on both nodes\n";
   }
   my $rc_text = rdfile( $class->$filer_string->conn, "/etc/rc" );
   my @lines = grep( !/^rdfile/, split( /\n/, $rc_text ) );
   foreach (@lines) {
      if ( $_ =~ /^ifconfig\s+$vfiler_ifc/ ) {
         $rc_line = $_;
      }
   }
   if ( $rc_line =~ /partner\s+$vfiler_ifc/i ) {
      NOTE::logcomment(
"Verified /etc/rc file and found vfiler interface is configured properly\n"
      );
   } else {
      croak "Vfiler interface is not configured for takeover";
   }
}

=head2 ::populate_iscsi

The library routine will export the lun on the filer, mount the lun
on the client, and then startup ISCSI traffic on the client. 
It will populate lun with data using RichData::populate.

=over

=item Arguments

B<initiator> - I<string>

I<Required.>
Node name of the ISCSI initiator

B<igroup> - I<string>

I<Required.>
Name of the ISCSI igroup to be created on Filer
Uses the same if this igroup is existing -  doesnot verify
whether the node name is the same as given initiator.

B<client> - I<string>

I<Optional.>
Name of the ISCSI linux host.
This module currently supports only linux hosts

B<timeout> - I<string>

I<Optional.>
Timeout for the operation in seconds

B<sanlun_path> - I<string>

I<Optional.>
Path of the sanlun binary in the ISCSI host m/c.

B<pop_opts> - I<RichData argument Hash>

Optional.

All parameters, in the form of a hash is passed to RichData::populate for processing.

=item Examples

=back

          $lun->populate_iscsi;

=cut

sub populate_iscsi {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

     shift ;
     my $self = shift;
     my ($device,$sayout) ;
    my (%args) = validate (@_, { initiator => {type=>SCALAR, optional=>0},
                             igroup => {type=>SCALAR, optional=>1},
                             client => {type=>SCALAR|UNDEF, default=>"localhost", optional=>1},
                             timeout => {type=>SCALAR|UNDEF, default=>600, optional=>1},
                             pop_opts => {type => HASHREF, default => undef, optional => 1},
                             sanlun_path => {type=>SCALAR|UNDEF, default=> '/usr/sbin', optional=>1,regex=>qr/^(\/\w+)+$/ },
                             sio => {type=>SCALAR|UNDEF, default=>0, optional=>1},
                             restart => {type=>SCALAR|UNDEF, default=>0, optional=>1},
                             sio_hash => {type => HASHREF, default => undef, optional => 1},
                             read => {type=>SCALAR|UNDEF, default=>60, optional=>1},
                             sio_threads => {type=>SCALAR|UNDEF, default=>1, optional=>1},
                             });
    $self->_map(%args);
    if(!$args{'restart'}) {
           iscsi_mountfs($self,%args);
    }
    my $mount_point = $self->{_mntconfig}{'mount'};
    my $client_conn = make_conn($args{client},"#rsh-persist");
   if($args{'restart'}) {
        if ($self->exists) {
            my @lun_dev = $self->_disks(client => $args{client}, sanlun_path=>$args{'sanlun_path'});
            NOTE::logcomment("Disk list: @lun_dev");
            $device = $lun_dev[0]; # pick first disk
        }
       my $out = SayWrap::sayEOF($client_conn, "sudo /bin/umount -f  $mount_point");
       croak $out if($out =~ /ERROR|FAILED/i);
       $sayout = SayWrap::sayEOF($client_conn, "sudo /bin/mount -t ext2 $device $mount_point");
       croak $sayout if($sayout =~ /ERROR|FAILED/i);
    }


    if($args{sio}){        
        my $sio_hash;
        
        if($args{restart} && defined($args{sio_hash})){
            $sio_hash = FcMetroclusterUtils::run_sio( mount_path => $mount_point, 
                                          client => $args{client},
                                          read => $args{read},
                                          sio_threads => $args{sio_threads},
                                          restart   => $args{restart},
                                          sio_hash  => $args{sio_hash},                                      
                                        );
            
        }
        else{
        $sio_hash = FcMetroclusterUtils::run_sio( mount_path => $mount_point, 
                                      client => $args{client},
                                      read => $args{read},
                                      sio_threads => $args{sio_threads},                                     
                                    );            
        }
        return $sio_hash;        
    }
    else{
        my $cmdHammer
            = "/usr/local/bin/hammer "
            . $mount_point
            . " -nofilercore -mode mixed -fill -bg ";

        my $host    = Tharn::host($args{client});
        my $lnxclnt = Tharn::connect('#rsh');
        Tharn::say( $lnxclnt, $cmdHammer, -noprompt );
        return 0;                
    }

}

=head2 ::iscsi_mountfs

This will mount the filer with lun
=back

          $lun->iscsi_mountfs;

=cut

sub iscsi_mountfs {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    croak "Cannot mount an offlined lun!" unless($self->{_online});
    my $sudo = "sudo";
    my $iscsid = "/etc/init.d/iscsi";
    my $mount = "/bin/mount";
    my $device;
    my (%args) = @_;
    NOTE::logcomment($args{client});
    my $client_conn = make_conn($args{client},"#rsh-persist");
    NOTE::logcomment(Dumper($client_conn));

    # Check whether the client is added as DiscoveryAddress in /etc/iscsi.conf of ISCSI HOST
    my $filer_default_ip = $self->Filer->defaultip;
    my $timeout = 60;
    my ($target,$host_ip,$out);
    my $iscsiadm_path = "/sbin/iscsiadm";
    my $cmd
        = "$sudo $iscsiadm_path --mode discovery --type sendtargets --portal $filer_default_ip";
    my (undef,undef,$after) = Tharn::say( $client_conn, $cmd, -timeout, $timeout);
    NOTE::logcomment("OUTPUT : $after \n");

    my @array = split (/[\r\n]/,$after);
    foreach( @array ) {
        if($_ =~ /^($filer_default_ip:\d+),\d+\s+(\S+)/i ) {
                 $host_ip = $1;
                $target = $2;
                print "host_id = $host_ip\ntarget = $target\n";
        }
    }


$cmd= "$sudo $iscsiadm_path --mode node --targetname $target --portal $host_ip --login";

(undef,undef,$after) = Tharn::say( $client_conn, $cmd, -timeout, $timeout);
                NOTE::logcomment("OUTPUT : $after \n");
sleep 10;
my $session;
my $str_filer_ip ="$filer_default_ip" ;
 $cmd ="$sudo $iscsiadm_path -m session" ;
 (undef,undef,$after) = Tharn::say( $client_conn, $cmd, -timeout, $timeout);
if($after=~/\[(\d+)\] $str_filer_ip/) {
        $session= $1 ;
}
NOTE::logcomment("session ==$session ==");
 $cmd ="$sudo $iscsiadm_path -m session -r $session --rescan ";
 (undef,undef,$after) = Tharn::say( $client_conn, $cmd, -timeout, $timeout);
if($after =~/Rescanning session \[sid: $session, target: $target, portal: /) {
        NOTE::logcomment("Rescanning the session");
}


    # Check whether the client in logged in to ISCSI HOST
    my $igroup_status = $self->Filer->say('igroup show '.$self->{_igroup});
    if($igroup_status =~ /not logged in/){
        warn "igroup is not logged into iscsi host.. sleeping for 5 seconds before rechecking";
        sleep 5;
    }
    croak $self->{_igroup}." igroup is not logged into ISCSI Host: $igroup_status" if($igroup_status =~ /not logged in/);

    # create and format the lun
    if ($self->exists) {
        my @lun_dev = $self->_disks(client => $args{client}, sanlun_path=>$args{'sanlun_path'});
        NOTE::logcomment("Disk list: @lun_dev");
        $device = $lun_dev[0]; # pick first disk
    }
    croak "Device list is empty! Lun Undetected" unless($device);
    NOTE::logcomment("Device is... $device");

    #Check whether chosen device is already mounted
   my $sayout = SayWrap::sayEOF($client_conn, "$sudo $mount | grep $device");
   logcomment("Device ".$device." out: ".$sayout);
    croak "Device already mounted: $sayout" if($sayout =~ /($device)/);
    # Make the file system
    Tharn::say($client_conn, "$sudo /sbin/mkfs -t ext2 $device", '-prompt', "Proceed anyway\?");
    (undef, undef, $out) = Tharn::say($client_conn, "y", '-timeout', $args{timeout});
    croak("$out\n") if ($sayout =~ /ERROR/i);

    # Mount the filesystem
    my $mountdir = "/t/".$args{client}."/dev_".$self->name;
    $sayout = SayWrap::sayEOF($client_conn, "$sudo mkdir -p $mountdir");
    croak $sayout if($sayout =~ /ERROR|FAILED/i);
    $sayout = SayWrap::sayEOF($client_conn, "$sudo $mount -t ext2 $device $mountdir");
    croak $sayout if($sayout =~ /ERROR|FAILED/i);
    $sayout = SayWrap::sayEOF($client_conn, "$sudo chmod 777 $mountdir");
    croak $sayout if($sayout =~ /ERROR|FAILED/i);
      $sayout = SayWrap::sayEOF($client_conn, "$sudo $mount | grep $device");
     logcomment("Device ".$device." out: ".$sayout);
     if($sayout =~ /($device)/) {
         logcomment("Device $device is mounted");
     } else {
	croak "Device $device is NOT mounted!";
     }
    $self->{_mntconfig} = {mount => $mountdir , client => $args{'client'}};
    NOTE::logcomment("Mount Point is ".$self->{_mntconfig}{'mount'});

    return $self;
}


=head2 ::fc_vfiler_snapmirror

To do all snapmirror operation via vfiler either for volume or qtree

=over

=item src

if it is for volume user should pass src volume object
else if qtree he should pass source qtree path


=item dst

if it is for volume user should pass dst volume object
else if qtree he should pass destination qtree path

=item Return Value

None

=item Examples

=back

        $vfiler_object->fc_Vfiler_snapmirror ;

=cut

sub fc_vfiler_snapmirror {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

        shift ;
        my $vf_obj_dst = shift;
        my %args = @_ ;
        my($action,$cmd,$result,$time_remaining ,$sm_wait,$mode);
        my($src_qtree, $src_path,$dst_path , $src_vol,$dst_vol) ;
        my $vf_obj_src = $args{src_obj} ;
        my $src_vf_nm=$vf_obj_src->{_name};
        my $dst_vf_nm = $vf_obj_dst->{_name} ;
        if($args{fail}) {
            $src_vf_nm = "ics_failover_vs" ;
        }
        elsif($args{multi}) {
            $src_vf_nm = "ics_multipath_vs" ;
        }
      if($args{qtree}) {
           $src_path = $args{src} ; # this is source qtree path
           $dst_path = $args{dst} ; # this is destination qtree path
           NOTE::logcomment("source path: $src_path");
           NOTE::logcomment("destination path: $dst_path");
      }
      elsif($args{volume}) {
          $src_vol = $args{src} ;
          $dst_vol = $args{dst} ;
            NOTE::logcomment(" src_vol = $src_vol == dst_vol == $dst_vol ===");
          $src_path = $src_vol->{_name} ;
          $dst_path = $dst_vol->{_name} ;
          if( $args{initialize} ) {
               $dst_vol->restrict;
          }
      }

     if($args{initialize} || $args{update} ) {
         my $throttle=$args{throttle} ;
         my @objects = ($vf_obj_src , $vf_obj_dst ) ;
         if($args{initialize}) {
           for ( my $flag = 0 ;$flag <= 1 ;$flag++ ) {
                        my $path= $objects[0]->{_Rootvol}->mount_path . "/etc/hosts" ;
                        my      $ip = $objects[1]->{_ipa} . " " . $objects[1]->{_name} ;
                        my $pfiler
                                = NOTE::Filer->connect( host => $objects[0]->{_pfiler}, mode => '#rsh' );
                       NOTE::Vfiler::_append_tofile(
                                $pfiler,
                                filepath => $path,
                                data     => $ip
                        );
                        @objects = ( $vf_obj_dst ,$vf_obj_src ) ;
            }
            $action= "initialize" ;
        }

         elsif ($args{update}) {
             $action= "update" ;
         }

         $throttle = $throttle ?"-k $throttle " : "" ;
         $cmd = "snapmirror  $action  $throttle -S $src_vf_nm:$src_path  $dst_path";
         $result = $vf_obj_dst->run(command => $cmd)
             or die "Could not call snapmirror operation on $dst_path: $@";
         if ( $result !~ /Transfer started/i) {
            die "Snapmirror $action failed to start:\n" . $result;
         }
      }

     my $TIMEOUT = $args{timeout}? $args{timeout} : 600 ;
    if($args{status}) {
      $cmd ="snapmirror status -l $dst_path";
     $result = $vf_obj_dst->run(command => $cmd)
      or die "Could not get snapmirror status: $@";
     if( $result =~/Quiesced/i )
       {
          $vf_obj_dst->run(command => "snapmirror resume  $dst_path") ;
       }
     if ($result =~ /State:\s+(.*)\b/) {
        NOTE::logcomment("snapmirror current state = $1 ==") ;
     }
  }
  if($args{status_wait}) {
     $sm_wait = 0 ;
    $cmd = "snapmirror status -l $dst_path";
    $result = $vf_obj_dst->run(command => $cmd)
       or die "Could not get snapmirror status: $@";
    while( ($result !~ /Status:\s+Idle/i) && ($result !~ /State:\s+Snapmirrored/i)  && ($sm_wait <= $TIMEOUT)) {
     sleep 10;
     $sm_wait += 10;
     $result = $vf_obj_dst->run(command => $cmd)
       or die "Could not get snapmirror status: $@";
     $time_remaining = $TIMEOUT-$sm_wait ;
       NOTE::logcomment(" time remainig  $time_remaining  ===== $result ====  ");
     if ($result =~ /State:\s+(.*)\b/) {
                NOTE::logcomment("snapmirror current state = $1 ==") ;
      }
   }
 }
  if($args{quiesce}) {
    $cmd= "snapmirror quiesce $dst_path" ;
    $result = $vf_obj_dst->run(command => $cmd);
    $sm_wait = 0 ;
    while( ($result !~ /State:\s+Quiesced/i) && ($sm_wait <= $TIMEOUT)) {
       sleep 10;
      $sm_wait += 10;
      $cmd = "snapmirror status -l $dst_path";
      $result = $vf_obj_dst->run(command => $cmd)
        or die "Could not get snapmirror status: $@";
       $time_remaining = $TIMEOUT-$sm_wait ;
       NOTE::logcomment(" time remainig  $time_remaining =====   ");
      if ($result =~ /State:\s+(.*)\b/) {
                NOTE::logcomment("snapmirror current state = $1 ==") ;
      }
    }
  }
 my ($src_filer, $dst_filer) ;
 if($args{compare}) {
   my $pfiler = NOTE::Filer->connect( host =>$vf_obj_dst->{_pfiler}, mode => '#rsh' );
   $src_filer = $vf_obj_src->{_pfiler};
   $dst_filer = $vf_obj_dst->{_pfiler};
   NOTE::logcomment(" src_filer == $src_filer  dst_filer == $dst_filer ");
   unless($src_path=~/\/vol\//) {
        $src_path = "/vol/" . $src_path ;
        $dst_path = "/vol/" . $dst_path ;
   }
   if($args{multi} || $args{fail}) {
          $src_filer = $src_vf_nm ;
    }
   $cmd = "priv set -q diag ;snapmirror check start   -Q -L data  -w -S $src_filer:$src_path  $dst_filer:$dst_path";
   $result = $pfiler->say($cmd) ;
    NOTE::logcomment( "result == $result ===");
   if($result =~/snapmirror check completed successfully/i) {
         NOTE::logcomment("No data differences found in Source and Destination");
   }
 }

  if($args{break}) {
   $cmd = "snapmirror break $dst_path";
   $result = $vf_obj_dst->run(command => $cmd) ;
   if($args{volume}) {
     $dst_vol->online if ($dst_vol->restricted );
   }
   $sm_wait = 0 ;
   while(($result !~ /State:\s+Broken-Off/i) && ($sm_wait <= $TIMEOUT) ) {
      sleep 10;
     $sm_wait += 10;
     $cmd = "snapmirror status -l $dst_path";
     $result = $vf_obj_dst->run(command => $cmd)
       or die "Could not get snapmirror status: $@";
     $time_remaining = $TIMEOUT-$sm_wait ;
     NOTE::logcomment(" time remainig   $time_remaining ==== ");
     if ($result =~ /State:\s+(.*)\b/) {
                NOTE::logcomment("snapmirror current state = $1 ==") ;
      }
    }
  }
}

=head2 ::ic_primary_set

Runs "ic primary set {0|1} [-r]" on the given filer and runs "ic primary show" to make sure 
that preferred port is set on correct NIC. If parameter "-r" is passed, NIC reset is also peformed.

=over

=item Arguments

Required: me=>1|partner=>1, nic=>0|1, reset=>1
B<reset> => I<1>

I<Optional>
Pass reset as 1 to perform NIC reset also for configured primary NIC.

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);

=cut
   
sub ic_primary_set {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
    my ($filer,$nic,$ic_primary_show);
	if(exists($options{me})) {
	$filer = 'me';
    }
    elsif(exists($options{partner})) {
	$filer = 'partner';
    }
    else {
	croak "Please provide the required parameters. e.g me=>|partner=>1\n";
    }
    if(exists($options{nic})) {
	$nic = $options{nic};
    }
    else {
	croak "Please provide the required parameters. e.g nic=>0|1\n";
    }
    if(exists($options{reset})) {
	$class->$filer->say("ic primary set $nic -r");
	sleep(5);
	$ic_primary_show = $class->ic_primary_show($filer=>1);
		if($ic_primary_show =~ /NIC $nic is set as primary link/i) {
		NOTE::logcomment("Set primary link on NIC $nic with NIC reset completed.\n");
		}
		else{
		croak "Failed to set primary link on NIC $nic & do a NIC reset successfully: $@ \n";
		}
	} else {
	$class->$filer->say("ic primary set $nic");
    sleep(2);
	$ic_primary_show = $class->ic_primary_show($filer=>1);
		if($ic_primary_show =~ /NIC $nic is set as primary link/i) {
		NOTE::logcomment("Configured NIC $nic as primary link\n");
		}
		else{
		croak "Could not configure primary link on NIC $nic: $@ \n";
		}
	}
}




=head2 ::ic_primary_unset

Runs "ic primary unset [-r]" on the given filer and runs "ic primary show" to make sure 
that preferred port setting is removed/unset from the filer. If parameter "-r" is passed, NIC reset is also peformed.

=over

=item Arguments

Required: me=>1|partner=>1
I<Optional:> B<reset> => I<1>

Pass reset as 1 to perform NIC reset also for configured primary NIC.

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_primary_unset(me=>1);

=cut
   
sub ic_primary_unset {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
    my ($filer,$ic_primary_show);
	if(exists($options{me})) {
	$filer = 'me';
    }
    elsif(exists($options{partner})) {
	$filer = 'partner';
    }
    else {
	croak "Please provide the required parameters. e.g me=>|partner=>1\n";
    }
    if(exists($options{reset})) {
	$class->$filer->say("ic primary unset -r");
	sleep(5);
	$ic_primary_show = $class->ic_primary_show($filer=>1);
		if($ic_primary_show =~ /Primary link is not set/i) {
		NOTE::logcomment("Unset primary link with NIC reset completed.\n");
		}
		else{
		croak "Failed to unset primary link & do a NIC reset successfully: $@ \n";
		}
	} else {
	$class->$filer->say("ic primary unset");
    sleep(2);
	$ic_primary_show = $class->ic_primary_show($filer=>1);
		if($ic_primary_show =~ /Primary link is not set/i) {
		NOTE::logcomment("Unset primary link completed.\n");
		}
		else{
		croak "Failed to unset primary link: $@ \n";
		}
	}
}


=head2 ::ic_primary_show

Runs "ic primary show" on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_primary_show(me=>1);

=cut
   
sub ic_primary_show {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
	my $output;
    if(exists($options{me})) {
	$output = $class->me->say("ic primary show");
	return $output;
    }
    elsif(exists($options{partner})){
	$output = $class->partner->say("ic primary show");
	return $output;
    }
	else {
	croak "No option was provided\n";
    }
}




=head2 ::ic_primary_check

Runs "ic primary show" on the given filer and checks whether provided NIC is set as primary link.

=over

=item Arguments

  Required: me=>1|partner=>1, nic=>0|1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_primary_check(me=>1, nic=>0);

=cut
   
sub ic_primary_check {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
	my ($filer, $nic, $ic_primary_show);
	if (exists ($options{nic})) {
		$nic = $options{nic};
		if(exists($options{me})) {
		$ic_primary_show = $class->ic_primary_show(me =>1);
		croak("ic primary link is not set properly.") unless($ic_primary_show =~ /NIC $nic is set as primary/i);
		}
		elsif(exists($options{partner})){
		$ic_primary_show = $class->ic_primary_show(partner =>1);
		croak("ic primary link is not set properly.") unless($ic_primary_show =~ /NIC $nic is set as primary/i);
		}
	}
	else {
	croak "Proper option was not provided for ic_primary_check\n";
    }
}

=head2 ::TakeOver_With_Checks

TakeOver subroutine to takeover partner filer after simulating site disaster on partner filer's site. This subroutine performs MetroCluster specific checks and collects configuration information before simulating site disaster & takeover.
For Fabric MetroCluster configurations, disables the partner switches, halts the partner to loader prompt without failover and does a Takeover from ME filer. 
For Stretch MetroCluster configurations, performs rpower off of partner site equipments such as filer & switches and does a Takeover from ME filer. 
This subroutine divides the Basic_CFOD subroutine in TakeOver and GiveBack part. This can be used in scripts where after taking over you have to perform some functions on the filers and then do a giveback. Used with GiveBack subroutine

=over

=item Arguments

  Required: None 

B<device_rpower_names> - I<string>

I<Optional.>
SMC specific argument. List of rpower names for devices on SMC partner site to do rpower off (for simulating site disaster). 
if this argument is not provided, subroutine tries to fetch it from nate_host file of partner filer through SHELF_RPOWER_NAMES & FIlER_RPOWER_NAME parameters 

=item Example

=back

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->TakeOver_With_Checks;
    
    To run a takeover from the partner filer, create a new mc object with filer as partner filer
    $mc_partner =  NOTE::FcMetrocluster->new(Filer =>$FILERB);
    $mc_partner->TakeOver_With_Checks;

=cut

sub TakeOver_With_Checks {

    my $self=shift;
    #---------------------------------------------------------------
    #Check The Cf status and check that Cluster is enabled or not
    #---------------------------------------------------------------
    $self->cf_enable( me => 1 );
    my  ( $failed_disk_count_before_A, @failed_disk_names_A, $redun_A );
    my ( $failed_disk_count_before_B, @failed_disk_names_B, $redun_B );
    #---------------------------------------------------------------
    #Check for the Failed Disk and Redundancy
    #---------------------------------------------------------------
    ( $failed_disk_count_before_A, @failed_disk_names_A, $redun_A ) =
        $self->pre_post_test_checks( me => 1 );
    ( $failed_disk_count_before_B, @failed_disk_names_B, $redun_B ) =
        $self->pre_post_test_checks( partner => 1 );
    $self->{failed_disk_me}=$failed_disk_count_before_A;
    $self->{failed_disk_partner}=$failed_disk_count_before_B;
    #----------------------------------------------------------------
    #Collect output of Basic commands
    #---------------------------------------------------------------
    my $aggr_cmd = 'aggr status -r;aggr status -v';
    $self->me->say("$aggr_cmd");
    $self->partner->say("$aggr_cmd");
    my $ic_cmd = "priv set -q diag;ic status;ic ood status";

    $self->me->say("$ic_cmd");
    $self->partner->say("$ic_cmd");

    my $cf_cmd = "priv set -q diag;cf monitor;cf monitor mb;options cf;registry walk options.cf";
    $self->me->say("$cf_cmd");
    $self->partner->say("$cf_cmd");
    my $reg_cmd = "registry walk options.cf";
    $self->me->say("$reg_cmd");
    $self->partner->say("$reg_cmd");
    my $df_cmd = 'df -Ah;df -Vh';
    $self->me->say("$df_cmd");
    $self->partner->say("$df_cmd");
    my $admin_cmd = 'priv set -q diag;fcadmin link_state';
    $self->me->say("$admin_cmd");
    $self->partner->say("$admin_cmd");

    #-----------------------------------------------------------------------
    #Calling the TakeOver  method, which will Disable partner switches,
    #halt the Partner Filer
    #-----------------------------------------------------------------------
    $self->TakeOver;
}

=head2 ::GiveBack_With_Checks

GiveBack subroutine to recover partner filer after CFOD and perform giveback. This subroutine performs MetroCluster specific checks and collects configuration information after giveback is complete. 
For Fabric Metrocluster, enables the switches connected to the partner, mirrors broken aggregates, boots up the partner (taken over) filer and then issues a give back from the me filer. Perform MetroCluster specific checks and collect configuration information.
For Stretch MetroCluster, performs ropower ON of partner site equipments (such as filer, disk shelves), mirrors broken aggregates, boots up the partner (taken over) filer and then issues a give back from the me filer. Perform MetroCluster specific checks and collect configuration information.

=over

=item Arguments

  Required: None 

B<device_rpower_names> - I<string>

I<Optional.>
SMC specific argument. List of rpower names for devices on SMC partner site to do rpower ON. 
if this argument is not provided, subroutine tries to fetch it from nate_host file of partner filer through SHELF_RPOWER_NAMES & FIlER_RPOWER_NAME parameters

 
=item Example

=back

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->GiveBack_With_Checks;

    To run a takeover from the partner filer, create a new mc object with filer as partner filer
    $mc_partner =  NOTE::FcMetrocluster->new(Filer =>$FILERB);
    $mc_partner->GiveBack_With_Checks;

=cut

sub GiveBack_With_Checks {

    my $self=shift;

    $self->GiveBack;
    #---------------------------------------------------------------
    #Check The Cf status and check that Cluster is enabled or not
    #---------------------------------------------------------------
    my  ( $failed_disk_count_after_A, @failed_disk_names_A, $redun_A );
    my ( $failed_disk_count_after_B, @failed_disk_names_B, $redun_B );
    #---------------------------------------------------------------
    #Check for the Failed Disk and Redundancy
    #---------------------------------------------------------------
    ( $failed_disk_count_after_A, @failed_disk_names_A, $redun_A ) =
      $self->pre_post_test_checks( me => 1 );
    ( $failed_disk_count_after_B, @failed_disk_names_B, $redun_B ) =
     $self->pre_post_test_checks( partner => 1 );
     my $fme=$self->{failed_disk_me};
     my $fpartner=$self->{failed_disk_partner};

     if (($failed_disk_count_after_A != $fme) || ($failed_disk_count_after_B != $fpartner))
     {
          NOTE::logresult('WARN', "Failed Disk Count has changed after CFOD , Before CFOD : \n  Failed Disk Count  Me Filer= $fme \n  Failed Disk Count  Partner Filer   $fpartner \n  After CFOD   Failed Disk Count  Me Filer= =  $failed_disk_count_after_A \n   Failed Disk Count  Partner Filer  = $failed_disk_count_after_B") ;
     } 
    #----------------------------------------------------------------
    #Collect output of Basic commands
    #---------------------------------------------------------------
    my $aggr_cmd = 'aggr status -r;aggr status -v';
    $self->me->say("$aggr_cmd");
    $self->partner->say("$aggr_cmd");
    my $ic_cmd = "priv set -q diag;ic status;ic ood status";

    $self->me->say("$ic_cmd");
    $self->partner->say("$ic_cmd");

    my $cf_cmd = "priv set -q diag;cf monitor;cf monitor mb;options cf;registry walk options.cf";
    $self->me->say("$cf_cmd");
    $self->partner->say("$cf_cmd");
    my $reg_cmd = "registry walk options.cf";
    $self->me->say("$reg_cmd");
    $self->partner->say("$reg_cmd");
    my $df_cmd = 'df -Ah;df -Vh';
    $self->me->say("$df_cmd");
    $self->partner->say("$df_cmd");
    my $admin_cmd = 'priv set -q diag;fcadmin link_state';
    $self->me->say("$admin_cmd");
    $self->partner->say("$admin_cmd");

 
}


=head2 ::get_fcvi_ports

Runs sysconfig -va and pulls out the fcvi port connected on the filer
if me=>1, it will pull info from the provided filer. if partner=>1, it will pull 
info from the partner of provided filer. It returns a hash which contains he FCVI slot
& port information. Following is the structure of the hash:
 
        $VAR1 = {
        'slot' => 1,
        'fcvi_ports' => [2a, 2b],
        };
=over

=item Arguments

  Required: me => 1 or partner => 1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $hashref = $mc->get_fcvi_ports( me => 1 );
    @fcvi_ports = @{$hashref->{fcvi_ports}};
    
OR
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $hashref = $mc->get_fcvi_ports( partner => 1 );
    @fcvi_ports = @{$hashref->{fcvi_ports}};


An example of printing all the FCVI ports on me filer
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    my $hashref = $mc->get_fcvi_ports( me => 1 );
    foreach my $port (@{$hashref->{fcvi_ports}}){
      NOTE::logcomment("FCVI Port: $port\n");
    }

=cut

sub get_fcvi_ports {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($fcvi, @slots, $slot, @fcvi_ports);
    
    if (exists($options{me})) {
        @slots = split(/slot/,$class->me->say('sysconfig -va'));
    }
    elsif(exists($options{partner})){
        @slots = split(/slot/,$class->partner->say('sysconfig -va'));        
    }
    else{
        croak("Insufficeint Parameter! Need either me=>1 or partner=>1 as argument");
    }

    foreach (@slots) { # $1 = slot number  $2 = port on filer such as 1a,1b
        if( $_ =~ /(\w+):\s+FCVI Host Adapter\s+(\w+)\s+.*(N|F)-port/si ) {
            push(@fcvi_ports,$2);
            $slot = $1;
        }
    }
    $fcvi->{slot} = $slot;
    $fcvi->{fcvi_ports} = \@fcvi_ports;
    
    return $fcvi;    
}


=head2 ::get_initiator_ports

Runs sysconfig -va and pulls out the initiator ports on the filer ( ports 
Connected to fabric switches for FMC or connected to disks for SMC).
if me=>1, it will pull info from the provided filer. if partner=>1, it will pull 
info from the partner of provided filer. It returns a hash which contains he initiator slot
& port information. Following is the structure of the hash:
 
        $VAR1 = {
        'slots' => [0,3],
        'initiator_ports' => [0c,0d,3a,3b],
        };
=over

=item Arguments

  Required: me => 1 or partner => 1

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $hashref = $mc->get_initiator_ports( me => 1 );
    @initiator_ports = @{$hashref->{initiator_ports}};
OR
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $hashref = $mc->get_initiator_ports( partner => 1 );
    @initiator_ports = @{$hashref->{initiator_ports}};


An example of printing all the FCVI ports on me filer
    my $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    my $hashref = $mc->get_initiator_ports( me => 1 );
    foreach my $port (@{$hashref->{initiator_ports}}){
      NOTE::logcomment("Initiator Port: $port\n");
    }

=cut

sub get_initiator_ports {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($initiator, @slots, %slots, @initiator_slots, @initiator_ports);
    
    if (exists($options{me})) {
        @slots = split(/slot/,$class->me->say('sysconfig -va'));
    }
    elsif(exists($options{partner})){
        @slots = split(/slot/,$class->partner->say('sysconfig -va'));        
    }
    else{
        croak("Insufficeint Parameter! Need either me=>1 or partner=>1 as argument");
    }

    if($class->is_fmc){
        foreach (@slots) { 
            if( $_ =~ /(\w+):\s+(FC|Fibre Channel)\s+Host Adapter\s+(\S+)\s+.*Switch Port:\s+(\S+):(\d+)/gs ) {
                push(@initiator_ports,$3);
                $slots{$1} = 1;
            }
        }        
    }
    else{
        foreach (@slots) { # $1 = slot number  $2 = port on filer such as 1a,1b
            if( $_ =~ /(\w+):\s+(FC|Fibre Channel)\s+Host Adapter\s+(\w+)\s+.*L-port/si ) {
                push(@initiator_ports,$3);
                $slots{$1} = 1;
            }
        }        
    }
    @initiator_slots = keys %slots;
    $initiator->{slots} = \@initiator_slots;
    $initiator->{initiator_ports} = \@initiator_ports;
    
    return $initiator;    
}

=head2 ::smc_rpower

Subroutine to perform rpower operations for a Stretched MetroCluster filer & its local shelves.

=over

=item Arguments

  Required: None 

B<device_rpower_names> - I<string>

I<Optional.>
String of comma separated rpower names for devices on SMC partner site to do rpower off (for simulating site disaster). 
if this argument is not provided, subroutine tries to fetch it from nate_host file of partner filer through SHELF_RPOWER_NAMES & FIlER_RPOWER_NAME parameters 

B<ignore_filer> - I<0|1>

I<Optional.>
If set to 1, filer is not considered for rpower operation. Ignored if device_rpower_names is not NULL. Default is 0 

B<partner> - I<0|1>

I<Optional.>
If set to 1, rpower operation is done on partner site else on local site. Ignored if device_rpower_names is not NULL. Default is 0 

B<rpower_cmd> => I<Absolute path to rpower command>

I<Optional.>
Defaults to '/usr/software/utils/bin/rpower'

B<client> => I<Client from which to issue rpower command>

I<Optional.>
Defaults to 'localhost'

B<on> => I<Flag to be set to 1 for rpower ON operation>

I<Optional.>
Defaults to 1

B<off> => I<Flag to be set to 1 for rpower OFF operation>

I<Optional.>
Defaults to 0

B<reset> => I<Flag to be set to 1 for rpower RESET operation>

I<Optional.>
Defaults to 0


=item Example

=back

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->smc_rpower;
OR

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->smc_rpower( ignore_filer => 1, off => 1);
OR

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->smc_rpower( device_rpower_names => "DS4243-1,DS4243-2,DS4243-3", off => 1);
    
OR

    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $mc->smc_rpower( partner => 1, off => 1);

=cut

sub smc_rpower{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($filer_rpower_name, $rpower_task);
    my ($device_rpower_names, @rpower_list, $filer, $filer_name);
    
    if($options{partner}){
        $filer_name = $class->partner->host;
        $filer = 'partner';
    }
    else{
        $filer_name = $class->me->host;
        $filer = 'me';
    }
                           
    #Gather rpower names
    if($options{device_rpower_names}){
        $device_rpower_names = $options{device_rpower_names};
    }
    else{
        my $hostrec = Hostrec->new( id => $filer_name);
        my %hostrec_fields = $hostrec->param_fields(); 
        $device_rpower_names = $hostrec_fields{SHELF_RPOWER_NAMES};
        croak ("Unable to fetch devices' rpower information") unless ($device_rpower_names);
        if(!$options{ignore_filer}){
            $filer_rpower_name = $hostrec_fields{FILER_RPOWER_NAME};
            $device_rpower_names = "$device_rpower_names".",$filer_rpower_name";            
            NOTE::logcomment("Setting AUTOBOOT bootarg variable to FALSE on $filer_name\n");
            $class->$filer->say("priv set -q diag; bootargs set AUTOBOOT false");
        }
    }    
    @rpower_list = split( /,/, $device_rpower_names );
    NOTE::logcomment("Rpower List: @rpower_list\n Proceeding with Rpower operation\n");                
    FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_list, %options );
}

=head2 ::cli_ic_stats_error

Runs "ic stats error -v" on the given filer

=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back

To do ic stats errov -v  on the filer
    $mc = NOTE::FcMetrocluster->new(Filer => $FILERA);
    $mc->cli_ic_stats_error( me => 1);

=cut

sub cli_ic_stats_error {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %options) = @_;
    my ($ic_stats );
    if(exists($options{me})) {
        $ic_stats = $class->me->say("priv set -q  advanced; ic stats error -v");
        sleep(20);
        return $ic_stats;
    }
    elsif(exists($options{partner})) {
        $ic_stats = $class->partner->say("priv set -q advanced ; ic stats error -v");
        sleep(20);
        return $ic_stats;
    }
}
=head2 ::cli_nic_reset_enable

Runs "ic enable nic_reset " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_nic_reset_enable(me=>1);

=cut

sub cli_nic_reset_enable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
        my $output;
    if(exists($options{me})) {
        $output = $class->me->say("priv set test; ic enable nic_reset");
        return $output;
    }
    elsif(exists($options{partner})){
        $output = $class->partner->say("priv set test; ic enable nic_reset");
        return $output;
    }
        else {
        croak "No option was provided\n";
    }
}

=head2 ::cli_nic_reset_disable

Runs "ic disable nic_reset " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_nic_reset_disable(me=>1);

=cut

sub cli_nic_reset_disable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
        my $output;
    if(exists($options{me})) {
        $output = $class->me->say("priv set test; ic disable nic_reset");
        return $output;
    }
    elsif(exists($options{partner})){
        $output = $class->partner->say("priv set test; ic disable nic_reset");
        return $output;
    }
        else {
        croak "No option was provided\n";
    }
}

=head2 ::cli_reset_rv

Runs "ic reset rv" on the given filer

=over

=item Arguments

  Required: me=>1 or partner=>1

=item Example

=back

To do ic reset rv on the filer
    $mc = NOTE::FcMetrocluster->new(Filer => $FILERA);
    $my $ic_reset_rv = $mc->cli_reset_rv( me => 1);

=cut

sub cli_reset_rv {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %options) = @_;
    my ($ic_reset_rv );
    if(exists($options{me})) {
        $ic_reset_rv = $class->me->say("priv set -q  diag; ic reset rv");
        sleep(20);
        return $ic_reset_rv;
    }
    elsif(exists($options{partner})) {
        $ic_reset_rv = $class->partner->say("priv set -q diag ; ic reset rv");
        sleep(20);
        return $ic_reset_rv;
    }
}
=head2 ::ic_dump_hw_stats

Run "ic dump hw_stats " on the given filer and return the result in a hash.

Following is the structure of hash returned:

 $VAR1 = {
          'Adapter_0' => {
                            'Invalid CRC Count' => '0',
                            'Link Failure Count' => '0',
                            'Invalid Transmission Word Count' => '0',
                            'Protocol Error Count' => '0',
                            'Loss of Signal Count' => '0',
                            'FW state' => 'ISP_STATE_READY'
                          },
           'Adapter_1' => {
                            'Invalid CRC Count' => '0',
                            'Link Failure Count' => '0',
                            'Invalid Transmission Word Count' => '0',
                            'Protocol Error Count' => '0',
                            'Loss of Signal Count' => '0',
                            'FW state' => 'ISP_STATE_READY'
                          }
         };
=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $hash_ref = $mc->ic_dump_hw_stats(me=>1);

=cut
sub cli_dump_hw_stats {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics
    my ( $class, %options ) = @_;
    my ( $hash, $hw_stats_output, $num );
    if ( exists( $options{me} ) ) {
        $hw_stats_output =
          $class->me->say("priv set -q  diag; ic dump hw_stats");
        sleep(20);
        foreach ( split( '[\n\r]', $hw_stats_output ) ) {
            if ( $_ =~ /\s*Stats\s*for\s*adapter\s*(\d)/i ) {
                $num = $1;
            }
            $hash->{"Adapter_$num"}->{"FW state"} = $1
              if ( $_ =~ /\s*FW\s*state:\s*(\w+)/i );
            $hash->{"Adapter_$num"}->{"Link Failure Count"} = $1
              if ( $_ =~ /\s*Loss\s*of\s*Sync\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Loss of Signal Count"} = $1
              if ( $_ =~ /\s*Loss\s*of\s*Signal\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Protocol Error Count"} = $1
              if ( $_ =~ /\s*Protocol\s*Error\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Invalid Transmission Word Count"} = $1
              if (
                $_ =~ /\s*Invalid\s*Transmission\s*Word\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Invalid CRC Count"} = $1
              if ( $_ =~ /\s*Invalid\s*CRC\s*Count\s*:\s*(\d+)/i );
        }
        return $hash;
    }
    elsif ( exists( $options{partner} ) ) {
        $hw_stats_output =
          $class->partner->say("priv set -q  diag; ic dump hw_stats");
        sleep(20);
        foreach ( split( '[\n\r]', $hw_stats_output ) ) {
            if ( $_ =~ /\s*Stats\s*for\s*adapter\s*(\d)/i ) {
                $num = $1;
            }
            $hash->{"Adapter_$num"}->{"FW state"} = $1
              if ( $_ =~ /\s*FW\s*state:\s*(\w+)/i );
            $hash->{"Adapter_$num"}->{"Link Failure Count"} = $1
              if ( $_ =~ /\s*Loss\s*of\s*Sync\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Loss of Signal Count"} = $1
              if ( $_ =~ /\s*Loss\s*of\s*Signal\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Protocol Error Count"} = $1
              if ( $_ =~ /\s*Protocol\s*Error\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Invalid Transmission Word Count"} = $1
              if (
                $_ =~ /\s*Invalid\s*Transmission\s*Word\s*Count\s*:\s*(\d+)/i );
            $hash->{"Adapter_$num"}->{"Invalid CRC Count"} = $1
              if ( $_ =~ /\s*Invalid\s*CRC\s*Count\s*:\s*(\d+)/i );
        }
        return $hash;
    }

}

=head2 ::cli_ic_channel_state

Runs "ic channel state " on the given filer and returns the result in a hash.

This command can be executed only in test privilege mode.

Following is the structure of hash returned:

 $VAR1 = {
           'channel_name' => 'VIA',
           'state' => {
                        'WAFL ' => 'UP',
                        'RAID ' => 'UP',
                        'BULK DATA' => 'UP',
                        'BULK CONTROL' => 'UP',
                        'STREAM0 ' => 'UP',
                        'KSTAT ' => 'UP',
                        'OFW ' => 'UP',
                        'MISC ' => 'UP',
                        'KILL ' => 'UP',
                        'STREAM1 ' => 'UP'
                      }
         };
=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $hash_ref = $mc->cli_ic_channel_state(me=>1);

=cut
sub cli_ic_channel_state {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;
    my ( $hash, $channel_state_output, $state, $name );
    if ( exists( $options{me} ) ) {
        $channel_state_output =
          $class->me->say("priv set -q  test; ic channel_state");
        sleep(20);
        foreach ( split( '[\n\r]', $channel_state_output ) ) {
            if ( $_ =~ /\s*(\w+)\s*Interconnect\s*Channels/i ) {
                $hash->{"channel_name"} = $1;
                next;
            }
            $hash->{"state"}->{"$1"} = $2
              if ( $_ =~ /\s*(\w+\s*\w*)\s*Channel\s*(\w+)/i );
        }
        return $hash;
    }
    elsif ( exists( $options{partner} ) ) {
        $channel_state_output =
          $class->partner->say("priv set -q  test; ic channel_state");
        sleep(20);
        foreach ( split( '[\n\r]', $channel_state_output ) ) {
            if ( $_ =~ /\s*(\w+)\s*Interconnect\s*Channels/i ) {
                $hash->{"channel_name"} = $1;
                next;
            }
            $hash->{"state"}->{"$1"} = $2
              if ( $_ =~ /\s*(\w+\s*\w*)\s*Channel\s*(\w+)/i );
        }
        return $hash;
    }
}

=head2 ::cli_ic_log_vi_error_enable

Runs "ic log_vi_error enable " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_ic_log_vi_error_enable(me=>1);

=cut

sub cli_ic_log_vi_error_enable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
        my $output;
    if(exists($options{me})) {
        $output = $class->me->say("priv set diag; ic log_vi_error enable");
        if ($output =~ /Enabled logging vi error/i) {
          NOTE::logcomment("Enabled logging vi error ! \n");
          return;
        }
       }
    elsif(exists($options{partner})){
        $output = $class->partner->say("priv set diag; ic log_vi_error enable");
        if ($output =~ /Enabled logging vi error/i) {
          NOTE::logcomment("Enabled logging vi error ! \n");
          return;
       }
      }
        else {
        croak "Not able to enable logging vi error\n";
    }
}

=head2 ::cli_ic_log_vi_error_disable

Runs "ic log_vi_error disable " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_ic_log_vi_error_disable(me=>1);

=cut

sub cli_ic_log_vi_error_disable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my($class,%options) = @_;
        my $output;
    if(exists($options{me})) {
        $output = $class->me->say("priv set diag; ic log_vi_error disable");
        if ($output =~ /Disabled logging vi error/i) {
          NOTE::logcomment("Disabled logging vi error ! \n");
          return;
         }
       }
    elsif(exists($options{partner})){
        $output = $class->partner->say("priv set diag; ic log_vi_error disable");
        if ($output =~ /Disabled logging vi error/i) {
          NOTE::logcomment("Disabled logging vi error ! \n");
          return;
         }
       }
        else {
        croak "Unable to disable loggin vi error\n";
    }
}
=head2 ::ic_disable_ofw

Runs "ic disable ofw " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_disable_ofw(me=>1);

=cut

sub ic_disable_ofw {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

   my ( $class, %options ) = @_;
   my ($timeout,$filer);

   if ( exists( $options{timeout} ) ) {
      $timeout = $options{timeout};
   } else {
      $timeout = 180;
   }
    if(exists($options{me})) {
        $filer = $class->me;
    }
    elsif(exists($options{partner})) {
        $filer = $class->partner;
    }

   $filer->say("priv set -q  diag; dbg level fm=1");
   sleep(5);
   $filer->say("priv set test; ic disable ofw");

     my $end_time =
        $timeout
        ? Tharn::timeout2time($timeout)
        : Tharn::timeout2time(-1);

    while ( time() < $end_time ) {
         my $time_remaining = $end_time - time();
         NOTE::logcomment("Timeout remaining: $time_remaining secs");
         sleep(60);
      }
    $filer->say("priv set -q  diag; dbg level fm=0");
}

=head2 ::ic_enable_ofw

Runs "ic enable ofw " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_enable_ofw(me=>1);

=cut

sub ic_enable_ofw {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

   my ( $class, %options ) = @_;
   my ($timeout,$filer);

   if ( exists( $options{timeout} ) ) {
      $timeout = $options{timeout};
   } else {
      $timeout = 180;
   }
    if(exists($options{me})) {
        $filer = $class->me;
    }
    elsif(exists($options{partner})) {
        $filer = $class->partner;
    }

   $filer->say("priv set -q  diag; dbg level fm=1");
   sleep(5);
   $filer->say("priv set test; ic enable ofw");

     my $end_time =
        $timeout
        ? Tharn::timeout2time($timeout)
        : Tharn::timeout2time(-1);

    while ( time() < $end_time ) {
         my $time_remaining = $end_time - time();
         NOTE::logcomment("Timeout remaining: $time_remaining secs");
         sleep(60);
      }
    $filer->say("priv set -q  diag; dbg level fm=0");
}

=head2 ::cf_ic_verify

This method calls cf_ic_status_check and wait till cf gets enable

=over

=item Arguments

B<timeout> => I<timeout in seconds>

I<Optional>
Timeout in seconds. default is set to 300 seconds

B<udpm>    => I<1>

I<Optional>
Verify connection roles of rvs when udpm is enabled

=item Return Value

1 on success, 0 otherwise

=item Examples

=back

        $mc->cf_ic_verify ;

=cut

sub cf_ic_verify {

    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($class, %args) = @_;
    my $TIMEOUT = $args{timeout}? $args{timeout} : 300 ;
   my $status = 0;

   my $end_time =
     $TIMEOUT ? Tharn::timeout2time($TIMEOUT) : Tharn::timeout2time(-1);

   while ( time() < $end_time ) {
      my $time_remaining = $end_time - time();
      NOTE::logcomment("Timeout remaining : $time_remaining secs\n");
      eval {
            $class->cf_ic_status_check(me=>1);
           };
      if($@) {
              NOTE::logcomment("Checking status again..\n");
             }else {
                    NOTE::logcomment("Cluster enabled\n");
                    $status=1;
                    last;
                    }
       sleep(15);
       }

      if(!$status) {
                    croak "Unable to recover from unsynchronized state \n";
                   }

   if($args{udpm}) {
   my($checkval_1, $checkval_2);
   if($class->{me_fabrics}->{sysid} > $class->{partner_fabrics}->{sysid} ) {
        $checkval_1 = "source_rv";
        $checkval_2 = "dest_rv";
    } else {
        $checkval_1 = "dest_rv";
        $checkval_2 = "source_rv";
    }
   my $udpm_link_me      = $class->ic_status(me      => 1);
   my $udpm_link_partner = $class->ic_status(partner => 1);

    if(   $udpm_link_me->{'cfo_rv'}->{'role'} =~ /$checkval_1/i 
       && $udpm_link_me->{'cfo_rv1'}->{'role'} =~ /$checkval_2/i 
       && $udpm_link_partner->{'cfo_rv'}->{'role'} =~ /$checkval_2/i
       && $udpm_link_partner->{'cfo_rv1'}->{'role'} =~ /$checkval_1/i )
    {    
        NOTE::logcomment(
            "Link verification done when UDPM is enabled\n"
        );
    } else {
        croak(
            "Link verification failed when UDPM enabled\n"
        );
    }

  }
   return 1;
}

=head2 ::get_aggr_disks

Runs aggr status -r for given aggregate and pulls out list of disks in Pool0 & Pool1 of
the aggregate
if me=>1, it will pull info from the provided filer. if partner=>1, it will pull 
info from the partner of provided filer. It returns a hash which contains the disk lists 
keyed using Pool number. Following is the structure of the hash:
 
        $VAR1 = {
        'pool0' => [3b.125L28, 3b.125L29, 3b.125L29 ],
        'pool1' => [3d.125L89, 3d.125L90, 3d.125L91 ],
        };
=over

=item Arguments

  Required: me => 1 or partner => 1
  Required: aggr_name

=item Example

=back
    
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $hashref = $mc->get_aggr_disks( me => 1 , aggr_name => aggr0 );
    @root_pool0_disks = @{$hashref->{pool0}};
OR
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $hashref = $mc->get_aggr_disks( partner => 1 , aggr_name => aggr1 );
    @aggr1_pool0_disks = @{$hashref->{pool0}};


An example of printing all the root aggr disks in pool0 on me filer
    my $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    my $hashref = $mc->get_aggr_disks( me => 1 , aggr_name => aggr0);
    foreach my $disk (@{$hashref->{pool0}}){
      NOTE::logcomment("Pool0 root aggr disk: $disk\n");
    }

=cut

sub get_aggr_disks {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%options) = @_;
    my ($aggr, @cmd_output, $aggr_disks, @pool0_disks, @pool1_disks);
    
    if(exists($options{aggr_name})){
        $aggr = $options{aggr_name};
    }
    else{
        croak("Insufficeint Parameter! Need aggr_name as argument");
    }
    if (exists($options{me})) {
        @cmd_output = split('[\n\r]',$class->me->say("aggr status -r $aggr"));
    }
    elsif(exists($options{partner})){
        @cmd_output = split('[\n\r]',$class->partner->say("aggr status -r $aggr"));        
    }
    else{
        croak("Insufficeint Parameter! Need either me=>1 or partner=>1 as argument");
    }
    
    #match the lines with disk and pool information and push disk name to an array
    foreach (@cmd_output){
        if($_ =~ /\s+\w+\s+(.+\.\w+)\s+\w+\s+\d+\s+\d+\s+FC:(A|B)\s+(\d)\s+/){
            if($3 == '1'){
                push(@pool1_disks, $1);
            }
            else{
                push(@pool0_disks, $1);
            }            
        }##Match for glowworm
        elsif($_ =~ /\s+\w+\s+(.+\.\d+\.\d+)\s+\w+\s+\d+\s+\d+\s+SA:(A|B)\s+(\d)\s+/){
            if($3 == '1'){
                push(@pool1_disks, $1);
            }
            else{
                push(@pool0_disks, $1);
            }
        }    
    }
    
    $aggr_disks->{'pool0'} = \@pool0_disks;
    $aggr_disks->{'pool1'} = \@pool1_disks;
    
    return $aggr_disks;   
}

=head2 ::cli_ic_reset_loop

Runs "ic reset loop start/stop " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1
  Required: flag=>'start' or flag=>'stop'

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_ic_reset_loop(me=>1, flag=>start);

=cut

sub cli_ic_reset_loop {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;
    my ( $output, $filer );
    $filer = $class->partner;
    $filer = $class->me if ( $options{me} );

    $output = $filer->say("priv set -q diag; ic reset loop $options{flag}");
    if ( $options{flag} =~ /start/i && $output =~ /Starting reset loop/i ) {
        NOTE::logcomment("ic reset loop started ! \n");
        return;
    }
    elsif ( $options{flag} =~ /stop/i && $output =~ /Stopping reset loop/i ) {
        NOTE::logcomment("ic reset loop stopped ! \n");
        return;
    }
    else {
        croak "Unable to start/stop ic reset loop";
    }
}

=head2 ::cli_ic_disable_nic

Runs "ic disable nic <nic_num> " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1 nic=>0|1


=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_ic_disable_nic(me=>1,  nic=>0);

=cut

sub cli_ic_disable_nic {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;
    my ( $output, $nic_num );

    if ( exists( $options{nic} ) ) {
        $nic_num = $options{nic};

        if ( exists( $options{me} ) ) {
            $output =
              $class->me->say("priv set -q test; ic disable nic $nic_num");
            return $output;
        }
        elsif ( exists( $options{partner} ) ) {
            $output =
              $class->partner->say("priv set -q test; ic disable nic $nic_num");
            return $output;
        }
    }
    else {
        croak "No option was provided\n";
    }
}

=head2 ::cli_ic_enable_nic

Runs "ic enable nic <nic_num> " on the given filer and returns the result.

=over

=item Arguments

  Required: me=>1|partner=>1 nic=>0|1


=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_ic_enable_nic(me=>1,  nic=>0);

=cut

sub cli_ic_enable_nic {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;
    my ( $output, $nic_num );

    if ( exists( $options{nic} ) ) {
        $nic_num = $options{nic};

        if ( exists( $options{me} ) ) {
            $output =
              $class->me->say("priv set -q test; ic enable nic $nic_num");
            return $output;
        }
        elsif ( exists( $options{partner} ) ) {
            $output =
              $class->partner->say("priv set -q test; ic enable nic $nic_num");
            return $output;
        }
    }
    else {
        croak "No option was provided\n";
    }
}

=head2 ::cli_ic_nvram

Runs "ic nvram " on the given filer and returns the result in a hash.

This command can be executed only in test privilege mode.

Following is the structure of hash returned:

 $VAR1 = {
           'Interconnect_port2_state' => '4x copper',
           'Memory_Size' => '2048 MB',
           'nv_lcl_size' => '999 MB',
           'Interconnect_port1_state' => '4x copper',
           'DIMM_Size' => '2048 MB',
           'nv_size' => '1998 MB',
           'Battery_Status' => 'Battery sufficiently charged (3942 mV)',
           'Firmware_version' => '4 (4.8.940)'
         };
=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->cli_ic_nvram(me=>1);

=cut

sub cli_ic_nvram {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;
    my ( $nvram_status, $hash, $filer );
    if ( exists( $options{me} ) ) {
        $filer = $class->me;
    }
    elsif ( exists( $options{partner} ) ) {
        $filer = $class->partner;
    }
    else {
        croak "No option given";
    }

    $nvram_status = $filer->say("priv set -q  test; ic nvram");

    foreach ( split( '[\n\r]', $nvram_status ) ) {
        $hash->{"Memory_Size"} = $1
          if ( $_ =~ /\s*Memory\s*Size:\s*(\d+\s*MB)/i );
        $hash->{"DIMM_Size"} = $1 if ( $_ =~ /\s*DIMM\s*Size:\s*(\d+\s*MB)/i );
        $hash->{"Battery_Status"} = $1
          if ( $_ =~ /\s*Battery1\s*Status:\s*([\w+\W+]*)/i );
        $hash->{"Firmware_version"} = $1
          if ( $_ =~ /\s*Running\s*Firmware:\s*([\w+\W+]*)/i );
        $hash->{"Interconnect_port1_state"} = $1
          if ( $_ =~ /\s*Cluster\s*Interconnect\s*Port\s*1:\s*([\w+\W+]*)/i );
        $hash->{"Interconnect_port2_state"} = $1
          if ( $_ =~ /\s*Cluster\s*Interconnect\s*Port\s*2:\s*([\w+\W+]*)/i );
        if ( $_ =~
/\s*The\s*NVRAM\s*card\s*is\s*in\s*slot\s*(\d+),\s*and\s*the\s*required\s*physical\s*HA\s*slot\s*is\s*(\d+)/i
          )
        {
            $hash->{"NVRAM_slot"} = $1;
            $hash->{"HA_slot"}    = $2;
        }
        $hash->{"nv_size"} = $1
          if ( $_ =~ /\(nv_size\):\s*0[x][\da-x]+\s*\((\d+\s*MB)\)/i );
        $hash->{"nv_lcl_size"} = $1
          if ( $_ =~ /\(nv_lcl_size\):\s*0[x][\da-x]+\s*\((\d+\s*MB)\)/i );
    }
    return $hash;
}

=head2 ::cli_ic_timeout

Runs "ic ic_timeout " on the given filer and returns the timeout value.


=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $timeout_value = $mc->cli_ic_timeout(me=>1);

=cut

sub cli_ic_timeout {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;
    my ( $output, $filer );
    if ( exists( $options{me} ) ) {
        $filer = $class->me;
    }
    elsif ( exists( $options{partner} ) ) {
        $filer = $class->partner;
    }
    else {
        croak "No option given";
    }
    $output = $filer->say("priv set -q diag; ic ic_timeout");
    if ( $output =~ /Current\s*ic_timeout\(msecs\):\s*(\d+)/i ) {
        return $1;
    }
    else { croak "Unable to fetch ic timeout value"; }
}

=head2 ::ic_udpm_status

Runs "ic unidirectional_mirroring status" on the given filer and populate all the information into a hash.

 $VAR1 = {
    'UDPM Status'           => 'enabled',
    'Outgoing_primary_link' => 0,
    'Outgoing_link_in_use'  => 0,
    'Incoming_primary_link' => 0,
    'Incoming_link_in_use'  => 0,
  }
  

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

 $mc = NOTE::FcMetrocluster->new(Filer =>$FILER);
 my $hash = $mc->ic_udpm_status(me=>1);

=cut

sub ic_udpm_status {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;
    my ( $hash, $ic_udpm_output, $temp );

    if ( exists( $options{partner} ) ) {
        $temp = 'partner';
    }
    else { $temp = 'me'; }

    $ic_udpm_output = $class->$temp->say("ic unidirectional_mirroring status");

    foreach ( split( '[\n\r]', $ic_udpm_output ) ) {    #split line by line

        $hash->{'UDPM Status'} = $1
          if ( $_ =~ /Unidirectional\s+mirroring\s+is\s+(\w+)/i );

        if ( $_ =~
/outgoing\s+data\s+primary\s+link\s+is\s+(\d)\s+and\s+current\s+link\s+in\s+use\s+is\s+(\d)/i
          )
        {
            $hash->{'Outgoing_primary_link'} = $1;
            $hash->{'Outgoing_link_in_use'}  = $2;
        }
        if ( $_ =~
/incoming\s+data\s+primary\s+link\s+is\s+(\d)\s+and\s+current\s+link\s+in\s+use\s+is\s+(\d)/i
          )
        {
            $hash->{'Incoming_primary_link'} = $1;
            $hash->{'Incoming_link_in_use'}  = $2;
        }

    }

    return $hash;

}

=head2 ::ic_udpm_on

Runs "ic unidirectional_mirroring on" on the given filer and runs "ic unidirectional_mirroring status" to make sure
that udpm is enabled.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_udpm_on(me=>1);

=cut

sub ic_udpm_on {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;

    my ( $hash, $temp );

    if ( exists( $options{partner} ) ) {
        $temp = 'partner';
    }
    else { $temp = 'me'; }

    $class->$temp->say("priv set -q admin; ic unidirectional_mirroring on ");
    sleep 15;
    my $hash = $class->ic_udpm_status( $temp => 1 );
    if ( $hash->{"UDPM Status"} =~ /enabled/i ) {
        NOTE::logcomment("UDP Mirroring status : Enabled\n");
        NOTE::logcomment ("Outgoing Primary Link:\n Default: $hash->{'Outgoing_primary_link'} \n In Use: $hash->{'Outgoing_link_in_use'}");
        NOTE::logcomment ("Incoming Primary Link:\n Default: $hash->{'Incoming_primary_link'} \n In Use: $hash->{'Incoming_link_in_use'}");
    }
    else {
        croak "Could not Enable UDP Mirroring: $@ \n";
    }
}

=head2 ::ic_udpm_off

Runs "ic unidirectional_mirroring off" on the given filer and runs "ic unidirectional_mirroring status" to make sure
that udpm is enabled.

=over

=item Arguments

  Required: me=>1|partner=>1

=item Example

=back

    my $mc = NOTE::FcMetrocluster->new(Filer=>$FILER);
    $mc->ic_udpm_off(me=>1);

=cut

sub ic_udpm_off {
    my $trace = $NOTE::TRACELOG->start_timer();    # log internal metrics

    my ( $class, %options ) = @_;

    my ( $hash, $temp );

    if ( exists( $options{partner} ) ) {
        $temp = 'partner';
    }
    else { $temp = 'me'; }

    $class->$temp->say("priv set -q admin; ic unidirectional_mirroring off ");
    sleep 15;
    my $hash = $class->ic_udpm_status( $temp => 1 );
    if ( $hash->{"UDPM Status"} =~ /disabled/i ) {
        NOTE::logcomment("UDP Mirroring status : Disabled\n");
        NOTE::logcomment ("Outgoing Primary Link:\n Default: $hash->{'Outgoing_primary_link'} \n In Use: $hash->{'Outgoing_link_in_use'}");
        NOTE::logcomment ("Incoming Primary Link:\n Default: $hash->{'Incoming_primary_link'} \n In Use: $hash->{'Incoming_link_in_use'}");
    }
    else {
        croak "Could not Enable UDP Mirroring: $@ \n";
    }
}

=head2 ::system_id

Returns system id of the node

=over

=item Examples

   $mc->system_id(me=>1);

=back

=cut

sub system_id {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($class,%options) = @_;
    if(exists($options{me})) {
    return $class->{me_fabrics}->{sysid};
    } else { 
    return $class->{partner_fabrics}->{sysid};
    }
}
=head1 AUTHOR

dl-dpgqa-automation@netapp.com

amit3@netapp.com

jawwad@netapp.com


=cut

sub DESTROY {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    NOTE::logcomment("FcMetrocluster Destroy has been called\n");
    return if ( $$ != $self->{_owner} );
    return if ( $self->{_destroyed} );

    $self->{_destroyed} = 1;
    NOTE::logcomment("FcMetrocluster:: DESTROY has completed");
    NOTE::logcomment( "Fc Disconnecting from the Filer ".$self->{me}->host );
    NOTE::logcomment( "Fc Disconnecting from the Filer ".$self->{partner}->host );
    Tharn::disconnect( $self->{_hostconn} ) if ( $self->{_hostconn} );

}

1;

