# -*- Perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/NOTE/FcSwitch.pm#1 $
#
## @summary  NOTE Fibre Channel Switch class
## @status   Public
## @pod      here
## @author   dl-dpgqa-automation@netapp.com
##           cpeter@netapp.com
##           jawwad@netapp.com

=head1 NAME

NOTE::FcSwitch

=head1 DESCRIPTION

This is the NOTE class for working with Brocade FC Switches

=head1 METHODS

=cut

package NOTE::FcSwitch;
our @ISA = qw(NOTE);
use strict;
use Data::Dumper;
use Tharn;
use NOTE::Registry;
use Carp qw(croak);
use San::SanSwitch;
use San::SanLog;
use San::SanSwitch::SwitchUtil;
use San::SanSwitch::PortMgr;
use San::SanSwitch::ZoneMgr;
use San::SanCache;
use Error;
use San::SanSwitch::FC::PortMgr::CiscoFCPort;
use dp::fmc::FcMetroclusterUtils;

# NOTE: Do *NOT* import any external functions into this module!
# This is to avoid namespace collisions namely prototype mismatches and
# all that. Call the external function directly by prepending the package
# name.
use Tharn qw();
use Net::Ping qw();
use Date::Manip qw();
my %_SWITCH_CONNS = ();

# use Auto loader to get quazi-inheritance with switch.pm
our $AUTOLOAD;

sub AUTOLOAD {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $name = $AUTOLOAD;
    my ( $self, @args ) = @_;

    NOTE::logcomment("ORIGINAL CALL: $name");
    $name =~ s/.*:/switch::switch_/;

    NOTE::logcomment("Should call:  $name");

    {
        no strict 'refs';
        &$name( $self->conn, @args );
    };
}

=head2 ::connect

Creates an object representing a switch
Uses L<C<Tharn::connect>|engineering::nate::releases::latest::html::Tharn/connect>
to establish a connection to the switch

=over

=item Arguments

B<host> => I<host spec>

I<Required.>
The host spec for a switch

B<mode> => C<#rsh> | C<#telnet> | C<#console>

I<Optional.>
The connection method to use.
The default is C<#telnet>.
Very rarely is this option needed.

=item Examples

=back

     $sw = NOTE::FcSwitch->connect(
         host => $SWITCH,
         mode => '#console',
     );

=cut

sub connect {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $class = shift;
    my %args;
    my $hosttype;

    # Allow for single argument to specify hostspec:
    if ( scalar @_ == 1 ) {
        $args{host} = shift;
    } else {
        %args = @_;
    }
    my $self = {};
    my $host = $args{host};    
    $args{timeout} = $args{timeout} || 300;
    #$args{host}        ||= q/$FILER/;
    $self->{_owner}    = $$;
    $self->{_debug}    = $args{debug};

    #Start SMC Specific
    #Workaround for SMC configurations where existing MC scripts call for 
    #NOTE::FcSwitch->connect for creating switch objects just after 
    #creating FcMetroCluster object. 
    #For SMC, $args{host} parameter will be filer name itself. If found, create a 
    #dummy object and return gracefully.
    #Note that this is a workaround only for existing FMC scripts to work in SMC too.
    #Any call other than connect, reconnect, switch_disable, switch_enable will fail ungracefully for SMC
    
    if ($host =~ /SMC_CONFIGURATION/){
        #Populate a dummy object for second switch call for SMC
        $self->{_host} = $args{host};
        bless $self, $class;
        $self->{_Switch} = $self;
        NOTE::reference($self);
        return $self;
    }
    else{ #Generic code for both FMC & SMC
        
        $self->{_hostspec} = NOTE::option( $args{host} )
          or croak "Host specification '"
          . $args{host}
          . "' doesn't resolve to anything";
        $self->{_hostrec} = Tharn::host( $self->{_hostspec} );
    
        $hosttype = $self->{_hostrec}->hosttype;
    }    
    
    if ($hosttype =~ /filer-cfe/i){
        #Populate an object with shelf rpower information for 1st switch call of SMC
        NOTE::logcomment("Host Type: $hosttype");
        NOTE::logcomment("Looks like a NOTE::FcSwitch\-\>connect call for SMC Testbed! Returning a dummy object!");
        $self->{_host} = $args{host};
        my %hostrec_fields = $self->{_hostrec}->param_fields(); 
        $self->{_shelf_rpower_list} = $hostrec_fields{SHELF_RPOWER_NAMES};
        $self->{_filer_rpower_name} = $hostrec_fields{FILER_RPOWER_NAME};
        $self->{_defaultip} = $self->{_hostrec}->default_ip();
        $self->{_conn} = Tharn::connect( $args{mode} || "#rsh" )
            or croak "Can't connect to " . $self->{_hostspec};
        
        bless $self, $class;
        $self->{_Switch} = $self;
        NOTE::reference($self);
        return $self;
    }
    #End SMC Specific
    
    NOTE::logcomment("host type for the switch is $hosttype \n");
    if ( $hosttype !~ /brocade/ ) {
        my $non_brocade_self = new SanSwitch( -host => $host, -persistent => 1 );
        $self->{_non_brocade_object}      = $non_brocade_self;        
        # $self->{_non_brocade_port_object} = $non_brocade_self->create_portmgr;
        
        my @port_info;
        my ( @eports, @port_details,@ports_fc,$no_of_ports );
        @port_info = $self->{_non_brocade_object}->execute("show int brief");
        my $i = 0;        
        #Populating the switch object with E-port information for Cisco switches only
            foreach (@port_info) {
               if ( $_ =~ /^fc/ ) {
                   @port_details = split( /\s+/, $_ );
                   push @ports_fc,$port_details[0];
                  # print "\n\n Port details are :- $port_details[0]\n\n";
                    if ( $port_details[6] eq 'TE' ) {
                        $eports[$i] = $port_details[0];
                        $i++;
                    }
                }
            }
           $self->{eports} = \@eports;
           $no_of_ports = scalar (@ports_fc);
           $self->{_no_of_ports}=$no_of_ports;                        		   
=head Old CODE written by Vivek 
       Below code snippet was used earlier. This is no longer required, but retained for any future use.
       $self->{_non_brocade_object}->execute("show int brief");
        
       $self->{_non_brocade_zone_object} = $non_brocade_self->create_zonemgr;
       if ( $hosttype =~ /cisco/ ) {
            my @port_info =
              $self->{_non_brocade_object}->execute("show int brief");
           my $i = 0;
           my ( @eports, @port_details );

       #Populating the switch object with E-port information for Cisco switches only
            foreach (@port_info) {
               if ( $_ =~ /^fc/ ) {
                   @port_details = split( /\s+/, $_ );
                    if ( $port_details[6] eq 'TE' ) {
                        $eports[$i] = $port_details[0];
                        $i++;
                    }
                }
            }
           $self->{eports} = \@eports;
		}
	   # End of earlier used code. 	
=cut
    } #end if ( $hosttype !~ /brocade/ )
    
    $self->{_defaultip} = $self->{_hostrec}->default_ip();
    $self->{_conn} = Tharn::connect( $args{mode} || "#telnet" )
        or croak "Can't connect to " . $self->{_hostspec};        

    # $self->{isl_info}->{_isl_port_num} = undef; # remembers the isl_show port

    # If there already was an object with this same connection object, return
    # a reference to it, rather than creating a new one. This is useful
    # because when this object gets destroyed, we know it is the last reference,
    # so we can print out status before actually disconnecting.
    return $_SWITCH_CONNS{"$self->{_conn}"}
      if ( $_SWITCH_CONNS{"$self->{_conn}"} );

    NOTE::logcomment(
        "Connected to switch using hostspec " . $self->{_hostspec} );
    $self->{_host} = $self->{_hostrec}->hostname();

    bless $self, $class;
    $self->{_Switch} = $self;
    NOTE::reference($self);

    $_SWITCH_CONNS{"$self->{_conn}"} = $self;
    $self->version;

    return $self;

}

=head2 ::reconnect

Reconnect to host with different connection method. Default method is '#telnet'.
Convinient way to switch from RSH to CONSOLE and back without need to create
extra instance of switchs object.
This method effects _hostrec and _conn private variables.

=over

=item Arguments

B<mode> => C<#rsh> | C<#telnet> | C<#console>

I<Optional.>
The connection method to use.
The default is C<#telnet>.

=item Examples

=back

    $switch->connect( host=>'brocade1', mode=>'#telnet' );
    ... do something ...
    # switch connection to console
    $switch->reconnect( mode=>'#console' );
    # bring network down
    # switch connection back to telnet
    $switch->reconnect();

=cut

sub reconnect {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, %args ) = @_;

    $self->{_hostrec} = Tharn::host( $self->{_hostspec} );
    $self->{_conn} = Tharn::connect( $args{mode} || "#telnet" )
      or croak "Can't connect to " . $self->{_hostspec};
    return $self;
}

=head2 ::portcfgspeed

Runs portcfgspeed command on a switch and calls port_ino method to make sure
the given speed is properly set

=over

=item Arguments

  Required: portnumber,portspeed (0, 1, 2, 4)

=item Example

=back

    my $sw = NOTE::FcSwitch->connect(host =$SWITCH);
    $sw->portcfgspeed(port=>$port , speed =>$speed);

=cut

sub portcfgspeed {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, %args ) = @_;

    my $port  = $args{port};
    my $speed = $args{speed};
    my ($post_speed);
    if ( defined($port) && defined($speed) ) {
        if ( $self->{_hostrec}->hosttype !~ /brocade/ ) {
			#$self->{_non_brocade_port_object}
            #  ->set_port_speed( port => $port, speed => $speed );
			
			NOTE::logcomment("port = $port speed = $speed\n");
            # In Cisco, the speed's are in MBPS, Speed passed through sciprts are in GBPS. Hence converting speed in MBPS
	        $speed = $speed * 1000;
            $self->{_non_brocade_object}->execute("configure t ;int $port ;switchport speed $speed ;end ;");
            sleep 1;
            $self->{_non_brocade_object}->execute("show int $port ;");       		
	}

=head Old Code  
			### Previously used code. No longer used. Retained for any future use
			#$self->{_non_brocade_port_object}
            # ->set_port_speed( port => $port, speed => $speed );
			#$post_speed = $self->portcfgspeed_show( port => $port );
   			# NOTE::logcomment("port speed has been set to $post_speed \n");
			### 
=cut 

         else {
            NOTE::logcomment("port = $port speed = $speed\n");
            my $output = $self->say("portcfgspeed $port $speed");
            sleep 10;
            $post_speed = $self->portcfgspeed_show( port => $port );
            if ( $output =~ /Same configuration for port/ ) {
                NOTE::logcomment("Same configuration for port ($port) found\n");
                return;
            }
            if ( $speed == 0 ) {
                if ( $post_speed =~ /N(\d+)/ ) {
                    NOTE::logcomment(
                        "port speed has been set to $post_speed \n");
                }
                return;
            } else {
                $speed .= 'G';
                croak "speed not set :$@" unless ( $post_speed == $speed );
            }
        }
    } else {
        croak "Port number or speed not passed ";
    }
}

=head2 ::ping

Ping switch on default ip. Uses Perl 'Net::Ping->new->ping' method. All arguments
are passed to Net::Ping as is.

=cut

sub ping {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    Net::Ping->new('external')->ping( shift->conn->hostp->default_ip, @_ );
}

=head2 ::say

Sends a command to the switchs CLI.
Returns the output from the command. This
is essentially a wrapper around Tharn::say command

=over

=item Arguments

Takes same arguments as say command accept hostrec.

=item Examples

=back

     @rawout = $switch->say( 'switchshow', -timeout, 120 );

=cut

sub say {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, @args ) = @_;
    my $match;
    if ( $self->{_hostrec}->hosttype !~ /brocade/ ) {
        $match = $self->{_non_brocade_object}->execute(@args);                          
    } else {
        ( undef, undef, $match, undef ) = Tharn::say( $self->conn, @args );
    }
    return $match;
}

=head2 ::version

Returns the hash reference to Fabric OS, Flash, Made, Kernel and Boot Prom version running on the switch.

=over

=item Arguments

None.

=item Examples

=back

     print Dumper( $switch->version );

Output:

      $VAR1 = {
                'Fabric OS' => 'v6.0.0b',
                'Flash' => 'Mon Mar 10 19:42:45 2008',
                'Made on' => 'Fri Jan 18 21:32:16 2008',
                'Kernel' => '2.6.14.2',
                'BootProm' => '4.6.6'
              };

=cut

sub version {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


    my $self = shift;
    if ( $self->{_hostrec}->hosttype !~ /brocade/ ) {
        my $version = $self->{_non_brocade_object}->get_firmware_version;
        return $version;
    }

    else {
        my $info = undef;
        foreach (
            split( '[\n\r]', $self->say('version') . $self->say('switchshow') )
          )
        {
            s/\s+$//g;
            $$info{$1} = $2 if (/^(\w+\s*\w*\s*\w*):\s+(\w+.*)/);
        }
        return $info;
    }
}

=head2 ::conn

Returns the Tharn connection object created
when L<C<::connect>|/::connect> was called.
This is for use with non-NOTE libraries.

=over

=item Arguments

None.

=item Examples

=back

     $conn = $switch->conn;

=cut

sub conn {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 return shift->{_conn} }

=head2 ::hostrec

Returns the host rec for the switch.
This is for use with non-NOTE libraries.

=over

=item Arguments

None.

=item Examples

=back

     $hostrec = $switch->hostrec;

=cut

sub hostrec {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 return shift->{_hostrec} }

=head2 ::hostspec

Returns the host spec for the switch.
This is for use with non-NOTE libraries.

=over

=item Arguments

None.

=item Examples

=back

     $hostspec = $switch->hostspec;

=cut

sub hostspec {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 return shift->{_hostspec} }

=head2 ::defaultip

Returns the ip addr of the switch.

=over

=item Arguments

None.

=item Examples

=back

     $defaultip = $switch->defaultip;

=cut

sub defaultip {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 return shift->{_defaultip} }

sub domain_id_change {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self      = shift;
    my $DOMAIN_ID = shift;
    $self->switch_disable;
    my ( @output, @checker, $found_flag, $iter );
    @output = Tharn::say( $self->conn, "configure", "Fabric parameters",
        -noprompt, -timeout, 180 );
    if ( $output[0] =~ /Fabric parameter/i ) {
        Tharn::say( $self->conn, 'yes', 'Domain: .*', -noprompt, -timeout,
            180 );
    }
    Tharn::say( $self->conn, "$DOMAIN_ID\r", -noprompt, -timeout, 180 )
      ;     #enter the given domain id
    do {    #continue to hit enter until you find the changes
        @checker =
          Tharn::say( $self->conn, "\r", '\[\S+\]', -noprompt, -timeout, 180 );
        foreach $iter (@checker) {
            if ( $iter =~
/WARNING: The domain ID will be changed. The port level zoning may be affected/i
              )
            {
                $found_flag = 1;
            }
            sleep(1);
        }
    } while ( $found_flag != 1 );
    if ( $found_flag == 1 ) {
        NOTE::logcomment("Domain ID was changed successfully! \n");
        return $found_flag;
    } else {
        croak "Domain ID could not be changed! \n";
    }
}

=head2 ::host

Returns the host name of the switch.

=over

=item Arguments

None.

=item Examples

=back

     $hostname = $switch->host;

=cut

sub host {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 return shift->hostrec->hostname }

=head2 ::tag

A tag is a text description for an object.
It is only stored in the object referring to the entity.
The purpose of the tag is to improve readability of the log.
If an object is tagged,
NOTE will include the tag in comments it logs about the object.

Returns the tag for the object.

=over

=item Arguments

I<string>

I<Optional.>
A tag to store in the object.

=item Examples

=back

     $switch->tag( 'The unzoned switch' );
     printf( "This is %s\n", $switch->tag );

=cut

sub tag {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $newtag ) = @_;

    if ( defined $newtag ) {
        $self->{_tag} = $newtag;
    }
    return $self->{_tag};
}

=head2 ::description

Returns a description of the switch consisting of
its switchs host spec, its connection mode and
the tag if any.

=over

=item Arguments

None

=item Examples

=back

=cut

sub description {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my $txt;

    $txt = $self->host;
    if ( $self->{_tag} ) {
        $txt .= " (" . $self->{_tag} . ")";
    }
    return $txt;
}

=head2 ::powercycle

Temporarily shuts off the power to the switch.
If called with no arguments
returns the switch to a normal up and running state.
The arguments control the selection
of other states to put the switch in
and the process by which states are attained.

=over

=item Examples

=back

 $switch->powercycle;


=cut

sub powercycle {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self) = shift;

    $self->{_hostrec}->powercycle;
}

=head2 ::reboot

Reboots the switch if it is up or boots it if it was not up
and returns the switch to a normal up and running state.
The arguments control the process by which states are attained.

=over

=item Examples

=back

     $switch->reboot;

=cut

#author: jawwad@netapp.com

sub reboot {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, %args ) = @_;
    my $timeout = $args{timeout} || 120;

    if ( $self->{_hostrec}->hosttype !~ /brocade/ ) {
         $self->{_non_brocade_object}->execute("no terminal dont-ask");
         $self->{_non_brocade_object}->reboot;
    } 
    else {

        my $switch_hostspec = $self->hostspec();
        my $switch = Tharn::host($switch_hostspec);
        my $switch_conn = Tharn::connect("#telnet");

       my ($output, $prompt);
       ( undef, undef, $output, $prompt ) = Tharn::say ($self->conn, "reboot", 
                                '-prompt',"reboot the switch [y/n]?",
                                '-timeout', $timeout );                                            
        if ( $output =~ /The system is going down for reboot NOW !!/ ) {
            NOTE::logcomment("Switch Rebooting...\n");
        } # Burt#487578 Handle the confirmation prompt on reboot for switches with FOS > 6.3.1
        elsif( $output =~ /Are you sure you want to /i){
           eval {
            ( undef, undef, $output, $prompt ) = Tharn::say($switch_conn, "y", '-noprompt', 
                                        '-prompt', "y", '-timeout', $timeout);
            };
            if( $output =~ /The system is going down for reboot NOW !!/ ) {
                NOTE::logcomment("Switch Rebooting...\n");
            }             
        }
        elsif (
            (
                $output =
                $self->say("switchreboot") =~
                /The system is going down for reboot NOW !!/
            )
          )
        {
            NOTE::logcomment("Switch Rebooting...\n");
        } else {
            croak "Switch reboot was not successfull.\n";
        }
        Tharn::disconnect($switch_conn);
        
        #Return the call immediately if validation for switch to come up after reboot
        #is not needed such as when we need to reboot both switches in a fabric at once.
        #Otherwise validate that switch comes up after reboot successfully.
        if($args{NO_REBOOT_VALIDATION}){
            return;
        }
        else{
            #Let's wait for switch to come up
            NOTE::logcomment("Let's wait for $timeout seconds for switch to reboot\n");
            #Ping the switch evey 5 seconds while we wait
            my $wait_period = $timeout > 30? $timeout - 5 : 30;
            for(my $count = 0; $count < $wait_period; $count +=5){
                $self->is_alive();
                sleep 5;
            }
            croak ("TIMED OUT..! Switch not pinging even after $timeout seconds of issuing reboot..!") if (!$self->is_alive());
            NOTE::logcomment("Verifying that switch is accessible after reboot\n");
            eval {
               $self->reconnect(); 
               $self->switch_show;
            };
            if ($@) {
                croak("Switch is not up after reboot after $timeout seconds. \nError connecting to it: $@");
            }
            else{
                NOTE::logcomment("Switch rebooted successfully.\n");
            }            
        }       
    }
}

=head2 ::fastboot

fastboots the switch if it is up or boots it if it was not up
and returns the switch to a normal up and running state.
The arguments control the process by which states are attained.

=over

=item Examples

=back

     $switch->fastboot;

=cut

#author: jawwad@netapp.com

sub fastboot {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, %args ) = @_;
    my $out = $self->say("fastboot");
    if ( $out =~ /The system is going down for reboot NOW !!/ ) {
        NOTE::logcomment("Switch reboot successfull\n");
    } else {
        croak "switch reboot was not successfull.\n";
    }
}

=head2 ::fcvi_port_finder

Runs 'nsshow' command on a switch and pull the fcvi port and returns it.

=over

=item Arguments

None

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $fcvi_port = $sw1->fcvi_port_finder;


	In Cisco Switch
	
	-------------------------------------------------------------------------------
	Interface  Vsan   Admin  Admin   Status          SFP    Oper  Oper   Port
		              Mode   Trunk                          Mode  Speed  Channel
	                         Mode                                 (Gbps)
	-------------------------------------------------------------------------------

	fc1/1      10     F      on      up               swl    F       8    --
=cut

sub fcvi_port_finder {
    my $self = shift;
    my $fcvi_port = 0;
	my @port_info;
	my @port_details;

	if ( $self->{_hostrec}->hosttype =~ /cisco/ ) {
		my @port_info;
        @port_info = $self->{_non_brocade_object}->execute("show int brief");
        foreach (@port_info) {
            if ( $_ =~ /^fc/ ) {
                @port_details = split( /\s+/, $_ );
                print "\nPorts 0 , 2, 3, 4 are:- $port_details[0] ; $port_details[2] ; $port_details[3] ; $port_details[4]\n";
                if ( ($port_details[2] eq "F") && ($port_details[4] =~ /up/) )  {
                    print "\nPorts 0 , 2, 3, 4 are:- $port_details[0] ; $port_details[2] ; $port_details[3] ; $port_details[4]\n";
                    $fcvi_port = $port_details[0];
                    print "Port is $fcvi_port \n\n";
                }
            }
        }
   
     return $fcvi_port;
	 } else {
	
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    
    if (
        ( grep( /fcvi/i, split( /[0-9a-f]{6};/, $self->say('nsshow') ) ) )[0] =~
        /Port Index:\s*(\d+)/ )
    {
        $fcvi_port = $1;
    }
    return $fcvi_port;

    
	}
}

=head2 ::switch_powercycle

Alias to FcSwitch::powercycle

=cut

sub switch_powercycle {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 &powercycle }

=head2 ::switch_reboot

Alias to FcSwitch::reboot

=cut

sub switch_reboot {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 &reboot }

=head2 ::switch_enable

Enables the switch using a switch object and checks whether
it is enabled or not by using switchshow command

=over

=item Arguments

None

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->switch_enable

=cut

#author: jawwad@netapp.com

sub switch_enable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    
    
    #Start SMC Specific
    #SMC Specific: Call appropriate function if function call has been made for SMC testbed
    #This is temporary fix for existing FMC scripts to work on SMC configs
    if($self->{_host} =~ /SMC_CONFIGURATION/){
        NOTE::logcomment("FcSwitch switch_enable function called for SMC!");
        NOTE::logcomment("Dummy call from second switch object for SMC! Returning...");
        return;
    }
    my $hosttype = $self->{_hostrec}->hosttype;
    if($hosttype =~ /filer-cfe/i){
        NOTE::logcomment("FcSwitch switch_enable function called for SMC!");
        NOTE::logcomment("Calling FcMetroclusterUtils::rpower_devices internally...\n");
        my @rpower_list = split( /,/, $self->{_shelf_rpower_list} );
        push(@rpower_list,$self->{_filer_rpower_name});
        FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_list);
        sleep(10);
        return;
    }
    #End SMC Specific
    
    if ( $self->{_hostrec}->hosttype =~ /cisco/ ) {
        $self->{_non_brocade_object}->execute("configure t ; int fc1/1-$self->{_no_of_ports} ; no shutdown ; end ;");
        $self->{_non_brocade_object}->execute("show int brief ;");                                                
        NOTE::logcomment( "Switch Enabled : " . $self->host );
	}

=head Old CODE 
	   Previously used code. No longer used. Retained for any future use
        my @port_details =
          $self->{_non_brocade_object}->execute("show int brief");
        my $no_of_ports = 0;
        foreach (@port_details) {
            if ( $_ =~ /^fc/ ) { $no_of_ports++; }
        }
        $self->{_non_brocade_port_object}->port_enable("fc1/1-$no_of_ports");
       End of earlier used code
=cut

 else {
        $self->say("switchenable");
        sleep(2);
        my $check = $self->say('switchshow');
        my $switch_state;
        foreach ( split( '[\n\r]', $check ) ) {
            my @n = /(\w+)/g;
            if ( grep( /^\s*switchState:\s*[oO]nline\s*/, $_ ) ) {
                $switch_state = 'online';
            }
            if ( grep( /^\s*switchState:\s*[oO]ffline\s*/, $_ ) ) {
                $switch_state = 'offline';
            }
        }
        if ( $switch_state eq 'online' ) {
            NOTE::logcomment( "Switch Enabled : " . $self->host );
        }
        if ( $switch_state eq 'offline' ) {
            croak "Error: Switch" . $self->host . " was not Enabled : ";
        }
    }
}

=head2 ::switch_disable

Disables the switch using a switch object, also checks
using switchshow whether switch is disabled or not

=over

=item Arguments

None

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->switch_disable

=cut

#author: jawwad@netapp.com

sub switch_disable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;

    #Start SMC Specific
    #SMC Specific: Call appropriate function if function call has been made for SMC testbed
    #This is temporary fix for existing FMC scripts to work on SMC configs
    if($self->{_host} =~ /SMC_CONFIGURATION/){
        NOTE::logcomment("FcSwitch switch_disable function called for SMC!");
        NOTE::logcomment("Dummy call from second switch object for SMC! Returning...");
        return;
    }
    my $hosttype = $self->{_hostrec}->hosttype;
    if($hosttype =~ /filer-cfe/i){
        NOTE::logcomment("FcSwitch switch_disable function called for SMC!"); 
        NOTE::logcomment("Calling FcMetroclusterUtils::rpower_devices internally...\n");
        my @rpower_list = split( /,/, $self->{_shelf_rpower_list} );
        push(@rpower_list,$self->{_filer_rpower_name});
        FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_list, off => 1);
        my @rpower_filer;
        push(@rpower_filer,$self->{_filer_rpower_name});
        FcMetroclusterUtils::rpower_devices( rpower_list => \@rpower_filer);
        return;
    }
    #End SMC Specific

    if ( $self->{_hostrec}->hosttype =~ /cisco/ ) {
        $self->{_non_brocade_object}->execute("configure t ; int fc1/1-$self->{_no_of_ports} ; shutdown ; end ;");
        $self->{_non_brocade_object}->execute("show int brief ;");                                            
        NOTE::logcomment( "Switch Disabled! : " . $self->host );

=head OLD CODE
	 Previously used code. No longer used. Retained for any future use
        my @port_details =
          $self->{_non_brocade_object}->execute("show int brief");
        my $no_of_ports = 0;
        foreach (@port_details) {
            if ( $_ =~ /^fc/ ) { $no_of_ports++; }
        }
        $self->{_non_brocade_port_object}->port_disable("fc1/1-$no_of_ports");
	 End 
=cut
    } else {
        $self->say("switchdisable");
        sleep(3);
        my $check = $self->say('switchshow');
        sleep(3);
        my $switch_state;
        foreach ( split( '[\n\r]', $check ) ) {
            my @n = /(\w+)/g;
            if ( grep( /^\s*switchState\s*:\s*[oO]nline\s*/, $_ ) ) {
                $switch_state = 'online';
            }
            if ( grep( /^\s*switchState:\s*[oO]ffline\s*/, $_ ) ) {
                $switch_state = 'offline';
            }
        }
        if ( $switch_state eq 'offline' ) {
            NOTE::logcomment( "Switch Disabled! : " . $self->host );
        }
        if ( $switch_state eq 'online' ) {
            croak "Error: Switch " . $self->host . " was not Disabled ";
        }
    }
}

# check for this if switch is disabled and trying to enable the port
#Switch was disabled, try enable port later.

=head2 ::switch_rename

Executes switchname command with the new name argument,
and renames the switch.

=over

=item Arguments

None

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->switch_rename($switch_name);

=cut

sub switch_rename {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $switch_name ) = @_;
    $self->say( "switchname $switch_name",
        -prompt, "$switch_name:admin>", '-timeout', 10 );
    sleep(5);
    my $hash         = $self->switch_show;
    my $switch_name2 = $hash
      ->{switchName};    #provided one and the one the is set should be the same
    if ( $switch_name2 eq $switch_name ) {
        NOTE::logcomment("Switch name successfully changed! \n");
    } else {
        NOTE::logcomment(
"Switch name could not be changed, please see the logs for more details. \n"
        );
    }
}

=head2 ::port_enable

Enables the port on a switch object, it covers switch-dell,
switch-mcdata,switch-non_brocade, and switch-brocade

=over

=item Arguments

port number

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->port_enable(0);

=cut

#author: jawwad@netapp.com

sub port_enable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self      = shift;
    my $port_num  = shift;
    my $host_type = $self->{_conn}->{hostp}->{hosttype};
    if ( $port_num eq undef ) {
        croak
"Port num was not provided as an argument. Example: \"\$sw1->port_disable(0)!\"\n";
    }
    if ( $host_type =~ /switch-mcdata/i ) {
        $self->say("config port blocked $port_num false");
    } elsif ( $host_type eq "switch-dell" ) {
        $self->say("configure");
        $self->say("interface ethernet 1/$port_num");
        $self->say("no shutdown");
        $self->say("exit");
        $self->say("exit");
    } elsif ( $host_type =~ /switch-cisco/i ) {
        $self->{_non_brocade_object}->execute("conf t ; int $port_num ;no shutdown ;end ;");
        sleep 1;
        $self->{_non_brocade_object}->execute("show int $port_num ;");                                           
        NOTE::logcomment(
            "Port Enabled! Port Number : $port_num , Switch :" . $self->host )
    } elsif ( $host_type =~ /switch-brocade/i ) {
        $self->say("portenable $port_num");
        NOTE::logcomment(
            "Port Enabled! Port Number : $port_num , Switch :" . $self->host );
    } else {
        croak " Error Enabling Port "
          . $port_num
          . " on Switch: "
          . $self->host . "\n";
    }
}

=head2 ::port_disable

Disables the port on a switch object

=over

=item Arguments

port number

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->port_disable(0);

=cut

#author: jawwad@netapp.com

sub port_disable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self      = shift;
    my $port_num  = shift;
    my $host_type = $self->{_conn}->{hostp}->{hosttype};
    if ( $port_num eq undef ) {
        croak
"Port num was not provided as an argument. Example: \"\$sw1->port_disable(0)!\"\n";
    }
    if ( $host_type =~ /switch-mcdata/i ) {
        $self->say("config port blocked $port_num true");
    } elsif ( $host_type eq "switch-dell" ) {
        $self->say("configure");
        $self->say("interface ethernet 1$port_num");
        $self->say("shutdown");
        $self->say("exit");
        $self->say("exit");
    } elsif ( $host_type =~ /switch-cisco/i ) {
        $self->{_non_brocade_object}->execute("conf t ; int $port_num ;shutdown ;end ;");
        sleep 1;
        $self->{_non_brocade_object}->execute("show int $port_num ;");                                             
        NOTE::logcomment(
            "Port Disabled : $port_num , on Switch :" . $self->host );
    } elsif ( $host_type =~ /switch-brocade/i ) {
        $self->say("portdisable $port_num");
        NOTE::logcomment(
            "Port Disabled : $port_num , on Switch :" . $self->host );
    } else {
        croak " Error Disabling Port "
          . $port_num
          . " on Switch: "
          . $self->host . "\n";
    }
}

#mc-sw3:admin> isl_show
#  1:  0->  0 10:00:00:05:1e:90:2e:99   1 mc-sw1          sp:  4.000G bw:  4.000G
#mc-sw3:admin>
#mcsw10:admin> isl_show
#1:  0 ->  0   10:00:00:60:69:c0:0c:2c   mcsw9   sp: 2G  bw: 2G
#mcsw10:admin>

=head2 ::isl_show

Execute isl_show command on the switch and populate
the information into a hash. Returns 0 if No ISL ports were
found.

=over

=item Arguments

None

=item Examples

=back

    print Dumper ($switch->isl_show);

=cut

#author: jawwad@netapp.com

sub isl_show {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my $isl  = undef;
    my $err  = 1;
	my $host_type = $self->{_hostrec}->hosttype;

    if ( $host_type =~ /switch-cisco/i ) {
        my @isl_ports = @{$self->{eports}};
        my $i = 1;
        print "The online ISL port is/are: @isl_ports\n"; 
        foreach my $eport (@isl_ports){
            $$isl{$i}{local_port}    = $eport;
            $$isl{$i}{remote_port}   = $eport; #assuming best practice to use same port across switch
            $$isl{$i}{wwnn}          = "NA_cisco";
            $$isl{$i}{domain}        = "NA_cisco";
            ###Retrieve the name of remote switch name
            
            my @port_info;
            @port_info = $self->{_non_brocade_object}->execute("show interface $eport");
            my $port_vsan;
            foreach (@port_info) {
                if ( $_ =~ /(.*)Port vsan is (\d+)/i ) {
                    $port_vsan = $2;
                    last;
                }
            }
            my @fcs_ie;
            @fcs_ie = $self->{_non_brocade_object}->execute("show fcs ie vsan $port_vsan | grep -i Adj");
            foreach(@fcs_ie){
                if ( $_ =~ /(.*)\(Adj\) (.*) \((.*)\)/i ) {
                    $$isl{$i}{remote_switch} = $3;
			last;
                }                
            }
            $i++;
        }
	return $isl;
    }

   else{
    foreach ( split( '[\n\r]', $self->say('islshow') ) ) {
        s/\s+$//g;
        if (/^\s*(\d+):\s+(\d+)\s*->\s*(\d+)\s+(\S+)\s+(\d*)\s*(\S+)/) {
            $$isl{$1}{local_port}    = $2;
            $$isl{$1}{remote_port}   = $3;
            $$isl{$1}{wwnn}          = $4;
            $$isl{$1}{domain}        = $5;
            $$isl{$1}{remote_switch} = $6;
        }
    }
    if ( $isl ne '' ) {
        return $isl;
    } else {
        NOTE::logcomment("No Online ISL Ports were found\n");
        return $err = 0;
    }
  } #End of else
}

=head2 ::isl_cut

Disables the ISL links, gets the ISL info using isl_show method,
also handles dual isl Links case

=over

=item Arguments

None

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->isl_cut;

=cut

#author: jawwad@netapp.com

sub isl_cut {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self      = shift;
    my $host_type = $self->{_hostrec}->hosttype;
    my ( $proto, $port, @eports, $i );
    if ( $host_type =~ /switch-cisco/i ) {
        my $eport  = $self->{'eports'};
        my @eports = @$eport;
        foreach (@eports) {
            $self->port_disable($_);
        }
    }

    else {

        my $ref = $self->isl_show;
        if ( !$ref ) {
            NOTE::logcomment("port already disabled! \n");
            return;
        }
        my $loc = 0;
        my ( $key, $val );
        while ( ( $key, $val ) = each %$ref ) {
            $self->{isl_info}->{$loc}->{_isl_port_num} =
              $ref->{$key}
              ->{local_port};    #storing the port into $self->{isl_info}
            $self->say("portdisable $self->{isl_info}->{$loc}->{_isl_port_num}")
              if $self->{isl_info}->{$loc}->{_isl_port_num} =~ /\d+/;
            NOTE::logcomment(
"ISL Link Disabled :  $self->{isl_info}->{$loc}->{_isl_port_num}\n"
            ) if $self->{isl_info}->{$loc}->{_isl_port_num} =~ /\d+/;
            $loc++;
        }
    }
}

=head2 ::isl_off

Alias method for isl_cut

=cut

sub isl_off {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 &isl_cut }

=head2 ::isl_join

Enables the ISL link, gets the ISL info saved from the object $self->{_port_num}
works only when you call with isl_cut in order to store isl information
from isl show while the port is enabled.

=over

=item Arguments

None

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->isl_join;

=cut

#author: jawwad@netapp.com

sub isl_join {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self      = shift;
    my $host_type = $self->{_hostrec}->hosttype;
    my ( $proto, $port, @port_details );
    if ( $host_type =~ /switch-cisco/i ) {
        my $eport  = $self->{'eports'};
        my @eports = @$eport;
        foreach (@eports) {
            $self->port_enable($_);
        }
    }

    else {

        my $loc = 0;
        while ( $self->{isl_info}->{$loc} ) {
            $self->say("portEnable $self->{isl_info}->{$loc}->{_isl_port_num}")
              if $self->{isl_info}->{$loc}->{_isl_port_num} =~ /\d+/;
            NOTE::logcomment( "ISL Link Enabled : "
                  . $self->{isl_info}->{$loc}->{_isl_port_num}
                  . " on Switch : "
                  . $self->host )
              if $self->{isl_info}->{$loc}->{_isl_port_num} =~ /\d+/;
            $loc++;
        }
    }
}

=head2 ::isl_on

Alias method for isl_join

=cut

sub isl_on {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 &isl_join }

=head2 ::switch_show

Runs the switchshow command on the switch, and populates
the hash with all the switch information uptill the port info.
Following is the structure of the hash.
    
    $VAR1 = {
	'switchWwn' => '10:00:00:05:1e:09:af:62',
	'switchType' => '34.0',
	'zoning' => 'OFF',
	'switchRole' => 'Subordinate',
	'switchName' => 'mc-sw13',
	'switchId' => 'fffc0d',
	'switchState' => 'Online',
	'switchBeacon' => 'OFF',
	'switchMode' => 'Native',
	'switchDomain' => '13'
	};

    
=over

=item Arguments

None

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $hash = $sw1->switch_show

=cut

#author: jawwad@netapp.com

sub switch_show {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;

    my $switch_show = {};
    if ( $self->{_hostrec}->hosttype !~ /brocade/ ) {
        $switch_show = $self->{_non_brocade_object}->info;
    } else {
        foreach ( split( '[\n\r]', $self->say('switchshow') ) ) {
            if (/^\s*(\w+):\s*(\S+)/i)
            {  #find all the info uptill "Area Port Media Speed State     Proto"
                $switch_show->{$1} = $2;
            }
        }
    }
    return $switch_show;
}

=head2 ::port_info

Issues a switchshow command on a given switch and extracts all the port information.
All the information is then populated into a hash reference which is a return value.
If No online ports are found, it reports it on STDIN. Structure of the hash is as
follows (e.g with 5 ports online):

    $VAR1 = {
	'9' => {
	    'proto' => 'F-Port',
	    'speed' => 'N4',
	    'media' => 'id',
	    'port' => '9',
	    'state' => 'Online'
	    },
	 '5' => {
	     'proto' => 'L-Port',
	     'speed' => 'N2',
	     'media' => 'id',
	     'port' => '5',
	     'state' => 'Online'
	     },
	  '3' => {
	      'proto' => 'F-Port',
	      'speed' => 'N4',
	      'media' => 'id',
	      'port' => '3',
	      'state' => 'Online'
	      },
	   'total_online_ports' => 6,
	   'port_num_array' => ['0','2','3','5','9','14'],
	   '14' => {
	       'proto' => 'L-Port',
	       'speed' => 'N2',
	       'media' => 'id',
	       'port' => '14',
	       'state' => 'Online'
	       },
	    '0' => {
		'proto' => 'E-Port',
		'speed' => 'N4',
		'media' => 'id',
		'port' => '0',
		 'state' => 'Online'
		 },
	    '2' => {
		'proto' => 'F-Port',
		'speed' => 'N4',
		'media' => 'id',
		'port' => '2',
		'state' => 'Online'
		 }
    };

=over

=item Arguments

None

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $port_info = $sw1->port_info;

How to Extract from the hash: 

    #$total_ports is an integer value returns the total count of ports e.g 6 in this case
    my $total_ports = $port_info->{total_online_ports};

You can manually extract online ports one by one:

    my $first_port = $port_info->{0}->{port};
    my $second_port = $port_info->{3}->{port};

Extract other port info (In this case extracting port0s information):

    my $media = $port_info->{0}->{media};
    my $speed = $port_info->{0}->{speed};
    my $state = $port_info->{0}->{state};
    my $proto = $port_info->{0}->{proto};

Printing all the online ports from an array inside a hash:

    for ($i=0; $i<$total_ports;$i++){
      NOTE::logcomment("port = $port_info->{port_num_array}->[$i]\n");
    }

Another example of checking all the L-ports and enabling it.

Checking for L-ports in the hash and enables it when finds it
    
    for($i=0; $i<$total_ports; $i++) {
	if($port_hash->{$port_hash->{port_num_array}->[$i]}->{proto} eq 'L-Port'){
	    my $port_num = $port_hash->{$port_hash->{port_num_array}->[$i]}->{port};
	  NOTE::logcomment("Enabling port: $port_num on Switch: $SWITCH\n");
	    $sw->port_enable($port_num);
	}
    }

=cut

#author: jawwad@netapp.com

sub port_info {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my ( $switch_show, $i, @array );
    my @port_details;
    my $port_info = { port_num_array => [] };
    $i = 0;
    my ( $version_flag, $proto, $detail );
    if ( $self->{_hostrec}->hosttype =~ /cisco/ ) {
        my @port_info;
        @port_info = $self->{_non_brocade_object}->execute("show int brief");
        foreach (@port_info) {
            if ( $_ =~ /^fc/ ) {
                @port_details = split( /\s+/, $_ );
                if ( $port_details[4] =~ /up|trunking/ ) {
                    $port_info->{"$port_details[0]"}->{port} = $port_details[0];
                    $port_info->{"$port_details[0]"}->{speed} = $port_details[7];
                    $array[$i] = $port_details[0];
                    if    ( $port_details[6] eq 'F' )  { $proto = 'F-Port'; }
                    elsif ( $port_details[6] eq 'FL' ) { $proto = 'L-Port'; }
                    elsif ( $port_details[6] eq 'TE' ) { $proto = 'E-Port'; }
                    $port_info->{"$port_details[0]"}->{proto} = $proto;
                    $port_info->{"$port_details[0]"}->{media} = '';
                    $port_info->{"$port_details[0]"}->{state} = $port_details[4];
                    $i++;
                }
            }
        }
        $port_info->{port_num_array}     = \@array;
        $port_info->{total_online_ports} = $i;
        return $port_info;

    }

    else {

        #extracting the online port string

        foreach ( split( '[\n\r]', $self->say('switchshow') ) ) {
            if ( grep( /(Online+\s+.*Port)/, $_ ) ) {
                $_ =~ s/(LE|LD|LS)//g;
                $switch_show .= "$_\n";
            }
        }

        #if nothing was found report it

        if ( $switch_show eq undef ) {
            NOTE::logcomment("No Online Ports were found\n");
            return;
        }

        #once the string built, populate the hash with the port information

        my $version = $self->version->{'Fabric OS'};
        $version =~ s/\D//g;
        $version_flag = 1 if ( $version > 611 );
        foreach ( split( '[\n\r]', $switch_show ) ) {
            if ( !$version_flag ) {
                if (
/\s*\w+\s+(\d+)\s+(\w+)\s+(\w+)\s+(\w+)\s+\w*\s+([a-z]-\w+).*/ig
                  )
                {
                    $port_info->{"$1"}->{port}  = $1;
                    $array[$i]                  = $1;
                    $port_info->{"$1"}->{media} = $2;
                    $port_info->{"$1"}->{speed} = $3;
                    $port_info->{"$1"}->{state} = $4;
                    $port_info->{"$1"}->{proto} = $5;
                }
            }

            else {
                if (
/\s*\w+\s+(\d+)\s+(\w+)\s+(\w+)\s+(\w+)\s+(\w+)\s+\w*\s+([a-z]-\w+).*/ig
                  )
                {
                    $port_info->{"$1"}->{port}  = $1;
                    $array[$i]                  = $1;
                    $port_info->{"$1"}->{media} = $3;
                    $port_info->{"$1"}->{speed} = $4;
                    $port_info->{"$1"}->{state} = $5;
                    $port_info->{"$1"}->{proto} = $6;
                }
            }
            $i++;
        }
        $port_info->{port_num_array}     = \@array;
        $port_info->{total_online_ports} = $i;
    }
    return $port_info;
}

sub porterrshow {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self   = shift;
    my $output = $self->say("porterrshow");
    if ( $output =~ /frames  enc / ) {
        NOTE::logcomment("porterrshow executed successfully\n");
    } else {
        croak "could not run porterrshow\n";
    }
}

=head2 ::portcfgshow

Return array of individual port configurations as hash references. Keys are taken from
brocade output as is.

converts this:

    > portcfgshow
    Ports of Slot 0    0  1  2  3    4  5  6  7    8  9 10 11   12 13 14 15
    -----------------+--+--+--+--+----+--+--+--+----+--+--+--+----+--+--+--
    Speed             AN AN AN AN   AN AN AN AN   AN AN AN AN   AN AN AN AN 
    AL_PA Offset 13   .. .. .. ..   .. .. .. ..   .. .. .. ..   .. .. .. .. 
    Trunk Port        .. ON ON ON   ON ON ON ON   ON ON ON ON   ON ON ON ON 
    Long Distance     .. .. .. ..   .. .. .. ..   .. .. .. ..   .. .. .. .. 

    Ports of Slot 0   16 17 18 19   20 21 22 23   24 25 26 27   28 29 30 31
    -----------------+--+--+--+--+----+--+--+--+----+--+--+--+----+--+--+--
    Speed             AN AN AN AN   AN AN AN AN   AN AN AN AN   AN AN AN 4G 
    AL_PA Offset 13   .. .. .. ..   .. .. .. ..   .. .. .. ..   .. .. .. .. 
    Trunk Port        ON ON ON ON   ON ON ON ON   ON ON ON ON   ON ON ON .. 
    Long Distance     .. .. .. ..   .. .. .. ..   .. .. .. ..   .. .. .. .. 

into this:

    [
	  {
	    'Mirror Port' => '..',
	    'Disabled E_Port' => '..',
	    'Speed' => 'AN',
	    'Locked L_Port' => '..',
	    'RSCN Suppressed' => '..',
	    'Persistent Disable' => '..',
	    'AL_PA Offset 13' => '..',
	    'Trunk Port' => '..',
	    'Long Distance' => '..',
	    'VC Link Init' => '..',
	    'QOS E_Port' => 'ON',
	    'Locked G_Port' => '..',
	    'ISL R_RDY Mode' => '..',
	    'NPIV capability' => 'ON'
	  },
	  {
	    'Mirror Port' => '..',
	    'Disabled E_Port' => '..',
	    'Speed' => 'AN',
	    'Locked L_Port' => '..',
	    'RSCN Suppressed' => '..',
	    'Persistent Disable' => '..',
	    'AL_PA Offset 13' => '..',
	    'Trunk Port' => 'ON',
	    'Long Distance' => '..',
	    'VC Link Init' => '..',
	    'QOS E_Port' => 'ON',
	    'Locked G_Port' => '..',
	    'ISL R_RDY Mode' => '..',
	    'NPIV capability' => 'ON'
	  },
	  ...
    ]

=over

=item Arguments

None

=item Examples

    # set trunking on all ports
    my $sw = NOTE::FcSwitch->connect( host => 'mc-sw1' );
    my @ports = $sw->portcfgshow;
    foreach( 0 .. $#ports ) {
	$sw->trunk_on($_) unless( $ports[$_]->{'Trunk Port'} =~ /ON/ );
    }

    # print speed of port '3'
    my $sw = NOTE::FcSwitch->connect( host => 'mc-sw1' );
    logcomment( "Port speed of port 3 is " . ($sw->portcfgshow)[3]->{'Speed'} );
    
=back

=cut

#author : cpeter@netapp.com

sub portcfgshow {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my @ports =
      ();    # Array of ports where each element is a hash of port properties

    ( my $raw = $self->say("portcfgshow") ) =~ s/\r//g;    # raw output from 'portcfgshow' without \r
    my @arr= grep /\+/, split( /\n/, $raw ) ;
    my @arr1;
    my $string;
    my @split_arr=split( /\+/, $arr[0] );
    my $i;
    my $fmt = undef
      ; # format for 'unpack' extracted from brocade's "------+--+--" output itself
    #$fmt .= sprintf "A%d", (split //, $_) + 1 foreach( split(/\+/, ( grep /\+/, split(/\n/, $raw) )[0] ) );
   foreach(@split_arr){
       @arr1=split( //, $_ );
       $string=@arr1 + 1 ;
       $fmt.=sprintf "A%d",$string;
    }
    my @portid = ()
      ;  # temporary re-indexer is required because output is divided into pages
    foreach ( grep /^[A-Za-z]/, split( /\n/, $raw ) )
    {    # extract port info from raw output line by lines
        my @data =
          unpack( $fmt, $_ );    # and convert it to an array according to $fmt
        $data[$_] =~ s/(^\s+|\s+$)//g
          foreach (@data);       # then clean up leading and trailing spaces
        my $key = shift @data;   # first element is a key, rest are values
        if ( $key =~ /Ports of Slot/ ) {    # if key is slot number
            @portid = @data;                # then rebuild re-indexer array
        } else {                            # else fill up port info with values
            $ports[ $portid[$_] ]->{$key} = $data[$_] foreach ( 0 .. $#data );
        }
    }
    return @ports;
}

=head2 ::portcfgspeed_show

Runs portcfgshow <Port No.> command on a switch and returns the speed of the given port

=over

=item Arguments

  Required: portnumber

=item Example

=back

    my $sw = NOTE::FcSwitch->connect(host =$SWITCH);
    $sw->portcfgspeed_show(port=>$port);

=cut

sub portcfgspeed_show {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, %args ) = @_;
    my $port = $args{port};
    if ( $self->{_hostrec}->hosttype !~ /brocade/ ) {
        my $output = $self->port_info;
        my $speed  = $output->{$port}->{'speed'};
        #return $speed ;
        return $speed . 'G';
    } else {
        my $raw = $self->say("portcfgshow $port");
        $raw =~ s/(\r|\t)//g;
        my @data = split( /\n/, $raw );
        foreach (@data) {
            if ( $_ =~ /speed/i ) {
                my @split_output = split( /\:/, $_ );
                $split_output[1] =~ s/\s+//g;
                return $split_output[1];
            }
        }
    }
}

=head2 ::trunk_on

Set trunking on given port if trunking is licensed

=cut

#author : cpeter@netapp.com

sub trunk_on {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $port ) = @_;
    if ( $self->is_licensed('Trunking') ) {
        $self->say("portcfgtrunkport $port 1")
          unless ( ( $self->portcfgshow )[$port]->{'Trunk Port'} =~ /ON/ );
        return 1;
    } else {
        croak( "Trunking is not licensed on switch " . $self->host );
    }
}

=head2 ::license_show

Returns hash of licenses

=over

=item Arguments

None

=item Examples or returned hash

=back

    %lic = {
     'web' => 'bSRyyyc9cecTSzzu',
     'extended_fabric' => 'SzezeSSyycT0f0fx',
     'fabric' => '9zbQQbSddA0cRRW',
     'trunking' => 'cRQdScbzSSlSReTP',
     'zoning' => 'ebzb9QeSfc0cAX',
     'fabric_Watch' => 'cRQdScbzSSfSReTJ',
     'full_ports_on_demand' => 'cRQdScbzSSdiVeTb'
    };

=cut

#author : cpeter@netapp.com

sub license_show {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my %lic  = undef;
    ( my $lstr = $self->say("licenseshow") ) =~ s/(\r|.*licenseshow\r\n|:)//g
      ;    # get plain license output and do some minor cleanup
    my %hstr = split( "\n", $lstr )
      ;    # convert array to hash as is: 'SebcezddbcTfcdfR' => 'Fabric'
    foreach ( keys %hstr ) {
        $hstr{$_} =~ s/(^\s+|\s+license.*)//g
          ;    # more cleaning - remove "license" word and tabs
        $hstr{$_} =~
          s/\s+/_/g;    # replace spaces to '_' within license name itself
        $lic{ lc( $hstr{$_} ) } =
          $_;           # swap key value to be 'fabric' => 'SebcezddbcTfcdfR',
    }    # license is converted to lower case.
    return %lic;
}

=head2 ::is_licensed

Returns license string if given feature is licensed or undef if not

=over

=item Arguments

B<feature> => I<string>

I<Required.>
Feature string. Can be in any case and contain spaces.

=item Examples

=back

    my $sw = NOTE::FcSwitch->connect( host => 'mc-sw1' );
    unless( $sw->is_licensed('Trunking') {
	croak( "Trunking is not licensed!" );
    }


    my $sw = NOTE::FcSwitch->connect( host => 'mc-sw1' );
    unless( $sw->is_licensed('Full Ports On Demand' ) {
	croak( "Additional ports are not licensed" );
    }

    my $sw = NOTE::FcSwitch->connect( host => 'mc-sw1' );
    unless( $sw->is_licensed('full_ports_on_demand' ) {
	croak( "Additional ports are not licensed" );
    }

=cut

sub is_licensed {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $feature ) = @_;
    my %lic = $self->license_show;

    $feature =~ s/(^\s+|\s+$)//g;    # normalize feature string down
    $feature =~ s/\s+/_/g;           # to our format just in case
    $feature = lc($feature);         # someone will give it incorrectly

    return $lic{$feature};
}

=head2 ::zone_create

Runs "zonecreate" command on a switch, requires zone_name and zone_members and runs zonecreate commands on the switch.
After running the command it runs zoneshow on a switch to make sure whether the given zone was created or not. 

=over

=item Arguments

Required: zone_name

Required: zone_members (all of them in a string)

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $sw2 = NOTE::FcSwitch->connect(host => $SWITCHB);
    my $hash_sw1 = $sw1->switch_show;
    my $hash_sw2 = $sw2->switch_show;
    my $sd1 = $hash_sw1->{switchDomain};
    my $sd2 = $hash_sw2->{switchDomain};
    my $zone_name = "jawwad"; 
    my $members = "$sd1,3;$sd2,3";
    $sw1->zone_create($zone_name,$members) 

=cut

sub zone_create {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $zone_name, $zone_member ) = @_;
    $self->say("zonecreate \"$zone_name\", \"$zone_member\"");
    my $hash = $self->zone_show;
    my @zones =
      @{ $hash->{zone_name_array} };    #populated @zones with all the zones
    my $found_flag = 0;
    foreach (@zones) {                  #looping thru all the available zones
        if ( $zone_name eq $_ ) {       # if found then set flag to true
            $found_flag = 1;
        }
    }
    if ($found_flag) {
        NOTE::logcomment("Zone : $zone_name created successfully !!\n");
    } else {
        croak "Zone Error : Could not create $zone_name please try again\n";
    }
}

=head2 ::configure_zoning


From the switch object on which zoning is called, islshow method is called and remote switch name is parsed.
The filer name that is passed  is used to get the fcvi ports and storage ports for both the local switch and remote switch
Creates the zone member strings to be passed while calling the zone create command.
Creates a config entry, enables the config.
Issues a cfgactv show command on both the local switch and remote switch.

=over

=item Arguments


B<filer> => I<Filer name> 
I<Required.>
Filer name to which switch is attached

B<fcvi_zone_name> => I<Name of the FCVI Zone>
I<Optional.>
Name to be given to the FCVI Zone. Defaults to FCVI


B<stor_zone_name> => I<Name of the STOR Zone>
I<Optional.>
Name to be given to the STOR Zone. Defaults to STOR


B<config_name> => I<Name of the configuration>
I<Optional.>
Name to be given to the configuration . Defaults to config

=item examples

=back
    $mc = NOTE::FcMetrocluster->new(Filer =>$FILERA);
    $sw1 = NOTE::FcSwitch->connect(host => $mc->{me_fabrics}->{switch1});
    $sw2 = NOTE::FcSwitch->connect(host => $mc->{me_fabrics}->{switch2});

sw1->configure_zoning(filer=>$FILERA,fcvi_zone_name=>'myzone1' ,stor_zone_name=>'myzone2',config_name=>'myconfig1');
$sw2->configure_zoning(filer=>$FILERA,fcvi_zone_name=>'myzone1' ,stor_zone_name=>'myzone2',config_name=>'myconfig1');

Alternatively this can also be used as 

sw1->configure_zoning(filer=>$FILERA);
$sw2->configure_zoning(filer=>$FILERA);


=cut

sub configure_zoning {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


    my ( $self, %args ) = @_;
    my $filer = $args{filer};
    my $fcvi_zone_name =
      defined $args{fcvi_zone_name} ? $args{fcvi_zone_name} : 'FCVI';
    my $stor_zone_name =
      defined $args{stor_zone_name} ? $args{stor_zone_name} : 'STOR';
    my $config_name =
      defined $args{config_name} ? $args{config_name} : 'config';
    my $remote_switch = $self->isl_show->{1}->{remote_switch};
    my $sw2           = NOTE::FcSwitch->connect( host => $remote_switch );
    my $mc            = NOTE::FcMetrocluster->new( Filer => $filer );
    my $zone1_members;
    my $zone2_members;
    my $port_info1 =
      $self->port_info;    #calling port_info method to get all the L-ports
    my $port_info2 = $sw2->port_info;
    my $sd1        = $self->switch_show
      ->{switchDomain};    #getting the switch domain of local switch
    my $sd2 = $sw2->switch_show
      ->{switchDomain};    #getting  the switch domain of remote switch
    my ( $hba, @filer_ports ) =
      $mc->get_fcvi_hba( me => 1 );    #getting ports for local
    my ( $hba2, @filer_ports2 ) = $mc->get_fcvi_hba( partner => 1 )
      ; #getting ports for partner because the other switch is on the partner side
    my $local_switch = $self->host;    #getting the switchname

    foreach my $iter (@filer_ports) {
        if ( $hba->{FCVI}->{$iter}->{$local_switch} )
        {    #iterating over the hash by using filer_ports
            $zone1_members .= "$sd1,$hba->{FCVI}->{$iter}->{$local_switch};"
              ;    #building member string  e.g "13,3;
        }
    }

    foreach my $iter (@filer_ports2)
    {              #different filer_ports from the partner side !!
        if ( $hba2->{FCVI}->{$iter}->{$remote_switch} ) {
            $zone1_members .= "$sd2,$hba2->{FCVI}->{$iter}->{$remote_switch};"
              ;    #building with another switch e.g 13,3;15,3;
        }
    }
    chop($zone1_members);    #chopping off the last semicolon i.e 13,3;15,3

    #Creating STOR members
    foreach my $iter (@filer_ports) {
        if ( $hba->{FC}->{$iter}->{$local_switch} ) {
            $zone2_members .= "$sd1,$hba->{FC}->{$iter}->{$local_switch};";
        }
    }
    foreach my $iter (@filer_ports2) {
        if ( $hba2->{FC}->{$iter}->{$remote_switch} ) {
            $zone2_members .= "$sd2,$hba2->{FC}->{$iter}->{$remote_switch};";
        }
    }
    my $port_hash    = $self->port_info;
    my $port_hash2   = $sw2->port_info;
    my $total_ports  = $port_hash->{total_online_ports};
    my $total_ports2 = $port_hash2->{total_online_ports};
    my ( $port_num, $i );

    for ( $i = 0 ; $i < $total_ports ; $i++ ) {
        if ( $port_hash->{ $port_hash->{port_num_array}->[$i] }->{proto} eq
            'L-Port' )
        {
            $port_num =
              $port_hash->{ $port_hash->{port_num_array}->[$i] }->{port};
            $zone2_members .= "$sd1,$port_num;";
        }
    }

    for ( $i = 0 ; $i < $total_ports2 ; $i++ ) {
        if ( $port_hash2->{ $port_hash2->{port_num_array}->[$i] }->{proto} eq
            'L-Port' )
        {
            $port_num =
              $port_hash2->{ $port_hash2->{port_num_array}->[$i] }->{port};
            $zone2_members .= "$sd2,$port_num;";
        }
    }
    chop($zone2_members);    #removing an extra semicolon

#########Now that we have all the members populated time to create zones and configs################

    $self->zone_create( $fcvi_zone_name, $zone1_members );
    $self->zone_create( $stor_zone_name, $zone2_members );
    my $config_members = "$fcvi_zone_name;$stor_zone_name";
    $self->cfg_create( $config_name, $config_members );
    $self->cfg_enable($config_name);
    $self->say("cfgactvshow");
    $sw2->say("cfgactvshow");

}

=head2 :: destroy_zoning
a.)	Disables the config
b.)	Deletes the config having name config_name (defaults to config, otherwise whatever parameter is passed)
c.)	Deletes both the fcvi and storage zones (defaults to FCVI and STOR, otherwise whatever parameter for the name is passed)
d.)	Saves the config


=over

=item Arguments



B<fcvi_zone_name> => I<Name of the FCVI Zone>
I<Optional.>
Name of the FCVI Zone to be deleted . Defaults to FCVI


B<stor_zone_name> => I<Name of the STOR Zone>
I<Optional.>
Name of the STOR Zone to be deleted. Defaults to STOR


B<config_name> => I<Name of the configuration>
I<Optional.>
Name of the configuration  to be deleted . Defaults to config

=item examples

=back

$sw1->destroy_zoning(fcvi_zone_name=>'myzone1' ,stor_zone_name=>'myzone2',config_name=>'myconfig1');
$sw2->destroy_zoning(fcvi_zone_name=>'myzone1' ,stor_zone_name=>'myzone2',config_name=>'myconfig1');

Alternatively this can also be used as 

$sw1->destroy_zoning();
$sw2->destroy_zoning();


=cut

sub destroy_zoning {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, %args ) = @_;
    my $fcvi_zone_name =
      defined $args{fcvi_zone_name} ? $args{fcvi_zone_name} : 'FCVI';
    my $stor_zone_name =
      defined $args{stor_zone_name} ? $args{stor_zone_name} : 'STOR';
    my $config_name =
      defined $args{config_name} ? $args{config_name} : 'config';

    $self->cfg_disable;
    NOTE::logcomment("Config Disabled!!\n");
    $self->cfg_delete($config_name);
    NOTE::logcomment("Config $config_name deleted !!\n");
    $self->zone_delete($fcvi_zone_name);
    NOTE::logcomment("Zone $fcvi_zone_name deleted!!\n");
    $self->zone_delete($stor_zone_name);
    NOTE::logcomment("Zone $stor_zone_name deleted!!\n");
    $self->cfg_save;
    NOTE::logcomment("Config saved!!\n");
}

=head2 ::zone_ti_create

Runs "zone --create" command on a switch, requires zone_name and zone_members and runs zone --create commands on the switch.
After running the command it runs zone --show on a switch to make sure whether the given zone was created or not. 

=over

=item Arguments

Required: zone_name

Required: zone_members (all of them in a string)

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $sw2 = NOTE::FcSwitch->connect(host => $SWITCHB);
    my $hash_sw1 = $sw1->switch_show;
    my $hash_sw2 = $sw2->switch_show;
    my $sd1 = $hash_sw1->{switchDomain};
    my $sd2 = $hash_sw2->{switchDomain};
    my $zone_name = "fcvi_ti_pri"; 
    my $members = "$sd1,3;$sd2,3;$sd1,19;$sd2,19"; # port 19 is red ISL
    $sw1->zone_ti_create($zone_name,$members) 

=cut

sub zone_ti_create {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $zone_name, $zone_member ) = @_;
    $self->say("zone --create -t ti \"$zone_name\" -p  \"$zone_member\"");
    $self->say("zone --activate \"$zone_name\"");
    my $hash = $self->zone_show;
    my @zones =
      @{ $hash->{zone_name_array} };    #populated @zones with all the zones
    my $found_flag = 0;
    foreach (@zones) {                  #looping thru all the available zones
        if ( $zone_name eq $_ ) {       # if found then set flag to true
            $found_flag = 1;
        }
    }
    if ($found_flag) {
        NOTE::logcomment("TI Zone : $zone_name created successfully !!\n");
    } else {
        croak
"TI Zone Create Error : Could not create $zone_name please try again\n";
    }
}

=head2 ::zone_delete

Runs zonedelete command on a switch, and makes sure it delete the given zone from the defined configuration.

=over

=item Arguments

Required: Zone Name

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->zone_delete($zone_name);

=cut

sub zone_delete {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $zone_name ) = @_;
    my $hash  = $self->zone_show;
    my @zones = @{ $hash->{zone_name_array} };
    my $found_flag;
    foreach (@zones) {    #check whether the given configuration is presetn
        if ( $_ eq $zone_name ) {
            $found_flag = 1;
        }
    }
    if ($found_flag)
    { #if it does then run the command otherwise exit saying config name not found
        $hash =
          undef; #now we found the configuration therefore flushing the old hash
        @zones      = undef;    #flushing the old array
        $found_flag = undef
          ; #flushing the old flag that tells you whether the config is there or not
        $self->say("zonedelete $zone_name");
        sleep(3);
        $hash = $self->zone_show; #run zoneshow after running the delete command
        if ( $hash->{zone_name_array} ) {
            @zones =
              @{ $hash->{zone_name_array} }; # and get the current configuration
            foreach (@zones) {    # now check to see if configuration is there
                if ( $_ eq $zone_name ) {
                    $found_flag = 1;
                }
            }

            if ($found_flag) {
                croak "zonedelete: Could not delete the config: $zone_name\n";
            } else {
                NOTE::logcomment(
                    "zonedelete: $zone_name is deleted successfully\n");
            }
        }
    } else {
        NOTE::logcomment("zonedelete: $zone_name not found\n");
    }
}

=head2 ::zone_delete

Runs zonedelete command on a switch, and makes sure it delete the given zone from the defined configuration.

=over

=item Arguments

Required: Zone Name

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->zone_delete($zone_name);

=cut

sub zone_ti_delete {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $zone_name ) = @_;
    my $hash  = $self->zone_show;
    my @zones = @{ $hash->{zone_name_array} };
    my $found_flag;
    foreach (@zones) {    #check whether the given configuration is presetn
        if ( $_ eq $zone_name ) {
            $found_flag = 1;
        }
    }
    if ($found_flag)
    { #if it does then run the command otherwise exit saying config name not found
        $hash =
          undef; #now we found the configuration therefore flushing the old hash
        @zones      = undef;    #flushing the old array
        $found_flag = undef
          ; #flushing the old flag that tells you whether the config is there or not
        $self->say("zone --delete $zone_name");
        sleep(3);
        $hash = $self->zone_show; #run zoneshow after running the delete command
        @zones =
          @{ $hash->{zone_name_array} };    # and get the current configuration
        foreach (@zones) {    # now check to see if configuration is there

            if ( $_ eq $zone_name ) {
                $found_flag = 1;
            }
        }

        if ($found_flag) {
            croak "zone_ti_delete: Could not delete the config: $zone_name\n";
        } else {
            NOTE::logcomment(
                "zone_ti_delete: $zone_name is deleted successfully\n");
        }
    } else {
        NOTE::logcomment("zone_ti_delete: $zone_name not found\n");
    }
}

=head2 ::zone_add

Runs zoneadd command on a switch, and adds the zone member to the given zone.
Currently it allows user to add one member at a call.

=over

=item Arguments

Required: Config Name

Required: Zone Name

Required: Zone Member(s)

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $config = "Config";
    my $stor = "STOR";
    my $members = "1,5";
    $sw->zone_add($config,$stor,$members);
    
=cut

sub zone_add {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $config_name, $zone_name, $zone_members ) = @_;
    my $zone_hash =
      $self->zone_show;    #runs zoneshow to check the member is already there
    my @zones = @{ $zone_hash->{zone_name_array} };    #pulls the current zones
    my $zone;
    foreach $zone (@zones) {
        foreach ( @{ $zone_hash->{$config_name}->{$zone} } ) {
            if ( $_ eq $zone_members )
            {    #search the given zone members if already there then report it
                croak "$zone_name already contains $zone_members\n";
            }
        }
    }
    my $output = $self->say("zoneadd $zone_name, \"$zone_members\"; ")
      ;          #now everthing looks good so we run the zoneadd command
    sleep(5);
    my $zone_hash1 =
      $self->zone_show
      ; #do the same thing and check in the zoneshow structure if the zone members got added
    my @zones1 = @{ $zone_hash->{zone_name_array} };
    my $zone1;
    foreach $zone1 (@zones1) {
        foreach ( @{ $zone_hash1->{$config_name}->{$zone1} } ) {
            if ( $_ eq $zone_members )
            { #if the zone member was found then report the success and return to the caller
                NOTE::logcomment(
"$_ were added to $zone_name and the Configuration is $config_name\n"
                );
                return;
            }
        }
    }
    croak "Could not add $zone_members to $zone_name because it couldnt find in the zone_show tree\n";
}

=head2 ::zone_remove

Runs zoneremove command on a switch, and removes the zone member to the given zone.
Currently it allows user to remove one member at a call.

=over

=item Arguments

Required: Config Name

Required: Zone Name

Required: Zone Member(s)

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $config = "Config";
    my $stor = "STOR";
    my $members = "1,5";
    $sw->zone_remove($config,$stor,$members);

=cut

sub zone_remove {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $config_name, $zone_name, $zone_members ) = @_;
    my $zone_hash =
      $self->zone_show;    #run zone show to see if the member is already there
    my @zones = @{ $zone_hash->{zone_name_array} };
    my ( $zone, $member_found );
    foreach $zone (@zones) {
        foreach ( @{ $zone_hash->{$config_name}->{$zone} } ) {
            if ( $_ eq $zone_members ) {
                $member_found = 1;    #if it is then set the flag to true
                last;
            } else {
                $member_found =
                  0;    #if zone member was not there then set the flg to false
            }
        }
    }
    if ($member_found) { #if member is available then run the zoneremove command
        my $output = $self->say("zoneremove $zone_name, \"$zone_members\"; ");
        sleep(25);
        my $zone_hash1 =
          $self->zone_show
          ; # after running the zoneremove command, check to see if it was removed by searching through
        my @zones1 =
          @{ $zone_hash->{zone_name_array}
          }; #the zoneshow structure please see "sub zone_show" for the structure
        my $zone1;
        foreach $zone1 (@zones1) {
            foreach ( @{ $zone_hash1->{$config_name}->{$zone1} } ) {
                if ( $_ eq $zone_members ) {
                    croak "$_ could not be removed from $zone_name\n";
                }
            }
        }
    }
    if ( $member_found == 0 ) {
        croak "$zone_members is not in $zone_name\n";
    }
    NOTE::logcomment("successfully removed  $zone_members from $zone_name \n");
}

=head2 ::cfg_create

Runs "cfgcreate" command on a switch, requires cfg_name and zone_names and runs cfgcreate commands on the switch.
After running the command it runs zoneshow on a switch to make sure whether the given zone was created or not. 

=over

=item Arguments

Required: cfg_name

Required: zone_names (all of them in a string)

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $sw2 = NOTE::FcSwitch->connect(host => $SWITCHB);
    my $hash_sw1 = $sw1->switch_show;
    my $hash_sw2 = $sw2->switch_show;
    my $sd1 = $hash_sw1->{switchDomain};
    my $sd2 = $hash_sw2->{switchDomain};
    my $zone_name = "jawwad"; 
    my $cfg_name = "Config_jawwad";
    my $members = "$sd1,3;$sd2,3";
    my $cfg_member = "$zone_name;STOR2";
    $sw1->zone_create($zone_name,$members) 
    $sw1->cfg_create($cfg_name,$cfg_member);

=cut

sub cfg_create {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $cfg_name, $cfg_member ) = @_;
    $self->say("cfgcreate \"$cfg_name\", \"$cfg_member\"");
    my $hash = $self->zone_show;
    my @cfg = @{ $hash->{cfg_name_array} }; #populated @cfg with all the configs
    my $found_flag = 0;
    foreach (@cfg) {    #looping thru all the available zones
        if ( $cfg_name eq $_ ) {    # if found then set flag to true
            $found_flag = 1;
        }
    }
    if ($found_flag) {
        NOTE::logcomment("Config : $cfg_name created successfully !!\n");
    } else {
        croak "Config Error : Could not create $cfg_name please try again\n";
    }
}

=head2 ::cfg_enable

Runs "cfgenable" command on a switch, it requires cfg_name as a parameter.
After running the command it runs zoneshow on a switch to make sure whether the given config was enabled or not. 

=over

=item Arguments

Required: cfg_name

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $sw2 = NOTE::FcSwitch->connect(host => $SWITCHB);
    my $hash_sw1 = $sw1->switch_show;
    my $hash_sw2 = $sw2->switch_show;
    my $sd1 = $hash_sw1->{switchDomain};
    my $sd2 = $hash_sw2->{switchDomain};
    my $zone_name = "jawwad"; 
    my $cfg_name = "Config_jawwad";
    my $members = "$sd1,3;$sd2,3";
    my $cfg_member = "$zone_name;STOR2";
    $sw1->zone_create($zone_name,$members) 
    $sw1->cfg_create($cfg_name,$cfg_member);
    $sw1->cfg_enable($cfg_name);

=cut

sub cfg_enable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $cfg_name ) = @_;
        my @output = Tharn::say(
            $self->conn,
            "cfgenable $cfg_name",
            'Do you want to enable .*',
            -noprompt, -timeout, 180
        );
        if ( $output[0] =~ /Do you want to enable/ ) {
            Tharn::say( $self->conn, 'yes', '>', -noprompt, -timeout, 180 );
        }
        sleep(4);
        my $zone_hash = $self->zone_show;
        my $e_cfg     = $zone_hash->{effective_config};
        if ( $cfg_name eq $e_cfg ) {
            NOTE::logcomment(
"cfgenable: configuration $cfg_name was successfully enabled !!\n"
            );
        } else {
            croak
"cfgenable : There was a problem enabling the following configuration: $cfg_name, please try again\n";
        }
    
}

=head2 ::cfg_disable

Runs "cfgdisable" command on a switch, cfgdisable does not require any argument.
it disables the effective configuration. Initially it checks whether there is any
configuration is in effect if there is then it runs cfgdisable on it otherwise returns
a nice message original to the switch.

=over

=item Arguments

None

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $sw2 = NOTE::FcSwitch->connect(host => $SWITCHB);
    my $hash_sw1 = $sw1->switch_show;
    my $hash_sw2 = $sw2->switch_show;
    my $sd1 = $hash_sw1->{switchDomain};
    my $sd2 = $hash_sw2->{switchDomain};
    my $zone_name = "jawwad"; 
    my $cfg_name = "Config_jawwad";
    my $members = "$sd1,3;$sd2,3";
    my $cfg_member = "$zone_name;STOR2";
    $sw1->zone_create($zone_name,$members) 
    $sw1->cfg_create($cfg_name,$cfg_member);
    $sw1->cfg_enable($cfg_name);
    $sw1->cfg_disable;

=cut

sub cfg_disable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self        = shift;
    my $zone_status = $self->zone_show;
    my $flag        = 0;
    my $remote_switch;
    my $sw2;
        $remote_switch = $self->isl_show->{1}->{remote_switch};
        if ( $zone_status->{zone_status} eq 'OFF' ) {
            NOTE::logcomment("No effective configuration found.\n");
            return;
        } else {
            my @output =
              Tharn::say( $self->conn, "cfgdisable",
                'Do you want to disable zoning configuration',
                -noprompt, -timeout, 180 );
            if ( $output[0] =~ /Do you want to disable zoning configuration/ ) {
                Tharn::say( $self->conn, 'yes','>',  -noprompt, -timeout, 180 );

            }
            sleep(4);
            my $zone_status = $self->zone_show;
            if ( $zone_status->{zone_status} eq 'OFF' ) {
                NOTE::logcomment(
                    "cfgdisable : configuration disabled succussfully !!\n");
            } else {
                $sw2 = NOTE::FcSwitch->connect( host => $remote_switch );
                $self->switch_disable;
                $sw2->switch_disable;
                my @output =
                  Tharn::say( $self->conn, "cfgdisable",
                    'Do you want to disable zoning configuration',
                    -noprompt, -timeout, 180 );
                if ( $output[0] =~
                    /Do you want to disable zoning configuration/ )
                {
                    Tharn::say( $self->conn, 'yes','>',  -noprompt, -timeout,
                        180 );
                }
                @output =
                  Tharn::say( $sw2->conn, "cfgdisable",
                    'Do you want to disable zoning configuration',
                    -noprompt, -timeout, 180 );
                if ( $output[0] =~
                    /Do you want to disable zoning configuration/ )
                {
                    Tharn::say( $sw2->conn, 'yes','>', -noprompt, -timeout,
                        180 );
                }
                $self->switch_enable;
                sleep 2;
                $sw2->switch_enable;
                sleep 120;
                $zone_status = $self->zone_show;
                if ( $zone_status->{zone_status} eq 'OFF' ) {
                    NOTE::logcomment(
                        "cfgdisable : configuration disabled succussfully !!\n"
                    );
                } else {
                    croak
"Error cfgdisable : There was a problem disabling the configuration \n";
                }
            }
            sleep(2);
            if ($sw2) {
                $zone_status = $sw2->zone_show;
		if ( $zone_status->{zone_status} eq 'OFF' ) {
                    NOTE::logcomment(
                        "cfgdisable : configuration disabled succussfully !!\n"
                    );
                } else {
                    croak
"Error cfgdisable : There was a problem disabling the configuration \n";
                }
            }
        }
    
}

=head2 ::cfg_save

Runs cfgsave command on a switch, and makes sure it ran properly.

=over

=item Arguments

None

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->cfg_save;

=cut

sub cfg_save {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
        my @output =
          Tharn::say( $self->conn, "cfgsave",
            'Do you want to save Defined zoning configuration only?',
            -noprompt, -timeout, 180 );
        if ( $output[0] =~
            /Do you want to save Defined zoning configuration only?/ )
        {
            my @output2 =
              Tharn::say( $self->conn, 'yes', '>', -noprompt, -timeout, 180 );
            NOTE::logcomment("cfgsave: Successfully ran cfgsave\n");
        } else {
            croak "cfgsave: Could not save the defined configuration\n";
        }
   
}

=head2 ::cfg_delete

Runs cfgdelete command on a switch, and makes sure it delete the given configuration from the defined configuration.

=over

=item Arguments

Required: Config Name

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->cfg_delete($cfg_name);

=cut

sub cfg_delete {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $cfg_name ) = @_;
    my $hash = $self->zone_show;               #run zone_show for pre-checks
    my @cfgs = @{ $hash->{cfg_name_array} };   #pulls the current configurations
    my $found_flag;
    foreach (@cfgs) {    #check whether the given configuration is presetn
        if ( $_ eq $cfg_name ) {
            $found_flag = 1;
        }
    }
    if ($found_flag)
    { #if it does then run the command otherwise exit saying config name not found
        my $hash1
          ;    #now we found the configuration therefore flushing the old hash
        my @cfgs1;    #flushing the old array
        $found_flag = undef
          ; #flushing the old flag that tells you whether the config is there or not
        $self->say("cfgdelete $cfg_name");
        sleep(3);
        $hash1 =
          $self->zone_show;    #run zoneshow after running the delete command
        if ( $hash1->{cfg_name_array} ) {
            @cfgs1 =
              @{ $hash1->{cfg_name_array} }; # and get the current configuration
            foreach (@cfgs1) {    # now check to see if configuration is there
                if ( $_ eq $cfg_name ) {
                    $found_flag = 1;
                }
            }

            if ($found_flag) {
                croak "cfgdelete: Could not delete the config: $cfg_name\n";
            } else {
                NOTE::logcomment(
                    "cfgdelete: $cfg_name is deleted successfully\n");
            }
        }
    } else {
        NOTE::logcomment("cfgdelete: $cfg_name not found\n");
    }
}

=head2 ::cfg_clear

Runs cfgclear command on a switch, and makes sure it ran properly.

=over

=item Arguments

None

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->cfg_clear;

=cut

sub cfg_clear {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my @output =
      Tharn::say( $self->conn, "cfgclear",
        "The Clear All action will clear all Aliases",
        -noprompt, -timeout, 180 );
    if ( $output[0] =~ /The Clear All action will clear all Aliases/i ) {
        Tharn::say( $self->conn, 'yes', -noprompt, -timeout, 180 );
    }
    $self->cfg_save;    #runs cfgsave to save the clear
    my $hash =
      $self->zone_show
      ;    #call zoneshow to check zoneshow structure in order to confirm it
    if (   ( $hash->{cfg_name_array} eq '' )
        && ( $hash->{zone_name_array} eq undef ) )
    {
        NOTE::logcomment("cfgclear: cfgclear successfull\n");
    }
}

=head2 ::zone_show

Runs zoneshow command on a switch, parses its output and populates a hash with all the needed
information about configuration, zone and members. It return a hash which contains all the 
information. Following is the structure of the hash.

    $VAR1 = {
	'NETAPP2' => {
	    'FCVI2' => [
			'13,3',
			'15,3'
			],
            'STOR2' => [
			'13,2',
			'13,5',
			'13,9',
			'13,14',   
			'15,2',
			'15,7',
			'15,9',
			'15,14'
			]
		      },
	 'cfg_name_array' => [
			      'NETAPP',
			      'NETAPP2'
			      ],
       'NETAPP' => {
	   'STOR' => [
		      '13,2',
		      '13,5',
		      '13,9',
		      '13,14',
		      '15,2',
		      '15,7',
		      '15,9',
		      '15,14'
		      ],
	   'FCVI' => [
		      '13,3',
		      '15,3'
		      ]
		    },
        'zone_name_array' => [
			      'FCVI',
			      'FCVI2',
			      'STOR',
			      'STOR2'
			      ]
  };

The above example parsed the following configuration from the swtich:

    mc-sw13:admin> zoneshow
    Defined configuration:
    cfg:   NETAPP  FCVI; STOR
    cfg:   NETAPP2 FCVI2; STOR2
    zone:  FCVI    13,3; 15,3
    zone:  FCVI2   13,3; 15,3
    zone:  STOR    13,2; 13,5; 13,9; 13,14; 15,2; 15,7; 15,9; 15,14
    zone:  STOR2   13,2; 13,5; 13,9; 13,14; 15,2; 15,7; 15,9; 15,14

    Effective configuration:
    no configuration in effect


=over

=item Arguments

None

=item examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my $zone_hash = $sw1->zone_show;
    my @zones = @{$hash->{zone_name_array}}; 
Following code will loop through the whole structure and prints the configuration, its 
coresponding zone and memebrs.

    foreach (@{$zone_hash->{cfg_name_array}}) {
	$cfg = $_;
      NOTE::logcomment("\nConfig : $cfg");
	foreach(@zones) {
	    $zone = $_;
	    foreach my $member (@{$zone_hash->{$cfg}->{$zone}}) {
	      NOTE::logcomment("zone : $zone its  members : $member\t");
		$i++;
	    }
	}
    }

zone_name_array is an array inside the returned hash (returned by the zone_show method) which contains the name of the present zones.
Similarly cfg_name_array is an array inside this hash which contains the name of the present configuration, not neccessarily the effective ones.

=cut

sub zone_show {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self      = shift;
    my $zone_hash = {};
    my $cfg_hash  = {};
    my $i         = 0;
    my $e_flag;
    my $output = $self->say('zoneshow');
    if ( grep( /no configuration in effect/, $output ) ) {
        $e_flag = 0;
        $cfg_hash->{zone_status} =
          'OFF';    #if no configuration is set then set the flag to OFF
    } else {
        $e_flag = 1;
        $cfg_hash->{zone_status} = 'ON';    # vice versa
    }
    foreach ( split( '[\n\r]', $output ) ) {
        last
          if ( $_ =~ /\s*Effective configuration:.*/ ); #skip the effective cfgs
        if ( $_ =~ /zone:/ ) {
            s/.*zone:\s+//
              ;    #remove any space b4 and after 'zone' and 'zone' itself
            s/;//g;    #remove ';' from the line
            my @zone = split( '\s+', $_ );    #split by spaces
            my $zone_name = shift @zone;      #putting zone name from @zone to .
            $zone_hash->{zone_name_array}->[$i] =
              $zone_name;                     #populating zone_name_array
            $zone_hash->{$zone_name} = \@zone; #the rmaining will be the members
            $i++;
        }
    }
    $i = 0;
    foreach ( split( '[\n\r]', $output ) ) {
        last if ( $_ =~ /\s*Effective configuration:.*/ );
        if ( $_ =~ /cfg:/ ) {
            s/.*cfg:\s+//;                     #remove 'cfg:' from the line
            s/;//g;                            #remove ';' from the line
            my @cfg = split( '\s+', $_ );
            my $cfg_name = shift @cfg;
            $cfg_hash->{cfg_name_array}->[$i] =
              $cfg_name;                       #populating cfg_name_array
            foreach (@cfg)
            {   #@cfg contains only the members since we choped of the zone name
                    #pointing zone and members (of cfg_hash) to zone_hash
                    #since it already contains the memebers
                $cfg_hash->{$cfg_name}->{$_} = \@{ $zone_hash->{$_} };
            }
            $i++;
        }
    }
    foreach ( grep( /cfg:/i, split( '[\n\r]', $output ) ) )
    {               #this part is handling the effective cfg names
        if ( $e_flag == 1 ) {    #this means effective config are set
            s/.*cfg:\s+//;
            my @e_cfgs = split( '\s+', $_ );
            $cfg_hash->{effective_config} = shift @e_cfgs;    #pull and populate
        }
    }

    $cfg_hash->{zone_name_array} = $zone_hash
      ->{zone_name_array}; #copying the zone name array in zone_hash to cfg_hash
    return $cfg_hash;
}

=head2 ::set_aptpolicy

Sets the required aptpolicy on the switch. First checks the current aptpolicy, if it is same as what is required to be set then just returns a message saying aptpolicy is same, otherwise disable the switch, changes the aptpolicy and then enables the switch. Croaks if an invalid aptpolicy is provided or the method is not able to set the aptpolicy.

=over

=item Arguments

Required: Aptpolicy

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $sw1->set_aptpolicy($aptpolicy);

=cut

sub set_aptpolicy {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $aptpolicy ) = @_;
    my $current_policy = $self->get_aptpolicy;    # Get the current policy
    if ( $current_policy !=
        $aptpolicy )    #See if the current policy is same as required policy
    {
        $self->say('switchdisable');             #Disable the switch
        my $output =
          $self->say("aptpolicy $aptpolicy");    #Set the required aptpolicy
        $self->say('switchenable');              #Enable the switch
        croak "Could not change aptpolicy : $output"
          if ( $output !~ m/Policy updated successfully/ )
          ;    #Check if policy has been updated, croak if not
        $current_policy =
          $self->get_aptpolicy;    #Verify the policy has been changed
        croak "\n could not change the aptpolicy to $aptpolicy \n"
          if ( $current_policy != $aptpolicy );
        NOTE::logcomment("Apt policy changed succesfully \n");
    } else {
        NOTE::logcomment("Switch has the same aptpolicy \n");
    }
}

=head2 ::get_aptpolicy

Gets the current aptpolicy on the switch. 

=over

=item Arguments


=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $current_policy=$sw1->get_aptpolicy();

=cut

sub get_aptpolicy {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self) = shift;
    my $output =
      $self->say('aptpolicy');    #issue the command to get the aptpolicy
    my @split_output   = split( /: /, $output );
    my @current_policy = split( /\s/, $split_output[1] );
    NOTE::logcomment("Current aptpolicy is $current_policy[0]  \n");
    return $current_policy[0];
}

=head2 ::check_port_traffic

Checks if traffic is flowing on the port number which is passed as an input. Issues a portperfshow command twice at an interval of 5 seconds and checks if the traffic for the given port has changed. If it has changed returns 1 otherwise returns 0.

=over

=item Arguments

Required: port

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    $return=$sw1->check_port_traffic($port);

=cut

sub check_port_traffic {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, $port ) = @_;
    my @ports_with_traffic = $self->port_perf_show;
    my $flag               = 0;

#Get the array of ports on which traffic is changing and check if the passed port no. is there in that array
    foreach (@ports_with_traffic) {
        if ( $_ == $port ) {
            NOTE::logcomment("\n port $port has traffic flowing\n");
            $flag = 1;
            last;
        }
    }
    return $flag;
}

=head2 ::port_perf_show

Issues a port perf show command twice at an interval of 5 secs and captures the first output at each instance. Compares the traffic on all the ports for both the instance and returns a list of ports which has traffic flowing

=over

=item Arguments


=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    @ports_with_traffic=$sw1->port_perf_show();

=cut

sub port_perf_show {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my $perf_output;
    my $before;
    my @response;
    my $i = 0;
    my $j = 0;
    my $k = 0;
    my @port_list;
    my @port_traffic_list;
    my @port_with_traffic;
    my $count;
    my $flag;

    #Get the portperfshow output twice
    my $hostname = $self->host;
    for ( $count = 0 ; $count < 2 ; $count++ ) {
        sleep 4;
        my $port_index         = 0;
        my $port_traffic_index = 0;
        eval {
            @response = Tharn::say( $self->conn, 'portperfshow -t 3',
                -prompt, "\r\n\r\n\r\n", -timeout, 8 );
        };

# Some switches have 3 /r/n separating next set of outputs some have 2, so in case there is 2, above say will timeout and then we'll capture the output and if there are two sets of portperfshow we will split them
        if ($@) {
            @response = ();
            $self->conn->send_intr();
            sleep 4;
            eval {
                @response =
                  Tharn::say( $self->conn, 'portperfshow -t 3', -timeout, "10" );
            };
            $flag = 1;
        }
        $before = $response[2];
        if ($flag) {
            my @data_before = split( /\r\n\r\n\s+0\s+1/, $before );
            $before = $data_before[0];
        }
        sleep 6;
        $self->conn->send_intr();

#Removing extra entries from port perfshow output like =,>,Total, portperfshow and line breaks.
        $before =~
s/((portperfshow -t 3)||(\=)||(\>)||(Total)||(\n)||($hostname)||(admin)||(\r)||(\:))//g;
        my @data = split( /\s+/, $before );

        #Since last and first data is not useful removing that from the array
        shift @data;
        delete $data[-1];
        my $data_count = @data;

# Assuming the switch always has even no. of ports, so total elements in the array will be a multiple of 4. This is required to make sure that
# there are no extra elements in the array expect the port numbers and their traffic part.

        croak
"\n data_count is not a multiple of 4, it is $data_count changes in the library is required \n"
          unless ( ( $data_count % 4 ) == 0 );

#Assuming each row contains 16 entries. This part might require change if the output changes

        my $row_count = $data_count / 16;
        my $port_count=0;
        for ( $i = 0 ; $i < $row_count ; $i++ ) {
            for ( $j = 0 ; $j < 16 &&($port_count<($data_count/2)) ; $j++ ) {
                my $index = $i * 16 + $j;
                $port_count++;
                #Forming an array of port numbers
                if ( $i % 2 == 0 ) {
                    if ( $count == 0 ) {
                        $port_list[$port_index] = $data[$index];
                    } else {
                        croak "Port numbers have changed "
                          if ( $port_list[$port_index] != $data[$index] );
                    }
                    $port_index++;
                }

                #Forming an array of traffic flowing through each ports
                else {
                    if ( $count == 0 ) {
                        $port_traffic_list[$port_traffic_index] = $data[$index];
                    } else {

#Check if the traffic across the port has changed during subsequent portperfshow

                        if ( $port_traffic_list[$port_traffic_index] !=
                            $data[$index] )
                        {
                            $port_with_traffic[$k] = $port_traffic_index;
                            $k++;
                        }
                    }
                    $port_traffic_index++;
                }
            }
        }
    }

    #return the list of ports where traffic is changing
    return @port_with_traffic;

}

=head2 ::check_port_online

Issues a sysconfig on the filer which is passed as an arguments and captures the details of all the switches and ports connected to it.If the switch on which this method is called, is connected to the filer then it checks if the port through which it is connected is online or not.Croaks if the port connected is not online. Logs a message if other switches are connected to the filer . Also returns an array containing the list of connected ports from the switch to the given filer.
=over

=item Arguments
Required : filername

=item Examples

=back

    my $sw1 = NOTE::FcSwitch->connect(host => $SWITCHA);
    my @connected_ports= $sw1->check_port_online('$FILER');

=cut

sub check_port_online {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


    my ( $self, $filer ) = @_;
    my $filer_obj =
      NOTE::Filer->connect( host => $filer );    # Connect to the filer
    my $sysconfig = $filer_obj->say("sysconfig -va");  #Get the sysconfig output
    my @slots     = split( /slot/, $sysconfig );       #Split in groups of slots
    my $i         = 0;
    my @switches;
    my @connected_ports;
    my @switch_port;

    foreach (@slots) {
        if (
/(\w+)\s+Host Adapter.*FC Node Name:\s+(\S+).*Switch Port:\s+(\S+):(\d+)/si
          )
        {
            $switches[$i] = $3
              ; #Split for each slot,if it has switches connected get the switch name and the port no.through which it is connected
            $switch_port[$i] = $4;
            $i++;
        }
    }
    my $host = $self->host;
    my $j    = 0;
    my $k    = 0;

#Iterate for all the ports, if the port belong to the switch on which this method is called, then check the status of port, croak if not online

    foreach (@switch_port) {
        if ( $switches[$j] =~ /$host/ ) {
            my $port_output = $self->say("portshow $switch_port[$j]");
            $connected_ports[$k] = $switch_port[$j];
            $k++;
            if ( $port_output =~ /portState: \d\s+Online/ ) {

                NOTE::logcomment("\n Port $switch_port[$j] is online \n");
            } else {
                croak "\n Port  $switch_port[$j] is not online \n";
            }
        } else {
            NOTE::logcomment(
"\n Port $switch_port[$j] is on $switches[$j] , use this function on switch $switches[$j] \n"
            );
        }
        $j++;
    }
    return @connected_ports;
}

=head2 ::configure_ti_zoning


From the switch object on which zoning is called, islshow method is called and remote switch name is parsed.
The filer name that is passed  is used to get the fcvi ports and storage ports for both the local switch and remote switch
The 1st ISL (RED) is used for FCVI traffic. 2nd ISL (BLUE) is the storage ISL.
Creates the zone member strings to be passed while calling the zone create command.
Create TI zone for FCVI Eg. zone --create -t ti -o f "zone_name" -p "A, ISLportA; A,FCVI-port, B, ISLPortB; B,FCVI-Port"
Creates a config entry, enables the config.
Issues a cfgactv show command on both the local switch and remote switch.

=over

=item Arguments


B<filer> => I<Filer name>
I<Required.>
Filer name to which switch is attached

B<zone_ti_name> => I<Name of the TI Zone>
I<Optional.>
Name to be given to the TI Zone. Defaults to "TI"

B<fcvi_zone_name> => I<Name of the FCVI Zone>
I<Optional.>
Name to be given to the FCVI Zone. Defaults to FCVI

B<stor_zone_name> => I<Name of the STOR Zone>
I<Optional.>
Name to be given to the STOR Zone. Defaults to STOR

B<config_name> => I<Name of the configuration>
I<Optional.>
Name to be given to the configuration . Defaults to config

=item examples

=back

$sw1->configure_ti_zoning(filer=>$FILERA);
$sw2->configure_ti_zoning(filer=>$FILERA);

=cut
sub configure_ti_zoning {

    my ( $self, %args ) = @_;
    my $filer = $args{filer};
    my $fcvi_zone_name =
      defined $args{fcvi_zone_name} ? $args{fcvi_zone_name} : 'FCVI';
    my $stor_zone_name =
      defined $args{stor_zone_name} ? $args{stor_zone_name} : 'STOR';
    my $config_name =
      defined $args{config_name} ? $args{config_name} : 'config';
    my $zone_ti_name =
      defined $args{ti_zone_name} ? $args{ti_zone_name} : 'TI';

    my $remote_switch = $self->isl_show->{1}->{remote_switch};
    my $RED_ISL_A = $self->isl_show->{1}->{local_port};
    my $RED_ISL_B = $self->isl_show->{1}->{remote_port};
    NOTE::logcomment("RED_ISL_A port : $RED_ISL_A   RED_ISL_B port : $RED_ISL_B\n");

    my $sw2           = NOTE::FcSwitch->connect( host => $remote_switch );
    my $mc            = NOTE::FcMetrocluster->new( Filer => $filer );
    my $zone1_members;
    my $zone2_members;
    my $port_info1 =
      $self->port_info;    #calling port_info method to get all the L-ports
    my $port_info2 = $sw2->port_info;
    my $sd1        = $self->switch_show
      ->{switchDomain};    #getting the switch domain of local switch
    my $sd2 = $sw2->switch_show
      ->{switchDomain};    #getting  the switch domain of remote switch
    my ( $hba, @filer_ports ) =
      $mc->get_fcvi_hba( me => 1 );    #getting ports for local
    my ( $hba2, @filer_ports2 ) = $mc->get_fcvi_hba( partner => 1 )
      ; #getting ports for partner because the other switch is on the partner side
    my $local_switch = $self->host;    #getting the switchname
    foreach my $iter (@filer_ports) {
        if ( $hba->{FCVI}->{$iter}->{$local_switch} )
        {    #iterating over the hash by using filer_ports
            $zone1_members .= "$sd1,$hba->{FCVI}->{$iter}->{$local_switch};"
              ;    #building member string  e.g "13,3;
        }
    }
  foreach my $iter (@filer_ports2)
    {              #different filer_ports from the partner side !!
        if ( $hba2->{FCVI}->{$iter}->{$remote_switch} ) {
            $zone1_members .= "$sd2,$hba2->{FCVI}->{$iter}->{$remote_switch};"
              ;    #building with another switch e.g 13,3;15,3;
        }
    }
    chop($zone1_members);    #chopping off the last semicolon i.e 13,3;15,3
    my $fcvi_ti_this = "$sd1,$RED_ISL_A;$sd2,$RED_ISL_B;".$zone1_members;
      NOTE::logcomment("fcvi_ti_this=$fcvi_ti_this\n");

    #Creating STOR members
    foreach my $iter (@filer_ports) {
        if ( $hba->{FC}->{$iter}->{$local_switch} ) {
            $zone2_members .= "$sd1,$hba->{FC}->{$iter}->{$local_switch};";
        }
    }
    foreach my $iter (@filer_ports2) {
        if ( $hba2->{FC}->{$iter}->{$remote_switch} ) {
            $zone2_members .= "$sd2,$hba2->{FC}->{$iter}->{$remote_switch};";
        }
    }
    my $port_hash    = $self->port_info;
    my $port_hash2   = $sw2->port_info;
    my $total_ports  = $port_hash->{total_online_ports};
    my $total_ports2 = $port_hash2->{total_online_ports};
    my ( $port_num, $i );
    for ( $i = 0 ; $i < $total_ports ; $i++ ) {
        if ( $port_hash->{ $port_hash->{port_num_array}->[$i] }->{proto} eq
            'L-Port' )
        {
            $port_num =
              $port_hash->{ $port_hash->{port_num_array}->[$i] }->{port};
            $zone2_members .= "$sd1,$port_num;";
        }
    }
    for ( $i = 0 ; $i < $total_ports2 ; $i++ ) {
        if ( $port_hash2->{ $port_hash2->{port_num_array}->[$i] }->{proto} eq
            'L-Port' )
        {
            $port_num =
              $port_hash2->{ $port_hash2->{port_num_array}->[$i] }->{port};
            $zone2_members .= "$sd2,$port_num;";
        }
    }
    chop($zone2_members);    #removing an extra semicolon

#########Now that we have all the members populated time to create zones and configs################


    my $config_members = "$zone_ti_name;$fcvi_zone_name;$stor_zone_name";
    $self->cfg_create($config_name,$config_members);
    $self->zone_ti_create($zone_ti_name,$fcvi_ti_this);
    $self->zone_create($fcvi_zone_name,$zone1_members);
    $self->zone_create($stor_zone_name,$zone2_members);
    $self->cfg_enable($config_name);
    $self->say("cfgactvshow");
    $sw2->say("cfgactvshow");
}


=head2 ::is_alive

Check if the switch is responding on the network

=over

=item Arguments

=item examples
    
=back
    my $flag = $self->is_alive();
=cut

sub is_alive {
    my ($self) = @_;

    my $hostname = $self->host();
    system("ping -c 1 $hostname");

    my $rc = $? >> 8;
    if ($rc == 0) {
        return 1;
    } else {
        return 0;
    }
}

=head1 SEE ALSO

<C<NOTE>|engineering::ontap_tests::agnostic::docs::lib-NOTE-pm/>

=head1 STATUS

Public.

$Id: //depot/prod/test/Rsierranevada/lib/NOTE/FcSwitch.pm#1 $

$Date: 2013/01/22 $

=head1 AUTHOR

dl-dpgqa-automation@netapp.com

jawwad@netapp.com

=cut

sub DESTROY {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;

    return if $self->hostrec->hosttype =~ /switch/i;
    return if ( $$ != $self->{_owner} );
    return if ( $self->{_destroyed} );

    $self->{_destroyed} = 1;
    NOTE::logcomment( "Disconnecting from switch " . $self->host );
    Tharn::disconnect( $self->{_hostconn} ) if ( $self->{_hostconn} );
}

1;

