#
# $Id: //depot/prod/test/main/lib/NOTE/FileBlockCorruption.pm#1 $
#
## @summary  NOTE FileBlockCorruption class
## @status   Public
## @pod      here
## @author   shail@netapp.com, dl-dpgqa-automation@netapp.com

=head1 NAME

NOTE::FileBlockCorruption

=head1 DESCRIPTION

This package provides a method for file block corruption at L0 and L1 inturn uses wreck and wafl iron

=head1 METHODS

=cut


package NOTE::FileBlockCorruption;

use NOTE;
use strict;
use base 'Exporter';
our @EXPORT = qw(init_fileblock_corruption);
use misc qw(cli);
use SnapCmds qw(snapshot_exists snapshot_create snapshot_delete);
use filer qw(filer_kernel_is_debug filer_file_destroy filer_showfh
  filer_file_exists);
use Volume::Setup qw(setup_volumes);
use Volume qw(vol_online);
use Wreck qw(wreck);
use boot qw(transit_to);
use Filer::Version qw(:releases get_wafl_version);
use ExportFS qw(export_path);
use FS::Util::Setup qw(common_cleanup mount_vol umount_vol);
use FS::Util::Utilities qw(write_file dumpblock version_check_ic2);
use Wafliron qw(wafliron_start);
use SayWrap qw(sayWAIT);
use FS::commonParams;
use Carp qw(croak);

my $Conn            = undef;
my $Cc              = undef;
my $Ch              = undef;
my $Flags_Hash      = undef;
my $Filename1       = undef;
my $Filename2       = undef;
my $Filename3       = undef;
my $Filename4       = undef;
my $FILER_VOLTEST1  = undef;
my $FILER_VOLTEST2  = undef;
my $Host            = undef;
my $Mount_Point     = undef;
my $Path1           = undef;
my $Path2           = undef;
my $Path3           = undef;
my $Path4           = undef;
my $Ref_Vol_Created = undef;
my $SUDO_PATH       = undef;
my $Snapname        = undef;
my $VOL_TYPE        = "flex";
my $VOL_SIZE        = "2g";
my $AGGR_SIZE       = "3";
my $FILER           = undef;
my $FILER_CONN      = "#rsh-socket";
my $CLIENT          = "localhost";
my $CLIENT_CONN     = undef;
my $FILER_AGGR      = 'aggr0';
my $CLEANUP         = undef;
my $filer           = undef;


=head2 ::new

Creates a FileBlockCorruption Object

=over

=item Arguments

none

=item Examples

=back

     $fbcObj = new NOTE::FileBlockCorruption();

=cut


sub new {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $package = shift;
    return bless( {}, $package );
}


=head2 ::init_fileblock_corruption

Performs L0/L1 file block corruptions and returns to the callee

=over

=item Arguments

B<FILER> => I<Source Filer>

I<Required.>
filer name, tharnhost file name

=item Examples

=back

     $fbcObj->init_fileblock_corruption(FILER=>'f3170-215-173');

=cut

sub init_fileblock_corruption($,%) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ( $self, %args ) = @_;
    $FILER =
      exists( $args{FILER} )
      ? $args{FILER}
      : croak("init_fileblock_corruption requires filer");
    &_modinit();
    &_setup();
    &_userfile_level0_afs();
    &_userfile_level1_afs();
    &_cleanup();
}

#Diagnostic and init routine to check the filer connectivity and establish connection...
sub _modinit() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    $Filename1 = "file1";
    $Filename2 = "file2";
    $Filename3 = "file1_30m";
    $Filename4 = "file2_30m";
    $Path1     = "/vol/$FILER_VOLTEST1/$Filename1";
    $Path2     = "/vol/$FILER_VOLTEST2/$Filename2";
    $Path3     = "/vol/$FILER_VOLTEST1/$Filename3";
    $Path4     = "/vol/$FILER_VOLTEST2/$Filename4";
    $Snapname  = "vol2_snap";
    eval {
        $filer = NOTE::Filer->connect(
            host  => $FILER,
            mode  => $FILER_CONN,
            stats => "-r -n"
        );
        $Host = $filer->host or croak("$@");
        $Conn = $filer->conn or croak("$@");
    };
    if ($@) {
        NOTE::logcomment( "Filer connection failed: $FILER_CONN " . $@ );
    }
}

#setup routine to create requisites for L0 and L1 corruption, includes creation of aggregates, volumes etc...
#this module also sets the corruption flag required prior invoking wreck function...
sub _setup() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    eval {
        NOTE::logcomment(
"Install debug kernel and verify that the version installed is debug kernel"
        );
        if ( !filer_kernel_is_debug( $filer->conn ) ) {
            croak("Filer needs to run on a debug kernel");
        }
        NOTE::logcomment( "Aggregate($FILER_AGGR) and volume($FILER_VOLTEST1, "
              . "$FILER_VOLTEST2) needed "
              . "for the test are created below" );
        $Host = $filer->hostrec;
        ( $Ref_Vol_Created, undef ) = setup_volumes(
            $Host,
            {
                aggr  => $FILER_AGGR,
                type  => 'aggr',
                disks => $AGGR_SIZE
            },
            {
                vol      => $FILER_VOLTEST1,
                type     => 'flex',
                aggr     => $FILER_AGGR,
                vol_size => $VOL_SIZE
            },
            {
                vol      => $FILER_VOLTEST2,
                type     => 'flex',
                aggr     => $FILER_AGGR,
                vol_size => $VOL_SIZE
            }
        );

        $filer->NOTE::logcomment("Saving value of flags");

        $Flags_Hash = $filer->printflag("wafl_inconsistent_panic_testing "
              . "wafl_iron_panic_on_corruption "
              . "wafl_lwr_trip");
        NOTE::logcomment("Setting the flag wafl_inconsistent_panic_testing to 1");
        NOTE::logcomment("Verfying the flag value has been set already or not");
        if ( $Flags_Hash->{'wafl_inconsistent_panic_testing'} != 1 ) {
        $filer->setflag(flag  => "wafl_inconsistent_panic_testing",value => 1);
        }
        else {
            NOTE::logcomment("Flag value has been set already.");
            delete( $Flags_Hash->{'wafl_inconsistent_panic_testing'} );
        }


        NOTE::logcomment("Flag wafl_iron_panic_on_corruption is being set to 0");

        NOTE::logcomment("Verfying the flag value has been set already or not");
        if ( $Flags_Hash->{'wafl_iron_panic_on_corruption'} != 0 ) {
        $filer->setflag(flag  => "wafl_iron_panic_on_corruption",value => 0);
        }
        else {
            NOTE::logcomment("Flag value has been set already.");
            delete( $Flags_Hash->{'wafl_iron_panic_on_corruption'} );
        }
        if ( $Flags_Hash->{'wafl_lwr_trip'} != 0 ) {
            $filer->setflag(flag  => "wafl_lwr_trip",value => 0);
        }
        else {
            NOTE::logcomment("Flag value has been set already.");
            delete( $Flags_Hash->{'wafl_lwr_trip'} );
        }

        NOTE::logcomment("Flag value have been set successfully");

    };

    if ($@) {
        NOTE::logcomment( "Setup configuration error" . $@ );
    }
}

#cleanup routine cleans up the files that were wrecked with injection string and cleans the snapshot if there were any...
sub _cleanup() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    NOTE::logcomment("CLEANUP");
    eval {
        if ( $CLEANUP ne 'none' )
        {
            NOTE::logcomment("Starting the wafliron");
            my ( $found, undef ) = wafliron_start( $Conn, $FILER_AGGR );
            if ( !defined($found) ) {
                NOTE::logcomment("Wafliron did not start");
            }
            NOTE::logcomment("Removing the files created for testing purpose");

            if ( filer_file_exists( $Conn, $Path1 ) ) {
                NOTE::logcomment("Removing file $Path1");
                filer_file_destroy( $Conn, $Path1 );
                NOTE::logcomment("File $Path1 removed");
            }

            if ( filer_file_exists( $Conn, $Path2 ) ) {
                NOTE::logcomment("Removing file $Path2");
                filer_file_destroy( $Conn, $Path2 );
                NOTE::logcomment("File $Path2 removed");
            }

            if ( filer_file_exists( $Conn, $Path3 ) ) {
                NOTE::logcomment("Removing file $Path3");
                filer_file_destroy( $Conn, $Path3 );
                NOTE::logcomment("File $Path2 removed");
            }

            if ( filer_file_exists( $Conn, $Path4 ) ) {
                NOTE::logcomment("Removing file $Path4");
                filer_file_destroy( $Conn, $Path4 );
                NOTE::logcomment("File $Path2 removed");
            }

            NOTE::logcomment("Removing the snapshots created");
            if ( snapshot_exists( $Conn, $FILER_VOLTEST2, $Snapname ) ) {

                NOTE::logcomment("Delete the snapshot");
                snapshot_delete( $Conn, $FILER_VOLTEST2, $Snapname );
            }
        }
        common_cleanup( $Conn, vol_list => $Ref_Vol_Created );

    };
    if ($@) {
        NOTE::logcomment($@);
    }
    NOTE::logcomment("Cleanup Completed");
}

# performs l0 corruption
sub _userfile_level0_afs() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    eval {
        my ( $fileid, $pvbn, $inject_string, $cmd, $filesize );
        _mount_nfs_client();
        $filesize = "100k";
        ( $fileid, $pvbn ) = _get_fileid_pvbn( $Path1, $Path2, $filesize );
        NOTE::logcomment("Inject corruption into L0s in the AFS using wreck");
        $inject_string =
          "ind_vol_bno inum=$fileid level=1 bno=0 new_bno=$pvbn index=0";
        _wreck_corruption($inject_string);
    };
    if ($@) {
        NOTE::logcomment( "The function userfile_level0s_af() failed" . $@ );
    }
}

# performs l1 corruption
sub _userfile_level1_afs() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    eval {
        my (
            $fileid,   $pvbn,   $inject_string, $cmd,
            $logh,     @output, $filer_console, $matched,
            $err_msgs, $filesize
        );
        $filesize = "30m";
        ( $fileid, $pvbn ) = _get_fileid_pvbn( $Path3, $Path4, $filesize );
        NOTE::logcomment("Inject corrutpion in L1s in the AFS using wreck");
        $inject_string =
          "ind_vol_bno inum=$fileid level=2 bno=0 new_bno=$pvbn index=0";

        _wreck_corruption($inject_string);
        $filer->setflag(flag  => "wafl_iron_panic_on_corruption",value => 0);
    };

    if ($@) {
        NOTE::logcomment( "The function userfile_level1_afs() failed" . $@ );
    }
}

# helper routine to get the file id and volume block number required by wreck
sub _get_fileid_pvbn() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


    my ( $path1, $path2, $file_size ) = @_;
    my ( $output, $result, %showfh, $dump, $file_id, $pvbn, $pvbn_num );
    NOTE::logcomment("Create the files with respective path and size");
    NOTE::logcomment(
        "Create files on volume " . "$FILER_VOLTEST1 and $FILER_VOLTEST2" );
    foreach my $path ( $path1, $path2 ) {
        write_file(
            filer => $filer->host,
            path  => $path,
            size  => $file_size
        ) or croak("$@");
    }
    NOTE::logcomment("The fileid of file is extracted below");
    NOTE::logcomment($Conn);
    NOTE::logcomment($Host);
    %showfh = filer_showfh( $filer->conn, $path2 );
    $file_id = hex( $showfh{fileid} );

    if ( !defined($file_id) ) {
        croak("The extraction of \$file_id failed");
    }
    NOTE::logcomment("The file id of $path2 is $file_id ");
    NOTE::logcomment("The diskid of the file is extracted below");
    NOTE::logcomment("Clearing the buffer");
    cli( $FILER, "priv set diag;wafl flush cache" );
    $dump = dumpblock(
        conn       => $filer->conn,
        block_type => 'info',
        volume     => $FILER_VOLTEST1,
        file       => $path1,
        blockno    => 0,
        level      => 0
    ) or croak("$@");

    foreach $pvbn ( keys %$dump ) {
        if ( $pvbn eq "pvbn" ) {
            $pvbn_num = $dump->{$pvbn};
        }
    }

    if ( !defined($pvbn_num) ) {
        croak("The extraction of \$pvbn_num failed");
    }
    NOTE::logcomment("The output of \$pvbn_num is $pvbn_num");
    return ( $file_id, $pvbn_num );

}

# corrupt the blocks with specified corruption parameters....
sub _wreck_corruption() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $string = shift;
    wreck(
        $Conn,
        $FILER_VOLTEST2,
        type          => "flex",
        inject_script => $string,
    ) or croak( "Wrecking failed" . $@ );
    NOTE::logcomment("Online the volume");
    vol_online( $Conn, $FILER_VOLTEST2 )
      or croak("Unable to make volume online");
}

# mounts the volumes created on the filer...........
sub _mount_nfs_client() {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    NOTE::logcomment("Mount the vol $FILER_VOLTEST2 on nfs client");
    export_path( $filer->conn, "/vol/$FILER_VOLTEST2",
        OPTIONS => "sec=sys,anon=0,rw" )
      or croak("Unable to export $FILER_VOLTEST2");
    my @vols = $filer->Volumes;
    foreach (@vols) {
        $_->mount();
        $Mount_Point = $_->mount_path;
        NOTE::logcomment($Mount_Point);
    }
}
1;
__END__
