#
# $Id: //depot/prod/test/main/lib/NOTE/NBU/NBU_sim.pm#1 $
#
## @summary  P3 image_test library routines.
## @status   Public
## @pod      here
## @author   weili@netapp.com dl-dpgqa-automation@netapp.com 

=head1 NAME

NOTE::NBU::NBU_sim

=head1 DESCRIPTION

This is the NOTE class for creating a p3 image_test object to handle p3 backups

=head1 METHODS

=cut

package NOTE::NBU::NBU_sim;

our @ISA = qw(NOTE);

use strict;
use Carp;
use Tharn;
use NOTE;
use Filer::Version qw (WV_GORDONBIERSCH_MAX);
use SayWrap;
use vars qw ($MyClientFc); 


BEGIN {
	(my $host = `hostname`) =~ s/\..*//;

	my $fh = host($host) || die $@;                        # Filer's host object
	$MyClientFc = connect("#rsh") || die $@; 

}

# Expose the following registry keys on the object
my $regSlice = {
    'snapvault.enable' => sub {
        'options.snapvault.enable'
    },
    'snapvault.access' => sub {
        'options.snapvault.access'
    },
    'snapvault.nbu.trace_category' => sub {
        'options.snapvault.nbu.trace_category'
    },
    'snapvault.nbu.trace_level' => sub {
        'options.snapvault.nbu.trace_level'
    },
    'snapvault.nbu.verify_backup' => sub {
        'options.snapvault.nbu.verify_backup'
    },
    'snapvault.nbu.verify_duplicate' => sub {
        'options.snapvault.nbu.verify_duplicate'
    },
};


=head2 B<Basic Operations>

=head2 ::new

Create a p3 image_test object  

=over

=item Arguments

B<volume> => I<object/string>

I<Required.>
A NOTE Volume object.

B<access> => I<string>

I<Optional.>
Set up snapvault.access option.

B<enable> => I<string>

I<Optional.>
Set up snapvault.enable option.

B<category> => I<string>

I<Optional.>
Set up snapvault.nbu.trace_category option.

B<level> => I<string>

I<Optional.>
Set up snapvault.nbu.trace_level option.

B<options_snapvault_nbu_verify_backup> => I<string>

I<Optional.>
Set up snapvault.nbu.verify_backup option.

B<duplicate> => I<string>

I<Optional.>
Set up snapvault.nbu.verify_duplicate option.


B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_ + random number

B<image_name> => I<string>

I<Optional>
image name of image_test, default: current timestamp 

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<action> => I<string>

I<Optional>
image_test type: <r (read)|w (write)|c (start unpacking)|  d (delete)| l (list)| p (properties) |  ll 
(LSU list) | df (delete files) | rc (read and compare)>, default: l 

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>  <incr|full>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

B<log_msgs> => I<string>

I<Optional>
image_test msg log: yes/no, default: no

B<log_writes> => I<string>

I<Optional>
image_test write log: yes/no, default: no

B<log_reads> => I<string>

I<Optional>
image_test read log: yes/no, default: no

B<_Asis_License> => I<string>

I<Optional>
Set asis license: y/n, default: n

B<_Asis> => I<string>

I<Optional>
Enable asis: on/off, default: off 



=item Examples

=back
	$sim = NOTE::NBU::NBU_sim->new(
			volume=>$vol    -- $vol is a NOTE:Volume object
		);

=cut

sub new {
	my ($class, %args) = @_;
	my $self = {};

	$self->{_owner}   = $$;
	
	if ( !$args{volume} || !$args{volume}->isa('NOTE::Volume') ){
		croak "A NOTE::Volume is required!";
	}
	
	$self->{_Volume} = $args{volume};
	$self->{_Filer} = $self->{_Volume}->Filer; 
	
	my @name = split /\_/, NOTE::newName(); ## to make policy name shorter
	my $name = $name[-1];

	$self->{_Base_name} = $args{base_name} ? $args{base_name} : 'base_'.$name;
	$self->{_Policy} = $args{policy} ? $args{policy} : 'policy_'.$name;
	$self->{_Image_name} = $args{image_name} ? $args{image_name} : time(); 
	$self->{_Tar_path} = $args{tar_file_path} ? $args{tar_file_path} : 'tarpath';
	$self->{_Meta_path} = $args{meta_file_path} ? $args{meta_file_path} : 'metapath';
	$self->{_Action} = $args{action};
	$self->{_Image_type} = $args{image_type} ? $args{image_type} : 'clear';
	$self->{_Backup_type} = $args{backup_type} ? $args{backup_type} : 'full';
	$self->{_Log_msgs} = $args{log_msgs} ? $args{log_msgs} : 'log_msgs=no';
	$self->{_Log_writes} = $args{log_writes} ? $args{log_writes} : 'log_writes=no';
	$self->{_Log_reads} = $args{log_reads} ? $args{log_reads} : 'log_reads=no';
	$self->{_Asis_License} = $args{asis_license} ? $args{asis_license} : 'n';
	$self->{_Asis} = $args{asis} ? $args{asis} : 'off';
	
	bless $self, $class;

  	# Make this object a tie variable so registry options can be accessed
    	# via a hash element of the object
    	tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;

	if (!$args{no_setup}){
		$self->{'snapvault.enable'} = exists $args{enable} ? $args{enable} : 'on';
		$self->{'snapvault.access'}= exists $args{access} ? $args{access} : 'all';
		$self->{'snapvault.nbu.trace_category'} = exists $args{category} ? $args{category} : 'all';
		$self->{'snapvault.nbu.trace_level'} = exists $args{level} ? $args{level} : 100;
		##$self->{'snapvault.nbu.verify_backup'} = exists $args{backup} ? $args{duplicate} : '';
		##$self->{'snapvault.nbu.verify_duplicate'} = exists $args{duplicate} ? $args{duplicate} : '';


		NOTE::logcomment("set sv_ontap_sec licenses");
       		$self->{_Filer}->check_license('sv_ontap_sec');
		croak $@ if $@;

		NOTE::logcomment("set nearstore_option license");
        	my $filer_model = $self->{_Filer}->filer_model();
        	if ($filer_model =~ /FAS30|FAS60|FAS31/) {
               		$self->{_Filer}->check_license('nearstore_option');
			croak $@ if $@;		
        	}
		else {
               		if ( $filer_model !~ /R200/) {croak "$filer_model is not a supported platform!"};
		}

		if ($self->{_Asis_License} eq 'y') {
			NOTE::logcomment("set a_sis license");
			## need to find out which version the a_sis started
       			if ($self->{_Filer}->wafl_version()> WV_GORDONBIERSCH_MAX){
       	        		$self->{_Filer}->check_license('a_sis');
				croak $@ if $@;
       			}
       			else {
       				$self->{_Filer}->check_license('nearstore_asis2');
				croak $@ if $@;
       			}

			Dense::dense_toggle(host=>$self->{_Filer}->host, path_name=>$self->{_Volume}->mount_path, state=>$self->{_Asis});
    			croak $@ if $@;
		}

		SnapCmds::snapsched_volume($self->{_Filer}->conn, $self->{_Volume}->name, "0 0 0");
		} ## end of no_setup

	NOTE::reference( $self );
	return $self;
}


=head2 ::attach

Creates an object representing an existing NBU_sim object.

=over

=item Arguments

B<volume> => I<object/string>

I<Required.>
A NOTE Volume object.

=item Arguments

B<base_name> => I<string>

I<Required>
Base name of image_test

B<policy> => I<string>

I<Required>
Policy name of image_test

B<image_name> => I<int>

I<Required>
image backup level

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back

	NOTE::NBU::NBU_sim->attach( 
				volume=>$vol, 
				base_name=>$sim->{_Base_name}, 
				policy=>$sim->{_Policy}, 
				image_name=>$sim->{_Image_name});
=cut

sub attach {
	my ($class, %args) = @_;
	my $self = {};

	if ( !$args{volume} || !$args{volume}->isa('NOTE::Volume') ){
		croak "A NOTE::Volume is required!";
	}

	if ( ref $class ne '' ) {
		$self = $class;
    	}
    	else {
		# Instantiate this object
        	$self = NOTE::NBU::NBU_sim->new(no_setup=>1, %args );
    	}

    	return $self;
}


## handle image_test command
sub _execute_image_test{

	my $self = shift @_;

	my $filer_with_cred = $self->{_Filer}->host.':root@'.param( 'PASSWORD', '-default', 'netapp' );
	my $image_test = param( 'TOOLSDIR', '-default', '/usr/local/test/bin').'/image_test';

	## make sure image and image_test exist.
        if ( $self->{_Action}=~ /w|c|wc|r|rc|p/){
             _check_p3_image($self->{_Tar_path}, $self->{_Meta_path}, $image_test);
        }
	
	my $command = "/usr/local/bin/sudo $image_test $filer_with_cred $self->{_Base_name} $self->{_Policy} $self->{_Image_name} $self->{_Tar_path} $self->{_Meta_path} $self->{_Action} /vol/$self->{_Volume}->{_name} $self->{_Image_type} $self->{_Backup_type} $self->{_Log_msgs} $self->{_Log_writes} $self->{_Log_reads} ";
	
        my $result;
	eval {$result =  sayEOF($MyClientFc, $command);};
	croak "operation failed: $@" if $@;
        return $result;
}

## set image_test parameters is any
sub _set_param {
	my $self = shift @_;
	my %args = @_;

	if ($args{base_name} ) { $self->{_Base_name} = $args{base_name}} ;
        if ($args{policy}) { $self->{_Policy} = $args{policy} };
        if ($args{image_name}) { $self->{_Image_name} = $args{image_name} }; 
        if ($args{tar_file_path}) { $self->{_Tar_path} = $args{tar_file_path}};
        if ($args{meta_file_path} ) { $self->{_Meta_path} = $args{meta_file_path} } ;
        if ($args{image_type}) {$self->{_Image_type} = $args{image_type} };
        if ($args{backup_type}) {$self->{_Backup_type} = $args{backup_type} };
        if ($args{log_msgs}) {$self->{_Log_msgs} = $args{log_msgs} };
        if ($args{log_writes}) {$self->{_Log_writes} = $args{log_writes} };
        if ($args{log_reads}) {$self->{_Log_reads} = $args{log_reads} };

	return $self;
}

# Check pre-requisites prior to running.
sub _check_p3_image {
        my ($tarfile, $metafile, $image_test) = @_;

        unless (-e $tarfile) {
                croak "Specified tar doesn't exist: $tarfile" ;
        }
        unless (-e $metafile) {
                croak "Specified tar metafile doesn't exist: $metafile";
        };
        unless (-e $image_test) {
                croak "image_test doesn't exist: $image_test" ;
        }
        return 1;
}



=head2 ::image_write

image_test: backup a tar file to a filer 

=over
/
=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_ + random number

B<image_name> => I<int>

I<Optional>
image backup level, default: current timestamp

B<tar_file_path> => I<string>

I<required>
Path of the tar file

B<meta_file_path> => I<string>

I<required>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back

	$sim->image_write ( 
		$vol,
		tar_file_path=>'/net/filer_sim/vol/compat_data/nightlies/un_expandable.img', 
		meta_file_path=>'/net/filer_sim/vol/compat_data/nightlies/un_expandable.img.output'
	);

=cut

sub image_write {
	my $self = shift @_;
	
	_set_param($self, @_);
	$self->{_Action} = 'w';
	$self->{_Image_name} = time();

	NOTE::logcomment("image_test Create ...");
	
	my $result = _execute_image_test($self);
	
	if ($result =~ /.+Message reply error: (\d+).+/sm) {
		eval { croak "image_test create failed with error=$1..."}; 
		return $1;
        }
	
        NOTE::logcomment("Write image successful");

	return 1;
}

=head2 ::image_close

close an image

=over

=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_ + random number

B<image_name> => I<int>

I<Optional>
image backup level, default: current timestamp

B<tar_file_path> => I<string>

I<optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back
        $sim->image_close();

=cut

sub image_close {
	my $self = shift @_;

        _set_param($self, @_);
        $self->{_Action} = 'c';
	
	NOTE::logcomment("image_test Close ...");

        my $result = _execute_image_test($self);
	my $error_code;
	if ($result =~ /.+Message reply error: (\d+).+/sm) {
                $error_code = $1;
                if ($result =~ /.+qtree is currently involved in another operation.+close retry worked.+/sm) 
		{
                       	NOTE::logcomment("image_close(): Encountered QTREE_BUSY_ERR but retry image_close succeeded");
               		return 1;
               	} 
		else {
       			eval {croak "image_close() failed: aborting with error=$error_code..."};
			return $error_code;
                }
  	}
	
        NOTE::logcomment("Close image successful");

	return 1;
}


=head2 ::image_write_close

image_test: write and close

=over

=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_+random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_+random number

B<image_name> => I<int>

I<Optional>
image backup level, default: current timestamp

B<tar_file_path> => I<string>

I<required>
Path of the tar file

B<meta_file_path> => I<string>

I<required>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back


=cut
sub image_write_close{
        my $self = shift @_;

        _set_param($self, @_);
        $self->{_Action} = 'wc';
        $self->{_Image_name} = time();

        NOTE::logcomment("image_test Create and Close ...");

        my $result = _execute_image_test($self);

        if ($result =~ /.+Message reply error: (\d+).+/sm) {
                eval { croak "image_test create failed with error=$1..."};
                return $1;
        }

        NOTE::logcomment("Write and close image successful");

        return 1;
}




=head2 ::image_read

read an image

=over

=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_ + random number

B<image_name> => I<int>

I<Required>
image backup level, to state which image need to restore

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back
        $sim->image_read();

=cut

sub image_read {
        my $self = shift @_;
	my %args = @_;

	croak "No image_name specified in image_read!" unless $args{image_name};

        _set_param($self, %args);
        $self->{_Action} = 'r';

	NOTE::logcomment("image_test Read ...");

        my $result = _execute_image_test($self);
	if ($result =~ /.+Message reply error: \d+.+/sm || $result =~ /ERROR:/) { 
		eval { croak "image_test read failed with error=$1..."}; 
		return $1;
	}

        NOTE::logcomment("Read image successful");

        return 1;
}

=head2 ::image_read_compare

read and compare an image

=over

=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_+random number

B<image_name> => I<int>

I<Required>
image backup level, to specify which backup's timestamp 

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back
        $sim->image_read_comapre();

=cut


sub image_read_compare {
        my $self = shift @_;
	my %args = @_;

	croak "No image_name specified in image_read_compare!" unless $args{image_name};

        _set_param($self, %args);
        $self->{_Action} = 'rc';
	
        NOTE::logcomment("image_test Read and comapre...");

        my $result = _execute_image_test($self);
        if ($result =~ /.+Message reply error: \d+.+/sm || $result =~ /ERROR:/) { 
		eval {croak "image_test read_compare failed with error=$1..."};
		return $1;
	}

	NOTE::logcomment("Comparison of image successful");

        return 1;
}


=head2 ::image_delete

Delete the give image 

=over

=item Arguments

B<base_name> => I<string>

I<Required>
Base name of image_test

B<policy> => I<string>

I<Required>
Policy name of image_test

B<image_name> => I<int>

I<Required>
image backup level, to state which image need to delete

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back
        $sim->image_delete ();

=cut

sub image_delete {
        my $self = shift @_;
	my %args = @_;

	croak "No base_name specified in image_delete!" unless $args{base_name};
	croak "No policy specified in image_delete!" unless $args{policy};
	croak "No image_name specified in image_delete!" unless $args{image_name};

        _set_param($self, @_);
        $self->{_Action} = 'd';

        NOTE::logcomment("image_test delete ...");

        my $result = _execute_image_test($self);
	if ($result =~ /.+Message reply error: \d+.+/sm || $result =~ /ERROR:/) {
		NOTE::logcomment("image_test delete failed with error=$1...");
		return $1;
        }

	NOTE::logcomment("Delete image successful");
	
	return 1;
}

=head2 ::image_list

List images on the given volume, will return a hash list of images :

	 return %image_list = ('base_name'=>'my_base'
			'policy'=>'my_policy',
			'date'=>'my_date' 
			)

=over

=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_ + random number

B<image_name> => I<int>

I<Optional>
image backup level, default: current timestamp

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back
        %image_list = $sim->image_list ();

=cut

sub image_list {
	my $self = shift @_;

        _set_param($self, @_);
        $self->{_Action} = 'l';

        NOTE::logcomment("image_test list ...");

        my $result = _execute_image_test($self);

	##if ($result =~ /.+Message reply error: \d+.+/sm || $result =~ /ERROR:/) {
	if ($result =~ /.+Message reply error: \d+.+/sm ) {
		NOTE::logcomment("image_test list failed with error=$1...");
		return $1;
        }
	my @list = split(/\n/, $result);

	## error msg here, check @list
	my (@img_list, @tmp, $value, );
	my $img = {};
	foreach $value (@list) {
		if ($value=~ /:Policy/){
			@tmp = split /\s+/, $';
			@tmp = grep (/:/,@tmp);
			($img->{base_name}, $img->{policy}) = split /:/, $tmp[0];
		}
		elsif ($value=~ /Date:/){
			@tmp = split /\s+/, $';
			@tmp = grep (/,/,@tmp);
			chop $tmp[0];
			$img->{image_name} = $tmp[0];	
			push @img_list, $img;
			$img = {};
		}
	}

	return @img_list;
}

=head2 ::image_property

List images properties of the given volume, will return a hash list of properties :

        %image_property = ('imo_server'=>'my_server',
                        'imo_lsu'=>'my_lsu',
                        'imo_size'=>'my_size',
			'imo_block_size'=>'my_block_size',
			'imo_status'=>'my_status' 
                        )

=over

=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_ + random number

B<image_name> => I<int>

I<Optional>
image backup level, default: current timestamp

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back
        %image_property = $sim->image_property();

=cut

sub image_property {
        my $self = shift @_;

        _set_param($self, @_);
        $self->{_Action} = 'p';

        NOTE::logcomment("image_test property ...");

        my $result = _execute_image_test($self);
	if ($result =~ /.+Message reply error: \d+.+/sm ) {
		NOTE::logcomment("image_test property failed with error=$1...");
		return $1;
        }

        my @list = split /\n/, $result;

	my (%property, $value);
	foreach $value (@list) {

	my @elements = split /\s+/, $value;
		if ($elements[0] =~ /imo_server/) {
			$property{imo_server} = $elements[-1];
		}
		elsif ($elements[0] =~ /imo_lsu/) {
			$property{imo_lsu} = $elements[-1];
		}
		if ($elements[0] =~ /imo_size/) {
			$property{imo_size} = $elements[-1];
		}
		if ($elements[0] =~ /imo_block_size/) {
			$property{imo_block_size} = $elements[-1];
		}
		if ($elements[0] =~ /imo_status/) {
			$property{imo_status} = $elements[-1];
		}
	} 

        return %property;
}


=head2 ::image_lsu

List LSU of the given filer, will return a list of LSU 

=over

=item Arguments

B<base_name> => I<string>

I<Optional>
Base name of image_test, default: base_ + random number

B<policy> => I<string>

I<Optional>
Policy name of image_test, default: policy_ + random number

B<image_name> => I<int>

I<Optional>
image backup level, default: current timestamp

B<tar_file_path> => I<string>

I<Optional>
Path of the tar file

B<meta_file_path> => I<string>

I<Optional>
Path of the meta file

B<image_type> => I<string>

I<Optional>
image type: <clear (image unpackable)| image (not unpackable)|soi>, default: clear

B<backup_type> => I<string>

I<Optional>
backup type: <incr|full>, default: full

=item Examples

=back
        %image_property = $sim->image_property();

=cut

sub image_lsu {
        my $self = shift @_;

        _set_param($self, @_);
        $self->{_Action} = 'll';

        NOTE::logcomment("image_test LSU list...");

        my $result = _execute_image_test($self);
	if ($result =~ /.+Message reply error: \d+.+/sm) {
		NOTE::logcomment("image_test LSU failed with error=$1...");
		return $1;
        }
        my @list = split(/\n/, $result);

        my (@lsu, $value);
        foreach $value (@list) {
		if ($value =~ /LSU list:/) {
			push @lsu, $';
                }
        }

        return @lsu;
}


# Object destructor
sub DESTROY {
    my $self = shift;

    # Only the creator can destroy the object
    return if ( $$ != $self->{_owner} );
    return if ( $self->{_spare_me} );    # Means don't destroy
    return if ( $self->{_destroyed} );   # Already dead
    return if ( NOTE->ALL_SPARED );

    # Destroy object
    ###$self->destroy;  
    $self->unreference;
}

