# $Id: //depot/prod/test/Rsierranevada/lib/NOTE/NDMP.pm#1 $
#
## @summary  NOTE NDMP
## @status   Public
## @pod      here
## @author   vvatsal@netapp.com, dl-dpgqa-automation@netapp.com

=head1 NAME

NOTE::NDMP

=head1 DESCRIPTION

This package provides methods for performing NDMP functions

=head1 METHODS

=cut
package NOTE::NDMP;
our @ISA = qw(NOTE);

use POSIX qw();
use strict;
use FSR::FSRutils;
use FSR::DR::DRutils;
use Data::Dumper;
use SayWrap qw(sayEOF);
use blocks_filer qw();
use NDMP::NdmpCmds;
use Tharn;
use Carp;
my $ret;

# ###########################################################
# # NDMP new Method
# ###########################################################


=head2 ::new

Creates a Ndmp Object on specified filers

=over

=item Arguments

B<src> => I<Source Filer Object>

I<Required.>
Object of Source Filer on which NDMP operations would be done.

B<dst> => I<Destination Filer Object>

I<Optional.>
Object of Destination Filer on which NDMP operations would be done.Defaults to source filer object.



=item Examples

=back

    $ndmp =NOTE::NDMP->new(
                src=>$filer_src,
                dst=>$filer_dst
    );

=cut


sub new {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%args) = @_;
    my $self = {};
    $self->{_owner} = $$;
    $self->{_src} = $args{src};
    $self->{_dst}= defined $args{dst} ? $args{dst}:$args{src};
    my $filer1_name=$self->{_src}->host;
    my $filer2_name=$self->{_dst}->host;
    NOTE::logcomment("Creating a new NDMP object with filers $filer1_name and $filer2_name \n");
    bless $self, $class;
    NOTE::reference( $self );
    croak $@ if $@;
    return $self;
}

# ###########################################################
# # NDMP enable Method
# ###########################################################

=head2 ::enable

Enable NDMP on the source/destination filer.

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, enables NDMP on the destination filer

=item Examples

=back

     $ndmp->enable; Enables ndmp on the source filer
     $ndmp->enable(destination=>1); Enables ndmp on the destination filer.;


=cut

sub enable{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) =@_;
    my $ret;
    if(($args{destination})==1)
        {
         my $filer_name=$self->{_dst}->host;
         NOTE::logcomment("Enabling NDMP on filer $filer_name \n");
         eval{$ret = NdmpCmds::enableNdmp($self->{_dst}->conn)};
         croak $@ if($@);
        }
     else
        {
         my $filer_name=$self->{_dst}->host;
         NOTE::logcomment("Enabling NDMP on filer $filer_name \n");
         eval { $ret = NdmpCmds::enableNdmp($self->{_src}->conn)};
         croak $@ if ( $@ );
         }

     }


# ###########################################################
# # NDMP disable Method
# ###########################################################

=head2 ::disable

Disables NDMP on the source/destination filer.

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, disables NDMP on the destination filer



=item Examples

=back

     $ndmp->disable; Disables ndmp on the source filer
     $ndmp->disable(destination=>1); Disables ndmp on the destination filer.;

=cut

sub disable{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) =@_;
    my $ret;
    if(($args{destination})==1)
        {
        my $filer_name=$self->{_dst}->host;
        NOTE::logcomment("Disabling NDMP on filer $filer_name \n");
        eval{$ret = NdmpCmds::disableNdmp($self->{_dst}->conn)};
        }
    else
        {
        my $filer_name=$self->{_dst}->host;
        NOTE::logcomment("Disabling NDMP on filer $filer_name \n");
        eval { $ret = NdmpCmds::disableNdmp($self->{_src}->conn) };
        }
    croak $@ if ( $@ );
    }


# ###########################################################
# # NDMP enable offset  Method
# ###########################################################




=head2 ::enable_offset_map

Enable offset map on the source/destination filer.

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, enables offset map on the destination filer


=item Examples

=back

     $ndmp->enable_offset_map; Enables offset map on the source filer
     $ndmp->enable(destination=>1); Enables offset map on the destination filer.;


=cut

sub enable_offset_map{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) =@_;
    my $ret;
    if(($args{destination})==1)
        {
         my $filer_name=$self->{_dst}->host;
         NOTE::logcomment("Enabling offset map on filer $filer_name \n");
         eval{$ret = NdmpCmds::enableOffsetMap($self->{_dst}->conn)};
         croak $@ if($@);
        }
     else
        {
         my $filer_name=$self->{_dst}->host;
         NOTE::logcomment("Enabling offset map on filer $filer_name \n");
         eval { $ret = NdmpCmds::enableOffsetMap($self->{_src}->conn)};
         croak $@ if ( $@ );
         }

     }

############################################################
## NDMP disable offset  Method
############################################################




=head2 ::disable_offset_map

Disable offset map on the source/destination filer.

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, Disable offset map on the destination filer


=item Examples

=back

     $ndmp->disable_offset_map; Disables offset map on the source filer
     $ndmp->disable(destination=>1); Disables offset map on the destination filer.


=cut

sub disable_offset_map{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) =@_;
    my $ret;
    if(($args{destination})==1)
        {
         my $filer_name=$self->{_dst}->host;
         NOTE::logcomment("disabling offset map on filer $filer_name \n");
         eval{$ret = NdmpCmds::disableOffsetMap($self->{_dst}->conn)};
         croak $@ if($@);
        }
     else
        {
         my $filer_name=$self->{_dst}->host;
         NOTE::logcomment("disabling offset map on filer $filer_name \n");
         eval { $ret = NdmpCmds::disableOffsetMap($self->{_src}->conn)};
         croak $@ if ( $@ );
         }

    }

# ###########################################################
# # NDMP get_version Method
# ###########################################################




=head2 ::get_version

Gets the NDMP version on the source/destination filer.

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, gets NDMP version on the destination filer



=item Examples

=back

     $ndmp->get_version; Gets ndmp version on the source filer
     $ndmp->get_version(destination=>1); Gets ndmp version on the destination filer.


=cut



sub get_version {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret ;
    if(($args{destination})==1)
        {
        eval {$ret = NdmpCmds::getNdmpVersion($self->{_dst}->conn) };
        }
    else { 
        eval { $ret = NdmpCmds::getNdmpVersion($self->{_src}->conn)};
        }
    croak $@ if ( $@ );
    return $ret;

    }


# ###########################################################
# # NDMP set_version Method
# ###########################################################



=head2 ::set_version

Sets the NDMP version on the source/destination filer.

=over

=item Arguments

B<version> => I<Version>

I<Required>
NDMP version to be set on the source/destination filer.
 
B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, sets NDMP version on the destination filer



=item Examples

=back

     $ndmp->set_version(version=>$version); Sets ndmp version on the source filer
     $ndmp->set_version(destination=>1,version=>$version); Sets ndmp version on the destination filer.


=cut



sub set_version {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


    my ($self,%args) = @_;
    my $ret ;
    my $value = ($args{version});
    if(($args{destination})==1)
        {
        eval{ $ret = NdmpCmds::setNdmpVersion($self->{_dst}->conn ,$value) };
        }
    else
        { 
        eval { $ret = NdmpCmds::setNdmpVersion($self->{_src}->conn,$value) };
        }
    croak $@ if ( $@ );
    }

# ###########################################################
# # NDMP set_auth_type Method
# ###########################################################

=head2 ::set_auth_type

Sets the NDMP authorization type the source/destination filer.

=over

=item Arguments

B<authtype> => I<AuthType>

I<Required.>
Auth type value to be set on the filer. If setting multiple value, the values should be separated by a comma.

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, set AuthType on the destination filer



=item Examples

=back

     $ndmp->set_auth_type(authtype=>'plaintext,challenge'); Sets ndmp auth type on the source filer
     $ndmp->set_auth_type(destination=>1,authtype=>'plaintext,challenge'); Sets ndmp auth type on the destination filer.


=cut


sub set_auth_type
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my $ret ;
    my $value = ($args{authtype});
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::setNdmpAuthType($self->{_dst}->conn ,$value) };
        }
    else
        {
        eval { $ret = NdmpCmds::setNdmpAuthType($self->{_src}->conn,$value)};
        }
    croak $@ if ( $@ );

    }

# ###########################################################
# # NDMP get_auth_type Method
# ###########################################################




=head2 ::get_auth_type

Gets the NDMP authorization type the source/destination filer.

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, gets auth type on the destination filer



=item Examples

=back

     $ndmp->get_auth_type; Gets ndmp auth type on the source filer
     $ndmp->get_auth_type(destination=>1); Gets ndmp auth type on the destination filer.


=cut


sub get_auth_type {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret ;
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::getNdmpAuthType($self->{_dst}->conn) };
        }
    else
        {
        eval { $ret = NdmpCmds::getNdmpAuthType($self->{_src}->conn) };
	}
    croak $@ if ( $@ );
    return $ret;

   }

# ###########################################################
# # NDMP get_access Method
# ###########################################################




=head2 ::get_access

Gets the NDMP access for the source/destination filer.

=over

=item Arguments


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, gets NDMP access on the destination filer


=item Examples

=back

     $ndmp->get_access; Gets ndmp access on the source filer
     $ndmp->get_access(destination=>1); Gets ndmp access on the destination filer.


=cut

sub get_access
{
my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
my ($self,%args) = @_;
    my $ret ;
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::getNdmpAccess($self->{_dst}->conn) };
        }
    else
        {
        eval { $ret = NdmpCmds::getNdmpAccess($self->{_src}->conn) };
        }
    croak $@ if ( $@ );
    return $ret;

    }

# ###########################################################
# # NDMP set_access Method
# ###########################################################



=head2 ::set_access

Sets the NDMP access for the source/destination filer.

=over

=item Arguments

B<access> => I<NDMP access type to be set on the filer >
I<Required.>
Access value to be set on the filer. 


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, sets NDMP access on the destination filer


=item Examples

=back

     $ndmp->set_access(access=>$access); Sets ndmp access on the source filer
     $ndmp->set_access(destination=>1,access=>$access); Sets ndmp access on the destination filer.


=cut


sub set_access {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

my ($self,%args) = @_;
    my $ret ;
    my $value = $args{access};
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::setNdmpAccess($self->{_dst}->conn ,$value)};
        }
    else
        {
        eval { $ret = NdmpCmds::setNdmpAccess($self->{_src}->conn,$value) };
        }
    croak $@ if ( $@ );

    }

# ###########################################################
# # NDMP set_debug Method
# ###########################################################



=head2 ::set_debug

Sets the NDMP debug for the source/destination filer.

=over

=item Arguments

B<debug> => I<NDMP debug to be set on the filer >
I<Required.>
Debug value to be set on the filer. 


B<destination> => I<Source/Destination Filer> 

I<Optional.>
0|1. Defaults to source filer. If 1, sets NDMP debug on the destination filer


=item Examples

=back

     $ndmp->set_debug(debug=>$debug); Sets ndmp debug on the source filer
     $ndmp->set_debug(destination=>1,debug=>$debug); Sets ndmp debug on the destination filer.


=cut


sub set_debug
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my $ret ;
    my $value = ($args{debug});
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::setNdmpDebug($self->{_dst}->conn ,$value) };
        }
    else
        {
        eval { $ret = NdmpCmds::setNdmpDebug($self->{_src}->conn,$value) };
        }
    croak $@ if ( $@ );

}


# ###########################################################
# # NDMP get_preferred_interface Method
# ###########################################################


=head2 ::get_preferred_interface

Gets the NDMP preferred_interface for the source/destination filer.

=over

=item Arguments


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, gets NDMP preferred_interface on the destination filer


=item Examples

=back

     $ndmp->get_preferred_interface; Gets ndmp preferred_interface on the source filer
     $ndmp->get_preferred_interface(destination=>1); Gets ndmp preferred_interface on the destination filer.


=cut


sub get_preferred_interface {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics


    my ($self,%args) = @_;
    my $ret ;
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::getPreferredInterface($self->{_dst}->conn) };
        }
    else
        {
        eval { $ret = NdmpCmds::getPreferredInterface($self->{_src}->conn) };
        }
     croak $@ if ( $@ );
     return $ret;

}

# ###########################################################
# # NDMP set_preferred_interface Method
# ###########################################################


=head2 ::set_preferred_interface

Sets the NDMP preferred_interface for the source/destination filer.

=over

=item Arguments

B<interface> => I<NDMP preferred_interface to be set on the filer >
I<Required.>
Debug value to be set on the filer. 


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, sets NDMP preferred_interface on the destination filer


=item Examples

=back

     $ndmp->set_preferred_interface(interface=>interface); Sets ndmp preferred_interface on the source filer
     $ndmp->set_preferred_interface(destination=>1,interface=>$interface); Sets ndmp preferred_interface on the destination filer.


=cut

sub set_preferred_interface {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret ;
    my $value = $args{interface};
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::setPreferredInterface($self->{_dst}->conn ,$value) };
        }
    else
        {
        eval { $ret = NdmpCmds::setPreferredInterface($self->{_src}->conn,$value)};
        }
    croak $@ if ( $@ );

    }

############################################################
## NDMP snap_restore Method
############################################################


=head2 ::snap_restore

Enable/Disable snap restore for the source/destination filer.

=over

=item Arguments

B<enable> => I<enable value for ndmp snap restore>
I<Required.>
snap restore enable value to be set on the filer.


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, sets NDMP snaprestore value  on the destination filer


=item Examples

=back

     $ndmp->snap_restore(enable=>1|0); Sets ndmp snap restore on the source filer
     $ndmp->snap_restore(destination=>1,enable=>1); Sets ndmp snap restore on the destination filer


=cut

sub snap_restore{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret ;
    my $value = $args{enable};
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::ndmpSnapRestore($self->{_dst}->conn ,$value) };
        }
    else
        {
        eval { $ret = NdmpCmds::setPreferredInterface($self->{_src}->conn,$value)};
        }
    croak $@ if ( $@ );
    return $ret;

    }



# ###########################################################
# # NDMP kill_session Method
# ###########################################################



=head2 ::kill_session

Sets the NDMP sessionId for the source/destination filer.

=over

=item Arguments

B<sessionId> => I<NDMP sessionId to be killed on the filer >
I<Required.>
NDMP SessionID  to be killed on the filer. 


B<destination> => I<Source/Destination Filer> 

I<Optional.>
0|1. Defaults to source filer. If 1, kills NDMP sessionId on the destination filer


=item Examples

=back

     $ndmp->kill_session(sessionId=>$sessionId); Kills ndmp sessionId on the source filer
     $ndmp->kill_session(destination=>1,sessionId=>$sessionId); Kills ndmp sessionId on the destination filer.


=cut


sub kill_session {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

 
    my ($self,%args) = @_;
    my $ret ;
    my $value = $args{sessionId};
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::killNdmp($self->{_dst}->conn ,$value) };
        }
    else
        {
        eval { $ret = NdmpCmds::killNdmp($self->{_src}->conn,$value) };
        }
        croak $@ if ( $@ );

    }


# ###########################################################
# # NDMP is_enable Method
# ###########################################################


=head2 :: is_enable

Checks if NDMP is enabled on the source/destination filer.Returns 1 if it is enabled , 0 if it is not.

=over

=item Arguments


B<destination> => I<Source/Destination Filer> 

I<Optional.>
0|1. Defaults to source filer. If 1, gets NDMP status on the destination filer


=item Examples

=back

     $ndmp->is_enable; Gets ndmp status on the source filer
     $ndmp->is_enable(destination=>1); Gets ndmp status on the destination filer.


=cut

sub is_enable {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret ;
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::IsNdmp($self->{_dst}->conn ) };
        }
    else
        {
        eval { $ret = NdmpCmds::IsNdmp($self->{_src}->conn) };
        }
    croak $@ if ( $@ );
    return $ret;
    }


# ###########################################################
# # is_offset_map_enabled Method
# ###########################################################


=head2 :: is_offset_map_enabled

Checks if offset map is enabled on the source/destination filer.Returns 1 if it is enabled , 0 if it is not.

=over

=item Arguments


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, gets NDMP status on the destination filer


=item Examples

=back

     $ndmp->is_offset_map_enabled; Gets ndmp status on the source filer
     $ndmp->is_offset_map_enabled(destination=>1); Gets ndmp status on the destination filer.


=cut

sub is_offset_map_enabled{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret ;
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::isOffsetMapEnabled($self->{_dst}->conn ) };
        }
    else
        {
        eval { $ret = NdmpCmds::isOffsetMapEnabled($self->{_src}->conn) };
        }
    croak $@ if ( $@ );
    return $ret;
    }

# ###########################################################
# # NDMP get_password Method
# ###########################################################




=head2 ::get_password

Gets the NDMP password for the source/destination filer.

=over

=item Arguments


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, gets NDMP password on the destination filer


=item Examples

=back

     $ndmp->get_password; Gets ndmp password on the source filer
     $ndmp->get_password(destination=>1); Gets ndmp password on the destination filer.


=cut


sub get_password {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret ;
    my $value = $args{userId};
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::ndmpPasswd($self->{_dst}->conn ,$value) };
        }
    else
        {
        eval { $ret = NdmpCmds::ndmpPasswd($self->{_src}->conn ,$value) };
        }
    croak $@ if ( $@ );
    return $ret;

    }

# ###########################################################
# # NDMP get_log_date Method
# ###########################################################



=head2 ::get_log_date

Gets the NDMP log_date for the source/destination filer.

=over

=item Arguments
B<timeout> => I<Timeout value defaults to 1 >
I<Optional.>
Timeout value to be set on the filer. 


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, gets NDMP log_date on the destination filer


=item Examples

=back

     $ndmp->get_log_date; Gets ndmp log_date on the source filer
     $ndmp->get_log_date(destination=>1); Gets ndmp log_date on the destination filer.


=cut


sub get_log_date
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my $ret ;
    my $value=defined $args{timeout} ? $args{timeout}:'-1';
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::getLogDate($self->{_dst}->conn ,$value) };
        }
    else
        {
        eval { $ret = NdmpCmds::getLogDate($self->{_src}->conn ,$value) };
        }
    croak $@ if ( $@ );
    return $ret;

    }

############################################################
## NDMP get_interface_with_IP Method
############################################################




=head2 ::get_interfaces_with_IP

Returns an array of all the interfaces with IP.

=over

=item Arguments


B<destination> => I<Source/Destination Filer>

I<Optional.>
0|1. Defaults to source filer. If 1, gets interfaces with IP on the destination filer


=item Examples

=back

     $ndmp->get_interfaces_with_IP; Gets interfaces with IP on the source filer
     $ndmp->get_interfaces_with_IP(destination=>1); Gets interfaces with IP on the destination filer.


=cut

sub get_interfaces_with_IP {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my @ret ;
    if(($args{destination})==1)
        {
        eval { @ret = NdmpCmds::getInterfacesWithIP($self->{_dst}->conn ) };
        }
    else
        { 
        eval { @ret = NdmpCmds::getInterfacesWithIP($self->{_src}->conn ) };
        }
    croak $@ if ( $@ );
    return @ret;

    }
############################################################
## NDMP get_all_interfaces Method
############################################################



=head2 ::get_all_interfaces

Returns an array of all the interfaces.

=over

=item Arguments


B<destination> => I<Source/Destination Filer> 

I<Optional.>
0|1. Defaults to source filer. If 1, gets all interfaces on the destination filer


=item Examples

=back

     $ndmp->get_all_interfaces; Gets all interfaces on the source filer
     $ndmp->get_all_interfaces(destination=>1); Gets all interfaces on the destination filer.


=cut


sub get_all_interfaces
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my @ret ;
    if(($args{destination})==1)
        {
        eval { @ret = NdmpCmds::getAllInterfaces($self->{_dst}->conn ) };
        }
    else
        {
        eval { @ret = NdmpCmds::getAllInterfaces($self->{_src}->conn ) };
        }
    croak $@ if ( $@ );
    return @ret;

    }

############################################################
## NDMP get_copy Method
############################################################




=head2 ::copy

Does a NDMP copy where the src volume is on the srcFiler passed in ndmp::new and dst volume is on dstFiler passed in the ndmp::new. 
If no value is passed for dstFiler then it defaults to srcFiler. If src volume and dst volume are on same filer then localNdmpCopy
is called otherwise remoteNdmpCopy is called

=over

=item Arguments

B<src_vol> => I<NOTE volume object for the source volume>
I<Required.>

B<dst_vol> => I<NOTE Volume object for the destination volume>
I<Required.>

B<src_password> => I<Password for the source filer>
I<Optional.>
Defaults to source filer password name from tharnhost. 

B<dst_username> => I<Username for the dst filer>
I<Optional.>
Defaults to dst filer user name from tharnhost. 

B<dst_password> => I<Password for the dst filer>
I<Optional.>
Defaults to dst filer password name from tharnhost. 

B<snapshot> => I<snap name on the source volume>
I<Optional.>
If given then src path given for ndmp copy will be /vol/<volname>/.snapshot/<snapshot>

B<timeout> => I<Timeout value>
I<Optional.>
Defaults to -1.


=item Examples

=back

     $ndmp->copy(src_vol=$Volume1,dst_vol=$Volume2);Performs a NDMP copy from source volume $Volume1 to destination volume $Volume2
     


=cut



sub copy
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my $ret ;
    my $srcUser=defined $args{src_username} ? $args{src_username}:$self->{_src}->conn->{username};
    my $srcPassword=defined $args{src_password} ? $args{src_password}:$self->{_src}->conn->{password};
    my $dstUser=defined $args{dst_username} ? $args{dst_username}:$self->{_src}->conn->{username};
    my $dstPassword=defined $args{dst_password} ? $args{dst_password}:$self->{_dst}->conn->{password};
    my $timeout=defined $args{timeout} ? $args{timeout}:'-1';
    my $isIncr=defined $args{isIncr} ? $args{isIncr}:'0';
    my $vol_path;
    my $srcVol=$args{src_vol} ;
    my $dstVol=$args{dst_vol} ;
    my $srcFiler=$srcVol->Filer;
    my $dstFiler=$dstVol->Filer;
    if($args{snapshot})
    {
        die "Snapshot provided does not exist, pleae provide the correct snapshot " unless($srcVol->snapshot_exists($args{snapshot}));
        $vol_path=$srcVol->mount_path."\/\.snapshot\/".$args{snapshot};
    }
    else { $vol_path=$srcVol->mount_path; }
    

    if (($srcFiler->host) eq ($dstFiler->host))
        {
         my $srcFilerHost=$srcFiler->host;
         NOTE::logcomment( "Calling local NDMP Copy, both src and dst are on $srcFilerHost");
         $ret = NdmpCmds::localNdmpCopy($srcFiler->conn,$vol_path,$dstVol->mount_path,$timeout ,1,$isIncr);
        }
    else {
         my $srcFilerHost=$srcFiler->host;
         my $dstFilerHost=$dstFiler->host;
         NOTE::logcomment( "Calling remote  NDMP Copy, src is on $srcFilerHost and dst is on $dstFilerHost");
        $ret = NdmpCmds::remoteNdmpCopy($srcFiler->conn,$srcFiler->host,$dstFiler->host,$srcUser,$dstUser, $srcPassword,$dstPassword,$vol_path,$dstVol->mount_path,$timeout ,1,$isIncr);
        }
    }

############################################################
## NDMP backup Method
############################################################



=head2 ::backup



Does a NDMP backup.For local backup tape and the source volume should be on the same filer .Source volume object is required
For 3wayBack up, tapefiler(NOTE object of the filer on which tape resides) is required .
It returns a hash, which has one of the element 'messages' that captures all the logs doing a backup operationis called 

=over

=item Arguments

B<src> =>I<NOTE Volume object of the volume to be backed up>
I<Required.>

B<data_filer> => I<Filer name or IP on which the volume to be backed up is resides, required only for negative testing, when the volume object passed does not reside on the data_filer, Also used in case of vfiler>
I<Optional.>

B<tapefiler> => I<NOTE Filer object for the tape resides,defaults to the filer on which volume to be backed up resides.In that cases local backup is done.Is a mandatory parameter in case 3 way backup is to be done>
I<Required.>

B<src_username> => I<Username for the src filer>
I<Optional.>
Defaults to src filer user name from tharnhost.Optional if LocalBackup is done 

B<src_password> => I<Password for the source filer>
I<Optional.>
Defaults to source filer password name from tharnhost.Optional if LocalBackup is done


B<tape_username> => I<Username for the tape filer>
I<Optional.>
Defaults to tape filer user name from tharnhost.Optional if 3wayBackup is done 

B<tape_password> => I<Password for the tape filer>
I<Optional.>
Defaults to tape filer password name from tharnhost.Optional if 3wayBackup is done

B<version> => I<Ndmp version used for doing tape backup>
I<Optional.>
Default NDMP version is 4. Otherwise provide the NDMP version 

B<mover_record> => I<mover record size>
I<Optional>
Defaults to 64512 ,mover required size

B<mover_window> => I<mover window size>
I<Optional.>
Defaults to -1,mover window size

B<tape> => I<Tape name>
I<Required.>
Name of the tape where the backup is to be done


B<isIncr> => I<Is increment value>
I<Optional.>
Defaults to 0.Is it an incremental backup.


B<snapshot> => I<snap name on the source volume>
I<Optional.>
If given then src path given for ndmp backup will be /vol/<volname>/.snapshot/<snapshot>

B<env> => I<Hash of the environment value>
I<Optional.>
Hash of the environment values. Includes the FILESYSTEM path of the volume to be backedup and other environment values.

=item Examples

=back
   For 3way backup
 
    my %hash1=$ndmp1->backup(src=>$srcVol,tapefiler=>$filer_a,tape=>'nrst0a');

For Local Backup
    my %hash1=$ndmp1->backup(src=>$srcVol,tape=>'nrst0a');


    my $mover_halt_msg=$hash1{messages}{mover_halt_reason};
    my $mover_paused_reason=$hash1{messages}{mover_paused_reason};
    my $log_message=$hash1{messages}{log_message};
    my $data_halt_reason=$hash1{messages}{data_halt_reason};


=cut

 
sub backup
{

    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my %ARGS;
    my ($self,%args) = @_;
    my $srcVol =$args{src};
    delete $args{src};
    my $srcFiler=$srcVol->Filer;
    my $srcFilerHost=$srcFiler->host;
    my $data_filer=defined $args{data_filer} ? $args{data_filer}:$srcFilerHost;
    if($args{data_filer}) {delete $args{data_filer}};
    my $tapefiler=defined $args{tapefiler}? $args{tapefiler}:$srcFiler;
    if($args{tapefiler}) {delete $args{tapefiler}};
    my $user=defined $args{src_username} ? $args{src_username}:$srcFiler->conn->{username};
    if($args{src_username}) {delete $args{src_username}};
    my $password=defined $args{src_password} ? $args{src_password}:$srcFiler->conn->{password};
    if($args{src_password}) {delete $args{src_password}};
    my $protocol=$args{version}? $args{version}:'4';
    if($args{version}) {delete $args{version}};
    my $mover_record=$args{mover_record}?$args{mover_record}:'64512' ;
    if($args{mover_record}) {delete $args{mover_record}};
    my $mover_window=$args{mover_window}?$args{mover_window}:'-1';
    if($args{mover_window}) {delete $args{mover_window}};
    my $tape=$args{tape};
    if($args{tape}) {delete $args{tape}};
    my $isIncr=$args{isIncr}?$args{isIncr}:'0';
    if($args{isIncr}) {delete $args{isIncr}};
    my $want_log=$args{want_log} ?$args{want_log} :'1';
    if($args{want_log}) {delete $args{want_log}};
    my $EXCLUDE=$args{EXCLUDE} ?$args{EXCLUDE} :'1';
    my $env=$args{env};
    $ARGS{$_} = $env->{$_} for ( keys %$env );
    my $tapefiler_host=$tapefiler->host;
    my $vol_mount_path;
    if($args{snapshot})
    {
        die "Snapshot provided does not exist, pleae provide the correct snapshot " unless($srcVol->snapshot_exists($args{snapshot}));

        $vol_mount_path=$srcVol->mount_path."\/\.snapshot\/".$args{snapshot};
    }
    else {
        $vol_mount_path=$srcVol->mount_path;
    }
    if($data_filer=~/$tapefiler_host/)
        {
        NOTE::logcomment( "Calling local backup, both tape and restore path is on $data_filer");
        my %ret =NdmpCmds::ndmpLocalBackup($data_filer,$user,$password,$protocol,$mover_record,$mover_window,$tape,$isIncr,FILESYSTEM=>$vol_mount_path,WANT_LOGS=>$want_log,%ARGS);
         return \%ret ;
        }
    else
        {
        NOTE::logcomment( "Calling 3Way backup , tape is on $tapefiler_host  and restore path is on $data_filer");
        my $tape_user=defined $args{tape_username} ? $args{tape_username}:$tapefiler->conn->{username};
        my $tape_password=defined $args{tape_password} ? $args{tape_password}:$tapefiler->conn->{password};

         my %ret =NdmpCmds::ndmp3WayBackup($data_filer,$tapefiler_host,$user,$tape_user,$password,$tape_password,$protocol,$mover_record,$mover_window,$tape,$isIncr,FILESYSTEM=>$vol_mount_path,WANT_LOGS=>$want_log,%ARGS);
         return \%ret ;
        }
    }

############################################################
## NDMP restore Method
############################################################



=head2 ::restore



Does a NDMP restore.For local restore tape and the destination volume should be on the same filer .destination volume object is required
For 3wayBack up, tapefiler(NOTE object of the filer on which tape resides) is required .
It returns a hash, which has one of the element 'messages' that captures all the logs doing a restore operation is called

=over

=item Arguments

B<dst> =>I<NOTE Volume object of the volume where data will be restored>
I<Required.>

B<data_filer> => I<Filer name or IP on which the volume  where data will be restored resides, required only for negative testing, when the volume object passed does not reside on the data_filer, also used in case of Vfiler>
I<Optional.>

B<tapefiler> => I<NOTE Filer object for the tape resides,defaults to the filer on which restored volume resides in that case local restore is done.Is a mandatory parameter in case 3 way restore is to be done>
I<Optional.>


B<dst_username> => I<Username for the src filer>
I<Optional.>
Defaults to src filer user name from tharnhost.Optional if LocalRestore is done 

B<dst_password> => I<Password for the source filer>
I<Optional.>
Defaults to source filer password name from tharnhost.Optional if Localrestore is done

B<tape_username> => I<Username for the tape filer>
I<Optional.>
Defaults to tape filer user name from tharnhost.Optional if 3wayRestore is done 

B<tape_password> => I<Password for the tape filer>
I<Optional.>
Defaults to tape filer password name from tharnhost.Optional if 3wayRestore is done

B<version> => I<Ndmp version used for doing tape restore>
I<Optional.>
Default is 4. Otherwise provide the NDMP version 

B<mover_record> => I<mover record size>
I<Optional.>
Defaults to 64512 ,mover record size

B<mover_window> => I<mover window size>
I<Optional.>
Defaults to -1,mover window size

B<tape> => I<Tape name>
I<Required.>
Name of the tape where the restore is to be done

B<isIncr> => I<Is increment value>
I<Optional.>
Defaults to 0.Is it an incremental backup.


B<env> => I<Hash of the environment value>
I<Optional.>
Hash of the environment values. 

=item Examples

=back
    For 3 way restore
     my %hash=$ndmp1->restore(dst=>$dstVol,tapefiler=>$filer_a,tape=>'nrst0a') ;
   
    For local restore
     my %hash=$ndmp1->restore(dst=>$dstVol,tape=>'nrst0a') ;
    
    my $mover_halt_msg=$hash1{messages}{mover_halt_reason};
    my $mover_paused_reason=$hash1{messages}{mover_paused_reason};
    my $log_message=$hash1{messages}{log_message};
    my $data_halt_reason=$hash1{messages}{data_halt_reason};

=cut

sub restore
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my %ARGS;
    my ($self,%args) = @_;
    my $dstVol =$args{dst};
    my $dstFiler=$dstVol->Filer;
    my $dstFilerHost=$dstFiler->host;
    my $dst_filer=defined $args{data_filer} ? $args{data_filer}:$dstFilerHost;
    my $tapefiler=defined $args{tapefiler}? $args{tapefiler}:$dstFiler;
    my $user=defined $args{dst_username} ? $args{dst_username}:$dstFiler->conn->{username};
    my $password=defined $args{dst_password} ? $args{dst_password}:$dstFiler->conn->{password};
    my $protocol=$args{version}? $args{version}:'4';
    my $mover_record=$args{mover_record}?$args{mover_record}:'64512' ;
    my $mover_window=$args{mover_window}?$args{mover_window}:'-1';
    my $tape=$args{tape};
    my $restoreTo=$dstVol->mount_path;
    my $isIncr=$args{isIncr}?$args{isIncr}:'0';
    my $want_log=$args{want_log} ?$args{want_log} :'1';
    my $env=$args{env};
    $ARGS{$_} = $env->{$_} for ( keys %$env );
    my $tapefiler_host=$tapefiler->host;
    if($dst_filer=~/$tapefiler_host/)
        {
        NOTE::logcomment( "Calling local restore, both tape and restore path is on $dst_filer");
        my %ret =NdmpCmds::ndmpLocalRestore($dst_filer,$user,$password,$protocol,$mover_record,$mover_window,$tape,$restoreTo,$isIncr,WANT_LOGS=>$want_log,%ARGS);
        return %ret ;
         }
    else
        {
        NOTE::logcomment( "Calling 3Way restore, tape is on $tapefiler_host  and restore path is on $dst_filer");
        my $tape_user=defined $args{tape_username} ? $args{tape_username}:$tapefiler->conn->{username};
        my $tape_password=defined $args{tape_password} ? $args{tape_password}:$tapefiler->conn->{password};
        my %ret =NdmpCmds::ndmp3WayRestore($dst_filer,$tapefiler_host,$user,$tape_user,$password,$tape_password,$protocol,$mover_record,$mover_window,$tape,$restoreTo,$isIncr,WANT_LOGS=>$want_log,%ARGS);
        return %ret ;

        }

    }


#######################################
####NDMP Restartable Backup Method#####
#######################################

=head2 ::restartable_backup



Does a NDMP restartable backup.For local backup tape and the source volume should be on the same filer .Source volume object is required

=over

=item Arguments

B<src> =>I<NOTE Volume object of the volume to be backed up>
I<Required.>

B<data_filer> => I<Filer name or IP on which the volume to be backed up is resides, required only for negative testing, when the volume object passed does not reside on the data_filer, Also used in case of vfiler>
I<Optional.>

B<tapefiler> => I<NOTE Filer object for the tape resides,defaults to the filer on which volume to be backed up resides.In that cases local backup is done.Is a mandatory parameter in case 3 way backup is to be done>
I<Required.>

B<src_username> => I<Username for the src filer>
I<Optional.>
Defaults to src filer user name from tharnhost.Optional if LocalBackup is done

B<src_password> => I<Password for the source filer>
I<Optional.>
Defaults to source filer password name from tharnhost.Optional if LocalBackup is done


B<tape_username> => I<Username for the tape filer>
I<Optional.>
Defaults to tape filer user name from tharnhost.Optional if 3wayBackup is done

B<tape_password> => I<Password for the tape filer>
I<Optional.>
Defaults to tape filer password name from tharnhost.Optional if 3wayBackup is done

B<version> => I<Ndmp version used for doing tape backup>
I<Optional.>
Default NDMP version is 4. Otherwise provide the NDMP version

B<mover_record> => I<mover record size>
I<Optional>
Defaults to 64512 ,mover required size

B<mover_window> => I<mover window size>
I<Optional.>
Defaults to -1,mover window size

B<tape> => I<Tape name>
I<Required.>
Name of the tape where the backup is to be done


B<snapshot> => I<snap name on the source volume>
I<Optional.>
If given then src path given for ndmp backup will be /vol/<volname>/.snapshot/<snapshot>


=item Examples

=back
   For 3way restartable backup

    my %hash1=$ndmp1->backup(src=>$srcVol,tapefiler=>$filer_a,tape=>'nrst0a');

For Local restartableBackup
    my %hash1=$ndmp1->restartable_backup(src=>$srcVol,tape=>'nrst0a');




=cut


sub restartable_backup
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my %ARGS;
    my ($self,%args) = @_;
    my $srcVol =$args{src};
    my $srcFiler=$srcVol->Filer;
    my $srcFilerHost=$srcFiler->host;
    my $data_filer=defined $args{data_filer} ? $args{data_filer}:$srcFilerHost;
    my $tapefiler=defined $args{tapefiler}? $args{tapefiler}:$srcFiler;
    my $user=defined $args{src_username} ? $args{src_username}:$srcFiler->conn->{username};
    my $password=defined $args{src_password} ? $args{src_password}:$srcFiler->conn->{password};
    my $protocol=$args{version}? $args{version}:'4';
    my $mover_record=$args{mover_record}?$args{mover_record}:'64512' ;
    my $mover_window=$args{mover_window}?$args{mover_window}:'-1';
    my $tape=$args{tape};
    $ARGS{$_} = $args{$_} for ( keys %args );
    my $tapefiler_host=$tapefiler->host;
    my $vol_mount_path;
    if($args{snapshot})
    {
        die "Snapshot provided does not exist, pleae provide the correct snapshot " unless($srcVol->snapshot_exists($args{snapshot}));


	 $vol_mount_path=$srcVol->mount_path."\/\.snapshot\/".$args{snapshot};
    }
    else {
        $vol_mount_path=$srcVol->mount_path;
    }
    if($data_filer=~/$tapefiler_host/)
        {

        NOTE::logcomment( "Calling local backup, both tape and restore path is on $data_filer");
        my $restart_point_low=NdmpCmds::ndmpRestartBackup($data_filer,$user,$password,$protocol,$mover_record,$mover_window,$tape, $vol_mount_path);
        $self->{restart_point}=$restart_point_low;
        }
    else
        {
        NOTE::logcomment( "Calling 3Way backup , tape is on $tapefiler_host  and restore path is on $data_filer");
        NOTE::logcomment( "THIS FUNCTIONALITY IS NOT YET IMPLEMENTED");
=head
#This part of the code will be added when 3way restartable backup code is added in NdmpCmds.pm

        my $tape_user=defined $args{tape_username} ? $args{tape_username}:$tapefiler->conn->{username};
        my $tape_password=defined $args{tape_password} ? $args{tape_password}:$tapefiler->conn->{password};

         my %ret =NdmpCmds::ndmp3WayRestartBackup($data_filer,$tapefiler_host,$user,$tape_user,$password,$tape_password,$protocol,$mover_record,$mover_window,$tape,$vol_mount_path);
         return \%ret ;
=cut
        }
    }
#######################################
####NDMP Restartable Restore Method#####
#######################################

=head2 ::restartable_restore



Does a NDMP restore for a restartable backup.

=over

=item Arguments

B<dst> =>I<NOTE Volume object of the volume where data will be restored>
I<Required.>


B<tapefiler> => I<NOTE Filer object for the tape resides,defaults to the filer on which volume to be backed up resides.
I<Required.>

B<username> => I<Username for the tape filer>
I<Optional.>
Defaults to tape filer user name from tharnhost.

B<password> => I<Password for the tapefiler>
I<Optional.>
Defaults to tape filer password name from tharnhost.Optional if LocalBackup is done

B<mover_window> => I<mover window size>
I<Optional.>
Defaults to -1,mover window size

B<tape> => I<Tape name>
I<Required.>
Name of the tape where the backup is to be done


B<mover_record> => I<mover record size>
I<Optional.>
Defaults to 64512

=item Examples

=back
   For 3way restartable backup

    my %hash1=$ndmp1->backup(src=>$srcVol,tapefiler=>$filer_a,tape=>'nrst0a');

For Local restartableBackup
    my %hash1=$ndmp1->restartable_backup(src=>$srcVol,tape=>'nrst0a');




=cut


sub restartable_restore
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my %ARGS;
    my ($self,%args) = @_;
    my $dstVol =$args{dst};
    my $tapefiler=$args{tapefiler};
    my $username=defined $args{username} ? $args{username}:$tapefiler->conn->{username};
    my $password=defined $args{password} ? $args{password}:$tapefiler->conn->{password};
    my $protocol=$args{version}? $args{version}:'4';
    my $mover_record=$args{mover_record}?$args{mover_record}:'64512' ;
    my $mover_window=$args{mover_window}?$args{mover_window}:'-1';
    my $tape=$args{tape};
    my $restartinfo=$self->{restart_point};
    croak "\n Restart info is not present, cannot do a restartable restore. \n" unless($restartinfo);
    $ARGS{$_} = $args{$_} for ( keys %args );
    my $tapefiler_host=$tapefiler->host;
    my $vol_mount_path;
    $vol_mount_path=$dstVol->mount_path;
        NdmpCmds::runRestartRestore($tapefiler_host,$tape,$vol_mount_path,$restartinfo,$username ,$password ,$mover_record,$mover_window);
    }

# ###########################################################
# # NDMP enable_firewall Method
# ###########################################################




=head2 ::enable_firewall

Enables firewall on the given set of ports  on the source/destination filer.

=over

=item Arguments

B<l_limit>  =>I<Staring value of the firwall ports to be enabled / all>
I<Required.>
#Number|all . Starting range of the firewall port number, or all , if All the ports should be enabled

B<u_limit>  =>I<End value of the firwall ports to be enabled >
I<Optional.>
#Number|all . Ending range of the firewall port number,not required if l_limit is set as all 

B<destination> => I<Source/DestinationFiler>

I<Optional.>
0|1. Defaults to source filer. If 1, enables firewall on the destination filer


=item Examples

=back

     $ndmp->enable_firewall(l_limit=>1000 ,u_limit=>1040); Enables firewall on ports from 1000 to 1040
     $ndmp->enable_firewall(l_limit=>all, destination=>1); Enables firewall on all the ports on the destination filer.


=cut

sub enable_firewall
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my %ARGS;
    my ($self,%args) = @_;
    my $ret ;
    my $l_limit= $args{l_limit};
    my $u_limit= $args{u_limit};
    if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::enableFirewall($self->{_dst}->conn ,$l_limit,$u_limit) };
        }
    else
        {
        eval { $ret = NdmpCmds::enableFirewall($self->{_src}->conn,$l_limit,$u_limit) };
        }
        croak $@ if ( $@ );

   }

=head2 ::usedDataPort

Returns the data port used in transfer

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>
I<Optional.>
0|1. Defaults to source filer. If 1, gets the port from the destination filer


=item Examples

=back

       $ret=$ndmp1->usedDataPort;
       $ret=$ndmp1->usedDataPort(destination=>1);

=cut
sub usedDataPort
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my %ARGS;
    my ($self,%args) = @_;
    my $ret ;
     if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::usedPort($self->{_dst}->conn)};
        }
    else
       {
       eval { $ret = NdmpCmds::usedPort($self->{_src}->conn)};
       }
   if($ret->{'data'})
       {
       return $ret->{'data'};
       }
   else { croak "DataPort not found :$@" ;}

} 

=head2 ::usedMoverPort

Returns the mover port used in transfer

=over

=item Arguments

B<destination> => I<Source/DestinationFiler>
I<Optional.>
0|1. Defaults to source filer. If 1, gets the port from the destination filer


=item Examples

=back

       $ret=$ndmp1->usedMoverPort;
       $ret=$ndmp1->usedMoverPort(destination=>1);

=cut

sub usedMoverPort
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my %ARGS;
    my ($self,%args) = @_;
    my $ret ;
     if(($args{destination})==1)
        {
        eval { $ret = NdmpCmds::usedPort($self->{_dst}->conn)};
        }
    else
       {
       eval { $ret = NdmpCmds::usedPort($self->{_src}->conn) };
       }
   if($ret->{'mover'})
       {
       return $ret->{'mover'};
       }
   else { croak "MoverPort not found :$@" ;}

}

=head2 ::tape_rewind

Rewinds the tape provided on the given filer

=over

=item Arguments

B<tape> => I<Name of the tape>
I<Required.>
Name of the tape which has to be rewind.

B<tapefiler> =>I<FilerObject>
I<Optional.>
Filer object where tape is located. Defaults to source in the ndmp object.

=item Examples

=back

       $ret=$ndmp1->tape_rewind(tape=>$tape,tapefiler=>$tapefiler);

=cut
  
sub tape_rewind
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my %ARGS;
    my ($self,%args) = @_;
    my $ret ;
    my $tape=$args{tape};
    croak "Tape name not passed while calling tapeRewind " unless($tape);
    my $tape_filer=defined $args{tapefiler} ? $args{tapefiler}:$self->{_src};
    my $filer_host=$tape_filer->host;
    croak "Tape name Contains 'vol' " if ($tape=~ /^\/vol/) ;
    NOTE::logcomment("Rewinding Tape $tape on filer $filer_host ");
    $ret=$tape_filer->say("mt -f $tape rewind");
    if($ret)
    {
        croak "tape Device does not exist on the filer " if($ret=~/is not a valid device name/);
        croak "Tape Rewind failed with : $ret \n";
    }
}


=head2 ::load_data

Loads the given volume with the compat data provided in the path

=over

=item Arguments

B<dst> => I<Volume Object>
I<Required.>
NOTE Volume Object where data is to be loaded


B<compat_path> =>I<Path>
I<Optional.>
Path where compat_data is located.Defaults to "/x/eng/http/html/engineering/qa/fsr/dump_images/compat_data.dmp.gz"

=item Examples

=back

       $ret=$ndmp1->load_data(dst=>$Volume1,path=>"/x/eng/http/html/engineering/qa/fsr/dump_images/compat_data.dmp.gz");


=cut

sub load_data
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my %ARGS;
    my ($self,%args) = @_;
    my $vol=$args{dst};
    croak "Volume where compat data is to be placed is not passed" unless($vol);
    my $path=$args{'compat_path'};
    my $vol_name=$vol->name;
    my $filer=$vol->Filer->host;
    NOTE::logcomment("Loading volume $vol_name on filer $filer with data on $path");
    my $vol_path=$vol->mount_path;
    eval{load_compat_data($filer,$vol_path,$path)};
    croak "load data failed with error :$@" if($@);
    
}

############################################################
## NDMP probe Method
############################################################

=head2 ::probe

It returns A reference to a hash containing the details of the 'ndmpd probe' command in the scalar context
else croaks with error.

OR 

It returns A reference to a hash containing the details of the 'ndmpd probe' along with reference to array contating session id's in the list context else croaks with error.

=over

=item Arguments

B<destination> => I<0|1>

I<Optional.>
0|1. Defaults to source filer. If set to 1, runs 'ndmpd probe' on the destination filer

=item Examples

=back

     $ndmp->probe; 
     $ndmp->probe(destination=>1); 


=cut

sub probe
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my ($host,$line, @lines,@data,$probe,%probe_info,$session_id,$no_of_sessions,@sessions);
	
    if(($args{destination})==1){
        $host = $self->{_dst}->host;
    }else{
	$host = $self->{_src}->host;
    }
	
    # Call 'ndmpd probe' on the filer
    my $filer_src = NOTE::Filer->connect( host => $host);
    $probe = $filer_src->say("ndmpd probe");

    # Error if first line is not 'ndmpd ON|OFF'
    if ( $probe !~ /^ndmpd (ON|OFF)/ ) {
        croak "ndmpd probe returned unexpected results: $probe";
    }

    # Error if no ndmpd sessions are active
    if ( $probe =~ /No ndmpd sessions active\./ ) {
        croak "No NDMP sessions currently active: $probe";
    }
 
    # Clean up the output
    $probe =~ s/\n\s+/\n/g;  # Remove leading spaces
    $probe =~ s/protocol version/protocol_version/g;  # Join 'protocol version'
    $probe =~ s/client addr/client_addr/g;  # Join 'client addr'
    $probe =~ s/:/ /g;  # Replace : with a space
    $probe =~ s/ndmpd ON.\n//g;  # Remove 'ndmpd ON.' from output
    $probe =~ s/(mover\.connect\.addr_type:)/$1  /g;
    
    # Split into lines, then into pairs and build a hash of hashes
    @lines = split("\n", $probe );

    # NDMP probe output give 32 key=>value pairs for each session
    $no_of_sessions = scalar(@lines) / 32 ;
    logcomment("Number of NDMP Sessions active : $no_of_sessions");

    foreach $line ( @lines ) {
        my ($key,$value) = split(/\s+/, $line, 2);
        if($key =~ /Session/){
	    $session_id = $value ;
            push @sessions,$session_id ;
        }else{
            $value =~ s/^\s+//;
	    $probe_info{$session_id}{$key} = $value ;
        }    
    }
      
    # Put the : back into the client_addr (removed above)
    foreach (@sessions){
        $probe_info{$_}{client_addr} =~ s/\s+/:/;
    }

    return (wantarray()?(\%probe_info,\@sessions):\%probe_info);
}

############################################################
## NDMP Parse Connect Log Method
############################################################

=head2 ::connect_log_parse

It returns a reference to a hash containing the details of the NDMP connection else croaks with error.

=over

=item Arguments

B<connect_logs> => I<connect log string>

I<Required.>
LogReader::Messages module can be used to grab the relevant section of the messages in the connect log
string.

B<sessionId> => I<NDMP Session ID>

I<Required.>
NOTE::NDMP::Probe method can be used to the get the ID of the NDMP session.

=item Examples

=back

     $ndmp->connect_log_parse(connect_logs=>$connect_log_string, sessionId=>$session_id);

=cut

sub connect_log_parse
{
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
    my ($self,%args) = @_;
    my (%connect_log_hash, $connect_log_pattern, $connect_log_string, $session_id);

    ($connect_log_string, $session_id) = ($args{connect_logs}, $args{sessionId});

    # Error if undefined connect log string passed
    if( !defined($connect_log_string) ){
        croak "Undefined connect log string passed";
    }

    # Error if undefined Session Id is passed
    if( !defined($session_id) ){
        croak "Undefined connect log string passed";
    }

    # Error if Session ID in the connect log string doesn't match with Session ID passed
    if( !($connect_log_string =~ /sessionId\s+=\s+$session_id/i) ) {
        croak "Session ID in the connect log string doesn't match with Session ID passed: \nconnect Log String: $connect_log_string\nSession Id: $session_id";
    }

    # Matching the connect log string format till BR
    if ($connect_log_string =~ /\[Java_Thread:info\]/) {

        $connect_log_pattern = qr/[Nn]dmp.+?:\s+?(ndmpd\.?.+?) for version\s+?=\s+?(\d),\s+?sessionId\s+?=\s+?($session_id),\s+?from src ip\s+?=\s+?(\d+?\.\d+?\.\d+?\.\d+?), dst ip\s+?=\s+?(.+?)\/(\d+?\.\d+?\.\d+?\.\d+?), src port\s+?=\s+?(\d+?), dst port\s+?=\s+?(\d+)/i;

        $connect_log_string =~ /$connect_log_pattern/;

        %connect_log_hash = (
            'destination_filer' => $5,
            'connection_status' => $1,
            'ndmp_version'=> $2,
            'session_id'=> $3,
            'source_ip'=> $4,
            'destination_ip'=> $6,
            'source_port'=> $7,
            'destination_port'=> $8,
        );

    }

    # Matching the connect log string with connect log format in RR.
    elsif ($connect_log_string =~ /\[.+?:.+?:info\]:/) {

        $connect_log_pattern = qr/\[(.+?):.+?\]:\s+?(ndmpd\.?.+?) for version\s+?=\s+?(\d+?),\s+?sessionId\s+?=\s+?($session_id),\s+?from src ip\s+?=\s+?(\d+?\.\d+?\.\d+?\.\d+?), dst ip\s+?=\s+?(\d+?\.\d+?\.\d+?\.\d+?), src port\s+?=\s+?(\d+?), dst port\s+?=\s+?(\d+)/i;

        $connect_log_string =~ /$connect_log_pattern/;

        %connect_log_hash = (
            'destination_filer' => $1,
            'connection_status' => $2,
            'ndmp_version'=> $3,
            'session_id'=> $4,
            'source_ip'=> $5,
            'destination_ip'=> $6,
            'source_port'=> $7,
            'destination_port'=> $8,
        );

    }

    # Error if connect log string passed is of different format than expected
    else {
        croak "Connect log string passed is of different format than expected: $connect_log_string";
    }

    # Converting the connection status element to common foramt
    # For ex: 1. "ndmpd(authtype+password) allowed" to "ndmpd_authtype_password_allowed"
    # For ex: 2. "ndmpd password denied" to "ndmpd_password_denied"
    $connect_log_hash{'connection_status'} =~ s/(:\s?|\)\s?)/ /g;
    $connect_log_hash{'connection_status'} =~ s/\s?successfully\s?//g;
    $connect_log_hash{'connection_status'} =~ s/[\s\.(+)]/_/g;

    return (\%connect_log_hash);
}

sub DESTROY {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

        my $self = shift;

        return if ( $$ != $self->{_owner} );
        NOTE::logcomment( "Disconnecting from filer ");
}


1;

