#
# $Id: //depot/prod/test/main/lib/NOTE/REP.pm $
#
## @summary  REP Functions
## @status   Review
## @pod      here
## @author   shail@netapp.com


=head1 NAME

NOTE::REP

=head1 DESCRIPTION

This is a simple raid error propogation library works by creating a sample ascii file and corrupts L0/L1 blocks
Disk type fiji is supported, wherein RAID_SET_BLOCK_PSEUDOBAD is altered...


=head1 METHODS

=cut


package NOTE::REP;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
use base 'Exporter';
our @EXPORT = qw(init_raiderr_corruption);
use NOTE;
$NOTE::_SPARE_ALL=1;
use Data::Dumper;
use CLI::FileIO qw(wrfile);
use Volume qw(vol_vvlabel vol_exists vol_status vol_container);
use FS::Util::Utilities qw(dumpblock);
use POSIX qw(ceil);
use Carp qw(croak);
use SayWrap qw(sayEOF);
use Params::Validate qw (validate validate_pos SCALAR OBJECT);
use filer;
use Wafliron qw(wafliron_start);
use SnapCmds qw(snapshot_create snapshot_status);

my $TRAD_SIZE  = 3;
my $VOL_SIZE   = "100m";
my $VOL_TYPE   = "flex";
my $FILER_AGGR = "aggr0";
my $FILER_VOL  = "vol0";
my $FILER_QT   = "qs";
my $BLOCKTYPE  = "1"; 
my $BLOCKNUM = 0;
my $BLOCKCOUNT = 999;
my $FILEID     = undef;
my $FILE       = '/dir1/file1' ; 
my @FDATA = ('a' .. 'z');
my $USERDATA = join(", ",@FDATA);
my $RAID_RETURN_PSEUDO_BAD_ERROR = 1;
my $RAID_SET_BLOCK_PSEUDOBAD     = 0;
my $SNAPNAME                     = undef;
my $CORRUPT_ACLS                 = 0;
my $CORRUPT_STREAMS              = 0;
my $CORRUPT_STREAMDIR            = 0;
my $Volume = undef;
my $FILER = undef;

=head2 ::new

Creates a Raid Error Propagation Lib Object

=over

=item Arguments

none

=item Examples

=back

     $repObj = new NOTE::REP();

=cut
sub new {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $package = shift;
    return bless( {}, $package );
}



=head2 ::init_raiderr_corruption

this function enables wafl_inconsistent_panic_testing flag and uses fiji error injection modules to induce raid errors
function is higly sensitive with ontap version as well as the hardware configuration. 
RAID_SET_BLOCK_PSEUDOBAD flag is set to 1 to induce the failure

=over

=item Arguments

B<FILER> => I<Filer Name with fiji drives version of ontap that supports RAID_SET_BLOCK_PSEUDOBAD error flags>

I<Required.>
filer with compatible ontap version & fiji drives

B<FILER_AGGR> => I<Aggregate name to be specified in case user prefers a 32 or 64 bit aggregate>

I<Optional.>
Aggregate Name

=item Examples

=back

     #$repObj->init_raiderr_corruption(FILER=>'f6070-215-40',FILER_AGGR=>'aggr0');

=cut

sub init_raiderr_corruption($,%) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

   my ( $self, %args ) = @_;
   $FILER =
      exists( $args{FILER} ) ? $args{FILER} : croak("init_raiderr_corruption requires a compatible filer");
	  
    my $FILER_AGGR = exists($args{FILER_AGGR})? $args{FILER_AGGR} : NOTE::logcomment("using default aggregate options...");

    my ( $Parent, $snapid, $fileid, $fileinfo, $container_fileid );
## @step Create a new volume or attach if it already exists
    $Volume = NOTE::Volume->create(
          Filer => { host => $FILER },
          type  => $VOL_TYPE,
          name  => $FILER_VOL,
          flex  => {
              size      => $VOL_SIZE,
              Aggregate => {
                  name    => $FILER_AGGR,
                  recycle => 1
              }
          },
          trad => {
              disks   => $TRAD_SIZE,
              recycle => 1
          },
          recycle => 1,
      );
      
      
      my $Aggr = $Volume->Aggregate();
      my ( $agstate, $agstatus, $agaggr, $agoptions ) =  $Aggr->status();
      my $uninit  = "uninit";
      if ($agstatus =~/$uninit/)
	  {
		 NOTE::logcomment($agstatus." fake init detected croaking; please ensure real zeroed filers for this operation");
         croak("Reinstate the filer with zeroed disk");
	  }
      
      
## @step Turn on wafl_inconsistent_panic_testing flag
    $Volume->Filer->say(
          "priv set test \;setflag wafl_inconsistent_panic_testing 1");
		  
	$Volume->Filer->say(
          "priv set test \;setflag wafl_iron_panic_on_corruption 0");		  

    # Get snapid of snapshot, if specified
    if ($SNAPNAME) {
          foreach my $snaprec (@snaps) {
              if ( $$snaprec{name} =~ /$SNAPNAME/ ) {
                  $snapid = $$snaprec{snapid};
                  last;
              }
          }
          croak "Specified snapshot $SNAPNAME doesnot exist in the volume "
            . $Volume->name
            if !($snapid);
    }
    else { $snapid = 0 }
## @step Check whether ontap version is suitable for this testing
    my $version = filer_version_get( $Volume->Filer->conn );
    croak
"Cannot inject RAID_SET_BLOCK_PSEUDOBAD fiji, unless ONTAP version is ironcity"
      and $RAID_SET_BLOCK_PSEUDOBAD = 0
      if ( $RAID_SET_BLOCK_PSEUDOBAD == 1 && $version !~ /ironcity/ );
    croak
"Both RAID_RETURN_PSEUDO_BAD_ERROR and RAID_SET_BLOCK_PSEUDOBAD are set to 0. No Fiji injection!"
      if ( !( $RAID_RETURN_PSEUDO_BAD_ERROR || $RAID_SET_BLOCK_PSEUDOBAD ) );

    $Parent = $Volume;

## @step Create qtree if FILER_QT is specified
    my $Qtree;
    if ( defined $FILER_QT ) {
          eval {
              $Qtree =
                NOTE::Qtree->create( Volume => $Volume, name => $FILER_QT );
          };
          if ( $@ =~ /qtree create: Can\'t create qtree \(File exists\)/ ) {
              $Qtree =
                NOTE::Qtree->attach( Volume => $Volume, name => $FILER_QT );
          }
          else {
              croak("Error during Qtree creation: $@");
          }
          $Parent = $Qtree;
    }

## @step Get fileid of file to be corrupted
    $fileid =
      $FILEID
      ? _check_fileid( $Parent, $FILEID, $snapid )
      : _check_file( $Parent, $FILE, $snapid );

    # Get 'Container fileid' of the volume
    my %vvlabel = vol_vvlabel(
          fc     => $Volume->Filer->conn,
          volume => $Volume->mount_path
      );

    $container_fileid = $vvlabel{'Container fileid'};

## @step Inject pseudo error fiji for this file
    _corrupt(
          Volume           => $Volume,
          fileid           => $fileid,
          level            => $BLOCKTYPE,
          snapid           => $snapid,
          container_fileid => $container_fileid
      );

## @step Inject psuedo error fiji for this file's ACL, STREAMS and STREAMDIR
    $fileinfo = _stat_fileid( $Parent, $fileid, $snapid );
    if ( $fileinfo->{'type'} =~ /1|2/ ) {

          if ($CORRUPT_ACLS) {
              NOTE::logcomment('No ACL associated with this file to corrupt')
                if !( exists $fileinfo->{'xinode'} );
              _corrupt(
                  Volume           => $Volume,
                  fileid           => $fileinfo->{'xinode'},
                  level            => $BLOCKTYPE,
                  snapid           => $snapid,
                  container_fileid => $container_fileid
              ) if ( $fileinfo->{'xinode'} );
          }
          else {
              NOTE::logcomment('CORRUPT_ACLS => 0 - ACL corruption disabled');
          }

          if ($CORRUPT_STREAMDIR) {
              NOTE::logcomment(
                  'No streamdir associated with this file to corrupt')
                if !( exists $fileinfo->{'sinode'} );
              _corrupt(
                  Volume           => $Volume,
                  fileid           => $fileinfo->{'sinode'},
                  level            => $BLOCKTYPE,
                  snapid           => $snapid,
                  container_fileid => $container_fileid
              ) if ( $fileinfo->{'sinode'} );
          }
          else {
              NOTE::logcomment(
                  'CORRUPT_STREAMDIR => 0 - Streamdir corruption disabled');
          }

          if ($CORRUPT_STREAMS) {
              NOTE::logcomment(
                  'No streams associated with this file to corrupt')
                if !( exists $fileinfo->{'streams'} );
              if ( $fileinfo->{'sinode'} && $fileinfo->{'streams'} ) {
                  foreach ( keys %{ $fileinfo->{'streams'} } ) {
                      _corrupt(
                          Volume           => $Volume,
                          fileid           => $fileinfo->{'streams'}->{$_},
                          level            => $BLOCKTYPE,
                          snapid           => $snapid,
                          container_fileid => $container_fileid
                      );
                  }
              }
          }
          else {
              NOTE::logcomment(
                  'CORRUPT_STREAMS => 0 - Streams corruption disabled');
          }
    }
## @step Flush cache before exiting
    $Volume->Filer->say("priv set -q diag;wafl flush cache");
}
sub _corrupt {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my %opts = validate( @_,
          {
              Volume => { type     => OBJECT },
              fileid => { type     => SCALAR, optional => 1 },
              snapid => { optional => 1, default => 0 },
              level  => { type     => SCALAR, optional => 1, default => 0 },
              container_fileid => { type => SCALAR },
          } );
      my ( $Volume, $fileid, $level, $snapid, $container_fileid );
      $Volume           = $opts{'Volume'};
      $fileid           = $opts{'fileid'};
      $level            = $opts{'level'};
      $snapid           = $opts{'snapid'};
      $container_fileid = $opts{'container_fileid'};

      my ( $dump, %vvlabel );

      # sync to ensure that dumpblock finds the block on disk
      $Volume->Filer->say("priv set -q diag; sync");

      # Get details of file using 'dumpblock'
      $dump = dumpblock(
          conn       => $Volume->Filer->conn,
          block_type => 'info',
          volume     => $Volume->name,
          fileid     => $fileid,
          snapid     => $snapid,
          level      => $level,
          blockno    => $BLOCKNUM
        );

      if ( $dump->{disk} =~ /unknown/i ) {
          croak("dumpblock failed");
      }

      NOTE::logcomment(
"Set fiji point for L$level corruption (in snapshot $snapid) at block $BLOCKNUM"
        );
      if ($RAID_RETURN_PSEUDO_BAD_ERROR) {
          _raid_return_pseudo_bad_error(
              conn    => $Volume->Filer->conn,
              vol     => $Volume->Aggregate->name,
              fileid  => $container_fileid,
              fbn     => $dump->{vvbn},
              snapid  => $snapid,
              is_priv => 1
          );
      }

      if ($RAID_SET_BLOCK_PSEUDOBAD) {
          _raid_set_block_pseudobad(
              conn    => $Volume->Filer->conn,
              vol     => $Volume->Aggregate->name,
              fileid  => $container_fileid,
              fbn     => $dump->{vvbn},
              snapid  => $snapid,
              is_priv => 1
          );
      }
      return $dump->{vvbn};
}

sub _check_file {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my ( $Parent, $file, $snapid ) = @_;
      NOTE::logcomment( "Create a non-zero content files on "
            . $Parent->mount_path
            . " for L$BLOCKTYPE corruption on Block $BLOCKNUM" );

      if ($snapid) {
          croak
"FILE(-n) should be specified along with SNAPNAME(-s) (When corruption is not at AFS)"
            if ( !defined $file );

          my $snap_filepath =
            $Parent->mount_path . "/.snapshot/$SNAPNAME/" . $file;

          NOTE::logcomment(
              "Verifying whether given FILE $snap_filepath exists");
          croak
            "Given FILE $file is not present in specified snapshot $SNAPNAME"
            if !( _stat( $Parent, $snap_filepath, $snapid ) );

          return $Parent->mount_path . "/" . $file;
      }

      # this file has to have more than 64 bytes of data for L0 corruption
      # and atleast 1mb for L1 corruption
      my $min_datasize = $BLOCKTYPE ? 1048576 : 64;
      $min_datasize = $min_datasize + $BLOCKNUM * 4096;
      my ( $file_stat, $file_path );

      if ( defined $file ) {
          $file_path = $Parent->mount_path . "/" . $file;
          $file_stat = _stat( $Parent, $file_path );
          if ( defined $file_stat && ref $file_stat =~ /HASH/ ) {
              if ( $file_stat->{type} == 2 ) {
                  NOTE::result( 'WARN',
"L$BLOCKTYPE corruption cannot be guaranteed for a directory"
                  ) if ($BLOCKTYPE);
                  return $file_path;
              }
              return $file_path if ( $file_stat->{size} >= $min_datasize );
          }

          if ( !defined $file_stat ) {
              if ( $file =~ /\/$/ ) {
                  $file =~ s/\/$//g;
                  _create_dir_structure( $Parent, $file );
                  NOTE::result( 'WARN',
"L$BLOCKTYPE corruption cannot be guaranteed for a directory"
                  ) if ($BLOCKTYPE);
                  return $Parent->mount_path . "/" . $file;
              }
              else {
                  _create_file( $Parent, $file );
              }
          }
      }
      else {
          $file_path = $Parent->mount_path . "/userfile_$$";
      }

      my $data = $USERDATA;
      $data = $data x ceil( $min_datasize / length($data) );

      wrfile( $Parent->Filer->conn, $file_path, $data )
        or croak("Could not write $file");
      return _stat( $Parent, $file_path )->{fileid};
}

sub _check_fileid {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my ( $Parent, $fileid, $snapid ) = @_;
      NOTE::logcomment(
          "Check whether a file with given fileid $fileid is present on "
            . $Parent->mount_path
            . " for L$BLOCKTYPE corruption on Block $BLOCKNUM" );
      my $dump;
      eval {
          $dump = dumpblock(
              conn       => $Volume->Filer->conn,
              block_type => 'info',
              volume     => $Volume->name,
              fileid     => $fileid,
              snapid     => $snapid,
              level      => $BLOCKTYPE,
              blockno    => $BLOCKNUM
          );
        };
      NOTE::logcomment($dump);
      croak
        "Unable to read L$BLOCKTYPE block $BLOCKNUM of file (fileid: $fileid)"
        if !($dump);
      return $fileid;
}

sub _create_file {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my ( $Parent, $file ) = @_;
      if ( $file =~ /\// ) {
          my ($dir) = $file =~ /^(.*)\/\w+$/;
          _create_dir_structure( $Parent, $dir ) if ($dir);
      }
      $Parent->nfs->create($file) if ( $file !~ /\/$/ );
}

sub _create_dir_structure {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my ( $Parent, $dir ) = @_;
      my $obj;
      eval { $obj = $Parent->nfs->mkdir($dir); };
      croak $@ if ( $@ =~ /Permission denied/ );
      if ( $@ =~ /No such file or directory/ ) {
          $dir =~ s/\/\w+$//g;
          create_dir_structure( $Parent, $dir );
      }
      NOTE::logcomment( Dumper( $obj->stat ) );
      croak "Dir creation did not happen" if !( $obj->stat );
}

sub _stat {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my ( $Parent, $file, $snapid ) = @_;
      $snapid = $snapid ? $snapid : 0;

      # Get slist info
      my $file_stat = $Parent->Filer->say("priv set -q test; slist $file");
      return if ( $file_stat =~ /Invalid path/ );
      my @stat = split /\n/, $file_stat;
      my ( $fileid, $size, $name ) = $stat[2] =~ /\s*(\w+)\s+(\w+)\s+(\S+)/;
      my %fileinfo;
      $fileinfo{'fileid'} = $fileid;
      $fileinfo{'size'}   = $size;
      $fileinfo{'name'}   = $name;
      $file_stat =
      $Parent->Filer->say( "priv set -q test; dumpblock ifile %s %s %s", $Volume->name, $fileinfo{fileid}, $snapid );
      @stat = split /,/, $file_stat;

      foreach (@stat) {
          if (/type\s*=\s*(\w+)/) {
              $fileinfo{type} = $1;
          }
          elsif (/(\w+)\s(\w+)/) {
              $fileinfo{$1} = $2;
          }
      }
      NOTE::logcomment(
          "Details of file under corruption:" . Dumper(%fileinfo) );
      return \%fileinfo;
}

sub _stat_fileid {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my ( $Parent, $fileid, $snapid ) = @_;
      $snapid = $snapid ? $snapid : 0;
      my %fileinfo;
      my $file_stat =
        $Parent->Filer->say( "priv set -q test; dumpblock ifile %s %s %s",
          $Volume->name, $fileid, $snapid );
      my @stat = split /,/, $file_stat;
      foreach (@stat) {
          if (/type\s*=\s*(\w+)/) {
              $fileinfo{type} = $1;
          }
          elsif (/(\w+)\s(\w+)/) {
              $fileinfo{$1} = $2;
          }
      }
      if ( exists $fileinfo{type} && $fileinfo{type} =~ /1|2/ ) {
          $file_stat =
            $Parent->Filer->say( "priv set -q test; inodepath -v %s -d %s %s",
              $Volume->name, $snapid, $fileid );
          $fileinfo{'name'} = $1
            if ( $file_stat =~ /Primary pathname = (\S+)/ );
      }
      if ( exists $fileinfo{name} && $fileinfo{name} =~ /\w/ ) {
          $file_stat =
            $Parent->Filer->say( "priv set -q test; slist %s",
              $fileinfo{name} );
          my @streams = split /\n/, $file_stat;
          my $streamcount = @streams;
          my %streams;
          for ( my $i = 3 ; $i < $streamcount ; $i++ ) {
              my ( $streamid, $streamsize, $streamname ) =
                $streams[$i] =~ /\s*(\w+)\s+(\w+)\s+(\S+)/;
              $streams{$streamname} = $streamid;
          }
          $fileinfo{'streams'} = \%streams;
      }
      NOTE::logcomment(
          "Details of file under corruption:" . Dumper(%fileinfo) );
      return \%fileinfo;
}

sub _raid_return_pseudo_bad_error (%) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my %opts = validate( @_,
          {
              conn   => { type => OBJECT },
              vol    => { type => SCALAR },
              fileid => { type => SCALAR, optional => 1 },
              fileop => { type => SCALAR, optional => 1, default => "==" },
              path    => { type => SCALAR, optional => 1 },
              snapid  => { type => SCALAR, optional => 1 },
              level   => { type => SCALAR, optional => 1 },
              fbn     => { type => SCALAR, optional => 1 },
              vbn     => { type => SCALAR, optional => 1 },
              count   => { type => SCALAR, optional => 1, default => 999 },
              is_priv => { type => SCALAR, optional => 1, default => 0 },
              iron    => { type => SCALAR, optional => 1, default => 0 },
          } );

      my $conn = $opts{conn};
      my $vol  = $opts{vol};

      my ( $fileid, $path, $snapid, $level, $fbn, $vbn, $count, $is_priv );

      my $fileidstr = "";
      my $snapidstr = "";
      my $levelstr  = "";
      my $fbnstr    = "";
      my $vbnstr    = "";
      my $countstr  = "";
      my $privstr   = "";

      my $version = filer_version_get($conn);
      if ( ( $version =~ /main.*/ )
          || ( $version =~ /br-main.*/ )
          || ( $version =~ /boiler.*/ )
          || ( $version =~ /br-boiler.*/ ) )
      {
          $is_priv = $opts{is_priv};
          $privstr = "(== \@is_priv $is_priv)";
      }
      if ( defined( $opts{fileid} ) ) {
          $fileid    = $opts{fileid};
          $fileidstr = "($opts{fileop} \@fileid $fileid)";
      }

      if ( defined( $opts{path} ) ) {
          $path = $opts{path};
      }

      # Use path only if fileid is not defined
      if ( !defined( $opts{fileid} ) && defined( $opts{path} ) ) {
          my $output = sayEOF( $conn, "priv set -q diag; showfh $path" );
          if ( !( $output =~ /fileid/i ) ) {
              croak("showfh $path: $output");
          }
          $output =~ /fileid=0x0*([0-9,a-f,A-F]*)/;    # leading 0x and 0's
          $fileid = hex($1);    # yes, hex() actually returns base 10
          $fileidstr = "($opts{fileop} \@fileid $fileid)";
      }

      if ( defined( $opts{snapid} ) ) {
          $snapid    = $opts{snapid};
          $snapidstr = "(== \@snapid $snapid)";
      }

      if ( defined( $opts{level} ) ) {
          $level    = $opts{level};
          $levelstr = "(== \@level $level)";
      }

      if ( defined( $opts{fbn} ) ) {
          $fbn    = $opts{fbn};
          $fbnstr = "(== \@fbn $fbn)";
      }

      if ( defined( $opts{vbn} ) ) {
          $vbn    = $opts{vbn};
          $vbnstr = "(== \@vbn $vbn)";
      }

      $count    = $opts{count};
      $countstr = "repeat $count";

      ## Introduce data corruption for the volume
      NOTE::logcomment("Introduce data corruption on volume");

      my $fijicmd =
"fiji filter \"( $countstr (raid.return.pseudo.bad.error (&& (eq \@vol $vol) $levelstr $fileidstr $privstr $snapidstr $fbnstr $vbnstr)  (return (true))))\"";
      my $output = sayEOF( $conn, $fijicmd );
      if ( !( $output =~ /filter set/i ) ) {
          croak("fiji filter: $output");
      }

      if ( $opts{iron} =~ /1/ ) {
          my ( $found, $error ) = wafliron_start(
              $conn,
              $vol,
              wait       => 0,
              scan_speed => 10
          );
      }
      return 0;
}

sub _raid_set_block_pseudobad (%) {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

      my %opts = validate( @_,
          {
              conn   => { type => OBJECT },
              vol    => { type => SCALAR },
              fileid => { type => SCALAR, optional => 1 },
              fileop => { type => SCALAR, optional => 1, default => "==" },
              path    => { type => SCALAR, optional => 1 },
              snapid  => { type => SCALAR, optional => 1 },
              level   => { type => SCALAR, optional => 1 },
              fbn     => { type => SCALAR, optional => 1 },
              vbn     => { type => SCALAR, optional => 1 },
              count   => { type => SCALAR, optional => 1, default => 999 },
              is_priv => { type => SCALAR, optional => 1, default => 0 },
              iron    => { type => SCALAR, optional => 1, default => 0 },
          } );

      my $conn = $opts{conn};
      my $vol  = $opts{vol};

      my ( $fileid, $path, $snapid, $level, $fbn, $vbn, $count, $is_priv );

      my $fileidstr = "";
      my $snapidstr = "";
      my $levelstr  = "";
      my $fbnstr    = "";
      my $vbnstr    = "";
      my $countstr  = "";
      my $privstr   = "";

      my $version = filer_version_get($conn);
      if ( ( $version =~ /main.*/ )
          || ( $version =~ /br-main.*/ )
          || ( $version =~ /boiler.*/ )
          || ( $version =~ /br-boiler.*/ ) )
      {
          $is_priv = $opts{is_priv};
          $privstr = "(== \@is_priv $is_priv)";
      }
      if ( defined( $opts{fileid} ) ) {
          $fileid    = $opts{fileid};
          $fileidstr = "($opts{fileop} \@fileid $fileid)";
      }

      if ( defined( $opts{path} ) ) {
          $path = $opts{path};
      }

      # Use path only if fileid is not defined
      if ( !defined( $opts{fileid} ) && defined( $opts{path} ) ) {
          my $output = sayEOF( $conn, "priv set -q diag; showfh $path" );
          if ( !( $output =~ /fileid/i ) ) {
              croak("showfh $path: $output");
          }
          $output =~ /fileid=0x0*([0-9,a-f,A-F]*)/;    # leading 0x and 0's
          $fileid = hex($1);    # yes, hex() actually returns base 10
          $fileidstr = "($opts{fileop} \@fileid $fileid)";
      }

      if ( defined( $opts{snapid} ) ) {
          $snapid    = $opts{snapid};
          $snapidstr = "(== \@snapid $snapid)";
      }

      if ( defined( $opts{level} ) ) {
          $level    = $opts{level};
          $levelstr = "(== \@level $level)";
      }

      if ( defined( $opts{fbn} ) ) {
          $fbn    = $opts{fbn};
          $fbnstr = "(== \@fbn $fbn)";
      }

      if ( defined( $opts{vbn} ) ) {
          $vbn    = $opts{vbn};
          $vbnstr = "(== \@vbn $vbn)";
      }
      $count    = $opts{count};
      $countstr = "repeat $count";
      NOTE::logcomment("Introduce data corruption on volume");
      my $fijicmd =
"fiji filter \"( $countstr (raid.set.block.pseudobad (&& (eq \@vol $vol) $levelstr $fileidstr $privstr $snapidstr $fbnstr $vbnstr)  (return (true))))\"";
      my $output = sayEOF( $conn, $fijicmd );
      if ( !( $output =~ /filter set/i ) ) {
          croak("fiji filter: $output");
      }

      if ( $opts{iron} =~ /1/ ) {
          my ( $found, $error ) = wafliron_start(
              $conn,
              $vol,
              wait       => 0,
              scan_speed => 10
          );
      }
      return 0;
}
1;
__END__

