#
# $Id: //depot/prod/test/Rsierranevada/lib/NOTE/SnapMirror.pm#1 $
#
## @summary  NOTE SnapMirror class
## @status   Public
## @pod      here
## @author   dl-dpgqa-automation@netapp.com

=head1 NAME

NOTE::SnapMirror

=head1 DESCRIPTION

This is the NOTE class for working with SnapMirror relationships.

=head1 METHODS

=cut

use NOTE;
use POSIX qw();
package NOTE::SnapMirror;
our @ISA = qw(NOTE);

use strict;
use Carp qw(croak);
use SnapMirror qw();
use SnapMirror::Helper qw();
use RegistryCmds qw(registrySet registryGet);

my $regSlice = {
    'snapmirror.connection.mode'=> sub {
	'options.files.snapmirror.connection.'.shift->{_ics_src}.'.mode'
    },
    'snapmirror.connection.ips1'=> sub {
	'options.files.snapmirror.connection.'.shift->{_ics_src}.'ips1'
    },
    'snapmirror.connection.ips2'=> sub {
	'options.files.snapmirror.connection.'.shift->{_ics_src}.'ips2'
    },
    'snapmirror.connection.ipd1'=> sub {
	'options.files.snapmirror.connection.'.shift->{_ics_src}.'ipd1'
    },
    'snapmirror.connection.ipd2'=> sub {
	'options.files.snapmirror.connection.'.shift->{_ics_src}.'ipd2'
    },
    'snapmirror.conf'		=> sub {
	'options.files.snapmirror.conf.'.shift->dst_path
    },
    'snapmirror.conf.args'	=> sub {
        'options.files.snapmirror.conf.'.shift->dst_path.'.args'
    },
    'snapmirror.conf.dest'	=> sub {
	'options.files.snapmirror.conf.'.shift->dst_path.'.dest'
    },
    'snapmirror.conf.source' 	=> sub {
	'options.files.snapmirror.conf.'.shift->dst_path.'.source'
    },
    'snapmirror.conf.sched' 	=> sub {
	'options.files.snapmirror.conf.'.shift->dst_path.'.sched'
    },
    'contents'			=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.contents'
    },
    'current_size'		=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.current_size'
    },
    'current_transfer_error' 	=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.current_transfer_error'
    },
    'current_transfer_type' 	=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.current_transfer_type'
    },
    'lag' 			=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.lag'
    },
    'last_transfer_duration' 	=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.last_transfer_duration'
    },
    'last_transfer_from' 	=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.last_transfer_from'
    },
    'last_transfer_size' 	=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.last_transfer_size'
    },
    'last_transfer_type' 	=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.last_transfer_type'
    },
    'snapshot' 			=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.snapshot'
    },
    'source' 			=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.source'
    },
    'state' 			=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.state'
    },
    'status' 			=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.status'
    },
    'timestamp' 		=> sub {
	'status.snapmirror.active.'.shift->dst_path.'.timestamp'
    },
};

sub ics_src {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self, $ics_src) = @_;

    my $old_ics_src = $self->{_ics_src};
    $self->{_ics_src} = $ics_src if $ics_src;
    return $old_ics_src;
}

=head2 B<Relationship Creation>

=head2 ::initialize

Creates a SnapMirror relationship
and returns an object representing the relationship.

The src and dst filers of the src and dst object will be used unless specifically overridden from the
command line with NOTE_SM_CONN parameter. This parameter is used for ICS processing, and the format is as
follows:

	NOTE_SM_CONN=<src1,dst1>[:<src2,dst2>][:<mode>]

	where <src1,dst1> and <src2,dst2> are upto 2 src and dst pairs to be used by snapmirror
		<mode> is optional and is either 'failover' or 'multi'. Default: 'multi'

=over

=item Arguments

B<src> => I<Qtree object> | I<Volume object> | I<Qtree spec> | I<Volume spec> | 'NOTE:Qtree' | 'NOTE::Volume'

I<Optional.>
A Qtree or a volume object.

B<dst> => I<Qtree object> | I<Volume object> | I<Qtree spec> | I<Volume spec> | 'NOTE:Qtree' | 'NOTE::Volume'

I<Optional.>
A Qtree or a volume object.

The types of B<src> and B<dst> must match.
Both arguments are optional when C<::initialize> or C<::new>
is called as a class method.

Neither may be specified when called via an object.

B<mode> => C<async> | C<sync> | C<semi-sync>

I<Optional.>
Specifies whether the relationship will be synchronous or asynchronous.
The default is asynchronous.

B<connection_mode> => C<inet> | C<inet6>


I<Optional.>
Specifies the connection mode the be used. inet is used for IPv4 connection and inet6 for IPv6 connection
Applicable only for IC.3 and other IPv6 supporting releases. If used for builds that do not support IPv6, initialize will fail


B<compression> => C<1>|C<0>


I<Optional.>
If compression is provided, it will enable compression for the snapmirror relationship. Value has to be 1 or 0. For 1 it will set compression=enable in the conf file for 0 it will set compression=disable in the conf file.


B<background> => C<0.0> .. C<1.0>

I<Optional.>
A real number between 0 and 1 (inclusive) that
determines how much of the initial data transfer
should take place in the background.
The units are percent/100;
i.e. 0 means to return after the initial transfer has completed
and 1 means to return immediately.
The result of the transfer must be checked later using
L<C<::wait>|/::wait> or the C<wait> method of the returned object.
If the status is not checked using one of the wait methods
any errors in the transfer will go unnoticed.
The default is 0.

I<Optional.>

B<parallel> => I<1|0>

Boolean flag to indicate whether the filer command will be executed in
'parallel' mode. If set admin commands are expected to be already performed
via the 'new' method.

=item Conventions

This method obeys the following conventions:

- If $SM_TYPE is 'rsm', mode => 'sync' is implied.

- When creating a source container and the source filer
isn't specified, the value obtained from
NOTE::option(q/$FILER_SRC:$FILER/) is used.
If mode => sync, the value obtained from
NOTE::option(q/$FILERA:$FILER_SRC:$FILER/) is used.

- When creating a destination container and the destination filer
isn't specified, the value obtained from
NOTE::option(q/$FILER_DST:$FILER/) is used.
If mode => sync, the value obtained from
NOTE::option(q/$FILERB:$FILER_DST:$FILER/) is used.

- When creating a source or destination volume and the
type of the volume isn't specified, the value obtained from
NOTE::option(q/$VOL_TYPE:flex/) is used.

- When creating a source flex volume and the
size of the volume isn't specified, the value obtained from
NOTE::option(q/$VOL_SIZE/) is used.
If mode => sync, the value obtained from
NOTE::option(q/$VOL_SIZE:10g/) is used.

- When creating a destination flex volume and the size of the volume
isn't specified, the size of the source volume is used.

- When creating a source aggregate and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_SRC_AGGR/)
is used.

- When creating a source volume and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_SRC_VOL/)
is used.

- When creating a source qtree and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_SRC_QTREE/)
is used.

- When creating a destination aggregate and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_DST_AGGR/)
is used.

- When creating a destination volume and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_DST_VOL/)
is used.

- When creating a destination qtree and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_DST_QTREE/)
is used.

- Unless otherwise specified, destination aggregates will be private
if $WAFLIRON = 1.

=item Examples

=back

     $sm = NOTE::SnapMirror->initialize(
         src => $src_vol,
         dst => $dst_vol,
     );

     $sm = NOTE::SnapMirror->initialize(
         src        => $src_vol,
         dst        => $dst_vol,
         mode       => 'async',
         background => 0.25   # return when the transfer is 75% complete
     );
     $sm = NOTE::SnapMirror->initialize(
          src        => $src_vol,
          dst        => $dst_vol,
          mode       => 'async',
          connection_mode => 'inet6' #  Use an IPv6 connection.
      );


     # Create both source and destination volumes, as well as
     # snapmirror relationship:
     $sm = NOTE::SnapMirror->initialize;

     # Same as above, but source and destination volumes will be 200MB:
     $sm = NOTE::SnapMirror->initialize( src => { size => '200m' } );

     # Create both source and destination qtrees, as well as
     # snapmirror relationship:
     $sm = NOTE::SnapMirror->initialize( src => q/NOTE::Qtree/ );


=cut

sub initialize {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%args) = @_;
    my $self = {};
    # most likely an object. note that 'ref $class' is not compared
    # to NOTE::SnapMirror because this method can be inherited by a
    # subclass and 'ref $class' will not reflect 'NOTE::SnapMirror'
    # in the subclass.
    if ( ref $class ne '') {
	$self = $class;
    
	if ( exists $args{src} || exists $args{dst} ) {
	    croak "Cannot override existing 'src' or 'dst' attribute ".
	      "in existing object.\n";
	}
    
    } else {
	$self = NOTE::SnapMirror->new( %args );    
    }

    $self->_set_sm_conn; # Set sm connection if provided at command line

    my $mode = 'async';
    if ( exists $args{mode} ) {
	if ( $args{mode} ne 'async' && $args{mode} ne 'sync' && $args{mode} ne 'semi-sync') {
	    croak "Mode has to be either 'async' or 'sync' or 'semi-sync'";
	} else {
	    $mode = $args{mode};
	}
    }

    delete $args{src};
    delete $args{dst};
    delete $args{mode};

    # If background is between 0 and 1, then it indicates
    # how much in the 'background' the transfer should be.
    my $kilos;
    my $background = (exists $args{background}) ? $args{background} : -1.0;
    if ( ($background < 1.0) && ($background > 0.0) ) {
	$kilos = int( $self->{_src}->used * (1.0 - $background) );
	$args{background} = 1;
    }
# Checking if connection mode is passed as a parameter in Snapmirror Initialize
     # Two values possible for connection_mode is inet and inet6 and is applicable only for IC.3 and other IPv6 supporting releases.
     if ( exists $args{connection_mode})
     {
     $self->{connection_mode} =$args{connection_mode};
     }
     # Checking if connection mode is passed as a parameter in Planfile. If connection mode parameter is not set in the test script, plan file settings
     # will hold. Two values possible for connection_mode is inet and inet6 and is applicable only for IC.3 and other IPv6 supporting releases.
     elsif (defined NOTE::option( q/$CONN_MODE/ ))
     {
         $args{connection_mode}=NOTE::option( q/$CONN_MODE/ );
         $self->{connection_mode} =$args{connection_mode};
     }
    
    my $op = "Snapmirror Initialize" . $self->description;
    NOTE::logcomment( $op );

    my $ret;
    my $start_time = time;
    my $timeout    = $args{'timeout'};
    $args{'timeout'} = $args{'timeout'} ? $args{'timeout'} : 8*60*60; # We are using 'smart timeout', so increasing this to give time for large data sets.

    unless ( $args{parallel} ) {
	$self->{_src}->Filer->check_license( "snapmirror" );
	$self->{_dst}->Filer->check_license( "snapmirror" );
    }
    eval {
	if ($mode eq 'async') {
	    $args{'background'} = 1 unless ( ($background <= 0) && defined $timeout );
	    # VSM or QSM
	    $args{srchost} = $self->{_src}->Filer->hostrec;
	    $args{dsthost} = $self->{_dst}->Filer->hostrec;
	    $args{src} = $self->src_path;
	    $args{dst} = $self->dst_path;
	    if ( $args{parallel} ) {
		$ret = SnapMirror::Helper::sm_init_proc(%args);
	    } else {
		$ret = SnapMirror::Helper::sm_init( %args );
	    }
	} else {
	    # RSM
	    $self->{_src}->Filer->check_license( "snapmirror_sync" );
	    $self->{_dst}->Filer->check_license( "snapmirror_sync" );
	    $ret = SnapMirror::Helper::rsm_init(
	        srchost => $self->{_src}->Filer->hostrec,
	        dsthost => $self->{_dst}->Filer->hostrec,
	        src	=> $self->src_path,
	        dst	=> $self->dst_path,
	        semi_sync=>$mode eq 'semi-sync'?1:0,
	        %args
	    );
	}
	if ( ref $ret eq "Subtest" ) { # Save, if it's an object
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	}
    };
    croak $@ if $@;
    NOTE::reference( $self );

    # the background option when set for RSM, it will return the control to
    # the caller when the mirror is transistioning to either full or CP sync
    # mode. during that transition phase, the idea of 'background' doesn't
    # work well as we have no clue (besides the lag time) on how un-sync
    # the mirror is in.
    if ( $mode eq 'async' ) {
	if ( $kilos || (($background <= 0) && !defined $timeout) ) {
	    $self->monitor( transfer_size => $kilos );
	}
    }
    if ( $background <= 0 && ! $args{parallel} ) { # Skip rsh command as zapi only in parallel run
	my $transfer_secs = time - $start_time;
	my $last_transfer_size = $self->{'last_transfer_size'};
	NOTE::logcomment( "Snapmirror Initialize transferred %.3f MB in %d seconds; throughput was %.3f MB/s",
			$last_transfer_size / 1000,
       		 $transfer_secs,
			($last_transfer_size / 1000) / $transfer_secs
	);
    }
    $self->dst->exportfs unless( $args{parallel} || $background > 0 ); # Sync dst's exportfs state with filer's if necessary
    return $self;
}

=head2 B<Monitor last snapmirror action>

=head2 ::monitor

Monitors the last backgrounded snapmirror action such as initialize, update.

=over

=item Arguments

B<transfer_size> => I<1>

I<Optional.>

Prints total transfer size if set

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 120 seconds for simulator and 60 for hardware filers.
The 'clock' is count down only when there are no activities.

B<interval> => I<number>

I<Optional.>

Polling interval for status. Default 2 secs.

=item Example

=back

     $sm->monitor();

     $sm->monitor(interval=>5);

=cut

sub monitor {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $transfer_size = $args{'transfer_size'};
    my $timeout = $args{'timeout'} || ($self->{_dst}->Filer->is_a_simulator ? 120 : 60);
    my $poll_interval = $args{'interval'} || 2; # seconds
    my $last_throughput = 0;
    my $progress = 0;
    my $last_progress;
    my $last_time = time;
    my $zero_throughputs = 0;
    my $max_zero_throughputs = POSIX::ceil( $timeout / $poll_interval );
    my $state;
    my $intro = 0;
    my $test = $self->{_subtests}->[-1]; # Can monitor only most recent subtest;
    return $self unless $test;
    my $op = $test->{operation};
    my $subtest = $test->{subtest};

    while ( ($state = (my $status = $self->status)->{'status'}) !~ /(idle|in-sync)/ ) {
	last unless ( $subtest->is_alive() );
        if ( $intro++ == 0 ) { # Only print once
	    NOTE::logcomment(
		"Monitoring transfer to %s",
		$self->dst->description,
	    );
	}
	$progress = $status->{'transfer-progress'};
	my $secs = (time - $last_time);
	my $bytes_transferred = ($progress - $last_progress);
	if ( $secs ) {
	    $last_throughput = $bytes_transferred / $secs;
	    if ( defined $transfer_size ) {
		NOTE::logcomment(
		    "Transferred %.3f MB of %.3f MB; throughput is %.3f MB/s; state is %s",
		    $progress/1000, $transfer_size/1000, $last_throughput/1000, $state,
		);
	    } else {
		NOTE::logcomment(
		    "Transferred %.3f MB; throughput is %.3f MB/s; state is %s",
		    $progress/1000, $last_throughput/1000, $state,
		);
	    }
	}
	if ( !defined $transfer_size || ($progress < $transfer_size) ) {
	    if ( $bytes_transferred > 0 ) {
		$zero_throughputs = 0;
	    } else {
		$zero_throughputs++;
		croak sprintf(
		    "Transfer timed-out after %d seconds of inactivity",
		    $zero_throughputs * $poll_interval,
		) unless ( $zero_throughputs < $max_zero_throughputs );
	    }
	    sleep $poll_interval;
	    next;
	}
	NOTE::logcomment( "Monitor complete; transfer may continue" );
	# We do NOT want to wait for the transfer to complete
	# because we may have been given criteria for early monitor
	# termination.
	return $self; # Progress meets or exceeds expected transfer size
    }
    pop @{$self->{_subtests}};
    $subtest->wait_finish();
    SnapMirror::Helper::_sm_extract_subtest_err( $subtest );
    if ( $intro ) { # Don't print unless transfer occurred
	NOTE::logcomment( "Transfer complete; state is %s", $state );
    }
    return $self;
}

=head2 ::attach

Creates an object representing a SnapMirror relationship,
but does B<not> affect or cause the creation
of a SnapMirror relationship on the filer.
If the source and destination given
are already in a SnapMirror relationship with each other
then other C<NOTE::SnapMirror> methods may be used
to control and manipulate the relationship
just as if L<C<::initialize>|/::initialize>
was used to create the relationship.
If the source and destination given
are not already in a SnapMirror relationship with each other
the C<::initialize> method
of the returned object may be used to create a relationship.

=over

=item Arguments

B<srcpath> => I<string>

I<Required.>
The host path of a Qtree or a volume.

B<dstpath> => I<string>

I<Required.>
The host path of a Qtree or a volume.

A host path is a full path name
prepended with the filer name.

The types of B<srcpath> and B<dstpath> must match.

B<ics_path> => I<string>

If exists will be used in place of the host name when
specifying src path.

=item Example

=back

     $sm = NOTE::SnapMirror->attach(
         srcpath => 'myfiler:/vol/myvol',
         dstpath => 'yourfiler:/vol/yourvol',
     );

=cut

# XXX Should this function check if there is actually a
# valid relationship between the source and destination?
# Thompson/ Thu Dec  8 15:14:46 PST 2005
sub attach {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class,%args) = @_;
    my $self = {};

    my $src_path = $args{srcpath};
    my $dst_path = $args{dstpath};

    # most likely an object. note that 'ref $class' is not compared
    # to NOTE::SnapMirror because this method can be inherited by a
    # subclass and 'ref $class' will not reflect 'NOTE::SnapMirror'
    # in the subclass.
    if ( ref $class ne '') {
	$self = $class;
    
	if ( exists $args{srcpath} || exists $args{dstpath} ) {
	    croak "Cannot override existing 'src_path' or 'dst_path'" .
	      "attribute in existing object.\n";
	}

	$src_path = $self->src_path;
	$dst_path = $self->dst_path;
    }

    my ($src_filer, $src_vol, $src_qtree);
    my ($dst_filer, $dst_vol, $dst_qtree);

    if ( $src_path =~ /([^:]+):\/vol\/([^\/]+)\/(.*)/ ) {
	($src_filer,$src_vol,$src_qtree) = ($1,$2,$3);
	$self->{_src} = NOTE::Qtree->attach(
	    Filer  => { host => $src_filer },
	    Volume => { name => $src_vol },
	    name   => $src_qtree,
	    force  => 1, # don't croak if qtree not present
	);
    } elsif ( $src_path =~ /([^:]+):(.*)/ ) {
	($src_filer,$src_vol) = ($1,$2);
	$self->{_src} = NOTE::Volume->attach(
	    Filer  => { host => $src_filer },
	    name   => $src_vol,
	    force  => 1, # don't croak if volume not present
	);
    } else {
	croak("Invalid source path: ".$src_path );
    }
    $self->{_src}->Filer->check_license( "snapmirror" );
    if ( $dst_path =~ /([^:]+):\/vol\/([^\/]+)\/(.*)/ ) {
	($dst_filer,$dst_vol,$dst_qtree) = ($1,$2,$3);
	$self->{_dst} = NOTE::Qtree->attach(
	    Filer  => { host => $dst_filer },
	    Volume => { name => $dst_vol },
	    name   => $dst_qtree,
	    force  => 1, # don't croak if qtree not present
	);
    } elsif ( $dst_path =~ /([^:]+):(.*)/ ) {
	($dst_filer,$dst_vol) = ($1,$2);
	$self->{_dst} = NOTE::Volume->attach(
	    Filer  => { host => $dst_filer },
	    name   => $dst_vol,
	    force  => 1, # don't croak if volume not present
	);
    } else {
	croak("Invalid destination path: ".$dst_path);
    }
    $self->{_Filer} = $self->{_dst}->Filer; # needed by NOTE::Registry.pm
    $self->{_Filer}->check_license( "snapmirror" );

    if ( exists $args{ics_path} ) {
	my ($ics_src, $src_path) = split ":", $args{ics_path};
	$self->{_ics_src} = $ics_src;
    }

    # not an object yet. bless it.
    if ( ref $class eq '' ) {
	$self->{_owner} = $$;
	bless $self, $class;
	tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;
	$self->SPARE; # Prevent relationship from being torn down automatically
    }

    return $self;
}

=head2 ::new

Creates an object representing a SnapMirror relationship,
but does B<not> cause the creation
of a SnapMirror relationship on the filer.
The L<C<::initialize>|/::initialize> method
of the returned object may be used to create the relationship.

This method should not normally be used;
use C<::initialize> directly instead.

=over

=item Arguments

B<src> => I<Qtree object> | I<Volume object> | I<Qtree spec> | I<Volume spec> | 'NOTE:Qtree' | 'NOTE::Volume'

I<Optional.>
A Qtree or a volume spec or object. May also be the string NOTE::Qtree or NOTE::Volume
to indicate directly that a Qtree or Volume with default parameters is to be created.
The default is 'NOTE::Volume'.

B<src> => I<Qtree object> | I<Volume object> | I<Qtree spec> | I<Volume spec> | 'NOTE:Qtree' | 'NOTE::Volume'

I<Optional.>
A Qtree or a volume spec or object. May also be the string NOTE::Qtree or NOTE::Volume
to indicate directly that a Qtree or Volume with default parameters is to be created.
The default is 'NOTE::Volume'.

The types of B<src> and B<dst> must match (i.e. if B<src> is a qtree, or resolves to one,
then B<dst> must also be a qtree, or resolve to one)

B<preprocess> => I<1|0>

I<Optional.>
Boolean flag to indicate whether preprocessing of
admin commands such as adding licensing and registry entries
should be performed. Default: 0

B<qt_src_exists} => I<1|0>

I<Optional.>
Boolean flag to indicate whether Qtree src exists or not.
This is only used in with the 'preprocess' parameter.
Default: 1

B<ics_src> => I<string>

If exists will be used in place of the host name when
specifying src path.

=item Conventions

This method obeys the following conventions:

- If $SM_TYPE is 'rsm', mode => 'sync' is implied.

- When creating a source container and the source filer
isn't specified, the value obtained from
NOTE::option(q/$FILER_SRC:$FILER/) is used.
If mode => sync, the value obtained from
NOTE::option(q/$FILERA:$FILER_SRC:$FILER/) is used.

- When creating a destination container and the destination filer
isn't specified, the value obtained from
NOTE::option(q/$FILER_DST:$FILER/) is used.
If mode => sync, the value obtained from
NOTE::option(q/$FILERB:$FILER_DST:$FILER/) is used.

- When creating a source or destination volume and the
type of the volume isn't specified, the value obtained from
NOTE::option(q/$VOL_TYPE:flex/) is used.

- When creating a source flex volume and the
size of the volume isn't specified, the value obtained from
NOTE::option(q/$VOL_SIZE/) is used.
If mode => sync, the value obtained from
NOTE::option(q/$VOL_SIZE:10g/) is used.

- When creating a destination flex volume and the size of the volume
isn't specified, the size of the source volume is used.

- When creating a source aggregate and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_SRC_AGGR/)
is used.

- When creating a source volume and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_SRC_VOL/)
is used.

- When creating a source qtree and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_SRC_QTREE/)
is used.

- When creating a destination aggregate and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_DST_AGGR/)
is used.

- When creating a destination volume and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_DST_VOL/)
is used.

- When creating a destination qtree and the name isn't
specified, the value obtained from NOTE::option(q/$FILER_DST_QTREE/)
is used.

- Unless otherwise specified, destination aggregates will be private
if $WAFLIRON = 1.

=item Example

=back

     $sm = NOTE::SnapMirror->new(
         src => $src_vol,
         dst => $dst_vol,
     );

=cut

use Data::Dumper;

sub new {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($class, %args) = @_;
    my $opts;
    my $self = {};

    $self->{_owner} = $$;

    $args{dst} ||= {};
    $args{src} ||= {};

    $self->{_mode} = $args{mode} ||
		     (NOTE::option( q/$SM_TYPE/ ) eq 'rsm' ? 'sync' : 'async');

    # TODO: For simulators, use guarentee=none

    my ($filerA,$filerA_vol,$filerA_qtree,$filerA_aggr);
    my ($filerB,$filerB_vol,$filerB_qtree,$filerB_aggr);
    if ( $self->{_mode} ne 'async' ) {
	$filerA       = q/$FILERA:/;
	$filerA_vol   = q/$FILERA_VOL:/;
	$filerA_qtree = q/$FILERA_QTREE:/;
	$filerA_aggr  = q/$FILERA_AGGR:/;
	$filerB       = q/$FILERB:/;
	$filerB_vol   = q/$FILERB_VOL:/;
	$filerB_qtree = q/$FILERB_QTREE:/;
	$filerB_aggr  = q/$FILERB_AGGR:/;
    }
    my $qsm_src_opts = {
	Filer  => $filerA.q/$FILER_SRC:$FILER/,
	name   => $filerA_qtree.q/$FILER_SRC_QTREE/,
	Volume => {
	    name    => $filerA_vol.q/$FILER_SRC_VOL/,
	    type    => q/$VOL_TYPE/,
	    recycle => 1,
	    flex => {
		size        => $self->{_mode} ne 'async' ? q/$VOL_SIZE:10g/ : q/$VOL_SIZE/,
		Aggregate => {
		    name    => $filerA_aggr.q/$FILER_SRC_AGGR/,
		    share   => 1,
		},
	    },
	},
    };
    my $vsm_src_opts = {
	Filer   => $filerA.q/$FILER_SRC:$FILER/,
	name    => $filerA_vol.q/$FILER_SRC_VOL/,
	type    => q/$VOL_TYPE/,
	recycle => 1,
	flex  => {
	    size        => $self->{_mode} ne 'async' ? q/$VOL_SIZE:10g/ : q/$VOL_SIZE/,
	    Aggregate => {
		name    => $filerA_aggr.q/$FILER_SRC_AGGR/,
		share   => 1,
	    },
	},
    };

    # Source:
    if ( ref \$args{src} eq 'SCALAR' ) {
	if ( $args{src} eq 'NOTE::Qtree' ) {
	    $opts = $qsm_src_opts;
	    $opts->{tag} ||= 'Source qtree';
	    $opts->{Volume}->{tag} ||= 'Source volume';
	    $opts->{Volume}->{flex}->{Aggregate}->{tag} ||= 'Source aggregate';
	    $self->{_src} = NOTE::Qtree->create( %{$opts} );
	} elsif ( $args{src} eq 'NOTE::Volume' ) {
	    $opts = $vsm_src_opts;
	    $opts->{tag}  ||= 'Source volume';
	    $opts->{flex}->{Aggregate}->{tag} ||= 'Source aggregate';
	    $self->{_src} = NOTE::Volume->create( %{$opts} );
	} else {
	    croak "Invalid src specification: ".$args{src};
	}
    } elsif ( ref $args{src} eq "HASH" ) {
        if ( $args{src}{Volume} || NOTE::option(q/SM_TYPE/) eq 'qsm' ) { # This must describe a qtree
	    $opts = $qsm_src_opts;
	    if ( ref $args{src} eq 'HASH' ) {
		$opts = NOTE::_hashOverlay( target => $opts, source => $args{src}, overwrite => 1 );
	    }
	    $opts->{tag} ||= 'Source qtree';
	    $opts->{Volume}->{tag} ||= 'Source volume';
	    $opts->{Volume}->{flex}->{Aggregate}->{tag} ||= 'Source aggregate';
	    $self->{_src} = NOTE::Qtree->create( %{$opts} );
	} else {
	    if ( ref \$args{src} eq 'SCALAR' && $args{src} ne 'NOTE::Volume' ) {
		croak "Invalid src specification: ".$args{src};
	    }
	    $opts = $vsm_src_opts;
	    if ( ref $args{src} eq 'HASH' ) {
		$opts = NOTE::_hashOverlay( target => $opts, source => $args{src}, overwrite => 1 );
	    }
	    $opts->{tag}  ||= 'Source volume';
	    $opts->{flex}->{Aggregate}->{tag} ||= 'Source aggregate';
	    $self->{_src} = NOTE::Volume->create( %{$opts} );
	}
    } elsif ( (ref($args{src}) =~ /^NOTE::/) && ($args{src}->isa("NOTE::Volume") || $args{src}->isa("NOTE::Qtree")) ) {
	$self->{_src} = $args{src}; # already an object
    } else {
	croak "Invalid src specification: ".ref $args{src};
    }

    # Destination:
    if ( ref $args{dst} eq "HASH" ) {
        if ( $self->{_src}->isa( "NOTE::Qtree" ) ) {
	    $opts = {
		Filer     => $filerB.q/$FILER_DST:$FILER_SRC:$FILER/,
	        name      => $filerB_qtree.q/$FILER_DST_QTREE/,
		Volume    => {
		    name => $filerB_vol.q/$FILER_DST_VOL/,
		    $self->{_src}->Volume->Properties,
		},
	    };
	    delete $opts->{Volume}->{dataset}; # Don't inherit dataset property from source
	    if ( $self->{_src}->Volume->type eq 'flex' ) {
	        $opts->{Volume}->{flex}->{Aggregate} => {
		    name    => $filerB_aggr.q/$FILER_DST_AGGR/,
		    recycle => 1,
		    $self->{_src}->Volume->Aggregate->Properties,
		    # No shared destination aggregate if wafliron will run
		    ## XXX Is this needed for QSM??? Check this.
		    share   => ! NOTE::option( q/$WAFLIRON/ ),
		},
	    }
	    $opts = NOTE::_hashOverlay( target => $opts, source => $args{dst}, overwrite => 1 );
	    if ( $opts->{Volume}->{type} eq 'flex' ) {
		# Since the source may be a trad vol, we must delete
		# any properties the source may have that are inappropriate for
		# a flex vol
		delete $opts->{Volume}->{disks};
		delete $opts->{Volume}->{raidtype};
		delete $opts->{Volume}->{rgsize};
	    } else { # Tradvol
		# Since the source may be a flex vol, we must delete
		# any properties the source may have that are inappropriate for
		# a trad vol
		delete $opts->{Volume}->{Aggregate};
		delete $opts->{Volume}->{size};
	    }
	    $opts->{tag} ||= 'Destination qtree';
	    $opts->{Volume}->{tag} ||= 'Destination volume';
	    $opts->{Volume}->{flex}->{Aggregate}->{tag} ||= 'Destination aggregate';
	    $self->{_dst} = NOTE::Qtree->new( %{$opts} ),
	} else {
	    $opts = {
		Filer => $filerB.q/$FILER_DST:$FILER_SRC:$FILER/,
	        name  => $filerB_vol.q/$FILER_DST_VOL/,
		$self->{_src}->Properties,
	    };
	    delete $opts->{dataset}; # Don't inherit dataset property from source
	    if ( $self->{_src}->type eq 'flex' ) {
	        $opts->{flex}->{Aggregate} = {
		    name    => $filerB_aggr.q/$FILER_DST_AGGR/,
		    recycle => 1,
		    $self->{_src}->Aggregate->Properties,
		    # No shared destination aggregate if wafliron will run
		    share   => ! NOTE::option( q/$WAFLIRON/ ),
		},
	    }
	    $opts = NOTE::_hashOverlay( target => $opts, source => $args{dst}, overwrite => 1 );
	    $opts->{tag} ||= 'Destination volume';
	    $opts->{flex}->{Aggregate}->{tag} ||= 'Destination aggregate';
	    if ( $self->{_src}->type eq 'flex' ) {
		if ( !defined $opts->{size} && !defined $opts->{flex}->{size} ) {
		    # Default to the size of the source
		    $opts->{flex}->{size} = $self->{_src}->size;
		}
	    }
	    $self->{_dst} = NOTE::Volume->create( %{$opts} );
	}
    } elsif ( (ref($args{dst}) =~ /^NOTE::/) && ($args{dst}->isa("NOTE::Volume") || $args{dst}->isa("NOTE::Qtree")) ) {    
	$self->{_dst} = $args{dst}; # already an object
    } else {
	croak "Invalid dst specification: ".ref $args{dst};
    }

    $self->{_Filer} = $self->{_dst}->Filer; # needed by NOTE::Registry.pm

    if ( exists $args{ics_src} ) {
	$self->{_ics_src} = $args{ics_src};
	delete $args{ics_src};
    }

    if ( $self->{_src}->isa( "NOTE::Qtree" ) ) {
	croak "Destination must be a qtree object if source is a qtree object"
	    unless ( $self->{_dst}->isa( "NOTE::Qtree" ) );
    }


    bless $self, $class;
    tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;

    if ( $args{preprocess} ) {
	delete $args{preprocess};
    	if (exists $args{qt_src_exists} ) {
		$args{no_qt_src} = ! $args{qt_src_exists};
		delete $args{qt_src_exists};
	}

	$self->{_src}->Filer->check_license( "snapmirror" );
	$self->{_dst}->Filer->check_license( "snapmirror" );
	$args{srchost} = $self->{_src}->Filer->hostrec;
	$args{dsthost} = $self->{_dst}->Filer->hostrec;
	$args{src} = $self->src_path;
	$args{dst} = $self->dst_path;
	SnapMirror::Helper::sm_init_pre(%args);
    }
    return $self;
}

sub _set_sm_conn {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self) = @_;

    if ( my $conn_opt = NOTE::option(q/$NOTE_SM_CONN/) ) {
    	my $err_fmt = "Invalid NOTE_SM_CONN, %s. Actual '$conn_opt', "
				."Expected: <src1,dst1>:[<src2,dst2>][:<failover|multi>].";
	my ($mode, $link_cnt) = ("multi", 0);

    	$self->{_sm_conn} = $conn_opt;
	$self->{_ics_src} = "sm_conn-".$$."-".time;
	NOTE::logcomment("Using ICS SRC Name: ".$self->{_ics_src});
	my $sm_conn = "options.files.snapmirror.connection.".$self->{_ics_src}.".";
	# Register ICS info on both src and dst so that it can be used with flip resync
	foreach my $item (split /:/, $conn_opt) {
		if ( $item =~ /,/ ) {
			$self->_clr_sm_conn (sprintf ($err_fmt, "Too many <src,dst> pair"))
				if ++$link_cnt > 2;
			my ($src_ip, $dst_ip) = split /,/, $item;
			if ( $src_ip && $dst_ip ) {
				NOTE::logcomment("Setting NOTE_SM_CONN from $src_ip to $dst_ip");
				# Override ics setting is any
				SnapMirror::Helper::sm_set_permissions
					(host=>$self->src->Filer->hostrec, allowed_host=>$dst_ip);
				SnapMirror::Helper::sm_set_permissions
					(host=>$self->dst->Filer->hostrec, allowed_host=>$src_ip);
				# Register regular src-dst on dst
				_reg_ics($self->dst->Filer, $sm_conn."ips".$link_cnt, $src_ip);
				_reg_ics($self->dst->Filer, $sm_conn."ipd".$link_cnt, $dst_ip);
				# Register flipped src-dst ip on src for flip resync
				_reg_ics($self->src->Filer, $sm_conn."ips".$link_cnt, $dst_ip);
				_reg_ics($self->src->Filer, $sm_conn."ipd".$link_cnt, $src_ip);
			} else {
				$self->_clr_sm_conn (sprintf ($err_fmt, "Missing dst"));
			}
		} else {
			if ( $item =~ /failover|multi/ ) {
				$mode = $item;
			} else {
				$self->_clr_sm_conn (sprintf ($err_fmt, "Invalid mode or missing dst"));
			}
		}
	}
	$self->_clr_sm_conn (sprintf ($err_fmt, "Missing <src,dst> pair"))
		unless $link_cnt;
	# Complete ics entry on both src and dst filers
	_reg_ics($self->src->Filer, $sm_conn."mode", $mode);
	_reg_ics($self->dst->Filer, $sm_conn."mode", $mode);
    }
}

sub _clr_sm_conn {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 
    my ($self, $err_msg) = @_;

    if ( exists $self->{_sm_conn} ) {
	delete $self->{_sm_conn};
	foreach my $f ($self->src->Filer, $self->dst->Filer) {
		eval {
			_reg_ics($f, "options.files.snapmirror.connection.".$self->{_ics_src});
		}; # Ignore any errors
	}
    }
    croak $err_msg if $err_msg;
}

sub _reg_ics {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($filer, $key, $value) = @_;

    my $reg = $filer->registry;
    if ( $value ) {
	$reg->{$key} = $value; # Register ICS info
    } else {
	
	delete $reg->{$key}; # Remove it if value is not given
    }
}

=head2 B<SnapMirror Commands>

The methods names in this section correspond to
ONTAP's C<snapmirror> subcommands.
Only administrator priviledge level subcommands are listed in this section.
These methods eventually call the self-named subcommands.
Only the briefest mention of what these subcommands do is given here.
See the SnapMirror documentation for more information on them.

SnapMirror commands that apply to the filer as a whole are part of NOTE::Filer.pm

=head2 ::abort

Aborts an active SnapMirror transfer.

=over

=item Arguments

B<hard> => C<0> | C<1>

I<Optional.>
If set to 1, does a hard abort,
such that the transfer is not restartable.
The default is 0.

=item Examples

=back

     $sm->abort;

     $sm->abort(
         hard => 1
     );

=cut

sub abort {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    NOTE::logcomment("Snapmirror Abort" . $self->description);
    eval {
	$ret = SnapMirror::snapmirror_abort(
	    host => $self->{_dst}->Filer->hostrec,
	    dst  => $self->dst_path,
	    %args
	);
    };
    croak $@ if $@;
    return $ret;
}

=head2 ::break

Breaks the SnapMirror relationship.

=over

=item Arguments

None.

=item Example

=back

     $sm->break;

=cut

sub break {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    NOTE::logcomment("Snapmirror Break" . $self->description);
    eval {
	$ret = SnapMirror::snapmirror_break(
	    host    => $self->{_dst}->Filer->hostrec,
	    dstpath => $self->dst_path,
	    %args
	);
    };
    croak $@ if $@;
    return $ret;
}

=head2 ::migrate

Synchronizes data and then transfers NFS filehandles
from the source volume to the destination volume.

=over

=item Arguments

B<throttle> => I<number>

I<Optional.>
Maximum transfer rate allowed, in KB/sec.

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 300 seconds.

=item Examples

=back

     $sm->migrate;

     $sm->migrate(
         throttle => 1000
     );

=cut

sub migrate {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    my $op = "Snapmirror Migrate" . $self->description;
    NOTE::logcomment( $op );

    my $timeout    = $args{'timeout'};
    my $background = $args{'background'};
    $args{'background'} = 1 unless ( ($background == 0) && defined $timeout );

    eval {
	$ret = SnapMirror::snapmirror_migrate(
	    host => $self->{_src}->Filer->hostrec,
	    src  => $self->src_path,
	    dst  => $self->dst_path,
	    %args
	);
	if ( ref $ret eq "Subtest" ) { # Save, if it's an object
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	}
    };
    croak $@ if $@;

    if ( ($background == 0) && (!defined $timeout) ) {
	$self->monitor;
    }
    return $ret;
}

=head2 ::quiesce

Suspends SnapMirror transfers.

=over

=item Arguments

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 300 seconds.

=item Examples

=back

     $sm->quiesce;

     $sm->quiesce(
         timeout => 600
     );

=cut

sub quiesce {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    my $op = "Snapmirror Quiesce" . $self->description;
    NOTE::logcomment( $op );

    my $timeout   = $args{'timeout'};
    my $background = $args{'background'};
    $args{'background'} = 1 unless ( ($background == 0) && defined $timeout );

    eval {
	$ret = SnapMirror::snapmirror_quiesce(
	    host    => $self->{_dst}->Filer->hostrec,
	    dstpath => $self->dst_path,
	    %args
	);

	if ( ref $ret eq "Subtest" ) { # Save, if it's an object
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	}
    };
    croak $@ if $@;

    if ( ($background == 0) && (!defined $timeout) ) {
	$self->monitor;
    }
    return $ret;
}

=head2 ::release

Releases the source-side resources being held
for a particular SnapMirror destination.

=over

=item Arguments

None.

=item Example

=back

     $sm->release;

=cut

sub release {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    NOTE::logcomment("Snapmirror Release" . $self->description);
    eval {
	$ret = SnapMirror::snapmirror_release(
	    host     => $self->{_src}->Filer->hostrec,
	    srcpath  => $self->src_name,
	    dst      => $self->dst_path,
	    %args
	);
    };
    croak $@ if $@;
    return $ret;
}

=head2 ::resume

Resumes quiesced transfers.

=over

=item Arguments

None.

=item Example

=back

     $sm->resume;

=cut

sub resume {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    NOTE::logcomment("Snapmirror Resume" . $self->description);
    eval {
	$ret = SnapMirror::snapmirror_resume(
	    host    => $self->{_dst}->Filer->hostrec,
	    dstpath => $self->dst_path,
	    %args
	);
    };
    croak $@ if $@;
    return $ret;
}

=head2 ::resync

Resynchronize the source and destination.
Returns a 'subtest' object,
which may be ignored if the transfer completes
before the call to C<::resync> returns.

Use of ICS source requires entry in the snapmirror.conf
file.  So when ICS source is used, the snapmirror.conf
file is modified to enable skipping '-S' option from the
command line.

=over

=item Arguments

B<src> => I<Qtree object> | I<volume object>

I<Optional.>
A new source.

B<dst> => I<Qtree object> | I<volume object>

I<Optional.>
A new destination.

Since resync requires a common base snapshot
probably the only valid use of these two options
is switching the source and destination,
i.e. reversing the direction of the transfer.

B<background> => C<0> | C<1>

I<Optional.>
If 1, returns while the transfer is still running in the background.
The result of the transfer must be checked later using
L<C<::wait>|/::wait> or the C<wait> method of the returned object.
If the status is not checked using one of the wait methods
any errors in the transfer will go unnoticed.
The default is 0.

B<snapshot> => I<string>

I<Optional.>
The name of a snapshot on the source to resynchronize from.
This option is only valid for QSM,
it does not work for VSM or RSM.

B<throttle> => I<number>

I<Optional.>
Maximum transfer rate allowed, in KB/sec.

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 300 seconds.

=item Examples

=back

     $sm->resync;

     $sm->resync(
         throttle => 1000
     );

     $subtest = $sm->resync(
         background => 1
     );
     ...
     $subtest->wait;   # Or could use $sm->wait if only one background task

=cut

sub resync {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my ($ret, $op, $old_sm, $old_src, $old_dst);

    my $timeout    = $args{'timeout'};
    $args{'timeout'} = $args{'timeout'} ? $args{'timeout'} : 2*60*60;
    my $background = $args{'background'};
    $args{'background'} = 1 unless ( ($background == 0) && defined $timeout );

    # Save old config entries if using ICS
    my $old_cfg;
    if ( $self->{_ics_src} ) {
	$old_cfg = SnapMirror::Helper::sm_get_config(host=>$self->dst->Filer->hostrec,
					src=>$self->src_path, dst=>$self->dst_path);
	$old_cfg->{sched} = { "sync" => 1 }
		if $self->{'snapmirror.conf.sched'} =~ /sync/i;
    }

    # Cleanup old relationship if new destination is given
    if ( (exists $args{src} && $self->src_path ne $args{src}->hostpath)
      || (exists $args{dst} && $self->dst_path ne $args{dst}->hostpath) ) {
        $old_src = $self->{_src};
        $old_dst = $self->{_dst};
    }

    $self->{_src} = $args{src} if ( $args{src} );
    $self->{_dst} = $args{dst} if ( $args{dst} );
    delete $args{src};
    delete $args{dst};
    if ( $self->{_ics_src} ) {
	# Use config file entry is using ICS
	my %rel = (host=>$self->dst->Filer->hostrec, src=>$self->src_path, dst=>$self->dst_path);
	# First remove config entry on new dst filer if exists
	SnapMirror::Helper::sm_del_config(%rel)
		if SnapMirror::Helper::sm_get_config(%rel);
	# Then duplicate config entries in snapmirror.conf file from old dst
	my %cfg = SnapMirror::Helper::cfg_to_param($old_cfg);
	SnapMirror::Helper::sm_set_config(%rel, %cfg);
	# To read new /etc/snapmirror.conf file
	$self->dst->Filer->snapmirror_enable;
    } else {
	# Specify src if not using ICS
	$args{src} = $self->src_path
    }
    $op = "Snapmirror Resync" . $self->description;
    NOTE::logcomment( $op );
    $ret = SnapMirror::snapmirror_resync(
        host => $self->{_dst}->Filer->hostrec,
        dst  => $self->dst_path,
        %args
    );
    if ( ref $ret eq "Subtest" ) { # Save, if it's an object
        push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
    }

    # Reference old relationship for cleanup
    if ( defined $old_dst ) {
        $old_sm = NOTE::SnapMirror->new( src => $old_src, dst => $old_dst );
        NOTE::reference( $old_sm );
    }

    if ( $background == 0 ) {
	$self->monitor unless defined $timeout;
	$self->dst->exportfs; # Sync dst's exportfs state with filer's if necessary
    }

    return $ret;
}

=head2 ::status

Returns a hash reference to the status of the relationship.
The elements that the hash may contain are described at
L<http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/mainN/web/snapmirror/index.html#snapmirror-status-info>.

Note: The hash reference returned by C<::status>
will not contain the status of the C<snapmirror check> command
if the command is still running.

=over

=item Arguments

B<Filer> => I<filer object>

I<Optional.>
The filer to get the status of the relationship from.
The default is the destination filer.

=item Examples

=back

     $status = $sm->status;   # Get status from destination filer

     $status = $sm->status(
         Filer => $sm->src->Filer   # Get status from source filer
     );

=cut

sub status {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,@args) = @_;
    my $status;
    my $which_one;
    my %args = ();
    if ( @args == 1 ) {
	# This is deprecated:
	$which_one = $args[0];
    } else {
	%args = @args;
    }
    my $filer = $args{Filer} || $self->{_dst}->Filer;
    eval {
	$status = SnapMirror::snapmirror_status(
	    host => $filer->hostrec,
	    src  => $self->src_path,
	    dst  => $self->dst_path,
	);
    };
    croak $@ if $@;
    if ( $which_one ) {
	return $status->{ $which_one };
    }
    return $status;
}

=head2 ::update

Starts an incremental transfer for a SnapMirror relationship.
Returns a 'subtest' object,
which may be ignored if the transfer completes
before the call to C<::update> returns.

If the SnapMirror relationship is synchronous,
the user is responsible for ensuring that jump-ahead
has taken place after C<snapmirror update> has completed.

=over

=item Arguments

B<background> => C<0.0> .. C<1.0>

I<Optional.>
A real number between 0 and 1 (inclusive) that
determines how much of the initial data transfer
should take place in the background.
The units are percent/100;
i.e. 0 means to return after the initial transfer has completed
and 1 means to return immediately.
The result of the transfer must be checked later using
L<C<::wait>|/::wait> or the C<wait> method of the returned object.
If the status is not checked using one of the wait methods
any errors in the transfer will go unnoticed.
The default is 0.

B<create_snapshot> => I<string>

I<Optional.>
The name of a snapshot to create on the destination
after the transfer completes.
Only valid for QSM transfers.

B<snapshot> => I<string>

I<Optional.>
The name of a snapshot on the source to synchronize from.
This option is only valid for QSM,
it does not work for VSM or RSM.

B<throttle> => I<number>

I<Optional.>
Maximum transfer rate allowed, in KB/sec.

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 300 seconds.

B<parallel> => I<1|0>

Boolean flag to indicate whether the filer command will be executed in
'parallel' mode via zapi. If set the rsh command used to obtain data to
calculate throuput will be restricted.

=item Examples

=back

     $sm->update;

     $subtest = $sm->update(
         throttle   => 1000,
         background => 0.25,   # transfer the last 25% in the background
     );
     ...
     $subtest->wait;   # Or $sm->wait to wait for all background tasks

=cut

sub update {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    my $op = "Snapmirror Update" . $self->description;
    NOTE::logcomment( $op );
    my $timeout = $args{'timeout'};

    # If background is between 0 and 1, then it indicates
    # how much in the 'background' the transfer should be.
    my $kilos;
    my $background = (exists $args{background}) ? $args{background} : -1.0;
    if ( ($background < 1.0) && ($background > 0.0) ) {
	# XXX FixME: This needs to be changed to compute the size of the transfer
        # not assume a full transfer will always be made
	$kilos = int( $self->{_src}->used * (1.0 - $background) );
	$args{background} = 1;
    }
    my $start_time = time;
    eval {
	$args{'background'} = 1 unless ( ($background <= 0) && defined $timeout );
	$ret = SnapMirror::snapmirror_update(
	    host => $self->{_dst}->Filer->hostrec,
	    src	=> $self->src_path,
	    dst	=> $self->dst_path,
	    timeout => 0xFFFFFFFF, # Default is infinite timeout
	    %args
	);
	if ( ref $ret eq "Subtest" ) { # Save, if it's an object
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	}
    };
    croak $@ if $@;

    if ( $kilos || (($background <= 0) && !defined $timeout) ) {
	$self->monitor( transfer_size => $kilos );
    }
    if ( $background <= 0 && ! $args{parallel} ) { # Skip rsh command as zapi only in parallel run
        my $transfer_secs = time - $start_time;
	my $last_transfer_size = $self->{'last_transfer_size'};
        NOTE::logcomment( "Snapmirror Update transferred %.3f MB in %d seconds; throughput was %.3f MB/s",
		$last_transfer_size / 1000,
	        $transfer_secs,
		($last_transfer_size / 1000) / $transfer_secs
	);
    }
    return $ret;
}

=head2 B<Checker/Fixer>

=head2 ::check, ::fix

Checks the consistency of the SnapMirror relationship.
C<::fix> will also repair any inconsistencies.

The destination must exist and be in a
'snapmirrored' or 'quiesced' state.
The checker/fixer process runs on the destination filer.

Prior to the execution of the command,
this function will check that a jump-ahead is not taking place.
This check will always take place
regardless of the type of the relationship,
i.e. Qtree or volume, asynchronous or synchronous.

Returns a 'subtest' object if background => 1.

=over

=item Arguments

B<background> => C<0> | C<1>

I<Optional.>
If 1, returns while checking is still running in the background.
The result of the check must be verify later using
L<C<::wait>|/::wait> or the C<wait> method of the returned object.
If the status is not checked using one of the wait methods
any errors in the check will go unnoticed.
The default is 0.

B<block_based> => C<0> | C<1>

If 1, then the block based checker/fixer is used.
Only valid for VSM and RSM.
The default is 0.

B<throttle> => I<number>

I<Optional.>
Maximum transfer rate allowed, in KB/sec.

B<check_level> => C<data> | C<checksum>

I<Optional.> I<::check> only.
Specify data comparison level as either data or checksum. 
The default is C<data>.

B<quick_mode> => C<0> | C<1>

I<Optional.> I<::check> only.
If 1, Starts C<snapmirror check> in quick mode.
The default is 0.

B<snapshot> => I<string>

I<Optional.> I<::check> only.
The name of a snapshot on the source to use.
By default the base snapshot for a Qtree-based mirror
and all common snapshots for a volume-based mirror will be used.

B<skip_restrict> => C<0> | C<1>

I<Optional.> I<::fix> only.
Normally, C<::fix> will restrict the destination volume during the
operation. This option allows that behavior to be overridden.

=item Examples

=back

     $sm->check;

     $subtest = $sm->check(
         background => 1
     );

     ...
     $subtest->wait_finish;   # Or $sm->wait to wait for all background tasks

     $sm->fix;

=cut

sub fix {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    my $op = "Snapmirror Fix" . $self->description;
    NOTE::logcomment( $op );
    my $have_sio = $self->dst->sio_proc;
    if ( $have_sio ) {
	NOTE::logcomment("Stopping dst SIO before 'sm fix' as dst is restricted for fix");
	$self->dst->sio(stop=>1);
    }
    eval {
	$ret = SnapMirror::snapmirror_fix(
	    host => $self->{_dst}->Filer->hostrec,
	    dst	=> $self->dst_path,
	    %args
	);
	if ( ref $ret eq "Subtest" ) { # Save, if it's an object
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	}
    };
    croak $@ if $@;
    $self->_burt223440_work_around(%args)
	if $self->dst->mounted;
    if ( $have_sio && ! $args{background} ) {
	NOTE::logcomment("Restarting dst SIO after 'sm fix'");
	$self->dst->sio(restart=>1);
    }
    return $ret;
}

# Because the filer view has been altered and the client doesn't know
# about it yet, we have to remount the destination.
# Also restart SIO traffic if running. For any other kind of traffic this
# work-around will not work
# See burt 223440
# This routine should only be called after running 'sm fix'

sub _burt223440_work_around {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self, %args) = @_;

    if ( $args{background} ) {
	# XXX Warn of work-around for burt223440
	NOTE::logcomment("Remount dst vol ".$self->dst->name." when snapmirror fixer completes");
    } else {
	NOTE::logcomment("Remount dst ".$self->dst->description.", work-around for BURT223440");
	$self->dst->umount;
	$self->dst->mount;
	NOTE::logcomment("End of Work-around for BURT223440");
    }
}

sub check {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self, %args) = @_;
    my $ret;

    my $op = "Snapmirror Check" . $self->description;
    NOTE::logcomment( $op );
    my $have_sio = $self->dst->sio_proc;
    if ( $args{fix} && $have_sio ) {
	NOTE::logcomment("Stopping dst SIO before 'sm fix' as dst is restricted for fix");
	$self->dst->sio(stop=>1);
    }
    unless ($args{timeout}){
        $args{timeout}=1200;
    }
    eval {
	$ret = SnapMirror::snapmirror_check(
	    host => $self->{_dst}->Filer->hostrec,
	    dst	=> $self->dst_path,
	    %args
	);
	if ( ref $ret eq "Subtest" ) { # Save, if it's an object
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	}
    };
    croak $@ if $@;
    if ( $args{fix} ) {
	$self->_burt223440_work_around(%args) if $self->dst->mounted;
	if ( $have_sio && ! $args{background} ) {
		NOTE::logcomment("Restarting dst SIO after 'sm fix'");
		$self->dst->sio(restart=>1);
	}
    }
    return $ret;
}

=head2 ::check_abort

Aborts a currently running snapmirror check process.
Does not abort currently running SnapMirror transfers.

=over

=item Arguments

None.

=item Example

=back

     $sm->check_abort;
     $sm->wait;   # Clear pending background task status

=cut

sub check_abort {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    NOTE::logcomment("Snapmirror Check Abort" . $self->description);
    eval {
	$ret = SnapMirror::snapmirror_check_abort(
	    host    => $self->{_dst}->Filer->hostrec,
	    dstpath => $self->dst_path,
	    %args
	);
    };
    croak $@ if $@;
    return $ret;
}

=head2 ::compare

Helper function that will call 'snapmirror check' command if both destination
and source filers are running Tsingtao and above. Otherwise, it will call 
'treecompare' command if both destination and source filers are each running
release between Scrimshaw and Anchorsteam.

This function will perform the following actions:

=over

1) Determines the version of both source and destination filer and type of
relationship to decide whether to use either 'snapmirror check' or 
'treecompare'.

2) If 'snapmirror check' is used, it will do data comparison. Using 
'snapmirror check' default, base snapshot for a Qtree-based mirror and all 
common snapshots for a volume-based mirror will be used for comparison.

3) If 'treecompare' is used and ...

3.1) If relationship is QSM, the base snapshots on both source and destination
filers are used. Comparison will ignore ctime and mtime mismatches.

3.2) If relattionship is VSM/RSM, the base snapshot on the destination filer 
is used. Comparison will ignore vdisk metadata/times mismatches.

On failure or any data mismatches, dies with an error string or the data
mismatches stored in C<$@> respectively.

=back

=over

=item Arguments

B<throttle> => I<number>

I<Optional.>
Maximum transfer rate allowed, in KB/sec.

B<treecompare> => C<0> | C<1>

I<Optional.>
If 1, forces the use of the C<treecompare> command
regardless of the version of either the source or destination filer.
The default is 0.

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 300 seconds.

=item Examples

=back

     $sm->compare;

     $sm->compare(
         throttle => 1000
     );

=cut

sub compare {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;

    $args{'timeout'} = $args{'timeout'} ? $args{'timeout'} : 7200;

    NOTE::logcomment("Snapmirror Compare" . $self->description);
    eval {
	SnapMirror::Helper::sm_compare_src_dst(
	    dsthost => $self->{_dst}->Filer->hostrec,
	    srchost => $self->{_src}->Filer->hostrec,
	    src	=> $self->src_path,
	    dst	=> $self->dst_path,
	    %args
	);
    };
    croak $@ if $@;
}

=head2 ::wreck

=head2 ::unwreck

A front-end to ::wreck_dstqt. Doesn't require any arguments.
C<::wreck> makes the destination of a QSM relationship
writeable so it can be corrupted. C<::unwreck> restores
the destination to read-only.

=over

=item Examples

=back

     $sm->wreck;
     $sm->dst->nfs->attach("/home/somefile")->mv( ".." );
     $sm->unwreck;
     $sm->update;
     $sm->fix; # Repair relationship
     $sm->compare;

=cut

sub wreck {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;

    croak "Already wrecked"
	if ( defined $self->{_wreck_snapid} );
    croak "Wreck currently only supported for qtrees"
	unless ( $self->dst->isa( "NOTE::Qtree" ) );

#    $self->{_wreck_src_snapshot} = $self->snapshot_create;
    $self->{_wreck_dummy} = NOTE::SnapMirror->initialize(
	src => $self->src,
	dst => NOTE::Qtree->new(
	    Volume => $self->dst->Volume,
	    tag    => "Dummy Qtree relationship (required for wreck)",
	),
    );
    $self->{_wreck_snapid} = $self->wreck_dstqt( snapid => 0 );
    $self->dst->exportfs( "anon=0,rw" );
}

sub unwreck {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;

    croak "Not wrecked"
	unless ( defined $self->{_wreck_snapid} );

    $self->wreck_dstqt( snapid => $self->{_wreck_snapid} );

    #$self->update( snapshot => $self->{_wreck_snapid} );  # Empty update required to complete corruption
    $self->update;  # Empty update required to complete corruption

    delete $self->{_wreck_snapid};
    $self->{_wreck_dummy}->destroy;
    delete $self->{_wreck_dummy};
}

=head2 ::wreck_dstqt

N.B. Do not use this function unless you know
what the C<wreck> command does.


Uses the C<wreck> command to make the destination Qtree
read/write or read-only based on the value of snapid.

If snapid is 0, the function first finds
out the snapshot id of the base snapshot
and then calls C<wreck> on the volume so as
to make the destination Qtree writeable.
Upon successful completion it returns the snapshot id.

If snapid is any other number other than 0,
then the C<wreck> command is called with that snapshot id
to make the destination Qtree read-only once again.
In this case the function returns
the same snapshot id that was passed to it.

For more information,
please see L<http://web.netapp.com/~arur/wreck_usage.htm>.

=over

=item Arguments

B<snapid> => I<number>

I<Required.>
ID of the target snapshot.

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 300 seconds.

=item Examples

=back

     $snapid = $sm->wreck_dstqt(
         snapid => 0
     );

=cut

sub wreck_dstqt {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $ret;
    my $op = "Snapmirror Wreck Volume Containing Destination Qtree" .
             $self->description;
    NOTE::logcomment( $op );

    eval {
        $ret = SnapMirror::Helper::sm_wreck_dstqt(
	    host => $self->{_dst}->Filer->hostrec,
	    qtsrc => $self->src_path,
	    qtdst => $self->dst_path,
	    %args
	);
    };
    croak "$@" if $@;
    return $ret;
}

=head2 ::compression
=item Arguments

B<enable> => I<number>


1|0. If nothing is given and method is called it sets compression=enable in the conf file.
Used to set the compression option as enable/disable in snapmirror.conf file for an existing relationship.

=over



=item Examples

=back

 $SnapMirror1->compression(enable=>1);
 $SnapMirror1->compression(enable=>0);
 $SnapMirror1->compression();

=cut

sub compression {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $conn= $self->dst->Filer->conn;
    my $dst=$self->dst_path;
    my $reg_key = "options.files.snapmirror.conf.".$dst;
    my $enable_string=$args{enable} ? 'enable':'disable' ;
    $enable_string='enable' unless{exists $args{enable}};
    my $reg_args_str="compression=".$enable_string ;
    my $get_string=registryGet($conn, "$reg_key.args") ;
    croak $@ if(($@) && ($@!~/does not exist/));
    unless($@){
    my @get_string= split(/args\=/, $get_string);
    $get_string[0]=~s/compression\=(en|dis)able//;
    $get_string[0]=~s/\-//;
    $reg_args_str=$get_string[0].",$reg_args_str" ;
    $reg_args_str=~s/\,\,/\,/;
    $reg_args_str=~s/^\,//;
    }
    registrySet($conn, "$reg_key.args",$reg_args_str) or croak $@;
}
=head2 B<Waiting>

=head2 ::wait

Waits for all background processes to complete.
These are the processes created
by using the B<background> option of
L<C<::initialize>|/::initialize>,
L<C<resync>|/::resync>,
L<C<::update>|/::update>,
::check.

This method is completely unrelated to L<C<::status_wait>|/::status_wait>.

=over

=item Arguments

None.

=item Example

=back

     $sm->wait;

=cut

sub wait {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my $obj;
    my $test;
    my $op;

    while ( $test = pop @{$self->{_subtests}} ) {
	$op  = $test->{operation};
	$obj = $test->{subtest};
	if ( $obj->is_alive() ) {
	    NOTE::logcomment( "Waiting for: ".$op );
	}
	$obj->wait_finish();
	# fix fix fix
	# the following will be made public really soon.
	SnapMirror::Helper::_sm_extract_subtest_err($obj);
    }
    return 0;
}

=head2 ::wait_sync

Waits for the snapmirror relationship to enter sync or CP sync mode. The
timer does not start if the mirror is in the last async transfer right
before entering sync mode.

This method is similar to L<C<::status_wait>|/::status_wait> but does 
additional checks to ensure that the lag time is 0 secs for NVLog sync
and >= 10 secs for CP sync.

=over

=item Arguments

B<cp_sync> => C<0> | C<1>

I<Optional.>
Boolean value to determine if the method should be checking
if the mirror goes into full sync (default) or CP sync. Defaults to 0.

B<timeout> => I<number>

I<Optional.>
Timeout value to wait for the mirror to go into either full
sync or CP sync. Defaults to 600 (10 mins).

=item Examples

=back

     $sm->wait_sync;

     $sm->wait_sync(
         cp_sync => 1,
     );

=cut

sub wait_sync {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self, %args) = @_;

    if ($self->{_mode} eq 'semi-sync'){
        $args{cp_sync} = 1;
    }
    eval {
	SnapMirror::Helper::sm_wait_sync(
	    host => $self->dst->Filer->hostrec,
	    src  => $self->src_path,
	    dst  => $self->dst_path,
	    %args,
	);
    };
    croak $@ if $@;
}


=head2 ::status_wait

Wait for the SnapMirror relationship to reach specific states.
Returns when all of the specified status elements
have their respective values I<at the same time>.
(Hint: don't use element/value combinations
that are mutually exclusive.)

Note that there are many SnapMirror status elements,
one of which is thoughtfully named 'status'.
Another one is named 'state'.
There are none however named 'element' (not yet, anyway).

The status elements checked are the same ones
that may appear in the hash generated by L<C<::status>|/::status>.
(Internally C<::status> is used;
this is a convenience method.)
For a description of the elements,
including their possible values, see
L<http://web.netapp.com/engineering/design-depot/appliance-mgmt/zephyr/zapidoc.cache/mainN/web/snapmirror/index.html#snapmirror-status-info>.

Because you are a valued NOTE user,
the values for the status elements may be given
as regular expressions.

This method is completely unrelated to L<C<::wait>|/::wait>.

=over

=item Arguments

I<status element name> => I<status element value> | I<regex>

I<Required.>
The status elements to check are given as named arguments.
Their values may be given as literal strings
or regular expressions that (hopefully) match valid values.

B<ignore_transfer_error> => C<0> | C<1>

I<Optional.>
If 1, don't fail if the status contains
the C<current-transfer-error> element.
The default is 0.

B<sleep> => I<number>

I<Optional.>
Number of seconds to sleep in between polling the filer's status.
The default is 5 seconds.

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 600 seconds.

=item Examples

=back

     $sm->status_wait(
         status => 'in-sync'
     );

     $sm->status_wait(
         status => 'transferring'
     );

     $sm->status_wait(
         base-snapshot => '\w'
     );

=cut

# visit prod/common/adminapi/zapi_snapmirror.c for all available keys ( '@element' in .c)
sub status_wait {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,%args) = @_;
    my $timeout = 600;	# timeout before croak
    my $sleepis = 5;	# sleep number of seconds between status read
    my $noerror = 0;	# croak by default if 'current-transfer-error'

    if( exists $args{'timeout'} ) {
	$timeout = $args{'timeout'};
	delete $args{'timeout'};
    }

    if( exists $args{'sleep'} ) {
	$sleepis = $args{'sleep'};
	delete $args{'sleep'};
    }

    if( exists $args{'ignore_transfer_error'} ) {
	$noerror++;
	delete $args{'ignore_transfer_error'};
    }

    if ( $args{'status'} =~ /in-sync/i && ! $args{'lag-time'} ) {
        $args{'lag-time'} = '^[-0]$';
    }

    my $matchall = 0;
    my $havekeys = scalar( keys %args );

    # Set expiration time
    my $etime = time + $timeout;
    my $wait_for = '';

    while( $matchall < $havekeys ) { 

        croak "Status Timeout waiting for:\n$wait_for" if time > $etime;

	my $status = $self->status;
	$matchall = 0;

	if( $status->{'current-transfer-error'} && $noerror < 1 ) {
	    croak $status->{'current-transfer-error'};
	}

	# check keys one by one and match all of them
	$wait_for = '';
	foreach( keys %args ) {
	    $matchall++ if( $status->{$_} =~ qr/$args{$_}/i );
	    $wait_for .= "$_ =~ /$args{$_}/ (currently '".$status->{$_}."')\n";
	}
	chomp( $wait_for );
	NOTE::logcomment("Waiting for status (time left: %d): ".$self->description."\n$wait_for", $etime-time);

        # Do not wait anymore if everything has been found
        last unless ( $matchall < $havekeys );

        # Wait for next iteration
        my $delta = $etime - time;
        $delta = 0 if $delta < 0;
        $sleepis = $delta if $delta < $sleepis;
	sleep $sleepis if( $sleepis > 0 );
    }

    return;
}

=head2 B<Component Objects>

=head2 ::src

Returns the object representing the current source.

=over

=item Arguments

I<volume object> | I<Qtree object>

I<Optional.>
If present, replaces the current source object.

=item Examples

=back

     $src = $sm->src;

=cut

sub src {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,$newsrc) = @_;
    if ( defined $newsrc ) {
	my $oldsrc = $self->{_src};
	$self->{_src} = $newsrc;
	return $oldsrc;
    } else {
	return $self->{_src};
    }
}

=head2 ::dst

Returns the object representing the current destination.

=over

=item Arguments

I<volume object> | I<Qtree object>

I<Optional.>
If present, replaces the current destination object.

=item Examples

=back

     $dst = $sm->dst;

=cut

sub dst {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my ($self,$newdst) = @_;
    if ( defined $newdst ) {
	my $olddst = $self->{_dst};
	$self->{_dst} = $newdst;
	return $olddst;
    } else {
	return $self->{_dst};
    }
}
=head2 B<Names and Paths>

=head2 ::src_name

Returns the name of the source volume or Qtree.

=over

=item Arguments

None.

=item Example

=back

     $srcname = $sm->src_name;

=cut

sub src_name {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    if ( $self->{_src}->isa( "NOTE::Qtree" ) ) {
	return $self->{_src}->mount_path;
    } else {
	return $self->{_src}->name;
    }
}

=head2 ::dst_name

Returns the name of the destination volume or Qtree.

=over

=item Arguments

None.

=item Example

=back

     $srcname = $sm->dst_name;

=cut

sub dst_name {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    if ( $self->{_src}->isa( "NOTE::Qtree" ) ) {
	return $self->{_dst}->mount_path;
    } else {
	return $self->{_dst}->name;
    }
}

=head2 ::src_path

Returns the host-qualified path name
of the source volume or Qtree.

=over

=item Arguments

None.

=item Example

=back

     $srcpath = $sm->src_path;

=cut

sub src_path {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my $src_host = $self->{_ics_src} || $self->{_src}->Filer->host;
    return $src_host.":".$self->src_name;
}

=head2 ::dst_path

Returns the host-qualified path name
of the destination volume or Qtree.

=over

=item Arguments

None.

=item Example

=back

     $dstpath = $sm->dst_path;

=cut

sub dst_path {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    return $self->{_dst}->Filer->host.":".$self->dst_name;
}

=head2 B<Object Identification>

=head2 ::description

Returns a description of the SnapMirror relationship
including a tag for the source
and a tag for the destination if set.

=over

=item Arguments

None.

=item Example

=back

     $desc = $sm->description;

=cut

sub description {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    my $desc = "";

    $desc .= "\n    +---";
    $desc .= $self->src_path;
    $desc .= " (".$self->src->tag.") " if ( $self->src->tag );
    $desc .= "\n    +-> ";
    $desc .= $self->dst_path;
    $desc .= " (".$self->dst->tag.") " if ( $self->dst->tag );
    return $desc;
}

=head2 ::SPARE

Prevent SnapMirror relationship from being torn down
when script exits..

=over

=item Argument

None.

=item Example

=back

     $sm->SPARE;

=cut

sub SPARE {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics
 shift->{_spare_me} = 1 }

=head2 ::destroy

Tear down a SnapMirror relationship.

=over

=item Argument

None.

=item Example

=back

     $sm->destroy;

=cut

sub destroy {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;
    $self->{_destroyed} = 1;
    NOTE::logcomment(
	"Destroying SnapMirror " . $self->description
    );
    eval {
	SnapMirror::Helper::sm_cleanup(
	    srchost => $self->{_src}->Filer->hostrec,
	    dsthost => $self->{_dst}->Filer->hostrec,
	    src	=> $self->src_path,
	    dst	=> $self->dst_path,
	);
    };
    croak $@ if $@;
    $self->_clr_sm_conn; # Remove sm conn entry from registry;
    NOTE::logcomment("SnapMirror relationship destroyed");
    return $self;
}

sub DESTROY {
    my $trace = $NOTE::TRACELOG->start_timer();  # log internal metrics

    my $self = shift;

    return if ( $$ != $self->{_owner} ); # Only the creator can destroy an object
    return if ( $self->{_spare_me} );	 # Means don't destroy
    return if ( $self->{_destroyed} );	 # Already dead
    return if ( NOTE->ALL_SPARED );
    $self->destroy;
}


=head1 STATUS

Public.

$Date: 2013/01/22 $

=head1 AUTHOR

Author information is in the
L<NOTE overview|engineering::ontap_tests::agnostic::docs::lib-NOTE-pm/::author>.

=cut

1;
