#
# $Id: //depot/prod/test/Rsierranevada/lib/NOTE/Volume.pm#1 $
#
## @summary  NOTE Volume class
## @status   Public
## @pod      here
## @author   dl-dpgqa-automation@netapp.com

=head1 NAME

NOTE::Volume

=head1 DESCRIPTION

This is the NOTE class for working with volumes.

=head1 METHODS

=cut

package NOTE::Volume;
our @ISA = qw(NOTE);

use strict;
use Carp;
use Params::Validate qw( validate validation_options SCALAR BOOLEAN HASHREF );
use wafl_utils qw( df );
use NOTE::Filer;
use NOTE::Aggregate;
use NOTE::Volume::Clone;
use NOTE::FileSystem::NFS;
use NOTE::Sis;
use NOTE::FileSystem::NFS::File;
use NOTE::Registry::Slice;
use File::Basename;
use POSIX qw();

# NOTE: Do *NOT* import any external functions into this module!
# This is to avoid namespace collisions namely prototype mismatches and
# all that. Call the external function directly by prepending the package 
# name.
use Tharn qw();
use Volume qw();
use Volume::Clone qw();
use Volume::Copy qw();
use Aggregate qw();
use Aggregate::ImplTemplate::RaidInfo_new qw();
use ExportFS qw();
use Wafliron qw();
use disk qw();
use SnapCmds qw();
use SnapMirror qw();
use GenericMount qw();
use fileSystem qw();
use Utils::Units qw();
use VFILER::Util qw();
use Dense qw();
use Dense::Utils qw();
use RichData qw();
use cluster::VolUtils qw( vol_mirror );	# syncmirror support
use Data::Dumper;
use SayWrap qw(sayEOF);
use filer;

my $regSlice = {
	"options.convert_ucode" 	      => sub { "options.vols.".shift->name.".convert_ucode" },
	"options.create_ucode"  	      => sub { "options.vols.".shift->name.".convert_ucode" },
	"options.lang"	    	      => sub { "options.vols.".shift->name.".lang" },
	"options.maxdirsize"    	      => sub { "options.vols.".shift->name.".maxdirsize" },
	"options.maxfiles"	    	      => sub { "options.vols.".shift->name.".maxfiles" },
	"options.minra"	    	      => sub { "options.vols.".shift->name.".minra" },
	"options.no_i2p"	    	      => sub { "options.vols.".shift->name.".no_i2p" },
	"options.nosnap"	    	      => sub { "options.vols.".shift->name.".nosnap" },
	"options.nosnapdir"	    	      => sub { "options.vols.".shift->name.".nosnapdir" },
	"options.nvfail"	    	      => sub { "options.vols.".shift->name.".nvfail" },
	"options.quotas"	    	      => sub { "options.vols.".shift->name.".quotas" },
	"options.raidsize"	    	      => sub { "options.vols.".shift->name.".raidsize" },
	"options.resyncsnaptime"	      => sub { "options.vols.".shift->name.".resyncsnaptime" },
	"options.no_atime_update"	      => sub { "options.vols.".shift->name.".no_atime_update" },
	"options.snaplock_default_period" => {
		get => sub { shift->options( 'snaplock_default_period' ) },
	    set => sub { shift->options( 'snaplock_default_period', shift ) },
	},
	"options.snaplock_maximum_period" => {
		get => sub { shift->options( 'snaplock_maximum_period' ) },
	    set => sub { shift->options( 'snaplock_maximum_period', shift ) },
	},
	"options.snaplock_minimum_period" => {
		get => sub { shift->options( 'snaplock_minimum_period' ) },
	    set => sub { shift->options( 'snaplock_minimum_period', shift ) },
	},
	"options.snapmirrored"            => sub { "options.vols.".shift->name.".snapmirrored" },
	"status.checksum.enabled"	      => sub { "status.vols.".shift->name.".checksum.enabled" },
	"status.checksum.style"	      => sub { "status.vols.".shift->name.".checksum.style" },
	"status.diskcount"	      	      => sub { "status.vols.".shift->name.".diskcount" },
	"status.filestotal"	      	      => sub { "status.vols.".shift->name.".filestotal" },
	"status.filesused"	      	      => sub { "status.vols.".shift->name.".filesused" },
	"status.inconsistent"	      => sub { "status.vols.".shift->name.".inconsistent" },
	"status.mirror_status"	      => sub { "status.vols.".shift->name.".mirror_status" },
	"status.name"	      	      => sub { "status.vols.".shift->name.".name" },
	"status.plex_count"	      	      => sub { "status.vols.".shift->name.".plex_count" },
	"status.quotainit"	      	      => sub { "status.vols.".shift->name.".quotainit" },
	"status.raidsize"	      	      => sub { "status.vols.".shift->name.".raidsize" },
	"status.raidstatus"	      	      => sub { "status.vols.".shift->name.".raidstatus" },
	"status.sizeavail.bytes"	      => sub { "status.vols.".shift->name.".sizeavail.bytes" },
	"status.sizetotal.bytes"	      => sub { "status.vols.".shift->name.".sizetotal.bytes" },
	"status.sizeused.bytes"	      => sub { "status.vols.".shift->name.".sizeused.bytes" },
	"status.sizeused.percent"	      => sub { "status.vols.".shift->name.".sizeused.percent" },
	"status.snaplock_compliance"      => sub { "status.vols.".shift->name.".snaplock_compliance" },
	"status.snaplock_enterprise"      => sub { "status.vols.".shift->name.".snaplock_enterprise" },
	"status.space_reservations"       => sub { "status.vols.".shift->name.".space_reservations" },
	"status.state"      	      => sub { "status.vols.".shift->name.".state" },
	"state.snapmirror.restart"	      => sub { "state.snapmirror.restart.vol.data.".shift->name },
	"snapmirror.contents"	      => sub { "status.snapmirror.active.".shift->fullname.".contents" },
	"snapmirror.current_size"	      => sub { "status.snapmirror.active.".shift->fullname.".current_size" },
	"snapmirror.current_transfer_error" =>
		sub { "status.snapmirror.active.".shift->fullname.".current_transfer_error" },
	"snapmirror.current_transfer_type" =>
		sub { "status.snapmirror.active.".shift->fullname.".current_transfer_type" },
	"snapmirror.lag" =>
		sub { "status.snapmirror.active.".shift->fullname.".lag" },
	"snapmirror.last_transfer_duration" =>
		sub { "status.snapmirror.active.".shift->fullname.".last_transfer_duration" },
	"snapmirror.last_transfer_from" =>
		sub { "status.snapmirror.active.".shift->fullname.".last_transfer_from" },
	"snapmirror.last_transfer_size" =>
		sub { "status.snapmirror.active.".shift->fullname.".last_transfer_size" },
	"snapmirror.last_transfer_type" =>
		sub { "status.snapmirror.active.".shift->fullname.".last_transfer_type" },
	"snapmirror.snapshot" =>
		sub { "status.snapmirror.active.".shift->fullname.".snapshot" },
	"snapmirror.source"	=>
		sub { "status.snapmirror.active.".shift->fullname.".source" },
	"snapmirror.state" =>
		sub { "status.snapmirror.active.".shift->fullname.".state" },
	"snapmirror.status"	=>
		sub { "status.snapmirror.active.".shift->fullname.".status" },
	"snapmirror.timestamp" =>
		sub { "status.snapmirror.active.".shift->fullname.".timestamp" },
};


sub _asis_shutdown {
	my $self = shift;
	 
	my $hostname = $self->{_Filer}->host;
	my $pathname = $self->mount_path;

	# Display statistics of all dense volumes
	Dense::dense_stat(host=>$hostname, path=>$pathname);
	croak $@ if $@;

	# Turn A-SIS off and undo de-duplication on the volume
	Dense::dense_off_undo(host=>$hostname, path_name=>$pathname);
	croak $@ if $@;
	Dense::dense_wait_for_status(host=>$hostname, path_name=>$pathname, status=>"NOSTATUS");
	croak $@ if $@;

	# Gather df -s
	$pathname =~ s/\/vol\///; # trim it down only to the volume name
	my ($result, %savings) = Dense::Utils::get_vol_savings(host=>$hostname, vol_name=>$pathname);
	croak $@ if $@;
	return 1;
}

sub _spareVols {
	my ($filer) = @_;
	my @spareVols = $filer->Volumes( spare_only => 1 );
	return sort { $a->size <=> $b->size } @spareVols;
}

# Recycling volumes
# Requirements:
#
# - When volumes with datasets are spared, they revert to their
# base snapshot and original size, unless otherwise specified with
# the 'age => 1' flag.
# - If aggregate is not specified, any spare volume with the appropriate
# dataset and size is recycled.
# - If aggregate object is specifed, recycled volume is chosen or
# created in that aggregate.
# - If aggregate spec is specified and aggregate is recycleable,
# a recycleable aggregate is created or chosen from any appropriate
# existing spare aggregates. The recycleable volume is then created
# or chosen from any existing spare volumes in the aggregate and
# spare volumes in the aggregate are deleted until the desired size
# volume is achieved.
#
# Choose the closest fitting free volume.
# Match against type, size and dataset.
# Match against aggregate specification, if any
#

# This internal function is used to log the reason that
# a resource was rejected as a recyclable candidate
sub _reject {
	my %args = @_;
	my $msg = "Rejecting ".$args{name}.": ".$args{reason};

	if ( $args{wanted} ) {
		$msg .= "\n    ";
		$msg .= $args{wanted};
	}
	if ( $args{got} ) {
		$msg .= "\n    ";
		$msg .= $args{got};
	}
	NOTE::logcomment( $msg );
}

sub _recycle {
	my ($self, %args) = @_;
	my $type = $self->{_type};
	my ($vol, $aggr);
	my $aggrQualities;
	my @volList = ();

	NOTE::logcomment( "Looking for recyclable volume" );
	if ( $args{Aggregate} ) {
		if ( ref $args{Aggregate} eq "HASH" ) {
		    if ( defined($args{Aggregate}->{name}) ) {
			# Resolve this now, because it might end up being resolved to undef
			$args{Aggregate}->{name} = NOTE::option( $args{Aggregate}->{name} );
		    }
		    if ( defined($args{Aggregate}->{name}) ) {
			# named aggregate
			# Only look for volumes within this aggregate
			eval {
			    $aggr = NOTE::Aggregate->attach(
				Filer => $self->{_Filer}, 
				%{$args{Aggregate}},
			    );
			};
			return 0 if ( $@ );
			@volList = $aggr->Volumes( spare_only => 1 );
		    } else {
			# anonymous aggregate
			# Here, I just want to note the characteristics
			# of the aggregate so I can match candidates
			# using them.
			$aggrQualities = $args{Aggregate};
			@volList = $self->{_Filer}->Volumes( spare_only => 1 );
		    }
		} elsif ( ref $args{Aggregate} eq "NOTE::Aggregate" ) {
		    @volList = $args{Aggregate}->Volumes( spare_only => 1 );
		}
	} else {
		@volList = $self->{_Filer}->Volumes( spare_only => 1 );
	}

	for $vol ( @volList ) { $vol->size } # Build 'size' cache
	@volList = sort { $a->{_size} <=> $b->{_size} } @volList;

	for $vol ( @volList ) {
	    eval { # If any error occurs, simply try the next candidate
		    next unless ( $vol->is_free );
		    unless ( $vol->type eq $type ) {
			_reject(
			    name   => $vol->name,
			    reason => "Volume types don't match",
			    wanted => "Requested type is [".$type."]",
			    got    => "Candidate type is [".$vol->type."]",
			);
			next;
		    }
		    if ( $type eq 'flex' ) {
			if ( defined $args{size} ) {
			    unless ($vol->{_size} == NOTE::units( $args{size} ) ) {
				_reject(
				    name   => $vol->name,
				    reason => "Volume sizes don't match",
				    wanted => "Requested size is [".NOTE::units($args{size})."k]",
				    got    => "Candidate size is [".$vol->{_size}."k]",
				);
			        # Since volume list is sorted from smallest to largest,
				# we can stop looking if a candidate is larger than the
				# requested size..
				last if ( $vol->{_size} > NOTE::units($args{size}) );
				next; # Just move on to the next candidate
			    }
			}
			# If flex reject volumes that aren't in aggregates
			# with characteristics that match specified disks,
			# raid type, etc..
			if ( $aggrQualities ) {
			    $aggr = $vol->Aggregate;
			    # Make sure we get a volume in a shareable aggregate if we say we want one
			    # or a volume in an unshareable aggregate if we say we don't want one.
			    if ( defined $aggrQualities->{share} ) {
				unless ( $aggrQualities->{share} eq $aggr->is_shareable ) {
				    _reject(
					name   => $vol->name,
					reason => "containing aggregate [".$aggr->name."] qualities don't match",
					wanted => "requested {share} [".$aggrQualities->{share}."]",
					got    => "candidate {share} [".$aggr->is_shareable."]",
				    );
				    next;
				}
			    }
			    # If we want a snaplock aggregate, make sure they're the same type
			    if ( $aggrQualities->{snaplock} ) {
				unless ( $aggrQualities->{snaplock} eq $aggr->snaplock ) {
				    _reject(
					name   => $vol->name,
					reason => "containing aggregate snaplock qualities don't match",
					wanted => "requested {snaplock} [".$aggrQualities->{snaplock}."]",
					got    => "candidate {snaplock} [".$aggr->snaplock."]",
				    );
				    next;
				}
			    } else {
				# If we don't want one, reject any that are.
				if ( $aggr->snaplock ) {
				    _reject(
					name   => $vol->name,
					reason => "containing aggregate snaplock qualities don't match",
					wanted => "requested no snaplock ",
					got    => "candidate {snaplock} [".$aggr->snaplock."]",
				    );
				    next;
				}
			    }
			    if ( $aggrQualities->{disks} ) {
				unless ( $aggrQualities->{disks} <= scalar($aggr->disks) ) {
				    _reject(
					name   => $vol->name,
					reason => "containing aggregate has too few disks",
					wanted => "requested disks [".$aggrQualities->{disks}."]",
					got    => "candidate disks [".$aggr->disks."]",
				    );
				    next;
				}
			    }
			    if ( $aggrQualities->{raidtype} ) {
				unless ( $aggrQualities->{raidtype} eq $aggr->options->{raidtype} ) {
				    _reject(
					name   => $vol->name,
					reason => "containing aggregate raidtype doesn't match",
					wanted => "requested raidtype [".$aggrQualities->{raidtype}."]",
					got    => "candidate raidtype [".$aggr->options->{raidtype}."]",
				    );
				    next;
				}
			    }
			    if ( $aggrQualities->{raidsize} ) {
				unless ( $aggrQualities->{raidsize} == $aggr->options->{rgsize} ) {
				    _reject(
					name   => $vol->name,
					reason => "containing aggregate raidsize doesn't match",
					wanted => "requested raidsize [".$aggrQualities->{raidsize}."]",
					got    => "candidate raidsize [".$aggr->options->{raidsize}."]",
				    );
				    next;
				}
			    }
			}
		    } else { # Trad vol:
			if ( defined $args{disks} ) {
			    unless (scalar($vol->disks) == $args{disks}) {
				_reject(
				    name   => $vol->name,
				    reason => "number of disks don't match",
				    wanted => "requested disks [".$args{disks}."]",
				    got    => "candidate disks [".$vol->disks."]",
				);
				next;
			    }
			}
			if ( defined $args{snaplock} ) {
			    unless ( $vol->snaplock eq $args{snaplock} ) {
				_reject(
				    name   => $vol->name,
				    reason => "volume snaplock type doesn't match",
				    wanted => "requested snaplock [".$args{snaplock}."]",
				    got    => "candidate snaplock [".$vol->snaplock."]",
				);
				next;
			    }
			} else {
			    # If we don't want a snaplock volume, reject any that are.
			    if ( $vol->snaplock ) {
				_reject(
				    name   => $vol->name,
				    reason => "volume snaplock qualities don't match",
				    wanted => "requested no snaplock ",
				    got    => "candidate {snaplock} [".$vol->snaplock."]",
				);
				next;
			    }
			}
		    }
		    unless ( $vol->writeable ) {
			_reject(
			    name   => $vol->name,
			    reason => "Volume isn't writeable",
			);
			next;
		    }
		    my ($prev_name,$new_name);
		    $prev_name = $vol->name;
		    if ( $self->{_dataset} ) {
			next unless ($vol->dataset eq $self->{_dataset});
			$new_name  = NOTE::datasetEncode( $self->{_name}, $self->{_dataset} );
			$self->{_name} = $new_name;
		    } else {
			# If no dataset required, reject volumes with datasets
			next if ( $vol->dataset );
			$new_name  = $self->{_name};
		    }
		    $self->{_Filer}      = $vol->Filer;
		    if ( $self->{_type} eq "flex" ) {
			$self->{_Aggregate} = $vol->Aggregate;
		    }
		    NOTE::logcomment(
			"Recycling %s volume:  %s --> %s ",
			$self->{_type}, $prev_name, $new_name
		    );
		    $vol->_rename( $new_name );
		    if ( $type eq 'flex' ) {
			if ( $aggrQualities->{snaplock} eq 'compliance' ) {
			    $self->{_Aggregate}->Filer->check_license( 'snaplock' );
			} elsif ( $aggrQualities->{snaplock} eq 'enterprise' ) {
			    $self->{_Aggregate}->Filer->check_license( 'snaplock_enterprise' );
			}
		    }
		    $self->{_was_recycled} = 1;
		    goto gotOne; # got one
		};
		NOTE::logcomment( "Attempted recycle failed: $@" ) if ($@);
	}
	NOTE::logcomment( "No candidate found" );
	# No candidate found

	# XXX If I want a volume with a dataset, I should be
	# able to convert an empty volume to a populated volume
	# if no appropriate populated volumes exists.
	# And vice-versa - If I want an empty volume,
	# and none exist, I might want to consider removing
	# data from a populated volume, if there are no more
	# disks available.
	return 0;
gotOne:
	return 1;
}

=head2 B<Creation, Naming, and Destruction>

=cut

=head2 ::create

Creates a volume and returns an object representing the created volume.

=over

=item Arguments

B<name> => I<string>

I<Optional.>
The name of the volume.
If absent, an anonymous volume name will be generated.
If present the volume and containing aggregate (if there is one)
will be spared during cleanup.

B<type> => C<flex> | C<trad>

I<Optional.>
The type of volume to create.
The default is C<flex>.
If the the value is C<trad>
then the B<Aggregate> option may not be present.

B<size> => I<size spec>

I<Optional.>
The size to make the volume.
Only valid for flexible volumes.
(The size of a traditional volume is determined
by how many disks it is composed of.)
If absent the default size is the minimum flexible volume size, 20MiB.

B<flex> => C<{> I<::create options> C<}>

I<Optional.>
Additional C<::create> options that only take effect
if the type of the volume being created is C<flex>.
Options that aren't specific to flexible volumes may be used.

B<trad> => C<{> I<::create options> C<}>

I<Optional.>
Additional C<::create> options that only take effect
if the type of the volume being created is C<trad>.
Options that aren't specific to tradtional volumes may be used.

Note that both B<flex> and B<trad> may be used
in the same argument list.
This can be useful when, for example,
the value of B<type> is determined by a variable.

B<Filer> => I<host spec>

I<Required.>
The filer on which to create the volume.

B<Aggregate> => I<object> | I<spec>

I<Optional.>
The aggregate in which to create the volume.
May be a
L<C<NOTE::Aggregate>|engineering::ontap_tests::main::docs::lib-NOTE-Aggregate-pm>
object or an aggregate spec.
If absent and the volume type is C<flex>
an anonymous aggregate will be used.

B<disklist> => I<disk specifier>

I<Optional.>
The disks to use when creating a traditional volume.
The create fails if the named disks aren't available.

B<disks> => I<number>

I<Optional.>
The number of disks to use when creating a traditional volume.

B<zeroed> => C<0> | C<1>

I<Optional.>
A flag to indicate whether or not to chose first already zeroed disks
when chosing disks for a traditional volume.
The default is to allow ONTAP to choose appropriate disks, zeroed or not.

B<dataset> => I<dataset specifier>

I<Optional.>
A description of some data to put in the volume.
The dataset will be fully created
before the call to C<::create> returns.
See L<C<::fill>|/::fill>
for a description of dataset specifiers.

B<exportfs> => C<{> I<::exportfs options> C<}>

I<Optional.>
NFS export options to set on the volume.
These are passed to L<C<::exportfs>|/::exportfs>
and are described there.
You will probably be sorry if you don't use at least 'C<anon=0>'.

B<anonymous> => C<0> | C<1>

I<Optional.>
A flag to indicate whether or not the created volume
should be recycled when its current life ends.
Requesting recycling also means that a previously
existing volume may be reused instead of actually
creating a new one.
The default behavior is to recycle anonymous objects
and not recycle named objects.

B<recycle> => C<0 | 1>

I<Optional.>
A flag to indicate whether or not the created Volume
should be recycled when its current life ends.
Requesting recycling also means that a previously
existing Volume may be reused instead of actually
creating a new one.
The default behavior is to recycle anonymous objects
and not recycle named objects.

Volumes with datasets may be recycled.
This is implemented by taking a snapshot
immediately after the dataset is created
and reverting to it when the volume is reused.
Dataset specifiers must be identical strings
to be considered exact matches and thus
equivalent for the purposes of recycling.

B<tag> => I<string>

I<Optional.>
A text description of the object.
See L<C<::tag>|/::tag> for details.

B<snaplock> => C<compliance | enterprise>

I<Optional.>
If set, specifies which type of snaplock volume to create. This
is valid only for traditional volumes. Flex vols inherit the
type from the containing aggregate.

B<richdata> => C<0 | 1>

I<Optional.>
If set, use RichData library instead of DataManip for
L<C<::populate>|/::populate> and L<C<::modify>|/::modify>.
The default is 0.

B<asis> => C<0 | 1>

I<Optional.>
If set, calls the asis method to de-dup the data on the flex volume.
Alternatively the 'asis' method can also be called by setting 'ASIS'
param in environment. The default is undef.

=item Examples

=back

	 $vol = NOTE::Volume->create(
	     Filer => $FILER,
	     tag   => 'destination'
	 );

=cut

sub create {
	my ($class,%args) = @_;
	my $self = {};
	my $exportfs_opts;
	my $diskspec;
	my $size;

	exists $args{type}      and $args{type}      = NOTE::option( $args{type} );
	exists $args{name}      and $args{name}      = NOTE::option( $args{name} );
	exists $args{disks}     and $args{disks}     = NOTE::option( $args{disks} );
	$args{recycle} = NOTE::RECYCLE() unless exists $args{recycle};
	exists $args{anonymous} and $args{anonymous} = NOTE::option( $args{anonymous} );
	exists $args{snaplock}  and $args{snaplock}  = NOTE::option( $args{snaplock} );
	exists $args{richdata}  and $args{richdata}  = NOTE::option( $args{richdata} );
	exists $args{dataset}   and $args{dataset}   = NOTE::option( $args{dataset} );
	exists $args{raidtype}  and $args{raidtype}  = NOTE::option( $args{raidtype} );
	exists $args{rgsize}    and $args{rgsize}    = NOTE::option( $args{rgsize} );
	exists $args{asis}      and $args{asis}      = NOTE::option( $args{asis} );
    exists $args{language}  and $args{language}  = NOTE::option( $args{language} );

	# Allow for specifying different arguments depending on
	# the volume type.
	if ( !defined $args{type} || $args{type} eq 'flex' ) {
		$self->{_type} = 'flex';
		if ( exists $args{flex} ) {
		    %args = %{NOTE::_hashOverlay( target => \%args, source => $args{flex}, overwrite => 0 )};
		}
		$args{size} and $args{size} = NOTE::option( $args{size} );
		if ( $args{dataset} && !defined $args{size} ) {
		    # In this case, don't allow volume size to be undefined,
		    # otherwise, the volume chosen might be bigger than "20m".
		    # If this is a source volume, the destination might then
		    # end up being smaller, if it also was created without
		    # specifying a size.
		    $args{size} = "20m";
		}
		$size = NOTE::units( $args{size} || "20m" );
	} else {
		$self->{_type} = 'trad';
		delete $args{Aggregate};
		if ( exists $args{trad} ) {
		    %args = %{NOTE::_hashOverlay( target => \%args, source => $args{trad}, overwrite => 0 )};
		}
	}
	delete $args{flex};
	delete $args{trad};
	delete $args{type};

	$self->{_debug}    = $args{debug};
	$self->{_owner}    = $$;

	$self->{_recycle}  = delete $args{recycle};
	$self->{_tag}      = delete $args{tag};
	$self->{_teardown} = delete $args{teardown};
	$self->{_snaplock} = delete $args{snaplock};
	$self->{_richdata} = delete $args{richdata};
	$self->{_dataset}  = delete $args{dataset};
	$self->{_asis}     = delete $args{asis};

	if ( !defined( $self->{_asis} ) ) {
	    $self->{_asis} = NOTE::option( q/$ASIS/ );
	}

	# user can't decide if they want trad or flex
	if ( ($self->{_type} eq 'trad') && defined $args{Aggregate} ) {
		croak "Cannot determine if flexvol or traditional volume is to " .
		  "be created";
	}

	if ( $args{name} ) {
		$self->{_name} = $args{name};
		# Allow named volumes that behave like anonymous volumes:
		$self->{_anonymous} = $args{anonymous};
	} else {			# Create a unique name
		$self->{_name} = NOTE::Volume->newName;
		$self->{_anonymous} = 1;
	}
	delete $args{name};
	delete $args{anonymous};

	# Default to recycle=>1 for anonymous volumes
	if ( $self->{_anonymous} && !defined($self->{_recycle}) ) {
		$self->{_recycle} = 1;
	}

	$exportfs_opts = $args{exportfs} || "rw,nosuid,anon=0";
	delete $args{exportfs};

	my $desc = $self->{_name};
	$desc .= " (".$self->{_tag}.")" if ( $self->{_tag} );

	NOTE::logcomment( "Require ".$self->{_type}." volume $desc" );

	# Get Filer object or spec from aggregate object or spec, if present..
	# This means that:
	#    $vol = NOTE::Volume->create(
	#        Filer => "somefiler",
	#        Aggregate => $aggr,
	#    );
	# will create the volume in an aggregate on "anotherfiler"
	#
	if ( $args{Aggregate} ) {
		if ( ref $args{Aggregate} eq "NOTE::Aggregate" ) {
		    $args{Filer} = $args{Aggregate}->Filer;
		} elsif ( $args{Aggregate} eq "HASH" ) {
		    $args{Filer} ||= $args{Aggregate}->{Filer};
		}
	}

	# if the $args{Filer} is a filer object, use it,
	# otherwise try to create one by calling the NOTE::Filer constructor.
	if ( !defined($args{Filer}) ) {
	    # In this case, there are no arguments (i.e. no hostname) for the
	    # filer. I call the filer method anyway and let it decide to
	    # handle it or croak.
	    $self->{_Filer} = NOTE::Filer->connect();
	} elsif ( ref $args{Filer} eq "HASH" ) {
		$self->{_Filer} = NOTE::Filer->connect( %{$args{Filer}} );
	} elsif ( ref $args{Filer} eq "NOTE::Filer" ) {
		$self->{_Filer} = $args{Filer}; # This is already an object
	} elsif ( ref \$args{Filer} eq "SCALAR" ) {
		# Is just a hostspec
		$self->{_Filer} = NOTE::Filer->connect( host => $args{Filer} );
	} else {
		croak "Invalid filer specification: ".ref \$args{Filer}."\n";
	}
	delete $args{Filer};

	if ( $self->{_type} eq 'flex' ) {
		if ( $self->{_recycle} ) {
		    if ( $self->{_anonymous} ) {
			# Only bother recycling anonymous flex vols with datasets
			if ( $self->{_dataset} && _recycle( $self, %args ) ) {
			    bless $self, $class;
			    NOTE::reference( $self );
			    tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;
			    $self->exportfs( $exportfs_opts ) if ( $exportfs_opts );
			    return $self;
			}
			# Otherwise, try to create it...
		    } else {  # Recycle named flexvol, if present
			my ($state, $status, $aggr, $options) = Volume::vol_status(
			    $self->{_Filer}->conn, $self->{_name},
			);
			if ( $status =~ /trad/ ) {
			    croak "Volume named ".$self->{_name}." is a tradvol";
			}
			if ( $status =~ /flex/ ) {
			    NOTE::logcomment( "Re-using Volume $desc" );
			    $self->{_Aggregate} = NOTE::Aggregate->attach(
				Filer => $self->{_Filer},
				name  => $aggr
			    );
			    bless $self, $class;
			    NOTE::reference( $self );
			    tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;
			    $self->exportfs( $exportfs_opts ) if ( $exportfs_opts );
			    return $self;
			}
			# Otherwise, try to create it...
		    }
		}
		# if the $args{Aggregate} is an aggregate object, use it,
		# otherwise try to create one by calling the NOTE::Aggregate 
		# constructor.
		if ( !defined($args{Aggregate}) ) {
		    my @extra = ();
		    # If the volume is a snaplock volume, propagate this information
		    # to the aggregate create method.
		    if ( $self->{_snaplock} ) {
			push @extra, 'snaplock' => $self->{_snaplock};
		    }
		    $self->{_Aggregate} = NOTE::Aggregate->create(
		        Filer => $self->{_Filer},
			tag => "Container of ".$desc,
			@extra,
   	    );

		} elsif ( ref $args{Aggregate} eq "HASH" ) {
		
		    unless( $args{Aggregate}->{tag} ) {
			$args{Aggregate}->{tag} = "Container of ".$desc;
		    }
		    # If the volume is a snaplock volume, propagate this information
		    # to the aggregate create/attach method.
		    if ( $self->{_snaplock} ) {
			$args{Aggregate}->{'snaplock'} = $self->{_snaplock};
		    }
		    # If it's named, try to attach it.
		    if ( $args{Aggregate}->{name} ) {
			eval {
			    $self->{_Aggregate} = NOTE::Aggregate->attach(
				Filer => $self->{_Filer},
				%{$args{Aggregate}},
			    );
			};
		    }
		    # If it's unnamed or the attach fails, try
		    # to create it.
		    if ( (!$args{Aggregate}->{name}) || $@ ) {
			$self->{_Aggregate} = NOTE::Aggregate->create(
			    Filer => $self->{_Filer},
			    %{$args{Aggregate}},
			);
		    }

		} else {
		    $self->{_Aggregate} = $args{Aggregate};
		}
		delete $args{Aggregate};

		# Make sure the snaplock licenses are properly set if necessary
		if ( $self->{_Aggregate}->snaplock eq 'compliance' ) {
		    $self->{_Aggregate}->Filer->check_license( 'snaplock' );
		} elsif ( $self->{_Aggregate}->snaplock eq 'enterprise' ) {
		    $self->{_Aggregate}->Filer->check_license( 'snaplock_enterprise' );
		}

		# If the volume isn't anonymous, make certain the parent
		# aggregate is spared.
		if ( ! $self->{_anonymous} ) {
		    $self->{_Aggregate}->SPARE;
		}

		# Get the Filer object now, because at this point
		# we can be sure that it is an object and not
		# a descriptor (i.e.  { host => "myfiler" } ).
	    $self->{_Filer} = $self->{_Aggregate}->Filer;
		
		my $attempt;
		my $error;
		my $max_attempts;
		if ( $self->{_Aggregate}->is_shareable ) {
		    # maximum of two attempts, or 20 for simulator
		    $max_attempts = $self->{_Filer}->is_a_simulator ? 20 : 2;
		} elsif ( $self->{_Aggregate}->is_anonymous ) {
		    # Allow many more aggregate grow attempts if the aggregate
		    # will be recycled or destroyed when the script exits.
		    $max_attempts = 64; # Just a big number
		} else {
		    $max_attempts = 1; # Don't try to grow named aggregates if vol create fails
		}
		delete $args{size};
        
		for $attempt ( 1..$max_attempts ) {
		    Volume::vol_create_flex(
			$self->{_Filer}->conn,
			$self->{_name},
			$self->{_Aggregate}->name,
			$size . 'k',
			%args
		    );
		    $error = $@;
		    if ( $error =~ /space cannot be guaranteed for the volume/ ) {
			NOTE::logcomment(
			    "Insufficient capacity in %s; trying to grow aggregate",
			    $self->{_Aggregate}->name,
			);
			# Try adding a disk
			$self->{_Aggregate}->add( disks => 1 );
			# If this fails, we could also try deleting free volumes
			# with datasets in this aggregate
			next;
		    }
		    last;
		}
		if ( $error && $attempt ) {
		    NOTE::logcomment( "Volume create failed even after growing aggregate" );
		}
		croak $error if $error;
	} elsif ( $self->{_type} eq 'trad' ) {
		# If a disklist is specified, the disks used will be from that
		# list. Otherwise:
		# If { zeroed => 1 } is specified, OR if the number of disks
		# is not specified, disks will be obtained from the zeroed
		# pool, if possible, even if they are not an optimal fit.
		# If { zeroed => 0 } is specified or undefined, ONTAP will
		# be allowed to choose the disks.

	    my $get_disks_from_zeroed_pool = $args{zeroed};
	    delete $args{zeroed};

		$args{raidtype} ||= NOTE::option( q/$RAIDTYPE:raid_dp/ ); # Default
		if ( !($args{disks}) && !($args{disklist}) ) {
		    if ( $args{raidtype} eq "raid4" ) {
			$args{disks} = "2";
		    } elsif ( $args{raidtype} eq "raid_dp" ) {
			$args{disks} = "3"; # Need at least 3 disks for raid_dp
		    } else {
			croak "Invalid raidtype: '".$args{raidtype}."'";
		    }
		}

		my $numdisks = scalar @{$args{disklist}} if ( $args{disklist} ); 
		$numdisks ||= $args{disks};
		NOTE::logcomment( "Creating %s trad volume with %d disks: $desc",
		    $args{raidtype},
		    $numdisks,
		);

		if ( $self->{_recycle} ) {
		    if ( $self->{_anonymous} ) {
			if ( _recycle( $self, %args ) ) {
			    bless $self, $class;
			    NOTE::reference( $self );
			    tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;
			    $self->exportfs( $exportfs_opts ) if ( $exportfs_opts );
			    # Make sure the snaplock licenses are properly set if necessary
			    if ( $self->{_snaplock} eq 'compliance' ) {
				$self->Filer->check_license( 'snaplock' );
			    } elsif ( $self->{_snaplock} eq 'enterprise' ) {
				$self->Filer->check_license( 'snaplock_enterprise' );
			    }
			    return $self;
			}
			# This is a bit tricky: If I wasn't able to recycle
			# a volume that was populated with a dataset, try to
			# recycle an empty one and then populate it as if
			# it was just created.
			if ( my $dataset = $self->{_dataset} ) {
			    $self->{_dataset} = undef;
			    my $gotit = _recycle( $self, %args );
			    $self->{_dataset} = $dataset;
			    goto initvol if ( $gotit );
			}
		    } else { # Named recyclable volume
			my ($state, $status, $aggr, $options) = Volume::vol_status(
			    $self->{_Filer}->conn, $self->{_name},
			);
			if ( $status =~ /flex/ ) {
			    croak "Volume named ".$self->{_name}." is a flexvol";
			}
			if ( $status =~ /trad/ ) {
			    bless $self, $class;
			    NOTE::reference( $self );
			    tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;
			    $self->exportfs( $exportfs_opts ) if ( $exportfs_opts );
			    # Make sure the snaplock licenses are properly set if necessary
			    if ( $self->{_snaplock} eq 'compliance' ) {
				$self->Filer->check_license( 'snaplock' );
			    } elsif ( $self->{_snaplock} eq 'enterprise' ) {
				$self->Filer->check_license( 'snaplock_enterprise' );
			    }
			    return $self;
			}
			# Otherwise try to create it.
		    }
		}
		delete $args{type};
		my ($success, $err);
		$diskspec = $args{disklist} ? $args{disklist} : $args{disks};
		if ( $args{disks} && $get_disks_from_zeroed_pool ) {
		    my @zd = disk::get_zeroed_spares($self->{_Filer}->conn);
		    if ( @zd >= $args{disks} ) {
			$diskspec = [@zd[0..($args{disks}-1)] ];
			$args{force} = 1 unless defined $args{force}; # force use of disks with different RPM
		    }
		}
		delete $args{disks};
		delete $args{disklist};

		# Commenting below two lines, since $args{snaplock has already deleted in the begining
                # $$self->{_snaplock} will not get snaplock type. So create() is failing to create
                # snaplock traditional volumes.

		#$self->{_snaplock} = $args{snaplock};
		#delete $args{snaplock};

		# Make sure the snaplock licenses are properly set if necessary
		# Fix %args up the way Volume::create wants them:
		if ( $self->{_snaplock} eq 'compliance' ) {
		    $self->{_Filer}->check_license( 'snaplock' );
		    $args{snaplock} = '1';
		} elsif ( $self->{_snaplock} eq 'enterprise' ) {
		    $self->{_Filer}->check_license( 'snaplock_enterprise' );
		    $args{snaplock_enterprise} = '1';
		}
		while ( 1 ) {
		    Volume::vol_create(
		        $self->{_Filer}->conn,
			$self->{_name},
			$diskspec,
			%args
		    );
		    goto initvol unless ($@);
		    if ( $@ =~ /disks specified, but at least|no disks are available/ ) {
			# Try again if we can free some disks
			next if NOTE::Aggregate::_releaseSpareAggr( $self->{_Filer} );
		    }
		    croak "Trad volume create error: $@";
		}
	} else {
		croak "Unknown volume type ".$args{type};
	}
initvol:
	bless $self, $class;
	NOTE::reference( $self );
	tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;

	# Turn on A-SIS deduplication if specified
	$self->asis if $self->{_asis};

	$self->exportfs( $exportfs_opts ) if ( $exportfs_opts );
	if ( $self->{_dataset} ) {
	    NOTE::logcomment( "Initializing dataset of ".$self->description );
		my $func = eval 'sub {'.$self->{_dataset}.'}';
		croak $@ if $@;
		eval { $self->$func };
		croak $@ if $@;
		if ( $self->_looksAnonymous ) {
		    $self->_rename( NOTE::datasetEncode( $self->{_name}, $self->{_dataset} ) );
		}
		# Make sure container of volume is spared if dataset is recyclable
		if ( $self->{_recycle} && $self->type eq 'flex' ) {
		    $self->Aggregate->SPARE;
		}
	}
	$self->preserve if $self->{_recycle};
	return $self;
}

=head2 ::attach

Creates an object representing an existing volume.

=over

=item Arguments

B<name> => I<string>

I<Required.>
The name of an existing volume.

B<type> => C<flex> | C<trad>

I<Optional.>
The type of volume given by B<name>.
If given the attach fails if the volume is not of that type.
If absent the actual type is recorded in the volume object.

B<Filer> => I<host spec>

I<Required.>
Specifies the filer where the volume exists.

B<mntpoint> => I<string>

I<Optional.>
The full path name on which to mount the volume.

B<exportfs> => I<export options>

I<Optional.>
Export options as described in L<C<::exportfs>|/::exportfs>.
You will probably be sorry if you don't use at least 'C<anon=0>'.

B<force> => C<1>

I<Optional.>
A flag to force the creation of a volume object
even if the named volume does not exist.

B<richdata> => C<0 | 1>

I<Optional.>
If set, use RichData library instead of DataManip for
L<C<::populate>|/::populate> and L<C<::modify>|/::modify>.
The default is 0.

=item Examples

=back

	 $vol = NOTE::Volume->attach(
	     Filer => $FILER,
	     name  => 'vold',
	 );

=cut

sub attach {
	my ($class,%args) = @_;
	my $self = {};
	my $filer;
	my $exportfs_opts = $args{exportfs} || "rw,nosuid,anon=0";
	my $force = $args{force};
	delete $args{force};

	exists $args{name} and $args{name} = NOTE::option( $args{name} );
	exists $args{type} and $args{type} = NOTE::option( $args{type} );

	$self->{_debug}    = $args{debug};
	$self->{_owner}    = $$;

	# if the $args{Filer} is a filer object, use it,
	# otherwise try to create one by calling the NOTE::Filer constructor.
	if ( !defined($args{Filer}) ) {
	    # In this case, there are no arguments (i.e. no hostname) for the
	    # filer. I call the filer method anyway and let it decide to
	    # handle it or croak.
	    $self->{_Filer} = NOTE::Filer->connect();
	} elsif ( ref $args{Filer} eq "HASH" ) {
		$self->{_Filer} = NOTE::Filer->connect( %{$args{Filer}} );
	} elsif ( ref $args{Filer} eq "NOTE::Filer" ) {
		$self->{_Filer} = $args{Filer}; # This is already an object
	} elsif ( ref \$args{Filer} eq "SCALAR" ) {
		# Is just a hostspec
		$self->{_Filer} = NOTE::Filer->connect( host => $args{Filer} );
	} else {
		croak "Invalid filer specification: ".ref \$args{Filer}."\n";
	}
	delete $args{Filer};

	croak "Volume attach requires a name" unless ( $args{name} );
	$self->{_name} = $args{name};
	$self->{_dataset} = NOTE::datasetDecode( $self->{_name} );

	$self->{_tag}     = $args{tag};
	delete $args{tag};

	$self->{_richdata} = $args{'richdata'};
	delete $args{'richdata'};

	my $desc = $self->{_name};
	$desc .= " (".$self->{_tag}.")" if ( $self->{_tag} );

	my ($state, $status, $aggr, $options) = Volume::vol_status(
		$self->{_Filer}->conn, $self->{_name},
	);
	croak $@ if ( $@ && !$force );

	if ( $status =~ /trad/ ) {
		if ( exists $args{type} && $args{type} ne "trad" ) {
	        croak "Volume ".$self->{_name}." is of type trad, not $args{type}";
		}
		$self->{_type} = "trad";
	} elsif ( $status =~ /flex/ ) {
		if ( exists $args{type} && $args{type} ne "flex" ) {
	        croak "Volume ".$self->{_name}." is of type flex, not $args{type}";
		}
		$self->{_type} = "flex";

		$self->{_Aggregate} = NOTE::Aggregate->attach(
		    Filer => $self->{_Filer},
		    name  => $aggr,
	    );
	}

	if ( exists $args{mntpoint} ) {
		$self->{_mntpoint} = $args{mntpoint};
	}

	bless $self, $class;
	tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;

	# It's possible this'll fail if volume is offline:
	eval { $self->exportfs( $exportfs_opts ) };

	return $self;
}

=head2 ::dup

Create (or reuse) a volume with attributes similar to an existing volume.

=over

=item Arguments

Takes the same arguments as NOTE::Volume::create.

=item Examples

  $vol_dst = $vol_src->dup;  # Make a destination volume similar to the source

  $vol_dst = $vol_src->dup( Filer => q/$FILER_DST/ ); # The same, but different filer

  $vol_dst = $vol_src->dup( Aggregate => $aggr_dst ); # The same, but different aggregate

=back

=cut

sub dup {
	my ($origin,%args) = @_;
	my %properties = $origin->Properties;

	%properties = %{NOTE::_hashOverlay( target => \%properties, source => \%args, overwrite => 1 )};
	$properties{Filer} ||= $origin->Filer;
	return NOTE::Volume->create( %properties );
}

sub Properties {
	my $self = shift;
	my ($curr_fs, $exportfs_opts);
	# Use default if failed
	eval {
		$curr_fs = $self->exportfs;
	};
	$exportfs_opts = $curr_fs || "rw,nosuid,anon=0";
	my %properties = (
		anonymous  => $self->{_anonymous},
		richdata   => $self->{_richdata},
		recycle    => $self->{_recycle},
		type       => $self->{_type},
		snaplock   => $self->snaplock,
		exportfs   => $exportfs_opts,
	);
	if ( $properties{type} eq 'flex' ) {
		$properties{Aggregate} = { $self->Aggregate->Properties };
		$properties{size}      = $self->size
	} else {
		my $opts = $self->options;
		$properties{disks}     = scalar $self->disks;
		$properties{raidtype}  = $opts->{raidtype};
		$properties{rgsize}    = $opts->{raidsize};
	}
	return %properties;
}

=head2 ::destroy

Destroys the volume and renders the volume object unuseable.
If the volume is a traditional volume
L<C<NOTE::Filer::zero_spares>|engineering::ontap_tests::main::docs::lib-NOTE-Filer-pm/::zero_spares>
is called.

=over

=item Arguments

B<force> => C<1>

I<Optional.>
A flag to force the destruction of a volume
even if it is corrupt.

=item Example

=back

	 $vol->destroy;

	 $vol->destroy( force => 1 );

=cut

sub destroy {
	my ($self,%args) = @_;
	my $force = delete $args{'force'};

	NOTE::logcomment( "Destroying Volume ".$self->description );
	Volume::vol_destroy( $self->Filer->conn, $self->{_name}, destroy_corrupt_volumes => $force, %args );
	croak $@ if $@;

	# zero spares on teardown:
	$self->Filer->zero_spares if ( $self->{_type} eq 'trad' );

	$self->{_destroyed} = 1;
	return $self;
}

=head2 ::purge

Explicitly offline and destroy a volume

=over

=item Arguments

Same arguments as ::destroy.

=item Example

		$vol->purge;

=back

=cut

sub purge {
	my ($self,%args) = @_;

	NOTE::logcomment( "Purging Volume ".$self->description );
	eval { $self->offline };
	$self->destroy( %args );
	return $self;
}

=head2 ::name

Returns the name of the volume.

=over

=item Arguments

None.

=item Example

=back

	 $name = $vol->name;

=cut

sub name { return shift->{_name} }
sub path { return shift->{_name} }   # deprecated

=head2 ::rename

Changes the name of the volume.

=over

=item Arguments

I<string>

I<Optional.>
The new name for the volume.
If absent a new anonymous name is generated
and the volume becomes anonymous.
If present the volume will no longer be anonymous,
causing its aggregate to be spared
(not destroyed) when the volume object is destroyed.

=item Examples

=back

	 $vol->rename;
	 $vol->rename( 'newName' );

=cut

sub rename {
	my ($self,$newname) = @_;
	my $oldname = $self->{_name};
	if ( defined $newname ) {
		$self->_rename( $newname );
		if ( $self->{_anonymous} && !$self->_looksAnonymous ) {
		    # Converting from anonymous to non-anonymous
		    $self->SPARE;
		    $self->{_anonymous} = 0;
		}
	} else {
		$newname = $self->newName;
		$self->{_anonymous} = 1;
		$self->_rename( $newname );
	}
	return $oldname;
}

sub _rename {
	my ($self,$newname) = @_;
	Volume::vol_rename( $self->{_Filer}->conn, $self->{_name}, $newname );
        sleep(5); #Wait 5 sec to reflect the change.
	croak $@ if $@;
	$self->{_name} = $newname;
}

=head2 B<Disk Space>

=cut

=head2 ::size

Returns the size of the volume in KB.

=over

=item Arguments

I<size spec>

I<Optional.>
A new size to make the volume.

=item Examples

=back

	 $size = $vol->size;
	 $size = $vol->size( '500m' );   # 500MB returned as kilobytes
	 $size = $vol->size( '+100m' );  # Increase volume size

=cut

sub size {
	my ($self,$newsize) = @_;
	my $ret;

	if ( defined $newsize ) {
	    # If there are no units specified, default to 'k'
	    $newsize .= "k" unless ( $newsize =~ /[tbkmg]$/i );
	}

	if ( $self->type eq 'flex' ) {
		if ( defined($newsize) ) {
		    $ret = Volume::vol_size( $self->Filer->conn, 
					     $self->{_name}, 
					     $newsize );
		    croak "Unable to resize ".$self->description.": $@" if ( $@ );
		} else {
		    $ret = Volume::vol_size( $self->Filer->conn, 
					     $self->{_name} );
		    croak "Unable to get size of ".$self->description.": $@" if ( $@ );
		}
	} else {
		if ( defined($newsize) ) {
		    croak "Cannot resize traditional volume. Add disks to it instead";
		} else {
		    $ret = $self->disk_free;
		    croak "Unable to get size of ".$self->description.": $@" unless( $ret );
		}
	}

	croak $@ if $@;
	# Caching the size helps performance when sorting arrays of volumes by size
	$self->{_size} = $ret;
	return $ret;
}

=head2 ::inode_status

Returns volumes's inode status from 'df -i' command
This is a wrapper to
L<C<wafl_utils::df -i>|engineering::ontap_tests::agnostic::docs::lib-wafl_utils-pm/::df>
routine.

=over

=item Argument

B<col> = I<string>

Specific 'df -i' colume to return. Returns all colume in a hash if undefined. Default: undef

=item Examples

	my $free_cnt = $vol->inode_status('ifree'); # Returns free inode count

=back

=cut

sub inode_status {
	my ($self, $col) = @_;

	my %df;
	eval {
		%df = wafl_utils::df($self->Filer->conn, $self->name, options=>'-i');
	};
	croak "inode_stats failed: $@" if $@;
	if ( $col ) {
		if ( exists $df{volume}->{$col} ) {
			return $df{volume}{$col};
		} else {
			croak "Invalid colume $col for 'df -i'";
		}
	} else {
		return $df{volume};
	}
}

=head2 ::disk_free

Returns a list of items about the disk usage
of the volume.
The items returned,
and the order in which they are returned are:

B<total> -- the total disk space extant in the volume

B<used> -- the total disk space currently used in the volume

B<avail> -- the total disk space currently available in the volume

B<capacity> -- the percentage of the total disk space
in the volume that is currently being used

There are also methods by each of the item names
above that return only the value for that particular item.

=over

=item Arguments

None.

=item Example

=back

	 ($total $used $avail $capacity) = $vol->disk_free;

=cut

sub disk_free {
	my ($self,%args) = @_;
	my $vol;

	exists $args{'type'} and $args{'space_type'} = delete $args{'type'};

	if ( $self->isa( "NOTE::Qtree" ) ) {
		# Do this until I figure out how to get the
		# stats of a qtree:
		$vol = $self->Volume;
	} else {
		$vol = $self;
	}
	my ($total,$used,$avail,$capacity) = Volume::vol_disk_free(
		$vol->Filer->conn,
		path => $vol->mount_path,
		%args
	);
	croak $@ if $@;
	return wantarray ? ($total,$used,$avail,$capacity) : $total;
}

=head2 ::total

Returns the total disk space extant in the volume.

=over

=item Arguments

None.

=item Examples

=back

	 $total = $vol->total;

=head2 ::used

Returns the total disk space currently used in the volume.

=over

=item Arguments

None.

=item Examples

=back

	 $used = $vol->used;

=head2 ::avail

Returns the total disk space currently available in the volume.

=over

=item Arguments

None.

=item Examples

=back

	 $total = $vol->avail;

=head2 ::capacity

Returns the percentage of the total disk space
in the volume that is currently being used.

=over

=item Arguments

None.

=item Examples

=back

	 $capacity = $vol->capacity;

=cut

sub total    { return (shift->disk_free( @_ ))[0] }
sub used     { return (shift->disk_free( @_ ))[1] }
sub avail    { return (shift->disk_free( @_ ))[2] }
sub capacity { return (shift->disk_free( @_ ))[3] }

=head2 ::add

Adds disks to the volume.
Only valid for traditional volumes.

If both the B<disks> and B<disklist> arguments are given
the B<disks> argument will be used and B<disklist> ignored.

If neither option is given the default is C<disks =E<gt> 1>.

=over

=item Arguments

B<disks> => I<positive integer>

I<Optional.>
The number of disks to add.
The specific disks will be picked by ONTAP.

B<disklist> => C<(> I<disk name> ... C<)>

I<Optional.>
A list of disks to add.

=item Examples

=back

	 $vol->add;
	 $vol->add( disks => 2 );
	 $vol->add( disklist => qw(0b.13, 0b.16) );

=cut

sub add {
	my ($self,%args) = @_;

	my $diskspec = $args{disks} || $args{disklist};
	delete $args{disks};
	delete $args{disklist};
	$diskspec = "1" unless ( $diskspec );

	if ( ref $diskspec eq "ARRAY" ) {
	    my $howmany = scalar @{$diskspec};
	    my $what = $howmany == 1 ? "disk" : "disks";
	    NOTE::logcomment( "Adding $howmany $what to ".$self->description );
	} else {
	    my $what = $diskspec == 1 ? "disk" : "disks";
	    NOTE::logcomment( "Adding $diskspec $what to ".$self->description );
	}
	# note: disks can't be added to a flexvol.
	# let ONTAP do the error checking for flexvol
	Volume::vol_add(
		$self->Filer->conn,
		$self->name,
		$diskspec,
		%args
	);
	croak $@ if $@;
	if ( $self->type eq 'trad' && $self->{_anonymous} && $self->{_recycle} ) {
		NOTE::logcomment(
		    "Added disk to anonymous, recyclable tradvol.\n".
		    "    Turning recycle flag off so volume will be destroyed at teardown.\n".
		    "    Name the volume if you want to get around this."
		);
		$self->{_recycle} = 0;
	}
	return $self;
}

=head2 ::disks

Returns a list of the disks
comprising a traditional volume
or the array of disks used by
the underlying aggregate
of a flexible volume.

=over

=item Arguments

None.

=item Example

=back

	 @disks = $vol->disks;

=cut

sub disks {
	my $self = shift;
	my @disks = Volume::vol_disks( $self->{_Filer}->conn, $self->{_name} );
	croak $@ if $@;
	if ( scalar @disks == 0 ) {
	    croak "Volume ".$self->description." reports ZERO disks";
	}
	return @disks;
}

=head2 B<Mounting>

=cut

=head2 ::exportfs

Gets or sets NFS export options on the volume.

If no option string is specified, returns the current exportfs
value for the volume.

If called multiple times, this function will not re-issue the
exportfs command to the filer unless the exportfs option string
changes or a call to
L<C<NOTE::Volume::unexportfs>|engineering::ontap_tests::main::docs::lib-NOTE-Volume-pm/::unexportfs>
is made.

=over

=item Arguments

Optional: A string containing NFS exportfs options.

=item Example

=back

	 $vol->exportfs( 'rw,nosuid,anon=0' );   # No spaces allowed within the list

	 printf "Volume export options are ", $vol->exportfs, "\n";

=cut

sub exportfs {
	my ($self,$exportfs_opts) = @_;

	$self->Filer->check_license( "nfs" );
	if ( defined $exportfs_opts ) {
		if ( $exportfs_opts ne $self->{_exportfs_opts} ) {
		    ExportFS::export_path( $self->Filer->conn, $self->mount_path, OPTIONS => $exportfs_opts );
		    croak $@ if $@;
		    $self->{_exportfs_opts} = $exportfs_opts;
		}
		return $self;
	} else {
		my $current_fs = ExportFS::list_exports( $self->Filer->conn )->{$self->mount_path};
		if ( exists $self->{_exportfs_opts} && $self->{_exportfs_opts} ne $current_fs ) {
			# Sync up filer's exportfs state with object state
			my $exp_fs = delete $self->{_exportfs_opts};
			NOTE::logcomment($self->description.": resync filer exportfs to $exp_fs");
			$self->exportfs($exp_fs);
		} else {
			# Set object's exportfs state
			$self->{_exportfs_opts} = $current_fs;
		}
		return $self->{_exportfs_opts};
	}
}

=head2 ::unexportfs

Unexport a path on the filer

=over

=item Arguments

None

=item Example

=back

	 $vol->unexportfs;

=cut

sub unexportfs {
	my ($self) = @_;

	$self->Filer->check_license( "nfs" );
	ExportFS::unexport_path( $self->Filer->conn, $self->mount_path, EXISTENT => 0 );
	croak $@ if $@;
	delete $self->{_exportfs_opts};
	return $self;
}

=head2 ::mount

NFS mounts the volume.

=over

=item Arguments

I<string>

I<Optional.>
A list of mount options.
Valid options are:

XXX

=item Example

=back

	 $vol->mount( chmod => '777' );

=cut

sub mount {
	my ($self,%opts) = @_;

	$self->Filer->check_license( "nfs" );
	my $reply = SayWrap::sayEOF( $self->Filer->conn, "nfs on");
	if( $reply !~ /NFS server is running/ ) {
		croak("Unable to bring NFS server up using 'nfs on' command.\nERROR: $reply");
	}
	NOTE::logcomment("NFS MOUNT: $reply");
	if ( exists $opts{'exportfs'} ) {
		$self->exportfs( $opts{'exportfs'} );
		delete $opts{'exportfs'};
	}
	unless ( $self->{_mntpoint} ) {
		$self->{_client} = exists $opts{client} ? $opts{client} : "localhost";
		NOTE::logcomment( "Mounting ".$self->description." on ".$self->{_client} );
	    $self->{_mntpoint} = GenericMount::mount(
		    $self->Filer->defaultip,
		    $self->mount_path,
		    %opts,
	    );
	}
	croak $@ if $@;
	return $self->{_mntpoint};
}

=head2 ::mounted

Returns the NFS mount point of the volume.
Returns C<undef> if the volume is not mounted.

=over

=item Arguments

None.

=item Example

=back

	 $mntpt = $vol->mounted;

=cut

sub mounted { return shift->{_mntpoint} }

=head2 ::mountpoint

A synonym for L<C<::mounted>|/::mounted>.

=cut

sub mountpoint { return shift->{_mntpoint} }

=head2 ::umount

Unmounts the volume.

=over

=item Arguments

I<string>

I<Optional.>
A list of unmount options.
Valid options are:

XXX

=item Example

=back

	 $vol->umount;

=cut

sub umount {
	my ($self, %opts) = @_;

	if ( $self->{_mntpoint} ) {
		$opts{client} = $self->{_client} unless $opts{client};
		NOTE::logcomment( "Unmounting ".$self->description." from ".$opts{client} );
		if ( GenericMount::umount( $self->{_mntpoint}, %opts ) ) {
		    delete $self->{_mntpoint};
		    return 1;
		}
		croak "NOTE::Volume::umount: $@" if $@;
	}
	return 0;
}

=head2 ::mount_path

Returns the full path name of the volume,
i.e. C</vol/><I<volume name>>

=over

=item Arguments

None.

=item Example

=back

	 $mntpt = $vol->mount_path;

=cut

sub mount_path { return "/vol/".shift->{_name} }

=head2 ::hostpath

Returns the name of the volume
prepended by its filer's name.

=over

=item Arguments

None.

=item Example

=back

	 $hp = $vol->hostpath;

=cut

sub hostpath {
	my $self = shift;
	return $self->{_Filer}->host.":".$self->{_name};
}

=head2 ::fullname

Returns the same thing as hostpath method, namely the volume name
prepended by its filer's name.

=over

=item Arguments

None.

=item Example

=back

	 $hp = $vol->fullname;

=cut

sub fullname {
	my $self = shift;
	return $self->{_Filer}->host.":".$self->{_name};
}


=head2 ::fsid

Sets or returns Volume's fsid

=over

=item new_fsid

I<Optional.>

If supplied will set the volume's fsid

=item Examples

		$id = $vol->fsid;

		$new_id = $vol->fsid($new_fsid);

=back

=cut

sub fsid {
	my ($self, $new_fsid, %args) = @_;
	my $fsid;

	if ( defined($new_fsid) ) {
		NOTE::logcomment( "Setting FSID of ".$self->description." to 0x%08x", $new_fsid );
		$fsid = Volume::vol_rewrite_fsid(
		    $self->Filer->conn, 
		    $self->name,
		    fsid => sprintf( "0x%x", $new_fsid ),
		    %args, 
		);
	} else {
	    $fsid = Volume::vol_read_fsid( $self->Filer->conn, $self->name );
	}
	croak $@ if $@;
	return hex $fsid;
}

=head2 ::uuid

Returns volume's UUID

=over

=item Arguments

None

=item Example

=back

	 $id = $vol->uuid;

=cut

sub uuid {
	my ($self) = @_;

	my $vinfo = Aggregate::ImplTemplate::RaidInfo_new::get_vol_raid_info($self->Filer->conn, $self->name);
	return $vinfo->{uuid};
}

=head2 ::offline, ::online, ::restrict

Sets volume offline, online and restrict respectively

=over

=item Arguments

None

=item Examples

=back

	 $vol->offline;

	 $vol->online;

	 $vol->restrict;

=cut

sub offline {
	my $self = shift;
	Volume::vol_offline( $self->Filer->conn, $self->{_name}, @_ );
	croak $@ if $@;
	return $self;
}

sub online {
	my $self = shift;
	Volume::vol_online( $self->Filer->conn, $self->{_name}, @_ );
	croak $@ if $@;
	return $self;
}

sub restrict {
	my $self = shift;
	Volume::vol_restrict( $self->Filer->conn, $self->{_name}, @_ );
	croak $@ if $@;
	return $self;
}

=head2 ::snaplock

If a volume is a lockvault volume then returns it's lockvault type,
whether compliance or enterprise. Otherwise returns 0

=over

=item Arguments

None.

=item Example

=back

	 $vol->snaplock;

=cut

sub snaplock {
	my $self = shift;
	# use cached value if available
	unless ( defined $self->{_snaplock} ) {
		if ( $self->type eq 'flex' ) {
		    # Snaplock type is inherited from containing aggregate
		    $self->{_snaplock} = $self->{_Aggregate}->snaplock;
		} else {
		    my $opts = $self->options;
		    if ( $opts->{'snaplock_compliance'} ) {
			$self->{_snaplock} = 'compliance';
		    } elsif ( $opts->{'snaplock_enterprise'} ) {
			$self->{_snaplock} = 'enterprise';
		    } else {
			$self->{_snaplock} = 0;
		    }
		}
	}
	return $self->{_snaplock};
}

=head2 B<Options and Attributes>

=cut

=head2 ::type

Returns the type of the volume.
The return value is one of
the strings C<'flex'> or C<'trad'>.

=over

=item Arguments

None.

=item Examples

=back

	 $type = $vol->type;

=cut

sub type { return shift->{_type} }

=head2 ::options

Set or get options of a volume

=over

=item Arguments

No param - Will return a hash of volume's options.

1 param - Returns the volume of the options indicated by the supplied key

2 param - Sets the options indicated by the first parameter to the value indicated by the second, and returns a hash of all the options.

=item Example

=back

	  $opts = $vol->options;

	  $is_fixed = $vol->options("fs_fixed_size");

	  $vol->options("maxdirsize"=>512);

=cut

sub options {
	my ($self,@args) = @_;
	if ( scalar(@args) == 0 ) { 
		my $options = Volume::vol_options( $self->Filer->conn, $self->name );
		croak $@ if $@;
		return $options;

	} elsif ( scalar(@args) == 1 ) {
		my $options = Volume::vol_options(
		    $self->Filer->conn,
		    $self->name
		);
		croak $@ if $@;
		return $options->{ $args[0] };

	} else {
		my $options = Volume::vol_options(
		    $self->Filer->conn,
		    $self->name,
		    @args
		);
		croak $@ if $@;
		return $options;
	}
}

=head2 ::status

Get the status of the volume.
The return value is a hash reference.

=over

=item Arguments

None.

=item Examples

=back

	 $status = $vol->status;

=cut

sub status {
	my $self = shift;
	Volume::vol_status( $self->Filer->conn, $self->name );
}

=head2 ::restricted

Returns a boolean indicating whether a volume is restricted or not

=over

=item Arguments

None.

=item Examples

=back

	  $vol->restricted;

=cut

sub restricted {
	my $self = shift;
	my ($state, $status, $aggr, $options) = Volume::vol_status(
	    $self->Filer->conn, $self->name
	);
	croak $@ if $@;
	return $state =~ /restricted/;
}

=head2 ::snapmirrored

Returned a boolean indicating whether the volume state is snapmirrored or not

=over

=item Arguments

None.

=item Examples

=back

	 $vol->snapmirrored

=cut

sub snapmirrored {
	my $self = shift;
	my ($state, $status, $aggr, $options) = Volume::vol_status(
	    $self->Filer->conn, $self->name
	);
	croak $@ if $@;
	return $state =~ /snapmirrored/;
}

=head2 ::writeable

Returns a boolean indicating whether the volume is writeable

=over

=item Arguments

None.

=item Examples

=back

	  $vol->writeable

=cut

sub writeable {
	my $self = shift;
	my ($state, $status, $aggr, $options) = Volume::vol_status(
	    $self->Filer->conn, $self->name
	);
	croak $@ if $@;
	return (
		$state =~ /online/ &&
		$status !~ /read-only/ &&
		$state !~ /restricted/
	);
}

=head2 ::readable

Returns a boolean indicating whether a volume is readable,
ie online and not restricted

=over

=item Arguments

None.

=item Examples

=back

	  $vol->readable

=cut

sub readable {
	my $self = shift;
	my ($state, $status, $aggr, $options) = Volume::vol_status(
	    $self->Filer->conn, $self->name
	);
	croak $@ if $@;

	return (
		$state =~ /online/ &&
		$state !~ /restricted/
	);
}

=head2 ::language

Sets or gets the volume's language setting

=over

=item Argument

None - Returns the current language setting for the volume

1 parameter - Sets the volume language to the supplied language

=item Examples

=back

	  $lang = $vol->language;

	  $vol->language("fr");

=cut

sub language {
	my ($self, $language, %opts) = @_;

	if ( defined $language ) {
		croak "Failed to set new language $language: $@"
			unless Volume::vol_lang($self->Filer->conn, $self->name,
						language=>$language,%opts);
		return 1;
	} else {
		my @result = Volume::vol_lang($self->Filer->conn, $self->name);
		croak $@ if $@;
		return wantarray ? @result : $result[0];
	}
}

=head2 B<Creating and Using Files>

=cut

=head2 ::fill

Fills a volume to a specified amount of data.

=over

=item Arguments

B<max> => I<number>

Optional

Number of MB to write to Volume. Default 1

B<percent> => I<number>

Optional

Percent of volume to fill rounded to the nearest MB, between 1 - 100.
If supplied supercedes the I<max> parameter above

B<proto> => I<nfs> | I<local>

Optional.

Method to create the file, from client (nfs) or mkfile (local). Default nfs

B<type> => I<spares> | I<data>

Optional.

Create spare file or a regular file (data), Default data

=item Examples

=back

	 $vol->fill(max=>10, proto=>"local");

	 $vol->fill(percent=>50, type=>data);

=cut

sub fill {
	my ($self,%args) = @_;
	my $was_mounted = $self->mounted;
	my $percent;
	my $start_time;
	my $total;
	my $log_interval = 5; # seconds
	my $timeout = $args{timeout} || 60;
	my $fname;
	my $maxmegs = $args{max};
	my $proto = $args{proto} || "nfs";
	my $type = $args{type} || "data";
	my $total_secs;

	if ( $args{max} =~ /[tgmbk]$/i ) { # unit specifier?
		# NOTE::units returns KB; convert to MB
		$maxmegs = POSIX::ceil(NOTE::units( $args{max} ) / 1024);
		$maxmegs = 1 if ( $maxmegs == 0 );
	} else {
		$maxmegs = $args{max};
	}
	
	$percent = $args{percent} || 100;
	# The volume will be mounted and permissions changed regardless of the
	# proto. This is done so that any subsequent calls to fill() or refill
	# do not run into "permission denied" issues.
	$self->exportfs( "rw,nosuid,anon=0" );
	unless ( $was_mounted ) {
	    $self->mount( 'chmod' => '777' );
	}
	if ( $proto eq 'local' ) {
		my $fname = $self->fillfile( proto => 'local' );

		VFILER::Util::rm_file( $self->Filer->conn, $fname );

		my ($vol_total,$vol_used,undef,undef) = $self->disk_free; # In KB
		$total = POSIX::ceil((($percent / 100 * $vol_total) - $vol_used) / 1024);
		
		if ($total > $vol_total / 1024){
		    $total = int($vol_total / 1024);
		}
		if ( defined $maxmegs ) {
		    $total = $total > $maxmegs ? $maxmegs : $total;
		}
	    
		if ( $type eq 'data' ) {
		    NOTE::logcomment( "Writing %d MB to $fname", $total );
		    $start_time = time;
		    VFILER::Util::mk_file(
			$self->Filer->conn,
			$total."m",
			$fname,
		    );
	        my $filename = basename($fname);
  	    my $fpath = $self->mountpoint;
		    my $cmd = "/usr/local/bin/sudo chmod 0666 $fpath/$filename";
	        NOTE::logcomment("cmd = $cmd");
		    system( $cmd);
		} elsif ( $type eq 'sparse' ) {
		    NOTE::logcomment( "Creating $total MB file $fname" );
		    my ($path,$name) = $fname =~ /^(.*)\/([^\/]*$)/;
		    my $kilos = $total * 1024;
		    $start_time = time;
		    
		    # Set the timeout to an 15min per 200g
		    my $min_timeout = POSIX::ceil($kilos/(200*1024*1024))*900;;
		    if ($timeout < $min_timeout){
		        $timeout = $min_timeout;
		    }
		    
		    # The unsparse command and therefore Volume::vol_unsparse 
		    # is intolerably unreliable.
		    # As a result, simply log whatever error may have occured
		    # and finish the job via NFS regardless
		    Volume::vol_unsparse( $self->Filer->conn, $path, $name, $kilos.'k', timeout=>$timeout);
		    my $error_during_unsparse = $@;
		    if ($error_during_unsparse){
		        NOTE::logcomment("Error during unsparse: $error_during_unsparse");
		    }
		    else{
		        $error_during_unsparse = 'Do not rely on $@';
		    }
		    # Sometimes the file left by unsparse is still sparse
		    # Detect that condition and truncate the file
		    my $vol_used_by_unsparse = ($self->used - $vol_used)*1024;
		    my $fillfile = $self->fillfile(proto => 'nfs');
		    if ( -f $fillfile ){
		        my @filestat = stat($fillfile);
	            my $size = $filestat[7];
	            if ($size > $vol_used_by_unsparse){
	    	    my $cmd = "/usr/local/bin/sudo chmod 777 $fillfile";
	    	    my $out = `$cmd`;                    
	                NOTE::logcomment("$cmd\n$out");        	    
	                NOTE::logcomment("Truncating file '$fillfile' from $size to $vol_used_by_unsparse");
	                truncate $fillfile, $vol_used_by_unsparse
	                    || croak "Unable to truncate file: $!";
	            }
		    }
	        $@ = $error_during_unsparse;
		}
		# If there was an error, make sure the job is finished by
		# resorting to NFS.
		if ( $@ ) {
		    $fname = $self->fillfile( proto => 'nfs' );
		    system( "/usr/local/bin/sudo chmod 0666 $fname" );
		    # OK, this is just plain ugly:
		    # Unfortunately, if unsparse determines there is not enough room in
		    # the volume, it gives up early (like up to 4g early!). So
		    # here, I compensate by finishing the job with NFS.
		    # When/If unsparse is fixed, this should be removed.
		    $args{'proto'} = 'nfs';
		    $args{'type'}  = 'data';
		    # If a 'max' was entered previously, substract from it what was actually
		    # written so far
		    if ($args{max}){
		        $args{max} -= int(($self->used - $vol_used) / 1024);
		    }
		    $self->fill( %args );
		}
		$total_secs = time - $start_time;
		$total_secs = 1 if ( $total_secs == 0 ); # Prevent divide-by-zero
		NOTE::logcomment( "Wrote %d MB in %d seconds, %.3f MB/s",
		    $total,
		    $total_secs,
		    $total / $total_secs
		);
	} elsif ( $proto eq 'nfs' ) {
		$total = ((($percent/100) * $self->total) - $self->used) / 1024;
		if ( defined $maxmegs ) {
		    $total = $total > $maxmegs ? $maxmegs : $total;
		}

		$fname = $self->fillfile;
		if ( open( F, "+<", $fname ) ) {
		    sysseek( F, 0, 2 ); # seek to end
		    NOTE::logcomment( "Open existing $fname" );
		} else {
		    open( F, ">", $fname ) or croak "Can't open $fname: $!";
		    NOTE::logcomment( "Open new $fname" );
		}

		if ( $total < 0 ) {
		    # Need to shrink filefile
		    my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$size,
			$atime,$mtime,$ctime,$blksize,$blocks)
			= stat(F);
		    
		    my $truncpos = (($size / (1024*1024)) - abs($total)) * (1024*1024);
		    if ( $truncpos < 0 ) {
			NOTE::logcomment( "Can't reduce ".$self->description );
		    } else {
			NOTE::logcomment( "Truncating $fname by %d MB", abs($total) );
			truncate( F, $truncpos );
		    }
		    close F;
		} else {
		    my $megs_to_write = $total;
		    my $total_secs;

		    if ( $type eq 'data' ) {
			my $buf = pack( "C", rand(0xff) ) x (1024*1024);
			NOTE::logcomment( "Writing %d MB to $fname", $megs_to_write );
			$start_time = time;
			my $next_log_time = $start_time + $log_interval;
			my $last_megs = $megs_to_write;
			while( $megs_to_write > 0 ) {
			    if ( time >= $next_log_time ) {
				my $megs_written = $last_megs - $megs_to_write;
				$next_log_time = time + $log_interval;
				NOTE::logcomment( "Wrote %d MB, %.3f MB/s, %.3f percent complete",
				    $total - $megs_to_write, $megs_written / $log_interval,
				    (($total - $megs_to_write) / $total) * 100 );
				$last_megs = $megs_to_write;
			    }
			    eval {
			       local $SIG{ALRM} = sub { die "Timeout writing to $fname\n" };
			       alarm $timeout;
			       syswrite( F, $buf );
			       alarm 0;
			    };
			    croak $@ if ($@);
			    $megs_to_write--;
			}
		    } elsif ( $type eq 'sparse' ) {
			NOTE::logcomment( "Creating $total MB hole in $fname" );
			$start_time = time;
			$megs_to_write = 0;
			sysseek( F, $total * (1024*1024)-1, 2 ); # seek to end + total
			syswrite( F, '\0', 1 ); # Write one byte
		    }
		    close F;
		    $total_secs = time - $start_time;
		    $total_secs = 1 if ( $total_secs == 0 ); # Prevent divide-by-zero
		    NOTE::logcomment( "Wrote %d MB in %d seconds, %.3f MB/s",
			$total - $megs_to_write,
			$total_secs,
			$total / $total_secs
		    );
		}
	}

	unless ( $was_mounted ) {
	    $self->umount;
	}
	return $self;
}

=head2 ::checkfill

Verify that the given file has the amount of data as given by stats
by reading the contents.

=over

=item Arguments

B<fillfh> - I<file handle>

Optional

File handle of the fillfile. Defaults to the internal fillfile

=item Examples

=back

	 $vol->checkfill;

=cut

sub checkfill {
	my ($self,$fillfh) = @_;
	my $was_mounted;
	my $fname;
	my $fh;
	my $total;
	my $buf;
	my $timeout = 60;
	
	if ( $fillfh ) {
		$fh = $fillfh;
		$fname = "<fillfile>";
	} else {
		$was_mounted = $self->mounted;
	
		$self->exportfs( "rw,nosuid,anon=0" );
		unless ( $was_mounted ) {
		    $self->mount( 'chmod' => '777' );
		}
		croak $@ if $@;
		$fname = $self->fillfile;
		open( F, "<$fname" ) or croak "Can't open $fname";
		$fh = *F;
	}

	my $megs = POSIX::ceil((stat($fh))[7] / (1024*1024));
	NOTE::logcomment( "Verifying %d MB in $fname", $megs );
	my $log_interval = 5; # seconds
	my $total_megs_read = 0;
	my $start_time = time;
	my $next_log_time = $start_time + $log_interval;
	my $last_megs = 0;

	eval {
	    local $SIG{ALRM} = sub { die "Timeout reading from $fname\n" };
	    alarm $timeout;
		while( sysread( $fh, $buf, 1024*1024 ) ) {
		    alarm 0;
		    if ( time >= $next_log_time ) {
			my $megs_read = $total_megs_read - $last_megs;
			$next_log_time = time + $log_interval;
			NOTE::logcomment( "Read %d MB; %.3f MB/s, %.3f complete",
			    $total_megs_read, $megs_read / $log_interval, 100 * ($total_megs_read / $megs) );
			$last_megs = $total_megs_read;
		    }
		    $total_megs_read++;
	        alarm $timeout;
		};
		alarm 0;
	};
	croak $@ if ($@);
	my $total_secs = time - $start_time;
	$total_secs = 1 if ( $total_secs == 0 );
	NOTE::logcomment( "Read a total of %d MB at %.3f MB/s in %d seconds from %s",
		$total_megs_read, $total_megs_read/$total_secs, $total_secs, $fname );

	unless( $fillfh ) {
		close F;
		unless ( $was_mounted ) {
		    $self->umount;
		}
	}
	if ( $megs != $total_megs_read ) {
		croak sprintf( "Expected %d MB, but read only %d MB", $megs, $total_megs_read );
	}
	return $self;
}

=head2 ::refill

Recreate the fillfile

=over

=item Arguments

B<fillfh> - I<file handle>

Optional.

File handle of file to recreate. If not supplied the default fillfile is recreated.

=item Examples

=back

	 $vol->refill;

	 $vol->refill(FH);

=cut

sub refill {
	my ($self,$fillfh) = @_;
	my $was_mounted;
	my $fname;
	my $fh;
	my $total;
	my $timeout = 60;
	
	if ( $fillfh ) {
		$total = POSIX::ceil((stat($fillfh))[7] / (1024*1024));
		my $fd = fileno( $fillfh );
		$fname = "<fillfile>";
		open( F, ">", "&".$fd ) or croak "Can't reopen $fname: $!";
		close( $fillfh );
	} else {
		$was_mounted = $self->mounted;
	
		$self->exportfs( "rw,nosuid,anon=0" );
		unless ( $was_mounted ) {
		    $self->mount( 'chmod' => '777' );
		}
		croak $@ if $@;
		$fname = $self->fillfile;
		open( F, "+<$fname" ) or croak "Can't reopen $fname: $!";
		$total = POSIX::ceil((stat(F))[7] / (1024*1024));
	}
	$fh = *F;

	my $log_interval = 5; # seconds
	my $buf = pack( "C", rand(0xff) ) x (1024*1024);
	my $megs_to_write = $total;
	NOTE::logcomment( "Rewriting %d MB to $fname", $megs_to_write );
	my $start_time = time;
	my $next_log_time = $start_time + $log_interval;
	my $last_megs = $megs_to_write;
	while( $megs_to_write > 0 ) {
		if ( time >= $next_log_time ) {
		    my $megs_written = $last_megs - $megs_to_write;
		    $next_log_time = time + $log_interval;
		    NOTE::logcomment( "Rewrote %d MB, %.3f MB/s, %.3f percent complete",
			$total - $megs_to_write, $megs_written / $log_interval,
			(($total - $megs_to_write) / $total) * 100 );
		    $last_megs = $megs_to_write;
		}
		eval {
		   local $SIG{ALRM} = sub { die "Timeout writing to $fname\n" };
		   alarm $timeout;
		   syswrite( F, $buf );
		   alarm 0;
		};
		croak $@ if ($@);
		$megs_to_write--;
	}
	close F;
	my $total_secs = time - $start_time;
	$total_secs = 1 if ( $total_secs == 0 ); # Prevent divide-by-zero
	NOTE::logcomment( "Rewrote %d MB in %d seconds, %.3f MB/s",
		$total - $megs_to_write,
		$total_secs,
		$total / $total_secs
	);
	unless ( $was_mounted ) {
	    $self->umount;
	}

	return $self;
}

=head2 ::unfill

Remove the fillfile from the volume

=over

=item Arguments

None.

=item Example

=back

	 $vol->unfill

=cut

sub unfill {
	my ($self,%args) = @_;

	my $fname = $self->fillfile( proto => 'local' );
	NOTE::logcomment( "Deleting $fname" );
	VFILER::Util::rm_file( $self->Filer->conn, $fname );
	return $self;
}

=head2 ::fillfile

Sets or gets the fillfile name

=over

=item Argument

B<name> => I<string>

Optional.

Name to set the fillfile to. Default I<__FILL__>

=item Example

=back

	 $vol->fillfile;

	 $vol->fillfile(name=>"testfile");

=cut

sub fillfile {
	my ($self,%args) = @_;
	my $newname = $args{'name'};
	my $proto = $args{'proto'} || 'nfs';
	my $fname;

	if ( defined $newname ) {
		$self->{_fillfile} = $newname;
	}
	if ( !$self->{_fillfile} ) {
		$self->{_fillfile} = "__FILL__";
	}
	if ( $proto eq 'nfs' ) {
		# Target must be mounted because otherwise mountpoint is undefined
		if ( $self->mounted ) {
		    $fname = $self->{_mntpoint};
		    $fname .= "/".$self->{_fillfile};
		} else {
		    croak "Tried to access fillfile of ".$self->description.
			  " when not mounted";
		}
	} elsif ( $proto eq 'local' ) {
	    $fname = $self->mount_path."/".$self->{_fillfile},
	}
	return $fname;
}


=head2 ::treecompare

Perform treecompare to perform treecompare between this object
with another NOTE object. Treecompare is done on the filer that the other
object resides on.

Returns 1 if match and croaks if any mismatches found.

=over

=item Arguments

B<other> - I<NOTE object>

The other object to perform treecompare with.

B<%args> - Hash of parameters that filerSystem::treecompare_vol accepts.

=item Examples

=back

	 $vol_src->treecompare($vol_dst);

	 $vol_src->treecompare($vol_dst, args=>"-v");

=cut

sub treecompare {
	my ($self,$other,%args) = @_;
	my $ret;

	$args{timeout} = ($self->Filer->is_a_simulator ? 900 : 300)
		unless $args{timeout};

	# treecompare requires snapmirror license
	$self->Filer->check_license("snapmirror");

	$ret = fileSystem::treecompare_vol(
		$self->Filer->conn,
		$self->name,
		$other->name,
		filer_dst => $other->Filer->conn,
		%args,
	);
	croak $@ if $@;
	return $ret;
}

=head2 ::wreck

Wrapper for L<C<Wreck>|http://web.netapp.com/engineering/ontap_tests/agnostic/docs/lib-Wreck-pm.html> module to provide the wreck functionality to this object.

=over

=item Arguments

B<cmds> = I<string>

A list of comma separated wreck commands to cause the required corruption.
See L<C<wreck cmd>|http://web.nane.netapp.com/mainN/files/man/man1t/na_wreck.1t.html>
for list of valid wreck commands

=item Arguments

B<inject_script> => I<text>

=item Examples

=back

	$vol->wreck(inject_script=>"block_pattern inum=1 bno=2 start=3 end=4 fill=5");

=cut

sub wreck {
	my ($self, %args) = @_;

	NOTE::logcomment("Name ".$self->name.", inject_script:\n".$args{inject_script});
	Wreck::wreck($self->Filer->conn, $self->name, type=>$self->type, %args);
	croak $@ if $@;
	$self->online
		if $self->restricted;
}

=head2 ::sio_proc

Returns the SIO process if one is running for this object else returns undef

=over

=item Example

=back

	my $sio = $vol->sio_proc;

=cut

sub sio_proc {
	my ($self) = @_;

	if ( $self->{_sio} ) {
		return $self->{_sio};
	} else {
		return undef;
	}
}

=head2 ::sio

Provide SIO support. See:
http://perf-web.corp.netapp.com/twiki/bin/view/Perfweb/SIO
for full details of SIO functionality.

The SIO process is continuously 'churned' until terminated by call with 'stop' set
or automatically at Teardown.

The SIO program is expected to be located at /usr/local/test/bin/sio

Returns the SIO object is successful, or 1 if stopping SIO

=over

=item Arguments

B<client> => I<hostspec>

Client to run SIO from. Default localhost or set by a previous call

B<read> => I<percent>

Percent of read access vs write access. Default 100

B<rand> => I<percent>

Percent of random access vs sequential access. Default 100

B<blksize> => I<IO block size>

Size of IO. Default 5k

B<start_byte> => I<integer>


Lower bound for access. Default 0

B<file_size> => I<size>

File size of each file used by sio. Default 20m

B<numthreads> => I<integer>

Number of IO generators. Default 1

B<count> => I<integer>

If specified determines number of files to create for sio to use. Default 5

B<file_list> => I<file list>

List of space separated files for SIO to access.
If not specified, <count> files of <file_size> size will be created for sio to use.
This override <count> option.

B<cycle> => I<integer>

How long to run sio before repeating, in seconds. A special value of zero indicates
only create data files and not to start sio process. Default 60

B<stop> => I<1|0>

Stop the SIO process if set. Default 0

B<restart> => I<integer>

Flag to indicate to start sio with parameters from previous invokation.

=item Examples

=back

	 # Default:  run 'read' sio on localhost against 5 files of 20m each
	 $vol->sio();

	 # Create file for SIO to use then return without starting sio
	 $vol->sio(count=>2, cycle=>0);

	 # create 3 files of 10m each for sio to use with 2 threads
	 $vol->sio(numthreads=>2, count=>3, file_size=>"10m");

	 # Tell sio to use listed files on client nam9
	 $vol->sio(client=>"nam9", file_list=>"f1 f2");

	 # Perform sio on src (25% read, 75% write) and dst (100% read) of a snapmirror relationship
	 $src_vol->sio(read=>25);
	 $sm->initialize(src=>$src_vol, dst=>$dst_vol);
	 $dst_vol->sio();

	 # Stop sio
	 $vol->sio(stop=>1);

=cut

sub sio {
	my $self = shift;
	my (%args) = validate (@_, {
				cycle => {type=>SCALAR, optional=>1},
				client => {type=>SCALAR, optional=>1},
				read => {type=>SCALAR, optional=>1, default=>100, regex=>qr/\d/},
				rand => {type=>SCALAR, optional=>1, default=>100, regex=>qr/\d/},
				blksize => {type=>SCALAR, optional=>1, default=>"50k"},
				start_byte => {type=>SCALAR, optional=>1, default=>0, regex=>qr/\d/},
				count => {type=>SCALAR, optional=>1, default=>5, regex=>qr/\d/},
				file_size => {type=>SCALAR, optional=>1},
				numthreads => {type=>SCALAR, optional=>1, default=>1, regex=>qr/\d/},
				file_list => {type=>SCALAR, optional=>1},
				stop => {type=>SCALAR, optional=>1, default=>0},
				restart => {type=>SCALAR, optional=>1, default=>0}

			});

	croak "sio 'stop' and 'restart' options are mutually exclusive"
		if $args{stop} && $args{restart};

	if ( $args{stop} ) {
		if ( $self->{_sio} ) {
			NOTE::logcomment("Stopping SIO for ".$self->description);
			$self->{_sio}->raise("TERM");
			_sio_stop_file($self, "create");
			sleep(5); # Pause a little to give sio chance to exit
			my $retry = 0;
			while ( $self->{_sio}->is_alive ) {
				croak "Unable to stop SIO for ".$self->description
					." after $retry attempts. Aborting..."
					unless $retry++ < 30;
				NOTE::logcomment("Send kill signal to SIO for ".$self->description);
				$self->{_sio}->kill;
				sleep(5);
			}
			$self->{_sio}->wait_finish(timeout=>120);
	    	FSR::FSRutils::subtestResults($self->{_sio}) or croak $@;
			delete $self->{_sio};
			_sio_stop_file($self, "remove");
			NOTE::logcomment("SIO process terminated");
		} else {
			croak "No SIO process running";
		}
		return 1;
	}

	croak ("sio already running...") if $self->{_sio};

	$args{client} = $args{client} || $self->{_client} || "localhost";
	$args{file_size}= ($self->Filer->is_a_simulator ? "2" : "20")."m"
			unless defined $args{file_size};
	croak "numthreads must be > 0"
		unless $args{numthreads} > 0;

	if ( $args{restart} && exists $self->{_sio_args} ) {
		NOTE::logcomment("Restore previous sio parameters");
		delete $args{restart};
		foreach my $key (keys %{$self->{_sio_args}}) {
			$args{$key} = $self->{_sio_args}{$key};
		}
	}
	%{$self->{_sio_args}} = %args;

	my $rw_mnt = undef;
	my $fname = $args{file_list};
	unless ( $fname ) {
		# XXX 'fill' method works on localhost only
		$rw_mnt = _rw_mount_check($self, "localhost");
		my $idx = 1;
		do {
			my $sio_file = "sio_".$idx."_".$$;
			my $fullpath = $self->mountpoint."/".$sio_file;
			if ( $rw_mnt && ! -e $fullpath ) {
				$self->fillfile(name=>$sio_file);
				$self->fill(proto=>'nfs', max=>$args{file_size});
			};
			croak "Unable to locate sio data file $sio_file"
				unless -e $fullpath;
			$fname .= " " if $fname;
			$fname .= $sio_file; # Add to sio 'dev' list
		} while $idx++ < $args{count};
		$self->umount unless $self->{_client} =~ $args{client};
	}

	return 1 if defined $args{cycle} && $args{cycle} <= 0; # Create and return data files only, do not start sio

	# Mount for SIO access
	$rw_mnt = _rw_mount_check($self, $args{client});
	# XXX SIO -stop file option only works for linux host, so use short 60 cycle
	#     unless this is the case
	my $stop_cmd = "";
	$stop_cmd = "-stop -"._sio_stop_file($self, "set")
		if $rw_mnt && Tharn::host($args{client})->hosttype =~ /linux/i;
	$args{cycle} = ($stop_cmd ? 900 : 60) # Default to long cycle time if able to use stop file
		unless defined $args{cycle};

	my $cmd = "rsh ".$args{client}." \"cd ".$self->mountpoint."; /usr/software/test/bin/sio"
			." $args{read} $args{rand} $args{blksize} $args{start_byte}"
			." $args{file_size} $args{cycle} $args{numthreads} $fname $stop_cmd"
			."\"";
	my $op = "Running: $cmd";
	NOTE::logcomment($op." (".$self->tag.")");
	my $ret = Tharn::subtest(sub {
					close STDIN;

					my ($rc, $run) = (0, 1);
					$SIG{TERM} = sub { 
							NOTE::logcomment("Stopping ".$self->description." SIO");
							$run = 0;
						};
					# continuous SIO loop until stopped by parent
					NOTE::logcomment("Starting ".$self->description. " SIO");
					do {
						NOTE::logcomment("Starting ".$self->description. " SIO yet again, run: $run");
						$rc = system($cmd);
					} while $run && ! $rc;
					Tharn::logresult("FATAL", "Attempt at ".$self->description
							." SIO failed: rc = $rc")
						if $rc > 0;
					NOTE::logcomment("SIO ".$self->description." stopped");
				}, '-bg', '-onexit', sub {});
	croak "Fail to start SIO: $@"
		if $@;
	$self->{_sio} = $ret;
	# Schedule SIO to be stopped during teardown:
	NOTE::reference( sub { $self->sio( stop => 1 ) if exists $self->{_sio} } );
	return $ret;
}

sub _rw_mount_check {
	my ($self, $client) = @_;

	NOTE::logcomment("Remount to determine r/w access to Volume/Qtree");
	$self->umount if $self->mounted;
	my $exp = ",nosuid,anon=0";
	eval {
		# Attempt r/w mount
		$self->mount(client=>$client, exportfs=>"rw".$exp, chmod=>"777");
	};
	if ( $self->mounted ) {
		return 1; # to indicate r/w mount
	} else {
		$self->mount(client=>$client, exportfs=>"ro".$exp);
		return 0; # to indicate r/o mount
	}
}

sub _sio_stop_file {
	my ($self, $mode) = @_;

	if ( $mode =~ /set/i ) {
	    # Stop file name was $self->tag."_sio_stop_".$$ earlier
	    # Changed 'tag' to 'name' due to Burt 486104
		$self->{_sio_stop_file} = $self->name."_sio_stop_".$$;
	} else {
		return undef unless defined $self->{_sio_stop_file};

		my $fullpath = $self->mountpoint."/" .$self->{_sio_stop_file};

		my $cmd = "rsh ".$self->{_client}." "
			.($mode =~ /create/i ? "touch" : "rm")
			." $fullpath";
		if ( $mode =~ /create/i ) {
			# Don't abort if stop file creation failed
			system($cmd);
			NOTE::logcomment("SIO stop file ".$self->{_sio_stop_file}." created")
				if -e $fullpath;
		}
	
		if ( $mode =~ /remove/i && -e $fullpath ) {
			# Attempt to remove stop file if exists
			if ( -e $fullpath ) {
				system($cmd);
				croak "Failed to removed stop file ".$self->{_sio_stop_file}
					if -e $fullpath;
			}
			delete $self->{_sio_stop_file}; # No need to save stop after
		}
	}

	return $self->{_sio_stop_file};
}

use DataManip qw();
use FSR::FSRutils;

=head2 ::populate

Populate volume with data using DataManip::populate_tree or RichData::populate.

=over

=item Arguments

B<richdata> => C<0 | 1>

I<Optional.>
If set, use RichData library instead of DataManip.
The default is the value set on L<C<::create>|/::create>
or L<C<::attach>|/::attach>.
Subsequent calls to either L<C<::populate>|/::populate>
or L<C<::modify>|/::modify> will use this setting as default.

B<%args> - I<dataManip|RichData argument Hash>

Optional.

All parameters, in the form of a hash is passed to DataManip::populate_tree
or RichData::populate for processing.

=item Examples

=back

	  $vol->populate;

	  $vol->populate("FILE_DEV" => 100, "DIR_DEPTH" => 3);

	  # Use RichData::populate to create streams, sparse files, links, etc.
	  # in addition to files and directories
	  $vol->populate(richdata => 1);

=cut

sub populate {
	my ($self,%args) = @_;

	# Allow populate/modify/compare/delete_tree to be bypassed:
	return $self if ( NOTE->DATAMANIP == 0 );

	my $was_mounted = $self->mounted;
	my ($percent, $ret, $fullpath);
	my $path = $self->pwd;
	my %ARGS = ();

	if ( exists $args{richdata} ) {
	    # Overwrite the richdata option for the object
	    $self->{_richdata} = $args{richdata};
	    delete $args{richdata};
	}

	my $client = $args{client} || $self->{_client} || "localhost";
	delete $args{client};

	$ARGS{uc $_} = $args{$_} for ( keys %args );

	# Communicate filer type to DataManip
	$ARGS{ENVIRONMENT_TYPE} ||= $self->Filer->is_a_simulator ? "simulator" : "filer";

	$self->exportfs( "rw,nosuid,anon=0" );
	unless ( $was_mounted ) {
	    $self->mount( 'chmod' => '777', client => $client );
	}

	my $op = "Populating $path of ".$self->description." from client $client";
	NOTE::logcomment( $op );
	$fullpath = $self->{_mntpoint}.$path;
	if ( $self->{_richdata} ) {
	    delete $args{background};
	    $args{client} = $client;
	    $ret = Tharn::subtest(
	        sub {
	            eval { RichData::populate(path => $fullpath, %args) };
	            Tharn::logresult('FATAL', "EVAL_ERROR=$@") if ($@);
	        }, '-bg', '-onexit', sub {}
	    );
		$ret->wait_finish unless $ARGS{BACKGROUND};
	    FSR::FSRutils::subtestResults($ret) or croak $@;
	} else {
		# Increase data generation for hardware filers only unless specified by caller
		$ARGS{DIR_AVG} = 4 unless (defined $ARGS{DIR_AVG} || $self->Filer->is_a_simulator);
	    $ret = DataManip::populate_tree(
		    $client,
		    $fullpath,
		    %ARGS
	    );
	    croak $@ if $@;
	}

	if ( ref $ret eq 'Subtest' ) {
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	} else {
		$ret = $self;
	}

	unless ( $was_mounted ) {
		if ( $ARGS{BACKGROUND} ) {
		    # Don't unmount if in background because it might
		    # still be running. Warn unless anonymous, because named volume
		    # mounts don't get cleaned up on teardown
		    NOTE::logcomment( "Warning: ".$self->name.
			" was not unmounted because populate was run in the background"
		    ) unless ( $self->{_anonymous} );
		} else {
		    $self->umount
		}
	}
	return $ret;
}

=head2 ::modify

Modify data using DataManip::modify_tree or RichData::modify.

=over

=item Arguments

B<richdata> => C<0 | 1>

I<Optional.>
If set, use RichData library instead of DataManip.
The default is the value set on L<C<::create>|/::create>
or L<C<::attach>|/::attach>.
Subsequent calls to either L<C<::populate>|/::populate>
or L<C<::modify>|/::modify> will use this setting as default.

B<%args> - I<dataManip|RichData argument Hash>

Optional.

All parameters, in the form of a hash is passed to DataManip::modify_tree
or RichData::modify for processing.

=item Examples

=back

	  # Modify volume using either DataManip or RichData according to
	  # the setting of the richdata flag on populate, create or attach
	  $vol->modify;

=cut

sub modify {
	my ($self,%args) = @_;

	# Allow populate/modify/compare/delete_tree to be bypassed:
	return $self if ( NOTE->DATAMANIP == 0 );

	my $was_mounted = $self->mounted;
	my ($percent, $ret, $fullpath);
	my $path = $self->pwd;
	my %ARGS = ();

	if ( exists $args{richdata} ) {
	    # Overwrite the richdata option for the object
	    $self->{_richdata} = $args{richdata};
	    delete $args{richdata};
	}

	my $client = $args{client} || $self->{_client} || "localhost";
	delete $args{client};

	$ARGS{uc $_} = $args{$_} for ( keys %args );

	# Communicate filer type to DataManip
	$ARGS{ENVIRONMENT_TYPE} ||= $self->Filer->is_a_simulator ? "simulator" : "filer";

	$self->exportfs( "rw,nosuid,anon=0" );
	unless ( $was_mounted ) {
	    $self->mount( 'chmod' => '777', client => $client );
	}

	my $op = "Modifying $path of ".$self->description." from client $client";
	NOTE::logcomment( $op );
	$fullpath = $self->{_mntpoint}.$path;
	if ( $self->{_richdata} ) {
	    delete $args{background};
	    $args{client} = $client;
	    $ret = Tharn::subtest(
	        sub {
	            eval { RichData::modify(path => $fullpath, %args) };
	            Tharn::logresult('FATAL', "EVAL_ERROR=$@") if ($@);
	        }, '-bg', '-onexit', sub {}
	    );
		$ret->wait_finish unless $ARGS{BACKGROUND};
	    FSR::FSRutils::subtestResults($ret) or croak $@;
	} else {
	    $ret = DataManip::modify_tree(
		    $client,
		    $fullpath,
		    %ARGS
	    );
	    croak $@ if $@;
	}

	if ( ref $ret eq 'Subtest' ) {
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	} else {
		$ret = $self;
	}

	unless ( $was_mounted ) {
		if ( $ARGS{BACKGROUND} ) {
		    # Don't unmount if in background because it might
		    # still be running. Warn unless anonymous, because named volume
		    # mounts don't get cleaned up on teardown
		    NOTE::logcomment( "Warning: ".$self->name.
			" was not unmounted because modify was run in the background"
		    ) unless ( $self->{_anonymous} );
		} else {
		    $self->umount
		}
	}
	return $ret;
}

=head2 ::compare

Calls dataManip::compare_tree to compare this object with the 'other' object

Returns results of DataManip::compare_tree

=over

=item Arguments

B<other> - I<NOTE object>

The other NOTE object to compare with

B<%args> - I<argument hash>

Optional

Hash of arguments passed to dataManip::compare_tree for processing.

=item Examples

=back

	 $vol_src->compare($vol_dst);

=cut

sub compare {
	my ($self,$other,%args) = @_;

	# Allow populate/modify/compare/delete_tree to be bypassed:
	return $self if ( NOTE->DATAMANIP == 0 );

	my $self_mounted = $self->mounted;
	my $other_mounted = $other->mounted;
	my $path_self = $self->pwd;
	my $path_other = $other->pwd;
	my %ARGS = ();

	$ARGS{uc $_} = $args{$_} for ( keys %args );

	$self->exportfs( "rw,nosuid,anon=0" );
	$self->mount unless $self_mounted;
	$other->mount unless $other_mounted;

	my $op = "Comparing\n" .
	         "         $path_self of " . $self->description . "\n" .
	         "    with $path_other of " . $other->description;
	NOTE::logcomment( $op );
	my $ret = DataManip::compare_trees(
		"localhost",
		$self->{_mntpoint}.$path_self,
		$other->{_mntpoint}.$path_other,
		%ARGS
	);
	croak $@ if $@;

	if ( ref $ret eq 'Subtest' ) {
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	} else {
		$ret = $self;
	}

	for my $volinfo ( [$self,$self_mounted], [$other,$other_mounted] ) {
		my ($vol,$was_mounted) = @{$volinfo};
		next if $was_mounted;
		if ( $ARGS{BACKGROUND} ) {
		    NOTE::logcomment( "Warning: ".$vol->name.
			" was not unmounted because compare was run in the background"
		    ) unless ( $vol->{_anonymous} );
		} else {
		    $vol->umount;
		}
	}
	return $ret;
}

=head2 ::delete

Calls DataManip::delete_tree to delete data from the 'pwd' of this object

=over

=item Arguments

None.

=item Examples

=back

	 $vol->delete;

=cut

sub delete {
	my ($self,%args) = @_;

	# Allow populate/modify/compare/delete_tree to be bypassed:
	return $self if ( NOTE->DATAMANIP == 0 );

	my $was_mounted = $self->mounted;
	my $path = $self->pwd;
	my %ARGS = ();

	$ARGS{uc $_} = $args{$_} for ( keys %args );

	$self->exportfs( "rw,nosuid,anon=0" );
	unless ( $was_mounted ) {
	    $self->mount( 'chmod' => '777' );
	}
	my $op = "Deleting $path of " . $self->description;
	NOTE::logcomment( $op );
	my $ret = DataManip::delete_tree(
		"localhost",
		$self->{_mntpoint}.$path,
		%ARGS
	);
	croak $@ if $@;

	if ( ref $ret eq 'Subtest' ) {
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	} else {
		$ret = $self;
	}

	unless ( $was_mounted ) {
		if ( $ARGS{BACKGROUND} ) {
		    # Don't unmount if in background because it might
		    # still be running. Warn unless anonymous, because named volume
		    # mounts don't get cleaned up on teardown
		    NOTE::logcomment( "Warning: ".$self->name.
			" was not unmounted because delete was run in the background"
		    ) unless ( $self->{_anonymous} );
		} else {
		    $self->umount
		}
	}
	return $ret;
}

=head2 ::dataset

Name of the dataset of this object

=over

=item Arguments

None.

=item Example

=back

	 $vol->dataset;

=cut

sub dataset { return shift->{_dataset} }

=head2 ::asis

Run asis on the volume

=over

=item Arguments

B<state> - I<on|off|start|stop>

Turn asis on, off, start or stop. Default on

B<sched> - I<string>

Optional

If set will have this schedule for ASIS operation. 

schedule is [day_list][@hour_list]
      or [hour_list][@day_list]
      or -
      or auto

If sched=>default, schedule is "sun-sat@0". If not set, schedule is "sun-sat@0-23@~0-59". 

=item Example

=back

	 $vol->asis(sched=>"sun-sat\@1-6"); This means everyday between 1am-6am.

=cut

sub asis {
        my ($self, %args) = @_;
        my $sched = (exists $args{sched} ? $args{sched} : "sun-sat\@0-23\@~0-59");
        my $state = (exists $args{state} ? lc($args{state}) : "on");
        my $hostname = $self->{_Filer}->host;
        my $pathname = $self->mount_path;

        if($sched =~ /~/) {
            NOTE::logcomment('Schedule being used is $sched : set an option sis.minutely_schedule');
            $self->{_Filer}->registry->{'options.sis.minutely_schedule'} = 'on';
        }

        # Set up A-SIS license
	if ($self->Filer->wafl_version < Filer::Version::WV_SIERRANEVADA_MIN ) {
            if (Dense::dense_is_licensed(host=>$hostname)) {
                NOTE::logcomment("ASIS is already licensed");
            } else {
                Dense::dense_set_license(host=>$hostname);
                croak $@ if $@;
            }
	}

        my %status = Dense::dense_status(host=>$hostname, path=>$pathname);
        my $my_status = $status{$pathname}{Status};
        if ( $state =~ /start/i ) {
            Dense::dense_start(host=>$hostname, path_name=>$pathname, scan=>1)
                if $my_status =~ /idle/i;
        } elsif ( $state =~ /stop/i ) {
            Dense::dense_stop(host=>$hostname, path_name=>$pathname)
                if $my_status =~ /active/i;
        } else {
            # Enable A-SIS on the volume
            Dense::dense_toggle(host=>$hostname, path_name=>$pathname, state=>$state);
            croak $@ if $@;

            # Set schedule on the volume
            if ($state =~ /on/i && $sched !~ /default/i){ 
                Dense::dense_config(host=>$hostname, path_name=>$pathname, schedule_string=>$sched, test_mode=>1);
            }
        }
        croak $@ if $@;
}

=head2 ::copy

Perform copy data from the supplied object to this object

=over

=item Arguments

B<src> - I<NOTE object>

Src of data to be copied

B<background> - I<Boolean>

Optional

If set will perform copying in the background. Default 0

=item Examples

=back

	 $vol->copy(src=>$vol_src);

=cut

sub copy {
	my ($self,%args) = @_;
	my $background = delete $args{background};
	my $src = delete $args{src};
	my %ARGS = ();
	my $was_restricted = $self->restricted;

	croak "src must be specified for copy" unless ( $src );
	
	my $op = "Copying\n" .
	         "    +---" . $src->description . "\n" .
	         "    +-> " . $self->description;
	NOTE::logcomment( $op );
	$ARGS{uc $_}    = $args{$_} for ( keys %args );
	$ARGS{COMPLETE} = $background ? 0 : 1;
	$ARGS{SOURCE}   = $src->hostpath;

	$self->restrict unless ( $was_restricted );
	my ($ret, $op_num) = Volume::Copy::volcopy_start(
		$self->Filer->conn,
		$self->name,
		%ARGS,
	);
	croak $@ if $@;

	if ( ref $ret eq 'Subtest' ) {
	    push @{$self->{_subtests}}, { operation => $op, subtest => $ret };
	} else {
		$ret = $self;
	}

	unless ( $was_restricted ) {
		my $try = 0;
		for $try ( 1..5 ) {
		    eval { $self->online };
		    last unless ( $@ );
		    sleep 1;
		}
		if ( $try >= 5 ) {
		    croak "Couldn't bring ".$self->name." online after volcopy: $@";
		}
		$self->exportfs; # Sync object's exportfs state with filer's
	}

	if ( wantarray ) {
	    return ($ret, $op_num);
	} else {
	    return $ret;
	}
}

=head2 ::store

Invoke 'snapmirror store' to backup volume data to tape

Returns a string representing the backup reference number.

=over

=item Arguments

B<device> - I<name>

Name of tape device to dump data to.

B<background> - I<Boolean>

Optional

If set returns immediately after starting backup.

=item Examples

=back

	 $vol->store(device=>"rst01");

=cut

sub store {
	my ($self,%args) = @_;
	my $ref;
	my $device = $args{device};
	my $background = $args{background} || 0;
	delete $args{background};
	delete $args{device};

	unless ( $device ) {
		croak "snapmirror store requires device name";
	}
	NOTE::logcomment( "Storing ".$self->description." to ".$device );

	$ref = SnapMirror::snapmirror_store(
		host     => $self->Filer->hostrec,
		vol      => $self->name,
		dumpfile => $device,
		%args,
	);
	croak $@ if ( $@ );
	if ( $background == 0 ) {
		SnapMirror::snapmirror_tape_status(
		    host => $self->Filer->hostrec,
		    vol  => $self->name,
		    type => "store",
		    wait => 1,
		    timeout => $args{timeout} || 900,
		);
		NOTE::logcomment( "Store complete; reference is $ref" );
	} else {
		NOTE::logcomment( "Store started; reference is $ref" );
	}
	return $ref;
}

=head2 ::retrieve

Invokes 'snapmirror retrieve' to restore data from a tape device

=over

=item Argument

B<device> - I<name>

Name of tape device holding data to be restore

B<background> - I<Boolean>

If set returns immediately upon starting the restore.
When ran in background mode, use SnapMirror::snapmirror_tape_status to
monitor restore's progress.

=item Examples

=back

	 $vol->retrieve(device=>"rst01");

=cut

sub retrieve {
	my ($self,%args) = @_;
	my $device  = $args{device};
	my $timeout = $args{timeout} || 900;
	my $background = $args{background} || 0;
	delete $args{device};
	delete $args{timeout};
	delete $args{background};

	unless ( $device ) {
		croak "snapmirror retrieve requires device name";
	}

	NOTE::logcomment( "Restoring ".$self->description." from ".$device );

	$self->restrict;
	SnapMirror::snapmirror_retrieve(
		host        => $self->Filer->hostrec,
		vol         => $self->name,
		restorefile => $device,
		%args,
	);
	croak $@ if ( $@ );
	if ( ! $background ) {
		SnapMirror::snapmirror_tape_status(
		    host    => $self->Filer->hostrec,
		    vol     => $self->name,
		    type    => "retrieve",
		    wait    => 1,
		    timeout => $timeout,
		);
		NOTE::logcomment( "Restore complete" );
	} else {
		NOTE::logcomment( "Restore started" );
	}
	return $self;
}

=head2 ::pwd

Returns the current working directory for the calling process.

=over

=item Arguments

None.

=item Example

=back

	 $curdir = $vol->pwd;

=cut

sub pwd { my $path = shift->{_wdir}; return $path ? $path : '/' }

=head2 ::cd

Sets the current working directory for the calling process.

=over

=item Arguments

I<string>

I<Required.>
A path name.

=item Example

=back

	 $vol->cd( '../testdir2' );

=cut

sub cd {
	my  ($self,$newdir) = @_;

	# Turn newdir into absolute path
	$newdir = '/' unless ( $newdir );
	unless ( $newdir =~ /^\// ) {
		$newdir = $self->pwd.'/'.$newdir;
	}
	unless ( $newdir =~ /\/$/ ) {
		$newdir .= '/';
	}
	# Collapse multiple '/'s to one '/'
	$newdir =~ s/[\/]{2,}//g;

	NOTE::logcomment( "Changing working directory to $newdir for ".$self->description );
	$newdir = undef if ( $newdir eq '/' );
	$self->{_wdir} = $newdir;
	return $self;    
}

=head2 snap_reserve

Get/Set the snap reserve for a particular volume

=over

=item Argument

I<percent>

Optional

If present, sets the snap reserve to the percentage specified
and returns the previous setting.

If not present, returns the current setting.

=item Examples

=back

 $vol->snap_reserve( 0 ); # Set no snap reserve

 my $save = $vol->snap_reserve( 20 ); # Set 20 percent snap reserve
 $vol->snap_reserve( $save ); # Restores to original setting.

 printf( "Snap reserve for %s is %d percent\n",
		$vol->description, $vol->snap_reserve );

=cut

sub snap_reserve {
	my ($self,$reserve) = @_;

	my $current = Volume::vol_snap_reserve( $self->Filer->conn, $self->name );
	croak $@ if $@;
	if ( defined $reserve ) {
	    Volume::vol_snap_reserve( $self->Filer->conn, $self->name, reserve => $reserve );
	    croak $@ if $@;
	}
	return $current;
}

=head2 snapshot_sched

Get/Set the snapshot sched for the volume

=over

=item Argument

I<schedule>

Optional

If present, sets the snapshot sched to the specified

If not present, returns the current setting.

=item Examples

=back

 @current_sched = $vol->snapshot_sched; # returen the current sched on $vol 

 $vol->snapshot_sched(sched => "0 0 0"; # set the snap sched on $vol to 0 0 0 
=cut

sub snapshot_sched {
		my ($self,%args) = @_;

		if ( !defined $args{sched} ) {
			my @current = SnapCmds::snapsched_volume($self->Filer->conn, $self->name);
			return @current;
		} else {
			SnapCmds::snapsched_volume($self->Filer->conn, $self->name, $args{sched} );
			croak $@ if $@;
		}
}


=head2 snapshot_create

Create a snapshot on the volume

=over

=item Argument

B<snapname> - I<name>

Optional

If not set then a anonymous name is used to create the snapshot with

=item Examples

=back

	 $vol->snapshot_create;

	 $vol->snapshot("special");

=cut

sub snapshot_create {
	my ($self,$snapname) = @_;

	$snapname = $self->SUPER::newName( "sn" ) unless( $snapname );

	NOTE::logcomment( "Creating snapshot ".$snapname." of ".$self->description );
	SnapCmds::snapshot_create(
	    $self->Filer->conn,
	    $self->name,
		$snapname,
	);
	croak $@ if ($@);
	return $snapname;
}

=head2 ::snapshot_restore

Restored a named snapshot

=over

=item Arguments

B<snapname> - I<name>

Name of the snapshot to restore

=item Example

=back

         $vol_snapshot_restore("special");
         $vol_snapshot_restore("special","DISABLE");
         $vol_snapshot_restore("special","ENABLE");
=cut

sub snapshot_restore {
        my ($self,$snapname,$sis_stat,%opts) = @_;

        croak "Need snapshot name" unless ( $snapname );
        NOTE::logcomment( "Restoring ".$self->description." using snapshot ".$snapname );

        my ($sis_active,$out,$cmd,$disable_sis);
        if ($sis_stat){
            unless ($sis_stat =~ /DISABLE|ENABLE/){
                croak "Invalid parameter to method";
                }
            $disable_sis = ($sis_stat =~ /DISABLE/) ? 1 : 0;
            }

        #Checking whether volume is dense volume
        $cmd = "priv set -q test; vol status ".$self->name;
        $out = $self->Filer->say($cmd);
        my $toggle_assis = 1 if ($out =~ /sis/i);

        #If dense volume, Checking whether SIS is active on the volume
        #If SIS Active, stopping SIS on the volume
        if ($toggle_assis){
            $cmd = "priv set -q test; sis status -l /vol/".$self->name;
            $out = $self->Filer->say($cmd);
            if (($out =~ /Status:\s+Active/) or ($out =~ /Status:\s+Pending/)){
                $sis_active=1;
                NOTE::logcomment("SIS is active OR pending on the Volume : ".$self->name);
                NOTE::logcomment("Stopping SIS on the volume : ".$self->name);
                $cmd = "priv set -q test; sis stop /vol/".$self->name;
                $out = $self->Filer->say($cmd);
                my $vol_name = "/vol/".$self->name;
                unless(( $out =~ /Operation is currently idle/) || ($out =~ /Operation was no longer pending.*/) || ($out =~ /The operation on \"$vol_name\" is being stopped/) ){
                    croak "SIS is still active on the Volume";
                    }
            }

            #Waiting for SIS status on the volume to be Idle
            $cmd = "priv set -q test; sis status -l /vol/".$self->name;
            do{
                $out = $self->Filer->say($cmd);
            }while ($out !~ /Status:\s+Idle/);
            NOTE::logcomment("SIS is Idle  on the Volume : ".$self->name);
         }

        #Performing Snaprestore
        $self->Filer->check_license( "snaprestore" );
        SnapCmds::snaprestore_volume(
            $self->Filer->conn,
            $self->name,
                $snapname,
                %opts,
        );
        croak $@ if ($@);

        #If SIS was active in beginning of snap restore, restart SIS on the volume.
        if ($toggle_assis){
            if ($sis_active && (!$disable_sis)){
                $self->asis(state=>"start");
                }
            }
        return $self;
        }

=head2 ::snapshot_list

Returns an array of the snapshots that exists on this volume

=over

=item Arguments

B<prefix> - I<string>

Optional

 the prefix of the snapshot name that you'd like to list

=item Example

=back

	 $vol->snapshot_list;
	 $vol->snapshot_list("SnapVaultNBU");

=cut

sub snapshot_list {
	my ($self, $prefix) = @_;
	my @list;

	my @snap_list = SnapCmds::snapshot_list(
	    $self->Filer->conn,
	    $self->name
	);
	croak $@ if ($@);

	if($prefix) {
		foreach my $snapshot(@snap_list) {
			if ($snapshot =~ /^$prefix/ ) {
				push (@list, $snapshot);
			}
		}
	} else { 
		@list = @snap_list;
	}

	return @list;
}

=head2 ::snapshot_status

Returns a reference to a hash of the status with the
snapshot names as keys.

=over

=item Arguments

None

=item Example

=back

	 $vol-snapshot_status;

=cut

sub snapshot_status {
	my $self = shift;
	my %status = ();
	my @status;

	@status = SnapCmds::snapshot_status(
		$self->Filer->conn,
		$self->name
	);
	croak $@ if ($@);

	for ( @status ) {
		$status{$_->{name}} = $_;
	}
	return \%status;
}

=head2 ::snapshot_exists

Returns a boolean to indicate whether a given named snapshot exists or not

=over

=item Arguments

B<snapshot> - I<name>

Name of the snapshot to check

=item Example

=back

	 $vol->snapshot_exists('special');

=cut

sub snapshot_exists {
	my ($self,$snapshot) = @_;
	my $ret;

	$ret = SnapCmds::snapshot_exists(
		$self->Filer->conn,
		$self->name,
		$snapshot
	);
	croak $@ if ($@);
	return $ret;
}

=head2 ::snapshot_delete

Delete a named snapshot

=over

=item Arguments

B<snapshot> - I<name>

Name of the snapshot to delete

=item Example

=back

	  $vol->snapshot_delete('special');

=cut

sub snapshot_delete {
	my ($self,$snapshot,%args) = @_;

	NOTE::logcomment( "Deleting snapshot ".$snapshot );

	SnapCmds::snapshot_delete(
		$self->Filer->conn,
		$self->name,
		$snapshot,
		# %args
	);
	croak $@ if ($@);
	return $self;
}

=head2 ::snap_delta

Returns the output of the snap delta command for the volume specified, 
in an array of hash references. Keys are taken from header.
Wrapper around Volume::vol_snap_delta.

=over

=item Arguments

B<beginning_snapshot> - I<name>

I<Optional> Name of the beginning_snapshot for the delta

B<ending_snapshot> - I<name>

I<Optional> Name of the ending_snapshot for the delta

=item Examples

=back

      $vol->snap_delta();
      $vol->snap_delta('beginning_snapshot' =>'snap1');
      $vol->snap_delta('beginning_snapshot' =>'snap1', 
                       'ending_snapshot' => 'snap2');

=cut

sub snap_delta {
    my ($self,%args) = @_;
    my $ret;

    $ret = Volume::vol_snap_delta(
	    $self->{_Filer}->conn,
            $self->{_name},
            %args
            );
    croak $@ if $@;
    return $ret;
}

=head2 ::split

After the split of the syncmirrored volume (*not* flexclone), 
a NOTE::Volume would be returned to caller for the split-off volume.

=over

=item Arguments

B<plex> - I<name>

Name of the plex to split off. Default last plex.

B<newvol> - I<name> 

Optional

New volume name. Default create an anonymous name.

=item Example

=back

	 $vol->split;

=cut

sub split {
	my ($parent,$plex,$newvol,%args) = @_;
	my $self = {};

	if ($parent->type eq 'flex') {
		croak "Cannot split a flexvol. Only applicable to a syncmirrored trad".
		  "vol.";
	}

	$parent->Filer->check_license( "flex_clone" );
	$self->{_debug}    = $args{debug};
	$self->{_owner}    = $$;
	$self->{_dataset}  = $parent->{_dataset};
	$self->{_tag}      = "SyncMirrored of ".$parent->description;
	$self->{_recycle}  = $args{recycle};
	delete $args{recycle};

	NOTE::logcomment( "Splitting ".$parent->description );
	if ( $newvol ) {
		$self->{_name} = $newvol;
		$self->{_anonymous} = $args{anonymous};
		delete $args{name};
		delete $args{anonymous};
	} else {			
		# Create a unique name
		$self->{_name} = NOTE::Volume->newName;
		$self->{_anonymous} = 1;
	}
	my $exportfs_opts = $args{exportfs} || "rw,nosuid,anon=0";
	delete $args{exportfs};

	$self->{_type}      = $parent->type;
	$self->{_Filer}	= $parent->Filer;

	# automatically grab the last plex as the one to be split off
	if ( !$plex ) {
		my $status = Volume::vol_raidgroup_status( 
	        $parent->Filer->conn,
		    $parent->name 
	    ) or croak $@;

		foreach my $plex_info ( @{$status->{plex}} ) {
		    $plex = $plex_info->{name};
		}
	}

	Volume::vol_split( $parent->Filer->conn, 
			       $plex,
			       $self->{_name} );
	croak $@ if $@;

	bless $self, ref($parent);
	NOTE::reference( $self );
	tie %{$self}, 'NOTE::Registry::Slice', $self, $regSlice;

	$self->exportfs( $exportfs_opts );

	return $self;
}

=head2 ::clone

Create a clone of this volume

=over

=item Arguments

All parameters are passed to NOTE::Volume::Clone->create;

=item Example

=back

	 $vol->clone;

=cut

sub clone {
	my $self = shift;
	return NOTE::Volume::Clone->create( $self, @_ );
}

=head2 ::clones

Returns clones for this volume

=over

=item Arguments

None.

=item Example

=back

	 @list = $vol->clones;

=cut

sub clones {
	my $self = shift;
	my @list = Volume::Clone::get_clones(
		conn => $self->Filer->conn,
		vol => $self->name,
	);
}

=head2 ::Clones

Returns a list of NOTE::Volume objects that are clones of this volume

=over

=item Argument

None.

=item Example

=back

	 @vol_objs = vol->Clones;

=cut

sub Clones {
	my $self = shift;
	my @objList = ();

	for my $name ( $self->clones ) {
		my $clone = NOTE::Volume::Clone->attach(
		    Filer => $self->Filer,
		    name => $name,
		);
		push @objList, $clone;
	}
	return @objList;
}


=head2 ::wafliron_start

Initiates wafliron on this volume if it is a trad vol, and on the containing
aggregate if a flex volumes.

=over

=item Arguments

B<%args> - I<hash>

Hash of arguments for Wafliron::wafliron_start

=item Example

=back

	  $vol->wafliron_start;

=cut

sub wafliron_start {
	my ($self,%args) = @_;

	return  ( $self->type eq 'flex'
		? $self->Aggregate->wafliron_start( %args )
		: NOTE::Aggregate::wafliron_start($self, %args)
	);
}

=head2 ::wafliron_stop

Aborts a wafliron running for this trad volume, or the containing
aggregate for a flex volume.

=over

=item Arguments

B<%args> - I<hash>

Hash of parameters for Wafliron::wafliron_stop

=item Example

=back

	 $vol->wafliron_stop;

=cut

sub wafliron_stop {
	my ($self,%args) = @_;

	return  ( $self->type eq 'flex'
		? $self->Aggregate->wafliron_stop( %args )
		: NOTE::Aggregate::wafliron_stop($self, %args)
	);
}

=head2 ::wafliron_status

Returns the current status of the wafliron running on this trad vol
or the containing aggregate for a flex vol

=over

=item Arguments

None.

=item Example

=back

	 $status = $vol->wafliron_status;

=cut

sub wafliron_status {
	my $self = shift;

	return  ( $self->type eq 'flex'
		? $self->Aggregate->wafliron_status()
		: NOTE::Aggregate::wafliron_stop($self)
	);
}

=head2 ::wafliron_wait

Wait for wafliron to complete

=over

=item Arguements

None.

=item Examples

=back

	 $vol->wafliron->wait;

=cut

sub wafliron_wait {
	my $self = shift;

	return  ( $self->type eq 'flex'
		? $self->Aggregate->wafliron_wait()
		: NOTE::Aggregate::wafliron_wait($self)
	);
}

=head2 B<Waiting>

=cut

=head2 ::wait

Waits for all background processes to complete.
These are the processes created by using the B<background> option of
L<C<::populate>|/::populate>, L<C<::modify>|/::modify>,
L<C<::compare>|/::compare>, or L<C<::delete>|/::delete>.

=over

=item Arguments

None.

=item Example

=back

	 $vol->wait;

=cut

sub wait {
	my $self = shift;
	my ($obj, $test, $op);

	while ( $test = pop @{$self->{_subtests}} ) {
	    $op  = $test->{operation};
	    $obj = $test->{subtest};
	    if ( $obj->is_alive() ) {
	        NOTE::logcomment( "Waiting for: ".$op );
	    }
	    $obj->wait_finish();
	    FSR::FSRutils::subtestResults($obj) or croak $@;
	}
	return $self;
}

=head2 B<Related Objects>

=cut

=head2 ::Filer

Returns an object representing the filer containing the volume.

=over

=item Arguments

None.

=item Example

=back

	 $filer = $vol->Filer;

=cut

sub Filer {
	my $self = shift;
	return $self->{_Filer} if ( $self->{_Filer} );
	return $self->{_Aggregate}->Filer if ( $self->{_Aggregate} );
	return undef;
}

=head2 ::Aggregate

Returns an object representing the aggregate
containing the volume.

=over

=item Arguments

None.

=item Example

=back

	 $aggr = $vol->Aggregate;

=cut

sub Aggregate { 
	my $self = shift;
	if ($self->type eq 'flex') {
		return $self->{_Aggregate};
	} else {
		croak "This traditional volume does not have an aggregate";
	}
}

=head2 ::container

A synonym for L<C<::Aggregate>|/::Aggregate>.

=cut

sub container { 
	return shift->Aggregate;
}

=head2 B<Object Identification>

=cut

=head2 ::tag

A tag is a text description for a filer entity.
It is only stored in the object referring to the entity.
The purpose of the tag is to improve readability of the log.
If an object is tagged,
NOTE will include the tag in comments it logs about the object.

Returns the tag for the object.

=over

=item Arguments

I<string>

I<Optional.>
A tag to store in the object.

=item Examples

=back

	 $vol->tag( 'source' );
	 $which = $vol->tag;

=cut

sub tag {
	my ($self,$newtag) = @_;

	if ( defined $newtag ) {
		$self->{_tag} = $newtag;
	}
	return $self->{_tag};
}

=head2 ::description

Returns a description of the volume consisting of
its filer's name, its mount path, and its tag, if any.

=over

=item Arguments

None.

=item Example

=back

	 $desc = $vol->description;

=cut

sub description {
	my $self = shift;
	my $txt;

	$txt = $self->hostpath;
	if ( $self->{_tag} ) {
		$txt .= " (".$self->{_tag}.")";
	}
	return $txt;
}


=head2 ::is_recycled

Returns a true if a volume was recycled
and false if it was physically created.

=over

=item Arguments

None.

=item Examples

=back

	  my $vol = NOTE::Volume->create( recycle => 1 );
	  if ( $vol->is_recycled ) {
	      printf( "This volume was recycled\n" );
	  } else {
	      printf( "This volume was created\n" );
	  }

=cut

sub is_recycled { return shift->{_was_recycled} }

=head2 ::is_recyclable

Returns 1 if the volume is a recyclable volume.
This is also assignable, if it is desired to change the
status.


=over

=item Arguments

None.

=item Examples

	if ( $vol->is_recyclable ) {
		printf( "Volume %s is recyclable\n", $vol->name );
	}

	...

	$vol->is_recycleable = 1;

=back

=cut

sub is_recyclable :lvalue { shift->{_recycle} };

=head2 ::is_free

Returns a boolean to indicate whether this volume is in the free pool

=over

=item Arguments

None.

=item Eample

=back

	for my $vol ( $filer->Volume ) {
		next unless ( $vol->is_free );
		printf( "Volume %s is available for reuse\n", $vol->name );
	}

=cut

sub is_free { return shift->{_name} =~ /^_vl/ };

=head2 ::is_anonymous

Returns true if this volume is anonymous.  This can also be
assigned to, in order to make a volume anonymous.

=over

=item Argument

None.

=item Example

=back

	 if ( $vol->is_anonymous ) {
	     ...
	 }

	 ...

	 $vol->is_anonymous = 1;

=cut

sub is_anonymous :lvalue { shift->{_anonymous} };

=head2 ::exists

Returns a boolean indicating is this volume exists on the filer

=over

=item Arguments

None.

=item Example

=back

	 $vol->exists;

=cut

sub exists {
	my $self = shift;
	return Volume::vol_exists( $self->Filer->conn, $self->name );
}

=head2 ::mirror

For trad volumes it mirrors volume using syncmirror if not already mirrored. For flex volumes
you should mirrors aggregate instead.

If syncmirror is licensed then it simply runs vol mirror command. If syncmirror is not licensed
then this method will license syncmirror_local service and will reboot filer because it's
required by syncmirror. Volume will be mirrored after reboot. Relies on cluster::VolUtils::vol_mirror.

=over

=item Arguments

B<timeout> = seconds

I<Optional> sync timout in seconds, default is 7200 seconds (2 hrs).

=item Examples

    $vol->mirror;

    $vol->mirror( timeout=>300 );

=back

=cut

sub mirror {
    my ( $self, %args ) = @_;
    my $sync_timeout = $args{timeout} ? $args{timeout} : 7200;
    return if( $self->is_mirrored );
    return cluster::VolUtils::vol_mirror( fc => $self->Filer->{_conn}, vol => $self->name, timeout => $sync_timeout );
}

=head2 ::is_mirrored

Checks if volume is syncmirrored. Returns 1 if it's mirrored.

=over

=item Arguments

None

=item Examples

    if( $vol_a->is_mirrored ) {
	$vol_b = $aggr_a->split;
    } else {
	croak "Unable to split volume, volume is not mirrored";
    }

=back

=cut

sub is_mirrored { 
    my $self = shift;
    return $self->type eq 'flex' ? $self->Aggregate->is_mirrored : ($self->status)[1] =~ /mirrored/;
}

=head2 ::newName

Returns a new name

=over

=item Argument

None.

=item Example

=back

	  $vol->newName;

=cut

sub newName { return shift->SUPER::newName( "vl" ) }

=head2 ::nfs

Returns an NFS object for this volume

=over

=item Arguments

None.

=item Example

=back

	 $nfs = $vol->nfs;

=cut

sub nfs {
	my ($self,%args) = @_;
	$self->{_NFS} ||= NOTE::FileSystem::NFS::File->init( $self, '/', %args );
	# The following makes it possible to unmount the filesystem..
	# Subsequent calls to 'nfs' cause it to be remounted again..
	# $qt->nfs->ls( '-lR' );
	# $qt->umount;
	# $qt->nfs->ls( '-lR' );
	$self->mount unless ( $self->mounted );
	return $self->{_NFS};
}

=head2 ::preserve

Create a snapshot that represents the volume as it was
 when it was first created. If the volume is recycled,
 this will be reverted to when the volume is put back
 into the free pool.

=over

=item Arguments

None.

=item Example

=back

	 $vol->preserve

=cut

sub preserve {
	my $self = shift;
	my ($name, $writable) = ($self->name, $self->writeable);

	if( $self->writeable ) {
		if ( $self->snapshot_exists( "__clean__" ) ) {
		    $self->snapshot_delete( "__clean__" );
		}
		$self->snapshot_create( "__clean__" );
	}
	return $self;
}

=head2 ::clean

Restore volume to pristine state
Returns 0 if no __clean__ snapshot exists

=over

=item Argument

None.

=item Example

=back

	 $vol->clean

=cut

sub clean {
	my $self = shift;

	if ( $self->writeable && $self->snapshot_exists( "__clean__" ) ) {
		$self->snapshot_restore( "__clean__" );
		croak $@ if $@;
		return 1;
	}
	return 0;
}

=head2 ::SPARE

Spare Volume from recycling

=over

=item Argument

None.

=item Example

=back

	 $vol->SPARE

=cut

sub SPARE {
	my $self = shift;
	$self->{_recycle}  = 0;
	$self->{_spare_me} = 1;
	if ( $self->{_type} eq "flex" ) {
		$self->Aggregate->SPARE;
	}
	return $self;
}

=head2 ::release

Release a volume when it is no longer needed by a script.
This means put it in the recyclable pool or destroy
it, as appropriate.

=over

=item Argument

None.

=item Example

=back

  # The following will recycle a new volume each time
  # through the loop, although it is likely to get the
  # same volume each time.
  for ( 0..10000 ) {
	  my $trad = NOTE::Volume->create( type => 'trad' );
	  $trad->fill( max => '500m' );
	  $trad->release;
  }

=cut

sub release {
	my $self = shift;
	$self->_release;
	$self->unreference; # Remove the volume's reference from teardown stack
	return $self;
}

=head2 ::meta_i2p

Return hashref of meta_i2p data for the volume,
with fileids as keys, and filenames as values.

=over

=item Arguments

None.

=item Examples

=back

     $data = $vol->meta_i2p();

=cut

sub meta_i2p {
    my $self = shift;
    my $data = Volume::vol_meta_i2p($self->Filer->conn(), $self->name());

    if (!$data) {
        croak 'Cannot fetch inode->pathname mapping for volume ' . $self->name();
    }
    return $data;
}

=head2 ::compliance_clock

Return volume compliance or system compliance clock time.

=over

=item Arguments

B<clock> = vcc | scc

I<Optional> If 'vcc' volume compliance clock will be returned, default is vcc
if 'scc' system compliance clock will be returned.

B<option> = verbose

I<Optional> if option is provied, hashref of compliance clock details will be returned.

=item Examples

=back

     $time = $vol->compliance_clock(
                        clock => 'scc',
                        option => 'verbose',
                );
=cut
sub compliance_clock {
    my ( $self , %args ) = @_;
    my $volume = $self->name();
    my $secure_clock = $args{clock} ? $args{clock} : "vcc";

    if ( ($secure_clock != 'vcc') or ($secure_clock != 'scc') ) { croak 'Invalid secure clock type' ;  }
    if ($secure_clock eq 'scc' ) { $volume = undef; }

    if ( !defined $args{option} ) {
        my $cc_time = snaplock::compliance_clock_status($self->Filer->conn(), $volume);
        if ( !$cc_time ) {
            croak 'Could not get compliance clock time for volume' . $volume;
        }
        return $cc_time;
    }elsif (( $args{option} ) && ( $args{option} eq 'verbose')) {
        if ( $secure_clock eq 'scc') { $volume = $self->Filer->vol_root; }
        my $data = snaplock::compliance_clock_status_verbose($self->Filer->conn(), $volume);
        if (!$data) {
            croak 'could not get compliance time for volume' . $volume;
        }
        return $data;
    } else  {
                croak 'Invalid option';
    }
}


=head2 ::playlist

Wrapper for L<C<Volume::Playlist>|http://web.netapp.com/engineering/ontap_tests/agnostic/docs/lib-Volume-Playlist-pm.html>.

=over

=item Arguments

None.

=item Examples

=back

     $str = $vol->playlist->dump();        # Returns contents of volume's playlist as a hashref
     $hashref = $vol->playlist->status();  # Returns volume's playlist status as a string
     $vol->playlist->copy_in($path);       # Copies a playlist file in from $path
     $vol->playlist->copy_out($path);      # Copies volume's playlist file out to $path

=cut

sub playlist {
    my $self = shift;

    if (!$self->{_playlist}) {
        $self->{_playlist} = Volume::Playlist->new($self->Filer->conn(), $self->name());
    }
    return $self->{_playlist};
}

=head2 ::wafl_touch

Expands buftree by generating place holder. This command is supported on only **DEBUG** builds.

=over

=item Arguments

B<inode_num> => I<integer>

I<Required>
Inode number of the file

B<private_node> => C<0> | C<1>

I<optional>
Sets to 1 , If inode is private inode. defaults to 0.

B<init_bad_block> => C<0> | C<1>

I<optional>
Defaults to 0.

B<level> => C<0> | C<1> | C<2> | C<3> | C<4>

I<optional>
Applicable values range from 0 to 3.Defaults to 0.

B<start_fbn> => I<start_of_fbn_number>
Beginning of fbn

I<optional>
Defaults to 0.

B<end_fbn> => I<end_of_fbn_number>
End of fbn

I<Required>

B<skip_fbn> => I<fbn_to_be_skipped>
fbn which user wants to skip.

I<optional>

=item Examples

=back

$volume->wafl_touch(inode_num=>"96",private_node=>"1",init_bad_block=>"1",level=>"4",start_fbn=>"0",end_fbn=>"4",skip_fbn=>"4");

=cut

sub wafl_touch(%args) {
    my ($self,%args) = @_;

    $self = shift;
    unless ( (defined $args{inode_num}) && (defined $args{level}) && (defined $args{start_fbn}) && (defined $args{end_fbn}) ){
	croak "Insufficient parameters to wafl touch command";
	}
    
    ##It runs only on debug builds
    if (!filer::filer_kernel_is_debug($self->{_Filer}->conn)){
	croak "This command requires debug build\n";
	}

    my $vol_name = $self->name;
    my $inode_num = $args{inode_num};
    my $level = $args{level};
    my $start_fbn = $args{start_fbn};
    my $end_fbn = $args{end_fbn};

    my $private_inode = " -private" if ($args{private_inode});
    my $init_bad_block = " -init_badblock" if ($args{init_bad_block});
    my $skip_fbn = $args{skip_fbn} if exists $args{skip_fbn};

    my $text = sayEOF($self->{_Filer}->conn,"priv set -q test; wafl touch -V $vol_name $inode_num $private_inode $init_bad_block $level $start_fbn $end_fbn $skip_fbn");
    if ($text =~ /wafl touch succeeded/i){
        NOTE::logcomment("Wafl touch succeeded: $text");
        return;
        }
    else {
	croak "Wafl touch failed: $text\n";
        }
    }


=head2 ::dumpblock_info

Dumps block information for a volume

=over

=item Arguments

B<inode_num> => I<integer>

I<Required>
Inode number of the file

B<block_num> => I<integer>

I<Required>
Block number.

B<private_node> => C<0> | C<1>

I<optional>
Sets to 1 , If inode is private inode. defaults to 0.

B<file_id> => I<integer>

I<Required>
File id on a volume.

B<level> => C<0> | C<1> | C<2> | C<3> | C<4>

I<optional>
Applicable values range from 0 to 4.Defaults to 0.

B<snap_id> => I<snapshot_id>

I<optional>
Snapshot id.

=item Examples

=back

$volume->dumpblock_info(fileid=> <File_ID>, Block_num => <Block_number>) ;

=cut

sub dumpblock_info(%args) {
    my ($self,%args) = @_;

    $self = shift;
    if ( (!defined $args{file_id}) || (!defined $args{block_num}) ){
	croak "Insufficient parameters to dumpblock info command";
	}

    my $vol_name = $self->name;
    my $file_id = $args{file_id};
    my $block_num = $args{block_num};

    my $private_inode = " -private" if exists $args{private_inode};
    my $level = $args{level} if exists $args{level};
    my $snap_id = $args{snap_id} if exists $args{snap_id};

    my $text = sayEOF($self->{_Filer}->conn,"priv set -q diag; dumpblock info -V $vol_name $file_id $private_inode $block_num $level $snap_id");
    if ($text =~ /block not found/i){
        NOTE::logresult("WARN","Block not found\n");
        return 1;
        }
    return 1;
    }

=head2 ::upgrade_64bit

Upgrades a volume from 32 bit to 64 bit

=over

=item Arguments

B<mode> => <String>

I<Optional>
Defaults to normal. Applicable values are normal & check.

=item Examples

=back

$volume->upgrade_64bit();

=cut

sub upgrade_64bit() {
    my ($self,%args) = @_;

    my ($mode,$filer,$vol_name,$count,$text);
    ##If mode is not specified, default it to normal mode	
    $mode = exists $args{mode} ? $args{mode} : "normal";
    $filer = $self->{_Filer};
    $vol_name = $self->name();
    $count = 0;

    ##Trigger 64 bit upgrade on volume 
    NOTE::logcomment("Upgrading volume $vol_name to 64 bit\n");
    $filer->say("priv set -q diag; vol status $vol_name");
    $text = $filer->say("priv set -q test;vol 64bit-upgrade start $vol_name -mode $mode");

    if ($text =~ /Checking for additional space required to upgrade/i){
	##If volume is %100 full, throw a warning on check
        if($mode =~ /check/i){
            if ($text =~ /increase the volume size/i){
                NOTE::logcomment("WARN: Volume $vol_name do not have enough free space: $text\n");
                return 1;
            }
            else{
                NOTE::logcomment("Output of check on volume before 64 bit upgrade:\n $text\n");
		return 1;
                }
            }
	else {
	    if($mode =~ /normal/i){
                ##If volume is %100 full, upgrade fails
                if ($text =~ /do not have enough free space to support the 64-bit upgrade/i) {
                    croak "64 bit upgrade on volume $vol_name failed: $text\n";
                    }
                ##aggr 64 bit upgrade is not supported in trad vols
                elsif ($text =~ /Invalid operation on traditional volume/i) {
                    croak "64 bit upgrade is not supported on traditional volume: $text\n";
                    }
                }

	    ##Verfying whether upgrade succeeded
            if ($text =~ /Started 64-bit upgrade on volume $vol_name/i) {
                NOTE::logcomment("*** 64-bit upgrade started on volume $vol_name");
                do {
                    if($count > 2){
                        croak "64-bit upgrade on volume $vol_name failed\n";
                        }
                    sleep 5;
                    $text = $filer->say("priv set -q diag; vol status $vol_name");
                    $count++;
                    }while($text !~ /64-bit/i);
                NOTE::logcomment("64-bit upgrade on volume $vol_name succeeded\n");
                }
           else {
               croak "Failed to start upgrading volume to 64 bit\n Error String: \n $text \n";
               }
           }
        }
        else {
                croak "Upgrade on volume $vol_name failed\n $text\n";
             }
        return 1;
      }


=head2 sd_test_differ

Run the test-mode C<sd test_differ> command on a node. Triggers a series of
WAFL_SNAP_DIFF messages to diff the named file in the given snapshot.

Returns an array reference of records that are the parsed lines from the
command, or a message string if there are no diffs to parse. Each record is a
hash reference with the keys C<fbn>, C<vbn>, C<count> and C<type>.

=over

=item Arguments

B<path> = string

File path to compare in the diffing operation

B<snap> = string

Snapshot directory under C<.snapshot> to compare to in the diff operation

B<opts> = hash table

I<Optional> hash table of arguments to the C<test_differ> command:

=over

=item B<base_snap> = string

Base snapshot to diff against

=item B<abort_on_corruption> = boolean

Whether to set the C<REP_OK> flag on the B<test_differ> command, which
causes it to abort upon detection of corruption. Default is 0 (false).

=item B<baseline_hole_info> = boolean

Without B<base_snap>, whether to return hole information. Default is 0 (false).

=item B<get_vbn> = boolean

Whether to retrieve VBNs for changed blocks. Default is 0 (false).

=item B<l1_diff> = boolean

Whether to diff only at L1 level. Default is 0 (false).

=item B<start_fbn> = integer

First FBN to process.

=item B<end_fbn> = integer

Last FBN to process.

=item B<max_fbns> = integer

Limit on change records per message. Default is 512.

=item B<expand_lun_clone> = boolean

Whether to expand LUN clones. Default is 0 (false).

=back

=item Examples

To check the changes to a file in the volume against a snapshot:

    # Assume $file is already created on the volume
    $volume->snapshot_create('snap1');
    # Make changes to $file
    my $change_recs = $volume->sd_test_differ($file, 'snap1');
    # The records in @$change_recs should match changes to $file

To check the changes to a file between two specific snapshots:

    # Assume $file is already created on the volume
    $volume->snapshot_create('snap1');
    # Make changes to $file
    $volume->snapshot_create('snap2');
    my $change_recs = $volume->sd_test_differ(
        $file, 'snap2',
        base_snap => 'snap1'
    );
    # The records in @$change_recs should match changes to $file

=back

=cut

sub sd_test_differ {
    my ($self, $path, $snap, @opts) = @_;
    my $cmd = 'sd test_differ';

    # Validate the params. Fairly easy here, as all are optional. Use
    # the opportunity to set default values to the flags, so that we
    # don't need "defined" tests later on...
    my %opts = validate(
        @opts => {
            base_snap => {
                optional => 1,
                type     => SCALAR,
            },
            abort_on_corruption => {
                optional => 1,
                type     => SCALAR,
                default  => 0,
            },
            baseline_hole_info => {
                optional => 1,
                type     => SCALAR,
                default  => 0,
            },
            get_vbn => {
                optional => 1,
                type     => SCALAR,
                default  => 0,
            },
            l1_diff => {
                optional => 1,
                type     => SCALAR,
                default  => 0,
            },
            start_fbn => {
                optional => 1,
                type     => SCALAR,
                regex    => qr/^\d+$/,
            },
            end_fbn => {
                optional => 1,
                type     => SCALAR,
                regex    => qr/^\d+$/,
            },
            max_fbns => {
                optional => 1,
                type     => SCALAR,
                regex    => qr/^\d+$/,
            },
            expand_lun_clone => {
                optional => 1,
                type     => SCALAR,
                default  => 0,
            },
        }
    );

    # Build up the args to the filer-level "sd test_differ" command
    my $args = q{};
    $args .= ' -r' if ($opts{abort_on_corruption});
    $args .= ' -c' if ($opts{expand_lun_clone});
    $args .= " -e $opts{end_fbn}" if (defined $opts{end_fbn});
    $args .= ' -h' if ($opts{baseline_hole_info});
    $args .= " -m $opts{max_fbns}" if (defined $opts{max_fbns});
    $args .= " -s $opts{start_fbn}" if (defined $opts{start_fbn});
    $args .= ' -t' if ($opts{l1_diff});
    $args .= ' -v' if ($opts{get_vbn});

    if (defined $opts{base_snap}) {
        my $base_path = sprintf '/vol/%s/.snapshot/%s/%s' =>
            $self->name, $opts{base_snap}, $path;
        $args .= " -b $base_path";
    }

    my $incr_path = sprintf '/vol/%s/.snapshot/%s/%s' =>
        $self->name, $snap, $path;
    $args .= " -i $incr_path";

    # XXX: In this form, this has no hope of working on a huge file.
    # We need to parse the output line-by-line as it comes back.
    my $text = $self->Filer->say("priv set -q test ; $cmd$args");

    # Parse and validate the header (first line of output). Use regexp
    # matching/substitution to avoid creating copies of $text, since it is
    # likely to be HUGE.
    $text =~ s/^(.*)(?:\n|\z)//;
    my $header = $1;

    # Look to see if we got a diagnostic line stemming from the use of the
    # "max_fbns" option. One of the two possibilities is actually an error
    # case:
    if ($header =~ /max_fbns too large, using \d+/) {
        # Slice out the record-lines from the remaining text so that only
        # diagnostic output remains. We'll want to provide all of this in the
        # error message when we croak.
        $text =~ s/^Start\s+FBN.*(?:\n|\z)//gm;
        croak "$cmd failed (max_fbns range): $header\n$text";
    }
    if ($header =~ /max_fbns set to \d+/) {
        # We ignore this one. Just re-load $header with the next line from
        # $text.
        $text =~ s/^(.*)(?:\n|\z)//;
        $header = $1;
    }

    # We passed the error-case, and if the second matched we reloaded $header.
    # Check that the header we got matches the parameters we passed. The
    # header should match this line:
    #
    # Using: start_fbn=0, end_fbn=0, need_vbn=true, differ_type=wsd, max_fbns=512
    #
    # Numbers may vary, need_vbn may be "true" or "false", and differ_type
    # may be "wsd" or "test".
    my ($start, $end, $get_vbn, $type, $max);
    if ($header =~ m{
                        ^ # Start of string
                        Using:\s+
                        start_fbn=(\d+),\s+        # $start
                        end_fbn=(\d+),\s+          # $end
                        need_vbn=(true|false),\s+  # $get_vbn
                        differ_type=(wsd|test),\s+ # $type
                        max_fbns=(\d+)\s*          # $max
                        $ # End of string
                    }x) {
        ($start, $end, $get_vbn, $type, $max) = ($1, $2, $3, $4, $5);
    } else {
        croak "Error parsing '$cmd' header: $header";
    }
    if (! ($start == ((defined $opts{start_fbn}) ? $opts{start_fbn} : 0) &&
           $end == ((defined $opts{end_fbn}) ? $opts{end_fbn} : 0) &&
           $get_vbn eq ($opts{get_vbn} ? 'true' : 'false') &&
           $type eq ($opts{l1_diff} ? 'test' : 'wsd') &&
           $max == ((defined $opts{max_fbns}) ? $opts{max_fbns} : 512))) {
        croak "header from '$cmd' did not match input parameters";
    }

    # Build a regexp to check for an empty record list by enumerating
    # the different messages that could signal that. This approach makes
    # it easier to extend the list without having an explicit RE that is
    # a pain to read.
    my @empty_record_messages = (
        'Files are identical',
        'The file is of 0 length',
        'The file is completely holey',
    );
    my $empty_record_re = join q{|} => @empty_record_messages;
    # Now, check for an empty record list.
    if ($text =~ /^($empty_record_re)$/) {
        return $text;
    }

    # Check whether the command explicitly passed on an error.
    if ($text =~ /^WAFL error: (.*?)$/m) {
        die "$1\n";
    }

    # Parse each line of output and accumulate the list of records. The
    # logic in the while() loop may look odd, but it is more efficient
    # than splitting the string into lines, as that would allocate a
    # potentially huge array of strings.
    #
    # Lines look like this:
    #
    # Start FBN: 3986, VBN: 13341, Count: 20, DATA_CHANGED
    #
    # Numbers may vary, and the last element may be one of several different
    # text values.
    my @records = ();
    while ($text ne q{} && $text =~ s/\A(.*?)(?:\n|\z)//) {
        my $line = $1;
        if ($line =~ m{
                          ^                  # Start of line
                          Start\s+
                          FBN:\s+(\d+),\s+   # Key "fbn"
                          VBN:\s+(\d+),\s+   # Key "vbn"
                          Count:\s+(\d+),\s+ # Key "count"
                          ([A-Z_]+)\s*       # Key "type"
                          $                  # End of line
                      }x) {
            push @records, { fbn => $1, vbn => $2, count => $3, type => $4 };
        } else {
            croak "error parsing '$cmd' output: $line";
        }
    }
    if (! @records) {
        croak "no records returned from '$cmd'";
    }

    return \@records;
}

=head2 ::storage_compression

Enable Storage Compression on filers having DATA ONTAP version >= RR.0

=over

=item Arguments

B<preprocess> => I<1|0>

I<Optional.>
Boolean flag to indicate whether preprocessing of
admin commands such as adding license should be performed. Default: 0

B<compression_value> => I<true|false>

I<Optional.>
Boolean flag to enable/disable the storage compression. Default: true

B<compress_data> => I<1|0>

I<Optional.>
Option which compresses AFS data(-C) during 'sis start'. Default: 1

B<blocks_shared> => I<1|0>

I<Optional.>
Option which includes the block shared(-a) during 'sis start'. Default: 0

B<AFS_blocks> => I<1|0>

I<Optional.>
Option which includes AFS blocks trapped in snapshots(-b) during 'sis start'.
Default: 0

B<force> => I<1|0>

I<Optional.>
Option which suppresses the prompt(-f) during 'sis start'. Default: 1

B<del_exist_chkpts> => I<1|0>

I<Optional.>
Option which deletes the existing checkpoint(-d) during 'sis start'. Default: 0

B<inline_compression> => I<true|false>

I<Optional.>
Boolean flag to enable/disable the inline storage compression(sis config -I).
Value will be set w.r.t user's input.

B<sleep> => I<number>

I<Optional.>
Number of seconds to sleep in between polling the filer's status.
The default is 5 seconds.

B<timeout> => I<number>

I<Optional.>
Maximum time allowed for commands to complete, in seconds.
The default is 600;

=item Examples

=back

     $Vol1->storage_compression;
     $Vol1->storage_compression(inline_compression => 'true');
     $Vol1->storage_compression(compress_data => 0,
                                blocks_shared => 1,
                                AFS_blocks    => 1,
                                del_exist_chkpts => 1
                               );

=cut

sub storage_compression{
    my ($self,%args) = @_;
    if ($self->Filer->wafl_version < Filer::Version::WV_ROLLINGROCK_MIN){
        croak "This is applicable to releases only beyond RR.0";
    }

    if($args{preprocess}){
        $self->Filer->license('nearstore_option');
        $self->Filer->license('a_sis');
        $self->Filer->license('compression');
    }

    $args{compression_value} = 'true' if ! exists $args{compression_value};
    my $compress_data = exists $args{compress_data} ? $args{compress_data} : 1;
    $compress_data = $compress_data == 1 ? '-C' : '';
    my $blocks_shared = $args{blocks_shared} == 1 ? '-a' : '';
    my $AFS_blocks = $args{AFS_blocks} == 1 ? '-b' : '';
    my $force = exists $args{force} ? $args{force} : 1;
    $force = $force == 1 ? '-f' : '';
    my $del_exist_chkpts = $args{del_exist_chkpts} == 1 ? '-d' : '';

    $args{timeout} = 600 if ! exists $args{timeout};
    $args{sleep}   = 5 if ! exists $args{sleep};
    my $mount_path = $self->mount_path;

    my $sis = NOTE::Sis->new(Volume => $self);
    $sis->on();

    my $say_output = $self->Filer->say("sis config -C $args{compression_value} %s",$mount_path);
    if($say_output ne '') {
        croak "Compression is not configured on the volume $mount_path";
    }

    $say_output = $self->Filer->say("sis config -I $args{inline_compression} %s",$mount_path) if exists $args{inline_compression};
    if($say_output ne '') {
        croak "Inline Compression is not configured on the volume $mount_path";
    }

    my $config_options ="-s $compress_data $blocks_shared $AFS_blocks $force $del_exist_chkpts ";
    $say_output = $self->Filer->say("priv set test;sis start $config_options %s",$mount_path);
    if ($say_output !~ /The\s+SIS\s+operation\s+for\s+"$mount_path"\s+is\s+started/) {
        croak "Sis is not started on the volume $mount_path";
    }

    $sis->idle_wait(sleep => $args{sleep},timeout => $args{timeout});
}

sub DESTROY {
	my $self = shift;

	return if ( $$ != $self->{_owner} );
	return if ( $self->{_destroyed} );
	return if ( NOTE->ALL_SPARED );

	if ( $self->{_spare_me} ) {
		NOTE::logcomment( "Volume ".$self->{_name}." has been spared" );
		return;
	}
	$self->release;
}

#
# This is an internal function that does all that is necessary
# to free a volume when it is no longer needed by a script.
#
sub _release {
	my $self = shift;

	return if ( $$ != $self->{_owner} );
	return if ( ! $self->{_anonymous} );
	return if ( $self->{_destroyed} );
	$self->{_destroyed} = 1;
	return if ( ! $self->{_anonymous} );

	# Code below only applies to anonymous volumes:
	$self->sio(stop=>1) if $self->sio_proc(); # stop any sio processes before umounting...
	$self->umount; # unmount ONLY anonymous volumes.
	if ( NOTE->RECYCLE && $self->{_recycle} ) {
		# Don't bother recycling anonymous flex volumes that are empty
		unless ( $self->type eq 'flex' && !$self->dataset ) {
		    eval { $self->unexportfs }; # Don't leave recyclable volumes in exports list

		    # Bring it online, in case it's offline.
		    eval { $self->online };
		    if ( $@ && $@ =~ /was left in an inconsistent/ ) {
			NOTE::logcomment( "Volume ".$self->{_name}." cannot be recycled" );
			eval { $self->offline }; # Might already be offline
			$self->destroy( force => 1 ); # No choice but to destroy the volume.
			return;
		    }
		    if ($self->{_asis}) {
	            $self->_asis_shutdown;
	        }
		    # Only non-anonymous volumes are preserved and recycled
		    # If the volume still isn't writeable, let it be destroyed.
		    if ( $self->writeable ) {
			$self->clean;
			NOTE::logcomment( "Returning Volume ".$self->{_name}." to free pool" );
			$self->rename( "_".$self->name );
			return;
		    }
		    goto done;
		}
	}
	if (($self->{_asis}) && ($self->type eq 'flex')) { 
	     eval { $self->online };
	     unless ($@) {
	            $self->_asis_shutdown;
	     }
	}
done:
	eval { $self->offline }; # Don't care if this fails
	$self->destroy( force => 1 );
}

=head2 ::dd

Executes the unix dd command to create/modify a particular file on the volume

=over

=item Arguments

B<output_file> = <file_name>

I<Required> Creates this file on the volume. Or if file already exists modifies
specified number of blocks of that file

B<count> = <number_of_blocks>

I<Required> Number of blocks to write

B<input_file> = <file_name>

I<Optional> Input file on client from where the blocks are read. By default,
'/dev/zero' file is taken as input file

B<seek> = <offset>

I<Optional> offset to modify the blocks. By default, offset is considered as 
zero.

B<block_size> = <block_size>

I<Optional> Block Size. Default is 4096 bytes.

B<conv> = <keyword>

I<Optional> Conv keyword for dd command. By default, it is 'notrunc'.

=item Examples

=back
    
    $vol->dd (output_file => 'file1,
                      count       =>  4081,);
                      
    $vol->dd (output_file => 'file1,
                      count       =>  2000,
                      seek        =>  2000);                      
    
=cut

sub dd {
     my ($self,%args) = @_;
     
    # Check if all required parameters are present.
    if (!exists($args{output_file}) || !exists($args{count})){
        croak 'Output_file and count are required parameter';
    }

    # Mount the volume
    $self->mount(chmod => '777');

    # Get the volume mount path
    my $path = $self->mounted;

    # Verify if proper mount path was returned
    croak 'Volume ' . $self->name . ' was not mounted' if (!defined($path));
    
    # Create a client object
    my $client = NOTE::Client->new(storage => $self);
    
    # Append the mount point of Volume to get absolute path of the output file.
    $args{output_file} = $path.'/'.$args{output_file};
    
    # Invoke the client dd command
    $client->dd(%args);

} ## end sub dd

=head1 SEE ALSO

L<C<NOTE>|engineering::ontap_tests::main::docs::lib-NOTE-pm>

L<C<NOTE::Aggregate>|engineering::ontap_tests::main::docs::lib-NOTE-Aggregate-pm>

L<C<NOTE::Filer>|engineering::ontap_tests::main::docs::lib-NOTE-Filer-pm>

L<C<NOTE::Qtree>|engineering::ontap_tests::main::docs::lib-NOTE-Qtree-pm>

L<C<NOTE::SnapMirror>|engineering::ontap_tests::main::docs::lib-NOTE-SnapMirror-pm>

=head1 STATUS

Public.

$Id: //depot/prod/test/Rsierranevada/lib/NOTE/Volume.pm#1 $

$Date: 2013/01/22 $

=head1 AUTHOR

Author information is in the
L<NOTE overview|engineering::ontap_tests::main::docs::lib-NOTE-pm/::author>.

=cut

1;
