use strict;
package NetSwitch;
use Expect;
use Net::Telnet;
use TracePkg;
use Data::Dumper;

my $expect_timeout = 30;
my $Trace = new TracePkg();

# THIS STUFF SHOULD LIVE IN A BETTER PLACE
my %Config = (
    'rtpgx-sw-cc01'=>{
	os        => "ios",
	sw_domain => "rtp.netapp.com",
	sw_user   => "tester",
	sw_pass   => "qxga",
    },
    'rtpgx-sw-dc01'=>{
	os        => "ios",
	sw_domain => "rtp.netapp.com",
	sw_user   => "tester",
	sw_pass   => "qxga",
    },
    'cat4948-3'=>{
	os        => "ios",
	sw_domain => "slabadmin.spin.eng.netapp.com",
	sw_user   => "emichael",
	sw_pass   => "letmein",
    },
    'qexl-data'=>{
	os        => "ios",
	sw_domain => "slabadmin.spin.eng.netapp.com",
	sw_user   => "emichael",
	sw_pass   => "letmein",
    },
    'qexl-cluster'=>{
	os        => "ios",
	sw_domain => "slabadmin.spin.eng.netapp.com",
	sw_user   => "emichael",
	sw_pass   => "letmein",
    },
    'kc7609-0'=>{
	os        => "ios",
	sw_domain => "slabadmin.spin.eng.netapp.com",
	sw_user   => "emichael",
	sw_pass   => "letmein",
    },
    'kc7609-1'=>{
	os        => "ios",
	sw_domain => "slabadmin.spin.eng.netapp.com",
	sw_user   => "emichael",
	sw_pass   => "letmein",
    },
    '7609-F11'=>{
	os        => "ios",
	sw_domain => "slabadmin.spin.eng.netapp.com",
	sw_user   => "emichael",
	sw_pass   => "letmein",
    },
    'qa-ericcson-1'=>{
	os        => "mosaic",
	sw_domain => "slabadmin.spin.eng.netapp.com",
	sw_user   => "emichael",
	sw_pass   => "letmein",
    },

);

sub new {
    $Trace->entry();
    my $invocant = shift;
    my $class = ref($invocant) || $invocant;
    my $Object = {};
    bless ($Object, $class);
    my $base_prompt;
    my %prompts;
    my %opt = @_;
    $Trace->debug(msg=>"new \%opt: ".Dumper\%opt);
    
    while (my ($k, $v) = each %opt){
	if (exists($Object->{$k})){
	    $Trace->comment(msg => "NetSwitch; Changing value of $k from $Object->{$k} to $v");
	} else {
	    $Trace->comment(msg => "NetSwitch; Defining value of $k as $v");
	}
	$Object->{$k} = $v;
    }
    
    # Pull in our config info
    $Object->{"config"} = \%Config;
    $Trace->debug(msg=>"new Object \%config: ".Dumper\$Object->{"config"});

    # Build our prompt definitions based on our os
    if ($Config{$opt{switch_name}}{os} eq "ios"){
	$base_prompt = $opt{switch_name};
	%prompts = (
	    top => $base_prompt."#",
	    config => qr "$base_prompt\(config\)\#",
	    config_if => qr "$base_prompt\(config-if\)\#",
	);
    } elsif ($Config{$opt{switch_name}}{os} eq "mosaic"){
	$base_prompt = $opt{switch_name};
	# there is a "-" between "interface" and "GigabitEthernet"
	my $interface_prompt = $opt{interface};
	$interface_prompt =~ s/interface //g;
	$Trace->debug(msg=>"Removed \"interface \" from $opt{interface}; $interface_prompt");
	%prompts = (
	    top => "<$base_prompt>",
	    config => qr "\[$base_prompt\]",
	    config_if => qr "\[$base_prompt-$interface_prompt\]",
	);
    }
    $Object->{"prompts"} = \%prompts;
    $Trace->debug(msg=>"new Object \%prompts: ".Dumper\$Object->{"prompts"});
    
    $Trace->exit(msg=>"returning $Object");
    return $Object;
} # end new

sub switch_if {
    $Trace->entry();
    my $self = shift();
    my %opt = (
	tasks => ["show_interface"],     # What do you want the switch to do?
	#interface => $self->{interface},         # Which swith interface to use
	#optional
	speed => "auto",
	duplex => "full",
	auto_neg => "true",
	@_,
    );
    $Trace->debug(msg=>"switch_if opt: ".Dumper\%opt);
    if (!$self->{"switch_name"}){
	$Trace->fatal(msg=>"switch_name is not set");
    } else {
	$Trace->debug(msg=>"Using ".$self->{"switch_name"}."...");
    }

# create a connection
    $Trace->debug(msg=>"Connecting to ".$self->{"switch_name"});
    $self->{con} = $self->exp_connect($self->{"switch_name"});
    if (!$self->{con}) {
	$Trace->fatal(msg=>"Crap! Our connect failed!");
    }
# push our opts into the object
    $self->{port_opts} = \%opt;

# do our tasks
    foreach my $task (@{$opt{tasks}}) {
	#print "Starting $task...\n";
	$Trace->comment(msg=>"Starting $task...");
	$self->$task();
    }

} # end switch_if

sub find_vlan {
    $Trace->entry();
    my $self = shift();
    my $hostname = $self->{"switch_name"};
    my $vlan;

    # Call the proper _exp_connect sub based on our switch's os
    if ($self->{config}{$hostname}{os} eq "ios"){
	$Trace->trace(msg=>"Using ios connect routine.");
	$vlan = $self->_exp_connect_ios($hostname);
    } elsif ($self->{config}{$hostname}{os} eq "mosaic") {
	$Trace->trace(msg=>"Using mosaic connect routine.");
	$vlan = $self->_exp_connect_mosaic($hostname);
    } else {
	$Trace->fatal(msg=>"Switch os unknown or undefined; $self->{config}{$hostname}{os}!");
    }
    $Trace->exit();
    return $vlan;
} # end find_vlan


sub exp_connect {
    $Trace->entry();
    my $self = shift();
    my $hostname = $self->{"switch_name"};
    my $exp;  # ref to the connection object
  
    # Call the proper _exp_connect sub based on our switch's os
    if ($self->{config}{$hostname}{os} eq "ios"){
	$Trace->trace(msg=>"Using ios connect routine.");
	$exp = $self->_exp_connect_ios($hostname);
    } elsif ($self->{config}{$hostname}{os} eq "mosaic") {
	$Trace->trace(msg=>"Using mosaic connect routine.");
	$exp = $self->_exp_connect_mosaic($hostname);
    } else {
	$Trace->fatal(msg=>"Switch os unknown or undefined; $self->{config}{$hostname}{os}!");
    }

    $Trace->exit();
    return $exp;
} # end exp_connect

sub _exp_connect_mosaic {
    $Trace->entry();
    my $self = shift();
    my $hostname = $self->{"switch_name"};
    my $full_name = $hostname.".".$self->{"config"}{$self->{"switch_name"}}{"sw_domain"};
    my $username = $self->{"config"}{$self->{"switch_name"}}{"sw_user"};
    my $pword = $self->{"config"}{$self->{"switch_name"}}{"sw_pass"};
    my $spawn_ok;
 
    $Trace->debug(msg=>"Spawning telnet to $full_name");
    my $switch_con = new Net::Telnet ($full_name)
	or $Trace->fatal(msg=>"Cannont telnet to $full_name!!!");
    my $exp = Expect->exp_init($switch_con);
    #$exp->debug(1);
     $exp->exp_internal(1);


    $exp->expect($expect_timeout,
	#[
	#qr'Username: $',
	#sub {
	#    $spawn_ok = 1;
	#    $Trace->debug(msg=>"DEBUG: saw Username...");
	#    my $fh = shift;
	#    $fh->send("$username\n");
	#    exp_continue;
	#}
	#],
	[
	'Password:',
	sub {
	    $Trace->debug(msg=>"DEBUG: saw Password:...");
	    $spawn_ok = 1;
	    my $fh = shift;
	    $fh->send("$pword\n");
	    exp_continue;
	}
	],
	[
	"<$hostname>",
	sub {
	    $Trace->comment(msg=>"Logged in...");
	    #my $fh = shift;
	    #$fh->send("\n");
	    #exp_continue;
	}
	],
	[
	eof =>
	sub {
	    if ($spawn_ok) {
		die "ERROR: premature EOF in login!\n";
	    } else {
		die "ERROR: Could not spawn telnet!\n";
	    }
	}
	],
	[
	timeout =>
	sub {
	    die "No login.\n";
	}
	],
    );
   
    $Trace->exit();
    return $exp;
} # end _exp_connect_mosaic


sub _exp_connect_ios {
    $Trace->entry();
    my $self = shift();
    my $hostname = $self->{"switch_name"};
    my $full_name = $hostname.".".$self->{"config"}{$self->{"switch_name"}}{"sw_domain"};
    my $username = $self->{"config"}{$self->{"switch_name"}}{"sw_user"};
    my $pword = $self->{"config"}{$self->{"switch_name"}}{"sw_pass"};
    my $spawn_ok;
    my $exp;  # ref to the connection object
    $Trace->debug(msg=>"Spawning telnet to $full_name");
    my $switch_con = new Net::Telnet ($full_name)
	or $Trace->fatal(msg=>"Cannont telnet to $full_name!!!");
    my $exp = Expect->exp_init($switch_con);
    #$exp->debug(1);
    # $exp->exp_internal(1);


    $exp->expect($expect_timeout,
	[
	qr'Username: $',
	sub {
	    $spawn_ok = 1;
	    $Trace->debug(msg=>"DEBUG: saw Username...");
	    my $fh = shift;
	    $fh->send("$username\n");
	    exp_continue;
	}
	],
	[
	'Password: $',
	sub {
	    $Trace->debug(msg=>"DEBUG: saw Password:...");
	    my $fh = shift;
	    $fh->send("$pword\n");
	    exp_continue;
	}
	],
	[
	"$hostname#",
	sub {
	    $Trace->comment(msg=>"Logged in...");
	    #my $fh = shift;
	    #$fh->send("\n");
	    #exp_continue;
	}
	],
	[
	eof =>
	sub {
	    if ($spawn_ok) {
		die "ERROR: premature EOF in login!\n";
	    } else {
		die "ERROR: Could not spawn telnet!\n";
	    }
	}
	],
	[
	timeout =>
	sub {
	    die "No login.\n";
	}
	],
    );

    $Trace->exit();
    return $exp;
} # end exp_connect


sub show_if_config {   
    $Trace->entry();
    my $self = shift;
    my $command = "show running-config";
    my $interface = $self->{interface};
  
     # Call the proper _show_if_config sub based on our switch's os
    if ($self->{config}{$self->{switch_name}}{os} eq "ios"){
	$Trace->trace(msg=>"Using ios _show_if_config routine.");
	$self->_show_if_config_ios();
    } elsif ($self->{config}{$self->{switch_name}}{os} eq "mosaic") {
	$Trace->trace(msg=>"Using mosaic _show_if_config routine.");
	$self->_show_if_config_mosaic();
    }

    $Trace->exit();
} # end show_if_config

sub _show_if_config_mosaic {   
    $Trace->entry();
    my $self = shift;
    my $command = "display";
    my $interface = $self->{interface};
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"_show_if_config_mosaic\" HAD NO CONNECTION!");
    }
    
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("$command $interface\n", $self->{prompts}{top});
    my $command_output = $self->{con}->before();
    print "*+*+*+*+*+*+*+*+*+*+*+*\n";
    print "$command_output\n";
    print "*+*+*+*+*+*+*+*+*+*+*+*\n";
    
    $Trace->exit();
} # end _show_if_config_ios


sub _show_if_config_ios {   
    $Trace->entry();
    my $self = shift;
    my $command = "show running-config";
    my $interface = $self->{interface};
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"_show_if_config_ios\" HAD NO CONNECTION!");
    }
    
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("$command $interface\n", $self->{prompts}{top});
    my $command_output = $self->{con}->before();
    print "*+*+*+*+*+*+*+*+*+*+*+*\n";
    print "$command_output\n";
    print "*+*+*+*+*+*+*+*+*+*+*+*\n";
    
    $Trace->exit();
} # end _show_if_config_ios


sub set_media_options {
    $Trace->entry();
    my $self = shift;
    
    # Call the proper _set_media_options sub based on our switch's os
    if ($self->{config}{$self->{switch_name}}{os} eq "ios"){
	$Trace->trace(msg=>"Using ios set_media_options routine.");
	$self->_set_media_options_ios($self->{switch_name});
    } elsif ($self->{config}{$self->{switch_name}}{os} eq "mosaic") {
	$Trace->trace(msg=>"Using mosaic set_media_options routine.");
	$self->_set_media_options_mosaic($self->{switch_name});
    }
    #TODO: verify media options!

    # Make sure the port is not in shutdown
    $self->port_up();
    
    $Trace->exit();
} # end set_media_options

sub _set_media_options_mosaic {
    $Trace->entry();
    my $self = shift;
    my $speed = $self->{"port_opts"}{"speed"};
    my $duplex = $self->{"port_opts"}{"duplex"};
    my $auto_neg = $self->{"port_opts"}{"auto_neg"};
    #my $switch_mtu = $self->{"port_opts"}{"switch_mtu"};
    my $switch_mtu = $self->{"port_opts"}{"mtu"};
    my $interface = $self->{interface};
    $Trace->debug(msg=> "DEBUG: 
	speed:<$speed>,
	duplex:<$duplex>,
	auto:<$auto_neg>,
	switch_mtu:<$switch_mtu>,
    ");
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"_set_media_options_mosaic\" HAD NO CONNECTION!");
    }
    if ($auto_neg eq "true" ) {
	$speed = "auto";
	$duplex = "auto";
    }
    if ($switch_mtu > 9000){
	$Trace->comment(msg=>"This switch does not allow mtu value to be set; enabling jumboframes for this port");
	$switch_mtu = "jumboframe enable";
    } else {
	$Trace->comment(msg=>"This switch does not allow mtu value to be set; disabling jumboframes for this port");
	$switch_mtu = "undo jumboframe enable";
    }
    $Trace->debug(msg=> "DEBUG: 
	speed:<$speed>,
	duplex:<$duplex>,
	auto:<$auto_neg>,
	switch_mtu:<$switch_mtu>,
    ");
    
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("system\n", $self->{prompts}{config});
    $self->_exp_dialog("$interface\n", $self->{prompts}{config_if});
    $self->_exp_dialog("duplex $duplex\n", $self->{prompts}{config_if});
    $self->_exp_dialog("speed $speed\n", $self->{prompts}{config_if});
    $self->_exp_dialog("$switch_mtu\n", $self->{prompts}{config_if});
    $self->_exp_dialog("\cZ", $self->{prompts}{top});
    # make sure the port is not shutdown
    $self->port_up();
    $Trace->exit();
} # end _set_media_options_mosaic

sub _set_media_options_ios {
    $Trace->entry();
    my $self = shift;
    my $speed = $self->{"port_opts"}{"speed"};
    my $duplex = $self->{"port_opts"}{"duplex"};
    my $auto_neg = $self->{"port_opts"}{"auto_neg"};
    my $switch_mtu = $self->{"port_opts"}{"switch_mtu"};
    my $interface = $self->{interface};
    $Trace->debug(msg=> "DEBUG: speed:<$speed>, duplex:<$duplex>, auto:<$auto_neg>");
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"set_media_options\" HAD NO CONNECTION!");
    }
    if ($auto_neg eq "true" ) {
	$speed = "auto";
	$duplex = "auto";
    }
    $Trace->debug(msg=> "DEBUG: speed:<$speed>, duplex:<$duplex>, auto:<$auto_neg>");
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("config term\n", $self->{prompts}{config});
    $self->_exp_dialog("$interface\n", $self->{prompts}{config_if});
    $self->_exp_dialog("duplex $duplex\n", $self->{prompts}{config_if});
    $self->_exp_dialog("speed $speed\n", $self->{prompts}{config_if});
    $self->_exp_dialog("mtu $switch_mtu\n", $self->{prompts}{config_if});
    $self->_exp_dialog("\cZ", $self->{prompts}{top});
    # make sure the port is not shutdown
    $self->port_up();
    $Trace->exit();
} # end _set_media_options_ios


sub port_up {
    $Trace->entry();
    my $self = shift;
   
    # Call the proper _port_up sub based on our switch's os
    if ($self->{config}{$self->{switch_name}}{os} eq "ios"){
	$Trace->trace(msg=>"Using ios port_up routine.");
	$self->_port_up_ios();
    } elsif ($self->{config}{$self->{switch_name}}{os} eq "mosaic") {
	$Trace->trace(msg=>"Using mosaic port_up routine.");
	$self->_port_up_mosaic();
    }

    $Trace->exit();
} # end port_up

sub _port_up_mosaic {
    $Trace->entry();
    my $self = shift;
    my $interface = $self->{interface};
    $Trace->debug(msg=> "DEBUG: Enabling $interface...");
    if (!$interface) {
	$Trace->fatal(msg=>"\"port_up\" has no interface to restore!!!");
    } 
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"port_up\" HAD NO CONNECTION!");
    }
   
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("system\n", $self->{prompts}{config});
    $self->_exp_dialog("$interface\n", $self->{prompts}{config_if});
    $self->_exp_dialog("undo shutdown\n", $self->{prompts}{config_if});
    $self->_exp_dialog("\cZ", $self->{prompts}{top});
   
    #$Trace->fatal(msg=>"DEBUG DIE");
    $Trace->exit();
} # end _port_up_mosaic

sub _port_up_ios {
    $Trace->entry();
    my $self = shift;
    my $interface = $self->{interface};
    $Trace->debug(msg=> "DEBUG: Enabling $interface...");
    if (!$interface) {
	$Trace->fatal(msg=>"\"port_up\" has no interface to restore!!!");
    } 
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"port_up\" HAD NO CONNECTION!");
    }
   
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("config term\n", $self->{prompts}{config});
    $self->_exp_dialog("$interface\n", $self->{prompts}{config_if});
    $self->_exp_dialog("no shutdown\n", $self->{prompts}{config_if});
    $self->_exp_dialog("\cZ", $self->{prompts}{top});
    
    $Trace->exit();
} # end _port_up_ios


sub up_link {
    $Trace->entry();
    my $self = shift;
   
    # Call the proper _up_link sub based on our switch's os
    if ($self->{config}{$self->{switch_name}}{os} eq "ios"){
	$Trace->fatal(msg=>"There is currently no way to drop/restore link on this switch!");
    } elsif ($self->{config}{$self->{switch_name}}{os} eq "mosaic") {
	$Trace->trace(msg=>"Using _port_up_mosaic routine");
	$self->_port_up_mosaic();
    }

    $Trace->exit();
} # end up_link


sub drop_link {
    $Trace->entry();
    my $self = shift;
   
    # Call the proper _drop_link sub based on our switch's os
    if ($self->{config}{$self->{switch_name}}{os} eq "ios"){
	$Trace->fatal(msg=>"There is currently no way to drop link on this switch!");
    } elsif ($self->{config}{$self->{switch_name}}{os} eq "mosaic") {
	$Trace->trace(msg=>"Using _port_down_mosaic routine");
	$self->_port_down_mosaic();
    }

    $Trace->exit();
} # end drop_link

sub port_down {
    $Trace->entry();
    my $self = shift;
   
    # Call the proper _port_down sub based on our switch's os
    if ($self->{config}{$self->{switch_name}}{os} eq "ios"){
	$Trace->trace(msg=>"Using ios port_down routine.");
	$self->_port_down_ios();
    } elsif ($self->{config}{$self->{switch_name}}{os} eq "mosaic") {
	$Trace->fatal(msg=>"There is no known way to \"portdown\" this switch without dropping link!");
	#$self->_port_down_mosaic();
    }

    $Trace->exit();
} # end port_down

sub _port_down_mosaic {
    $Trace->entry();
    my $self = shift;
    my $interface = $self->{interface};
    $Trace->debug(msg=> "DEBUG: Enabling $interface...");
    if (!$interface) {
	$Trace->fatal(msg=>"\"_port_down_mosaic\" has no interface to restore!!!");
    } 
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"_port_down_mosaic\" HAD NO CONNECTION!");
    }
   
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("system\n", $self->{prompts}{config});
    $self->_exp_dialog("$interface\n", $self->{prompts}{config_if});
    $self->_exp_dialog("shutdown\n", $self->{prompts}{config_if});
    $self->_exp_dialog("\cZ", $self->{prompts}{top});
   
    $Trace->exit();
} # end _port_down_mosaic

sub _port_down_ios {
    $Trace->entry();
    my $self = shift;
    my $interface = $self->{interface};
    $Trace->debug(msg=> "DEBUG: Enabling $interface...");
    if (!$interface) {
	$Trace->fatal(msg=>"\"_port_down_ios\" has no interface to disable!!!");
    } 
    if (!$self->{"con"}) {
	$Trace->fatal(msg=>"\"_port_down_ios\" HAD NO CONNECTION!");
    }
   
    $self->_exp_dialog("\n", $self->{prompts}{top});
    $self->_exp_dialog("config term\n", $self->{prompts}{config});
    $self->_exp_dialog("$interface\n", $self->{prompts}{config_if});
    $self->_exp_dialog("shutdown\n", $self->{prompts}{config_if});
    $self->_exp_dialog("\cZ", $self->{prompts}{top});
    
    $Trace->exit();
} # end _port_down_ios


sub _exp_dialog {
    $Trace->entry();
    my $self = shift;
    my $send = shift;
    my $expect = shift;
    my $timeout = shift;

    if (!$timeout){
	$timeout = $expect_timeout;
    }

    $Trace->debug(msg=>"SENDING: <$send>, EXPECTING: <$expect>...");
    $self->{con}->send("$send");
    $self->{con}->expect($timeout, 
	[$expect],
	[timeout => \&_exp_dialog_timeout],
    );
    # check for any errors
    my $exp_error = $self->{con}->error();
    if ($exp_error){
	$Trace->fatal(msg=>"expect returned the following error:\n $exp_error");
    }

    $Trace->exit();
} # end exp_dialog

sub _exp_dialog_timeout {
    $Trace->entry();
    $Trace->fatal(msg=>"Encountered a timeout during last _exp_dialog!");
    $Trace->exit();
}

1;
