# $Id:$
#
# Copyright (c) 2010 Network Appliance Inc.
# All right reserved.
#
## @summary Routines for running ipspaces utilites 
##
## @description
##         These functions are used to run various networking
##         utilities on devices independent of the device type.
##        These functions help in abstracting the test scripts
##        from subtle differences in command usage and command
##        outputs on various platforms.
##
## @dependencies
##
## use Tharn;
##
## @author  dl-nwkg-test (SCON QA)

package Network::IPSpaces;

require Exporter;
use strict;
use Data::Dumper;
use Tharn;
use filer qw(
	filer_options_get 
	filer_options_set
	);
use VFILER::Vfiler qw(
                vfiler_add
                vfiler_create
                vfiler_ifc_setup
                vfiler_create_non_default_ipspace
                vfiler_create_default_ipspace
                vfiler_run_vfiler_setup
                vfiler_create_ipv6
                vfiler_run_vfiler_setup_ipv6
             );
use Network::Utils qw(
                do_ifconfig
                get_ifconfig_info
             );
use VFILER::IPspace qw(
                ifc_in_ipspace
                create_ipspace
                assign_ipspace
                assign_default_ipspace);
use Network::IPv6 qw(
                _get_ra_based_address
                _get_link_local_address);


use vars qw (@ISA @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT_OK = qw (vfiler_setup
		 get_vserver_id
		 tcp_client_create
		 tcp_server_create 
            );

my $debug = 1;

## @name ::vfiler_setup()
##
## @summary sets up a set of vfilers with different options given 
##
## @description
##
## this function takes a hash of vfiler info and configures vfilers based on options.
## supports ipv4/v6 vfiler creation 
##
## returns 1 if the setup succeeds and 0 otherwise 
##
## @arg : will add later ... thinking .. 
##

sub vfiler_setup {


    

}

sub get_vserver_id{

    my ($conn, $interface,$ip) = @_;
    my %ifconfig_info = ();
    my $ifconfig_cmd = "ifconfig";
    my $vserver_id= undef;
    sayEOF($conn,"priv set diag");
    my $filer_out = sayEOF($conn, "$ifconfig_cmd $interface");
    if ($filer_out =~ /(no such interface|device not found)/i) {
        die fail_result("failed to execute ifconfig command on $interface");
    }
    my @ifconfig_out = split(/\n/, $filer_out);
    $interface = undef;
    foreach my $line ( @ifconfig_out ) {
        # Skip blank lines
        next if ( $line =~ m/^\s*?$/ );
        if($line =~m/inet*(\d*)\s($ip).* Vserver ID:\s+(\d+)/){
            $vserver_id=$3;
        }
    }
    if (!defined($vserver_id)) {
        die fail_result("Not able to get vserver_id");
    }
    return($vserver_id);
}

sub tcp_client_create(){
    my $args = shift;
    if($args->{stack} eq "BSD"){
        print "login to systemshell";
        _login_systemshell($args->{conn});
    }
    print "create TCP_CLIENT $args->{addr_family} in $args->{stack} for vserver $args->{ip_addr} with role $args->{lif_role} port_no $args->{port_no}";
    my ($a,undef,undef)=say($args->{conn},"tcp_client $args->{addr_family} $args->{client_ip_addr} $args->{client_port_no} $args->{server_ip_addr} $args->{server_port_no} $args->{vserver_id} $args->{lif_role}", -noprompt, '-wait',"25 bytes sent",-timeout, 600 );
    if($a=~/25 bytes sent/sgi){
        print "Communication is happening between client and server succesfully";
        return 1;
    }else{
        print "TCP_CLIENT creation failed";
        say($args->{conn}, "exit" );
        say($args->{conn}, "exit" );
        return 0;
    }
}

sub tcp_server_create(){
    my $args = shift;
    if($args->{stack} eq "BSD"){
        print "login to systemshell";
        _login_systemshell($args->{conn});
    }
    print "create TCP_SERVER $args->{addr_family} in $args->{stack} for vserver $args->{ip_addr} with role $args->{lif_role} port_no $args->{port_no}";
    my ($a,undef,undef)=say($args->{conn},"tcp_server $args->{addr_family} $args->{ip_addr} $args->{port_no} $args->{vserver_id} $args->{lif_role}", -noprompt, '-wait',"Started Listening on",-timeout, 600 );
    if($a=~/Listening on/sgi){
        print "Servere is created succesfully";
        return 1;
    }else{
        print "TCP_SERVER creation failed";
        say($args->{conn}, "exit" );
        return 0;
    }
}

