# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Network/IPv6.pm#1 $
#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
#
## @summary Library of functions for basic filer commands
## @description

package Network::IPv6;

use strict;
#use base qw(Exporter);

BEGIN {
require Exporter;
use vars qw (@ISA @EXPORT_OK);
@ISA = qw(Exporter);
our @EXPORT_OK = qw(
    _remove_ipv4_on_an_interface
    _autoconf_ip_present
    _ipv6_enabled
    _basic_conf_ipv6
    _get_link_local_address
    _get_ra_based_address
    _verify_ping6
    _ipv6_address_is_configured
    _is_route_present
    __ipv4_alias
    __check_pattern
    set_mount_options
    mount_vol_ipv6
);
}

use filer qw(filer_sysstat_get);
use nfs::GenericExport qw(export_path unexport_path);
use filer qw(filer_options_get filer_options_set);
use SayWrap qw(sayEOF);
use Hammer;
my $Nfs_hammer_runid = 0;
use Tharn;
use Admin::License qw(is_licensed license);
use Carp qw(croak);
use ExportFS qw(share_add share_delete);
use Fcntl;
use File::Temp qw(:mktemp);
use FileManip qw(read_file);
use GenericMount qw(mount umount);
use LogReader;
use MapShare qw(map_cifs_share unmap_cifs_share);
use Params::Validate qw(validate OBJECT SCALAR HASHREF UNDEF);
#use Tharn;

#-----------------------------------------------------------------------------------------
## Removes or checks for ipv4 alias configured on an interface
## __ipv4_alias( { fc => $fc, intfc => "e0b", action => "remove" } );
## __ipv4_alias( { fc => $fc, intfc => "e0b", action => "check" } );
sub __ipv4_alias {
   my $args = shift;
   my $res = sayEOF( $args->{fc}, "ifconfig $args->{intfc}" );
   my $ret = __check_pattern(
      {
	 variable => $res,
	 pattern  => "(?:\d{1,3}\.){3}.*?alias",
	 action   => 'check'
      }
   );
   if ($ret) {
      logcomment("IPv4 address alias is present");
   }
   else {
      logcomment("IPv4 address alias is not present");
   }
   if ( $args->{action} eq "remove" && $ret ) {
   my $ipv4 = __check_pattern(
	 {
	    variable => $res,
	    pattern  => "((?:\d{1,3}\.){3}\d{1,3}).*?alias",
	    action   => 'match'
	 }
      );
      sayEOF( $args->{fc}, "ifconfig $args->{intfc} -alias $ipv4" );
   }
   return $ret;
}
#-----------------------------------------------------------------------------------------
## Removes the ipv4 address configured on a interface
## _remove_ipv4_on_an_interface( { fc => $fc, intfc => "e0b" } );
sub _remove_ipv4_on_an_interface {
   my $args  = shift;
   my $fc    = $args->{fc};
   my $intfc = $args->{intfc};
   my $res   = sayEOF( $fc, "ifconfig $intfc" );
   if ( $res =~ /(?:\d{1,3}\.){3}/ ) {
   __ipv4_alias( { fc => $fc, intfc => $intfc, action => "remove" } );
   sayEOF( $fc, "ifconfig $intfc 0.0.0.0 down" );
      return 1;
   }
   else {
      return 1;
   }
   return 0;
}
#-----------------------------------------------------------------------------------------
## Used to check or match a specified pattern against the contents of a variable, action
## can be 'check' or 'match'
## __check_pattern(
##            {
##                variable => $autoconf,
##                pattern  => "fe80::.*?autoconf.*?(?!tentative)",
##                action   => "check"
##            }
##        );
sub __check_pattern {
   my $args = shift;
   if ( $args->{variable} =~ /$args->{pattern}/si ) {
      return $1 if ( $args->{action} eq 'match' );
      return 1;
   }
   else {
      return 0;
   }
}
#-----------------------------------------------------------------------------------------
## Checks if autoconf ips are properly assigned on an interface. You can check for either
## Link Local Address, RA based address or both
## _autoconf_ip_present( { fc => $fc, intfc => "e0b", type => 'both' } );
## _autoconf_ip_present( { fc => $fc, intfc => "e0b", type => 'LLA' } );
## _autoconf_ip_present( { fc => $fc, intfc => "e0b", type => 'RA' } );
sub _autoconf_ip_present {
   my $args     = shift;
   my $fc       = $args->{fc};
   my $intfc    = $args->{intfc};
   my $type     = $args->{type};
   my $autoconf = sayEOF( $fc, "ifconfig $intfc" );
   my $ret_status;
   if ( $type eq 'LLA' ) {
      $ret_status = __check_pattern(
	 {
	    variable => $autoconf,
	    pattern  => "fe80::.*?autoconf.*?(?!tentative)",
	    action   => "check"
	 }
      );
      return $ret_status;
   }
   elsif ( $type eq 'RA' ) {
      $ret_status = __check_pattern(
	 {
	    variable => $autoconf,
	    pattern  => "(?!fe80::).*?autoconf.*?(?!tentative)",
	    action   => "check"
	 }
      );
	 return $ret_status;
      }
      elsif ( $type eq 'both' ) {
      my $ret_status1 = __check_pattern(
	 {
	    variable => $autoconf,
	    pattern  => "fe80::.*?autoconf.*?(?!tentative)",
	    action   => "check"
	 }
      );
      my $ret_status2 = __check_pattern(
	 {
	    variable => $autoconf,
	    pattern  => "(?!fe80::).*?autoconf.*?(?!tentative)",
	    action   => "check"
	 }
      );
      if ( $ret_status1 && $ret_status2 ) {
	 logcomment(
	    "Both Link local address and Route based address have been autoconfigured on the interface $intfc"
	 );
	 return 1;
      }
      elsif ( $ret_status1 && !$ret_status2 ) {
	 logcomment(
	 "Only Link local address has been autoconfigured on the interface $intfc"
	 );
	 return 0;
      }
      elsif ( !$ret_status1 && $ret_status2 ) {
	 logcomment(
	 "Only Route based address has been autoconfigured on the interface $intfc"
	 );
	 return 0;
      }
      else {
	 logcomment(
	 "Both Link local address and Route based address have not been autoconfigured on the interface $intfc"
	 );
	 return 0;
        }
    }
}
#-----------------------------------------------------------------------------------------
## Checks or enables ipv6 option on the filer
## _ipv6_enabled( { fc => $fc, action => "enable" } )
## _ipv6_enabled( { fc => $fc, action => "check" } )
sub _ipv6_enabled {
   my $args = shift;
   my $action = shift || 'check';
   if ( filer_options_get( $args->{fc}, "ip.v6.enable" ) eq "on" ) {
      return 1;
   }
   else {
      if ( $args->{action} eq "enable" ) {
	 filer_options_set( $args->{fc}, "ip.v6.enable", "on" );
	 return _ipv6_enabled( { fc => $args->{fc}, action => 'check' } );
      }
      else {
	 return 0;
      }
   }
}
#-----------------------------------------------------------------------------------------
## Performs basic ipv6 configurations and checks. You can choose to enable ipv6 or just
## check if it is enabled
## _basic_conf_ipv6({fc => $fc,v6action => "enable",intfclist => "$VIF_NAME_1"});
## _basic_conf_ipv6({fc => $fc,v6action => "check",intfclist => "e0b e0c"});
sub _basic_conf_ipv6 {
   my $args = shift;
   logcomment("list: $args->{intfclist}.......");
   my @intfs = split( / /, $args->{intfclist} );
   logcomment("intfclist @intfs");
   sleep(20);
   if ( _ipv6_enabled( { fc => $args->{fc}, action => $args->{v6action} } ) ) {
      logcomment("IPv6 is enabled on the filer");
   }
   sleep(20); 
   foreach my $intf (@intfs) {
      # Check if e0a & e0b do not have an IPv4 address configured
      if (
	 _remove_ipv4_on_an_interface(
	    { fc => $args->{fc}, intfc => $intf }
	 )
      )
      {
	 logcomment("$intf does not have an IPv4 address configured now");
      }
      else {
	 logresult( "FAIL", "there was a problem configuring $intf down" );
	 exit(0);
      }
      # bring the interfaces up
      sayEOF( $args->{fc}, "ifconfig $intf up" );
      sleep(20);
      # verify if autoconf IP is present
      _autoconf_ip_present(
      { fc => $args->{fc}, intfc => $intf, type => 'both' } );
   }
}
#-----------------------------------------------------------------------------------------
## Returns link local address for an interface
## my $e0a_lla = _get_link_local_address({fc => $fc1,intfc => "e0a"});
sub _get_link_local_address {
   my $args = shift;
   my $autoconf = sayEOF( $args->{fc}, "ifconfig $args->{intfc}" );
   return __check_pattern(
      {
	 variable => $autoconf,
	 pattern  => "(fe80[^ ]+).*?autoconf",
	 action   => 'match'
      }
   );
}
#-----------------------------------------------------------------------------------------
## Returns the RA based autoconf address
## _get_ra_based_address({fc => $fc, intfc => "e0b", vlan_begin_ip => "6ffe"})
sub _get_ra_based_address {
   my $args = shift;
   my $autoconf = sayEOF( $args->{fc}, "ifconfig $args->{intfc}" );
   my @lines = split(/\n/, $autoconf);
   my @autoconf_ips;
   foreach my $line (@lines) {
      if($line =~ /autoconf/i && $line !~ /fe80/i) {
#	      logcomment("line is $line index: $#autoconf_ips");
	 $autoconf_ips[$#autoconf_ips+1] = $line; 	
      }
   }
   if(defined($args->{vlan_begin_ip})) {
      my $vip = $args->{vlan_begin_ip};
      foreach my $line (@autoconf_ips) {
	 if($line =~ /$vip/i) {
	    $line =~ /inet6 ([a-f0-9:]+) /i;
	    return $1;
	 }	
      }
   }
   $autoconf_ips[0] =~ /inet6 ([a-f0-9\:]+) /i;
#   logcomment("line is $autoconf_ips[0] $1");
   return $1;
}
#-----------------------------------------------------------------------------------------
## Verifies ping to ipv6 addreses, and returns 1 on success and 0 on failure
## _verify_ping6({fc => $cc,dest => $IPv6_ADDR_1,command => "ping"})
sub _verify_ping6 {
   my $args = shift;
   my $result;
   if($args->{fc}->hostp->hosttype() =~ m/solaris/)
   {
	   $args->{command} = "ping";
   }
   else
   {
	    $args->{command} = "ping6";

   }
   if ( defined( $args->{scope} ) ) {
      $result =
      sayEOF( $args->{fc},
      "$args->{command} -c 1 $args->{dest}%$args->{scope}" );
   }
   else {
      $result = sayEOF( $args->{fc}, "$args->{command} -c 1 $args->{dest}" );
   }
   if (
      __check_pattern(
	 {
	    variable => $result,
	    pattern  => "(?:bytes from|is alive)",
	    action   => 'check'
	 }
      )
   )
   {
      return 1;
   }
   else {
      return 0;
   }
}
#-----------------------------------------------------------------------------------------
## verifies if ipv6 address is configured on an interface
## _ipv6_address_is_configured({fc => $fc1,intfc => 'any',ipv6 => $IPv6_ADDR_1})
sub _ipv6_address_is_configured {
   my $args = shift;
   my $ifcg;
   if ( $args->{intfc} ne 'any' ) {
      $ifcg = sayEOF( $args->{fc}, "ifconfig $args->{intfc}" );
   }
   else {
        $ifcg = sayEOF( $args->{fc}, "ifconfig -a" );
   }
   return __check_pattern(
      { variable => $ifcg, pattern => $args->{ipv6}, action => 'check' } );
}
#-----------------------------------------------------------------------------------------
## verifies if route is present for a particular address
## _is_route_present({fc => $fc1,address => 'LLA'})
## _is_route_present({fc => $fc1,address => "fe80::.*?$FILER1_IF_1"})
sub _is_route_present {
   my $args = shift;
   my $result = sayEOF( $args->{fc}, "route -s" );
   if ( $args->{address} eq 'LLA' ) {
      $args->{address} = "fe80::";
   }
   return __check_pattern(
      { variable => $result, pattern => $args->{address}, action => 'check' }
   );
}
#---------------------------------------------------------------------

#sub nfs_hammer_start_ipv6(%;) {
#   my (%opts) = validate (
#      @_,
#      {
#	 fc             => { type => OBJECT, optional => 1 },
#	 client         => { type => SCALAR, default => 'localhost' },
#	 filer_ip       => { type => SCALAR, optional => 1 },
#	 hammer_bin     => { type => SCALAR,
#	    default => '/usr/local/test/bin/hammer' },
#	 hammer_time    => { type => SCALAR, default => -1 },
#	 panic          => { type => SCALAR, default => 'yes',
#	 regex => qr/^(no|yes)$/ },
#	 proto          => { type => SCALAR, default => 'tcp',
#	 regex => qr/^(udp|tcp)$/ },
#	 rsize          => { type => SCALAR, default => 32768,
#	 regex => qr/^\d+$/ },
#	 server         => { type => OBJECT, optional => 1 },
#	 version        => { type => SCALAR, default => 3 },
#	 verify_traffic => { type => SCALAR, default => 'no',
#	 regex => qr/^(yes|no)$/ },
#	 vol_name       => { type => SCALAR },
#	 mount_point    => { type => SCALAR },  
#	 wsize          => { type => SCALAR, default => 32768,
#	 regex => qr/^\d+$/ },
#      }
#   );
#   my ($end_time, $err, %fields, $hammer_args, $hammer_cmd, $hammer_obj,
#   $hammer_path, $hostname, %id, $line, $logdir, $logfile, $mode7,
#   $mount_dir, $server, $srv_vol);
#   if (!defined($opts{fc}) && !defined($opts{server})) {
#      croak('nfs_hammer_start: Either the fc or server parameter must ' .
#	 'be specified.');
#   }
#   $Log->comment("nfs_hammer_start: It's NFS Hammer Time!");
#   if (defined($ENV{LOGDIR})) {
#      $logdir = $ENV{LOGDIR};
#   } else {
#      croak ("Problem with NATE - LOGDIR not set in env");
#   }
#   # If a Server.pm object was specified then see if filer is 7 mode. If
#   # so then use the filer connection from the Server.pm object. If a
#   # Server.pm object was not passed in then filer is 7-mode.
#   if (defined($opts{server})) {
#      $opts{fc} = _server_to_fc($opts{server});
#      $mode7 = _is_7_mode($opts{server});
#   } else {
#      $mode7 = 1;
#   }
#   $id{licensed} = 0;
#   $id{client} = $opts{client};
#   if ($mode7) {
#      # 7 mode
#      $id{fc} = $opts{fc};
#      if (!is_licensed($opts{fc}, 'nfs')) {
#	 $id{licensed} = 1;
#	 license($opts{fc}, 'nfs');
#	 croak $@ if $@;
#      }
#      export_path(
#	 $opts{fc},
#	 "/vol/$opts{vol_name}",
#	 OPTIONS => 'rw,anon=0',
#      ) or croak $@;
#      $id{export} = "/vol/$opts{vol_name}";
#      # Default the IP address to send traffic to the filer to be the
#      # filer default IP address if not specified as an option.
#      if (!defined($opts{filer_ip})) {
#	 $opts{filer_ip} = $opts{fc}->hostp->default_ip;
#      }
#      $srv_vol = "/vol/$opts{vol_name}";
#   } else {
#      # 10 mode
#      $server = $id{server} = $opts{server};
#      # Grab the DATA_IP1 address for the NFS servers IP address
#      %fields = $opts{fc}->hostp()->param_fields();
#      if (!defined($fields{DATA_IP1})) {
#	 croak("Running hammer on 10-mode filer requires the DATA_IP1 " .
#	 "parameter to be defined in the filers hostspec");
#      }
#      if (!defined($opts{filer_ip})) {
#	 $opts{filer_ip} = $fields{DATA_IP1};
#      }
#      $srv_vol = "/$opts{vol_name}";
#   }
#   $hostname = $opts{fc}->hostp()->hostname();
#   $id{runid} = ++$Nfs_hammer_runid;
#   #$mount_dir = mount(
#   #    $opts{filer_ip},
#   #    $srv_vol,
#   #    client => $opts{client},
#   #    chmod  => '0777',
#   #    mntopts => "vers=$opts{version},proto=$opts{proto},noac,timeo=600," .
#   #               "rsize=$opts{rsize},wsize=$opts{wsize}",
#   #);
#   #if (!$mount_dir) {
#   #    $err = $@;
#   #    nfs_hammer_stop(id => \%id);
#   #    croak $err;
#   #}
#   my $ch = host $opts{client};
#   my $cc= connect("#telnet");
#   sayEOF($cc,"mkdir -p $opts{mount_point}");
#   sayEOF($cc,"mount [$opts{filer_ip}]:$srv_vol $opts{mount_point}");
#   my $text = sayEOF($cc,"mount");
#   logcomment("Mount value is $text"); 
#   my $mount_regex = "$opts{mount_point}.*$opts{filer_ip}"; 
#   logcomment("regex is $mount_regex");
#   if($text !~ m/$mount_regex/)
#   {
#      logcomment("Mount point failed");
#   }
#   if(not(grep(/$mount_regex/,$text))) {
#      logcomment("Mount point failed");
#      #     return 0;
#   }
#   $id{mountdir} = $opts{mount_point};
#   # This will run hammer with its output going to $LOGDIR/nfs_hammer.*.txt
#   # Stderr is not redirected, so if there is a problem, it is captured
#   # and printed at the end of the subtest, and should appear in the
#   # subtest log.
#   $hammer_path = "$mount_dir/hammer_nfs_$id{runid}";
#   $logfile = mktemp("$logdir/hammer_nfs.XXXXXX");
#   $hammer_args = "$hammer_path";
#   if ($opts{panic} eq 'yes') {
#      $hammer_args .= " -filercore " . $hostname;
#   } else {
#      # Hammer on Linux needs the nofilercore option
#      if (host($opts{client})->hosttype() =~ /linux/) {
#	 $hammer_args .= " -nofilercore ";
#      }
#   }
#   $hammer_args .= " -runtime $opts{hammer_time} -blocksize random";
#   $hammer_cmd = qq{\{
#      if (!-e \'$hammer_path\') {
#	 sayEOF($cc,"mkdir -p \'$hammer_path\'")  or die 'mkdir failed';
#      }
#      qx($opts{hammer_bin} $hammer_args > $logfile);
#   \}};
#   eval {
#      $hammer_obj =
#      subtest(
#	 $hammer_cmd,
#	 -bg,
#	 -client, $opts{client},
#	 -onexit, sub {},
#      );
#   };
#   if ($@) {
#      $err = $@;
#      #        nfs_hammer_stop(id => \%id);
#      croak ("Could not start hammer execution: $err");
#   }
#   $id{subtest} = $hammer_obj;
#   $id{logfile} = $logfile;
#   if ($mode7) {
#      $err = _verify_hammer($opts{fc}, $mode7, $opts{verify_traffic},
#      $hammer_obj, 'NFS');
#   } else {
#      $err = _verify_hammer($opts{server}, $mode7, $opts{verify_traffic},
#      $hammer_obj, 'NFS');
#   }
#   if ($err) {
#      #nfs_hammer_stop(id => \%id);
#      croak ($err);
#   }
#   return (\%id);
#}
#
#-----------------------------------------------------------------------------------------
sub _verify_hammer($$$$$;) {
   my ($obj, $mode7, $verify_traffic, $hammer_obj, $type) = @_;
   my ($avg, $end_time, $line, @lines, $msg, $next, $out, %stats);
   $msg = '';
   $end_time = time() + 60;
   if ($verify_traffic eq 'no') {
      $Log->comment("Wait 60 secs for the hammer execution to start");
      while (!$hammer_obj->is_alive() && time() < $end_time) {
	 sleep(5);
      }
      # After waiting 60 seconds make sure subtest is still running
      # and did not exit because of hammer error
      if (!$hammer_obj->is_alive()) {
	 $msg = 'Hammer subtest did not continue running';
      }
   } else {
      $Log->comment("Verify $type traffic starts up before 60 seconds");
      do {
	 if ($mode7) {
	    # Collect 2 samples of stats every 5 secs
	    %stats = filer_sysstat_get($obj,
	    {'FILER_SYSSTAT_COUNT'    => 2,
	    'FILER_SYSSTAT_INTERVAL' => 5});
	    $avg = $stats{$type}{Avg};
	 } else {
	    $out = $obj->statistics_periodic(interval   => 5,
	    iterations => 2,
	    summary    => 'true',
	    '-Timeout' => 120);
	    @lines = split(/\n/, $out);
	    $avg = $next = 0;
	    foreach $line (@lines) {
	       if ($line =~ /^Averages/) {
		  $next = 1;
		  next;
	       }
	       if ($next) {
		  # Remove whitespace from front of line
		  $line =~ s/^\s+//;
		  @lines = split(/\s+/, $line);
		  if ($type eq 'NFS') {
		     $avg = $lines[3];
		  } else {
		     $avg = $lines[4];
		  }
		  last;
	       }
	    }
	 }
      } while (($avg == 0) && (time() < $end_time));
      if ($avg == 0) {
	 $msg = "Initial $type traffic did not start up";
      }
   }
   return ($msg);
}
#---------------------------------------------------------------------
## Description: 
## This proc sets the nfs options and also does exportfs
## Parameters
## fc : Filer connection handle
## protocol: udp|tcp default:tcp transport layer  protocol on which the volume is to be mounted
## vol_name: volume name on which exportfs is to be executed
##
## Usage example:
## set_mount_options(
##    {
##    fc=>$fc,
##    protocol=>"udp",
##    vol_name=>"vol0"
##    vfiler_name=>"vfiler1"
##    })
sub set_mount_options {
    my $args = shift;
    my $filer_conn = $args->{fc};
    my $client_conn = $args->{cc};
    my $vol_name = $args->{vol_name};
    my $protocol = $args->{protocol};
    my $vfiler_name = $args->{vfiler_name}; #don't set this option if not using vfiler
    my $vfiler_str = "";
    if (defined($vfiler_name)) {
	$vfiler_str = "vfiler run $vfiler_name";
    }
    sayEOF($filer_conn,"$vfiler_str options nfs.ipv6.enable on");
    sayEOF($filer_conn,"$vfiler_str options nfs.v4.enable on");
    if (defined($protocol) && $protocol eq "udp") {
        sayEOF($filer_conn,"$vfiler_str options nfs.udp.enable on");
        sayEOF($filer_conn,"$vfiler_str options nfs.tcp.enable off");
    } else {
        sayEOF($filer_conn,"$vfiler_str options nfs.udp.enable off");
        sayEOF($filer_conn,"$vfiler_str options nfs.tcp.enable on");
    }
    sayEOF($filer_conn,"$vfiler_str options nfs.mount_rootonly off");
    sayEOF($filer_conn,"$vfiler_str nfs off");
    sayEOF($filer_conn,"$vfiler_str nfs on");
    sayEOF($filer_conn,"$vfiler_str exportfs -p sec=sys,rw,anon=0,nosuid /vol/$vol_name");
    sayEOF($filer_conn,"$vfiler_str exportfs -io sec=sys,rw,anon=0,nosuid /vol/$vol_name");
    my $val = sayEOF($filer_conn,"$vfiler_str exportfs");
    if($val !~ m/\/vol\/$vol_name\s+-sec=sys,rw,anon=0/) {
	return 0;
    }
    return 1;
}
#---------------------------------------------------------------------
## Description
## This proc mounts the volume over ipv6
## Parameters:
## fc: Filer connection handle
## cc: Client connection handle
## filer_ip: ipv6 address of filer over which to mount the volume
## vol_name: volume name of the filer
## mount_point: directory path on which the vol/$vol_name of the filer is to be mounted 
##
## Usage example:
## mount_vol_ipv6 (
##   {
##     cc=>$cc,
##     filer_ip=>$IPV6_ADDR,
##     mount_point=>"/NFStests/FILER/NFSMOUNTPT1",
##     vol_name=>"vol0"
##    })
sub mount_vol_ipv6 {
    my $args = shift;
    my $client_conn = $args->{cc};
    my $filer_ip = $args->{filer_ip};;
    my $mount_point = $args->{mount_point};
    my $vol_name = $args->{vol_name};
    logcomment("Inside mount vol function");
    #unmount if the dir is already mounted
    say($client_conn, "umount -f $mount_point");
    sleep 10;
    #sayEOF($client_conn,"rm -rf $mount_point");
    sayEOF($client_conn,"mkdir -p $mount_point");
    my $res = sayEOF($client_conn,"mount [$filer_ip]:/vol/$vol_name $mount_point");
    my $mntpoint = sayEOF($client_conn,"mount | grep $mount_point");
    if($mntpoint) {
        logcomment("Mount successful. Mount point is $mntpoint");
    }
    else {
        logcomment("Mount failed");
        return 0;
    }
    return 1;
}


1;
