package Network::Ifgrp;

use strict;
use warnings;
use Data::Dumper;
use TCD;
use Carp qw(croak);
use Tharn qw($Log);

## Export Componenet and task libraries
use NATE::BaseException qw(:try);
use NACL::C::Node;
use NACL::C::EventLog;
use NACL::C::NetworkPort;
use NACL::STask::NetworkInterface;
use NACL::STask::NetworkPortIfgrp;
use NACL::MTask::FreeDataPort;
use NACL::C::NetworkInterface;
use vars qw (@ISA @EXPORT_OK);

@ISA       = qw(Exporter);
@EXPORT_OK = qw (create_ifgrp create_lif verify_ifgrp_status);

## @name ::create_ifgrp
##
## @summary runs netstat on specified device to extract information
##
## @description
## This function creates ifgrp and returns the ifgrp object.

sub create_ifgrp {
    #my $val = shift;
    my ($class, $Node, $Ifgrp, $Node_name, %args) = @_;
    my ($lif_name, $vserver, $role, $address, $netmask);
    my $lif_creation =
      defined($args{'create_lif'}) ? $args{'create_lif'} : 'false';
    my $mode      = defined($args{'mode'})   ? $args{'mode'}   : 'singlemode';
    my $num_ports = defined($args{'number'}) ? $args{'number'} : 0;
    my $ports     = defined($args{'ports'})  ? $args{'ports'}  : undef;
    my $dist_fn   = defined($args{'distr-func'})  ? $args{'distr-func'}  : 'port';

    if ($lif_creation eq 'true') {
        $lif_name =
          defined($args{'lif_name'})
          ? $args{'lif_name'}
          : croak('lif_name is a reqd. param');
        $vserver =
          defined($args{'vserver'})
          ? $args{'vserver'}
          : croak('vserver is a reqd. param');
        $role =
          defined($args{'role'})
          ? $args{'role'}
          : croak('role is a reqd. param');
        $address =
          defined($args{'address'})
          ? $args{'address'}
          : croak('address is a reqd. param');
        $netmask =
          defined($args{'netmask'})
          ? $args{'netmask'}
          : croak('netmask is a reqd. param');
    } ## end if ($lif_creation eq 'true')
    my $home_port = $Ifgrp;
    my (@Free_p_ports, $lif);
    $Log->trace('check the availability of free ports');
    my @free_port_objs =
      NACL::MTask::FreeDataPort->discovery(command_interface => $Node);

    foreach my $port_obj (@free_port_objs) {
        push @Free_p_ports, $port_obj->port();
    }

    $Log->trace("no of available data ports" . scalar @Free_p_ports);
my $lif;
foreach my $port qw(data1 data2 cluster_mgmt) {
        #foreach my $port qw(data1 data2 ) {
if(($port eq 'data1') || ($port eq 'data2')) {
            $lif = $Node_name . "_" . $port;
} else {
    $lif = $port;
}

            try {
                my $interface =
                  NACL::C::Interface->find(command_interface => $Node,
                                           filter            => {lif => $lif},
                                           allow_empty       => 1);
                $interface->delete() if (defined $interface);
            }
            otherwise {
                $Log->warn("Exception during cleanup: $@");
            };
        } ## end foreach my $port qw(data1 data2 )
        @free_port_objs =
          NACL::MTask::FreeDataPort->discovery(command_interface => $Node);
        foreach my $port_obj (@free_port_objs) {
            push @Free_p_ports, $port_obj->port();
        }

    $Log->trace("Free_p_ports are ", Dumper(@Free_p_ports));
    $Log->trace("Check for the lifs hosted on the ifgrp $Ifgrp");
    my $ifgrp_lif;
        my $if_grp_obj =
          NACL::C::NetworkPortIfgrp->find(command_interface => $Node,
                                          filter            => {
                                                     node  => $Node,
                                                     ifgrp => $Ifgrp},
                                          allow_empty => 1);
        if (($if_grp_obj) && ($lif_creation eq 'true')) {
            $ifgrp_lif =
              NACL::C::NetworkInterface->find(command_interface => $Node,
                                                filter => {lif => $lif_name},
                                                allow_empty => 1);

        }
        if ($ifgrp_lif) {
            NACL::C::NetworkInterface->delete(command_interface => $Node,
                                              lif               => $lif_name,
                                              vserver           => $vserver);
        }

    $Log->trace("Create a $mode ifgrp");
    my $ifgrp_obj =
      NACL::STask::NetworkPortIfgrp->create(command_interface => $Node,
                                            node              => $Node_name,
                                            ifgrp             => $Ifgrp,
                                            num_ports         => $num_ports,
                                            mode              => $mode,
                                            'distr-func'      => $dist_fn,
                                            ports             => $ports,
                                            if_exists         => 'purge');

    verify_ifgrp_status($Node, $Ifgrp) if ($num_ports > 0);

    if ($lif_creation eq 'true') {
        NACL::STask::NetworkInterface->create(command_interface => $Node,
                                              lif               => $lif_name,
                                              vserver           => $vserver,
                                              role              => $role,
                                              'home-node'       => $Node_name,
                                              'home-port'       => $home_port,
                                              address           => $address,
                                              netmask           => $netmask,
                                              nacltask_if_exists=> 'purge');
    } ## end if ($lif_creation eq 'true')

    return $ifgrp_obj;
} ## end sub create_ifgrp

sub verify_ifgrp_status {

my ($Node, $ifgrp_name, %args) = @_;
    my $exp_ifgrp_status  = defined($args{'exp_ifgrp_status'})   ? $args{'exp_ifgrp_status'}   : 'up';
    my ($NetworkPort_state, $state);
    my $NetworkPortIfgrp =
      NACL::C::NetworkPortIfgrp->find(command_interface => $Node,
                                      filter            => {
                                                 node  => $Node,
                                                 ifgrp => $ifgrp_name},);
        $NetworkPort_state =
          $NetworkPortIfgrp->state(requested_fields => [qw(ifgrp up-ports activeports)]);
my $ifgrp_status = $NetworkPort_state->{'activeports'};
   if($exp_ifgrp_status eq 'up') {
croak ("the ifgrp $ifgrp_name is $exp_ifgrp_status which is not expected") unless($ifgrp_status eq 'full');
} else {
croak ("the ifgrp $ifgrp_name is $exp_ifgrp_status which is not expected") unless($ifgrp_status eq 'none');
}
} ## end sub verify_ifgrp_status

1;
