#-*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Network/Ipv6Utils.pm#2 $
package Network::Ipv6Utils;

use strict;
use warnings;

BEGIN {
    require Exporter;
    use vars qw (@ISA @EXPORT_OK);
    @ISA = qw(Exporter);
    @EXPORT_OK = qw(do_ifconfig_ipv6_filer
                verify_ifconfig_info_ipv6_filer
                do_ifconfig_ipv6_client
                ipv6_run_ping_timeout
                ipv6_run_ping
		        get_ifconfig_info_ipv6
		        vlan_create_ipv6
		        remove_ipv6_address_client
		        pktt_match
		        get_link_local_solaris
                ipv6_pmtu_cleanup
                nfs_hammer_start_ipv6
                verify_hammer
    );
};

use Tharn;
use Data::Dumper;
use Carp;
use Admin::License qw(is_licensed license);
use nfs::GenericExport qw(export_path unexport_path);
use SayWrap qw(sayEOF);
use Carp qw(croak);
use File::Temp qw(:mktemp);
use SPARSE::Util::Logger qw(logStep logDbg logTrc logWarn);
use RegistryCmds qw(registryGet registrySet);
use Params::Validate qw(validate validate_pos ARRAYREF HASHREF OBJECT SCALAR UNDEF);
use filer;
use Network::Utils qw(get_ifconfig_info verify_ifconfig_info run_ping);
use Network::IPv6 qw(_get_link_local_address _get_ra_based_address __check_pattern);
use Hammer qw(nfs_hammer_stop);
use Network::Vlan qw(vlan_create);

#---------------------------------------------------------------------
# Version for package
#---------------------------------------------------------------------
our $VERSION = '1.0';
BEGIN {
    # Die with a script error when an
    # input parameter fails validation
    Params::Validate::validation_options(
        on_fail => sub { my @msgs = @_;
                         chomp(@msgs);
                         Carp::confess(Tharn::script_result(@msgs)) }
    );
};

#------------------------------------------------------------------------
#Function Prototypes
#------------------------------------------------------------------------
sub get_link_local_solaris;
sub pktt_match;
sub vlan_create_ipv6;
sub do_ifconfig_ipv6_filer;
sub verify_ifconfig_info_ipv6_filer;
sub remove_ipv6_address_client;
sub do_ifconfig_ipv6_client;
sub ipv6_run_ping_timeout(%);
sub ipv6_run_ping ($$%);
sub get_ifconfig_info_ipv6;

##--------------------------------------------------------------------------
## Package local variable
##--------------------------------------------------------------------------
my $sudo = '/usr/software/bin/sudo';
my $ifconfig = '/usr/sbin/ifconfig';
my $linux_ifconfig = '/sbin/ifconfig';
my $mount = '/usr/sbin/mount';
my $chmod = '/usr/bin/chmod';
my $mkdir = '/usr/bin/mkdir';

=head2 get_link_local_solaris ( $cc);
Purpose of this subroutine is to fetch the 
Link local IPv6 address (addr begining with fe80)
that is autoconfigured on the sloaris client.

The following named args are accepted
=over

=item Arguments

=over

=item cc

(Required): Hostspec of the solaris client

=item Returns

Returns the link local IPv6 address IF found
Else, croaks if not found.

=cut

sub get_link_local_solaris
{

    my (@ifconfig_out, $index, @out, $flag);
    my $args = shift;
    my $cmd_out = sayEOF( $args->{cc}, "$sudo $ifconfig $args->{ifc} inet6" );
    $flag = 0;
    @ifconfig_out=split("\n",$cmd_out);
    foreach $index (@ifconfig_out)
        {
            if ($index =~ /inet6 (fe80::.*)/ig){
                @out = split("/",$1);
                $flag = 1;
                return $out[0];
            }
        }
    if($flag == 0){
	croak "The interface $args->{ifc} does not have a link local".
	    "IPv6 address autoconfigured";
    }
}

=head2 pktt_match
pktt_match( pktt_client => $pktt_client,
	    trace_file  => $trc_file,
	    patternA    => $pattern_A );
pktt_match will match either one or two set of patterns in a packet trace output.
Note that this subroutine should be called after pktt_stop.

The following arguments are accepted:

=over

=item Arguments

=over

=item pktt_client

(Required): A pktt_client SHOULD be a Linux client. Linux clients render the output of the 
tcpdump in a readable format as compared to the tcpdump out put which is in Hexadecimal format.

=back

=item trace_file

(Required): This param will contain the name of the tracefile generated from the pktt_start

=back

=item pattern_count

(Optional): This param will indicate the number of patterns that has to be found in the
pktt trace file.
Default is 1, but presently, fir NICQA needs a max of 2 patterns can be searched in the trace file.

=back

=item PatternA

(Required): This parameter will contain the (first) pattern that the user is searching  for in the 
trace file.

=back

=item PatternB

(optional): This parameter will contain the (second) pattern that the user is searching for in the 
trace file

=back

=item tcpdump_path

(Optional): This parameter will hold the value of the "which tcpdump" output on the Linux client

=back

=item Returns

Returns 1 if the pattern(s) is found.
Returns 0 if the pattern(s) is not found

=cut

sub pktt_match
{
    my ($cmd_out, $pktt_line, $match_criterion, $i, $found, @pktt_output, $line_number, $next_packet);
    $i = 0;
    $line_number = 0;
    $found=0;
    my (%opts) = validate (
                           @_,
                           {
                            pktt_client  => {type => OBJECT},
                            trace_file   => {type => SCALAR},
                            pattern_count => {type => SCALAR, default => 1},
                            patternA      => {type => SCALAR},
                            patternB      => {type => SCALAR|UNDEF, optional =>1},
                            tcpdump_path  => {type => SCALAR|UNDEF, optional =>1},
                        }
                       );
    logStep("\n!!!!!!!!!!!!!!!!!!!!!TCP Dump Start!!!!!!!!!!!!!!!\n");
    $cmd_out = sayEOF($opts{pktt_client},"$opts{tcpdump_path} -e -n -r $opts{trace_file}");
    logStep("\n!!!!!!!!!!!!!!!!!!!!TCP Dump End!!!!!!!!!!!!!!!!!!\n");
    if($opts{pattern_count} ==1){
	@pktt_output = split("\n",$cmd_out);
	foreach  $pktt_line(@pktt_output){
	    if($pktt_line =~ m/$opts{patternA}/ig){
                $found = $i++;
	    }
	}
	if( ($found >1) || ($found == 1) ) {
	    logStep(" Pattern: $opts{patternA} seen in $found packets");
	    return 1;
	}
	else{
	    return 0;
	}
    }
       elsif($opts{pattern_count} == 2){
	   @pktt_output = split("\n",$cmd_out);
	   foreach  $pktt_line(@pktt_output){
	       $line_number++;
	       if($pktt_line =~ m/$opts{patternA}/ig){
		   $next_packet=$pktt_output[$line_number];
		   if($next_packet =~ m/$opts{patternB}/ig){
		       $found =$i++;
		   }
	       }
	   }
	   if ($found >=1){
	       logStep(" Patterns: $opts{patternA} and $opts{patternB} seen in $found packets");
	       return 1;
	   }
	   else{
	       return 0;
	   }
       }
}

=head2 vlan_create_ipv6

vlan_create_ipv6(
		 fc => $fc,
		 gvrp =>'on',
		 if_name => $ifc,
		 reuse => 'no',
		 vlanid => $vlan_id
		 );

The subroutine is used to create vlan interface on the filer/client

=item Arguments

=over

=item fc

(Required): Host_spec of the client/filer

=back

=item gvrp

(Optional): Optional param, gvrp can be either turned on or off
            Default is 'off'
=back

=item if_name

(Required): Physical interface name on which VLAN has to be configured

=back

=item reuse

(Optional): reuse can be set to either NO or YES, deafult is NO
=back

=item vlanid
(Required): The VLAN ID of the VLAN interface

=back

=item Return value

Returns the VLAN interface upon successful creation, or croaks on failure.

=cut


sub vlan_create_ipv6
{
    my ($cmd, $out, $device_name, $vlanid, $if_name, $vlan_ifc, $device_ppa, $ppa);
    my (%opts) = validate(
                          @_,
                          {
                           fc      => { type => OBJECT },
                           gvrp    => { type => SCALAR, default => 'off' },
                           if_name => { type => SCALAR },
                           vlanid  => { type => SCALAR },
                       }
                      ); 
    if ($opts{fc}->hostp()->hosttype =~ /filer/) {
        # Bring the filer interface down
        do_ifconfig_ipv6_filer(
            $opts{fc},
            $opts{if_name},
            ip_addr =>"0"
        );
            
        if ($opts{gvrp} eq 'off') {
            $cmd = "vlan create -g off $opts{if_name} $opts{vlanid}";
        } elsif ($opts{gvrp} eq 'on') {
            $cmd = "vlan create -g on $opts{if_name} $opts{vlanid}";
        }
        
        $out = sayEOF($opts{fc}, $cmd);
        if ($out !~ /($opts{if_name}\-$opts{vlanid})\s+has\s+been\s+created/i) {
            croak "vlan_create error: $out";
        }
        $vlan_ifc = $1;
    } elsif ($opts{fc}->hostp()->hosttype =~ /solaris/i) {
        if( $opts{if_name} =~ /(\w+)(\d+)/ ) {
            #Device name will be "hme" or "pcn"
            $device_name =$1;
            #Device PPA will be the "0,1,2" of the "hme0,pcn1.."
            $device_ppa = $2;
        } else {
            croak "unable to extract device PPA and device name";
        }
        my $PPA = ($opts{vlanid}*1000)+$device_ppa;
        $vlan_ifc = "$device_name"."$PPA";
    } elsif ($opts{fc}->hostp()->hosttype =~ /unix\-linux/i) {
        $vlan_ifc = vlan_create(
            fc => $opts{fc}, 
            if_name => $opts{if_name},
            vlanid => $opts{vlanid});
    }
    return($vlan_ifc);
}

=head2 do_ifconfig_ipv6_filer

    do_ifconfig_ipv6_filer (
                            $conn,
                            $interface,
                            ip_addr     => $ipaddr,
                            prefixlen   => $prefix_len,
			    mtu         => $mtu,
			    flowcontrol => $flowcontrol,
			    state       => $state,
			    media_type  => $mediatype
			 );
The Subroutine is used to configure a filer interface with the various 
interface settings. The Subroutine can be used to set an IPv6 address,
set mtu,FC and mediatype settings on an interface 

The following arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required): The connection object of the Filer

=over

=item interface

(Required): The interface name under test

=over

=item ip_addr

(Optional): The IPV6 address that has to be configured on the IUT

=over

=item prefixlen

(Optional): Required param if ip_addr is being passed to the call.
            The prefix length of the IPv6 addr. Defaults to 64.

=over

=item mtusize

(Optional): The MTU size that has to be configured on the IUT.

=over

=item flowcontrol

(Optional): The FC settings on the IUT. FC setting could be
full, none, receive, send.

=over

=item state 

(Optional): The state refers to the administrative state of the IUT.
Could be ON or OFF.

=over

=item media_type

(Optional): The mediatype to be configured on the IUT.

=over

=item Returns

1, if the configuration was successful, croaks on failure.

=cut


sub do_ifconfig_ipv6_filer
{
    my ($conn, $interface, %options) = @_;
    my $hostp = $conn->hostp;
    my $hostname = $hostp->hostname;
    my $ifconfig_cmd = undef;
    my $ipaddr = $options{ip_addr};
    my $prefixlen = $options{'prefixlen'} || "64";
    my $mtusize = $options{'mtusize'};
    my $flowcontrol = $options{'flowcontrol'};
    my $state = $options{'state'};
    my $media_type = $options{'media_type'};
    if (defined($options{flowcontrol}) &&
        $options{flowcontrol} !~ /^(none|receive|send|full)$/) {
        croak "$options{flowcontrol} is invalid value for flowcontrol";
    }

    # If state is down, invoke ifconfig <ifc> down
    if ( defined($options{state})) {
        if ($options{state} eq 'down') {
	    sayEOF($conn, "ifconfig $interface down");
	    verify_ifconfig_info($conn,  $interface,
				 flags => 'DOWN' )
		and croak "verify_ifconfig_info failed - $@";
        }elsif ($options{state} eq 'up' && !defined($ipaddr)
	       && !defined($options{'alias'})) {
	    my $cmd_out = sayEOF($conn, "ifconfig $interface up");
	    verify_ifconfig_info($conn,  $interface,
				 flags => 'UP' )
		or croak "verify_ifconfig_info failed - $cmd_out";
	}
    }
    if (defined($ipaddr))
    {
        # If ip_addr is specified, then set the ip addr on the interface
	my $cmd_str = undef;
        if ( $ipaddr eq "0" )
	{
            # Set state to whatever caller specified defaulting to down
            if (defined($options{state})) {
                $state = $options{state};
            }else {
                $state = 'down';
	    }
	    my $cmd_out = sayEOF($conn,"ifconfig $interface inet6 $ipaddr $state");
	    verify_ifconfig_info_ipv6_filer($conn,  $interface, ip_addr => "0")
		or croak "verify_ifconfig_info failed - $cmd_out";
	}else
	{
	    $cmd_str =
		" $interface inet6 $ipaddr prefixlen $prefixlen $options{'state'}";
	    logDbg("invoking the command: ifconfig $cmd_str");
	    my $cmd_out = sayEOF($conn, "ifconfig $cmd_str");
	    if ( $cmd_out =~ m/\w/ ) {# cmd is not expected to print any output.
                logWarn("ifconfig printed some output: $cmd_out");
            }
	    verify_ifconfig_info_ipv6_filer( $conn, $interface,ip_addr => $ipaddr)
                or croak "verify_ifconfig_info failed - $cmd_out";
	}
    }
    if(defined($options{mtusize}))
    {
	my $cmd_str = "inet6 mtusize $options{mtusize}";
	my $cmd_out = sayEOF($conn, "ifconfig $interface $cmd_str");
	if($cmd_out =~ m/\w/){
	    logWarn("Filer ifconfig does not support the MTU size  $options{mtusize}");
	}
    }

    #If media_type is specified, configure media_type only on the filer
    if (defined($options{media_type}))
    {
	my $cmd_str = "mediatype $options{'media_type'}";
	my $cmd_out = sayEOF($conn, "ifconfig $interface $cmd_str");
	if ($cmd_out =~ /no such media type/i) {
	    croak "Filer ifconfig does not support mediatype " .
		"$options{'media_type'}";
	}
	# Changing media type can take a little while to take effect so
	# poll for up to 60 seconds to do the verification.
	my $retry = 12;
	while ($retry > 0) {
	    sleep(5);
	    if (verify_ifconfig_info($conn, $interface,
				     media_type => $options{'media_type'})) {
		$retry = -1;
	    } else {
		$retry--;
	    }
	}
	croak "verify_ifconfig_info failed setting mediatype " .
	    "$options{media_type}" if ($retry == 0);
    }
    #If flowcontrol is specified, configure flowcontrol
    if (defined($options{flowcontrol})) {
	my $cmd_str = "flowcontrol $options{'flowcontrol'}";
	my $cmd_out = sayEOF($conn, "ifconfig $interface $cmd_str");
	verify_ifconfig_info($conn, $interface,
			     flowcontrol => $options{'flowcontrol'})
	    or croak "verify_ifconfig_info failed - $cmd_out";
    }
    return 1;
}

## @description gets the inconfig_info using get_ifconfig_info() and compares
## the parameters passed for the specified connection.
## @arg $conn - connection object
## @arg $interface - name of the interface
## @arg %args - hash of various options
##            "ip_addr" - ipv6 address to configure
##            "flowcontrol"- 'flowcontrol setting none|receive|send|full'
##            "media_type" - 'media type to set to, if it contains the word
##                            auto then it is set to auto'
##            "state"   - 'up' or 'down' 
## @return 1 upon success or 0 upon failure.

sub verify_ifconfig_info_ipv6_filer
{
    my($fc, $interface, %args) = @_;
    sleep(10);
    #my $ifconfig_info = get_ifconfig_info_ipv6($fc, $interface) or
    my $ifconfig_info = get_ifconfig_info($fc, $interface) or
        croak "get_ifconfig_info failed - $@";
    if ( !exists($ifconfig_info->{$interface}) ) {
        return 0;
        # Verify the state
        if ( defined($args{state}) ) {
            if ( $ifconfig_info->{$interface}{state} ne $args{state} ) {
                logDbg("ifconfig_info contained invalid state");
                logDbg(Dumper($ifconfig_info->{$interface}));
                return 0;
            }
            logDbg("ifconfig_info has valid value for state");
        }
        # Verify the ip addr
        if ( defined($args{ip_addr}) ) {
            if ( $args{ip_addr} eq "0") {

                if ( exists($ifconfig_info->{$interface}{'ip_addr'})) {
                    logDbg("ifconfig_info contained ip addr");
                    return 0;
                }
            } else {

                # XXX\gajria- burt184737 prevents the tagging of ip addresses
                # as base address or alias because the addresses are displayed
                # unordered. So here we compare ip_addr to all addresses
                # returned by ifconfig.

                if ($ifconfig_info->{$interface}{'ip6_addr_UA'} ne $args{ip_addr}
                    && !(exists
                         ($ifconfig_info->{$interface}{alias}{$args{ip_addr}}))){
                    logDbg("ifconfig_info contained invalid ip addr");
                    logDbg(Dumper($ifconfig_info->{$interface}));
                    return 0;
                }
                logDbg("ifconfig_info has valid value for ip_addr");
            }
        }
    }
    # Verify the flags
    if ( defined($args{flags}) ) {
	if ( $ifconfig_info->{$interface}{flags} !~ m/$args{'flags'}/i ) {
	    logDbg("ifconfig_info does not contain expected flags");
                logDbg(Dumper($ifconfig_info->{$interface}));
	    return 0;
	}
	logDbg("ifconfig_info has valid value for flags");
    }
    # XXX\gajria- burt184737 prevents the tagging of ip addresses
    # as base address or alias because the addresses are displayed
    # unordered. So here we compare ip_addr to all addresses
    # returned by ifconfig.
    if (defined($args{alias})) {
            if (!exists($ifconfig_info->{$interface}{'alias'}{$args{'alias'}})
                and $ifconfig_info->{$interface}{'ip_addr'} ne $args{'alias'}){
                return 0;
            }
            logDbg("ifconfig_info has information of alias");
    }
        if (defined($args{media_type})) {
            if (exists($ifconfig_info->{$interface}{'media_type'})) {
                if ($args{'media_type'} eq 'auto') {
                    if ($ifconfig_info->{$interface}{'auto'} == 0) {
                        return 0;
                    }
                } elsif ($ifconfig_info->{$interface}{'media_type'} !~
                         /$args{'media_type'}/){
                    return 0;
                }
            }
	    logDbg("ifconfig_info has valid value for mediatype");
        }
    if (defined($args{flowcontrol})) {
	if (exists($ifconfig_info->{$interface}{'flowcontrol'})) {
                if ($ifconfig_info->{$interface}{'flowcontrol'} ne $args{'flowcontrol'}){
                    return 0;
                }
	}
            logDbg("ifconfig_info has valid value for flowcontrol");
    }
    return 1;
}

=head2 remove_ipv6_address_client

remove_ipv6_address_client( $conn,
			    $ifc, 
			    ip_addr => $ip_address
			    state => $state 
			    );
Removes the ipv6 address on the SOLARIS Client using the "removeif" command

=over

=item Arguments

=over

=item conn

(Required): Connection object of the client

=over

=item interface

(Required): interface name 

=over

=item ip_addr

(Required): ipv6 address to be removed

=over

=item state

(Required): The state at which the interface should remain after removal of ip address

=over

=item Returns

none, croaks on failure

=cut

sub remove_ipv6_address_client
{
    my ($conn, $interface, %options) = @_;
    if(!defined($interface) || !defined($conn) ){
	die "remove_ipv6_address_client: client interface and conn".
            "fields were not passed in the call to this subroutine";
    }
    my ($cmd_remif, $cmd);
    if ($conn->hostp()->hosttype =~ /(solaris)/) {
        $cmd = "$sudo $ifconfig $interface inet6 removeif $options{ip_addr} $options{state}";
    } elsif ($conn->hostp()->hosttype =~ /(unix\-linux)/){
        $cmd = "$sudo $linux_ifconfig $interface inet6 del $options{ip_addr}/$options{prefixlen}";
    }
    $cmd_remif =sayEOF($conn,$cmd);
    if ($cmd_remif =~ /Cannot assign requested address/){
	logStep ("IP address on $interface cannot be removed.
               Reason: $cmd_remif");
        return 0;
    }else{
        return 1;
    }
}

=head2 do_ifconfig_ipv6_client

do_ifconfig_ipv6_client ($conn,
			 $inetrface,
			 state      => $state,
			 ip_addr    => $ip_address,
			 prefix_len => $prefix_length,
			 mtu        => $mtusize
			 );
The Subroutine is used to set IPv6 address on the client interface,
set the interface state to up/down, set the mtu on the interface.

=over

=item Arguments

=over

=item conn

(Required): The connection object of the client interface

=over

=item interface

(Required): The interface name on the client

=over

=item state

(Optional): The state of the interface. Values can be ON or OFF

=over

=item ip_addr

(Optional): IPv6 address to be configured on the client interface

=over

=item prefix_len 

(Optional): The prefix length of the IPv6 address

=over

=item mtu

(Optional): The mtu to be set on the client interface

=over

=item Returns

None, or croaks on failure

=cut

sub do_ifconfig_ipv6_client {
    my ($conn, $interface, %options) = @_;
    if(!defined($interface) || !defined($conn) ){
	    die "do_ifconfig_ipv6_client: client interface and conn fileds".
            "were not passed in the call to this subroutine";
    }
    my ($cmd_plumb, $cmd_add, $cmd_unplumb, $cmd);
    if((defined($options{state})) && ($options{state} eq "up")){
        if ($conn->hostp()->hosttype =~ /solaris/){
	        $cmd_plumb= sayEOF($conn, "$sudo $ifconfig $interface inet6 plumb up");
        }
	    if( (defined($options{ip_addr})) && (defined($options{prefix_len})) ) {
            if ($conn->hostp()->hosttype =~ /solaris/){
                $cmd_add = "$sudo $ifconfig $interface inet6 addif".
                        " $options{ip_addr}/$options{prefix_len} $options{state}";
            } elsif($conn->hostp()->hosttype =~ /unix\-linux/){
                $cmd_add = "$sudo $linux_ifconfig $interface inet6 add".
                        " $options{ip_addr}/$options{prefix_len} $options{state}";
            }
	        if(defined($options{mtu})) {
                $cmd_add = $cmd_add . " mtu $options{mtu}";
            }
            my $cmd_addif = sayEOF($conn,$cmd_add);
	        if ($cmd_addif =~ /Cannot assign requested address/){
		        croak "Interface created but could not assign request IP address.
                           Reason:IP address already exists";
	        }
	    } else {
	        logStep "The client interface was PLUMBED UP, but".
                    ", neither IP address was assigned to it nor removed";
	    }
    } 
    
    if ( defined($options{mtu}) ){
        if ($conn->hostp()->hosttype =~ /solaris/){
            $cmd = "$sudo $ifconfig $interface inet6 mtu $options{mtu}";
        } elsif($conn->hostp()->hosttype =~ /unix\-linux/){
            $cmd = "$sudo $linux_ifconfig $interface inet6 mtu $options{mtu}";
        }
	    my $cmd = sayEOF($conn,$cmd );
	    if($cmd =~ /Invalid argument/){
	        croak " Trying to set an invalid mtu size on the interface: $cmd";
	    }
    }
    
    if( (defined($options{state}) ) && ($options{state} eq "down") ) {
	    if ($conn->hostp()->hosttype =~ /solaris/) {
            $cmd_plumb = "$sudo $ifconfig $interface inet6 unplumb";
        } elsif($conn->hostp()->hosttype =~ /unix\-linux/) { 
            $cmd_plumb = "$sudo $linux_ifconfig $interface inet6 down";
        }
        $cmd_unplumb = sayEOF($conn, $cmd_plumb);

	    if ($cmd_unplumb =~ m/no such interface/) {
	        croak "You are trying to UNPLUMB an interface which doesn't exist, Reason: $cmd_unplumb";
	    }
    }
    return 1;
}
=head2 ipv6_run_ping_timeout

ipv6_run_ping_timeout( src_conn => $cc,
		       dst_ip   => $destination_ip,
		       count    => $count,
		       timeout  => $timeout
		       );

This subroutine inturn calls the ipv6_run_ping, waits FOR 
timeout number of seconds FOR the ping to go through before failing.

=over

=item Arguments:

=over

=item src_conn 

Source connection, from which the ping originates

=over

=item dst_ip

The Destination IP which needs to be probed

=over

=item count 

The cumber of times the ping command has to be invoked onto dest_ip

=over

=item timeout 

The timeout seconds after which the ping will be declared as failure

=over

=item Returns

1 on successful ping, 0 on ping failure

=cut


sub ipv6_run_ping_timeout(%) {
    my (%opts) = validate (
        @_,
        {
            src_conn  => { type  => OBJECT },
            dst_ip    => { type  => SCALAR },
	        count     => { type  => SCALAR, default => 2 },
	        interface => { type  => SCALAR, optional => 1 },
            timeout   => { type  => SCALAR, default => 300, regex => qr/\d+/},
        }
    );

    my ($end_time, $ping);

    $end_time = time() + $opts{timeout};
    while (time() < $end_time) {
	$ping = 0;
        eval {
            $ping = ipv6_run_ping($opts{src_conn}, $opts{dst_ip},
                        count => $opts{count},
                        interface => $opts{interface},
            );
        };
        if ($ping > 0) {
            return (1);
        } else {
            sleep(5);
        }
    }
    return (0);
}

=head2 ipv6_run_ping

ipv6_run_ping($conn,
	      $destn,
	      count => $count);
This subroutine will ping the client IP (Dest) from the filer (conn)

=over

=item Arguments

=over

=item conn

Connection object of the filer

=over

=item destn

Destination (client) Ip address

=over

=item count

The number of times the ping needs to be invoked

=over

=item Returns

1 upon successful ping, 0 on ping failure

=cut


sub ipv6_run_ping ($$%) {
    my ($conn,$destn,%args) = @_;
    my $count = $args{'count'};
    my $packet_size = $args{'pkt_size'};
    my $interface = $args{'interface'};
    my @output;
    my $flags = "";
    my $hosttype = $conn->hostp->hosttype();
    my $prepend ="";
    if (($hosttype =~ /filer/i)||($hosttype =~ /linux/i)||($hosttype =~ /unix-linux/i)){
        if ($hosttype =~ /filer/) {
            $prepend = "priv set -q advanced ; ";
        } elsif (($hosttype =~ /linux/i) || ($hosttype=~ /unix-linux/i)) {
            $prepend = "/bin/";
        }
	    if(defined ($count)){
	        $flags = "-c $count"
	    }
        if (defined ($packet_size)) {
            if ($hosttype =~ /filer/) {
                $flags = "$flags"." -d $packet_size";
            } elsif (($hosttype =~ /linux/i) || ($hosttype=~ /unix-linux/i)) {
                $flags = "$flags"." -s $packet_size";
            }
        }
        if(defined($interface)) {
            if ($hosttype =~ /filer/) {
                $flags = "$flags"." -i $interface"; 
            } elsif (($hosttype =~ /linux/i) || ($hosttype=~ /unix-linux/i)) {
                $flags = "$flags"." -I $interface"; 
            }
        }
	    @output = say($conn, "$prepend"."ping6 $flags $destn");
	    if (($output[2] =~ /is alive/) || ($output[2] =~ /(\d+)\sbytes\sfrom\s$destn/)) {
	        return (1);
	    } else {
	        return (0);
	    }
    } elsif ($hosttype =~ /unix-solaris/i) {
        $prepend = "/usr/sbin/ping";
        if(defined($packet_size)) {
            $flags = $packet_size;
        }

        if (defined($count)) {
            $flags = "$flags $count";
        }
        
        my $opts = "";
        if(defined($interface)) {
            $opts = "-i $interface"; 
        }

        @output = say($conn, "$prepend $opts $destn $flags");
	    if ($output[2] =~ /is alive/) {
	        return (1);
	    } else {
	        return (0);
	    }
    } else {
	    croak "Make sure you are using this subroutine to ping FROM the filer or linux client";
    }
}

## @description invokes ifconfig command and gathers the information about
## specified interface (IPv6 information) or all interfaces in a hash and returns the reference
## to hash.
## @arg $conn - connection object
## @arg $interface - name of the interface (optional, defaults to all interface)
## @return reference to hash upon success else croaks.
## example: get_ifconfig_info($conn);
##
sub get_ifconfig_info_ipv6{
    my ($conn, $interface) = @_;
    my $host_type = 0;
    my $filer_out;
    my %ifconfig_info = ();
    my $ifconfig_cmd = undef;
    if ( ! defined($interface) ) {
        croak "get_ifconfig_info_ipv6: No interface mentioned";
    }
    $ifconfig_cmd ="ifconfig";
    my $hosttype = $conn->hostp->hosttype();
    if ($hosttype =~ /solaris/) {
        $host_type = 1;
    } elsif ($hosttype =~ /linux/) { 
        $ifconfig_cmd ="/sbin/ifconfig";
    }

    if ($host_type == 1) {
        $filer_out = sayEOF($conn, "$sudo $ifconfig $interface inet6");
    } else {
        $filer_out = sayEOF($conn, "$ifconfig_cmd $interface");
    }

    if ($filer_out =~ /(no such interface|device not found)/i) {
        croak $conn->hostp()->hostname . ": $filer_out";
    }
    
    my @ifconfig_out = split(/\n/, $filer_out);
    $interface = undef; 
    foreach my $line ( @ifconfig_out ) {
	    # Skip blank lines
	    next if ( $line =~ m/^\s*?$/ );
	    if ( $line =~ m/^(.*):\sflags=(.*>)\smtu\s(\d+)/ ) {
	        # This 'if' works for Filer and Solaris
	        $interface = $1;
            $ifconfig_info{$interface}{flags} = $2;
	        $ifconfig_info{$interface}{mtu} = $3;
	    } elsif ($line =~ m/^(.*)\s+Link encap\:Ethernet\s+HWaddr/i) {
            $interface = $1;
            $interface  =~ s/^\s+//;
            $interface  =~ s/\s+$//;
        } elsif ( $line =~ m/inet\s((\d+\.){3}(\d+))\snetmask(?:-or-prefix)?\s(.*?)\s(broadcast\s((\d+\.){3}(\d+)))?/ ) {
	        if (!exists($ifconfig_info{$interface}{'ip4_addr'})) {
		        $ifconfig_info{$interface}{ip4_addr} = $1;
		        $ifconfig_info{$interface}{ip4_netmask} = $4;
		        $ifconfig_info{$interface}{ip4_broadcast} = $6 if defined($6);
	        } else{
		        # alias line
		        $ifconfig_info{$interface}{'alias'}{$1}{'netmask'} = $4;
		        $ifconfig_info{$interface}{'alias'}{$1}{'broadcast'} = $6
		        if defined($6);
	        }
	    } elsif (($line =~ m/inet6\s(fe80::.*)\sprefixlen\s(\d+)/ )) {
	        $ifconfig_info{$interface}{ip6_addr_LLA} = $1;
	        $ifconfig_info{$interface}{ip6_addr_LLA_prefixlen} =$2;
	    } elsif (($line =~ m/inet6\s(fe80::.*)(\/)(\d+)/ )) {
            $ifconfig_info{$interface}{ip6_addr_LLA} = $1;
            $ifconfig_info{$interface}{ip6_addr_LLA_prefixlen} =$3;
        } elsif ($line =~ m/inet6\saddr\:\s(fe80::.*)\/(\d+)\sScope\:Link/i) {
            $ifconfig_info{$interface}{ip6_addr_LLA} = $1;
            $ifconfig_info{$interface}{ip6_addr_LLA_prefixlen} = $2;          
        } elsif ($line =~ m/inet6\s(fd20:.*)\sprefixlen\s(\d+)/){
	        $ifconfig_info{$interface}{ip6_addr_RA} =$1;
	        $ifconfig_info{$interface}{ip6_addr_RA_prefixlen} =$2;
	    } elsif ($line =~ m/inet6\s((\d+\:){2}(:\d+))\sprefixlen\s((\d+))/ ){
	        $ifconfig_info{$interface}{ip6_addr_UA} =$1;
	        $ifconfig_info{$interface}{ip6_addr_UA_prefixlen} =$4;
	    } elsif ( $line =~ m/ether\s((\w+:){5}(\w+))\s\((.*)\)\sflowcontrol\s(\w+)/ ) {
	        $ifconfig_info{$interface}{mac_addr} = $1;
	        $ifconfig_info{$interface}{media_type} = $4;
	        $ifconfig_info{$interface}{flowcontrol} = $5;
	        # Parse media_type into sub fields
	        my @values = split('-', $ifconfig_info{$interface}{media_type});
	        foreach my $val (@values) {
		        if ($val eq 'auto') {
                    $ifconfig_info{$interface}{auto} = 1;
                } elsif ($val eq 'tp') {
                    $ifconfig_info{$interface}{speed} = 10;
                    $ifconfig_info{$interface}{media} = $val;
		        } elsif ($val eq 'up') {
                    $ifconfig_info{$interface}{up} = 1;
                } elsif ($val =~ /down/) {
                    $ifconfig_info{$interface}{up} = 0;
                } elsif ($val eq 'fd') {
		            $ifconfig_info{$interface}{duplex} = 'full';
                } elsif ($val =~ /^(\d+)(.*)/) {
                    $ifconfig_info{$interface}{speed} = $1;
                    $ifconfig_info{$interface}{media} = $2;
                }
            }
	        my $tmp = $ifconfig_info{$interface}{speed};
	        if ((($tmp == 10) || ($tmp == 100)) &&
		        !defined($ifconfig_info{$interface}{duplex})) {
		        $ifconfig_info{$interface}{duplex} = 'half';
	        }
	        # Set media type to a value that the filer would accept on
	        # an ifconfig command. Strip auto and up/down status.
	        $tmp = $ifconfig_info{$interface}{media_type};
	        $tmp =~ s/^auto-//;
	        $tmp =~ s/-(up|down)//;
	        $ifconfig_info{$interface}{media_type}= $tmp;
	    } elsif ( $line =~ m/partner\s+inet\s+((\d+\.){3}(\d+))\s+\((.*)\)/ ) {
		   # This 'if' works for Filer
		   $ifconfig_info{$interface}{partner_ip_addr} = $1;
		   $ifconfig_info{$interface}{partner_ifc} = $2;
	    } else {
		   logDbg("get_ifconfig_info: unrecognized line in output: $line\n");
		   next;
	    }
    }
	if ( !(%ifconfig_info)) {
	    return 0;
	}

	return(\%ifconfig_info);
}

=head2 ipv6_pmtu_cleanup

ipv6_pmtu_cleanup{
        %setup_h
}
This subrioutine will cleanup the configs on the router,client,filer setup by 
ipv6_pmtu_setup() subroutine
=over

=item Arguments

=over

=item setup_h

The setup_h hash will contain the setup config for router,client and filer
This hash is populated by ipv6_pmtu_setup() subroutine

=over

=item Returns

=over

The subroutine issues Warning messages wherever applicable.
The subroutine returns after completion.

=cut

sub ipv6_pmtu_cleanup{
    my (%opts) = validate (
        @_,
        {
            setup_h          => { type => HASHREF },
        }
    );
    my $set_h  = $opts{setup_h};
    my $fc     = $set_h->{fc};
    my $cc     = $set_h->{ifc_list}[0]{cc};
    my $rc     = $set_h->{ifc_list}[0]{rc};
    my $ifc    = $set_h->{ifc_list}[0]{ifc};
    my $ifc_ip = $set_h->{ifc_list}[0]{ifc_ip};
    my $f_net  = $set_h->{ifc_list}[0]{filer_network};
    my $cli_net= $set_h->{ifc_list}[0]{client_network};
    my $rf_ifc = $set_h->{ifc_list}[0]{router_filer_int};
    my $rc_ifc = $set_h->{ifc_list}[0]{router_client_int};
    my $rf_ip  = $set_h->{ifc_list}[0]{router_filer_ip};
    my $rc_ip  = $set_h->{ifc_list}[0]{router_client_ip};
    my $prefix = $set_h->{ifc_list}[0]{router_client_prefixlen};
    #Cleanup the filer

    #Zero the IPv6 address on the filer IFC.
    eval{
        sayEOF($fc,"ifconfig $ifc inet6 0");
    };
    logWarn("ipv6_pmtu_cleanup: Cannot zero the $ifc IPV6 address")if($@);

    #Delete the IPv6 route(Network)
    if (defined ($cli_net)) {
        eval{
            sayEOF($fc,"route delete inet6 net $cli_net"."::");
        };
        logWarn("ipv6_pmtu_cleanup: Cannot delete network route ($cli_net)")if($@);
    }
    if (defined ($f_net)) {
        eval{
            sayEOF($fc,"route delete inet6 net $f_net"."::");
        };
        logWarn("ipv6_pmtu_cleanup: Cannot delete network route ($f_net)")if($@);
    }

    #Delete the IPv6 route (Host)

    if (defined ($ifc_ip)) {
        eval{
            sayEOF($fc,"route delete inet6 host $ifc_ip");
        };
        logWarn("ipv6_pmtu_cleanup: Cannot delete host route ($ifc_ip)")if($@);
    }
    if (defined ($rf_ip)) {
        eval{
            sayEOF($fc, "route delete inet6 host $rf_ip");
        };
        logWarn("ipv6_pmtu_cleanup: Cannot delete host route ($rf_ip)")if($@);
    }

    #Delete the inerface IP on the router

    if (defined($rf_ip) && defined($rf_ifc) ) {
        eval{
            sayEOF($rc, "$sudo /sbin/ifconfig $rf_ifc del $rf_ip/$prefix");
        };
        logWarn("ipv6_pmtu_cleanup: Cannot delete router filer IP ($rf_ip)")
            if($@);
    }
    if (defined($rc_ifc) && defined($rc_ip)) {
        eval{
            sayEOF($rc, "$sudo /sbin/ifconfig $rc_ifc del $rc_ip/$prefix");
        };
        logWarn("ipv6_pmtu_cleanup: Cannot delete router client IP ($rc_ip)")
            if($@);
    }
    return;
}
#---------------------------------------------------------------------------------
sub nfs_hammer_start_ipv6(%) {
    my (%opts) = validate (
        @_,
        {
            fc             => { type => OBJECT, optional => 1 },
            client         => { type => SCALAR, default => 'localhost' },
            filer_ip       => { type => SCALAR, optional => 1 },
            hammer_bin     => { type => SCALAR,
                                default => '/usr/software/test/bin/hammer' },
            hammer_time    => { type => SCALAR, default => -1 },
            panic          => { type => SCALAR, default => 'yes',
                                regex => qr/^(no|yes)$/ },
            proto          => { type => SCALAR, default => 'tcp',
                                regex => qr/^(udp|tcp)$/ },
            rsize          => { type => SCALAR, default => 32768,
                                regex => qr/^\d+$/ },
            server         => { type => OBJECT, optional => 1 },
            version        => { type => SCALAR, default => 3 },
            verify_traffic => { type => SCALAR, default => 'no',
                                regex => qr/^(yes|no)$/ },
            vol_name       => { type => SCALAR },
            mount_point    => { type => SCALAR },  
            wsize          => { type => SCALAR, default => 32768,
                                regex => qr/^\d+$/ },
        }
    );
    my ($end_time, $err, %fields, $hammer_args, $hammer_cmd, $hammer_obj,
        $hammer_path, $hostname, %id, $line, $logdir, $logfile, $mode7,
        $mount_dir, $server, $srv_vol);
    my $Nfs_hammer_runid = 0;
    if (!defined($opts{fc}) && !defined($opts{server})) {
        croak('nfs_hammer_start: Either the fc or server parameter must ' .
                  'be specified.');
    }
    $Log->comment("nfs_hammer_start: It's NFS Hammer Time!");
    if (defined($ENV{LOGDIR})) {
        $logdir = $ENV{LOGDIR};
    } else {
        croak ("Problem with NATE - LOGDIR not set in env");
    }
    # If a Server.pm object was specified then see if filer is 7 mode. If
    # so then use the filer connection from the Server.pm object. If a
    # Server.pm object was not passed in then filer is 7-mode.
    if (defined($opts{server})) {
        $opts{fc} = _server_to_fc($opts{server});
        $mode7 = _is_7_mode($opts{server});
    } else {
        $mode7 = 1;
    }
    $id{licensed} = 0;
    $id{client} = $opts{client};
    if ($mode7) {
        # 7 mode
        $id{fc} = $opts{fc};
        if (!is_licensed($opts{fc}, 'nfs')) {
            $id{licensed} = 1;
            license($opts{fc}, 'nfs');
            croak $@ if $@;
        }
        export_path(
            $opts{fc},
            "/vol/$opts{vol_name}",
            OPTIONS => 'rw,anon=0',
        ) or croak $@;
        $id{export} = "/vol/$opts{vol_name}";
        # Default the IP address to send traffic to the filer to be the
        # filer default IP address if not specified as an option.
        if (!defined($opts{filer_ip})) {
            $opts{filer_ip} = $opts{fc}->hostp->default_ip;
        }
        $srv_vol = "/vol/$opts{vol_name}";
    } else {
        # 10 mode
        $server = $id{server} = $opts{server};
        # Grab the DATA_IP1 address for the NFS servers IP address
        %fields = $opts{fc}->hostp()->param_fields();
        if (!defined($fields{DATA_IP1})) {
            croak("Running hammer on 10-mode filer requires the DATA_IP1 " .
                      "parameter to be defined in the filers hostspec");
        }
        if (!defined($opts{filer_ip})) {
            $opts{filer_ip} = $fields{DATA_IP1};
        }
        $srv_vol = "/$opts{vol_name}";
    }
    $hostname = $opts{fc}->hostp()->hostname();
    $id{runid} = ++$Nfs_hammer_runid;
    my $ch = host $opts{client};
    my $cc = connect('#rsh', -timeout, 300);
    $opts{mount_point} = "$opts{mount_point}/$opts{vol_name}";
    my $mount_regex = "$opts{mount_point}.*$opts{filer_ip}"; 

    if ( $cc->hostp->hosttype() =~ /linux/ ) {
        $mount = '/bin/mount';
        $chmod = '/bin/chmod';
        $mkdir = '/bin/mkdir';
        $mount_regex = "[$opts{filer_ip}]"; 
    }

    sayEOF($cc,"$sudo $mkdir -p $opts{mount_point}");
    sayEOF($cc,"$sudo $mount [$opts{filer_ip}]:$srv_vol $opts{mount_point}");
    sayEOF($cc,"$sudo $chmod 0777 $opts{mount_point}");
    my $text = sayEOF($cc,"$mount");
    if($text =~ m/$mount_regex/) {
        logcomment("Mount Successful");
    } else {
        croak("Mount point failed");
    }
    
    $id{mountdir} = $opts{mount_point};
    # This will run hammer with its output going to $LOGDIR/nfs_hammer.*.txt
    # Stderr is not redirected, so if there is a problem, it is captured
    # and printed at the end of the subtest, and should appear in the
    # subtest log.
    $hammer_path = "$opts{mount_point}";
    $logfile = mktemp("$logdir/hammer_nfs.XXXXXX");
    $hammer_args = "$hammer_path";
    if ($opts{panic} eq 'yes') {
        $hammer_args .= " -filercore " . $hostname ;
    } else {
            $hammer_args .= " -nofilercore ";
    }

    $hammer_args .= " -runtime $opts{hammer_time} -blocksize random -logfile $logfile";
    $hammer_cmd = "$opts{hammer_bin} $hammer_args";
    logcomment("hammer command: $hammer_cmd");

    eval {
        $hammer_obj =
            subtest(
                $hammer_cmd,
                -bg,
                -client, $opts{client},
                -onexit, sub {},
            );
    };
    if ($@) {
        $err = $@;
        nfs_hammer_stop(id => \%id);
        croak ("Could not start hammer execution: $err");
    }

    logcomment("Hammer has been started as subtest");
    $id{subtest} = $hammer_obj;
    $id{logfile} = $logfile;
    logcomment("logfile is $logfile");
    if ($mode7) {
        $err = verify_hammer($opts{fc}, $mode7, $opts{verify_traffic},
                              $hammer_obj, 'NFS');
    } else {
        $err = verify_hammer($opts{server}, $mode7, $opts{verify_traffic},
                              $hammer_obj, 'NFS');
    }
    if ($err) {
        nfs_hammer_stop(id => \%id);
        croak ($err);
    }
    return (\%id);
}

#---------------------------------------------------------------------------------
# sub verify_hammer()
#---------------------------------------------------------------------------------
sub verify_hammer($$$$$) {
    my ($obj, $mode7, $verify_traffic, $hammer_obj, $type) = @_;
    my ($avg, $end_time, $line, @lines, $msg, $next, $out, %stats);
    $msg = '';
    $end_time = time() + 60;
    if ($verify_traffic eq 'no') {
        $Log->comment("Wait 60 secs for the hammer execution to start");
        while (!$hammer_obj->is_alive() && time() < $end_time) {
            sleep(5);
        }
        if (!$hammer_obj->is_alive()) {
            $msg = 'Hammer subtest did not continue running';
        }
    }else {
        $Log->comment("Verify $type traffic starts up before 60 seconds");
        do {
            if ($mode7) {
                %stats = filer_sysstat_get($obj,
                                           {'FILER_SYSSTAT_COUNT'    => 2,
                                            'FILER_SYSSTAT_INTERVAL' => 5});
                $avg = $stats{$type}{Avg};
            }else {
                $out = $obj->statistics_periodic(interval   => 5,
                                                 iterations => 2,
                                                 summary    => 'true',
                                                 '-Timeout' => 120);
                @lines = split(/\n/, $out);
                $avg = $next = 0;
                foreach $line (@lines) {
                    if ($line =~ /^Averages/) {
                        $next = 1;
                        next;
                    }
                    if ($next) {
                        # Remove whitespace from front of line
                        $line =~ s/^\s+//;
                        @lines = split(/\s+/, $line);
                        if ($type eq 'NFS') {
                            $avg = $lines[3];
                        }else {
                            $avg = $lines[4];
                        }
                        last;
                    }
                }
            }
        } while (($avg == 0) && (time() < $end_time));
        if ($avg == 0) {
            $msg = "Initial $type traffic did not start up";
        }
    }
    return;
}

1;
