#!/usr/bin/perl -w
# $Id: //depot/prod/test/Rsierranevada/lib/Network/Routes.pm#1 $
#
# Copyright (c) 2010 Network Appliance Inc.
# All right reserved.
#
## @summary Routines for running route commands
##
## @description
##         These functions are used to add/delete routes in sk and also to get routes from sk and bsd.
##          And also to login to systemshell
##        These functions help in abstracting the test scripts
##        from subtle differences in command usage and command
##        outputs on various platforms.
##
## @dependencies
##
## use Tharn;
##
## @author  dl-nwkg-test (SCON QA)

package Network::Routes;

require Exporter;
use strict;
use Data::Dumper;
use Tharn;
use vars qw (@ISA @EXPORT_OK);
@ISA = qw(Exporter);
@EXPORT_OK = qw (login_systemshell
                 get_routes_bsd
                 get_routes_sk
                 route_add
                 route_delete                 
            );

my $debug = 1;

##Subroutines.
## @name    ::login_systemshell
## @summary Helps in logging into systemshell
## @arg $fc Required: Filer's connection object.
sub login_systemshell {
    my $fc=shift;
    logcomment("Logging to systemshell");
    my $Hostname=$fc->hostp->hostname();
    my $hosttype = $fc->hostp->hosttype();
    logcomment("hosttype=$hosttype");
    if($hosttype !~ /filer-ng-cfe/ && $hosttype !~ /filer-ng-vsim/) {
    say( $fc, "priv set advanced" ,"-swallow");
    say( $fc, "systemshell", -noprompt, -prompt, "login:", -timeout, 600,-swallow);
    say( $fc, "diag", -noprompt, -prompt, "Password:", -timeout, 600 );
    say( $fc, "netapp1!",-prompt,"$Hostname.*" );
    sleep(2);
    return 1;
    } else {
    logcomment("Am I coming in this else ?");
    say( $fc, "systemshell -node $Hostname", -noprompt, -prompt, "login:", -timeout, 600,-swallow);
    say( $fc, "diag", -noprompt, -prompt, "Password:", -timeout, 600 );
    say( $fc, "netapp1!",-prompt,"$Hostname.*" );
    sleep(2);
    return 1;
    }
}



#Subroutines.
## @name    ::get_routes_bsd
## @summary Helps in from bsd for all fibnums
## address_type needs to specifid.
## @Return value:- Reference to hash . fibnum and Destination is the key of hash
## @arg $fc Required: Filer's connection object.
## %args :- hash conataining address type
##address_type => Is a required parameter . Values can be either inet or inet6
##@Examples
##Getting ipv4 routes from bsd:- get_routes_bsd($fc,address_type=>"ipv4");
##Getting ipv6 routes from bsd:- my $bsd_routes_ipv6= get_routes_bsd($fc,address_type=>"ipv6");

sub get_routes_bsd {
    my ( $fc, %args ) = @_;
    login_systemshell($fc);
    my %result = ();
    my ($cmd_out,$fibnum);
    my $cmd="sudo netstat -rnG -f";
    if($args{address_type}=~ m/ipv4/)
    {
        $cmd.=" inet";
    }
    elsif($args{address_type}=~ m/ipv6/)
    {
      $cmd.=" inet6";
    }
    
    my $Hostname=$fc->hostp->hostname();
    my $hosttype = $fc->hostp->hosttype();
    ( undef, undef, $cmd_out ) = say( $fc, "$cmd",'-swallow',-prompt,"$Hostname%");
    if($args{address_type} =~ m/ipv6/)
    {
#my @cmd_out_ipv6=split( /Internet6:/, $cmd_out );
#     $cmd_out=$cmd_out_ipv6[1];
    }
    $cmd_out =~ s/$cmd//;
    $cmd_out =~ s/Internet.*\n//g;
    $cmd_out =~ s/Routing tables.*\n//g;
    $cmd_out =~ s/^\n$|^\r$//g;
    $cmd_out =~ s/Destination.*\n//g;
   my $i=0; 
    if($hosttype=~/filer-cfe/)
    {
    my @routing_fibnums = split( /Fib/, $cmd_out );
    logcomment("Array is  @routing_fibnums");
    foreach my $val (@routing_fibnums) {
        if($val =~ m/num:\s*(\d+)/i)
        {
		if($1 < 2)
		{
	            $fibnum=$1;
			$i=$1;    
		}
		else
		{
		    $fibnum= $i;
		    $i+=1;
		}
		
        }
        else
        {
            $fibnum=0; 
        }
        $val =~ s/num:.*\n//g;
        my @lines = split( /\n/, $val );
	logcomment("Array of each lines @lines");
        foreach my $val1 (@lines)
        {
            if ( $val1 =~ /^default|^\w+|^::/) {
                my @values = split( /\s+/, $val1 );
                $result{$fibnum}{ $values[0] }{gateway} = $values[1];
                $result{$fibnum}{ $values[0] }{flags} = $values[2];
                if($args{address_type} =~ m/ipv4/)
                {
                    $result{$fibnum}{ $values[0] }{refs} = $values[3];
                    $result{$fibnum}{ $values[0] }{use} = $values[4];
                    $result{$fibnum}{ $values[0] }{interface}= $values[5];
                }
                elsif($args{address_type} =~ m/ipv6/)
                {
                    $result{$fibnum}{ $values[0] }{interface}= $values[3];
                }
            }
        }
    }
    }
    elsif($hosttype =~/filer-ng-cfe/ || $hosttype =~ /filer-ng-vsim/) 
    {
	   $fibnum=0; 
#my $val = $cmd_out;
	   logcomment("Value is $cmd_out");
	my @lines = split( /\n/, $cmd_out );
        foreach my $val1 (@lines)
        {
		logcomment("Val1 $val1");
            if ( $val1 =~ /^default|^\w+|^::/) {

		logcomment("Val1 $val1");
                my @values = split( /\s+/, $val1 );
                $result{$fibnum}{ $values[0] }{gateway} = $values[1];
                $result{$fibnum}{ $values[0] }{flags} = $values[2];
                if($args{address_type} =~ m/ipv4/)
                {
                    $result{$fibnum}{ $values[0] }{refs} = $values[3];
                    $result{$fibnum}{ $values[0] }{use} = $values[4];
                    $result{$fibnum}{ $values[0] }{interface}= $values[5];
                }
                elsif($args{address_type} =~ m/ipv6/)
                {
                    $result{$fibnum}{ $values[0] }{interface}= $values[3];
                }
            }
        }
    }
    
    say($fc,"exit");
    logcomment("routing table");
    logcomment(Dumper(\%result));
    return ( \%result );
}

#Subroutines.
## @name    ::get_routes_sk
## @summary Helps in getting ipv4 or ipv6 routes from all vfiler contexts
## address_type and vfiler context needs to specifid.
## @Return value:- Reference to hash . Destination is the key of hash
## @arg $fc Required: Filer's connection object.
## %args :- hash conataining addres type details. Required parameter.
##  address_type => Is a required parameter . Values can be either inet or inet6
##  vfiler => if not specified vfiler0 is the default.
##@Examples
##Getting ipv4 routes from vfiler0 context in sk:-  get_routes_sk($fc,address_type=>"ipv4");
##Getting ipv6 routes from vfiler0 context in sk:-  get_routes_sk($fc,address_type=>"ipv6");
##Getting ipv4 routes from vfiler1(vfiler in non-default-ipspace) context in sk:- get_routes_sk($fc,address_type=>"ipv4",vfiler=>"vfiler1");
##Getting ipv6 routes from vfiler1(vfiler in non-default-ipspace) context in sk:- get_routes_sk($fc,vfiler=>"vfiler1",address_type=>"ipv6",vfiler=>"vfiler1");

sub get_routes_sk {
    my ( $fc, %args ) = @_;
    my %result = ();
    my $vfiler=$args{vfiler} || "";
    my $cmd_out;
    my $cmd="";
    if($vfiler)
    {
     $cmd="vfiler run $vfiler route -sn";
    }
    else
    {
     $cmd="route -sn";
    }
     my $Hostname=$fc->hostp->hostname();
     my $hosttype = $fc->hostp->hosttype();
     logcomment("hosttype=$hosttype");
     if($hosttype =~ /filer-ng-cfe/ || $hosttype =~ /filer-ng-vsim/) {
	      $cmd="run local route -sn";
     }
    if($args{address_type}=~ m/ipv4/)
    {
     $cmd.=" inet";
    }
    elsif($args{address_type}=~ m/ipv6/)
    {
     $cmd.=" inet6";
    }
    ( undef, undef, $cmd_out ) = say( $fc, "$cmd",'-swallow' );
    $cmd_out =~ s/$cmd//;
    $cmd_out =~ s/Internet.*\n//g;
    $cmd_out =~ s/Routing tables.*\n//g;
    $cmd_out =~ s/^\n$|^\r$//g;
    $cmd_out =~ s/Destination.*\n//g;
    my @lines = split( /\n/, $cmd_out );
    foreach my $val (@lines) {
        if ( $val =~ /^default|^\w+|^::/) {
            my @values = split( /\s+/, $val );
#logcomment("Value is $values[0]");
#	my $val = $values[0];

#	    logcomment("Value is $val");
            $result{ $values[0] }{gateway}   = $values[1];
            $result{ $values[0] }{flags}     = $values[2];
            $result{ $values[0] }{refs}      = $values[3];
            $result{ $values[0] }{use}       = $values[4];
            $result{ $values[0] }{interface} = $values[5];
        }
    }
    return ( \%result );
}
#Subroutines.
## @name    ::route_add
## @summary Helps in adding ipv4 or ipv6 host and net routes from all vfiler contexts
## @Return value:- Reference to hash . Destination is the key of hash
## @arg $fc Required: Filer's connection object.
## %args :- hash conataining addres type details. Required parameter.
##  address_type => if not specified inet is the default. Values can be either inet or inet6
##  vfiler => if not specified vfiler0 is the default.
## fibnum => for verifying routes in bsd. 0 is the default
## prefix=> Is the required parameter with net routes. to make verification easier.
## route_type=> host or net. host i the default.
## destination -> required. Destination to which route needs to be added.
## gateway => required, gateway for the above mentioned destination.
## metric => required, metric for the route
## @return values
## 1 for successfully adding roues 0 for failure
##@Examples
## Adding host and net ipv4 routes in vfiler0 context :-
## route_add($fc,destination=>"192.168.2.5",gateway=>"10.72.219.82",metric=>1,fibnum=>1);
## route_add($fc,destination=>"192.168.2.0",route_type=>net,prefix=>24,gateway=>"10.72.219.82",metric=>1,fibnum=>1); 
## Adding host and net routes in vfiler in non-default ipspace context :-
##route_add($fc,destination=>"192.168.4.7",gateway=>"192.168.1.1",metric=>1,fibnum=>2,vfiler=>"vfiler1");
##route_add($fc,destination=>"192.167.0.0",route_type=>net,prefix=>16,gateway=>"192.168.1.37",metric=>1,fibnum=>2,vfiler=>"vfiler1");


sub route_add
{
    my ($fc,%args) = @_;
    my $cmd="";
    my $vfiler =$args{vfiler} || "";
    my $address_type=$args{address_type} || "ipv4";
    my $route_type = $args{route_type};
    my $fibnum = $args{fibnum}||"0";
    my $prefix=$args{prefix} || "";
    my $netmask=$args {netmask} || "";
    my $destination;
    if($vfiler)
    {
        $cmd= "vfiler run $args{vfiler} route add";
    }
    else
    {
        $cmd= "route add";
    }
    if($address_type =~ m/ipv6/)
    {
        $cmd.=" inet6";
    }
    else
    {
        $cmd.=" inet";
    }
    if ($route_type =~ m/net/ && $args{'destination'} !~ m/default/i)
    {
	if(!$args{'prefix'})
	{
		$prefix=24;
		$args{'prefix'}=24;
	}
        $cmd.=" net";
        $destination="$args{'destination'}"."/"."$args{'prefix'}";
    }
    else
    {
        $cmd.=" host";
        $destination="$args{'destination'}";
    }
    if($prefix || $netmask)
    {
      $cmd.=" $args{'destination'}/$prefix $args{'gateway'} $args{'metric'}" if($prefix);
      $cmd.=" $args{'destination'}&netmask $args{'gateway'} $args{'metric'}" if($netmask);  
    }
    else
    {
        $cmd.=" $args{'destination'} $args{'gateway'} $args{'metric'}";
    }
    my (undef,undef,$res)= say($fc,"$cmd",'-swallow');
    
     if($res !~ m/add (net|host) ($args{'destination'}|$args{'destination'}\/$args{'prefix'}): gateway $args{'gateway'}/ || $res =~ m/entry already exists/) {
        logcomment("Route addition failed. See whether the route exists");
       return 0;
    }
     my $route_output= get_routes_sk($fc,address_type=>$address_type,vfiler=>"$args{vfiler}");
     my $dest_sk1 = $args{'destination'};
     $dest_sk1 =~ s/\.0//g if($route_type=~ m/net/);
     my $dest_sk2 = "$dest_sk1"."/"."$args{prefix}" if($route_type=~ m/net/);
     if($route_output->{$dest_sk1}{gateway} !~ m/$args{'gateway'}/ && $route_output->{$dest_sk2}{gateway} !~ m/$args{'gateway'}/ )
     {
        return 0;
     }
     my $routes_bsd = get_routes_bsd($fc,address_type=>$address_type);
     if($routes_bsd->{$fibnum}{$destination}{gateway} !~ m/$args{'gateway'}/)
     {
        return 0;
     }
     return 1;
}

#Subroutines.
## @name    ::route_add
## @summary Helps in adding ipv4 or ipv6 host and net routes from all vfiler contexts
## @Return value:- Reference to hash . Destination is the key of hash
## @arg $fc Required: Filer's connection object.
## %args :- hash conataining addres type details. Required parameter.
##  address_type => if not specified inet is the default. Values can be either inet or inet6
##  vfiler => if not specified vfiler0 is the default.
## fibnum => for verifying routes in bsd. 0 is the default
## prefix=> Is the required parameter with net routes. to make verification easier in bsd.
## route_type=> host or net. host i the default.
## destination -> required. Destination to which route needs to be added.
## @return values
## 1 for successfully deletion of routes 0 for failure
##@Examples
##Deleting host and net ipv4 routes in vfiler0 context :-
##route_delete($fc,destination=>"192.168.2.5",fibnum=>1);
##route_delete($fc,route_type=>"net",destination=>"192.168.2.0",prefix=>24,fibnum=>1);
##Deleting host and net routes in vfiler in non-default ipspace context :-
##route_delete($fc,route_type=>"host",destination=>"192.168.4.7",fibnum=>2,vfiler=>"vfiler1");
##route_delete($fc,route_type=>"net",destination=>"192.167.0.0",prefix=>16,fibnum=>2,vfiler=>"vfiler1");
sub route_delete
{
    my ($fc,%args) = @_;
    my $cmd="";
    my $vfiler =$args{vfiler} || "";
    my $address_type=$args{address_type} || "ipv4";
    my $route_type = $args{route_type};
    my $fibnum = $args{fibnum};
    my $destination;
    if($vfiler)
    {
        $cmd= "vfiler run $args{vfiler} route delete";
    }
    else
    {
        $cmd= "route delete";
    }
    if($address_type =~ m/ipv6/)
    {
        $cmd.=" inet6";
    }
    else
    {
        $cmd.=" inet";
    }
    if ($route_type =~ m/net/)
    {
        $cmd.=" net";
        $destination="$args{'destination'}"."/"."$args{'prefix'}";
    }
    else
    {
        $cmd.=" host";
        $destination="$args{'destination'}";
    }
    $cmd.=" $args{'destination'}";
    my (undef,undef,$res)= say($fc,"$cmd",'-swallow');
    if ($res !~ m/delete (net|host) ($args{'destination'}|$args{'destination'}\/$args{'prefix'})/ || $res =~ m/not in table/){
       return 0;
    }
     my $route_output= get_routes_sk($fc,address_type=>$address_type,vfiler=>"$args{vfiler}");
     my $dest_sk1 = $args{'destination'};
     $dest_sk1 =~ s/\.0//g if($route_type=~ m/net/);
     my $dest_sk2 = "$dest_sk1"."/"."$args{prefix}" if($route_type=~ m/net/);
     
     if(exists($route_output->{$dest_sk1}) || exists($route_output->{$dest_sk2}) )
     {
        return 0;
     }
     my $routes_bsd = get_routes_bsd($fc,address_type=>$address_type);
     if(exists($routes_bsd->{$fibnum}{$destination}))
     {
        return 0;
     }
     return 1;
}
1;
