# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Network/TCP_Checksum_Offload.pm#1 $
#Copyright (c) 2002 Network Appliance, Inc.
#All rights reserved.
#
## @summary Library of functions for basic filer commands
## @description

package Network::TCP_Checksum_Offload;

require Exporter;
use strict;
use Tharn;
use Data::Dumper;
use Carp;
use SayWrap qw(sayEOF);
use Carp qw(croak);
use SPARSE::Util::Logger qw(logStep logDbg logTrc logWarn);
use filer qw(filer_if_netmask_get);
use RegistryCmds qw(registryGet registrySet);
use Params::Validate qw(validate_pos SCALAR OBJECT);
use Utils::Units qw(:constants);
use Socket;
use filer;
use Network::Utils qw (get_ifstat_counter parse_ifstat_counter  run_sio);
use vars qw (@ISA @EXPORT_OK);
@ISA = qw(Exporter);
our @EXPORT_OK = qw(
    tcp_cksum_offload_functionality_check
);
sub tcp_cksum_offload_functionality_check {
    my $args = shift;
    my $filer_conn = $args->{filer_conn};
    my $intf = $args->{intf};
    my $client = $args->{client};
    my $client_conn = $args->{client_conn};
    my $sio_path = $args->{sio_path};
    my $sio_file_path = $args->{sio_file_path};
    my $tcp_cksum_offload_enable = $args-> {tcp_cksum_offload_enable};
    my $min_threshold = $args -> {minimum_threshold};
    my $max_tolerance = $args -> {max_tolerance};
    my $sio_traffic_secs = $args->{sio_traffic_secs};
    my $blksize = $args->{blksize}||1;
    #print (Dumper($args));
    #this command executes only from test or advanced mode
    say ($filer_conn, "priv set advanced");
    my $ifstat_v_output = sayEOF($filer_conn, "ifstat -v $intf");
    my $tcp6_count1 = parse_ifstat_counter (
        {
            ifstat_v_output=>$ifstat_v_output,
            counter => "Good TCP6 cksum"
        }
    );
    my $hw_tcp6_count1 = parse_ifstat_counter (
        {
            ifstat_v_output=>$ifstat_v_output,
            counter => "HW TCP6 cksums"
        }
    );
    logcomment ("Running sio traffic for $sio_traffic_secs seconds");
    say ($client_conn, "$sio_path 0 0 $blksize 0 10m $sio_traffic_secs 20 $sio_file_path");
    #sleep ($sio_traffic_secs + 30); 
    sleep 10;
    $ifstat_v_output = sayEOF($filer_conn, "ifstat -v $intf");
    #check that at least min_threshold packets have gone through
    my $total_frames = parse_ifstat_counter (
        {
            ifstat_v_output=>$ifstat_v_output,
            counter => "Total frames"
        }
    );
    if ($total_frames < $min_threshold ) {
	logcomment ("Total frames counter is $total_frames which is less than minimum threshold: $min_threshold required for testcase to pass");
        return 0;
    }
    my $tcp6_count2 = parse_ifstat_counter (
        {
            ifstat_v_output=>$ifstat_v_output,
            counter => "Good TCP6 cksum"
        }
    );
    my $hw_tcp6_count2 = parse_ifstat_counter (
        {
            ifstat_v_output=>$ifstat_v_output,
            counter => "HW TCP6 cksums"
        }
    );
    print "Final counters: tcp6_count1: $tcp6_count1 tcp6_count2: $tcp6_count2 hw_tcp6_count1: $hw_tcp6_count1 hw_tcp6_count2: $hw_tcp6_count2 \n\n";
    if ($tcp_cksum_offload_enable eq "on") {
        if ( $tcp6_count2 < ($tcp6_count1 + $min_threshold) ) {
            logcomment("TCP6 counter difference not sufficient to pass the test when checksum offloading is Enabled");
            return 0;
        }
        if ( $hw_tcp6_count2 < ($hw_tcp6_count1 + $min_threshold) ) {
            logcomment("HW TCP6 counter difference not sufficient to pass the test when checksum offloading is Enabled");
            return 0;
        }
        logcomment ("Counters are okey with TCP checksum offloading Enabled");
    } else {
            if ( $tcp6_count2 > ($tcp6_count1 + $max_tolerance) ) {
		logcomment("TCP6 counter difference not sufficient to pass the test when checksum offloading is disabled");
		return 0;
            }   
            if ( $hw_tcp6_count2 > ($hw_tcp6_count1 + $max_tolerance) ) {
		logcomment("HW TCP6 counter difference not sufficient to pass the test when checksum offloading is disabled");
		return 0;
            }
            logcomment ("Counters are okey with TCP checksum offloading Disabled");
    }
    my $bad_tcp6_count = get_ifstat_counter (
    {
        fc => $filer_conn,
        counter => "Bad TCP6 cksum",
        intf => $intf
    }
    );
    if ( $bad_tcp6_count > 0) {
        logcomment("Bad TCP6 Checksum found");
        return 0;
    }
    return 1;
}


