# this is basically -*- perl -*-
# change in eclipse
# $Id: //depot/prod/test/main/lib/VifMgr/VifConfig.pm#40 $
#
# Copyright (c) 2008 Network Appliance, Inc., All Rights Reserved
# Permission is granted to use this script on a Network Appliance Product.
#   Any other use, modification, or distribution is prohibited
#   without prior written consent from Network Appliance, Inc
#
## @summary     Virtual Interface Configuration class
## @author      dl-cluster-qa
## @pod         here

=head1 NAME

VifConfig.pm

=head1 AUTHOR

dl-cluster-qa

=head1 DESCRIPTION

C<VifConfig> will handle all Virtual Interface configuration needs. This includes,
but is not limited to:
    - creation/deletion/modification
    - enable/disable
    - Vserver config (to put VIFs in a fresh Vserver)
    - VIF state verification
    - VIF Migration

=head1 SYNOPSIS

    use VifMgr::VifConfig;
    $Vif_Cfg = new VifMgr::VifConfig;
    my $CURR_VIF = "create_vif_";
    my $deleted = $Vif_Cfg->delete_vif(vif => $CURR_VIF, server => $CURR_VS);
    my $num_created = $Vif_Cfg->create_vif(num_vifs => $num_vifs, vif => $CURR_VIF,
    address  => $Opts_Hash->{DATA_INTERFACES});
    my $Result = $Vif_Cfg->status_admin_vif(vif_name => $CURR_VIF, state => "up");

=head1 ARGUMENTS

=over 4

=item DATA_IF_NETMASK
(Optional)

Usage: [DATA_IF_NETMASK=255.254.0.0]

Description: The netmask to be used when configuring the vifs

=item VIF
(Optional)

Usage: [VIF="data_vif_"]

Description: Name of VIFs to create, followed by incrementer

=item GATEWAY
(Optional)

Usage: [GATEWAY=9.0.0.1]

Description: "Default gateway for network interface routes to use

=item ROUTE_METRIC
(Optional)

Usage: [ROUTE_METRIC=20]

Description: Metric for network interface routes to create

=item DELETE_IP_IF_CREATED
(Optional)

Usage: [DELETE_IP_IF_CREATED=1]

Description: Flag to delete a pre-existing LIF if it has the
same IP address (of a LIF we are trying to configure). If a previous
testcase does not cleanup properly, there is no reason for all downstream
tests need to fail with "Duplicate ip address".

=back

=head1 PUBLIC METHODS

=over 6

=item create_vif

Parametess: Attributes to create the VIF (defaults to object settings).
Returns: Number of VIFs created. Purpose: To create a VIF on the filer
with this specified configuration.

=item delete_vif

Parameters: Attributes used to delete VIFs. Returns: Number of VIFs deleted.
Purpose: To delete VIFs based on the name passed in.

=item status_admin_vif

Parameters: Parameters to status-admin up or down a vif. Returns: Number of
VIFs that were affected. Purpose: To have a seperate method that will handle
status-admining a VIF up or down. VIF name is a required parameter.

=item migrate_vifs

Parameters: server, vifs to migrate. Returns: Number of VIFs successfully migrated.
Purpose: Utilizing the Failover::Migrate class to move VIFs. Added here
so it can be encapsulated in the VifConfig class along with the rest of the Vif manager
needs.

=item get_vif_name_incr

Parameters: None. Returns: Scalar (integer). Purpose: An access method so calling
method is able to see the _vif_name_incr in case it needs to know what VIFs we are
currently working on.

=item get_vif_info

Parameters: Attribute to get, VIF name, server. Returns: Value for requested attribute.
Purpose: To have a method that can quickly pick out a VIF attribute (i.e.
the IP address).

=item build_ip_array

Parameters: <starting ip address>/<range> Where starting_ip_address is the
address to start building from and range is the number of addresses process.
Returns: An array of size <range> starting with <starting_ip_address> and
incrementing each address from there. Purpose: Utility to build a list of IP
addresses

=item get_address

Parameters: None
Returns: lif_address - the IP address for lif creation.
Purpose: To allow a more scalable way to create lifs, and keep track of
which IP addresses are used vs. available.

=item connectivity_checker

Parameters:
    lif_address   : IPv4 address of lif
    lif_netmask   : netmask of lif network
    ping_count    : number of packets to send
    ping_interval : delay between send packets
    ping_runid    : process id to use with subtest
    client_array  : client array to perform the connectivity check on
    garp_count    : number of packets to capture
    garp_interval : delay between packet sampling
    garp_runid    : process id to use with subtest

Purpose: Performs ping, and garp tests from the client to the lif address
which gives the ability to verify the datapath to the node.

=item ping_checker

Parameters:
    ping_address  : lif address to ping
    ping_count    : number of packets to send
    ping_interval : delay between send packets
    ping_runid    : process id to use with subtest
    client_array  : client array to perform the ping on

Purpose: Performs a ping subtest from the client to the lif address
which gives the ability to verify the datapath to the node.

=item garp_checker

Parameters:
    garp_address  : lif address
    garp_netmask  : lif netmask
    garp_count    : number of packets to capture
    garp_interval : delay between packet sampling
    garp_runid    : process id to use with subtest
    client_array  : client array to perform the garp application on

Purpose: Performs a sampling of ethernet packets, and determines the
network device's ability to resolve the network address.

=back


=head1 PRIVATE METHODS

=over 3

=item _set_home_ports

Parameters: output - Output from a "network interface show" command
role   - Vif Role of the home ports we are looking for.
ipspace - Vif ipspace of the home ports we are looking for.
Returns: Array of available home ports. Purpose: To compose an array of
all of the available home ports based on the specified role and pulled
from show command output. If that output doesnt exist, a general "network
interface show" will be issued.

=item _get_home_port

Parameters: None. Returns: One home-port stored in the $self->{'_home-port'} array.
Purpose: To extract a home-port from within the object. Assumes
_set_home_ports was already called to initialize $self->{'_home-port'}. In
addition to returning a port, this method shifts the first port off of the beginning
of the array and pushes it onto the end of the array to try to get uniform
distribution across all calls to get home ports.

=item _get_home_node

Parameters: None. Returns: One home-node stored in the $self->{'_home-node'} array.
Purpose: To extract a home-node from within the object. In addition to returning a
node, this method shifts the first node off of the beginning of the array and
pushes it onto the end of the array to try to get uniform distribution across all calls
to get home nodes.

=item _select_net_dev

Parameters:
    ip: IP address of Logical interface
  mask: Subnet mask of Logical interface
client: Network device used for checking port configuration.
Returns: Network port of device on the same subnet as the Logical interface
Purpose: Find the port that resides in our subnet for communication to the lif.

=back

=cut

package Network::VIFMgr::VifConfig;

# Standard Libs

use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use Net::Netmask;
use Socket qw(inet_ntoa);
use POSIX qw(strftime);
use Switch;
use Net::IP;

# Netapp Specific Libs

use NATE::Result;
use NATE::Log qw(log_global);
use NATE::Result;
use NATE::BaseException qw(:try);
use NATE::Process;

# NACL

use NACL::APISet;
use NACL::Exceptions::NoElementsFound;
use NACL::C::Node;
use NACL::STask::NetworkInterface;
use NACL::STask::NetworkRoutingGroupsRoute;
use NACL::CS::NetworkRoutingGroups;
use NACL::CS::Aggregate;
use NACL::CS::Vserver;
use NACL::STask::NetworkPort;
use NACL::C::Client;
use NACL::C::NetworkRoute;

use Options;

use Network::VIFMgr::Failover::Migrate;
use Network::VIFMgr::Failover::PortFail;

# Globals
my $Result;
my $Options = new Options();
my $Log     = log_global();
my @Nodes;
my $APISet;
#############################################################################
# Subroutine: new
# Parameters: None
# Returns: Reference to this object
# Purpose: To instantiate and configure the VifConfig object
#############################################################################

sub new {

   $Log->enter();
   my $class = shift(@_);
   
   #find @Nodes
   $Log->comment('VifConfig->new');

   # Get parameters to new()
   my %params = validate(
                     @_,
                     {
                       use_vif_name_incr => {
                                        type     => SCALAR,
                                        optional => 1,
                                        default  => 0,
                       },
                       find_ports => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => 0,
                       },
                       nacl_c_nodes => {
                                   type     => ARRAYREF,
                                   optional => 1,
                              },
                     }
   );

   my $self = {};

    # make sure we have Nodes
    # check node list passed in
    if ( defined($params{nacl_c_nodes}) ) {
        $Log->comment('VifConfig: node list passed in');
        foreach my $_n ( @{$params{nacl_c_nodes}} ) {
            if ( !Scalar::Util::blessed($_n) || !$_n->isa('NACL::C::Node')) {
                $Log->error("nacl_c_node element not a NACL::C::Node isa " . ref($_n));
                return(undef);
            }
        }
        @Nodes = @{$params{nacl_c_nodes}};
    }
    # create our own node list
    else {
        $Log->comment('VifConfig: node list being created');
        @Nodes = NACL::C::Node->find();
    }


    $Log->comment('VifConfig: making APISet using node ' . $Nodes[0]->hostrec->{'hostname'} );
    $APISet = NACL::APISet->new(
                                hostobj   => $Nodes[0]->hostrec(),
                                category  => "Node",
                                interface => "CLI",
                                set       => "CMode"
    );


    # check for ipspace ssupport
    $Log->comment('VifConfig: checking for IPSpace support using node ' . $Nodes[0]->hostrec->{'hostname'} );
    my $bool = $Nodes[0]->is_feature_supported( 'feature' => 'ipspaces_feature');
    if ( $bool ) { $self->{'ipspaces_feature'} = 1; }
    else         { $self->{'ipspaces_feature'} = 0; }
    $Log->comment("VifConfig: IPSpace supported:  $bool "
                 . sprintf("%s", $self->{'ipspaces_feature'} ? 'yes' : 'no')
    );


   # Get the nate_host file so that we can set some defaults for the
   # options

   my $host = $Nodes[0]->hostrec;

   # Get the command line options. These will become the defaults for
   # VIF configuration. However, the calling method is still capable of
   # passing in values to override these to each of the relative subroutines.

   $Options->parameters(
      definitions => [
         {
            name        => "DATA_IF_NETMASK",
            description =>
             "The netmask to be used when configuring the data vifs",
            default => $host->{DATA_MASK1},
         },
         {
            name        => "CLUSTER_IF_NETMASK",
            description =>
             "The netmask to be used when configuring the cluster vifs",
            default => $host->{CLUSTER_MASK1},
         },
         {
            name        => "VIF",
            description => "Name of VIFs to create, followed by incrementer",
            default     => "data_vif_",
         },
         {
            name        => "GATEWAY",
            description =>
             "Default gateway for network interface routes to use",
            default => $host->{DATA_GATEWAY1},
         },
         {
            name        => "ROUTE_METRIC",
            description => "Metric for network interface routes to create",
            default     => "20",
         },
         {
            name        => "GARP_TEST",
            description => "The relative path to the garpfilter binary.",
            default => "nblade/vifmgr/net/util/garpfilter/linux/garpfilter",
            file_tests => "rx",
         },
         {
            name        => "RTP_DATA_INTERFACES",
            description => "A list of IPs to use in the RTP labs",
         },
         {
            name        => "PTC_DATA_INTERFACES",
            description => "A list of IPs to use in the PTC labs",
         },
         {
            name        => "RTP_CLUSTER_INTERFACES",
            description => "A list of IPs to use in the RTP labs",
         },
         {
            name        => "PTC_CLUSTER_INTERFACES",
            description => "A list of IPs to use in the PTC labs",
         },
         {
            name        => "RTP_GATEWAY",
            description => "Gateway to use in the RTP labs",
         },
         {
            name        => "RTP_CLUSTER_GATEWAY",
            description => "Gateway to use in the RTP labs",
         },
         {
            name        => "PTC_GATEWAY",
            description => "GATEWAY to use in the PTC labs",
         },
         {
            name        => "PTC_CLUSTER_GATEWAY",
            description => "Gateway to use in the PTC labs",
         },
         {
            name        => "DEBUG_MODE",
            description => "Flag to run this test in debug mode which will "
             . "print out additional diagnostic commands for ping "
             . "operations",
            default => "0",
         },
         {
            name        => "DELETE_IP_IF_CREATED",
            description => "Flag to delete a pre-existing LIF if it has "
             . "same IP address configured.",
            default => "0",
         },
         {
            name        => 'LIF_COUNT',
            description => 'Number of LIFs to add',
            spec        => 'i',
            default     => '1',
         },

      ]
   );
   $Options->process();

   # Check to see if we need to process our site specific IPs

   my ( $rtp_ips, $ptc_ips ) =
     $Options->get( option => [qw(RTP_DATA_INTERFACES PTC_DATA_INTERFACES)] );
   if ( defined($rtp_ips) || defined($ptc_ips) ) {
      process_interfaces( site => $host->{SITE} );
   }

   $Log->debug( "Options = " . Dumper($Options) );
   my $opts = $Options->get_hash();


   # Configure the VifMgr object (copying over options locally)

   $self->{_data_ifs}        = $opts->{DATA_INTERFACES};
   $self->{_data_if_mask}    = $opts->{DATA_IF_NETMASK};
   $self->{_cluster_ifs}     = $opts->{CLUSTER_INTERFACES};
   $self->{_cluster_if_mask} = $opts->{CLUSTER_IF_NETMASK};
   $self->{_vif_name}        = $opts->{VIF};
   $self->{_gateway}         = $opts->{GATEWAY};
   $self->{_metric}          = $opts->{ROUTE_METRIC};
   $self->{'_home-nodes'}    = $opts->{NODES};
   $self->{_garp_test}       = $opts->{GARP_TEST};
   if ( !defined( $opts->{vserver} ) ) {
      $self->{_server} = "vs0";
   }
   else {
      $self->{_server} = $opts->{vserver};
   }
   $self->{_root_vol}   = "root_" . $self->{_server};
   $self->{_debug_mode} = $opts->{DEBUG_MODE};
   $self->{_delete_ip}  = $opts->{DELETE_IP_IF_CREATED};

   # The calling method can specify to not use the vif_name_incr. This is good
   # for scripts that are just creating one VIF at a time, and not doing them
   # in bulk.

   if ( $params{use_vif_name_incr} ) {
      $self->{_vif_name_incr} = 0;
   }
   else {
      $self->{_vif_name_incr} = "";
   }

   # The calling method can tell VifConfig if it wants it to dynamically find the
   # ports to create VIFs on. If the calling method is going to handle that itself,
   # then find_ports will be flagged to no (undef), and it will have to tell us where
   # place VIFs.

   if ( $params{find_ports} ) {
      my $home_ports = _set_home_ports( nodes => \@Nodes );
      $self->{'_home-ports'} = $home_ports;
   }
   else {
      $self->{'_home-ports'} = undef;
   }

   bless( $self, $class );

   $Log->debug(   "Initialized Network::VIFMgr::VifConfig object with the "
             . "following configuration:\n"
             . Dumper($self) );

   $Log->exit("Returning blessed reference: $self");
   return $self;

} ## end sub new

###############################################################################
# Subroutine: process_interfaces
# Parameters: none
# Returns: void
# Purpose: To properly set DATA_INTERFACES before the tests are run
###############################################################################

sub process_interfaces {

   $Log->enter();

   # Define the parameters

   my %params = validate(
                     @_,
                     {
                       site => {
                              type     => SCALAR,
                              optional => 1,
                              default  => "rtp",
                       },
                     },
   );

   $Log->comment("Currently running in $params{site}");

   # Set the site specific parameters

   my $ifs;
   my $gateway;
   my $clifs;
   my $cgateway;

   switch ( $params{site} ) {
      case /rtp/ {

         $ifs     = $Options->get( option => "RTP_DATA_INTERFACES" );
         $gateway = $Options->get( option => "RTP_GATEWAY" );

         my $clif_range =
           $Options->get( option => "RTP_CLUSTER_INTERFACES" );

         if ($clif_range) {
            $clifs = build_ip_array( "", ip => $clif_range );
            $cgateway = $Options->get( option => "RTP_CLUSTER_GATEWAY" );

            $Options->set( CLUSTER_INTERFACES => $clifs );
            $Options->set( CLUSTER_GATEWAY    => $cgateway );
         }

         # Verify that the user specified what we needed

         if ( !defined($ifs) ) {
            $Log->error(   "Did not specify RTP_DATA_INTERFACES to "
                      . "run in $params{site}. Exiting..." );
         }

      } ## end case rtp
      case "ptc" {

         $ifs     = $Options->get( option => "PTC_DATA_INTERFACES" );
         $gateway = $Options->get( option => "PTC_GATEWAY" );
         my $clif_range =
           $Options->get( option => "PTC_CLUSTER_INTERFACES" );

         if ($clif_range) {
            $clifs = build_ip_array( "", ip => $clif_range );
            $cgateway = $Options->get( option => "PTC_CLUSTER_GATEWAY" );

            $Options->set( CLUSTER_INTERFACES => $clifs );
            $Options->set( CLUSTER_GATEWAY    => $cgateway );
         }

         # Verify that the user specified what we needed

         if ( !defined($ifs) ) {
            $Log->error(   "Did not specify PTC_DATA_INTERFACES to "
                      . "run in $params{site}. Exiting..." );
         }

      } ## end case ptc
      else {

         $Log->error( "Tests are currently not configured to run at "
               . "site $params{site}. Currently set up for RTP and PTC." );

      } ## end else
   } ## end switch

   # Build the IP array

   $Options->set( DATA_INTERFACES => $ifs );

   # If we did not find a PTC or RTP GATEWAY, we can always use the default
   # set in the Options above (which is the nate_host file)

   if ( defined($gateway) ) {
      $Options->set( GATEWAY => $gateway );
   }

   # Call process

   $Options->process();

   # Log and return
   $Log->comment( "Successfully set DATA_INTERFACES = " . Dumper($ifs) );
   $Log->comment( "Successfully set CLUSTER_INTERFACES = " . Dumper($clifs) );
   $Log->exit( "Successfully set DATA_INTERFACES = " . Dumper($ifs) );

   return;

} ## end sub process_interfaces

#############################################################################
# Subroutine: create_vif
# Parameters: Attributes to create the VIF (defaults to object settings)
# Returns: Number of VIFs created
# Purpose: To create a VIF on the filer with this specified configuration
#############################################################################

sub create_vif {
   $Log->enter();
   my $self = shift(@_);
   my ( @LIFS, );

   # Initialize parameters with defaults set during object creation
   # Can be overriden if parameter specified by calling routine.
   # Note that the number of vifs created will be based on the num_vifs
   # and not the sive of the address array.

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vif => {
                              type     => SCALAR,
                              optional => 1,
                              default  => $self->{_vif_name}
                       },
                       vif_name_incr => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default => $self->{_vif_name_incr}
                       },
                       role => {
                              type     => SCALAR,
                              optional => 1,
                              default  => "data"
                       },
                              ipspace => {
                                        type     => SCALAR,
                                        optional => 1,
                                        default  => "Default"
                              },
                       status_admin => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => undef,
                       },
                       failover_policy => {
                                       type     => SCALAR,
                                       optional => 1,
                                       default  => "nextavail",
                       },
                       address => {
                                 type     => ARRAYREF | SCALAR,
                                 optional => 1,
                                 default  => $self->{_data_ifs}
                       },
                       if_netmask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => $self->{_data_if_mask}
                       },
                       home_node => {
                                  type     => SCALAR,
                                  optional => 1,
                                  default  => undef,
                       },
                       home_port => {
                                  type     => SCALAR,
                                  optional => 1,
                                  default  => undef,
                       },
                       use_failover_group => {
                                         type     => SCALAR,
                                         optional => 1,
                                         default  => undef,
                       },
                       auto_revert => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => undef,
                       },
                       num_vifs => {
                                 type     => SCALAR,
                                 optional => 1,
                                 default  => 1
                       },
                       destination => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => "0.0.0.0"
                       },
                       route_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0.0.0.0"
                       },
                       group_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0"
                       },
                       gateway => {
                                 type     => SCALAR,
                                 optional => 1,
                                 default  => $self->{_gateway}
                       },
                       metric => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_metric}
                       },
                       index => {
                               type     => SCALAR,
                               optional => 1,
                               default  => 0
                       },
                       rtn_name => {
                                 type     => SCALAR,
                                 optional => 1,
                                 default  => 0
                       },
                       create_route => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => 1,
                       },
                       data_protocol => {
                                     type     => SCALAR | UNDEF,
                                     optional => 1,
                                     default  => undef,
                       },
                       server_args => {
                                    type     => ARRAYREF | UNDEF,
                                    optional => 1,
                                    default  => undef,
                       }
                     }
   );

   #number of lifs to create

   $Options->set( LIF_COUNT => $params{num_vifs} );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # If the user is just creating one vif, then address will be a scalar and not
   # an array reference. We need to cast it here.

   if ( ref( $params{address} ) ne 'ARRAY' ) {
      my @tmp_arr = ( $params{address} );
      $params{address} = \@tmp_arr;
   }

   my $is_ipv4;
   my $is_ipv6;
   foreach ( @{ $params{address} } ) {
      $is_ipv4 = Net::IP::ip_is_ipv4($_);
      $is_ipv6 = Net::IP::ip_is_ipv6($_);
      if ( $is_ipv4 || $is_ipv6 ) {
         $Log->comment( "$_ is a valid address\n", $_ );
      }
      else {
         $Log->warn( "$_ is NOT a valid address. Returning...\n", $_ );
         return 0;
      }
   }

   my ( $_lif, $_lif_name );
   $_lif_name = $params{vif};
   my @ip_addresses = @{ $params{address} };
   my $end          = $params{index} + $params{num_vifs};
   my @vif_names;

   # params{address} should have enough addresses to create num_vifs.

   if ( scalar( @{ $params{address} } ) < $params{num_vifs} ) {
      $Log->warn(   "Not enough addresses neccessary to create VIFs. "
               . "Needed: $params{num_vifs}, Found: "
               . scalar( @{ $params{address} } )
               . ". Will only "
               . "create up to "
               . scalar( @{ $params{address} } ) );
      $end = scalar( @{ $params{address} } );
   }

   # Check to make sure we are creating something

   if ( $end < 1 ) {
      $Log->error("Not creating any LIFs. Exiting.");
      return 0;
   }

   for ( my $i = $params{index} ; $i < $end ; $i++ ) {

      # Only get the node and/or port if the calling method did not specify
      # it.

      my $node;
      my $port;

      if ( !defined( $params{home_node} ) ) {
         $node = _get_home_node($self);
      }
      else {
         $node = $params{home_node};
      }

      if ( !defined( $params{home_port} ) ) {
            my $home_ports;
            if ( $self->{'ipspaces_feature'} ) {
                $home_ports = _set_home_ports( nodes => \@Nodes, ipspace => $params{ipspace} );
            }
         else {
                $home_ports = _set_home_ports( nodes => \@Nodes, role => $params{role} );
            }
         $self->{'_home-ports'} = $home_ports;
         $port = _get_home_port( $self, node => $node );
      }
      else {
         $port = $params{home_port};
      }

      # If the user passed in 'use-failover-groups' then we will specify
      # that as well. Otherwise, let the value default.

      my %extra_params;

      if ( defined( $params{use_failover_group} ) ) {
         $extra_params{'use-failover-group'} = $params{use_failover_group};
      }

      # If the user passed in 'auto-revert' then we will specify
      # that as well. Otherwise, let the value default.

      if ( defined( $params{auto_revert} ) ) {
         $extra_params{'auto-revert'} = $params{auto_revert};
      }

      # If the user passed in 'status-admin' then we will specify
      # that as well. Otherwise, let the value default.

      if ( defined( $params{status_admin} ) ) {
         $extra_params{'status-admin'} = $params{status_admin};
      }

      # If the user passed in 'data_protocol' on a data port, then we will specify
      # that as well. Otherwise, let the value default.  If someone mistakenly tries
      # to set this on a non-data port we silently ignore it.

        if ( $self->{ipspace_feature} ) {
            if ( defined( $params{data_protocol} ) && ( $params{ipspace} eq "Default" ) )
                {$extra_params{'data-protocol'} = $params{data_protocol};}
        }
        else {
          if ( defined( $params{data_protocol} ) && ( $params{role} eq "data" ) )
              {$extra_params{'data-protocol'} = $params{data_protocol};}
        }
      # Verify that the IP does not currently exist

      if ( $self->{_delete_ip} ) {
         my @lif;
         try {
            @lif = NACL::CS::NetworkInterface->fetch(
                        'command_interface' => $Nodes[0],
                        filter => { address => $params{address}->[$i] }, );

            $Log->comment(   "IP $params{address}->[$i] exists. "
                        . "DELETE_IP_IF_CREATED is flagged to true. "
                        . "Deleting..." );
            NACL::STask::NetworkInterface->purge(
                                    'command_interface' => $Nodes[0],
                                    vserver => $lif[0]{server},
                                    lif     => $lif[0]{vif},
            );

           }
           catch NACL::Exceptions::NoElementsFound with {
            $Log->comment(
                  "IP $params{address}->[$i] does not " . "yet exist..." );
           }
           otherwise {
            $Log->warn("Unexpected error: $@. Returning...");
            return 0;
           };

      }

      # Create the interface

      # De-queue one address and add the lif to a certain port
      $params{vif} = $_lif_name . $i;
      if($is_ipv4){
         $_lif = NACL::STask::NetworkInterface->create(
                                    'command_interface' => $Nodes[0],
                                    'lif'       => $params{vif},
                                    'vserver'   => $params{server},
                                    'role'      => $params{role},
                                    'home-node' => $node,
                                    'home-port' => $port,
                                    'address' => shift @ip_addresses,
                                    'netmask' => $params{if_netmask},
                                    'nacltask_if_exists' => 'purge',
         );
      }
      elsif($is_ipv6){
         $_lif = NACL::STask::NetworkInterface->create(
                                    'command_interface' => $Nodes[0],
                                    'lif'       => $params{vif},
                                    'vserver'   => $params{server},
                                    'role'      => $params{role},
                                    'home-node' => $node,
                                    'home-port' => $port,
                                    'address' => shift @ip_addresses,
                                    'netmask' => $params{if_netmask},
                                    'netmask-length' => '64',
                                    'nacltask_if_exists' => 'purge',
         );
      }
      push( @LIFS, $_lif );

      # Save the VIF name for future reference
      push( @vif_names, $params{vif} );

      sleep 2;

      # Add the route if specified

      if ( $params{create_route} ) {
         create_route(
                    $self,
                    server      => $params{server},
                    vif         => $params{vif},
                    gateway     => $params{gateway},
                    destination => $params{destination},
                    route_mask  => $params{route_mask},
                    group_mask  => $params{group_mask},
                    metric      => $params{metric},
                    server_args => $params{server_args},
         );
      } ## end if ($params{create_route...

   } ## end for (my $i = $params{index...

   # Copy the $params{vif_name_incr} back over to the object so it can
   # be re-initialized for the next use (so we are not using the same values
   # over and over)

   $self->{_vif_name_incr} = $params{vif_name_incr};

   $Log->exit( "Successfully created " . $params{num_vifs} . " VIFs" );

   # There may be a case when we want this method to return the VIF name
   # as opposed to the number of VIFs created. That will be flagged here
   # Also note that we are returning the number of VIFs the user requested
   # to create as opposed to the number of VIFs that were actually created.
   # We don't want the upper level tests to fail just because we were not
   # allocated enough IP addresses. We are still warning with the number
   # of VIFs that were really created and that should be sufficient.

   if ( $params{rtn_name} ) {
      return \@vif_names;
   }
   else {
      return $params{num_vifs};
   }
} ## end sub create_vif

#############################################################################
# Subroutine: delete_vif
# Parameters: Attributes used to delete VIFs
# Returns: Number of VIFs deleted
# Purpose: To delete VIFs based on the name passed in
#############################################################################

sub delete_vif {

   $Log->enter();
   my $self = shift(@_);

   my ( $_cleanup, $_error, $_obj );

   # Initialize parameters. Only ones need to delete a VIF is the server
   # and the VIF name.

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vif => {
                              type     => SCALAR | ARRAYREF,
                              optional => 1,
                              default  => $self->{_vif_name}
                       },
                       delete_group => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => 1,
                       }
                     }
   );
   my $num_deletes = 0;

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Perform the delete. If vif is an array of VIF names, iterate
   # through them. Otherwise, just delete

   if ( ref( $params{vif} ) ne 'ARRAY' ) {
      $params{vif} = [ $params{vif} ];
   }

   #get vif objects
   my @LIFS;

   foreach my $vif ( @{ $params{vif} } ) {

      # First, check to see if we are deleting the routing-group. If we are,
      # we need that group name after we delete the VIF.

      my $group = undef;
      if ( $Nodes[0]->has_uichange( uichange => "routing-groups" )
          && ( $params{delete_group} ) )
      {
         my @show = NACL::CS::NetworkInterface->fetch(
            'command_interface' => $Nodes[0],
            filter              => {
                     'lif'     => $vif,
                     'vserver' => $params{server}
            },

         );
         $group = $show[0]->{'routing_group'};
      }

      NACL::STask::NetworkInterface->purge(
                                   'command_interface' => $Nodes[0],
                                   'lif'               => $vif,
                                   'vserver' => $params{server}
      );
      sleep 2;

      # Now, before we delete the routing-group, let's make sure this is the
      # last VIF in the group. Again, only do if we are running with 'routing-
      # groups' (which we can tell by check if $group is set).

      if ( defined($group) ) {

         try {
            NACL::CS::NetworkInterface->fetch(
                'command_interface' => $Nodes[0],
                filter              =>
                  { server => $params{server}, 'routing-group' => $group },
            );
            $Log->comment(   "Found LIFs using $group. Will not "
                        . "delete routing-group." );
           }
           catch NACL::Exceptions::NoElementsFound with {
            if (
                !delete_routing_group(
                                 $self,
                                 server => $params{server},
                                 group  => $group
                )
              )
            {
               $Log->warn("No group to delete for vif $vif");
               return 0;
            } ## end if (!delete_routing_group...
           }
           otherwise {
            $Log->warn(   "Unexpected error message while checking "
                     . "for LIFs in group $group: $@." );
           };

      } ## end if (defined($group))

      # NOTE: num_deletes will only show the number of deletes performed.
      # If the calling method appended a *, then that is one delete for
      # all of the VIFs actually deleted.

      $num_deletes++;

   } ## end foreach my $vif (@{$params{...

   $Log->exit("Sucessfully performed $num_deletes delete(s)");
   return $num_deletes;
} ## end sub delete_vif

#############################################################################
# Subroutine: status_admin_vif
# Parameters: Parameters to status-admin up or down a vif
# Returns: Number of VIFs that were affected
# Purpose: To have a seperate method that will handle status-admining a VIF
#          up or down. VIF name is a required parameter.
#############################################################################

sub status_admin_vif {

   $Log->enter();
   my $self = shift(@_);

   # Initialize parameters. Only ones need to admin a VIF is the server
   # and the VIF name.

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vif_name => { type => SCALAR | ARRAYREF },
                       state    => {
                               type     => SCALAR,
                               optional => 1,
                               "up"
                       },
                     }
   );
   my $num_changes = 0;

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Perform the modify. If vif_name is an array of VIF names, iterate
   # through them. Otherwise, just perform action

   if ( ref( $params{vif_name} ) ne 'ARRAY' ) {
      $params{vif_name} = [ $params{vif_name} ];
   }

   foreach my $vif ( @{ $params{vif_name} } ) {
      NACL::STask::NetworkInterface->modify(
                                    command_interface => $Nodes[0],
                                    vserver => $params{server},
                                    lif     => $vif,
                                    'status-admin' => $params{state}
      );
      $num_changes++;
   }

   $Log->exit("Sucessfully status-admined $num_changes VIFs");
   return $num_changes;

} ## end sub status_admin_vif

#############################################################################
# Subrotine: migrate_vifs
# Parameters: server, vifs to migrate
# Returns: Number of VIFs successfully migrated
# Purpose: Utilizing the Failover::Migrate class to move VIFs. Added here
#          so it can be encapsulated in the VifConfig class along with the
#          rest of the Vif manager needs.
##############################################################################

sub migrate_vifs {

   $Log->enter();
   my $self = shift(@_);

   # Initialize parameters, need VIFs and verserver to perform migrate

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vifs         => { type => SCALAR | ARRAYREF },
                       migrate_type => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => "RANDOM",
                       }
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Since the calling method can pass in an array of VIFs to migrate
   # (all with the same parameters), cast the scalar (in the case of a
   # single VIF) to an array if necessary

   if ( ref( $params{vifs} ) ne 'ARRAY' ) {
      $params{vifs} = [ $params{vifs} ];
   }

   # Use the VifMgr::Failover::Migrate class. This will handle gratutious
   # arps, and ping checks for us.

   my $vif_count = 0;
   foreach my $vif ( @{ $params{vifs} } ) {
      my $migr_vif = new Network::VIFMgr::Failover::Migrate(
                                vif          => $vif,
                                vserver      => $params{server},
                                migrate_type => $params{migrate_type},
      );
      $migr_vif->failover_event();
      $vif_count++;
   }

   $Log->exit("Sucessfully migrated $vif_count VIFs");
   return $vif_count;

} ## end sub migrate_vifs

#############################################################################
# Subrotine: failover_vifs
# Parameters: server, vifs to failover
# Returns: Number of VIFs successfully failed over
# Purpose: Utilizing the Failover::PortFail class to move VIFs. Added here
#          so it can be encapsulated in the VifConfig class along with the
#          rest of the Vif manager needs.
##############################################################################

sub failover_vifs {

   $Log->enter();
   my $self = shift(@_);

   # Initialize parameters, need VIFs and verserver to perform migrate
   # NOTE: Since we are defaulting the fail_type to DOWN_PORT (as opposed
   # to POWER_CYCLE) we will default the power_port to JUNK because it
   # is not needed in this case. If the calling method wants to POWER_CYCLE
   # it must supply a relevant port as well.

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vifs      => { type => SCALAR | ARRAYREF },
                       fail_type => {
                                  type     => SCALAR,
                                  optional => 1,
                                  default  => "DOWN_PORT",
                       },
                       power_port => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "JUNK",
                       },
                       check_vifs => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => 1,
                       },
                       role => {
                              type     => SCALAR,
                              optional => 1,
                              default  => "data",
                       },
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Since the calling method can pass in an array of VIFs to failover
   # (all with the same parameters), cast the scalar (in the case of a
   # single VIF) to an array if necessary

   if ( ref( $params{vifs} ) ne 'ARRAY' ) {
      $params{vifs} = [ $params{vifs} ];
   }

   # Use the VifMgr::Failover::PortFail class. This will handle gratutious
   # arps, and ping checks for us.

   my $vif_count = 0;
   foreach my $vif ( @{ $params{vifs} } ) {
      my $fail_vif = new Network::VIFMgr::Failover::PortFail(
                                  vif         => $vif,
                                  vserver     => $params{server},
                                  role        => $params{role},
                                  fail_device => $params{power_port},
                                  fail_type   => $params{fail_type},
                                  check_vifs  => $params{check_vifs},
      );
      $fail_vif->failover_event();
      $vif_count++;
   } ## end foreach my $vif (@{$params{...

   $Log->exit("Sucessfully failed over $vif_count VIFs");
   return $vif_count;

} ## end sub failover_vifs

#############################################################################
# Subroutine: create_route
# Parameters: Attributes for route creation for a particular VIF
# Returns: 1 on success, 0 on failure
# Purpose: Wrapper procedure to handle routing-groups versus legacy route
#          creation.
#############################################################################

sub create_route {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vif => {
                              type     => SCALAR,
                              optional => 1,
                              default  => $self->{_vif_name}
                       },
                       destination => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => "0.0.0.0"
                       },
                       route_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0.0.0.0"
                       },
                       group_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0"
                       },
                       gateway => {
                                 type     => SCALAR,
                                 optional => 1,
                                 default  => $self->{_gateway}
                       },
                       metric => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_metric}
                       },
                       server_args => {
                                    type     => ARRAYREF | UNDEF,
                                    optional => 1,
                                    default  => undef,
                       },
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Now, add the route. Must account for routing-groups in BR.0 and beyond.

   if ( $Nodes[0]->has_uichange( uichange => "routing-groups" ) ) {

      # First, get the routing-group name. Can be extracted from the 'network
      # interface show' output.

      my @show = NACL::C::NetworkInterface->show_routing_group(
         command_interface => $Nodes[0],
         filter            => {
            vserver => $params{server},
            lif     => $params{vif},

         }
      );
      my $group = $show[0]->{'routing_group'};
      $Log->comment(
              "############The routing group is $group.##################");
      if ( !$group ) {

         # Add only if it does not already exist
         NACL::STask::NetworkRoutingGroupsRoute->create(
            command_interface => $Nodes[0],
            vserver           => $params{server},
            'routing-group'   => $group,
            destination       => "$params{destination}/$params{group_mask}",
            gateway           => $params{gateway},
            nacltask_if_exists => 'reuse',    # default 'die'

         );
      }
   }
   else {
      NACL::STask::NetworkRoutingGroupsRoute->create(
         command_interface  => $Nodes[0],
         vserver            => $params{server},
         lif                => $params{vif},
         destination        => $params{destination},
         gateway            => $params{gateway},
         metric             => $params{metric},
         nacltask_if_exists => 'purge',                # default 'die'

      );

   } ## end else [ if

   # Log and return

   $Log->exit(   "Route to $params{gateway} for $params{vif} on "
            . "$params{server} successfully created" );
   return 1;

} ## end sub create_route

#############################################################################
# Subroutine: modify_route
# Parameters: Attributes for route modification for a particular VIF
# Returns: 1 on success, 0 on failure
# Purpose: Wrapper procedure to handle routing-groups versus legacy route
#          modification.
#############################################################################

sub modify_route {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vif => {
                              type     => SCALAR,
                              optional => 1,
                              default  => $self->{_vif_name}
                       },
                       destination => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => "0.0.0.0"
                       },
                       route_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0.0.0.0"
                       },
                       group_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0"
                       },
                       gateway => {
                                 type     => SCALAR,
                                 optional => 1,
                                 default  => undef,
                       },
                       metric => {
                                type     => SCALAR,
                                optional => 1,
                                default  => undef,
                       },
                       server_args => {
                                    type     => ARRAYREF | UNDEF,
                                    optional => 1,
                                    default  => undef,
                       },
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Since we only want to modify what the calling method passed in, we will build a
   # hash here, and pass that into the modify command.

   my %mod_params = ();

   if ( defined( $params{gateway} ) ) {
      $mod_params{gateway} = $params{gateway};
   }
   if ( defined( $params{metric} ) ) {
      $mod_params{metric} = $params{metric};
   }

   # Are we modifying anything?

   if ( !scalar( keys(%mod_params) ) ) {
      $Log->warn("No modification parameters were specified. Returning.");
      return 0;
   }

   # Now, modify the route. Must account for routing-groups in BR.0 and beyond.
   # Server.pm is not currently performing verifications for modifications
   # (see burt 298304). Therefore, we will issue the show here.

   if ( $Nodes[0]->has_uichange( uichange => "routing-groups" ) ) {

      # The modify command has not yet been implemeted for routing-groups

      $Log->comment(
             "Modification of routes has not yet been "
           . "implemented for routing-groups. Returning.",

      );
      $Log->exit();
      return 0;

      # First get the group-name

      my @show = NACL::C::NetworkInterface->show_routing_group(
         command_interface => $Nodes[0],
         filter            => {
            vserver => $params{server},
            lif     => $params{vif},

         }
      );
      my $group = $show[0]->{'routing-group'};

      # Modify only if it already exists

      try {
         NACL::CS::NetworkRoutingGroups->fetch(
                                     command_interface => $Nodes[0],
                                     filter            => {
                                        vserver => $params{server},
                                        'routing-group' => $group,
                                     }
         );
         NACL::C::NetworkRoute->modify(
                                 command_interface => $Nodes[0],
                                 vserver           => $params{server},
                                 'routing-group'   => $group,
                                 %mod_params,
         );
         NACL::CS::NetworkRoutingGroups->fetch(
              command_interface => $Nodes[0],
              filter            => {
                 server          => $params{server},
                 'routing-group' => $group,
                 gateway         => $params{gateway},
                 destination => "$params{destination}/$params{group_mask}",
              }
         );

        }
        catch NACL::Exceptions::NoElementsFound with {

         $Log->warn( "No route to modify (Server: $params{server}, "
             . "Group: $group, Destination: $params{destination}/"
             . "$params{group_mask}, Gateway: $params{gateway}). Returning."
         );
         return 0;

        };

   }    ##end if
   else {

      #in STask there is no NetworkRoute->modify

      NACL::STask::NetworkRoutingGroupsRoute->modify(
         vserver     => $params{server},
         lif         => $params{vif},
         destination => $params{destination},
         netmask     => $params{route_mask},
         %mod_params,

      );

      # Server.pm is not currently performing verifications for modifications
      # (see burt 298304). Therefore, we will issue the show here.

      NACL::CS::NetworkRoute->fetch(
                              command_interface => $Nodes[0],
                              filter            => {
                                 vserver     => $params{server},
                                 lif         => $params{vif},
                                 destination => $params{destination},
                                 netmask     => $params{route_mask},
                                 %mod_params
                              },
      );
   }

   # Log and return

   $Log->exit(   "Route to $params{gateway} for $params{vif} on "
            . "$params{server} successfully modified" );
   return 1;

} ## end sub modify_route

#############################################################################
# Subroutine: show_route
# Parameters: Attributes to show a route for a particular VIF
# Returns: reference to show output on success, 0 on failure
# Purpose: Wrapper procedure to handle routing-groups versus legacy route
#          show.
#############################################################################

sub show_route {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server},
                       },
                       vif => {
                              type     => SCALAR,
                              optional => 1,
                              default  => $self->{_vif_name},
                       }
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Now, show the route. Must account for routing-groups in BR.0 and beyond.

   my @output;
   if ( $Nodes[0]->has_uichange( uichange => "routing-groups" ) ) {

      # First get the group-name

      my @show = NACL::CS::NetworkRoutingGroups->fetch(
         command_interface => $Nodes[0],
         filter            => {
            vserver => $params{server},

            #    lif     => $params{vif}
         },
      );
      my $group = $show[0]->{'routing-group'};

      # Now, perform a show based on the routing-group

      @output = NACL::CS::NetworkRoutingGroups->fetch(
                                     command_interface => $Nodes[0],
                                     filter            => {
                                         server => $params{server},
                                         'routing-group' => $group
                                     },
      );

   }
   else {

      # Perform a show prior to routing-group implementation

      @output = NACL::CS::NetworkRoutingGroupsRoute->fetch(
         command_interface => $Nodes[0],
         filter            => {
            vserver => $params{server},

            # lif     => $params{vif},
         }
      );
   }

   $Log->exit("Successfully performed show. Returning output...");
   return \@output;
} ## end sub show_route

#############################################################################
# Subroutine: delete_route
# Parameters: Attributes for route deletion for a particular VIF
# Returns: 1 on success, 0 on failure
# Purpose: Wrapper procedure to handle routing-groups versus legacy route
#          deletion.
#############################################################################

sub delete_route {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       vif => {
                              type     => SCALAR,
                              optional => 1,
                              default  => $self->{_vif_name}
                       },
                       destination => {
                                    type     => SCALAR,
                                    optional => 1,
                                    default  => "0.0.0.0"
                       },
                       route_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0.0.0.0"
                       },
                       group_mask => {
                                   type     => SCALAR,
                                   optional => 1,
                                   default  => "0"
                       },
                       server_args => {
                                    type     => ARRAYREF,
                                    optional => 1,
                                    default  => undef,
                       },
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Now, delete the route. Must account for routing-groups in BR.0 and beyond.

   if ( $Nodes[0]->has_uichange( uichange => "routing-groups" ) ) {

      # First get the group-name

      my @show = NACL::CS::NetworkInterface->fetch(
                                     command_interface => $Nodes[0],
                                     filter            => {
                                        vserver => $params{server},
                                        lif     => $params{vif}
                                     },
      );
      my $group = $show[0]->{'routing-group'};

      # Delete only if it already exist

      try {
         NACL::STask::NetworkRoutingGroupsRoute->purge(
            command_interface => $Nodes[0],
            vserver           => $params{server},
            'routing-group'   => $group,
            destination       => "$params{destination}/$params{group_mask}",

         );
        }
        catch NACL::Exceptions::NoElementsFound with {
         $Log->warn(   "No route to delete (Server: $params{server}, "
                  . "Group: $group, Destination: $params{destination}/"
                  . "$params{group_mask}). Returning." );
         return 0;
        }

   }
   else {
      ##not shown in the lib of STask
      NACL::STask::NetworkRoute->purge(
         command_interface => $Nodes[0],
         vserver           => $params{server},
         lif               => $params{vif},
         destination       => $params{destination},
         netmask           => $params{route_mask},

      );
   }

   # Log and return

   $Log->exit(   "Route to $params{destination} for $params{vif} on "
            . "$params{server} successfully deleted" );
   return 1;

} ## end sub delete_route

#############################################################################
# Subroutine: delete_routing_group
# Parameters: Attributes for routing group deletion
# Returns: 1 on success, 0 on failure
# Purpose: To delete an entire routing group. There MUST be no VIFs configured
#          that utilize this routing-group. (That is why this is not part of
#          delete_route - so delete_route can be called before the VIF is
#          actually deleted),
###############################################################################

sub delete_routing_group {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       group => {
                               type     => SCALAR,
                               optional => 0,
                               default  => "*",
                       },
                       server_args => {
                                    type     => ARRAYREF,
                                    optional => 1,
                                    default  => undef,
                       },
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Now, delete the group. Must account for routing-groups in BR.0 and beyond.

   if ( $Nodes[0]->has_uichange( uichange => "routing-groups" ) ) {

      # Delete only if it already exist

      try {
         NACL::CS::NetworkRoutingGroups->fetch(
                                 command_interface => $Nodes[0],
                                 filter            => {
                                    vserver => $params{server},
                                    'routing-group' => $params{group}
                                 },
         );
         NACL::STask::NetworkRoutingGroupsRoute->purge(
                                   command_interface => $Nodes[0],
                                   server => $params{server},
                                   'routing-group' => $params{group},
                                   destination     => "*",
         );

         #   $Server->network_routing_groups_delete(
         #                             server => $params{server},
         #                             'routing-group' => $params{group},
         #                             @{ $params{server_args} },
         #   );
        }
        catch NACL::Exceptions::NoElementsFound with {
         $Log->comment(   "No group to delete (Server: $params{server}, "
                     . "Group: $params{group}). Returning." );
         return 0;

        }

   }
   else {
      $Log->comment( "Routing-groups are only supported on BR.0 and beyond. "
                  . "Returning." );
      return 0;
   }

   # Log and return

   $Log->exit(   "Successfully deleted group $params{group} on server "
            . "$params{server}." );
   return 1;

}    # end sub delete_routing_group

#############################################################################
# Subroutine: get_vif_name_incr
# Parameters: None
# Returns: Scalar (integer)
# Purpose: An access method so calling method is able to see the _vif_name_incr
#          in case it needs to know what VIFs we are currently working on.
#############################################################################

sub get_vif_name_incr {

   $Log->enter();
   my $self = shift(@_);

   $Log->exit( "Returning with: " . $self->{_vif_name_incr} );
   return $self->{_vif_name_incr};
}

#############################################################################
# Subroutine: get_vif_info
# Parameters: Attribute to get, VIF name, server
# Returns: Value for requested attribute
# Purpose: To have a method that can quickly pick out a VIF attribute (i.e.
#          the IP address)
#############################################################################

sub get_vif_info {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate(
                     @_,
                     {
                       vif    => { type => SCALAR },
                       server => {
                                type     => SCALAR,
                                optional => 1,
                                default  => $self->{_server}
                       },
                       attr => {
                              type     => SCALAR,
                              optional => 1,
                              default  => "address"
                       },
                       index => {
                               type     => SCALAR,
                               optional => 1,
                               default  => 0
                       },
                     }
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # If the caller wants a routing-group back and we are not running with that
   # change, we will simply exit with undef.

   if ( !$Nodes[0]->has_uichange( uichange => "routing-groups" )
       && ( $params{attr} eq 'routing-group' ) )
   {
      $Log->exit("Not running with routing-groups. Exiting");
      return undef;
   }

   # Issue the show on the VIF

   my @show = NACL::CS::NetworkInterface->fetch(
                                     command_interface => $Nodes[0],
                                     filter            => {
                                        vserver => $params{server},
                                        lif     => $params{vif}
                                     },
   );

   # We could either be returning a single value if only one VIF name was
   # passed in, or we could be returing an array of values if multiple
   # vif names were passed in (i.e. -vif vif_name*)

   if ( scalar(@show) > 1 ) {

      # Loop through each VIF that was shown and append its attribute value
      # to an array before returning

      my @return_this;
      foreach my $hash_ref (@show) {
         my %output = %{$hash_ref};
         push( @return_this, $output{ $params{attr} } );
      }
      $Log->exit(   "Found: $params{attr}: "
               . Dumper( \@return_this )
               . " for VIF: $params{vif}" );
      return \@return_this;
   }
   else {

      # Extract the VIF at the requested index (defaults to the first one)

      my %output = %{ $show[ $params{index} ] };
      my $found  = $output{ $params{attr} };

      $Log->exit("Found: $params{attr} $found for VIF: $params{vif}");
      return $found;
   } ## end else [ if (scalar(@show) > 1)

} ## end sub get_vif_info

############################################################################
# Subroutine: get_spare_port
# Parameters: Attributes to match port on
# Returns: The name of a port that is hosting no VIFs with specified attributes
# Purpose: There are cases when the calling tests need to find a port that
#          is not hosting any VIFs. The most common example is the node-mgmt VIF
#          testcases need to find a spare data port so that they can change
#          it to a node-mgmt port to run the test.
#############################################################################

sub get_spare_port {

   $Log->enter();
   my $self = shift(@_);

   # Initialize parameters

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 0,
                       },
                       role => {
                              type     => SCALAR,
                              optional => 1,
                              default  => "data",
                       },
                              ipspace => {
                                        type     => SCALAR,
                                        optional => 1,
                                        default  => "Default",
                              },
                       link => {
                              type     => SCALAR,
                              optional => 1,
                              default  => undef,
                       },
                     },
   );

   # Verify server exists

   if ( !_server_exists( server => $params{server} ) ) {
      $Log->warn("Server: $params{server} does not exist. Returning...");
      return 0;
   }

   # Do we care about the link state or not?

   my $link_key   = "";
   my $link_value = "";

   if ( defined( $params{link} ) ) {
      $link_key   = "link";
      $link_value = $params{link};
   }

   # Get a list of all current ports that we are looking for

    my @port_list;
    if  ( $self->{'ipspaces_feature'} ) {
        @port_list = NACL::CS::NetworkPort->fetch(
            command_interface => $Nodes[0],
            filter            => {
                node    => $params{server},
                ipspace => $params{ipspace},
            },
        );

    }
    else {
       @port_list = NACL::CS::NetworkPort->fetch(
          command_interface => $Nodes[0],
          filter            => {
             node => $params{server},
             role => $params{role},
          },
       );
    }
   # Now we can loop through that list, and as long as there are no VIFs
   # currently configured, we can return it
   my $port;
   my $port_output;
   my $portname;
   foreach $port (@port_list) {

      try {
         $port_output = NACL::CS::NetworkInterface->fetch(
                                  command_interface => $Nodes[0],
                                  filter            => {
                                     'curr-node' => $params{server},
                                     'curr-port' => $port->{port},
                                     'home-node' => $params{server},
                                     'home-port' => $port->{port}
                                  },
         );

        }
        catch NACL::Exceptions::NoElementsFound with {

         $Log->comment("Successfully found port $port->{port}.");
         $portname = $port->{port};
         return $portname;

        }
        otherwise {
         $Log->debug(   "Port $port->{port} is currently hosting "
                   . "VIFs, Continuing..." );
        };

   }
   if ( !defined $port_output ) {
      $Log->warn(   "Found no ports matching requested criteria: "
               . "Node: $params{server}, "
                    . ($self->{'ipspaces_feature'}
                        ? "IPSpace: $params{ipspace}"
                        : "Role: $params{role}"
                      )
        );
      return 0;
   }
   return $portname;

} ## end sub get_spare_port

############################################################################
# Subroutine: build_ip_array
# Parameters: <starting ip address>/<range>
#             Where starting_ip_address is the address to start building
#             from and range is the number of addresses process
# Returns: An array of size <range> starting with <starting_ip_address>
#          and incrementing each address from there.
# Purpose: Utility to build a list of IP addresses
#############################################################################

sub build_ip_array {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate(
                     @_,
                     {
                       ip => {
                             type     => SCALAR,
                             optional => 0,
                       },
                     }
   );

   # Verify $params{ip} is in the format of <starting_ip_address>/<range>

   if ( $params{ip} !~ /(.+)\/(.+)/ ) {

      $Result =
        NATE::Result::Config->new(
               "IP specified is not in the correct format. "
            . "Expected: <starting_ip_address>/<range>. Actual: $params{ip}"
        );
      $Result->log();
   }

   
   # Extract the address and verify

   my ( $ipaddress, $range ) = ( $1, $2 );
   $ipaddress =  new Net::IP($ipaddress);


   if(!Net::IP::ip_is_ipv4($ipaddress)&&!Net::IP::ip_is_ipv6($ipaddress)){
      $Result =
        NATE::Result::Config->new(
               "IP specified is not in the correct format. "
            . "Expected: ipv4 or ipv6 address. Actual: $ipaddress"
        );
      $Result->log();
   }
   # Array to hold IP list

   my @ip_list;

   # Save starting IP address
   push( @ip_list, $ipaddress->ip() );

   # Build the list (start at 1 since we already pushed first IP)

   for ( my $iteration = 1 ; $iteration < $range ; $iteration++ ) {

      # Increment to next valid address
      $ipaddress++;
      if(!Net::IP::ip_is_ipv4($ipaddress)&&!Net::IP::ip_is_ipv6($ipaddress)){

         $Result =
           NATE::Result::Config->new( "Could not build list of size $range, "
                               . "starting at $ipaddress." );
         $Result->log();

      }

      # Save newly created IP

      push( @ip_list,$ipaddress->ip() );
   } ## end for (my $iteration = 1 ...
   $Log->debug( "IP List: " . Dumper( \@ip_list ) );

   $Log->exit(   "Successfully created IP list of size $range, starting "
            . "at $ipaddress" );
   return \@ip_list;

} ## end sub build_ip_array

#############################################################################
# Subroutine: get_address
# Parameters: LIF role - If we are obtaining an address for a data or mgmt LIF,
#             we can extract the address from the data_ifs array. If this is a
#             cluster LIF we can look at the cluster IPs currently configured
#             on box and grab the next one.
# Returns: lif_address - the IP address for lif creation.
# Purpose: To allow a more scalable way to create lifs, and keep track of
#          which IP addresses are used vs. available.
#############################################################################

sub get_address {

   $Log->enter();
   my $self = shift;

   my %params = validate(
                     @_,
                     {
                       role => {
                              type     => SCALAR,
                              optional => 1,
                              default  => "data",
                       },
                     }
   );

   my $lif_address;

   # Cluster LIFs are a special case.

   if ( $params{role} =~ m/^cluster$/ ) {
      my $clif_queue = $self->{_cluster_ifs};
      $Log->debug( "Cluster IP list: " . Dumper($clif_queue) );

      # Pop one off for usage.

      $lif_address = shift(@$clif_queue);
      $Log->debug(
          "Lif Address: $lif_address, delta queue: " . Dumper($clif_queue) );
      $self->{_cluster_ifs} = $clif_queue;
      return $lif_address;
   }

   # Grab array of data addresses

   my $lif_queue = $self->{_data_ifs};
   $Log->debug( "IP list: " . Dumper($lif_queue) );

   # Pop one off for usage.

   $lif_address = shift(@$lif_queue);
   $Log->debug(
           "Lif Address: $lif_address, delta queue: " . Dumper($lif_queue) );
   $self->{_data_ifs} = $lif_queue;

   # Check to make sure the address is NOT null.

   if ( $lif_address eq "" ) {

      $Result =
        NATE::Result::Config->new(
                "There are no more addresses in the queue to "
              . "dispense.\nPlease check the number of addresses passed "
              . "into DATA_INTERFACES, vs. the number of interfaces needed "
              . "for the test." );
      $Result->log();

   }
   elsif(!Net::IP::ip_is_ipv4($lif_address)&&!Net::IP::ip_is_ipv6($lif_address)){
      $Result =
        NATE::Result::Config->new(
                      "The address parameter: $lif_address is not "
                   . "in the correct IP format." );
   }

   # Log and return

   $Log->exit("Returning with $lif_address");
   return $lif_address;
}

#############################################################################
# Subroutine: get_cluster_address
# Parameters: None
# Returns: lif_address - the IP address for lif creation.
# Purpose: To return the next available IP in the _cluster_ifs hash for a
#          particular node.
#
# WARNING: This is a hack only to be used in the RTP CQA Labs
#############################################################################

sub get_cluster_address {

   $Log->enter();
   my $self = shift;

   my %params = validate(
                     @_,
                     {
                       node => {
                              type     => SCALAR,
                              optional => 1,
                              default  => undef,
                       },
                     }
   );

   # First, check if the cluster ifs hash has been initialized. If not, we will
   # do that. This is not done in new() because most of the time, it will
   # not be used. Only when we are dealing with cluster IPs is when we will go
   # through this extra processing.

   #if (!defined($self->{_cluster_ifs})) {
   #    if (!init_cluster_ifs($self)) {
   #        $Log->error(  "Unable to initialze cluster IPs.");
   #    }
   #}

 # If the calling method did not specific a node in which we will be configuring
 # the LIF, we can assume it will be the first one

   if ( !defined( $params{node} ) ) {
      my @Nodes = NACL::C::Node->find();
      $params{node} = $Nodes[0];
   }

   # Grab the unused IP address

   my $lif_queue = $self->{_cluster_ifs}->{ $params{node} };
   $Log->debug( "IP list for $params{node}: " . Dumper($lif_queue) );

   # Pop one off for usage.

   my $lif_address = shift(@$lif_queue);
   $self->{_cluster_ifs}->{ $params{node} } = $lif_queue;

   # Make sure it is not null

   if ( $lif_address eq "" ) {
      $Log->error(   "There are no more cluster IP addresses to hand out "
                . "on node $params{node}. Exiting..." );
   }

   $Log->exit("Returning with $lif_address");
   return $lif_address;
}

#############################################################################
# Subroutine: init_cluster_ifs
# Parameters: None
# Returns: 1 on success, 0 on failure
# Purpose: In our RTP lab setup, each node can be configured with the following
#          cluster IPs: 192.168.[20-23].X. We have 192.168.16.1-192.168.23.254
#          reserved. Since 192.168.[16-19].X are unused, this subroutine will
#          initialize the cluster ifs hash with the respective 192.168.[16-19].X
#          IPs for each node in the cluster.
#
# WARNING: This is a hack only to be used in the RTP CQA Labs
#############################################################################

sub init_cluster_ifs {

   $Log->enter();
   my $self = shift;

   # Obtain a list of all nodes in the cluster

   my @Nodes = NACL::C::Node->find();

   # Iterate through each node, grabbing the last octet in the IP address,
   # and add its respective 192.168.[16-19].X to the hash

   my %ip_hash;
   foreach my $node (@Nodes) {

      # Get the IPs currently configured

      my @clus_ifs;
      try {
         @clus_ifs = NACL::CS::NetworkInterface->fetch(
                                     command_interface => $Nodes[0],
                                     filter            => {
                                             role => "cluster",
                                             'curr-node' => $node
                                     },
         );
        }
        otherwise {
         $Log->warn(   "Error while querying for cluster LIFs on "
                  . "node: $node. Returning..." );
         return 0;
        };

      # Get the last octet of the first IP (they should all be the same).

      my @octets = split( /\./, $clus_ifs[0]{address} );

      # Build an IP address array

      my @ips;
      for ( my $i = 16 ; $i < 20 ; $i++ ) {
         push( @ips, "192.168.$i.$octets[3]" );
      }

      # Build the hash

      $ip_hash{$node} = \@ips;
   }
   $self->{_cluster_ifs} = \%ip_hash;

   $Log->exit("Successfully built Cluster IP array");
   return 1;
}

###########################################################################
# Subroutine: connectivity_checker
#
# Parameters:
#    lif_address   : IPv4 address of lif
#    lif_netmask   : netmask of lif network
#    client_array  : client array to perform the connectivity check on
#
# Purpose: Performs ping, and garp tests from the client to the lif address
#          which gives the ability to verify the datapath to the node.
#############################################################################

sub connectivity_checker {
   $Log->enter();
   $Log->comment("In Connectivity checker");
   my $self = shift;
   my %params = validate(
                     @_,
                     {
                       lif_address  => { type => SCALAR, },
                       lif_netmask  => { type => SCALAR, },
                       client_array =>
                        { type => ARRAYREF, optional => 1 },
                       ping_only => { type => SCALAR, default => 1 },
                       ping_fail => {
                                  type     => SCALAR,
                                  optional => 1,
                                  default  => 0
                       },
                       network_ping_fail => {
                                        type     => SCALAR,
                                        optional => 1,
                                        default  => 0
                       },
                       node => {
                              type     => SCALAR | UNDEF,
                              optional => 1,
                              default  => undef
                       },
                     }
   );

   $self->ping_checker(
                   ping_address      => $params{lif_address},
                   client_array      => $params{client_array},
                   ping_fail         => $params{ping_fail},
                   network_ping_fail => $params{network_ping_fail},
                   node              => $params{node},
   );

   # If ping_only, skip garp_checker

   if ( !$params{ping_only} ) {
      $self->garp_checker(
                      garp_address => $params{lif_address},
                      garp_netmask => $params{lif_netmask},
                      client_array => $params{client_array},
      );
   }
   else {
      $Log->comment(
               "Skipping garp_checker because ping_only " . "was flagged" );
   }

   $Log->exit();
}

#############################################################################
# Subroutine: ping_checker
# Parameters:
#   ping_address  : lif address to ping
#   ping_count    : number of packets to send
#   ping_interval : delay between send packets
#   ping_runid    : process id to use with subtest
#   ping_client   : client to perform the ping on
#   ping_bin      : path to ping binary
# Purpose: Performs a ping subtest from the client to the lif address
#          which gives the ability to verify the datapath to the node.
#############################################################################

sub ping_checker {

   $Log->enter();
   $Log->comment("In Ping checker");

   my $self      = shift;
   my $timestamp = strftime( ".%m%d%y.%H%M%S", localtime );

   my %params = validate(
                     @_,
                     {
                       ping_address  => { type    => SCALAR, },
                       ping_count    => { default => 5 },
                       ping_interval => { default => 1 },
                       ping_runid    => { default => 'ping_' },
                       client_array  => {
                                    type     => ARRAYREF | UNDEF,
                                    optional => 1
                       },
                       ping_fail => {
                                  type     => SCALAR,
                                  optional => 1,
                                  default  => 0
                       },
                       network_ping_fail => {
                                        type     => SCALAR,
                                        optional => 1,
                                        default  => 0
                       },
                       node => {
                              type     => SCALAR | UNDEF,
                              optional => 1,
                              default  => undef
                       },
                     }
   );

   # So that the subtest does not config out, we will skip this portion entirely
   # if ping_fail is set

   if ( defined( $params{client_array} ) && !$params{ping_fail} ) {

      # If we are in debug mode, we want to dump this information before
      # the ping

      if ( $self->{_debug_mode} ) {
         $Log->comment(   "Dumping debugging... DEBUG_MODE flagged to "
                     . "$self->{_debug_mode}" );
         dump_port_info( ip => $params{ping_address} );
      }

      #pingcheck needs to be verified

      # Get the arguments for ping test.

      my $ping_args = " -c $params{ping_count} -i $params{ping_interval} $params{ping_address}";

      # Execute ping subtest on client.
      # subtest fails for most of the cases. Hence changeed subtest to rsh connectivity
      # and ping from client to the lif IP. Burt

      foreach my $client ( @{ $params{client_array} } ) {

         #my $host         = host($client);
         #my $Client_Conn1 = connect("#rsh");
         #my $ping         = sayEOF( $Client_Conn1, "ping $ping_args" );

         my $proc = NATE::Process->new(
                     codespec => "/bin/ping",
                     args     => ["$ping_args"],
                     runid => $params{ping_runid} . $client . $timestamp,
                     client => $client,
         );

         $proc->start();
         $proc->wait();

         #my $ping = Tharn::subtest(
         #    "ping $ping_args",
         #    "-search", [],
         #    -runid  => $params{ping_runid}.$client.$timestamp,
         #    -onexit => sub {},
         #    -client => $client,
         #);

         # Evaluate subtest results.
         # exit_status() will return 0 on success, 1 on failure

         # Ping check threshold increased to accomodate 1 packet loss during LIF create. BURT 541454
         $Log->comment(
                   "#####dump proc exit status:" . $proc->{exit_status} );
         if ( ( $proc->{exit_status} ) ) {

            # Dump debugging and error

            dump_port_info( ip => $params{ping_address} );
            $Log->error(   "Ping check of $params{ping_address} "
                      . "failed when expected to pass." );
         }
         else {
            $Log->comment("PING Succeeded");

         }
         $proc->destroy();
      }
   }

   # Issue a "network ping" from the cluster as well (if we are running in
   # Boilermaker or beyond).

   my $version_manager = $Nodes[0]->get_version_manager();
   my $release         =
     $version_manager->get_version_attribute( attribute => "release" );
   my $codename =
     $version_manager->map_release_to_codename( release => $release );

   if ( $codename !~ /tricky/i ) {

      # If we are in debug mode, we want to dump this information before
      # the ping

      if ( $self->{_debug_mode} ) {
         $Log->comment(   "Dumping debugging... DEBUG_MODE flagged to "
                     . "$self->{_debug_mode}" );
         dump_port_info( ip => $params{ping_address} );
      }

      if ( !defined( $params{node} ) ) {
         my @nodes = NACL::C::Node->find();
         $params{node} = $nodes[0];
      }

      $Log->comment("#######perform a network ping");
      my $node_ping = $APISet->network_ping(
                           node => $params{node}->hostrec->{"hostname"},
                           destination => $params{ping_address},
                           count       => $params{ping_count},
                           wait        => $params{ping_interval},
      );
      $Log->comment( "####network ping : " . $node_ping->{processed_output} );
      if ( ( $node_ping->{processed_output} !~ /alive/ )
          && !$params{network_ping_fail} )
      {

         # Dump debugging and error

         dump_port_info( ip => $params{ping_address} );
         $Log->error(
             "Ping failed from node $params{node}->hostrec->{\"hostname\"} "
               . "when expected to pass" );

      }
      elsif ( ( $node_ping->{processed_output} !~ /alive/ )
            && $params{network_ping_fail} )
      {
         $Log->comment(
              "Ping failed from node $params{node} " . "when expected to" );
      }
      elsif ( ( $node_ping->{processed_output} =~ /alive/ )
            && $params{network_ping_fail} )
      {

         # Dump debugging and error

         dump_port_info( ip => $params{ping_address} );
         $Log->error(   "Ping succeeded from node $params{node} "
                   . "when expected to fail" );
      }

   }
   else {
      $Log->comment(   "Skipping 'network ping' as it is not "
                  . "implemented in Tricky." );
   }

   $Log->exit();
}

#############################################################################
# Subroutine: garp_checker
# Parameters:
#   garp_address  : lif address
#   garp_netmask     : lif netmask
#   garp_count    : number of packets to capture
#   garp_interval : delay between packet sampling
#   garp_runid    : process id to use with subtest
#   garp_client   : client to perform the garp application on
#   garp_bin      : path to garp binary
# Purpose: Performs a sampling of ethernet packets, and determines the
#          network device's ability to resolve the network address.
#############################################################################

sub garp_checker {

   # Note: The name of the underlying operating system is stored in the
   # variable $^O.

   $Log->enter();
   my $self = shift;
   my $timestamp = strftime( ".%m%d%y.%H%M%S", localtime );
   my %params = validate(
                     @_,
                     {
                       garp_address  => { type    => SCALAR, },
                       garp_netmask  => { type    => SCALAR, },
                       client_array  => { type    => ARRAYREF, },
                       garp_count    => { default => 3 },
                       garp_interval => { default => 2 },
                       garp_runid    => { default => 'garp_' },
                     }
   );

   # Find the client's network device on the same subnet.
   my @clients = NACL::C::Client->find();
   
   foreach my $client ( @clients ) {

      ##this needs to be changed for real client
      my $host = $client->{hostrec};
      if ( $host->{OS} eq 'Linux' ) {

         # Find the garpfilter test.

         $params{garp_test} = NATE::Inc::find_file( $self->{_garp_test} );
         $params{garp_net_dev} = &_select_net_dev(
                                 dev_ip      => $params{garp_address},
                                 dev_netmask => $params{garp_netmask},
                                 client      => $client->{'client'},
         );

         # Get the garpfilter test arguments.

         my $garp_delay =
           ( $params{garp_count} * $params{garp_interval} * 10 );
         my $garp_args =
             " $params{garp_test} $params{garp_address} $params{garp_count}"
           . " $params{garp_net_dev} $garp_delay 1";

         # Execute garpfilter subtest.

         my $garp = NATE::Process->new(
                     codespec => "sudo ",
                     args     => [$garp_args],
                     runid => $params{garp_runid} . $client->{'client'} . $timestamp,
                     client => $client->{'client'},
         );

         $garp->start();
         $garp->wait();

         my @garp_results = $garp->results();
         $garp->destroy();

      }
      else {
         $Log->warn(   "The client OS: $host->{OS} is not supported "
                  . "with garpfilter. Use Linux, or Solaris clients." );
      }
   }

   $Log->exit();
}

#######################################################################
# Function:    dump_port_info
# Description: On a connectivity failure, we may need the following information.
# Parameters:  ip: IP address to issue a FreeBSD ping to
# Returns:     Void
#
# Note:        This was originally added for connectivity_check failures,
#              but can be used for anything.
########################################################################

sub dump_port_info {

   # Define the parameters

   my %params = validate(
                     @_,
                     {
                       ip => {
                             type     => SCALAR,
                             optional => 1,
                             default  => undef
                       },
                       node => {
                              type     => SCALAR,
                              optional => 1,
                              default  => undef
                       },
                       sudo => {
                              type     => SCALAR,
                              optional => 1,
                              default  => "/usr/bin/sudo"
                       },
                     },
   );

   # If the calling method specifies a node, we will only run the commands
   # on that node. Otherwise, we will run them on all nodes.

   my @nodes;

   if ( !defined( $params{node} ) ) {
      my @cluster_nodes = NACL::C::Node->find();
      foreach my $node (@cluster_nodes) {
         push( @nodes, $node );
      }
   }
   else {
      my $node = NACL::C::Node->find(       command_interface => $Nodes[0],
                              filter => { node => $params{node}, } );
      push( @nodes, $node );
   }

   # Certain commands need sudo access

   my $sudo = $params{sudo};

   foreach my $node_obj (@nodes) {

      my $sys_shell = NACL::APISet->new(
                                 hostobj    => $node_obj->hostrec(),
                                 category   => 'Node',
                                 interface  => 'CLI',
                                 set        => 'Systemshell',
                                 shell_type => "CMode",
      );

      # Netstat commands
      my $node = $node_obj->name();

      $sys_shell->execute_raw_command( command => "netstat -s -p ip" );
      $sys_shell->execute_raw_command( command => "netstat -s -p icmp" );
      $sys_shell->execute_raw_command(
               command => "run -node $node -command \"netstat -s -p ip\"" );
      $sys_shell->execute_raw_command(
            command => "run -node $node -command \"netstat -s -p icmp\" " );

      if ( defined( $params{ip} ) ) {

         # Start tcpdump. Give it a little more time than the default

         my $timestamp = strftime( "%m%d%y.%H%M%S", localtime );
         my $device = get_pseudo_device( ip => $params{ip} );

         $sys_shell->execute_raw_command(
                         command => "$sudo tcpdump -i $device -w "
                           . "/mroot/tcpdump.$timestamp.pseudo_dev&", );
         $sys_shell->execute_raw_command(
                              command => "$sudo tcpdump -i lo0 -w "
                                . "/mroot/tcpdump.$timestamp.lo0&", );
         $sys_shell->execute_raw_command(
                              command => "$sudo tcpdump -i lofb -w "
                               . "/mroot/tcpdump.$timestamp.lofb&", );
         $sys_shell->execute_raw_command(
                              command => "$sudo tcpdump -i ips0 -w "
                               . "/mroot/tcpdump.$timestamp.ips0&", );

         # Start pktt

         $sys_shell->execute_raw_command(
                command => "run -node $node -command \"pktt start all\"" );

      }

      # If a ping IP is specified, we will issue that from FreeBSD as well

      if ( defined( $params{ip} ) ) {
         $sys_shell->execute_raw_command(
                                command => "ping -c 10 $params{ip}" );
      }

      # Netstat commands after ping

      $sys_shell->execute_raw_command( command => "netstat -s -p ip" );
      $sys_shell->execute_raw_command( command => "netstat -s -p icmp" );

      $sys_shell->execute_raw_command(
               command => "run -node $node -command \"netstat -s -p ip\"" );
      $sys_shell->execute_raw_command(
             command => "run -node $node -command \"netstat -s -p icmp\"" );

      if ( defined( $params{ip} ) ) {

         # Kill tcpdump

         $sys_shell->execute_raw_command( command => "$sudo killall tcpdump",
         );

         # Stop pktt

         $sys_shell->execute_raw_command(
                 command => "run -node $node -command \"pktt dump all\"" );

         $sys_shell->execute_raw_command(
                 command => "run -node $node -command \"pktt stop all\"" );
      }

      # Get interface information

      $sys_shell->execute_raw_command(
                          command => "$sudo pcpconfig 0 stat_vif all" );
      $sys_shell->execute_raw_command( command => "ifconfig" );
      $sys_shell->execute_raw_command(
                  command => "run -node $node -command \"ifconfig -a\"" );
      $sys_shell->execute_raw_command(
                    command => "run -node $node -command \"ifstat -a\"" );

      # Get routing information

      $sys_shell->execute_raw_command(
                   command => "run -node $node -command \"route -gsn\"" );
      $sys_shell->execute_raw_command( command => "$sudo netstat -rn" );
      $sys_shell->execute_raw_command( command => "$sudo netstat -in" );

   } ## end foreach my $node_obj (@nodes)

   return;

} ## end sub _dump_port_info
#############################################################################
# Subroutine: get_pseudo_device
# Parameters: IP address
# Returns: Name of pseudo device that the IP address resides on
# Purpose: Utility to find the psuedo device (used for running FreeBSD
#          commands)
#############################################################################

sub get_pseudo_device {

   $Log->enter();

   # Define the parameters

   my %params = validate(
                     @_,
                     {
                       ip => {
                             type     => SCALAR,
                             optional => 0,
                       },
                     },
   );

   # The device is comprised of the server name and the routing-group

   my @lif = NACL::CS::NetworkInterface->fetch(
                                 command_interface => $Nodes[0],
                                 filter => { address => $params{ip} } );

   # Extract what we need and create the device name

   my $p_device = $lif[0]{server} . "_" . $lif[0]{'routing-group'};

   $Log->exit("Returning pseudo device $p_device for IP $params{ip}");
   return $p_device;

} ## end sub get_pseudo_device

#############################################################################
# Subroutine: _set_home_ports
# Parameters: output  - Output from a "network interface show" command
#             role    - Vif Role of the home ports we are looking for
#             ipspace - Vif ipspace of the home ports we are looking for
# Returns: Array of available home ports
# Purpose: To compose an array of all of the available home ports based
#          on the specified role and pulled from show command output. If that
#          output doesnt exist, a general "network interface show" will be
#          issued.
#############################################################################

sub _set_home_ports {

   $Log->enter();

   my %params = validate(
      @_,
      {
         nodes => {
                type => ARRAYREF
           },

         role => {
            type     => SCALAR,
            optional => 1,
            # default  => "data"
         },
           ipspace => {
               type     => SCALAR,
               optional => 1,
               # default  => "Default"
           },
         link => {
                type     => SCALAR,
                optional => 1,
                default  => "up"
         },
      }
   );

   # Check if there is output. If not, issue a show command
   my ( @Ports, @node_ports, %home_ports );

   foreach my $node ( @{ $params{nodes} } ) {
      $home_ports{ $node->{hostrec}->{'hostname'} } = [];
   }

   #fetch all port objects
    # role based
    if ( $params{role} ) {
       @Ports = NACL::CS::NetworkPort->fetch(
            command_interface => $params{nodes}[0],
          filter => { role => $params{role} }
        );
    }
    # ipspace based
    elsif ($params{ipspace}) {
        @Ports = NACL::CS::NetworkPort->fetch(
            command_interface => $params{nodes}[0],
            filter => { ipspace => $params{ipspace} }
        );
    }
    # all ports
    else {
        @Ports = NACL::CS::NetworkPort->fetch(
            command_interface => $params{nodes}[0],
        );
    }
   foreach my $port (@Ports) {
       $Log->comment("the port->node : $port->{node}");
       $Log->comment("the $home_ports{$port->{node}}");

      @node_ports = @{ $home_ports{ $port->{node} } };
      push( @node_ports, $port->{port} )
      unless ( grep $_ eq $port->{port}, @node_ports );
      $home_ports{ $port->{node} } = \@node_ports;
   }

   # Extract the node_ports array for the particular node we are working
   # with and push the new part on there if it does not already exist

   foreach my $_port (@Ports) {
      # If this node isn't part of $params{nodes}, we can continue. This is
      # the cases where we aren't running the test on all nodes in the cluster.
      if ( grep $_ eq $_port->{node}, @{ $params{nodes} } ) {

         #      $Log->comment( $port_show{'port'} );
         $Log->debug("Found node: $_port->{node}.");
      }
      else {
         next;
      }
   }
   $Log->exit();
   return \%home_ports;

} ## end sub _set_home_ports

###########################################################################
# Subroutine: _get_home_port
# Parameters: None
# Returns: One home-port stored in the $self->{'_home-port'} array
# Purpose: To extract a home-port from within the object. Assumes
#          _set_home_ports was already called to initialize
#          $self->{'_home-port'}. In addition to returning a port, this method
#          shifts the first port off of the beginning of the array and
#          pushes it onto the end of the array to try to get uniform
#          distribution across all calls to get home ports.
############################################################################

sub _get_home_port {

   $Log->enter();
   my $self = shift(@_);

   my %params = validate( @_, { node => { type => SCALAR }, } );

   # Get the array of ports for the current node

   my %port_hash  = %{ $self->{'_home-ports'} };
   my @port_array = @{ $port_hash{ $params{node} } };

   # Verify that there are ports available for this node

   if ( scalar(@port_array) < 1 ) {

      NACL::CS::NetworkPort->fetch( command_interface => $Nodes[0], );

      $Log->error(   "There are no ports available for node "
                . "$params{node}. Exiting." );
   }

   # Shift the new port off the beginning and push it back on the end of
   # the array to rotate

   my $home_port = shift(@port_array);
   push( @port_array, $home_port );

   # Save the values for the next run

   $port_hash{ $params{node} } = \@port_array;
   $self->{'_home-ports'} = \%port_hash;

   $Log->exit("Found port: $home_port");
   return $home_port;

} ## end sub _get_home_port

###########################################################################
# Subroutine: _get_home_node
# Parameters: None
# Returns: One home-node stored in the $self->{'_home-node'} array
# Purpose: To extract a home-node from within the object. In addition to
#          returning a node, this method shifts the first node off of the
#          beginning of the array and pushes it onto the end of the array
#          to try to get uniform distribution across all calls to get home
#          nodes.
############################################################################

sub _get_home_node {

   $Log->enter();

   my $self = shift(@_);

   # Rotate the array while extracting a home-node

   my @node_array = @{ $self->{'_home-nodes'} };
   my $home_node  = shift(@node_array);
   push( @node_array, $home_node );
   $self->{'_home-nodes'} = \@node_array;

   $Log->exit("Found node: $home_node");
   return $home_node;

} ## end sub _get_home_node

##############################################################################
# Subroutine: _server_exists
# Parameters: Server name
# Returns: 1 if server exists, otherwise 0
# Purpose: We cannot simply do a $Server->vserver_exists() because if the
#          calling methods are configuring cluster or node-mgmt VIFs, then they
#          will be on a node, and not on a vserver. This method checks both
#          cases. Further, we aren't always aware of the VIF role (ie. on a
#          delete, the calling method doesn't have to supply a role). So we
#          just check if the "server" in general exists.
###############################################################################

sub _server_exists {

   $Log->enter();

   my %params = validate(
                     @_,
                     {
                       server => {
                                type     => SCALAR,
                                optional => 0,
                       }
                     }
   );

   # First we can do a quick grep through the servers array to see
   # if $params{server} is an actual node the tests are running on
   # in the cluster (this would be the case for node-mgmt and cluster VIFs).

   if ( grep( /^$params{server}$/, @Nodes ) ) {
      $Log->comment("Found server: $params{server}.");
      $Log->exit();
      return 1;
   }

   # Since its not an actual node, we can now check to see if it is
   # a configured vserver
   try {
      my @vserver_states = NACL::CS::Vserver->fetch(
                              command_interface => $Nodes[0],
                              filter => { vserver => $params{server} }, );
      $Log->comment("Found vserver: $params{server}");
      $Log->exit();
      return 1;
     }
     catch NACL::Exceptions::NoElementsFound with {
      $Log->comment( "vserver: $params{server} does not " . "yet exist..." );
      $Log->exit();
      return 1;
     };

   #  otherwise {

   # If we are here, $params{server} failed both the node and vserver
   # check. Exit on failure.
   #   $Log->comment(   "$params{server} is neither a node being tested "
   #               . "in or a vserver. Exiting on failure." );
   #   $Log->exit();
   #   return 0;
   #  };
} ## end sub _server_exists

##############################################################################
# Subroutine: _select_net_dev
# Parameters:
#   ip: IP address of Logical interface
# mask: Subnet mask of Logical interface
# client: Network device used for checking port configuration.
# Returns: Network port of device on the same subnet as the Logical interface
# Purpose: Find the port that resides in our subnet for communication to the
#          lif.
###############################################################################

sub _select_net_dev {

   $Log->enter();
   my $timestamp = strftime( ".%m%d%y.%H%M%S", localtime );

   my %params = validate(
                     @_,
                     {
                       dev_ip      => { type    => SCALAR },
                       dev_netmask => { type    => SCALAR },
                       client      => { type    => SCALAR },
                       route_bin   => { default => '/sbin/route' },
                       route_runid => { default => 'route_' },
                     }
   );

   my $block     = new Net::Netmask( $params{dev_ip}, $params{dev_netmask} );
   my $cidr_bits = $block->bits();
   my $cidr_info = $block->desc();
   my $network_addr = $block->base();
   my $network_mask = $block->mask();


   $Log->comment("############## the network_addr:".$network_addr."with netmask: ".$network_mask);
   # Get our routing table so we can look for a device

   my $find_route = NATE::Process->new(
      codespec => $params{route_bin},
      runid    => $params{route_runid} . $params{client} . $timestamp,
      client   => $params{client},

   );

   $find_route->start();
   $find_route->wait();
   
   $Log->comment( "This is the output from the client route command: "
             . Dumper($find_route) );

   my $find_route_Log = $find_route->{logfile};
   open( LogFILE, $find_route_Log )
     || $Log->error("Could not open Logfile: $!\n");

   while (<LogFILE>) {

      # Check each line for the network address.

      if ( $_ =~m/$network_addr\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/){

         # When a line with the network address is found, check the mask.

         if ( $2 eq $network_mask ) {

            # If the mask is correct, then we have our device.

            $Log->debug(     "We will use device interface: $7 on network: $cidr_info\n"
            );
            $Log->exit();
            $find_route->destroy();
            return $7;
         }
      }
   }

   $Log->error( "No interface on this host for network: $cidr_info}.\n", );
   $find_route->destroy();
}

1;
