# $Id: //depot/prod/test/Rsierranevada/lib/Network/Vif.pm#1 $
#
## @summary Routines for invoking VIF operations.
##
## @description
##   This module contains all the library routines needed to configure 
##   VIF on the filer.
##
## @dependencies
##
## use Tharn;
##
## @author patchai@netapp.com

package Network::Vif;
require Exporter;

use strict;
use Tharn;
use Data::Dumper;
use SayWrap qw(sayEOF);
use Carp qw(croak);

use SPARSE::Util::Logger qw(logDbg logTrc logWarn);

use Network::Utils qw(run_ping);
use Network::Utils qw(do_ifconfig get_ifconfig_info);

use vars qw (@ISA @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT_OK = qw ( 
        vif_command
        vif_create 
	vif_create_multilevel
        vif_destroy 
        get_vif_status 
        verify_vif_status 
        vif_linkfake
        verify_load_balance
        vif_add
        vif_delete
        );

sub _get_vif_command {
    my $fc = shift;
    my $vif_cmd;
    my (undef,undef,$version) = say($fc,"version",-swallow);
    $vif_cmd="ifgrp";
#    if($version =~ /(?:main|boiler|8\.0|br|fsb|rr|Rrollingrock|rollingrock|8.\0\.1|8\.1)/i) {
    if($version =~ /(?:7\.2|7\.3|Rironcity|gordon)/i) {
        $vif_cmd = "vif";
    }
    
    return $vif_cmd;
}

sub vif_create
{
    my ($fc, %args) = @_;

    # Creation
    if ( ! defined($args{type}) ) {
        $args{type} = 'multi';
    }
    if ( !defined($args{netmask}) ) {
        $args{netmask} = "255.255.255.0";
    }
    
            foreach my $ifc ( split(/ /,$args{ifc_list}) ) {

   say($fc, "ifconfig $ifc 0.0.0.0");  
   say($fc, "ifconfig $ifc down");  
	    }

    my $filer_out = vif_command($fc, 'create' , %args);
    if ( $filer_out =~ /(vif|ifgrp):/ ) {
        logDbg("vif create failed step 1");
        return 0;
    }
sleep(10);
    my $vif_name = $args{name};
    my $vif_status = get_vif_status($fc, $vif_name); 
#    logcomment(Dumper($vif_status));
    
    # Verify vif is created and it has all the specified interfaces
    if ( exists $vif_status->{$vif_name} ) {

        # Check if the type is matching.
        if ( $vif_status->{$vif_name}{type} !~ m/$args{type}/ ) {
            croak "vif configure failed - type invalid";
        } 

        if ( defined($args{load_balance}) ) {

            my %map_load_balance = ( rr => 'Round-Robin Load balancing',
                                     mac => 'MAC Load balancing',
                                     port => 'IP+port Load balancing',
                                     ip => 'IP Load balancing' );

            my $expected_load_balance = $args{load_balance};
            if ( $args{load_balance} eq "" ) {
                $expected_load_balance = 'ip';
            }

            if ( $vif_status->{$vif_name}{load_balance} ne
                $map_load_balance{$expected_load_balance} ) {
                croak "vif configure failed - load balance invalid $vif_status->{$vif_name}{load_balance} ne $map_load_balance{$expected_load_balance}";
            }

        }

        if ( defined($args{ifc_list}) ) {
            foreach my $ifc ( split(/ /,$args{ifc_list}) ) {

                if ( ! exists $vif_status->{$args{name}}{$ifc} ) {
                    croak "vif configure failed - $ifc  not found";
                }
            }
        }
        if ( defined($args{ip_addr}) ) {
            do_ifconfig($fc, $vif_name, ip_addr => $args{ip_addr}, netmask => $args{netmask});
            logDbg("vif configure succeeded");
        }
    
    } else {
        croak "vif_create failed";
    }

    return 1;
}

sub vif_destroy
{
    my($fc, $vif_name) = @_;

#   do_ifconfig($fc, $vif_name, ip_addr => "0.0.0.0"); 
#       croak "do_ifconfig 0.0.0.0 failed for $vif_name";
#    do_ifconfig($fc, $vif_name, state => "down");
   say($fc, "ifconfig $vif_name 0.0.0.0 down");  
   say($fc, "ifconfig $vif_name down");  
#    do_ifconfig($fc, $vif_name, ip_addr => "0.0.0.0");
#   do_ifconfig($fc, $vif_name, state=>"down");
#    say($fc,"ifconfig $vif_name 0.0.0.0 down");  
    my %args;

    $args{name} = $vif_name;

    my $filer_out = vif_command($fc, 'destroy', %args);
    if ( $filer_out =~ /(?:vif|ifgrp):/ ) {
        logDbg("vif destroy printed some output: $filer_out");
        return 0;
    }
    
    #my $vif_status = get_vif_status($fc, $vif_name); 
    #
    #if ( exists $vif_status->{$vif_name} ) {
    #    logDbg(Dumper(%$vif_status));
    #    return 0;
    #}

    return 1;
}

sub vif_add {
    my ($fc, %args) = @_;

    logDbg(Dumper(%args));
    my $filer_out = vif_command($fc, 'add', %args);

    #if ( $filer_out ne "" ) {
    #    croak "vif_add: filer printed some output - $filer_out";
    #}
     
    if($filer_out =~ /is already attached to a/i) {
	logcomment("interface already attached to the vif");
	return 0; 
    } 
    # Verify the addtion.

    sleep 60;
    my $vif_status = get_vif_status($fc, $args{name}) or
        croak "vif_add: get_vif_status failed - $@";
    
    my @ifc_list = split(/,/, $args{ifc_list});

    foreach my $ifc ( @ifc_list ) {
        if ( ! exists $vif_status->{$args{name}}{$ifc} ) {
            my $ifc = $vif_status->{$args{name}}{$ifc};
            croak "vif_add: $ifc does not exist in vif - $@";
        }
    }
    
    return 1;
}

sub vif_delete {
    my ($fc, %args) = @_;

    my $filer_out = vif_command($fc, 'delete', %args);

    #if ( $filer_out ne "" ) {
    #    croak "vif_delete: filer printed some output - $filer_out";
    #}

    # Verify the deletion.

    my $vif_status = get_vif_status($fc, $args{name}) or
        croak "vif_delete: get_vif_status failed - $@";

    if ( exists $vif_status->{$args{name}}{ifc} ) {
        my $ifc = $vif_status->{$args{name}}{ifc};
        croak "vif_delete: $ifc still exists in vif - $@";
    }
    
    return 1;
}


sub vif_command
{
    my($fc, $command, %args) = @_;
    
    
    my $vf_cmd = _get_vif_command($fc);
    
    my $type = $args{type} if defined($args{type});
    my $load_balance = "-b ". $args{load_balance} 
        if ( defined($args{load_balance}) && $args{load_balance} ne "" );
    my $name = $args{name} if defined($args{name});

    my $cmd_str = "$vf_cmd $command $type $name $load_balance $args{ifc_list}";
    logDbg("vif_command: cmd_str = $cmd_str");

    my (undef,undef,$filer_out) = say($fc, $cmd_str);

    return($filer_out);
}


sub get_vif_status
{
    my($fc, $vif_name) = @_;
    my($filer_out) = @_;

    my %args;
    $args{name} = $vif_name;
    
    $filer_out = vif_command($fc, "status", %args);
    my @lines = split(/\n/, $filer_out);
    my %vif_status = ();

    if ( $#lines <= 1 ) {
        logDbg("vif status returned empty, vif may not exist");
	return \%vif_status;
    }

    my $vif_name = undef;
    my $pif_name = undef;
    my @up_links = ();
    my @down_links = ();
    my @broken_links =();
    foreach my $line ( @lines ) {

        if ( $line =~ m/^default/ ) {
            next;

        } elsif ( $line =~ m/(.*): (\d+) link(s)*, transmit '(.*)', (?:Ifgrp|VIF) Type '(.*)' fail '(.*)'/i ) {

			# if vif_name is already defined, then this line is for new vif.
		    # so let us save common info for vif_name so far collected.
			if ( defined ($vif_name) ) {
    			# point the up_links and down_links inside vif_status
    			$vif_status{$vif_name}{up_links} = \@up_links;	
    			$vif_status{$vif_name}{down_links} = \@down_links;	
			}

            $vif_name = $1;
            #$vif_status{vif_name} = $1;

            $vif_status{$vif_name}{links} = $2;
            $vif_status{$vif_name}{load_balance} = $4;
            $vif_status{$vif_name}{type} = $5;
            $vif_status{$vif_name}{fail} = $6;

        } elsif ( defined($vif_name) && !defined($pif_name)) {
            if ( $line =~ m/(?:Ifgrp|VIF) Status/ ) {
                if ( $line =~ m/Up/ ) {
                    $vif_status{$vif_name}{status} = 'up';
                }  else {
                    $vif_status{$vif_name}{status} = 'down';
                }
                next ;    #Skip the first line
            } elsif ( $line =~ m/^(down|up|broken):$/ ) {
                next ; # Skip the status line
            } elsif ( $line =~ m/^\s+(.*): state (.*), since (.*)/ ) {
            
                $pif_name = $1;
                $vif_status{$vif_name}{$pif_name}{status} = $2;
                $vif_status{$vif_name}{$pif_name}{uptime} = $3;
	        logcomment("$1...$2");
				# Put the up links and down links into arrays.
				if ( $2 eq 'up' ) {
					push @up_links, $1;
				} elsif ( $2 eq 'down' ||
                                          $2 eq 'lag_inactive') {
					push @down_links, $1;
				} elsif ($2 eq 'broken')
				{
					push @broken_links,$1;
				}
				else {
					croak "vif_status : value = $2; parsing error";	
				}

            } else {
                next;
            }

        } elsif ( defined($vif_name) && defined($pif_name) ) {
            if ( $line =~ m/mediatype: (.*)/ ) {
                $vif_status{$vif_name}{$pif_name}{media} = $1;
            } elsif ( $line =~ m/flags: (.*)/ ) {
                $vif_status{$vif_name}{$pif_name}{flags} = $1;
            } elsif ( $line =~ m/input packets (\d+), input bytes (\d+)/ ) {
                $vif_status{$vif_name}{$pif_name}{input_packets} = $1;
                $vif_status{$vif_name}{$pif_name}{input_bytes} = $2;
            } elsif ( $line =~ m/output packets (\d+), output bytes (\d+)/ ) {
                $vif_status{$vif_name}{$pif_name}{output_packets} = $1;
                $vif_status{$vif_name}{$pif_name}{output_bytes} = $2;
            } elsif ( $line =~ m/up indications (\d+), broken indications (\d+)/ ) {
                $vif_status{$vif_name}{$pif_name}{up_indications} = $1;
                $vif_status{$vif_name}{$pif_name}{broken_indications} = $2;

            } elsif ( $line =~ m/drops \(if\) (\d+), drops \(link\) (\d+)/ ) {

                $vif_status{$vif_name}{$pif_name}{drops_if} = $1;
                $vif_status{$vif_name}{$pif_name}{drops_link} = $2;

            } elsif ( $line =~ m/indication: (.*)/ ) {
                $vif_status{$vif_name}{$pif_name}{indication} = $1;
            } elsif ( $line =~ m/consecutive (\d+), transitions (\d+)/ ) {
                $vif_status{$vif_name}{$pif_name}{consecutive} = $1;
                $vif_status{$vif_name}{$pif_name}{transitions} = $2;
                $pif_name = undef;
            } else {
                logTrc("unrecongnized interface line : $line\n");
            }
        } else {
            logTrc("unrecognized virtual interface line: $line");
        }
    }
	# if vif_name is already defined, then this line is for new vif.
	# so let us save common info for vif_name so far collected.
	if ( defined ($vif_name) ) {
    	# point the up_links and down_links inside vif_status
    	$vif_status{$vif_name}{up_links} = \@up_links;	
    	$vif_status{$vif_name}{down_links} = \@down_links;	
    	$vif_status{$vif_name}{broken_links} = \@broken_links;	
	logcomment("@up_links @down_links @broken_links");
	}


    return(\%vif_status);
}


sub verify_vif_status
{
    my ($fc, $vif_name, %args) = @_;

    my $vif_status = get_vif_status($fc, $vif_name) or
        die  "vif_status failed - $@";
 
    if ( exists($vif_status->{$vif_name}) ) {

        # Check the vif type
        my $vif_type = $args{type};
        if ( $vif_status->{$vif_name}{type} !~ m/$vif_type/ ) {
            logDbg("vif type is not $vif_type");
            logDbg(Dumper(%$vif_status));
            return(0);
        } else {
            logDbg("Vif creation successful and type is $vif_type");
        }

    } else {
        logDbg("vif status did not show $vif_name");
        logDbg(Dumper(%$vif_status));
        return(0);
    }

    return(1);
}

sub vif_linkfake
{
    my($fc, %args) = @_;
    
    my $filer_out = sayEOF($fc, 
        "priv set diag; vif linkfake $args{state} $args{interface}"); 

    if ( $filer_out ne "" ) {
        die "vif_linkfake failed - $filer_out";
    }
}


sub verify_load_balance
{
    my($fc, $cc, %args ) = @_;

    my $vif_name = $args{vif_name};
    my $load_balance = $args{load_balance} eq "" ? 'ip' : $args{load_balance};
    my $client_ip = $args{client_ip};
    my $client_ifc = $args{client_ifc};
    my $num_interfaces = undef;

    my $time_to_ping = time + 60;
    while ( time < $time_to_ping ) {
        run_ping($fc, $client_ip, 'interface' => $vif_name )
            or die "ping failed using $vif_name - $@";
    }

    my $vif_status = get_vif_status($fc, $vif_name) or
        die $@;
    logDbg("Capturing the output_packets using vif status");

    my $total_output_packets  = 0;
    #my @keys_in_vif_status = keys %{$vif_status->{$vif_name}};
    my $arr_ref = $vif_status->{$vif_name}{up_links};
    my @up_links_in_vif = @$arr_ref;
logDbg("up links are ".Dumper(@up_links_in_vif));
    my %ifc_output_packets = ();
    foreach my $link ( @up_links_in_vif ) {
    #    if ( ref ($vif_status->{$vif_name}{$key}) ) {
   #         $ifc_output_packets{$link} = 
   #             $vif_status->{$vif_name}{$link}{output_packets};
            $total_output_packets += 
                $vif_status->{$vif_name}{$link}{output_packets};
   #     }
    }
  #  logDbg("output packets = ".Dumper(%ifc_output_packets));
    logDbg("total_output_packets = $total_output_packets");

    # Verify IP Load balancing
    #my @interface_list = keys %ifc_output_packets;
    my @new_interface_list = @up_links_in_vif;
    my @interface_list= reverse @new_interface_list;
    $num_interfaces = $#up_links_in_vif + 1;
    if ( $load_balance eq 'ip' || 
         $load_balance eq 'mac' ) {

        my ($src_last_byte, $dest_last_byte) = 
            get_last_byte_in_addr($fc, $cc, $vif_name, $load_balance, $client_ifc);

        my $ifc_index = ( $src_last_byte ^ $dest_last_byte ) 
            % $num_interfaces;
        logDbg("src_addr=$src_last_byte;dest_addr=".
               "$dest_last_byte; num_interfaces = $num_interfaces; ".
               "index = $ifc_index");
        my $total_packets = undef;
    
        my $chosen_interface = $interface_list[$ifc_index];
        my $ifc_packets = $vif_status->
		{$vif_name}{$chosen_interface}{output_packets};
        
        logDbg("Chosen Interface is $chosen_interface: packets = $ifc_packets");
        if ( $ifc_packets > int ((75/100)*$total_output_packets)) {
            logDbg("Verified that $chosen_interface xmits 75% of traffic");
        } else {
            logDbg("chosen interface $chosen_interface xmits ".
                "less than 75% of traffic");
            return 0;
        }

    } elsif ( $load_balance eq 'rr' ) {

        foreach my $ifc ( keys %ifc_output_packets ) {
            if ( $ifc_output_packets{$ifc} < 
                ( int ( ((1/$num_interfaces)/100)*$total_output_packets )
                  -  int ( (10/100)*$total_output_packets ) ) ) {
                logDbg("interface $ifc xmits less than 1/$num_interfaces".
						" of traffic");
                return 0;
            }
        }
        logDbg("all interfaces xmit equal amount of traffic in rr scheme");
        
    } else {
        die "load_balance not passed as arguments";
    }

    return 1;

}

sub get_last_byte_in_addr
{
    my ($fc, $cc, $vif_name,  $load_balance, $client_ifc) = @_;

    my $filer_ifconfig_info = get_ifconfig_info($fc, $vif_name) or
        die "cannot get ifconfig info of vif $vif_name";
    my $client_ifconfig_info = get_ifconfig_info($cc, $client_ifc) or
        die "cannot get ifconfig info from client ifc = $client_ifc";
    
    my $src_ip_addr = $filer_ifconfig_info->{$vif_name}{ip_addr};
    my $src_mac_addr = $filer_ifconfig_info->{$vif_name}{mac_addr};

    my $dest_ip_addr = $client_ifconfig_info->{$client_ifc}{ip_addr};
    my $dest_mac_addr = $client_ifconfig_info->{$client_ifc}{mac_addr};

    logDbg("src_ip_addr = $src_ip_addr; src_mac_addr = $src_mac_addr ".
        "dest_ip_addr = $dest_ip_addr; dest_mac_addr = $dest_mac_addr ");

    my $src_last_byte = undef;
    my $dest_last_byte = undef;
    if ( $load_balance eq 'ip' ) {

        $src_ip_addr =~ m/.*\.(\d+)/;    
        $src_last_byte = int $1;
        $dest_ip_addr =~ m/.*\.(\d+)/;
        $dest_last_byte = int $1;
        
    } elsif ( $load_balance eq 'mac' )  {
        
        $src_mac_addr =~ m/.*:(\w\w)/;    
        $src_last_byte = hex $1;
        $dest_mac_addr =~ m/.*:(\w\w)/;
        $dest_last_byte = hex $1;

    } 

    logDbg("src_last_byte = $src_last_byte; dest_last_byte = $dest_last_byte");


    return($src_last_byte, $dest_last_byte);
}

sub vif_create_multilevel
{
	my(%args) = @_;

	my $fc = $args{filer_conn};
	my $vif_name = $args{vif_name};
	my $vif_mode = $args{vif_type};
	my $filer_ifc = $args{filer_ifc};

	my @ifc_list = split($filer_ifc, /,/);
	
	my $level = 1;
	foreach my $ifc ( @ifc_list ) {
		vif_create( $fc, name => "vif_" . $ifc . "_$level",
		type => $vif_mode,
		ifc_list => $ifc ) or
		croak("vif_create_multilevel: level $level vifs creation using $ifc ".
			 "failed");
	}




	
}
