# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Network/VirtualIP.pm#1 $
#
# Copyright (c) 2006 Network Appliance Inc.
# All right reserved.
#
## @summary Routines for running network utilities
##
## @description
##         These functions are used to run various Virtual IP related 
## commands and operations
##
## @dependencies
##
## use Tharn;
##
## @author  dl-maryv (FSS Protocols QA)
##

package Network::VirtualIP;

require Exporter;
use strict;
use Tharn;
use Data::Dumper;
use SayWrap qw(sayEOF);
use SPARSE::Util::Logger qw(logStep logDbg logTrc logWarn);
use filer qw(filer_if_netmask_get);
use Utils::Units qw(:constants);
use Socket;
use filer;

use vars qw (@ISA @EXPORT_OK);

@ISA = qw(Exporter);
@EXPORT_OK = qw (virtual_ip_config
		 get_virtual_ip_info
		);

##
## @name :: get_virtual_ip_info
## 
## @description invokes ifconfig command and gathers information about
## virtual ip and returns the reference to hash.
##
## @arg $conn - connection object
## 
## @return reference to hash of ip addresses upon success
##         croaks on failure
##
## example: get_virtual_ip_info($conn);
##

sub get_virtual_ip_info
{
    my ($conn) = @_;
    my %ifconfig_info = ();

    my $filer_out = sayEOF($conn, "ifconfig vip");

    my @ifconfig_out = split(/\n/, $filer_out);

    foreach my $line ( @ifconfig_out ) {
	if ( $line =~ m/^vip/ ) {
	    next;
	} elsif ( $line =~ m/inet\s((\d+\.){3}(\d+))\snetmask\s(.*)\sbroadcast\s((\d+\.){3}(\d+))/ ) {
	    $ifconfig_info{$1} = "1";
	} else {
	    logDbg("get_virtual_ip_info:unrecognized line in output: $line\n");
	    next;
	}
    }
    return(\%ifconfig_info);
}


##
## @name :: virtual_ip_config
## 
## @description 
## Configures (add/delete) virtual ip on a filer
##
## @arg $conn - connection object
## 
## @arg %args - hash of various options
##            "ip" -  ip address to configure
##            "action"  - 'add' or 'remove' or 'delete'
##
## @notes The action argument values have the following meanings
##        add    - add this ip as a virtual ip
##        remove - gracefully remove this virtual ip using RIP metric 16
##        delete - forcefully remove the virtual ip without RIP updates
##
## @return 1 upon success or 0 upon failure.
##
## example: virtual_ip_config($conn, 
##                               ip => '172.16.102.101',
##                               action =>  'add'
##

sub virtual_ip_config
{
    my ($conn,%args) = @_;

    if ($args{'action'} !~ /add|delete|remove/) {
	return(0);
    }

    my $result = sayEOF($conn,"ifconfig vip $args{'action'} $args{'ip'}");

    if ($args{'action'} =~ m/add/i) {
	if ($result !~ m/Added VIP in active mode/) {
	    return(0);
	}
    } elsif ($args{'action'} =~ /delete|remove/) {
	if ($result =~ m/ifconfig: ioctl \(Could not delete VIP/) {
	    return(0);	
	}
    }
    
    return(1);
}


1;

