#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/Node/Cfe.pm#1 $
## @summary     Module to define commands for CFE
## @author dl-central-libs
## @pod here

=head1 NAME

Node::Cfe - Module to define commands for CFE

=head1 AUTHOR

dl-central-libs

=head1 SYNOPSIS

Internal Use

=head1 DESCRIPTION

This package is used to modularize the command definitions for CFE.

=head1 METHODS

none

=cut


#
# It seems odd we would put a package scope here when everywhere else we avoid package
# scopes like the plague so we can have things pop in all over the namespace at will.
# But it turns out when we are dealing with different Server objects in the same piece of
# code, the "use Node::Cfe"; will populate the first namespace just fine (IE if it is a Node::NG
# object we fill in Node::NG::Cfe like expected), but the subsequent uses are not pulled into their
# respective name spaces (IE a Node::Classic object won't get the Node::Classic::Cfe)  So we've 
# hardcoded this as Node::Cfe for now so we can get past this problem.  If we don't, the AUTOLOADER
# attempts to access the file and blows chunks all over the place.
#
# What if??? -- instead we just loaded this into Node (which would scope it as Node::Cfe) and then told
# the common module loader to process "Node::Cfe"  Each object should still process SetupCfeCommands
# like it should and we wouldn't need this here.  
#

package Node;

use strict;
use TracePkg;
my $Trace = new TracePkg;

my %Options = (
);


my %CommandDefinitions = (
    boot_primary_wipeclean_true => {
        command => "boot_primary",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "240",
        additional_prompts => {
            "CFE>"    => 'FINAL_PROMPT',
	        "LOADER.*>" => 'FINAL_PROMPT',
            "SIMLOADER>" => 'FINAL_PROMPT',
        },
    },
    boot_primary => {
        command => "boot_primary",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        additional_prompts => [
        {
            prompt => 'login:',
            response => "",
            timeout => 300,
        },
        ],
    },
    version => {
        command => "version",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        optional_options => undef,
    },
    printenv => {
        command => "printenv",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        optional_options => undef,
        #optional_args => "arg",
        no_validation => "1",
    },
    setenv => {
        command => "setenv",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "30",
        additional_prompts => {
            "CFE>"       => 'FINAL_PROMPT',
            "LOADER.*>"    => 'FINAL_PROMPT',
            "SIMLOADER>" => 'FINAL_PROMPT',
        },
    },
    unsetenv => {
        command => "unsetenv",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    bye => {
        command => "bye",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    flash => {
        command => "flash",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    'setdefaults' => {
        command => "set-defaults",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    'ifconfig' => {
        command => "ifconfig",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    ping => {
        command => "ping",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    priv_set_advanced => {
        command => "priv set advanced",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    rlm_reset => {
        command => "rlm reset",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
    cfe_reboot => {
        command => "bye",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "240",
          additional_prompts => {
                'Starting AUTOBOOT'  => "\cc",
                'for command prompt' => "\cc",
                'LOADER.*>'            => "FINAL_PROMPT",
                'SIMLOADER>'         => "FINAL_PROMPT",
                'CFE>'               => "FINAL_PROMPT",},
    },    
    # the following cdef is for error handling testing only
    foo => {
        command => "foo",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "cfe_output_handler",
        no_validation => "1",
        optional_options => undef,
    },
);

my %Compatability = (
);


sub SetupCfeCommands {
    my ($self, @servers) = @_;
    $Trace->entry();

    $self->_define_commands(Definitions => \%CommandDefinitions,
                            Compatability => \%Compatability,
                            Servers => \@servers);
    $Trace->exit();
    return 1;
}

sub cfe_output_handler {
    my ($self, %params) = @_;
    my $want_array = $params{want_array};
    $want_array = [] if (!$want_array);
    my $output = $params{output_in};
    my @output = split(/\n/, $output);

    $Trace->debug(msg=>"Params: " . Dumper(%params));
    #the fas2040's LOADER tosses a failure when a param is unset that is not already set.
    #catch that error so that we do not FATAL.
    my $fas2040_failure = 0;
    foreach my $line (@output) {
        my %return_output;
        if ($line =~ /Environment variable not found/) {
            $fas2040_failure = 1;
            next;
        }
        if ($line =~ /^\*+\s+command\s+status/){
            unless ($fas2040_failure) {
                $Trace->fatal(msg=>"Command failed!");
            }
        }
        if ($line =~ /Variable\sName/) {next};
        if ($line =~ /(\w+)\s+([a-z,A-Z,0-9,-,._]+)/){
            $return_output{$1} = $2;
            #$Trace->debug(msg=>"Match key $1, value $2....");
        }
        push(@$want_array, \%return_output);
    }

    # handle requests for scalar output
    if (!$params{want_array}) {  # don't check $want_array here!
        #$Trace->debug(msg=>"\@\$want_array is: @$want_array");
        #if (keys %{@$want_array} gt 1) {
        #    $Trace->warn(msg=>"Scalar output was requested, but there are multiple elements!");
        #}
        # grab the value of the last element; we really only expect one element
        foreach my $x (@$want_array){
            foreach my $v (values %{$x}){
                $want_array = $v;
            }
        }
    } else {
        #$Trace->debug(msg=>"\@\$want_array has ".scalar @$want_array." elements: ".Dumper(\@$want_array));
    }
    
    $Trace->exit(msg=>"Exiting cfe_output_handler2; returning $want_array." );
    return $want_array;
}

1;
