#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Enables remote programmatic administration of a Filer running
##          6.X
## @author dl-central-libs
## @pod here

=head1 NAME

6_X.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module is meant to be programmatic way of administering 6.X filers

=head1 SYNOPSIS

This module is instantiated by Server.pm and is not directly called.
See documentation below for help on individual methods 

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_6_X}) {
    my $newModule = $ENV{OVERRIDECLASSIC_6_X};
    delete $ENV{OVERRIDECLASSIC_6_X};
    print "The main Classic::6_X module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

package Node::Classic::6_X;
use strict;
use base qw(Node::Classic);
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF);

use TracePkg;
my $Trace = new TracePkg();

our @Cmd_nodes = qw(aggregate network_ports network_interfaces exportruleset_rules volume);
our %Cmd_node_definitions = (
			     discover => {
				 aggregate => "aggr_status",
				 network_ports => "network_ports_show",
				 network_interfaces => "network_interfaces_show",
				 volume => [qw(vol_show aggregate \$resource_params->{aggregate})],
				 vserver => "vfiler_status",
				 exportruleset_rules => "protocol_exportruleset_rule_show",
			     },
			     create => {
				aggregate => "aggr_create",
				network_interfaces => "network_interfaces_new",
				volume => [qw(volume_new aggregate \$resource_params->{aggregate})],
				exportruleset_rules => "protocol_exportruleset_rule_new",
			     },
			     exists => {
				 aggregate => "aggr_exists",
				 network_interfaces => "network_interfaces_exists",
				 volume => [qw(vol_exists aggregate \$resource_params->{aggregate})],
				 exportruleset_rules => "protocol_exportruleset_rule_exists",
			     },
			     delete => {
				 aggregate => "aggr_destroy",
				 network_interfaces => "network_interfaces_delete",
				 volume => [qw(volume_delete)],
				 exportruleset_rules => "protocol_exportruleset_rule_delete",
			     },
			     );

my @Modules = qw(Aggr);

sub _define_modules {
    my $self = shift;
    $self->{Trace}->entry();
    $self->{modules} = \@Modules;
    $self->{Trace}->exit();
}

sub _define_cmd_nodes {
    my $self = shift;
    $self->{Trace}->entry();

    $self->{cmd_nodes} = \@Cmd_nodes;
    $self->{cmd_node_definitions} = \%Cmd_node_definitions;

    $self->{Trace}->exit();
}

sub convert_export_path {
    my $self = shift;
    my %params = (export => undef,		  
		  aggregate => undef,
		  volume => undef,
		  @_);

    $self->{Trace}->entry(msg=>"volume=$params{volume}, aggregate=>$params{aggregate}");

    my $export = "/vol/$params{aggregate}/$params{volume}";
    $self->{Trace}->exit(msg=>"return $export");
    return ($export, $export);
}

sub filter_vifs
{
    my ($self, %params) = validate_mixed(
					 @_,
					 [ { type => OBJECT } ],
					 {
					     volume => {type => HASHREF},
					     interfaces => {type => ARRAYREF},
					     aggregate => {type => HASHREF},
					     traffic => {type => SCALAR},
					     topology => {type => HASHREF},
					 }
					 );
    my $traffic = $params{traffic};
    my $volume_name = $params{volume}->{volume};

    $Trace->entry(msg => "Filtering vifs for $volume_name (cluster traffic: $traffic)");
    my @filtered_vifs = @{$params{interfaces}};

    if ($traffic eq 'remote') {
	$Trace->fatal(msg => "CLUSTER_TRAFFIC='$traffic' is not supported against 6.X filers", result=>"CONFIG");
    } else {
        $Trace->trace(msg => "no filering occurs against 6.X. Returning the original list.");
    }	

    $Trace->exit(msg => "Returning ".join("\n", (map( { "IP: ". $_->{address}." with Home-Filer: ".$_->{'home-filer'} } @filtered_vifs))));
    return map( { $_->{address} } @filtered_vifs);
}

1; # Do NOT remove this. It is needed by the package.

