#!/usr/local/perl/bin
#$Id: //depot/prod/test/Rsierranevada/lib/Node/Classic/ASUP.pm#1 $
#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary These are the command definitions for calling 7mode autosupport options.
## @author Christopher Wilcox, dl-cluster-qa
## @pod here

=head1 NAME

ASUP.pm

=head1 AUTHOR

Christopher Wilcox, dl-cluster-qa

=head1 DESCRIPTION

This module implements all of the autosupport command options in ONTAP 7mode

=head1 SYNOPSIS

The standard naming convention for the command definitions take the option name,
and substitute dots (.) for underscores (_)
=head1 EXAMPLES

use Server;

my $Server  = Server->new();

$output = $Server->autosupport_cifs_verbose(enable=>1);

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_ASUP}) {
    my $newModule = $ENV{OVERRIDECLASSIC_ASUP};
    delete $ENV{OVERRIDECLASSIC_ASUP};
    print "The main Classic::ASUP module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %autosupport_conversion = (
			);
my %options = (
        enable => "",
        content => "",
        message => "",
        address => "",
        id => "",
        host => "",
        count => "",
        interval => "",
        proxy => "",
        url => "",
        'put-url' => "",
        transport => "",
        payload_format => "",
        periodic_tx_window => "",
        to=>"-to",
        noteto=>"-noteto",
        'autosupport-message'=>"-autosupport-message",
        'seq-num'=>"-seq-num",
        'prio-num'=>"-prio-num",
        subsystem=>"-subsystem",
        'cmd-tgt'=>"-cmd-tgt",
        'body-file'=>"-body-file",
        cmd=>"-cmd",
        query=>"-query",
        'size-collected'=>"-size-collected",
        status=>"-status",
        error=>"-error",
        'content-type'=>"-content-type",
        'asup-seq'=>"-asup-seq",
        destination=>"-destination",
        'trigger'=>"-trigger",
        'last-update'=>"-last-update",
        'attempt-count'=>"-attempt-count",
        subject=>"-subject",
        'autosupport-message'=>"-autosupport-message",
        'basic-default'=>"-basic-default",
        'troubleshooting-default'=>"-troubleshooting-default",
        'additional-content'=>"-additional-content",
        'basic-additional'=>"-basic-additional",
        'troubleshooting-additional'=>"-troubleshooting-additional",
        'subsystem'=>"-subsystem",
        'seq-num'=>"-seq-num",
        'uri'=>"-uri",
        'size-limit'=>"-size-limit",
        'time-limit'=>"-time-limit",
      );

my %command_definitions = (
                    autosupport_cifs_verbose => {
                                   command => "options autosupport.cifs.verbose",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_content => {
                                   command => "options autosupport.content",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(content)],
                               },
                    autosupport_doit => {
                                   command => "options autosupport.doit",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(message)],
                               },
                    autosupport_enable => {
                                   command => "options autosupport.enable",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_from => {
                                   command => "options autosupport.from",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(address)],
                               },
                    autosupport_local_nht_data_enable => {
                                   command => "options autosupport.local.nht_data.enable",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_local_performance_data_enable => {
                                   command => "options autosupport.local.performance_data.enable",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_minimal_subject_id => {
                                   command => "options autosupport.minimal.subject.id",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(id)],
                               },
                    autosupport_mailhost => {
                                   command => "options autosupport.mailhost",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(host)],
                               },
                    autosupport_nht_data_enable => {
                                   command => "options autosupport.nht_data.enable",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_noteto => {
                                   command => "options autosupport.noteto",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(address)],
                               },
                    autosupport_performance_data_enable => {
                                   command => "options autosupport.performance_data.enable",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_retry_count => {
                                   command => "options autosupport.retry.count",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(count)],
                               },
                    autosupport_retry_interval => {
                                   command => "options autosupport.retry.interval",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(interval)],
                               },
                    autosupport_support_enable => {
                                   command => "options autosupport.support.enable",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_support_proxy => {
                                   command => "options autosupport.support.proxy",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(proxy)],
                               },
                    autosupport_support_to => {
                                   command => "priv set diag; options autosupport.support.to",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(address)],
                               },
                    autosupport_support_transport => {
                                   command => "options autosupport.support.transport",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(transport)],
                               },
                    autosupport_support_url => {
                                   command => "priv set diag; options autosupport.support.url",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(url)],
                               },
                    autosupport_payload_format => {
                                   command => "priv set diag; options autosupport.payload_format",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(payload_format)],
                               },
                    autosupport_throttle => {
                                   command => "options autosupport.throttle",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_to => {
                                   command => "options autosupport.to",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(address)],
                               },
                    autosupport_partner_to => {
                                   command => "options autosupport.partner.to",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(address)],
                               },
                    autosupport_performance_data_doit => {
                                   command => "options autosupport.performance_data.doit",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(message)],
                               },
                    autosupport_change_filext => {
                                   command => "priv set diag; options autosupport.change_filext",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_max_http_size => {
                                   command => "options autosupport.max_http_size",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(count)],
                               },
                    autosupport_max_smtp_size => {
                                   command => "options autosupport.max_smtp_size",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(count)],
                               },
                    autosupport_periodic_tx_window => {
                                   command => "options autosupport.periodic.tx_window",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(periodic_tx_window)],
                               },
                    autosupport_validate_digital_certificate => {
                                   command => "options autosupport.validate_digital_certificate",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_filter_show => {
                                    command => "priv set diag; autosupport filter show",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(autosupport-message to noteto)],
                               },
                    autosupport_filter_modify => {
                                    command => "priv set diag; autosupport filter modify",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(autosupport-message to noteto)],
                               },
                    autosupport_manifest_show => {
                                    command => "priv set diag; autosupport manifest show -instance",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(seq-num prio-num subsystem cmd-tgt body-file cmd query size-collected status error content-type)],
                               },
                    autosupport_history_show => {
                                    command => "priv set diag; autosupport history show -instance",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(seq-num destination trigger last-update status attempt-count subject uri error)],
                               },
                    autosupport_history_retransmit => {
                                    command => "priv set diag; autosupport history retransmit",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(seq-num uri size-limit)],
                               },
                    autosupport_trigger_show => {
                                    command => "priv set diag; autosupport trigger show -instance",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(autosupport-message to noteto basic-default troubleshooting-default additional-content basic-additional troubleshooting-additional time-limit)],
                               },
                    autosupport_trigger_modify => {
                                    command => "priv set diag; autosupport trigger modify",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(autosupport-message to noteto basic-additional troubleshooting-additional time-limit)],
                               },
                    autosupport_put_url => { 
                                   command => "priv set diag; options autosupport.support.put_url",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(put-url)],
                               },
                    autosupport_budget_show => {
                                    command => "priv set diag; autosupport budget show -instance",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(subsystem size-limit time-limit)],
                               },
                    autosupport_budget_modify => {
                                    command => "priv set diag; autosupport budget modify",
                                    conversion => \%autosupport_conversion,
                                    options => \%options,
                                    optional_options => [qw(subsystem size-limit time-limit)],
                               },
                    autosupport_ondemand_state => {
                                   command => "priv set diag; options autosupport.ondemand.state",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
                    autosupport_ondemand_polling_interval => {
                                   command => "priv set diag; options autosupport.ondemand.polling_interval",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(interval)],
                               },
                    autosupport_ondemand_server_url => {
                                   command => "priv set diag; options autosupport.ondemand.server_url",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(url)],
                               },
                    autosupport_ondemand_remotediag_state => {
                                   command => "priv set diag; options autosupport.ondemand.remotediag.state",
                                   conversion => \%autosupport_conversion,
                                   options => \%options,
                                   optional_options => [qw(enable)],
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupASUPCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
