#***********************************************************************
#
# NAME
#   Aggr.pm - remote programmatic administration of a NetApp Aggr.
#
# DESCRIPTION
#   This Perl module is meant to include only the vol commands
#
# METHODS
#
#************************************************************************
## @author dl-central-libs
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_AGGR}) {
    my $newModule = $ENV{OVERRIDECLASSIC_AGGR};
    delete $ENV{OVERRIDECLASSIC_AGGR};
    print "The main Classic::Aggr module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

use strict;
# only require so that the current namespace is not modified.
require Aggregate;

our %Aggrconversion = (
			);
my %options = (
	       aggregate => " ",
	       blocks => "-b",
	       verbose => "-v",
	       );

my %command_definitions = (
		      aggr_status => {
			  command => "aggr status",
			  conversion => \%Aggrconversion,
			  options => \%options,
			  optional_options => [qw(aggregate blocks verbose)],
			  verification => "_process_aggr_status",
			  mutual_exclusions => {
			  },
			  failure_output => [("No aggrgate named")],
		      },
		      
		      );

my %Compatability = (
		     storage_aggregate_show => "aggr_status",
		     );

sub SetupAggrCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

sub aggr_exists {
    my ($self, %params) = @_;
    $self->{Trace}->entry();

    my $rc = Aggregate::aggr_exists($self->get_connection(type=>"cli_connection"),$params{aggregate});
    $self->{Trace}->exit(msg => "returning $rc");
}

sub _process_aggr_status {
    my ($self, %params) = @_;
    $self->{Trace}->entry(msg=>"aggregate=$params{aggregate}");
    my $want_array = $params{want_array};
    $want_array = [] if (!$want_array);
    my $output = $params{output_in};

    my $more_volumes;
    my $parse = undef;
    if ($params{aggregate}) {
	my %aggr = ();
	foreach my $line (split(/\n/, $output)) {
	    if ($line =~ /\s+Aggr\s+/) {
		$parse = 1;
		next;
	    }
	    next if (!$parse);
	    if ($line =~ /^\s*$params{aggregate}\s+(\S+)\s+raid\S+, (\S+)/) {
		$aggr{aggregate} = $params{aggregate};
		$aggr{state} = $1;
		$aggr{type} = $2;
	    } elsif ($line =~ /\s+(32-bit|64-bit)\s+/) {
                $aggr{'block-type'} = $1;
            } elsif ($line =~ /\s+Volumes: (\S+),*/) {
		$aggr{volumes} = [$1];
		$more_volumes = 1 if ($line =~ /,\s*$/);
	    } elsif ($more_volumes && $line =~ /(\S+),*/) {
		push(@{$aggr{volumes}}, $1);
		$more_volumes = undef if ($line !~ /,\s*$/);
	    } elsif ($line =~ /$params{aggregate}\s+(\d+)\s+(\d+)\s+(\d+)/) {
		my $block_size = $1;
		$aggr{size} = _size_in_MB($2, $block_size);
		$aggr{availsize} = _size_in_MB($3, $block_size);
	    }
	}
	$aggr{filers} = $self->{server};
	$aggr{voltype} = "flex";
	push(@$want_array, \%aggr);
    } else {
	foreach my $line (split(/\n/, $output)) {
	    next if ($line =~ /\s+Aggr State/);
	    next if ($line =~ /^\s{12,}/);
	    next if ($line !~ /aggr|trad/);
	    next if ($line =~ /\(\d+\) offline/);
	    if ($line =~ /^\s*(\S+)\s+\S+/) {
		my $aggr_list = $self->aggr_status(aggregate=>$1, verbose=>" ");
		my $aggrh = $$aggr_list[0];
		$aggr_list = $self->Df(aggregate=>$1, MB=>"true");
		my $blocksh = $$aggr_list[0];
		grep($aggrh->{$_} = $blocksh->{$_}, keys %$blocksh);
		push(@$want_array, $aggrh);
	    }
	}
    }
    $self->{Trace}->exit(msg => "Exiting with $want_array");
    return $want_array;
}

sub _size_in_MB {
    my ($blocks, $numblocks) = @_;
    
    return int ((($blocks * $numblocks) / 1024) / 1024);
}

__DATA__
sub _WaitForAggregate {
    my $self = shift;
    my $method = "Ontag::Aggr::_WaitForAggregate";
    my %params = @_;
    my $rc = undef;
    my $output;

    while ($params{wait}) {
	$self->storage_aggregate_show(aggregate=>$params{aggregate}, output=>\$output);
	foreach my $line (split(/\n/, $output)) {
	    if ($line =~ /\s+$params{aggregate}\s+online/) {
		$rc = 1;
		goto done;
	    }
	}
	sleep 1;
    }
    
  done:
    return $rc;
}

1;
