#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "arp" commands in ONTAP 
## @author dl-central-libs
## @pod here

=head1 NAME

Arp.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "arp" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#************************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_ARP}) {
    my $newModule = $ENV{OVERRIDECLASSIC_ARP};
    delete $ENV{OVERRIDECLASSIC_ARP};
    print "The main Classic::Arp module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Arpconversion = (
			);

my %options = (
		    HostName => " ",
		    All => "-a",
		    EthernetAddress=> " ",
		    IPOnly => "-n",
		    Temporary => "temp",
		    Publish => "pub",
		    );

#************************************************************************

=head1 C<Arp()>

=over 4

=item Description

This method implements the "arp [-n] {<hostname> | -a}" command

=item Arguments

=over 4

=item HostName
(Required)

This is the specific hostname to perform the arp entry

Either HostName or All must be specified, but not both

=item All
(Required)

Implements the -a flag

Either HostName or All must be specified, but not both.

Supported values: "true" or "false"

=item IPOnly
(Optional)

Implements the -n flag

Display IP addresses instead of hostnames

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<ArpDelete()>

=over 4

=item Description

This method implements the "arp -d <hostname>" command

=item Arguments

=over 4

=item HostName
(Required)

This is the specific hostname to perform the arp delete on.

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<ArpCreate()>

=over 4

=item Description

This method implements the "arp -s <hostname> <ethernet-address> [temp] [pub]" command

=item Arguments

=over 4

=item HostName
(Required)

This is the specific hostname to perform the arp delete on.

=item EthernetAddress
(Required)

This is the ethernet address assigned to the new arp entry

=item Temporary
(Optional)

Implements the "temp" keyword and makes the new arp entry temporary

Supported Values: "true" or "false"

=item Publish
(Optional)

Implements the "pub" keyword and publishes the new arp entry

=back

=item Returns

None

=back

=cut

#************************************************************************


my %command_definitions = (
			       Arp => {
				   command => "arp",
				   conversion => \%Arpconversion,
				   options => \%options,
				   required_options => [qw(HostName All)],
				   optional_options => [qw(IPOnly)],
				   failure_output => [("Unknown host")],
				   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     HostName => [qw(All)],
                                     All => [qw(HostName)],
                                   },
				   supported_values => {
                                     All => [qw(true false)],
                                     IPOnly => [qw(true false)],
                                   },
			       },
			       ArpDelete => {
                                   command => "arp -d",
                                   conversion => \%Arpconversion,
                                   options => \%options,
				   required_options => [qw(HostName)],
				   successful_output => 'deleted',
                                   failure_output => [("Unknown host","cannot locate")],
                                   verification => "_ReturnOutputverification",
                               },
			       ArpCreate => {
                                   command => "arp -s",
                                   conversion => \%Arpconversion,
                                   options => \%options,
                                   required_options => [qw(HostName EthernetAddress)],
				   post_command_options => [qw(Temporary Publish)],
                                   failure_output => [("Unknown host","invalid Ethernet address","cannot locate","writing to routing socket\: File exists")],
				   supported_values => {
                                     Temporary => [qw(true false)],
                                     Publish => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupArpCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
