#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "backup" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Backup.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "backup" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#************************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_BACKUP}) {
    my $newModule = $ENV{OVERRIDECLASSIC_BACKUP};
    delete $ENV{OVERRIDECLASSIC_BACKUP};
    print "The main Classic::Backup module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Backupconversion = (
			);
my %options = (
		    BackupID => " ",
		    );
#************************************************************************

=head1 C<BackupStatus()>

=over 4

=item Description

This method implements the "backup status" command

=item Arguments

=over 4

=item BackupID
(Optional)

This is the specific backup job to get status on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<BackupTerminate()>

=over 4

=item Description

This method implements the "backup terminate" command

=item Arguments

=over 4

=item BackupID
(Required)

This is the specific backup job to terminate 

=back

=item Returns

None

=back

=cut

#************************************************************************


my %command_definitions = (
			       BackupStatus => {
				   command => "backup status",
				   conversion => \%Backupconversion,
				   options => \%options,
				   optional_options => [qw(BackupID)],
				   failure_output => [("Specific backup does not exist")],
				   verification => "_ReturnOutputverification",
			       },
			       BackupTerminate => {
                                   command => "backup terminate",
                                   conversion => \%Backupconversion,
                                   options => \%options,
                                   required_options => [qw(BackupID)],
                                   failure_output => [("Specific backup does not exist")],
                                   verification => "_ReturnOutputverification",
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupBackupCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
