#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "cf" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Cf.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "cf" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#************************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_CF}) {
    my $newModule = $ENV{OVERRIDECLASSIC_CF};
    delete $ENV{OVERRIDECLASSIC_CF};
    print "The main Classic::Cf module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Cfconversion = (
			);
my %options = (
		   Force => "-f",
		   Mirrors => "-d",
		   Module => " ",
		    );
#************************************************************************

=head1 C<CfDisable()>

=over 4

=item Description

This method implements the "cf disable" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfEnable()>

=over 4

=item Description

This method implements the "cf enable" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfForcegiveback()>

=over 4

=item Description

This method implements the "cf forcegiveback" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfForcetakeover()>

=over 4

=item Description

This method implements the "cf forcetakeover" command

=item Arguments

=over 4

=item Force
(Optional)

Implements the "-f" flag, and forces takeover to occur

Supported Values: "true" or "false"

=item Mirrors
(Optional)

Implements the "-d" flag and forces takeover to occur when 
remotesyncmirror is licensed

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfGiveback()>

=over 4

=item Description

This method implements the "cf giveback" command

=item Arguments

=over 4

=item Force
(Optional)

Implements the "-f" flag, and forces giveback to occur so long as no 
data corruption could occur.

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfMonitor()>

=over 4

=item Description

This method implements the "cf monitor" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfPartner()>

=over 4

=item Description

This method implements the "cf partner" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfStatus()>

=over 4

=item Description

This method implements the "cf status" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfTakeover()>

=over 4

=item Description

This method implements the "cf takeover" command

=item Arguments

=over 4

=item Force
(Optional)

Implements the "-f" flag, and forces takeover to occur so long as no
data corruption could occur.

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfNFOEnable()>

=over 4

=item Description

This method implements the "cf nfo enable" command

=item Arguments

=over 4

=item Module 
(Required)

The negotiated failover module to enable.

Supported Values: "disk_shelf"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfNFODisable()>

=over 4

=item Description

This method implements the "cf nfo disable" command

=item Arguments

=over 4

=item Module
(Required)

The negotiated failover module to enable.

Supported Values: "disk_shelf"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<CfNFOStatus()>

=over 4

=item Description

This method implements the "cf nfo status" command

=item Arguments

=item Module

None

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       CfDisable => {
				   command => "cf disable",
				   conversion => \%Cfconversion,
				   options => \%options,
				   verification => "_ReturnOutputverification",
			       },
			       CfEnable => {
				   command => "cf enable",
				   conversion => \%Cfconversion,
				   options => \%options,
				   verification => "_ReturnOutputverification",
			       },
			       CfForcegiveback => {
				   command => "cf forcegiveback",
				   conversion => \%Cfconversion,
				   options => \%options,
				   verification => "_ReturnOutputverification",
			       },
			       CfForcetakeover => {
				   command => "cf forcetakeover",
				   conversion => \%Cfconversion,
				   options => \%options,
                                   optional_options => [qw(Force Mirrors)],
				   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Force => [qw(true false)],
                                     Mirrors => [qw(true false)],
                                   },
			       },
			       CfGiveback => {
				   command => "cf giveback",
				   conversion => \%Cfconversion,
				   options => \%options,
                                   optional_options => [qw(Force)],
                                   successful_output => [("giveback initiated by operator")],
                                   failure_output => [("not in takeover state")],
				   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Force => [qw(true false)],
                                   },
			       },
			       CfMonitor => {
				   command => "cf monitor",
				   conversion => \%Cfconversion,
				   options => \%options,
				   verification => "_ReturnOutputverification",
			       },
			       CfPartner => {
				   command => "cf partner",
				   conversion => \%Cfconversion,
				   options => \%options,
				   verification => "_ReturnOutputverification",
			       },
			       CfStatus => {
				   command => "cf status",
				   conversion => \%Cfconversion,
				   options => \%options,
				   verification => "_ReturnOutputverification",
			       },
			       CfTakeover => {
				   command => "cf takeover",
				   conversion => \%Cfconversion,
				   options => \%options,
                                   optional_options => [qw(Force)],
                                   successful_output => [("takeover initiated by operator")],
                                   failure_output => [("in takeover mode")],
				   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Force => [qw(true false)],
                                   },
			       },
			       CfNFOEnable => {
                                   command => "cf nfo enable",
                                   conversion => \%Cfconversion,
                                   options => \%options,
                                   required_options => [qw(Module)],
                                   failure_output => [("not a valid module name")],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Module => [qw(disk_shelf)],
                                   },
                               },
			       CfNFODisable => {
                                   command => "cf nfo disable",
                                   conversion => \%Cfconversion,
                                   options => \%options,
                                   required_options => [qw(Module)],
                                   failure_output => [("not a valid module name")],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Module => [qw(disk_shelf)],
                                   },
                               },
			       CfNFOStatus => {
                                   command => "cf nfo status",
                                   conversion => \%Cfconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },

			       );
			       
my %Compatability = (
		     );

sub SetupCfCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
