#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "config" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Config.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "config" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#************************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_CONFIG}) {
    my $newModule = $ENV{OVERRIDECLASSIC_CONFIG};
    delete $ENV{OVERRIDECLASSIC_CONFIG};
    print "The main Classic::Config module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Configconversion = (
			);
my %options = (
		   Filer => " ",
		   RemoteUser => " ",
		   OutputFile => "-o",
		   ConfigFile => " ",
		   ConfigFile1 => " ",
		   ConfigFile2 => " ",
		   Force => "-f",
		   Volume => "-v",
		    );
#************************************************************************

=head1 C<ConfigClone()>

=over 4

=item Description

This method implements the "config clone" command

=item Arguments

=over 4

=item Filer 
(Required)

This is the specific filer to clone

=item RemoteUser 
(Required)

This is the remote user:password

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<ConfigDiff()>

=over 4

=item Description

This method implements the "config diff" command

=item Arguments

=over 4

=item ConfigFile1 
(Required)

This is the config file to use for comparision

=item OutputFile 
(Optional)

Implements the "-o" flag, this is the output file

=item ConfigFile2
(Optional)

This is the second config file to compare, instead of current config

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<ConfigDump()>

=over 4

=item Description

This method implements the "config dump" command

=item Arguments

=over 4

=item ConfigFile
(Required)

This is the output config file to dump to

=item Force
(Optional)

Implements the "-f" flag, this forces the dump operation to
override any exisitng config backup file 

Supported Values: "true" or "false"

=item Volume
(Optional)

Implements the "-v" flag, this causes volume specific info to be backed up as well

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<ConfigRestore()>

=over 4

=item Description

This method implements the "config restore" command

=item Arguments

=over 4

=item ConfigFile
(Required)

This is the output config file to restore 

=item Volume
(Optional)

Implements the "-v" flag, this causes volume specific info to be restored as well

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************


my %command_definitions = (
			       ConfigClone => {
				   command => "config clone",
				   conversion => \%Configconversion,
				   options => \%options,
                                   required_options => [qw(Filer RemoteUser)],
				   failure_output => [("Permission denied")],
				   verification => "_ReturnOutputverification",
			       },
			       ConfigDiff => {
				   command => "config diff",
				   conversion => \%Configconversion,
				   options => \%options,
                                   required_options => [qw(ConfigFile1)],
                                   optional_options => [qw(OutputFile ConfigFile2)],
				   failure_output => [("doesn't exists")],
				   verification => "_ReturnOutputverification",
			       },
			       ConfigDump => {
				   command => "config dump",
				   conversion => \%Configconversion,
				   options => \%options,
                                   required_options => [qw(ConfigFile)],
                                   optional_options => [qw(Force Volume)],
				   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Force => [qw(true false)],
                                     Volume => [qw(true false)],
                                   },
			       },
			       ConfigRestore => {
				   command => "config restore",
				   conversion => \%Configconversion,
				   options => \%options,
                                   required_options => [qw(ConfigFile)],
                                   optional_options => [qw(Volume)],
				   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Volume => [qw(true false)],
                                   },
			       },
			       );
			       
my %Compatability = (
		     );

sub SetupConfigCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
