#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "date" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Date.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "date" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#************************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_DATE}) {
    my $newModule = $ENV{OVERRIDECLASSIC_DATE};
    delete $ENV{OVERRIDECLASSIC_DATE};
    print "The main Classic::Date module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Dateconversion = (
			);
my %options = (
	       Universal => "-u",
	       Date => " ",
	       );
#************************************************************************

=head1 C<Date()>

=over 4

=item Description

This method implements the "date" command

=item Arguments

=over 4

=item Universal
(Optional)

Implements the "-u" flag, and uses UTC time zone

Supported Values: "true" or "false"

=item Date
(Optional)

The new date string

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			  Date => {
			      command => "date",
			      conversion => \%Dateconversion,
			      options => \%options,
			      optional_options => [qw(Universal Date)],
			      verification => "_ReturnOutputverification",
			      supported_values => {
				  Universal => [qw(true false)],
			      },
			  },
			  );

my %Compatability = ();

sub SetupDateCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
