#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "df" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Df.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "df" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#************************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_DF}) {
    my $newModule = $ENV{OVERRIDECLASSIC_DF};
    delete $ENV{OVERRIDECLASSIC_DF};
    print "The main Classic::Df module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Dfconversion = (
			);
my %options = (
	       inodes => "-i",
	       reserved => "-r",
	       human => "-h",
	       KB => "-k",
	       MB => "-m",
	       GB => "-g",
	       TB => "-t",
	       aggregate => "-A",
	       pathname => " ",
	       );

#************************************************************************

=head1 C<Df()>

=over 4

=item Description

This method implements the "df" command

=item Arguments

=over 4

=item inodes
(Optional)

Implements the "-i" flag, and displays info on free inodes

Mutually exclusive with reserved

Supported Values: "true" or "false"

=item reserved 
(Optional)

Implements the "-r" flag, and displays info on reserved space

Mutually exclusive with inodes 

Supported Values: "true" or "false"

=item human 
(Optional)

Implements the "-h" flag, and displays info in human readable terms 

Mutually exclusive with KB, MB, GB, and TB 

Supported Values: "true" or "false"

=item KB 
(Optional)

Implements the "-k" flag, and displays info in kilobytes 

Mutually exclusive with human, MB, GB, and TB

Supported Values: "true" or "false"

=item MB
(Optional)

Implements the "-m" flag, and displays info in megabytes

Mutually exclusive with human, KB, GB, and TB

Supported Values: "true" or "false"

=item GB
(Optional)

Implements the "-g" flag, and displays info in gigabytes

Mutually exclusive with human, KB, MB, and TB

Supported Values: "true" or "false"

=item TB
(Optional)

Implements the "-t" flag, and displays info in terabytes

Mutually exclusive with human, KB, MB, and GB

Supported Values: "true" or "false"

=item pathname
(Optional)

The full path to the specific volume to display info on. 

=back

=item Returns

None

=back

=cut

#************************************************************************


my %command_definitions = (
			       Df => {
				   command => "df",
				   conversion => \%Dfconversion,
				   options => \%options,
                                   optional_options => [qw(inodes reserved human KB MB GB TB pathname)],
				   post_command_options => [qw(aggregate)],
				   verification => "_process_df",
				   mutual_exclusions => {
                                     inodes => [qw(reserved)],
				     reserved => [qw(inodes)],
				     human => [qw(KB MB GB TB)],
				     KB => [qw(human MB GB TB)],
				     MB => [qw(human KB GB TB)],
				     GB => [qw(human KB MB TB)],
				     TB => [qw(human KB MB GB)],
                                   },
				   supported_values => {
                                     inodes => [qw(true false)],
                                     reserved => [qw(true false)],
                                     human => [qw(true false)],
                                     KB => [qw(true false)],
                                     MB => [qw(true false)],
                                     GB => [qw(true false)],
                                     TB => [qw(true false)],
                                   },
			       },
			       );
			       
my %Compatability = (
		     );

sub SetupDfCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

sub _process_df {
    my ($self, %params) = @_;
    $self->{Trace}->entry();
    my $want_array = $params{want_array};
    $want_array = [] if (!$want_array);
    my $output = $params{output_in};

    my $pathname = $params{pathname} || $params{aggregate};
    if ($pathname) {
	my %path = ();
	foreach my $line (split(/\n/, $output)) {
	    if ($line =~ /$pathname\s+(\d+)\S*\s+(\d+)\S*\s+(\d+)/) {
		$path{size} = $1;
		$path{availsize} = $3;
	    }
	}
	push(@$want_array, \%path);
    }
    $self->{Trace}->exit(msg => "Exiting with $want_array");
    return $want_array;
}


1;
