#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "disk" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Disk.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "disk" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_DISK}) {
    my $newModule = $ENV{OVERRIDECLASSIC_DISK};
    delete $ENV{OVERRIDECLASSIC_DISK};
    print "The main Classic::Disk module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Diskconversion = (
			);
my %options = (
		   DiskName => " ",
		   All => "all",
		   Number => "-n",
		   Pool => "-p",
		   OwnerName => "-o",
		   SystemID => "-s",
		   Force => "-f",
		   Immediate => "-i",
		   OldSystemName => "-o",
		   OldSystemID => "-s",
		   NewSystemName => "-n",
		   NewSystemID => "-d",
		   DiskList => " ",
		   Pattern1 => "-p",
		   Pattern2 => "-p",
		   Pattern3 => "-p", 
		   Cycles => "-c",
		   Unassigned => "-n",
		   ListAll => "-v",
		   Assigned => "-a",
		    );
#************************************************************************

=head1 C<DiskAssign()>

=over 4

=item Description

This method implements the "disk assign" command

=item Arguments

=over 4

=item DiskName
(Required)

This is the specific disk(s) to assign to the filer.

Mutually exclusive with All and Number

=item All 
(Required)

Implements the "all" keyword, and specifies that all unowned disks should be assigned to the filer

Mutually exclusive with DiskName and Number

Supported Values: "true" or "false"

=item Number 
(Required)

Implements the "-n" flag, and specifies the number of disks that should be assigned to the filer

Mutually exclusive with DiskName and All 

=item Pool
(Optional)

Implements the "-p"" flag and specifies which pool to use

Supported Values: "0" or "1"

=item OwnerName 
(Optional)

Implements the "-o" flag and specifies the ownername of a remote filer we are assigning the disks to 

=item SystemID 
(Optional)

Implements the "-s" flag and specifies the system ID of a remote filer we are assigning the disks to

=item Force 
(Optional)

Implements the "-f" flag and forces the disks assignment

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskFail()>

=over 4

=item Description

This method implements the "disk fail" command

=item Arguments

=over 4

=item DiskName
(Required)

This is the specific disk to fail.

=item Immediate 
(Optional)

Implements the "-i" flag and skips Rapid RAID Recovery 

Supported Values: "true" or "false"

=item Force
(Optional)

Implements the "-f" flag and skips the warning

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskReassign()>

=over 4

=item Description

This method implements the "disk reassign" command

=item Arguments

=over 4

=item OldSystemName 
(Required)

Implements the "-o" flag and specifies the name of the filer that currently owns the disks

Mutually Exclusive with OldSystemID

=item OldSystemID
(Required)

Implements the "-s" flag and specifies the ID of the filer that currently owns the disks

Mutually Exclusive with OldSystemName

=item NewSystemName
(Optional)

Implements the "-n" flag and specifies the name of the filer that should now own the disks

=item NewSystemID
(Optional)

Implements the "-d" flag and specifies the ID of the filer that should now own the disks

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskRemove()>

=over 4

=item Description

This method implements the "disk remove" command

=item Arguments

=over 4

=item DiskName
(Required)

The disk that should be removed

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskSwap()>

=over 4

=item Description

This method implements the "disk remove" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskSanitizeStart()>

=over 4

=item Description

This method implements the "disk sanitize start" command

=item Arguments

=over 4

=item DiskList
(Required)

The list of disks that need to be sanitized

=item Pattern1
(Optional)

Implements the "-p" flag and is the first pattern to write to the disks

=item Pattern2
(Optional)

Implements the "-p" flag and is the second pattern to write to the disks

=item Pattern3
(Optional)

Implements the "-p" flag and is the third pattern to write to the disks

=item Cycles
(Optional)

Implements the "-c" flag and is the number of cycles of pattern writes

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskSanitizeAbort()>

=over 4

=item Description

This method implements the "disk sanitize abort" command

=item Arguments

=over 4

=item DiskList
(Required)

The list of disks to stop the sanitization on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskSanitizeStatus()>

=over 4

=item Description

This method implements the "disk sanitize status" command

=item Arguments

=over 4

=item DiskList
(Optional)

The list of disks to get sanitization info on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskSanitizeRelease()>

=over 4

=item Description

This method implements the "disk sanitize release" command

=item Arguments

=over 4

=item DiskList
(Required)

The list of disks to move to the spare pool

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskScrubStart()>

=over 4

=item Description

This method implements the "disk scrub start" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskScrubStop()>

=over 4

=item Description

This method implements the "disk scrub stop" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskUnswap()>

=over 4

=item Description

This method implements the "disk unswap" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskUpgradeOwnership()>

=over 4

=item Description

This method implements the "disk upgrade ownership" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskShow()>

=over 4

=item Description

This method implements the "disk upgrade ownership" command

=item Arguments

=over 4

=item OwnerName 
(Optional)

Implements the "-o" flag and displays disks owned by specified owner name

Mutually Exclusive with SystemID Unassigned ListAll Assigned

=item SystemID 
(Optional)

Implements the "-s" flag and displays disks owned by specified system ID 

Mutually Exclusive with OwnerName Unassigned ListAll Assigned

=item Unassigned 
(Optional)

Implements the "-n" flag and displays unowned disks

Mutually Exclusive with OwnerName SystemID ListAll Assigned

Supported Values: "true" or "false"

=item ListAll 
(Optional)

Implements the "-v" flag and displays all disks

Mutually Exclusive with OwnerName SystemID Unassigned Assigned

Supported Values: "true" or "false"

=item Assigned 
(Optional)

Implements the "-a" flag and displays all assigned disks

Mutually Exclusive with OwnerName SystemID Unassigned ListAll

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<DiskZeroSpares()>

=over 4

=item Description

This method implements the "disk zero spares" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************
my %command_definitions = (
			       DiskAssign => {
				   command => "disk assign",
				   conversion => \%Diskconversion,
				   options => \%options,
                                   required_options => [qw(DiskName All Number)],
                                   optional_options => [qw(Pool OwnerName SystemID Force)],
				   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     DiskName => [qw(All Number)],
				     All => [qw(DiskName Number)],
				     Number => [qw(DiskName All)],
                                   },
				   supported_values => {
				     All => [qw(true false)],
				     Pool => [qw(0 1)],
				     Force => [qw(true false)],
				   },
				   failure_output => [("does not exist", "Could not find any unowned disks to assign", "Unrecognized pool option", "Unable to locate a system named")],
			       },
			       DiskFail => {
                                   command => "disk fail",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   required_options => [qw(DiskName)],
                                   optional_options => [qw(Immediate Force)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("not found")],
				   supported_values => {
                                     Immediate => [qw(true false)],
                                     Force => [qw(true false)],
                                   },
                               },
			       DiskReassign => {
                                   command => "disk reassign",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   required_options => [qw(OldSystemName OldSystemID)],
                                   optional_options => [qw(NewSystemName NewSystemID)],
                                   verification => "_ReturnOutputverification",
                                   mutual_exclusions => {
                                     OldSystemName => [qw(OldSystemID)],
                                     OldSystemID => [qw(OldSystemName)],
                                   },
				   failure_output => [("Did not recognize option")],
                               },
			       DiskRemove => {
                                   command => "disk remove",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   required_options => [qw(DiskName)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("not found")],
                               },
			       DiskSwap => {
                                   command => "disk swap",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
				   failure_output => [("This command not required when hot swapping on a clustered system")],
                               },
			       DiskSanitizeStart => {
                                   command => "disk sanitize start",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   required_options => [qw(DiskList)],
                                   optional_options => [qw(Pattern1 Pattern2 Pattern3 Cycles)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Use of sanitization is not licensed")],
                               },
			       DiskSanitizeAbort => {
                                   command => "disk sanitize abort",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   required_options => [qw(DiskList)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Couldn't find")],
                               },
			       DiskSanitizeStatus => {
                                   command => "disk sanitize status",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   optional_options => [qw(DiskList)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Couldn't find")],
                               },
			       DiskSanitizeRelease => {
                                   command => "disk sanitize release",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   required_options => [qw(DiskList)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Couldn't find")],
                               },
			       DiskScrubStart => {
                                   command => "disk scrub start",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       DiskScrubStop => {
                                   command => "disk scrub stop",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       DiskUnswap => {
                                   command => "disk unswap",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
				   failure_output => [("There is no swap in progress")],
                               },
			       DiskUpgradeOwnership => {
                                   command => "disk upgrade_ownership",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       DiskShow => {
                                   command => "disk show",
                                   conversion => \%Diskconversion,
                                   options => \%options,
				   optional_options => [qw(OwnerName SystemID Unassigned ListAll Assigned)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     OwnerName => [qw(SystemID Unassigned ListAll Assigned)],
                                     SystemID => [qw(OwnerName Unassigned ListAll Assigned)],
                                     Unassigned => [qw(OwnerName SystemID ListAll Assigned)],
                                     ListAll => [qw(OwnerName SystemID Unassigned Assigned)],
                                     Assigned => [qw(OwnerName SystemID Unassigned ListAll)],
                                   },
				   failure_output => [("No disks match option -n")],
				   supported_values => {
                                     Unassigned => [qw(true false)],
                                     ListAll => [qw(true false)],
                                     Assigned => [qw(true false)],
                                   },
                               },
			       DiskZeroSpares => {
                                   command => "disk zero spares",
                                   conversion => \%Diskconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupDiskCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
