#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "ems" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

EMS.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "ems" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_EMS}) {
    my $newModule = $ENV{OVERRIDECLASSIC_EMS};
    delete $ENV{OVERRIDECLASSIC_EMS};
    print "The main Classic::EMS module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

#************************************************************************

=head1 C<EMSStatus()>

=over 4

=item Description

This method implements the "ems status" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<EMSEventStatus()>

=over 4

=item Description

This method implements the "ems event status" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<EMSLogStatus()>

=over 4

=item Description

This method implements the "ems log status" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<EMSLogDump()>

=over 4

=item Description

This method implements the "ems log dump" command

=item Arguments

=over 4

=item NumberDays
(Required)

The number of days to dump ems info for. Syntax is "#d"

Mutally exclusive with NumberHours

=item NumberHours
(Required)

The number of hours to dump ems info for. Syntax is "#h"

Mutally exclusive with NumberDays

=back

=item Returns

None

=back

=cut

#************************************************************************
our %EMSconversion = (
			);
my %options = (
    time => " ",
    log => "-l",
    trap => "-t",
    version => "-v",
    severity => "-s",
    event => " ",
    arguments => " ",
    enable => " ",
    );

my %command_definitions = (
			       EMSStatus => {
                       command => "ems status",
                       conversion => \%EMSconversion,
                       options => \%options,
                       #verification => "_process_status",
                   },
			       EMSEventStatus => {
				       command => "ems event status",
				       conversion => \%EMSconversion,
				       options => \%options,
			       },
			       EMSLogStatus => {
                       command => "ems log status",
                       conversion => \%EMSconversion,
                       options => \%options,
                   },
			       EMSLogDump => {
                       command => "ems log dump",
                       conversion => \%EMSconversion,
                       options => \%options,
				       required_options => [qw(time)],
                   },
                   EMSListevents => {
                       command => "priv set diag;ems listevents",
                       converstion => \%EMSconversion,
                       options => \%options,
                   },
                   EMSLogstats => {
                       command => "priv set diag;ems logstats",
                       converstion => \%EMSconversion,
                       options => \%options,
                   },
                   EMSInvoke => {
                       command => "priv set diag;ems invoke",
                       converstion => \%EMSconversion,
                       options => \%options,
                       optional_options => [qw(log trap version severity event arguments)], 
                       supported_values => {
                           severity => [qw(NODE_FAULT SVC_FAULT NODE_ERROR SVC_ERROR WARNING NOTICE INFO DEBUG)],
                       },
                   },
                   EMSAutosuppressEnable => {
                       command => "options ems.autosuppress.enable",
                       converstion => \%EMSconversion,
                       options => \%options,
                       required_options => [qw(enable)],
                   },
                   EMSLogDupSuppressEnable => {
                       command => "priv set diag;options ems.log.dupsuppress.enable",
                       converstion => \%EMSconversion,
                       options => \%options,
                       required_options => [qw(enable)],
                   },
			       );
			       
my %Compatability = (
		     );

sub SetupEMSCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

# This sub can be used to process the output and return the data in a well formated array or hash.  See verification= parameter above.
#sub _process_status {
#    my ($self, %params) = @_;
#    my $output = $params{output_in};
#    return $output;
#}

1;
