#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "environment" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Environment.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "environment" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_ENVIRONMENT}) {
    my $newModule = $ENV{OVERRIDECLASSIC_ENVIRONMENT};
    delete $ENV{OVERRIDECLASSIC_ENVIRONMENT};
    print "The main Classic::Environment module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Environmentconversion = (
			);
my %options = (
		   Adapter => " ",
		   All => "all",
		   Voltage => "Voltage",
		   Temperature => "Temperature",
		   PowerSupplies => "'Power Supplies'",
		   Battery => "Battery",
		    );
#************************************************************************

=head1 C<EnvironmentStatus()>

=over 4

=item Description

This method implements the "environment status" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<EnvironmentStatusShelf()>

=over 4

=item Description

This method implements the "environment status shelf" command

=item Arguments

=over 4

=item Adapter
(Optional)

The specific adapter that you want to get status for

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<EnvironmentStatusChassis()>

=over 4

=item Description

This method implements the "environment status chassis" command

=item Arguments

=over 4

=item All
(Optional)

Implements the "all" keyword and displays status info for all chassis FRUS

Supported Values: "true" or "false"

Default: "false"

Mutually Exclusive with Voltage Temperature PowerSupplies Battery

=item Voltage
(Optional)

Implements the "Voltage" keyword and displays status info on the chassis voltage 

Supported Values: "true" or "false"

Default: "false"

Mutually Exclusive with All Temperature PowerSupplies Battery

=item Temperature
(Optional)

Implements the "Temperature" keyword and displays status info on the chassis temperature 

Supported Values: "true" or "false"

Default: "false"

Mutually Exclusive with All Voltage PowerSupplies Battery

=item PowerSupplies
(Optional)

Implements the "Power Supplies" keyword and displays status info on the chassis power supplies 

Supported Values: "true" or "false"

Default: "false"

Mutually Exclusive with All Voltage Temperature Battery

=item Battery 
(Optional)

Implements the "Battery" keyword and displays status info on the chassis battery units 

Supported Values: "true" or "false"

Default: "false"

Mutually Exclusive with All Voltage Temperature PowerSupplies

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       EnvironmentStatus => {
                                   command => "environment status",
                                   conversion => \%Environmentconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       EnvironmentStatusShelf => {
				   command => "environment status shelf",
				   conversion => \%Environmentconversion,
				   options => \%options,
				   optional_options => [qw(Adapter)],
				   verification => "_ReturnOutputverification",
			       },
			       EnvironmentStatusChassis => {
                                   command => "environment status chassis",
                                   conversion => \%Environmentconversion,
                                   options => \%options,
				   optional_options => [qw(All Voltage Temperature PowerSupplies Battery)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     All => [qw(Voltage Temperature PowerSupplies Battery)],
                                     Voltage => [qw(All Temperature PowerSupplies Battery)],
                                     Temperature => [qw(All Voltage PowerSupplies Battery)],
                                     PowerSupplies => [qw(All Voltage Temperature Battery)],
                                     Battery => [qw(All Voltage Temperature PowerSupplies)],
                                   },
				   supported_values => {
                                     All => [qw(true false)],
                                     Voltage => [qw(true false)],
                                     Temperature => [qw(true false)],
                                     PowerSupplies => [qw(true false)],
                                     Battery => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupEnvironmentCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
