#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "fcp" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Fcp.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "fcp" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_FCP}) {
    my $newModule = $ENV{OVERRIDECLASSIC_FCP};
    delete $ENV{OVERRIDECLASSIC_FCP};
    print "The main Classic::Fcp module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Fcpconversion = (
			);
my %options = (
		   Adapter => " ",
		   Up => "up",
		   Down => "down",
		   Partner => "partner",
		   PartnerAdapter => " ",
		   None => "None",
		   DefaultPartner => "-partner",
		   MediaType => "mediatype",
		   command => " ",
		   Nodename => " ",
		   Verbose => "-v",
		   Zero => "-z",
		   Interval => "-i",
		   All => "-a",
		   Count => "-c",
		    );
#************************************************************************

=head1 C<FcpConfig()>

=over 4

=item Description

This method implements the "fcp config" command

=item Arguments

=over 4

=item Adapter
(Optional)

This is the specifc adapter to configure

=item Up
(Optional)

Implements the "up" keyword, and sets the specified adapter online

Supported Values: "true" or "false"

=item Down
(Optional)

Implements the "down" keyword and sets the specified adapter offline

Supported Values: "true" or "false"

=item Partner
(Optional)

Implements the "partner" keyword and configures th adapter's failover partner

Supported Values: "true" or "false"

=item PartnerAdapter
(Optional)

The adapter to failover to

=item None
(Optional)

Implements the "None" keyword, and specifies that the adapter should not failover to any other adapter

Supported Values: "true" or "false"

=item DefaultPartner
(Optional)

Implements the "-partner" flag and specifies the default failover adapter

Supported Values: "true" or "false"

=item MediaType
(Optional)

Implements the "mediatype" keyword and configures the Fibre channel media type of the adapter

Supported Values: "ptp", "auto", and "loop"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpHelp()>

=over 4

=item Description

This method implements the "fcp help" command

=item Arguments

=over 4

=item command
(Required)

This is the command to get help on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpNodename()>

=over 4

=item Description

This method implements the "fcp nodename" command

=item Arguments

=over 4

=item Nodename 
(Optional)

The new fcp nodename for this filer

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpSetup()>

=over 4

=item Description

This method implements the "fcp setup" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpShowAdapter()>

=over 4

=item Description

This method implements the "fcp show adapter" command

=item Arguments

=over 4

=item Adapter
(Optional)

The specific adapter to show info on

=item Verbose
(Optional)

Implements the "-v" flag and displays additional detail

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpShowCfmode()>

=over 4

=item Description

This method implements the "fcp show cfmode" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpShowInitiator()>

=over 4

=item Description

This method implements the "fcp show initiator" command

=item Arguments

=over 4

=item Adapter
(Optional)

The specific adapter to show info on

=item Verbose
(Optional)

Implements the "-v" flag and displays additional detail

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpStats()>

=over 4

=item Description

This method implements the "fcp stats" command

=item Arguments

=over 4

=item Adapter
(Optional)

The specific adapter to show stats on

Mutually Exclusive with "All"

=item All
(Optional)

Implements the "-a" flag and displays stats for all adapters 

Supported Values: "true" or "false"

Mutually Exclusive with "Adapter"

=item Zero
(Optional)

Implements the "-z" flag and zeros the stats for the adapter(s)

Supported Values: "true" or "false"

=item Interval
(Optional)

Implements the "-i" flag and displays stats over the specified interval (seconds)

=item Count
(Optional)

Implements the "-c" flag and specifies the number of intervals to display stats for

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpStart()>

=over 4

=item Description

This method implements the "fcp start" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpStatus()>

=over 4

=item Description

This method implements the "fcp status" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FcpStop()>

=over 4

=item Description

This method implements the "fcp stop" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************


my %command_definitions = (
			       FcpConfig => {
                                   command => "fcp config",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
				   optional_options => [qw(Adapter Up Down Partner PartnerAdapter None DefaultPartner MediaType)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     Up => [qw(Down)],
				     Down => [qw(Up)],
				     PartnerAdapter => [qw(None DefaultPartner)],
				     None => [qw(PartnerAdapter DefaultPartner)],
				     DefaultPartner => [qw(PartnerAdapter None)],
				     Partner => [qw(DefaultPartner)],
				     DefaultPartner => [qw(Partner)],
                                   },
				   dependencies => {
				     Up => {
					Adapter => "presence",
				     },
				     Down => {
					Adapter => "presence",
				     },
				     PartnerAdapter => {
					Partner => "presence",
				     },
				     None => {
					Partner => "presence",
				     },
				   },
				   supported_values => {
                                     MediaType => [qw(ptp auto loop)],
                                     Up => [qw(true false)],
                                     Down => [qw(true false)],
                                     None => [qw(true false)],
                                     Partner => [qw(true false)],
                                     DefaultPartner => [qw(true false)],
                                   },

                               },
			       FcpHelp => {
                                   command => "fcp help",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   required_options => [qw(command)],
                                   verification => "_ReturnOutputverification",
                               },
			       FcpNodename => {
                                   command => "fcp nodename",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   optional_options => [qw(Nodename)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Invalid nodename")],
                               },
			       FcpSetup => {
                                   command => "fcp setup",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       FcpShowAdapter => {
                                   command => "fcp show adapter",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   optional_options => [qw(Adapter Verbose)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("not found")],
				   supported_values => {
                                     Verbose => [qw(true false)],
                                   },

                               },
			       FcpShowCfmode => {
                                   command => "fcp show cfmode",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },	
			       FcpShowInitiator => {
                                   command => "fcp show initiator",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   optional_options => [qw(Adapter Verbose)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("not found")],
				   supported_values => {
                                     Verbose => [qw(true false)],
                                   },
                               },
			       FcpStats => {
                                   command => "fcp stats",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   optional_options => [qw(Adapter Zero Interval All Count)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     Adapter => [qw(All)],
                                     All => [qw(Adapter Zero)],
                                     Zero => [qw(Interval All Count)],
                                     Interval => [qw(Zero)],
                                     Count => [qw(Zero)],
                                   },
				   failure_output => [("not found")],
				   supported_values => {
                                     All => [qw(true false)],
                                     Zero => [qw(true false)],
                                   },
                               },
			       FcpStart => {
                                   command => "fcp start",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
				   successful_output => [("FCP service is running")],
                               },
                               FcpStatus => {
                                   command => "fcp status",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
				   successful_output => [("FCP service is running")],
                               },
                               FcpStop => {
                                   command => "fcp stop",
                                   conversion => \%Fcpconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
				   successful_output => [("FCP service stopped")],
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupFcpCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
