#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "fcstat" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Fcstat.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "fcstat" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_FCSTAT}) {
    my $newModule = $ENV{OVERRIDECLASSIC_FCSTAT};
    delete $ENV{OVERRIDECLASSIC_FCSTAT};
    print "The main Classic::Fcstat module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Fcstatconversion = (
			);
my %options = (
		   Channel => " ",
		   LinkStats => "link_stats",
		   FcalStats => "fcal_stats",
		   DeviceMap => "device_map",
		    );
#************************************************************************

=head1 C<FcpHelp()>

=over 4

=item Description

This method implements the "fcstat" command

=item Arguments

=over 4

=item LinkStats
(Required)

Implements the "link_stats" command

Supported Values: "true" or "false"

Mutually Exclusive with "FcalStats" and "DeviceMap"

=item FcalStats
(Required)

Implements the "fcal_stats" command

Supported Values: "true" or "false"

Mutually Exclusive with "LinkStats" and "DeviceMap"

=item DeviceMap 
(Required)

Implements the "device_map" command

Supported Values: "true" or "false"

Mutually Exclusive with "LinkStats" and "FcalStats"

=item Channel
(Optional)

The specific channel to list fibre channel stats on

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       Fcstat => {
                                   command => "fcstat",
                                   conversion => \%Fcstatconversion,
                                   options => \%options,
				   required_options => [qw(LinkStats FcalStats DeviceMap)],
				   post_command_options => [qw(Channel)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     LinkStats => [qw(FcalStats DeviceMap)],
                                     FcalStats => [qw(LinkStats DeviceMap)],
                                     DeviceMap => [qw(LinkStats FcalStats)],
				   },
				   supported_values => {
                                     LinkStats => [qw(true false)],
                                     FcalStats => [qw(true false)],
                                     DeviceMap => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupFcstatCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
