#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "file" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

File.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "file" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_FILE}) {
    my $newModule = $ENV{OVERRIDECLASSIC_FILE};
    delete $ENV{OVERRIDECLASSIC_FILE};
    print "The main Classic::File module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Fileconversion = (
			);
my %options = (
		   File => " ",
		   Enable => "enable",
		   Disable => "disable",
		    );
#************************************************************************

=head1 C<FileReservation()>

=over 4

=item Description

This method implements the "file reservation" command

=item Arguments

=over 4

=item File
(Required)

This is the file to set options on

=item Enable
(Optional)

Implements the "enable" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "Disable"

=item Disable
(Optional)

Implements the "disable" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "Enable"

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       FileReservation => {
                                   command => "file reservation",
                                   conversion => \%Fileconversion,
                                   options => \%options,
				   required_options => [qw(File)],
				   post_command_options => [qw(Enable Disable)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     Enable => [qw(Disable)],
                                     Disable => [qw(Enable)],
				   },
				   successful_output => [("space reservations for file", "fill reservation disabled", "on")],
				   supported_values => {
                                     Enable => [qw(true false)],
                                     Disable => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupFileCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
