#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "filestats" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Filestats.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "filestats" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_FILESTATS}) {
    my $newModule = $ENV{OVERRIDECLASSIC_FILESTATS};
    delete $ENV{OVERRIDECLASSIC_FILESTATS};
    print "The main Classic::Filestats module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Filestatsconversion = (
			);
my %options = (
		   PerGroup => "-g",
		   PerUser => "-u",
		   Ages => "ages",
		   Expression => "expr",
		   TimeType => "timetype",
		   Sizes => "sizes",
		   Style => "style",
		   Volume => "volume",
		   Snapshot => "snapshot",
		    );
#************************************************************************

=head1 C<Filestats()>

=over 4

=item Description

This method implements the "filestats" command

=item Arguments

=over 4

=item Snapshot
(Required)

Implements the "snapshot" keyword, and is the snapshot name to get file stats on

=item PerGroup
(Optional)

Implements the "-g" flag, and breakdown the stats on a per-group basis

Supported Values: "true" and "false"

=item PerUser
(Optional)

Implements the "-u" flag, and breakdown the stats on a per-user basis

Supported Values: "true" and "false"

=item Ages
(Optional)

Implements the "ages" keyword, and breakdown the stats by age. Values
are in seconds or you can use #D for days and #H for hours

=item Expression
(Optional)

Implements the "expr" keyword and breaksdown the stats based on boolean expression specified

=item TimeType
(Optional)

Implements the "timetype" keyword and specifies the time used in the "age" comparision

Supported Values: "a" for access time
		  "m" for modification time
		  "c" for change time
		  "cr" for creation time

=item Sizes
(Optional)

Implements the "sizes" keyword and sepcifies the breakdown in sizes. Values are in bytes
but #K, #M, and #G can be specified to list in KB, MB, or GB, respectively. Use "*" for
all unique file sizes

=item Style
(Optional)

Implements the "style" keyword and controls the output style

Supported Values: "readable" (default)
		  "table" (colon separated values)
		  "html" (webpage)

=item Volume
(Optional)

Implements the "volume" keyword and specifies the volume to look for the snapshot on.
Default is "vol0"

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       Filestats => {
                                   command => "filestats",
                                   conversion => \%Filestatsconversion,
                                   options => \%options,
				   required_options => [qw(Snapshot)],
				   optional_options => [qw(PerGroup PerUser Ages Expression TimeType Sizes)],
				   post_command_options => [qw(Style Volume)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     TimeType => [qw(a m c cr)],
                                     Style => [qw(readable table html)],
                                     PerGroup => [qw(true false)],
                                     PerUser => [qw(true false)],
                                   },
				   failure_output => [("No such snapshot", "No such volume")],
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupFilestatsCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
