#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement the "flexvol" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Vol.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "flexvol" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_FLEXCACHEVOL}) {
    my $newModule = $ENV{OVERRIDECLASSIC_FLEXCACHEVOL};
    delete $ENV{OVERRIDECLASSIC_FLEXCACHEVOL};
    print "The main Classic::FlexCacheVol module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

use strict;
use Data::Dumper;
# only require so that the current namespace is not modified.
use nfs::GenericExport qw(export_path);

use TracePkg;
our $Trace;
our %Volconversion = (
			);
my %options = (
	       volume => " ",
	       aggregate => " ",
	       size => " ",
	       source => "-S",
	       verbose => "-v",
	       );
#************************************************************************

#************************************************************************

my %command_definitions = (
		      flexcache_vol_create => {
			  command => "vol create",
			  conversion => \%Volconversion,
			  options => \%options,
			  required_options => [qw(volume aggregate size source)],
			  optional_options => [],
			  defaults => {
			  },
			  SuccessfulOutput => ["has completed", "already exists"],
			  verification => "flexcache_vol_status",
			  supported_values => {
                          },
			  mutual_exclusions => {
			  },
			  dependencies => {
			  },
		      },
		      flexcache_vol_status => {
			  command => "vol status",
			  conversion => \%Volconversion,
			  options => \%options,
			  optional_options => [qw(volume verbose)],
			  defaults => {
			      verbose => "true",
			  },
			  verification => "_process_flexcache_vol_status",
			  FailureOutput => [("No volume named")],
		      },
		      );

my %Compatability = (
		     );

sub SetupFlexCacheVolCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

sub flexcache_vol_create {
    my ($self, %params) = @_;
    $self->{Trace}->entry(msg=>join(" ", %params));
    $self->run_command(%params, AUTOLOAD_COMMAND=>"Node::Classic::flexcache_vol_create");
    
    if ($params{ruleset}) {
	my $export_options = $self->{options_object}->get(option=>"EXPORT_OPTIONS");
	my %export = $self->protocol_exportruleset_rule_show(rulesetname=>$params{ruleset}, vserver=>$params{vserver});
	$export_options .= ",anon=$export{anon}";
	# could support rwrule
	# could support rorule
	# could support clientmatch

	# Removing trailing or leading commas, if it's included
	$export_options =~ s/^,//g;
	$export_options =~ s/,$//g;

	my $volumes = $self->vol_status(%params);
	export_path($self->get_connection(type=>"cli_connection"), 
		    $volumes->[0]->{'junction-path'},
		    OPTIONS=>$export_options,
		    PERSISTENT_OPT=>1);
	if ($@) {
	    if ($@ =~ /waiting for echo back/) {
		$self->{Trace}->warn(msg=>"Could not export $params{volume}: $@");
		sleep 10;
            } elsif ($@ =~ /Could not create persistent entry/) {
                $self->{Trace}->warn(msg=>$@);
	    } else {
		$self->{Trace}->fatal(msg=>"Could not export $params{volume}: $@");
	    }
	}
    }
    $self->{Trace}->exit();
}

sub _process_flexcache_vol_status {
    my ($self, %params) = @_;
    $self->{Trace}->entry();
    my $want_array = $params{want_array};
    $want_array = [] if (!$want_array);
    my $output = $params{output_in};

    my $more_options = undef;
    if ($params{volume}) {
	my %vol = ();
	my $options;
	foreach my $line (split(/\n/, $output)) {
	    next if ($line =~ /\s+Volume State/);
	    if ($line =~ /^\s*\S*$params{volume}\s+(.+)/) {
		$vol{volume} = $params{volume};
		($vol{state}, $vol{status}, $options, $vol{source}) = split(/\s{2,}/, $1);
	    } elsif ($line =~ /\s+Containing aggregate: '(\S+)',*/) {
		$vol{aggregate} = $1;
		last;
	    } elsif ($line =~ /^\s+(\S.+)\s{2,}(\S.+)/) {
		$vol{status} .= ",$1";
		$options .= $2;
	    } elsif ($line =~ /^\s+(.+)/) {
		$options .= $1;
	    }
	}
	$vol{'junction-path'} = "/vol/$params{volume}";
	$vol{status} =~ s/\s//g;
	$options =~ s/\s//g;
	foreach my $option (split(/,/, $options)) {
	    $vol{$1} = $2 if ($option =~ /(.+)=(.+)/);
	}
	push(@$want_array, \%vol);
    } else {
	foreach my $line (split(/\n/, $output)) {
	    next if ($line =~ /\s+Volume State/);
	    next if ($line =~ /^vol status/);
	    if ($line =~ /\s*(\S+)\s+\S+/) {
		my $volume = $self->vol_status(volume=>$1);
		push(@$want_array, @$volume);
	    }
	}
    }
    $self->{Trace}->exit(msg => "Exiting with $want_array");
    return $want_array;
}


1;
