#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "fpolicy" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Fpolicy.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "fpolicy" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_FPOLICY}) {
    my $newModule = $ENV{OVERRIDECLASSIC_FPOLICY};
    delete $ENV{OVERRIDECLASSIC_FPOLICY};
    print "The main Classic::Fpolicy module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Fpolicyconversion = (
			);

my %options = (
		   command => " ",
		   PolicyName => " ",
		   PolicyType => " ",
		   Force => "-f",
		   Include => "include",
		   Exclude => "exclude",
		   ExtensionList => " ",
		   Required => "required",
		   On => "on",
		   Off => "off",
		   SecondaryServers => "secondary_servers",
		   ServerList => " ",
		   Show => "show",
		   Stop => "stop",
		   ScannerIPAddress => " ",
		    );
#************************************************************************

=head1 C<Fpolicy()>

=over 4

=item Description

This method implements the "fpolicy" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyHelp()>

=over 4

=item Description

This method implements the "fpolicy help" command

=item Arguments

=over 4

=item command
(Optional)

The command to get additional help on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyCreate()>

=over 4

=item Description

This method implements the "fpolicy create" command

=item Arguments

=over 4

=item PolicyName
(Required)

The name of the new file policy

=item PolicyType
(Required)

The file policy type

Supported Values: "screen" for file screening

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyDestroy()>

=over 4

=item Description

This method implements the "fpolicy destroy" command

=item Arguments

=over 4

=item PolicyName
(Required)

The name of the file policy to destroy

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyDisable()>

=over 4

=item Description

This method implements the "fpolicy disable" command

=item Arguments

=over 4

=item PolicyName
(Required)

The name of the file policy to disable

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyEnable()>

=over 4

=item Description

This method implements the "fpolicy enable" command

=item Arguments

=over 4

=item PolicyName
(Required)

The name of the file policy to enable

=item Force
(Optional)

Implements the "-f" flag and forces the file policy to enabled even if there are no servers
to enable it on.

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyExtensions()>

=over 4

=item Description

This method implements the "fpolicy extensions" command

=item Arguments

=over 4

=item PolicyName
(Required)

The name of the file policy

=item Include
(Required)

Implements the "include" keyword

Supported values: "true" or "false"

Mutually Exclusive with "Exclude"

=item Exclude
(Required)

Implements the "exclude" keyword

Supported values: "true" or "false"

Mutually Exclusive with "Include"

=item Reset
(Optional)

Implements the "reset" keyword

Supported Values: "true" or "false"

=item Set
(Optional)

Implements the "set" keyword

Supported Values: "true" or "false"

=item Add
(Optional)

Implements the "add" keyword

Supported Values: "true" or "false"

=item Remove
(Optional)

Implements the "remove" keyword

Supported Values: "true" or "false"

=item ExtensionList
(Optional)

The list of extensions

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<Fpolicyoptions()>

=over 4

=item Description

This method implements the "fpolicy options" command

=item Arguments

=over 4

=item PolicyName
(Required)

The name of the file policy to set options for

=item Required 
(Optional)

Implements the "required" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "SecondaryServers"

=item On 
(Optional)

Implements the "on" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "Off"

=item Off
(Optional)

Implements the "off" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "On"

=item SecondaryServers
(Optional)

Implements the "secondary_servers" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "Required"

=item ServerList
(Optional)

The list of servers

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyServers()>

=over 4

=item Description

This method implements the "fpolicy servers" command

=item Arguments

=over 4

=item PolicyName
(Optional)

The name of the file policy to display info for

=item Show
(Optional)

Implements the "show" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "Stop"

=item Stop 
(Optional)

Implements the "stop" keyword

Supported Values: "true" or "false"

Mutually Exclusive with "Show"

=item ScannerIPAddress
(Optional)

The list of IP addresses to stop

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<FpolicyShow()>

=over 4

=item Description

This method implements the "fpolicy show" command

=item Arguments

=over 4

=item PolicyName
(Required)

The name of the file policy to display info for

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       Fpolicy => {
                                   command => "fpolicy",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       FpolicyHelp => {
                                   command => "fpolicy help",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
				   optional_options => [qw(command)],
                                   verification => "_ReturnOutputverification",
                               },
			       FpolicyCreate => {
                                   command => "fpolicy create",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   required_options => [qw(PolicyName PolicyType)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Can't access file policies unless CIFS is running")],
				   supported_values => {
                                     PolicyType => [qw(screen)],
                                   },
                               },
			       FpolicyDestroy => {
                                   command => "fpolicy destroy",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   required_options => [qw(PolicyName)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Can't access file policies unless CIFS is running")],
                               },
			       FpolicyDisable => {
                                   command => "fpolicy disable",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   required_options => [qw(PolicyName)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Can't access file policies unless CIFS is running")],
                               },
			       FpolicyEnable => {
                                   command => "fpolicy enable",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   required_options => [qw(PolicyName)],
				   optional_options => [qw(Force)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Can't access file policies unless CIFS is running")],
				   supported_values => {
                                     Force => [qw(true false)],
                                   },
                               },
			       FpolicyExtensions => {
                                   command => "fpolicy extensions",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   required_options => [qw(PolicyName Include Exclude)],
                                   optional_options => [qw(Reset Set Add Remove)],
				   post_command_options => [qw(ExtensionList)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     Include => [qw(Exclude)],
                                     Exclude => [qw(Include)],
				     Reset => [qw(Set Add Remove)],
				     Set => [qw(Reset Add Remove)],
				     Add => [qw(Reset Set Remove)],
				     Remove => [qw(Reset Set Add)],
                                   },
				   dependencies => {
                                     Set => {
                                        ExtensionList => "presence",
                                     },
                                     Add => {
                                        ExtensionList => "presence",
                                     },
                                     Remove => {
                                        ExtensionList => "presence",
                                     },
                                   },
				   failure_output => [("Can't access file policies unless CIFS is running")],
				   supported_values => {
                                     Include => [qw(true false)],
                                     Exclude => [qw(true false)],
                                     Reset => [qw(true false)],
                                     Set => [qw(true false)],
                                     Add => [qw(true false)],
                                     Remove => [qw(true false)],
                                   },
                               },
			       Fpolicyoptions => {
                                   command => "fpolicy options",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   required_options => [qw(PolicyName)],
                                   post_command_options => [qw(Required On Off SecondaryServers ServerList)],
                                   verification => "_ReturnOutputverification",
                                   mutual_exclusions => {
                                     On => [qw(Off)],
                                     Off => [qw(On)],
				     Required => [qw(SecondaryServers)],
				     SecondaryServers => [qw(Required)],
                                   },
				   supported_values => {
                                     On => [qw(true false)],
                                     Off => [qw(true false)],
                                     Required => [qw(true false)],
                                     SecondaryServers => [qw(true false)],
                                   },
                                   dependencies => {
				     On => {
                                        Required => "presence",
                                     },
				     Off => {
                                        Required => "presence",
                                     },
				     ServerList => {
                                        SecondaryServers => "presence",
                                     },
                                   },
				   failure_output => [("Can't access file policies unless CIFS is running")],
                               },
			       FpolicyServers => {
                                   command => "fpolicy servers",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
				   optional_options => [qw(PolicyName Show Stop)],
                                   post_command_options => [qw(ScannerIPAddress)],
                                   verification => "_ReturnOutputverification",
                                   mutual_exclusions => {
                                     Show => [qw(Stop)],
                                     Stop => [qw(Show)],
                                   },
				   supported_values => {
                                     Show => [qw(true false)],
                                     Stop => [qw(true false)],
                                   },
                                   dependencies => {
				     ScannerIPAddress => {
                                        Stop => "presence",
                                     },
				     Show => {
                                        PolicyName => "presence",
                                     },
				     Stop => {
                                        PolicyName => "presence",
                                     },
                                   },
				   failure_output => [("Can't access file policies unless CIFS is running")],
                               },
			       FpolicyShow => {
                                   command => "fpolicy show",
                                   conversion => \%Fpolicyconversion,
                                   options => \%options,
                                   required_options => [qw(PolicyName)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("Can't access file policies unless CIFS is running")],
                               },
			       );
			       
my $Compatability = (
		     );

sub SetupFpolicyCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
