#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "halt" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Halt.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "halt" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_HALT}) {
    my $newModule = $ENV{OVERRIDECLASSIC_HALT};
    delete $ENV{OVERRIDECLASSIC_HALT};
    print "The main Classic::Halt module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Haltconversion = (
			);
my %options = (
		   HaltString => "-d",
		   Time => "-t",
		   Force => "-f",
		    );
#************************************************************************

=head1 C<Halt()>

=over 4

=item Description

This method implements the "halt" command

=item Arguments

=over 4

=item HaltString
(Optional)

Implements the "-d" flag and indicates the reason why we are halting

=item Time
(Optional)

Implements the "-t" flag and indicates the time (in minutes) to wait before halting

=item Force
(Optional)

Implements the "-f" flag and prevents the partner from taking over

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       Halt => {
                                   command => "halt",
                                   conversion => \%Haltconversion,
                                   options => \%options,
				   optional_options => [qw(HaltString Time Force)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Force => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
			 );

sub SetupHaltCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
