#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "httpstat" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Httpstat.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "httpstat" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_HTTPSTAT}) {
    my $newModule = $ENV{OVERRIDECLASSIC_HTTPSTAT};
    delete $ENV{OVERRIDECLASSIC_HTTPSTAT};
    print "The main Classic::Httpstat module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Httpstatconversion = (
			);

my %options = (
		   HideHeader => "-h",
		   Zero => "-z",
		   Details => "-d",
		   Errors => "-e",
		   Requests => "-r",
		   Services => "-s",
		   Timeouts => "-t",
		   All => "-a",
		   Count => "-c",
		   Interval => " ",
		    );
#************************************************************************

=head1 C<Httpstat()>

=over 4

=item Description

This method implements the "httpstat" command

=item Arguments

=over 4

=item HideHeader
(Optional)

Implements the "-h" flag and suppresses display of header info

Supported Values: "true" or "false"

=item Zero
(Optional)

Implements the "-z" flag and zeros all statistics

Mutually Exclusive with "Count", "Interval", "HideHeader", "Services"

Supported Values: "true" or "false"

=item Details
(Optional)

Implements the "-d" flag and selects detailed info to be displayed

Supported Values: "true" or "false"

=item Errors
(Optional)

Implements the "-e" flag and selects errors info to be displayed

Supported Values: "true" or "false"

=item Requests
(Optional)

Implements the "-r" flag and selects requests info to be displayed

Supported Values: "true" or "false"

=item Services
(Optional)

Implements the "-s" flag and selects services info to be displayed

Supported Values: "true" or "false"

=item Timeouts
(Optional)

Implements the "-t" flag and selects timeouts info to be displayed

Supported Values: "true" or "false"

=item All
(Optional)

Implements the "-a" flag and selects all info to be displayed

Supported Values: "true" or "false"

=item Count
(Optional)

Implements the "-c" flag and is number of intervals to display stats for

=item Interval
(Optional)

The interval (in seconds) between stat collection

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       Httpstat => {
                                   command => "httpstat",
                                   conversion => \%Httpstatconversion,
                                   options => \%options,
				   optional_options => [qw(HideHeader Zero Details Errors Requests Services Timeouts All Count Interval)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     Zero => [qw(Count Interval HideHeader Services)],
                                     Count => [qw(Zero)],
                                     Interval => [qw(Zero)],
                                   },
				   supported_values => {
                                     HideHeader => [qw(true false)],
                                     Zero => [qw(true false)],
                                     Details => [qw(true false)],
                                     Errors => [qw(true false)],
                                     Requests => [qw(true false)],
                                     Services => [qw(true false)],
                                     Timeouts => [qw(true false)],
                                     All => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
			 );

sub SetupHttpstatCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
