#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "igroup" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Igroup.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "igroup" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_IGROUP}) {
    my $newModule = $ENV{OVERRIDECLASSIC_IGROUP};
    delete $ENV{OVERRIDECLASSIC_IGROUP};
    print "The main Classic::Igroup module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Igroupconversion = (
			);
my %options = (
		   IgroupName => " ",
		   WWN => " ",
		   FCP => "-f",
		   ISCSI => "-i",
		   OStype => "-t",
		   Force => "-f",
		   command => " ",
		   Attribute => " ",
		   Value => " ",
		   Verbose => "-v",
		   Throttles => "-t",
		   Interval => "-i",
		   Count => "-c",
		   All => "-a",
		    );
#************************************************************************

=head1 C<IgroupAdd()>

=over 4

=item Description

This method implements the "igroup add" command

=item Arguments

=over 4

=item IgroupName
(Required)

The name of the existing igroup to add the new WWN to

=item WWN
(Required)

The WWN to add to the igroup

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IgroupCreate()>

=over 4

=item Description

This method implements the "igroup create" command

=item Arguments

=over 4

=item IgroupName
(Required)

The name of the new igroup

=item FCP
(Required)

Implements the "-f" flag and designates the igroup to accept WWN in FCP format

Supported Values: "true" or "false"

Mutually Exclusive with "ISCSI"

=item ISCSI
(Required)

Implements the "-i" flag and designates the igroup to accept WWN in ISCSI format

Supported Values: "true" or "false"

Mutually Exclusive with "FCP"

=item OStype
(Optional)

Implements the "-t" flag and specifies the operating system type for this igroup

Supported Values: "default", "solaris", "windows", "hpux", "aix", "linux"

=item WWN
(Optional)

The first WWN to assign to the newly created igroup

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IgroupDestroy()>

=over 4

=item Description

This method implements the "igroup destroy" command

=item Arguments

=over 4

=item IgroupName
(Required)

The name of the igroup to destroy

=item Force
(Optional)

Implements the "-f" flag and destroys the igroup even if there are existing lun maps

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IgroupHelp()>

=over 4

=item Description

This method implements the "igroup help" command

=item Arguments

=over 4

=item command
(Required)

The command we need to get help on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IgroupRemove()>

=over 4

=item Description

This method implements the "igroup remove" command

=item Arguments

=over 4

=item IgroupName
(Required)

The name of the igroup that the WWN is a part of

=item WWN
(Required)

The WWN to remove

=item Force
(Optional)

Implements the "-f" flag and removes the WWN from the igroup even if there are existing lun maps

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IgroupSet()>

=over 4

=item Description

This method implements the "igroup set" command

=item Arguments

=over 4

=item IgroupName
(Required)

The igroup whose parameters we want to modify

=item Attribute
(Required)

The attribute whose setting we wish to modify

Supported Values: "ostype", "throttle_reserve", "throttle_borrow"

=item Value
(Required)

The value to set Attribute to

Supported Values: ostype: "default", "solaris", "windows", "hpux", "aix", "linux"
		  throttle_reserve: 0-99
		  throttle_borrow: "yes"

=item Force
(Optional)

Implements the "-f" flag and overrides all warnings

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IgroupShow()>

=over 4

=item Description

This method implements the "igroup show" command

=item Arguments

=over 4

=item IgroupName
(Optional)

The specific igroup whose info we want to display

=item Verbose
(Optional)

Implements the "-v" flag and gets a verbose listing

Supported Values: "true" or "false"

=item Throttles
(Optional)

Implements the "-t" flag and displays throttles

Supported Values: "true" or "false"

=item Interval
(Optional)

Implements the "-i" flag and specified the interval over which statistics are displayed.
Not sure what units are.

=item Count
(Optional)

Implements the "-c" flag and specifies the number of intervald to display stats for

=item All
(Optional)

Implements the "-a" flag and displays all throttles

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       IgroupAdd => {
                                   command => "igroup add",
                                   conversion => \%Igroupconversion,
                                   options => \%options,
				   required_options => [qw(IgroupName WWN)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("No such initiator group exists", "Not a valid world-wide name")],
                               },
			       IgroupCreate => {
                                   command => "igroup create",
                                   conversion => \%Igroupconversion,
                                   options => \%options,
                                   required_options => [qw(FCP ISCSI IgroupName)],
                                   optional_options => [qw(OStype)],
                                   post_command_options => [qw(WWN)],
                                   verification => "_ReturnOutputverification",
				   mutual_exclusions => {
                                     FCP => [qw(ISCSI)],
                                     ISCSI => [qw(FCP)],
                                   },
                                   supported_values => {
                                     OSType => [qw(default solaris windows hpux aix linux)],
                                     FCP => [qw(true false)],
                                     ISCSI => [qw(true false)],
                                   },
				   failure_output => [("Invalid options: -f, and -i are mutually exclusive", "Initiator group already exists", "Not a valid world-wide name", "Illegal iSCSI node name")],
                               },
			       IgroupDestroy => {
                                   command => "igroup destroy",
                                   conversion => \%Igroupconversion,
                                   options => \%options,
                                   required_options => [qw(IgroupName)],
                                   optional_options => [qw(Force)],
                                   verification => "_ReturnOutputverification",
				   failure_output => [("No such initiator group exists")],
				   supported_values => {
                                     Force => [qw(true false)],
                                   },
			       },
			       IgroupHelp => {
                                   command => "igroup help",
                                   conversion => \%Igroupconversion,
                                   options => \%options,
                                   required_options => [qw(command)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     command => [qw(add create destroy help remove set show)],
                                   },
                               },
			       IgroupRemove => {
                                   command => "igroup remove",
                                   conversion => \%Igroupconversion,
                                   options => \%options,
                                   required_options => [qw(IgroupName WWN)],
                                   optional_options => [qw(Force)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Force => [qw(true false)],
                                   },
                               },
			       IgroupSet => {
                                   command => "igroup set",
                                   conversion => \%Igroupconversion,
                                   options => \%options,
                                   required_options => [qw(IgroupName Attribute Value)],
                                   optional_options => [qw(Force)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Attribute => [qw(ostype throttle_reserve throttle_borrow)],
                                     Force => [qw(true false)],
				     Value => [qw(default solaris windows hpux aix linux 0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 yes)],
                                   },
				   failure_output => [("No such initiator group exists")],
                               },
			       IgroupShow => {
                                   command => "igroup show",
                                   conversion => \%Igroupconversion,
                                   options => \%options,
                                   optional_options => [qw(Verbose IgroupName Throttles Interval Count All)],
                                   verification => "_ReturnOutputverification",
				   dependencies => {
				     Interval => {
                                        Throttles => "presence",
                                     },
				     Count => {
                                        Interval => "presence",
                                     },
				     All => {
                                        Interval => "presence",
                                     },
                                   },
				   failure_output => [("No such initiator group exists")],
				   supported_values => {
                                     Verbose => [qw(true false)],
                                     Throttles => [qw(true false)],
                                     All => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupIgroupCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
