#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "ipsec" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Ipsec.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "ipsec" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_IPSEC}) {
    my $newModule = $ENV{OVERRIDECLASSIC_IPSEC};
    delete $ENV{OVERRIDECLASSIC_IPSEC};
    print "The main Classic::Ipsec module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Ipsecconversion = (
			);
my %options = (
		   SourceIP => "-s",
		   DestinationIP => "-t",
		   Protocol => "-p",
		   EncryptionAlgorithm => "-e",
		   AuthenticationAlgorithm => "-a",
		   Direction => "-d",
		   DisableMirror => "-m",
		   UpperLayerProtocol => "-f",	
		   Layer => "-l",
		   All => "all",
		   Index => "-i",
		   SPI => "spi",
		   Verbose => "-v",
		   Zero => "-z",
		    );
#************************************************************************

=head1 C<IpsecPolicyAdd()>

=over 4

=item Description

This method implements the "ipsec policy add" command

=item Arguments

=over 4

=item Protocol
(Required)

Implements the "-p" flag 

Supported Values: "esp", "ah", "none"

=item Direction
(Required)

Implements the "-d" flag

Supported Values: "in", "out"

=item SourceIP
(Optional)

Implements the "-s" flag

=item DestinationIP
(Optional)

Implements the "-t" flag

=item EncryptionAlgorithm
(Optional)

Implements the "-e" flag

Supported Values: "3des", "des", "null"

Mutually Exclusive with "AuthenticationAlgorithm"

=item AuthenticationAlgorithm
(Optional)

Implements the "-a" flag

Supported Values: "sha1", "md5", "null"

Mutually Exclusive with "EncryptionAlgorithm"

=item DisableMirror
(Optional)

Implements the "-m" flag

Supported Values: "true" or "false"

=item UpperLayerProtocol
(Optional)

Implements the "-f" flag

=item Layer
(Optional)

Implements the "-l" flag

Supported Values: "restrict", "permit"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpsecPolicyDelete()>

=over 4

=item Description

This method implements the "ipsec policy delete" command

=item Arguments

=over 4

=item All
(Required)

Implements the "all" keyword and deletes all policies

Supported Values: "true" or "false"

Mutually Exclusive with "Index"

=item Index 
(Required)

Implements the "-i" flag and specifies the specific policy to delete

Mutually Exclusive with "All"

=item SourceIP
(Optional)

Implements the "-s" flag

=item DestinationIP
(Optional)

Implements the "-t" flag

=item Direction
(Optional)

Implements the "-d" flag

Supported Values: "in", "out"

=item DisableMirror
(Optional)

Implements the "-m" flag

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpsecPolicyShow()>

=over 4

=item Description

This method implements the "ipsec policy show" command

=item Arguments

=over 4

=item SourceIP
(Optional)

Implements the "-s" flag

=item DestinationIP
(Optional)

Implements the "-t" flag

=item UpperLayerProtocol
(Optional)

Implements the "-f" flag

=item Direction
(Optional)

Implements the "-d" flag

Supported Values: "in", "out"

=item Protocol
(Optional)

Implements the "-p" flag

Supported Values: "esp", "ah"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpsecSaShow()>

=over 4

=item Description

This method implements the "ipsec sa show" command

=item Arguments

=over 4

=item SPI
(Optional)

Implements the "spi" keyword

Supported Values: "true", "false"

Mutually exclusive with "SourceIP", "DestinationIP", "Protocol"

=item SourceIP
(Optional)

Implements the "-s" flag

=item DestinationIP
(Optional)

Implements the "-t" flag

=item Protocol
(Optional)

Implements the "-p" flag

Supported Values: "esp", "ah"

=item Verbose
(Optional)

Implements the "-v" flag

Supported Values: "true", "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpsecStat()>

=over 4

=item Description

This method implements the "ipsec stat" command

=item Arguments

=over 4

=item Zero
(Optional)

Implements the "-z" flag and zeros all stats

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       IpsecPolicyAdd => {
                                   command => "ipsec policy add",
                                   conversion => \%Ipsecconversion,
                                   options => \%options,
				   required_options => [qw(Protocol Direction)],
				   optional_options => [qw(SourceIP DestinationIP EncryptionAlgorithm AuthenticationAlgorithm DisableMirror UpperLayerProtocol Layer)],
                                   verification => "_ReturnOutputverification",
                                   supported_values => {
                                     Protocol => [qw(esp ah none)],
				     EncryptionAlgorithm => [qw(3des des null)],
				     AuthenticationAlgorithm => [qw(sha1 md5 null)],
				     Direction => [qw(in out)],
				     Layer => [qw(restrict permit)],
				     DisableMirror => [qw(true false)],
                                   },
				   mutual_exclusions => {
                                     EncryptionAlgorithm => [qw(AuthenticationAlgorithm)],
                                     AuthenticationAlgorithm => [qw(EncryptionAlgorithm)],
                                   },
                               },
			       IpsecPolicyDelete => {
                                   command => "ipsec policy delete",
                                   conversion => \%Ipsecconversion,
                                   options => \%options,
                                   required_options => [qw(All Index)],
                                   optional_options => [qw(SourceIP DestinationIP Direction DisableMirror)],
                                   verification => "_ReturnOutputverification",
                                   mutual_exclusions => {
                                     All => [qw(Index SourceIP DestinationIP Direction DisableMirror)],
                                     Index => [qw(All SourceIP DestinationIP Direction DisableMirror)],
				     SourceIP => [qw(All Index DestinationIP)],
				     DestinationIP => [qw(All Index SourceIP)],
				     Direction => [qw(All Index)],
				     DisableMirror => [qw(All Index)],
                                   },
				   supported_values => {
				     Direction => [qw(in out)],
				     All => [qw(true false)],
				     DisableMirror => [qw(true false)],
				   },
                               },
			       IpsecPolicyShow => {
                                   command => "ipsec policy show",
                                   conversion => \%Ipsecconversion,
                                   options => \%options,
                                   optional_options => [qw(SourceIP DestinationIP UpperLayerProtocol Direction Protocol)],
                                   verification => "_ReturnOutputverification",
                                   supported_values => {
                                     Protocol => [qw(esp ah)],
                                     Direction => [qw(in out)],
                                   },
                               },
			       IpsecSaShow => {
                                   command => "ipsec sa show",
                                   conversion => \%Ipsecconversion,
                                   options => \%options,
                                   optional_options => [qw(SPI SourceIP DestinationIP Protocol Verbose)],
                                   verification => "_ReturnOutputverification",
                                   supported_values => {
                                     Protocol => [qw(esp ah)],
                                     Verbose => [qw(true false)],
                                     SPI => [qw(true false)],
                                   },
				   mutual_exclusions => {
                                     SPI => [qw(SourceIP DestinationIP Protocol)],
                                     SourceIP => [qw(SPI)],
                                     DestinationIP => [qw(SPI)],
                                     Protocol => [qw(SPI)],
				   },
				   dependencies => {
				     DestinationIP => {
                                        SourceIP => "presence",
                                     },
				     SourceIP => {
                                        DestinationIP => "presence",
                                     },
				     Protocol => {
                                        DestinationIP => "presence",
                                     },
                                   },
                               },
			       IpsecStats => {
                                   command => "ipsec stats",
                                   conversion => \%Ipsecconversion,
                                   options => \%options,
                                   optional_options => [qw(Zero)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Zero => [qw(true false)],
                                   },
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupIpsecCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
