#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "ipspace" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Ipspace.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "ipspace" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_IPSPACE}) {
    my $newModule = $ENV{OVERRIDECLASSIC_IPSPACE};
    delete $ENV{OVERRIDECLASSIC_IPSPACE};
    print "The main Classic::Ipspace module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Ipspaceconversion = (
			);
my %options = (
		   IPSpaceName => " ",
		   InterfaceList => " ",
		   InterfaceName => " ",
		    );
#************************************************************************

=head1 C<IpspaceCreate()>

=over 4

=item Description

This method implements the "ipspace create" command

=item Arguments

=over 4

=item IPSpaceName
(Required)

The name of the IP space

=item InterfaceList
(Optional)

The list of interfaces to assign to this IP space

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpspaceDelete()>

=over 4

=item Description

This method implements the "ipspace delete" command

=item Arguments

=over 4

=item IPSpaceName
(Required)

The name of the IP space to delete

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpspaceAssign()>

=over 4

=item Description

This method implements the "ipspace assign" command

=item Arguments

=over 4

=item IPSpaceName
(Required)

The name of the IP space

=item InterfaceName
(Required)

The IP to assign to this IP space

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpspaceList()>

=over 4

=item Description

This method implements the "ipspace list" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IpspaceHelp()>

=over 4

=item Description

This method implements the "ipspace help" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       IpspaceCreate => {
                                   command => "ipspace create",
                                   conversion => \%Ipspaceconversion,
                                   options => \%options,
				   required_options => [qw(IPSpaceName)],
				   optional_options => [qw(InterfaceList)],
                                   verification => "_ReturnOutputverification",
                               },
			       IpspaceDelete => {
                                   command => "ipspace delete",
                                   conversion => \%Ipspaceconversion,
                                   options => \%options,
                                   required_options => [qw(IPSpaceName)],
                                   verification => "_ReturnOutputverification",
                               },
			       IpspaceAssign => {
                                   command => "ipspace assign",
                                   conversion => \%Ipspaceconversion,
                                   options => \%options,
                                   required_options => [qw(IPSpaceName InterfaceName)],
                                   verification => "_ReturnOutputverification",
                               },
			       IpspaceList => {
                                   command => "ipspace list",
                                   conversion => \%Ipspaceconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       IpspaceHelp => {
                                   command => "ipspace help",
                                   conversion => \%Ipspaceconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       );
			       
my %Compatability = (
		     );

sub SetupIpspaceCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
