#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "iscsi" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Iscsi.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "iscsi" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_ISCSI}) {
    my $newModule = $ENV{OVERRIDECLASSIC_ISCSI};
    delete $ENV{OVERRIDECLASSIC_ISCSI};
    print "The main Classic::Iscsi module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

our %Iscsiconversion = (
			);
my %options = (
		   Adapter => " ",
		   State => " ",
		   NewNodeName => " ",
		   Zero => "-z",
		   Method => "-s",
		   InPassword => "-p",
		   InUserName => "-n",
		   OutPassword => "-o",
		   OutUserName => "-m",
		    );
#************************************************************************

=head1 C<IscsiConfig()>

=over 4

=item Description

This method implements the "iscsi config" command

=item Arguments

=over 4

=item Adapter
(Optional)

The adapter to configure

=item State
(Optional)

The state to place the adapter in

Supported Values: "up" or "down"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiNodename()>

=over 4

=item Description

This method implements the "iscsi nodename" command

=item Arguments

=over 4

=item NewNodeName
(Optional)

The new nodename

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiStart()>

=over 4

=item Description

This method implements the "iscsi start" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiStop()>

=over 4

=item Description

This method implements the "iscsi stop" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiShowAdapter()>

=over 4

=item Description

This method implements the "iscsi show adapter" command

=item Arguments

=over 4

=item Adapter
(Optional)

The adapter to show info on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiShowInitiator()>

=over 4

=item Description

This method implements the "iscsi show initiator" command

=item Arguments

=over 4

=item Adapter
(Optional)

The initiator to show info on

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiStats()>

=over 4

=item Description

This method implements the "iscsi stats" command

=item Arguments

=over 4

=item Adapter
(Optional)

The adapter to show stats on

=item Zero
(Optional)

Implements the "-z" flag and zeros out all stats

Supported Values: "true" or "false"

=back

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiStatus()>

=over 4

=item Description

This method implements the "iscsi status" command

=item Arguments

None

=item Returns

None

=back

=cut

#************************************************************************

=head1 C<IscsiSecurityDefault()>

=over 4

=item Description

This method implements the "iscsi security default" command

=item Arguments

=over 4

=item Method
(Required)

Implements the "-s" flag

=item InPassword
(Required)

Implements the "-p" flag 

=item InUserName
(Required)

Implements the "-n" flag

=item OutPassword
(Optional)

Implements the "-o" flag

=item OutUserName
(Optional)

Implements the "-m" flag

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
			       IscsiConfig => {
                                   command => "iscsi config",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
				   optional_options => [qw(Adapter State)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     State => [qw(up down)],
                                   },
				   dependencies => {
                                     State => {
                                                Adapter => "presence",
                                              },
                                   },
                               },
			       IscsiNodename => {
                                   command => "iscsi nodename",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
                                   optional_options => [qw(NewNodeName)],
                                   verification => "_ReturnOutputverification",
                               },
			       IscsiStart => {
                                   command => "iscsi start",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       IscsiStop => {
                                   command => "iscsi stop",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       IscsiShowAdapter => {
                                   command => "iscsi show adapter",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
				   optional_options => [qw(Adapter)],
                                   verification => "_ReturnOutputverification",
                               },
			       IscsiShowInitiator => {
                                   command => "iscsi show initiator",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
				   optional_options => [qw(Adapter)],
                                   verification => "_ReturnOutputverification",
                               },
			       IscsiStats => {
                                   command => "iscsi stats",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
                                   optional_options => [qw(Adapter Zero)],
                                   verification => "_ReturnOutputverification",
				   supported_values => {
                                     Zero => [qw(true false)],
                                   },
                               },
			       IscsiStatus => {
                                   command => "iscsi status",
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
                               },
			       IscsiSecurityDefault => {
                                   command => "iscsi security default",
				   required_options => [qw(Method InPassword InUserName)],
				   optional_options => [qw(OutPassword OutUserName)],
                                   conversion => \%Iscsiconversion,
                                   options => \%options,
                                   verification => "_ReturnOutputverification",
 				   dependencies => {
                                     InPassword => {
                                                Method => "CHAP",
                                                InUserName => "presence",
                                              },
                                     InUserName => {
                                                Method => "CHAP",
                                                InPassword => "presence",
                                              },
				     OutPassword => {
                                                Method => "CHAP",
                                                InPassword => "presence",
                                                InUserName => "presence",
                                                OutUserName => "presence",
                                              },
				     OutUserName => {
                                                Method => "CHAP",
                                                InPassword => "presence",
                                                InUserName => "presence",
                                                OutPassword => "presence",
                                              },
                                   },
                               },

			       );
			       
my %Compatability = (
		     );

sub SetupIscsiCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

1;
