#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "qtree" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Qtree.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "qtree" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Node::Classic.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_QTREE}) {
    my $newModule = $ENV{OVERRIDECLASSIC_QTREE};
    delete $ENV{OVERRIDECLASSIC_QTREE};
    print "The main Classic::Qtree module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

use strict;
use QtreeCmds qw(qtreeCreate qtreeStatus qtreePresent qtreeDelete qtreeComponents qtreeList);
use nfs::GenericExport qw(list_exports export_path get_export_options);

my %command_definitions = (
			       );
			       
my %Compatability = (
		     );

sub SetupQtreeCommands {
    my ($self, @servers) = @_;
    return 1;
}

sub qtree_exists {
    my ($self, %params) = @_;
    
    $self->{Trace}->entry(msg=>"qtree=>$params{qtree}");
    my $rc = qtreePresent($self->get_connection(type=>"cli_connection"), $params{qtree});
    $self->{Trace}->exit(msg=>"return $rc");
    return $rc;
}

sub qtree_status {
    my ($self, %params) = @_;
    
    $self->{Trace}->entry(msg=>"qtree=>$params{qtree}");
    $self->{Trace}->fatal(msg=>$@) if (!qtreeStatus($self->get_connection(type=>"cli_connection"), $params{qtree}));
    $self->{Trace}->exit();
}

sub qtree_create {
    my $self = shift;
    my %params = (
		  qtree => undef,
		  @_);
    
    $self->{Trace}->entry(msg=>"qtree=>$params{qtree}");
    my $conn = $self->get_connection(type=>"cli_connection");
    if (!qtreeCreate($conn, $params{qtree}) && $@ !~ /File exusts/) {
        $self->{Trace}->fatal(msg=>$@);
    }
    $self->{Trace}->exit();
    return \%params;
}

sub qtree_delete {
    my ($self, %params) = @_;
    
    $self->{Trace}->entry(msg=>"qtree=>$params{qtree}");
    my $conn = $self->get_connection(type=>"cli_connection");
    my $exporth = list_exports($conn);
    my ($volname, $qtreename) = qtreeComponents($params{qtree});
    my $export = "/vol/$volname";
    my %opts = get_export_options($conn, $export);
    if ($opts{anon} ne "anon=0") {
	export_path($conn, $export, OPTIONS=>"anon=0", PERSISTENT_OPT=>1);
    }
    $self->{Trace}->fatal(msg=>$@) if (!qtreeDelete($conn, $params{qtree}));
    if (!defined($opts{anon}) || $opts{anon} != 0) {
	my $options;
	foreach my $opt (keys %opts) {
	    $options .= ",$opts{$_}" if ($opts{$opt});
	}
	$self->{Trace}->fatal(msg=>$@) if ($options && !export_path($conn, $export, OPTIONS=>$options, PERSISTENT_OPT=>1));
    }
    $self->{Trace}->exit();
}

sub qtree_show {
    my ($self, %params) = @_;
    
    $self->{Trace}->entry(msg=>"qtree=>$params{qtree}");
    my @qtrees = qtreeList($self->get_connection(type=>"cli_connection"));
    $self->{Trace}->exit(msg=>join(" ", @qtrees));
    return @qtrees;
}

1;
