#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Serer.pm to implement all "vfiler" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

VFiler.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "vfiler" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Server.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_VFILER}) {
    my $newModule = $ENV{OVERRIDECLASSIC_VFILER};
    delete $ENV{OVERRIDECLASSIC_VFILER};
    print "The main Classic::VFiler module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

# only require so that the current namespace is not modified.
use TracePkg;
our $Trace;
our %VFilerconversion = (
			 vserver => "vfiler_template",
			);
my %options = (
	       vfiler_template => " ",
	       interfaces => "-r",
	       all => "-a",
	       );
#************************************************************************

#************************************************************************

my %command_definitions = (
			   vfiler_status => {
			       command => "vfiler status",
			       options => \%options,
			       optional_options => [qw(interfaces all vfiler_template)],
			       mutual_exclusions => {
				   interfaces => [qw(all)],
				   all => [qw(interfaces)],
			       },
			       verification => "_process_vfiler_status",
			   },
		      );

my %Compatability = (
		     vserver_show => "vfiler_status",
		     );

sub SetupVFilerCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

sub _process_vfiler_status {
    my ($self, %params) = @_;
    $self->{Trace}->entry();
    my $want_array = $params{want_array};
    $want_array = [] if (!$want_array);
    my $output = $params{output_in};

    my $more_options = undef;
    if ($params{vfiler_template}) {
	my %vfiler = ();
	foreach my $line (split(/\n/, $output)) {
	    if ($line =~ (/^($params{vfiler_template})\s+(\S+)/)) {
		$vfiler{vserver} = $1;
		$vfiler{status} = $2;
		$vfiler{interfaces} = [];
	    } elsif ($line =~ /\s+IP address:\s+(\S+)/) {
		push(@{$vfiler{interfaces}}, $1);
	    }
	}
	push(@$want_array, \%vfiler);
    } else {
	foreach my $line (split(/\n/, $output)) {
	    if ($line =~ /Use of vfilers not licensed/ ||
		$line =~ /Use of vfilers requires a multistore license/) {
		push(@$want_array, {
		    vserver => "N/A",
		});
		last;
	    }
	    if ($line =~ /\s*(\S+)\s+\S+/) {
		my $vfiler = $self->vfiler_status(vfiler_template=>$1, interfaces=>"");
		push(@$want_array, @$vfiler);
	    }
	}
    }
    $self->{Trace}->exit(msg => "Exiting with $want_array");
    return $want_array;
}

1;
