#***********************************************************************
# This is the comment section for tharndoc
#***********************************************************************
## @summary Perl module used by Filer.pm to implement all "vol" commands in ONTAP
## @author dl-central-libs
## @pod here

=head1 NAME

Vol.pm

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

This module implements all of the "vol" commands in ONTAP

=head1 SYNOPSIS

This module is instantiated by all test scripts that use the Filer.pm module
and is not directly called.
See the descriptions below if you need the syntax for using a specific method

=head1 EXAMPLES

No examples needed

=cut

#***********************************************************************
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
my $LibraryLoaded;
if ($ENV{OVERRIDECLASSIC_VOL}) {
    my $newModule = $ENV{OVERRIDECLASSIC_VOL};
    delete $ENV{OVERRIDECLASSIC_VOL};
    print "The main Classic::Vol module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

use strict;
use Data::Dumper;
# only require so that the current namespace is not modified.
use nfs::GenericExport qw(export_path);
use Volume qw(vol_exists);
use VolCmds qw(maxFiles);

use TracePkg;
our %Volconversion = (
			);
my %options = (
		   volume => " ",
		   RaidType => "-t",
		   RaidSize => "-r",
		   NumberDisks => " ",
		   NumberDisksAtSize => " ",
		   DiskList => "-d",
		   DiskListMirror => "-d",
		   Mirror => "-m",
		   SnapLock => "-L",
		   NoExecute => "-n",
		   Language => "-l",
		   aggregate => " ",
		   size => " ",
		   Verbose => "-v",
		   RAIDInfo => "-r",
		   DisksInfo => "-d",
		   LanguageInfo => "-l",
		   ChecksumInfo => "-c",
		   SnapSizeInfo => "-b",
		   SpareInfo => "-s",
		   FailedDisksInfo => "-f",
		   Force => "-f",
		   Guarantee => "-s",
		    );
#************************************************************************

=head1 C<VolCreate()>

=over 4

=item Description

This method implements the "vol create" command

=item Arguments

=over 4

=item volume
(Required)

The volume name

=item NumberDisks
(Required)

The number of disks to use in the volume creation

Mutually exclusive with "NumberDisksAtSize", "DiskList"

=item NumberDisksAtSize
(Required)

Specifies minimum sized disks (in GB) to use in vol creation.
Syntax is <num_disk>@<size_in_GB>. For example "5@20" means
use 5 disks that are at least 20GB

Mutually exclusive with "NumberDisks", "DiskList"

=item DiskList
(Required)

Implements the "-d" flag and specifies a space separated list of disks
to use in the volume creation

Mutually exclusive with "NumberDisks", "NumberDisksAtSize"

=item RaidType
(Optional)

Implements the "-t" flag and specifies the RAIDtype to use for the volume

Supported Values: "raid4", "raid_dp"

=item RAIDSize
(Optional)

Implements the "-r" flag and specifies the maximum number of disks in each RAID group of the volume

=item DiskListMirror
(Optional)

If the "-m" option is set, the DiskListMirror implements the optional second "-d" list of disks to 
be used for the mirror plex.

=item Mirror
(Optional)

Implements the "-m" flag and specifies that the new volume should be mirrored

Supported Values: "true", "false" 

=item SnapLock
(Optional)

Implements the "-L" flag and specifies that the new volume should use SnapLock

Supported Values: "true", "false"

=item NoExecute
(Optional)

Implements the "-n" flag and causes the command to be printed as it will be used, but not actually executed.
Used for debugging

Supported Values: "true", "false"

=item Language
(Optional)

Implements the "-l" flag and specifies the language type to use for the volume

Supported Values: c (POSIX)
		  da (Danish)
		  de (German)
		  en (English)
		  en_US (English (US))
		  es (Spanish)
		  fi (Finnish)
		  fr (French)
		  he (Hebrew)
		  it (Italian)
		  ja (Japanese euc-j)
		  ja_JP.PCK (Japanese PCK (sjis))
		  ko (Korean)
		  no (Norwegian)
		  nl (Dutch)
		  pt (Portugese)
		  sv (Swedish)
		  zh (Simplified Chinese)
		  zh.GBK (Simplified Chines (GBK))
		  zh_TW (Traditional Chinese euc-tw)
		  zh_TW.BIG5 (Traditional Chinese Big 5)

=back

=item Returns

None

=back

=cut

#************************************************************************

my %command_definitions = (
		      vol_create => {
			  command => "vol create",
			  conversion => \%Volconversion,
			  options => \%options,
			  required_options => [qw(volume NumberDisks NumberDisksAtSize DiskList aggregate size)],
			  optional_options => [qw(RaidType RAIDSize DiskListMirror Mirror SnapLock NoExecute Language Garantee)],
			  defaults => {
				Guarantee => "none",
			  },
			  successful_output => ["has completed", "already exists"],
			  verification => "vol_status",
			  supported_values => {
                              RaidType => [qw(raid4 raid_dp)],
                              Mirror => [qw(true false)],
                              SnapLock => [qw(true false)],
                              NoExecute => [qw(true false)],
                              Language => [qw(C da de en en_US es fi fr he it ja ja_JP.PCK ko no nl pt sv zh zh.GBK zh_TW zh_TW.BIG5)],
                          },
			  mutual_exclusions => {
                              NumberDisks => [qw(NumberDisksAtSize DiskList size)],
                              NumberDisksAtSize => [qw(NumberDisks DiskList size)],
                              DiskList => [qw(NumberDisks NumberDisksAtSize size)],
                              size => [qw(NumberDisks NumberDisksAtSize DiskList)],
			  },
			  dependencies => {
                              DiskListMirror => {
                                  Mirror => "true",
                              },
			  },
		      },
		      vol_offline => {
			  command => "vol offline",
			  conversion => \%Volconversion,
			  options => \%options,
			  required_options => [qw(volume)],
			  successful_output => ["is now offline", "already offline"],
		      },
		      vol_destroy => {
			  command => "vol destroy",
			  conversion => \%Volconversion,
			  options => \%options,
			  required_options => [qw(volume)],
			  post_command_options => [qw(Force)],
			  successful_output => ["destroyed", "No volume names .+ exists"],
		      },
		      vol_status => {
			  command => "vol status",
			  conversion => \%Volconversion,
			  options => \%options,
			  optional_options => [qw(volume Verbose RAIDInfo DisksInfo LanguageInfo ChecksumInfo SnapSizeInfo SpareInfo FailedDisksInfo)],
			  verification => "_process_vol_status",
			  mutual_exclusions => {
			      Verbose => [qw(RAIDInfo DisksInfo LanguageInfo ChecksumInfo SnapSizeInfo SpareInfo FailedDisksInfo)],
			      RAIDInfo => [qw(Verbose DisksInfo LanguageInfo ChecksumInfo SnapSizeInfo SpareInfo FailedDisksInfo)],
			      DisksInfo => [qw(Verbose RAIDInfo LanguageInfo ChecksumInfo SnapSizeInfo SpareInfo FailedDisksInfo)],
			      LanguageInfo => [qw(Verbose RAIDInfo DisksInfo ChecksumInfo SnapSizeInfo SpareInfo FailedDisksInfo)],
			      ChecksumInfo => [qw(Verbose RAIDInfo DisksInfo LanguageInfo SnapSizeInfo SpareInfo FailedDisksInfo)],
			      SnapSizeInfo => [qw(Verbose RAIDInfo DisksInfo LanguageInfo ChecksumInfo SpareInfo FailedDisksInfo)],
			      SpareInfo => [qw(Verbose RAIDInfo DisksInfo LanguageInfo ChecksumInfo SnapSizeInfo FailedDisksInfo)],
			      FailedDisksInfo => [qw(Verbose RAIDInfo DisksInfo LanguageInfo ChecksumInfo SnapSizeInfo SpareInfo)],
			  },
			  failure_output => [("No volume named")],
		      },
		      
		      );

my %Compatability = (
		     );

sub SetupVolCommands {
    my ($self, @servers) = @_;
    $self->_define_commands(Definitions => \%command_definitions,
			    Compatability => \%Compatability,
			    Servers => \@servers);
    return 1;
}

sub vol_exists {
    my ($self, %params) = @_;
    $self->{Trace}->entry(msg=>"volume=$params{volume}, aggr=>$params{aggregate}");
    my $rc;

    my @aggrs = $self->{server_object}->search_topology(cmd_node=>"aggregate", key=>"aggregate", value=>$params{aggregate});
    my $aggrh = $aggrs[0];

    if ($aggrh->{type} eq "trad") {
	my $qtree = "/vol/$params{aggregate}/$params{volume}";
	$rc = $self->qtree_exists(%params, qtree=>$qtree);
    } else {
	$rc = Volume::vol_exists($self->get_connection(type=>"cli_connection"),$params{volume});
    }
    $self->{Trace}->exit(msg => "returning $rc");
    return $rc;
}

sub volume_exists {
    my ($self, %params) = @_;
    if (!$params{volume}) {
        $self->{Trace}->fatal(msg=>"'volume' is a required param", result=>"SCRIPT");
    }
        
    $self->{Trace}->entry(msg=>"volume=$params{volume}");

    my $volume;
    if ($self->vol_exists(%params)) {
        $volume = $self->vol_status(%params);
    }
    $self->{Trace}->exit(msg=>"volume=$volume");
    return $volume;
}

sub volume_show {
    my ($self, %params) = @_;
    $self->{Trace}->entry(msg=>"volume=$params{volume}");

    my $volume = [];
    if ($params{volume}) {
	$volume = $self->vol_status(%params);
    }
    $self->{Trace}->exit(msg=>"");
    return $volume;
}

sub volume_new {
    my ($self, %params) = @_;
    $self->{Trace}->entry(msg=>"volume=$params{aggregate},$params{volume},$params{size}");

    my $volumes;
    my @aggrs = $self->{server_object}->search_topology(cmd_node=>"aggregate", key=>"aggregate", value=>$params{aggregate});
    my $aggrh = $aggrs[0];

    if ($aggrh->{type} eq "trad") {
	my $qtree = "/vol/$params{aggregate}/$params{volume}";
	my $hash = $self->qtree_create(qtree=>$qtree);
	$hash->{'junction-path'} = $qtree;
	push(@$volumes, $hash);
    } else {
        # translate size into something that DOT understands and make sure we weren't given a floating point
        # number which DOT can't handle
        $params{size} =~ s/KB/k/;
        $params{size} =~ s/MB/m/;
        $params{size} =~ s/GB/g/;
        $params{size} =~ s/TB/t/;
        if ( $params{size} =~ /(\d+\.\d+)(.)/ ) {
            my $rounded_size = int($params{size} + .5);
            $self->{Trace}->warn(msg => "Given a floating point number for volume size which isn't
                valid, rounding '$params{size}' to '$rounded_size'");
            $params{size} = $rounded_size;
        }

        $volumes = $self->vol_create(%params);
    }

    if ($params{ruleset}) {
	my $export_options = $self->{options_object}->get(option=>"EXPORT_OPTIONS");
	my %export = $self->protocol_exportruleset_rule_show(rulesetname=>$params{ruleset}, vserver=>$params{vserver});
	$export_options .= ",anon=$export{anon}";

	# Removing trailing or leading commas, if it's included
	$export_options =~ s/^,//g;
	$export_options =~ s/,$//g;
	
	# could support rwrule
	# could support rorule
	# could support clientmatch

        eval {
            export_path($self->get_connection(type=>"cli_connection"), 
                        $volumes->[0]->{'junction-path'}, 
                        OPTIONS=>$export_options, 
                        PERSISTENT_OPT=>1) 
	    or die "exportfs failed: $@";
        };
        if ($@) {
            if ($@ =~ /Could not create persistent entry/) {
                $self->{Trace}->warn(msg=>$@);
            } else {
                die $@;
            }
        }
    }

    if ($params{files}) {
	my $rc = maxFiles($self->get_connection(type=>"cli_connection"), $params{volume}, $params{files});
	if (!defined($rc) && $@ !~ /The new maximum number of files will be rounded/ &&
           $@ !~ /New count must be larger than current/) {
	    $self->{Trace}->fatal(msg=>"Could not set maxfile for $params{volume}: $@");
	}
    }
    
    $self->{Trace}->exit(msg => "");
    return $volumes;
}

sub volume_delete {
    my ($self, %params) = @_;
    my $rc;
    $self->{Trace}->entry(msg=>"volume=$params{volume}");
    my @aggrs = $self->{server_object}->search_topology(cmd_node=>"aggregate", key=>"aggregate", value=>$params{aggregate});
    my $aggrh = $aggrs[0];
    if ($aggrh->{type} eq "trad") {
	my $qtree = "/vol/$params{aggregate}/$params{volume}";
	$rc = $self->qtree_delete(qtree=>$qtree);
    } else {
	$self->vol_offline(%params);
	$self->vol_destroy(%params, Force=>" ");
    }
    $self->exportfs_reload();
    $self->{Trace}->exit(msg => "returning $rc");
    return $rc;
}

sub volume_mount_point {
    my ($self, %params) = @_;
    my $volume = $params{volume};
    my $share;
    $self->{Trace}->entry(msg=>"volume=$volume");

    my @aggrs = $self->{server_object}->search_topology(cmd_node=>"aggregate", key=>"aggregate", value=>$params{aggregate});
    my $aggrh = $aggrs[0];
    if ($aggrh->{type} eq "trad") {
	my @qtrees = $self->qtree_show();
	foreach my $qtree (@qtrees) {
	    if ($qtree =~ m!/vol/$params{aggregate}/$params{volume}!) {
		if ($share) {
		    $self->{Trace}->fatal(msg=>"multiple mount_points exist for volume '$params{volume}'");
		}
		$share = $qtree 
	    } elsif ($qtree =~ m!^/vol/$params{volume}/!) {
		$share = "/vol/$params{volume}";
		last;
	    }
	}
	if (!$share) {
	    $self->{Trace}->fatal(msg=>"Could not find the export for '$params{volume}'");
	}
    } else {
	$share = "/vol/$volume";
    }
    $self->{Trace}->exit(msg=>"return '$share'");
    return $share;
}

sub _process_vol_status {
    my ($self, %params) = @_;
    $self->{Trace}->entry();
    my $want_array = $params{want_array};
    $want_array = [] if (!$want_array);
    my $output = $params{output_in};

    my $more_options = undef;
    if ($params{volume}) {
	my %vol = ();
	my $options;
	foreach my $line (split(/\n/, $output)) {
	    next if ($line =~ /\s+Volume State/);
	    if ($line =~ /^\s*\S*$params{volume}\s+(.+)/) {
		$vol{volume} = $params{volume};
		($vol{state}, $vol{status}, $options, $vol{source}) = split(/\s{2,}/, $1);
	    } elsif ($line =~ /\s+Containing aggregate: '(\S+)',*/) {
		$vol{aggregate} = $1;
		last;
	    } elsif ($line =~ /^\s+(\S.+)\s{2,}(\S.+)/) {
		$vol{status} .= ",$1";
		$options .= $2;
	    } elsif ($line =~ /^\s+(.+)/) {
		$options .= $1;
	    }
	}
	$vol{'junction-path'} = "/vol/$params{volume}";
	$vol{status} =~ s/\s//g;
	$options =~ s/\s//g;
	foreach my $option (split(/,/, $options)) {
	    $vol{$1} = $2 if ($option =~ /(.+)=(.+)/);
	}
	$vol{'junction-path'} = "/vol/$params{volume}";
	$vol{filers} = $self->{server};
        
        # don't add a {comment} to the volume structure since TestManager is
        #  relying on its absence for detecting that the volume is not on
        #  a Tricky box
	push(@$want_array, \%vol);
    } else {
	foreach my $line (split(/\n/, $output)) {
	    next if ($line =~ /\s+Volume State/);
	    next if ($line =~ /^vol status/);
	    if ($line =~ /\s*(\S+)\s+\S+/) {
		my $volume = $self->vol_status(volume=>$1);
		push(@$want_array, @$volume);
	    }
	}
    }
    $self->{Trace}->exit(msg => "Exiting with $want_array");
    return $want_array;
}

1;
