#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/Node/NG/Bootmenu.pm#1 $
## @summary     Module to define commands for the Boot Menu. 
## @author      dl-central-libs
## @pod here

=head1 NAME

Node::NG::Bootmenu - Module to define commands for the Boot Menu.

=head1 AUTHOR

dl-central-libs

=head1 SYNOPSIS

Internal Use

=head1 DESCRIPTION

This package is used to modularize the command definitions for the Boot Menu.

=head1 METHODS

none

=cut


use strict;
use TracePkg;
my $Trace = new TracePkg;

my %Options = (
);

my %CommandDefinitions = (
    boot_menu_install => {
        command => "install",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts => [
        {
            prompt => 'Do you wish to continue',
            response => "y",
        },
        {
            prompt => 'What is the URL for the package',
            response => '-url',
            timeout => 1800,
        },
        {
            prompt => 'subsequent reboots',
            response => '-default',
        },
        {
            prompt => 'want to reboot now',
            response => '-reboot',
            timeout => 180,
        },
        {
            prompt => 'CFE(?:-\w+)?\*?>\s',
            respose => "",
        },
        ],
    },
    boot_menu_install_fail => {
        command => "install",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts => [
        {
            prompt => 'Do you wish to continue',
            response => "y",
        },
        {
            prompt => 'What is the URL for the package',
            response => '-url',
            timeout => 120,
        },
        {
            prompt => 'Please make a selection:',
            response => "",
        },
        ],
    },
    netboot_menu_install => {
        command => "install",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts => [
        {
            prompt => 'Do you wish to continue',
            response => "y",
        },
        {
            prompt => 'What is the URL for the package',
            response => '-url',
            timeout => 1800,
        },
        {
            prompt => 'want to reboot now',
            response => '-reboot',
            timeout => 1800,
        },
        {
            prompt => 'CFE(?:-\w+)?\*?>\s',
            respose => "",
        },
        ],
    },
    netboot_menu_install_util => {  #This is called util becasue this command_definition is to be used as a utility.
        command => "install",       #netboot_menu_install is to be used as an actual test of install and thus is an ARRAY or HASH
        options => \%Options,       #netboot_menu_install_util is to be used as a utility to get new bits on a node and thus is a HASH
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "1980",
        additional_prompts => {
            "Do you wish to continue"         => "y",
            "Do you want to continue"                 => "y",
            "What is the URL for the package" => '-url',
            "want to reboot now"              => '-reboot',
            "either reboot or retry install"  => 'FINAL_PROMPT',
            #"Please make a selection"        => 'FINAL_PROMPT',
            'CFE(?:-\w+)?\*?>\s'               => 'FINAL_PROMPT',
            "Starting AUTOBOOT press"         => "\cc",
            'LOADER(?:-\w+)?\*?>\s'            => 'FINAL_PROMPT',
            "Invalid package URL"             => 'FATAL_PROMPT',
            "Install Failed"                  => 'FATAL_PROMPT',
            "Couldn't resolve host"           => 'FATAL_PROMPT',
            "Unrecognized archive format"     => 'FATAL_PROMPT',
            "access denied"                   => 'FATAL_PROMPT',
            "Failed to open file"             => 'FATAL_PROMPT',
            #"Failed to download package"     => 'FATAL_PROMPT',
            "newly installed software as the default" => "y",  #needed temporarily for excelsior support
            "to boot immediately"                     => "\r", #needed temporarily for excelsior support
            "any other key for command prompt"        => "\r", #needed temporarily for excelsior support
        },
    },
    netboot_menu_fake_install_util => {  #This is called util becasue this command_definition is to be used as a utility.
        command => "fake_install",       #netboot_menu_fake_install is to be used as an actual test of install and thus is an ARRAY or HASH
        options => \%Options,	         #netboot_menu_fake_install_util is to be used as a utility to get new bits on a node and thus is a HASH
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "1980",
        additional_prompts => {
            "Do you wish to continue"                 => "y",
            "Do you want to continue"                 => "y",
            "What is the URL for the package"         => '-url',
            "want to reboot now"                      => '-reboot',
            "to continue booting"                     => 'FINAL_PROMPT',
            'CFE(?:-\w+)?\*?>\s'                      => 'FINAL_PROMPT',
            "Starting AUTOBOOT press"                 => "\cc",
            'LOADER(?:-\w+)?\*?>\s'                   => 'FINAL_PROMPT',
            "Invalid package URL"                     => 'FATAL_PROMPT',
            "Install Failed"                          => 'FATAL_PROMPT',
            "Couldn't resolve host"                   => 'FATAL_PROMPT',
            "Unrecognized archive format"             => 'FATAL_PROMPT',
            "access denied"                           => 'FATAL_PROMPT',
            "Failed to open file"                     => 'FATAL_PROMPT',
            "out of swap space"                       => 'FATAL_PROMPT',
            "newly installed software as the default" => "y",  #needed temporarily for excelsior support
            "to boot immediately"                     => "\r", #needed temporarily for excelsior support
            "any other key for command prompt"        => "\r", #needed temporarily for excelsior support
        },
    },
     netboot_menu_force_install_util => {  #This is called util becasue this command_definition is to be used as a utility.
        command => "force_install",       #netboot_menu_fake_install is to be used as an actual test of install and thus is an ARRAY or HASH
        options => \%Options,	         #netboot_menu_fake_install_util is to be used as a utility to get new bits on a node and thus is a HASH
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "1980",
        additional_prompts => {
            "Do you wish to continue"                 => "y",
            "Do you want to continue"                 => "y",
            "What is the URL for the package"         => '-url',
            "want to reboot now"                      => '-reboot',
            'CFE(?:-\w+)?\*?>\s'                      => 'FINAL_PROMPT',
            "Starting AUTOBOOT press"                 => "\cc",
            'LOADER(?:-\w+)?\*?>\s'                   => 'FINAL_PROMPT',
            "Invalid package URL"                     => 'FATAL_PROMPT',
            "Install Failed"                          => 'FATAL_PROMPT',
            "Couldn't resolve host"                   => 'FATAL_PROMPT',
            "Unrecognized archive format"             => 'FATAL_PROMPT',
            "access denied"                           => 'FATAL_PROMPT',
            "Failed to open file"                     => 'FATAL_PROMPT',
            "Selection not recognized"                => 'FATAL_PROMPT',
            "out of swap space"                       => 'FATAL_PROMPT',
            "newly installed software as the default" => "y",  #needed temporarily for excelsior support
            "to boot immediately"                     => "\r", #needed temporarily for excelsior support
            "any other key for command prompt"        => "\r", #needed temporarily for excelsior support
        },
    },  
    boot_menu_setup_util => {  #This is called util becasue this command_definition is to be used as a utility.
        command => "setup",    #boot_menu_setup is to be used as an actual test of BM:setup and thus is an ARRAY or HASH
        options => \%Options,  #boot_menu_setup_util is to be used as a utility to run BM:setup and thus is a HASH
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "180",
        additional_prompts => {
	         'Do you want to setup autoconf'          => '-auto_configuration',
             'Do you want to set up autoconf'         => '-auto_configuration',
             'Please enter the node hostname'                => '-hostname', 
             'Where is the node located'                     => '-location',
             'configure a management interface'              => '-config_mgmt', 
             'Which port do you want to use'                 => '-netdev', 
             'Address for this interface'                    => '-ip', 
             'Netmask for this interface'                    => '-nm', 
             'Address of the default gateway'                => '-gw', 
             'Do you want to run DNS resolver'               => '-config_dns', 
             'Please enter DNS domain name'                  => '-dns_comains', 
             'address for the first nameserver'              => '-dns_ip1', 
             'address for the second nameserver'             => '-dns_ip2', 
             'enable remote management'                      => '-remote_mgmt', 
             'remote management username'                    => "admin", 
             'can this user use'                             => ' ',
             '(E|e)nter a password for user'              => '-pass',
             '(E|e)nter it again'                         => '-pass',
             '(ERROR|Error): Failed to contact name service on host' => "FATAL_PROMPT",
             'The node has now been configured'              => "FINAL_PROMPT",
         },
    },
    boot_menu_setup => {
        command => "setup",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts => [
        {
            prompt => 'Please enter the node hostname',
            response => '-hostname',
        },
        {
            prompt => 'Where is the node located',
            response => '-location',
        },
        {
            prompt => 'Do you want to configure a management interface',
            response => '-config_mgmt',
            timeout => 60,
        },
        {
            prompt => 'Which port do you want to use',
            response => '-netdev',
            timeout => 60,
        },
        {
            prompt => 'Please enter the IP Address for this interface',
            response => '-ip',
            timeout => 60,
        },
        {
            prompt => 'Please enter the Netmask for this interface',
            response => '-nm',
            timeout => 120,
        },
        {
            prompt => 'Please enter the IP Address of the default gateway',
            response => '-gw',
            timeout => 60,
        },
        {
            prompt => 'Do you want to run DNS resolver',
            response => '-config_dns',
            timeout => 60,
        },
        {
            prompt => 'Please enter DNS domain name',
            response => '-dns_comains',
            timeout => 60,
        },
        {
            prompt => 'Please enter the IP address for the first nameserver',
            response => '-dns_ip1',
            timeout => 60,
        },
        {
            prompt => 'Please enter the IP address for the second nameserver',
            response => '-dns_ip2',
            timeout => 60,
        },
        {
            prompt => 'Do you want to enable remote management',
            response => "yes",
            timeout => 60,
        },
        {
            prompt => 'Please specify the remote management username',
            response => "admin",
            timeout => 60,
        },
        {
            prompt => 'can this user use',
            response => "console, http, ssh, telnet",
            timeout => 60,
        },
        {
            prompt => 'The node has now been configured',
            response => "",
        },
        ],
    },
    boot_menu_autoconf_setup => {
        command => "setup",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "1200",
        additional_prompts => {
            'Do you want to setup autoconfig' => '-auto_configuration',
            'Do you want to set up autoconf'  => '-auto_configuration',
            'What is the name of the cluster' => '-clus_name',
            'What port do you want to use for the management interface' => '-mgmt_port',
            'IP addresses for the management interfaces' => '-mgmt_ip',
            'management interface netmask' => '-mgmt_mask',
            'management interface default gateway' => '-mgmt_gateway',
            'IP addresses for the cluster interfaces' => '-cluster_ip',
            'cluster interface netmask' => '-cluster_mask',
            'cluster license' => '-cluster_license',
            'Do you want to run DNS resolver' => '-config_dns',
            'DNS domain name' => '-dns_domains',
            'first nameserver' => '-dns_ip1',
            'second nameserver' => '-dns_ip2',
            '(E|e)nter a password for user' => '-pass',
            '(E|e)nter it again' => '-pass',
            'Entering no will take you back' => 'yes',
            'Proceeding with autoconfig' => "FINAL_PROMPT",
            'set_mgmt_port' => "FATAL_PROMPT",
         },
    },
    boot_menu_fake_init_util => {
        command => "fake_init",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "360",
        additional_prompts => {
            "Starting d-blade"  => "\n",
            "DUMPCORE:"         => 'FATAL_PROMPT',
            "Dump Abort"        => 'FATAL_PROMPT',
            "out of swap space" => 'FATAL_PROMPT',
            "Zeroing disks"     => 'FINAL_PROMPT',
            "parity disks will not be valid" => 'FINAL_PROMPT',
            "will not zero the disks before" => 'FINAL_PROMPT',
            "feature should only be used by" => 'FINAL_PROMPT',
        },
    },
    boot_menu_init_util => {
        command => "init",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "-disk_init_timeout",
        additional_prompts => {
            "Zero disks and install a new file system" => "y",
            "Starting d-blade"                         => '\n',
            "Giving up waiting for mroot"              => '\n',
            "DUMPCORE:"                                => 'FATAL_PROMPT',
            "out of swap space"                        => 'FATAL_PROMPT',
            "Dump Abort"                               => 'FATAL_PROMPT',
            "Cannot log to file"                       => 'FINAL_PROMPT',
            "mroot i.*s now available"                 => 'FINAL_PROMPT',
            "login:"                                   => 'FINAL_PROMPT', 
        },
    },
    boot_menu_444a_init_util => {
        command => "44/4a",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "-disk_init_timeout",
        additional_prompts => {
            "Zero disks and install a new file system" => "y",
            "Starting d-blade"                         => '\n',
            "Giving up waiting for mroot"              => '\n',
            "DUMPCORE:"                                => 'FATAL_PROMPT',
            "out of swap space"                        => 'FATAL_PROMPT',
            "Dump Abort"                               => 'FATAL_PROMPT',
            "Cannot log to file"                       => 'FINAL_PROMPT',
            "mroot i.*s now available"                 => 'FINAL_PROMPT',
            "login:"                                   => 'FINAL_PROMPT', 
        },
    },
	boot_menu_4a_forced_init_util => {
        command => "4a_forced",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "-disk_init_timeout",
        additional_prompts => {
            "Zero disks and install a new file system" => "y",
            "Starting d-blade"                         => '\n',
            "Giving up waiting for mroot"              => '\n',
            "DUMPCORE:"                                => 'FATAL_PROMPT',
            "out of swap space"                        => 'FATAL_PROMPT',
            "Dump Abort"                               => 'FATAL_PROMPT',
            "Cannot log to file"                       => 'FINAL_PROMPT',
            "mroot i.*s now available"                 => 'FINAL_PROMPT',
            "login:"                                   => 'FINAL_PROMPT',
        },
    },

    boot_menu_fake_wipeclean_util => { #This is called util becasue this command_definition is to be used as a utility.
        command => "fake_wipeclean",   #boot_menu_fake_wipeclean is to be used as an actual test of BM:fake_wipeclean and thus is an ARRAY or HASH
        options => \%Options,	       #boot_menu_fake_wipeclean_util is to be used as a utility to run BM:fake_wipeclean and thus is a HASH
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "360",
        additional_prompts => {
            "want to continue"                 => "y",
            "y or n"                           => "y",
            "to boot immediately"              => "\r",
            "any other key for command prompt" => "\r",
            'CFE(?:-\w+)?\*?>\s'               => 'FINAL_PROMPT',
            "Starting AUTOBOOT press"          => "\cc",
            'LOADER(?:-\w+)?\*?>\s'            => 'FINAL_PROMPT',  #this is for excelsior support since there is no CFE
            "No Targets available for dump"    => 'FATAL_PROMPT',  #Found a panic so die
        },
    },
    boot_menu_wipeclean_util => { #This is called util becasue this command_definition is to be used as a utility.
        command => "wipeclean",	  #boot_menu_wipeclean is to be used as an actual test of BM:wipeclean and thus is an ARRAY or HASH
        options => \%Options,	  #boot_menu_wipeclean_util is to be used as a utility to run BM:wipeclean and thus is a HASH
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "10800",
        additional_prompts => {
            "want to continue"        => "y",
            'CFE(?:-\w+)?\*?>\s'      => 'FINAL_PROMPT',
            "Starting AUTOBOOT press" => "\cc",
            'LOADER(?:-\w+)?\*?>\s'   => 'FINAL_PROMPT',  #this is for excelsior support since there is no CFE
            "No Targets available for dump"    => 'FATAL_PROMPT',  #Found a panic so die
        },
    },
    boot_menu_fake_wipeclean => {
        command => "fake_wipeclean",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts => [
        {
            prompt => 'want to continue',
            response => "y",
            timeout => 360,
        },
        {
            prompt => 'CFE(?:-\w+)?\*?>\s',
            response => "",
        },
        ],
    },
    boot_menu_wipeclean => {
        command => "wipeclean",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts => [
        {
            prompt => 'want to continue',
            response => "y",
            timeout => 5400,
        },
        {
            prompt => 'CFE(?:-\w+)?\*?>\s',
            response => "",
        },
        ],
    },
    boot_menu_maint_maintcli_util => {   #This is called util becasue this command_definition is to be used as a utility.
        command => "maint",		 #boot_menu_maint_maintcli_util is to be used as a utility to get into maintcli and thus is a HASH
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "210",
        additional_prompts => {
            "login:"     => "admin",
            "l.o.g.i"    => "admin",
            "l..o..g..i" => "admin",
            "lo.g.i"     => "admin",
            "lo.g..i"    => "admin",
            "l.og.i"     => "admin",
            "l.og..i"    => "admin",
            "l..og.i"    => "admin",
            "l..og..i"   => "admin",
            "log.i"      => "admin",
            "log..i"     => "admin",
            "l.ogi"      => "admin",
            "l..ogi"     => "admin",
            "g.i.n.:"    => "admin",
            "g.in:"      => "admin",
            "gin:"       => "admin",
            "Password:"  => "\r",
            "Pas"        => "\r",
            "ord:"       => "\r",
            "P.a.s.s"    => "\r",
            "o.r.d.:"    => "\r",
            #"pure virtual method called" => "FATAL_PROMPT", #commented out becasue I want to see more of why the smokes are failing in the logs
            #"terminate called without an active exception" => "FATAL_PROMPT",
            #"Abort trap" => "FATAL_PROMPT",                 #if I wait longer before failing then there shoudl be more output from the console in the logs
            "Last login:" => "FINAL_PROMPT",
            "::>"        => "FINAL_PROMPT",
            "WARNING MINIDISKS" => "FATAL_PROMPT",
        },
    },
    boot_menu_maint => {
        command => "maint",
        options => \%Options,
        connection_type => "console_connection",
        output_handler => "bootmenu_output_handler",
        no_validation => "1",
        additional_prompts_timeout => "210",
        additional_prompts => {
            "login:"     => "root",
            "l.o.g.i"    => "root",
            "l..o..g..i" => "root",
            "lo.g.i"     => "root",
            "lo.g..i"    => "root",
            "l.og.i"     => "root",
            "l.og..i"    => "root",
            "l..og.i"    => "root",
            "l..og..i"   => "root",
            "log.i"      => "root",
            "log..i"     => "root",
            "l.ogi"      => "root",
            "l..ogi"     => "root",
            "g.i.n.:"    => "root",
            "g.in:"      => "root",
            "gin:"       => "root",
            "Password:"  => "admin",
            "Pas"        => "admin",
            "ord:"       => "admin",
            "P.a.s.s"    => "admin",
            "o.r.d.:"    => "admin",
            "Last login:"=> "FINAL_PROMPT",
            "^#"         => "FINAL_PROMPT",
            "[a|b]#"    => "FINAL_PROMPT",
            "\\*>"        => "FINAL_PROMPT",
            "WARNING MINIDISKS" => "FATAL_PROMPT",
            
        },
    },
);

my %Compatability = (
);

sub SetupBootmenuCommands {
    my ($self, @servers) = @_;
    $Trace->entry();

    $self->_define_commands(Definitions => \%CommandDefinitions,
        Compatability => \%Compatability,
        Servers => \@servers);
    $Trace->exit();
    return 1;
}

sub bootmenu_output_handler {
    $Trace->entry();
    
    $Trace->exit();
    return 1;
} 

1;
