=head1 NAME

Compatability.pm

=head1 DESCRIPTION

This module attempts to map changes in the 10M UI to the currently loaded CDEFs.

=head1 SYNOPSIS

This module is instantiated by Server.pm and is not directly called.

=head1 EXAMPLES

No examples needed

=cut

use strict;
use Data::Dumper;
# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
## @author dl-central-libs
my $LibraryLoaded;
if ($ENV{OVERRIDENG_VOLUME}) {
    my $newModule = $ENV{OVERRIDENG_VOLUME};
    delete $ENV{OVERRIDENG_VOLUME};
    print "The main Node::NG::Volume module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

use TracePkg;
my $Trace = new TracePkg;

sub _platform_compatibility {
    my ($self, @modules) = @_;
    my $host = main::host($self->{server});
    my $hosttype = $host->hosttype();
    my $hostname = $host->hostname();
    $Trace->trace(msg=>"The hosttype of $hostname is: '$hosttype'");

    # All platform specifc changes
    my %platform_compat = (
             'filer-ng-sim' => {
                 "setenv" => {
                    command => "set",
                    param_value_seperator => '=',
                 },
                 "unsetenv" => {
                    command => "unset",
                 },
                 "bye" => {
                    command => "reboot",
                 },
                 "cfe_reboot" => {
                    command => "reboot",
                 },
             },);
    
#    $Trace->debug(msg=>"Commands hash is::\n" . Dumper($self->{Commands}));
    foreach my $type (keys(%platform_compat)) {
        $Trace->debug(msg=>"'$type':'$hosttype'");
        if ($type eq $hosttype) {
            $Trace->trace(msg=>"There are platform specifc compatibilty translations that are needed for $hostname!");
            foreach my $cmd (keys(%{$self->{Commands}})) {
                foreach my $plat_cmd (keys(%{$platform_compat{$type}})) {
                    #$Trace->debug(msg=>"'$cmd':'$plat_cmd'");
                    if ($cmd eq $plat_cmd) {
                    #    $Trace->debug(msg=>"'$cmd':'$plat_cmd'");
                        foreach my $param (keys(%{$platform_compat{$type}{$cmd}})) {
                    #        $Trace->debug(msg=>"BEFORE: $self->{Commands}{$cmd}{$param}");
                            $self->{Commands}{$cmd}{$param} = $platform_compat{$type}{$plat_cmd}{$param};
                    #        $Trace->debug(msg=>"AFTER: $self->{Commands}{$cmd}{$param}");
                        }
                    }
                }
            }
        }
    }
    #$Trace->debug(msg=>"Commands hash is::\n" . Dumper($self->{Commands}));
}
#end _platform_compatibility


# 
# Create a %compatability hash in the Node object.  It'll contain elements
# keyed by complete or partial command strings.  Those commands and/or their fields
# will then contain translations for new command or field syntax.  
# Building of the hash is triggered by the changes a filer lists in its
# "filer changes" command.
#
sub _setup_compatability {
    my ($self) = @_;
    my ($cmdRef, $cmd_pattern, $value, %compatability, %compat_mirror, %new_commands);

    # All pre-uichanges type changes
    %compatability = (
        "network_interfaces_routes_" => {
            new_command => "network_interface_route_",
        },
        "network_interfaces_" => {
            new_command => "network_interface_",
        },
        "network_ports_aggregates_" => {
            new_command => "network_port_aggregate_",
        },
        "network_ports_" => {
            new_command => "network_port_",
        },
        "protocols_" => {
            new_command => "protocol_",
        },
        "security_certificates_" => {
            new_command => "security_certificate_",
        },
        "services_nis_domain_" => {
            new_command => "vserver_services_nis_domain_",
        },
        "vserver_cifs_domain_preferred_dc_create" => {
            new_command => "vserver_cifs_domain_preferred_dc_add",
        },
        "vserver_cifs_domain_preferred_dc_delete" => {
            new_command => "vserver_cifs_domain_preferred_dc_remove",
        },
        "services_kerberos_realm_"  => {
            new_command => "vserver_services_kerberos_realm_",
        },
        "services_unix_user_"   => {
            new_command => "vserver_services_unix_user_",
        },
        "services_unix_group_"   => {
            new_command => "vserver_services_unix_group_",
        },
    );
    
    my %all_changes = (
			"group-migrate" => {
			    "network_interface_migrate" => {
				new_command => "network_interface_migrate",
				conversion => {
				    "node"   => "dest-node",
				    "port"   => "dest-port",
				    "filer"  => "dest-node",
				},
			    },
			},
			"typos" => {
			    "protocol_" => {
				new_command => "protocol_",
				conversion => {
				    "v2-enable" => "v2-enabled",
				    "v3-enable" => "v3-enabled",
				    "udp-enable" => "udp-enabled",
				    "tcp-enable" => "tcp-enabled",
				},
			    },
			},
			"filer-to-node" => {
			    "filer_" => {
				new_command => "node_",
			    },
			    "_ALL_COMMANDS" => {
				conversion => {
				    "filer" => "node",
				    "filers" => "nodes",
				    "tofiler" => "tonode",
				    "byfiler" => "bynode",
				    "fromfiler" => "fromnode",
				    "panic-filer" => "panic-node",
				    "filer-uuid" => "node-uuid",
				    "filername" => "nodename",
				    "home-filer" => "home-node",
				    "curr-filer" => "curr-node",
				},
			    },
			},
			"gx-license" => {
			    "cluster_create" => {
				new_command => "cluster_create",
				conversion => {
				    "serialnumber" => "license",
				},
			    },
			},
			"min-snapshot-space" => {
			    "volume_" => {
				new_command => "volume_",
				conversion => {
				    "reserve-percent" => "min-snapshot-space",
				},
			    },
			},
			"vol-rename-mirror-type" => {
			    "volume_rename" => {
				new_command => "volume_rename",
				conversion => {
				    "new-name" => "newname",
				},
			    },
			},
			"sfo-ui-inspection" => {
			    "storage_failover_" => {
				new_command => "storage_failover_",
				conversion => {
				    "nodename" => "node-name",
				    "partnername" => "partner-name",
				    "takeover-by-partner" => "partner-reason",
				    "takeover-by-partner-enum" => "partner-reason-enum",
				},
			    },
			    "storage_failover_interconnect_statistics_error_zeroout" => {
				new_command => "storage_failover_interconnect_statistics_error_zero_out",
			    },
			},
			"snapshot-ui-inspection" => {
			    "volume_snapshot_" => {
				new_command => "volume_snapshot_",
				conversion => {
				    "crtime" => "create-time",
				    "description" => "comment",
				},
			    },
			},
			"export-policy-default" => {
			    "protocol_exportruleset_" => {
				new_command => "protocol_export_policy_",
			    },
			},
			"cli-directories" => {
			    "cluster_" => {
				new_command => "cluster_",
				conversion => {
				    "node-uuid" => "uuid",
				},
			    },
			    "node_" => {
				new_command => "system_",
				conversion => {
				    "node-uuid" => "uuid",
				},
			    },
			    "license_" => {
				new_command => "system_license_",
			    },
			    "protocol_cifs_server_" => {
				new_command => "vserver_cifs_",
			    },
			    "protocols_cifs_server_" => {       ## HACK HACK
				new_command => "vserver_cifs_",
			    },
			    "protocol_nfs_server_" => {
				new_command => "vserver_nfs_",
			    },
			    "protocols_nfs_server_" => {        ## HACK HACK
				new_command => "vserver_nfs_",
			    },
			    "protocol_" => {
				new_command => "vserver_",
			    },
			    "system_service_" => {
				new_command => "system_services_",
			    },
			},
			"export-policy-default-redux" => {
			    "vserver_export_policy_" => {
				new_command => "vserver_export_policy_",
				conversion => {
				    "rulesetname" => "policyname",
				    "rulesetid" => "policyid",
				    "newrulesetname" => "newpolicyname",
				    "newrulesetname" => "newpolicyname",
				},
			    },
			},
			"cifs-nfs-ui-inspection" => {
			    "vserver_cifs_" => {
				new_command => "vserver_cifs_",
			    },
			    "_ALL_COMMANDS" => {
				conversion => {
				    "kerbconfig" => "kerberos-config",
				    "dc1-ip" => "dc1-address",
				    "dc2-ip" => "dc2-address",
				    "dc3-ip" => "dc3-address",
				    "dc4-ip" => "dc4-address",
				    "dc5-ip" => "dc5-address",
				},
			    },
			    "vserver_nfs_kerberos_config_" => {
				new_command => "vserver_nfs_kerberos_config_",
				conversion => {
				    "username" => "admin-username",
				    "password" => "admin-password",
				},
			    },
			},
			"vol-policy-router-metric" => {
			    "network_interface_route_" => {
				new_command => "network_interface_route_",
				conversion => {
				    "hopcount" => "metric",
				},
			    },
			    "volume_" => {
				new_command => "volume_",
				conversion => {
				    "ruleset" => "policy",
				    "ruleset-override" => "policy-override",
				},
			    },
			},
			"nfs-enabled-fields" => {
			    "vserver_nfs_" => {
				new_command => "vserver_nfs_",
				conversion => {
				    "v2-enabled" => "v2",
				    "v3-enabled" => "v3",
				    "udp-enabled" => "udp",
				    "tcp-enabled" => "tcp",
				},
			    },
			},
            "v4.0" => {
                "vserver_nfs_" => {
                new_command => "vserver_nfs_",
                conversion => {
                    "v4" => "v4.0",
                    "v4-acl" => "v4.0-acl",
                    "v4-read-delegation" => "v4.0-read-delegation",
                    "v4-write-delegation" => "v4.0-write-delegation",
                    "v4-acl-preserve" => "v4.0-acl-preserve",
                    "v4-req-open-confirm" => "v4.0-req-open-confirm",
                },
                },
            },
			"dashboard" => {
			    "statistics_" => {
				new_command => "statistics_",
				conversion => {
				    "category-instance" => "object",
				},
			    },
			},          
			"restriping-lite" => {
			    "storage_aggregate_add" => {
				new_command => "storage_aggregate_add_disks",
			    },
			    "storage_aggregate_add_member" => {
				new_command => "storage_aggregate_add_members",
			    },
			    "storage_aggregate_member_add" => {
				new_command => "storage_aggregate_member_add_disks",
			    },
			},
			"vif-to-lif" => {
			    "network_interface_migrate" => {
				new_command => "network_interface_migrate",
				conversion => {
				    "vif" => "lif",
				    "vif_role" => "lif_role",
				    "vif-total" => "lif-total",
				    "vif-online" => "lif-online",
				    "vif-nothome" => "lif-nothome",
				    "vif-no-failover" => "lif-no-failover",
				    "vif-nothosted" => "lif-nothosted",
				    "vif-name" => "lif-name",
				    "vif-id" => "lif-id",
				},
			    },
			    "_ALL_COMMANDS"  => {
				conversion => {
				    "vif" => "lif",
				    "vif_role" => "lif_role",
				    "vif-total" => "lif-total",
				    "vif-online" => "lif-online",
				    "vif-offline" => "lif-offline",
				    "vif-nothome" => "lif-nothome",
				    "vif-no-failover" => "lif-no-failover",
				    "vif-nothosted" => "lif-nothosted",
				    "vif-name" => "lif-name",
				    "vif-id" => "lif-id",
				},
			    },
			},
			"vserver-role-change" => {
			    "vserver_show" => {
				new_command => "vserver_show", 
				conversion => {
				    "role" => "type",
				},
			    },
			},
            "cifs-server" => {
                "vserver_cifs_share_ACL_create" => {
                new_command => "vserver_cifs_share_access_control_create",
                },
                "vserver_cifs_share_ACL_delete" => {
                new_command => "vserver_cifs_share_access_control_delete",
                },
                "vserver_cifs_share_ACL_modify" => {
                new_command => "vserver_cifs_share_access_control_modify",
                },
                "vserver_cifs_share_ACL_show" => {
                new_command => "vserver_cifs_share_access_control_show",
                },
            },
            "volops-directory" => {
                "volume_move" => {
                new_command => "volume_move_start",
                },
                "volume_member_move" => {
                new_command => "volume_member_move_start",
                },
                "volume_copy" => {
                new_command => "volume_copy_start",
                },
                
            },
            "server_to_vserver_network" => {
                "network_interface_" => {
                new_command => "network_interface_",
                conversion => {
                    "server" => "vserver",
               },
               },
                "network_connections_" => {
                new_command => "network_connections_",
                conversion => {
                    "server" => "vserver",
               },
               },
                "network_routing_groups_" => {
                new_command => "network_routing_groups_",
                conversion => {
                    "server" => "vserver",
               },
               },
            },
            "remove-background-from-bg-compression" => {
                "volume_efficiency_" => {
                new_command => "volume_efficiency_",
                conversion => {
                    "background-compression" => "compression",
               },
               },
 	        },        		
        );

#    $Trace->debug(msg=>"Commands hash is::\n" . Dumper($self->{Commands}));
    
    # Create the %compatability hash for all found uichanges
    foreach my $change (@{$self->{uichanges}}) {

        $Trace->debug(msg=>"Processing uichanges for: $change");
        # Look to see if we have a compatability entry for the defined changes
        if ( defined $all_changes{$change} ) {

            #$Trace->debug(msg=>"Changes found for: $change");
            foreach $cmdRef ( \%{$all_changes{$change}} ) {
                while ( ($cmd_pattern, $value) = each %$cmdRef ) {

                    $Trace->debug(msg=>"Processing: $cmd_pattern");
                    # %compatability is keyed by method patterns.  Each element may 
                    # contain a 'conversion' hash that defines field conversions.
                    #
                    # Commands and fields that change multiple times need to be supported
                    # When searching for a methode pattern, there maybe an exact match, 
                    #  or a partial match.  Partial matching is supported at
                    #  word boundries.  A command may be volume_new or volume_

                    # Do we already have this exact match
                    if ( exists $compatability{$cmd_pattern} ) {
                        $Trace->debug(msg=>"Command: $cmd_pattern seen twice, " .
                        	      "merge the two conversion hashes");

                        # This is the second time we've encountered this cmd_pattern.
                        # Merge the 'conversion' fields from the last entry, and overwrite
                        # the 'newcommand' with the latest.
                        if ( exists $value->{conversion} &&
                             exists $compatability{$cmd_pattern}{conversion} ) {
                            $Trace->debug(msg=>"The cmd_pattern: $cmd_pattern already existed, merging");
                            %{$value->{conversion}} = ( %{$compatability{$cmd_pattern}{conversion}}, 
                                        %{$value->{conversion}} );
                        }
                    }
                    # Look to see if this new cmd_pattern (hash key) is part of an existing new_command 
                    # pattern.
                    # If so, update the existing new_comand pattern with this info.
                    foreach my $existing_cmd_pattern ( keys %compatability ) {
                        if ( $compatability{$existing_cmd_pattern}{new_command} =~ /^$cmd_pattern/ ) {
                            $Trace->debug(msg=>"We've already seen new cmd_pattern: $cmd_pattern, " .
                            		  "as a target for $existing_cmd_pattern");
                            $Trace->debug(msg=>"Updating new_command: " . 
                            		  $compatability{$existing_cmd_pattern}{new_command} .
                            		  ", with new pattern: " . $value->{new_command});
                            $compatability{$existing_cmd_pattern}{new_command} =~ 
                            s/$cmd_pattern/$value->{new_command}/;

                            # Now merge the conversion fields into the new entry
                            my %existing_hash = (exists $compatability{$existing_cmd_pattern}{conversion}) ? 
                            %{$compatability{$existing_cmd_pattern}{conversion}} : ();
                            my %new_hash = (exists $value->{conversion}) ? 
                            %{$value->{conversion}} : ();
                            $Trace->debug(msg=>"The cmd_pattern: $cmd_pattern already existed, merging");
                            #$Trace->debug(msg=>"Existing conversion hash: " . Dumper(%existing_hash));
                            #$Trace->debug(msg=>"New conversion hash: " . Dumper(%new_hash));
                            if ($existing_cmd_pattern eq $cmd_pattern) {
                                %{$value->{conversion}} = ( %existing_hash, %new_hash );
                            }
                            # Now update the conversion fields of the old entry
                            %{$compatability{$existing_cmd_pattern}{conversion}} = (%existing_hash, %new_hash);
                        }
                    }

                    # Add or update the cmd_pattern
                    $compatability{$cmd_pattern} = $value;
                    $Trace->debug(msg=>"For compatability{$cmd_pattern}: " . Dumper($value));
                } #end while
            } #end foreach
        } #end if
    } #end foreach
    $Trace->debug(msg=>"The \%compatability hash is: " . Dumper(%compatability));

    # Now for each old->new compat element, create a new->old mirror
    # element.  This is used to stuff the output from commands
    foreach $cmdRef ( keys %compatability  ) {
        if ( defined $compatability{$cmdRef}{conversion} ) {
            while ( my ($old_field, $new_field) = each %{$compatability{$cmdRef}{conversion}} ) {
		$compat_mirror{$cmdRef}{conversion}{$new_field} = $old_field;
            }
        }
    }
#    $Trace->debug(msg=>"compat_mirror:\n" . Dumper( %compat_mirror ));
#    $Trace->debug(msg=>"compatability:\n" . Dumper( %compatability ));

#    $Trace->debug(msg=>"_ALL_COMMANDS to be converted:\n" . Dumper($compatability{"_ALL_COMMANDS"}{conversion}));
    if ( defined $compatability{"_ALL_COMMANDS"}{conversion} ) {
        foreach my $uichange (keys %all_changes) {
            foreach my $oldcommand (keys %{$all_changes{$uichange}}) {
                unless ($oldcommand =~ /(_ALL_COMMANDS|[a-zA-Z0-9]+_)$/) {
                    $Trace->debug(msg=>"Looking at command: '$oldcommand' => '$all_changes{$uichange}{$oldcommand}{new_command}'");
                    if ( (exists($all_changes{$uichange}{$oldcommand}{conversion}))  &&
                         (defined($all_changes{$uichange}{$oldcommand}{conversion})) &&
                         (length($all_changes{$uichange}{$oldcommand}{conversion}))) {
                        $Trace->debug(msg=>"Commands to be converted:\n" . Dumper($all_changes{$uichange}{$oldcommand}{conversion}));
                        my ($uikey, $uivalue, $allkey, $allvalue);
                        my @conversions;
                        while (($uikey, $uivalue) = each(%{$all_changes{$uichange}{$oldcommand}{conversion}})) {
                            push (@conversions, $uikey,$uivalue);
                        }
                        $Trace->debug(msg=>"The conversions are: '@conversions'");
                        while (($allkey, $allvalue) = each(%{$compatability{"_ALL_COMMANDS"}{conversion}})) {
                            $Trace->debug(msg=>"Looking for: '$allkey' or '$allvalue'");
                            if (my $match = grep(/^($allkey|$allvalue)$/, @conversions)) {
                                $Trace->debug(msg=>"Matched on: '$match'");
                                push(@{$self->{compatability}{_ALL_COMMANDS}{conversion}{exclusion}}, $all_changes{$uichange}{$oldcommand}{new_command});
                                push(@{$self->{Commands}{_ALL_COMMANDS}{exclusion}}, $all_changes{$uichange}{$oldcommand}{new_command});
                            }
                        }
                    }
                }
            }
        }
        $Trace->debug(msg=>"ALL_COMMANDS to be excluded:\n" . Dumper($self->{Commands}{_ALL_COMMANDS}{exclusion}));
    }
    


    # Now create a back_compat hash.  Basically for any all_changes element not found
    # in the device's uichanges, create a BACK_<ui_change> with the keys and values
    # reversed.  This is used for converting new script syntax back when running against
    # older product.

    # some code here would be a Good Thing


    # Now create an ordered list of %compatability entries.  Longer keys need
    # to be matched first.
    @{$self->{compat_ordered}} = sort { $b cmp $a } keys %compatability;
    $Trace->debug(msg=>"The ordered list is: " . Dumper( $self->{compat_ordered}));
    # Now stuff the params from the 'conversion' elements of the compatability hash, into
    # the original CDEF.  This is needed because Node::run_command() expects all valid
    # params to be defined in the CDEF.  Which breaks in compatability screnarios.
#    $Trace->debug(msg=>"Before playing with Commands hash:\n" . Dumper($self->{Commands}));
    my $key;
    my @similar_commands;
    my $subsystem;
    foreach $cmdRef (@{$self->{compat_ordered}}) {
        $Trace->debug(msg=>"Command: $cmdRef");
        # For each *like* command, see if it's getting converted to this command
        $subsystem = undef;
        @similar_commands = ();
        if ($cmdRef =~ /^([a-zA-Z0-9]+_\w{2,6})/) {
            $subsystem = $1;
        }
        $Trace->debug(msg=>"***'$cmdRef' : '$subsystem'***");
        unless ($cmdRef eq "_ALL_COMMANDS") {
            if ($subsystem) {
                @similar_commands = grep(/$subsystem/, @{$self->{compat_ordered}});
            }
            unless (grep(/^$cmdRef$/, @similar_commands)) {
                unshift(@similar_commands, $cmdRef);
                $Trace->debug(msg=>"Adding base subsystem $cmdRef");
            }
            $Trace->debug(msg=>"Working on these like commands: @similar_commands");
            foreach $key (@similar_commands) {
                $Trace->debug(msg=>"COMPAT_ORDERED key: $key");
                if ( defined $compatability{$key}{new_command} ) {
                    # OK, so this command is converted from something else, is it the
                    # command we're looking for?
                    if ($cmdRef =~ /^$compatability{$key}{new_command}/ ) {
                        $Trace->debug(msg=>"Found that: $cmdRef is converted from: $key");
#                        $Trace->debug(msg=>"Conversion:\n".Dumper(\$compatability{$key}{conversion}));
                        if ( defined $compatability{$key}{conversion} ) {
                            # Fix up the original Commands hash
                            $self->_update_cdef( $cmdRef, $key,  %compatability );
                        }
                    }
                }
            }
        }
        # Finally update for generic substitutions
        if ( defined $compatability{"_ALL_COMMANDS"}{conversion} ) {
            $self->_update_cdef( $cmdRef, "_ALL_COMMANDS", %compatability );
        }
    }

    # Let's try to fix the _ALL_COMMANDS conversion case.  For each and every CDEF, process the 
    # _ALL_COMMANDS field conversions.
    foreach $cmdRef ( keys %{$self->{Commands}}  ) {
	if ( defined $compatability{"_ALL_COMMANDS"}{conversion} ) {
	    $self->_update_cdef( $cmdRef, "_ALL_COMMANDS", %compatability );
	}
    }

    $self->{compatability} = \%compatability;
    $self->{compat_mirror} = \%compat_mirror;

#    $Trace->debug(msg=>"Commands hash is::\n" . Dumper($self->{Commands}));
}

sub _update_cdef {

    my ($self, $cmdRef, $compatKey, %compatability) = @_;

#    $Trace->debug(msg=>"Entered with cmdRef: $cmdRef, compatKey: $compatKey");
    while ( my ($key, $newKey) = each %{$compatability{$compatKey}{conversion}} ) {
        if ($compatKey =~ /_ALL_COMMANDS/) {
            if (grep(/^$cmdRef$/, @{$self->{Commands}{_ALL_COMMANDS}{exclusion}})) {
                $Trace->debug(msg=>"$cmdRef: IS EXCLUDED FROM CONVERSION!");
                last;
            }
#            $Trace->trace(msg=>"$cmdRef: $key => $newKey");
        }
        # Update the old param key to use the new syntax and/or add the old field to the
        # options hash.
        if ( defined $self->{Commands}{$cmdRef}{options}{$key} ||
             defined $self->{Commands}{$cmdRef}{options}{$newKey} ) { 
#            $Trace->debug(msg=>"Need to fix: " . Dumper( $self->{Commands}{$cmdRef} ));
            $self->{Commands}{$cmdRef}{options}{$key} = "-$newKey";
#            $Trace->debug(msg=>"$key : $newKey");
            # Now squirt the new syntax into the CDEF, so I'll pass validation
            $self->{Commands}{$cmdRef}{options}{$newKey} = "-$newKey";
            # Finally, fix up the specific commands
            if ( grep($_ eq $key, @{$self->{Commands}{$cmdRef}{optional_options}}) ) { 
                push @{$self->{Commands}{$cmdRef}{optional_options}}, $newKey;
            }
            if ( grep($_ eq $key, @{$self->{Commands}{$cmdRef}{required_options}}) ) { 
                push @{$self->{Commands}{$cmdRef}{required_options}}, $newKey;
            }
#            $Trace->debug(msg=>"Fixed up: " . Dumper( $self->{Commands}{$cmdRef} ));
        } else {
#            $Trace->debug(msg=>"Didn't find: $key or $newKey in the options hash");
        }
    }
}


1;


##  Example 
#    my %compatability = (
#                        # Translate just a single command and a field
#			 "network_interfaces_show" => {
#			     new_command => "network_interface_show",
#			 },
#                        # Translate an entire node
#			 "ems_" => {
#			     new_command => "event_",
#			     conversion => {
#				 "old-name" => "messagename",
#				  },
#			 },
#			 );
