# $Id$
## @summary remote programmatic administration of Network functions
## @author dl-central-libs
## @pod here

=head1 NAME

Network.pm - remote programmatic administration of Network functions

=head1 AUTHOR

dl-central-libs

=head1 DESCRIPTION

C<Network> provides routines to handle all aspects of Network
maintenance, such as migrating VIFs back to their original configs

=head1 SYNOPSIS

    All methods are AUTOLOADED by the Server package, so no 
    additional "use" line is necessary.

=head1 ROUTINES

=cut

# The following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one).
my $LibraryLoaded;
if ($ENV{OVERRIDENG_NETWORK}) {
    my $newModule = $ENV{OVERRIDENG_NETWORK};
    delete $ENV{OVERRIDENG_NETWORK};
    print "The main Node::NG::Network module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

use TracePkg;
use Tharn;
use strict;
use Module::Common qw(validate_mixed);
use Params::Validate qw(CODEREF ARRAYREF SCALAR UNDEF OBJECT);


my $Trace = new TracePkg;

our %NetworkConversion = (
		         );
my %Options = (
	      );

my %CommandDefinitions = (
		         );

my %Compatability = (
		    );

sub SetupNetworkCommands {
    my ($self, @servers) = @_;
    $Trace->entry();

    $self->_define_commands(Definitions => \%CommandDefinitions,
                            Compatability => \%Compatability,
                            Servers => \@servers);
    $Trace->exit(msg=>"exiting with 1");
    return 1;
}

#***********************************************************************

=head2 restore_vif_config

    restore_vif_config(failed_node         => "qdeux_6",
		       original_vif_config => $Orig_config,
                      );

C<restore_vif_config> should be used after an event that takes a node down and thus causes 
a VIF failover to occur. This method takes an original config, and for all VIFS originally on
the failed_node, migrates that VIF back to the original location

=over

=item failed_node

(Required) The node that went down

=item original_vif_config

(Required) Pointer to array of hashes detailing the original vif 
configuration (i.e. a pointer to the result of a network_interface_show
call) 

=back

=cut

sub restore_vif_config {

    $Trace->entry();
    my ($self, %params) = validate_mixed(
        @_,
        [ { type => OBJECT, isa => 'Node::NG' } ],
        {
            failed_node               => { type => SCALAR },
            original_vif_config       => { type => ARRAYREF },
        }
    );

    $Trace->debug(msg => "failed_node         is $params{failed_node}");
    $Trace->debug(msg => "original_vif_config is $params{original_vif_config}");
    my $output = "";
    my $current_config = "";
    my $Hash = "";
    my $destination_node = "";

    my $timeout = 100;
    vif_loop: for (my $i=0; $i<=(scalar(@{$params{original_vif_config}}) - 1); $i++) {
	########################################################################
	# We're only interested in data VIF's whose original curr_node was the 
	# failed_node
	########################################################################
        next if ($params{original_vif_config}[$i]{role} ne "data");
        next if ($params{original_vif_config}[$i]{'curr-node'} ne $params{failed_node});
	########################################################################
	# Get this VIF's current location
	########################################################################
        my $elapsedtime =0;           #amount of time that has elapsed since doing a net int show looking for vifs
        my $begtime = time();         #the time when the loop started looking for vifs is started
        my $curtime;                  #the current time while looping
        my $maxtime = 60;             #time to retry the show looking for "something" in the vif table
        while ($elapsedtime <= $maxtime) {  #issue net int show for 60 seconds until it populatest"
           if ($current_config = $self->network_interface_exists(server => $params{original_vif_config}[$i]{server},
	    						         vif    => $params{original_vif_config}[$i]{vif},
							         -Timeout  => 100, 
							         output    => \$output,
						                ) ) {
               $curtime = time();
               $elapsedtime = ($curtime - $begtime);
               $Trace->comment(msg=>"It took $elapsedtime seconds to see the net int show table populate!");
               last;
           } else {
               $curtime = time();
               $elapsedtime = ($curtime - $begtime);
           }
        }
            
        if ($elapsedtime > $maxtime) {
            $Trace->fatal(msg=>"We failed to find vifs in the net int show table after $elapsedtime seconds", result=>"FATAL");                            
        }

        if ( ($$current_config[0]{'curr-node'} ne $params{original_vif_config}[$i]{'curr-node'}) or
	     ($$current_config[0]{'curr-port'} ne $params{original_vif_config}[$i]{'curr-port'}) 
	   ) {
	    $Trace->comment(msg => "VIF $params{original_vif_config}[$i]{vif} is not on its original location. Migrating back ...");
	    $self->network_interface_migrate(server    => $params{original_vif_config}[$i]{server},
                                             vif       => $params{original_vif_config}[$i]{vif},
					     node      => $params{original_vif_config}[$i]{'curr-node'},
					     port      => $params{original_vif_config}[$i]{'curr-port'},
                                             -Timeout  => 100,
                                             output    => \$output,
                                            );
	    ########################################################################
	    # Loop waiting for the VIF to migrate to its new location
	    ########################################################################
	    wait_migrate_loop: while ($timeout) {
	        $current_config = $self->network_interface_show(server => $params{original_vif_config}[$i]{server},
                                                                vif    => $params{original_vif_config}[$i]{vif},
                                                                -Timeout  => 100,
                                                                output    => \$output,
                                                               );
		if ( ($$current_config[0]{'curr-node'}    ne $params{original_vif_config}[$i]{'curr-node'})    or
             	     ($$current_config[0]{'curr-port'}    ne $params{original_vif_config}[$i]{'curr-port'})    or
             	     ($$current_config[0]{'status-admin'} ne $params{original_vif_config}[$i]{'status-admin'}) or
             	     ($$current_config[0]{'status-oper'}  ne $params{original_vif_config}[$i]{'status-oper'})  
           	   ) {
		    $Trace->comment(msg => "Still waiting for VIF $params{original_vif_config}[$i]{vif} to migrate to its original location\n".
					   "Issued ".(100 - $timeout)." 'network interface show' commands",
				   );
		    $timeout--;
		    next wait_migrate_loop;
		} else {
		    $Trace->comment(msg => "VIF $params{original_vif_config}[$i]{vif} is now on its original location");
		    next vif_loop;
		}
	    }
	    if (!$timeout) {
        	$Trace->fatal(msg    => "Timed out waiting for VIF $params{original_vif_config}[$i]{vif} to migrate to its original location and restore its original state",
                      	      result => "FATAL",
                     	     );
            }
        } else {
	    ########################################################################
	    # Since we don't have any auto-revert feature, we should be concerned
	    # if we expected a VIF to failover and it did not. 
	    ########################################################################
            my $failover = $self->network_interface_failover_show(server    => $params{original_vif_config}[$i]{server},
                                                                  vif       => $params{original_vif_config}[$i]{vif},
                                                                 );
            # if there is more than 1 entry, then some failover rules are in place.
            if (scalar(@$failover) > 1) {
                $Trace->warn(msg => "VIF $params{original_vif_config}[$i]{vif} was not failed over as a result of the failure to node $params{failed_node}");
            } else {
                $Trace->comment(msg => "VIF $params{original_vif_config}[$i]{vif} was not setup for failover");
            }
	}
    }
    $Trace->comment(msg=>"we verified that all VIFs original on $params{failed_node} are back in their original location and in their original states");
    $Trace->exit();
}

1;

