#Hey emacs, this is basically -*- perl -*-
## @author dl-central-libs

use strict;
use TracePkg;
my $Trace = new TracePkg;

#
# chk_current_image
#
sub chk_current_image {
    #
    # Takes: an expected image (image1 for example).
    # Returns: 
    #          undef if expected image is the current
    #          the current image name if no expected image passed in
    #
    my $self = shift ();
    my %opt = (
        filer => undef,
        exp_image => "CLI",
        @_,
    );
    my $output;

    $Trace->entry();

# make sure the correct image was booted..
    $output = $self->system_image_show(node=>$opt{filer}, iscurrent=>"true");
    $Trace->debug(msg=>"Current image info:\r\n".Dumper(\$output));
    my $current_image = $output->[0]->{image};
    if (!$opt{exp_image}) {
        return $current_image;
        $Trace->exit();
    }
    if ($current_image ne $opt{exp_image}){
        $Trace->comment(msg=>"The current image is $current_image, not $opt{exp_image} as expected!");
        return $current_image;
        $Trace->exit();
    }
    $Trace->comment(msg=>"The expected image, $opt{exp_image}, was booted...");

    $Trace->exit();
    return;
} # end chk_current_image


#
# chk_state
#
sub chk_state {

    # Takes an expected state
    #
    # Returns UNDEF if node is in the expected state
    # or the node's current state if it's not in the
    # expected state.
    #

    $Trace->entry();
    my $self = shift ();
    my %opt = (
        expected_state => "CLI",
        @_,
    );
    my $expected_state = $opt{expected_state};

    my $current_state = $self->state();
    $Trace->debug(msg => "\$current_state is: $current_state");
    if (ref($expected_state)){
        $Trace->debug(msg=>"$expected_state is a ref...");
        my $match;
        foreach my $state (@{$expected_state}) {
            $Trace->debug(msg=>"Checking $state...");
            if ($current_state eq $state) {
               $match = "true"; 
               $Trace->debug(msg=>"$state matches $current_state...");
               last;
            }
        }
        if (!$match) {
            $Trace->comment(msg=>"Filer is not in the expected state!");
            $Trace->exit();
            return $current_state;
        }
    } elsif ($current_state ne $expected_state) {
        $Trace->comment(msg=>"Filer is not in the expected state of $expected_state!");
        $Trace->exit();
        return $current_state;
    }
    $Trace->exit(msg=>"Filer is in the expected state of $current_state.");
    return;
} # end chk_state

sub chk_bsd_mounts {
    $Trace->entry();
    my $self = shift ();
    my $output;
    my $missing = [];
    my %mounts = ( 
	# mountpoint => [ device, mount options ]
	"cfcard" => [ "/dev/ad4s1", "\\(msdosfs, local\\)" ],
        "dev" => [ "devfs", "\\(devfs, local\\)" ],
        "var" => ["/dev/nvrd1", "\\(ufs, local, synchronous\\)"],
        "proc" => [ "procfs", "\\(procfs, local\\)"],
        "mroot" => [".*", "\\(spin\\)"],
    );
   
    # run bsd's mount command
    $self->shell(cmd=>"mount", output=>\$output);

    # find mroot's ID
    my $mrootID;
    if ($output =~ /(.*)\s+on\s+\/mroot/){
        $mrootID = $1;
        $Trace->debug(msg=>"mroot ID: $mrootID");
    }

    # parse output of mount command
    while ( (my $mnt_point, my $fs) = each %mounts ) {
        $Trace->comment(msg=>"Checking: $mnt_point...");
        if ($output =~ /$$fs[0]\s+on\s+\/$mnt_point\s+$$fs[1]/){
            if ($mnt_point eq "mroot") {
                $Trace->debug(msg=>"$mrootID is mounted on \/$mnt_point. Options: $$fs[1]\n");
            } else {
                $Trace->debug(msg=>"$$fs[0] is mounted on \/$mnt_point. Options: $$fs[1]\n");
            }
        } else {
            $Trace->debug(msg=>"$mnt_point is NOT mounted!\n");
            push @$missing, $mnt_point;
        } 
    }
    if (@$missing){
        $Trace->debug(msg=>"One or more mounts is missing!\n");
        foreach my $m (@$missing){
            $Trace->debug(msg=>"$m is NOT mounted!\n");
        }
        $Trace->exit();
        return $missing;
    }

    $Trace->exit(msg=>"Returning undef...");
    return undef;
}

1;
