#***********************************************************************
#
# $Id$
#
# NAME
#   Topology.pm - remote programmatic administration of a Filer
#
# DESCRIPTION
#
# METHODS
#
# USAGE
#   To be filled in later...
# 
# AUTHOR
#    dl-central-libs
#
#***********************************************************************

=head1 NAME

Topology.pm

=head1 AUTHOR

dl-central-libs

=head1 SYNOPSIS

Remote programmatic administration of a Filer

=cut

# the following chunk of code allows this module to be overridded with another
# one if the variable is set. If it is set then print a warning, delete the ENV
# variable so that it is not seen in the new module, load the new one, and
# set the flag that indicates that we have already loaded the module (so that
# we don't load the new one
#

# Please Note: "network interface routes ..." has been removed from this module
# in order to support routing-groups in BR.0 and beyond. Please use 
# lib/VifMgr/VifConfig.pm to configure routes and/or routing groups in your tests.

my $LibraryLoaded;
if ($ENV{OVERRIDENG_TOPOLOGY}) {
    my $newModule = $ENV{OVERRIDENG_TOPOLOGY};
    delete $ENV{OVERRIDENG_TOPOLOGY};
    print "The main Node::NG::Topology module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

use TracePkg;
my $Trace = new TracePkg;

our @Cmd_nodes = qw(cluster filer disk aggregate vserver network_ports 
                    network_interfaces nfs nis name_mapping kerberos 
                    exportruleset exportruleset_rules volume volume_mirror 
                    cifs_server cifs_share cifs_superuser avoa_policy);
our %Cmd_node_definitions = (
			    discover => {
				cluster => "cluster_identity_show",
				filer => "filer_show",
				disk => "storage_disk_show",
				aggregate => "storage_aggregate_show",
				sfo => "storage_failover_show",
				vserver => "vserver_show",
				network_ports => "network_ports_show",
				network_interfaces => "network_interfaces_show",
				exportruleset => "protocol_exportruleset_show",
				exportruleset_rules => "protocol_exportruleset_rule_show",
				volume => "volume_show",
				volume_mirror => "volume_mirror_show",
				nfs => "protocol_nfs_server_show",
				nis => "services_nis_domain_show",
				name_mapping => "protocol_name_mapping_show",
				kerberos => "services_kerberos_realm_show",
				cifs_server => "protocol_cifs_server_show",
				cifs_share => "protocol_cifs_share_show",
				cifs_superuser => "protocol_cifs_superuser_show",
				schedule_cron => "job_schedule_cron_show",
				member_aggregate => "storage_aggregate_member_show",
				avoa_policy => "antivirus_on_access_policy_show",
			    },
			    create => {
				cluster => "create_cluster",
				#aggregate => "storage_aggregate_new",
				aggregate => "create_aggregate",
				vserver => "vserver_new",
				network_interfaces => "network_interfaces_new",
				exportruleset => "protocol_exportruleset_new",
				exportruleset_rules => "protocol_exportruleset_rule_new",
				volume => "volume_new_and_modify",
				volume_mirror => "volume_mirror_new",
				nfs => "protocol_nfs_server_new",
				nis => "services_nis_domain_new",
				name_mapping => "protocol_name_mapping_new",
				kerberos => "services_kerberos_realm_new",
				cifs_server => "protocol_cifs_server_new",
				cifs_share => "protocol_cifs_share_new",
				cifs_superuser => "protocol_cifs_superuser_new",
				avoa_policy => "antivirus_on_access_policy_create",
			    },
			    exists => {
				cluster => "cluster_exists",
				aggregate => "storage_aggregate_exists",
				vserver => "vserver_exists",
				network_interfaces => "network_interface_exists",
				exportruleset => "protocol_exportruleset_exists",
				exportruleset_rules => "protocol_exportruleset_rule_exists",
				volume => "volume_exists",
				volume_mirror => "volume_mirror_exists",
				nfs => "protocol_nfs_server_exists",
				nis => "services_nis_domain_exists",
				name_mapping => "protocol_name_mapping_exists",
				kerberos => "services_kerberos_realm_exists",
				cifs_server => "protocol_cifs_server_exists",
				cifs_share => "protocol_cifs_share_exists",
				cifs_superuser => "protocol_cifs_superuser_exists",
				avoa_policy => "antivirus_on_access_policy_exists",
			    },
			    delete => {
				aggregate => "storage_aggregate_delete",
				vserver => "vserver_delete",
				network_interfaces => "network_interfaces_delete",
				exportruleset => "protocol_exportruleset_delete",
				exportruleset_rules => "protocol_exportruleset_rule_delete",
				volume => "volume_purge",
				volume_mirror => "volume_mirror_delete",
				nfs => "protocol_nfs_server_delete",
				nis => "services_nis_domain_delete",
				name_mapping => "protocol_name_mapping_delete",
				kerberos => "services_kerberos_realm_delete",
				cifs_server => "protocol_cifs_server_delete",
				cifs_share => "protocol_cifs_share_delete",
				cifs_superuser => "protocol_cifs_superuser_delete",
				avoa_policy => "antivirus_on_access_policy_delete",
			    },
			    );

sub cluster_exists {
    my $self = shift;
    my %cluster = (
		   Topology=>undef, # intentionally uppercase to not collide with inputs
		   @_,
		   );

    $Trace->entry();

    my $topology = $cluster{Topology};

    # find the cluster port MTUs and add the entries to the cluster params
    eval {
        my @cluster = $self->cluster_identity_show();
    }; 

    #
    # If the cluster doesn't exist, cluster_identity_show will die.  Since
    # we want information about cluster existence and not an error, return 
    # undef() instead.
    #
    if ($@) {
        $Trace->exit();
        return undef();
    } 

    $Trace->exit();
    return ($cluster[0]{cluster-name} eq $topology{cluster}{cluster-name});
}

#sub show_cluster {
#    my $self = shift;
#    my %cluster = (
#                   Topology=>undef, # intentionally uppercase to not collide with inputs
#                   @_,
#                   );
#    my $topology = $cluster{Topology};
#    $Trace->entry();

#    # find the cluster port MTUs and add the entries to the cluster params
#    my $cluster = $self->cluster_identity_show();
#    $Trace->exit();
#}

sub create_aggregate {
    my $self = shift();
    my %params = @_;

    $Trace->entry();

    if (exists($params{disklist})) {
        delete($params{disklist});
    }

    $self->storage_aggregate_new(%params);

    $Trace->exit();
}

sub create_cluster {
    my $self = shift;
    my %params = (
		  Topology=>undef, # intentionally uppercase to not collide with inputs
		  @_,
		   );
    my $topology = $params{Topology};
    $Trace->entry();

    my $serv = new Server();

    # find the first cluster member and perform the createcluster on it
    my %cluster;

    # get the common cluster information
    $cluster{serialnumber} = $params{"serialnumber"};
    $cluster{clustername} = $params{"name"};
    
    my $filerh = shift @{$topology->{filer}};
    $self->_gather_cluster_info($filerh->{filer}, $topology, \%cluster);
    $serv->cluster_create(-Server => $filerh->{filer},  %cluster, timeout => 240);
    
    # the remaining joins need a cluster ip
    my $cluster_ip = $cluster{ipaddr1};

    # now join the remaining filers into the cluster
    foreach my $filerh (@{$topology->{filer}}) {
	%cluster = ();
	$cluster{clusteripaddr} = $cluster_ip;
	$self->_gather_cluster_info($filerh->{filer}, $topology, \%cluster);
	$serv->cluster_join(-Server => $filerh->{filer}, %cluster, timeout => 240);
    }
    
    $Trace->exit();
}

sub _gather_cluster_info {
    my $self = shift;
    my $filer = shift;
    my $topology = shift;
    my $cluster = shift;

    $Trace->entry();
    #$cluster->{filername} = $filer;

    # find the cluster port MTUs and add the entries to the cluster params
    foreach my $net_port (@{$topology->{network_ports}}) {
	if ($net_port->{role} eq "cluster" && $net_port->{port} eq "e0a") {
	    $cluster->{mtu1} = $net_port->{mtu};
	}
	if ($net_port->{role} eq "cluster" && $net_port->{port} eq "e0b") {
	    $cluster->{mtu2} = $net_port->{mtu};
	}
    }

    # find the cluster port IPs/masks and add the entries to the cluster params
    foreach my $interface (@{$topology->{network_interfaces}}) {
        $Trace->debug(msg => "Looking at interface: role:".$interface->{role}.", home-port:".$interface->{'home-port'}." home-filer:".$interface->{'home-filer'}.". Trying to find an interface for $filer");
	if ($interface->{role} eq "cluster" && $interface->{"home-port"} eq "e0a" && $interface->{'home-filer'} eq $filer) {
	    $cluster->{ipaddr1} = $interface->{address};
	    $cluster->{netmask1} = $interface->{netmask};
	}
	if ($interface->{role} eq "cluster" && $interface->{"home-port"} eq "e0b" && $interface->{'home-filer'} eq $filer) {
	    $cluster->{ipaddr2} = $interface->{address};
	    $cluster->{netmask2} = $interface->{netmask};
	}
    }

    if (!defined($cluster->{ipaddr1}) && !defined($cluster->{ipaddr2})) {
        $Trace->fatal(msg => "Cannot find any cluster interfaces in topology: ".Dumper($topology->{network_itnerfaces}), result => 'CONFIG');
    }

    $Trace->exit();
}
1;
