# $Id: //depot/prod/test/Rsierranevada/lib/OntapSSH/SecureAdmin.pm#1 $
#
# Copyright (c) 2006 Network Appliance, Inc.
# All rights reserved.
#
## @summary Library of functions to configure and test Ontap SSH administration 
## @author Dave Anderson (davea@netapp.com)
## @status public
## @pod here


package OntapSSH::SecureAdmin;
use strict;
use misc;
use Tharn;
use Data::Dumper;
require "tharnit_lib.pl";
use OntapSSH qw( 
    backup_config
    change_keysize
    check_mount_point
    compare_config
    create_ssh_sessions
    get_client
    get_client_name
    get_config
    get_debug_module_verbose
    get_debug_ssh_verbose
    get_filer_ipv6
    get_hostname
    get_password
    get_ssh1_enable
    get_ssh1_hostkeysize
    get_ssh1_serverkeysize
    get_ssh2_enable
    get_ssh2_hostkeysize
    get_ssh_access
    get_ssh_connect_auth_method
    get_ssh_connect_protocol
    get_ssh_enable
    get_ssh_idle_timeout
    get_ssh_passwd_auth_enable
    get_ssh_port
    get_ssh_pubkey_auth_enable
    get_stale_opt
    make_sure_ssh_connect_fails_and_log_result
    make_sure_ssh_connect_succeeds_and_log_result
    restore_config
    run_command_over_ssh
    run_command_after_login_to_filer_using_ssh
    run_command_debug_enable
    set_debug_module_verbose
    set_debug_ssh_verbose
    set_filer_ipv6
    set_password
    set_ssh1_enable
    set_ssh1_hostkeysize
    set_ssh1_serverkeysize
    set_ssh2_enable
    set_ssh2_hostkeysize
    set_ssh_access
    set_ssh_connect_auth_method
    set_ssh_connect_protocol
    set_ssh_enable
    set_ssh_idle_timeout
    set_ssh_passwd_auth_enable
    set_ssh_port
    set_ssh_pubkey_auth_enable
    set_ssh_alternate_username
    set_ssh_alternate_password
    set_ssh_alternate_ip
    set_ssh_sessions
    ssh_can_connect
    ssh_client_config_backup
    ssh_client_config_restore
    ssh_client_rm_known_hosts
    ssh_keygen
    verify_max_ssh_sessions
);
    
use vars qw(@ISA @EXPORT );



use Exporter;
@ISA = qw(Exporter OntapSSH);
#our @EXPORT_OK = qw(ssh_config ssh_backup ssh_restore);
#->our @EXPORT_OK = qw( get_config backup_config restore_config );
our @EXPORT_OK = qw( 
    change_keysize
    check_mount_point
    compare_config
    get_client
    get_client_name
    get_config
    get_debug_module_verbose
    get_debug_ssh_verbose
    get_filer_ipv6
    get_hostname
    get_password
    get_ssh1_enable
    get_ssh1_hostkeysize
    get_ssh1_serverkeysize
    get_ssh2_enable
    get_ssh2_hostkeysize
    get_ssh_access
    get_ssh_connect_auth_method
    get_ssh_connect_protocol
    get_ssh_enable
    get_ssh_idle_timeout
    get_ssh_passwd_auth_enable
    get_ssh_port
    get_ssh_pubkey_auth_enable
    get_stale_opt
    make_sure_ssh_connect_fails_and_log_result
    make_sure_ssh_connect_succeeds_and_log_result
    set_debug_module_verbose
    set_debug_ssh_verbose
    set_filer_ipv6
    set_password
    set_ssh1_enable
    set_ssh1_hostkeysize
    set_ssh1_serverkeysize
    set_ssh2_enable
    set_ssh2_hostkeysize
    set_ssh_access
    set_ssh_connect_auth_method
    set_ssh_connect_protocol
    set_ssh_enable
    set_ssh_idle_timeout
    set_ssh_passwd_auth_enable
    set_ssh_port
    set_ssh_pubkey_auth_enable
    set_ssh_alternate_username
    set_ssh_alternate_password
    set_ssh_alternate_ip
    set_ssh_sessions
    ssh_can_connect
    ssh_client_config_backup
    ssh_client_config_restore
    ssh_client_rm_known_hosts
    ssh_keygen
    update_filer_settings
);

=head1 NAME

OntapSSH::SecureAdmin - Functions for setting SSH parameters on filer,
including key generation, and for testing SSH connectivity.

=head1 SYNOPSIS

Note: C<$CLIENT> B<MUST> be a UNIX machine.

See the example scripts linked in the "SEE ALSO" section beginning with
"_sample_*" for example test scripts and basic instructions on writing tests.
This document is primarily intended for library maintainers.

Note that the methods backup_config() and restore_config() are only used in
the wrapper script to preserve the configuration on the filer prior to
invoking subtests, and to restore the configuration after subtests are
complete.

=head1 DESCRIPTION

Note: C<$CLIENT> B<MUST> be a UNIX machine.

This module defines routines used to modify SSH-related settings on the filer
and validate whether SSH may be used to connect to the filer using various
authentication schemes and protocol versions.

Please see the document for OntapSSH.pm which contains documentation for
the many methods which are inherited by this module.

Note that the methods backup_config() and restore_config() are only used in
the wrapper script to preserve the configuration on the filer prior to
invoking subtests, and to restore the configuration after subtests are
complete.

This module is still in beta status, testers welcome, but please be warned
that the behavior and interface may change.

This form is used in the wrapper script:

    use OntapSSH qw( get_config backup_config restore_config );

    $cfg = get_config($FILER);
    $rv = $cfg->backup_config();
    $rv = $cfg->restore_config();


This form is used in the subtest scripts:

    use OntapSSH::SecureAdmin qw( 
        get_config
        backup_config
        .
        .
        .
        get_debug_ssh_verbose
        get_debug_module_verbose
    );
    

    
    # For working examples,  please see the sample scripts mentioned in the
    # "SEE-ALSO" section.
    

=head1 FUNCTIONS

All functions, unless otherwise stated, return a true value on success, and
false on failure.


=cut


=head2 update_filer_settings()

Updates the SSH configuration settings on C<$FILER> with values stored in
the config object. This implementation uses secureadmin wherever possible to
make changes on the filer, verifying them with the options command wherever
possible. For an alternate implementation see Options.pm. Returns the config
object that was passed in, with the exception that $self->get_stale_opt() is
set to undef on success. If $self->get_stale_opt() is still true after
calling this function, then the update failed.



B<Usage:> C<$cfg-E<gt>update_filer_settings($setting)>

=cut

sub update_filer_settings {

    my $self = shift;
    my $rslt = undef;   # Scalar to hold return value

    my $dbg = $self->{'_debug_module_verbose'}; # Set Debug flag

    $dbg ? logcomment("Entering SecureAdmin::update_filer_settings()") : "";

    $self->change_keysize();
    
    if ( $self->{'_stale_opt'} ) { # Skip if no changes
    
        cli($self->{'_filer'}, "secureadmin disable all");
        cli($self->{'_filer'}, "options ssh.access $self->{'_ssh_access'}");
        cli($self->{'_filer'}, "options ssh.passwd_auth.enable $self->{'_ssh_passwd_auth_enable'}");
        cli($self->{'_filer'}, "options ssh.pubkey_auth.enable $self->{'_ssh_pubkey_auth_enable'}");
        cli($self->{'_filer'}, "options ssh.port $self->{'_ssh_port'}");

        if ( $self->{'_ssh_enable'} ne $self->{'_ssh2_enable'} ) {
        
            logresult("SCRIPT", "ssh.enable and ssh2.enable cannot be set to different values.");
        }
        
        # Convert "on/off" stored in config object to "enable/disable"
        # or bogus value, then set value on filer
        
        if ( $self->{'_ssh1_enable'} eq "on" ) {
        
            cli($self->{'_filer'}, "secureadmin enable ssh1");
        }
        
        elsif ( $self->{'_ssh1_enable'} eq "off" ) {
        
            cli($self->{'_filer'}, "secureadmin disable ssh1");
        }
        
        else {
        
            # Pass value for negative test case
            cli($self->{'_filer'}, "secureadmin " . $self->{'_ssh1_enable'} . " ssh1");
        }
        
        # Convert "on/off" stored in config object to "enable/disable"
        # or bogus value, then set value on filer
        
        if ( $self->{'_ssh2_enable'} eq "on" ) {
        
            cli($self->{'_filer'}, "secureadmin enable ssh2");
        }
        
        elsif ( $self->{'_ssh2_enable'} eq "off" ) {
        
            cli($self->{'_filer'}, "secureadmin disable ssh2");
        }
        
        else { 
        
            # Pass value for negative test case
            cli($self->{'_filer'}, "secureadmin " . $self->{'_ssh2_enable'} . " ssh2");
        }

        cli($self->{'_filer'}, "options ssh.idle.timeout "
                                        ."$self->{'_ssh_idle_timeout'}");

        # Convert "on/off" stored in config object to "enable/disable"
        # or bogus value, then set value on filer
        
        if ( $self->{'_ssh_enable'} eq "on" ) {
        
            cli($self->{'_filer'}, "secureadmin enable ssh");
        }
        
        elsif ( $self->{'_ssh_enable'} eq "off" ) {
        
            cli($self->{'_filer'}, "secureadmin disable ssh");
        }
        
        else { 
        
            # Pass value for negative test case
            cli($self->{'_filer'}, "secureadmin " . $self->{'_ssh_enable'} 
                                                                    . " ssh");
        }
        
    }
    
    else {
    
        # Keys were not changed -- drop through & log comment
        logcomment("Options are same as filer settings, will not change");
    }

    # Since compare_config() already verifies filer options against the config
    # object, we don't need to do anything special here to verify filer
    # settings using options & registry commands; just call compare_config().
    
    if ( $self->compare_config() ) {
    
        $dbg ? logcomment("update_filer_settings() succeeded!") : "";
        $self->{'_stale_opt'} = undef;
        
        $rslt = "1";
    }
    
    else {
    
        logcomment("WARN", "update_filer_settings() Failed!");
        
        $rslt = undef;
    }
    
    # change flag to reflect whether config object matches filer settings
    
    $dbg ? logcomment("Leaving SecureAdmin::update_filer_settings()") : "";
    
    return $self;
}



=head1 SEE ALSO

=for html
<b>
<a href="./lib-OntapSSH-pm.html">OntapSSH::SecureAdmin</a><br>
<a href="./lib-OntapSSH-Options-pm.html">OntapSSH::Options</a><br>
<a href="./SSH-_sample_wrapper-thpl.html">../SSH/_sample_wrapper.thpl</a><br>
<a href="./SSH-_sample_Options-thpl.html">../SSH/_sample_Options.thpl</a><br>
<a href="./SSH-_sample_SecureAdmin-thpl.html">../SSH/_sample_SecureAdmin.thpl</a><br>
<a href="./SSH-_sample_tests-thpl.html">../SSH/_sample_tests.thpl</a><br>

<b>


=head1 HISTORY

=head1 TODO

=over 

=item * Add handling for negative test cases via <attr>_bogus flags

=item * Need check for idle timeout and access function

=item * TBD after code review

=back

=cut

=head1 COPYRIGHT

Copyright (c) 2006 Network Appliance, Inc.

=cut

1;
