#Hey emacs, this is basically -*- perl -*-
#!/usr/local/bin/perl
# $Id$
## @attributes listtestcases=yes
## @status TBD
## @author kwhitney@netapp.com,anas@netapp.com

#####################################################################
# Script: BSDTester.pm                                              #
# Description: A generic library for executing FreeBSD (MHOST)      #
#              resident test code                                   #
# @change    : Initial contents and check-in                        #
# Release:   SN.0 + X                                                 #
#####################################################################

##  @summary : A generic library for executing FreeBSD (MHOST) resident test code
##
##
## @synopsis
##   ntest {scriptname of script using Os::BSDTester.pm} {args}
##
## @usage
##	my $result = runMhostTest(\%TestDescriptions, \%Parameter_defs);
##
## 1)      Fill out the perl HASH describing your command line test in the TESTS part of the  TestDescriptions hash
## 2)      Fill out the order of tests in the CONTROL part of the TestDescriptions hash
## 3)      Decide what the max time out (seconds) is and fill in "TEST_EXECUTION_TIMEOUT"
## 4)      Add any default and/or command line parameters to the Params data structure
##
## Note: Any command line parameter, like FILER or MYTESTTIMEOUT can be used in the text of the
##       %Testdescriptions structure.  For instance, if you have FILER=fred.nane.netapp.com, then
##       putting %FILER% into any text will be replaced with "fred.nane.netapp.com".
##
## @Example
## my %TestDescriptions = (
##    tests       =>{
##            mytest   =>{
##                       'description'            => "Test example: my filer is %FILER%",
##                       'sourcelocation'        => /u/kwhitney/perltests/mytest<94>,
##
## Will be translated to this at run time:
##
## ## ## my %TestDescriptions = (
##    tests       =>{
##            mytest   =>{
##                       'description'            => "Test example: my filer is fred.nane.netapp.com",
##                       'sourcelocation'        => /u/kwhitney/perltests/mytest<94>,
##
##
## Steps to use this module:
##
## Create the main:
##
## use Os::BSDTester;
##
## &main();
##
## sub main {
##    my $result = runMhostTest(\%TestDescriptions, \%Parameter_defs);
##    return $result;
##}
##
## The smallest TestDescriptions hash looks like this.  <93>setupsteps<94>, <93>executionsteps<94> and <93>cleanupsteps<94> are FreeBSD command line operations.  Actually they are lists of operations separated by commas.   Any failure of any of the command line operations fails the whole test.
##
## my %TestDescriptions = (
##    tests       =>{
##            mytest   =>{
##                       'description'            => "Test example",
##                       'sourcelocation'        => /u/kwhitney/perltests/mytest<94>,
##                       'destinationlocation'    => "/tmp,
##                       'setupsteps'             => undef,
##                       'executionsteps'        => [ /tmp/mytest ],
##                       'cleanupsteps'            => undef,
##                        },
##    	            },
##    	control     =>{
##		testorder   =>{  ['mytest']  }
##		}
##   );
##
## The smallest Parameter_defs is:
##
## my %Parameter_defs = (definitions =>
##               [
##               {
##                name          => "FILER",
##                description   => "Controller to use",
##                required      => "FILER is a required parameter"
##               },
##               {
##                name          => "TEST_EXECUTION_TIMEOUT",
##                description   => "Timeout for the executation of the C programs",
##                default       => "900"
##               },
##               {
## 		name          => "VERBOSE",
## 		description   => "Verbosity switch",
## 		default       => "1"
##		}
##		]);
##

###################################################################
##   Enhancements that have to be considered in the near future
###################################################################
## List possible enhancements as needed :
package Os::BSDTester;

## Compiler Directives.
use strict;

## Import Modules

use Tharn;
use TCD;
use Data::Dumper;

use Params::Validate qw(validate_pos validate_with ARRAYREF HASHREF);
use Module::Common;
use Options;

#use Os::RunningStat;
use TestUtils qw(get_site_info);
use NACL::APISet;
use NACL::Transit;
use File::Basename;
use Data::Dumper;
use Os::EZIO_BASE;
use Os::EZIO_DT_CIFS;
use Os::EZIO_HMR_CIFS;

#use Env qw(PERL5LIB);

## Import NACL Execptions
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::Transit::Exceptions::TransitException qw(:try);
use NACL::Transit::Exceptions::RetriesExceeded qw(:try);

package main;
## Globals
use vars qw($Bsdobj
  $Cooked_name
  $Release
  $Filer
  $h
  $Timeout
  @Transit_args
  $Transit_obj
  $Download_server
  $Verbose
  $Version_manager
  $Test
  $Params
  $_Testcases
  $_staf
  $_dt_client
  $_hmr_client
  $_job_mode
);

## Testcase definitions
my $_Testcases      = \();
my $_Parameter_defs = \();

sub set_config {
    my (%config) = @_;
    $_Testcases = %config;
}

sub get_config {
    return $_Testcases;
}

sub get_params {
    return $_Parameter_defs;
}

sub runMhostTest {
    my ( $tc, $pd ) = @_;
    $_Testcases      = $tc;
    $_Parameter_defs = $pd;
    my $returncode = $TCD::PASS;
    ## Debugger break point
    ## Do not change

    $DB::single = 2;
    my $tests         = get_config()->{'control'}->{'testorder'};
    my @TCD_Testcases = ();
    {
        no strict 'refs';
        foreach my $testname (@$tests) {
            eval {
                *{$testname} = sub { return generic_test("$testname"); };
            };
            if ($@) {
                print("FATAL: $@\n");
                return $TCD::CONFIG;
            }
            push(
                @TCD_Testcases,
                (
                    $testname, get_config()->{'tests'}{$testname}{'description'}
                )
            );
        }
        foreach my $testname (qw(init setup cleanup uninit)) {
            eval {
                my $internalname = "_" . $testname;
                *{$testname} = sub { return &$internalname; };
            };
            if ($@) {
                print("FATAL: $@\n");
                return $TCD::CONFIG;
            }
        }
    }

    ## Create TCD Object
    $Test = new TCD( -testcases => [@TCD_Testcases] );
    if ( $Test->error ) {
        $Test->nlog( "FAIL: " . $Test->errmsg );
        return $TCD::FAIL;
    }
    $Test->nlog( "Timeout: " . $Timeout );
    ## Performs method callbacks
    $Test->run_test();
    if ( $Test->error ) {
        $Test->nlog( "Test run failed with: " . $Test->errmsg );
        return $TCD::FAIL;
    }

    exit(0);
}    # end main()

########################################################################
# INIT #################################################################
########################################################################

sub _init {

    ## Local Variables
    my $hosttype       = undef;
    my $original_state = undef;
    $_job_mode = undef;

    #Am I a PREJOB, POSTJOB or NORMALTEST?
    $_job_mode = Tharn::param('PRE_POST_TYPE') || 'NORMALTEST';

    my $site      = undef;
    my %site_info = ();

    # Options processing
    my $Options = new Options();

    ## CLI args definitions
    $Options->parameters( get_params() );

    ## Process the CLI args
    $Options->process();
    $Params = $Options->get_hash();

    ## Store the CLI args into
    ## variables
    $Filer = $Params->{FILER};

    $Timeout = $Params->{TEST_EXECUTION_TIMEOUT};
    $Verbose = $Params->{VERBOSE};

    ## Get the host object handle for the Filer
    ## that was provided by the user
    $h        = host($Filer);
    $hosttype = $h->hosttype();
    $site     = $h->SITE();

    $Test->nlog("\n-Using filer $Filer");

    ## Fetch the ftp server from the site_info
    ## file
    eval { %site_info = TestUtils::get_site_info(); };

    if ($@) {
        $Test->nlog(
            "Unable to read the contents of the site info file. \n " . $@ );
        return $TCD::CONFIG;
    }

    ## Read the download server from the site info file
    $Download_server = $site_info{$site}->{netboot_server};
    $Download_server = 'ftp://' . $Download_server;

    ## Create the NACL::Transit object
    $Transit_obj = NACL::Transit->new( name => $Filer );

    ## Get the original state of the Filer
    $original_state = $Transit_obj->get_state();
    $Test->nlog("\n- The initial state of the Filer is : $original_state\n");

    ## Transit to CLI is the Filer is not already there
    ## prior to creating the APISet object
    if ( $original_state ne 'CLI' ) {
        $Test->nlog("- The Filer was not in the CLI state");
        $Test->nlog("- Changing the state to CLI");
        $Transit_obj->change_state( to => 'CLI' );
    }
    ## CMode code path
    my $filer_version = "unknown";
    if ( $hosttype =~ /ng/ ) {
        $filer_version = "CMode";
    }
    elsif ( !( $hosttype =~ /ng/ ) ) {
        $filer_version = "7Mode";
    }
    else {
        $Test->nlog("-  $hosttype  is not a recognized host type");
        return $TCD::CONFIG;
    }

    $Test->nlog(
        ' This is a ' . $filer_version . ' Filer with host type ' . $hosttype );

    ## Create the $filer_version Version object
    $Test->nlog("Creating the $filer_version Version object");
    $Version_manager = NACL::APISet::Version->new(
        hostobj    => $h,
        category   => "Node",
        shell_type => $filer_version
    );
    ## Create the FreeBSD APISet object
    $Test->nlog("Creating the $filer_version APISet object");
    $Bsdobj = NACL::APISet->new(
        hostobj    => $h,
        category   => "Node",
        interface  => "CLI",
        set        => "Systemshell",
        shell_type => $filer_version,
    );
    my $output = $Bsdobj->execute_raw_command(
        'command'            => 'sudo csh',
        'connectrec-timeout' => $Timeout,
    );
    ## Get the raw release name of the controller
    $Release =
      $Version_manager->get_version_attribute( attribute => 'release' );

    ## Map the codename to the release name
    $Cooked_name =
      $Version_manager->map_release_to_codename( release => $Release );

    $Test->nlog("Release: $Release ($Cooked_name)");
    my @cmds = ["uname -a"];

    ## Get machine info (uname stuff) for logs
    my $result_code;
    my $total_output;
    ( $result_code, $total_output ) = _execute_list_of_commands(@cmds);
    $Test->nlog("Machine Info: $total_output");

    return $TCD::PASS;

}    # end init()
########################################################################
# SETUP ################################################################
########################################################################
sub _setup {

    $_dt_client  = undef;
    $_hmr_client = undef;

    $Test->description("Setup filer $Filer and any clients");

    ## Verify that the Filer has a supported release
    $Test->nlog("\n- The release of Filer $Filer is $Release");

    my $cifs_server_name = $Filer;
    $cifs_server_name =~ s/-//g;
    $cifs_server_name = uc($cifs_server_name) . 'VS0';
    my $cifs_sharename = $Params->{"OSKQA_SHARENAME"};
    $Test->nlog( "CIFS server: " . $cifs_server_name );

    eval
    {
        if ( defined( $Params->{"OSKQA_HMR_WCLIENT"} ) )
        {
            my $client = $Params->{OSKQA_HMR_WCLIENT};
            $Test->nlog( "CIFS I/O server: "
                  . $cifs_server_name
                  . " client: "
                  . $client
                  . " Operation: HAMMER" );
            $_hmr_client = new Os::EZIO_HMR_CIFS(
                client         => $client,
                servername     => $cifs_server_name,
		sharename      => $cifs_sharename,
                runtimeseconds => 0
            );
            $_hmr_client->start();
        }
        if ( defined( $Params->{OSKQA_DT_WCLIENT} ) )
	{
            my $client = $Params->{OSKQA_DT_WCLIENT};
            $Test->nlog( "CIFS I/O server: "
                  . $cifs_server_name
                  . " client: "
                  . $client
                  . " Operation: DT" );
            $_dt_client = Os::EZIO_DT_CIFS->new(
                client         => $client,
                servername     => $cifs_server_name,
		sharename      => $cifs_sharename,		
                runtimeseconds => 0
            );
            $_dt_client->start();
        }
    };

    if ($@)
    {
      $Test->nlog( "Error initiating I/O on client\n\t" . $@ );
      if ( defined($_hmr_client) ) {
	  $_hmr_client->stop();
	  $Test->nlog( "HAMMER output:\n" . $_hmr_client->{_output} );
	  $_hmr_client = undef;
      }
      if ( defined($_dt_client) ) {
	  $_dt_client->stop();
	  $Test->nlog( "DT output:\n" . $_dt_client->{_output} );
	  $_dt_client = undef;
      }
      return $TCD::CONFIG;
    }
    ## Install tests
    my $tests = get_config()->{'control'}{'testorder'};
    my $result_code;
    my $output;
    foreach my $testname (@$tests) {
        eval { ( $result_code, $output ) = _install_test_program($testname); };

        if ($@) {
            $Test->nlog(
                "\n-- ERROR: Error while intalling test program $testname : $@"
            );
            return $TCD::CONFIG;
        }
        if ( $result_code > 0 ) { last; }
    }
    if ( $result_code > 0 ) {
        return $TCD::FAIL;
    }
    else {
        return $TCD::PASS;
    }

}    # end setup()

#########################################################################
# CLEANUP ###############################################################
#########################################################################
sub _cleanup {

    $Test->description("Cleanup");
    my $result = $TCD::PASS;

    ## Uninstall the test program
    ## Install tests
    my $tests = get_config()->{'control'}{'testorder'};
    foreach my $testname (@$tests) {
        eval { &_uninstall_test_program($testname); };

        if ($@) {
            $Test->nlog("\n-- ERROR: Error while unintalling test program $testname : $@"
            );
            $result = $TCD::CONFIG;
        }
    }

    if ( defined($_hmr_client) ) {
        $_hmr_client->stop();
        $Test->nlog( "HAMMER output:\n" . $_hmr_client->{_output} );
    }
    if ( defined($_dt_client) ) {
        $_dt_client->stop();
        $Test->nlog( "DT output:\n" . $_dt_client->{_output} );
    }
    return $result;

}    # end cleanup()

#########################################################################
# UNINIT ################################################################
#########################################################################
sub _uninit {

    ## Cleanup all NACL objects
    if ($Bsdobj) {
        $Bsdobj->cleanup_APISet();
    }

    return $TCD::PASS;

}    # end uninit()

########################################################################
# Start of Private/Local Subroutines ###################################
# Note: The Routines Are Listed in Alphabetical Order ##################
########################################################################

####################
# GENERIC TESTCASE
####################
sub generic_test {

    my ($testname) = @_;
    if ( !$testname ) {
        $Test->nlog(
            "No testname supplied to internal generic test subroutine.");
        return $TCD::FAIL;
    }

    $Test->description( get_config()->{tests}{$testname}{description} );

    ## Initialize local variables
    my $output      = "DEBUG mode output";
    my $result_code = 0;

    ## Setup the test program
    my $commands2execute = get_config()->{tests}{$testname}{setupsteps};

    #    $Test->nlog("DUMPER (setupsteps ): " . Dumper($commands2execute) );

    ( $result_code, $output ) = _execute_list_of_commands($commands2execute);

    if ( $result_code == 0 ) {
        ## Start the test program
        $commands2execute = get_config()->{tests}{$testname}{executionsteps};

        #	$Test->nlog("DUMPER (executionsteps): " . Dumper($commands2execute));
        ( $result_code, $output ) =
          _execute_list_of_commands($commands2execute);

        ## Cleanup the test program
        $commands2execute = get_config()->{tests}{$testname}{cleanupsteps};
        my ( $cleanup_result_code, $cleanup_output ) =
          _execute_list_of_commands($commands2execute);
    }
    if ( $result_code == 0 ) {
        $Test->nlog("- Test case $testname passed\n");
        return $TCD::PASS;
    }
    elsif ( $result_code > 0 ) {
        $Test->nlog(
"-- ERROR: Error while running test program $testname : $result_code, $output, -- $@ --"
        );
        $Test->nlog("- Test case $testname failed");
        return $TCD::FAIL;
    }

    return $TCD::PASS;

}    # end of generic test case

###########################################################
# Routine : _install_test_program()                       #
# Description : Local subroutine to install the test      #
# program on a Filer                                      #
# Args : $program                                         #
# Returns : Nothing                                       #
# Dependencies : None                                     #
###########################################################
sub _install_test_program {

    ## Initialize local variables
    my ($program) = @_;
    my $sourcelocation = get_config()->{tests}{$program}{"sourcelocation"};
    my $destinationlocation =
      get_config()->{tests}{$program}{"destinationlocation"};
    my $cmd                 = undef;
    my $result_code         = 0;
    my $output              = "";
    my $total_output        = "";
    my $result_code_command = 'echo $?';

    my @cmds;
    if ( defined($sourcelocation) ) {
        $Test->nlog(
"- Installing test program : $sourcelocation to $destinationlocation"
        );
        @cmds = [
            "wget " . $Download_server . $sourcelocation,
            "mv -f /var/home/diag/$program $destinationlocation",
            "chmod 777 $destinationlocation"
        ];

        ## Install the test program under $destinationlocation
        ( $result_code, $total_output ) = _execute_list_of_commands(@cmds);

        ## Print the C program's output to standard out
        ## only if Verbose is set to 1
        if ($Verbose) {
            $Test->nlog(
"-- DEBUG (VERBOSE=1): Standard output from test program:: $cmd is $total_output."
            );
        }
    }
    return $result_code;
}    # end _install_test_program()

###########################################################
# Routine : _execute_list_of_commands()                   #
# Description : Local subroutine to start the test        #
# program                                                 #
# Args : @list_of _commands_2_execute                     #
#        (Array of command lines                          #
# Returns : Nothing                                       #
# Dependencies : None                                     #
###########################################################

sub _execute_list_of_commands {
    my ($commands2execute) = @_;
    my $output             = "unknown";
    my $total_output       = "";
    my $result_code        = 0;
    my $tmp_result         = 0;
    my $response;

    eval {

#	$Test->nlog("DUMPER: " . Dumper($commands2execute) . ": " . ref($commands2execute));
        foreach my $cmd (@$commands2execute) {
            if ( defined($cmd) == 1 ) {
                try {
                    ## Kick off the test program
                    my $commnd = _substituteParam( \%$Params, $cmd );

   #                    $Test->nlog("\n- Executing : :$cmd: ($commnd) " . "\n");

                    $output = $Bsdobj->execute_raw_command(
                        'command'            => $commnd,
                        'connectrec-timeout' => $Timeout,
                    );

                    $tmp_result = $Bsdobj->execute_raw_command(
                        'command'            => 'echo $?',
                        'connectrec-timeout' => $Timeout,
                    );
                    ## Print the C program's output to standard out
                    ## only if Verbose is set to 1
                    if ($Verbose) {
                        $Test->nlog(
"-- DEBUG (VERBOSE=1): Output from program:: $commnd is \"$output\". RC=$tmp_result"
                        );
                    }
                    $tmp_result = $tmp_result + 0;
                    $result_code += $tmp_result + 0;    # Coerce to a number
                    if ( $result_code > 0 ) {
                        $Test->nlog(
                            "Result " . "RC=($result_code):Output:\n$output" );
                    }
                    $total_output = $total_output . $output;
                }
                catch NACL::APISet::Exceptions::CommandFailedException with {
                    my $exception_object = shift;
                    $Test->nlog(
                        "Exception is of type: " . ref($exception_object) );
                    $Test->nlog( "Error sending command ($cmd):  "
                          . $exception_object->text() );
                    $result_code += 1;
                };
            }
        }
    };
    if ($@) {
        return ( 1, $@ );
    }
    return ( $result_code, $total_output );

}    # end _start_test_program()

###########################################################
# Routine : _uninstall_test_programs()                    #
# Description : Local subroutine to uninstall the test    #
# program                                                 #
# Args : None                                             #
# Returns : Nothing                                       #
# Dependencies : None                                     #
###########################################################
sub _uninstall_test_program {

    ## Initialize local variables
    my ($program) = @_;
    my $destinationlocation =
      get_config()->{tests}{$program}{"destinationlocation"};
    my $cmd    = undef;
    my $output = undef;

    ## UnInstall tests
    if ( defined($destinationlocation) ) {
        eval {
            $Test->nlog("Uninstalling the test program: $destinationlocation");
            $cmd = "sudo rm -f $destinationlocation";
            try {
                $output = $Bsdobj->execute_raw_command( command => $cmd );

    # NOTE: We don't check for result code here because it really doesn't matter
    #       and reporting a FAIL or CONFIG may break subsequent test activity
    #       If you have something special going on, see the "setupsteps"
    #       and "cleanupsteps" part of the configuration information hash
    #	    $Test->nlog("Result from executing $cmd: \n$output")
            }
            catch NACL::APISet::Exceptions::CommandFailedException with {
                my $exception_object = shift;
                $Test->nlog(
                    "Exception is of type: " . ref($exception_object) );
                $Test->nlog( "Error text is: " . $exception_object->text() );
                die("ERROR: Error sending command $cmd");
            };
        };
    }

    if ($@) {
        $Test->nlog(
"\n-- ERROR: Error while uninstalling test program $destinationlocation : $@"
        );
        return $TCD::CONFIG;
    }

}    # end _uninstall_test_program()

###########################################################
# Routine : _substituteParam()                           #
# Description : Local subroutine to replace text in       #
# TestDescriptions hash                                   #
# Args :                                                  #
#      : %params  Hash of key=value pairs                 #
#      : %tc      ref to TestDescriptions hash            #
# Returns : Nothing                                       #
# Dependencies : None                                     #
###########################################################
sub _substituteParam {
    my ( $params, $target ) = @_;
    my ( $key, $valuehash, $value );

    #    $Test->nlog("Parameter Substitution");

    no strict 'refs';
    while ( ( $key, $value ) = each(%$params) ) {

# DEBUG	$Test->nlog("Parameter Substitution: key="  . $key . " value=" . $value . "(". $params->{$key} . ")");
        if ( my $str = $target =~ m/($key)/ ) {

            #	    my $before = $target;
            $target =~ s/%($key)%/$value/g;

            #	    my $after = $target;
            #	   $Test->nlog( "REPLACE: ". $before . " " . $after . "\n");
        }
    }
    return $target;
}

sub _timestamp {

    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst ) =
      localtime(time);
    my $format = "%04d%02d%02d %02d%02d%02d";
    my $timestamp =
      sprintf( $format, $year + 1900, $mon + 1, $mday, $hour, $min, $sec );
    return $timestamp;

}
1;

