#Hey emacs, this is basically -*- perl -*-
#$Id$
## @summary Interface implementation for 7m coredump functions
## @author dl-kernel-qa
## @status private
## @pod ../CoreMgr.pm

package Os::CoreMgr::CoreMgr_7m;

## Imports
use strict;
use NACL::APISet;
use NACL::Transit;
use Data::Dumper qw(Dumper);
use Module::Common qw(validate_mixed);
use Options;
use Params::Validate qw(SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use Time::Local;
use Time::ParseDate qw(parsedate);
use TestUtils qw(get_site_info);
use Tharn qw($Log);
use base qw(Os::CoreMgr);

### Import exceptions
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::Transit::Exceptions::TransitException qw(:try);
use NACL::Transit::Exceptions::RetriesExceeded qw(:try);

### Globals/constants
use constant CORETOOL_LOC => "/x/eng/localtest/noarch/share/coretool";
use constant MEMORY_DFLT => "4G";
my $Core_of={};


###############################################################################
# Method:    coretool
# Objective: return output from running coretool on core
# Details:   See POD documentation at the end of this file.
###############################################################################
sub coretool_OLD {
    my ($self,%args) = @_;
    my (
        $bsdObj,
        $cmd,
        $filter,
		$name,
		$node,
        $output,
        );

    ($self,%args) = validate_mixed (
		@_,
		[ { type => OBJECT} ],
		{
			cid     => {type => SCALAR,
					callbacks=>{
					    	'legal values'=>sub{shift()=~
						/\d+/i
					}}},
			missing_blocks    => {type => SCALAR, optional=>1},
			file_size         => {type => SCALAR, optional=>1},
		}
	);


	$node=$self->get_node();
	$name = $self->{cores_of}{$node}{$args{cid}}{name};
	unless ($name) {
		die "Unable to find corefile with cid '$args{cid}'";
	}

    $bsdObj=$self->get_conn_obj(type=>"bsd");

    if (not defined $self->{coretool}{$args{cid}}){
        $cmd="/mroot/etc/crash/coretool /mroot/etc/crash/".$name;
        $self->{coretool}{$args{cid}}=$bsdObj->execute_raw_command(command=>$cmd);
    }

    $filter = 'missing blocks\s+(\d+)' if ($args{missing_blocks});
    $filter = 'file size\s+(\S+)' if ($args{file_size});

    if ($filter) {
        ($output)=$self->{coretool}{$args{cid}}=~/$filter/;
    } else {
        $output = $self->{coretool}{$args{cid}};
    }

    return $output
}


###############################################################################
# Method:    clear core area
# Objective: removed all unsaved cores
# Details:   See POD documentation at the end of this file.
###############################################################################
sub delete_unsaved_cores {
    my ($self,%args)=@_;
    my (
	    $cmd,
	    $console_conn,
	    $node,
	    $output,
	    $partner
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    cid      => {type => SCALAR, optional=>1, default=>0,
					callbacks=>{
						'legal values'=>sub{shift()=~
						/1|0/i
					}}},
		    partner      => {type => SCALAR, optional=>1, default=>0,
					callbacks=>{
						'legal values'=>sub{shift()=~
						/1|0/i
					}}},
	    }
    );

    ### Initialize variables
    $partner=($args{partner}) ? "partner" : "";
    $cmd="priv set test; $partner savecore -k";
    $console_conn=$self->get_conn_obj(type=>"console");
    $node=($args{partner}) ? $self->{partner} : $self->get_node();

    ### Execute command and parse output
    &Tharn::say($console_conn,"$cmd");

    ### Clean up global core hash
    if ($args{cid}){
	    $self->_log(msg=>"removing unsaved core ".
				$self->{cores_of}{$node}{$args{cid}}{name},
				indent=>3
			);
	    delete $self->{cores_of}{$node}{$args{cid}};
    }
    else {
	    foreach my $cid (keys %{$self->{cores_of}{$node}}){
		    my $core_of=$self->{cores_of}{$node}{$cid};
		    if ($core_of->{status}=~/^unsaved$/){
			    $self->_log(
				    msg=>"removing unsaved core ".$core_of->{name},
				    indent=>3
			    );
			    delete $self->{cores_of}{$node}{$cid};
		    }
	    }
    }


    return $self;

}


###############################################################################
# Method:    delete_saved_cores
# Objective: delete all core files
# Details:   See POD documentation at the end of this file.
###############################################################################
sub delete_saved_cores {
    my ($self,%args)=@_;
    my (
	    $cmd,
	    $cliObj,
	    $node,
	    $output,
	    $partner,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    partner      => {type => SCALAR, optional=>1, default=>0,
					    callbacks=>{
						    'legal values'=>sub{shift()=~
						    /1|0/i
					    }}},
	    }
    );

    ### Initialize variables
    $partner="partner " if $args{partner};
    $cmd="priv set test; rmcores";
    $cliObj=$self->get_conn_obj(type=>"cli");
    $node=($args{partner}) ? $self->{partner} : $self->get_node();
    ### Execute command and parse output
    $cliObj->execute_raw_command(command=>"$cmd");


    foreach my $key (keys %{$self->{cores_of}{$node}}){
	    my $core_of=$self->{cores_of}{$node}{$key};
	    if ($core_of->{status}=~/^saved$/){
		    $self->_log(
			    msg=>"removing saved core ".$core_of->{name},
			    indent=>3
		    );
		    delete $self->{cores_of}{$node}{$key};
	    }
    }


    #$self->{coreObjs}=undef;

    return $self;
}

###############################################################################
# Method:    destroy coremgr_7m object
# Objective: log unique marker in messages file
# Details:   See POD documentation at the end of this file.
###############################################################################
sub destroy {
    my ($self)=@_;

    ### Set back to ONTAP defaults
    $self->set_save_onboot(1);
    $self->set_sparse_core(1);
    $self->set_auto_giveback(1);
    $self->set_auto_giveback_panic(1);
    $self->mroot_minfree_limit(bytes=>0);
    $self->dump_attempts(value=>2);

}

###############################################################################
# Method:    disk_fail
# Objective: fail a disk
# Details:   See POD documentation at the end of this file.
###############################################################################
sub disk_fail {
    my ($self,%args)=@_;
    my (
	    $cmd,
	    $cliObj,
	    $rc,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    disk      => {type => SCALAR, },
	    }
    );

    ### Initialize variables
    $cmd="priv set test; dumpcore_fail -d ".$args{disk};
    $rc=1;
    $cliObj=$self->get_conn_obj(type=>"cli");
    ### Execute command and parse output
    try {
	    $cliObj->execute_raw_command(command=>"$cmd");
    } catch NATE::Error with {
	    $rc=0;
    };

    return $rc;
}

###############################################################################
# Method:    dump_attempts
# Objective: set number of attempts if dump fails
# Details:   See POD documentation at the end of this file.
###############################################################################
sub dump_attempts  {
    my ($self,%args)=@_;
    my (
	    $cliObj,
	    $cmd,
	    $output,
	    $retry_cnt,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		     value   => {
			    type => SCALAR, optional=>1,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/\d+/i;
			     }}},
    }
    );

    ### Initialize variables
    $retry_cnt = $args{value};
    $cmd = "options coredump.dump.attempts";
    $cmd .= " ".$args{value} if (defined $retry_cnt);
    $cliObj=$self->get_conn_obj(type=>"cli");
    $self->_log(
	    msg=>"setting dump retry cnt to ".$retry_cnt,
	    indent=>3) if defined ($retry_cnt);

    $output = $cliObj->execute_raw_command(command=>"$cmd");

    if (not defined $retry_cnt){
	    ($retry_cnt)=$output=~/\S+\s+(\d+)/;
    }

    return $retry_cnt;;
}

###############################################################################
# Method:    dumpcore
# Objective: panic node to dump a core
# Details:   See POD documentation at the end of this file.
###############################################################################
sub dumpcore {
    my ($self,%args)=@_;
    my (
	    $cid,
	    $cmd,
	    $coreObj,
	    $console_conn,
	    $output,
	    $partner,
	    $prompt,
	    $sp_conn,
	    $t1,
	    $t2,
	    %dump_of,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    partner      => {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},
		    reason       => {type => SCALAR, optional=>1, default=>"test"},
		    sp           => {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},
		    wait         => {type => SCALAR, optional=>1, default=>1},
	    }
    );

    if ($args{sp}) {

       ## Initialize variables
	#$partner="partner " if $args{partner};
	$cmd="system core";
	$sp_conn=$self->get_conn_obj(type=>"sp");

	### Execute command to panic system
	(undef,undef,$output,$prompt) =
	    &Tharn::say($sp_conn,$cmd,'-prompt',"This will cause a dirty shutdown of your appliance. *");

	    if ($prompt =~ /shutdown/i) {
	    (undef,undef,$output,undef) = &Tharn::say($sp_conn,"y",-blind );
	    }
    }
    else {
    ### Initialize variables
    $partner="partner " if $args{partner};
    $cmd=$partner."reboot -d $args{reason}";
    $console_conn=$self->get_conn_obj(type=>"console");
    
    # let's make sure we get a CLI prompt to execute the command 
    $self->{transit}->change_state(to => "CLI"); 

    ### Execute command to panic system
    (undef,undef,$output,undef) =
	    &Tharn::say($console_conn,$cmd,-blind);
    }
    ### Set connections to non exist
    $self->{conn_cli_exists}=0;
    $self->{conn_bsd_exists}=0;
    $self->{conn_console_exists}=0;
    $self->{cid}=undef;

    ### Wait for dump to complete
    $self->_log(msg=>"dumping core",indent=>3);
    $self->_log(msg=>"command: ".$cmd,indent=>3);
    %dump_of=$self->_wait_for_dump(wait=>$args{wait});

    ### Determine core id from timestamp and
    ### save with dump hash record
    $cid=parsedate($dump_of{timestamp});
    $dump_of{cid}=$cid;
    my @disks =  split(/\s+/,$dump_of{disks});
    $dump_of{disks}=[@disks];

    ### Store the dump record in the object
    $self->{dumpinfo}{$cid}=\%dump_of;

    my ($cnt,$compressed,$type);
    $cnt=scalar(@{$self->get_dump_info(cid=>$cid)->{disks}});
    $compressed=$self->get_dump_info(cid=>$cid)->{compressed};

    $type="none"       if ($cnt==0 and not defined $compressed);
    $type="spray"      if ($cnt > 1 and not $compressed);
    $type="spare"      if ($cnt==1);
    $type="compressed" if ($cnt > 1 and $compressed);

    $self->{dumpinfo}{$cid}{type}=$type;

    ### Store the last dump cid in the object
    $self->{last_cid}=$cid;

    return $cid;
}


###############################################################################
# Method:    find_dump_type
# Objective: get type of core dump will create
# Details:   See POD documentation at the end of this file.
###############################################################################
sub find_dump_type {
    my ($self,%args)=@_;
    my (
	    $cmd,
	    $cliObj,
	    $output,
	    $partner,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    partner      => {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},
	    }
    );

    ### Initialize variables
    $partner="partner " if $args{partner};
    $cmd="priv set test; ".$partner."savecore -i";
    $cliObj=$self->get_conn_obj(type=>"cli");

    ### Execute command and parse output
    $output=$cliObj->execute_raw_command(command=>"$cmd");

return "spray" if ($output =~ /sprayed dumpcore feature/);
    return "spare" if ($output =~ /sparecore dumpcore feature/);
    return "compressed" if ($output =~ /compressed dumpcore feature/);
    return "none" if ($output =~ /No coredump will be done/);
    return "";
}

###############################################################################
# Method:    get_available_space
# Objective: min bytes avaiable in mroot needed to save core
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_available_space {
    my ($self,%args) = @_;
    my (
	    $bsdObj,
	    $cmd,
	    $size,
    );

    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $cmd="df -ha | egrep /mroot\$ | awk '{print \$4}'";
    $size=$bsdObj->execute_raw_command(command=>"$cmd");

    return $self->_size_to_bytes($size);

}
###############################################################################
# Method:    get_conn_obj
# Objective: create a connection to node
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_conn_obj{
    my ($self,%args) = @_;
    my (
	    $connObj,
	    $errmsg,
	    $hostObj,
	    $retry_cnt,
	    $retry_limit,
	    $set_mode,
       );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		     type   => {
			    type => SCALAR,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/cli|bsd|console|sp/i;
			     }}},
		     nologin => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/i;
			     }}},
    }
    );

    #$self->_log(msg=>"ISSUING Tharn::host for ".$self->get_node());
    $hostObj=Tharn::host($self->get_node());
    #$self->_log(msg=>Dumper($self->{hostObj}));
    $set_mode=($self->_get_hosttype()=~/filer-ng/i) ? "CMode" : "7Mode";
    $retry_cnt=0;
    $retry_limit=5;

    #$self->_log(msg=>"Mode: ".$set_mode);
    if ($args{type}=~/cli/i) {

	    while (1) {
		    eval {
			    if ($self->{apiObjCli}) {
					    if (not $self->{apiObjCli}->api_is_session_alive()) {
						    $self->_log(msg => "reconnecting APISet object ($set_mode-RSH) for ".
							    $self->get_node(), indent=>5);
						    $self->{apiObjCli}->api_reconnect_session();
					    }
			    }
			    else {
				    $self->_log(msg => "creating APISet object ($set_mode-RSH) for ".
					    $self->get_node(), indent=>5);
				    $self->{apiObjCli} = NACL::APISet->new(
					    connection=>Tharn::connect("#rsh"),
					    category => "Node",
					    interface => "CLI",
					    set=>$set_mode,
					    );
				    $self->{apiObjCli}->set_timeout('connectrec-timeout'=>$self->{timeout});
				    $self->{apiObjCli}->set_privilege('privilege-level'=>'test');
			    }
			    $self->{conn_cli_exists}=1;
			    last;
		    };
		    if ($@){
			    my $exception_object = shift;
			    if ($retry_cnt++ < $retry_limit) {
				    $self->_log(msg=>"unable to connect, retrying again",
							    indent=>5);
				    sleep(1);
				    redo;
			    }
			    die $exception_object;
		    };
	    }

	    $connObj=$self->{apiObjCli};

    }
    elsif ($args{type}=~/bsd/i) {

	    while(1) {
		    eval {
			    if ($self->{apiObjBsd}) {
				    if (not $self->{conn_bsd_exists} or
					    not $self->{apiObjBsd}->api_is_session_alive()){
					    $self->_log(msg => "reconnecting APISet object (Systemshell-SSH) for ".
						    $self->get_node(), indent=>5);
					    $self->{apiObjBsd}->api_reconnect_session();
				    }
			    }
			    else {
				    $self->_log(msg => "creating APISet object (Systemshell-SSH) for ".
					    $self->get_node(), indent=>5);

				    $self->{apiObjBsd} = NACL::APISet->new(
					    hostobj=>$hostObj,
					    category => "Node",
					    interface => "CLI",
					    set=>"Systemshell"
					    );
				    $self->{apiObjBsd}->set_timeout('connectrec-timeout'=>$self->{timeout});
			    }
			    $self->{conn_bsd_exists}=1;
			    last;

		    };
		    #catch NACL::APISet::Exceptions::ConnectionFailedException with {
		    if ($@) {
			    my $exception_object = shift;
			    if ($retry_cnt++ < $retry_limit) {
				    $self->_log(msg=>"unable to connect, retrying again",
							    indent=>5);
				    sleep(1);
				    redo;
			    }
			    die $exception_object;
			    #$exception_object->throw();
		    };

	    }

	    $connObj= $self->{apiObjBsd};
    }elsif ($args{type}=~/sp/){
	    #if (not $self->{conn_sp_exists}) {
		    my $node=$self->get_node();
		    my $conn_type = '';
		    if (exists $self->{hostObj}->{conn}->{sp_login}) {
			$conn_type = "sp_login";
		    } elsif (exists $self->{hostObj}->{conn}->{rlm}) {
			$conn_type = "rlm";	    
		    } else {
			die "Tharnhost file does not have sp_login or rlm entry.";
		    }
		    
		    $self->_log(msg=>"creating a $conn_type connection for ".$node,
					    indent=>5);
		    $self->{sp_conn}= Tharn::connect(
			    $self->{$node}.
			    "#"."$conn_type",
			    "-login",
			    -eval=>sub{

				    if (@_) {
					    $errmsg = (@_)[0]->{messages};
					    die $errmsg;
				    }
			    }
			    );

		    $self->{conn_sp_exists}=1;
	    #}

	    $connObj=$self->{sp_conn};

    }
    elsif ($args{type}=~/console/){
	    if (not $self->{conn_console_exists}) {
		    my $node=$self->get_node();
		    $self->_log(msg=>"creating a console connection for ".$node,
					    indent=>5);
		    $self->{console_conn}= Tharn::connect(
			    $self->{$node}.
			    "#console",
			    ($args{nologin}) ? "-nologin" : "-login",
			    -eval=>sub{
				    if (@_) {
					    $errmsg = (@_)[0]->{messages};
					    die $errmsg;
				    }
			    }
			    );

		    $self->{conn_console_exists}=1;
	    }

	    $connObj=$self->{console_conn};

    } else {
	    die "connection type '".$args{conn}."' unknown";
    }

    return $connObj;

}
###############################################################################
# Method:    get_conn_obj
# Objective: create a connection to node
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_conn_obj_OLD{
    my ($self,%args) = @_;
    my (
	    $connObj,
	    $errmsg,
	    $hostObj,
	    $retry_cnt,
	    $set_mode,
       );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		     type   => {
			    type => SCALAR,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/cli|bsd|console/i;
			     }}},
		     nologin => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/i;
			     }}},
    }
    );

    #$self->_log(msg=>"ISSUING Tharn::host for ".$self->get_node());
    $hostObj=Tharn::host($self->get_node());
    #$self->_log(msg=>Dumper($self->{hostObj}));
    $set_mode=($self->_get_hosttype()=~/filer-ng/i) ? "CMode" : "7Mode";
    $retry_cnt=0;

    #$self->_log(msg=>"Mode: ".$set_mode);
    if ($args{type}=~/cli/i) {
	    if (not $self->{conn_cli_exists}) {

		    $self->_log(msg => "creating APISet object ($set_mode-RSH) for ".
					    $self->get_node(), indent=>5);
		    while (1) {
			    try {
				    $self->{apiObjCli} = NACL::APISet->new(
					    connection=>Tharn::connect("#rsh"),
					    category => "Node",
					    interface => "CLI",
					    set=>$set_mode,
					    );
				    $self->{apiObjCli}->set_timeout('connectrec-timeout'=>$self->{timeout});
			    } catch NACL::APISet::Exceptions::ConnectionFailedException with {
				    my $exception_object = shift;
				    if ($retry_cnt++ < 1) {
					    $self->_log(msg=>"unable to connect, retrying again",
								    indent=>5);
					    redo;
				    }
				    $exception_object->throw();
			    };
			    last;
		    }
		    ### set connection to true
		    $self->{conn_cli_exists}=1;
	    }

	    $connObj=$self->{apiObjCli};

    } elsif ($args{type}=~/bsd/i) {
	    if (not $self->{conn_bsd_exists}) {
		    $self->_log(msg => "creating APISet object (Systemshell-SSH) for ".
					    $self->get_node(), indent=>5);
		    while(1) {
			    try {
				    $self->{apiObjBsd} = NACL::APISet->new(
					    hostobj=>$hostObj,
					    #hostobj=>$self->{hostObj},
					    category => "Node",
					    interface => "CLI",
					    set=>"Systemshell"
					    );
				    $self->{apiObjBsd}->set_timeout('connectrec-timeout'=>$self->{timeout});
			    } catch NACL::APISet::Exceptions::ConnectionFailedException with {
					    my $exception_object = shift;
					    if ($retry_cnt++ < 1) {
						    $self->_log(msg=>"unable to connect, retrying again",
									    indent=>5);
						    redo;
					    }
					    $exception_object->throw();
			    };
			    last;
		    }
		    ### set connection to true
		    $self->{conn_bsd_exists}=1;
	    }

	    $connObj= $self->{apiObjBsd};

    } elsif ($args{type}=~/console/){
	    if (not $self->{conn_console_exists}) {
		    my $node=$self->get_node();
		    $self->_log(msg=>"creating a console connection for ".$node,
					    indent=>5);
		    $self->{console_conn}= Tharn::connect(
			    $self->{$node}.
			    "#console",
			    ($args{nologin}) ? "-nologin" : "-login",
			    -eval=>sub{
				    if (@_) {
					    $errmsg = (@_)[0]->{messages};
					    die $errmsg;
				    }
			    }
			    );

		    $self->{conn_console_exists}=1;
	    }

	    $connObj=$self->{console_conn};

    } else {
	    die "connection type '".$args{conn}."' unknown";
    }

    return $connObj;

}

###############################################################################
# Method:    get_core_cnt
# Objective: retrieve core count
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_core_cnt {
    my ($self,%args)=@_;
    my (
	    $cnt,
	    $node,
	    $status,
	    @coreObjs,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    partner   => {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},
		    status   => {type => SCALAR, optional=>1,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /^saved$|^unsaved$/i
						    }}},
	    }
    );

    ### Initialize variables;
    $cnt=0;
    $status=$args{status};
    $node=($args{partner}) ? $self->{partner} : $self->get_node();

    if ($status){
	    foreach my $key (keys %{$self->{cores_of}{$node}}) {
		    if ($self->{cores_of}{$node}{$key}{status}=~/^$status$/){
			    $cnt++;
		    }
	    }
    }
    else {
	    $cnt=scalar(keys  %{$self->{cores_of}{$node}});
    }

    return $cnt;

}

###############################################################################
# Method:    get_core_ids
# Objective: retrieve list of core ids
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_core_ids {
    my ($self,%args) = @_;
    my (
	    $node,
	    @cids,
    );
    $node=$self->get_node();
    push @cids, keys %{$self->{cores_of}{$node}};

    return (wantarray()) ? @cids : [@cids];

}

###############################################################################
# Method:    get_core_info
# Objective: retrieve core metadata in hsh
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_core_info {
    my ($self,%args) = @_;
    my (
	    $node,
	    @cids,
    );
    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    cid       => {type => SCALAR},
		    #name      => {type => SCALAR, optional=>1},
		    #status    => {type => SCALAR, optional=>1},
	    }
    );

    $node=$self->get_node();
    @cids= (keys %{$self->{cores_of}{$node}});

    unless (grep $args{cid}, @cids){
	    die "Unable to find core record with coreid '".$args{cid}."'";
    }

    return $self->{cores_of}{$node}{$args{cid}};

}


###############################################################################
# Method:    get_dump_info
# Objective: return whether dump was compressed or not
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_dump_info {
    my ($self,%args)=@_;

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    cid  => {type => SCALAR, optional=>1, default=>$self->{last_cid},
				    callbacks=>{'legal values'=>sub{shift()>=1}}}
	    },
    );

    return $self->{dumpinfo}{$args{cid}};
}

###############################################################################
# Method:    get_last_cid
# Objective: retrieve core id of last core dumped
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_last_cid {
    my ($self)=@_;

    return $self->{last_cid};
}

###############################################################################
# Method:    get_save_info
# Objective: return savecore info in hsh structure
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_save_info {
    my ($self,%args) =@_;
    my (
	    $cliObj,
	    $console_conn,
	    $cmd,
	    $hsh_of,
	    $output,
	    $partner,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    partner      => {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},
	    }
    );

    $partner=($args{partner}) ? "partner" : "";
    $cmd="priv set test; $partner savecore -i";

    $console_conn=$self->get_conn_obj(type=>"console");
    (undef,undef,$output)=&Tharn::say($console_conn,"$cmd");
    $self->_log(msg=>"$output");


=comment
1 unsaved core is occupying 3 disks.
29024478 blocks are available on 14 disks that support sprayed dumpcore.
The sprayed dumpcore feature will be used.

csiqa-3070-63*> savecore -i
1 unsaved core is occupying 3 disks.
Unsaved cores are at risk of being overwritten by sparecore.  Please run savecore.
The sparecore dumpcore feature will be used.

1 unsaved core is occupying 3 disks.
20731770 blocks are available on 10 disks that support sprayed dumpcore.
The compressed dumpcore feature will be used.


3 unsaved cores are occupying 9 disks.\n
8292708 blocks are available on 4 disks that support sprayed dumpcore.\n
The sprayed dumpcore feature will be used.\n


1 unsaved core is occupying 1 disk.\r\r\r\n
1 partial core is occupying 2 disks.\r\r\r\n
20731770 blocks are available on 10 disks that support sprayed dumpcore.\r\r\r\n
The sprayed dumpcore feature will be used.\r\r\r\n


1 unsaved core is occupying 3 disks.\r\r\r\n
22804947 blocks are available on 11 disks that support sprayed dumpcore.\r\r\r\n
The sprayed dumpcore feature will be used.\r\r\r\n

20100924 183727 [01_coresave_ha] <<csiqa-3070-64#console|priv set test; partner savecore -i\r\r\r\n
20100924 183728 [01_coresave_ha] <<csiqa-3070-64#console|2 unsaved cores are occupying 4 disks.\r\r\r\n
20100924 183728 [01_coresave_ha] <<csiqa-3070-64#console|18658593 blocks are available on 9 disks that support sprayed dumpcore.\r\r\r\n
20100924 183728 [01_coresave_ha] <<csiqa-3070-64#console|The sprayed dumpcore feature will be used.\r\r\r\n

=cut
    my $unsaved_cnt=0;
    my $partial_unsaved_cnt=0;
    my $disk_used_cnt=0;
    my $partial_disk_used_cnt=0;
    my $disk_free_cnt=0;
    my $dump_type="";
    my $blocks=0;

    ($unsaved_cnt, $disk_used_cnt)=$output=~
	    /(\d+) unsaved core.*? occupying (\d+) disk.*?/si;

    #$self->_log(msg=>"UNSAVED_CNT: ".$unsaved_cnt);

    ($partial_unsaved_cnt, $partial_disk_used_cnt)=$output=~
	    /(\d+) partial core.*? occupying (\d+) disk.*?/si;

    ($blocks,$disk_free_cnt,$dump_type)=$output=~
	    /(\d+) blocks .*? on (\d+) disks .*?The (\S+) dumpcore/si;

    if (not $blocks) {
	    ($dump_type) = $output =~/The (\S+)core dumpcore feature/si;
    }

    $disk_used_cnt += (defined $partial_disk_used_cnt)
	    ? $partial_disk_used_cnt : 0;

    $unsaved_cnt   += (defined $partial_unsaved_cnt)
	    ? $partial_unsaved_cnt : 0;

    $hsh_of={
	    blocks=>$blocks,
	    disk_free_cnt=>(defined $disk_free_cnt) ? $disk_free_cnt : 0,
	    disk_used_cnt=>(defined $disk_used_cnt) ? $disk_used_cnt : 0,
	    dump_type=>$dump_type,
	    unsaved_cnt=>(defined $unsaved_cnt) ? $unsaved_cnt : 0
    };

    return $hsh_of;
}

###############################################################################
# Method:    get_local_disks
# Objective: return list of local disks
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_local_disks {
    my ($self)=@_;
    my (
	    $cliObj,
	    $home,
	    @disks,
	    $disks_of,
	    $output,
    );

    $cliObj=$self->get_conn_obj(type=>"cli");

    $output=$cliObj->disk_show()->get_parsed_output();

    $home=$self->get_node();

    foreach my $disks_of  (@{$output}){
	    push @disks, $disks_of->{disk}
		    if ($disks_of->{home}=~/^$home/);
    }

    return (wantarray()) ? sort @disks : [sort @disks];
}

###############################################################################
# Method:    giveback
# Objective: giveback storage to takenover partner
# Details:   See POD documentation at the end of this file.
###############################################################################
sub giveback {
    my ($self,%args)=@_;
    my (
	    $cmd,
	    $apiCliObj,
	    $warning,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    force   => {type => SCALAR, optional=>1,
					    callbacks=>{
						    'legal values'=>sub{shift()=~
						    /1|0/i
					    }}},
	    }
    );

    ### Initialize variables
    $apiCliObj=$self->get_conn_obj(type=>"cli");
    $cmd = "cf giveback";
    $cmd .= " -f" if ($args{force});

    $self->_log(msg=>"command: ".$cmd,indent=>3);
    my $output=$apiCliObj->execute_raw_command(command=>$cmd);

    $self->_log(msg=>$output);

    return 1;
}

###############################################################################
# Method:    info_dump
# Objective: return info about dump
# Details:   See POD documentation at the end of this file
################################################################################
sub info_dump {
    my ($self,%args) = @_;
    my (
	    $cid,
	    @bufs,
	    %hsh_of,
	    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    cid   => {type => SCALAR,
					    callbacks=>{
						    'legal values'=>sub{shift()=~/\d+/i
					    }}},
	    }
    );

    $hsh_of{cid} = $args{cid} if (defined $args{cid});

    push @bufs,
    "    core id:    ".$args{cid},
    "    compressed: ".$self->get_dump_info(%hsh_of)->{compressed},
    "    diskerrs:   ".$self->get_dump_info(%hsh_of)->{diskerrs},
    "    duration:   ".$self->get_dump_info(%hsh_of)->{duration},
    "    message:    ".$self->get_dump_info(%hsh_of)->{message},
    "    recursive:  ".$self->get_dump_info(%hsh_of)->{recursive},
    "    starts:     ".$self->get_dump_info(%hsh_of)->{starts},
    "    status:     ".$self->get_dump_info(%hsh_of)->{status},
    "    timestamp:  ".$self->get_dump_info(%hsh_of)->{timestamp},
    "    disks:      @{$self->get_dump_info(%hsh_of)->{disks}}",
    "";


    return (wantarray()) ? @bufs : [@bufs];
}

###############################################################################
# Method:    info_cores
# Objective: display formatted metadata
# Details:   See POD documentation at the end of this file.
###############################################################################
sub info_cores {
    my ($self,%args) = @_;
    my (
		$first,
		$node,
        @bufs,
		%cores_of,
    );

	### Parse input
    ($self,%args) = validate_mixed (
		@_,
		[ { type => OBJECT} ],
		{
			cid    => {type => SCALAR, optional=>1,
                          callbacks=>{
								'legal values'=>sub{shift()=~
								/\d+/i
							}}},

			detail    => {type => SCALAR, optional=>1, default=>0,
                          callbacks=>{
								'legal values'=>sub{shift()=~
								/1|0/i
							}}},
			partner    => {type => SCALAR, optional=>1, default=>0,
                          callbacks=>{
								'legal values'=>sub{shift()=~
								/1|0/i
							}}},
		}
	);

    ### Initialize variables
    $first=1;

    $node=($args{partner}) ? $self->{partner} : $self->get_node();
    #$self->_log(msg=>"NODE: ".$node);
    %cores_of= (exists $self->{cores_of}{$node})
	    ? %{$self->{cores_of}{$node}}
	    : {};

    foreach my $key (keys %cores_of) {

	    next if (defined $args{cid} and $key !~ /^$args{cid}$/i);

	    if ($args{detail}){
		    push @bufs,
		    #  "Ref:               ".$self,
			    "    Name:              ".$cores_of{$key}->{name},
			    "    Status:            ".$cores_of{$key}->{status},
			    "    Coreid:            ".$cores_of{$key}->{core_id},
			    "    Timestamp:         ".$cores_of{$key}->{timestamp},
			    "    Os Version:        ".$cores_of{$key}->{version},
			    "    Panic:             ".$cores_of{$key}->{panic_string},
			    "    Size:              ".$cores_of{$key}->{size};
	    }
	    else {
		    #$header=(defined $args{header}) ? $args{header} : 1;
		    if ($first) {
			    push @bufs,
				    sprintf(
					    "    %-39.39s  %-8.8s %-10.10s %-6.6s %-28.28s\n".
					    "    %-39.39s  %-8.8s %-10.10s %-6.6s %-28.28s",
					    "Name","Status","Coreid","Size","Timestamp",
					    "-"x39,"-"x8,"-"x10,"-"x6,"-"x28
				    );
			    $first=0;
		    }
		    #push @bufs, $self->_info_header() if ($header);

		    push @bufs,
			    sprintf(
				    "    %-39.39s  %-8.8s %10.10s %5.6s  %-s",
				    $cores_of{$key}->{name},
				    $cores_of{$key}->{status},
				    $cores_of{$key}->{core_id},
				    $cores_of{$key}->{size},
				    $cores_of{$key}->{timestamp},
				    );
	    }
    }
    return (wantarray()) ? @bufs : [@bufs];
}

###############################################################################
# Method:    is_auto_giveback
# Objective: detemine whether 'auto giveback' is set
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_auto_giveback {
    my ($self) = @_;
    return $self->{auto_giveback};
}

###############################################################################
# Method:    is_auto_giveback_panic
# Objective: detemine whether 'auto giveback after panic' is set
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_auto_giveback_panic {
    my ($self) = @_;
    return $self->{auto_giveback_panic};
}

###############################################################################
# Method:    is_core_file
# Objective: verify if core file exists
###############################################################################
sub is_core_file {
    my ($self,%args) = @_;
    my (
	    $bsdObj,
	    $cmd,
	    $exists,
	    $name,
	    $node,
	    $output,
	    );

    ### Parse input
    ($self,%args) = validate_mixed (
		@_,
		[ { type => OBJECT} ],
		{
			cid    => {type => SCALAR,
				callbacks=>{
					  'legal values'=>sub{shift()=~
					  /\d+/i
				  }}},
			partner => {type => SCALAR, optional=>1, default=>0,
				callbacks=>{
					  'legal values'=>sub{shift()=~
					  /1|0/i
				  }}},
		}
	);

    ### Initialize variables
    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $node=($args{partner}) ? $self->{partner} : $self->get_node();
    #$self->_log(msg=>Dumper($self->{cores_of}));
    $name = $self->{cores_of}{$node}{$args{cid}}{name};
    if ($args{partner}){
	    $bsdObj->execute_raw_command(command=>"sudo mount_partner");
    }

    $cmd  = ($args{partner})
		? "ls -l1 /partner/etc/crash/".$name
		: "ls -l1 /mroot/etc/crash/".$name;

    $exists=1;

    ### Execute command
    $output = $bsdObj->execute_raw_command(command=>$cmd);
    $exists = 0 if ($output=~/no such file/i);

    return $exists;
}

###############################################################################
# Method:    is_failover
# Objective: Determine failover state
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_failover {
    my ($self,%args)=@_;
    my (
	    $cmd,
	    $cliObj,
	    $output,
	    $rc,
    );

    ### Initialize variables
    $cmd = "cf status";
    $cliObj=$self->get_conn_obj(type=>"cli");

    $output=$cliObj->execute_raw_command(command=>$cmd);

    if ($output !~ /disabled/i) {
	    $rc=1;
    } else {
	    $rc=0;
    }

    return $rc;
}

###############################################################################
# Method:    is_save_on_boot
# Objective: detemine whether 'save on boot' is set
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_save_onboot {
    my ($self) = @_;
    return $self->{save_onboot};
}

###############################################################################
# Method:    is_sparse_core
# Objective: determine whether sparse core is set
# Details:   See POD documentation at the end of this file.
###############################################################################
sub is_sparse_core {
    my ($self) = @_;
    return $self->{sparse_core};
}


###############################################################################
# Method:    log_marker
# Objective: log unique marker in messages file
# Details:   See POD documentation at the end of this file.
###############################################################################
sub log_marker {
    my ($self,%args) = @_;
    my (
	    $cmd,
	    $cliObj,
	    $marker,
	    $rc,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    name       => {type => SCALAR, optional=>1,
					       default=>"NATE coredump test"},
	    }
    );

    ### Initialize variables
    $marker=$args{name}." - ".localtime();
    $cmd = "priv set test; log \"$marker\"";
    $cliObj=$self->get_conn_obj(type=>"cli");
    ## $self->{cookie}=$cookie;

    ### Execute command and parse output
    $cliObj->execute_raw_command(command=>"$cmd");

    $rc = $self->_verify_marker(marker=>$marker);

    ### Make sure cookie gets logged
    ### $self->read_messages(marker=>$cookie);

    return ($rc==0) ? $rc : $marker;
}

###############################################################################
# Method:    mroot_minfree_limit
# Objective: min bytes avaiable in mroot needed to save core
# Details:   See POD documentation at the end of this file.
###############################################################################
sub mroot_minfree_limit {
    my ($self,%args) = @_;
    my (
	    $bsdObj,
	    $bytes,
	    $cmd,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    bytes   => {type => SCALAR, optional=>1,
					    callbacks=>{
						    'legal values'=>sub{shift()=~
						    /\d+/i
					    }}},
	    }
    );

    $bytes=$args{bytes};

    $bsdObj=$self->get_conn_obj(type=>"bsd");

    if (defined $bytes) {
	    $self->_log(msg=>"converting mroot minfree limit to blocks",indent=>3);
	    $cmd=($bytes)
		    ? "echo ".int($bytes/4096)." > /mroot/etc/crash/minfree"
		    :  "rm /mroot/etc/crash/minfree";
    }
    else {
	    $cmd = "cat /mroot/etc/crash/minfree";
    }

    eval {
	    $bytes=$bsdObj->execute_raw_command(command=>"$cmd");
    };
    if ($@){
	    $bytes = 0 if ($@ =~ /no such file/i);
    }

    return $bytes;
}

###############################################################################
# Method:    read_messages
# Objective: read /mroot/etc/log/messages file
# Details:   See POD documentation at the end of this file.
###############################################################################
sub read_messages {
    my ($self,%args) = @_;
    my (
	    $bsdObj,
	    $check_marker,
	    $cmd,
	    $output,
	    $retry_cnt,
	    @lines,
    );


    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    marker       => {type => SCALAR},
	    }
    );


    ### Verify that messages log is up to date by
    ### logging a marker and waiting for it to be
    ### written.
    $check_marker=$self->log_marker(name=>"sync check");
    unless ($check_marker){
	    die "/mroot/etc/log/messages is not current";
    };

    ### Retrieve messages since 'log_marker'
    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $cmd="grep -A 9999 '".$args{marker}."' /mroot/etc/log/messages";
    $output=$bsdObj->execute_raw_command(command=>"$cmd");

    @lines = split(/\r*\n+/, $output);
    map(s/^\s+//,@lines);
    map(s/\s+$//,@lines);
    map(s/\cM//g,@lines);
    @lines = grep (/\S+/,@lines);

    $self->{messages}=[@lines];

    return (wantarray()) ? @{$self->{messages}} : $self->{messages};
}

###############################################################################
# Method:    refresh
# Objective: retrieve core file size
# Details:   See POD documentation at the end of this file.
###############################################################################
sub refresh {
    my ($self,%args) = @_;
    my (
        $cid,
		$found,
		$partner,
		@coreObjs,
	);

	### Validate input arguments
	($self,%args) = validate_mixed (
		@_,
		[ { type => OBJECT} ],
		{
		partner      => {type => SCALAR, optional=>1, default=>0,
						callbacks=>{
							'legal values'=>sub{shift()=~
							/1|0/i
						}}},
		}
	);

	### Retrieve the hash list of all core types
    $self->_log(msg=>"refreshing core data",indent=>3);

    $self->_load_cores(partner=>$args{partner});

    return $self;
}
################################################################################
# Method: restrict_memory
# Objective: set memory restriction
################################################################################
sub restrict_memory {
    my ($self,%args) = @_;
    my (
	    $current_restrict_size,
	    $memory_restrict_size,
       );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    size  => {type => SCALAR, optional=>1, default=>MEMORY_DFLT},
	    }
    );

    $memory_restrict_size=$args{size};

    ### Update restrict memsize if MEMORY param passed into
    ### test differs from the restrict memsize bootarg. Otherwise,
    ### if no MEMORY param passed in and a restrict memsize bootarg
    ### exists then remove the restrict memsize bootarg.

    $current_restrict_size = $self->_get_restricted_memsize();

    if ($memory_restrict_size) {
	    if ($current_restrict_size !~ /^$memory_restrict_size$/i) {
		    $self->_log(msg=>"updating bootarg to restrict memsize to ".
					    $memory_restrict_size,
					    indent=>3);
		    $self->_set_restricted_memsize(memory=>$memory_restrict_size);
	    }
	    else {
		    $self->_log(msg=>"memory already restricted",indent=>3);
	    }
    }

    return 1;
}



###############################################################################
# Method:    savecore
# Objective: savecore
# Details:   See POD documentation at the end of this file.
###############################################################################
sub savecore {
    my ($self,%args)=@_;
    my (
	    $cid,
	    $console_conn,
	    $cmd,
    $conn,
	    $ha,
	    $node,
	    $output,
	    $partner,
    @options,
	    $rc,
    );

    #initialize variables
    $rc=0;

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    abort    => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/;
			     }}},
		    cid       => {
			    type => SCALAR, optional=>1,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/\d+/;
			     }}},
		    force    => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/;
			     }}},
		    giveback    => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/;
			     }}},
		    giveback_force    => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/;
			     }}},
		    partner       => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/;
			     }}},
		    synchronous    => {
			    type => SCALAR, optional=>1, default=>0,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/;
			     }}},
		    wait           => {
			    type => SCALAR, optional=>1, default=>1,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0/;
			     }}},
    }
    );

    push @options, "-f" if ($args{force});
    push @options, "-w" if ($args{synchronous});
    push @options, $args{cid} if ($args{cid});

    delete $args{cid};

    ### Initialize variables
    $cmd .= "partner " if ($args{partner});
    $cmd .= "savecore ";
    $cmd .= "@options " if (@options);

    $node=($args{partner}) ? $self->{partner} : $self->get_node();

    ### Execute command to save core
    $console_conn=$self->get_conn_obj(type=>"console");
    $self->_log(msg=>"command: ".$cmd, indent=>3);
    &Tharn::say($console_conn,$cmd,-swallow,-blind);


    ### Wait for save to complete
    $ha = $self->is_failover();
    if ( ($ha and $args{synchronous}==0) or
	     (not $ha and $args{force}==0 and $args{synchronous}==0) ) {
	    $self->_log(msg=>"waiting for save asynchronously",indent=>3);
	    $rc=$self->wait_for_savecore(
		    wait    => $args{wait},
		    abort   => $args{abort},
		    giveback_force   => $args{giveback_force},
		    giveback   => $args{giveback},
		    partner => $args{partner}
	    )
    }
    else {
	    $self->_log(msg=>"waiting for save synchronously ( $cmd)",indent=>3);
	    $rc=$self->wait_for_savecore(
		    wait    => $args{wait},
		    abort   => $args{abort},
		    partner => $args{partner},
		    _async  => 0
		    );
    }

    ### Get duration time
    $self->refresh(partner=>$args{partner}); #if ($cmd=~/^savecore$/);

    return $rc
}

###############################################################################
# Method:    set_auto_giveback
# Objective: turn auto giveback on or off
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_auto_giveback  {
    my ($self,$value)=@_;
    my (
	    $cliObj,
	    $cmd,
	    $state,
    );

    unless ($value=~/1|0|on|off|true|false/){
	    die "Invalid value '$value' for setting auto-giveback state";
    }

    ### Initialize variables
    $state=($value=~/1|t|on/) ? "on" : "off";
    $cmd = "options cf.giveback.auto.enable ".$state;
    $cliObj=$self->get_conn_obj(type=>"cli");


    ### Execute command
    $self->_log(
	    msg=>"setting auto-giveback ".$state,
	    indent=>3);

    $cliObj->execute_raw_command(command=>"$cmd");

    $self->{auto_giveback}=$value;

    return 1;
}

###############################################################################
# Method:    set_auto_giveback_panic
# Objective: turn auto giveback on or off
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_auto_giveback_panic  {
    my ($self,$value)=@_;
    my (
	    $cliObj,
	    $cmd,
	    $state,
    );

    unless ($value=~/1|0|on|off|true|false/){
	    die "Invalid value '$value' for setting auto-giveback after panic takeover state";
    }

    ### Initialize variables
    $state=($value=~/1|t|on/) ? "on" : "off";
    $cmd = "options cf.giveback.auto.after.panic.takeover ".$state;
    $cliObj=$self->get_conn_obj(type=>"cli");


    ### Execute command
    $self->_log(
	    msg=>"setting auto-giveback-after-panic ".$state,
	    indent=>3);

    $cliObj->execute_raw_command(command=>"$cmd");

    $self->{auto_giveback_panic}=$value;

    return 1;
}


###############################################################################
# Method:    set_failover
# Objective: define whether failover is on or off
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_failover {
    my ($self,$value)=@_;
    my (
	    $cliObj,
	    $cmd,
	    );

    unless ($value=~/1|0|on|off|true|false/){
	    die "Invalid value '$value' for setting set-failover state";
    }


    $self->_log(
	    msg=>"setting failover ".(($value=~/1|on/i) ? "on" : "off"),
	    indent=>3);

    ### Initialize variables
    $cliObj=$self->get_conn_obj(type=>"cli");
    $cmd=($value=~/1|on|t/) ? "cf enable" : "cf disable";

    ### Execute command and parse output
$cliObj->execute_raw_command(command=>"$cmd");

     ### make sure takeover on panic is set
    $cmd="options cf.takeover.on_panic on";

    ### Execute command and parse output
    $cliObj->execute_raw_command(command=>"$cmd");


    return 1;
}


###############################################################################
# Method:    set_save_onboot
# Objective: define whether core is automatically saved on bootup
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_save_onboot {
    my ($self,$value)=@_;
    my $state;

    unless ($value=~/1|0|on|off|true|false/){
	    die "Invalid value '$value' for setting save-onboot state";
    }

    $state=($value=~/1|on|t/i) ? 1 : 0;

    $self->_log(
	    msg=>"setting save-onboot to ".$state,
	    indent=>3);

    if ($state){
	    $self->_modify_rc(from=>'^#savecore$', to=>'savecore');
    }
    else {
	    $self->_modify_rc(from=>'^savecore$', to=>'#savecore');
    }

    $self->{save_onboot} = $state;

    return 1;
}

###############################################################################
# Method:    set_sparse_core
# Objective: sparse_cre
# Details:   See POD documentation at the end of this file.
###############################################################################
sub set_sparse_core  {
    my ($self,$value)=@_;
    my (
	    $cliObj,
	    $cmd,
	    $state,
    );

    unless ($value=~/1|0|on|off|true|false/){
	    die "Invalid value '$value' for setting sparse-core state";
    }

    $state=($value=~/1|on|t/i) ? "on" : "off";

    ### Initialize variables
    $cmd = "options coredump.metadata_only ".$state;
    $cliObj=$self->get_conn_obj(type=>"cli");

    ### Execute command
    $self->_log(
	    msg=>"setting sparse core ".$state,
	    indent=>3);

    $cliObj->execute_raw_command(command=>"$cmd");

    $self->{sparse_core}=($value=~/1|on|t/i) ? 1 : 0;

    return 1;
}
###############################################################################
# Method:    tp_clear
# Objective: clear test points
# Details:   See POD documentation at the end of this file.
###############################################################################
sub tp_clear {
    my ($self) = @_;
    my (
	    $cliObj,
	    $cmd,
	    $output,
    );

    $cliObj=$self->get_conn_obj(type=>"cli");
    $cmd = "priv set test; tp clear";
    $output=$cliObj->execute_raw_command(command=>"$cmd");

    return $self;
}

###############################################################################
# Method:    tp_error
# Objective: set a test point
# Details:   See POD documentation at the end of this file.
###############################################################################
sub tp_error {
    my ($self,%args) = @_;
    my (
	    $cliObj,
	    $cmd,
	    $output,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    module  => {type => SCALAR, optional=>1, default=>"dumpcore" },
		    id      => {type => SCALAR },
		    cnt     => {type => SCALAR },
		    parm1  =>  {type => SCALAR },
		    parm2  =>  {type => SCALAR, optional=>1},
	    }
    );

    $cliObj=$self->get_conn_obj(type=>"cli");

    $cmd = "priv set test; tp error ".$args{module}." ".$args{id}." ".$args{cnt}." ".
	   $args{parm1}." ".$args{parm2};

    $output=$cliObj->execute_raw_command(command=>"$cmd");

    return $self;
}

###############################################################################
# Method:    tp_list
# Objective: list test points that are currently set
# Details:   See POD documentation at the end of this file.
###############################################################################
sub tp_list {
    my ($self)=@_;
    my (
	    $cliObj,
	    $cmd,
	    $output,
	    @lines,
    );
    $cliObj=$self->get_conn_obj(type=>"cli");
    $cmd = "priv set test; tp list";
    $output=$cliObj->execute_raw_command(command=>"$cmd");

    @lines = split(/\r*\n+/, $output);
    map(s/^\s+//,@lines);
    map(s/\s+$//,@lines);
    map(s/\cM//g,@lines);
    @lines = grep (/^\[|No testpoints/i,@lines);

    return (wantarray()) ? @lines : [@lines];
}

################################################################################
# Method: _unrestrict_memory
# Objective: remove memory restriction
################################################################################
sub unrestrict_memory {
    my ($self) = @_;

    if ($self->_has_restricted_memory()) {
	    $self->_set_restricted_memsize(memory=>-1);
    }
    else {
	    $self->_log(msg=>"no restriction to remove",indent=>3);
    }
}

###############################################################################
# Method:    wait_for_boot
# Objective: boot node
# Details:   See POD documentation at the end of this file.
###############################################################################
sub wait_for_boot {
    my ($self,%args)=@_;
    my (
	    $cmd,
	    $t1,
	    $t2,
	    $duration,
    );

    ### Boot up system and measure duration
    $self->_log(msg=>"booting ontap",indent=>3);
    $t1=timelocal(localtime());
    $self->{transit}->change_state(to=>"CLI");
    $t2=timelocal(localtime());

    ### Store boot duration in dump hash
    $duration=($t2-$t1);
    $self->_log(msg=>"boot duration: ".$duration."s", indent=>5);

    ### Re-establish connections
    $self->_log(
	    msg=>"re-establishing connections to ".$self->get_node(),
	    indent=>3
    );
    $self->get_conn_obj(type=>"bsd");
    $self->get_conn_obj(type=>"cli");

    $self->refresh();
    return $self;
}

###############################################################################
# Method:    wait_for_fmdisk
# Objective: wait for auto savecore operation to start
# Details:   See POD documentation at the end of this file.
###############################################################################
sub wait_for_fmdisk {
    my ($self,%args)=@_;
    my (
	    $console_conn,
	    $done_filter,
	    $timeout,
	    $rc,
	    @lines,
	    @values,
    );


    ###Initialize variables
    $rc=1;
    $timeout=600;

    push @values,
	    "fmdisk_reserve_spare",
	    "coredump.save.started";

    $done_filter=join('|',@values);

    #$console_conn=$self->get_conn_obj(type=>"console");
    $self->_log(msg => "Use console_conn to wait for msgs $done_filter");       
    while (1) {
	    my (undef, $preoutput, $output) = &Tharn::say(
		$self->{console_conn}, "",
		".*\r",
		-noprompt,
		-nocr,
		-swallow,
		-timeout => $timeout,
		#-maxidle => 60,
		-eval_error
	    );		    
		    
            if ($@ && $@ =~ /Err=TIMEOUT/i) {
		    $self->_log(msg=>"Timed out waiting for fmdisk (timeout=$timeout)");
		    $self->_log(msg=>"Error Msg is [[[$@]]]");
		    $rc=0;
		    last;
	    }    
			    
            $output = $preoutput . $output;			    		    
			    
            if ($output &&
		($output =~ /fmdisk_reserve_spare/mgs or
		 $output =~ /coredump\.save\.started/mgs)) {
		    $self->{fmdisk}=[@values];	    
		    $rc=1;
		    last;
	    }
    }

    return $rc;
}


###############################################################################
# Method:    wait_for_giveback
# Objective: wait until giveback of other node storage is complete
# Details:   See POD documentation at the end of this file.
###############################################################################
sub wait_for_giveback {
    my ($self,%args)=@_;

    $args{msg}="Failover enabled.*is up";

    eval {
        $self->_wait(%args);
    };
    if ($@){
        $self->_log(msg=>$@,indent=>3);
        return 0;
    }

    return 1;
}
###############################################################################
# Method:    _wait_for_savecore
# Objective: wait for core to be saved
###############################################################################
sub wait_for_savecore {
    my ($self,%args)=@_;
    my (
		$cmd,
		$cnt,
		$complete,
		$core,
		$console_conn,
		$core_msg,
		$done_filter,
		$fail_cnt,
		$first,
		$gbf_set,
		$last_core,
		$last_pct,
		$loop_cnt,
		$output,
		$partner,
		$pct,
		$pct_last,
		$rc,
		$retry_cnt,
		$save_cnt,
		$tp_set,
		$t1,
		$t2,
		$value,
		$wait,
		@lines,
		@values,
    );


    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    abort 		=> {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
		    giveback 	=> {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
		    giveback_force 	=> {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
		    partner      => {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},
		    retries      => {type => SCALAR, optional=>1, default=>1,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
		    sleep      => {type => SCALAR, optional=>1, default=>5,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
		    timeout      => {type => SCALAR, optional=>1, default=>30,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
		    timeout_retries  => {type => SCALAR, optional=>1, default=>10,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
		    wait         => {type => SCALAR, optional=>1, default=>1,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},


		    _async 		=> {type => SCALAR, optional=>1, default=>1,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /\d+/i
						    }}},
	    }
    );

    ### Initialize variables
	$partner  = $args{partner};
	$cmd      = "partner " if ($partner);
    $cmd     .= "savecore -s";
	$complete = 0;
	$fail_cnt = 0;
	$first    = 1;
	$loop_cnt = 0;
	$pct      = 0;
	$pct_last = 0;
	$rc       = 0;
	$save_cnt = 0;
	$tp_set   = 0;
	$retry_cnt= 0;
	$wait     = $args{wait};

    ### Retrieve console handle
	$console_conn=$self->get_conn_obj(type=>"console");

	### Start duration timer since there's a core
	### to be saved
    $t1=timelocal(localtime());

	### Messages that will end waiting for savecore
	### operation
	push @values,
		"No core needs to be saved",
		"No savecore in progress",
		"Savecore completed",
		"partner: Not in takeover mode",
		"Previous attempts to save core failed";
	$done_filter=join('|',@values);

	### Sit in while loop until one of the savecore completion
	### messages is seen.
    while (1) {

		### Determein what say call to use depending on
		### whether an async or sync operation is specified
		if ($args{_async}) {
			(undef,undef,$output)=&Tharn::say(
					$console_conn,
					$cmd,
					'-timeout' => $args{timeout},
					'-eval_error' );
		}
		else {
			$output=&Tharn::say(
				$console_conn,
				"",
				".*\r",
				'-noprompt',
				'-swallow',
				'-timeout'=>$args{timeout},
				'-eval_error');

		}

		#$self->_log(msg=>"|$output|");

		### Exit if no filer response is seen after retry limit for
		### timeout period
		if ($@ && $@ =~ /Err=TIMEOUT/i) {
			### Lets try again
			if ($retry_cnt++ < $args{timeout_retries}) {
				$self->_log(msg=>"");
				$self->_log(msg=>"WARNING: timed out after waiting ".
								 $args{timeout}."s for savecore response ".
								 "(retry: $retry_cnt)",
								 indent=>5);
				redo;
			}
			else {
				$self->_log(msg=>"");
				$self->_log(msg=>"ERROR: timed out after waiting ".
							     $args{timeout_retries}. "times for ".
							     $args{timeout}."s for savecore response");
				$rc=0;
				last;
			}
		}

		### Parse the output that may contain 1 or more lines
		### due to busy console
		@lines = split(/\r*\n+/, $output);
		map(s/^\s+//,@lines);
		map(s/\s+$//,@lines);
		map(s/\cM//g,@lines);

		### Iterate over the output displaying selected
		### messages of interest
		foreach my $line (@lines){

			#[csiqa-3070-63:coredump.save.started:info]: Saving 497M to core.118046071.2010-09-21.21_12_20.nz (
			if ($line=~/coredump.save.started.*]: (.* to (\S+))/) {
					$core=$2;
					$core_msg = $1;
			}
			elsif ($line=~/coredump.save.complete.*\]: (.*)/) {
				$self->_log(msg=>"$1",indent=>3);
				$save_cnt+=1;
			}
			elsif ($line=~/]: (Testpoint .*)/ ) {
				$self->_log(msg=>"");
				$self->_log(msg=>"$1",indent=>3);
				$rc=0;
			}
			elsif ($line=~/$done_filter/si){
				$complete = ($save_cnt > 0) ? 1 : 0;
				if ($save_cnt==0 and $loop_cnt++ >= $args{retries}) {
					$fail_cnt++;
					$rc=0;
				} else {
					$rc=1;
				}
				last;
			}
			elsif (
				$line=~/coredump.save.create.failed.*\]: (.*)/ or
				$line=~/coredump.save.error.*\]: (.*)/ or
				$line=~/coredump.save.aborted.*\]: (.*)/ or
				$line=~/coredump.save.nospace.*\]: (.*)/ or
				$line=~/coredump.save.attempts.count.*\]: (.*)/ ) {
				$self->_log(msg=>"WARNING: $1");
				$fail_cnt++;
				$rc=0;
				last;
			}
		}

		### If a complete or fail_cnt flag is set
		### then exit the while loop
		last if ($complete or $fail_cnt);

		### Filter out the line showing status of save
		### for whether savecore operation is async or
		### sync
		my $filter;
		if ($args{_async}){
			@lines  = grep (/Saving core/,@lines);
			$filter = '(\S+): (\d+)';
		} else {
			@lines = grep (/done\)/,@lines);
			$filter = '\((\d+)\% done\)';
		}

		### Parse the status line and display new status
		### output format to STDOUT
		if ($lines[0] =~ /$filter/) {
            if ($args{_async}){
				$core = $1;
				$value=$2;
			} else {
				$value=$1;
			}

			next if ($value==0);
			#next if ($core !~ /^core/ and $value !~ /\d+/);
            $first=1 if ($last_core !~ /$core/);
            if ($first) {
                #$self->_log(msg=>"  - Done") if ($cnt++>0);
				$self->_log(msg=>$core_msg,indent=>3) if (defined $core_msg);
                $self->_log(msg=>"Saving $core ...",nocr=>1,indent=>3);
            }

			### Only display out once it reaches
			### one of the completion stage
            $pct = 10 if ($value>=10 and $value<=19);
            $pct = 20 if ($value>=20 and $value<=29);
            $pct = 30 if ($value>=30 and $value<=39);
            $pct = 40 if ($value>=40 and $value<=49);
            $pct = 50 if ($value>=50 and $value<=59);
            $pct = 60 if ($value>=60 and $value<=69);
            $pct = 70 if ($value>=70 and $value<=79);
            $pct = 80 if ($value>=80 and $value<=89);
            $pct = 90 if ($value>=90 and $value<=99);

            $self->_log(msg=>$pct."%",nocr=>1) if ($pct != $last_pct);
            $last_pct=$pct;
            $last_core=$core;
            $first=0;

			### Exit while loop if 'wait' option is set to false
			if (defined $pct and not $wait) {
				$self->_log(msg=>"aborting wait for save core completion",
						    indent=>3);
				$rc=0;
				last;
			}

			### Set testpoint of abort option is set to true
			if ($args{abort} and $pct >= $args{abort} and not $tp_set){
				$self->tp_error(
					module=>"dumpcore",
					id=>"abort_save",
					cnt=>1,
					parm1=>1,
					);
				$tp_set=1;
			}

			### Issue forced giveback if requested
			if ($args{giveback_force} and $pct >= $args{giveback_force} and not $gbf_set){
				$self->giveback(force=>1);
				$gbf_set=1;
			}

			### Issue giveback if requested
			if ($args{giveback} and $pct >= $args{giveback} and not $gbf_set){
				$self->giveback(force=>0);
				$gbf_set=1;
			}
        }
		### Sleep before requesting
		### next savecore status
		sleep($args{sleep});
    };

    ### Display duration time if a
	### a core was saved
	if ($save_cnt) {
		$t2=timelocal(localtime());
		if (defined $t2 and defined $t1){
			 $self->_log(msg=>"save duration: ".
						($t2-$t1)."s",indent=>5);
		}
	}

	#$self->_log(msg=>"rc=$rc");

    return $rc;
}

###############################################################################
# Method:    wait_for_takeover
# Objective: wait until takeover of other node storage is complete
# Details:   See POD documentation at the end of this file.
###############################################################################
sub wait_for_takeover {
    my ($self,%args)=@_;

    $args{msg}="ready for giveback";

    eval {
        $self->_wait(%args);
    };
    if ($@){
		$self->_log(msg=>$@,indent=>3);
        return 0;
    }

    return 1;
}

###############################################################################
#  P  R  I  V  A  T  E
###############################################################################

###############################################################################
# Method:    _ems_check
# Objective: Check for existance of ems event in messages file
###############################################################################
sub _ems_check {
    my ($self,%args) = @_;
	    my (
	    $event,
	    @lines,
	    @messages,
	    );

    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    event    => {type => SCALAR},
		    messages => {type => ARRAYREF,
					     optional=>1, default=>$self->{messages}}
	    }
    );

    @lines = grep (/$args{event}/, @{$args{messages}});
    @messages=(@lines) ? @lines : "ERROR: EMS message not found ($args{event})";
    if ($self->_get_loghdl()) {
	    if (@lines){
		    foreach my $line (@lines) {
			    my ($ems)=$line=~/(\[.*\])/;
			    $self->_log(msg=>"ems message found: ".$ems,indent=>3);
		    }
	    }
	    else {
		    $self->_log(msg=> "ERROR: ems message not found ($args{event})");
	    }
    }
    else {
	    $Tharn::Log->comment(@messages);
    }

    return (@lines) ? 1 : 0;
}

###############################################################################
# Method:    _get_partner
# Objective: retrieve partner name
###############################################################################
sub _get_partner {
    my ($self) = @_;
    my (
	    $cliObj,
	    $cmd,
	    $name,
	    $result,
	    @lines,
	);

    $cliObj=$self->get_conn_obj(type=>"cli");

    # UP 01:24:01, partner 'csiqa-3070-64', cluster monitor disabled
    ### Get partner name
    $cmd    = "cf monitor";
    $result = $cliObj->execute_raw_command(command=>$cmd);
    ($name) = $result =~ /partner \'(\S+)\'/si;

    return $name;
}

###############################################################################
# Method:    _get_restricted_memsize
# Objective: retrieve restricted memsize bootarg parameter
###############################################################################
sub _get_restricted_memsize {
    my ($self) = @_;
    my (
	    $bsdObj,
	    $cmd,
	    $result,
	    $size,
	    @lines,
	    );

    $size = undef;
    $bsdObj=$self->get_conn_obj(type=>"bsd");
    eval {
	    ### Check if memory is restricted
	    $cmd    = "kenv | egrep \"\^bootarg.init.restricted_memsize=\"";
	    $result = $bsdObj->execute_raw_command(command=>$cmd);
	    ($size) = $result =~ /\=\"(\S+)\"/;
    };

    return $size;
}

###############################################################################
# Method:    _get_size
# Objective: retrieve core file size
###############################################################################
sub _get_size {
    my ($self,%args) = @_;
    my (
    $bsdObj,
	    $cmd,
	    $name,
	    $node,
    $parent,
	    $size,
	    @cmds,
	    );

    ### Parse input
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    cid    => {type => SCALAR,
			    callbacks=>{
				      'legal values'=>sub{shift()=~
				      /\d+/i
			      }}},
		    partner   => {type => SCALAR, optional=>1, default=>0,
			    callbacks=>{
				      'legal values'=>sub{shift()=~
				      /\d+/i
			      }}},
	    }
    );

    ### Initialize variables
    $node = ($args{partner}) ? $self->{partner} : $self->get_node();
    $bsdObj=$self->get_conn_obj(type=>"bsd");

    #$self->_log(msg=>Dumper($self->{cores_of}));

    $name = $self->{cores_of}{$node}{$args{cid}}{name};
    if ($args{partner}) {
	    $bsdObj->execute_raw_command(command=>'sudo mount_partner');
    }

    $cmd = ($args{partner})
	    ? "ls -lh /partner/etc/crash/".$name." | awk '{print \$5}'"
	    : "ls -lh /mroot/etc/crash/".$name." | awk '{print \$5}'";

    ### Execute command
    $size = $bsdObj->execute_raw_command(command=>$cmd);
    $size = 0 if ($size=~/no such file/i);
    $size =~ s/\r*\n+//g;

    return $size;
}

###############################################################################
# Method:    _get_state
# Objective: Return machine connection state
###############################################################################
sub _get_state {
    my ($self)=@_;
    my ($state);

    $self->_log(msg=>"retrieving ".$self->get_node()." state",indent=>5);
    $state = $self->{transit}->get_state();
    $self->_log(msg=>"state: ".$state,indent=>7);


    ### return state
    return $state;
}

###############################################################################
# Method:    _has_restricted_memory
# Objective: retrieve restricted memsize bootarg parameter
###############################################################################
sub _has_restricted_memory {
    my ($self) = @_;
    my (
	    $bsdObj,
	    $cmd,
	    $rc,
	    $result,
	    );

    $rc=0;
    $bsdObj=$self->get_conn_obj(type=>"bsd");

    eval {
	    ### Check if memory is restricted
	    $cmd    = "kenv | egrep \"\^bootarg.init.restricted_memsize=\"";
	    $result = $bsdObj->execute_raw_command(command=>$cmd);
	    $rc=1 if ($result =~ /\=\"(\S+)\"/);
    };

    return $rc;
}

###############################################################################
# Method:    _init
# Objective: Iniitialize set of core objects
###############################################################################
sub _init_OLD {
    my ($self) = @_;
    my (
	    $cores_of,
    );

    ### Retrieve the hash list of all core types
    $cores_of=$self->_list_cores();

    ### Iterate through list and create core objects
    foreach my $core_of (@{$cores_of}){
	    $core_of->{parent}=$self;
	    push @{$self->{coreObjs}}, Os::Core->new(%$core_of);
    }


    return (wantarray()) ? @{$self->{coreObjs}} : $self->{coreObjs};
}

##############################################################################
# Method:    _install_coretool
# Objective: Install coretool on filer
###############################################################################
sub _install_coretool {
    my ($self)=@_;
    my (
        $arch,
		$bsdObj,
        $cmd,
        $dir,
        $ctool_loc,
        $errmsg,
        $options,
        $os,
        %site_of,
        $site,
        );

    ### BUG: get_site_info() doesn't know where site file lives unless an
    ### Option option is instantiated.  The Option instance doesn't see site
    ### variable unless a Server object is created (I'll get site info from
    ### the env myself).
    $options = new Options();
    $options->process();
    $site= $ENV{"SITE"};
    %site_of = get_site_info();

    $dir       = $self->{dir};
    $arch      = $self->_get_arch();
    $ctool_loc = $self->{ctool_loc};

    ### Copy over coretool file to the host directory under test
    ### using wget.
	unless ($site_of{$site}->{install_server}) {
		die "ERROR: No site_info file found using SITE parameter '$site'\n".
			"Verify that a SITE parameter is defined in ~/.ntest file\n";
	}

	$bsdObj=$self->get_conn_obj(type=>"bsd");

	$cmd =
		"wget " .
		$site_of{$site}->{install_proto}  . '://' .
		$site_of{$site}->{install_user}   . ':' .
		$site_of{$site}->{install_pass}   . '@' .
		$site_of{$site}->{install_server} .
		$ctool_loc."/".$arch."/coretool";


	$self->_log(msg=>["upload 'CTOOL_LOC:$arch/coretool' to '$dir'"],indent=>5);
	$bsdObj->execute_raw_command(command=>$cmd);


	### Change permissions on ftest binary so it can be executed
	$bsdObj->execute_raw_command(
				command=>"chmod 777 ".$self->{dir}."/coretool");


    return 1;

}

##############################################################################
# Method:    _is_coretool_installed
# Objective: Verify if coretool is installed on node
###############################################################################
sub _is_coretool_installed {
	my ($self) = @_;
	my (
		$bsdObj,
		$cmd,
		$exist,
		$output,
		);

	### Initialize variables
    $cmd  = "ls -lh /mroot/etc/crash/coretool";

	$bsdObj=$self->get_conn_obj(type=>"bsd");

    ### Execute command
    $output = $bsdObj->execute_raw_command(command=>$cmd);
    $exist = ($output=~/no such file/i) ? 0 : 1;
	return $exist;
}

###############################################################################
# Method:    _load_cores
# Objective: retrieve list of cores
###############################################################################
sub _load_cores {
    my ($self, %args) = @_;
    my (
	    $cliObj,
	    $cmd,
	    $console_conn,
	    $core_of,
	    $filter,
	    $found,
	    $node,
	    $output,
	    $partner,
	    @lines,
	    #%core_of,
	    @cores_of,
    );

    #$self->_log(msg=>"_load_cores called()");

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    partner      => {type => SCALAR, optional=>1, default=>0,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /1|0/i
						    }}},
		    status       => {type => SCALAR, optional=>1, default=>"all",
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /all|^saved$|^unsaved$/i
						    }}},
		    name         => {type => SCALAR, optional=>1,
						    callbacks=>{
							    'legal values'=>sub{shift()=~
							    /.*\.nz/i
						    }}},
	    }
    );

    ### Initialize variables
    #$self->{cores_of}=[];
    $partner="partner " if $args{partner};
    $console_conn=$self->get_conn_obj(type=>"console");

    $node=($partner) ? $self->{partner} : $self->get_node();

    $cliObj=$self->get_conn_obj(type=>"cli");

    $cmd="priv set test; ".$partner."savecore -l";
    $filter="" if ($args{status}=~/all/);
    $filter=".* will be created by" if ($args{status}=~/^unsaved$/i);
    $filter="\$" if ($args{status}=~/^saved$/i);

    ### Execute command and parse output
    if ($args{partner}){
	    (undef,undef,$output)=Tharn::say($console_conn,$cmd,-swallow);
    }
    else {
	    $output=$cliObj->execute_raw_command(command=>"$cmd");
    }

    $Tharn::Log->comment("OUTPUT (savecore -l):\n".$output);

    @lines = split(/\r*\n+/, $output);
    map(s/^\s+//,@lines);
    map(s/\s+$//,@lines);
    map(s/\cM//g,@lines);
    @lines = grep (/\S+/,@lines);

    ### Construct hash structure for core data
    foreach my $line (@lines) {
	    if ($line =~ /(core.*.nz)$filter/) {
	$core_of={};
		    $core_of->{name} = $1;
		    $core_of->{status} = "saved";
		    $core_of->{core_id} = "";
		    next if (defined $args{name} and $core_of->{name} !~ /^$args{name}$/);
		    $found = 1;
	    }
	    next unless ($found == 1);
	    if ($line =~ /Panic string: (.*)/) {
		    $core_of->{panic_string} = $1;
	    } elsif ($line =~ /Core ID: (\d+)/) {
		    $core_of->{core_id} = $1;
		    $core_of->{status} = "unsaved";
	    } elsif ($line =~ /Panic time: (.*)/) {
		    $core_of->{timestamp} = $1;
		    $core_of->{core_id} = parsedate($core_of->{timestamp})
			    if ($core_of->{status} =~ /^saved$/i);
	    } elsif ($line =~ /OS version: (.*)/) {
		    $core_of->{version} = $1;
		    #push @{$self->{cores_of}{$core_of->{core_id}}}, $core_of;
		    my $cid=parsedate($core_of->{timestamp});

		    $Core_of->{$node}{$cid}{name}=$core_of->{name};
		    $Core_of->{$node}{$cid}{status}=$core_of->{status};
		    $Core_of->{$node}{$cid}{core_id}=$core_of->{core_id};
		    $Core_of->{$node}{$cid}{panic_string}=$core_of->{panic_string};
		    $Core_of->{$node}{$cid}{timestamp}=$core_of->{timestamp};
		    $Core_of->{$node}{$cid}{version}=$core_of->{version};
		    $Core_of->{$node}{$cid}{size}=
			    $self->_get_size(cid=>$cid,partner=>$args{partner});

		    $found= 0;
	    }
    }

    ### Return the result
    return $self->{cores_of};

}

###############################################################################
# Method:    _modify_rc
# Objective: Modify /mroot/etc/rc file
###############################################################################
sub _modify_rc {
    my ($self,%args) = @_;
    my (
	    $bsdObj,
	    $cmd,
	    $from,
	    $to,
	    $output,
	    $rc,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    from   =>  {type => SCALAR},
		    to     =>  {type => SCALAR},
	    }
    );
    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $rc = "/mroot/etc/rc";
    $from = $args{from};
    $to   = $args{to};

    $cmd="sed -i.orig 's/$from/$to/g' ".$rc;
    $output=$bsdObj->execute_raw_command(command=>$cmd);

    return 1;
}

################################################################################
# Method: _new()
# Objective: Create a new Coredump_7m object
################################################################################
sub _new {
    my ($class, %args) = @_;
    my (
		$bsdObj,
		$cliObj,
		$consObj,
		$errmsg,
		$hostObj,
		$memory_restrict,
		$options,
		$self,
		%site_of,
		#$spObj,
        );

    ### bless instance into class
    $self= bless{}, $class;

    ### Validate input arguments
    ($class,%args) = validate_mixed (
	    @_,
	    [ { type => SCALAR} ],
	    {
		    #client     =>  {type => SCALAR},
		    loghdl     =>  {type => SCALARREF, optional=>1 },
		    ctool_loc  =>  {type => SCALAR|UNDEF, optional=>1 },
		    node       =>  {type => SCALAR},
		    save_onboot  => {
			    type => SCALAR, optional=>1, default=>1,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0|on|off/i;
			     }}},
		    sparse => {
			    type => SCALAR, optional=>1, default=>1,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0|on|off/i;
			     }}},
		    timeout    =>  {type => SCALAR, optional => 1 }
	    }
    );

    ### Store metadata in object
    $self->{hostObj}       = Tharn::host($args{node});
    $self->{loghdl}        = $args{loghdl};
    $self->{node}          = $args{node};
    $self->{timeout}       = $args{timeout};
    $self->{coreObjs}      = [];
    $self->{sparse_core}   = $args{sparse};
    $self->{save_onboot}  = $args{save_onboot};
    $self->{cores_of}     = $Core_of;
    ### Create connections to node
    $self->_log(
	    msg=>"creating connections to ".$self->get_node(),
	    indent=>3
    );

    $cliObj =  $self->get_conn_obj(type=>"cli");
    $bsdObj =  $self->get_conn_obj(type=>"bsd");
    $consObj = $self->get_conn_obj(type=>"console");

    ### Create a transit object
    $self->_log(msg=>"creating a transit object",indent=>3);
    $self->{transit} = NACL::Transit->new(name => $self->get_node());

    ### Retrieve ontap version
    $self->{version_of}=$cliObj->version()->get_parsed_output()->[0];

    ### Make sure save on boot and sparse are set
    $self->_set_autoboot(enable=>1);
    $self->_set_gdb_recursive_panic(enable=>0);
    $self->set_save_onboot($self->{save_onboot});
    $self->set_sparse_core($self->{sparse_core});
    $self->set_auto_giveback(0);
    $self->set_auto_giveback_panic(0);
    $self->dump_attempts(value=>2);
    $self->{partner}=$self->_get_partner();
    $self->{ctool_loc}=(defined $args{ctool_loc})
	    ? $args{ctool_loc} : CORETOOL_LOC;

    ### Add missing FW prompt
    #push @{$self->{hostObj}->{prompt_take}},$self->{partner}.'\(takeover\)';

    #$self->_log(msg=>Dumper($self->{hostObj}));


    ### Set directory for bsd connection to /mroot/etc/crash
    $self->{dir}="/mroot/etc/crash";
    $self->_log(msg=>"setting directory to ".$self->{dir},indent=>3);
    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $bsdObj->execute_raw_command(command=>"cd ".$self->{dir});

    ### Retrieve architecture to locate correct coretool
    ### version
    $self->_log(msg=>"retrieving host architecture ",indent=>3);
    $self->{arch} = $bsdObj->execute_raw_command(command=>"uname -p");
    $self->{arch} =~ s/\n//g;
    $self->_log(msg=>$self->{arch},indent=>5);

    ### Upload coretool to node if it doesn't exist on node already
    $self->_log(msg=>"upload coretool to node",indent=>3);
    if ($self->_is_coretool_installed()) {
	    $self->_log(msg=>"coretool already installed", indent=>5);
    } else {
	    $self->_install_coretool();
    }

    ### Create objects for any existing cores on node
    $self->_log(msg=>"creating core objects for any existing cores", indent=>3);

    my @cores=$self->_load_cores();

    ### Return handle to object
    return $self;
}


################################################################################
# Method: _set_autoboot()
# Objective: Set state of autoboot
################################################################################
sub _set_autoboot {
    my ($self,%args)=@_;
    my (
	    $bsdObj,
	    $cmd,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    enable    => {
			    type => SCALAR,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0|on|off|true|false/i;
			     }}},
    }
    );

    $self->_log(
	    msg=>"setting autoboot ".(($args{enable}=~/1|on|t/i) ? "true" : "false"),
	    indent=>3);

    ### Initialize variables
    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $cmd=($args{enable}=~/1|on|t/)
	    ? "sudo sysctl kern.bootargs=AUTOBOOT=true"
	    : "sudo sysctl kern.bootargs=AUTOBOOT=false";

    ### Execute command and parse output
    $bsdObj->execute_raw_command(command=>"$cmd");

    return 1;
}

################################################################################
# Method: _set_gdb_recursive_anic
# Objective: set where ontap drops into gdb on recursive panic
################################################################################
sub _set_gdb_recursive_panic {
    my ($self,%args)=@_;
    my (
	    $bsdObj,
	    $cmd,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    enable    => {
			    type => SCALAR,
			     callbacks=>{
				     'legal values'=>sub{shift()=~/1|0|on|off|true|false/i;
			     }}},
    }
    );

    $self->_log(
	    msg=>"setting gdb-on-recursive-panic ".(($args{enable}=~/1|on|t/i) ? "true" : "false"),
	    indent=>3);

    ### Initialize variables
    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $cmd=($args{enable}=~/1|on|t/)
	    ? "sudo sysctl kern.bootargs=gdb-on-recursive-panic?=true"
	    : "sudo sysctl kern.bootargs=gdb-on-recursive-panic?=false";

    ### Execute command and parse output
    $bsdObj->execute_raw_command(command=>"$cmd");

    return 1;
}

###############################################################################
# Method:    _set_restricted_memsize()
# Objective: Set up firmware environment
###############################################################################
sub _set_restricted_memsize {
    my ($self,%args)=@_;
    my (
        $cmd,
		$console_conn,
		$errmsg,
    );

    ### Transit to firmware
    unless ($self->{transit}->get_state() =~ /FIRMWARE/) {
	    $self->_log(msg=>"changing state to FIRMWARE",indent=>3);
	    $self->{transit}->change_state(to=>'FIRMWARE');
	    #$self->{transit}->change_state(to=>'LOADER_BANNER');
	    $self->{conn_cli_exists}=0;
	    $self->{conn_bsd_exists}=0;
    }

    ### Change environment
    $console_conn=$self->get_conn_obj(type=>"console");
    $cmd = (length $args{memory} and $args{memory} != -1)
	    ? "setenv bootarg.init.restricted_memsize"." ".$args{memory}
	    : "unsetenv bootarg.init.restricted_memsize";
    $self->_log(msg=>"command: ".$cmd,indent=>3);
    &Tharn::say($console_conn,$cmd,-noprompt);


    ### Transit to firmware
    unless ($self->{transit}->get_state() =~ /CLI/) {
	    $self->_log(msg=>"changing state to CLI",indent=>3);
	    $self->{transit}->change_state(to=>"CLI",from=>"FIRMWARE");
    }

    $self->_log(msg=>"re-establishing connection",indent=>3);

    ### Re-establish connections
    $self->get_conn_obj(type=>"cli");
    $self->get_conn_obj(type=>"bsd");
    $self->get_conn_obj(type=>"console");

    return $self;
}

################################################################################
# Subroutine Name: _transit_to
# Objective: transition machine to requested state
################################################################################
sub _transit_to {
    my ($self,%args)=@_;
    my (
		$console_conn,
		$rc,
		$t1,
		$t2,
	);
	use boot;

    ### Validate input parameters
    ($self,%args) = validate_mixed (
        @_,
        [ { type => OBJECT} ],
        {
         mode    => {type => SCALAR,
                       callbacks=>{
                          'legal values'=>sub{shift()=~
                            /FIRMWARE|BOOT_MENU|MAINT|UP/i
                       }}},
         timeout => {type => SCALAR, optional=>1, default=>60,
                              regex => qr/^\d+$/ },
        }
    );

    ### Transit to the requested state
    $console_conn=$self->get_conn_obj(type=>"console",nologin=>1);
	$self->_log(msg=>"transitioning to ".uc($args{mode}).
					 " mode", indent=>7);


	$rc=boot::transit_to(
		$console_conn,
		$args{mode},
		timeout=>$args{timeout});


	return $rc;
}

###############################################################################
# Method:    _verify_marker
# Objective: verify unique marker was written to /mroot/etc/log/messages file
###############################################################################
sub _verify_marker {
    my ($self,%args) = @_;
    my (
	    $bsdObj,
	    $cmd,
	    $duration,
	    $output,
	    $retry_cnt,
	    $t1,
	    $t2,
	    @lines,
    );

    ### Validate input arguments
    ($self,%args) = validate_mixed (
	    @_,
	    [ { type => OBJECT} ],
	    {
		    marker       => {type => SCALAR},
	    }
    );

    $self->_log(
	    msg=>"Waiting for message cookie: '".$args{marker}."'",
	    indent=>3);

    $bsdObj=$self->get_conn_obj(type=>"bsd");
    $cmd="grep '".$args{marker}."' /mroot/etc/log/messages";

    $t1=timelocal(localtime());
    while (1) {
	    $output=$bsdObj->execute_raw_command(command=>"$cmd");
	    if ($output !~ /$args{marker}/) {
		    if ($retry_cnt++<120){
			    sleep(1);
			    redo;
		    }
		    return 0;
	    }
	    last;
    }

    $t2=timelocal(localtime());
    $duration=$t2-$t1;

    ### Display the time duration waiting for 'messages file' to
    ### recieve expected log markder
    $self->_log(msg=>"WARNING: /mroot/etc/log/messages took ".$retry_cnt."s ".
				 "to get log marker") if ($retry_cnt);

    $self->_log(msg=>"duration: ".$duration."s") if ($duration>=2);

    return 1;
}

###############################################################################
# Method:    _wait
# Objective: wait until operation is complete
###############################################################################
sub _wait {
    my ($self,%args)=@_;
        my (
        $cmd,
		$console_conn,
        $raw,
       );

    ### Validate input arguments
	($self,%args) = validate_mixed(
	@_,
	[ { type => OBJECT } ],
	{
		msg      => { type => SCALAR },
		sleep    => {type => SCALAR, default=>$self->{sleep},
					  callbacks=>{
						'Is An Integer>0'=>sub{shift()>0}}},
		timeout  => {type => SCALAR, default=>$self->{timeout},
					  callbacks=>{
						'Is An Integer>0'=>sub{shift()>0}}},
	});

 	my $apiObjCli=$self->get_conn_obj(type=>"cli");


    ### Create command
    $cmd="cf status";

	$self->_log(msg=>"waiting for msg '$args{msg}'",indent=>3);
    ### Execute command in loop until state or
    ### timeout met
    eval {
        local $SIG{ALRM} = sub { die "command timeout (".$args{timeout}."s)"; };
        alarm $args{timeout};
        while (1) {
            eval {
				$raw=$apiObjCli->execute_raw_command(command=>$cmd);
				$Tharn::Log->comment("OUTPUT: |$raw|");
            };
            if ($@){
                next if ($@=~/error sending/i);
                die $@;
            }
            last if ($raw=~/$args{msg}/si);

            Tharn::snooze($args{sleep});
        }
        alarm 0;
    };
    if ($@){
        die $@;
    }

    return $self;
}


1;
