
use Os::EZIO_BASE;
package Os::EZIO_DT_CIFS;
@ISA = qw( Os::EZIO_BASE );
use strict;

use NATE::STAF qw( staf_global staf_wrap );
use Carp;

#sub new {
#    my ($class, %args) = @_;
#    my $workingdir = "/tmp";
#    my $workingpath = "C:/tmp";
#    my $self = bless {
#                    _client         => $args{client}        || croak("missing CLIENT"),
#                    _servername     => $args{servername}    || croak("missing FILER"),
#                    _driveletter    => $args{driveletter}   || "N:",
#                    _rtparameters   => $args{rtparameters}  || undef,
#                    _workdir        => $args{workdir}       || $workingdir,
#                    _sharename      => $args{sharename}     || "ROOTSHARE",
#                    _cifsdomain     => $args{cifsdomain}    || "CSIQA",
#                    _username       => $args{username}      || "Administrator",
#                    _password       => $args{password}      || "netapp",
#                    _runtimeseconds => $args{runtimeseconds} || 0,
#                    _source_file => "/x/eng/localtest/arch/win32/SAN/bin/common/dt.exe",
#                    _source_lib  => "/x/eng/localtest/arch/win32/SAN/bin/common/cygwin1.dll",
#                    _dest_file   =>  $workingpath . "/" . "dt.exe",
#                    _dest_lib    =>  $workingpath . "/" . "cygwin1.dll",
#                    _is_deployed    => 0,
#                    _is_setup       => 0,
#		    _ezio_handle    => "NONE;0;0",,
#		    _output	    => "No Output Captured\n",
#                    _datafilename   => "",
#                    _dispose        =>  $args{dispose}      || 1,
#                    }, $class;
#    my $drive = uc($self->{_driveletter});
#    $drive =~ s/://g;
#    $self->{_driveletter} = $drive;
#    if (! defined($self->{_rtparameters})) {
#        my $uniquenumber = int( rand(100000) );
#        my $datafile = $drive .":" . "/DATA_" . sprintf("%06d",$uniquenumber);
#        my $runtime = " runtime=$self->{_runtimeseconds}";
#        if ($self->{_runtimeseconds} == 0) {
#            $runtime = "";
#        }
#        my $dispose = " dispose=keep ";
#        if ($self->{_dispose} == 1) {
#            $dispose = "";
#        }
#        $self->{_rtparameters} = " of=$datafile bs=33k limit=1m flags=direct " . $dispose . $runtime;
#        my $timestamp = $self->_timestamp();
#    }
#    return $self;
#}
sub _init {
    my ($self, %args) = @_;
    my $workingdir = "/tmp";
    my $workingpath = "C:/tmp";
    
    $self->{_source_file} = "/x/eng/localtest/arch/win32/SAN/bin/common/dt.exe";
    $self->{_source_lib}  = "/x/eng/localtest/arch/win32/SAN/bin/common/cygwin1.dll";
    $self->{_dest_file}   =  $workingpath . "/" . "dt.exe";
    $self->{_dest_lib}    =  $workingpath . "/" . "cygwin1.dll";
    $self->{_workdir}     =  $args{workdir}   || $workingdir;
    my $drive = uc($self->{_driveletter});
    $drive =~ s/://g;
    $self->{_driveletter} = $drive;
    if (! defined($self->{_rtparameters})) {
        my $uniquenumber = int( rand(100000) );
        my $datafile = $drive .":" . "/DATA_" . sprintf("%06d",$uniquenumber);
        my $runtime = " runtime=$self->{_runtimeseconds}";
        if ($self->{_runtimeseconds} == 0) {
            $runtime = " runtime=86400 ";
        }
        my $dispose = " dispose=keep ";
        if ($self->{_dispose} == 1) {
            $dispose = "";
        }
        $self->{_rtparameters} = " of=$datafile bs=33k limit=1m flags=direct " . $dispose . $runtime;
    }
    if (defined($self->{_restorefile})) {
	$self->restoreme($self->{_restorefile});
    }
    $self->validate_parameters();

}

sub validate_parameters {
    my ($self) = @_;
    my $drive  = $self->{_driveletter} ;
 
    if (! defined($self->{_client})) {
        usage();
        croak("Missing CLIENT name.");
    }
    if (! defined($self->{_servername})) {
        usage();
        croak("Missing CIFS Server name.");
    }
    my $time = $self->{_runtimeseconds};
    my $dtparams    = $self->{_rtparameters} ;  

    if (($drive lt "F") || ($drive gt "Z")) {
        usage();
        croak("Invalid drive letter: [" . $drive . "]\n");
    }
    if ($time != 0) {
        if (($time < 100 ) || ($time > 100000)) {
            usage();
            croak("Invalid runtimeseconds parameter");
        }
    }
      if (! defined($self->{_workdir})) {
        usage();
        croak("Invalid \"workdir\" ". "]\n");
    }   
}
sub usage {
    my$usage = "USAGE:\n";
    $usage .= "\t\$dt_tester = new EZIO_DT_CIFS(client=>\"fred\" servername=>\"MYFILERSERVERNAME\" runtimeseconds => 120 ); \n";
    $usage .= "\t\$dt_tester->start();\n";
    $usage .= "\t Do some testing...\n";    
    $usage .= "\t\$dt_tester->stop(); \n";
    $usage .= "\tprint(\$dt_tester->get_output()); \n\n";    
    $usage .= "\tOptional parameters\n";
    $usage .= "\t\tcifsdomain\tCIFS domain for your filer.  Defaults to CSIQA \n";
    $usage .= "\t\tsharename\tshare from your filer.  Defaults to ROOTSHARE \n";
    $usage .= "\t\tusername\tCIFS domain user.  Defaults to <your domain>\\Administrator \n";    
    $usage .= "\t\tpassword\tCIFS domain user password.  Defaults to \"netapp\" \n";    
    $usage .= "\t\tdriveletter\tdriveletter to mount test share on.  Drive F: to Z:.  Defaults to N:\n";
    $usage .= "\t\trtparameters\trun-time parameters to use for DT.EXE.  Defaults to \"of=N:/DATA_XXX bs=33k limit=1m dispose=keep flags=direct \" \n";
    $usage .= "\t\truntimeseconds\tseconds to run. Values 100 to 100000.  Defaults to 0 which is RUN FOREVER.\n";
    $usage .= "\n";
    print $usage;
    return $usage;
}
sub printme {
    my ($self) = @_;
    my $prme = "EZIO_DT_CIFS"
            .  "\n\t_client $self->{_client}"
            . "\n\t_servername $self->{_servername}"
            . "\n\t_driveletter $self->{_driveletter}"
            . "\n\t_rtparameters $self->{_rtparameters}"
            . "\n\t_filecount $self->{_filecount}"
            . "\n\t_workdir $self->{_workdir}"
            . "\n\t_sharename $self->{_sharename}"
            . "\n\t_cifsdomain $self->{_cifsdomain}"
            . "\n\t_username $self->{_username}"
            . "\n\t_password $self->{_password}"
            . "\n\t_source_file $self->{_source_file}"
            . "\n\t_source_lib $self->{_source_lib}"
            . "\n\t_dest_file $self->{_dest_file}"
            . "\n\t_dest_lib $self->{_dest_lib}"
            . "\n\t_is_deployed $self->{_is_deployed}"
            . "\n\t_is_setup $self->{_is_setup}"
            . "\n\t_output $self->{_output}"
            . "\n";
    print($prme);
    return $prme;
}

1;

############################################
# P O D
###########################################

=pod

=head1 EZIO_DT_CIFS

Os::EZIO_DT_CIFS - EZ I/O using DT on a Windows client

=head1 SYNOPSIS

This is a convenience class to create generic DT style I/O (one file, lots of activity) to the filer on a CIFS share.
Created by the OSK/QA group to support creating I/O concurrently with other testing.

B<IMPORTANT:>

This is so simple because it B<ASSUMES the standard OSK/QA filer setup using gx_boot and deux_init>.  If you set up your filer some other way, you will need to use the optional parameters when instantiating one of these objects.


=head1 DESCRIPTION

This class is based on Os::EXIO_BASE.  This class can be instantiated with as few as 2 parameters or as many as 12.  In most cases you
will create the EZIO_DT_CIFS object and call $dt->start() during your test SETUP() routine and call $dt->stop() during your CLEANUP routine.  

USAGE:
        
        $dt_tester = new EZIO_DT_CIFS(
            client=>"fred" servername=>"MYFILERSERVERNAME" runtimeseconds => 120 );

        $dt_tester->start();
         ------- Do some testing... ------------
        $dt_tester->stop(); 
        print($dt_tester->get_output()); 

        Optional parameters
                cifsdomain      CIFS domain for your filer. 
                                        Defaults to CSIQA 
                sharename       share from your filer.
                                        Defaults to ROOTSHARE 
                username        CIFS domain user.
                                        Defaults to <your domain>\Administrator 
                password        CIFS domain user password.  Defaults to "netapp" 
                driveletter     driveletter to mount test share on.
                                        Drive F: to Z:.  Defaults to N:
                rtparameters    parameters to use for DT.EXE.
                                        Defaults to "of=N:/DATA_XXX bs=33k limit=1m dispose=keep flags=direct " 
                runtimeseconds  seconds to run. Values 100 to 100000.
                                        Defaults to 0 which is 1 Day.

=head1 Examples

=over 4

=head2 Example: I want to run DT during my test which takes 2 minutes to run.  

    my $dt = new Os::EZIO_DT_CIFS(
                     client         => "sti-2003-0276.ctl.gdl.englab.netapp.com",
                     servername     => "CSIQA607039VS0",
                     runtimeseconds => 150
                     );
    $dt->start();

    < DO my testing HERE >

    $dt->stop();

    $results = $dt->get_output();

This will run DT.EXE on the client machine using the default share of the given filer and it will run for 150 seconds.
Notice that DT is set to run for 150 seconds even though the test runs for 2 minutes (120 seconds).  That's OK because $dt->stop()
actually kills DT if it is still running.  When done, use $dt->get_output() method.  It returns a giant string with the output of
the DT run.  You may want to put it in your NTEST log.

=head2 Example: I need DT to run for a minimum of 10 minutes, but my test may finish earlier.  How do I make sure DT runs the required 10 minutes?

    my $dt = new Os::EZIO_DT_CIFS(
                     client         => "sti-2003-0276.ctl.gdl.englab.netapp.com",
                     servername     => "CSIQA607039VS0",
                     runtimeseconds => 600
                     );
    $dt->start();

    < DO my testing HERE >

    $dt->finish();

Notice that I called $dt->finish() instead of stop().  finish() will let DT "finish" or run to completion before the call returns.

=head2 Example: How do I know if DT is finished during my test?

    my $dt = new Os::EZIO_DT_CIFS(
                     client         => "sti-2003-0276.ctl.gdl.englab.netapp.com",
                     servername     => "CSIQA607039VS0",
                     runtimeseconds => 150
                     );
    $dt->start();

    < DO my testing HERE >
    
    my $output="";
    my $isdone = $dt->isdone();
    if ($isdone) {
        $output = $dt->get_output();
    } else {
        $dt->finish();
        $output = $dt->get_output();
    }


=back

=head1 Constructor and Initialization

        sub new {
            my ($class, %args) = @_;
            my $workingdir = "/tmp";
            my $workingpath = "C:/tmp";
            my $self = bless {
                            _client         => $args{client}        || croak("missing CLIENT"),
                            _servername     => $args{servername}    || croak("missing FILER"),
                            _driveletter    => $args{driveletter}   || "N:",
                            _rtparameters   => $args{rtparameters}  || undef,
                            _workdir        => $args{workdir}       || $workingdir,
                            _sharename      => $args{sharename}     || "ROOTSHARE",
                            _cifsdomain     => $args{cifsdomain}    || "CSIQA",
                            _username       => $args{username}      || "Administrator",
                            _password       => $args{password}      || "netapp",
                            _runtimeseconds => $args{runtimeseconds} || 0,
                            _source_file => "/x/eng/localtest/arch/win32/SAN/bin/common/dt.exe",
                            _source_lib  => "/x/eng/localtest/arch/win32/SAN/bin/common/cygwin1.dll",
                            _dest_file   =>  $workingpath . "/" . "dt.exe",
                            _dest_lib    =>  $workingpath . "/" . "cygwin1.dll",
                            _is_deployed    => 0,
                            _is_setup       => 0,
                            _ezio_handle    => "NONE;0;0",,
                            _output	    => "No Output Captured\n",
                            _datafilename   => "",
                            _dispose        =>  $args{dispose}      || 1,
                            }, $class;
                            
=head2 Class and object methods

=head1 ENVIRONMENT

This class assumes the environment provided by the OSK/QA filer initialization method of gx_boot, deux_init with standard QA parameters.

=head1 FILES

This class inherits from Os::EZIO_BASE.pm


