
use Os::EZIO_BASE;
package Os::EZIO_HMR_CIFS;
@ISA = qw( Os::EZIO_BASE );
use strict;
use NATE::STAF qw( staf_global staf_wrap );
use Carp;

## NOTE:
## The location of executables is taken from:
##     http://thebrewery-web.corp.netapp.com/Brewery/brewery/QA/Tools/IOTools/Hammer/index.html
##
	#hammer version $Id: //depot/prod/test/main/ontap/main/wafl/hammer/hammer.c#3 $
	#Usage: hammer [path] -filercore FILER [OPTION ..]
	#-api {win32 | posix}         use specified API
	#-bg                          don't use stdin or stdout
	#-checkinodes                 file system reusing inodes?
	#-filercore FILERNAME         have filer coredump upon corruption. Default
	#-nofilercore                 disable default -filercore option
	#-fill                        fill disk and then keep it full
	#-interactive                 use interactive mode
	#-iterations NUMBER           number of iterations to execute
	#-logfile FILE                use logfile FILE
	#-mode {mixed | creates | overwrites}      use specified mode
	#-nocleanup                   don't remove files upon completion
	#-noflush                     don't use FlushFileBuffers()
	#-onlyflush                   always use FlushFileBuffers()
	#-nostreams                   don't use NT stream files
	#-runtime N                   stop hammering after N seconds
	#-seed N                      set initial random seed to N
	#-version                     print hammer version
	#
	#-blocksize NBYTES            use blocksize NBYTES
	#-blocksize random            use a random blocksize
	#-minbsize NBYTES             set minimum block size to NBYTES
	#-maxbsize NBYTES             set maximum block size to NBYTES
	#-minfsize NBYTES             set minimum file  size to NBYTES
	#-maxfsize NBYTES             set maximum file  size to NBYTES

sub _init {
    my ($self, %args) = @_;
    my $workingdir = "/tmp";
    my $workingpath = "C:/tmp";
    
    $self->{_source_file} = "/u/mueller/hammer/windows/hammer.exe",
    $self->{_dest_file}   =  $workingpath . "/" . "hammer.exe",
    $self->{_workdir}     =  $args{workdir}   || $workingdir;
    my $drive = uc($self->{_driveletter});
    $drive =~ s/://g;
    $self->{_driveletter} = $drive;
    if (! defined($self->{_rtparameters})) {
        my $uniquenumber = int( rand(100000) );
        my $datafile = $drive .":" . "/ ";
        my $runtime = " -runtime $self->{_runtimeseconds}";
        if ($self->{_runtimeseconds} == 0) {
            $runtime = "";
        }
        my $dispose = " -nocleanup ";
        if ($self->{_dispose} == 1) {
            $dispose = "";
        }
        $self->{_rtparameters} = " $datafile -api win32 -nofilercore " . $dispose . $runtime;
    }
    if (defined($self->{_restorefile})) {
	$self->restoreme($self->{_restorefile});
    }    
    $self->validate_parameters();
}

sub validate_parameters {
    my ($self) = @_;
    my $drive  = $self->{_driveletter} ;
  
    if (! defined($self->{_client})) {
        usage();
        croak("Missing CLIENT name.");
    }
    if (! defined($self->{_servername})) {
        usage();
        croak("Missing CIFS Server name.");
    }
    my $time = $self->{_runtimeseconds};
    my $dtparams    = $self->{_rtparameters} ;  

    if (($drive lt "F") || ($drive gt "Z")) {
        usage();
        croak("Invalid drive letter: [" . $drive . "]\n");
    }
   if (! defined($self->{_workdir})) {
        usage();
        croak("Invalid \"workdir\" " . "]\n");
    }    
    if ($time != 0) {
        if (($time < 100 ) || ($time > 100000)) {
            usage();
            croak("Invalid runtimeseconds parameter");
        }
    }
}
sub usage {
    my$usage = "USAGE:\n";
    $usage .= "\t\$hmr_tester = new EZIO_HMR_CIFS(client=>\"fred\" servername=>\"MYFILERSERVERNAME\" runtimeseconds => 120 ); \n";
    $usage .= "\t\$hmr_tester->start();\n";
    $usage .= "\t Do some testing...\n";    
    $usage .= "\t\$hmr_tester->stop(); \n";
    $usage .= "\tprint(\$hmr_tester->get_output()); \n\n";    
    $usage .= "\tOptional parameters\n";
    $usage .= "\t\tcifsdomain\tCIFS domain for your filer.  Defaults to CSIQA \n";
    $usage .= "\t\tsharename\tshare from your filer.  Defaults to ROOTSHARE \n";
    $usage .= "\t\tusername\tCIFS domain user.  Defaults to <your domain>\\Administrator \n";    
    $usage .= "\t\tpassword\tCIFS domain user password.  Defaults to \"netapp\" \n";    
    $usage .= "\t\tdriveletter\tdriveletter to mount test share on.  Drive F: to Z:.  Defaults to N:\n";
    $usage .= "\t\trtparameters\tparameters to use for HAMMEER.EXE.  Defaults to \" N:\ -api win32 -nofilercore  \" \n";
    $usage .= "\t\truntimeseconds\tseconds to run. Values 100 to 100000.  Defaults to 0 which is RUN FOREVER.\n";
    $usage .= "\n";
    print $usage;
    return $usage;
}
sub printme {
    my ($self) = @_;
    my $prme = "EZIO_HMR_CIFS"
            .  "\n\t_client $self->{_client}"
            . "\n\t_servername $self->{_servername}"
            . "\n\t_driveletter $self->{_driveletter}"
            . "\n\t_rtparameters $self->{_rtparameters}"
            . "\n\t_filecount $self->{_filecount}"
            . "\n\t_workdir $self->{_workdir}"
            . "\n\t_sharename $self->{_sharename}"
            . "\n\t_cifsdomain $self->{_cifsdomain}"
            . "\n\t_username $self->{_username}"
            . "\n\t_password $self->{_password}"
            . "\n\t_source_file $self->{_source_file}"
            . "\n\t_source_lib $self->{_source_lib}"
            . "\n\t_dest_file $self->{_dest_file}"
            . "\n\t_dest_lib $self->{_dest_lib}"
            . "\n\t_is_deployed $self->{_is_deployed}"
            . "\n\t_is_setup $self->{_is_setup}"
            . "\n\t_output $self->{_output}"
            . "\n";
    print($prme);
    return $prme;
}
1;

############################################
# P O D
###########################################

=pod

=head1 EZIO_HMR_CIFS

Os::EZIO_HMR_CIFS - EZ I/O using Hammer on a Windows client

=head1 SYNOPSIS

This is a convenience class to create generic Hammer style I/O (one file, lots of activity) to the filer on a CIFS share.
Created by the OSK/QA group to support creating I/O concurrently with other testing.

B<IMPORTANT:>

This is so simple because it B<ASSUMES the standard OSK/QA filer setup using gx_boot and deux_init>.  If you set up your filer some other way, you will need to use the optional parameters when instantiating one of these objects.


=head1 DESCRIPTION

This class is based on Os::EXIO_BASE.  This class can be instantiated with as few as 2 parameters or as many as 12.  In most cases you
will create the EZIO_HMR_CIFS object and call $hmr->start() during your test SETUP() routine and call $hmr->stop() during your CLEANUP routine.  

USAGE:
        
        $hmr_tester = new EZIO_HMR_CIFS(
            client=>"fred" servername=>"MYFILERSERVERNAME" runtimeseconds => 120 );

        $hmr_tester->start();
         ------- Do some testing... ------------
        $hmr_tester->stop(); 
        print($hmr_tester->get_output()); 

        Optional parameters
                cifsdomain      CIFS domain for your filer. 
                                        Defaults to CSIQA 
                sharename       share from your filer.
                                        Defaults to ROOTSHARE 
                username        CIFS domain user.
                                        Defaults to <your domain>\Administrator 
                password        CIFS domain user password.  Defaults to "netapp" 
                driveletter     driveletter to mount test share on.
                                        Drive F: to Z:.  Defaults to N:
                rtparameters    parameters to use for HAMMER.EXE.
                                        Defaults to "" N:\ -api win32 -nofilercore " 
                runtimeseconds  seconds to run. Values 100 to 100000.
                                        Defaults to 0 which is RUN FOREVER.

=head1 Examples

=over 4

=head2 Example: I want to run DT during my test which takes 2 minutes to run.  

    my $hmr = new Os::EZIO_HMR_CIFS(
                     client         => "sti-2003-0276.ctl.gdl.englab.netapp.com",
                     servername     => "CSIQA607039VS0",
                     runtimeseconds => 150
                     );
    $hmr->start();

    < DO my testing HERE >

    $hmr->stop();

    $results = $hmr->get_output();

This will run HAMMER.EXE on the client machine using the default share of the given filer and it will run for 150 seconds.
Notice that HAMMER is set to run for 150 seconds even though the test runs for 2 minutes (120 seconds).  That's OK because $hmr->stop()
actually kills HAMMER if it is still running.  When done, use $hmr->get_output() method.  It returns a giant string with the output of
the HAMMER run.  You may want to put it in your NTEST log.

=head2 Example: I need HAMMER to run for a minimum of 10 minutes, but my test may finish earlier.  How do I make sure HAMMER runs the required 10 minutes?

    my $hmr = new Os::EZIO_HMR_CIFS(
                     client         => "sti-2003-0276.ctl.gdl.englab.netapp.com",
                     servername     => "CSIQA607039VS0",
                     runtimeseconds => 600
                     );
    $hmr->start();

    < DO my testing HERE >

    $hmr->finish();

Notice that I called $hmr->finish() instead of stop().  finish() will let DT "finish" or run to completion before the call returns.

=head2 Example: How do I know if HAMMER is finished during my test?

    my $hmr = new Os::EZIO_HMR_CIFS(
                     client         => "sti-2003-0276.ctl.gdl.englab.netapp.com",
                     servername     => "CSIQA607039VS0",
                     runtimeseconds => 150
                     );
    $hmr->start();

    < DO my testing HERE >
    
    my $output="";
    my $isdone = $hmr->isdone();
    if ($isdone) {
        $output = $hmr->get_output();
    } else {
        $hmr->finish();
        $output = $hmr->get_output();
    }


=back

=head1 Constructor and Initialization

        sub new {
            my ($class, %args) = @_;
            my $workingdir = "/tmp";
            my $workingpath = "C:/tmp";
            my $self = bless {
                            _client         => $args{client}        || croak("missing CLIENT"),
                            _servername     => $args{servername}    || croak("missing FILER"),
                            _driveletter    => $args{driveletter}   || "N:",
                            _rtparameters   => $args{rtparameters}  || undef,
                            _workdir        => $args{workdir}       || $workingdir,
                            _sharename      => $args{sharename}     || "ROOTSHARE",
                            _cifsdomain     => $args{cifsdomain}    || "CSIQA",
                            _username       => $args{username}      || "Administrator",
                            _password       => $args{password}      || "netapp",
                            _runtimeseconds => $args{runtimeseconds} || 0,
                            _source_file => "/u/mueller/hammer/windows/hammer.exe",
                            _source_lib  =>  undef,
                            _dest_file   =>  $workingpath . "/" . "dt.exe",
                            _dest_lib    =>  undef,
                            _is_deployed    => 0,
                            _is_setup       => 0,
                            _ezio_handle    => "NONE;0;0",,
                            _output	    => "No Output Captured\n",
                            _datafilename   => "",
                            _dispose        =>  $args{dispose}      || 1,
                            }, $class;
                            
=head2 Class and object methods

=head1 ENVIRONMENT

This class assumes the environment provided by the OSK/QA filer initialization method of gx_boot, deux_init with standard QA parameters.

=head1 FILES

This class inherits from Os::EZIO_BASE.pm

