#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/Os/Event.pm#1 $

## @summary Event interface
##
## @author knopp@netapp.com
## @status private
## @pod here

package Os::Event;

use San::SanLog;
use Error;
use vars qw( @EXPORT @ISA);

my $Cnt=int(rand(1000));
my %Mib;

@ISA = qw(Error SanLog);

################################################################################
###
###  P U B L I C   F U N C T I O N S
###
################################################################################


################################################################################
### Subroutine Name: create_snmp_message()
### Objective:       create a new snmp message to send
### Details:         See POD documentation at the end of this file
################################################################################
sub create_snmp_message {
    my ($self,%args)=@_;
    $self->_clear_err(); 

    return $self->_trim(sprintf($self->get_snmp_format(),$self->_get_param_values(%args)));
}

##################################################################################
### Subroutine Name: create_syslog_message()
### Objective:       create a new syslog message to send
### Details:         See POD documentation at the end of this file
##################################################################################
sub create_syslog_message {
    my ($self,%args)=@_;
    $self->_clear_err();

    return $self->_trim(sprintf($self->get_syslog_format(),$self->_get_param_values(%args)));
}

##################################################################################
### Subroutine Name: generate()
### Objective:       Invoke event message
### Details:         See POD documentation at the end of this file
##################################################################################
sub generate {
    my ($self,%args)=@_;
    my (
	$values,
	@values,
	);
    
    $self->_clear_err();    

    $large=0;

    ### parse arguments
    foreach (keys %args) {
      if (/-?value/i) {
	if (not length($values= $args{$_})) {
	    $self->_set_err("Error: Event::generate() Invalid value for argument '-values'");
	    return $self;
	} 

        if ($values eq "ARRAY") {
	    $values=join(",",@{$values});
	}
	
	next;
      }
      else {
	$self->_set_err("Error: Event::generate() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    }     

    $values=(defined $values)
	? $values
	: join(",",@{$self->{values}});

    if (not defined $values){
	$self->_set_err("Error: Event::generate() No argument values were provided. Values can be",
	                "provided as as array ref using parameter '-values', or by executing",
	                "create_snmp_message() or create_syslog_message() prior to running ",
	                "this command");
	return $self;                      	  
    }

    eval {
	$self->{server}=$self->{parent}->get_server();
        $self->{server}->set_execution_server(-Server=>$self->get_node_name());
        $self->{server}->event_generate(messagename=>$self->get_name(),values=>$values);
    };
    if ($@){
	$self->_set_err("ERROR: Event::generate() Event ".$self->get_name()." with params was unsuccessful");
    }

    return $self;
}
##################################################################################
### Subroutine Name: get_log_entry()
### Objective:       return event log entry
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_log_entry {
    my ($self,%args)=@_;
    my (
	$found,
	$query_cnt,
	$retry,
	$range,
	$seq_id,
	$seq_id_low,
	$seq_id_high,
	@output,
	);


    $self->_clear_err();

    $found=0;
    $retry=3;
    $range=10;
    $query_cnt=1;

    # parse arguments
    foreach (keys %args) {
      if (/-?retry/) {
	if (not length($retry = $args{$_})) {
		    $self->_set_err("Error: Catalog::get_log_entry() Invalid value for argument '-retry'");
		    return $self;
	}        
	next;
      }
      elsif (/-?range/) {
	if (not length($range = $args{$_})) {
		    $self->_set_err("Error: Catalog::get_log_entry() Invalid value for argument '-range'");
		    return $self;
	}        
	next;
      }
     else {
	$self->_set_err("Error: Catalog::get_log_entry() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    } 
    
    $self->{server}=$self->{parent}->get_server();
    $self->{server}->set_execution_server(-Server=>$self->get_node_name());

    $seq_id=$self->{parent}->get_last_seqid();

    while ($query_cnt<=$retry){
	eval {
	    ### NOTE: Search for event within a range of sequence numbers for best performance.  
	    ### A range is needed since other events are occuring during test that makes 
	    ### it impossible to accurately predict event seqid to use for fast query match. 
	    ### Don't search using "<sequid" or by just the event name since this causes a full
	    ### scan of the event log which is VERY SLOW.  Try query $retry times while increasing 
	    ### the search range by a factor of $row_span. 
	    $seq_id_low=$seq_id-($range*$query_cnt);
            $seq_id_high=$seq_id+($range*$query_cnt);
	    @output=$self->{server}->event_log_show(
		    node=>$self->get_node_name(), 
		    messagename=>$self->get_name(),
		    seqnum=>$seq_id_low."..".$seq_id_high);
	    $found=1;

	};
	if ($@){
	    $self->sanlog(-msg=>"\t\tWARNING: Log of event not found within log sequence range(".$seq_id_low."..".$seq_id_high.")");
	    $seq_id=$self->{parent}->get_last_seqid();
	}
	$query_cnt++;
	last if ($found);

    }

    if (not $found){
	    $self->_set_err("ERROR: Catalog::get_log_entry() Unable to locate event in last ".$range." log entries",$@);
	    return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @output : [@output];
}


##################################################################################
### Subroutine Name: get_name()
### Objective:       return event name
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_name {
    my ($self)=@_;
    $self->_clear_err(); 

    return $self->{name};
}

##################################################################################
### Subroutine Name: get_node_name()
### Objective:       return node responsible for event
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_node_name {
    my ($self)=@_;
    $self->_clear_err(); 

    return (defined $self->{node_name})
	? $self->{node_name} 
	: ($self->{parent}->get_server()->servers())[0];
}

##################################################################################
### Subroutine Name: get_param_types()
### Objective:       return event parameter types
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_param_types {
    my ($self,$event)=@_;
    my (
	@params,
	@types,
    );

    $self->_clear_err();

    @params=@{$self->{event}->{param}} if (defined @{$self->{event}->{param}});

    ### Iterate through event parameters and populate with correct datatype 
    foreach my $param (@params){
	push @types,$param->{type};
    }	

    return @types;
}
##################################################################################
### Subroutine Name: get_route()
### Objective:       return event destinations
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_route {
    my ($self)=@_;
    my (
	$output,
	@output,
	);

    $self->{server}=$self->{parent}->get_server();
    $self->{server}->set_execution_server(-Server=>$self->get_node_name());

    eval {
	@output=$self->{server}->event_route_show(messagename=>$self->get_name());	
	$output=$output[0]->{destinations};   
    };
    if ($@){
	$self->_set_err("ERROR: Event::get_route() Unable to retrieve event ".$event->get_name()." route ");
	return "";
    }

    return $output;
}


##################################################################################
### Subroutine Name: get_severity()
### Objective:       return event severity
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_severity {
    my ($self)=@_;
    $self->_clear_err(); 

    return $self->{severity};
}

##################################################################################
### Subroutine Name: get_snmp_format()
### Objective:       return snmp format to use for event description
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_snmp_format {
    my ($self)=@_;
    $self->_clear_err(); 

    return $self->{snmp_format};
}

##################################################################################
### Subroutine Name: get_syslog_format()
### Objective:       return syslog format to use for event description
### Details:         See POD documentation at the end of this file
##################################################################################
sub get_syslog_format {
    my ($self)=@_;
    $self->_clear_err(); 

    return $self->{syslog_format};
}

##################################################################################
### Subroutine Name: is_snmp_supported()
### Objective:       return 1|0 on whether event supports snmp
### Details:         See POD documentation at the end of this file
##################################################################################
sub is_snmp_supported {
    my ($self)=@_;
    $self->_clear_err(); 

    return $self->{snmp_support};
}

##################################################################################
### Subroutine Name: modify_route()
### Objective:       modify route destinations
### Details:         See POD documentation at the end of this file
##################################################################################
sub route_modify{
    my ($self,$destinations,%args)=@_;
    my (
	$frequency,
	$time,
	%optional_of,
	);

    $self->_clear_err();

    # parse arguments
    foreach (keys %args) {
      if (/-?freq/) {
	if (not length($frequency = $args{$_})) {
		    $self->_set_err("Error: Event::route_modify() Invalid value for argument '-frequencythreshold'");
		    return $self;
	}        
	$optional_of{frequencythreshold}=$frequency;
	next;
      }
      elsif (/-?time/) {
	if (not length($time = $args{$_})) {
		    $self->_set_err("Error: Event::route_modify() Invalid value for argument '-timethreshold'");
		    return $self;
	}        
	$optional_of{timethreshold}=$time;
	next;
      }
     else {
	$self->_set_err("Error: Event::route_modify() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    } 


    $destinations="-" if (not defined $destinations);

    eval {
      $self->{server}=$self->{parent}->get_server();
      $self->{server}->set_execution_server(-Server=>$self->get_node_name());
      $self->{server}->event_route_modify(messagename=>$self->get_name(), destinations=>$destinations, %optional_of);
    };
    if ($@) {
      $self->_set_err("ERROR: Unable to modify ".$self->get_name()." route to ".$destinations,$@);
      return 0;
    }

    return 1;
}


################################################################################
### Subroutine Name: route_show()
### Objective:       Display event route
### Details:         See POD documentation at the end of this file
################################################################################
sub route_show {
    my ($self)=@_;
    my (
	 @buffer,
	 @output,
	);

    $self->_clear_err();

    eval {
        $self->{server}=$self->{parent}->get_server();
        $self->{server}->set_execution_server(-Server=>$self->get_node_name());
	@buffer=$self->{server}->event_route_show(messagename=>$self->get_name());	
    };
    if ($@){
	$self->_set_err("ERROR: Event::route_show() Unable to show event route");
	return 0;
    }

    foreach my $row (@buffer){
      foreach my $key (keys %$row){
        if (ref $row->{$key} eq "ARRAY"){
	    push @output,sprintf("%35.35s: %-s",$key,join(",",@{$row->{$key}}));
	}
	else {
	    push @output,sprintf("%35.35s: %-s",$key,$row->{$key});   
	}
      }
    }

    return (wantarray()) ? @output : [@output];

}


##################################################################################
### Subroutine Name: set_node_name()
### Objective:       set node responsible for event
### Details:         See POD documentation at the end of this file
##################################################################################
sub set_node_name {
    my ($self,$node_name)=@_;
    $self->_clear_err(); 

    $self->{node_name}=$node_name;


}

################################################################################
### Subroutine Name: status_reset()
### Objective:       reset event status
### Details:         See POD documentation at the end of this file
################################################################################
sub status_reset {
    my ($self)=@_;
    my (
	 @buffer,
	 @output,
	);

    $self->_clear_err();

    eval {
        $self->{server}=$self->{parent}->get_server();
        $self->{server}->set_execution_server(-Server=>$self->get_node_name());
	$self->{server}->event_status_reset(messagename=>$self->get_name(), node=>$self->get_node_name());	
    };
    if ($@){
	my $name = $self->get_node_name();
	$self->_set_err("ERROR: Event::status_reset() Unable to reset event message statistics: $name");
	return 0;
    }

    
    return 1;

}



################################################################################
### Subroutine Name: status_show()
### Objective:       Display current event configuration attributes
### Details:         See POD documentation at the end of this file
################################################################################
sub status_show {
    my ($self)=@_;
    my (
	 @buffer,
	 @output,
	);

    $self->_clear_err();

    eval {
        $self->{server}=$self->{parent}->get_server();
        $self->{server}->set_execution_server(-Server=>$self->get_node_name());
	@buffer=$self->{server}->event_status_show(messagename=>$self->get_name());	
    };
    if ($@){
	$self->_set_err("ERROR: Event::status_show() Unable to show event route");
	return 0;
    }

    foreach my $row (@buffer){
      foreach my $key (keys %$row){
        if (ref $row->{$key} eq "ARRAY"){
	    push @output,sprintf("%35.35s: %-s",$key,join(",",@{$row->{$key}}));
	}
	else {
	    push @output,sprintf("%35.35s: %-s",$key,$row->{$key});   
	}
      }
    }

    return (wantarray()) ? @output : [@output];

}



################################################################################
###
###  P R I V A T E   F U N C T I O N S
###
################################################################################

##################################################################################
### Subroutine Name: get_param_values()
### Objective:       return random parameter values
### Details:         See POD documentation at the end of this file
##################################################################################
sub _get_param_values {
    my ($self,%args)=@_;
    my (
	@params,
	@values, 
	$large,
    );

    $self->_clear_err();

    $large=0;

    foreach (keys %args) {
      if (/-?large/i) {
	if (not length($large= $args{$_})) {
	    $self->_set_err("Error: Event::_get_param_values() Invalid value for argument '-large'");
	    return $self;
	} 	
	next;
      }
      else {
	$self->_set_err("Error: Event::get_param_values() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    }     


    @params=@{$self->{event}->{param}} if (defined @{$self->{event}->{param}});

    ### Iterate through event parameters and populate with correct datatype 
    if ($large){
	foreach my $param (@params){
	    if ($param->{type}=~/^INT$|^PTR$/) {
		push @values,2147483647;  
	    }
	    elsif($param->{type}=~/^LONGINT$/) {
		## BUG: INTHEX doesn't work unless only integer
		#push @values,++$cnt."F";
		push @values,9223372036854775807;
	    }
	    elsif($param->{type}=~/^INTHEX$/) {
		## BUG: INTHEX doesn't work unless only integer
		push @values,2147483647;
	    }
	    elsif($param->{type}=~/^STRING$|^STRINGARRAY$/) {
		my $str="large".++$Cnt;
		push @values,"$str"x32;
	    }
	    else{
		$set->_set_err("ERROR: Event::get_max_values() Unable to find parameter type ".$param->{type});
		return "";
	    }
	}
    }
    else {
	foreach my $param (@params){
	    if ($param->{type}=~/^INT$|^LONGINT$|^PTR$/) {
		push @values,++$Cnt;  
	    }
	    elsif($param->{type}=~/^INTHEX$/) {
		## BUG: INTHEX doesn't work unless only integer
		#push @values,++$cnt."F";
		push @values,++$Cnt;
	    }
	    elsif($param->{type}=~/^STRING$|^STRINGARRAY$/) {
		push @values,"str".++$Cnt;
	    }
	    else{
		$self->_set_err("ERROR: Event::get_param_values() Unable to find parameter type ".$param->{type});
		return "";
	    }
	}
    }

    ### Note:  The diagnostic 'event generate' command has '-values' as a required parameter. For events ",
    ### that don't have arguments use an '*' as the default value;

    $self->{values}=(@values) ? [@values] : [("*")];

    return (@values) ? @values : ("*");
}


##################################################################################
### Subroutine Name: _new()
### Objective:       Create a new event object
### Details:         See POD documentation at the end of this file
##################################################################################
sub _new {
    my ($class, $event_of, %args) = @_;
    my ( 
        $log,
	$build_root,
	$find_str,
	$parent,
	$self,
	@rows,
	);

    $self={};

    bless $self,$class;	

    # parse arguments
    foreach (keys %args) {
      if (/-?build/i) {
	if (not length($build_root = $args{$_})) {
		    $self->_set_err("Error: Event::_new() Invalid value for argument '-build_root'");
		    return $self;
	}        
	next;
      }
      elsif (/-?parent/) {
	if (not length($parent = $args{$_})) {
		    $self->_set_err("Error: Event::_new() Invalid value for argument '-parent'");
		    return $self;
	}        
	next;
      }
      elsif (/-?log/) {
	if (not length($log = $args{$_})) {
		    $self->_set_err("Error: Event::_new() Invalid value for argument '-loghdl'");
		    return $self;
	}        
	next;
      }
      else {
	$self->_set_err("Error: Event::_new() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    }     
   
    $self->{log}=$log if (defined $log);
   

    ### Retrieve hash of event types that support snmp
    if (not defined %Mib){
	#$find_str = "'{ netapp 0 ' ".$build_root."/smf/src/snmp/mibs/network-appliance.mib";
	$find_str = "'{ netapp 0 ' ".$build_root."/ontap/files/netapp.mib";
	@rows=`grep $find_str`;
	foreach my $row (@rows){
	    my ($type)=$row=~/netapp 0 (\d+)/;
	    $Mib{$type}=1;
	}
    }

    $self->{parent}=$parent;
    $self->{node_name}=$self->get_node_name();
    $self->{snmp_support}=0;
    $self->{snmp_format}="";


    if (defined $event_of->{'snmp'}){
	
	my $type=$event_of->{snmp}->[0]->{'enterprise-trap'}->[0]->{'specific-type'};

	if ($Mib{$type}){
	    $self->{snmp_support}=1;
	    $format_snmp=$event_of->{snmp}->[0]->{'enterprise-trap'}->[0]->{'string-var'}->[0]->{format}->[0];
	    $format_snmp=~s/\cJ/ /g;
	    $format_snmp=~s/\cI//g;
	    $format_snmp=~s/\\t/ /g;
	    $format_snmp=~s/\\n//g;    
	    $self->{snmp_format}=$format_snmp;
	}
    }

    $self->{event}=$event_of;
    $self->{name}=$event_of->{name};
    $self->{severity}=(defined $event_of->{syslog}->[0]->{severity})
	? $event_of->{syslog}->[0]->{severity}
        : $event_of->{severity};

    $self->{syslog_format}=$event_of->{syslog}->[0]->{format}->[0];
    $self->{syslog_format}=~s/\cJ/ /g;
    $self->{syslog_format}=~s/\cI//g;	  


    return $self;

}

##################################################################################
### Subroutine Name: _remove whites space 
### Objective:       Create a new event object
### Details:         See POD documentation at the end of this file
##################################################################################
sub _trim {
    my ($self,@out) = @_;
    for (@out) {
        s/^\s+//;
        s/\s+$//;
    }
    return wantarray ? @out : $out[0];
}


1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

Event.pm

=head1 Author

Gary Knopp

=head1 Description

Event.pm provides an API interface for executing node event commands using the diagnostic utility 'event generate'.  Instantiation of events 
objects using this package is automatically done during creation of an EventMgr 
object.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error>

B<San::SanLog>

=head1 Public Methods

=head2 B<create_snmp_message(I<-large>)>

This method creates and returns the snmp message that will be sent by the event

=over 4

=item Inputs

=item * I<-large> default:0

This parameter determines whether large parameters values are created for string and number 
parameter types.  For a number type the largest value for that platform is used. Whereas each 
string type usse the format value'large'.$cnt++ repeated 64 times.  By default, number and string 
parameters use an icrementing counter to set thier values as  $cnt++ and 'str'.$cnt++ respectively.

=item Outputs

Returns a scalar containing the message the event will when $event_obj->generate() is issued.
If an error occured the error flag is set, errmsg is set to the failure reason.

=item Examples

     # create and return the expected event message
     $snmp_msg = $event_obj->create_snmp_message();

=back

=head2 B<create_syslog_message(I<-large>)>

This method creates and returns the syslog message that will be sent by the event

=over 4

=item Inputs

=item * I<-large> default: 0

This parameter determines whether large parameters values are created for string and number 
parameter types.  For a number type the largest value for that platform is used. Whereas each 
string type usse the format value'large'.$cnt++ repeated 64 times.  By default, number and string 
parameters use an icrementing counter to set thier values as  $cnt++ and 'str'.$cnt++ respectively.

=item Outputs

Returns a scalar containing the message the event will when $event_obj->generate() is issued.
If an error occured the error flag is set, errmsg is set to the failure reason.

=item Examples

     # create and return the expected event message
     $snmp_msg = $event_obj->create_syslog_message();

=back

=head2 B<generate(I<-values>)>

This method generates the event message.

=over 4

=item Inputs

=item * I<-values> default: undef

An array reference or scalar list of parameter values for the required event to use in generating 
its event message.


=item Outputs

Returns its object handle. If an error occured the error flag is set, errmsg is set to the failure
reason.

=item Examples

     # generate event
     $event_obj->generate();
     if ($event_obj->errmsg) {
	 $Test->nlog($event_obj->errmsg());
	 return 0;
     }

=back

=head2 B<get_log_entry($seq_id,I<-row_span,-retry>)>

This method retrieves the event entry from the node event log.   

=over 4

=item Inputs

=item * $seq_id

Event sequence id to use (required) as the starting point to search for the event.  The log is 
searched for the event between log sequence id [seq_id] to [seq_id + row_span].   

=item * I<-row_span> default: 15

The number of rows to search for in the event log for the event entry.  A search range is needed
since other events are occuring on the node that are logged and prevent a predictable sequence id
for a generated event.

=item * I<-query_cnt> default: 3

The number of times to perform the search while incrementing the search range up to [row_span * query_cnt]

=item Outputs

Returns a scalar value of 1 on success and 0 on failure. If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

    # delete cg snapshot
    $seq_id=$eventMgr_obj->get_last_seqid();
    $cgObj->get_log_entry(-seq_id=>$seq_id);   
    if ($cgObj->error) {
	 $Test->nlog($cgObj->errmsg()
	 return $MTD::FAIL;
    }

=back

=head2 B<get_name()>

Retrieve event name.

=over 4

=item Inputs

None 

=item Outputs

Returns event name.

=item Examples

     #retrieve cgid value
     $name=$event_obj->get_name();
    

=back

=head2 B<get_node_name()>

Retrieve node name responsible for issuing event

=over 4

=item Inputs

None 

=item Outputs

Returns event name.

=item Examples

     #retrieve cgid value
     $node_name=$event_obj->get_node_name();
    

=back

=head2 B<get_param_types()>

Retrieve parameter types for event arguments

=over 4

=item Inputs

None 

=item Outputs

Returns an array or array reference of parameter types 

=item Examples

     #retrieve cgid value
     @types=$event_obj->get_param_types();
    

=back

=head2 B<get_route()>

Retrieve route destinations for event

=over 4

=item Inputs

None 

=item Outputs

Returns an array or array reference of event destinations (client host names).

=item Examples

     #retrieve cgid value
     @destinations=$event_obj->get_route();
    

=back



=head2 B<get_severity()>

Retrieve severity level of event

=over 4

=item Inputs

None

=item Outputs

Returns a scalar string containing the severity level. 

=item Examples

    # restore cg snapshot
    $severity=$event_obj->get_severity();   

=back

=head2 B<get_snmp_format()>

This method returns the format string to use for the snmp message.

=over 4

=item Inputs

None

=item Outputs

Returns a scalar containing the snmp message format. If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

    # delete cg snapshot
    $format=$event_obj->get_snmp_message();   

=back

=head2 B<get_syslog_format()>

This method returns the format string to use for the syslog message.

=over 4

=item Inputs

None

=item Outputs

Returns a scalar containing the syslog message format. If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

    # delete cg snapshot
    $format=$event_obj->get_syslog_message();   

=back


=head2 B<is_snmp_supported()>

This method is used to determine if the event support snmp traps 

=over 4

=item Inputs

None 

=item Outputs

Returns a scalar value of 1 if snp is supported, otherwise 0. 

=item Examples

    # start cg snapshot operation
    $snmp_support=$event_obj->is_snmp_suupported();

=back

=head2 B<route_modify($destinations)>

This method modifies what destination routes the event will be sent too 

=over 4

=item Inputs

=item * B<destinations> default: "-"

A scalar containing a destination or comma separated list of destinations.

=item Outputs

Returns a scalar value of 1 if the event destination was modified, otherwize 0. If an error
occured the error flag is set, errmsg is set to the failure reason.

=item Examples

    # delete cg snapshot
    $destinations="dest1,dest2";
    $modified=$event_obj->route_modify($destinations);   

=back

=head2 B<status_reset()>

Reset event message statistics

=over 4

=item Inputs

None

=item Outputs

Returns a scalar value of 1 if the event status was reset, otherwize 0. If an error
occured the error flag is set, errmsg is set to the failure reason.

=item Examples

    # reset event message status
    $event_obj->status_reset();   

=back

=head2 B<status_show()>

Display current event status

=over 4

=item Inputs

None

=item Outputs

Returns an array or array ref containing the output.  If an error occurred the error flag is set, 
errmsg is set to the failure reason.

=item Examples

    # reset event message status
    @buffer=$event_obj->status_show();   

=back

=cut
