#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/Os/EventMgr.pm#1 $

## @summary EventMgr Interface
##
## @author knopp@netapp.com
## @status private
## @pod here


package Os::EventMgr;

use strict;

use XML::Simple qw(:strict);
use Os::Event;
use San::SanLog;
use Error;
use File::Basename;
use vars qw( @EXPORT @ISA );

@ISA = qw(Error SanLog);


################################################################################
###
###  P U B L I C   F U N C T I O N S
###
################################################################################


################################################################################
### Subroutine Name: config_modify()
### Objective:       Modify event configuration attributes
### Details:         See POD documentation at the end of this file
################################################################################
sub config_modify {
    my ($self,%args)=@_;
    my (
	$suppression,
	$mailfrom,
	$mailserver,
	%options,
	);


    # parse arguments
    foreach (keys %args) {
      if (/-?suppress/i) {
	if (not length($suppression = $args{$_})) {
		    $self->_set_err("Error: EventMgr::config_modify() Invalid value for argument '-suppression'");
		    return $self;
	}      
	$options{suppression}=$suppression;
	next;
      }
      elsif (/-?mailfrom/i) {
	if (not length($mailfrom = $args{$_})) {
		    $self->_set_err("Error: EventMgr::config_modify() Invalid value for argument '-mailfrom'");
		    return $self;
	}
	$options{mailfrom}=$mailfrom;
	next;
      }
      elsif (/-?mailserver/i) {
	if (not length($mailserver = $args{$_})) {
		    $self->_set_err("Error: EventMgr::config_modify() Invalid value for argument '-mailserver'");
		    return $self;
	}  
	$options{mailserver}=$mailserver;
	next;
      }
      else {
	$self->_set_err("Error: EventMgr::config_modify() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    }     
    
    eval {
      $self->{server}->event_config_modify(%options);	                           	                           
    };
    if ($@) {
      $self->_set_err("ERROR: Unable to turn event suppression off",$@);
      return 0;
    }

    return 1;
}

################################################################################
### Subroutine Name: config_show()
### Objective:       Display current event configuration attributes
### Details:         See POD documentation at the end of this file
################################################################################
sub config_show {
    my ($self)=@_;
    my (
	 @buffer,
	 @output,
	);

    $self->_clear_err();

    eval {
	@buffer=$self->{server}->event_config_show();	
    };
    if ($@){
	$self->_set_err("ERROR: EventMgr::config_show() Unable to show event configuration");
	return 0;
    }

    foreach my $row (@buffer){
      foreach my $key (sort keys %$row){
	push @output,sprintf("%35.35s: %-s",$key,$row->{$key});   
      }
    }

    return (wantarray()) ? @output : [@output];

}

################################################################################
### Subroutine Name: destination_create()
### Objective:       Create a new destination
### Details:         See POD documentation at the end of this file
################################################################################
sub destination_create{
    my ($self,$destination)=@_;

    $self->_clear_err();

    eval {
      $self->{server}->event_destination_new(name=>$destination);
    };
    if ($@) {
      $self->_set_err("ERROR: Unable to create event destination",$@);
      return 0;
    }

    return 1;
}

################################################################################
### Subroutine Name: destination_delete()
### Objective:       Delete an existing destination
### Details:         See POD documentation at the end of this file
################################################################################
sub destination_delete{
    my ($self,$destination)=@_;

    $self->_clear_err();

    eval {
      $self->{server}->event_destination_delete(name=>$destination);
    };
    if ($@) {
      $self->_set_err("ERROR: Unable to delete event destination",$@);
      return 0;
    }

    return 1;
}

################################################################################
### Subroutine Name: destination_modify
### Objective:       Modify what hosts support each destination route type
### Details:         See POD documentation at the end of this file
################################################################################
sub destination_modify {    
    my ($self,$destination,%args)=@_;
    my (
	$mail,
	$snmp,
	$syslog,
	%routes,
	@buffer,
	@output,
	);

    $self->_clear_err();

    # parse arguments
    foreach (keys %args) {
      if (/-?snmp/) {
	if (not length($snmp = $args{$_})) {
		    $self->_set_err("Error: EventMgr::add_destination() Invalid value for argument '-snmp'");
		    return $self;
	}
        $routes{snmp}=$snmp; 
	next;
      }
      elsif (/-?syslog/) {
	if (not length($syslog = $args{$_})) {
		    $self->_set_err("Error: EventMgr::add_destination() Invalid value for argument '-syslog'");
		    return $self;
	}     
	$routes{syslog}=$syslog;
	next;
      }
      elsif (/-?mail/) {
	if (not length($mail = $args{$_})) {
		    $self->_set_err("Error: EventMgr::add_destination() Invalid value for argument '-mail'");
		    return $self;
	}  
	$routes{mail}=$mail;
	next;
      }
     else {
	$self->_set_err("Error: EventMgr::add_destination() Invalid parameter specified \'$_\'");
	return $self;                      	  
      }
    }     
     
    foreach my $key (sort keys %routes){
	eval {	  
	  $self->{server}->event_destination_modify(name=>$destination,$key=>$routes{$key});
	};
	if ($@) {
	  $self->_set_err("ERROR: Unable to modify event destination",$@);
	  return 0;
	}
    }

    @buffer=$self->{server}->event_destination_show(name=>$destination);
    foreach my $row (@buffer){
      foreach my $key (sort keys %$row){
	push @output,sprintf("%35.35s: %-s",$key,$row->{$key});   
      }
    }
    return 1;

}

################################################################################
### Subroutine Name: destination_show()
### Objective:       Display current destination names
### Details:         See POD documentation at the end of this file
################################################################################
sub destination_show {
    my ($self,$destination)=@_;
    my (
	 @buffer,
	 @output,
	);

    $self->_clear_err();

    unless (defined $destination){
	$self->_set_err("ERROR: EventMgr::show_destination() Parameter $destination is required");
    }
 
    eval {
	@buffer=$self->{server}->event_destination_show(name=>$destination);	
    };
    if ($@){
	$self->_set_err("ERROR: EventMgr::show_destination() Unable to show destination named $destination");
    }

    foreach my $row (@buffer){
      foreach my $key (sort keys %$row){
        if (ref $row->{$key} eq "ARRAY"){
	    push @output,sprintf("%35.35s: %-s",$key,join(",",@{$row->{$key}}));
	}
	else {
	    push @output,sprintf("%35.35s: %-s",$key,$row->{$key});   
	}
      }
    }

    return (wantarray()) ? @output : [@output];

}

################################################################################
### Subroutine Name: get_events()
### Objective:       Retrieve list of event objects
### Details:         See POD documentation at the end of this file
################################################################################
sub get_events {
    my ($self,%args)=@_;
    my (
	$cnt,
	$event,
	$idx,
	$ignore_asup,
	$max,
	$name,
	$severity,
	$snmp,
	@event_objs,
	@names,
	@names_severity,
	);    

        $idx=0;
        $ignore_asup=0;
	$cnt=0;
	$max=1000000;

	# parse arguments
	foreach (keys %args) {
	  if (/-?name/i) {
	    if (not length($name = $args{$_})) {
			$self->_set_err("Error: EventMgr::get_events() Invalid value for argument '-name'");
			return $self;
	    }        
	    next;
	  }
	  elsif (/-?cnt/i) {
	    if (not length($max = $args{$_})) {
			$self->_set_err("Error: EventMgr::get_events() Invalid value for argument '-cnt'");
			return $self;
	    }        
	    next;
	  }
	  elsif (/-?severity/i) {
	    if (not length($severity = $args{$_})) {
			$self->_set_err("Error: EventMgr::get_events() Invalid value for argument '-severity'");
			return $self;
	    }
	    next;
	  }
	  elsif (/-?snmp/i) {
	    if (not length($snmp = $args{$_})) {
			$self->_set_err("Error: EventMgr::get_events() Invalid value for argument '-snmp'",
			                "valid values /1,0,true,false,yes/no/");
			return $self;
	    }  
	    $snmp=($snmp=~/1|^t|^y|/i) ? 1 : 0;
	    
	    next;
	  }
	  elsif (/-?ignore_asup/i) {
	    if (not length($ignore_asup = $args{$_})) {
			$self->_set_err("Error: EventMgr::get_events() Invalid value for argument '-ignore_asup'");
			return $self;
	    }
	    next;
	  }

	  else {
	    $self->_set_err("Error: EventMgr::new() Invalid parameter specified \'$_\'");
	    return $self;                      	  
	  }
	}   

    if (scalar(@{$self->{nodes}}>1)) {
	$self->sanlog(-msg=>"event generation will alternate between nodes: ".join(',',@{$self->{nodes}}));
    }
    else {
	$self->sanlog(-msg=>"event generation will occur on node ".join(',',@{$self->{nodes}}));
    }

    ### filter events by name
    @names=(defined $name)
	? grep /$name/, $self->_get_names()
        : $self->_get_names();

    ### filter events by severity level
    $severity="(.*)" unless (defined $severity);
    @names_severity=grep ($self->{name_severity_of}->{$_}=~/$severity/i,@names);
    
    foreach my $name (@names_severity){
	my $event_obj=$self->{name_event_of}{$name};
        next if (defined $snmp and $event_obj->is_snmp_supported() ne $snmp);
	next if ($event_obj->get_name()=~/^callhome|asup/ and ($ignore_asup==1));

	$event_obj->set_node_name(@{$self->{nodes}}[$idx]);
        $idx=($idx==scalar(@{$self->{nodes}}-1)) ? 0 : $idx+1;

	#$self->sanlog(-msg=>$event_obj->get_name().": ".$event_obj->get_node_name());

	push @event_objs,$event_obj if ($cnt++ < $max);
    }

    return (wantarray()) ? @event_objs : [@event_objs];

}

################################################################################
### Subroutine Name: get_last_seqid()
### Objective:       Retrieve last event sequence id
### Details:         See POD documentation at the end of this file
################################################################################
sub get_last_seqid {
    my ($self)=@_;
    my (
	$seq,
	@output,
	);
    
    $self->_clear_err();

    eval {
	@output=$self->{server}->shell(cmd=>"tail -2 /mroot/log/messages.ems | grep eventID");
    };
    if ($@ or not length ($output[0])){
	$self->_set_err("ERROR: EventMgr::get_last_seqid() Unable to retrieve last sequence id from event log",$@);
	return "";
    }

    ($seq)="@output"=~/eventID=\"(\d+)\"/;
 
    return $seq; 
}

################################################################################
### Subroutine Name: get_server()
### Objective:       Retrieve last event sequence id
### Details:         See POD documentation at the end of this file
################################################################################
sub get_server {
    my ($self)=@_;
    return $self->{server};
}

################################################################################
### Subroutine Name: new()
### Objective:       Create a new eventMgr object
### Details:         See POD documentation at the end of this file
################################################################################
sub new {
    my ($class, $server,%args) = @_;
    my ( 
	$build_root,
	$catalog,
	$catalog_of,
	$format_syslog,
	$format_snmp,
        $log,
	$self,
	@event_objs,
	%name_event_of,
	%name_severity_of,
	);

	$self={};

	bless $self,$class;	

	# parse arguments
	foreach (keys %args) {
	  if (/-?loghdl/) {
	    if (not length($log = $args{$_})) {
			$self->_set_err("Error: EventMgr::new() Invalid value for argument '-loghdl'");
			return $self;
	    }        
	    next;
	  }
	  else {
	    $self->_set_err("Error: EventMgr::new() Invalid parameter specified \'$_\'");
	    return $self;                      	  
	  }
	}     

        ### Check for required server variables
	unless (defined $server){
	    $self->_set_err("Error: EventMgr::new() Required parameter missing: -server");
	    return $self;
	}
        $self->{server}=$server;

        $self->{nodes}=[$self->{server}->servers()];

	### Retrieve build root of server
	### MRD
	$build_root=$self->_get_build_root($server);
		if ($build_root =~ /R10/) {
			# use this for Tricky
			$catalog=$build_root."/ontap/prod/build/eventCatalog-4.xml";
		}
		else {
			# use this for BR
        	$catalog=$build_root."/ontap/prod/build/eventCatalog-1.xml";
		}

	return if ($self->error());

        ### Read in ems catalog 
	$self->{log}=$log if (defined $log);
	$self->sanlog(-msg=>"catalog: ".$catalog);
	eval {
	     $self->sanlog(-msg=>"Loading catalog map");
	     my $xs = new XML::Simple();
	     $catalog_of = $xs->XMLin($catalog,ForceArray=>1,KeyAttr=>[]);
	};
	if ($@){
	    $self->_set_err("Error: EventMgr::new() Opening/Loading of catalog ",
		            "'".$catalog."' failed");
	    return $self;
	}
	$self->sanlog(-msg=>"Done");

        ### Create an event object lookup by event name
	foreach my $event (@{$catalog_of->{'event-def'}}){

            my $event_obj=Os::Event->_new($event, -parent=>$self, -build_root=>$build_root,-loghdl=>$self->{log});		
	    $name_event_of{$event->{name}}=$event_obj;
	    $name_severity_of{$event->{name}}=$name_event_of{$event->{name}}->{severity};
	}
	  
        foreach my $node (@{$self->{nodes}}){
	    eval{	     
	    $self->sanlog(-msg=>"Resetting event status on node ".$node);
	    $self->{server}->set_execution_server(-Server=>$node);
	    $self->{server}->event_status_resetall(node=>$node);
	    };
	    if ($@){
		$self->_set_err("ERROR: EventMgr::new()  Unable to reset event status for node $node");
		return $self;
	    }
	    
	}


        ### Order event object by name
	#foreach my $name (sort keys %name_event_of){
	    #push @event_objs,$name_event_of{$name};	    
	#}

	### initilize the object members
	$self->{name_event_of}=\%name_event_of;	
	$self->{name_severity_of}=\%name_severity_of;
	$self->{event_objs}=\@event_objs;
	return $self;

}

################################################################################
###
###  P R I V A T E   F U N C T I O N S
###
################################################################################

sub _get_build_root {
    my ($self,$server)=@_;
    my (
	$cmd,
	$badpath,
	$rname,
	$goodpath,
	@buffer,
	@output,
	);

    $cmd = "ident /boot/modules/maytag.ko | grep NtapBuildPath | awk '{print \$2}'";

    eval {
		@output=$server->shell(cmd=>$cmd);
    };
    if ($@){
		$self->_set_err("ERROR: Unable to retrieve server build path");
		return "";
    }

	($badpath) = "@output";	
	$badpath =~ s/\/$//;
	$rname = basename($badpath);
	$goodpath = `/usr/local/bin/buildinfo -l find $rname`;
	@buffer = split(/\s/, $goodpath);
	
	return $buffer[$#buffer];

}

################################################################################
### Subroutine Name: _get_names()
### Objective:       Retrieve list of event names
### Details:         See POD documentation at the end of this file
################################################################################
sub _get_names {
    my ($self)=@_;
    my (
	@names,
	);

    @names=(sort keys %{$self->{name_event_of}});
    
    return (wantarray()) ? @names : [@names];

}


1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

Event.pm

=head1 Author

Gary Knopp

=head1 Description

EventMgr.pm provides an API interface for working with cluster-wide configuration and logging.
The EventMgr object uses the event catalog to create event objects.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error>

B<San::SanLog>

=head1 Public Methods

=head2 B<config_modify(I<-suppression, -mailfrom, -mailserver>)>

This method modifies cluster-wide configuration attributes for event notification and logging

=over 4

=item Inputs

=item * I<-suppression> default:0

This parameter defines whether duplicate event messages that occur within a given timeframe
are suppressed.

=item * I<-mailfrom> default: admin@localhost

This parameter defines the name of the user that will appear for events sent using smtp.

=item * I<-mailserver> default: localhost

This parameter defines the name of the mail server

=item Outputs

Returns a scalar value of 1 if modification was successful, otherwise 0. If an error occured 
the error flag is set, errmsg is set to the failure reason.

=item Examples

     # Turn event suppression off
     $rtn=$eventMgr_obj->config_modify(-suppression=>0);

=back

=head2 B<config_show()>

This method shows the current cluster-wide event configuration

=over 4

=item Inputs

None

=item Outputs

Returns an array or array ref containing the output.  If an error occurred the error flag is set, 
errmsg is set to the failure reason.

=item Examples

     # Display the current event configuration
     $Test->nlog($eventMgr_obj->config_show());

=back

=head2 B<destination_create($destination)>

Create a new event destination

=over 4

=item Inputs

=item * B<$destination>

Name of cluster-wide event destination to create

=item Outputs

Returns a scalar value of 1 if successful, otherwisw 0.  If an error occured the error flag is set, errmsg is set to the failure
reason.

=item Examples

     # create a new event destination
     $eventMgr_obj->destination_create($destination);

=back

=head2 B<destination_delete($destination)>

Delete an existing event destination

=over 4

=item Inputs

=item * B<$destination>

Name of cluster-wide event destination to delete

=item Outputs

Returns a scalar value of 1 if successful, otherwisw 0.  If an error occured the error flag is set, errmsg is set to the failure
reason.

=item Examples

     # create a new event destination
     $eventMgr_obj->destination_delete($destination);

=back


=head2 B<destination_modify($destination,I<-mail,-snmp,-syslog>)>

This method modify the email addresses, SNMP clients, and remote syslog daemons to associate with the destination. 

=over 4

=item Inputs

=item * B<$destination>

Name of destination to modify

=item * I<-mail>

One of more comma delimited email addresses

=item * I<-snmp> 

One of more comma delimited SNMP clients

=item * I<-syslog>

One or more comma delimted remote syslog daemons

=item Outputs

Returns a scalar value of 1 on success and 0 on failure. If an error occured the error flag 
is set, errmsg is set to the failure reason.

=item Examples

    # Modify event destination
    $eventMgr_obj->destination_modify("Mydest1",-mail=>'root@naneqa-2k6');

=back

=head2 B<destination_show($destination)>

This method shows the current cluster-wide event destinations that can be assigned to an event

=over 4

=item Inputs

=item * B<$destination>

Name of destination to show

=item Outputs

Returns an array or array reference displaying cluster-wide event destinations.

=item Examples

     #retrieve cgid value
     @lines=$eventMgr_obj->destination_show();
    

=back

=head2 B<get_events(I<-name,-severity,-snmp,-ignore_asup)>

Retrieve node events as objects based on fiter criteria.

=over 4

=item Inputs

=item * I<-name> default: (.*)

Filter list of event objects by event name using regular expressions.

=item * I<-severity> default: (.*)

Filter list of event objects by event severity using regular expressions.
For example, -severity=>"emerg|crit|err";

=item * I<-snmp> default: 0

Filter list of event objects by whether event supports snmp traps.

=item * I<-asup_supported> default: 0

Filter list of event objects by whether event supports autosupport.


=item Outputs

Returns an array or array ref of event objects based on input filer criteria.  If an error 
occured the error flag is set, errmsg is set to the failure reason.

=item Examples

     #retrieve all monitor and disk events support snmp traps
     @event_objs=$eventMgr_obj->get_events(-name=>'monit|disk',-snmp=>1);
    

=back

=head2 B<get_last_seqid()>

Retrieve last sequence id used for logging an event

=over 4

=item Inputs

None 

=item Outputs

Returns a scalar containing the last sequence id used.  If an error 
occured the error flag is set, errmsg is set to the failure reason.
=item Examples

     #retrieve event log seq id
     $seqid=$eventMgr->get_last_seqid();
    

=back

=head2 B<new($server,I<-loghdl>)>

Create a new EventMgr object to manage events

=over 4

=item Inputs

=item * B<$server> 

Server object to use by test.  The first node in the server list will be used for 
the event generation.

=item * I<-loghdl>

File handle that library can use to log output too.

=item Outputs

Returns an EventMgr object handle.  If an error occured the error flag is set, 
errmsg is set to the failure reason.

=item Examples

     #retrieve cgid value
     $eventMgr_obj=Os::EventMgr->new($server);
    

=back

=cut
