#Hey emacs, this is basically -*- perl -*-
#$Id$
#Copyright (c) 2012 Network Appliance, Inc.
#All rights reserved.#
## @summary Interface implementation for IOTools functions
## @author dl-kernel-qa
## @status private
## @pod ./IOZONE.pm

package Os::IOTools::IOZONE;
use strict;
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use Data::Dumper qw(Dumper);
use Params::Validate
  qw(validate validate_with BOOLEAN HASHREF OBJECT ARRAYREF SCALAR UNDEF);
use warnings;
use TestUtils qw(get_site_info);
use Tharn qw($Log);
use base qw(Os::IOTools);
use POSIX qw(strftime);
use NATE::BaseException qw(:try);

sub _new {
    my $class = shift;
    my %args  = validate_with(
        params => \@_,
        spec   => {
            api_set_obj => { isa  => 'NACL::APISet' },
            node        => { type => SCALAR, optional => 1 },
            tool        => { type => SCALAR }
        },
        allow_extra => 1
    );
    my ( $self, );
    $self = bless {}, $class;

    $self->{api_set_obj} = $args{api_set_obj};

    $self->{tool}        = $args{tool};
    if ( $self->_install_tool() == $TCD::FAIL ) {
        $Log->comment("Failed to init the object: tool installation failed");
        return undef;
    }
    else {
        return $self;
    }
}

sub cleanup {
    my ($self) = @_;
    $self->_uninstall_tool();
    return $TCD::PASS;
}

sub set_value {
    my ($self, %args) = @_;
    
    my $field = $args{field};
    my $value = $args{value};
    $self->{$field}= $value; 
    return $TCD::PASS;
}

sub _install_tool {
    my ($self)          = @_;
    my $api_set_obj     = $self->{api_set_obj};
    my $tool_image_src  = "";
    my $tool_image_tgt  = "/mroot/" . $self->{tool};
    my $site            = $self->{api_set_obj}->{hostobj}->{SITE};
    my %site_info       = TestUtils::get_site_info();

    my @supported32 = qw(FAS3050 FAS2040);

    #   Note: there are a few iozone binaries
    #   "/x/eng/systest/p4/test/cluster/fs/iozone/freebsd/iozone" : uses older lib
    #   "/x/eng/rtpeng-testbeds/csiqa/golden/p4/test/os/osk_burts/bin/iozone7":
    #          does not work if specifying file path.
    #   use /x/eng/localtest/arch/amd64-bsd-freebsd8/bin/ 
    $tool_image_src =
          "/x/eng/localtest/arch/amd64-bsd-freebsd8/bin/iozone";
    
    
    my $check_exists = $api_set_obj->execute_raw_command( command => "ls -ltr $tool_image_tgt" );
    my @cmds = ();
    if ($check_exists =~ /No such file or directory/ig) {
        $Log->comment("$tool_image_tgt does not exist, will wget the file from server.");   
        unless ($site_info{$site}->{install_server}) {
        NATE::BaseException->throw("IOZONE.pm: Undefined install server ip address");	
	       unless ($site_info{$site}->{install_proto}) {
		NATE::BaseException->throw("IOZONE.pm: Undefined install protocol");		
        }
	}     
        unless ($site_info{$site}->{install_user} && $site_info{$site}->{install_pass}) {      
        @cmds = (
            #wget ftp://10.229.128.4/x/.../os/tools/iozone
            "wget "
              . $site_info{$site}->{install_proto} . '://'
              . $site_info{$site}->{install_server}. '/'
              . "$tool_image_src -O $tool_image_tgt"
        );		
        }
	else {
	@cmds = (
            "wget "
              . $site_info{$site}->{install_proto} . '://'
              . $site_info{$site}->{install_user} . ':'
              . $site_info{$site}->{install_pass} . '@'
              . $site_info{$site}->{install_server}. '/'
              . "$tool_image_src -O $tool_image_tgt"
        );
	}

        $self->{_install_by_self} = 1;
    } else {
        $Log->comment("$tool_image_tgt exists, no need to wget.");
    }
    
    push   @cmds, "chmod 755 $tool_image_tgt";

    foreach my $cmd (@cmds) {
        try {
            my $response = $api_set_obj->execute_raw_command( command => $cmd );
            $Log->comment("$response");
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            my $message          = $exception_object->text();
            $Log->comment($message);
            return $TCD::FAIL;
        };
    }

    $check_exists = $api_set_obj->execute_raw_command( command => "ls -ltr $tool_image_tgt" );
    if ($check_exists =~ /No such file or directory/ig) {
            $Log->comment("Failed to install iozone on the filer");
            $self->{_install_by_self} = undef;
            return $TCD::FAIL;        
    } else {
        $self->{executable} = $tool_image_tgt;
        return $TCD::PASS;
    }
}

sub _uninstall_tool {
    my ($self) = @_;

    if (!defined $self->{_install_by_self} ) {
        if (!defined $self->{_force_cleanup} || $self->{_force_cleanup} == 0) {
            $Log->comment("Not the installer of the inzone bin and Not asked to force cleanup");
            $Log->comment("So leave the binary on filer.");
            return $TCD::PASS;
        }
    }
    else {
        if (defined $self->{_force_nocleanup} && $self->{_force_nocleanup} == 1) {
            $Log->comment("Installer of the iozone bin, but told to leave the bin ");
            $Log->comment("So leave the binary on filer.");
            return $TCD::PASS;
        }
    }
    
    my $cmd = "rm " . $self->{executable};

    try {
        my $response =
          $self->{api_set_obj}->execute_raw_command( command => $cmd );
        $Log->comment("$response");
    }

    catch NATE::BaseException with {
        my $exception_object = shift;
        my $message          = $exception_object->text();
        $Log->comment($message);
        return $TCD::FAIL;
    };

    return $TCD::PASS;
}


sub iozone {
    my ( $self, %args ) = @_;
    my $cmd_options =
      ( defined $args{cmd_options} ) ? $args{cmd_options} : undef;
    my $mode = ( defined $args{test_mode} ) ? $args{test_mode} : "auto";
    my $file = ( defined $args{file} )      ? $args{file}      : undef;
    my $path = ( defined $args{path} )      ? $args{path}      : undef;
    my $maximum_filesize =
      ( defined $args{maximum_filesize} ) ? $args{maximum_filesize} : 2048;
    my $processes = ( defined $args{processes} ) ? $args{processes} : 1;
    my $test_type = ( defined $args{test_type} ) ? $args{test_type} : undef;
    my $timeout  = ( defined $args{timeout} ) ? $args{timeout} : 3600;
    my $original_timeout =  $self->{api_set_obj}->get_timeout();
    my $time_stamp = strftime ("%Y%m%d%H%M%S", localtime );

    if ( defined $cmd_options ) {
        $Log->comment(
            "Use user specified options [$cmd_options] to run iozone");
    }
    else {
        $Log->comment(
            "Use user specified mode, file, path, test_type etc for iozone");
        if ( $mode eq "auto" ) {
            $cmd_options .= " -a -g $maximum_filesize";
        }
        elsif ( $mode eq "Auto2" ) {
            #$cmd_options .= " -A -g $maximum_filesize";
            $cmd_options .= " -az -i0 -i1 -g $maximum_filesize";
        }
        elsif ( $mode eq "throughput" ) {
            $cmd_options .= " -t $processes";
        }
        elsif ( $mode eq "compatibility" ) {
            $cmd_options .= " -Z";
        }
        
        if ( defined $file ) {
            $cmd_options .= " -f $file.$time_stamp";
        }

        if ( defined $path ) {
            ##Add timestamp to each of the specified file path
            foreach (@{$path}) {
                $_ = $_.".$time_stamp";
            }
            my $path_orig = $path;
            while (scalar @{$path} < $processes) {
                $path = [@{$path}, @{$path_orig}];
                
            }
            $cmd_options .= " -F " . join( " -F ", @{$path} );
        }

        if ( defined $test_type ) {
            $cmd_options .= " -i " . join( " -i ", @{$test_type} );
        }
    }

    $self->{api_set_obj}->set_timeout("connectrec-timeout" => $timeout);
    my $cmd = "sudo " . $self->{executable} . " " . $cmd_options;
    my $response = "NA";
    try {
        $response =
          $self->{api_set_obj}->execute_raw_command( command => $cmd );
        $Log->comment("$response");
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        my $message          = $exception_object->text();
        $Log->comment($message);
        return $TCD::FAIL;
    };

    $self->{api_set_obj}->set_timeout("connectrec-timeout" => $original_timeout);
    if ( $response !~ /iozone test complete/imsg ) {
        $Log->comment("IOZONE failed");
        $Log->comment("[$response]");
        return $TCD::FAIL;
    }
    else {
        $Log->comment("[$response]");        
        $Log->comment("IOZONE ran successfully!");   
        $Log->comment("Reminder: checking the log file is a best practice");        
        return $TCD::PASS;
    }
}
1;

############################################################################
#  P O D
############################################################################

=pod

=head1 Package Name

Os::IOTools::IOZONE

=head1 Author

dl-kernel-qa@netapp.com

=head1 Description

Os::IOTools::IOZONE provides a API interface to run IO tool iozone.
output.


=head1 Public Methods

=head2 B<_new( B<api_set_obj>, B<tool> )>

Creates a new Os::IOTools::IOZONE object.

=over 4

=item Inputs

=over 4

=item B<api_set_obj>

Required : the api_set_obj used for testing

=item B<tool>

Required : tool must be 'iozone'

=back

=item Outputs

Returns a Os::IOTools::IOZONE object.

=item Examples

Os::IOTools::IOZONE->_new (tool        => "iozone",
        api_set_obj => $Api_set_obj
    );

=back


=head2 B<cleanup( )>

calls _uninstall_tool, returns with $TCD::PASS

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns $TCD::PASS

=item Examples

 $IOTool->cleanup();

=back

=head2 B<set_value( )>

set field of the object to a specified value

=over 4

=item Inputs

=over 4

=item B<field>

The field that will receive a new value

=item B<value>

Set this value to the specified field

=back

=item Outputs

Returns $TCD::PASS

=item Examples

 $IOTool->set_value(field=>"tool", value=>"iozone");

=back

=head2 B<_install_tool( )>

 Select iozone binary based on ONTAP version from : iozone72, iozone81 etc
 Use wget, copy iozone binary from netboot server to filer /mroot/ dir,
 then rename the binary to "iozone"

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns $TCD::PASS | $TCD::FAIL

=item Examples

This is an internal function, not recommend to use outside the module.
$self->_install_tool()

=back

=head2 B<_uninstall_tool( )>

 Based on rules, delete or keep iozone binary on filer
 Here is the rule whether to delete:
 When Not to delete:
 If _install_by_self is not defined within the object, and _force_cleanup is 0 or undefined;
 Or
 If _install_by_self is defined, and _force_nocleanup is 1.
 When to delete:
 All other situations.

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns $TCD::PASS | $TCD::FAIL

=item Examples

 This is an internal function, not recommend to use outside the module.
 $self->_uninstall_tool()

=back

=head2 B<iozone( )>

execute iozone command

=over 4

=item Inputs

=over 4

=item B<test_mode>

default to be \'auto\', other acceptable values are Auto2|throughput

=item B<file>

This is the value passed to -f option for iozone cmd

=item B<path>

This is the value passed to -F option for iozone cmd

=item B<maximum_filesize>

This is the value passed to -g option for iozone cmd, file size in mb, default to 2048

=item B<processes>

This is the value passed to -t option for iozone cmd, default to 1.

=item B<test_type>

This is the value passed to -i option for iozone cmd

=item B<time_out>

This is the value for iozone command to timeout in seconds, default is 3600 

=back

=item Outputs

Returns $TCD::PASS|$TCD::FAIL

=item Examples

  my $ret = $IO_obj->iozone( file => "/clus/$Vserver/$Test_File" );

=back

=cut
