# Hey emacs, this is basically -*- perl -*-
# Copyright (c) 2009 Network Appliance, Inc.
# All Rights reserved
#!/usr/local/bin/perl
##
## @summary 

=head1 NAME

##   OsUtils.pm - Common functions used by Os/FreeBSD test scripts  

=cut

## @description

=head1 DESCRIPTION

##   This library provides a number of functions used
##   in FreeBSD shell testing. See the perldoc for
##   a list and description  of the fucntions.

=cut

## @status 
#   		Change History
##   Last Change: xx-xx-xxxx		Initial Checkin
##
## @author jackm@netapp.com
##

package Os::OsUtils;

use strict;
use vars qw(@ISA @EXPORT);
use Tharn;
use TracePkg;
use Data::Dumper;
use Utils qw(make_host);
use boot qw(transit_to);
use NACL::APISet::Version;

require Exporter;

@ISA = qw(Exporter);

=head1 FUNCTIONS

		shell_initialize_testcase
		shell_basic_testcase
		shell_systemshell_login
		shell_system_login
		myConnectPlus 
		myDisconnectPlus
		prepConsole
		login
		setupNGSH
		startShell 
		exitShell 
		startConsole
		lock_diag_password 
		unlock_diag_password 
		reset_user_password 
		lock_diag_account 
		unlock_diag_account 
		check_diag_unlocked 
		change_diag_password 
		updateSecurityRoles
		Options_Hash 
		trace_message 
		shell_navigate_test
		shell_command_test
		enable_toor 

=cut
	
 @EXPORT = qw(
		shell_initialize_testcase
		shell_basic_testcase
		shell_systemshell_login
		shell_system_login
		myConnectPlus 
		myDisconnectPlus
		prepConsole
		login
		setupNGSH
		startShell 
		exitShell 
		startConsole
		lock_diag_password 
		unlock_diag_password 
		reset_user_password 
		lock_diag_account 
		unlock_diag_account 
		check_diag_unlocked 
		change_diag_password 
		updateSecurityRoles
		Options_Hash 
		trace_message 
		shell_navigate_test
		shell_command_test
		enable_toor 
                );                

my $Test = undef;
my $Options = undef;

my $t = TracePkg->new();

my $adminUser = "admin";
my $adminPass = "netapp1!";
my $rootUser  = "root";
my $rootPass  = "netapp1!";
my $testUser  = "testuser";
my $testPass  = "testP\@ssw0rd";
my $rlmUser   = "naroot";
my $rlmUser_7 = "naroot";
my $rlmUser_C = "admin";
my $rlmPass   = "netapp1!";
my $diagUser  = "diag";
my $diagPass  = "netapp1!";
my $diagPrompt = "%";
my $altPass  = "netapp2!";
my $timeout  = "60";

################################

=head2 shell_initialize_testcase 

Common code to initialize all testscases. Gets testcase
name and displays testcase description from Options hash.

shell_initialize_testcase ($Test_Object, $Options_Hash)
returns ($Testcase_Name) 

=cut

sub shell_initialize_testcase {

    ($Test, $Options) = @_;

    trace_message ($t, "CALL", "********** shell_initialize_testcase **********");
    
    my $Testcase = $Test->get_testcase();
    my $TC_options = Options_Hash($Testcase, $Options);
    $Test->description($Options->get(option=>"TC_description"));
    return $Testcase;

}

################################

=head2 shell_basic_testcase 

     The primary purpose of this routine is to log into the system, 
     login to the systemshell, execute a specified command, verify 
     the result (both of these are defined in the testcase hash), 
     then return an indicator if the expected result was found. 
     
     However, not all testcases are that simple in design, but many 
     of the steps involved in doing this (like logging into the system 
     and then into the systemshell) are used in every testcases. So 
     the function actually does different things, and returns different
     values, based on the value of the Logout variable.
     
     Logout = 0 (or Logout = undef) aka shell_basic_testcase

     This is the actual basis for the basic systemshell tests:
     1 - log in to the system via the connection method specified
     2 - start the systemshell and login as diag
     3 - execute the verification command and check the result
     4 - log out of the system shell and system

     Using it this way, with Logout = undef or Logout = 0, makes it 
     basically act as "the test", with all the test variables being 
     passed in via $Options. 

     For testcases that are negative, where you expect the command
     to fail and the variable testcase variable $TC_status is 0 in the
     testcase hash, the value of the verification is reversed, so that
     if it failed but you expected it to a Pass value is returned.

     RETURNS: $TCD::PASS or $TCD::FAIL based on result of verification test
    
     Logout = 1 aka shell_systemshell_login
     
     Provides a generic method to start the systemshell:
     1 - log in to the system via the connection method specified
     2 - start the systemshell and login as diag

     Using it this way, with Logout = 1, skips executing the
     command and logging out. It is the up to the calling routine
     to remember to call myDisconnectPlus when done. 

     RETURNS: $conObj if connection succeeded or undef if not 
    
     Logout = 2 aka shell_system_login

     Provides a generic method to login to the system:
     1 - log in to the system via the connection method specified

     Using it this way, with Logout = 2, skips logging in
     to the system shell. It is the up to the calling routine
     to remember to call myDisconnectPlus when done. 

     RETURNS: $conObj if connection succeeded or undef if not 

 shell_basic_testcase ($Testcase_Name, $Logout)

 Note: This function is the only one in the library that returns the
 TCD values for PASS or FAIL. All the rest use the perl values for
 1/true = pass or 0/false = fail.
 
=cut

sub shell_basic_testcase {

 
    my $Testcase = shift;
    my $Logout = shift;
    my $fail_cnt = 0;
    my $conObj = undef;
    my $doLogin = 1;
    my $user; my $pass;
    my $split=",";

    trace_message ($t, "CALL", "********** shell_basic_testcase **********");

    # my $Node = $Test->get_param("FILER");
    # This was changed because the values for NODES vs FILER
    # are different depending on where the test is executed.
    
    my $Node = ($Test->get_param("NODES"))[0];
    unless ($Node) {
        $Node = ($Test->get_param("FILER"))[0];
    }
    unless ($Node) {
        $Test->ndisplay("ABORT: Could not determine FILER");
        return $TCD::CONFIG;
    } 

    my $hconn = make_host($Node);
    my $hosttype = $hconn->hosttype();
    my $hostname = $hconn->hostname();

    # In 7mode root is both the root and admin user, 
    # but in 10mode they are 2 different users.
    # So if running in 7mode make them the same.   
    if ($hosttype !~ /-ng-/) {
     	    $adminUser = $rootUser;
    	    $adminPass = $rootPass;
    }
    
    my $TC_connection = $Options->get(option=>"CONNECTION");
    my $TC_user = $Options->get(option=>"USERNAME");
    my $TC_pass = $Options->get(option=>"PASSWORD");
    my $TC_node = $Options->get(option=>"SNODE");
    my $TC_state = $Options->get(option=>"SSHELL");
    my $TC_verification = $Options->get(option=>"VERIFICATION");
    my $TC_result = $Options->get(option=>"RESULT");
    my $TC_status = $Options->get(option=>"STATUS");

    if ($TC_state =~ /systemshell/) {
     	$user = $adminUser;
    	$pass = $adminPass;
    } else {
    	$user = $TC_user;
    	$pass = $TC_pass;
    }
      
    $Test->ndisplay("TC_verification command: ".$TC_verification);
    my @TC_result_array = split($split,$TC_result);
    for (my $x=0; $x < scalar(@TC_result_array); $x++) {  
        $Test->ndisplay("TC_result string $x: ".$TC_result_array[$x]);
    }

    $Test->ndisplay("$Testcase - connectiontype: $TC_connection shell: $TC_state user: $TC_user pass: $TC_pass");
    eval {
        $conObj = myConnectPlus($hostname, $user, $pass, $doLogin, $TC_connection);
    };
    # If you got an error or you were just logging into the system, return
    # the connection object (if there was an error it should be undef). In 
    # either case it is up to the caller to check for the return = undef.

    # NOTE: there were two version of this code segment in 7mode. 
    # I think that this is the correct one as it handles Logout = 0 
    # properly and returns $TCD::FAIL on error, not undef.

    if (($@) || ($conObj == undef)) { 
	    $Test->ndisplay("ERROR: Connection failed $@");
    	    if ($@ =~ /Connection refused/) {
    	    	    $Test->ndisplay("*** Console Connection Refused ***");
    	    	    $Test->ndisplay("Verify the console terminal server supports multiple connections as this is a test environment requirement.");
	            return $TCD::NA;
	    }
            if ($Logout == 0) {
	        return $TCD::FAIL;
	    } else {
                return undef;
            } 
    } 
    if ($Logout == 2) { 
            return $conObj;
    }           

    # Alternate Version (can run still it, should just have no effect)
    if (($@) || ($Logout == 2)) { 
        if ($@) { $Test->ndisplay("ERROR: $@"); return undef; }
        else    { return $conObj; }
    }           

    # if the test state is systemshell then log into it on the specified node 
    if ($TC_state =~ /systemshell/) {

        $fail_cnt = startShell ($conObj, $TC_node, $TC_user, $TC_pass) ? 0 : 1;
        # the return value is the reverse of what we need so flip it into fail_cnt ....
    }

    # If $Logout is undef or 0 then do the verification, log out and return the correct result
    # based on fail_cnt and TC_status (TC_status = 0 means the logic of the test should fail)
    # Note: $RESPONSE is the field that say uses to hold what it gets back if no output fields 
    # are defined.
    
    unless ($Logout) {  
 
        if ($fail_cnt == 0) {
            eval { say($conObj, $TC_verification, -noprompt, "-wait", @TC_result_array, -timeout, 120); };
            if ($@) { 
                $Test->ndisplay("ERROR: Verification Failed: $@"); 
                $fail_cnt++;
            } else {
                for (my $x=0; $x < scalar(@TC_result_array); $x++) {  
                if ($RESPONSE =~ $TC_result_array[$x])
                    # If you find any of the waitstrings in RESPONSE set the
                    # value of $TC_result to the value you found. This allows
                    # processing of arrays of waitstrings to work properly.
                    { $TC_result = $TC_result_array[$x]; }
            }}

            if ($RESPONSE !~ /$TC_result/i) {
                $Test->ndisplay("Incorrect verification response detected: $RESPONSE");
                $fail_cnt++;
            } else {
                $Test->ndisplay("Found $TC_result response: $RESPONSE");
                $fail_cnt = 0;
            }
            $conObj = myDisconnectPlus ($conObj, $TC_connection);       
        }
        
	# if fail_cnt == 0 above then the only place it could have failed was in
	# startShell, and it that routine always runs myDisconnectPlus if there
	# was a failure so don't bother doing it again here

        # Determine if any failures occurred and if the testcase was a 
        # negative test (TC_status == 0), then decide what value to return. 
        
        if ($fail_cnt == 0) {
            return ($TC_status == 1) ? $TCD::PASS : $TCD::FAIL; 
        } else {
            return ($TC_status == 0) ? $TCD::PASS : $TCD::FAIL; 
        }

    # Otherwise just return the connection object so the calling routine can use it.

    } else {
        if ($fail_cnt == 0) {
            return ($conObj); 
        } else {
            return undef; 
        }
    }
}

################################

=head2 shell_systemshell_login 

Executes the shell_basic_testcase with Logout = 1, which
causes the test to log into the systemshell and then
return the connection object. The basic test/verify
step is skipped.

shell_systemshell_login ($Testcase_Name)
returns ($Connection_Object) 

=cut

sub shell_systemshell_login {
    my $Testcase = @_;

    trace_message ($t, "CALL", "********** shell_systemshell_login **********");

    my $conObj = shell_basic_testcase ($Testcase, 1);
    return $conObj;
}

################################

=head2 shell_system_login 

Executes the shell_basic_testcase with Logout = 2, which
causes the test to log into the systems and then
return the connection object. Login to the systemshell
and the basic test/verify step are both skipped.

shell_system_login ($Testcase_Name)
returns ($Connection_Object) 

=cut

sub shell_system_login {
    my $Testcase = @_;

    trace_message ($t, "CALL", "********** shell_system_login **********");

    my $conObj = shell_basic_testcase ($Testcase, 2);
    return $conObj;
}

################################

=head2 myConnectPlus 

This routine creates the connection object by logging into the system. 
It was "cloned" from Mike Edmond's myConnect routine and uses Tharn::connect 
to connect via any connection type: 
(0 == ssh, 1 == console, 2 == telnet, 3 == rlogin, 4 == rlm).
It normally logs in using the supplied username and password ($doLogin = 1), 
but can also be used to just create an object and not log in, allowing the 
caller to use the connection as required ($doLogin = 0).

It also understands the some of the idosyncrancies of the system and
can deal with them. For example, when connecting via the RLM it logs into
the RLM as naroot, then starts the system console and logs into to that as
the specified username. Also, it knows how to set up the cli by enabling 
privilege mode the correct way based on with mode the filer is running in. 

 myConnectPlus ($Node_Nane, $Username, $Password, $doLogin, $connection_Type)
 returns ($Connection_Object) or undef if the connection failed

=cut

sub myConnectPlus
{

    my ($NODE, $user, $pass, $doLogin, $conType)=@_;
    my $hconn = make_host($NODE);
    my $hosttype = $hconn->hosttype();
    my $hostname = $hconn->hostname();
    my $RLM_IP = $hconn->RLM_IP();

    trace_message ($t, "CALL", "********** myConnectPlus **********");
    sleep (1); # Before you start trying to connect wait a second so the
               # system can reach a stable state after disconnect first. 

    my $login = "-login";
    my $cType;

    if ($conType =~ "ssh")
    {
        $cType = "ssh-persist";
    }
    elsif ($conType =~ "console")
    {
        $cType = "pm_cons";
    }
    elsif ($conType =~ "telnet")
    {
        $cType = "telnet";
    }
    elsif ($conType =~ "rlogin")
    {
        $cType = "rlogin";
    }
    elsif ($conType =~ "rsh")
    {
        $cType = "rsh-persay";
    }
    elsif ($conType =~ "rlm")
    {
        $cType = "ssh-persist";
    }
    elsif ($conType == 0)
    {
        $cType = "ssh-persist";
    }
    elsif ($conType == 1)
    {
        $cType = "pm_cons";
    }
    elsif ($conType == 2)
    {
        $cType = "telnet";
    }
    elsif ($conType == 3)
    {
        $cType = "rlogin";
    }
    elsif ($conType == 4)
    {
        $cType = "ssh-persist";
    }
    else
    {
        trace_message ($t, "ERROR", "Invalid connection type defined: $conType");
        return undef;
    }

    trace_message ($t, "INFO", "Create a $cType ($conType) connection to $hostname for $user with Login = $doLogin");

    my $conn = undef;
    my $cObj = undef;
    # make a random connection ID so that a new connection is made everytime
    my $conID = rand();

    my $Release = release($hostname, $hosttype);
    if (( $Release !~ /boilermaker/i ) && ( $hosttype =~ /-ng-/i )) { 
    	$rlmUser = $rlmUser_C;		# RR/SN/+ in Cmode (admin)
    } else {
    	$rlmUser = $rlmUser_7; 		# BR or any 7mode  (naroot)
    }		
    	
    trace_message ($t, "INFO", "Release: $Release Hosttype: $hosttype RlmUser: $rlmUser");

    # We need to set up the correct parameters for Tharn::connect to use to log in to the Node.
    # Normally these are based on the values defined in the testcase. The RLM is an exception
    # though, as it requires two logins (rlm and system console) to get to the ngsh. In that case
    # the username password to connect to the RLM are always the same, and the username password
    # in the testcase are used to log into the system console instead. Also, changed from using
    # connhost=$hostname$RLM to connhost=$RLM_IP for labs that don't have the RLM in DNS.
        
    if (($conType == 4) || ($conType =~ "rlm")) {
        my $RLM = "-rlm";
        $hconn->{"connection_fields"} = {$conID => {conntype => $cType, username => $rlmUser, password => $rlmPass}};
    	$conn = "#$conID;conntype=$cType;connhost=$RLM_IP;username=$rlmUser;password=$rlmPass";
    } else {
        $hconn->{"connection_fields"} = {$conID => {conntype => $cType, username => $user, password => $pass}};
        $conn = "#$conID;conntype=$cType;username=$user;password=$pass";
    }

    # Now log in normally, using the values set in $hconn and $conn. And
    # if $doLogin = 0 (do not login) just connect and return the conObj.
    
    if ($doLogin == 0)
    {
        trace_message ($t, "INFO", "Execute Tharn::connect for $cType but do not log in.");
        eval { $cObj = Tharn::connect($conn, "-nologin", "-timeout", $timeout); };
        if ($@) { 
            trace_message ($t, "INFO", "Tharn::connect failed. Return conObj = undef. $@");
            return undef;
	}
        trace_message ($t, "INFO", "Tharn::connect successful.");
        return $cObj;
    }
     
    #
    # changes starts here - TPho - 09/09/2011
    # If we are trying to login to the console, don't use the login built
    # into connect().
    # In order to log into the filer we first need to log into the terminal
    # server. 
    # So first connect to the console terminal server and then get the Console
    # into an expected State. Once that is done then continue to log into
    # the filer.
    #

    if (($conType == 1)  || ($conType =~ "console"))
    {
        ### -login
        trace_message ($t, "INFO", "Execute Tharn::connect for $cType with"
                      . " -login.");
        eval { 
             $cObj = Tharn::connect($conn, "-login",  "-timeout", $timeout);
        };
        if ($@) { 
    	    trace_message ($t, "INFO", "Tharn::connect console failed '$@'."
                          ." Return console object = $cObj");
            return undef;
        }
        elsif (!($cObj)) {
    	    trace_message ($t, "INFO", "Tharn::connect console returned "
                          ." null object '$cObj'. Will retry with 'no-login'");
        }
        else {
    	    trace_message ($t, "INFO", "myConnectPlus successfully completed."
                          ." Return conObj = $cObj");
            return $cObj;
        }
 
        ### -nologin
        trace_message ($t, "INFO", "Execute Tharn::connect for $cType with"
                      . " -nologin.");
	eval { $cObj = Tharn::connect($conn, '-nologin', "-timeout", $timeout); };
        if ($@) { 
            trace_message ($t, "INFO", "Tharn::connect failed. "
                            ."Return conObj = $cObj. Error: '$@'");
            return undef;
	}
        trace_message ($t, "INFO", "Tharn::connect successful."
                      ." connected to console.");
        trace_message ($t, "INFO", "Execute prepConsole.");
        if (prepConsole($cObj) < 0)
        {
            trace_message ($t, "INFO", "PrepConsole failed."
                           ." Return conObj = undef.");
            return undef;
        }

        trace_message ($t, "INFO", "PrepConsole successful.");
    }   ### if (($conType == 1)  || ($conType =~ "console"))

    trace_message($t, "INFO", "Execute Tharn::connect for $cType with -login.");
    eval { $cObj = Tharn::connect($conn, "-login",  "-timeout", $timeout); };
    if ($@) { 
        trace_message ($t, "INFO", "Tharn::connect failed. Return conObj = undef. $@");
        return undef;
    }
    trace_message ($t, "INFO", "Tharn::connect successful. cObj after Login = $cObj");
    
    if ($doLogin == 1) {

    # For some reason once in a while the login strings show up after the return from
    # this routine. Not sure why but it screws up the other commands. So wait a few
    # seconds after the connect returns and attempt to clear the buffers before continuing.   
 
    sleep (5);
    eval { say( $cObj, -blind, -swallow); };

    # At this point all connection types have created a conObj and logged in. 
    # However, the RLM requires a secondary login to get to the ngsh.
    
    if (($conType == 4) || ($conType =~ "rlm"))
    {
        trace_message ($t, "INFO", "Execute RLM startConsole");
        if (startConsole ($cObj, $user, $pass) == 0)
        {
            trace_message ($t, "INFO", "startConsole failed. Return conObj = undef");
            return undef;
	} else {
            trace_message ($t, "INFO", "startConsole successful.");
        }
    }}

    # And unless this is the 10mode root user, do some NGSH setup
    my $hosttype = ($cObj->hostp())->hosttype();
    
    if (($user eq $rootUser) && ($hosttype =~ /-ng-/))
    {
        trace_message ($t, "INFO", "Skipping CLI setup");
    } else {
        trace_message ($t, "INFO", "Execute CLI setup");
        setupNGSH($cObj);
    } 
    
    trace_message ($t, "INFO", "myConnectPlus successfully completed! Return conObj = $cObj");
    return $cObj;
}

################################

=head2 myDisconnectPlus 

This the "enhanced" disconnect routine, as it does more than just disconnect,
It understands how to do a graceful exit, and assumes the worst case, that the
user is in systemshell via the RLM, so it exeutes a number of sequnces in an
attempt to get back to the login prompt before it actually closes the connection.

It attempts to catch all error conditions and just ignores them, as it does not
know what state it was called from, so some command sequences may have no meaning.

 myDisconnectPlus ($connection_object, $connection_type)
 returns (undef) 

=cut

sub myDisconnectPlus
{

    my ($cObj, $conType)=@_;
    
    trace_message ($t, "CALL", "********** myDisconnectPlus **********");

    # Just in case this is accidently called with a null connection object
    unless ($cObj) { return undef; }
    
    my $hosttype = ($cObj->hostp())->hosttype();
 
    # Since you really do not know where the disconnect was called from,
    # assume the worst (deepest) case of being in the systemshell, which
    # requires two exits, and maybe a ctrl-D, to get out off.
    
    trace_message ($t, "INFO", "Disconnect connection $conType hosttype $hosttype");

    # This block is supposed to get you out of wherever you are and to a known state,
    # which should be logged out. You need to do more than disconnect, as when you
    # connect again you could wind up in the same state you were in before.
    
	    # This section is somewhat confusing, as what you do is based on type:
	    # TELNET - The exit method for telnet is dependent on the options value
	    # of telnet.distinct.enable. If "on" then "exit" will close the connect. 
	    # If "off" you must use "ctrl-D". So to make sure we send both. Does 
	    # not hurt and you don't need to keep checking the option in 7mode.
	    # SSH - Same as for telnet (don't ask, has to do with borrowing code). 
	    # RLM - Actually needs both, with the Ctrl-D first.
	    # RSH - Node of this is necessary but it simplifies the code. :)
	    # To make it more confusing, in 10mode the Ctrl-D is not needed for
	    # console, ssh or telnet, but it doesn't hurt and allows common usage.
	    # As for why we send "exit" twice, it makes sure we are not stuck in
	    # the systemshell when the next connection occurs.

        if (($conType =~ "console") || ($conType == 1))
        {
            trace_message ($t, "INFO", "Disconnect console");
    	    eval { say( $cObj, "exit", -noprompt, -swallow); };
    	    eval { say( $cObj, -blind, -swallow); };
    	    eval { say( $cObj, "exit", -noprompt, -swallow); };
    	    eval { say( $cObj, -blind, -swallow); };
    	    eval { say( $cObj, "\cD", -noprompt, -swallow); }; 
    	    eval { say( $cObj, -blind, -swallow); };
    	    eval { say( $cObj, "\c]", -noprompt, -swallow); }; 
    	    eval { say( $cObj, -blind, -swallow); };
    	    eval { say( $cObj, "quit", -noprompt, -swallow); };
    	    eval { say( $cObj, -blind, -swallow); };

        } else {   
            trace_message ($t, "INFO", "Disconnect ssh, telnet, rsh or rlm");
    	    eval { say( $cObj, "exit", -noprompt, -swallow); };
    	    eval { say( $cObj, -blind, -swallow); };
    	    eval { say( $cObj, "exit", -noprompt, -swallow); };
    	    eval { say( $cObj, -blind, -swallow); };
            eval { say( $cObj, "\cC", -noprompt, -swallow); }; 
            eval { say( $cObj, -blind, -swallow); };
    	    eval { say( $cObj, "\cD", -noprompt, -swallow); }; 
    	    eval { say( $cObj, -blind, -swallow); };
    	    eval { say( $cObj, "exit", -noprompt, -swallow); };
    	    eval { say( $cObj, -blind, -swallow); };
 	}

    eval { say( $cObj, -blind, -swallow); };
    eval { disconnect($cObj, "-logout"); };   
    $cObj = undef;

    trace_message ($t, "INFO", "Disconnect complete");
    return $cObj;
}

################################

=head2 prepConsole 

This routine only executes if the connection type being used is console.
It runs before the login attempt to try to get to a login prompt if
not already there. In most cases it just pokes the interface and returns,
but in some it will actually do a transit_to UP to try to get someplace
it understands.

 prepConsole ($connection_Object)
 returns (1=pass or 0=fail) 

=cut

sub prepConsole
{
    my ($conObj) = @_;
    my $exit;
    my $match;
    
    trace_message ($t, "CALL", "********** prepConsole **********");

    trace_message ($t, "INFO", "Execute setup for Console connection");

    my $hosttype = ($conObj->hostp())->hosttype();

    if ($hosttype =~ /-ng-/) {
        $exit = "exit";
        # For 10mode exit will logout of ngsh and get you to the login prompt
    } else {
        $exit = "\cD";
        # For 7mode ctrl-D will logout of cli and get you to the password prompt
    }
    
    for (my $to = 0; $to < 4 ; $to++)
    {

# At this point if we are in 7mode we are either at the cli prompt or the Password prompt

        eval{ $match = say( $conObj, "\n", -noprompt, "-timeout", 10, "-wait", qr/Password:/i, qr/login:/i, "%", "-ignore", "Login incorrect:");};
        if ($@)
        {
            trace_message ($t, "INFO", "Attempt $to failed");
            eval { say($conObj, $exit, -noprompt, "-timeout", 10, "-wait", qr/Password:/i, qr/login:/i); };
            if (!$@)
            {
                $to = 4;
            }
        }
        else
        {
            if ($match =~ /%/) {
                # If what was found was %, assume you in the systemshell and do an exit
                trace_message ($t, "INFO", "Found systemshell state. Exit");
                eval{ say($conObj, "exit", -noprompt ); };
            } elsif ($match =~ /^\*>/) {
                # if you find ^*> assume you are in maintenance mode and try to get back to
                # the login prompt (use boot::transit_to as it think it will work for 7 & 10)
                trace_message ($t, "INFO", "Found maintenance mode state. Go to UP");
               my $state = transit_to ($conObj, "UP", timeout=>300);           
            } elsif ($match =~ /\*#$/) {
                # if you find \*#$ assume you are in Borne shell mode and try to get back to
                # the login prompt (use boot::transit_to as it think it will work for 7 & 10)
                trace_message ($t, "INFO", "Found Bourne shell state. Go to UP");
                my $state = transit_to ($conObj, "UP", timeout=>300);           
	    } else {
                # Otherwise assume you are at the login or password prompt and return
                trace_message ($t, "INFO", "Assuming state is Login or Password. Stay here");
                $to = 4;
            }
        }
    }
    if($@)
    {
        return 0;
    }
    return 1;
}

################################

=head2 login 

This routine tries to log in using the specified username and password.
It assumes an already existing connection and that it will get a
standard login/password prompt. 

 login ($connection_object, $username, $password, $connection_type)
 returns (1=pass or 0=fail) 

=cut

sub login
{
    my ($conObj, $user, $pass, $app) = @_;

    trace_message ($t, "CALL", "********** login **********");

    trace_message ($t, "INFO", "Execute generic login for $user via $app");
    my $hosttype = ($conObj->hostp())->hosttype();
    
    if ($app == 1)
    {
        if(prepConsole($conObj) < 0)
        {
            return 0;
        }
    }

    eval {    
        say($conObj, $user, -noprompt, "-wait", qr/password:/i);
        say($conObj, $pass, "-prompt", $conObj->prompts()); 
    };

    if (($@) && ($app == 0) && ($hosttype !~ /-ng-/))
    {
        # if you get an error trying to login to a 7mode ssh
        # connection, try poking the interface for a prompt.
        # If that fails try sending the Password.
        eval {
            say($conObj, "\n", "-prompt", $conObj->prompts()); 
	};
        if($@)
        {
            say($conObj, $pass, "-prompt", $conObj->prompts()); 
        }
    }
    if($@)
    {
        trace_message ($t, "ERROR", "while trying to login $user with $pass: $@");
        return 0;
    }
    return 1;
}

################################

=head2 setupNGSH 

This routine tries to set up the CLI environment. What commands it uses
depends of the filer type. At a minimum it sets the privilege mode. It
assumes you are logged in and at the cli already.

 setupNGSH ($connection_object)
 returns (1=pass or 0=fail) 

=cut

sub setupNGSH
{
    my ($conObj) = @_;

    trace_message ($t, "CALL", "********** setup CLI environment **********");

    my $hosttype = ($conObj->hostp())->hosttype();
    trace_message ($t, "INFO", "Setup CLI environment for $hosttype");

    if ($hosttype =~ /-ng-/) {
        eval {    
        say($conObj, "set -confirmations off", "-prompt", $conObj->prompts());
        say($conObj, "rows 0", "-prompt", $conObj->prompts());
        say($conObj, "set -priv advanced", "-prompt", $conObj->prompts());
        };
    } else {
        eval {    
        say($conObj, "priv set advanced" );
        };
    }
    if($@)
    {
        trace_message ($t, "ERROR", "while trying to setup NGSH: $@");
        return 0;
    }
    return 1;
}

################################

=head2 startShell 

This routine tries to log in to the systemshell using the 
specified username and password. It assumes an already existing 
connection to the cli, that setupNGSH has been run and that it
will get a standard login/password prompt. 

 startShell ($connection_object, $node, $username, $password)
 returns (1=pass or 0=fail) 

=cut

sub startShell {

    my $conObj = shift;
    my $node   = shift;
    my $user   = shift;
    my $pass   = shift;

    my $fail_cnt = 0;
    my $diagPrompt = "%";
    my $after = undef;
    my $systemshell = undef;
    my $setupNGSH;
    
    trace_message ($t, "CALL", "********** startShell **********");
    trace_message ($t, "INFO", "Start SystemShell on $node");

    eval { say($conObj, "\n", -noprompt, "-wait", $diagPrompt, "-timeout", 15); };
    # Poke, and if you find the % prompt assume systemshell is already started
    if ($RESPONSE =~ /$diagPrompt/) {   
                return 1;  }

    my $hosttype = ($conObj->hostp())->hosttype();
    my $hostname = ($conObj->hostp())->hostname();
    
    # Check the node type and whether it is in maintenance mode
    if (($hosttype =~ /-ng-/) && ($node !~ /MAINT/)) {
	$systemshell = "systemshell -node local";
    } else {
	$systemshell = "systemshell";
    }
    
    # Decide if you need to set privilege
    if ($node !~ /MAINT/) {
        unless (setupNGSH($conObj)) { $fail_cnt++; }
    }

    if ($fail_cnt == 0) {
        trace_message ($t, "INFO", "Execute $systemshell command");
        eval { (undef, undef, $after, undef) = say($conObj, $systemshell, -noprompt, "-wait", qr/login:/i, "-timeout", 60, -swallow); };
        if ($after =~ /too many bad logins/i) {
	     # Tharn::connect does not seem to deal with this problem too well, so try to fix it yourself
             sleep(10); # wait for the login timeout to expire, poke it just once and try again
             eval { (undef, undef, $after, undef) = say($conObj, "\n", -noprompt, "-wait", qr/login:/i, "-timeout", 60, -swallow); };
	}
        if ($RESPONSE !~ qr/login:/i) {
                trace_message ($t, "ERROR", "Login prompt not detected");
                $fail_cnt++;
    }}

    if ($fail_cnt == 0) {
        eval { say($conObj, "$user", -noprompt, "-wait", qr/Password:/i, "-timeout", 30); };
        if ($RESPONSE !~ qr/Password:/i) {
                trace_message ($t, "ERROR", "Password prompt not detected");
                $fail_cnt++;
    }}
 
    if ($fail_cnt == 0) {
        eval { say($conObj, "$pass", -noprompt, "-wait", $diagPrompt, "-timeout", $timeout); };
        if ($RESPONSE !~ /$diagPrompt/) {
                trace_message ($t, "ERROR", "Shell prompt not detected");
                $fail_cnt++;
    }}
   
    # The return from this routine is a little tricky. Since logging into the
    # systemshell hangs if it fails (you stay in the login/password prompt loop)
    # and just doing a disconnect does not always get you out, we call exitShell
    # if an error was detected to try to hack our way out. Then we will  
    # will disconnect the session and then return error. So it is possible that
    # the caller may need to reconnect if it wishes to continue (like the 
    # acct_locked tests do as they want to unlock at the end).
    
    if ($fail_cnt != 0) { 
        exitShell ($conObj, $node);     
        myDisconnectPlus ($conObj); 
    }     

    trace_message ($t, "startShell fail_cnt: $fail_cnt");
    return ($fail_cnt == 0) ? 1 : 0; 

}

################################

=head2 exitShell 

This routine is used to exit the systemshell. The problem here
is that if the login fails the only way to get back to the
cli prompt is to try to login as admin with no password. Not
sure why, but that is handled differently than trying to login 
as root. This also resets the privilege level set in setupNGSH.

 exitShell ($connection_object, $node)
 returns (1=pass or 0=fail) 

=cut

sub exitShell {

    my $conObj = shift;
    my $node   = shift;
    my $user = "admin";
    my $pass;
    my $diagPrompt = "%";
    my $maintPrompt = "*>";

    my $fail_cnt = 0;
    
    trace_message ($t, "CALL", "********** exitShell **********");
    trace_message ($t, "INFO", "Force Exit of SystemShell on $node");

    eval { say($conObj, "\n", -noprompt, "-wait", qr/$diagPrompt/, "-timeout", 2); };
    if (($@) && ($node !~ /MAINT/)) { 
        trace_message ($t, "INFO", "NOT in systemshell");
        return ($fail_cnt == 0) ? 1 : 0; 
    }

    my $hosttype = ($conObj->hostp())->hosttype();
    my $hostname = ($conObj->hostp())->hostname();

    if ($hosttype =~ /-ng-/) {
        $pass = "netapp1!";
    } else {
        $pass = "\n";
    }
    
    eval { say($conObj, "\n", -noprompt, -swallow); };
    eval { say($conObj, "$user", -noprompt, "-wait", qr/Password:/i, "-timeout", 2); };
    if ($RESPONSE =~ qr/Password:/i) {
        eval { say($conObj, "$pass", -prompt, $conObj->prompts(), "-timeout", 4); };
    } elsif ($RESPONSE =~ qr/login:/i) {
        eval { say($conObj, "$user", -noprompt, "-wait", qr/Password:/i, "-timeout", 2); };
        eval { say($conObj, "$pass", -prompt, $conObj->prompts(), "-timeout", 4); };
    } else {
        trace_message ($t, "ERROR", "Forced Exit of SystemShell Failed");
    }
    # An $@ here is not important, as long as the RESPONSE strings are correct.
    
    if ($hosttype =~ /-ng-/) {
        eval {    
        say($conObj, "set -priv admin", "-prompt", $conObj->prompts(), "-timeout", 4);
        };
    } else {
        eval {    
        say($conObj, "priv set admin" );
        };
    }
    if ($@) { 
        trace_message ($t, "ERROR", "reset privilege failed: $@");
        $fail_cnt++; }

    # It is possible that this routine was called in error, so we are going to
    # ignore any errors about logging in and allow the caller to figure it out.
    # Instead it will return fail if we do not think we are at a valid prompt.

    return ($fail_cnt == 0) ? 1 : 0; 

}

################################

=head2 startConsole 

This routine tries to log in to the RLM system console using the 
specified username and password. It assumes an already existing 
connection to the RLM, and that it will get a standard 
login/password prompt. It is also capable of dealing with 
closing and existing RLM connection.

 startConsole ($connection_object, $username, $password)
 returns (1=pass or 0=fail) 

=cut

sub startConsole
{
    my ($conObj, $user, $pass) = @_;
    my $wait; 
    my $before; 
    my $after; 
    my $prompt;
    
    trace_message ($t, "CALL", "********** startConsole **********");
    trace_message ($t, "INFO", "Start RLM System Console");
    
    my $hosttype = ($conObj->hostp())->hosttype();

    # When starting the RLM system console you do not know if you will get a 
    # login prompt or not. And in 7mode you may get just a password prompt. 
    # So if you got a login prompt or password prompt start from there. If
    # neither run prepConsole to see if you can find one of them.
      
    eval { ($wait, $before, $after, $prompt) = say($conObj, "system console", "-wait", qr/login:/i, qr/Password:/i, qr/Type Ctrl-D to exit/i, "-noprompt", -timeout, 10);  };
    if ($@)
    {
        trace_message ($t, "INFO",  "RLM system console command failed");
        trace_message ($t, "INFO",  "| $wait | $before | $after | $prompt | $RESPONSE |"); 

        # However, do not give up just because an error occurred
	# Look for the already connected message
	# 	User naroot has an active console session.
	# 	Would you like to disconnect that session, and start yours [y/n]?
	# (and if you didn't notice, it is because of stuff like this that 
	#  routines like transit_to were developed) 	

        if ($before =~ /Would you like to disconnect that session/) {
    		eval { ($wait, $before, $after, $prompt) = say($conObj, "y", "-wait", qr/login:/i, qr/Password:/i, "-noprompt", -timeout, 10);  };
    		if($@) { trace_message ($t, "INFO",  "$@"); }
        } else {
                # Sometimes the dialog does not show correctly, so poke at the RLM to make sure
                eval { ($wait, $before, $after, $prompt) = say($conObj, "\n", "-wait", qr/login:/i, qr/Password:/i, "-noprompt", -timeout, 10);  };
        }
    }

    if (($wait =~ qr/login:/i) || ($after =~ qr/login:/i))
    {
        if ( login($conObj, $user, $pass, 0) == 0 ) {
            trace_message ($t, "ERROR", "RLM system console login failed");
            return 0;
        }
    }
    elsif (($wait =~ qr/Password:/i) || ($after =~ qr/Password:/i))
    {
        eval {    
            say($conObj, $pass, "-noprompt"); 
        };
        if($@)
        {
            trace_message ($t, "ERROR", "RLM system console password failed: $@");
            # Just in case, don't return here, let it fail looking for the prompt. 
        }
     }
     else
     {
         if ( login($conObj, $user, $pass, 1) == 0 )
         {
            trace_message ($t, "ERROR", "RLM system console login/prepConsole failed");
            return 0;
         }
    } 

    # For some reason just using a \n to poke the console here stopped working in 7mode.  
    # So, after a short nap to allow the login to complete, we will execute a mode dependent command instead.
    sleep (2);
    if ($hosttype =~ /-ng-/) {
        eval {    
            say($conObj, "\n", "-blind"); 
            say($conObj, "system image show -node local -field version -iscurrent true\n", "-prompt", $conObj->prompts(), -swallow); 
	    # All of that just to get the version :)
        };
    } else {
        eval {    
            say($conObj, "\n", "-blind"); 
            say($conObj, "version\n", "-prompt", $conObj->prompts(), -swallow); 
        };
    }
    if($@)
    {
        trace_message ($t, "ERROR", "RLM system console command failed: $RESPONSE");
        return 0;
    }

    return 1;
}


################################

=head2 lock_diag_password 

This routine uses the FreeBSD pw lock command to disable diag
user login. 

 lock_diag_password ($connection_object)
 returns (1=pass or 0=fail) 

=cut

sub lock_diag_password {

    my $conObj = shift;
    
    trace_message ($t, "CALL", "********** lock_diag_password **********");
    trace_message ($t, "INFO", "Disable the password for diag");
    # Actually this uses the FreeBSD locking command to disable login. 
    # This is NOT the same as locking it from ngsh via security login.
    
    eval { 
        say($conObj, "sudo /usr/sbin/pw lock diag", "-prompt", "%", "-timeout", 10); 
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not disable diag password: $@");
        return 0;
    }

    return 1;
}
################################

=head2 unlock_diag_password 

This routine uses the FreeBSD pw unlock command to disable diag
user login. 

 unlock_diag_password ($connection_object)
 returns (1=pass or 0=fail) 

=cut

sub unlock_diag_password {

    my $conObj = shift;
    
    trace_message ($t, "CALL", "********** unlock_diag_password **********");
    trace_message ($t, "INFO", "Enable the password for diag");

    eval { 
        say($conObj, "sudo /usr/sbin/pw unlock diag", "-prompt", "%", "-timeout", 10); 
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not enable diag password: $@");
        return 0;
    }

    return 1;

}

################################

=head2 reset_user_password 

This routine uses the FreeBSD passwd command to reset the
password of the specified user. It does assume that the
options security.rules have been disabled. 

 reset_user_password ($connection_object, $username, $password)
 returns (1=pass or 0=fail) 

=cut

sub reset_user_password {

    my $conObj = shift;
    my $username = shift;
    my $password = shift;
    unless ($username) { $username = $diagUser; }
    unless ($password) { $password = $diagPass; }
    
    trace_message ($t, "CALL", "********** reset_user_password **********");
    trace_message ($t, "INFO", "Set the password for $username to $password");

    eval { 
        say($conObj, "sudo /usr/bin/passwd $username", "-noprompt", "-wait", "New Password", "-timeout", 10); 
        say($conObj, $password, "-noprompt", "-wait", "New Password", "-timeout", 10); 
        say($conObj, $password, "-prompt", "%", "-timeout", 10); 
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not set $username password: $@");
        return 0;
    }

    return 1;
}

################################

=head2 lock_diag_account 

This routine uses the security login or useradmin diaguser 
lock command to disable diag user login. 

 lock_diag_account ($connection_object)
 returns (1=pass or 0=fail) 

=cut

sub lock_diag_account {

    my $conObj = shift;
    my $command1; my $command2;
   
    trace_message ($t, "CALL", "********** lock_diag_account **********");
    trace_message ($t, "INFO", "Lock the account for diag");
    my $hosttype = ($conObj->hostp())->hosttype();

    if ($hosttype =~ /-ng-/) {
	$command1 = "security login lock -username diag" ;
	$command2 = "security login show -username diag -acctlocked yes -instance" ;
    } else {
        $command1 = "useradmin diaguser lock";
        $command2 = "useradmin diaguser show";
    }

    eval { 
        say($conObj, $command1, "-timeout", 10); 
        say($conObj, $command2, "-wait", "Locked: yes", "-timeout", 10);         
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not unlock diag account: $@");
        return 0;
    }

    return 1;
}

################################

=head2 unlock_diag_account 

This routine uses the security login or useradmin diaguser 
unlock command to disable diag user login. 

 unlock_diag_account ($connection_object)
 returns (1=pass or 0=fail) 

=cut

sub unlock_diag_account {

    my $conObj = shift;
    my $command1; my $command2;
    
    trace_message ($t, "CALL", "********** unlock_diag_account **********");
    trace_message ($t, "INFO", "Unlock the account for diag");
    my $hosttype = ($conObj->hostp())->hosttype();

    if ($hosttype =~ /-ng-/) {
	$command1 = "security login unlock -username diag" ;
	$command2 = "security login show -username diag -acctlocked no -instance" ;
    } else {
        $command1 = "useradmin diaguser unlock";
        $command2 = "useradmin diaguser show";
    }

    eval { 
        say($conObj, $command1, "-timeout", 10); 
        say($conObj, $command2, "-wait", "Locked: no", "-timeout", 10);         
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not unlock diag account: $@");
        return 0;
    }

    return 1;
}

################################

=head2 check_diag_unlocked 

This routine uses the security login or useradmin diaguser show 
command check the lock state of the diag user. 

 check_diag_unlocked ($connection_object)
 returns (1=unlocked or 0=locked) 

=cut

sub check_diag_unlocked {

    my $conObj = shift;
    my $node = shift;
    my $command;
    
    trace_message ($t, "CALL", "********** check_diag_unlocked **********");
    trace_message ($t, "INFO", "Check if account for diag is unlocked");
    # returns True if UnLocked and False if Locked
    my $hosttype = ($conObj->hostp())->hosttype();

    if ($hosttype =~ /-ng-/) {
	$command = "security login show -username diag -acctlocked no -instance" ;
    } else {
        $command = "useradmin diaguser show";
    }
   
    eval { 
        say($conObj, $command, "-wait", "Locked: no", "-timeout", 10);         
    };
    if ($@) { return 0;  }
    return 1;
}

################################

=head2 change_diag_password 

This routine uses the security login or useradmin diaguser password 
command change the password of the diag user. 

 change_diag_password ($connection_object)
 returns (1=pass or 0=fail) 

=cut

sub change_diag_password {

    my $conObj = shift;
    my $password = shift;
    my $command;
    unless ($password) { $password = $diagPass; }
    
    trace_message ($t, "CALL", "********** change_diag_password **********");
    trace_message ($t, "INFO", "Change the password for diag account");
    # returns True if Succeeded and False if Not
    my $hosttype = ($conObj->hostp())->hosttype();

    if ($hosttype =~ /-ng-/) {
	$command = "security login password -username diag" ;
    } else {
        $command = "useradmin diaguser password";
    }

    eval { 
        say($conObj, $command, -noprompt, "-wait", ["Please enter a new password","Enter a new password"], "-timeout", 10); 
        say($conObj, $password, "-noprompt", "-wait", ["Please enter it again","Enter it again"], "-timeout", 10); 
        say($conObj, $password, "-prompt", "-timeout", 10); 
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not change diag account password: $@");
        return 0;
    }

    return 1;
}

################################

=head2 updateSecurityRoles 

This routine changes the security password rules to allow
the same passowrd to be used repeatedly.

 updateSecurityRoles ($connection_object, $count)
 returns (1=pass or 0=fail) 

=cut

sub updateSecurityRoles
{
    my $conObj = shift;
    my $count = shift;
    my $command;
    unless ($count) { $count = 1; }
    
    trace_message ($t, "CALL", "********** updateSecurityRoles **********");
    trace_message ($t, "INFO", "Disable/Enable Security Roles");
    my $hosttype = ($conObj->hostp())->hosttype();

    if ($hosttype =~ /-ng-/) {
	$command = "security login role config modify -role * -disallowed-reuse $count " ;
    } else {
        $command = "options security.passwd.rules.history $count";
    }
    eval {    
        say($conObj, $command, "-prompt", $conObj->prompts());
    };
    if($@)
    {
        trace_message ($t, "ERROR", "while trying to disable/enable security roles: $@");
        return 0;
    }

    # Not totally sure what is going on but sometimes this seems to get missed. Not sure if it takes a
    # few moments for the values to get updated so wait a second before continuing and make sure the
    # buffer is clear. 
    sleep(5);  
    say($conObj, -blind, -swallow); 

    if ($hosttype =~ /-ng-/) {
	$command = "security login role config show -role admin -fields disallowed-reuse" ;
        eval {    
            say($conObj, $command, "-wait", "admin $count", "-timeout", 10);         
        };
        if($@)
        {
            trace_message ($t, "ERROR", "while trying to disable/enable security roles: $@");
            return 0;
        }
    }

    return 1;
}

################################

=head2 autoboot 

This routine attempts to change the AUTOBOOT variable 
to the specified mode by using the sysctl command.

 autoboot ($connection_object, $mode)
 returns (1=pass or 0=fail) 

=cut

sub autoboot {

    my $conObj = shift;
    my $mode = shift;
   
    trace_message ($t, "CALL", "********** autoboot **********");
    trace_message ($t, "INFO", "Change AUTOBOOT to $mode");

    my $command = "sudo sysctl kern.bootargs=AUTOBOOT=$mode";
    eval { 
        say($conObj, $command, "-wait", "kern.bootargs:  -> true", "-timeout", 10);         
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not change AUTOBOOT setting: $@");
        return 0;
    }

    return 1;
}

################################

=head2 Options_Hash 

This routine is used to make sure the testcase starts with the correct
set of options. It updates those Options with the ones
defined just for this testcase. This is slightly different than the
way TestManager uses Options, as in it once you change a value it does
not get reset to the default. It is called as part of shell_initialize_testcase.

 Options_Hash ($Testcase_Name, $Options_Hash)
 returns (1=pass or 0=fail) 

=cut

sub Options_Hash {
    my $Testcase = shift; # Name of current testcase
    my $Options  = shift; # Reference to Options  
    my $Params;		  # updated options hash

    trace_message ($t, "CALL", "********** Options_Hash **********");
     
    $Options->process();
    $Params = $Options->get_hash();
    
    my $Testcase_Hash = $Options->get(testcase=>$Testcase);
    while (my ($key, $value) = each(%$Testcase_Hash) ) {
       $Params->{ $key } = $value;
    }        

    $Params = $Options->get_hash();

    return $Params;
}    

################################

=head2 trace_message 

This routine is used to make Trace messages look better when viewed in
a TCD logfile. Nothing fancy, they just look nicer.

 trace_message ($Trace_Object, $Trace_Class, $Message)
 returns (1=pass or 0=fail) 

=cut

sub trace_message {

    my $trace = shift;
    my $class = shift;
    my $message = shift;

    $trace->comment(msg=> "<---------------------------------------->");
    $trace->comment(msg=> "$class: $message");
    if ($Test) { $Test->ndisplay("$class: $message"); }
    unless ($trace) { print ("________ ______ [trace_message] $class $message"); }
    $trace->comment(msg=> "<---------------------------------------->");

}

################################

=head2 shell_navigate_test 

This routine is is uses as part of the navigate testcases.
It contains a number of FreeBSD commands that are used to
navigate around in the systemshell and look at various things.

 shell_navigate_test ($connection_object, $node, $connection_type)
 returns (1=pass or 0=fail) 

=cut

sub shell_navigate_test {

    my $conObj = shift;
    my $node = shift;
    my $mode = shift;
    my $fail_cnt = 0;
    my $wait; my $before; my $after; my $prompt;

    trace_message ($t, "CALL", "********** shell_navigate_test **********");

    my $hconn = make_host($node);
    my $hosttype = $hconn->hosttype();
    my $hostname = $hconn->hostname();

    eval {
    ($wait, $before, $after, $prompt) = say($conObj, "pwd", -prompt, "%", "-wait", "/var/home/diag", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "ps -ax", -prompt, "%", "-wait", "/sbin/init", -timeout, 30, -swallow);

    ($wait, $before, $after, $prompt) = say($conObj, "cd /cfcard/x86*/freebsd/image1", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "pwd", -prompt, "%", "-wait", "/freebsd/image1", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "cat BUILD", -prompt, "%", "-wait", "CLIENT", -timeout, 5, -swallow); 

    ($wait, $before, $after, $prompt) = say($conObj, "cd /", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "cd etc", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "pwd", -prompt, "%", "-wait", "/etc", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "sudo cat hosts", -prompt, "%", "-wait", "localhost", -timeout, 15, -swallow); 

    ($wait, $before, $after, $prompt) = say($conObj, "cat sudoers", -prompt, "%", "-wait", "Permission denied", -timeout, 15, -swallow); 
    # For some reason the output from this command gets mixed in with the pwd below, so skip it
    ($wait, $before, $after, $prompt) = say($conObj, "sudo cat sudoers", -prompt, "%", "-wait", "sudoers file", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, -blind, -swallow); 

    ($wait, $before, $after, $prompt) = say($conObj, "cd /var", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "cd log", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "tail *.log", -noprompt, "-wait", "netapp_mgmt.log", -timeout, 30, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "pwd", -prompt, "%", "-wait", "/var/log", -timeout, 5, -swallow); 

    ($wait, $before, $after, $prompt) = say($conObj, "cd ..", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "pwd", -prompt, "%", "-wait", "/var", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "ls crash", -prompt, "%", "-wait", "Permission denied", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "sudo ls -la crash", "-wait", "root", -prompt, "%", -timeout, 5, -swallow); 

    ($wait, $before, $after, $prompt) = say($conObj, "cd /", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "pwd", -prompt, "%", "-wait", "/", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "cat VERSION", -prompt, "%", "-wait", "_", ".", -timeout, 5, -swallow); 

    ($wait, $before, $after, $prompt) = say($conObj, "cd", -prompt, "%", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "pwd", -prompt, "%", "-wait", "/var/home/diag", -timeout, 5, -swallow); 

    if (($hosttype =~ /-ng-/) && ($mode !~ /Maint/)) { 
      ($wait, $before, $after, $prompt) = say($conObj, "ps -ax | grep vldb", -prompt, "%", "-wait", "/sbin/vldb", -timeout, 5, -swallow); 
      ($wait, $before, $after, $prompt) = say($conObj, "pkill vldb", -prompt, "%", "-wait", "Operation not permitted", -timeout, 5, -swallow); 
      ($wait, $before, $after, $prompt) = say($conObj, "sudo pkill vldb", -prompt, "%", -timeout, 5, -swallow); 
      ($wait, $before, $after, $prompt) = say($conObj, "ps -ax | grep vldb", -prompt, "%", "-wait", "/sbin/vldb", -timeout, 5, -swallow); 
    }
    
    ($wait, $before, $after, $prompt) = say($conObj, "ps -ax | grep mgwd | grep -v grep", -prompt, "%", "-wait", "mgwd", -timeout, 5, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "pkill mgwd", -prompt, "%", "-wait", "Operation not permitted", -timeout, 5, -swallow); 
    };

    if ($@) {
        $Test->ndisplay("NAVIGATION ERROR: $wait | $before | $after | $prompt");
        $Test->ndisplay("$@");
        return 0;
    }
    return 1;
        
}

################################

=head2 shell_command_test 

This routine is is uses as part of the command testcases.
It contains a number of FreeBSD commands that are executed,
some with sudo, some not, to make sure they execute
properly in the systemshell.

 shell_command_test ($connection_object, $node, $connection_type)
 returns (1=pass or 0=fail) 

=cut

sub shell_command_test {

    my $conObj = shift;
    my $node = shift;
    my $mode = shift;
    my $fail_cnt = 0;
    my $wait; my $before; my $after; my $prompt;

    trace_message ($t, "CALL", "********** shell_command_test **********");
    # Why does this code have say -blind -swallow in it you ask ? Because that  
    # was the only way I could get say to correctly process the wait string !!
    
    my $hconn = make_host($node);
    my $hosttype = $hconn->hosttype();
    my $hostname = $hconn->hostname();

    eval { 
    if ($mode =~ /Maint/) { # if this test is in maintenance mode the commands are slightly different
    ($wait, $before, $after, $prompt) = say($conObj, "/usr/bin/uname -a", -prompt, "%", "-wait", "Data ONTAP", -timeout, 15, -swallow); 
    } else {
    ($wait, $before, $after, $prompt) = say($conObj, "hostname", -prompt, "%", "-wait", "$hostname", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "/usr/bin/uname -a", -prompt, "%", "-wait", "$hostname", -timeout, 15, -swallow); 
    }
    ($wait, $before, $after, $prompt) = say($conObj, "/sbin/halt", -prompt, "%", "-wait", "Operation not permitted", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "/sbin/shutdown", -prompt, "%", "-wait", "Permission denied", -timeout, 15, -swallow);
    ($wait, $before, $after, $prompt) = say($conObj, "/sbin/ifconfig -a", -prompt, "%", "-wait", "LOOPBACK", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, -blind, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "/usr/bin/netstat", -noprompt, "-wait", "Active Internet connections", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "\cC", -prompt, "%", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, -blind, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "sudo /sbin/ping -c 3 localhost", -prompt, "%", "-wait", "3 packets transmitted", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, -blind, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "/usr/bin/top", -noprompt, "-wait", "Mem:", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "\cC", -prompt, "%", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, -blind, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "sudo sysctl kern.bootargs=AUTOBOOT", -noprompt, "-wait", "kern.bootargs", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, -blind, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "sudo /usr/sbin/tcpdump -i lo0", -noprompt, "-wait", "listening", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, "\cC", -prompt, "%", -timeout, 15, -swallow); 
    ($wait, $before, $after, $prompt) = say($conObj, -blind, -swallow); 
    };
    if ($@) {
        $Test->ndisplay("BASIC COMMAND ERROR DETECTED !");
        #$Test->ndisplay("STRINGS: | $wait | $before | $after | $prompt |");
        $Test->ndisplay("RESPONSE: $RESPONSE");
        $Test->ndisplay("ERROR: $@");
        $fail_cnt++;
    }
    return ($fail_cnt == 0) ? $TCD::PASS : $TCD::FAIL; 
        
}

################################

=head2 enable_toor 

This routine attempts to enable the FreeBSD toor user.
It is not actually used in the tests, but you never know.

 enable_toor ($connection_object)
 returns (1=pass or 0=fail) 

=cut

sub enable_toor {

    my $conObj = shift;
    
    trace_message ($t, "CALL", "********** enable_toor **********");
    trace_message ($t, "INFO", "Enable toor account");

    eval { 
        say($conObj, "cd /tmp", "-prompt", "%", "-timeout", 10); 
        say($conObj, "cp /var/etc/passwd .passwd", "-prompt", "%", "-timeout", 10);         
        say($conObj, "cat /var/etc/passwd | grep toor | sed 's/\/usr\/sbin\/nologin/\/bin\/csh/' > toor", "-prompt", "%", "-timeout", 10); 
        say($conObj, "cat /var/etc/passwd | grep -v toor > passwd", "-prompt", "%", "-timeout", 10); 
        say($conObj, "cat toor >> passwd", "-prompt", "%", "-timeout", 10); 
        say($conObj, "sudo mv passwd /var/etc/passwd", "-prompt", "%", "-timeout", 10);         

        say($conObj, "sudo passwd toor", "-noprompt", "-wait", "New Password", "-timeout", 10); 
        say($conObj, "netapp1!", "-noprompt", "-wait", "Retype New Password", "-timeout", 10); 
        say($conObj, "netapp1!", "-prompt", "%", "-timeout", 10); 
        say($conObj, "exit", "-prompt", $conObj->prompts(), "-timeout", 10); 
    };
    if ($@) {
        trace_message ($t, "ERROR", "Could not enable toor: $@");
        return 0;
    }

    return 1;
}

# -------------- PRIVATE ROUTINES ----------
sub release {

    # Get the release information for this node, and return all 3
    # values as it is the safest way to ensue you get what you want.
    
    my $HostName = shift;
    my $HostType = shift;
    my $host = Tharn::host($HostName);
    if ( $HostType =~ /ng/i ) {
    	$HostType = "CMode"; 
    } else {
    	$HostType = "7Mode"; 
    }
    trace_message ($t, "INFO", "Create a $HostType Version object for $HostName");
    my $version_manager = NACL::APISet::Version->new(hostobj    => $host,
                                                 category   => "Node",
                                                 shell_type => "$HostType"
                                                 );

    my $branch  = $version_manager->get_version_attribute( attribute => "branch");
    my $release = $version_manager->get_version_attribute( attribute => "release");
    my $mapped_release = $version_manager-> map_release_to_codename(release => $release);
    my $release_strings = $release." ".$mapped_release." ".$branch;

    return $release_strings;
}

return (1);

