#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Os/PTS.pm#2 $
## @summary Package profiding ability to easily execute an existing
##          3rd party posix test suite pjd-fstest-20080816 via NATE
## @author knopp@netapp.com
## @status private
## @pod here

package Os::PTS;
use strict;
use IO::File;
use Module::Common qw(validate_mixed);
use Params::Validate qw(validate SCALAR SCALARREF BOOLEAN OBJECT ARRAYREF UNDEF);
use Time::HiRes qw(gettimeofday);
use Utils qw(make_host make_conn);
use Os::PTS::Cmd;
use Error;
use Tharn qw ($Log);
use Data::Dumper;
use Options;
use Tie::IxHash;
use TestUtils qw(get_site_info);
my %GlobalOf;
my %MapOf;
my $RsId;

use constant PTS_LOC
    => "/x/eng/localtest/noarch/share/pjd/pjd-fstest-20080816";

our @ISA = qw(Error);

###############################################################################
# Method:    Cleanup()
# Objective: Cleanup files copied to host dir under test
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cleanup {
    my ($self)=@_;
    
    ### Initialize
    $self->_clear_err();
    
    eval {
        $self->_rm_files();
    };
    if ($@) {
        ($self->on_err()=~/die/) ? die $@ : $self->_set_err($@);    
        return 0;
    }
    
    return 1;
}

###############################################################################
# Method:    get_hosttype()
# Objective: Get client host type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute {
    my ($self,%args)=@_;
    my (
        $alias,
        $assert_cnt,
        $cmdObjs,
        $fail_cnt,
        $name,
        $rs_id,
        @failures,
    );

    ### Initialize variables
    $assert_cnt=0;
    $fail_cnt=0;
    

    $self->_log(msg=>["CONFIGURATION:",""],indent=>0 );
    
    foreach my $cmdObj (@{$args{cmdObjs}}){

        ### display headers
        if ($cmdObj->get_type() =~ /cmd|param/i) {
            $self->_log(msg=>"    ".$cmdObj->get_raw(),indent=>0);
        }
        elsif ($cmdObj->get_type() =~ /assertion/) {
            $self->_log(msg=>[sprintf("%s\n%s %d: ",
                "-"x80,
                "ASSERTION",
                $cmdObj->get_tcid())],indent=>0);
            $assert_cnt++;
        }
       
        ### Execute command
        $cmdObj->execute();
	
        ### display command result
        if (($cmdObj->get_type() =~ /param/i) &&
            ($cmdObj->get_raw() =~ /\$|\`/)){
            $self->_log(msg=>["    ".$cmdObj->get_result(),""],indent=>0);
        }
        elsif ($cmdObj->get_type() =~ /assertion/i) {
            
            my $len=length($cmdObj->get_cmd());
            
            $self->_log(msg=>[sprintf("    test: %s%s\n    result: %s%s\n",
		$cmdObj->get_cmd(),
		($cmdObj->is_expect()) ? "" : (' 'x(80-$len-20))."**FAIL**",
		$cmdObj->get_result(),
		(not $cmdObj->is_expect())
                    ? "\n    expect:  ".$cmdObj->get_expect()
		    : "")], indent=>0
            );
            
            if (not $cmdObj->is_expect()){
                $fail_cnt++;
                push @failures,$cmdObj->get_tcid();
            }
                       
        }
        
    }
    
    $rs_id="R".$RsId++;
    $name=$self->{name};
    $alias=$self->{alias};
    
    $MapOf{$rs_id}={
        alias=>$alias,
        name=>$name,
        assert_cnt=>$assert_cnt,
        fail_cnt=>$fail_cnt,
        failures=>(@failures) ? join(",",@failures) : "None"
    };
    
    $self->_log(msg=>["",sprintf(
        "%-20.20s %-10.10s %-5.5s %-5.5s %-s\n".
        "%-20.20s %-10.10s %-5.5s %-5.5s %-s\n".
        "%-20.20s %-10.10s %4d  %4d   %-s",
        "TEST NAME","POSIX NAME","TESTS","FAILS","FAILURE IDS",
        "="x20,"="x10,"="x5,"="x5,"="x40,
        $MapOf{$rs_id}{alias},
        $MapOf{$rs_id}{name},
        $MapOf{$rs_id}{assert_cnt},
        $MapOf{$rs_id}{fail_cnt},
        $MapOf{$rs_id}{failures},
        )],indent=>0
    );
      
    return $fail_cnt;
}

###############################################################################
# Method:    get_arch()
# Objective: Get client architecture
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_arch{
    my($self) = @_;
    
    return $self->{arch};
    
}

###############################################################################
# Method:    get_hosttype()
# Objective: Get client host type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_hosttype{
    my($self) = @_;
    return $self->{conn}{hostp}{hosttype};
}


###############################################################################
# Method:    get_os()
# Objective: Get client host type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_os {
    my($self) = @_;
    my(
       $hosttype, 
       $os,
       @outputs,
       );
    
    $hosttype=$self->get_hosttype();
    
    if ($hosttype=~/ng/i){
	@outputs=$self->_execute(cmd=>"uname -a");
	if ($outputs[0]=~/boilermaker|rollingrock|sierranevada|fullsteam/i){
	    $os=($outputs[0]=~/boilermaker/i) 
		? "freebsd6"
		: "freebsd7";            
	}             
    }
    elsif ($hosttype=~/lin/i) {
        $os="linux";
    }
    else {
        $os=$hosttype;
    }

    return $os;
}
###############################################################################
# Method:    load()
# Objective: load testcase script
# Details:   See POD documentation at the end of this file.
###############################################################################
sub load {
    my ($self,%args)=@_;
    my (
        $buffer,
        $cmd,
        $errmsg,
        $fh,
        $filename,
        $tcid,
        $line,
        $raw,
        $value,
        @cmdObjs,
        %hsh,
        );
    
    ### Initialize variables
    $self->_clear_err();
    $tcid=1;
    
    eval {   
        ### Validate input arguments
        ($self,%args) = validate_mixed (
            @_,
            [ { type => OBJECT} ],
            {
                alias  => {type => SCALAR},  
                name  => {type     => SCALAR,
                    callbacks=>{
                    'invalid format (testname.t)'
                        =>sub{shift()=~/\S+\.t/i
                    }}},
            }
        );
        
        ### Construct full path to testcase
        $self->{name}=$args{name};
        $self->{alias}=$args{alias};
        $filename=$self->{pts_loc}."/tests/".$self->{file_op}."/".$args{name};
    
        ### Open test scenario file
        $fh = IO::File->new($filename, O_RDONLY)
            or die "Couldn't open $filename for reading: $!\n";

        ### Override default location of fstest 
        my $cmdObj=
            Os::PTS::Cmd->_new(
                type=>"param",
                raw=>"fstest=\${fstest}",
                #cmd=>$self->_fstest("fstest"),
                cmd=>$self->_set_param_echo
                    ("fstest",$self->{dir}."/fstest"),
                conn=>$self->{conn},
                on_err=>"rtn",
                );
            
        push @cmdObjs, $cmdObj,$self->_parse($fh);
       
        $self->{cmdObjs}=[@cmdObjs];
        
        close ($fh);
    };
    if ($@){
        if (my $msg=$self->_input_validation($@)) {
            $errmsg=$msg; 
        }
        else {
            $errmsg=$@;
        }
        
        ($self->on_err()=~/die/)? die $errmsg: $self->_set_err($errmsg);
            
        return $self;
    }
    
    return (wantarray()) ? @{$self->{cmdObjs}} : $self->{cmdObjs};
}

###############################################################################
# Method:    new
# Objective: Create new class object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class,%args) = @_;
    my (
      $cmd,
      $errmsg,
      $self,
      $tharnhost, 
      @outputs, 
    );
    
    ### Create instance of object
    $self = bless {}, $class;
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    eval {
        
        ### Validate input arguments
        ($class,%args) = validate_mixed (
            @_,
            [ { type => SCALAR} ],
            {
                hostname  => {type     => SCALAR},
                on_err    => {type => SCALAR, optional=>1, default=>"die",
                             callbacks=>{
                            'legal values'=>sub{shift()=~/die|rtn/i
                            }}},
                loghdl    => {type => SCALARREF, },
                pass      => {type => SCALAR, optional=>1,},
                pts_loc   => {type => SCALAR|UNDEF, optional=>1 },
                dir       => {type => SCALAR,},
                file_op   => {type => SCALAR,},
                timeout   => {type => SCALAR, optional=>1,regex => qr/^\d+$/ },  
                user      => {type => SCALAR, optional=>1,},
               
            }
        );
        
        ###  Lets specify what handler to use
        $self->{on_err}=$args{on_err};
        $self->{pts_loc}=(defined $args{pts_loc}) ? $args{pts_loc} : PTS_LOC;
        $self->{dir}=$args{dir};
        $self->{log} = $args{loghdl};
        $self->{file_op}=$args{file_op};
        
        $self->_log(msg=>"PTS_LOC: ".$self->{pts_loc}, indent=>3);
        
        
        $self->_log(
            msg=>"creating ssh connection to ".$args{hostname},indent=>3);
        
        ### Create a connection to the host
        $self->{conn}=$self->_connect(
               hostname => $args{hostname},
               user     => $args{user},
               pass     => $args{pass},
               timeout  => $args{timeout},
        );
        
        $self->_log(msg=>"setting directory to ".$self->{dir},indent=>3);
        $self->_execute(cmd=>"cd ".$self->{dir});
        
        $self->_log(msg=>"retrieving host architecture ",indent=>3);
        @outputs=$self->_execute(cmd=>"uname -p");
        $self->{arch}="@outputs";
        $self->_log(msg=>$self->{arch},indent=>4);
        

        $self->_copy_files();
        
        $self->_log(msg=>"sourcing file ".$self->{dir}."/misc.sh",indent=>3);
        $self->_execute(cmd=>"source ".$self->{dir}."/misc.sh", stderr=>1);
        
        $self->_log(msg=>"sourcing file ".$self->{dir}."/conf",indent=>3);
        $self->_execute(cmd=>"source ".$self->{dir}."/conf", stderr=>1);
        
        $self->_misc_sh_env();
    };
    if ($@){
      
        if (my $msg=$self->_input_validation($@)) {
            $errmsg=$msg;
        }
        else {
            $errmsg=$@;
        }
        
        ($self->on_err()=~/die/)? die $errmsg: $self->_set_err($errmsg);
            
        return $self;
    }

    tie %MapOf,'Tie::IxHash';

    return $self;
}

###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$type)=@_; 
    
    if (defined $type) {
        $self->{on_err}=$type;
    }
    
    return $self->{on_err}; 
}

###############################################################################
# Method:    show_env
# Objective: show env variables accessible by all tests
# Details:   See POD documentation at the end of this file.
###############################################################################
sub show_env {
    my ($self)=@_;
    my (
        $cnt,
        %cfgOf,
       );
        
    %cfgOf=%GlobalOf;

    foreach my $key (sort keys %cfgOf){
        $self->_log(
            msg=>[$cnt++.") ".$key."=".$cfgOf{$key},""],
            indent=>0
        );
    }
    
    return 1;
}

###############################################################################
# Method:    summary
# Objective: Summary of tests and assertion results
# Details:   See POD documentation at the end of this file.
###############################################################################
sub summary {
    my ($self)=@_;
    my (
        $assert_cnt,
        $fail_cnt,
        );
    
    $self->_log(msg=>[sprintf(
        "%-20.20s %-10.10s %-5.5s %-5.5s %-s\n".
        "%-20.20s %-10.10s %-5.5s %-5.5s %-s",
        "TEST NAME","POSIX NAME","TESTS","FAILS","FAILURE IDS",
        "="x20,"="x10,"="x5,"="x5,"="x40,
        )],indent=>0
    );
    
    foreach my $key (keys %MapOf){
        $self->_log(msg=>[sprintf(
            "%-20.20s|%-10.10s|%4d |%4d | %-s",
            $MapOf{$key}{alias},
            $MapOf{$key}{name},
            $MapOf{$key}{assert_cnt},
            $MapOf{$key}{fail_cnt},
            $MapOf{$key}{failures},
            )],indent=>0
        );
        $assert_cnt+=$MapOf{$key}{assert_cnt};
        $fail_cnt+=$MapOf{$key}{fail_cnt};
    }
    
    $self->_log(msg=>[sprintf(
        "%-20.20s %-10.10s %-5.5s %-5.5s %-s\n".
        "%-20.20s %-10.10s %4d  %4d  %-s",
        "="x20,"="x10,"="x5,"="x5,"="x40,
        "","",$assert_cnt,$fail_cnt,""
        )],indent=>0
    );
 
}

###############################################################################
###  P R I V A T E
###############################################################################
##############################################################################
# Method:    _connect
# Objective: Create ssh connection to host machine
###############################################################################
sub _connect {
    my ($self,%input)=@_;
    my (
        $connObj,
        $connstr,
        $errmsg,
        %args,
       );
    
    ### Merge tharnhost defaults with user input
    eval {
        
        ### Remove input that has undefined values.
        ### Don't want this to override tharnhost
        ### defaults
        foreach my $key (keys %input){
            delete $input{$key} if(not defined $input{$key});      
        } 
        
        ### merge tharnhost with user input
        %args=($self->_get_thlogin_info(%input),%input);
        
    };
    if ($@){
       die $@;
    };
    
    ### Construct the connect string
    $connstr=$args{hostname}.
        "#shell;conntype=ssh-persist;".
        "username=".$args{user}.";".
        "password=".$args{pass}.";".
        "timeout=".$args{timeout}.";".
        "shell_type=bash;shell_path='sudo bash';";
        
        
    ### Create connection object
    $connObj = Tharn::connect(
	$connstr, -login,    
        -eval=>sub{
	    if (@_) {
                $errmsg = (@_)[0]->{messages};
                die @$errmsg;
                return $self;   
	    }
        }
    );
    
    if (not defined $connObj){
        die "ERROR: Unable to connect to host ".
                $args{hostname}.
                "(".$args{user}."/".$args{pass}.")";    
    }
    
    
    #$Log->comment(Dumper($connObj));
    
    return $connObj;

} # end sub connect

##############################################################################
# Method:    _copy_files
# Objective: Copy fstest binary and misc.sh to fs dir under test
###############################################################################
sub _copy_files {
    my ($self)=@_;
    my (
        $arch,
        $cmd,
        $dir,
        $pts_loc,
        $errmsg,
        $options,
        $os,
        %site_of,
        $site,
        );
   
    ### BUG: get_site_info() doesn't know where site file lives unless an
    ### Option option is instantiated.  The Option instance doesn't see site
    ### variable unless a Server object is created (I'll get site info from
    ### the env myself).
    $options = new Options();
    $options->process();
    $site= $ENV{"SITE"};
    %site_of = get_site_info();
   
    $dir=$self->{dir};
    $arch=$self->get_arch();
    $os=$self->get_os();
    $pts_loc=$self->{pts_loc};
    
    ### Copy over the 2 required files to the host directory under test
    ### using wget.  
    eval {
	
	unless ($site_of{$site}->{install_server}) {
	    die "ERROR: No site_info file found using SITE parameter '$site'\n".
	        "Verify that a SITE parameter is defined in ~/.ntest file\n";
	}
	
        foreach my $file (
            "tests/misc.sh",
            "tests/conf",
            "../bin/".$os."/".$arch."/fstest"){
            $cmd =
                "wget " .
                $site_of{$site}->{install_proto}  . '://' .
                $site_of{$site}->{install_user}   . ':' .
                $site_of{$site}->{install_pass}   . '@' .
                $site_of{$site}->{install_server} .
                $pts_loc."/".$file;
                   
            $self->_log(msg=>["copying 'PTS_LOC:$file' to '$dir'"],indent=>3);
            $self->_execute(cmd=>$cmd);
        }
        
        ### Change permissions on ftest binary so it can be executed
        $self->_execute(cmd=>"chmod 777 ".$self->{dir}."/fstest", stderr=>0);
        
    };
    if ($@) {
        ($self->on_err()=~/die/) ? die $@ : $self->_set_err($@);    
        return 0;
    }

    return 1;
      
}

###############################################################################
# Method:    _execute
# Objective: Issue arbitrary command to host and get results.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _execute {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $status,
        $t0,
        $t1,
        @outputs,
       );
    
    # Clear errors.
    $self->_clear_err;


    ### Validate input arguments
    eval{
        ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {   
           cmd      => { type => SCALAR },
           stderr   => { type => SCALAR, optional=>1 },
        })
    };
    if ($@){  
        if (my $msg=$self->_input_validation($@)) {
            $errmsg=$msg;
        }
        else {
            $errmsg=$@;
        }
        
        ($self->on_err()=~/die/)
            ? die $errmsg
            : $self->_set_err($errmsg);
    }
    
    # Init execution timer
    $t0=gettimeofday;
    ($status, @outputs) = $self->_execute_cmd($self->{conn},%args);
    $t1=gettimeofday;
    $self->{command_time}=$t1-$t0;

    # Check for standard set of errors returned from the CLI.
    if ($status) {
        ($self->on_err()=~/die/)
            ? die @outputs
            : $self->_set_err(@outputs);
        return (wantarray()) ? @outputs : \@outputs;
    }

    return (wantarray()) ? @outputs : \@outputs;

} # end sub cmd

##############################################################################
# Method:    _execute_cmd
# Objective: Execute command and return results
###############################################################################
sub _execute_cmd {
    my ($self,$connection,%args)=@_;        
    my ($cmd, $timeout, $stderr);
    my ($env_output, $output, @output, $status);
    my ($match, $before, $after, $sentinel, $default_timeout);

    $cmd        =$args{cmd};
    $stderr     =(defined $args{stderr}) ? $args{stderr} : 1; 
    $timeout    =$args{timeout};
    $default_timeout = "1800";
    @output = ();

    # set the default timeout if not specified
    $timeout = $default_timeout if (!defined($timeout) or $timeout =~ m/^\s*$/);
    
    # For unix determine whether to include or discard stderr with stdout
    if ($connection->{hostp}->{hosttype} =~ /filer|linux|unix|vmware/) {
        $cmd.=  ($stderr) ? " 2>&1" : " 2>/dev/null";
    }

    # add to the command the echo of status to get that information from
    # the unix shell. Also add the sentinel string DoNe to command so that
    # we can match on that string.
    $sentinel = 'DoNe';
    $cmd = $cmd . ";echo \"rstatus=\$\?\" ;echo $sentinel";

    # libssh/libpersay do not print out normal headers letting you know
    # command being executed. Add these.
    my $hostname = $connection->{hostp}->hostname();
    my $conntype = $connection->conntype();
    
    # execute the passed command. Use just timeout since we check earlier
    # now if it was not passed and set the default if that is the case.
    ($match, $output, $after) =  &Tharn::say($connection, $cmd, $sentinel,
                                             '-timeout', $timeout, 
                                             '-ignore', "echo $sentinel",
                                             '-comment', "Attempted '$cmd'");

    # strip output and remove CR/LF
    chomp($output);
    $output =~ s/\r//g;

    # remove first line 
    $cmd =~ s/"rstatus=\$\?" ;echo $sentinel//g;
    $output =~ s/"rstatus=\$\?" ;//g;

    # strip sentinel and then status from output
    $output =~ s/\n$sentinel.*//;

    # strip any telnet ctrl characters that get
    # added to the output (this problem seems 
    # to only occur with certain platforms).
    $output =~ s/ \cH//g;

    # locate status.  Remember that commands
    # echo themselves so make sure you don't 
    # get the command line 
    ($status) = $output =~ /rstatus=(\d+)/;

    # handle case where rstatus is not in the output
    # stream. One case is when NATE times out the command
    # In case of filer, do not do this since rstatus only
    # exists for clients.
    if (not defined $status and 
        $connection->{hostp}->{hosttype} =~ /unix|vmware/) {
        $status = 1;
    }

    $output =~ s/rstatus=\d+//;

    # create an array from the output
    @output = split(/\n/, $output);

    ## We need to determine if command is multiline 
    ## and exists in the output.  IMPORTANT: Not all
    ## commands echo back their input so don't forget to
    ## check before modifying the output.
    my @cmds = split(/\n/, $cmd);

    ## If output contains second line of cmd 
    ## then cmd was echoed to output and is 
    ## multiline
    if (defined $cmds[1] and length $cmds[1]) {
        ## Remove continuation character and parens (from group commands
        ## added recently)
        $cmds[1] =~ s/\\//g;
        $cmds[1] =~ s/\)//g;
        $cmds[1] =~ s/\(//g;
        if ($output[0] =~ /$cmds[1]/) {
          ## determine size of command and strip
          ## from there.
          @output = @output[@cmds-1..@output-1]; 

        }
    }

    ## Go through the output and remove any line that has the command in it
    ## We do this, because none of the function that call execute expect
    ## to have to filter out their cmd from the returned output.
    my @output_sorted;
    foreach my $line (@output) {
        my $match = 0;
        foreach my $cmd (@cmds) {
            if ($cmd eq $line) {
                $match = 1;
                last;
            }
        }

        if ($match == 0) {
            push (@output_sorted, $line);
        }
    }

    @output = @output_sorted;

    # case with dt that run w/aios flag - error status from dt is not reliable
    # add this case to catch/compensate
    $status = 1 if ($output =~ /Error number \d+ occurred on/i);
    
    return($status, @output);

}


################################################################################
## Method: _fstest
################################################################################
sub _fstest {
    my ($self,$param)=@_;
    my (
        $cmd,
       );
    
    $cmd = "$param\='/tmp/fstest';".
           "echo \"$param=\"\$$param";
           
    return $cmd;
}

################################################################################
# Method: _generate_random_string
# Objective: generate a random string of specific length
################################################################################
sub _generate_random_string{
    my ($self,$size)=@_;
    my (
        $length,
        $random_string,
        @chars,
        );
    
    $length=$size;
   
    @chars=('a'..'z','A'..'Z','0'..'9','_');
    $random_string;
    foreach (1..$length) 
    {
            # rand @chars will generate a random 
            # number between 0 and scalar @chars
            $random_string.=$chars[rand @chars];
    }
    return $random_string;
}
###############################################################################
# Method:    _get_thlogin_info
# Objective: Retrieve any missing connection metadata required for ssh login
###############################################################################
sub _get_thlogin_info {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $host_of,
        %login_of,
    );
   
    ### Initialize login structure
    %login_of=(
        user=>'',
        pass=>'',
        timeout=> '20',
    );
 
    ### If tharndoc file exists retrieve connection defaults
    ### from there
    eval {
        
        ### Create a host object
        $host_of = Utils::make_host($args{hostname});
        ### Determine if the machines a filer, otherwise assume
        ### its a client
        if ($host_of->{hosttype}=~/cfe|ng/){
            
           ### There no standard shell connection record tag 
           ### for logging into shell as diags. 
           %login_of=(
                user=>'diag',
                pass=>'netapp1!',
                timeout=>20,
           ); 
        }
        else {
            ### Machine must be a client, use standard login
            ### defaults if there are none in tharndoc
            %login_of=(
                user=>
                    (defined $host_of->{connection_fields}{ssh}{username}) 
                    ?  $host_of->{connection_fields}{ssh}{username}
                    :  "root",
                pass=>
                    (defined $host_of->{connection_fields}{ssh}{password}) 
                    ?  $host_of->{connection_fields}{ssh}{password}
                    :  "netapp",    
                timeout=>
                    (defined $host_of->{connection_fields}{ssh}{timeout}) 
                    ?  $host_of->{connection_fields}{ssh}{timeout}
                    :  20,    
           );    
        }
        
    };
    if ($@){
        die "No tharndoc file exists for machine ".$args{hostname};
    };
    
    ### return the login info
    return %login_of;
}

###############################################################################
# Method:    _input_validation()
# Objective: Parse and format error message
###############################################################################
sub _input_validation {
    my ($self,$err)=@_;
    my (
        $callback,
        $errmsg,
        $param,
        $value,
        );
    $Log->comment("ERR: |".$err."|");
    
    if ($err=~/Mandatory parameter/i){
        ($param)=$err=~/Mandatory parameter (.*) /;
        $errmsg="Mandatory parameter ".$param;
    }
    elsif ($err=~/was not listed/i){
        ($param)=$err=~/options: (\S+)/;
        $errmsg="Invalid parameter ".$param;
    }
    elsif ($err=~/callback/i){
        ($param,$value,$callback)=
            $err=~/'(\S+)' parameter \(\"(\S+)\"\).*'(.*)' callback/i;
        $errmsg="Failed callback '".$callback. "' for ". $param."=>".$value;
    }
    elsif ($err=~/did not pass/i){
        ($param,$value)=$err=~/The '(\S+)' parameter \((\S+)\)/;
        $errmsg="Invalid value ".$param."=>".$value;
    }
    else {
        ### Error was not due to function input
        $errmsg="";
    }
    
    return $errmsg;
}

################################################################################
# Method: _log
# Objective: display library messages to stdout and natelog
################################################################################
sub _log{
    my ($self,%args) = @_;
    my (
        $bFirst,
        $dash,
        @lines,
        $indent,
    );

    $indent=0;

    @lines=(ref $args{msg} eq "ARRAY") ? @{$args{msg}} : ($args{msg});
    $indent=(defined $args{indent}) ? $args{indent} : 0;
    $bFirst=1;
    
    if (defined $self->{log}){
        if (not $indent){
            foreach my $line (@lines){
                print {${$self->{log}}} $line."\n";
            }
        }
        else {
            foreach my $line (@lines){
                $dash = ($bFirst == 1)
                    ? " "x($indent-1)."-"
                    : " "x$indent;
                $bFirst=0;
                print {${$self->{log}}} $dash.$line."\n";
            }
        }
        $Log->comment("@lines");
      
    }

}

##############################################################################
# Method:    _misc_sh_env
# Objective: Retrieve environment variables and store in pgm hash
###############################################################################
sub  _misc_sh_env {
    my ($self)=@_;
    my (
        @outputs,
        @params,
    );
    
    @params=qw(
        name253
        name255
        name256
        name_max
        path1021
        path1023
        path1024
        name_max_val
        path_max_val
        num_of_dirs
        long_dir
        too_long
    );
    
    foreach my $param(@params) {
        @outputs=$self->_execute(cmd=>"echo \$$param", stderr=>0);
        $GlobalOf{$param}=$outputs[0];
    }    
     
    return 1; 
}

################################################################################
# Method: _name_generate
# Objective: generate a random name 
################################################################################
sub _name_generate {
    my ($self,$param,$size)=@_;
    my (
        $cmd,
        #$md5,
        $length,
        $prefix,
        $value,
       );
    
    $prefix="fstest_";
    $size=10 if (not defined $size);
    
    $length=$size-length($prefix);
    $value=$self->_generate_random_string($length);
    
    $cmd = "$param\='fstest_'".$value.";".
    "echo \"$param=\"\$$param";

    return $cmd;
}

################################################################################
# Method: _parse
# Objective: parse posix tests into an array of command objects  
################################################################################
sub _parse {
    my ($self,$fh)=@_;
    my (
        $case_block,
        $case_value,
        $case_option,
        $case_option_match,
        $case_option_ignore,
        $cmd,
        $errmsg,
        $expect,
        $for_param,
        $line,
        $os,
        $raw,
        $size,
        $tcid,
        $type,
        $while_loop,
        @cmdObjs,
        @for_values,
        %hsh,
        );
    
    ### Initialize variables
    $tcid=1;
    
    ### Maintain hash insertion order  
    tie %hsh,'Tie::IxHash';
   
    eval {

        while (my $line=<$fh>){
            
            ### Remove line break (\n) 
            chomp $line;
           
            ### Trim leading spaces
            $line=~s/^\s+//;
            
            ### Initialize variables
            $expect="na";
            $raw=$line;
            $type="";
 
            ### Skip over following command directives and blank
            ### lines in posix test script.
            next if ($line=~
                (/^dir=|^desc=|^\#|misc.sh/) or length($line)<=1);

            ### If 'Case block start' - determine case value to
            ### match on.  NOTE: One of the tests contains a case
            ### statement value '${r}' that cannot be determined
            ### until runtime.  Its values have been hardcoded
            ### here for ease. 
            if ($line=~/^case /i) {
                    if ($line=~/\{os\}/){
                    
                        #my $os_val = $self->get_os();
                        my $os_val = ($self->get_os()=~/freebsd/i)
                                     ? "freebsd"
                                     : $self->get_os();
                                                         
                        $case_value=$os_val;
                    }
                    elsif($line=~/\{r\}/) {
                        $case_value="EBIG|EINVAL";
                    }
                    $case_block=1;
            }

            ### If 'Case block option' check if option matches case
            ### block value
            if ($line=~/\)$/){
		next if ($case_option_ignore);
                ($case_option)=$line=~/(.*)\)/i;
		$case_option=".*" if ($case_option eq "*"); 
		 
		if ($case_option=~/$case_value/i) {
		    $case_option_match=1;   
		}

		next;
            }
            
            ### If 'Case option end' and case option match already
            ### occured set flag to ignore remaining case block
            if ($line=~/;;/i){
		if ($case_option_match) {
			$case_option_match=0;
			$case_option_ignore=1;
		}
		next;
            }

            ### If 'Case block end' reset all case flags
            if ($line=~/^esac/i){
		$case_option_match=0;
		$case_block=0;
		$case_option_ignore=0;
		next;
            }
            
	    ### Skip case options that don't match case value
            next if ($case_block and not $case_option_match);

            
            ### If 'for loop' block gather values to loop through
            if ($line=~/^for /){
                my ($param,$values)=$line=~/(\S+) in (.*); do/;
                $for_param=$param;
                @for_values=split(' ',$values);    
            }
            
            
            ### If test assertion statement 
            if ($line=~/^expect|^\s+expect|^test_check/i){
                
                $type="assertion";
                
                ### If 'for loop' values substitute each value
                ### in assertion statement and save away in
                ### ordered hash.   
                if (@for_values) {
                        
                    foreach my $for_value (@for_values) {
                        my $copy_line=$line;
                        $copy_line=~s/\$\{$for_param\}/$for_value/g;
                        push @{$hsh{$for_value}},$copy_line;
                    }
                    
                    next;
                }
            
                ### If 'expect' statement separate expected results from
                ### command request.  If 'test check' function, construct
                ### shell command which echo's result back
                if ($line=~/^expect/){
                    my($exp0,$cmd0) = $line =~ /expect (\S+)\s+(.*)/;
                    $cmd="\${fstest} ".$cmd0;
                    $expect=$exp0;
                }
                elsif ($line=~/test_check/) {
                    my ($check,$n1,$oper,$n2) =
                        $line =~ /(\S+) (\S+) (\S+) (\S+)/;
                    $cmd="[ ".$n1." ".$oper." ".$n2. " ]; echo \$\?";
                    $expect=0;
                }
                
            }
            
            ### if 'for loop' is done, process ordered hash of
            ### commands.  
            elsif ($line=~/^done/i and @for_values)  {
                
                ### Initialize variables
                $type="assertion";
                my @assertions;
                
                ### Unset for-loop variables
                $for_param=undef;
                @for_values=(); 
                
                
                ### Store contents of ordered hash into
                ### an array 
                foreach my $key (keys %hsh) {
                    push @assertions,@{$hsh{$key}};
                }

                ### Iterate over assertions 
                foreach my $assertion (@assertions) {
                    
                    ### Initialize
                    $cmd="";
                    
                    ### If 'expect' statement separate expected results from
                    ### command request.  If 'test check' function, construct
                    ### shell command which echo's result back
                    if ($assertion=~/^expect/){
                        my($exp0,$cmd0) = $assertion=~ /expect (\S+)\s+(.*)/;
                        $cmd="\${fstest} ".$cmd0;
                        $expect=$exp0;
                    }
                    elsif ($assertion=~/test_check/) {
                        my ($check,$n1,$oper,$n2) =
                            $assertion =~ /(\S+) (\S+) (\S+) (\S+)/;
                        $cmd="[ ".$n1." ".$oper." ".$n2. " ]; echo \$\?";
                        $expect=0;
                    }
                    
                    ### Create command object
                    my $cmdObj =
                    Os::PTS::Cmd->_new(
                        on_err=>'rtn',
                        type=>$type,  # test,param,cmd
                        raw=>$_,
                        cmd=>$cmd,
                        conn=>$self->{conn},
                        expect=>$expect,
                        tcid=>($type=~/assertion/) ? $tcid++ : 0);
                    
                    die $cmdObj->errmsg() if ($cmdObj->error);    
                    
                    ### Store object in array
                    push @cmdObjs,$cmdObj;
                    
                }
                
                ### ReInitialize and re-tie 'for-loop' ordered hash
                %hsh=();
                tie %hsh,'Tie::IxHash';
                
                next;
            }
            
            ### If 'shell command' ...   
            elsif ($line=~/^cd |^dd |sleep|newfs|mount|umount\
                        |mdconfig|echo|\.\//i) {
                 
                $type="cmd";
                $cmd=$line;
                
            }
            ### If 'parameter initialization' ...
            elsif ($line=~/\=/ and not @for_values and not $while_loop) {
                
                $type="param";
                
                my ($param,$value)=$line=~/(\S+)=(.*)/; 
                
                if ($value=~/namegen/) {
                    (undef,$size)=$value=~/(\S+) (\d+)/;
                    $size=32 if (not defined $size);
                    $cmd=$self->_name_generate($param, $size);
                }
                else{
                   $cmd=$self->_set_param_echo($param,$value);
                }  
                
            }
            ### If 'while loop' ...
            elsif ($line=~/while /i){
                $while_loop=1;
                $cmd=$line." ";
                next;
            }    
            elsif ($while_loop) {
                if ($line=~/done/i) {
                    $cmd.=" ".$line;
                    $raw=$cmd;
                    $while_loop=0;
                    $type="cmd";
                }
                elsif ($line=~/^if/i) {
                    $cmd.=" ".$line;
                    next;
                }
                else {
                    $cmd.=" ".$line.";";
                    next;
                }
            }
            else {next;}
           
            my $cmdObj =
                Os::PTS::Cmd->_new(
                    on_err=>'rtn',
                    type=>$type,  # test,param,cmd
                    raw=>$raw,
                    cmd=>$cmd,
                    conn=>$self->{conn},
                    expect=>$expect,
                    tcid=>($type=~/assertion/) ? $tcid++ : 0
            );
            die $cmdObj->errmsg() if ($cmdObj->error);    

            push @cmdObjs,$cmdObj;
            $cmd="";
        }
    
    };
    if ($@){
        if (my $msg=$self->_input_validation($@)) {
            $errmsg=$msg; 
        }
        else {
            $errmsg=$@;
        }
        
        ($self->on_err()=~/die/)? die $errmsg: $self->_set_err($errmsg);
            
        return $self;
    }
    
    #unless (@cmdObjs){
#	die "No commands found in test file $self->{name}"; 
#    }

    return (wantarray()) ? @cmdObjs : [@cmdObjs];
}

###############################################################################
# Method:    _rm_files
# Objective: Copy fstest binary and misc.sh to fs dir under test
###############################################################################
sub _rm_files {
    my ($self)=@_;
    my (
        $cmd,
        $dir,
        $errmsg,
        );
   
    $dir=$self->{dir};
    
    eval {
        foreach my $file ("conf*","misc.sh*","fstest*"){
            $cmd = "rm -rf ".$dir."/"."$file";
            $self->_log(msg=>["Removing '$file' from '$dir'"]);
            $self->_execute(cmd=>$cmd);
        }
    };
    if ($@) {
        die $@;    
    }

    return 1;
      
}

################################################################################
# Method: _set_param_and_echo
# Objective: set param value in env and echo definition
################################################################################
sub _set_param_echo {
    my ($self,$param,$value)=@_;
    my (
        $cmd,
        );
    
    $cmd=$param."=".$value.
          ";echo \"$param=\"\$$param";
}

1;

############################################################################
#  P O D
############################################################################

=pod

=head2 B<on_err( I<$type> )>

Defines what error handling mechanism to use on ClientRPC instance

=over 4

=item Inputs

=over 4

=item * I<$type>

Indicates what errhandler type to use for all ClientRPC::Interface methods
calls.  If no errorhandle is provided, it returns the current handler type 

=begin html

<ul>=pod

=head1 Package Name

Os::PTS

=head1 Author

knopp@netapp.com

=head1 Description

Os::PTS provides an API interface to run one of the standard Posix Test Suites of file 
operations for verifying posix compliance.  Its a wrapper library around an existing 
3rd party suite of shell based posix test scripts which transforms their command 
content can be controlled and executed via NATE ntest.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error.pm>

=head1 Public Methods

=head2 B<new(%args)>

Creates a new OS::PTS object. 

=over 4

=item Inputs

=over 4

=item * B<file_op>

File operation to test.  Valid values are chflags chmod, chown, link, mkdir, 
mkfifo, open, rename, rmdir, symlink, truncate, or unlink

=item * B<hostname>

Machine to execute file operation tests from to determein posix compliance .

=item * B<loghdl> 

Log handle from test to use for logging messages to both STDOUT and NATE from 
library function  

=item * I<pts_loc> (default: /usr/software/test/agnostic/os/posix/pjd-fstest-20080816)

Posix Test Suite location

=item * I<dir> (default: /tmp)

Host name's filesystem directory to execute tests from. 

=item * I<err_on>  (default: die)   

Type of error handling to use for handling configuration errors. By default, 'die'  
will be used to throw errors and require the function caller to catch them using
an eval block.  Specifying err_on=>'rtn' will require the caller after the function 
returns to make a inquiry on the Os::PTS object whether an error occurred.

=item * I<user> (default: diag for ONTAP, root for Non-ONTAP)

User name to use to initiate a ssh connection to the host machine.  If ONTAP,  
sudo will be used to aquire root access after log in.

=item * I<pass> (default: netapp1! for ONTAP, tharnhost password for Non-ONTAP)

Password to use to initiate a ssh connection to the host machine.  

=back

=item Outputs

Returns a Os::PTS object.

=item Examples

 ### Create a PTS object
 use TCD;
 ...
 eval {
  	 $Test->nlog("- Create PTS object");
	 $PtsObj = Os::PTS->new(
		 hostname=>$node_name,
		 pts_loc=>$pts_location,
		 dir=>$dir_under_test,
		 file_op=>"open",
		 on_err=>"die",
		 loghdl=>$Test->get_loghdl());	
 };
 if ($@){
	 $Test->nlog($@);
	 return $TCD::FAIL;
 }

=back

=head2 B<cleanup()>

Removes any files copied to the host's file system directory under test.

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns 1 on success and 0 on failure.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
argument 'err_on' as defined during object instantiation.

=item Examples

 $PtsObj->cleanup();

=back

=head2 B<execute(cmdObjs)>

This is a helper function that executes and displays a standard output format 
for each command object executed.   NOTE: The function's capability can be 
easily reproduced within the test script if the user wishes to produce a different
output format. 

=over 4

=item Inputs

=over 4

=item * B<cmdObjs>

A hash reference to an array of Os::PTS::Cmd objects each containing commands to execute 
on the host file system under test.

=back

=item Outputs

Returns 1 on success and 0 on failure.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
argument 'err_on' as defined during object instantiation.

=item Examples

 eval {
	 ### Execute Testcase
	 $rtn=$rtn=$PsObj->execute(cmdObjs=>$cmdObjs);
 };
 if ($@) {
	 $Test->nlog($@);
	 return $TCD::FAIL;
 }

=back

=head2 B<get_arch()>

Returns the architecture of the host under test. 

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the host architecture, such as i386, amd64, or x86_64

=item Examples

 $arch=$PtsObj->get_arch();

=back 

=head2 B<get_hosttype()>

Returns the host type stored in the machine's tharnhost file 

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the tharnhost file host type;

=item Examples

 $arch=$PtsObj->get_hosttype();

=back 

=head2 B<get_os()>

Returns the operating system of the host under test 

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the operating system name

=item Examples

 $arch=$PtsObj->get_os();

=back 

=head2 B<load(alias, name)>

This function loads and parses an existing 3rd party posix test script that is written in shell 
and transforms it into a set of command objects.

=over 4

=item Inputs

=over 4

=item * B<alias>

A hash key identifying another test name that is more user friendly then the standard 
naming convention used by the posix test shell script.  For example, 'link_enotdir' rather
than 01.t.

=item * B<name>

Actual name of the posix test shell script

=back

=item Outputs

Returns an array or array reference of Os::PTS::Cmd objects encapsulating the commands of 
the posix test shell script.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
the argument 'err_on' as defined during object instantiation.

=item Examples

 eval {
 	 ###  Load and transform test script into set of cmd objects
	 $cmdObjs=$PsObj->load(alias=>"link_enotdir", name=>"01.t");
 };
 if ($@) {
	 $Test->nlog($@);
	 return $TCD::FAIL;
 }

=back

=head2 B<on_err( I<$type> )>

Defines what error handling mechanism to use on ClientRPC instance

=over 4

=item Inputs

=over 4

=item * I<$type>

Indicates what errhandler type to use for all ClientRPC::Interface methods
calls.  If no errorhandle is provided, it returns the current handler type 

=begin html

<ul>
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=back

=item Outputs

Returns error handle type  on success, throws error on failure

=item Examples

 # Set the defualt error handle from 'die' to 'rtn'   
 $hdlr=$PtsObj->on_err('rtn');
 $Test->nlog("Error handler changed to '".$hdlr."'");

=back

=head2 B<show_env()>

This is a helper function that displays the global parameter values that 
are defined for each host's operating system.  They are exported to the 
ssh environment and available for all tests to use.  NOTE: The function's 
capability can be easily reproduced within the test script if the user 
wishes to produce a different output format. 

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Displays the overall set of parameter values to STDOUT and the NATE log that 
are used by one of more tests in the suite.  

=item Examples


 ### Display global variables
 $PsObj->show_env();

=back

=head2 B<summary()>

This is a helper function that summarizes the number of posix test assertions 
executed along with which ones failed.  NOTE: The function's capability can be 
easily reproduced within the test script if the user wishes to produce a different
output format. 

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Displays the overall summary to STDOUT and the NATE log. 

=item Examples


 ### Display global variables
 $PsObj->show_env();

=back
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=back

=item Outputs

Returns error handle type  on success, throws error on failure

=item Examples

 # Example 1: Set the defualt error handle from 'die' to 'rtn'   
 $hdlr=$clientRpcObject->on_err('rtn');
 $test->nlog("Error handler changed to '".$hdlr."'");

=back
