# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Os/PTS/Cmd.pm#1 $
## @summary     Package to store commands, expected, and actual results
## @author      dl-striping-qa@netapp.com
## @status      private
## @pod here

package Os::PTS::Cmd;

use strict;
use Module::Common qw(validate_mixed);
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF UNDEF);
use Time::HiRes qw(gettimeofday);
use Tharn qw ($Log say);
use Data::Dumper;
use Error;
my %Param_Of;
our @ISA=qw(Error);

sub _new {
    my ($class,%args)=@_;
    my (
        $errmsg,
        $self,
    );
    
    ### Create instance of object
    $self = bless {}, $class;
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    eval {
        
        ### Validate input arguments
        ($class,%args) = validate_mixed (
            @_,
            [ { type => SCALAR} ],
            {
                type   => {type => SCALAR},
                raw    => {type => SCALAR},
                cmd    => {type => SCALAR},
                expect => {type => SCALAR, optional=>1, default=>"na"},
                on_err => {type => SCALAR, optional=>1, default=>"die",
                             callbacks=>{
                            'legal values'=>sub{shift()=~/die|rtn/i
                            }}},
                tcid    => {type => SCALAR, optional=>1, default=>"0"},    
                conn   => {type => OBJECT}, 
              }
        );
        
        ###  Lets specify what handler to use
        $self->{conn}=$args{conn};
        $self->{on_err}=$args{on_err};
        $self->{type}=$args{type},
        $self->{raw}=$args{raw};
        $self->{cmd}=$args{cmd};
        $self->{tcid}=$args{tcid};
        $self->{expect}=$args{expect};
        $self->{expect}=~s/\"//g;
        
    };
    if ($@){
        $Log->comment("ERROR: Cmd::_new() ".Dumper($@));
        if (my $msg=$self->_input_validation($@)) {
            $errmsg=$msg;
        }
        else {
            $errmsg=$@;
        }
        
        ($self->on_err()=~/die/)? die $errmsg: $self->_set_err($errmsg);
            
        return $self;
    }
   
    return $self;
}

###############################################################################
# Method:    execute
# Objective: Issue arbitrary command to host and get results.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $status,
        $t0,
        $t1,
        @outputs,
       );
    
    # Clear errors.
    $self->_clear_err;

    # Init execution timer
    $t0=gettimeofday;
    ($status, @outputs) = $self->_execute($self->{conn},$self->{cmd});
    $t1=gettimeofday;
    $self->{command_time}=$t1-$t0;

    $Log->comment("Outputs:".Dumper(@outputs));

    if ("@outputs"=~/\=/){
        my ($param,$value)="@outputs"=~/(\S+)=(.*)/;
        $Param_Of{$param}=$value;
        #$Log->comment("PARAMS\n".Dumper(%Param_Of));
    }

    ### Resolve expect value
    foreach my $name (keys %Param_Of) {
        $self->{expect} =~ s/\$\{$name\}/$Param_Of{$name}/i;
    }
    
    ### Store result in object as scalar
    if (scalar(@outputs)>1){
         $self->{result}=$outputs[scalar(@outputs)-1];
    }
    else {
        $self->{result}="@outputs";
    }
    
    $self->{status}=$status;
    
    # Check for standard set of errors returned from the CLI.
    if ($status) {
        ($self->on_err()=~/die/)
            ? die @outputs
            : $self->_set_err(@outputs);
        return (wantarray()) ? @outputs : \@outputs;
    }
    
    
    return (wantarray()) ? @outputs : [@outputs];

} # end sub cmd

sub get_cmd {
    my ($self)=@_;
    
    return $self->{cmd};
}

sub get_expect {
    my ($self)=@_;
    
    return $self->{expect};
}

sub get_raw {
    my ($self)=@_;
    
    return $self->{raw};
}

sub get_result {
    my ($self)=@_;
    
    return $self->{result};
}

sub get_tcid{
    my ($self)=@_;
    
    return $self->{tcid};
}

sub get_status {
    my ($self)=@_;
    
    return $self->{status};
}

sub get_type {
    my ($self)=@_;
    
    return $self->{type};
}

sub is_expect {
    my ($self)=@_;
    my (
        $expect,
        $result,
    );
    
    $expect=$self->get_expect();
    $result=$self->get_result();
    #$Log->comment("RESULT=|".$result."|");
    #$Log->comment("EXPECT=|".$expect."|");
    return ($result =~ /^$expect$/i) ? 1 : 0;
   
}

###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$type)=@_; 
    
    if (defined $type) {
        $self->{on_err}=$type;
    }
    
    return $self->{on_err}; 
}

sub _execute {
    my ($self,$connection, $cmd, $timeout) = @_;

    my ($env_output, $output, @output, $status);
    my ($match, $before, $after, $sentinel, $default_timeout);
    my ($cmdlog);

    $default_timeout = "600";
    $cmdlog = 1;
    @output = ();

    # set the default timeout if not specified
    $timeout = $default_timeout if (!defined($timeout) or $timeout =~ m/^\s*$/);

    # Adding 2>&1 to the end of the command string for all UNIX plaforms
   # $cmd = $cmd . " 2>/dev/null" if ($cmd !~ /sudo/);
    $cmd = $cmd . " 2>&1";
    # add to the command the echo of status to get that information from
    # the unix shell. Also add the sentinel string DoNe to command so that
    # we can match on that string.
    $sentinel = 'DoNe';
    $cmd = $cmd . ";echo \"rstatus=\$\?\" ;echo $sentinel";

    # libssh/libpersay do not print out normal headers letting you know
    # command being executed. Add these.
    my $hostname = $connection->{hostp}->hostname();
    my $conntype = $connection->conntype();
    
    # execute the passed command. Use just timeout since we check earlier
    # now if it was not passed and set the default if that is the case.
    ($match, $output, $after) =  &Tharn::say($connection, $cmd, $sentinel,
                                             '-timeout', $timeout, 
                                            # '-ignore', "echo $sentinel",
                                             '-comment', "Attempted '$cmd'");

    # strip output and remove CR/LF
    chomp($output);
    $output =~ s/\r//g;

    # remove first line 
    $cmd =~ s/"rstatus=\$\?" ;echo $sentinel//g;
    $output =~ s/"rstatus=\$\?" ;//g;

    # strip sentinel and then status from output
    $output =~ s/\n$sentinel.*//;

    # strip any telnet ctrl characters that get
    # added to the output (this problem seems 
    # to only occur with certain platforms).
    $output =~ s/ \cH//g;

    # locate status.  Remember that commands
    # echo themselves so make sure you don't 
    # get the command line 
    ($status) = $output =~ /rstatus=(\d+)/;

    # handle case where rstatus is not in the output
    # stream. One case is when NATE times out the command
    # In case of filer, do not do this since rstatus only
    # exists for clients.
    if (not defined $status and 
        $connection->{hostp}->{hosttype} =~ /unix|vmware/) {
        $status = 1;
    }

    $output =~ s/rstatus=\d+//;

    # create an array from the output
    @output = split(/\n/, $output);

    ## We need to determine if command is multiline 
    ## and exists in the output.  IMPORTANT: Not all
    ## commands echo back their input so don't forget to
    ## check before modifying the output.
    my @cmds = split(/\n/, $cmd);

    ## If output contains second line of cmd 
    ## then cmd was echoed to output and is 
    ## multiline
    if (defined $cmds[1] and length $cmds[1]) {
        ## Remove continuation character and parens (from group commands
        ## added recently)
        $cmds[1] =~ s/\\//g;
        $cmds[1] =~ s/\)//g;
        $cmds[1] =~ s/\(//g;
        if ($output[0] =~ /$cmds[1]/) {
          ## determine size of command and strip
          ## from there.
          @output = @output[@cmds-1..@output-1]; 

        }
    }

    ## Go through the output and remove any line that has the command in it
    ## We do this, because none of the function that call execute expect
    ## to have to filter out their cmd from the returned output.
    my @output_sorted;
    foreach my $line (@output) {
        my $match = 0;
        foreach my $cmd (@cmds) {
            if ($cmd eq $line) {
                $match = 1;
                last;
            }
        }

        if ($match == 0) {
            push (@output_sorted, $line);
        }
    }

    @output = @output_sorted;

    # case with dt that run w/aios flag - error status from dt is not reliable
    # add this case to catch/compensate
    $status = 1 if ($output =~ /Error number \d+ occurred on/i);
    
    return($status, @output);

}

###############################################################################
# Method:    _input_validation()
# Objective: Parse error message returned via rpc for method input errors
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _input_validation {
    my ($self,$err)=@_;
    my (
        $callback,
        $errmsg,
        $param,
        $value,
        );
    
    $Log->comment("Cmd input validation");
     
    if ($err=~/Mandatory parameter/i){
        ($param)=$err=~/Mandatory parameter (.*) /;
        $errmsg="Mandatory parameter ".$param;
    }
    elsif ($err=~/was not listed/i){
        ($param)=$err=~/options: (\S+)/;
        $errmsg="Invalid parameter ".$param;
    }
    elsif ($err=~/callback/i){
        ($param,$value,$callback)=
            $err=~/'(\S+)' parameter \(\"(\S+)\"\).*'(.*)' callback/i;
        $errmsg="Failed callback '".$callback. "' for ". $param."=>".$value;
    }
    elsif ($err=~/did not pass/i){
        ($param,$value)=$err=~/The '(\S+)' parameter \((\S+)\)/;
        $errmsg="Invalid value ".$param."=>".$value;
    }
    else {
        ### Error was not due to function input
        $errmsg="";
    }
    
    return $errmsg;
}

1;

############################################################################
#  P O D
############################################################################

=pod

=head1 Package Name

Os::PTS::Cmd

=head1 Author

knopp@netapp.com

=head1 Description

Os::PTS::Cmd provides an API for storing host commands for execution along with expected results.  

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error.pm>

=head1 Public Methods

=head2 B<execute()>

Executes the command stored in the object along with its result. 

=over 4

=item Inputs

=over 4

=item * B<cmdObjs>

A hash reference to an array of Os::PTS::Cmd objects each containing commands to execute 
on the host file system under test.

=back

=item Outputs

Returns 1 on success and 0 on failure.  If an error occured the error flag is set, errmsg
is set to the failure reason, and the error is returned or thrown depending on the setting of 
argument 'err_on' as defined during object instantiation.

=item Examples

 eval {
	 ### Execute command
	 $CmdObj->execute();
 };
 if ($@) {
	 $Test->nlog($@);
	 return $TCD::FAIL;
 }

=back

=head2 B<get_cmd()>

Returns the host command stored in the object 

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the command to execute on the host

=item Examples

 $cmd=$CmdObj->get_cmd();

=back 

=head2 B<get_expect()>

Returns the expected command result stored in the object

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the expected output when the command is executed

=item Examples

 $expect=$CmdObj->get_expect();

=back 

=head2 B<get_raw()>

Returns the raw command as read from an input file before parsed apart and stored
in a command object

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the raw command 

=item Examples

 $expect=$CmdObj->get_raw();

=back 

=head2 B<get_result()>

Returns the actual result output after executing the command on the host 

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the actual output when the command is executed

=item Examples

 $expect=$CmdObj->get_result();

=back 

=head2 B<get_type()>

Returns a tag identifying the type of command operation

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar containing the command type

=item Examples

 $type=$CmdObj->get_type();

=back 

=head2 B<is_expect()>

Returns whether the cmd result matches what was expected after execution

=over 4

=item Inputs

=over 4

None

=back

=item Outputs

Returns a scalar value containing 1 if the command execution output is valid, otherwise 0.

=item Examples

 $is_expect=$CmdObj->is_expect();

=back 

=head2 B<on_err( I<$type> )>

Defines what error handling mechanism to use on ClientRPC instance

=over 4

=item Inputs

=over 4

=item * I<$type>

Indicates what errhandler type to use for all ClientRPC::Interface methods
calls.  If no errorhandle is provided, it returns the current handler type 

=begin html

<ul>
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=back




