package RunningStat;

sub new() {
    my $this = shift;
    my $buckets = shift;
    die if !$buckets;
    my $self = {
        n => 0,
        oldM => 0,
        newM => 0,
        oldS => 0,
        newS => 0,
        buckets => $buckets,
        histogram => [],
    };
    bless $self, $this;
    return $self;
}

sub accumulate {
    my ($self, $x) = @_;
    $self->{n}++;
    if ($self->{n} == 1) {
        $self->{oldM} = $self->{newM} = $x;
        $self->{oldS} = 0;
        $self->{min} = $x;
        $self->{max} = $x;
    } else {
        $self->{newM} = $self->{oldM} + ($x - $self->{oldM}) / $self->{n};
        $self->{newS} = $self->{oldS} + ($x - $self->{oldM}) * ( $x - $self->{newM});
        $self->{oldM} = $self->{newM};
        $self->{oldS} = $self->{newS};
        $self->{min} = ($self->{min} < $x) ? $self->{min} : $x;
        $self->{max} = ($self->{max} > $x) ? $self->{max} : $x;
    }
    my $i = 0;
    while ($x > $self->{buckets}->[$i]) {
        $i++;
    }
    $self->{histogram}->[$i] += 1;
}

sub clear {
    my $self = shift;
    $self->{n} = 0;
}

sub count {
    my $self = shift;
    return $self->{n};
}

sub mean {
    my $self = shift;
    return ($self->{n} > 0) ? $self->{newM} : 0;
}

sub variance {
    my $self = shift;
    return ($self->{n} > 1) ? $self->{newS} / ($self->{n} - 1) : 0;
}

sub stddev {
    my $self = shift;
    return sqrt($self->variance());
}

sub histogram {
    my $self = shift;
    my @res;
    my $min = '0.00';
    for (my $i = 0; $i < @{$self->{histogram}}; $i++) {
        push @res, {'min' => $min, 
            'max' => $self->{buckets}->[$i],
            'count' => $self->{histogram}->[$i]};
        $min = $self->{buckets}->[$i];
    }
    return @res;
}

sub summary {
    my ($self, $fmt) = shift;
    my $res = "";
    my $show;
    for my $i ($self->histogram()) {
        if ($show ||= $i->{count}) {
            $res .= sprintf "%6.2f - %6.2f: %5d\n", $i->{min}, $i->{max}, $i->{count};
        }
    }
    $res .= "\n";
    $res .= sprintf "  Total Samples: %5d\n", $self->count();
    $res .= sprintf "          Range: %5.2f - %5.2f\n", $self->{min}, $self->{max};
    $res .= sprintf "        Average: %5.2f\n", $self->mean();
    $res .= sprintf "   Standard Dev: %5.2f\n\n", $self->stddev();
    return $res;
}

1;
