#Hey emacs, this is basically -*- perl -*-
#!/usr/local/bin/perl
# $Id$
## @attributes listtestcases=yes
## @status TBD
## @author kwhitney@netapp.com
####
#
## Copyright (c) 2001-2011 NetApp, Inc., All Rights Reserved
##   Any use, modification, or distribution is prohibited
##   without prior written consent from NetApp, Inc.
##
#

## @summary HA Task Module
## @author rajaram@netapp.com,dl-ha-qa@netapp.com
## @status shared
## @pod here

=head1 NAME

Os::osk_generic_node_init.pm

=head1 DESCRIPTION

This is a Perl class which encapsulates the most common NACL and APISet objects that OS/K QA people usually create for using in their tests.  This class is intended to either replace or mostly replace the INIT section of tests.

 # Note that $NodeName is just that: the host name of the filer node
 my $Node0 = new osk_generic_node_init( tcd => $Test, nodename => $NodeName)
 
 An option to set the APISet timeout value is "timeout => xxx".  The default is 600 seconds or 10 minutes. You
 can also set the 'verbose' option to '1' and that will print internal debugging information.
 
 The $Node0 variable is an “osk_generic_node_init” object which contains all the “usual” objects that people use in their testing:
 1)	Node object
 2)	ConnectRec object
 3)	HostRec object
 4)	APISet object in CLI mode
 5)	APISet object in SYSTEMSHELL mode
 6)	Transition object
 7)	Version object
 8)	Your working TCD object

These are the objects created for mosts tests.   I timed it, and you don’t suffer any real penalty for getting all the objects as opposed to a subset.

With this, you can do the same thing to multiple nodes, like transit them to CLI mode:

 Foreach $node (@Nodes) {
     $node->{_transitobj}->change_state( to => CLI );
 }
 There is some convenience data about each node:

        Nodename    :   csiqa-6070-39
        Hosttype    :   filer-ng-cfe
        Mode        :   CMode
        Timeout     :   6000
        Release     :   sierranevada__8.2.0
        Codename    :   sierranevada_0
        Model       :   FAS6070
        Build       :   /x/eng/rlse52/DOT/nightly/main_ib_bibN_120411_0040/
        Build Type  :   DEBUG
        Is Promoted :   0

=head2  NOTE: It takes only one line of code to do generic initialization (of a test)
for a cluster of *any* size

    my @Nodes  = osk_generic_node_init->get_all_nodes( tcd => $Test);

=head1 Examples of usage

Note that in order to access the contained objects, you must remember that a Perl
object is actually a hash.  So to access the contained objects, refer to them as a
hash member of the osk_generic_node_init object:

 To show the host type:  $Test->nlog("My filer is host type " . $Node0->{_hosttype});
 
=head2 Available objects and Properties (data fields, not objects)

=head2        OBJECTS

        _tcd                Your TCD object supplied by you
        _connectrec         A ConnectRec object used to talk to the Filer
        _hostrec            A HostRec object used for this node
        _transitobj         A Transit object initially used to bring filer to CLI state
        _versionmgr         A Version object
        _apiset_cmode       An APISet object in CMode for this Filer
        _apiset_systemshell An APISet object in SYSTEMSHELL mode for this Filer
        _node               A Node object for this filer

=head2        PROPERTIES

        _hosttype            As in "filer-ng"
        _mode                As in CMode, etc.        
        _verbose             Internal debug info gets printed 0/1
        _nodename            The name of the node that this object represents
        _timeout             Timeout value assigned to the APISet objects        
        _release             The text release name
        _codename            The text codename for the loaded release
        _model               The model name
        _build_path          The build path for loaded release
        _build_options       The build option info for the loaded release
        _is_promoted         Flag for debug/promoted
        _uname               If all else fails, this is the output of "uname -a"


=head2 EXAMPLE: Use the apiset_cmode object

    try {
        $response = $node->{_apiset_cmode}->vserver_cifs_share_show(
            'vserver'    => $vserver,
            'share-name' => $sharename,
        );
        $rc = 0;
    }
    catch ...

=head2 EXAMPLE: Display information about a node or test for proper OS version
 
     $Test->nlog(
        "The ONTAP release on $Node1->{_model} $Node1->{_nodename} is ",
        "$Node1->{_build_path} ($Node1->{_build_options}), $Node1->{_mode}"
    );
 Which looks like this:
 
         The ONTAP release on FAS6070 csiqa-6070-39 is
                /x/eng/rlse62/DOT/nightly/mainN_120509_1027/ (DEBUG), CMode
        The ONTAP release on FAS6070 csiqa-6070-40 is
                /x/eng/rlse62/DOT/nightly/mainN_120509_1027/ (DEBUG), CMode

    if ( $Node0->{_codename} =~ /ironcity/ ) {
        $Test->nlog(
            "This script doesn't support $Node0->{_nodename} that is running $Node0->{_release}"
        );

=head2 EXAMPLE: Initialize a test for a 2-Node cluster

Each $Node object is a generic init object and contains all of the utility objects
commonly used by the OS/Kernel QA group.

    my @Nodes
        = osk_generic_node_init->get_all_nodes( $Test, verbose => 0 );
        
    if ( scalar(@Nodes) != 2 ) {
        $Test->nlog(
            "Required number of NODES wrong.  Expected 2, got scalar(@Nodes)"
        );
        return $TCD::CONFIG;
    }

    $Node0 = $Nodes[0];
    $Node1 = $Nodes[1];

        
=head2 EXAMPLE: Create one volume for testing

    my $result = $Node0->create_volume("vs0", "test_vol01", "test_aggr01", "10G");
    
    The result is 0 for success, non-zero for fail.
    
=head2 EXAMPLE: Clean up the test volume you just made

At cleanup time, you want to get rid of the test volumes.  Sometimes you
don't care if the cleanup is successful.  In that case, you can ignore
the return code.  No exceptions are thrown.  Note that delete_volume uses
the "purge" API, so the volume will be unmounted, offline'd or whatever
is needed before it is deleted.

    my $rc = $Node0->delete_volume( "vs0", "test_vol01" );

=head2 EXAMPLE: Create a set of 4 volumes for testing

 sub create_volumes_for_test_path() {
    my ( $Node0, $Node1, $volsize ) = @_;

    my $size   = $volsize;
    my $result = 1;
    if ( $Node0->create_volume( $VS0, $Node0VOL0, $Node0AGGR, $size ) == 0 ) {
        if ( $Node0->create_volume( $VS0, $Node0VOL1, $Node0AGGR, $size ) == 0 ) {
            if ( $Node1->create_volume( $VS0, $Node1VOL0, $Node1AGGR, $size ) == 0 ) {
                if ($Node1->create_volume( $VS0, $Node1VOL1, $Node1AGGR, $size ) == 0 ) {
                    $result = 0;
                }
            }
        }
    }
    if ( $result == 1 ) {
        $Test->nlog("Cannot create volumes required for testing");
    }
    return $result;

 } ## end sub create_volumes_for_test_path()

=head1 DEBUGGING

The Verbose flag will turn on a lot of internal debugging messages.  The most useful
however, may be the command log.  The command log is a stack of the commands that
you sent to the filer using this object.  You can see each command set to the filer
and it's result (0 = Success) by using the "printme()" method.  The printme() method
will print extra information when Verbose = 1.  The extra information is the command
stack of cammands issued to the filer.

=head2 EXAMPLE: Command Stack display

This is the sequence of operations from node 6070-39 that help set up a
test environment including volume creation, mounting and initialization of
data on those volumes.  This is what printme() produces with Verbose=1:

 Print info on csiqa-6070-39
        Nodename    :   csiqa-6070-39
        Hosttype    :   filer-ng-cfe
        Mode        :   CMode
        Timeout     :   6000
        Release     :   sierranevada__8.2.0
        Codename    :   sierranevada_0
        Model       :   FAS6070
        Build       :   /x/eng/rlse62/DOT/nightly/mainN_120509_1027/
        Build Type  :   DEBUG
        Is Promoted :   0
        Command List
 "0: Create volume csiqa_6070_39_vol0 on vserver vs0 using aggregate csiqa_6070_39_aggr1 size 10737418240" = 0
 "1: Create volume csiqa_6070_39_vol1 on vserver vs0 using aggregate csiqa_6070_39_aggr1 size 10737418240" = 0
 "2: Mount volume csiqa_6070_39_vol0 on vserver vs0 at junction /csiqa_6070_39_vol0" = 0
 "3: Mount volume csiqa_6070_39_vol1 on vserver vs0 at junction /csiqa_6070_39_vol0/csiqa_6070_40_vol0/csiqa_6070_39_vol1" = 0
 "4: wget ftp://10.227.64.233/u/kwhitney/fops/fop_freebsd64" = 0
 "5: mv -f /var/home/diag/fop_freebsd64 /var/home/diag" = 0
 "6: chmod 777 /var/home/diag/fop_freebsd64" = 0
 "7: /var/home/diag/fop_freebsd64 -H 2 -W 2 -o c -s 4k -t 10 -n 2000  /clus/vs0/csiqa_6070_39_vol0 " = 0
 "8: /var/home/diag/fop_freebsd64 -H 2 -W 2 -o c -s 4k -t 10 -n 2000  /clus/vs0/csiqa_6070_39_vol0/csiqa_6070_40_vol0 " = 0
 "9: /var/home/diag/fop_freebsd64 -H 2 -W 2 -o c -s 4k -t 10 -n 2000  /clus/vs0/csiqa_6070_39_vol0/csiqa_6070_40_vol0/csiqa_6070_39_vol1 " = 0
 "10: /var/home/diag/fop_freebsd64 -H 2 -W 2 -o c -s 4k -t 10 -n 2000  /clus/vs0/csiqa_6070_39_vol0/csiqa_6070_40_vol0/csiqa_6070_39_vol1/csiqa_6070_40_vol1 " = 0
 
  
 
=head1 METHODS

=cut

package Os::osk_generic_node_init;

#Compiler Directives.
use strict;

## Import Modules
use Tharn qw($Log );
use TCD;
use NACL::APISet;
use Data::Dumper;
use Options;
use NACL::C::Node();
use NACL::C::Aggregate();
use NACL::Transit;
use diagnostics;
## Import NACL Execptions
use NACL::APISet::Exceptions::CommandFailedException qw(:try);
use NACL::Transit::Exceptions::TransitException qw(:try);
use NACL::Transit::Exceptions::RetriesExceeded qw(:try);
use NATE::BaseException qw(:try);
use TestUtils qw(get_site_info);

## Global Variables.
use vars qw(    );
use Carp;

=head2 new () Creates a new osk_generic_node_init object and all of the contained objects.

The minimum inputs are a TCD object (usually called $Test) and a node name.

   $MyNode0 = new osk_generic_node_init(tcd => $Test, nodename => "Fred" );

Failure returns an "undefined".
   
=cut
sub new {
    my ( $class, %args ) = @_;

    my $self = bless {
        _tcd                => $args{tcd}        || undef,
        _verbose            => $args{verbose}    || 0,
        _nodename           => $args{nodename}   || undef,
        _timeout            => $args{timeout}    || 600,
        _connectrec         => undef,
        _hostrec            => undef,
        _hosttype           => undef,
        _mode               => undef,
        _transitobj         => undef,
        _versionmgr         => undef,
        _apiset_cmode       => undef,
        _apiset_systemshell => undef,
        _node               => undef,
        _release            => "unknown",
        _codename           => "unknown",
        _model              => "unknown",
        _build_path         => "unknown",
        _build_options      => "unknown",
        _is_promoted        => "unknown",
        _cmdlist            => [],
        _rclist             => [],
        _uname              => "unknown",
        _warningpresented   => 0
    }, $class;
    my $result = $self->_init(%args);
    if ( $result > 0 ) {
        print("Cannot initialize NODE_INIT object");
        return undef;
    } else {
        return $self;
    }
} ## end sub new

sub reboot_and_wait {
    my ($self) = @_;
    my $rc = 0;

    my $result = $self->{_transitobj}->reboot();
    if ($result ne "CLI") {
        $rc = 1;
        $self->{_tcd}->nlog("REBOOTRESULT: ". Dumper($result));
    } 
    $self->refresh_connections();
    return $rc;
}

sub refresh_connections {
    my ($self) = @_;
    my $result = 0;
    
    eval {
        $self->{_node}->refresh_command_interface();
        $self->{_apiset_cmode}->api_reconnect_session();
        if (!$self->{_apiset_cmode}->api_is_session_alive()) {
            $self->_nlog("ERROR cannot refresh connection on CMODE apiset");
            $result = 1;
        }
        $self->{_apiset_systemshell}->api_reconnect_session();
        if (!$self->{_apiset_systemshell}->api_is_session_alive()) {
            $self->_nlog("ERROR cannot refresh connection on SYSTEMSHELL apiset");
            $result = 1;            
        }
    };
    if ($@) {
        $self->_nlog("ERROR refreshing connectiongs: " . $@);
        $result = 1;
    }
    return $result;
}
###############################################
# Routine: init                            #
# Description: Initialize NACL api object     #
#              Parse command line argument    #
#              Create host object             #
#              Create Transit Object          #
#              Transit test host to CLI if not#
#              Intialize VOL, VS, AGGR        #
#              Set transition timeout         #
# Argument: no                                #
# Return: CONFIG, PASS                        #
###############################################

sub _nlog {
    my ( $self, $msg ) = @_;
    if ( $self->{_verbose} ) {
        $self->{_tcd}->nlog( _timestamp() . " " . $msg );
    }
} ## end sub _nlog

sub _timestamp {

    my ( $sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst )
        = localtime(time);
    my $format = "%04d%02d%02d %02d%02d%02d";
    my $timestamp
        = sprintf( $format, $year + 1900, $mon + 1, $mday, $hour, $min,
        $sec );
    return $timestamp;

} ## end sub _timestamp
###########################################################
# Return an array of node_init objects- one for each node
###########################################################

=head2 get_all_nodes

Return an array of osk_generic_node_init objects, one for each node
in the cluster.
 
 Similar to "new()" except you don't specify any node names.

 Example usage:
 
    my @Nodes = osk_generic_node_init->get_all_nodes( $Test, verbose => 0 );
        
    if ( @Nodes != 2 ) {
        $Test->nlog(
            "Required number of NODES wrong.  Expected 2, got scalar(@Nodes)"
        );
        return $TCD::CONFIG;
    }

    $Node0 = $Nodes[0];
    $Node1 = $Nodes[1];

=cut

sub get_all_nodes {
    my ( $self, $tcd, %opts ) = @_;
    my @nodes;
    my @node_inits;

    if ( !defined($tcd) ) {
        return undef;
    }
    @nodes = NACL::C::Node->find();
    foreach my $node (@nodes) {
        push(
            @node_inits, Os::osk_generic_node_init ->new(
                tcd      => $tcd,
                nodename => $node->node(),
                %opts
                )
        );
    }
    return (@node_inits);
} ## end sub get_all_nodes

sub _init {

    my ( $self, %args ) = @_;

    if ( !defined( $self->{_tcd} ) ) {
        return 1;
    }
    my $Test = $self->{_tcd};

    # Verify required parameters are defined
    # Get the Filer name or Vsim name

    if ( !defined( $self->{_nodename} ) ) {
        $self->{_tcd}->nlog("\"nodename\" is a required parameter");
        return 1;
    }

    $self->_nlog("NODE is $self->{_nodename}. ");

    # Sometimes "hosttype" parameter is appended to hostname
    # Clean it up.
    if ( $self->{_nodename} =~ /hosttype/ ) {
        my @nodeinfo = split( /;/, $self->{_nodename} );
        $self->{_nodename} = $nodeinfo[0];
    }

    # get the host object handler
    $self->_nlog("Creating Hostrec object");
    $self->{_hostrec} = Hostrec->new( id => $self->{_nodename} );

    # get hosttype
    $self->{_hosttype} = $self->{_hostrec}->hosttype();
    $self->{_model}    = $self->{_hostrec}->{'model'};
    $self->_nlog(
        "host type from NACL is $self->{_model} / $self->{_hosttype}");

    $self->_nlog("Creating Node object");
    $self->{_node} = NACL::C::Node->new( hostrec => $self->{_hostrec} );

    ## Set the mode flag
    $self->{_mode} = $self->{_node}->mode();

    # get console connection handle
    eval {
        $self->_nlog("Creating the console object for $self->{_nodename}");
        $self->{_connectrec} = Connectrec->new(
            host     => $self->{_hostrec},
            template => "ssh"
        );
    };
    if ($@) {
        $Test->nlog(
            "Cannot connect to the console of $self->{_nodename}:\n\t" . $@ );
        return 1;
    }
    ## Create the NACL::Transit object and transit to
    ## the CLI if the controller is not already there
    $self->_nlog("Creating Transit object");
    $self->{_transitobj} = NACL::Transit->new( name => $self->{_nodename} );
    my $Original_state = $self->{_transitobj}->get_state();
    $self->_nlog("The $self->{_nodename} is at: $Original_state state");

    ## Transit to CLI if the controller is not already there
    ## prior to creating the APISet object
    my $ret = $self->go_to_cli();
    if ( $ret == 1 ) {
        $Test->nlog("Cannot transit to CLI on console of $self->{_nodename}");
        return 1;
    }

    ## Create the NACL objects based on the hosttype/mode
    if (   $self->{_hosttype} eq "filer-ng-cfe"
        || $self->{_hosttype} eq "filer-ng-vsim"
        || $self->{_hosttype} eq "filer-ng-vsa" ) {
        $self->_nlog(
            "This is a CMode controller with host type $self->{_hosttype}");

        ## Create the APISet object
        $self->_nlog("Creating the CMode APISet object");
        $self->{_apiset_cmode} = NACL::APISet->new(
            connection => $self->{_connectrec},
            category   => "Node",
            interface  => "CLI",
            set        => "CMode"
        );
        $self->{_apiset_cmode}
            ->set_timeout( 'connectrec-timeout' => $self->{_timeout} );
        $self->_nlog( "APISet connect timeout value is "
                . $self->{_apiset_cmode}->get_timeout() );

        ## Create the CMode Version object
        $self->_nlog("Creating the CMode Version object");
        $self->{_versionmanager}
            = $self->{_apiset_cmode}->get_version_manager();
        $self->{_build_path} = $self->{_versionmanager}
            ->get_version_attribute( attribute => 'build_path' );
        $self->{_build_options} = $self->{_versionmanager}
            ->get_version_attribute( attribute => 'build_options' );
        $self->{_release} = $self->{_versionmanager}
            ->get_version_attribute( attribute => 'release' );
        $self->{_is_promoted} = $self->{_versionmanager}
            ->get_version_attribute( attribute => 'is_promoted' );
        ## Get code name from release name
        $self->{_codename} = $self->{_versionmanager}
            ->map_release_to_codename( release => $self->{_release} );

        $self->_nlog("Creating the SYSTEMSHELL APISet object");
        $self->{_apiset_systemshell} = NACL::APISet->new(
            hostobj   => $self->{_hostrec},
            category  => "Node",
            interface => "CLI",
            set       => "Systemshell"
        );
        my $output = $self->{_apiset_systemshell}->execute_raw_command(
            'command'            => 'sudo csh',
            'connectrec-timeout' => $self->{_timeout},
        );
    } else {
        $Test->nlog("ERROR: This script only support C-mode filer");
        return 1;
    }

    ## Verify that the filer has a supported release
    $self->_nlog(
        "The ONTAP release on $self->{_model} $self->{_nodename} is $self->{_build_path} ($self->{_build_options}), $self->{_mode}"
    );

    #unless ($Code_name =~ /rollingrock/ || $Code_name =~ /boilermaker/) {
    if ( $self->{_codename} =~ /ironcity/ ) {
        $Test->nlog(
            "This script doesn't support $self->{_nodename} that is running $self->{_release}"
        );
        return 1;
    }
    return 0;
}    # end of init

#################################################
# Routine: uninit                            #
# Description: Ininialize NACL Api object       #
# Argument: No                                  #
# Return: Pass                                  #
#################################################
sub uninit {
    my ($self) = @_;
    ## Destroy the APISet object
    $self->{_tcd}->nlog("Uninitialize Objects");
    return $self->_uninit();
} ## end sub uninit

#################################################
# Routine: _uninit                            #
# Description: Ininialize NACL Api object       #
# Argument: No                                  #
# Return: Pass                                  #
#################################################
sub _uninit {
    my ($self) = @_;
    ## Destroy the APISet object
    $self->{_nodename}   = undef;
    $self->{_connectrec} = undef;
    $self->{_hostrec}    = undef;
    $self->{_hosttype}   = undef;
    $self->{_mode}       = undef;
    $self->{_timeout}    = undef;
    $self->{_transitobj} = undef;

    if ( defined( $self->{_apiset_cmode} ) ) {
        $self->{_apiset_cmode}->cleanup_APISet();
        $self->{_apiset_cmode} = undef;
    }
    if ( defined( $self->{_apiset_systemshell} ) ) {
        $self->{_apiset_systemshell}->cleanup_APISet();
        $self->{_apiset_systemshell} = undef;
    }
    $self->{_versionmgr}    = undef;
    $self->{_tcd}           = undef;
    $self->{_release}       = undef;
    $self->{_codename}      = undef;
    $self->{_model}         = undef;
    $self->{_build_path}    = undef;
    $self->{_build_options} = undef;
    return 0;
} ## end sub _uninit

##########################################################

=head2  go_to_cli

 change filer from current state to cli   
 
 Args: No                                              
 Return: 0 for success 1 for fail
 
=cut

##########################################################
sub go_to_cli {
    my ($self) = @_;
    my $temp_state;
    my $resp;
    my $ret;
    my $cmd;

    #    $self->_nlog("Transit filer to CLI state if needed");
    ## Get current filer state
    $temp_state = $self->{_transitobj}->get_state();
    if ( $temp_state ne 'CLI' ) {
        $self->_nlog("Filer is not at CLI state. will transit to CLI");
        try {
            $self->{_transitobj}->change_state( to => "CLI" );
        }
        catch NACL::Transit::Exceptions::TransitException with {
            my $exception_object = shift;
            $self->{_tcd}->nlog( "Exception: " . ref($exception_object) );
            $self->{_tcd}->nlog( $exception_object->text() );
            $self->{_tcd}->nlog("ERROR : NACL::Transit failed");
            return 1;
        };
    }

    #    $self->_nlog("Filer is at CLI state");
    return 0;
} ## end sub go_to_cli


###########################################################

=head2   create_volume

 create a volume
 
 Args: $vserver, $volname, $aggr, $size
  
 $size defaults to 10G
 
 Return: 0 for success 1 for fail
 
=cut

###########################################################
sub create_volume {
    my ( $self, $vserver, $volname, $aggr, $size ) = @_;
    my $rc = 0;

    # Handle error-checking here
    if ( !defined($vserver) ) {
        $self->{_tcd}->nlog("Cannot create volume.  Vserver undefined.");
        return 1;
    }
    if ( !defined($volname) ) {
        $self->{_tcd}->nlog("Cannot create volume.  Volume name undefined.");
        return 1;
    }
    if ( !defined($aggr) ) {
        my $aggrname = $self->{_nodename} . "_aggr1";
        $self->{_tcd}
            ->nlog("Aggregate name undefined.  Defaulting to $aggrname");
        $aggr = $aggrname;
        $aggr =~ s/-/_/g;
    }
    if ( !defined($size) ) {
        $size = "10G";
        $self->{_tcd}
            ->nlog("Volume size not undefined.  Defaulting to $size");
    }

    ## Create the volume
    try {
        my $volume = NACL::STask::Volume->create(
            command_interface => $self->{_node},
            volume            => $volname,
            vserver           => $vserver,
            aggregate         => $aggr,
            size              => $size
        );
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $self->{_tcd}->nlog( "Exception: " . ref($exception_object) );
        $self->{_tcd}->nlog($exception_object);
        $self->{_tcd}->nlog("Failed to create volume $volname.");
        $rc = 1;
    };

    # Save for debugging
    if ( $self->{_verbose} ) {
        my $cmd
            = "Create volume $volname on vserver $vserver using aggregate $aggr size $size";
        push( @{ $self->{_cmdlist} }, $cmd );
        push( @{ $self->{_rclist} },  $rc );
    }
    return $rc;
} ## end sub create_volume

###########################################################
=head2 mount_volume

mount a junction volume     

 Args: $vserver, $volname, $junction_path

 Return: 0 for success 1 for fail
 
=cut
###########################################################
sub mount_volume {
    my ( $self, $vserver, $volname, $jpath ) = @_;
    my $rc = 0;

    # Handle error-checking here
    if ( !defined($vserver) ) {
        $self->{_tcd}->nlog("Cannot mount volume.  Vserver undefined.");
        return 1;
    }
    if ( !defined($volname) ) {
        $self->{_tcd}->nlog("Cannot mount volume.  Volume name undefined.");
        return 1;
    }
    if ( !defined($jpath) ) {
        $self->{_tcd}->nlog("Cannot mount volume.  Junction path undefined.");
        return 1;
    }

    try {
        NACL::STask::Volume->mount(
            command_interface => $self->{_node},
            volume            => $volname,
            vserver           => $vserver,
            'junction-path'   => $jpath,
        );
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $self->{_tcd}->nlog( "Exception: " . ref($exception_object) );
        $self->{_tcd}->nlog($exception_object);
        $self->{_tcd}->nlog("Failed to mount volume $volname");
        $rc = 1;
    };

    # Save for debugging
    if ( $self->{_verbose} ) {
        my $cmd
            = "Mount volume $volname on vserver $vserver at junction $jpath";
        push( @{ $self->{_cmdlist} }, $cmd );
        push( @{ $self->{_rclist} },  $rc );
    }
    return $rc;
} ## end sub mount_volume

###########################################################
=head2 unmount_volume                               

 unmount a volume
 
 Args: $vserver, $volname, $force
 
 $force is "true"/"false" defaults to "false"
 
 Return: 0 for success 1 for fail                       

=cut
###########################################################
sub unmount_volume {
    my ( $self, $vserver, $volname, $force ) = @_;
    my ( $rc, $response );

    # Handle error-checking here
    if ( !defined($vserver) ) {
        $self->{_tcd}->nlog("Cannot unmount volume.  Vserver undefined.");
        return 1;
    }
    if ( !defined($volname) ) {
        $self->{_tcd}->nlog("Cannot unmount volume.  Volume name undefined.");
        return 1;
    }
    if ( !defined($force) ) {
        $force = "false";
    }

    ## Make sure that the filer is at CLI prompt
    $rc = $self->go_to_cli();
    if ( $rc == 0 ) {
        try {
            $response = $self->{_apiset_cmode}->volume_unmount(
                vserver => $vserver,
                volume  => $volname,
                force   => $force
            );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $self->{_tcd}->nlog( "Exception: " . ref($exception_object) );
            $self->{_tcd}->nlog( $exception_object->text() );
            $response = $exception_object->get_response_object();
            $rc       = $response->get_execution_status();
            $self->{_tcd}
                ->nlog("Failed to unmount volume $volname: $response");
        };
    }

    else {
        $self->{_tcd}->nlog("Failed to transfer filer to CLI state");
    }

    # Save for debugging
    if ( $self->{_verbose} ) {
        my $cmd = $response->get_command();
        push( @{ $self->{_cmdlist} }, $cmd );
        push( @{ $self->{_rclist} },  $rc );
    }
    return $rc;
} ## end sub unmount_volume
###########################################################
=head2 offline_volume

 offline a volume   

 Args: $vserver, $volname

 Return: 0 for success 1 for fail               

=cut
###########################################################
sub offline_volume {
    my ( $self, $vserver, $volname ) = @_;
    my ( $rc, $response );

    # Handle error-checking here
    if ( !defined($vserver) ) {
        $self->{_tcd}->nlog("Cannot offline volume.  Vserver undefined.");
        return 1;
    }
    if ( !defined($volname) ) {
        $self->{_tcd}->nlog("Cannot offline volume.  Volume name undefined.");
        return 1;
    }

    ## Make sure that the filer is at CLI prompt
    $rc = $self->go_to_cli();
    unless ( $rc == 1 ) {
        try {
            $response = $self->{_apiset_cmode}->volume_offline(
                vserver => $vserver,
                volume  => $volname
            );
        }
        catch NACL::APISet::Exceptions::ResponseException with {
            my $exception_object = shift;
            $self->{_tcd}->nlog( "Exception: " . ref($exception_object) );
            $self->{_tcd}->nlog( $exception_object->text() );
            $response = $exception_object->get_response_object();
            $rc       = $response->get_execution_status();
            $self->{_tcd}
                ->nlog("Failed to offline volume $volname: $response");
        }
    }

    else {
        $self->{_tcd}->nlog("Failed to transfer filer to CLI state");
    }

    # Save for debugging
    if ( $self->{_verbose} ) {
        my $cmd = $response->get_command();
        push( @{ $self->{_cmdlist} }, $cmd );
        push( @{ $self->{_rclist} },  $rc );
    }
    return $rc;
} ## end sub offline_volume
###########################################################
=head2 delete_volume

 delete (purge) a volume                 

Args: $vserver, $volname

 Return: 0 for success 1 for fail
 
=cut
###########################################################
sub delete_volume {
    my ( $self, $vserver, $volname ) = @_;
    my $rc = 0;

    # Handle error-checking here
    if ( !defined($vserver) ) {
        $self->{_tcd}->nlog("Cannot delete volume.  Vserver undefined.");
        return 1;
    }
    if ( !defined($volname) ) {
        $self->{_tcd}->nlog("Cannot delete volume.  Volume name undefined.");
        return 1;
    }
    try {
        my $volume = NACL::STask::Volume->purge(
            command_interface => $self->{_node},
            vserver           => $vserver,
            volume            => $volname
        );
    }

    # NOTE: "purge()" captures all the 'normal' exceptions
    #        This must be something that doesn't belong
    catch NATE::BaseException with {
        my $exception_object = shift;
        $self->{_tcd}->nlog( "\tException: " . ref($exception_object) );

        #        $self->{_tcd}->nlog($exception_object);

        #	  $response = $exception_object->get_response_object();#
        #	  $rc       = $response->get_execution_status();
        $self->{_tcd}->nlog("\tFailed to delete volume $volname");
        $rc = 1;
    };

    # Save for debugging
    if ( $self->{_verbose} ) {
        my $cmd = "Purge volume $volname";
        push( @{ $self->{_cmdlist} }, $cmd );
        push( @{ $self->{_rclist} },  $rc );
    }
    return $rc;
} ## end sub delete_volume

=head2 printme

Debug helper that prints the most useful information about this object.

 Example Output:
        Nodename    :   csiqa-6070-39
        Hosttype    :   filer-ng-cfe
        Mode        :   CMode
        Timeout     :   6000
        Release     :   sierranevada__8.2.0
        Codename    :   sierranevada_0
        Model       :   FAS6070
        Build       :   /x/eng/rlse52/DOT/nightly/main_ib_bibN_120411_0040/
        Build Type  :   DEBUG
        Is Promoted :   0
        
=cut

sub printme {
    my ( $self, $msg ) = @_;
    ## Destroy the APISet object
    if ( defined($msg) ) {
        $self->{_tcd}->nlog($msg);
    }
    $self->{_tcd}->nlog("Print info on $self->{_nodename}");

    $self->{_tcd}->nlog( "\tNodename    :\t" . $self->{_nodename} );
    $self->{_tcd}->nlog( "\tHosttype    :\t" . $self->{_hosttype} );
    $self->{_tcd}->nlog( "\tMode        :\t" . $self->{_mode} );
    $self->{_tcd}->nlog( "\tTimeout     :\t" . $self->{_timeout} );
    $self->{_tcd}->nlog( "\tRelease     :\t" . $self->{_release} );
    $self->{_tcd}->nlog( "\tCodename    :\t" . $self->{_codename} );
    $self->{_tcd}->nlog( "\tModel       :\t" . $self->{_model} );
    $self->{_tcd}->nlog( "\tBuild       :\t" . $self->{_build_path} );
    $self->{_tcd}->nlog( "\tBuild Type  :\t" . $self->{_build_options} );
    $self->{_tcd}->nlog( "\tIs Promoted :\t" . $self->{_is_promoted} );

    if ( $self->{_verbose} ) {
        $self->{_tcd}->nlog("\tCommand List");
        my $i           = 0;
        my @commandlist = @{ $self->{_cmdlist} };
        my @rclist      = @{ $self->{_rclist} };
        for ( $i = 0; $i < scalar(@rclist); $i++ ) {
            $self->{_tcd}->nlog( "\t\"" 
                    . $i . ": "
                    . $commandlist[$i] . "\" = "
                    . $rclist[$i] );
        }
    }
    return 0;
} ## end sub printme

=head2 ngsh

Execute an NGSH command.  Returns a list containing the return code
from the operation and the text output from the command.
 
  Arg : $command
  
 Example usage:
 my ($return_code, $output) = $Node0->ngsh("volume show");

=cut

sub ngsh {
    my ( $self, $command ) = @_;

    return $self->systemshell( "ngsh -c " . $command );
}

=head2 systemshell

Execute a FreeBSD shell command.  Returns a list containing the return code
from the operation and the text output from the command.
 
 Example usage:
 my ($return_code, $output) = $Node0->systemshell("ls -l /clus/vs0");

=cut

sub systemshell {
    my ( $self, $command ) = @_;
    my $result;
    my $output;
    try {

        $self->_nlog( "\n\tSYSTEMSHELL Executing ::$command " . "\n" );

        $output = $self->{_apiset_systemshell}->execute_raw_command(
            'command'            => $command,
            'connectrec-timeout' => $self->{_timeout},
        );

        $result = $self->{_apiset_systemshell}->execute_raw_command(
            'command'            => 'echo $?',
            'connectrec-timeout' => $self->{_timeout},
        );
        ## Print the C program's output to standard out
        ## only if Verbose is set to 1
        $self->_nlog( "Output from program:: $command is "
                . " \"$output\". RC=$result" );

        if ( ( scalar($result) + 0 ) > 0 ) {
            $self->_nlog( "SystemShell Error: "
                    . "\n\tCMD=$command\n\tRC=$result\n\tOutput:\n$output" );
        }
    }
    catch NACL::APISet::Exceptions::CommandFailedException with {
        my $exception_object = shift;
        $self->{_tcd}
            ->nlog( "Exception is of type: " . ref($exception_object) );
        $self->{_tcd}->nlog( "Error sending command ($command):  "
                . $exception_object->text() );
        $result = 255;
    };

    # Save for debugging
    if ( $self->{_verbose} ) {
        push( @{ $self->{_cmdlist} }, $command );
        push( @{ $self->{_rclist} },  $result );
    }
    return ( $result, $output );
} ## end sub systemshell

###########################################################

=head2 execute_list_of_commands     

Execute a series of FreeBSD shell commands

 Args : @list_of _commands_2_execute                    
        (Array of command lines                         
 Returns : Sum of all result codes and concatenation of output from all commands
 
 Example usage:
    my $program             = "fop_freebsd64";
    my $basepath            = "/clus/vs0";
    my $executable          = $destinationlocation . "/" . $program;
    my $options             = "-H 2 -W 2 -o c -s 4k -t 10 -n 2000 ";
    my @cmds                = [
        "$executable $options $basepath$JPATH1 ",
        "$executable $options $basepath$JPATH2 ",
        "$executable $options $basepath$JPATH3 ",
        "$executable $options $basepath$JPATH4 ",
    ];

    $Test->nlog("Executing commands on filer ");
   ( $rc, $response ) = $node->execute_list_of_commands(@cmds);
   if ( $rc > 0 ) {
        $Test->nlog("ERROR...");
   }
   
=cut 
###########################################################

sub execute_list_of_commands {
    my ( $self, $commands2execute ) = @_;
    my $output       = "unknown";
    my $total_output = "";
    my $result_code  = 0;
    my $tmp_result   = 0;
    my $rc;

#	$Test->nlog("DUMPER: " . Dumper($commands2execute) . ": " . ref($commands2execute));
    foreach my $cmd (@$commands2execute) {
        if ( defined($cmd) == 1 ) {
            ( $rc, $output ) = $self->systemshell($cmd);
            $result_code += $rc;
            $total_output = $total_output . $output;
        }
    }
    return ( $result_code, $total_output );

}    # end _start_test_program()
###########################################################

=head2 install_test_program     

 Install a test program on a Filer.  This would probably be a binary or shell test program.
 
 Args : $sourcelocation
            complete path to test program (without program name)
        $destinationlocation
            complete path (location) on filer FreeBSD file system where program should reside
        $program
            file name for test program
 
 Example usage:
    my $sourcelocation      = "/u/kwhitney/fops";
    my $destinationlocation = "/var/home/diag";
    my $program             = "fop_freebsd64";
 
    # Move FOP executable to filer
    $Test->nlog("Installing test program ($program) on filer");
    $rc = $node->install_test_program( $sourcelocation, $destinationlocation, $program );
 
  Returns : 0 if successful else non-zero

=cut
###########################################################
sub install_test_program {

    ## Initialize local variables
    my ( $self, $sourcelocation, $destinationlocation, $program ) = @_;
    my $cmd          = undef;
    my $result_code  = 0;
    my $output       = "";
    my $total_output = "";
    our %Siteinfo = ();
    my @cmds;
    my $default_site_info_file = "TestUtils/site_info.dat";

    my $site = $self->{_hostrec}->SITE();
    ## Fetch the ftp server from the site_info
    ## file

    eval {
        %Siteinfo = TestUtils::get_site_info( host => $self->{_hostrec} );
    };
    if ($@) {
        if ( ! $self->{_warningpresented}) {
            $self->{_tcd}->nlog(
                "WARN: Unable to read the contents of the site info file for site $site \n "
                    . $@ );
            $self->{_tcd}->nlog("\tSITE INFO for site $site:  not available.");
            $self->{_tcd}->nlog("\t#####################################################");
            $self->{_tcd}->nlog("\t#### WARNING         ################################");
            $self->{_tcd}->nlog("\t#### Defaulting to $default_site_info_file ##########");
            $self->{_tcd}->nlog("\t#####################################################");
            $self->{_warningpresented} = 1;
        }
        my $code = NATE::Inc::get_file($default_site_info_file);

        # The following 'eval' creates variable %Siteinfo
        eval $code;
    }

    ## Read the download server from the site info file
    my $download_server = %Siteinfo->{$site}->{'netboot_server'};

    if (   defined($download_server)
        && defined($sourcelocation)
        && defined($destinationlocation) ) {

        $download_server = 'ftp://' . $download_server;
        $self->_nlog( "- Installing test program : ",
            " $sourcelocation/$program to $destinationlocation" );
        @cmds = [
            "wget " . $download_server . $sourcelocation . "/" . $program,
            "mv -f /var/home/diag/$program $destinationlocation",
            "chmod 777 $destinationlocation/" . $program
        ];

        ## Install the test program under $destinationlocation
        ( $result_code, $total_output )
            = $self->execute_list_of_commands(@cmds);
        $self->_nlog("INSTALL RESULT: RC=$result_code OUTPUT=$total_output");

    } else {
        $self->{_tcd}->nlog(
            "Cannot download source($sourcelocation) to dest($destinationlocation) ",
            "using download server($download_server)"
        );
        $result_code = 1;
    }
    return $result_code;
}    # end _install_test_program()

# UNDEF all object references
sub DESTROY {
    my ($self) = @_;
    $self->_uninit();
}

1;
