# $Id: //depot/prod/test/Rsierranevada/lib/PExec.pm#1 $
# $Author: build $
## @author: nsesha@netapp.com [ dl-raid-qa ]

use strict;
package PExec;
#==========================================
#       Library Includes
#==========================================
# TODO: Uncomment these tests
use Stest::Block;
use ConfigMaker;

use Clone qw(clone);
use Data::Dumper;
use Params::Validate qw(:all);
use XMLDotPlan;
use XML::XPath;

our @ISA = qw(Utility);

my $Package = "PExec";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    
    my %opts = validate(
        @_, {
            xmlfile         => { type => SCALAR, optional => 0},
            filter          => { type => SCALAR | UNDEF, optional => 1},
            params_filter   => { type => SCALAR | UNDEF, optional => 1 },            
            debug_level     => { type => SCALAR, optional => 1, default => 0},
        }
    );
    my $self = $class->SUPER::new(debug_level=>$opts{debug_level});    
    %opts = (%{$self}, %opts);
    $self = \%opts;
    bless $self, $class;
    
    $self->set_xmlfile(xmlfile=>$opts{xmlfile});
    $self->set_filter(filter=>$opts{filter});
    $self->set_params_filter(params_filter=>$opts{params_filter});
    $self->set_debug_level(debug_level=>$opts{debug_level});
    
    $self->log(1, "$Package: Info: Initializing PExec...");
    $self->init();
    
    return $self;
} 

sub init{
    my $self = shift;
    
    $self->log(1,"$Package: Info: Validating XML file for well-formedness");
    if(!$self->is_valid_xml()){
        $self->log(1,"$Package: Info: Invalid xmlplan file, may be this is a dotplan ?");
        my $dotplan = $self->read_file(filename=>$self->get_xmlfile());
        $self->set_dotplan(dotplan=>$dotplan);
    } else {
        $self->log(1,"$Package: Info: Translating xmlplan to dotplan");
        my $xmlPlan = XMLDotPlan->new(xmlfile=> $self->get_xmlfile(),    
            filter => $self->get_filter(),
            debug_level => $self->get_debug_level(),
        );
        
        $xmlPlan->printXML(nodexml=>$xmlPlan->get_xmlplan(original=>0));    # TODO: Comment this if not required.
        $self->set_xmlplan(xmlplan=>$xmlPlan);    
    }
    # TODO: Uncomment this
    $self->{configMaker} = ConfigMaker->new(debug_level=>$self->get_debug_level());
}

sub is_valid_xml{
    my $self = shift;
    eval{
        use XML::Simple;
        XMLin($self->get_xmlfile());
    };
    if($@){
        $self->log(1, "$Package: Error: Not a well-formed xml - ".$self->get_xmlfile()."\n$@");
        return 0;
    }
    return 1;
}

sub static_parallelism{
    my $self = shift;
    my %opts = validate(
        @_, {
            num_of_instance => { type=>SCALAR, optional=>1, default=>2},
            dotplan_based   => { type=>SCALAR, optional=>1, default=>0}
        }
    );

    if($opts{dotplan_based}){
        
        my $codespec_arr = $self->get_codespec(dotplan=>$self->get_dotplan());
        
        print "CODESPEC===>\n",Dumper($codespec_arr),"\n";
        my $blockspec_arr = $self->get_static_blockspec(codespec_arr=>$codespec_arr, num_of_instance=>$opts{num_of_instance});
        $self->start_execution(blockspec_arr=>$blockspec_arr);
    } else {
        my $xmlPlan = $self->get_xmlplan();
        my $codespec_arr = [];
        if($xmlPlan->has_testcase()){
            my $node = $xmlPlan->get_xmlplan(original=>0);

            my $dotplan = Testplan->generateDotPlan(
                nodexml         =>$node, 
                xmlfile         =>$self->get_xmlfile(), 
                params_filter   =>$self->get_params_filter(),            
                debug_level     =>$self->get_debug_level(),
            );
            print $dotplan;
            my $codespec_arr = $self->get_codespec(dotplan=>$dotplan);
            
            print "CODESPEC===>\n",Dumper($codespec_arr),"\n";
            my $blockspec_arr = $self->get_static_blockspec(codespec_arr=>$codespec_arr, num_of_instance=>$opts{num_of_instance});
            $self->start_execution(blockspec_arr=>$blockspec_arr);
        } else {
            die "$Package: Error: There are no TCs generated by your filter\n";
        }
    }
}

sub dynamic_parallelism{
    my $self = shift;
    
    my $xmlPlan = $self->get_xmlplan();
    my $codespec_arr = [];
    if($xmlPlan->has_testcase()){
        my $node = $xmlPlan->get_xmlplan(original=>0);
        my $dotplan = Testplan->generate_filtered_XML(
            nodexml         =>$node, 
            params_filter   =>$self->get_params_filter(),            
            debug_level     =>$self->get_debug_level(),
        );
        
        $xmlPlan->printXML(xmlString=>$dotplan->toString());    # TODO: Comment this if not required.
        $self->set_filtered_XML(element=>$dotplan);
        $self->disintegrate_filtered_XML();
        
        # TODO: Uncomment this
        $self->{configMaker}->init_filer(mirror=>0);
        my $blockspec_arr = $self->get_dynamic_blockspec();   
        
        $self->{exec_type} = "dynamic";
        $self->start_execution(blockspec_arr=>$blockspec_arr);
    } else {
        die "$Package: Error: There are no TCs generated by your filter\n";
    }
}

sub get_codespec {
    my $self = shift;
    my %opts = validate(
        @_, {
            dotplan => { type => SCALAR }
        }
    );
    
    my $dotplan = $opts{dotplan};
    my @codespec_arr = ();
    my $countFilerVol = 1;
    
    my @arr = grep !/^\s*##/, split("\n", $dotplan );
    foreach my $testcase (@arr){
        my @vars = split( / /, $testcase) if length($testcase);
        if(! scalar @vars){
            next;
        }
        # print Dumper(\@vars)."\n\n";
        my %codespec = ();
        foreach my $var (@vars){
            if($var =~ /(\.pl|\.thpl|\.py|\.sh)\s*$/i){
                $codespec{codespec} = $var;
                next;
            }
            if($var !~ /\w*\=\w*/i){
                next;
            }
            my ($pkey, $pvalue) = split(/=/,$var);
            if($pkey =~ /filer_vol/i){
                $pvalue = $pvalue."_$countFilerVol";
                $countFilerVol += 1;
            }
            $codespec{params}->{$pkey} = $pvalue;
        }
        push(@codespec_arr, \%codespec);
    }
    return \@codespec_arr;
}

sub get_static_blockspec {
    my $self = shift;
    my (%opts) = validate(
        @_, {   
            codespec_arr  => { type => ARRAYREF },
            num_of_instance  => { type => SCALAR },
        }
    );    
    
    my @codespec = @{$opts{codespec_arr}};
    my $num_of_instance = $opts{num_of_instance};
    
    my @blockspec = ();
	# DEFECT: Below code adds undef to last blockspec as padding for remaining $num_of_instance.
    # while( scalar @codespec) {
    #    push(@blockspec, [[@codespec[0..$num_of_instance-1]]]);
    #    @codespec = @codespec[$num_of_instance..$#codespec];
    # }

    while( scalar @codespec ) {
        my @arr = ();
        for(my $i=0;$i<$num_of_instance;$i++){
            last if(!scalar(@codespec));
            push(@arr, shift @codespec);
        }
        push(@blockspec, [[@arr]]);
    }

    return \@blockspec;
}

sub get_dynamic_blockspec{
    my $self = shift;
    my $function = "get_dynamic_blockspec";
    my %xml_tags = ();
    $xml_tags{test_script} = "./test_script";
    $xml_tags{tc} = "./tc";
    my $element = $self->get_filtered_XML();

    my @blockspec=();
    my @tc_arr=();
    
    # Run this loop twice - once for mirror=0 and then for mirror=1
    foreach my $iMirror (0,1){
        my $loopCount=0;
        $self->init_resource(mirror=>$iMirror);
        while($element->findnodes($xml_tags{test_script})->size){
            foreach my $test_script ( $self->get_test_script(element=>$element)->get_nodelist()){
                foreach my $tc ( $self->get_tc(test_script=>$test_script)->get_nodelist()){
                    $self->log(3, "$Package:$function: Debug: \n\tLoop- $loopCount :".$tc->toString());
                    my $spare_disks = $self->has_script_dependency(test_script=>$test_script) 
                        ? $self->get_spare_disks(dependency=>$self->get_script_dependency(test_script=>$test_script))
                        : $self->get_spare_disks(dependency=>$self->get_global_dependency());
                    $self->log(4, "$Package:$function: Debug: Spare Disks as listed in dependency :".Dumper($spare_disks));
                    
                    my %attr = $self->getAttributesFromElement(element=>$tc);
                    my $mirror = exists($attr{MIRROR}) ? $attr{MIRROR} : 0;
                    my $hybrid = exists($attr{HYBRID}) ? $attr{HYBRID} : 0;

                    $self->log(4, "$Package:$function: Debug: \n\tAttributes:"
                        .Dumper(\%attr)."\n\t"
                        ."MIRROR : $mirror\n\t"
                        ."HYBRID : $hybrid"
                    );
                    if($mirror != $iMirror){
                        next;
                    }
                    
                    my $resource = $self->manage_resource(spare_disks=>$spare_disks, mirror=>$mirror, hybrid=>$hybrid);
                    $self->log(4, "$Package:$function: Debug: Resource obtained from manage_resource:".Dumper($resource));
                    if($resource->get_selected()){
                        $self->log(3, "$Package:$function: Debug: Selected TC for blob :".$self->getTextNodeValue(element=>$tc));
                        # TODO: Here find replace STORAGE_TYPE's value with $resource->get_storage_type();
                        push(@tc_arr, $self->getTextNodeValue(element=>$tc));
                        $test_script->removeChild($tc);
                    }
                    if($resource->get_empty() and scalar(@tc_arr) > 0){
                        # here pass "mirror" value to init_resource()
                        $self->init_resource(mirror=>$mirror);
                        my $tcStr = join("\n", @tc_arr);
                        $self->log(3, "$Package:$function: Debug: CodeSpec :".Dumper($self->get_codespec(dotplan=>$tcStr)));
                        push(@blockspec, [ $self->get_codespec(dotplan=>$tcStr) ]);
                        undef @tc_arr;
                        @tc_arr=();
                    }
                }
                if(! $test_script->findnodes($xml_tags{tc})->size){
                    $element->removeChild($test_script);
                }
            }
            
            if(scalar(@tc_arr) > 0){
                # This will happen when number of tests are less than resources
                # $self->init_resource();  # May be we may not need init_resource here ? As all TCs may have already exhaused here.
                my $tcStr = join("\n", @tc_arr);
                $self->log(3, "$Package:$function: Debug: CodeSpec :".Dumper($self->get_codespec(dotplan=>$tcStr)));
                push(@blockspec, [ $self->get_codespec(dotplan=>$tcStr) ]);
                undef @tc_arr;
                @tc_arr=();
            }

            if($loopCount == 3){
                $self->log(3, "$Package: Debug: Exceeded MAX Loop Count and ".
                    "following TCs couldn't be accommodated in given resource :");
                foreach my $test_script ( $element->findnodes($xml_tags{test_script})->get_nodelist()){
                    $self->log(4, "$Package:$function: Debug: ".$test_script->toString());
                }
                last;
            }
            $loopCount++;
        }
    }
    return \@blockspec;
    #   Iterate all TCs
    #       if(manageResource(spares) == valid):
    #           use resource
    #           push TC to Blob
    #           remove TC from element
    #           all TCs iterated once, then create blob
    #           resetResource
    #       else:
    #           Drop TC if cannot be accommodated in given resource
}

sub init_resource{
    my $self = shift;
    # TODO: Uncomment this
    my %opts = validate(
        @_, {
            mirror      => { type => SCALAR, optional => 1, default => 0},
        }
    );
    $self->{configMaker}->init_resource(mirror=>$opts{mirror});
}

sub manage_resource{
    my $self = shift;
    my %opts = validate(
        @_, {
            spare_disks => { type => ARRAYREF },
            mirror      => { type => SCALAR, optional => 1, default => 0},
            hybrid      => { type => SCALAR, optional => 1, default => 0},
        }
    );

    # TODO: Comment this
    #return (selected=>int(rand(3)), empty=>int(rand(2)));
    # TODO: Uncomment this
    return $self->{configMaker}->manage_resource( 
        spare_disks => $opts{spare_disks},
        mirror      => $opts{mirror},
        hybrid      => $opts{hybrid},
    );
}

sub disintegrate_filtered_XML{
    my $self = shift;
    my $element = $self->get_filtered_XML();

    my %xml_tags = ();
    $xml_tags{global_dependency} = "./global_dependency";
    
    my $xmlroot = XML::XPath::Node::Element->new("xmlplan");
    if($element->findnodes( $xml_tags{global_dependency} )->size){
        my @globalDeps = $element->findnodes( $xml_tags{global_dependency} )->get_nodelist();
        $self->set_global_dependency(global_dependency => $globalDeps[0]);
    }
    # $self->set_test_script(test_script=>$element->find($xml_tags{test_script}));
}

sub start_execution{
    my $self = shift;
    my (%opts) = validate(
        @_, {   
            blockspec_arr  => { type => ARRAYREF },
        }
    );    
    my $blockspec_arr = $opts{blockspec_arr};
    
    print "BLOCKSPEC===>\n",Dumper($blockspec_arr),"\n";
    
    # TODO:
    # Before running blockspec, create two tasks
    #   1. contentTask
    #   2. supportTask
    
    my $count=1;
    my @inner_blk_arr = (); 
    foreach my $blockspec (@{$blockspec_arr}){
        print "Running round - $count\n";
        my $supportTask = "support Tasks block";#Stest::Block->new(
            # blockspec => [
                # [   
                    # { 
                        # params   => {
                            # TYPE_OF_IO          => "filersio",
                            # USE_EXISTING_VOLS   => 1
                        # },
                        # codespec => "raidqa/utilities/IOGen.thpl" 
                    # },
                    # { codespec => "Filer/version.thpl" },
                # ],  
            # ],  
            # mode        => Stest::Block::PARALLEL,
            # iterations  => 1,
            # dirname     => "supportTasks",
        # );          
        my $contentTask = $blockspec->[0]; 
        my @blobTask = ($contentTask);
        push(@blobTask, $supportTask) if defined($supportTask);
        
        print Dumper({blockspec=>\@blobTask, mode => "Stest::Block::PARALLEL", iterations  => 1, dirname => "blob_$count"});
        print "==>MIRROR: ",$blockspec->[0]->[0]->{params}->{MIRROR},"\n";
        # TODO: Comment this cut
#=cut        
        eval {
            my $sblock = Stest::Block->new(
                blockspec   =>\@blobTask, 
                mode        => Stest::Block::PARALLEL, 
                iterations  => 1,  
                exit_on_errors => "", 
                # ignore_results=>1,
                dirname     => "blob_$count"
            );

            if(defined $self->{exec_type} and $self->{exec_type} =~ /dynamic/i){
                # TODO: extract MIRROR param from $blockspec and pass it to init_filer()
                my $mirror = defined( $blockspec->[0]->[0]->{params}->{MIRROR} )
                    ? $blockspec->[0]->[0]->{params}->{MIRROR}
                    : 0;
                $self->{configMaker}->init_filer(mirror=>$mirror);
            }
            $sblock->start();
            $sblock->wait();
            $sblock->destroy();
        };  
        if( $@){
            print "Error found: $@\n";
        } else {
            print "iteration $count passed\n";
        }   
        #   push(@inner_blk_arr, $sblock);
#=cut
        # TODO: Comment this cut
        $count += 1;
    }         
}

sub has_global_dependency{
    my $self = shift;
    my $element = $self->get_filtered_XML();
    
    my %xml_tags = ();
    $xml_tags{global_dependency} = "./global_dependency";    
    return $element->findnodes($xml_tags{global_dependency})->size;
}

sub has_script_dependency{
    my $self = shift;
    my %opts = validate(@_, {
        test_script => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    my $test_script = $opts{test_script};
    
    my %xml_tags = ();
    $xml_tags{script_dependency} = "./dependency";
    
    return $test_script->findnodes($xml_tags{script_dependency})->size;
}

sub get_xmlfile{
    my $self = shift;
    return $self->{xmlfile};
}

sub get_filter{
    my $self = shift;
    return $self->{filter};
}

sub get_params_filter{
    my $self = shift;
    return $self->{params_filter};
}

sub get_debug_level{
    my $self = shift;
    return $self->{debug_level};
}

sub get_dotplan{
    my $self = shift;
    return $self->{dotplan};
}

sub get_xmlplan{
    my $self = shift;
    return $self->{xmlplan};
}

sub get_global_dependency{
    my $self = shift;
    return $self->{global_dependency};
}

sub get_script_dependency{
    my $self = shift;
    my %opts = validate(@_, {
        test_script => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    my $test_script = $opts{test_script};
    
    my %xml_tags = ();
    $xml_tags{script_dependency} = "./dependency";
    
    my @scriptDeps = $test_script->findnodes($xml_tags{script_dependency})->get_nodelist();
    return $scriptDeps[0];
}

sub get_filtered_XML{
    my $self = shift;
    return $self->{filtered_XML};
}

sub get_test_script{
    my $self = shift;
    my %opts = validate(@_, {
        element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    my $element = $opts{element};
    
    my %xml_tags = ();
    $xml_tags{test_script} = "./test_script";
    
    return $element->find($xml_tags{test_script});
}

sub get_tc{
    my $self = shift;
    my %opts = validate(@_, {
        test_script => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    my $test_script = $opts{test_script};
    
    my %xml_tags = ();
    $xml_tags{tc} = "./tc";
    
    return $test_script->find($xml_tags{tc});
}

sub get_spare_disks{
    my $self = shift;
    my %opts = validate(@_, {
        dependency => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    my $dependency = $opts{dependency};
    
    my %xml_tags = ();
    $xml_tags{disks} = "./spare_disks/disk";
    
    my @disks = ();
    foreach my $disk ($dependency->findnodes($xml_tags{disks})->get_nodelist()){
        my %attrs = $self->getAttributesFromElement(element=>$disk);
        push(@disks, clone(\%attrs));
    }
    return \@disks;
}

sub set_xmlfile{
    my $self = shift;
    my %opts = validate(
        @_, {
            xmlfile => { type => SCALAR }
        }
    );
    $self->{xmlfile} = $opts{xmlfile};
}

sub set_filter{
    my $self = shift;
    my %opts = validate(
        @_, {
            filter => { type => SCALAR | UNDEF }
        }
    );
    $self->{filter} = $opts{filter};
}

sub set_params_filter{
    my $self = shift;
    my %opts = validate(
        @_, {
            params_filter => { type => SCALAR | UNDEF}
        }
    );
    $self->{params_filter} = $opts{params_filter};
}

sub set_debug_level{
    my $self = shift;
    my %opts = validate(
        @_, {
            debug_level => { type => SCALAR }
        }
    );
    $self->{debug_level} = $opts{debug_level};
}

sub set_dotplan{
    my $self = shift;
    my %opts = validate(
        @_, {
            dotplan => { type => SCALAR }
        }
    );
    $self->{dotplan} = $opts{dotplan};
}

sub set_xmlplan{
    my $self = shift;
    my %opts = validate(
        @_, {
            xmlplan => { type => OBJECT, isa=>"XMLDotPlan" }
        }
    );
    $self->{xmlplan} = $opts{xmlplan};
}

sub set_global_dependency{
    my $self = shift;
    my %opts = validate(@_, {
        global_dependency => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    $self->{global_dependency} = clone($opts{global_dependency});
}

sub set_filtered_XML{
    my $self = shift;
    my %opts = validate(@_, {
        element => { type=>OBJECT, isa=>"XML::XPath::Node::Element"},
    });
    $self->{filtered_XML} = clone($opts{element});
}

1;
