# $Id: //depot/prod/test/Rsierranevada/lib/PExecConfig.pm#2 $
# $Author: kjp $
## @author: nsesha@netapp.com [ dl-raid-qa ]

package PExecConfig;

use strict;
##==========================================
##       Library Includes
##==========================================
use Clone qw(clone);
use XML::LibXML;
use XML::XPath;
use XML::Twig;
use XML::Parser;
use Data::Dumper;
use Params::Validate qw(:all);
use XMLDotPlan;
use Stest::Block;
use NACL::C::Client;

our @ISA = qw(Utility);

my $Package = "PExecConfig";

use vars qw($VERSION %xml_tags);
$VERSION  = "1.0";

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    
    my %opts = validate(
        @_, {
            configxml   => { type => SCALAR, optional => 0},
            debug_level => { type => SCALAR, optional => 1, default => 1},
        }
    );
    my $self = $class->SUPER::new(debug_level=>$opts{debug_level});    
    %opts = (%{$self}, %opts);
    $self = \%opts;
    bless $self, $class;
    
    $self->{xml_tags}->{globals} = "/configxml/globals";
    $self->{xml_tags}->{globals_content_task} = "/configxml/globals/content_task";
    $self->{xml_tags}->{globals_support_task} = "/configxml/globals/support_task";
    $self->{xml_tags}->{support_task} = "/configxml/support_task[\@runme='yes']";
    $self->{xml_tags}->{support_task_tool} = "/configxml/support_task/tool";
    $self->{xml_tags}->{support_task_behavior} = "/configxml/support_task/behavior";
    
    # Few initialization methods
    $self->init();
    return $self;
} 

sub init{
    my $self = shift;
    
    $self->log(1,"$Package: Info: Validating XML file for well-formedness");
    if(!$self->is_valid_xml()){
        die("$Package: Error: Invalid configxml file");
    } else {
        $self->load_configxml();
        $self->validate_configxml();
    }
}

sub load_configxml{
    my $self = shift;
    $self->log(1, "$Package: Loading configxml file... " . $self->get_configxml());
    $self->{root_element} = XML::XPath->new( filename => $self->get_configxml() );
    $self->log(1, "$Package: Loading configxml... done");
} 

sub is_valid_xml{
    my $self = shift;
    eval{
        use XML::Simple;
        XMLin($self->get_configxml());
    };
    if($@){
        $self->log(1, "$Package: Error: Not a well-formed xml - ".$self->get_configxml()."\n$@");
        return 0;
    }
    return 1;
}

sub validate_configxml {
    my $self = shift;
    $self->log(1, "$Package: Started configxml validation ... " );

    # Validation logic:
    # If globals is not found, then defaults are considered.
    #   Default are...
    #   content_task/mode           PARALLEL
    #   content_task/iterations     1
    #   support_task/mode           PARALLEL
    #   support_task/iterations     1
    #   support_tasks/dirname       supportTasks
    #   support_task/instances      0
    # Atleast one of support_task should exist
    # Atleast one of support_task/tool should exist
    # Atleast one of support_task/tool/path should exist
    # Ignore support_task/tool/params if it doesn't exist
    # If support_task/behavior is not found, then defaults are considered. 
    #   Defaults are...
    #   support_tasks/behavior/iterations       1
    #   support_tasks/behavior/ignore_results   ignore this 
    #   support_tasks/behavior/runtime          ignore this
    #   support_tasks/behavior/dirname          tool_name_<randno>
    
    # Validate presence of 0..1 <globals>
    if($self->{root_element}->findnodes( $self->{xml_tags}->{globals} )->size){
        if($self->{root_element}->findnodes( $self->{xml_tags}->{globals} )->size > 1){
            die("$Package: Error: globals is found more than one instance in this configxml: \n".
                $self->get_configxml()."\n"
            );
        }

        # Validate presence of 0..1 <globals/content_task>
        if($self->{root_element}->findnodes( $self->{xml_tags}->{globals_content_task} )->size){
            if($self->{root_element}->findnodes( $self->{xml_tags}->{globals_content_task} )->size > 1){
                die("$Package: Error: <globals/content_task> is found more than one instance in this configxml: \n".
                    $self->get_configxml()."\n"
                );
            }
        }

        # Validate presence of 0..1 <globals/support_task>
        if($self->{root_element}->findnodes( $self->{xml_tags}->{globals_support_task} )->size){
            if($self->{root_element}->findnodes( $self->{xml_tags}->{globals_support_task} )->size > 1){
                die("$Package: Error: <globals/support_task> is found more than one instance in this configxml: \n".
                    $self->get_configxml()."\n"
                );
            }
        }
    }
    
    # Validate presence of 1..n <support_task>
    if(!$self->{root_element}->findnodes( $self->{xml_tags}->{support_task} )->size){
        die("$Package: Error: configxml file does not have even a single <support_task> tag with attribute runme=\"yes\"");
    }
    
    foreach my $supportTask ( $self->{root_element}->findnodes( $self->{xml_tags}->{support_task} )->get_nodelist() ) {
        if(!$supportTask->findnodes( "tool" )->size){
            die("$Package: Error: <support_task> does not have even a single <tool> tag: "
                .$supportTask->toString()."\n");
        }
        if(!$supportTask->findnodes( "tool/path" )->size){
            die("$Package: Error: <support_task/tool> does not have even a single <path> tag: "
                .$supportTask->toString()."\n");
        }
    }
    
    $self->log(1, "$Package: configxml validation passed" );
    return 1;
} 

sub get_support_tasks{
    my $self = shift;
    
    my $tag_name = $self->{xml_tags}->{support_task};
    my @sTasks = ();
    foreach my $supportTask ( $self->{root_element}->findnodes($tag_name)->get_nodelist() ) {
        my $path_elem = $supportTask->find("tool/path" );
        my $path = $self->getTextNodeValue(element=>${$path_elem}[0]);
        my %eachTool=(
           blockspec => [
               [
                   {
                       codespec => $path,
                       params => $self->get_params(tag_name=>"tool", element=>$supportTask),
                   },
               ],
           ],
           iterations     => $self->get_iterations(tag_name=>"behavior", element=>$supportTask),
           dirname        => $self->get_dirname(tag_name=>"behavior", path=>$path, element=>$supportTask),
           ignore_results => $self->get_ignore_results(tag_name=>"behavior", element=>$supportTask),
        );
        if(defined $self->get_runtime(tag_name=>"behavior", element=>$supportTask)){
            $eachTool{runtime} = $self->get_runtime(tag_name=>"behavior", element=>$supportTask);
        }
        # TODO: Replace this for NATE execution
        push (@sTasks, Stest::Block->new(%eachTool));
        # push(@sTasks, clone(\%eachTool));
    }
    
    my $num_of_instance = $self->get_instances();
    $num_of_instance = defined($num_of_instance) ? $num_of_instance : scalar(@sTasks);
    $num_of_instance = $num_of_instance > scalar(@sTasks) ? scalar(@sTasks) : $num_of_instance;
    my @sTaskBlob = ();
    while( scalar @sTasks ) {
        my @arr = ();
        for(my $i=0;$i<$num_of_instance;$i++){
            last if(!scalar(@sTasks));
            push(@arr, shift @sTasks);
        }
        push(@sTaskBlob, [@arr]);
    }
    
    my @supportTasks = ();
    $tag_name = $self->{xml_tags}->{globals_support_task};
    my $element = $self->{root_element};
    foreach my $bSpec ( @sTaskBlob ){
        my %sTask = (
            blockspec       => clone($bSpec),
            mode            => $self->get_mode(tag_name=>$tag_name, element=>$element),
            iterations      => 1,
            dirname         => $self->get_dirname(tag_name=>$tag_name, element=>$element),
            ignore_results  => $self->get_ignore_results(tag_name=>$tag_name, element=>$element),
        );
        if(defined $self->get_runtime(tag_name=>$tag_name, element=>$element)){
            $sTask{runtime} = $self->get_runtime(tag_name=>$tag_name, element=>$element);
        }
        # TODO: change this to 
        push(@supportTasks, Stest::Block->new(%sTask));
        # push(@supportTasks, clone(\%sTask));
    }
        
    my %supportTask =  (
        blockspec   => [ @supportTasks ],
        mode        => Stest::Block::SERIAL,
        iterations  => $self->get_iterations(tag_name=>$tag_name, element=>$element),
        dirname     => "supportTasks",
    );
    # TODO: Use Stest::Block->new() here 
    # return \%supportTask;
    return Stest::Block->new(%supportTask); 
}

sub get_configxml{
    my $self = shift;
    if(!defined($self->{configxml})){
        die "$Package: Error: error in configxml";
    }
    if(! -e $self->{configxml}){
        die "$Package: Error: configxml file not found - $self->{configxml}";
    }    
    return $self->{configxml};
}

# Returns xml string form for root_element
sub toString{
    my $self = shift;
    $self->log(1, "$Package: Get string format of xml file");
    my @nodeSet=$self->{root_element}->findnodes('/');
    my $xml=XML::XPath::XMLParser::as_string($nodeSet[0]);
    return $xml;    
}

sub get_params{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
            element  => { type => OBJECT, isa=>"XML::XPath::Node::Element"},            
        }
    );
    
    if(!$opts{element}->findnodes( $opts{tag_name}."/params/*" )->size){
        return {};
    }
    
    my %params = ();
    foreach my $param ($opts{element}->findnodes( $opts{tag_name}."/params/*" )->get_nodelist()){
        if($param->getName() =~ /^CLIENTS$/){
            my $clients_string = $self->getTextNodeValue(element=>$param);
            $params{$param->getName()} = $self->get_clients(clients_string=> $clients_string);
        } else {
            $params{$param->getName()} = $self->getTextNodeValue(element=>$param);
        }
    }
    return \%params;
}

sub get_mode{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
            element  => { type => OBJECT},            
        }
    );
    # TODO: Change Stest::Block::PARALLEL from string to macro.
    if(!$opts{element}->findnodes( $opts{tag_name}."/mode" )->size){
        return Stest::Block::PARALLEL;
    }
    
    my $mode_elem = $opts{element}->find( $opts{tag_name}."/mode" );
    my $mode = $self->getTextNodeValue(element=>${$mode_elem}[0]);
    # if($mode =~ /^\s*(parallel|random|serial)\s*$/i){
    #     return "Stest::Block::".uc($mode);
    # }
    return Stest::Block::SERIAL if($mode =~ /^\s*(serial)\s*$/i);
    return Stest::Block::RANDOM if($mode =~ /^\s*(random)\s*$/i);
    return Stest::Block::PARALLEL;
}

sub get_iterations{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
            element  => { type => OBJECT},            
        }
    );

    if(!$opts{element}->findnodes( $opts{tag_name}."/iterations" )->size){
        return 1;
    }
    
    my $iter_elem = $opts{element}->find( $opts{tag_name}."/iterations" );
    my $iter = $self->getTextNodeValue(element=>${$iter_elem}[0]);
    if($iter =~ /^\s*(\-?\d+)\s*$/i){
        return $iter;
    }
    
    return 1;
}

sub get_ignore_results{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
            element  => { type => OBJECT},            
        }
    );

    if(!$opts{element}->findnodes( $opts{tag_name}."/ignore_results" )->size){
        return 0;
    }
    
    my $ir_elem = $opts{element}->find( $opts{tag_name}."/ignore_results" );
    my $ir = $self->getTextNodeValue(element=>${$ir_elem}[0]);
    if($ir =~ /^\s*(\d+)\s*$/i){
        return $ir;
    }
    
    return 0;
}

sub get_runtime{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
            element  => { type => OBJECT},            
        }
    );

    if(!$opts{element}->findnodes( $opts{tag_name}."/runtime" )->size){
        return undef;
    }
    
    my $runtime_elem = $opts{element}->find( $opts{tag_name}."/runtime" );
    my $runtime = $self->getTextNodeValue(element=>${$runtime_elem}[0]);
    if($runtime =~ /^\s*(\-?\d+)\s*$/i){
        return $runtime;
    }
    
    return undef;
}

sub get_instances{
    my $self = shift;
    my $tag_name = $self->{xml_tags}->{globals_support_task};
    my $element = $self->{root_element};
    if(!$element->findnodes( $tag_name."/instances" )->size){
        return undef;
    }
    
    my $instances_elem = $element->find( $tag_name."/instances" );
    my $instances = $self->getTextNodeValue(element=>${$instances_elem}[0]);
    if($instances =~ /^\s*(\d+)\s*$/i){
        return $instances;
    }
    
    return undef;
}

sub get_dirname{
    my $self = shift;
    my %opts = validate(
        @_, {
            tag_name => { type => SCALAR },
            path     => { type => SCALAR|UNDEF, optional => 1, default => undef },
            element  => { type => OBJECT},            
        }
    );

    my $notFound = 0;
    if(!$opts{element}->findnodes( $opts{tag_name}."/dirname" )->size){
        $notFound = 1;
    } else {
        my $dirname_elem = $opts{element}->find( $opts{tag_name}."/dirname" );
        my $dirname = $self->getTextNodeValue(element=>${$dirname_elem}[0]);
        if($dirname !~ /^\s*$/i){
            return $dirname."_".int(rand(100));
        } else {
            $notFound = 1;
        }
    }
    if($notFound){
        if(!defined $opts{path}){
            return "supportTasks";
        } else {
            my @arr =  split('/', $opts{path});
            my @toolname = split('\.', $arr[-1]);
            pop(@toolname);
            return join('_', @toolname)."_".int(rand(100));
        }    
    }
    return "supportTasks";
}

# Return unique entries from input array
sub uniq{         
    my $self = shift;                                                                                                                                                       
    my %hash;
    return grep { !$hash{$_}++ } @_;
}

# Returns clients if CLIENTS is available as param
sub get_clients{
    my $self = shift;
    my %opts = validate(@_, {
        clients_string => { type => SCALAR }
    });
    my @availableClients = NACL::C::Client->find();
    die "Error: Reconfigure testbed with some valid clients\n" if(!scalar(@availableClients));
    
    my @linclients = ();
    my @winclients = ();
    foreach my $client (@availableClients){
        if ($client->hostrec()->hosttype() =~ /(linux|unix)/i) {
            push @linclients, $client->client();
        } elsif($client->hostrec()->hosttype() =~ /(windows)/i) {
            push @winclients, $client->client();
        }
    }
    
    my %clientHash = ( 
        "LINCLIENT" => \@linclients,
        "WINCLIENT" => \@winclients,
    );
    
    my @cArr = split(/,/, $opts{clients_string});
    my %hash = ();
    
    foreach my $clientType ("LINCLIENT", "WINCLIENT"){
        my @uniqClientArr = sort(grep /^$clientType/, $self->uniq(@cArr));
        my @clients = @{$clientHash{$clientType}};
        
        my %tmpHash = ();
        if(scalar(@uniqClientArr) <= scalar(@clients)){
            %tmpHash = map {$uniqClientArr[$_] => $clients[$_] } 0..$#uniqClientArr;
        } else {
            %tmpHash = map {$uniqClientArr[$_] => $clients[$_] } 0..$#clients;
            my $diff = scalar(@clients) - scalar(@uniqClientArr);
            for(my $i = scalar(@clients); $i<scalar(@uniqClientArr); $i++){
                $tmpHash{$uniqClientArr[$i]} = $clients[int(rand(scalar(@clients)))];
            }
        }
        %hash = (%hash, %tmpHash);
    }
    
    my @finalCArr = ();
    foreach my $client (@cArr){
        push @finalCArr, $hash{$client};
    }

    return join(",", @finalCArr);
}

1;
#    Data-Structure format:
#    # use Stest::Block;
#    # my $eachTool= Stest::Block->new(
#            # blockspec => [
#                # [
#                    # {
#                        # codespec => "path/to/test.thpl",
#                        # params => {
#                            # "KEY" => "PARAM1",
#                            # "KEY" => "PARAM1"
#                        # }
#                    # },
#                # ],
#            # ],
#            # iterations       => 1000,
#            # ignore_results   => 0,
#            # runtime          => "1d",
#            # dirname          => $tool.int(rand(100)),
#        # );
#    
#    # my $supportTask = Stest::Block->new(
#            # blockspec => [
#                # $eachTool_1,
#                # $eachTool_N,
#            # ],
#            # mode        => Stest::Block::PARALLEL,
#            # iterations  => 1,
#            # dirname     => "supportTasks",
#        # );
#    # return $supportTask
