# vim: set tabstop=4 shiftwidth=4 expandtab shiftround:
## @summary     common parameter definitions for Options.pm users
## @pod here

=head1 NAME

Params::NDISP - common NDISP definitions for Options.pm parameters

=head1 SYNOPSIS

   use Options;
   use Params::NDISP;

=head1 DESCRIPTION

This package is used to define the common NDISP parameters used by
testers. It can be used by anyone using the L<Options> package
as a convenient way to keep your options all consistent.

=head1 METHODS

none

=cut

use Options;

our $Log;

$Options = new Options;
$Options->parameters(
    definitions => [
        {   name        => "TESTBED_NDISPD",
            description => "NDISPD Server and Port for this the testbed (server:port)",
            subroutine  => [ \&_parse_testbed_ndispd ],
        },
        {
            name        => "NDISPD_SERVER",
            description => "Server name/ip for NDISPD",
        },
        {
            name        => "NDISPD_PORT",
            description => "Port number for NDISPD",
        },
    ]
);
$Options->compatability(
    definitions => {
    }
);


#
# Parse the value in TESTBED_NDISPD and set the default for NDISPD_SERVER and NDISPD_PORT
#
sub _parse_testbed_ndispd {
    $Log->enter();
    my	$testbed_ndispd	= shift;

    if ( $testbed_ndispd =~ /^(\d+\.\d+\.\d+\.\d+|\S+):(\d+)$/ ) {
        $Log->trace("Setting global default for 'NDISPD_SERVER' to '$1'");
        $Options->set_default( 'NDISPD_SERVER' => $1 );
        $Log->trace("Setting global default for 'NDISPD_PORT' to '$2'");
        $Options->set_default( 'NDISPD_PORT' => $2 );
    } else {
        die NATE::Result::Config->new(
            "Value for TESTBED_NDISPD '$testbed_ndispd' is not in the correct format 'server:port'");
    }
    $Log->exit();
}	# ----------  end of subroutine _parse_testbed_ndispd  ----------

=head1 PARAMS

=over

=item TESTBED_NDISPD

Defines the NDISPD server and port for the testbed

=back

=head1 SEE ALSO

L<Options>

=head1 BUGS

none

=head1 STATUS

  Shared

=head1 AUTHOR

  Dan Schirato (schirato@netapp.com)
  $Id$
  Last revised $DateTime$

=head1 COPYRIGHT

Copyright 2010, NetApp, All Rights Reserved

=cut

1;
