## @summary     common parameter definitions for Options.pm users
## @pod here

=head1 NAME

Params::NFS - common NFS definitions for Options.pm parameters

=head1 SYNOPSIS

   use Options;
   use Params::NFS;

=head1 DESCRIPTION

This package is used to define the common NFS parameters used by
GX testers. It can be used by anyone using the L<Options> package
as a convienient way to keep your options all consistent.

=head1 METHODS

none

=cut

use Options;

$Options = new Options;
$Options->parameters(definitions=>
    [
      {   # don't provide a default; leave it to the test
	      name => "nfsversion",
	      valid_values => [qw(2 3 4 4_1)],
	      description => "The version of NFS to run",
      },
      {   # don't provide a default; leave it to the test
	      name => "protocol",
	      valid_values => [qw(udp tcp tcp6 udp6)],
	      description => "the transport protocol to use",
      },
      {
	      name => "EXPORT_OPTIONS",
	      description => "Export option to use",
      },
      {
          name => "RSIZE",
          default => "32768",
          description => "how much data to read at a time",
      },
      {
          name => "WSIZE",
          default => "32768",
          description => "how much data to write at a time",
      },
      {
          name => "MOUNT_TYPE",
          default => "hard",
          description => "a hard or soft mount type",
          valid_values => [qw(hard soft)],
      },
      # Options for configuring NFS on the filer
      {
          name         => "NFS4_1",
          description  => "Enable NFS v4.1 configuration",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
      {
          name         => "NFS4_0",
          description  => "Enable NFS v4.0 configuration",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
      {
          name         => "NFS3_0",
          description  => "Enable NFS v3 configuration",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
      {
          name => "NFS2_0",
          description => "Enable NFS v2 configuration",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
      {
          name => "NFS_ACCESS",
          description => "Enable general NFS access",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
      {
          name          => "NFS_RPCSEC_CTX_HI",
          description   => "RPC GSS Context Cache High Water Mark",
          spec          => "s",
          greater_equal => 0,
          default       => undef,
      },
      {
          name          => "NFS_RPCSEC_CTX_IDLE",
          description   => "RPC GSS Context Cache Idle",
          spec          => "s",
          greater_equal => 0,
          default       => undef,
      },
      {
          name         => "NFS_UDP",
          description  => "Enable UDP protocol for NFS",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
      {
          name         => "NFS_TCP",
          description  => "Enable TCP protocol for NFS",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
      {
          name         => "NFS_SPINAUTH",
          description  => "Enable Spin Authentication for NFS",
          spec         => "s",
          valid_values => [ 0, 1, undef ],
          default      => undef,
      },
    ]);

$Options->compatability(definitions=>
    {
        NFS_VERSION => "nfsversion",
        PROTOCOL    => "protocol",
        NFS41       => "NFS4_1",
        NFS40       => "NFS4_0",
        NFS4        => "NFS4_0",
        NFS3        => "NFS3_0",
        NFS2        => "NFS2_0",
    });


=head1 PARAMS

=over

=item PROTOCOL

This parameter specifies transport protocol to use during NFS mount. All possible values of protocol for various clients are provided and a client may not support all valid_values. User has to set correct values.

=back

=head1 SEE ALSO

L<Options>
L<TestManager>

=head1 BUGS

none

=head1 STATUS

  Shared

=head1 AUTHOR

  Sankaran P (psankara@netapp.com)
  $Id: $
  Last revised $Date: 2008/08/14 $

=head1 COPYRIGHT

Copyright 2006, Network Appliance., All Rights Reserved

=cut

1;
