# this is basically -*- perl -*-
# vim: set ft=perl tabstop=4 shiftwidth=4 expandtab shiftround: 
#
# $Id:$
#
# Copyright (c) 2009 Network Appliance Inc.
# All rights reserved.
#

=head1 NAME

Params::Security - common Security definitions for Options.pm parameters

=head1 SYNOPSIS

   use Options;
   use Params::Security;

=head1 DESCRIPTION

This package is used to define the common Security parameters used by
Options. It can be used by anyone using the L<Options> package
as a convenient way to keep your options all consistent across many libraries
and thpls.

=head1 METHODS

none

=cut

our $Log;

use Options;

$Options = new Options;
$Options->parameters( definitions=> [
        {
            name	    => "SECD_TRACE_SPEC",
            description => "Spec to use for enabling tracing inside of SECD",
            spec	    => "s@",
            default	    => undef,
            delimiter	=> "#",
            subroutine  => [\&validate_secd_trace_spec],
        },
        {
            name	=> "SECD_TRACE_TARGETS",
            description	=> "Targets that are valid for setting/clearing tracing on",
            spec	=> "s@",
            default	=> [qw(vserverids ipaddrs domain-names dc-names rpc-names module-names)],
            valid_values => [qw(vserverids ipaddrs domain-names dc-names rpc-names module-names)],
        }
]);

# If there are any options that have changed names here is where we would put them
$Options->compatability( definitions=>
    {
    }
);

# make sure that anyone passing in a SECD_TRACE_SPEC passes it in correctly
sub validate_secd_trace_spec {
    my $secd_trace_spec = shift;
    $Log->enter("secd_trace_spec = '$secd_trace_spec'");

    my $secd_trace_targets = $Options->get(option => "SECD_TRACE_TARGETS");
    # walk through the secd_trace_spec and make sure all the targets are valid
    foreach my $spec ( @$secd_trace_spec ) {
        $Log->debug("spec = '$spec'");
        my $target = ( split /=/, $spec )[0];
        if ( !grep(/$target/, @$secd_trace_targets) ) {
            die $Log->error("Given a target in the SECD_TRACE_SPEC that is not defined in the " .
                "SECD_TRACE_TARGETS.  You must add '$target' to SECD_TRACE_TARGETS in order " .
                "to use it in SECD_TRACE_SPEC and make sure the cdefs know about the target.");
        }
    }
    $Log->exit();
}

1;
