## @summary     volume-specific parameter definitions for Options.pm users
## @author      okeefe@netapp.com
## @pod here

=pod

=head1 NAME

Params::Volume - volume-specific definitions for Options.pm parameters

=head1 SYNOPSIS

   use Options;
   use Params::Volume;

=head1 DESCRIPTION

This package is used to define the volume-specific parameters used by
GX testers. This package is included automatically by L<TestManager>
tests. It can be used by anyone also using the L<Options> package.

=head1 METHODS

none

=cut

use Options;
#use TracePkg;
$Trace = new TracePkg;

$Options = new Options;

$Options->parameters(definitions =>
		     [
		      {
			  name => "volume",
			  spec => "s@",
			  description => "list of comma-separated volumes",
		      },
		      {
			  name => "VOLUME",
			  spec => "s@",
			  description => "list of comma-separated volumes",
		      },
		      {
			  name => "VOL_FILES",
			  spec => "e",
			  description => "Maximum number of files for each volume",
		      },
		      {
			  name => "VOL_MAXDIR_SIZE",
			  spec => "e",
			  description => "Maximum directory size for each volume",
		      },
		      {
			  name => "VOL_SIZE",
			  spec => "e",
			  description => "The Size of the volume to be created is assumed to be Bytes unless specified with an appropriate data unit quantifier. MB,mb GB, gb, TB, tb PB, pb, EB, eb.",
		      },
		      {
			  name => "VOL_STRIPE_WIDTH",
			  spec => "e",
			  description => "Striped Data Volume Width. Range is 256KB..1GB in 4KB increments (default: 64MB)",
		      },
		      {
			  name => "VOL_STRIPE_COUNT",
			  spec => "s",
			  description => "Striped Data Volume Count (default: The number of storage aggregate members)",
		      },
		      {
			  name => "VOL_CONCURRENCY",
			  spec => "s",
			  valid_values => [ qw(low medium high peak) ],
			  description => "Configure Striped volume concurrency setting",
		      },
                      {   
                          name => "VOL_OPTIMIZE",
                          spec => "s",
                          valid_values => [ qw(max-performance performance space max-space) ],
                          description => "Configure Striped volume performance setting",
                      },		      
		      {
			  name => "VOL_LOOKUP_CACHE",
			  spec => "s",
			  valid_values => [ qw(0 no false disable disabled 1 yes true enable enabled) ],
			  description => "Enable/Disable Striped Volume Directory Name Caching - Deprecated",
		      },	  
		      {
			  name => "VOL_SECURITY_STYLE",
			  spec => "s",
			  description => "Security style for the volume(s)",
			  valid_values => [ qw(unix ntfs mixed) ],
		      },
                      {
                       name => "VOLUME_STYLE",
                       description => "The volume style required on selected aggregates",
                       valid_values => [ qw(flex striped *) ],
                       spec => "s",
                      },
		      {
                       name => "VOLUME_TYPE",
                       description => "type of volume",
                       valid_values => [ qw(RW LS DR DP TMP TEMP UNKNOWN SNAPSHOT *) ],
                       default => "RW",
		      },
                      {
                          name => "VOL_STRIPE_FORMAT",
                          spec => "s",
                          description => "Stripe format for the volume(s) full (Acro) or data (Coral)",
                          valid_values => [ qw(data full) ],
                      },
		      {
                          name => "VOL_INDEX_DIR_ENABLED",
			  spec => "boolean",
			  description => "Create Sidewinder enabled volumes (Flexvols only) Requires privilege level of test, will not work on a promoted build.",
                      }
                     ]);

$Options->compatability(definitions =>
			{
			    VOLUME => "volume",
			    FILER_VOL => "volume",
			});

=pod

=head1 PARAMS

=over

=item VOLUME

This param can be used to indicate a comma-separated list of volumes to use.

=item VOL_FILES

This param indicates the number of files that a given volume should use. It is
passed to the command that actually creates the volume on the filer.

=item VOL_MAXDIR_SIZE

This param indicates the maximum size that a directory should be allowed to grow
to in a volume. It is passed to the command that actually creates the volume on the filer.

=item VOL_SIZE

Specifies the size of any new volumes which are created. The
new volumes will default to 1000 MB. The size is assumed to be in MB unless
a different modifier is used (GB, TB)

=item VOLUME_SECURITY_STYLE

Security style for the volume(s) being created.

=item VOLUME_TYPE

Type of volume to use: RW LS DR TEMP UNKNOWN SNAPSHOT *. Defaults to RW.

=item VOLUME_STYLE

Style of volume to create: flex, striped or *. Defaults to flex.

=item VOL_STRIPE_WIDTH

This param  allows specification of the Striped Data Volume Width. A striped volume width range is 256KB..1GB in 4KB increments (default: 64MB).
It is passed to the command that actually creates the volume on the filer.

=item VOL_STRIPE_COUNT

This param  allows specification of the Striped Data Volume Count. A striped volume count range is (2..64), the default is the number of storage aggregate members.
It is passed to the command that actually creates the volume on the filer.

=item VOL_STRIPE_FORMAT

Format of striped volumes to create. Valid values are full (Acro) or data (Coral)

=item VOL_INDEX_DIR_ENABLED

Create volumes that use the Sidewinder directory format. 
This option is only applicable to Flexvols. Acro Volumes always use Sidewinder. 

=item VOL_CONCURRENCY

Configure Striped volume concurrency setting. Controls file timestamp behavior.
Valid options: low medium high peak

=back

=head1 SEE ALSO

L<Options>
L<TestManager>

=head1 BUGS

none 

=head1 STATUS

  Shared

=head1 AUTHOR

  Brian O'Keefe (okeefe@netapp.com)
  $Id: //depot/prod/test/Rsierranevada/lib/Params/Volume.pm#1 $
  Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

Copyright 2006, Network Appliance., All Rights Reserved

=cut

1;
