## @summary     Common WAFL parameter definitions for Options.pm users
## @author      gvg@netapp.com
## @pod here

=head1 NAME

Params::WAFL.pm - ONTAP Classic WAFL common definitions for Options.pm parameters

=head1 SYNOPSIS

   use Options;
   use Params::WAFL;

=head1 DESCRIPTION

This package is used to define the common parameters used by
Classic WAFL testers. This package is included automatically by 
L<WaflTestManager> tests. It can be used by anyone also using 
the L<Options> package.

=head1 METHODS

none

=cut

use Options;

$Options = new Options;
$Options->parameters(definitions=>
		     [
		      {
			  name => "TOPOLOGY_NAME",
			  default => "wafl_default",
			  description => "Topology name for tests to use",
			  required => "Need atleast one topology to run. Please specify",
		      },
		      {
			  name => "CIFS_USER",
			  default => "administrator",
			  description => "CIFS administrator user",
		      },
		      {
			  name => "CIFS_DRIVE",
			  default => "*",
			  description => "Cifs drive for tests to use",
		      },
		      {
			  name => "CIFS_PASSWD",
			  default => "cifspass",
			  description => "Cifs password used for cifs client logins",
		      },
		      {
			  name => "CIFS_SHARE",
			  default => "testshare",
			  description => "Cifs share to use",
		      },
		      {
			  name => "CMDTIMEOUT",
			  default => "120",
			  description => "Command timeout for commands",
		      },
		      {
			  name => "FILER_IP",
			  default => undef(),
			  description => "Filer Ip address",
		      },
		      {
			  name => "FILER_QTREE",
			  default => "filerqtree",
			  description => "Qtree name of the filer",
		      },
		      {
			  name => "MOUNTPOINT",
			  default => undef(),
			  description => "Mount point (directory path) used for mounts",
		      },
		      {
			  name => "NBLADE_START",
			  default => undef(),
			  description => "GX use",
		      },
		      {
			  name => "NBLADE_HOST",
			  default => undef(),
			  description => "GX specific",
		      },
		      {
			  name => "NBLADE_DIR",
			  default => undef(),
			  description => "GX specific",
		      },
		      {
			  name => "PFILER",
			  default => "",
			  description => "GX specific",
		      },
		      {
			  name => "TIMEOUT",
			  default => "600",
			  description => "Timeout",
		      },
		      {
			  name => "MOUNT_VERS",
			  default => undef(),
			  description => "Mount version used for testing",
		      },
		      {
			  name => "WAFLIRON_TIMEOUT",
			  default => "2400",
			  description => "Timeout for wafliron process",
		      },
		      {
			  name => "SLCC_STRICT",
			  default => "1",
			  description => "",
		      },
		      {
			  name => "FILER_MODE",
			  default => "mono",
			  description => "",
		      },
		      {
			  name => "RAID_TYPE",
			  default => undef(),
			  description => "Raid type used for testing",
		      },	
		      {
			  name => "LICENSES",
			  default => "nfs,cifs",
			  description =>"Licenses required to be installed",
		      },		  
		      ]);

1;
