# -*- perl -*-
# $Id: $
#
## @author   dl-perf-qa@netapp.com
## @summary  Callout wrapper for NACL components
## @status   public
##
## @pod here

=head1 NAME

Perf::AAMP::Callout::NACL

=head1 DESCRIPTION

Perf::AAMP::Callout::NACL allows NACL components to be called as AAMP/SLSL
callouts.

=cut

package Perf::AAMP::Callout::NACL;

use strict;
use warnings;
use Params::Validate qw(:all);
use Perf::AAMP::Callout qw(:slsl_results);
use base q(Exporter);

our %EXPORT_TAGS = (
    all => [
        qw(
            nacl_component
        ),
    ],
);
our @EXPORT_OK = @{$EXPORT_TAGS{all}};


sub nacl_component :Callout {
    my $resources = shift;
    my %opts      = validate(
        @_, {
            component => { type => SCALAR,  optional => 0  },
            method    => { type => SCALAR,  optional => 0  },
            args      => { type => HASHREF, default  => {} },
        },
    );

    my $ci     = $resources->command_interface();
    my $pkg    = $opts{component};
    my $method = $opts{method};
    my $args   = $opts{args};

    # Execute the specified NACL call in a list context.
    eval "require $pkg";
    my @results = $pkg->$method(
        command_interface => $ci,
        %{$args},
    );

    # AAMP has no use for command_interface object in
    # ComponentState objects. Therefore, delete them.
    foreach my $elem (@results) {
        delete $elem->{command_interface};
    }

    # Return the results to AAMP as the callout payload.
    return (SLSL_RESULT_SUCCESS, \@results);
}

1;

