version = "1.2.1"

from pprint import pprint

from aamp.slsl.macros import *

from qos_test import QoSTest, Events
from events import Events


class CPUbound(QoSTest):

    """Subclass of QoSTest. Create a cpubound test."""

    def __init__ (self):
        """Initialize a QoS cpubound test & do setup."""
        # instantiate the super class
        QoSTest.__init__(self)

        # do the setup for the test
        self.setup()
        return

    def create_workload(self, **keywords):
        """Specify parameters for a cpubound workload & extend
        QoSTest.create_workload."""
        keywords['read_pct'] = 100
        keywords['rand_pct'] = 0

        return QoSTest.create_workload(self, **keywords)

    def create_default_events (self, events_obj):
        """Create events required for a CPUbound test.
        Arguments:-
        events_obj: An instance of the Events class specifying the events to
                    occur"""
        nodes = test_obj.get_nodes()

        # TODO: Need to specify the counters to track for CPUbound tests.
        # node_name:system:system:cpu_busy - the node must be the node receiving
        # I/O.
        events_obj.create(event_name='track_counters',
                          event_phase='start_iteration')
        events_obj.create(event_name='start_sample',
                          event_phase='start_measurement_after')
        events_obj.create(event_name='stop_sample',
                          event_phase='end_measurement_after')
        events_obj.create(event_name='cpu_util_check',
                          event_phase='end_measurement_after', util=80,
                          iteration=iteration_type)
        return

    def create_iteration (self, **keywords):
        """Default to cpu utilization check as a feedback & extend
        QoSTest.create_iteration."""
        if keywords['type'] == 'baseline':
            if 'feedback_callout' not in keywords:
                keywords['feedback_callout'] = {
                    'name':'disk_util_check',
                    'phase':'end_measurement_after',
                    'args':{'util':80, 'iteration':keywords['type']}
                }
        return QoSTest.create_iteration(self, **keywords)
