version = "1.2.1"

from pprint import pprint

from aamp.slsl.macros import *
from aamp.slsl.setlist_parser import callout
from aamp.test_execution.iteration import Iteration
from aamp.test_execution.test_runner import TestRunner


class Events(object):

    """Create & keep a list of SLSL events."""

    def __init__(self, caller='iteration'):
        """Initialize an events instance. Build event list dict based on the
        caller."""
        self.list = {}
        AAMP_PHASES = []

        if caller == 'testrunner':
            AAMP_PHASES = TestRunner.phases
        elif caller == 'iteration':
            AAMP_PHASES = Iteration.phases
        else:
            "Unknown caller '%s' for event" %caller

        for phase in AAMP_PHASES:
            self.list[phase] = []

        return

    def create(self, event_name, event_phase, event_type='callout', **keywords):
        """Create a callout for the specified event & add it to the
        event list."""

        if event_type == 'callout':
            callback = None
            if keywords is None:
                callback = callout(type ='nate', callback_name=event_name)
            else:
                callback =  callout(type ='nate', callback_name=event_name,
                                    args=keywords)

            self.list[event_phase].append(callback)
        else:
            'Unsupported event type: ', event_type

        return

    def add(self, event, event_phase):
        """Append the specified event to the event list."""
        self.list[event_phase].append(event)

    def get_list(self):
        """Return the event dict"""
        return self.list
