# -*- perl -*-
#
## @author   dl-perf-qa@netapp.com
## @summary  Base class for AAMP test results
## @status   public
##
## @pod here

=head1 NAME

Perf::AAMP::Test::Results

=head1 DESCRIPTION

Perf::AAMP::Test::Results is the abstract base class for AAMP test results.
Each test runner should have a results class that extends this class and
implements its methods.

Result objects are constructed by the Perf::AAMP::Test class, and are
accessible through methods of that class.

=head1 METHODS

=cut

package Perf::AAMP::Test::Results;

use strict;
use warnings;
use Carp qw(confess);


=head2 new

Construct and return a test results object.

=cut

sub new {
    confess 'Method must be implemented in subclass: new';
}


=head2 test_results

Return NATE::Result object that indicates the pass/fail status of the test.

=over

=item Arguments

None

=back

=cut

sub test_results {
    confess 'Method must be implemented in subclass: test_results';
}


=head2 query

Perform the named query of the AAMP results and return the requested data.
The supported query names (and their default) depend on the specific test
runner.

=over

=item Arguments

B<name> = string

I<Optional> The name of the query to perform.

=back

=cut

sub query {
    confess 'Method must be implemented in subclass: query';
}

1;

