# -*- perl -*-
#
## @author   dl-perf-qa@netapp.com
## @summary  Results class for LocalNATE tests
## @status   public
##
## @pod here

=head1 NAME

Perf::AAMP::Test::Results::LocalNATE

=head1 DESCRIPTION

Perf::AAMP::Test::Results::LocalNATE is the results class for the LocalNATE
test runner.

=cut

package Perf::AAMP::Test::Results::LocalNATE; {
    use strict;
    use warnings;
    use Carp qw(confess);
    use NATE::Result;
    use Object::InsideOut qw(Perf::AAMP::Test::Results);
    use Params::Validate qw(:all);


    my @activity_log         :Field
                             :Type(ARRAY_ref);
    my @nate_result          :Field;
    my @test_results_params  :Field
                             :Arg(Name => 'test_results_params', Mandatory => 1);


    sub _init :Init {
        my $self = shift;

        my $activity_log = $test_results_params[$$self]->{activity_log};
        confess 'Missing required test_results_params field: activity_log'
            if !defined($activity_log) || ref($activity_log) ne 'ARRAY';
        $activity_log[$$self] = $activity_log;

        my @errors   = grep { $_->{status} eq q(ERROR)   } @{$activity_log};
        my @failures = grep { $_->{status} eq q(FAILURE) } @{$activity_log};

        if (scalar(@errors) > 0) {
            $nate_result[$$self] = NATE::Result::Script->new(
                'Error(s) in callout(s): '
              . join(', ', map { $_->{callback_name} } @errors),
            );
        }
        elsif (scalar(@failures) > 0) {
            $nate_result[$$self] = NATE::Result::Fail->new(
                'Failure(s) in callout(s): '
              . join(', ', map { $_->{callback_name} } @failures),
            );
        }
        else {
            $nate_result[$$self] = NATE::Result::Pass->new('Passed');
        }
        return;
    }


    sub test_results {
        my $self = shift;
        return $nate_result[$$self];
    }


    sub query {
        my $self = shift;
        my %opts = validate(
            @_, {
                name => { type => SCALAR, default => 'log' },
            },
        );
        my $name = $opts{name};

        confess "Invalid query: name => '$name'"
            if $name ne 'log';

        return $activity_log[$$self];
    }
}

1;

