# -*- perl -*-
#
## @author   dl-perf-qa@netapp.com
## @summary  Results class for PerfANT tests
## @status   public
##
## @pod here

=head1 NAME

Perf::AAMP::Test::Results::PerfANT

=head1 DESCRIPTION

Perf::AAMP::Test::Results::PerfANT is the results class for the PerfANT
test runner.

=cut

package Perf::AAMP::Test::Results::PerfANT; {
    use strict;
    use warnings;
    use Carp qw(confess);
    use NATE::Result;
    use Object::InsideOut qw(Perf::AAMP::Test::Results);
    use Params::Validate qw(:all);


    my @nate_result         :Field;
    my @perfant_result_list :Field
                            :Type('XML::LibXML::Element');
    my @perfant_runid       :Field
                            :Type(scalar);
    my @perfant_status      :Field
                            :Type(scalar);
    my @test_results_params :Field
                            :Arg(Name => 'test_results_params', Mandatory => 1);


    sub _init :Init {
        my $self = shift;

        # PerfANT 'runid' is a required test_results_params field.
        my $perfant_runid = $test_results_params[$$self]->{runid};
        confess 'Missing required test_results_params field: runid'
            if !defined($perfant_runid);
        $perfant_runid[$$self] = $perfant_runid;

        # PerfANT 'status' is a required test_results_params field.
        my $perfant_status = $test_results_params[$$self]->{status};
        confess 'Missing required test_results_params field: status'
            if !defined($perfant_status);
        $perfant_status[$$self] = $perfant_status;

        # Parsed XML 'result_list' is a required test_results_params field.
        my $perfant_result_list = $test_results_params[$$self]->{result_list};
        confess 'Missing required test_results_params field: result_list'
            if !defined($perfant_result_list)
               || !$perfant_result_list->isa('XML::LibXML::Element');
        $perfant_result_list[$$self] = $perfant_result_list;

        # Create appropriate NATE result based on the PerfANT StatusCode.
        if ($perfant_status eq 'SUCCESS') {
            $nate_result[$$self] = NATE::Result::Pass->new(
                "PerfANT success for runid $perfant_runid",
            );
        }
        elsif ($perfant_status =~ m/^ERR_/) {
            $nate_result[$$self] = NATE::Result::Config->new(
                "Fatal PerfANT error for runid $perfant_runid: $perfant_status",
            );
        }
        else {
            confess "Unrecognized perfant_status: '$perfant_status'";
        }
        return;
    }


    # Note: This result only indicates whether the PerfANT test completed
    # successfully or not. Test scripts will need to use other means of
    # determining the pass/fail status of their test cases.
    sub test_results {
        my $self = shift;
        return $nate_result[$$self];
    }


    sub query {
        my $self = shift;
        my %opts = validate(
            @_, {
                name => { type => SCALAR, default => 'result_list' },
            },
        );
        my $name = $opts{name};

        # TODO: What queries should this support?!! (For now,
        # just return the parsed XML 'ResultList' element.)
        confess "Invalid query: name => '$name'"
            if $name ne 'result_list';

        return $perfant_result_list[$$self];

    }
}

1;

