# -*- perl -*-
# $Id: //depot/prod/test/main/lib/Perf/CM.pm
# Copyright (c) 2007 Network Appliance, Inc.
# All rights reserved.
# lib/Perf/CM.pm
#
## @summary  Perl library for Counter Manager Object
## @author   dl-perf-qa@netapp.com
## @status   private

package Perf::CM;

use strict;
use warnings;
use Tharn;
use TCD;
use Carp;
use Server;
use List::MoreUtils qw(none);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use perf::counter_manager::Counter_Manager;
use NACL::C::Node;
use Data::Dumper;

# Function to create a new Counter Manager Object

sub new {
    my $class  = shift;
    my %params = validate(
        @_,
        {
            test => {
                type     => OBJECT,
                required => 1,
            },

            test_type => {
                type     => SCALAR,
                optional => 1,
                regex    => qr/fiji|non-fiji/,
                default  => 'non-fiji'
            },

            top_logdir => {
                type     => SCALAR,
                required => 1
            },
        }
    );
    my $self = \%params;
    bless $self, $class;

    my $site       = $self->{test}->get_param( 'SITE',       '' );
    my $filer_conn = $self->{test}->get_param( 'FILER_CONN', '#rsh' );
    $self->{site} = $site;

    my @node_objs = NACL::C::Node->find();
    $self->{node_objs} = \@node_objs;

    $self->{host} = $self->{node_objs}->[0]->node;
    $self->{fh}   = host( $self->{node_objs}->[0]->node );
    my $host = $self->{host};

    if ( host($host)->hosttype =~ /-ng-/ ) {

        # Workaround to prevent Server.pm slowness.
        param( 'SWALLOW_TIMEOUT', '-default', 0 );

        $self->{is_cmode} = 1;
        my @known_nodes;

        #my $server      = Server->new();
        #my @known_nodes = $server->servers();
        #confess "Server object cannot connect to '$self->{filer}'"
        #  if ( none { $_ eq $self->{host} } @known_nodes );

        foreach my $node_obj(@{$self->{node_objs}}){
            push(@known_nodes, $node_obj->node);
        }
        $self->{nodes}  = \@known_nodes;
        #$self->{fc}     = $server;
        #$self->{server} = $server;
        my $cluster_vserver =
          $self->{test}->get_param( 'CLUSTER_VSERVER' || 'vs0' );
        $self->{cluster_vserver} = $cluster_vserver;

    }
    else {

        $self->{is_cmode} = 0;
        $self->{fc}       = connect( $self->{host} . $filer_conn );
        $self->{nodes}    = [ $self->{host} ];
        $self->{server}   = undef;
    }
    return $self;
}

sub setup {
    my $self = shift;

    if ( $self->{is_cmode} ) {

        # By default vs0 will be used for the all the tests
        param( 'VSERVER', '-set',
            $self->{test}->get_param( 'VSERVER', 'vs0' ) );
    }

    # Add the test objects to the cm object depending on the type of process
    my @userspace_test_objects = qw(testcomputed testmanagedsimple testregular);
    my @kernel_test_objects =
      qw(test_dblade test_nblade test_dblade:container1 test_dblade:container2);

    $self->{user_test_objects}   = \@userspace_test_objects;
    $self->{kernel_test_objects} = \@kernel_test_objects;

    # Add the counters for each of the test objects :
    my @compute_counters    = qw(total_raw total_avg total_pct);
    my @managed_counters    = qw(total_delta time_rate);
    my @regular_counters    = qw(total);
    my @testdblade_counters = qw(counter_custom
                                container1_name
                                container1_uuid
                                container2_name
                                container2_uuid);
    $self->{testcomputed}->{counters}      = \@compute_counters;
    $self->{testmanagedsimple}->{counters} = \@managed_counters;
    $self->{testregular}->{counters}       = \@regular_counters;
    $self->{test_dblade}->{counters}       = \@testdblade_counters;

    $self->{rel_version} = get_release_branch($self);
    $Log->comment("Release path : $self->{rel_version}");

    return 1;
}

1;
