## @author secrest@netapp.com
## @summary
## @status private
##
## @pod here

=head1 NAME

Element::Raw

=head1 DESCRIPTION

This class exists as a Base class so raw devices can override some of
the functionality present in Element::File

=cut

package Perf::DataSet::Element::Raw;
{
    use strict;
    use warnings;
    use Tharn qw($Log say);
    BEGIN {
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Object::InsideOut;

    # The amount of space available to a client using this raw device
    # to access a LUN
    my @raw_size :Field :Arg(name => 'raw_size', mandatory => 1);

    # The name of the LUN this raw device file is associated with
    my @lun_id   :Field :Arg(name => 'lun_id', mandatory => 1) :Get(get_lun_id);


=head2 get_actual_size

Return the amount of space available via the raw device as determined
by the San libraries.

=cut

    sub get_actual_size {
        my $self = shift();

        return $raw_size[$$self];
    }
}

1;
