# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

package Perf::DataSet::GetActualSizes;

use strict;
use warnings;
use Carp qw(confess);
use Tharn qw($Log);
BEGIN{
    my @c = caller(2);
    $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
}
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Module::Common qw(validate_mixed);
use Perf::Utils qw(get_file_size as_string);
use Perf::DataSet::Command;
use Cwd qw();

use base qw(Perf::DataSet::Command);

sub validate {
    my $class = shift;
    my %params = validate_mixed(
           @_,
           [ ],
           {
            description => {type => SCALAR | UNDEF, optional => 1, default => "GetActualSizes"},
            element_type => {type => SCALAR | UNDEF, optional => 0},
            debug => {type => SCALAR | UNDEF, optional => 1, default => 0},
           }
        );
}
sub new {
    my $class = shift;
    my %params = $class->validate(@_);
    my $self = \%params;
    bless $self, $class;
    return $self;
}

sub pre_execute($$){
    my $self = shift;
    my $owner = shift;
    if (!$owner || !ref($owner)){
        confess "You must specify an owner";
    }

    if ($owner->isa('Perf::DataSet::Worker')){
        my $process_uid = $owner->get_process_uid;
        my $full_path = $self->{worker_full_paths}->{$process_uid};
        if (!$full_path){
            confess "No full path found for process '$process_uid'!\n" .
                    "Worker fullpaths: " . as_string($self->{worker_full_paths});
        }
        $owner->{full_path} = $full_path;

        $Log->comment("Changing to directory '$full_path'");
        chdir $full_path or confess "Unable to chdir to '$full_path': $!";
    }
    elsif($owner->isa('Perf::DataSet')){
        # Mount all the workers and store the mountpoints in the Addition
        $owner->mount;

        # Embed the worker paths to the command so that workers can load their own
        $self->{worker_full_paths} = $owner->get_worker_full_paths;

        $self->{container} = $owner->get_container->filter({type=>$self->{'element_type'}});
    }
    else{
        confess "The owner must be a Perf::DataSet or Perf::DataSet::Worker and not a " . ref($owner);
    }
}

sub execute {
    my $self = shift();
    my %args = @_;
    my $elements = $self->{container}->get_elements($args{element_type});
    my $returns;

    for my $path (keys(%$elements)) {
        $Log->comment("Getting size of $path");
        $returns->{$path} = $elements->{$path}->get_actual_size($path);
    }

    return $returns;
}


sub post_execute($$$){
    my $self = shift;
    my $owner = shift;
    my $returns = shift;

    if ($owner->isa('Perf::DataSet::Worker')){
        return $returns;
    }
    elsif ($owner->isa('Perf::DataSet')){
        # Merge results from workers
        # This is currently not necessary since this command is sent
        # to a single process. Make sure though.
        confess "Operation not supported yet" if ( scalar keys %$returns > 1 );
        my ($process_uid) = keys %$returns;

        $owner->unmount(users_ok => 1);

        return $returns->{$process_uid};
    }
}

1;
