# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Manager subclass for NFSv4 protocol
## @status   private
##
## @pod here

package Perf::DataSet::Manager::NFSv4Manager;
use strict;
use warnings;
use Carp qw(confess);
use Tharn qw($Log);
BEGIN{
    my @c = caller(2);
    $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
}
use Perf::Depot;
use namespace::clean;
use Perf::DataSet::Manager::NFSManager;
use base qw(  Perf::DataSet::Manager::NFSManager );

sub get_mount_opts {
    my ($self,$dataset_uid,%args) = @_;
    # Filers only support NFS4 over TCP
    $self->{nfs_mntopts}->{vers}  = 4;
    $self->{nfs_mntopts}->{timeo} ||= 6000;
    my %mount_opts = $self->SUPER::get_mount_opts($dataset_uid, %args);

    return %mount_opts;
}

sub check_domains {
    my ($self, $dataset_uid) = @_;
    my $dataset = Perf::Depot->get_resource(class => 'DataSet', name=>$dataset_uid);
    if (!$dataset){
        confess "Unknown Perf::DataSet $dataset_uid";
    }
    my $appliance = $dataset->get_appliance_resource;
    my $domain_id = $appliance->get_nfsv4_domain_id;
    my $clients = $dataset->get_client_resources;
    my $client_domain_id;
    for my $c (@$clients){
        my $c_domain_id = $c->get_nfsv4_domain_id;
        if ($c_domain_id ne $domain_id){
            confess(  "NFSv4 domain id on client '"
                    . $c->get_name()
                    . "' ($c_domain_id) does not match filer ($domain_id)");
        }
    }
}

sub initialize {
    my ($self, $dataset_uid) = @_;
    $self->check_domains($dataset_uid);
    return $self->SUPER::initialize($dataset_uid);
}

1;
