package Perf::DataSet::Manager::SANManager::FCPManager;

use strict;
use warnings;
use Carp qw(confess);
use Tharn qw($Log);
BEGIN{
    my @c = caller(2);
    $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
}
use Params::Validate qw(validate SCALAR OBJECT UNDEF ARRAYREF);

use namespace::clean;
use Perf::DataSet::Manager::SANManager;
use base qw(Perf::DataSet::Manager::SANManager);


sub initialize {
    my $self = shift();

    1; # no-op
}


sub register_dataset {
    my $self = shift();
    my %args = @_;

    my $dataset_obj = $self->SUPER::register_dataset(%args);

    my $appliance = $dataset_obj->get_dependency(class => 'Appliance');
    $appliance->detect_fcp_adapters();

    my $clients = $dataset_obj->get_dependencies(class => 'Host');
    $_->detect_fcp_adapters() foreach (@$clients);

    $appliance->detect_fcp_connectivity($clients);

    $self->create_services($dataset_obj);

    return $dataset_obj;
}


sub create_workers {
    my $self        = shift();
    my $dataset_uid = shift();
    my $workers     = $self->SUPER::create_workers($dataset_uid);

    foreach my $worker (@$workers) {
        my $host = $worker->get_process->get_host();
        my $host_hbas =
          Perf::Depot->get_resources(class     => 'Port::FCP',
                                              container => $host);
        confess("No host FCP ports found")
            if (!defined($host_hbas) || 0 == scalar($host_hbas));

        foreach my $host_hba (@$host_hbas) {
            my $appliance_hbas = $host_hba->get_accessible_by();

            $worker->add_dependencies([@$appliance_hbas, $host_hba]);
        }
    }

    return $workers;
}


# make FCP devices depend on the filer FCP port they'll be accessing
sub depend_dev_on_filer_resource {
    my $self = shift();
    my %param = validate(@_,
                         {dataset_uid => {type => SCALAR},
                          raw_dev     => {
                                     type => OBJECT,
                                     isa => 'Perf::Depot::Resource::SANDevice'
                            },});

    my $raw_dev  = $param{raw_dev};
    my $host_obj = $raw_dev->get_host();
    my $workers =
      Perf::Depot->get_resources(class => 'Worker',
                                          name  => $param{dataset_uid},
                                          host  => $host_obj);

    my @fcp_ports;
    foreach my $worker (@$workers) {
        my $ports = $worker->get_appliance_fcp_ports();
        push(@fcp_ports, @$ports);
    }

    $raw_dev->add_dependencies(\@fcp_ports);
}


sub get_protocol {
    my $self = shift();

    return 'fcp';
}


sub get_service_type {
    my $self = shift();

    return 'SANService::FCP';
}


1;
