# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File - Base class for Workload File Operations

=head1 DESCRIPTION

This class is the base class for all File Operation subclasses. It provides all the common functionality.

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Aligned;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }    
    our @EXPORT_OK = qw( get_aligned_read_buffer get_aligned_write_buffer );
    use base qw( Exporter );

    # Class attributes
    our $BUFFER_ALIGN = 512;    
    my %read_buffers;
    my %write_buffers;
    my $unit_buffer = 'x' x $BUFFER_ALIGN;
    
    sub get_aligned_read_buffer {
        my $buffer_info = $read_buffers{$_[0]};
        if (!$buffer_info){
            my $size = $_[0];
            $buffer_info = _create_buffer_info($size);
            # buffer templates that are likely to be reused get cached
            if ( $size % 4096 == 0 ){
                $read_buffers{$size} = $buffer_info;
            }                    
        }
        return $buffer_info;
    }

    sub get_aligned_write_buffer {
        my $buffer_info = $write_buffers{$_[0]};
        if (!$buffer_info){
            my $size = $_[0];        
            $buffer_info = _create_buffer_info($size);
            # buffer templates that are likely to be reused get cached
            if ( $size % 4096 == 0 ){
                $write_buffers{$size} = $buffer_info;
            }                    
        }
        return $buffer_info;
    }
    
    sub _create_buffer_info {
        my $buffer = 'x' x ( $_[0] + $BUFFER_ALIGN ); 
        #my $buffer = $unit_buffer x ( $_[0] / $BUFFER_ALIGN );
        # Compute offset for alignement
        my $align_offset = unpack( "J", pack "p", $buffer ) % $BUFFER_ALIGN;
        $align_offset = $BUFFER_ALIGN - $align_offset if ($align_offset);         
        return [\$buffer,$align_offset];            
    }
}
1;