# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File::Append - File append operation

=head1 DESCRIPTION

#TODO

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Append;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use FileHandle;
    use IO::Seekable;    
    use Perf::DataSet::Operation::File::Write;
    use base qw(Perf::DataSet::Operation::File::Write);
    
    # Aliases
    use constant FILE         => Perf::DataSet::Operation::ELEMENT;
    use constant OFFSET       => Perf::DataSet::Operation::File::Block::OFFSET;
    use constant SIZE         => Perf::DataSet::Operation::File::Block::SIZE;
    use constant AMOUNT_ADDED => Perf::DataSet::Operation::File::Write::AMOUNT_ADDED; 
    
    sub _init {
        my ($self,$args) = @_;
        my $offset = $args->{offset};
        my $file = $args->{element} || confess "You must specify an 'element' argument";
        if (!defined($offset)){
            $offset = $file->get_append_offset;
            $args->{offset} = $offset;
        }
        else{
            if (!$file->get_sparse){
                my $allowed_offset = $file->get_append_offset;
                confess "Offset must be at end of file ($offset instead of $allowed_offset)" 
                    if ($offset != $allowed_offset);
            }
            else{
                my $end_file = $file->get_size;
                confess "Operation offset '$offset' lower than end of file '$end_file'" 
                    if ($offset < $end_file);            
            }
        }        
        $self->SUPER::_init($args);
        confess "Operation not supported" if ($file->is_slice && !$file->is_last_slice);
    }

    sub get_categories {
        return ('write','append');
    }
    
    sub process {
        my ($self) = @_;
        # 
        # This is an append, therefore $file is the last slice
        # and the offset is equal or greater than the size of $file
        # 
        my $file = $self->[FILE];
        my $offset = $self->[OFFSET];
        my $size = $self->[SIZE];
        $file->set_size( $offset + $size );
        $file->set_size_whole( $offset + $size );
        $file->set_size_on_disk( $file->get_size_on_disk + $size );
        
        $self->SUPER::process;        
        
        my $amount_added = $self->[AMOUNT_ADDED];
        confess "Size of operation ($size) isn't consistent with amount_added ($amount_added)" 
            if ($amount_added != $size);        
        
     }

}
1;
    