# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File::Chmod - Chmod Operation

=head1 DESCRIPTION

This class is the base class for all File Operation subclasses. It provides all the common functionality.

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Chmod;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }    
    use Perf::DataSet::Operation;   
    use Exception::Class ('Perf::DataSet::Operation::FileChmodException' =>  
                            {
                                isa => 'Perf::DataSet::Operation::Exception',
                            },
                         );
    use Perf::DataSet::Operation::File;
    use Perf::DataSet::Operation::File::Modification;
    use base qw(Perf::DataSet::Operation::File Perf::DataSet::Operation::File::Modification);
    
    # Member attributes
    use constant PERMISSIONS => 10;

    # Aliases
    use constant FILE => Perf::DataSet::Operation::ELEMENT;    
    
    sub _init {
        my ($self,$args) = @_;
        $self->SUPER::_init($args);    
        my $perms = delete $args->{permissions};
        confess "You must speciy permissions" if (! defined $perms);
        if ($perms =~ /^0/){
            # In case we were passed a string
            $perms= oct($perms);
        }
        $self->[PERMISSIONS] = $perms;
    }
    
    sub _as_hash {
        my ($self,$hash) = @_;
        $self->SUPER::_as_hash($hash);
        $hash->{permissions}   = sprintf("%lo",$self->[PERMISSIONS]);
    }
    
    sub execute {
        my ($self) = @_;
        my $filepath = $self->[FILE]->get_path;
        chmod $self->[PERMISSIONS], $filepath 
            or Perf::DataSet::Operation::FileChmodException->throw(
                    error=>"Unable to set permissions " . sprintf("%lo",$self->[PERMISSIONS]) 
                            . " on '$filepath': $!");
    }
    
    sub get_categories {
        return ('chmod');
    }
 }
1; 