# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File::Lock::Read - Read Lock Operation

=head1 DESCRIPTION

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Lock::Read;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Exception::Class ('Perf::DataSet::Operation::FileLockException' =>  
                            {
                                isa => 'Perf::DataSet::Operation::Exception',
                            },
                         );    
    use Perf::DataSet::Operation::File::Lock;
    use base qw(Perf::DataSet::Operation::File::Lock);
    BEGIN{
        if ($^O !~ /Win32/i){    
            require Perf::DataSet::Operation::File::Lock::Read::Unix;
        }    
    }
    
    sub new {
        my $class = shift;
        return $class->Perf::DataSet::Operation::File::Lock::new(@_) if ($class ne __PACKAGE__);         
        if ($^O =~ /Win32/i){
            confess "Locks on Windows not supported yet";
        }
        else{
            return Perf::DataSet::Operation::File::Lock::Read::Unix->new(@_);
        }
    }
    
    sub get_categories {
        return ('lock_read');
    }
 }
1; 