# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File::Lock::Unix - Exclusive Lock Operation on Unix

=head1 DESCRIPTION

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Lock::Unix;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Perf::Utils qw();
    use File::FcntlLock;
    use Perf::DataSet::Operation;
    use Perf::DataSet::Operation::File::Lock;
    use base qw(Perf::DataSet::Operation::File::Lock);
    
    # Member attributes
    use constant LOCK_STRUCT => 30;
    
    # Aliases
    use constant FILE => Perf::DataSet::Operation::ELEMENT;
    
    sub _init {
        my ($self,$args) = @_;
        $self->SUPER::_init($args);
        $self->[LOCK_STRUCT] = File::FcntlLock->new( l_whence => SEEK_SET,
                                                     l_start  => 0,
                                                     l_len    => 0 );
    }

    sub _as_hash {
        my ($self,$hash) = @_;
        $hash->{lock_struct} = Perf::Utils::as_string($self->[LOCK_STRUCT]);
    }
    
    sub get_lock_structure {
        return $_[0]->[LOCK_STRUCT];
    }
    
    sub _l_type {
        my ($self, $type) = @_;
        $self->[LOCK_STRUCT]->l_type($type);
        return;
    }
    
    sub execute {
        my ($self,$sequence) = @_;
        my $file = $self->[FILE];
        my $fh = $sequence->get_fh($file);
        $self->[LOCK_STRUCT]->lock($fh, F_SETLKW) 
            or Perf::DataSet::Operation::FileLockException->throw(error=>"Unable to execute lock " 
                                                                    . $file->get_path . ":$!");
        return;
    }
    
    sub unlock {
        my ($self, $sequence) = @_;
        $self->[LOCK_STRUCT]->l_type( F_UNLCK );
        $self->execute($sequence);
        return;
    }
    
 }
1; 