# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File::Overwrite - File overwrite operation

=head1 DESCRIPTION

#TODO

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Overwrite;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    
    use Perf::DataSet::Operation;
    use Perf::DataSet::Operation::File::Block;
    use Perf::DataSet::Operation::File::Write;
    use base qw(Perf::DataSet::Operation::File::Write);
    
    # Aliases
    use constant FILE   => Perf::DataSet::Operation::ELEMENT;
    use constant OFFSET => Perf::DataSet::Operation::File::Block::OFFSET;
    use constant SIZE   => Perf::DataSet::Operation::File::Block::SIZE;
    use constant AMOUNT_ADDED => Perf::DataSet::Operation::File::Write::AMOUNT_ADDED;
    
    sub _init {
        my ($self,$args) = @_;
        $self->SUPER::_init($args);
        # Verify that the range fits in the file
        my $file_end = $self->[FILE]->get_end_offset;
        if ( $self->[OFFSET] + $self->[SIZE] > $file_end ){
            my $offset = $self->[OFFSET];
            my $size = $self->[SIZE];
            confess "Operation ($offset,$size) goes past end of file '$file_end'" 
        }    
    }
    
    sub process {
        my ($self) = @_;
        $self->SUPER::process;
        my $amount_added = $self->[AMOUNT_ADDED];
        if ($amount_added){
            my $file = $self->[FILE];
            $file->set_size_on_disk($file->get_size_on_disk + $amount_added );
        }
    }

}
1;
    