# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File::Remove - Remove Operation

=head1 DESCRIPTION

This class is the base class for all File Operation subclasses. It provides all the common functionality.

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Remove;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Perf::DataSet::Operation;
    use Exception::Class ('Perf::DataSet::Operation::FileRemoveException' =>  
                            {
                                isa => 'Perf::DataSet::Operation::Exception',
                            },
                         );    
    
    use Perf::DataSet::Operation;
    use Perf::DataSet::Operation::File;
    use Perf::DataSet::Operation::File::Modification;
    use base qw(Perf::DataSet::Operation::File Perf::DataSet::Operation::File::Modification);
    
    # Aliases
    use constant FILE => Perf::DataSet::Operation::ELEMENT;

    sub _init {
        my ($self,$args) = @_;
        $self->SUPER::_init($args);
        confess "Operation not allowed on slices" if ($self->[FILE]->is_slice);
    }
    
    sub execute {
        my ($self,$sequence) = @_;
        my $file = $self->[FILE];
        my $filepath = $file->get_path;
        
        # Close all filehandles associated to the file
        # and remove them from cache
        $sequence->remove_fh($file);

        unlink($filepath) or Perf::DataSet::Operation::FileRemoveException->throw(
                                error=>"Unable to unlink '$filepath': $!"); 
        $file->mark_deleted;
        return;
    }
    
    sub get_categories {
        return ('remove');
    }
 }
1; 