# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

=head1 NAME

File::Unlock - Unlock Operation

=head1 DESCRIPTION

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::DataSet::Operation::File::Unlock;
{
    use strict;
    use warnings;
    use Carp qw( confess );    
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Exception::Class ('Perf::DataSet::Operation::FileUnlockException' =>  
                            {
                                isa => 'Perf::DataSet::Operation::Exception',
                            },
                         );        
    use Perf::DataSet::Operation::File;
    use base qw(Perf::DataSet::Operation::File);
    
    # Member attributes
    use constant LOCK => 20;
    
    # Aliases
    use constant ELEMENT => Perf::DataSet::Operation::ELEMENT;
    
    sub _init {
        my ($self, $args) = @_;
        my $l = delete $args->{lock};
        confess "You must provide a 'lock' argument" if (!$l);
        $args->{element} = $l->[ELEMENT];
        $self->[LOCK] = $l;
        $self->SUPER::_init($args);
    }

    sub _as_hash {
        my ($self,$hash) = @_;
        $self->SUPER::_as_hash($hash);
        $hash->{lock} = $self->[LOCK]->dump;
    }
    
    sub execute {
        my ($self, $sequence) = @_;
        eval {$self->[LOCK]->unlock($sequence)};
        if ($@){ 
            Perf::DataSet::Operation::FileUnlockException->throw( error=>"Unable to unlock '" .
                                                                   $self->[ELEMENT]->get_path ."':\n$@");
        }                                                           
        return;
    }
    
    sub get_categories {
        return ('unlock');
    }
 }
1; 