# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

package Perf::DataSet::test_suite::ModularWorkload::RemoveCreateReadAppend;{

    use strict;
    use warnings;
    use Tharn qw($Log);
    use Carp qw(confess);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Perf::Utils qw(in_bytes);
    use File::Spec::Functions qw(splitpath);
    use Perf::DataSet::Sequence;
    use Object::InsideOut qw(Perf::DataSet::Sequence::File);
    use Perf::DataSet::Operation::File::Remove;
    use Perf::DataSet::Operation::File::Create;
    use Perf::DataSet::Operation::File::Read;
    use Perf::DataSet::Operation::File::Append;

    my @io_size :Field :Name('io_size') :Arg(Name=>'io_size',      Mandatory=>1);

    sub _init :Init {
        my $self = shift;
        $io_size[$$self] = in_bytes($io_size[$$self]);
    }

    sub get_operations {
        return ['Perf::DataSet::Operation::File::Remove',
                'Perf::DataSet::Operation::File::Create',
                'Perf::DataSet::Operation::File::Append',
                'Perf::DataSet::Operation::File::Read'];
    }

    sub next_operation {
        my ($self) = @_;
        # 0 for create, 1 for remove, <= 6 for read, <= 11 for append
        my $rand = $self->rand(12);
        # Don't fall below 2 files
        if ($self->get_container->count('files', include_deleted=>0) < 2){
            $rand = 0;
        }
        my $file = $self->random_element;
        if (!$rand){
            my $filepath = $file->get_path;
            # Remove file name. We'll create a new file in same directory
            my ($drive_letter,$reldir,$me) = splitpath($filepath);
            $filepath = $reldir . $self->get_container->get_random_filename;
            return Perf::DataSet::Operation::File::Create->new( path=>$filepath, block_size=>$file->get_block_size );
        }
        elsif($rand == 1){
            confess "File is a slice!" if ($file->is_slice);
            return Perf::DataSet::Operation::File::Remove->new(element=>$file);
        }
        elsif($rand <= 6) {
            my $offset = $self->random_offset($file, $io_size[$$self]);
            my $size = $file->before_eof($offset, $io_size[$$self]);
            return Perf::DataSet::Operation::File::Read->new( element => $file,
                                                        offset  => $offset,
                                                        size    => $size);
        }
        else{
            return Perf::DataSet::Operation::File::Append->new(element => $file,
                                                         size    => $io_size[$$self]);
        }
    }
}
1;
