package Perf::DataSet::test_suite::ModularWorkload::SequentialWrite;
{
    use strict;
    use Tharn qw($Log);
    use Perf::Utils qw(in_bytes);
    use Perf::DataSet::Operation::File::Overwrite;
    use Object::InsideOut qw(Perf::DataSet::Sequence::File);

    # Define member attribute 'io_size'
    my @io_size :Field :Arg(Name => 'io_size', Mandatory => 1 ) :Get(get_io_size);
    my @file :Field :Type(Perf::DataSet::Element::File) :Name('file');
    my @offset :Field :Type(numeric) :Name('offset');

    sub _init :Init {
        my $self = shift();
        $io_size[$$self] = in_bytes($io_size[$$self]);
    }

    sub get_operations {
        return ['Perf::DataSet::Operation::File::Overwrite'];
    }

    sub pre_execute {
        my $self = shift();
        my $owner = shift();

        $self->SUPER::pre_execute($owner);

        if ($owner->isa('Perf::DataSet::Worker')) {
            $file[$$self] = $self->random_element();
            $offset[$$self] = $file[$$self]->get_offset();
        }
    }

    sub next_operation {
        my $self   = shift();
        my $size   = $file[$$self]->before_eof($offset[$$self], $io_size[$$self]);

        if (0 >= $size) {
            $offset[$$self] = $file[$$self]->get_offset();
            $size = $file[$$self]->before_eof($offset[$$self], $io_size[$$self]);
        }

        my $op =
          Perf::DataSet::Operation::File::Overwrite->new(element => $file[$$self],
                                                   offset  => $offset[$$self],
                                                   size    => $size);

        $offset[$$self] += $size;

        return $op;
    }
}
1;
