# -*- perl -*-
# $Id: //depot/prod/test/main/ontap/boilermaker/lib/Perf/Delete_Logging.pm#3 $
#
## @author   jroussin@netapp.com
## @summary  
## @status   private
##
## @pod here

print("Loading ".__FILE__."\n");
package Perf::Delete_Logging;

use strict;
use warnings; 
use Carp qw(confess); 
use Perf::Utils qw(as_string wait_for_reclamation);   
use Tharn;

require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw(delete_logging_on delete_logging_off delete_logging_info
                 delete_logging_is_on delete_logging_reset delete_logging_counters);


sub delete_logging_is_on($$) {
    my $fc = shift;
    my $aggr = shift;
    my (undef, undef, $out) = say($fc, "priv set -q test; aggr options $aggr");
    if ($out =~ /no_delete_log=on/){
        return 0;
    }
    else{
        return 1;
    }
}

sub delete_logging_on($$) {
    my $fc = shift;
    my $aggr = shift;
    my (undef, undef, $out) = say($fc, "priv set -q test; aggr options $aggr no_delete_log off");
    if ($out){
        confess "Unable to turn on delete logging on $aggr: $out";
    }
}

sub delete_logging_off($$) {
    my $fc = shift;
    my $aggr = shift;
    my (undef, undef, $out) = say($fc, "priv set -q test; aggr options $aggr no_delete_log on");
    if ($out){
        confess "Unable to turn off delete logging on $aggr: $out";
    }    
}

sub delete_logging_info($$) {
    my $fc = shift;
    my $aggr = shift;
    my (undef, undef, $out) = say($fc, "priv set -q diag; wafl dlog_info $aggr");
    my @lines = split("\n",$out);
    my $results = {};
    for my $line (@lines){
        if ($line =~ /(Delete log|Purge file|Overflow file \d) \((\w+)\):? blks total (\d+)(?:, indirect (\d+))?/){
            $results->{$1}->{inum} = $3;
            $results->{$1}->{blocks} = $4;
            $results->{blocks} += $4;
            $results->{$1}->{indirects} = $5;
            $results->{indirects} += $5;
            if ($2 =~ /unloaded/){
                $results->{$1}->{loaded} = 0;
            }else{
                $results->{$1}->{loaded} = 1;
            }
        }
    }
    if (! keys %$results){
        confess "Unable to get delete logging info: $out";
    }
    return $results;
}

sub delete_logging_reset($$) {
    my $fc = shift;
    my $aggr = shift;
    delete_logging_info($fc, $aggr);
    delete_logging_off($fc, $aggr);
    wait_for_reclamation($fc,$aggr,5);
    delete_logging_on($fc, $aggr);
}

sub delete_logging_counters {
    my @a = ( 'del_log',
             'dlog_frees',
             'dlog_keys',
             'dlog_commit_and_split_time',
             'purge_vbns',
             'purge_send',
             'purge_block',
             'pre_split_failed',
             'pre_bufs_loaded',
             'purge_overflow',
             'purge_overflow_min_lev',
             'purge_overflow_no_split',
             'dlog_overuse_mem',
            );    
    if (wantarray){
        return @a;
    }            
    else{
        return \@a;
    }
}

1;
