# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::CIFSShare - Representation of a CIFS share in a volume

=head1 DESCRIPTION

This class represent a CIFS share used by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::CIFSShare;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Perf::Depot::Resource::Volume;
    use Object::InsideOut qw(Perf::Depot::Resource);

    my @volume  :Field :Arg(name=>'volume', mandatory=>1)
                :Type(Perf::Depot::Resource::Volume) :Get(get_volume);
    my @disable_oplocks
        :Field
        :Arg(name => 'disable_oplocks', default => 0)
        :Get(disabled_oplocks);

    sub _init :Init {
        my $self = $_[0];
        $self->add_dependencies($volume[$$self]);
        $self->set_name( $volume[$$self]->get_name );
        my $appliance = $volume[$$self]->get_appliance;
        $appliance->cifs_share_create(
            name => $self->get_name(),
            volume => $volume[$$self],
            disable_oplocks => $disable_oplocks[$$self]);

        # Even though the CIFS share was successfully created,
        # mounting may fail if attempted before it's been fully
        # initialized.
        sleep(30);
        return;
    }

    sub cleanup {
        my ($self) = @_;
        my $appliance = $volume[$$self]->get_appliance;
        $appliance->cifs_share_delete($self->get_name);
        return;
    }
}

1;
