## @author   dl-perf-qa-automation@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::LUNMap - Representation of a mapping between a LUN and an iGroup

=head1 DESCRIPTION

This class handles the mapping of a LUN tracked by a Perf::DataSet to
an iGroup.

=cut


package Perf::Depot::Resource::LUNMap;
{
    use strict;
    use warnings;
    use Carp qw(confess);
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Object::InsideOut qw(Perf::Depot::Resource);

    my @lun :Field
            :Arg(name => 'lun', mandatory => 1)
            :Get(get_lun)
            :Type(Perf::Depot::Resource::Lun);
    my @igroup :Field
               :Arg(name => 'igroup', mandatory => 1)
               :Get(get_igroup)
               :Type(Perf::Depot::Resource::IGroup);


    sub _init : Init {
        my $self      = shift();
        my $lun       = $lun[$$self];
        my $igroup    = $igroup[$$self];
        my $appliance = $lun->get_appliance();

        $appliance->lun_map(lun    => $lun,
                            igroup => $igroup,);

        $self->set_name($igroup->get_name() . ' <> ' . $lun->get_name());

        $self->add_dependencies([$lun, $igroup]);
    }

    sub cleanup {
        my $self      = shift();
        my $lun       = $lun[$$self];
        my $igroup    = $igroup[$$self];
        my $appliance = $lun->get_appliance();

        $appliance->lun_unmap(lun => $lun, igroup => $igroup,);
    }
}

1;
