# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Host - Representation of a host

=head1 DESCRIPTION

This class represent a filer tracked by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Localhost;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw($Log host);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use namespace::clean;
    use Perf::Depot::Resource::Host::Unix;
    use Object::InsideOut qw(Perf::Depot::Resource::Host::Unix);

    sub _populate {
        my $self = $_[0];
        # The following may fail if the localhost isn't a client from a testbed
        # in which case we'll assume the execution server is a linux host
        eval{
            my $ch = host($self->get_name);
            $self->set_hosttype($ch->hosttype());
        };
        if ($@){
            $Log->comment("Unable to get hosttype from host '". $self->get_name ."'. Assuming 'unix-linux-redhat':\n$@");
            $self->set_hosttype('unix-linux-redhat') if ($@);
        }

        $self->SUPER::_populate();
    }

    sub run_cmd {
        my ($self,$cmd) = @_;
        $Log->comment($self->get_name . ">> $cmd");
        my $out = `$cmd`;
        $Log->comment($out);
        return $out;
    }

    #sub get_ifconfig {
    #    my $self = $_[0];
    #    my $out = $self->run_cmd("/sbin/ifconfig -a");
    #    return $out;
    #}

    #sub get_class {
    #    return "Host";
    #}

    #sub can_ping {
    #    my ($self, $ipaddress) = @_;
    #    confess "You must pass an argument" if (!$ipaddress);
    #    confess "You must pass a Perf::Depot::Resource::Interface"
    #        if (! $ipaddress->isa('Perf::Depot::Resource::Interface'));
    #    my $out = $self->run_cmd("ping " . $ipaddress->get_ip . " -c 1");
    #    if ($out !~ /1 received/){
    #        return 0;
    #    }
    #    return 1;
    #}

    sub cleanup {
        # NOOP
    }
}
1;
