# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Mountpoint - Representation of a mountpoint on a host

=head1 DESCRIPTION

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Mountpoint;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Data::Dumper;
    use Object::InsideOut qw(Perf::Depot::Resource);

    my @host   :Field Arg(Name=>'host', Mandatory=>1) :Get(get_host) :Type(Perf::Depot::Resource::Host);
    my @latched :Field :Get(is_latched) :Name(latched);
    my @never_latched :Field :Get(was_never_latched) :Name(never_latched);

    sub _init :Init {
        my $self = shift;
        $latched[$$self] = 0;
        $never_latched[$$self] = 1;
        $self->add_dependencies($host[$$self]);
    }

    sub latch {
        my $self = $_[0];
        return if ($self->is_latched);
        $Log->debug($self->get_description . " is being latched");
        $self->do_latch;
        $latched[$$self] = 1;
        $never_latched[$$self] = 0;
        return;
    }

    sub unlatch {
        my $self = $_[0];
        return if (!$self->is_latched);
        $Log->debug($self->get_description . " is being unlatched");
        $self->do_unlatch;
        $latched[$$self] = 0;
        return;
    }

    sub do_latch {
        confess "You must implement method in subclass";
    }

    sub do_unlatch {
        confess "You must implement method in subclass";
    }

    sub set_latched :Restricted {
        my ($self, $value) = @_;
        confess "You must pass a value" if (! defined($value) );
        confess "Value must be 0|1" if ($value !~ /^0|1$/);
        $latched[$$self] = $value;
        if ($value eq '1' ){
            $never_latched[$$self] = 0;
        }
        return;
    }

    sub get_description {
        my $self = $_[0];
        return $self->SUPER::get_description . " on " . $host[$$self]->get_description;
    }
}
1;
