# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Mountpoint::CIFS - Representation of a CIFS mountpoint

=head1 DESCRIPTION

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Mountpoint::CIFS;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }

    use namespace::clean;
    use Perf::Depot::Resource::Mountpoint::NAS;
    use Object::InsideOut qw(Perf::Depot::Resource::Mountpoint::NAS);


    sub _init :Init {
        my $self = $_[0];
        my $cifs_share = Perf::Depot->get_resource( class=>'CIFSShare', volume=>$self->get_volume );
        confess "No CIFS share found on " . $self->get_volume->get_description if (!$cifs_share);
        $self->add_dependencies( $cifs_share );
    }


    sub get_mount_target {
        my $self = shift();
        my $mount_point = (  $self->was_never_latched()
                           ? $self->get_host()->take_drive_letter()
                           : $self->get_name());
        my %arg = (share       => $self->get_volume()->get_name(),
                   mount_point => $mount_point,);

        return %arg;
    }


    sub cleanup {
        my $self = shift();

        $self->SUPER::cleanup();

        $self->get_host()->release_drive_letter($self->get_name());
    }
}
1;
