# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Node - Representation of a 10-mode filer

=head1 DESCRIPTION

This class represent a filer tracked by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Node;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log host);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use List::MoreUtils qw(all);
    use Perf::Depot;
    #use NACL::C::Client::Ping;
    use NACL::C::Network;
    use NACL::STask::Node;
    use NATE::BaseException qw(:try);
    use Sys::Hostname;
    use namespace::clean;
    use Perf::Depot::Resource::Appliance::GX;
    use Object::InsideOut qw(Perf::Depot::Resource::Appliance::GX);

    sub get_conn {
        my $self = $_[0];
        my $conn = $self->SUPER::get_conn;

        if (!$conn || !$conn->isa('NACL::C::SystemNode')){
            $Log->comment("Creating new command interface for " . $self->get_description);
            $conn = NACL::STask::Node->new(name => $self->get_name());
            $self->set_conn($conn);
        }

        return $conn;
    }


    sub has_connection {
        my $self = shift;

        while (my (undef, $apiset) = $self->get_conn()->_apisets_each()) {
            return 0
                if (!$apiset->api_is_session_alive());
        }

        return 1;
    }


    sub sync {
        my ($self) = @_;

        $self->get_conn()->apiset(set => 'Nodescope')->sync();
    }

    sub nfs_start {
        # NO-OP. NFS is a vserver thing.
    }

    sub add_user {
        my ($self, $resource) = @_;
        my $was_added = $self->SUPER::add_user($resource);
        return if (!$was_added);
        if ($resource->isa('Perf::Depot::Resource::DataSet')){
            my $dataset_type = $resource->get_type;
            if ($dataset_type eq 'nfs'){
                $self->nfs_start;
            }
            elsif( $dataset_type eq 'nfsv4' ){
                $self->nfs_start(version=>4);
            }
        }
        return $was_added;
    }

    sub get_physical_appliance {
        my $self = shift();
        return $self;
    }


    sub toggle_fcp_ports {
        my $self = shift();

        return $self->_toggle_fcp_ports('Nodescope');
    }
}
1;
