# -*- perl -*-
#
## @author   dl-perf-qa-automation@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Port::Ethernet - Representation of an ethernet port

=head1 DESCRIPTION

This class represent a network port tracked by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Port::Ethernet;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Data::Dumper;

    use namespace::clean;
    use Perf::Depot::Resource::Port;
    use Object::InsideOut qw(Perf::Depot::Resource::Port);
    use Perf::Depot::Resource::Port::Ethernet::Appliance;
    use Perf::Depot::Resource::Port::Ethernet::Host;


    my @mtu             :Field :Arg(Name=>'mtu', Mandatory=>1)  Get(get_mtu);
    my @original_mtu    :Field :Name(original_mtu) Get(get_original_mtu);

    sub _init :Init {
        my $self = shift();
        $original_mtu[$$self] = $mtu[$$self];
    }

    sub new {
        my $class = shift;
        if ($class ne __PACKAGE__){
            return $class->Object::InsideOut::new(@_);
        }
        my %args = @_;
        my $c    = $args{container};
        confess "Argument 'container' is mandatory" if (!$c);
        confess "Argument 'container' must be a Resource" if (!ref($c));
        if ($c->isa('Perf::Depot::Resource::Appliance')) {
            return Perf::Depot::Resource::Port::Ethernet::Appliance->new(@_);
        }
        elsif ($c->isa('Perf::Depot::Resource::Host')) {
            return Perf::Depot::Resource::Port::Ethernet::Host->new(@_);
        }
        else {
            confess "Unsupported 'container' argument: " . ref($c);
        }
    }

    sub is_public {
        my $self = $_[0];
        my $ips = $self->get_users(type => 'Interface');
        for my $ip (@$ips) {
            return 1 if ($ip->is_public);
        }
        return 0;
    }

    sub set_mtu {
        my ($self, $mtu) = @_;
        $mtu[$$self] = $self->_mtu($mtu);
    }

    sub _mtu {
        confess "This method must be implemented by a subclass";
    }

    # Common code used by _client_mtu and _filer_mtu. Query
    # or update MTU on client or filer, as specified.
    sub _ifconfig_mtu {
        confess('TODO');

        # my ($self,$params, $mtu)   = @_;
        # my $conn = $self->get_conn;
        # my $if = $self->get_name;

        # # If an MTU was specified, set it.
        # if (defined $mtu){

        #     # Validate new MTU value.
        #     if ($mtu !~ m/^\d+$/){
        #         confess "Invalid MTU, must be a positive integer: $mtu";
        #     }

        #     # No need to check command's output since we will ensure
        #     # the setting changed below.

        #     say($conn, $params->{update} . $mtu);
        # }

        # # Fetch the current MTU value.
        # my (undef, undef, $cfg) = say($conn, $params->{query});

        # if ($cfg !~ m/$params->{match}/s){
        #     confess "Could not find MTU for " . $self->get_description;
        # }

        # # If an MTU was specified, ensure the setting was changed
        # # to the correct value.
        # if (defined $mtu and $1 ne $mtu){
        #     confess "Could not set MTU to $mtu for " . $self->get_description;
        # }

        # # Before returning wait until 'wait_for' regex for as long as 'wait_max' seconds
        # if ($params->{wait_match}){
        #     my $max_time = $params->{timeout};
        #     $max_time = defined($max_time)?$max_time:10;
        #     my $start_time = time;
        #     my $matched = 0;
        #     while (time - $start_time < $max_time){
        #         (undef, undef, $cfg) = say($conn, $params->{query});
        #         if ($cfg =~ m/$params->{wait_match}/s){
        #             $matched = 1;
        #             last;
        #         }
        #         sleep 2;
        #     }
        #     if (!$matched){
        #         confess "Timeout while waiting for mtu update on " . $self->get_description;
        #     }
        # }

        # return;
    }

    sub cleanup{
        my $self = $_[0];
        # Reset port mtu back to original value
        if ($mtu[$$self] != $original_mtu[$$self]){
            $self->_mtu($original_mtu[$$self]);
        }
        return;
    }
}
1;
