# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Port::Ethernet::Appliance - Representation of a network port on an appliance

=head1 DESCRIPTION

This class represent a networkk port tracked by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Port::Ethernet::Appliance;
{
    use strict;
    use warnings;
    use Carp qw(confess);
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use namespace::clean;
    use Perf::Depot::Resource::Port::Ethernet;
    use Object::InsideOut qw(Perf::Depot::Resource::Port::Ethernet);


    sub new {
        my $class = shift;
        if ($class ne __PACKAGE__){
            return $class->Object::InsideOut::new(@_);
        }
        my %args = @_;
        my $c = $args{container};

        confess "Argument 'container' argument must be an Appliance"
          if (   !ref($c)
              || !$c->isa('Perf::Depot::Resource::Appliance'));

        if ($c->isa('Perf::Depot::Resource::Filer')){
            eval "use Perf::Depot::Resource::Port::Ethernet::Filer";
            confess("Unable to load Port::Ethernet::Filer: $@") if ($@);
            return Perf::Depot::Resource::Port::Ethernet::Filer->new(@_);
        }
        elsif ($c->isa('Perf::Depot::Resource::Node')){
            eval "use Perf::Depot::Resource::Port::Ethernet::Node";
            confess("Unable to load Port::Ethernet::Node: $@") if ($@);
            return Perf::Depot::Resource::Port::Ethernet::Node->new(@_);
        }
        else{
            confess "Unsupported 'container' argument: " . ref($c);
        }
    }

    sub get_appliance {
        my $self = $_[0];
        return $self->get_container;
    }

    sub get_conn {
        my $self = $_[0];
        return $self->get_appliance->get_conn;
    }
}
1;
