# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Port::Ethernet::Host - Representation of a network port on a Host machine

=head1 DESCRIPTION

This class represent a network port on a Host machine tracked by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Port::Ethernet::Host;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Data::Dumper;
    use Object::InsideOut qw(Perf::Depot::Resource::Port::Ethernet);

    sub get_host {
        my $self = $_[0];
        return $self->get_container;
    }

    sub get_connection {
        my $self = shift;
        return $self->get_host->get_connection;
    }

    # Return MTU of the specified port on the client, optionally
    # setting it first if a second argument is supplied.
    sub _mtu {
        my ($self,$mtu) = @_;
        my $if = $self->get_name;
        my $params = {
            match  => "^$if\\s.*MTU:(\\d+)",               # ifconfig regex
            query  => "/sbin/ifconfig $if",                # ifconfig query cmd
            update => "sudo /sbin/ifconfig $if mtu "       # ifconfig mtu update cmd
        };
        return $self->_ifconfig_mtu($params,$mtu);
    }

}
1;
