# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Port::Ethernet::Node - Representation of a network port on a Node

=head1 DESCRIPTION

This class represent a network port on a Node tracked by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Port::Ethernet::Node;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Object::InsideOut qw(Perf::Depot::Resource::Port::Ethernet::Appliance);

    sub _init :Init {
        my $self = $_[0];
        # Verify appliance is a filer
        confess "Appliance argument must be a Node"
            if (!$self->get_appliance->isa('Perf::Depot::Resource::Node'));
    }

    sub get_node {
        my $self = $_[0];
        return $self->get_appliance;
    }

}
1;
