# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Process::Base - Representation of a remote process responsible for starting others

=head1 DESCRIPTION

This class represent a filer tracked by a Perf::DataSet.
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Process;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log subtest );
    use NATE::RPC::Freeze qw(freeze);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Data::Dumper;
    use Object::InsideOut qw(Perf::Depot::Resource::Process::Base);

    my @factory :Field :Arg(Name=>'factory', Mandatory=>1) :Type('Perf::Depot::Resource::Process::Factory');
    my @endpoint :Field :Arg(Name=>'endpoint', Mandatory=>1);
    my @handle   :Field :Arg(Name=>'handle', Mandatory=>1);
    my @connection :Field :Name('connection') :Type('NATE::STAF::Peer') :Get('get_staf_peer');
    my @clean    :Field :Name('clean') :Default(0) :Std('clean');

    sub _init :Init {
        my $self = $_[0];
        $self->add_dependencies( $factory[$$self] );
        $connection[$$self] = NATE::STAF::Peer->new(handle   => $handle[$$self],
                                                    endpoint => $endpoint[$$self]);
        return;
    }

    sub is_alive {
        my $self = shift;
        return $factory[$$self]->is_process_alive( uid=>$self->get_uid );
    }

    sub quit {
        my ($self) = shift;
        return if ($clean[$$self]);
        $Log->debug("Stopping RPC");
        $self->get_staf_peer->quit;
        return;
    }

    sub cleanup {
        my $self = $_[0];
        return if ($clean[$$self]);
        $self->quit;
        if (!$factory[$$self]){
            $Log->debug("No Factory found");
            return;
        }
        if ( !$factory[$$self]->is_alive){
            $Log->debug("Factory has already shut down");
            return;
        }
        $Log->debug("Removing process from factory");
        $factory[$$self]->cleanup_process( uid => $self->get_uid );
        $clean[$$self] = 1;
        return;
    }
}
1;
