# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::Process::Group - Representation of a group of remote processes

=head1 DESCRIPTION

It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::Process::Group;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log );
    use Params::Validate qw(validate OBJECT ARRAYREF);

    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Data::Dumper;
    use Object::InsideOut qw(Perf::Depot::Resource);

    my @processes :Field :Name('processes');
    my @factories :Field :Name('factories');

    sub add_processes {
        my $self = shift;
        my %args = validate(    @_,
                                {   factory  =>  {  type => OBJECT,
                                                    isa => 'Perf::Depot::Resource::Process::Factory',
                                                 },
                                    processes => {  type => ARRAYREF },
                                }
                            );
        my $factory = $args{factory};
        my $factory_uid = $factory->get_uid;
        $factories[$$self]->{ $factory_uid } = $factory;
        my $ps = $args{processes};
        for my $process (@$ps){
            confess "Argument must be Perf::Depot::Resource::Process"
                if ( ! $process->isa('Perf::Depot::Resource::Process') );
            $processes[$$self]->{ $factory_uid }->{ $process->get_uid } = $process;
        }
        $self->add_dependencies( $factory );
        $self->add_dependencies( $ps );
        return;
    }

    sub get_processes {
        my ($self) = @_;
        my @ps;
        for my $fp ( values %{ $processes[$$self] }){
            push @ps, values %$fp;
        }
        return \@ps;
    }

    sub cleanup {
        my $self = shift;
        while ( my ($factory_uid, $factory) = each %{$factories[$$self]} ){
            my @processes_uid = keys %{ $processes[$$self]->{$factory_uid} };
            my @ps = values %{ $processes[$$self]->{$factory_uid} };
            $_->quit for ( @ps );
            $factory->cleanup_processes_start( @processes_uid );
        }
        while ( my ($factory_uid, $factory) = each %{$factories[$$self]} ){
            $factory->wait_reply;
            # Don't leave the reply out there
            # This will die if an error was encountered during the call above
            $factory->read_reply;
            my @ps = values %{ $processes[$$self]->{$factory_uid} };
            $_->set_clean(1) for (@ps);
        }
        return;
    }

}
1;
