#
## @author   dl-perf-qa-automation@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::SANService - Representation of the SAN service on a host

=cut


package Perf::Depot::Resource::SANService;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }

    use Object::InsideOut qw(Perf::Depot::Resource);

    my @host
        :Field
        :Arg('Name'=>'host', 'Mandatory'=>1)
        :Type(Perf::Depot::Resource::Host)
        :Get(get_host);
    my @appliances
        :Field
        :Arg('Name'=>'appliances', 'Mandatory'=>0)
        :Type(ARRAY_ref(Perf::Depot::Resource::Appliance))
        :Default([])
        :Get(get_appliances);

    sub _init :Init {
        my $self = $_[0];
        $self->set_name($host[$$self]->get_name);
        $self->add_dependencies([$host[$$self], @{$appliances[$$self]}]);
        $self->start();
    }

    sub start {
        confess("This method must be implemented in a subclass of " . __PACKAGE__);
    }

    sub stop {
        confess("This method must be implemented in a subclass of " . __PACKAGE__);
    }

    sub configure {
        confess("This method must be implemented in a subclass of " . __PACKAGE__);
    }

    sub check_busy {
        my $self = shift();

        if ($self->is_used){
            # The call to *_configure below is going to restart a SAN
            # service.  This will cause all kinds of troubles if we
            # already have a SAN workload going for example. Make
            # sure none of the Perf::DataSet users are busy.
            my $dataset_users = $self->get_users( class => 'DataSet');
            my @busy_datasets = grep { $_->is_busy } @$dataset_users;
            if (@busy_datasets){
                my $str = join(', ', map { $_->get_name} @busy_datasets);
                confess "Operation aborted because of the following busy Perf::DataSet(s): $str";
            }
        }

        return;
    }

    sub has_appliance {
        my ($self, $resource) = @_;
        confess "You must specify an argument" if (!$resource);
        confess "Argument must be a Perf::Depot::Resource::Appliance" if (!$resource->isa('Perf::Depot::Resource::Appliance'));
        return 1 if ($resource->found_in($appliances[$$self]));
        return 0;
    }

    sub add_appliance {
        my ($self, $resource) = @_;
        confess "You must specify an argument" if (!$resource);
        confess "Argument must be a Perf::Depot::Resource::Appliance" if (!$resource->isa('Perf::Depot::Resource::Appliance'));
        return if ($self->has_appliance($resource));
        push @{$appliances[$$self]}, $resource;
        $self->configure;
        return;
    }

    sub cleanup {
        my $self = shift();

        $self->stop();
    }
}

1;
