# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary
## @status   private
##
## @pod here

=head1 NAME

Resource::SANService::iSCSI - Representation of the iSCSI service on a
host

=head1 DESCRIPTION

This class represent the iSCSI service on a host
It is written using the "inside-out" object paradigm and the package Object::InsideOut specifically.

=over

=item References

=over

=item More on inside-out objects:      http://www.perlfoundation.org/perl5/index.cgi?inside_out_object

=item Object::InsideOut Documentation: http://search.cpan.org/dist/Object-InsideOut/lib/Object/InsideOut.pod

=back

=back

=head1 METHODS

=cut

package Perf::Depot::Resource::SANService::iSCSI;
{
    use strict;
    use warnings;
    use Carp qw( confess );
    use Tharn qw( $Log );
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Data::Dumper;
    use Object::InsideOut qw(Perf::Depot::Resource::SANService);

    sub start {
        my $self = $_[0];
        $self->get_host()->iscsi_start();
        return;
    }


    sub stop {
        my $self = shift();
        $self->get_host()->iscsi_stop();
        return;
    }


    sub configure {
        my $self = $_[0];

        # TODO: It would be nice to be able to bail
        # if the host is already configured

        $self->check_busy();

        ## Configure iscsi so the host sees the filers
        $self->get_host()->iscsi_configure($self->get_appliances());

        return;
    }


    sub cleanup {
        my $self = shift();
        my $host = $self->get_host();
        $host->san_cleanup('iscsi');
        $self->stop();
    }
}
1;
