# -*- perl -*-
# $Id: //depot/prod/test/main/ontap/rollingrock/lib/Perf/Flexscale.pm#11 $
#
## @author   jroussin@netapp.com, bshelton@netapp.com
## @summary  Convenience functions for FlexScale
## @status   public

package Perf::Flexscale;

use strict;
use warnings;
use Carp qw(confess);
use Tharn;
use Admin::License;
use Perf::Utils qw(as_string in_blocks in_bytes);
use Perf::Iomem qw(iomem_config_show);
use Aggregate qw(aggr_list);
use Perf::Iomem qw(iomem_config_disable
                   iomem_config_enable
                   iomem_config_show);
use Server;
use Perf::ModeAgnostic qw(say_wrap
                          filer_reboot_wrap
                          check_C_or_7_mode);
use vars qw($test $fc $fh $ch $aggr $vol $vols $stats $initial_stats @TC_LIST $filer);
use filer;
use List::MoreUtils qw(none);
use Filer::Version qw(are_we_ontapng);

use base qw(Exporter);
our @EXPORT_OK = qw(flexscale_is_licensed
                    pamii_license
                    flexscale_license
                    flexscale_unlicense
                    flexscale_option_enable
                    flexscale_option_lopri_blocks
                    flexscale_option_normal_data_blocks
                    flexscale_option_max_io_qdepth
                    flexscale_option_mbuf_inserts
                    flexscale_option_rewarm
                    flexscale_ecc_error_limit
                    flexscale_error_restart
                    flexscale_offline_panic
                    flexscale_waffinity
                    flexscale_blocks
                    flexscale_config
                    flexscale_status
                    flexscale_status_string
                    flexscale_ems_messages
                    flexscale_ems_files_shift
                    flexscale_invalidate
                    flexscale_lookup
                    change_all_cards
                    change_1_card_state
                    find_enabled_cards
                    find_disabled_cards
                    find_cards
                    find_card_type
                    find_card_type_with_size
                    ext_cache_pcs_size
                    change_tagstore_size
                    flexscale_option_pcs_high_res
                    flexscale_option_pcs_size
                    match_mib_cm_values
                    flexscale_reset_options
                    flexshare_status
                    get_filer_memory_size
                    get_filer_model_name
                    get_extcache_type
                    get_cm_values
                    get_snmp_values
                    compare_extcache_snmp
                    options_flexscale
                    card_status
                    card_size
                    flexscale_setflag_lopri_blocks
                    use_pamii_bootarg
                    ext_cache_enable_rewarm_bootarg
                    dbg_ext_cache_level_bootarg
                    flush_ec
                    ec_type
                    ec_size_gb
                    $test $fc $fh $ch $aggr $vol $filer);


# Check if FlexScale is licensed.

sub flexscale_is_licensed {
    my $fc   = shift;
    my $code = $Admin::License::Default::DEFAULT_LICENSES{filer}{flex_scale};

    my $cmd = 'license';
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    return 1 if $out =~ m/flex_scale\s+site\s+$code/;
    return 0 if $out =~ m/flex_scale\s+not\s+licensed/;

    confess "Unexpected output from '$cmd': $out";
}

# Add PAMII license.

sub pamii_license {
    my $fc = shift;
    # pamii is now Flash Cache which is a free license automatically on, so it is no longer needed to be enabled.

    return 1;
}

# Add FlexScale license.

sub flexscale_license {
    my $fc = shift;

    my $type = find_card_type($fc);
    logcomment("License Card : $type");

    ## if it is host create a connection object
    $fc =
        eval { $fc->isa('Connectrec') }
        ? $fc
        : connect( $fc . "#rsh" );
    if (( $type =~ /Flash Cache/ )) {
        logcomment("License Flash Cache");
        Admin::License::license( $fc, 'flash_cache' );
        # Library seems to have removed flash_cache since it's a free license now
        say ($fc, 'license add QLAHIJE');
    } elsif ( $type eq 'Performance Acceleration Module I' ) {
        Admin::License::license( $fc, 'flex_scale' );
    } else {
        Admin::License::license( $fc, 'ext_cache' )
            or confess 'Unable to license flex_scale';
    }

    return 1;
}

sub ext_cache_license {
    my $fc = shift;    #Host or Filer Connection Object
    ## if it is host create a connection object
    $fc =
        eval { $fc->isa('Connectrec') }
        ? $fc
        : connect( $fc . "#rsh" );
    Admin::License::license( $fc, 'ext_cache' )
        or Admin::License::license( $fc, 'flex_scale' )
        or confess 'Unable to license ext_cache';

    return 1;
}


# Remove FlexScale license.

sub flexscale_unlicense {
    my $fc = shift;    #Host or Filer Connection Object
    ## if it is host create a connection object
    $fc =
        eval { $fc->isa('Connectrec') }
        ? $fc
        : connect( $fc . "#rsh" );
           Admin::License::unlicense( $fc, 'flash_cache' )
        or Admin::License::unlicense( $fc, 'flex_scale' )
        or Admin::License::unlicense( $fc, 'ext_cache' )
        or confess 'Unable to unlicense flexscale';

    return 1;
}


# Return flexscale.enable option value, optionally setting
# it first if a second argument is supplied.

sub flexscale_option_enable {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_option($fc, 'enable', $val);
}



# Return flexscale.lopri_blocks option value, optionally setting
# it first if a second argument is supplied.

sub flexscale_option_lopri_blocks {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_option($fc, 'lopri_blocks', $val);
}


# Return flexscale.normal_data_blocks option value, optionally
# setting it first if a second argument is supplied.

sub flexscale_option_normal_data_blocks {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_option($fc, 'normal_data_blocks', $val);
}


# Return flexscale.max_io_qdepth option value, optionally
# setting it first if a second argument is supplied.

sub flexscale_option_max_io_qdepth {
    my $fc  = shift;
    my $val = shift;

    # max_io_qdepth option has been removed. Just returning value so major test rework
    # is not needed.
    return $val;





    # If new value defined, set it.

    if (defined $val) {

        # Validate value. Here we only ensure it's a string of digits.
        # In actuality, only a small range of values are valid. If called
        # with a value outside this range, we'll detect it when we verify
        # the new setting.

        confess "Invalid value for flexscale.max_io_qdepth: $val"
            if $val !~ m/^\d+$/;

        # Set value.

        my $cmd = "options flexscale.max_io_qdepth $val";
        my (undef, undef, $out) = say_wrap($fc, $cmd);

        confess 'Must license FlexScale before setting options'
            if $out =~ m/FlexScale is not licensed/;

        confess "Unexpected output from '$cmd': $out"
            if $out !~ m/partner node/ and $out ne "" and $out !~ $cmd;
    }

    # Return current setting.

    my $cmd = 'options flexscale.max_io_qdepth';
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    confess "Unexpected output from '$cmd': $out"
        if $out !~ m/flexscale\.max_io_qdepth\s+(\d+)/;

    my $new_val = $1;

    # If we attempted to set a new value, make sure we succeeded.

    if (defined $val) {
        confess "Could not set flexscale.max_io_qdepth to $val"
            if $new_val != $val;
    }

    return $new_val;
}


# Return flexscale.mbuf_inserts option value, optionally
# setting it first if a second argument is supplied.

sub flexscale_option_mbuf_inserts {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_option($fc, 'mbuf_inserts', $val);
}


# Return flexscale.rewarm option value, optionally
# setting it first if a second argument is supplied.

sub flexscale_option_rewarm {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_option($fc, 'rewarm', $val);
}


# Return wafl_ext_cache_ecc_limit flag value, optionally setting
# it first if a second argument is supplied.

sub flexscale_ecc_error_limit {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_flag($fc, 'wafl_ext_cache_ecc_limit', $val);
}


# Return wafl_ext_cache_error_restart flag value, optionally setting
# it first if a second argument is supplied.

sub flexscale_error_restart {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_flag($fc, 'wafl_ext_cache_error_restart', $val);
}


# Return wafl_ext_cache_offline_panic flag value, optionally setting
# it first if a second argument is supplied.

sub flexscale_offline_panic {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_flag($fc, 'wafl_ext_cache_offline_panic', $val);
}


# Return wafl_ext_cache_waffinity flag value, optionally setting
# it first if a second argument is supplied.

sub flexscale_waffinity {
    my $fc  = shift;
    my $val = shift;

    return _flexscale_flag($fc, 'wafl_ext_cache_waffinity', $val);
}


# Returns number of 4k blocks in the FlexScale cache, as reported
# by the ext_cache_obj:ec0:blocks counter.

sub flexscale_blocks {
    my $fc = shift;    # Filer Connection Object or Host

    # Query the counter for the size in 4k blocks.

    my $cmd = 'priv set -q diag; stats show ext_cache_obj:ec0:blocks';
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    confess "Unexpected output from '$cmd': $out"
        if $out !~ m/ext_cache_obj:ec0:blocks:(\d+)/;

    return $1;
}


# Return the sysconfig entries for FlexScale cards in an arrayref.
# Each entry in the array is a hashref of fields in the entry.

sub flexscale_config {
    # For now this is a simple wrapper around Iomem::iomem_config_show
    return iomem_config_show(@_);
}


# Return flexscale status as reported by the ext_cache::state counter.
#
# Here are the possible values:
#
#   0 - Permanentaly disable at boot
#   1 - Unknown state (only used during boot)
#   2 - Uninitialized
#   3 - Initialized (ready to go but not turned on)
#   4 - Active (turned on)
#   5 - Error (cache turned off on error)
#   6 - Error offline pending (fatal error detected, will be offlined shortly)

sub flexscale_status {
    my $fc = shift;

    my $cmd = 'stats show ext_cache::state';
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    confess "Unexpected output from '$cmd': $out"
        if $out !~ m/ext_cache:ext_cache:state:(\d+)/;

    return $1;
}


# Return flexscale status as reported by the ext_cache::state-string counter.

sub flexscale_status_string {
    my $fc = shift;

    my $cmd = 'stats show ext_cache::state-string';
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    confess "Unexpected output from '$cmd': $out"
        if $out !~ m/ext_cache:ext_cache:state-string:(\S+)/;

    return $1;
}


# Return arrayref of current FlexScale EMS messages.

sub flexscale_ems_messages {
    my $fc    = shift;
    my $regex = shift || '<extCache_.*?\/>';
    my @msgs;

    my $count;
    my (undef, undef, $out) =
        say_wrap($fc, 'priv set -q diag; rdfile /etc/log/ems', -timeout, 600, -maxidle, 180);
    while ($out =~ /No such file or directory/ && $count < 10) {
        (undef, undef, $out) =
            say_wrap($fc, 'priv set -q diag; rdfile /etc/log/ems', -timeout, 600, -maxidle, 180);
        $count++;
        Tharn::snooze(30);
    }
    if ($count == 10) {
        (undef, undef, $out) =
            say_wrap($fc, 'priv set -q diag; rdfile /etc/log/ems.0', -timeout, 600, -maxidle, 180);
    }
    push @msgs, $&
        while ($out =~ m/$regex/sg);

    return \@msgs;
}

# Move ems files from ems.x to ems.x+1 for a clean slate for new tests.

sub flexscale_ems_files_shift {
    my $fc = shift;

    my $ems_file_cnt = 0;
    my (undef, undef, $out) = say_wrap($fc, 'priv set -q diag; ls /etc/log');
    while ($out =~ m/ems/sg) {
        $ems_file_cnt++;
    }

    my $mroot = '';
    if (are_we_ontapng($fc)) {
        $mroot = '/mroot';
    }
    for(; $ems_file_cnt >= 1; $ems_file_cnt--) {
        my $file_name_cnt = $ems_file_cnt - 1;
        my (undef, undef, $out) =
            say_wrap($fc, "priv set -q diag; mv " . $mroot . "/etc/log/ems.$file_name_cnt " . $mroot . "/etc/log/ems.$ems_file_cnt");
    }
    (undef, undef, $out) =
        say_wrap($fc, 'priv set -q diag; mv ' . $mroot . '/etc/log/ems ' . $mroot . '/etc/log/ems.0');

    return;
}

# Invalidate blocks from the victim cache

sub flexscale_invalidate {
    my $fc = shift;
    my $aggrname = shift;
    my @aggrs;
    if ($aggrname){
        push @aggrs, $aggrname;
    }
    else{
        @aggrs = Aggregate::aggr_list($fc);
        if (!@aggrs){
            confess "No aggregates found";
        }
    }

    for my $aggr (@aggrs){
        my (undef, undef, $out) = say_wrap($fc, "priv set -q diag; wafl ec invalidate $aggr all");
        if ($out){
            confess "Unable to invalidate blocks for aggr '$aggr':$out";
        }
    }

    return;
}


# Lookup pvbn in EC

sub flexscale_lookup {
    my $fc = shift;
    my $aggr = shift;
    my $pvbn = shift;

    my (undef, undef, $out) = say_wrap($fc, "priv set -q diag; wafl ec lookup $aggr $pvbn");

    # Example:
    # crane*> wafl ec lookup aggr1 42
    # Aggregate: aggr1, pvbn: 42 found at ecbn 341, cache object 0
    #  Tag detail: pvol_indx:2 ref:0 lock:0 used:1 fbn:18446744073709551615 btid:0
    #
    # If you read it this way, will it pull that pvbn into BC as well?
    #
    # Nah, this just does a tagstore lookup, it doesn't actually read it in.

    # fbn might be bad?
    # I'm not sure how the fbn could be bad but still building ok.  Haven't looked at it in detail.
    #
    # Ecbn is good to know, it also maps to the fal address space.
    #
    # Cache object 0 is a throwback to the old PCS days when you could have multilevel caches.  Right now it's always going to be 0.  In theory we will one day support multiple caches again..  we won't implement it the same way Rob did but we may reuse the numbering scheme.
    #
    # The refcount is interesting.  That's how many times the tag has been read since insertion.  'used' should be consistent with having valid data in the tag.  'locked' indicates it's undergoing read or write i/o.  You should never see 'locked' with Virgo & writes, only reads.
    #
    # btid = buftree id.  Might be interesting with clone volumes.

    my %ec_hash;
    if ($out !~ /found at ecbn \d+/) {
        return %ec_hash;
    }
    else {
        $out =~ /found at ecbn (\d+), cache object (\d+)\s+pvol_indx:(\d+) ref:(\d+) lock:(\d+) used:(\d+) fbn:(\d+) btid:(\d+)/;
        my $ec_hash->{ecbn} = $1;
        my $ec_hash->{cache_obj} = $2;
        my $ec_hash->{pvol_indx} = $3;
        my $ec_hash->{ref} = $4;
        my $ec_hash->{lock} = $5;
        my $ec_hash->{used} = $6;
        my $ec_hash->{fbn} = $7;
        my $ec_hash->{btid} = $8;
        return $ec_hash;
    }
}


# Private helper function for getting/setting options.

sub _flexscale_option {
    my $fc  = shift;
    my $opt = shift;
    my $val = shift;

    my $opt_cmd;

    my (undef, undef, $version) = say_wrap($fc, 'version');
    if ($version !~ /7\.2|gordonbiersch/) {
        $opt_cmd = 'flexscale';
    }
    else {
        $opt_cmd = 'ext_cache';
        # In GB, there is no 'pcs' value, only 'on' as there are no Pisces cards in GB
        if ($opt eq 'enable' && ($val eq 'pcs' || $val eq 'PCS')) {
            $val = 'on';
        }
    }
    # If new value defined, set it.

    if (defined $val) {

        # Validate value.
        if ( $opt eq 'pcs_size' ) {

            # Set value after converting it from bytes to GB.
            $val = in_bytes($val);
            $val = int( $val / ( 1024 * 1024 * 1024 ) );
        }
        else {
            confess "Invalid value for $opt_cmd.$opt: $val"
                if $val !~ m/^(?:1|on|ON|yes|YES|true|TRUE|pcs|PCS|
                                 0|off|OFF|no|NO|false|FALSE)$/x;
        }

        # Set value.

        my $cmd = "options $opt_cmd.$opt $val";
        my (undef, undef, $out) = say_wrap($fc, $cmd);

        confess 'Must license FlexScale before setting options'
            if $out =~ m/FlexScale is not licensed/;

        confess "Unexpected output from '$cmd': $out"
            if $out !~ m/partner node/
               and $out !~ m/FlexScale PCS .* only are available/
               and $out !~ m/$cmd/
               and $out ne q{};
    }

    # Return current setting.

    my $cmd = "options $opt_cmd.$opt";
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    confess "Unexpected output from '$cmd': $out"
        if $out !~ m/$opt_cmd\.$opt\s+(on|off|pcs|\d+\S+)/;

    my $new_val = $1;

    if ( ( defined $val ) && ( $opt eq 'pcs_size' ) ) {
        $val = $val . 'GB';
        confess "Could not set flexscale.pcs_size to $val"
            if $new_val ne $val;
    }
    return $new_val;
}


# Private helper function for getting/setting flags.

sub _flexscale_flag {
    my $fc   = shift;
    my $flag = shift;
    my $val  = shift;

    # If new value defined, set it.

    if (defined $val) {

        # Validate value.

        confess "Invalid value for flag: $val"
            if $val !~ m/^\d+$/;

        # Set value.

        my $cmd = "priv set -q diag; setflag $flag $val";
        my (undef, undef, $out) = say_wrap($fc, $cmd);

        confess "Unexpected output from '$cmd': $out"
            if $out ne q{};
    }

    # Return current setting.

    my $cmd = "priv set -q diag; printflag $flag";
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    confess "Unexpected output from '$cmd': $out"
        if $out !~ m/$flag = (\d+)/;

    return $1;
}


sub change_1_card_state {
    #determine card slots
    my %defs  = (new_state => 'Disabled',            # Default to Disabled
                 card_type => 'Performance Acceleration Module I',  # Pisces
                 card_slot => -1,
                 persistent => 1,
                 );
    my %opts  = (%defs, @_);

    if (defined($opts{fc})) {
        $fc = $opts{fc};
    }

    my @card_list;
    if ( $opts{card_slot} >= 0 ) {
        logcomment("\n Card slot is avilable and is $opts{card_slot}");
        logcomment(
                  "Changing card $opts{card_slot} state to $opts{new_state}");
        if ( $opts{new_state} eq "Enabled" ) {
            if ( card_status( $opts{fc} ) eq "Disabled" ) {
                my $cmd = "priv set -q diag; iomem config enable";
                $cmd .= " -c $opts{card_slot}";
                if ( $opts{persistent} ) {
                    $cmd .= " -P";
                }
                my ( undef, undef, $out ) = say_wrap( $fc, $cmd );

                if ( $out !~ m/Card enabled/ ) {
                    confess "Unexpected output from '$cmd': $out";
                }
                return ( $opts{card_slot} );
            }
            else {
                logcomment("Card $opts{card_slot} is already enabled");
                return (-1);
            }
        }
        else {
            if ( card_status( $opts{fc} ) eq "Enabled" ) {
                my $cmd = "priv set -q diag; iomem config disable";
                $cmd .= " -c $opts{card_slot}";
                if ( $opts{persistent} ) {
                    $cmd .= " -P";
                }
                my ( undef, undef, $out ) = say_wrap( $fc, $cmd );

                if ( $out !~ m/Card disabled/ ) {
                    confess "Unexpected output from '$cmd': $out";
                }
                return ( $opts{card_slot} );
            }
            else {
                logcomment("Card $opts{card_slot} is already enabled");
                return (-1);
            }
        }
    }
    else {
        my @card_list;
        if ($opts{new_state} eq 'Enabled') {
            @card_list = find_disabled_cards(%opts);
        }
        else {
            @card_list = find_enabled_cards(%opts);
        }
        if ($#card_list >= 0) {
            my $rnd_card = @card_list[int(rand($#card_list))];
            if ($opts{new_state} eq "Disabled") {
                logcomment("Disabling card $rnd_card");
                my $cmd = "priv set -q diag; iomem config disable";
                $cmd .= " -c $rnd_card";
                if ( $opts{persistent} ) {
                    $cmd .= " -P";
                }
                my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
                if ( $out !~ m/Card disabled/ ) {
                    confess "Unexpected output from '$cmd': $out";
                }
                return $rnd_card;
            }
            else {
                logcomment("Enabling card $rnd_card");
                my $cmd = "priv set -q diag; iomem config enable";
                $cmd .= " -c $rnd_card";
                if ( $opts{persistent} ) {
                    $cmd .= " -P";
                }
                my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
                if ( $out !~ m/Card enabled/ ) {
                    confess "Unexpected output from '$cmd': $out";
                }
                return $rnd_card;
            }
        }
        else {
            logcomment("There are no more cards to change to $opts{new_state}.");
            return (-1);
        }
    }
}


sub change_all_cards {
    my %defs  = (new_state => 'Disabled',            # Default to Disabled
                 card_type => 'Performance Acceleration Module I',  # Pisces
                 );
    my %opts  = (%defs, @_);

    my $card_count = 0;
    logcomment("Change all $opts{card_type} cards to $opts{new_state}");
    while (change_1_card_state(%opts) != -1) {
        $card_count++;
    }
    logcomment("Changed $card_count $opts{card_type} cards to $opts{new_state}.");
    return ($card_count);
}



# Find ext cache cards in filer and their state

sub find_cards {
    my %defs  = (card_type => 'Performance Acceleration Module I',  # Pisces
                 );
    my %opts  = (%defs, @_);

### Changes below to handle Sysconfig not ready for Cluster mode
#    my $cfg = Sysconfig->new($opts{filer});
#    my @cards = $cfg->all_slots();
    if (defined($opts{fc})) {
        $fc = $opts{fc};
    }
    my %slots;
    my ($slot_num, $state);
    my (undef,undef,$sysconfig) = say_wrap($fc,'sysconfig -v');
    #my @lines = split(/\r|\n/, $sysconfig);
    my $cards = 0;
    my @lines = split(/slot\s+/, $sysconfig);
    foreach my $line (@lines) {

    if ($line =~ /(\d): $opts{card_type}.*\s+State:\s+(Enabled|Disabled)/) {
               $slots{$1} = $2 ;

            logcomment("found slot $1 and is $slots{$1}");
            $cards++;
        }
    }
    my $str = ($cards > 1) ? " cards" : " card";
    logcomment("found " . $cards . $str );

#    my $type;
#    for (my $slot_num = 0; $slot_num <= ($#cards + 1); $slot_num++) {
#        # Command will fail if no card is in a slot, so ignore failures
#        eval {
#            if ($cfg->slot($slot_num, 'type') =~ $opts{card_type}) {
#                $slots{$slot_num} = $cfg->slot($slot_num, 'state');
#                logcomment("$opts{card_type} in slot = $slot_num and is $slots{$slot_num}");
#            }
#        }; warn $@ if $@;
#    }
    # return hash with slot/status
    return %slots;
}

# Find ext cache card type (Pisces = PAM I, Virgo = PAM II)

sub find_card_type {
    my $fc = shift;    # Filer Connection Object or Host

    my (undef,undef,$sysconfig) = say_wrap($fc,'sysconfig -v');
    my $type = 'none';
    my @lines = split(/slot\s+/, $sysconfig);
    foreach my $line (@lines) {
        if (($line =~ /\d: (Performance Accelerat(ion|or) Module.*)\s+State:\s+(Enabled)/)||
            ($line =~ /\d: (Flash Cache.*)\s+State:\s+(Enabled)/)) {
            my $card_type = $1;
            if ($card_type =~ /Flash Cache \((\d+)(.*)\)/) { #
            #    $type = "Flash Cache \($1$2\)";
                $type = "Flash Cache";
                logcomment("Using card type: $type");
                return $type;
            }
            elsif ($card_type =~ /Performance Accelerat(ion|or) Module I/) {
                $type = 'Performance Acceleration Module I';
                logcomment('Found Performance Acceleration Module I card.');
            }
            else {
                logcomment('No PAM card found.');
            }
        }; warn $@ if $@;
    }
    logcomment("Using card type: $type");
    return $type;
}

sub card_size {
    my $card_type = shift;

    if ($card_type =~ /512/) {
        return in_bytes('512g');
    }
    elsif ($card_type =~ /256/) {
        return in_bytes('256g');
    }
    else {
        return in_bytes('16g');
    }
}

# Display card's status of Enabled/Disabled

sub card_status {
    my $fc = shift;    #  Filer Connection Object or Filer Name

    my ( undef, undef, $sysconfig ) = say_wrap( $fc, "sysconfig -v" );
    my $model;
    my $slot;
    my $state;
    my @lines = split( /slot\s+/, $sysconfig );
    foreach my $line (@lines) {
        if ( $line =~ /(\d)\: (Performance Acceleration Module I).*State\:\s+(\w+).*Memory Size\:\s*/s) {
            logcomment(
                   "using card type : $2 with slot number :$1 and state: $3");
            $slot  = $1;
            $state = $3;
        }
        elsif ( $line =~ /(\d)\: (Flash Cache\s\((256|512|1)(.*))\)\s+ State:\s+(\w+)$/s) {
            logcomment("using card type : $2 with slot number : $1 and state: $3");
            $slot  = $1;
            $state = $3;
        }
        else {
            logcomment("Unknown card type");
        }
    }
    logcomment("Card is in slot: $slot and it's state is : $state");
    return $state;

}


sub find_enabled_cards {
    my %opts  = @_;

    my %slots = find_cards(%opts);
    my @enabled_cards;
    foreach my $card (keys %slots) {
        if ($slots{$card} eq 'Enabled') {
            push @enabled_cards, $card;
        }
    }
    return @enabled_cards;
}


sub find_disabled_cards {
    my %opts  = @_;

    my %slots = find_cards(%opts);
    my @disabled_cards;
    foreach my $card (keys %slots) {
        if ($slots{$card} eq 'Disabled') {
            push @disabled_cards, $card;
        }
    }
    return @disabled_cards;
}


sub flexscale_setflag_lopri_blocks {
    my $fc  = shift;    # Filer Connection Object or Filer Name
    my $val = shift;

    if (defined $val) {

        # Validate value.

        confess "Invalid value for wafl_ext_cache_oppor_lopri: $val"
            if ($val < 0 || $val > 100);

        # Set value for opportunistic caching.

        my $cmd = "priv set -q test; setflag wafl_ext_cache_oppor_lopri $val";
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd );

        confess "Unexpected output from '$cmd': $out"
            if $out !~ m/partner node/
               and $out !~ m/$cmd/
               and $out ne q{};
    }

    # Return current setting.

    my $cmd = "priv set -q test; printflag wafl_ext_cache_oppor_lopri";
    my (undef, undef, $out) = say_wrap($fc, $cmd);

    confess "Unexpected output from '$cmd': $out"
        if $out !~ m/wafl_ext_cache_oppor_lopri = (\d+)/;

    return $1;
}


sub use_pamii_bootarg {
    my $fc  = shift;    # Filer Connection Object or Filer Name
    my $val = shift;

    my $out;
    my $cmd1 = "priv set -q test;bootargs set ext-cache-use-pam-ii? $val";
    my $cmd2 = "priv set -q test;bootargs print ext-cache-use-pam-ii?";
    if ($val) {
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd1 );
        filer_reboot_wrap($fc);
    } else {
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd2 );
        confess "Unexpected output: $out"
           if $out !~ /ext-cache-use-pam-ii\? == (false|true)/;
    }
}


### NOTE: This will change to ext-cache-disable-rewarm? when they turn it on by default in RR
sub ext_cache_enable_rewarm_bootarg {
    my $fc  = shift;    # Filer Connection Object or Filer Name
    my $val = shift;

    my $out;
    my $cmd1 = "priv set -q test;bootargs set ext-cache-enable-rewarm? $val";
    my $cmd2 = "priv set -q test;bootargs print ext-cache-enable-rewarm?";
    if ($val) {
        (undef,undef,$out) = say_wrap( $fc, $cmd2 );
        confess "Unexpected output: $out"
           if $out !~ /ext-cache-enable-rewarm\? == (false|true|undefined)/;
        if ($1 ne $val) {
            (undef,undef,$out) = say_wrap( $fc, $cmd1 );
            confess "Unexpected output: $out"
               if $out !~ /ext-cache-enable-rewarm\? == (false|true|undefined)/;
            filer_reboot_wrap($fc);
        }
    }
    (undef,undef,$out) = say_wrap( $fc, $cmd2 );
    confess "Unexpected output: $out"
       if $out !~ /ext-cache-enable-rewarm\? == (false|true|undefined)/;
    return $1;
}

sub dbg_ext_cache_level_bootarg {
    my $fc  = shift;    # Filer Connection Object or Filer Name
    my $val = shift;
    # where meaningful values of $val are currently { 0, 10, 20, 30, 40, 50 }

    my $out;
    my $cmd1 = "priv set -q test;bootargs set dbg_ext_cache_level $val";
    my $cmd2 = "priv set -q test;bootargs print dbg_ext_cache_level";
    if ($val) {
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd1 );
        filer_reboot_wrap($fc);
    } else {
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd2 );
        confess "Unexpected output: $out"
           if $out !~ /dbg_ext_cache_level == (\d+)/;
    }
}

sub change_tagstore_size {

    my $fc    = shift;    # Filer Connection Object or Filer Name
    my $value = shift;
    confess "The tagstore value is not within the range"
        if ( ( $value < 0 ) || ( $value > 100 ) );

    if ( defined $value ) {
        my $cmd =
            "priv set test; bootargs set ext-cache-tagstore-oversize-percent $value";
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
        confess "Unexpected output from $cmd : $out"
            if $out !~ /ext-cache-tagstore-oversize-percent == (\d+)/;
        filer_reboot_wrap($fc);
    } else {
        my $cmd =
            "priv set test; bootargs print ext-cache-tagstore-oversize-percent";
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
        confess "Unexpected output from $cmd : $out"
            if $out !~ /ext-cache-tagstore-oversize-percent == (\d+)/;

        return $1;
    }

}

sub ext_cache_pcs_size {

    my $fc  = shift;    # Filer Connection Object or Host
    my $val = shift;
    my $default_value;

    if ( defined $val ) {

        #Check if the value is defined or not. Only digits are allowed

        confess "Invalid value for ext_cache_pcs_size: $val"
            if $val !~ m/^\d+$/;

        # Set value.
        logcomment("Set the new value");
        my $cmd = "priv set test; bootargs set ext-cache-pcs-size $val";
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
        confess "Unexpected output for $cmd : $out"
            if $out ne q{}
                and $out !~ /ext-cache-pcs-size == (\d+)/;

        $default_value = $1;
        logcomment("Reboot the Filer");
        filer_reboot_wrap($fc);
    }

    # Return current setting.
    my $pcs_size;
    my $cmd = 'priv set test; bootargs print ext-cache-pcs-size';

    my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
    confess "Unexpected output from $cmd: $out"
        if $out !~ /ext-cache-pcs-size\s==\s(\d+)/;
    $pcs_size = $1;

    # If we attempted to set a new value, make sure we succeeded.

    if ( defined $val ) {
        confess "Could not set ext-cache-pcs-size to $val"
            if $pcs_size != $val;
    }
    logcomment("pcs size : $pcs_size");
    return $pcs_size;

}

sub flexscale_option_pcs_high_res {

    my $fc  = shift;    # Filer Connection Object or Host
    my $val = shift;

    return _flexscale_option( $fc, 'pcs_high_res', $val );
}

sub flexscale_option_pcs_size {

    my $fc  = shift;    # Filer Connection Object or Host
    my $val = shift;
    return _flexscale_option( $fc, 'pcs_size', $val );
}

sub flexscale_reset_options {
    my %opt = @_;
    flexscale_option_enable( $opt{fc}, 'on' );
    flexscale_option_lopri_blocks( $opt{fc}, 'off' );
    flexscale_option_normal_data_blocks( $opt{fc}, 'on' );
    logcomment("card_type : $opt{card_type}");
    if ( $opt{card_type} =~ m/SPCS/ ) {
        flexscale_option_pcs_high_res( $opt{fc}, 'off' );
        flexscale_option_pcs_size( $opt{fc}, '64GB' );
    }
}

#Check the status of Flexshare
#Returns the Flexshare status

sub flexshare_status {

    my $fc = shift;    # Filer Connection Object or Host
    my $flexshare_status;
    my ( undef, undef, $out ) = say( $fc, "priv set diag; priority show" );

    confess "Unexpected value returned for the command"
        if $out eq q{};
    if ( $out =~ /Priority scheduler is stopped\./s ) {
        $flexshare_status = 1;
    } else {
        $flexshare_status = 2;
    }
    return $flexshare_status;
}

#Calculate average of readio and writeio counter values
sub get_average($) {
    my $value = shift;
    $value =~ m/(\d+)\/(\d+)/;
    my $value1    = $1;
    my $value2    = $2;
    my $avg_value = $value1 / $value2;
    $avg_value = sprintf "%.0f", $avg_value;
    return $avg_value;
}

#Function to retrieve the ExtCache Type and Subtype
# Pass the connecrec object  or host and return an array reference
sub get_extcache_type {

    my $fc    = shift;    # Filer Connection Object or Host
    my @array = ();

    my $val = flexscale_option_enable($fc);
    if ( $val eq 'off' ) {
        push( @array, ( 'none', 'none' ) );
    } else {
        my $cmd = 'priv set -q diag; stats show -R ext_cache_obj:ec0:type';
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
        confess "Unexpected output from $cmd : $out "
            if $out !~ /ext_cache_obj:ec0:type:(IOMEM|IOMEM-FLASH|^PCS|SPCS|None|none)/;

        $out = $1;
        print "\n EXT CACHE VALUE IS : $out";
        push( @array, $out );

        $cmd = 'priv set -q diag; stats show -R ext_cache_obj:ec0:media';
        my ( undef, undef, $out ) = say_wrap( $fc, $cmd );
        confess "Unexpected output from $cmd : $out "
            if $out !~ /ext_cache_obj:ec0:media:(DRAM|Flash|None|NONE)/;

        $out = $1;
        push( @array, $out );
    }
    return \@array;
}

#Function to get the memory size of the Filer
sub get_filer_memory_size {

    my $fc = shift;    # Filer Connection Object or Host
    my ( undef, undef, $out ) = say_wrap( $fc, "sysconfig -v" );
    $out =~ m/System Board.*Memory Size\:\s*(\d+).*NVRAM\s/s;
    my $output = $1;
    return $output;
}

sub get_filer_model_name {

    my $fc = shift;    # Filer Connection Object or Host
    my ( undef, undef, $out ) = say_wrap( $fc, "sysconfig -v" );
    $out =~ m/System Board.*Model Name\:\s*(FAS\d\d\d\d).*Part Number\:\s*/s;
    my $output = $1;
    logcomment("Model Name : $output");
    return $output;
}

#Function to get SNMP values of the known counters

sub get_snmp_values {
    my %opt = @_;
    my %snmp_counter;
    if ( $opt{snmp_setup} == 1 ) {
        %snmp_counter = $opt{statsextcache_obj}->snmp_walk( $opt{mib_hash} );
    } else {
        confess "Error occured during snmp setup for Ext Cache Obj";
    }
    logcomment( "SNMP COUNTER values" . as_string( \%snmp_counter ) );
    return %snmp_counter;
}

# Display the flexscale options for SPCS, Pisces and Virgo

sub options_flexscale {
    my $fc         = shift;    # Filer Connection Object or Host
    my $cache_type = shift;
    my $cache_type = defined($cache_type) ? $cache_type : 'SPCS';
    my %flexscale_options;
    $flexscale_options{'status_string'} = flexscale_status_string($fc);
    $flexscale_options{'lopri_blocks'}  = flexscale_option_lopri_blocks($fc);
    $flexscale_options{'normal_dblocks'} =
        flexscale_option_normal_data_blocks($fc);
    if ( $cache_type =~ m/SPCS/ ) {
        $flexscale_options{'pcs_high_res'} =
            flexscale_option_pcs_high_res($fc);
        $flexscale_options{'pcs_size'} = flexscale_option_pcs_size($fc);
    }
    return \%flexscale_options;

}

sub compare_extcache_snmp {
    my %opt = @_;
    $opt{snmp_counter}->{'extcache64Uptime'} =
        $opt{ext_counter}->{'extcache64Uptime'};

    if ( (    ( $opt{ext_counter}->{'extcacheSubType'} eq 'DRAM' )
           && ( $opt{snmp_counter}->{'extcacheSubType'} eq 'PAM1' ) )
         || (    ( $opt{ext_counter}->{'extcacheSubType'} eq 'Flash' )
              && ( $opt{snmp_counter}->{'extcacheSubType'} eq 'PAM2' ) )
         || (    ( $opt{ext_counter}->{'extcacheSubType'} eq 'None' )
              && ( $opt{snmp_counter}->{'extcacheSubType'} eq 'none' ) )
       )
    {

        $opt{snmp_counter}->{'extcacheSubType'} =
            $opt{ext_counter}->{'extcacheSubType'};

    }

    if (    ( $opt{ext_counter}->{'extcache64Size'} == 0 )
         || ( $opt{snmp_counter}->{'extcache64Usedsize'} == 0 ) )
    {
        $opt{snmp_counter}->{'extcache64Usedsize'} = 0;
    } else {
        $opt{snmp_counter}->{'extcache64Usedsize'} =
            int( ( $opt{snmp_counter}->{'extcache64Usedsize'} /
                       $opt{ext_counter}->{'extcache64Size'}
                 ) * 100
               );
    }
    my $test_status = 'BAD';

    foreach my $key1 ( keys %{ $opt{ext_counter} } ) {

        #Check if value is defined for the given key
        if ( defined $opt{ext_counter}->{$key1} ) {

            #  If defined compare the value with the snmp_hash
            logcomment(
                "key value is : $key1 and counter value is : $opt{ext_counter}->{$key1}"
            );
            logcomment(
                "key value is : $key1 and snmp value is : $opt{snmp_counter}->{$key1} "
            );
            if (    ( $opt{ext_counter}->{$key1} =~ m/\d+/ )
                 && ( $opt{snmp_counter}->{$key1} =~ m/\d+/ ) )
            {
                if (    ( $opt{ext_counter}->{$key1} != 0 )
                     && ( $opt{snmp_counter}->{$key1} != 0 ) )
                {

                    my $diff =
                        ( $opt{snmp_counter}->{$key1} -
                          $opt{ext_counter}->{$key1} ) /
                        ( $opt{ext_counter}->{$key1} );
                    logcomment("Diff value : $diff");
                    if ( ( $diff == 0 ) || ( $diff <= 0.01 ) ) {
                        logcomment("Diff value is within range");
                        $test_status = 'GOOD';
                    } else {
                        logcomment("Diff value $diff is out of range");
                        $test_status = 'BAD';
                        return $test_status;
                    }
                } elsif (    ( $opt{ext_counter}->{$key1} == 0 )
                          && ( $opt{snmp_counter}->{$key1} == 0 ) )
                {
                    logcomment("Ext_Counter and SNMP_Counter are equal");
                    $test_status = 'GOOD';
                } else {
                    logcomment(
                        "Values returned by CM : $opt{ext_counter}->{$key1} or SNMP : $opt{snmp_counter}->{$key1} are incorrect"
                    );
                    $test_status = 'BAD';
                    return $test_status;

                }
            } elsif ( ( $opt{ext_counter}->{$key1} =~ m/(\s?[a-zA-Z]+[\d]?)/ )
                && ( $opt{snmp_counter}->{$key1} =~ m/(\s?[a-zA-Z]+[\d]?)/ ) )
            {
                if ( $opt{snmp_counter}->{$key1} =~ m/$1/ ) {

                    logcomment(
                             "$key1 values for ext_cache and snmp are equal");
                    $test_status = 'GOOD';
                } else {
                    logcomment(
                         "$key1 values for ext_cache and snmp are not equal");
                    $test_status = 'BAD';
                    return $test_status;
                }
            } else {
                $test_status = 'BAD';
                logcomment(
                    "Value of $test_status if failed : $key1 with values $opt{ext_counter}->{$key1} "
                        . " and $opt{snmp_counter}->{$key1}" );
                return $test_status;
            }

        } else {
            confess " Value is not defined for $key1";
        }
    }

    logcomment("Test status : $test_status");
    return $test_status;
}

sub find_card_type_with_size {

    my $fc = shift;    # Filer Connection Object or Host

    my ( undef, undef, $sysconfig ) = say_wrap( $fc, 'sysconfig -v' );
    my $type = 'none';
    my @lines = split( /slot\s+/, $sysconfig );
    foreach my $line (@lines) {
        if (($line =~ /\d: (Performance Accelerat(ion|or) Module.*)\s+State:\s+(Enabled)/)||
            ($line =~ /\d: (Flash Cache.*)\s+State:\s+(Enabled)/)){
            my $card_type = $1;
            if ( $card_type =~/Flash Cache \((\d+)(.*)\)/ ) {
                $type = "Flash Cache \($1$2\)";
                logcomment("Using card type : $type");
                return $type;

            } elsif ($card_type =~ /Performance Accelerat(ion|or) Module I$/ ) {
                $type = 'Performance Acceleration Module I';
            } else {
                logcomment('No PAM card found.');
            }
        }
        warn $@ if $@;
    }
    logcomment("Using card type: $type");
    return $type;

}

sub calculate_cm_counter_value {

    my %opt = @_;
    my $counter_value;
    $counter_value = $opt{value};
    my $key = $opt{key};
    if ( $counter_value eq '0/0' ) {
        $opt{ext_counter_hash}->{$key} = 0;
    } else {
        $opt{ext_counter_hash}->{$key} = get_average( $opt{value} );
    }
    return %{ $opt{ext_counter_hash} };
}

sub get_cm_values {

    my %opt            = @_;
    my %sample_counter = %{ $opt{sample} };
    my %ext_counter    = %{ $opt{counter_hash} };

    my $ext_array_ref = get_extcache_type( $opt{fc} );

    # Hash is populated with extcache type and subtype for future comparisons
    $ext_counter{'extcacheType'}    = $ext_array_ref->[0];
    $ext_counter{'extcacheSubType'} = $ext_array_ref->[1];
    my $key1 = 'extcache64Size';
    my $key2 = 'extcache64TotalBuffers';

    while ( my ( $key, $value ) = each(%sample_counter) ) {
        my $counter_value;

        if ((  $key =~
               m/(extcache64Uptime|extcache64Usedsize|extcache64Hits|extcache64Misses|extcache64Inserts|extcache64Evicts|extcache64Invalidates|extcache64BlocksRef0|extcache64HitNormalL0|extcache64MetaData|extcache64ReadLatency|extcache64WriteLatency|extcache64RCLength|extcache64WCLength|ec_blocks)/
            )&& ( $opt{cache_status_string_flag} == 1 )) {

            $counter_value = 0;
            if ( $key =~ /ec_blocks/ ) {
                my $counter_value2 = 0;
                $ext_counter{$key1} = $counter_value;
                $ext_counter{$key2} = $counter_value2;
            } else {
                $ext_counter{$key} = $counter_value;
            }

        } elsif ( $opt{cache_status_string_flag} == 2 ) {
            if ( $key =~
                m/(extcache64Uptime|extcache64Usedsize|extcache64Hits|extcache64Misses|extcache64Inserts|extcache64Evicts|extcache64Invalidates|extcache64BlocksRef0|extcache64HitNormalL0|extcache64MetaData)/
               ) {

                $ext_counter{$key} = $value;
            } elsif ( $key =~
                m/(extcache64ReadLatency|extcache64WriteLatency|extcache64RCLength|extcache64WCLength)/) {

                calculate_cm_counter_value(cache_status_string_flag =>
                                               $opt{cache_status_string_flag},
                                           key              => $key,
                                           value            => $value,
                                           ext_counter_hash => \%ext_counter,
                                          );
            } elsif ( $key =~ /ec_blocks/ ) {

                my $counter_value2;

                # Converting blocks into bytes to match SNMP values
                $counter_value = $value * 4 * 1024;

               #Product of set and associativity for Pisces/Virgo returns the total buffers to match SNMP values

                my $set           = $opt{sample}->{'ec_set'};
                my $associativity = $opt{sample}->{'ec_associativity'};

                if ( $ext_counter{'extcacheType'} eq "IOMEM" ) {
                    $counter_value2 = $set * $associativity;
                }

                elsif ( $ext_counter{'extcacheType'} eq "IOMEM-FLASH" ) {
                    $counter_value2 = ( $set * $associativity ) / 2;
                }

                else {
                    $counter_value2 = $opt{sample}->{'ec_blocks'};
                }
                $ext_counter{$key1} = $counter_value;
                $ext_counter{$key2} = $counter_value2;

            }

        }
    }
    $ext_counter{'extcacheOptions'} = $opt{flex_value};
    logcomment( "Ext Counter values" . as_string( \%ext_counter ) );
    return %ext_counter;
}



sub flush_ec {
    my $fc = shift;
    my $aggr_name = shift;
    my $pvbn = shift;

    if ($aggr_name) {
        if ($pvbn) {
            my (undef,undef,$out) = say($fc,"priv set diag; wafl ec invalidate $aggr_name $pvbn");
            if ($out) {
                if ($out !~ /Warning: These diagnostic commands are for use by NetApp/) {
                    confess "Command failed: $out";
                }
            }
        }
        else {
            my (undef,undef,$out) = say($fc,"priv set diag; wafl ec invalidate $aggr_name all");
            if ($out) {
                if ($out !~ /Warning: These diagnostic commands are for use by NetApp/) {
                    confess "Command failed: $out";
                }
            }
        }
        return 1;
    }
    else {
        my @aggrs = aggr_list($fc);
        foreach $aggr (@aggrs) {
            my (undef,undef,$out) = say($fc,"priv set diag; wafl ec invalidate $aggr all");
            if ($out) {
                if ($out !~ /Warning: These diagnostic commands are for use by NetApp/) {
                    confess "Command failed: $out";
                }
            }
        }
    return scalar(@aggrs);
    }
    # If gets here, we did no work
    return 0;
}


sub ec_type {
    my $fc = shift;
    my $filer;
    if(check_C_or_7_mode($fc)) {
        logcomment("IS_NG = 1");
        $filer = $fc;
    }
    else {
        logcomment("IS_NG = 0");
        $filer = $fc->hostp()->hostname();
    }
    my $st = new Perf::Stats(node => $filer, counters => ["ext_cache_obj:ec0:type"]);

    my $samp1 = $st->sample();

    return $samp1->{"ext_cache_obj:ec0:type"};
}


sub ec_size_gb {
    my $fc = shift;
    my $filer;
    if(check_C_or_7_mode($fc)) {
        logcomment("IS_NG = 1");
        $filer = $fc;
    }
    else {
        logcomment("IS_NG = 0");
        $filer = $fc->hostp()->hostname();
    }
    my $st = new Perf::Stats(node => $filer, counters => ["ext_cache_obj:ec0:size"]);

    my $samp1 = $st->sample();

    return $samp1->{"ext_cache_obj:ec0:size"};
}


1;

__END__

## @pod here

=head1 NAME

    Flexscale.pm - convenience functions for FlexScale

=head1 SYNOPSIS

    use Perf::Flexscale qw(flexscale_is_licensed
                           flexscale_licensed
                           flexscale_blocks);
    use Tharn;

    my $fc = connect($FILER);
    if (!flexscale_is_licensed($fc)) {
        flexscale_license($fc);
    }

    my $vcblks = flexscale_blocks($fc);

    ...

=head1 DESCRIPTION

    This module provides functions for interacting with FlexScale
    (the WAFL external victim cache).

=head1 FUNCTIONS

=head2 flexscale_is_licensed

    Return true if FlexScale is licensed, false otherwise.

    Arguments:
        $fc - filer connection or host

=head2 flexscale_license

    Add Flash Cache (Virgo) or FlexScale (Pisces) or ExtCache (GB) site license.

    Arguments:
        $fc - filer connection or host

=head2 flexscale_unlicense

    Remove Flash Cache or FlexScale or ExtCache license.

    Arguments:
        $fc - filer connection or host

=head2 flexscale_option_enable

    Return current setting of flexscale.enable option. If the optional
    second argument ($val) is supplied, first set flexscale.enable to
    its value. (Legal values are off, OFF, on, ON, true, TRUE, false,
    FALSE, 1, or 0.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for options flexscale.enable

=head2 flexscale_option_lopri_blocks

    Return current setting of flexscale.lopri_blocks option. If the
    optional second argument ($val) is supplied, first set
    flexscale.lopri_blocks to its value. (Legal values are off, OFF,
    on, ON, true, TRUE, false, FALSE, 1, or 0.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for options flexscale.lopri_blocks

=head2 flexscale_option_normal_data_blocks

    Return current setting of flexscale.normal_data_blocks option. If
    the optional second argument ($val) is supplied, first set
    flexscale.normal_data_blocks to its value. (Legal values are off,
    OFF, on, ON, true, TRUE, false, FALSE, 1, or 0.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for options flexscale.normal_data_blocks

=head2 flexscale_option_max_io_qdepth

    Return flexscale.max_io_qdepth option value, optionally
    setting it first if a second argument is supplied.

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for options flexscale.max_io_qdepth

=head2 flexscale_option_mbuf_inserts

    Return current setting of flexscale.mbuf_inserts option. If
    the optional second argument ($val) is supplied, first set
    flexscale.mbuf_inserts to its value. (Legal values are off,
    OFF, on, ON, true, TRUE, false, FALSE, 1, or 0.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for options flexscale.mbuf_inserts

=head2 flexscale_option_rewarm

    Return current setting of flexscale.rewarm option. If
    the optional second argument ($val) is supplied, first set
    flexscale.rewarm to its value. (Legal values are off,
    OFF, on, ON, true, TRUE, false, FALSE, 1, or 0.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for options flexscale.rewarm

=head2 flexscale_ecc_error_limit

    Return current setting of wafl_ext_cache_ecc_limit flag. If
    the optional second argument ($val) is supplied, first set
    wafl_ext_cache_ecc_limit to its value. (Legal values are
    nonnegative integers.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for wafl_ext_cache_ecc_limit

=head2 flexscale_error_restart

    Return current setting of wafl_ext_cache_error_restart flag. If
    the optional second argument ($val) is supplied, first set
    wafl_ext_cache_error_restart to its value. (Legal values are
    nonnegative integers.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for wafl_ext_cache_error_restart

=head2 flexscale_offline_panic

    Return wafl_ext_cache_offline_panic flag value, optionally setting
    it first if a second argument is supplied.

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for wafl_ext_cache_offline_panic

=head2 flexscale_waffinity

    Return wafl_ext_cache_waffinity flag value, optionally setting
    it first if a second argument is supplied.

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for wafl_ext_cache_waffinity

=head2 flexscale_blocks

    Return the size of the FlexScale external victim cache in 4k
    blocks, as reported by the appropriate ext_cache_obj counter.

    FlexScale must be enabled before flexscale_blocks is called.

    Arguments:
        $fc - filer connection or host

=head2 flexscale_config

    Return reference to an array of FlexScale card info hashrefs.

    Arguments:
        $fc - filer connection or host

    Options:
        $id - sysconfig identifier string of FlexScale card

=head2 flexscale_status

    Return flexscale status as reported by the ext_cache::state counter.

    Arguments:
        $fc - filer connection or host

=head2 flexscale_status_string

    Return flexscale status as reported by the ext_cache::state-string counter.

    Arguments:
        $fc - filer connection or host

=head2 flexscale_ems_messages

    Return reference to an array of the FlexScale messages in
    the current EMS log.

    Arguments:
        $fc - filer connection or host

    Options:
        $regex - regular expression to recognize FlexScale messages

=head2 flexscale_invalidate

    Invalidate blocks from the victim cache.

    Arguments:
        $fc - filer connection or host

    Options:
        $aggrname - aggregate to invalidate blocks for

=head2 change_1_card_state

    Change the state of one PAM card. Return card slot.

    Arguments:

    Options:
        new_state => 'Disabled' (default) or 'Enabled'
        card_type => 'Performance Acceleration Module I' (default)
        card_slot => <card slot 0-8> (default is random slot with PAM)

=head2 change_all_cards

    Change the state of all PAM cards. Return # of cards changed.

    Arguments:

    Options:
        new_state => 'Disabled' (default) or 'Enabled'
        card_type => 'Performance Acceleration Module I' (default)

=head2 find_cards

    Find all PAM cards. Return hash with slot{state}

    Arguments:

    Options:
        card_type => 'Performance Acceleration Module I' (default)

=head2 card_status

    Find the state one PAM card. Return 'Enabled' or 'Disabled'

    Arguments:

    Options:
        card_slot => <card slot 0-8>

=head2 find_enabled_cards

    Return array of enabled PAM cards.

    Arguments:

    Options:
        card_type => 'Performance Acceleration Module I' (default)

=head2 find_disabled_cards

    Return array of disabled PAM cards.

    Arguments:

    Options:
        card_type => 'Performance Acceleration Module I' (default)

=head2 find_card_type

    Return the card type (ie. Flash Cache).

    Arguments:
        $fc       - filer connection

    Options:

=head2 card_size

    Return the card size in bytes.

    Arguments:
        $card_type       - card type from sysconfig -v (see find_card_type)

    Options:

=head2 flexscale_setflag_lopri_blocks

    Return current setting of wafl_ext_cache_oppor_lopri setflag for opportunistic caching.
    If the optional second argument ($val) is supplied, first set
    wafl_ext_cache_oppor_lopri to its value. (Legal values are 0 - 100)

    Arguments:
        $fc  - filer connection

    Options:
        $val - new value for setflag flexscale.lopri_blocks

=head2 use_pamii_bootarg

    Return current setting of ext-cache-use-pam-ii? bootarg. If the optional
    second argument ($val) is supplied, first set ext-cache-use-pam-ii? to
    its value. (Legal values are true and false.)

    Arguments:
        $fc - filer connection or host

    Options:
        $val - new value for ext-cache-use-pam-ii?

=head2 get_filer_model_name

    Returns the model name of the filer

    Arguments:
        $fc                   - filer connection object or host

=head2 find_card_type_with_size

    Returns the memory size of the filer in text form '32 GB'

    Arguments:
       $fc                   - filer connection object or host


=head2 get_snmp_values

    Returns hash of snmp mib names and the corresponding values

    Arguments:
        $opts                - hash of option names and settings


=head2 get_filer_memory_size

    Returns memory size of the filer

    Arguments:
        $fc                  - filer connection object or host


=head2 get_extcache_type

    Returns ext_cache type and ext_cache subtype

    Arguments:
        $fc                  - filer connection object or host


=head2 get_cm_values

    Returns hash of selected external cache counter names and the corresponding values

    Arguments:
        $opts                - hash of option names and settings

=head2 get_average

    Returns average value

    Arguments:
        $val                - ext_cache_obj:ec0:readio/writeio_* counter values


=head2 flexshare_status

    Returns flexshare status

    Arguments:
        $fc                  - filer connection object or host


=head2 flexscale_reset_options

    Reset all the flexscale options to default values of the Filer

    Arguments:
        $opts                - hash of flexscale option names and settings


=head2 flexscale_option_pcs_size

    Returns the pcs_size value only if SPCS is used  optionally setting it first if $val is provided.

    Arguments:
        $fc                   - filer connection object or host

    Options:
        $val                  - new value for pcs-size

=head2 flexscale_option_pcs_high_res

    Returns the pcs_high_res value only if SPCS is used  optionally setting it first if $val is provided.

    Arguments:
        $fc                   - filer connection object or host

    Options:
        $val                  - new value for pcs-high-res


=head2 compare_extcache_snmp

    Compare the external cache hash values and snmp hash values. Return status as GOOD(1) or BAD(0) depending on the comparison

    Arguments:
        $opt                  - hash of option names and settings


=head2 change_tagstore_size

    Change the tagstore value to a desired value. The change is follwed by filer reboot

    Arguments:
        $fc                   - filer connection object or host
        $val                  - Incremental value as percentage


=head2 options_flexscale

    Returns the flexscale options for SPCS, Pisces and Virgo

    Arguments:
        $fc                   - filer connection object or host

    Options:
        $cache_type           - 'SPCS' (default)


=head2 ext_cache_pcs_size

    Returns the current status of ext-cache-pcs-size boot argument, optionally setting it first if $val is provided.
    If a new value is set, it is followed by a Filer reboot.

    Arguments:
        $fc                   - filer connection object or host

    Options:
        $val                  - new value for ext-cache-pcs-size


    Return current setting of ext-cache-use-pam-ii? bootarg. If the optional
    second argument ($val) is supplied, first set ext-cache-use-pam-ii? to
    its value. (Legal values are true and false.)

    Arguments:
        $fc  - filer connection or host

    Options:
        $val - new value for ext-cache-use-pam-ii?

=head2 flush_ec

    Flush an aggregate out of EC (PAM). If no aggregate is specified, it will flush all aggregates.
    Returns the number of aggregates flushed. This is for 7-mode (or node scoped) only.

    Arguments:
        $fc - filer connection or host

    Options:
        $aggr_name - name of aggregate to be flushed (defaults to all)


=head2 ec_type

    Return the EC type from ext_cache_obj:ec0:type.
    IOMEM = Pisces (PAM I)
    IOMEM-FLASH = Virgo (Flash Cache)
    SPCS = SPCS, same as PCS

    Arguments:
        $fc - filer connection or host

    Options:


=cut

