# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Perf/PerfArchiver/Parser.pm#5 $
package Perf::PerfArchiver::Parser;

use strict;
use warnings;
use NACL::C::Node;
use NACL::APISet::Version;
use Tharn;
use vars qw($archiver_headers
	$archiver_so);
use NATE::Log qw(log_global);

our $archiver_inline = '';
BEGIN {

	my $node_obj = NACL::C::Node->find();
	my $node	= $node_obj->node;
	my $version_obj = NACL::APISet::Version->new(hostobj => host($node),
						category => 'Node',
						shell_type => 'CMode');
	my $buildpath = $version_obj->get_version_attribute(attribute => 'build_path');
	$archiver_so = $buildpath . 'cm/cmtools/bedrock/import/linux64/package-l/uliba-l/cmapl/';
	$archiver_headers = $buildpath . 'cm/cmtools/bedrock/import/linux64/package-l/headers/';
	$archiver_inline = $::LOGDIR . '/PerfArchiver_Inline';
	mkdir $archiver_inline;
}

use Inline(
	C => 'DATA',
	DIRECTORY => $archiver_inline,
	INC  => "-I$archiver_headers",
	LIBS => "-L$archiver_so -L/usr/lib/ -lcmapl -lxml2",
	);

Inline->init;

use Carp;
use File::Basename qw(basename);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Data::Dumper;
use vars qw(@ISA @EXPORT);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_object_info
			get_instance_info
			get_counter_info
			get_first_counter
			get_first_object
			get_first_instance
			get_first_record
			get_first_counter_data
			get_specific_object
			get_specific_counter
			get_specific_counter_by_id
			get_header_info
			get_file_timestamps
			get_node_uuid
			incorrect_obj_inst_counter_names
			get_incorrect_file_err_code
			get_read_failed_err_code
			get_seek_failed_err_code
			get_counter_values
			get_directory_contents
			get_incorrect_sample_period_err_code
			get_invalid_args_error_code
			get_invalid_timestamp_err_code
			get_specific_instance
			read_core_files
			);



# Function to retrieve object info
sub get_object_info {
	my %args = validate(
                          @_,
                            {	datafile => {type => SCALAR, required => 1},
				object => {type => SCALAR, required => 1},
							},
						);
	my $value = _find_specific_object(\%args);

	print "\n Value is : $value";
	return $value;
}

# Function to retrieve instance info
sub get_instance_info {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object		=> {type => SCALAR, required => 1},
				instance	=> {type => SCALAR, required => 1},
							},
						);
	my $value = _get_object_counter_properties(\%args);

	print "\n Value is : $value";
	return $value;
}

# Function to retrieve counter info
sub get_counter_info {
	my $datafile = shift;
	my $counter_props = _get_counter_properties($datafile);
	return $counter_props;
}

# Function to retrieve first counter
sub get_first_counter {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object	=> {type => SCALAR, required => 1},
							},
						);
	my $counter_value = _get_first_counter(\%args);
	return $counter_value;
}

# Function to retrieve the first object
sub get_first_object {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
							},
						);
	my $counter_value = _get_first_object(\%args);
	return $counter_value;
}

# Function to retrieve the first instance
sub get_first_instance {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object		=> {type => SCALAR, required => 1},
							},
						);
	my $instance_value = _find_first_instance(\%args);
	return $instance_value;
}


# Function to retrieve the first record
sub get_first_record {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object		=> {type => SCALAR, required => 1},
				key_cntr_instances => {type => HASHREF, required =>1}
							},
						);
	my $counter_value = _find_first_timestamp(\%args);
	print "\n Counter Values :". Dumper($counter_value);
	return $counter_value;
}

# Function to retrieve the first record
sub get_first_counter_data {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object		=> {type => SCALAR, required => 1},
							},
						);
	my $counter_value = _get_first_counter_data(\%args);
	return $counter_value;
}


# Function to retrieve a specific object
# if object is not found return error code
sub get_specific_object {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object	=> {type => SCALAR, required => 1},
							},
						);
	my $error_code = _find_specific_object(\%args);
	return $error_code;
}

# Function to find a specific counter 
# Returns counter information as hash ref
# Else error code as a array ref

sub get_specific_counter {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object	=> {type => SCALAR, required => 1},
				counter_name => {type => SCALAR, required => 1},
							},
						);
	my $error_code = _find_specific_counter(\%args);
	return $error_code;
}

# Function to find a specific counter name by counter id
# Returns counter information as hash ref
# Else error code as a array ref

sub get_specific_counter_by_id {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object	=> {type => SCALAR, required => 1},
				counter_id => {type => SCALAR, required => 1},
							},
						);
	my $error_code = _find_specific_counter_by_id(\%args);
	return $error_code;
}

# Function to find a specific instances
# Returns counter information as hash ref
# Else error code as a array ref

sub get_specific_instance {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object		=> {type => SCALAR, required => 1},
							},
						);

	my $error_code = _find_specific_instance(\%args);
	return $error_code;
}


# Function to retrieve header information
sub get_header_info {
	my $datafile = shift;
	my $header_info = _get_header_information($datafile);
	return $header_info;
}

# Function to get oldest and latest timestamps
# Returns a hashref as follows:
# {oldest_timestamp => $oldest_timestamp,
#	latest_timestamp => $latest_timestamp};
sub get_file_timestamps {
	my $datafile = shift;
	my $timestamps = get_timestamps($datafile);
	return $timestamps;
}

# Function to get ipaddress from the datafil
# Returns a arrayref with IP address

sub get_ip_address {
	my $datafile = shift;
	my $ipaddress = _get_ipaddress($datafile);
	return $ipaddress;
}

# Function to return an array ref of error codes when
# incorrect/non-existent objectname, instance_name, counter name
# and counter data is queried
# Returns an array ref with all the error codes.

sub incorrect_obj_inst_counter_names {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				obj_name	=> {type => SCALAR, required => 1},
				inst_name	=> {type => SCALAR, optional => 1},
				inst_value	=> {type => SCALAR, optional => 1},
				counter     => {type => SCALAR, optional => 1},
			},
		);

	my $error_codes = _get_incorrect_obj_inst_counter_names(\%args);
	print "\n Error codes :". Dumper($error_codes);
}

# Function to return arrayref of error codes when
# incorrect file name is requested
# Returns array ref of error code
sub get_incorrect_file_err_code {
	my $file = shift;
	my $error_code = _get_incorrect_file($file);
	print "\n Incorrect file got from main program: $file";
	return $error_code;
}

# Function to get read failed error code
# Returns an array ref with error codes
sub get_read_failed_err_code {
	my $datafile = shift;
	my $error_code = _get_read_failed($datafile);
	return $error_code;
}

# Function to get seek failed error code
# Returns an array ref with error codes
sub get_seek_failed_err_code {
	my $datafile = shift;
	my $error_code = _get_seek_failed($datafile);
	return $error_code;
}

# Function to find all counter values
sub get_counter_values {
	my $datafile = shift;
	print "\n Datafile is : $datafile";
	my $counter_values = _get_counter_values($datafile);
	return $counter_values;
}

# Function to get directory contents
sub get_directory_contents {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				start_time	=> {type => SCALAR, required => 1},
				end_time	=> {type => SCALAR, required => 1},
			},
		);
	my $contents = _get_dir_contents(\%args);
	return $contents;
}

# Function to get error code when incorrect sample period is specified
sub get_incorrect_sample_period_err_code {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				period	=> {type => SCALAR, required => 1},
			},
		);
	my $err_code =  _get_incorrect_sample_period(\%args);
	return $err_code;
}


# Function to return error code when invalid arguments in cmapl_find_instance
# function
# Returns array ref of error codes.

sub get_invalid_args_error_code {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object		=> {type => SCALAR, required => 1},
			},
		);
	my $error_code = _get_invalid_args_err_code(\%args);
	return $error_code;
}

# Function to return error code when invalid timstamps are provided
# to query a datafile record

sub get_invalid_timestamp_err_code {
	my %args = validate(
                          @_,
                            {	datafile	=> {type => SCALAR, required => 1},
				object		=> {type => SCALAR, required => 1},
			},
		);
	my $error_code = _get_invalid_timestamp(\%args);
	return $error_code;	
}

1;

__DATA__
__C__

#include <cmapl/cmapl_ext.h>

// Function to retrieve object properties for all objects from datafile 
// Returns a hash ref containing object names and ids as well as error codes if 
// any

SV* _get_object_properties(char* datafile) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* object_array = newAV();
	HV* obj_properties = newHV();
		
	/* Open a datafile */
	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
	
	/* Get the header handle of the datafile */
	cmapl_header_handle_t* header;
	err = cmapl_get_header(file_handle, &header);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	/* Iterate through all objects in the datafile */
	cmapl_object_handle_t* current_obj;
	err = cmapl_first_object(header, &current_obj);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	while(current_obj != NULL) {
		/* Get object info */
		
		HV* obj_name_id = newHV();

		cmapl_object_def_t obj_info;
		err = cmapl_get_object_info(current_obj,&obj_info);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		} 
		
		SV* obj_name = newSVpvf("%s", obj_info.name);
		SV* obj_id   = newSViv(obj_info.obj_id);

		hv_store(obj_name_id, "obj_name", 8, obj_name, 0);
		hv_store(obj_name_id, "obj_id", 6, obj_id, 0);
			
		av_push(object_array, newRV_noinc((SV*)obj_name_id));
		
		// Get the next object in the preset
		err = cmapl_next_object(header, &current_obj);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
	}

	/* Close the archive file handle */
    err = cmapl_close_data_file(file_handle);
    	
	if (err != CMAPL_SUCCESS){
		printf("Error while closing datafile :",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
	
	hv_store(obj_properties, "object_array", 12, newRV_noinc((SV*)object_array), 0);
	hv_store(obj_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);

	/* Return result */
	return newRV_noinc((SV*)obj_properties);
}

// Function to find a particular object
// If the object is available return the object and properties else
// return error code

SV* _find_specific_object(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* object_array = newAV();
	HV* obj_properties = newHV();

	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	char* key;
	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		}
	}
	
	/* Open the data file */
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
	
	/* Get the header handle of the datafile */
	cmapl_header_handle_t* header;
	err = cmapl_get_header(file_handle, &header);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	/* Find a specific object in the datafile */
	cmapl_object_handle_t* current_obj;
	err = cmapl_find_object(header, obj_name , &current_obj);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	if(current_obj != NULL) {
		/* Get object info */

		cmapl_object_def_t obj_info;
		err = cmapl_get_object_info(current_obj,&obj_info);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		} 
		
		/* Dump Object Info */
		av_push(object_array, newSVpvf("%s",obj_info.name));
		av_push(object_array, newSViv(obj_info.obj_id));
	}

	/* Close daatfile */
    err = cmapl_close_data_file(file_handle);
    	
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
		
	hv_store(obj_properties, "object_array", 12, newRV_noinc((SV*)object_array), 0);
	hv_store(obj_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);

	/* Return result */
	return newRV_noinc((SV*) obj_properties);
}

//Function to get the header information of a datafile 
//Returns hash ref with object, counters and their sample period

SV* _get_header_information(char* datafile ) {
        cmapl_error_t err = CMAPL_SUCCESS;
		HV* hash_objs_cntrs = newHV();
		int num_objs = 0;
        
	// Open the specified archive file
        cmapl_file_handle_t* file;
        const char* meta_path;
        const char* data_path;
        err = cmapl_open_data_file(datafile, "r", &file, &meta_path, &data_path);
        if (err != CMAPL_SUCCESS){
                printf("Error opening file with error label %s:",cmapl_error_labels[err]);
        }

        // Get header handle
	cmapl_header_handle_t* header_handle;
	err = cmapl_get_header(file, &header_handle);
	if(err != CMAPL_SUCCESS) {
		printf("Error retriving header handle");
	}

	// Get object handle
	cmapl_object_handle_t* obj_handle;
	
	// Get the first object
	err = cmapl_first_object(header_handle, &obj_handle);
	if(err != CMAPL_SUCCESS) {
		printf("Error retrieving the first object with error label %s:", cmapl_error_labels[err]);
	}
	while (obj_handle != NULL) {
		int len;
		AV* obj_array = newAV();

		// Get object info
		cmapl_object_def_t obj_info;
		err = cmapl_get_object_info(obj_handle, &obj_info);
		if(err != CMAPL_SUCCESS){
			printf("Error retrieving obj info with error label %s:", cmapl_error_labels[err]);
		}

		len = strlen(obj_info.name);
		num_objs = num_objs + 1;
				
		cmapl_error_t err = CMAPL_SUCCESS;
		cmapl_counter_handle_t* cntr_handle;
		err = cmapl_first_counter(obj_handle, &cntr_handle);
	
		if(err != CMAPL_SUCCESS) {
			printf("Error retrieving first counter with error label %s:", cmapl_error_labels[err]);
		}
			
		while(cntr_handle != NULL) {
			HV* cntr_hash = newHV();
			
			// Get counter information
			cmapl_counter_def_t cntr_info;
					
			err = cmapl_get_counter_info(cntr_handle, &cntr_info);
			if(err != CMAPL_SUCCESS) {
				printf("Error retrieving counter information with error label %s :", cmapl_error_labels[err]);
			}
			
			hv_store(cntr_hash, "counter", 7, newSVpv(cntr_info.name, 0),0);
			hv_store(cntr_hash, "period", 6, newSViv(cntr_info.periods[0]/1000),0);
			av_push(obj_array, newRV_noinc((SV*) cntr_hash));
			
			// Get next counter
			err = cmapl_next_counter(obj_handle, &cntr_handle);
			if(err != CMAPL_SUCCESS) {
				printf("Error retrieving first counter with error label %s:", cmapl_error_labels[err]);
			}
		}
	 	
		hv_store(hash_objs_cntrs, obj_info.name, len, newRV_noinc((SV*) obj_array ), 0);

		// Get next object from metadata file
		err = cmapl_next_object(header_handle, &obj_handle);
		if(err != CMAPL_SUCCESS) {
			printf("Error retrieving the next object with error label %s:", cmapl_error_labels[err]);
		}
	}
	
	// close the archive file handle
        err = cmapl_close_data_file(file);
        if (err != CMAPL_SUCCESS){
             printf("Error closing file with error label %s:",cmapl_error_labels[err]);
        }
	
	//hv_store(hash_objs_cntrs, "num_objects", 11 , newSViv(num_objs), 0);
	return newRV_noinc((SV*) hash_objs_cntrs);
}

/* Function to get oldest and latest timestamps 
Returns an array ref of oldest and latest timestamps */

SV* get_timestamps(char* datafile) {
	cmapl_error_t err = CMAPL_SUCCESS;
	HV* timestamps = newHV();
	AV* error_array		= newAV();
	int num_objs = 0;
	uint64_t oldest_timestamp = 0;
	uint64_t latest_timestamp = 0;
       
	// Open the specified archive file
	cmapl_file_handle_t* file_handle;
	const char* meta_path;
	const char* data_path;
	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
		printf("Error opening file with error label %s:",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}

	// Get the oldest timestamp
	err = cmapl_get_oldest_record_time(file_handle, &oldest_timestamp);
	if(err != CMAPL_SUCCESS) {
		printf("Error while retrieving oldest timestamp with error label %s:",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	// Get the latest timestamp
	err = cmapl_get_newest_record_time(file_handle, &latest_timestamp);
	if(err != CMAPL_SUCCESS) {
		printf("Error while retrieving latest timestamp with error label %s:",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}

	// close the archive file handle
	err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		printf("Error closing file with error label %s:",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	hv_store(timestamps, "oldest_timestamp", 16 , newSVuv(oldest_timestamp), 0);
	hv_store(timestamps, "latest_timestamp", 16 , newSVuv(latest_timestamp), 0);
	return newRV_noinc((SV*) timestamps);
}


/* Function get error code for incorrect file */
/* Returs error array */

SV* _get_incorrect_file(char* datafile) {

	AV* error_array = newAV();
	cmapl_error_t err = CMAPL_SUCCESS;
	const char* meta_path;
	const char* data_path;
	cmapl_file_handle_t* file_handle;
	
	printf("\n Open datafile");
	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
		printf("Error opening file with error label : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}

	// close the archive file handle
	err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		printf("Error closing file with error label %s:",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}
	return newRV_noinc((SV*) error_array);
}

/* Function to return error code when read failed */
/* Returns error array */

SV* _get_read_failed(char* datafile) {
	AV* error_array = newAV();
	cmapl_error_t err = CMAPL_SUCCESS;
	const char* meta_path;
	const char* data_path;
	cmapl_file_handle_t* file_handle;
	
	// Enable fiji
	err = cmapl_enable_fiji(CMAPL_READ_FAILED);
	if (err != CMAPL_SUCCESS){
		printf("\n Error enabling the fiji : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}

	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		printf("\n Error opening file with error label : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}
	
	err = cmapl_disable_fiji(CMAPL_READ_FAILED);
	if (err != CMAPL_SUCCESS){
		printf("\n Error disabling the fiji : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}
	return newRV_noinc((SV*) error_array);
}

/* Function to return error code when seek failed */
/* Returns error array */

SV* _get_seek_failed(char* datafile) {
	AV* error_array = newAV();
	cmapl_error_t err = CMAPL_SUCCESS;
	const char* meta_path;
	const char* data_path;
	cmapl_file_handle_t * file_handle;
	
	// Enable fiji
	err = cmapl_enable_fiji(CMAPL_SEEK_FAILED);
	if (err != CMAPL_SUCCESS){
		printf("\n Error enabling the fiji : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}

	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		printf("\n Error opening file with error label : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}
	
	err = cmapl_disable_fiji(CMAPL_SEEK_FAILED);
	if (err != CMAPL_SUCCESS){
		printf("\n Error disabling the fiji : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
	}
	return newRV_noinc((SV*) error_array);
}

/* Function to get counter values */
/* Returns hash ref of objects and counter values along with timestamps and sample periods */
/* It also returns error array */

SV* _get_counter_values(char * datafile){
	cmapl_file_handle_t * file_handle;
	const char* meta_path;
	const char* data_path;

	uint64_t start_time = 0;
	uint64_t end_time = 0;
	
	cmapl_error_t err = CMAPL_SUCCESS;
	
	AV* error_array = newAV();
	AV* object_array = newAV();
	AV* obj_error_array = newAV();
	HV* object_counters_hash = newHV();

	/* Open datafile */
	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
		printf("\nError opening file with error label : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*)error_array);
	}
	
	/* Get start time and end time from the datafile */
	err = cmapl_get_oldest_record_time(file_handle, &start_time);
	if(err != CMAPL_SUCCESS) {
		printf("\nError for oldest time :%s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*)error_array);
	}
		
	err = cmapl_get_newest_record_time(file_handle, &end_time);
	if(err != CMAPL_SUCCESS) {
		printf("\nError for latest time :%s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*)error_array);
	}
		
	/* Get the header handle of the datafile */
	cmapl_header_handle_t* header;
	err = cmapl_get_header(file_handle, &header);
	if(err != CMAPL_SUCCESS) {
		printf("\nError while retrieving header information : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*)error_array);
	}
	
	/* iterate through all the objects in the datafile */
	cmapl_object_handle_t* current_obj;
	err = cmapl_first_object(header, &current_obj);
	if (err != CMAPL_SUCCESS) {
		printf("\nError while retrieving first object handle : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*)error_array);
	}
	
	/* Create a hash to create the object hash */
	HV* object_hash = newHV();
	HV* obj_error_hash = newHV();

	while(current_obj != NULL) {
		/* Get Object Info */
		cmapl_object_def_t obj_info;
		err = cmapl_get_object_info(current_obj, &obj_info);
		if (err != CMAPL_SUCCESS) {
			printf("\nError while retrieving object information : %s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*)error_array);
		}
		
		SV* object_name = newSVpv(obj_info.name,0);
		//printf ("\n Object name : %s", obj_info.name);

		/* Iterate through all the periods of an object */
		uint32_t num_periods = 0;
		uint64_t* periods;

		err = cmapl_get_object_periods(current_obj, &periods, &num_periods);
		if (err != CMAPL_SUCCESS) {
			printf("\nError while retrieving object periods : %s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			//return newRV_noinc((SV*)error_array);
		}
		uint32_t i;
		
		HV* periods_hash = newHV();
		HV* error_hash = newHV();
		AV* empty_records_array = newAV();

		for (i = 0; i < num_periods; i++){
			//printf("\nPeriod value: %"PRIu64"\n", periods[i]/1000);

			/* Convert the period from unsigned long to char* to be used as hash key */
			uint64_t period2;
			period2 = periods[i]/1000;
			const int num = snprintf(NULL, 0, "%lu", period2);
			char period_string[num+1];
			snprintf(period_string, num+1, "%lu", period2);

			/* Get counter name for each counter at this time period */
			cmapl_counter_handle_t* current_cnt;
			uint32_t cnt_idx = 0;
			const char* cnt_names[1000000];
				
			err = cmapl_first_counter_by_period(current_obj, periods[i], &current_cnt);
			if (err != CMAPL_SUCCESS) {
				printf("\nError while retrieving first counter by period : %s",cmapl_error_labels[err]);
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				//return newRV_noinc((SV*)error_array);
			}
			
			while(current_cnt != NULL) {
				cmapl_counter_def_t cnt_info;
				
				/* Get Counter Info */
				err = cmapl_get_counter_info(current_cnt, &cnt_info);
				if (err != CMAPL_SUCCESS) {
					//free(periods);
					printf("\nError while retrieving counter information : %s",cmapl_error_labels[err]);
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					//return newRV_noinc((SV*)error_array);
				}
				
				// Store the counter name for query
				cnt_names[cnt_idx++] = cnt_info.name;
				
				/* Retrieve the next counter for this period and for this object */
				err = cmapl_next_counter_by_period(current_obj, periods[i], &current_cnt);
				if (err != CMAPL_SUCCESS) {
					printf("\nPeriod value for which counter was not found: %"PRIu64"\n", periods[i]/1000);
					printf("\n Object name : %s" ,obj_info.name);
					printf("\nError while retrieving next counter by period : %s",cmapl_error_labels[err]);
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					//return newRV_noinc((SV*)error_array);
				}
			} // end of counter name retrieval
				
				
			/* Create a query to find object and counters at this time period */
			cmapl_query_def_t query_def;
			cmapl_set_latest_version(&query_def);

			query_def.object_name = obj_info.name;
			query_def.num_cnts = cnt_idx;
			query_def.cnt_names = cnt_names;
			query_def.start_time = start_time;
			query_def.end_time = end_time;
			query_def.period = periods[i];
			query_def.num_key_cnts = 0;
				
			cmapl_query_handle_t* query;
			err = cmapl_query(file_handle, &query_def, &query);
			if (err != CMAPL_SUCCESS) {
				free(periods);
				printf("\nError while creating query handle : %s",cmapl_error_labels[err]);
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				//return newRV_noinc((SV*)error_array);
			}

			/* Dump query result */
			if(query == NULL) {
				printf("\n No counters have been sampled at this time period :%s",cmapl_error_labels[err]);
				av_push(error_array, newSVpvf("%s","CMAPL_SUCCESS"));
				//return newRV_noinc((SV*)error_array);
				
			} else {
					
				/* Iterate through all the timestamps in query result that have counter data*/
				cmapl_timestamp_handle_t* timestamp;
				
				err = cmapl_first_timestamp(query, &timestamp);
				if (err == CMAPL_RECORD_EMPTY){
					printf("\nAll records sampled at this period were empty in this file.\n\n");
					//av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					
				} else if (err != CMAPL_SUCCESS){
					printf("\nError while retrieving first timestamp : %s",cmapl_error_labels[err]);
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					//return newRV_noinc((SV*)error_array);
				}
					
				HV* timestamp_hash = newHV();
				uint32_t num_empty_records;

				while(timestamp != NULL || err == CMAPL_RECORD_EMPTY) {
					if(timestamp != NULL) {
						/* Dump timestamp */
						uint64_t time;
						err = cmapl_get_timestamp(timestamp, &time);
						if (err != CMAPL_SUCCESS){
							printf("\nError while retrieving first timestamp value : %s",cmapl_error_labels[err]);
							av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
							//return newRV_noinc((SV*)error_array);
						}
							
						/* Convert the timestamp from unsigned long to char* to be used as hash key */
						unsigned long time2;
						time2 = time;
						const int n = snprintf(NULL, 0, "%lu", time2);
						char time_string[n+1];
						snprintf(time_string, n+1, "%lu", time2);
							
						//printf("\n Timestamp: %"PRIu64"", time);

						/* Iterate through all instances at this timestamp */
							
						cmapl_instance_handle_t* instance;;
						err = cmapl_first_instance(timestamp, &instance);
						if (err != CMAPL_SUCCESS){
							printf("\nError while retrieving first instance value : %s",cmapl_error_labels[err]);
							av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
							//return newRV_noinc((SV*)error_array);
						}
							
						AV* instance_counters_array = newAV();

						while(instance != NULL) {
								
							/* Get instance data */
							cmapl_instance_data_t instance_data;
							err = cmapl_get_instance_data(instance, &instance_data);
							if (err != CMAPL_SUCCESS){
								printf("\nError while retrieving first instance value : %s",cmapl_error_labels[err]);
								av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
								//return newRV_noinc((SV*)error_array);
							}

							// dump key counters data (instance name)
							uint32_t i;
								
							HV* hash_key_counters = newHV();

							for (i = 0; i < instance_data.num_key_cnts; i++){
								//printf("\nRetrieve Key Counter Names and values:\n");
								//printf("\nKey counter name: %s\n", instance_data.key_cnt_names[i]);
								//printf("Value: ");
									
								AV* key_counter_values = newAV();
															
								uint32_t count;
								SV* cnt_value;

								for (count = 0; count < instance_data.key_cnt_values[i].num_values; count++){
									switch(instance_data.key_cnt_values[i].value_type){
										case CMAPL_CNT_4BYTES:
											cnt_value = newSViv(instance_data.key_cnt_values[i].values[count].val_4B);
											//printf("\n Instance Value: %d", instance_data.key_cnt_values[i].values[count].val_4B);
											break;
													
										case CMAPL_CNT_8BYTES:
											cnt_value = newSVuv(instance_data.key_cnt_values[i].values[count].val_8B);
											//printf("\n Instance Value: %"PRIu64, instance_data.key_cnt_values[i].values[count].val_8B);
											break;
													
										case CMAPL_CNT_STRING:
											cnt_value = newSVpv(instance_data.key_cnt_values[i].values[count].val_str,0);
											//printf("\n Instance value : %s", instance_data.key_cnt_values[i].values[count].val_str);
											break;
									}
									if (count != (instance_data.key_cnt_values[i].num_values - 1)){
										//printf(", ");
									}
										
									av_push(key_counter_values, cnt_value);
																	
								} // End of for
									
								 // For all key instances values store the same counter array
									
								hv_store(hash_key_counters, instance_data.key_cnt_names[i], strlen(instance_data.key_cnt_names[i]), newRV_noinc((SV*)key_counter_values),0);
									
							} // End of for
				
								
							/* Iterate through all counter data in this instance */
							cmapl_counter_handle_t* ret_cnt;
							cmapl_counter_data_handle_t* cnt_data;
							err = cmapl_first_counter_data(instance, &ret_cnt, &cnt_data);
							if (err != CMAPL_SUCCESS){
								printf("\nError while retrieving first instance value : %s",cmapl_error_labels[err]);
								av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
								//return newRV_noinc((SV*)error_array);
							}
								
							HV* counter_value_hash = newHV();

							while (cnt_data != NULL){
								/* Get counter info */
								cmapl_counter_def_t cnt_info;
								err = cmapl_get_counter_info(ret_cnt, &cnt_info);
								if (err != CMAPL_SUCCESS){
									printf("\nError while retrieving counter info : %s",cmapl_error_labels[err]);
									av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
									//return newRV_noinc((SV*)error_array);
								}
									
								//SV* counter_name = newSVpv(cnt_info.name, 0);
									
								/* Get counter data */
								cmapl_counter_data_t cnt_data_value;
								err = cmapl_get_counter_data_value(cnt_data, &cnt_data_value);
								if (err != CMAPL_SUCCESS){
									printf("\nError while retrieving counter info : %s",cmapl_error_labels[err]);
									av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
									//return newRV_noinc((SV*)error_array);
								}

								/* Dump counter data */
															
								uint32_t count;
								SV* counter_value;

								// Store all the counter values in an array
								AV* counter_array = newAV();
									
								for (count = 0; count < cnt_data_value.num_values; count++){
										
									switch(cnt_data_value.value_type){
										case CMAPL_CNT_4BYTES:
											counter_value = newSViv(cnt_data_value.values[count].val_4B);
											//printf("\n Counter Value: %d", cnt_data_value.values[count].val_4B);
											break;
											
										case CMAPL_CNT_8BYTES:
											counter_value = newSVuv(cnt_data_value.values[count].val_8B);
											//printf("\n Counter Value : %"PRIu64, cnt_data_value.values[count].val_8B);
											break;
											
										case CMAPL_CNT_STRING:
											counter_value = newSVpv(cnt_data_value.values[count].val_str, 0);
											//printf("\n Counter Value: %s", cnt_data_value.values[count].val_str);
											break;
									}

									if (count != (cnt_data_value.num_values - 1)){
										//printf(", ");
									}
										
									// Push counter values in the array
									av_push(counter_array, counter_value);
								}
														
								hv_store(counter_value_hash, cnt_info.name, strlen(cnt_info.name), newRV_noinc((SV*)counter_array), 0);	
									
								/* Get next counter data in this instance */
								err = cmapl_next_counter_data(instance, &ret_cnt, &cnt_data);
								if (err != CMAPL_SUCCESS){
									printf("\nError while retrieving counter info : %s",cmapl_error_labels[err]);
									av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
									//return newRV_noinc((SV*)error_array);
								}
							} // end of while
								
							/* Get next instance at this timestamp */
							err = cmapl_next_instance(timestamp, &instance);
							if (err != CMAPL_SUCCESS){
								printf("\nError while retrieving first instance value : %s",cmapl_error_labels[err]);	
								av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
								//return newRV_noinc((SV*)error_array);
							}
							
							av_push(instance_counters_array, newRV_noinc((SV*)counter_value_hash));
							//av_push(instance_counters_array, newRV_noinc((SV*)hash_key_counters));
						}
							
						//return newRV_noinc((SV*)instance_counters_array);
						hv_store(timestamp_hash, time_string, strlen(time_string), newRV_noinc((SV*)instance_counters_array), 0);
			
						
					} // End of if checking if timestamp is NULL
					
					/* Get the next timestamp in the query result */
					err = cmapl_next_timestamp(query, &timestamp);
						
					if (err != CMAPL_RECORD_EMPTY && err != CMAPL_SUCCESS){
						av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
						av_push(empty_records_array, newSVpvf("%s",cmapl_error_labels[err]));
					}

				} // End of retrieval of timestamps

				const char* error_value_key = "CMAPL_RECORD_EMPTY";
				hv_store(periods_hash, period_string, strlen(period_string), newRV_noinc((SV*)timestamp_hash), 0);
				hv_store(error_hash, period_string, strlen(period_string), newRV_noinc((SV*)empty_records_array), 0);
			
			} // Check if query handle is not null
		
		} // End of for 
		
		free(periods);
		
		hv_store(object_hash, obj_info.name, strlen(obj_info.name), newRV_noinc((SV*)periods_hash), 0);
		hv_store(obj_error_hash, obj_info.name, strlen(obj_info.name),newRV_noinc((SV*)error_hash), 0);
		
		err = cmapl_next_object(header, &current_obj);
		if (err != CMAPL_SUCCESS) {
			printf("\nError while retrieving first object handle : %s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			//return newRV_noinc((SV*)error_array);
		}
	}
	
	av_push(object_array, newRV_noinc((SV*)object_hash));
	av_push(obj_error_array, newRV_noinc((SV*)obj_error_hash));

	/* Close Datafile */
	err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		printf("\nError closing file with error label %s:",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		//return newRV_noinc((SV*)error_array);
	}
	
	hv_store(object_counters_hash, "obj_cntr_array", 14, newRV_noinc(object_array), 0);
	hv_store(object_counters_hash, "error_array", 11, newRV_noinc(obj_error_array), 0);

	return newRV_noinc((SV*) object_counters_hash);
	//return newRV_noinc((SV*) object_array);
	
}

/* Function to get counter size */

char* get_counter_size(cmapl_cnt_size_t c_size)
{
        switch(c_size){
                case CM_CNT_4B:
			return "4";
                case CM_CNT_8B:
			return "8";
                case CM_CNT_16B:
			return "16";
                case CM_CNT_32B:
			return "32";
		case CM_CNT_64B:
			return "64";
                case CM_CNT_96B:
			return "96";
                case CM_CNT_128B:
			return "128";
                case CM_CNT_160B:
			return "160";
                case CM_CNT_252B:
			return "252";
                        break;
                default:
                        return "false";
        }
		return "type";
}

// Function to get counter type

char* get_counter_type(cmapl_cnt_size_t c_type)
{
        switch(c_type){
                case CM_CNT_RAW:
			return "RAW";
                case CM_CNT_RATE:
			return "RATE";
                case CM_CNT_DELTA:
			return "DELTA";
                case CM_CNT_PERCT:
			return "PERCT";
                case CM_CNT_AVERAGE:
			return "AVERAGE";
		case CM_CNT_STRING:
                    	return "STRING";    
			break;	
                default:
                        return "false";
        }
	return "true";
}

// Function to get counter units

char* get_counter_unit(cmapl_cnt_unit_t c_unit)
{
        switch(c_unit){
                case CM_NO_SUFFIX:
					return "no_suffix";
                case CM_PER_SEC:
					return "per_sec";
                case CM_PERCT:
					return "percent";
                case CM_B_PER_SEC:
					return "b_per_sec";
                case CM_KB_PER_SEC:
					return "kb_per_sec";
                case CM_MSECS:
					return "msecs";
                case CM_USECS:
					return "usecs";
                case CM_SECS:
					return "sec";
                case CM_MB_PER_SEC:
					return "mb_per_sec";
                case CM_MB:
					return "mb";
                case CM_CNT_NODISP:
					return "no_display";
                        break;
                default:
					return "false";
        }
        return "true";
}

// Function to get counter_properties

SV* _get_counter_properties(char* datafile) {
	cmapl_error_t err = CMAPL_SUCCESS;
	
	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* object_array = newAV();
	HV* obj_counters = newHV();
	HV* obj_counter_hash = newHV();

	printf("\n Open datafile");
	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	
	/* get header handle */
	cmapl_header_handle_t* header_handle;
	err = cmapl_get_header(file_handle, &header_handle);
	
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	
	/* retrieve object handle */
	cmapl_object_handle_t* current_obj;
	err = cmapl_first_object(header_handle, &current_obj);
	
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	} 

	// Get object properties
		
	while(current_obj != NULL) {
		cmapl_object_def_t obj_info;
				
		AV* obj_cntr_array = newAV();
				
		err = cmapl_get_object_info(current_obj, &obj_info);
		if (err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
			
		err = cmapl_get_object_info(current_obj, &obj_info);
		if (err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
			
		// Get all counters and their properties for this object
		
		cmapl_counter_handle_t * cntr_handle;
		err = cmapl_first_counter(current_obj, &cntr_handle);

		if (err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		while(cntr_handle != NULL) {

			// Hash to store all counter properties

			HV* counter_props = newHV();

			// Get counter information
			cmapl_counter_def_t cntr_info;

			err = cmapl_get_counter_info(cntr_handle, &cntr_info);
			if (err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}

			cmapl_cnt_size_t * c_size;
			cmapl_cnt_type_t* c_type;
			cmapl_cnt_unit_t* c_unit;
			err = cmapl_get_counter_type(cntr_handle, &c_size, &c_type, &c_unit);
			if (err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
				
			char* counter_size  = get_counter_size(c_size);
			char* counter_type  = get_counter_type(c_type);
			char* counter_unit  = get_counter_unit(c_unit);
				
			SV* cntr_name = newSVpvf("%s", cntr_info.name);
			SV* cntr_id   = newSViv(cntr_info.cnt_id);
			SV* cntr_size = newSVpvf("%s", counter_size);
			SV* cntr_type = newSVpvf("%s", counter_type);
			SV* cntr_unit = newSVpvf("%s", counter_unit);
			
			hv_store(counter_props, "cntr_name", 9, cntr_name, 0);
			hv_store(counter_props, "cntr_id", 7, cntr_id, 0);
			hv_store(counter_props, "cntr_size", 9, cntr_size, 0);
			hv_store(counter_props, "cntr_type", 9, cntr_type, 0);
			hv_store(counter_props, "cntr_unit", 9, cntr_unit, 0);
				
			av_push(obj_cntr_array, newRV_noinc((SV*)counter_props));

			// Get next counter
			err = cmapl_next_counter(current_obj, &cntr_handle);
			if(err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
		
		}
			
		// Store the object and counter properties
		hv_store(obj_counters, obj_info.name, strlen(obj_info.name),newRV_noinc((SV*)obj_cntr_array),0);

		// Get next object handler
		err = cmapl_next_object(header_handle, &current_obj);
	
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
	}
	
	av_push(object_array, newRV_noinc((SV*)obj_counters));

	// Close the archive file handle
	err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*)error_array);
	}

	hv_store(obj_counter_hash, "object_array", 12, newRV_noinc((SV*)object_array), 0);
	hv_store(obj_counter_hash, "error_array", 11, newRV_noinc((SV*)error_array), 0);

	return newRV_noinc((SV*)obj_counter_hash);
}


// Function to get directory contents
// Returns a hash ref of arrays 
SV* _get_dir_contents(SV* hash_ref) {
	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* directory;
	unsigned long start_time;
	unsigned long end_time;
	char* key;

	
	if(! SvROK(hash_ref)) {
		croak("\n Hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "directory") == 0) {
			directory = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "start_time") == 0) {
			start_time = SvUV(sv_value);
			
		} else if (strcmp(key, "end_time") == 0) {
			end_time = SvUV(sv_value);
		}
	}
	
	AV* error_array = newAV();
	AV* meta_path_array = newAV();
	AV* data_path_array = newAV();

	HV* hash_datafiles = newHV();

	cmapl_error_t error = CMAPL_SUCCESS;
	cmapl_directory_def_t dir_def;
    cmapl_set_latest_version(&dir_def);
    dir_def.directory_path = directory;
    dir_def.start_time = start_time;
    dir_def.end_time = end_time;
    dir_def.period = 1000;

    cmapl_directory_handle_t* dir;
    error = cmapl_open_directory(&dir_def, &dir);
    if (error != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[error]));
		return newRV_noinc((SV*)error_array);
    }

    // find all the files
    uint32_t num_files;
    cmapl_file_def_t_v6_0* file_infos;
    error = cmapl_get_file_infos(dir, &num_files, (void**)&file_infos);
    if (error != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[error]));
		return newRV_noinc((SV*)error_array);
    }
    uint32_t file_index;
    for (file_index = 0; file_index < num_files; file_index++){
		av_push(meta_path_array , newSVpvf("%s", file_infos[file_index].meta_path));
		av_push(data_path_array , newSVpvf("%s", file_infos[file_index].data_path));
     }

	// close directory handle
	error = cmapl_close_directory(dir);
	if (error == CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[error]));
	} else {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[error]));
		return newRV_noinc((SV*)error_array);
	}
	
	hv_store(hash_datafiles, "meta_files", 10, newRV_noinc((SV*)meta_path_array),0);
	hv_store(hash_datafiles, "data_files", 10, newRV_noinc((SV*)data_path_array),0);
	hv_store(hash_datafiles, "error_array", 11, newRV_noinc((SV*)error_array),0);
	
	return newRV_noinc((SV*)hash_datafiles);
}

// Function to retrieve error code for invalid sample period
// Returns an array ref of error codes
SV*  _get_incorrect_sample_period(SV* hash_ref) {
	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* datafile;
	int sample_period;
	
	char* key;
	
	if(! SvROK(hash_ref)) {
		croak("\n Hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			datafile = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "sample_period") == 0) {
			sample_period = SvIV(sv_value);
		} 
	}
	
	AV* error_array = newAV();

	cmapl_error_t err = CMAPL_SUCCESS;
	const char* meta_path;
	const char* data_path;
	cmapl_file_handle_t* file_handle;
	
	err = cmapl_open_data_file(datafile, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	if(file_handle != NULL){
		// Get the start time and end time of the datafile
		uint64_t start_time = 0;
		err = cmapl_get_oldest_record_time(file_handle, &start_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		uint64_t end_time = 0;
		err = cmapl_get_newest_record_time(file_handle, &end_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Get the header handle of the datafile
		cmapl_header_handle_t* header;
		err = cmapl_get_header(file_handle, &header);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Create a query object
		cmapl_query_def_t query_def;
		cmapl_query_handle_t* query_handle;
		cmapl_set_latest_version(&query_def);
		
		// Add attributes to query def object
		query_def.object_name = "volume";

		// Assign instance info
		const char *key_cnt_names[] = {"instance_name"};
		query_def.key_cnt_names = key_cnt_names;
		
		cmapl_cnt_value_t val;
		val.val_str = "vol0";
		cmapl_cnt_value_t vals[] = {val};

		cmapl_counter_data_t_v6_0 key_cnt_value;
		key_cnt_value.value_type = CMAPL_CNT_STRING;
		key_cnt_value.num_values = 1;
		key_cnt_value.values = vals;

		cmapl_counter_data_t_v6_0 key_cnt_values[] = {key_cnt_value};
		query_def.key_cnt_values = key_cnt_values;
		query_def.num_key_cnts = 0;
		
		query_def.start_time = start_time;
		query_def.end_time = end_time;
		query_def.period = sample_period;

		// Assign counter info
		query_def.num_cnts = 2;
		const char* cnt_names[] = {"read_ops", "write_ops"};
		query_def.cnt_names = cnt_names;
		
		err = cmapl_query(file_handle, &query_def, &query_handle);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
	
	} else {
		printf("\n File handle is null");
	}

	//Close the archive file handle
    err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}

	return newRV_noinc((SV*)error_array);
}

// Function to get the first counter 
// Returns a hash ref with object array
// and error array

SV* _get_first_counter(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* object_array = newAV();
	HV* obj_properties = newHV();

	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	char* key;
	uint64_t start_time = 0;
	uint64_t end_time = 0;

	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} 
	}
	
	/* Open the data file */
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
	
	/* Get the start time  */
	err = cmapl_get_oldest_record_time(file_handle, &start_time);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }

	/* Get the end time */
	err = cmapl_get_newest_record_time(file_handle, &end_time);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }

	/* Get the header handle of the datafile */
	cmapl_header_handle_t* header;
	err = cmapl_get_header(file_handle, &header);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	/* Find a specific object in the datafile */
	cmapl_object_handle_t* current_obj;
	err = cmapl_find_object(header, obj_name , &current_obj);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	if(current_obj != NULL) {
		/* Get counter handle */
		
		cmapl_counter_handle_t * cntr_handle;
		err = cmapl_first_counter(current_obj, &cntr_handle);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		if(cntr_handle != NULL) {
			cmapl_counter_def_t counter_info;
			err = cmapl_get_counter_info(cntr_handle,&counter_info);
			if(err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			} 
		
			/* Dump Counter Info */
			av_push(object_array, newSVpvf("%s",counter_info.name));
		}
	}

	/* Close daatfile */
    err = cmapl_close_data_file(file_handle);
    	
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
		
	hv_store(obj_properties, "object_array", 12, newRV_noinc((SV*)object_array), 0);
	hv_store(obj_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);

	/* Return result */
	return newRV_noinc((SV*) obj_properties);
}

// Function to get the first object 
// Returns a hash ref with object array
// and error array

SV* _get_first_object(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* object_array = newAV();
	HV* obj_properties = newHV();

	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* key;
	uint64_t start_time = 0;
	uint64_t end_time = 0;

	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		}
	}
	
	/* Open the data file */
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }

	/* Get the header handle of the datafile */
	cmapl_header_handle_t* header;
	err = cmapl_get_header(file_handle, &header);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	/* Find a first object in the datafile */
	cmapl_object_handle_t* first_obj;
	
	err = cmapl_first_object(header, &first_obj);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	if(first_obj != NULL) {
		/* Get Object Info */

		cmapl_object_def_t obj_info;
		err = cmapl_get_object_info(first_obj, &obj_info);
		
		if (err != CMAPL_SUCCESS) {
			printf("\nError while retrieving object information : %s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*)error_array);
		}
		
		av_push(object_array, newSVpvf("%s",obj_info.name));
		
	}

	/* Close daatfile */
    err = cmapl_close_data_file(file_handle);
    	
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
		
	hv_store(obj_properties, "object_array", 12, newRV_noinc((SV*)object_array), 0);
	hv_store(obj_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);

	/* Return result */
	return newRV_noinc((SV*) obj_properties);
}


// Function to get invalid args error code
// Returns an array ref

SV* _get_invalid_args_err_code(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	
	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	char* key;
	uint64_t start_time = 0;
	uint64_t end_time = 0;

	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} 
	}
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	if(file_handle != NULL){
		// Get the start time and end time of the datafile
		
		uint64_t start_time = 0;
		
		err = cmapl_get_oldest_record_time(file_handle, &start_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		uint64_t end_time = 0;
		err = cmapl_get_newest_record_time(file_handle, &end_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Get the header handle of the datafile
		cmapl_header_handle_t* header;
		err = cmapl_get_header(file_handle, &header);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Create a query object
		cmapl_query_def_t query_def;
		cmapl_query_handle_t* query_handle;
		cmapl_set_latest_version(&query_def);
		
		// Add attributes to query def object
		query_def.object_name = obj_name;

		// Assign instance info
		const char *key_cnt_names[] = {"instance_name", "vserver_name"};
		query_def.key_cnt_names = key_cnt_names;
		
		cmapl_cnt_value_t volume_val;
		volume_val.val_str = "vol0";
		cmapl_cnt_value_t volume_vals[] = {volume_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_volume;
		key_cnt_value_volume.value_type = CMAPL_CNT_STRING;
		key_cnt_value_volume.num_values = 1;
		key_cnt_value_volume.values = volume_vals;
		
		cmapl_cnt_value_t vserver_val;
		vserver_val.val_str = "vs0";
		cmapl_cnt_value_t vserver_vals[] = {vserver_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_vserver;
		key_cnt_value_vserver.value_type = CMAPL_CNT_STRING;
		key_cnt_value_vserver.num_values = 1;
		key_cnt_value_vserver.values = vserver_vals;

		cmapl_counter_data_t_v6_0 key_cnt_values[] = {key_cnt_value_volume, key_cnt_value_vserver};
		query_def.key_cnt_values = key_cnt_values;
		query_def.num_key_cnts = 2;
		
		query_def.start_time = start_time;
		query_def.end_time = end_time;
		query_def.period = 1000;

		// Assign counter info
		query_def.num_cnts = 2;
		const char* cnt_names[] = {"read_ops", "write_ops"};
		query_def.cnt_names = cnt_names;
		
		err = cmapl_query(file_handle, &query_def, &query_handle);
		
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		if(query_handle != NULL) {
			cmapl_timestamp_handle_t* timestamp;
			err = cmapl_first_timestamp(query_handle, &timestamp);
			if(err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
			

			while(timestamp != NULL) {
				/*  Dump timestamp */
				uint64_t time = 0;
				err = cmapl_get_timestamp(timestamp, &time);
				if (err != CMAPL_SUCCESS){
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				cmapl_instance_handle_t* instance_handle;
				err = cmapl_find_instance(timestamp, key_cnt_names, key_cnt_values, 1, &instance_handle);
				if(err != CMAPL_SUCCESS) {
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}

				err = cmapl_next_timestamp(query_handle, &timestamp);
				if(err != CMAPL_SUCCESS) {
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
			} 

		} else {
			printf("\n Query handle is NULL");
		}
	
	} else {
		printf("\n File handle is null");
	}
	//Close the archive file handle
    err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	return newRV_noinc((SV*) error_array);
}


// Function to get error code when invalid timestamp
// are provided
// Returns error codes

SV* _get_invalid_timestamp(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	
	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	char* key;
	uint64_t start_time = 0;
	uint64_t end_time = 0;

	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} 
	}
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	if(file_handle != NULL){
		// Get the start time and end time of the datafile
		
		uint64_t start_time = 0;
		
		err = cmapl_get_oldest_record_time(file_handle, &start_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		uint64_t end_time = 0;
		err = cmapl_get_newest_record_time(file_handle, &end_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Get the header handle of the datafile
		cmapl_header_handle_t* header;
		err = cmapl_get_header(file_handle, &header);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Create a query object
		cmapl_query_def_t query_def;
		cmapl_query_handle_t* query_handle;
		cmapl_set_latest_version(&query_def);
		
		// Add attributes to query def object
		query_def.object_name = obj_name;

		// Assign instance info
		const char *key_cnt_names[] = {"instance_name", "vserver_name"};
		query_def.key_cnt_names = key_cnt_names;
		
		cmapl_cnt_value_t volume_val;
		volume_val.val_str = "vol0";
		cmapl_cnt_value_t volume_vals[] = {volume_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_volume;
		key_cnt_value_volume.value_type = CMAPL_CNT_STRING;
		key_cnt_value_volume.num_values = 1;
		key_cnt_value_volume.values = volume_vals;
		
		cmapl_cnt_value_t vserver_val;
		vserver_val.val_str = "vs0";
		cmapl_cnt_value_t vserver_vals[] = {vserver_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_vserver;
		key_cnt_value_vserver.value_type = CMAPL_CNT_STRING;
		key_cnt_value_vserver.num_values = 1;
		key_cnt_value_vserver.values = vserver_vals;

		cmapl_counter_data_t_v6_0 key_cnt_values[] = {key_cnt_value_volume, key_cnt_value_vserver};
		query_def.key_cnt_values = key_cnt_values;
		query_def.num_key_cnts = 2;
		
		query_def.start_time = end_time;
		query_def.end_time = start_time;
		query_def.period = 1000;

		// Assign counter info
		query_def.num_cnts = 2;
		const char* cnt_names[] = {"read_ops", "write_ops"};
		query_def.cnt_names = cnt_names;
		
		err = cmapl_query(file_handle, &query_def, &query_handle);
		
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

	} 

	//Close the archive file handle
    err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	return newRV_noinc((SV*) error_array);
}


// Function to find a specific counter 
// Returns the counter properties as hash ref 
// else error code as array ref

SV* _find_specific_counter(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* counter_array = newAV();
	HV* obj_properties = newHV();

	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	char* cntr_name;
	int cntr_id;
	char* key;
	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "counter_name") == 0) {
			cntr_name = SvPV(sv_value, PL_na);
		}
	}
	
	/* Open the data file */
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
	
	/* Get the header handle of the datafile */
	cmapl_header_handle_t* header;
	err = cmapl_get_header(file_handle, &header);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	/* Find a specific object in the datafile */
	cmapl_object_handle_t* current_obj;
	err = cmapl_find_object(header, obj_name , &current_obj);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	if(current_obj != NULL) {
		/* Get object info */

		cmapl_object_def_t obj_info;
		err = cmapl_get_object_info(current_obj,&obj_info);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		} 

		cmapl_counter_handle_t* cntr_handle;
		err = cmapl_find_counter_by_name(current_obj,cntr_name, &cntr_handle);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		} 

		if(cntr_handle != NULL) {

			/* Get counter information */

			cmapl_counter_def_t cntr_info;
			err = cmapl_get_counter_info(cntr_handle,&cntr_info);
			if(err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
			av_push(counter_array, newSVpvf("%s",cntr_info.name));
			av_push(counter_array, newSViv(cntr_info.cnt_id));
		}
		
	}

	/* Close daatfile */
    err = cmapl_close_data_file(file_handle);
    	
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
		
	hv_store(obj_properties, "counter_array", 13, newRV_noinc((SV*)counter_array), 0);
	hv_store(obj_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);

	/* Return result */
	return newRV_noinc((SV*) obj_properties);
}

// Function to find a specific counter 
// Returns the counter properties as hash ref 
// else error code as array ref

SV* _find_specific_counter_by_id(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* counter_array = newAV();
	HV* obj_properties = newHV();

	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	int cntr_id;
	char* key;
	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "counter_id") == 0) {
			cntr_id = SvIV(sv_value);
		}
	}
	
	/* Open the data file */
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
	
	/* Get the header handle of the datafile */
	cmapl_header_handle_t* header;
	err = cmapl_get_header(file_handle, &header);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	/* Find a specific object in the datafile */
	cmapl_object_handle_t* current_obj;
	err = cmapl_find_object(header, obj_name , &current_obj);
	if(err != CMAPL_SUCCESS) {
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
		
	if(current_obj != NULL) {
		/* Get object info */

		cmapl_object_def_t obj_info;
		err = cmapl_get_object_info(current_obj,&obj_info);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		} 

		cmapl_counter_handle_t* cntr_handle;
		err = cmapl_find_counter_by_id(current_obj,cntr_id, &cntr_handle);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		} 

		if(cntr_handle != NULL) {

			/* Get counter information */

			cmapl_counter_def_t cntr_info;
			err = cmapl_get_counter_info(cntr_handle,&cntr_info);
			if(err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
			av_push(counter_array, newSVpvf("%s",cntr_info.name));
			av_push(counter_array, newSViv(cntr_info.cnt_id));
		}
		
	}

	/* Close daatfile */
    err = cmapl_close_data_file(file_handle);
    	
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
    }
		
	hv_store(obj_properties, "counter_array", 13, newRV_noinc((SV*)counter_array), 0);
	hv_store(obj_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);

	/* Return result */
	return newRV_noinc((SV*) obj_properties);
}

// Function to find a particular instance 
// Returns error code if instance is not found
// Else returns all the instances for all time periods

SV* _find_specific_instance(SV* hash_ref) {
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* object_instances_array = newAV();
	HV* obj_inst_properties = newHV();

	HV* hash;
	HE* hash_entry;
	HE* hash_instance_entry;
	int num_keys, i, g, result, num_instances;
	SV* sv_key;
	SV* sv_value;
	SV* sv_key_inst;
	SV* sv_key_val;
	char* file_path;
	const char* obj_name;
	const char* inst_key;
	const char* vserver_key;
	const char* inst_value;
	const char* vserver_value;

	HV* hash_instances;
	char* key;
	char* key_val;
	int sample_period;

	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} 
	}
	
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}

	if(file_handle != NULL){
		
		uint64_t start_time = 0;
		err = cmapl_get_oldest_record_time(file_handle, &start_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		uint64_t end_time = 0;
		err = cmapl_get_newest_record_time(file_handle, &end_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		// Get the header handle of the datafile
		cmapl_header_handle_t* header;
		err = cmapl_get_header(file_handle, &header);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Create a query object
		cmapl_query_def_t query_def;
		cmapl_query_handle_t* query_handle;
		cmapl_set_latest_version(&query_def);
		
		// Add attributes to query def object
		query_def.object_name = obj_name;

		// Assign instance info
		const char *key_cnt_names[] = {"instance_name","vserver_name"};
		query_def.key_cnt_names = key_cnt_names;
		
		cmapl_cnt_value_t volume_val;
		volume_val.val_str = 'root_vs0';
		cmapl_cnt_value_t volume_vals[] = {volume_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_volume;
		key_cnt_value_volume.value_type = CMAPL_CNT_STRING;
		key_cnt_value_volume.num_values = 1;
		key_cnt_value_volume.values = volume_vals;
		
		cmapl_cnt_value_t vserver_val;
		vserver_val.val_str = "vs0";
		cmapl_cnt_value_t vserver_vals[] = {vserver_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_vserver;
		key_cnt_value_vserver.value_type = CMAPL_CNT_STRING;
		key_cnt_value_vserver.num_values = 1;
		key_cnt_value_vserver.values = vserver_vals;

		cmapl_counter_data_t_v6_0 key_cnt_values[] = {key_cnt_value_volume, key_cnt_value_vserver};
		query_def.key_cnt_values = key_cnt_values;
		query_def.num_key_cnts = 2;
		
		query_def.start_time = start_time;
		query_def.end_time = end_time;
		query_def.period = 5000;

		// Assign counter info
		query_def.num_cnts = 2;
		const char* cnt_names[] = {"read_ops", "write_ops"};
		query_def.cnt_names = cnt_names;
		
		err = cmapl_query(file_handle, &query_def, &query_handle);
		
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		if(query_handle != NULL) {
			printf("\n Retrieve the timestamp handle");
			cmapl_timestamp_handle_t* timestamp;
			err = cmapl_first_timestamp(query_handle, &timestamp);
			
			
			if(err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}

			if (err == CMAPL_RECORD_EMPTY){
				printf("\t\t\tAll records sampled at this period were empty in this file.\n\n");
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
			
			HV* timestamp_hash = newHV();

			while(timestamp != NULL) {
				/*  Dump timestamp */
				uint64_t time = 0;
				err = cmapl_get_timestamp(timestamp, &time);
				if (err != CMAPL_SUCCESS){
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				/* Convert the timestamp from unsigned long to char* to be used as hash key */
				unsigned long time2;
				time2 = time;
				const int n = snprintf(NULL, 0, "%lu", time2);
				char time_string[n+1];
				snprintf(time_string, n+1, "%lu", time2);
				
				cmapl_instance_handle_t* instance_handle;
				err = cmapl_find_instance(timestamp, key_cnt_names, key_cnt_values, 2, &instance_handle);
				if(err != CMAPL_SUCCESS) {
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				AV* instance_counters_array = newAV();
				if(instance_handle != NULL) {
					
					HV* hash_key_counters = newHV();

					/* Get instance data */
					cmapl_instance_data_t instance_data;
					err = cmapl_get_instance_data(instance_handle, &instance_data);
					if (err != CMAPL_SUCCESS){
						av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
						return newRV_noinc((SV*) error_array);
					}
					
					// dump key counters data (instance name)
					uint32_t i;
					for (i = 0; i < instance_data.num_key_cnts; i++){
						AV* key_counter_values = newAV();
						
						uint32_t j;
						SV* cnt_value;

						for (j = 0; j < instance_data.key_cnt_values[i].num_values; j++){
							switch(instance_data.key_cnt_values[i].value_type){
								case CMAPL_CNT_4BYTES:
                                    					cnt_value = newSViv(instance_data.key_cnt_values[i].values[j].val_4B);
                                    					break;
                                				
								case CMAPL_CNT_8BYTES:
                                    					cnt_value = newSVuv(instance_data.key_cnt_values[i].values[j].val_4B);
                                    					break;
                                
								case CMAPL_CNT_STRING:
                                    					cnt_value = newSVpv(instance_data.key_cnt_values[i].values[j].val_4B,0);
                                    					break;
							}
							
							if (j != (instance_data.key_cnt_values[i].num_values - 1)){
									//printf(", ");
							}
							av_push(key_counter_values, cnt_value);		
																					
						} // End of for

						// For all key instances values store the same counter array
							
						hv_store(hash_key_counters, instance_data.key_cnt_names[i], strlen(instance_data.key_cnt_names[i]), newRV_noinc((SV*)key_counter_values),0);
					} // end of for
					
					av_push(instance_counters_array, newRV_noinc((SV*)hash_key_counters));

				} 

				err = cmapl_next_timestamp(query_handle, &timestamp);
				if(err != CMAPL_SUCCESS) {
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}

				hv_store(timestamp_hash, time_string, strlen(time_string), newRV_noinc((SV*)instance_counters_array), 0);
			}
			
			av_push(object_instances_array , newRV_noinc((SV*)timestamp_hash));

		} 
	
	} 
	//Close the archive file handle
    	err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	hv_store(obj_inst_properties, "obj_instance_array", 18, newRV_noinc((SV*)object_instances_array), 0);
	hv_store(obj_inst_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);
	
	/* Return result */
	return newRV_noinc((SV*) obj_inst_properties);
}

// Function to get the first timestamp
// Returns the error code if timestamp is not found
// Else returns timestamp array

SV* _find_first_timestamp(SV* hash_ref) {

	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	AV* key_counter_values = newAV();
	HV* obj_inst_properties = newHV();

	HV* hash;
	HV* hash_instances;
	HE* hash_entry;
	HE* hash_instance_entry;
	int num_keys, i, g, result, num_instances;
	SV* sv_key;
	SV* sv_value;
	SV* sv_key_inst;
	SV* sv_key_val;
	char* file_path;
	const char* obj_name;
	char* key;
	char* key_val;
	char* inst_value;
	char* inst_key;
	char* vserver_value;
	char* vserver_key;
	
	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "key_cntr_instances") == 0) {

			if(! SvROK(sv_value)) {
				croak("Hash of instances is not a reference");
			}

			hash_instances = (HV*)SvRV(sv_value);
			num_instances = hv_iterinit(hash_instances);
			for(g = 0 ; g < num_instances; g++) {
				hash_instance_entry = hv_iternext(hash_instances);
				sv_key_inst = hv_iterkeysv(hash_instance_entry);
				sv_key_val = hv_iterval(hash_instances, hash_instance_entry);
				key_val = (char*)SvPV(sv_key_inst, PL_na);
				if(strcmp(key_val, "instance_name") == 0) {
					inst_value = SvPV(sv_key_val, PL_na);
					inst_key = (char*)SvPV(sv_key_inst, PL_na);
				} else if (strcmp(key_val, "vserver_name") == 0) {
					vserver_value = SvPV(sv_key_val, PL_na);
					vserver_key = (char*)SvPV(sv_key_inst, PL_na);
				}
			}
		}
	}
	
	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}

	if(file_handle != NULL){
		
		uint64_t start_time = 0;
		err = cmapl_get_oldest_record_time(file_handle, &start_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		uint64_t end_time = 0;
		err = cmapl_get_newest_record_time(file_handle, &end_time);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		// Get the header handle of the datafile
		cmapl_header_handle_t* header;
		err = cmapl_get_header(file_handle, &header);
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Create a query object
		cmapl_query_def_t query_def;
		cmapl_query_handle_t* query_handle;
		cmapl_set_latest_version(&query_def);
		
		// Add attributes to query def object
		query_def.object_name = obj_name;

		// Assign instance info
		const char *key_cnt_names[] = {inst_key,vserver_key};
		query_def.key_cnt_names = key_cnt_names;
		
		cmapl_cnt_value_t volume_val;
		volume_val.val_str = inst_value;
		cmapl_cnt_value_t volume_vals[] = {volume_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_volume;
		key_cnt_value_volume.value_type = CMAPL_CNT_STRING;
		key_cnt_value_volume.num_values = 1;
		key_cnt_value_volume.values = volume_vals;
		
		cmapl_cnt_value_t vserver_val;
		vserver_val.val_str = vserver_value;
		cmapl_cnt_value_t vserver_vals[] = {vserver_val};
		
		cmapl_counter_data_t_v6_0 key_cnt_value_vserver;
		key_cnt_value_vserver.value_type = CMAPL_CNT_STRING;
		key_cnt_value_vserver.num_values = 1;
		key_cnt_value_vserver.values = vserver_vals;

		cmapl_counter_data_t_v6_0 key_cnt_values[] = {key_cnt_value_volume, key_cnt_value_vserver};
		query_def.key_cnt_values = key_cnt_values;
		query_def.num_key_cnts = 2;
		
		query_def.start_time = start_time;
		query_def.end_time = end_time;
		query_def.period = 1000;

		// Assign counter info
		query_def.num_cnts = 2;
		const char* cnt_names[] = {inst_key,vserver_key};
		query_def.cnt_names = cnt_names;
		
		err = cmapl_query(file_handle, &query_def, &query_handle);
		
		if(err != CMAPL_SUCCESS) {
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		if(query_handle != NULL) {
			printf("\n Retrieve the timestamp handle");
			cmapl_timestamp_handle_t* timestamp;
			err = cmapl_first_timestamp(query_handle, &timestamp);
			 if(err != CMAPL_SUCCESS) {
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
			
			while(timestamp != NULL) {
				/*  Dump timestamp */
				uint64_t time = 0;
				err = cmapl_get_timestamp(timestamp, &time);
				if (err != CMAPL_SUCCESS){
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				av_push(key_counter_values, newSVuv(time));	

				cmapl_instance_handle_t* instance_handle;
				err = cmapl_first_instance(timestamp,&instance_handle);
				if(err != CMAPL_SUCCESS) {
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				while(instance_handle != NULL) {
					
					/* Get instance data */
					cmapl_instance_data_t instance_data;
					err = cmapl_get_instance_data(instance_handle, &instance_data);
					if (err != CMAPL_SUCCESS){
						av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
						return newRV_noinc((SV*) error_array);
					}
					
					// dump key counters data (instance name)
					uint32_t i;
					for (i = 0; i < instance_data.num_key_cnts; i++){
						
						uint32_t j;
						SV* cnt_value;

						for (j = 0; j < instance_data.key_cnt_values[i].num_values; j++){
							switch(instance_data.key_cnt_values[i].value_type){
								case CMAPL_CNT_4BYTES:
                                    					cnt_value = newSViv(instance_data.key_cnt_values[i].values[j].val_4B);
                                    					break;
                                
								case CMAPL_CNT_8BYTES:
                                    					cnt_value = newSVuv(instance_data.key_cnt_values[i].values[j].val_8B);
                                    					break;
                                				case CMAPL_CNT_STRING:
                                    					cnt_value = newSVpv(instance_data.key_cnt_values[i].values[j].val_str,0);
                                    					break;
							}
							
							if (j != (instance_data.key_cnt_values[i].num_values - 1)){
									//printf(", ");
							}
							av_push(key_counter_values, cnt_value);		
																					
						} // End of for

					} // end of for
					
					err = cmapl_next_instance(timestamp, &instance_handle);
					if (err != CMAPL_SUCCESS){
						printf("\nError while retrieving first instance value : %s",cmapl_error_labels[err]);	
						av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
						return newRV_noinc((SV*)error_array);
					}

				} // end of instance handle

				err = cmapl_next_timestamp(query_handle, &timestamp);
				if(err != CMAPL_SUCCESS) {
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}

			} // end of timestamp handle

		} // end of query handle
	
	} // end of file handle 

	//Close the archive file handle
    err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	hv_store(obj_inst_properties, "timestamp_array", 15, newRV_noinc((SV*)key_counter_values), 0);
	hv_store(obj_inst_properties, "error_array", 11, newRV_noinc((SV*)error_array), 0);
	
	/* Return result */
	return newRV_noinc((SV*) obj_inst_properties);
	
}

// Function to find the first instance and first counter data
// Returns a hash of timestamps and instances along with counter values
// Else returns an error array

SV* _find_first_instance(SV* hash_ref){
	
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    cmapl_file_handle_t* file_handle;
    const char* meta_path;
    const char* data_path;
	
	AV* error_array = newAV();
	HV* timestamp_hash = newHV();
	
	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	char* key;
	uint64_t start_time = 0;
	uint64_t end_time = 0;

	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} 
	}

	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    if (err != CMAPL_SUCCESS){
		printf("\n Error opening file with error label : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	if(file_handle != NULL){
		// Get the start time and end time of the datafile
		
		uint64_t start_time = 0;
		err = cmapl_get_oldest_record_time(file_handle, &start_time);
		if(err != CMAPL_SUCCESS) {
			printf("\nError for oldest time :%s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		uint64_t end_time = 0;
		err = cmapl_get_newest_record_time(file_handle, &end_time);
		if(err != CMAPL_SUCCESS) {
			printf("\nError for latest time :%s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		// Get the header handle of the datafile
		cmapl_header_handle_t* header;
		err = cmapl_get_header(file_handle, &header);
		if(err != CMAPL_SUCCESS) {
			//printf("\nError while retrieving header information : %s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Create a query object
		cmapl_query_def_t query_def;
		cmapl_query_handle_t* query_handle;
		cmapl_set_latest_version(&query_def);
		
		// Add attributes to query def object
		query_def.object_name = "volume";

		// Assign instance info
		const char *key_cnt_names[] = {"instance_name"};
		query_def.key_cnt_names = key_cnt_names;
		
		cmapl_cnt_value_t val;
		val.val_str = "root_vs0";
		cmapl_cnt_value_t vals[] = {val};

		cmapl_counter_data_t_v6_0 key_cnt_value;
		key_cnt_value.value_type = CMAPL_CNT_STRING;
		key_cnt_value.num_values = 1;
		key_cnt_value.values = vals;

		cmapl_counter_data_t_v6_0 key_cnt_values[] = {key_cnt_value};
		query_def.key_cnt_values = key_cnt_values;
		query_def.num_key_cnts = 1;
		
		query_def.start_time = start_time;
		query_def.end_time = end_time;
		query_def.period = 5000;

		// Assign counter info
		query_def.num_cnts = 2;
		const char* cnt_names[] = {"read_ops", "write_ops"};
		query_def.cnt_names = cnt_names;
		
		err = cmapl_query(file_handle, &query_def, &query_handle);
		
		if(err != CMAPL_SUCCESS) {
			//printf("\nError while query handle :%s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		if(query_handle != NULL) {
			//printf("\n Retrieve the timestamp handle");
			cmapl_timestamp_handle_t* timestamp;
			err = cmapl_first_timestamp(query_handle, &timestamp);
			if(err != CMAPL_SUCCESS) {
				//printf("\nError while retrieving first timestamp :%s",cmapl_error_labels[err]);
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
			

			while(timestamp != NULL) {
				/*  Dump timestamp */
				uint64_t time = 0;
				err = cmapl_get_timestamp(timestamp, &time);
				if (err != CMAPL_SUCCESS){
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				//printf("\t\t\tTimestamp: %"PRIu64"\n\n", time);
				
				/* Convert the timestamp from unsigned long to char* to be used as hash key */
				unsigned long time2;
				time2 = time;
				const int n = snprintf(NULL, 0, "%lu", time2);
				char time_string[n+1];
				snprintf(time_string, n+1, "%lu", time2);

				cmapl_instance_handle_t* instance_handle;
				err = cmapl_first_instance(timestamp, &instance_handle);
				if(err != CMAPL_SUCCESS) {
					printf("\nError while retrieving first instance :%s",cmapl_error_labels[err]);
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				AV* instance_counters_array = newAV();

				if(instance_handle != NULL) {
			
					/* Get instance data */
					cmapl_instance_data_t instance_data;
					err = cmapl_get_instance_data(instance_handle, &instance_data);
					if (err != CMAPL_SUCCESS){
						printf("\nError while retrieving first instance value : %s",cmapl_error_labels[err]);
						av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
						return newRV_noinc((SV*) error_array);
					}
					
					HV* hash_key_counters = newHV();

					// dump key counters data (instance name)
					uint32_t i;
					for (i = 0; i < instance_data.num_key_cnts; i++){
						AV* counter_values = newAV();

						uint32_t j;
						SV* cnt_value;
						for (j = 0; j < instance_data.key_cnt_values[i].num_values; j++){
							switch(instance_data.key_cnt_values[i].value_type){
								case CMAPL_CNT_4BYTES:
                                    					cnt_value = newSViv(instance_data.key_cnt_values[i].values[j].val_4B);
                                    					break;
                                				
								case CMAPL_CNT_8BYTES:
									cnt_value = newSVuv(instance_data.key_cnt_values[i].values[j].val_8B);    
                                    					break;
                                				case CMAPL_CNT_STRING:
                                    					cnt_value = newSVpv(instance_data.key_cnt_values[i].values[j].val_str,0);
                                    					break;
							}
							
							if (j != (instance_data.key_cnt_values[i].num_values - 1)){
									//printf(", ");
							}
							av_push(counter_values , cnt_value);
							
						} // End of for
						printf("\n");
					
						hv_store(hash_key_counters,instance_data.key_cnt_names[i], strlen(instance_data.key_cnt_names[i]), newRV_noinc((SV*)counter_values),0 );
					
					} // end of for
					
					av_push(instance_counters_array, newRV_noinc((SV*)hash_key_counters));
				
				} else {
					printf("\n Instance handle is NULL");	
				}

				hv_store(timestamp_hash, time_string, strlen(time_string), newRV_noinc((SV*)instance_counters_array), 0);
				
				err = cmapl_next_timestamp(query_handle, &timestamp);
				if(err != CMAPL_SUCCESS) {
					printf("\nError while retrieving next timestamp :%s",cmapl_error_labels[err]);
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
			} 

		} else {
			printf("\n Query handle is NULL");
		}
	
	} else {
		printf("\n File handle is null");
	}
	//Close the archive file handle
    	err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		printf("\nError while closing datafile :",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	return newRV_noinc((SV*)timestamp_hash);
}

// Function to get counter data
// Returns error array if error occurs 
// Else hash ref with all the counters

SV* _get_first_counter_data(SV* hash_ref){
	
	cmapl_error_t err = CMAPL_SUCCESS;

	// Open the specified archive file_handle
    	cmapl_file_handle_t* file_handle;
    	const char* meta_path;
    	const char* data_path;
	
	AV* error_array = newAV();
	HV* timestamp_hash = newHV();
	
	HV* hash;
	HE* hash_entry;
	int num_keys, i, result;
	SV* sv_key;
	SV* sv_value;
	char* file_path;
	char* obj_name;
	char* key;
	uint64_t start_time = 0;
	uint64_t end_time = 0;

	if(! SvROK(hash_ref)) {
		croak("hash ref is not a reference");
	}
	
	hash = (HV*)SvRV(hash_ref);
	num_keys = hv_iterinit(hash);
	for(i = 0; i < num_keys; i++) {
		hash_entry = hv_iternext(hash);
		sv_key = hv_iterkeysv(hash_entry);
		sv_value = hv_iterval(hash, hash_entry);
		key = (char*)SvPV(sv_key, PL_na);
		if(strcmp(key, "datafile") == 0) {
			file_path = SvPV(sv_value, PL_na);
		} else if (strcmp(key, "object") == 0) {
			obj_name = SvPV(sv_value, PL_na);
		} 
	}

	err = cmapl_open_data_file(file_path, "r", &file_handle, &meta_path, &data_path);
    	if (err != CMAPL_SUCCESS){
		printf("\n Error opening file with error label : %s",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	if(file_handle != NULL){
		// Get the start time and end time of the datafile
		printf("\n Start time");
		uint64_t start_time = 0;
		printf("\n Retrieving start time");
		err = cmapl_get_oldest_record_time(file_handle, &start_time);
		if(err != CMAPL_SUCCESS) {
			printf("\nError for oldest time :%s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		printf("\t\t\tStart Timestamp: %"PRIu64"\n\n", start_time);

		printf("\n End time");
		uint64_t end_time = 0;
		err = cmapl_get_newest_record_time(file_handle, &end_time);
		if(err != CMAPL_SUCCESS) {
			printf("\nError for latest time :%s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}
		
		printf("\t\t\tEnd Timestamp: %"PRIu64"\n\n", end_time);

		// Get the header handle of the datafile
		cmapl_header_handle_t* header;
		err = cmapl_get_header(file_handle, &header);
		if(err != CMAPL_SUCCESS) {
			printf("\nError while retrieving header information : %s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		// Create a query object
		cmapl_query_def_t query_def;
		cmapl_query_handle_t* query_handle;
		cmapl_set_latest_version(&query_def);
		
		printf("\n Assign object info to query def");
		// Add attributes to query def object
		query_def.object_name = obj_name;

		query_def.num_key_cnts = 0;
		
		query_def.start_time = start_time;
		query_def.end_time = end_time;
		query_def.period = 5000;

		// Assign counter info
		query_def.num_cnts = 2;
		const char* cnt_names[] = {"read_ops", "write_ops"};
		query_def.cnt_names = cnt_names;
		
		err = cmapl_query(file_handle, &query_def, &query_handle);
		
		if(err != CMAPL_SUCCESS) {
			printf("\nError while query handle :%s",cmapl_error_labels[err]);
			av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
			return newRV_noinc((SV*) error_array);
		}

		if(query_handle != NULL) {
			printf("\n Retrieve the timestamp handle");
			cmapl_timestamp_handle_t* timestamp;
			err = cmapl_first_timestamp(query_handle, &timestamp);
			if(err != CMAPL_SUCCESS) {
				printf("\nError while retrieving first timestamp :%s",cmapl_error_labels[err]);
				av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
				return newRV_noinc((SV*) error_array);
			}
			

			while(timestamp != NULL) {
				/*  Dump timestamp */
				uint64_t time = 0;
				err = cmapl_get_timestamp(timestamp, &time);
				if (err != CMAPL_SUCCESS){
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				printf("\t\t\tTimestamp: %"PRIu64"\n\n", time);
				
				/* Convert the timestamp from unsigned long to char* to be used as hash key */
				unsigned long time2;
				time2 = time;
				const int n = snprintf(NULL, 0, "%lu", time2);
				char time_string[n+1];
				snprintf(time_string, n+1, "%lu", time2);

				cmapl_instance_handle_t* instance_handle;
				err = cmapl_first_instance(timestamp, &instance_handle);
				if(err != CMAPL_SUCCESS) {
					printf("\nError while retrieving first instance :%s",cmapl_error_labels[err]);
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}
				
				AV* instance_counters_array = newAV();

				if(instance_handle != NULL) {
			
					HV* hash_key_counters = newHV();
					
						/* Iterate through all counter data in this instance */
						cmapl_counter_handle_t* ret_cnt;
						cmapl_counter_data_handle_t* cnt_data;
						err = cmapl_first_counter_data(instance_handle, &ret_cnt, &cnt_data);
						if (err != CMAPL_SUCCESS){
							printf("\nError while retrieving first instance value : %s",cmapl_error_labels[err]);
							av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
							return newRV_noinc((SV*)error_array);
						}
						
						HV* counter_value_hash = newHV();

						while (cnt_data != NULL){

							/* Get counter info */
							cmapl_counter_def_t cnt_info;
							err = cmapl_get_counter_info(ret_cnt, &cnt_info);
							if (err != CMAPL_SUCCESS){
								printf("\nError while retrieving counter info : %s",cmapl_error_labels[err]);
								av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
								return newRV_noinc((SV*)error_array);
							}
							
							//SV* counter_name = newSVpv(cnt_info.name, 0);
							
							/* Get counter data */
							cmapl_counter_data_t cnt_data_value;
							err = cmapl_get_counter_data_value(cnt_data, &cnt_data_value);
							if (err != CMAPL_SUCCESS){
								printf("\nError while retrieving counter info : %s",cmapl_error_labels[err]);
								av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
								return newRV_noinc((SV*)error_array);
							}

							/* Dump counter data */
													
							uint32_t count;
							SV* counter_value;

							// Store all the counter values in an array
							AV* counter_array = newAV();
							
							for (count = 0; count < cnt_data_value.num_values; count++){
								
								switch(cnt_data_value.value_type){
									case CMAPL_CNT_4BYTES:
										counter_value = newSViv(cnt_data_value.values[count].val_4B);
										//printf("\t %d", cnt_data_value.values[count].val_4B);
										break;
									
									case CMAPL_CNT_8BYTES:
										counter_value = newSVuv(cnt_data_value.values[count].val_8B);
										//printf("%"PRIu64, cnt_data_value.values[count].val_8B);
										break;
									
									case CMAPL_CNT_STRING:
										counter_value = newSVpv(cnt_data_value.values[count].val_str, 0);
										//printf("\t %s", cnt_data_value.values[count].val_str);
										break;
								}

								if (count != (cnt_data_value.num_values - 1)){
									//printf(", ");
								}
								
								// Push counter values in the array
								av_push(counter_array, counter_value);
							}
												
							hv_store(counter_value_hash, cnt_info.name, strlen(cnt_info.name), newRV_noinc((SV*)counter_array), 0);	
							
							/* Get next counter data in this instance */
							err = cmapl_next_counter_data(instance_handle, &ret_cnt, &cnt_data);
							if (err != CMAPL_SUCCESS){
								printf("\nError while retrieving counter info : %s",cmapl_error_labels[err]);
								av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
								return newRV_noinc((SV*)error_array);
							}
						} // end of while counter data
					
					av_push(instance_counters_array, newRV_noinc((SV*)counter_value_hash));
				}	

				hv_store(timestamp_hash, time_string, strlen(time_string), newRV_noinc((SV*)instance_counters_array), 0);
				
				err = cmapl_next_timestamp(query_handle, &timestamp);
				if(err != CMAPL_SUCCESS) {
					printf("\nError while retrieving next timestamp :%s",cmapl_error_labels[err]);
					av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
					return newRV_noinc((SV*) error_array);
				}

			} // end of timestamp handle

		} else {
			printf("\n Query handle is NULL");
		}
	
	} else {
		printf("\n File handle is null");
	}
	//Close the archive file handle
    	err = cmapl_close_data_file(file_handle);
	if (err != CMAPL_SUCCESS){
		printf("\nError while closing datafile :",cmapl_error_labels[err]);
		av_push(error_array, newSVpvf("%s",cmapl_error_labels[err]));
		return newRV_noinc((SV*) error_array);
	}
	
	return newRV_noinc((SV*)timestamp_hash);
}

// Function to read core files
SV* _read_core_files(char * corefile) {
	
}
