# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Perf/Priority.pm#1 $
#
## @author   dl-perf-qa@netapp.com
## @summary  NATE wrapper for ONTAP "priority" commands.
## @status   private
##
## @pod here

=head1 NAME

Perf::Priority - NATE wrapper for ONTAP "priority" commands

=head1 DESCIPTION

This is a factory class for Perf::Priority objects; it returns
an object of the appropriate class as specified by its "type"
argument. Currently, the supported types are (7-mode) "CLI"
and "ZAPI".

The only method defined in this class is "new"; see the
Perf::Priority subclasses for the list of methods they define.

=head1 METHODS

=head2 new

Returns Perf::Priority::CLI or Perf::Priority::ZAPI object

=over

=item Arguments

host - hostname of filer being tested

type - filer interface to use (CLI or ZAPI)

=back

=cut

package Perf::Priority;

use strict;
use warnings;
use Carp;
use Params::Validate qw(validate SCALAR);

sub new {
    my $class  = shift;
    my %params = validate(@_,
                          {
                              host => {
                                  type     => SCALAR,
                                  optional => 0,
                              },

                              type => {
                                  type     => SCALAR,
                                  optional => 1,
                                  default  => 'CLI',
                                  regex    => qr/^(?:CLI|ZAPI)$/,
                              },
                          });

    my $host   = $params{host};
    my $type   = $params{type};

    if ($type eq 'CLI') {
        require Perf::Priority::CLI;
        return Perf::Priority::CLI->new(host => $host);
    }
    elsif ($type eq 'ZAPI') {
        require Perf::Priority::ZAPI;
        return Perf::Priority::ZAPI->new(host => $host);
    }
    else {
        # Why wasn't this caught in params validation above?
        confess "Invalid type: '$type'";
    }
}

1;

