# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Alias for CM Aliases
## @status   review
## @pod here

package Perf::Stats::Alias::CM;{
    use strict;
    use warnings;
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Carp qw(confess);
    use List::MoreUtils qw(uniq);
    use Perf::Stats::Alias;
    use Object::InsideOut qw(Perf::Stats::Entry::CM Perf::Stats::Alias);

    sub _init :Init {
        my ($self) = @_;
        if ($self->get_all_types =~ /string/){
            confess("Aliasing is not supported on counters of type 'string'");
        }
    }

    sub compute {
        my ($self) = @_;
        my $type = $self->get_all_types;
        if ($type !~ /average|percent/){
            return $self->SUPER::compute;
        }
        my $value;
        my $sum = 0;
        my $count = 0;
        for my $counter (@{$self->get_counters}){
            $sum   += $counter->get_sum;
            $count += $counter->get_count;
        }
        $value = "$sum/$count";
        if ($type =~ /percent/){
            $value .= "%";
        }
        return $value;
    }
    sub get_calc_value {
        my ($self,$value) = @_;
        my $type = $self->get_all_types;

        if($type =~ /rate/)
        {
            my $value;
            for my $counter (@{$self->get_counters}){
                $value += $counter->get_calc_value;
            }
            return $value;
        }
        elsif ($type =~ /average|percent/) {
            my $total;
            my $count;
            if ($value =~/(\d+\.?\d+?)\/(\d+\.?\d+?)/){
                $total = $1;
                $count = $2;
            }
            if($count > 0) {
                return sprintf("%.2f",$total/$count);
            }
            elsif (!$count && !$total) {
                    return 0;
            }
            else{
                Carp::cluck "This should never happen!\n".
                            "Entry: ".$self->get_name."\n" .
                            "Delta Count: $count\n".
                            "Delta Total: $total\n";     # But sometimes it does...
            }

        }
        else {
            return $value;
        }
    }

}
1;
