# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Class responsible for generating cluster aliases
## @status   review
## @pod here

package Perf::Stats::AliasFactory::Cluster;{
    use strict;
    use warnings;
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Data::Dumper;
    use Carp qw( confess );
    use Perf::Stats::AliasFactory;
    use Object::InsideOut qw( Perf::Stats::AliasFactory );

    sub detect_nodes_in_cluster {
        my ($self) = @_;
        my $parent = $self->get_parent;
        my $node_cluster_map = {};
        my $nodes = $parent->get_nodes;
        for my $node (@$nodes){
            if (!$node_cluster_map->{$node}){
                my $handlers = $parent->_get_handlers( node => $node );
                confess "No handlers found for node '$node'?" if (!$handlers || !@$handlers);
                # Just pick one
                my $handler = $handlers->[0];
                my $cluster = $handler->get_cluster_name();
                foreach my $node (@{$handler->get_cluster_nodes()}) {
                    $node_cluster_map->{$node} = $cluster;
                }
            }
            confess "Node '$node' not detected?" if (!$node_cluster_map->{$node});
        }
        $Log->trace("Cluster information detected:\n" . Dumper($node_cluster_map));
        return $node_cluster_map;
    }

    sub process_aliases {
        my ($self, $counters) = @_;
        my $node_cluster = $self->detect_nodes_in_cluster;
        #
        # Counters names are of form "node:object:instance:name". Alias names
        # will be "cluster:object:instance:name"
        #
        my %alias_names;
        for my $counter (@$counters){
            my $node = $counter->get_node;
            my $cluster = $node_cluster->{$node};
            confess("No cluster found for node '$node'") if (!$cluster);
            my $alias_name = "$cluster:" . $counter->get_name;
            if (!$alias_names{$alias_name}){
                $alias_names{$alias_name} = [$counter];
            }
            else{
                push @{$alias_names{$alias_name}}, $counter;
            }
        }

        #
        # Create the aliases
        #
        my @aliases;
        for my $alias_name (keys %alias_names){
            my $counters = $alias_names{$alias_name};
            my $alias = Perf::Stats::Alias->new(  name                => $alias_name,
                                            counters            => $counters,
                                         );
            #                                        dynamic             => 0,
            #                                        counters_as_objs    => 1,
            #                                        hide_counters       => $self->does_hide_counters,
            #                                        source              => $self->get_source_name,
            #                                    );
            #confess "No alias returned?" if (!$alias);
            #if ($self->is_dynamic){
            #    # Override the regexs so that new nodes get picked up automatically
            #    # Replace the cluster name with .*
            #    my $alias_regex = $alias->get_name;
            #    $alias_regex =~ s/^[^:]+/.*/;
            #    $Log->comment("Setting alias regex to '$alias_regex'");
            #    $alias->set_regexs([$alias_regex]);
            #}
            push @aliases, $alias;
        }
        return \@aliases;
    }

}
1;
