# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Base class for Stats Counter
## @status   review
## @pod here

package Perf::Stats::Counter;{
    use strict;
    use warnings;
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Carp qw(confess);
    use Perf::Stats::Entry;
    use Object::InsideOut qw( Perf::Stats::Entry );
    use Perf::Stats::Counter::CM;
    use Scalar::Util qw(weaken);

    my @handler     :Field :Arg(Name=>'handler',   Mandatory=>1) :Weak;
    my @value       :Field :Arg(Name=>'value',     Mandatory=>0) :Std(value);
    my @hidden      :Field :Get(is_hidden);
    my @aliases     :Field :Name('aliases'); # Keeps track of aliases referencing it


    sub _init :Init {
        my ($self) = @_;
        $aliases[$$self] = [];
    }

    sub get_source {
        my ($self) = @_;
        return $handler[$$self]->get_source;
    }

    sub hide {
        my ($self) = @_;
        $hidden[$$self] = 1;
        return;
    }

    sub unhide {
        my ($self) = @_;
        $hidden[$$self] = 0;
        return;
    }

    sub add_alias {
        my ($self, $alias) = @_;
        confess "You must pass an Alias argument"
            if (!$alias || !ref($alias) || !$alias->isa('Perf::Stats::Alias'));
        push @{$aliases[$$self]}, $alias;
        weaken($aliases[$$self]->[-1]);
        return;
    }

    sub get_aliases {
        my ($self) = @_;
        # Remove the undef entries
        my @filtered;
        for my $a (@{$aliases[$$self]}){
            push @filtered, $a if ($a);
        }
        $aliases[$$self] = \@filtered;
        return \@filtered;
    }


    sub delta {
        my $self = shift();

        confess('Delta not supported for cpstats')
            if ('cpstats' eq $self->get_source());

        return $self->SUPER::delta(@_);
    }
}
1;
