# This is a common class for counters sourced from the Counter Manager
# RPC daemon such as ZAPI and cm_client counters

package Perf::Stats::Counter::CM::RPC; {
    use strict;
    use warnings;
    use Tharn qw($Log);
    use Carp qw(confess);
    use Data::Dumper;
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Object::InsideOut qw(Perf::Stats::Counter::CM);

    # RPC counters do not have a numerator/denominator, instead
    # referencing another counter as the divisor
    my @base_counter :Field :Get('get_base_counter');

    # RPC counters do not return field labels along with array counter
    # values, instead the index of a given label must be tracked
    my @label_index  :Field :Get('get_label_index');

    sub _preinit :PreInit {
        my $self = shift();
        my $args = shift();

        $label_index[$$self] = delete($args->{spec}->{index});

        confess("Required parameter 'index' missing from counter spec:\n"
                    . Dumper($args->{spec}))
            if (!defined($label_index[$$self])
                    || q{} eq $label_index[$$self]);

        # optional fields
        $base_counter[$$self] = delete($args->{spec}->{base_counter});
    }
}

1;
