# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Base class for Stats Counter and Stats Alias
## @status   review
## @pod here

package Perf::Stats::Entry::CM;{
    use strict;
    use warnings;
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Carp qw(confess);
    use Params::Validate qw(validate SCALAR UNDEF);
    use Object::InsideOut qw( Perf::Stats::Entry );

    sub get_zero_value {
        my ($self) = @_;
        if ($self->is_of_type('percent') || $self->is_of_type('average')){
            return '0/0';
        }
        return 0;
    }

    sub delta {
        my $self = shift;
        my %args = validate (   @_,
                                {
                                    old         => { type => SCALAR },
                                    new         => { type => SCALAR },
                                    raw_counter => { type => SCALAR },
                                    exact_format=> { type => SCALAR },
                                },
                              );
        my $older_samp = $args{old};
        my $newer_samp = $args{new};
        my $raw_counter = $args{raw_counter};
        my $exact_format = $args{exact_format};
        my $name = $self->get_name;
        my $type = $self->get_all_types;
        if (!$type){
            confess("Unable to find type for entry '$name'");
        }

        if ($type =~ /average|percent/){
            my $new_count;
            my $new_total;
            my $old_count;
            my $old_total;
            if ($newer_samp =~ /(\d+)\/(\d+)/){
                $new_total = $1;
                $new_count = $2;
            }
            else{
                confess("Entry '$name' of type '$type' isn't of the format we expect: $newer_samp");
            }
            if ($older_samp =~ /(\d+)\/(\d+)/){
                $old_total = $1;
                $old_count = $2;
            }
            else{
                confess("Entry '$name' of type '$type' isn't of the format we expect: $older_samp");
            }
            my $delta_count = $new_count - $old_count;
            my $delta_total = $new_total - $old_total;
            if ( $delta_count > 0){
                if ($exact_format){
                    return "$delta_total/$delta_count";
                }
                else{
                    return sprintf("%.2f",$delta_total/$delta_count);
                }
            }
            elsif (!$delta_count && !$delta_total){
                # Neither the count nor the total changed
                # Record a zero
                return 0;
            }
            else{
                Carp::cluck "This should never happen!\n".
                            "Entry: $name\n" .
                            "Delta Count: $delta_count\n".
                            "Delta Total: $delta_total\n";     # But sometimes it does...
            }
        }
        elsif ($type =~ /string/){
            return $newer_samp;
        }
        elsif ($type =~ /rate/){
            return $newer_samp - $older_samp;
        }
        else{
            return $self->SUPER::delta(%args);
        }

    }

}
1;
