package Perf::Stats::Handler::BufStats; {
    use strict;
    use warnings;
    use Carp qw(confess);
    use Tharn qw($Log);

    BEGIN {
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Perf::Stats::Handler;
    use Perf::Stats::Preprocessor::BufStats;
    use Object::InsideOut qw(Perf::Stats::Handler);

    sub _init : Init {
        my $self = shift();

        $self->preprocessor_add(\&bufstats_key_values);
        $self->preprocessor_add(\&bufstats_count_by_type);
        $self->preprocessor_add(\&bufstats_q_counts);
        $self->preprocessor_add(\&bufstats_eviction_stats);
    }


    # None of the wafl bufstats counters appear to be incremental so
    # default them all to type raw
    sub get_default_counter_type {
        return 'raw';
    }


    sub cache_build_command {
        my $self = shift();
        return 'wafl bufstats';
    }

    sub parse_out_counters {
        my $self     = shift();
        my $show_out = shift();
        my $regex    = shift();
        my @lines    = split(/\r?\n/, $$show_out);
        my @counters;

        foreach my $line (@lines) {
            if ($line =~ /^($regex) = [[:digit:]]+$/) {
                my $c = $1;
                $c =~ s/[[:space:]]+$//;
                push(@counters, $c);
            }
        }

        return \@counters;
    }

    sub sample_build_command {
        my $self = shift();
        return 'wafl bufstats';
    }

    sub sample_build {
        my $self   = shift();
        my $out    = shift();
        my $sample = {};

        my @lines = split(/\r?\n/, $out);

        foreach my $line (@lines) {
            if ($line =~ /^([^=]+) = ([[:digit:]]+)$/) {
                my $c               = $1;
                my $v               = $2;
                my $tracked_counter = $self->tracked_get($c);
                if ($tracked_counter) {
                    $Log->trace(  "Detected tracked counter '"
                                . $tracked_counter->get_name
                                . "'");
                    $tracked_counter->set_value($v);
                    if (!$tracked_counter->is_hidden) {
                        $sample->{$c} = $v;
                    }
                }
            }
        }

        return $sample;
    }
}

1;
