# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Handler for 'statistics' command output.
##                   Specific to cluster wide counters.
## @status   review
## @pod here
package Perf::Stats::Handler::CM::Cluster;{

    use strict;
    use warnings;
    use Carp qw(confess);
    use Tharn qw($Log host);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Readonly;
    use Params::Validate qw(validate SCALAR HASHREF UNDEF);
    use Object::InsideOut;
    use NATE::BaseException qw(:try);
    use Data::Dumper;


    my @current_node            :Field :Arg(Name=>'current_node',) :Std('current_node');
    my @nodes                   :Field :Arg(Name=>'nodes') :Std('nodes');

    #return active nodes
    sub _find_available_nodes {
        my $self = shift;

        my $nodes = $nodes[$$self];
        my @available_nodes = ();
        for my $node (@$nodes){
            try {
                #ensure node can be connected to.
                my $hostrec = host($node);
                my $conn = $self->create_connection($hostrec);
                push (@available_nodes,$node);
                #free up connection
                #ZAPI ApiSets can't call get_connection or disconnect
                if($conn->can('get_connection')){
                    $conn->get_connection->disconnect;
                }
            }
            catch NATE::BaseException with {
                #ignore error not able to connect to node
            }
        }
        return \@available_nodes;
    }

    #checks nodes going down and finds a new connection if able.
    sub check_connection {
        my ($self,%args) = @_;
        my $toggled_nodes = $args{nodes};
        my $toggle = $args{toggle};

        my $nodes = $self->_find_available_nodes();
        #disconnect node going down;
        if(!$toggle  && $toggled_nodes->{$current_node[$$self]}){
            $self->disconnect;
        }

        my $success = 0;
        #Best effort to make sure always connected
        if($self->is_disconnected) {
            for my $node (@$nodes){
                #skip nodes we know are going down
                next if ($toggled_nodes->{$node}
                            && !$toggle );
                my $hostrec = host($node);
                my $conn = $self->create_connection($hostrec);
                $self->set_connection($conn);
                $current_node[$$self] = $node;
                $success = 1;
                last;
            }
        }
        else{
            $success = 1;
        }

        if(!$success){
            $Log->warn("No active nodes available for cluster handler to reconnect,
                disabling cluster counters");
            $self->set_enabled(0);
        }else {
            $self->set_enabled(1);
            #reset statistics settings if needed
            $self->_set_counter_flag if $self->can('_set_counter_flag');
        }
        return;
    }
    sub toggle {
        my ($self,%args) = @_;
        #ensure handler is connected to a filer.
        $self->check_connection(%args);
        return;
    }



}
1;