# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Handler class for wafl cpstats output
## @status   review
## @pod here

package Perf::Stats::Handler::CPStats;{

    use strict;
    use warnings;
    use Carp qw(confess);
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Perf::Stats::Handler;
    use Object::InsideOut qw(Perf::Stats::Handler);
    use Perf::Stats::Counter;
    use Perf::Stats::Preprocessor::CPStats;
    use Data::Dumper;

    sub _init :Init {
        my ($self) = @_;
        # Default preprocessor
        $self->preprocessor_add( \&blkfree_pvol );
        $self->preprocessor_add( \&blkfree_vvol );
    }

    sub cache_build_command {
        my ($self) = @_;
        return 'wafl cpstats';
    }

    sub sample_build_command {
         my ($self) = @_;
        return 'wafl cpstats';
    }

    sub sample_build {
        my ($self, $out) = @_;
        confess("Cannot get wafl cpstats: $out") if ($out !~ /Per CP stats/);
        my $sample = {};
        my $already_seen = {};
        # Parse output and ensure valid data before and after equal sign.
        while ($out =~ /\s*([^=^\n^\r]+?)\s*=\s+(\S[^\n^\r]*)\s*/g){
            my $c = $1;
            my $v = $2;
            my $tracked_counter = $self->tracked_get($c);
            if ($tracked_counter){
                $Log->trace("Detected tracked counter '" . $tracked_counter->get_name ."'");
                if (!$already_seen->{$c}){
                    $tracked_counter->set_value([]);
                    $already_seen->{$c} = 1;
                }
                my $value = $tracked_counter->get_value;
                push @$value, $v;
                if (!$tracked_counter->is_hidden){
                    $sample->{$c} = $value;
                }
            }
        }
        return $sample;
    }

    sub tracked_add {
        my ($self, $counter) = @_;
        $counter->set_value([]);
        $counter->set_type('raw');
        return $self->SUPER::tracked_add($counter);
    }

    sub parse_out_counters {
        my ($self, $show_out, $regex) = @_;
        my @counters;
        while ($$show_out =~ /($regex)\s*=\s+/g){
            my $counter = $1;
            #ignore counters with = sign in name;
            #line below will break test. Skipping for now
            #    wafl_mpcleaner_thread1: nmsgs_rcvd=        33		mpc_write_allocated =         87
            next if ($counter =~ /=/);
            # Remove trailing spaces
            $counter =~ s/\s*$//;
            # Remove leading spaces
            $counter =~ s/^\s*//;
            push @counters, $counter;
        }
        return \@counters;
    }

    sub zero {
        #NOOP
    }

}
1;
