# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Handler class for wafl stats
## @status   review
## @pod here

package Perf::Stats::Handler::WaflStats;{

    use strict;
    use warnings;
    use Carp qw(confess);
    use Tharn qw($Log);
    BEGIN{
        my @c = caller(2);
        $Log->debug("Loading " . __FILE__ . " from $c[1]:$c[2]");
    }
    use Perf::Stats::Preprocessor::WaflStats;
    use Perf::Stats::Handler;
    use Object::InsideOut qw(Perf::Stats::Handler);


    sub _init :Init {
        my ($self) = @_;
        # Default preprocessor
        $self->preprocessor_add( \&waflstats_blocks_read_by_type );
        $self->preprocessor_add( \&waflstats_blocks_read_ahead_by_type );
    }

    sub cache_build_command {
        return 'wafl stats';
    }

    sub sample_build_command {
        return 'wafl stats';
    }

    sub sample_build {
        my ($self, $out) = @_;
        confess("Cannot get wafl stats: $out") if ($out !~ /WAFL statistics over/);
        my $sample = {};
        # Parse output
        my @lines = split(/\r?\n/, $out);
        for my $line (@lines){
            if ($line =~ /^\s*([^=]*?)\s*=\s+([\d.]+)\s*$/){
                my $c = $1;
                my $v = $2;
                my $tracked_counter = $self->tracked_get($c);
                if ($tracked_counter){
                    $Log->trace("Detected tracked counter '" . $tracked_counter->get_name ."'");
                    $tracked_counter->set_value($v);
                    if (!$tracked_counter->is_hidden){
                        $sample->{$c} = $v;
                    }
                }
            }
        }
        return $sample;
    }

    sub zero {
        my ($self) = @_;
        $self->execute_start('wafl stats zero');
        my $out = $self->execute_expect;
        if ($out){
            confess("Unable to zero 'wafl stats': $out");
        }
        return;
    }

    sub parse_out_counters {
        my ($self, $show_out, $regex) = @_;
        my @counters;
        my @lines = split(/\r?\n/, $$show_out);
        for my $line (@lines){
            #ignore counters with = sign in name;
            #line below will break test otherwise. Skipping for now
            #    wafl_mpcleaner_thread1: nmsgs_rcvd=        33		mpc_write_allocated =         87
            next if (($line =~ tr/=//) > 1);
            if ($line =~ /^\s*($regex)\s*=\s*[\d.]+\s*$/){
                my $c = $1;
                # Remove trailing white space
                $c =~ s/\s+$//;
                push @counters, $c;
            }
        }
        return \@counters;
    }
}
1;
