# c-mode LUN instance appears as '<LUN UUID>'
# 7-mode LUN instance appears as '<LUN path>-<LUN serial number>'

package Perf::Stats::InstanceInfo::lun; {
    use strict;
    use warnings;
    use Carp;
    use Tharn qw($Log host);
    use NACL::STask::Node;
    use NACL::STask::Lun;
    use Perf::Stats::InstanceInfo;
    use Object::InsideOut qw(Perf::Stats::InstanceInfo);

    my @is_cmode :Field;

    sub _init :Init {
        my $self = shift();

        my $hostp = host($self->get_node());
        $is_cmode[$$self] = ($hostp->hosttype() =~ /-ng-/);
    }

    sub path {
        my $self = shift();

        if ($is_cmode[$$self]) {
            return $self->_cmode_path(@_);
        }
        else {
            return $self->_7mode_path(@_);
        }
    }

    # 7-mode LUN counters are formatted as:
    #   <LUN path>-<LUN serial number>
    # This is problematic because both the path and serial number can
    # contain a '-'.  The solution is to query all the LUNs until we
    # find the one matching this instance.
    sub _7mode_path {
        my $self     = shift();
        my $instance = shift();

        my $ci = NACL::STask::Node->new(node => $self->get_node());
        my @luns = NACL::STask::Lun->find(command_interface => $ci);

        foreach my $lun (@luns) {
            my $lun_state    = $lun->state();
            my $lun_path     = $lun_state->path();
            my $lun_instance = $lun_path . '-' . $lun_state->serial();

            return $lun_path
                if ($instance eq $lun_instance);
        }

        confess(  "LUN instance format has changed\n"
                . "\tExpecting: <LUN path>-<LUN serial #>\n"
                . "\t      Got: $instance");
    }

    sub _cmode_path {
        my $self = shift();
        my $instance = shift();

        my $connection = $self->get_connection();

        my $response =
          $connection->lun_show(uuid => $instance, fields => 'vserver,path');
        my $lines = $response->get_parsed_output();

        confess("Couldn't get LUN info from LUN UUID '$instance'")
          if (   !defined($lines)
              || 1 != scalar(@$lines)
              || !defined($lines->[0]->{path})
              || !defined($lines->[0]->{vserver}));

        return "$lines->[0]->{vserver}_$lines->[0]->{path}";
    }
}

1;
