package Perf::Stats::Preprocessor::BufStats;

use strict;
use warnings;
use Carp;

use Tharn qw($Log);

our (@ISA, @EXPORT);
BEGIN {
    require Exporter;
    @ISA = qw(Exporter);
    @EXPORT = qw(bufstats_key_values
                 bufstats_count_by_type
                 bufstats_q_counts
                 bufstats_eviction_stats);
}



#WAFLBUFS:
#
#	wafl_priv.buf_cnt:		6509888
#	wafl_priv.buf_data_count:	6509888
#	wafl_priv.buf_reserve:		11200
#	wafl_priv.mem_per_buf:		4744
#	wafl_priv.water.dirty_buf_high:	813736
#	wafl_priv.water.avail_buf_low:	813736
#	wafl_avail_bufs:		6337020
#	wafl_avail_bufs_for_cp:		6342620
#	wafl_memory_tight:		2
#	wafl_priv.total_mem:		31712543936
#	wafl_priv.map_mem:		31712543936
#
#	valid:				180556
#	dirty:				53
#	read:				0
#	write:				24
#	cpio:				0
#	recycle:			179330
#	norecycle:			51301
#	stolen:				51179
#	ec_inflight:			0
#	cont_ind_pin:			0

sub key_values {
    my $lines = shift();
    my $header;

    foreach my $line (@$lines) {
        if ($line =~ /^(WAFLBUFS|BDATAS):/) {
            $header = $1;
        }
        elsif ($line =~ /^Wafl Buf count by type:/) {
            return;
        }
        elsif (
            $line =~ m{^[[:space:]]+
                       ([^:]+):
                        [[:space:]]+
                       ([[:digit:]]+)
                        [[:space:]]*$
                      }msx
          ) {
            my $val = $2;
            (my $key = $1) =~ s/ /_/g;
            my $new_line = "$header.$key = $val";
            push(@$lines, $new_line);
        }
    }
}


sub bufstats_key_values {
    return key_values(@_);
}


#Wafl Buf count by type:
#
#Priority 1
#               level:             0             1             2             3
#                type
#         normal file:        132187             0             0             0
#       active bitmap:         17817             0             0             0
#            metafile:             6             0             0             0
#      vvol container:             0            59             0             0
#
#Priority 2
#               level:             0             1             2             3
#                type
#         normal file:           691           246             3             0
#           directory:             3             0             0             0
#         block types:            18             0             0             0
#            metafile:             9             0             0             0
#      vvol container:            13             0             0             0
#  private inode file:            48             0             0             0
#          delete log:             0             2             0             0
#
#Priority 3
#               level:             0             1             2             3
#                type
#         normal file:         10350             0             0             0
#           directory:            40             0             0             0
#         block types:             7             0             0             0
#            metafile:            17             0             0             0
#          delete log:             0             1             0             0
#
#Priority 4
#               level:             0             1             2             3
#                type
#         normal file:          4502           565             7             0
#           directory:            23             0             0             0
#         block types:             2             0             0             0
#            metafile:            17             0             0             0
#      vvol container:             4           388             9             1
#  private inode file:          1609            24             7             0
#   public inode file:             2             0             0             0
#              fsinfo:             3             0             0             0
#          purge file:             0             1             0             0
#
#Priority 5
#               level:             0             1             2             3
#                type
#         normal file:             0            16             0             0
#           directory:            97             0             0             0
#       active bitmap:            44            23             0             0
#        snap summary:             1             6             0             0
#         block types:            55             3             1             0
#            spacemap:             1             3             0             0
#            metafile:            21             4             0             0
#      vvol container:             0           191             0             0
#  private inode file:            93            11             1             0
#   public inode file:            37            10             1             0
#              fsinfo:             6             0             0             0
#          delete log:             0             1            16            16
#          purge file:             0             3             2             0
#   private inode map:             1             0             0             0
#
#Priority 6
#               level:             0             1             2             3
#                type
#         normal file:             0             6             0             0
#       active bitmap:           215            21             0             0
#        snap summary:             0             4             0             0
#         block types:            56             0             0             0
#            spacemap:             2             0             0             0
#            metafile:             2             0             0             0
#      vvol container:             0          3065             0             0
#  private inode file:             3             0             0             0
#   public inode file:            25             0             0             0
#          delete log:             0            12             0             0
#
#Priority 7
#               level:             0             1             2             3
#                type
# old/new inomap file:             1             0             0             0
#           directory:             6             0             0             0
#       active bitmap:          3858            28             0             0
#        snap summary:          3147             0             0             0
#         block types:           326             0             0             0
#            spacemap:            49             0             0             0
#            metafile:            10             0             0             0
#  private inode file:            11             0             0             0
#   public inode file:           211             0             0             0
#              fsinfo:            19             0             0             0
#          delete log:             0           104             0             0
#          purge file:             2             8             0             0
#   private inode map:             2             0             0             0

sub count_by_type {
    my $lines = shift();
    my $in_buf_count = 0;
    my $header = 'Wafl Buf count by type';
    (my $prefix = $header) =~ s/ /_/g;
    my $cur_priority;

    foreach my $line (@$lines) {
        if (!$in_buf_count && $line =~ /^$header:/) {
            $in_buf_count = 1;
        }
        elsif ($in_buf_count) {
            if ($line =~ /^Queue/) {
                return;
            }
            elsif ($line =~ /^Priority ([[:digit:]]+)/) {
                $cur_priority = $1;
            }
            elsif (   $line =~ /^[[:space:]]+level:[[:space:]].*$/
                   || $line =~ /^[[:space:]]+type[[:space:]]*$/) {
                next;
            }
            elsif (
                $line =~ m{^[[:space:]]+
                              ([^:]+):
                              [[:space:]]+
                              ([[:digit:]]+)
                              [[:space:]]+
                              ([[:digit:]]+)
                              [[:space:]]+
                              ([[:digit:]]+)
                              [[:space:]]+
                              ([[:digit:]]+)$
                         }msx
              ) {
                my @lvls = ($2, $3, $4, $5);
                (my $type = $1) =~ s/ /_/g;

                foreach my $i (0 .. $#lvls) {
                    push(@$lines,
                         "$prefix.Priority_$cur_priority.$type.level_$i = $lvls[$i]"
                         );
                }
            }
        }
    }
}


sub bufstats_count_by_type {
    return count_by_type(@_);
}


#Per Affinity Recycle Q Counts
#--------------------------------
#
#                   Pri 1      Pri 2      Pri 3      Pri 4      Pri 5      Pri 6      Pri 7
#              ---------- ---------- ---------- ---------- ---------- ---------- ----------
#Affinity  0         1153         24         18       1283          7        476         19
#Affinity  1            0          0          0          0          0          0          0
#Affinity  2            3          5          5          7         32         37         99
#Affinity  3            0         27          0          1          0          0          0
#Affinity  4            0          0          0          0          0          0          0
#Affinity  5            3          9          4          1          6          4          8
#Affinity  6           54          6          6         10         48        132        225
#Affinity  7            0          0          0          0          0          0          0
#Affinity  8            0          0          0          0          0          0          0
#Affinity  9            0          0          0          0          0          0          0
#Affinity 10            0          0          0          0          0          0          0
#Affinity 11            0         10          4          1          1          0          2
#Affinity 12            0         15         50         29         97         25        183
#Affinity 13            0          0          0          0          0          0          0
#Affinity 14            0          0          0          0          0          0          0
#Affinity 15            0          0          0          0          0          0          0
#Affinity 16        22195       1248       6893       1386          0          0          0
#Affinity 17        19782       1678       5258       1037          0          0          0
#Affinity 18        18642       1793       4697        642          0          0          0
sub q_counts {
    my $lines = shift();
    my $in_q_counts = 0;
    my $header = 'Per Affinity Recycle Q Counts';
    (my $prefix = $header) =~ s/ /_/g;

    foreach my $line (@$lines) {
        if (!$in_q_counts && $line =~ /^$header/) {
            $in_q_counts = 1;
        }
        elsif ($in_q_counts) {
            if ($line =~ /^[[:space:]]+Recycle[[:space:]]+Scavenged[[:space:]]+Scavenged/) {
                return;
            }
            elsif ($line =~ /^Affinity[[:space:]]+([[:digit:]]+)/) {
                my $affinity = $1;
                my @items = split(/[[:space:]]+/, $line);

                foreach my $i (2..$#items) {
                    my $pri = $i - 1;
                    my $new_line = "$prefix.Affinity_$affinity.Pri_$pri = $items[$i]";
                    push(@$lines, $new_line);
                }
            }
        }
    }
}


sub bufstats_q_counts {
    return q_counts(@_);
}

#free cache refill AGE eviction stats (count)
#Affinity    Queue 1    Queue 2    Queue 3    Queue 4    Queue 5    Queue 6    Queue 7
#   0              0          0          0          0          0          0          0
#   1              0          0          0          0          0          0          0
#   2              0          0          0          0          0          0          0
#   3              0          0          0          0          0          0          0
#   4              0          0          0          0          0          0          0
#   5              0          0          0          0          0          0          0
#   6              0          0          0          0          0          0          0
#   7              0          0          0          0          0          0          0
#   8              0          0          0          0          0          0          0
#   9              0          0          0          0          0          0          0
#  10              0          0          0          0          0          0          0
#  11              0          0          0          0          0          0          0
#  12              0          0          0          0          0          0          0
#  13              0          0          0          0          0          0          0
#  14              0          0          0          0          0          0          0
#  15              0          0          0          0          0          0          0
#  16              0          0          0          0          0          0          0
#  17              0          0          0          0          0          0          0
#  18              0          0          0          0          0          0          0
#
#free cache refill AGE eviction stats (age_avg)
#Affinity      Queue 1     Queue 2     Queue 3     Queue 4     Queue 5     Queue 6     Queue 7  Overall age_avg
#   0                0           0           0           0           0           0           0                0
#   1                0           0           0           0           0           0           0                0
#   2                0           0           0           0           0           0           0                0
#   3                0           0           0           0           0           0           0                0
#   4                0           0           0           0           0           0           0                0
#   5                0           0           0           0           0           0           0                0
#   6                0           0           0           0           0           0           0                0
#   7                0           0           0           0           0           0           0                0
#   8                0           0           0           0           0           0           0                0
#   9                0           0           0           0           0           0           0                0
#  10                0           0           0           0           0           0           0                0
#  11                0           0           0           0           0           0           0                0
#  12                0           0           0           0           0           0           0                0
#  13                0           0           0           0           0           0           0                0
#  14                0           0           0           0           0           0           0                0
#  15                0           0           0           0           0           0           0                0
#  16                0           0           0           0           0           0           0                0
#  17                0           0           0           0           0           0           0                0
#  18                0           0           0           0           0           0           0                0
sub eviction_stats {
    my $lines   = shift();
    my $header  = 'free cache refill AGE eviction stats';
    my $collect = 0;
    my $prefix;

    foreach my $line (@$lines) {
        if ($line =~ /^($header.*)/) {
            $collect = 1;
            ($prefix = $1) =~ s/ /_/g;
        }
        elsif ($collect) {
            if ($line =~ /^Bdata refcount histogram :/) {
                return;
            }
            elsif ($line =~ /^Affinity/) {
                next;
            }
            elsif ($line =~ /^[[:space:]]+[[:digit:]]+/) {
                (my $data = $line) =~ s/^[[:space:]]+//;
                $line =~ s/[[:space:]]$//;

                my @items = split(/[[:space:]]+/, $data);
                my $affinity = $items[0];

                foreach my $i (1 .. $#items) {
                    my $new_line;

                    if ($i == 8) {
                        $new_line =
                          "$prefix.Affinity_$affinity.Overall_age_avg = $items[$i]";
                    }
                    else {
                        $new_line =
                          "$prefix.Affinity_$affinity.Queue_$i = $items[$i]";
                    }

                    push(@$lines, $new_line);
                }
            }
        }

    } ## end foreach my $line (@$lines)
} ## end sub eviction_stats


sub bufstats_eviction_stats {
    return eviction_stats(@_);
}


1;
