# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Repository for wafl stats preprocessors
## @status   review
## @pod here

package Perf::Stats::Preprocessor::CPStats;

use strict;
use warnings;
use Carp;
use Tharn qw($Log);
our (@ISA, @EXPORT);
BEGIN {
    require Exporter;
    @ISA = qw(Exporter);
    @EXPORT = qw(blkfree_pvol
                blkfree_vvol);


}

sub blkfree {
    my $header = shift;
    my $lines = shift;
    my @new_lines;
    my $found_section = 0;
    foreach my $line (@$lines) {
        if (!$found_section && $line && $line =~ /\s*$header:\s*/ ){
            $found_section = 1;
            next;
        }
        if($found_section){
            if (
                $line =~/^\s*(.*)=\s*(\d)/
              ) {
                my $val = $2;
                my $key = $1;
                # Remove trailing white space
                $key =~ s/\s+$//;
                my $new_line = "$header.$key = $val";
                push(@new_lines, $new_line);
            }
            else{
                $found_section = 0;
            }
            undef $line;
        }
    }
    $Log->trace("Lines generated by preprocessor: \n". join("\n",@new_lines));
    push @$lines, @new_lines;
}
 # blkfree.pvol:
    # vol_hit           =         18
    # global_hit        =          4
    # scavenge          =          0
    # mode_vic_vbns     =          0
    # nextcp_num_stages =          0
    # nextcp_vbns       =          0
    # nextcp_out_of_mem =          0
    # zombie_stages     =          0

    # Should Become:
    # blkfree.pvol.vol_hit = 18
    # blkfree.pvol.global_hit = 18
    # blkfree.pvol.scavenge = 0
    # blkfree.pvol.mode_vic_vbns = 0
    # blkfree.pvol.nextcp_num_stages = 0
    # blkfree.pvol.nextcp_vbns = 0
    # blkfree.pvol.nextcp_out_of_mem =0
    # blkfree.pvol.zombie_stages =  0

sub blkfree_pvol {
    return blkfree('blkfree.pvol',@_);
}
 # blkfree.vvol:
     # vol_hit           =         18
     # global_hit        =          4
     # scavenge          =          0
     # mode_vic_vbns     =          0
     # nextcp_num_stages =          0
     # nextcp_vbns       =          0
     # nextcp_out_of_mem =          0
     # zombie_stages     =          0
sub blkfree_vvol {
    return blkfree('blkfree.vvol',@_);
}


