# -*- perl -*-
#
## @author   jroussin@netapp.com
## @summary  Repository for wafl stats preprocessors
## @status   review
## @pod here

package Perf::Stats::Preprocessor::WaflStats;

use strict;
use warnings;
use Carp;
use Tharn qw($Log);
our (@ISA, @EXPORT);
BEGIN {
    require Exporter;
    @ISA = qw(Exporter);
    @EXPORT = qw(waflstats_blocks_read_by_type
                 waflstats_blocks_read_ahead_by_type);

}

sub blocks_by_type {
    my ($section_name, $lines) = @_;
    confess "No lines!" if (!$lines || !@$lines);
    my $found_section = 0;
    my @new_lines;
    (my $section_name_one_word = $section_name) =~ s/\s/_/g;
    for my $line (@$lines){
        if (!$found_section && $line && $line =~ /\s*$section_name:\s*/ ){
            $found_section = 1;
        }
        if ($found_section){
            if ($line =~ /^\s*([^:]+):\s+(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/){
                my $type = $1;
                if ($type ne 'level'){
                    my @level_values;
                    push @level_values, $2;
                    push @level_values, $3;
                    push @level_values, $4;
                    push @level_values, $5;
                    # Turn type into single word
                    $type =~ s/\s/_/g;
                    my $i = 0;
                    for my $level_value (@level_values){
                        push @new_lines, "$section_name_one_word.$type.level_$i = $level_value";
                        $i ++;
                    }
                }
            }
            elsif($line =~/^\s*$/){
                # End of section
                last;
            }
            # Remove processed output
            undef $line;
        }
    }
    # It's possible the table will be empty and no lines will be generated
    #confess "No new lines generated!" if (!@new_lines);
    $Log->trace("Lines generated by preprocessor for section '$section_name':\n"
                   . join("\n",@new_lines)
                 );
    push @$lines, @new_lines;
}


# waflstats_blocks_read_by_type: Turns the table into list of key/value pairs
#
#blocks read by type:
#               level:             0             1             2             3
#                type
# old/new inomap file:           279            37             0             0
#         normal file:          7740         20258           196             0
#           directory:             2             4             0             0
#        special file:             0           675             3             0
#       active bitmap:          3937          1674           121             0
#        snap summary:          2125           667             0             0
#         block types:            95           123             2             0
#            spacemap:           657           128             0             0
#            metafile:          1118         29930           305           122
#      vvol container:           365           402           177             2
#           inodefile:          1750          1240           137             0
#              fsinfo:           382             0             0             0
#          delete log:             0           267           347           194
#          purge file:           132           269           132             0
#
# Should become:
#
#blocks_read_by_type.old/new_inomap_file.level_0 = 279
#blocks_read_by_type.old/new_inomap_file.level_1 = 37
#blocks_read_by_type.old/new_inomap_file.level_2 = 0
#blocks_read_by_type.old/new_inomap_file.level_3 = 0
#blocks_read_by_type.normal_file.level_0 = 7740
#blocks_read_by_type.normal_file.level_1 = 20258
#blocks_read_by_type.normal_file.level_2 = 196
#blocks_read_by_type.normal_file.level_3 = 0
# etc...
#

sub waflstats_blocks_read_by_type {
    return blocks_by_type('blocks read by type', @_);
}

# waflstats_blocks_read_ahead_by_type: same as above
#
#blocks read ahead by type:
#               level:             0             1             2             3
#                type
# old/new inomap file:             0             5             0             0
#         normal file:      13801219         28450             0             0
#           directory:           375             0             0             0
#       active bitmap:          4394           975             0             0
#        snap summary:          4440           127             0             0
#         block types:           106            14             0             0
#            spacemap:           944             0             0             0
#            metafile:           577             0             0             0
#      vvol container:         10613         34467             0             0
#           inodefile:             0             3             0             0
#          delete log:             0           376             0             0
#
sub waflstats_blocks_read_ahead_by_type {
    return blocks_by_type('blocks read ahead by type', @_);
}


