# $Id: //depot/prod/test/main/ontap/rollingrock/lib/Perf/Uptime.pm#1 $
#
# Copyright (c) 2007 Network Appliance, Inc.
# All rights reserved.
#
# lib/Perf/Uptime.pm
#
## @summary Does check for uptime of a test, mainly to check for panics.
##
## @description
##   This library does uptime checks. The uptime should be more than the test time.
## @synopsis
##   use lib/Perf/Uptime.pm
##
##   $uptime = new Uptime(node=>$node);
##   $uptime->start;
##   ... run test...
##   $fail =+ $uptime->has_booted;
#
##
## @keywords   uptime
## @status     public
## @see-also   lib/TCD.pm
## @author     heinze@netapp.com

package Uptime;


use strict;
use warnings;
use TCD;
use NOTE;
use Tharn;
use Carp;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF GLOBREF);

use filer;
use Module::Common qw(validate_mixed);
use Filer::Version qw(are_we_ontapng);
use Perf::Stats;

=remove
# Global variables. See below for their initializations.
use vars qw(
    $aggr
    $aggrs
    $ndisks
    $cc
    $ch
    @clients
    $datasize
    $filer
    $fc
    $fh
    $raid_type
    $stats
    $test
    @TESTS
    $vol
    $vol_size
    $vol_type
    $lg_aggr
    $purge
    @TC_LIST
    $clients
    $CLIENTS
    @clients
    $node
    $server
);

=cut

sub new {
    my $class = shift;
    my $node = shift;

    my $IS_NG;                                 # Is this NG (Cluster mode)
    my $server;                                # Server object

    print "uptime node = $node";

    my $counters = [
        "system:system:uptime",
    ];
    my $stats = new Perf::Stats(node=>$node, counters=>$counters);

    bless {
        node            => $node,
        IS_NG           => $IS_NG,
        stats           => $stats,
        start_uptime    => 0,
        start_time      => 0,
    }, $class;
}


# Initialize test. Automatically called before tests are executed.

sub start {
    my $self = shift;

    my $start = $self->{stats}->checkpoint;
    my $delta = $self->{stats}->delta;
    $self->{start_uptime} = $delta->{uptime};
    # Wait 10 seconds to compensate for clock speeds different between filer and client
    Tharn::snooze(10);
    $self->{start_time} = time;
}


sub has_booted {
    my $self = shift;

    $self->{stats}->checkpoint();
    my $delta = $self->{stats}->delta();
    my $uptime = $delta->{'system:system:uptime'};
    my $run_time = time - $self->{start_time};
    my $time_diff = $uptime - $self->{start_uptime};
    if ($time_diff < $run_time) {
        logcomment("FAIL: Uptime ($uptime) is less than run time ($run_time), check for panic!");
        return 1;
    }
    return 0;
}



1;

__END__

## @pod here

=head1 NAME

    Uptime.pm - convenience functions for deteriming uptime

=head1 SYNOPSIS

    use Perf::Uptime;
    use Tharn;

    my $uptime = Uptime->new($node);
    $uptime->start;
    ... do test ...
    $fail = $uptime->has_booted;


=head1 DESCRIPTION

    This module provides checks for system uptime.

=head1 FUNCTIONS

=head2 new

    Creates Uptime object.


=head2 start

    Sets initial uptime and run time.


=head2 has_booted

    Returns 1 if the uptime is less than run time.


