# -*- perl -*-
# $Id: //depot/prod/test/main/ontap/boilermaker/lib/Perf/UtilsTemp.pm#1 $
#
## @author   jroussin@netapp.com
## @summary  Subroutines that ought to be moved out of this Ironcity dependent module
## @status   private
##
## @pod here

package Perf::UtilsTemp;

use strict;
use warnings;
use Carp qw(confess);
use Tharn;
use filer;
use Storage::DiskUtils qw(get_disk_hash);
use Aggregate qw(:df);
use Utils::Units qw(:constants);

sub wait_for_scan {
    my $fc = shift;
    my %args = @_;
    my $type = $args{type};
    my $target = $args{target};
    my $status = $args{status};
    my $no_status = $args{no_status};
    my $interval = $args{interval};
    my $timeout = $args{timeout};

    if (!$interval){
        $interval = 5;
    }
    if ($type !~ /vol|aggr/){
        confess "Argument 'type' must be vol|aggr";
    }
    my $cmd = "wafl scan status ";
    if ($type =~ /aggr/){
        $cmd .= "-A ";
    }
    if ($target){
        $cmd .= $target;
    }

    my $start_time = time;
    while (($timeout && time - $start_time < $timeout) || (!$timeout)){
        my $out = filer_cli_diag_say($fc, $cmd);
        if ($status && $out =~ /$status/){
            return 1;
        }
        if ($no_status && $out !~ /$no_status/){
            return 1;
        }
        sleep $interval;
    }
    return 0;
}

sub ndmp_copy {
    my $fc = shift;
    my %args = @_;
    my $src_path = $args{src};
    my $dst_path = $args{dst};
    my $timeout = $args{timeout};

    if ($src_path !~ /^\/vol/ || $dst_path !~ /^\/vol/){
        confess "You must specify a source path and destination path";
    }

    if (!$timeout){
        $timeout = 3600;
    }

    my $host = host($fc->hostp()->hostname());
    if ($host->hosttype() =~ /-ng-/){
        logcomment("Using C-mode ndmpd");
        my (undef, undef, $out_on) = say($fc, "ndmpd on -node *");
        my (undef, undef, $out_status) = say($fc, "ndmpd show");
        if ($out_status !~ m/true\s+true/s){
            confess "Unable to start ndmpd: $out_on";
        }
    }
    else {
        logcomment("Using 7-mode ndmpd");
        my (undef, undef, $out_on) = say($fc, "ndmpd on");
        # 7-mode check
        my (undef, undef, $out_status) = say($fc, "ndmpd status");
    }
    my $out;
    eval{
        (undef, undef, $out) = say($fc, "ndmpcopy $src_path $dst_path", -maxidle,$timeout,-timeout,$timeout);
    };
    if ($@){
        confess "Unable to perform ndmpcopy: $@";
    }
    if ($out !~ /Transfer successful/){
        confess "Unable to perform ndmpcopy: $out";
    }
}

sub printflag($$) {
    my $fc = shift;
    my $flag = shift;

    my (undef, undef, $out) = say($fc,"priv set -q diag; printflag $flag");
    if ($out =~ /$flag = ([\S]+)/){
        return $1;
    }
    else{
        confess "Unable to get flag '$flag'";
    }
}

sub setflag($$$) {
    my $fc = shift;
    my $flag = shift;
    my $value = shift;

    my (undef,undef, $set_out) = say($fc,"priv set -q diag; setflag $flag $value");
    my $curr_value;
    eval{ $curr_value = printflag($fc, $flag); };
    if ($curr_value ne $value){
        confess "Unable to set flag '$flag' to '$value': $set_out";
    }
}

sub configure_aggr_data_disk_activity {
    my $stats = shift;
    my $aggr = shift; # NOTE::Aggregate
    if (!$stats || ! $stats->isa("Perf::Stats")){
        confess "You need to specify a Perf::Stat";
    }
    if (!$aggr || ! $aggr->isa("NOTE::Aggregate")){
        confess "You need to specify a NOTE::Aggregate";
    }
    my $fc = $aggr->Filer->conn;

    # Detect data disks of aggregate and their UIDs
    my @aggr_data_disks_uids;
    my @aggr_disks = $aggr->disks;
    my $no_uid = 0;
    for my $d (@aggr_disks){
        if ($d->{type} eq 'data'){
            my $disk_name = $d->{name};
            my %disk_info = get_disk_hash($fc, $disk_name);
            if ($disk_info{UID}){
                push @aggr_data_disks_uids, $disk_info{UID};
            }
            elsif($disk_info{WWN}){
                $no_uid = 1;
                my $s = $disk_info{WWN};
                # Remove the semi-colons since they don't match with the format used in the counter names
                $s =~ s/\://g;
                # Add a semi-colon every 8 character
                $s =~ s/(\S{8})(\S)/$1:$2/g;
                # Finish formatting
                push @aggr_data_disks_uids, uc($s);

            }
        }
    }
    my $aggr_data_disks_count = scalar @aggr_data_disks_uids;
    # Build regex for user_reads and user_writes
    my $first_uid = shift @aggr_data_disks_uids;
    my $disk_regex = "disk:($first_uid";
    for my $uid (@aggr_data_disks_uids){
        $disk_regex .= "|$uid";
    }
    #$disk_regex .= "):user_(read|write)_(blocks|latency)";
    $disk_regex .= ")";
    if ($no_uid){
        $disk_regex .= "(:00000000){8}";
    }
    $disk_regex .= ":user_(read|write)_blocks";

    # Counters to monitor
    my $counters = [
        $disk_regex,
    ];
    $stats->add($counters);
    $stats->alias(name=>"user_read_blocks", counters=>['disk:.*:user_read_blocks'], hide_counters=>1,
                    action=>"sum");
    $stats->alias(name=>"user_write_blocks", counters=>['disk:.*:user_write_blocks'], hide_counters=>1,
                    action=>"sum");
    #$stats->alias(name=>"user_write_latency", counters=>['disk:.*:user_write_latency'], hide_counters=>1,
    #                action=>"avg");
    #$stats->alias(name=>"user_read_latency", counters=>['disk:.*:user_read_latency'], hide_counters=>1,
    #                action=>"avg");
    return 1;
}

sub wait_for_reclamation {
    my $fc = shift;
    my $aggr = shift;
    my $poll_interval = shift;
    if (!defined($poll_interval)){
        $poll_interval = 5;
    }

    my $disk_usage = aggr_disk_free($fc,
                                    path => $aggr,
			                        space_type => DF_SPACE,
			                        report_type => UNIT_KB);
    my $last_disk_usage;
    while (1){
        sleep $poll_interval;
        $last_disk_usage = $disk_usage;
        $disk_usage = aggr_disk_free($fc,
                                    path => $aggr,
			                        space_type => DF_SPACE,
			                        report_type => UNIT_KB);
        # Make sure capacity is going down
        if ($disk_usage->{used} eq $last_disk_usage->{used}){
            last;
        }
    }
}

1;
