# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Perf/Vserver_Setup_IPM.pm#1 $
# Copyright (c) 2007 Network Appliance, Inc.
# All rights reserved.
# lib/Perf/Vserver_Setup_IPM.pm
#
## @summary  Perl library for Counter Manager Object
## @author   dl-perf-qa@netapp.com
## @status   private

package Perf::Vserver_Setup_IPM;

use strict;
use warnings;
use Tharn;
use TCD;
use Carp;
use Server;
use List::MoreUtils qw(none);
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Network::SCON_DB::IPReservation;
use NACL::C::NetworkInterface;
use NACL::C::Node;
use NACL::C::Vserver;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use Data::Dumper;

use vars qw(@ISA @EXPORT);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(get_ip_address_IP_Manager
             return_ip_address_IP_Manager
             create_vserver
             delete_vserver);


=head1 get_ip_address_IP_Manager
    Description : Function to get IPs from IP Manager :
    Arguments   : Number of vservers
    Returns     : A hash reference which contains the following:
        ipaddresses : Data and Management IPs to configure data
        and mgmt interfaces for per vserver per node
        gateway : Gateway to configure Vserver Interfaces
        
        netmask : Netmask to configure Vserver Interfaces
        ports : Home ports per node to configure Vserver Interfaces
        job_id_mgmt : Required by IP Manager Nanny process during cleanup
        job_id_data : Required by IP Manager Nanny process during cleanup
        ip_mgr_data : Required by IP Manager Nanny process during cleanup
        ip_mgr_mgmt : Required by IP Manager Nanny process during cleanup

=cut

sub get_ip_address_IP_Manager {
	my %args = validate( @_,
                            {   num_vservers  => {type => SCALAR, optional => 1,
													default => 1},
							},
                          );

	my @node_objs = NACL::C::Node->find();

	my $ip_mgr_version =  4;
	my ($ip_mgr_obj_data, $error_data, $num_vservers, $home_ports, $ip_addresses,
		$ip_mgr_obj_mgmt, $error_mgmt, $ipmgr_obj);
	
	$num_vservers = $args{num_vservers};

	confess 'Number of Vservers cannot exceed number of nodes in the cluster'
		if(scalar(@node_objs) < $args{num_vservers});
	
	
	my $num_ips = $num_vservers;
	my $role_data = 'data';
	my $role_mgmt = 'management';
	
	my $ntw_id = '192.168.151.0';
	
	my $query = 'any';
	my $pass_phrase = 'rtp-qa-perf';

	my $job_id = $ENV{'JOBID'};
	my $job_id_data = $job_id .'PERF_QA_DATA';
	my $job_id_mgmt = $job_id .'PERF_QA_MGMT';

	
	$Log->comment('Create an IP Manager Object for role data');

	($ip_mgr_obj_data, $error_data) = 
			Network::SCON_DB::IPReservation->ip_addresses('network_id'	=> $ntw_id,
														'role'			=> $role_data,
														'count'			=> $num_ips,
														'job_id'		=> $job_id_data,
														'query'			=> $query,
														'version'		=> $ip_mgr_version,
														'passphrase'	=> $pass_phrase);
		unless ($ip_mgr_obj_data) {
			foreach my $message (@{$ip_mgr_obj_data->{'IP'}->{'message'}}) {
				$Log->comment("Message is : $message");
			}
			$Log->comment("Last SQL: $ip_mgr_obj_data->{'last_sql'}");
			$Log->error("Error MGMT IP : $error_data");
		}
	
	$Log->comment('Create an IP Manager Object for role management');

	($ip_mgr_obj_mgmt, $error_mgmt) = 
			Network::SCON_DB::IPReservation->ip_addresses('network_id'	=> $ntw_id,
														'role'			=> $role_mgmt,
														'count'			=> $num_ips,
														'job_id'		=> $job_id_data,
														'query'			=> $query,
														'version'		=> $ip_mgr_version,
														'passphrase'	=> $pass_phrase);
		unless ($ip_mgr_obj_mgmt) {
			foreach my $message (@{$ip_mgr_obj_mgmt->{'IP'}->{'message'}}) {
				$Log->comment("Message is : $message");
			}
			$Log->comment("Last SQL: $ip_mgr_obj_mgmt->{'last_sql'}");
			$Log->error("Error MGMT IP : $error_mgmt");
		}



	my @ip_list_data = $ip_mgr_obj_data->ip_address_list;
	my @ip_list_mgmt = $ip_mgr_obj_mgmt->ip_address_list;

	$Log->comment("IP Data List:". Dumper(\@ip_list_data));
	$Log->comment("IP Mgmt List:". Dumper(\@ip_list_mgmt));

	my $netmask = $ip_mgr_obj_data->ip_address_netmask;
	my $gateway = $ip_mgr_obj_data->ip_address_gateway;
	
	foreach my $node_obj (@node_objs) {
		
		my $port = _get_home_port(node_obj => $node_obj);
		$home_ports->{$node_obj->node} = $port;
		$ip_addresses->{$node_obj->node}->{mgmt} = pop(@ip_list_mgmt);
		$ip_addresses->{$node_obj->node}->{data} = pop(@ip_list_data);
	}
	
	$ipmgr_obj->{ip_addresses} = $ip_addresses;
	$ipmgr_obj->{gateway} = $gateway;
	$ipmgr_obj->{netmask} = $netmask;
	$ipmgr_obj->{ports} = $home_ports;
	$ipmgr_obj->{job_id_mgmt} = $job_id_mgmt;
	$ipmgr_obj->{job_id_data} = $job_id_data;
	$ipmgr_obj->{ip_mgr_data} = $ip_mgr_obj_data;
	$ipmgr_obj->{ip_mgr_mgmt} = $ip_mgr_obj_mgmt;
	
	return $ipmgr_obj;
}

# Private function
# Function to retrieve the home ports:
# Returns an port name of the specifed node

sub _get_home_port {
	my %args = validate( @_,
                            {   node_obj => {type => OBJECT, required => 1},
								
                            },
                          );
	my $node_obj = $args{node_obj};
	my $network_interface = NACL::C::NetworkInterface->find(command_interface => $node_obj,
														filter => {role => 'data'});

	my $port = $network_interface->get_one_state_attribute('home-port');
	return $port;
}



# Function to return IPManager IP addresses back to the pool
# Returns 0 if pass else 1 failed

sub return_ip_address_IP_Manager {
	my %args = validate( @_,
                            {   ipmgr_obj  => {type => HASHREF, required => 1},
							},
                          );

	my $fail = 0;
	my $ipmgr_obj = $args{ipmgr_obj};
	my $ip_mgr_obj_data = $ipmgr_obj->{ip_mgr_data};
	my $ip_mgr_obj_mgmt = $ipmgr_obj->{ip_mgr_mgmt};
	my $job_id_data = $ipmgr_obj->{job_id_data};
	my $job_id_mgmt = $ipmgr_obj->{job_id_mgmt};

	my ($cleanup_data, $cleanup_mgmt , $error_data, $error_mgmt);

	if(defined $ip_mgr_obj_data->{'IP'}->{'dbh'}) {
		($cleanup_data, $error_data) = $ip_mgr_obj_data->ip_address_return('job_id' => $job_id_data);
		if($cleanup_data) {
			$Log->comment("IP Address has been returned successfully : $cleanup_data ");
		} else {
			$Log->warn("Error while returning IP Address: $error_data");
			$fail++;
		}
	}
	
	if(defined $ip_mgr_obj_mgmt->{'IP'}->{'dbh'}) {
		($cleanup_mgmt, $error_mgmt) = $ip_mgr_obj_mgmt->ip_address_return('job_id' => $job_id_mgmt);
		if($cleanup_mgmt) {
			$Log->comment("IP Address has been returned successfully : $cleanup_mgmt");
		} else {
			$Log->warn("Error while returning IP Address: $error_mgmt");
			$fail++
		}
	}

	return $fail;
}


=head1 create_vserver
    Description :  Function to create a new vserver:

    Mandatory parameters : ipmgr_obj : IP Manager Object
                           num_vservers : Number of Vservers
    Returns : Return an arrayref of vserver objects
            vserver_setup_obj   : Required during cleanup 
            vserver_name		: Vserver_name
            home node			: Node on which the vserver virtually resides
            mgmt_ipaddress	    : Vserver mgmt interface ipaddress
            data_ipaddress	    : Vserver data interface ipaddress
            username			: Vserver login username
            password			: Vserver authentication password
            cli_obj			    : CLI Object to send CLI commands in vserver context
            zapi_obj			: ZAPI Object to send ZAPI commands in Vserver context
=cut

sub create_vserver {
	my %args = validate( @_,
                            {   
								ipmgr_obj		=> {type => HASHREF, required => 1},
								num_vservers	=> {type => SCALAR, required => 1},
								nfs_workload	=> {type => SCALAR, optional => 1,
													default => 0},
								cifs_workload	=> {type => SCALAR, optional => 1,
													default => 0},
								iscsi_workload	=> {type => SCALAR, optional => 1,
													default => 0 },
								aggr_name		=> {type => SCALAR, optional => 1,
													default => 'task_root_aggr'},
								root_vol		=> {type => SCALAR, optional => 1,
													default => 'task_root_vol'},
								vserver_name	=> {type => SCALAR, optional => 1,
													default => 'task_vserver'},
								ns_switch		=> {type => SCALAR, optional => 1,
													default => 'file'},
								nm_switch		=> {type => SCALAR, optional => 1,
													default => 'file'},
								lif_mgmt_name	=> {type => SCALAR, optional => 1,
													default => 'lif_mgmt'},
								lif_data_name	=> {type => SCALAR, optional => 1,
													default => 'lif_data'},
								username		=> {type => SCALAR, optional => 1,
													default => 'vsadmin'},
								password		=> {type => SCALAR, optional => 1,
													default => 'netapp123'},
							},
                          );

	my ($num_vservers, $ipmgr_obj, $ip_addresses, $ports, $netmask, 
		$gateway, $aggr_name, $root_vol, $vserver_name, @ns_switch, @nm_switch, @node_objs, 
		$lif_mgmt_name, $lif_data_name, $username, $password,$policy_name, @vserver_config_objs,
		$test_rule, $direction, $flag);
	
	@node_objs = NACL::C::Node->find();
	local $Data::Dumper::Maxdepth = 4;

	$ipmgr_obj		= $args{ipmgr_obj};
	$num_vservers	= $args{num_vservers};
	$ip_addresses	= $ipmgr_obj->{ip_addresses};
	$netmask		= $ipmgr_obj->{netmask};
	$gateway		= $ipmgr_obj->{gateway};
	@ns_switch		= ($args{ns_switch});
	@nm_switch		= ($args{nm_switch});
	$username		= $args{username};
	$password		= $args{password};
	$ports			= $ipmgr_obj->{ports};
	$test_rule      = 777;
	$direction		= 'unix-win';

	for (my $i = 0; $i < $args{num_vservers}; $i++) {
		my $vserver_config;
		my $node_obj = $node_objs[$i];

		my $node_cli_obj = $node_obj->apiset(interface => 'CLI');
		my $node_name = $node_obj->node;
		
		$vserver_name	= $args{vserver_name} . '_'. $node_name;
		$aggr_name		= $args{aggr_name} . '_'. $node_name;
		$root_vol		= $args{root_vol} . '_'. $node_name;
		$lif_mgmt_name	= $args{lif_mgmt_name} . '_'. $node_name;
		$lif_data_name	= $args{lif_data_name} . '_'. $node_name;
		$policy_name	= 'default';


		my $vserver_params = {	'vserver'	=> $vserver_name,
							'aggregate' => $aggr_name,
							'rootvolume'	=> $root_vol,
							'ns-switch'		=> \@ns_switch,
							'nm-switch'		=> \@nm_switch,
							'rootvolume-security-style' => 'mixed',
							'language'	=> 'C',
							'snapshot-policy'	=> 'default',
							'antivirus-on-access-policy'	=> 'default',
							'nacltask_if_exists'	=> 'purge',
						};

		my $aggr_params = {'aggregate' => $aggr_name,
						'diskcount' => 3,
						'nodes' => [$node_name],
						'raidtype' => 'raid4',
						'volume-style'	=> 'flex',
						'nacltask_if_exists' => 'reuse',
						'nacltask_wait' => 1};

		my $network_interface_params =[ {'lif'	=> $lif_mgmt_name,
										'vserver'	=> $vserver_name,
										'role'		=> 'data',
										'home-node' => $node_name,
										'home-port' => $ports->{$node_name},
										'address'	=> $ip_addresses->{$node_name}->{mgmt} ,
										'netmask'	=> $netmask,
										'data-protocol' => ['none'],
										},
									
										{'lif'	=> $lif_data_name,
										'vserver'	=> $vserver_name,
										'role'		=> 'data',
										'home-node' => $node_name,
										'home-port' => $ports->{$node_name},
										'address'	=> $ip_addresses->{$node_name}->{data} ,
										'netmask'	=> $netmask,
										'data-protocol' => ['nfs'],
										}];
		
		my $nfs_params		= {vserver => $vserver_name,
								if_exists => 'purge',
								tcp => 'enabled',
								udp => 'enabled',
								access => 'true',
								if_exists => 'purge'};

		my $export_policy_params = {vserver => $vserver_name,
									policyname => $policy_name};

		my $export_policy_rule_params = {ruleindex   => $test_rule,
										vserver     => $vserver_name,
										policyname  => $policy_name,
										clientmatch => '0.0.0.0/0',
										superuser	=> ["any"],
										rorule      => ["any"],
										protocol    => ["any"],
										rwrule      => ["any"],
										anon		=> 0,
										'allow-suid' => 'true',
										'allow-dev'	 => 'true'};
		
		my $name_mapping_params		= { vserver     => $vserver_name,
										direction   => $direction,
										position    => 101,
										pattern     => 'SETUPTEST',
										replacement => 'test_new'
									};

		my $security_login_params = {vserver => $vserver_name,
								username => $username,
								newpwd => $password,
								newpwd2 => $password};

		my $routing_group_params = {vserver => $vserver_name,
									destination => '0.0.0.0/0',
									metric => 20};

		$Log->comment("Create the new vserver with the above parameters on node : $node_name");
		my $vserver_setup_obj = NACL::MTask::VserverSetup->setup(command_interface => $node_obj,
																vserver_params => $vserver_params,
																aggregate_params => $aggr_params,
																network_interface_params => $network_interface_params,
																nfs_params	=> $nfs_params,
																export_policy_params => $export_policy_params,
																export_policy_rule_params => $export_policy_rule_params,
																security_login_params => $security_login_params);
	
	
		# Create unix user for mapping
		# vserver services unix-user create -vserver trend_gx -user root -id 0 -primary-gid 0
		$node_cli_obj->vserver_services_unix_user_modify(vserver => $vserver_name,
														user	=> 'root',
														id		=> 0,
														'primary-gid'	=> 0);

		$Log->comment('Modify the firewall policy for the new data lif');
		$node_cli_obj->network_interface_modify('vserver' => $vserver_name,
												'lif' => $lif_mgmt_name,
												'firewall-policy' => 'mgmt');

		# NOTE: New routes are not created due to burt625336
		#$Log->comment('Create a route');
		#$ip_addresses->{$node_name}->{mgmt} =~ /([\d]+[.][\d]+[.][\d]+).*/;
		#$Log->comment("Routing group : $1");
		#my $routing_group = 'd' .$1 . '.0/24';

		#$node_cli_obj->network_routing_groups_route_create(vserver => $vserver_name,
		#													'routing-group' => $routing_group,
		#													'gateway' => $gateway,
		#													destination => '0.0.0.0/0',
		#													);
		my $resp_obj = $node_cli_obj->security_login_show(vserver => $vserver_name);
		$Log->comment('Security Login Show:'. Dumper($resp_obj->get_parsed_output()));
		
		# Create Vserver Object with CLI/ZAPI objects for each node

		my $vserver_obj = NACL::C::Vserver->find(command_interface => $node_obj,
												filter => {vserver => $vserver_name});

		$vserver_obj->apiset_must(	username => $username,
									password => $password);
			
		my $cli_obj = $vserver_obj->apiset(interface => 'CLI',
											ipaddress => $ip_addresses->{$node_obj->node}->{mgmt});
		my $zapi_obj = $vserver_obj->apiset(interface => 'ZAPI',
											ipaddress => $ip_addresses->{$node_obj->node}->{mgmt});
			
		$vserver_config->{vserver_setup_obj} = $vserver_setup_obj;
		$vserver_config->{vserver_name} = $vserver_name;
		$vserver_config->{home_node} = $node_name;
		$vserver_config->{mgmt_ipaddress} = $ip_addresses->{$node_name}->{mgmt};
		$vserver_config->{data_ipaddress} = $ip_addresses->{$node_name}->{data};
		$vserver_config->{username} = $username;
		$vserver_config->{password} =  $password;
		$vserver_config->{vserver_obj} = $vserver_obj;
		$vserver_config->{cli_obj} = $cli_obj;
		$vserver_config->{zapi_obj} = $zapi_obj;

		push(@vserver_config_objs, $vserver_config);
	}
	
	return \@vserver_config_objs;
}

=head1 delete_vserver
    Description : Function to delete Vserver and configure data_lif for VS0
    Mandatory Argument : Vserver Setuo Object
    Returns : None
=cut

sub delete_vserver {
	my %args = validate( @_,
                            {   vserver_setup_obj => {type => OBJECT, required => 1},
                            },
                          );

	my $vserver_setup_obj = $args{vserver_setup_obj};

	$Log->comment('Delete the newly created Vserver....');

	$vserver_setup_obj->vserver_obj()->purge()
		if(defined $vserver_setup_obj->vserver_obj());

	$Log->comment('Delete aggregate');
	
	$vserver_setup_obj->aggr_obj()->purge()
		if(defined $vserver_setup_obj->aggr_obj());
	
}


1;
