/**
 * @mainpage Counter Manager Clustered Archiver Parsing Library (CMAPL)
 * @author Michael Cao
 * @date February 22, 2012
 *
 * The Counter Manager Archiver Parsing Library (CMAPL) is a C library
 * used to manipulate the CM Archiver Binary File Format.
 *
 * For more information, go to: http://wikid.netapp.com/w/Performance_Counters/SierraNevada/Specs/CM_Archiver_Parsing_Library
 *
 * @section ds Data Structures Files
 * @li cmapl_common_defs.h	Contains all the external data structures, unions, and enums
 * @li cmapl_ext_defs.h		Contains all the external data handles of the CMAPL
 *
 * @section rsl Read-side Library Files
 * @li cmapl_read.h		Contains functions to parse, read, and query a single file
 * @li cmapl_directory.h		Contains functions to parse a directory for all archive files
 *
 * @section wsl Write-side Library Files
 * @li cmapl_write.h		Contains functions to create and write to an archive file
 *
 * @section howto How to use the CMAPL
 * @li \ref howtopage		Contains detailed instructions about how the CMAPL is used
 */

/**
 * @file cmapl_ext.h
 *
 * This is the main header file for the Counter Manager Archiver Parsing Library API.
 * It contains all the external header files needed to use the CMAPL API.
 */

#ifndef _CMAPL_H_
#define _CMAPL_H_

#include "cmapl_ext_defs.h"
#include "cmapl_common_defs.h"
#include "cmapl_write.h"
#include "cmapl_read.h"
#include "cmapl_directory.h"
#include "cmapl_corefile.h"

#endif /* _CMAPL_H_ */
