/**
 * @file cmapl_ext_defs.h
 *
 * The CMAPL external definitions file. It contains various data
 * structures, enums, and unions used by the CMAPL API.
 *
 */

#ifndef _CMAPL_EXT_DEFS_H_
#define _CMAPL_EXT_DEFS_H_

// TODO: include correct headers in all header files

/****************
 * Data Handles *
 ****************/

/**
 * @name cmapl_counter_handle_t
 *
 * This opaque handle contains all the data related to a single counter
 * in the CMAPL. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_counter_handle_t;

/**
 * @name cmapl_file_handle_t
 *
 * This opaque handle contains all the data related to a specific data
 * file. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_file_handle_t;

/**
 * @name cmapl_header_handle_t
 *
 * This opaque handle contains all the data related to the XML header of
 * a data file. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_header_handle_t;

/**
 * @name cmapl_object_handle_t
 *
 * This opaque handle contains all the data related to a single object
 * in the CMAPL. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_object_handle_t;

/**
 * @name cmapl_record_handle_t
 *
 * This opaque handle contains all the data related to a single record
 * in the CMAPL. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_record_handle_t;

/**
 * @name cmapl_counter_data_handle_t
 *
 * This opaque handle contains all the data related to the counter data
 * of a single counter. The CMAPL API contains functions that allow the
 * user to retrieve data from this handle.
 */
typedef void cmapl_counter_data_handle_t;

/**
 * @name cmapl_instance_handle_t
 *
 * This opaque handle contains all the data related to a single instance
 * of some object. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_instance_handle_t;

/**
 * @name cmapl_timestamp_handle_t
 *
 * This opaque handle contains all the data related to a single timestamp
 * of a query handle. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_timestamp_handle_t;

/**
 * @name cmapl_query_handle_t
 *
 * This opaque handle contains all the data related to some specific query
 * made on a data file. The CMAPL API contains functions that allow the user
 * to retrieve data from this handle.
 */
typedef void cmapl_query_handle_t;

/**
 * @name cmapl_directory_handle_t
 *
 * This opaque handle contains all the data related to a directory opened
 * by the user. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_directory_handle_t;


/**
 * @name cmapl_corefile_handle_t
 *
 * This opaque handle contains all the data related to a corefile opened
 * by the user. The CMAPL API contains functions that allow the user to
 * retrieve data from this handle.
 */
typedef void cmapl_corefile_handle_t;

#endif /* _CMAPL_EXT_DEFS_H_ */
