/*!
\page howtopage Developer How To

<div class="toc"><h1>Table of Contents</h1>
<ol>
   <li class="level1">\ref howtouse
      <ol>
         <li class="level2">\ref uselinux
            <ol>
               <li class="level3">\ref uselinuxinc</li>
               <li class="level3">\ref uselinuxinperl</li>
            </ol>
         </li>
         <li class="level2">\ref examples
            <ol>
               <li class="level3">\ref openandclosefile</li>
               <li class="level3">\ref openandclosedir</li>
               <li class="level3">\ref queryforcounters</li>
               <li class="level3">\ref parsequery</li>
            </ol>
         </li>
         <li class="level2">\ref samplefiles</li>
      </ol>
   </li>
</ol>
</div>

\section howtouse How to use the CMAPL

The CMAPL consists of a number of shared libraries and header files. In order to use the CMAPL, you will need to link the libraries with your program/tool and include the necessary headers.

All of the CMAPL files can be found in the nightly build at <tt>/x/eng/rlse/DOT/sn_mhost_vserverN/</tt>.

The libraries are found in different locations depending on the OS version you need. The exact locations will be described in the sections below.

The libraries include the following files:
\li libcmaplutil.so
\li libcmaplwrite.so
\li libcmaplread.so
\li libcmapldir.so


The headers can be found at <tt>/x/eng/rlse/DOT/sn_mhost_vserverN/cm/cmapl/src/api/</tt>.

The headers include the following files:
\li cmapl_common_defs.h
\li cmapl_directory.h
\li cmapl_ext_defs.h
\li cmapl_ext.h
\li cmapl_read.h
\li cmapl_write.h

\subsection uselinux Using the CMAPL in Linux

The Linux version library can be found in the nightly build at <tt>/x/eng/rlse/DOT/sn_mhost_vserverN/cm/cmapl/bedrock/internal/linux64/ulibso-l/</tt>.

\subsubsection uselinuxinc Using the CMAPL with your C program

The following is a list of steps to get the CMAPL to link and compile with your C program in Linux. Also, in order to prevent having to do the following steps every time you use the CMAPL, you should either include the following commands in your .bashrc file, or you can have them in your makefile.
<ol>
<li>Libraries: Copy all the CMAPL libraries to one of the directories that your $LIBRARY_PATH and $LD_LIBRARY_PATH environment variables point to. If you chose not to copy them over to one of those directories, then you'll need to add the directory that libraries are in to the $LIBRARY_PATH and $LD_LIBRARY_PATH variables. You can do this with the following commands:
\code
 % export LIBRARY_PATH=$LIBRARY_PATH:/x/eng/rlse/DOT/sn_mhost_vserverN/cm/cmapl/bedrock/internal/linux64/ulibso-l/
 % export LD_LIBRARY_PATH=$LD_LIBRARY_PATH:/x/eng/rlse/DOT/sn_mhost_vserverN/cm/cmapl/bedrock/internal/linux64/ulibso-l/
\endcode
Note that if you're an admin, you could add the libraries to /usr/lib/.
</li>
<li>libxml2 library: Install libxml2 if it is not already present. You'll need the libxml2 library for XML parsing. This should already be located at /usr/lib/, which should be included in $LIBRARY_PATH. If it is not, you will need to add it like the previous step.
</li>
<li>Header files: The environment variable $C_INCLUDE_PATH should point to include directories where header files are located. You'll need to copy the CMAPL headers to one of these directories, or you can add a new include directory. You can do this with the following commmand:
\code
 % export C_INCLUDE_PATH=$C_INCLUDE_PATH:<include_directory>
\endcode
Note that if you're an admin, you could use /usr/include/ as the include directory. Now under the include directory that you've chosen, create a cmapl directory using mkdir. Now copy all the CMAPL headers mentioned above to this directory. Now you'll be able to easily include the CMAPL headers into your program.
To include the CMAPL in your program(let's just call this program.c for now), you can include the following to your program:
\code
 #include <cmapl/cmapl_ext.h>
\endcode
This will give your program access to all of the CMAPL's functionality.
</li>
<li>Compilation: To compile your program and link all the libraries, you will need to run the following:
\code
 % gcc -lxml2 -lcmapldir -lcmaplread -lcmaplwrite -lcmaplutil -o program program.c -D_CMAPL_USER_SPACE_
\endcode
This will link all the CMAPL and libxml2 libraries with your program and produce an executable called "program".
</li>
</ol>

\subsubsection uselinuxinperl Using the CMAPL with your Perl program

To get the CMAPL to link and compile with a Perl script, the steps are very similar to the steps mentioned above for using the CMAPL with a C program. You will still need to update all the mentioned environment variables and copy over the .so files and header files.

The main difference is the very last step. You cannot just use gcc to compile your program with the CMAPL since it is written in Perl. In order to use the CMAPL, which is written in C, in a Perl script, you will need a way to expose the library to be used in Perl. You can achieve this by using Inline::C which lets a user embed C inside of Perl. Explanations of how Inline::C works and how it is used, you can take a look at these links:

\li http://www.perlmonks.org/?node_id=151114
\li http://search.cpan.org/~sisyphus/Inline-0.50/C/C-Cookbook.pod#Exposing_Shared_Libraries

The following is a simple example of a Perl script using Inline::C to call functions from the CMAPL. This example just opens and closes a single archive file. It assumes you have already set all the necessary environment variables and copied over the library and header files mentioned in the previous section. Note that when specifying the "-I" and "-L" Inline::C options, you should specify the directories where you've copied the header and libraries files to. "-I" should point to the directory containing the cmapl directory you created that has all the headers. "-L" should point to the directory containing all the CMAPL libraries.
\code{.pl}
#!/usr/bin/perl

use Inline C => DATA =>
           INC  => '-I/usr/include/' =>
           LIBS => '-L/usr/lib/ -lxml2 -lcmaplutil -lcmaplwrite -lcmaplread -lcmapldir';

cmapl_test();

__END__
__C__

#include <cmapl/cmapl_ext.h>

int cmapl_test( ) {
        printf("hello\n");
        cmapl_error_t err = CMAPL_SUCCESS;

        // open the specified archive file
        cmapl_file_handle_t* file;
        const char* meta_path;
        const char* data_path;
        err = cmapl_open_data_file("000084_1330118442971_298000.ccma", "r", &file, &meta_path, &data_path);
        if (err != CMAPL_SUCCESS){
                printf("Error opening file with error %d!\n", err);
                return err;
        }

        // close the archive file handle
        err = cmapl_close_data_file(file);
        if (err != CMAPL_SUCCESS){
                printf("Error closing file with error %d!\n", err);
                return err;
        }

        printf("goodbye\n");
        return 1;
}
\endcode

\subsection examples Examples

This is a list of examples showing how to use the CM Archiver Parsing Library API to achieve some common use cases. A more thorough example is located in the CMAPL dump file at ///cm/cmapl/src/api/cmapl_dump.c . This dump program dumps all the sampled data of a single archive file or all the archive files in a single directory. The following examples are mostly taken from this dump program.

\subsubsection openandclosefile Open and close an archive file
This example will concisely show how to open and close a data file using the CMAPL. The user will specify some file path to open and get back a file handle to use for reading and querying.

One thing to point out is the name of an archive file. When writing an archive file, the CMAPL will create two separate files. One meta data file with a name that ends in *.ccma.meta, and one compressed data file with a name that ends in *.ccma.gz. However, in order to reference a specific file, the user should only specify the generic name of the archive file which is *.ccma . This is shown below when trying to open the archive file 000008_1329197135031_300000.ccma.

\code
// open the specified archive file
const char* file_path = "/mroot/etc/log/stats/ccma/kernel/000008_1329197135031_300000.ccma";
cmapl_file_handle_t* file;
const char* meta_path;
const char* data_path;
cmapl_error_t err = cmapl_open_data_file(file_path, "r", &file, &meta_path, &data_path);
if (err != CMAPL_SUCCESS){
        printf("Error opening file: %s with error %d!\n", file_path, err);
        return err;
}

// close the archive file handle
err = cmapl_close_data_file(file);
if (err != CMAPL_SUCCESS){
        printf("Error closing file: %s with error %d!\n", file_path, err);
        return err;
}
\endcode

\subsubsection openandclosedir Open and close a directory
This example shows how to open a directory with a given time range and period, iterate through all the files in the directory given the parameters you provided, and close the directory handle.

The time range that you specify, defined by a start and end time in epoch in ms, is used to only find files that fall in between that range. To learn more about epoch time, such as how to get the time in different languages and how to convert the time from human readable times, you can check out this website http://www.epochconverter.com/ .

\code
// open the specified directory containing archive files
cmapl_directory_def_t dir_def;
cmapl_set_latest_version(&dir_def);
dir_def.directory_path = "/mroot/etc/log/stats/ccma/kernel/";   // path of the directory to open
dir_def.start_time = 1234567890;                                // epoch time in ms to start looking for files
dir_def.end_time = 1334567891;                                  // epoch time in ms to stop looking for files
dir_def.period = 1000;                                          // value in ms to round the start and end time to

cmapl_directory_handle_t* dir;
cmapl_error_t err = cmapl_open_directory(&dir_def, &dir);
if (err == CMAPL_FILE_NOT_FOUND){
        printf("No archive files in between the given time range in directory!\n");
        return CMAPL_SUCCESS;
} else if (err != CMAPL_SUCCESS){
        printf("Error opening directory with error %d!\n", err);
        return err;
}

// get the first file in this directory handle
cmapl_file_handle_t* file;
err = cmapl_find_and_open_oldest_data_file(dir, &file);
if (err != CMAPL_SUCCESS){
        cmapl_close_directory(dir);
        return err;
}

// iterate through all the files in this directory handle
while (file != NULL){
        // close and free the file handle
        err = cmapl_close_data_file(file);
        if (err != CMAPL_SUCCESS){
                cmapl_close_directory(dir);
                return err;
        }
        file = NULL;

        // get the next file in this directory handle
        err = cmapl_find_and_open_next_data_file(dir, &file);
        if (err != CMAPL_SUCCESS){
                cmapl_close_directory(dir);
                return err;
        }
}

// close directory handle
err = cmapl_close_directory(dir);
if (err != CMAPL_SUCCESS){
        printf("Error closing directory with error %d!\n", err);
        return err;
}
\endcode

\subsubsection queryforcounters Query for certain counters
This example shows how to query for counters given certain filtering parameters and return the result inside a query handle. This query will filter on the volume CM Object, the ''vol0'' Instance, the read_ops and write_ops CM Counters.

Starting in SN.0, instances of objects are uniquely defined by their key counters. So in order to query for a specific instance of an object, the user must specify the key counter that they'd like to filter by, in addition to the value of that key counter.

To query for a counter, a time range must again be specified, defined by a start and end time in epoch in ms. They are used to only find counter data that fall in between that range. As mentioned earlier, to learn more about epoch time, such as how to get the time in different languages and how to convert the time from human readable times, you can check out this website http://www.epochconverter.com/ .

And finally, a period must be specified to query for counters. A period defines how often a certain counter is sampled in ms. So when a certain query period is specified, only counter data sampled at that period will be returned in the query result.

\code
...
cmapl_query_def_t query_def;
cmapl_set_latest_version(&query_def);

// assign object info
const char * obj_name = "volume";                       // object to query for
query_def.object_name = obj_name;

// assign instances info
const char *key_cnt_names[] = {"instance_name"};        // key counter of instance to filter by
query_def.key_cnt_names = key_cnt_names;

cmapl_cnt_value_t val;
val.val_str = "vol0";
cmapl_cnt_value_t vals[] = {val};

cmapl_counter_data_t_v6_0 key_cnt_value;                // value of key counter to filter by
key_cnt_value.value_type = CMAPL_CNT_STRING;
key_cnt_value.num_values = 1;
key_cnt_value.values = vals;

cmapl_counter_data_t_v6_0 key_cnt_values[] = {key_cnt_value};
query_def.key_cnt_values = key_cnt_values;
query_def.num_key_cnts = 1;

// assign counter info
query_def.num_cnts = 2;
const char* cnt_names[] = {"read_ops", "write_ops"};    // query for these 2 counters
query_def.cnt_names = cnt_names;

// assign period info
query_def.period = 5*1000;                              // only get counter data sampled every 5000ms

// assign time stamp info
query_def.start_time = 1234567890;                      // epoch time in ms to start looking for counter data
query_def.end_time = 1234765890;                        // epoch time in ms to stop looking for counter data

// make query
cmapl_query_handle_t * query;
err = cmapl_query(file, query_def, &query);
if (err != CMAPL_SUCCESS){
        printf("Error making query with error %d!\n", err);
        return err;
}

// return result
return query;
...
\endcode

\subsubsection parsequery Parse query result

This thorough example shows how to retrieve and print out all the data from a query result. The general process is iterating through all the timestamps of a query, then iterating through all the instances of each timestamp, and finally iterating through all the counter data of each instance. This example goes into more detail by actually printing out the value of each handle as they are encountered.

For a reminder on the hierarchy of the query handle, please refer to the <a href="http://wikid.netapp.com/w/Performance_Counters/SierraNevada/Specs/CM_Archiver_Parsing_Library#Overview">API Overview section</a> in the Wiki.

\code
// query, of type cmapl_query_handle_t, is retrieved from the previous example

cmapl_error_t err = CMAPL_SUCCESS;

// iterate through all the timestamps in the query result that have counter data
cmapl_timestamp_handle_t* timestamp;
err = cmapl_first_timestamp(query, &timestamp);
if (err != CMAPL_SUCCESS){
        return err;
}
if (err == CMAPL_RECORD_EMPTY){
        printf("\t\t\tNo samples found in this file for this query period and object.\n\n");
        err = CMAPL_SUCCESS;
} else if (err != CMAPL_SUCCESS){
        return err;
}

while (timestamp != NULL){
        // dump timestamp
        uint64_t time = 0;
        err = cmapl_get_timestamp(timestamp, &time);
        if (err != CMAPL_SUCCESS){
                return err;
        }
        printf("\t\t\tTimestamp: %"PRIu64"\n\n", time);

        // iterate through all the instances at this timestamp that have data
        cmapl_instance_handle_t* instance;;
        err = cmapl_first_instance(timestamp, &instance);
        if (err != CMAPL_SUCCESS){
                return err;
        }
        while (instance != NULL){
                // get instance data
                cmapl_instance_data_t instance_data;
                err = cmapl_get_instance_data(instance, &instance_data);
                if (err != CMAPL_SUCCESS){
                        return err;
                }

                // dump key counters data (instance name)
                uint32_t i;
                for (i = 0; i < instance_data.num_key_cnts; i++){
                        printf("\t\t\t\tKey Counter:\n");
                        printf("\t\t\t\t\tName: %s\n", instance_data.key_cnt_names[i]);
                        printf("\t\t\t\t\tValue: ");
                        uint32_t j;
                        for (j = 0; j < instance_data.key_cnt_values[i].num_values; j++){
                                // print out key counter data value depending on the counter type
                                switch(instance_data.key_cnt_values[i].value_type){
                                        case CMAPL_CNT_4BYTES:
                                                printf("%d", instance_data.key_cnt_values[i].values[j].val_4B);
                                                break;
                                        case CMAPL_CNT_8BYTES:
                                                printf("%"PRIu64, instance_data.key_cnt_values[i].values[j].val_8B);
                                                break;
                                        case CMAPL_CNT_STRING:
                                                printf("%s", instance_data.key_cnt_values[i].values[j].val_str);
                                                break;
                                }
                                // print out multiple values if it's an array counter
                                if (j != (instance_data.key_cnt_values[i].num_values - 1)){
                                        printf(", ");
                                }
                        }
                        printf("\n");
                }
                printf("\n");

                // iterate through all the counter data in this instance
                cmapl_counter_handle_t* ret_cnt;
                cmapl_counter_data_handle_t* cnt_data;
                err = cmapl_first_counter_data(instance, &ret_cnt, &cnt_data);
                if (err != CMAPL_SUCCESS){
                        return err;
                }
                while (cnt_data != NULL){
                        // get counter info
                        cmapl_counter_def_t cnt_info;
                        err = cmapl_get_counter_info(ret_cnt, &cnt_info);
                        if (err != CMAPL_SUCCESS){
                                return err;
                        }

                        // get counter data
                        cmapl_counter_data_t cnt_data_value;
                        err = cmapl_get_counter_data_value(cnt_data, &cnt_data_value);
                        if (err != CMAPL_SUCCESS){
                                return err;
                        }

                        // dump counter data
                        printf("\t\t\t\t\tCounter:\n");
                        printf("\t\t\t\t\t\tName: %s\n", cnt_info.name);
                        printf("\t\t\t\t\t\tValue: ");
                        uint32_t j;
                        for (j = 0; j < cnt_data_value.num_values; j++){
                                // print out counter data value depending on the counter type
                                switch(cnt_data_value.value_type){
                                        case CMAPL_CNT_4BYTES:
                                                printf("%d", cnt_data_value.values[j].val_4B);
                                                break;
                                        case CMAPL_CNT_8BYTES:
                                                printf("%"PRIu64, cnt_data_value.values[j].val_8B);
                                                break;
                                        case CMAPL_CNT_STRING:
                                                printf("%s", cnt_data_value.values[j].val_str);
                                                break;
                                }
                                // print out multiple values if it's an array counter
                                if (j != (cnt_data_value.num_values - 1)){
                                        printf(", ");
                                }
                        }
                        printf("\n");

                        // get next counter data in this instance
                        err = cmapl_next_counter_data(instance, &ret_cnt, &cnt_data);
                        if (err != CMAPL_SUCCESS){
                                return err;
                        }
                }
                printf("\n");

                // get next instance at this timestamp
                err = cmapl_next_instance(timestamp, &instance);
                if (err != CMAPL_SUCCESS){
                        return err;
                }
        }

        // get the next timestamp in the query result
        err = cmapl_next_timestamp(query, &timestamp);
        if (err != CMAPL_SUCCESS){
                return err;
        }
}

return err;
\endcode

\subsection samplefiles Sample Archive files

Here is a zip of some real archive files that you can play around and test with.

<a href="http://wikid.netapp.com/uploads/c/c3/Cmapl_test_files.zip">Sample Archive Files</a>

*/
