/**
 * The CMAPL internal definitions file. It contains various data
 * structures, enums, and unions used internallly by the CMAPL API.
 *
 */

#ifndef _CMAPL_INT_DEFS_H_
#define _CMAPL_INT_DEFS_H_

#ifdef _KERNEL
#include <fio/fio_stream.h>
#include <fio/fio.h>
#include <fio/fio_dir.h>
#include <sk/sk_common.h>
#include <machine/stdarg.h>
#include <lib/kma.h>
#include <sk/sk_return_address.h>
#include <legacy/string.h>
#include <lib/sscanf.h>
#include <zlib/zlib.h>

// implementation from ///ontap/prod/common/db/db_ontap.h
/*
 * convert all the FILE* routines to use
 * fio file descriptors
 */
#define CMAPL_FILE		FIO_FILE
#define CMAPL_FSEEK		fio_fseek
#define CMAPL_FOPEN(x,y)	fio_fopen((char*)x,(char*)y)
#define CMAPL_FCLOSE		fio_fclose
#define CMAPL_FWRITE(buf,size,file)	fio_fwrite(buf,sizeof(char),size,file)
#define CMAPL_FREAD(buf,size,file)	fio_fread(buf,sizeof(char),size,file)
#define CMAPL_UNLINK		fio_unlink

#define CMAPL_DIR		dir_handle_t
#define CMAPL_DIRENT		unix_dirent_t
#define CMAPL_OPENDIR		fio_opendir
#define CMAPL_READDIR		fio_readdir
#define CMAPL_REWINDDIR		fio_rewinddir
#define CMAPL_CLOSEDIR		fio_closedir

int cmapl_vfprintf(CMAPL_FILE *fp, const char *format, va_list ap);
int cmapl_fprintf(CMAPL_FILE *fp, const char *format, ...);

#define CMAPL_FPRINTF		cmapl_fprintf

#define CMAPL_GZFILE            gzFile
#define CMAPL_GZSEEK            gzseek
#define CMAPL_GZOPEN            gzopen
#define CMAPL_GZCLOSE           gzclose
#define CMAPL_GZWRITE(buf,size,file)	gzwrite(file,buf,size)
#define CMAPL_GZREAD(buf,size,file)     gzread(file,buf,size)
#define CMAPL_GZPRINTF          gzprintf

MALLOC_DECLARE(M_CMAPL);

#define CMAPL_ZALLOC(x)		sk_kmem_zalloc(x,M_CMAPL,SKMA_SLEEP)
#define CMAPL_CALLOC(x,y)	sk_kmem_zalloc((x)*(y),M_CMAPL,SKMA_SLEEP)
#define CMAPL_REALLOC(x,y)	sk_kmem_realloc(x,y,M_CMAPL,SKMA_SLEEP)
#define CMAPL_BZERO		bzero
#define CMAPL_FREE(x)		if(x != NULL){ \
					sk_kmem_free((char*)x); \
					(x) = NULL; \
					(void)(x); \
				}

#define CMAPL_ATOI		atoi
#define CMAPL_SSCANF            sscanf
#define CMAPL_STRDUP(x)		sk_kmem_strdup(x,M_CMAPL,SKMA_NOSLEEP)
#define CMAPL_STRCMP            strcmp
#define CMAPL_STRLEN		strlen
#define CMAPL_STRNCAT		strncat
#define CMAPL_STRNCMP           strncmp
#define CMAPL_STRNCPY           strncpy
#define CMAPL_STRCHR            strchr
#define CMAPL_STRRCHR		strrchr
#define CMAPL_STRSTR		strstr
#define CMAPL_STRTOK		strtok
#define CMAPL_STRTOULL          strtoull
#define CMAPL_SNPRINTF		snprintf

#define CMAPL_MEMCPY		memcpy
#define CMAPL_MEMSET		memset

#define CMAPL_ASSERT		ASSERT
#endif /* _KERNEL */

#ifdef _CMAPL_USER_SPACE_
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <dirent.h>
#define Byte dont_conflict_Byte
#include <zlib.h>
#undef Byte

// 7z decompression headers
#include <lzma/7zCrc.h>
#include <lzma/7zFile.h>
#include <lzma/7zVersion.h>
#include <lzma/Archive/7z/7zAlloc.h>
#include <lzma/Archive/7z/7zExtract.h>
#include <lzma/Archive/7z/7zIn.h>

#define CMAPL_FILE              FILE
#define CMAPL_FOPEN		fopen
#define CMAPL_FCLOSE		fclose
#define CMAPL_FPRINTF           fprintf
#define CMAPL_FSCANF            fscanf
#define CMAPL_UNLINK            remove

#define CMAPL_GZFILE            gzFile
#define CMAPL_GZOPEN            gzopen
#define CMAPL_GZCLOSE           gzclose
#define CMAPL_GZPRINTF          gzprintf
#define CMAPL_GZSCANF           gzscanf

#ifdef _CMAPL_FIJI_
// global fiji variables
extern bool cmapl_is_read_fail;
extern bool cmapl_is_seek_fail;
extern bool cmapl_is_write_fail;

#define CMAPL_FWRITE(buf,size,file)     ((cmapl_is_write_fail) ? -1 : fwrite(buf,sizeof(char),size,file))
#define CMAPL_FREAD(buf,size,file)      ((cmapl_is_read_fail) ? -1 : fread(buf,sizeof(char),size,file))
#define CMAPL_FSEEK(file,offset,flag)	((cmapl_is_seek_fail) ? -1 : fseek(file,offset,flag))

#define CMAPL_GZWRITE(buf,size,file)    ((cmapl_is_write_fail) ? -1 : gzwrite(file,buf,size))
#define CMAPL_GZREAD(buf,size,file)     ((cmapl_is_read_fail) ? -1 : gzread(file,buf,size))
#define CMAPL_GZSEEK(file,offset,flag)	((cmapl_is_seek_fail) ? -1 : gzseek(file,offset,flag))
#else
#define CMAPL_FWRITE(buf,size,file)	fwrite(buf,sizeof(char),size,file)
#define CMAPL_FREAD(buf,size,file)	fread(buf,sizeof(char),size,file)
#define CMAPL_FSEEK			fseek

#define CMAPL_GZWRITE(buf,size,file)    gzwrite(file,buf,size)
#define CMAPL_GZREAD(buf,size,file)     gzread(file,buf,size)
#define CMAPL_GZSEEK                    gzseek
#endif /* _CMAPL_FIJI_ */

#define CMAPL_DIR               DIR
#define CMAPL_DIRENT            struct dirent
#define CMAPL_OPENDIR           opendir
#define CMAPL_READDIR           readdir
#define CMAPL_REWINDDIR         rewinddir
#define CMAPL_CLOSEDIR          closedir

#define CMAPL_ZALLOC(x)		calloc(1,x)
#define CMAPL_CALLOC		calloc
#define CMAPL_REALLOC		realloc
#define CMAPL_BZERO             bzero
#define CMAPL_FREE(x)		if((x) != NULL){ \
					free((char*)(x)); \
					(x) = NULL; \
					(void)(x); \
				}

#define CMAPL_ATOI		atoi
#define CMAPL_SSCANF		sscanf
#define CMAPL_STRDUP            strdup
#define CMAPL_STRCMP		strcmp
#define CMAPL_STRLEN		strlen
#define CMAPL_STRSTR		strstr
// TODO: switch to strn* functions
#define CMAPL_STRNCAT		strncat
#define CMAPL_STRNCMP		strncmp
#define CMAPL_STRNCPY		strncpy
#define CMAPL_STRCHR            strchr
#define CMAPL_STRRCHR		strrchr
#define CMAPL_STRTOK		strtok
#define CMAPL_STRTOULL          strtoull
#define CMAPL_SNPRINTF		snprintf

#define CMAPL_MEMCPY		memcpy
#define CMAPL_MEMSET		memset

#define CMAPL_ASSERT		assert
#endif /* _CMAPL_USER_SPACE_ */

// global internal definitions
#define CMAPL_INITIAL_NUM_FILES		672	// number of files collected for a week at rotation
						// length of 15min; TODO: will need to change if
						// rotation changes to 5min
#define CMAPL_DEFAULT_NUM_OBJECTS	200	// default number of objects in the preset; should
						// match CM_MAX_OBJECTS in cm_kernel.h
#define CMAPL_DEFAULT_NUM_ATTR_DEFS	32	// default attribute definitions array size
#define CMAPL_DEFAULT_NUM_ITER_RECS	4	// default iterative record offsets array size
#define CMAPL_CURRENT_SECTION_VERSION	1	// current section definition version

// data file access type
#define CMAPL_READ_ONLY		1
#define CMAPL_WRITE_ONLY	2
#define CMAPL_READ_WRITE	3

// macros to manipulate the MSB; the MSB is used in the offsets into
// the binary data to indicate whether or not there is an iterative
// record for a specific object, period, and timestamp; 0 means it
// is not iterative and 1 means it is iterative
#define CMAPL_IS_ITERATIVE(x)		(0x8000000000000000LLU & (x))
#define CMAPL_SET_ITERATIVE(x)		(x) |= 0x8000000000000000LLU
#define CMAPL_GET_OFFSET(x)		(0x7FFFFFFFFFFFFFFFLLU & (x))
#define CMAPL_MAX_INT			((uint64_t)(-1))

// flag to indicate there is no record at a certain timestamp
#define CMAPL_NO_RECORD                 (CMAPL_GET_OFFSET(CMAPL_MAX_INT))

// calculates which bucket this timestamp belongs to with respect to the start time of the file
// using the formula (index of bucket using msb) - (offset index using start time of file)
// current_time: time to find the bucket for
// start_time: time of the first sample in the file
// period: specifies which period's bucket to search in
#define CMAPL_GET_BUCKET(current_time, start_time, period)  (((current_time)/(period)) - ((start_time)/(period)))
#define CMAPL_ROUND_TIME(time, period) ((time) - ((time) % (period)))

// metadata and compressed data file extensions
#define CMAPL_META_EXT		".meta"
#define CMAPL_META_EXT_LEN	5
#define CMAPL_ZALLOC_META_NAME(basefilename,metafilename) { \
        metafilename = (char*)CMAPL_ZALLOC(CMAPL_STRLEN(basefilename) + CMAPL_META_EXT_LEN + 1); \
        if (metafilename != NULL){ \
                CMAPL_SNPRINTF(metafilename, CMAPL_STRLEN(basefilename) + CMAPL_META_EXT_LEN + 1, "%s"CMAPL_META_EXT, basefilename); \
        } }

#define CMAPL_GZ_EXT            ".gz"
#define CMAPL_GZ_EXT_LEN        3
#define CMAPL_ZALLOC_GZ_NAME(basefilename,datafilename) { \
        datafilename = (char*)CMAPL_ZALLOC(CMAPL_STRLEN(basefilename) + CMAPL_GZ_EXT_LEN + 1); \
        if (datafilename != NULL){ \
                CMAPL_SNPRINTF(datafilename, CMAPL_STRLEN(basefilename) + CMAPL_GZ_EXT_LEN + 1, "%s"CMAPL_GZ_EXT, basefilename); \
        } }

// the archive files have the following format: <file index>_<start time>_<duration>.ccma
#define CMAPL_CURRENT_FILE	"_0_0."	// this is part of the format of the current file being written to;
					// an example would be 000001_0_0.ccma; both the start time and duration
					// are set to zero
#define CMAPL_EMPTY_FILE	"_0."	// this is part of the format of an empty file with a duration of 0;
					// an example would be 000001_1234567890123_0.ccma; the duration is zero
					// whereas the start time has been set

// array of all possible periods; updating this involves also updating the
// implementation of period_map in cmapl_util.c
#define CMAPL_MAX_NUM_PERIODS 14
extern const uint64_t _cmapl_period_map[CMAPL_MAX_NUM_PERIODS];

// config period is 999999 seconds (stored in milliseconds)
#define CMAPL_CONFIG_PERIOD 999999000
#define CMAPL_CONFIG_PRESET_STR "config"

// a single 64-bit offset
typedef uint64_t cmapl_offset_t;

/*********************************************************
 * These are internal enums used in the parsing library. *
 *********************************************************/

/**
 * This enum lists all the different types of indexes
 * that are used in the CMAPL.
 */
typedef enum {
	CMAPL_OBJECT_INDEX,
	CMAPL_NEXT_HEADER,
	TOTAL_NUM_SECTIONS
} cmapl_section_t;

typedef enum {
	CMAPL_CNT_ID,
	CMAPL_CNT_KEY,
	CMAPL_CNT_PERIOD
} cmapl_cnt_group_t;

/*************************
 * CMAPL Data Structures *
 *************************/

/**
 * This structure contains all the information for a single
 * section definition.
 */
typedef struct {
	cmapl_section_t				section_type;			/**< the section type for this file */
	uint32_t				section_version;		/**< the version of the section */
	cmapl_offset_t				section_offset;			/**< offset to where this section is located
											in the file */
	cmapl_offset_t				header_offset;			/**< offset to where this section is written to
											in the XML header, used to rewrite the
											value of the offset if needed */
} cmapl_section_def_t_v6_0;

/**
 * These structures contain all the properties of a single counter.
 */
typedef struct {
	struct cmapl_object_handle*		object;				/**< the object this counter belongs to */
	struct cmapl_counter_handle*            next_cnt_by_id;			/**< pointer to the next counter per object in
											sequential order by their id */
} cmapl_counter_int_t_v5_0;

typedef struct {
	struct cmapl_object_handle*             object;                         /**< the object this counter belongs to */
	struct cmapl_period*			periods[CMAPL_MAX_NUM_PERIODS];	/**< the periods this counter belongs to */
	cmapl_offset_t				record_offsets[CMAPL_MAX_NUM_PERIODS];	/** offset into record of counter value
											at its periods */
	struct cmapl_counter_handle*		next_cnt_by_id;			/**< pointer to the next counter per object in
											sequential order by their id */
	struct cmapl_counter_handle*		next_cnt_by_key;		/**< pointer to the next key counter per object
											in sequential order by their id */
	struct cmapl_counter_handle*		next_cnt_by_period[CMAPL_MAX_NUM_PERIODS];	/**< pointer to the next counter per
											object per period in sequential order by their
											id */
#ifdef _CMAPL_TEST_
        // test fields
	uint64_t				last_value;                             /**< test field used to validate if this
                                                                                                counter's value is monotonically
                                                                                                increasing */
#endif
} cmapl_counter_int_t_v6_0;

typedef struct cmapl_counter_handle {
	uint32_t				major_v;			/**< major version number */
	uint32_t				minor_v;			/**< minor version number */
	union {
		cmapl_counter_def_t_v5_0	e5_0;
		cmapl_counter_def_t_v6_0        e6_0;
	};									/**< version specific external information */
	union {
		cmapl_counter_int_t_v5_0	i5_0;
		cmapl_counter_int_t_v6_0	i6_0;
	};									/**< version specific internal information */
} cmapl_counter_handle_t;


/**
 * This structure holds offset information for iterative records.
 * Given an object, period, and timestamp, there could be more than
 * one record. A linked list is used to allow for more than one
 * record.
 */
typedef struct {
	uint32_t				num_offsets;			/**< current number of iterative record offsets for
											this object, period, and timestamp */
	cmapl_offset_t*				offsets;			/**< array of iterative record offsets */
	uint32_t				offsets_size;			/**< number of elements in the iterative record
											offsets array */
} cmapl_iterative_record_offsets_t;

/**
 * This is a linked list data structure of periods
 */
typedef struct cmapl_period {
	struct cmapl_object_handle*		object;				/**< the object this period belongs to */
	uint64_t				period;				/**< the period associated with this list in
											milliseconds */
	uint32_t				period_index;			/**< the index into the period map for this period */
	struct cmapl_period*			next_period;			/**< pointer to the next period */
	cmapl_counter_handle_t*			cnt_list;			/**< all counters of one object with this period
											sorted by counter id from lowest to highest */
        uint32_t                                num_cnts;                       /**< number of counters sampled at this period and object */
	uint64_t                                cnts_size;                      /**< binary size in bytes of all counters in this period;
											this includes all key counters */
	cmapl_offset_t*				record_offsets;			/**< array of record offsets, with one offset per
											timestamp in the file that sampling occurs for
											this period; the first bit of this offset is a
											flag representing whether the offset points
											to an interative record (1) or not (0) */
	cmapl_iterative_record_offsets_t*	iterative_record_offsets;	/**< array of iterative record offsets to binary records
											for this period; each element in the array is
											a list of iterative record offsets for a specific
											timestamp in the file */
	cmapl_counter_handle_t*                 current_cnt;                    /**< current counter being looked at, used for
                                                                                        iteration */
	uint32_t				num_offsets;			/**< number of time and record offsets; equals the
											time duration of the whole file divided by
											this period */
} cmapl_period_t;

/**
 * These structures contain all the properties of a single object.
 */
typedef struct {
	struct cmapl_header_handle*             header;                         /**< the XML header this object belongs to */
        uint32_t                                iterator_index;			/**< index of the next counter for the
											iterator to return */
	uint32_t				num_cnts_sampled;		/**< number of counters in this object being sampled */
        cmapl_counter_handle_t*			cnt_list;			/**< linked list of counters sampled in this object */
} cmapl_object_int_t_v5_0;

typedef struct {
	struct cmapl_header_handle*             header;                         /**< the XML header this object belongs to */
        uint32_t                                iterator_index;                 /**< index of the next counter for the
                                                                                        iterator to return */
	cmapl_counter_handle_t*			key_cnt_list;			/**< linked list of key counters for this object */
	uint32_t				num_key_cnts_sampled;		/**< number of key counters in this object */
	cmapl_period_t*				period_list;			/**< linked list of counters grouped by their
											period, sorted from smallest to largest */
	uint32_t				num_periods;			/**< number of unique periods in this object */
	cmapl_counter_handle_t*			cnt_list;			/**< linked list of all counters sampled in this
											object ordered by counter id */
	uint32_t                                num_cnts_sampled;               /**< number of total counters in this object */
	cmapl_counter_handle_t*                 current_cnt;                    /**< current counter being looked at, used for
											iteration */
	struct cmapl_object_handle*		next_object;			/**< pointer to the next object handle in this
											XML header ordered by object ID */
} cmapl_object_int_t_v6_0;

typedef struct cmapl_object_handle {
	uint32_t				major_v;			/**< major version number */
        uint32_t				minor_v;			/**< minor version number */
        union {
		cmapl_object_def_t_v5_0		e5_0;
		cmapl_object_def_t_v6_0		e6_0;
	};									/**< version specific external information */
	union {
                cmapl_object_int_t_v5_0		i5_0;
		cmapl_object_int_t_v6_0		i6_0;
        };									/**< version specific internal information */
} cmapl_object_handle_t;

/**
 * This structure contains all the system information for an archive file.
 */
typedef struct {
	uint32_t				major_v;			/**< major version number */
        uint32_t				minor_v;			/**< minor version number */
	union {
		cmapl_system_def_t_v5_0		e5_0;
		cmapl_system_def_t_v6_0		e6_0;
	};									/**< version specific external information */
} cmapl_header_system_t;

/**
 * These structures contain all the properties of the XML header.
 */
typedef struct {
	struct cmapl_file_handle*		data_file;			/**< the data file this XML header belongs to */
	cmapl_header_system_t			system;				/**< system information for this archive file */
        cmapl_attribute_def_t_v5_0**		attr_defs;			/**< list of attribute definitions */
	uint32_t                                attr_array_size;                /**< size of attribute definitions array */
	uint32_t                                num_attr_defs;                  /**< total number of attribute definitions
                                                                                        for this archive file */
        cmapl_record_header_def_t_v5_0		record_def;			/**< declares all the size information about
											the record headers used to parse each
											record */
        uint32_t                                iterator_index;			/**< index of the next object for the iterator
											to return or of the next object to be added */
	uint32_t				num_objects_sampled;		/**< number of objects in the XML header
											preset being sampled */
	cmapl_object_handle_t**			objects;			/**< array of objects for this XML header,
											indexed by object id */
	uint32_t				objects_array_size;		/**< size of objects array */
} cmapl_header_int_t_v5_0;

typedef struct {
	struct cmapl_file_handle*               data_file;                      /**< the data file this XML header belongs to */
        cmapl_header_system_t			system;				/**< system information for this archive file */
        cmapl_attribute_def_t_v6_0**		attr_defs;			/**< list of attribute definitions */
	uint32_t				attr_array_size;		/**< size of attribute definitions array */
	uint32_t				num_attr_defs;			/**< total number of attribute definitions
											for this archive file */
	uint32_t                                num_objects_sampled;		/**< number of objects in the XML header
											preset being sampled */
	cmapl_object_handle_t**                 objects;			/**< array of objects for this XML header,
											indexed by object id */
	uint32_t                                objects_array_size;		/**< size of objects array */
	cmapl_offset_t				header_offset;			/**< offset in to the file where this header starts,
											from the beginning of the data file */
	cmapl_section_def_t_v6_0*		section_defs;			/**< array of section definitions, types of sections
											in this XML header and where those
											sections are located within the file */
	cmapl_offset_t*				object_index;			/**< array of offsets into the object index block
											to where that object and period's timestamps
											are located; the size is the total number of
											objects in the XML header multiplied by the
											maximum number of periods */
	cmapl_object_handle_t*                  object_list;                    /**< linked list of objects ordered by object ID */
	cmapl_object_handle_t*			current_object;			/**< current object being looked at, used for iteration */
	cmapl_offset_t				duration_header_offset;		/**< offset in to the XML header of the file to where
											the duration is located */
	cmapl_offset_t                          start_time_header_offset;       /**< offset in to the XML header of the file to where
                                                                                        the start time information is located */
	cmapl_offset_t                          end_time_header_offset;         /**< offset in to the XML header of the file to where
                                                                                        the end time information is located */

	// DEBUG INFO for burt 582812
        uint64_t                                pc;                             /**< program counter of the last time the objects
                                                                                        array was freed */
        void*                                   pid;                            /**< the thread id of the thread that last freed
                                                                                        the objects array */
} cmapl_header_int_t_v6_0;

typedef struct cmapl_header_handle {
	uint32_t				major_v;			/**< major version number */
        uint32_t				minor_v;			/**< minor version number */
	union {
		cmapl_header_def_t_v5_0		e5_0;
		cmapl_header_def_t_v6_0		e6_0;
	};									/**< version specific external information */
	union {
                cmapl_header_int_t_v5_0		i5_0;
		cmapl_header_int_t_v6_0         i6_0;
        };									/**< version specific internal information */
} cmapl_header_handle_t;

/**
 * These structures contain all the properties of a single archive file.
 */
typedef struct {
        CMAPL_FILE*				data_file;			/**< handle to actual archive file being
											written to */
	char*                                   file_path;                      /**< path to the archive file */
	cmapl_header_handle_t*			header;				/**< all the XML header information for this
											archive file */
	cmapl_offset_t				iterator_offset;		/**< offset into the binary of the next record
											for the iterator to return */
	struct cmapl_query_handle*              query;                          /**< the result of the most recent query made
                                                                                        to this file */
} cmapl_file_int_t_v5_0;

typedef struct {
        CMAPL_FILE*				metadata_file;                  /**< handle to metadata and index file */
	char*					metadata_file_path;		/**< path to metadata and index file */
	CMAPL_GZFILE				data_file;			/**< handle to binary record data file */
	char*                                   data_file_path;                 /**< path to binary record data file */
        uint32_t				access_flag;			/**< flag indicating the file's read/write access
											type */
        cmapl_header_handle_t*			header;				/**< all the XML header information for this
											archive file */
	uint64_t                                start_time;                     /**< Msecs since start of file of the first
                                                                                        record */
        uint64_t                                end_time;                       /**< Msecs since start of file of the last
                                                                                        record */
	uint64_t				metadata_written_count;		/**< number of bytes written to the metadata file */
	uint64_t                                data_written_count;             /**< number of bytes written to the data file */
        cmapl_offset_t				iterator_offset;		/**< offset into the binary of the next record
											for the iterator to return */
	cmapl_offset_t				current_offset;			/**< current offset into the file */
	uint64_t				duration;			/**< duration of time this file will be sampling
											for in milliseconds */
	struct cmapl_query_handle*		query;				/**< the result of the most recent query made
											to this file */
        struct cmapl_record_handle*             current_record;                 /**< current record being accessed by the user */
} cmapl_file_int_t_v6_0;

typedef struct cmapl_file_handle {
	uint32_t				major_v;			/**< major version number */
        uint32_t				minor_v;			/**< minor version number */
	union {
		cmapl_file_def_t_v5_0		e5_0;
		cmapl_file_def_t_v6_0		e6_0;
	};									/**< version specific external information */
	union {
		cmapl_file_int_t_v5_0		i5_0;
                cmapl_file_int_t_v6_0		i6_0;
        };									/**< version specific internal information */
} cmapl_file_handle_t;

/**
 * These structures contain the value, type, and metadata for
 * a single counter data.
 */
typedef struct {
	struct cmapl_instance_handle*		instance;			/**< the instance this counter data belongs to */
	cmapl_counter_handle_t*			counter;			/**< the counter handle of this counter data */
} cmapl_counter_data_int_t_v5_0;

typedef cmapl_counter_data_int_t_v5_0 cmapl_counter_data_int_t_v6_0;

typedef struct {
	uint32_t                                major_v;                        /**< major version number */
        uint32_t                                minor_v;                        /**< minor version number */
	union {
                cmapl_counter_data_t_v5_0       e5_0;
                cmapl_counter_data_t_v6_0       e6_0;
        };                                                                      /**< version specific external information */
        union {
                cmapl_counter_data_int_t_v5_0   i5_0;
                cmapl_counter_data_int_t_v6_0   i6_0;
        };									/**< version specific internal information */
} cmapl_counter_data_handle_t;

/**
 * These structures contain all the properties of a single instance. This
 * instance can either be a whole instance or a partial instance.
 * Each whole instance contains all the counter data, regardless of period,
 * that's sampled for a specific object at a certain timestamp. Whole
 * instances will have a pointer to the timestamp handle it belongs to.
 * Each partial instance contains all the counter data for a single record,
 * and therefore, a single period for a specific object. Partial instances
 * will have a pointer to the record handle it belongs to.
 */
typedef struct {
	struct cmapl_timestamp_handle*          timestamp;                      /**< the timestamp this instance belongs to */
        struct cmapl_record_handle*             record;                         /**< the record this instance belongs to */
	char*					instance_buffer;		/**< the binary data buffer of this instance */
	uint64_t				size;				/**< size of the binary data buffer in bytes */
	uint32_t				next_counter_index;		/**< index of the next counter to be returned */
	cmapl_counter_data_handle_t*		current_counter_data;		/**< handle to the current counter data being
											requested */
} cmapl_instance_int_t_v5_0;

typedef cmapl_instance_int_t_v5_0 cmapl_instance_int_t_v6_0;

typedef struct cmapl_instance_handle {
	uint32_t				major_v;			/**< major version number */
        uint32_t				minor_v;			/**< minor version number */
	union {
		cmapl_instance_data_t_v5_0	e5_0;
		cmapl_instance_data_t_v6_0      e6_0;
	};									/**< version specific external information */
	union {
                cmapl_instance_int_t_v5_0       i5_0;
		cmapl_instance_int_t_v6_0       i6_0;
        };									/**< version specific internal information */
} cmapl_instance_handle_t;

/**
 * These structures contain all the properties of a single record.
 */
typedef struct {
	char*					data_buffer;			/**< buffer containing the record's binary data */
	uint32_t				num_instances;			/**< total number of instances in this record */
        uint64_t                                instance_offset;                /**< offset to the next instance in the buffer */
        cmapl_instance_handle_t*                current_instance;               /**< pointer to the current instance when iterating
                                                                                        through all the instances; only one instance
                                                                                        is kept in memory at a time; getting the next
                                                                                        instance handle will cause this current one to
                                                                                        be freed and replaced with the new one */
        cmapl_offset_t                          offset;                         /**< offset of this record within the data file */
        cmapl_object_handle_t*                  object;                         /**< object this record samples */
        cmapl_period_t*                         period;                         /**< period this record samples at */
        cmapl_counter_handle_t**                counters;                       /**< counters this record samples */
        uint32_t                                num_cnts;                       /**< number of counters this record samples */
} cmapl_record_int_t_v5_0;

typedef cmapl_record_int_t_v5_0 cmapl_record_int_t_v6_0;

typedef struct cmapl_record_handle {
	uint32_t				major_v;			/**< major version number */
        uint32_t				minor_v;			/**< minor version number */
        union {
		cmapl_record_header_data_t_v5_0	e5_0;
		cmapl_record_header_data_t_v6_0	e6_0;
	};									/**< version specific external information */
	union {
                cmapl_record_int_t_v5_0         i5_0;
		cmapl_record_int_t_v6_0         i6_0;
        };									/**< version specific internal information */
} cmapl_record_handle_t;

/**
 * These structures contain the data for all the whole instances of
 * a given object at a certain timestamp.
 */
typedef struct {
	struct cmapl_query_handle*		query;				/**< query handle this timestamp belongs to */
	uint64_t				timestamp;			/**< the programmed timestamp of these instances */
	uint32_t				num_instances;			/**< number of whole instances at this timestamp */
	uint64_t				size;				/**< size of the instances data buffer in bytes */
	char*					instances_buffer;		/**< buffer containing all the binary data of all the
											whole instances */
	uint64_t				next_instance;			/**< offset to the next instance in the buffer */
	cmapl_instance_handle_t*		current_instance;		/**< pointer to the current instance when iterating
											through all the instances; only one instance
											is kept in memory at a time; getting the next
											instance handle will cause this current one to
											be freed and replaced with the new one */
} cmapl_timestamp_int_t_v5_0;

typedef cmapl_timestamp_int_t_v5_0 cmapl_timestamp_int_t_v6_0;

typedef struct cmapl_timestamp_handle {
	uint32_t                                major_v;                        /**< major version number */
        uint32_t                                minor_v;                        /**< minor version number */
	union {
		cmapl_timestamp_int_t_v5_0	i5_0;
		cmapl_timestamp_int_t_v6_0	i6_0;
        };									/**< version specific internal information */
} cmapl_timestamp_handle_t;

/**
 * These structures contain all the result of a single query.
 * The result of the query is first split up by the timestamp
 * and then by all the whole instances within each timestamp.
 */
typedef struct {
	struct cmapl_file_handle*		data_file;			/**< data file this query belongs to */
	cmapl_object_handle_t*			object;				/**< object this query is searching for */
	uint64_t*				timestamps;			/**< array of all timestamps returned in this query */
	uint32_t				num_timestamps;			/**< number of timestamps */
	cmapl_counter_handle_t**		counters;			/**< array of counter handles that are being searched for */
	uint32_t                                next_timestamp;                 /**< index to the next timestamp to return */
	cmapl_timestamp_handle_t*		current_timestamp;		/**< current timestamp handle of all the timstamps
											returned by this query; each timestamp handle
											contains all the whole instances returned
											for that timestamp; only one timestamp
											is kept in memory at a time; getting the
											next timestamp handle will cause this current
											one to be freed and replaced with the new one */
} cmapl_query_int_t_v5_0;

typedef struct {
	struct cmapl_file_handle*               data_file;                      /**< data file this query belongs to */
	cmapl_object_handle_t*                  object;                         /**< object this query is searching for */
        uint64_t*                               timestamps;                     /**< array of all timestamps returned in this query */
        uint32_t                                num_timestamps;                 /**< number of timestamps */
        cmapl_counter_handle_t**                counters;                       /**< array of counter handles that are being searched for */
	cmapl_counter_handle_t**                key_counters;                   /**< array of key counter handles that are being searched for */
        cmapl_period_t**                        periods;                        /**< array of all the unique counters' periods */
        uint32_t                                num_periods;                    /**< number of unique counters' periods */
	uint32_t                                next_timestamp;                 /**< index to the next timestamp to return */
        cmapl_timestamp_handle_t*               current_timestamp;              /**< current timestamp handle of all the timstamps
                                                                                        returned by this query; each timestamp handle
                                                                                        contains all the whole instances returned
                                                                                        for that timestamp; only one timestamp
                                                                                        is kept in memory at a time; getting the
                                                                                        next timestamp handle will cause this current
                                                                                        one to be freed and replaced with the new one */
} cmapl_query_int_t_v6_0;

typedef struct cmapl_query_handle {
	uint32_t                                major_v;                        /**< major version number */
        uint32_t                                minor_v;                        /**< minor version number */
	union {
                cmapl_query_def_t_v5_0		e5_0;
                cmapl_query_def_t_v6_0		e6_0;
        };                                                                      /**< version specific external information */
        union {
                cmapl_query_int_t_v5_0		i5_0;
                cmapl_query_int_t_v6_0		i6_0;
        };                                                                      /**< version specific internal information */
} cmapl_query_handle_t;

typedef struct {
	uint32_t				num_data_files;			/**< number of data files within the given time range */
	cmapl_file_def_t_v5_0*                  file_infos;                     /**< array of each data file's metadata, including generic
                                                                                        file path and start and end time in UTC milliseconds */
	uint32_t				array_size;			/**< size of the file_infos array */
	uint32_t				file_index;			/**< index of the file currently being looked at */
} cmapl_directory_int_t_v5_0;

typedef struct {
	uint32_t                                num_data_files;                 /**< number of data files within the given time range */
	cmapl_file_def_t_v6_0*			file_infos;			/**< array of each data file's metadata, including generic
											file path, meta file path, data file path, and start
											and end time in UTC milliseconds */
        uint32_t                                array_size;                     /**< size of the file_infos array */
        uint32_t                                file_index;                     /**< index of the next file to be looked at */
} cmapl_directory_int_t_v6_0;

typedef struct cmapl_directory_handle {
	uint32_t                                major_v;                        /**< major version number */
        uint32_t                                minor_v;                        /**< minor version number */
	union {
                cmapl_directory_def_t_v5_0      e5_0;
                cmapl_directory_def_t_v6_0      e6_0;
        };									/**< version specific external information */
	union {
                cmapl_directory_int_t_v5_0	i5_0;
                cmapl_directory_int_t_v6_0	i6_0;
	};									/**< version specific internal information */
} cmapl_directory_handle_t;

#ifdef _CMAPL_USER_SPACE_
typedef struct {
        const char**                            file_names;                     /**< name of each file within the corefile */
        uint32_t                                num_files;                      /**< number of files within the corefile */
        uint32_t                                file_index;                     /**< index of the next file to be looked at */
        cmapl_file_handle_t*                    current_file;                   /**< handle of current file that's opened */
        const char**                            dir_names;                      /**< name of each directory within the corefile */
        uint32_t                                num_dirs;                       /**< number of directories within the corefile */
        uint32_t                                dir_index;                      /**< index of the next directory to be looked at */
        cmapl_directory_handle_t*               current_dir;                    /**< handle of current directory that's opened */
        const char*                             temp_dir;                       /**< temporary directory to extract files to */

        // lzma data
        CFileInStream                           archiveStream;                  /**< stream used to read the file */
        CLookToRead                             lookStream;                     /**< stream used to read the file */
        CSzArEx                                 db;                             /**< database that contains information on each file
                                                                                        file in the corefile */
        ISzAlloc                                allocImp;                       /**< specifies allocation and free functions to use */
        ISzAlloc                                allocTempImp;                   /**< specifies allocation and free functions to use */
} cmapl_corefile_int_t_v6_0;

typedef struct cmapl_corefile_handle {
        uint32_t                                major_v;                        /**< major version number */
        uint32_t                                minor_v;                        /**< minor version number */
        union {
                cmapl_corefile_def_t_v6_0       e6_0;
        };
        union {
                cmapl_corefile_int_t_v6_0       i6_0;
        };
} cmapl_corefile_handle_t;
#endif /* _CMAPL_USER_SPACE_ */

/************************
 * CMAPL Util Functions *
 ************************/

/**
 * This function gets the offset into the object index section of the
 * file to where all the record offsets for a particular object and
 * period are.
 */
extern cmapl_error_t
cmapl_get_object_period_offset(
				cmapl_header_handle_t* header,
				uint32_t obj_count,
				uint64_t period,
				cmapl_offset_t* offset);

/**
 * This function sets the offset into the object index section of the
 * file to where all the record offsets for a particular object and
 * period are.
 */
extern cmapl_error_t
cmapl_set_object_period_offset(
				cmapl_header_handle_t* header,
				uint32_t obj_count,
				uint64_t period,
				cmapl_offset_t offset);

/**
 * This function adds a counter handle to a counter linked list
 * in sorted order by counter id. The linked list group that this
 * counter will be added to will depend on the type parameter. The
 * type of list can be either id (type 0), key (type 1), or period
 * (type 2). The first element of the new linked list is passed back
 * through the counter_list parameter. The return value will be of
 * type cmapl_error_t indicating either success or failure.
 */
extern cmapl_error_t
cmapl_add_counter_to_list(
				cmapl_counter_handle_t** counter_list,
				cmapl_counter_handle_t* counter,
				cmapl_cnt_group_t type,
				uint64_t period);

/**
 * This function adds an object handle to an object linked list
 * in sorted order by object id. The first element of the new linked
 * list is passed back through the object_list parameter. The return
 * value will be of type cmapl_error_t indicating either success or
 * failure.
 */
extern cmapl_error_t
cmapl_add_object_to_list(
                                cmapl_object_handle_t** object_list,
                                cmapl_object_handle_t* object);

/**
 * This function XML encodes a list of strings and passes back the encoded
 * strings through the dsts parameter. If the string contained a special
 * character, was encoded, and new memory was allocated for it, the encoded_flag
 * will be set to true for that string. Those strings will need to be freed by
 * the cmapl_free_xml_encoded_strs function. The characters that are encoded
 * are '<', '>', '&', '\'', and '\"'.
 */
extern cmapl_error_t
cmapl_xml_encode(uint32_t num_strs, const char** srcs, char** dsts, bool* encoded_flags);

/**
 * This function frees all the strings that have been encoded, meaning that new
 * memory was allocated for those strings. The encoded_flags determine which
 * strings to free.
 */
extern void
cmapl_free_xml_encoded_strs(uint32_t num_strs, char** dsts, bool* encoded_flags);

/**
 * This function converts a valid period into a number
 * used to index into an array.
 */
extern cmapl_error_t
cmapl_get_period_index(uint64_t period, uint32_t* period_index);

/**
 * This function checks if a given period is valid.
 */
extern bool
cmapl_is_period_valid(uint64_t period);

/**
 * This function checks if the specified major and minor version
 * numbers match that of this data structure.
 */
extern bool
cmapl_is_version(void * data, uint32_t major_v, uint32_t minor_v);

/**
 * This function parses the filename for the start and end time of
 * the file. In other words, the time of the oldest record sampled
 * in the file and the time of the most recent record sampled in the
 * file. It also parses out the index of the file and the longest
 * sampled period in the file. The times and period returned are all
 * in milliseconds.
 */
extern cmapl_error_t
cmapl_parse_filename(char * filename, uint32_t * file_index, uint64_t * period, uint64_t * start_time, uint64_t * end_time);

/**
 * These functions free all the different internal data structures.
 */
extern void cmapl_free_counter_data(cmapl_counter_data_handle_t* counter_data);
extern void cmapl_free_instance(cmapl_instance_handle_t* instance);
extern void cmapl_free_timestamp(cmapl_timestamp_handle_t* timestamp);
extern void cmapl_free_query(cmapl_query_handle_t* query);
extern void cmapl_free_record(cmapl_record_handle_t* record);
extern void cmapl_free_counter(cmapl_counter_handle_t* counter);
extern void cmapl_free_period(cmapl_period_t* period_list);
extern void cmapl_free_object(cmapl_object_handle_t* object);
extern void cmapl_free_system(cmapl_header_system_t syst);
extern void cmapl_free_attr_def(void* attr_def);
extern void cmapl_free_header(cmapl_header_handle_t* header);
extern cmapl_error_t cmapl_free_file(cmapl_file_handle_t* data_file);
extern void cmapl_free_directory(cmapl_directory_handle_t* dir);
#ifdef _CMAPL_USER_SPACE_
extern void cmapl_free_corefile(cmapl_corefile_handle_t* core);
#endif /* _CMAPL_USER_SPACE_ */

/**
 * These functions copy all external definition structures. Any
 * memory that is allocated by these functions is expected to
 * be freed when the file is closed.
 *
 * Precondition: The dst argument must be allocated zero'ed memory.
 */
extern cmapl_error_t cmapl_copy_attr_def_v6_0(cmapl_attribute_def_t_v6_0 * src, cmapl_attribute_def_t_v6_0 * dst);
extern cmapl_error_t cmapl_copy_counter_def_v6_0(cmapl_counter_def_t_v6_0 * src, cmapl_counter_def_t_v6_0 * dst);
extern cmapl_error_t cmapl_copy_directory_def_v6_0(cmapl_directory_def_t_v6_0 * src, cmapl_directory_def_t_v6_0 * dst);
extern cmapl_error_t cmapl_copy_header_def_v6_0(cmapl_header_def_t_v6_0 * src, cmapl_header_def_t_v6_0 * dst);
extern cmapl_error_t cmapl_copy_object_def_v6_0(cmapl_object_def_t_v6_0 * src, cmapl_object_def_t_v6_0 * dst);
extern cmapl_error_t cmapl_copy_query_def_v6_0(cmapl_query_def_t_v6_0 * src, cmapl_query_def_t_v6_0 * dst);
extern cmapl_error_t cmapl_copy_system_def_v6_0(cmapl_system_def_t_v6_0 * src, cmapl_system_def_t_v6_0 * dst);

/**
 * These functions validate all external definition structures and
 * check if any pointer field in the structure is NULL that shouldn't
 * be.
 */
extern cmapl_error_t cmapl_validate_attr_def_v6_0(cmapl_attribute_def_t_v6_0 * attr_def);
extern cmapl_error_t cmapl_validate_counter_def_v6_0(cmapl_counter_def_t_v6_0 * counter_def);
extern cmapl_error_t cmapl_validate_directory_def_v6_0(cmapl_directory_def_t_v6_0 * dir_def);
extern cmapl_error_t cmapl_validate_header_def_v6_0(cmapl_header_def_t_v6_0 * header_der);
extern cmapl_error_t cmapl_validate_object_def_v6_0(cmapl_object_def_t_v6_0 * object_def);
extern cmapl_error_t cmapl_validate_query_def_v6_0(cmapl_query_def_t_v6_0 * query_def);
extern cmapl_error_t cmapl_validate_system_def_v6_0(cmapl_system_def_t_v6_0 * system_def);

/******************************************
 * XML header constant string definitions *
 ******************************************/

#define CMAPL_XML_HEADER_XML_STR_V6_0 \
"<?xml version=\"1.0\" encoding=\"UTF-8\" standalone=\"yes\"?>\n"

#define CMAPL_XML_HEADER_DTD_STR_V6_0 \
"<!DOCTYPE archive [\n" \
"	<!ELEMENT archive (system, attribute_def+, index_def*, preset)>\n" \
"	<!ELEMENT system        (version, base_time, node_uuid, hostname, model, serial_number, system_id)>\n" \
"	<!ELEMENT version       (#PCDATA)>\n" \
"	<!ELEMENT base_time	(#PCDATA)>\n" \
"	<!ELEMENT node_uuid	(#PCDATA)>\n" \
"	<!ELEMENT hostname      (#PCDATA)>\n" \
"	<!ELEMENT model         (#PCDATA)>\n" \
"	<!ELEMENT serial_number (#PCDATA)>\n" \
"	<!ELEMENT system_id     (#PCDATA)>\n" \
"	<!ELEMENT attribute_def (name, value, description*)>\n" \
"	<!ELEMENT name          (#PCDATA)>\n" \
"	<!ELEMENT value         (#PCDATA)>\n" \
"	<!ELEMENT description   (#PCDATA)>\n" \
"	<!ELEMENT section_def	(section_type, section_version, section_offset)>\n" \
"	<!ELEMENT section_type	(#PCDATA)>\n" \
"	<!ELEMENT section_version	(#PCDATA)>\n" \
"	<!ELEMENT section_offset	(#PCDATA)>\n" \
"	<!ELEMENT preset        (attribute_def*, object+)>\n" \
"	<!ELEMENT object        (name, id, counter+)>\n" \
"	<!ELEMENT counter       (name, description, id, base_id, size, period+, privilege, x_name*, y_name*)>\n" \
"	<!ELEMENT id            (#PCDATA)>\n" \
"	<!ELEMENT base_id       (#PCDATA)>\n" \
"	<!ELEMENT size          (#PCDATA)>\n" \
"	<!ELEMENT period	(#PCDATA)>\n" \
"	<!ELEMENT privilege	(#PCDATA)>\n" \
"	<!ELEMENT x_name        (#PCDATA)>\n" \
"	<!ELEMENT y_name        (#PCDATA)>\n" \
"	<!ELEMENT variable (name, size, description)>\n" \
"	<!ATTLIST archive version CDATA #REQUIRED>\n" \
"	<!ATTLIST archive preset_version CDATA #REQUIRED>\n" \
"	<!ATTLIST archive generation CDATA #REQUIRED>\n" \
"	<!ATTLIST archive header_offset CDATA #REQUIRED>\n" \
"	<!ATTLIST archive duration CDATA #REQUIRED>\n" \
"	<!ATTLIST archive start_time CDATA #REQUIRED>\n" \
"	<!ATTLIST archive end_time CDATA #REQUIRED>\n" \
"	<!ATTLIST counter is_key CDATA #REQUIRED>\n" \
"	<!ATTLIST counter ctype CDATA #REQUIRED>\n" \
"	<!ATTLIST variable type CDATA #REQUIRED>\n" \
"]>\n"

#define CMAPL_XML_HEADER_ARCHIVE_BEGIN_STR_V6_0 \
"<archive version=\"%u.%u\" generation=\"%u\" preset_version=\"%s\" header_offset=\"%"PRIu64"\" duration=\""

#define CMAPL_XML_HEADER_ARCHIVE_BEGIN_DURATION_STR_V6_0 \
"%16"PRIX64"\" start_time=\""

#define CMAPL_XML_HEADER_ARCHIVE_BEGIN_START_TIME_STR_V6_0 \
"%16"PRIX64"\" end_time=\""

#define CMAPL_XML_HEADER_ARCHIVE_BEGIN_END_TIME_STR_V6_0 \
"%16"PRIX64"\">\n"

#define CMAPL_XML_HEADER_SYSTEM_STR_V6_0 \
"	<system>\n" \
"		<version>%s</version>\n" \
"		<base_time>%"PRIu64"</base_time>\n" \
"		<node_uuid>%s</node_uuid>\n" \
"		<hostname>%s</hostname>\n" \
"		<model>%s</model>\n" \
"		<serial_number>%s</serial_number>\n" \
"		<system_id>%s</system_id>\n" \
"	</system>\n"

#define CMAPL_XML_HEADER_ATTR_DEF_STR_V6_0 \
"	<attribute_def>\n" \
"		<name>%s</name>\n" \
"		<value>%u</value>\n" \
"		<description>%s</description>\n" \
"	</attribute_def>\n"

#define CMAPL_XML_HEADER_SECTION_DEF_BEGIN_STR_V6_0 \
"	<section_def>\n" \
"		<section_type>%u</section_type>\n" \
"		<section_version>%u</section_version>\n" \
"		<section_offset>"

#define CMAPL_XML_HEADER_SECTION_DEF_END_STR_V6_0 \
"%16"PRIX64"</section_offset>\n" \
"	</section_def>\n"

#define CMAPL_XML_HEADER_PRESET_BEGIN_STR_V6_0 \
"	<preset>\n"

#define CMAPL_XML_HEADER_OBJECT_BEGIN_STR_V6_0 \
"		<object>\n" \
"			<name>%s</name>\n" \
"			<id>%u</id>\n"

#define CMAPL_XML_HEADER_COUNTER_BEGIN_STR_V6_0 \
"			<counter is_key=\"%d\" ctype=\"%u\">\n" \
"				<name>%s</name>\n" \
"				<description>%s</description>\n" \
"				<id>%u</id>\n" \
"				<base_id>%u</base_id>\n" \
"				<size>%u</size>\n" \
"				<privilege>%s</privilege>\n"

#define CMAPL_XML_HEADER_COUNTER_PERIOD_STR_V6_0 \
"				<period>%"PRIu64"</period>\n" \

#define CMAPL_XML_HEADER_COUNTER_X_ITEM_STR_V6_0 \
"				<x_name>%s</x_name>\n"

#define CMAPL_XML_HEADER_COUNTER_Y_ITEM_STR_V6_0 \
"				<y_name>%s</y_name>\n"

#define CMAPL_XML_HEADER_COUNTER_END_STR_V6_0 \
"			</counter>\n"

#define CMAPL_XML_HEADER_OBJECT_END_STR_V6_0 \
"		</object>\n"

#define CMAPL_XML_HEADER_PRESET_END_STR_V6_0 \
"	</preset>\n"

#define CMAPL_XML_HEADER_ARCHIVE_END_STR_V6_0 \
"</archive>\n"

#endif /* _CMAPL_INT_DEFS_H_ */
