/**
 * The CMAPL internal read-side function declarations. These functions
 * are version specific declarations. Different version data files will
 * use functions that will have different implementations. The external
 * functions will call one of these functions depending on the version
 * off the data file.
 *
 * The descriptions of the external CMAPL functions related to each of
 * these internal functions are described in cmapl_read.h.
 *
 * Syntax of functions:
 *
 *      <function_name>_v<major_version>_<minor_version>
 *
 *      function_name   - the external function name
 *      major_version   - the major version number of the data file
 *      minor_version   - the minor version number of the data file
 *
 *      For Example:    cmapl_find_counter_v6_0 stands for the
 *                      cmapl_find_counter function used on
 *                      a version 6.0 data file.
 *
 */

#ifndef _CMAPL_READ_INT_H_
#define _CMAPL_READ_INT_H_

#include <libxml/parser.h>
#include <libxml/tree.h>

/**
 * The internal *_find_*, *_first_*, and *_next_* functions include an extra parameter
 * from their external version. The bool parameter update_iter specifies whether or not the
 * internal iterator of that specific data structure should be incremented or updated.
 * All external calls to these functions should set the parameter to true. All internal
 * calls to these functions should set the parameter to false. This is because we don't
 * want internal calls to affect the end user without their knowledge.
 */
cmapl_error_t
cmapl_find_counter_by_id_v6_0(
                        cmapl_object_handle_t * object,
                        uint32_t counter_id,
                        cmapl_counter_handle_t ** counter,
			bool update_iter);

cmapl_error_t
cmapl_find_counter_by_name_v6_0(
                        cmapl_object_handle_t * object,
                        const char * counter_name,
                        cmapl_counter_handle_t ** counter,
			bool update_iter);

cmapl_error_t
cmapl_find_counter_data_v6_0(
			cmapl_instance_handle_t * instance,
			const char * counter_name,
			cmapl_counter_handle_t ** counter,
			cmapl_counter_data_handle_t ** counter_data,
			bool update_iter);

cmapl_error_t
cmapl_find_instance_v6_0(
                        cmapl_timestamp_handle_t * timestamp,
                        const char ** key_cnt_names,
			cmapl_counter_data_t_v6_0 * key_cnt_values,
                        uint32_t num_key_cnts,
                        cmapl_instance_handle_t ** instance,
			bool update_iter);

cmapl_error_t
cmapl_find_object_v6_0(
                        cmapl_header_handle_t * header,
                        const char * object_name,
                        cmapl_object_handle_t ** object,
			bool update_iter);

cmapl_error_t
cmapl_first_counter_v6_0(
                        cmapl_object_handle_t * object,
                        cmapl_counter_handle_t ** counter,
			bool update_iter);

cmapl_error_t
cmapl_first_counter_by_period_v6_0(
                        cmapl_object_handle_t * object,
                        uint64_t period,
                        cmapl_counter_handle_t ** counter,
			bool update_iter);

cmapl_error_t
cmapl_first_counter_data_v6_0(
                        cmapl_instance_handle_t * instance,
			cmapl_counter_handle_t ** counter,
                        cmapl_counter_data_handle_t ** counter_data,
			bool update_iter);

cmapl_error_t
cmapl_first_instance_v6_0(
                        cmapl_timestamp_handle_t * timestamp,
                        cmapl_instance_handle_t ** instance,
			bool update_iter);

cmapl_error_t
cmapl_first_object_v6_0(
                        cmapl_header_handle_t * header,
                        cmapl_object_handle_t ** object,
			bool update_iter);

cmapl_error_t
cmapl_first_timestamp_v6_0(
                        cmapl_query_handle_t * query,
                        cmapl_timestamp_handle_t ** timestamp,
			bool update_iter);

cmapl_error_t
cmapl_get_counter_data_value_v6_0(
                        cmapl_counter_data_handle_t * counter_data,
                        cmapl_counter_data_t_v6_0 * counter_data_value);

cmapl_error_t
cmapl_get_counter_info_v6_0(
                        cmapl_counter_handle_t * counter,
                        cmapl_counter_def_t_v6_0 * counter_info);

cmapl_error_t
cmapl_get_counter_type_v6_0(
                        cmapl_counter_handle_t * counter,
                        cmapl_cnt_size_t * size,
                        cmapl_cnt_type_t * type,
                        cmapl_cnt_unit_t * unit);

cmapl_error_t
cmapl_get_file_info_v6_0(
                        cmapl_file_handle_t * data_file,
                        cmapl_file_def_t_v6_0 * file_info);

cmapl_error_t
cmapl_get_header_v6_0(
                        cmapl_file_handle_t * data_file,
                        cmapl_header_handle_t ** header);

cmapl_error_t
cmapl_get_header_info_v6_0(
                        cmapl_header_handle_t * header,
                        cmapl_header_def_t_v6_0 * header_info);

cmapl_error_t
cmapl_get_instance_data_v6_0(
                        cmapl_instance_handle_t * instance,
                        cmapl_instance_data_t_v6_0 * instance_data);

cmapl_error_t
cmapl_get_object_info_v6_0(
                        cmapl_object_handle_t * object,
                        cmapl_object_def_t_v6_0 * object_info);

cmapl_error_t
cmapl_get_object_periods_v6_0(
                        cmapl_object_handle_t * object,
                        uint64_t ** periods,
                        uint32_t * num_periods);

cmapl_error_t
cmapl_get_system_info_v6_0(
                        cmapl_header_handle_t * header,
                        cmapl_system_def_t_v6_0 * system_info);

cmapl_error_t
cmapl_get_timestamp_v6_0(
                        cmapl_timestamp_handle_t * timestamp,
                        uint64_t * time);

cmapl_error_t
cmapl_next_counter_v6_0(
                        cmapl_object_handle_t * object,
                        cmapl_counter_handle_t ** counter,
			bool update_iter);

cmapl_error_t
cmapl_next_counter_by_period_v6_0(
                        cmapl_object_handle_t * object,
                        uint64_t period,
                        cmapl_counter_handle_t ** counter,
			bool update_iter);

cmapl_error_t
cmapl_next_counter_data_v6_0(
                        cmapl_instance_handle_t * instance,
			cmapl_counter_handle_t ** counter,
                        cmapl_counter_data_handle_t ** counter_data,
			bool update_iter);

cmapl_error_t
cmapl_next_instance_v6_0(
                        cmapl_timestamp_handle_t * timestamp,
                        cmapl_instance_handle_t ** instance,
			bool update_iter);

cmapl_error_t
cmapl_next_object_v6_0(
                        cmapl_header_handle_t * header,
                        cmapl_object_handle_t ** object,
			bool update_iter);

cmapl_error_t
cmapl_next_timestamp_v6_0(
                        cmapl_query_handle_t * query,
                        cmapl_timestamp_handle_t ** timestamp,
			bool update_iter);

cmapl_error_t
cmapl_query_v6_0(
                        cmapl_file_handle_t * data_file,
                        cmapl_query_def_t_v6_0 * query_def,
                        cmapl_query_handle_t ** query);


///////////////////////////////////////////////////////////////////////////
// The following functions are internal-only fuctions, meaning they have no
// general external version of them.
///////////////////////////////////////////////////////////////////////////

/**
 * @name cmapl_add_instance
 *
 * @param[in]		timestamp		Handle of timestamp
 * @param[in]		instance_index		Index of the instance to be added from
 *						the first record
 * @param[in]		records			Array of all the records to search
 * @param[in]           num_records		Number of records in the array
 * @return		cmapl_error_t		Error flag
 *
 * This method creates and adds a single whole instance to a specific timestamp handle.
 * The specific instance to add will be determined by instance_index, which specifies
 * which instance to add from the first record in the array of records. The rest of the
 * records will be examined to see if they also contain that same instance. All the records
 * must contain the instance for the instance to be added. If some record doesn't contain
 * the instance, then the function will return successfully but no instance will be added.
 * If all the records contain the instance, then the data from each record for the instance
 * will all be concatinated together, removing duplicate key counter data. All this data
 * combined will result in a whole instance.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_add_instance_v6_0(
                        cmapl_timestamp_handle_t * timestamp,
                        uint32_t instance_index,
                        cmapl_record_handle_t ** records,
                        uint32_t num_records);

/**
 * @name cmapl_add_record_data
 *
 * @param[in]           record                  Handle of record
 * @param[in]           record_offset           Offset into the file of the next chunk
 *						of data for this record
 * @param[in]           data_file               Handle of data file containing the record
 * @return              cmapl_error_t           Error flag
 *
 * This method adds additional record data from the file to a specific record handle. This
 * function will be called multiple times for iterative records. The data to be added is
 * pointed to by the record_offset parameter. Once the data is retrieved, it will be added
 * to the record handle and it's metadata will be updated.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_add_record_data_v6_0(
                        cmapl_record_handle_t * record,
                        cmapl_offset_t record_offset,
                        cmapl_file_handle_t * data_file);

/**
 * @name cmapl_first_instance_from_record
 *
 * @param[in]           record          Handle of record
 * @param[out]          instance        Handle of first instance in record
 * @return              cmapl_error_t   Error flag
 * @retval              CMAPL_SUCCESS           Function finished successfully
 * @retval              CMAPL_INVALID_ARGUMENTS The argument was NULL or invalid
 * @retval              CMAPL_INVALID_VERSION   The version of the handle is not known
 *
 * This method takes in a handle of a record, and retrieves the first instance in
 * that record. The instance is returned back to the user through the instance
 * parameter in the form of a handle. NULL is returned for the instance handle
 * either if the record doesn't contain any instances or if there was some error.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_first_instance_from_record_v6_0(
                        cmapl_record_handle_t * record,
                        cmapl_instance_handle_t ** instance);

/**
 * @name cmapl_first_record
 *
 * @param[in]           data_file       Handle of data file
 * @param[out]          record          Handle of first record in data file
 * @return              cmapl_error_t   Error flag
 * @retval              CMAPL_SUCCESS           Function finished successfully
 * @retval              CMAPL_INVALID_ARGUMENTS The argument was NULL or invalid
 * @retval              CMAPL_INVALID_VERSION   The version of the handle is not known
 *
 * This method takes in a handle of a data file, and retrieves a handle of the first
 * record in the data file. This will reset the internal iterator for the data file
 * to point to the first record.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_first_record_v6_0(
                        cmapl_file_handle_t * data_file,
                        cmapl_record_handle_t ** record);

/**
 * @name cmapl_get_counter_data
 *
 * @param[in]           counter			Handle of counter
 * @param[in]           counter_buffer		Binary data containing counter value
 * @param[out]          counter_data		Counter data value
 * @post					The function allocates memory for any
 *						string counter values. This will need to
 *						be freed by the user.
 * @return              cmapl_error_t           Error flag
 *
 * This method gets the counter data value for a specific counter from the buffer containing
 * the value of the counter in binary. This value is passed back through the counter_data
 * parameter.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_counter_data_v6_0(
                        cmapl_counter_handle_t * counter,
                        char* counter_buffer,
                        cmapl_counter_data_t_v6_0 * counter_data);

/**
 * @name cmapl_get_counter_offset
 *
 * @param[in]           instance                Handle of instance
 * @param[in]           counter                 Handle of counter to search for
 * @param[out]          counter_offset          Offset into instance buffer to where
 *						counter data value is located
 * @return              cmapl_error_t           Error flag
 *
 * This method gets the offset into the instance buffer to where the counter data value
 * is located. The offset is passed back through the counter_offset parameter.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_counter_offset_v6_0(
                        cmapl_instance_handle_t * instance,
                        cmapl_counter_handle_t * counter,
                        cmapl_offset_t * counter_offset);

/**
 * @name cmapl_get_counters_periods
 *
 * @param[in]           query                   Handle of query
 * @return              cmapl_error_t           Error flag
 *
 * This method gets all the unique periods of all the counters being queried for in a
 * query handle. This array of unique periods is stored inside the query handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_counters_periods_v6_0(
                        cmapl_query_handle_t * query);

/**
 * @name cmapl_get_instance_key_counters
 *
 * @param[in]           instance		Handle of instance
 * @return              cmapl_error_t           Error flag
 *
 * This method gets all the key counters names and values for this instance. That information
 * is stored in the instance handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_instance_key_counters_v6_0(
                        cmapl_instance_handle_t * instance);


/**
 * @name cmapl_get_instance_size
 *
 * @param[in]           record                  Handle of record
 * @pre                                         The record handle must not be NULL.
 * @return              cmapl_error_t           Error flag
 *
 * This method gets the size of a single instance in a specific record.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
uint32_t
cmapl_get_instance_size_v6_0(
                        cmapl_record_handle_t * record);

/**
 * @name cmapl_get_instances
 *
 * @param[in]           timestamp               Handle of timestamp
 * @return              cmapl_error_t           Error flag
 *
 * This method gets the data for all the whole instances at a specific timestamp. All the data
 * from each whole instance is concatenated together into one big buffer. This buffer is stored
 * in the timestamp handle. A whole instance is a single instance at a particular timestamp
 * containing all the data from every counter that was queried for in every record, regardless
 * of period.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_instances_v6_0(
                        cmapl_timestamp_handle_t * timestamp);

/**
 * @name cmapl_get_query_counters
 *
 * @param[in]           query_h			Handle of query
 * @param[in]		query_def		Query metadata
 * @return              cmapl_error_t           Error flag
 *
 * This method gets all the counter handles of the key counters and regular counters
 * specified in the query definition structure. If no regular counters were specified,
 * then all counters of the object being queried will be retrieved.
 *
 * For each regular counter retrieved, that counter's name will get copied to the query
 * handle's list of counter names in its external query definition structure.
 *
 * This function is to be used with the cmapl_copy_query_def_v6_0() routine. After the
 * copy routine is called, this function should then be called to get the counter handles.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_query_counters_v6_0(
			cmapl_query_handle_t * query_h,
			cmapl_query_def_t_v6_0 *query_def);

/**
 * @name cmapl_get_record
 *
 * @param[in]           timestamp               Handle of timestamp
 * @param[in]           period			Period of the record
 * @param[out]          record			Handle of record
 * @return              cmapl_error_t           Error flag
 *
 * This method creates and returns a record handle. The specific record is specified by
 * a timestamp handle and the period of the record. Iterative records will be contained
 * in one record handle. The record handle will have the data buffer of all the counters
 * sampled for the specific object, time, and period. The record handle is passed back
 * through the record parameter.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_record_v6_0(
                        cmapl_timestamp_handle_t * timestamp,
                        cmapl_period_t * period,
                        cmapl_record_handle_t ** record);

/**
 * @name cmapl_get_timestamps
 *
 * @param[in]           query			Handle of query
 * @return              cmapl_error_t           Error flag
 *
 * This method gets all the valid timestamps for some query. A valid timestamp is one that's
 * within the query time range, a multiple of the query period, and has records for every
 * counter asked for. This array of timestamps is stored in the query handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_get_timestamps_v6_0(
                        cmapl_query_handle_t * query);

/**
 * @name cmapl_is_instance_equal
 *
 * @param[in]           key_cnts		Array of key counter handles
 * @param[in]           key_cnt_vals		Array of key counter values
 * @param[in]           num_key_cnts		Number of key counters
 * @param[in]		period			The period this instance was sampled at
 * @param[in]           instance_buffer		The instance buffer to check
 * @param[out]          is_equal                Boolean flag indicating if the instance is equal
 * @return              cmapl_error_t           Error flag
 *
 * This method checks if an instance, represented by an instance buffer, is equal to an
 * array of key counters and their values. If the instance's key counter values match with
 * the ones passed in, then the instance is equal and true is passed back through the boolean
 * is_equal parameter.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_is_instance_equal_v6_0(
                        cmapl_counter_handle_t ** key_cnts,
                        cmapl_counter_data_t_v6_0 * key_cnt_vals,
                        uint32_t num_key_cnts,
			cmapl_period_t * period,
			char * instance_buffer,
                        bool * is_equal);

/**
 * @name cmapl_is_timestamp_valid
 *
 * @param[in]           timestamp		Timestamp to check
 * @param[in]		query			Handle of query
 * @param[out]		is_valid		Boolean flag indicating if the timestamp is valid
 * @return              cmapl_error_t           Error flag
 *
 * This method checks if some timestamp is valid for some query. A valid timestamp is one that's
 * within the query time range, a multiple of the query period, and has records for every
 * counter asked for. The boolean flag indicating whether or not the timestamp is valid is passed
 * back through the is_valid parameter.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_is_timestamp_valid_v6_0(
                        uint64_t timestamp,
                        cmapl_query_handle_t * query,
                        bool * is_valid);

/**
 * @name cmapl_next_instance_from_record
 *
 * @param[in]           record          Handle of record
 * @param[out]          instance        Handle of next instance in record
 * @pre                                 A previous call to either cmapl_next_instance_from_record,
 *					or cmapl_first_instance_from_record must have been called
 *                                      on this record.
 * @return              cmapl_error_t   Error flag
 * @retval              CMAPL_SUCCESS           Function finished successfully
 * @retval              CMAPL_INVALID_ARGUMENTS The argument was NULL or invalid
 * @retval              CMAPL_INVALID_VERSION   The version of the handle is not known
 *
 * This method takes in a handle of a record, and retrieves a handle of the next
 * instance in the record. The next instance returned depends on how many previous
 * calls to cmapl_next_instance_from_record and cmapl_first_instance_from_record there were.
 * The internal iterator in the record will be incremented with each call to this function.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_next_instance_from_record_v6_0(
                        cmapl_record_handle_t * record,
                        cmapl_instance_handle_t ** instance);

/**
 * @name cmapl_next_record
 *
 * @param[in]           data_file       Handle of data file
 * @param[out]          record          Handle of next record in data file
 * @pre                                 A previous call to either cmapl_next_record,
 *					or cmapl_first_record must have been called
 *                                      on this data file.
 * @return              cmapl_error_t   Error flag
 * @retval              CMAPL_SUCCESS           Function finished successfully
 * @retval              CMAPL_INVALID_ARGUMENTS The argument was NULL or invalid
 * @retval              CMAPL_INVALID_VERSION   The version of the handle is not known
 *
 * This method takes in a handle of a data file, and retrieves a handle of the next
 * record in the data file. The next record returned depends on how many previous
 * calls to cmapl_next_record and cmapl_first_record there were. The internal iterator
 * in the data file will be incremented with each call to this function.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_next_record_v6_0(
                        cmapl_file_handle_t * data_file,
                        cmapl_record_handle_t ** record);

/**
 * @name cmapl_parse_attribute_def
 *
 * @param[in]           attr_def_node           An attribute definition node in the XML header
 * @param[in]           header			Handle of XML header
 * @return              cmapl_error_t           Error flag
 *
 * This method parses an attribute definition element in the XML header and adds that
 * information to the XML header handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_attribute_def_v6_0(
                        xmlNode * attr_def_node,
                        cmapl_header_handle_t * header);

/**
 * @name cmapl_parse_counter
 *
 * @param[in]           counter_node            A counter node in the XML header
 * @param[in]           object                  Handle of object
 * @return              cmapl_error_t           Error flag
 *
 * This method parses a counter element in the XML header and adds that
 * information to the object handle it belongs to.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_counter_v6_0(
                        xmlNode * counter_node,
                        cmapl_object_handle_t * object);

/**
 * @name cmapl_parse_iterative_record_offsets
 *
 * @param[in]           data_file               Handle of data file
 * @param[in]           period			Period of the record
 * @param[in]           index			Index to which timestamp to look for
 * @return              cmapl_error_t           Error flag
 *
 * This method parses the object index and retrieves the iterative record offsets
 * for a specific object, period, and timestamp.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_iterative_record_offsets_v6_0(
                        cmapl_file_handle_t * data_file,
                        cmapl_period_t * period,
                        uint32_t index);

/**
 * @name cmapl_parse_object
 *
 * @param[in]           object_node             An object node in the XML header
 * @param[in]           header                  Handle of header
 * @return              cmapl_error_t           Error flag
 *
 * This method parses a single object element in the XML header and adds that
 * information to the XML header handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_object_v6_0(
                        xmlNode * object_node,
                        cmapl_header_handle_t * header);

/**
 * @name cmapl_parse_object_index
 *
 * @param[in]           data_file		Handle of data file
 * @return              cmapl_error_t           Error flag
 *
 * This method parses the whole object index and stores that information in the data
 * file handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_object_index_v6_0(
                        cmapl_file_handle_t * data_file);

/**
 * @name cmapl_parse_objects
 *
 * @param[in]           preset_node             A preset node in the XML header
 * @param[in]           header                  Handle of header
 * @return              cmapl_error_t           Error flag
 *
 * This method parses all the object elements in the XML header and adds their
 * information to the XML header handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_objects_v6_0(
                        xmlNode * preset_node,
                        cmapl_header_handle_t * header);

/**
 * @name cmapl_parse_record_offsets
 *
 * @param[in]           data_file		Handle of data file
 * @param[in]           period			Period of of the records
 * @param[in]		obj_count		The n'th object sampled in the file
 * @return              cmapl_error_t           Error flag
 *
 * This method parses the object index for all the record offsets for a specific object
 * at a certain period. This information is stored within the period data structure.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_record_offsets_v6_0(
                        cmapl_file_handle_t * data_file,
                        cmapl_period_t * period,
                        uint32_t obj_count);

/**
 * @name cmapl_parse_section_def
 *
 * @param[in]           section_def_node        A section definition node in the XML header
 * @param[in]           header                  Handle of header
 * @return              cmapl_error_t           Error flag
 *
 * This method parses a section definition element in the XML header and adds that
 * information to the XML header handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_section_def_v6_0(
                        xmlNode * section_def_node,
                        cmapl_header_handle_t * header);

/**
 * @name cmapl_parse_system
 *
 * @param[in]           system_node             A system node in the XML header
 * @param[in]           data_file		Handle of data file
 * @param[in]           header_def		XML header definition
 * @param[out]          header                  Handle of header
 * @post					The memory for the header handle is allocated
 *						by the function. This memory will be freed when
 *						the data file is closed.
 * @return              cmapl_error_t           Error flag
 *
 * This method parses the system element in the XML header and adds that
 * information to the XML header handle. Using a header definition and the
 * found system information, it creates and passes back a XML header handle
 * through the header parameter.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_system_v6_0(
                        xmlNode * system_node,
                        cmapl_file_handle_t * data_file,
                        cmapl_header_def_t_v6_0 * header_def,
                        cmapl_header_handle_t ** header);

/**
 * @name cmapl_parse_xml_header
 *
 * @param[in]           root_node		The root node of the XML header
 * @param[in]           data_file               Handle of data file
 * @return              cmapl_error_t           Error flag
 *
 * This method parses the whole XML header and stores that information in the data file
 * handle.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_parse_xml_header_v6_0(
                        xmlNode * root_node,
                        cmapl_file_handle_t * data_file);

/**
 * @name cmapl_set_counter_value_offsets
 *
 * @param[in]           header                  Handle of header
 * @return              cmapl_error_t           Error flag
 *
 * This method sets the offset to where each counter's value data is located in their respective
 * records. This offset is stored in each of the counters' handles. Key counters will have an
 * array of record offsets, one for each period of its object. This is because the offset into a
 * record for a key counter will be different depending on the period and the number of counters
 * sampled at that period.
 *
 * The return value will be of type cmapl_error_t indicating either success or failure.
 */
cmapl_error_t
cmapl_set_counter_value_offsets_v6_0(
                        cmapl_header_handle_t * header);

#endif /* _CMAPL_READ_INT_H_ */
