# this is basically -*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment.pm#1 $
#
## @summary	Environment.pm - Provide a hardware independent environment
##		object.
## @author	dl-platform-qa@netapp.com
## @status	public
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod		here

=head1 NAME

Environment - Provide a hardware independent environment object

=head1 SYNOPSIS

  use Platform::Environment;

  # Environment constructor
  $ess_obj = Platform::Environment->new(hostspec => $hostspec);

=head1 DESCRIPTION

Provides a hardware independent environment object to be used for the
Environmental Subsystem.

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-platform-qa@netapp.com
  $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment.pm#1 $
  Last revised $Date: 2013/01/22 $

head1 COPYRIGHT

  Copyright 2008 Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Platform::Environment;

use strict;
use warnings;

use Sysconfig;
use Carp qw(croak);
use Params::Validate qw(validate SCALAR);
use Server;

#---------------------------------------------------------------------
# Version for package
#---------------------------------------------------------------------
our $VERSION = '1.0';

#---------------------------------------------------------------------
# Global variables
#---------------------------------------------------------------------
our @ISA;

BEGIN {
    # Die with a script error when an
    # input parameter fails validation
    Params::Validate::validation_options(
        on_fail => sub { my @msgs = @_;
                         chomp(@msgs);
                         Carp::confess(Tharn::script_result(@msgs)) }
    );
};

#---------------------------------------------------------------------

=head2 new

  $ess_obj = Platform::Environment->new(hostspec => $hostspec,
                                        [, connspec => $connspec ]);

Creates an independent hardware environment object.

The following named arguments are accepted:

=over

=item Arguments

=over

=item hostspec

(Required) : Hostspec of environment.

=item connspec

(Optional) : Connection specification for the environment.
Defaults to '#rsh-socket'.

=back

=item Returns

Environment object or croak on failure.

=back

=for html <hr />

=cut

#---------------------------------------------------------------------
sub new {
    my $class = shift;
    my (%opts) = validate (
        @_,
        {
            hostspec => { type => SCALAR },
            connspec => { type => SCALAR, default => '#rsh-socket' },
        }
    );
    my (%args, $obj, $group);

    # Get connectrec object to switch
    Tharn::host($opts{hostspec});
    $args{conn} = Tharn::connect($opts{connspec});

    # Get the model name
    $obj = Sysconfig->new($opts{hostspec});
    $args{model} = $obj->slot('0', 'model_name');

    # Get the platform unique sensors
    ($group) = ($args{model} =~ /(\D+\d\d)\d*/g);
    if ($group =~ 'FAS27') {
        require Platform::Environment::FAS200;
        @ISA = qw(Platform::Environment::FAS200);
    } elsif ($group =~ 'FAS2') {
        require Platform::Environment::FAS2000;
        @ISA = qw(Platform::Environment::FAS2000);
    } elsif ($group =~ 'FAS9') {
        require Platform::Environment::FAS900;
        @ISA = qw(Platform::Environment::FAS900);
    } elsif ($group =~ 'FAS3') {
        require Platform::Environment::FAS3000;
        @ISA = qw(Platform::Environment::FAS3000);
    } elsif ($group =~ 'FAS6') {
        require Platform::Environment::FAS6000;
        @ISA = qw(Platform::Environment::FAS6000);
    } else {
        croak "Model is not supported: $args{model}";
    }

    return $class->SUPER::new(%args);
}

#---------------------------------------------------------------------
# Destructor
#---------------------------------------------------------------------
sub DESTROY {
    my $self = shift;

    # Call destructor of base class if it exists
    if ( $self->can('SUPER::DESTROY') ) {
        $self->SUPER::DESTROY;
    }
}
#---------------------------------------------------------------------
sub Gx_new {
    my $class = shift;
    my (%opts) = validate (
        @_,
        {
            filer => { type => SCALAR },
            connspec => { type => SCALAR, default => '#ssh-persist' },
        }
    );
    my ($Server, $filer, $obj, $model, %args, $group);

    # Get connectrec object to switch
    $filer = $opts{filer};
    $Server = Server->new(nodes => "$filer");
    $args{conn} = $Server;
    

    # Get the model name
    my $cmd_sys = "run local sysconfig -av 0";
    $obj = $Server->cli(cmd =>$cmd_sys);
    foreach my $line (split(/\n/,$obj))
    {
      if($line =~ /Model Name:\s+(\w+)/i)
       {
         $model = $1;
       }
    }
    print "The value of model : $model \n";
    $args{model} = $model;

    # Get the platform unique sensors
    ($group) = ($args{model} =~ /(\D+\d\d)\d*/g);
    if ($group =~ 'FAS27') {
        require Platform::Environment::FAS200;
        @ISA = qw(Platform::Environment::FAS200);
    } elsif ($group =~ 'FAS2') {
        require Platform::Environment::FAS2000;
        @ISA = qw(Platform::Environment::FAS2000);
    } elsif ($group =~ 'FAS9') {
        require Platform::Environment::FAS900;
        @ISA = qw(Platform::Environment::FAS900);
    } elsif ($group =~ 'FAS3') {
        require Platform::Environment::FAS3000;
        @ISA = qw(Platform::Environment::FAS3000);
    } elsif ($group =~ 'FAS6') {
        require Platform::Environment::FAS6000;
        @ISA = qw(Platform::Environment::FAS6000);
    } else {
        croak "Model is not supported: $args{model}";
    }

    return $class->SUPER::new(%args);

} #End of Gx_new

#---------------------------------------------------------------------


1;
