# this is basically -*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS200.pm#1 $
#
## @summary	FAS200.pm - Provides a class that contains platform-specific
##		data to be used by Base.pm
## @author	dl-platform-qa@netapp.com
## @status	public
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod		here

=head1 NAME

FAS200 - Provides a class that contains platform-specific data for the class
Base.pm

=head1 SYNOPSIS

  use Platform::Environment::FAS200;

=head1 DESCRIPTION

Provides a class that contains platform-specific data to be used by Base.pm
which is the base class that allows hardware independency for the Environmental
Subsystem.

 # Example usage (Model Name: FAS270)
 #  Usage: environment status |
 #           [status] [shelf [<adapter>]] |
 #           [status] [shelf_log] |
 #           [status] [shelf_stats] |
 #           [status] [shelf_power_status] |
 #           [status] [chassis [all | list-sensors | Voltage | Temperature |
 #                              Power Supplies | Battery]]

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-platform-qa@netapp.com
  $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS200.pm#1 $
  Last revised $Date: 2013/01/22 $

head1 COPYRIGHT

  Copyright 2008, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Platform::Environment::FAS200;

use strict;
use warnings;

use Platform::Environment::Base;
use Params::Validate qw(validate OBJECT SCALAR);

use base qw(Platform::Environment::Base);

#---------------------------------------------------------------------
# Local variables
#---------------------------------------------------------------------
my %class_map_default = (
    all     => 'all',
    sensors => 'list-sensors',
    voltage => 'Voltage',
    temp    => 'Temperature',
    psu     => '"Power Supplies"',
    battery => 'Battery',
);
my %loc_map_default = (
    volt => [
        '3.3V',
        '5V',
        '2.5V',
        '1.2V',
        '12V',
    ],
    temp => [
        'Front Panel Temp',
        'Motherboard Temp',
    ],
    psu => [
        'PSU 1 Present',
        'PSU 1 AC OK',
        'PSU 1 DC OK',
        'PSU 2 Present',
        'PSU 2 AC OK',
        'PSU 2 DC OK',
    ],
    psu_fan => [
        'PSU 1 Fan 1',
        'PSU 1 Fan 2',
        'PSU 2 Fan 1',
        'PSU 2 Fan 2',
    ],
    nvram_batt => [
        'VBatt',
    ],
);
# Some of these are FRUS and some are informational (includes quantity)
my %fru_map_default = (
);

# All the mappings from model to maps
my %class_map = (
    default => \%class_map_default,
);

my %loc_map = (
    default => \%loc_map_default,
);

my %fru_map = (
    default => \%fru_map_default,
);

#---------------------------------------------------------------------

=head2 new

  $base_obj = Platform::Environment::FAS200->new(conn  => $conn,
                                                 model => $model);

This is a method in a derived class that acts as a wrapper around a method in
the base class (Base.pm). The SUPER pseudoclass lets you invoke an overridden
base class method without having to specify which class defined that method.
Do not use this constructor directly, use Platform::Environment->new() instead.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub new {
    my $class = shift;
    my (%opts) = validate (
        @_,
        {
            conn  => { type => OBJECT },
            model => { type => SCALAR },
        }
    );
    my $model = $opts{model};

    if (defined($class_map{$model})) {
        $opts{_class_map} = $class_map{$model};
    } else {
        $opts{_class_map} = $class_map{default};
    }
    if (defined($loc_map{$model})) {
        $opts{_loc_map} = $loc_map{$model};
    } else {
        $opts{_loc_map} = $loc_map{default};
    }
    if (defined($fru_map{$model})) {
        $opts{_fru_map} = $fru_map{$model};
    } else {
        $opts{_fru_map} = $fru_map{default};
    }
    return $class->SUPER::new(%opts);
}

1;
