# this is basically -*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS2000.pm#1 $
#
## @summary	FAS2000.pm - Provides a class that contains platform-specific
##		data to be used by Base.pm
## @author	dl-platform-qa@netapp.com
## @status	public
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod		here

=head1 NAME

FAS2000 - Provides a class that contains platform-specific data for the class
Base.pm

=head1 SYNOPSIS

  use Platform::Environment::FAS2000;

=head1 DESCRIPTION

Provides a class that contains platform-specific data to be used by Base.pm
which is the base class that allows hardware independency for the Environmental
Subsystem.

 # Example usage (Model Name: FAS2020)
 # Usage: environment status |
 #          [status] [shelf [<adapter>]] |
 #          [status] [shelf_log] |
 #          [status] [shelf_stats] |
 #          [status] [shelf_power_status] |
 #          [status] [chassis [all | list-sensors | Voltage | Temperature |
 #      Power Supplies OutVoltage | Battery | Power Supply 1 | Power Supply 2]]

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-platform-qa@netapp.com
  $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS2000.pm#1 $
  Last revised $Date: 2013/01/22 $

head1 COPYRIGHT

  Copyright 2008, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Platform::Environment::FAS2000;

use strict;
use warnings;

use Platform::Environment::Base;
use Params::Validate qw(validate OBJECT SCALAR);

use base qw(Platform::Environment::Base);

#---------------------------------------------------------------------
# Local variables
#---------------------------------------------------------------------
my %class_map_default = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    ps_outvoltage => '"Power Supplies OutVoltage"',
    battery       => 'Battery',
    psu1          => '"Power Supply 1"',
    psu2          => '"Power Supply 2"',
);
my %loc_map_default = (
    volt => [
        '1.1V',
        '1.2V',
        '1.5V',
        '1.8V',
        '12.0V',
        '2.5V',
        '3.3V',
        'CPU 1.2V',
    ],
    temp => [
        'Board Temp Bot',
        'Board Temp Top',
        'CPU Temp',
        'Chassis Temp Top',
        'Chassis Temp Bot',
    ],
    psu => [
        'Power Status',
        'PSU1',
        'PSU1 12V',
        'PSU1 5V',
        'PSU1 3V',
        'PSU1 12 Amp',
        'PSU1 5 Amp',
        'PSU2',
        'PSU2 12V',
        'PSU2 5V',
        'PSU2 3V',
        'PSU2 12 Amp',
        'PSU2 5 Amp',
    ],
    psu_fan => [
        'PSU1 Fan',
        'PSU2 Fan',
    ],
    psu_temp => [
        'PSU1 Temp',
        'PSU2 Temp',
    ],
    batt => [
        'Batt 8.0V',
        'Batt Amp',
        'Batt Run Time',
    ],
    batt_temp => [
        'Batt Temp',
    ],
    nvmem_batt_nvram => [
        'NVMEM 1.8V',
        'NVMEM 8.0V',
    ],
);

# Some of these are FRUS and some are informational (includes quantity)
my %fru_map_default = (
    'DIMM1' => 1,
    'DIMM2' => 1,
    'Baseboard Management Controller' => 1,
    'NVMEM Battery' => 1,
    'Mellanox MT25204 IB Cluster Interconnect' => 1,
    'SFP Transceiver' => 2,
#    'NetApp ATA/IDE Adapter' => 1, ## Commented by Prabha: Removing NetApp string since this is not matching with the output.
    'ATA/IDE Adapter' => 1,
);

# All the mappings from model to maps
my %class_map = (
    default => \%class_map_default,
);

my %loc_map = (
    default => \%loc_map_default,
);

my %fru_map = (
    default => \%fru_map_default,
);

#---------------------------------------------------------------------

=head2 new

  $base_obj = Platform::Environment::FAS2000->new(conn  => $conn,
                                                  model => $model);

This is a method in a derived class that acts as a wrapper around a method in
the base class (Base.pm). The SUPER pseudoclass lets you invoke an overridden
base class method without having to specify which class defined that method.
Do not use this constructor directly, use Platform::Environment->new() instead.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub new {
    my $class = shift;
    my (%opts) = validate (
        @_,
        {
            conn  => { type => OBJECT },
            model => { type => SCALAR },
        }
    );
    my $model = $opts{model};

    if (defined($class_map{$model})) {
        $opts{_class_map} = $class_map{$model};
    } else {
        $opts{_class_map} = $class_map{default};
    }
    if (defined($loc_map{$model})) {
        $opts{_loc_map} = $loc_map{$model};
    } else {
        $opts{_loc_map} = $loc_map{default};
    }
    if (defined($fru_map{$model})) {
        $opts{_fru_map} = $fru_map{$model};
    } else {
        $opts{_fru_map} = $fru_map{default};
    }
    return $class->SUPER::new(%opts);
}

1;
