# this is basically -*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS3000.pm#1 $
#
## @summary	FAS3000.pm - Provides a class that contains platform-specific
##		data to be used by Base.pm
## @author	dl-platform-qa@netapp.com
## @status	public
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod		here

=head1 NAME

FAS3000 - Provides a class that contains platform-specific data for the class
Base.pm

=head1 SYNOPSIS

  use Platform::Environment::FAS3000;

=head1 DESCRIPTION

Provides a class that contains platform-specific data to be used by Base.pm
which is the base class that allows hardware independency for the Environmental
Subsystem.

 # Example usage (Model Name: FAS3040)
 #  Usage: environment status |
 #           [status] [shelf [<adapter>]] |
 #           [status] [shelf_log] |
 #           [status] [shelf_stats] |
 #           [status] [shelf_power_status] |
 #           [status] [chassis [all | list-sensors | Temperature | PSU 1 |
 #       PSU 2 | Voltage | SYS FAN | NVRAM6-temperature-1 | NVRAM6-battery-1]]

 # Example usage (Model Name: FAS3070)
 #  Usage: environment status |
 #           [status] [shelf [<adapter>]] |
 #           [status] [shelf_log] |
 #           [status] [shelf_stats] |
 #           [status] [shelf_power_status] |
 #           [status] [chassis [all | list-sensors | Temperature | PSU 1 |
 #       PSU 2 | Voltage | SYS FAN | NVRAM6-temperature-1 | NVRAM6-battery-1]]

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-platform-qa@netapp.com
  $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS3000.pm#1 $
  Last revised $Date: 2013/01/22 $

head1 COPYRIGHT

  Copyright 2008, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Platform::Environment::FAS3000;

use strict;
use warnings;

use Platform::Environment::Base;
use Params::Validate qw(validate OBJECT SCALAR);

use base qw(Platform::Environment::Base);

#---------------------------------------------------------------------
# Local variables
#---------------------------------------------------------------------
my %class_map_default = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => '"SYS FAN"',
    nvram_battery => 'NVRAM6-battery-1',
    nvram_temp    => 'NVRAM6-temperature-1',
);
my %class_map_FAS3070 = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => '"SYS FAN"',
    nvram_battery => 'NVRAM6-battery-3',
    nvram_temp    => 'NVRAM6-temperature-3',
);

my %class_map_FAS3140 = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => '"SYS FAN"',
    nvram_battery => 'NVRAM7-battery-0',
    nvram_temp    => 'NVRAM7-temperature-0',
);

my %class_map_FAS3160 = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => '"SYS FAN"',
    nvram_battery => 'NVRAM7-battery-0',
    nvram_temp    => 'NVRAM7-temperature-0',
);

my %class_map_FAS3170 = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => '"SYS FAN"',
    nvram_battery => 'NVRAM7-battery-0',
    nvram_temp    => 'NVRAM7-temperature-0',
);


my %loc_map_default = (
    fan => [
        'SYS_FAN_1 Fan1',
        'SYS_FAN_1 Fan2',
        'SYS_FAN_2 Fan1',
        'SYS_FAN_2 Fan2',
    ],
    volt => [
        'CPU 3.3V Active',
        'CPU 3.3V Standby',
        'CPU 5V',
        'CPU 12V',
    ],
    temp => [
        'LCD board Temperature',
        'MB Front Zone Temperature',
        'MB Rear Zone Temperature',
    ],
    psu => [
        'PSU1 Present',
        'PSU1 AC IN',
        'PSU1 12V',
        'PSU1 5V',
        'PSU2 Present',
        'PSU2 AC IN',
        'PSU2 12V',
        'PSU2 5V',
    ],
    psu_fan => [
        'PSU1 FAN1',
        'PSU1 FAN2',
        'PSU2 FAN1',
        'PSU2 FAN2',
    ],
    nvram_temp => [
        'NVRAM6-temperature-1',
    ],
    nvram_batt => [
        'NVRAM6-battery-1',
    ],
);
my %loc_map_FAS3070 = (
    fan => [
        'SYS_FAN_1 Fan1',
        'SYS_FAN_1 Fan2',
        'SYS_FAN_2 Fan1',
        'SYS_FAN_2 Fan2',
    ],
    volt => [
        'CPU 3.3V Active',
        'CPU 3.3V Standby',
        'CPU 5V',
        'CPU 12V',
    ],
    temp => [
        'LCD board Temperature',
        'MB Front Zone Temperature',
        'MB Rear Zone Temperature',
    ],
    psu => [
        'PSU1 Present',
        'PSU1 AC IN',
        'PSU1 12V',
        'PSU1 5V',
        'PSU2 Present',
        'PSU2 AC IN',
        'PSU2 12V',
        'PSU2 5V',
    ],
    psu_fan => [
        'PSU1 FAN1',
        'PSU1 FAN2',
        'PSU2 FAN1',
        'PSU2 FAN2',
    ],
    nvram_temp => [
        'NVRAM6-temperature-3',
    ],
    nvram_batt => [
        'NVRAM6-battery-3',
    ],
);

my %loc_map_FAS3140 = (
    fan => [
    	'SYS_FAN_B1_Fan1',
	'SYS_FAN_B1_Fan2',
	'SYS_FAN_B2_Fan1',
	'SYS_FAN_B2_Fan2',
	'SYS_FAN_B3_Fan1',
	'SYS_FAN_B3_Fan2',
    ],
    volt => [
        'CPU_3.3V_Active',
        'CPU_3.3V_Standby',
        'CPU_5V',
        'CPU_12V',
    ],
    temp => [    
    	'MB_Temperature-CPU',  
	'MB_Temperature-Front',
	'MB_Temperature-Central',
    ],
    psu => [
    	'PSU1_Present',
	'PSU1_Power_Input',
	'PSU1_12V',
	'PSU1_Status',
	'PSU1_Temperature',
	'PSU2_Present',
	'PSU2_Power_Input',
	'PSU2_12V',
	'PSU2_Status',
	'PSU2_Temperature',
    ],
    psu_fan => [
        'PSU1_FAN1',
        'PSU1_FAN2',
        'PSU2_FAN1',
        'PSU2_FAN2',
    ],
    nvram_temp => [
        'NVRAM7-temperature-0',
    ],
    nvram_batt => [
        'NVRAM7-battery-0',
    ],
    unmonitored => [
    	'CPU1_Temperature',
    	'CPU2_Temperature',
		'P0_PCIRST_L',
		'HT1000_PWOK',
		'PWRSEQ_3_GD_A',
		'PWRSEQ_3_GD_B',
		'PWRSEQ_3_GD_C',
		'PWRSEQ_5_GD_A',
		'PWRSEQ_5_GD_B',
		'PWRSEQ_6_GD',
		'PWRSEQ_7_GD_CPU1',
		'PWRSEQ_7_GD_CPU2',
		'PWRSEQ_8_GD',
		'PWRSEQ_2_GD',
		'BTN_IN_L',
		'12V_PG',
		'PARTNER_PRESENT_L',
		'RTC Battery',
	],
);


my %loc_map_FAS3160 = (
    fan => [
    	'SYS_FAN_B1_Fan1',
	'SYS_FAN_B1_Fan2',
	'SYS_FAN_B2_Fan1',
	'SYS_FAN_B2_Fan2',
	'SYS_FAN_B3_Fan1',
	'SYS_FAN_B3_Fan2',
    ],
    volt => [
        'CPU_3.3V_Active',
        'CPU_3.3V_Standby',
        'CPU_5V',
        'CPU_12V',
    ],
    temp => [    
    	'MB_Temperature-CPU',  
	'MB_Temperature-Front',
	'MB_Temperature-Central',
    ],
    psu => [
    	'PSU1_Present',
	'PSU1_Power_Input',
	'PSU1_12V',
	'PSU1_Status',
	'PSU1_Temperature',
	'PSU2_Present',
	'PSU2_Power_Input',
	'PSU2_12V',
	'PSU2_Status',
	'PSU2_Temperature',
    ],
    psu_fan => [
        'PSU1_FAN1',
        'PSU1_FAN2',
        'PSU2_FAN1',
        'PSU2_FAN2',
    ],
    nvram_temp => [
        'NVRAM7-temperature-0',
    ],
    nvram_batt => [
        'NVRAM7-battery-0',
    ],
    unmonitored => [
    	'CPU1_Temperature',
    	'CPU2_Temperature',
		'P0_PCIRST_L',
		'HT1000_PWOK',
		'PWRSEQ_3_GD_A',
		'PWRSEQ_3_GD_B',
		'PWRSEQ_3_GD_C',
		'PWRSEQ_5_GD_A',
		'PWRSEQ_5_GD_B',
		'PWRSEQ_6_GD',
		'PWRSEQ_7_GD_CPU1',
		'PWRSEQ_7_GD_CPU2',
		'PWRSEQ_8_GD',
		'PWRSEQ_2_GD',
		'BTN_IN_L',
		'12V_PG',
		'PARTNER_PRESENT_L',
		'RTC Battery',
	],
);

my %loc_map_FAS3170 = (
    fan => [
    	'SYS_FAN_B1_Fan1',
	'SYS_FAN_B1_Fan2',
	'SYS_FAN_B2_Fan1',
	'SYS_FAN_B2_Fan2',
	'SYS_FAN_B3_Fan1',
	'SYS_FAN_B3_Fan2',
    ],
    volt => [
        'CPU_3.3V_Active',
        'CPU_3.3V_Standby',
        'CPU_5V',
        'CPU_12V',
    ],
    temp => [    
    	'MB_Temperature-CPU',  
	'MB_Temperature-Front',
	'MB_Temperature-Central',
    ],
    psu => [
    	'PSU1_Present',
	'PSU1_Power_Input',
	'PSU1_12V',
	'PSU1_Status',
	'PSU1_Temperature',
	'PSU2_Present',
	'PSU2_Power_Input',
	'PSU2_12V',
	'PSU2_Status',
	'PSU2_Temperature',
    ],
    psu_fan => [
        'PSU1_FAN1',
        'PSU1_FAN2',
        'PSU2_FAN1',
        'PSU2_FAN2',
    ],
    nvram_temp => [
        'NVRAM7-temperature-0',
    ],
    nvram_batt => [
        'NVRAM7-battery-0',
    ],
    unmonitored => [
    	'CPU1_Temperature',
    	'CPU2_Temperature',
		'P0_PCIRST_L',
		'HT1000_PWOK',
		'PWRSEQ_3_GD_A',
		'PWRSEQ_3_GD_B',
		'PWRSEQ_3_GD_C',
		'PWRSEQ_5_GD_A',
		'PWRSEQ_5_GD_B',
		'PWRSEQ_6_GD',
		'PWRSEQ_7_GD_CPU1',
		'PWRSEQ_7_GD_CPU2',
		'PWRSEQ_8_GD',
		'PWRSEQ_2_GD',
		'BTN_IN_L',
		'12V_PG',
		'PARTNER_PRESENT_L',
		'RTC Battery',
	],
);

# Some of these are FRUS and some are informational (includes quantity)
my %fru_map_default = (
    'CPU Board' => 1,
    'PS1' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'DIMM3' => 1,
    'DIMM4' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 4,
    'ATA/IDE Adapter' => 1,
    'NVRAM6' => 1,
);
my %fru_map_FAS3070 = (
    'CPU Board' => 1,
    'PS1' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'DIMM3' => 1,
    'DIMM4' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 1,
    'ATA/IDE Adapter' => 1,
    'NVRAM6' => 1,
);

my %fru_map_FAS3140 = (
    'CPU Board' => 1,
    'PS1' => 1,
    'PS2' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 1,
    'ATA/IDE Adapter' => 1,
    'NVRAM7' => 1,
);

my %fru_map_FAS3160 = (
    'CPU Board' => 1,
    'PS1' => 1,
    'PS2' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'DIMM5' => 1,
    'DIMM6' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 1,
    'ATA/IDE Adapter' => 1,
    'NVRAM7' => 1,
);

my %fru_map_FAS3170 = (
    'CPU Board' => 1,
    'PS1' => 1,
    'PS2' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'DIMM3' => 1,
    'DIMM4' => 1,
    'DIMM5' => 1,
    'DIMM6' => 1,
    'DIMM7' => 1,
    'DIMM8' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 1,
    'ATA/IDE Adapter' => 1,
    'NVRAM7' => 1,
);


# All the mappings from model to maps
my %class_map = (
    default => \%class_map_default,
    FAS3070 => \%class_map_FAS3070,
    FAS3140 => \%class_map_FAS3140,
    FAS3160 => \%class_map_FAS3160,
    FAS3170 => \%class_map_FAS3170,
);

my %loc_map = (
    default => \%loc_map_default,
    FAS3070 => \%loc_map_FAS3070,
    FAS3140 => \%loc_map_FAS3140,
    FAS3160 => \%loc_map_FAS3160,
    FAS3170 => \%loc_map_FAS3170, 
);

my %fru_map = (
    default => \%fru_map_default,
    FAS3070 => \%fru_map_FAS3070,
    FAS3140 => \%fru_map_FAS3140,
    FAS3160 => \%fru_map_FAS3160,
    FAS3170 => \%fru_map_FAS3170,
);

#---------------------------------------------------------------------

=head2 new

  $base_obj = Platform::Environment::FAS3000->new(conn  => $conn,
                                                  model => $model);

This is a method in a derived class that acts as a wrapper around a method in
the base class (Base.pm). The SUPER pseudoclass lets you invoke an overridden
base class method without having to specify which class defined that method.
Do not use this constructor directly, use Platform::Environment->new() instead.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub new {
    my $class = shift;
    my (%opts) = validate (
        @_,
        {
            conn  => { type => OBJECT },
            model => { type => SCALAR },
        }
    );
    my $model = $opts{model};

    if (defined($class_map{$model})) {
        $opts{_class_map} = $class_map{$model};
    } else {
        $opts{_class_map} = $class_map{default};
    }
    if (defined($loc_map{$model})) {
        $opts{_loc_map} = $loc_map{$model};
    } else {
        $opts{_loc_map} = $loc_map{default};
    }
    if (defined($fru_map{$model})) {
        $opts{_fru_map} = $fru_map{$model};
    } else {
        $opts{_fru_map} = $fru_map{default};
    }
    return $class->SUPER::new(%opts);
}

1;
