# this is basically -*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS6000.pm#1 $
#
## @summary	FAS6000.pm - Provides a class that contains platform-specific
##		data to be used by Base.pm
## @author	dl-platform-qa@netapp.com
## @status	public
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod		here

=head1 NAME

FAS6000 - Provides a class that contains platform-specific data for the class
Base.pm

=head1 SYNOPSIS

  use Platform::Environment::FAS6000;

=head1 DESCRIPTION

Provides a class that contains platform-specific data to be used by Base.pm
which is the base class that allows hardware independency for the Environmental
Subsystem.

 # Example usage (Model Name: FAS6080)
 #  Usage: environment status |
 #           [status] [shelf [<adapter>]] |
 #           [status] [shelf_log] |
 #           [status] [shelf_stats] |
 #           [status] [shelf_power_status] |
 #           [status] [chassis [all | list-sensors | Voltage | PSU 1 | PSU 2 |
 #                              Temperature | System_Fan | CF_status |
 #                              NVRAM6-temperature-1 | NVRAM6-battery-1]]

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-platform-qa@netapp.com
  $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS6000.pm#1 $
  Last revised $Date: 2013/01/22 $

head1 COPYRIGHT

  Copyright 2008, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Platform::Environment::FAS6000;

use strict;
use warnings;

use Platform::Environment::Base;
use Params::Validate qw(validate OBJECT SCALAR);

use base qw(Platform::Environment::Base);

#---------------------------------------------------------------------
# Local variables
#---------------------------------------------------------------------
my %class_map_default = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => 'System_Fan',
    cf_status     => 'CF_status',
    nvram_battery => 'NVRAM6-battery-1',
    nvram_temp    => 'NVRAM6-temperature-1',
);
my %class_map_FAS6030 = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => 'System_Fan',
    cf_status     => 'CF_status',
    nvram_battery => 'NVRAM6-battery-1',
    nvram_temp    => 'NVRAM6-temperature-1',
);
my %class_map_FAS6040 = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Voltage',
    temp          => 'Temperature',
    psu1          => '"PSU 1"',
    psu2          => '"PSU 2"',
    fan           => 'System_Fan',
    cf_status     => 'CF_status',
    nvram_battery => 'NVRAM6-battery-2',
    nvram_temp    => 'NVRAM6-temperature-2',
);
my %loc_map_default = (
    volt => [
        'CPU3.3V Active',
        'CPU5V Active',
        'CPU12V',
        'CPU3.3V Standby',
        'CPU5V Standby',
        'IOB 3.3V Standby',
        'IOB +12V',
        'IOB 5V',
        'IOB 3.3V Active',
        'IOB HTLE PCI-EX',
        'IOB VLDT',
        'IOB HTLE Core',
        'CPU0 Core Voltage',
        'CPU1 Core Voltage',
        'CPU0&1 VDD25',
        'CPU0&1 VTT',
        'RLM Standby Voltage',
        'CPU2 Core Voltage',
        'CPU3 Core Voltage',
        'CPU2&3 VDD25',
        'CPU2&3 VTT',
	'HTLE SB1 Core Voltage',
        'HTLE VLDT Voltage',
        'HTLE PCI-Ex Voltage',
        'FC Core Voltage',
        'FC 2.5 Voltage',
        'FC 5 Voltage',
        'FC 12 Voltage',
        'FC 15 Voltage',
        'FC 3.3 Main Voltage',
        'FC 3.3 Standby Voltage',
    ],
    psu => [
        'PSU1 present',
        'PSU1 AC status',
        'PSU 1 12V',
        'PSU 1 5V',
        'PSU 1 AC Voltage',
        'PSU2 present',
        'PSU2 AC status',
        'PSU 2 12V',
        'PSU 2 5V',
        'PSU 2 AC Voltage',
    ],
    psu_temp => [
        'PSU 1 Temperature',
        'PSU 2 Temperature',
    ],
    psu_fan => [
        'PSU 1 FAN 1',
        'PSU 1 FAN 2',
        'PSU 2 FAN 1',
        'PSU 2 FAN 2',
    ],
    temp => [
        'Front Panel Temperature',
        'IO Board Temperature',
        'MB Zone 1 Temperature',
        'MB Zone 2 Temperature',
        'CPU0 Temperature',
        'CPU1 Temperature',
        'CPU2 Temperature',
        'CPU3 Temperature',
    ],
    fan => [
        'Fan FRU 1 present',
        'FAN FRU#1.TopLeft.a',
        'FAN FRU#1.TopLeft.b',
        'Fan FRU 2 present',
        'FAN FRU#2.TopRight.a',
        'FAN FRU#2.TopRight.b',
        'Fan FRU 3 present',
        'FAN FRU#3.BottomLeft.a',
        'FAN FRU#3.BottomLeft.b',
        'Fan FRU 4 present',
        'FAN FRU#4.BottomCenter.a',
        'FAN FRU#4.BottomCenter.b',
        'Fan FRU 5 present',
        'FAN FRU#5.BottomRight.a',
        'FAN FRU#5.BottomRight.b',
    ],
    batt => [
        'RTC Battery',
    ],
    nvram_temp => [
        'NVRAM6-temperature-1',
    ],
    nvram_batt => [
        'NVRAM6-battery-1',
        'NVRAM6-battery2-1',
    ],
    cluster => [
        'CF presence',
    ],
);
my %loc_map_FAS6030 = (
    volt => [
        'CPU3.3V Active',
        'CPU5V Active',
        'CPU12V',
        'CPU3.3V Standby',
        'CPU5V Standby',
        'IOB 3.3V Standby',
        'IOB +12V',
        'IOB 5V',
        'IOB 3.3V Active',
        'IOB HTLE PCI-EX',
        'IOB VLDT',
        'IOB HTLE Core',
        'CPU0 Core Voltage',
        'CPU1 Core Voltage',
        'CPU0&1 VDD25',
        'CPU0&1 VTT',
        'RLM Standby Voltage',
        'HTLE SB1 Core Voltage',
        'HTLE VLDT Voltage',
        'HTLE PCI-Ex Voltage',
        'FC Core Voltage',
    ],
    psu => [
        'PSU1 present',
        'PSU1 AC status',
        'PSU 1 12V',
        'PSU 1 5V',
        'PSU 1 AC Voltage',
        'PSU2 present',
        'PSU2 AC status',
        'PSU 2 12V',
        'PSU 2 5V',
        'PSU 2 AC Voltage',
    ],
    psu_temp => [
        'PSU 1 Temperature',
        'PSU 2 Temperature',
    ],
    psu_fan => [
        'PSU 1 FAN 1',
        'PSU 1 FAN 2',
        'PSU 2 FAN 1',
        'PSU 2 FAN 2',
    ],
    temp => [
        'Front Panel Temperature',
        'IO Board Temperature',
        'MB Zone 1 Temperature',
        'MB Zone 2 Temperature',
        'CPU0 Temperature',
        'CPU1 Temperature',
    ],
    fan => [
        'Fan FRU 1 present',
        'FAN FRU#1.TopLeft.a',
        'FAN FRU#1.TopLeft.b',
        'Fan FRU 2 present',
        'FAN FRU#2.TopRight.a',
        'FAN FRU#2.TopRight.b',
        'Fan FRU 3 present',
        'FAN FRU#3.BottomLeft.a',
        'FAN FRU#3.BottomLeft.b',
        'Fan FRU 4 present',
        'FAN FRU#4.BottomCenter.a',
        'FAN FRU#4.BottomCenter.b',
        'Fan FRU 5 present',
        'FAN FRU#5.BottomRight.a',
        'FAN FRU#5.BottomRight.b',
    ],
    batt => [
        'RTC Battery',
    ],
    nvram_temp => [
        'NVRAM6-temperature-1',
    ],
    nvram_batt => [
        'NVRAM6-battery-1',
    ],
    cluster => [
        'CF presence',
    ],
);
my %loc_map_FAS6040 = (
    volt => [
        'CPU3.3V Active',
        'CPU5V Active',
        'CPU12V',
        'CPU3.3V Standby',
        'CPU5V Standby',
        'IOB 3.3V Standby',
        'IOB +12V',
        'IOB 5V',
        'IOB 3.3V Active',
        'IOB HTLE PCI-EX',
        'IOB VLDT',
        'IOB HTLE Core',
        'CPU0 Core Voltage',
        'CPU1 Core Voltage',
        'CPU0&1 VDD25',
        'CPU0&1 VTT',
        'RLM Standby Voltage',
        'HTLE SB1 Core Voltage',
        'HTLE VLDT Voltage',
        'HTLE PCI-Ex Voltage',
        'FC Core Voltage',
        'FC 2.5 Voltage',
        'FC 5 Voltage',
        'FC 12 Voltage',
        'FC 15 Voltage',
        'FC 3.3 Main Voltage',
        'FC 3.3 Standby Voltage',
    ],
    psu => [
        'PSU1 present',
        'PSU1 AC status',
        'PSU 1 12V',
        'PSU 1 5V',
        'PSU 1 AC Voltage',
        'PSU2 present',
        'PSU2 AC status',
        'PSU 2 12V',
        'PSU 2 5V',
        'PSU 2 AC Voltage',
    ],
    psu_temp => [
        'PSU 1 Temperature',
        'PSU 2 Temperature',
    ],
    psu_fan => [
        'PSU 1 FAN 1',
        'PSU 1 FAN 2',
        'PSU 2 FAN 1',
        'PSU 2 FAN 2',
    ],
    temp => [
        'Front Panel Temperature',
        'IO Board Temperature',
        'MB Zone 1 Temperature',
        'MB Zone 2 Temperature',
        'CPU0 Temperature',
        'CPU1 Temperature',
    ],
    fan => [
        'Fan FRU 1 present',
        'FAN FRU#1.TopLeft.a',
        'FAN FRU#1.TopLeft.b',
        'Fan FRU 2 present',
        'FAN FRU#2.TopRight.a',
        'FAN FRU#2.TopRight.b',
        'Fan FRU 3 present',
        'FAN FRU#3.BottomLeft.a',
        'FAN FRU#3.BottomLeft.b',
        'Fan FRU 4 present',
        'FAN FRU#4.BottomCenter.a',
        'FAN FRU#4.BottomCenter.b',
        'Fan FRU 5 present',
        'FAN FRU#5.BottomRight.a',
        'FAN FRU#5.BottomRight.b',
    ],
    batt => [
        'RTC Battery',
    ],
    nvram_temp => [
        'NVRAM6-temperature-2',
    ],
    nvram_batt => [
        'NVRAM6-battery-2',
    ],
    cluster => [
        'CF presence',
    ],
    unmonitored => [
	'PSU1 hard status',
	'PSU2 hard status',
	'PSU1 soft on',
	'PSU1 DC status',
	'PSU2 soft on',
	'PSU2 DC status',
	'FCAL I/O Board power status',
   ],
);

# Some of these are FRUS and some are informational (includes quantity)
my %fru_map_default = (
    'Mother Board' => 1,
    'IO Board' => 1,
    'Front Panel' => 1,
    'Onboard Fcal' => 1,
    'CPU Board' => 1,
    'PS1' => 1,
    'PS2' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'DIMM3' => 1,
    'DIMM4' => 1,
    'DIMM5' => 1,
    'DIMM6' => 1,
    'DIMM7' => 1,
    'DIMM8' => 1,
    'DIMM9' => 1,
    'DIMM10' => 1,
    'DIMM11' => 1,
    'DIMM12' => 1,
    'DIMM13' => 1,
    'DIMM14' => 1,
    'DIMM15' => 1,
    'DIMM16' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 2,
    'ATA/IDE Adapter' => 1,
    'NVRAM6' => 1,
);
my %fru_map_FAS6030 = (
    'Mother Board' => 1,
    'IO Board' => 1,
    'Front Panel' => 1,
    'Onboard Fcal' => 1,
    'CPU Board' => 1,
    'PS1' => 1,
    'PS2' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'DIMM3' => 1,
    'DIMM4' => 1,
    'DIMM5' => 1,
    'DIMM6' => 1,
    'DIMM7' => 1,
    'DIMM8' => 1,
    'DIMM9' => 1,
    'DIMM10' => 1,
    'DIMM11' => 1,
    'DIMM12' => 1,
    'DIMM13' => 1,
    'DIMM14' => 1,
    'DIMM15' => 1,
    'DIMM16' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 2,
    'ATA/IDE Adapter' => 1,
    'NVRAM6' => 1,
);
my %fru_map_FAS6040 = (
    'Mother Board' => 1,
    'IO Board' => 1,
    'Front Panel' => 1,
    'Onboard Fcal' => 1,
    'CPU Board' => 1,
    'PS1' => 1,
    'PS2' => 1,
    'DIMM1' => 1,
    'DIMM2' => 1,
    'DIMM3' => 1,
    'DIMM4' => 1,
    'DIMM5' => 1,
    'DIMM6' => 1,
    'DIMM7' => 1,
    'DIMM8' => 1,
    'RLM Controller' => 1,
    'SFP Transceiver' => 8,
    'ATA/IDE Adapter' => 1,
    'NVRAM6' => 1,
);

# All the mappings from model to maps
my %class_map = (
    default => \%class_map_default,
    FAS6030 => \%class_map_FAS6030,
    FAS6040 => \%class_map_FAS6040,
);

my %loc_map = (
    default => \%loc_map_default,
    FAS6030 => \%loc_map_FAS6030,
    FAS6040 => \%loc_map_FAS6040,
);

my %fru_map = (
    default => \%fru_map_default,
    FAS6030 => \%fru_map_FAS6030,
    FAS6040 => \%fru_map_FAS6040,
);

#---------------------------------------------------------------------

=head2 new

  $base_obj = Platform::Environment::FAS6000->new(conn  => $conn,
                                                  model => $model);

This is a method in a derived class that acts as a wrapper around a method in
the base class (Base.pm). The SUPER pseudoclass lets you invoke an overridden
base class method without having to specify which class defined that method.
Do not use this constructor directly, use Platform::Environment->new() instead.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub new {
    my $class = shift;
    my (%opts) = validate (
        @_,
        {
            conn  => { type => OBJECT },
            model => { type => SCALAR },
        }
    );
    my $model = $opts{model};

    if (defined($class_map{$model})) {
        $opts{_class_map} = $class_map{$model};
    } else {
        $opts{_class_map} = $class_map{default};
    }
    if (defined($loc_map{$model})) {
        $opts{_loc_map} = $loc_map{$model};
    } else {
        $opts{_loc_map} = $loc_map{default};
    }
    if (defined($fru_map{$model})) {
        $opts{_fru_map} = $fru_map{$model};
    } else {
        $opts{_fru_map} = $fru_map{default};
    }
    return $class->SUPER::new(%opts);
}

1;
