# this is basically -*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS900.pm#1 $
#
## @summary	FAS900.pm - Provides a class that contains platform-specific
##		data to be used by Base.pm
## @author	dl-platform-qa@netapp.com
## @status	public
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod		here

=head1 NAME

FAS900 - Provides a class that contains platform-specific data for the class
Base.pm

=head1 SYNOPSIS

  use Platform::Environment::FAS900;

=head1 DESCRIPTION

Provides a class that contains platform-specific data to be used by Base.pm
which is the base class that allows hardware independency for the Environmental
Subsystem.

 # Example usage (Model Name: FAS960)
 #  Usage: environment status |
 #           [status] [shelf [<adapter>]] |
 #           [status] [shelf_log] |
 #           [status] [shelf_stats] |
 #           [status] [shelf_power_status] |
 #           [status] [chassis [all | list-sensors | Fan | Power | Temp |
 #                              Power Supply | RTC Battery |
 #                              NVRAM4-temperature-7 | NVRAM4-battery-7]]

=head1 STATUS

  Shared

=head1 AUTHOR

  dl-platform-qa@netapp.com
  $Id: //depot/prod/test/Rsierranevada/lib/Platform/Environment/FAS900.pm#1 $
  Last revised $Date: 2013/01/22 $

head1 COPYRIGHT

  Copyright 2008, Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Platform::Environment::FAS900;

use strict;
use warnings;

use Platform::Environment::Base;
use Params::Validate qw(validate OBJECT SCALAR);

use base qw(Platform::Environment::Base);

#---------------------------------------------------------------------
# Local variables
#---------------------------------------------------------------------
my %class_map_default = (
    all           => 'all',
    sensors       => 'list-sensors',
    voltage       => 'Power',
    temp          => 'Temp',
    psu           => '"Power Supply"',
    battery       => '"RTC Battery"',
    nvram_temp    => 'NVRAM4-temperature-7',
    nvram_battery => 'NVRAM4-battery-7',
);
my %loc_map_default = (
    fan => [
        'Hotswap Fan 1',
        'Baseboard Fan 1',
        'Baseboard Fan 2',
        'Hotswap Fan 2',
        'Baseboard Fan 3',
        'Baseboard Fan 4',
        'Hotswap Fan 3',
        'Baseboard Fan 5',
        'Baseboard Fan 6',
    ],
    volt => [
        'CIOB \+2.5V',
        'CMIC \+2.5V',
        'CSB5 \+2.5V',
        'Baseboard 3.3V',
        '3.3 Volt Standby',
        'Baseboard 5V',
        'MB \+5V Standby',
        'Baseboard 12V',
        'Baseboard -12V',
        'MemoryBrd \+2.5V',
        'Processor VCCP',
    ],
    temp => [
        'Baseboard Temp',
        'MemoryBrd Temp',
        'Front Panel Temp',
        'Processor 1 Temp',
        'Processor 2 Temp',
        'PwrDst 1 Temp',
        'PwrDst 2 Temp',
    ],
    psu => [
        'Power Supply 1: Removed',
        'Power Supply 1: Failure',
        'Power Supply 1: AC lost',
        'PS 1 3.3V',
        'PS 1 5V',
        'PS 1 12V',
        'PS 1 AC',
        'PS 1 Crnt 3.3V',
        'PS 1 Crnt 5V',
        'PS 1 Crnt 12V',
        'Power Supply 2: Removed',
        'Power Supply 2: Failure',
        'Power Supply 2: AC lost',
        'PS 2 3.3V',
        'PS 2 5V',
        'PS 2 12V',
        'PS 2 AC',
        'PS 2 Crnt 3.3V',
        'PS 2 Crnt 5V',
        'PS 2 Crnt 12V',
    ],
    psu_fan => [
        'PS Fan 1',
        'PS Fan 2',
        'PS Fan 3',
        'PS Fan 4',
    ],
    batt => [
        'RTC Battery',
    ],
    nvram_temp => [
        'NVRAM4-temperature-7',
    ],
    nvram_batt => [
        'NVRAM4-battery-7',
    ],

);

# Some of these are FRUS and some are informational (includes quantity)
my %fru_map_default = (
    'LED Board' => 1,
    'Memory Board' => 1,
    'Chassis Power Supply' => 2,
    'Memory DIMM' => 8,
    'Baseboard Management Controller' => 1,
    'ATA/IDE Adapter' => 2,
    'Dual-channel, QLogic 2312' => 1,
    'Mellanox IB Cluster Interconnect Adapter' => 1,
    'NVRAM4' => 1,
);

# All the mappings from model to maps
my %class_map = (
    default => \%class_map_default,
);

my %loc_map = (
    default => \%loc_map_default,
);

my %fru_map = (
    default => \%fru_map_default,
);

#---------------------------------------------------------------------

=head2 new

  $base_obj = Platform::Environment::FAS900->new(conn  => $conn,
                                                 model => $model);

This is a method in a derived class that acts as a wrapper around a method in
the base class (Base.pm). The SUPER pseudoclass lets you invoke an overridden
base class method without having to specify which class defined that method.
Do not use this constructor directly, use Platform::Environment->new() instead.

=for html <hr />

=cut

#---------------------------------------------------------------------
sub new {
    my $class = shift;
    my (%opts) = validate (
        @_,
        {
            conn  => { type => OBJECT },
            model => { type => SCALAR },
        }
    );
    my $model = $opts{model};

    if (defined($class_map{$model})) {
        $opts{_class_map} = $class_map{$model};
    } else {
        $opts{_class_map} = $class_map{default};
    }
    if (defined($loc_map{$model})) {
        $opts{_loc_map} = $loc_map{$model};
    } else {
        $opts{_loc_map} = $loc_map{default};
    }
    if (defined($fru_map{$model})) {
        $opts{_fru_map} = $fru_map{$model};
    } else {
        $opts{_fru_map} = $fru_map{default};
    }
    return $class->SUPER::new(%opts);
}

1;
