# -*- perl -*-
#/* $Id: //depot/prod/test/main/lib/Platform/internal_switch/SwitchUtils.pm
#
#
# Copyright (c) 2013 NetApp, Inc.
# All rights reserved.
#
## @summary
## SwitchUtils.pm is a common module for all the  scripts under
##             platqa/motherboard/ports/internal_switch
## @description
## This module privodes common functions for use in testing
##        scripts under platqa/motherboard/ports/internal_switch
##
## Sub-routines:
##    Get_Sysconfig_Port_Link_Status
##    Get_Switch_Port_Link_Status_7m
##    Get_Switch_by_Port_id_7m
##    Get_Switch_Port_Link_Status_Cm
##    Get_Switch_Id_And_Port_Id_Cm
##
## @status private
##
## @author dhana@netapp.com, dl-platform-qa@netapp.com
##
###############################################################################

package Platform::internal_switch::SwitchUtils;

use strict;
use warnings;
use Data::Dumper;
use Params::Validate qw(:all);

#
# Import exceptions
#

require Exporter;
our @ISA = qw(Exporter);

our @EXPORT_OK = qw(
    Get_Sysconfig_Port_Link_Status
    Get_Switch_Port_Link_Status_7m
    Get_Switch_by_Port_id_7m
    Get_Switch_Port_Link_Status_Cm
    Get_Switch_Id_And_Port_Id_Cm
);

our $VERSION = '1.0';

#---------------------------------------------------------------------------
# FUNCTION: This function returns the port link status from sysconfig
#                                
# INPUT: Filer connection object, Port name
#
# RETURN: Port link status for Port name
#---------------------------------------------------------------------------
sub Get_Sysconfig_Port_Link_Status {

    my %args = validate ( @_, {
       apiset  => { type => OBJECT },
       port    => { type => SCALAR }
    } );
   
    my $Api_Set_Obj = $args{'apiset'};
    my $Port = $args{'port'};

    my ($sys_raw, @sys_parse, $port_status);
    my $result = 'unknown';

    $sys_raw = $Api_Set_Obj->sysconfig("detailed-information" => "-a");
    @sys_parse = @{ $sys_raw->get_parsed_output() };

    foreach my $slot_info (@{ $sys_parse[0]->{'slot_information'} }) {
       
    if($Port =~ /wrench/i) {
        if(defined($slot_info->{"$Port"})) {
        $port_status = $slot_info->{"$Port"};
        }
    } else {     
        if(defined($slot_info->{'ports'})) {
            foreach my $found_port (@{ $slot_info->{'ports'} }) {
                if($found_port->{'port_number'} eq $Port) {
                $port_status = $found_port->{'status'};
                last;
                }
            } #End of foreach found_port

        } #End of if condition
    } #End of else
    } #End of foreach slot_info
    if(defined($port_status)) {
        if($port_status =~ /down/i) {
        $result = 'down';
        } elsif($port_status =~ /up/i) {
        $result = 'up';
        }
    }

    return $result;
}    ##end of sub Get_Sysconfig_Port_Link_Status
#---------------------------------------------------------------------------
# FUNCTION: This function returns the port link status from switch 
#           statistics of 7Mode cli, ifswitch
#                                
# INPUT: Filer connection object, Port name
#
# RETURN: Port link status for Port name
#---------------------------------------------------------------------------
sub Get_Switch_Port_Link_Status_7m {

    my %args = validate( @_, {
       apiset      => { type => OBJECT },
       switch_port => { type => SCALAR }
    } );

    my $Api_Set_Obj = $args{'apiset'};
    my $sw_port = $args{'switch_port'};

    my ($switch_read_raw,  @switch_read_parse, $link_status );
    my $result = 'unknown';

    $switch_read_raw = $Api_Set_Obj->ifswitch("all" => 1);
    @switch_read_parse = @{ $switch_read_raw->get_parsed_output() };

    foreach my $switch_info (@{ $switch_read_raw->get_parsed_output() }) {
        if($switch_info->{'name'} eq $sw_port) {
        $link_status = $switch_info->{'link'};
        } #End of if condition
    } #End of foreach

    if(defined($link_status)) {
    $result = $link_status;
    }

    return $result;
}  #end of Get_Switch_Port_Link_Status_7m
#---------------------------------------------------------------------------
# FUNCTION: This function returns the port link status from switch 
#           statistics of CMode cli, system node internal-switch show
#                                
# INPUT: Filer connection object, Node name, Port name
#
# RETURN: Port link status for Port name
#---------------------------------------------------------------------------
sub Get_Switch_Port_Link_Status_Cm {

    my %args = validate( @_, {
       apiset      => { type => OBJECT },
       node_name   => { type => SCALAR },
       switch_port => { type => SCALAR }
    } );

    my $Api_Set_Obj = $args{'apiset'};
    my $Node_Name = $args{'node_name'};
    my $sw_port = $args{'switch_port'};

    my ($switch_read_raw,  @switch_read_parse, $link_status );
    my $result = 'unknown';

    $switch_read_raw = $Api_Set_Obj->system_node_internal_switch_show(
                            'node' => $Node_Name,
                            'port-name' => $sw_port,
                            );
    @switch_read_parse = @{ $switch_read_raw->get_parsed_output() };
    $link_status = $switch_read_parse[0]->{'link'};

    if(defined($link_status)) {
        if($link_status =~ /up/i) {
        $result = 'up';
        } elsif($link_status =~ /down/i) {
        $result = 'down';
        }
    } 

    return $result;
}  #end of Get_Switch_Port_Link_Status_7m
#---------------------------------------------------------------------------
# FUNCTION: This function returns the switch-id/port-id for requested port
#           name from ifswitch -a 7mode cli output. 
#                                
# INPUT: Filer connection object, Port name [ex: sw-wrench for wrench port]
#
# RETURN: switch-id/port-id number
#---------------------------------------------------------------------------
sub Get_Switch_by_Port_id_7m {

    my %args = validate( @_, {
       apiset      => { type => OBJECT },
       switch_port => { type => SCALAR }
    } );

    my $Api_Set_Obj = $args{'apiset'};
    my $sw_port = $args{'switch_port'};

    my ($switch_read_raw,  @switch_read_parse, $port_num );
    my $result = 'unknown';

    $switch_read_raw = $Api_Set_Obj->ifswitch("all" => 1);
    @switch_read_parse = @{ $switch_read_raw->get_parsed_output() };

    foreach my $switch_info (@{ $switch_read_raw->get_parsed_output() }) {
        if($switch_info->{'name'} eq $sw_port) {
        $port_num = $switch_info->{'port'};
        } #End of if condition
    } #End of foreach

    if(defined($port_num)) {
    $result = $port_num;
    }

    return $result;
}  #end of Get_Switch_by_Port_id_7m
#---------------------------------------------------------------------------
# FUNCTION: This function returns the switch-id and port-id for requested
#           port name from system node internal-switch Cmode cli output. 
#                                
# INPUT: Filer connection object, Port name [ex: sw-wrench for wrench port]
#
# RETURN: switch-id and port-id number
#---------------------------------------------------------------------------
sub Get_Switch_Id_And_Port_Id_Cm {

    my %args = validate( @_, {
       apiset      => { type => OBJECT },
       node_name   => { type => SCALAR },
       switch_port => { type => SCALAR }
    } );

    my $Api_Set_Obj = $args{'apiset'};
    my $Node_Name = $args{'node_name'};
    my $sw_port = $args{'switch_port'};

    my ($switch_read_raw,  @switch_read_parse, $switch_id, $port_id );
    my $result_switch_id = 'unknown';
    my $result_port_id = 'unknown';

    $switch_read_raw = $Api_Set_Obj->system_node_internal_switch_show(
                                    'node' => $Node_Name,
                                    'port-name' => $sw_port,
                                    );
    @switch_read_parse = @{ $switch_read_raw->get_parsed_output() };

    $switch_id = $switch_read_parse[0]->{'switch-id'};
    $port_id = $switch_read_parse[0]->{'port-id'};

    if(defined($switch_id)) {
    $result_switch_id = $switch_id;
    }
    if(defined($port_id)) {
    $result_port_id = $port_id;
    }

    return ($result_switch_id, $result_port_id);
}  #end of Get_Switch_Id_And_Port_Id_Cm
#---------------------------------------------------------------------------
1;
