package dp::mcc::Pluggable;

use strict;
use warnings;
use TCD;
use Carp;
use Tharn;
use Subtest;
use Data::Dumper;
use Time::HiRes qw(usleep nanosleep);

### NACL Component libs ###
use NACL::C::Node;
use NATE::Process;

sub new {
    my $class = shift;
    my %self = @_;

    #verify all descriptions were defined, verify all values are strings and defined

    #if no plugins are being run
    if(!defined $self{'plugins'} || $self{'plugins'} eq "" || $self{'plugins'} eq " "){
        $self{'plugins'} = undef;
    } else {
        my @plugins = split /,/, $self{'plugins'};
        $self{'plugins'} = \@plugins;
    }

    #marshall arguments
    $self{'plugin_args'}{'JOBID'}{value} = $ENV{'JOBID'};
    $self{'plugin_args'}{'JOBID'}{description} = "Parent job's id";
    $self{'plugin_args'}{'NODES'}{value} = $self{'nodes'};
    $self{'plugin_args'}{'NODES'}{description} = "List of Node names";


    #make list of params passed
    my $args_keys = "";
    foreach my $key ( keys %{ $self{'plugin_args'} } ) {
        $args_keys .= $key.",";
    }
    chop($args_keys);
    $self{'plugin_args'}{'ARGS'}{value} = $args_keys;
    $self{'plugin_args'}{'ARGS'}{description} = "Keys of Arguments passed to Child";

    print Dumper(\%self);

    my $args_str = "";
    foreach my $key( keys %{ $self{'plugin_args'} } ) {
        $args_str.= $key."=>'".$self{'plugin_args'}->{$key}->{'value'}."',";
    }
    chop($args_str);

    #start child processes
    my @process_handles = ();
    foreach my $plugin ( @{ $self{'plugins'} } ) {
        my $plugin_name;
        if ( $plugin =~ /(\w+[.]thpl)/ ) {
                    $plugin_name = $1;
                }

        eval 'push @process_handles, NATE::Process->new(
                     runid => "'.$plugin_name.'",
                  codespec => "'.$plugin.'",
                    params => { '.$args_str.' },
                    onexit => \&NATE::Process::on_exit_propagate_worst_result,
                  teststep => "'.$plugin_name.'" );';
        die "Error: $@\n" if ($@);
        #eval "push \@subtest_handles, subtest( '$plugin', ".$args_str."'-bg','-runid', '$plugin_name' );";
    }

    foreach my $proc (@process_handles){
        $proc->start();
        Tharn::snooze(5);
    }

    $self{'process_handles'} = \@process_handles;
    bless \%self,$class;
    return \%self;
}

sub run_socket {
    my ( $self, $socket_name) = @_;
    print "Socketname: ".$socket_name;

    if(!defined $self->{'plugins'}){
        return;
    }

    if($self->{'plugin_execution'} eq 'serial') {
        foreach my $proc (@{$self->{'process_handles'}}){
            if($proc->is_running()){
            $proc->message_put(message => $socket_name, type => "run_socket");

            }
            if($proc->is_running()){
            eval {
                $proc->message_get(type => "plugin_complete");
            };
                if($@ and $@ !~ /dead peer/){
                    die "Unknown error when trying message_get: ".$@;
                }
            }
        }
    } elsif($self->{'plugin_execution'} eq 'parallel') {
        foreach my $proc (@{$self->{'process_handles'}}){
            if($proc->is_running()){
            $proc->message_put(message => $socket_name, type => "run_socket");
            }
        }

        foreach my $proc (@{$self->{'process_handles'}}){
            if($proc->is_running()) {
            eval {
                $proc->message_get(type => "plugin_complete");
            };
                if($@ and $@ !~ /dead peer/){
                    die "Unknown error when trying message_get: ".$@;
                }
            }
        }
    }
}

sub cleanup {
    my ( $self, $socket_name) = @_;
    if(!defined $self->{'plugins'}){
        return;
    }

    # call cleanup of every script
    foreach my $proc (@{$self->{'process_handles'}}){
        if($proc->is_running()){
        $proc->message_put(message => "cleanup", type => "run_socket");
        }
        if($proc->is_running()){
            eval {
            $proc->message_get(type => "plugin_complete");
        };
        if($@ and $@ !~ /dead peer/){
                die "Unknown error when trying message_get: ".$@;
        }
        }
        $proc->wait();
    }
}

1;
