use strict;
use warnings;
use TCD;
use Carp;
use Tharn;
use Subtest;
use Data::Dumper;
use Time::HiRes qw(usleep nanosleep);


### NACL Component libs ###
use NACL::C::Node;
use NATE::ParamSet;
use NATE::STAF::Peer qw($Parent);
use Connectrec qw();
use Hostrec qw();

sub plugin_get_args {
    #go through all the arguments in the script
    #return them as a hash
    my @args = split /,/, param('ARGS', '-mesg', 'ARGS param not defined');
    my %param_map;
    foreach my $arg ( @args ){
        $param_map{$arg} = param($arg, '-mesg', $arg.' param not defined');
    }

    my @nodes = split /,/, $param_map{'NODES'};

    #change the node list string to an array of node names
    $param_map{'NODES'} = \@nodes;

    my @alive_nodes = ();
    foreach my $node ( @nodes ){
        eval {
            my $conn = Connectrec->new(host=> $node, template => "ssh", connect  => 0);
            $conn->connect(timeout=>5);
            push @alive_nodes,$node;
        };
    }

    #my @testnodes = NACL::C::Node->find();#
    #foreach my $testnode ( @testnodes ){
    #   $param_map{'NODE_TEST'}{$testnode->name} = $testnode;
    #}

    #add to param_map node objects and apiset objects
    foreach my $node ( @alive_nodes ){
        $param_map{'NODE_OBJ'}{$node} = NACL::C::Node->new(node=>$node);
        $param_map{'APISET_CMODE'}{$node} = NACL::APISet->new( name => $node,
                                                               category => 'Node',
                                                               interface => 'CLI',
                                                                   set => 'CMode' );
                $param_map{'APISET_SYSTEMSHELL'}{$node} = NACL::APISet->new( name => $node,
                                                               category => 'Node',
                                                               interface => 'CLI',
                                                                   set => 'Systemshell' );
                $param_map{'APISET_7MODE'}{$node} = NACL::APISet->new( name => $node,
                                                               category => 'Node',
                                                               interface => 'CLI',
                                                                   set => 'Nodescope' );
    }
    #$Test->nlog(Dumper(\%param_map));

    return \%param_map;
}

sub plugin_run {
    my $message = "";
    my $exit = 0;
    my $failed_already = 0;
    my $entry;
    my $result;
    do {
        my $message = $Parent->message_get(type => "run_socket", timeout=>1800,
                                    on_timeout=> sub{ eval "cleanup();";
                                              $Test->nlog("Error: $@\n") if ($@); die;
                                            });

        if($message eq 'cleanup'){
            $exit = 1;
            eval "cleanup();";
            $Test->nlog("Error: $@\n") if ($@);
        }
        elsif(!$failed_already){
            $result = eval $message."();";
            if ($@ or ($result != $TCD::PASS)){
                $Test->nlog("Error: Plugin Failed during Execution of socket ".$message.". $@\n");
                $failed_already = 1;
            }
        }
        $entry = $Parent->message_put(type => "plugin_complete", message => "complete");
    } while(!$exit);
}

sub plugin_refresh_connections {
    my $argsref = shift;
    my %args = %$argsref;

    my @alive_nodes = ();
    foreach my $node ( @{$args{'NODES'}} ){
        eval {
            my $conn = Connectrec->new(host=> $node, template => "ssh", connect  => 0);
            $conn->connect(timeout=>5);
            push @alive_nodes,$node;
        };
    }

    $Test->nlog(Dumper(@alive_nodes));

    #add to param_map node objects and apiset objects
    foreach my $node ( @alive_nodes ){
        $Test->nlog("Refreshing connections for ".$node);
        $args{'NODE_OBJ'}{$node} = NACL::C::Node->new(name=>$node);
        $args{'APISET_CMODE'}{$node} = NACL::APISet->new( name => $node,
                                                               category => 'Node',
                                                               interface => 'CLI',
                                                                   set => 'CMode' );
                $args{'APISET_SYSTEMSHELL'}{$node} = NACL::APISet->new( name => $node,
                                                               category => 'Node',
                                                               interface => 'CLI',
                                                                   set => 'Systemshell' );
                $args{'APISET_7MODE'}{$node} = NACL::APISet->new( name => $node,
                                                               category => 'Node',
                                                               interface => 'CLI',
                                                                   set => 'Nodescope' );
    }
    return \%args;
}

1;
