#
# $Id: //depot/prod/test/Rsierranevada/lib/PostProc/Log.pm#1 $
#

use strict;
use warnings;

package PostProc::Log;

#
# this should go away.
#

use TracePkg;
use Carp qw/confess/;
use Data::Dumper;
use IO::File;
use PostProc::LogMessage;

#
# class data & methods
#

my $Trace = new TracePkg();
$Trace->set_options(separateNewline => 1);

our @tossable_str = ();

sub Factory
{
    my %p = @_;
    if ($p{logfile} =~ m/mgwd/) {
        use PostProc::Log::Mgwd;
        $Trace->comment(msg => "Creating new PostProc::Log::Mgwd for $p{logfile}");
        return new PostProc::Log::Mgwd(@_);
    } elsif ($p{logfile} =~ m/vldb/) {
        use PostProc::Log::Vldb;
        $Trace->comment(msg => "Creating new PostProc::Log::Vldb for $p{logfile}");
        return new PostProc::Log::Vldb(@_);
    } elsif ($p{logfile} =~ m/ems/) {
	use PostProc::Log::Ems;
	$Trace->comment(msg => "Creating new PostProc::Log::Ems for $p{logfile}");
	return new PostProc::Log::Ems(@_);
    } elsif ($p{logfile} =~ m/vifmgr/) {
        use PostProc::Log::VifMgr;
        $Trace->comment(msg => "Creating new PostProc::Log::vifMgr for $p{logfile}");
        return new PostProc::Log::VifMgr(@_);
    } elsif ($p{logfile} =~ m/notifyd/) {
        use PostProc::Log::Notifyd;
        $Trace->comment(msg => "Creating new PostProc::Log::Notifyd for $p{logfile}");
        return new PostProc::Log::Notifyd(@_);
    } else {
        $Trace->warn(msg => "Creating new PostProc::Log for $p{logfile}");
        return new PostProc::Log(@_);
    }
}


sub init_tossable_re
{
    $Trace->entry();
    push(@tossable_str, 
        qw!
        ^\p{IsXDigit}{8}\.\p{IsXDigit}{8}$
        ^(0x|0X)?\p{IsXDigit}+(=|:)?$
        ^(Mon|Tue|Wed|Thu|Fri|Sat|Sun)$
        ^\[?(Jan|Feb|Mar|Apr|May|Jun|Jul|Aug|Sep|Oct|Nov|Dec)\]?$
        ^\[?\d+\]?$

        ^\d{2}:\d{2}:\d{2}$

        ^0x\p{IsXDigit}+$
        ^-?\d$
        ^(\+|\-)\d{2}:\d{2}$
        ^::$
        ^(\.\d+)+$

        ^\p{IsXDigit}{8}-\p{IsXDigit}{4}-\p{IsXDigit}{4}-\p{IsXDigit}{4}-\p{IsXDigit}{12}$
        ^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}$
       
        ^([qmnd](deux|exl)-h?\d+|\w+-vsim\d+)$

        ^\[id$
        ^\d+(\]|,|\))?\)?,?$
        ^\<\d+\,\d+\>$
        !
    );

    $Trace->exit(msg => join(" ", @tossable_str));
    return [map( { qr/$_/; } @tossable_str)];
}

sub init_substitutions
{
    my @subst;

    #push(@subst,
    #    qw!
    #    \d+
    #    \p{IsXDigit}+
    #    !
    #);

    return [@subst];
}

#sub filter_unique
#{
#    my $self = shift();
#    my $inp = shift();
#    my %unique;

#    foreach my $in (@{$inp}) {
#        $unique{$in}++;
#    }

#    return [keys(%unique)];
#}

#
# Methods 
#

sub new ()
{
    $Trace->entry();
    my $class = shift();
    my %p = @_;

    my $self = ();

    if (! -e $p{logfile}) {
        confess("$p{logfile} doesn't exist.");
    } else {
        $Trace->comment(msg => "Opening log file $p{logfile}");
    }

    $self->{logfile} = $p{logfile};
    $self->{file} = new IO::File($p{logfile}, O_RDONLY) || confess($!);

    @{$self->{lines}} = ();
    $self->{nondescript_lines} = 0;
    $self->{processed} = 0;

    $self->{tossable} = init_tossable_re();

    bless($self, $class);

    $Trace->exit();
    return $self;
}


sub process 
{
    my $self = shift();

    my $l = 0;

    if ($self->{processed}) {
        return;
    }

    while (defined($_ = $self->{file}->getline())) {
        push(@{$self->{lines}}, $_);
        push(@{$self->{msgs}}, 
            new PostProc::LogMessage(
                line => $_, 
                num => ++$l,
                'toss-regexes' => $self->{tossable},
                'substitution-regexes' => undef()
            )
        );

        $self->{msgs}->[$#{$self->{msgs}}]->toss_tokens();
        $self->{msgs}->[$#{$self->{msgs}}]->hash_entry();

        if ($self->{msgs}->[$#{$self->{msgs}}]->is_nondescript()) {
            $self->{nondescript_lines}++;
        }
    }

    $self->{log_lines} = $l;
    $self->{log_bytes} = $self->{file}->tell();
    $self->{processed} = 1;

    $Trace->comment(msg => "Read in $l lines/$self->{log_bytes} bytes");
}

sub log_size
{
    my $self = shift();
    return (wantarray() ? 
        ($self->{log_lines}, $self->{log_bytes}, $self->{nondescript_lines}) :
        $self->{log_bytes});
}

sub get_filename
{
    my $self = shift();
    return $self->{logfile};
}

sub DESTROY
{
    my $self = shift();
    undef($self->{file});
    $Trace->comment(msg => "Closing $self->{logfile}");
}

1;
