use strict;
use warnings;
use Data::Dumper;
use PostProc::EmsLogMessage;

package PostProc::Log::Ems;

my $Trace = new TracePkg();
$Trace->set_options(separateNewline => 1);

use base qw/PostProc::Log/;
#use Fcntl qw(:seek);

sub new 
{
    $Trace->entry();
    my $self = PostProc::Log::new(@_);
 
    $Trace->exit();
    return $self;
}

sub process
{
    my $self = shift();

    my $l = 0;

    if ($self->{processed}) {   #skip processing if the file is already processed
	return;
    }

    open(FH, $self->{logfile});		# Open the file
    my @lines = <FH>;		        # Read it into an array

    my $count = 0;
    my @occur;
    foreach my $line (@lines) {  
	++$count;
	if ($line =~ /<LR/) {       # Extract the line number with "<LR" as text in it 
	    push(@occur, $count);
	}
    }

    my $TotalLines = @lines;  #Get the total number of lines in the file
   
    seek(FH, 0, 0);  # Go to the very beginning of the file
    open (FH,$self->{logfile});
    local $/ = "</LR>";

    while (<FH>) {
	
	push(@{$self->{lines}}, $_);
	push(@{$self->{msgs}}, 
	     new PostProc::EmsLogMessage(line => $_, 
				      num => shift(@occur),
				      'toss-regexes' => $self->{tossable},
				      'substitution-regexes' => undef()));

	#$self->{msgs}->[$#{$self->{msgs}}]->toss_tokens();
	$self->{msgs}->[$#{$self->{msgs}}]->hash_entry();

	if ($self->{msgs}->[$#{$self->{msgs}}]->is_nondescript()) {
	    $self->{nondescript_lines}++;
	}  
    }

    $self->{log_lines} = $TotalLines;
    $self->{log_bytes} = tell(FH);
    $self->{processed} = 1;

    $Trace->comment(msg => "Read in $TotalLines lines/$self->{log_bytes} bytes");

    close(FH);
}

1;
