#
# NetPort subclass to handle powering on/off specific network ports.
#

package Power::NetPort;

use lib "/x/eng/systest/perl";

use Power;

#
# This class inherits all the methods other than the constructor and a tracing method from its superclass.
#
@ISA = qw(Power);

sub new 
{
    my $this = shift;
    my $class = ref($this) || $this;

    $self = {
        NetPort => undef(),
        Port => undef(),
        Server => undef(),
        debug => 0,
        trace_on => 0,
        @_,
    };

    bless($self, $class);

    ($self->{NetPortServerName}, $self->{NetPortName}) = split(/:/, $self->{NetPort});

    $self->_trace("Server: ".$self->{NetPortServerName}."\n");
    $self->_trace("Port ".$self->{NetPortName}."\n");

    $self->_trace("DATA: ".$RconsPorts::machPort{$self->{NetPortServerName}}."\n");
    $self->_trace("DKEYS: ".join(", ",keys(%{$RconsPorts::machPort{$self->{NetPortServerName}}}))."\n");

    #
    # Poke the correct values into the object's data structures so that calls to inherited methods will get the correct values.
    #
    if (defined($RconsPorts::machPort{$self->{NetPortServerName}}->{NetPorts}->{$self->{NetPortName}})) {
        $self->_trace("Found Network port power information: ".$RconsPorts::machPort{$self->{NetPortServerName}}->{NetPorts}->{$self->{NetPortName}}."\n");
        $self->{Server} = $RconsPorts::machPort{$self->{NetPortServerName}}->{NetPorts}->{$self->{NetPortName}}[0];
        $self->{Port} = $RconsPorts::machPort{$self->{NetPortServerName}}->{NetPorts}->{$self->{NetPortName}}[1];
    } else {
        return undef();
    }

    $self->_trace("PowerServer: ".$self->{Server}."\n");
    $self->_trace("PowerPort: ".$self->{Port}."\n");

    return $self;
}

sub _trace
{
    my $self = shift();
    if ($self->{trace_on}) {
        print shift();
    }
}

1;
