#***********************************************************************
#
# NAME
#   RDB.pm - 
#
# SYNOPSIS
#   RDB Library for the invocation and verfication of RR and higher Replicated Database Features.
#
# DESCRIPTION
#   This library is a wrapper for the RDB Subroutines that an end user
#   may desire for the RDB interaction.  The current library was designed to use CM SN node.
#
# AUTHOR
#   dl-cluster-qa
#
# HISTORY
# 11/29/11 Initial revision
#***********************************************************************

## @summary     MOUNT_SVC library    

package RDB;

#use statements
use strict;
use Data::Dumper;
use Server;
use Tharn qw($Log snooze host connect script_result fatal_result);
use Options;
use Params::Common;
use Params::Validate qw(:all);

use Carp;
use Sys::Hostname;

use File::Path;
use File::Basename;
use NACL::APISet;
use NACL::C::Node;
use NACL::C::Cluster;
use NACL::C::CommandInterface;
use NACL::C::StorageFailover;
use NACL::STask::Aggregate;
use NACL::STask::Volume;
use NACL::STask::Lun;
use NACL::CS::ClusterRing;
use NACL::CS::ClusterHa;
use NACL::C::ClusterHa;
use NACL::C::SystemConfigurationRecoveryCluster;
use NACL::C::SystemConfigurationBackup;
use NACL::Transit::Exceptions::TransitException qw(:try);
use NACL::STask::Spm;
use NACL::APISet::Exceptions::MgwdCrashedException;
use Scalar::Util qw(blessed);
use Clone;


#Global variables
my @Nodes = NACL::C::Node->find();
my $Node=$Nodes[0];
my $RDB_PATH="/mroot/etc/cluster_config/rdb";
my $RDB_BACKUP_PATH="/mroot/etc/cluster_config/rdb/corruption";
#my $RDB_BACKUP_PATH="/mroot/etc/backups/corruption";
my $NVRAM_CORRUPT_FILE="/mroot/etc/cluster_config/monitor_mroot.nvfail";
my $SITELIST="/var/rdb/_sitelist";
my $RDB_VOLUME_FILE="VLDB/vvolTable";
my $LOW_SITELIST_ID="1000";
#get the vserver uuid from the name and use that for the grep(upper case, no dashes)
my $RDB_VSERVER_FILE_RR="Management/quota_policy_table";
my $RDB_VSERVER_FILE_SN="Management/quota_policy_table";
my $RDB_LUN_IGROUP_FILE="Bcom/igroupConfig";
my $RDB_VIF_FILE="VifMgr/vif_table";
my $RDB_BOOT_ARG="bootarg.rdb_corrupt";
my $RDB_DISABLE_ARG="bootarg.rdb_autorecovery_enabled";
my $opts = new Options;
my $RING_COUNT=4;
my $BACKUP_COUNT=4;
my $RDB_TEST_CORRUPT_CMD="debug cluster rdb corrupt";
#my $RDB_TEST_CORRUPT_CMD="debug corrupt rdb replicas";
my @RDB_EMS_MESSAGES=("rdb.env.replicaCorrupt","rdb.corruption.reported","rdb.recovery.started","rdb.recovery.succeeded","rdb.recovery.failed","rdb.env.IOFailure","rdb.internal.error");
my $VM=NACL::APISet::Version->new(hostobj    => host($Nodes[0]->name()),
                                                 category       => "Node",
                                                 shell_type     => "CMode",
                                                 );
my $PROMOTED_BUILD=$VM->is_promoted();
#my $AUTO_RECOVERY=_check_for_autorecovery(node=>$Nodes[0]);
my $opt_href = $opts->get_hash();


my $enable_regex = qr/1|on|ON|yes|YES|true|TRUE|0|off|OFF|no|NO|false|FALSE|enable|disable/;
my $id_regex = qr/hostname|systemid|true|false/i;

#command may come in handy...Clear cluster uuid mailboxes for each pair of nodes in the cluster (debug smdb table clear_uuid).
#bryan walsh: maybe a useful cmd for rdb uuid mailbox info

############################################################################
# Subroutine: join_unjoin_cluster
# Parameters: None
# Returns: 0 if the node is not joined/unjoined, 1 if the cluster is joined/unjoined
# Purpose: Join or Unjoin a node from a cluster
#############################################################################
#unjoin - all i have to do is delete * on the vserver & network interface w/ name node name, then disable SFO=
#Vols  Remove or move  
#ii.	Lifs  Remove or move
#iii.	SFO/HA configuration - Disable
#iv.    wipconfig
#v.     24/6
sub join_unjoin_cluster {
    my %params = (
            exec_node=>undef,
            node=>undef,
            action=>undef,
            @_
    );
    if ( !defined($params{exec_node}) && !defined($params{node}) && !defined($params{action})){
      $Log->error("No execution node, node and action specifed, exiting...");
      return 0;
    }  
    my $result=0;
    try {
        #will have to find home node the ha pair
        #unjoin that node OR turn off sfo unjoin both nodes
        #
        if ( $params{action} eq "unjoin" ){
            NACL::C::Cluster->unjoin( command_interface => $params{exec_node},
                'node'    => $params{node}->name(),
                #'clustername' => get_cluster_name(node=>$params{node}),
            );
        }elsif ( $params{action} eq "unjoin" ){
            NACL::C::Cluster->join( command_interface => $params{exec_node},
                'clusteripaddr'    => get_cluster_ip(node=>$params{node}),
            );
        }          
        $result=1;
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->error( "Error in function join_unjoin_cluster on component(s) NACL::C::Cluster->unjoin/NACL::C::Cluster->join is: ".$exception_object->text() );
        $result = 0;
    };
    
    return $result;
}


###########################################################################
# Subroutine: get_RDB_master_node()
# Parameters: None
# Returns: RDB Master Node Name
# Purpose: Find the RDB Master Node
###########################################################################
sub get_RDB_master_node
{
    my %params = (
            master=>"true",
            unitname=>undef,
            @_
    );
    my $exec_node;
	my $return_node="";
    my $found=0;
    my @vals;
    try{
        @vals   = NACL::CS::ClusterRing->fetch(
            command_interface => $Nodes[0],
        );          
    }catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function get_RDB_master_node on component NACL::CS::ClusterRing->fetch, attempting to reconnect APISET session ".$exception_object->text() );
        my $Api_Set_ObjC = $Nodes[0]->apiset(
                            'interface' => 'CLI',
                          );            
        $Api_Set_ObjC->api_reconnect_session();
        @vals = NACL::CS::ClusterRing->fetch(command_interface => $Nodes[0]);
    };

	foreach my $val (@vals){
            if ( $found ){
                last;
            }
            next if ( $val->online eq "offline" );
            next if ( $params{master} eq "false" && $val->online ne "secondary");
            $return_node=$val->master;
            $Log->debug("Master value from cluster ring show:  $return_node");
            my $index=0;
            foreach my $node (@Nodes){
                $Log->debug("Checking node for ".$node->name()." master node");
                if ($node->name() =~ /$return_node/ ){
                    $Log->debug("Found master node, ".$return_node);
                    if ( $params{master} eq "true" ){
                        $return_node=$node;
                        $found=1;
                        last;
                    }else{
                        if ( $index == 0 ){
                            $index=$index+1;
                        }
                        else{
                            $index=$index-1;
                        }
                        $Log->debug("Secondary node is at index $index");
                        $return_node=$Nodes[$index];
                        $Log->debug("Secondary node is $return_node");
                    }
                    $found=1;
                    last;
                }
                    $index++;
            }
	}
        if ( $return_node ne "" ){
            if ( $params{master} eq "true"){
                $Log->comment("Return rdb master node: ".$return_node->name());
            }else{
                $Log->comment("Return rdb secondary node: ".$return_node->name());
            }
            
        }
        return $return_node;
}


###########################################################################
# Subroutine: check_rdb_ring()
# Parameters: RDB Ring Name
# Returns: 0 if the ring value do not match on all nodes, 1 if does
# Purpose: Find the RDB Master Node
###########################################################################
sub check_rdb_ring
{
    my %params = (
            node=>undef,
            ring=>undef,
            type=>undef,
            @_
    );
    my @vals;
    if ( !defined($params{node}) && !defined($params{ring}) && !defined($params{type})){
        $Log->comment("Node, ring, and type not specified, exiting");
        return 0;
    }       
    try{
        @vals   = NACL::CS::ClusterRing->fetch(
            command_interface => $params{node},
        );
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->debug("Error in function check_rdb_ring on component NACL::C::Cluster->fetch is ".$exception_object->text() );
    };
    _get_rdb_ring_count(node=>$params{node});
    my $matched=1;
    my $size=scalar(@vals) - 1;
    my $type=$params{type};
    $Log->debug("Size of Cluster Ring show state arrary: $size");
    #may to validate every node against each other, right now just validating the first node against the rest
    #foreach cluster ring item, loop through the nodes array, if the node matches and type matches
    foreach my $cluster_ring (@vals){
        if (!$matched){
            last;
        }
        my $unit_name=$cluster_ring->unitname;
        next if ( $unit_name ne $params{ring} );
        my $node_name=$cluster_ring->node;
        my $obj_count=$cluster_ring->$type;
        for (my $i=0; $i <= $size ; $i++){
            next if ( $vals[$i]->unitname ne $params{ring} );
            $Log->comment("Ring vals:  ".$node_name.",".$vals[$i]->node.",".$unit_name.",".$vals[$i]->unitname.",".$obj_count.",".$vals[$i]->$type);
            if ( $obj_count ne $vals[$i]->$type ){
                $matched=0;
                last;
            }
        }
    }
    $Log->debug("Return rdb ring check: ".$matched);
    return $matched;
}


###########################################################################
# Subroutine: _get_rdb_ring_count()
# Parameters: RDB Ring Name
# Returns: Nothing, resets the Global ring count value
# Purpose: Find the number of ring units
###########################################################################
sub _get_rdb_ring_count
{
    my %params = (
            node=>undef,
            @_
    );
    my @vals;
    if ( !defined($params{node})){
        $Log->comment("Node not specified, exiting");
        return 0;
    }       
    try{
        @vals   = NACL::CS::ClusterRing->fetch(
            command_interface => $params{node},
        );
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function _get_rdb_ring_count on component NACL::C::Cluster->fetch is ".$exception_object->text() );
    };
    my $node_name=$params{node}->name();
    my $count=0;
    foreach my $val (@vals){
        if ( $val->node eq $node_name ){
            $count++;
            $Log->debug("Ring count:  ".$count);
        }      
    }
    
    $RING_COUNT=$count;
        
}

###########################################################################
# Subroutine: check_cluster_ha()
# Parameters: None
# Returns: 1 if enabled, 0 if not
# Purpose: Check if cluster ha is enabled
###########################################################################
sub check_cluster_ha
{
    my %params = (
            node=>undef,
            @_
    );
    if ( !defined($params{node})){
        $Log->comment("Node not specified, exiting");
        return 0;
    } 
    my $result="";
    try{
        my $ClusterHa_state = NACL::CS::ClusterHa->fetch(command_interface => $params{node});
        $result=$ClusterHa_state->configured;
        undef $ClusterHa_state;
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function check_cluster_ah on component NACL::C::ClusterHa->fetch is ".$exception_object->text() );
    };      
    $Log->comment("Return cluster ha state: ".$result);
    return $result;
}


###########################################################################
# Subroutine: get_cluster_health()
# Parameters: None
# Returns: 0 if unhealth, 1 if healthy
# Purpose: Check if cluster health for a cluster
###########################################################################
sub get_cluster_health
{
    my %params = (
            node=>undef,
            @_
    );
    my @ClusterHa_states;
    my $result=1;
    my $eligibility;
    my $node_health;
    my $exec_node=$Nodes[0];
    if (scalar(@Nodes) > 2){
        $exec_node=$Nodes[-1];
    }
    if ( defined($params{node})){
        my $state=$params{node}->state('method-timeout'=>30);
        if ( $state->eligibility !~ /true/ || $state->health !~ /true/){
            return 0;
        }
    }else{
        try{
            @ClusterHa_states = NACL::CS::Cluster->fetch(command_interface => $exec_node);
            foreach my $state (@ClusterHa_states){
                $eligibility=$state->eligibility;
                $node_health=$state->health;
                $Log->debug("Node State and Health:  $eligibility/$node_health");
                if(  ($node_health =~ /false/ && $eligibility =~ /true/) || ($node_health =~ /\-/ && $eligibility =~ /\-/)){
                    $result=0;
                    last;
                }
            }            
        }catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function get_cluster_health on component NACL::CS::Cluster->fetch, attempting to reconnect APISET session ".$exception_object->text() );
            my $Api_Set_ObjC = $exec_node->apiset(
                                'interface' => 'CLI',
                              );            
            $Api_Set_ObjC->api_reconnect_session();
            @ClusterHa_states = NACL::CS::Cluster->fetch(command_interface => $exec_node);
            foreach my $state (@ClusterHa_states){
                $eligibility=$state->eligibility;
                $node_health=$state->health;
                $Log->debug("Node State and Health:  $state/$node_health");                
                if( ($node_health =~ /false/ && $eligibility =~ /true/) || ($node_health =~ /\-/ && $eligibility =~ /\-/)){
                    $result=0;
                    last;
                }
            }
        };
        
        
    }
    $Log->debug("Return cluster health: ".$result);
    return $result;
}


###########################################################################
# Subroutine: check_health_of_eligible_nodes_in_cluster()
# Parameters: None
# Returns: 0 if unhealth, 1 if healthy
# Purpose: Check the eligibilty, then validate all rings are up and interfaces are reachable
###########################################################################
sub check_health_of_eligible_nodes_in_cluster
{
    my @vals;
    my @bad_nodes;
    $Log->debug("Checking the RDB Rings");
    try{
        @vals   = NACL::CS::ClusterRing->fetch(
            command_interface => $Nodes[0],
        );          
    }catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->warn( "All Rings not up: ".$exception_object->text() );
    };
    my $ring_status_count=scalar(@vals);
    my $node_count=scalar(@Nodes);
    my $expected_ring_count=$node_count * $RING_COUNT;
    if ( $ring_status_count !=  $expected_ring_count){
        $Log->warn("Number of expected online rings $ring_status_count did not match the expected count $expected_ring_count");
        my @cvals=NACL::CS::Cluster->fetch(
            command_interface => $Nodes[0],
        );
        foreach my $cval (@cvals){
            my $nname=$cval->node;
            my $health=$cval->health;
            if ($health !~ /true/){
                $Log->warn("Node $nname has a health of $health");
                push(@bad_nodes,$nname);
            }
        }
    }
    my $nacl_node;
    my $count=1;
    my $temp_name="";
	foreach my $val (@vals){
        my $node_name=$val->node;
        $Log->debug("Bad Nodes, Node Name:  @bad_nodes, $node_name");
        next if ( grep { $_ eq $node_name } @bad_nodes);
        next if ($node_name eq $temp_name);
        foreach my $n (@Nodes) {
            if ( $n->node() eq $node_name ) {
                $nacl_node = $n;
                last;
            }
        }
        my $state=$nacl_node->state('method-timeout'=>30);
        if ( $state->eligibility !~ /true/){
            $Log->warn("Node $node_name is ".$state->eligibility);
            next;
        }
        $Log->debug("Eligibility ".$state->eligibility." of node $node_name");
        my $ring_state=$val->online;
        my $ring_process=$val->unitname;
        $Log->debug("Online status of $ring_state for process $ring_process on node $node_name");
        if (  $ring_state ne "master" &&  $ring_state ne "secondary"){
            $Log->error("Ring process $ring_process has a status of $ring_state for node $node_name");
            $temp_name=$node_name;
            next;
        }else{
            try{            
                NACL::C::Cluster->ping_cluster(
                    command_interface => $nacl_node,
                    'node' => $node_name,
                    #'method-timeout' => 30
                    );
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->warn( "Unable to ping all nodes in cluster from node $node_name: ".$exception_object->text() );
            };         
        }
        $temp_name=$node_name;
	}
    return 1;
}

###########################################################################
# Subroutine: get_cluster_name()
# Parameters: None
# Returns: Name of the cluster
# Purpose: 
###########################################################################
sub get_cluster_name
{
    my %params = (
            node=>undef,
            @_
    );
    if ( !defined($params{node})){
        $Log->comment("Node not specified, exiting");
        return 0;
    }     
    my $result="";
    try{
        my $ClusterIdentity_state  = NACL::CS::ClusterIdentity->fetch(command_interface => $params{node});
        $result=$ClusterIdentity_state->name;
        undef $ClusterIdentity_state;
        
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function get_cluster_name on component NACL::CS::ClusterIdentity->fetch is ".$exception_object->text() );
    };      
    $Log->debug("Return cluster name: ".$result);
    return $result;
}

###########################################################################
# Subroutine: get_cluster_ip()
# Parameters: None
# Returns: Name of the cluster
# Purpose: 
###########################################################################
sub get_cluster_ip
{
    my %params = (
            node=>undef,
            @_
    );
    if ( !defined($params{node})){
        $Log->comment("Node not specified, exiting");
        return 0;
    }     
    my $result="";
    my $node_name=$params{node}->name();
    try {
        my @NetworkInterfaces = NACL::CS::NetworkInterface->fetch(command_interface =>$params{node});
        foreach my $interface (@NetworkInterfaces){
            next if ( $interface->home_node ne $node_name || $interface->status_oper eq "down" );
            $Log->debug("Interface: ".$interface->lif." Role: ".$interface->role);
            if ($interface->role eq "cluster"){
                $result=$interface->address;
                last;
          }
        }
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function get_cluster_ip on component NACL::CS::NetworkInterface->fetch is ".$exception_object->text() );
    };          
    $Log->debug("Return cluster ip: ".$result);
    return $result;
}

###########################################################################
# Subroutine: _check_model()
# Parameters: None
# Returns: TCD Execution Result (passing)
# Purpose: Find the MODEL Type
###########################################################################
sub _check_model() {
    my $model="FAS";
    try{
        my $Node=NACL::C::Node->find();
        my $SystemNode_state   = NACL::CS::SystemNode->fetch(
            command_interface => $Node,
        );
        #$Log->comment("State Obj:  ".Dumper($SystemNode_state));
        $model=$SystemNode_state->model;
        undef $Node;
        undef $SystemNode_state;
    }
    catch NACL::Exceptions::NoElementsFound with{
        my $exception_object = shift;
        $Log->comment("Execption: ".$exception_object->text());	
    };
    $Log->comment("Model:  $model");
    return $model;
}

###########################################################################
# Subroutine: modify_cluster_ha()
# Parameters: None
# Returns: 1 if modified, 0 if not
# Purpose: Modify cluster ha
###########################################################################
sub modify_cluster_ha
{
    my %params = (
            node=>undef,
            configured=>undef,
            @_
    );
    if ( !defined($params{node}) || !defined($params{configured})){
        $Log->comment("Node or state not specified, exiting");
        return 0;
    }    
    my $result=1;
    #if ( $params{node}->hostrec()->hosttype() =~ /sim/){
    #    $Log->comment("Skipping cluster ha disable for VSIM");
    #    return $result;
    #}
    try{
        NACL::C::ClusterHa->modify(command_interface =>$params{node},
                                   configured=>$params{configured},
                                   );
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function modify_cluster_ha on component NACL::C::ClusterHa->modify is ".$exception_object->text() );
        $result=0;
    };      
    $Log->comment("Return cluster ha state: ".$result);
    return $result;
}

###########################################################################
# Subroutine: find_siteid_node()
# Parameters: None
# Returns: node if found, empty string is not
# Purpose: Find the node in the site list w/ the lowest site id
###########################################################################
sub find_siteid_node{
    my %params = (
            exec_node=>undef,
            id=>"low",
            @_
    );
    if ( !defined($params{exec_node}) ){
        $Log->comment("Execution node not specified, exiting");
        return 0;
    }
    my $low_sitelist_node="";
    my $api_node;
    foreach my $node (@Nodes){
        try{
            $node->name();
            $api_node=$node;
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function find_siteid_node on component $node->name()".$exception_object->text() );
            $Log->comment("Node $node is stale, checking next node");
            next;
        };  
    }
    my $Api_Set_ObjC = $api_node->apiset(
                                    interface => 'CLI',
                                     set      => "Systemshell",
                                  );    
    my $response = $Api_Set_ObjC->execute_raw_command(command=>"tail +30r $SITELIST | awk '{print \$1,\$2}'");
    my @entries=split(/\n/,$response);
    my $size=scalar(@Nodes) - 1;
    $Log->debug("Sitelist Entries: @entries");
    my $found = 0;
    my $site_id=$LOW_SITELIST_ID;
    if ($params{id} ne "low"){
        if ($size < 10){
            $site_id="100".$size;
        }else{
            $site_id="10".$size;
        }
    }
    foreach my $entry (@entries){
        if ($found ){
            last;
        }
        foreach my $node (@Nodes){
            my $node_name=$node->name();
            if ($entry =~ /$site_id/ && $entry =~ /$node_name/){
                $Log->debug("Found node $node_name in sitelist $entry");
                $low_sitelist_node=$node;
                $found=1;
            }
        }        
    }
    
    $Log->comment("Return low site list node: ".$low_sitelist_node->name());
    
    
    return $low_sitelist_node;
}

###########################################################################
# Subroutine: check_sitelist()
# Parameters: None
# Returns: 1 if found, 0 if not found string is not
# Purpose: Checks the site list on each node for a node name and state
###########################################################################
sub check_sitelist{
    my %params = (
            node=>undef,
            state=>undef,
            reset_nodes=>0,
            single_node_check=>0,
            single_node=>undef,
            @_
    );
    if ( !defined($params{node}) && !defined($params{state})){
        $Log->comment("Node and state not specified, exiting");
        return 0;
    }    
    my $result=1;
    my $status="0 1";
    if ( $params{state} eq "offline"){
        $status="0 0";
    }elsif ( $params{state} eq "online"){
        $status="0 1";
    }elsif ( $params{state} eq "master"){
        $status="1 1";
    }
    my $found=0;
    my $exec_node=$params{node};
    my $single_node_name=$params{node}->name();
    if ( defined($params{single_node})){
        $exec_node=$params{single_node};
        $single_node_name=$params{single_node}->name();
    }
    
    if ( $params{reset_nodes} || $PROMOTED_BUILD){
        @Nodes = NACL::C::Node->find();
        foreach my $node (@Nodes){
            my $name = $node->name();
            if ( $single_node_name eq $name ){
                $Log->debug("Found exec node $name");
                $exec_node=$node;
                last;
            }
        }
    }
    if ( $params{single_node_check}){
        my $Api_Set_ObjC = $exec_node->apiset(
                                    'interface' => 'CLI',
                                     set => "Systemshell",
                                  );         
        my $parse_string="\".".$params{node}->name().". $status\"";
        $Log->comment("Parse string for sitelist:  $parse_string");
        my $response = $Api_Set_ObjC->execute_raw_command(command=>"grep $parse_string $SITELIST");
        my @entries=split(/"/,$response);
        $Log->debug("Sitelist Entries: @entries");
        if ( scalar(@entries) < 1 ){
            $result=0;
        }
        
    }else{
        foreach my $val (@Nodes){
            my $Api_Set_ObjC = $val->apiset(
                                        'interface' => 'CLI',
                                         set => "Systemshell",
                                      );         
            #my $parse_string="\"".$params{node}->name().$status."\"";
            my $parse_string="\".".$params{node}->name().". $status\"";
            $Log->comment("Parse string for sitelist:  $parse_string");
            my $response = $Api_Set_ObjC->execute_raw_command(command=>"grep $parse_string $SITELIST");
            my @entries=split(/"/,$response);
            $Log->debug("Sitelist Entries: @entries");
            if ( scalar(@entries) < 1 ){
                $result=0;
            }
            
        }
    }
    $Log->comment("Return check site list for node w/ state $params{state}: ".$params{node}->name().": ".$result);
    
    return $result;
}


###########################################################################
# Subroutine: _check_for_autorecovery()
# Parameters: None
# Returns: 1 if autorecovery exists, 0 if not
# Purpose:
###########################################################################
sub _check_for_autorecovery{
    my %params = (
            node=>undef,
            @_
    );
    my $Host=host($params{node}->name());
    my $Api_Set_ObjS;
    my $Api_Set_ObjC;
    my $conn1;
     my $conn2;
    if ($PROMOTED_BUILD) {
        $Api_Set_ObjS = $params{node}->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }else{
        $conn1 = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );        
        $Api_Set_ObjS = $params{node}->apiset(
                                connection=>$conn1,
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }
    if ($Api_Set_ObjS->execute_raw_command(command=>"sudo kenv $RDB_DISABLE_ARG") =~ /false/){
        return 0;
    }
    if ($PROMOTED_BUILD) {
        $Api_Set_ObjS = $params{node}->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }else{
        $conn2 = Connectrec->new(
                host=>$Host,
                username=>'admin',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );      
        $Api_Set_ObjC = $params{node}->apiset(
                                connection=>$conn2,
                                'interface' => 'CLI',
                              );
    }    
    my $response;
    my $found=0;
    $Log->debug("PROMOTED BUILD:  $PROMOTED_BUILD");
    # Check if this is a promoted build, grep the recovery script
    if ($PROMOTED_BUILD) {
        $response=$Api_Set_ObjS->execute_raw_command(command=>"grep BACKUPDIR /etc/netapp_sync_node_with_cluster");
         if ( $response =~ /BACKUPDIR\"rdb/){
            return $found=1;
         }        
    }    
    try{
        $Api_Set_ObjC->set_privilege('privilege-level' => "test");
        $response=$Api_Set_ObjC->execute_raw_command(command=>"$RDB_TEST_CORRUPT_CMD");
        $Log->debug("Response return value:  $response");
        if ( $response =~ /rdb_process_name/){
            $found=1;
        }
    }catch NATE::BaseException with {
        my $exception_object = shift;
        my $exception_string=$exception_object->text();
        $Log->comment( "Error in function _check_for_autorecovery on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_string );
        #for some reason this does not work in this function
        try{
            $Api_Set_ObjC->api_reconnect_session();
        }catch NATE::BaseException with {
            $Log->comment( "Error in function _check_for_autorecovery on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
            my $host=host($params{node}->name());
            $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "CMode",
                                     );
        };
        $response=$Api_Set_ObjC->execute_raw_command(command=>"$RDB_TEST_CORRUPT_CMD");
        if ( $response =~ /rdb_process_name/){
            $found=1;
        }
        
    };
    if (!$PROMOTED_BUILD){
        $conn1->destroy();
        $conn2->destroy();        
    }

    $Log->debug("Function _check_for_autorecovery returned:  $found");
    return $found;
}
    
###########################################################################
# Subroutine: validate_rdb_error_banner_message()
# Parameters: None
# Returns: 1 if autorecovery exists, 0 if not
# Purpose:
###########################################################################
sub validate_rdb_error_banner_message{
    my %params = (
            node=>undef,
            banner_message=>undef,
            @_
    );
    if ( !defined($params{node}) && !defined($params{banner_message})){
        $Log->comment("Node and banner message not specified, exiting");
        return 0;
    }        
    my $response;
    my $found=0;
    my $message=$params{banner_message};
    my $node_name=$params{node}->name();
    change_process_state(node=>$params{node},service=>"mgwd",command=>"stop");
    change_process_state(node=>$params{node},service=>"mgwd",command=>"start");
    try{
        #try case where we expect no one logged in and not
        my $conn = connect("$node_name#console", -timeout, "5");
        $response = $conn->execute( command => "admin\r",
                        timeout => 30,
                        match_table => [ #"login"    => "admin\r",
                                         "Password" => "netapp1!\r",
                                       ],
                        prompts => ['::>'] );
        if ( $response =~ /not a recognized command/i){
            $response = $conn->execute( command => "exit\r",
                        timeout => 30,
                        match_table => [ "login"    => "admin\r",
                                         "Password" => "netapp1!\r",
                                       ],
                        prompts => ['::>'] );
        }
        $Log->comment("Banner Login message:  \n****$response***\n");
        $response =~ s/\n//g;
        $response =~ s/\r//g;
        $Log->comment("Checking message $response against $message...");
        if ( $response =~ /$message/i){
            $Log->comment("Found message $message...");
            $found=1;
        }
        $conn->destroy();

    }catch NATE::BaseException with {
        my $exception_object = shift;
        my $exception_string=$exception_object->text();
        $Log->comment( "Error in function validate_rdb_error_banner_message on component connection->execute ".$exception_string );
    };
    $Log->comment("Function validate_rdb_error_banner_message returned:  $found");
    return $found;
}

###########################################################################
# Subroutine: corrupt_rdb_database()
# Parameters: None
# Returns: 1 if the file was edited successfully, 0 if not
# Purpose: Put random text in an RDB database file
###########################################################################
sub corrupt_rdb_database{
    my %params = (
            node=>undef,
            database=>undef,
            type=>undef,
            @_
    );
    if ( !defined($params{node}) && !defined($params{database})){
        $Log->comment("Node and datbase not specified, exiting");
        return 0;
    }
    my $database_file="";
    my $result=1;
    if ($params{database} eq "vifmgr") {
        $Log->debug("Attempting to corrupt $params{database} database.") ;
        $database_file="$RDB_PATH/$RDB_VIF_FILE";
    } elsif ($params{database} eq "vldb") {
        $Log->debug("Attempting to corrupt $params{database} database.") ;
        $database_file="$RDB_PATH/$RDB_VOLUME_FILE";
    } elsif ($params{database} eq "mgwd") {
        $Log->debug("Attempting to corrupt $params{database} database.") ;
        $database_file="$RDB_PATH/$RDB_VSERVER_FILE_SN";
    } elsif ($params{database} eq "bcomd") {
        $Log->debug("Attempting to corrupt $params{database} database.");
        $database_file="$RDB_PATH/$RDB_LUN_IGROUP_FILE" ;
    } else {
        $Log->debug("Cannot validate this database type") ;
        $result=0;
    }
    my $response;
    my $node_name=$params{node}->name();
    my $unit=$params{database};
    if ( $params{database} eq "mgwd"){
        $unit="mgmt";
    }    
    if ( $PROMOTED_BUILD==0 && _check_for_autorecovery(node=>$params{node})){
        $Log->comment("******Auto-recovery detected******");
        my $Host=host($node_name);
        if ( defined($params{type})){
            my $conn = Connectrec->new(
                            host=>$Host,
                            username=>'diag',
                            password=>'netapp1!',
                            template=>'ssh-persist',
                            login=>1,
                            );             
            my $Api_Set_ObjC = $params{node}->apiset(
                                connection=>$conn,
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );              
            try{
                $Api_Set_ObjC->execute_raw_command(command=>"sudo touch $NVRAM_CORRUPT_FILE");
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                #for some reason this does not work in this function
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->api_reconnect_session, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                         set => "Systemshell",
                                         context_definition => {username => "diag", password => "netapp1!"}
                                         );
                 };
            
            };
            $Api_Set_ObjC->execute_raw_command(command=>"sudo touch $NVRAM_CORRUPT_FILE");
            change_process_state(node=>$params{node},service=>"mgwd",command=>"stop");
            change_process_state(node=>$params{node},service=>"mgwd",command=>"start");            
            $response=$Api_Set_ObjC->execute_raw_command(command=>" grep -i callhome.root.vol.recovery.reqd /mroot/etc/log/mlog/messages.log");
            my @entries=split(/\n/,$response);
            $Log->debug("Corrupted entry: @entries");
            if ( scalar(@entries) < 1){
                $result=0;
            }
            $Api_Set_ObjC->cleanup_APISet();
            $conn->destroy();
        }else{
            my $conn = Connectrec->new(
                            host=>$Host,
                            username=>'admin',
                            password=>'netapp1!',
                            template=>'ssh-persist',
                            login=>1,
                            );             
            my $Api_Set_ObjC = $params{node}->apiset(
                                    connection=>$conn,
                                    'interface' => 'CLI',
                                  );
            try{
                $Api_Set_ObjC->set_privilege('privilege-level' => "test");
                $response=$Api_Set_ObjC->execute_raw_command(command=>"$RDB_TEST_CORRUPT_CMD -application $params{database}");
                my $command_check=1;
                $Log->comment("Command output:  $response");
                if ( lc($response) =~ /command not found/){
                    $command_check=0;
                }                
                $Log->debug("Response return value:  $response");
                $response=$Api_Set_ObjC->execute_raw_command(command=>"set d;clus ring show -node $node_name -unitname $unit");
                if ( lc($response) !~ /error/ && $command_check){
                    $result=1;
                }
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                #for some reason this does not work in this function
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                             category  => "Node",
                                             interface => "CLI",
                                             set => "CMode",
                                             );
                };
                $response=$Api_Set_ObjC->execute_raw_command(command=>"set d;clus ring show -node $node_name -unitname $unit");
                if ( lc($response) !~ /error/){
                    $result=1;
                }       
                
            };
            $Api_Set_ObjC->cleanup_APISet();
            $conn->destroy();
        }
    }else{
        my $response;
        if ( defined($params{type})){
            my $Api_Set_ObjC = $params{node}->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );        
            try{
                $Api_Set_ObjC->execute_raw_command(command=>"sudo touch $NVRAM_CORRUPT_FILE");             
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                #for some reason this does not work in this function
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->api_reconnect_session, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                         set => "Systemshell",
                                         context_definition => {username => "diag", password => "netapp1!"}
                                         );
                 };
            
            };
            $Api_Set_ObjC->execute_raw_command(command=>"sudo touch $NVRAM_CORRUPT_FILE");
            change_process_state(node=>$params{node},service=>"mgwd",command=>"stop");
            change_process_state(node=>$params{node},service=>"mgwd",command=>"start");            
            $response=$Api_Set_ObjC->execute_raw_command(command=>"ngsh -c \"set d;clus ring show -node $node_name -unitname $unit\"");
            if ( lc($response) !~ /error/){
                $result=1;
            }             
        }else{        
            my $Api_Set_ObjC = $params{node}->apiset(
                                    'interface' => 'CLI',
                                     set => "Systemshell",
                                  );        
            try{
                 $Api_Set_ObjC->execute_raw_command(command=>"cp $database_file ".$database_file.".backup");
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                #for some reason this does not work in this function
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->api_reconnect_session, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                             category  => "Node",
                                             interface => "CLI",
                                             set => "Systemshell",
                                             context_definition => {username => "diag", password => "netapp1!"}
                                             );
                };
                
            };      
            if ( length($response) < 1){
                $response = $Api_Set_ObjC->execute_raw_command(command=>"cp $database_file ".$database_file.".backup");
            }
            $response = $Api_Set_ObjC->execute_raw_command(command=>"sudo echo \"record {CORRUPTING RECORD\}\" >> $database_file");
            $response = $Api_Set_ObjC->execute_raw_command(command=>"sudo grep \"CORRUPTING\" $database_file");
            my @entries=split(/\n/,$response);
            #for some reason you have to restart the ring now for corruption to be detected
            change_process_state(node=>$params{node},service=>$params{database},command=>"stop");
            change_process_state(node=>$params{node},service=>$params{database},command=>"start");    
            $Log->debug("Corrupted entry: @entries");
            if ( scalar(@entries) < 1){
                $result=0;
            }
        }
    }


    $Log->debug("Return corruption for node ".$params{node}->name().": ".$result);
    
    
    return $result;
}

###########################################################################
# Subroutine: clear_rdb_backups()
# Parameters: None
# Returns: 1 if the var was updated correctly, 0 if not
# Purpose: Clear out the RDB backup directories
###########################################################################
sub clear_rdb_backups{
    my %params = (
            node=>undef,
            ring=>undef,
            @_
    );
    my $result=0;
    if ( !defined($params{node}) && defined($params{ring})){
        $Log->comment("Node and ring not specified, exiting");
        return 0;
    }
    my $RING="";
    if ( $params{ring} eq "vifmgr"){
        $RING="VifMgr";
    }elsif ($params{ring} eq "mgwd"){
        $RING="Management";
    }elsif ($params{ring} eq "bcomd"){
        $RING="Bcom";
    }elsif ($params{ring} eq "vldb"){
        $RING="VLDB";
    }else{
        $Log->error("Ring $params{ring} not found");
        return $result;
    }
    my $Api_Set_ObjC;
    my $Host=host($params{node}->name());
    my $conn = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );
    $Api_Set_ObjC = $params{node}->apiset(
                connection =>$conn,
                interface=>"CLI", 
                set => "Systemshell",
                );        
    try{
        $result=$Api_Set_ObjC->execute_raw_command(command=>"sudo rm -rf $RDB_BACKUP_PATH/$RING*");
    }catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function clear_rdb_backups on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
            #for some reason this does not work in this function
            try{
                $Api_Set_ObjC->api_reconnect_session();
            }catch NATE::BaseException with {
                $Log->comment( "Error in function clear_rdb_backups on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                my $host=host($params{node}->name());
                $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                        set => "Systemshell",
                                        context_definition => {username => "diag", password => "netapp1!"}
                                    );
            };
            $result=$Api_Set_ObjC->execute_raw_command(command=>"sudo rm -rf $RDB_BACKUP_PATH/$RING*");
            $Api_Set_ObjC->cleanup_APISet();
    }; 
    $Log->debug("Return deletion of backups for ring ".$params{ring}." on node".$params{node}->name().": ".$result);
    $conn->destroy();
    return $result;
}

###########################################################################
# Subroutine: validate_rdb_backup()
# Parameters: None
# Returns: 1 if the var was updated correctly, 0 if not
# Purpose: Validate a  RDB backup exists
###########################################################################
sub validate_rdb_backup{
    my %params = (
            node=>undef,
            ring=>undef,
            index=>undef,
            value=>undef,
            @_
    );
    my $result=1;
    if ( !defined($params{node}) && defined($params{ring}) && defined($params{index})){
        $Log->comment("Node, index and ring not specified, exiting");
        return 0;
    }
    my $RING="";
    if ( $params{ring} eq "vifmgr"){
        $RING="VifMgr";
    }elsif ($params{ring} eq "mgwd"){
        $RING="Management";
    }elsif ($params{ring} eq "bcomd"){
        $RING="Bcom";
    }elsif ($params{ring} eq "vldb"){
        $RING="VLDB";
    }else{
        return $result;
    }
    my $Api_Set_ObjC;
    my $Host=host($params{node}->name());
    my $conn = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );
    $Api_Set_ObjC = $params{node}->apiset(
                connection =>$conn,
                interface=>"CLI", 
                set => "Systemshell",
                );
    my $count=0;
    try{
        $result=$Api_Set_ObjC->execute_raw_command(command=>"ls $RDB_BACKUP_PATH/$RING*");
        my @temp=split(/\n/,$result);
        $count=scalar(@temp);
    }catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function clear_rdb_backups on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
            #for some reason this does not work in this function
            try{
                $Api_Set_ObjC->api_reconnect_session();
            }catch NATE::BaseException with {
                $Log->comment( "Error in function clear_rdb_backups on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                my $host=host($params{node}->name());
                $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                        set => "Systemshell",
                                        context_definition => {username => "diag", password => "netapp1!"}
                                    );
            };
            $result=$Api_Set_ObjC->execute_raw_command(command=>"ls $RDB_BACKUP_PATH/$RING*");
            my @temp=split(/\n/,$result);
            $count=scalar(@temp);
            $Api_Set_ObjC->cleanup_APISet();
    };
    $Log->comment("Result/Count:  $result/$count");
    if ( $result =~ /No such file or directory/  || $count != $params{index}  || $count > $BACKUP_COUNT){
        if (!defined($params{value})){
            $result=0;            
        }

    }else{
        if (!defined($params{value})){
            $result=1;            
        }
    }
    $Log->debug("Checking for backup file for ring ".$params{ring}." on node".$params{node}->name().": ".$result);
    $conn->destroy();
    return $result;
}

###########################################################################
# Subroutine: validate_rdb_ems_message()
# Parameters: None
# Returns: 1 if the var was updated correctly, 0 if not
# Purpose: Validate a  RDB backup exists
###########################################################################
sub validate_rdb_ems_message{
    my %params = (
            node=>undef,
            ring=>undef,
            failure=>"false",
            @_
    );
    my $result=1;
    if ( !defined($params{node}) && defined($params{ring})){
        $Log->comment("Node and ring not specified, exiting");
        return 0;
    }
    my $Api_Set_ObjC;
    my $node_name=$params{node}->name();
    my $Host=host($node_name);
    my $conn = Connectrec->new(
                host=>$Host,
                username=>'admin',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );
    $Api_Set_ObjC = $params{node}->apiset(
                connection =>$conn,
                interface=>"CLI", 
                );
    my $time="5m";
    if ( scalar(@Nodes) == 2){
        $time="10m";
    }
    try{
        foreach my $ems (@RDB_EMS_MESSAGES){
            next if (($ems eq "rdb.recovery.succeeded" && $params{failure} eq "true") || ($ems eq "rdb.recovery.failed" && $params{failure} eq "false" || $ems eq "rdb.recovery.failed" && $params{failure} eq "nvfail" || $ems eq "rdb.recovery.failed" && $params{failure} eq "fiji") ||
                     ($ems eq "rdb.env.replicaCorrupt" && $params{failure} eq "fiji" || $ems eq "rdb.env.IOFailure" && $params{failure} ne "fiji" || $ems eq "rdb.internal.error" && $params{failure} ne "fiji") );
            if ($params{failure} ne "fiji"){
                $result=$Api_Set_ObjC->execute_raw_command(command=>"event log show -node $node_name -source ".$params{ring}." -event ".$ems."* -time >$time");
            }else{
                $result=$Api_Set_ObjC->execute_raw_command(command=>"event log show -node $node_name -event ".$ems."* -time >$time");
            }
            if ($result =~ /There are no entries matching your query/i){
                $result=0;
                last;
            }
        }
    }catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function validate_rdb_ems_message on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
            #for some reason this does not work in this function
            try{
                $Api_Set_ObjC->api_reconnect_session();
            }catch NATE::BaseException with {
                $Log->comment( "Error in function validate_rdb_ems_message on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                my $host=host($params{node}->name());
                $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                        set => "Systemshell",
                                        context_definition => {username => "diag", password => "netapp1!"}
                                    );
            };
            foreach my $ems (@RDB_EMS_MESSAGES){
                next if (($ems eq "rdb.recovery.succeeded" && $params{failure} eq "true") || ($ems eq "rdb.recovery.failed" && $params{failure} eq "false"));
                $result=$Api_Set_ObjC->execute_raw_command(command=>"event log show -node $node_name -source ".$params{ring}." -event ".$ems."* -time >$time");
                if ($result =~ /There are no entries matching your query/i){
                    $result=0;
                    last;
                }
            }
            $Api_Set_ObjC->cleanup_APISet();
    };
    if ( $result =~ /There are no entries matching your query/i){
        $result=0;
    }
    $Log->debug("Checking for ems messages file for ring ".$params{ring}." on node".$node_name.": ".$result);
    $conn->destroy();
    return $result;
}


###########################################################################
# Subroutine: check_corruption_env_var()
# Parameters: None
# Returns: 1 if the var was updated correctly, 0 if not
# Purpose: Restore the RDB database from the .backup file
###########################################################################
sub check_corruption_env_var{
    my %params = (
            node=>undef,
            database=>undef,
            failure=>"false",
            value=>undef,
            @_
    );
    my $result=1;
    if ( !defined($params{node})){
        $Log->comment("Node not specified, exiting");
        return 0;
    }
    my $Api_Set_ObjC;
    my $Host=host($params{node}->name());
    my $conn;
    my $exec_node=$params{node};
    my $single_node_name=$params{node}->name();
    if ( $PROMOTED_BUILD){
        @Nodes = NACL::C::Node->find();
        foreach my $node (@Nodes){
            my $name = $node->name();
            if ( $single_node_name eq $name ){
                $Log->debug("Found exec node $name");
                $exec_node=$node;
                last;
            }
        }
    }      
    if ($PROMOTED_BUILD) {
        $Api_Set_ObjC = $exec_node->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }else{
        $conn = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );        
        $Api_Set_ObjC = $params{node}->apiset(
                                connection=>$conn,
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }    
    my $corruption_val;
    my $count=1;
    while ( $count <= 5 ){
        try{
             $corruption_val=$Api_Set_ObjC->execute_raw_command(command=>"sudo kenv $RDB_BOOT_ARG");
             if ( $corruption_val !~ /unable to get/){
                last;
             }
        }catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function check_corruption_env_var on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
            #for some reason this does not work in this function
            try{
                $Api_Set_ObjC->api_reconnect_session();
            }catch NATE::BaseException with {
                $Log->comment( "Error in function check_corruption_env_var on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                my $host=host($params{node}->name());
                $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                        set => "Systemshell",
                                        context_definition => {username => "diag", password => "netapp1!"}
                                    );
            };
             $corruption_val=$Api_Set_ObjC->execute_raw_command(command=>"sudo kenv $RDB_BOOT_ARG");
             if ( $corruption_val !~ /unable to get/){
                last;
             }
             $Api_Set_ObjC->cleanup_APISet();
        };
        Tharn::sleep(15);
        $count++;
    }    
    #if ( length($corruption_val) < 1){
    #    $corruption_val=$Api_Set_ObjC->execute_raw_command(command=>"sudo kenv $RDB_BOOT_ARG");
    #}       
    #check for state of the environment var as well as logs
    #1000 = vldb corrupt
    #0100 = mgwd corrupt
    #0010 = vifmgr
    #0001 = bcomd
    
    $Log->comment("Corruption Environment Value:  ".$corruption_val);
    if (defined($params{value})){
        $result=$corruption_val;
    }else{    
        if ( $params{failure} eq "false"){
            if ($corruption_val !~ /1000/ && $params{database} eq "vldb" ||
                $corruption_val !~ /0100/ && $params{database} eq "mgwd" ||
                $corruption_val !~ /0010/ && $params{database} eq "vifmgr" ||
                $corruption_val !~ /0001/ && $params{database} eq "bcomd" ){
                    $result=0;
            }
        }else{
            if ($corruption_val !~ /2000/ && $params{database} eq "vldb" ||
                $corruption_val !~ /0200/ && $params{database} eq "mgwd" ||
                $corruption_val !~ /0020/ && $params{database} eq "vifmgr" ||
                $corruption_val !~ /0002/ && $params{database} eq "bcomd" ){
                    $result=0;
            }         
        }
    }

    $Log->debug("Return restoration for node ".$params{node}->name().": ".$result);
    if ( !$PROMOTED_BUILD){
        $conn->destroy();
    }
    return $result;
}

###########################################################################
# Subroutine: modify_rdb_autorecovery()
# Parameters: None
# Returns: 1 if the var was updated correctly, 0 if not
# Purpose: Update the rdb recovery boot arg
###########################################################################
sub modify_rdb_autorecovery{
    my %params = (
            node=>undef,
            state=>undef,
            ring=>undef,
            @_
    );
    my $result=1;
    if ( !defined($params{node}) && !defined($params{state}) && !defined($params{ring})){
        $Log->comment("Node, state, and datbase not specified, exiting");
        return 0;
    }
    my $Api_Set_ObjC;
    my $Host=host($params{node}->name());
    my $conn = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );
    $Api_Set_ObjC = $params{node}->apiset(
                connection =>$conn,
                interface=>"CLI", 
                set => "Systemshell",
                );        
    my $corruption_val;
    my $count=1;
    my $value="";
    if ($params{state} eq "enable"){
        $value="true";
    }else{
        $value="false";
    }
    while ( $count <= 5 ){
        try{
             $Api_Set_ObjC->execute_raw_command(command=>"sudo kenv $RDB_DISABLE_ARG=$value");
             $corruption_val=$Api_Set_ObjC->execute_raw_command(command=>"sudo kenv $RDB_DISABLE_ARG");
             if ( $corruption_val !~ /unable to get/){
                $Api_Set_ObjC->cleanup_APISet();
                last;
             }
        }catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function check_corruption_env_var on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
            #for some reason this does not work in this function
            try{
                $Api_Set_ObjC->api_reconnect_session();
            }catch NATE::BaseException with {
                $Log->comment( "Error in function check_corruption_env_var on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                my $host=host($params{node}->name());
                $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                        set => "Systemshell",
                                        context_definition => {username => "diag", password => "netapp1!"}
                                    );
            };
             $Api_Set_ObjC->execute_raw_command(command=>"sudo kenv $RDB_DISABLE_ARG=$value");
             $corruption_val=$Api_Set_ObjC->execute_raw_command(command=>"sudo kenv $RDB_DISABLE_ARG");
             if ( $corruption_val !~ /unable to get/){
                 $Api_Set_ObjC->cleanup_APISet();
                last;
             }
             $Api_Set_ObjC->cleanup_APISet();
        };
        Tharn::sleep(15);
        $count++;
    }    
    $Log->comment("Corruption Environment Value:  ".$corruption_val);
    if ( $params{failure} eq "false"){
        if ($corruption_val !~ /true/ &&  $corruption_val !~ /false/){
                $result=0;
        }
    }
    if ( $params{ring} ne "mgwd"){
        change_process_state(node=>$params{node},service=>$params{ring},command=>"stop");
        change_process_state(node=>$params{node},service=>"mgwd",command=>"stop");
        change_process_state(node=>$params{node},service=>$params{ring},command=>"start");    
        change_process_state(node=>$params{node},service=>"mgwd",command=>"start");        
    }else{
        change_process_state(node=>$params{node},service=>$params{ring},command=>"stop");
        change_process_state(node=>$params{node},service=>$params{ring},command=>"start");    
    }

    $Log->debug("Return restoration for node ".$params{node}->name().": ".$result);
    Tharn::sleep(45);
    $conn->destroy();
    
    return $result;
}

###########################################################################
# Subroutine: restore_rdb_database()
# Parameters: None
# Returns: 1 if the file was edited successfully, 0 if not
# Purpose: Restore the RDB database from the .backup file
###########################################################################
sub restore_rdb_database{
    my %params = (
            node=>undef,
            database=>undef,
            type=>undef,
            @_
    );
    if ( !defined($params{node}) && !defined($params{database})){
        $Log->comment("Node and datbase not specified, exiting");
        return 0;
    }
    my $database_file="";
    my $result=0;
    my $response;
    my $node_name=$params{node}->node();
    my $unit=$params{database};
    if ( $params{database} eq "mgwd"){
        $unit="mgmt";
    }
    my $Host=host($node_name);
    if ( !defined($params{type}) && _check_for_autorecovery(node=>$params{node})){
        $Log->comment("******Auto-recovery detected******");
        if (scalar(@Nodes) == 2 && check_cluster_ha(node=>$params{node}) eq "true"){
            $Log->comment("2 Nodes detected, need to TakeOver Corrupted Node for Auto-recovery...");
            my $node_name=$params{node}->name();
            my $exec_node;
            foreach my $node (@Nodes){
                if ( $node_name ne $node->name()){
                    $exec_node=$node;
                    last;
                }
            }
            failover_node(node=>$node_name,exec_node=>$exec_node,skip_lif=>"true");
        }
        my $conn1 = Connectrec->new(
                host=>$Host,
                username=>'admin',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );        
        my $Api_Set_ObjC = $params{node}->apiset(
                                connection=>$conn1,
                                'interface' => 'CLI',
                              );
        my $count=1;
        while ( $count <= 10 ){
            try{
                 $response=$Api_Set_ObjC->execute_raw_command(command=>"clus ring show -node $node_name -unitname $unit -field online");
                if ( $response =~ /error/i || $response =~ /offline/i){
                    $count++;
                    Tharn::sleep(15);
                    next;
                }
                 if ( lc($response) =~ /master/ || lc($response) =~ /secondary/){
                    $result=1;
                    last;
                 }
                 $count++;
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->comment( "Error in function restore_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                #for some reason this does not work in this function
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function restore_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                             category  => "Node",
                                             interface => "CLI",
                                             set => "CMode",
                                             );
                };
                $response=$Api_Set_ObjC->execute_raw_command(command=>"clus ring show -node $node_name -unitname $unit");
                if ( $response =~ /error/i || $response =~ /offline/i){
                    $count++;
                    Tharn::sleep(15);
                    next;
                }
                if ( lc($response) =~ /master/ || lc($response) =~ /secondary/){
                    $result=1;
                    last;
                }
            };
            Tharn::sleep(15);
            $count++;
        }
        if ($Api_Set_ObjC->api_is_session_alive()){
            $Api_Set_ObjC->cleanup_APISet();
            $conn1->destroy();
        }
        
    }elsif (defined($params{type})){
            my $conn2 = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );         
            my $Api_Set_ObjC = $params{node}->apiset(
                                connection=>$conn2,
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );         
 
            try{
                 $Api_Set_ObjC->execute_raw_command(command=>"sudo rm $NVRAM_CORRUPT_FILE");
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                #for some reason this does not work in this function
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function corrupt_rdb_database on component Api_Set_ObjC->api_reconnect_session, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                         category  => "Node",
                                         interface => "CLI",
                                         set => "Systemshell",
                                         context_definition => {username => "diag", password => "netapp1!"}
                                         );
                 };
            
            };
            change_process_state(node=>$params{node},service=>"mgwd",command=>"stop");
            $Api_Set_ObjC->execute_raw_command(command=>"sudo rm $NVRAM_CORRUPT_FILE");
            $Api_Set_ObjC->execute_raw_command(command=>"sudo kenv -u bootarg.init.boot_recovery");
            $Api_Set_ObjC->execute_raw_command(command=>"sudo kenv -u $RDB_BOOT_ARG");
            change_process_state(node=>$params{node},service=>"mgwd",command=>"start");         
            my $count=1;
            while ( $count <= 10 ){
                try{
                    $response=$Api_Set_ObjC->execute_raw_command(command=>"ngsh -c \"set d;clus ring show -node $node_name -unitname $unit -field online\"");
                    if ( $response =~ /error/i || $response =~ /offline/i){
                        $count++;
                        Tharn::sleep(15);
                        next;
                    }
                    if ( lc($response) =~ /master/ || lc($response) =~ /secondary/){
                        $result=1;
                        last;
                    }
                }catch NATE::BaseException with {
                    my $exception_object = shift;
                    $Log->comment( "Error in function restore_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                    #for some reason this does not work in this function
                    try{
                        $Api_Set_ObjC->api_reconnect_session();
                    }catch NATE::BaseException with {
                        $Log->comment( "Error in function restore_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to buid a new APISET ".$exception_object->text() );
                        my $host=host($params{node}->name());
                        $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                                 category  => "Node",
                                                 interface => "CLI",
                                                 set => "CMode",
                                                 );
                    };
                    $response=$Api_Set_ObjC->execute_raw_command(command=>"ngsh -c \"set d;clus ring show -node $node_name -unitname $unit\"");
                    if ( $response =~ /error/i || $response =~ /offline/i){
                        $count++;
                        Tharn::sleep(15);
                        next;
                    }
                    if ( lc($response) =~ /master/ || lc($response) =~ /secondary/){
                        $result=1;
                        last;
                    }
                };
                Tharn::sleep(15);
                $count++;
            }
            if ($Api_Set_ObjC->api_is_session_alive()){
                $Api_Set_ObjC->cleanup_APISet();
                $conn2->destroy();
            }
            
            
    }else{        
            $Log->comment("Using node for restoration: $params{node}");
            if ($params{database} eq "vifmgr") {
                $Log->debug("Attempting to corrupt $params{database} database.") ;
                $database_file="$RDB_PATH/$RDB_VIF_FILE";
            } elsif ($params{database} eq "vldb") {
                $Log->debug("Attempting to corrupt $params{database} database.") ;
                $database_file="$RDB_PATH/$RDB_VOLUME_FILE";
            } elsif ($params{database} eq "mgwd") {
                $Log->debug("Attempting to corrupt $params{database} database.") ;
                $database_file="$RDB_PATH/$RDB_VSERVER_FILE_SN";
            } elsif ($params{database} eq "bcomd") {
                $Log->debug("Attempting to corrupt $params{database} database.");
                $database_file="$RDB_PATH/$RDB_LUN_IGROUP_FILE" ;
            } else {
                $Log->debug("Cannot validate this database type") ;
                $result=0;
            }
            my $conn2 = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );             
            my $Api_Set_ObjC = $params{node}->apiset(
                                        connection=>$conn2,
                                        'interface' => 'CLI',
                                         set => "Systemshell",
                                      );
            try{
                $Api_Set_ObjC->execute_raw_command(command=>"mv ".$database_file.".backup"." $database_file ");
            }catch NATE::BaseException with {
                my $exception_object = shift;
                $Log->comment( "Error in function restore_rdb_database on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                #for some reason this does not work in this function
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function restore_rdb_database on component Api_Set_ObjC->api_reconnect_session, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                             category  => "Node",
                                             interface => "CLI",
                                             set => "Systemshell",
                                             context_definition => {username => "diag", password => "netapp1!"}
                                             );
                };
                
            };    
            my $response = $Api_Set_ObjC->execute_raw_command(command=>"mv ".$database_file.".backup"." $database_file ");
            #unset the boot arg for a corrupted mgwd as well
            $Api_Set_ObjC->execute_raw_command(command=>"sudo kenv -u $RDB_BOOT_ARG");
            #remove the nvram failure file or check for debug rdb corrupt command
            #$Api_Set_ObjC->execute_raw_command(command=>"ngsh -c \"set test;debug rdb corrupt foo\"");
            Tharn::sleep(30);
            $response = $Api_Set_ObjC->execute_raw_command(command=>"sudo grep \"CORRUPTING\" $database_file");
            my @entries=split(/\n/,$response);
            $Log->debug("Corrupted entry: @entries");
            if ( scalar(@entries) > 1 ){
                $Log->error("Process $params{database} on node $params{node}->name() was not restored");
                $result=0;
            }
            $conn2->destroy();
    }
    $Log->comment("Return restoration for node ".$params{node}->name().": ".$result);
    return $result;
}

###########################################################################
# Subroutine: modify_eligibility()
# Parameters: None
# Returns: 1 if  modified, 0 if not
# Purpose: Modify node eligibility
###########################################################################
sub modify_eligibility
{
    my %params = (
            node=>undef,
            configured=>undef,
            @_
    );    
    my $result=0;
    if ( !defined($params{node}) && !defined($params{configured})){
        $Log->comment("Node or configuration value not specified, exiting");
        return $result;
    }
   
    my $exec_node=$Nodes[0];
    my $node_name=$params{node}->name();
    @Nodes = NACL::C::Node->find();
    foreach my $node (@Nodes){
        my $state=$node->state('method-timeout'=>30);
        if ( $state->eligibility =~ /true/ && $state->health =~ /true/ && $node_name ne $node->name()){
            $exec_node=$node;
            last;
        }
    }
    $Log->comment("Nodes are ".$exec_node->name()." ".$params{node}->node());
    try{
        NACL::C::Cluster->modify(command_interface=>$exec_node,
                                node=>$params{node},
                                eligibility=>$params{configured});
        $result=1;
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function modify_eligibility on component NACL::C::Cluster->modify is ".$exception_object->text() );
        $result=0;
    };          
    Tharn::sleep(60);
    $Log->debug("Return cluster ha modify: ".$result);
    return $result;
}

###########################################################################
# Subroutine: validate_rdb_update()
# Parameters: None
# Returns: 1 or 0 if the event is saved to the rdb
# Purpose: Trigger the RDB event using the Volume, LIF, Management, or LUN manipulation
###########################################################################
sub validate_rdb_update {
    my %params = (
            obj=>undef,
            is_delete => 0,
            @_
    );
    my $classname = blessed($params{obj}) ;
    my $result=0;
    $Log->log("Received a $classname object") ;
    if ($params{obj}->isa("NACL::C::NetworkInterface")) {
        $Log->debug("It was a VIF.") ;
        $result=_validate(table_name=>"virtual_interface",smf_params=>"-lif ".$params{obj}->lif(),rdb_file_name=>"$RDB_PATH/$RDB_VIF_FILE",parse_string=>$params{obj}->lif(),is_delete=>$params{is_delete}) ;
    } elsif ($params{obj}->isa("NACL::C::Volume")) {
        $Log->debug("It was a volume.") ;        
        #full name of volume - cluster_name://vserver_name/volume_name
        $result=_validate(table_name=>"volume",smf_params=>"-volume ".$params{obj}->volume(),rdb_file_name=>"$RDB_PATH/$RDB_VOLUME_FILE",parse_string=>$params{obj}->volume(),is_delete=>$params{is_delete}) ;
    } elsif ($params{obj}->isa("NACL::C::VolumeQuotaPolicy")) {
        $Log->debug("It was a VolumeQuotaPolicy.") ;
        my $rdb_file="$RDB_PATH/$RDB_VSERVER_FILE_RR";
        $result=_validate(table_name=>"quota_policy_table",smf_params=>"-policy-name ".$params{obj}->policy_name(),rdb_file_name=>$rdb_file,parse_string=>$params{obj}->policy_name(),is_delete=>$params{is_delete}) ;
    } elsif ($params{obj}->isa("NACL::C::LunIgroup")) {
        $Log->debug("It was a Lun Igroup.") ;        
        $result=_validate(table_name=>"igroup",smf_params=>"-igroup ".$params{obj}->igroup(),rdb_file_name=>"$RDB_PATH/$RDB_LUN_IGROUP_FILE",parse_string=>$params{obj}->igroup(),is_delete=>$params{is_delete}) ;
    } else {
        $Log->debug("Cannot validate this object type") ;
        $result=0;
    }
    $Log->debug("Vaidate RDB returning value: $result");
    return $result;
}


############################################################################
# Subroutine: _validate
# Parameters: None
# Returns: 0 if the object is not in the RDB, 1 if the object is in the RDB
# Purpose: 
#############################################################################
sub _validate {
    my %params = (
            table_name=>undef,
            smf_params => undef,
            rdb_file_name => undef,
            parse_string=> undef,
            is_delete => 0,
            @_
    );    
    #validate in the smf table as well as the ascii file
    my $result=1;
    my $exec_node=get_RDB_master_node();
    my $Api_Set_ObjC = $exec_node->apiset(
                                'interface' => 'CLI',
                                );
    try{
        $Api_Set_ObjC->execute_raw_command(command=>"set diag;debug smdb table $params{table_name} show $params{smf_params}");
    }catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function _validate on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
        my $good_node;
        foreach my $node (@Nodes){
            if ( $node->name() eq $exec_node->name()){
                $Api_Set_ObjC = $node->apiset(
                                'interface' => 'CLI',
                              );
                $Log->comment( "Error in function _validate on component Api_Set_ObjC->exec_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
                try{
                    $Api_Set_ObjC->api_reconnect_session();
                }catch NATE::BaseException with {
                    $Log->comment( "Error in function _validat on component Api_Set_ObjC->api_reconnect_session, attempting to buid a new APISET ".$exception_object->text() );
                    my $host=host($params{node}->name());
                    $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                             category  => "Node",
                                             interface => "CLI",
                                             set => "CMode",
                                             );
                };                
                last;
            }
        }         
         
    };    
    my $response = $Api_Set_ObjC->execute_raw_command(command=>"set diag;debug smdb table $params{table_name} show $params{smf_params}");
    $Log->comment("Response for Table Show: $response");
    if ( $response =~ /There are no entries matching your query/ && $params{is_delete} == 0 ){
        $Log->debug("Unable locate $params{type} in SMF");
        return 0;
    }
    
    foreach my $node (@Nodes){
        $result=1;
        next if (!defined($node));
        my $SystemNode_state   = NACL::CS::SystemNode->fetch(
            command_interface => $node,
        );
        next if ( $SystemNode_state->eligibility =~ /offline/i);
        if (!parse_rdb_file(node=>$node,file=>$params{rdb_file_name},parse_string=>$params{parse_string},is_delete=>$params{is_delete})){
            $result=0;
            last
        }
    }
    
    #for SN and higher
    #parse_rdb_file(file=>$params{rdb_file_name},mode=>"bdb");
    $Log->debug("_Validate returning value: $result");
    return $result;
}


############################################################################
# Subroutine: parse_rdb_file
# Parameters: None
# Returns: 0 if the string is not found, 1 if string is found
# Purpose: Check for a string within the rdb  files
#############################################################################
#EXAMPLE RDB ENTRY FOR ASCII BASED DATABASE
# Txn: <3,17>
#
#record {
#    {"_index", {int64:-922407119798943269}}
#    {"_version", {byte_seq:[03000000000000001100000000000000]}}
#    {"1", {uint64:1026}}
#    {"2", {uint64:2147484674}}
#    {"3", {string:"rdb_test_vol_1325709144"}}
#    {"4", {uint32:3}}
#    {"5", {byte_seq:[B8AFC64C1337E111B11E123478563412]}}
#    {"6", {byte_seq:[01000000]}}
#    {"7", {uint64:1}}
#    {"8", {uint32:0}}
#    {"9", {uint64:2147484674}}
#    {"10", {uint32:1325709245}}
#    {"11", {byte_seq:[00]}}
#    {"12", {string:""}}
#    {"14", {uint32:0}}
#    {"19", {byte_seq:[01000000]}}
#    {"20", {byte_seq:[01000000]}}
#    {"21", {byte_seq:[01000000]}}
#    {"22", {uint32:1048}}
#    {"13", {uint64:12884901889}}
#    {"15", {string:""}}
#    {"18", {uint64:1}}
#}

sub parse_rdb_file {
    my %params = (
            node=>$Nodes[0],
            file => undef,
            mode=>"ascii",
            parse_string=> undef,
            is_delete=>0,
            @_
    );
    if ( !defined($params{file}) && !defined($params{parse_string})){
        $Log->comment("RDB file and parse string not specified, exiting");
        return 0;
    }

    my %delete_ids;

    my $result=0;
    my $Api_Set_ObjC = $params{node}->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    my $response;
    my $line_number;
    try{
        $line_number=$Api_Set_ObjC->execute_raw_command(command=>"grep -n $params{parse_string} $params{file} | cut -f1 -d:");
        if ( length($line_number) > 0 ) {
            chomp($line_number);
            $response=$Api_Set_ObjC->execute_raw_command(command=>"tail -n $line_number $params{file}");
        }
    }catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function parse_rdb_file on component Api_Set_ObjC->execute_raw_command, attempting to reconnect APISET session ".$exception_object->text() );
        #for some reason this does not work in this function
        try {
            $Api_Set_ObjC->api_reconnect_session();
        } catch NATE::BaseException with {
            $Log->comment( "Error in function parse_rdb_file on component Api_Set_ObjC->api_reconnect_session, attempting to build a new APISET ".$exception_object->text() );
            my $host=host($params{node}->name());
            $Api_Set_ObjC=NACL::APISet->new(hostobj   => $host,
                                            category  => "Node",
                                            interface => "CLI",
                                            set       => "Systemshell",
                                            context_definition => {username => "diag", password => "netapp1!"}
                                            );
        };
        
    };  
    if ( $params{mode} eq "ascii"){
        my $retries = 5;
        while ( $retries && (length($response) < 1)) {
            Tharn::sleep(30);
            $retries--;
            my $line_number=$Api_Set_ObjC->execute_raw_command(command=>"grep -n $params{parse_string} $params{file} | cut -f1 -d:");
            if ( length($line_number) > 0 ) {
                chomp($line_number);
                $response=$Api_Set_ObjC->execute_raw_command(command=>"tail -n $line_number $params{file}");
                last;
            }
        }

        if ( length($response) < 1 ) {
            $Log->warn("File: ".$params{file}." does not contain entry $params{parse_string}");
            return 0;
        }
        my @entries=split(/\n/,$response);
        $Log->comment("RDB Entries: @entries");
        my $id="";
        if ( $params{is_delete}){
            #make sure the id and index match in the same delete record
            #delete {id = -6752515088248084014, version = {epoch = 3, tnum = 174}}\r\
            my @delete_entries = grep(/delete /, @entries);

            my $position = 0;
            foreach my $del_entry ( @delete_entries ) {
                if ( $del_entry =~ /id = -?(\d+)/ ) {
                    $delete_ids{$1} = $position;
                    $position++;
                }
            }
        }
        my @record_entries = grep(/_index|string:\"$params{parse_string}\"/, @entries);
        my $index  = undef;
        my $string = undef;
        foreach my $entry (@record_entries){
            if ( $entry =~ /_index/ && $entry =~ /:-?(\d+)/ ) {
                $index = $1;
            }            
            if ( $entry =~ /string:\"(\S+)\"/ ) {
                $string = $1;

                # did we find the string we were searching for?
                if ( $string eq $params{parse_string} ) {
                    # See if we already encountered a delete record for this index
                    if ( $params{is_delete} ) {
                        if ( defined $index ) {
                            if ( exists $delete_ids{$index} ) {
                                return 1;
                            } else {
                                return 0;
                            }
                        } else {
                            return 0;
                        }
                    } else {
                        return 1;
                    }
                }
            }
        }
    } else {
        #TODO: dump the file to xml OR parse the log file      
    }
    $Log->debug("Parse DB File returning value: $result");
    
        
    return $result;
} 

###########################################################################
# Subroutine: who_has_epsilon
# Parameters: None
# Returns: node with epsilon
# Purpose: find the node holding epsilon
#############################################################################
sub who_has_epsilon {
    my %opt = ( use_cs_object => 'NACL::CS::Cluster',
                @_ );
    my $es;
    my @es;
    my @prev_es;
    my $retries       = 5;
    my $sleep_between = 0;
    my $attempt = 0;
    my $how_to_report_unsynced = 'comment';

RETRY:

    $sleep_between += 3;

    my $epsilon_mismatch = 0;
    my $prev_num_epsilon = undef;
    my $prev_epsilon_node = undef;
    $attempt++;

    if ( $attempt > 3 ) {
       $how_to_report_unsynced = 'warn';
    }
    try{
        foreach my $node (@Nodes) {
            @es = $opt{use_cs_object}->fetch( command_interface => $node, 
                                         allow_empty => 1, 
                                         filter => { epsilon => 'true' }  );
            if ( defined $prev_num_epsilon ) {
                if ( $prev_num_epsilon != scalar(@es) ) {
                    $Log->$how_to_report_unsynced("Epsilon not the same on all nodes, retry $attempt");
                    $epsilon_mismatch++;
                    last;
                } else {
                    # These conditions should always be warnings as they are just wrong
                    if ( $prev_num_epsilon > 1 ) {
                        $Log->warn("Epsilon should only be set on 0|1 nodes, but was found on ".
                                    $prev_num_epsilon." nodes, retry $attempt");
                        $epsilon_mismatch++;
                        last;
                    } else {
                        if ( $prev_num_epsilon ) {
                            if ( $prev_epsilon_node ne $es[0]->node() ) {
                                $Log->warn("Epsilon is ".$prev_epsilon_node.
                                      " from one node and ".$es[0]->node().
                                      " on another, retry $attempt");
                                $epsilon_mismatch++;
                                last;
                            }
                        }
                    }
                }
            }
            $prev_num_epsilon = scalar(@es);
            if ( $prev_num_epsilon ) {
                $prev_epsilon_node = $es[0]->node(); 
            }
            @prev_es = @es;
            }

            if ( $epsilon_mismatch ) {
                $retries--;
                if ( $retries ) {
                    Tharn::sleep($sleep_between);
                    goto RETRY;
                } else {
                    die "Epsilon not the same on all nodes";
                }
            }
    }catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function who_has_epsilon on component STask->cluster_show: ".$exception_object->text() );
    };    
    if ( @es ) {
        $es = $es[0];
    } else {
        $es = 0;
    }

    return $es;
}

###########################################################################
# Subroutine: recreate_cluster()
# Parameters: Node
# Returns: 1 if  recreated, 0 if not
# Purpose: Recreate Cluster
###########################################################################
sub recreate_cluster
{
    my %params = (
            node=>undef,
            node_name=>undef,
            exec_node=>undef,
            backup=>undef,
            @_
    );    
    my $result=0;
    my $recreate_str="";
    if ( !defined($params{node}) && (!defined($params{node_name}) || !defined($params{backup}))){
        $Log->comment("Node, node name or backup value not specified, exiting");
        return $result;
    }

    if ( defined($params{backup})){
        if ( $params{backup} !~ /7z/){
            $recreate_str=$params{backup}.".7z";
        }else{
            $recreate_str=$params{backup};
        }
        my $Host=host($params{node_name});
        my $conn = Connectrec->new(
                    host=>$Host,
                    username=>'admin',
                    password=>'netapp1!',
                    template=>'ssh-persist',
                    login=>1,
                    );
        my $Api_Set_Obj_s = $params{node}->apiset(
                    connection =>$conn,
                    interface=>"CLI", );
        $Api_Set_Obj_s->set_privilege('privilege-level' => "diag");
        try{
            $Api_Set_Obj_s->system_configuration_recovery_cluster_recreate(from=>"backup",backup=>$recreate_str);
        }
        catch NACL::APISet::Exceptions::MgwdCrashedException with {
            my $exception_object = shift;
            $Log->comment( "Error in function recreate_cluster on component ApiSet->system_configuration_recovery_cluster_recreate is ".$exception_object->text() );
            $result=1;
        };
        #@Nodes = NACL::C::Node->find();
        my $count=1;
        my $health=get_cluster_health();
        while ( $count <= 5){
            $health=get_cluster_health();
            if ( $health == 1){
                $result=1;
                last;
            }
            Tharn::sleep(15);
            $count++;
        }
        $Api_Set_Obj_s->cleanup_APISet();
        $conn->destroy();
    }else{
        $recreate_str="node $params{node_name}";
        try{
            NACL::C::SystemConfigurationRecoveryCluster->recreate(
                command_interface => $params{node},
                from=>$recreate_str
                );
            $result=1;
            Tharn::sleep(120);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function recreate_cluster on component NACL::C::SystemConfigurationRecoveryCluster->recreate is ".$exception_object->text() );
        };
    }
    if ( !defined($params{backup})){
        my $Transit_obj;
        try {
              $Transit_obj = NACL::Transit->new(name => $params{node_name});
              $Transit_obj->change_state(to=>"CLI");
        } catch NACL::Transit::Exceptions::TransitException with {
              my $exception_object = shift;
              $Log->comment("Failed to bring the node back up.");
              $result = 1;
            if ($exception_object->text() =~ /partner has taken over/) {
                my $i = 0;
                my $current_state=$Transit_obj->get_state(timeout=>300);
                $Log->comment("Current node state is: $current_state\n");
                while ($current_state ne 'TAKEN_OVER_WAITING_ONTAP' && $i < 10) {
                       Tharn::sleep(15);
                       $current_state = $Transit_obj->get_state();
                       $Log->comment("Current node state is: $current_state\n");
                       $i++;
                }
                    
                if ($current_state eq 'TAKEN_OVER_WAITING_ONTAP') {
                    $Log->comment("Trying to force giveback..."); 
                    NACL::C::StorageFailover->giveback(command_interface=>$params{exec_node},
                                  fromnode=>$params{exec_node}->name(),
                                  'override-vetoes'=>"true");
                } else {
                    $Log->comment("After waiting 2 minutes, node is in state $current_state"); 
                }
                my $j = 0;  
                while ($j < 10 ) {
                       Tharn::sleep(15);
                       $Log->comment("Checking the node state\n");
                       $current_state = $Transit_obj->get_state(timeout=>300);
                       $Log->comment("Current node state is: $current_state\n");
                       if ($current_state eq 'USERNAME') {
                           $result = 0;
                           @Nodes=NACL::C::Node->find();
                           last;
                       } else {
                         $j++;
                       }
                     }
              }
         };
        Tharn::sleep(60);
    }
    $Log->debug("Return cluster recreate: ".$result);
        
    return $result;
}


###########################################################################
# Subroutine: rejoin_cluster()
# Parameters: Node
# Returns: 1 if  rejoin, 0 if not
# Purpose: Recreate Cluster
###########################################################################
sub rejoin_cluster
{
    my %params = (
            node=>undef,
            node_name=>undef,
            @_
    );    
    my $result=0;
    my $recreate_str="";
    if ( !defined($params{node}) && !defined($params{node_name})){
        $Log->comment("Node or node name value not specified, exiting");
        return $result;
    }
    my $attempts=1;
    my $found=0;
    my $exec_node_name=$params{node}->name();
    my $exec_node=$params{node};
    while ( !$found && $attempts < 6 ){
        try{
            #my $rejoin=NACL::C::SystemConfigurationRecoveryCluster->rejoin(
            #    command_interface => $exec_node,
            #    node=>$params{node_name}
            #    );
            #$result=1;
            my $Host=host($exec_node_name);
            my $conn = Connectrec->new(
                    host=>$Host,
                    username=>'admin',
                    password=>'netapp1!',
                    template=>'ssh-persist',
                    login=>1,
                    );
            my $Api_Set_Obj_s = $params{node}->apiset(
                    connection =>$conn,
                    interface=>"CLI", );
            $Api_Set_Obj_s->set_privilege('privilege-level' => "diag");
            $result=$Api_Set_Obj_s->system_configuration_recovery_cluster_rejoin(node=>$params{node_name});
            $found=1;
            $conn->destroy();
            Tharn::sleep(120);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function rejoin_cluster on component Apiset->system_configuration_recovery_cluster_rejoin is ".$exception_object->text() );
            Tharn::sleep(15);
        };
        $attempts++;
    }
    $Log->debug("Return cluster rejoin: ".$result);
        
    return $result;
}

###########################################################################
# Subroutine: sync_cluster()
# Parameters: Node
# Returns: 1 if  sync'ed, 0 if not
# Purpose: Recreate Cluster
###########################################################################
sub sync_cluster
{
    my %params = (
            node=>undef,
            node_name=>undef,
            @_
    );    
    my $result=0;
    my $recreate_str="";
    if ( !defined($params{node}) && !defined($params{node_name})){
        $Log->comment("Node or node name value not specified, exiting");
        return $result;
    }
    my $attempts=1;
    my $found=0;
    my $exec_node_name=$params{node}->name();
    my $exec_node=$params{node};
    while ( !$found && $attempts < 6 ){
        try{
            #my $rejoin=NACL::C::SystemConfigurationRecoveryCluster->rejoin(
            #    command_interface => $exec_node,
            #    node=>$params{node_name}
            #    );
            #$result=1;
            my $Host=host($exec_node_name);
            my $conn = Connectrec->new(
                    host=>$Host,
                    username=>'admin',
                    password=>'netapp1!',
                    template=>'ssh-persist',
                    login=>1,
                    );
            my $Api_Set_Obj_s = $params{node}->apiset(
                    connection =>$conn,
                    interface=>"CLI", );
            $Api_Set_Obj_s->set_privilege('privilege-level' => "diag");
            $result=$Api_Set_Obj_s->system_configuration_recovery_cluster_sync(node=>$params{node_name});
            $found=1;
            $conn->destroy();
            Tharn::sleep(120);
        }
        catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function sync_cluster on component Apiset->system_configuration_recovery_cluster_sync is ".$exception_object->text() );
            Tharn::sleep(15);
        };
        $attempts++;
    }
    $Log->debug("Return cluster sync: ".$result);
        
    return $result;
}

###########################################################################
# Subroutine: create_backup_config()
# Parameters: Node
# Returns: 1 if  created, 0 if not
# Purpose: Create Backup configuration file
###########################################################################
sub create_backup_config
{
    my %params = (
            node=>undef,
            backup_name=>undef,
            backup_type=>undef,
            @_
    );    
    my $result=0;
    my $recreate_str="";
    if ( !defined($params{node}) && !defined($params{backup_name}) && !defined($params{backup_type})){
        $Log->comment("Node, backup name, backup type value not specified, exiting");
        return $result;
    }
    try{
        NACL::C::SystemConfigurationBackup->create(
                command_interface => $params{node},
                'backup-name'=>$params{backup_name},
                node=>$params{node}->name(),
                'backup-type'=>$params{backup_type},
                );
        $result=1;
        Tharn::sleep(120);
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function create_backup_config on component NACL::C::SystemConfigurationBackup->create is ".$exception_object->text() );
    };     

 
    $Log->debug("Return cluster create backup: ".$result);
    return $result;
}

###########################################################################
# Subroutine: delete_backup_config()
# Parameters: Node
# Returns: 1 if  deleted, 0 if not
# Purpose: delete Backup configuration file
###########################################################################
sub delete_backup_config
{
    my %params = (
            node=>undef,
            backup_name=>undef,
            @_
    );    
    my $result=0;
    if ( !defined($params{node}) && !defined($params{backup_name})){
        $Log->comment("Node, backup name, backup type value not specified, exiting");
        return $result;
    }
    my $backup=$params{backup_name};
    if ( $backup !~ /7z/){
        $backup=$params{backup_name}.".7z";
    }
    try{
        NACL::C::SystemConfigurationBackup->delete(
                command_interface => $params{node},
                node=>$params{node}->name(),
                'backup'=>$backup,
                );
        $result=1;
        Tharn::sleep(120);
    }
    catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function delete_backup_config on component NACL::C::SystemConfigurationBackup->delete is ".$exception_object->text() );
    };     

 
    $Log->debug("Return cluster delete backup: ".$result);
    return $result;
}




###########################################################################
# Subroutine: reboot_node
# Parameters: None
# Returns: result of execution of system menu
# Purpose: reboot the node using the transit
#############################################################################
sub reboot_node {
    my %params = (
            node=>"local",
            exec_node=>undef,
            command=>undef,
            @_
    );    
    my $had_error=0;
    #try{
    #    $Log->comment("Enable auto-giveback on all nodes...");
    #    NACL::C::StorageFailover->modify(  command_interface=>$params{exec_node},
    #                                    node=>'*',
    #                                    'auto-giveback'=>'true');
    #} catch NATE::BaseException with {
    #      my $exception_object = shift;
    #      $Log->comment( "Error in function reboot_node on component NACL::C::StorageFailover->modify is ".$exception_object->text() );
    #};

    $Log->comment("Reboot the node $params{node} ...");
    my $Transit_obj;
    try {
          $Transit_obj = NACL::Transit->new(name => $params{node});
          $Transit_obj->reboot();
    } catch NACL::Transit::Exceptions::TransitException with {
          my $exception_object = shift;
          $Log->comment("Failed to bring the node back up.");
          $Log->comment( "Error in function reboot_node on component NACL::Transit->new is ".$exception_object->text() );
          $had_error = 1;
        if ($exception_object->text() =~ /partner has taken over/) {
            my $i = 0;
            my $current_state=$Transit_obj->get_state(timeout=>300);
            $Log->comment("Current node state is: $current_state\n");
            while ($current_state ne 'TAKEN_OVER_WAITING_ONTAP' && $i < 10) {
                   Tharn::sleep(15);
                   $current_state = $Transit_obj->get_state();
                   $Log->comment("Current node state is: $current_state\n");
                   $i++;
            }
            if ($current_state eq 'TAKEN_OVER_WAITING_ONTAP') {
                $Log->comment("Trying to force giveback..."); 
                NACL::C::StorageFailover->giveback(command_interface=>$params{exec_node},
                              fromnode=>$params{exec_node}->name(),
                              'override-vetoes'=>"true");
            } else {
                $Log->comment("After waiting 2 minutes, node is in state $current_state"); 
            }
            $i = 0;  
            while ($i < 10 ) {
                   Tharn::sleep(15);
                   $Log->comment("Checking the node state\n");
                   $current_state = $Transit_obj->get_state(timeout=>300);
                   $Log->comment("Current node state is: $current_state\n");
                   if ($current_state eq 'USERNAME') {
                       $had_error = 0;
                       @Nodes=NACL::C::Node->find();
                       last;
                   } else {
                     $i++;
                   }
                 }
          }
     };

    if ($had_error) {
         return 1;
    }
}

###########################################################################
# Subroutine: halt_node
# Parameters: None
# Returns: result of execution of system menu
# Purpose: halt the node using the transit
#############################################################################
sub halt_node {
    my %params = (
            node=>"local",
            exec_node=>undef,
            state=>"FIRMWARE",
            @_
    );    
    my $had_error=0;
    #if ($params{state} eq "FIRMWARE" ){
    #    $Log->comment("Enable auto-giveback on all nodes...");
    #    NACL::C::StorageFailover->modify(  command_interface=>NACL::C::Node->new(name=>$params{exec_node}),
    #                                    node=>'*',
    #                                    'auto-giveback'=>'true');
    #}
    my $Transit_obj;
     $Log->comment("Halt/Restore the node $params{node} ...");
     try {
        $Transit_obj = NACL::Transit->new(name => $params{node});
        $Transit_obj->change_state(to=>$params{state});
     } catch NACL::Transit::Exceptions::TransitException with {
        my $exception_object = shift;
        $Log->comment("Failed to bring node up with: " . $exception_object->text());
        $had_error =1;
        if ($exception_object->text() =~ /partner has taken over/) {
            my $i = 0;
            my $current_state;
            $current_state = $Transit_obj->get_state(timeout=>300);
            $Log->comment("Current node state is: $current_state\n");
            while ($current_state ne 'TAKEN_OVER_WAITING_ONTAP' && $i < 10) {
                   Tharn::sleep(15);
                   $current_state = $Transit_obj->get_state();
                   $Log->comment("Current node state is: $current_state\n");
                   $i++;
            }
                
            if ($current_state eq 'TAKEN_OVER_WAITING_ONTAP') {
                $Log->comment("Trying to force giveback..."); 
                NACL::C::StorageFailover->giveback(command_interface=>$params{exec_node},
                              fromnode=>$params{exec_node}->name(),
                              'override-vetoes'=>"true");
            } else {
                $Log->comment("After waiting 2 minutes, node is in state $current_state"); 
            }
            my $j = 0;  
            while ($j < 10 ) {
                   Tharn::sleep(15);
                   $Log->comment("Checking the node state\n");
                   $current_state = $Transit_obj->get_state(timeout=>300);
                   $Log->comment("Current node state is: $current_state\n");
                   if ($current_state eq 'USERNAME') {
                       $had_error = 0;
                       @Nodes=NACL::C::Node->find();
                       last;
                   } else {
                     $j++;
                   }
                 }
          }
     };

     if ($had_error) {
         return 1;
     }
     #reget the node objects after reboot
     @Nodes = NACL::C::Node->find();   
}

###########################################################################
# Subroutine: failover_node
# Parameters: None
# Returns: result of execution of system menu
# Purpose: failover the node using the transit
#############################################################################
sub failover_node {
    my %params = (
            node=>undef,
            exec_node=>undef,
            skip_lif=>"false",
            @_
    );
    if ( !defined($params{node}) || !defined($params{exec_node})){
        $Log->comment("Node or execution node not specified, exiting");
        return 0;
    }    
    my $had_error=0;
    #if ( _check_model() !~ /SIM/){
    #    $Log->comment("Enable auto-giveback on all nodes...");
    #    NACL::C::StorageFailover->modify(  command_interface=>$params{exec_node},
    #                                    node=>'*',
    #                                    'auto-giveback'=>'true');
    #}
    $Log->comment("Failover the Node $params{node} ...");
    my $Transit_obj = NACL::Transit->new(name => $params{node});
    try{
        NACL::C::StorageFailover->takeover(  command_interface=>$params{exec_node},
                                        ofnode=>$params{node},
                                        'skip-lif-migration'=>$params{skip_lif},
                                        );

        Tharn::sleep(30);
    } catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function failover_node on component NACL::C::StorageFailover->takeover is ".$exception_object->text() );
        $had_error=1;
    };
    try{
        my $x;
        my $state=$Transit_obj->get_state();
        $Log->comment("Waiting for taken over node to reboot...");
        while ( $state ne 'FIRMWARE' && $x < 10) {
               Tharn::sleep(15);
               $state = $Transit_obj->get_state();
               $Log->comment("Current node state is: $state\n");
               $x++;
        }
    } catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function failover_node on component Transit_obj->get_state is ".$exception_object->text() );
        $had_error=1;
    };
    
    try {
         $Log->comment("Attempting to bring the node back up...");
         $Transit_obj->change_state(to=>'USERNAME');
    } catch NACL::Transit::Exceptions::TransitException with {
          my $exception_object = shift;
          $Log->comment("Failed to bring node up with: " . $exception_object->text());
          $had_error =1;
          if ($exception_object->text() =~ /partner has taken over/) {
              my $i = 0;
              my $current_state;
              $current_state = $Transit_obj->get_state(timeout=>300);
              $Log->comment("Current node state is: $current_state\n");
              while ($current_state ne 'TAKEN_OVER_WAITING_ONTAP' && $i < 10) {
                     Tharn::sleep(15);
                     $current_state = $Transit_obj->get_state();
                     $Log->comment("Current node state is: $current_state\n");
                     $i++;
              }
                  
              if ($current_state eq 'TAKEN_OVER_WAITING_ONTAP') {
                  $Log->comment("Trying to force giveback..."); 
                  NACL::C::StorageFailover->giveback(command_interface=>$params{exec_node},
                                fromnode=>$params{exec_node}->name(),
                                'override-vetoes'=>"true");
              } else {
                  $Log->comment("After waiting 2 minutes, node is in state $current_state"); 
              }
              my $j = 0;  
              while ($j < 10 ) {
                     Tharn::sleep(15);
                     $Log->comment("Checking the node state\n");
                     $current_state = $Transit_obj->get_state(timeout=>300);
                     $Log->comment("Current node state is: $current_state\n");
                     if ($current_state eq 'USERNAME') {
                         $had_error = 0;
                         @Nodes=NACL::C::Node->find();
                         last;
                     } else {
                       $j++;
                     }
                   }
            }
    };        

     if ($had_error) {
         return 1;
     }
}

############################################################################
# TestCase: check_process_state
# Parameters: None
# Returns: result of execution of spmctl
# Purpose: Change the service state using spmctl
#############################################################################
sub check_process_state{
    my %params = (
            service => undef,
            node=>undef,
            @_
    );    
    if ( !$params{service}  && !$params{node}){
        $Log->error("Service name and node not specified, exiting");
        return 0;
    }
    $params{node}->refresh_command_interface();
    my $result=0;
    my $Api_Set_ObjS = $params{node}->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );      

    my $response = $Api_Set_ObjS->execute_raw_command(command=>"ps -aux |grep $params{service}");;
    if ( $response =~ /\/sbin\/$params{service}/){
        $Log->comment("process $params{process} running");
        $result=1;
    }
    $Api_Set_ObjS->cleanup_APISet();
    $Log->debug("Return ps result: $result");
    return  $result; 

}


###########################################################################
# TestCase: change_process_state   
# Parameters: None
# Returns: 
# Purpose: restart the process and re-establish connections
#############################################################################
sub change_process_state {
    my %params = (
            service => undef,        
            node=>undef,
            command=>undef,
            @_
    );
    my $Host=host($params{node}->name());
    my $Api_Set_Obj_s;
    my $conn;
    my $exec_node=$params{node};
    my $single_node_name=$params{node}->name();
    if ( $PROMOTED_BUILD){
        @Nodes = NACL::C::Node->find();
        foreach my $node (@Nodes){
            my $name = $node->name();
            if ( $single_node_name eq $name ){
                $Log->debug("Found exec node $name");
                $exec_node=$node;
                last;
            }
        }
    }    
    if ($PROMOTED_BUILD) {
        $Api_Set_Obj_s = $exec_node->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }else{
        $conn = Connectrec->new(
                host=>$Host,
                username=>'diag',
                password=>'netapp1!',
                template=>'ssh-persist',
                login=>1,
                );        
        $Api_Set_Obj_s = $params{node}->apiset(
                                connection=>$conn,
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }    
    my $result=0;
    my $found=0;
    my $response;
    try{
        if ( $params{command} eq "stop"){
            $result = $Api_Set_Obj_s->spmctl(handle =>$params{service},stop=>1);
        }
        if ( $params{command} eq "start"){
            my $attempts=1;
            $Log->debug("Attempting to start process $params{service}");
            while ( !$found && $attempts < 6 ){
                try{
                    $response=$Api_Set_Obj_s->spmctl(handle =>$params{service},exec=>1);
                    $found=1;
                    $result=1;
                }catch NATE::BaseException with{
                    my $exception_object = shift;
                    $Log->comment("Error in function change_process_state on Api_Set_Obj_s->spmctl is " . $exception_object->text());
                    $Log->comment("Response:  $response");
                    if ( $response =~ /14/){
                        $Log->comment("Process already started, exiting loop");
                        $found=1;
                        $result=1;
                    }
                };
                $attempts++;
                Tharn::sleep(30);
            }               
        }
    } catch NATE::BaseException with {
          my $exception_object = shift;
          $Log->comment( "Error in function change_mgwd_process_state on component Apiset->spmctl is ".$exception_object->text() );
          $result=0;
    };
    $Api_Set_Obj_s->cleanup_APISet();
    if (!$PROMOTED_BUILD){
        $conn->destroy();        
    }
    $Log->debug("Return spmctl response: ".$result);
    return  $result; 
}

############################################################################
# TestCase: parse_log
# Parameters: None
# Returns: 0 if the string is not found, 1 if string is found
# Purpose: Check for a string within the logs
#############################################################################
sub parse_log {
    my %params = (
            exec_node=>undef,
            node=>"local",
            log => undef,
            message=>undef,
            method => "log",
            @_
    );
    if ( !$params{log} && !$params{message} && !defined($params{exec_node})){
        $Log->comment("Log file and string to match not specified or execution node is undefined, exiting");
        return 0;
    }
    my $result=0;
    my $logfile;
    my $Api_Set_ObjS;
    my $conn;
    if ( $params{log} =~  /mgwd/ ){
        my $Host=host($params{exec_node}->name());
        $conn = Connectrec->new(
                    host=>$Host,
                    username=>'diag',
                    password=>'netapp1!',
                    template=>'ssh-persist',
                    login=>1,
                    );
        $Api_Set_ObjS = $params{exec_node}->apiset(
                    connection =>$conn,
                    interface=>"CLI", 
                    set => "Systemshell",);        
    }else{
        $Api_Set_ObjS = $params{exec_node}->apiset(
                                'interface' => 'CLI',
                                 set => "Systemshell",
                              );
    }    
   
    try{
        if(ref($params{message}) =~ /array/i){
            my $temp;
            $logfile="";
            foreach my $str (@{$params{message}}){
               $temp=$Api_Set_ObjS->execute_raw_command(command=>"grep \"$str\" $params{log}");
                $logfile=$temp.$logfile;
            }
        }else{
            $logfile=$Api_Set_ObjS->execute_raw_command(command=>"grep \"$params{message}\" $params{log}");
        }
    }catch NATE::BaseException with {
        my $exception_object = shift;
        $Log->comment( "Error in function parse_log on component Api_Set_ObjS->execute_raw_command, attempting to reconnect APISET ".$exception_object->text() );
        #for some reason this does not work in this function
        try{
            $Api_Set_ObjS->api_reconnect_session();
        }catch NATE::BaseException with {
            $Log->comment( "Error in function parse_log on component Api_Set_ObjS->api_reconnect_session, attempting to buid a new APISET ".$exception_object->text() );
            my $host=host($params{exec_node}->name());
            $Api_Set_ObjS=NACL::APISet->new(hostobj   => $host,
                                     category  => "Node",
                                     interface => "CLI",
                                     set => "Systemshell",
                                     context_definition => {username => "diag", password => "netapp1!"}
                                     );
        };
        
    }; 
    
    if ( $params{method} eq "log"){
        my $Api_Set_ObjC = $params{exec_node}->apiset(
                                'interface' => 'CLI',
                              );        
        my $response=$Api_Set_ObjC->system_node_date_show(node=>$params{node});
        my @parsed_output = @{$response->get_parsed_output()};
        $Log->comment("Parsed output: @parsed_output, size: ".scalar(@parsed_output));
        #my $date = "<=".$parsed_output[0]->{'utcdateandtime'};
        my $date=">1m";
        #$Log->error("date: $date");
        try {
            $Api_Set_ObjC->debug_log_files_modify("incl-files"=>$params{log});
            my $logfile=$Api_Set_ObjC->debug_log_show(time=>$date,node=>$params{node});
            $Log->debug("Raw output: ".$logfile->get_raw_output());
            if ($logfile->get_raw_output() =~ /$params{message}/) {
                $Log->debug("Found message $params{message} in log file $params{log}");    
                $result=1;
            }
        } catch NATE::BaseException with {
            my $exception_object = shift;
            $Log->comment( "Error in function parse_log on component Apiset->debug_log_files_modify is ".$exception_object->text() );
            $result=0;
        };
        $Api_Set_ObjS->cleanup_APISet();
        
        
    }else{
        if(ref($params{message}) =~ /array/i){
            my $temp;
            $logfile="";
            foreach my $str (@{$params{message}}){
               $temp=$Api_Set_ObjS->execute_raw_command(command=>"grep \"$str\" $params{log}");
                $logfile=$temp.$logfile;
            }
        }else{
            $logfile=$Api_Set_ObjS->execute_raw_command(command=>"grep \"$params{message}\" $params{log}");
        }        
        my @values=split(/\n/,$logfile);
        $Log->debug("Values from grepping log $params{log}:  @values");
        if (scalar(@values) >= 1) {
            $Log->comment("Found message $params{message} in log file $params{log}");    
            $result=1;
        }
        $Api_Set_ObjS->cleanup_APISet();
        if ( $params{log} =~  /mgwd/ ){
            $conn->destroy();
        }
    }
    
    return $result;
}
=pod

=head1 NAME

RDB.pm - RDB configuration, generation, and verification library for Cmode SN.
=head1 SYNOPSIS



=head1 DESCRIPTION

The methods included in this library were developed in order to assist
engineers in generation and verification of RDB transactions.  The current library
supports  Cmode nodes and invokes communication to the node via the
NACL library.  The RDBpm library allows the user to configure
and verify RDB states and log data.
  
    
=head1 METHODS
            

    reboot_node();
        Description:
            This method will reboot a node
                
        Inputs:
            node=>"local",
            exec_node=>undef,
        Outputs:
            The result of command1/2.


        
=head1 EXAMPLES

Typical example:


=cut

