#$Id: //depot/prod/test/main/lib/Raid/Diskcopy.pm#1 $
##
## @summary  - Library module for disk copy command
## @author kbala@netapp.com
## @status testme
## @pod here

=head1 NAME

Label

=head1 DESCRIPTION

This library contains disk copy command and utililties.

=head1 SYNOPSIS

use Diskcopy;

my $host = host($FILER);

=cut

package Diskcopy;

use strict;
use warnings;

use Carp qw(confess croak);
use Params::Validate qw(validate OBJECT SCALAR BOOLEAN ARRAYREF HASHREF UNDEF);
use Tharn qw(say script_result host logcomment logresult param);
use Module::Common qw(validate_mixed);

BEGIN {
       Params::Validate::validation_options(
				            on_fail => sub { my @msgs = @_; chomp(@msgs);
        			            Carp::confess( Tharn::script_result(@msgs) ) }
  			 	           );
}

use vars ('@ISA', '@EXPORT', '@EXPORT_OK');
require Exporter;

@ISA = qw(Exporter);

@EXPORT = qw(
	     disk_copy
);

use Utils qw(make_conn);
use RaidInfo;
use SayWrap;

#------------------------------------------------------------------------------

=head2 disk_copy

disk_copy(host=>$host, conntype=>$conntype, disk1=>$disk, disk2=>$disk)

Copy the contents of one disk to another 

It is only available in maintenance mode.

The -s option specifies the disk to copy information from. 

The -d option specifies the destination disk to write the data to. 
NOTE: All data on this disk will be overwritten with the contents of the source disk specified. 

Ex: diskcopy -s 7a.18 -d 7a.16 

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. "#console" by default as this command is available only in MAINT mode.

=item disk1

(Required) : Name of disk1

=item disk2

(Required) : Name of disk2

=item sector

(Optional) : Sector size different than 2048 sectors

=item retries

(Optional) : Do no retries of I/O's at the SCSI layer

=item auto_skip

(Optional) : Enable auto-skip mode initially

=item timeout

(Optional) : Timeout vale for scsi format. Default value 12000.
Timeout value need to be increased for disks of larger size.

=back

=cut

#------------------------------------------------------------------------------

sub disk_copy {
    my (%opts) = validate( @_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                conntype => {type=>SCALAR, default=>"#console",  optional=>1},
                                disk1 => { type=>SCALAR },
                                disk2 => { type=>SCALAR },
				sector => { type=>SCALAR, optional=>1 },
				retries => { type=>BOOLEAN, default=>0, optional=>1 },
				auto_skip => { type=>BOOLEAN, default=>0, optional=>1 },
				timeout => { type=>SCALAR, default=>12000, optional=>1 },
                               }
                         );

    my $host = $opts{host};
    my $conntype = $opts{conntype};
    my $disk1 = $opts{disk1};
    my $disk2 = $opts{disk2};
    my $sector = $opts{sector};
    my $retries = $opts{retries};
    my $auto_skip = $opts{auto_skip};
    my $timeout = $opts{timeout};
    my ($cmd, $output);

    $cmd = "diskcopy -s $disk1 -d $disk2";
    $cmd = $cmd . " -n $sector" if ( defined($sector) );
    $cmd = $cmd . " -o" if ( defined($retries) );
    $cmd = $cmd . " -a" if ($auto_skip );

    my $fc = make_conn($host, $conntype);

    say($fc, "$cmd", -prompt, "diskcopy *");
    (undef, undef, $output) = say($fc, "yes", -prompt, "\\*> ", -timeout, $timeout);
    return $output;
}

1;
