# $Id: $
##
## @package RaidErrResync
## @summary RaidErrResync.pm - resyncIO and ErrHandling testcases comman functions are added to this package.
##

package RaidErrResync;

use strict;
use warnings;

# Module Imports
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);
use RaidLW;
use Data::Dumper;
use NACL::APISet;
use Tharn qw($Log);
use Data::Dumper;
use Utils qw(make_conn);


require Exporter;

sub new {
    my $self = shift ;
    my (%opts) = validate(@_, {
                    api_set_obj => {type => OBJECT, isa => 'NACL::APISet'},
                    tc_data=>{type=>HASHREF}, 
                    input_hash=>{type=>HASHREF},
                    conn=>{type => OBJECT},
                    test_obj => {type => OBJECT},
                    test     =>{type=>SCALAR},
                    });
    $self = {};
    $self->{api_set_obj} = $opts{api_set_obj};
    $self->{tc_data}     = $opts{tc_data};
    $self->{input_hash} = $opts{input_hash};
    $self->{conn}       = $opts{conn};
    $self->{test_obj}   = $opts{test_obj};
    $self->{test}       = $opts{test};
    $self->{system_opt_lw}   = "raid.lost_write.enable";
    $self->{aggr_opt_lw}     = "raid_lost_write";
    $self->{TIMEOUT}         = 600;
    $self->{CMDTIMEOUT}      = 200;

    bless $self;
    return $self;
}


################ resync_task ##########################################
## @name :: resync_task
## @summary :: Common function to execute resyncIO testcases.
## @input :: raid_flags_hash => flags to modify and set original value in cleanup.
## @return :: Die with error string in case of failure.
###############################################################################

sub resync_task {
    my $self = shift;
    my (%opts) = validate(
        @_,
        {                                                                  
           raid_flags_hash       => { type => HASHREF},
        }
    );

    my $Api_Set_Obj     = $self->{'api_set_obj'};    
    my $ems_event       = $self->{tc_data}->{'ems_event'};

    my $Test            = $self->{'test_obj'};
    my $test            = $self->{'test'}; 
    my $raid_flags_hash = $opts{'raid_flags_hash'};
    my $Conn            =  $self->{'conn'};

    my $filer_vol       = $self->{input_hash}->{'filer_vol'};                           
                                                            
    my $test_hash            = $self->{tc_data};
    $Log->comment( "Testcase data for TEST - $test\n\t This can be used for debugging in case of failure : \n\n" . Dumper($test_hash) );
    
    my ($output,          $result,              $error,
        $offline_plex,    $online_plex,         $raid_lw_enabled,
        $raid_lost_write, $group_raid_lw_scrub, $group_name,
        $disk_dbn
    );
    my $victim_aggr = undef;
    my ($resyncIO_err_count, $reconsIO_err_count,
        $scrubIO_err_count,  $total_error_detect_count,
		$verifyIO_err_count,
        )
        = 0;
    my $file_to_corrupt = "/vol/$filer_vol/testfile";
    my @scrub_error_summary
        = ( "pi", "cksum_err", "media_err" );

    if ( $self->{tc_data}->{rlw_status} eq "disabled" ) {

      #-----------------------------------------------------------------------
      # Disable system wide option on filer.
      #-----------------------------------------------------------------------
        $Test->step("Disable $self->{system_opt_lw} option on filer");
        my $result = RaidSqa_set_raid_lw_opt(
            api_set_obj  => $Api_Set_Obj,
            option_value => 'off'
        );
        if ( !$result ) {
            die("Disable system wide operation failed");
        }
        $raid_lw_enabled     = 0;
        $raid_lost_write     = "inoperative";
        $group_raid_lw_scrub = "INACTIVE";

    } else {
        $raid_lw_enabled     = 1;
        $raid_lost_write     = "on";
        $group_raid_lw_scrub = "DONE";
    }

    #-------------------------------------------------------
    # Create volume
    #-------------------------------------------------------
    $Test->step("Create volume");
    $self->_create_test_volume();

    #-------------------------------------------------------
    # Check aggr flags
    #-------------------------------------------------------
    $Test->step("Check aggr flags after vol creation");
    $Log->comment("$self->{test} : Check aggr flag's for vol/aggr $filer_vol");
    $output = RaidSqa_check_aggr_lw_flags(
        api_set_obj     => $Api_Set_Obj,
        filer_vol       => $filer_vol,
        raid_lw_enabled => $raid_lw_enabled,
        raid_lost_write => $raid_lost_write,
        raid_lw_allowed => 1
    );
    if ( !$output ) {
        die("Check for lost write flags failed for $filer_vol");
    }

    #-------------------------------------------------------
    # Check group flags
    #-------------------------------------------------------
    my @group_list = RaidSqa_get_group_list(
        api_set_obj => $Api_Set_Obj,
        filer_vol   => $filer_vol
    );
    $Test->step("Check group flag group_raid_lw_scrub");
    $Log->comment(
        "$test : Check group flag group_raid_lw_scrub for rg @group_list");
    $output = RaidSqa_check_group_lw_flags(
        api_set_obj         => $Api_Set_Obj,
        group_list          => \@group_list,
        group_raid_lw_scrub => $group_raid_lw_scrub
    );
    if ( !$output ) {
        die(      "Check for group flag group_raid_lw_scrub for @group_list failed; Expected - $group_raid_lw_scrub"
        );
    }

    if ( $self->{tc_data}->{mirror_create} eq "victim_aggr" ) {

      #-----------------------------------------------------------------------
      # Split one plex from vol/aggr filer_vol
      #-----------------------------------------------------------------------
        $victim_aggr = "split_vol";
        $Test->step("Split the vol/aggr $filer_vol");
        my @plex_list = RaidSqa_get_plex_list(
            api_set_obj => $Api_Set_Obj,
            filer_vol   => $filer_vol
        );
        $Log->comment(
            "$test : Split plex $plex_list[0] to new vol/aggr $victim_aggr");
        $result = RaidSqa_aggr_split(
            api_set_obj  => $Api_Set_Obj,
            plex_name    => $plex_list[0],
            splitted_vol => $victim_aggr
        );
        if ($result) {
            $Log->comment(
                "$test : Split of plex $plex_list[0] to new vol/aggr $victim_aggr succeeded"
            );
        }
        else {
            die("Split operation failed for plex $plex_list[0]");
        }
        if (!RaidSqa_aggr_offline(
                api_set_obj => $Api_Set_Obj,
                filer_vol   => $victim_aggr,
                aggr_type   => $self->{input_hash}->{aggr}
            )
            )
        {
            die("Failed to offline Victim Aggr: $victim_aggr");
        }
    } ## end if ($mirror_create eq ...

    if ( lc($self->{tc_data}->{plex_state}) eq "offline" ) {

      #-----------------------------------------------------------------------
      # Check for fsm state and offline the plex.
      #-----------------------------------------------------------------------
        $Test->step("Offline the plex");
        $Log->comment("$self->{test} : offline one plex from vol/aggr $filer_vol");
        my $plex_hash = $self->_change_plex_state(raid_flags_hash => $raid_flags_hash);
        $offline_plex = $plex_hash->{'offline_plex'};
        $online_plex  = $plex_hash->{'online_plex'};
        $group_name   = $plex_hash->{'group_name'};
        $Log->comment(
            "$test : Offline plex - $offline_plex ; Online plex - $online_plex ; Group name - $group_name"
        );
    } ## end if (lc($plex_state) eq...

    if ( uc($self->{tc_data}->{aggr_state}) ne "NORMAL" ) {

      #-----------------------------------------------------------------------
      # Check for fsm state and offline the plex.
      #-----------------------------------------------------------------------
        $Test->step("Change aggr $filer_vol state");
        $Log->comment(
            "$test : Change aggr $filer_vol fsm_state to ".$self->{tc_data}->{aggr_state});
        my $plex_hash = $self->_change_aggr_fsm_state(raid_flags_hash => $raid_flags_hash);
        $online_plex = $plex_hash->{'online_plex'};
        $group_name  = $plex_hash->{'group_name'};
        $Log->comment(
            "$test : Online plex - $online_plex ; Group name - $group_name");
    }
    else {
        my @group_list = RaidSqa_get_group_list(
            api_set_obj => $Api_Set_Obj,
            filer_vol   => $filer_vol
        );
        $group_name = $group_list[0];
    }

    #-----------------------------------------------------------------------
    # Create file and start hammer
    #-----------------------------------------------------------------------
    $Test->step("Create file and start hammer");
    $Log->comment(  "$test : Create file filename - $file_to_corrupt and start hammer on vol/aggr $filer_vol");
    $disk_dbn = $self->_create_test_file(file_to_corrupt => $file_to_corrupt);

    #-----------------------------------------------------------------------
    # set EMS mark.
    #-----------------------------------------------------------------------
    $Log->comment("$self->{test} : Starting EMS logreader for resyncIO");
    my $ems_mark = RaidSqa_ems_log_get_mark( api_set_obj => $Api_Set_Obj);

    if ( $self->{tc_data}->{inject_at} =~ /during/i ) {

      #-----------------------------------------------------------------------
      # Create resync condition before injecting error
      #-----------------------------------------------------------------------
        $Test->step("Create resync condition before injecting error");
        $offline_plex = $self->_create_resync_state(
            victim_aggr   => $victim_aggr,
            plex_name     => $offline_plex,
        );
    }

    #-----------------------------------------------------------------------
    # Inject lost write error
    #-----------------------------------------------------------------------
    $Test->step("Inject lost write error");
    $Log->comment("$self->{test} : Inject lost write error on group $group_name");
    $self->_inject_lost_write_error(
        group_name => $group_name,
        disk_dbn   => $disk_dbn);
    my @dbn = keys(%$disk_dbn);
    $Log->comment( "$test : Total number of lostwrite errors injected : "
            . scalar(@dbn) );

    #-----------------------------------------------------------------------
    # Start resyncing
    #-----------------------------------------------------------------------
    $Test->step("Start resync");
    $Log->comment("$self->{test} : Start resync operation on vol/aggr $filer_vol");
    $self->_start_resync(
        plex_name                       => $offline_plex,
        victim_aggr                     => $victim_aggr,
        group_name                      => $group_name,     
    );

    sleep(30);
    my %ems_events_resync;

	if (($self->{input_hash}->{raid_type} eq "raid4" && ($self->{tc_data}->{aggr_state} =~ /DEGRADED/i 
		|| $self->{tc_data}->{online_plex_fsm_state} =~ /DEGRADED/i)) || ($self->{input_hash}->{raid_type} eq "raid_dp" 
		&& ($self->{tc_data}->{aggr_state} =~ /DOUBLEDEGRADED/i || $self->{tc_data}->{online_plex_fsm_state} =~ /DOUBLEDEGRADED/i)))  {
		%ems_events_resync = (resync => {event_detect=>'raid.multierr.lw.id.inconsist'});
	} else {
	    %ems_events_resync = %$ems_event;
		delete $ems_events_resync{scrub};
		delete $ems_events_resync{recons};
		delete $ems_events_resync{verify};
	}

    #-------------------------------------------------------
    # Verify error after resync
    #-------------------------------------------------------
    $Test->step("Verify error after resync");
    $Log->comment(
        "$test : Verify error after resync for vol/aggr $filer_vol");
    $self->_verify_error_detection(
        dbn       => \@dbn,
        ems_event => \%ems_events_resync,
        ems_mark  => $ems_mark,
    );
    $resyncIO_err_count = $ems_event->{detect}->{count};
    $Log->comment(
        "$test : Total lostwrite errors detected and fixed by resyncIO : $resyncIO_err_count"
    );
    $ems_event->{detect}->{count} = 0;

   # add code for innjecting error and run verifyIO
   my @dbn_verifyIO = ();
   if ($self->{input_hash}->{io_type} eq "verifyio") {
	   # inject rlw error and run verify io

		#-----------------------------------------------------------------------
		# Inject lost write error
		#-----------------------------------------------------------------------
		$Test->step("Inject lost write error and execute verifyIO");
		$Log->comment("$self->{test} : Inject lost write error on group $group_name");
		$self->_inject_lost_write_error(group_name => $group_name,disk_dbn => $disk_dbn);

		#-----------------------------------------------------------------------
		# set EMS mark.
		#-----------------------------------------------------------------------
		$Log->comment("$self->{test} : Starting EMS logreader for verifyIO");
		$ems_mark
			= RaidSqa_ems_log_get_mark( api_set_obj => $Api_Set_Obj );

		$self->run_verifyio(filer_vol => $filer_vol);

		my %ems_events_verify;

		if (($self->{input_hash}->{raid_type} eq "raid4" && ($self->{tc_data}->{aggr_state} =~ /DEGRADED/i 
			|| $self->{tc_data}->{online_plex_fsm_state} =~ /DEGRADED/i)) || ($self->{input_hash}->{raid_type} eq "raid_dp" 
			&& ($self->{tc_data}->{aggr_state} =~ /DOUBLEDEGRADED/i || $self->{tc_data}->{online_plex_fsm_state} =~ /DOUBLEDEGRADED/i)))  {
			$ems_events_verify{verify}->{event_detect} = 'raid.readerr.lw.id.inconsist';
			$ems_events_verify{verify}->{event_fix} = 'raid.mirror.readerr.rewrite';
		} else {
			%ems_events_verify = %$ems_event;
			delete $ems_events_verify{scrub};
			delete $ems_events_verify{recons};
			delete $ems_events_verify{resync};
		}
		
		@dbn_verifyIO = keys(%$disk_dbn);
		#-------------------------------------------------------
		# Verify error after verifyIO
		#-------------------------------------------------------
		$Test->step("Verify error after verifyIO");
		$Log->comment(
			"$test : Verify error after verifyIO for vol/aggr $filer_vol");
		$self->_verify_error_detection(
			dbn       => \@dbn_verifyIO,
			ems_event => \%ems_events_verify,
			ems_mark  => $ems_mark,
		);
		$verifyIO_err_count = $ems_event->{detect}->{count};
		$Log->comment(
			"$test : Total lostwrite errors detected and fixed by verifyIO : $verifyIO_err_count"
		);
		$ems_event->{detect}->{count} = 0;
   }

    #-----------------------------------------------------------------------
    # Check for fsm state before starting scrub.
    #-----------------------------------------------------------------------
    if ( $self->{tc_data}->{aggr_state} =~ /DEGRADED/i || $self->{tc_data}->{online_plex_fsm_state} =~ /DEGRADED/i )
    {
          $self->{input_hash}->{raid_type} =
          RaidSqa_get_aggr_options(api_set_obj  => $Api_Set_Obj, filer_vol => $filer_vol, options => 'raidtype');
        if ($self->{input_hash}->{raid_type} eq "raid4"
            || uc($self->{tc_data}->{aggr_state})            eq "DOUBLEDEGRADED"
            || uc($self->{tc_data}->{online_plex_fsm_state}) eq "DOUBLEDEGRADED" )
        {

			#-----------------------------------------------------------------------
			# set EMS mark.
			#-----------------------------------------------------------------------
            $Log->comment("$self->{test} : Starting EMS logreader for reconsIO");
            $ems_mark
                = RaidSqa_ems_log_get_mark( api_set_obj => $Api_Set_Obj );

            #---------------------------------------------------------------
            # Enable reconstruction
            #---------------------------------------------------------------
            $Test->step("Start recons and wait for completion");
            $Log->comment("Start recons on vol/aggr $filer_vol");
            $Log->comment(
                "Test $test :: set flag raid_disable_new_recons to 0 enable new recons"
            );
            RaidSqa_setflag(
                api_set_obj => $Api_Set_Obj,
                flag_name   => 'raid_disable_new_recons',
                flag_value  => 0
            );

            sleep(60);
            $Log->comment(
                "Wait for recons to complete on vol/aggr $filer_vol");
            RaidSqa_wait_for_reconstruction_completion(
                api_set_obj => $Api_Set_Obj,
                filer_vol   => $filer_vol
            );
            sleep(30);

            my %ems_events_recons;

			if (($self->{input_hash}->{raid_type} eq "raid4" && ($self->{tc_data}->{aggr_state} =~ /DEGRADED/i 
				|| $self->{tc_data}->{online_plex_fsm_state} =~ /DEGRADED/i)) || ($self->{input_hash}->{raid_type} eq "raid_dp" 
				&& ($self->{tc_data}->{aggr_state} =~ /DOUBLEDEGRADED/i || $self->{tc_data}->{online_plex_fsm_state} =~ /DOUBLEDEGRADED/i)))  {
				%ems_events_recons = (recons => {event_detect=>'raid.multierr.lw.id.inconsist'});
			} else {
				%ems_events_recons = %$ems_event;
				delete $ems_events_resync{scrub};
				delete $ems_events_resync{resync};
				delete $ems_events_resync{verify};
			}

            #-------------------------------------------------------
            # Verify error after recons
            #-------------------------------------------------------
            $Test->step("Verify error after recons");
            $Log->comment(
                "$test : Verify error after recons for vol/aggr $filer_vol");
            $self->_verify_error_detection(
                dbn       => \@dbn,
                ems_event => \%ems_events_recons,
                ems_mark  => $ems_mark,
            );
            $reconsIO_err_count = $ems_event->{detect}->{count};
            $Log->comment(
                "$test : Total lostwrite errors detected and fixed by recons : $reconsIO_err_count"
            );
            $ems_event->{detect}->{count} = 0;
        } ## end if ($raid_type eq "raid4"...
    } ## end if ($aggr_state =~ /DEGRADED/i...
	
	# Sometimes scrub starts automatically after recons; hence waiting for any running scrub to complete.
    RaidSqa_wait_for_scrub_completion(
        api_set_obj => $Api_Set_Obj,
        filer_vol   => $filer_vol
    );

    #-----------------------------------------------------------------------
    # set EMS mark.
    #-----------------------------------------------------------------------
    $Log->comment("$self->{test} : Starting EMS logreader for scrubIO");
    $ems_mark = RaidSqa_ems_log_get_mark( api_set_obj => $Api_Set_Obj );

    #-------------------------------------------------------
    # Start scrub on filer_vol and it should not report any error
    #-------------------------------------------------------
    $Test->step("Start scrub and wait for completion");
    $Log->comment("Start scrub on vol/aggr $filer_vol");
    ( $output, $error ) = RaidSqa_aggr_scrub_start(
        api_set_obj => $Api_Set_Obj,
        filer_vol   => $filer_vol
    );
    if ( !$error ) {
        die("Scrub start operation failed for vol/aggr $filer_vol");
    }
    else {
        $Log->comment("$self->{test} : Scrub started on vol/aggr $filer_vol");
    }

    #-------------------------------------------------------
    # Wait for scrub to complete.
    #-------------------------------------------------------
    $Log->comment("$self->{test} : Wait for scrub to complete on $filer_vol");
    RaidSqa_wait_for_scrub_completion(
        api_set_obj => $Api_Set_Obj,
        filer_vol   => $filer_vol
    );

    my %ems_events_scrub;

	if (($self->{input_hash}->{raid_type} eq "raid4" && ($self->{tc_data}->{aggr_state} =~ /DEGRADED/i 
		|| $self->{tc_data}->{online_plex_fsm_state} =~ /DEGRADED/i)) || ($self->{input_hash}->{raid_type} eq "raid_dp" 
		&& ($self->{tc_data}->{aggr_state} =~ /DOUBLEDEGRADED/i || $self->{tc_data}->{online_plex_fsm_state} =~ /DOUBLEDEGRADED/i)))  {
		%ems_events_scrub = (recons => {event_detect=>'raid.multierr.lw.id.inconsist'});
	} else {
		%ems_events_scrub = %$ems_event;
		delete $ems_events_resync{recons};
		delete $ems_events_resync{resync};
		delete $ems_events_resync{verify};
	}

    #-------------------------------------------------------
    # Verify error after scrub run
    #-------------------------------------------------------
    $Test->step("Verify error after scrub run");
    $Log->comment(
        "$test : Verify error after scrub run for vol/aggr $filer_vol");
    $self->_verify_error_detection(
        dbn       => \@dbn,
        ems_event => \%ems_events_scrub,
        ems_mark  => $ems_mark,
    );

    $scrubIO_err_count = $ems_event->{detect}->{count};
    $Log->comment(
        "$test : Total lostwrite errors detected and fixed by scrub : $scrubIO_err_count"
    );

    $total_error_detect_count
        = ( $resyncIO_err_count + $reconsIO_err_count + $scrubIO_err_count + $verifyIO_err_count );

#-------------------------------------------------------
# Check if total errors injected is equal to total error detected and fixed if RLW is enabled.
#-------------------------------------------------------
    my $total_error_expected_count;
	if ($self->{input_hash}->{io_type} eq "verifyio") {
		$total_error_expected_count = scalar(@dbn) + scalar(@dbn_verifyIO);
	} else {
		$total_error_expected_count = scalar(@dbn);
	}

    $Log->comment(
        "Total lostwrite error injected; count - $total_error_expected_count" );
    my $value_raid_lost_write = RaidSqa_get_options(
        api_set_obj => $Api_Set_Obj,
        option_name => $self->{system_opt_lw}
    );
    if ( $value_raid_lost_write eq "off" ) {
        $Log->comment(
            "System wide option $self->{system_opt_lw} is disabled; No lostwrite errors should be detected"
        );
        if ( $total_error_detect_count == 0 ) {
            $Log->comment(
                "Total lostwrite errors detected and fixed; count - $total_error_detect_count"
            );
        }
        else {
            my $error_count = ( $reconsIO_err_count + $scrubIO_err_count );
            if ( $error_count > 0 ) {
                die("With system wide option off reconsIO and scrubIO should not detect any LW error Expected - 0; Actual - $error_count"
                );
            }
            else {
                $Log->comment(  "Total lostwrite errors detected by resyncIO; count - $total_error_detect_count"
                        . "possible reason: Before system wide option is disabled during resync;errors got detected"
                );
            }
        }
    }
    else {
        $Log->comment(
            "System wide option $self->{system_opt_lw} is enabled; All lostwrite errors should be detected and fixed"
        );
        if ( $total_error_expected_count == $total_error_detect_count ) {
            $Log->comment(
                "Total lostwrite errors detected and fixed; count - $total_error_detect_count"
            );
        }
        else {
            if ($self->{tc_data}->{aggr_state} =~ /DEGRADED/i
                || $self->{tc_data}->{online_plex_fsm_state} =~ /DEGRADED/i )
            {
                if ($self->{input_hash}->{raid_type} eq "raid4"
                    || uc($self->{tc_data}->{aggr_state})            eq "DOUBLEDEGRADED"
                    || uc($self->{tc_data}->{online_plex_fsm_state}) eq "DOUBLEDEGRADED" )
                {
                    my $has_bad_block = RaidSqa_get_aggr_has_bad_blocks(
                        api_set_obj => $Api_Set_Obj,
                        filer_vol   => $filer_vol
                    );
                    if ($has_bad_block) {
                        $Log->comment("In case of multierror has_bad_block flag is set to 1"
                        );
                        my $bad_block_count = RaidSqa_get_aggr_bad_block_count(api_set_obj => $Api_Set_Obj,filer_vol=>$filer_vol);
                        if ($bad_block_count == scalar(@dbn)) {
                            $Log->comment("bad_block_count is equal to total number of RLW error injected - $bad_block_count");
                        } else {
                            die("bad_block_count is not equal to total number of RLW errors injected in case ".
                                "of multierror : Expected - $total_error_expected_count Actual - $bad_block_count");
                        }
                    }
                    else {
                        die("has_bad_block flag is not set in case of multierror : Expected - 1; Actual - $has_bad_block"
                        );
                    }
                } else {
                    die("All error not fixed Expected count - $total_error_expected_count ; Actual count - $total_error_detect_count" );
                }
            }
            else {
                die("All error not fixed Expected count - $total_error_expected_count Actual count - $total_error_detect_count" );
            }
        } ## end else [ if (scalar(@dbn) == $total_error_detect_count)
    } ## end else [ if ($value_raid_lost_write...

    #-------------------------------------------------------
    # Check scrub summary report for all groups.
    #-------------------------------------------------------
    $Test->step("Check scrub summary report for all groups");
    @group_list = RaidSqa_get_group_list(
        api_set_obj => $Api_Set_Obj,
        , filer_vol => $filer_vol
    );
    foreach my $group_name (@group_list) {
        my $ref_report = RaidSqa_get_scrub_report_by_group(
            api_set_obj => $Api_Set_Obj,
            group_name  => $group_name
        );
        foreach my $error (@scrub_error_summary) {
            if ( $ref_report->{$error} ) {
                die("Scrub summary report failed, $error occurred for $ref_report->{$error} times for group $group_name");
            } else {
                $Log->comment("Scrub summary report passed: $error did not occur for group $group_name");
            }
        }
    } ## end foreach my $group_name (@group_list)
} ## end sub resync_task


################ _create_test_volume ##########################################
## @name :: _create_test_volume
## @summary :: Creates a test volume of name $filer_vol
## @return :: Volume name created on success or $TCD::FAIL on failure
###############################################################################

sub _create_test_volume {
    my $self = shift;

    my $Api_Set_Obj     = $self->{'api_set_obj'};
    my $filer_vol       = $self->{input_hash}->{'filer_vol'};                           
    my $raid_type       = $self->{input_hash}->{'raid_type'};                           
    my $fake_la         = $self->{input_hash}->{'fake_la'};                                                             
    my $raid_size       = $self->{input_hash}->{'raid_size'};                           
    my $mirror          = $self->{input_hash}->{'mirror'};                      
    my $disk_count      = $self->{input_hash}->{'disk_count'};                      
    my $aggr            = $self->{input_hash}->{'aggr'};                            
    my $storage_type    = $self->{input_hash}->{'storage_type'};                    
    my $aggr_bits       = $self->{input_hash}->{'aggr_bits'};                   
    my $fake_vol_size   = $self->{input_hash}->{'fake_vol_size'};               
    my $vserver         = $self->{input_hash}->{'vserver'};                     
    my $LA              = $self->{input_hash}->{'la'};                              
    my $checksum_type   = $self->{input_hash}->{'checksum_type'};                   


    my $testname = $self->{test};
    my $Test     =  $self->{test_obj}; 
    
    $Log->comment("TEST ::: $testname");
    #------------------------------------------------------------------
    #define test id to append to volume name if volume already exists
    #------------------------------------------------------------------
    my $testid = "";
    if ( $testname =~ /(\d+)/ ) {
        $testid = "_$1";
    }
    elsif ( $filer_vol =~ /(\d)+/ ) {
        $testid = "_" . ( $1 + 1 );
    }
    else {
        $testid = "_01";
    }

    #------------------------------------------------------------------
    # Check RAID_SIZE for volume
    #------------------------------------------------------------------
    my $req_raid_size = ( $raid_type eq 'raid4' ) ? 2 : 3;
    if ( $fake_la and $raid_size < ( $req_raid_size + 1 ) ) {
        die("$testname: For fake_la, raid_size should be more than $req_raid_size"
        );
    }
    elsif ( $raid_size < $req_raid_size ) {
        die("$testname: raid_size should be $req_raid_size or more but it is $raid_size");
    }

    # Check for disk count
    my $req_disk_count = ( $raid_size + $mirror * $raid_size );
    if ( $disk_count < $req_disk_count ) {
        die("$testname: Disk count should be $req_disk_count or more");
    }

    #-----------------------------------------------------------------
    # Create a testvolume $filer_vol
    #-----------------------------------------------------------------
    $Log->comment("$testid: creating testvolume $filer_vol");
    if (   RaidSqa_check_aggr_exist_anytype(
            api_set_obj => $Api_Set_Obj,
            filer_vol   => $filer_vol
        )
        )
    {
        $filer_vol .= $testid;
    }

    #-------------------------------------------------------------
    # Check if sufficient spare disks are available
    #-------------------------------------------------------------
    $Log->comment("Check if sufficient spare disks are available");

    my ( @pool0_disks, @pool1_disks, $required_disks, @all_disks, $case_type,
        $result, @disk_list, $pref_storage_type );

    $required_disks = $disk_count;
    $case_type      = 1;

# case_type=1 => create volume using disk count if sufficient spares are available in both spare pool
# this is default way for creating volume.

    if ( !defined($storage_type) ) {
        $pref_storage_type = RaidSqa_get_preferred_storage_type(
            api_set_obj => $Api_Set_Obj );
    }
    else {
        $pref_storage_type = $storage_type;
    }

    @pool0_disks = RaidSqa_get_disk_list_by_pool(
        api_set_obj => $Api_Set_Obj,
        pool_name   => "Pool0",
        spare_only  => "yes"
    );

    @pool1_disks = RaidSqa_get_disk_list_by_pool(
        api_set_obj => $Api_Set_Obj,
        pool_name   => "Pool1",
        spare_only  => "yes"
    );

    @all_disks = RaidSqa_get_disk_list_by_storage_type(
        api_set_obj  => $Api_Set_Obj,
        spare_only   => "yes",
        storage_type => $pref_storage_type
    );

    if ($mirror) {
        $required_disks = $disk_count / 2;
        if ( $required_disks >= scalar(@pool1_disks) ) {
            die("$testid: Pool1 does not have sufficient disks");
        }

        if ( $required_disks >= scalar(@pool0_disks) ) {
            die("$testid: Pool0 does not have sufficient disks");
        }
    }
    else {

#case_type=2=> create volume using disk list if sufficient spares are not available in any one spare pool.
        if ( $required_disks >= scalar(@all_disks) ) {
            die("$testid: There are no sufficient spares disks to create volume"
            );
        }

        if (    $required_disks >= scalar(@pool0_disks)
            and $required_disks >= scalar(@pool1_disks) )
        {
            $Log->comment(
                "Creating volume with disk_list instead of disk_count");
            @disk_list = splice( @all_disks, 0, $required_disks );
            $case_type = 2;
        }
    } ## end else [ if ($mirror)

    #---------------------------------------------------------------
    # Create test volume
    #---------------------------------------------------------------
    $Log->comment("Create test volume");
    my %aggr_create_args = (
        api_set_obj   => $Api_Set_Obj,
        filer_vol     => $filer_vol,
        raid_type     => $raid_type,
        aggr_type     => $aggr,
        raid_size     => $raid_size,
        mirror        => $mirror,
        fake_la       => $fake_la,
        aggr_bits     => $aggr_bits,
        fake_vol_size => $fake_vol_size,
        vserver       => $vserver,
        la            => $LA,
        checksum_type => $checksum_type
    );

    if ( $case_type == 1 ) {
        $aggr_create_args{disk_count} = $disk_count;
    }
    elsif ( $case_type == 2 ) {
        $aggr_create_args{disk_list} = \@disk_list;
    }

    $result = RaidSqa_aggr_create(%aggr_create_args);

    if ( !$result ) {
        die("$testid: aggr create failed to create $filer_vol");
    }
    else {
        $Log->comment("$testid: Volume $filer_vol created successfully");
        return 1;
    }
} ## end sub _create_test_volume



################ _change_plex_state ##########################################
## @name :: _change_plex_state
## @summary :: Changes the plex state to offline and change fsm state of onlined plex.
## @input  raid_flags_hash => To update this changes in this subroutine.
## @return :: die with error string in case of failure else return offline plex and online plex value.
###############################################################################

sub _change_plex_state {
    my $self = shift;

    my (%opts) = validate(
        @_,
        {                                                                  
           raid_flags_hash       => { type => HASHREF},
        }
    );

    my $raid_flags_hash = $opts{'raid_flags_hash'};
    my $fsm_state = $self->{tc_data}->{'online_plex_fsm_state'};
    my $filer_vol = $self->{input_hash}->{'filer_vol'};
    my $raid_type = $self->{input_hash}->{'raid_type'};
    my $disk_type_fail1 = $self->{input_hash}->{'disk_type_fail1'};
    my $disk_type_fail2 = $self->{input_hash}->{'disk_type_fail2'};

    my $test            = $self->{'test'};
    my $Api_Set_Obj     =  $self->{'api_set_obj'};
    my $Test            = $self->{'test_obj'};

    my $result;
    my @disk_list;
    my %plex_hash;

    my @plex_list = RaidSqa_get_plex_list(
        api_set_obj => $Api_Set_Obj,
        filer_vol   => $filer_vol
    );
    my $test_plex = $plex_list[0];

    if ( defined($disk_type_fail1) ) {
        my @disk = RaidSqa_get_disk_list_by_raid_type(
            api_set_obj => $Api_Set_Obj,
            raid_type   => lc($disk_type_fail1),
            plex_name   => $plex_list[1]
        );
        if ( !@disk ) {
            die("Get disk of raid_type $disk_type_fail1 failed for plex_name $plex_list[1]"
            );
        }
        $Log->comment("Disk of raid_type - $disk_type_fail1 from plex - $plex_list[1] : @disk");
        push( @disk_list, $disk[0] );
    }

    if ( defined($disk_type_fail2) ) {
        my @disk = RaidSqa_get_disk_list_by_raid_type(
            api_set_obj => $Api_Set_Obj,
            raid_type   => lc($disk_type_fail2),
            plex_name   => $plex_list[1]
        );
        if ( !@disk ) {
            die("Get disk of raid_type $disk_type_fail2 failed for plex_name $plex_list[1]"
            );
        }
            $Log->comment("Disk of raid_type - $disk_type_fail1 from plex - $plex_list[1] : @disk");
            push( @disk_list, $disk[0] );
    }

    if ( !@disk_list ) {
        @disk_list = RaidSqa_get_disk_list(
            api_set_obj => $Api_Set_Obj,
            plex_name   => $plex_list[1]
        );
    }

    my $group_name = RaidSqa_get_disk_group_name(
        api_set_obj => $Api_Set_Obj,
        disk_name   => $disk_list[0]
    );
    $plex_hash{'group_name'} = $group_name;
    if ( $fsm_state =~ /DEGRADED/i ) {
        if ( $raid_type eq "raid4" && uc($fsm_state) eq "DOUBLEDEGRADED" ) {
            die("Double degraded plex is allowed only for raid_dp and not for raid4"
            );
        }
    }

    if ( $fsm_state !~ /NORMAL/ ) {
        $Test->step("Disable reconstruction");

        #---------------------------------------------------------------
        # Disable reconstruction
        #---------------------------------------------------------------
        $Log->comment(
            "Test $test :: set flag raid_disable_new_recons 1 to disable new recons"
        );
        $raid_flags_hash->{raid_disable_new_recons}->{original_val} = RaidSqa_get_setflag(
            api_set_obj => $Api_Set_Obj,
            flag_name   => 'raid_disable_new_recons'
        );
        $raid_flags_hash->{raid_disable_new_recons}->{required_val}=1;
        RaidSqa_setflag(
                api_set_obj => $Api_Set_Obj,
                flag_name   => 'raid_disable_new_recons',
                flag_value  => $raid_flags_hash->{raid_disable_new_recons}->{required_val} ); 
            
        my @disks_to_be_failed = ( uc($fsm_state) eq "DEGRADED" )
            ? ( $disk_list[0] )
            : @disk_list[ 0, 1 ];
        foreach my $disk (@disks_to_be_failed) {
            $Test->step("Fail disk $disk");
            if (         !RaidSqa_disk_fail(
                    api_set_obj => $Api_Set_Obj,
                    disk_name   => $disk,
                    options     => '-i'
                )
                )
            {
                sleep(3);
                if (!RaidSqa_check_disk_fsm_state(
                        api_set_obj => $Api_Set_Obj,
                        disk_name   => $disk,
                        fsm_state   => "FAILED"
                    )
                    )
                {
                    die("Disk fail operation failed for disk $disk");
                }
            }
        }
        sleep(10);
        $Test->step("Check fsm state of group after failing disk as ".uc($fsm_state));
        $result = RaidSqa_get_group_fsm_state(
            api_set_obj => $Api_Set_Obj,
            group_name  => $group_name
        );
        if ( $result ne uc($fsm_state) ) {
            die(         "Check for fsm_state failed for group $group_name : expected - "
                    . uc($fsm_state)
                    . " actual - "
                    . uc($result) );
        }
    } ## end if ($fsm_state !~ /NORMAL/)

    #offline plex
    $result = RaidSqa_plex_offline(
        api_set_obj => $Api_Set_Obj,
        plex_name   => $test_plex
    );
    if ( !$result ) {
        die("Plex offline operation failed for plex $test_plex");
    }
    else {
        $Log->comment("Plex offline operation succeeded for plex $test_plex");
    }
    $plex_hash{'online_plex'}  = $plex_list[1];
    $plex_hash{'offline_plex'} = $test_plex;
    return ( \%plex_hash );
} ## end sub _change_plex_state


################ _change_aggr_fsm_state ##########################################
## @name :: _change_aggr_fsm_state
## @summary :: Add disk to aggr during resync.
## @input raid_flags_hash => Update this hash with the changes in this subroutine.
## @return :: die with error string in case of failure.
###############################################################################

sub _change_aggr_fsm_state {
    my $self = shift;

    my (%opts) = validate(
        @_,
        {                                                                  
           raid_flags_hash       => { type => HASHREF},
        }
    );

    my $raid_flags_hash = $opts{'raid_flags_hash'};
    my $Api_Set_Obj     = $self->{'api_set_obj'};
    my $test            = $self->{'test'};

    my $disk_type_fail1 = $self->{input_hash}->{'disk_type_fail1'};
    my $disk_type_fail2 = $self->{input_hash}->{'disk_type_fail2'};
    my $filer_vol       = $self->{input_hash}->{'filer_vol'};
    my $raid_type       = $self->{input_hash}->{'raid_type'};

    my $fsm_state = $self->{tc_data}->{'aggr_state'};
    my $Test            = $self->{'test_obj'};

    my $result;
    my @disk_list;
    my %plex_hash;

    my @group_list = RaidSqa_get_group_list(
        api_set_obj => $Api_Set_Obj,
        filer_vol   => $filer_vol
    );
    my $test_group = $group_list[0];
    $plex_hash{'group_name'} = $test_group;
    if ( defined($disk_type_fail1) ) {
        my @disk = RaidSqa_get_disk_list_by_raid_type(
            api_set_obj => $Api_Set_Obj,
            raid_type   => lc($disk_type_fail1),
            group_name  => $test_group
        );
        if ( !@disk ) {
            die("Get disk of raid_type $disk_type_fail1 failed for group $test_group"
            );
        }
        $Log->comment("Disk of raid_type - $disk_type_fail1 from rg - $test_group : @disk");
        push( @disk_list, $disk[0] );

    }

    if ( defined($disk_type_fail2) ) {
        my @disk = RaidSqa_get_disk_list_by_raid_type(
            api_set_obj => $Api_Set_Obj,
            raid_type   => lc($disk_type_fail2),
            group_name  => $test_group
        );
        if ( !@disk ) {
            die("Get disk of raid_type $disk_type_fail2 failed for group $test_group"
            );
        }
        $Log->comment("Disk of raid_type - $disk_type_fail2 from rg - $test_group : @disk");
        push( @disk_list, $disk[0] );
    }

    my $online_plex = RaidSqa_get_disk_plex_name(
        api_set_obj => $Api_Set_Obj,
        disk_name   => $disk_list[0]
    );
    $plex_hash{'online_plex'} = $online_plex;

    $result = RaidSqa_get_group_fsm_state(
        api_set_obj => $Api_Set_Obj,
        group_name  => $test_group
    );
    if ( $result eq uc($fsm_state) ) {
        $Log->comment("Group $test_group fsm state : $fsm_state");
        return ( \%plex_hash );
    }
    if ( $fsm_state =~ /DEGRADED/i ) {
        if ( $raid_type eq "raid4" && uc($fsm_state) eq "DOUBLEDEGRADED" ) {
            die("Double degraded is allowed only for raid_dp and not for raid4"
            );
        }
    }

    $Log->comment(
        "Test $test :: set flag raid_disable_new_recons to 1 disable new recons"
    );

    if ( $fsm_state !~ /NORMAL/ ) {
        
        $Test->step("Disable reconstruction");
        #---------------------------------------------------------------
        # Disable reconstruction
        #---------------------------------------------------------------
        $raid_flags_hash->{raid_disable_new_recons}->{original_val} = RaidSqa_get_setflag(
            api_set_obj => $Api_Set_Obj,
            flag_name   => 'raid_disable_new_recons'
        );
        $raid_flags_hash->{raid_disable_new_recons}->{required_val}= 1;
        RaidSqa_setflag(
            api_set_obj => $Api_Set_Obj,
            flag_name   => 'raid_disable_new_recons',
            flag_value  => $raid_flags_hash->{raid_disable_new_recons}->{required_val}
        );
        my @disks_to_be_failed = ( uc($fsm_state) eq "DEGRADED" )
            ? ( $disk_list[0] )
            : @disk_list[ 0, 1 ];
        if ( uc($fsm_state) eq "DEGRADED" ) {
            if ( scalar(@disks_to_be_failed) == 1 ) {
                $Log->comment("Fail disk @disks_to_be_failed");
            }
            else {
                die("Need 1 disk to fail for $fsm_state");
            }
        }
        elsif ( uc($fsm_state) eq "DOUBLEDEGRADED" ) {
            if ( scalar(@disks_to_be_failed) == 2 ) {
                $Log->comment("Fail disk @disks_to_be_failed");
            }
            else {
                die("Need 2 disk to fail for $fsm_state");
            }
        }
        foreach my $disk (@disks_to_be_failed) {
            $Test->step("Fail disk $disk");
            if (!RaidSqa_disk_fail(
                    api_set_obj => $Api_Set_Obj,
                    disk_name   => $disk,
                    options     => '-i'
                )
                )
            {
                sleep(3);
                if (!RaidSqa_check_disk_fsm_state(
                        api_set_obj => $Api_Set_Obj,
                        disk_name   => $disk,
                        fsm_state   => "FAILED"
                    )
                    )
                {
                    die("Disk $disk could not be failed");
                }
            }
        }
        sleep(10);
        $Test->step("Check fsm state of group after failing disk ".uc($fsm_state));
        $result = RaidSqa_get_group_fsm_state(
            api_set_obj => $Api_Set_Obj,
            group_name  => $test_group
        );
        if ( $result ne uc($fsm_state) ) {
            die(      "Group $test_group fsm state: expected - "
                    . uc($fsm_state)
                    . " actual - "
                    . uc($fsm_state) );
        }
    } ## end if ($fsm_state !~ /NORMAL/)
    return (\%plex_hash );
} ## end sub _change_aggr_fsm_state


################ _create_test_file ##########################################
## @name :: _create_test_file
## @summary :: Creates a test file based on the parameters
## @input :: file_to_corrupt => name of the file to be created.
## @return :: return a hash ref which contains reference to disk and DBN info of the file.
###############################################################################

sub _create_test_file {
    my $self = shift;
    my (%opts) = validate(@_,
        {   file_to_corrupt       => { type => SCALAR },
        }
    );

#-------------------------------------------------------
# Create a file which will be used to corrupt genid.
# $max_block_no => This variable is used to get number of dbn values
#                  if file_size is small it will extract only one dbn at block 0 else
#                  it will extract 5 dbn values depending on the stripe_size param.
#                  if stripe_size=1 then 5 consecutive dbn values will be obtained else
#                  if stripe_size=2/3/4.. then 5 dbn values at regular interval will be obtained.
# $no_of_lines =>  This variable is used to multiply with the string to be written for a tetris write.
# $max_line    =>  This variable create a long string.
# $line_block  =>  This variable is used to extract 4kb(1 tetris write) of data from $max_line
# $line        =>  This variable is complete amount of data to be written to the file.
#-------------------------------------------------------
    my $file_to_corrupt = $opts{'file_to_corrupt'};

    my $file_size       = $self->{tc_data}->{'file_size'};
    my $stripe_size     = $self->{tc_data}->{'stripe_size'};

    my $raid_size       = $self->{input_hash}->{'raid_size'};
    my $filer_vol       = $self->{input_hash}->{'filer_vol'};
    my $raid_type       = $self->{input_hash}->{'raid_type'};
    my $checksum_type   = $self->{input_hash}->{'$checksum_type'};

    my $Api_Set_Obj     = $self->{'api_set_obj'};
    my $Conn            = $self->{'conn'};
    my $test            = $self->{'test'};
    my $Test            = $self->{'test_obj'};


    my $data_disk_count = ( $raid_type eq "raid_dp" )
        ? ( $raid_size - 2 )
        : ( $raid_size - 1 );
    my $max_block_no = ( $file_size =~ /small/i ) ? 1 : 5;
    my $no_of_lines = $max_block_no * $data_disk_count * $stripe_size;
    my $max_line
        = "the quick brown fox jumps over the lazy dog.1234567890" x 1000;
    my $line_block         = substr( $max_line, 0, 4096 );
    my $line               = $line_block x $no_of_lines;
    my @data_to_be_written = $line;
    my %disk_dbn;
    $Log->comment(
        "Test $test :: create the file $file_to_corrupt to corrupt the genid"
    );

    filer_file_append( $Conn, $file_to_corrupt, @data_to_be_written );
    $Log->comment("Test $test :: Adding more files to $filer_vol");

    my $file = "/vol/$filer_vol/dummy_file";
    my $cmd  = "priv -level -q diag; mkfile 50m $file 20";
    RaidSqa_filer_cmd( api_set_obj => $Api_Set_Obj, cmd => $cmd,timeout=>'600');

    my $cmd_list  = "priv -level -q diag; ls /vol/$filer_vol";
	RaidSqa_filer_cmd( api_set_obj => $Api_Set_Obj, cmd => $cmd_list);

    $self->_flush_cache();

	my $disk_dbn = $self->_get_file_block_info(max_block_no=>$max_block_no,file_to_corrupt=>$file_to_corrupt);

    #-------------------------------------------------------
    # Start IO(hammer) on test volume
    #-------------------------------------------------------
    $Test->step("Start hammer on $filer_vol");
    $Log->comment("$test :: Start hammer on $filer_vol ");
    RaidSqa_hammer_start(
        api_set_obj => $Api_Set_Obj,
        filer_vol   => $filer_vol
    );

    return ( $disk_dbn );
} ## end sub _create_test_file

################ _get_file_block_info ##########################################
## @name :: _get_file_block_info
## @summary :: Create resync state by offlining the volume when resync in place.
## @input :: max_block_no => No of blocks for which dbn is required.
## @input :: start_block_no => Block start number to get dbn.
## @input :: file_to_corrupt => Name of the file.
## @return :: die with error string in case of failure else returns disk dbn hash.
###############################################################################
sub _get_file_block_info {
    my $self = shift;
    my (%opts) = validate(@_,
        {   max_block_no       => { type => SCALAR, default => 1,optional=>1 },
		    start_block_no       => { type => SCALAR, default => 0,optional=>1 },
			file_to_corrupt     => { type => SCALAR }
        }
    );
    my $file_to_corrupt = $opts{'file_to_corrupt'};
    my $stripe_size     = $self->{tc_data}->{'stripe_size'};
    my $filer_vol       = $self->{input_hash}->{'filer_vol'};
    my $Api_Set_Obj     = $self->{'api_set_obj'};
    my $test            = $self->{'test'};
    my $Test            = $self->{'test_obj'};
    my $max_block_no    = $opts{'max_block_no'};
	my $start_block_no  = $opts{'start_block_no'};

    my %disk_dbn;
    #-------------------------------------------------------
    # Create dbn and disk hash.
    #-------------------------------------------------------
    for ( my $count = $start_block_no; $count < $max_block_no; $count++ ) {
        my $block = $count * $stripe_size;
        $Log->comment("Test $test :: Get dbn and disk for block 0");
        my ( $disk, $dbn, $vbn ) = RaidSqa_get_file_block_info(
            api_set_obj  => $Api_Set_Obj,
            filer_vol    => $filer_vol,
            file_name    => $file_to_corrupt,
            block_number => $block
        );

        if ( $disk =~ /unknown/ ) {
            die
                "Get disk name failed for file - $file_to_corrupt , block - $block";
        }

        if ( $disk and $dbn ) {
            $Log->comment( "Test:$test : DBN - $dbn, Disk - $disk, "
                    . "file - $file_to_corrupt, block - $block" );
            $disk_dbn{$dbn} = $disk;
        }
        else {
            die("Get dbn value for file - $file_to_corrupt , block - $block");
        }
    } ## end for (my $count = 0 ; $count...

	return(\%disk_dbn);

}

################ _create_resync_state ##########################################
## @name :: _create_resync_state
## @summary :: Create resync state by offlining the volume when resync in place.
## @input :: mirror_create => Mirror trad volume using victim_aggr or spare disks.
## @input :: victim_aggr => Name of the victim_aggr used during mirroring..
## @input :: plex_name => Name of the offlined plex.
## @return :: die with error string in case of failure else returns offline plex name.
###############################################################################

sub _create_resync_state {
    my $self = shift;
    my (%opts) = validate(
        @_,
        {   mirror_create =>    { type => SCALAR | UNDEF, default => undef, optional => 1 },
            victim_aggr   =>    { type => SCALAR | UNDEF, default => undef, optional => 1 },
            plex_name     =>    { type => SCALAR | UNDEF, default => undef, optional => 1 },
        }
    );

    my $plex_name     = $opts{'plex_name'};
    my $victim_aggr   = $opts{'victim_aggr'};

    my $mirror_create = $self->{tc_data}->{'mirror_create'};

    my $Api_Set_Obj  = $self->{'api_set_obj'};
    my $Test         = $self->{'test_obj'};

    my $filer_vol    = $self->{input_hash}->{'filer_vol'};

    my $aggr         = $self->{input_hash}->{'aggr'};                            

    # Declare variable for this subroutine.
    my $result;

    if (!defined($mirror_create) && !defined($plex_name)) {
        die("Failed : To create resync state any one value should be provided:  mirror_create or plex_name");
    }

    #-------------------------------------------------------
    # Check for hammer status before creating resync state
    #-------------------------------------------------------
    if ( !RaidSqa_check_hammer_running( api_set_obj => $Api_Set_Obj ) ) {
        RaidSqa_hammer_start(
            api_set_obj => $Api_Set_Obj,
            filer_vol   => $filer_vol
        );
    }

    my $resync_level = ( $mirror_create =~ /spares/i ) ? 0 : 1;
    if ( defined($mirror_create) ) {
        my @plex_list_old = RaidSqa_get_plex_list(
            api_set_obj => $Api_Set_Obj,
            filer_vol   => $filer_vol
        );
        if ( $mirror_create =~ /victim_aggr/i ) {
            $Test->step("Create resync state by mirroring aggr with the split vol");

            #first offline splitvol
            $Log->comment(
                "Mirror vol/aggr $filer_vol using victim_aggr $victim_aggr");
            $result = RaidSqa_aggr_mirror(
                api_set_obj  => $Api_Set_Obj,
                filer_vol    => $filer_vol,
                splitted_vol => $victim_aggr
            );
        }
        else {
            $Test->step("Create resync state by mirroring aggr using spare disks");
            $Log->comment("Mirror vol/aggr $filer_vol using spare disks");
            $result = RaidSqa_aggr_mirror(
                api_set_obj => $Api_Set_Obj,
                filer_vol   => $filer_vol
            );
        }
        if ( !$result ) {
            die("Aggr mirror failed ; Expected : 1; Actual : $result");
        }
        else {
            my @plex_list = RaidSqa_get_plex_list(
                api_set_obj => $Api_Set_Obj,
                filer_vol   => $filer_vol
            );
            my %plex_hash = map { $_, 1 } $plex_list_old[0];
            my @new_plex = grep( !$plex_hash{$_}, @plex_list );
            $plex_name   = $new_plex[0];
            $victim_aggr = undef;
        }
    }
    else {
        $Test->step("Create resync state by onlining offlined plex");
        $Log->comment("Online plex $plex_name");
        $result = RaidSqa_plex_online(
            api_set_obj => $Api_Set_Obj,
            plex_name   => $plex_name
        );
        if ( !$result ) {
            die(         "Plex online failed for plex $plex_name; Expected - 1; Actual - $result"
            );
        }
    }

    my $timeout_resync = time + 120;
    while (!RaidSqa_check_plex_resyncing(
            api_set_obj  => $Api_Set_Obj,
            plex_name    => $plex_name,
            resync_level => $resync_level
        )
        && time < $timeout_resync
        )
    {
        sleep(1);
    }
    if (   !RaidSqa_check_plex_resyncing(
            api_set_obj  => $Api_Set_Obj,
            plex_name    => $plex_name,
            resync_level => $resync_level
        )
        )
    {
        die("Plex resync failed for plex $plex_name");
    }
    $Test->step("Offline aggr $filer_vol to halt resync and inject rlw error");
	RaidSqa_hammer_abort(api_set_obj => $Api_Set_Obj);
    $result = RaidSqa_aggr_offline(api_set_obj => $Api_Set_Obj,filer_vol   => $filer_vol,aggr_type => $aggr);
    if ( !$result ) {
        die("Aggr offline failed for vol/aggr $filer_vol;");
    } else {
        $Log->comment( "Aggr offline was successful for vol/aggr $filer_vol");
    }
    return ($plex_name);
} ## end sub _create_resync_state


################ _inject_lost_write_error #####################################
## @name :: _inject_lost_write_error
## @summary :: Injects lostwrite error on specified dbn values.
## @input :: group_name => Group name to extract the disks.
## @input :: disk_dbn => disk dbn hash.
## @return :: die with error string in case of failure.
###############################################################################

sub _inject_lost_write_error {
    my $self = shift;
    my (%opts) = validate(
        @_,
        {   group_name => { type => SCALAR },
            disk_dbn   => { type => HASHREF },
        }
    );

    my $disk_dbn   = $opts{'disk_dbn'};
    my $group_name = $opts{'group_name'};

    my $Api_Set_Obj = $self->{'api_set_obj'};
    my $disk_type_inject1 = $self->{input_hash}->{'disk_type_inject1'},
    my $disk_type_inject2 = $self->{input_hash}->{'disk_type_inject2'},
    my $gen_count     = $self->{input_hash}->{'gen_count'};
    my $random_number = $self->{input_hash}->{'random_number'};

    # Declare variable for this subroutine.
    my (   $Raidlw,   $result,    @dbn,
        @lw_disk,  @disk_type, @disk_type1,
        %disk_dbn, %lw_hash,   %disk_raid_type
    );
    %disk_dbn = %$disk_dbn;
    @dbn      = keys(%$disk_dbn);
    my $unmount = (@dbn > 1) ? 0 : 1;
    #-------------------------------------------------------
    # Get disk based on disk raid_type
    #-------------------------------------------------------
    if ( !defined($disk_type_inject1) && !defined($disk_type_inject2) ) {
        die("Atleast one of the parameter should be defined : disk_type_inject1 or disk_type_inject2"
        );
    }

    if ( defined($disk_type_inject1) ) {
        @disk_type = RaidSqa_get_disk_list_by_raid_type(
            api_set_obj => $Api_Set_Obj,
            raid_type   => lc($disk_type_inject1),
            group_name  => $group_name
        );
        if (@disk_type) {
            $Log->comment(
                "List of disk from group $group_name by raid_type $disk_type_inject1 : @disk_type"
            );
            $disk_raid_type{ $disk_type[0] } = $disk_type_inject1;
            push( @lw_disk, $disk_type[0] );
        }
        else {
            die(         "Get disk of raid_type $disk_type_inject1 failed for group $group_name"
            );
        }
    } ## end if (defined($DISK_TYPE...

    if ( defined($disk_type_inject2) ) {
        @disk_type1 = RaidSqa_get_disk_list_by_raid_type(
            api_set_obj => $Api_Set_Obj,
            raid_type   => lc($disk_type_inject2),
            group_name  => $group_name
        );
        if (@disk_type1) {
            $Log->comment(
                "List of disk from group $group_name by raid_type $disk_type_inject2 : @disk_type1"
            );
            $disk_raid_type{ $disk_type1[0] } = $disk_type_inject2;
            push( @lw_disk, $disk_type1[0] );
        }
        else {
            die("Get disk of raid_type $disk_type_inject2 failed for group $group_name"
            );
        }
    } ## end if (defined($DISK_TYPE1...

    #-------------------------------------------------------
    # Build lw hash based on disk raid_type for injecting lw error
    #-------------------------------------------------------
    my $disk_rgdn;
    foreach my $dbn (@dbn) {
        foreach my $lw_disk (@lw_disk) {
            $Log->comment("Inject error on disk - $lw_disk dbn - $dbn");
            if ( $disk_raid_type{$lw_disk} =~ /parity/i ) {
                $disk_rgdn = RaidSqa_get_disk_rgdn(
                    api_set_obj => $Api_Set_Obj,
                    disk_name   => $disk_dbn{$dbn}
                );
                if ( defined($disk_rgdn) ) {
                    $Log->comment("RGDN value for data disk $disk_dbn{$dbn} : $disk_rgdn"
                    );
                }
                else {
                    die("Get disk_rgdn value failed for data disk $disk_dbn{$dbn}"
                    );
                }
                $lw_hash{$lw_disk}->{gen_count}->{$disk_rgdn} = $gen_count
                    if ( defined($gen_count) );
            }
            else {
                $lw_hash{$lw_disk}->{gen_count} = $gen_count
                    if ( defined($gen_count) );
            }
            $lw_hash{$lw_disk}->{type}          = $disk_raid_type{$lw_disk};
            $lw_hash{$lw_disk}->{dbn}           = $dbn;
            $lw_hash{$lw_disk}->{random_number} = $random_number
                if ( defined($random_number) );
            $Raidlw = new RaidLW(
                api_set_obj => $Api_Set_Obj,
                disk_name => $disk_dbn{$dbn},
                dbn       => $dbn,
                unmount   => $unmount
            );
            $Log->comment("Dumpblock output before injecting error");
            my $cmd = "priv set test;dumpblock dbn -c $disk_dbn{$dbn} $dbn";
            RaidSqa_filer_cmd( api_set_obj => $Api_Set_Obj, cmd => $cmd );
            $Log->comment("Dumpblock output after injecting error");
            $Raidlw->inject_lost_write_error( lost_write_hash => \%lw_hash );
            RaidSqa_filer_cmd( api_set_obj => $Api_Set_Obj, cmd => $cmd );

        } ## end foreach my $lw_disk (@lw_disk)
    } ## end foreach my $dbn (@dbn)
} ## end sub _inject_lost_write_error

################ _start_resync ##########################################
## @name :: _start_resync
## @summary :: Offline the volume when resync starts.
## @input plex_name                          => Name of the offlined plex
## @input victim_aggr                        => Name of the split vol.
## @input group_name                         => Name of the group to get disk list
## @return :: die with error string in case of failure.
###############################################################################

sub _start_resync {
    my $self  = shift;
    my (%opts) = validate(
        @_,
        {   plex_name =>
                { type => SCALAR | UNDEF, default => undef, optional => 1 },
            victim_aggr =>
                { type => SCALAR | UNDEF, default => undef, optional => 1 },
            group_name  => { type => SCALAR, },
        }
    );

    my $plex_name               = $opts{'plex_name'};
    my $victim_aggr             = $opts{'victim_aggr'};
    my $group_name              = $opts{'group_name'};

    my $aggr                    = $self->{input_hash}->{'aggr'};
    my $raid_type               = $self->{input_hash}->{'raid_type'};
    my $raid_size               = $self->{input_hash}->{'raid_size'};
    my $filer_vol               = $self->{input_hash}->{'filer_vol'};

    my $add_disk_during_resync          = $self->{tc_data}->{'add_disk_during_resync'};
    my $change_rg_during_resync         = $self->{tc_data}->{'change_rg_during_resync'};
    my $change_plex_state_during_resync = $self->{tc_data}->{'change_plex_state_during_resync'};
    my $change_aggr_state_during_resync = $self->{tc_data}->{'change_aggr_state_during_resync'};
    my $change_rlw_state_during_resync  = $self->{tc_data}->{'change_rlw_state_during_resync'};
    my $generate_io_during_resync       = $self->{tc_data}->{'generate_io_during_resync'};
    my $aggr_undestroy_during_resync    = $self->{tc_data}->{'aggr_undestroy_during_resync'};
    my $disk_add_variant                = $self->{tc_data}->{'disk_add_variant'};
    my $mirror_create                   = $self->{tc_data}->{'mirror_create'};
    my $inject_at                       = $self->{tc_data}->{'inject_at'};


    my $Api_Set_Obj                     =  $self->{'api_set_obj'};
    my $test                            = $self->{'test'};
    my $Test                            = $self->{'test_obj'};


    my $resync_level = ( $mirror_create =~ /spares/i ) ? 0 : 1;
    my $result;

    #-------------------------------------------------------
    # Check for hammer status before onlining offlined plex
    #-------------------------------------------------------
    if ( !RaidSqa_check_hammer_running( api_set_obj => $Api_Set_Obj ) ) {
        RaidSqa_hammer_start(
            api_set_obj => $Api_Set_Obj,
            filer_vol   => $filer_vol,
            file_size   => 10
        );
    }

    if ( $inject_at =~ /during/ ) {
        $result = RaidSqa_aggr_online(
            api_set_obj => $Api_Set_Obj,
            filer_vol   => $filer_vol,
            aggr_type   => $aggr
        );
        if ( !$result ) {
            die("Aggr online failed for vol/aggr $filer_vol; Expected - 1; Actual - $result"
            );
        }
    }
    else {
        if ( defined($mirror_create) ) {
            my @plex_list_old = RaidSqa_get_plex_list(
                api_set_obj => $Api_Set_Obj,
                filer_vol   => $filer_vol
            );
            if ( $mirror_create =~ /victim_aggr/i ) {
                $Test->step("Mirror aggr $filer_vol using victim_aggr $victim_aggr");
                $result = RaidSqa_aggr_mirror(
                    api_set_obj  => $Api_Set_Obj,
                    filer_vol    => $filer_vol,
                    splitted_vol => $victim_aggr
                );
            }
            else {
                $Test->step("Mirror aggr $filer_vol using spare disks");
                $result = RaidSqa_aggr_mirror(
                    api_set_obj => $Api_Set_Obj,
                    filer_vol   => $filer_vol
                );
            }
            if ( !$result ) {
                die("Aggr mirror failed ; Expected : 1; Actual : $result");
            }
            else {
                my @plex_list = RaidSqa_get_plex_list(
                    api_set_obj => $Api_Set_Obj,
                    filer_vol   => $filer_vol
                );
                my %plex_hash = map { $_, 1 } $plex_list_old[0];
                my @new_plex = grep( !$plex_hash{$_}, @plex_list );
                $plex_name   = $new_plex[0];
                $victim_aggr = undef;
            }
        }
        else {
            $result = RaidSqa_plex_online(
                api_set_obj => $Api_Set_Obj,
                plex_name   => $plex_name
            );
            if ( !$result ) {
                die(            "Plex online failed for plex $plex_name; Expected - 1; Actual - $result"
                );
            }
        }
    } ## end else [ if ($inject_at =~ /during/)

    #-------------------------------------------------------
    # Wait for plex to start resyncing
    #-------------------------------------------------------
    my $timeout_resync = time + 120;
    while (!RaidSqa_check_plex_resyncing(
            api_set_obj  => $Api_Set_Obj,
            plex_name    => $plex_name,
            resync_level => $resync_level
        )
        && time < $timeout_resync
        )
    {
        sleep(1);
    }

    if (   !RaidSqa_check_plex_resyncing(
            api_set_obj  => $Api_Set_Obj,
            plex_name    => $plex_name,
            resync_level => $resync_level
        )
        )
    {
        die("Plex resync failed for plex $plex_name");
    }
    else {
        $Log->comment("Resync operation succeeded for plex $plex_name");
        if ( $change_rlw_state_during_resync eq "disabled" ) {

      #-----------------------------------------------------------------------
      # Disable system wide option on filer.
      #-----------------------------------------------------------------------
            $Test->step("Disable $self->{system_opt_lw} option on filer");
            $result = RaidSqa_set_raid_lw_opt(
                api_set_obj  => $Api_Set_Obj,
                option_value => 'off'
            );
            if ( !$result ) {
                die("Disable system wide operation failed");
            }
        } ## end if ($change_rlw_state_during_resync...

        if ($add_disk_during_resync) {
            $Test->step("Adding disk during resync");
            $self->_add_disk_during_resync (
                          group_name => $group_name,
            );
        }
        elsif ($change_rg_during_resync) {
            $Test->step("Change raid group during resync");
             $self->_change_raid_group_during_resync();
        }
        elsif ($change_plex_state_during_resync) {
            $Test->step("Change plex state during resync");
             $self->_change_plex_state_during_resync(
                plex_name => $plex_name,    
            );
        }
        elsif ($change_aggr_state_during_resync) {
            $Test->step("Change aggr state during resync");
             $self->_change_aggr_state_during_resync();
        }
        elsif ($generate_io_during_resync) {
            $Test->step("Run heavy IO during resync");
            $self->_generate_io_during_resync();
        } elsif ($aggr_undestroy_during_resync) {
            $Test->step("Destroy and undestroy aggr during resync");
            $self->_aggr_undestroy_during_resync();
        }
        sleep(10); # After aggr/plex state is changed resync resumes agains
    } ## end else [ if (!RaidSqa_check_plex_resyncing...

    #-------------------------------------------------------
    # Wait for plex resyncing to complete.
    #-------------------------------------------------------
    $Test->step("Wait for resync to complete");
    RaidSqa_wait_for_plex_resync_completion(
        api_set_obj  => $Api_Set_Obj,
        plex_name    => $plex_name,
        resync_level => $resync_level
    );

# This code is added because sometimes resync is getting aborted in between and resuming after sometime
# and due to this scrub is failing to start; hence making double check after sleep of 60 sec.
    sleep(60);
    if (   RaidSqa_check_plex_resyncing(
            api_set_obj  => $Api_Set_Obj,
            plex_name    => $plex_name,
            resync_level => $resync_level
        )
        )
    {
        RaidSqa_wait_for_plex_resync_completion(
            api_set_obj  => $Api_Set_Obj,
            plex_name    => $plex_name,
            resync_level => $resync_level
        );
    }

} ## end sub _start_resync


################ _verify_error_detection ######################################
## @name :: _verify_error_detection
## @summary :: Check for resync and scrub related ems events and verifies total error injected and detected.
## @input :: ems_mark => Ems logreader start mark
## @input :: ems_event => List of ems events to check.
## @input :: dbn => List of dbn's to verify error detection and fixed block.
## @return :: die with error string in case of failure.
###############################################################################

sub _verify_error_detection {
    my $self = shift;
    my (%opts) = validate(@_, {         
            ems_mark  => { type => SCALAR },
            ems_event => { type => HASHREF },
            dbn       => { type => ARRAYREF },
        }
    );

    my $mark_ems      = $opts{'ems_mark'};
    my $ref_ems_event = $opts{'ems_event'};
    my $dbn           = $opts{'dbn'};
    my $Api_Set_Obj   = $self->{'api_set_obj'};
    my @dbn           = @$dbn;
    my $ems_event     = $ref_ems_event;

    my $ref_events = RaidSqa_ems_log_dump_by_type(
        api_set_obj => $Api_Set_Obj,
        timestamp   => $mark_ems
    );
    my $ref_ids = RaidSqa_ems_log_dump_by_id(
        api_set_obj => $Api_Set_Obj,
        timestamp   => $mark_ems
    );
    
    #code of raid lost write flag
    my $value_raid_lost_write = RaidSqa_get_options(
        api_set_obj => $Api_Set_Obj,
        option_name => $self->{system_opt_lw}
    );

    foreach my $feature  (keys %{$ems_event})
    {
        if ($feature =~ /detect/i) {
            next;
        }
        my $event_ids = $ref_events->{$ems_event->{$feature }->{event_detect}};
        
        if ($event_ids) {
            foreach my $eventid (@$event_ids) {
                my $dbn = $ref_ids->{$eventid}->{'stripe'};
                if ( grep( $dbn, @dbn ) ) {
                    $Log->comment("Event - ".$ems_event->{$feature }->{event_detect}." found at dbn - $dbn");

                    # Check if event got fixed at same dbn
					if (exists($ems_event->{$feature }->{event_fix})) {
						my $event_fix = $ems_event->{$feature }->{event_fix};
						my $key;
						if ($event_fix eq 'raid.mirror.readerr.rewrite') {
							$key = "blockNum";
						} else {
							$key = "stripe_num";
						}
						my $ret       = RaidSqa_ems_log_check(
							api_set_obj => $Api_Set_Obj,
							eventtype   => $event_fix,
							key         => $key,
							value       => $dbn,
							timestamp   => $mark_ems
						);
						if ( !$ret ) {
							die( "Check for ems event - $event_fix failed at dbn - $dbn");
						}
						else {
							$Log->comment( "Ems event - $event_fix detected at dbn - $dbn");
						}
						my $count = $ems_event->{detect}->{count};
						$ems_event->{detect}->{count} = $count + 1;
					}
                }
                else {
                    die("Check for ems event - $feature failed; Expected dbn - @dbn Actual dbn - $dbn"
                    );
                }
            } ## end foreach my $eventid (@$event_ids)
        } ## end if ($event_ids)
        #check for other events occurrence  if defined.
        if(!defined $ems_event->{$feature}->{event_other}->[0] || $value_raid_lost_write eq "off") {
            next;
        }
        my @all_events = @{$ems_event->{$feature}->{event_other}};
        foreach my $event_id (@all_events) {
            my $event_count = 0;
            foreach my $event (keys %$ref_events) {
                if ($event eq $event_id) {
                    $event_count++;
                    last;
                }
            }
            if ($event_count < 1) {
                die ("Failed to detect ems event - $event_id");
            }
        } 

    } ## end foreach my $event ($ems_event...

} ## end sub _verify_error_detection


################ _add_disk_during_resync ##########################################
## @name :: _add_disk_during_resync
## @summary :: Add disk to aggr during resync.
## @return :: die with error string in case of failure.
###############################################################################
sub _add_disk_during_resync {
    my $self = shift;

    my (%opts) = validate(
        @_,
        {   
            group_name => { type => SCALAR, },
        }
    );

    my $group_name  = $opts{'group_name'};

    my $Api_Set_Obj =  $self->{'api_set_obj'};
    my $test        = $self->{'test'};

    my $filer_vol   = $self->{input_hash}->{'filer_vol'};
    my $raid_size   = $self->{input_hash}->{'raid_size'};

    my $disk_add_variant = $self->{tc_data}->{'disk_add_variant'};
    my $result;
    my @disk_list;
    my %plex_hash;

   if (!defined $disk_add_variant) {
        $disk_add_variant = 'generic';      
   }
    my $test_group = $group_name;
    $plex_hash{'group_name'} = $test_group;
    
    my @spare_pool0_disks = RaidSqa_get_disk_list_by_pool( #change this
        api_set_obj => $Api_Set_Obj,
        pool_name   => "Pool0",
        spare_only  => "yes"
    );

    my @spare_pool1_disks = RaidSqa_get_disk_list_by_pool(
        api_set_obj => $Api_Set_Obj,
        pool_name   => "Pool1",
        spare_only  => "yes"
    );
    my (@spare_disks,$no_of_spares);
    my $no_of_disk = 1; 
    if($disk_add_variant eq 'disks') {
        $no_of_disk = 2;
    }

    $no_of_spares = push (@spare_disks, @spare_pool0_disks, @spare_pool1_disks);

    if ( int($no_of_spares/2) < $no_of_disk  ) {
        die ("RaidErrResync:: _add_disk_during_resync: not enough disk to add in raid group");
    }

    #get existing disk info 
    my @disk = RaidSqa_get_disk_list(
            api_set_obj => $Api_Set_Obj,
            group_name  => $test_group
    );
    if ( !@disk ) {
            die( "$test: RaidErrResync:: _add_disk_during_resync failed to get disk from raidgroup: $test_group"
            );
            push( @disk_list, $disk[0] );
    }


    my $existing_disk_info = $disk[0];
    my $storage_type = RaidSqa_get_disk_storage_type ( 
                                            api_set_obj => $Api_Set_Obj,
                                            disk_name => $existing_disk_info,
                        );
    my $disk_size =  RaidSqa_get_disk_used_size ( 
                                            api_set_obj => $Api_Set_Obj,
                                            disk_name => $existing_disk_info,
                        );


    #get_disks_by_varient
    my ($curent_disk_size, $curent_disk_st_type, @disk_to_add );
    foreach my $disk (@spare_disks) {
            if(lc($disk_add_variant) eq 'largesize') {
                $curent_disk_size =  RaidSqa_get_disk_used_size ( 
                                            api_set_obj => $Api_Set_Obj,
                                            disk_name => $disk,
                                     );
                if ($curent_disk_size >$disk_size) {
                    push (@disk_to_add, $disk);
                }
            }
            if(lc($disk_add_variant) eq 'diffstoragetype') {
                $curent_disk_st_type =  RaidSqa_get_disk_storage_type ( 
                                            api_set_obj => $Api_Set_Obj,
                                            disk_name => $disk,
                                        );
                if ($curent_disk_st_type ne $storage_type) {
                    push (@disk_to_add, $disk);
                }
            }
    }
    if(lc($disk_add_variant) eq 'generic' || lc($disk_add_variant) eq 'disks'){
        @disk_to_add = @spare_disks;
        $Log->comment("disk_to_add: ". Dumper( \@disk_to_add));
    }

    if ( int(@disk_to_add/2) < $no_of_disk  ) {
        die ("$test: RaidErrResync:: _add_disk_during_resync: not enough disk to add in raid group");
    }

    #parse group name
    my $grp;
    if ($test_group =~ /\/.*\/.*\/(.*)/) {
        $grp = $1;
    }
    if (!$grp) {
        die("Could not get group id");
    } else {
        $Log->comment("Group id : $grp");
    }

    my $output;
    my @disk_list0 = splice(@disk_to_add,0,$no_of_disk);
    my @disk_list1 = splice(@disk_to_add,0,$no_of_disk);
    my $raid_size_value =  $raid_size + $no_of_disk;

    RaidSqa_aggr_options ( api_set_obj => $Api_Set_Obj,
                            filer_vol   => $filer_vol,
                            options     => 'raidsize',
                            value       => $raid_size_value,
                        );
   ($output, $result) = RaidSqa_aggr_add(
                                    api_set_obj => $Api_Set_Obj,
                                    filer_vol   => $filer_vol,
                                    group_name  => $grp,
                                    disk_list => \@disk_list0,
                                    disk_list1 => \@disk_list1,
                                    options    => "-f",
                            );

    if ( !$result ) {
        die("$test:RaidErrResync.pm :: _add_disk_during_resync: Failed to Add disk to Raid Group: $test_group, Error: $output");                    
    }


    # need to add code for other variants
}

################ _change_raid_group_during_resync ##########################################
## @name :: _change_raid_group_during_resync
## @summary :: Change raidgroup of aggr from raid4 to raid_dp and vice-versa during resync.
## @return :: die with error string in case of failure.
###############################################################################
sub _change_raid_group_during_resync {
    my $self = shift;

    my $Api_Set_Obj = $self->{'api_set_obj'};
    my $filer_vol = $self->{input_hash}->{'filer_vol'};
    

    #-------------------------------------------------------
    # Get raidgroup of the volume
    #-------------------------------------------------------
    my $original_raid_type =
      RaidSqa_get_aggr_options(api_set_obj  => $Api_Set_Obj, filer_vol => $filer_vol, options => 'raidtype');
    $Log->comment("RaidErrResync.pm: raidtype of $filer_vol - $original_raid_type");
    
    my $new_raid_type = ($original_raid_type eq "raid_dp" ? "raid4" : "raid_dp");

    #-------------------------------------------------------
    # Convert raidtype for filer_vol
    #-------------------------------------------------------
    $Log->comment("RaidErrResync.pm: Convert raidtype $original_raid_type to $new_raid_type");
    RaidSqa_aggr_options(api_set_obj  => $Api_Set_Obj,
                         filer_vol => $filer_vol,
                         options   => 'raidtype',
                         value     => $new_raid_type);
    sleep(30);

   RaidSqa_wait_for_reconstruction_completion(api_set_obj  => $Api_Set_Obj,
                                                   filer_vol => $filer_vol) if ($new_raid_type eq "raid_dp");

    my $raid_type =
          RaidSqa_get_aggr_options(api_set_obj  => $Api_Set_Obj, filer_vol => $filer_vol, options => 'raidtype');

    if ($raid_type eq $new_raid_type) {
        $Log->comment(" raidtype changed from $original_raid_type to $new_raid_type for vol/aggr $filer_vol");
    } else {
        die("Failed to change raidtype for vol/aggr $filer_vol Expected - $new_raid_type; Actual - $raid_type");
    }
    # need to add code for other variants
}

################ _change_plex_state_during_resync #############################
## @name :: _change_plex_state_during_resync
## @summary :: Change plex state from offline to online and vice-versa during resync.
## @input :: plex_name -> Name of the plex whose state is to be changed.
## @return :: die with error string in case of failure.
###############################################################################

sub _change_plex_state_during_resync {
    my $self = shift;

    my (%opts) = validate(@_,
        { 
        plex_name => { type => SCALAR },
        }
    );

    my $Api_Set_Obj = $self->{api_set_obj};
    my $plex_name = $opts{plex_name};

    # Get current plex state
    my $original_plex_state = RaidSqa_check_plex_state_name(api_set_obj=>$Api_Set_Obj, plex_name => $plex_name, state_name=>"online");

    my $new_plex_state = ($original_plex_state == 1 ? "offline" : "online");
    my $result;
    my $ems_mark = RaidSqa_ems_log_get_mark( api_set_obj => $Api_Set_Obj );

    $Log->comment("RaidErrResync.pm: Change plex $plex_name state");
    # Change the plex state
    if ($new_plex_state eq "offline") {
        $Log->comment("RaidErrResync.pm: Offline plex $plex_name state");
        $result = RaidSqa_plex_offline(api_set_obj=>$Api_Set_Obj,plex_name => $plex_name);
    } else {
        $Log->comment("RaidErrResync.pm: Online plex $plex_name state");
        $result = RaidSqa_plex_online(api_set_obj=>$Api_Set_Obj,plex_name => $plex_name);
    }
    
    if ($result) {
        $Log->comment(" plexstate changed for plex $plex_name");
    } else {
        die("Failed to change plexstate for plex $plex_name Expected - 1 ; Actual - 0");
    }

    sleep(5);
###ems marker

    my $ref_events = RaidSqa_ems_log_dump_by_type(  api_set_obj => $Api_Set_Obj,
                                                    timestamp   => $ems_mark
                                                    );

    my $event_id = 'raid.rg.resync.aborted';
    my $event_found= 0;
    foreach my $event (keys %$ref_events) {
                if ($event eq $event_id) {
                    $event_found = 1;
                    last;
                }
    }
    if ($event_found) {
        $Log->comment("Event:$event_id found in ems log");
    } else {
        die ("Event:$event_id did not occur in ems log");
    }

    $Log->comment("RaidErrResync.pm: Change plex $plex_name state to original state");
    # Change the plex state to original state
    if ($new_plex_state eq "offline") {
        $Log->comment("RaidErrResync.pm: Online plex $plex_name state");
        $result = RaidSqa_plex_online(api_set_obj=>$Api_Set_Obj,plex_name => $plex_name);
    } else {
        $Log->comment("RaidErrResync.pm: Offline plex $plex_name state");
        $result = RaidSqa_plex_offline(api_set_obj=>$Api_Set_Obj,plex_name => $plex_name);
    }

    if ($result) {
        $Log->comment(" plex state changed for plex $plex_name");
    } else {
        die("Failed to change plex state for plex $plex_name Expected - 1 ; Actual - 0");
    }
    sleep(30); # sleep for sometime so that resync starts again.
}

################ _change_aggr_state_during_resync #############################
## @name :: _change_aggr_state_during_resync
## @summary :: Change aggr state from offline to online and vice-versa during resync.
## @return :: die with error string in case of failure.
###############################################################################
sub _change_aggr_state_during_resync {
    my $self = shift;

    my $Api_set_obj = $self->{'api_set_obj'};
    my $filer_vol   = $self->{input_hash}->{'filer_vol'};
    my $aggr_type = $self->{input_hash}->{aggr};

    # Get current aggr state
    my $original_aggr_state = RaidSqa_check_aggr_state_name(api_set_obj=>$Api_set_obj, filer_vol => $filer_vol, aggr_type=>$aggr_type, state_name=>"online");

    my $new_aggr_state = ($original_aggr_state == 1 ? "offline" : "online");
    my $result;
    my $ems_mark = RaidSqa_ems_log_get_mark( api_set_obj => $Api_set_obj );

    $Log->comment("RaidErrResync.pm: Change aggr $filer_vol state");
    # Change the aggr state
    if ($new_aggr_state eq "offline") {
        $Log->comment("RaidErrResync.pm: Offline aggr $filer_vol state");
        $result = RaidSqa_aggr_offline(api_set_obj=>$Api_set_obj,filer_vol => $filer_vol,aggr_type => $aggr_type);
    } else {
        $Log->comment("RaidErrResync.pm: Online aggr $filer_vol state");
        $result = RaidSqa_aggr_online(api_set_obj=>$Api_set_obj,filer_vol => $filer_vol,aggr_type => $aggr_type);
    }
    
    if ($result) {
        $Log->comment(" aggr state_name changed for vol/aggr $filer_vol");
    } else {
        die("Failed to change aggr state_name for vol/aggr $filer_vol Expected - 1 ; Actual - 0");
    }

    sleep(5);
    my $ref_events = RaidSqa_ems_log_dump_by_type(  api_set_obj => $Api_set_obj,
                                                    timestamp   => $ems_mark
                                                    );
    my $event_id = 'raid.rg.resync.aborted';
    my $event_found= 0;
    foreach my $event (keys %$ref_events) {
                if ($event eq $event_id) {
                    $event_found = 1;
                    last;
                }
    }
    if ($event_found) {
        $Log->comment("Event:$event_id found in ems log");
    } else {
        die ("Event:$event_id did not occur in ems log");
    }


    $Log->comment("RaidErrResync.pm: Change aggr state to original state");
    # Change the plex state to original state
    if ($new_aggr_state eq "offline") {
        $Log->comment("RaidErrResync.pm: Online aggr $filer_vol state");
        $result = RaidSqa_aggr_online(api_set_obj=>$Api_set_obj,filer_vol => $filer_vol,aggr_type => $aggr_type);
    } else {
        $Log->comment("RaidErrResync.pm: Offline aggr $filer_vol state");
        $result = RaidSqa_aggr_offline(api_set_obj=>$Api_set_obj,filer_vol => $filer_vol,aggr_type => $aggr_type);
    }

    if ($result) {
        $Log->comment(" aggr state_name changed for vol/aggr $filer_vol");
    } else {
        die("Failed to change aggr state_name for vol/aggr $filer_vol Expected - 1 ; Actual - 0");
    }
    sleep(30); # sleep for sometime so that resync starts again.
}

################ _generate_io_during_resync #############################
## @name :: _generate_io_during_resync
## @summary :: Run hammer 10 times to create heavy traffic
## @return :: None
###############################################################################
sub _generate_io_during_resync {
    my $self = shift;

    my $Api_set_obj = $self->{'api_set_obj'};
    my $filer_vol   = $self->{input_hash}->{'filer_vol'};
    my $file_name   = "hammer_file";

    #-------------------------------------------------------
    # Start IO(hammer) on test volume
    #-------------------------------------------------------
    $Log->comment("Start hammer on vol/aggr $filer_vol during resync");
    for (my $count = 0;$count < 10 ;$count++) {
        RaidSqa_hammer_start(
            api_set_obj => $Api_set_obj,
            filer_vol   => $filer_vol,
            file_name   => "$file_name"."$count",
        );
    }
}

################ _flush_cache #############################
## @name :: _flush_cache
## @summary :: Flush the cache after file create.
## @return :: die with error string in case of failure.
###############################################################################
sub _flush_cache {
    my $self  =  shift;
    my $Api_Set_Obj  = $self->{'api_set_obj'};
    my $filer_vol  = $self->{input_hash}->{'filer_vol'}; 
    my $checksum_type  = $self->{input_hash}->{'checksum_type'};     
    my $cmd_cache = undef;

    if ( $checksum_type eq "advanced_zoned" ) {
        $cmd_cache = "priv set test;raid_flush_zcsv2_cache $filer_vol";
    }
    else {
        $cmd_cache = "priv set test;wafl flush cache";
    }
    foreach my $cmd ( "priv set test;sync", $cmd_cache ) {
        RaidSqa_filer_cmd(
            api_set_obj => $Api_Set_Obj,
            cmd         => $cmd,
            timeout     => $self->{CMDTIMEOUT}
        );
        sleep(5);
    }
    sleep(10);
}

################ _aggr_undestroy_during_resync #############################
## @name :: _aggr_undestroy_during_resync
## @summary :: Destroy and undestroy aggr while resync in progress.
## @return :: die with error string in case of failure.
###############################################################################
sub _aggr_undestroy_during_resync {
    my $self  =  shift;
    $Log->comment("Destroy vol/aggr $self");
    my $Api_Set_Obj  = $self->{'api_set_obj'};
    my $filer_vol    = $self->{input_hash}->{'filer_vol'};                          
    my $test         = $self->{test};

    #===============================================================
    # Destroy the volume
    $Log->comment("Destroy vol/aggr $filer_vol");
    RaidSqa_aggr_remove(api_set_obj => $Api_Set_Obj,
                        filer_vol => $filer_vol,
                        aggr_type => $self->{input_hash}->{aggr});

    if (RaidSqa_check_aggr_exist(api_set_obj      => $Api_Set_Obj,
                                 filer_vol => $filer_vol,
                                 aggr_type => $self->{input_hash}->{aggr})
      ) {
        die("Volume $filer_vol could not be removed");
    }
    sleep(60);

    #===============================================================
    # Undestroy the destroyed volume for reparity to start        
    $Log->comment("Undestroy vol/aggr $filer_vol");
    if (!RaidSqa_aggr_undestroy(api_set_obj => $Api_Set_Obj, filer_vol => $filer_vol, aggr_type => $self->{input_hash}->{aggr})) {
        die("Failed to undestroy the volume $filer_vol");
    }
    
    #===============================================================
    # Check for reparity on vol/aggr FILER_VOL        
    sleep(10);
    my @group_list = RaidSqa_get_group_list(api_set_obj=>$Api_Set_Obj, filer_vol=>$filer_vol);
    $Log->comment("Check if reparity is running on groups @group_list");
    foreach my $group (@group_list) {
        if (!RaidSqa_check_group_reparity_in_progress(api_set_obj => $Api_Set_Obj, group_name => $group)){
            die("Error: Failed as reaprity is not running on group $group");
        }
    }
    $Log->comment("Test $test :: Wait for reparity completion on vol/aggr $filer_vol");
    RaidSqa_wait_for_reparity_completion(api_set_obj=>$Api_Set_Obj, filer_vol => $filer_vol);

	$Log->comment("Test $test :: Run walfiron to clear wafl-inconsistent flag");
	RaidSqa_aggr_wafliron_start(api_set_obj=>$Api_Set_Obj, filer_vol => $filer_vol, aggr_type => $self->{input_hash}->{aggr});
	while (RaidSqa_aggr_wafliron_status(api_set_obj=>$Api_Set_Obj, filer_vol => $filer_vol,aggr_type => $self->{input_hash}->{aggr})) {
		sleep 30;
	}
}

################ run_verifyio #############################
## @name :: run_verifyio
## @summary :: Execute verifyio on filer_vol
## @return :: die with error string in case of failure.
###############################################################################
sub run_verifyio {
	my $self  =  shift;
	my %opts = validate(@_, {filer_vol => {type=>SCALAR, default=>$self->{input_hash}->{'filer_vol'}, optional=>1}});
	my $filer_vol = $opts{filer_vol};
	my $Api_Set_Obj  = $self->{'api_set_obj'};
	$Log->comment("run_verifyio: Info: Starting verify on aggr - $filer_vol");
	my $error = RaidSqa_aggr_verify_start(api_set_obj => $Api_Set_Obj,filer_vol=>$filer_vol);
	if (!$error ) {
		die("run_verifyio: Info: starting verify failed for aggr - $filer_vol");
	}
	sleep(30);
	$Log->comment("run_verifyio: Info: waiting for verify to complete on aggr - $filer_vol");
	RaidSqa_wait_for_verify_completion(api_set_obj => $Api_Set_Obj, filer_vol => $filer_vol);
	$Log->comment("run_verifyio: Info: completed run_verifyio on filer_vol - $filer_vol");
	return 1;
}

1;
