# Hey, emacs, this is basically -*-perl-*-
#$Id: //depot/prod/test/Rsierranevada/lib/RaidInfo_Cmode.pm#1 $

## @summary     Access routines all raid objects
##
## @description
##      This library provides access to all raid objects (aggregates,
##      volumes, plexes, raid groups, and disks) and internal values
##      associated with these objects. These values are obtained
##      through Ontap's priviledged raid_config info command, 
##      therefore none of the library functionswill depend on the 
##      UI commands.
##      
## @synopsis
##      use RaidInfo_Cmode;
##      my $r = new RaidInfo_Cmode( $c );
##      where $c is connection to filer or node and $r is a 
##      reference to a RaidInfo_Cmode object (described below.)
##
## @example
##      Before using this library, please run Example/testRaidInfo_Cmode.thpl.
##      It prints out all attributes that objects know about. 
##
## @usage
##
## <hr><center><b>RaidInfo_Cmode Object</b></center>
##
## <p>Creating a new object of type RaidInfo_Cmode queries
## the filer about all of its raid information and creates all
## of the objects of type Aggregate, Volume, Plex, Group and Disk.</p>
## <pre>
##    $r = new RaidInfo_Cmode( $c );
## </pre>
## <p>where $c is connection to filer or node and $r is a 
## reference to a RaidInfo_Cmode object. If you suspect or know 
## that the raid situation on the filer has changed, you need
## to refresh this object by executing $r->refresh().</p>
## <p>The following is a list of methods you can use on the
## Raid Info object:</p>
## <pre>
##    my %vols = $r->all_aggregates();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of all aggregate objects on the filer, keyed by aggregate name.
## <pre>
##    my $v = $r->root_aggregate();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns the name
## of the root aggregate object.
## <pre>
##    my $v = $r->aggregate("name");
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a reference to a aggregate object whose name matches string "name".
## <pre>
##    my %vols = $r->all_volumes();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of all volume objects on the filer, keyed by volume name.
## <pre>
##    my $v = $r->root_volume();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns the name
## of the root volume object.
## <pre>
##    my $v = $r->volume("name");
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a reference to a volume object whose name matches string "name".
## <pre>
##    my %plexes = $r->all_plexes();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of all plex objects on the filer, keyed by plex name.
## <pre>
##    my $p = $r->plex("name");
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a reference to a plex object whose name matches string "name".
## <pre>
##    my %groups = $r->all_raid_groups();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of all raid group objects on the filer, keyed by group name.
## <pre>
##    my $g = $r->raid_group("name");
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a reference to a raid group object whose name matches string "name".
## <pre>
##    my %disks = $r->all_disks();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of all disk objects on the filer, keyed by disk name.
## <pre>
##    my $d = $r->disk("name");
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a reference to a disk object whose name matches string "name".
## <pre>
##    my %disks = $r->my_disks();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all disks owned by this host, 
## keyed by disk name.
## <pre>
##    my %disks = $r->failed_disks();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all failed disks, keyed by disk name.
## <pre>
##    my %disks = $r->prefailed_disks();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all prefailed disks, keyed by disk name.
## <pre>
##    my %disks = $r->sdc_target_disks();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all SDC(sick disk copy) target disks, 
## keyed by disk name.
## <pre>
##    my %disks = $r->spare_disks();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all spare disks, keyed by disk name.
## <pre>
##    my %disks = $r->spare_disks_pool0();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all spare disks of pool0, keyed by disk name.
## <pre>
##    my %disks = $r->spare_disks_pool1();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all spare disksu of pool1, keyed by disk name.
## <pre>
##    my %disks = $r->partner_disks();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method returns
## a hash of disk objects for all partner disks, keyed by disk name.
## <pre>
##    $r->print_info();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method prints
## all of the information about the RaidInfo_Cmode object.
## <pre>
##    $r->refresh();
## </pre>
## where $r is an initialized RaidInfo_Cmode object. The method will refresh
## all the raid information on the host held by $r. This method will
## eliminate the need to destroy and recreate a new RaidInfo_Cmode object in
## order to get the latest raid information.
##
## <hr><center><b>Volume Object</b></center>
##
## <p>Creating a RaidInfo_Cmode object described above will automatically
## create all necessary Volume objects. You should never have to
## create this object directly. The following is a list of methods
## you can execute on an initialized Volume object:</p>
## <pre>
##    my %plexes = $v->all_plexes();
## </pre>
## where $v is a reference to a Volume object. The method returns
## a hash of all plex objects on that volume, keyed by plex name.
## <pre>
##    my $p = $v->plex("name");
## </pre>
## where $v is a reference to a Volume object. The method returns
## a reference to a plex object whose name matches string "name".
## <pre>
##    my $p = $v->plex_by_HA("HA_name");
## </pre>
## where $v is a reference to a Volume object. The method returns
## a reference to a plex object whose disks are on adapter specified
## by HA_name. Obvious warning: if you are running in a configuration
## where plexes share the disks on the same adapter (you should not!)
## this routing will return the first one it finds.
## <pre>
##    my %groups = $v->all_raid_groups();
## </pre>
## where $v is a reference to a Volume object. The method returns
## a hash of all raid group objects on that volume, keyed by raid group name.
## <pre>
##    my $g = $v->raid_group("name");
## </pre>
## where $v is a reference to a Volume object. The method returns
## a reference to a raid group object whose name matches string "name".
## <pre>
##    my %disks = $v->all_disks();
## </pre>
## where $v is a reference to a Volume object. The method returns
## a hash of all disk objects on that volume, keyed by disk name.
## <pre>
##    my $d = $v->disk("name");
## </pre>
## where $v is a reference to a Volume object. The method returns
## a reference to a disk object whose name matches string "name".
## <pre>
##    my $state = $v->state();
## </pre>
## where $v is a reference to a Volume object. The method returns
## the state of the volume.
## <pre>
##    my $mirror = $v->mirrored();
## </pre>
## where $v is a reference to a Volume object. The method returns
## the mirror state of the volume.
## <pre>
##    my $chksum = $v->checksum();
## </pre>
## where $v is a reference to a Volume object. The method returns
## the checksum type for the volume.
## <pre>
##    my $name = $v->name();
## </pre>
## where $v is a reference to a Volume object. The method returns
## volume name.
## <pre>
##    my $value = $v->parameter("key");
## </pre>
## where $v is a reference to a Volume object. The method returns
## the value of the volume parameter whose name is "key". Currently,
## Volume objects have the following parameters: quota_init,
## nosnapdir, tree, resyncsnaptime, wafl_mounted, diskroot,
## raidtree_type, snapmirrored, size, no_atime_update, plex_count,
## host, raidname, files_used, cksum, cksum_type, id, fsm_state,
## minra, blocks_used, wafl_online, name, files_max, mirror_state,
## blocks_max, type, nvfail, state, ignore_inconsist, disk_count,
## root, mirror_state_name, parent_id, state_name, raidsize, nosnap.
## <pre>
##    $v->print_info();
## </pre>
## where $v is a reference to a Volume object. The method prints
## all of the information about the volume.
##
## <hr><center><b>Plex Object</b></center>
##
## <p>Creating aRaidInfo_Cmode object described above will automatically
## create all necessary Plex objects. You should never have to
## create this object directly. The following is a list of methods
## you can execute on an initialized Plex object:</p>
## <pre>
##    my %groups = $p->all_raid_groups();
## </pre>
## where $p is a reference to a Plex object. The method returns
## a hash of all raid group objects for that plex, keyed by raid group name.
## <pre>
##    my $g = $p->raid_group("name");
## </pre>
## where $p is a reference to a Plex object. The method returns
## a reference to a raid group object whose name matches string "name".
## <pre>
##    my %disks = $p->all_disks();
## </pre>
## where $p is a reference to a Plex object. The method returns
## a hash of all disk objects for  that plex, keyed by disk name.
## <pre>
##    my $d = $p->disk("name");
## </pre>
## where $p is a reference to a Plex object. The method returns
## a reference to a disk object whose name matches string "name".
## <pre>
##    my $state = $p->state();
## </pre>
## where $p is a reference to a Plex object. The method returns
## the state of the plex.
## <pre>
##    $p->is_online();
## </pre>
## where $p is a reference to a Plex object. The method returns
## 1 if plex is online.
## <pre>
##    my $v = $p->parent();
## </pre>
## where $p is a reference to a Plex object. The method returns
## a reference to the parent volume object.
## <pre>
##    my $name = $p->name();
## </pre>
## where $p is a reference to a Plex object. The method returns
## plex name.
## <pre>
##    my $value = $p->parameter("key");
## </pre>
## where $p is a reference to a Plex object. The method returns
## the value of the plex parameter whose name is "key". Currently,
## Plex objects have the following parameters: online, fsm_state,
## outofdate, tree, resyncing, raid_name, raid_size, parent_id,
## id, type.
## <pre>
##    $p->print_info();
## </pre>
## where $p is a reference to a Plex object. The method prints
## all of the information about the plex.
##
## <hr><center><b>RaidGroup Object</b></center>
##
## <p>Creating RaidInfo_Cmode object described above will automatically
## create all necessary RaidGroup objects. You should never have to
## create this object directly. The following is a list of methods
## you can execute on an initialized RaidGroup object:</p>
## <pre>
##    my %disks = $g->all_disks();
## </pre>
## where $g is a reference to a RaidGroup object. The method returns
## a hash of all disk objects for that raid group.
## <pre>
##    my $d = $g->disk("name");
## </pre>
## where $g is a reference to a RaidGroup object. The method returns
## a reference to a disk object whose name matches string "name".
## <pre>
##    my $state = $g->state();
## </pre>
## where $g is a reference to a RaidGroup object. The method returns
## the state of the raid group.
## <pre>
##    my $p = $g->parent();
## </pre>
## where $g is a reference to a RaidGroup object. The method returns
## a reference to the parent plex object.
## <pre>
##    my $name = $g->name();
## </pre>
## where $g is a reference to a RaidGroup object. The method returns
## raid group name.
## <pre>
##    my $value = $g->parameter("key");
## </pre>
## where $g is a reference to a RaidGroup object. The method returns
## the value of theraid group parameter whose name is "key". Currently,
## RaidGroup objects have the following parameters: fsm_state,
## tree, raid_name, raid_size, parent_state, parent_id, parent_type,
## id, type.
## <pre>
##    $g->print_info();
## </pre>
## where $g is a reference to a RaidGroup object. The method prints
## all of the information about the raid group.
##
## <hr><center><b>Disk Object</b></center>
##
## <p>CreatingRaidInfo_Cmode object described above will automatically
## create all necessary Disk objects. You should never have to
## create this object directly. The following is a list of methods
## you can execute on an initialized Disk object:</p>
## <pre>
##    my $p = $d->parent();
## </pre>
## where $d is a reference to a Disk object. The method returns
## a reference to the parent object.
## <pre>
##    my $name = $d->name();
## </pre>
## where $d is a reference to a Disk object. The method returns
## the disk name.
## <pre>
##    my $value = $d->parameter("key");
## </pre>
## where $d is a reference to a Disk object. The method returns
## the value of the disk parameter whose name is "key". Currently,
## RaidGroup objects have the following parameters: used_sz,
## parity_disk, tree, bps, avl_waflblks, parent_state, fdr_pending,
## bay, size, waflblks, broken, avl_mb, serial, host_adapter,
## host, zero, raid_size, spare, plex_id, id, used_blks, fsm_state,
## no_parity_vol, serialno, vendor, filesys, name, chan_name, used,
## type, model, raid_name, avl_blks, chan_suffix, chan_suffix2,
## pending, parent_id, fw_revision, diskid, shelf, parent_type.
## <pre>
##    $d->print_info();
## </pre>
## where $d is a reference to a Disk object. The method prints
## all of the information about the disk.
##
##
## @dependencies 
## This is work in progress. 
## If you find any problems, let me know!
##      
##
## @author      skg@netapp.com
##
## @status      TESTME

use strict;
use Server;
use vars ('@ISA','@EXPORT');

require Exporter;

package RaidInfo_Cmode;

@RaidInfo_Cmode::ISA = ( 'Exporter' );
@RaidInfo_Cmode::EXPORT = ('all_aggregates', 'aggregate', 'all_volumes', 'volume', 
		     'all_plexes', 'plex', 'all_raid_groups', 'raid_group', 
		     'all_disks', 'disk', 'failed_disks', 'spare_disks', 'prefailed_disks',
		     'sdc_target_disks', 'spare_disks_pool0', 'spare_disks_pool1', 'my_disks', 
		     'partner_disks', 'print_info', 'root_aggregate', 
		     'root_volume', 'refresh');

my $RaidInfo_TIMEOUT = 300;

sub new ($){
#
#  new - a function to create an uninitialized RaidInfo_Cmode object
#
  my( $proto ) = shift;
  my( $c ) = @_;
  my( $class ) = ref( $proto ) || $proto;
  my( $self ) = {};

  $self = {
	   'AGGRS' => {},
	   'VOLS' => {},
	   'ROOT_AGGR' => {},
	   'ROOT_VOL' => {},
	   'PLEXES' => {},
	   'RAIDGRPS' => {},
	   'DISKS' => {},
	   'MY_DISKS' => {},
	   'SPARES' => {},
	   'SPARES_POOL0' => {},
	   'SPARES_POOL1' => {},
	   'FAILED' => {},
	   'PREFAILED' => {},
	   'SDC_TARGET' => {},
	   'PARTNER' => {},
	   'NAME' => $c,
	  };

  bless( $self, $class );
  return $self;
}

sub DESTROY {
  my( $self ) = shift;
}

sub _destroy_raidobj {
#
# call this function if you want to destroy all the raid objects
# held by $self. do not call $self->SUPER::DESTROY() as that will
# destroy the RaidInfo_Cmode object itself and you have to recreate
# a new RaidInfo_Cmode object.
#
  my( $self ) = shift;

  foreach my $key ( keys %{$self} ){

    # leave this key alone. we still want the connspec to the filer
    if ( $key eq "NAME" ){
      next;
    }

    if (ref( $self->{$key} )){
      foreach my $k ( keys %{$self->{$key}} ){
        $self->{$key}->{$k}->DESTROY();

        # even though the object has been destroyed, we still
        # need to delete the hashkey containing the name of each
        # of the raid objects.
        delete $self->{$key}->{$k};
      }
    }
    else {
      $self->{ROOT_AGGR} = {};
      $self->{ROOT_VOL}  = {};
    }
  }
}

sub _get_aggr_info {
#
# _get_aggr_info - get and process aggregate information
#
  my( $self ) = shift;

  # XXX/tapl is there a better way to do this than indicating to Perl we
  #          require this package here in this method explicitly?
  #          without this line, Perl will croak when we do
  #          'my( $aggr ) = new RaidInfo_Cmode::Aggregate( $line );'
  package RaidInfo_Cmode::Aggregate;

  my( $c ) = ($self->{NAME})->{hostp}->{hostname};
  my $server= new Server;
  $server->set_execution_server(-Server => $c);
  my $output = $server->cli(cmd=>"run local \"priv set -q diag;raid_config info listaggr\" ");
  my( @info ) = split( /\n/, $output );
  foreach my $line ( @info ){
    chomp $line;
    if ($line =~ /host=0/){
      my( $aggr ) = new RaidInfo_Cmode::Aggregate( $line );

      # If this method is called on a console connection,
      # the first line passed in may be an echo of the command we sent,
      # in which case the RaidGroup object will not have a name
      if ( !defined($aggr->name()) ){
	next;
      }
      $self->{AGGRS}->{$aggr->name()} = $aggr;
      $self->{ROOT_AGGR} = $aggr->name() if ($line =~ /root=1/);
    }
    elsif ($line =~ /host=1/){
      #this is partner's aggregate, do nothing
    }
    # the command may be echoed back
    elsif ($line !~ /priv set -q diag|raid_config info listvol/) {
      #something is seriously wrong if we ended up here
      print "This line:\n$line\n does not belong to this cluster pair!\n";
      die("Got Unexpected Output");
      return 1;
    }
  }
}

sub _get_vol_info {
#
# _get_vol_info - get and process volume information
#
  my( $self ) = shift;

  package RaidInfo_Cmode::Volume;

  my( $c ) = ($self->{NAME})->{hostp}->{hostname};
  my $server= new Server;
  $server->set_execution_server(-Server => $c);
  my $output = $server->cli(cmd=>"run local \"priv set -q diag;raid_config info listvol\" ");
  my( @info ) = split( /\n/, $output );
  foreach my $line ( @info ){
    chomp $line;
    if ($line =~ /host=0/){
      my( $vol ) = new RaidInfo_Cmode::Volume( $line );
      if ( !defined($vol->name()) ){
	next;
      }
      $self->{VOLS}->{$vol->name()} = $vol;
      $self->{ROOT_VOL} = $vol->name() if ($line =~ /root=256/);
      my( $aggr_name ) = $vol->parameter( "tree" );
      my( $aggr ) = $self->{AGGRS}->{$aggr_name};
      $aggr->add_volume( $vol ) if (defined $aggr);
    }
    elsif ($line =~ /host=1/){
      #this is partner's volume, do nothing
    }
    # the command may be echoed back
    elsif ($line !~ /priv set -q diag|raid_config info listvol/) {
      #something is seriously wrong if we ended up here
      print "This line:\n$line\n does not belong to this cluster pair!\n";
      die("Got Unexpected Output");
      return 1;
    }
  }
}

sub _get_plex_info {
#
# _get_plex_info - get and process plex information
#
  my( $self ) = shift;

  package RaidInfo_Cmode::Plex;

  my( $c ) = ($self->{NAME})->{hostp}->{hostname};
  my $server= new Server;
  $server->set_execution_server(-Server => $c);
  my $output = $server->cli(cmd=>"run local \"priv set -q diag;raid_config info listplex\" " );
  my( @info ) = split( /\n/, $output );
  foreach my $line ( @info ){
    chomp $line;
    my( $plex ) = new RaidInfo_Cmode::Plex( $line );
    if ( !defined($plex->name()) ){
      next;
    }
    $self->{PLEXES}->{$plex->name()} = $plex;
    my( $junk, $aggr_name ) = split( /\//, $plex->{raid_name});
    my( $aggr ) = $self->{AGGRS}->{$aggr_name};
    $aggr->add_plex( $plex ) if (defined $aggr);
  }
}

sub _get_raidgrp_info {
#
# _get_raidgrp_info - get and process raid informatioin
#
  my( $self ) = shift;

  package RaidInfo_Cmode::RaidGroup;

  my( $c ) = ($self->{NAME})->{hostp}->{hostname};
  my $server= new Server;
  $server->set_execution_server(-Server => $c);
  my $output = $server->cli(cmd=>"run local \"priv set -q diag;raid_config info listgroup\" ");
  my( @info ) = split( /\n/, $output );
  foreach my $line ( @info ){
    
    # The group name is "rg0", not "rg0<CR>"
    # So, strip off trailing whitespaces, don't just chomp
    $line=~s/\s+$//;
    my( $group ) = new RaidInfo_Cmode::RaidGroup( $line );
    if ( !defined($group->name()) ){
      next;
    }
    $self->{RAIDGRPS}->{$group->name()} = $group;
    my( $junk, $aggr_name, $plex_name ) = split( /\//, $group->{raid_name});
    my( $plex ) = $self->{PLEXES}->{"/".$aggr_name."/".$plex_name};
    $plex->add_raid_group( $group ) if (defined $plex);
    my ( $aggr ) = $plex->parent();
    $aggr->add_raid_group( $group ) if (defined $aggr);
  }
}

sub _get_disk_info {
#
# _get_disk_info - get and process disk information
#
  my( $self ) = shift;

  package RaidInfo_Cmode::Disk;

  my( $c ) = ($self->{NAME})->{hostp}->{hostname};
  my $server= new Server;
  $server->set_execution_server(-Server => $c);
  my $output = $server->cli(cmd=>"run local \"priv set -q diag;raid_config info listdisk\" ");
  my( @info ) = split( /\n/, $output );
  foreach my $line ( @info ){
    chomp $line;
    my( $disk ) = new RaidInfo_Cmode::Disk( $line );
    if ( !defined($disk->name()) ){
      next;
    }
    $self->{DISKS}->{$disk->name()} = $disk;
    if ( $line =~ /host=0/){
      $self->{MY_DISKS}->{$disk->name()} = $disk ;
    }
    if ( $disk->{"tree"} =~ /POOL:Pool0/ && $line =~ /host=0/){
      $self->{SPARES_POOL0}->{$disk->name()} = $disk ;
      $self->{SPARES}->{$disk->name()} = $disk ;
      $disk->set_parent( $self );
    }
    elsif ( $disk->{"tree"} =~ /POOL:Pool1/ && $line =~ /host=0/){
      $self->{SPARES_POOL1}->{$disk->name()} = $disk ;
      $self->{SPARES}->{$disk->name()} = $disk ;
      $disk->set_parent( $self );
    }
    elsif ( $disk->{"tree"} =~ /POOL:Pool/ && $line =~ /host=1/){
      $self->{PARTNER}->{$disk->name()} = $disk; 
    }      
    elsif ( $disk->{"tree"} eq "POOL:broken" ){ # can we check for broken=1 ?
      $self->{FAILED}->{$disk->name()} = $disk; 
      $disk->set_parent( $self );
    }
    elsif ( $disk->{"prefail"} eq 1 ){
      $self->{PREFAILED}->{$disk->name()} = $disk; 
      $disk->set_parent( $self );
    }
    elsif ( $disk->{"fsm_state"} eq "SDC_TARGET" ){
      $self->{SDC_TARGET}->{$disk->name()} = $disk; 
      $disk->set_parent( $self );
    }
    elsif ( $disk->{"tree"} eq "POOL:partner" ){
      $self->{PARTNER}->{$disk->name()} = $disk; 
    }
    else{
      my( $aggr ) = $self->{AGGRS}->{$disk->{tree}};
      $aggr->add_disk( $disk ) if (defined $aggr);
      my( $group, $plex );
      foreach $group ( keys %{$self->{RAIDGRPS}} ){
	if ( $disk->{parent_id} == $self->{RAIDGRPS}->{$group}->{id} ){
	  $self->{RAIDGRPS}->{$group}->add_disk( $disk );
	  $plex = $self->{RAIDGRPS}->{$group}->parent();
	  $plex->add_disk( $disk );
	} 
      }
    }
  }
}

sub all_aggregates {
  my( $self ) = shift;
  my( $key ) = $self->{AGGRS};

  # Check for the existance of the hash key and ensure that the
  # hash is not empty. If it's non-existant or empty, populate
  # the hashkey.
  if ( !defined($key) || !(%{$key}) ){
    #
    # XXX/tapl How to determine what private methods to call to
    #          gather the info.
    # An aggregate is the superset of volumes, plexes, raidgroups,
    # and disks. However, for a volume, it's the lowest denominator
    # hence it does not contain any plexes, raidgroups, or disks. So,
    # for a volume, you only call $self->_get_vol_info().
    #
    $self->_get_aggr_info();
    $self->_get_vol_info();
    $self->_get_plex_info();
    $self->_get_raidgrp_info();
    $self->_get_disk_info();
  }
  return %{$self->{AGGRS}};
}

sub root_aggregate {
  my( $self ) = shift;
  my( $key ) = $self->{ROOT_AGGR};

  if ( !defined($key) ){
    $self->_get_aggr_info();
    $self->_get_vol_info();
    $self->_get_plex_info();
    $self->_get_raidgrp_info();
    $self->_get_disk_info();
  }
  return $self->{ROOT_AGGR};
}

sub aggregate($) {
  my( $self ) = shift;
  my( $name ) = @_;
  my( $key ) = $self->{AGGRS}->{$name};

  if ( !defined($key) ){
    $self->_get_aggr_info();
    $self->_get_vol_info();
    $self->_get_plex_info();
    $self->_get_raidgrp_info();
    $self->_get_disk_info();
  }
  return $self->{AGGRS}->{$name};
}

sub all_volumes {
  my( $self ) = shift;
  my( $key ) = $self->{VOLS};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_vol_info();
  }
  return %{$self->{VOLS}};
}

sub root_volume {
  my( $self ) = shift;
  my( $key ) = $self->{ROOT_VOL};

  if ( !defined($key) ){
    $self->_get_vol_info();
  }
  return $self->{ROOT_VOL};
}

sub volume($) {
  my( $self ) = shift;
  my( $name ) = @_;
  my( $key ) = $self->{VOLS}->{$name};

  if ( !defined($key) ){
    $self->_get_vol_info();
  }
  return $self->{VOLS}->{$name};
}

sub all_plexes {
  my( $self ) = shift;
  my( $key ) = $self->{PLEXES};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_plex_info();
    $self->_get_raidgrp_info();
    $self->_get_disk_info();
  }
  return %{$self->{PLEXES}};
}

sub plex($) {
  my( $self ) = shift;
  my( $name ) = shift;
  my( $key ) = $self->{PLEXES}->{$name};

  if ( !defined($key) ){
    $self->_get_plex_info();
    $self->_get_raidgrp_info();
    $self->_get_disk_info();
  }
  return $self->{PLEXES}->{$name};
}

sub all_raid_groups {
  my( $self ) = shift;
  my( $key ) = $self->{RAIDGRPS};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_plex_info();
    $self->_get_raidgrp_info();
    $self->_get_disk_info();
  }
  return %{$self->{RAIDGRPS}};
}

sub raid_group($) {
  my( $self ) = shift;
  my( $name ) = shift;
  my( $key ) = $self->{RAIDGRPS}->{$name};

  if ( !defined($key) ){
    $self->_get_plex_info();
    $self->_get_raidgrp_info();
    $self->_get_disk_info();
  }
  return $self->{RAIDGRPS}->{$name};
}

sub all_disks {
  my( $self ) = shift;
  my( $key ) = $self->{DISKS};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{DISKS}};
}

sub disk($) {
  my( $self ) = shift;
  my( $name ) = shift;
  my( $key ) = $self->{DISKS}->{$name};

  if ( !defined($key) ){
    $self->_get_disk_info();
  }
  return $self->{DISKS}->{$name};
}

sub failed_disks {
  my( $self ) = shift;
  my( $key ) = $self->{FAILED};

  if (!defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{FAILED}};
}

sub prefailed_disks {
  my( $self ) = shift;
  my( $key ) = $self->{PREFAILED};

  if (!defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{PREFAILED}};
}

sub sdc_target_disks {
  my( $self ) = shift;
  my( $key ) = $self->{SDC_TARGET};

  if (!defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{SDC_TARGET}};
}

sub spare_disks {
  my( $self ) = shift;
  my( $key ) = $self->{SPARES};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{SPARES}};
}

sub spare_disks_pool0 {
  my ( $self ) = shift;
  my( $key ) = $self->{SPARES_POOL0};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{SPARES_POOL0}};
}

sub spare_disks_pool1 {
  my ( $self ) = shift;
  my( $key ) = $self->{SPARES_POOL1};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{SPARES_POOL1}};
}

sub my_disks {
  my ( $self ) = shift;
  my( $key ) = $self->{MY_DISKS};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{MY_DISKS}};
}

sub partner_disks {
  my ( $self ) = shift;
  my( $key ) = $self->{PARTNER};

  if ( !defined($key) || !(%{$key}) ){
    $self->_get_disk_info();
  }
  return %{$self->{PARTNER}};
}

sub print_info {         

  my( $self ) = shift;
  print "\n\nRaid Information for  ", $self->{NAME}, "\n\n";
  my( $key, $k );

  # need to populate all the keys since we are printing all the
  # hash keys.
  $self->refresh();

  foreach $key ( sort keys %{$self} ){
    # we don't care abt this key
    if ( $key eq "NAME" ){
      next;
    }

    print "$key\n";
    if (ref( $self->{$key} )){
      foreach $k ( keys %{$self->{$key}} ){
        print $self->{$key}->{$k}->name(), "  ";
      }
    }
    print "\n";
  }
  print "\n";
}

sub refresh {
  my( $self ) = shift;

  # destroy all the raid objects held by $self. updating the reference
  # to the new raid objects will cause a memory leak as the old objects
  # are still around.
  $self->_destroy_raidobj();

  # get and process aggregate information
  $self->_get_aggr_info();

  # get and process volume information
  $self->_get_vol_info();

  # get and process plex information
  $self->_get_plex_info();

  # get and process raid group information
  $self->_get_raidgrp_info();

  # get and process disk information
  $self->_get_disk_info();
}

package RaidInfo_Cmode::Aggregate;

@RaidInfo_Cmode::Aggregate::ISA = ( 'Exporter' );
@RaidInfo_Cmode::Aggregate::EXPORT = ('all_volumes', 'volume', 'all_plexes', 'plex', 
				'all_raid_groups', 'raid_group', 'all_disks', 
				'disk', 'mirrored', 'state', 'checksum',
				'name', 'parameter', 'print_info');
sub new {
#
#  new - a function to create an uninitialized Volume object
#  $line - a line of output from raid_config info listvol
#
  my( $proto ) = shift;
  my( $line ) = shift;
  my( $class ) = ref( $proto ) || $proto;
  my( $self ) = {};

  $self = {
	   'VOLS' => {},
	   'PLEXES' => {},
	   'RAIDGRPS' => {},
	   'DISKS' => {},
	  };

  my( @aggr_info ) = split( /,/, $line );
  my( $entry );
  foreach $entry ( @aggr_info ){
    my( $key, $value ) = split( /=/, $entry );
    $self->{$key} = $value;
  }    

  bless( $self, $class );
  return $self;
}

sub DESTROY {
  my( $self ) = shift;
}

sub add_volume($) {
  my( $self ) = shift;
  my( $vol ) = shift;
  $self->{VOLS}->{$vol->name()} = $vol;
  $vol->set_parent( $self );
}

sub add_plex($) {
  my( $self ) = shift;
  my( $plex ) = shift;
  $self->{PLEXES}->{$plex->name()} = $plex;
  $plex->set_parent( $self );
}

sub add_raid_group($) {
  my( $self ) = shift;
  my( $group ) = shift;
  $self->{RAIDGRPS}->{$group->name()} = $group;
}

sub add_disk($) {
  my( $self ) = shift;
  my( $disk ) = shift;
  $self->{DISKS}->{$disk->name()} = $disk;
}

sub all_volumes {
  my( $self ) = shift;
  return %{$self->{VOLS}};
}

sub volume($) {
  my( $self ) = shift;
  my( $name ) = shift;
  return $self->{VOLS}->{$name};
}

sub all_plexes {
  my( $self ) = shift;
  return %{$self->{PLEXES}};
}

sub plex($) {
  my( $self ) = shift;
  my( $name ) = shift;
  return $self->{PLEXES}->{$name};
}

sub plex_by_HA($) {
  my( $self ) = shift;
  my( $ha ) = shift;
  my %plexes = $self->all_plexes();
  foreach my $p ( values %plexes ) {
    my %disks = $p->all_disks();
    foreach my $d ( values %disks ){
      my $check = $d->parameter( 'host_adapter' );
      if ( "$check" eq "$ha" ){
	# we have found our match.
	return $p;
      }
    }
  }
  # we did not find a match
  return undef;
}

sub all_raid_groups {
  my( $self ) = shift;
  return %{$self->{RAIDGRPS}};
}

sub raid_group($) {
  my( $self ) = shift;
  my( $name ) = shift;
  return $self->{RAIDGRPS}->{$name};
}

sub all_disks {
  my( $self ) = shift;
  return %{$self->{DISKS}};
}

sub disk($) {
  my( $self ) = shift;
  my( $name ) = shift;
  return $self->{DISKS}->{$name};
}

sub mirrored {
  my( $self ) = shift;
  return $self->{mirror_state_name}
}

sub state {
  my( $self ) = shift;
  return $self->{state_name};
}

sub checksum {
  my( $self ) = shift;
  if ( $self->{cksum_type} eq "520byte" ){
    return "block checksums";
  }
  elsif ( $self->{cksum_type} eq "fixedVbns" ){
    return "zoned checksums";
  }
  else{
    return $self->{cksum_type};
  }
}

sub name {
  my( $self ) = shift;
  return $self->{name};
}

sub parameter($) {
  my( $self ) = shift;
  my( $key ) = shift;
  return $self->{$key};
}

sub print_info {
  my( $self ) = shift;
  print "\nAggregate  ",  $self->{name}, "\n\n";
  my( $key );
  foreach $key ( sort keys %{$self} ){
    next if ref( $self->{$key} );
    print "Key: '$key' Value: '$self->{$key}'\n";
  }
  print "\n";
}


package RaidInfo_Cmode::Volume;

@RaidInfo_Cmode::Volume::ISA = ( 'Exporter' );
@RaidInfo_Cmode::Volume::EXPORT = ('mirrored', 'state', 'checksum',
			     'parent', 'name', 'parameter', 'print_info');

sub new {
#
#  new - a function to create an uninitialized Volume object
#  $line - a line of output from raid_config info listvol
#
  my( $proto ) = shift;
  my( $line ) = shift;
  my( $class ) = ref( $proto ) || $proto;
  my( $self ) = {};

  $self = {};

  my( @vol_info ) = split( /,/, $line );
  my( $entry );
  foreach $entry ( @vol_info ){
    my( $key, $value ) = split( /=/, $entry );
    $self->{$key} = $value;
  }    

  bless( $self, $class );
  return $self;
}

sub DESTROY {
  my( $self ) = shift;
}

sub set_parent($) {
  my( $self ) = shift;
  my( $aggr ) = shift;
  $self->{PARENT} = $aggr;
}

sub mirrored {
  my( $self ) = shift;
  return $self->{mirror_state_name}
}

sub state {
  my( $self ) = shift;
  return $self->{state_name};
}

sub checksum {
  my( $self ) = shift;
  if ( $self->{cksum_type} eq "520byte" ){
    return "block checksums";
  }
  elsif ( $self->{cksum_type} eq "fixedVbns" ){
    return "zoned checksums";
  }
  else{
    return $self->{cksum_type};
  }
}

sub name {
  my( $self ) = shift;
  return $self->{name};
}

sub parent {
  my( $self ) = shift;
  return $self->{PARENT};
}

sub parameter($) {
  my( $self ) = shift;
  my( $key ) = shift;
  return $self->{$key};
}

sub print_info {
  my( $self ) = shift;
  print "\n\nVolume  ",  $self->{name}, "\n\n";
  my( $key );
  foreach $key ( sort keys %{$self} ){
    next if ref( $self->{$key} );
    print "Key: '$key' Value: '$self->{$key}'\n";
  }
  print "\n";
}


package RaidInfo_Cmode::Plex;

@RaidInfo_Cmode::Plex::ISA = ( 'Exporter' );
@RaidInfo_Cmode::Plex::EXPORT = ('all_raid_groups', 'raid_group', 'all_disks', 'disk', 'parameter',
			   'state', 'is_online', 'parent', 'name', 'print_info');

sub new {
#
#  new - a function to create an uninitialized Plex object
#
  my( $proto ) = shift;
  my( $line ) = shift;
  my( $class ) = ref( $proto ) || $proto;
  my( $self ) = {};

  $self = {
	   'RAIDGRPS' => {},
	   'DISKS' => {},
	  };

  my( @plex_info ) = split( /,/, $line );
  my( $entry );
  foreach $entry ( @plex_info ){
    my( $key, $value ) = split( /=/, $entry );
    $self->{$key} = $value;
  }    

  bless( $self, $class );
  return $self;
}

sub DESTROY {
  my( $self ) = shift;
}

sub add_raid_group($) {
  my( $self ) = shift;
  my( $group ) = shift;
  $self->{RAIDGRPS}->{$group->name()} = $group;
  $group->set_parent( $self );
}

sub add_disk($) {
  my( $self ) = shift;
  my( $disk ) = shift;
  $self->{DISKS}->{$disk->name()} = $disk;
}

sub set_parent($) {
  my( $self ) = shift;
  my( $aggr ) = shift;
  $self->{PARENT} = $aggr;
}

sub all_raid_groups {
  my( $self ) = shift;
  return %{$self->{RAIDGRPS}};
}

sub raid_group($) {
  my( $self ) = shift;
  my( $name ) = shift;
  return $self->{RAIDGRPS}->{$name};
}

sub all_disks {
  my( $self ) = shift;
  return %{$self->{DISKS}};
}

sub disk($) {
  my( $self ) = shift;
  my( $name ) = shift;
  return $self->{DISKS}->{$name};
}

sub state {
  my( $self ) = shift;
  if ( $self->{fsm_state} eq "NORMAL" ){
    return "normal";
  }
  else{
    return $self->{fsm_state};
  }
}

sub is_online {
  my( $self ) = shift;
  return $self->{online};
}

sub parent {
  my( $self ) = shift;
  return $self->{PARENT};
}

sub name {
  my( $self ) = shift;
  return $self->{raid_name};
}

sub parameter($) {
  my( $self ) = shift;
  my( $key ) = shift;
  return $self->{$key};
}

sub print_info {
  my( $self ) = shift;
  print "\n\nPlex  ",  $self->{raid_name}, "\n\n";
  my( $key );
  foreach $key ( sort keys %{$self} ){
    next if ref( $self->{$key} );
    print "Key: '$key' Value: '$self->{$key}'\n";
  }
  print "\n";
}

package RaidInfo_Cmode::RaidGroup;

@RaidInfo_Cmode::RaidGroup::ISA = ( 'Exporter' );
@RaidInfo_Cmode::RaidGroup::EXPORT = ('all_disks', 'disk', 'state', 'name', 'parameter', 
				'parent', 'print_info');

sub new {
#
#  new - a function to create an uninitialized RaidGroup object
#
  my( $proto ) = shift;
  my( $line ) = shift;
  my( $class ) = ref( $proto ) || $proto;
  my( $self ) = {};

  $self = {
	   'DISKS' => {},
	  };

  my( @group_info ) = split( /,/, $line );
  my( $entry );
  foreach $entry ( @group_info ){
    my( $key, $value ) = split( /=/, $entry );
    $self->{$key} = $value;
  }    

  bless( $self, $class );
  return $self;
}

sub DESTROY {
  my( $self ) = shift;
}

sub set_parent($) {
  my( $self ) = shift;
  my( $plex ) = shift;
  $self->{PARENT} = $plex;
}

sub add_disk($) {
  my( $self ) = shift;
  my( $disk ) = shift;
  $self->{DISKS}->{$disk->name()} = $disk;
  $disk->set_parent( $self );
}

sub all_disks {
  my( $self ) = shift;
  return %{$self->{DISKS}};
}

sub disk($) {
  my( $self ) = shift;
  my( $name ) = shift;
  return $self->{DISKS}->{$name};
}

sub state {
  my( $self ) = shift;
  if ( $self->{fsm_state} eq "NORMAL" ){
    return "normal";
  }
  else{
    return $self->{fsm_state};
  }
}

sub name {
  my( $self ) = shift;
  return $self->{raid_name};
}

sub parameter($) {
  my( $self ) = shift;
  my( $key ) = shift;
  return $self->{$key};
}

sub parent {
  my( $self ) = shift;
  return $self->{PARENT};
}

sub print_info {
  my( $self ) = shift;
  print "\n\nRaid Group  ",  $self->{raid_name}, "\n";
  my( $key );
  foreach $key ( sort keys %{$self} ){
    next if ref( $self->{$key} );
    print "  Key: '$key' Value: '$self->{$key}'\n";
  }
  print "\n";
}

package RaidInfo_Cmode::Disk;

@RaidInfo_Cmode::Disk::ISA = ( 'Exporter' );
@RaidInfo_Cmode::Disk::EXPORT = ('parent', 'name', 'parameter', 'print_info');

sub new {
#
#  new - a function to create an uninitialized Disk object
#
  my( $proto ) = shift;
  my( $line ) = shift;
  my( $class ) = ref( $proto ) || $proto;
  my( $self ) = {};

  my( @disk_info ) = split( /,/, $line );
  my( $entry );
  foreach $entry ( @disk_info ){
    my( $key, $value ) = split( /=/, $entry );
    $self->{$key} = $value;
  }    

  bless( $self, $class );
  return $self;
}

sub DESTROY {
  my( $self ) = shift;
}

sub set_parent($) {
  my( $self ) = shift;
  my( $name ) = shift;
  $self->{PARENT} = $name;
}

sub parent {
  my( $self ) = shift;
  return $self->{PARENT};
}

sub name {
  my( $self ) = shift;
  return $self->{name};
}

sub parameter($) {
  my( $self ) = shift;
  my( $key ) = shift;
  return $self->{$key};
}

sub print_info {
  my( $self ) = shift;
  print "\n\nDisk  ",  $self->{name}, "\n";
  my( $key );
  foreach $key ( sort keys %{$self} ){
    next if ref( $self->{$key} );
    print "  Key: '$key' Value: '$self->{$key}'\n";
  }
  print "\n";
}

1;

