#$Id:$
##
## @summary RaidLMUtils.pm - RAID SQA library for commands and utilities
#sub new:                                  This subroutine create object of RaidLMUtils.pm
#sub parse_tc_hash:                        This subroutine parses the hash by calling RaidLM_create_resources
#sub RaidLM_create_resources:              This subroutine creates aggregates and spare setup 
#sub RaidLM_initiate_evacuation:           This subroutine fails disk by calling _initiate_failure   
#sub RaidLM_verify_layout_correction:      This subroutine checks for layout correction if wait_for_completion is set to 1     
#										   and returns evacuation details, else wait_for_completion is set to 0 then returns   
#										   returns evacuation details before completion of evacuation                          
#sub RaidLM_wait_for_evacuation_completion:his subroutine checks for layout correction if wait_for_completion is set to 1     
#										   and returns evacuation details 
#sub RaidLM_destroy_aggr:                  This subroutine removes aggr
#sub RaidLM_delete_resource                This subroutine cleans up the setup
#sub _get_disk_list_from_same_carrier      This subroutine returns disk list from same carrier
#sub _symmetric_diff_array                 This subroutine returns symmetric diff. of two arrays
#sub _initiate_failure                     This subroutine fails a disk depending upon failure type and retruns related ems and Asup event.

package RaidLMUtils;

use strict;
use warnings;

# Module Imports
use vars qw(@ISA @EXPORT);



use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use RaidSqav3 qw(AUTOLOAD);
use RaidLW;
use NACL::APISet;
use Tharn qw($Log);
use Data::Dumper;
use Storable qw(dclone);
require Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(_initiate_failure _get_disk_list_from_same_carrier);

########### new ###################################################
# Function:    new
#
# Description: This subroutine create object of RaidLMUtils.pm
#
###################################################################
sub new {
    my $self = shift;
    my (%opts) = validate(@_, {
                    api_set_obj => {type => OBJECT, isa => 'NACL::APISet'},
                    tc_data     => {type=>HASHREF},                     
                    });
    $self = {};
    $self->{api_set_obj} = $opts{api_set_obj};
    $self->{tc_data} = $opts{tc_data};
	$Log->comment("RaidLMUtils: Creating object " );

    bless $self;
    $Data::Dumper::Indent=1;

    return $self;
}

########### parse_tc_hash ###################################################
# Function:    parse_tc_hash
#
# Description: This subroutine parses the hash by calling RaidLM_create_resources
#
##################################################################################
sub parse_tc_hash {
	my $self=shift;
	my (%opts) =validate(@_,
						   { node => {type => SCALAR, default => "local", optional => 1},
						   }
						);
	my $api_set_obj = $self->{api_set_obj};
	my $tc_data = $self->{tc_data};
	my $node = $opts{node};
	$Log->comment("RaidLMUtils: Input TC Data Hash ". Dumper $tc_data );
	$self->RaidLM_create_resources(node=>$node);
	$Log->comment("RaidLMUtils: TC hash after creating resource ".Dumper $tc_data);

}

########### RaidLM_create_resources ###################################################
# Function:    RaidLM_create_resources
#
# Description: This subroutine creates aggregates and spare setup 
#
#####################################################################################################

sub RaidLM_create_resources {
	
	my $self=shift;
	my (%opts) =validate(@_,
						   { node => {type => SCALAR, default => "local", optional => 1},
						   }
	                    );
	my $tc_data = $self->{tc_data};
	my $api_set_obj = $self->{api_set_obj};	
	my $node = $opts{node};

	
	my $aggr_arrey_ref = $tc_data->{aggregates};
	my $spare_disks_info = $tc_data->{spare_disk}; 
	my $undesirable_layout = $tc_data->{undesirable_layout}; 
	my $disk_count = 3;
	if (exists  $tc_data->{disk_count}) {
		$disk_count = $tc_data->{disk_count};
	}
	
	my @all_boab_disk = RaidSqa_get_disk_list_by_storage_type (api_set_obj => $api_set_obj, spare_only=>'yes',storage_type=>'MSATA',desirable_layout=>0);
	my @same_carrier_disks = _get_disk_list_from_same_carrier(api_set_obj => $api_set_obj, spare_only=>'yes');
	my @disk_list = _symmetric_diff_array(\@all_boab_disk, \@same_carrier_disks);	
	push (@same_carrier_disks, @disk_list);
	
	#create hash for each aggr
	foreach my $aggr_name (@$aggr_arrey_ref) {
		$self->{tc_data}->{$aggr_name}->{disks} = ();
	} 
	$self->{tc_data}->{spare}->{disks}=[];
	my $spare = $self->{tc_data}->{spare}->{disks};
	
	my @arr_aggr_unowned = @$aggr_arrey_ref;
	push (@arr_aggr_unowned, 'unowned');
	foreach my $arrayref (@{$spare_disks_info}) {
		#get both disk from carrier and assign to aggr or spare hash mentioned in spare_disk hash 
		my $disk1 = shift @same_carrier_disks;
		my $disk2 = shift @same_carrier_disks;
		if($arrayref->[0] =~ /spare/) {
			push (@{$spare}, $disk1);
		} elsif (grep /$arrayref->[0]/, @arr_aggr_unowned) {
			push (@{$self->{tc_data}->{$arrayref->[0]}->{disks}},$disk1);
		} else {
			die "RaidLMUtils: FAIL: $arrayref->[0] String mentioned is niether aggr name nor spare";
		}

		if($arrayref->[1] =~ /spare/) {
			push (@{$spare}, $disk2);
		} elsif (grep /$arrayref->[1]/, @arr_aggr_unowned) {
			push (@{$self->{tc_data}->{$arrayref->[1]}->{disks}},$disk2);
		} else {
			die "RaidLMUtils: FAIL: $arrayref->[1] String mentioned is niether aggr name nor spare";
		}
	}
	my @spare_disks = @{$self->{tc_data}->{spare}->{disks}};
	my $spare_ref = $self->{tc_data}->{spare};
	foreach my $disk_name (@{$spare_ref->{disks}}) {
			my $carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $disk_name);
			push (@{$spare_ref ->{carrier_disk}->{$carrier}}, $disk_name);
	}
	
	my $unowned_hash = $tc_data->{unowned};
	foreach my $disk_name (@{$unowned_hash->{disks}}) {
			my $carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $disk_name);
			push (@{$unowned_hash ->{carrier_disk}->{$carrier}}, $disk_name);
			push (@{$self->{tc_data}->{removed_disks}}, $disk_name);
	}


	foreach my $aggr_name (@$undesirable_layout) {
		if(@same_carrier_disks < 2) {
			die ("RaidLMUtils: FAIL: Not enough spare disk to create undesirable");
		}
		my @two_disks_in_carrier = splice( @same_carrier_disks,0,2);
		push (@{$self->{tc_data}->{$aggr_name}->{disks}},@two_disks_in_carrier);
	}

	foreach my $aggr_name (@{$aggr_arrey_ref}) {
		my $current_aggr = $tc_data->{$aggr_name};
		my @aggr_disks;
		if (defined $current_aggr->{disks}) {
			@aggr_disks = @{$current_aggr->{disks}};
		}		

		foreach my $disk_name ( @aggr_disks ) {
			my $carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $disk_name);
			push (@{$current_aggr->{carrier_disk}->{$carrier}}, $disk_name);
		}
		if (@aggr_disks < $disk_count) {			
			for (my $i=0; $i<=$#same_carrier_disks; $i++) {
				my $free_disk_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $same_carrier_disks[$i]);
				my @aggr_carriers = (keys %{{$current_aggr}->{carrier_disk}});
				if (!grep/$free_disk_carrier/, @aggr_carriers) {
					my @splice_disk = splice(@same_carrier_disks,$i,1);
					push (@{$current_aggr->{disks}},$splice_disk[0]);
					push (@{$current_aggr->{carrier_disk}->{$free_disk_carrier}}, $splice_disk[0]);
					#@aggr_disks = @{$current_aggr->{disks}};
					if (@{$current_aggr->{disks}} >= 3) {
						last;
					}
				}
			}		
		}
	}
	#stop raid lm service 
	$Log->comment("RaidLMUtils: Stop Raid LM service ");

	$self->RaidLM_enable_disable(enable=>0);
	$Log->comment("RaidLMUtils: Creating Aggregates ");
	foreach  my $aggr_name (@{$aggr_arrey_ref}) {
		$Log->comment("RaidLMUtils: Creating Aggregate: $aggr_name  ");
		my @disk_list = @{$tc_data->{$aggr_name}->{disks}};
		if ( !RaidSqa_aggr_create(api_set_obj   => $api_set_obj,
								filer_vol     => $aggr_name,
								raid_type     => 'raid_dp',								
								aggr_type     => '1',
								aggr_bits     =>'64',                          
								disk_list     => \@disk_list,
								node          => $node,
								allow_same_carrier => 1,
							   )) {
			die ( "RaidLMUtils: FAIL: Aggr $aggr_name was not created" );
			#$Log->comment( "RaidLMUtils: FAIL: Aggr $aggr_name was not created" );
		} else {
			$Log->comment("RaidLMUtils: Aggr $aggr_name created" );	
		}
	}	

	$Log->comment("RaidLMUtils: Removing ownership of unwanted disks @same_carrier_disks ");
	if(!_RaidSqa_disk_remove_ownership ( api_set_obj => $api_set_obj, disk_list => \@same_carrier_disks)){
		die ("RaidLMUtils: FAIL: Unable to remove disks ownership");
	}
	push (@{$self->{tc_data}->{removed_disks}}, @same_carrier_disks);	
}

########### RaidLM_initiate_evacuation ###################################################
# Function:    RaidLM_initiate_evacuation
#
# Description: This subroutine fails disk by calling _initiate_failure          
#
#####################################################################################################

sub RaidLM_initiate_evacuation { 
	my $self=shift;
	my (%opts) =validate(@_,
					   {fail_hash       => {type => HASHREF},
						failure_type    => {type => SCALAR, default => "failed", optional => 1},
						check_asup      => {type => SCALAR, default => 0, optional => 1},
						timeout         => {type => SCALAR, default => 400, optional => 1},
						wait_for_recons => {type => SCALAR, default => 0, optional => 1},
						node            => {type => SCALAR, default => "local", optional => 1},
					   }
	);
	my $fail_hash       = $opts{fail_hash};
	my $failure_type    = $opts{failure_type};
	my $wait_for_recons = $opts{wait_for_recons};
	my $node            = $opts{node};
	my $time_out = $opts{timeout};
	my $tc_data = $self->{tc_data};
	my $api_set_obj = $self->{api_set_obj};	
	my $chk_asup=0;

	my ($failing_disk,$carrier_details) = each %$fail_hash;

	my @carrier_array = @$carrier_details;
	my $other_element;

	if ($failing_disk eq $carrier_array[0]) {
		$other_element = $carrier_array[1];
	} elsif ($failing_disk eq $carrier_array[1]) {
		$other_element = $carrier_array[0];
	} else { 
		die "RaidLMUtils: Wrong entry in fail_hash";
	}
	my @filer_carrier =  keys %{$tc_data->{$failing_disk}->{carrier_disk}};
	#remove carrier from @filer_carrier if it is already failed.
	my @failed_disk;
	if (defined $tc_data->{disk_failed}) {	
		@failed_disk = @{$tc_data->{disk_failed}};
	}
	foreach my $disk (@failed_disk) {
		my $carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $disk);
		for (my $i=0;$i<@filer_carrier;  $i++) {
			if ($filer_carrier[$i] eq $carrier) {
				my @rm_aggr = splice(@filer_carrier,$i,1);
			}
		}		
	}

	my @other_carrier =  keys %{$tc_data->{$other_element}->{carrier_disk}};
	
	$Log->comment(" filer_carrier @filer_carrier" );
	$Log->comment(" other_carrier @other_carrier" );
	my $disk_to_fail;
	foreach my $carrier (@filer_carrier) {
		if (($carrier_array[0] eq $carrier_array[1]) && (@{$tc_data->{$failing_disk}->{carrier_disk}->{$carrier}}) > 1) {
			$disk_to_fail = $tc_data->{$failing_disk}->{carrier_disk}->{$carrier}->[0];
			last;
		}
		if ((grep/$carrier/, @other_carrier) && ($carrier_array[0] ne $carrier_array[1])) {
			$disk_to_fail = $tc_data->{$failing_disk}->{carrier_disk}->{$carrier}->[0];
			last;
		}
	}


	my $ems_mark1 = RaidSqa_ems_log_get_mark (api_set_obj => $api_set_obj);
	$Log->comment(" RaidLMUtils: Failing disk $disk_to_fail" );
	my $fail_event_flag = 0;
	my $asup_event_flag = 0;
	$Log->comment("RaidLMUtils: Initiating failure with failure type $failure_type");

	my ($asup_event, $fail_event) = _initiate_failure(api_set_obj => $api_set_obj, failure_type => $failure_type, disk_to_fail => $disk_to_fail, node => $node);
	if ($failure_type =~ /remove_ownership/i) {
		#push(@{$tc_data->{unowned}},$disk_to_fail);
		#if disk is spare disk then there is no event generated after removing ownership of spare disks
		if (grep /$disk_to_fail/ ,@{$tc_data->{spare}->{disks}}) {
			$fail_event = "";
			$fail_event_flag = 1;
			push (@{$tc_data->{removed_disks}}, $disk_to_fail);
		}
	}
	if ($fail_event eq '') {
		$fail_event_flag = 1;
	}
	sleep 10;
	my $timeout_event = time() + 60;
	
	$Log->comment("RaidLMUtils: Waiting for $fail_event event to occur" );
	while ($timeout_event > time() && ($fail_event ne "")) {		
		my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
                                                    timestamp   => $ems_mark1);
		my $event_ids = '';
		my $asup_ids  = '';
		$event_ids = $ref_events->{$fail_event};
		if ($chk_asup) {
			$asup_ids = $ref_events->{$asup_event};
			if ($asup_ids) {
				$asup_event_flag = 1;
			}
		}
        if ($event_ids) {
			$fail_event_flag = 1;
            $Log->comment("RaidLMUtils: Info: Ems event-$fail_event found");            
            last;
        } else {
            sleep(10);
        }
	}
	if (!$fail_event_flag) {
		die "RaidLMUtils: FAIL: Event $fail_event not found in log dump";
	}
	
	if ($chk_asup) {
			if ($asup_event_flag) {
				$Log->comment("RaidLMUtils: Info: Asup event-$asup_event found");     
			} else {
				die "RaidLMUtils: FAIL: event $asup_event not found in log dump";
			}
	}
	push(@{$tc_data->{disk_failed}},$disk_to_fail);
	my @spares = @{$tc_data->{spare}->{disks}};
	$Log->comment("check spares @spares"); 
	if (!$wait_for_recons ) {
		return $disk_to_fail;
	}
	if (($failing_disk !~ /spare/i) && (scalar @spares > 0)) { 	

		my $recons_event = "raid.rg.recons.done";
		my $recons_event_flag = 0;
		$Log->comment("RaidLMUtils: Waiting for  $recons_event even to occur" );
		$timeout_event = time() + $time_out;
		while ($timeout_event > time()) {		
			my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
														timestamp   => $ems_mark1);
			my $event_ids = '';
			my $asup_ids  = '';
			$event_ids = $ref_events->{$recons_event};
			
			if ($event_ids) {
				$recons_event_flag = 1;
				$Log->comment("RaidLMUtils: Info: Ems event-$recons_event found");            
				last;
			} else {
				sleep(30);
			}
		}
		if (!$recons_event_flag) {
			die "RaidLMUtils: FAIL:event $recons_event not found in log dump";
		}

		my $target_disk = RaidSqa_ems_get_value_by_key(api_set_obj => $api_set_obj,
												  eventtype   => $recons_event,
												  key         => 'disk',
                                                  timestamp   => $ems_mark1);


		$Log->comment("RaidLMUtils: Target Recons disk is: $target_disk ");

		#update hash with reconstructed disk new disk 
		my $target_carrier = RaidSqa_get_disk_carrier ( api_set_obj => $api_set_obj, disk_name => $target_disk); 
		my $source_carrier = RaidSqa_get_disk_carrier ( api_set_obj => $api_set_obj, disk_name => $disk_to_fail); 
		push (@{$tc_data->{$failing_disk}->{carrier_disk}->{$target_carrier}}, $target_disk) ;
		
		if ($disk_to_fail eq $tc_data->{$failing_disk}->{carrier_disk}->{$target_carrier}->[0]) {
			shift @{$tc_data->{$failing_disk}->{carrier_disk}->{$source_carrier}};
		} else {
			pop @{$tc_data->{$failing_disk}->{carrier_disk}->{$source_carrier}};
		}
		#if all disk are not used from carrier then remove that carrier hash from tc_hash
		# and remove the src disk from raidgroup carrier_disk hash
		if (scalar @{$tc_data->{$failing_disk}->{carrier_disk}->{$source_carrier}} == 0) {
			delete $tc_data->{$failing_disk}->{carrier_disk}->{$source_carrier};
		}
		#remove target disk from spare hash 
		
		if ($target_disk eq $tc_data->{spare}->{carrier_disk}->{$target_carrier}->[0]) {
			shift @{$tc_data->{spare}->{carrier_disk}->{$target_carrier}};
		} else {
			pop @{$tc_data->{spare}->{carrier_disk}->{$target_carrier}};
		}
		#if all disk are not used from carrier then remove that carrier hash from tc_hash
		# and remove the src disk from raidgroup carrier_disk hash
		if (scalar @{$tc_data->{spare}->{carrier_disk}->{$target_carrier}} == 0) {
			delete $tc_data->{spare}->{carrier_disk}->{$target_carrier};
		}
	} else {
		 
		my $source_carrier = RaidSqa_get_disk_carrier ( api_set_obj => $api_set_obj, disk_name => $disk_to_fail); 
		if ($disk_to_fail eq $tc_data->{$failing_disk}->{carrier_disk}->{$source_carrier}->[0]) {
			shift @{$tc_data->{$failing_disk}->{carrier_disk}->{$source_carrier}};
		} else {
			pop @{$tc_data->{$failing_disk}->{carrier_disk}->{$source_carrier}};
		}

	
	}
	return $disk_to_fail;
}

########### RaidLM_verify_layout_correction ###################################################
# Function:    RaidLM_verify_layout_correction
#
# Description: This subroutine checks for layout correction if wait_for_completion is set to 1 
#              and returns evacuation details, else wait_for_completion is set to 0 then returns 
#			   returns evacuation details before completion of evacuation              
#
#####################################################################################################
sub RaidLM_verify_layout_correction {
	my $self=shift;
	my (%opts) =validate(@_,
					   {wait_for_completion => {type => SCALAR, default => 1, optional => 1},
						layout_correction   => {type => SCALAR, default => "undesirable_layout", optional => 1},						
						timeout             => {type => SCALAR, default => 400, optional => 1},
						event               => {type => SCALAR, Default => undef, optional => 1},
						ems_mark            => {type => SCALAR, Default => undef, optional => 1},
						node                => {type => SCALAR, default => "local", optional => 1},
					   }
	);

	
	my $tc_data = $self->{tc_data};
	my $api_set_obj = $self->{api_set_obj};
	my $wait_for_completion = $opts{wait_for_completion};
	my $layout_correction = $opts{layout_correction};
	my $time_out = $opts{timeout};
	my $ems_mark = $opts{ems_mark};
	my $node             = $opts{node};
	if (!defined $ems_mark) {	
		$ems_mark = RaidSqa_ems_log_get_mark (api_set_obj => $api_set_obj);
		$tc_data->{ems_mark}= $ems_mark;
	}
	
	$Log->comment("RaidLMUtils: Start Raid LM service ");

	$self->RaidLM_enable_disable(enable=>1);

	sleep 20;
	
 	my $src_evac_disk;
	my $dst_evac_disk;
	my $src_carrier ;
	my $dst_carrier; 
	my $diskcopy_event              = "raid.rg.diskcopy.start";
	my $evac_start                  = "raidlm.carrier.evac.start";
	my $carrier_evac_start          = "raidlm.carrier.copy.start";
	my $event_cannot_make_progress  = "raidlm.cannotMakeProgress";
	my $diskcopy_complete_event     = "raid.rg.diskcopy.done";
	my $timeout_event = time() + $time_out;
	my $found_flag = 0;
	my $event_ids_evac_start;
	my $event_ids_disk_copy_start;
	my $event_ids_carrier_copy_start;
	my $event_ids_cannot_make_progress;
	my $event_ids_disk_copy_done;
	my @src_evac_disks;
	my @dst_evac_disks;
	my @src_evac_disks_done;
    my @dst_evac_disks_done;
	my $no_of_evacs;
	$Log->step("RaidLMUtils: Waiting for Evacuation to start  ");
	while ($timeout_event > time()) {		
		my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
                                                    timestamp   => $ems_mark);
		my $event_ids = '';
		$event_ids_evac_start         = $ref_events->{$evac_start};
		$event_ids_disk_copy_start    = $ref_events->{$diskcopy_event};
		$event_ids_carrier_copy_start = $ref_events->{$carrier_evac_start};
		$event_ids_cannot_make_progress  =  $ref_events->{$event_cannot_make_progress};
		

		$found_flag=0;
		$Log->comment(" event_ids_evac_start ".Dumper $event_ids_evac_start );
		$Log->comment(" event_ids_disk_copy_start ".Dumper $event_ids_disk_copy_start );
		$Log->comment(" event_ids_carrier_copy_start ".Dumper $event_ids_carrier_copy_start );
		$Log->comment(" event_ids_cannot_make_progress ".Dumper $event_ids_cannot_make_progress );

		
		if (defined $event_ids_cannot_make_progress ) {
			my $ref_ids = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);
			my @event_arr = @$event_ids_cannot_make_progress;
			$Log->comment(" stalmate mate disks 1 : ".Dumper \@event_arr );
			my $non_evac_disk = $ref_ids->{$event_arr[0]}->{disk};			
			$tc_data->{stalemate_disk} = $non_evac_disk;
			foreach my $event_id (@event_arr) {
				$Log->comment(" stalmate mate disks 2 : ".Dumper $ref_ids->{$event_id} );
				push (@{$tc_data->{stalemate_disks}}, $ref_ids->{$event_id}->{disk});
			}
			my $asup_event = "callhome.disk.carrier.evacuation.notpossible";
			my $event_ids_asup   = $ref_events->{$asup_event};
			if (defined $event_ids_asup ) {
				my @asup_disks;
				$Log->comment(" Cannot make progress Asup Detected fo disk $non_evac_disk".Dumper $event_ids_asup );
				foreach my $asup_event_id (@$event_ids_asup) { 
					$Log->comment(" Asup list " . Dumper $asup_event_id);
					my $cmp_str = $ref_ids->{$asup_event_id}->{subject};
					$Log->comment(" cmp str : $cmp_str " );
					if ($cmp_str =~ /Disk:\s*(.+L\d)\)?/) {
						push(@asup_disks, $1);	
					}
				}
				$Log->comment(" asup disks are : @asup_disks");
				foreach my $disk (@asup_disks) {
					if(!(grep/$disk/, @{$tc_data->{stalemate_disks}})) {
						$Log->warn("Asup message is generated for different disk $disk");
					}
				}
			}
			die "RaidLMUtils: FAIL: Evacution will not start since event $event_cannot_make_progress found for disk $non_evac_disk";
		}

		if ($layout_correction =~ /evac_start/i) {	
			if(!defined $event_ids_evac_start) {
				sleep 10;
				next;
			}
		}

		if ($layout_correction =~ /undesirable_layout/i) {	
			if(!defined $event_ids_carrier_copy_start) {
				sleep 10;
				next;
			}
		}
	
		if(!defined $event_ids_disk_copy_start) {
			sleep 10;
			next;
		}

		$no_of_evacs = @$event_ids_disk_copy_start;
	
		#code for "raidlm.carrier.evac.start";
		my $ref_ids = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);
		if ($layout_correction =~ /evac_start/i) {	
			foreach my $eventid (@$event_ids_evac_start) {
				my $evac_disk = $ref_ids->{$eventid}->{disk};
				my $ref_ids_diskcopy = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);
				foreach my $eventid_dc (@$event_ids_disk_copy_start) {
					if ($ref_ids_diskcopy->{$eventid_dc}->{source} eq $evac_disk) {
						$src_evac_disk = $evac_disk;
						$dst_evac_disk = $ref_ids_diskcopy->{$eventid_dc}->{target};
						if (!grep /$src_evac_disk/,@src_evac_disks) {		
							push ( @src_evac_disks,$src_evac_disk);
							push ( @dst_evac_disks,$dst_evac_disk);
							if (!$wait_for_completion) {
								my $hash_evac_src_dst = {};
								$src_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $src_evac_disks[0]);
								$dst_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $dst_evac_disks[0]);
								$hash_evac_src_dst->{src}->{$src_evac_disks[0]}= $src_carrier;                                            
								$hash_evac_src_dst->{dst}->{$dst_evac_disks[0]}= $dst_carrier;                                            
								$Log->comment(" Evacuation Hash ".Dumper $hash_evac_src_dst );	                                          
								push (@{$tc_data->{evacuations_start}}, $hash_evac_src_dst);
								return $hash_evac_src_dst;
							}
							$found_flag++;
							$evac_disk ='';
						}
					}
				}
			}
		}
		
		if ($layout_correction =~ /undesirable_layout/i) {	
		#code for raidlm.carrier.copy.start
			foreach my $eventid (@$event_ids_carrier_copy_start) {
				my $evac_disk = $ref_ids->{$eventid}->{source_disk};
				my $ref_ids_diskcopy = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);
				foreach my $eventid_dc (@$event_ids_disk_copy_start) {
					if ($ref_ids_diskcopy->{$eventid_dc}->{source} eq $evac_disk) {
						$src_evac_disk = $evac_disk;
						$dst_evac_disk = $ref_ids_diskcopy->{$eventid_dc}->{target};
						if (!grep /$src_evac_disk/,@src_evac_disks) {	
							push ( @src_evac_disks,$src_evac_disk);
							push ( @dst_evac_disks,$dst_evac_disk);
							if (!$wait_for_completion) {
								my $hash_evac_src_dst = {};
								$src_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $src_evac_disks[0]);
								$dst_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $dst_evac_disks[0]);
								$hash_evac_src_dst->{src}->{$src_evac_disks[0]}= $src_carrier;                                            
								$hash_evac_src_dst->{dst}->{$dst_evac_disks[0]}= $dst_carrier;                                            
								$Log->comment(" Evacuation Hash".Dumper $hash_evac_src_dst );	                                          
								push (@{$tc_data->{evacuations_start}}, $hash_evac_src_dst);
								return $hash_evac_src_dst;
							}
							$found_flag++;
							$evac_disk ='';
						}
					}
				}
			}

		}	

		if ($layout_correction =~ /disk_copy/i) {	
			foreach my $eventid (@$event_ids_disk_copy_start) {
				my $evac_disk = $ref_ids->{$eventid}->{source};
				$Log->comment("1 Evacuation Hash in disk copy $evac_disk :");
				my $ref_ids_diskcopy = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);
				foreach my $eventid_dc (@$event_ids_disk_copy_start) {
					if ($ref_ids_diskcopy->{$eventid_dc}->{source} eq $evac_disk) {
						$Log->comment("2 Evacuation Hash in disk copy $evac_disk :");
						$src_evac_disk = $evac_disk;
						$dst_evac_disk = $ref_ids_diskcopy->{$eventid_dc}->{target};
						if (!grep /$src_evac_disk/,@src_evac_disks) {		
							$Log->comment("3 Evacuation Hash in disk copy $src_evac_disk :");
							push ( @src_evac_disks,$src_evac_disk);
							push ( @dst_evac_disks,$dst_evac_disk);
							if (!$wait_for_completion) {
								my $hash_evac_src_dst = {};
								$src_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $src_evac_disks[0]);
								$dst_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $dst_evac_disks[0]);
								$hash_evac_src_dst->{src}->{$src_evac_disks[0]}= $src_carrier;                                            
								$hash_evac_src_dst->{dst}->{$dst_evac_disks[0]}= $dst_carrier;                                            
								$Log->comment(" Evacuation Hash ".Dumper $hash_evac_src_dst );	                                          
								push (@{$tc_data->{evacuations_start}}, $hash_evac_src_dst);
								return $hash_evac_src_dst;
							}
							$found_flag++;
							$evac_disk ='';
						}
					}
				}
			}
		}
	
		#wait till undesirable layout is corrected
		if ($layout_correction =~/undesirable_layout/i && $wait_for_completion) {
			foreach my $aggr_name (@{$tc_data->{aggregates}}) {
			
				my $aggr_desirable_layout = RaidSqa_check_desirable_layout (api_set_obj => $api_set_obj,
														   filer_vol => $aggr_name,	 );

				if ($aggr_desirable_layout ){
					next;
				} else {
					$Log->comment("RaidLMUtils: Wait for other evacuations ");
					$timeout_event = time() + $time_out;					
					last;
				}
				$Log->comment("RaidLMUtils: Aggregate $aggr_name desirable layout check passed");
			}

		}
        sleep(20);     	 
	}


#	if (!$event_ids_disk_copy_start ) {
#		die "RaidLMUtils: No evacuations in progress";
#	}


	
	my $evac_hash_arr = [];
	for(my $i=0; $i < @src_evac_disks; $i++) {
		my $hash_evac_src_dst = {};
		$src_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $src_evac_disks[$i]);
		$dst_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $dst_evac_disks[$i]);
		$hash_evac_src_dst->{src}->{$src_evac_disks[$i]}= $src_carrier;                                            
		$hash_evac_src_dst->{dst}->{$dst_evac_disks[$i]}= $dst_carrier;                                                 
		push (@$evac_hash_arr, $hash_evac_src_dst);	
		push (@{$tc_data->{evacuations_start}}, $hash_evac_src_dst);
	}
	
	$Log->comment("RaidLMUtils: Evacuation_started ");
	if ($wait_for_completion == 0 ) {
		return $tc_data->{evacuations_start};
	}


	
	$Log->comment("RaidLMUtils: Waiting for evacutions to complete");
	$timeout_event = time() + $time_out;
	@src_evac_disks_done = ();;
    @dst_evac_disks_done = ();
	my $found_flag_done = 0;
	while ($timeout_event > time()) {	
		my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
														  timestamp   => $ems_mark);

		$event_ids_disk_copy_done    = $ref_events->{$diskcopy_complete_event};

		if(!defined $event_ids_disk_copy_done) {
			sleep 20;
			next;
		}
		
		$Log->comment("RaidLMUtils: Event_ids_disk_copy_done" . Dumper $event_ids_disk_copy_done );
		$Log->comment("RaidLMUtils: Evac_hash_arr" . Dumper $evac_hash_arr );
		################
			foreach my $evac (@$evac_hash_arr) {
				my ($src_disk,undef) = each %{$evac->{src}};
				my $ref_ids_diskcopy_done = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);
				foreach my $eventid_dc (@$event_ids_disk_copy_done) {
					if ($ref_ids_diskcopy_done->{$eventid_dc}->{source} eq $src_disk) {
						
						$src_evac_disk = $src_disk;
						$dst_evac_disk = $ref_ids_diskcopy_done->{$eventid_dc}->{target};
						if (!grep /$src_evac_disk/,@src_evac_disks_done) {	
							push ( @src_evac_disks_done,$src_evac_disk);
							push ( @dst_evac_disks_done,$dst_evac_disk);
							$found_flag_done++;
							$src_disk ='';
						}
					}
				}
			}
		################
		if ($found_flag_done < $no_of_evacs) {
			sleep (20);
		} else {
			$Log->comment("RaidLMUtils: No_of_evacs $no_of_evacs done" );
			last;
		}
	}

	my $evac_hash_arr_done = [];
	for(my $i=0; $i < @src_evac_disks_done; $i++) {
		my $hash_evac_src_dst = {};
		$src_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $src_evac_disks_done[$i]);
		$dst_carrier = RaidSqa_get_disk_carrier (api_set_obj   => $api_set_obj, disk_name => $dst_evac_disks_done[$i]);
		$hash_evac_src_dst->{src}->{$src_evac_disks_done[$i]}= $src_carrier;                                            
		$hash_evac_src_dst->{dst}->{$dst_evac_disks_done[$i]}= $dst_carrier;   
		push (@$evac_hash_arr_done, $hash_evac_src_dst);	
		push (@{$tc_data->{evacuations}}, $hash_evac_src_dst);
	}
			


	
	if ($layout_correction =~/undesirable_layout/i) {
		foreach my $aggr_name (@{$tc_data->{aggregates}}) {
		
			my $aggr_desirable_layout = RaidSqa_check_desirable_layout (api_set_obj => $api_set_obj,
													   filer_vol => $aggr_name,	 );
			if (!$aggr_desirable_layout ) {
				die ("RaidLMUtils: Aggregate $aggr_name is not in desirable layout ");

			}			
			$Log->comment("RaidLMUtils: Aggregate $aggr_name desirable layout check passed");
		}

	}
	if ($layout_correction =~/spare_optimization/i) {
		#todo: check for spare optimization
	}
	

	my $aggr_ref=$tc_data->{aggregates};
	foreach my $aggr ( (@{$aggr_ref}) ){
		$Log->comment("RaidLMUtils: Modifing aggr $aggr hash");
		foreach my $evac (@$evac_hash_arr_done) {
			my ($src_disk,undef) = each %{$evac->{src}};
			my ($dst_disk,undef) = each %{$evac->{dst}};

			my $src_carrier = $evac->{src}->{$src_disk};
			my $dst_carrier = $evac->{dst}->{$dst_disk};
			$Log->comment("RaidLMUtils: Evacuating disk from source carrier: $src_carrier to destination carrier: $dst_carrier");
									

			foreach my $carrier (keys %{$tc_data->{$aggr}->{carrier_disk}}) {
				if ($carrier eq $src_carrier) {
					if (grep /$src_disk/, @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}}) {
						#ToDo
						#add new replaced disk in raidgroup carrier_disk hash
						 push (@{$tc_data->{$aggr}->{carrier_disk}->{$dst_carrier}}, $dst_disk) ;
						 if ($src_disk eq $tc_data->{$aggr}->{carrier_disk}->{$carrier}->[0]) {
							 shift @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}};
						 } else {
							 pop @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}};
						 }
						#if all disk are not used from carrier the remove that carrier hash from tc_hash
						# and remove the src disk from raidgroup carrier_disk hash
						 if (scalar @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}} == 0) {
							 delete $tc_data->{$aggr}->{carrier_disk}->{$carrier};
						 }

						#remove target disk from spare hash 
						if ($dst_disk eq $tc_data->{spare}->{carrier_disk}->{$dst_carrier}->[0]) {
							shift @{$tc_data->{spare}->{carrier_disk}->{$dst_carrier}};
						} else {
							pop @{$tc_data->{spare}->{carrier_disk}->{$dst_carrier}};
						}
						#if all disk are not used from carrier then remove that carrier hash from tc_hash
						# and remove the src disk from raidgroup carrier_disk hash
						if (scalar @{$tc_data->{spare}->{carrier_disk}->{$dst_carrier}} == 0) {
							delete $tc_data->{spare}->{carrier_disk}->{$dst_carrier};
						}

					}
				}
			}
		}		
	}

	return $evac_hash_arr;
	
}

########### RaidLM_wait_for_evacuation_completion ###################################################
# Function:    RaidLM_wait_for_evacuation_completion
#
# Description: This  subroutine waits for evacuation to complete and updates hash with replaced disks
#
#####################################################################################################

sub RaidLM_wait_for_evacuation_completion { 
	my $self=shift;
	my (%opts) =validate(@_,
					   {evac_hash           => {type => HASHREF,},
		                ems_mark            => {type => SCALAR, Default => undef, optional => 1},
						wait_for_completion => {type => SCALAR, default => 1, optional => 1},
						chk_asup            => {type => SCALAR, default => 0, optional => 1},
						ems_not_expected    => {type => SCALAR, Default => undef, optional => 1},
						asup_not_expected    => {type => SCALAR, Default => undef, optional => 1},
						timeout             => {type => SCALAR, default => 400, optional => 1},
						node => {type => SCALAR, default => "local", optional => 1},
					   }
		    	);
	my $hash_evac_src_dst = $opts{evac_hash};
	my $ems_mark = $opts{ems_mark};
	my $time_out = $opts{timeout};
	my $wait_for_completion = $opts{wait_for_completion};
	my $chk_asup = $opts{chk_asup};
	my $ems_not_expected = $opts{ems_not_expected};
	my $asup_not_expected = $opts{asup_not_expected};
	my $node             = $opts{node};
	my ($src_disk) = keys %{$hash_evac_src_dst->{src}};
	my ($dst_disk) = keys %{$hash_evac_src_dst->{dst}};
	my $src_carrier = $hash_evac_src_dst->{src}->{$src_disk};
	my $dst_carrier = $hash_evac_src_dst->{dst}->{$dst_disk};
	my $tc_data = $self->{tc_data};
	my $api_set_obj = $self->{api_set_obj};
	if (!defined $ems_mark) {	
		$ems_mark = $tc_data->{ems_mark};
	}

	#check if evacuation is complete and modify 
	#carrier and disk info in rgs where evacuation happend
	my $diskcopy_complete_event = "raid.rg.diskcopy.done";
	my $asup_carrier_fault = "callhome.carrier.fault";
	my $carrier_remove_event = "raid.carrier.remove";
	my $timeout_event = time() + $time_out;
	my $disk_copy_flag = 0;
	while ($timeout_event > time()) {		
		my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj,
                                                      timestamp   => $ems_mark);
		my $event_ids_disk_copy_done    = $ref_events->{$diskcopy_complete_event};
		
		my $event_ids_carrier_fault = $ref_events->{$asup_carrier_fault};
		if (defined $ems_not_expected ) {
			my $event_ids_not_expected ="";
			$event_ids_not_expected = $ref_events->{$ems_not_expected};
			if (defined $event_ids_not_expected) {
				die "RaidLMUtils: FAIL: Unexpected event: $ems_not_expected found  ";
			}
		}

		if (defined $asup_not_expected ) {
			my $asup_ids_not_expected ="";
			$asup_ids_not_expected = $ref_events->{$asup_not_expected};
			if (defined $asup_ids_not_expected) {
				die "RaidLMUtils: FAIL: Unexpected event: $asup_not_expected found  ";
			}
		}
		my $ref_ids_diskcopy_done = RaidSqa_ems_log_dump_by_id(api_set_obj => $api_set_obj, timestamp => $ems_mark);

		foreach my $eventid_dc (@$event_ids_disk_copy_done) {
			if (($ref_ids_diskcopy_done->{$eventid_dc}->{source} eq $src_disk) &&  ($wait_for_completion ) ){
				    #$ref_ids_diskcopy_done->{$eventid_dc}->{target} eq  $dst_disk)    ) {
				 $dst_disk = $ref_ids_diskcopy_done->{$eventid_dc}->{target};
				 $disk_copy_flag = 1;
			     last;					
			}			
		}

		
        if ( $disk_copy_flag) {
            $Log->comment("RaidLMUtils: Info: Ems event-$diskcopy_complete_event found");            
            last;
        } else {
            sleep(50);
        }	 

		if ($chk_asup) {
			if ($event_ids_carrier_fault) {
				$Log->comment("RaidLMUtils: Info: Asup event-$asup_carrier_fault found");     
			} else {
				die "RaidLMUtils: FAIL: carrier fault event $asup_carrier_fault not found in log dump";
			}
		}
	}

	#modify hash by removing evacuated disk and replace old disk
	my $aggr_ref=$tc_data->{aggregates};
	foreach my $aggr ( (@{$aggr_ref}) ){
		foreach my $carrier (keys %{$tc_data->{$aggr}->{carrier_disk}}) {
			if ($carrier eq $src_carrier) {
				if (grep /$src_disk/, @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}}) {
					#ToDo
					#add new replaced disk in raidgroup carrier_disk hash
					 push (@{$tc_data->{$aggr}->{carrier_disk}->{$dst_carrier}}, $dst_disk) ;
					 if ($src_disk eq $tc_data->{$aggr}->{carrier_disk}->{$carrier}->[0]) {
						 shift @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}};
					 } else {
						 pop @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}};
					 }
					#if all disk are not used from carrier the remove that carrier hash from tc_hash
					# and remove the src disk from raidgroup carrier_disk hash
					 if (scalar @{$tc_data->{$aggr}->{carrier_disk}->{$carrier}} == 0) {
						 delete $tc_data->{$aggr}->{carrier_disk}->{$carrier};
					 }

					#remove target disk from spare hash 
					if ($dst_disk eq $tc_data->{spare}->{carrier_disk}->{$dst_carrier}->[0]) {
						shift @{$tc_data->{spare}->{carrier_disk}->{$dst_carrier}};
					} else {
						pop @{$tc_data->{spare}->{carrier_disk}->{$dst_carrier}};
					}
					#if all disk are not used from carrier then remove that carrier hash from tc_hash
					# and remove the src disk from raidgroup carrier_disk hash
					if (scalar @{$tc_data->{spare}->{carrier_disk}->{$dst_carrier}} == 0) {
						delete $tc_data->{spare}->{carrier_disk}->{$dst_carrier};
					}

				}
			}
		}		
	}	

	$Log->comment("RaidLMUtils: Evacuating disk from source carrier: $src_carrier to destination carrier: $dst_carrier");
	#add evacuate information in to evacuations hash
	push ( @{$tc_data->{evacuations}}, $hash_evac_src_dst);
	 $Log->comment("RaidLMUtils: Evacuation_completion ");
}

########### RaidLM_destroy_aggr ###################################################
# Function:    RaidLM_destroy_aggr
#
# Description: This  subroutine removes aggr 
#
###################################################################################
sub RaidLM_destroy_aggr {
	my $self=shift;
	my (%opts) =validate(@_,
					   {
		                filervol       => {type => SCALAR,},	
						node => {type => SCALAR, default => "local", optional => 1},
					   }
		    	);
	my $filervol = $opts{filervol};
	my $node     = $opts{node};
	my $tc_data = $self->{tc_data};
	my $api_set_obj = $self->{api_set_obj};

	my @disk_list ;
	my $current_aggr = $tc_data->{$filervol};
	if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
								 filer_vol   => $filervol,
								 aggr_type   => 1)
			) {
			
			push (@disk_list,@{$current_aggr->{disks}});

			foreach my $carrier (keys %{$current_aggr->{carrier_disk}}) {
				my @carrier_disks = @{$current_aggr->{carrier_disk}->{$carrier}};
				push (@{$tc_data->{spare}->{carrier_disk}->{$carrier}},@carrier_disks);				
			}
			
			if (!RaidSqa_aggr_remove(api_set_obj => $api_set_obj,
								filer_vol   => $filervol,
								aggr_type   => 1)) {
				$Log->comment("RaidLMUtils: Unable to remove the aggregate $filervol");
				return 0;
			}
			#delete hash related to removed filervol
			delete $tc_data->{$filervol};
            #update aggregate array
			for (my $i=0;$i<@{$tc_data->{aggregates}};  $i++) {
				if ($tc_data->{aggregates}->[$i] eq $filervol) {
					my @rem_aggr = splice(@{$tc_data->{aggregates}},$i,1);
					last;
				}
			}					
	}
	return 1;
}

########### RaidLM_delete_resource ###################################################
# Function:    RaidLM_delete_resource
#
# Description: This subroutine removes aggr cleans the setup 
#
###################################################################################
sub RaidLM_delete_resource { 
	my $self=shift;
	my (%opts) =validate(@_,
					   {		                	
						node => {type => SCALAR, default => "local", optional => 1},
					   }
		    	);
	my $node    = $opts{node};
	my $tc_data = $self->{tc_data};
	my $api_set_obj = $self->{api_set_obj};

	my $aggr_arrey_ref = $tc_data->{aggregates};
	my $failed_disks =  $tc_data->{disk_failed};
			
	foreach my $temp_aggr (@$aggr_arrey_ref) {
	if (RaidSqa_check_aggr_exist(api_set_obj => $api_set_obj,
								 filer_vol   => $temp_aggr,
								 aggr_type   => 1)
			) {
			RaidSqa_aggr_remove(api_set_obj => $api_set_obj,
								filer_vol   => $temp_aggr,
								aggr_type   => 1) or 
			$Log->comment("RaidLMUtils: Unable to remove the aggregate $temp_aggr");
		}
	}
	RaidSqa_fiji_clear(api_set_obj => $api_set_obj,  fiji_point=> 'all'); 
	foreach my $disk_name (@$failed_disks) {
		RaidSqa_disk_unfail(api_set_obj => $api_set_obj, 
								disk_name   => $disk_name, 
								options     => "-s");
	}
	sleep 10;
	RaidSqa_disk_assign (api_set_obj => $api_set_obj,disk_list => $self->{tc_data}->{removed_disks});

}


########### RaidLM_enable_disable ###################################################
# Function:    RaidLM_enable_disable
#
# Description: This subroutine starts or stops RaidLm feature on filer 
#
#####################################################################################################

sub RaidLM_enable_disable {
	my $self=shift;
	my (%opts) =validate(@_,
					   {
		                enable  => {type => SCALAR, default => 1, optional => 1},	
						node => {type => SCALAR, default => "local", optional => 1},
					   }
		    	);
	my $enable = $opts{enable};
	my $node   = $opts{node};
	my $api_set_obj = $self->{api_set_obj};
	my $start_stop;
	if ($enable) {
		$start_stop = "start";
	} else {
		$start_stop = "stop";
	}
	$Log->comment("RaidLMUtils: $start_stop Raid LM service ");

	my $apiset_shell = NACL::APISet->new(
                                    hostobj     => $api_set_obj->{hostobj},
                                    category    => 'Node', 
                                    interface   => 'CLI', 
                                    set         => 'Systemshell',
                                );	
	if ($enable) {
		$apiset_shell->execute_raw_command(command => "rm /mroot/etc/raid/raid_lm/raid_lm.log");
	}
    	$apiset_shell->execute_raw_command(command => "raid_cm $start_stop");
	$apiset_shell->execute_raw_command(command => "spmctl");

}

########### _get_disk_list_from_same_carrier ###################################################
# Function:    _get_disk_list_from_same_carrier
#
# Description: This subroutine returns carrier disk list 
#
#####################################################################################################

sub _get_disk_list_from_same_carrier {
	my (%opts) =validate(@_,
					   {api_set_obj     => {type => OBJECT, isa     => "NACL::APISet"},
						node            => {type => SCALAR, default => "local", optional => 1},
                        conntype        => {type => SCALAR, default => "#console", optional => 1},
						spare_only      => {type => SCALAR, default => "yes", optional => 1},
					   }
		    	);

	my $api_set_obj       = $opts{api_set_obj};
    my $node             = $opts{node};
    my $conntype         = $opts{conntype};
	my $spare_only       = $opts{spare_only};

	my @disk_list;
	my $carrier_disk_mapping =  RaidSqa_get_carrier_disks_mapping(api_set_obj => $api_set_obj,forward=>1,spare_only=>$spare_only, node => $node);
	foreach my $carrier_id(keys %$carrier_disk_mapping) {
		if (scalar @{$carrier_disk_mapping->{$carrier_id}} > 1 ) {
			push(@disk_list, @{$carrier_disk_mapping->{$carrier_id}} );
		}
	}

	if (!scalar @disk_list) {
		die ("RaidLMUtils: FAIL: Disk from same carrier not found");
	}
	return @disk_list;
}
########### Subroutine for _symmetric_diff_array ###################################################
# Function:    _symmetric_diff_array
#
# Description: This  subroutine returns symmetric diff of 2 arrays
#     example: @arr1 = qw( a b c d e f)
#              @arr2 = qw(d f)
#              then subroutine returns (a b c e);
#
#####################################################################################################
sub _symmetric_diff_array {
	my ($arr1,$arr2) = @_;

	sub dec($) {
		$_[0] ? $_[0]--: 0
	};

	my @all_disks = @$arr1;
	my @subset_disk = @$arr2;

	my %hash_of_arr_ele;

	$hash_of_arr_ele{$_}++ for (@subset_disk);
	my @diff_array = map { dec $hash_of_arr_ele{$_} ? () : $_ } @all_disks;

	return @diff_array;
} #end of _symmetric_diff_array

########### _initiate_failure ######################################################################
# Function:    _initiate_failure
#
# Description: This subroutine fails disk depending upon type of failure and retruns ems event and ASUP 
#              event related to that failure.
#
#####################################################################################################
sub _initiate_failure {
	my (%opts) =validate(@_,
					   {api_set_obj     => {type => OBJECT, isa     => "NACL::APISet"},
						disk_to_fail    => {type => SCALAR},
                        failure_type    => {type => SCALAR},
						node            => {type => SCALAR, default => "local", optional => 1},
					   }
		    	);
	my $api_set_obj    = $opts{api_set_obj};
    my $disk_to_fail   = $opts{disk_to_fail};
    my $failure_type   = $opts{failure_type};
	my $node           = $opts{node};
    my $ems_event;
	my $asup_event;
	#ToDo: exact event not known ...change asup event once we get to know Asup event
	$Log->comment("RaidLMUtils: Disk_to_fail used for intiating failure $disk_to_fail" );

	if ($failure_type =~ /^failed/i) {
		my $disk_parent_state = RaidSqa_get_disk_parent_state (api_set_obj => $api_set_obj, disk_name => $disk_to_fail);
		if ($disk_parent_state =~ /PARITY|DPARITY|DATA/i) {
			$ems_event = "raid.config.filesystem.disk.failed"; #for aggr disks
		} else {
			$ems_event = "raid.config.disk.failed"; #for spare disk
		}
		$asup_event = "ASUP.MSG.DISKS.FAILED.IN.CARRIER"; 
		my $fiji1 ="repeat 999 (raid.label.read.done (eq \@disk $disk_to_fail) (return (buf_Errno E_DISK_FAILED)))";
		if (!RaidSqa_fiji_filter(api_set_obj => $api_set_obj, fiji_point  => $fiji1)) {
			die("RaidLMUtils: FAIL: Failed to set fiji $fiji1 on $disk_to_fail");        
		}
		my @labelmaint_arr= ($disk_to_fail);
		RaidSqa_labelmaint_isolate(api_set_obj => $api_set_obj, arg_list =>\@labelmaint_arr);
		RaidSqa_labelmaint_unisolate(api_set_obj => $api_set_obj);
		
		return ($asup_event, $ems_event);
	}

	if ($failure_type =~ /remove_ownership/i) {
		
#$ems_event = "raid.disk.ownership.change.forced"; # event for remove ownership of filesystem disk
		$ems_event = "raid.fs.disk.ownership.change";
		if(!_RaidSqa_disk_remove_ownership ( api_set_obj => $api_set_obj, disk_name=>$disk_to_fail)){
			die ("RaidLMUtils: FAIL: Unable to remove disks ownership");
		}
	  	return ($asup_event, $ems_event);
	}


	if ($failure_type =~ /start_sdc/i) {		
		
		$ems_event = "shm.pullLogWarning"; #
		$asup_event = '';
		if(!RaidSqa_disk_fail( api_set_obj => $api_set_obj, disk_name=>$disk_to_fail, node => $node)){
			die ("RaidLMUtils: FAIL: Unable to fail disk $disk_to_fail");
		}
	  	return ($asup_event, $ems_event);
	}
	
	if ($failure_type =~ /adminfailed/i) {		
		
		$ems_event = ''; #
		$asup_event = '';
		if(!RaidSqa_disk_fail( api_set_obj => $api_set_obj, disk_name=>$disk_to_fail, options => '-i')){
			die ("RaidLMUtils: FAIL: Unable to fail disk $disk_to_fail");
		}
	  	return ($asup_event, $ems_event);
	}


	if ($failure_type =~ /disk_maint/i) {		
		
		$ems_event = ''; #
		$asup_event = '';
		my $cmd = "priv set test; disk maint start -f -c 1 -d $disk_to_fail";
		my $ret = RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
		if (!$ret) {
			die "RaidLMUtils: FAIL: failed to run disk maint command on disk $disk_to_fail";
		}
	  	return ($asup_event, $ems_event);
	}

	if ($failure_type =~ /diskoffline/i) {		
		
		$ems_event = ''; #
		$asup_event = '';

		if(!RaidSqa_disk_offline( api_set_obj => $api_set_obj, disk_name=>$disk_to_fail)){
			die ("RaidLMUtils: FAIL: Unable to offline disk $disk_to_fail");
		}
	  	return ($asup_event, $ems_event);
	}

	if ($failure_type =~ /labelmaint/i) {		
		
		$ems_event = ''; #
		$asup_event = '';
		my @labelmaint_arr= ($disk_to_fail);
		if(!RaidSqa_labelmaint_isolate(api_set_obj => $api_set_obj, arg_list =>\@labelmaint_arr)){
			die ("RaidLMUtils: FAIL: Unable to labelmaint disk $disk_to_fail");
		}
	  	return ($asup_event, $ems_event);
	}


	if ($failure_type =~ /fail_disk_at_boot/i) {		
		
		$ems_event = ''; #
		$asup_event = '';

	  	return ($asup_event, $ems_event);
	}

	if ($failure_type =~ /no_fail/i) {		
		
		$ems_event = ''; #
		$asup_event = '';

	  	return ($asup_event, $ems_event);
	}

	if ($failure_type =~ /rawsize/i) {		
		
		$ems_event = ''; #
		$asup_event = '';
		my $cmd = "priv set -q test ; disk gen_asup $disk_to_fail 'rawsize shrank' 'Filesystem Disk'";
		my $ret = RaidSqa_filer_cmd(api_set_obj => $api_set_obj, cmd => $cmd);
		if (!$ret) {
			#die "RaidLMUtils: FAIL: failed to run disk maint command on disk $disk_to_fail";
		}
	  	return ($asup_event, $ems_event);	  	
	}


	#add other failure functions  
}

sub validate_events {
	my $self=shift;
	my (%opts) =validate(@_,
					   {
		                carrier_evac_start => {type => SCALAR, default => undef, optional => 1},
						carrier_remove     => {type => SCALAR, default => undef, optional => 1},
						carrier_evacuated  => {type => SCALAR, default => undef, optional => 1},
						timeout            => {type => SCALAR, default => 400,   optional => 1},
						ems_mark           => {type => SCALAR},
						node        => {type => SCALAR, default => "local", optional => 1},
					   }
		    	);
	my $carrier_evac_start = $opts{carrier_evac_start};
	my $carrier_remove     = $opts{carrier_remove};
	my $carrier_evacuated  = $opts{carrier_evacuated};
	my $timeout            = $opts{timeout};
	my $ems_mark_1         = $opts{ems_mark};
	my $node               = $opts{node};
	my $tc_data = $self->{tc_data};
	my $api_set_obj = $self->{api_set_obj};
	my $event_id_carrier_evac_start;
	my $event_id_carrier_evacuated;
	my $event_id_carrier_remove;
	

	my $found_events =0;
	my $timeout_event = time() + 400;
	while ($timeout_event > time()) {
		my $ref_events = RaidSqa_ems_log_dump_by_type(api_set_obj => $api_set_obj, timestamp   => $ems_mark_1);
		$event_id_carrier_evac_start = $ref_events->{$carrier_evac_start};
		$event_id_carrier_evacuated   = $ref_events->{$carrier_evacuated};
		$event_id_carrier_remove      = $ref_events->{$carrier_remove};
		$Log->comment("event_id_carrier_evac_start ".Dumper $event_id_carrier_evac_start);
		$Log->comment("event_id_carrier_evacuated ".Dumper $event_id_carrier_evacuated);
		$Log->comment("event_id_carrier_remove ".Dumper $event_id_carrier_remove);
		if (defined $carrier_evac_start && !defined $event_id_carrier_evac_start ) {
				sleep 20;
				next;
		}
		if (defined $carrier_remove && !defined $event_id_carrier_remove ) {
				sleep 20;
				next;
		}
		if (defined $carrier_evacuated && !defined $event_id_carrier_evacuated ) {
				sleep 20;
				next;
		}
		if (defined $event_id_carrier_evacuated && defined $event_id_carrier_remove && defined $event_id_carrier_evac_start ) {
				$found_events =1;
				last;
		}
	}
	if (!$found_events) {
		die "one of this events $carrier_remove, $carrier_evac_start and $carrier_evacuated not found";
	}

}


sub _RaidSqa_disk_remove_ownership {
    my (%opts) = validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_list => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        disk_name => {type => SCALAR | UNDEF,   default => undef, optional => 1},
                        timeout   => {type => SCALAR,           default => 1200,  optional => 1},
						node        => {type => SCALAR, default => "local", optional => 1},
		               }
	             );
    my $api_set_obj      = $opts{api_set_obj};
    my $conntype         = $opts{conntype};
    my $disk_name        = $opts{disk_name};
	my $disk_list        = $opts{disk_list};
    my $node             = $opts{node};
    my $timeout          = $opts{timeout};

	if (!defined($disk_name) && !defined($disk_list)) {
        $Log->comment(
              "None of the parameter $disk_list or $disk_name defined, need to define one of them");
        return 0;
    } elsif (defined($disk_name) && defined($disk_list)) {
        $Log->comment(
            "Both of the parameter $disk_list or $disk_name defined, need to define only one of them"
        );
        return 0;
    }
	
	my @disk_list_t;
	if(defined $disk_list){
		push (@disk_list_t, @{$disk_list});
	}
	if(defined $disk_name) {
		push (@disk_list_t, $disk_name);
	}

	foreach my $disk (@disk_list_t) {
		if(!RaidSqa_disk_remove_ownership ( api_set_obj => $api_set_obj, disk_name=>$disk)) {
			return 0;
		}
	}
	return 1;
}



sub RaidSqa_get_disk_carrier_serialno {
    my (%opts) =  validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet', optional => 1},
                        node        => {type => SCALAR, default => "local",        optional => 1},
                        filermode => {type => SCALAR, default => "normal", optional => 1},
                        conntype  => {type => SCALAR, default => "#rsh", optional => 1},
                        disk_name => {type => SCALAR},});

	my $api_set_obj      = $opts{api_set_obj};
    my $node              = $opts{node};
    my $filermode = $opts{filermode};
    my $conntype  = $opts{conntype};
    my $disk_name = $opts{disk_name};
	
	# create connection
    my ($host, $conn);
    if (defined($api_set_obj->{hostobj})) {
        $host = $api_set_obj->{hostobj};
        $conn = make_conn($host, $conntype);
    } elsif (defined($api_set_obj->{connection})) {
        $conn = $api_set_obj->{connection};
    }
    if ($filermode ne "normal") {
        $conntype = "#console";
    }

    #my $conn        = make_conn($host, $conntype);
    my $raid_obj    = new RaidInfo($conn);
    my $disk_object = $raid_obj->disk("$disk_name");
    if (!defined($disk_object)) {
        logcomment("RaidSqa_7mode.pm Disk - $disk_name not found");
        return (0);
    } else {
        if (!$disk_object->parameter("is-multidisk-carrier")) {
            logcomment("RaidSqa_7mode.pm Disk - $disk_name is not Boab disk");
            return (0);
        }
        return ($disk_object->parameter("carrier_serialno"));
    }
} ## end sub RaidSqa_get_disk_carrier

=head
sub RaidSqa_disk_remove_ownership {
    my (%opts) =
      validate(@_,
                       {api_set_obj => {type => OBJECT, isa     => 'NACL::APISet'},
                        conntype    => {type => SCALAR, default => "#ssh", optional => 1},
                        disk_list => {type => ARRAYREF | UNDEF, default => undef, optional => 1},
                        disk_name => {type => SCALAR | UNDEF,   default => undef, optional => 1},
                        timeout   => {type => SCALAR,           default => 1200,  optional => 1},});
    my $api_set_obj      = $opts{api_set_obj};
    my $conntype         = $opts{conntype};
    my $disk_name        = $opts{disk_name};
    my $disk_list        = $opts{disk_list};
    my $node             = $opts{node};
    my $timeout          = $opts{timeout};
    my $original_timeout = $api_set_obj->get_timeout();
    $Log->comment("The current timeout value is $original_timeout");
    
    $api_set_obj->set_timeout("connectrec-timeout" => $timeout);
    my $new_timeout = $api_set_obj->get_timeout();
    $Log->comment("New timeout $new_timeout");
    my ($result, $output, @disk_list, $disk_string);

    if (!defined($disk_name) && !defined($disk_list)) {
        $Log->comment(
              "None of the parameter $disk_list or $disk_name defined, need to define one of them");
        return 0;
    } elsif (defined($disk_name) && defined($disk_list)) {
        $Log->comment(
            "Both of the parameter $disk_list or $disk_name defined, need to define only one of them"
        );
        return 0;
    }

    if (defined($disk_name)) {
        @disk_list = ($disk_name);
    } elsif (defined($disk_list)) {
        @disk_list = @$disk_list;
    }

    $result = 1;
    $disk_string = join(", ", @disk_list);
    try {
        my $response =
          $api_set_obj->storage_disk_removeowner(disk => $disk_string, force => "true");
        $output = $response->get_raw_output();
        $Log->comment("resetting timeout to $original_timeout now");
        $api_set_obj->set_timeout("connectrec-timeout" => $original_timeout);
        $Log->comment("The current timeout value is $original_timeout");
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $exception_object = shift;
        _handle_nacl_exception($exception_object);
        $Log->comment("resetting timeout to $original_timeout now");
        $api_set_obj->set_timeout("connectrec-timeout" => $original_timeout);
        $Log->comment("The current timeout value is $original_timeout");
        $result = 0;
    };
    return $result;
} ## end sub RaidSqa_disk_remove_ownership_func

=cut










1;

