#$Id: //depot/prod/test/Rsierranevada/lib/RaidSqav3.pm#1 $
##
## @summary RaidSqa.pm - RAID SQA wrapper library for RaidSqa_7mode.pm and RaidSqa_Cmode.pm
##
## @status TESTME
##
## @author pravin.salgond@netapp.com;
##
## @change
##
## @pod here
##
package RaidSqav3;

use strict;
no strict "subs";
no strict "refs";
require Exporter;
use vars qw(@ISA @EXPORT_OK);
use Data::Dumper;
use Tharn qw($Log say script_result host logcomment logresult param);
use NACL::Transit;
our $Mode = undef;

@ISA       = qw(Exporter);
@EXPORT_OK = qw(AUTOLOAD);

our $AUTOLOAD = undef;

sub AUTOLOAD {
    my @sub_args = @_;
    my %opts     = @sub_args;

    #defining new hash as using same hash adds unwanted parameter api_set_obj

    my $method = $AUTOLOAD;
    #logcomment("RaidSqa.pm:: InsideAutoload for $method");

    # If object has not defined destroy but has defined autoload, then on call of destroy, autoload will be called
    # this will throw error saying RaidSqa_Xmode::DESTROY is not defined
    if ($method =~ /DESTROY/) {
        &DESTROY();
        return 1;
    }
    $method =~ s/.*://;
 
    #If its not RaidSqa method, then its probably a subroutine from other lib which RaidSqa_7mode.pm imports
    if($method !~ /raidsqa/i) {
        $Log->comment("RaidSqav3: Warn: This is not RaidSqa method - $method, may be forgot to include corrosponding library");
    }

    if(defined($opts{api_set_obj}) && !$opts{api_set_obj}->api_is_session_alive()) {
	my $transit_obj  = NACL::Transit->new(name=>$opts{api_set_obj}->{hostobj});
	my $current_state = $transit_obj->get_state();
	if($current_state =~ /LOGIN|UP|CLI|SHELL/) {
		$opts{api_set_obj}->api_reconnect_session();
	}
    }
	
    my $host;
    if (defined($opts{host})) {
        $host = $opts{host};
    } elsif (defined($opts{api_set_obj}->{hostobj})) {
        $host = $opts{api_set_obj}->{hostobj};
    } elsif (defined($opts{node})) {
        $host = $opts{node};
    }
    
    if (!(defined($host) or defined($RaidSqa::Mode))) {
        die("RaidSqa.pm: host parameter is mandatory for loading RaidSqa modules - $method");
    }

    my $hosttype = $host->{hosttype};
    if (!defined($RaidSqa::Mode)) {
        $RaidSqa::Mode = ($hosttype =~ /ng/) ? "Cmode" : "7mode";
    }
    my $module = "RaidSqa_" . $RaidSqa::Mode . "v3";
    my $sub_qname     = $module . "::" . $method;    #This represents the qualified name of $method
    my $use_statement = "use $module";

    #Update parameters based on mode
    if ($RaidSqa::Mode eq "7mode" && !defined($opts{host})) {
        if ($method =~ /RaidSqa_/) {
            #logcomment("RaidSqa.pm: Adding host parameter to subroutine for 7mode");
            my %args = @sub_args;
            $args{host} = $host;
            @sub_args = %args;
        }
    }

    #Load module
    eval $use_statement;
    die("Error loading $module: $@") if ($@);
    #logcomment("RaidSqa.pm: Executing method $sub_qname");
    my @return_values = eval { &$sub_qname(@sub_args) };
    if ($@) {
        my $error = $@;     
        die("RaidSqa.pm: Error occured during execution $sub_qname: $error");
    } else {       
        if (wantarray) {
            return @return_values;
        } else {
            return shift @return_values;
        }
    } ## end else [ if ($@)
} ## end sub AUTOLOAD

sub DESTROY {
    return 1;
}

#package RaidSqa;
1;
