package randomData;

use strict 'vars';

sub convertSize
{
    my ($size) = @_;

    if ($size =~ /(\d+)m/i) {
	$size = $1 * 1024 * 1024;
    }
    if ($size =~ /(\d+)k/i) {
	$size = $1 * 1024;
    }
    return $size;
}

my @characters = (a..z);
my @uppercharacters = (A..Z);
my @numbers = (0..9);
my @hex = (a..f);
my @upperhex = (A..F);
my @validCifsUTF8;

# this fills the validCifsUTF8 array with all the UTF8 characters in the Latin-1 Character set
# we know Windows can send us
use bytes; # consider everything in this array a hex value
use utf8;  # consider the bytes as utf8 characters not ascii characters
foreach my $char( 0x41..0x5A, 0x61..0x7A, 0xAA, 0xBA, 0xC0..0xD6, 0xD8..0xF6, 0xF8..0xff,
    0x120..0x121, 0x123..0x129, 0x12b..0x12e, 0x130, 0x13b, 0x13d, 0x140..0x15b,
    0x15d..0x17b, 0x17d..0x17e, 0x1a1..0x1BA, 0x1BB, 0x1BC..0x1BF, 0x1C0..0x1C3,
    0x1C4..0x1ff, 0x223..0x229, 0x22b..0x22e, 0x230..0x233, 0x250..0x25b, 0x25d..0x27b,
    0x27d..0x27e, 0x2a1..0x2AD, 0x2B0..0x2B8, 0x2E0..0x2E4, 0x1e20..0x1e21, 0x1e23..0x1e29,
    0x1e2b..0x1e2e, 0x1e30..0x1e39, 0x1e3b, 0x1e3d, 0x1e40..0x1e5b, 0x1e5d..0x1e7b,
    0x1e7d..0x1e7e, 0x1ea1..0x1EF9, 0x212B, 0xFF21, 0xff23..0xff29, 0xff2b..0xff2e,
    0xff30..0xFF39, 0xFF41..0xFF5A){
    push @validCifsUTF8, chr($char);
}
no utf8;  # turn off utf8 character encoding
no bytes; # turn off byte considerations

package randomString;

sub new
{
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = {
	size => 1,
	hex => 0,
	mixcase => 0,
	upper => 0,
	number => 0,
	cifsutf8 => 0,
	randomsize => 0,
	@_
    };
    bless($self, $class);

    $self->{size} = randomData::convertSize($self->{size});
    my @array = ();
    if ($self->{hex}) {
	push(@array, @numbers);
	if ($self->{mixcase}) {
	    push(@array, @hex, @upperhex);
	} elsif ($self->{upper}) {
	    push(@array, @upperhex);
	} else {
	    push(@array, @hex);
	}
    } elsif ($self->{upper}) {
	@array = @uppercharacters;
    } elsif ($self->{number}) {
	@array = @numbers;
    } elsif ($self->{cifsutf8}) {
        push (@array, @validCifsUTF8);
    } else {
	@array = @characters;
	push(@array, @uppercharacters) if ($self->{mixcase});
    }
    $self->{max} = $#array + 1;
    $self->{array} = [@array];
    return $self;
}    

sub DESTROY
{
}

sub String
{
    my $self = shift;
    my %params = (
	size => undef,
	randomsize => undef,
	cifsutf8 => undef,
	@_,
    );
    my $string;
    
    # verify that we aren't overriding the objects settings
    if ( $params{size} ) {
	$self->{size} = $params{size};
    }
    if ( $params{randomsize} ) {
	$self->{randomsize} = $params{randomsize};
    }
    if ( $params{cifsutf8} ) {
	$self->{cifsutf8} = $params{cifsutf8};
    }
    
    my $ref = $self->{array};
    # if the randomsize flag is set, treat the size as a max size and get a random number for the size instead
    if ($self->{randomsize}){
	my $randMax = int(rand $self->{size}) || 1; # we never want 0
	for (my $i = 0; $i < $randMax; $i++) {
	    $string .= $$ref[rand($self->{max})];
	}
    } else {
	for (my $i = 0; $i < $self->{size}; $i++) {
	    $string .= $$ref[rand($self->{max})];
	}
    }
    return $string;
}

package randomFile;
sub new 
{
    my $this = shift;
    my $class = ref($this) || $this;
    my $self = {
	size => 0,
	random => 0,
	character => 'a',
	@_
    };
    bless($self, $class);

    my $size = 0;
    my $writesize;
    my $bytesleft = randomData::convertSize($self->{size});

    if (!exists($self->{writesize})) {
	if ($bytesleft > 65536) {
	    $writesize = 65536;
	} elsif ($bytesleft > 32768) {
	    $writesize = 32768;
	} elsif ($bytesleft > 16384) {
	    $writesize = 16384;
	} elsif ($bytesleft > 8192) {
	    $writesize = 8192;
	} elsif ($bytesleft > 4096) {
	    $writesize = 4096;
	} elsif ($bytesleft > 2048) {
	    $writesize = 2048;
	} else {
	    $writesize = 1024;
	}
	$self->{writesize} = ($bytesleft < $writesize) ? $bytesleft : $writesize;
    }
    $self->{bytesleft} = $bytesleft;

    return $self;
}

sub Write
{
    my ($self, $name, $debug) = @_;
    my $data;
    if ($name eq "") {
	die "Please specify a filename\n";
    }
    open(FILE, ">$name") || warn "Could not open $name: $!\n";
    my $bytesleft = $self->{bytesleft};
    while ($bytesleft > 0) {
	if ($self->{random}) {
	    $data = randomString->new(size => $self->{writesize},
				      hex => $self->{hex},
				      @_);
	    print FILE $data->String();
	} else {
	    print FILE $self->{character} x $self->{writesize};
	}
	$bytesleft -= $self->{writesize};
	print "wrote ", $self->{writesize}, " bytes\n" if ($debug);
    }
    close FILE;
}

sub DESTROY
{
}

1;
