#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/ResourceAgitation.pm#1 $

## @summary ResourceAgitation Interface
##
##dblade> setral ?
##setral ?
##usage: setral (Set Resource Agitation Levels)
##        [ show ] [ reset ]
##        [[ cpu       <consume-%> ]
##         [ malloc    <consume-%> <random-deny 1-out-of-N (0=off)> ]
##         [ kma_sleep <random-sleep 1-out-of-N (0=off)> <sleep N msecs>]
##         [ mbuf      <consume-%> <random-deny 1-out-of-N (0=off)> ]
##         [ waflbuf   <consume-%> <random-deny 1-out-of-N (0=off)> ]
##         [ fd        <consume-%> <random-deny 1-out-of-N (0=off)> ]
##         [ pBlk      <consume-%> <random-deny 1-out-of-N (0=off)> ]
##         [ raidrm    <consume-%> <random-deny 1-out-of-N (0=off)> ]
##         [ java      <random-deny 1-out-of-N (0=off)> ]
##         [ gc        <random-gabage collect 1-out-of-N (0=off)> ]
##         [ page      <random-deny 1-out-of-N (0=off)> ]
##         [ hseg      <random-deny 1-out-of-N (0=off)> ]
##         [ socket    <random-deny 1-out-of-N (0=off)> ]
##         [ nwdrop    <random-drop 1-out-of-N (0=off)> ]
##         [ nwcorrupt <random-corrupt 1-out-of-N (0=off)> ]
##         [ nscorrupt <random-corrupt 1-out-of-N (0=off)> ]
##         [ spinhicsm <random-deny 1-out-of-N (0=off)> ]
##         [ spinhimem <random-deny 1-out-of-N (0=off)> ]
##         [ nblhostrpc <random-deny 1-out-of-N (0=off)> ]
##         [ incl <rtag> ..]
##         [ excl <rtag> ..]]
##                        (The 'incl' or 'excl' list must be last)
##dblade>
##

## @author mrd@netapp.com
## @status private
## @pod here


package ResourceAgitation;

use strict;
use Server;
use Error;
use vars qw( @ISA $DXSERVER );

@ISA = qw(Error);


################################################################################
###
###  P U B L I C   F U N C T I O N S
###
################################################################################

################################################################################
### Subroutine Name: modify_agitation()
### Objective:       Modify agitation levels
### Details:         See POD documentation at the end of this file
################################################################################
sub modify_agitation {
    my ($self,%args)=@_;
    my ( 
		$cpu,
		$malloc,
		$kma_sleep,
		$mbuf,
		$waflbuf,
		$fd,
		$pBlk,
		$raidrm,
		$java,
		$gc,
		$page,
		$hseg,
		$socket,
		$nwdrop,
		$nwcorrupt,
		$nscorrupt,
		$spinhicsm,
		$spinhimem,
		$nblhostrpc,
		$incl,
		$excl,
		$nodes,
		$group,
		$dexserver,
		%options,
		@nodes,
	);

	$self->_clear_err();

	# Save the default execution server so that we can restore it after
	$self->_save_default_execution_server();

	# parse arguments
	foreach (keys %args) {
	  if (/-?cpu/) {
		#[ cpu       <consume-%> ]
	    if (not length($cpu = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-cpu'");
			return 0;
	    }        
		$options{cpu}=$self->_parse_arg($cpu);
	    next;
	  }
	  elsif (/-?malloc/) {
		#[ malloc    <consume-%> <random-deny 1-out-of-N (0=off)> ]
	    if (not length($malloc = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-malloc'");
			return 0;
	    }        
		$options{malloc}=$self->_parse_arg($malloc);
	    next;
	  }
	  elsif (/-?kma_sleep/) {
		#[ kma_sleep <random-sleep 1-out-of-N (0=off)> <sleep N msecs>]
	    if (not length($kma_sleep = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-kma_sleep'");
			return 0;
	    }        
		$options{kma_sleep}=$self->_parse_arg($kma_sleep);
	    next;
	  }
	  elsif (/-?mbuf/) {
		#[ mbuf      <consume-%> <random-deny 1-out-of-N (0=off)> ]
	    if (not length($mbuf = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-mbuf'");
			return 0;
	    }        
		$options{mbuf}=$self->_parse_arg($mbuf);
	    next;
	  }
	  elsif (/-?waflbuf/) {
		#[ waflbuf   <consume-%> <random-deny 1-out-of-N (0=off)> ]
	    if (not length($waflbuf = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-waflbuf'");
			return 0;
	    }        
		$options{waflbuf}=$self->_parse_arg($waflbuf);
	    next;
	  }
	  elsif (/-?fd/) {
		#[ fd        <consume-%> <random-deny 1-out-of-N (0=off)> ]
	    if (not length($fd = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-fd'");
			return 0;
	    }        
		$options{fd}=$self->_parse_arg($fd);
	    next;
	  }
	  elsif (/-?pBlk/) {
		#[ pBlk      <consume-%> <random-deny 1-out-of-N (0=off)> ]
	    if (not length($pBlk = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-pBlk'");
			return 0;
	    }       
		$options{pBlk}=$self->_parse_arg($pBlk);
	    next;
	  }
	  elsif (/-?raidrm/) {
		#[ raidrm    <consume-%> <random-deny 1-out-of-N (0=off)> ]
	    if (not length($raidrm = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-raidrm'");
			return 0;
	    }        
		$options{raidrm}=$self->_parse_arg($raidrm);
	    next;
	  }
	  elsif (/-?java/) {
		#[ java      <random-deny 1-out-of-N (0=off)> ]
	    if (not length($java = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-java'");
			return 0;
	    }        
		$options{java}=$self->_parse_arg($java);
	    next;
	  }
	  elsif (/-?gc/) {
		#[ gc        <random-gabage collect 1-out-of-N (0=off)> ]
	    if (not length($gc = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-gc'");
			return 0;
	    }        
		$options{gc}=$self->_parse_arg($gc);
	    next;
	  }
	  elsif (/-?page/) {
		#[ page      <random-deny 1-out-of-N (0=off)> ]
	    if (not length($page = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-page'");
			return 0;
	    }        
		$options{page}=$self->_parse_arg($page);
	    next;
	  }
	  elsif (/-?hseg/) {
		#[ hseg      <random-deny 1-out-of-N (0=off)> ]
	    if (not length($hseg = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-hseg'");
			return 0;
	    }        
		$options{hseg}=$self->_parse_arg($hseg);
	    next;
	  }
	  elsif (/-?socket/) {
		#[ socket    <random-deny 1-out-of-N (0=off)> ]
	    if (not length($socket = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-socket'");
			return 0;
	    }        
		$options{socket}=$self->_parse_arg($socket);
	    next;
	  }
	  elsif (/-?nwdrop/) {
		#[ nwdrop    <random-drop 1-out-of-N (0=off)> ]
	    if (not length($nwdrop = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-nwdrop'");
			return 0;
	    }        
		$options{nwdrop}=$self->_parse_arg($nwdrop);
	    next;
	  }
	  elsif (/-?nwcorrupt/) {
		#[ nwcorrupt <random-corrupt 1-out-of-N (0=off)> ]
	    if (not length($nwcorrupt = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-nwcorrupt'");
			return 0;
	    }        
		$options{nwcorrupt}=$self->_parse_arg($nwcorrupt);
	    next;
	  }
	  elsif (/-?nscorrupt/) {
		#[ nscorrupt <random-corrupt 1-out-of-N (0=off)> ]
	    if (not length($nscorrupt = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-nscorrupt'");
			return 0;
	    }        
		$options{nscorrupt}=$self->_parse_arg($nscorrupt);
	    next;
	  }
	  elsif (/-?spinhicsm/) {
		#[ spinhicsm <random-deny 1-out-of-N (0=off)> ]
	    if (not length($spinhicsm = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-spinhicsm'");
			return 0;
	    }        
		$options{spinhicsm}=$self->_parse_arg($spinhicsm);
	    next;
	  }
	  elsif (/-?spinhimem/) {
		#[ spinhimem <random-deny 1-out-of-N (0=off)> ]
	    if (not length($spinhimem = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-spinhimem'");
			return 0;
	    }        
		$options{spinhimem}=$self->_parse_arg($spinhimem);
	    next;
	  }
	  elsif (/-?nblhostrpc/) {
		#[ nblhostrpc <random-deny 1-out-of-N (0=off)> ]
	    if (not length($nblhostrpc = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-nblhostrpc'");
			return 0;
	    }        
		$options{nblhostrpc}=$self->_parse_arg($nblhostrpc);
	    next;
	  }
	  elsif (/-?incl/) {
		#[ incl <rtag> ..] (The 'incl' or 'excl' list must be last)
	    if (not length($incl = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-incl'");
			return 0;
	    }        
		$options{incl}=$self->_parse_arg($incl);
	    next;
	  }
	  elsif (/-?excl/) {
		#[ excl <rtag> ..] (The 'incl' or 'excl' list must be last)
	    if (not length($excl = $args{$_})) {
			$self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-excl'");
			return 0;
	    }        
		$options{excl}=$self->_parse_arg($excl);
	    next;
	  }
      elsif (/-?nodes/) {
        #node or a list of nodes to perform agitation on
		if (not length($nodes = $args{$_})) {
          $self->_set_err("Error: ResourceAgitation::modify_agiation() Invalid value for argument '-nodes'");
          return 0;
        }
		foreach my $n (split(/,/, $nodes)) {
          if (grep(/^${n}$/, @{$self->{nodes}})) {
            push(@nodes, $n);
          }
          else {
            $self->_set_err("Error: ResourceAgitation::modify_agiation() Invalid node name '-nodes'");
            return 0;
          }
		}
        next;
      }
      elsif (/-?group/) {
        # New option for predefined agitation groups
        if (not length($group = $args{$_})) {
          $self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid value for argument '-group'");
          return 0;
        }
        # Call this method "modify_agitation()" with all of the predefined group args
        eval {
          $self->modify_agitation($self->_get_group($group));
        };
        if ($@) {
          $self->_set_err("ERROR: ResourceAgitation::modify_agitation() failed during _get_group for group $group");
          return 0;
        }

		return 1;
        #next; 
      }
	  else {
	    $self->_set_err("Error: ResourceAgitation::modify_agitation() Invalid parameter specified \'$_\'");
	    return $self;                      	  
	  }
	}     

	if (scalar(@nodes < 1)) {
		@nodes = @{$self->{nodes}};
	}

	foreach my $node (@nodes) {
		eval {
			$self->{server}->set_execution_server(-Server => $node);
		};
		if ($@) {
			$self->_set_err("ERROR: ResourceAgitation::modify_agitation() Unable to set execution server to $node.",$@);
			return 0;
		}

		foreach my $option (keys %options) { 
			eval {
				$self->{server}->dblade(cmd => "priv set -q advanced; setral $option $options{$option}");
			};
			if ($@) {
				$self->_set_err("ERROR: ResourceAgitation::modify_agitation() Unable to set setral option.",$@);
				return 0;
			}
		}
	}

	# Setting the default execution server back so that nothing weird happens in the .thpl
	$self->_restore_default_execution_server();

    return 1;
}

################################################################################
### Subroutine Name: show_agitation()
### Objective:       Display current agitation configuration
### Details:         See POD documentation at the end of this file
################################################################################
sub show_agitation {
	my ($self,%args)=@_;
    my (
		$nodes,
		@buffer,
		@nodes,
		@output,
		%options,
	);

    $self->_clear_err();

	# Save the default execution server so that we can restore it after
	$self->_save_default_execution_server();

	# parse arguments
	foreach (keys %args) {
      if (/-?nodes/) {
        #node or a list of nodes to perform agitation on
		if (not length($nodes = $args{$_})) {
          $self->_set_err("Error: ResourceAgitation::show_agiation() Invalid value for argument '-nodes'");
          return 0;
        }
		foreach my $n (split(/,/, $nodes)) {
          if (grep(/^${n}$/, @{$self->{nodes}})) {
            push(@nodes, $n);
          }
          else {
            $self->_set_err("Error: ResourceAgitation::show_agiation() Invalid node name '-nodes'");
            return 0;
          }
		}
        next;
      }
	  else {
	    $self->_set_err("Error: ResourceAgitation::show_agitation() Invalid parameter specified \'$_\'");
	    return $self;                      	  
	  }
	}     

	if (scalar(@nodes < 1)) {
		@nodes = @{$self->{nodes}};
	}

	foreach my $node (@nodes) {
		eval {
			$self->{server}->set_execution_server(-Server => $node);
		};
		if ($@) {
			$self->_set_err("ERROR: ResourceAagitation::show_agitation() Unable to set execution server to $node.",$@);
			return 0;
		}

    	eval {
			@buffer=$self->{server}->dblade(cmd => "priv set -q advanced; setral show");
    	};
    	if ($@){
			$self->_set_err("ERROR: ResourceAagitation::show_agitation() Unable to show agitation configuration");
			return 0;
    	}

		unshift(@buffer, "$node\n");
	}

	# Setting the default execution server back so that nothing weird happens in the .thpl
	$self->_restore_default_execution_server();

	return (wantarray()) ? @buffer : [@buffer];
}

################################################################################
### Subroutine Name: reset_agitators()
### Objective:       Reset all agitators to default (off)
### Details:         See POD documentation at the end of this file
################################################################################
sub reset_agitators {
	my ($self,%args)=@_;
    my (
		$nodes,
		$agitator,
		$agitator_args,
		@nodes,
		%options,
	);

    $self->_clear_err();

	# Save the default execution server so that we can restore it after
	$self->_save_default_execution_server();

	# parse arguments
	foreach (keys %args) {
      if (/-?nodes/) {
        #node or a list of nodes to perform agitation on
		if (not length($nodes = $args{$_})) {
          $self->_set_err("Error: ResourceAgitation::reset_agitators() Invalid value for argument '-nodes'");
          return 0;
        }
		foreach my $n (split(/,/, $nodes)) {
          if (grep(/^${n}$/, @{$self->{nodes}})) {
            push(@nodes, $n);
          }
          else {
            $self->_set_err("Error: ResourceAgitation::reset_agitators() Invalid node name '-nodes'");
            return 0;
          }
		}
        next;
      }
      elsif (/-?agitator/) {
        if (not length($agitator = $args{$_})) {
            $self->_set_err("Error: ResourceAgitation::reset_agitators() Invalid value for argument '-agitator'");
            return 0;
        }
        $agitator_args=$self->_get_zero_values($agitator);
        next;
      }
	  else {
	    $self->_set_err("Error: ResourceAgitation::reset_agitators() Invalid parameter specified \'$_\'");
	    return $self;                      	  
	  }
	}     

	if (scalar(@nodes < 1)) {
		@nodes = @{$self->{nodes}};
	}

	foreach my $node (@nodes) {
		eval {
			$self->{server}->set_execution_server(-Server => $node);
		};
		if ($@) {
			$self->_set_err("ERROR: ResourceAagitation::reset_agitators() Unable to set execution server to $node.",$@);
			return 0;
		}

		if (length($agitator_args) > 0) {

			eval {
				$self->modify_agitation(
										-node => $node,
										-$agitator => $agitator_args,
										 );
			};
			if ($@) {
				$self->_set_err("ERROR: ResourceAgitation::reset_agitators() failed during modify_agitation");
				return 0;
			}
		}
		else {

			eval {
				$self->{server}->dblade(cmd => 'priv set -q advanced; setral reset');
			};
			if ($@) {
				$self->_set_err("ERROR: ResourceAagitation::reset_agitators() Unable to reset setral options.",$@);
				return 0;
			}
		}
	}

	# Setting the default execution server back so that nothing weird happens in the .thpl
	$self->_restore_default_execution_server();

    return 1;
}

################################################################################
### Subroutine Name: get_server()
### Objective:       Retrieve server handle
### Details:         See POD documentation at the end of this file
################################################################################
sub get_server {
    my ($self)=@_;
    return $self->{server};
}

################################################################################
### Subroutine Name: new()
### Objective:       Create a new ResourceAgitation object
### Details:         See POD documentation at the end of this file
################################################################################
sub new {
    my ($class, $server, %args) = @_;
    my ( 
		$self,
		$release,
	);

	$self={};

	bless $self,$class;

	### Check for required server variables
	unless (defined $server){
	    $self->_set_err("Error: ResourceAgitation::new() Required parameter missing: -server");
	    return $self;
	}
	$self->{server}=$server;

	$self->{nodes}=[$self->{server}->servers()];

	# set/modify all agitators
	eval{
		$self->modify_agitation(%args);
	};
	if ($@){
		$self->_set_err("ERROR: ResourceAgitation::new()  Unable to modify agitation levels");
		return $self;
	}

	return $self;

}

################################################################################
###
###  P R I V A T E   F U N C T I O N S
###
################################################################################

################################################################################
# Used to make sure we have resonable arguments to deal with
################################################################################
sub _parse_arg {
	my ($self,$arg) = @_;

	if ($arg =~ /\%/) {
		$arg =~ s/\%//;
	}

	if ($arg =~ /\,/) {
		$arg =~ s/\,/ /;
	}

	for (0..(length($arg) - 1)) {
		if (substr($arg,$_,1) !~ /\d|\s/) {
			$self->_set_err("ERROR: ResourceAgitation::_parse_arg() Argument contains invalid characters.",$@);
			return 0;
		}
	}

	return $arg;
}

################################################################################
# Save the default "Server.pm" execution server so that we can restore it
################################################################################
sub _save_default_execution_server {
	my ($self)=@_;

	$self->_clear_err();

	# Jumping through some hoops to get the currently set execution 
	# server so that we can restore it when we need to.
	eval {
		$DXSERVER = $self->{server}->set_execution_server(-Server => ${$self->{nodes}}[0]);
	};
	if ($@) {
		$self->_set_err("ERROR: ResourceAgitation::_save_default_execution_server() Unable to set execution server to ${$self->{nodes}}[0].",$@);
		return 0;
	}

	$self->_restore_default_execution_server();

	return 1;

}

################################################################################
# Restore the default "Server.pm" execution server so that nothing weird happens
################################################################################
sub _restore_default_execution_server {
	my ($self)=@_;

	$self->_clear_err();

	# Set it back to the default for now
	eval {
		$self->{server}->set_execution_server(-Server => $DXSERVER);
	};
	if ($@) {
		$self->_set_err("ERROR: ResourceAgitation::_restore_default_execution_server() Unable to set execution server to $DXSERVER.",$@);
		return 0;
	}

	return 1;
}

################################################################################
# Get the correct "zero" values for a given agitator to reset it independently
################################################################################
sub _get_zero_values {
	my ($self,$arg) = @_;
	my $zero_string;

	$self->_clear_err();

	if ($arg =~ /^cpu$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^malloc$) {
		$zero_string = '0 0';
	}
	elsif ($arg =~ /^kma_sleep$/) {
		$zero_string = '0 0';
	}
	elsif ($arg =~ /^mbuf$/) {
		$zero_string = '0 0';
	}
	elsif ($arg =~ /^waflbuf$/) {
		$zero_string = '0 0';
	}
	elsif ($arg =~ /^fd$/) {
		$zero_string = '0 0';
	}
	elsif ($arg =~ /^pBlk$/) {
		$zero_string = '0 0';
	}
	elsif ($arg =~ /^raidrm$/) {
		$zero_string = '0 0';
	}
	elsif ($arg =~ /^java$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^gc$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^page$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^hseg$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^socket$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^nwdrop$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^nwcorrupt$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^nscorrupt$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^spinhicsm$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^spinhimem$/) {
		$zero_string = '0';
	}
	elsif ($arg =~ /^nblhostrpc$/) {
		$zero_string = '0';
	}
	else {
		$self->_set_err("ERROR: ResourceAgitation::_get_zero_values() Unable to determine zero value for $arg");
       	return 0;
	}
	
	return $zero_string;
}

################################################################################
# Contains all of the predefined group definitions
################################################################################
sub _get_group {
	my ($self,$arg) = @_;
	my %nargs;

	$self->_clear_err();

	if ($arg =~ /network_high/) {
		%nargs = (
					'-socket' => 10,
					'-nwdrop' => 100,
					'-nwcorrupt' => 100,
				);
	}
	elsif ($arg =~ /network_low/) {
		%nargs = (
					'-socket' => 100,
					'-nwdrop' => 1000,
					'-nwcorrupt' => 1000,
				);
	}
	elsif ($arg =~ /system_high/) {
		%nargs = (
					'-malloc' => "10 100",
					'-kma_sleep' => "10 100",
					'-mbuf' => "10 100",
					'-pBlk' => "10 100",
					'-waflbuf' => "10 100",
					'-nwdrop' => 100,
					'-nwcorrupt' => 300,
				);
	}
	elsif ($arg =~ /system_low/) {
		%nargs = (
					'-malloc' => "1 50000",
					'-kma_sleep' => "50000 1",
					'-mbuf' => "1 50000",
					'-pBlk' => "1 50000",
					'-waflbuf' => "1 50000",
					'-nwdrop' => 50000,
					'-nwcorrupt' => 150000,
				);
	}
	elsif ($arg =~ /filer_high/) {
		%nargs = (
					'-mbuf' => "0 10",
					'-nwdrop' => 10,
					'-cpu' => 50,
					'-malloc' => "75 100",
				);
	}
	elsif ($arg =~ /filer_low/) {
		%nargs = (
					'-mbuf' => "0 10000",
					'-nwdrop' => 10000,
					'-cpu' => 5,
					'-malloc' => "7 1000",
				);
	}
	elsif ($arg =~ /raid_high/) {
		%nargs = (
					'-raidrm' => "5 5",
				);
	}
	elsif ($arg =~ /raid_low/) {
		%nargs = (
					'-raidrm' => "1 50",
				);
	}

	return %nargs;
}

1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

ResourceAgitation.pm

=head1 Author

Michael Detweiler

=head1 Description

ResourceAgitation.pm provides an API interface for working with cluster-wide system resource agitation.
The ResourceAgitation object uses the "setral" command, found in the dlblade to create agitation/system 
load scenarios on a given filer or cluster of filers.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error>

=head1 Public Methods

=head2 B<modify_agitation(I<-cpu, -malloc, -kma_sleep, -mbuf, -waflbuf, -fd, -pBlk, -raidrm, -java, -gc, -page, -hseg, -socket, -nwdrop, -nwcorrupt, -nscorrupt, -spinhicsm, -spinhimem, -nblhostrpc, -incl, -excl, -node, -group>)>

###

This method modifies cluster-wide or node specific "setral" configuration attributes

=over 4

=item Inputs

=item * I<-cpu> default:0

<consume-%>

=item * I<-malloc> default: 0 0 

<consume-%> <random-deny 1-out-of-N (0=off)>

=item * I<-kma_sleep> default: 0 0 

<random-sleep 1-out-of-N (0=off)> <sleep N msecs>

=item * I<-mbuf> default: 0 

<consume-%> <random-deny 1-out-of-N (0=off)>

=item * I<-waflbuf> default: 0

<consume-%> <random-deny 1-out-of-N (0=off)> 

=item * I<-fd> default: 0 

<consume-%> <random-deny 1-out-of-N (0=off)>

=item * I<-pBlk> default: 0

<consume-%> <random-deny 1-out-of-N (0=off)> 

=item * I<-raidrm> default: 0 0

<consume-%> <random-deny 1-out-of-N (0=off)> 

=item * I<-java> default: 0

<random-deny 1-out-of-N (0=off)> 

=item * I<-gc> default: 0

<random-garbage collect 1-out-of-N (0=off)>

=item * I<-page> default: 0

<random-deny 1-out-of-N (0=off)>

=item * I<-hseg> default: 0

<random-deny 1-out-of-N (0=off)> 

=item * I<-socket> default: 0

<random-deny 1-out-of-N (0=off)> 

=item * I<-nwdrop> default: 0

<random-drop 1-out-of-N (0=off)> 

=item * I<-nwcorrupt> default: 0

<random-corrupt 1-out-of-N (0=off)>

=item * I<-nscorrupt> default: 0

<random-corrupt 1-out-of-N (0=off)>

=item * I<-spinhicsm> default: 0

<random-deny 1-out-of-N (0=off)> 

=item * I<-spinhimem> default: 0

<random-deny 1-out-of-N (0=off)> 

=item * I<-nblhostrpc> default: 0

<random-deny 1-out-of-N (0=off)> 

=item * I<-incl> default: 0

<rtag> ..
(The 'incl' or 'excl' list must be last)

=item * I<-excl> default: 0

<rtag> ..
(The 'incl' or 'excl' list must be last)

=item * I<-node> default: undef

A node or comma separated list of valid cluster nodes to perform agitation on

=item * I<-group> default: undef

A predefined agitation group name (network_high, network_low, system_high, system_low, filer_high, filer_low, raid_high, raid_low)

=item Outputs

Returns a scalar value of 1 if modification was successful, otherwise 0. If an error occured 
the error flag is set, errmsg is set to the failure reason.

=item Examples

     # Set the cpu to sqander 50%
     $rtn=$ResourceAgitation_obj->modify_agitation(-cpu => '50%');

=back

=head2 B<agitation_show()>

This method shows the current cluster-wide agitation configuration

=over 4

=item Inputs

None

=item Outputs

Returns an array or array ref containing the output.  If an error occurred the error flag is set, 
errmsg is set to the failure reason.

=item Examples

     # Display the current agitation configuration
     $Test->nlog($ResourceAgitation_obj->agitation_show());

=back

=head2 B<agitation_reset()>

Reset all agitation levels to defaults (Stop all agitation)

=over 4

=item Inputs

None

=item Outputs

Returns a scalar value of 1 if successful, otherwisw 0.  If an error occured the error flag is set, errmsg is set to the failure
reason.

=item Examples

     # Reset all agitation levels
     $ResourceAgitation_obj->agitation_reset();

=back

=head2 B<new($server,I<-loghdl>)>

Create a new ResourceAgitation object to manage setral levels

=over 4

=item Inputs

=item * B<$server> 

Server object to use by test.  All nodes in the server list will be used
in agitation.

=item Outputs

Returns an ResourceAgitation object handle.  If an error occured the error flag is set, 
errmsg is set to the failure reason.

=item Examples

     # Retrieve ResourceAgitation oblect value
     $ResourceAgitation_obj=ResourceAgitation->new($server);
    

=back

=cut
