#Hey emacs, this is basically -*- perl -*-
# $Id$

#***********************************************************************
#
# NAME
#   SHM.pm - programatic interfaces for System Health Monitor automation
#
# SYNOPSIS
#   Enables remote programmatic interfaces of System Health  Automation
#
# DESCRIPTION
#   This Perl module is meant to be programmatic way of automating 
#   Diagnosis project
#
# AUTHOR
#       dl-csi-qa
#
#***********************************************************************


#use vars qw/$Server $FILER $NOHEALTHCHECK/;

#***********************************************************************
#Function Name	:start
#Paremeters	:NULL
#Return Type	:0 on sucess and Error code on failure
#Description	:It tests whether system is ready to start test.
#***********************************************************************
#use strict;
package SHM;

use Data::Dumper;
use Server;
use Tharn qw($Log snooze host connect script_result fatal_result);
use boot qw(reboot reboot_to boot transit_to); 
use Options;
use Params::Common;
use Params::Validate qw(:all);
use NACL::MTask::SCP;
use Carp;
use Sys::Hostname;

use File::Path;
use File::Basename;
use NACL::APISet;
use NACL::Transit;
use NACL::C::Node;
use NACL::C::Cluster;
use NACL::C::CommandInterface;
use NACL::C::DebugMtraceBuffer;
use NACL::C::StorageFailover;
use NACL::C::SystemHealthAlert;
use NACL::C::SystemHealthConfig;
use NACL::C::SystemHealthAlertDefinition;
use NACL::C::SystemHealthPolicyDefinition;
use NACL::C::SystemHealthSubscriptions;
use NACL::C::SystemHealthSubsystem;
use NACL::C::SystemHealthAutosupportTriggerHistory;
use NACL::CS::DebugMtraceLog;
use NACL::CS::SystemHealthStatus;
use NACL::CS::SystemHealthNodeConnectivityAdapter;
use NACL::CS::SystemHealthNodeConnectivityDisk;
use NACL::CS::SystemHealthNodeConnectivityShelf;
use NATE::BaseException qw(:try);
use NACL::Exceptions::NoElementsFound;
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::Exceptions::OperationFailed;
use Clone;
use EMS;
use MOUNT_SVC;
use NATE::Inc qw(find_file);

my $Server = new Server();
my @node_names  = $Server->servers();
my @Nodes = NACL::C::Node->find();
my $Version = EMS::_get_version();
my $EHM_POLICY_FILE="ehm_policy.hmi";
my $EHM_SYSCTL_STRING="hw.hm.example_sysctl";
my $EHM_ALERTID1="FoodGettingOld_Alert";
my $EHM_ALERTID2="MoldyBread_Alert";
my $EHM_RESOURCE1="Bernard";
my $EHM_RESOURCE2="Marge";
my $EHM_RESOURCE3="Olivia";
my @EMS_ALERTS=("mgmt.nfsserver.failure","mgmt.nfsserver.success");
my $EHM_LOG;
my $HM_DIR="/mroot/etc/hm";

my $EMS_NCHM_FILE = NATE::Inc::find_file( "SHM/nchm_custom_policy_ems.hmi" );

my $_ontap_version;

sub _get_nacl_node {
    my $node_name = shift;
    my $node;

    foreach my $n (@Nodes) {
        my $nodeN = $n->name();
        if ( $nodeN eq $node_name ) {
            $node = $n;
            last;
        }
    }   

    if ( !$node ) {
        NATE::BaseException->throw( "SHM::_get_nacl_node Could not look up NACL node from $node_name" );
    }

    return $node;
}

# Borrowed from ASUP.pm.  We were including ASUP.pm which includes a TON of
# stuff, all for this simple routine
###############################################################################
#Sub: _get_version
#Input Params: <none>
#Return values: Scalar of the version running
#Summary: Retrieves the current Ontap version running on the node
###############################################################################
sub _get_version {
    my %params = (
            force=>0, # 1 = do not use cached value of $_ontap_version (re-query filer)
            @_,
    );

    if ((!$_ontap_version) || ($params{force} eq '1')) {

        if (!$_ontap_version) {
            $Log->comment("ONTAP version is unknown; obtaining from filer...");
        }
        elsif ($params{force}) {
            $Log->comment("Force parameter specified; obtaining version from filer...");
        }

        my $build;
        my $version;
        my $Node = $Nodes[0];

        # Automation branch test:main now reflects Sierra Nevada
        # We will therefore assume SN and set 10M or 7M based on result from NACL
        if ($Node->mode() eq 'CMode') {
            $version="SN-10M";
        }
        else {
            $version="SN-7M";
        }

        $Log->comment("_get_version: Found node is version $version \n");
        $_ontap_version = $version;
    }
    else {
        $Log->comment("_get_version: Using cached ONTAP version $_ontap_version \n");
    }

    return $_ontap_version;
}

#***********************************************************************
#Function Name	:system_health_status_is_bad
#Paremeters	:NULL
#Return Type	:0-OK;1-OK_with_suppressed;2-degraded;3-unreachable;4-other
#Description	:It gives the status of the system health monitor. 
#***********************************************************************

sub system_health_status_is_bad {
	my $out = health_monitor_status_show();
	if($out =~ /OK_with_suppressed/i) {
		return 1;
	} elsif ($out =~ /degraded/i) {
		return 2;
	} elsif ($out =~ /OK/i) {
		return 0;
	} elsif ($out =~ /unreachable/i) {
		return 3;
	} else {
		return 4;
	}
}

#***********************************************************************
#Function Name	:get_subsystems
#Paremeters	:optional command option and its value
#Return Type	:Reruns available subsystems in the cluster
#Description	:It runs system health config show command and list available subsystems
#***********************************************************************
sub get_subsystems
{
	return get_health_config_values(config=>"subsystem");
}

#***********************************************************************
#Function Name	:get_alertids
#Paremeters	:optional command option and its value
#Return Type	:Reruns available alerts in the cluster
#Description	:It runs system health alerts show command and list available alert ids
#***********************************************************************
sub get_alertids
{
	return get_health_config_values(config=>"alertid");
}

#***********************************************************************
#Function Name	:get_hms;
#Paremeters	:optional command option and its value
#Return Type	:Reruns available HMs  in the cluster
#Description	:It runs system health config show command and list available health monitors
#***********************************************************************
sub get_hms
{
	return get_health_config_values(config=>"health");
}

#***********************************************************************
#Function Name  :get_SCHM_node;
#Paremeters     :optional command option and its value
#Return Type    :Reruns available node where SCHM is running
#Description    :It runs system health config show command and list available health monitors
#***********************************************************************
sub get_SCHM_node
{
        if ( SHM::_get_version() =~ /7M/ ) {
		$Log->comment("No SCHM in 7M, returning NCHMD node");
		my @nodesarr=$Server->servers();
		return $nodesarr[0];
	}
	my $Node = _get_nacl_node( $node_names[0] );
	my $node_SCHM="";
		try{
			my @vals   = NACL::CS::SystemHealthConfig->fetch(
				command_interface => $Node,
				);
			foreach $val (@vals){
				if ($val->monitor eq "system-connect"){
					$node_SCHM=$val->node;
				}
			}
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			start_ehm();
			Tharn::sleep(60);
			my @vals   = NACL::CS::SystemHealthConfig->fetch(
				command_interface => $Node,
				);
			foreach $val (@vals){
				if ($val->monitor eq "system-connect"){
					$node_SCHM=$val->node;
					last;
				}
			}
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};	

	$Log->comment("Return schm node: $node_SCHM");
    return $node_SCHM;
}

#***********************************************************************
#Function Name  :get_SHM_node;
#Paremeters     :optional command option and its value
#Return Type    :Reruns available node where SHM is running
#Description    :It runs system health config show command and list available health monitors
#***********************************************************************
sub get_SHM_node
{
        if ( SHM::_get_version() =~ /7M/ ) {
		$Log->comment("No SCHM in 7M, returning NCHMD node");
		my @nodesarr=$Server->servers();
		return $nodesarr[0];
	}	

	my $node_SHM="";
		try{
		    FOO: {
		       foreach $node (@Nodes) {    
			      my @vals   = NACL::CS::SystemHealthConfig->fetch(
				     command_interface => $node,
				  );
			      foreach $val (@vals) {
				     if ($val->monitor eq "system") {   
					   $node_SHM=$val->node;
					   last FOO;
				     }
			      }
		       }
		    }
			
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			$state=$node_SHM;
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};	

	# if no node has system monitor, return the first node. won't break test
	if ( $node_SHM eq "" ) {  
	    return $Nodes[0];
	} else {
       return $node_SHM;
	}
}

#***********************************************************************
#Function Name	:get_aggrs
#Paremeters	:hm
#Return Type	:Returns all the aggregators of given hm
#Description	:It runs system health config show command and returns the aggrgator
#***********************************************************************
sub get_aggrs
{
	my %params = (
            monitor=>undef,
	    @_
	);
	if (!defined($params{monitor})){
		$Log->error("No Monitor Specified");
		return 0;
	}
	return _get_health_config_values(monitor=>$params{monitor},fields=>"aggregator");
}

#***********************************************************************
#Function Name	:_get_health_config_values
#Paremeters	:hm
#Return Type	:Returns all the values of given hm
#Description	:It runs system health config show command and returns the values based on params
#***********************************************************************
sub _get_health_config_values{
	my %params = (
	    node=>"*",
            monitor=>undef,
	    fields=>undef,
	    @_
	);
	my @values;
        if ( SHM::_get_version() =~ /7M/ ) {
		$Log->comment("Invoking the 7M command");
		my $options="-node local";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{fields})){
			$options=$options." -fields $params{fields}";
		}			
		my @show_details=split(/\n/,($Server->cli(cmd=>"system health config show $options")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->comment("Value:  $show_details[$i]");
			my @temps=split(/\s/,$show_details[$i]);
			if (length($temps[-1]) > 0 ){
				push(@values,$temps[-1]);
				next;
			}
		}		
	}else{	
		my $NGSH = $Nodes[0]->apiset(
		    interface => "CLI",
		    );
		$NGSH->set_privilege('privilege-level' => "diag");
		my $response = $NGSH->system_health_config_show(node=>$params{node}, monitor=>$params{monitor}, fields=>"$params{fields}");
		@parsed_output = @{$response->get_parsed_output()}; 
		foreach my $hash (@parsed_output){
			foreach my $hvalue (keys %{$hash}){
				$Log->debug("Hash Key for Cluster Show: $hvalue");
				if ( $hvalue eq $params{fields}){
					push(@values,$hash->{$hvalue});
					$Log->debug("Adding Execution Node Name $node"); 
				}
			}
		}
	}
	$Log->comment("System Health Config Values:  @values");
	return @values;
}

#***********************************************************************
#Function Name	:_get_health_policy_definition_values
#Paremeters	:hm
#Return Type	:Returns all the values of given hm
#Description	:It runs system health config show command and returns the values based on params
#***********************************************************************
sub _get_health_policy_definition_values{
	my %params = (
	    node=>"*",
            monitor=>undef,
	    fields=>undef,
	    @_
	);
	my @values;
        if ( SHM::_get_version() =~ /7M/ ) {
		$Log->comment("Invoking the 7M command");
		my $options="-node local";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{fields})){
			$options=$options." -fields $params{fields}";
		}			
		my @show_details=split(/\n/,($Server->cli(cmd=>"system health policy definition show $options -instance")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			my @temps=split(/\: /,$show_details[$i]);
			if ($temps[0] eq "Policy" ){
				push(@values,$temps[-1]);
				next;
			}
		}		
	}else{		
		my $NGSH = $Nodes[0]->apiset(
		    interface => "CLI",
		    );
		$NGSH->set_privilege('privilege-level' => "diag");
		my $response = $NGSH->system_health_policy_definition_show(node=>$params{node}, monitor=>$params{monitor}, fields=>"$params{fields}");
		@parsed_output = @{$response->get_parsed_output()}; 
		foreach my $hash (@parsed_output){
			foreach my $hvalue (keys %{$hash}){
				$Log->debug("Hash Key for Cluster Show: $hvalue");
				if ( $hvalue eq $params{fields}){
					push(@values,$hash->{$hvalue});
					$Log->debug("Adding Execution Node Name $node"); 
				}
			}
		}
	}
	$Log->debug("System Health Policy Definition Values:  @values");
	return @values;
}

#***********************************************************************
#Function Name	:_policies_enabled
#Paremeters	:none
#Return Type	:Returns 1 if all the policies are enabled, 0 if any are disabled
#Description	:
#***********************************************************************
sub _policies_enabled{
	my @policies=get_policy_ids(monitor=>"*");
	my $result=1;
	foreach $policy (@policies){
		my @show_details=split(/\n/,($Server->cli(cmd=>"system health policy definition show -policy-id $policy -instance")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			my @temps=split(/\: /,$show_details[$i]);
			if (lc($temps[0]) eq "policy status" ){
				if ( lc($temps[-1]) eq "false" ){
					$result=0;
					last;
				}
			}
		}		
	}
	$Log->debug("Check_policy state return value: $result");
	return $result;
}

#***********************************************************************
#Function Name	:get_policy_ids
#Paremeters	:hm
#Return Type	:Returns all the policy id of given hm
#Description	:It runs system health policy definition show command and returns the policy_id
#***********************************************************************
sub get_policy_ids
{
	my %params = (
            monitor=>undef,
	    @_
	);
	if (!defined($params{monitor})){
		$Log->error("No Monitor Specified");
		return 0;
	}
	return _get_health_policy_definition_values(monitor=>$params{monitor},fields=>"policy-id");
}

#***********************************************************************
#Function Name	:get_hm_state
#Paremeters	: hm
#Return Type	:Reruns state of HM
#Description	:It runs system health config show command and returns the state of requested hm
#***********************************************************************
sub get_hm_state
{
	my %params = (
            monitor=>undef,
	    @_
	);
	if (!defined($params{monitor})){
		$Log->error("No Monitor Specified");
		return 0;
	}
	return _get_health_config_values(monitor=>$params{monitor},fields=>"health");
}


#***********************************************************************
#Function Name	:get_nchm_adapters
#Paremeters	:node(optional)
#Return Type	:Returns list of adapters listed in nchm
#Description	:It runs system health node-connectivity adapter show and returns list of adapters
#***********************************************************************
sub get_nchm_adapters
{
	return get_node_connectivity_hwvalues(hwtype=>"adapter");
}


#***********************************************************************
#Function Name	:get_nchm_disks
#Paremeters	:node(optional)
#Return Type	:Returns list of disks listed in nchm
#Description	:It runs system health node-connectivity disks show and returns list of disks
#***********************************************************************
sub get_nchm_disks
{
	return get_node_connectivity_hwvalues(hwtype=>"disk");
}

#***********************************************************************
#Function Name	:get_nchm_shelves
#Paremeters	:node(optional)
#Return Type	:Returns list of shelves listed in nchm
#Description	:It runs system health node-connectivity shelf show and returns list of shelves
#***********************************************************************
sub get_nchm_shelves
{
	return get_node_connectivity_hwvalues(hwtype=>"shelf");
}

#***********************************************************************
#Function Name	:get_sas_channels
#Paremeters	:NULL
#Return Type	:Returns list of available SAS channels(adapters)
#Description	:It sends sasadmin channels command to dblade cli and returns available channels
#***********************************************************************
sub get_sas_channels
{
	my %params = (
            adapter=>undef,
	    @_
	);
	if (!defined($params{adapter})){
		$Log->error("No Adapter Specified");
		return 0;
	}
	my $cmd='sasadmin channels';
        my $cmdout = run_dblade_command(command=>$cmd);	
	my @cmdout_array=split(/\n/,$cmdout);
	my @channels;
	foreach(@cmdout_array)
	{
		my @array_out=split(/\s+/);
		push(@channels,$array_out[1]);# unless(grep(/$array_out[1]/,@shelves));
	}
	$Log->debug("SAS Channels:  @channels");
	return @channels;	
}

#***********************************************************************
#Function Name	:get_sas_shelves
#Paremeters	:sas adapter(optional)
#Return Type	:Returns list of available SAS shelves for the given adapter. provides all shelves if not passed adapter
#Description	:It sends sasadmin shelf command to dblade cli and returns available shelves
#***********************************************************************
sub get_sas_shelves
{
	my %params = (
            adapter=>undef,
	    @_
	);
	if (!defined($params{adapter})){
		$Log->error("No Adapter Specified");
		return 0;
	}
	my $cmd='sasadmin shelf '.$params{adapter};
        my $cmdout = run_dblade_command(command=>$cmd);		
	my @cmdout_array=split(/\n/,$cmdout);
	my @shelves;
	my $adapter;
	my $idflag=0;
	my $opened=0;
	foreach(@cmdout_array)
	{
		next if(/Cannot Complete operation/);
		if(/-------/)
		{
			$idflag=1;
			next;
		}
		if(/Expanders on channel/)
		{
			my @array_out=split(/\s+/);
			chop($array_out[3]);
			$adapter=$array_out[3];	
			next;
		}	
		if($idflag)
		{
			if($opened)
			{
				$opened=0;	
				$idflag=0;
				next;
			} else {
				my @array_out=split(/\|/);
				$array_out[0] =~ /\d+/;
				my $shelfid=$&;
				$idflag=0;
				my $shelf=$adapter.'.shelf'.$shelfid;
				push(@shelves,$shelf);
				$opened=1;
			}
		}
	}
	$Log->debug("SAS Shelves:  @shelves");
	return @shelves;
}

#***********************************************************************
#Function Name	:get_sas_disks
#Paremeters	:sas adapter(optional)
#Return Type	:Returns list of available SAS disks for the given adapter. provides all disks if not passed adapter
#Description	:It sends sasadmin expander command to dblade cli and returns available shelves
#***********************************************************************
sub get_sas_disks
{
	my %params = (
            adapter=>undef,
	    @_
	);
	if (!defined($params{adapter})){
		$Log->error("No Adapter Specified");
		return 0;
	}
	my @adapters;
	if($params{adapter})
	{
		push(@adapters,$params{adapter});
	} else {
		@adapters = &get_sas_channels;
	}
	my @disks;
	foreach $adapter(@adapters)
	{
		my $cmd='sasadmin expander '.$params{adapter};
		my $cmdout = run_dblade_command(command=>$cmd);			
		my @cmdout_array=split(/\n/,$cmdout);
		#tstlog -comment, "ADAPTERS:@adapters  C ADAPTER:$adapter";
		foreach(@cmdout_array)
		{
			if(/SAS Device/)
			{
				next if(/Virtual/);
				/\d\w\.\d{2}\.\d+/;
				push(@disks,$&);
			}	
		}
	}
	#
	$Log->debug("SAS Disks:  @disks");
	return @disks;
}

#***********************************************************************
#Function Name	:get_sas_adapter_state
#Paremeters	:sas adapter
#Return Type	:Returns list of available SAS disks for the given adapter. provides all disks if not passed adapter
#Description	:It sends sasadmin expander command to dblade cli and returns available shelves
#***********************************************************************
sub get_sas_adapter_state
{
	my %params = (
            adapter=>undef,
	    @_
	);
	if (!defined($params{adapter})){
		$Log->error("No Adapter Specified");
		return 0;
	}
        my $cmd='sasadmin adapter_state '.$params{adapter};
        my $cmdout = run_dblade_command(command=>$cmd);	
	my @cmdout_array=split(/\n/,$cmdout);
	foreach (@cmdout_array)
	{
		if(/State for adapter channel/)
		{
			tstlog -comment, "$LINE:$_";
			/[A-Z][A-Z]+/;
			return $&;
		}
	}
	return 0;
}

#***********************************************************************
#Function Name  :disable_sas_adapter
#Paremeters     :sas adapter
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It sends sasadmin adapter_disable <adapter name> to the dblade cli
#***********************************************************************
sub disable_sas_adapter
{
	my %params = (
            adapter=>undef,
	    node=>$node_names[0],
	    @_
	);
	my $count =0;
	$Log->debug("Attempting to make adpater $params{adapter} offline...");
	if (!defined($params{adapter})){
		$Log->error("No Adapter Specified");
		return 0;
	}
    my $cmd='sasadmin adapter_offline '.$params{adapter};
    my $cmdout = run_dblade_command(node=>$params{node},command=>$cmd);
    while ($count < 30) { 
       my $state = get_adapter_state();
       if( $state !~ /OFFLINE|DOWN/i ) {
          $count++;
          Tharn::sleep(5) ;
       }  else {
          $Log->debug("Adapter $params{adapter} reached disable state");
          last;
       }   
    }
    if ($count >= 30) {
        $Log->warn("Adapter $params{adapter} cannot reach disable state");
        return 0;
    } else {
        return 1;
    }
}


#***********************************************************************
#Function Name  :disk_power_on
#Paremeters     :sas disk
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It finds phy number and power on through sasadmin
#***********************************************************************
sub disk_power_on
{
	my ($shelf_id,$phy)=@_;
	my %params = (
            shelf_id=>undef,
	    phy=>undef,
	    @_
	);
	$Log->debug("Attempting to power on $params{shelf_id}...");
	if (!defined($params{adapter}) && !defined($params{phy}) ){
		$Log->error("No shelf id or physical device Specified");
		return 0;
	}	
	my $cmd='sasadmin expander_phy_power_on '."$shelf_id".' '."$phy".'"';
	my $cmdout = run_dblade_command(command=>$cmd);
	return 1;
}
#TBD:Need to verify whether state is changed or not and return 0 if not changed

#***********************************************************************
#Function Name  :get_phy_num
#Paremeters     :sas disk
#Return Type    :Returns phy number on success 0 on Failure
#Description    :It finds phy number through sasadmin
#***********************************************************************
sub get_phy_num
{
	my %params = (
            disk=>undef,
	    @_
	);
	if (!defined($params{disk})){
		$Log->error("No Disk Specified");
		return 0;
	}
	#assuming the shelf id is the first item in the disk name
	#0b.02.21
	#$offset="-3";
	#$offset="-2" if (length($disk) == 7);
	#my $shelf_id= substr $disk,0,$offset;
	my @disk_desc = split(/\./,$params{disk});
	if (length($disk_desc[0]) < 2 ){
		$Log->error("Unable to retrieve shelf id from disk name $params{disk}");
		return 0;
	}
	my $cmd='sasadmin expander '.$disk_desc[0];
	my $cmdout = run_dblade_command(command=>$cmd);
	$Log->debug("Expander output:  $cmdout");
	my @outlines=split(/\n/,$cmdout);
	$Log->debug("Outlines: @outlines");	
	my $phy=0;
	foreach my $line (@outlines)
	{
		$Log->debug("Expander Line:  $line");
		if($line =~ /$params{disk}/)
		{
			$Log->debug("Found disk $params{disk} in line $line");
			$line=~/\d+\//;
			$phy=$&;
			chop $phy;	
			last;
		}	
	}
	$Log->debug("Physical disk number $phy");
	return $phy;
}

#***********************************************************************
#Function Name  :disk_power_off
#Paremeters     :sas disk
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It finds phy number and power off through sasadmin
#***********************************************************************
sub disk_power_off
{
	my ($shelf_id,$phy)=@_;
	my %params = (
            shelf_id=>undef,
	    phy=>undef,
	    @_
	);
	$Log->debug("Attempting to power on $params{shelf_id}...");
	if (!defined($params{adapter}) && !defined($params{phy}) ){
		$Log->error("No shelf id or physical device Specified");
		return 0;
	}	
	my $cmd='sasadmin expander_phy_power_off '."$shelf_id".' '."$phy".'"';
	my $cmdout = run_dblade_command(command=>$cmd);
	return 1;
}
#TBD:Need to verify whether state is changed or not and return 0 if not changed

#***********************************************************************
#Function Name  :enable_sas_adapter
#Paremeters     :sas adapter
#Return Type    :Returns 1 on success 0 on Failure
#Description    :It sends sasadmin adapter_enable <adapter name> to the dblade cli
#***********************************************************************
sub enable_sas_adapter
{
	my %params = (
            adapter=>undef,
	    @_
	);
	$Log->debug("Attempting to enable adapater $params{adapter}...");
	if (!defined($params{adapter})){
		$Log->error("No Adapter Specified");
		return 0;
	}
        my $cmd='sasadmin adapter_online '.$params{adapter};
        my $cmdout = run_dblade_command(command=>$cmd);
#TBD:Need to verify whether state is changed or not and return 0 if not changed
}

#***********************************************************************
#Function Name  :wait_for_alert;
#Paremeters     :Null
#Return Type    :Returns 1 if alerts arriaved with in 15 mins. 0 otherwise
#Description    :It sends alert show for 30 sec and returns 1 if there is no 'This table is currently empty' message;
#***********************************************************************
sub wait_for_alert
{
	my %params = (
            count=>0,
	    node=>"local",
	    'alert-id'=>undef,
	    @_
	);
	my $version=SHM::_get_version();
        if ( $version =~ /7M/ ) {
		if ( !defined($params{node})){
			@nodes=get_nodes(type=>"name");
			$params{node}=$nodes[0];
		}
	}
	while ($params{count} < 15)
	{
		my $result=health_alert_show(node=>$params{node});
		if ( defined($params{'alert-id'}) ){
			$result=health_alert_show('alert-id'=>$params{'alert-id'},node=>$params{node});
		}
		$Log->comment("wait_for_alert - Result of alert show: $result");
		#node names shouldn't have a space character, only error messages
		if($result =~ /There are no entries matching your query/ || $result =~ /No matching system health alert/ )
		{
			Tharn::sleep(60);
			$params{count}++;
		} else {
			return 1;
		}
	}
	$Log->warn("After 15 minutes, the expected alert doesn't come... please investigate your filer to see if the policy enabled.");
	return 0;	
}



#***********************************************************************
#Function Name  :wait_for_ok;
#Paremeters     :Null
#Return Type    :Returns 1 if status changed to OK with in 15 minutes. 0 otherwise
#Description    :It sends status show for 30 sec and returns 1 if status is OK. 0 otherwise
#***********************************************************************
sub wait_for_ok
{
my $count=0;
while ($count < 30)
{
	my $cmdout=health_monitor_status_show();
	if($cmdout =~ /OK/i and $cmdout !~ /OK_with_suppressed/i)
	{
		return 1;
	} else {
	Tharn::sleep 30;
	$count++;
	}
}
return 0;
}


#***********************************************************************
#Function Name  :get_nodes
#Paremeters     :name, object
#Return Type    :Returns an array of nodes in the cluster
#Description    :It sends cluster show and find out node name or node objects in the cluster
#***********************************************************************
sub get_nodes
{
	my %params = (
            type=>undef,
	    @_
	);

    if ( SHM::_get_version() =~ /7M/i ) {
		return $Server->servers();
	} else {	
		my @node_objects;
		my @node_names;
		my $clus=NACL::C::Cluster->find(command_interface=>$Nodes[0]);
		@node_objs=NACL::C::Node->find(command_interface=>$clus);
		$Log->debug("Sizes of nodes array:  ".scalar(@nodes));
		foreach my $node (@node_objs) {
			push(@node_names,$node->name);
			push(@node_objects,$node);
		}
		if ( $params{type} eq "name"){
			$Log->debug("Nodes in cluster: @node_names");
			return @node_names;		
		} else {
			$Log->debug("Nodes in cluster: @node_objects");
			return @node_objects;		
		}
	}

}

#*******************************************************************************************
#Function Name  :counter_value
#Paremeters     :counter name
#Return Type    :Returns the value for the respective counter and zero if no instance found
#Description    :It sends counter name and find out the value for the respective counter
#*******************************************************************************************

sub counter_value {
	my %params = (
            counter_name=>undef,
	    @_
	);
	if (!defined($params{counter_name})){
		$Log->error("No Counter Name Specified");
		return 0;
	}	
	my $shell = $Nodes[0]->apiset(
		    interface => "CLI",
		    set => "Systemshell",
		    );
	my $cmd_out=$shell->execute_raw_command(command=>"cm_client -p nchm -a object-get-instances -o sys_mgmt_and_diagnosis -c $params{counter_name}");
        my @lines = split(/\n/, $cmd_out);
        foreach my $line(@lines) {
                if ( $line =~ /Num Instances/i) {
                        @list = split(/Num Instances:/,$line);
                        return 0 if($list[-1] =~ 0);
                }elsif ( $line =~ /$params{counter_name}/) {
                        @value = split(/value:/,$line);
			$Log->debug("Found value:  @value");
                        last;
                }
        }
        chomp($value[-1]);
        $value[-1] =~ tr/ "//d;
	if ($value[-1] =~ /\w/ ){
		$Log->debug("Counter Returned a string $value[-1], expecting a numeric value returning an empty string");
		return "";
	}else{
		$Log->debug("Return count value:  $value[-1]");
		return $value[-1];		
	}
	
}

#*******************************************************************************************
#Function Name  :run_dblade_command
#Paremeters     :command 
#Return Type    :Output of command
#Description    :Runs a dblade command
#*******************************************************************************************
sub run_dblade_command {
	my %params = (
            node=>"local",
            command => undef,
            @_
	);
	if (!defined($params{command})){
		$Log->error("No command specified for system node run");
		return 0;		
	}
        if ( SHM::_get_version() =~ /7M/i ) {
		return $Server->cli(cmd=>"priv set diag;$params{command}");
	}else{
		if ($params{command}){
            # For some reason we are refreshing @Nodes, but not actually
            # using @nodes here
			my @nodes=get_nodes();		
			my $Node = _get_nacl_node( $node_names[0] );
			my $buffer = NACL::C::Node->run(
				command_interface => $Node,


				node              => $params{node},
				command           => $params{command},
			);
			$Log->debug("Dblade output:  $buffer");
			return $buffer;
		}
	}
}

#*******************************************************************************************
#Function Name  :health_monitor_status_show
#Paremeters     :command 
#Return Type    :status field of the health monitor
#Description    :Gets the current health monitor status
#*******************************************************************************************
sub health_monitor_status_show {
	my %params = (
            node=>"local",
            @_
	);
	my $state="";	
        if ( SHM::_get_version =~ /7M/i ) {
		@show_details=split(/\n/,($Server->cli(cmd=>"system health status show")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			my @temps=split(/\: /,$show_details[$i]);
			if (lc($temps[0]) eq "ok" || lc($temps[0]) eq "degraded" ){
				$state=$temps[-1];
				last;
			}
		}		
	}else{
		my @nodes = get_nodes();
		my $Node = _get_nacl_node( $node_names[0] );
        
		try{
			my $SystemHealthStatus_state   = NACL::CS::SystemHealthStatus->fetch(
				command_interface => $Node,

			);
			$state = $SystemHealthStatus_state->status();
			
			$Log->debug("System Health Status State output:  $SystemHealthStatus_state, $state");
		
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
		}		
		catch NACL::Exceptions::NoElementsFound with{
		   my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};	
	}
	$Log->debug("System Health Status Show output:  $state");
	return $state;

}

#*******************************************************************************************
#Function Name  :check_health_alert_table
#Paremeters     :command 
#Return Type    :alerts table
#Description    :Gets the current health alerts 
#*******************************************************************************************
sub check_health_alert_table {
	my %params = (
            node=>"local",
            @_
	);
	my @values;
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		$state=$Server->cli(cmd=>"system health alert show");
	}else{		
		my $Node = _get_nacl_node( $node_names[0] );

		try{
			my $SystemHealthAlert_state   = NACL::CS::SystemHealthAlert->fetch(
				command_interface => $Node,
			);
			$state = $SystemHealthAlert_state;
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state = $exception_object->text();	
		};
	}
	$Log->debug("System Health Alert State output:  $state");
	return $state;

}

#*******************************************************************************************
#Function Name  :check_health_autosupport_history_table
#Paremeters     :command 
#Return Type    :autosupport history table
#Description    :Gets the current autosupport history alerts 
#*******************************************************************************************
sub check_health_autosupport_history_table {
	my %params = (
            node=>"local",
            @_
	);
	my @values;
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		$state=$Server->cli(cmd=>"system health autosupport trigger history show");
	}else{		
		my $Node = _get_nacl_node( $node_names[0] );

		try{
			my $SystemHealthAlert_state   = NACL::CS::SystemHealthAutosupportTriggerHistory->fetch(
				command_interface => $Node,
			);
			$state = $SystemHealthAlert_state;
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state = $exception_object->text();	
		};
	}
	$Log->debug("System Health Autosupport Trigger History State output:  $state");
	return $state;

}

#*******************************************************************************************
#Function Name  :health_alert_show
#Paremeters     :command 
#Return Type    :alerts table
#Description    :Gets the current health alerts 
#*******************************************************************************************
sub health_alert_show {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    'alerting-resource'=>undef,
	    acknowledge=>undef,
	    policy=>undef,
	    'alert-id'=>undef,
	    subsystem=>undef,
	    acknowledger=>undef,
	    'indication-time'=>undef,
	    'perceived-severity'=>undef,
            'probable-cause'=>undef,
            'probable-cause-description'=>undef,
            'corrective-actions'=>undef,
            'possible-effect'=>undef,
            'suppress'=>undef,
            'suppressor'=>undef,
            'alerting-resource-name'=>undef,	    
            @_
	);

	my $ref = \%params;
	$Log->debug("System Health Alert Show Filters:  ".Dumper(%params));
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		my $options="-node $params{node}";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{acknowledge})){
			$options=$options." -acknowledge $params{acknowledge}";
		}
		if (defined($params{policy})){
			$options=$options." -policy $params{policy}";
		}
		if (defined($params{'alert-id'})){
			$options=$options." -alert-id $params{'alert-id'}";
		}
		if (defined($params{subsystem})){
			$options=$options." -subsystem $params{subsystem}";
		}
		if (defined($params{acknowledger})){
			$options=$options." -acknowledger $params{acknowledger}";
		}
		if (defined($params{'indication-time'})){
			$options=$options." -indication-time $params{'indication-time'}";
		}
		if (defined($params{'perceived-severity'})){
			$options=$options." -perceived-severity $params{'perceived-severity'}";
		}
		if (defined($params{'probable-cause'})){
			$options=$options." -probable-cause $params{'probable-cause'}";
		}
		if (defined($params{'probable-cause-description'})){
			$options=$options." -probable-cause-description $params{'probable-cause-description'}";
		}
		if (defined($params{'corrective-actions'})){
			$options=$options." -corrective-actions $params{'corrective-actions'}";
		}
		if (defined($params{'possible-effect'})){
			$options=$options." -possible-effect $params{'possible-effect'}";
		}
		if (defined($params{'suppress'})){
			$options=$options." -suppress $params{'suppress'}";
		}
		if (defined($params{'suppressor'})){
			$options=$options." -suppressor params{'suppressor'}";
		}		
		if (defined($params{'alerting-resource-name'})){
			$options=$options." -alerting-resource-name $params{'alerting-resource-name'}";
		}			
		#$state=$Server->cli(cmd=>"system health alert show $options");
		@show_details=split(/\n/,($Server->cli(cmd=>"system health alert show $options -instance")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			if ( $show_details[$i] =~ /no entries/i){
				return "There are no entries matching your query";
			}			
			my @temps=split(/\: /,$show_details[$i]);
			if ($temps[0] eq "Node" ){
				$state=$temps[-1];
				last;
			}
		}		
        }	
	else{
		my $Node = _get_nacl_node( $node_names[0] );
		try{
			my $SystemHealthAlert   = NACL::C::SystemHealthAlert->find(
				command_interface => $Node,
				filter=>$ref,
			);
			my $alert_state = $SystemHealthAlert->state();
			$state=$alert_state->node;
		
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			$state=$params{node};
		}		
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};
	}

	$Log->debug("System Health Alert Show output:  $state");
	return $state;

}

#*******************************************************************************************
#Function Name  :health_alert_definition_show
#Paremeters     :command 
#Return Type    :alerts definition table
#Description    :Gets the current health alert definitions
#*******************************************************************************************
sub health_alert_definition_show {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    'alert-id'=>undef,
	    'alert-type'=>undef,
	    'perceived-severity'=>undef,
            'probable-cause'=>undef,
            'probable-cause-description'=>undef,
            'corrective-actions'=>undef,
            'possible-effect'=>undef,
            'subsystem'=>undef,
            'additional-information'=>undef,
	    @_
	);
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		my $options="-node $params{node}";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{'alert-id'})){
			$options=$options." -alert-id $params{'alert-id'}";
		}
		if (defined($params{'alert-type'})){
			$options=$options." -alert-type $params{'alert-type'}";
		}
		if (defined($params{'perceived-severity'})){
			$options=$options." -perceived-severity $params{'perceived-severity'}";
		}
		if (defined($params{'probable-cause'})){
			$options=$options." -probable-cause $params{'probable-cause'}";
		}
		if (defined($params{subsystem})){
			$options=$options." -subsystem $params{subsystem}";
		}
		if (defined($params{'probable-cause-description'})){
			$options=$options." -probable-cause-description $params{'probable-cause-description'}";
		}
		if (defined($params{'corrective-actions'})){
			$options=$options." -corrective-actions $params{'corrective-actions'}";
		}
		if (defined($params{'additional-information'})){
			$options=$options." -additional-information $params{'additional-information'}";
		}
		if (defined($params{'possible-effect'})){
			$options=$options." -possible-effect $params{'possible-effect'}";
		}		
		#$state=$Server->cli(cmd=>"system health alert definition show $options");
		@show_details=split(/\n/,($Server->cli(cmd=>"system health alert definition show $options -instance")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			my @temps=split(/\: /,$show_details[$i]);
			if ($temps[0] eq "Node" ){
				$state=$temps[-1];
				last;
			}
		}		
	}else{
		my $Node = _get_nacl_node( $node_names[0] );
		my $ref = \%params;
		try{
			my $SystemHealthAlertDefinition   = NACL::C::SystemHealthAlertDefinition->find(
				command_interface => $Node,
				filter=>$ref,
			);
			my $alert_state = $SystemHealthAlertDefinition->state();
			$state=$alert_state->node;
			
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			$state=$params{node};			
		}		
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};
	}
	$Log->debug("System Health Alert Definition Show output:  $state");
	return $state;

}

#*******************************************************************************************
#Function Name  :health_config_show
#Paremeters     :command 
#Return Type    :alert config table
#Description    :Gets the current health config 
#*******************************************************************************************
sub health_config_show {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    health=>undef,
	    'mon-version'=>undef,
	    'pol-version'=>undef,
	    'sub-pol-versions'=>undef,
	    context=>undef,
	    aggregator=>undef,
	    resources=>undef,
	   #subsystem=>undef,

            @_
	);
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		my $options="-node $params{node}";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{health})){
			$options=$options." -health $params{health}";
		}
		if (defined($params{'mon-version'})){
			$options=$options." -mon-version $params{'mon-version'}";
		}
		if (defined($params{'pol-version'})){
			$options=$options." -pol-version $params{'pol-version'}";
		}
		if (defined($params{'sub-pol-versions'})){
			$options=$options." -sub-pol-versions $params{'sub-pol-versions'}";
		}
		if (defined($params{context})){
			$options=$options." -context $params{context}";
		}
		if (defined($params{aggregator})){
			$options=$options." -aggregator $params{aggregator}";
		}
		if (defined($params{resources})){
			$options=$options." -resources $params{resources}";
		}
		if (defined($params{subsystem})){
			$options=$options." -subsystem $params{subsystem}";
		}		
		@show_details=split(/\n/,($Server->cli(cmd=>"system health config show $options -instance")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			my @temps=split(/\: /,$show_details[$i]);
			if ($temps[0] eq "Node" ){
				$state=$temps[-1];
				last;
			}
		}		
	}else{	
		my $Node = _get_nacl_node( $node_names[0] );
		my $ref = \%params;
		try{
			my $SystemHealthConfig   = NACL::C::SystemHealthConfig->find(
				command_interface => $Node,
				filter=>$ref,
			);
			my $alert_state = $SystemHealthConfig->state();
			$state=$alert_state->node;
			
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			$state=$params{node};
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};
	}
	$Log->debug("System Health Config Show output:  $state");
	return $state;

}

#*******************************************************************************************
#Function Name  :health_policy_definition_show
#Paremeters     :command 
#Return Type    :alerts definition table
#Description    :Gets the current health alert definitions
#*******************************************************************************************
sub health_policy_definition_show {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    'alert-id'=>undef,
	    'alert-count'=>undef,
	    'policy-id'=>undef,
            'rule-expression'=>undef,
            'where'=>undef,
            'enable'=>undef,
            'prev-alert-creation-time'=>undef,
            'responsible-resource-info'=>undef,
	    @_
	);
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		my $options="-node $params{node}";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{health})){
			$options=$options." -health $params{health}";
		}
		if (defined($params{'alert-id'})){
			$options=$options." -alert-id $params{'alert-id'}";
		}
		if (defined($params{'alert-count'})){
			$options=$options." -alert-count $params{'alert-count'}";
		}
		if (defined($params{'policy-id'})){
			$options=$options." -policy-id $params{'policy-id'}";
		}
		if (defined($params{'rule-expression'})){
			$options=$options." -rule-expression $params{'rule-expression'}";
		}
		if (defined($params{where})){
			$options=$options." -where $params{where}";
		}
		if (defined($params{enable})){
			$options=$options." -enable $params{enable}";
		}
		if (defined($params{'prev-alert-creation-time'})){
			$options=$options." -prev-alert-creation-time $params{'prev-alert-creation-time'}";
		}
		if (defined($params{'responsible-resource-info'})){
			$options=$options." -responsible-resource-info $params{'responsible-resource-info'}";
		}		
		@show_details=split(/\n/,($Server->cli(cmd=>"system health policy definition show $options -instance")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			my @temps=split(/\: /,$show_details[$i]);
			if ($temps[0] eq "Node" ){
				$state=$temps[-1];
				last;
			}
		}
				
	}else{		
		my $Node = _get_nacl_node( $node_names[0] );
		my $ref = \%params;
		try{
			my $SystemHealthPolicyDefinition   = NACL::C::SystemHealthPolicyDefinition->find(
				command_interface => $Node,
				filter=>$ref,
			);
			my $alert_state = $SystemHealthPolicyDefinition->state();
			$state=$alert_state->node;
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			$state=$params{node};			
		}		
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};
	}
	$Log->debug("System Health Policiy Definition Show output:  $state");
	return $state;


}
#*******************************************************************************************
#Function Name  :health_subsystem_show
#Paremeters     :command 
#Return Type    :alerts definition table
#Description    :Gets the current health alert definitions
#*******************************************************************************************
sub health_subsystem_show {
	my %params = (
		subsystem => undef,
	    health =>undef,
	    'init-state'=> undef,
	    'outstanding-alert-count'=>undef,
	   'suppressed-alert-count'=>undef,
	    @_
	);
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		my $options="";
		if (defined($params{subsystem})){
			$options=$options." -subsystem $params{subsystem}";
		}		
		if (defined($params{health})){
			$options=$options." -health $params{health}";
		}
		if (defined($params{'init-state'})){
			$options=$options." init-state $params{'init-state'}";
		}
		if (defined($params{'outstanding-alert-count'})){
			$options=$options." -outstanding-alert-count $params{'outstanding-alert-count'}";
		}
		if (defined($params{'suppressed-alert-count'})){
			$options=$options." -suppressed-alert-count $params{'suppressed-alert-count'}";
		}	
		@show_details=split(/\n/,($Server->cli(cmd=>"priv set diag;system health subsystem show $options ")));
		$Log->comment("Temps array".Dumper( @show_details));
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			push(@temps,$show_details[$i]);
		}
		$state=$temps[0];
	}else{		
		my $Node = _get_nacl_node( $node_names[0] );
		my $ref = \%params;
		try{
			my $SystemHealthSubsystem   = NACL::CS::SystemHealthSubsystem->fetch(
				command_interface => $Node,
				filter=>$ref,
			);
			$state = $SystemHealthSubsystem;
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			$state=$params{node};			
		}		
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};
	}
	$Log->debug("System Health Policiy Definition Show output:  $state");
	return $state;
}
#*******************************************************************************************
#Function Name  :health_subscriptions_show
#Paremeters     :command 
#Return Type    :health subscriptions table
#Description    :Gets the current health subscriptions
#*******************************************************************************************
sub health_subscriptions_show {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    'event-type'=>undef,
	    'subscription-id'=>undef,
	    'notify-dest-node'=>undef,
            'notify-dest-hm'=>undef,
            'class-name'=>undef,
	    'creation-time' => undef,
            'max-notify-period' => undef,
            'psc-option' => undef,
            'notify-table' => undef,
            'fail-thresh' => undef,
            'instance-name' => undef,
            'notify-type' => undef,
            'time-gap-notify' => undef,	    
	    @_
	);
	my $state="";
        if ( SHM::_get_version() =~ /7M/i ) {
		$Log->comment("Invoking the 7M command");
		my $options="-node $params{node}";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{'event-type'})){
			$options=$options." -event-type $params{'event-type'}";
		}
		if (defined($params{'subscription-id'})){
			$options=$options." -subscription-id $params{'subscription-id'}";
		}
		if (defined($params{'notify-dest-node'})){
			$options=$options." -notify-dest-node $params{'notify-dest-node'}";
		}
		if (defined($params{'notify-dest-hm'})){
			$options=$options." -notify-dest-hm $params{'notify-dest-hm'}";
		}
		if (defined($params{'class-name'})){
			$options=$options." -class-name $params{'class-name'}";
		}
		if (defined($params{'creation-time'})){
			$options=$options." -creation-time $params{'creation-time'}";
		}
		if (defined($params{'max-notify-period'})){
			$options=$options." -max-notify-period $params{'max-notify-period'}";
		}
		if (defined($params{'psc-option'})){
			$options=$options." -psc-option $params{'psc-option'}";
		}
		if (defined($params{'notify-table'})){
			$options=$options." -notify-table $params{'notify-table'}";
		}
		if (defined($params{'fail-thresh'})){
			$options=$options." -fail-thresh $params{'fail-thresh'}";
		}
		if (defined($params{'instance-name'})){
			$options=$options." -instance-name $params{'instance-name'}";
		}
		if (defined($params{'notify-type'})){
			$options=$options." -notify-type $params{'notify-type'}";
		}
		if (defined($params{'time-gap-notify'})){
			$options=$options." -'time-gap-notify' $params{'time-gap-notify'}";
		}		
		$state=$Server->cli(cmd=>"system health policy subscriptions show $options");
	}else{		
		my $Node = _get_nacl_node( $node_names[0] );
		my $ref = \%params;
		try{
			my $SystemHealthsubscriptions   = NACL::C::SystemHealthSubscriptions->find(
				command_interface => $Node,
				filter=>$ref,
			);
			my $alert_state = $SystemHealthsubscriptions->state();
			$state=$alert_state->node;
		
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};
	}
	$Log->debug("System Health Policiy Definition Show output:  $state");
	return $state;

}
#*******************************************************************************************
#Function Name  :get_node_connectivity_hwvalues
#Paremeters     :command 
#Return Type    :Disk, adapter, or shelf names
#Description    :Returns all the names of the specified hardware type
#*******************************************************************************************
sub get_node_connectivity_hwvalues {
	my %params = (
            node=>"local",
	    hwtype=>undef,
            @_
	);
	my @vals;
	
	if ( defined($params{hwtype})){
		my $hwstate;
		if ( SHM::_get_version() =~ /7M/i ) {
			$Log->comment("Invoking the 7M command");
			my @show_details=split(/\n/,($Server->cli(cmd=>"priv set diag;system health node-connectivity $params{hwtype} show")));
			foreach my $hwstate (@show_details){
				#$Log->comment("Hwstate:  $hwstate");
				my @temps=split(/\s/,$hwstate);
				foreach my $temp (@temps){
					$Log->debug("Hwstate:  $hwstate");
					if ($temp =~ /^[0-9][a-z]/){
						$Log->debug("Temp:  $temp");
						my @disks=split(/\s/,$temp);
						$Log->debug("Disk:  $disks[-1]");
						push(@vals,$disks[-1]);
					}
				}
			}			
		}else{
		    my $Node = _get_nacl_node( $node_names[0] );
			try{
				if ( $params{hwtype} eq "disk"){
					@hwstate   = NACL::CS::SystemHealthNodeConnectivityDisk->fetch(
					command_interface => $Node,
					);
					foreach my $hwstate (@hwstate){
						push(@vals,$hwstate->disk_name);
					}			
				}
				elsif ( $params{hwtype} eq "shelf"){
					@hwstate   = NACL::CS::SystemHealthNodeConnectivityShelf->fetch(
					command_interface => $Node,
					);
					foreach my $hwstate (@hwstate){
						push(@vals,$hwstate->shelf_name);
					}			
				}
				else{
					@hwstate   = NACL::CS::SystemHealthNodeConnectivityAdapter->fetch(
					command_interface => $Node,
					);
					foreach my $hwstate (@hwstate){
						push(@vals,$hwstate->adapter_name);
					}			
				}
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};
		}
			$Log->debug("System Health Node Connectivity output: @vals");
		
	}
	else{
		$Log->error("No hardware type specified for system health node-connectivity show");
	}
	return @vals;

}

#*******************************************************************************************
#Function Name  :get_health_config_values
#Paremeters     :command 
#Return Type    :Subsystem, alertid, monitor, 
#Description    :Returns all the config value
#*******************************************************************************************
sub get_health_config_values {
	my %params = (
            node=>"local",
	    config=>undef,
            @_
	);
	my $version=SHM::_get_version();
	my $SystemHealthConfig;
	my $SystemHealthConfig_state;
	my @values;
	my $hash;
	my $value;
	my $key;
	if ( defined($params{config})){

		if ($version =~ /7M/i){
			my @show_details;
			my %mapping=( 'monitor-version'=>'mon-version',
				      'policy-file-version'=>'pol-version',
			              'resource'=>'resources',
				      'subordinate-policy-versions'=>'sub-pol-versions');			
			if ($params{config} eq "zapi"){
				@show_details=split(/\n/,($Server->cli(cmd=>"system health config show -node local -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$value=~s/_/-/g;
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
					if ($key =~ /resource/){
						$Log->debug(sub {"7M hash of alerts: ".Dumper($hash);});
						$Log->debug(sub {"Monitor value: ".$hash->{'monitor'};});
						push(@values,$hash);
						$hash={};
					}
					
				}
				
				$state=\@values;
				$Log->debug("Array of 7M hashes, size: @values, ".@values);
			}elsif ($params{config} eq "all"){
				@show_details=split(/\n/,($Server->cli(cmd=>"system health config show -node local")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/ );
					last if (exists $hash{$temps[0]});
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-" || length($value) < 1);
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$value=~s/_/-/g;
					$Log->debug("Adding key/value $key, $value");					
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
				}
				$Log->debug("7M hash of alerts: ".Dumper($hash));
				$state=$hash;		
			}else{
				$params{config}=~s/_/-/g;
				my $query="system health config show  -node local -fields $params{config}";				
				@show_details=split(/\n/,($Server->cli(cmd=>$query)));
				#assume the field is the last value in the string
				#skip the first two, the column header and dashes
				my $size = scalar(@show_details) - 1;
				for(my $i = 2; $i<=$size;$i++){
					next if ( $show_details[$i] =~ /entries/ );
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\s/,$show_details[$i]);
					$Log->debug("final val: $temps[-1]");
					push(@values,$temps[-1]);
				}
				$state=$values[0];				
			}

		}else{
		    my $Node = _get_nacl_node( $node_names[0] );
			try{
				$SystemHealthConfig  = NACL::C::SystemHealthConfig->find(command_interface =>$Node);
				$SystemHealthConfig_state = $SystemHealthConfig->state();
				if ( $params{config} eq "subsystem"){
					$state=$SystemHealthConfig_state->subsystem;
				}elsif ( $params{config} eq "mon-version"){
					$state=$SystemHealthConfig_state->mon_version;
				}elsif ( $params{config} eq "monitor"){
					$state=$SystemHealthConfig_state->monitor;
				}elsif ( $params{config} eq "node"){
					$state=$SystemHealthConfig_state->node;
				}elsif ( $params{config} eq "pol-version"){
					$state=$SystemHealthConfig_state->pol_version;
				}elsif ( $params{config} eq "context"){
					$state=$SystemHealthConfig_state->context;
				}elsif ( $params{config} eq "aggregator"){
					$state=$SystemHealthConfig_state->aggregator;
				}elsif ( $params{config} eq "resources"){
					$state=$SystemHealthConfig_state->resources;
				}elsif ( $params{config} eq "health"){
					$state=$SystemHealthConfig_state->health;
				}elsif ( $params{config} eq "sub-pol-versions"){
					$state=$SystemHealthConfig_state->sub_pol_versions;
				}elsif ( $params{config} eq "all"){
					$state = $SystemHealthConfig_state
				}else{
					$Log->error("Invalid config type $params{config} specified for system health config show");
				}
				$Log->debug("System Health Alert Field output: ".Dumper($state));	
	 
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};
		}
	}else{
		$Log->error("No config type specified for system health alert show");
	}
	return $state;

}


#*******************************************************************************************
#Function Name  :get_health_subsystem_values
#Paremeters     :command 
#Return Type    :Subsystem, alertid, monitor, 
#Description    :Returns all the config value
#*******************************************************************************************
sub get_health_subsystem_values {
	my %params = (
            node=>"local",
			config=>undef,
			filter=>undef,
            @_
	);
	#$cmd='system health config show -'.$key.' '.$value;
	my $version=SHM::_get_version();
	my $SystemHealthSubsystem;
	my $SystemHealthSubsystme_state;
	my @values;
	my $hash;
	my $value;
	my $key;
	
	if ( defined($params{config})){

		if ($version =~ /7M/i){
			my @show_details;
			my %mapping=( 'monitor-version'=>'mon-version',
				      'policy-file-version'=>'pol-version',
			              'resource'=>'resources',
				      'subordinate-policy-versions'=>'sub-pol-versions');			
			if ($params{config} eq "zapi"){
				#BURT prevents the instance param
				#@show_details=split(/\n/,($Server->cli(cmd=>"system health subsystem show -instance")));
				@show_details=split(/\n/,($Server->cli(cmd=>"priv set diag;system health subsystem show -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/ );
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$value=~s/_/-/g;
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
					if ($key =~ /suppressed/i){
						$Log->debug(sub {"7M hash of alerts: ".Dumper($hash);});
						$Log->debug(sub {"Subsystem value: ".$hash->{'subsystem'};});
						push(@values,$hash);
						$hash={};
					}
					
				}
				$state=\@values;
				$Log->debug("Array of 7M hashes, size: @values, ".@values);
			}elsif ($params{config} eq "all"){
				if (defined($params{filter})){
					my %hash=%{$params{filter}};
					my $filter_string="";
					my $value;
					foreach my $key ( keys %hash ){
						$value=$hash{$key};
						$key=~s/_/-/g;
						$filter_string=$filter_string."-$key $value ";
						$Log->comment("filter string:  $filter_string");
						$temp="";
					}
					@show_details=split(/\n/,($Server->cli(cmd=>"system health subsystem show $filter_string -instance")));
				}else{
					@show_details=split(/\n/,($Server->cli(cmd=>"system health subsystem show -node local -instance")));	
				}
				
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/ );
					last if (exists $hash{$temps[0]});
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-" );
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}					
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
				}
				$Log->debug("7M hash of alerts: ".Dumper($hash));
				$state=$hash;	
			}else{
				$params{config}=~s/_/-/g;
				my $query="system health subystem show -fields $params{config}";				
				@show_details=split(/\n/,($Server->cli(cmd=>$query)));
				#assume the field is the last value in the string
				#skip the first two, the column header and dashes
				my $size = scalar(@show_details) - 1;
				for(my $i = 2; $i<=$size;$i++){
					next if ( $show_details[$i] =~ /entries/i );
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\s/,$show_details[$i]);
					push(@values,$temps[-1]);
				}
				$state=$values[0];				
			}

		}else{
			my $Node = _get_nacl_node( $node_names[0] );
			try{
				$SystemHealthSubsystem_state  = NACL::CS::SystemHealthSubsystem->fetch(command_interface =>$Node);
				
				if ( $params{config} eq "subsystem"){
					$state=$SystemHealthSubsystem_state->subsystem;
				}elsif ( $params{config} eq "health"){
					$state=$SystemHealthSubsystem_state->health;
				}elsif ( $params{config} eq "inti-state"){
					$state=$SystemHealthSubsystem_state->init_state;
				}elsif ( $params{config} eq "outstanding-alert-count"){
					$state=$SystemHealthSubsystem_state->outstanding_alert_count;
				}elsif ( $params{config} eq "suppressed-alert-count"){
					$state=$SystemHealthSubsystem_state->suppressed_alert_count ;
				}elsif ( $params{config} eq "all"){
					$state = $SystemHealthSubsystem_state;
				}else{
					$Log->error("Invalid config type $params{config} specified for system health subsystem show");
				}
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};
		}
	}else{
		$Log->error("No config type specified for system health alert show");
	}
	return $state;

}


#*******************************************************************************************
#Function Name  :get_health_alert_values
#Paremeters     :command 
#Return Type    :Subsystem, alertid, monitor, 
#Description    :Returns all the alert value
#*******************************************************************************************
sub get_health_alert_values {
	my %params = (
            node=>"local",
	    config=>undef,

	    filter=>undef,
            @_
	);
	my $state=check_health_alert_table();
	if ( $state =~ /No matching system health alert/i || $state =~ /There are no entries matching your query/i ){
		return $state;
	}
	$Log->debug("System Health Alert Field output: $state ");
	my $version=SHM::_get_version();
	my $SystemHealthConfig;
	my $SystemHealthConfig_state;
	my @values;
	my $hash;
	my $value;
	my $key;
	if ( defined($params{config})){
		if ($version =~ /7M/i){
			my @show_details;
			my @extra;
			my $index;
			my %mapping=( 'class-of-alert'=>'alert-id',
			      'description'=>'probable-cause-description',
			      'additional-information'=>'additonal-info');
			if ($params{config} eq "zapi"){
				@show_details=split(/\n/,($Server->cli(cmd=>"system health alert show -node local -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				my %mapping=( 'alert-id'=>'alert_id',
					      'alerting-resource'=>'alerting_resource');				
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$value=~s/_/-/g;
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
					if ($key =~ /corrective/i){
						$Log->debug(sub {"7M hash of alerts: ".Dumper($hash);});
						$Log->debug(sub {"Monitor value: ".$hash->{'monitor'};});
						push(@values,$hash);
						$hash={};
					}
					
				}
				$state=\@values;
				$Log->debug("Array of 7M hashes, size: @values, ".@values);
			}elsif ($params{config} eq "all"){
				
				if (defined($params{filter})){
					my %hash=%{$params{filter}};
					my $filter_string="";
					my $value;
					foreach my $key ( keys %hash ){
						$value=$hash{$key};
						$key=~s/_/-/g;
						$filter_string=$filter_string."-$key $value ";
						$Log->debug("filter string:  $filter_string");
						$temp="";
					}
					@show_details=split(/\n/,($Server->cli(cmd=>"system health alert show $filter_string -instance")));
				}else{
					@show_details=split(/\n/,($Server->cli(cmd=>"system health alert show -node local -instance")));	
				}
				
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					last if (exists $hash{$temps[0]});
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-" || $key =~ /Shelf/i || $key =~ /Number/i );
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}					
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
				}
				$Log->debug("7M hash of alerts: ".Dumper($hash));
				$state=$hash;
			}else{
				$params{config}=~s/_/-/g;
				my $query="system health alert show  -node local -fields $params{config}";
				@show_details=split(/\n/,($Server->cli(cmd=>$query)));
				#assume the field is the last value in the string
				#skip the first two, the column header and dashes
				my $size = scalar(@show_details) - 1;
				my $index=0;
				my @headers=split(/\s/,$show_details[0]);
				foreach $header (@headers){
					last if ( $params{config} eq $header );
					next if (length($header) == 0);


					$Log->debug("Checking non-whitspace $header.");
					$index++;
				}
				for(my $i = 2; $i<=$size;$i++){
					next if ( $show_details[$i] =~ /entries/i );
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\s/,$show_details[$i]);


					$Log->debug("final val: $temps[$index]");
					$state=$temps[$index];
				}
			
			}

		}else{
			my $Node = _get_nacl_node($node_names[0]);
			try{
				if ( defined($params{filter})){
					$Log->comment("Using filter $params{filter} for Health Alert Show".Dumper($params{filter}));
					$SystemHealthAlert = NACL::C::SystemHealthAlert->find(command_interface =>$Node,filter=>$params{filter});
                   


				}else{
					$SystemHealthAlert = NACL::C::SystemHealthAlert->find(command_interface =>$Node);
				}				
				$SystemHealthAlert_state = $SystemHealthAlert->state();
				if ( $params{config} eq "subsystem"){
					$state=$SystemHealthAlert_state->subsystem;
				}elsif ( $params{config} eq "alert_id"){
					$state=$SystemHealthAlert_state->alert_id;
				}elsif ( $params{config} eq "monitor"){
					$state=$SystemHealthAlert_state->monitor;
				}elsif ( $params{config} eq "node"){
					$state=$SystemHealthAlert_state->node;
				}elsif ( $params{config} eq "acknowledge"){
					$state=$SystemHealthAlert_state->acknowledge;
				}elsif ( $params{config} eq "acknowledger"){
					$state=$SystemHealthAlert_state->acknowledger;
				}elsif ( $params{config} eq "corrective_actions"){
					$state=$SystemHealthAlert_state->corrective_actions;
				}elsif ( $params{config} eq "recommended_actions"){
					$state=$SystemHealthAlert_state->recommended_actions;
				}elsif ( $params{config} eq "perceived_severity"){
					$state=$SystemHealthAlert_state->perceived_severity;
				}elsif ( $params{config} eq "alerting_resource"){
					$state=$SystemHealthAlert_state->alerting_resource;
				}elsif ( $params{config} eq "alerting-resource"){
                    $state=$SystemHealthAlert_state->alerting_resource;
				}elsif ( $params{config} eq "alerting_resource_name"){
					$state=$SystemHealthAlert_state->alerting_resource_name;
				}elsif ( $params{config} eq "policy"){
					$state=$SystemHealthAlert_state->policy;
				}elsif ( $params{config} eq "probable_cause_description"){
					$state=$SystemHealthAlert_state->probable_cause_description;
				}elsif ( $params{config} eq "probable_cause"){
					$state=$SystemHealthAlert_state->probable_cause;
				}elsif ( $params{config} eq "possible_effect"){
					$state=$SystemHealthAlert_state->possible_effect;
				}elsif ( $params{config} eq "additional_info"){
					$state=$SystemHealthAlert_state->additional_info;
				}elsif ( $params{config} eq "suppress"){
					$state=$SystemHealthAlert_state->suppress;
				}elsif ( $params{config} eq "indication_time"){
					$state=$SystemHealthAlert_state->indication_time;
				}elsif ( $params{config} eq "all"){
					$state=$SystemHealthAlert_state;
				}else{
					$Log->error("Invalid config type $params{config} specified for system health alert show");
				}
				$Log->debug("System Health Alert Field output: $state ");	
	 
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};
		}

	}else{
		$Log->error("No config type specified for system health alert show");
	}
	return $state;

}

#*******************************************************************************************
#Function Name  :get_health_policy_definition_values
#Paremeters     :command 
#Return Type    :Subsystem, alertid, monitor, 
#Description    :Returns the the policy value
#*******************************************************************************************
sub get_health_policy_definition_values {
	my %params = (
            node=>"local",
	    config=>undef,
            @_
	);
	my $version=SHM::_get_version();
	my $SystemHealthConfig;
	my $SystemHealthConfig_state;
	my @values;
	my $hash;
	my $value;
	my $key;
	if ( defined($params{config})){
		if ($version =~ /7M/i){
			my %mapping=( 'policy'=>'policy-id',
				      'variable-equivalence'=>'where',
				      'table-and-id-of-resource-at-fault'=>'responsible-resource-info',
				      'policy-status'=>'enable');			
			my @show_details;
			if ($params{config} eq "zapi"){
				%mapping=( 'policy'=>'policy_id');
				@show_details=split(/\n/,($Server->cli(cmd=>"system health policy definition show -node local -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$value=~s/_/-/g;
					$hash->{$key}=$value;
					if ($key =~ /enable/){
						$Log->debug("7M hash of alerts: ".Dumper($hash));
						push(@values,$hash);
						$hash={};
					}
					
				}
				
				$state=\@values;
				$Log->debug("Array of 7M hashes, size: @values, ".@values);
			}elsif ($params{config} eq "all"){
				@show_details=split(/\n/,($Server->cli(cmd=>"system health policy definition show -node local -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					last if (exists $hash{$temps[0]});
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
				}
				$Log->debug("7M hash of alerts: ".Dumper($hash));
				$state=$hash;				
			}else{
				$params{config}=~s/_/-/g;
				my $query="system health alert show  -node local -fields $params{config}";				
				@show_details=split(/\n/,($Server->cli(cmd=>$query)));
				#assume the field is the last value in the string
				#skip the first two, the column header and dashes
				my $size = scalar(@show_details) - 1;
				for(my $i = 2; $i<=$size;$i++){
					next if ( $show_details[$i] =~ /entries/i );
					my @temps=split(/\s/,$show_details[$i]);
					push(@values,$temps[-1]);
				}
				$state=$values[0];				
			}

		}else{
			my $Node = _get_nacl_node( $node_names[0] );
			try{
				my $SystemHealthPolicyDefinition = NACL::C::SystemHealthPolicyDefinition->find(command_interface =>$Node);
				my $SystemHealthPolicyDefinition_state = $SystemHealthPolicyDefinition->state();
				if ( $params{config} eq "where"){
					$state=$SystemHealthPolicyDefinition_state->where;
				}
				elsif ( $params{config} eq "alert_id"){
					$state=$SystemHealthPolicyDefinition_state->alert_id;
				}
				elsif ( $params{config} eq "monitor"){
					$state=$SystemHealthPolicyDefinition_state->monitor;
				}
				elsif ( $params{config} eq "node"){
					$state=$SystemHealthPolicyDefinition_state->node;
				}
				elsif ( $params{config} eq "enable"){
					$state=$SystemHealthPolicyDefinition_state->enable;
				}
				elsif ( $params{config} eq "prev-alert-creation-time"){
					$state=$SystemHealthPolicyDefinition_state->prev_alert_creation_time;
				}		
				elsif ( $params{config} eq "responsible-resource-info"){
					$state=$SystemHealthPolicyDefinition_state->responsible_resource_info;
				}
				elsif ( $params{config} eq "alert-count"){
					$state=$SystemHealthPolicyDefinition_state->alert_count;
				}
				elsif ( $params{config} eq "rule-expression"){
					$state=$SystemHealthPolicyDefinition_state->rule_expression;
				}
				elsif ( $params{config} eq "policy-id"){
					$state=$SystemHealthPolicyDefinition_state->policy_id;
				}		
				elsif ( $params{config} eq "all"){
					$state=$SystemHealthPolicyDefinition_state;
				}		
				else{
					$Log->error("Invalid config type $params{config} specified for system health alert show");
				}
				$Log->debug("System Health Policy Definition Field output: ".Dumper($state));
			 
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};		

		}
	}else{
		$Log->error("No config type specified for system health policy definition command");
	}
	return $state;

}

#*******************************************************************************************
#Function Name  :get_health_alert_definition_values
#Paremeters     :command 
#Return Type    :Subsystem, alertid, monitor, 
#Description    :Returns the the alert definition value
#*******************************************************************************************
sub get_health_alert_definition_values {
	my %params = (
            node=>"local",
	    config=>undef,
            @_
	);
	$Log->debug("System Health Definition Field output: $state ");
	#$cmd='system health definitnion show -'.$key.' '.$value;
	my $version=SHM::_get_version();
	my $SystemHealthConfig;
	my $SystemHealthConfig_state;
	my @values;
	my $hash;
	my $value;
	my $key;
	if ( defined($params{config})){
		if ($version =~ /7M/i){
			my %mapping=( 
			      'severity-of-alert'=>'perceived-severity',
			      'class-of-alert'=>'alert-id',
			      'the-type-of-alert'=>'alert-type',
			      'additional-relevant-data'=>'additonal-information',
			      'subsystem-name'=>'subsystem');			
			my @show_details;
			if ($params{config} eq "zapi"){
				my %mapping=( 'class-of-alert'=>'alert_id');
				@show_details=split(/\n/,($Server->cli(cmd=>"system health alert definition show -node local -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$value=~s/_/-/g;
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
					if ($key =~ /corrective/i){
						$Log->debug("7M hash of alerts: ".Dumper($hash));
						push(@values,$hash);
						$hash={};
					}
					
				}
				
				$state=\@values;
				$Log->debug("Array of 7M hashes, size: @values, ".@values);
				
			}elsif ($params{config} eq "all"){
				@show_details=split(/\n/,($Server->cli(cmd=>"system health alert definition show -node local -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					last if (exists $hash{$temps[0]});
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}					
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
				}
				#$Log->comment("7M hash of alerts: ".Dumper($hash));
				$state=$hash;			
			}else{
				$params{config}=~s/_/-/g;
				my $query="system health alert show  -node local -fields $params{config}";				
				@show_details=split(/\n/,($Server->cli(cmd=>$query)));
				#assume the field is the last value in the string
				#skip the first two, the column header and dashes
				my $size = scalar(@show_details) - 1;
				for(my $i = 2; $i<=$size;$i++){
					next if ( $show_details[$i] =~ /entries/i );
					#$Log->comment("Value:  $show_details[$i]");
					my @temps=split(/\s/,$show_details[$i]);
					#$Log->comment("final val: $temps[-1]");
					push(@values,$temps[-1]);
				}
				$state=$values[0];				
			}

		}else{
		    my $Node = _get_nacl_node( $node_names[0] );
			try{
				my $SystemHealthAlertDefinition = NACL::C::SystemHealthAlertDefinition->find(command_interface =>$Node);
				my $SystemHealthAlertDefinition_state = $SystemHealthAlertDefinition->state();
				if ( $params{config} eq "probable-cause"){
					$state=$SystemHealthAlertDefinition_state->probable_cause;
				}
				elsif ( $params{config} eq "alert_id"){
					$state=$SystemHealthAlertDefinition_state->alert_id;
				}
				elsif ( $params{config} eq "monitor"){
					$state=$SystemHealthAlertDefinition_state->monitor;
				}
				elsif ( $params{config} eq "node"){
					$state=$SystemHealthAlertDefinition_state->node;
				}
				elsif ( $params{config} eq "probable-cause-description"){
					$state=$SystemHealthAlertDefinition_state->probable_cause_description;
				}
				elsif ( $params{config} eq "recommended-actions"){
					$state=$SystemHealthAlertDefinition_state->recommended_actions;
				}		
				elsif ( $params{config} eq "subsystem"){
					$state=$SystemHealthAlertDefinition_state->subsystem;
				}
				elsif ( $params{config} eq "alert-type"){
					$state=$SystemHealthAlertDefinition_state->alert_type;
				}
				elsif ( $params{config} eq "additional-information"){
					$state=$SystemHealthAlertDefinition_state->additional_information;
				}
				elsif ( $params{config} eq "percieved-severity"){
					$state=$SystemHealthAlertDefinition_state->percieved_severity;
				}
				elsif ( $params{config} eq "possible-effect"){
					$state=$SystemHealthAlertDefinition_state->possible_effect;
				}
				elsif ( $params{config} eq "corrective-actions"){
					$state=$SystemHealthAlertDefinition_state->corrective_actions;
				}		
				elsif ( $params{config} eq "all"){
					$state=$SystemHealthAlertDefinition_state;
				}		
				else{
				$Log->error("Invalid config type $params{config} specified for system health alert show");
				}
				$Log->debug("System Health Alert Definition Field output: ".Dumper($state));
			 
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};		
			
	
		}
	}else{
		$Log->error("No config type specified for system health alert definition command");
	}
	return $state;

}

#*******************************************************************************************
#Function Name  :get_health_alert_definition_fields
#Paremeters     :command 
#Return Type    :array 
#Description    :Returns altert filter fields
#*******************************************************************************************
sub get_health_alert_definition_fields {
	my %params = (
            node=>"local",
            @_
	);

	my @fields;
	if (SHM::_get_version() =~ /7M/i){
		my @show_details;
		@show_details=split(/\n/,($Server->cli(cmd=>"system health alert definition show -node local -instance")));
		my $size = scalar(@show_details) - 1;
		for(my $i = 2; $i<=$size;$i++){
			next if ( $show_details[$i] =~ /entries/i );
			$Log->debug("Value:  $show_details[$i]");
			$show_details[$i] =~ s/^\s+|\s+$//g;
			my @temps=split(/:/,$show_details[$i]);
			if ($temps[0] =~ /\s/){
				$temps[0]=~s/ /-/g;
				last if (exists $hash{$temps[0]});
				#$Log->comment("Adding key/value $temps[0]");
				push(@fields,$temps[0]);
			}
		}				
		
	}else{		
		my $Node = _get_nacl_node( $node_names[0] );
		try{
			my $SystemHealthAlertDefinition  = NACL::C::SystemHealthAlertDefinition->find(command_interface =>$Node);
			my $SystemHealthAlertDefinition_state = $SystemHealthAlertDefinition->state();
			foreach my $key (keys %{$SystemHealthAlertDefinition_state}) {
				my $modkey=$key;
				my $value=$SystemHealthAlertDefinition_state->$key();
				if ($modkey =~ /_/){
					$modkey=~s/_/-/g;
				}
				push(@fields,$modkey);
			}
			$Log->debug("System Health Alert Definition Filter Fields: @fields");		
		 
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		};	

	}
	return @fields;

}

#*******************************************************************************************
#Function Name  :get_health_autosupport_trigger_history_values
#Paremeters     :command 
#Return Type    :Subsystem, alertid, monitor, 
#Description    :Returns all the alert value
#*******************************************************************************************
sub get_health_autosupport_trigger_history_values {
	my %params = (
            node=>"local",
	    config=>undef,
	    filter=>undef,
            @_
	);
	my $state=check_health_autosupport_history_table();
	if ( $state =~ /No matching system health alert/i || $state =~ /There are no entries matching your query/i ){
		return $state;
	}
	$Log->debug("System Health Autosupport Trigger History Field output: $state ");
	my $version=SHM::_get_version();
	my $SystemHealthConfig;
	my $SystemHealthConfig_state;
	my @values;
	my $hash;
	my $value;
	my $key;
	if ( defined($params{config})){
		if ($version =~ /7M/i){
			my @show_details;
			my @extra;
			my $index;
			my %mapping=( 'class-of-alert'=>'alert-id',
			      'description'=>'probable-cause-description',
			      'additional-information'=>'additonal-info');
			if ($params{config} eq "zapi"){
				@show_details=split(/\n/,($Server->cli(cmd=>"system health autosupport trigger history show -node local -instance")));
				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				my %mapping=( 'alert-id'=>'alert_id',
					      'alerting-resource'=>'alerting_resource');				
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-");
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}
					$value=~s/_/-/g;
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
					if ($key =~ /corrective/){
						$Log->debug(sub {"7M hash of alerts: ".Dumper($hash);});
						$Log->debug(sub {"Monitor value: ".$hash->{'monitor'};});
						push(@values,$hash);
						$hash={};
					}
					
				}
				$state=\@values;
				$Log->debug("Array of 7M hashes, size: @values, ".@values);
			}elsif ($params{config} eq "all"){
				if (defined($params{filter})){
					my %hash=%{$params{filter}};
					my $filter_string="";
					my $value;
					foreach my $key ( keys %hash ){
						$value=$hash{$key};
						$key=~s/_/-/g;
						$filter_string=$filter_string."-$key $value ";
						$Log->debug("filter string:  $filter_string");
						$temp="";
					}
					@show_details=split(/\n/,($Server->cli(cmd=>"system health autosupport trigger history show $filter_string -instance")));
				}else{
					@show_details=split(/\n/,($Server->cli(cmd=>"system health autosuppport trigger history show -node local -instance")));	
				}				$Log->debug("Temps array @show_details");
				my $size = scalar(@show_details) - 1;
				for(my $i = 0; $i<=$size;$i++){
					next if (length($show_details[$i]) < 1);
					next if ( $show_details[$i] =~ /entries/i );
					last if (exists $hash{$temps[0]});
					$show_details[$i] =~ s/^\s+//g;
					$show_details[$i] =~ s/\s+$//g;
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\: /,$show_details[$i]);
					my $key=$temps[0];
					my $value=$temps[-1];
					next if ( $value eq "-" || $key =~ /Shelf/i || $key =~ /Number/i );
					if ($key eq $value ){
						$Log->debug("Only adding the first item if the show has multiple lines of output");
						next;
					}
					$key=lc($temps[0]);
					$key=~s/ /-/g;
					$Log->debug("mapping value:  ".$mapping{$key});
					if ($mapping{$key}){
						$key=$mapping{$key};
					}					
					$Log->debug("Adding key/value $key, $value");
					$hash->{$key}=$value;
				}
				$Log->debug("7M hash of alerts: ".Dumper($hash));
				$state=$hash;
			}else{
				$params{config}=~s/_/-/g;
				my $query="system health autosupport trigger history show  -node local -fields $params{config}";
				@show_details=split(/\n/,($Server->cli(cmd=>$query)));
				#assume the field is the last value in the string
				#skip the first two, the column header and dashes
				my $size = scalar(@show_details) - 1;
				my $index=0;
				my @headers=split(/\s/,$show_details[0]);
				foreach $header (@headers){
					last if ( $params{config} eq $header );
					next if (length($header) == 0);
					#$Log->comment("Checking non-whitspace $header.");
					$index++;
				}
				for(my $i = 2; $i<=$size;$i++){
					next if ( $show_details[$i] =~ /entries/i );
					$Log->debug("Value:  $show_details[$i]");
					my @temps=split(/\s/,$show_details[$i]);
					$Log->debug("final val: $temps[$index]");
					$state=$temps[-1];
				}
			
			}

		}else{
		    my $Node = _get_nacl_node( $node_names[0] );
			try{
				if ( defined($params{filter})){
					$Log->debug("Using filter $params{filter} for Health Autosupport Trigger History".Dumper($params{filter}));
					$SystemHealthAlert = NACL::C::SystemHealthAutosupportTriggerHistory->find(command_interface =>$Node,filter=>$params{filter});
				}else{
					$SystemHealthAlert = NACL::C::SystemHealthAutosupportTriggerHistory->find(command_interface =>$Node);
				}
				$SystemHealthAlert_state = $SystemHealthAlert->state();
				if ( $params{config} eq "subsystem"){
					$state=$SystemHealthAlert_state->subsystem;
				}elsif ( $params{config} eq "alert_id"){
					$state=$SystemHealthAlert_state->alert_id;
				}elsif ( $params{config} eq "alert-id"){
                    $state=$SystemHealthAlert_state->alert_id;
				}elsif ( $params{config} eq "monitor"){
					$state=$SystemHealthAlert_state->monitor;
				}elsif ( $params{config} eq "node"){
					$state=$SystemHealthAlert_state->node;
				}elsif ( $params{config} eq "acknowledge"){
					$state=$SystemHealthAlert_state->acknowledge;
				}elsif ( $params{config} eq "acknowledger"){
					$state=$SystemHealthAlert_state->acknowledger;
				}elsif ( $params{config} eq "corrective_actions"){
					$state=$SystemHealthAlert_state->corrective_actions;
				}elsif ( $params{config} eq "recommended_actions"){
					$state=$SystemHealthAlert_state->recommended_actions;
				}elsif ( $params{config} eq "perceived_severity"){
					$state=$SystemHealthAlert_state->perceived_severity;
				}elsif ( $params{config} eq "alerting_resource"){
					$state=$SystemHealthAlert_state->alerting_resource;
				}elsif ( $params{config} eq "alerting-resource"){
                     $state=$SystemHealthAlert_state->alerting_resource;
				}elsif ( $params{config} eq "alerting_resource_name"){
					$state=$SystemHealthAlert_state->alerting_resource_name;
				}elsif ( $params{config} eq "policy"){
					$state=$SystemHealthAlert_state->policy;
				}elsif ( $params{config} eq "probable_cause_description"){
					$state=$SystemHealthAlert_state->probable_cause_description;
				}elsif ( $params{config} eq "probable_cause"){
					$state=$SystemHealthAlert_state->probable_cause;
				}elsif ( $params{config} eq "possible_effect"){
					$state=$SystemHealthAlert_state->possible_effect;
				}elsif ( $params{config} eq "additional_info"){
					$state=$SystemHealthAlert_state->additional_info;
				}elsif ( $params{config} eq "suppress"){
					$state=$SystemHealthAlert_state->suppress;
				}elsif ( $params{config} eq "indication_time"){
					$state=$SystemHealthAlert_state->indication_time;
				}elsif ( $params{config} eq "all"){
					$state=$SystemHealthAlert_state;
				}else{
					$Log->error("Invalid config type $params{config} specified for system health alert show");
				}
				$Log->debug("System Health Autosupport Trigger History Field output: $state ");	
	 
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};
		}

	}else{
		$Log->error("No config type specified for system health alert show");
	}
	return $state;

}

#*******************************************************************************************
#Function Name  :delete_health_alert
#Paremeters     :command 
#Return Type    : 
#Description    :Delete's an SHM alert
#*******************************************************************************************
sub delete_health_alert {
	my %params = (
            node=>"local",
	    alert_id=>undef,
	    monitor=>undef,
	    alerting_resource=>undef,
            @_
	);
	my $state=check_health_alert_table();
	if ( $state =~ /No matching system health alert/i || $state =~ /There are no entries matching your query/i ){
		return $state;
	}
	#$cmd='system health config show -'.$key.' '.$value;
	if ( defined($params{alert_id}) && defined($params{monitor}) && defined($params{alerting_resource})){
		if (SHM::_get_version() =~ /7M/i){
			$state=$Server->cli(cmd=>"system health alert delete -node local -alert-id $params{alert_id} -monitor $params{monitor} -alerting-resource $params{alerting_resource}");
		}else{	
		    my $Node = _get_nacl_node( $node_names[0] );
			try{
				my $SystemHealthAlert  = NACL::C::SystemHealthAlert->new(command_interface =>$Node,alert_id=>$params{alert_id},alerting_resource=>$params{alerting_resource},monitor=>$params{monitor},node=>$params{node});
				$state = $SystemHealthAlert->delete();
			 
			}




			catch NATE::BaseException with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};			
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};		
		}

		$Log->debug("System Health Alert Delete output: $state ");
	}
	else{
		$Log->error("Cannot delete alert, missing alert id, monitor, or alerting resource value");
	}
	return $state;

}

#*******************************************************************************************
#Function Name  :modify_health_alert
#Paremeters     :command 
#Return Type    : 
#Description    :Delete's an SHM alert
#*******************************************************************************************
sub modify_health_alert {
	my %params = (
            node=>"local",
	    alert_id=>undef,
	    monitor=>undef,
	    alerting_resource=>undef,
	    modify_vals=>undef,
            @_
	);
	my $state=check_health_alert_table();
	if ( $state =~ /No matching system health alert/i || $state =~ /There are no entries matching your query/i ){
		return $state;
	}
	if ( defined($params{alert_id}) && defined($params{monitor}) && defined($params{alerting_resource})){
		my %vals=%{$params{modify_vals}};
		if (SHM::_get_version() =~ /7M/i){
			my $mod_string=" ";
			foreach my $key (keys %vals) {
				$mod_string = $mod_string."-$key $vals{$key}";
			}
			$Log->debug("Mod string: $mod_string");
			$Server->cli(cmd=>"system health alert modify -node local -alert-id $params{alert_id} -monitor $params{monitor} -alerting-resource $params{alerting_resource} $mod_string");
			my @show_details=split(/\n/,($Server->cli(cmd=>"system health alert show -node local -alert-id $params{alert_id} -monitor $params{monitor} -alerting-resource $params{alerting_resource} $mod_string -instance")));
			$Log->debug("Temps array @show_details");
			my $size = scalar(@show_details) - 1;
			for(my $i = 0; $i<=$size;$i++){
				next if (length($show_details[$i]) < 1);
				$show_details[$i] =~ s/^\s+//g;
				$show_details[$i] =~ s/\s+$//g;

				$Log->debug("Value:  $show_details[$i]");
				my @temps=split(/\: /,$show_details[$i]);
				if ($temps[0] eq "Node" ){
					$state=$temps[-1];
					last;
				}
			}			
		}else{			
		    my $Node = _get_nacl_node( $node_names[0] );
			try{
				my $SystemHealthAlert = NACL::C::SystemHealthAlert->new(command_interface =>$Node,alert_id=>$params{alert_id},alerting_resource=>$params{alerting_resource},monitor=>$params{monitor},node=>$params{node});

				$SystemHealthAlert->modify(%vals);
				my $alert_state=$SystemHealthAlert->state();
				$state=$alert_state->node;
			 
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};		
		}

		$Log->debug("System Health Alert Modify output: $state ");

	}
	else{
		$Log->error("Cannot modify alert, missing alert id, monitor, or alerting resource value");
	}
	return $state;

}

#*******************************************************************************************
#Function Name  :modify_health_policy_definition
#Paremeters     :command 
#Return Type    : 
#Description    :Delete's an SHM alert
#*******************************************************************************************
sub modify_health_policy_definition {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    policy_id=>undef,
	    modify_vals=>undef,
            @_
	);
	my $state;
	#$cmd='system health policy definition show -'.$key.' '.$value;
	if ( defined($params{policy_id}) && defined($params{monitor})){
		my %vals=%{$params{modify_vals}};
		if (SHM::_get_version() =~ /7M/i){
			my $mod_string=" ";
			foreach my $key (keys %vals) {
				$mod_string = $mod_string."-$key $vals{$key}";
			}
			$Log->debug("Mod string: $mod_string");
			$Server->cli(cmd=>"system health policy definition modify -policy-id $params{policy_id} -monitor $params{monitor} $mod_string");
			my @show_details=split(/\n/,($Server->cli(cmd=>"system health policy definition show -policy-id $params{policy_id} -monitor $params{monitor} $mod_string -instance")));
			$Log->debug("Temps array @show_details");
			my $size = scalar(@show_details) - 1;
			for(my $i = 0; $i<=$size;$i++){
				next if (length($show_details[$i]) < 1);
				$show_details[$i] =~ s/^\s+//g;
				$show_details[$i] =~ s/\s+$//g;
				$Log->debug("Value:  $show_details[$i]");
				my @temps=split(/\: /,$show_details[$i]);
				if ($temps[0] eq "Node" ){
					$state=$temps[-1];
					last;
				}
			}			
		}else{
		    my $Node = _get_nacl_node( $node_names[0] );
			try{
				my $SystemHealthPolicyDefinition = NACL::C::SystemHealthPolicyDefinition->new(command_interface =>$Node,policy_id=>$params{policy_id},monitor=>$params{monitor},node=>$params{node});
				$SystemHealthPolicyDefinition->modify(%vals);
				my $alert_state=$SystemHealthPolicyDefinition->state();
				$state=$alert_state->node;
			 
			}
			catch NACL::Exceptions::NoElementsFound with{
				my $exception_object = shift;
				$state=$exception_object->text(); 
			};		
		}
		$Log->debug("System Health Policy Definition Modify output: $state ");
	}
	else{
		$Log->error("Cannot modify health policy definition, missing policy id or monitor value");
	}
	return $state;

}


#*******************************************************************************************
#Function Name  :create_health_subscriptions
#Paremeters     :command 
#Return Type    : 
#Description    :Delete's an SHM alert
#*******************************************************************************************
sub create_health_subscriptions {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    event_type=>undef,
	    subscription_id=>undef,
	    notify_dest_node=>undef,
            notify_dest_hm=>undef,
            class_name=>undef,
#	    creation_time => undef,
#            max_notify_period => undef,
#            psc_option => undef,
#            notify_table => undef,
#            fail_thresh => undef,
#            instance_name => undef,
#            notify_type => undef,
#            time_gap_notify => undef,
	   additional_vals => undef,
	    @_
	);
	my $state;
	my %vals;
	#"system health subscriptions modify -node local -subscription-id test_shelf -monitor node_connect -notify-dest-node $schm_node -notify-dest-hm system_connect -class-name nschm_shelf_info -event-type rm_mod -notify-table test_mgwd_schm_helper_view -psc-option true -time-gap-notify 2 -max-notify-period 1 -fail-thresh 1";
	if ( defined($params{event_type}) && defined($params{monitor}) && defined($params{subscription_id}) && defined($params{notify_dest_node}) && defined($params{notify_dest_hm}) && defined($params{class_name})){
		if ( $params{additional_vals}){
			%vals = %{$params{additional_vals}};
		}
		if (SHM::_get_version() =~ /7M/i){
			$state=$Server->cli(cmd=>"priv set test;system health subscriptions create -node local -monitor $params{monitor} -event-type $params{event_type} -subscription-id $params{subscription_id} -notify-dest-node $params{notify_dest_node} -notify-dest-hm $params{notify_dest_hm} -class-name $params{class_name} ".%vals);
		}else{	
		    my $Node = _get_nacl_node( $node_names[0] );
			my $SystemHealthSubscriptions;
			#the component fails if you pass additional options on the max_notify_period
			if ( $params{additional_vals}){
				#$SystemHealthSubscriptions = NACL::C::SystemHealthSubscriptions->create(command_interface =>$Node,'event-type'=>$params{event_type},monitor=>$params{monitor},node=>$params{node},'class-name'=>$params{class_name},'subscription-id'=>$params{subscription_id},'notify-dest-node'=>$params{notify_dest_node},'notify-dest-hm'=>$params{notify_dest_hm},%vals);
				$Log->comment("Subscriptions not supported in 7M");
			}
			else{
				$SystemHealthSubscriptions = NACL::C::SystemHealthSubscriptions->create(command_interface =>$Node,'event-type'=>$params{event_type},monitor=>$params{monitor},node=>$params{node},'class-name'=>$params{class_name},'subscription-id'=>$params{subscription_id},'notify-dest-node'=>$params{notify_dest_node},'notify-dest-hm'=>$params{notify_dest_hm});
			}
			my $alert_state=$SystemHealthSubscriptions->state();
			$state=$alert_state->node;
		}
		$Log->debug("System Health Subscriptions Create output: $state ");
	}
	else{
		$Log->error("Cannot create health subscription event type, subscritpion id, notify destination node or hm, class name or monitor value");
	}
	return $state;

}

#*******************************************************************************************
#Function Name  :modify_health_subscriptions
#Paremeters     :command 
#Return Type    : 
#Description    :Delete's an SHM alert
#*******************************************************************************************
sub modify_health_subscriptions {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    event_type=>undef,
	    subscription_id=>undef,
	    notify_dest_node=>undef,
            notify_dest_hm=>undef,
            class_name=>undef,
#	    creation_time => undef,
#            max_notify_period => undef,
#            psc_option => undef,
#            notify_table => undef,
#            fail_thresh => undef,
#            instance_name => undef,
#            notify_type => undef,
#            time_gap_notify => undef,
	   additional_vals => undef,
	    @_
	);
	$Log->debug("Modifying w/ values: ".Dumper($params{additional_vals}));
	my %vals = %{$params{additional_vals}};
	#"system health subscriptions modify -node local -subscription-id test_shelf -monitor node_connect -notify-dest-node $schm_node -notify-dest-hm system_connect -class-name nschm_shelf_info -event-type rm_mod -notify-table test_mgwd_schm_helper_view -psc-option true -time-gap-notify 2 -max-notify-period 1 -fail-thresh 1";
	if ( defined($params{event_type}) && defined($params{monitor}) && defined($params{subscription_id}) && defined($params{notify_dest_node}) && defined($params{notify_dest_hm}) && defined($params{class_name})){
		if (SHM::_get_version() =~ /7M/i){
			#$state=$Server->cli(cmd=>"priv set test;system health subscriptions modify -node local -monitor $params{monitor} -event-type $params{event_type} -subscription-id $params{subscription_id} -notify-dest-node $params{notify_dest_node} -notify-dest-hm $params{notify_dest_hm} -class-name $params{class_name} ".%vals);
			$Log->comment("Subscriptions not supported in 7M");
		}else{			
		    my $Node = _get_nacl_node( $node_names[0] );
			my $SystemHealthSubscriptions = NACL::C::SystemHealthSubscriptions->new(command_interface =>$Node,event_type=>$params{event_type},monitor=>$params{monitor},node=>$params{node},class_name=>$params{class_name},subscription_id=>$params{subscription_id},notify_dest_node=>$params{notify_dest_node},notify_dest_hm=>$params{notify_dest_hm});
			
			$Log->debug("Modifying w/ values: ".Dumper(%vals));
			$SystemHealthSubscriptions->modify(%vals);
			my $alert_state=$SystemHealthSubscriptions->state();
			$state=$alert_state->node;
		}
		$Log->debug("System Health Subscriptions Modify output: $state ");
	}
	else{
		$Log->error("Cannot modify health subscription event type, subscritpion id, notify destionation node or hm, class name or monitor value");
	}
	return $state;

}


#*******************************************************************************************
#Function Name  :delete_health_subscriptions
#Paremeters     :command 
#Return Type    : 
#Description    :Delete's an SHM alert
#*******************************************************************************************
sub delete_health_subscriptions {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    event_type=>undef,
	    subscription_id=>undef,
	    notify_dest_node=>undef,
            notify_dest_hm=>undef,
            class_name=>undef,
	    modify_vals=>undef,	    
	    @_
	);
	my $state=0;
	#"system health subscriptions modify -node local -subscription-id test_shelf -monitor node_connect -notify-dest-node $schm_node -notify-dest-hm system_connect -class-name nschm_shelf_info -event-type rm_mod -notify-table test_mgwd_schm_helper_view -psc-option true -time-gap-notify 2 -max-notify-period 1 -fail-thresh 1";
	if ( defined($params{event_type}) && defined($params{monitor}) && defined($params{subscription_id}) && defined($params{notify_dest_node}) && defined($params{notify_dest_hm}) && defined($params{class_name})){
		if (SHM::_get_version() =~ /7M/i){
			#$Server->cli(cmd=>"priv set test;system health subscriptions delete -node local -monitor $params{monitor} -event-type $params{event_type} -subscription-id $params{subscription_id} -notify-dest-node $params{notify_dest_node} -notify-dest-hm $params{notify_dest_hm} -class-name $params{class_name}");
			$Log->comment("Subscriptions not supported in 7M");
		}else{
		    my $Node = _get_nacl_node( $node_names[0] );
			my $SystemHealthSubscriptions = NACL::C::SystemHealthSubscriptions->delete(command_interface =>$Node,'event-type'=>$params{event_type},monitor=>$params{monitor},node=>$params{node},'class-name'=>$params{class_name},'subscription-id'=>$params{subscription_id},'notify-dest-node'=>$params{notify_dest_node},'notify-dest-hm'=>$params{notify_dest_hm});
		}
		$state=get_health_subscriptions_count('subscription-id'=>$params{subscription_id},node=>$params{node});
		$Log->debug("System Health Subscriptions Delete output: $state ");
	}
	else{
		$Log->error("Cannot delete health subscription event type, subscritpion id, notify destionation node or hm, class name or monitor value");
	}
	return $state;

}


#*******************************************************************************************
#Function Name  :get_health_subscriptions_count
#Paremeters     :command 
#Return Type    :alerts definition table
#Description    :Gets the current health alert definitions
#*******************************************************************************************
sub get_health_subscriptions_count {
	my %params = (
            node=>"local",
	    monitor=>undef,
	    'event-type'=>undef,
	    'subscription-id'=>undef,
	    'notify-dest-node'=>undef,
            'notify-dest-hm'=>undef,
            'class-name'=>undef,
	    @_
	);
	my @values;
	if ($version =~ /7M/i){
		$Log->comment("Subscriptions not supported in 7M");
		@values=();
	}else{	
		my $Node = _get_nacl_node( $node_names[0] );
		my $NGSH = $Node->apiset(
		    interface => "CLI",
		    );
		$NGSH->set_privilege('privilege-level' => "test");
		try {
			my $response = $NGSH->system_health_subscriptions_show(%params);
			@parsed_output = @{$response->get_parsed_output()}; 
			foreach my $hash (@parsed_output){
				foreach my $hvalue (keys %{$hash}){
					$Log->debug("Hash Key for Health Subscriptions Show: $hvalue");
					if ( $hvalue eq 'subscription-id'){
						push(@values,$hash->{$hvalue});
						$Log->debug("Adding Execution Node Name $node"); 
					}
				}
			}
		}
		catch NATE::BaseException with {
			my $exception_object = shift;
			$Log->debug( "Error is: " . $exception_object->text() );
		};
	}

	$Log->debug("System Health Subscription values:  @values");
	return scalar(@values);

}
###########################################################################
# TestCase: get_adapter_state
# Parameters: None
# Returns: TCD Execution Result
# Purpose: Return the state of all the adapdters
#############################################################################
sub get_adapter_state {
    my %params = (
            node=>"local",
	    @_
    );
    if ( SHM::_get_version() =~ /7M/i ) {
	$Log->comment("Invoking the 7M command");
	return $Server->cli(cmd=>"priv set test;sasadmin adapter_state");
    }else{
	my $NGSH = $Nodes[0]->apiset(
		    interface => "CLI",
		    );
	$NGSH->set_privilege('privilege-level' => "test");
	my $state = $NGSH->system_node_run(node=>$params{node}, command=>"sasadmin adapter_state")->get_raw_output();
	$Log->comment("Adapter State: $state");
	return $state;
    }
}

#*******************************************************************************************
#Function Name  :start_ehm
#Paremeters     :Policy file to pass to the Example health monitor
#Return Type    :Returns EHM log file if the server started, 0 if the server is not
#Description    :Start the Example Health Monitor
#*******************************************************************************************

sub start_ehm {
	my %params = (
	    node=>undef,
        policy_file=>undef,
		clear_mdb=>"true",
	    @_
	);
	$Log->debug("Params hash:  ".Dumper(%params));
	my $result=undef;
	my $Node;
	if ( EMS::_get_version() =~ /7M/i){
		my @temp=$Server->servers();
		$Node=$temp[0];
	}else{
		$Node=$node_names[0];
	}
	if ( defined($params{node}) ){
		$Node=$params{node};	
	}
	$Log->comment("Starting the EHM on node $Node");
    my $nacl_node = _get_nacl_node( $Node );
	my $shell = $nacl_node->apiset(
		    interface => "CLI",
            set       => "Systemshell",
		    );
    
	my $ehm_start;
	if ( $Version =~ /FS/) {
	    $ehm_start="/sbin/ehmd -t 1 -f";
	} else {
	    $ehm_start="/sbin/ehmd -f";
	}
	my $check_for_binary="ls /sbin/ehmd";
	$cmd_out=$shell->execute_raw_command(command=>"$check_for_binary");
	if ( $cmd_out =~ /No such file or directory/i){
		$Log->error("EHM binary not found, returning.");
		return 0;
	}		
	my $cmd_out=$shell->execute_raw_command(command=>"ps -aux |grep ehmd|grep -v grep");
	$EHM_LOG = "/tmp/ehm_".time().".txt";
	if ( $cmd_out =~ /ehmd/){
		$Log->comment("EHM already started, output from ps command:  $cmd_out");
	    $Log->comment("Killing ehmd processes...");
	    $shell->execute_raw_command(command=>"sudo pkill -9 ehmd");
	    $result=1;
	}
    # Cleanup old mdb directory
	if ($params{clear_mdb} eq "true"){
		$cmd_out=$shell->execute_raw_command(command=>"rm -rf /mroot/etc/cluster_config/mdb/ehmd");
		Tharn::sleep(10);
		$cmd_out=$shell->execute_raw_command(command=>"ls -l /mroot/etc/cluster_config/mdb/ehmd");
	    if ($cmd_out !~ /No such file/i) {
	        $Log->warn("mdb is not deleted yet, one more try...");
	        $cmd_out=$shell->execute_raw_command(command=>"rm -rf /mroot/etc/cluster_config/mdb/ehmd");
	    }
	}
	$cmd_out=$shell->execute_raw_command(command=>"rm -rf /tmp/ehm_*");
	$cmd_out=$shell->execute_raw_command(command=>"rm -rf $HM_DIR");
	if (defined($params{policy_file})){
		$Log->debug("Copying new policy file into /etc/hm directory");
		write_hm_file(node=>$params{node},file=>$params{policy_file});
	}	
	$cmd_out=$shell->execute_raw_command(command=>"sudo $ehm_start >& $EHM_LOG &");
	Tharn::sleep(30);
	$cmd_out=$shell->execute_raw_command(command=>"ps -aux |grep ehmd|grep -v grep");
	if ( $cmd_out =~ /ehmd/){
		$Log->debug("EHM in process list, output from ps command:  $cmd_out");
        $result=$EHM_LOG;
        $Log->debug("Got logfile result $result");
	} 
	
	return $result;
	
}

#*******************************************************************************************
#Function Name  :wait_for_teststart
#Paremeters     :EHM log file
#Return Type    :Returns EHM test id if the server started, -1 if the server is not running
#Description    :Start the Example Health Monitor
#*******************************************************************************************
sub wait_for_teststart {
    my %params = (
        node=>"local",
        log_file=>undef,
        @_
    );
    my $Node;
    if ( $Version =~ /7M/i){
        my @temp=$Server->servers();
        $Node=$temp[0];
    }
    if ( defined($params{node}) ){
        $Node=$params{node};    
    }
    
    if (!defined($params{log_file})) {
        $Log->error("log file not found");
        return -1;
    }

    my $nacl_node = _get_nacl_node( $Node );
	my $shell = $nacl_node->apiset(
		    interface => "CLI",
            set       => "Systemshell",
		    );
        my $grepstring="'TEST'";
        my $cmd_out;
        my $timeout = 0;
        do {
            Tharn::sleep(20);
            $timeout = $timeout + 20;
            $cmd_out=undef;
            $cmd_out=$shell->execute_raw_command(command=>"tail -1 $params{log_file} | grep $grepstring");
            $Log->debug("Got tail output $cmd_out");
            if ($cmd_out =~ /Assertion failed/i) {
                $Log->error("EHM Crashed!!!!!");
                return -1;
            }
            if ($cmd_out =~ /PASSED/i) {
                $Log->debug("Test passed");
                return 0;
            }
            if ($cmd_out =~ /SETUP COMPLETE/i) {
                if ($cmd_out =~ /TEST ([0-9]*)\:[a-zA-Z0-9]*/) {
                    $Log->comment("Got current running test value $1");
                    return $1;
                } else {
                    $Log->error("Failed to get test id");
                }
            }
            if ($cmd_out =~ /No such file/i) {
                $Log->error("Fail to find the file. $cmd_out");
            }
        } while($timeout < 1800 );   #ehmd max running time is 20 minutes, set timeout 30 minute
    return -1;
}

#*******************************************************************************************
#Function Name  :stop_ehm
#Paremeters     :None
#Return Type    :Returns 1 if the server stopped, 0 if the server is not
#Description    :Stop the Example Health Monitor
#*******************************************************************************************

sub stop_ehm {
	my %params = (
	    node=>undef,
	    @_
	);
	my $Node;
	if ( SHM::_get_version() =~ /7M/i){
		my @temp=$Server->servers();
		$Node=$temp[0];
	}else{
		$Node=$Nodes[0]->name;
	}
	if ( defined($params{node}) ){
		$Node=$params{node};	
	}
    my $nacl_node = _get_nacl_node( $Node );
	my $shell = $nacl_node->apiset(
		    interface => "CLI",
            set       => "Systemshell",
		    );
	my $ehm_start="/sbin/ehmd -f";
	my $cmd_out=$shell->execute_raw_command(command=>"ps -aux |grep ehmd");	
	if ( $cmd_out !~ /$ehm_start/){
		$Log->debug("EHM not started, skipping kill");
		return 1;
	}
	my $result=1;
	my $ehm_stop="sudo pkill -9 ehmd";
	$cmd_out=$shell->execute_raw_command(command=>"sudo $ehm_stop");
	Tharn::sleep(10);
	$shell->execute_raw_command(command=>"sudo rm /tmp/ehm_*");
	$cmd_out=$shell->execute_raw_command(command=>"ps -aux |grep ehmd");	
	if ( $cmd_out =~ /ehmd -f/){
		$Log->debug("EHM in process list, output from ps command:  $cmd_out");
		$result=0;
	}
	return $result;
}


#*******************************************************************************************
#Function Name  :update_sysctl_value
#Paremeters     :sysctl string and value
#Return Type    :Returns 1 if the systcl is updated, 0 if the systcl is not
#Description    :Update a sysctl value
#*******************************************************************************************

sub update_sysctl_value {
	my %params = (
	    node=>undef,
            sysctl_string=>undef,
	    value=>undef,
            @_
	);
	$Log->debug("Sysctl params: $params{node}, $params{sysctl_string}, $params{value}");
	if (defined($params{sysctl_string}) && defined($params{value})){
		my $Node;
		if ( SHM::_get_version() =~ /7M/i){
			my @temp=$Server->servers();
			$Node=$temp[0];
		}else{
			$Node=$Nodes[0]->name;
		}
		if ( $params{node} ){
			$Node=$params{node};	
		}		
        my $nacl_node = _get_nacl_node( $Node );
	    my $shell = $nacl_node->apiset(
		    interface => "CLI",
            set       => "Systemshell",
		    );
		my $command="sudo sysctl $params{sysctl_string}=$params{value}";
		my $check_return="echo \$?";
		my $cmd_out=$shell->execute_raw_command(command=>"$command");
		my $return_code=$shell->execute_raw_command(command=>"$check_return");
		$Log->debug("Return code: $return_code");
		if ( $return_code !~ /0/ ){
			$Log->error("Error setting the sysctl $params{sysctl_string} to the value $params{value}.  Result:  $cmd_out");
			return 0;
		}
		else{
			return 1;
		}
	}
	else{
		$Log->error("No sysctl string or value specified");
		return 0;
	}
}

###########################################################################
# Subroutine: trigger_alert()
# Parameters: None
# Returns: TCD Execution Result (passing)
# Purpose: Trigger the alert using the SAS or EHM
###########################################################################

sub trigger_alert {
    my %params = (
        flag=>"false",
		policy_file=>undef,
        @_
    );	
    $Log->debug("Trigger Alert depending on hardware");
    my $alert_status=0;
    if ( check_for_sas_adapter() ){
      $alert_status=trigger_sas_alert(flag=>$params{flag});
    }else{
      $alert_status=trigger_ehm_alert(policy_file=>$params{policy_file});
    }
    #$alert_status=trigger_ehm_alert();
    $Log->comment("Trigger Alert Return value: $alert_status");
    return $alert_status;
}

###########################################################################
# Subroutine: trigger_sas_alert()
# Parameters: None
# Returns: TCD Execution Result (passing)
# Purpose: Trigger SAS Alert by bring down one of the adapters
###########################################################################
#noticed you can also trigger an alert if CLUSTER HA is disabled on a 2 Node cluster(CMODE ONLY)
sub trigger_sas_alert {
    my %params = (
        flag=>"false",
		nowait=>undef,
        @_
    );
    $Log->debug("Triggering a SAS Alert");
    ## Make sure there are multiple SAS adpater before offline Adapater
    if ( !check_for_sas_adapter() ){
        $Log->error("Unable to run an alert. NODE must be hardware with at least 2 up adapters");
        return $TCD::FAIL; 
    }
    
    #randomly pick an adapter to offline
    my @nchm_adapters = get_nchm_disks();
    $Log->debug("Adapters:  @nchm_adapters");
    my @adapter= split(/\./,$nchm_adapters[0]);
    $test_adapter = $adapter[0];
    $Log->comment("***Disabling the adapter $test_adapter***") ;    
    disable_sas_adapter(adapter=>"$test_adapter");
    if(defined($params{nowait}))
    {
		$Log->comment("***Not waiting for the alert to be genereated ***") ;    
		return $test_adapter;
    }
    $Log->debug("***Waiting for alert status***") ;
    my $alert_status=wait_for_alert(count=>1,node=>$node_names[0]);
    if ($params{flag} eq "false" ){
		if ($alert_status !~ /-/ ){
			#try one more time by restarting the service
			spmctl_exec(node=>$node_names[0],service=>"nchmd",command=>"stop");
			$Log->comment("Restarting the $service");
            my $nacl_node = _get_nacl_node( $node_names[0] );
	        my $shell = $nacl_node->apiset(
		        interface => "CLI",
                set       => "Systemshell",
		    );
			my $command="sudo rm -rf /mroot/etc/cluster_config/mdb/nchmd";			
			Tharn::sleep(15);
			spmctl_exec(node=>$node_names[0],service=>"nchmd",command=>"start");  		
		}
		wait_for_alert(count=>5);
		my $status = health_monitor_status_show();
		$Log->debug("Health Monitor Status:  $status");
		if(  lc($status) !~ /degraded/ && _policies_enabled())
		{
			$Log->error("After alert generation still system is in not degraded state");
			enable_sas_adapter(adapter=>$test_adapter);
			return 0;
		 }	
    }else{
		if( !$alert_status)
		{
			$Log->comment("Alert not generated");
			enable_sas_adapter(adapter=>$test_adapter);
			return "";
		}		
    }
    return $test_adapter;
}

###########################################################################
# Subroutine: trigger_ehm_alert()
# Parameters: None
# Returns: 1 for pass, 0 for fail
# Purpose: Trigger EHM Alert by starting the EHM and changing its sysctl to '2'
###########################################################################

sub trigger_ehm_alert {
    my %params = (
        node=>undef,
		policy_file=>undef,
        @_
    );	
    $Log->debug("Triggering a EHM Alert");
    $Log->debug("***Starting the EHM on node $params{node}***") ;
	my $exec_node=$node_names[0];
	if ( defined($params{node})){
		$exec_node=$params{node};
	}

	$Log->comment("Triggering alert on node $exec_node");
	my $ehm_log_file=start_ehm(node=>$exec_node,policy_file=>$params{policy_file});
    if ( !ehm_log_file ){
		$Log->error("Unable to start the EHM on node $exec_node, exiting");
		return 0;

    }
	my $testcaseid=wait_for_teststart(node=>$exec_node, log_file=>$ehm_log_file);	
    $Log->debug("***Waiting for alert status***") ;  
    while ($testcaseid <= 0 && $testcaseid != -1) {
        Tharn::sleep(5);
        $testcaseid=wait_for_teststart(node=>$exec_node, log_file=>$ehm_log_file);    }
        $Log->debug("EHM running testcase id $testcaseid");
        if ($testcaseid == 1) {
            wait_for_alert();
            if (health_alert_show('alert-id'=>$EHM_ALERTID1,'alerting-resource'=>"$EHM_RESOURCE1") != $exec_node || health_alert_show('alert-id'=>$EHM_ALERTID2,'alerting-resource'=>$EHM_RESOURCE2) != $exec_node)
            {
    			$Log->error("Unable to trigger an alert with EHM");
    			return 0;
            }
    }  

    return 1;
}

###########################################################################
# Subroutine: trigger_ems_alert()
# Parameters: None
# Returns: 1 for pass, 0 for fail
# Purpose: Trigger EMS Alert by starting invoking an alert w/ the event menu
###########################################################################
sub trigger_ems_alert() {
    $Log->debug("Triggering a EMS Alert");
    my %params = (
        node=>undef,
            @_
    );

    my $node=$Nodes[0]->name();
    if ( defined($params{node}) ){
	$node=$params{node};
    }

    if ( !start_ehm(node=>$node) ){
	$Log->error("Unable to start the EHM on node $node, exiting");
	return 0;
    }
    if (!(EMS::create_EMS_eventcatalog())) {
       $Log->error("Unable to load EMS events, exiting...\n");
       return $TCD::FAIL;
    }
    EMS::invoke_event(event=>$EMS_ALERTS[0]);
    $Log->debug("***Waiting for alert status***") ;  
    wait_for_alert(node=>$node,'alert-id'=>"Ems*");
    return $EMS_ALERTS[0];
}


###########################################################################
# Subroutine: restore_sas_adapter()
# Parameters: 1 for pass, 0 for fail
# Returns: TCD Execution Result (passing)
# Purpose: Bring up one of the SAS adapters
###########################################################################

sub restore_sas_adapter {
    my %params = (
        adapter=>undef,
	@_
    );
    $Log->debug("Restoring SAS Adapter");
    if ( !defined($params{adapter}) || length($params{adapter}) == 1 ){
	$Log->error("Invalid or no adapter $params{adapter} given, exiting");
	return 0;
    }
    $Log->debug("***Enabling the adapter $params{adapter}***") ;    
    enable_sas_adapter(adapter=>"$params{adapter}");
    if(get_adapter_state() !~ /UP/i)
    {
	    $Log->error("Failed to enable the $params{adapter}, so exiting");
	    return 0;
    }
    return 1;
}

###########################################################################
# Subroutine: _check_model()
# Parameters: None
# Returns: TCD Execution Result (passing)
# Purpose: Find the MODEL Type
###########################################################################
sub _check_model() {
	my $model="FAS";
	if ( SHM::_get_version() !~ /7M/i ){
		try{
			my $Node=$Nodes[0];
            # Node is an alias of SystemNode
			my $SystemNode_state   = NACL::CS::Node->fetch(
				command_interface => $Node,
			);
			$model=$SystemNode_state->model;
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state = $exception_object->text();	
		};
	}else{
		if ($node_names[0] =~ /vsim/i){
			$model="SIM";
		}
	}
	$Log->comment("Model:  $model");
	return $model;
}
###########################################################################
# Subroutine: check_for_sas_adapter()
# Parameters: 1 for pass, 0 for fail
# Returns: TCD Execution Result (passing)
# Purpose: Check to see if system has SAS adapters and they are online
###########################################################################

sub check_for_sas_adapter() {
    my $response;
    my $found = 0;
	if ( _check_model() =~ /SIM/i){
		return $found;
	}
    if ( SHM::_get_version() =~ /7M/i ) {
	my @show_details=split("\n",($Server->cli(cmd=>"storage show disk -T")));
	foreach my $val (@show_details){
		if ( lc($val) =~ /sas/ ){
			$Log->debug("Found SAS adapter ".$val);
			$found = 1;
			last;
		}
	}
    }else{
	my $NGSH = $Nodes[0]->apiset(
	    interface => "CLI",
	  );
	$NGSH->set_privilege('privilege-level' => "test");
	$response = $NGSH->storage_disk_show(fields=>"type");
	@parsed_output = @{$response->get_parsed_output()}; 
	foreach my $hash (@parsed_output){
		foreach my $hvalue (keys %{$hash}){
			$Log->debug("Hash Key for Cluster Show: $hvalue");
			if ( lc($hvalue) eq "type"){
				if ( lc($hash->{$hvalue}) =~ /sas/ ){
					$Log->debug("Found SAS adapter ".$hash->{$hvalue});
					$found = 1;
					last;
				}
			}
		}
		if ( $found ){
			last;
		}	
	}
    }
    if ( $found ){
	$found=0;
        my $cmd='sasadmin adapter_state';
        my $cmdout = run_dblade_command(command=>$cmd);	
	my @states=split(/\n/,$cmdout);
	my $count = 0;
	foreach my $state (@states){
		$Log->debug("Adapter state: $state");
		if ( lc($state) =~ /up/  ){
			$count++;
			if ($count >= 2){
				$Log->debug("Found mulitple online adapters");
				$found = 1;
				last;
			}
		}
		
	}
    }
    return $found;
}
###########################################################################
# Subroutine: restore_ehm()
# Parameters: 'all' 
# Returns: TCD Execution Result (passing)
# Purpose: Cleanup the ehm(s) and stop the process(es)
###########################################################################

sub restore_ehm() {
    # remove the following codes, because ehmd only running on $Nodes[0]
    #my @ehm_nodes = get_nodes(type=>"name");
    #foreach my $node (@ehm_nodes){   
		stop_ehm();
    #}
    
}

###########################################################################
# Subroutine: restore_hm()
# Parameters: SAS adapter or integer for EHM
# Returns: TCD Execution Result (passing)
# Purpose: Cleanup the ehm(s) and stop the process(es) or bring the SAS adapter online
###########################################################################

sub restore_hm {
    my %params = (
        adapter=>undef,
	@_
    );
    $Log->debug("Restoring EHM");
    if ( !defined($params{adapter}) ){
		$Log->error("No adapter $params{adapter} given, exiting");
		return 0;
    }
    if ( length($params{adapter}) > 1 ){
		restore_sas_adapter(adapter=>$params{adapter});
    }else{
		restore_ehm();
    }
    
}


###########################################################################
# TestCase: asup_create_alert
# Parameters: As applicable 
# Returns: TCD Execution Result
# Purpose: Tiggers and Health Auto support 
#############################################################################
sub asup_create_alert {
	my %params = (
        node=>undef,
	    alert_id=>undef,
	    monitor=>undef,
	    alerting_resource=>undef,
		autosupport_triggered =>undef,
	    @_,
	);
	my $state= "";
	$Log->debug("System Health Autosupport Trigger");
	if (SHM::_get_version() =~ /7M/i){
		$state=$Server->cli(cmd=>"priv set diag;system health autosupport trigger history create -node local -alert-id $params{alert_id} -monitor $params{monitor} -alerting-resource $params{alerting_resource} -autosupport-triggered $params{autosupport_triggered}");
	}else{
		my $Node = _get_nacl_nodes( $node_names[0] );
		try {
				my $SystemHealthAutosupportTriggerHistory_obj = NACL::C::SystemHealthAutosupportTriggerHistory->create(
																	command_interface =>$Node,
																	'alert-id'=>$params{alert_id},
																	'alerting-resource'=>$params{alerting_resource},
																	monitor=>$params{monitor},node=>$params{node}, 
																	'autosupport-triggered' => $params{autosupport_triggered}
																);
				my $temp = $SystemHealthAutosupportTriggerHistory_obj->state;
				$state = $temp->node;
		}
		catch NACL::APISet::Exceptions::CommandFailedException  with {
																my $exception_object = shift;
																my $resp = $exception_object->get_response_object();
																$state=$resp->get_raw_output;
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
																my $exception_object = shift;
																my $resp = $exception_object->get_response_object();
																$state=$resp->get_raw_output;
		};
	}
	
	return $state;
	
}

###########################################################################
# TestCase: asup_modify_alert
# Parameters: As applicable 
# Returns: TCD Execution Result
# Purpose: Modify System Health Alterting resource  
#############################################################################
sub asup_modify_alert {
	my %params = (
        node=>undef,
	    alert_id=>undef,
	    monitor=>undef,
	    alerting_resource=>undef,
		autosupport_triggered=>undef,
	    @_,
	);
	my $state="";
	$Log->debug("System Health Autosupport Modify");
	if (SHM::_get_version() =~ /7M/i){
		$Log->comment("Modifying autosupport history entries not supported in 7M");	
	}else{	
		my $Node = _get_nacl_node( $node_names[0] );
		try {
				my $SystemHealthAutosupportTriggerHistory_obj = NACL::C::SystemHealthAutosupportTriggerHistory->modify(
																	command_interface =>$Node,
																	'alert-id'=>$params{alert_id},
																	'alerting-resource'=>$params{alerting_resource},
																	monitor=>$params{monitor},
																	node=>$params{node},
																	'autosupport-triggered' => $params{autosupport_triggered},
																);
				$state = $SystemHealthAutosupportTriggerHistory_obj;
	
		}
		catch NACL::Exceptions::NoElementsFound with {
			    my $exception_object = shift;
				$state=$exception_object->text(); 
		};
	}
	return $state;
}
 
###########################################################################
# TestCase: asup_delete_alert
# Parameters: As applicable 
# Returns: TCD Execution Result
# Purpose: Delete System Health alert resource  
#############################################################################
sub asup_delete_alert {
	my %params = (
        node=>undef,
	    alert_id=>undef,
	    monitor=>undef,
	    alerting_resource=>undef,
	    @_,
	);
	$Log->debug("System Health Autosupport Trigger");
	my $state="";
	if (SHM::_get_version() =~ /7M/i){
		$state=$Server->cli(cmd=>"priv set diag;system health autosupport trigger history delete -node local -alert-id $params{alert_id} -monitor $params{monitor} -alerting-resource $params{alerting_resource}");
	}else{	
		my $Node = _get_nacl_node( $node_names[0] );
		try {
			my $SystemHealthAutosupportTriggerHistory_obj = NACL::C::SystemHealthAutosupportTriggerHistory->delete(
																command_interface =>$Node,
																'alert-id'=>$params{alert_id},
																'alerting-resource'=>$params{alerting_resource},
																monitor=>$params{monitor},
																node=>$params{node},
															);
			$state=$SystemHealthAutosupportTriggerHistory_obj ;
		}
		catch NATE::BaseException with {
			my $exception_object = shift;
			$Log->debug( "Error is: " . $exception_object->text() );
		};		
		catch NACL::Exceptions::NoElementsFound with{
		    my $exception_object = shift;
				$state=$exception_object->text(); 
		};
	}
	return $state;
	
}
###########################################################################
# TestCase: asup_show_alert
# Parameters: As applicable 
# Returns: TCD Execution Result
# Purpose: Show System Health Alert
#############################################################################
sub asup_show_alert {
	my %params = (
        node=>"local",
	monitor=>undef,
	subsystem=>undef,
	'alert-id'=>undef,
	monitor=>undef,
	'alerting-resource'=>undef,
	'autosupport-triggered'=>"true",
	@_,	
	);
	my $state="";
	$Log->debug("Params: ".Dumper(%params));
	if (SHM::_get_version() =~ /7M/i){
		my $options="-node local";
		if (defined($params{monitor})){
			$options=$options." -monitor $params{monitor}";
		}
		if (defined($params{'alert-id'})){
			$options=$options." -alert-id $params{'alert-id'}";
		}
		if (defined($params{subsystem})){
			$options=$options." -subsystem $params{subsystem}";
		}
		if (defined($params{'alerting-resource'})){
			$options=$options." -alerting-resource $params{'alerting_resource'}";
		}		
		my @show_details=split(/\n/,($Server->cli(cmd=>"system health autosupport trigger history show $options -instance")));
		$Log->debug("Temps array @show_details");
		my $size = scalar(@show_details) - 1;
		for(my $i = 0; $i<=$size;$i++){
			next if (length($show_details[$i]) < 1);
			$show_details[$i] =~ s/^\s+//g;
			$show_details[$i] =~ s/\s+$//g;
			$Log->debug("Value:  $show_details[$i]");
			my @temps=split(/\: /,$show_details[$i]);
			if ($temps[0] eq "Node" ){
				$state=$temps[-1];
				last;
			}
		}	
	}else{	
		my $Node = $Nodes[0];
		my $ref = \%params;
		try{
			my $SystemHealthAutosupportTriggerHistory_return   = NACL::C::SystemHealthAutosupportTriggerHistory->find(
				command_interface => $Node,
				filter=>$ref,
			);
		
			my $alert_state = $SystemHealthAutosupportTriggerHistory_return->state();
			$state=$alert_state->node;
			 
		}
		catch NACL::APISet::Exceptions::ResponseException with{
			$Log->comment("Ignoring RPC error, probably because of EHM");
			$state=$params{node};			
		}		
		catch NACL::Exceptions::NoElementsFound with{
		    my $exception_object = shift;
				$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;
		};
	}
	Tharn::sleep(90);
	$Log->debug("System Health Config Show output:  $state");
	return $state;
}

############################################################################
# TestCase: check_ehm_log
# Parameters: None
# Returns: $invoked_asup
# Purpose: Check for a string in the output of the Example Health Monitor
#############################################################################
sub check_ehm_log {
	my %params = (
            message=>undef,
            @_
	);
	my $shell = $Nodes[0]->apiset(
	    interface => "CLI",
        set       => "Systemshell",
	  );
	my $cmd_out=$shell->execute_raw_command(command=>"cat $EHM_LOG");
        my @lines = split(/\n/, $cmd_out);
        foreach my $line(@lines) {
		    $Log->debug("EHM message $line");
            if ( $line =~ /$params{message}/) {
                $Log->debug("Found message $params{message} in log $EHM_LOG");
                return 1
		    }
        }
    return 0;
}

############################################################################
# TestCase: spmctl_exec
# Parameters: None
# Returns: result of execution of spmctl
# Purpose: Change the service state using spmctl
#############################################################################
sub spmctl_exec {
    my %params = (
	    node => undef,
        service => undef,
        command => undef,
        @_
    );    
    if ( !$params{node} && !$params{service} && !$params{command} ){
        $Log->error("Node, Service name and command not specified, exiting");
        return 0;
    }

    my $nacl_node;
    if ( $params{node}->isa("NACL::C::Node") ) {
        $nacl_node = $params{node};
    } else {
        $nacl_node = _get_nacl_node( $params{node} );
    }

	$Api_Set_ObjS = $nacl_node->apiset(
	    interface => "CLI",
        set       => "Systemshell",
	  );

    my $response;
    my @processes;
    my $service = $params{service};
   
    if ( $params{command} eq "start" ){
        $response = $Api_Set_ObjS->spmctl(handle=>$params{service},exec=>1);
    }
    elsif ( $params{command} eq "stop" ){
        $response = $Api_Set_ObjS->spmctl(handle=>$params{service},stop=>1);
    }
    elsif ( $params{command} eq "list" ){
        @processes = @{$Api_Set_ObjS->spmctl(handle=>$params{service},list=>1)->get_parsed_output()};
        foreach my $hash (@processes){        
            foreach my $hvalue (keys %{$hash}){
                if (  $hvalue eq "Exec" ){
                    if ( $hash->{$hvalue} =~ /$service/  ){
                        $Log->comment("Found process $params{service} in spmctl");
                        $response =  $hash->{$hvalue};
                    }
                }
            }
        }
    }
    Tharn::sleep(20);
    $Log->comment("Return spmctl response: $response");
    return  $response; 

}

############################################################################
# TestCase: pre_init_check
# Parameters: None
# Returns: $invoked_asup
# Purpose: Check and delete any alerts present in the system before running the test cases.
#############################################################################
sub pre_init_check{
        my $state = 0;

        #Some times there will be alerts only but the system health can be in OK state. Hence Deleteing the alerts and then checking the system health
        if(system_health_status_is_bad()){
                $Log->warn("system health status show did not return ok");
                # Enabling all the adapters
                restore_sas_adapter_all();
                $Log->comment("Enabled all the adapters on all the nodes by default. \n");
        }
        $Log->comment(" Trying to delete all existing alerts in the system \n");
        if(delete_all_alerts()){
                        $state = 1;
                        $Log->comment("pre_init_check: Some issues while deleting the alerts \n");
        }
        if(shm_schm_node() && SHM::_get_version !~ /7M/i){
            $Log->comment("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% \n");
            $Log->comment("Ensure that SHM and SCHM runs on the same node. .. this can cause intermitent failures in the test cases \n");
            $Log->comment("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% \n");
        }
        return $state;
}

############################################################################
# TestCase: restore_sas_adapter_all
# Parameters: None
# Returns: $invoked_asup
# Purpose: Check and delete any alerts present in the system before running the test cases.
#############################################################################
sub restore_sas_adapter_all {
        @Nodes = get_nodes();
    # Enabling all the adapters
	if ( _check_model() =~ /SIM/i){
		return;
	}		
        $Log->comment (" restore_sas_adapter_all : By default enabling all the adapters \n");
        foreach my $tmp_node(@Nodes) {
                        my @adapters = get_node_connectivity_hwvalues(node=>$tmp_node,hwtype=>adapter);
                        foreach my $adapters (@adapters) {
                                        my $tmp1=$tmp_node;
                                        $Log->comment (" Going to enable the adapter on the node $tmp1 , $adapters \n");
                                        my $cmd = 'sasadmin adapter_online '.$adapters;
					my $output;
					if ( SHM::_get_version() !~ /7M/i){
						my $tmp_node = $Nodes[0];
						my $output = NACL::C::Node->run( command_interface => $tmp_node,
                                                         node              => $tmp_node->name(),
                                                         command           => $cmd,);
					}else{
						$output = run_dblade_command(command=>$cmd);
					}
                        }
                        $Log->comment (" Enabling the adapter showed the result = $output \n");
        }
        # after restoring the adapters I am manually deleting the alerts.
	delete_all_alerts();
        Tharn::sleep 30;
}

############################################################################
# TestCase: delete_all_alerts
# Parameters: None
# Returns: $invoked_asup
# Purpose: Delete any alerts present in the system before running the test cases.
#############################################################################
sub delete_all_alerts {
	delete_health_alert(monitor=>"*",alert_id=>"*",alerting_resource=>"*",node=>"*");
	asup_delete_alert(node=>"*",monitor=>"*",alert_id=>"*",alerting_resource=>"*");
    return 0;
}
############################################################################
# TestCase: shm_schm_node
# Parameters: None
# Returns: $invoked_asup
# Purpose: Validate SCHM and SHM processes are running on the same node
#############################################################################
sub shm_schm_node{
    my $schm_node = get_SCHM_node();
    my $shm_node = get_SHM_node();
    my $state=0;
    if ( $shm_node ne $schm_node){
        $state =1;
    }
    return $state;
}

############################################################################
# TestCase: debug_mtrace_clear
# Parameters: None
# Returns: $invoked_asup
# Purpose: Clears the mtrace buffer 
#############################################################################
sub debug_mtrace_clear {
	my %params = (
			buffer=>undef,
			node => undef,
			process =>undef,
            @_,
    );  
	  NACL::C::DebugMtraceBuffer->clear(
                                command_interface => $Nodes[0],
	                            'buffer' => $params{buffer},
	                            'node' => $params{node},
	                            'process' => $params{process},
                                );
								
}

############################################################################
# TestCase: debug_mtrace_clear
# Parameters: None
# Returns: $invoked_asup
# Purpose: Clears the mtrace buffer 
############################################################################# 
sub debug_mtrace_find {
	my %params = (
			buffer=>undef,
			node => undef,
			process =>undef,
			filename => undef,
			message => undef,
            @_,
    );  
    my $ref = \%params;
    my $state;
	try{
			my $DebugMtraceBuffer_obj  =  NACL::CS::DebugMtraceLog->fetch(
																								command_interface=>$Nodes[0],
																								filter=>$ref, );
			#my $alert_state = $DebugMtraceBuffer_obj->state();
			$state=$DebugMtraceBuffer_obj->message();
		}
		catch NACL::Exceptions::NoElementsFound with{
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::InvalidParamValueException with {
			my $exception_object = shift;
			my $resp = $exception_object->get_response_object();
			$state=$resp->get_raw_output;		
		};
		$Log->comment("System Health Alert Show Filters:  ".Dumper($DebugMtraceBuffer_obj));
		return $state;
}

###########################################################################
# TestCase: reboot_node
# Parameters: None
# Returns: result of execution of system menu
# Purpose: reboot the node using the apiset
#############################################################################
#7M:  use boot qw(reboot reboot_to boot transit_to);             reboot_filer(node => $Nodes[0]);
sub reboot_node {
    my %params = (
            node=>"local",
            exec_node=>undef,
            @_
    );    
    my $ret;
    my $had_error=0;
	my $auto_giveback="true";
	#error on vsims trying to set the option: cf.giveback.auto.enable
	if ( _check_model() =~ /SIM/i){
		$auto_giveback="false";
	}else{
		$Log->comment("Enable auto-giveback on all nodes...");
	}
    
     NACL::C::StorageFailover->modify(command_interface=>
                                        NACL::C::Node->new(name=>$params{exec_node}),
                                        node=>'*',
                                        'auto-giveback'=>$auto_giveback);

     $Log->comment("Reboot the filer $params{node} ...");
     try {
          my $Transit_obj = NACL::Transit->new(name => $params{node});
          $Transit_obj->reboot();
     } catch NACL::Transit::Exceptions::TransitException with {
          my $exception_object = shift;
          $Log->comment("Failed to bring the filer back up.");
          $Log->comment("Error is: " . $exception_object->text());
          $had_error = 1;
     };

     if ($had_error) {
         return $TCD::FAIL;
     }
        
}

#*******************************************************************************************
#Function Name  :refresh_handles
#Paremeters     :NA
#Return Type    :NA
#Description    :Refreshes the handle can be used after reboot
#*******************************************************************************************
sub refresh_handles()
{
    if ( SHM::_get_version() =~ /7M/i ) {
	    $Log->step("I am running on 7 mode \n");
	    my @node_names  = $Server->servers();
    } else {
	    $Log->step("I am running on C mode \n");
# FWR
        map { $_->refresh_command_interface() } @Nodes;
	}
}

#*******************************************************************************************
#Function Name  :fetch_StorageFailover
#Paremeters     :NA
#Return Type    :NA
#Description    :fetch SFO object
#*******************************************************************************************

sub fetch_StorageFailover{
  my %params = (
            node=>undef,
            @_
    );
	my $SFO = NACL::CS::StorageFailover->fetch(command_interface => NACL::C::Node->new(name=>$params{node}));
	return $SFO;
}

############################################################################
# TestCase: network_port_show
# Parameters: None
# Returns: $invoked_asup
# Purpose: Show network ports on particular node
#############################################################################
 sub network_port_show {
	my %params = (
				node => undef,
				role => undef,
				@_,
		);
		my $ref = \%params;
		my @eth_ports;
        my $nacl_node = _get_nacl_node( $params{node} );
		my $NGSH = $nacl_node->apiset(
			interface => "CLI",
            );
  
		my $resp = $NGSH->network_port_show(node=> $params{node},role=> $params{role});
		my @ports = @{$resp->get_parsed_output()};
		foreach my $port(@ports) {
			if( $port->{role} eq $params{role} )
			{
				$Log->comment( "value is ->>>>>>>>> ". $port->{port});
				push ( @eth_ports , $port->{port});
			}
		}		
		return @eth_ports;
}

############################################################################
# TestCase: network_port_modify
# Parameters: None
# Returns: $invoked_asup
# Purpose: Enable or disable selected network port
#############################################################################
sub network_port_modify{
	my %params = (
				node => undef,
				port => undef,
				enable => undef,
				@_,
		);
		my $state="";
		$Log->comment(" \n");
		try {
				my $NetworkPort_object = NACL::C::NetworkPort->modify(command_interface => NACL::C::Node->new(name=>$params{node}),
																node =>$params{node},
																port=> $params{port},
																"up-admin"=>$params{enable},
															);
					$state = $params{node};
		}
		catch NACL::Exceptions::OperationFailed with {
			     my $exception_object = shift;
				$state=$exception_object->text(); 
		}
		catch NACL::Exceptions::NoElementsFound with{
           my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		catch NACL::APISet::Exceptions::CommandFailedException with {
			my $exception_object = shift;
			$state=$exception_object->text(); 
		}
		otherwise{ 
				my $exception_object = shift;
				$state=$exception_object->text();
		};
		
		return $state;	
}

############################################################################
# TestCase: parse_log
# Parameters: None
# Returns: 0 if the string is not found, 1 if string is found
# Purpose: Check for a string within the logs
#############################################################################

sub parse_log {
    my %params = (
            node=>$Nodes[0],
            log => undef,
            message=>undef,
            @_
    );
    my $result=0;
    if ( !$params{log} && !$params{message} ){
        $Log->comment("Log file and string to match not specified, exiting");
        return 0;
    }
    my $result=0;

    my $nacl_node = _get_nacl_node( $params{node} );
	my $Api_Set_ObjS = $nacl_node->apiset(
			interface => "CLI",
		    set       => "Systemshell",
            );
    my $logfile=$Api_Set_ObjS->execute_raw_command(command=>"grep \"$params{message}\" /mroot/etc/log/mlog/$params{log}");
    my @values=split(/\n/,$logfile);
    $Log->comment("Values from grepping log $params{log}:  @values");
    if (scalar(@values) >= 1) {
	$Log->comment("Found message $params{message} in log file $params{log}");    
	$result=1;
    }        
    return $result;
}

############################################################################
# TestCase: write_hm_file
# Parameters: File, Node, Remove white space 
# Returns: 0 if the file is not written, 1 if file is written
# Purpose: Write a new hm file to /mroot/etc/hm
#############################################################################

sub write_hm_file {
    my %params = (
            node=>undef,
            file => undef,
            remove_ws=>"true",
            @_
    );
    my $result=0;
    if ( !$params{node} && !$params{file} ){
        $Log->comment("HM file and node not specified, exiting");
        return 0;
    }
    my $result=0;
  
    foreach my $node (@Nodes) {
	    my $Api_Set_ObjS = $node->apiset(
			    interface => "CLI",
		        set       => "Systemshell",
                );
        $Api_Set_ObjS->execute_raw_command(command=>"sudo mkdir $HM_DIR");
        my $custom_file="nchm_custom_policy.hmi";
    	if ( $params{file} =~ /ehm/i ){
    		$custom_file="ehm_custom_policy.hmi";
    	}
    	$Api_Set_ObjS->execute_raw_command(command=>"sudo rm $HM_DIR/$custom_file");
    	if ( SHM::_get_version() =~ /7M/i ){
    	    open(FILE, $params{file}) or die("Unable to open file");
    	    my @data = <FILE>;
    		close(FILE);
    	    foreach my $line (@data){
    			if ($params{remove_ws} eq "true"){
    					$line=~ s/^\s+|\s+$//g;
    					$Api_Set_ObjS->execute_raw_command(command=>"echo $line >> $HM_DIR/$custom_file");
    			}else{
    				$Api_Set_ObjS->execute_raw_command(command=>"echo $line >> $HM_DIR/$custom_file");	
    			}
    		}		
    	}else{
    		_scp_hm_file(node=>$node,file=>$params{file});
    	}
    	$Api_Set_ObjS->execute_raw_command(command=>"sudo mv $HM_DIR/*.hmi $HM_DIR/$custom_file");
    	if ($Api_Set_ObjS->execute_raw_command(command=>"file $HM_DIR/$custom_file") =~ /ASCII/i){
    		$result=1;
    	}else {
    	    return 0;
    	}
    }
    return $result;
}

############################################################################
# TestCase: rm_hm_file
# Parameters: Node
# Returns: None
# Purpose: Remove all the custom hm policy files
#############################################################################

sub rm_hm_file {
    my %params = (
            node=>undef,
            @_
    );
    my $result=0;
    if ( !$params{node} ){
        $Log->comment("Node not specified, exiting");
        return 0;
    }
    foreach my $node (@Nodes) {
            my $nodeN = $node->name();
            if ($nodeN eq $params{node}) {
        	    my $Api_Set_ObjS = $node->apiset(
			    interface => "CLI",
		        set       => "Systemshell",
                );
                $Api_Set_ObjS->execute_raw_command(command=>"sudo rm -rf $HM_DIR/*");
            }
    }
    return 1;
}

sub _scp_hm_file {
    my %params = (
            node=>undef,
			file=>undef,
            @_
    );
    my @clients = NACL::C::Client->find();
    my $unix_client;
    foreach my $client (@clients) {
        if ( $client->hostrec()->hosttype() =~ /unix/i ) {
            $unix_client = $client;
            last;
        }
    }   
    my $target_node = $params{node};
    my $scp = NACL::MTask::SCP->new(
                    source_command_interface => $unix_client ,
                    target_command_interface => $target_node    );
    my $files = [$params{file}];        
    $Log->comment (" Going to start the copy \n");
    $scp->copy_from_source_to_target(
            'files_to_copy'         => $files,
            set_directory_on_target => '/mroot/etc/hm/', 
        );
    $Log->comment(" Complete the copy of the file \n");
    
}
1
