#!/bin/sh /usr/software/bin/perl 

#
#  Class that models SMF field
#
package Field;

use strict;
#no strict 'refs';
use FindBin;
use lib "$FindBin::Bin/../../lib";
use TracePkg;

my $t = TracePkg->new();

sub new {

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    #  a hash.
    my ($self, %arg) = @_;

    # Fill the member vars for this instance
    my $object = {
	_name       => $arg{name} || "???" ,
	_desc       => $arg{desc} || "???" ,
	_type       => $arg{type} || "???" ,
	_qual       => $arg{qual} || "???" ,
	_priv       => $arg{priv} || "???",
	_required   => 0,
	_hidden     => $arg{hidden},
	_show       => undef,
	_createCmd  => $arg{createCmd},
	_preCValue  => undef,
	_preCAttr   => undef,
	_postCValue => undef,
	_postCAttr  => undef,
	};

    # if the _qual is key or write the field is required for
    # new commands

    if ( $object->{_qual} =~ /(key|write|create|in)/ ) {
	$object->{_required} = 1;
    }

    # if the name begins with a !, the field isn't required
    if ( $object->{_name} =~ s/\!// ) {
	$object->{_required} = 0;
    }

    # This tags the hash pointed to by $object as a 
    #  type of Field
    bless $object, $self;

    $t->debug(msg=>"Created Field: " . $object->{_name} . "\n");

    # And give the caller the referece to the bless object.
    return $object;
}


sub getName {
    my ($self) = @_;
    return $self->{_name};
}


sub getDesc {
    my ($self) = @_;
    return $self->{_desc};
}


sub getCreateCmd {
    my ($self, $int) = @_;
    return $self->{_createCmd};
}


sub setCreateCmd {
    my ($self, $createCmd) = @_;
    $self->{_createCmd} = $createCmd;
}


sub getType {
    my ($self) = @_;
    return $self->{_type};
}


sub getQualifier {
    my ($self) = @_;
    return $self->{_qual};
}


sub setQualifier {
    my ($self, %arg) = @_;
    $self->{_qual} = $arg{qual};
}


# Update the privilege for this Field
sub setPriv {
    my ($self, %arg) = @_;
    $self->{_priv} = $arg{priv};
}


sub getPriv {
    my ($self) = @_;
    return $self->{_priv};
}


sub setPreC {
    my ($self, %arg) = @_;
    $self->{_preCValue} = $arg{value};
    $self->{_preCAttr} = $arg{attr};
}


sub getPreC {
    my ($self) = @_;

    # If the condition specifically exists for this
    # field, return it.
    if ( exists $self->{_preCValue} &&
	 exists $self->{_preCAttr} ) {
	return ($self->{_preCValue}, 
		$self->{_preCAttr});
    } else {
	return undef;
    }
}
	

sub setPostC {
    my ($self, %arg) = @_;
    $self->{_postCValue} = $arg{value};
    $self->{_postCAttr} = $arg{attr};
}


sub getPostC {
    my ($self) = @_;
    if ( exists $self->{_postCValue} &&
	 exists $self->{_postCAttr} ) {
	return ($self->{_postCValue}, 
		$self->{_postCAttr});
    } else {
	return undef;
    }
}


sub setGlobal {
    my ($self, %arg) = @_;
    $self->{_globalValue} = $arg{value};
    $self->{_globalAttr} = $arg{attr};
}


sub getGlobal {
    my ($self) = @_;
    if ( exists $self->{_globalValue} &&
	 exists $self->{_globalAttr} ) {
	return ($self->{_globalValue}, 
		$self->{_globalAttr});
    } else {
	return undef;
    }
}

sub setShowField {
    my ($self) = @_;
    $self->{_show} = 1;
}


sub isShowField {
    my ($self) = @_;
    return $self->{_show};
}


sub isRequired {
    my ($self) = @_;
    return $self->{_required};
}


sub setRequired {
    my ($self, %arg) = @_;
    $self->{_required} = $arg{required};
}


sub isHidden {
    my ($self) = @_;
    return $self->{_hidden};
}


sub isKey {
    my ($self) = @_;
    if ( $self->{_qual} =~ /key/ ) {
	$t->debug(msg=>"Field: $self->{_name} is a key");
	return 1;
    }
	$t->debug(msg=>"Field: $self->{_name} is a not key");
    return undef;
}

sub printMe {
    my ($self) = @_;

    print "\t\t$self->{_name}, $self->{_desc}, $self->{_type}, ";
    print "$self->{_qual}, ";
    print $self->{_required} ? "required, " : "not required, ";
    print "$self->{_priv}, ", $self->{_createCmd};
    print "\n";
}


1;
