#!/usr/local/bin/perl 

#
#  Generate command structures from SMF tables
#
package GenModel;

use strict;

# Following use pulls in the 'test/lib/' directory
use lib "/x/eng/systest/p4/ontap/main/test/lib";
use lib "/usr/local/test/share/nate/latest/lib";
use FindBin;
use lib "$FindBin::Bin/../../lib";
#use lib "$FindBin::Bin/lib";
use Data::Dumper;

use TracePkg;

use SMF::Numeric;
use SMF::String;
use SMF::Enum;
use SMF::Pattern;

use SMF::Table;
use SMF::View;
use SMF::Field;
use SMF::Method;
use SMF::Command;
use SMF::Type;

use Getopt::Long;

our %typeDefHash;
our %typeObjHash;
our %allCommands;
our %allTables;

my ($t, $traceD);
my %options = (
   "smf_type" => "$FindBin::Bin/smf.types",
   );

#GetOptions ( "traceD=s" => \$traceD );
#if ( $traceD ) {
#    $t = TracePkg->new( traceD => $traceD );
#} else {
#    $t = TracePkg->new( );
#}
$t = TracePkg->new( );

# If called directly, parse the passed file name and print it.
if ( ! caller ) {

    my ($verbose, $stub, $dumpCmd, $dumpType, $dumpField, $dumpAll);
    my ($cmdHash, $cmd, $key, $fieldKey, $dumpTable);
    
    # Will want to put this in a more 'wellknown' place.
    my $baseTypeFile = $options{smf_type};

    GetOptions ( verbose   => \$verbose,
		 "stub=s"  => \$stub,
		 dumpCmd   => \$dumpCmd,
		 dumpType  => \$dumpType,
		 dumpField  => \$dumpField,
		 "baseTypeFile=s" => \$baseTypeFile,
		 "dumpTable=s"  => \$dumpTable
		 ) || die("GetOptions error");
    
    if (@ARGV == 0) {
	die "Calling GenModule.pm driectly requires smf input\n";
    }
    if ($dumpCmd || $dumpType || $dumpField || $dumpTable) {
	$dumpAll = 0;
    } else {
	$dumpAll = 1;
    }

    my $fileName;
    my ($tableKey, $table, $cmd);

    # Before we start generating the model, we will read in and process
    # the smf base types.  
    genBaseTypes( fileName => $baseTypeFile );
    
    while ( $fileName = shift ) {	
	genModel( fileName => $fileName );
    }
    
    genTypeObjects();

    if ( $dumpAll )  {
	my @keyNames = keys %allTables;
	$t->debug(msg=>"All table keys: @keyNames\n");
	for $key (keys %allTables) {
	    $t->debug(msg=>"Dumping table: $key\n");
	    $table = $allTables{$key};
	    $table->printMe();
	}
    }
    if ( $dumpCmd ) {
	# Dump all the commands
	for $key (keys %allCommands) {
	    print $key, "\n";
	    #$allCommands{$key}->printMe();
	    #print $allCommands{$key}->getCmdString(), "\n";
	}
    } 
    if ( $dumpType ) {
	# Dump all the Types
	for $key (keys %typeObjHash) {
	    #print $key, "\n";
	    $typeObjHash{$key}->printMe();
	    #print $allCommands{$key}->getCmdString(), "\n";
	}
    } 
    if ( $dumpField ) {
	# Dump all the Fields
	my ($allFields, $fieldKey);

	for $key (keys %allTables) {
	    $allFields = $allTables{$key}->getFields();
	    for $fieldKey (keys %{ $allFields }) {
		print $allFields->{$fieldKey}->getName(), ":",
		      $allFields->{$fieldKey}->getType(), ":",
		      $allFields->{$fieldKey}->getQualifier(), ":",
		      $allTables{$key}->getName(),
		      "\n";
	    }
	}
    } 
    if ( $dumpTable ) {
	if ( exists $allTables{$dumpTable} ) {
	    $table = $allTables{$dumpTable};
	    $table->printMe();
	} else {
	    print "Table $dumpTable not found.\n";
	}
    } 
}	

# 
# Populate the type hash with the base smf types.
#  
sub genBaseTypes {
    
    my %arg = @_;
    
    die "fileName must be passed in" if (!($arg{fileName}));

    $t->entry(msg=>"Processing type file: $arg{fileName}\n");

    open (basefH, "< $arg{fileName}") || die "Can't open $arg{fileName}";
    
    my ($line, $str);
    while ($line = <basefH> ) {
	$str .= $line;
    }

    my $rawData = eval $str;
    
    # Relocate the items stored in rawData to typeDefHash
    for my $item ( keys %{ $rawData } ) {
	for my $field ( keys %{ $rawData->{$item} } ) {
	    $typeDefHash{$item}{$field} = $rawData->{$item}{$field};
	}
    }
    $t->exit(msg=>"Processed type file: $arg{fileName}\n");
}


#
# Input parms
# an SMF file to parse
#
sub genModel {
    my %arg = @_;

    my $fileName = $arg{fileName};

    $t->entry(msg=>"Working on $fileName\n");

    # May want to skip unrelated file types.

    my ($table);

    if ( !($fileName) ) {
	die("SMF input file required");
    }

    #print "Processing $fileName\n";
    open(fhSmf, "< $fileName") || die("Can't open $fileName: $!");

    while ( <fhSmf> ) {
	if ( skipBlank() ) { next; }
	
	if ( /^\s*enum/ ) {
	    #print "Found a enum in:\n";
	    #print "$_\n";

	    # Process the enum section
	    procEnum ( "fhSmf" );
	    next;
	}
	if ( /^\s*table/ ) {
	    
	    # Process the table section
	    $table = procTable( fileHandle => "fhSmf");

	    if ( $table ) {
		$allTables{ $table->getName() } = $table;
	    }

	    #$table->printMe();
	    next;
	}
	# action tables are much like regular tables.
	if ( /^\s*action/ ) {
	    
	    # Process the table section
	    $table = procTable( fileHandle => "fhSmf",
				type       => "ACTION",
				);

	    if ( $table ) {
		$allTables{ $table->getName() } = $table;
	    }

	    #$table->printMe();
	    next;
	}
	# view tables made up of other tables and use those tables
	# to define the keys and fields of this table.
	if ( /^\s*view/ ) {
	    
	    # Process the table section
	    $table = procTable( fileHandle => "fhSmf",
				type       => "VIEW",
				);

	    if ( $table ) {
		$allTables{ $table->getName() } = $table;
	    }

	    #$table->printMe();
	    next;
	}

	# Look for a section we don't understand yet
	if ( /^\s*snmp .*\{/ ) {
	    $t->debug(msg=>"Found a non-supported section snmp\n");
	    skipSection( "fhSmf" );
	    next;
	}

	# Developers will describe smf types inside of their smf 
	# type header and smf table files.  However the definition
	# must follow the following semantics or it will be ignored.
	#    
	# SMF_TYPE_NAME = NAME
	# SMF_TYPE_PATTERN = MIN:(Regular-Expression) \
        #                    MAX:(Regular-Expression) \
	#                    <SAMPLE: String>
	# SAMPLE is optional value, but it must be the last item listed.
	# 
	# The name will represent the "type" that is listed in the 
	# smf table.  With the initial implementation, the comments 
	# must be listed in this order.  
	#  TODOs:
	#   1.  Ability to handle case where pattern stretches over one line.
	# 
	if ( /.*\s*SMF_TYPE_NAME\s*=\s*(.*)/ ) {
	    my $name = $1;
	    $t->debug(msg=>"Found SMF_TYPE: $name\n");

	    while (<fhSmf>) {
		if ( skipBlank() ) { next; }		
		if ( /.*\s*SMF_TYPE_PATTERN\s*=\s*(.*)/ ) {
		    # We found a pattern defined in the file.
		    # First store the pattern.
		    my $pattern = $1;

		    # The following loop will handle collapsing patterns
		    # that stretch over multiple lines.  Each line that is
		    # continued on the subsequent line must end with a \
		    while ($pattern =~ /(.*)\\$/) {
			# re-assign pattern to be everything minus the 
			# continuation "\".
			$pattern = $1;
			# Read in the next line.  
			my $line = <fhSmf>;
			# First remove the new-line character.
			chomp $line;
			# We also need to trim off leading comment lines
			# "//".  There may be a better way to do this..
			if ($line =~ /^\/\/(.*)/) {
			    $line = $1;
			}
			# Add it to the end of the pattern string
			$pattern .= $line;
		    }
		    # 
		    # Extract the max and min patterns from the string.
		    if (!($pattern =~/\s*MIN:(\S*)\s+MAX:(.*)/)) {
			die "We didn't find the correct pattern in $pattern\n";
		    }
		    my $min_pat = $1;		    
		    my $max_pat = $2;
		    my $sample = "";

		    # The second string returned, may or may not include 
		    # a sample description.  If it does, pull it out and 
		    # store it for later.
		    if ($max_pat =~/\s*(\S*)\s+SAMPLE:(\S*)/) {
			$max_pat = $1;
			$sample = $2;
		    }
		    $t->debug(msg=>"$name pattern: $pattern\n");

		    # Make sure that type doesn't already exist:
		    if (exists $typeDefHash{$name}) {
			# It could exist, if the type is Default
			if ($typeDefHash{$name}{type} ne "DEFAULT") {
			    die "Already found $name";
			}
		    }
		    $typeDefHash{$name}{name} = $name;
		    $typeDefHash{$name}{min_pat} = $min_pat;
		    $typeDefHash{$name}{max_pat} = $max_pat;
		    $typeDefHash{$name}{sample} = $sample;
		    $typeDefHash{$name}{type} = "pattern";
		}
		else {
		    $t->debug(msg=>"Pattern not found for $name, skipping\n");
		}
		last;
	    }
	}	    
    }
    $t->exit(msg=>"Processed $fileName\n");
}


sub getModel {
    return %allTables;
}

sub genTypeObjects() {

    $t->entry(msg=>"Creating Type objects\n");
    
    foreach my $index (keys %typeDefHash ) {
	my $isList = 0;
	
	# If we are dealing with a list, we need to populate our 
	# hash entry with the information from the 'baseType' stored in the
	# list. 
	if ( $typeDefHash{$index}{isList} ) {
	    $isList = 1;
	    my $baseType = $typeDefHash{$index}{type};

	    # We may need to redefine the hash's info for this list type.
	    if ( exists $typeDefHash{$baseType}  ) {
		# The basetype for this list is defined in another hash 
		# entry, we need to copy that info over to this entry.
		# May choose to do it an cleaner way down the road.
		$typeDefHash{$index}{type} = $typeDefHash{$baseType}{type};
		$typeDefHash{$index}{desc} = $typeDefHash{$baseType}{desc};
		$typeDefHash{$index}{min_pat} =
		                 $typeDefHash{$baseType}{min_pat};
		$typeDefHash{$index}{max_pat} =
		                 $typeDefHash{$baseType}{max_pat};
		$typeDefHash{$index}{sample} =
     		                 $typeDefHash{$baseType}{sample};
		$typeDefHash{$index}{values} = $typeDefHash{$baseType}{values};
		# For the case where we are dealing with a list of ranges
		# we don't want to automatically over-ride these.
		if ( ! exists $typeDefHash{$index}{min} ) {
		    $typeDefHash{$index}{min} = $typeDefHash{$baseType}{min};
		    $typeDefHash{$index}{max} = $typeDefHash{$baseType}{max};
		}
	    } elsif ( $typeDefHash{$index}{type} ne "numeric" &&
		      $typeDefHash{$index}{type} ne "string"  &&
		      $typeDefHash{$index}{type} ne "enum"    &&
		      $typeDefHash{$index}{type} ne "pattern" ) {
		# If our base type is not one of the known types, we will
		# need to set it to the default type.
		$typeDefHash{$index}{type} = "DEFAULT";
	    }
	}
	
	if ( $typeDefHash{$index}{type} eq "pattern" ) {
	    $t->debug(msg=>"Creating Type $index of type $typeDefHash{$index}{type}\n");
	    $typeObjHash{$index} = Pattern->new( name => $index,
						 desc => "unknown",
						 min_pat => $typeDefHash{$index}{min_pat},
						 max_pat => $typeDefHash{$index}{max_pat},
						 sample => $typeDefHash{$index}{sample},
						 isList => $isList
						 );
	}
	elsif ($typeDefHash{$index}{type} eq "string" ) {
	    $t->debug(msg=>"Creating Type $index of type $typeDefHash{$index}{type}\n");
	    $typeObjHash{$index} = String->new( name => $index,
						desc => "unknown",
						min => $typeDefHash{$index}{min},
						max => $typeDefHash{$index}{max},
						isList => $isList
						);
	}
	elsif ($typeDefHash{$index}{type} eq "numeric" ) {
	    $t->debug(msg=>"Creating Type $index of type $typeDefHash{$index}{type}\n");
	    $typeObjHash{$index} = Numeric->new(name => $index,
						desc => "unknown",
						min => $typeDefHash{$index}{min},
						max => $typeDefHash{$index}{max},
						isList => $isList
						);
	}
	elsif ($typeDefHash{$index}{type} eq "enum" ) {
	    $t->debug(msg=>"Creating Type $index of type $typeDefHash{$index}{type}\n");
	    $typeObjHash{$index} = Enum->new(name => $index,
					     desc => $typeDefHash{$index}{desc},
					     values => \@{$typeDefHash{$index}{values}},
					     isList => $isList
					     );
	}
	elsif ($typeDefHash{$index}{type} eq "DEFAULT" ) {
	    $t->debug(msg=>"Creating Type $index of type $typeDefHash{$index}{type}\n");
	    $typeObjHash{$index} = String->new( name=> $index,
						desc => "unknown",
						min => 1,
						max => 255,
						sample => $index,
						isList => $isList
						);
	}
	else {
	    # Fail since we encountered a type we don't understand.
	    die "We encountered a type we did not understand($typeDefHash{$index}{type})";
	}
		
    }
    $t->exit(msg=>"Type objects created\n");
}				 

#
# procEnum
# Process a enum section.
#
sub procEnum {
    
    my $fh = shift;
    my ($name, $desc, @items);

    #enum enum1 "start|stop|resume|suspend|status" {
    if ( /^\s*enum\s+(.*?)\s*\"(.*)\"\s*\{\s*$/ ) {
	$name = $1;
	$desc = $2;
	#print "Found enum: $name - $desc\n";
    } elsif ( /^\s*enum\s+(\w+)/ ) {
	$name = $1;
    }
    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for the end of the enum section
	if ( /\}\s*$/ ) {
	    # Store the type information
	    $typeDefHash{$name}{name} = $name;
	    $typeDefHash{$name}{values} = \@items;
	    $typeDefHash{$name}{type} = "enum";
	    $typeDefHash{$name}{desc} = $desc;
	    return;
	} 
	#print "$_\n";
	# Get the items (should be the description)
	#  start=1   "Start Scrubbing"
	if ( /^\s*(\S+)/ ) {
	    my $value = $1;
	    $value =~ s/=\d+//;
	    push @items, $value;
	    $t->debug(msg=>"Found value: $value for enum: $name");
	    next;
	}	    
    }
}


#
# procTable
# Process a table section.  Namely, process the contained "attributes",
# "fields", "methods", and commands sections.
# arg1 - the file handle of the input SMF file
# arg2 - the current line (contains "table" keyword
#
sub procTable {
    
    my %arg = @_;
    my $fh = $arg{fileHandle};
    my $type = $arg{type};
    my ($tableName, $tableDesc, $tableAttrs, $table, $altTableName);

    # This table line is assumed to be of the format:
    # table <tableName> "table description" { table attributes }
    # Maybe an action table, or regular table
    if ( /^\s*table\s+(.*?)\s*\"(.*)\"\s*\{\s*(.*)\s*\}/ ) {
	$tableName  = $1;
	$tableDesc  = $2;
	$tableAttrs = $3;
	$t->debug(msg=>"table tableName: $1, tableDesc: $2, tableAttrs: $3");
    } elsif ( /^\s*action\s+(.*?)\s*\"(.*)\"\s*\{\s*(.*)\s*\}/ ) {
	$tableName  = $1;
	$tableDesc  = $2;
	$tableAttrs = $3;
	$t->debug(msg=>"action tableName: $1, tableDesc: $2, tableAttrs: $3");
    } elsif ( /^\s*view\s+(.*?)\s*\"(.*)\"\s*\{\s*(.*)\s*\}/ ) {
	$tableName  = $1;
	$tableDesc  = $2;
	$tableAttrs = $3;
	$t->debug(msg=>"view tableName: $1, tableDesc: $2, tableAttrs: $3");
    } elsif ( /^\s*(table|action|view)\s+(.*?)\s*\"(.*)\"/ ) {
	$tableName  = $1;
	$tableDesc  = $2;
	while ( <$fh> ) {
	    /\s*\{\s*(.*)\s*\}/;
	    $tableAttrs = $1;
	    last;
	}
	$t->debug(msg=>"table, extended attrs tableName: $tableName, tableDesc: $tableDesc, tableAttrs: $tableAttrs");
    }

    if ( !$tableName ) {
	$t->debug(msg=>"tableName: $tableName, tableDesc: $tableDesc, tableAttrs: $tableAttrs");
	$t->fatal(msg=>"Not able to parse table info from line:\n$_\n");
    }
    $t->entry(msg=>"Creating $type table: $tableName, $tableDesc, $tableAttrs\n");

    # Create a Table object
    if ( $type eq "VIEW" ) {
	$table = 
	    View->new( name   => $tableName,
		       desc   => $tableDesc,
		       attrs  => $tableAttrs,
		       type   => $type,
		       );
    } else { 
	$table = 
	    Table->new( name   => $tableName,
			desc   => $tableDesc,
			attrs  => $tableAttrs,
			type   => $type,
			);
    }
    # Prep the pivilege defaults
    # admin for tables, fields/commands/methods default 
    # to the same priv as table (unless specified)
    $table->setPriv( default => "admin" );
    
    while ( <$fh> ) {
	#print $_, "\n";;

	if ( skipBlankNComments() ) { next; }

	# Skip some sections for now
	if ( /^\s*dist_keys/ ) {
	    next;
	}

	# Skip some sections for now
	if ( /^\s*zephyr/ ) {
	    skipSection( $fh );
	    next;
	}

	# alternateKeys have the format:
	#     alternateKeys <newtableName> {
        #          newKey1 [newKey2 ...]
	#     }
	if ( /^\s*alternateKeys\s+(\S+)\s*/ ) {
	    $t->debug(msg=>"Found an alternateKeys section\n");
	    $altTableName  = $1;
	    
	    procAltKeys( fileHandle => $fh,
			 tableObj   => $table,
			 altTableName => $altTableName,
			 );
	    next;
	} 

	# Look for and process the views's keys
	if ( /^\s*keys from/ ) {

	    procKeysFrom( fileHandle => $fh,
			  tableObj   => $table
			  );
	    next;
	}

	# Look for and process the views's values
	if ( /^\s*values\s*\{/ ) {

	    procValues( fileHandle => $fh,
			tableObj   => $table
			);
	    next;
	}

	# Look for and process the views's fields
	if ( /^\s*keys for/ ) {

	    $t->debug(msg=>"Skipping 'keys for' <table key> pairs for: " . 
		      $table->getName());
	    skipSection( $fh );
	    next;
	}

	# Look for and process the table's fields
	if ( /^\s*fields/ ) {

	    procFields( fileHandle => $fh,
			tableObj   => $table
			);
	    next;
	}

	# Look for and process the table's methods
	if ( /^\s*methods/ ) {
	    $t->debug(msg=>"Found a methods section in:\n" .
		      "$_\n");
	    procMethods ( fileHandle => $fh,
			  tableObj   => $table
			  );
	    next;
	}

	# Look for and process the table's commands
	if ( /^\s*command/ ) {
	    $t->debug(msg=>"Found a command section in the table.\n" .
		      "$_\n" );
	    procCommand ( fileHandle => $fh,
			  table      => $table
			  );
	    next;
	}

	# Look for and process the field's privileges
	if ( /^\s*privilege/ ) {
	    $t->debug(msg=>"Found a privilege section in the table.\n" .
		      "$_\n" );
	    procFieldPriv ( fileHandle => $fh, 
			    tableObj   => $table 
			    );
	    next;
	}

	# Look for a section we don't understand yet
	if ( /^\s*(.*?)\s*\{/ ) {
	    $t->debug(msg=>"Found a non-supported section $1\n");
	    skipSection( $fh );
	    next;
	}

	# Look for the end of this table section
	if ( /^\s*\}/ ) {
	    if ( $table->getCmdDir() ) {
		# If there was a "command" section, we'll have commands
		# And lastly, if there are no keys for this table, 
		# then remove the 'new' and 'delete' commands
		if ( $table->getKeys() ) {
		    $t->exit(msg=>"Table created, with keys: $tableName\n");    
		} else {
		    $t->exit(msg=>"Table created, without keys: $tableName\n");
		    my @cmds = qw(new delete);
		    removeCmds( cmds => [@cmds], table => $table);
		}
		return $table;
	    } else {
		# No command section found, no UI component
		my @cmds = qw(new delete modify);
		removeCmds( cmds => [@cmds], table => $table);
		$t->exit(msg=>"Table created, with no commands: $tableName\n");
		# Hmm, I guess I could clean up here.....
		return $table;
	    }
	} 

    }
}


# Create a new table that's an alias to (actually a duplicate of ) an existing table, 
# but has different keys (alternateKeys table)
sub procAltKeys {
    my %arg = @_;

    my $fh = $arg{fileHandle};
    my $srcTableObj = $arg{tableObj};
    my $altTableName = $arg{altTableName};

    $t->entry(msg=>"Proccessing alternate key table: $altTableName");

    # Create the new table
    my $altTableObj = 
	Table->new( name   => $altTableName,
		    desc   => $srcTableObj->getDesc(),
		    attrs  => $srcTableObj->getAttrs(),
		    );
    
    # Add the same Fields from the 'source' table to this one
    $altTableObj->setFields(fields => $srcTableObj->getFields() );
    $altTableObj->setFieldsOrder(fieldsOrder => $srcTableObj->getFieldsOrder() );

    # Set this table's keys accordingly
    # First, find all the alternateKeys
    # Actually, this is a bug.  The keys are only keys in the context of the 
    #  table, now across all tables.  This should still be ok, since the main
    #  use of keys now is one way to understand required fields.  If we need to
    #  have a definative picture of key, this is a bug.
    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for the end of the section
	if ( /^\s*\}/ ) {
	    $t->exit(msg=>"End of section hit"); 
	    last;
	}
	
	# There may be multiple keys on a single line, process them
	my @answer = undef;
	my $keyLine = $_;
	$keyLine =~ s/^\s+//;
	$keyLine =~ s/\s+$//;
	$t->debug(msg=>"Trimmed processed line of key(s): $keyLine");
	@answer = split(/ /, $keyLine);

	for my $key ( @answer ) {
	    if ( $key == " " ) {next;}
	    $t->debug(msg=>"alt key to update: $key");
	    my $newKeyField = $altTableObj->getField($key);
	    
	    # Set the 'key' attribute, if not already set
	    if ( ! ($newKeyField->getQualifier() =~ /key/) ) { 
		$t->debug(msg=>"Qualifier was: " . $newKeyField->getQualifier());
		$newKeyField->setQualifier(qual=>"key " .
					   $newKeyField->getQualifier());
		$t->debug(msg=>"Qualifier is: " . $newKeyField->getQualifier());
	    }
	}
    }

    # Add the new table to the allTables list
    $allTables{ $altTableObj->getName() } = $altTableObj;
    
    $t->exit(msg=>"Alt keys processes and table: " .
	     $altTableObj->getName() . " added to table list");
}


# return the first blank delimitted word from the passed string.
# Also return the string after the word and blank have been removed.
sub getNextWord {
    my %arg = @_;
    my $string = $arg{string};
    my %answer = ();

    ($answer{word}, $answer{string}) = split / /, $string, 2;
    return %answer;
}  



# Process the table [value] info from a "values" section, these are
# really just referecens to other existing Fields
sub procValues {
    my %arg = @_;

    my $fh = $arg{fileHandle};
    my $table = $arg{tableObj};
    my ($refTable, $refField, $tableName, $keyName, $remapName);
    
    $t->entry(msg=>"Processing 'values' <table key> pairs for: " . 
	      $table->getName());

    # Grab all the "table key" pairs
    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for the end of the section
	if ( /^\s*\}/ ) {
	    $t->exit(msg=>"End of section hit"); 
	    return;
	}
	
	if ( /^\s*(\S+)\s*(\S+)\s*$/ ) {
	    $tableName = $1;
	    $keyName = $2;
	    if ( $keyName =~ /(\S+)\[/ ) {
		$keyName = $1;
		$t->debug(msg=>"Stripping key order info");
	    }
	    if ( $keyName =~ /(\S+)\s*=\s*(\S+)/ ) {
		$keyName = $1;
		$remapName = $2;
	    }
	    $refTable = $allTables{$tableName};
	    if ( !$refTable ) {
		$t->fatal(msg=>"Table not yet parsed: $tableName");
	    }
	    # Looks like some fields are marks as hidden, quick fix for that
	    $keyName =~ s/^~//;
	    if ( $remapName ) { 
		$table->addViewField( table =>$refTable,
				      field => $keyName,
				      remap => $remapName );
		$remapName = undef;
	    } else {
		$table->addViewField( table =>$refTable,
				      field => $keyName);
	    }
	} else {
	    $t->fatal(msg=>"Bad 'values' section?"); 
	}
    }
}


# Process the table [key] info from a "keys from" section
sub procKeysFrom {
    my %arg = @_;

    my $fh = $arg{fileHandle};
    my $table = $arg{tableObj};
    my ($refTable, $keyName, $tableName, $remapName);

    $t->entry(msg=>"Looking for 'keys from' <table key> pairs for: " . 
	      $table->getName());

    if ( /^\s*keys from/ ) {
	
	# Must be of form:
	# keys from { <tableName> <fieldName> ... }
	# and this will break if someone puts more than one
	# table key   pair on the same line
	if ( /^\s*keys from\s+\{\s*(\S+)\s*(\S+)\s*\}\s*/ ) {
	    
	    $refTable = $allTables{$1};
	    $table->addViewField( table =>$refTable,
				  field => $2);
	    
	} elsif ( /^\s*keys from\s+\{\s*$/ )  {
	    # Grab all the "table key" pairs
	    while ( <$fh> ) {
		if ( skipBlankNComments() ) { next; }
		
		# Look for the end of the section
		if ( /^\s*\}/ ) {
		    $t->exit(msg=>"End of section hit"); 
		    return;
		}
		if ( /^\s*(\S+)\s*(\S+)=(\S+)\s*$/ ) {
		    $tableName = $1;
		    $keyName = $2;
		    $remapName = $3;
		    $refTable = $allTables{$tableName};
		    if ( $refTable eq undef ) {
			$t->fatal(msg=>"Table not already processed, rework included order: $tableName");
		    }
		    if ( $keyName =~ /(\S+)\[/ ) {
			$keyName = $1;
			$t->debug(msg=>"Stripping key order info");
		    }
		    $table->addViewField( table => $refTable,
					  field => $keyName,
					  remap => $remapName );
		} elsif ( /^\s*(\S+)\s*(\S+)\s*$/ ) {
		    $tableName = $1;
		    $keyName = $2;
		    # A key of - means no keys from this table, directly
		    if ( $keyName eq "-" ) { 
			$t->trace(msg=>"Skipping keys for table: $keyName");
		    } else { 
			if ( $keyName =~ /(\S+)\[/ ) {
			    $keyName = $1;
			    $t->debug(msg=>"Stripping key order info");
			}
			$refTable = $allTables{$tableName};
			$table->addViewField( table => $refTable,
					      field => $keyName);
		    }
		} else {
		    $t->fatal(msg=>"Bad 'table keys' section?"); 
		}
	    }
	}
    } elsif ( /^\s*keys from\s+(\S+)\s*/ ) {
        # Maybe it's the simple form:
	# keys from <tableName>
	
	$t->debug(msg=>"Adding all fields");
	$refTable = $allTables{$1};
	$table->addViewField( table => $refTable,
			      field => "--ALLKEYS--");
	$t->exit(msg=>"All keys added from table: $1");
	return;
    } else {
	$t->fatal(msg=>"Bad 'table keys' section?\n$_\n"); 
    }
}


# Process the key to key mapping within the view tables
sub procKeysFor {
    my %arg = @_;

    my $fh = $arg{fileHandle};
    my $table = $arg{tableObj};

    $t->debug(msg=>"Skipping  'keys for' <table key> pairs for: " . 
	      $table->getName());
}


# Process the fields within a table
sub procFields {
    my %arg = @_;

    my $fh = $arg{fileHandle};
    my $table = $arg{tableObj};
    my (@fields, $field);
    my ($name, $desc, $type, $qual, $hidden);

    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for the end of the section
	if ( /^\s*\}/ ) {
	    return;
	} else {
	    
	    # Create a Field object and add it to the
	    #  Table object
	    ($name, $desc, $type, $qual) = 
		/^\s+(.*?)\s+\"(.*)\"\s+(.*?)\s+(.*?)\s*$/;
	    my $keyName = $name; $keyName =~ s/ |\(|\)|,\S//g;
	    if (!$keyName) { 
		# First let's see if they split the line
		($name, $desc, $type) = 
		    /^\s+(.*?)\s+\"(.*)\"\s+(.*?)\s*$/;
		if ( $name ) {
		    $keyName = $name; $keyName =~ s/\!| |\(|\)|,\S//g;
		    while ( <$fh> ) {
			if ( /^\s*(.*)\s*$/ ) {
			    $qual = $1;
			    last;
			} else { 
			    $t->fatal(msg=>"Give up parsing: $_");
			}
		    }
		} else {
		    next; 
		}
	    }

	    #print "About to add Field $name to table $table\n";
	    if ( $keyName =~ s/^~// ) {
		$hidden = 1;
	    }

	    $field = Field->new( name => $keyName,
				 desc => $desc,
				 type => $type,
				 qual => $qual,
				 hidden => $hidden,
				 );	
	    # Reset conditional fields:
	    $hidden = undef;

	    $table->addField( name  => $field->getName(),
			      field => $field
			      );

	    # Create the Type object for the type defined for this Field,
	    #  if it doesn't exist.
	    my $type = $field->getType();
	    # default the name to the type
	    my $name = $type;
	    my ($min, $max, $pfx);
	    my $isList = 0;
	    my $specialCase = 0;
	    # We need to do things differently if we are dealing with 
	    # a list or a range. 
	    if ($name =~/^list<(.*)>/) {
		$type = $1;
		# redefine type to be named "list-<listType>"
		$name = "list-".$type;
		$isList = 1;
		$specialCase = 1;
	    }
	    if ($name =~ /^\s*(.*)(integer|text)<(.*)\.\.(.*)>/) {
		# We are dealing with a range.
		$pfx = $1;
		$type = $2 eq "integer" ? "numeric" : "string";
		$min = $3;
		$max = $4;
		# redefine type to be named "<type>-<min>-<max>"
		$name = $pfx.$2."-".$min."-". $max;
		$specialCase = 1;
	    }
	    if ( ! (exists $typeDefHash{$name}) ) {
		$typeDefHash{$name}{isList} = $isList;
		$typeDefHash{$name}{name}   = $name;
		# When dealing with 'lists' or ranges, we need to store
		# the type.  In other cases, the name/index will be the
		# same as the type. 
		if (!$specialCase) {
		    $typeDefHash{$name}{type} = "DEFAULT";
		} else {
		    $typeDefHash{$name}{type} = $type;
		}
		# If min or max is defined store it.
		if ($min ne undef) {
		    $typeDefHash{$name}{min} = $min;
		    $typeDefHash{$name}{max} = $max;
		}
	    }
	}
    }
}


# Process the methods within a table
sub procMethods {
    my %arg = @_;
    my $fh = $arg{fileHandle};
    my $table = $arg{tableObj};
    my $method;

    my @methods;

    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	$t->debug(msg=>"Working on line in method: " . $_);
	
	# Look for and process the methods, assuming line format:
	# method <methodName> "Method description" { method privs } 
	if ( /^\s*method\s*(.*?)\s*\"(.*)\"\s*\{(.*)\}/ ) {
	    my ($name, $desc, $priv) = ($1, $2, $3);

	    # Get the default table priv is none set for this meth
	    $priv =~ s/ //;
	    if ( ! $priv ) {
		$priv = $table->getPriv();
	    }
	    #print "Found a method: $name, $desc, $priv, \n";
	    #print "$_\n";
	    # We have enough to create the new Method
	    $method = Method->new( name => $name,
				   desc => $desc,
				   priv => $priv,
				   table => $table,
				   );
	    $table->addMethod( name   => $name,
			       method => $method,
			       );
	    #push @methods, ( $method );

	    #   Does the line end with and empty section
	    if ( /\{\s*\}\s*$/ ) {
		$t->debug(msg=>"Found an empty method section");
		# Grab the command from this line
		my $cmd = procMethCmd ( $fh );
		$cmd->setTable($table);
		$method->addCommand( $cmd );
	    } else {
		# Now go get the rest of the Method definition
		procMethod ( fileHandle => $fh, 
			     methObj    => $method,
			     table      => $table,
			     );
	    }
	} else {
	    # Look for the end of the methods section
	    if ( /^\s*\}/ ) {
		$t->exit(msg=>"Hit the end of the methods section");
		#$t->debug(msg=>Dumper($table));
		return @methods;
	    } else {
	    }
	}
    }
}


# Process a method within a methods section.
# $fh - is the SMF input filehandle
# $method - is the created Method object to be complete
sub procMethod {
    my %arg = @_;
    my $fh = $arg{fileHandle};
    my $method = $arg{methObj};
    my $table = $arg{table};
    my $cmd;

    $t->entry(msg=>"Looking for args and cmds in method section");

    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for a section we don't understand yet
	if ( /^\s*zephyr .*\{/ ) {
	    $t->debug(msg=>"Found a non-supported section zephyr\n");
	    skipSection( $fh );
	    next;
	}

	# Look for an args section
	if ( /^\s*args \{/ ) {
	    # Check for an empty args section, like:
	    # 	    args { }
	    if ( /\}\s*/ ) { next; }
	    #print "Found an args section.\n";
	     procMethArgs ( fileHandle => $fh,
			    methObj => $method
			    );
	     next;
	}

	# Look for a command section
	if ( /^\s*command/ ) {
	    #print "Found a command section.\n";
	    #print "$_\n";
	    $cmd = procMethCmd ( $fh );
	    $cmd->setTable($table);
	    $method->addCommand( $cmd );
	    next;
	}

	# Look for the arg's privileges for this method
	if ( /^\s*privilege/ ) {
	    #print "Found a privilege section in the method.\n";
	    #print "$_\n";
	    procArgPriv ( fileHandle => $fh, 
			  methObj    => $method
			  );
	    next;
	}

	# Look for the end of the method section
	if ( /^\s*\}/ ) {
	    $t->exit(msg=>"Hit end of method section");
	    if ( !$method->getCommand() ) {
		my$cmdObj = Command->new( name => $method->getName(),
					  command => $method->getName());		
		# Stuff the commands obj for quick access later.
		$t->debug(msg=>"Adding method: " . $cmdObj->getCmdString());
		$allCommands{$cmdObj->getName()} = $cmdObj;
		$cmdObj->setTable($table);
		$method->addCommand( $cmdObj );
	    }	    
	    return;
	} else {
	    #print "$_\n";
	}

    }
}

# Process the args within a method
sub procMethArgs {
    my %arg = @_;
    my $fh = $arg{fileHandle};
    my $method = $arg{methObj};

    my (@args, $field);
    my ($name, $desc, $type, $qual, $hidden);

    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for the end of the args section
	if ( /\}\s*$/ ) {
	    return;
	} else {
	    # Create a Field object and add it to the Method
	    ($name, $desc, $type, $qual) = 
		/^\s+(.*?)\s+\"(.*)\"\s+(.*?)\s+(.*?)\s*$/;
	    if ( $name =~ s/^~// ) {
		$hidden = 1;
	    }
	    $t->debug(msg=>"Adding arg: $name to method " .
		      $method->getName() . "\n");
	    $field = Field->new( name => $name,
				 desc => $desc,
				 type => $type,
				 qual => $qual,
				 hidden => $hidden,
				 ); 
	    $method->addArg( name => $field->getName(),
			     arg => $field);
	    
	}
    }
}


sub addActionCmd {

    my %arg = @_;

    my $commandStr = $arg{command};
    my $table = $arg{table};
    my $priv = $table->getPriv();

    $commandStr =~ /\s*(.*)\s+(.*)\s*$/;
    my $directory = $1;
    my $command = $2;

    #print "action command dir: $directory, cmd: $command\n";
    $table->setCmdDir( $directory );
    my $cmdObj = Command->new( name    => $command,
			       command => $commandStr,
			       priv    => $priv,
			       table   => $table,
			       );
    
    $table->addCommand( name    => $cmdObj->getName(),
			command => $cmdObj);
    
    # Stuff the commands obj for quick access later.
    $t->debug(msg=>"Adding action command " . $cmdObj->getCmdString());
    $allCommands{$cmdObj->getCmdString()} = $cmdObj;
}


# Process the commands within a table
sub procCommand {
    my %arg = @_;
    my $fh = $arg{fileHandle};
    my $table = $arg{table};

    my ($cmdObj, $command, $priv, $fields, $collapse);

    # Does the line end with an empty section.  YASC (yet another special case), 
    # mostly appears with action tables.
    if ( /\s*command\s+\"\s*(.*?)\s*\"\s*\{\s*\}\s*$/ ) {    
	# All the info we need is on one line....
	addActionCmd ( command => $1,
		       table   => $table,
		       );
	return;
    }

    # The directory is the "path" to the command.  The actual
    # command is the last word before -fields, which isn't present
    # on the first line of a command section. (Unless 'collapse' is
    # present, see below).
    /\s*command\s+\"\s*(.*?)\s*\"\s*\{/;
    my $directory = $1;

    if ( $directory ) {
	$table->setCmdDir( $directory );
	#print "Just set dir: $directory, ", $table->isActionTable(), "\n";
	# If this is an action table, create a command from the directory
	if ( $table->isActionTable() ) {
	    addActionCmd ( command => $directory,
			   table   => $table,
			   );
	}
    }

    # Now look for any commands and "remove" directives
    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Remove any commands
	if ( /\s*remove/ ) {

	    my ($cmdRemoved, $rmCmdObj, @cmds);
	    # remove syntax maybe:
	    # remove <cmd>
	    #  or
	    # remove { <cmd ...> }
	    if ( /\s*remove\s+\{\s*(.*)\s*\}\s*$/ ) {
		$cmdRemoved = $1;
		$t->debug(msg=>"Found remove, braced syntax: $cmdRemoved");
	    } elsif ( /\s*remove\s+(.*)\s*$/ ) {
		$cmdRemoved = $1;
		$t->debug(msg=>"Found remove, non-braced syntax: $cmdRemoved");
		if ( $cmdRemoved eq "http" ) {
		    $t->debug(msg=>"Found remove for http, not caring right now.");
		    next;
		}
	    } else {
		$t->debug(msg=>"Found remove, but have issue finding commands");
		next;
	    }

	    # Remove the whole command set if the removed command is "ami"
	    if ( $cmdRemoved eq "ami" ) {
		my $cmds = $table->getCmds();
		$t->debug(msg=>"Found remove ami, nuke all the commands in this table");
		@cmds = keys %{ $cmds };	       
	    } else {
		@cmds = split / /, $cmdRemoved;
	    }

	    # Remove the commands from the table
	    removeCmds( cmds => [@cmds], table => $table);
	    
	    next;
	}
	# If collapse present, rework the command after reading it
	if ( /^\s*collapse\s*$/ ) {
	    $collapse = 1;
	    next;
	}

	# Skip externalize for now
	if ( /\s*externalize\s+(.*)\s*$/ ) {
	    next;
	}

	# Skip bootlevel for now
	if ( /\s*bootlevel\s+(.*)\s*$/ ) {
	    next;
	}

	if ( /^\s*grouping/ ) {
	    next;
	}

	# This command is hidden (which doesn't mean much to me right now)
	if ( /\s*hidden\s*$/ ) {
	    $t->debug(msg=>"This is a hidden command set, must type exactly to issue.");
	    next;
	}

	# Skip any help_ sections (for now)
	if ( /\s*help_/ ) { next }

	# Grab command, privs and fields
	# Are all the fields defined on this line?
	if ( /\{.*\}\s*\{.*\}/ ) {
	    # Easier, grab the command, privs and fields
	    ($command, $priv, $fields) =
		/\s*(.*?)\s*\{\s*(.*?)\s*\}\s*\{\s*(.*)\s*\}/;	
	    #print "Easier: $command, $priv, $fields \n$_\n";

	} else {
	    # Harder, assume the fields wrap on to multiple lines
	    ($command, $priv) =
		/\s*(.*?)\s*\{\s*(.*?)\s*\}/;
	    
	    # Grab the fields section as a chunk
	    my $fieldsSection = getSection( $fh );
	    #print "Need to parse a multi-line section\n";	    
	}
	
	# Create the command (if we found one)
	if ( $command ) {
	    if ( $priv ) { } else {
		$priv = $table->getPriv();
	    }
	    #print "Creating and adding command: $directory $command\n";
	    # Check to see if the command already exists.  It may, in the 
	    # case where the same command acts differently at different
	    # priv levels.
	    my $name;
	    if ( $table->getCommand( $command ) ) {
		$name = "$command++$priv";
	    } else {
		$name = $command;
	    }

	    # If the command is a show-qualifier type, don't really add
	    # the command.  Add a Field with the _show flag set.
	    if ( $name =~ /show-(\S+)/ ) {
		# This is a show-qualifier "command", 
		# create the -qualifier field
		$t->debug(msg=>"Adding Field: $1 to table: " .
			  $table->getName() . "\n");
		my $field = Field->new( name => $1 );
		$field->setShowField();
		$table->addField( name  => $1,
				  field => $field
				  );
	    } else {
		# Now add the command
		# If there was a 'collapse' present, the last cmd dir node is
		# actaully the command (and we'll throw away the actual command).
		if ( $collapse ) {
		    $t->debug(msg=>"Collapsing dir: $directory   cmd: $command  for table: " .
			      $table->getName());
		    $directory =~ /(.*?)\s*(\S+)$/; 
		    $directory = $1;
		    $command = $2;
		    $name = $command;
		    #$collapse = undef;
		    $table->setCmdDir($directory);
		    $t->debug(msg=>"Collapsed dir: $directory   cmd: $command  for table: " .
			      $table->getName());
		}
		$cmdObj = Command->new( name    => $name,
					command => "$directory $command",
					priv    => $priv,
					table   => $table,
					);		
		$table->addCommand( name    => $cmdObj->getName(),
				    command => $cmdObj);		
		# Stuff the commands obj for quick access later.
		$t->debug(msg=>"Adding command: " . $cmdObj->getCmdString());
		$allCommands{$cmdObj->getCmdString()} = $cmdObj;
	    }
	}	     

	# Now update the new/delete/modify commands if present
	my $cmddir = $table->getCmdDir();
	for my $cmd ("new","modify","delete") {
	    $t->debug(msg=>"Looking for cmd: $cmd in table: " . $table->getName());
	    if ( $table->hasCommand($cmd) ) {
		# Complete the command
		$cmdObj = $table->getCommand($cmd);
		$cmdObj->setTable($table);
		$cmdObj->setCmdString( "$cmddir $cmd");
		$cmdObj->setPriv($table->getPriv());
		
		# Stuff the commands obj for quick access later.
		$t->debug(msg=>"Adding updated command " . $cmdObj->getCmdString());
		$allCommands{$cmdObj->getCmdString()} = $cmdObj;
	    }
	}

	# Skip next inner section
	if ( /\{\s*$/ ) {
	    skipSection( $fh );
	    next;
	}
	# Look for the end of the entire section
	if ( /\}\s*$/ ) {
	    last;
	} 
    }
    return ;
}


sub removeCmds {
    my (%arg) = (@_);
    my @cmds = @{$arg{cmds}};
    my $table = $arg{table};
    
    my ($rmCmdObj, $cmdRemoved);

    foreach $cmdRemoved (@cmds) {
	$rmCmdObj = $table->getCommand($cmdRemoved);
	$table->delCommand( $cmdRemoved );
	
	# if the command exists, remove it
	if ( $rmCmdObj ) {
	    delete $allCommands{$rmCmdObj->getCmdString()};
	    $t->debug(msg=>"Removing command: $cmdRemoved");
	} else {
	    $t->debug(msg=>"Couldn't find Command obj to remove: $cmdRemoved");
	}
    }
}


# Process the command within a method
sub procMethCmd {
    my $fh = shift;
    my $cmdObj;

    $t->entry(msg=>"Processing a method command");
    /\s*command\s+\"(.*)\"\s+\{/;
    my $command = $1;
    if ( !$command ) {
	# Try pulling it from first line
	/\s*method\s+(.*)\s+\"(.*)\"\s+\{/;
	$command = $1;
    }


    $cmdObj = Command->new( name => $command,
			    command => $command);

    # Stuff the commands obj for quick access later.
    $t->debug(msg=>"Adding method: " . $cmdObj->getCmdString());
    $allCommands{$cmdObj->getName()} = $cmdObj;

    # Does the line end with and empty section
    if ( /\{\s*\}\s*$/ ) {    
	$t->exit(msg=>"Hit empty braces at EOL");
	return $cmdObj;
    }
    # Look for the end of this command section
    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for the end of the section
	if ( /^\s*\}\s*$/ ) {
	    $t->exit(msg=>"Hit closing brace, alone on line");
	    return $cmdObj;
	} else {
	    #print "$_\n";
	}
    }
}


# Process the privilege for Field entries
sub procFieldPriv {
    my %arg = @_;

    my $priv = "???";
    my @fields = undef;
    my ($fKey, $field);
    my $fh = $arg{fileHandle};

    # Check to see if the section header has all the data
    #     privilege diagnostic { value4 value5 }
    if ( /^\s*privilege\s*(.*?)\s*\{\s*(.*)\s*\}/ ) {
	$priv = $1;
	@fields = split / /, $2;

    } else {
	# Fields continue on multiple lines?
	#   privilege advanced {vserver-root 
	if ( /^\s*privilege\s*(.*?)\s*\{\s*(.*?)\s*[^\}]/ ) {
	    $priv = $1;
	    push @fields, split / /, $2;
	}
	# Get the rest of the fields
	while ( <$fh> ) {
	    if ( skipBlankNComments() ) { next; }

	    # Look for the end of the privilege section
	    if ( /^\s*\}/ ) {
		last;
	    } else {
		if ( /^\s*(.*?)\s*$/ ) {
		    $t->debug(msg=>"About to push some priv fields: $_");
		    push @fields, split / /, $1;
		    # remove last element incase s } slipped in there
		    if ( $fields[$#fields] =~ /\}/ ) { 
			$#fields--; 
			last;
		    }
		}
	    }
	}
    }
    
    # If we have a priv, use it to update the 
    # mentioned field's priv
    if ( @fields ) {
	for $fKey ( @fields ) {
	    $t->debug(msg=>"Working on priv: $priv for field: $fKey\n");
	    if ( !($fKey) ) { next; }
	    $field = $arg{tableObj}->getField($fKey);
	    $field->setPriv(priv => $priv);
	}
    }
}


# Process the privilege for method entries
sub procMethodPriv {
    my $fh = shift;

    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for the end of the privilege section
	if ( /^\s*\}/ ) {
	    return;
	} else {
	    #print "$_\n";
	}
    }
}


# Process the privilege for method arg entries
sub procArgPriv {
    my %arg = @_;

    my $priv = "???";
    my @args;

    # Check to see if the section header has all the data
    if ( /^\s*privilege\s*(.*?)\s*\{\s*(\S+)\s*\}\s*$/ ) {
	$priv = $1;
	@args = split / /, $2;
	# Now update the mentioned field's priv
	my ($argKey, $argObj);
	for $argKey ( @args ) {
	    $t->debug(msg=>"Processing arg: $argKey\n");
	    $argObj = $arg{methObj}->getArg($argKey);
	    #$t->debug(msg=>"The arg is:\n" . Dumper($argObj) .
	    #         "\n");
	    #$t->debug(msg=>"The args are:\n" . 
	    #	   Dumper($arg{methObj}->getArgs()) .
		#      "\n");
	    $argObj->setPriv(priv => $priv);
	}
    } else {
	while ( <$arg{fileHandle}> ) {
	    if ( skipBlankNComments() ) { next; }

	    # Look for the end of the privilege section
	    if ( /^\s*\}/ ) {
		last;
	    } else {
		#print "$_\n";
	    }
	}
    }
}


# The passed filehandle will be advanced beyond the current
# brace delimited section. 
sub skipSection {

    my $fh = shift;

    my $section = $_;

    # First check to see if this section is just a one liner
    if ( /\{.*\}\s*$/ ) {
	return $section;
    }

    my $openCount = 0;
    my $keepLooking = 1;

    # Since the first line of the section is already in $_
    # Look for opening braces
    $openCount += tr/\{//;
    $keepLooking = 0 if ( $openCount > 0 ) ;

    # Look for and subtract closing braces
    $openCount -= tr/\}//;

    # Special case.  If the line ends in a { but has an openCount
    # of 0, add 1 to the open count.  This can happen if the 1st 
    # line has the format   } {   Which clearly isn't the end
    # of the section but does present balanced braces.
    if ( /\{\s*/ && $openCount == 0 ) {
	$openCount = 1;
    }

    #print "$openCount\n";
    while ( <$fh> ) {
	if ( skipBlankNComments() ) { next; }

	# Look for opening braces
	$openCount += tr/\{//;
	$keepLooking = 0 if ( $openCount > 0 ) ;

	#print "$openCount   -    ";

	# Look for and subtract closing braces
	$openCount -= tr/\}//;

	#print "$openCount\n";
	# Look for the end of the command section
	if ( $openCount == 0 &&
	     $keepLooking == 0) {
	    return $section;
	} else {
	    $section = "$section $_";
	    #print "$_\n";
	}
    }

}

sub skipBlankNComments() {
    my $fh = shift;

    return (skipComments() || skipBlank() || skipOpenBrace());
}

sub skipOpenBrace { 
    my $fh = shift;
    
    chomp;
    if (/^\s*\{\s*$/ ) { 
	return 1;
    } else {
	return 0;
    }
}
    

sub skipComments { 
    my $fh = shift;
    
    chomp;
    if (/^\s*\/\// ) { 
	return 1;
    } else {
	return 0;
    }
}
    

sub skipBlank {
    my $fh = shift;

    chomp;
    # Skip blank and comment lines
    if ( /^\s*$/ ) {
	return 1;
    } else {
	return 0;
    }
}


sub getBlock {
}
    



# The passed filehandle will be used to find and return
# the last section.  "Last" is defined as the last completed
# section on the current line, or the completed section that
# is started on the current line.  For example:
#
# foo { bar1 bar2 }
# 
# foo {
#    bar1 bar2 }
#
# "bar1 bar2" will be returned in both cases
sub getSection {


}


#
# updTypes
# Update Types from a predefined source file (not an SMF file)
# These types will be used to augment the learned definitions of
# Types created from parse SMF tables.
#
sub updTypes {

    # This should probably removed. for now, skip...
    return;
    my %arg = @_;

    my $fileName = $arg{typeFile};

    if ( !($fileName) ) {
	die("Types input file required");
    }
    
    open(fhTypes, "< $fileName") || die("Can't open $fileName: $!");

    my ($line, $str);
    while ( $line = <fhTypes> ) {
	$str .= $line;
    }
    my $rawData =  eval $str;

    close(fhTypes);

    # Now for each defined hash, create a Type object
    my $typeName;

    for $typeName ( keys %{ $rawData } ) {

	if ( !(exists  $rawData->{$typeName}) ) {
	    die( "Type $typeName must exist before it can be updated");
	}
	
	$t->debug(msg=>"Creating Type: $rawData->{$typeName}{_name}\n");
	$typeObjHash{$typeName} = 
	    Type->new( name      => $rawData->{$typeName}{_name},
		       baseType  => $rawData->{$typeName}{_baseType},
		       min       => $rawData->{$typeName}{_min},
		       max       => $rawData->{$typeName}{_max},
		       pattern   => $rawData->{$typeName}{_pattern},
		       );
    }
    return %typeObjHash;
}


#
# updFields
# Update Fields from a predefined source file (not an SMF file)
# These fields will be used to augment the learned definitions of
# Fields created from parse SMF tables.
# 
sub updFields {
    my %arg = @_;

    my $fileName = $arg{fieldFile};
    my (%fieldHash, $fieldName, $fieldObj, $tmp);
    my ($createCmd, $directory, $fields, $table);

    if ( !($fileName) ) {
	die("Field input file required");
    }
    
    open(fhFields, "< $fileName") || die("Can't open $fileName: $!");
    my ($line, $str);
    while ( $line = <fhFields> ) {
	$str .= $line;
    }
    my $rawData =  eval $str;
    close(fhTypes);

    # Now for each defined hash, create a Field object
    my ($directory);

    for $directory ( keys %{ $rawData } ) {

	if ( !(exists  $rawData->{$directory}->{create}) ) {
	    print "No create section in dir: $directory\n";
	    next;
	}

	#print "working on create in dir: $directory\n";
	$createCmd = $rawData->{$directory}->{create}->{cmd};
	$fields = $rawData->{$directory}->{create}->{fields};
	
	# Look for the cmd (we really need to table to get to the 
	# field, but the cmd has the parent table, shortcut).
	if ( exists $allCommands{"$directory $createCmd"} ) {

	    # Command exists, get the table, to get the right Field
	    $table = $allCommands{"$directory $createCmd"}->getTable();
	    for $fieldName ( @{$fields} ) {
		$fieldObj = $table->getField($fieldName);
		$tmp = "$fieldName is a type: " . $fieldObj->getType() . "\n";
		$t->debug(msg=>$tmp);

		if ( $fieldObj ) {
		    #print "Found existing field $fieldName, updating\n";
		    $fieldObj->setCreateCmd("$directory $createCmd");
		    #$fieldObj->printMe();
		} else {
		    die "Field object must exist before it can ",
		         "be updated: $fieldName\n";
		}
	    }
	} else {   # if ( exists $allCommands{"$directory $createCmd"} )
	    die "Command structure must exist before its fields can ",
	        "be updated: $directory $createCmd\n";
	}
    }
}


sub getAllTypes {
    return %typeObjHash;
}

    
sub getAllCommands {
    return %allCommands;
}

sub dumpTypeDefHash {
    for my $index (keys %typeDefHash) {
	print "Index $index\n";
	for my $value ( keys %{$typeDefHash{$index}} ) {
	    print "\t $value = $typeDefHash{$index}{$value}\n";
	}
    }    
}


# Poor man's findBy for tables by cmdDir
sub findTableByCmdDir {
    my (%arg) = @_;
    my $cmdDirNeedle = $arg{cmdDir};

    my ($tableKey, $cmdDir);
    my @tables = undef;

    # Walk the tables getting and comparing the cmdDir
    for $tableKey (keys %allTables) {

	$cmdDir = $allTables{$tableKey}->getCmdDir();
	if ( $cmdDir eq $cmdDirNeedle ) {
	    $t->debug(msg=>"Found that table: $tableKey, contains cmdDir: " .
		      "$cmdDir\n");
	    push @tables, ($tableKey);
	}
    }
    if ( @tables ) {
	return @tables;
    } 
    $t->debug(msg=>"Didn't find a table that contains cmdDir: $cmdDirNeedle\n");
    return undef;
}


# Given a Command object and containing table, return the required fields
sub getRequiredFields {
    my %arg = @_;
    my $cmd = $arg{command};
    my $table = $cmd->getTable();

    if ( ref($table) eq "Method" ) {
	# The "table" for a Command in a Method is actually the
	# Method.  We need to ask the Method for the real
	# Table object
	$table = $table->getTable();
    }

#    $t->entry(msg=>"Working on cmd: " . $cmd->getName() . " in table: " .
#	      $table->getName());
    $t->entry(msg=>"Working on cmd: " . $cmd->getName() );
    $t->debug(msg=>"  in table: " . $table->getName());

    my ($method, $fieldKey, $field, $fieldName, $fields);

    my $reqOptsString = undef;
    
    # "show" doesn't require any fields
    if ($cmd->getName() =~ /show/) {
	$t->exit(msg=>"Done because of 'show' command");
	return $reqOptsString;
    }

    # If this is a method, check for required args (kinda
    # like Fields)
    $method = $cmd->getTable();
    if ( ref($method) eq "Method" ) {
	$t->debug(msg=>$cmd->getCmdString() . 
		  " - is a Method, process args\n");
	if ( $method->getArgs() ) { 
	    for $fieldKey (keys %{ $method->getArgs() }) {
		$field = $method->getArg($fieldKey);
		if ( $field->isRequired() &&
		     !$field->isHidden() ) {
		    $fieldName = 
			$method->getArg($fieldKey)->getName();
		    #$fieldName =~ s/-/_/g;
		    $reqOptsString .= "$fieldName ";
		}
	    }
	}
    } else {
	$t->debug(msg=>$cmd->getCmdString() . 
		  " - is a type: ". ref($method) . "\n");
    }
    # Print the required table fields for this command (or Method)
    $fields = $table->getFields();
    for $fieldKey (keys %{ $fields }) {
	$field = $fields->{$fieldKey};
	
	# If the field isn't mark as required, and it's
	# not a key, skip it (always skip hidden fields)
	if  ( (!($field->isRequired()) &&
	      !($field->getQualifier() =~ /key/)) ||
	      $field->isHidden()) {
	    $t->debug(msg=>"Skipping Field: " . $field->getName() .
		      " , it's not required and not a key (or hidden) for command: " .
		      $cmd->getCmdString() . "\n");
	    next;
	}

	$fieldName = $field->getName();
	#$fieldName =~ s/-/_/g;

	# If it's a new command or an action cmd, all required fields are needed
	if ( (($cmd->getName() eq "new") ||
	      ($table->isActionTable())) &&
	     ($field->isRequired()) ) {
	    $t->debug(msg=>"Field: " . $field->getName() .
		      " added, ('new' or 'action') and required for command: " .
		      $cmd->getCmdString() . "\n");
	    $reqOptsString .= "$fieldName ";
	    
	# If is's anything else (expect 'new'), just the keys are required
	} elsif ( ($field->getQualifier() =~ /key/) &&
		  !($cmd->getName() eq "new") ) {
	    $reqOptsString .= "$fieldName ";
	    $t->debug(msg=>"Field: " . $field->getName() .
		      " added, since it's a key for command: " .
		      $cmd->getCmdString() . "\n");
	} else {
	    $t->debug(msg=>"Skipping Field: " . $field->getName() .
		      " for command: " .
		      $cmd->getCmdString() . "\n");
	}
    }
    $t->exit(msg=>"Done, returning $reqOptsString");
    return $reqOptsString;
}


# Given a Command object and containing table, return the non-required fields
sub getOptionalFields {
    my %arg = @_;
    my $cmd = $arg{command};
    my $table = $cmd->getTable();

    $t->entry(msg=>"Working on cmd: " . $cmd->getName() . " in table: " .
	      $table->getName());

    my ($method, $fieldKey, $field, $fieldName);
    my ($nonReqOpts, $fields);

    # Delete has no optional Fields, all key fields are required
    if ( $cmd->getName() eq "delete" ) {
	$t->exit(msg=>"Done because of 'delete' command");
	return undef;
    } 

    # If this is a method, check for non-required args (kinda
    # like Fields)
    $method = $cmd->getTable();
    if ( ref($method) eq "Method" ) {
	$t->debug(msg=>$cmd->getCmdString() . 
		  " - is a Method, process args\n");
	if ( $method->getArgs() ) { 
	    for $fieldKey (keys %{ $method->getArgs() }) {
		$field = $method->getArg($fieldKey);
		# Skip if the field is hidden.
		# Return all non-required args.
		if ( !$field->isHidden() && 
		     !($field->isRequired()) ) {
		    $fieldName = 
			$method->getArg($fieldKey)->getName();
		    #$fieldName =~ s/-/_/g;
		    $nonReqOpts .= "$fieldName ";
		}
	    }
	}
	$t->exit(msg=>"Done early (Method), returning: $nonReqOpts");
	return $nonReqOpts;
    } else {
	$t->debug(msg=>$cmd->getCmdString() . 
		  " - is a type: ". ref($method) . "\n");
    }
    # Print the non-required table fields for this command
    $fields = $table->getFields();
    for $fieldKey (keys %{ $fields }) {
	$field = $fields->{$fieldKey};
	
	if ( $field->isHidden() ) { 
	    $t->debug(msg=>"Field: $fieldKey is hidden, skipping");
	    next; 
	}

	$fieldName = $field->getName();
	#$fieldName =~ s/-/_/g;

	# If a show command, return all fields as optional
	if ( $cmd->getName() =~ /show/ ) {
	    $nonReqOpts .= "$fieldName ";
	} 
	# On non-new commands, everything except keys are
	# optional.
	elsif ( ($cmd->getName() ne "new") &&
		!($field->getQualifier() =~ /key/) &&
		($field->getQualifier() =~ /write|modify|in/) ) {
	    $nonReqOpts .= "$fieldName ";
	}
	# On new commands, use the required attribute and creatable fields
	elsif ( ($cmd->getName() eq "new") &&
		!($field->isRequired()) &&
		($field->getQualifier() =~ /write|create|in|key/) ) {
	    $nonReqOpts .= "$fieldName ";
	}
    }
    $t->exit(msg=>"Done, returning: $nonReqOpts");
    return $nonReqOpts;
}


1;
