#!/bin/sh /usr/software/bin/perl 

#
#  Class that models an SMF Method
#
package Method;
use TracePkg;

use strict;
no strict 'refs';

my $t = TracePkg->new();

sub new {

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    #  a hash.
    my ($self, %arg) = @_;

    # Fill the member vars for this instance
    my $object = {
	_name     => $arg{name} || "???" ,
	_desc     => $arg{desc} || "???" ,
	_priv     => $arg{priv} || "???" ,
	_args     => undef,
	_argsOrder => undef,
	_command  => undef,
	_table => $arg{table},
	};

    # This tags the hash pointed to by $object as a 
    #  type of Method
    bless $object, $self;

    $t->debug(msg=>"Added method: $arg{name}");

    # And give the caller the referece to the bless object.
    return $object;

}


# For quick printing
sub getName {
    my ($self) = @_;

    return $self->{_name};
}


sub getTable {
    my ($self) = @_;

    return $self->{_table};
}


# To update the set of args this method supports.
sub addArg {

    my ($self, %arg) = @_;

    # Add the name of the arg (Feild) to a list of names
    # I need to do this to maintain the order that I inserted
    # the elements into the %_args hash.
    push @{ $self->{_argsOrder} }, ( $arg{name} );

    # Now add the new arg (Field object) to the hash.  
    $self->{_args}{$arg{name}} = $arg{arg};

    # Now set the default priv for the arg.  It (initially)
    # matches that of the method
    $arg{arg}->setPriv( priv => $self->{_priv} );
    #print "Arg ", $arg{arg}->getName(), " set to priv ", 
    #       $self->{_priv}, "\n";

}


# Return the Arg (acutally a Field) object
sub getArg {
    
    my ($self, $name) = @_;
    return $self->{_args}{$name};
}


sub getArgs { 
    my ($self) = @_;

    return $self->{_args};
}


# Add the command used for this method
sub addCommand {
    
    my ($self, $command) = @_;
    $self->{_command}  = $command;
}


sub getCommand {
    my ($self) = @_;
    return $self->{_command};
}


sub printMe {

    my ($self) = @_;
    my $key;

    print "\t$self->{_name}, $self->{_desc}, $self->{_priv}\n";
    print "\t\tArgs: \n";
    #print "\t\tArgs hash: ",  %{$self->{_args}}, "\n";
    # Print the args in the order they were discovered/created
    for $key (  @{$self->{_argsOrder}} ) {
	$self->{_args}{$key}->printMe();
    } 

    if ( $self->{_command} ) {
	$key = $self->{_command};
	print "\t\tCommand: "; print $key->getName(), "\n";
    }
    print "\n";
}


1;
