#!/bin/sh /usr/software/bin/perl 

#
#  Pattern Class, derived version of type.
#
package Pattern;
use SMF::Type;
use Random;
@ISA = ("Type");

use strict;

sub new {
    my $self = shift;
    
    #  We now will pull off any params we need locally.  We will
    # blindly pass any others to our parent.
    my %params = (
		  sample => "",
		  @_
		  );
    # Investigate if there is a simpler way to do this?  
    my $min_pat = $params{min_pat};
    my $max_pat = $params{max_pat};
    my $sample = $params{sample};
    delete $params{min_pat}, $params{max_pat}, $params{sample};
    
    $params{baseType} = "pattern";
    
    # Call our parents constructor.
    my $object = self->SUPER::new(%params);

    $object->{_minPattRegExp} = $min_pat;
    $object->{_minPattRegExp} = $max_pat;
    $object->{_sample}        = $sample;
    # So that for the life of the object we return the same min
    # and max values, we will generate the strings once. 
    my $foo = new String::Random;
    $object->{_minPattern} = $foo->randregex($min_pat);
    $object->{_maxPattern} = $foo->randregex($max_pat);

    bless $object, $self;

    return $object;
}

sub GetMin {
    my $self = shift;
    my %params = (prefix => undef,
		  @_,
		  );
    if ($params{prefix} ne undef) {
	return $self->_GetPrefixedPatt(prefix => $params{prefix},
				       pattern => $self->{_minPattern});
    } else {
	return $self->{_minPattern};
    }
}

sub GetMax {
    my $self = shift;
    my %params = (prefix => undef,
		  @_,
		  );
    if ( $params{prefix} ne undef ) {
	return $self->_GetPrefixedPatt(prefix => $params{prefix},
				       pattern => $self->{_maxPattern});
    } else {
	return $self->{_maxPattern};
    }
}

sub GetSample {
    my $self = shift;
    return $self->{_sample};
}

sub _GetPrefixedPatt {
    my $self = shift;
    my %params = (prefix => undef,
		  pattern => undef,
		  @_);
    if ($params{prefix} eq undef || $params{pattern} eq undef) {
	die "Caller needs to specify prefix and pattern";
    }

    # Determine the lengths of the prefix and pattern
    my $pfx_length = length($params{prefix});
    my $pat_length = length($params{pattern});
    
    if ($pfx_length > $pat_length) {
	die "Prefix length longer than pattern length\n";
    }

    # Set up the start_index
    my $start_index = 0;

    # Need to determine if the pattern is a "quoted" string
    if ($params{pattern} =~ /^"(.*)"$/) {
	# if starting with a quote, we will increment our start position.
	$start_index++;
    }	
    
    substr($params{pattern}, $start_index, $pfx_length) = $params{prefix};

    $pat_length = length $params{pattern};
    
    return $params{pattern};
}

sub printMe {
    my $self = shift;

    # First call parent class to dump common information
    $self->SUPER::printMe();

    # Add to the end, dump out sample.
    if ($self->{_sample} ne undef) {
	print " Sample = $self->{_sample}\n";
    }
}
