#!/bin/sh /usr/software/bin/perl 

#
#  String Class, derived version of type.
#
package String;
use SMF::Pattern;
use Random;
@ISA = ("Pattern");


use strict;

sub new {
    my $self = shift;
    
    # We now will pull off any params we need locally. We will 
    # blindly pass any others up to our parent.  We will also
    # default our min and max.
    my %params = (
		  min => 1,
		  max => 255,
		  @_
		  );
    # Investigate if there is a simpler way to do this?
    my $min = $params{min};
    my $max = $params{max};
    delete $params{min};
    delete $params{max};  
		
    # Build the patterns to pass to our parent
    $params{min_pat} = _build_pat($min);
    $params{max_pat} = _build_pat($max);

    # Call our parents constructor.
    my $object = self->SUPER::new(%params);

    # We need to define
    $object->{_min} = $min;
    $object->{_max} = $max;

    bless $object, $self;

    #And give the caller the reference to the blessed object
    return $object;
}

sub _build_pat {
    
    my $length = shift;
    my $return_string;
    
    # If the length is 3 or greater, we need to include an or
    # clause for a quoted string. For instance if the length is
    # 5 than both (itself) and ("one") are valid strings.
    if ($length < 3) {
	$return_string = "\\w{".$length."}";
    }
    else {
	my $quoted_length = $length - 2;
	$return_string = "(\\w{".$length."})|(";
	$return_string .= "\"\\w{".$quoted_length."}\")";
    }
    return $return_string;
}
    

