#!/bin/sh /usr/software/bin/perl 

#
#  Base Class that models SMF types
#
package Type;

use strict;
no strict 'refs';

sub new {

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    #  a hash. 
    my $self = shift;

    my %arg = (isList => 0,
	       @_);

    # Fill the member vars for this instance
    my $object = {
	_name      => $arg{name},
	_desc      => $arg{desc},
	_baseType  => $arg{baseType},
	_isList    => $arg{isList},
	};

    # This tags the hash pointed to by $object as a 
    #  type of Field
    bless $object, $self;

    # And give the caller the referece to the bless object.
    return $object;
}

sub isList {
    my $self = shift;
    return $self->{_isList};
}
    
sub getName {
    my ($self) = @_;
    return $self->{_name};
}


sub getDesc {
    my ($self) = @_;
    return $self->{_desc};
}


sub getBaseType {
    my ($self) = @_;
    return $self->{_baseType};
}


sub getMin {
    # Change to fatal.  Needs to be redefined in derived version
}


sub getMax {
    # Change to fatal.  Needs to be redefined in derived version
}

sub getValues {
    # Change to fatal.  Needs to be redefined in derived version
}

sub getMaxListItems {
    # Should move this to be the equiv to a macro in perl 
    # Need to determine if this is a good count?
    return 10;
}


# Get values, returned as a string
sub getValsStr {
    my ($self) = @_;
    
    my ($values, $val) = ("", "");
    
    if ( exists $self->{_values} ) {
	$values = join(' ', @{ $self->{_values} } );		   
    }
    return $values;
}


sub isNumber {
    my ($self) = @_;
    return $self->{_baseType} =~ /integer/ ? 1 : 0;
}


sub isText {
    my ($self) = @_;
    return $self->{_baseType} =~ /text/ ? 1 : 0;
}


sub isEnum {
    my ($self) = @_;
    return $self->{_baseType} =~ /enum/ ? 1 : 0;
}


sub genMin {
    my ($self) = @_;
    return $self->{_min};
}


sub genPattern {
    my ($self) = @_;
    return $self->{_name};
}


sub getValue {
    my ($self, $int) = @_;

    return $self->{_values}[$int];
}


sub printMe {

    my ($self) = @_;
    # Should probably move to the derived versions.

    print "\n##################\n";
    print "Name = $self->{_name} \n Description = $self->{_desc}\n Type = $self->{_baseType}\n";
    my $min = $self->GetMin();
    my $max = $self->GetMax();
    print " min = $min \n max = $max\n";
    my $isList = $self->isList();
    print " Is list = $isList\n";
    my $print=0;
    for my $value ( @{$self->{_values}} ) {
	if ($print == 0) {
	    print "Values = ";
	    $print = 1;
	}	   
	print "$value, ";
    }
}


1;
