#!/bin/sh /usr/software/bin/perl 

#
#  Class that models SMF views
#
package View;
@ISA = ( "Table" );

use strict;
no strict "refs";

use Data::Dumper;
use FindBin;
use lib "$FindBin::Bin/../../lib";
use TracePkg;

my $t = TracePkg->new();

sub new {

    # an instance of "this" is always the first arg
    # the second arg is a set of named arguments used to init
    #  a hash.
    my ($self, %arg) = @_;
    $t->entry(msg=>"Creating a new View: $arg{name}"); 

    # Call our parent's constructor
    my $object = self->SUPER::new(%arg);

    # This tags the hash pointed to by $object as a 
    #  type of Table
    bless $object, $self;

    # And give the caller the referece to the bless object.
    $t->exit(msg=>"View created: $arg{name}"); 
    return $object;
}


sub addViewField {

    # Add a Table and optional keys that will be used when keys
    # for this Table are requested.
    my ($self, %arg) = @_;
    my $refTable = $arg{table};
    my ($refField, $fieldName, $field, @fields);

    $t->entry(msg=>"Passed: " . join(",", @_));

    if ( $arg{field} eq "--ALLKEYS--" ) {
	$fieldName = "--ALLKEYS--";
	foreach ( keys %{ $refTable->getFields() } ) {
	    push( @fields, $refTable->getField( $_ ) );
	}
    } else {
	$refField = $refTable->getField( $arg{field} );
	$fieldName = $refField->getName();
	push( @fields,  $refField );
    }
    $t->trace(msg=>"Adding field: $arg{field} from table: " . $refTable->getName() .
	      " into View: $self->{_name}"); 
    
    foreach ( @fields ) {
	$t->debug(msg=>"Adding field: " . $_->getName());

	if ( defined $arg{remap} ) {
	    $t->debug(msg=>"Remapping field: $arg{field} to new name: $arg{remap}");
	    $fieldName = $arg{remap};
	} else {
	    $fieldName = $_->getName();
	}
	$field = Field->new( name => $fieldName,
			     desc => $_->getDesc(),
			     type => $_->getType(),
			     qual => $_->getQualifier(),
			     hidden => $_->isHidden(),
			     );    
	$field->setRequired( required => $_->isRequired() );
	$self->addField( name  => $field->getName(),
			 field => $field
			 );
    }

    push(@{$self->{$arg{table}}}, $arg{field});
    $t->exit(msg=>"Field added");
}


1;
