#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/main/lib/SMTP_Server.pm#1

# Don't buffer output
$| = 1;

#***********************************************************************
#
# NAME
#   SMTP_Server.pm - 
#
# SYNOPSIS
#   An HTTP server implemented via PERL.
#
# DESCRIPTION
#   This library provides a basic SMTP server under Perl.  It's currently
#       catered to ASUP, but can be easily modified for other needs.
#
# AUTHOR
#   dl-cluster-qa
#
# HISTORY
# 7/13/2009 Initial revision
# 8/19/2009 Added retry logic when creating server/binding to port
# 6/3/2010 Modified to use client spec IP rather than nslookup
#***********************************************************************

## @summary     SMTP_Server.pm

package SMTP_Server;

use strict;

use Options;
use Tharn qw(host snooze subtest $Parent $Log fatal_result);

use NATE::Result;
use Data::Dumper;
use Carp;
use SMTP_Daemon;
use Net::SMTP::Server::Client;
use Net::SMTP::Server::Relay;
use MIME::Parser;
use Data::Dumper;


sub new {
    my $this = shift;
    my $class = ref($this) || $this;
    my %options = @_;
    my @subArgs;
    if (!$options{client}) {
        $Log->error("SMTP Server requires a client parameter \n");
        return 0;
    }
    if ($options{log_dir}) {
        $options{asup_dir}=$options{log_dir};
    }
    if ($options{asup_dir}) {

        @subArgs = ('{use SMTP_Server; SMTP_Server::run(asup_dir=>"'.$options{asup_dir}.'"'.
                                                 ',client=>"'.$options{client}.
                                                 '",ipv6=>"'.$options{ipv6}.'"); }',
                                                 '-bg', '-username', 'root', '-client', "$options{client}");
    }
    else {
        @subArgs = ('{use SMTP_Server; SMTP_Server::run(client=>"'.$options{client}.
                                                        '",ipv6=>"'.$options{ipv6}.'"); }',
                    '-bg', '-username', 'root', '-client', "$options{client}");
    }
    my $subtest = subtest @subArgs;
    my $self = {
        "subtest" => $subtest,
        "client" => $options{client},
        "asup_dir" => $options{asup_dir},
        "ipv6" => $options{ipv6},
        @_,
    };
    bless($self, $class);
    return $self;
}

sub is_ready {
    my $me = (caller(0))[3];
    my $self = shift;
    my %opts = (
        timeout => 60,
        sleep   => 1,
        check   => 'Server now ready for connections',
        @_
    );

    my $subtest = $self->{subtest};
    my $runid   = $subtest->runid();
    my $server_file = $subtest->logfile();

    my $timeout = $opts{timeout};
    my $sleep   = $opts{sleep};
    my $check   = $opts{check};

    my $end = time() + $timeout;
    while ( time() < $end ) {
        if ( -e $server_file && -f $server_file ) {
            my $result = `grep "$check" $server_file`;
            chomp( $result );

            if ( $result =~ /$check/ ) {
                $Log->comment("$me: Server $runid is ready");
                return 1;
            }
        }
        Tharn::sleep( $sleep );
    }

    die "$me: Server with runid $runid did not become ready within $timeout seconds";
}

sub ready {
    my $self = shift;
    my $message;
    my $i = 0;
    while ($i++ < 10) {
       eval { $message=$self->{"subtest"}->message_get(type => "VERIFY",
       timeout => 3);
       };
       if ($message =~ /SMTPREADY/) {
           $Log->comment("SMTP server started\n");
           return 1;
       }
       Tharn::sleep(20);

    }
    $Log->comment("Failed to start SMTP server\n");
    return 0;
}

sub stop {
    my $self = shift;
    $Log->trace("Stopping the server...\n");
    $self->{"subtest"}->kill();
    return;
}

sub get_entry {
    my $self = shift;
    my $message=undef;

    #get the single message from the queue
    #$message=$self->{"subtest"}->message_get(filter => sub{$_[0]->{message} =~ qr/SMTP/}, type => "SMTP", timeout => 3);
    $message=$self->{"subtest"}->message_get(type => "SMTP", timeout => 3);
    $Log->debug("Found raw message:  ".Dumper($message)."\n");
    return $message;
}

sub get_all {
    my @msgConcat = ();
    my $msgTmp = undef;

    while (1) {
        $msgTmp=undef;
        #get an entry that is on the queue, a message reported from the server
        eval {
            $msgTmp = get_entry(@_);
        };
        #if no message available, then exit
        if (!defined $msgTmp) {
            last;
        }
        #else add the message to the array
        else {
            $Log->debug("Adding message to array: ".Dumper($msgTmp));
            push @msgConcat, $msgTmp;
        }
    }
    #return the array of messages
    return \@msgConcat;
}

sub run {
    my %params = (
            client=>undef,
            asup_dir=>undef,
            ipv6=>undef,
            @_,
    );
    if (!$params{client}) {
        return fatal_result("SMTP Server run requires a client parameter \n");
    }

    my $addr;
    
    # BURT 629776 - sendmail by default bound to port 25 using SMTP
    # need to stop sendmail to free up the port
    $Log->comment("Attempting to stop sendmail service...");
    system ("sudo /sbin/service sendmail stop");    

    if ( $params{ipv6} ) {
        $addr = $params{ipv6};
    } else {
        my $clientobj = host($params{client});
        $addr = $clientobj->default_ip();
    }
    
    my @name;
    my $emailcounter=0;
    $Log->comment("Creating SMTP Server with Address: $addr, port 25\n");
    my $emailserver = SMTP_Daemon->new($addr, 25) || undef;
    if (!defined($emailserver)) {
        for (my $retries = 1; (!defined ($emailserver) && $retries < 11); $retries++) {
            $emailserver = SMTP_Daemon->new($addr, 25) || undef;
            $Log->comment("Try $retries: Waiting 6 seconds for SMTP server to start on port 25...Error: $!\n");
            snooze(6);
        }

        if (!defined($emailserver)) {
            $Log->error("Could not bind to port or server failed to start...\n");
            return fatal_result("No server, unable to handle connetions");
        }
    }
    $Log->comment("SMTP Server now ready for connections, use CTRL+C to stop process\n\n");
    $Parent->message_put(type => "VERIFY", message => "SMTPREADY");

    while(my $conn = $emailserver->accept()) {
        my $client = new Net::SMTP::Server::Client($conn) || croak("Unable to handle client connection: $!\n");
        $client->process || next; # Process the client.  This command will block until the connecting client completes the SMTP transaction.
        #Add the data to a file
        $Log->comment("EMAIL Received...\n");
        $emailcounter++;
        my $dir=undef;
        if (defined $params{asup_dir}) {
            if ( !-d $params{asup_dir} ) {
                $Log->comment("SMTP_Server: Creating non-existant directory ".$params{asup_dir});
                system ("mkdir $params{asup_dir}");
            }
            $dir=$params{asup_dir}."/SMTP-".$emailcounter;
            if ( !-d $dir ) {
                $Log->comment("SMTP_Server: Creating non-existant directory ".$dir);
                system ("mkdir $dir");
            }
            my $filename="$dir/email";
            $Log->comment("Saving email and attachments to directory: $dir\n");

            open (datafile,">$filename");

            # Make datafile unbuffered
            select datafile; $| = 1;

            print datafile "SENDER: $client->{FROM}\n";
            print datafile "RECEIVER: @{$client->{TO}}\n";
            print datafile "MESSAGE: $client->{MSG}\n";

            close (datafile);

            if (@name=$client->{MSG} =~ /Content-Disposition: attachment; filename=\"(.*)\"/g) {
                $Log->comment("Email Attachments found: @name\n\n");
            }
            else {
             $Log->comment("No attachments found!\n\n");
            }

            #this will save all of the attachments using their filenames as included in the email
            #it will also save the message text in a file titled msg-2816-1.txt
            my $parser= new MIME::Parser;
            $parser->output_dir($dir);
            $parser->output_to_core; 
            open  (INPUT, "$filename");
            $parser->read(\ *INPUT);
            close (INPUT);

        } 

        # Search for host within ASUP to verify      
        if ($client->{MSG} =~ /Subject: (.*)\r\n/) {
            $Log->comment("Subject: $1, Saved to directory $dir \n");
            $Parent->message_put(type => "SMTP", message => "SMTP: Email received, $1 . Saved to directory $dir");
        }

        $Log->comment("I've now received $emailcounter total EMAIL requests\n");
    }
}


1;


=pod

=head1 NAME

SMTP_Server.pm - Basic SMTP server.


=head1 SYNOPSIS

    use SMTP_Server;
    my $smtp_server = new SMTP_Server(client => $params{client}, asup_dir=>$params{asup_save_dir});
    my $single_post = $smtp_server->get_entry();
    my $posts = $smtp_server->get_all();
    $smtp_server->stop();

=head1 DESCRIPTION

This library provides a basic SMTP server in perl.
  
    
=head1 METHODS

new(client=>CLIENT);
        Description:
            Creates a new instance of the SMTP_Server object.  It will bind to
                port 25 on the specified client.
        Inputs:
            client => the client to run this SMTP_Server on.
            asup_dir => directory to save asups to
        Outputs:
            SMTP_Server object instance
        Examples:
            <see synopsis above>
            
stop();
        Description:
            Stops an SMTP_Server instance
        Inputs:
            <none>
        Outputs:
            <none>
        Examples:
            <see synopsis above>
            
get_entry();
        Description:
            Gets a single entry that was sent to the SMTP Server
        Inputs:
            <none>
        Outputs:
            $post => A single entry that was sent to the server.
        Examples:
            <see synopsis above>

get_all();
        Description:
            Get all entries that were sent to the SMTP Server
        Inputs:
            <none>
        Outputs:
            $posts => All entries that were sent to the server.
        Examples:
            <see synopsis above>
            
run(client=>CLIENT);
        Description:
            The main SMTP server method.  This method will bind to a specific
                port on the specified client and listen for incoming request,
                like an SMTP server.  When an email is received, it will parse
                the email for header information (subject, from, etc.) and will
                place certain email information on the SMTP server queue to be
                read by a client application.  Also if requested, the server
                method will parse off the email request and save the full
                request to a location (decoding mime attachments, etc.)
        Inputs:
            client => client to run the server on
            asup_dir => the directory to save off the email request
        Outputs:
            <none>
        Examples:
            <see synopsis above>
=cut

