#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/ontap/main/test/lib/SPARSE/Util/Utilities.pm# $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary	Library that defines general purpose utility routines
## @description
##	File copying, socket operations, miscellaneous parsing, simulator
##      booting functions
##
## @author dl-sparseeng

package SPARSE::Util::Utilities;

use strict;
no strict "subs";

use Carp;
use IO::Handle;
use IO::Socket::INET;
use Socket;
use POSIX qw(:sys_wait_h strftime);
use POSIX;
use SPARSE::Util::Logger;
use config_base qw(host_is_alive);
use SPARSE::Constants;
use Tharn;
use Data::Dumper;
# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&uStatus
		&uSystem
		&uCommand
		&uDate

		&uParams
		&uParamParse
		&uParamFile
		&uParamList

		&uReadHeaders
		&uChkVals
		&uStrHi
		&uDiffDFValues

		&uCopy
		&uSeqStr &uSendSeq &uReadSeq
		&uSendBlk

		&uGetSrcIP
		&uGetFQDN
		&uSoShutdownReader
		&uSoShutdownWriter
	    );

# ------------------------------------------------------------------------
#= Miscellaneous utilities

# OBSOLETE -- use uCommand() instead
#= Convert process exit status to a legible error string - or undef if 0
#= :status like $?
#= >undef|result string
sub uStatus($)
{
	my $status = shift;
	my $tmp;

	if (WIFEXITED($status)) {
		$tmp = WEXITSTATUS($status);
		if ($tmp) {
			return $tmp;
		}
		return undef;
	}
	
	if (WIFSIGNALED($status)) {
		$tmp = WTERMSIG($status);
		return $tmp;
	}

	return $status;
}

# OBSOLETE -- use uCommand() instead
#= Run a shell command and report its exit status or verify its output
#= :command	command to run
#= :[cond]	output success expr "=~ /good/" or "!~ /fail/"
#= >undef|error string
sub uSystem($;$)
{
	my ($cmd, $exp) = @_;

	logTrc("Running command: $cmd");

	if (!defined($exp)) {
		my $pid = fork();
		logError("fork failed: $!") unless defined($pid);

		if ($pid == 0) {	# child
			exec($cmd);
			#logError("exec($cmd): $!");
		}

		if (($_ = waitpid($pid, 0)) != $pid) {
			logError("waitpid found wrong child: $_  expect $pid");
		}

		return uStatus($?);
	}

	# $exp is defined from here on.
	# grab stout and stderr
	open(RES, "$cmd 2>&1 |")	or logError("could not start $cmd: $!");
	my @res = <RES>;
	close(RES);
	logDbg("$cmd: \"@res\"");

	# check against expected output
	my $exc = $@;	# preserve any current exception
	my $match = 0;
	if (!@res) {
		logError("bad result from \"$cmd\": \"$_\"");
	}
	for (@res) {
		if (eval("q{$_} $exp")) {
			$match = 1;
			last;
		}
	}
	logError("bad result from \"$cmd\": \"@res\"") if $match == 0;

	$@ = $exc;
	return undef;
}

## @name	::uCommand
## @summary	run command on host and check the result
## @arg		cmd required
## @arg		regexp optional must-match pattern
## @arg		regexp optional must-not-match pattern
## @arg		exit-code optional must-exit
## @arg		cmd-output optional ref to scalar
## @return	exit-code
## @example
#	$out = uCommand("ls -l", "total", undef, 0, \$ret);
sub uCommand($;$$$$)
{
	my ($cmd, $must, $mustnot, $mustexit, $output) = @_;

	logTrc("Run \"$cmd\"");

	# grab stdout and stderr
	open(CMD, "$cmd 2>&1 |")	or logError("could not start $cmd: $!");
	my $out = join("", <CMD>);
	close(CMD);
	my $ret = $?;
	logDbg("$cmd returned $ret: \"$out\"");

	# check the reply for must-match or must-not-match and exit
	if (defined($must)	  && $out !~ /$must/
	    || defined($mustnot)  && $out =~ /$mustnot/
	    || defined($mustexit) && $ret != $mustexit) {

		logError("unexpected result from \"$cmd\"\nRequired:"
			. (defined($must)     ? "\nmatch \"$must\"" : "")
			. (defined($mustnot)  ? "\n!match \"$mustnot\"" : "")
			. (defined($mustexit) ? "\nexit=$mustexit" : "")
			. "\nResult:\nexit=$ret\noutput=\"$out\"")
	}
	$$output = $out if defined($output);
	return $ret;
}

#= Generate date format based on epoch seconds
#= :time	epoch time eg time()+30
#= >date string
sub uDate($)
{
	return strftime("%a, %d %b %Y %H:%M:%S GMT", gmtime($_[0]));
}

#= Diff 'df' values
#= :val1 	hash ref 1
#= :val2	hash ref 2
#= :[pcdrift]  max acceptable percentage deviation among val1 and val2
sub uDiffDFValues($$;$)
{
	my ($val1, $val2, $pcdrift) = @_;
	$pcdrift = $pcdrift || 10;

	for my $attr (keys(%{$val1})) {
		if ($val1->{$attr} =~ /(\d+)%/) {
			$val1->{$attr} = $1;
		}
		if ($val2->{$attr} =~ /(\d+)%/) {
			$val2->{$attr} = $1;
		}
	}

        logTrc("comparing ". Dumper($val1) ." and ". Dumper($val2));
		
	for my $attr (keys(%{$val1})) {
		if ((($val1->{$attr}-$val2->{$attr})/$val1->{$attr}) * 100
			> $pcdrift) {
			logError("discrepancy in df values: $val1->{$attr} and".
				 " $val2->{$attr} differ by more than".
				 " $pcdrift%");
		}
	}

	return undef;
}

# ------------------------------------------------------------------------
#= Parameter utilities

# Parse and build a params hash
#	assumes all params have defaults (changes must be to a known name)
#	cmd line options override file options which override defaults
sub uParams($;$$)
{
	my ($defaults, $argsfile, $args) = @_;
	$argsfile = $main::ARGSFILE	unless $argsfile;
	$args     = $main::ARGS		unless $args;

	my %av;
	logTrc("param setting: default=\"$defaults\" "
		. "file=\"$argsfile\" args=\"$args\"");
	uParamParse($defaults, \%av);
	uParamFile($argsfile, \%av, \%av)       if $argsfile;
	uParamParse($args, \%av, \%av)          if $args;
	return \%av;
}

# Parse argument string of name-value pairs into a hash:
# eg.  "foo=bar,tenk=10KB,fourk=0x1000,multi=-foo -bar,foo+=foo,tenm=0xaMB"
#
# $argv->{fourk}  = 4096
# $argv->{foo}    = bar foo
# $argv->{multi}  = -foo -bar
# $argv->{tenk}   = 10240
# $argv->{tenmeg} = 10485760
#
# XXX commas in a value are verboten
#
sub uParamParse($$;$)
{
	my ($str, $argv, $keys) = @_;

	logDbg("setting param from \"$str\"");
	for (split(/,/, $str)) {

		my ($n, $o, $v) = m@(\w+)\s*(\+?=)\s*(.*)@
				or logError("bad param: \"$_\" in $str");
		if ($keys && !defined($keys->{$n})) {
			logError("unknown name \"$n\" names="
				 . join(' ', keys(%{$keys})))
		}
		my $mult;
		# number followed by scale
		if (my ($u) = $v =~ m@^(?:(?:0x[\da-fA-F]+)|\d+)([KMG])B?$@i) {
			$v =~ s/[KMB]B?$//i;
			$mult = ($u =~ /^[Kk]/ ? 1024
				: ($u =~ /^[Mm]/ ? 1024*1024
				    : 1024*1024*1024));
		}
		$v = oct($v) if $v =~ /^0/;
		$v *= $mult  if $mult;

		if ($o eq "+=") {
			$argv->{$n} .= " $v";
		} else {
			$argv->{$n} = $v;
		}
		logDbg("  set $n to $argv->{$n}");
	}
	return $argv;
}

# Parse argument file of name-value pairs into a hash:
# eg:
#	# file comment
#	foo=bar
#	tenk=10KB
#
sub uParamFile($$;$)
{
	my ($fname, $argv, $keys) = @_;
	my $str;

	logDbg("getting params from $fname");
	open(FH, $fname)	or logError("open $fname failed: $!");
	while (<FH>) {
		next if /^#|^$/;	# skip comments and blank lines
		chomp;
		$str .= "$_,";
	}
	close(FH);
	chop($str);			# eat last comma
	return uParamParse($str, $argv, $keys);
}

# Helper to pretty-print arg lists
#	take the hash, sort the names and create string name=value pairs
#	one pair to a line indented by a tab
sub uParamList($)
{
	my ($av) = @_;

	return "ARGS :\n\t"
		. join("\n\t",
			map { "$_=$av->{$_}" } sort(keys(%{$av})))
		. "\n";
}

# ------------------------------------------------------------------------
#= String utilities

#= Read and parse headers from stream
#= :fh		input stream
#= :hdr|undef	validation hash {hdr=>value}
#= >hash of header-values
sub uReadHeaders($;$)
{
	my ($fh, $chkHdr) = @_;
	logDbg("read headers fd=". fileno($fh));
	my %hdrs;

	my $MSGHDR = '^([^:]+):' . '\s*' . '(.*)\r\n$';
	while (<$fh>) {
		logTrc($_);
		last if (/^\r\n$/);

		next if ($_ !~ /$MSGHDR/o);
		$hdrs{$1} = $2;
	}
	uChkVals(\%hdrs, $chkHdr);
	return \%hdrs;
}

#= Verify hash
#= :data	input hash
#= :chk		validation hash
sub uChkVals($$)
{
	my ($data, $chk) = @_;

	return unless (defined($chk));	# optional
	for (keys(%{$chk})) {
		logDbg("check $_ is $chk->{$_}");
		$data->{$_} eq $chk->{$_}
			or logError("'$_' has bad value [$data->{$_}], "
				  . "expecting [$chk->{$_}]\n");
	}
}

#= Highlight multiline string for display
#= :str
#= :[prefix]
sub uStrHi($;$)
{
	my ($msg, $pfx) = @_;
	$pfx = "> " unless defined($pfx);

	chomp($msg);
	return $pfx . join("\n$pfx", split(/\n/, $msg)). "\n";
}

# ------------------------------------------------------------------------
#= Data movers
#= Copy byte streams to and fro.

my $BUFSIZ = 4096;

#= Copy all data until EOF
#= :fh-in	input stream
#= :fh-out	output stream
#= >bytes copied
sub copyUntilEOF($$)
{
	my ($r, $w) = @_;
	my $rlen = 0;
	my $buf;
	while (my $buflen = sysread($r, $buf, $BUFSIZ)) {
		$rlen += $buflen;

		my $wrote = syswrite($w, $buf);
		if (!defined($wrote) || $wrote != $buflen) {
			logError("bad write $wrote $! [$buflen]\n");
		}
	}
	return $rlen;
}

use Fcntl qw( F_GETFL F_SETFL O_NONBLOCK );;
#= Set/ unset blocking mode on stream
#= :fh		stream
#= :flags|0	set flags or enable O_NONBLOCK
#= >old flags
sub blocking($$)
{
	my ($fh, $flags) = @_;

	my $oflags = fcntl($fh, F_GETFL, 0)
		or logError("fcntl F_GETFL failed: $!\n");
	if (!$flags) {
		# no flags:- set non-blocking mode
		$flags = $oflags | O_NONBLOCK;
	}
	fcntl($fh, F_SETFL, $flags) or logError("fcntl failed: $!\n");
	return $oflags;
}

#= Copy all outstanding data
#= :fh-in	input stream
#= :fh-out	output stream
#= >bytes copied
sub copyUntilEmpty($$)
{
	my ($r, $w) = @_;

	my $rlen = 0;
	my $flags = blocking($r, 0);

	while (1) {
		my $buf;

		my $buflen = sysread($r, $buf, $BUFSIZ);
		last if ($buflen == 0);
		$rlen += $buflen;

		my $wrote = syswrite($w, $buf);
		if (!defined($wrote) || $wrote != $buflen) {
			logError("bad write $wrote $! [$buflen]\n");
		}

	}
	blocking($r, $flags);
	return $rlen;
}

#= Copy data from one file descriptor to another
#= :fh-in	input stream
#= :len		"all", "pending" or byte count
#= :fh-out	output stream
#= >bytes copied
sub uCopy($$$)
{
	my ($r, $len, $w) = @_;
	logDbg("copy data: ".fileno($r). " -> ". fileno($w). " $len bytes");

	return copyUntilEOF($r, $w) if ($len eq "all");
	return copyUntilEmpty($r, $w) if ($len eq "pending");
	logError("bad length to read $len\n") unless ($len =~ /\d+/);

	my $rlen = 0;
	while ($len > 0) {
		my $buf;
		sysread($r, $buf, ($len > $BUFSIZ) ? $BUFSIZ : $len)
			or logError("copy read failed: $rlen $!\n");
		$rlen += length($buf);

		my $wlen = syswrite($w, $buf);
		(defined($wlen) and $wlen == length($buf))
			or logError("copy write failed: $wlen $!\n");

		$len -= length($buf);
	}
	return $rlen;
}

# ------------------------------------------------------------------------
#= Data sequences
#= A sequence is a printable ASCII byte pattern which can be validated
#= given the initial seed.  It is typically used in place of raw data.
#= We use 80 bytes of ASCII data.

my $seq = "0123456789abcdef"	# 16 * 5 == 80 bytes
	. "ghijklmnopqrstuv"
	. "wxyzABCDEFGHIJKL"
	. "MNOPQRSTUVWXYZ@#"
	. "&%+-*/=?<([{}])>";

# extract sequences from this buffer
my $seqBuf = $seq x ($BUFSIZ / length($seq) + 1);
my $seqBufLen = length($seqBuf);

#= Get sequence substring
#= :offset	sequence offset
#= :len		bytes
#= >sequence substring
sub uSeqStr($$)
{
	my $seqoff = $_[0] % length($seq);
	my $str = substr($seqBuf, $seqoff, $_[1]);
	my $len = $_[1] - length($str);
	while ($len > $seqBufLen) {
		$str .= $seqBuf;
		$len -= $seqBufLen;
	}
	$str .= substr($seqBuf, 0, $len);
	return $str;
}

#= Write a sequence string to a socket
#= :offset	sequence offset
#= :len		byte count
#= :fh		output stream
#= >final sequence offset
sub uSendSeq($$$)
{
	my ($seqoff, $len, $w) = @_;

	logDbg("SendSeq: seqoff=$seqoff [" . ($seqoff % length($seq))
		. "] len=$len fd=". fileno($w));
	while ($len) {
		my $towrite = ($len > $BUFSIZ) ? $BUFSIZ : $len;

		my $str = &uSeqStr($seqoff, $towrite);
		logInfo("Seq write @ $seqoff: \"$str\"");

		my $wlen = syswrite($w, &uSeqStr($seqoff, $towrite));
		logError("seq write failed: $!\n") if (!defined($wlen));

		$len -= $wlen;
		$seqoff += $wlen;
	}
	return $seqoff;
}

my $BLKSIZE = 4096;

#= Write 4KB of sequence string to a socket
#= :seqoff	sequence offset
#= :fh		output stream
#= :seqoff_dup	repeat count of the byte on the seqoff inserted at the 
#=		beginning of the 4KB string. This is used to generate
#=		more unique 4KB sequence string with the 80 bytes sequence.
#=    For example, if the seqoff is 3 and seqoff_dup is 4, the beginning
#=    of the 4KB string will be:
#=      33333456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ
#=      @#&%+-*/=?<([{}])>0123456789abcdefghijklmnopqrstuvwxyz
#=	ABCDEFGHIJKLMNOPQRSTUVWX........
#=    The byte at the seqoff 3 is '3', it is repeated 4 times before the
#=    regular sequence string pattern (when seqoff_dup is 0).
#=
#= >number of bytes written
sub uSendBlk($$;$)
{
        my ($seqoff, $w, $seqoff_dup) = @_;
        $seqoff_dup = 0 unless (defined($seqoff_dup));
        my $towrite = $BLKSIZE;

        logDbg("SendBlk: seqoff=$seqoff [" . ($seqoff % length($seq))
                . "] fd=". fileno($w) . " seqoff_dup=$seqoff_dup");

        my $str = '';
        for ( my $i = 0; $i < $seqoff_dup; $i++) {
                $str .= substr($seqBuf, $seqoff, 1);
        }
        $str .= substr($seqBuf, $seqoff, $towrite - $seqoff_dup);
        my $len = $towrite - length($str);
        while ($len > $seqBufLen) {
                $str .= $seqBuf;
                $len -= $seqBufLen;
        }
        $str .= substr($seqBuf, 0, $len);
	my $strlen = length($str);
        logInfo("Seq write $strlen @ $seqoff: \"$str\"");

        my $wlen = syswrite($w, $str);
        logError("seq blk write failed [$wlen]: $!\n")
                        if (!defined($wlen) || $wlen != $BLKSIZE);
	return $wlen;
}

#= Read and verify a sequence
#= :fh		input stream
#= :offset	sequence offset
#= :len		"all" or byte count
#= >final sequence offset
sub uReadSeq($$$)
{
	my ($r, $seqoff, $len) = @_;

	$len = -1 if ($len eq "all");
	logDbg("ReadSeq: seqoff=$seqoff [" . ($seqoff % length($seq))
		. "] len=$len fd=". fileno($r));
	while ($len) {
		my $buf;
		sysread($r, $buf, ($len > $BUFSIZ or $len == -1) ? $BUFSIZ : $len);
		my $rlen = length($buf);

		if ($rlen == 0) {
			last if ($len == -1);
			logError("seq read failed: seqoff=$seqoff $!");
		}
		$len -= $rlen unless ($len == -1);
		logInfo("Seq read @ $seqoff: \"$buf\"");

		# check first byte
		my $rseq = substr($buf, 0, 1);
		my $xseq = &uSeqStr($seqoff, 1);
		($rseq eq $xseq)
			or logError("seq mismatch \"$rseq\" != \"$xseq\"");

		$seqoff += $rlen;
	}
	return $seqoff;
}

# ------------------------------------------------------------------------
#= Networking Operations

#= Try to connect to a host and find the source ip-address of the connected
#= socket.
#= :target	name or ipaddr
#= :port	port
#= >stringified ipaddr
sub uGetSrcIP($$)
{
	my ($target, $tgtport) = @_;
	
	my $socket = IO::Socket::INET->new("$target:$tgtport") 
		or logError("couldn't connect to '$target:$tgtport': $!");

	my $saddr = getsockname($socket);
	close($socket);
	my ($port, $ipaddr) = sockaddr_in($saddr);
	return inet_ntoa($ipaddr);
}

#= Look up host name and extract FQDN (for netapp.com) from response
#= :name	hostname
#= >FQDM
sub uGetFQDN($)
{
	my $host = $_[0];
	my ($name, $aliases) = gethostbyname($host);

	for (($name, split(' ', $aliases))) {
		return $_ if /\.netapp\.com/;
	}
	logError("cannot get FQDN for $host");
}

#= Socket Operations
#= closing read ends, write ends etc.

#= shutdown read end of a socket
sub uSoShutdownReader($)
{
	my ($socket) = $_[0];
	logDbg("shutdown read end fd=". fileno($socket));
	shutdown($socket, 0);
}

#= shutdown write end of a socket
sub uSoShutdownWriter($)
{
	my ($socket) = $_[0];
	logDbg("shutdown write end fd=". fileno($socket));
	shutdown($socket, 1);
}


# ------------------------------------------------------------------------
1;
# ------------------------------------------------------------------------
