#Hey emacs, drop dead
#
# $Id: //depot/prod/test/Rsierranevada/lib/SPARSE/Util/Zapi_Utils.pm#1 $
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary	Library that defines convenient zapi routines
## @description
##	This package provides procedural API for managing the filer
##	using the ZAPI
##
## @usage
##	use Zapi_Utils;
##
## @author dl-sparseeng

package SPARSE::Util::Zapi_Utils;

use strict;
use Tharn;
use NaElement;
use SPARSE::Constants;
use SPARSE::Resource::Filer;
use SPARSE::Resource::Volume;
use SPARSE::Util::Logger;
use SPARSE::Util::Utilities;
use Data::Dumper;

no warnings("uninitialized");

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
	        &zVolInfoList
	        &zAggrInfoList
		&zInvoke
		&zTree
	    );

# ------------------------------------------------------------------------

$::ZAPI_snoop = 1 if (logCond("dbg"));	# extra debug from ZAPI

sub zVolInfoList($)
{
	my ($filer) = @_;

	my $zv = $filer->{z}->invoke("volume-list-info")
		or logError("no vol list info");
	logInfo($zv->sprint_pretty(" "));

	return (@{$zv->child_get("volumes")->{children}});
}

sub zAggrInfoList($)
{
	my ($filer) = @_;

	my $zv = $filer->{z}->invoke("aggr-list-info")
		or logError("no aggr list info");
	logInfo($zv->sprint_pretty(" "));

	return (@{$zv->child_get("aggregates")->{children}});
}


# ------------------------------------------------------------------------

#
# my $res = zInvoke($filer, "stat qtrees on $name",
#		    "qtree-list", "volume" => $name);
sub zInvoke(@)
{
	my $filer = shift;
	my $msg   = shift;

	logTrc("$filer->{hostName} $msg : @_");
	my $r = $filer->{z}->invoke(@_);
	logInfo($r->sprint_pretty(" "));
	logError("$msg failed: ". $r->results_reason()) unless $r->results_ok();
	return $r;
}

#
# zTree([ "nfs-exportfs-append-rules",
# 		[ "persistent", "true" ],
# 		[ "rules",
# 			[ "exports-rule-info",
# 				[ "pathname", "/vol/foo" ],
# 				[ "anon", "0" ]
# 			]
# 		]
# 	] );
# 
sub zTree($)
{
	my ($ref) = shift;
	my $name = shift(@{$ref});

	my $root = new NaElement($name);

	for my $elt (@{$ref}) {
		if (!ref($elt)) {
			$root->content_add($elt);

		} elsif (ref($elt) eq "ARRAY") {
			$root->child_add(zapiTree($elt));
		} else {
			logError("elt: $elt");
		}
	}
	return $root;
}


# ------------------------------------------------------------------------
1;
# ------------------------------------------------------------------------
