#$Id: //depot/prod/test/main/lib/SPARSE/cmode/ClientObject.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.

## @summary	
##      Library which defines Client Object properties 
##      and actions.
## @description
##	Abstract common object tasks.
##
## @usage
##	use SPARSE::cmode::ClientObject;
##
## @author mica@netapp.com

package SPARSE::cmode::ClientObject;

use strict;
use Tharn;
use filer;
use mount();
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use GenericMount qw(mount umount);
use MapShare qw(map_cifs_share unmap_cifs_share);


# Internal reference classes 
use SPARSE::cmode::AggrObject qw();
use SPARSE::cmode::VserverObject qw();
use SPARSE::cmode::ClientOpsObject_Win32 qw(clientOpsWin32);
use SPARSE::cmode::ClientOpsObject qw(clientOps);

#NACL libs
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&client
		);

# ------------------------------------------------------------------------
## Client class definition

## @name	::client
## @summary	create a linux/solaris/windows client object
## @description
##	Create a client object.
## @arg		opts optional
## @return	client object
				
sub client(%)
{
   my %fparams = validate(
        @_,
        {
           name  => { type => SCALAR  },
           opts    => { type => HASHREF },
        });

   my $client = {};
   my $clientops= {};
   my $clientName = $fparams{name};
   my $opts = $fparams{opts};

   $client->{name}  = $clientName;
   $client->{hdl}     = $opts->{hdl};
   $client->{conn}   = $opts->{conn}; 
   $client->{type}   = $opts->{type};
   $client->{os}   = $opts->{os};
   $client->{test}    = $opts->{test}; 
   $client->{hostname}    = $opts->{hostname}; 
   $client->{clientops} = $clientops;

   bless($client);
	
   #$opts->{test}->nlog("client object created : " . Dumper($client));
   return $client;
}

sub DESTROY($)
{
	my ($client) = @_;
	my $exc_state = $@;
	$client->{test}->nlog("destroying $client->{name} object");
	$@ = $exc_state;
}


# ------------------------------------------------------------------------
## client class methods

sub mount_nfsv3 ($$) {
         my ($client, $opts) = @_ ; 
         my $test                    = $client->{test};
         my $clientHdl            = $client->{hdl};
         my $filerDataIp          = $opts->{filerdataip};
         my $filerJunctionPath = $opts->{filerjunctionpath}; 
         my $clientMountPath = $opts->{clientmountpath}; #what we get from user
         my $mountOpts         = $opts->{mntopts};
         my $clientOpsName   = $opts->{clientopsname};
         my $mntPoint;my $i;
         my $retrycount = 20;  

         TOP:
      	 for($i=1;$i<=$retrycount;$i++) {
         $test->nlog("Nfs Mount command on attempt $i"); 
        
	  #  eval {        
               $mntPoint = mount($filerDataIp,$filerJunctionPath,
            									   client => $clientHdl,
										   mountretry => "10",
            									   mntopts => $mountOpts);
												   
			$test->nlog( "Mount point: $mntPoint ");

	   #}; 
	     if ($@) {
               $test->nlog("Mount is unsuccessful ... on attempt $i Reason: $@");
            }	    


	    if (defined($mntPoint)) {
               $test->nlog("Mount is successful ... on attempt $i"); 
	       last TOP;
	    }
		
		sleep 5;
	 }
	 
	 if (!defined($mntPoint)) {
               $test->nlog("Mount is not successful even after ... $retrycount attempts"); 
               return 0;
         }

         $opts->{clientmountedpath} = $mntPoint; # what we get from call 
         my $clientOpsObj = clientOps(name => $clientOpsName ,
			opts => $opts, client => $client);
         $client->{clientops}->{"$clientOpsName"} = $clientOpsObj;
		$test->nlog("New Mount point is :: $mntPoint"); 

         # if ($@) {
	    # $test->nlog("Cannot mount the volume : Epic FAIL in lib : $@"); 
            # return 0;
         # }

         return 1;
}

sub addClientOpsObject ($$) {
         my ($client, $opts) = @_ ; 
         my $test                    = $client->{test};
         my $clientHdl            = $client->{hdl};
         my $filerDataIp          = $opts->{filerdataip};
         my $filerJunctionPath = $opts->{filerjunctionpath}; 
         my $clientMountPath = $opts->{clientmountpath}; #what we get from user
         my $mountOpts         = $opts->{mntopts};
         my $clientOpsName   = $opts->{clientopsname};
         my $mntPoint;

         eval {
	    $opts->{clientmountedpath} = $clientMountPath; 
	    my $clientOpsObj = clientOps(name => $clientOpsName ,
			opts => $opts, client => $client);
	    $client->{clientops}->{"$clientOpsName"} = $clientOpsObj;
	    $test->nlog("Added Mount point is :: $clientMountPath"); 
	 };
         if ($@) {
	    $test->nlog("Cannot add the existing clientops object : $@"); 
            return 0;
         }
         return 1;

}

sub mount_nfsv4 ($$) {
         my ($client, $opts) = @_ ; 
         my $test                    = $client->{test};
         my $clientHdl            = $client->{hdl};
         my $filerDataIp          = $opts->{filerdataip};
         my $filerJunctionPath = $opts->{filerjunctionpath}; 
         my $clientMountPath = $opts->{clientmountpath}; #what we get from user
         my $mountOpts         = $opts->{mntopts};
         my $clientOpsName   = $opts->{clientopsname};
         my $mntPoint;my $i;
         my $retrycount = 20;  

         TOP:
      	 for($i=1;$i<=$retrycount;$i++) {
         $test->nlog("Nfs Mount command on attempt $i"); 
        
	    eval {        
               $mntPoint = mount($filerDataIp,$filerJunctionPath,
            									   client => $clientHdl,
										   mountretry => "10",
            									   mntopts => $mountOpts);

	    }; 

	    if (defined($mntPoint)) {
               $test->nlog("Mount is successful ... on attempt $i"); 
	       last TOP;
	    }
 
            sleep 5;
	 }
	 
	 if (!defined($mntPoint)) {
               $test->nlog("Mount is not successful even after ... $retrycount attempts"); 
               return 0;
         }

         $opts->{clientmountedpath} = $mntPoint; # what we get from call 
         my $clientOpsObj = clientOps(name => $clientOpsName ,
			opts => $opts, client => $client);
         $client->{clientops}->{"$clientOpsName"} = $clientOpsObj;
	 $test->nlog("New Mount point is :: $mntPoint"); 

         if ($@) {
	    $test->nlog("Cannot mount the volume : Epic FAIL in lib : $@"); 
            return 0;
         }

         return 1;
}

sub umount_nfsv3 ($$) {
        my ($client, $opts) = @_ ; 
        my $test                    = $client->{test};
        my $clientHdl            = $client->{hdl};
        my $clientOpsName   = $opts->{clientopsname};
        my $clientOpsObj      = $client->{clientops}->{$clientOpsName}; # object here 
        my $result;

        eval {        
	         $test->nlog("Umount started for path : $clientOpsObj->{clientmountedpath}"); 
             $result = umount($clientOpsObj->{clientmountedpath},
            										force=>1, client => $clientHdl,
            									   );
        }; 

         if (!defined($result)) {
            	return 0;
         }
            
            # after we delete this mount point 
            # 1) we need to delete the ops object
            # 2) delete the reference in this class.
            delete $client->{clientops}->{$clientOpsName};
	        $test->nlog("Umount Done & ClientOps Object named : $clientOpsName deleted "); 
	        #$test->nlog("Client object is now : ". Dumper($client) ); 
            
        if ($@) {
	        $test->nlog("Cannot umount the volume : Epic FAIL in lib : $@"); 
            return 0;
        }

        return 1;
}

sub umount_nfsv4 ($$) {
        my ($client, $opts) = @_ ; 
        my $test                    = $client->{test};
        my $clientHdl            = $client->{hdl};
        my $clientOpsName   = $opts->{clientopsname};
        my $clientOpsObj      = $client->{clientops}->{$clientOpsName}; # object here 
        my $result;

        eval {        
	         $test->nlog("Umount started for path : $clientOpsObj->{clientmountedpath}"); 
                 $result = umount($clientOpsObj->{clientmountedpath},
            										force=>1, client => $clientHdl,
            									   );
        }; 

         if (!defined($result)) {
            	return 0;
         }
            
            # after we delete this mount point 
            # 1) we need to delete the ops object
            # 2) delete the reference in this class.
            delete $client->{clientops}->{$clientOpsName};
	        $test->nlog("Umount Done & ClientOps Object named : $clientOpsName deleted "); 
	        #$test->nlog("Client object is now : ". Dumper($client) ); 
            
        if ($@) {
	        $test->nlog("Cannot umount the volume : Epic FAIL in lib : $@"); 
            return 0;
        }

        return 1;
}

sub addClientOpsObj();
sub removeClientOpsObj();

sub map_share ($$){
		 
	my ($client, $opts) = @_ ;
	my $test                    = $client->{test};
        my $clientHdl            = $client->{hdl};
        my $hostname = $client->{hostname};   
        my $cifs_server			= $opts->{cifs_server_ip};
	my $cifs_share 			= $opts->{cifs_share};
	my $cifs_domain			= $opts->{cifs_domain};
	my $ad_user			= $opts->{ad_user};
	my $ad_password		= $opts->{ad_password};
	my $Drive;
	my $clientOpsName   = $opts->{clientopsname};
        my $retrycount = 5; my $i;
        my $Drive = "nil";
        my $c = Connectrec->new(host     => "$hostname",
                            template => "rsh-persist",
                            connect  => 1);
	
	
        my $output = $c->execute(command => " net use /persistent:no",
                 timeout => 60);
        $c->destroy();
		
		TOP:
      	        for($i=0;$i<=$retrycount;$i++) {
                  eval {
		           $Drive = map_cifs_share(server   => $cifs_server,
                                share    => $cifs_share,
                                domain   => $cifs_domain,
                                username => $ad_user,
                                password => $ad_password,
                                client   => $clientHdl,
                                drive    => "*",
                                timeout => '200',
			   ); 
                           if ($Drive =~/[a-z,A-Z]/) {
        	            $test->nlog("Mapped successfully on attempt $i"); 
		            last TOP;
		           }
		  };
	          if ($@) {
                           $test->nlog("Failed to map the drive at attempt $i."); 
	          }
                  
		  $test->nlog("Sleeping for 10 secs before retrying ... "); 
      	          sleep 10; 
		}         
		   
                if ($Drive eq "") {
 	            $test->nlog("Failed to map the drive even after $retrycount attempts .$@"); 
		    return 0;
		}
				
		$opts->{mappeddrive} = $Drive;  
	        #my $clientOpsObj = ClientOpsObject_win32::clientOps(name => $clientOpsName ,
	        #opts => $opts, client => $client);
	        my $clientOpsObj = SPARSE::cmode::ClientOpsObject_Win32::clientOpsWin32(name => $clientOpsName ,
	        opts => $opts, client => $client);
                $client->{clientops}->{"$clientOpsName"} = $clientOpsObj;
	        #$test->nlog("Mount point is :: $mntPoint"); 
	        #$test->nlog("Client object is now : ". Dumper($client) ); 
	        return 1;
							   
}


sub unmap_share($$)
{
	 my ($client, $opts) = @_ ; 
         my $test                  = $client->{test};
         my $hostname = $client->{hostname};    
         my $clientOpsName    = $opts->{clientopsname};
      	 my $drive = $client->{clientops}->{$clientOpsName}->{clientmappeddrive};

	 eval{
	 my $c = Connectrec->new(host     => "$hostname",
                            template => "rsh-persist",
                            connect  => 1);
	
	
	 my $output = $c->execute(command => "net use $drive /delete /yes",
                timeout => 60,
               
                );
	print "\nReply: $output\n";    
    $c->destroy();
	};
	 if ($@) {
               $test->nlog("Failed to unmap the drive ");
	       return 0;
	 }
	 delete $client->{clientops}->{$clientOpsName};
	 $test->nlog("Unmap Done & ClientOps Object named : $clientOpsName deleted "); 
	 return 1;
	 
}
      
1;
