#$Id: //depot/prod/test/main/lib/SPARSE/fcluster/ClientOpsObject.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.

## @summary	
##      Library which defines Client Ops Object properties 
##      and actions.
## @description
##	Abstract common object tasks.
##
## @usage
##	use SPARSE::cmode:ClientOpsObject;
##
## @author mica@netapp.com

package SPARSE::cmode::ClientOpsObject;

use strict;
use Tharn;
use filer;
use mount();
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
# our modded Filemanip Below ;)
use SPARSE::cmode::FclusterFileManip qw(check_file write_file read_file remove_file get_dir_entries get_file_entries
                 remove_dir stat_file make_dir setattr_file fh_Stat link_file mk_node rename_file file_truncate exec_cmd read_link open_file close_file);

# Internal reference classes 
use SPARSE::cmode::AggrObject qw();
use SPARSE::cmode::VserverObject qw();
use SPARSE::cmode::ClientObject qw();

#NACL libs
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);

use Errno qw(:POSIX);
use IO::File;
use IO::Handle;
use RichData;

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&clientOps
		);

# ------------------------------------------------------------------------
## Client class definition

## @name	::clientOps
## @summary	create a linux/solaris/windows client ops object
## @description
## Create a client ops object.
## @arg		opts optional
## @return	client object
				
sub clientOps(%)
{
    my %fparams = validate(
        @_,
        {
           name  => { type => SCALAR  },
           client  => { type => HASHREF },
           opts    => { type => HASHREF },
        });

    my $clientOps = {};
    my $clientOpsName   =  $fparams{name};
    my $opts                   =  $fparams{opts};
    my $client                 =  $fparams{client};  
   
    $clientOps->{name}  =  $clientOpsName;
    $clientOps->{filerjunctionpath}   =    $opts->{filerjunctionpath};
    $clientOps->{clientmountedpath}   =  $opts->{clientmountedpath};
    $clientOps->{client}  =  $client;
    $clientOps->{type}  =  "unix";
    
    #$client->{test}->nlog("clientOps object created : " . Dumper($clientOps));

    bless($clientOps);
    return $clientOps;
}

sub DESTROY($)
{
	my ($clientOps) = @_;
	my $exc_state = $@;
	$clientOps->{client}->{test}->nlog("destroying $clientOps->{name} object");
	$@ = $exc_state;
}


# ------------------------------------------------------------------------
## client class methods
#fioOpen($dvol, "$dirname/$filename", "+>", undef, 0777, EACCES);
sub fioOpen ($$) {
        use Errno qw(:POSIX);
        my ($clientOps, $opts) = @_ ; 

        my $fileName  = $opts->{filename};
        my $client    = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my $filerjunctionPath = $clientOps->{filerjunctionpath};

        my $fileMode  = (defined($opts->{mode})  ? $opts->{mode} : '+<');
        my $fileMask  = (defined($opts->{mask})  ? $opts->{mask} : 0000); 
        my $filePerms = (defined($opts->{perms}) ? $opts->{perms} : 0666); 
        my $err = (defined($opts->{error}) ? $opts->{error} : '0'); 
        my $user      = (defined($opts->{user})  ? $opts->{user} : 'root'); 

        my ($got,$op,$expected);
        my $ret=1;
        my $retHash = ();

        eval {        
 	         $client->{test}->nlog("Running fioOpen as user:($user) ### open flags : $fileMode ### perms : $filePerms"); 
             $got = check_file( file => "$mountPath/$fileName", 
             			        client => $clientHdl,
             			        mode => $fileMode,
             			        perms=> $filePerms,
             			        mask => $fileMask, 
             			        user=> $user );

          	$op = "open file $mountPath/$fileName with open flags ";
		$expected = $err;   
             
		if (ref($got) ne "HASH"){
			$client->{test}->nlog("Error code: $got ");
				$got = $got + 0;
			if ($err) {
                		$client->{test}->nlog("Error got: $!");
		                my $res = _process_error($got, $err, $client);
				$ret = $res;
              			return;
		        	}
				
			$ret = 0;
			return 0;
				
		 }				
             # if a file is created succesfully , dont enter this check 
             if ($got->{fh} !~ /GLOB/) {      
          		$client->{test}->nlog("$op executed : got $got->{fh} : expected $expected , $got->{fh} = $expected");

				if ($got->{fh} && ($got->{fh} != $expected)) {
					# got unexpected error
					$client->{test}->nlog("$op got ".sprintf("%d", $got->{fh})
					." ($got->{fh}) instead of expected $expected ");
					$ret=0;
				}
     
				if ($expected && !$got->{fh}) {
					# expected error, but did not get it
					$client->{test}->nlog("$op succeeded when expected error $expected");
					$ret=0;
				}

			 } else {		
				$client->{test}->nlog("$op executed : got $got->{fh} : file created succesfully at $mountPath/$fileName");
                $retHash->{fh} = $got->{fh}; 
				$ret=$retHash;
             }
        }; 
        if ($@) {
	         $client->{test}->nlog("Cannot fioOpen File $@"); 
             $ret=0;
        }
 
    return $ret;
}

## fh related routines 
sub fhStatCheck ($$) {
    my ($clientOps, $opts) = @_ ; 

    my $fileName  = $opts->{filename};
    my $client    = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my $filerjunctionPath = $clientOps->{filerjunctionpath};

    #my $fileMode  = (defined($opts->{mode})  ? $opts->{mode} : '+<');
    my $fileMode  = $opts->{mode};
    my $fileMask  = (defined($opts->{mask})  ? umask($opts->{mask}) : ''); 
    my $filePerms = (defined($opts->{perms}) ? $opts->{perms} : '0666'); 
    my $fileError = (defined($opts->{error}) ? $opts->{error} : '0'); 
    my $user      = (defined($opts->{user})  ? $opts->{user} : 'root'); 

    my %content;
    my $ret=1;
    my $retHash;

    eval {        
        $retHash = fh_Stat(file => "$mountPath/$fileName", 
             			   client => $clientHdl,
             			   user=> $user );
	    $client->{test}->nlog("fh_Stat hash". Dumper($retHash)); 
	    #qw( dev ino mode nlink uid gid rdev
     	#    size atime mtime ctime blksize blocks );

        # strip mode of perms etc 
        my $realgivenmode = $fileMode & 07777;
        
        my $realmode = $retHash->{mode} & 07777;
        my $user_rwx = ($realmode & 00700) >> 6;
        my $target_bits = $user_rwx & 05;
        my $group_rwx = ($realmode & 00070) >> 3;
        my $other_rwx = ($realmode & 00007);

       	$client->{test}->nlog("given mode :: $fileMode,Got mode $retHash->{mode}, rwx is : $user_rwx$group_rwx$other_rwx"); 
        $ret=$retHash;
    }; 
    if ($@) {
	     $client->{test}->nlog("Filehandle stat cannot be run: $@"); 
         $ret=0;
    }

    return $ret;

}


sub createFile ($$) {
    my ($clientOps, $opts) = @_ ; 
        my $fileCount       = $opts->{filecount};
        my $fileSize         = $opts->{filesize}; 
        my $fileName = $opts->{filename};
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my @attrs;
	my $fixed_start;
	my $fixed_end;
  	#my $length_of_randomstring=59999;
	my $length_of_randomstring = (defined($opts->{stringlength}) ? $opts->{stringlength} : '59999'); 
 
    	my @chars=('a'..'z','A'..'Z','0'..'9','_','$','#');
	my $random_string;
        my $ret = 1;   

	if ( defined($opts->{fixed_start}) && $length_of_randomstring > 32 ) {
	    $fixed_start = "File Starts Here" ;
	    $fixed_end = "File Ends Here!!" ;
	    $length_of_randomstring = $length_of_randomstring -32;
	}
	
        foreach (1..$length_of_randomstring) {
	    $random_string.=$chars[rand @chars];
	}

	$random_string = $fixed_start . $random_string . $fixed_end;
        eval {        
             @attrs = write_file(file => "$mountPath/$fileName", 
             										contents => "$random_string", fstat =>1,
             										client => $clientHdl,
             										user=>$user);
	     $client->{test}->nlog("File $mountPath/$fileName created successfully its NIX stat are " . Dumper(@attrs) ); 
             $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Cannot create files in the mount : Epic FAIL in lib : $@"); 
             $ret=0;
        }

        return $ret;
}

sub modifyFile ($$) {
    my ($clientOps, $opts) = @_ ; 
        my $fileName = $opts->{filename};
        my $fileData = $opts->{data};
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my @attrs;
        my $ret = 1;   

        eval {        
             @attrs = write_file(file => "$mountPath/$fileName", 
             										contents => "$fileData", fstat =>1,
             										client => $clientHdl,
             										user=>$user);
	     $client->{test}->nlog("File $mountPath/$fileName modified successfully its NIX stat are " . Dumper(@attrs) ); 
             $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Cannot modify files in the mount : Epic FAIL in lib : $@"); 
             $ret=0;
        }

        return $ret;
}

sub readFile ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $fileCount = $opts->{filecount};
        my $fileSize   = $opts->{filesize}; 
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $fileName = $opts->{filename};
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my @content;
        my $ret = 1;
        eval {        
            @content = read_file(file => "$mountPath/$fileName",client => $clientHdl,user => $user);
	    #$client->{test}->nlog("File $mountPath/$fileName contents are read : " . Dumper(@content) . "NIX Read Successfully");                                 
	    $client->{test}->nlog("File $mountPath/$fileName contents are read : " . " NIX Read Successfully");                                 
            $ret=\@content;
        }; 

        if ($@) {
	    $client->{test}->nlog("Cannot read file in the mount : $@"); 
            $ret=0;
        }

        return $ret;
}

sub modifyFile ($$) {
    my ($clientOps, $opts) = @_ ; 
        my $fileName = $opts->{filename};
        my $fileData = $opts->{data};
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my @attrs;
        my $ret = 1;   

        eval {        
             @attrs = write_file(file => "$mountPath/$fileName", 
             										contents => "$fileData", fstat =>1,
             										client => $clientHdl,
             										user=>$user);
	     $client->{test}->nlog("File $mountPath/$fileName modified successfully its NIX stat are " . Dumper(@attrs) ); 
             $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Cannot modify files in the mount : Epic FAIL in lib : $@"); 
             $ret=0;
        }

        return $ret;
}

sub deleteFile ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $fileName = $opts->{filename};
        my $recursive = $opts->{recursive};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my @content;
        my $ret=1;

        eval {        
              @content = get_dir_entries(dir => "$mountPath" ,
                                        client => $clientHdl,
                                        user=>$user);
 	          $client->{test}->nlog("Dir listing : @content"); 

              @content = remove_file(file => "$mountPath/$fileName", 
             									           client => $clientHdl,
             									           recursive => 1,
             									           user => $user);
              @content = get_dir_entries(dir => "$mountPath" ,
                                        client => $clientHdl,
                                        user=>$user);
 	          $client->{test}->nlog("Dir listing : @content"); 

              $ret = 1;
        }; 

        if ($@) {
	         $client->{test}->nlog("Cannot delete file in the mount : $@ ::: end "); 
             $ret = 0;
        }
        return $ret;
}

sub setAttr ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $fileCount = $opts->{filecount};
        my $fileSize = $opts->{filesize}; 
        my $fileName = $opts->{filename};
        my $fileMode = $opts->{mode};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
	my $gid = (defined($opts->{gid}) ? $opts->{gid} : undef);
	my $size = (defined($opts->{size}) ? $opts->{size} : undef);
	my $uid = (defined($opts->{uid}) ? $opts->{uid} : undef);
	my $mode = (defined($opts->{mode}) ? $opts->{mode} : undef);
	my $err = $opts->{error};
        my @content;
        my $ret=1;  
        eval {        
 	        $client->{test}->nlog("Running setattr as user:($user)"); 
             	$ret = setattr_file(file => "$mountPath/$fileName",
                                     client => $clientHdl,
                                     mode => $mode,
				     uid => $uid,
				     gid => $gid,
				     size => $size,
                                     user => $user);
	        #$client->{test}->nlog("File/Dir $mountPath/$fileName is modified with attrs $opts->{mode} "); 
             	
        }; 

        if ($@) {
	        $client->{test}->nlog("Cannot modify the attr of file/dir in the mount : $@"); 
             	$ret=0;
		return $ret;
        }
	 if ($err) {    
	        	$client->{test}->nlog("Error got: $ret");
			$ret = _process_error($ret, $err, $client);
			return $ret;
		}
	

        return $ret;
}

sub isFilePresent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my @content;
    my $ret=1;
    eval {        
         @content = stat_file(file => "$mountPath/$fileName",
                              client => $clientHdl,
                              user => $user);
	     $client->{test}->nlog("File $mountPath/$fileName is present :". Dumper(@content)); 
         $ret=1;
    }; 

    if ($@) {
	     $client->{test}->nlog("File $mountPath/$fileName is not present : $@"); 
         $ret=0;
    }

    return $ret;
}

sub vfyFilePresent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath/$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              user => $user);
	     $client->{test}->nlog("File $mountPath/$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");

         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=1;
    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath/$fileName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}


sub vfyFileAbsent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath/$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              user => $user);
	     $client->{test}->nlog("File $mountPath/$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");

         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=0;
    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath/$fileName is not present : $@"); 
         $ret=1;
    }
    return $ret;
}

sub createDir ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $dirName = $opts->{dirname};
        my $dirMode = $opts->{mode};
        my $recursive = $opts->{recursive};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my @content;
	my $err = $opts->{error};
        my $ret=1;
        eval {        
             @content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
					user => $user
                                        );
 	        $client->{test}->nlog("Dir listing : @content"); 
             	$ret = make_dir(dir => "$mountPath/$dirName",
                                 client => $clientHdl,
                                 mode => $dirMode,
                                 user=>$user);
             	@content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
					user => $user);
 	        $client->{test}->nlog("Dir listing : @content"); 
		$client->{test}->nlog("ret:  $ret");
             
        }; 
	if ($@)
	{	
		$ret = 0;
		$client->{test}->nlog("Cannot create dir in the mount path:$@ ::: end ");
		return $ret;
		
	}

        if ($err) {
                $client->{test}->nlog("Error got: $ret");
                $ret = _process_error($ret, $err, $client);
                return $ret;
		}
        return $ret;
}

sub deleteDir ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $dirName = $opts->{dirname};
        my $recursive = $opts->{recursive};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my @content;
        my $ret=1;
	my $err = $opts->{error};
        eval {        
             @content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
                                        user => $user);
 	         $client->{test}->nlog("Dir listing : @content"); 
             $ret = remove_dir(dir => "$mountPath/$dirName",
                                   client => $clientHdl,
                                   recursive => $recursive,
                                   user => $user);

             @content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
                                        user => $user);
 	         $client->{test}->nlog("Dir listing : @content"); 
 
             
        }; 
        if ($@) {
	       	$client->{test}->nlog("Cannot delete file in the mount : $@ ::: end "); 
      	       	$ret=0;
		return $ret;
        }
	if($err){
		$ret = _process_error($ret, $err, $client);
		return $ret;
	}
        return $ret;
}

sub lsDir ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my @content;
    my $ret=1; 
  
    if(defined($opts->{dirname})){
		$mountPath = $mountPath."/".$opts->{dirname}; 
	}	

    eval {        
         @content = get_dir_entries(dir => "$mountPath",
                                    client => $clientHdl,
                                    user=>$user
                                    );

	$client->{test}->nlog("Directory listing:".Dumper(@content)); 
        $ret=\@content;
    }; 
    if ($@) {
	     $client->{test}->nlog("Directory listing error : $@"); 
             $ret=0;
    }
    return $ret;
}


sub vfyDirPresent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $dirName = $opts->{dirname};
    my $dirMode = $opts->{mode};
    my $recursive = $opts->{recursive};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my %content;
    my $ret=1; 
    eval {        
         %content = stat_file(file => "$mountPath/$dirName",
                              client => $clientHdl,
                              rtn_hash=>1,
                              user=>$user);

	     $client->{test}->nlog("Directory $mountPath/$dirName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=1;
    }; 
    if ($@) {
	     $client->{test}->nlog("Directory $mountPath/$dirName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}

sub vfyDirAbsent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $dirName = $opts->{dirname};
    my $dirMode = $opts->{mode};
    my $recursive = $opts->{recursive};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my %content;
    my $ret=1;
    
    eval {        
         %content = stat_file(file => "$mountPath/$dirName",
                              client => $clientHdl,
                              rtn_hash=>1,
                              user=>$user);
	     $client->{test}->nlog("Directory $mountPath/$dirName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=0;
    }; 
    if ($@) {
	     $client->{test}->nlog("Directory $mountPath/$dirName is not present : $@"); 
         $ret=1;
    }
    return $ret;
}

sub deleteBaseDir ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my @content;
        my $ret=1;
       	@content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
                                        user=>$user);
 	#     $client->{test}->nlog("Dir listing : @content"); 

	$client->{test}->nlog("Dir listing : @content");
        LINE: for my $dir (@content){
                if ($dir =~ /.snapshot/){
                next LINE ;
                }
                eval{
                $ret = remove_dir(dir => "$mountPath/$dir", recursive => 1, client => $clientHdl);
                };
                if ($@) {
                $client->{test}->nlog("Cannot delete dir in the mount : $@ ::: end ");
                }
       }

       #      @content = remove_dir(dir => "$mountPath",
       #                            client => $clientHdl,
       #                            recursive => 1,
       #                            contents_only =>1,
       #                            );
        @content = get_dir_entries(dir => "$mountPath",
                                       client => $clientHdl,
                                       );
 #      $client->{test}->nlog("Dir listing : @content");
        LINE1: for my $dir (@content){
                if ($dir =~ /.snapshot/){
                next LINE1 ;
                }
                eval{
                $ret = remove_file(file => "$mountPath/$dir", client => $clientHdl, recursive => 1);
                };
                if ($@) {
                $client->{test}->nlog("Cannot delete file in the mount : $@ ::: end ");
                $ret = 0;
                }
        }

        $ret=1;
        return $ret;
}

sub stat ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath/$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              user => $user);
	     $client->{test}->nlog("File $mountPath/$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         #$content{mode} = "$user_rwx$group_rwx$other_rwx";

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=\%content;
    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath/$fileName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}

sub statCheck ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath/$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              user => $user);

	     $client->{test}->nlog("File $mountPath/$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         #$content{mode} = "$user_rwx$group_rwx$other_rwx";

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");

      	 foreach my $key (keys(%$opts)) {
                next if $key eq "filename";
     		    next if $key eq "user";

				if ($content{$key} ne $opts->{$key}) {
					$client->{test}->nlog("Attributes do not match ,
					 expected $key = $opts->{$key} but got $key = $content{$key}");
                    $ret=0;
				}
     	 }

    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath/$fileName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}

sub statCompare ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $stat1 = $opts->{stat1};
    my $stat2 = $opts->{stat2};
    my $ignore = $opts->{ignore};
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmountedpath};
    my $ret=1;
    my %hash_stat = %$stat1;
    eval {        
     	for (keys(%hash_stat)) {
     		next if $_ eq "dev";	# may not match
     		next if $_ =~ $ignore;
     		$stat1->{$_} == $stat2->{$_}
     			or $client->{test}->nlog("Stat is different $_"
     				. " $stat1->{$_} != $stat2->{$_}");
            $ret=0;
     	}

        $ret=1;
    }; 
    if ($@) {
	     $client->{test}->nlog("Error : $@"); 
         $ret=0;
    }
    return $ret;
}


#kgokul

sub Link($$)
{
	my ($clientOps, $opts) = @_ ; 
	my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
     	my $filename = $opts->{filename};
	my $linkname = $opts->{linkname};
	my $client = $clientOps->{client};
     	my $clientHdl = $client->{hdl};
     	my $mountPath = $clientOps->{clientmountedpath};
	my $symbolic = (defined($opts->{symbolic}) ? $opts->{symbolic} : 0); #boolean value
	my $err = $opts->{error};
	my $ret = 1;
	$filename = "$mountPath/$filename";
	$linkname = "$mountPath/$linkname";
	eval {
	$ret = link_file(file => "$filename", flink => "$linkname", client => $clientHdl, user => $user, symbolic => $symbolic);
	};
	if ($ret != 1) {
	$ret = $ret + 0;
	 if ($err || ($ret == 0)) {
                $client->{test}->nlog("Error got: $@");
                $ret = _process_error($ret, $err, $client);
		return $ret;
    	}
	$client->{test}->nlog("Cannot create link: "); 
     	$ret=0;
     	}
	return $ret;
	 
}

sub unlink($$){
        my ($clientOps, $opts) = @_ ; 
        my $fileName = $opts->{filename};
        my $recursive = (defined($opts->{recursive}) ? $opts->{recursive} : 1);
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my @content;
        my $ret=1;
	my $err = $opts->{error};

        eval{       
              @content = get_dir_entries(dir => "$mountPath" ,
                                       client => $clientHdl,
					user => $user
                                        );
 	      $client->{test}->nlog("Dir listing : @content"); 

              $ret = remove_file(file => "$mountPath/$fileName",
					client => $clientHdl,
					recursive => $recursive,user => $user);
              @content = get_dir_entries(dir => "$mountPath" ,
                                       client => $clientHdl,
					user => $user,
                                        );
 	      $client->{test}->nlog("Dir listing : @content"); 

              
        }; 
	if ($@)
	{
		$ret = 0;
		$client->{test}->nlog("Cannot delete file in the mount : $@ ::: end ");
		return $ret;
	}
        if ($err) {
		$client->{test}->nlog("Processing error");
         	$client->{test}->nlog("Error got");
		$ret = _process_error($ret, $err, $client);
                return $ret; 
             	
        }
        return $ret;	
}

sub writeFile($$){
	my ($clientOps, $opts) = @_ ; 
	my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
	my $offset = (defined($opts->{offset}) ? $opts->{offset} : 0);
	my $fileName = $opts->{filename};
     	my $client = $clientOps->{client};
     	my $clientHdl = $client->{hdl};
     	my $mountPath = $clientOps->{clientmountedpath};
	my @chars=('a'..'z','A'..'Z','0'..'9','_','$','#');
	my $random_string;
     	my $ret = 1;  
	my @attr;
	my $length_of_randomstring  = (defined($opts->{string_length}) ? $opts->{string_length} : 1000);
 	foreach (1..$length_of_randomstring) {
		$random_string.=$chars[rand @chars];
	}
	eval {
	@attr = write_file(file => "$mountPath/$fileName", contents => "$random_string", fstat =>1,client => $clientHdl,user=>$user, offset => $offset);
	$client->{test}->nlog("File $mountPath/$fileName written successfully its stat are " . Dumper(@attr) ); 
     	$ret=1; 
	};
	if ($@) {
	$client->{test}->nlog("Cannot write file : Epic FAIL in lib : $@"); 
     	$ret=0;
     	}
	return $ret;
	 
}

sub mkfifo()
{
	my ($clientOps, $opts) = @_ ; 
	my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
   	#my $filename = $opts->{filename};
	my $name = $opts->{pipename};
     	my $client = $clientOps->{client};
     	my $clientHdl = $client->{hdl};
     	my $mountPath = $clientOps->{clientmountedpath};
	my $mask = (defined($opts->{mask})  ? $opts->{mask} : 0000); 
	my $mode = (defined($opts->{mode}) ? $opts->{mode}: 0777);
	my $err = $opts->{error};
	my $ret = 1;
	eval {
		$ret = mk_node(name => "$mountPath/$name", mode => $mode, mask =>$mask, client => $clientHdl, user => $user);
	 	     		 
	};
	if ($@)
	{
		$ret = 0;
		$client->{test}->nlog("Error got $@");
		return $ret;
	}
	if ($err) {
	        $client->{test}->nlog("Error got");
                $ret = _process_error($ret, $err, $client);
                return $ret;
		}
	return $ret;
}

sub renameFile()
{
	my ($clientOps, $opts) = @_ ; 
	my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    	my $fileName = $opts->{filename};
   	 my $client = $clientOps->{client};
    	my $clientHdl = $client->{hdl};
    	my $mountPath = $clientOps->{clientmountedpath};
	my $filename = $opts->{file};
	my $nfile = $opts->{newfile};
	my $err = $opts->{error};
	my $ret = 1;
	eval{
		$ret = rename_file(file => "$mountPath/$filename", nfile => "$mountPath/$nfile", client => $clientHdl, user => $user);
	};
	if ($@)
	{
		$ret = 0;
		$client->{test}->nlog("Rename file failed: $@");
		return $ret;
	}
	if ($err) {
		$client->{test}->nlog("Error got");
		$ret = _process_error($ret, $err, $client);
		return $ret;
	}

	
	return $ret;	
}

sub _process_error()
{
	my ($got, $expected, $client) = @_;
	#$op = "" unless $op;
	$expected = 0 unless $expected;
	 $client->{test}->nlog("op got ".sprintf("%d", $got) );
	if ($got && ($got != $expected)) {
		$client->{test}->nlog("op got ".sprintf("%d", $got)
		 ." ($got) instead of expected $expected ");
		return 0;
	}
	if ($expected && !$got) {
		$client->{test}->nlog("op succeeded when expected error $expected");
		return 0;
		
	}
	return 1;
	
}

sub truncateFile()
{
	my ($clientOps, $opts) = @_ ; 
	my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
	my $fileName = $opts->{filename};
    	my $client = $clientOps->{client};
    	my $clientHdl = $client->{hdl};
    	my $mountPath = $clientOps->{clientmountedpath};
	my $size = (defined($opts->{size})? $opts->{size} : 0);
	eval{
		file_truncate(file => "$mountPath/$fileName", size => "$size", client => $clientHdl, user => $user);
	};
	if ($@)
	{
		$client->{test}->nlog("Truncate file failed: $@");
		return 0;
	}
	return 1;
}


sub execute_cmd()
{
	my ($clientOps, $opts) = @_ ; 
	my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    	my $cmd = $opts->{command};
    	my $client = $clientOps->{client};
    	my $clientHdl = $client->{hdl};
    	my $mountPath = $clientOps->{clientmountedpath};
	my $ret;
	my $isFunc = (defined($opts->{isFunc}) ? $opts->{isFunc} : 1);
	eval{
		print "\n ## calling filemanip $cmd ## \n";
		$ret = SPARSE::cmode::FclusterFileManip::exec_cmd(command => $cmd, client => $clientHdl, user => $user, isFunc => $isFunc);
		print "\n ## ret val: $ret ## \n";
		
	};
	if ($@)
	{
		$client->{test}->nlog("Command failed $@ ");
		return 0;
	}
	return $ret;
}

sub readLink()
{
	my ($clientOps, $opts) = @_ ; 
	my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    	my $file = $opts->{linkname};
    	my $client = $clientOps->{client};
    	my $clientHdl = $client->{hdl};
    	my $mountPath = $clientOps->{clientmountedpath};
	my $ret ;
	eval{
		$ret = read_link(file => "$mountPath/$file", client => $clientHdl, user => $user);
		
	};
	if ($@)
	{
		$client->{test}->nlog("readlink failed $@");
		$ret = 0;
	}
	return $ret;
}

sub createDataSet ($$) {
	my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $clientHostname = $client->{hostname};
        my $mountPath = ( defined($opts->{writepath}) ? $opts->{writepath} : $clientOps->{clientmountedpath});
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my $filecount = (defined($opts->{files}) ? $opts->{files} : '50'); 
        my $filesize = (defined($opts->{filesize}) ? $opts->{filesize} : '500'); 
        my $filesizeAvg = (defined($opts->{filesizeavg}) ? $opts->{filesizeavg} : '10000'); 
        my $timeout = (defined($opts->{timeout}) ? $opts->{timeout} : '9000'); 

        my $ret = 1;   

        eval {        
            my %options = RichData::total_files_pop_opts(files => "$filecount");
            RichData::populate(path=>"$mountPath" , client => $clientHostname, file_size_avg=>"$filesizeAvg" ,
			              dir_avg=>"2", file_avg=>"2", sparse_num=>"2", link_num=>"2", slink_num=> "2",
				      compress_num=>"2", encrypt_num=>"2", archive_num=>"2", hidden_num=>"2", timeout=>"$timeout");
            #RichData::populate(path=>"$mountPath" , client => $clientHostname, file_size_avg=>"$filesizeAvg" ,
		#	              dir_avg=>"10", file_avg=>"10", sparse_num=>"5", link_num=>"5", slink_num=> "5",
		#		      compress_num=>"5", encrypt_num=>"5", archive_num=>"5", hidden_num=>"5", timeout=>"9000");
            $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Problem with createDataSet: $@"); 
             $ret=0;
        }

        return $ret;
}


sub createData ($$) {
	my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $clientHostname = $client->{hostname};
        my $mountPath = ( defined($opts->{writepath}) ? $opts->{writepath} : $clientOps->{clientmountedpath});
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my $filecount = (defined($opts->{files}) ? $opts->{files} : '100'); 
        my $dircount = (defined($opts->{dirs}) ? $opts->{files} : '10'); 
        my $filesize = (defined($opts->{filesize}) ? $opts->{filesize} : '500'); 
        my $timeout = (defined($opts->{timeout}) ? $opts->{timeout} : '9000'); 
        my ($dcount,$fcount);
        my $ret = 1;   

        eval {        
            for ($dcount = 1; $dcount <= $dircount; $dcount++) {
                $ret = $clientOps->createDir({'dirname'=>"micadir1_$dcount"});
                    for ($fcount = 1; $fcount <= $filecount; $fcount++) {
	                $ret = $clientOps->createFile({'filename'=>"micadir1_$dcount"."/"."micafile1_$fcount",
                    });
                }
            }

            $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Problem with createData: $@"); 
             $ret=0;
        }

        return $ret;
}


sub readDataSet1 ($$) {
	my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $clientHostname = $client->{hostname};
        my $mountPath = ( defined($opts->{readpath}) ? $opts->{readpath} : $clientOps->{clientmountedpath});
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my $ret = 1;   

        eval {        
            RichData::read_tree(path=>"$mountPath" , client => $clientHostname);
            $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Problem with readDataSet: $@"); 
             $ret=0;
        }

        return $ret;

}


sub readDataSet ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = ( defined($opts->{readpath}) ? $opts->{readpath} : $clientOps->{clientmountedpath});
    my @filesList;
    my $ret=1; 
  
    if(defined($opts->{dirname})){
		$mountPath = $mountPath."/".$opts->{dirname}; 
	}	

    eval {        
        @filesList = get_file_entries(dir => "$mountPath",
                                    client => $clientHdl,
                                    user=>$user
                                    );

	$client->{test}->nlog("Files Recursive listing:".Dumper(@filesList)); 

        foreach (@filesList) {
	    my $fileName = $_;
 	    $client->{test}->nlog("Reading File : $_ ");
            my @contents = read_file(file => "$fileName",client => $clientHdl,user => $user);
	    $client->{test}->nlog("File $mountPath/$fileName contents are read : " . " UNIX Read Successfully");                                 
        }

         $ret = 1 ;
    }; 

    if ($@) {
	     $client->{test}->nlog("Directory listing error : $@"); 
             $ret=0;
    }

    return $ret;

}

sub getFilesInDataSet ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = ( defined($opts->{readpath}) ? $opts->{readpath} : $clientOps->{clientmountedpath});
    my @filesList;
    my $ret=1; 
  
    if(defined($opts->{dirname})){
		$mountPath = $mountPath."/".$opts->{dirname}; 
	}	

    eval {        
        @filesList = get_file_entries(dir => "$mountPath",
                                    client => $clientHdl,
                                    user=>$user
                                    );

	$client->{test}->nlog("Files Recursive listing:".Dumper(@filesList)); 
        $ret = \@filesList ;
    }; 

    if ($@) {
	     $client->{test}->nlog("Directory listing error : $@"); 
             $ret=0;
    }

    return $ret;

}

sub cexec_cmd($$){
        my ($clientOps,$opts) = @_;
        my $client = $clientOps->{client};
        my $drive = $clientOps->{clientmountedpath};
        my $hostname = $client->{hostname}; 
	my $clientHdl    = $client->{hdl};
        my $cmd = $opts->{command};    
	my($spec) = $clientHdl  ;   
	my $ret = 1;    
	my $timeout = (defined($opts->{timeout}) ? $opts->{timeout}:700);        #$spec .= ";hosttype=windows";
        my $c = Connectrec->new(host     => "$hostname",
                            template => "rsh",
                            connect  => 1,
                            timeout => $timeout);
	(undef, undef, $ret) = say($c,$cmd,"-eval_error"); 
	if ($ret == ''){$ret = 1 ;}
	if ($@ && $@ =~ /Err=TIMEOUT/i) {
        	$ret = 0;        
		$client->{test}->nlog("TIMEOUT during command execution: $cmd");
        }
	print "\n Return Value: $ret \n";
        return $ret;
}

sub openFile($$){
        
	my ($clientOps, $opts) = @_ ; 
        my $fileName  = $opts->{filename};
        my $client    = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my $filerjunctionPath = $clientOps->{filerjunctionpath};

        my $fileMode  = (defined($opts->{mode})  ? $opts->{mode} : '+<');
        my $fileMask  = (defined($opts->{mask})  ? $opts->{mask} : 0000); 
        my $filePerms = (defined($opts->{perms}) ? $opts->{perms} : 0666); 
        my $fileError = (defined($opts->{error}) ? $opts->{error} : '0'); 
        my $user      = (defined($opts->{user})  ? $opts->{user} : 'root'); 
	my $ret;


        eval {        
              $ret = open_file(file => "$mountPath/$fileName",
					client => $clientHdl,
             			        mode => $fileMode,
             			        perms=> $filePerms,
             			        mask => $fileMask, 
             			        user=> $user );
	     $client->{test}->nlog("File $mountPath/$fileName opened successfully " ); 
             
        }; 

        if ($@) {
	     $client->{test}->nlog("Cannot create files in the mount : Epic FAIL in lib : $@"); 
             $ret=0;
        }
	return $ret;
}


sub closeFile($$){        
        my ($clientOps, $opts) = @_ ;
        my $filehandle  = $opts->{filehandle};
        my $client    = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmountedpath};
        my $filerjunctionPath = $clientOps->{filerjunctionpath};

        my $user      = (defined($opts->{user})  ? $opts->{user} : 'root'); 
        my $ret;


        eval {  
              $ret = close_file(file => "$filehandle",
					client => $clientHdl,
			 		user=> $user );
		$client->{test}->nlog("File  closed successfully " );
		$ret = 1;
        }; 
        if ($@) {            
	 $client->{test}->nlog("Cannot create files in the mount : Epic FAIL in lib : $@");             
         $ret=0;
        }
        return $ret;
}


sub open_a_file($$)
{

		my ($clientOps,$opts) = @_;        
		my $client = $clientOps->{client};
		my $mount = $clientOps->{clientmountedpath};
		my $hostname = $client->{hostname}; 
		my $clientHdl    = $client->{hdl};
		my $filename = $opts->{filename};   
		my $option = $opts->{option}; 
		my($spec) = $clientHdl  ;        
		my $hostname = $client->{hostname};        
		my $ret = 1;        
		my $c;        
		eval{        
		$c = Connectrec->new(host     => "$hostname",
                            	template => "rsh-persist",
                            	connect  => 1);
       
		
	                
        my $reply = $c->execute(command => "/usr/software/bin/perl /usr/software/test/agnostic/sparse/cmode/lockstaterep/fileopen.pl  $mount/$filename",
                timeout => 60,
                match_table => [
                    ("Read/Write/Lock>") => "$option"],
		additional_prompts => ["Operation Success", "<shared/excl> <w/nw> <start_byte> <range> : "] 
		                            
                );
                
        
		if ($reply =~ /.*.failed.*/i) 
		{
               $client->{test}->nlog("Failed: $reply");
               $ret = 0;
               return 0;
       	}
		elsif ($reply !~ /.*File opened .*/i)
		{
			$client->{test}->nlog("Failed: $reply");
			$ret = 0;
			return 0;
		}
	};if ($@){
		$client->{test}->nlog("Failed:$@");
		$ret = 0;
		return 0;
	}
		if ($ret != 0)
		{
        	$ret = $c;
		}
		return $ret;
}

sub close_a_file($$)
{   
        my ($clientOps,$opts) = @_;
        my $client = $clientOps->{client};
        my $mount = $clientOps->{clientmountedpath};
        my $hostname = $client->{hostname};       
        my $clientHdl    = $client->{hdl}; 
        my $conn = $opts->{connrec}; 
        my $ret = 1;
        my (undef,undef,$reply,undef) = say($conn,"\r");
        print "\nReply: $reply\n";
        if ($reply =~ /Close failed.*/)
        {
                $ret = 0;
        }       
        if ($@) {
                $client->{test}->nlog("Close failed:$@ ");
                $ret = 0;
        }       
        
      
    return $ret;
}

sub lock_file($$)
{  
    my ($clientOps,$opts) = @_;
    my $client = $clientOps->{client};
	my $filename = $opts->{filename};
    my $mount = $clientOps->{clientmountedpath};
    my $hostname = $client->{hostname};      
    my $clientHdl    = $client->{hdl};
    my $ret = 1;
	my $mount = $clientOps->{clientmountedpath};
	my $lock_type = $opts->{lock_type};
	my $waited =  ($opts->{waited} == 1 ? "w" : "nw");
	my $start = $opts->{start};
	my $range = $opts->{range};
	my $c;
	my $reply;
		
      eval{
	  
	    $c = Connectrec->new(host     => "$hostname",
                                template => "rsh-persist",
                                connect  => 1);
    
   
                 
      $reply = $c->execute(command => "//usr/software/bin/perl //usr/software/test/agnostic/sparse/cmode/lockstaterep/fileopen.pl $mount/$filename",
                timeout => 60,
                match_table => [
                    ("Read/Write/Lock>") => "l"],
        additional_prompts => ["<shared/excl> <w/nw> <start_byte> <range> : "]
       
                );
 
		$reply = $c->execute(command => "$lock_type $waited $start $range",
                timeout => 60,
                additional_prompts => ["Locked!!", "WLocking"]

                );
      
	$ret = $c;
		};
        if ($reply =~ /Locking failed:.*/)
        {
                $ret = 0;
        }      
        if ($@) {
                $client->{test}->nlog("Locking failed:$@ ");
                $ret = 0;
        }      
       
     
    return $ret;
}

