#$Id: //depot/prod/test/main/lib/SPARSE/fcluster/ClientOpsObject.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.

## @summary	
##      Library which defines Client Ops Object properties 
##      and actions.
## @description
##	Abstract common object tasks.
##
## @usage
##	use SPARSE::cmode:ClientOpsObject;
##
## @author mica@netapp.com

package SPARSE::cmode::ClientOpsObject_Win32;

use strict;
use Tharn;
use filer;
use mount();
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
# our modded Filemanip Below ;)
use SPARSE::cmode::FclusterFileManip qw(check_file write_file read_file remove_file get_dir_entries get_file_entries
                remove_dir stat_file make_dir setattr_file fh_Stat mk_node read_stream write_stream 
		rename_stream remove_stream rename_file file_truncate getattr_file);

# Internal reference classes 
use SPARSE::cmode::AggrObject qw();
use SPARSE::cmode::VserverObject qw();
use SPARSE::cmode::ClientObject qw();


#NACL libs
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);

use Errno qw(:POSIX);
use IO::File;
use IO::Handle;
use TestEnv::MinWin qw(execute_cmd minwin_compatible);
use RichData;

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&clientOps
		&clientOpsWin32
		);

# ------------------------------------------------------------------------
## Client class definition

## @name	::clientOps
## @summary	create a windows client ops object
## @description
## Create a client ops object.
## @arg		opts optional
## @return	client object
				
sub clientOpsWin32(%)
{
    my %fparams = validate(
        @_,
        {
           name  => { type => SCALAR  },
           client  => { type => HASHREF },
           opts    => { type => HASHREF },
        });

    my $clientOps = {};
    my $clientOpsName   =  $fparams{name};
    my $opts                   =  $fparams{opts};
    my $client                 =  $fparams{client};  
   
	$clientOps->{name}  =  $clientOpsName;
	$clientOps->{filerjunctionpath}   =    $opts->{filerjunctionpath};
	$clientOps->{clientmappeddrive}   =  $opts->{mappeddrive};
    $clientOps->{client}  =  $client;
    $clientOps->{type}  =  "windows";
    
	#$client->{test}->nlog("clientOps object created : " . Dumper($clientOps));

	bless($clientOps);
	return $clientOps;
}



sub DESTROY($)
{
	my ($clientOps) = @_;
	my $exc_state = $@;
	$clientOps->{test}->nlog("destroying $clientOps->{name} object");
	$@ = $exc_state;
}


# ------------------------------------------------------------------------
## client class methods
#fioOpen($dvol, "$dirname/$filename", "+>", undef, 0777, EACCES);
sub fioOpen ($$) {
        use Errno qw(:POSIX);
        my ($clientOps, $opts) = @_ ; 

        my $fileName  = $opts->{filename};
        my $client    = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        #my $filerjunctionPath = $clientOps->{filerjunctionpath};

        my $fileMode  = (defined($opts->{mode})  ? $opts->{mode} : '+<');
        my $fileMask  = (defined($opts->{mask})  ? $opts->{mask} : 0000); 
        my $filePerms = (defined($opts->{perms}) ? $opts->{perms} : 0666); 
        my $fileError = (defined($opts->{error}) ? $opts->{error} : '0'); 
        #my $user      = (defined($opts->{user})  ? $opts->{user} : 'root'); 

        my ($got,$op,$expected);
        my $ret=1;
        my $retHash = ();

        eval {        
 	         #$client->{test}->nlog("Running fioOpen as user:($user) ### open flags : $fileMode ### perms : $filePerms"); 
             $got = check_file( file => "$mountPath\\$fileName", 
             			        client => $clientHdl,
             			        mode => $fileMode,
             			        perms=> $filePerms,
             			        mask => $fileMask, 
             			         );

          	 $op = "open file $mountPath\\$fileName with open flags ";
             $expected = $fileError;   
			            
             # if a file is not created and an error code is returned
			 if (ref($got) ne "HASH"){
				$client->{test}->nlog("Error code: $got ");
				return 0;
			 }
			 # if a file is created succesfully , dont enter this check 
             if ($got->{fh} !~ /GLOB/) {      
          		$client->{test}->nlog("$op executed : got $got->{fh} : expected $expected , $got->{fh} = $expected");

				if ($got->{fh} && ($got->{fh} != $expected)) {
					# got unexpected error
					$client->{test}->nlog("$op got ".sprintf("%d", $got->{fh})
					." ($got->{fh}) instead of expected $expected ");
					$ret=0;
				}
     
				if ($expected && !$got->{fh}) {
					# expected error, but did not get it
					$client->{test}->nlog("$op succeeded when expected error $expected");
					$ret=0;
				}

			 } else {		
				$client->{test}->nlog("$op executed : got $got->{fh} : file created succesfully at $mountPath/$fileName");
                $retHash->{fh} = $got->{fh}; 
				$ret=$retHash;
             }
        }; 
        if ($@) {
	         $client->{test}->nlog("Cannot fioOpen File $@"); 
             $ret=0;
        }
 
    return $ret;
}

## fh related routines 
sub fhStatCheck ($$) {
    my ($clientOps, $opts) = @_ ; 

    my $fileName  = $opts->{filename};
    my $client    = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my $filerjunctionPath = $clientOps->{filerjunctionpath};

    #my $fileMode  = (defined($opts->{mode})  ? $opts->{mode} : '+<');
    my $fileMode  = $opts->{mode};
    my $fileMask  = (defined($opts->{mask})  ? umask($opts->{mask}) : ''); 
    my $filePerms = (defined($opts->{perms}) ? $opts->{perms} : '0666'); 
    my $fileError = (defined($opts->{error}) ? $opts->{error} : '0'); 
    my $user      = (defined($opts->{user})  ? $opts->{user} : 'root'); 

    my %content;
    my $ret=1;
    my $retHash;

    eval {        
        $retHash = fh_Stat(file => "$mountPath\\$fileName", 
             			   client => $clientHdl,
						      );
	    $client->{test}->nlog("fh_Stat hash". Dumper($retHash)); 
	    #qw( dev ino mode nlink uid gid rdev
     	#    size atime mtime ctime blksize blocks );

        # strip mode of perms etc 
        my $realgivenmode = $fileMode & 07777;
        
        my $realmode = $retHash->{mode} & 07777;
        my $user_rwx = ($realmode & 00700) >> 6;
        my $target_bits = $user_rwx & 05;
        my $group_rwx = ($realmode & 00070) >> 3;
        my $other_rwx = ($realmode & 00007);

       	$client->{test}->nlog("given mode :: $fileMode,Got mode $retHash->{mode}, rwx is : $user_rwx$group_rwx$other_rwx"); 
        $ret=$retHash;
    }; 
    if ($@) {
	     $client->{test}->nlog("Filehandle stat cannot be run: $@"); 
         $ret=0;
    }

    return $ret;

}



sub unlink($$){
        my ($clientOps, $opts) = @_ ; 
        my $fileName = $opts->{filename};
        my $recursive = $opts->{recursive};
        #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my @content;
        my $ret=1;

        eval {        
              @content = get_dir_entries(dir => "$mountPath" ,
                                        client => $clientHdl
                                        );
 	          $client->{test}->nlog("Dir listing : @content"); 

              @content = remove_file(file => "$mountPath\\$fileName", 
             									           client => $clientHdl,
             									           recursive => 1
             									           );
              @content = get_dir_entries(dir => "$mountPath" ,
                                        client => $clientHdl
                                        );
 	          $client->{test}->nlog("Dir listing : @content"); 

              $ret = 1;
        }; 

        if ($@) {
	         $client->{test}->nlog("Cannot delete file in the mount : $@ ::: end "); 
             $ret = 0;
        }
        return $ret;	
}

sub createFile ($$) {
    my ($clientOps, $opts) = @_ ; 
        #my $fileCount       = $opts->{filecount};
        #my $fileSize         = $opts->{filesize}; 
        my $fileName = $opts->{filename};
        my $client = $clientOps->{client};
        # my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
		 
	print "\n Name: $hostname , $clientHdl\n";
        # my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my @attrs;
  	my $length_of_randomstring=50000;
    	my @chars=('a'..'z','A'..'Z','0'..'9','_','$','#');
	my $random_string;
        my $ret = 1;   

	foreach (1..$length_of_randomstring) {
		      $random_string.=$chars[rand @chars];
	}

        eval {        
            @attrs = write_file(file => "$mountPath\\$fileName", 
             										contents => "$random_string",
             										client => $clientHdl
             										);
	    $client->{test}->nlog("File $mountPath\\$fileName created successfully its stat are " . Dumper(@attrs) ); 
            $ret=1; 
        }; 

        if ($@) {
	    $client->{test}->nlog("Cannot create files in the share : Epic FAIL in lib : $@"); 
            $ret=0;
        }

        return $ret;
}

sub modifyFile ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $fileData = $opts->{data};
        my $fileName = $opts->{filename};
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
        my @attrs;
        my $ret = 1;   

        eval {        
            @attrs = write_file(file => "$mountPath\\$fileName", 
             										contents => "$fileData", fstat =>1,
             										client => $clientHdl
             										);
	    $client->{test}->nlog("File $mountPath\\$fileName modified successfully its stat are " . Dumper(@attrs) ); 
            $ret=1; 
        }; 

        if ($@) {
	    $client->{test}->nlog("Cannot modify files in the share : Epic FAIL in lib : $@"); 
            $ret=0;
        }

        return $ret;
}

sub readFile ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $fileCount       = $opts->{filecount};
        my $fileSize         = $opts->{filesize}; 
        # my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $fileName = $opts->{filename};
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my @content;
        my $ret = 1;
        eval {        
            @content = read_file(file => "$mountPath\\$fileName", client => $clientHdl,);
	    $client->{test}->nlog("File $mountPath\\$fileName  SMB read Successfully"); 
            $ret=\@content;
        }; 

        if ($@) {
	    $client->{test}->nlog("Cannot read file in the mount : $@"); 
            $ret=0;
        }

        return $ret;
}

sub deleteFile ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $fileName = $opts->{filename};
        my $recursive = $opts->{recursive};
        #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my @content;
        my $ret=1;

        eval {        
              @content = get_dir_entries(dir => "$mountPath" ,
                                        client => $clientHdl);
 	          $client->{test}->nlog("Dir listing : @content"); 

              @content = remove_file(file => "$mountPath\\$fileName", 
             									           client => $clientHdl,
             									           recursive => 1,
             									           );
              @content = get_dir_entries(dir => "$mountPath" ,
                                        client => $clientHdl,
                                        );
 	          $client->{test}->nlog("Dir listing : @content"); 

              $ret = 1;
        }; 

        if ($@) {
	         $client->{test}->nlog("Cannot delete file in the mount : $@ ::: end "); 
             $ret = 0;
        }
        return $ret;
}

sub setAttr ($$) {
        my ($clientOps, $opts) = @_ ; 
       # my $fileCount = $opts->{filecount};
        #my $fileSize = $opts->{filesize}; 
        my $fileName = $opts->{filename};
        #my $fileMode = $opts->{mode};
        #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my @content;
        my $ret=1;  
        eval {        
 	         #$client->{test}->nlog("Running setattr as user:($user)"); 
             #@content = setattr_file(file => "$mountPath\\$fileName",
             #                        client => $clientHdl,
             #                        hidden => "$opts->{hidden}",
             #                        archive => "$opts->{archive}",		
             #                        );
	     #    $client->{test}->nlog("File\\Dir $mountPath\\$fileName is modified with attrs $opts->{mode} "); 
		delete $opts->{filename};
		@content = setattr_file(file => "$mountPath\\$fileName",
					client => $clientHdl,
					%$opts
					);
		$client->{test}->nlog("File\\Dir $mountPath\\$fileName modified with attributes << ".Dumper(%$opts)." >>");
             $ret=1;
        }; 

        if ($@) {
	         $client->{test}->nlog("Cannot modify the attr of file/dir in the mount : $@"); 
             $ret=0;
        }

        return $ret;
}

sub getAttr ($$) {
        my ($clientOps, $opts) = @_ ;
        my ($fileName, $mountPath, $client, $clientHdl, $ret);
        $fileName = delete $opts->{filename};
        $client = $clientOps->{client};
        $clientHdl = $client->{hdl};
        $mountPath = $clientOps->{clientmappeddrive};
        eval {
                $ret = {getattr_file(file => "$mountPath\\$fileName", client => $clientHdl)};
                $client->{test}->nlog("File\\Dir $mountPath\\$fileName has attributes :\n".Dumper(%$ret)."\n");
        };

        if ($@) {
                $client->{test}->nlog("Could not fetch the attr of file/dir in the mount : $@");
                $ret=0;
        }

        return $ret;
}

sub isFilePresent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my @content;
    my $ret=1;
    eval {        
         @content = stat_file(file => "$mountPath\\$fileName",
                              client => $clientHdl,
                              );
	     $client->{test}->nlog("File $mountPath\\$fileName is present :". Dumper(@content)); 
         $ret=1;
    }; 

    if ($@) {
	     $client->{test}->nlog("File $mountPath\\$fileName is not present : $@"); 
         $ret=0;
    }

    return $ret;
}

sub vfyFilePresent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
   # my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath\\$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              );
	     $client->{test}->nlog("File $mountPath\\$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");

         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=1;
    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath\\$fileName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}


sub vfyFileAbsent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath\\$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              );
	     $client->{test}->nlog("File $mountPath\\$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");

         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=0;
    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath\\$fileName is not present : $@"); 
         $ret=1;
    }
    return $ret;
}


sub createDir ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $dirName = $opts->{dirname};
        #my $dirMode = $opts->{mode};
        my $recursive = $opts->{recursive};
        #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my @content;
        my $ret=1;
        eval {        
             @content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
                                        );
 	     $client->{test}->nlog("Dir listing : @content"); 
             @content = make_dir(dir => "$mountPath\\$dirName",
                                 client => $clientHdl,
                                 
                                 );
             @content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
                                        );
 	         $client->{test}->nlog("Dir listing : @content"); 
             $ret=1;
        }; 

        if ($@) {
	        $client->{test}->nlog("Cannot create dir in the mount path: $@ ::: end "); 
            $ret=0;
        }
        return $ret;
}

sub deleteDir ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $dirName = $opts->{dirname};
        my $recursive = $opts->{recursive};
        #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my @content;
        my $ret=1;
        eval {        
             @content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
                                        );
 	         $client->{test}->nlog("Dir listing : @content"); 
             @content = remove_dir(dir => "$mountPath\\$dirName",
                                   client => $clientHdl,
                                   recursive => $recursive,
                                   );

             @content = get_dir_entries(dir => "$mountPath",
                                        client => $clientHdl,
                                        );
 	         $client->{test}->nlog("Dir listing : @content"); 
 
             $ret=1;
        }; 

        if ($@) {
	         $client->{test}->nlog("Cannot delete file in the share : $@ ::: end "); 
             $ret=0;
        }
        return $ret;
}

sub lsDir ($$) {
    my ($clientOps, $opts) = @_ ; 
    #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my @content;
    my $ret=1; 
  
    if(defined($opts->{dirname})){
		$mountPath = $mountPath."\\".$opts->{dirname}; 
	}	

    eval {        
         @content = get_dir_entries(dir => "$mountPath",
                                    client => $clientHdl,
                                    );

	     $client->{test}->nlog("Directory listing:".Dumper(@content)); 
         $ret=1;
    }; 
    if ($@) {
	     $client->{test}->nlog("Directory listing error : $@"); 
         $ret=0;
    }
    return $ret;
}

sub vfyDirPresent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $dirName = $opts->{dirname};
    #my $dirMode = $opts->{mode};
    my $recursive = $opts->{recursive};
   # my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my %content;
    my $ret=1; 
    eval {        
         %content = stat_file(file => "$mountPath\\$dirName",
                              client => $clientHdl,
                              rtn_hash=>1,
                              );

	     $client->{test}->nlog("Directory $mountPath\\$dirName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=1;
    }; 
    if ($@) {
	     $client->{test}->nlog("Directory $mountPath\\$dirName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}

sub vfyDirAbsent ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $dirName = $opts->{dirname};
   # my $dirMode = $opts->{mode};
    my $recursive = $opts->{recursive};
  #  my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my %content;
    my $ret=1;
    
    eval {        
         %content = stat_file(file => "$mountPath\\$dirName",
                              client => $clientHdl,
                              rtn_hash=>1,
                              );
	     $client->{test}->nlog("Directory $mountPath\\$dirName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=0;
    }; 
    if ($@) {
	     $client->{test}->nlog("Directory $mountPath\\$dirName is not present : $@"); 
         $ret=1;
    }
    return $ret;
}

# Cannot remove the mapped drive . The following fn doesn't apply for cifs.
sub deleteBaseDir ($$) {
        my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my @content;
        my $ret=1;
                
      	@content = get_dir_entries(dir => "$mountPath",
        			    client => $clientHdl,
                                        );
       	$client->{test}->nlog("Dir listing : @content"); 
	LINE: for my $dir (@content){
	       	if ($dir =~ /.snapshot/){
            	next LINE ;
         	}
		eval{
            	$ret = remove_dir(dir => "$mountPath\\$dir", recursive => 1, client => $clientHdl);
		};
		if ($@) {
                $client->{test}->nlog("Cannot delete dir in the mount : $@ ::: end ");
		}
       }		

       #      @content = remove_dir(dir => "$mountPath",
       #                            client => $clientHdl,
       #                            recursive => 1,
       #                            contents_only =>1,
       #                            );
       	@content = get_dir_entries(dir => "$mountPath",
                                       client => $clientHdl,
                                       );
 #	$client->{test}->nlog("Dir listing : @content"); 
	LINE1: for my $dir (@content){
            	if ($dir =~ /.snapshot/){
             	next LINE1 ;
            	}
		eval{
            	$ret =  remove_file(file => "$mountPath\\$dir", client =>$clientHdl, recursive => 1);
		};
		if ($@) {
                $client->{test}->nlog("Cannot delete file in the mount : $@ ::: end ");
		$ret = 0;
		}
        }
             									           
        $ret=1;
	 

       # if ($@) {
#	         $client->{test}->nlog("Cannot delete dir in the mount : $@ ::: end "); 
#             $ret=0;
#        }
        return $ret;
}

sub stat ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath\\$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              );
	     $client->{test}->nlog("File $mountPath\\$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         #$content{mode} = "$user_rwx$group_rwx$other_rwx";

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");
         $ret=\%content;
    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath\\$fileName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}

sub statCheck ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $fileName = $opts->{filename};
    #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my %content;
    my $ret=1;
    eval {        
         %content = stat_file(file => "$mountPath\\$fileName",
                              client => $clientHdl,
                              rtn_hash => 1,
                              );

	     $client->{test}->nlog("File $mountPath\\$fileName is present :"); 
         $client->{test}->nlog("ctime  : $content{ctime}");
         $client->{test}->nlog("rdev   : $content{rdev}");
         $client->{test}->nlog("uid    : $content{uid}");
         $client->{test}->nlog("blocks : $content{blocks}");

         my $mode = $content{mode} & 07777;
         my $user_rwx = ($mode & 00700) >> 6;
         my $target_bits = $user_rwx & 05;
         my $group_rwx = ($mode & 00070) >> 3;
         my $other_rwx = ($mode & 00007);
         #$content{mode} = "$user_rwx$group_rwx$other_rwx";

         $client->{test}->nlog("mode   : $content{mode} ($user_rwx$group_rwx$other_rwx)");
         $client->{test}->nlog("mtime  : $content{mtime}");
         $client->{test}->nlog("size   : $content{size}");
         $client->{test}->nlog("nlink  : $content{nlink}");
         $client->{test}->nlog("io_size: $content{io_size}");
         $client->{test}->nlog("atime  : $content{atime}");
         $client->{test}->nlog("ino    : $content{ino}");
         $client->{test}->nlog("gid    : $content{gid}");

      	 foreach my $key (keys(%$opts)) {
                next if $key eq "filename";
     		    next if $key eq "user";

				if ($content{$key} ne $opts->{$key}) {
					$client->{test}->nlog("Attributes do not match ,
					 expected $key = $opts->{$key} but got $key = $content{$key}");
                    $ret=0;
				}
     	 }

    }; 
    if ($@) {
	     $client->{test}->nlog("File $mountPath\\$fileName is not present : $@"); 
         $ret=0;
    }
    return $ret;
}

sub statCompare ($$) {
    my ($clientOps, $opts) = @_ ; 
    my $stat1 = $opts->{stat1};
    my $stat2 = $opts->{stat2};
    my $ignore = $opts->{ignore};
    #my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my $ret=1;
    eval {        
     	for (keys(%{$stat1})) {
     		next if $_ eq "dev";	# may not match
     		next if $_ =~ $ignore;
     		$stat1->{$_} == $stat2->{$_}
     			or $client->{test}->nlog("Stat is different $_"
     				. " $stat1->{$_} != $stat2->{$_}");
            $ret=0;
     	}

        $ret=1;
    }; 
    if ($@) {
	     $client->{test}->nlog("Error : $@"); 
         $ret=0;
    }
    return $ret;
}

sub addAclUser($$){
	
	my ($clientOps,$opts) = @_;
	my $acluser = $opts->{acluser};
	my $aclpass = $opts->{aclpass};
	my $client = $clientOps->{client};
	my $hostname = $client->{hostname};
		
	my $command = "net user $acluser $aclpass /add /domain" ;
	my @op = execute_cmd($hostname, $command);
	
	
}

sub deleteAclUser($$){
	
	my ($clientOps,$opts) = @_;
	my $acluser = $opts->{acluser};
	my $client = $clientOps->{client};
	my $hostname = $client->{hostname};
		
	my $command = "net user $acluser /delete /domain" ;
	execute_cmd($hostname, $command);
	
}

sub addAclGroup($$){
	my ($clientOps,$opts) = @_;
	my $aclgroup = $opts->{aclgroup};
	my $client = $clientOps->{client};
	my $hostname = $client->{hostname};
	
	my $command = "net group $aclgroup  /add /domain";
	execute_cmd($hostname, $command);
}

sub deleteAclGroup($$){
	my ($clientOps,$opts) = @_;
	my $aclgroup = $opts->{aclgroup};
	my $client = $clientOps->{client};
	my $hostname = $client->{hostname};
	
	my $command = "net group $aclgroup  /delete /domain";
	execute_cmd($hostname, $command);
}

sub addAclUsertoGroup($$){
	my ($clientOps,$opts) = @_;
	my $aclgroup = $opts->{aclgroup};
	my $acluser = $opts->{acluser};
	my $client = $clientOps->{client};
	my $hostname = $client->{hostname};
	
	my $command = "net group $aclgroup $acluser  /delete /domain";
	execute_cmd($hostname, $command);
}

sub editAcl($$){
	my ($clientOps,$opts) = @_;
	my $filename = $opts->{filename};
	my $client = $clientOps->{client};
	my $clientname = $client->{hostname};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $user = $opts->{acluser};
	my $perm = $opts->{permission};
	my $path = "$drive\\$filename" ;
	my $clientHdl    = $client->{hdl};
	my $conn = Connectrec->new(host     => "$clientname",
                            template => "rsh-persist",
                            connect  => 1);

	my $command = "CACLS.EXE $path /G $user:$perm /E" ;
	#$clientHd;
	#my (undef,undef,$ret,$prt) = say($conn, "$command","-prompt","[A-Z,a-z,\s,\(,\),\?,/]*");
	#print "\n\n$ret \n\n $prt\n \n";
	say($conn,"$command");
	#say($conn,"y");
	
	#$conn->execute(command=>"$command" , must_match => "Are\syou\ssure\s\(Y\/N\)\?" , match_table => ["[A-Z,a-z,\s,\(,\),\?,/]*" => "y"]);#Are\syou\ssure\s\(Y\/N\)\?[A-Z,a-z,\s,\(,\),\?,/]*" => "y" ],); 
	
}


sub editAcl_Revoke($$){        
	my ($clientOps,$opts) = @_;
	my $filename = $opts->{filename};
	my $client = $clientOps->{client};
	my $clientname = $client->{hostname};
	my $drive = $clientOps->{clientmappeddrive};        
	my $hostname = $client->{hostname};        
	my $user = $opts->{acluser};        
	my $perm = $opts->{permission};        
	my $path = "$drive\\$filename" ;        
	my $clientHdl    = $client->{hdl};        
	my $conn = Connectrec->new(host     => "$clientname",
	template => "rsh-persist",
	connect  => 1);   
	my $command = "CACLS.EXE $path /R $user:$perm /E" ; 
	say($conn,"$command");
            
       
}



sub editAcl_overwrite($$){
	my ($clientOps,$opts) = @_;
	my $filename = $opts->{filename};
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $user = $opts->{acluser};
	my $perm = $opts->{permission};
	my $path = "$drive\\$filename" ;
	my $clientHdl    = $client->{hdl};
	my $conn1 = $opts->{conn};
	#my $conn = connect($conn1);
	my $conn = Connectrec->new(host     => "$hostname",
                            template => "rsh-persist",
                            connect  => 1);

	my $command = "CACLS.EXE $path /P $user:$perm" ;
	#$clientHd;
	#my (undef,undef,$ret,$prt) = say($conn, "$command","-prompt","[A-Z,a-z,\s,\(,\),\?,/]*");
	#print "\n\n$ret \n\n $prt\n \n";
	say($conn,"$command",'-prompt',"[A-Z,a-z,\s,\(,\),\?,/]*");
	say($conn,"y");
	
	#$conn->execute(command=>"$command" , must_match => "Are\syou\ssure\s\(Y\/N\)\?" , match_table => ["[A-Z,a-z,\s,\(,\),\?,/]*" => "y"]);#Are\syou\ssure\s\(Y\/N\)\?[A-Z,a-z,\s,\(,\),\?,/]*" => "y" ],); 
	
	return 1;
}

#fn to exec commands on win client
sub exec_cmd($$){
	my ($clientOps,$opts) = @_;
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $clientHdl    = $client->{hdl};
	my $cmd = $opts->{command};
	my($spec) = $clientHdl  ;
	my $ret;
	my $cifs_timeout = (defined($opts->{timeout}) ? $opts->{timeout}:300);
    	#$spec .= ";hosttype=windows";
    	#unless ($winclient =~ /;hosttype=windows/);
    	my $c = Connectrec->new(host     => "$hostname",
                            template => "rsh",
                            connect  => 1,
			    timeout => $cifs_timeout);
	
	(undef, undef, $ret) = say($c,$cmd,"-eval_error");
    	if ($@ && $@ =~ /Err=TIMEOUT/i) {
	$ret = 0;
     	$client->{test}->nlog("TIMEOUT during command execution: $cmd");
    	}
	return $ret;

}

sub writeStream()
{
	my ($clientOps,$opts) = @_;
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $clientHdl    = $client->{hdl};
	my $streamname = $opts->{streamname};
	my $filename = $opts->{filename};
	 my $mountPath = $clientOps->{clientmappeddrive};
	my $length_of_randomstring = 50000;
	my $random_string;
	my $offset = (defined($opts->{offset}) ? $opts->{offset} : 0);
	my $append = (defined($opts->{append}) ? $opts->{append} : 0); 
    	my $ret = 1;   
	my @attr;
	my @chars=('a'..'z','A'..'Z','0'..'9','_','$','#');

	 foreach (1..$length_of_randomstring) {
	      $random_string.=$chars[rand @chars];
	 }
	eval{
		@attr = write_stream(client => $clientHdl, stream => $streamname, file => "$mountPath\\$filename", 
		 contents => $random_string, offset => $offset, append => $append );
		$client->{test}->nlog("Stream written. Attrs are:".Dumper(@attr));
	};
	if ($@){
		$client->{test}->nlog("Write Stream failed: $@");
		$ret = 0;
	}
	return $ret;
			
}

sub readStream()
{
	my ($clientOps,$opts) = @_;
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $clientHdl    = $client->{hdl};
	my $streamname = $opts->{streamname};
	my $filename = $opts->{filename};
	my $mountPath = $clientOps->{clientmappeddrive};
	my $ret;
	eval{
		my @content = read_stream(client => $clientHdl, stream => $streamname, file => "$mountPath\\$filename");
		$ret = 1;
		$client->{test}->nlog("Stream Read: ");#Dumper(@content));
	};
	if ($@)
	{
		$client->{test}->nlog("Read stream failed: $@");
		$ret = 0;
	}
	
	return $ret;
	
}


sub writeFile($$){    
    my ($clientOps, $opts) = @_ ;
   #my $user = (defined($opts->{user}) ? $opts->{user} : 'root');
    my $offset = (defined($opts->{offset}) ? $opts->{offset} : 0);
    my $fileName = $opts->{filename};
    my $client = $clientOps->{client};
    my $clientHdl = $client->{hdl};
    my $mountPath = $clientOps->{clientmappeddrive};
    my @chars=('a'..'z','A'..'Z','0'..'9','_','$','#');
    my $random_string;
    my $ret = 1;  
    my $length_of_randomstring = 1000;
    my @attr;
    foreach (1..$length_of_randomstring) {
    $random_string.=$chars[rand @chars];
    }
    eval {    
        @attr = write_file(file => "$mountPath\\$fileName", contents => "$random_string", fstat =>1,client => $clientHdl, offset => $offset);
        $client->{test}->nlog("File $mountPath\\$fileName written successfully its stat are " . Dumper(@attr) );
        $ret=1;
       };
   
    if ($@) {
    
        $client->{test}->nlog("Cannot write file : Epic FAIL in lib : $@");
        $ret=0;
        }
        return $ret;
}

sub removeStream
{
	my ($clientOps,$opts) = @_;
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $clientHdl    = $client->{hdl};
	my $streamname = $opts->{streamname};
	my $filename = $opts->{filename};
	my $mountPath = $clientOps->{clientmappeddrive};
	my $ret;
	eval{
		remove_stream(client => $clientHdl, stream => $streamname, file => "$mountPath\\$filename");
		$ret = 1;
		
	};
	if ($@)
	{
		$client->{test}->nlog("Remove stream failed: $@");
		$ret = 0;
	}
	
	return $ret;
}

sub renameStream()
{
	my ($clientOps,$opts) = @_;
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $clientHdl    = $client->{hdl};
	my $streamname = $opts->{streamname};
	my $nstream = $opts->{newstreamname};
	my $filename = $opts->{filename};
	my $mountPath = $clientOps->{clientmappeddrive};
	my $ret;
	eval{
		rename_stream(client => $clientHdl, stream => $streamname, file => "$mountPath\\$filename", nstream => $nstream);
		$ret = 1;
		
	};
	if ($@)
	{
		$client->{test}->nlog("Rename stream failed: $@");
		$ret = 0;
	}
	
	return $ret;
}
	


sub renameFile()
{
        my ($clientOps, $opts) = @_ ;
        my $fileName = $opts->{filename};
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my $filename = $opts->{file};
        my $nfile = $opts->{newfile};
        my $err = $opts->{error};
        my $ret = 1;
        eval{
                $ret = rename_file(file => "$mountPath\\$filename", nfile => "$nfile", client => $clientHdl);
        };
        if ($@)
        {
                $ret = 0;
                $client->{test}->nlog("Rename file failed: $@");
                return $ret;
        }
        if ($err) {
                $client->{test}->nlog("Error got");
                $ret = _process_error($ret, $err, $client);
                return $ret;
        }

       
        return $ret;
} 


sub truncateFile()
{
        my ($clientOps, $opts) = @_ ;
       # my $user = (defined($opts->{user}) ? $opts->{user} : 'root');
        my $fileName = $opts->{filename};
        my $client = $clientOps->{client};
        my $clientHdl = $client->{hdl};
        my $mountPath = $clientOps->{clientmappeddrive};
        my $size = (defined($opts->{size})? $opts->{size} : 0);
        eval{
                file_truncate(file => "$mountPath\\$fileName", size => "$size", client => $clientHdl);
        };
        if ($@)
        {
                $client->{test}->nlog("Truncate file failed: $@");
                return 0;
        }
        return 1;
}



sub verify_acls() {
	#(undef, undef, my $reply) = say($cc, "CACLS.EXE $path", '-swallow');
	#&verify_acls($reply, $unc, "$acldir", "N","$acluser");
	my ($clientOps,$opts) = @_;
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
    	my $reply = $opts->{reply};
    	#my $unc = shift @_;
    	my $dir = $opts->{filename};
	my $acl = $opts->{permission};
	my $user = $opts->{user};
	my $remove = $opts->{remove};
	my $ret = 1;
    
    	my @lines = split /\n/, $reply;
	my $user_found = 0;
    
    	foreach my $line (@lines) {
		my $test_user = "";
		my $test_perm = "";
		print "1 - line = $line\n";
		my @tst_str = split(" ", $line);
		if($acl ne "W"){
			$line = $tst_str[@tst_str - 1];
		} else {
			$line = $tst_str[@tst_str - 2];
		}
		chomp($line);
		print "2 - line = $line, user = $user\n";
        if ($line =~ /$user/) {			
			$user_found = 1;
			if($line =~ /(\(.*\))/){
				$line =~ /(.*\:)(.*\))(.*)/;
				$test_user = $1;
				$test_perm = $3;
			} else {
				$line =~ /(.*\:)(.*)/;
				$test_user = $1;
				$test_perm = $2;
			}	
			print "perm = $test_perm, acl = $acl\n";
			
        if(($test_user =~ /$user/) && ($test_perm eq $acl) && ($acl ne "W")) {
                $client->{test}->nlog("ACL on $dir for $test_user is ok");
            } elsif (($test_user =~ /$user/) && ($acl eq "W") && ($test_perm =~ /special/)){
				$client->{test}->nlog("ACL on $dir for $test_user is ok");
			}else {
                $client->{test}->nlog( "ACL on $dir for $test_user is not expected.  Expected - $acl, got - $test_perm");
		$ret = 0;
            }
			last;
        } else {
			$user_found = 0;
		}
    	}
	if(!$user_found && $remove) {
		$client->{test}->nlog( "ACL on $dir for $user is correct - removed from ACL");
		$ret = 1;
	} elsif(!$user_found) {
		$client->{test}->nlog( "ACL on $dir for $user is not expected.  $user not found");
		$ret = 0;
	}
	return $ret;
}


sub createDataSet ($$) {
	my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $clientHostname = $client->{hostname};
        my $mountPath = ( defined($opts->{writepath}) ? $opts->{writepath} : $clientOps->{clientmappeddrive});
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my $filecount = (defined($opts->{files}) ? $opts->{files} : '150'); 
        my $filesize = (defined($opts->{filesize}) ? $opts->{filesize} : '900'); 
        my $filesizeAvg = (defined($opts->{filesizeavg}) ? $opts->{filesizeavg} : '90000'); 

	$client->{test}->nlog("createDataSet in share : $mountPath win32: $@"); 

        my $ret = 1;   

        eval {        
            my %options = RichData::total_files_pop_opts(files => "$filecount");
            RichData::populate(path=>"$mountPath" , client => $clientHostname, file_size_avg=>"$filesizeAvg" ,
			              dir_avg=>"2", link_num=>"0", slink_num=> "0", file_avg=>"2");

            #RichData::populate(path=>"$mountPath" , client => $clientHostname, file_size_avg=>"$filesizeAvg" ,
	#		              dir_avg=>"2", file_avg=>"2", sparse_num=>"0", link_num=>"0", slink_num=> "0",
	#			      compress_num=>"0", encrypt_num=>"0", archive_num=>"0", hidden_num=>"0", timeout=>"9000");


            $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Problem with createDataSet in win32: $@"); 
             $ret=0;
        }

        return $ret;
}

sub createData ($$) {
	my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $clientHostname = $client->{hostname};
        my $mountPath = ( defined($opts->{writepath}) ? $opts->{writepath} : $clientOps->{clientmappeddrive});
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
        my $filecount = (defined($opts->{files}) ? $opts->{files} : '100'); 
        my $dircount = (defined($opts->{dirs}) ? $opts->{files} : '10'); 
        my ($dcount,$fcount);

	$client->{test}->nlog("createDataSet in share : $mountPath win32: $@"); 

        my $ret = 1;   

        eval {        
            for ($dcount = 1; $dcount <= $dircount; $dcount++) {
                $ret = $clientOps->createDir({'dirname'=>"micadir1_$dcount"});
                    for ($fcount = 1; $fcount <= $filecount; $fcount++) {
	                $ret = $clientOps->createFile({'filename'=>"micadir1_$dcount"."/"."micafile1_$fcount",
                    });
                }
            }

            $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Problem with createDataSet in win32: $@"); 
             $ret=0;
        }

        return $ret;
}

sub readDataSetOld ($$) {
	my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $clientHostname = $client->{hostname};
        my $mountPath = ( defined($opts->{readpath}) ? $opts->{readpath} : $clientOps->{clientmappeddrive});
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
	$client->{test}->nlog("readDataSet in share : $mountPath win32: $@"); 
        my $ret = 1;   

        eval {        
            RichData::read_tree(path=>"$mountPath" , client => $clientHostname);
            $ret=1; 
        }; 

        if ($@) {
	     $client->{test}->nlog("Problem with readDataSet: $@"); 
             $ret=0;
        }

        return $ret;

}

sub readDataSet ($$) {
	my ($clientOps, $opts) = @_ ; 
        my $client = $clientOps->{client};
        my $user = (defined($opts->{user}) ? $opts->{user} : 'root'); 
        my $clientHdl = $client->{hdl};
        my $errorok = (defined($opts->{errorok}) ? $opts->{errorok} : 'true'); 
        my $clientHostname = $client->{hostname};
        my $mountPath = ( defined($opts->{readpath}) ? $opts->{readpath} : $clientOps->{clientmappeddrive});
        my $filerjunctionPath = $clientOps->{filerjunctionpath};
	$client->{test}->nlog("readDataSet in share : $mountPath win32: $@"); 
        my @filesList;
        my $ret = 1; my $errorHit=0;  

        if(defined($opts->{dirname})){
		$mountPath = $mountPath."\\".$opts->{dirname}; 
	}	

        eval {        
	        @filesList = get_file_entries(dir => "$mountPath",
                                    client => $clientHdl,
                                    );

  	    $client->{test}->nlog("Files Recursive listing:".Dumper(@filesList)); 
           
            MYLOOP: foreach (@filesList) {
		my $fileName = $_;
		$fileName  =~ s/\//\\/g;
		$client->{test}->nlog("Reading File in SMB : $fileName \n");
		eval {
		    my @contents = read_file(file => "$fileName",client => $clientHdl);  
		    $client->{test}->nlog("File $fileName contents are read : " . " SMB Read Successfully");                                 
		}; 
		if ($@) {
    		    if ($errorok =~ /true/) {
			next MYLOOP;
		    } else {
			$client->{test}->nlog("File Read error : $@"); 
			$ret=0;$errorHit=1;
		    }
		}
            }
             if($errorHit =~ /1/) { 
		$ret = 0 ;
	     } else {
		$ret = 1 ;
	     }
        }; 

	if ($@) {
	     $client->{test}->nlog("Directory listing error : $@"); 
	     $ret=0;
	}

	return $ret;

}


sub open_file()
{
	my ($clientOps,$opts) = @_;
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};
	my $hostname = $client->{hostname};
	my $clientHdl    = $client->{hdl};
	my $filename = $opts->{filename};
	my $options = $opts->{options};
	my $cmd = "o $drive\\".$filename." ".$options;
	my($spec) = $clientHdl  ;
	my $hostname = $client->{hostname};
	my $ret = 1;
	my $c;
	my $fh;
	my %rethash;
	my $cifs_timeout = (defined($opts->{timeout}) ? $opts->{timeout} :30);
	eval{
    	$c = Connectrec->new(host     => "$hostname",
                            template => "rsh-persist",
                            connect  => 1);
	

	#say($c,"x:\\bin\\open32.exe $drive\\$cmd","-eval_error", "-wait",".*(open succeeded).*","-prompt","[a-z,=,\,,\s,:]*");
	say($c, "x:\\bin\\cifs_test_new.exe","-prompt", "bingo");
	my (undef,undef,$reply,undef) = say($c,$cmd,"-prompt", "bingo");
	print "\nReply: $reply\n";        
	if ($reply =~ /.*.failed.*/i)
        {
                $client->{test}->nlog("Failed: $reply");
		$ret = 0;
                return 0;
        }
	$reply =~ /(=\s\d)/;
        $fh = $1;  
      	$fh =~ s/=\s//; 
       	print "\n filehandle: $fh\n";   
     	$rethash{'filehandle'} = $fh;  
      	$rethash{'connrec'} = $c;    
	print "Connrec:  \n $c \n$rethash{'connrec'}\n";
    	$ret = \%rethash;
	};
	if ($@ && $@ =~ /Err=TIMEOUT/i) {
	     	$client->{test}->nlog("TIMEOUT during command execution: $cmd");
		$ret = 0;
    	}
	

    return $ret;
}


sub close_file()
{   
	my ($clientOps,$opts) = @_;        
	my $client = $clientOps->{client};       
	my $drive = $clientOps->{clientmappeddrive};        
	my $hostname = $client->{hostname};       
	my $clientHdl    = $client->{hdl};       
	my $fh = $opts->{filehandle};       
	my $conn = $opts->{connrec};       
	my $ret = 1;
#	eval{
	my (undef,undef,$reply,undef) = say($conn,"c $fh", "-prompt", "bingo");
    	print "\nReply: $reply\n";
    	if ($reply !~ /SUCCESS/)
	{
		$ret = 0;
	}
	say($conn,"q");
#    	};
	if ($@ && $@ =~ /Err=TIMEOUT/i) {
                $client->{test}->nlog("TIMEOUT during command execution: ");
                $ret = 0;
        }
       

    return $ret;
}

sub fileops()
{
	my ($clientOps,$opts) = @_;       
	my $client = $clientOps->{client};
	my $drive = $clientOps->{clientmappeddrive};   
	my $hostname = $client->{hostname};        
	my $clientHdl    = $client->{hdl};        
	my $cmd = $opts->{command};        
	my $hostname = $client->{hostname};        
	my $ret = 1;        
	my $c = $opts->{connrec};        
	my $fh;        
	my $reply;
	eval{
	    
	  #say($c, "$cmd","-prompt", "bingo");        
	#  $ret = $c->execute(command => "$cmd");#,prompts => /bingo/)
	   (undef,undef,$reply,undef) = say($c,$cmd,"-prompt", "bingo");
	   print "\n O/P: $reply \n";	
	  
 	
	
	};
	if ($@ && $@ =~ /Err=TIMEOUT/i) {
	$client->{test}->nlog("TIMEOUT during command execution: $cmd");
	$ret = 0;
	}
	if ($reply =~ /.*.failed.*/i)
	{
		$client->{test}->nlog("Failed: $reply");
		$ret = 0;
	}
	else{
	$ret = 1;
	}
	
return $ret;
	
}
