package FLEXCACHE_LIB_CMODE;
use warnings;
use NATE::BaseException qw(:try);
use NACL::C::Node();
use NACL::C::Aggregate();
use NACL::C::Volume();
use NACL::APISet;
use Tharn qw($Log);
use Data::Dumper;

require     Exporter;
our @ISA=qw(Exporter);
our @EXPORT = qw( Create_vol Create_aggr Flexcache_delete file_size num_blocks kill_proc system_health write_file_blocks_randomly);
use Data::Dumper;

sub Create_vol($$$$;$$$);
sub Create_aggr($$;$);
sub Flexcache_delete($$$);
sub file_size($);
sub num_blocks($);
sub system_health($);
sub write_file_blocks_randomly($$);

sub Create_vol($$$$;$$$)
{
  my ($node, $vserver, $vol_name, $aggr_name, $size, $junc_path, $perm);
  $node      = $_[0];
  $vserver   = $_[1];
  $vol_name  = $_[2];
  $aggr_name = $_[3];
  $size      = $_[4] ? $_[4] : "100mb";
  $junc_path = $_[5] ? $_[5] : "/$vol_name";
  $perm      = $_[6] ? $_[6] : '0777';

  my ($vol, $vol_state);

  try {
        # Does the volume already exist?
        $vol = NACL::C::Volume->find(
                                       command_interface => $node,
                                       filter            => { volume => $vol_name },
                                    );
      }
      catch NACL::Exceptions::NoElementsFound with
      {
        # The volume does not exist, so create it.
        $vol = NACL::C::Volume->create(
                                        command_interface  => $node,
                                        volume             => $vol_name,
                                        aggregate          => $aggr_name,
                                        vserver            => $vserver,
                                        size               => $size,
                                        'unix-permissions' => $perm,
                                        'junction-path'    => $junc_path,
                                      );

      };


  # What is the state of the volume?
  $vol_state = $vol->state()->state();

  # $Log->comment("volume state=$vol_state");
  if ( $vol_state !~ /online/i )
  {
     NATE::BaseException->throw("\n\n.We quit because volume $vol_name was not online.  Volume state was $vol_state.\n\n");
  }

  return true;

} # end of Create_vol()




sub Create_aggr($$;$)
{
  my ($node, $aggr_name, $ndisks);
  $node      = $_[0];
  $aggr_name = $_[1];
  $ndisks    = $_[2] ? $_[2] : 3;

  my ($aggr, $r);

  try {
        # Does the aggregate already exist?
        $aggr = NACL::C::Aggregate->find ( command_interface => $node,
                                           filter => { 'aggregate' => $aggr_name }
                                         );
      }
      catch NACL::Exceptions::NoElementsFound with
      {
         # The aggregate does not exist, so create it.
         $aggr = NACL::C::Aggregate->create (
                                              'command_interface' => $node,     # $main::server,
                                              'aggregate'         => $aggr_name,
                                              'nodes'             => [ $node ],
                                              'diskcount'         => $ndisks,
                                              'allow-mixed'       => 'true',
                                            );
      };

  # What is the state of the aggregate?
  $r = $aggr->state()->state();

  if ( $r =~ /offline/i )
  {
    NATE::BaseException->throw("\n\nAggregate $aggr_name is OFFLINE.  Therefore, we quit !!!\n\n");
  }

  # if the aggregate is being created, then it is not online yet.
  while( $r !~ /online/i )
  {
    $Log->comment("The state of aggregate is: $r");
    sleep 60;
    $r = $aggr->state()->state();
  }

  return true;
} # End of Create_aggr()




sub system_health($)
{
   my ( $output, $Api_Set_Obj, $rarray, $i, $msg );
   $Api_Set_Obj  = $_[0];

   $output = $Api_Set_Obj->system_node_show();
   Dumper($output-> get_parsed_output());
   $rarray    = $output-> get_parsed_output();

   $i = 0;
   while( defined($rarray ->[$i]{'health'}) )
   {
      $msg = "Health " . $rarray ->[$i]{'node'} . " " . $rarray ->[$i]{'health'};
#     print "$msg \n";
      $Log->comment($msg);
      if ( $rarray ->[$i]{'health'} ne "true")
      {
          return (0);  # false: Not all nodes are healthy/down.
      }
      $i++;
   }
   return (1); # All nodes were healthy/up.
}



sub Flexcache_delete($$$)
{
  my ($Api_Set_Obj, $vserver, $flexcache);
  $Api_Set_Obj = $_[0];
  $vserver     = $_[1];
  $flexcache   = $_[2];

  $Api_Set_Obj-> volume_unmount(
                                 vserver => "$vserver"  ,
                                 volume  => "$flexcache",
                                );

  $Api_Set_Obj-> volume_offline(
                                 vserver => "$vserver"  ,
                                 volume  => "$flexcache",
                               );

  $Api_Set_Obj-> volume_delete(
                                vserver => "$vserver"  ,
                                volume  => "$flexcache",
                              );
}



sub file_size($)    # Return the size of a file.
{
  my ($file_name, $r, $rem, $size, $block, @column );

  $file_name = $_[0];
  $r = `ls -la $file_name 2>&1`;
  if ($r =~ /No such file or directory/i)
  {
    print "\nError: No such file or directory ==> $file_name \n\n";
    exit 1;
  }

  @column = split(" ", $r);
  $size = $column[4];         # file size is in 4th column.

  return ($size);
}



sub num_blocks($)   # Returns how many blocks a file has, counting from 1 and not 0.
{
  my ($file_name, $r, $rem, $size, $block, @column );

  $file_name = $_[0];
  $size = file_size($file_name);

  $rem = $size % 4096;
  $block = int($size / 4096);
  if ($rem > 0)
  {
    $block = $block + 1;
  }
# print "$block\n";
  return $block;
}



sub kill_proc($;;;;)
{
  ##############################################################################################
  # kills multiple instances of the same program or any program that has that string in it.
  # Assume you have 5 "cat"s program running on a client and you want to kill them all:
  # kill_proc cat
  # This also kills a program like "catalog" or "procat" as well since it has cat in it.
  # Also you can kill a set of commands as well i.e.  mykill cat find tail
  # kills all commands that has cat or find or tail in it.  Hence you need to be very carefull.
  ##############################################################################################
  if ($#_ < 0)
  {
    print "\nkill_proc <commands to kill>   \n";
    print "For example: kill_proc cat find  \n\n";
    exit 1;
  }

  # find the userid
  $logname = `id`;
  chomp $logname;
  $logname = substr($logname, index($logname,"(") + 1);
  $logname = substr($logname, 0, index($logname,")"));


#  print `ps -eaf | grep  $logname`, "\n";

  open(FD, "ps -eaf | grep " . $logname . " |");

  $myid = $$;
  $matche_any = join("|", @_);
  $matche_any =~ s/ls /^ls /;                             # Avoid killing xterm with -ls on the exterm line.
  print "\nProcess(es) to kill = \"$matche_any\"\n";

  while ( !(eof FD) )
  {
     $buf = <FD>;
     @lines = split("\n", $buf);
     for ($i = 0; $i <= $#lines; $i++)
     {
       @col = split(" ", $lines[$i]);
       if ( $myid != $col[1] )        # do not kill yourself.
       {
          $cmd_line = join(" ", @col[7..$#col]);
          if ( $cmd_line =~ $matche_any  )
          {
             print "Killing => $cmd_line\n";
             kill 9, $col[1];
          }
       }
     } # end of for loop
  }  # end of while loop
  close FD;
}



sub write_file_blocks_randomly($$)
{
  my ($prog_name_modify_a_block, $file_name, $byte_to_seek_to, $num_blocks, $random_block);

  if ($#_ != 1)
  {
    print "2 parameters were expected: <program name> <file name>, but we got:\n";
    print "program_name=<$_[0]>  file_name=<$_[1]>\n";
    exit 1;
  }

  $prog_name_modify_a_block = $_[0];
  $file_name                = $_[1];

  $num_blocks = num_blocks( $file_name );  # I want the size each time, in case file is growing/shrinking dynamiclly.

  $random_block = int(rand($num_blocks));

  $byte_to_seek_to = $random_block * 4096;
  #  print "$prog_name_modify_a_block \"$file_name\" $byte_to_seek_to X\n\n";
  `$prog_name_modify_a_block \"$file_name\" $byte_to_seek_to X`;
}

1;

