#$Id: //depot/prod/test/main/lib/SPARSE/cmode/NodeObject.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.

## @summary	
##      Library which defines Node Object class  
##      which is container class for all other classes.
## @description
##	Abstract common object Node tasks.
##
## @usage
##	use SPARSE::fcluster::NodeObject;
##
## @author mica@netapp.com

package SPARSE::cmode::NodeObject;

use strict;
use Tharn;
use filer;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);

# Internal reference classes 
use SPARSE::cmode::AggrObject qw(aggr);
use SPARSE::cmode::VserverObject qw(vserver);
use SPARSE::cmode::VolumeObject qw(volume);

# NACL libs
use NACL::APISet;
use NACL::C::Aggregate;
use NACL::C::Vserver;
use NACL::C::ClusterIdentity;
use NACL::C::Snapshot;
use NACL::C::Failover;
use NACL::C::Node;
#use NACL::C::QosPolicyGroup;
use NACL::C::Statistics;
use NACL::C::StatisticsV1;
use NACL::C::AntivirusOnAccessPolicy;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);
use dblade::wafl::tools::panamax::Version qw(get_version_number);

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&node
		);

# ------------------------------------------------------------------------
## Node class definition

## @name	::node
## @summary	create a 10mode node object
## @description
##	Create a 10Mode node object.
## @arg		server object to filer required
## @arg		nodename required
## @arg     test object required
## @arg		opts optional
## @return	node object

sub node(%)
{
   my %fparams = validate(
   @_,
    {
           server  => { type => HASHREF },
           opts    => { type => HASHREF },
    });

   my $server = $fparams{server};
   my $opts= $fparams{opts};

   my $aggr    = {};
   my $vserver = {};
   my $node    = {};
   my $test    = $opts->{test};

   $node->{name}	  = $opts->{name};   # this is key for this object 
   $node->{server}	  = $server; 
   $node->{aggrs}     = $aggr;  # this is a hash of hashrefs of aggr objs  
   $node->{vservers}  = $vserver; # this is a hash of hashrefs of vserver objs
   $node->{test}      = $opts->{test};
   $node->{params}    = $opts->{params};
   $node->{execname}  = $opts->{execname};

   #$test->nlog("Created node object : " . Dumper($node));
	
   bless($node);
   return $node;
}

sub DESTROY($)
{
	my ($node) = @_;
	my $exc_state = $@;
	#$node->{test}->nlog("destroying $node->{name} object");
	$@ = $exc_state;
}

# ------------------------------------------------------------------------
## Node Object methods 

sub createAggr($$)
{
    my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $exec_server = $node->{execname}; 
    my $result = 1;
    my $aggrName = $opts->{aggrname};
    my $aggrSize = $opts->{aggrsize};
    my $raidtype;
    $raidtype = ($aggrSize == 2) ? 'raid4':'raid_dp';
    my $newaggr;
    try {
             $newaggr = NACL::C::Aggregate->create(
             command_interface => $Server,
             aggregate         => $aggrName,
             diskcount         => $aggrSize,
             nodes             => [$exec_server],
			 raidtype          => $raidtype
		 );

     	$test->nlog("Waiting till the aggregate is created ".
     		        "on node ".$node->{name}."\n");
        while ($newaggr->state()->state() !~ /online/) {
     		sleep 5;
     	}
     	$test->nlog("Aggregate ".$newaggr->state()->aggregate()." created".
     			    " on node ".$node->{name}."\n");
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Aggr Create Failed with Exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    
    #Add the aggr obj to our data representation if aggregate creation passed  
    if ($result) {
		$opts = {};$opts->{name} = $aggrName;
		my $aggrObj = aggr(node => $node,opts => $opts);
		$node->{aggrs}->{$aggrName} = $aggrObj;
    }
	return $result;	    
}

sub createStripeAggr($$)
{
    my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
#   my $exec_server = $node->{execname}; 
    my $result = 1;
    my $aggrNodes = $opts->{aggrnodes};   
    my $aggrName = $opts->{aggrname};
    my $aggrSize = $opts->{aggrsize};
    my $raidtype;
	$raidtype = ($aggrSize == 4) ? 'raid4':'raid_dp';
	$test->nlog("The array aggrNodes contains: @$aggrNodes  \n");
	my $newaggr;
    try {
             $newaggr = NACL::C::Aggregate->create(
             command_interface => $Server,
             aggregate         => $aggrName,
             diskcount         => $aggrSize,
             nodes             => $aggrNodes, 
			 raidtype          => $raidtype
		 );

     	$test->nlog("Waiting till the stripe aggregate is created ".
     		        "on @$aggrNodes nodes "."\n");        ## add teh name of one more node here.
        while ($newaggr->state()->state() !~ /online/) {
     		sleep 5;
     	}
     	$test->nlog("Aggregate ".$newaggr->state()->aggregate()." created".
     			    " on nodes ". @$aggrNodes."\n");

    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Aggr Create Failed with Exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    
   
    #Add the aggr obj to our data representation if aggregate creation passed  
    if ($result) {
		$opts = {};$opts->{name} = $aggrName;
		my $aggrObj = aggr(node => $node ,opts => $opts);
		$node->{aggrs}->{$aggrName} = $aggrObj;
    }
	return $result;	    
}
sub deleteAggr($$)
{
	my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $result = 1;
    my $aggrNameFilter = $opts->{aggrname};
	my $aggr_obj;
	
    try {
        $aggr_obj = NACL::C::Aggregate->find( 
						command_interface => $Server, 
						filter            => { 
											   aggregate => "$aggrNameFilter"
											 });  
		$aggr_obj->delete();
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Aggregate $aggrNameFilter delete Failed with exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    

    if ($result) {
		delete $node->{aggrs}->{"$aggrNameFilter"};
	}
	return $result;	    
}

sub showAggr($$)
{        
    my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $exec_server = $node->{execname};
    my $result = 1;
    my $aggrNameFilter = $opts->{aggrname};

    try {
        my $aggr_members = NACL::C::Aggregate->find( 
			command_interface => $Server, 
            filter            => { state => "online",
                                   nodes => [$exec_server],
                                   aggregate => "$aggrNameFilter",
                                  });  
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Aggregate $aggrNameFilter not present - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    

    return $result;	    
    
}

sub addAggrObject($$)
{
    my ($node,$opts) = @_;
    my $aggrname = $opts->{'name'};        
    my $aggrObj = aggr($node,$opts);
    $node->{aggrs}->{"$aggrname"} = $aggrObj;
}

sub createVserver($$)
{
    my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $result = 1;
    my $vserverName = $opts->{vservername};
    my $vserverrootvol = $vserverName."rootvol".$$;
	my $newvserver;
	
   try {
        $newvserver = NACL::C::Vserver->create(
                      command_interface => $Server,
                      vserver           => $vserverName,
                      rootvolume        => $vserverrootvol,
                      aggregate         => $opts->{rootaggr},
                      "ns-switch"       => "file",
                      "rootvolume-security-style" => "mixed",
				      );


    $test->nlog("Vserver ".$newvserver->state()->vserver()." created".
     			    " on node ".$node->{name}."\n");

   } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Vserver create fails with Exception - ".$ex->text);
        $result = 0;
   } catch NATE::Error with {
        my $ex = shift;
        $test->nlog("Vserver create fails with Exception - ".$ex->text);
        $result = 0;
   };
    
   
    #Add the aggr obj to our data representation if aggregate creation passed  
    if ($result) {
		$opts = {};$opts->{name} = $vserverName;
		$node->addVserverObject($opts);
    }
	return $result;	    
}

sub deleteVserver {
    my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $result = 1;
    my $vserverName = $opts->{vservername};
    my $vserver_obj;
    my $rootvolname;
    my $vserver = $node->{vservers}->{$vserverName}; 

   eval {
      $rootvolname = $node->{vservers}->{$vserverName}->get_root_vol();   
 
      my $opts = {};
      $opts->{type} = 'RW'; 
      $opts->{volname} = $rootvolname;
      $opts->{nacl_volume} = ''; 
      
      my $volumeObj = volume(vserver => $vserver,opts => $opts);
      $node->{vservers}->{$vserverName}->{vols}->{$rootvolname} = $volumeObj;
      #$node->{vservers}->{$vserverName}->{vols}->{$rootvolname}->unmount(); 
      $test->nlog("Offlining Root Volume");
      $node->{vservers}->{$vserverName}->{vols}->{$rootvolname}->offline(); 
      $test->nlog("Deleting Volumes");
      $node->{vservers}->{$vserverName}->deleteFlexVol({volname => "$rootvolname"});    

   };
   
   if ($@) {
      $test->nlog("Epic FAIL in lib : $@"); 
      return 0;	
   }	
	
   try {
        $vserver_obj = NACL::C::Vserver->find(
                      command_interface => $Server,
                      filter            =>  {
						vserver           => $vserverName
					     }	
                      );

       $vserver_obj->delete();
   } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Vserver delete fails with Exception - ".$ex->text);
        $result = 0;
   } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
   };
    	
   delete $node->{vservers}->{"$vserverName"};	
   return $result;
}

sub showVserver($$)
{
    my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $result = 1;
    my $vserverNameFilter = $opts->{vservername};

    try {
        my @vserver_members = NACL::C::Vserver->find( 
							  command_interface => $Server, 
							  filter            => { 
														vserver=> "$vserverNameFilter",
													}); 
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Vserver not present - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    
    return $result;	    
}

sub addVserverObject($$)
{
    my ($node,$opts) = @_;
    my $vservername = $opts->{'name'};        
    my $vserverObj = vserver(node=>$node,opts=>$opts);
    $node->{vservers}->{"$vservername"} = $vserverObj;
}


sub getNumberOfNodesInCluster($$) {
    my ($node,$opts) = @_;
    #my $nodes = $node->{server}->{types}->{'clustered'}; 
    my $noOfNodes = $node->{params}->{NUM_NODES};     
    #$noOfNodes += scalar keys %$nodes;
    #$node->{test}->nlog("Number of nodes are : $noOfNodes"); 
    return $noOfNodes;
}

sub getClusterName($$) {
	my ($node, $opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $result = 1;
    my $clusname;
	
	try {
        $clusname = NACL::C::Vserver->find(
                      command_interface => $Server,
                      filter            =>  {
												type  => "admin"
                      });

	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("admin vserver not present - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
	};
    
	$result = $clusname->vserver();
	return $result;
#	my $clusname = NACL::C::Cluster->find(command_interface => $node);

}

sub getNodeName($$) {
	my ($node, $opts) = @_;
    my $nodename = $node->{execname};
	return $nodename;
}

sub snapshotShow($$){
	my ($node,$opts) = @_; 
	my $Server = $node->{server};
	my $test = $node->{test};
    	my $exec_server = $node->{execname}; 
	
	
    	my $result = 1;
	my @ret;
	try {
        my @snapshots = NACL::C::Snapshot->find(command_interface => $Server); 

        #iterate and show neatly
        foreach my $snapshot (@snapshots) {
            my $snapstate =  $snapshot->state();
	    $test->nlog( "Volume name =" . $snapstate->volume() );     
            $test->nlog( "Volume vserver =" . $snapstate->vserver() );     
            $test->nlog( "Blocks =" . $snapstate->blocks() );     
            $test->nlog( "Snapshot =" . $snapstate->snapshot() );     
            $test->nlog( "dsid =" . $snapstate->dsid() );     
            $test->nlog( "msid =" . $snapstate->msid() );     
            $test->nlog( "create_time =" . $snapstate->create_time() );     
            $test->nlog( "Logical snap id =" . $snapstate->logical_snap_id() );     
	    push @ret,$snapstate;
                 
        }        

    } catch NATE::BaseException with {
        $test->nlog( "No snapshots  exists  ");  
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        #$test->nlog("In the BaseException catch");
        #$test->nlog(
        #                  "\nfile is       " . $ex->file()
        #                . "\nline is       " . $ex->line()
        #                . "\ntext is       " . $ex->text
        #                . "\nstringify is  $ex"
        #                . "\nobject ref is $ref_of_obj\n"
        #                . "\nstacktrace:\n" . $ex->stacktrace()
        #                . "\n\n\n\n"
        #                . "\ndumpout:\n" . $dumpout);
        $result = 0;
		return $result;
    } catch NATE::Error with {
        $test->nlog( "No snapshots exist");     
        #$test->nlog("In the Error catch");
        $result = 0;
		return $result;
    };
    if ($@) {
        $test->nlog( "No snapshots exist in the node");     
        #$test->nlog("except: In the conditional");
        $result = 0;
	return $result;
    }
	return @ret;
}


sub nodeRun()
{
	my ($node,$opts) = @_; 
	my $Server = $node->{server};
    	my $test = $node->{test};
    	my $exec_server = $node->{execname}; 
	my $command = $opts->{command};
	my $result = 1;
	try {
	
	$result = NACL::C::Node->run(command_interface => $Server, 
					'command'=> $command, 'node' => $exec_server );
	
					
	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	$test->nlog("Node run $command result:".Dumper($result));
	return $result;
}

sub avPolicyCreate()
{
	my ($node,$opts) = @_;
	my $policyname = $opts->{policyname};
	my $Server = $node->{server};        
	my $test = $node->{test};
	my $result = 1;
	my $protocols = (defined($opts->{protocols}) ? $opts->{protocols} : "cifs,nfs4" );
	my $vserver = $opts->{vserver};
	try {
        my $ret= NACL::C::AntivirusOnAccessPolicy->create( 
							  command_interface => $Server, 
							  name		=> $policyname,
							  scan		=> "on",
							  'scan-mandatory' => "on",
							  protocols	=> $protocols,
							  vserver => $vserver,
							  ); 
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Vserver not present - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	return $result;
	
}




sub avPolicyModify()
{
	my ($node,$opts) = @_;
	my $policyname = $opts->{policyname};
	my $scan = $opts->{scan};
	my $Server = $node->{server};        
	my $test = $node->{test};
	my $result = 1;
	my $vserver = $opts->{vserver};
	try {
        my $ret= NACL::C::AntivirusOnAccessPolicy->modify( 
							  command_interface => $Server, 
							  name		=> $policyname,
							  scan		=> "$scan",
							  vserver => $vserver,
							  ); 
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Could not modify policy - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	return $result;
}

sub avPolicyDelete()
{
	my ($node,$opts) = @_;
	my $policyname = $opts->{policyname};
	my $Server = $node->{server};                 
	my $test = $node->{test};
	my $result = 1;
	my $vserver = $opts->{vserver};
	try {
        my $ret= NACL::C::AntivirusOnAccessPolicy->delete( 
							  command_interface => $Server, 
							  name		=> $policyname,
							  vserver => $vserver,
							  ); 
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Vserver not present - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	return $result;
	
}

sub raw_send()
{
      my ($node,$opts) = @_;
      my $Server = $node->{server};                 
      my $test = $node->{test};
      my $result = 1;
      my $exec_server = $node->{execname}; 
      my $cmd = $opts->{cmd};
      
      eval {
	 my $c = Connectrec->new(host => "$exec_server",
                            template => "ssh",
			    conntype => "libssh-persist",
			    username => 'admin',
			    password => 'netapp1!',
                            connect  => 1,
                            timeout => '200');
       
         (undef, undef, $result) = say($c,$cmd,"-eval_error");
      };
      if ($@) {
	 $test->nlog("Epic FAIL in lib : $@"); 
	 return 0;	
      } else {
	 return $result;
      }
}

sub sfoTakeover($$)
{
   my ($node,$opts) = @_;
   my $Server = $node->{server};
   my $test = $node->{test};
   my $result = 1;
   my $nodeName =  $node->{execname};
	
   try {
        NACL::C::Failover->takeover(
                      'command_interface' => $Server,
                      'bynode' => $nodeName,
		      'option' => 'force',
 		     );

    $test->nlog("Takeover Initiated by node : $nodeName");

   } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Takeover Failed due to ".$ex->text);
        $result = 0;
   } catch NATE::Error with {
        my $ex = shift;
        $test->nlog("Takeover Failed due to ".$ex->text);
        $result = 0;
   };
    
   return $result;	    
}


sub sfoGiveback($$)
{
    my ($node,$opts) = @_;
    my $Server = $node->{server};
    my $test = $node->{test};
    my $result = 1;
    my $nodeName =  $node->{execname};
	
   try {
        NACL::C::Failover->giveback(
                      'command_interface' => $Server,
                      'fromnode' => $nodeName,
                      'require-partner-waiting' => 'false',
		      'override-vetoes' => 'true'
 		     );

    $test->nlog("Giveback Initiated by node : $nodeName");

   } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Giveback Failed due to ".$ex->text);
        $result = 0;
   } catch NATE::Error with {
        my $ex = shift;
        $test->nlog("Giveback Failed due to ".$ex->text);
        $result = 0;
   };
    
   return $result;	    
}

sub sendHomeDlifs($$)
{
    my ($node,$opts) = @_;    
    my $test = $node->{test};	 
    my $Server = $node->{server};
    my $scriptParams = $node->{params};

    #my ($node,$opts) = @_;
    #my $Server = $node->{server};
    #my $test = $node->{test};
    my $exec_server = $node->{execname};       
    my $result = 1;my ($i,$j,$k);
    my $interface_obj;
	
    my $numNodes = $scriptParams->{NUM_NODES};
    for ($i=1;$i<=$numNodes;$i++) {
    for ($j=1;$j<=($scriptParams->{NUM_VSERVERS});$j++) {
	
    my $numDlifs = $scriptParams->{"NODE"."$i"."_VSERVER"."$j"."_NUMDLIFS"};
    	for ($k=1;$k<=$numDlifs;$k++) {
		$test->nlog("Checking up VSERVER dlifs " . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME" );

 		my $dlifName;
		my $curVserverName = $scriptParams->{"VSERVER" . "$j" . "_NAME"};
                $test->nlog("VS NAME = $curVserverName");
                    
		    if($scriptParams->{VSIM_RUN} eq 'true') {
                        $test->nlog("*********Running in a VSIM *********");
                         
                        if ($scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"} !~ /_data1/) {
                            $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"} . "_data1";
                        } else {
                            $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"};
                        }
                    } else {
                            $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"};
                    }
					
		try {
			$interface_obj =  NACL::C::Interface->find(
									command_interface => $Server,
									filter => {
										'server' => "$curVserverName",
										'lif' => "$dlifName"
									}
								);
                       my $todoLif = $interface_obj->state()->lif();
		       my $todoLifHome = $interface_obj->state()->home_node();
		       my $todoLifCurrHome = $interface_obj->state()->curr_node();
	   	       if($interface_obj->state()->home_node() ne $interface_obj->state()->curr_node() ) {
			  $test->nlog("\n $todoLif  does not exist on its home node $todoLifHome , it lives on remote node $todoLifCurrHome, so start migrate \n");

			   # start migrate dlifs are not safe without being in home
			   NACL::C::Interface->migrate(
			             command_interface => $Server,
			             server            => $curVserverName,
			             lif               => $dlifName,
			             'dest-node'       => $interface_obj->state()->home_node(),
			             'source-node'     => $interface_obj->state()->curr_node(),
			   );
	                }

		} catch NATE::BaseException with {
	            my $ex = shift;
	            $test->nlog("In the BaseException catch");
	            $test->nlog("\nException caused is       " . $ex->text);
	            $result = 0;
		};

	}
    }
    }
    
   return $result;	    

}

sub qosPolicyGroupCreate($$)
{
    my ($node,$opts) = @_;    
    my $test = $node->{test};	 
    my $Server = $node->{server};
    my $scriptParams = $node->{params};
    my $exec_server = $node->{execname};  
    my $pol_name = $opts->{'policy-name'};
    my $vserver = defined($opts->{vserver})?$opts->{vserver}:$scriptParams->{'VSERVER1_NAME'}; 
    my $max_throughput = defined($opts->{max_throughput})?$opts->{max_throughput}:'INF'; 
    my $target_throughput = defined($opts->{target_throughput})?$opts->{target_throughput}:0;
    my $target_latency = defined($opts->{target_latency})?$opts->{target_latency}:'INF';
    my $min_latency = defined($opts->{min_latency})?$opts->{min_latency}:'INF';
    my $result = 1;
    my $interface_obj;
    my $Api_Set_Obj;
    my $version = get_version_number($Server);
     
    try {
        
        my $host = host($exec_server);
        my $version_manager = NACL::APISet::Version->new (hostobj => $host, category => "Node", shell_type => "CMode");
        $Api_Set_Obj = NACL::APISet->new(hostobj => $host, 
                                category => "Node", 
                                interface => "CLI", 
                                set => "CMode",
                                #cdef_path => "/u/kgokul/web/updated_cdefs/", ## hard coded cdef path. Remove the parameter "cdef_path" later when NACL builds the cdef
                                version_info => {
                                compatibility => 0, # Disable compatibility
                                version_manager => $version_manager,
                                
                                      }
				 
                                 );
        if ($version =~ /8\.1/ || $version =~ /8\.0/ || $version =~ /7\.\d/){
	    $Api_Set_Obj->qos_policy_group_create('policy-group' => $pol_name,
                      'max-throughput' => $max_throughput,
                      'target-throughput' => $target_throughput,
                      'target-latency' => $target_latency,
                      'min-latency' => $min_latency);
	} else {
	    $Api_Set_Obj->qos_policy_group_create(
		      'policy-group' => $pol_name,
                      'max-throughput' => $max_throughput,
                      vserver => $vserver);
	}
        
        # NACL::C::QosPolicyGroup->create(
                      # 'command_interface' => $Server,
                      # 'policy-group' => $pol_name,
                      # 'max-throughput' => $max_throughput,
                      # 'target-throughput' => $target_throughput,
                      # 'target-latency' => $target_latency,
                      # 'min-latency' => $min_latency,
		      
 		# );

    $test->nlog("Qos Policy-Group create");

   } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Qos Policy-Group create Failed due to ".$ex->text);
        $result = 0;
   } catch NATE::Error with {
        my $ex = shift;
        $test->nlog("Qos Policy-Group create Failed due to ".$ex->text);
        $result = 0;
   };
   
    return $result;
}


sub qosPolicyGroupDelete($$)
{
    my ($node,$opts) = @_;    
    my $test = $node->{test};	 
    my $Server = $node->{server};
    my $scriptParams = $node->{params};
    my $exec_server = $node->{execname};  
    my $pol_name = $opts->{'policy-name'};
    my $result = 1;
    
    try {
        
        my $host = host($exec_server);
        my $version_manager = NACL::APISet::Version->new (hostobj => $host, category => "Node", shell_type => "CMode");
        my $Api_Set_Obj = NACL::APISet->new(hostobj => $host, 
                                category => "Node", 
                                interface => "CLI", 
                                set => "CMode",
                                #cdef_path => "/u/kgokul/web/updated_cdefs/", ## hard coded cdef path. Remove the parameter "cdef_path" later when NACL builds the cdef
                                version_info => {
                                compatibility => 0, # Disable compatibility
                                version_manager => $version_manager,
                                
                                      }
                                 );
        $Api_Set_Obj->qos_policy_group_delete('policy-group' => $pol_name);
        
        # NACL::C::QosPolicyGroup->delete(
                      # 'command_interface' => $Server,
                      # 'policy-group' => $pol_name,
                      
		      
 		# );

    $test->nlog("Qos Policy-Group delete");

   } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Qos Policy-Group delete Failed due to ".$ex->text);
        $result = 0;
   } catch NATE::Error with {
        my $ex = shift;
        $test->nlog("Qos Policy-Group delete Failed due to ".$ex->text);
        $result = 0;
   };
   
    return $result;
}

sub statisticsShow($$){
    my ($node,$opts) = @_; 
    my $test = $node->{test};	 
    my $Server = $node->{server};
    my $scriptParams = $node->{params};
    my $exec_server = $node->{execname};  
    my $workload = $opts->{workload};
    my $object = defined($opts->{object})?$opts->{object}:'workload';
    my $result = 1;
    my @stats_obj;
    my $stats={};

    my @counters = qw(hit_pure_perct hit_verify_perct miss_pure_perct miss_verify_perct proxy_perct);
    my $version = get_version_number($Server);

    try {
	 if ($version =~ /8\.1/ || $version =~ /8\.0/ || $version =~ /7\.\d/){
        	@stats_obj =  NACL::C::Statistics->find (
                                command_interface => $Server,
                                filter => {
				    node => $exec_server,
				    object => $object,
				    instance => $workload,
				 }
			      );
	 } else {
	    @stats_obj =   NACL::C::StatisticsV1->find (
				 command_interface => $Server,
				 filter => {
				    node => $exec_server,
				    object => $object,
				    instance => $workload,
				 }
			   );
	 }
	
	} catch NATE::BaseException with {
	    my $ex = shift;
	    $test->nlog("Unable to show statistics - ".$ex->text);
	    $result = 0;
	    return $result;
	
	} catch NATE::Error with {
	    my $ex = shift;
	    $test->nlog("In the Error catch".$ex->text);
	    $result = 0;
	    return $result;
        };
	       

        if($result) {

	foreach my $stat (@stats_obj){
        my $curState = $stat->state();	     
	    my $uniqueid = $curState->{counter};
	    $stats->{"$uniqueid"."_value"} = $curState->{value};
	    $stats->{"$uniqueid"."_delta"} = $curState->{delta};
        }

        }
     		
    return $stats;

}


sub getStatistics($$){
    my ($node,$opts) = @_; 
    my $Server = $node->{server};
    my $scriptParams = $node->{params};
    my $exec_server = $node->{execname};
    my $instance = $opts->{instance};
    my $object = $opts->{object};
    my $counter = $opts->{counter};

    my $version = get_version_number($Server);
    my $statistics = "statistics-v1";

    if ($version =~ /8\.1/ || $version =~ /8\.0/ || $version =~ /7\.\d/){
        $statistics = "statistics";
    }
    my $cmd = "\"set -privilege diag -confirmations off; $statistics show -node $exec_server -object $object -instance $instance -counter $counter\"";	 
    my $ret = $node->raw_send({cmd => $cmd});
         		
    return $ret;

}


# .... likewise

1;

 
   
