#$Id: //depot/prod/test/main/lib/SPARSE/cmode/VolumeObject.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.

## @summary	
##      Library which defines Volume Object properties 
##      and actions.
## @description
##	Abstract common object tasks.
##
## @usage
##	use SPARSE::cmode::VolumeObject;
##
## @author mica@netapp.com

package SPARSE::cmode::VolumeObject;

use strict;
use Tharn;
use filer;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);

# Internal reference classes 

# NACL libs
use NACL::APISet;
use NACL::C::Volume;
use NACL::C::VserverCifsShare;
use NACL::C::Statistics;
use NACL::C::Snapshot;
use NACL::C::Node;
use NACL::C::VolumeQtree;
use NACL::C::QosWorkload;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&volume
		);
our %temp_hash = ( 'Filename' => 'filename',
                                'Type'  => 'type',
                                'Mode'  => 'mode',
                                'Nlinks'    => 'nlinks',
                                'UID'   => 'uid',
                                'GID'   => 'gid',
                                'Size'  => 'size',
                                'Blocks'    => 'blocks',
                                'Fsid'  => 'fsid',
                                'Fileid'    => 'fileid',
                                'Xinode'    => 'xinode',
                                'Sinode'    => 'sinode',
                                'AV version'    => 'avversion',
                                'Security'  => 'security',
                                'Last access time'  => 'last-access-time',
                                'Last modified time'    => 'last-modified-time',
                                'Last change time'  => 'last-change-time',
                                'Creation time' => 'creation-time',
                                'Inode Generation #'    => 'inode-generation',
                                'FlexCache Time since last fill' => 'time-since-last-fill',
                                'FlexCache Local Blocks'    => 'local-blocks',
                                'FlexCache Local Flags' => 'local-flags'
                            );
                      
# ------------------------------------------------------------------------
## Volume class definition

## @name	::volume
## @summary	create a 10mode volume object
## @description
##	Create a 10Mode volume object.
## @arg		server object to filer required
## @arg		nodename required
## @arg		opts optional
## @return	node object
				
sub volume(%)
{
    my %fparams = validate(
    @_,
    {
           vserver  => { type => OBJECT },
           opts       => { type => HASHREF },
    });

    my $vserver = $fparams{vserver};
    my $opts    = $fparams{opts};
    my $volume = {};
    my $cifsshares = {};
	
    $volume->{name}	 = $opts->{volname};
    $volume->{vserver}  = $vserver;
    $volume->{nacl_volume} = $opts->{nacl_volume};
    $volume->{type} = $opts->{type}; 
    $volume->{cifsshares} = $cifsshares;
    
    #$vserver->{node}->{test}->nlog("volume object created : " . Dumper($volume));
    bless($volume);

    return $volume;
}

sub DESTROY($)
{
	my ($volume) = @_;
	my $exc_state = $@;
	$volume->{vserver}->{node}->{test}->nlog("destroying $volume->{name} object");
	$@ = $exc_state;
}


# ------------------------------------------------------------------------
## Volume class methods

sub volmount($$) {
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
	my $volname = $volume->{name};
    my $nacl_volume;
    #$nacl_node->apiset_must( { set => 'CMode' } );
   
    try {
            $nacl_volume = NACL::C::Volume->mount( 
			command_interface => $Server, 
            volume => $volname,
            vserver => "$vserver->{name}",
			"junction-path" => "/$volname",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("Volume mount fails with exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }

    return $result;	    
}


sub unmount($$) {
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    #my $nodehostrec = host($exec_server);
    #my $nacl_node = NACL::C::Node->new(name => $exec_server,
	#								   hostrec => $nodehostrec);
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_volume;
    #$nacl_node->apiset_must( { set => 'CMode' } );
   
    try {
            $nacl_volume = NACL::C::Volume->unmount( 
			command_interface => $Server, 
            volume => "$volume->{name}",
            vserver => "$vserver->{name}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("Volume unmount fails with exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    #if ($@) {
    #    $test->nlog("except: In the conditional");
    #    $result = 0;
    #}

    return $result;	    
}

sub offline($$) { 
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->offline( 
			command_interface => $Server, 
            volume => "$volume->{name}",
            vserver => "$vserver->{name}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
	$test->nlog("Volume offline fails with Exception -".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    #if ($@) {
    #    $test->nlog("except: In the conditional");
    #    $result = 0;
    #}

    return $result;	    
}

sub online($$) { 
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->online( 
			command_interface => $Server, 
            volume => "$volume->{name}",
            vserver => "$vserver->{name}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    #if ($@) {
    #    $test->nlog("except: In the conditional");
    #    $result = 0;
    #}

    return $result;	    

}

sub move($$) { 
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};   
	my $destAggr = $opts->{destAggr};
    my $result = 1;
	
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->move( 
			command_interface => $Server, 
            volume => "$volume->{name}",
            vserver => "$vserver->{name}",
			"destination-aggregate" => $destAggr,
			foreground => "true",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
	$test->nlog("Volume move fails with Exception -".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }

    return $result;	    
}

sub showVol ($$) {
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $vol = $volume->{name};
    my $result = 1;
    my $nacl_volume;
    my $opts = {};
    
    try {
        $nacl_volume = NACL::C::Volume->find( 
							command_interface => $Server, 
							filter            => { volume => "$vol",
										 vserver => $volume->{vserver}->{name},
										 %{$opts},
										}); 
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog( "NATE exception No volume : $vol exists in vserver $volume->{vserver}->{name}".$ex->text);  
	$result = 0;
    } catch NATE::Error with {
        my $ex = shift;
	$test->nlog( "NATE Error : $vol exists in vserver $volume->{vserver}->{name}".$ex->text);     
        $result = 0;
    };
    if ($@) {
        $test->nlog( "Exception : $vol exists in vserver $volume->{vserver}->{name} $@");     
        $result = 0;
    }
	
    if($result) {
		my $vol_state = $nacl_volume->state();
		return $vol_state;
    }
    return $result;	    
}


sub rename($$) { 
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->rename( 
			command_interface => $Server, 
			volume => "$volume->{name}",
			vserver => "$vserver->{name}",
			newname => "$opts->{newname}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }

    # need to modify the name inside the object only, dont change object. 
    $volume->{name} = $opts->{newname};

    return $result;	    

}

sub setSize($$) { 
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->modify( 
			command_interface => $Server, 
			volume => "$volume->{name}",
			vserver => "$vserver->{name}",
			size => "$opts->{size}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }

    return $result;	    

}

sub vldb_check($$) {
	my ($volume_obj, $opts) = @_;
	my $Server = $volume_obj->{vserver}->{node}->{server};
        my $test = $volume_obj->{vserver}->{node}->{test}; 
        my $vserver = $volume_obj->{vserver};        
        my $origin = $opts->{origin};
	my $origin_vol_obj;
	my $cache_vol_obj;
        my ($origin_msid,$cache_msid,$origin_dsid,$cache_dsid);

	$origin_vol_obj = NACL::C::Volume->find( 
							command_interface => $Server, 
							filter            => { volume => $origin,
												   vserver => $vserver->{name},
												  }); 
	$cache_vol_obj = NACL::C::Volume->find( 
							command_interface => $Server, 
							filter            => { volume => $volume_obj->{name},
												   vserver => $vserver->{name},
												  }); 
        $origin_msid = $origin_vol_obj->state()->msid();
	$cache_msid = $cache_vol_obj->state()->msid();
	$origin_dsid = $origin_vol_obj->state()->dsid();
	$cache_dsid = $cache_vol_obj->state()->dsid();

	if( $origin_msid != $cache_msid ) {
    		$test->nlog("Origin and Cache Volumes should have same MSID but they do not have.");
                $test->nlog("Origin MSID : ". $origin_msid.", Cache MSID : ".$cache_msid);
                $test->nlog("Origin DSID : ". $origin_dsid.", Cache DSID : ".$cache_dsid);
		return 0;
	}
	
	if( $origin_dsid == $cache_dsid ) {
		$test->nlog("Origin and Cache Volumes should not have same DSID but they have.");
                $test->nlog("Origin MSID : ". $origin_msid.", Cache MSID : ".$cache_msid);
                $test->nlog("Origin DSID : ". $origin_dsid.", Cache DSID : ".$cache_dsid);
		return 0;
	}

	$test->nlog("Cache volume MSID->DSID Mapping is correct");
        $test->nlog("Origin MSID : ". $origin_msid.", Cache MSID : ".$cache_msid);
        $test->nlog("Origin DSID : ". $origin_dsid.", Cache DSID : ".$cache_dsid);

        return 1;
}

sub setFlexCachePolicy($$){
        my ($volume,$opts) = @_; 
        my $test = $volume->{vserver}->{node}->{test};
        my $exec_server = $volume->{vserver}->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        my $vserver = $volume->{vserver};
        my $volname = $volume->{name};
        my $parsed_output;
        my $parsed_hash = 1;
        eval {
            my $raw_output;
      	    $Server->set_execution_server(-Server => $exec_server);
            $parsed_output = $Server->volume_modify(vserver=>$vserver->{name},
                             volume=>"$volname",
                             'flexcache-cache-policy'=>"$opts->{policy}"); 
        }; 
        if ($@) {
		    $test->nlog("Flexcache cache policy apply for volume faileds : $@"); 
            return 0;
	    }
	    return 1;
}

sub getFlexCachePolicy ($$) {
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname};       
    my $result = 1;

    $test->nlog( "opts :: volname = " . $opts->{volname});     
   
    try {
        my @vol_members = NACL::C::Volume->find( 
			command_interface => $Server, 
                        filter  => {
			    volume => "$opts->{volname}",
			    vserver => "$volume->{vserver}->{name}"
			    },); 

        #iterate and show neatly
        foreach my $vol_member (@vol_members) {
            #This loop executes only once.
            my $volState =  $vol_member->state();
            $test->nlog( "Volume name =" . $volState->volume() );     
            $test->nlog( "Volume vserver =" . $volState->vserver() );     
            $test->nlog( "Volume aggregate =" . $volState->aggregate() );     
            $test->nlog( "Volume size =" . $volState->size() );     
            $test->nlog( "Volume dsid =" . $volState->dsid() );     
            $test->nlog( "Volume msid =" . $volState->msid() );     
            $test->nlog( "Volume type =" . $volState->type() );     
            $test->nlog( "Volume junction path =" . $volState->junction_path() );     
            $test->nlog( "Volume state =" . $volState->state() );     
            $test->nlog( "Volume Flexcache cache policy =" . $volState->flexcache_cache_policy() );     
            $result = $volState->flexcache_cache_policy();
        }        

    } catch NATE::BaseException with {
        $test->nlog( "No volume :$opts->{volname} exists in vserver $volume->{vserver}->{name}");     
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        #$test->nlog("In the BaseException catch");
        #$test->nlog(
        #                  "\nfile is       " . $ex->file()
        #                . "\nline is       " . $ex->line()
        #                . "\ntext is       " . $ex->text
        #                . "\nstringify is  $ex"
        #                . "\nobject ref is $ref_of_obj\n"
        #                . "\nstacktrace:\n" . $ex->stacktrace()
        #                . "\n\n\n\n"
        #                . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog( "No volume :$opts->{volname} exists in vserver $volume->{vserver}->{name}");     
        #$test->nlog("In the Error catch");
        $result = 0;
    };
    if ($@) {
        $test->nlog( "No volume :$opts->{volname} exists in vserver $volume->{vserver}->{name}");     
        #$test->nlog("except: In the conditional");
        $result = 0;
    }
 
    return $result;	    
}

sub getFlexCachePolicy2($$){
        my ($volume,$opts) = @_; 
        my $test = $volume->{vserver}->{node}->{test};
        my $exec_server = $volume->{vserver}->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        my $vserver = $volume->{vserver};
        my $volname = $volume->{name};
        my $parsed_output;
        my $parsed_hash = 1;
        eval {
            my $raw_output;
      	    $Server->set_execution_server(-Server => $exec_server);
            $parsed_output = $Server->volume_show(vserver=>$vserver->{name},
                             volume=>"$volname",
                             'flexcache-cache-policy'=>"$opts->{policy}"); 
        }; 
        if ($@) {
		    $test->nlog("Flexcache cache policy apply for volume faileds : $@"); 
            return 0;
	    }
	    return 1;
}
sub setExportPolicy($$){
        my ($volume,$opts) = @_; 
        my $test = $volume->{vserver}->{node}->{test};
        my $exec_server = $volume->{vserver}->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        my $vserver = $volume->{vserver};
        my $volname = $volume->{name};
        my $parsed_output;
        eval {
      	    $Server->set_execution_server(-Server => $exec_server);
            $parsed_output = $Server->volume_modify(vserver=>$vserver->{name},
                             volume=>"$volname",
                             'policy'=>"$opts->{policy}"); 
        }; 
        if ($@) {
		    $test->nlog("Export Policy modify for volume failed with : $@"); 
            return 0;
	    }
	    return 1;
}


sub flexcacheStatisticsShow($$){
    my ($volume,$opts) = @_; 
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $volname = $volume->{name};
    my $object = defined($opts->{object})?$opts->{object}:'flexcache';
    my $result = 1;
    my @stats_obj;
    my $stats={};

    my @counters = qw(hit_pure_perct hit_verify_perct miss_pure_perct miss_verify_perct proxy_perct);

    try {
	    @stats_obj =   NACL::C::Statistics->find (
									    command_interface => $Server,
									    filter            => {
													node => $exec_server,
													object => $object,
													instance => $volname,
													counter => "*pure*,*verify*,proxy_perct"
                                                                                                         
												    }
									);
	} catch NATE::BaseException with {
	    my $ex = shift;
	    $test->nlog("Unable to show statistics - ".$ex->text);
	    $result = 0;
	    return $result;
	
	} catch NATE::Error with {
	    my $ex = shift;
	    $test->nlog("In the Error catch".$ex->text);
	    $result = 0;
	    return $result;
        };
	       

        if($result) {

	foreach my $stat (@stats_obj){
             my $curState = $stat->state();	     
	     my $uniqueid = $curState->{counter};
	     $stats->{"$uniqueid"."_value"} = $curState->{value};
	     $stats->{"$uniqueid"."_delta"} = $curState->{delta};
        }

	#	$test->nlog("stats_obj ::: ".Dumper($stats_obj->state()) );
	#	$test->nlog("stats_obj ::: ".Dumper($stats_obj) );
		#$stats->{"$counter"."_value"} = $stats_obj->state()->{value};
		#$stats->{"$counter"."_delta"} = $stats_obj->state()->{delta};
        }
     		
    return $stats;

}


sub flexcacheStatisticsShowRaw($$){
    my ($volume,$opts) = @_; 
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $volname = $volume->{name};
    my $object = defined($opts->{object})?$opts->{object}:'flexcache';
    my $result = 1;my $parsed_hash;my @splitLines;

    eval {
    $result = $volume->{vserver}->{node}->raw_send({cmd =>"set diagnostic -confirmations off;statistics show -object $object -instance $volname -node $exec_server -counter *pure*,*verify*,proxy_perct"});
    $test->nlog("Flexcache Statistics Raw cmd : ");
    $test->nlog("$result");
    @splitLines = split(/\n/,$result); 
    $parsed_hash->{'string'} =  $result;
    $parsed_hash->{'hit_pure_perct_value'} =  (split(' ',@splitLines[6]))[1] ;
    $parsed_hash->{'hit_verify_perct_value'} =  (split(' ',@splitLines[7]))[1] ;
    $parsed_hash->{'miss_pure_perct_value'} =  (split(' ',@splitLines[8]))[1] ;
    $parsed_hash->{'miss_verify_perct_value'} =  (split(' ',@splitLines[9]))[1] ;
    $parsed_hash->{'proxy_perct_value'} =  (split(' ',@splitLines[10]))[1] ;
    $parsed_hash->{'hit_pure_perct_delta'} =  (split(' ',@splitLines[6]))[2] ;
    $parsed_hash->{'hit_verify_perct_delta'} =  (split(' ',@splitLines[7]))[2] ;
    $parsed_hash->{'miss_pure_perct_delta'} =  (split(' ',@splitLines[8]))[2] ;
    $parsed_hash->{'miss_verify_perct_delta'} =  (split(' ',@splitLines[9]))[2] ;
    $parsed_hash->{'proxy_perct_delta'} =  (split(' ',@splitLines[10]))[2] ;
    };
    
    if ($@) {
	$test->nlog("Epic FAIL in lib : $@"); 
	return 0;	
    } else {
        return $parsed_hash;
    }

	#20101118 192519 [01_smokefc] <<f6080-234-58#ssh_4|    Object.Instance.Counter                                 Value         Delta\r\n
	#20101118 192519 [01_smokefc] <<f6080-234-58#ssh_4|    ----------------------------------------------- ------------- -------------\r\n
	#20101118 192519 [01_smokefc] <<f6080-234-58#ssh_4|    flexcache.cache_testvol12.hit_pure_perct                   0%             -\r\n
	#20101118 192519 [01_smokefc] <<f6080-234-58#ssh_4|    flexcache.cache_testvol12.hit_verify_perct                99%             -\r\n
	#20101118 192519 [01_smokefc] <<f6080-234-58#ssh_4|    flexcache.cache_testvol12.miss_pure_perct                  0%             -\r\n
	#20101118 192519 [01_smokefc] <<f6080-234-58#ssh_4|    flexcache.cache_testvol12.miss_verify_perct                0%             -\r\n
	#20101118 192519 [01_smokefc] <<f6080-234-58#ssh_4|    flexcache.cache_testvol12.proxy_perct                      0%             -\r\n


}

sub flexcacheFstat($$) {
        my ($volume,$opts) = @_; 
        my $test = $volume->{vserver}->{node}->{test};
        my $exec_server = $volume->{vserver}->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        my $vserver = $volume->{vserver};
        my $volname = $volume->{name};
        my $path = $opts->{path};
        my $parsed_output;
        my $parsed_hash = {};
		
		# The node on whch flexcache fstat has to be run 
		# default value is the node whose connection is used to send the command
		my $node = defined $opts->{node} ? $opts->{node} : $exec_server;
		
        eval {
            my $raw_output;
      	    $Server->set_execution_server(-Server => $exec_server);
            $parsed_output = $Server->system_node_run(node=>$node,
           	    command => "flexcache fstat /vol/$volname$path", output => \$raw_output);
            my $outputString = join('',$parsed_output);
            my @splitLines = split(/\n/,$outputString); 
 
            $test->nlog("splitLines: \n\n\n\n " . Dumper(@splitLines));
           
            foreach my $line (@splitLines){
            #remove extra spaces
            $line =~ s/^\s+//;
            if (defined ($temp_hash{(split(/:/,$line))[0]}))
            {
                $parsed_hash->{$temp_hash{(split(/:/,$line))[0]}} = (split(/:/,$line))[1];
            }
                            
            }
                                   
            $test->nlog("phash -> parsed_hash : \n\n\n\n " . Dumper($parsed_hash)); 

        }; 
        if ($@) {
		    $test->nlog("Flexcache fstat Failed : $@"); 
		    return 0;
	}
	return $parsed_hash;

#$VAR1 = 'Properties:';
#$VAR2 = '    Filename: /vol/cache_testvol7//myfile1_1';
#$VAR3 = '    Attributes:';
#$VAR4 = '        Type: file';
#$VAR5 = '        Mode: 0666';
#$VAR6 = '        Nlinks: 1';
#$VAR7 = '        UID: 65534';
#$VAR8 = '        GID: 65534';
#$VAR9 = '        Size: 50000';
#$VAR10 = '        Blocks: 14';
#$VAR11 = '        Fsid: 0x6967a0b9';
#$VAR12 = '        Fileid: 96';
#$VAR13 = '        Xinode: 0 (gen 0)';
#$VAR14 = '        Sinode: 0 (gen 0)';
#$VAR15 = '        AV version: 0';
#$VAR16 = '        Security: 0 (qtree=0 sec=0)';
#$VAR17 = '        Last access time: Tue, 15 Jun 2010 17:26:16 (+702224 uSec) GMT';
#$VAR18 = '        Last modified time: Tue, 15 Jun 2010 17:25:55 (+715638 uSec) GMT';
#$VAR19 = '        Last change time: Tue, 15 Jun 2010 17:25:55 (+715638 uSec) GMT';
#$VAR20 = '        Creation time: Tue, 15 Jun 2010 17:25:55 (+704638 uSec) GMT';
#$VAR21 = '        Inode Generation #: 437363276';
#$VAR22 = '        FlexCache Time since last fill: 974716 uSec (0d 0h 0m 0s 974716uSec)';
#$VAR23 = '        FlexCache Local Blocks: 14';
#$VAR24 = '        FlexCache Local Flags:';

}


sub flexcacheEject();
sub flexcacheStats();
sub flexcacheBlockStats();
sub flexcacheDelegations();

sub cifs_share_create($$){
	my ($volume,$opts) = @_; 
	my $test = $volume->{vserver}->{node}->{test};
	my $vol = $volume->{name};
	my $vserver = $volume->{vserver}->{name};
	my $share_name = $opts->{share_name};
	my $exec_server = $volume->{vserver}->{node}->{execname}; 
	my $Server = $volume->{vserver}->{node}->{server};
	my $result = 1;
        my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
        my $shareproperties = ['oplocks','browsable','changenotify','showsnapshot']; 
	try {
	  	NACL::C::VserverCifsShare->create(command_interface => $Server, vserver => $vserver, 'share-name' => $share_name,path => "/$vol", 'share-properties' => $shareproperties);
	
		#add the share to the volume object tree...
		$volume->{cifsshares}->{$share_name} = $share_name;

	} catch NATE::BaseException with {

        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());

        if ($reuse =~ /yes/ && $ex->text =~ /Duplicate entry/ ){
            $test->nlog("Reuse active , adding only the objects");
            $volume->{cifsshares}->{$share_name} = $share_name;
	    $result = 1;
	    
	} else {
	    $test->nlog("In the BaseException catch");
	    $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line());
	    $result = 0;
	}    
    } catch NATE::Error with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        $test->nlog("Error : $dumpout");
        $test->nlog("In the Error catch");
        $result = 0; 
    };
	
    $test->nlog("Sleeping for 10 secs share to get stable...");	
    sleep 10;	
    return $result;	

}					 

sub cifs_share_show($$){
	my ($volume,$opts)  = @_;
	my $vol = $volume->{name};
	my $vserver = $volume->{vserver}->name;
	my $share_name = $opts->{share_name};
	my $exec_server = $volume->{vserver}->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
	$Server->set_execution_server(-Server => $exec_server);
	
	$Server->vserver_cifs_share_show(
                       'vserver'       => $vserver,
                       'share-name'  => $share_name
                       );

}

sub cifs_share_delete($$){
	my ($volume,$opts) = @_; 
	my $test = $volume->{vserver}->{node}->{test};
	my $vol = $volume->{name};
	my $vserver = $volume->{vserver}->{name};
	my $share_name = $opts->{share_name};
	my $Server = $volume->{vserver}->{node}->{server};
	$test->nlog("Deleting share $share_name");
	my $result = 1;
	
	try {	
        NACL::C::VserverCifsShare->delete(command_interface => $Server, 
					'vserver'=> $vserver, 'share-name' => $share_name);
					
	#remove the share to the volume object tree...
        delete $volume->{cifsshares}->{$share_name};
	
	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
    };

    return $result;
}


sub snapshotCreate($$){
	my ($volume,$opts) = @_; 
	my $test = $volume->{vserver}->{node}->{test};
	my $vol = $volume->{name};
	my $vserver = $volume->{vserver}->{name};
	my $name = $opts->{name};
	my $Server = $volume->{vserver}->{node}->{server};
	my $result = 1;
	try {
	
	NACL::C::Snapshot->create(command_interface => $Server, 
			'vserver'=> $vserver, 'snapshot' => $name, 'volume' => $vol );
					
	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	return $result;

}

sub snapshotDelete($$){
	my ($volume,$opts) = @_; 
	my $test = $volume->{vserver}->{node}->{test};
	my $vol = $volume->{name};
	my $vserver = $volume->{vserver}->{name};
	my $name = $opts->{name};
	my $Server = $volume->{vserver}->{node}->{server};
	my $result = 1;
	try {
	
	NACL::C::Snapshot->delete(command_interface => $Server, 
			'vserver'=> $vserver, 'snapshot' => $name, 'volume' => $vol );
					
	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	return $result;

}

sub snapshotShow($$){
	my ($volume,$opts) = @_; 
	my $test = $volume->{vserver}->{node}->{test};
	my $vol = $volume->{name};
	my $vserver = $volume->{vserver}->{name};
	my $Server = $volume->{vserver}->{node}->{server};
	my @ret;
	my $result = 1;
 	try {
        my @snapshots = NACL::C::Snapshot->find( 
			command_interface => $Server, 
            		filter => { volume => "$vol",
			 }); 

        #iterate and show neatly
        foreach my $snapshot (@snapshots) {
            my $snapstate =  $snapshot->state();
	    $test->nlog( "Volume name =" . $snapstate->volume() );     
            $test->nlog( "Volume vserver =" . $snapstate->vserver() );     
            $test->nlog( "Blocks =" . $snapstate->blocks() );     
            $test->nlog( "Snapshot =" . $snapstate->snapshot() );     
            $test->nlog( "dsid =" . $snapstate->dsid() );     
            $test->nlog( "msid =" . $snapstate->msid() );     
            $test->nlog( "create_time =" . $snapstate->create_time() );     
            $test->nlog( "Logical snap id =" . $snapstate->logical_snap_id() );     
	    push @ret,$snapstate;
        }        

    } catch NATE::BaseException with {
        $test->nlog( "No snapshot :$opts->{name} exists in vserver $vserver");     
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $result = 0;
		return $result;
    } catch NATE::Error with {
        $test->nlog( "No volume :$opts->{volname} exists in vserver $vserver->{name}");     
        #$test->nlog("In the Error catch");
        $result = 0;
	return $result;
    };
    if ($@) {
        $test->nlog( "No volume :$opts->{volname} exists in vserver $vserver");     
        #$test->nlog("except: In the conditional");
        $result = 0;
	return $result;
    }
    return @ret;
}

sub snapshotRestore($$){
	my ($volume,$opts) = @_; 
	my $test = $volume->{vserver}->{node}->{test};
	my $vol = $volume->{name};
	my $vserver = $volume->{vserver}->{name};
	my $name = $opts->{name};
	my $Server = $volume->{vserver}->{node}->{server};
	my $result = 1;
	try {
    	NACL::C::Snapshot->restore(command_interface => $Server, 
			'vserver'=> $vserver, 'snapshot' => $name, 'volume' => $vol );
					
	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
    };
    return $result;
}

sub flexcacheFstati($$) {        
	my ($volume,$opts) = @_;        
	#my $Server = $volume->{vserver}->{node}->{server};
        my $test = $volume->{vserver}->{node}->{test};
        my $exec_server = $volume->{vserver}->{node}->{execname};
        my $Server = Server->new(-Server => "$exec_server");
        my $vserver = $volume->{vserver};
        my $volname = $volume->{name};
        my $inode = $opts->{inode};
        my $parsed_output;
        my $parsed_hash = {};
	my $outputString;
	my $flag = 0;
        try {
            my $raw_output;
            $Server->set_execution_server(-Server => $exec_server);
            $parsed_output = $Server->system_node_run(node=>"$exec_server",
                    command => "flexcache fstat -i /vol/$volname/inode:$inode", output => \$raw_output);
	  #$Server = $volume->{vserver}->{node}->{server};
	   # $parsed_output = NACL::C::Node->run(command_interface => $Server,
           #                             'command'=> "flexcache fstat -i /vol/$volname/inode:$inode", 'node' => $exec_server );
            $outputString = join('',$parsed_output);
	    if ($outputString =~ m/Object does not exist in cache/)
	    {
		$test->nlog("The inode was not found on the volume");
		$flag = 1;
	    }
	    else {
        
            my @splitLines = split(/\n/,$outputString);
            
            $test->nlog("splitLines: \n\n\n\n " . Dumper(@splitLines));
                              
            foreach my $line (@splitLines){
            #remove extra spaces
            $line =~ s/^\s+//;
            if (defined ($temp_hash{(split(/:/,$line))[0]}))
            {
                $parsed_hash->{$temp_hash{(split(/:/,$line))[0]}} = (split(/:/,$line))[1];
            }
                            
            }
            
            $test->nlog("phash -> parsed_hash : \n\n\n\n " . Dumper($parsed_hash));
	    }
	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        return 0;
    	} catch NATE::Error with {
        $test->nlog("In the Error catch");
        return 0;

	};
	if ($@) {
                    $test->nlog("Flexcache fstat Failed : $@");
                    return 0;
        }
        if ($flag == 1)
	{
		return $outputString;
	}
	else
	{
		return $parsed_hash;
	}
}

sub setAntivirusPolicy()
{
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->modify( 
			command_interface => $Server, 
			volume => "$volume->{name}",
			vserver => "$vserver->{name}",
			'antivirus-on-access-policy' => "$opts->{av_policy}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
#    if ($@) {
#        $test->nlog("except: In the conditional");
#        $result = 0;
#    }

    return $result;	    
}

sub setSecurityStyle($$) {
    my ($volume,$opts) = @_;
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname};
    my $vserver = $volume->{vserver};
    my $result = 1;
    my $nacl_volume;
  
    try {
            $nacl_volume = NACL::C::Volume->modify(
			command_interface => $Server,
			volume => "$volume->{name}",
			vserver => "$vserver->{name}",
			'security-style' => "$opts->{security_style}",  );
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;

    };
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }

    return $result;
}

sub setUnixPerm($$) {
    my ($volume,$opts) = @_;
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname};
    my $vserver = $volume->{vserver};
    my $result = 1;
    my $nacl_volume;
 
    try {
            $nacl_volume = NACL::C::Volume->modify(
                        command_interface => $Server,
                        volume => "$volume->{name}",
                        vserver => "$vserver->{name}",
                        'unix-permissions' => "$opts->{unix_permissions}" );
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;

    };
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }

    return $result;
}

sub qtreeCreate(){
	my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_qtree;
	if ($opts->{'security-style'})
	{
	try {
            $nacl_qtree = NACL::C::VolumeQtree->create(
                        command_interface => $Server,
                        volume => "$volume->{name}",
                        vserver => "$vserver->{name}",
			%$opts,);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;

    };
	
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }
	}else
	{
			try {
            $nacl_qtree = NACL::C::VolumeQtree->create(
                        command_interface => $Server,
                        volume => "$volume->{name}",
                        vserver => "$vserver->{name}",
                        qtree => "$opts->{qtree}"
						);
			} catch NATE::BaseException with {
			my $ex = shift;
			my $dumpout = Dumper($ex);
			my $ref_of_obj = ref($ex->object());
			$test->nlog("In the BaseException catch");
			$test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
			$result = 0;
		} catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;

		};
	
		if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
		}
	}
	
	 return $result;
}


sub qtreeDelete(){
        my ($volume,$opts) = @_;
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $vserver = $volume->{vserver};
    my $result = 1;
    my $nacl_qtree;
        try {
            $nacl_qtree = NACL::C::VolumeQtree->delete(
                        command_interface => $Server,
                        volume => "$volume->{name}",
                        vserver => "$vserver->{name}",
                        qtree => "$opts->{qtree}",
        );
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;

    };

    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }
         return $result;
}


sub setQtreeSecStyle(){
	my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $nacl_qtree;
	
	try {
            $nacl_qtree = NACL::C::VolumeQtree->modify(
                        command_interface => $Server,
                        volume => "$volume->{name}",
                        vserver => "$vserver->{name}",
                        qtree => "$opts->{qtree}",
						'security-style' => "$opts->{'security-style'}");
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;

    };
	
    if ($@) {
        $test->nlog("except: In the conditional");
        $result = 0;
    }
	return $result;
}

sub volQosPolicyModify
{
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $qos_policy_name = defined($opts->{qos_policy}) ? $opts->{qos_policy} : "none";
    my $result = 1;
    my $nacl_qtree;
    my $host = host($exec_server);
    try {
            
        
        my $version_manager = NACL::APISet::Version->new(hostobj => $host, category => "Node", shell_type => "CMode");
        my $Api_Set_Obj = NACL::APISet->new(hostobj => $host, 
                                category => "Node", 
                                interface => "CLI", 
                                set => "CMode",
                                #cdef_path => "/u/kgokul/web/updated_cdefs/", ## hard coded cdef path. Remove the parameter "cdef_path" later when NACL builds the cdef
                                version_info => {
                                compatibility => 0, # Disable compatibility
                                version_manager => $version_manager,
                                
                                      }
                                 );
        
            
        $Api_Set_Obj->volume_modify(
            volume => "$volume->{name}",
            vserver => "$vserver->{name}",
            "qos-policy-group" => $qos_policy_name,
            
            ) ;
            
            # $nacl_volume = nacl::c::volume->modify( 
            #volume => "$volume->{name}",
            #vserver => "$vserver->{name}",
            # command_interface => $server, 
            # "qos-policy-group" => $qos_policy_name,
            
        # );
              
          

    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");

        
    } catch NATE::Error with {
        $test->nlog("In the Error catch " );
        $result = 0; 
    };

    if ($@) {
        $test->nlog("except: In the conditional : $@");
        $result = 0;
    }
    return $result;
}

sub getQosWorkload
{
    my ($volume,$opts) = @_; 
    my $Server = $volume->{vserver}->{node}->{server};
    my $test = $volume->{vserver}->{node}->{test};
    my $exec_server = $volume->{vserver}->{node}->{execname}; 
    my $vserver = $volume->{vserver};        
    my $result = 1;
    my $interface_obj;
    my $Api_Set_Obj;
    try {
        
        $result = NACL::C::QosWorkload->find(
                      'command_interface' => $Server,
                       filter   => {
								volume => $volume->{name}
								}
		      
 		);

    

    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Qos Policy-Group workload get  Failed due to ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        my $ex = shift;
        $test->nlog("Qos Policy-Group workload get due to ".$ex->text);
        $result = 0;
    };
    if ($result != 0)
    {
        return $result->workload();
    }else
    {
        return $result;
    }
    
   
}



