#$Id: //depot/prod/test/main/lib/SPARSE/fcluster/VserverObject.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary	
##      Library which defines Vserver Object properties 
##      and actions.
##
## @description
##	Abstract common object tasks for vserver.
##
## @usage
##	use SPARSE::fcluster::VserverObject;
##
## @author mica@netapp.com

package SPARSE::cmode::VserverObject;

use strict;
use Tharn;
use filer;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);
use TCD;

# Internal reference classes 
use SPARSE::cmode::VolumeObject qw(volume);

# NACL libs
use NACL::APISet;
use NACL::C::Node;
use NACL::C::Volume;
use NACL::C::Interface;
use NACL::C::Route;
use NACL::C::ExportPolicy;
use NACL::C::ExportPolicyRule;
use NACL::C::VserverNFS;
use NACL::C::RoutingGroups;
use NACL::C::VserverServicesDns;
use NACL::C::VserverNameMapping;
use NACL::C::VserverCifs;
use NACL::C::VserverCifsShare;
use NACL::C::Snapmirror;
use NACL::C::UnixUser;
use NACL::C::UnixGroup;
use NACL::C::VolumeFlexcacheOptions;
use NACL::C::VserverLocks;
use NACL::C::JobScheduleInterval;
use NACL::C::VolumeFlexcache;
use NACL::C::VolumeFlexcacheCachePolicy;
use NACL::C::VolumeFlexcacheOptions;
use NACL::C::VolumeEfficiency;
use NACL::C::RepositoryVolume;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);
use NACL::STask::Snapmirror;

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&vserver
		);

# ------------------------------------------------------------------------
## vserver class definition

## @name	::vserver
## @summary	create a 10mode vserver object
## @description
##	Create a 10Mode vserver object.
## @arg		node object to filer required
## @arg		nodename required
## @arg		opts optional
## @return	vserver object



				
sub vserver(%)
{
    my %fparams = validate(
        @_,
        {
           node    => { type => HASHREF },
           opts    => { type => HASHREF },
    });

    my $node = $fparams{node};
    my $opts = $fparams{opts};
    my $vols = {};
    my $dlifs = {};
    my $vserver = {};
    my $properties = {};
    my $cachepolicies = {};
    my $exportpolicies = {};

    $vserver->{name} = $opts->{name};
    $vserver->{node} = $node;
    $vserver->{vols} = $vols;    
    $vserver->{dlifs} = $dlifs;
    $vserver->{properties} = $properties;
    $vserver->{cachepolicies} = $cachepolicies;
    $vserver->{exportpolicies} = $exportpolicies;
     
    #$node->{test}->nlog("vserver object : " . Dumper($vserver));
    bless($vserver);

    return $vserver;
}

sub DESTROY($)
{
	my ($vserver) = @_;
	my $exc_state = $@;
	$vserver->{node}->{test}->nlog("destroying $vserver->{name} object");
	$@ = $exc_state;
}

# ------------------------------------------------------------------------
## Vserver Object methods 

sub showVols ($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $vol = $opts->{volname};
    delete $opts->{volname};
    my $result = 1;
    my $nacl_volume;

    try {
        $nacl_volume = NACL::C::Volume->find( 
            command_interface => $Server, 
            filter            => {
		volume => $vol,
		vserver => $vserver->{name},
		%{$opts}
	    }
	); 
    }
    catch NATE::BaseException with {
        my $ex = shift;
        if($ex =~ /command failed: Duplicate volume/) {
	    $result = 2;	    
	}
	else {
            $test->nlog( "BaseException : No volume :$vol exists in vserver $vserver->{name}".$ex->text);  
	    $result = 0;
	}
    };
	    
    if($result == 1) {
	my $vol_state = $nacl_volume->state();
	return $vol_state;
    }

    if($result == 2) {
	return 1;
    }

    return $result
}


sub createFlexVol($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $nacl_volume;
    my $user = defined($opts->{user})?$opts->{user}:'root'; 
    my $group = defined($opts->{group})?$opts->{group}:'root'; 
    my $policy = defined($opts->{policy})?$opts->{policy}:'default'; 
    my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
    my $sec_style = defined($opts->{security_style}) ? $opts->{security_style} : "mixed";
    my $SServer = Server->new(-Server => "$exec_server");
    

    try {
            $nacl_volume = NACL::C::Volume->create( 
	    command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}",
            aggregate => "$opts->{aggrname}",
            size => "$opts->{volsize}",
            "unix-permissions"=>"$opts->{volperms}",
            "junction-path"=>"$opts->{voljunction}",
	    "user"=>"$user", 
	    "group"=>"$group", 
	    "policy"=>"$policy","method-timeout"  => "300",
	    "security-style" => $sec_style,
        );
              
          my $volEfficiencyFlag = $vserver->{node}->{params}->{"VOL_EFFICIENCY"};
          #$test->nlog("Hmmmm :" . Dumper($vserver->{node}->{params}) ); 

          if (!defined($volEfficiencyFlag)) {
	    $volEfficiencyFlag="false";
	  }
          if ($volEfficiencyFlag =~ /true/) {
	  # by default turn on volume efficiency/dense/compression ....  

#          my $VolumeEfficiency_object = NACL::C::VolumeEfficiency->on(
#                                       command_interface => $Server,
#     	                                 volume => "$opts->{volname}",
#     	                                 vserver => "$vserver->{name}",
#                                       
#				      );
#          $VolumeEfficiency_object = NACL::C::VolumeEfficiency->modify(
#                                      command_interface => $Server,
#     	                                volume => "$opts->{volname}",
#     	                                vserver => "$vserver->{name}",
#                                      'compression' => "true",'inline-compression' => "true",  
#                                      );
#          $VolumeEfficiency_object = NACL::C::VolumeEfficiency->start(
#                                       command_interface => $Server,
#     	                                volume => "$opts->{volname}",
#     	                                vserver => "$vserver->{name}",
#                                       'compression' => "true",'dedupe' => "true",'scan-old-data'=>"true",  
#                                       path => "/$opts->{volname}",
#				      );

          eval {
	        my $raw_output;
	        $SServer->set_execution_server(-Server => $exec_server);
                my @parsed_output = $SServer->volume_efficiency_on(volume => "$opts->{volname}",vserver => "$vserver->{name}",output => \$raw_output); 
                @parsed_output = $SServer->volume_efficiency_modify(volume => "$opts->{volname}",vserver => "$vserver->{name}",'compression' => "true",'inline-compression' => "true",output => \$raw_output);
                @parsed_output = $SServer->volume_efficiency_start(volume => "$opts->{volname}",vserver => "$vserver->{name}",'compression' => "true",'dedupe' => "true",'scan-old-data'=>"true",output => \$raw_output);
    	  };
	 
	  if ($@) {
               $test->nlog("Vserverobject Exception during Volume Efficiency Setting : $@");	    
	       $@ = '';
	  }



         }

    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");

        if ($reuse =~ /yes/ && $ex->text =~ /Duplicate volume/ ){
            $test->nlog("Reuse active , adding only the objects");
            $test->nlog("Hmmmm :" . Dumper($vserver->{node}->{params}) ); 
	    
	    #$opts->{type} = "RW"; 
            #$opts->{nacl_volume} = ""$nacl_volume->state(); 
            #my $volumeObj = volume(vserver => $vserver,opts => $opts);
            #$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
	    $result = 1;	    
	} else { 
                $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex");
                $result = 0;
	}
    };

    $opts->{type} = "RW"; 
    if ($reuse !~ /yes/) {
	$opts->{nacl_volume} = $nacl_volume->state(); 
    }
    my $volumeObj = volume(vserver => $vserver,opts => $opts);
    $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
 
    return $result;	    
}

sub createFlexVolNS($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $nacl_volume;
    my $user = defined($opts->{user})?$opts->{user}:'root'; 
    my $group = defined($opts->{group})?$opts->{group}:'root'; 
    my $policy = defined($opts->{policy})?$opts->{policy}:'default'; 
    my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
    my $sec_style = defined($opts->{security_style}) ? $opts->{security_style} : "mixed";
    my $SServer = Server->new(-Server => "$exec_server");

    try {

            $nacl_volume = NACL::C::RepositoryVolume->create( 
	    command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}",
            'sys-aggr-list' => [$opts->{aggrname}],
            size => "$opts->{volsize}",
            "unix-permissions"=>"$opts->{volperms}",
            "junction-path"=>"$opts->{voljunction}",
	    "user"=>"$user", 
	    "group"=>"$group", 
	    "policy"=>"$policy",
	    
	    "method-timeout"  => "300",
            );

    } catch NATE::BaseException with {

        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");

        if ($reuse =~ /yes/ && $ex->text =~ /Duplicate volume/ ) {
            $test->nlog("Reuse active , adding only the objects");
            $test->nlog("Hmmmm :" . Dumper($vserver->{node}->{params}) ); 
	    
	    #$opts->{type} = "RW"; 
            #$opts->{nacl_volume} = ""$nacl_volume->state(); 
            #my $volumeObj = volume(vserver => $vserver,opts => $opts);
            #$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
	    $result = 1;	    
	} else { 
                $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex");
                $result = 0;
	}

    };

    $opts->{type} = "RW"; 
    if ($reuse !~ /yes/) {
	$opts->{nacl_volume} = $nacl_volume->state(); 
    }

    my $volumeObj = volume(vserver => $vserver,opts => $opts);
    $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
 
    return $result;	    
}

sub deleteFlexVol($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->delete( 
			command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}","method-timeout"  => "300",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Volume delete fails with exception - ".$ex->text);
        $result = 0;
    };

    delete $vserver->{vols}->{"$opts->{volname}"};
    return $result;	    
}

sub createAcroVol($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $nacl_volume;

    try {
            $nacl_volume = NACL::C::Volume->create( 
			command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}",
            aggregate => "$opts->{aggrname}",
            size => "$opts->{volsize}",
            "unix-permissions"=>"$opts->{volperms}",
            "stripe-format" => "full",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    };

    $opts->{type} = "RW"; 
    $opts->{nacl_volume} = $nacl_volume->state(); 
    my $volumeObj = volume(vserver => $vserver,opts => $opts);
    $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
 
    return $result;	    
}

sub deleteAcroVol($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->delete( 
			command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Volume delete fails with exception - ".$ex->text);
        $result = 0;
    };

    delete $vserver->{vols}->{"$opts->{volname}"};
    return $result;	    
}
sub createCacheVolOld($$) {
        my ($vserver,$opts) = @_;          
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
	my $raw_output;
        my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 

		if($opts->{cachepolicy}) {
			eval {
				$Server->set_execution_server(-Server => $exec_server);
				my @parsed_output = $Server->volume_new(vserver => "$vserver->{name}" , 
					volume=>"$opts->{volname}",aggregate=>"$opts->{aggrname}" ,
							size=>"$opts->{volsize}",
							"flexcache-origin-volume"=>"$opts->{originvol}",
							"flexcache-cache-policy"=>"$opts->{cachepolicy}",
							type=>"DC",-Timeout  => '3000',
							output => \$raw_output);
		
				#$test->nlog("output : @parsed_output");
				#create and add a vol object for later use
				$opts->{type} = "DC"; 
				my $volumeObj = volume(vserver => $vserver,opts => $opts);
				$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
			}; 
		}
		else{
			eval {
				$Server->set_execution_server(-Server => $exec_server);
				my @parsed_output = $Server->volume_new(vserver => "$vserver->{name}" , 
					volume=>"$opts->{volname}",aggregate=>"$opts->{aggrname}" ,
							size=>"$opts->{volsize}",
							"flexcache-origin-volume"=>"$opts->{originvol}",
							type=>"DC",-Timeout  => '3000',
							output => \$raw_output);
		 
				#$test->nlog("output : @parsed_output");
				#create and add a vol object for later use
				$opts->{type} = "DC"; 
				my $volumeObj = volume(vserver => $vserver,opts => $opts);
				$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
			}; 
		}
        if ($@) {
    	    my $err = $@;
	    if ($reuse =~/yes/ && $err =~ /Duplicate volume/) { #need to call nacl libs
                $vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
	        $test->nlog("Reuse active , adding only the objects");
		$opts->{type} = "DC"; 
		my $volumeObj = volume(vserver => $vserver,opts => $opts);
		$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
                return 1;   
	    } else {	    
		$test->nlog("Epic FAIL in lib : $@"); 
		return 0;	
	    }
	}
        
	$vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
        return 1;
}

#### new stuff

sub createCacheVol($$) {
        my ($vserver,$opts) = @_;          
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        #my $Server = Server->new(-Server => "$exec_server");
        my $Server = $vserver->{node}->{server};
	my $raw_output;
        my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
        my $autosize = defined($opts->{autosize})?$opts->{autosize}:'true'; 
        my $autosize_increment = defined($opts->{autosize_increment})?$opts->{autosize_increment}:'50m'; 
        my $max_autosize = defined($opts->{max_autosize})?$opts->{max_autosize}:'50m'; 
		
        my $nacl_volume; my $ret = 1;
        $test->nlog("Using NACL calls ..... ");

		if($opts->{cachepolicy}) {
                                try {
			            $nacl_volume = NACL::C::Volume->create( 
			    		                command_interface => $Server,
							vserver => "$vserver->{name}" , 
     					                volume=>"$opts->{volname}",
							aggregate=>"$opts->{aggrname}" ,
							size=>"$opts->{volsize}",
							"flexcache-origin-volume"=>"$opts->{originvol}",
							"flexcache-cache-policy"=>"$opts->{cachepolicy}",
							'autosize'=>"$autosize",
							'autosize-increment'=>"$autosize_increment",
							'max-autosize'=>"$max_autosize",
							type=>"DC","method-timeout"=>'3000',
	     			    );
    
	    			    #$test->nlog("output : @parsed_output");
				    #create and add a vol object for later use

				    $opts->{type} = "DC"; 
			 	    my $volumeObj = volume(vserver => $vserver,opts => $opts);
				    $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
    			        			    								
				} catch NATE::BaseException with {
			            my $ex = shift;
			            $test->nlog("Volume create Fails Here : In the NATE BaseException");
			            $ret = 0;
				    #return 0;

			        };
#				catch NATE::Error with {
#			            $test->nlog("Volume create : In the Error catch");
#			            $ret = 0;
#				    #return 0;
#
#     			        };
#
#			        if ($@) {
#		        	    my $err = $@;
#
#			    	    if ($reuse =~/yes/ && $err =~ /Duplicate volume/) { #need to call nacl libs
#		                        $vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
#				        $test->nlog("Reuse active , adding only the objects");
#					$opts->{type} = "DC"; 
#					my $volumeObj = volume(vserver => $vserver,opts => $opts);
#					$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
#		                        $ret = 0;
#					#return 1;   
#		    	           } else {	    
#		    		        $test->nlog("Epic FAIL in lib : $@"); 
#				        $ret = 0;
#					#return 0;	
#			           }
#                     
#	                        }
			$vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
		}
		else{
    		                try {
			            $nacl_volume = NACL::C::Volume->create( 
			    		                command_interface => $Server,
			    		                vserver => "$vserver->{name}" , 
     					                volume=>"$opts->{volname}",
							aggregate=>"$opts->{aggrname}" ,
							size=>"$opts->{volsize}",
							"flexcache-origin-volume"=>"$opts->{originvol}",
							type=>"DC","method-timeout"=>'3000',
	     			    );
    
	    			    #$test->nlog("output : @parsed_output");
				    #create and add a vol object for later use

				    $opts->{type} = "DC"; 
			 	    my $volumeObj = volume(vserver => $vserver,opts => $opts);
				    $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
    			        			    								
				} catch NATE::BaseException with {
			            my $ex = shift;

			    	    if ($reuse =~/yes/ && $ex->text =~ /Duplicate volume/) { #need to call nacl libs
		                        $vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
				        $test->nlog("Reuse active , adding only the objects");
					$opts->{type} = "DC"; 
					my $volumeObj = volume(vserver => $vserver,opts => $opts);
					$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
    		                        $ret = 1;
					#return 1;   

				    } else { 
			               $test->nlog("Volume create Error : NATE BaseException : $ex->text");
                                       if ($ex->text =~ /Sparse volume limit reached on the node/) {
					    $ret = 2;
				       } else {
					    $ret = 0;
				       }
			               #return 0;
				    }  

			        };
#				catch NATE::Error with {
#			            my $ex = shift;
#			            $test->nlog("Volume create : In the Error catch :" . $ex->text);
#			            $ret = 0;
#				    #return 0;
#
#     			        };
#
#			        if ($@) {
#		        	    my $err = $@;
#			    	    if ($reuse =~/yes/ && $err =~ /Duplicate volume/) { #need to call nacl libs
#		                        $vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
#				        $test->nlog("Reuse active , adding only the objects");
#					$opts->{type} = "DC"; 
#					my $volumeObj = volume(vserver => $vserver,opts => $opts);
#					$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
#		                        $ret = 1;
#					#return 1;   
#		    	           } else {	    
#		    		        $test->nlog("Epic FAIL in lib : $@"); 
#				        $ret = 0;
#					#return 0;	
#			           }
#	                        }
		}
        
	
        return $ret;


}


#####


sub deleteCacheVol($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       

    my $result = 1;
    my $nacl_volume;

    try {
            $nacl_volume = NACL::C::Volume->delete( 
			command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}","method-timeout"  => "300",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    }; 

    delete $vserver->{vols}->{"$opts->{volname}"};
    return $result;	    
}

sub createMirrorVol($$) {
        my ($vserver,$opts) = @_;          
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        eval {
            my $raw_output;
      	    $Server->set_execution_server(-Server => $exec_server);
            my @parsed_output = $Server->volume_new(vserver => "$vserver->{name}" , 
     			volume=>"$opts->{volname}",aggregate=>"$opts->{aggrname}" ,
                        size=>"$opts->{volsize}",
                        type=>"DP",
                        output => \$raw_output);
     
            #$test->nlog("output : @parsed_output");
            #create and add a vol object for later use
            $opts->{type} = "DP"; 
            my $volumeObj = volume(vserver => $vserver,opts => $opts);
            $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
        }; 
        if ($@) {
	         $test->nlog("Epic FAIL in lib : $@"); 
            return 0;	
	     }
        return 1;
}
sub deleteMirrorVol($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $nacl_volume;
   
    try {
            $nacl_volume = NACL::C::Volume->delete( 
			command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Volume delete fails with exception - ".$ex->text);
        $result = 0;
    };

    delete $vserver->{vols}->{"$opts->{volname}"};
    return $result;	    
}


sub renameVol($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       

    my $result = 1;
    my $nacl_volume;

    try {
            $nacl_volume = NACL::C::Volume->rename( 
			command_interface => $Server, 
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}",
	    newname => "$opts->{newname}","method-timeout"  => "300",
			);
    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    };

    # need to modify the name inside the object only, dont change object. 
    $vserver->{vols}->{"$opts->{volname}"}->{name} = $opts->{volname};

    return $result;	    
}


sub flexcacheCreateOld($$) {
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        my $parsed_output; my $hashRefItem;
        my $raw_output;
		my $result="1";
        eval {
      	    $Server->set_execution_server(-Server => $exec_server);
            $parsed_output = $Server->volume_flexcache_create(vserver => "$vserver->{name}" , 
                   		'origin-volume'=>"$opts->{originvolume}",-Timeout  => '300',
                        output => \$raw_output);
     
	    $test->nlog("After flexcacheCreate" . Dumper($parsed_output) ) ; 
            #create and add a vol object for each cache volume created later use
            #my $volumeObj = volume(vserver => $vserver,opts => $opts);
            #$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
           }; 

        if ($@) {
            # This cdef always shows the vols also , and error is seen need to burt it.
            #00512 192650 [01_smokefc] F Node::NG::_cli_output_handler:1054 Found more than 1 matching result during verification
	    #$test->nlog("Epic FAIL in lib : $@"); 
            #return 0;	
            #Below is to build up the cache volume objects 
             eval {
                  $parsed_output = $Server->volume_flexcache_show(vserver => "$vserver->{name}" , 
                   		'origin-volume'=>"$opts->{originvolume}",-Timeout  => '300',
                        output => \$raw_output);
	               $test->nlog("After flexcacheCreate" . Dumper($parsed_output) ) ; 
                  
                  #my $arraysize = @$parsed_output;                        
                  # create n number of cache volume objects.
					     #2 194510 [01_smokefc]           'cache-state' => 'online',
                  #20100512 194510 [01_smokefc]           'cache-volume' => 'voll2_cache_dataaggr40', 
                  #20100512 194510 [01_smokefc]           'vserver' => 'vs1',
                  #20100512 194510 [01_smokefc]           'cache-aggregate' => 'dataaggr40',
                  #20100512 194510 [01_smokefc]           'origin-volume' => 'voll2',
                  #20100512 194510 [01_smokefc]           'origin-aggregate' => 'dataaggr37',
                  #20100512 194510 [01_smokefc]           'cache-available' => '20881408',
                  #20100512 194510 [01_smokefc]           'connection-status' => 'todo',
                  #20100512 194510 [01_smokefc]           'cache-size' => '20971520',
                  #20100512 194510 [01_smokefc]           'origin-state' => 'online',
                  #20100512 194510 [01_smokefc]           'cache-percent-used' => '0'
                  #20100512 194510 [01_smokefc]         };

                  foreach $hashRefItem (@$parsed_output) {                                    
                       $opts->{'volname'} = $hashRefItem->{'cache-volume'};
                       $opts->{'cache-aggregate'} = $hashRefItem->{'cache-aggregate'};
                       $opts->{'origin-volume'} = $hashRefItem->{'origin-volume'};
                       $opts->{'origin-aggregate'} = $hashRefItem->{ 'origin-aggregate'};
                       $opts->{'cache-available'} = $hashRefItem->{'cache-available'};
                       $opts->{'origin-state'} = $hashRefItem->{'origin-state'};
                       $opts->{'cache-percent-used'} = $hashRefItem->{'cache-percent-used'};
                       $opts->{type} = "DC"; 
                       my $volumeObj = volume(vserver => $vserver,opts => $opts);
                       $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
                  }
            
            }; if ($@) { $result = "0"; }

	     }
        return $result;
}

###### new stuff

sub flexcacheCreate($$) {
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = $vserver->{node}->{server};
        my $parsed_output; my $arrRefItem;
        my $raw_output;
	my $result="1";
        my $VolumeFlexcache_object;
        $test->nlog("Using NACL calls ..... ");
	
        try {
	        my $VolumeFlexcache_object = NACL::C::VolumeFlexcache->create(
                    command_interface => $Server,
     	            vserver => "$vserver->{name}" ,
		    'origin-volume' =>"$opts->{originvolume}", 
                    "method-timeout"  => "600" 
                );
		
		my @flexcaches = NACL::C::VolumeFlexcache->find(
		    command_interface => $Server,
		    filter            => {
			vserver => $vserver->{name},
			'origin-volume' => $opts->{originvolume},
		    }
		);
		
		foreach my $cache (@flexcaches) {
		    my $cache_state = $cache->state();
		    my $cache_name = $cache_state->cache_volume();
		    
		    $opts->{'origin-volume'} = $cache_state->origin_volume();
		    $opts->{'cache-percent-used'} = $cache_state->cache_percent_used();
		    $opts->{'cache-available'} = $cache_state->cache_available();
		    $opts->{'cache-aggregate'} = $cache_state->cache_aggregate();
		    $opts->{'connection-status'} = $cache_state->connection_status();
		    $opts->{'origin-state'} = $cache_state->origin_state();
		    $opts->{'cache-size'} = $cache_state->cache_size();
		    $opts->{'volname'} = $cache_name;
		    $opts->{'cache-state'} = $cache_state->cache_state();
		    $opts->{'origin-aggregate'} = $cache_state->origin_aggregate();
		    $opts->{type} = "DC"; 
		    
		    $vserver->{vols}->{$cache_name} = volume(vserver => $vserver, opts => $opts);
		}
            } catch NATE::BaseException with {
                my $ex = shift;
                $test->nlog("Flexcache create fails with exception - ".$ex->text);
                $result = 0;
            }; 
	
        return $result;
}


#############

sub flexcacheDelete($$) {
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = $vserver->{node}->{server};
	my $result = 1;
        try {
	        my $VolumeFlexcache_object = NACL::C::VolumeFlexcache->delete(
                                        command_interface => $Server,
     	                                vserver => "$vserver->{name}" ,
					'origin-volume' =>"$opts->{originvolume}" 
                                        );

                #create and add a vol object for each cache volume created later use
                #my $volumeObj = volume(vserver => $vserver,opts => $opts);
                #$vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
           	   
	    } catch NATE::BaseException with {
              my $ex = shift;
              $test->nlog("Volume delete fails with exception - ".$ex->text);
              $result = 0;
            }; 
        
	return 1;
}

sub flexcacheShow($$) {
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server1 = Server->new(-Server => "$exec_server");
        my $Server = $vserver->{node}->{server};
        my $hashRefItem; my $parsed_output;
        my @newArr; my $arrRefItem;
	my $result = 1;

        try {
	    my @flexcaches = NACL::C::VolumeFlexcache->find(
		command_interface => $Server,
		filter            => {
		    vserver => $vserver->{name},
		    'origin-volume' => $opts->{originvolume},
		}
	    );
	    
	    foreach my $cache (@flexcaches) {
		my $cache_state = $cache->state();
		
		my $newHsh = {};
		$newHsh->{'origin-volume'} = $cache_state->origin_volume();
		$newHsh->{'cache-percent-used'} = $cache_state->cache_percent_used();
		$newHsh->{'cache-available'} = $cache_state->cache_available();
                $newHsh->{'cache-aggregate'} = $cache_state->cache_aggregate();
		$newHsh->{'connection-status'} = $cache_state->connection_status();
		$newHsh->{'origin-state'} = $cache_state->origin_state();
		$newHsh->{'cache-size'} = $cache_state->cache_size();
		$newHsh->{'cache-volume'} = $cache_state->cache_volume();
		$newHsh->{'cache-state'} = $cache_state->cache_state();
		$newHsh->{'origin-aggregate'} = $cache_state->origin_aggregate();
		$test->nlog("Detected cache:\n" . Dumper($newHsh));
		
		push(@newArr,$newHsh);
	    }

            $result = \@newArr;
	    $test->nlog("Dumper at work " . Dumper(@newArr) ) ; 

           } catch NATE::BaseException with {
             my $ex = shift;
             $test->nlog("Flexcache show fails with exception - ".$ex->text);
             $result = 0;

          };
	
        return $result;
}

sub snapmirrorDPCreate($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
	my $destCluster = $opts->{destCluster};
	my $srcCluster = $opts->{srcCluster};
	my $destVolume = $opts->{destVolume};
	my $srcVolume = $opts->{srcVolume};
	my $serverName = $vserver->{name};
    my $palomaDP;

    try {
            $palomaDP = NACL::C::Snapmirror->create( 
			command_interface => $Server, 
			"destination-cluster" => $destCluster,
			"destination-vserver" => $serverName,
			"destination-volume" => $destVolume,
			"source-cluster" => $srcCluster,
			"source-vserver" => $serverName,
			"source-volume" => $srcVolume,
			type => "DP",
			);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    
}

sub snapmirrorInitialize($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $destCluster = $opts->{destCluster};
    my $destVolume = $opts->{destVolume};
    my $serverName = $vserver->{name};

    try {

	my $dst_path = $serverName.":".$destVolume;
	NACL::STask::Snapmirror->initialize(
	    command_interface          => $Server,
	    'destination-path'         => $dst_path,
	    'vserver'                  => $serverName,
	    'nacltask_wait'            => 1,
	    'nacltask_wait_for_status' => 'Idle',
	    'nacltask_wait_for_state'  => 'Snapmirrored',
	);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    
}

sub snapmirrorLSCreate($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $destCluster = $opts->{destCluster};
    my $srcCluster = $opts->{srcCluster};
    my $destVolume = $opts->{destVolume};
    my $srcVolume = $opts->{srcVolume};
    my $serverName = $vserver->{name};
    my $palomaLS;
		
    #my $ret_value = NACL::C::Snapmirror

    try {
            $palomaLS = NACL::C::Snapmirror->create( 
			command_interface => $Server, 
			"destination-cluster" => $destCluster,
			"destination-vserver" => $serverName,
			"destination-volume" => $destVolume,
			"source-cluster" => $srcCluster,
			"source-vserver" => $serverName,
			"source-volume" => $srcVolume,
			type => "LS",
			);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    
}

sub snapmirrorInitializeLSSet($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
	my $srcCluster = $opts->{srcCluster};
	my $srcVolume = $opts->{srcVolume};
	my $serverName = $vserver->{name};
    my $paloma;
	
	
    try {
            $paloma = NACL::C::Snapmirror->initialize_ls_set( 
			command_interface => $Server, 
			"source-cluster" => $srcCluster,
			"source-vserver" => $serverName,
			"source-volume" => $srcVolume,
			"foreground" => "true",
			);
#	$test->nlog("Waiting till the snapmirror's state changes to <initialized> and <idle>.");
#	while ($paloma->state()->state() !~ /snapmirrored/) { sleep 5; }
#	while ($paloma->state()->status() !~ /idle/) { sleep 5; }

    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };
    
    return $result;	    
}

sub snapmirrorUpdate($$){

    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $destCluster = $opts->{destCluster};
    my $destVolume = $opts->{destVolume};
    my $serverName = $vserver->{name};

    try {
        my $dst_path = $serverName.":".$destVolume;
	NACL::STask::Snapmirror->update(
	    command_interface          => $Server,
	    'destination-path'         => $dst_path,
	    'vserver'                  => $serverName,
	    'nacltask_wait'            => 1,
	    'nacltask_wait_for_status' => 'Idle',
	    'nacltask_wait_for_state'  => 'Snapmirrored',
	);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    
}

sub scheduleIntervalCreate($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $intervalName = $opts->{intervalName};
    my $day = $opts->{days};
    my $hr = $opts->{hours};
    my $min = $opts->{minutes};
    my $sec = $opts->{seconds};
    my $myschedule;
	
	
    try {
            $myschedule = NACL::C::JobScheduleCron->create( 
			command_interface => $Server, 
			name => $intervalName,
			minute => [$min],
			);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    

}

sub scheduleIntervalDelete($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $intervalName = $opts->{intervalName};
    my $myschedule;
	
	
	#my $ret_value = NACL::C::Snapmirror

    try {
            $myschedule = NACL::C::JobScheduleCron->delete(
			command_interface => $Server, 
			name => $intervalName,
			);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    

}

sub snapmirrorModifySchedule($$){

	my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
	my $destCluster = $opts->{destCluster};
	my $destVolume = $opts->{destVolume};
	my $sched = $opts->{sched};
	my $serverName = $vserver->{name};
    my $nacl_volume;
	
	
	#my $ret_value = NACL::C::Snapmirror

    try {
            $nacl_volume = NACL::C::Snapmirror->modify( 
			command_interface => $Server, 
			"destination-cluster" => $destCluster,
			"destination-vserver" => $serverName,
			"destination-volume" => $destVolume,
			"schedule" => $sched,
			);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    
}

sub snapmirrorScheduleShow ($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $destCluster = $opts->{destCluster};
	my $destVolume = $opts->{destVolume};
	my $serverName = $vserver->{name};
	my $sched = $opts->{sched};
    my $result = 1;
    my $nacl_mirror;

    try {
        $nacl_mirror = NACL::C::Snapmirror->find( 
				command_interface => $Server, 
				filter            => { "destination-cluster" => $destCluster,
						   "destination-vserver" => $serverName,
						   "destination-volume" => $destVolume,
						   schedule => $sched,
				  }); 
	} catch NATE::BaseException with {
        my $ex = shift;
        if($ex =~ /command failed: Duplicate volume/) {
	    $result = 2;	    
		} else {
            $test->nlog( "BaseException : No volume of the type exists in vserver $vserver->{name}".$ex->text);  
	    $result = 0;
		}

    };
	    
    if($result == 1) {
	my $vol_state = $nacl_mirror->state();
	return $vol_state;
    }

    if($result == 2) {
	return 1;
    }

    return $result;	    

}


sub snapmirrorBreak($$){

	my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
	my $destCluster = $opts->{destCluster};
	my $destVolume = $opts->{destVolume};
	my $serverName = $vserver->{name};
    my $nacl_volume;
	
	
	#my $ret_value = NACL::C::Snapmirror

    try {
            $nacl_volume = NACL::C::Snapmirror->break( 
			command_interface => $Server, 
			"destination-cluster" => $destCluster,
			"destination-vserver" => $serverName,
			"destination-volume" => $destVolume,
			);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    
}

sub snapmirrorResync($$){

	my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
	my $destCluster = $opts->{destCluster};
	my $destVolume = $opts->{destVolume};
	my $serverName = $vserver->{name};

    try {
        my $dst_path = $serverName.":".$destVolume;
	NACL::STask::Snapmirror->resync(
	    command_interface          => $Server,
	    'destination-path'         => $dst_path,
	    'vserver'                  => $serverName,
	    'nacltask_wait'            => 1,
	    'nacltask_wait_for_status' => 'Idle',
	    'nacltask_wait_for_state'  => 'Snapmirrored',
	);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    
}


sub snapmirrorDelete($$){

    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $destCluster = $opts->{destCluster};
    my $destVolume = $opts->{destVolume};
    my $serverName = $vserver->{name};
    my $nacl_volume;
	
	
	#my $ret_value = NACL::C::Snapmirror

    try {
            $nacl_volume = NACL::C::Snapmirror->delete( 
			command_interface => $Server, 
			"destination-cluster" => $destCluster,
			"destination-vserver" => $serverName,
			"destination-volume" => $destVolume,
			);
    } catch NATE::BaseException with {
       my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $result = 0;
    };

    return $result;	    

}

sub flexcacheOptionsModify($$){
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        my $hashRefItem; 
        my $parsed_output = 1;
		$test->nlog("The opts variable has: $opts\n");
        eval {
            my $raw_output;
      	    $Server->set_execution_server(-Server => $exec_server);
			$test->nlog("Setting the privelege to test\n");
			$parsed_output = $Server->set('privilege' => "test");
#			if($parsed_output == 0) {$test->nlog("Unable to set the privilege level to test!\n");  }
#			else {
				$parsed_output = $Server->volume_flexcache_options_modify(vserver => "$vserver->{name}",
								%{$opts}
								);
#								output => \$raw_output);
				$test->nlog("Cache policy dumper" . Dumper($parsed_output)); 
#				}
			}; 
        if ($@) {
            $parsed_output = 0; 
	        $test->nlog("Flexcache options modify command failed : $@"); 
	     }

        return $parsed_output;
}


sub flexcacheOptionsShow($$) {
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = Server->new(-Server => "$exec_server");
        my $hashRefItem; 
        my $parsed_output = 1;
        eval {
            my $raw_output;
      	    $Server->set_execution_server(-Server => $exec_server);
			$test->nlog("Setting the privelege to test\n");
			$parsed_output = $Server->set('privilege' => "test");
#			if($parsed_output == 0) {$test->nlog("Unable to set the privilege level to test!\n");  }
#			else {
				$parsed_output = $Server->volume_flexcache_options_show(vserver => "$vserver->{name}",
								output => \$raw_output);
				$test->nlog("Cache policy dumper" . Dumper($parsed_output)); 
#				}
			}; 
        if ($@) {
            $parsed_output = 0; 
	        $test->nlog("Flexcache options show command failed : $@"); 
	     }

        return $parsed_output;
}


sub flexcachePolicyCreate($$){

        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = $vserver->{node}->{server};
        my $hashRefItem; 
        my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
        my $parsed_output = 1;
        my $flexcachePolicyObject; 	

        try {
            $flexcachePolicyObject = NACL::C::VolumeFlexcacheCachePolicy->create(
                                    command_interface => $Server,
     	                            vserver => "$vserver->{name}", 
                   		    'policy' =>"$opts->{policy}",
				    'reg-ttl'=>"$opts->{'reg-ttl'}",
                   		    'dir-ttl'=>"$opts->{'dir-ttl'}"
				);
	
        } catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Flexcache policy create fails with exception - " . $ex->text);

	    if ($reuse =~/yes/ && $ex->text =~ /duplicate entry/) { #need to call nacl libs
                $test->nlog("reuse active adding objects"); 
                $@ ='';
		$parsed_output = 1;
                $vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
	    } else {
                $parsed_output = 0;
	    }

        };
    
#    if ($@) {
#	    #fix for id error 
#            my $err= $@;
#            if ($err =~ /Verification failure for field/) { #need to call nacl libs
#                $@ ='';
#                $parsed_output = 1;
#	    } elsif ($reuse =~/yes/ && $err =~ /duplicate entry/) { #need to call nacl libs
#                $test->nlog("reuse active adding objects"); 
#                $@ ='';
#		$parsed_output = 1;
#            } else {
#                $test->nlog("Flexcache Cache policy create failed : $@"); 
#                $parsed_output = 0;
#            } 	
#	}
       
        $vserver->{cachepolicies}->{"$opts->{policy}"} = $opts->{policy};
        return $parsed_output;
}

sub flexcachePolicyDelete($$){
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = $vserver->{node}->{server};
        my $hashRefItem;
        my $parsed_output=1;
	my $new_privilege = $opts->{privilege};
	my ($apiset, $current_priv);
	
	if (defined($new_privilege)) {
	    # Remember the current privilege level, to reset it afterwards
	    $apiset = $Server->apiset();
	    $current_priv = $apiset->get_privilege();
	    $apiset->set_privilege( 'privilege-level' => $new_privilege );
	}
     
        try {
            my $flexcachePolicyObject = NACL::C::VolumeFlexcacheCachePolicy->delete(
                                    command_interface => $Server,
     	                            vserver => "$vserver->{name}", 
                   		    'policy' =>"$opts->{policy}",
				);
	
        } catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Flexcache policy delete fails with exception - ".$ex->text);
            $parsed_output = 0;

        };
	
	if (defined($new_privilege)) {
	    # Reset the privilege 
	    $apiset->set_privilege( 'privilege-level' => $current_priv );
	}
 
#        if ($@) {
#	    $test->nlog("Flexcache Cache policy delete failed : $@"); 
#            my $err= $@;
#            if ($err =~ /Verification failure for field/) { #need to call nacl libs
#                $parsed_output=1;   
#            } else {
#                $parsed_output=0;
#            } 	
#            # return 0;  
#            # flexcache-cache-policy delete also calls show , and show fails 
#	}

        delete $vserver->{cachepolicies}->{"$opts->{policy}"};
        return $parsed_output;
}

sub flexcachePolicyModify($$){
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = $vserver->{node}->{server};
        my $hashRefItem; my $parsed_output = 1 ;

        try {
            my $flexcachePolicyObject = NACL::C::VolumeFlexcacheCachePolicy->modify(
                                        command_interface => $Server,
     	                                vserver => "$vserver->{name}", 
										'policy' =>"$opts->{policy}",
										'reg-ttl'=>"$opts->{'reg-ttl'}",
										'dir-ttl'=>"$opts->{'dir-ttl'}",
				    );

        } catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Flexcache policy modify fails with exception - " . $ex->text);
            $parsed_output = 0;

        };

        #if ($@) {
	#    $test->nlog("Flexcache Cache policy modify failed : $@"); 
	#    $parsed_output = 0;	
	#}

        return $parsed_output;

}

sub flexcachePolicyShow($$){
        my ($vserver,$opts) = @_; 
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname}; 
        my $Server = $vserver->{node}->{server};
        my $hashRefItem; 
        my $parsed_output = 1;
        my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
        my $newHsh; my $arrRefItem; my @newArr;
     
        try {
            my $flexcachePolicyObject = NACL::C::VolumeFlexcacheCachePolicy->find(
                                        command_interface => $Server,
     	                                filter            => { 	vserver => "$vserver->{name}",
									'policy' =>"$opts->{policy}",
							       }
                   			);

	    my @policy_state = $flexcachePolicyObject->state();
	    $test->nlog("Flexcache show using NACL" ) ; 
            foreach $arrRefItem (@policy_state) {
	
                       $newHsh->{'reg-ttl'} = $arrRefItem->{'reg_ttl'};
	               $newHsh->{'dir-ttl'} = $arrRefItem->{'dir_ttl'};
                       $newHsh->{'policy'} = $arrRefItem->{'policy'};
                       $newHsh->{'id'} = $arrRefItem->{'id'};

                       $test->nlog("policy : $arrRefItem->{'policy'} ");
                       $test->nlog("reg-ttl : $arrRefItem->{'reg_ttl'} ");
                       $test->nlog("dir-ttl : $arrRefItem->{'dir_ttl'} ");
                       $test->nlog("id : $arrRefItem->{'id'} ");
                       $test->nlog("\n");

                       push(@newArr,$newHsh);
	    }

	    #$test->nlog("Dumper at work " . Dumper(@newArr) ) ; 
            $parsed_output = \@newArr;

        } catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Flexcache policy show fails with exception - ".$ex->text);
            $parsed_output = 0;

        };

#        if ($@) {
#	    $test->nlog("Flexcache Cache policy show failed : $@"); 
#            $parsed_output = 0;
#	}

        return $parsed_output;

}


sub addVolObject (%)
{
        my ($vserver,$opts) = @_;  
        my $volumeName = $opts->{volname};
        my $volumeObj = volume(vserver => $vserver,opts => $opts);
        $vserver->{vols}->{"$volumeName"} = $volumeObj;
        return 1;
}

# .... likewise , based on tests 

sub createDlif {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $errorOutOnDlifFailoverEnabled = defined($vserver->{node}->{params}->{DLIF_FAILOVER_ERROR})?$vserver->{node}->{params}->{DLIF_FAILOVER_ERROR}:'yes';
    my $interface_state;

    try {
            $interface_state =  NACL::C::Interface->create( 
									command_interface => $Server, 
									server            => $vserver->{"name"},
									lif               => $opts->{"name"},
									role              => $opts->{"role"},
									"home-node"       => $exec_server,
									"home-port"       => $opts->{"port"},
									"failover-policy" => 'disabled',
									"use-failover-group" => 'disabled',
  									address           => $opts->{"ip"}, 
									netmask           => $opts->{"mask"}
								);
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Dlif Create failed with Exception - ".$ex->text);
        $result = 0;
    };
    
	if($result) {
            if ($errorOutOnDlifFailoverEnabled =~ /yes/) {
		if($interface_state->state()->status_oper() ne 'up' || $interface_state->state()->status_admin() ne 'up' || $interface_state->state()->is_home() eq 'false' || $interface_state->state()->failover_policy() ne 'disabled') {
			$test->nlog("Failed: Either the dlif is down or is migrated or the dlif failover policy is enabled, please correct that manually and run tests again ...");
			$result = 0;
		}
            } else {
		if($interface_state->state()->status_oper() ne 'up' || $interface_state->state()->status_admin() ne 'up' || $interface_state->state()->is_home() eq 'false') {
			$test->nlog("Failed: Either the dlif is down or is migrated, please correct that manually and run tests again ...");
			$result = 0;
		}
            }

	}

	if ($result) {
		$opts->{"routing_group"} = $interface_state->state()->routing_group();
		$vserver->{dlifs}->{$opts->{"name"}}= $opts;
	}
    
	return $result;	
}

sub migrateDlif
    {
        my ($vserver,$opts) = @_;
        my $Server = $vserver->{node}->{server};
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname};
        my $result = 1;
        my $interface_state;
    
        try {
                $interface_state =  NACL::C::Interface->migrate(
                                                                            command_interface => $Server,
                                                                            server            => $vserver->{"name"},
                                                                            lif               => $opts->{"lif"},
                                                                            "dest-node"       => $opts->{"dest-node"},
                                                                    );
        } catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Dlif migrate failed with Exception - ".$ex->text);
            $result = 0;
        };
     
        if ($result) {
                    $vserver->{dlifs}->{$opts->{"name"}}= $opts;
        }
    
        return $result;
}

sub interfaceRevert
{
    my ($vserver,$opts) = @_;
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};
    my $lif = ( $opts->{'lif'} eq "" ) ? "*" : $opts->{'lif'};
    my $result = 1;
    #my $interface_state;

    try {
            NACL::C::Interface->revert(
                                                                        command_interface => $Server,
                                                                        vserver            => $vserver->{"name"},
                                                                        lif               => $lif,
                                                        );
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Interface revert failed with Exception - ".$ex->text);
        $result = 0;
    };

        #if ($result) {
        #        $vserver->{dlifs}->{$opts->{"name"}}= $opts;
        #}

        return $result;
}

sub showDlif {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $errorOutOnDlifFailoverEnabled = defined($vserver->{node}->{params}->{DLIF_FAILOVER_ERROR})?$vserver->{node}->{params}->{DLIF_FAILOVER_ERROR}:'yes';
    my $result = 1;
    my $interface_obj;
	
	try {
		$interface_obj =  NACL::C::Interface->find(
									command_interface => $Server,
									filter => {
										address => $opts->{"ip"},
									}
								);

	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("In the BaseException catch");
        $test->nlog("\nException caused is       " . $ex->text);
        $result = 0;
	};
	
	if (!$result){
		return $result;
	}
	
	# Check if Data Ip exists in given vserver and Node
	if($interface_obj->state()->server() ne $vserver->{name} || $interface_obj->state()->home_node() ne $exec_server){
		$test->nlog("\n The Data Ip $opts->{\"ip\"} does not exist on given vserver $vserver->{name} or Node $exec_server but exists on another vserver or node\n");
		return 2;
	}
	
	# Check if dlif is up or not and also is present in same node
	#if($interface_obj->state()->status_oper() ne 'up' || $interface_obj->state()->status_admin() ne 'up' || $interface_obj->state()->is_home() eq 'false') {
        # fix here for for failover-policy check
        if ($errorOutOnDlifFailoverEnabled =~ /yes/) {
 	    if($interface_obj->state()->status_oper() ne 'up' || $interface_obj->state()->status_admin() ne 'up') {
			$test->nlog("Failed: The dlif is down");
			return 2;
	    }
		elsif($interface_obj->state()->is_home() eq 'false' || $interface_obj->state()->failover_policy() ne 'disabled' ){
		
			my $ret = $vserver->interfaceRevert({lif => $interface_obj->{lif}});
			if (!$ret){ return 2; }
			NACL::C::Interface->modify(command_interface => $Server,
												lif => "$interface_obj->{lif}",
												'failover-policy' => "disabled",
												vserver => $vserver->{name});
			
		}
    	} else {
	    if($interface_obj->state()->status_oper() ne 'up' || $interface_obj->state()->status_admin() ne 'up' ) {
			$test->nlog("Failed: The dlif is down");
			return 2;
	    }
		elsif($interface_obj->state()->is_home() eq 'false' ){
		
			my $ret = $vserver->interfaceRevert({lif => $interface_obj->{lif}});
			if (!$ret){ return 2; }
		}
    	}
		

	
	# Check if Ip is attached to lif with same name
	# If not rename dlif and return 1
	my $interface_rename_obj;
	if($interface_obj->state()->lif() ne $opts->{"name"}) {
		# Rename dlif to given name
		try {
			$interface_obj->rename(
				newname           => $opts->{"name"}
			);
		} catch NATE::BaseException with {
			my $ex = shift;
			my $dumpout = Dumper($ex);
			my $ref_of_obj = ref($ex->object());
			$test->nlog("In the net interface rename BaseException catch");
			$test->nlog("\n Interface Rename Failed with Exception -    " . $ex->text);
			$result = 2;
		};
		if ($result == 2) {
			return $result;
		}
	}
	
	$opts->{"routing_group"}=$interface_obj->state()->routing_group();
	return $result;
}	

sub createRoute {
	my ($vserver,$opts) = @_; 
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $result = 1;
	my $route_object;
	try {
		$route_object = NACL::C::Route->create( 
							command_interface => $Server,
							server            => $vserver->{name},
							gateway           => $opts->{"gateway"},
							'routing-group'   => $opts->{"routing_group"},
							destination       => '0.0.0.0/0'
						);
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Route Create failed with Exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        my $ex = shift;
		$test->nlog("In the Error catch - ".$ex->text);
        $result = 0;
 
    };
    return $result;
}

sub showRoute {
	my ($vserver,$opts) = @_; 
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $result = 1;
	my $route_object;
	try {
		$route_object = NACL::C::Route->find( 
							command_interface => $Server,
							filter            => {
													server            => $vserver->{name},
													gateway           => $opts->{"gateway"},
													'routing-group'   => $opts->{"routing_group"}
												}
						);
	} catch NATE::BaseException with {
		my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog("\nException caused is       " . $ex->text);
		$result = 0;
	};	
	return $result;
}

sub export_policy_show {
	my ($vserver, $policy) = @_;
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $result = 1;
	my $export_policy_object;
	
	try {
		$export_policy_object = NACL::C::ExportPolicy->find(
									command_interface => $Server,
									filter => {
										vserver           => $vserver->{name},
										policyname        => $policy
									}
								);

	
	} catch NATE::BaseException with {
        $test->nlog("\nExport-policy $policy is not created for vserver $vserver->{name}");
		$result = 0;
	};	
	
	if($result) {
		return $export_policy_object;
	}
	return $result;
}

sub export_policy_create {
	my ($vserver, $policy,$opts) = @_;
	my $Server = $vserver->{node}->{server};
        my $test = $vserver->{node}->{test};
	my $result = 1;
	my $export_policy_object;
        my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
	my $ex;
	
	try {
		$export_policy_object = NACL::C::ExportPolicy->create(
									command_interface => $Server,
									vserver           => $vserver->{name},
									policyname        => $policy
								);

	
	} catch NATE::BaseException with {
            $ex = shift;
    	    if ($reuse =~/yes/ && $ex->text =~ /duplicate entry/) { 
		$test->nlog("\nReuse active , add only objects");
                $result = 1;   
            } else {
                $test->nlog("Flexcache Cache policy create failed :"); 
  	        $test->nlog("\nExport-policy create failed with Exception -".$ex->text);
                $result = 0;
            } 	
	};	
	
	if($result && $reuse !~ /yes/) {
		return $export_policy_object;
	}

	return $result;
}

sub export_policy_delete {
	my ($vserver, $policy) = @_;
	my $Server = $vserver->{node}->{server};
        my $test = $vserver->{node}->{test};
	my $result = 1;
	my $export_policy_object;
	
	try {
		$export_policy_object = NACL::C::ExportPolicy->delete(
									command_interface => $Server,
									vserver           => $vserver->{name},
									policyname        => $policy
								);

	} catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\nExport-policy delete failed with Exception -".$ex->text);
		$result = 0;
	};	
	
	return $result;
}

sub export_policy_rule_show {
	my ($vserver, $policy, $index) = @_;
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $result = 1;
	my $export_policy_rule_object;
	
	try {
		$export_policy_rule_object = NACL::C::ExportPolicyRule->find(
										command_interface => $Server,
										filter => {
											vserver           => $vserver->{name},
											policyname        => $policy,
											ruleindex         => $index
										}
									);

	
	} catch NATE::BaseException with {
        $test->nlog("\n Rule for Export-policy $policy is not created for vserver $vserver->{name}");
		$result = 0;
	};	
	
	if($result) {
		return $export_policy_rule_object;
	}
	return $result;
} 
sub export_policy_rule_modify {
	my ($vserver, $export_policy_obj, $opts) = @_;
	my $test = $vserver->{node}->{test};
	my $result = 1;
	
	try {
		$export_policy_obj->modify(
									%{$opts}
								);

	
	} catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\nExport-policy Rule modify failed with Exception -".$ex->text);
		$result = 0;
	};	
	
	return $result;
}
sub export_policy_rule_create {
	my ($vserver, $policy, $opts) = @_;
	my $Server = $vserver->{node}->{server};
        my $test = $vserver->{node}->{test};
	my $result = 1;
	my $export_policy_rule_object;
        my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
	my $ex;

	try {
		$export_policy_rule_object = NACL::C::ExportPolicyRule->create(
									 command_interface => $Server,
									 policyname        => $policy,
									 vserver           => $vserver->{name},
									 %{$opts}
								);

    	} catch NATE::BaseException with {
            $ex = shift;
    	    if ($reuse =~/yes/ && $ex->text =~ /duplicate entry/) { 
		$test->nlog("\nReuse active , add only objects");
                $result = 1;   
            } else {
		$test->nlog("\nExport-policy Rule create failed with Exception -".$ex->text);
                $result = 0;
            } 	

	};	
	
	if($result && $reuse !~ /yes/) {
		return $export_policy_rule_object;
	}
	return $result;
}

sub nfs_show {
	my $vserver = @_[0];
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $result = 1;
	my $nfs_object;
	
	try {
		$nfs_object = NACL::C::VserverNFS->find(
										command_interface => $Server,
										filter => {
											vserver           => $vserver->{name},
										}
									);

	
	} catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\n NFS is not created for vserver $vserver->{name}".$ex->text);
		$result = 0;
	};	
	
	if($result) {
		return $nfs_object;
	}
	return $result;
}
sub nfs_create {
	my ($vserver, $opts) = @_;
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $result = 1;
	my $nfs_object;
	
	try {
		$nfs_object = NACL::C::VserverNFS->create(
									 command_interface => $Server,
									 vserver           => $vserver->{name},
									 "method-timeout"  => "300",
									 %{$opts}
								);

	
	} catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\n NFS create failed with Exception -".$ex->text);
		$result = 0;
	};	
	
	if($result) {
		return $nfs_object;
	}
	return $result;
}
sub nfs_modify {
	my ($vserver, $nfs_obj, $opts) = @_;
    my $test = $vserver->{node}->{test};
	my $result = 1;
	
	try {
		$nfs_obj->modify(
						        "method-timeout"  => "300",
							%{$opts}
						);

	
	}catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\n NFS modify failed with Exception -".$ex->text);
		$result = 0;
	};	
	
	return $result;
}

sub nfs_delete() {
	my $vserver = @_[0];
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $result = 1;
	my $nfs_object;
	
	try {
		$nfs_object = NACL::C::VserverNFS->find(
										command_interface => $Server,
										filter => {
											vserver           => $vserver->{name},
										}
									);

		$nfs_object->delete();
	} catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\n NFS Delete failed with Exception - ".$ex->text);
		$result = 0;
	};	
	
	return $result;
}
sub get_all_vols {
	my $vserver = @_[0];
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my @volume_obj;
	my @volume_state_obj;
	
	try {
		@volume_obj = NACL::C::Volume->find (
												command_interface => $Server,
												filter => {
													vserver           => $vserver->{name},
												}
											);
	} catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\n Volume Show failed with Exception -".$ex->text);
		return 0;
	};	
	
	foreach(@volume_obj) {
		push(@volume_state_obj,$_->state());
	}
	
	return @volume_state_obj;
}

sub get_root_vol {
	my $vserver = @_[0];
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $vserver_obj;
	
	try {
		$vserver_obj = NACL::C::Vserver->find(
												command_interface => $Server,
												filter => {
													vserver           => $vserver->{name},
												}
											);
	} catch NATE::BaseException with {
        my $ex = shift;
		$test->nlog("\n Vserver does not exist -".$ex->text);
		return 0;
	};	
	return $vserver_obj->state()->rootvolume();
}

sub deleteRoute($) {
	my ($vserver,$opts) = @_;
	my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
	my $route_object;
	my $result = 1;
	try {
		$route_object = NACL::C::Route->find( 
							command_interface => $Server,
							filter            => {
													server            => $vserver->{name},
													gateway           => $opts->{"gateway"},
													'routing-group'   => $opts->{"routing_group"}
												}
						);
		$route_object->delete();
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Route Delete failed with Exception - ".$ex->text);
		$result = 0;
	} catch NATE::Error with {
        $test->nlog("In the Error catch");
		$result = 0;
	};
    if ($result) {
		delete $opts->{gateway};
	}
	return $result;
}

sub deleteDlif($) {
	my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $result = 1;
    my $interface_obj;

    try {
        $interface_obj =  NACL::C::Interface->find(
									command_interface => $Server,
									filter => {
										server            => $vserver->{"name"},
										lif               => $opts->{"name"}
									}
								);
		$interface_obj->delete();
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Dlif delete failed with Exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    if ($result) {
		delete	$opts->{"ip"};
		delete	$opts->{"port"};
		delete	$opts->{"mask"};
		delete	$opts->{"mtu"};
		delete	$opts->{"role"};
	}
	return $result;
}

sub deleteRoutingGroup($) {
	my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $result = 1;
    my $dlif_obj;

    try {
            $dlif_obj =  NACL::C::RoutingGroups->find( 
									command_interface => $Server, 
									filter            => {
										server            => $vserver->{"name"},
										'routing-group'   => $opts->{"routing_group"}
									}
								);
			$dlif_obj->delete();
    } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Routing Group delete failed with Exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
    if ($result) {
		delete $vserver->{dlifs}->{"$opts->{name}"};	
	}
	return $result;
}

sub cifs_server_create($$){
	 my ($vserver,$opts) = @_;    
	 my $test = $vserver->{node}->{test};	 
	 my $node = $vserver->{node} ;
	 my $cifs_server = $opts->{cifs_server}; 
	 my $domain = $opts->{domain};
	 my $username = $opts->{username};
	 my $password = $opts->{password};
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $result = 1;
	 my $cifs_server_obj;
	
	NACL::C::VserverCifs->create(command_interface => $Server, 'vserver' =>$vserver_name, 'domain' =>$domain, 
													'cifs-server' =>$cifs_server, 
													'username'=>$username, 
													'password'=>$password,
													'ou' => "CN=Computers");
								
			
	
	try {
		$cifs_server_obj = NACL::C::VserverCifs->find(command_interface => $Server,
											 filter            => {
																	vserver => $vserver_name
																  }
											);
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Could not create Cifs server for this vserver ".$ex->text); 
        $result = 0;
	};
 
    
	return $result;
	
}

sub cifs_server_delete($$){
	 my ($vserver,$opts) = @_;          
	 my $test = $vserver->{node}->{test};
	 my $node = $vserver->{node} ;
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $username = $opts->{username};
	 my $password = $opts->{password};
	 my $result = 1;
	 try {
	
	NACL::C::VserverCifs->delete(command_interface => $Server,
							'vserver'=>$vserver_name,'username'=>$username, 
							'password'=>$password);
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("CIFS server delete failed with Exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	return $result;
}

sub cifs_server_show($){
	 my ($vserver) = @_;          
	 my $test = $vserver->{node}->{test};
	 my $node = $vserver->{node} ;
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $result = 1;
	 my $server_state;
	 try {
		$server_state = NACL::C::VserverCifs->find(command_interface => $Server,
											 filter            => {
																	vserver => $vserver_name
																  }
											);
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Cifs server not started for this vserver ".$ex->text); 
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	if($result) {
		return $server_state->state();
	}
	return $result;
	 
}

sub cifs_share_show($){
	my ($vserver) = @_; 
	my $test = $vserver->{node}->{test};
	my $node = $vserver->{node} ;
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my @shares = NACL::C::VserverCifsShare->find(command_interface => $Server, filter  => {  'vserver' => $vserver_name });
	my $share_state;
	my @ret = "";
	my $result = 1;
	try {
	foreach my $Share (@shares) {
	 $share_state = $Share->state();
	my $share_name = $share_state->share_name();
	my $path = $share_state->path();
	$test->nlog(" Share Name: ".$share_name);
	$test->nlog(" Path : ".$path);
	#$test->nlog(" Share_properties: ".$share_state->share_properties());
	$test->nlog(" Share_properties: ".Dumper($share_state->share_properties()));
	$test->nlog(" Symlink_properties: ".Dumper($share_state->symlink_properties()));
	$test->nlog(" File_umask: ".$share_state->file_umask());
	$test->nlog(" Dir_umask: ".$share_state->dir_umask());
	$test->nlog(" Comment: ".$share_state->comment());
	$test->nlog(" ACL: ".Dumper($share_state->acl()));
	push @ret,$share_state;
	}
	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	
	
	return @ret;;
	
}

sub namemapping_create($$){
	my ($vserver,$opts) = @_;          
	 my $test = $vserver->{node}->{test};
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $direction  = $opts->{direction};
	 my $pattern = $opts->{pattern};
	 my $replacement = $opts->{replacement};
	 my $position = $opts->{position};
	 my $result = 1;
	 try {
	 NACL::C::VserverNameMapping->create(command_interface => $Server, 'vserver'=>$vserver_name, 'direction'=>$direction, 'pattern'=> $pattern,
										'position' => $position, 'replacement' => $replacement);
	
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Name Mapping create failed with Exception - ",$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	return $result;
}

sub namemapping_delete($$){
	my ($vserver,$opts) = @_;          
	my $test = $vserver->{node}->{test};
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my $direction  = $opts->{direction};
	my $position = $opts->{position};
	my $result = 1;
	try {
	NACL::C::VserverNameMapping->delete( command_interface => $Server,
				'vserver'=>$vserver_name, 'direction'=>$direction,
				'position' => $position);
				
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Name-mapping delete failed with Exception - ".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	
	return $result; 
}

sub namemapping_show($$){
	my ($vserver,$opts) = @_;          
	 my $test = $vserver->{node}->{test};
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my @ret;
	 my $mapping_state ;
	 my $result = 1;
	 try {
	 my @mappings = NACL::C::VserverNameMapping->find(command_interface => $Server,
										filter => { 'vserver' => $vserver_name});
	 foreach my $name (@mappings)
	 {
		$mapping_state = $name->state();
		my $direction = $mapping_state->direction();
		my $pattern = $mapping_state->pattern();
		print "\n\n Direction : $direction\n\n";
		#$ret{$direction} = $pattern;
		$test->nlog("vserver: ".$mapping_state->vserver());
		$test->nlog("Direction: ".$mapping_state->direction());
		$test->nlog("position: ".$mapping_state->position());
		$test->nlog("Pattern: ".$mapping_state->pattern());
		$test->nlog("Replacement: ".$mapping_state->replacement());
		push @ret,$mapping_state;
	 }
	 } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Name-Mapping not found".$ex->text);
        $result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	 return @ret;
	 
}
	 
sub dns_create($$){
	my ($vserver,$opts) = @_;          
	 my $test = $vserver->{node}->{test};
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $domain = $opts->{domain};
	 my $name_servers = $opts->{name_servers};
	 my $state = $opts->{state};
	 my $result = 1;
	 try {
	 NACL::C::VserverServicesDns->create('command_interface' => $Server, 'domains' => $domain, 
							'name-servers' => $name_servers, 'vserver' => $vserver_name, 'state' => $state );

	} catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");
        $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex"
                        . "\nobject ref is $ref_of_obj\n"
                        . "\nstacktrace:\n" . $ex->stacktrace()
                        . "\n\n\n\n"
                        . "\ndumpout:\n" . $dumpout);
        $result = 0;
    } catch NATE::Error with {
        my $ex=shift;
		$test->nlog("In the Error catch".$ex->text);
        $result = 0;
 
    };
	
	
	return $result;
} 

sub dns_delete($$){
	my ($vserver) = @_ ;       
	 my $test = $vserver->{node}->{test};
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $result = 1;
	 try {
	 NACL::C::VserverServicesDns->delete('command_interface' => $Server,'vserver' => $vserver_name);
	 
	 } catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Dns delete failed with Exception -".$ex->text);
		$result = 0;
    } catch NATE::Error with {
        $test->nlog("In the Error catch");
        $result = 0;
 
    };
	
	return $result;
}

sub dns_show($){

	 my ($vserver) = @_ ;
	 my $test = $vserver->{node}->{test};
	 my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $result = 1;
	 my $dns_state;
	 try {
	 $dns_state = NACL::C::VserverServicesDns->find('command_interface' => $Server,
												filter               => {
																			vserver=>$vserver->{name}
																		}
												);
	 
	 } catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Dns entries not present for this vserver - ".$ex->text);
	    $result = 0;
        } catch NATE::Error with {
	    $test->nlog("In the Error catch");
	    $result = 0;
    };
    if($result){
		return $dns_state->state();
    }
    return $result;
}

sub unix_user_show ($$) {
	my ($vserver,$opts) = @_ ;
	my $test = $vserver->{node}->{test};
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my $user = ($opts->{user})?$opts->{user}:'root';
	my $result = 1;
	my $unix_user_state;
	try {
		$unix_user_state = NACL::C::UnixUser->find('command_interface' => $Server,
													filter               => {
																				vserver			  => $vserver_name,
																				user              => $user,
																			}
												);

	 
	} catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Unix User entries not present for this vserver - ".$ex->text);
			$result = 0;
        } catch NATE::Error with {
			$test->nlog("In the Error catch");
			$result = 0;
	};
    if($result){
		return $unix_user_state->state();
    }
    return $result;
}

sub unix_user_create () {
	my ($vserver,$opts) = @_ ;
	my $test = $vserver->{node}->{test};
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my $user = ($opts->{user})?$opts->{user}:'root';
	my $uid = ($opts->{uid})?$opts->{uid}:0;
	my $gid = ($opts->{gid})?$opts->{gid}:0;
	my $result = 1;
	my $unix_user_state;
	try {
		$unix_user_state = NACL::C::UnixUser->create( 
							command_interface => $Server,
							vserver           => $vserver_name,
							user              => $user,
							id                => $uid,
							'primary-gid'     => $gid,
						);

	 
	} catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Unix User create failed with exception - ".$ex->text);
			$result = 0;
        } catch NATE::Error with {
			$test->nlog("In the Error catch");
			$result = 0;
	};
	return $result;
}

sub unix_user_delete () {
	my ($vserver,$opts) = @_ ;
	my $test = $vserver->{node}->{test};
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my $user = ($opts->{user})?$opts->{user}:'root';
	my $result = 1;
	my $unix_user_state;
	try {
		$unix_user_state = NACL::C::UnixUser->find('command_interface' => $Server,
													filter               => {
																				vserver			  => $vserver_name,
																				user              => $user,
																			}
												);
		$unix_user_state->delete();
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Unix User Delete failed with Exception - ".$ex->text);
		$result = 0;
	} catch NATE::Error with {
        $test->nlog("In the Error catch");
		$result = 0;
	};
	
	return $result;
}

sub unix_group_show () {
	my ($vserver,$opts) = @_ ;
	my $test = $vserver->{node}->{test};
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my $group = ($opts->{group})?$opts->{group}:'root';
	my $result = 1;
	my $unix_group_state;
	try {
		$unix_group_state = NACL::C::UnixGroup->find('command_interface' => $Server,
													filter               => {
																				vserver			  => $vserver_name,
																				name              => $group,
																			}
												);

	 
	} catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Unix Group entries not present for this vserver - ".$ex->text);
			$result = 0;
        } catch NATE::Error with {
			$test->nlog("In the Error catch");
			$result = 0;
	};
    if($result){
		return $unix_group_state->state();
    }
    return $result;
}

sub unix_group_create () {
	my ($vserver,$opts) = @_ ;
	my $test = $vserver->{node}->{test};
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my $group = ($opts->{group})?$opts->{group}:'root';
	my $gid = ($opts->{gid})?$opts->{gid}:0;
	my $result = 1;
	my $unix_group_state;
	try {
		$unix_group_state = NACL::C::UnixGroup->create( 
							command_interface => $Server,
							vserver           => $vserver_name,
							name              => $group,
							id                => $gid
						);

	 
	} catch NATE::BaseException with {
            my $ex = shift;
            $test->nlog("Unix Group create failed with exception - ".$ex->text);
			$result = 0;
        } catch NATE::Error with {
			$test->nlog("In the Error catch");
			$result = 0;
	};
	return $result;
}

sub unix_group_delete () {
	my ($vserver,$opts) = @_ ;
	my $test = $vserver->{node}->{test};
	my $Server = $vserver->{node}->{server};
	my $vserver_name = $vserver->{name};
	my $group = ($opts->{group})?$opts->{group}:'root';
	my $result = 1;
	my $unix_group_state;
	try {
		$unix_group_state = NACL::C::UnixGroup->find('command_interface' => $Server,
													filter               => {
																				vserver			  => $vserver_name,
																				name              => $group,
																			}
												);
		$unix_group_state->delete();
	} catch NATE::BaseException with {
        my $ex = shift;
        $test->nlog("Unix User Delete failed with Exception - ".$ex->text);
		$result = 0;
	} catch NATE::Error with {
        $test->nlog("In the Error catch");
		$result = 0;
	};
	
	return $result;
}

sub lockShow($$)
{
	 my ($vserver,$opts) = @_;    
	 my $test = $vserver->{node}->{test};	 
	 my $node = $vserver->{node} ;
	 my $volume = $opts->{volume};
	 #my $Server = $vserver->{node}->{server};
	 my $vserver_name = $vserver->{name};
	 my $lock;
	 my @ret;
	 my $result = 1;
	 my $exec_server = $vserver->{node}->{execname};
         my $Server = Server->new(-Server => "$exec_server");
	 my $parsed_output;
###############
#	 try {
	
#	my @locks =  NACL::C::VserverLocks->find(command_interface => $Server, filter  => {  'vserver' => $vserver_name });

#	foreach my $entry(@locks)
#	 {
#		$lock = $entry->state();
#		push @ret,$lock;
#	}
#	
#	} catch NATE::BaseException with {
#        my $ex = shift;
#        my $dumpout = Dumper($ex);
#        my $ref_of_obj = ref($ex->object());
#        $test->nlog("In the BaseException catch");
#        $test->nlog(
#                          "\nfile is       " . $ex->file()
#                        . "\nline is       " . $ex->line()
#                        . "\ntext is       " . $ex->text
#                        . "\nstringify is  $ex"
#                        . "\nobject ref is $ref_of_obj\n"
#                        . "\nstacktrace:\n" . $ex->stacktrace()
#                        . "\n\n\n\n"
#                        . "\ndumpout:\n" . $dumpout);
#        $result = 0;
#    } catch NATE::Error with {
#        $test->nlog("In the Error catch");
#        $result = 0;
# 
#    };

#############

	# Sometimes the lock state tables take some time to reflect the changes
	#       Adding in some delay
	sleep 15;

  eval {
            my $raw_output;
            $Server->set_execution_server(-Server => $exec_server);
            
		$parsed_output = $Server->vserver_locks_show(vserver => "$vserver->{name}",
                                volume => $volume,
                                output => \$raw_output);
        };
        if ($@) {
            $test->nlog("Vserver Lock Show failed : $@");
            $result = 0;
	    # Resetting the '$@'
	    $@ = "";
        }
	if ($result != 0){
		$test->nlog(Dumper($parsed_output));
		return $parsed_output;
	}
	else{
		return $result;
	}
}

sub locksBreak($$){

	my ($vserver,$opts) = @_; 
	my ($lif,$path);
    	my $Server = $vserver->{node}->{server};
    	my $test = $vserver->{node}->{test};
    	my $exec_server = $vserver->{node}->{execname};       
    	my $result = 1;
	my $serverName = $vserver->{name};
	my $volume = $opts->{volume};
	$lif = ( $opts->{lif} eq "" ) ? "*" : $opts->{lif};
	$path = ( $opts->{path} eq "" ) ? "*" : $opts->{path};
	my $nacl_volume;
	
    	try {
            	$nacl_volume = NACL::C::VserverLocks->break( 
			command_interface => $Server, 
			vserver => $serverName,
			volume => $volume,
			path => $path,
			lif => $lif,
		);
    	} catch NATE::BaseException with {
       		my $ex = shift;
        	my $dumpout = Dumper($ex);
        	my $ref_of_obj = ref($ex->object());
        	$test->nlog("In the BaseException catch");
        	$result = 0;
    	};

	return $result;	    
}


sub createFlexVolQos($$) {
    my ($vserver,$opts) = @_; 
    my $Server = $vserver->{node}->{server};
    my $test = $vserver->{node}->{test};
    my $exec_server = $vserver->{node}->{execname};       
    my $result = 1;
    my $nacl_volume;
    my $user = defined($opts->{user})?$opts->{user}:'root'; 
    my $group = defined($opts->{group})?$opts->{group}:'root'; 
    my $policy = defined($opts->{policy})?$opts->{policy}:'default'; 
    my $reuse = defined($opts->{reuse})?$opts->{reuse}:'no'; 
    my $sec_style = defined($opts->{security_style}) ? $opts->{security_style} : "mixed";
    my $SServer = Server->new(-Server => "$exec_server");
    my $qos_policy_name = defined($opts->{qos_policy}) ? $opts->{qos_policy} : "none";


    try {
            
        my $host = host($exec_server);
        my $version_manager = NACL::APISet::Version->new (hostobj => $host, category => "Node", shell_type => "CMode");
        my $Api_Set_Obj = NACL::APISet->new(hostobj => $host, 
                                category => "Node", 
                                interface => "CLI", 
                                set => "CMode",
                                version_info => {
                                compatibility => 0, # Disable compatibility
                                version_manager => $version_manager,
                                
                                      }
                                 );
        
            
        $Api_Set_Obj->volume_create(
            volume => "$opts->{volname}",
            vserver => "$vserver->{name}",
            aggregate => "$opts->{aggrname}",
            size => "$opts->{volsize}",
            "unix-permissions"=>"$opts->{volperms}",
            "junction-path"=>"$opts->{voljunction}",
            "user"=>"$user", 
            "group"=>"$group", 
            "policy"=>"$policy",
            "security-style" => $sec_style,
            "qos-policy-group" => $qos_policy_name,
            type => "RW") ;
            
            # $nacl_volume = nacl::c::volume->create( 
            # command_interface => $server, 
            # volume => "$opts->{volname}",
            # vserver => "$vserver->{name}",
            # aggregate => "$opts->{aggrname}",
            # size => "$opts->{volsize}",
            # "unix-permissions"=>"$opts->{volperms}",
            # "junction-path"=>"$opts->{voljunction}",
            # "user"=>"$user", 
            # "group"=>"$group", 
            # "policy"=>"$policy","method-timeout"  => "300",
            # "security-style" => $sec_style,
            # "qos-policy-group" => $qos_policy_name,
            # type => "RW",
        # );
              
          

    } catch NATE::BaseException with {
        my $ex = shift;
        my $dumpout = Dumper($ex);
        my $ref_of_obj = ref($ex->object());
        $test->nlog("In the BaseException catch");

        if ($reuse =~ /yes/ && $ex->text =~ /Duplicate volume/ ){
            $test->nlog("Reuse active , adding only the objects");
            $test->nlog("Hmmmm :" . Dumper($vserver->{node}->{params}) ); 
        $result = 1;	    
	} else { 
                $test->nlog(
                          "\nfile is       " . $ex->file()
                        . "\nline is       " . $ex->line()
                        . "\ntext is       " . $ex->text
                        . "\nstringify is  $ex");
                $result = 0;
	}

    };
    
    $opts->{type} = "RW";
    my $volumeObj = volume(vserver => $vserver,opts => $opts);
    $vserver->{vols}->{"$opts->{volname}"} = $volumeObj;
 
    return $result;	    
}

sub modifyDlif {

        my ($vserver,$opts) = @_;
        my $Server = $vserver->{node}->{server};
        my $test = $vserver->{node}->{test};
        my $exec_server = $vserver->{node}->{execname};
        my $result = 1;
        my $interface_state;

        try {
                $interface_state =  NACL::C::Interface->modify (
                        command_interface => $Server,
                        server            => $vserver->{"name"},
                        lif               => $opts->{"lif"},
                        %{$opts},
                );
        } catch NATE::BaseException with {
                my $ex = shift;
                $test->nlog("Dlif migrate failed with Exception - ".$ex->text);
                $result = 0;
        } catch NATE::Error with {
                $test->nlog("In the Error catch");
                $result = 0;
        };
        if ($result) {
                $vserver->{dlifs}->{$opts->{"name"}}= $opts;
        }
        return $result;

}



1;	
