#$Id: //depot/prod/test/main/lib/SPARSE/cmode/init.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2010 Network Appliance, Inc.
# All rights reserved.
#
## @summary	
##      Library which init/unit/setup/cleanup 
##      routines.
##
## @description
##	All scripts call this routine.
##
## @usage
##	use SPARSE::cmode::init;
##
## @author mica@netapp.com

package SPARSE::cmode::init;

# NetApp Specific Libs
use strict;
use warnings;

use TCD;
use Server;
use Tharn;
use Options;
use Params::Common;
use Params::Volume;
use Params::NS;
use TracePkg;
use Subtest;
use Data::Dumper;
use Params::Validate qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF);

#fcluster libs 
use SPARSE::cmode::NodeObject qw(node);
use SPARSE::cmode::AggrObject qw(aggr);
use SPARSE::cmode::ClientObject qw(client);
use SPARSE::cmode::ClientOpsObject qw(clientOps);
use SPARSE::cmode::ClientOpsObject_Win32 qw(clientOpsWin32);
use SPARSE::cmode::VolumeObject qw(volume);

#NACL libs
use NACL::APISet;
use NACL::C::Node;
use NACL::C::Cluster;
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::RowNotFoundException qw(:try);
use NACL::APISet::Exceptions::CDEFNotFoundException qw(:try);
use NACL::APISet::Exceptions::MissingRequiredParamException qw(:try);
use NACL::APISet::Exceptions::InvalidParamValueException qw(:try);
use NACL::APISet::Exceptions::MethodNotFoundException qw(:try);
use NACL::APISet::Exceptions::UnsupportedOptionException qw(:try);
use dblade::wafl::tools::panamax::Version qw(get_version_number get_model);


use vars qw(
  $VERSION
  @ISA
  @EXPORT_OK
  $test
);

require Exporter;

@ISA = qw(Exporter);

@EXPORT_OK = qw(
  &BasicInit
  &BasicUnInit
  &BasicSetup 
  &BasicCleanup 
  &TestCleanUp
);


# -------------------
# Package vars
# -------------------
my $test;
sub TestCleanUp(%);

# -------------------
# basic functions
# -------------------

sub BasicInit(%) {
   my %fparams = validate(
       @_,
       {
           test    => {type => HASHREF},
           scriptParams => {type =>HASHREF},
       });

   my $nodeObjectsHash = {};
   my $clientObjectsHash = {};
   my $resourceObjectsHash = {};

   my $opts = {}; my $i=0;
   my $test = $fparams{test};
   my $scriptParams = $fparams{scriptParams};
   
   my @Nodes = $scriptParams->{FILER};
   # get the node objects 
   my $numNodes = $scriptParams->{NUM_NODES};
   
   $test->nlog("Nodes are ::: " . Dumper(@Nodes));
   my $numNodesArgs = $#{$Nodes[0]} + 1;
	
   if ($numNodes != $numNodesArgs) {
       $test->nlog("Supply the correct number of elements in FILER : ($numNodesArgs) and NUM_NODES ($numNodes)");
       return $TCD::FAIL;
   }
 
   # Configure and return node objects
   # Need dynamic probing code here.
   # Only 2 filers now for vsims 

   my ($server1,$server2,$server1_host,$server2_host,$cdef_path);
   $cdef_path = $scriptParams->{CDEF_PATH};
   my $serverHash = {}; 
   eval{
        # Set server and exec contexts 
        #$server1 = Server->new(-Server => "$Nodes[0][0]"); #control for node1
        #$server2 = Server->new(-Server => "$Nodes[0][1]"); #control for node2

        for ($i=1;$i<=$numNodes;$i++) {
          $serverHash->{"server"."$i"."_host"} = host($Nodes[0][$i-1]);     		
          $serverHash->{"server"."$i"} =  NACL::C::Node->new(node => $Nodes[0][$i-1],
                                    hostrec => $serverHash->{"server"."$i"."_host"}); 
		  $serverHash->{"server"."$i"}->apiset_must({ set => 'CMode' });
		}
	};

   if($@)
   {
	     $test->nlog("Cannot create NACL server objects from FILER : $@");
   }
   
   # Starting with SN, low-end platforms are not supporting flexcache on C-mode
   my @deprecated_models = qw(FAS3240 V3240 N6240
                              FAS3210 V3210 N6210
                              FS3160 V3160 N6060
                              FAS3140 V3140 N6040);
   for ($i=1; $i<=$numNodes; $i++) {
      my $server = "server" . $i;
      my $version = get_version_number($serverHash->{$server});
      my $model = get_model($serverHash->{$server});
      if ((defined($model)) && ($version ge '8.2')) {
         $test->nlog("Platform for node $i: $model");
         if (grep(/^$model$/, @deprecated_models)) {
            $Tharn::Log->log(na_result(
               "Platform $model is deprecated for C-mode flexcache.\n"
               . "Release: $version.\n"
               . "It is highly advisable that you please rerun this test\n"
               . "on a platform different than the following:\n"
               . Dumper(\@deprecated_models)
            ));
            exit;
         }
      }
   }
   
   # Check if Nodes are in same cluster
   my @cluster_object = NACL::C::Cluster->find(command_interface => $serverHash->{"server".1});
   
   my @cluster_nodes;
   
   foreach(@cluster_object) {
		push(@cluster_nodes,$_->node());
   }
   
   my %cluster_hash_nodes = map { $_ => $_ } @cluster_nodes;
   my %nodes_hash = map { $_ => $_ } @{$Nodes[0]};
   
   foreach (keys %nodes_hash) {
		if ($nodes_hash{$_} ne $cluster_hash_nodes{$_}) {
			$test->nlog("Given Nodes are not a part of same cluster");
			return $TCD::FAIL;
		}
   }
	foreach(@cluster_object) {
		if ($_->state()->health() ne 'true' || $_->state()->eligibility() ne 'true') {
			$test->nlog("Given Nodes are part of same cluster but node ".$_->node()." health or eligibility is false.");
			return $TCD::FAIL;
		}
    }
	
	
   # Get the Server object Hash and create respective Node objects
   $opts->{test}=$test;    
   $opts->{params}=$scriptParams;    
   #$test->nlog("Server Object Hash " . Dumper($server1));
   my $nodeHash ={};   
   for ($i=1;$i<=$numNodes;$i++) {
       $test->nlog("Configuring Node Node$i ..............");
       $opts->{name}="Node"."$i";
       $opts->{execname}="$Nodes[0][$i-1]";
       $nodeHash->{"Node"."$i"} = node(server => $serverHash->{"server"."$i"}, opts => $opts);
       $nodeObjectsHash->{"Node"."$i"} = $nodeHash->{"Node"."$i"};
   }
   
   # Client Configuration 
   # Configure and return client objects, be it linux/solaris/windows 
   my @Clients = $scriptParams->{CLIENT};
   $test->nlog("Clients are ::: " . Dumper(@Clients));
   my $numClients = $scriptParams->{NUM_CLIENTS};   
   my $clientHash = {}; 

   my $numClientsArgs = $#{$Clients[0]} + 1 ;
   if ($numClients != $numClientsArgs) {
       $test->nlog("Supply the correct number of elements in CLIENT and NUM_CLIENTS");
       return $TCD::FAIL;
   }
 
   my ($client1,$client2,$clientConn1,$clientConn2,$client1_host,$client2_host);
   eval {
       for ($i=1;$i<=$numClients;$i++) {
           $clientHash->{"client"."$i"."_host"} = host($Clients[0][$i-1]); 
       }

        #$client1_host = host($Clients[0][0]); #control for client1
        #$client2_host = host($Clients[0][1]); #control for client2

        ##$clientConn1 = connect($Clients[0][0]);
        ##$clientConn2 = connect($Clients[0][1]);
 
   };

   if($@)
   {
	    $test->nlog("Cannot create clients handles from CLIENT : $@" );
        return $TCD::FAIL;
   }

   
   my $clientObjHash = {};
   for ($i=1;$i<=$numClients;$i++) {
       $opts={};
       $opts->{test}=$test;    
       $opts->{hdl}=$clientHash->{"client"."$i"."_host"};    
       $opts->{hostname}=$Clients[0][$i-1];
       $opts->{os}=$clientHash->{"client"."$i"."_host"}->hosttype();
         
       $clientObjHash->{"Client"."$i"."Obj"} = client(name=>"Client$i", opts => $opts);
       $clientObjectsHash->{"Client"."$i"} = $clientObjHash->{"Client"."$i"."Obj"};
   }

   $resourceObjectsHash->{filers} = $nodeObjectsHash;
   $resourceObjectsHash->{clients} = $clientObjectsHash;

   
   return $resourceObjectsHash;

}

sub BasicUnInit (%) {
   return 1;  
}

sub updateParam {
    my $oldval = $_[0];
    my $find    = '-';
    my $replace = '_';
    $find = quotemeta $find; 
    $oldval =~ s/$find/$replace/g;
    return $oldval;
}

sub BasicSetup(%) {
    my %fparams = validate(
    @_,
    {
           test    => {type => HASHREF},
           scriptParams => {type =>HASHREF},
           resources => {type =>HASHREF},
	   tasks => {type => SCALAR, optional => 1}
    });
    my $unix_user = 'root'; 
    my $opts = {};
    my $test = $fparams{test};
    my $scriptParams = $fparams{scriptParams};
    my $Tasks = $fparams{tasks};
    my $resourceObjectsHash = $fparams{resources};
    my $nodeObjectsHash = $resourceObjectsHash->{filers};
    my $clientObjectsHash = $resourceObjectsHash->{clients};
    my $setupDone = $scriptParams->{SETUP_DONE};
    my $paramNodeAggr;
    my $paramNodeAggrSize;
    my $key; 
    my $win_user = defined($scriptParams->{AD_SERVER_USER})?$scriptParams->{AD_SERVER_USER}:'administrator'; 
    my $vsim_run = $scriptParams->{"VSIM_RUN"};	
    
    # get the node objects 
    my $numNodes = $scriptParams->{NUM_NODES};
	if ($scriptParams->{SETUP_DONE} =~ /preclean/) {
		$test->nlog("Deleting all volumes of Vserver . . .");
		my ($i, $vservername);
		for($i = 1;$i<=$scriptParams->{NUM_VSERVERS};$i++) {
			$opts = {};$opts->{name} = $scriptParams->{"VSERVER"."$i"."_NAME"};
			my $vserversInNode = $nodeObjectsHash->{"Node1"}->showVserver({vservername=>"$opts->{name}"});
			if ($vserversInNode == 0) {
				next;
			}
			$nodeObjectsHash->{"Node1"}->addVserverObject($opts);
			$vservername = $opts->{name};
			my @all_vols = $nodeObjectsHash->{"Node1"}->{vservers}->{"$opts->{name}"}->get_all_vols();
			my $vserver_root_vol = $nodeObjectsHash->{"Node1"}->{vservers}->{"$opts->{name}"}->get_root_vol();
			foreach(@all_vols) {
				# Delete all volumes from vserver except for vserver root volume
				if ($_->volume() ne $vserver_root_vol) {
					if ($_->type() eq 'DC'){
						$test->nlog("Deleting ".$_->volume());
						$opts = {};
						$opts->{type} = $_->type(); 
						$opts->{volname} = $_->volume();
						$opts->{nacl_volume} = ''; 
						my $volumeObj = volume(vserver => $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"},opts => $opts);
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"} = $volumeObj;
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"}->offline(); 
                                                $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->deleteFlexVol(
                       				{volname => "$opts->{volname}"}); 	
					}
				}
			}

			@all_vols = $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->get_all_vols();
			foreach(@all_vols) {
				# Delete all volumes from vserver except for vserver root volume
				if ($_->volume() ne $vserver_root_vol) {
					if ($_->type() eq 'DP'){
						$test->nlog("Deleting ".$_->volume());
						$opts = {};
						$opts->{type} = $_->type(); 
						$opts->{volname} = $_->volume();
						$opts->{nacl_volume} = ''; 
						my $clusName = $nodeObjectsHash->{"Node1"}->getClusterName();
						my $volumeObj = volume(vserver => $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"},opts => $opts);
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"} = $volumeObj;
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->snapmirrorDelete({ destCluster=>"$clusName", destVolume=>"$opts->{volname}",});
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"}->unmount();
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"}->offline(); 
                        $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->deleteFlexVol(
                       				{volname => "$opts->{volname}"}); 
                     
					}
				}
			}
			
			@all_vols = $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->get_all_vols();
			foreach(@all_vols) {
				# Delete all volumes from vserver except for vserver root volume
				if ($_->volume() ne $vserver_root_vol) {
					if ($_->type() eq 'RW'){
						$test->nlog("Deleting ".$_->volume());
						$opts = {};
						$opts->{type} = $_->type(); 
						$opts->{volname} = $_->volume();
						$opts->{nacl_volume} = ''; 
						my $volumeObj = volume(vserver => $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"},opts => $opts);
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"} = $volumeObj;
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"}->unmount();
						$nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->{vols}->{"$opts->{volname}"}->offline(); 
                        $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"}->deleteFlexVol(
                       				{volname => "$opts->{volname}"}); 
                     
					}
				}
			}
			delete $nodeObjectsHash->{"Node1"}->{vservers}->{"$vservername"};
		}
	}
    # if ($scriptParams->{SETUP_DONE} == 1) {
    # mica 
    if ($scriptParams->{SETUP_DONE} =~ /1/) {
		$test->nlog("Setup is done. Just add objects");
		for $key (keys %$nodeObjectsHash) 
		{
		    my $nodeObj = $nodeObjectsHash->{"$key"};
		    $paramNodeAggr = $scriptParams->{"$key"."_DATAAGGR"};
		    $test->nlog("paramNodeAggr : "."$key"."_DATAAGGR");
		    $paramNodeAggrSize = $scriptParams->{"$key"."_DATAAGGR_DISKS"};
		    $test->nlog("paramNodeAggr : "."$key"."_DATAAGGR_DISKS");
			$test->nlog("Aggregate : $paramNodeAggr "." is present in Node : $nodeObj->{name} just add object");                
			$opts = {};$opts->{name} = $paramNodeAggr;
			my $aggrObj = aggr(node => $nodeObj,opts => $opts);
			$nodeObj->{aggrs}->{$paramNodeAggr} = $aggrObj;
		}
		$test->nlog("Adding Vserver objects for numnodes");
		my ($i,$j,$k)=(1,1,1);
		my $paramNodeVserver;
		for ($i=1;$i<=$numNodes;$i++) {
			for ($j=1;$j<=($scriptParams->{NUM_VSERVERS});$j++) {
				$test->nlog("Adding Vserver object : $j Total vservers : $scriptParams->{NUM_VSERVERS} ");
				$paramNodeVserver=$scriptParams->{"VSERVER"."$j"."_NAME"}; 
				#just add object
				$test->nlog("Vserver : $paramNodeVserver "." is present in Node : $nodeObjectsHash->{\"Node\".\"$i\"}->{name} just add object");                
				$opts = {};$opts->{name} = $paramNodeVserver;
				$nodeObjectsHash->{"Node"."$i"}->addVserverObject($opts);
				my $param_default_volume;
				if($Tasks =~ /create_default_origin_cache/) {
					my $vol_type = 'DC';
					if($i == 1){
						$vol_type = 'RW';
					}
					$param_default_volume = $scriptParams->{"NODE"."$i"."_VSERVER"."$j"."_VOL"};
					$opts = {};
					$opts->{type} = $vol_type; 
					$opts->{volname} = $param_default_volume;
					$opts->{nacl_volume} = ''; 
					my $volumeObj = volume(vserver => $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver},opts => $opts);
					$nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->{vols}->{"$param_default_volume"} = $volumeObj;
					if($vol_type eq 'DC') {
						my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->{vols}->{"$param_default_volume"}->vldb_check({ 'origin' => $scriptParams->{"NODE1"."_VSERVER"."$j"."_VOL"}});
						if(!$result) {
							return $TCD::FAIL;
						}
					}
				}
				#get the details from .thpl or .plan about data ips,port,mtu,gw per dlif
				my $numDlifs = $scriptParams->{"NODE"."$i"."_VSERVER"."$j"."_NUMDLIFS"};
				for ($k=1;$k<=$numDlifs;$k++) {
					$test->nlog("Setting up VSERVER dlifs " . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME" );
				        my $dlifName;
                                        if($scriptParams->{VSIM_RUN} eq 'true') {
                                              $test->nlog("*********Running in a VSIM *********");
                                              if ($scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"} !~ /_data1/) {
                                                 $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"} . "_data1";
                                              } else {
                                                 $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"};
                                              }
                                        } else {
                                              $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"};
                                        }
					
					# Getting the values from tharnhost
					my $dlifIp = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j" . "_NODE" . "$i" ."_DLIF"."$k"."_IP_NAME_THARN"}};
					my $dlifPort = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_PORT_NAME_THARN"}};
					my $dlifMask = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_MASK_NAME_THARN"}};
					my $dlifGW = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_GATEWAY_NAME_THARN"}};
					my $dlifMTU = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_MTU_NAME_THARN"}};               
					
					# Lets ensure that all interface parameters are defined in tharnhost
					#if(!defined($dlifIp) || !defined($dlifPort) || !defined($dlifMask) || !defined($dlifGW) || !defined($dlifMTU)) {
					if(!defined($dlifIp) || !defined($dlifPort) || !defined($dlifMask) || !defined($dlifGW)) {
						$test->nlog("Defined values ip : $dlifIp port : $dlifPort mask : $dlifMask gw : $dlifGW mtu : $dlifMTU");
						$test->nlog("Please define the Interface related parameters properly in Plan file and Tharnhost");
						return $TCD::FAIL;
					}
					
					my $dlifHash = {};
					$dlifHash->{"name"}= $dlifName;
					$dlifHash->{"ip"}= $dlifIp;
					$dlifHash->{"port"}=$dlifPort;
					$dlifHash->{"mask"}=$dlifMask;
					$dlifHash->{"gateway"}=$dlifGW;
					$dlifHash->{"mtu"}=$dlifMTU;
					$dlifHash->{"role"}='data';
					my $dlif_present = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->showDlif($dlifHash);
				
					if($dlif_present == 2) {
						# Incase of exceptions thrown 
						return $TCD::FAIL;
					}
					$nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->{dlifs}->{$dlifName}= $dlifHash;
				}
			}
		}
	}
	if ($scriptParams->{SETUP_DONE} =~ /reuse/ || $scriptParams->{SETUP_DONE} =~ /0/ || $scriptParams->{SETUP_DONE} =~ /preclean/) {
		# Check that aggrs are present in nodes.

               $test->nlog("\n\n\n------------------------------------------------------------------");   
	       $test->nlog("Aggr Setup:");   
	       $test->nlog("------------------------------------------------------------------\n\n");   

		for $key (keys %$nodeObjectsHash) 
		{
		    my $nodeObj = $nodeObjectsHash->{"$key"};
		    $paramNodeAggr = $scriptParams->{"$key"."_DATAAGGR"};
		    $test->nlog("paramNodeAggr : "."$key"."_DATAAGGR");
		    $paramNodeAggrSize = $scriptParams->{"$key"."_DATAAGGR_DISKS"};
		    $test->nlog("paramNodeAggr : "."$key"."_DATAAGGR_DISKS");	 
			my $aggrsInNode = $nodeObj->showAggr({aggrname=>"$paramNodeAggr"});            
			if ($aggrsInNode == 0) {
				# have to create the aggr.	          
				$test->nlog("Aggregate : $paramNodeAggr "." is not present in Node : $nodeObj->{name} create + add object");                
				my $result = $nodeObj->createAggr({'aggrname' => "$paramNodeAggr",'aggrsize' => "$paramNodeAggrSize"});
				if (!$result) {
					$test->nlog("Unable to create Aggregate");
					return $TCD::FAIL;
				}
			} else {
			    # just add object
			    $test->nlog("Aggregate : $paramNodeAggr "." is present in Node : $nodeObj->{name} just add object");                
			    $opts = {};$opts->{name} = $paramNodeAggr;
			    my $aggrObj = aggr(node => $nodeObj,opts => $opts);
			    $nodeObj->{aggrs}->{$paramNodeAggr} = $aggrObj;
			} 
		}

	   #  VSERVER setup code 
            $test->nlog("\n\n\n------------------------------------------------------------------");   
	    $test->nlog("Vserver Setup: ");   
            $test->nlog("------------------------------------------------------------------\n\n");   
 	    $test->nlog("Adding Vserver objects for numnodes");
	   my ($i,$j,$k)=(1,1,1);
	   my $paramNodeVserver;
           # do for each node
	   for ($i=1;$i<=$numNodes;$i++) {

                  $test->nlog("\n\n\n------------------------------------------------------------------");   
	          $test->nlog("Vserver Setup:");   
                  $test->nlog("------------------------------------------------------------------\n\n");   

		  for ($j=1;$j<=($scriptParams->{NUM_VSERVERS});$j++) {
			  $test->nlog("Adding Vserver object : $j Total vservers : $scriptParams->{NUM_VSERVERS} ");
                          $test->nlog("\n\n\n------------------------------------------------------------------");   
	                  $test->nlog("Vserver Setup: for Node $i Vserver $j ");   
                          $test->nlog("------------------------------------------------------------------\n\n");   

			  $paramNodeVserver=$scriptParams->{"VSERVER"."$j"."_NAME"}; 
	                  $test->nlog("Configuring VServer :::: $paramNodeVserver");   
			  my $vserversInNode = $nodeObjectsHash->{"Node"."$i"}->showVserver({vservername=>"$paramNodeVserver"});
	                  #$test->nlog("DUMPERRRRRR ::::".Dumper($nodeObjectsHash->{"Node"."$i"}->{params}));   
			  if ($vserversInNode == 0){
					#have to create the vserver.	          
					$test->nlog("Vserver : $paramNodeVserver "." is not present in Node : $nodeObjectsHash->{\"Node\".\"$i\"}->{name} create + add object");
					my $result = $nodeObjectsHash->{"Node"."$i"}->createVserver({'vservername' => "$paramNodeVserver",'rootaggr'=>"$scriptParams->{Node1_DATAAGGR}"});
					if (!$result) {
						$test->nlog("Unable to create Vserver");
						return $TCD::FAIL;
					}
			  } else{
					#just add object
					$test->nlog("Vserver : $paramNodeVserver "." is present in Node : $nodeObjectsHash->{\"Node\".\"$i\"}->{name} just add object");                
					$opts = {};$opts->{name} = $paramNodeVserver;
					$nodeObjectsHash->{"Node"."$i"}->addVserverObject($opts);
				}
				
				my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->export_policy_show($scriptParams->{"VSERVER".$j."_EXPORT_POLICY"});
				if(!$result) {
					$test->nlog("Export-Policy ".$scriptParams->{"VSERVER".$j."_EXPORT_POLICY"}." not created. Creating it now . . .");
					my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->export_policy_create($scriptParams->{"VSERVER".$j."_EXPORT_POLICY"});
					if(!$result) {
						return $TCD::FAIL;
					}
				}
				$result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->export_policy_rule_show($scriptParams->{"VSERVER".$j."_EXPORT_POLICY"},1);
			        my $rulearr = ['any'];my $superuserarr = ['any'];my $protoarr = ['any'];
				if(!$result) {
					$test->nlog("Rule for Export-Policy ".$scriptParams->{"VSERVER".$j."_EXPORT_POLICY"}." not created. Creating it now . . .");
                                        my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->export_policy_rule_create($scriptParams->{"VSERVER".$j."_EXPORT_POLICY"},
								{ruleindex=>1,clientmatch=>'0.0.0.0/0',rorule=>$rulearr,rwrule=>$rulearr,"allow-suid"=>'true',"allow-dev"=>'true',protocol=>$protoarr,superuser=>$superuserarr,anon=>0});
					if(!$result) {
						return $TCD::FAIL;
					}
				}
				else {
					my $flag = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->export_policy_rule_modify($result,
								{clientmatch=>'0.0.0.0/0',rorule=>$rulearr,rwrule=>$rulearr,"allow-suid"=>'true',"allow-dev"=>'true',protocol=>$protoarr,superuser=>$superuserarr,anon=>0});
					if(!$flag) {
						return $TCD::FAIL;
					}
				}
				
				$result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->unix_user_show({user => 'root'});
				
				if (!$result) {
					my $flag = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->unix_user_create({user => 'root', uid => 0, gid => 0});
					if (!$flag) {
						return $TCD::FAIL;
					}
				}
				
				$result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->unix_group_show({group => 'root'});
				
				if (!$result) {
					my $flag = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->unix_group_create({group => 'root', gid => 0});
					if (!$flag) {
						return $TCD::FAIL;
					}
				}

				my $param_default_volume;
				if($Tasks =~ /create_default_origin_cache/) {
                  		        $test->nlog("\n\n\n------------------------------------------------------------------");   
		                        $test->nlog("Creating origin-cache volume pair: ");   
		                        $test->nlog("------------------------------------------------------------------\n\n");   

					my $vol_type = 'DC';
					if($i == 1){
						$vol_type = 'RW';
					}
					$param_default_volume = $scriptParams->{"NODE"."$i"."_VSERVER"."$j"."_VOL"};
					my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->showVols({volname=>$param_default_volume});
					if($result) {
						my $check = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->showVols({volname=>$param_default_volume, state=>'online', type=>$vol_type, aggregate=>$scriptParams->{"Node"."$i"."_DATAAGGR"}});
						if(!$check) {
							my $aggr = $scriptParams->{"Node"."$i"."_DATAAGGR"};
							$test->nlog("Volume is already present but either it is offline or not $vol_type volume or not present in $aggr"); 
							return $TCD::FAIL;
						}
						$opts = {};
						$opts->{type} = $vol_type; 
						$opts->{volname} = $param_default_volume;
						$opts->{nacl_volume} = $result; 
						my $volumeObj = volume(vserver => $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver},opts => $opts);
						$nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->{vols}->{"$param_default_volume"} = $volumeObj;	
					}
					else {
						$test->nlog("Volume not present. Creating now . . .");	
						if ($vol_type eq 'RW') {
							my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->createFlexVol({
																											'volname'=>$param_default_volume,'aggrname'=>$scriptParams->{Node1_DATAAGGR},
																											'volsize'=>'100MB','volperms'=>'rwxrwxrwx',
																											'voljunction'=>"/$param_default_volume",
                                                                                                                                                                                                                        'reuse'=>"yes"});
							if (!$result) {
								$test->nlog("Unable to create Origin Volume");
								return $TCD::FAIL;
							}
						}
						else {
                                                       # do a check for VSIM Run and rename aggrs as per test.
                                                       my $modAggr;
                                                       if($vsim_run eq 'true') {
                                                          $modAggr = updateParam($scriptParams->{"Node".$i."_DATAAGGR"}) . "_aggr1";
                                                       } else {
                                                          $modAggr = $scriptParams->{"Node".$i."_DATAAGGR"};
                                                       }
                                                   
							my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->createCacheVol({
																											'volname'=>$param_default_volume,'aggrname'=>$modAggr,
																											'volsize'=>'100MB','originvol'=>$scriptParams->{"NODE1"."_VSERVER"."$j"."_VOL"},
                                                                                                                                                                                                                        'reuse'=>"yes"});
							if (!$result) {
								$test->nlog("Unable to create Cache Volume");
								return $TCD::FAIL;
							}
						}
					}
					
					if($vol_type eq 'DC') {
						my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->{vols}->{"$param_default_volume"}->vldb_check({'origin'=>$scriptParams->{"NODE1"."_VSERVER"."$j"."_VOL"}});
						if(!$result) {
							return $TCD::FAIL;
						}
					}
					$nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->{vols}->{"$param_default_volume"}->setExportPolicy({policy=>$scriptParams->{"VSERVER".$j."_EXPORT_POLICY"}});
				}
				
				$result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->nfs_show();
				if(!$result) {
					$test->nlog("NFS not created. Creating it now . . .");
					my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->nfs_create({access=>'true',v2=>'disabled',v3=>'enabled','v4.0'=>'enabled',
								'default-win-user'=>"administrator",'v4.0-acl'=>'enabled','v4.0-read-delegation'=>'enabled','v4.0-write-delegation'=>'enabled',
								'v4-id-domain'=>'eng.btc.netapp.in','v4.1'=>'enabled','v4.1-pnfs'=>'enabled','v4.1-acl'=>'enabled'});
								
					if(!$result) {
						return $TCD::FAIL;
					}
				}
				else {
					my $flag = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->nfs_modify($result, 
					{access=>'true',v2=>'disabled',v3=>'enabled','v4.0'=>"enabled","default-win-user"=>'administrator','v4.0-acl'=>'enabled','v4.0-read-delegation'=>'enabled','v4.0-write-delegation'=>'enabled',
					"v4-id-domain"=>'eng.btc.netapp.in','v4.1'=>'enabled','v4.1-pnfs'=>'enabled','v4.1-acl'=>'enabled'});
					if(!$flag) {
						return $TCD::FAIL;
					}
				}
				
                		$test->nlog("\n\n\n------------------------------------------------------------------");   
		                $test->nlog("Dlif Setup: For node $i For Vserver $j ");   
		                $test->nlog("------------------------------------------------------------------\n\n");   

				#get the details from .thpl or .plan about data ips,port,mtu,gw per dlif
				my $numDlifs = $scriptParams->{"NODE"."$i"."_VSERVER"."$j"."_NUMDLIFS"};
				for ($k=1;$k<=$numDlifs;$k++) {
					$test->nlog("Setting up dlifs in Node $i for param " . "VSERVER$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME" );

				        my $dlifName;
                                        if($scriptParams->{VSIM_RUN} eq 'true') {
                                              $test->nlog("*********Running in a VSIM *********");
                                              if ($scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"} !~ /_data1/) {
                                                 $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"} . "_data1";
                                              } else {
                                                 $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"};
                                              }
                                        } else {
                                              $dlifName = $scriptParams->{"VSERVER" . "$j" . "_NODE" . "$i" . "_DLIF" . "$k" . "_NAME"};
                                        }

      					# Getting the values from tharnhost
                                        #$test->nlog(Dumper($nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}));
                                        
      					my $dlifIp = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j" . "_NODE" . "$i" ."_DLIF"."$k"."_IP_NAME_THARN"}};
					my $dlifPort = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_PORT_NAME_THARN"}};
					my $dlifMask = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_MASK_NAME_THARN"}};
					my $dlifGW = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_GATEWAY_NAME_THARN"}};
					my $dlifMTU = $nodeObjectsHash->{"Node"."$i"}->{server}->{_apisets}->{"Node,CLI,CMode"}->{hostobj}->{$scriptParams->{"VSERVER"."$j". "_NODE" . "$i" ."_DLIF"."$k"."_MTU_NAME_THARN"}};               
					
					# Lets ensure that all interface parameters are defined in tharnhost
					#if(!defined($dlifIp) || !defined($dlifPort) || !defined($dlifMask) || !defined($dlifGW) || !defined($dlifMTU)) {
					if(!defined($dlifIp) || !defined($dlifPort) || !defined($dlifMask) || !defined($dlifGW)) {
						$test->nlog("Defined values ip : $dlifIp port : $dlifPort mask : $dlifMask gw : $dlifGW mtu : $dlifMTU");
						$test->nlog("Please define the Interface related parameters properly in Plan file and Tharnhost");
						return $TCD::FAIL;
					}
					
					my $dlifHash = {};
					$dlifHash->{"name"}= $dlifName;
					$dlifHash->{"ip"}= $dlifIp;
					$dlifHash->{"port"}=$dlifPort;
					$dlifHash->{"mask"}=$dlifMask;
					$dlifHash->{"gateway"}=$dlifGW;
					$dlifHash->{"mtu"}=$dlifMTU;
					$dlifHash->{"role"}='data';
					
					my $dlif_present = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->showDlif($dlifHash);
				
					if($dlif_present == 2) {
						# Incase of exceptions thrown 
						return $TCD::FAIL;
					}
					if(!$dlif_present) {
						$test->nlog("Dlif not present. Creating Dlif");
						my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->createDlif($dlifHash);
						
						if (!$result) {
							$test->nlog("Unable to create Dlif");
							return $TCD::FAIL;
						}
					}
					else {
						# Dlif present. Just add Object
						$nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->{dlifs}->{$dlifName}= $dlifHash;
					}		
					my $route_present = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->showRoute($dlifHash);
					if (!$route_present) {
						$test->nlog("Route not present. Adding Route now . . .");
						my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->createRoute($dlifHash);
						if (!$result){
							$test->nlog("Unable to add Route");
							return $TCD::FAIL;
						}
					}
				}

      				# Cifs setup
                                my $configCifs = defined($scriptParams->{CONFIG_CIFS})?$scriptParams->{CONFIG_CIFS}:'yes'; 
				if($configCifs =~/yes/ && $Tasks =~ /cifs_setup/ ) {

                		       $test->nlog("\n\n\n------------------------------------------------------------------");   
		                       $test->nlog("Cifs Setup: For node $i For Vserver $j ");   
      		                       $test->nlog("------------------------------------------------------------------\n\n");   

					cifs1:
					my $dns_obj = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->dns_show();
					my $domain;
					my $cifs_server;
					my $dns_server;
					if ($dns_obj eq '0') {
						$domain = $scriptParams->{DNS_DOMAIN};
						$cifs_server = $scriptParams->{AD_SERVER};
						$dns_server = $scriptParams->{DNS_SERVER};
						my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->dns_create({domain=>[$domain],state=>'enabled',name_servers=>[$cifs_server,$dns_server]});
						if (!$result) {
							return $TCD::FAIL;
						}
					}
					else {
						# dunno why we delete and recreate.. 
                                                #$nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->dns_delete();
						#goto cifs1;
					}
					cifs2:
					my @namemap_obj = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->namemapping_show();
					my $win_domain_user;
					my @split_domain = split(/\./,$scriptParams->{CIFS_DOMAIN_LONG});
					$win_domain_user = $split_domain[0].'\\\\'.$win_user;
					if (@namemap_obj == 0) {
						cifs4:
						my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->namemapping_create({direction=>'unix-win',pattern=>$unix_user,replacement=>$win_domain_user,position=>1});
						if (!$result) {
							return $TCD::FAIL;
						}
						$result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->namemapping_create({direction=>'win-unix',pattern=>$win_domain_user,replacement=>$unix_user,position=>1});
						if (!$result) {
							return $TCD::FAIL;
						}
					}
					else {
						foreach (@namemap_obj) {
							if($_->{direction} eq 'unix-win' || $_->{direction} eq 'win-unix') {
								$nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->namemapping_delete({position=>$_->{position},direction=>$_->{direction}});
							}
						}
						goto cifs2;
					}
					my $cifs_obj = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->cifs_server_show();
					if ($cifs_obj == 0) {
						my $result = $nodeObjectsHash->{"Node"."$i"}->{vservers}->{$paramNodeVserver}->cifs_server_create({cifs_server=>$paramNodeVserver,domain=>$scriptParams->{CIFS_DOMAIN_LONG},username=>$win_user,password=>$scriptParams->{AD_SERVER_PASSWORD}});
						if (!$result) {
							return $TCD::FAIL;
						}
					}
				}
			}
		}
	}	

        #$test->nlog("INSIDE SETUP ---------------------------------------Q@@#!#@!#!@#!@#");   
        #$test->nlog(Dumper($resourceObjectsHash));   
 	return $resourceObjectsHash;

}

sub BasicCleanup(%) {
}

# Tasks based undo of tests and cleanup
sub TestCleanUp(%) {
	my %fparams = validate(
		@_,
		{
			test    => {type => HASHREF},
			resources => {type => HASHREF},
			tasks => {type => SCALAR},
			params => {type => HASHREF}
		});
		   
		my $resourceObjectsHash = $fparams{resources};
		my $test = $fparams{test};
		my $tasks = $fparams{tasks};
		my $scriptParams = $fparams{params};

		$test->nlog("\n\n\n------------------------------------------------------------------");   
		$test->nlog("Entering Cleanup: ");   
		$test->nlog("------------------------------------------------------------------\n\n");   
	 
		my $nodeObjectsHash  = $resourceObjectsHash->{filers};
		my $clientObjectsHash = $resourceObjectsHash->{clients};

		my ($nodeObject,$aggrObject,$vserverObject,$volumeObject,$node,$server);

	if ($tasks =~ /all/) {
		# construct the $tasks to contain all below strings.   
		TestCleanUp(test => $test, resources => $resourceObjectsHash,
						 tasks => "delete-vservers", params=>$scriptParams);    
		for $nodeObject ( keys %$nodeObjectsHash ) {
			my $aggrObjectHash = $nodeObjectsHash->{$nodeObject}->{aggrs};
			for $aggrObject ( keys %$aggrObjectHash) { 
				$nodeObjectsHash->{$nodeObject}->deleteAggr({aggrname=>$aggrObjectHash->{$aggrObject}->{name}});
			}
		}
	}

	# clean files and dirs in mount point 
	if ($tasks =~ /clean-files-dirs/) {
		$test->nlog("1. Cleaning files/dirs in mount point");
		my ($clientObject,$clientOpsObject);  
		my $ret ;
                #$test->nlog("####Client Tree Dump :".Dumper($clientObjectsHash));
		for $clientObject ( keys %$clientObjectsHash ) {
			$test->nlog("Inside Client Object : " . $clientObjectsHash->{$clientObject}->{name});
                        my $clientOpsHash = $clientObjectsHash->{$clientObject}->{clientops};		           
                           for $clientOpsObject (keys %$clientOpsHash) {
				my $client = $clientOpsHash->{$clientOpsObject}->{client};
				my $clientOpsName = $clientOpsHash->{$clientOpsObject}->{name};
				my $clientHostname = $client->{hostname};
				my $mountPath;
                                
                                # different paths for windows and nix 
                                if ($clientOpsHash->{$clientOpsObject}->{type} =~ /windows/) {
                                  $mountPath = $clientOpsHash->{$clientOpsObject}->{clientmappeddrive}; 
                                } else {
                                  $mountPath = $clientOpsHash->{$clientOpsObject}->{clientmountedpath}; 
                                }				

                                if ($clientOpsHash->{$clientOpsObject}->{type} =~ /windows/) {
                                    $test->nlog("Doing deleteBaseDir for Windows Drive : $clientOpsName");
                                    #mica : can't delete a drive so dummy entry. 
                                } else {
                                    $test->nlog("Doing deleteBaseDir for NFS mount: $clientOpsName");
                                    my $result = $clientOpsHash->{$clientOpsObject}->deleteBaseDir(); 
                                }				
								
				# unmount the mounts 
				if ($tasks =~ /unmount-volumes/ && $clientOpsHash->{$clientOpsObject}->{type} =~ /unix/) {
                                    $test->nlog("Unmounting nfs mounted client mounts");
                                    my $ret = $client->umount_nfsv3({
                                    'clientopsname'=>"$clientOpsName" });   
				}

				# unmap the shares 
				if ($tasks =~ /unmap-shares/ && $clientOpsHash->{$clientOpsObject}->{type} =~ /windows/) {
                                    $test->nlog("Unmapping shares mapped in cifs clients");
                                    my $ret = $client->unmap_share({
                                    'clientopsname'=>"$clientOpsName" });   
				}

			}
		}       

	}


	# unmount nfs mounts 
	if ($tasks =~ /unmount-volumes/) {
		$test->nlog("Unmount the nfs mounts in the clients");
		my ($clientObject,$clientOpsObject);  
		my $ret ;
		for $clientObject ( keys %$clientObjectsHash ) {
			$test->nlog("Inside Client Object : " . $clientObjectsHash->{$clientObject}->{name});
                        my $clientOpsHash = $clientObjectsHash->{$clientObject}->{clientops};		           
                           for $clientOpsObject (keys %$clientOpsHash) {
				my $client = $clientOpsHash->{$clientOpsObject}->{client};
				my $clientOpsName = $clientOpsHash->{$clientOpsObject}->{name};
				my $clientHostname = $client->{hostname};
				my $mountPath;
                                
                                # different paths for windows and nix 
                                if ($clientOpsHash->{$clientOpsObject}->{type} =~ /windows/) {
                                  $mountPath = $clientOpsHash->{$clientOpsObject}->{clientmappeddrive}; 
                                } else {
                                  $mountPath = $clientOpsHash->{$clientOpsObject}->{clientmountedpath}; 
                                }				
								
				# unmount the mounts 
				if ($tasks =~ /unmount-volumes/ && $clientOpsHash->{$clientOpsObject}->{type} =~ /unix/) {
                                    $test->nlog("Unmounting nfs mounted client mounts");
                                    my $ret = $client->umount_nfsv3({
                                    'clientopsname'=>"$clientOpsName" });   
				}

			}
		}       

	}


	# unmap windows shares 
	if ($tasks =~ /unmap-shares/) {
		$test->nlog("Unmount the nfs mounts in the clients");
		my ($clientObject,$clientOpsObject);  
		my $ret ;
		for $clientObject ( keys %$clientObjectsHash ) {
			$test->nlog("Inside Client Object : " . $clientObjectsHash->{$clientObject}->{name});
                        my $clientOpsHash = $clientObjectsHash->{$clientObject}->{clientops};		           
                           for $clientOpsObject (keys %$clientOpsHash) {
				my $client = $clientOpsHash->{$clientOpsObject}->{client};
				my $clientOpsName = $clientOpsHash->{$clientOpsObject}->{name};
				my $clientHostname = $client->{hostname};
				my $mountPath;
                                
                                # different paths for windows and nix 
                                if ($clientOpsHash->{$clientOpsObject}->{type} =~ /windows/) {
                                  $mountPath = $clientOpsHash->{$clientOpsObject}->{clientmappeddrive}; 
                                } else {
                                  $mountPath = $clientOpsHash->{$clientOpsObject}->{clientmountedpath}; 
                                }				
								
				# unmap the shares 
				if ($tasks =~ /unmap-shares/ && $clientOpsHash->{$clientOpsObject}->{type} =~ /windows/) {
                                    $test->nlog("Unmapping shares mapped in cifs clients");
                                    my $ret = $client->unmap_share({
                                    'clientopsname'=>"$clientOpsName" });   
				}

			}
		}       

	}


	if($tasks =~ /clean-volumes/) {
		#check in each nodeobject 
		# unmount/offline volumes [ first do for cache volumes ] 
		$test->nlog("Unmounting Offlining volumes");
		
		for $nodeObject ( keys %$nodeObjectsHash ) {
			my $vserverObjectHash = $nodeObjectsHash->{$nodeObject}->{vservers};
			for $vserverObject ( keys %$vserverObjectHash) {        
				my $volumeObjectHash = $vserverObjectHash->{$vserverObject}->{vols};
				for $volumeObject ( keys %$volumeObjectHash ) {
					my $volType = $volumeObjectHash->{$volumeObject}->{type};
					if ($volType =~ /DC/) {
						$test->nlog("Offlining Volumes");
						$volumeObjectHash->{$volumeObject}->offline(); 
						$test->nlog("Deleting Volumes");
						$vserverObjectHash->{$vserverObject}->deleteFlexVol(
                       		{volname => "$volumeObjectHash->{$volumeObject}->{name}"}); 
					}
				}         
			}
		}
 
		# unmount/offline volumes [ then do for flex volumes .. bug in flex-cluster code ] 
		for $nodeObject ( keys %$nodeObjectsHash ) {
			my $vserverObjectHash = $nodeObjectsHash->{$nodeObject}->{vservers};
			for $vserverObject ( keys %$vserverObjectHash) {        
				my $volumeObjectHash = $vserverObjectHash->{$vserverObject}->{vols};
				for $volumeObject ( keys %$volumeObjectHash ) {
					my $volType = $volumeObjectHash->{$volumeObject}->{type};

                                     	if($tasks =~ /delete-shares/) {
                                                my $share; my $sharehash; my $share_name;
					        $sharehash = $volumeObjectHash->{$volumeObject}->{cifsshares};
                                                for $share ( keys %$sharehash ) {
                                                    $share_name = $sharehash->{$share};
						    $test->nlog("Deleting cifs share [ $share_name ] for the volume");
                                                    $volumeObjectHash->{$volumeObject}->cifs_share_delete({share_name => $share_name});
                                                }
                                        }

                                        if ($volType =~ /(RW|DP)/) {
						$test->nlog("Unmounting Volumes");
						$volumeObjectHash->{$volumeObject}->unmount(); 
						$test->nlog("Offlining Volumes");
						$volumeObjectHash->{$volumeObject}->offline(); 
						$test->nlog("Deleting Volumes");
						if ($volType =~ /DP/) {
							$vserverObjectHash->{$vserverObject}->deleteMirrorVol(
												{volname => "$volumeObjectHash->{$volumeObject}->{name}"});
						} else {
							$vserverObjectHash->{$vserverObject}->deleteFlexVol(
												{volname => "$volumeObjectHash->{$volumeObject}->{name}"});
						}
					}
				}         
			}
		}
	}

	#delete flexcache-cache-policies in vservers    
	if ($tasks =~ /delete-cache-policies/) {
		$test->nlog("Removing all cache policies for vservers");
		my $cachepolicy;
		for $nodeObject ( keys %$nodeObjectsHash ) {
		        $test->nlog("step1". $nodeObjectsHash->{$nodeObject});
			my $vserverObjectHash = $nodeObjectsHash->{$nodeObject}->{vservers};
			for $vserverObject ( keys %$vserverObjectHash) {        
                                $test->nlog("step2". $vserverObjectHash->{$vserverObject});
				my $cachepolicyHash = $vserverObjectHash->{$vserverObject}->{cachepolicies};
				for $cachepolicy ( keys %$cachepolicyHash ) {
                                        $test->nlog("Removing the cache policy : [ $cachepolicy ]");
					$vserverObjectHash->{$vserverObject}->flexcachePolicyDelete({policy => "$cachepolicy"}); 
				}         
			}
		}
	}

	# delete dlifs / routes / routing groups   
	if ($tasks =~ /delete-dlif-routes-group/) {
		$test->nlog("Deleting Dlifs,routes,routing-groups");
		my $dlif;
		for $nodeObject ( keys %$nodeObjectsHash ) {
			my $vserverObjectHash = $nodeObjectsHash->{$nodeObject}->{vservers};
			for $vserverObject ( keys %$vserverObjectHash) {
				my $dlifHash = $vserverObjectHash->{$vserverObject}->{dlifs};
				for $dlif (keys %$dlifHash) {
					$vserverObjectHash->{$vserverObject}->deleteRoute($dlifHash->{$dlif});
					$vserverObjectHash->{$vserverObject}->deleteDlif($dlifHash->{$dlif});
					$vserverObjectHash->{$vserverObject}->deleteRoutingGroup($dlifHash->{$dlif});
				}
			}
		}	
	}
	
	if ($tasks =~ /nfs-cifs-delete/) {
		for $nodeObject ( keys %$nodeObjectsHash ) {
			my $vserverObjectHash = $nodeObjectsHash->{$nodeObject}->{vservers};
			for $vserverObject ( keys %$vserverObjectHash) {
				$vserverObjectHash->{$vserverObject}->cifs_server_delete({username=>$scriptParams->{AD_SERVER_USER},password=>$scriptParams->{AD_SERVER_PASSWORD}});
				$vserverObjectHash->{$vserverObject}->nfs_delete();
			}
		}
	}
	
	# delete vservers
	if ($tasks =~ /delete-vservers/) {
		$test->nlog("Deleting Vservers");
		TestCleanUp(test => $test, resources => $resourceObjectsHash,
						 tasks => "delete-dlif-routes-group,clean-volumes,nfs-cifs-delete",params=>$scriptParams);
		
		my $flag = 1;
		# Deleting Root Volume
		for $nodeObject ( keys %$nodeObjectsHash ) {
			my $vserverObjectHash = $nodeObjectsHash->{$nodeObject}->{vservers};
			for $vserverObject ( keys %$vserverObjectHash) {
				if ($flag){
					my $rootvol = $vserverObjectHash->{$vserverObject}->get_root_vol();
					my $opts = {};
					$opts->{type} = 'RW'; 
					$opts->{volname} = $rootvol;
					$opts->{nacl_volume} = ''; 
					my $volumeObj = volume(vserver => $vserverObjectHash->{$vserverObject},opts => $opts);
					$vserverObjectHash->{$vserverObject}->{vols}->{"$rootvol"} = $volumeObj;
					$vserverObjectHash->{$vserverObject}->{vols}->{"$rootvol"}->unmount(); 
					$test->nlog("Offlining Root Volume");
					$vserverObjectHash->{$vserverObject}->{vols}->{"$rootvol"}->offline(); 
					$test->nlog("Deleting Volumes");
					$vserverObjectHash->{$vserverObject}->deleteFlexVol(
															{volname => "$rootvol"}); 
					$flag = 0;
				}
				$nodeObjectsHash->{$nodeObject}->deleteVserver({vservername=>"$vserverObjectHash->{$vserverObject}->{name}"});
			}
		}
	}
   
   $test->nlog("\n\n\n------------------------------------------------------------------");   
   $test->nlog("Cleanup Done: ");   
   $test->nlog("------------------------------------------------------------------\n\n");   
}
1;
