#Hey emacs, this is basically -*- perl -*-
#$Id$
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary	Library that defines a Filer object.
## @description
##	Abstract common filer tasks and data.
##	Reuse applicable NATE lib functionality.
##
## @usage
##	use Filer;
## @author dl-sparseeng

package SPARSE::Resource::Filer10M;

use strict;
use Aggregate qw(aggr_spare_status aggr_exists aggr_purge aggr_create aggr_add);
use SnapCmds qw(snapreserve_volume snapsched_volume);
use Socket;
use Tharn;
use SPARSE::Frameworks::filer;
use disk qw(get_disks);
use Data::Dumper;
use CLI::FileIO;
use ZapiTest;
use SPARSE::Resource::Volume10M;
use SPARSE::Util::Logger;
use SPARSE::Constants;
use Data::Dumper;

# mica - include 10 mode libs below 
use SPARSE::Frameworks::filer;
#qw(filer_options_get filer_options_set);
use Server;


no warnings("uninitialized");



# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&sparse_filer10M
		&sparse_vfiler10M
		);

# ------------------------------------------------------------------------
my $rule_counter = 1000;
# ------------------------------------------------------------------------

## @name	::sparse_filer10M
## @summary	create a filer object
## @description
##	Connect to filer, instantiate a new filer object, and return it.
## @arg		hostspec Required:
## @arg		opts optional {nfsAcls=>1} ..
## @return	filer object
sub sparse_filer10M($;$)
{
	my ($hostspec, $opts) = @_;
	my $filer = {};
    my $f = host($hostspec);
    my $ip = $f->default_ip_gx();
   
	logTrc("## mica Default IP of GX = $ip");

    my $Server ;

	$filer->{hostSpec} = $hostspec;
	$filer->{hostName} = (split(';', $hostspec))[0];

	logTrc("sparse_filer(origin) in 10M  $filer->{hostName}");
	
    eval { 
        $Server = new Server();
        $Server->set_execution_server(-Server => $hostspec, noprompt => 1); 
    };
	

        $opts->{filerConn} = DFLT_FILER_CONN	unless $opts->{filerConn};
	#$filer->{conn} = connect($hostspec.$opts->{filerConn}, -timeout, 10)
	#	or logError("connect to $hostspec failed: $!");

        $filer->{conn} = $Server; # mica the conn object is 10Mode obj 
	$filer->{hostIP} = host($hostspec)->default_ip_gx(); #$filer->{conn}{hostp}{default_ip};
	$filer->{z} = new ZapiTest($filer->{hostIP}, "root",
				   $filer->{conn}{hostp}{default_password})
		or logError("no zapi");
    $filer->{dataIP} = $f->{DATA_IP1};

	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;
	my $VSERVER_ROOTVOL = SPARSE::Constants::VSERVER_ROOTVOL;
	my $VSERVER_ROOTVOLUME_SECURITY_STYLE = SPARSE::Constants::VSERVER_ROOTVOLUME_SECURITY_STYLE; 
	my $FILER10M_AGGR = SPARSE::Constants::FILER10M_AGGR;
    my $VSERVER_ROOTAGGR = SPARSE::Constants::VSERVER_ROOTAGGR;
	my $VSERVER_DLIF = SPARSE::Constants::VSERVER_DLIF;
    my $VSERVER_DATAPORT = $f->{DATA_PORT1};
    my $VSERVER_DATAIP = $f->{DATA_IP1}; 
    my $VSERVER_DATAMASK = $f->{DATA_MASK1}; 
    my $VSERVER_AGGR_TYPE = SPARSE::Constants::VSERVER_AGGR_TYPE;
    my $VSERVER_AGGR_DISKS = SPARSE::Constants::VSERVER_AGGR_DISKS;
    my $VSERVER_DATA_GATEWAY = $f->{DATA_GATEWAY1}; 
	my $FILER_HOST = $f->hostname();

	logTrc("## mica filer obj dump ## " . Dumper($filer));
	bless($filer);

	# if this is called from subtest, don't bother to repeat the setup
	if ($opts->{subTest}) {
		return $filer;
	}
  
	my $output;
	my @aggrs;

	# mica create a aggr if not present 
	logTrc("##mica checking the 10Mode/Tricky aggregate");
    eval {
        @aggrs = $Server->storage_aggregate_show(aggregate=>$FILER10M_AGGR);
    };
    if ($@) {
		logTrc("##mica creating the 10Mode/Tricky aggregate");
		eval {
			$Server->storage_aggregate_new(aggregate=>$FILER10M_AGGR,
                            diskcount=>$VSERVER_AGGR_DISKS,
                            voltype=>$VSERVER_AGGR_TYPE,
                            raidtype=>"raid_dp");
		};
		if ($@) {
			logTrc("##mica error creating the 10Mode/Tricky aggregate : $@");
		}		
    } else {              
		logTrc("##mica 10Mode/Tricky aggregate already exists");
	}
	
	# mica create a vserver if not present 
	logTrc("##mica checking the vserver");
	my $errorFlag = 1;
    
	eval {
	   	$Server->vserver_show(vserver=>$VSERVER_NAME);
	};
    if ( $@) {
        eval {
			logTrc("##mica create the vserver");
			$Server->vserver_new(vserver=>$VSERVER_NAME, rootvolume=>$VSERVER_ROOTVOL, 
				aggregate=>$VSERVER_ROOTAGGR,'size'=>'2g', 
				'rootvolume-security-style'=>$VSERVER_ROOTVOLUME_SECURITY_STYLE,
	            'ns-switch' => 'file',
				'language' => 'C',
				'snapshot-policy' => 'default',
				'antivirus-on-access-policy' => 'default',
                -Timeout => 60,
                output => \$output,
        	);
	    };
	    if ( $@) {
			logTrc("##mica The vserver create failed :$@");
			exit -1;
		}
	} else {
		logTrc("##mica The vserver exists");
	}
        
	# mica enable nfs license  
        #if (filer_service_is_licensed($filer->{conn}, "nfs") == 0) {
	#$filer->license("nfs");
	#}
	
	logTrc("##mica Checking the nfs service in 10M/Tricky ");
	eval {
           $Server->protocol_nfs_server_show(vserver=>$VSERVER_NAME);
  	};
    if ( $@) {
           logTrc("##mica Begin nfs server create in 10M/Tricky");
           $Server->protocol_nfs_server_new(vserver=>$VSERVER_NAME);
    } else {
           logTrc("##mica nfs server exists in 10M/Tricky");
	} 


    # mica create a data interface for the vserver 
	logTrc("##Filer10M.pm mica Purging the data lif in 10M/Tricky ");
    eval {
		#$Server->network_interface_show(vif=>$VSERVER_DLIF);
        $Server->network_interface_delete(vif=>$VSERVER_DLIF,server=>$VSERVER_NAME);
 
	};
    if ( $@) {
        eval {
            logTrc("##Filer10M.pm mica Creating the data lif network interface");
            $Server->network_interface_new(vif=>$VSERVER_DLIF,
                    server=>$VSERVER_NAME,
                    role=>"data",
                    'home-node'=>$FILER_HOST,
                    'home-port'=>$VSERVER_DATAPORT,
					address=>$VSERVER_DATAIP,
                    netmask=>$VSERVER_DATAMASK);
        };

        if ($@) {
            logTrc("ERROR: Error creating data lif in 10M/Tricky : $@");
            exit -1;
        }
        
    } else {
		#logTrc("##mica data lif network interface exists ");
        eval {
            logTrc("##Filer10M.pm mica Creating the data lif network interface");
            $Server->network_interface_new(vif=>$VSERVER_DLIF,
                    server=>$VSERVER_NAME,
                    role=>"data",
                    'home-node'=>$FILER_HOST,
                    'home-port'=>$VSERVER_DATAPORT,
					address=>$VSERVER_DATAIP,
                    netmask=>$VSERVER_DATAMASK,
					'status-admin'=>'up');
        };

        if ($@) {
            logTrc("ERROR: Error creating data lif in 10M/Tricky : $@");
            exit -1;
        }

	}


    # mica create a route for data interface for the vserver 
	# This is different for tricky nodes so include a check 
	my $isTricky=0;
	eval {
        #Finding the version in a tricky filer
		my $result = $Server->system_image_show(node=>$FILER_HOST,
						'iscurrent'=>'true');
        my $entry;
        my $osString; 
		foreach $entry (@$result) {
			my %hashVal = %$entry;
            $osString = $hashVal{'version'};
			logTrc(Dumper($entry));
            logTrc("Oh my version = $osString");

		}
		logTrc("OSversion : $osString");

		if (($osString =~ /ricky/) || ($osString =~ /10.0.4/)) {
			$isTricky = 1;
		}
	};
    if ($@) {
        #Finding the version in a BR filer
		my $result = $Server->system_node_image_show(node=>$FILER_HOST,
						'iscurrent'=>'true');
        my $entry;
        my $osString; 
		foreach $entry (@$result) {
			my %hashVal = %$entry;
            $osString = $hashVal{'version'};
			logTrc(Dumper($entry));
            logTrc("Oh my version = $osString");

		}
		logTrc("OSversion : $osString");

		if (($osString =~ /ricky/) || ($osString =~ /10.0.4/)) {
			$isTricky = 1;
		}
		
		#logTrc("ERROR: Error finding the OS version : $@");
		#exit -1;
    }
					
    eval {
			logTrc("##mica Purging the existing route for the data lif network interface");
            if ($isTricky == 1) {
				logTrc("##mica Purging the route for data lif route in Tricky ");
				$Server->network_interface_route_delete(server=>$VSERVER_NAME, 
					"vif"=> "$VSERVER_DLIF",
					"destination"=>"*");
			} else {
				#logTrc("##mica Purging the route for data lif route in 10M ");
				#$Server->network_routing_groups_route_delete(server=>$VSERVER_NAME, 
				#	"routing-group"=> "d"."$VSERVER_DATAIP"."/25",
				#	"destination"=>"0.0.0.0/0");
            }
	};
    if ( $@) {
        eval {
            if ($isTricky == 1) {
			logTrc("##mica Creating the route for the data lif network interface in Tricky");
				$Server->network_interface_route_new(server=>$VSERVER_NAME, 
					"vif"=> "$VSERVER_DLIF",
					"destination"=>"0.0.0.0",
					"netmask"=>"$VSERVER_DATAMASK",
					"gateway"=>"$VSERVER_DATA_GATEWAY"
				);
			} else {
				#logTrc("##mica Creating the route for the data lif network interface in 10M");
				#$Server->network_routing_groups_route_create(server=>$VSERVER_NAME, 
				#	"routing-group"=> "d"."$VSERVER_DATAIP"."/25",
				#	"destination"=>"0.0.0.0/0",
				#	"gateway"=>"$VSERVER_DATA_GATEWAY",
				#	"metric"=>20);
			}
   		};
        if ($@) {
            logTrc("ERROR: Error creating route for data lif in 10M/Tricky : $@");
            exit -1;
        }
        
    } else {
        eval {

            if ($isTricky == 1) {
				logTrc("##mica Creating the route for the data lif network interface in Tricky");
				$Server->network_interface_route_new(server=>$VSERVER_NAME, 
					"vif"=> "$VSERVER_DLIF",
					"destination"=>"0.0.0.0",
					"netmask"=>"$VSERVER_DATAMASK",
					"gateway"=>"$VSERVER_DATA_GATEWAY"
				);
			} else {
				#logTrc("##mica Creating the route for the data lif network interface in 10M");
				#$Server->network_routing_groups_route_create(server=>$VSERVER_NAME, 
				#	"routing-group"=> "d"."$VSERVER_DATAIP"."/25",
				#	"destination"=>"0.0.0.0/0",
				#	"gateway"=>"$VSERVER_DATA_GATEWAY",
				#	"metric"=>20);
    		}
		
		};
        if ($@) {
            logTrc("ERROR: Error creating route for data lif in 10M/Tricky : $@");
            exit -1;
        }
	}
 

    # mica create the export-policy rule for the vserver 
	logTrc("## mica Checking export-policy in 10M ");
	eval {
            if ($isTricky == 1) {
	            $Server->protocol_exportruleset_rule_show(vserver=>$VSERVER_NAME,
	            policyname=>"default",
	            ruleindex=>"1"
		        );
			} else {
	            $Server->vserver_export_policy_rule_show(vserver=>$VSERVER_NAME,
	            policyname=>"default",
	            ruleindex=>"1"
		        );
			}	
    };
    if ($@) {

		eval{
            logStep("export-policy rule create");

			if ($isTricky == 1) {
                $Server->protocol_exportruleset_rule_new(vserver=>$VSERVER_NAME,
                policyname=>"default",
                ruleindex=>"1",
                protocol=>"any",
                clientmatch=>"0.0.0.0/0",
                rorule=>"any",
                rwrule=>"any",
                anon=> "0",    #"65534",
                superuser=>"any"
                );            

   		    } else {
                $Server->vserver_export_policy_rule_new(vserver=>$VSERVER_NAME,
                policyname=>"default",
                ruleindex=>"1",
                protocol=>"any",
                clientmatch=>"0.0.0.0/0",
                rorule=>"any",
                rwrule=>"any",
                anon=> "0",    #"65534",
                superuser=>"any"
                );
			}
		};
    	if ($@) {
            logStep("##mica error in export-policy rule create : $@");
    	}

	} else {
           logTrc("##mica export-policy exists in 10M/Tricky");
	}


	#if ($opts->{nfsAcls}) {
        #        $filer->filer_say10M("options nfs.v4.enable on", "^\$");
        #        $filer->filer_say10M("options nfs.v4.acl.enable on", "^\$");
        #}

	# See if we vfiler testing is enabled and also if there are
	# enough arguments
	#if (VFILER_TESTING) {
	#	if ($#main::FILER_SRC == 1 &&
	#	    $main::FILER_SRC[0] =~ /$hostspec/) {
	#		logTrc("creating a sparse vfiler on the source");
	#		return sparse_vfiler10M($main::FILER_SRC[1], $filer);
	#	}
	#	if ($#main::FILER_DST == 1 &&
	#	    $main::FILER_DST[0] =~ /$hostspec/) {
	#		logTrc("creating a sparse vfiler on the dest");
	#		return sparse_vfiler10M($main::FILER_DST[1],$filer);
	#	}
	#}
	return $filer;
}

## @name	::sparse_vfiler10M
## @summary	create a vfiler object with a pointer back to the base vfiler
## @description
##	Iterate of the interfaces of a vfiler and determine ifconfig info.
##	Create the vfiler if necessary with an appropriate root volume.
##	Then create the sparse filer reference to the vfiler with a vfiler0
##	pointer back to the base vfiler.  Return the sparse vfiler.
## @arg		hostspec Required:
## @arg		base_sparse_filer Required:
## @return	filer object
sub sparse_vfiler10M($$)
{
	my ($hostspec, $vfiler0) = @_;
	my $filer = {};
	my $vf0_fc = $vfiler0->{conn};
	my @iplist;

	$vfiler0->license('multistore');

	# Check to see if the hostIP address is already configured
	my $ipspace = $vfiler0->filer_say10M("ipspace list");
	logError("Could not get list of interfaces") unless
		($ipspace  =~ /default-ipspace\s*\(([\w\s]+)/ );
	@iplist = split(' ', $1);

	my $ifnum;
	my $netmask;

	for ($ifnum = 0; $ifnum < @iplist; $ifnum++) {
		$netmask = $vfiler0->filer_say10M("ifconfig $iplist[$ifnum]");

		logTrc("sparse_vfiler: getting interface information for " .
		       "$iplist[$ifnum]");

		if ($netmask  =~ /(netmask\s*[\dxa-f]+\s*broadcast\s*[\d\.]+)/){
	     		$netmask = $1;
			last;
		}
		logTrc("Could not get netmask on interface $iplist[$ifnum]");
	}

	if ($ifnum == @iplist) {
		logError("Could not find any valid interfaces on " .
			 "$vfiler0->{hostName}");
	}

	if ($hostspec !~ /^([\w\.\-\d]+);/) {
		logError("$hostspec: hostname not found");
	}

	my $hostname = $1;
	my $hostIP;
	my $vname = DFLT_VFILER;
	my $vfroot_name = DFLT_VFILER_ROOT;
	my $use_interface = 0;
	my $iface;

	# if host was an IP address, no conversion needed
	if ($hostname =~ /^[\d][\d\.]+$/) {
		$hostIP = $hostname;
	} else {
		$_ = inet_ntoa(scalar(gethostbyname($hostname)));
		logError("$hostname not found") unless (/([\d\.]+)$/);
		$hostIP = $1;
	}

	# see if the IP already exists and use that intferface if it does.
	for (my $num = $ifnum + 1; $num < @iplist; $num++) {
		my $ifline = $vfiler0->filer_say10M("ifconfig $iplist[$num]");
		if ($ifline =~ /$hostIP/) {
			$ifnum = $num;

			# use netmask if available of current interface
			if ($ifline  =~
			    /(netmask\s*[\dxa-f]+\s*broadcast\s*[\d\.]+)/) {
	     			$netmask = $1;
			}
			$use_interface = 1;
			last;
		}
	}

	$iface = $iplist[$ifnum];
	if ($vfiler0->filer_say10M("vfiler status $vname") =~ /not found/) {

		if ($use_interface == 1) {
			logTrc("create vfiler on interface $iface:$hostIP");
 			$vfiler0->filer_say10M("ifconfig $iface down");
		} else {
			logTrc("create vfiler on alias $iface:$hostIP");
			$vfiler0->filer_say10M("ifconfig $iface -alias $hostIP");
		}

		sparse_volume($vfiler0, $vfroot_name, {purge=>1, nomount=>1});

		filer_vfiler_create($vf0_fc, "$vname", $hostIP,
				    "/vol/$vfroot_name");

	}

	# ensure that the vfiler interface is up
	if ($use_interface == 1) {
		$vfiler0->filer_say10M("ifconfig $iface $hostIP $netmask up");
	} else {
		$vfiler0->filer_say10M("ifconfig $iface alias $hostIP $netmask");
	}


	$filer->{hostSpec} = $hostspec;
	$filer->{hostName} = (split(';', $hostspec))[0];
	$filer->{vfiler0} = $vfiler0;
	$filer->{vfilerName} = $vname;

	logTrc("sparse_vfiler $filer->{hostName}");

	$filer->{conn} = connect($hostspec.DFLT_FILER_CONN, -timeout, 10)
		or logError("connect to $hostspec failed: $!");
	$filer->{hostIP} = $filer->{conn}{hostp}{default_ip};

	logInfo("new filer " . Dumper($filer));
	bless($filer);

	# make sure the root is exported persistently
	$filer->filer_say10M("exportfs -p anon=0 /vol/$vfroot_name", "^\$");

	return $filer;
}

sub filer_say($$;$$$)
{
	my ($filer, $cmd, $must, $mustnot, $opts) = @_;
	return filer_say10M($filer, $cmd, $must, $mustnot, $opts);
}


## @name	::filer_say10M
## @summary	send command to filer and check the response
## @arg		cmd required
## @arg		regexp optional must-match pattern
## @arg		regexp optional must-not-match pattern
## @arg		opts optional {timeout=>}
## @return	result of command
sub filer_say10M($$;$$$)
{
	my ($filer, $cmd, $must, $mustnot, $opts) = @_;

	# run the command on base filer in the vfiler context.
	#if ($filer->{vfiler0}) {
	#	$cmd = "vfiler run $filer->{vfilerName} " . $cmd;
	#	$filer = $filer->{vfiler0};
	#}

	#my $timeout = $opts->{timeout} || CMDTIMEOUT;

	#logTrc("say to $filer->{hostName}: \"$cmd\""
	#	. ($must    ? "\nmust match \"$must\"" : "")
	#	. ($mustnot ? "\nmust not match \"$mustnot\"" : ""));

	#my $reply = filer_cli_say($filer->{conn}, $cmd,
	#			  { timeout=>"$timeout", show=>'all', allowerror=>0 } );

	#logDbg("reply was \"$reply\"");

	# strip off any vfiler reply
	#$reply =~ s/\r?\n===== \w+\r?\n?//;

	# check the reply for must-match or must-not-match
	#if ($must && $reply !~ /$must/ || $mustnot && $reply =~ /$mustnot/) {
	#	logError("unexpected result from \"$cmd\": \"$reply\"");
	#}
	#return $reply;
	return "ok";
}

## @name	::filer_blind_say10M
## @summary	send command to filer (async)
## @arg		cmd required
## @arg		opts optional {timeout=>}
## @notes	uses Tharn::say() directly
sub filer_blind_say10M($$;$)
{
	my ($filer, $cmd, $opts) = @_;

	my $timeout = CMDTIMEOUT;
	if ($opts && ref($opts) eq 'HASH' && $opts->{timeout}) {
		$timeout = $opts->{timeout};
	}
	logTrc("say to $filer->{hostName}: \"$cmd\"");
	say($filer->{conn}, $cmd, -blind, -timeout, $timeout);
}

## @name	::license
## @summary	enables a given service on a 10M filer
## @arg		service required 
## @arg		license optional
sub license($$;$)
{
	my ($filer, $service, $license) =@_;

	# only license on the default vfiler.
	# $filer = $filer->{vfiler0} if (defined($filer->{vfiler0}));

	logTrc("#mica license service=$service on 10Mode/Tricky Filer ");
	#filer_service_license($filer->{conn}, $service, $license)
	#	    if (!filer_service_is_licensed($filer->{conn}, $service))

}

## @name	::license_flexcache
## @summary	enables a FlexCache on a filer using either old or new licenses
sub license_flexcache($)
{
    my ($filer) =@_;

	#if (filer_service_is_licensable($filer->{conn}, 'flex_cache')) {
	#license($filer, 'flex_cache');     # Add an old license if this is GB or earlier
	#} elsif (filer_service_is_licensable($filer->{conn}, 'flexcache_nfs')) {
	#license($filer, 'flexcache_nfs');  # Add the new license in IC or later
	#} else {
	#ExitError("Unable to find supported FlexCache license type");
	#}
}

## @name	::unlicense
## @summary	deletes a license for a given service on a filer
## @arg		service required
sub unlicense($$)
{
	my ($filer, $service) = @_;

	# only license on the default vfiler.
	$filer = $filer->{vfiler0} if (defined($filer->{vfiler0}));

	logTrc("unlicense service=$service");
	filer_service_unlicense($filer->{conn}, $service);
}

## @name	::license_flexcache10M
## @summary	enables a FlexCache on a filer using either old or new licenses
sub unlicense_flexcache10M($)
{
    my ($filer) =@_;

    if (filer_service_is_licensable($filer->{conn}, 'flex_cache')) {
	unlicense($filer, 'flex_cache');       # Remove old license if this is GB or earlier
    } elsif (filer_service_is_licensable($filer->{conn}, 'flexcache_nfs')) {
	unlicense($filer, 'flexcache_nfs');    # Remove new license in IC or later
    } else {
	ExitError("Unable to find supported FlexCache license type");
    }
}

## @name	::halt10M
## @summary	halts a filer
## @arg		filer required
## @notes	uses Tharn::say() directly
sub halt10M($)
{
	my ($filer) = @_;

	say($filer->{conn}, "halt", -blind);
}

## @name	::start_net_outage10M
## @summary	simulates network outage on filer for specific connection
## @arg		required local ip addr[:port]
## @arg		required foreign ip addr[:port]
## @arg		optional protocol (tcp, udp, etc.) - default all
## @return	first of the two consecutive ipfw rules that were created
## @example
##    $r = $filer->start_net_outage("10.10.20.30.21", "10.10.20.31:22", "tcp");
##
##    $r = $filer->start_net_outage($filer->{hostIP}, $sfiler->{hostIP});
##    ...
##    $filer->stop_net_outage($r)
sub start_net_outage10M($$$;$)
{
	my ($filer, $laddr, $faddr, $proto) = @_;
	my $r = $rule_counter;
	my $rfmt;

	$rule_counter += 2;

	$proto = "ip" unless $proto;
	$laddr =~ s/:/ /;
	$faddr =~ s/:/ /;

	$rfmt = sprintf("%05d", $r);
	logTrc("start network outage on $proto [$laddr, $faddr]");
	$filer->filer_say10M(PRIVSETTEST . "ipfw delete $r");
	$filer->filer_say10M(PRIVSETTEST .
		"ipfw add $r drop $proto from $laddr to $faddr",
		"$rfmt drop $proto from $laddr to $faddr");
	$r++;
	$filer->filer_say10M(PRIVSETTEST . "ipfw delete $r");
	$rfmt = sprintf("%05d", $r);
	$filer->filer_say10M(PRIVSETTEST .
		"ipfw add $r drop $proto from $faddr to $laddr",
		"$rfmt drop $proto from $faddr to $laddr");
	$filer->filer_say10M(PRIVSETTEST . "options ipfw.enable on", "^\$");
	$filer->filer_say10M(PRIVSETTEST . "ipfw show") if (logCond("trc"));
	return ($r - 1);
}

## @name	::stop_net_outage10M
## @summary	undo network outage on filer for specific connection
## @arg		first of two consecutive ipfw rules to be destroyed
sub stop_net_outage10M($$)
{
  	my ($filer, $rule) = @_;

	logTrc("stop network outage $filer->{hostName} : $rule");
	$filer->filer_say10M(PRIVSETTEST . "ipfw delete $rule", "^\$")
		unless ($filer->filer_say10M(PRIVSETTEST . "ipfw show")
			!~ /$rule/);
	$rule++;
	$filer->filer_say10M(PRIVSETTEST . "ipfw delete $rule", "^\$")
		unless ($filer->filer_say10M(PRIVSETTEST . "ipfw show")
			!~ /$rule/);
}

## @name	::setup_aggr10M
## @summary	create an aggregate
## @arg		aggrname optional
## @arg		options optional aggrSize=>2, aggrPurge=>1, aggrMinSize=>51200
## @example
##	$filer->setup_aggr("PFStestAggr", {aggrSize=>2});
sub setup_aggr10M($;$$)
{
	my ($filer, $aname, $opts) = @_;
	my $fc		  = $filer->{conn};
	$aname		  = DFLT_AGGR unless defined($aname);
	$opts->{aggrSize} = DFLT_AGGR_SIZE unless defined($opts->{aggrSize});
	$opts->{aggrRaidType}
			  = DFLT_RAID_TYPE unless defined($opts->{aggrRaidType});
	$opts->{aggrMinSize}
			  = MIN_AGGR_SIZE unless defined($opts->{aggrMinSize});

	my @aggr_opts =  ($opts->{aggrOpts} ? @{$opts->{aggrOpts}} : ());
	if ($opts->{aggrRaidType}) {
		push(@aggr_opts, (raidtype=>$opts->{aggrRaidType}));
	}
	my $exists = aggr_exists($fc, $aname);
	if ($exists && $opts->{aggrPurge}) {
		logTrc("purge aggr $aname");
		aggr_purge($fc, $aname)
			or logError("purge aggr $aname failed: $@");
		$exists = 0;
	}
	logError("aggr \"$aname\" does not exist")
		if (!$exists && $opts->{aggrNocreate});

	if (!$exists) {
		logTrc("create aggr $aname $opts->{aggrSize} disks");
		aggr_create($fc, $aname, $opts->{aggrSize}, @aggr_opts)
			or logError("create aggr $aname failed: $@");
	}
	return if ($opts->{aggrBasic});	# skip min aggr sizing

	# Keep adding spares until we reach the minimum
	# which will allow for 2 vvols (eg for vfiler)
	while (1) {
		$_ = $filer->filer_say10M("df -A $aname");
		logError("Cannot parse df -A output for $aname")
			unless (/$aname\s+(\d+)/);

		last if $1 >= $opts->{aggrMinSize};
		my $res = aggr_spare_status($fc) or logError("$@");
		my $disk = $res->{0}[0]->{name}
			or logError("Not enough spare disks for aggr");

		logTrc("adding $disk to $aname");
		aggr_add($fc, $aname, [$disk])
			or logError("aggr_add failed for disk $disk: $@");

	}
}

## @name        ::snapvault_throttle10M
## @summary     throttle SnapVault RoD
## @arg         rate required
sub snapvault_throttle10M($$)
{
        my ($filer, $rate) = @_;

        $filer->filer_say10M("options snapvault.restore_on_demand.throttle $rate",
			  "set to $rate KB/s");
}

## @name	::rod_license10M
## @summary	set RoD and snapvault licenses as required
## @arg		primary-filer required
## @arg		secondary-filer required
sub rod_license10M($$)
{
	my ($primary_filer, $secondary_filer) = @_;


	# Remove all the old licenses, because snapvault options could 
	# be in the incorrect state. (i.e. snapvault.enable, etc...)

	$secondary_filer->unlicense10M('sv_ontap_pri')
		if (filer_service_is_licensed($secondary_filer->{conn}, 
									  'sv_ontap_pri'));
	$secondary_filer->unlicense10M('rapid_restore')
		if (filer_service_is_licensed($secondary_filer->{conn}, 
									  'rapid_restore'));

	$primary_filer->unlicense10M('sv_ontap_sec')
		if (filer_service_is_licensed($primary_filer->{conn}, 
							  		  'sv_ontap_sec'));

	$secondary_filer->license10M('sv_ontap_sec');	# 'YQCLSHL'
	$primary_filer->license10M('sv_ontap_pri');	
	$primary_filer->license10M('rapid_restore');
}

## @name	::verify_log10M
## @summary	search logfile for particular pattern
## @arg		pattern required
## @arg		opts optional {log=>.., date=>.., timeout=>..}
sub verify_log10M($$;$)
{
	my ($filer, $pattern, $opts) = @_;

	my $datestring = $opts->{date} || $filer->filer_say10M("date");
	my $logfile    = $opts->{log} || "/etc/log/messages";
	my $timeout    = defined($opts->{timeout}) ?
				$opts->{timeout} : CMDTIMEOUT;
        my $endtime    = time() + $timeout;

	my %months = (
        "Jan"  => "0",
        "Feb"  => "1",
        "Mar"  => "2",
        "Apr"  => "3",
        "May"  => "4",
        "Jun"  => "5",
        "Jul"  => "6",
        "Aug"  => "7",
        "Sep"  => "8",
        "Oct"  => "9",
        "Nov" => "10",
        "Dec" => "11",
	);

	my $old = 40;

	my ($day, $mth, $date, $time, $zone, $year) = split(/\s+/, $datestring);
	my ($hour, $min, $sec) = split(/:/, $time);

	# every logfile seems to have its own timestamp format
	my $timefmt = '^';
	if ($logfile eq "/etc/log/snapmirror") {
		$timefmt .= '\w+\s+';
	}
	$timefmt .= $day . '\s+' . $mth . '\s+' . $date . '\s+'
		 . '(\d+):(\d+):(\d+)\s+' . $zone . '.*';

	use Time::Local;
	my $reftime = timegm($sec, $min, $hour, $date,
			     $months{$mth}, $year-1900);
	# messages can be as old of (t - 40) sec
	$reftime -= $old;

	while (time() < $endtime) {

		my @string = split(/\n/, $filer->filer_say10M("rdfile $logfile",
						undef,
						"^$logfile: No such file "));
		logTrc("searching $logfile for \"$pattern\"");
		foreach my $line (@string) {

			logInfo("log line: $line");

			my ($loghr, $logmin, $logsec) = ($line =~ /$timefmt/);

			if (!defined($loghr)) {
				logInfo("skip timestamp: $line");
				next;
			}
			my $logtime = timegm($logsec, $logmin, $loghr, $date,
						$months{$mth}, $year - 1900);
			
			if ($logtime > $reftime && ($line =~ /$pattern/)) {
				logTrc("$filer->{hostName}: pattern $pattern"
				       . " found in $logfile");
				return 1;
			}
		}
		sleep(5);
        }

	logError("$filer->{hostName}: pattern \'$pattern\' not found in ".
		 "$logfile in the past $old sec");
}

## @name	::statfs10M
## @summary	get 'df' output from the filer
## @description
##	Get 'df' output for a particular volume or aggregate.
##
##	Options are as follows:
##		A=><anything>	given volume is really an aggregate
##		L=><anything>	show local "df" for caching volumes
##		i=><anything>	show inode rather than block stats
##
##	The output format is a hashref which contains data for the
##	vol/aggregate and its snapshots in $ret->{vol} and $ret->{snap}
##	respectively. Each is a hash reference (units in kbytes):
##	[kbytes, used, avail, capacity]
## @arg		filer required
## @arg		volume/aggregate required
## @arg		opts optional
## @return	usage statistics, dependent on options
sub statfs10M($$;$)
{
	my ($filer, $vol, $opts) = @_;

	sub procDfLine
	{
		my ($names, $output) = @_;
		my @elts = split(/\s+/, $output);

		# remove FileSystem path column
		shift @elts;
		# remove next non-digit column
		shift @elts if ($elts[0] !~ /\d+/);

		my %res = map { $_ => shift(@elts) } @$names;
		return \%res;
	}

	my $volopts = "";
	$volopts .= " -A" if $opts->{A};
	$volopts .= " -L" if $opts->{L};
	$volopts .= " -i" if $opts->{i};

	my @names = $opts->{i} ? qw(iused ifree pused) :
		qw(kbytes used avail capacity);

	my $foo = $filer->filer_say10M("df $volopts $vol",
		'^(Filesystem|Aggregate)');
	my @lines = split(/\n/, $foo);

	my $ret = {};

	$ret->{vol} = procDfLine([@names], $lines[1]);
	$ret->{snap} = procDfLine([@names], $lines[2])
		unless ($opts->{i} || $opts->{c});

	return $ret;
}

## @name	::setflag10M
## @summary	issue "setflag" on a filer
## @description
##	Issue "setflag" on a filer
##
## @arg		filer	required
## @arg		opts	required
##
## This kinda sorta should go up into filer.pm
##
sub setflag10M($$$)
{
	my ($filer, $opts) = @_;
	foreach (sort keys %$opts) {
		my ($option, $value) = ($_, $opts->{$_});
		logTrc("setflag $option => $value");
		my $lines = $filer->filer_say10M(PRIVSETDIAG
			. "setflag $option $value");
		logError(@$lines) if $lines;
	}
}

## @name	::stats_start
## @summary	issue "stats start" on a filer
## @description
##	Issue "stats start" on a filer
##
## @arg		filer	required
## @arg		key	optional
## @return	"stats start" identifier string for "stats_stop"
##
## This kinda sorta should go up into filer.pm
##
sub stats_start($;$)
{
	#my ($filer, $key) = @_;

	#logTrc("Start stats on $filer->{hostName}");
	#my $stats = $filer->filer_say10M(PRIVSETDIAG . "stats start $key");
	#my @lines = split(/\n/, $stats);

	#while (@lines) {
	#	my $line = shift @lines;
		# Stats identifier name is 'Ind0x4ec0dae8'
		#	if ($line =~ /^Stats identifier name is '(.*)'$/) {
		#	my $id = $1;
		#	logDbg("Returning '$id'");
		#	return $id;
		#}
		#}

		#undef;
    return 1;
}

## @name	::stats_stop
## @summary	issue "stats stop" on a filer
## @description
##	Issue "stats stop" on a filer
##
## @arg		filer	required
## @arg		id	required
##
## @return	stats
##
## This kinda sorta should go up into filer.pm
##
sub stats_stop($$)
{
	my ($filer, $id) = @_;
    my $opts;
	$opts->{"key"}="volume";
	#logTrc("Stop stats '$id' on $filer->{hostName}");
	#my $stats = $filer->filer_say10M(PRIVSETDIAG . "stats stop -I $id");
	#_stats_parse($stats);
	return stats($filer,$opts); 
}

## @name	::stats
## @summary	get 'stats show' output from the filer
## @description
##	Get 'stats show' output from a filer
##
##	Options
##		raw => <anything>	Get raw stats
##		key => object[:instance[:counter]]
##
## @arg		filer	required
## @arg		opts	optional
## @return	hashref containing stats
##
## WARNING: haven't tested with 2D array CM types
##
## This kinda sorta should go up into filer.pm
##
sub stats($;$)
{
	my ($filer, $opts) = @_;
	my $flags = $opts->{raw} ? "-R" : "";
	my $key = $opts->{key};
    my $Server =  $filer->{conn};  
    my $volName = $main::FILER_VOL;

	#statistics show -node f3070-217-107 -category volume -object volume.rootvol.nfs_read_ops

	logTrc("####Mica Get stats from 10M filer $filer->{hostName}");

	#318         $after_read_ops =
	#319             $after_stats->{'volume'}->{$FILER_VOL}->{'flexcache_read_ops'};
	#320         $after_write_ops =
	#321             $after_stats->{'volume'}->{$FILER_VOL}->{'flexcache_write_ops'};

	if ($key =~ /volume/) {
		my $statsHashRef= {};
		my @returnArrayHashRef;
		my $readOpsVal = 120;
		my $writeOpsVal = 120;
		my $line;
	
		eval {
			 @returnArrayHashRef = $Server->statistics_show(node=>"$filer->{hostName}",
             category=>'volume', object=>"$volName"."*", counter=>"flexcache_read_ops");
    	
 			 foreach (@returnArrayHashRef) {
 				$readOpsVal = $_->{'value'};
			 }

			 @returnArrayHashRef = $Server->statistics_show(node=>"$filer->{hostName}",
             category=>'volume', object=>"$volName"."*", counter=>"flexcache_write_ops");
    	
 			 foreach (@returnArrayHashRef) {
 				$writeOpsVal = $_->{'value'};
			 }

			$statsHashRef->{'volume'}->{"$volName"} = {
                                          				"flexcache_read_ops" => "$readOpsVal",
                                        				"flexcache_write_ops" => "$writeOpsVal" 
													  };
		};

		if ($@) {
			logError("##mica error checking the statistics on the 10Mode/Tricky Filer : $@");
		}		
	
		return $statsHashRef;

	} else {
		return 1;
	}

	#my $stats = $filer->filer_say10M(PRIVSETDIAG . "stats show $flags $key");
	#_stats_parse($stats);

}


sub _stats_parse($)
{
	my ($stats) = @_;
	my @lines = split(/\r?\n/, $stats);
	my $ret = {};

	while (@lines) {
LINE:
		my $line = shift @lines;
		# N.B.: instance names may have embedded ':' characters in them
		# (e.g., "disk"), hence the weird computations. We haven't yet
		# seen objects or counters with embedded ':' characters ...
		if ($line =~ /:$/) {
			# multi-line stat
			my @elts = split(/:/, $line);
			my $object = $elts[0];
			my $instance = join(":", @elts[1 .. $#elts - 1]);
			my $counter = $elts[$#elts];
			while (@lines) {
				my $nextline = shift @lines;
				if ($nextline !~ /^\s/) {
					unshift @lines, $nextline;
					goto LINE;
				}
				$nextline =~ s/\s+//g;
				my ($elem, $value) = split(/:/, $nextline, 2);

				$ret->{$object}{$instance}{$counter}{$elem} =
					$value;
				logDbg("object=$object instance=$instance counter=$counter elem=$elem");
			}
		} else {
			# simple stat
			my @elts = split(/:/, $line);
			my $object = $elts[0];
			my $instance = join(":", @elts[1 .. $#elts - 2]);
			my $counter = $elts[$#elts - 1];
			my $value = $elts[$#elts];

			$ret->{$object}{$instance}{$counter} = $value;
			logDbg("object=$object instance=$instance counter=$counter value=$value");
		}
	}

	return $ret;
}

sub DESTROY10M($)
{
	my ($filer) = @_;

	my $exc_state = $@;

	logDbg("destroying $filer->{hostName} object");
	$@ = $exc_state;
}


1;

__END__
