#$Id: //depot/prod/test/main/lib/SPARSE/Resource/Vol.pm#1 $
#
#Hey emacs, this is basically -*- perl -*-
#vim:filetype=perl
#
# Copyright (c) 2004 Network Appliance, Inc.
# All rights reserved.

## @summary	Library which defines a Volume object.
## @description
##	Abstract common volume tasks and data.
##	Reuse applicable NATE lib functionality.
##
##	This is the base "volume" class; the intent is for
##	other kinds of application volumes (e.g., PFS, RoD,
##	dense) to inherit from this class. The first example
##	of inheritance is SPARSE::Resource::Volume.pm
##
## @usage
##	use SPARSE::Resource::Vol;
## @author dl-sparseeng

package SPARSE::Resource::Vol10M;

use strict;
use Tharn;
use SPARSE::Frameworks::filer;
use mount();
use SPARSE::Frameworks::Volume qw(vol_exists vol_purge vol_status
		vol_create_flex vol_create vol_destroy
		vol_online vol_offline vol_restrict
		vol_container
		);
	# WARNING: see burt105917.
	# vol_create_flex is subject to being renamed as "vol_create".
	# vol_create is subject to being renamed as "vol_create_paired".
use Data::Dumper;
use SPARSE::Constants;
use SPARSE::Util::Logger;
use SPARSE::Util::Utilities;

#mica 
use Server;

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
		&vol10M
		);

# ------------------------------------------------------------------------

## @name	::volume
## @summary	create a volume object
## @description
##	Create or assimilate volume on filer.
## @arg		filer required
## @arg		volname required
## @arg		opts optional
## @return	volume object
sub vol10M($$;$)
{
	my ($vfiler, $name, $opts) = @_;
	my $vol = {};

	$vol->{name}	 = $name;
	$vol->{filer}	 = $vfiler;

	logTrc("#mica 10Mode volume name=$name filer=$vfiler->{hostName}");
	_vol_create($vol, $opts);

	logInfo("volume " . Dumper($vol));
	bless($vol);

	#if (!$opts->{subTest}) {
	#    $vol->exportfs10M($opts);
	#}

	$vol->mount10M($opts) unless $opts->{nomount};	# user will do it later
	return $vol;
}

sub DESTROY($)
{
	my ($vol) = @_;

	my $exc_state = $@;

	# if this process did not mount, it will not unmount
	$vol->umount10M() if ($vol->{mountPid} == $$);
	logDbg("destroying $vol->{name} object");

	$@ = $exc_state;
}

## @name	::exportfs10M
## @summary	export the volume
## @arg		opts optional
## @example	$vol->exportfs()
sub exportfs10M($;$)
{
    #mica - 10mode does not have exportfs concept , export-policy is already done 
	#my ($vol, $opts) = @_;
	#my $o =  ($opts->{noPersistExport}) ? "-io" : "-p";
	#$vol->{filer}->filer_say10M("exportfs $o anon=0 /vol/$vol->{name}", "^\$");
}

## @name	::mount10M
## @summary	mount the volume
## @arg		opts optional
## @example	
##		$vol->mount({mountopts=>"vers=2"});
##		For nfsv4 ACLS,
##			$vol->mount({nfsAcls=1})
sub mount10M($;$)
{
	my ($vol, $opts) = @_;
    my $host    = $vol->{filer}{hostName};
	my $fname   = "/$vol->{name}";
	my $mnt     = "/t/$host/$vol->{name}";
	my $mntopts = $opts->{mountopts} ? $opts->{mountopts}
						 : "actimeo=0,noac,hard,intr";
    my $f = host($main::FILER);
	my $dlifIp = $f->{DATA_IP1};
	logTrc("10mode mount $host (data IP : $dlifIp ): $fname $mnt");
	$vol->{mountPoint} = mount::mount($dlifIp, $fname,
				mountpoint=>$mnt,
				mountopts=>$mntopts,
				vers=> $opts->{nfsAcls} ? "4" : "",
				proto=> $opts->{nfsAcls} ? "tcp" : "",
				timeout=> CMDTIMEOUT,
				)
			or logError("10Mode mount failed: $@");

	$vol->set_testdir();
	$vol->{mountPid} = $$;
}

## @name	::umount10M
## @summary	unmount the volume
sub umount10M($)
{
	logTrc("## mica umount10M in vol10M.pm");
	my ($vol) = @_;
	return unless $vol->{mountPoint};
	##mica remove any files/dirs in the volume ;)
	
	logTrc("unmount $vol->{mountPoint}");
		# NATE workaround
		# cannot run mount::umount($vol->{mountPoint}) in a destructor
		uSystem(mount::_umount_command(
				host("localhost")->hosttype(),
				$vol->{mountPoint}))
			and logError("unmount failed: $!");

	$vol->{mountPoint} = undef;
	$vol->{mountPid} = 0;
}

## @name	::set_testdir10M
## @summary	set test dir on volume
## @arg		dir optional default is DFLT_TEST_DIR
sub set_testdir($;$)
{
	my ($vol, $dir) = @_;
	$dir = DFLT_TEST_DIR unless $dir;
	$vol->{testDirBaseName} = $dir;
	$vol->{testDir} = "$vol->{mountPoint}/$dir";
	$vol->{testDirVolName} = "/$vol->{name}/$dir";
	logTrc("set test dir name=$vol->{testDir}");
}

## @name	::make_testdir10M
## @summary	create test dir on volume
## @arg		dir optional
sub make_testdir10M($;$)
{
	my ($vol, $dir) = @_;

	$vol->set_testdir($dir);

	# set up dir if needed
	if (!-e $vol->{testDir}) {
		logTrc("create test dir name=$vol->{testDir}");

		# XXX need root priv
		system("r mkdir -p $vol->{testDir}")
			and logError("mkdir $vol->{testDir} failed");
	}
	# XXX need root priv
	system("r chmod 0777 $vol->{testDir}")
		and logError("chmod $vol->{testDir} failed");
}

## @name	::make_testdir
## @summary	create test dir on volume
## @arg		dir optional
sub make_testdir($;$)
{
	my ($vol, $dir) = @_;

	$vol->set_testdir($dir);

	# set up dir if needed
	if (!-e $vol->{testDir}) {
		logTrc("create test dir name=$vol->{testDir}");

		# XXX need root priv
		system("r mkdir -p $vol->{testDir}")
			and logError("mkdir $vol->{testDir} failed");
	}
	# XXX need root priv
	system("r chmod 0777 $vol->{testDir}")
		and logError("chmod $vol->{testDir} failed");
}

## @name	::mkfile10M
## @summary	run "mkfile" on a filer
## @arg		size required
## @arg		basename required
## @example	$vol->mkfile("32m", "$vol->{testDirBaseName}/bigfile");
sub mkfile10M($$$)
{
	my ($vol, $size, $basename) = @_;
	my $fc	  = defined($vol->{filer}{vfiler0}) ? 
					$vol->{filer}{vfiler0}{conn} :
					$vol->{filer}{conn};

	return filer_file_make($fc, "/vol/$vol->{name}/$basename", $size);
}

## @name	::rawstat10M
## @summary	Lookup a raw CM stat
## @arg		counter required
## @arg		val optional expected match
## @return	if above val specified confirm a match, else value of counter
## @example	for getting a value
##			my $stat = $dvol->rawstat(
##				"wv_fsinfo_blks_used_by_plane0");
## 		for checking a value
##			$dvol->rawstat("wv_fsinfo_blks_used_by_plane0", 1000);
sub rawstat10M($$;$)
{
	my ($vol, $counter, $val) = @_;
	my $object = "volume";
	my $instance = $vol->{name};
	my $key = "$object:$instance:$counter";

	my $ret = $vol->{filer}->stats10M({raw => 1, key => $key});
	my $value = $ret->{$object}{$instance}{$counter};

	if (!defined($val)) {
		return $value;
	}

	if ($val != $value) {
		logError("Value of $key was $value ".
			 "and not the expected value $val");
	}
}

# ------------------------------------------------------------------------
#			internal routines
# ------------------------------------------------------------------------

## @name	::_vol_create
## @summary	create a (flexible) volume on a filer (internal)
## @arg		size required
## @arg		purge required
sub _vol_create($$$)
{
	my ($vol, $opts) = @_;

	# get the vfiler0 console
	#my $vfiler0 = defined($vol->{filer}{vfiler0})
	#			? $vol->{filer}{vfiler0}
	#			: $vol->{filer};
	#my $fc	    = $vfiler0->{conn};
    my $Server = $vol->{filer}{conn};
	my $vname   = $vol->{name};
    my $exists = 0;

	my $f = host($main::FILER);
	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;
	my $VSERVER_ROOTVOL = SPARSE::Constants::VSERVER_ROOTVOL;
	my $VSERVER_ROOTVOLUME_SECURITY_STYLE = SPARSE::Constants::VSERVER_ROOTVOLUME_SECURITY_STYLE; 
	my $FILER10M_AGGR = SPARSE::Constants::FILER10M_AGGR;
    my $VSERVER_DLIF = SPARSE::Constants::VSERVER_DLIF;
    my $VSERVER_DATAPORT = $f->{DATA_PORT1};
    my $VSERVER_DATAIP = $f->{DATA_IP1}; 
    my $VSERVER_DATAMASK = $f->{DATA_MASK1}; 
    my $VSERVER_AGGR_TYPE = SPARSE::Constants::VSERVER_AGGR_TYPE;
    my $VSERVER_AGGR_DISKS = SPARSE::Constants::VSERVER_AGGR_DISKS;
	my $FILER_HOST = $f->hostname();
    my $VSERVER_DATA_GATEWAY = $f->{DATA_GATEWAY1}; 

	my $isTricky=0;
	eval {
        #Finding the version in a tricky filer
		my $result = $Server->system_image_show(node=>$FILER_HOST,
						'iscurrent'=>'true');
        my $entry;
        my $osString; 
		foreach $entry (@$result) {
			my %hashVal = %$entry;
            $osString = $hashVal{'version'};
			logTrc(Dumper($entry));
            logTrc("Oh my version = $osString");

		}
		logTrc("OSversion : $osString");

		if (($osString =~ /ricky/) || ($osString =~ /10.0.4/)) {
			$isTricky = 1;
		}
	};
    if ($@) {
        #Finding the version in a BR filer
		my $result = $Server->system_node_image_show(node=>$FILER_HOST,
						'iscurrent'=>'true');
        my $entry;
        my $osString; 
		foreach $entry (@$result) {
			my %hashVal = %$entry;
            $osString = $hashVal{'version'};
			logTrc(Dumper($entry));
            logTrc("Oh my version = $osString");

		}

		logTrc("OSversion : $osString");

		if (($osString =~ /ricky/) || ($osString =~ /10.0.4/)) {
			$isTricky = 1;
		}
		
		#logTrc("ERROR: Error finding the OS version : $@");
		#exit -1;
    }

	#mica check if volume/aggr exists , else create aggr,vol 
	# mica create a aggr if not present 
	#logTrc("##mica checking the 10Mode aggregate");
	#my @aggrs;
	#eval {
	#    @aggrs = $Server->storage_aggregate_show(aggregate=>$FILER10M_AGGR);
	#};
	#if ($@) {
	#	logTrc("##mica creating the 10Mode aggregate");
	#	eval {
	#		$Server->storage_aggregate_new(aggregate=>$FILER10M_AGGR,
	#                        diskcount=>$VSERVER_AGGR_DISKS,
	#                        voltype=>$VSERVER_AGGR_TYPE,
	#                        raidtype=>"raid_dp");
	#	};
	#	if ($@) {
	#		logTrc("##mica error creating the 10Mode aggregate : $@");
	#	}		
	#} else {              
	#	logTrc("##mica 10Mode aggregate already exists");
	#}

	# mica create a volume if not present 
	logTrc("##mica Checking 10Mode volume $vname.");
	eval {
        	$Server->volume_show(vserver=>$VSERVER_NAME,
                			volume=>$vname);
	};
	if ($@) {
		if($opts->{nocreate}){
			logTrc("##mica 10Mode volume $vname does not exist and options 
				are set not to create it ... bammbamm  ");
            exit -1;
		}
		eval {
        	logTrc("Creating volume: $vname on aggr: $FILER10M_AGGR");
			#$Server->volume_new(vserver=>$VSERVER_NAME,
			#    			volume=>$vname,
			#                aggregate=>$FILER10M_AGGR,
			#                size=>SPARSE::Constants::DFLT_STDSRC_SIZE,
			#				"unix-permissions"=>"---rwxrwxrwx",
			#                "junction-path"=>"/$vname");

			if ( $isTricky == 1 ) {
						if ($VSERVER_AGGR_TYPE =~ /striped/) {
						logStep("Creating Tricky (striped) volume: $vname on aggr: $FILER10M_AGGR");
						} else {
						logStep("Creating Tricky (flex) volume: $vname on aggr: $FILER10M_AGGR");
						}
        				$Server->volume_new(vserver=>$VSERVER_NAME,
                			volume=>$vname,
                            aggregate=>$FILER10M_AGGR,
                            size=>'3g',
							"unix-permissions"=>"---rwxrwxrwx",
                            "junction-path"=>"/$vname");
					
			} else {
						if ($VSERVER_AGGR_TYPE =~ /striped/) {
							logStep("Creating BR striped (coral) volume: $vname on aggr: $FILER10M_AGGR");
        					$Server->volume_new(vserver=>$VSERVER_NAME,
                			volume=>$vname,
                            aggregate=>$FILER10M_AGGR,
                            size=>'3g',
							'stripe-format'=>'data',
							"unix-permissions"=>"---rwxrwxrwx",
                            "junction-path"=>"/$vname");					
						} else { 
							logStep("Creating BR (flex) volume: $vname on aggr: $FILER10M_AGGR");
        					$Server->volume_new(vserver=>$VSERVER_NAME,
                			volume=>$vname,
                            aggregate=>$FILER10M_AGGR,
                            size=>'3g',
							"unix-permissions"=>"---rwxrwxrwx",
                            "junction-path"=>"/$vname");
                    	}
					
			}


		};
    	if ($@) {
			logTrc("##mica error creating 10Mode volume : $@.");
		} 
    } else {
		$exists = 1;
		logTrc("##mica 10Mode volume $vname already exists .");
		logTrc("##mica purging 10Mode volume $vname as it already exists .");
 		
		if($exists && $opts->{purge}) {

        #mica remove data interfaces 
		logTrc("##Vol10M.pm mica deleting data interface .");
		eval {
			$Server->network_interface_delete(vif=>$VSERVER_DLIF,
	        server=>$VSERVER_NAME);
        };
		if ($@) {
			#logTrc("ERROR: Error Recreating data lif in 10M : $@");
			#exit -1;
		}


		#remove junctions 
		logTrc("##mica unmounting junctions .");
		$Server->volume_unmount(vserver=>$VSERVER_NAME,
                			volume=>$vname);

        #offlining volume   
        logStep("##mica offlining volume .");
        $Server->volume_modify(vserver=>$VSERVER_NAME,
                            volume=>$vname,
                            state=>"offline");

		#delete volume   
		logTrc("##mica deleting volume .");
		$Server->volume_delete(vserver=>$VSERVER_NAME,
                			volume=>$vname);
		# mica we need to delete and create again

		#create the lif interface again 
		logTrc("##mica Recreating the data lif network interface");
		eval {
			$Server->network_interface_new(vif=>$VSERVER_DLIF,
		            server=>$VSERVER_NAME,
		            role=>"data",
		            'home-node'=>$FILER_HOST,
		            'home-port'=>$VSERVER_DATAPORT,
						address=>$VSERVER_DATAIP,
		            netmask=>$VSERVER_DATAMASK);
		};
		if ($@) {
		    logTrc("ERROR: Error Recreating data lif in 10M : $@");
		    exit -1;
		}

        # mica create a route for data interface for the vserver 
	    logTrc("##mica Deleting the route for data lif route in 10M ");
    	eval {
            if ($isTricky == 1) { 
				logTrc("##mica Deleting the existing route for data lif route in Tricky ");
				$Server->network_interface_route_delete(server=>$VSERVER_NAME, 
					"vif"=> "$VSERVER_DLIF",
					"destination"=>"*");
			} else {
				#logTrc("##mica Deleting the existing route for the data lif network interface");
				#$Server->network_routing_groups_route_delete(server=>$VSERVER_NAME, 
				#	"routing-group"=> "d"."$VSERVER_DATAIP"."/25",
				#	"destination"=>"0.0.0.0/0");
			}
		};
    	if ( $@) {
        eval {
            if ($isTricky == 1) { 
				logTrc("##mica Recreating the route for the data lif network interface in Tricky");
				$Server->network_interface_route_new(server=>$VSERVER_NAME, 
					"vif"=> "$VSERVER_DLIF",
					"destination"=>"0.0.0.0",
					"netmask"=>"$VSERVER_DATAMASK",
					"gateway"=>"$VSERVER_DATA_GATEWAY"
				);
			} else {
				#logTrc("##mica Recreating the route for the data lif network interface");
				#$Server->network_routing_groups_route_create(server=>$VSERVER_NAME, 
				#	"routing-group"=> "d"."$VSERVER_DATAIP"."/25",
				#	"destination"=>"0.0.0.0/0",
				#	"gateway"=>"$VSERVER_DATA_GATEWAY",
				#	"metric"=>20);
			}
		};

        if ($@) {
            logTrc("ERROR: Error Recreating route for data lif in 10M : $@");
            exit -1;
        }
        
    	} else {
        	eval {
            if ($isTricky == 1) { 
				logTrc("##mica Recreating the route for the data lif network interface in Tricky");
				$Server->network_interface_route_new(server=>$VSERVER_NAME, 
					"vif"=> "$VSERVER_DLIF",
					"destination"=>"0.0.0.0",
					"netmask"=>"$VSERVER_DATAMASK",
					"gateway"=>"$VSERVER_DATA_GATEWAY"
				);
			} else {
					#logTrc("##mica Recreating the route for the data lif network interface");
					#$Server->network_routing_groups_route_create(server=>$VSERVER_NAME, 
					#	"routing-group"=> "d"."$VSERVER_DATAIP"."/25",
					#	"destination"=>"0.0.0.0/0",
					#	"gateway"=>"$VSERVER_DATA_GATEWAY",
					#	"metric"=>20);
			}
        	};

        	if ($@) {
            	logTrc("ERROR: Error Recreating route for data lif in 10M : $@");
            	exit -1;
        	}
		}


		logTrc("Recreating volume: $vname on aggr: $FILER10M_AGGR");
		eval {
		       		#$Server->volume_new(vserver=>$VSERVER_NAME,
					#		volume=>$vname,
					#        aggregate=>$FILER10M_AGGR,
					#        size=>SPARSE::Constants::DFLT_STDSRC_SIZE,
					#		"unix-permissions"=>"---rwxrwxrwx",
					#        "junction-path"=>"/$vname");

		
				if ( $isTricky == 1 ) {
						if ($VSERVER_AGGR_TYPE =~ /striped/) {
						logStep("ReCreating Tricky (striped) volume: $vname on aggr: $FILER10M_AGGR");
						} else {
						logStep("ReCreating Tricky (flex) volume: $vname on aggr: $FILER10M_AGGR");
						}
        				$Server->volume_new(vserver=>$VSERVER_NAME,
                			volume=>$vname,
                            aggregate=>$FILER10M_AGGR,
                            size=>'3g',
							"unix-permissions"=>"---rwxrwxrwx",
                            "junction-path"=>"/$vname");
					
				} else {
						if ($VSERVER_AGGR_TYPE =~ /striped/) {
							logStep("ReCreating BR striped (coral) volume: $vname on aggr: $FILER10M_AGGR");
        					$Server->volume_new(vserver=>$VSERVER_NAME,
                			volume=>$vname,
                            aggregate=>$FILER10M_AGGR,
                            size=>'3g',
							'stripe-format'=>'data',
							"unix-permissions"=>"---rwxrwxrwx",
                            "junction-path"=>"/$vname");					
						} else { 
							logStep("ReCreating BR (flex) volume: $vname on aggr: $FILER10M_AGGR");
        					$Server->volume_new(vserver=>$VSERVER_NAME,
                			volume=>$vname,
                            aggregate=>$FILER10M_AGGR,
                            size=>'3g',
							"unix-permissions"=>"---rwxrwxrwx",
                            "junction-path"=>"/$vname");
                    	}
			
				}
		
		};
    	if ($@) {
			logTrc("##mica error Recreating 10Mode volume : $@.");
		}
		# mica TODO  	
	    } #end of if 
     }#end of else 

	#my $exists = vol_exists($fc, $vname);
	#logTrc("vol $vname ". ($exists ? "already exists" : "does not exist"));

	#if ($exists && $opts->{purge}) {	# exterminate
	#	logTrc("purging vvol $vname");
	#	vol_purge($fc, $vname)
	#		or logError("purge vol $vname failed: $@");
	#	$exists = 0;
	#}

	#logError("volume \"$vname\" does not exist")
	#if (!$exists && $opts->{nocreate});

	# create and online vol if necessary
	#if (!$exists) {
	#	if ($opts->{tradVol}) {
	#		my $disklist = $opts->{aggrSize} || DFLT_AGGR_SIZE;
	#		logTrc("creating pvol $vname");
	#		vol_create($fc, $vname, $disklist, timeout=>CMDTIMEOUT)
	#			or logError("create aggr $vname failed: $@");
	#	} else {
	#		# pick aggr and set it up
	#		$vol->{aggrName} = ($opts->{aggr} || DFLT_AGGR);
	#		$vfiler0->setup_aggr10M($vol->{aggrName}, $opts);
    #		my $size = $opts->{volSize} || DFLT_VOL_SIZE;
	#		logTrc("creating vvol $vname size=$size");
	#		vol_create_flex(
	#			$fc, $vname, $vol->{aggrName}, $size,
	#			timeout	=> CMDTIMEOUT,
	#			$opts->{volOpts} ? @{$opts->{volOpts}} : ()
	#			)
	#			or logError("create volume $vname failed: $@");
	#	}
	#} elsif (!$opts->{tradVol}) {
	#	# exists and is a vvol -- fill in the aggr
	#	$vol->{aggrName} = vol_container($fc, $vname);
	#}

	# finally move the newly created volume to the vfiler
	#if (defined($vol->{filer}{vfiler0})) {
	#	my $vfiler = $vol->{filer};
	#	$vfiler0->filer_say10M("vfiler move vfiler0 "
	#			   ."$vfiler->{vfilerName} /vol/$vname");
	#}

	$vol->{volPid} = $$;	# creating process
}
