#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/ontap/main/test/lib/SPARSE/Resource/Volume.pm#3 $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary	Library which defines a sparse Volume object.
## @description
##	Abstract common sparse volume tasks and data.
##	Reuse applicable NATE lib functionality.
##
## @usage
##	use SPARSE::Resource::Volume;
## @author dl-sparseeng

package SPARSE::Resource::Volume10M;

use strict;
use Carp;
use Tharn;
use SnapCmds qw(snapreserve_volume snapsched_volume snapshot_list);
use SnapVault qw(:xfer :conf); 
use SPARSE::Frameworks::filer;
use fileSystem qw(treecompare);
use File::Basename;
use Data::Dumper;
use mount();
use TestEnv::MinWin qw(:all);
use SPARSE::Frameworks::Volume qw(vol_exists vol_purge vol_status
		vol_create_flex vol_create vol_destroy
		vol_online vol_offline vol_restrict
		);
use SPARSE::Util::Logger;
use SPARSE::Util::Utilities;
use SPARSE::Constants;
use HTTP::Date qw(str2time);
use Params::Validate
	qw(validate_pos validate BOOLEAN SCALAR ARRAYREF HASHREF);
use Module::Common qw(validate_mixed);
use SPARSE::Resource::Filer10M;
use SPARSE::Resource::Filer; # for setting up sparse relation
use SPARSE::Resource::Vol10M;
use SPARSE::Resource::Volume;
use MapShare;	# for map_cifs_share

#mica 
use Server;


no warnings("uninitialized");

# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(
		Exporter
		SPARSE::Resource::Vol10M
		);
@EXPORT	= qw(
		&sparse_volume10M
		);

use constant QT_ROOT	  => "/qt_root";	# qt name used for qt0
use constant DFLT_SV_MODE => "rod_sv_qtree";	# rod_snapvault
use constant SV_TIMEOUT => 3600;


# ------------------------------------------------------------------------

## @name	::sparse_volume
## @summary	create a volume object
## @description
##	Create or assimilate volume on filer.
## @arg		filer required
## @arg		volname required
## @arg		opts optional
## @return	volume object
sub sparse_volume10M($$;$)
{
	my ($vfiler, $name, $opts) = @_;
	bless vol10M($vfiler, $name, $opts);
}

sub sparse_mode_sv10M($)  { return $_[0] =~ /rod_sv_|rod_snapvault/; }

## @name	::create_cifs_share10M
## @summary	create a CIFS share for the volume
## @arg		share-name required
sub create_cifs_share10M($$)
{
	my ($vol, $share) = @_;

	if ($vol->{mode} =~ /pfs/) {
		logTrc("trying to create a CIFS share on FlexCache volume ".
			"$vol->{name}, should FAIL");
		$vol->{filer}->filer_say10M("cifs shares -add $share ".
					 "/vol/$vol->{name}",
					 "sparse volume does not support CIFS");
		exit(1);	# ?!
	}
	logTrc("creating CIFS share for $vol->{name}");
	$vol->{filer}->filer_say10M("cifs shares -add $share ".
				 "/vol/$vol->{name}", undef,
				 "error|unable|cannot");
}

## @name	::map_share10M
## @summary	map a CIFS share
## @arg		winclient required
## @arg		sharename required
sub map_share10M($$$)
{
	my ($vol, $winclient, $share) = @_;

	logError("need hosttype for client \"$winclient\"")
		unless $winclient =~ /windows/;

	my $sharepath = "\\\\$vol->{filer}{hostName}\\$share";
	logTrc("map share $sharepath on $winclient");
	$vol->{shareClient} = $winclient;

	$vol->{shareDrive} = MapShare::map_cifs_share(
					client=>$vol->{shareClient},
					drive=>"*",
					share=>$sharepath)
		or logError("could not map CIFS on $sharepath");
	$vol->{shareTestPath} = "$vol->{shareDrive}\\"
				.basename($vol->{testDir});
	logDbg("mapped share $sharepath "
		."drive=$vol->{shareDrive} path=$vol->{shareTestPath}");
}

## @name	::unmap_share10M
## @summary	unmap current CIFS share
sub unmap_share10M($)
{
	my ($vol) = @_;

	logTrc("unmap $vol->{shareDrive} on $vol->{shareClient}");
	MapShare::unmap_cifs_share(client=>$vol->{shareClient},
				   drive=>$vol->{shareDrive})
		or logError("unmap_cifs_share failed: $@");
	$vol->{shareClient} = undef;
	$vol->{shareDrive} = undef;
	$vol->{shareTestPath} = undef;
}

## @name	::create_mapped_share10M
## @summary	create cifs share and mount
## @arg		winclient optional default $CLIENT
## @arg		sharename optional default share_<volume name>
## @example	$vol->create_share($CLIENT)
sub create_mapped_share10M($;$$)
{
	my ($vol, $winclient, $share) = @_;

	$winclient = $main::CLIENT unless defined($winclient);
	$share     = "share_".$vol->{name} unless defined($share);

	logStep("create cifs share $share on $vol->{filer}->{hostName}"
		." using client $winclient");
	$vol->create_cifs_share10M($share);

	my $type = "mixed";
	logTrc("set qtree security for $vol->{name} to $type");
	$vol->{filer}->filer_say10M("qtree security /vol/$vol->{name} $type");
	$vol->{filer}->filer_say10M("qtree security /vol/$vol->{name}",
				 "$type security");

	$vol->map_share10M($winclient, $share);
}

## @name	::flush10M
## @summary	flush client system NFS state
sub flush10M($)
{
	my ($vol) = @_;

	logTrc("re-mount $vol->{mountPoint}");

	$vol->umount10M() unless (!$vol->{mountPoint});
	$vol->mount10M();
	$vol->{filer}->filer_say10M(PRIVSETDIAG."sync; sync", "^\$");
}

## @name	::snapshot
## @summary	create a snapshot of the volume
## @arg		name required
sub snapshot($$)
{
	my ($vol, $snap) = @_;
	my $vname = $vol->{name};
	logTrc("#mica create snapshot $snap of 10mode volume $vname");
	my $filer = $vol->{filer};
    my $Server = $vol->{filer}{conn};
	my $vname   = $vol->{name};
	my $f = host($main::FILER);
	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;
	my $FILER_HOST = $f->hostname();

	my $isTricky=0;
	eval {
        #Finding the version in a tricky filer
		my $result = $Server->system_image_show(node=>$FILER_HOST,
						'iscurrent'=>'true');
        my $entry;
        my $osString; 
		foreach $entry (@$result) {
			my %hashVal = %$entry;
            $osString = $hashVal{'version'};
			logTrc(Dumper($entry));
            logTrc("Oh my version = $osString");

		}
		logTrc("OSversion : $osString");

		if (($osString =~ /ricky/) || ($osString =~ /10.0.4/)) {
			$isTricky = 1;
		}
	};
    if ($@) {
        #Finding the version in a BR filer
		my $result = $Server->system_node_image_show(node=>$FILER_HOST,
						'iscurrent'=>'true');
        my $entry;
        my $osString; 
		foreach $entry (@$result) {
			my %hashVal = %$entry;
            $osString = $hashVal{'version'};
			logTrc(Dumper($entry));
            logTrc("Oh my version = $osString");

		}
		logTrc("OSversion : $osString");

		if (($osString =~ /ricky/) || ($osString =~ /10.0.4/)) {
			$isTricky = 1;
		}
		
		#logTrc("ERROR: Error finding the OS version : $@");
		#exit -1;
    }

	eval {
        if ($isTricky == 1) { 
			$Server->volume_snapshot_new(vserver=>$VSERVER_NAME,
	                			volume=>$vname,
								snapshot=>$snap);
			$Server->volume_snapshot_show(vserver=>$VSERVER_NAME,
	                			volume=>$vname,
								snapshot=>$snap); 
		} else {
			$Server->volume_snapshot_create(vserver=>$VSERVER_NAME,
	                			volume=>$vname,
								snapshot=>$snap);
			$Server->volume_snapshot_show(vserver=>$VSERVER_NAME,
	                			volume=>$vname,
								snapshot=>$snap);   	
		}
	};
   	if ($@) {
   			logError("##mica error creating the snapshot for 10Mode/Tricky volume : $@.");

	}	

	#$vol->{filer}->filer_say10M("snap create $vname $snap");
	# Ugh: rsh returns "creating snapshot...", console doesn't
	#$vol->{snapshot} = $snap;
	#$vol->{filer}->filer_say10M("df /vol/$vname/.snapshot/$snap",
	#			 "^Filesystem");
}

## @name	::snaprestorevol
## @summary	snaprestore to a snapshot of the volume
## @arg		name required
sub snaprestorevol($$)
{
	my ($vol, $snap) = @_;
	my $vname = $vol->{name};
	logTrc("#mica snaprestore to snapshot $snap of 10mode volume $vname");
	my $filer = $vol->{filer};
    my $Server = $vol->{filer}{conn};
	my $vname   = $vol->{name};
	my $f = host($main::FILER);
	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;
	my $FILER_HOST = $f->hostname();

	my $isTricky=0;
	eval {
		my $result = $Server->system_image_show(node=>$FILER_HOST,
						'iscurrent'=>'true');
        my $entry;
        my $osString; 
		foreach $entry (@$result) {
			my %hashVal = %$entry;
            $osString = $hashVal{'version'};
			logTrc(Dumper($entry));
            logTrc("Oh my version = $osString");

		}
		logTrc("OSversion : $osString");

		if (($osString =~ /ricky/) || ($osString =~ /10.0.4/)) {
			$isTricky = 1;
		}
	};


	eval {
        if ($isTricky == 1) { 
			$Server->volume_snapshot_promote(vserver=>$VSERVER_NAME,
	                			volume=>$vname,
								snapshot=>$snap);
 
		} else {
			$Server->volume_snapshot_promote(vserver=>$VSERVER_NAME,
	                			volume=>$vname,
								snapshot=>$snap);
		}
	};
   	if ($@) {
   			logError("##mica error snaprestoring to the snapshot for 10Mode/Tricky volume : $@.");
	}	

	#$vol->{filer}->filer_say10M("snap create $vname $snap");
	# Ugh: rsh returns "creating snapshot...", console doesn't
	#$vol->{snapshot} = $snap;
	#$vol->{filer}->filer_say10M("df /vol/$vname/.snapshot/$snap",
	#			 "^Filesystem");
}

## @name	::online10M
## @summary	online a volume
sub online($)
{
	my ($vol) = @_;
	my $filer = $vol->{filer};
    my $Server = $vol->{filer}{conn};
	my $vname   = $vol->{name};
	my $f = host($main::FILER);
	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;

	# option -> no_atime_update
	logTrc("Setting volume online in 10M ");
	eval {
		$Server->volume_unmount(vserver=>$VSERVER_NAME,
	                			volume=>$vname);
	    $Server->volume_modify(vserver=>$VSERVER_NAME,
							   volume=>$vname,
						   	   'state'=>'online');
   	};
   	if ($@) {
   			logError("##mica error setting 10Mode/Tricky volume online : $@.");
	}	

	#logTrc("online volume $vol->{name}");
	#vol_online($vol->{filer}{conn}, $vol->{name})
	#	or logError("online failed $vol->{name}: $@");
	#$vol->verify_status10M({State=>"online"});
}

## @name	::offline10M
## @summary	offline a volume
sub offline($)
{
    #mica modified 
	my ($vol) = @_;
	my $filer = $vol->{filer};
    my $Server = $vol->{filer}{conn};
	my $vname   = $vol->{name};
	my $f = host($main::FILER);
	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;

	# option -> no_atime_update
	logTrc("Setting volume offline in 10M ");
	eval {
		$Server->volume_unmount(vserver=>$VSERVER_NAME,
	                			volume=>$vname);
	    $Server->volume_modify(vserver=>$VSERVER_NAME,
							   volume=>$vname,
						   	   'state'=>'offline');
   	};
   	if ($@) {
   			logError("##mica error setting 10Mode/Tricky volume offline : $@.");
	}	

	#logTrc("offline volume $vol->{name}");
	#vol_offline($vol->{filer}{conn}, $vol->{name})
	#	or logError("offline failed $vol->{name}: $@");
	$vol->verify_status({State=>"offline"});

}

## @name	::destroy10M
## @summary	destroy a volume
sub destroy10M($)
{
	my ($vol) = @_;
	logTrc("destroying volume $vol->{name}");
	vol_purge($vol->{filer}->{conn}, $vol->{name})
			or logError("purge vol $vol->{name} failed: $@");
}

## @name	::restrict10M
## @summary	restrict a volume
sub restrict10M($)
{
	my ($vol) = @_;

	logTrc("restrict volume $vol->{name}");
	vol_restrict($vol->{filer}{conn}, $vol->{name})
		or logError("restrict failed $vol->{name}: $@");
	$vol->verify_status10M({State=>"restricted"});
}

## @name	::read_fsid10M
## @summary	return fsid of a volume
## @arg		vol required
## @return	fsid in decimal
sub read_fsid10M($s)
{
	my ($vol) = @_;
	my $result = $vol->{filer}->filer_say10M(PRIVSETTEST .
			"vol read_fsid $vol->{name}");
	my @out = split(/[\s.]/, $result);
	my $fsid = hex($out[6]);
	return $fsid;
}

## @name	::verify_status
## @summary	verifies volume status for specified volume status
## @arg		status required eg {State=>online, Status=>"raid4|raid_dp"}
##		WARNING: "Options" not really supported
sub verify_status($$)
{
	my ($vol, $exp) = @_;
    my %hashExp = %{$exp};
	my $filer = $vol->{filer};
    my $Server = $vol->{filer}{conn};
	my $vname   = $vol->{name};
	my $f = host($main::FILER);
	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;

	#mica parse the input hash for options 
	for my $attr (keys %hashExp) {
		# Volume::vol_status uses lowercase keys; all callers pass in
		# capitalized keys
		my $lcattr = lc($attr);
		my $lcvalue = lc($hashExp{$attr});
		#logError("unexpected volume status $vol->{name}:"
		# . " $attr=\"$result->{$lcattr}\""
		# . " expected \"$exp->{$attr}\"")
		#unless $result->{$lcattr} =~ /$exp->{$attr}/;

		if ($lcvalue eq 'state') {
			# option State=> offline/online
			logTrc("Checking if volume $vname is $lcvalue state in 10M ");
			eval {
				$Server->volume_show(vserver=>$VSERVER_NAME,
	                			volume=>$vname,state=>$lcvalue);
   			};
   			if ($@) {
   					logError("##mica error the 10Mode/Tricky volume $vname is not $lcvalue : $@.");
			} else {
   					logTrc("##mica success the 10Mode/Tricky volume $vname is $lcvalue :");
			}		

			#my $result = vol_status($vol->{filer}->{conn}, $vol->{name});
		}


	}#end of for loop 

}

## @name	::vol_options
## @summary	sets a specified option to a specified value
## @arg		option required
## @arg		value required
sub vol_options($$$)
{
	#mica changes here ... 
	my ($vol, $option, $value) = @_;
	my $filer = $vol->{filer};
    my $Server = $vol->{filer}{conn};
	my $vname   = $vol->{name};

	my $f = host($main::FILER);
	my $VSERVER_NAME = SPARSE::Constants::VSERVER_NAME;
	my $VSERVER_DLIF = SPARSE::Constants::VSERVER_DLIF;
	my $VSERVER_DATAPORT = $f->{DATA_PORT1};
	my $VSERVER_DATAIP = $f->{DATA_IP1}; 
	my $VSERVER_DATAMASK = $f->{DATA_MASK1}; 
	my $VSERVER_DATA_GATEWAY = $f->{DATA_GATEWAY1}; 

	logTrc("#mica set on 10Mode/Tricky volume $vol->{name}: equivalent of the 7M/7G option $option = $value");
	#$filer->filer_say("vol options $vol->{name} $option $value", "^\$");


	#logTrc("Setting previlege diag in 10M");
	#$Server->set(privilege=>"diag");

	# option -> no_atime_update
	if($option eq "no_atime_update") {
		logTrc("Setting volume option 7M (no_atime_update) , which is (Access Time Update Enabled:) 
			in 10M/Tricky");
        my $val="true";
		if ($value eq "off") {
			$val="false";
		}else {
			$val="true";
		}
		eval {
        	$Server->volume_modify(vserver=>$VSERVER_NAME,
								   volume=>$vname,
							   	   'atime-update'=>$val);
    	};
    	if ($@) {
    			logTrc("##mica error setting 10Mode/Tricky volume option : $@.");
		}	
	 }


 
 }

## @name	::sparse_stat
## @summary	lookup or confirm a specific stat on sparse volume
## @arg		counter required
## @arg		val optional expected match
## @return	if above val specified confirm a match, else value of counter
## @example	for getting a value
##			my $stat = $dvol->sparse_stat("conn_active");
## 		for checking a value
##			$dvol->sparse_stat("conn_active", 1);
sub sparse_stat10M($$;$)
{
	my ($vol, $counter, $val) = @_;
	my $object = "sparse";
	my $instance = $vol->{name};
	my $key = "$object:$instance:$counter";

	logTrc("get sparse stat $key");

	my $ret = $vol->{filer}->stats({raw => 1, key => $key});
	my $value = $ret->{$object}{$instance}{$counter};

	if (!defined($val)) {
		return $value;
	}

	if ($val != $value) {
		logError("Value of $key was $value ".
			 "and not the expected value $val");
	}
}

## @name	:: sparse_stats_start
## @summary	Get stats for a sparse volume using the 'stats' command
## @arg		filer required
## @return	An ID string for use with "stats stop"
## @example	my $id = $vol->sparse_stats_start();
## 		my $stats = $vol->sparse_stats_stop($id);
sub sparse_stats_start10M($)
{
	my ($dvol) = @_;
	my ($object, $instance) = ("sparse", $dvol->{name});
	return $dvol->{filer}->stats_start("$object:$instance");
}

## @name	:: sparse_stats_stop
## @summary	Get stats for a sparse volume using the 'stats' command
## @arg		filer required
## @arg		id required
## @return	A hash ref containing sparse stats for the volume
## @example	my $id = $vol->sparse_stats_stop();
## 		my $stats = $vol->sparse_stats_stop($id);
sub sparse_stats_stop10M($$)
{
	my ($dvol, $id) = @_;
	my ($object, $instance) = ("sparse", $dvol->{name});
	my $stats = $dvol->{filer}->stats_stop($id);
	return $stats->{$object}{$instance};
}

## @name	::nrv_server_stat
## @summary	lookup a specific stat on nrv_server
## @arg		peer-sparse-volume required
## @arg		counter required
## @return	value of counter specified above
sub nrv_server_stat10M($$$)
{
	my ($svol, $dvol, $counter) = @_;
	my $object = "nrv";
	my $instance = "$dvol->{filer}{hostName}($dvol->{name})";
	my $key = "$object:$instance:$counter";
	my $ret = $svol->{filer}->stats({raw => 1, key => $key});
	my $value = $ret->{$object}{$instance}{$counter};
	return $value;
}

## @name	::qtree_list
## @summary	get list of qtree names for volume
## @return	returns list of qtrees
sub qtree_list10M($)
{
	my ($vol) = @_;
	my $filer = $vol->{filer};

	my @list;
	my $reply = $filer->filer_say("qtree status $vol->{name}", "^Volume");
	# Volume   Tree     Style Oplocks  Status
	# -------- -------- ----- -------- ---------
	# vol0              unix  enabled  normal
	# vol0     qt       unix  enabled  normal
	#
	my @lines = split(/[\r\n]+/, $reply);
	for (@lines[2..$#lines]) {
		next unless /$vol->{name}\s+(\w+)?(?:\s+\w+){3}/;
		push(@list, "/" . ($1 ? $1 : "-"));
	}
	logDbg("qtree list: \"" . join("\", \"", @list) . "\"");
	return @list;
}

# ------------------------------------------------------------------------

## @name	::setup_sparse_relation
## @summary	set up a sparse relation between two volumes , this between 
#           10Mode and 7Mode volumes. 
## @arg		mode required rod_remote, rod_snapvault, rod_loopback, pfs
## @arg		dest-filer required
## @arg		dest-volname optional
## @arg		opts optional {volSize=>, aggr=>, rate=>}
sub setup_sparse_relation10M($$$;$$)
{
	my ($svol, $mode, $dfiler, $dvolname, $opts) = @_;
	# mica - $dfiler is 7M/7G filer 
	$dvolname  = DFLT_DVOL unless defined($dvolname);
	my $sfiler = $svol->{filer};
	my $dvol;

	# fix generic "snapvault" mode
	$mode = DFLT_SV_MODE if $mode eq "rod_snapvault";

	logTrc("setup_sparse_relation with $mode"
	       . " from $svol->{name} to $dfiler->{hostName} : $dvolname");

	my $snap;
	if ($mode =~ /rod_remote|rod_loopback/ && !$opts->{noSnap}) {
		$snap = "RodSnap";
		$svol->snapshot($snap);
	}

	if ($mode eq "rod_remote") {

		$sfiler->filer_say10M(PRIVSETTEST .
			"rod enable secondary; " .
			"options snapvault.access all",
			"^\$");
		$dvol = $svol->rod_remote10M($snap, $dfiler, $dvolname, $opts)

	} elsif (sparse_mode_sv10M($mode)) {

		$sfiler->filer_say10M(PRIVSETTEST."options snapvault.access all", 
						  "^\$");
		$dfiler->rod_license($sfiler);
		$opts->{rate} = 64 unless defined($opts->{rate});
		$dvol = $svol->snapvault_restore($dfiler, $dvolname, $opts);

	} elsif ($mode eq "pfs") {
        # mica - this is the mapper 
		#$sfiler->filer_say10M("options flexcache.access all", "^\$");
		#$sfiler->filer_say10M("options flexcache.enable on", "^\$");
		logTrc("### mica entering pfs_map call ... ");
		$dfiler->license_flexcache();
		$dvol = $svol->pfs_map10M($dfiler, $dvolname, $opts);

	} elsif ($mode eq "pfs_loopback") {

		$dvol = $svol->pfs_map10M($svol->{filer}, $dvolname, $opts)

	} elsif ($mode eq "rod_loopback") {

		$dvol = $svol->rod_loopback($snap, $dvolname, $opts)

	} else {
		logError("unknown mode $mode");
	}

	$dvol->{mode} = $mode;
	return $dvol;
}

## @name	::complete_sparse_relation
## @summary	complete the sparse relation between two volumes
sub complete_sparse_relation10M($)
{
	my ($vol) = @_;
	my $mode = $vol->{mode};

	return unless defined($mode);

	logStep("completing sparse relationship ($mode)\n");

	if ($mode =~ /rod_remote|rod_loopback/) {
		$vol->rod_resume({waitfor=>1});
	} elsif (sparse_mode_sv10M($mode)) {
		$vol->rod_wait();
	}
	$vol->{mode} = undef;
}

# ------------------------------------------------------------------------
# ROD Specific Methods
#
# We support two restore methods, RoD and SnapVault.  The former is
# for internal and testing use only.  Both use the same set of sparse
# primitives.  We provide interfaces for both.
#
# We would like to evolve the rod interfaces to take account of the
# volume mode and call either the native RoD or Snapvault methods.

## @name	::rod_remote
## @summary	create a RoD volume from a source
## @arg		src-snap required
## @arg		dest-filer required
## @arg		dest-volname optional
## @arg		opts optional
sub rod_remote10M($$$;$$)
{
	my ($svol, $snap, $dfiler, $dvolname, $opts) = @_;

	my $sfiler	 = $svol->{filer};
	my $aggr	 = $opts->{aggr} || DFLT_AGGR;
	$dvolname	 = $dvolname || DFLT_DVOL;
	$opts->{volSize} = $opts->{volSize} || DFLT_VOL_SIZE;

	logTrc("rod remote volume $svol->{name} to $sfiler->{hostName}");

	if (vol_exists($dfiler->{conn}, $dvolname)) {
		vol_purge($dfiler->{conn}, $dvolname)
				or logError("purge vol failed: $@");
	}

	$dfiler->setup_aggr($aggr, $opts);

	$dfiler->filer_say(PRIVSETTEST
			   . "rod remote $sfiler->{hostName}"
			   . " $svol->{name} $snap"
			   . " $aggr $dvolname $opts->{size}",
			   "has completed");

	# we just mapped it, do not purge
	$opts->{purge} = 0;
	$opts->{aggrPurge} = 0;

	return sparse_volume10M($dfiler, $dvolname, $opts);
}

## @name	::rod_resume
## @summary	start/resume RoD transfer to a volume
## @arg		opts optional {phase=>-i|-d|-f, rate=>, waitfor=>1}
sub rod_resume10M($;$)
{
	my ($dvol, $opts) = @_;

	logTrc("rod resume volume $dvol->{name} $dvol->{mode}");

	my $cmd = "rod resume $dvol->{name} $opts->{phase}";
	$cmd .= " -k $opts->{rate}" if (defined($opts->{rate}));

	$dvol->{filer}->filer_say(PRIVSETTEST . $cmd,
			"Resuming Rapid Restore", "paused mode");

	if ($opts->{waitfor}) {
		rod_wait($dvol, $opts);
	}
}

## @name	::rod_wait
## @summary	wait for a specific RoD restore state
## @arg		opts optional {phase=>-i|-d|-f, timeout=>100}
sub rod_wait10M($;$)
{
	my ($dvol, $opts) = @_;
	my $timeout = 1200 unless defined($opts->{timeout});
	my $endtime = time() + $timeout;
	my $phase = ($opts->{phase} =~ /-f|-d|-i/) ? $opts->{phase} : "done";

	logTrc("wait for rod volume $dvol->{name} phase=$phase");

	logInfo("If RoD of other volumes are active on $dvol->{filer}{hostName}".
			", we could spin in here until timing out.... ");
	while (1) {
		my $reply = $dvol->{filer}->filer_say(PRIVSETTEST
					. "rod status", undef, "invalid");

		# bail if paused or no longer restoring
		last if ($reply =~ /Paused./ || $reply !~ /Restore currently/);

		logError("wait for rod phase=$phase timed out")
						if time() > $endtime;
		sleep(5);
	}

	if ($phase eq "done") {
		$dvol->{filer}->filer_say(PRIVSETTEST."rod status $dvol->{name}", 
					  undef, "Paused");
		$dvol->verify_status({State=>"online", 
				      Status=>"raid4|raid_dp"});
		$dvol->{mode} = undef;
		logTrc("$dvol->{name} is online and completely restored");
	} else {
		$dvol->verify_status({State=>"online", Status=>"sv-restoring"});
		logTrc("$dvol->{name} is online and paused ($phase)");
	}
}

## @name	::rod_pause
## @summary	pause RoD
sub rod_pause10M($)
{
	my ($dvol) = @_;

	logTrc("rod pause volume $dvol->{name} $dvol->{mode}");
	$dvol->{filer}->filer_say(PRIVSETTEST . "rod pause $dvol->{name}",
				  "^\$");
	$dvol->{filer}->filer_say(PRIVSETTEST . "rod status $dvol->{name}",
				  "Paused.");
}

## @name	::rod_abort
## @summary	abort RoD
sub rod_abort10M($)
{
	my ($dvol) = @_;

	logTrc("rod abort volume $dvol->{name} $dvol->{mode}");
	$dvol->{filer}->filer_say(PRIVSETTEST . "rod abort $dvol->{name}",
				  "^\$");
	$dvol->verify_status({State=>"offline", Status=>"needs check"});
}

## @name	::rod_status
## @summary	return status of RoD
## @arg		status optional expected match
sub rod_status10M($;$)
{
	my ($dvol, $status) = @_;

	if (sparse_mode_sv10M($dvol->{mode})) {
		$dvol->snapvault_verify_status($status);
		return;
	}

	return $dvol->{filer}->filer_say(PRIVSETTEST
			. "rod status $dvol->{name}", $status, "invalid");
}

## @name	::rod_reserve
## @summary	return reserve counts of RoD
sub rod_reserve10M($)
{
	my ($dvol) = @_;

	logTrc("rod reserve count for volume $dvol->{name}");

	my $s = $dvol->rod_status("Paused");
	my $line = (split(/\n/, $s))[0];	# first line
	return (split(' ', $line))[1];		# second token
}

## @name	::rod_kill
## @summary	abort RoD
sub rod_kill10M($)
{
	my ($dvol) = @_;

	if ($dvol->{mode} eq "rod_remote") {
		$dvol->rod_abort();
	} elsif (sparse_mode_sv($dvol->{mode})) {
		$dvol->snapvault_abandon();
	} else {
		logError("unsupported mode for rod kill: $dvol->{mode}");
	}
	$dvol->{mode} = undef;
	$dvol->verify_status({State=>"offline", Status=>"needs check"});
}

# ------------------------------------------------------------------------
# SnapVault Specific Methods
#
## @name	::snapvault_restore
## @summary	set up snapvault RoD relationship
## @arg		dest-filer required
## @arg		dest-volname required
## @arg		opts optional
sub snapvault_restore10M($$$;$)
{
	my ($svol, $dfiler, $dvolname, $opts) = @_;

	my $sfiler	 = $svol->{filer};

	logTrc("sv restore volume $svol->{name} to $sfiler->{hostName}");

	my %o = %$opts if $opts;

	$o{purge} = 1;
	$o{nomount} = 1;

	logTrc("create a new vol");
	my $dvol = sparse_volume($dfiler, $dvolname, \%o);

	# hardcoding this to the smallest value so that snapvault
	# restore won't be finished instantly while testing.

	my $cmd = "snapvault restore";
	$cmd .= " -f" unless ($opts->{noforce}); 
	$cmd .= " -S $sfiler->{hostName}:/vol/$svol->{name}";
	$cmd .= " -k $opts->{rate}" if (defined $opts->{rate});
	$cmd .= " /vol/$dvolname";

	my $must    = $opts->{must} || "Beginning Rapid Restore";
	my $mustnot = $opts->{mustnot} || "paused mode";

	$dfiler->filer_say($cmd, $must, $mustnot);

	$dvol->exportfs($opts);
	$dvol->mount() unless $opts->{nomount};
	return $dvol;
}

## @name	::snapvault_resume
## @summary	resume a snapvault restore to a volume
## @arg		src-vol required
sub snapvault_resume10M($$)
{
	my ($dvol, $svol) = @_;

	logTrc("sv resume volume $dvol->{name}");
	$dvol->{filer}->filer_say("snapvault restore -f"
			. " -S $svol->{filer}{hostName}:/vol/$svol->{name}"
			. " /vol/$dvol->{name}",
			"Resuming Rapid Restore", "paused mode");
}

## @name	::snapvault_pause
## @summary	pause snapvault ROD (snapvault abort)
sub snapvault_pause10M($)
{
	my ($dvol) = @_;

	logTrc("snapvault abort(pause) volume $dvol->{name}");
	$dvol->{filer}->filer_say("snapvault abort $dvol->{name}", "^\$");

	$dvol->snapvault_verify_status("Paused");
}

## @name	::snapvault_abandon
## @summary	abandon SnapVault RoD
sub snapvault_abandon10M($)
{
	my ($dvol) = @_;

	logTrc("sv hard abort volume $dvol->{name}");
	$dvol->{filer}->filer_say("snapvault abort -h -f $dvol->{name}");
	$dvol->verify_status({State=>"offline", Status=>"needs check"});
}

## @name	::snapvault_replicate
## @summary	replicate volume data using snapvault
## @arg		server-filer required
## @arg		server-volname optional
## @arg		opts optional {volSize=>.., aggr=>..}
sub snapvault_replicate10M($$;$$)
{
	my ($svol, $dfiler, $dvolname, $opts) = @_;

	# 's' is the source aka the client
	# 'd' is the destination aka the server
	# RoD populates: src is primary, dst is the secondary
	#
	$opts->{aggr} = DFLT_AGGR unless defined($opts->{aggr});
	$dvolname     = DFLT_SVOL unless defined($dvolname);
	my $sfiler    = $svol->{filer};	# aka client filer (rpl source)

	# XXX assign "" for whole volume replicate
	my @list = $svol->qtree_list();

	logTrc("replicate from $sfiler->{hostName}:$svol->{name} "
		. "to $dfiler->{hostName}:$dvolname");

	$sfiler->rod_license($dfiler);

	$sfiler->filer_say("options snapvault.access host=all", 
			   "^\$");
	$dfiler->filer_say("options snapvault.access host=all", 
			   "^\$");

	my %o = %$opts if $opts;
	$o{nomount} = 1;
	$o{purge} = 1 unless $opts->{nopurge};

	logTrc("setup a destination vol");
	my $dvol = sparse_volume10m($dfiler, $dvolname, \%o);
	snapsched_volume($dfiler->{conn}, $dvol->{name}, 0, 0, 0);
	snapreserve_volume($dfiler->{conn}, $dvol->{name}, 0);

	# XXX Old relations trip ASSERTs - even if the volume is new
	for (@list) {
		my $dst = ($_ eq "/-") ? QT_ROOT : $_;	# default qtree
		$dfiler->filer_say("snapvault stop -f /vol/$dvolname$dst");
	}
	for (@list) {
		my $dst = ($_ eq "/-") ? QT_ROOT : $_;	# default qtree

		logTrc("replicate /vol/$svol->{name}$_ to /vol/$dvolname$dst");
		$dfiler->filer_say("snapvault start"
			. " -w"	# wait 
			. " -S $sfiler->{hostName}:/vol/$svol->{name}$_"
			. " /vol/$dvolname$dst", "Transfer started", "aborted",
			$opts);
	}

	return $dvol;
}

## @name	::snapvault_status
## @summary	returns snapvault status and state for given volume
## @return	status and state of volume
sub snapvault_status10M($)
{
	my ($vol) = @_;
	my $filername = (split(/\./, $vol->{filer}->{hostName}))[0];

	use SnapVault qw(:status);
	my ($srcvol, $dstvol, $state, $lag, $status)
			= SnapVault::snapvault_status($vol->{filer}->{conn},
				"$filername:/vol/$vol->{name}");
	logError("error while retrieving snapvault status : $@") if ($@);

	return ($status, $state);
}

## @name	::snapvault_verify_status
## @summary	verify snapvault status
## @arg		regexp must-match pattern for status
## @arg		regexp optional must-match pattern for state
## @arg		opts optional notfatal => 1
## @return	returns success
sub snapvault_verify_status10M($$;$$)
{
        my ($vol, $match_status, $match_state, $opts) = @_;

	my ($status, $state) = $vol->snapvault_status();
	logTrc("snapvault status=$status, state=$state");

	my $statusOk = (!$match_status || $status =~ /$match_status/);
	my $stateOk  = (!$match_state || $state =~ /$match_state/);
	my $ok = $statusOk && $stateOk;

	return $ok
		if ($ok || $opts->{notfatal});

	logError("snapvault status failed: ".
		$stateOk ? "expected $match_status, got $status"
			 : "expected $match_state, got $state");
}

## @name	::snapvault_resync
## @summary	resync backup from a vol (on source)
## @arg		src-vol required backing up FROM this vol
sub snapvault_resync10M($$;$)
{
	my ($svol, $dvol, $opts) = @_;
	my $sfiler = $svol->{filer};	# backing up FROM
	my $dfiler = $dvol->{filer};	# backing up TO
	logTrc("snapvault resync $sfiler->{hostName} -> $dfiler->{hostName}");

	my %nopts = {};
	$nopts{COMPLETE} = 1 unless $opts->{bg};
	$nopts{TIMEOUT} = SV_TIMEOUT;
	my @list = $svol->qtree_list();	# assume qtrees for now
	for my $dst (@list) {
		next if $dst eq "/-"; # skip root since of svol since it was never 
							  # part of the snapvault configuration
		logDbg("resync $sfiler->{hostName}:/vol/$svol->{name}$dst"
			. " $dfiler->{hostName}:/vol/$dvol->{name}$dst");
		sv_resync($dfiler->{conn}, "/vol/$dvol->{name}$dst",
			SOURCE => "$sfiler->{hostName}:/vol/$svol->{name}$dst",
			%nopts);
		if ($@) {
			if (defined($opts->{exp_err})) {
				return $@;
			}
			logError("EVAL_ERROR=$@");
		}
	}
}

## @name	::snapvault_update
## @summary	run backup from a vol (on source)
## @arg		src-vol required
sub snapvault_update10M($$)
{
	my ($svol, $dvol) = @_;
	my $sfiler = $svol->{filer};	# backing up FROM
	my $dfiler = $dvol->{filer};	# backing up TO
	logTrc("snapvault update $sfiler->{hostName} -> $dfiler->{hostName}");

	my @list = $svol->qtree_list();	# assume qtrees for now
	for my $dst (@list) {
		next if $dst eq "/-"; # skip root since of svol since it was never 
							  # part of the snapvault configuration
		logDbg("update $sfiler->{hostName}:/vol/$svol->{name}$dst"
			. " $dfiler->{hostName}:/vol/$dvol->{name}$dst");
		sv_update($dfiler->{conn}, "/vol/$dvol->{name}$dst",
			COMPLETE => 1, TIMEOUT => SV_TIMEOUT)
			or logError("EVAL_ERROR=$@");
	}
}


# ------------------------------------------------------------------------
# Rod Loopback Specific Methods

## @name	::rod_loopback
## @summary	set up loopback RoD relationship
## @arg		src-snap required
## @arg		dest-volname required
## @arg		opts optional
sub rod_loopback10M($$$;$)
{
	my ($svol, $snap, $dvolname, $opts) = @_;

	my $filer     = $svol->{filer};
	$opts->{aggr} = $opts->{aggr} || DFLT_AGGR;

	logTrc("loopback volume $svol->{name} to $dvolname");
	if (vol_exists($filer->{conn}, $dvolname)) {
		vol_purge($filer->{conn}, $dvolname)
				or logError("purge vol failed: $@");
	}

	$filer->license('sv_ontap_pri');	# 'YAYCNCG'

	$filer->filer_say(PRIVSETTEST
			  . "rod loopback $svol->{name} $snap"
				. " $opts->{aggr} $dvolname",
			  "^\$");

	return sparse_volume10M($filer, $dvolname, $opts);
}

# ------------------------------------------------------------------------
# PFS Specific Methods

## @name	::pfs_map10M
## @summary	create a mapped PFS cache volume from a source, from 
sub pfs_map($$$;$)
{
	my ($svol, $dfiler, $dvolname, $opts) = @_ ;
	return pfs_map10M($svol, $dfiler, $dvolname, $opts);
}


## @name	::pfs_map10M
## @summary	create a mapped PFS cache volume from a source
## @arg		dest-filer required
## @arg		dest-volname required
## @arg		opts optional
sub pfs_map10M($$$;$)
{
	my ($svol, $dfiler, $dvolname, $opts) = @_;

	my $sfiler	 = $svol->{filer};
	my $aggr	 = $opts->{aggr} || DFLT_AGGR;
	$opts->{volSize} = $opts->{volSize} || DFLT_VOL_SIZE;
	my $vfiler_dst = $dfiler;

	logTrc("vol create volume $svol->{name} mapped to $sfiler->{hostName}");

	# do volume operations on the base vfiler
	$dfiler = $dfiler->{vfiler0} if (defined($dfiler->{vfiler0}));
		
    # mica - cleanup destvol if exists 
	if (vol_exists($dfiler->{conn}, $dvolname)) {
		vol_purge($dfiler->{conn}, $dvolname)
				or logError("purge vol failed: $@");
	}

    # mica - no 10M calls as dfiler is in 7M/7G
	$dfiler->setup_aggr($aggr, $opts);
	$dfiler->filer_say("vol create $dvolname $aggr $opts->{volSize}"
			   . " -S $sfiler->{dataIP}:$svol->{name}",
			   "has completed");

	if ($vfiler_dst->{vfiler0}) {
		logTrc("move $dvolname to vfiler $vfiler_dst->{vfilerName}");
		$dfiler->filer_say("vfiler move vfiler0 "
				   ."$vfiler_dst->{vfilerName} /vol/$dvolname");
	}

	# we just mapped vol on aggr, do not purge
	$opts->{purge} = 0;
	$opts->{aggrPurge} = 0;

	#return sparse_volume10M($vfiler_dst, $dvolname, $opts);
    return SPARSE::Resource::Volume::sparse_volume($vfiler_dst, $dvolname, $opts);
}

## @name	::pfs_unmap
## @summary	unmap and destroy a PFS cache volume
## @arg		dest-filer required
sub pfs_unmap10M($)
{
	my ($dvol) = @_;
	my $dfiler = $dvol->{filer};

	logTrc("vol destroy volume $dfiler->{hostName}:$dvol->{name}");
	$dfiler->filer_say("vol offline $dvol->{name}",
			   "Volume \'$dvol->{name}\' is now offline.");
	$dfiler->filer_say("vol destroy $dvol->{name} -f",
			   "Volume \'$dvol->{name}\' destroyed.");
}

## @name	::pfs_fstat
## @summary	pfs fstat file on pfs volume
## @description
##		The output format is a hashref indexed by the description
##		of the attribute (the words before the ':')
## @arg		volume required
## @arg		file_name required
## @arg		opts optional
## @return	attribute string
sub pfs_fstat10M($$;$)
{
	# XXX TODO REMOVE THIS CODE.
	#
	# XXX TODO need to change Daisuke's validate_mixed()
	# so that later we can substitute $options HASHREF instead
	# of the %options HASH to make things consistent
	my ($vol, $file_name, %options) =
		validate_mixed(@_,
			[
			 { isa => 'SPARSE::Resource::Volume' },
			 { type => SCALAR }
			],
			{
			 epoch_time=>{type=>SCALAR, optional=>1, default=>0}
			}
		);

	my $opts = \%options;

	my $filer = $vol->{filer};
	$file_name  = "/vol/$vol->{name}".$file_name;
	my $expected_text = "Filename: $file_name";

	logTrc("Running flexcache fstat $file_name equivalent on 10M filer");

	my $reply = $filer->filer_say10M("flexcache fstat $file_name", $expected_text);

	$reply =~ /Attributes:\s+/;
	$reply = $';

	# remove extraneous white space
	$reply =~ s/^\s+//gm;

	# convert to hash
	my @lines = split /^/m, $reply;
	my $irs = $/;
	$/ = "\n";
	chomp @lines; # remove trailing newlines
	$/ = "\r";
	chomp @lines; # remove trailing CR
	$/ = $irs;
	#print "TAKE2\n<@lines>\n";

	my %res = map { split /: / } @lines;

	if ($opts->{'epoch_time'}) {
		# small helper routine that will convert linux time into uint
		my $string_to_time = sub($) {
			my ($time_string) = @_;
			if (! defined($time_string)) {
				logError("undefined time string");
			}
			# get rid of annoying (+776553 uSec)-like linuxy part
			$time_string =~ s/\(.*\)//;
			logDbg("converting (( $time_string ))");
			if (!(defined($time_string = str2time($time_string)))) {
				logError("could not convert time string");;
			}
			logDbg("converted epoch time is $time_string");
			return $time_string;
		};
		$res{'Last access time'}
			= &$string_to_time($res{'Last access time'});
		$res{'Last modified time'}
			= &$string_to_time($res{'Last modified time'});
		$res{'Last change time'}
			= &$string_to_time($res{'Last change time'});
		$res{'Creation time'}
			= &$string_to_time($res{'Creation time'});

	}

	return \%res;
}

## @name	::pfs_eject
## @summary	pfs eject file on pfs volume
## @arg		file_name required
## @arg		options optional {force=>1}
sub pfs_eject10M($$;$)
{
	my ($vol, $file_name, $opts) = @_;

	my $filer = $vol->{filer};
	$file_name = "/vol/$vol->{name}".$file_name;
	my $expected_text = "$file_name: ejected";
	my $force = defined($opts->{force}) ? "-f" : "";

	logTrc("flexcache eject $file_name $force");

	$filer->filer_say("flexcache eject $file_name $force", $expected_text);
}

## @name	::pfs_refresh
## @summary	pfs refresh a volume
## @arg		opts optional ex: opts=>{cmd_opts=>"-a", timeout=>"10"}
sub pfs_refresh10M($;$$)
{
	my ($vol, $opts) = @_;

	logTrc("flexcache refresh $vol->{name}");

	my $timeout = defined($opts->{timeout}) ? $opts->{timeout} : 60;
	my $waitfor = defined($opts->{waitfor}) ? $opts->{waitfor} : 1;
	my $filer = $vol->{filer};
	my $expected_output  = "FlexCache refresh of mapped volume ".
		"$vol->{name} completed successfully";
	my $in_background = "Refreshing FlexCache mapped volume "
		."$vol->{name} in background";
	my $cmd = "flexcache refresh ";

	$cmd .= " $opts->{cmd}" if (defined($opts->{cmd}));
	$cmd .= " $vol->{name}";

	logDbg("command issued will be $cmd \n");

	my $output = $filer->filer_say($cmd, $in_background);
	if ($waitfor) {
		logTrc("refreshing in the background. ".
		       "waiting $timeout seconds to finish.");
		$filer->verify_log($expected_output, {timeout=>$timeout});
	}
}

## @name	::pfs_options
## @summary	set an option on a PFS cache volume
## @arg		dest-volume required
## @arg		opt-name required
## @arg		opt-value required
sub pfs_options10M($$$)
{
	my ($dvol, $optname, $optval) = @_;
	my $dfiler = $dvol->{filer};

	logTrc("set on pfs volume $dvol->{name}: $optname = $optval");

	$dfiler->filer_say("priv set -q advanced;"
		. " vol options $dvol->{name} $optname $optval", "^\$");
}

## @name	::pfs_status
## @summary	query status of PFS volume
## @arg		match optional expected match
sub pfs_status10M($;$)
{
	my ($dvol, $match) = @_;

	$dvol->{filer}->filer_say("priv set -q advanced;"
		. " vol status -v $dvol->{name}", $match);
}

## @name	::flexcache_stats
## @summary	query stats of FlexCache/PFS volume
## @arg		vol, options, match optional expected match

sub flexcache_stats10M($;$$)
{
	my $response;
	my ($vol, $options, $match) = @_;
	$response = $vol->{filer}->filer_say("flexcache stats $options", $match);
}

## @name	::flexcache_stat
## @summary	lookup or confirm a specific stat on pfs volume
## @arg		counter required
## @arg		val optional expected match
## @return	if above val specified confirm a match, else value of counter
## @example	for getting a value
##			my $stat = $dvol->flexcache_stat("trunc_blks_capacity");
## 		for checking a value
##			$dvol->flexcache_stat("trunc_blks_capacity", 1);
sub flexcache_stat10M($$;$)
{
	my ($vol, $counter, $val) = @_;
	my $object = "flexcache";
	my $instance = $vol->{name};
	my $key = "$object:$instance:$counter";

	logTrc("get flexcache stat $key");

	my $ret = $vol->{filer}->stats({raw => 1, key => $key});
	my $value = $ret->{$object}{$instance}{$counter};

	if (!defined($val)) {
		return $value;
	}

	if ($val != $value) {
		logError("Value of $key was $value ".
			 "and not the expected value $val");
	}
}

## @name	:: flexcache_stats_start
## @summary	Get stats for a PFS volume using the 'stats' command
## @arg		filer required
## @return	An ID string for use with "stats stop"
## @example	my $id = $vol->flexcache_stats_start();
## 		my $stats = $vol->flexcache_stats_stop($id);
sub flexcache_stats_start10M($)
{
	my ($dvol) = @_;
	my ($object, $instance) = ("flexcache", $dvol->{name});
	return $dvol->{filer}->stats_start("$object:$instance");
}

## @name	:: flexcache_stats_stop
## @summary	Get stats for a PFS volume using the 'stats' command
## @arg		filer required
## @arg		id required
## @return	A hash ref containing pfs stats for the volume
## @example	my $id = $vol->flexcache_stats_stop();
## 		my $stats = $vol->flexcache_stats_stop($id);
sub flexcache_stats_stop10M($$)
{
	my ($dvol, $id) = @_;
	my ($object, $instance) = ("flexcache", $dvol->{name});
	my $stats = $dvol->{filer}->stats_stop($id);
	return $stats->{$object}{$instance};
}

# ------------------------------------------------------------------------

## @name	::rod_treecompare
## @summary	Performs RoD src and dst treecompare
## @description
## @arg src_vol RoD source sparse volume object
## @arg %opt list of options:
## 		dst_vol - RoD dstination sparse volume object
##		during_rod - treecompare requested during RoD,
##					hence need to create snapshot
##		data_subtype - Indicates which subtype have been modified, and check
##						for those differences

sub rod_treecompare10M($%)
{
	my ($dst_vol, %opt) = validate_mixed(@_,
		[ { TYPE=>HASHREF } ]
	,	{
			src_vol => { TYPE=>HASHREF, optional=>0 }
		,	during_rod => { TYPE=>SCALAR, optional=>1, default=>0 }
		,	data_subtype => { TYPE=>SCALAR, optional=>1 }
		}
	);
	
	my $src_snpath = _rod_snapshot_name($opt{src_vol});
	my $dst_snpath;
	if ( $opt{during_rod} ) {
		my $dst_snapname = "dst_rod_snap";
		snapvault_snap_sched($dst_vol->{filer}->{conn},
							$dst_vol->{name},
							$dst_snapname,
							"1\@sun\@0")
			unless snapvault_snap_sched($dst_vol->{filer}->{conn},
										$dst_vol->{name},
										$dst_snapname);
		snapvault_snap_create($dst_vol->{filer}->{conn},
							$dst_vol->{name},
							$dst_snapname);

		snapvault_snap_unsched($dst_vol->{filer}->{conn},
							$dst_vol->{name},
							$dst_snapname);
	}
	$dst_snpath = _rod_snapshot_name($dst_vol);

	logTrc("Treecompare snapshots: Src $src_snpath, Dst $dst_snpath");
	# In case of 'modify' we want to skip list of changes from treecompare
	# that have been modified
	my @skip_list;
	push @skip_list, "ctime mismatch for .",
					"/TEST/lvl",
					"/TEST/level_dir/file_level"
		 if $opt{data_subtype} eq "all" ||
			$opt{data_subtype} eq "level_files"
			;
	my $rc = 0;
	eval {
		$rc = treecompare($dst_vol->{filer}->{conn}, $dst_snpath,
						$src_snpath,
						{filer_dst=>$opt{src_vol}->{filer}->{conn},
						skip=>\@skip_list});
	};
	croak("treecompare differences found: $@") unless $rc
}

sub _rod_snapshot_name10M
{
	my ($vol) = @_;

	my @snap_list = snapshot_list($vol->{filer}->{conn}, $vol->{name});
	croak("No snapshot on $vol->{filer}->{hostname}:$vol->{name}")
			unless $#snap_list >= 0;
	return "/vol/".$vol->{name}."/.snapshot/".$snap_list[0];
}

1;

__END__
