#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/main/lib/SPARSE/Util/Client.pm#1 $
#
# Copyright (c) 2002 Network Appliance, Inc.
# All rights reserved.

## @summary	Library that defines useful client-related routines
## @description
##             Client queries, stats
##
## @author dl-sparseeng

package SPARSE::Util::Client;

use strict;

use SPARSE::Util::Logger;
use SPARSE::Util::Utilities;
use SPARSE::Constants;
use POSIX;
use Errno;
use Tharn;
# ------------------------------------------------------------------------

use vars ('@ISA', '@EXPORT');
require Exporter;
@ISA	= qw(Exporter);
@EXPORT	= qw(
	     &clntProperty
	     &clntDiffFstatValues
	    );

## @name    ::clntProperty
## @summary Checks for certain attributes of the client machine
## @arg     msg required. Reason why we need the property.
## @arg     opts required. Hash ref of properties, value pairs
## @example
##	clntProperty("Because ...", { open_flag => O_EXCL });
sub clntProperty($$)
{
	my ($msg, $opts) = @_;

	my $bad_value = sub {
		my ($opts, $key) = @_;
		logError("unknown property value $key => $opts->{$key}");
	};
	my $bad_property_error = sub {
		my ($msg, $opts, $key) = @_;
		logError("Client failed to satisfy required property: "
			. "<$key> => $opts->{$key}: $msg");
	};
	my $bad_property_warn = sub {
		my ($msg, $opts, $key) = @_;
		logWarn("Client failed to satisfy property "
			. "$key => $opts->{$key}: $msg");
		exit();
	};

	my $osname;
	my $osrlse;
	uCommand("uname -s", undef, undef, 0, \$osname);
	uCommand("uname -r", undef, undef, 0, \$osrlse);
	my @osrlse = split(/\./, $osrlse);

	logTrc("clientProperty : $msg");
	foreach my $key (keys(%$opts)) {
		$key = lc($key);

		logDbg("property : $key => $opts->{$key}");
		if ($key eq 'open_flag') {
			if ($opts->{$key} ne "O_EXCL") {
				&$bad_value($opts, $key);
			}
			# SunOS has O_EXCL on opens
			&$bad_property_warn($msg, $opts, $key)
				unless ($osname =~ /sunos/i);

			next;
		}

		# tools/nfsv4_testing/v4t
		if ($key eq 'v4_t') {
			# SunOS 5.8 supports this tool
			&$bad_property_error($msg, $opts, $key)
				unless ($osname =~ /sunos/i
					&& $osrlse[0] >= 5 && $osrlse[1] >= 8);
			next;
		}

		if ($key eq 'nfsv4') {
			# SunOS 5.10 supports V4
			&$bad_property_error($msg, $opts, $key)
				unless ($osname =~ /sunos/i
					&& $osrlse[0] >= 5 && $osrlse[1] >= 10);
			next;
		}
		if ($key eq "readdirplus") {
			# only solaris currently issues READDIRPLUS
			&$bad_property_warn($msg, $opts, $key)
			  	unless ($osname =~ /sunos/i);
			next;
		}
		if ($key eq "chmod_flag") {
		  	# chmod isnt sending setattr for solaris
		  	&$bad_property_warn($msg, $opts, $key)
			  	unless ($osname =~ /linux/i
				       || ($osname =~ /sunos/i 
					  && $osrlse[0] eq 5 
					   && $osrlse[1] <= 9));
			next;
		}
			
		#.... more interesting properties go here

		logError("unknown property $key => $opts->{$key} ");
	}
}


## @name    ::clntDiffFstatValues
## @summary Diff 'pfs fstat' values - just the client visible ones
## @arg     val1 required hash ref 1
## @arg     val2 required hash ref 2
## @example
##  clntDiffFstatValues($svol->pfs_fstat(), $dvol->pfs_fstat());
sub clntDiffFstatValues($$)
{
	my ($val1, $val2) = @_;
	my @chkattrs = (
		"Type",
		"Mode",
		"Nlinks",
		"UID",
		"GID",
		"Size",
		"Blocks",
		"Fileid",
		"Last access time",
		"Last modified time",
		"Last change time",
		"Creation time",
		"Inode Generation #");

	for my $attr (@chkattrs) {
		if (!defined($val1->{$attr}) || !defined($val2->{$attr})) {
			logError("source or dest is missing attribute:$attr");
		}
		if ($val1->{$attr} ne $val2->{$attr}) {
			logError("discrepancy in fstat values for $attr:" .
				 "$val1->{$attr} and $val2->{$attr} differ");
		}
	}
}

# ------------------------------------------------------------------------
1;
# ------------------------------------------------------------------------

