# -*- perl -*-
# $Id: //depot/prod/test/main/lib/nfs/CommonParams.pm#7 $

package nfs::CommonParams;

## @summary
##  Common parameters used in NFS tests.
##
## @description
##  This sets up correct environment variables. Note that the
##  variables are coming out of main's name space.
##
## @usage
##  use nfs::CommonParams;
##
## @status      review
##
## @author      dl-nfsqa-only (NFS QA Sunnyvale)
## @notes       Created by EP Network Storage Performance Lab, Inc.
## @notes
##  Put the 'use nfs::CommonParams;' right after the BEGIN
##  block with the thpl script param statements to make sure
##  the the variables below are defined before other library
##  modules are used.

use strict;

#
# The following package declaration is required for these variables
# to be declared global in the main name space. It was done this
# way because 'use vars' doesn't support package names in the
# variable name, ex: $main::variable. Neither does the 'our'
# statement.
package main;

# Please keep the list in alphabetic order
use vars qw(
	    $AGGR_SIZE
	    $CACHE_AGGR_TYPE
        $CIFS_DOMAIN
        $CIFS_DOMAIN_ADMIN
        $CIFS_DOMAIN_ADMIN_PASSWD
	    $CIFS_DRIVE
	    $CIFS_PASSWD
	    $CIFS_SHARE
	    $CIFS_USER
	    $CLEANUP
	    $CLIENT
	    $CMDTIMEOUT
	    $EXTERN_SETUP
	    $FILER_MODE
	    $FILER
	    $FILERA
	    $FILERB
	    $FILER_AGGR
	    $FILER_CONN
	    $FILER_IP
	    $FILER_QTREE
	    $FILER_VOL
	    $FILER_VOL_MNT
	    $LARGE_AGGR
        $MOUNTPOINT
	    $NBLADE_START
	    $NBLADE_HOST
	    $NBLADE_DIR
        $NFS_V4_DELEGATIONS
        $NFS_V4_ID_DOMAIN
        $NIS_DOMAIN
        $NIS_DOMAIN_SERVER
	    $ONTAPDIR
        $ORIGIN_AGGR_TYPE
	    $PFILER
	    $TIMEOUT
	    $VOL_SIZE
	    $TRAD_SIZE
	    $FLEX_SIZE
	    $VOL_TYPE
	    $RAID_TYPE
	    $MOUNT_VERS
	    $CONTAINER
	    $SETUP_DONE
	    $WORM_VOL
	    $WORM_AGGR
	    $WORM_RETENTION
	    $WAFLIRON_TIMEOUT
	    $SKIP_SNAPLOCK_RETENTION_SETUP
	    $SLCC_STRICT
		$SRC_10MODE
		$FILER10M_AGGR
		$VSERVER_ROOTAGGR
		$VSERVER_NAME
		$VSERVER_ROOTVOL
		$VSERVER_ROOTVOLUME_SECURITY_STYLE
		$VSERVER_DLIF
		$VSERVER_DATAPORT
		$VSERVER_DATAIP
		$VSERVER_DATAMASK
		$VSERVER_AGGR_TYPE
		$VSERVER_AGGR_DISKS
	   );

# Change the name space back
package nfs::CommonParams;

use Tharn;
# param sets up the variables in main's name space
param('AGGR_SIZE',      '-default', 3);
# mica_

param('CACHE_AGGR_TYPE', '-default', undef);
param('CIFS_DOMAIN',    '-default', 'cifsqa.lab.netapp.com');
param('CIFS_DOMAIN_ADMIN', '-default', "$::CIFS_DOMAIN\\administrator");
param('CIFS_DOMAIN_ADMIN_PASSWD', '-default', 'Hello123');
param('CIFS_DRIVE',     '-default', '*');
param('CIFS_PASSWD',    '-default', 'cifspass');
param('CIFS_SHARE',     '-default', 'testshare');
param('CIFS_USER',      '-default', 'administrator');
param('CLEANUP',        '-default', 'test');
param('CLIENT',         '-default', 'localhost');
param('CMDTIMEOUT',     '-default', 120); # 2 mins
param('SETUP_DONE',     '-default', 'reuse'); #this will replace EXTERN_SETUP in the future
param('EXTERN_SETUP',   '-default', $::SETUP_DONE);
param('FILER_AGGR',     '-default', 'testaggr');
param('FILER_CONN',     '-default', '#rsh-socket');
param('FILER_IP',       '-default', undef);
param('FILER_QTREE',    '-default', 'filerqtree');
param('FILER_VOL_MNT',  '-default', undef);
param('LARGE_AGGR',     '-default', undef);
param('CIFS_DOMAIN',    '-default', 'cifsqa.lab.netapp.com');
param('MOUNTPOINT',     '-default', undef);
param('NBLADE_START',   '-default', undef());
param('NBLADE_HOST',    '-default', undef());
param('NBLADE_DIR',     '-default', undef());
param('NFS_V4_DELEGATIONS', '-default', 'on');
param('NFS_V4_ID_DOMAIN', '-default', 'lab.netapp.com');
param('NIS_DOMAIN',     '-default', 'lab.netapp.com');
param('NIS_DOMAIN_SERVER', '-default', '172.19.3.11');
param('ONTAPDIR',       '-default', '/rlse/mainN');
param('PFILER',         '-default', '');
param('TIMEOUT',        '-default', 600); # 10 mins
param('MOUNT_VERS',     '-default', undef); #could be 4
param('FLEX_SIZE',      '-default', '30m');
param('TRAD_SIZE',      '-default', 3);
param('WORM_RETENTION', '-default', '30s');
param('WORM_AGGR',      '-default', 'wormaggr');
param('WAFLIRON_TIMEOUT', '-default', 2400);
param('SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 0);
param('SLCC_STRICT',    '-default', 1);
param('FILER_MODE',     '-default', 'mono');
param('RAID_TYPE',      '-default', undef);
param('ORIGIN_AGGR_TYPE', '-default', undef);

# mica - 10Mode source volume , if present take val or else 0
param( 'SRC_10MODE'        , '-default', $ENV{SRC_10MODE} || 0); 
param( 'FILER10M_AGGR'     , '-default', $ENV{FILER10M_AGGR} || "testaggr");
param( 'VSERVER_ROOTAGGR'  , '-default', $ENV{VSERVER_ROOTAGGR} || "aggr0");
param( 'VSERVER_NAME'      , '-default', $ENV{VSERVER_NAME} || "pfs_vs1");
param( 'VSERVER_ROOTVOL'   , '-default', $ENV{VSERVER_ROOTVOL} || "pfs_vs1_rootvol");
param( 'VSERVER_ROOTVOLUME_SECURITY_STYLE' , '-default', $ENV{VSERVER_ROOTVOLUME_SECURITY_STYLE} || "mixed"); 
param( 'VSERVER_DLIF'      , '-default', $ENV{VSERVER_DLIF} || "dlif1");
param( 'VSERVER_DATAPORT'  , '-default', $ENV{VSERVER_DATAPORT} || "e0d");
param( 'VSERVER_DATAIP'    , '-default', $ENV{VSERVER_DATAIP} || "10.72.234.61");
param( 'VSERVER_DATAMASK'  , '-default', $ENV{VSERVER_DATAMASk} || "255.255.255.128");
param( 'VSERVER_AGGR_TYPE' , '-default', $ENV{VSERVER_AGGR_TYPE} || "flex");
param( 'VSERVER_AGGR_DISKS', '-default', $ENV{VSERVER_AGGR_DISKS} || "3");

use config_base;
use Filer::Version qw(get_wafl_version WV_SCRIMSHAW_MAX);
my $wv;

if ($::FILER_MODE eq 'mono') {
    param('FILER',      '-mesg', '  hostspec of filer');
} else {
    param('FILERA',     '-mesg', '  FILERA for cluster');
    param('FILERB',     '-mesg', '  FILERB for cluster');
    param('FILER',      '-set',  $::FILERA);
}

my $fh = host($::FILER);
eval {
	#mica - only check 7m7G filers
	if ($::SRC_10MODE) {
	} else {
    	$wv = get_wafl_version($fh);
	}
};
if ($@ and ! defined($ENV{"VOL_TYPE"}) and ! defined($::VOL_TYPE)) {
    logresult('CONFIG', "filer/sim is down or unreachable -- define \$VOL_TYPE to continue");
} elsif ($wv > WV_SCRIMSHAW_MAX) {
    param('VOL_TYPE',   '-default', 'flex');
} else {
    param('VOL_TYPE',   '-default', 'trad');
    param('SKIP_SNAPLOCK_RETENTION_SETUP', '-default', 1);
}

if ($::VOL_TYPE eq 'flex') {
    param('VOL_SIZE',   '-default', $::FLEX_SIZE);
    param('FILER_VOL',  '-default', 'testvol_flex');
    param('CONTAINER',  '-default', $::FILER_AGGR);
    param('WORM_VOL',   '-default', 'testvol_worm_flex');
} else {
    param('VOL_SIZE',   '-default', $::TRAD_SIZE);
    param('FILER_VOL',  '-default', 'testvol_trad');
    param('CONTAINER',  '-default', $::FILER_VOL);
    param('WORM_VOL',   '-default', 'testvol_worm_trad');
}

1;

