## $Id: //depot/prod/test/Rsierranevada/lib/SSQA/DPHelper.pm#2 $
package SSQA::DPHelper;
use strict;
use Exporter;
use Tharn;
use vars ('@ISA','@EXPORT');

use Aggregate qw(aggr_create aggr_exists);
use filer qw(filer_date_get filer_vol_option_set);
use Filer::Version qw(get_ontap_version version_compare);
use CLI::FileIO qw(rdfile wrfile);
use Data::Dumper;
use Dense qw (dense_is_licensed dense_set_license dense_start dense_toggle);
use dp_limits qw(
		filer_vol_compress_start
		get_limits_dump 
		get_curr_ndmp_sessions 
		get_curr_dump_sessions 
		get_curr_vsm_xfers_dst 
		get_curr_qsm_xfers_dst 
		get_curr_ssm_xfers_dst
		get_curr_sm_checker_sessions
		get_curr_sm_active_endpoints
		);
use ExportFS qw(export_path);
use mount qw(mount);
use NACL::APISet;
use QtreeCmds qw(qtreeCreate qtreePresent);
use SayRetry qw( say_retry );
use SnapCmds qw(snapsched_volume);
use SnapMirror qw(
		snapmirror_destinations 
		snapmirror_status_vol 
		snapmirror_initialize 
		snapmirror_enable 
		snapmirror_update
		);
use VFILER::Vfiler qw(
		vfiler_create 
		vfiler_add 
		vfiler_options_set 
		vfiler_status 
		vfiler_limit 
		vfiler_limit_hard_limit 
		vfiler_limit_set
		vfiler_run_vfiler_exportfs
		);
use Volume qw(vol_exists vol_create vol_create_flex vol_restrict);
use VolCmds qw(volRestrict);
use Zapi;
use ZapiTest;


@ISA = qw(Exporter);
@EXPORT = qw(	
		add_data_to_vols
		add_data_to_vols_cmode
		add_vfilers_qtrees
		add_vfilers_vols
		connect_ssh_persist
		create_vols_src
		create_vols_dst
		create_qtrees_src
		create_qtrees_dst
		create_sm_job
		create_sm_job_zapi
		create_vfilers_src
		create_vfilers_dst
		create_vfilers_qtree_roots
		create_vfilers_qtrees_src
		create_vfilers_qtrees_dst
		filer_aggr_exists
		filer_vol_exists
		filer_service_is_licensed
		sis_has_fingerprint
		dump_zigzag_update
		get_node_version
		license_snapmirror_cmode
		ndmp_zigzag_update
		snapmirror_initialize_destination_vfiler
		snapmirror_initialize_destination
		snapmirror_create_cmode
		snapmirror_update_cmode
		snapmirror_wait_for_state
		snapmirror_zigzag_update_conf_file
		snapmirror_zigzag_update
		snapmirror_zigzag_update_cmode
		snapmirror_zigzag_update_cmode_cron
		snapmirror_zigzag_check_cmode
             );

sub connect_ssh_persist ($$) {
    my $id=shift;
    my $node=shift;
    use Connectrec;

        my $conn;
        my $connection=0;
        my $conn_failure_count=0;
        while (!$connection) {
            eval {
		$conn = Connectrec->new(template=>"ssh-persist",host=>host($node),id=>$id);
            };

            if ($@) {
                $conn_failure_count++;
                logresult("WARN", "Connection failed ($conn_failure_count times): $@");
                if($conn_failure_count == 40) {
                    logresult("FATAL", "Too many connection failures");
                }
                # Can't have more than 10 ssh sessions in 10 seconds so
                # randomly sleep so as to not start at same time
		srand($id);
                my $sleeptime = int(rand(60)+10);
		logcomment("Sleeping $sleeptime seconds");
                sleep $sleeptime;
            } else {
		#verify connection is good
		eval {
		    say($conn, "version");
		};
		if ($@) {
		    $conn_failure_count++;
		    logresult("WARN", "Connection failed ($conn_failure_count times): $@");
		    if($conn_failure_count == 40) {
			logresult("FATAL", "Too many connection failures");
		    }
		    # Can't have more than 10 ssh sessions in 10 seconds so
		    # randomly sleep so as to not start at same time
		    srand($id);
		    my $sleeptime = int(rand(60)+10);
		    logcomment("Sleeping $sleeptime seconds");
		    sleep $sleeptime;
		} else {
		    $connection++;
		}
            }
        }


    logcomment("Returning connection id $id");
    return $conn;
}

sub create_sm_job_zapi ($%) {


    my ($filer, %args) = @_;
    my $zapi_user = $args{'zapi_user'} || "admin";
    my $zapi_password = $args{'zapi_password'} || "netapp1!";
    my $change_value = $args{'change_value'} || die "change_value not defined";
    my $job_name = $args{'job_name'} || die "job_name not defined";
    my $server = new ZapiTest($filer, $zapi_user, $zapi_password);
    $server->set_legacy_http_conn(0);
    my $o;

    #my $job_created=0;
    #my $date;
    #my (undef, undef, $date_out) = say ($fc, "set diag -confirmations off; system node date show -node local");
    #foreach my $line (split(/\n/,$date_out)) {
    #    next if $line =~ /UTC/;
    #    next if $line !~ /\//;
    #    if ( $line =~ /Date [Aa]nd Time/) {
    #        $date = $line;
    #        $date =~ s#.*Date [Aa]nd Time: \d+/\d+/\d+ (.*)#$1#;
    #    }
    #}

    my $date = `date`;
logcomment("date is $date");
    chomp($date);
    my $min = $date;
    $min =~ s/.*:(\d+):.*/$1/;
    $min =~ s/^0//;
    my $hour = $date;
    $hour =~ s/.* (\d+):.*:.*/$1/;
    #$hour =~ s/(\d+):.*:.*/$1/;
    $hour =~ s/^0//;
    my $new_min = $min+$change_value;
    while($new_min >= 60) {
        $new_min -= 60;
        $hour++;
    }
    if ($hour >= 24) {$hour -= 24;}

    # Create job if needed
    logcomment("Checking for $job_name cron job entry");
    logcomment("sending ontapi $filer $zapi_user $zapi_password job-schedule-cron-get job-schedule-name $job_name");
    $o = $server->Zapi::invoke_api(
	'job-schedule-cron-get',
	'job-schedule-name', "$job_name",
	);

    my $reason = $o->results_reason();
    my $job_exists=1;
    if( $o->attr_get("status") ne "passed") {
	if ($reason eq "entry doesn't exist") {
	    $job_exists=0;
	} else {
	    die("Zapi request failed ($reason).");
	}
    }

    if(!$job_exists) {
	logcomment("Creating $job_name cron job entry at $hour:$new_min");
	$o = $server->Zapi::invoke_api(
	    'job-schedule-cron-create',
	    'job-schedule-name', "$job_name",
	    'job-schedule-cron-hour', "$hour",
	    'job-schedule-cron-minute', "$new_min",
	    );
    } else {
	logcomment("modifying $job_name cron job entry at $hour:$new_min");
	logcomment("sending ontapi $filer $zapi_user $zapi_password job-schedule-cron-modify job-schedule-name $job_name job-schedule-cron-hour $hour job-schedule-cron-minute $new_min");
	$o = $server->Zapi::invoke_api(
	    'job-schedule-cron-modify',
	    'job-schedule-name', "$job_name",
	    'job-schedule-cron-hour', "$hour",
	    'job-schedule-cron-minute', "$new_min",
	    );
    }


    $reason = $o->results_reason();
    if( $o->attr_get("status") eq "passed") {
	return 1;
    } else {
	warn("Zapi request failed to return valid XML (reason: $reason).");
	return 0;
    }


#    $o = $server->Zapi::invoke_api(
#	'job-schedule-cron-get',
#	'job-schedule-name', "$job_name",
#	);
#
#    # Check job was created
#    my $job_attrs = $o->child_get("attributes");
#    my $job_info = $job_attrs->child_get("job-schedule-cron-info");
#    logcomment("job_info is $job_info");
#    
#    foreach my $key (keys %$job_info) {
#    	logcomment("key is $key");
#    	logcomment("value is " . $$job_info{$key});
#	if ($$job_info{$key} =~ /ARRAY/) {
#	    logcomment("array is @{$$job_info{$key}}");
#	    for(my $x=0; $x < @{$$job_info{$key}};$x++) {
#	        logcomment("array value is $$job_info{$key}[$x]");
#		foreach my $arraykey (keys %{$$job_info{$key}[$x]}) {
#		    logcomment("arraykey is $arraykey");
#		    logcomment("arraykeyvalue is " . $$job_info{$key}[$x]{$arraykey});
#		}
#	    }
#	}
#    }
#    my $job_hour = $$job_info{'job-schedule-cron-info'}[0]{'job-schedule-cron-hour'}[0]{'cron-hour'};
#    my $job_min = $$job_info{'job-schedule-cron-info'}[0]{'job-schedule-cron-minute'}[0]{'cron-minute'};
#    #my $name = $job_info->child_get_string("job-schedule-name");
#
#    logcomment("job_hour is $job_hour, job_min is $job_min, name is 1 $name 2 $name2 3 $name3 4 $name4");
#    my $all_checks=0;
#    my $job_created=0;
#    if ( $name =~ /$job_name/) {
#	    $all_checks++;
#    }
#    if ($job_hour =~ /$hour/) {
#	    $all_checks++;
#    }
#    if ($job_min =~ /$new_min/) {
#	    $all_checks++;
#    }
#
#    if($all_checks == 3) {
#	$job_created = 1;
#    }
#
    #return $job_created;
}

sub create_sm_job ($%) {
    my ($fc, %args) = @_;
    my $node = $args{'node'} || die "host not defined";
    my $change_value = $args{'change_value'} || die "change_value not defined";
    my $job_name = $args{'job_name'} || die "job_name not defined";

    my $job_created=0;
    my $date;
    my (undef, undef, $date_out) = say ($fc, "set diag -confirmations off; system node date show -node local");
    foreach my $line (split(/\n/,$date_out)) {
        next if $line =~ /UTC/;
        next if $line !~ /\//;
        if ( $line =~ /Date [Aa]nd Time/) {
            $date = $line;
            $date =~ s#.*Date [Aa]nd Time: \d+/\d+/\d+ (.*)#$1#;
        }
    }

    my $min = $date;
    $min =~ s/.*:(\d+):.*/$1/;
    $min =~ s/^0//;
    my $hour = $date;
    $hour =~ s/(\d+):.*:.*/$1/;
    $hour =~ s/^0//;
    my $new_min = $min+$change_value;
    while($new_min >= 60) {
        $new_min -= 60;
        $hour++;
    }
    if ($hour >= 24) {$hour -= 24;}

    # Create job if needed
    my (undef, undef, $job_show) = say ($fc, "job schedule cron show -name $job_name", -timeout, 600);
    my $job_exists=1;
    foreach my $line (split(/\n/, $job_show)) {
	if ( $line =~ /no entries/) {
	    $job_exists=0;
	}
    }

    if(!$job_exists) {
	say ($fc, "job schedule cron create -name $job_name -hour $hour -min $new_min", -timeout, 600);
    } else {
	say ($fc, "set diag -confirmations off; job schedule cron modify -name $job_name -hour $hour -min $new_min", -timeout, 3600);
    }

    # Check job was created
    my $all_checks=0;
    (undef, undef, $job_show) = say ($fc, "job schedule cron show -name $job_name", -timeout, 600);
    foreach my $line (split(/\n/, $job_show)) {
	if ( $line =~ /Name: $job_name/) {
	    $all_checks++;
	} elsif ($line =~ /Hour: $hour/) {
	    $all_checks++;
	} elsif ($line =~ /Minute: $new_min/) {
	    $all_checks++;
	}
    }

    if($all_checks == 3) {
	$job_created = 1;
    }

    return $job_created;
}

sub dump_zigzag_update ($%) {
    my ($fc, %args) = @_;
    my $max_xfers = $args{'max_xfers'} || die "max_xfers not defined";
    my $vols = $args{'vols'} || die "Missing vols";
    my $dump_level = $args{'dump_level'} || "0";
    my $dump_device = $args{'dump_device'} || "null";
    my $dump_wait = defined($args{'dump_wait'}) ? $args{'dump_wait'} : 5;
    my $zigzag = defined($args{'zigzag'}) ? $args{'zigzag'} : 1;
    my @vols = @$vols;

    my @percentages;
    if($zigzag){ 
	@percentages = qw(.20 .40 .60 .80 1); 
    } else {
        @percentages = qw(1);
    }

    my @subtests=();
    
    foreach my $percentage (@percentages) {
	my $print_percentage = $percentage;
	$print_percentage =~ s/\.//;
	if($print_percentage == 1) {$print_percentage = 100;}
	my $xfer_limit = int($max_xfers*$percentage);
	if($xfer_limit == 0) {$xfer_limit = 1 ;}

	my $volume_to_update = 0;
	for(my $curr_dump_sessions = get_curr_dump_sessions($fc);
	    $curr_dump_sessions < $xfer_limit; 
	    $curr_dump_sessions = get_curr_dump_sessions($fc)) {

	    logcomment("Current dump sessions is $curr_dump_sessions");	
	    logcomment("Increasing transfers up to " . $xfer_limit . " (${print_percentage}% of max)");
	    push (@subtests, subtest sub {
		# my $logname = $vols[$volume_to_update];
		# logcomment("start logname is $logname");
		# $logname =~ s/.*\/(\w+)$/$1/; # Remove path
		# logcomment("end logname is $logname");

		# Check to make sure a dump isn't already in progress on the volume
		my $found_path = 0;
		my @dump_info = get_limits_dump($fc);
		logcomment('Dump info is: ' . Data::Dumper->Dump(\@dump_info));
		for(my $x=0;$x<@dump_info;$x++) {
		    logcomment("path is ". $dump_info[$x]{'path'} ."and vol is ". $vols[$volume_to_update]);
		    if ($dump_info[$x]{'path'} eq $vols[$volume_to_update]) {
		        $found_path++;
		    }
		}
		eval {
		    if (!$found_path) {
			my $dump_start_time = time;
			say($fc, "dump ${dump_level}f $dump_device /vol/" . $vols[$volume_to_update], -noprompt, -timeout => 3600, -wait => '[DUMP IS DONE|DUMP: Deleting|Too many active dumps]');
			my $dump_duration = time - $dump_start_time;
			logcomment("Dump of $vols[$volume_to_update] took $dump_duration seconds");
		    }
		};
		if ($@) {
		    if ($@ =~ /busy/) {
		        logresult("WARN", "$@");
		     } else {
		        logresult("FATAL", "Error is $@");
		    }
		}
	    }, -bg, -runid, "Dump_update_${volume_to_update}");

	    logcomment("Sleeping $dump_wait ...");
	    sleep $dump_wait;

	    if ($volume_to_update == $max_xfers) {
		$volume_to_update=0;
	    } else {
		$volume_to_update++;
	    }

	}

	foreach my $t (@subtests) {
	    sleep 1 while ($t->isAlive());
	}

	if ($print_percentage != 100 && $zigzag == 1) {
	    # Reduce transfers by 10%
	    my $decreased_limit=int($max_xfers*($percentage - .10));
	    logcomment("decreased limit is $decreased_limit");
	    while ( (get_curr_dump_sessions($fc)) > $decreased_limit ) {
		logcomment("Reducing transfers back down below $decreased_limit (".($print_percentage-10)."% of max)");
		sleep 5;
	    }
	}
    }
}

sub sis_has_fingerprint ($$) {
    my $fc = shift;
    my $vol_path = shift;

    if((not defined($vol_path)) or ($vol_path =~ /^$/)){
	return 0;
    }
    eval {
	say($fc, "priv set test\; sis ls $vol_path", '-wait' => "fingerprint");
    };
    if($@){
	return 0;
    } else {
        return 1;
    }
}

sub snapmirror_wait_for_state($$$){
    my $apiset = shift;
    my $location = shift;
    my $state = shift;
    my $response;
    my $sm_state;
    my $output_ref;
 
    do {
        sleep 5;
        eval {
            $response = $apiset->snapmirror_get_status();
        };
        if(not(defined($response) and ($response->{'execution_status'} eq 'passed'))){
            return 0;
        }
        $output_ref = $response->get_parsed_output();

        my @sm_statii = @{$output_ref->[0]{'snapmirror-status'}[0]{'snapmirror-status-info'}};

        $sm_state = _get_sm_state_for_location(\@sm_statii, $location);

        if(not defined($sm_state)){
            return 0;
        }
    } while($sm_state !~ /$state/);
    return 1;
}

sub snapmirror_zigzag_update_conf_file ($%) {
    my ($fc, %args) = @_;
    my $max_xfers = $args{'max_xfers'} || die "max_xfers not defined";
    my $sm_info = $args{'sm_info'} || die "sm_info not defined";
    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $sm_args = $args{'sm_args'} || "-";
    my $sm_schedule = $args{'sm_schedule'} || "";
    my $vfiler_volname_modifier = $args{'vfiler_volname_modifier'} || '$vfiler_volname = $vfiler_volname';
    my %vfiler_sm_info = %$sm_info;
    my $snapmirror_conf_file = "/etc/snapmirror.conf";

    foreach my $percentage qw(.20 .10 .40 .30 .60 .50 .80 .70 1) {
	my $print_percentage = $percentage;
	$print_percentage =~ s/\.//;
	if($print_percentage == 1) {$print_percentage = 100;}
	my $xfer_limit = int($max_xfers*$percentage);
	if($xfer_limit == 0) {$xfer_limit = 1 ;}

	SM_UPDATE:
	if ($sm_type !~ /ssm/i) {
	    # Get a time 2 minutes from now to kick off the snapmirror updates
	    my $min = filer_date_get($fc);
	    $min =~ s/.*:(\d\d):.*/$1/;
	    $min =~ s/^0//;
	    my $new_min = $min+2;
	    if ($new_min >= 60) {$new_min -= 60;}
	    $sm_schedule = "$new_min * * *";
	}

	logcomment("Increasing transfers up to " . $xfer_limit . " (${print_percentage}% of max)");

	my $xfer_counter = 0;
	# Remember that 'vfiler' can mean just 'filer' here
	foreach my $vfiler (sort keys %vfiler_sm_info) {
	    my @snapmirror_conf_lines = ();
	    for(my $x=0; $x < @{$vfiler_sm_info{$vfiler}}; $x++) {
		if ($vfiler_sm_info{$vfiler}[0]{'source_path'} =~ /-vf/) {
		    my $vfiler_volname = $vfiler;
		    eval $vfiler_volname_modifier; 
		    $snapmirror_conf_file = "/vol/${vfiler_volname}/etc/snapmirror.conf";
		}
		if(defined($vfiler_sm_info{$vfiler}[$x]{'arguments'})) {
		    $sm_args = $vfiler_sm_info{$vfiler}[$x]{'arguments'};
		}
		if(defined($vfiler_sm_info{$vfiler}[$x]{'sm_schedule'})) {
		    $sm_schedule = $vfiler_sm_info{$vfiler}[$x]{'sm_schedule'};
		}
		if ($xfer_counter >= $xfer_limit) {next;}
		$vfiler_sm_info{$vfiler}[$x]{'snapmirror_conf_orig'} = rdfile($fc, $snapmirror_conf_file);
		push(@snapmirror_conf_lines, $vfiler_sm_info{$vfiler}[$x]{'source_path'}." ".$vfiler_sm_info{$vfiler}[$x]{'destination_path'}." ".$sm_args." ".$sm_schedule);
		$xfer_counter++;
	    }

	    if (@snapmirror_conf_lines >=1) {
		wrfile($fc, $snapmirror_conf_file,join("\n", @snapmirror_conf_lines))
			or die("Error creating $snapmirror_conf_file file: $@");
	    }
	    if ($xfer_counter >= $xfer_limit) {last;}
	}

	my $current_xfers=0;
	my $start_time = time();
	my $wait_time = 300;
	logcomment("time is $start_time");
	do {
	    logcomment("Target $sm_type Transfers is $xfer_limit");
	    $current_xfers = get_current_xfers($fc, "$sm_type");
	    logcomment("Current $sm_type Transfers is $current_xfers");
	    sleep 5;
	    if(time() > ($start_time + $wait_time)) {
	    	logcomment("Exceeded wait time.  Restarting Updates");
		goto SM_UPDATE;
	    }
	} until ($current_xfers == $xfer_limit);
    }

    # Put snapmirror.conf back to original state
    my $is_vfiler=1;
    foreach my $vfiler (sort keys %vfiler_sm_info) {
	if ($vfiler =~ /-vf/) {
	    $snapmirror_conf_file = "/vol/${vfiler}/etc/snapmirror.conf";
	} else {
	    # Not a vfiler
	    $is_vfiler=0;
	}
	for(my $x=0; $x < @{$vfiler_sm_info{$vfiler}}; $x++) {
	    my $orig_file = "";
	    if(defined($vfiler_sm_info{$vfiler}[$x]{'snapmirror_conf_orig'})) {
		$orig_file = $vfiler_sm_info{$vfiler}[$x]{'snapmirror_conf_orig'};
	    }
	    wrfile($fc, $snapmirror_conf_file,"$orig_file")
		    or die("Error creating snapmirror.conf file: $@");
	    if (!$is_vfiler) {last;}
	}
    }
}

sub get_node_version ($) {
    my $fc = shift;
    my $version = "UNKNOWN";
    my (undef, undef, $node_version) = say ($fc, "run local version", -timeout, 300);
    if ( $node_version =~ /rollingrock/i) {
	$version = "rollingrock";
    } elsif ($node_version =~ /boilermaker/i) {
	$version = "boilermaker";
    } else {
	$version = "unknown";
    }
    return $version;
}

sub license_snapmirror_cmode ($) {
    my $fc = shift;
    my (undef, undef, $license_check) = say ($fc, "license show -feature SnapMirror_DP", -timeout, 300);
    foreach my $line (split(/\n/,$license_check)) {
	if ( $line =~ /no entries/) {
	    say ($fc, "license add -license-code FEDPEKPJYBFDAA");
	}
    }
}

sub get_current_xfers ($$) {
    my $fc = shift;
    my $sm_type = shift;
    my $current_xfers = 0;

    if ($sm_type =~ /vsm/i) {
	$current_xfers = get_curr_vsm_xfers_dst($fc);
    } elsif ($sm_type =~ /qsm/i) {
	$current_xfers = get_curr_qsm_xfers_dst($fc);
    } elsif ($sm_type =~ /[rs]sm/i) {
	$current_xfers = get_curr_ssm_xfers_dst($fc);
    } else {
    	logresult("CONFIG", "Invalid SM Type");
    }
    return $current_xfers;
}

sub ndmp_zigzag_update ($%) {
    my ($fc, %args) = @_;
    my $filer = $args{'host'} || die "host not defined";
    my $max_xfers = $args{'max_xfers'} || die "max_xfers not defined";
    my $ndmp_info = $args{'ndmp_info'} || logcomment "vfiler_info not defined";
    my $ndmp_auth = $args{'ndmp_auth'} || logcomment "vfiler_info not defined";
    my %filer_ndmp_info = %$ndmp_info;
    my $fh = host($filer);
    
    my $ndmp_debug;
    if(defined($args{'debug'})){
        $ndmp_debug = '-d';
    }

    my $zigzag = 1;
    if(defined($args{'zigzag'})) {
    	$zigzag=$args{'zigzag'};
    }
    my @subtests=();
    foreach my $percentage qw(.20 .40 .60 .80 1) {
	my $print_percentage = $percentage;
	$print_percentage =~ s/\.//;
	if($print_percentage == 1) {$print_percentage = 100;}
	my $xfer_limit = int($max_xfers*$percentage);
	if($xfer_limit == 0) {$xfer_limit = 1 ;}

	my $volume_to_update = 0;
	while ( (get_curr_ndmp_sessions($fc)) < $xfer_limit ) {
	    logcomment("Increasing transfers up to " . $xfer_limit . " (${print_percentage}% of max)");

	    push (@subtests, subtest sub {
		my $logname = $filer_ndmp_info{$filer}[$volume_to_update]{'destination_path'};
		$logname =~ s/.*\/(\w+)$/$1/; # Remove qtree path
		$logname =~ s/.*:(\w+)$/$1/; # Remove volume path
		subtest sub {
		    eval {
			# Run ndmpcopy
			say_retry($fc, 
				"ndmpcopy -sa $ndmp_auth $ndmp_debug $filer_ndmp_info{$filer}[$volume_to_update]{'source_path'} $filer_ndmp_info{$filer}[$volume_to_update]{'destination_path'}", 
				"Transfer successful", 
				-timeout => 3600,
				-retry => 3,
				-iter_sleep => 30);
			};
		    if ($@) {
			# Due high CPU, there's an edge case where the script could try and 
			# exceed the max allowable transfers.
			if ($@ =~ /busy|Too many active dumps/) {
			    logresult("WARN", "$@");
			} else {
			    logresult("FATAL", "Error is $@");
			}
		    }
		}, -runid, $logname
	    }, -bg, -runid, "NDMP_copy");
	    sleep 20;

	    if ($volume_to_update == $max_xfers) {
		$volume_to_update=0;
	    } else {
		$volume_to_update++;
	    }

	}

	foreach my $t (@subtests) {
	    sleep 1 while ($t->isAlive());
	}

	if ($print_percentage != 100 && $zigzag == 1) {
	    # Reduce transfers by 10%
	    my $decreased_limit=int($max_xfers*($percentage - .10));
	    logcomment("decreased limit is $decreased_limit");
	    while ( (get_curr_ndmp_sessions($fc)) > $decreased_limit ) {
		logcomment("Reducing transfers back down below $decreased_limit (".($print_percentage-10)."% of max)");
		sleep 5;
	    }
	}
    }
}

sub snapmirror_zigzag_update ($%) {
    my ($fc, %args) = @_;
    my $filer = $args{'host'} || die "host not defined";
    my $max_xfers = $args{'max_xfers'} || die "max_xfers not defined";
    my $sm_info = $args{'sm_info'} || logcomment "vfiler_info not defined";
    my $sm_type = $args{'sm_type'} || die"sm_type not defined";
    my $kbs = $args{'kbs'} || '';
    my %filer_sm_info = %$sm_info;
    my $fh = host($filer);

    my $zigzag = 1;
    if(defined($args{'zigzag'})) {
    	$zigzag=$args{'zigzag'};
    }

    my @percentages;
    if($zigzag == 0){
	@percentages = ( 1 );
    } else {
	@percentages = qw(.20 .40 .60 .80 1);
    }
   
    my @subtests=();
    foreach my $percentage ( @percentages ) {
	my $print_percentage = $percentage;
	$print_percentage =~ s/\.//;
	if($print_percentage == 1) {$print_percentage = 100;}
	my $xfer_limit = int($max_xfers*$percentage);
	if($xfer_limit == 0) {$xfer_limit = 1 ;}

	my $volume_to_update = 0;
	while ( (get_current_xfers($fc, "$sm_type")) < $xfer_limit ) {
	    logcomment("Increasing transfers up to " . $xfer_limit . " (${print_percentage}% of max)");

		my $logname = $filer_sm_info{$filer}[$volume_to_update]{'destination_path'};
		$logname =~ s/.*\/(\w+)$/$1/; # Remove qtree path
		$logname =~ s/.*:(\w+)$/$1/; # Remove volume path
		
		# Check for empty logname 
		if("$logname" =~ /^$/){ next; }

		my @sm_args = (	host => $fh,	 
				dst => "$filer_sm_info{$filer}[$volume_to_update]{'destination_path'}",
				src => "$filer_sm_info{$filer}[$volume_to_update]{'source_path'}",
				parallel => 1,
				timeout => 900 );
  		if($kbs){ push(@sm_args, 'throttle' => $kbs); };

		push(@subtests, subtest sub {
		    eval {
			snapmirror_update(@sm_args);
		    };
		    if ($@) {
			# Due high CPU, there's an edge case where the script could try and 
			# exceed the max allowable transfers.
			if ($@ =~ /could not get version|unexpected end of output|requested snapshot that does not exist|network error|read timeout|maximum number of destination configs for this filer|too many active transfers|could not establish connection|busy/) {
			    logresult("WARN", "$@");
			} else {
			    logresult("FATAL", "Error is $@");
			}
		    }
		}, -runid, "$logname", -bg);

	    if ($volume_to_update == $max_xfers) {
		$volume_to_update=0;
	    } else {
		$volume_to_update++;
	    }

	}

	foreach my $t (@subtests) {
	    sleep 1 while ($t->isAlive());
	}

	if ($print_percentage != 100 && $zigzag == 1) {
	    # Reduce transfers by 10%
	    my $decreased_limit=int($max_xfers*($percentage - .10));
	    logcomment("decreased limit is $decreased_limit");
	    while ( (get_current_xfers($fc, "$sm_type")) > $decreased_limit ) {
		logcomment("Reducing transfers back down below $decreased_limit (".($print_percentage-10)."% of max)");
		sleep 5;
	    }
	}
    }
}

sub snapmirror_zigzag_update_cmode_cron ($%) {
    my ($node, %args) = @_;
    my $sm_type = $args{'sm_type'};
    my $sm_info = $args{'sm_info'};
    my $use_zapi = $args{'use_zapi'} || "0";
    my $max_sm_endpoints = $args{'max_endpoints'};
    my $zigzag = $args{'zigzag'};
    my @array_of_vols = @$sm_info;
    my @percentages = qw(.20 .10 .40 .30 .60 .50 .80 .70 1);
    logcomment("zigzag is $zigzag");
    if(!$zigzag) {
        @percentages = qw(1);
    }

    my @subtests=();
    my $i=0;
    foreach my $percentage (@percentages) {
        my $print_percentage = $percentage;
        $print_percentage =~ s/\.//;
        if($print_percentage == 1) {$print_percentage = 100;}
        my $xfer_limit = int($max_sm_endpoints*$percentage);
        if($xfer_limit == 0) {$xfer_limit = 1 ;}

	my $xfer_limit_xtra=int($xfer_limit*.20);
	my $wait_mins = int(@array_of_vols/4);
	if($wait_mins < 15) {$wait_mins=15;}
	START:
	logcomment("Increasing transfers up to " . $xfer_limit . " (${print_percentage}% of max)");
	logcomment("Extra relationships is $xfer_limit_xtra");

	# Create Job Schedule for relationships
	my $update_time = time() + ($wait_mins*60);
	use Connectrec;
	my $conn = Connectrec->new(template=>"ssh-persist",host=>host($node),id=>++$i);
	create_sm_job($conn,
		    'node'         => $node,
		    'change_value' => $wait_mins,
		    'job_name'     => "snapmirror_update_job",
		    )
		    or die "Unable to modify cron job";
	create_sm_job($conn,
		    'node'         => $node,
		    'change_value' => ($wait_mins+30),
		    'job_name'     => "snapmirror_update_job_na",
		    )
		    or die "Unable to modify cron job";

	my $counter=1;
	for(my $x=0; $x < @array_of_vols; $x++) {

            my $sm_relationship = $array_of_vols[$x];

	    # Modify SM Relationships for new job schedule
	    for(my $y=0; $y < @{$sm_relationship->{'destination_vols'}}; $y++) {
		subtest sub {
		    my $destination_node = $sm_relationship->{'destination_node'};
		    logcomment("Updating SnapMirror Relationship",
			    "Source:      ".$sm_relationship->{'source_vol'},
			    "Destination: ".$sm_relationship->{'destination_vols'}[$y]);
		    my $source_path = $sm_relationship->{'cluster_src'} . "://" .
				    $sm_relationship->{'vserver_src'} . "/" .
				    $sm_relationship->{'source_vol'};
		    my $destination_path = $sm_relationship->{'cluster_dst'} . "://" .
				    $sm_relationship->{'vserver_dst'} . "/" .
				    $sm_relationship->{'destination_vols'}[$y];

		    my $sched_name = "snapmirror_update_job";
		    if($counter >= ($xfer_limit+$xfer_limit_xtra)) {
			$sched_name = "snapmirror_update_job_na";
		    }
		    my $return = snapmirror_modify_cmode($conn,
				'source_path'      => $source_path,
				'destination_path' => $destination_path,
				'schedule'         => $sched_name,
				'foreground'       => "true",
				'timeout'          => 3600,
				'filer'            => $node,
				);
		    if ($return !~ /SnapMirror: done/i) {
			logresult("FATAL", "SnapMirror update Error is $return");
		    }
		    #$conn->destroy();
		}, -runid, "SM_Mods";
		$counter++;
	    }
        }

	if($update_time < time()) {
	    $wait_mins+=5;
	    logcomment("The SnapMirror modifies didn't complete in time...adding 5 minutes and retrying");
	    goto START;
	}
	my $sleep_time = ($update_time - time());
	logcomment("Sleeping for $sleep_time seconds while data is added to source vols");
	sleep ($sleep_time);

	logcomment("Checking for current limit of $xfer_limit");
	my $hit_limit=0;
	my $start_time = time();
	while(!$hit_limit) {
	    my $curr_update_sessions = get_curr_sm_active_endpoints($node, $sm_type);
	    if($curr_update_sessions >=$xfer_limit) {
		$hit_limit=1;
		logcomment("Hit ${print_percentage}% of limit with $xfer_limit transfers");
	    } else {
		sleep 5;
		if(time() > ($start_time+300)) {
		    $xfer_limit_xtra++;
		    goto START;
		}
	    }
	}

        #if ($print_percentage != 100 && $zigzag == 1) {
            # Reduce transfers by 10%
        #    my $decreased_limit=int($max_sm_endpoints*($percentage - .10));
        #    logcomment("decreased limit is $decreased_limit");
        #    while ( (get_curr_sm_active_endpoints($node, "$sm_type")) > $decreased_limit ) {
        #        logcomment("Reducing transfers back down below $decreased_limit (".($print_percentage-10)."% of max)");
        #        sleep 5;
        #    }
        #}
	$conn->destroy();
    }
}

sub snapmirror_zigzag_update_cmode ($%) {
    my ($node, %args) = @_;
    my $sm_type = $args{'sm_type'};
    my $cluster = $args{'cluster'};
    my $cluster_src = $args{'cluster_src'} || $args{'cluster'};
    my $cluster_dst = $args{'cluster_dst'} || $args{'cluster'};
    my $vserver = $args{'vserver'};
    my $vserver_src = $args{'vserver_src'} || $args{'vserver'};
    my $vserver_dst = $args{'vserver_dst'} || $args{'vserver'};
    my $sm_info = $args{'sm_info'};
    my $use_zapi = $args{'use_zapi'} || "0";
    my $max_sm_endpoints = $args{'max_endpoints'};
    my $zigzag = $args{'zigzag'};
    my @array_of_vols = @$sm_info;

    my @percentages = qw(.20 .40 .60 .80 1);
    if(!$zigzag) {
        @percentages = qw(1);
    }

    my @subtests=();
    my $i=0;
    foreach my $percentage (@percentages) {
        my $print_percentage = $percentage;
        $print_percentage =~ s/\.//;
        if($print_percentage == 1) {$print_percentage = 100;}
        my $xfer_limit = int($max_sm_endpoints*$percentage);
        if($xfer_limit == 0) {$xfer_limit = 1 ;}

        my $curr_update_sessions = get_curr_sm_active_endpoints($node, $sm_type);
        my $remaining_sessions = $xfer_limit - $curr_update_sessions;
        my $remaining_sessions_extra = int(($remaining_sessions*.20)+$remaining_sessions);
        if ($remaining_sessions_extra == $remaining_sessions) { $remaining_sessions_extra += 2;}

	my $i=0;
        while ( $curr_update_sessions < $xfer_limit ) {
	    logcomment("remaining sessions extra is $remaining_sessions_extra");
            for(my $x=1; $x<= $remaining_sessions_extra; $x++) {
            logcomment("Increasing transfers up to " . $xfer_limit . " (${print_percentage}% of max)");

            if(@array_of_vols == 0) {
                @array_of_vols = @$sm_info;
            }
            my $sm_relationship = shift(@array_of_vols);

	    for(my $y=0; $y < @{$sm_relationship->{'destination_vols'}}; $y++) {
	    #use Connectrec;
	    $i++;
            push (@subtests, subtest sub {
	        my $destination_node = $sm_relationship->{'destination_node'};
		my $conn = connect_ssh_persist($i,$destination_node);
		#my $conn;
		#my $connected=0;
		#while(!$connected) {
		#    eval {
		#	$conn = Connectrec->new(template=>"ssh-persist",host=>host($destination_node),id=>++$i);
		#    };
#
#		    if(!$@) {
#			$connected=1;
#		    } else {
#		        logresult("WARN", "Not able to connect to node. Retrying in 5 seconds");
#			sleep 5;
#		    }
#		}

                logcomment("Updating SnapMirror Relationship",
                        "Source:      ".$sm_relationship->{'source_vol'},
                        "Destination: ".$sm_relationship->{'destination_vols'}[$y]);
		my $source_path = $sm_relationship->{'cluster_src'} . "://" .
				$sm_relationship->{'vserver_src'} . "/" .
				$sm_relationship->{'source_vol'};
		my $destination_path = $sm_relationship->{'cluster_dst'} . "://" .
				$sm_relationship->{'vserver_dst'} . "/" .
				$sm_relationship->{'destination_vols'}[$y];
                subtest sub {

			my $return = "";
			if ($sm_type =~ /ls/i) {
			    $return = snapmirror_update_cmode($conn,
					'source_path' => $source_path,
					'sm_type'     => $sm_relationship->{'sm_type'},
					'foreground'  => "true",
					'use_zapi'    => $use_zapi,
					'timeout'     => 3600,
					'filer'       => $node,
					);
			    if ($return !~ /SnapMirror: done/i) {
				logcomment("source_path is $source_path");
				logcomment("destination_path is $destination_path");
				logcomment("sm_type is ". $sm_relationship->{'sm_type'});
				logresult("FATAL", "SnapMirror update Error is $return");
			    }
			} else {
			    $return = snapmirror_update_cmode($conn,
					'source_path' => $source_path,
					'destination_path' => $destination_path,
					'sm_type'     => $sm_relationship->{'sm_type'},
					'foreground'  => "true",
					'use_zapi'    => $use_zapi,
					'timeout'     => 3600,
					'filer'       => $node,
					);
			    if ($return !~ /SnapMirror: done/i) {
				logcomment("source_path is $source_path");
				logcomment("destination_path is $destination_path");
				logcomment("sm_type is ". $sm_relationship->{'sm_type'});
				logresult("FATAL", "SnapMirror update Error is $return");
			    }
			}
                }, -runid, $sm_relationship->{'destination_vols'}[$y];
	        $conn->destroy();
            }, -bg, -runid, "SM_updates");
	    }
	    }
            sleep 10;
            $curr_update_sessions = get_curr_sm_active_endpoints($node, $sm_type);
            $remaining_sessions = $xfer_limit - $curr_update_sessions;
            $remaining_sessions_extra = int(($remaining_sessions*.20)+$remaining_sessions);
            if ($remaining_sessions_extra == $remaining_sessions) { $remaining_sessions_extra += 2;}
            logcomment("Current Sessions are $curr_update_sessions");
            logcomment("Remaining Sessions are $remaining_sessions");

        }

        foreach my $t (@subtests) {
            sleep 1 while ($t->isAlive());
        }

        if ($print_percentage != 100 && $zigzag == 1) {
            # Reduce transfers by 10%
            my $decreased_limit=int($max_sm_endpoints*($percentage - .10));
            logcomment("decreased limit is $decreased_limit");
            while ( (get_curr_sm_active_endpoints($node, "$sm_type")) > $decreased_limit ) {
                logcomment("Reducing transfers back down below $decreased_limit (".($print_percentage-10)."% of max)");
                sleep 5;
            }
        }
        if ($print_percentage != 100) {
            logcomment("Sleeping for 15 minutes while data is added to source vols");
            sleep 900; # Wait for the volumes to fill up with new data
        }
    }
}

sub snapmirror_zigzag_check_cmode ($%) {
    my ($node, %args) = @_;
    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $max_sm_checker_sessions = $args{'max_checker'} || die "max_checker not defined";
    my $sm_info = $args{'sm_info'} || die "sm_info not defined";
    my $use_zapi = $args{'use_zapi'} || "0";
    my $zigzag = $args{'zigzag'};

    if (defined($args{'zigzag'})) {
	$zigzag = $args{'zigzag'};
    } else {
	$zigzag = 1;
    }

    my @percentages = qw(.20 .10 .40 .30 .60 .50 .80 .70 1);
    if(!$zigzag) {
        @percentages = qw(1);
    }
    logcomment("zigzag is $zigzag");

    my @array_of_vols = @$sm_info;
    my @subtests=();
    foreach my $percentage (@percentages) {
        my $print_percentage = $percentage;
        $print_percentage =~ s/\.//;
        if($print_percentage == 1) {$print_percentage = 100;}
        my $checker_limit = int($max_sm_checker_sessions*$percentage);
        if($checker_limit == 0) {$checker_limit = 1 ;}

        my $curr_checker_sessions = dp_limits::get_curr_sm_checker_sessions($node);
        my $remaining_sessions = $checker_limit - $curr_checker_sessions;
	my $remaining_sessions_extra = int(($remaining_sessions*.20)+$remaining_sessions);
	if ($remaining_sessions_extra == $remaining_sessions) { $remaining_sessions_extra += 2;}
	my $i=0;
        while ( $curr_checker_sessions < $checker_limit ) {
            logcomment("Increasing checker sessions up to " . $checker_limit . " (${print_percentage}% of max)");
            for(my $x=1; $x<= $remaining_sessions_extra; $x++) {

                if(@array_of_vols == 0) {
                    @array_of_vols = @$sm_info;
                }
                my $sm_relationship = shift(@array_of_vols);

		$i++;
                push (@subtests, subtest sub {
		    my $destination_node = $sm_relationship->{'destination_node'};
		    my $conn = connect_ssh_persist($i,$destination_node);

                    logcomment("Running checker on SnapMirror Relationship",
                            "Source:      " . $sm_relationship->{'source_vol'},
                            "Destination: " . $sm_relationship->{'destination_vol'});
                    subtest sub {
                        my $source_path = $sm_relationship->{'cluster_src'} . "://" .
                                          $sm_relationship->{'vserver_src'} ."/" .
                                          $sm_relationship->{'source_vol'};
                        my $destination_path = $sm_relationship->{'cluster_dst'} . "://" .
                                          $sm_relationship->{'vserver_dst'} ."/" .
                                          $sm_relationship->{'destination_vol'};
                        my $target_snapshot = $sm_relationship->{'snapshot'};
			my $return = snapmirror_check_cmode($conn,
			    'source_path' => $source_path,
			    'destination_path' => $destination_path,
			    'sm_type'     => $sm_type,
			    'foreground'  => "true",
			    'use_zapi'    => $use_zapi,
			    'timeout'     => 3600,
			    'filer'       => $node,
			    );

                        if ($return !~ /SnapMirror: done/) {
                            # Move to the next SM relationship if this one is still in progress
                            if ($return =~ /already being checked|is in progress/) {
                                logresult("WARN", "$return");
                            } else {
                                logresult("FATAL", "Error is $return");
                            }
                        }
                    }, -runid, $sm_relationship->{'destination_vol'};
		    $conn->destroy();
                }, -bg, -runid, "SM_checker_sessions");

            }
            sleep 10;
            $curr_checker_sessions = dp_limits::get_curr_sm_checker_sessions($node);
            $remaining_sessions = $checker_limit - $curr_checker_sessions;
	    $remaining_sessions_extra = int(($remaining_sessions*.20)+$remaining_sessions);
	    if ($remaining_sessions_extra == $remaining_sessions) { $remaining_sessions_extra += 2;}
            logcomment("Current Sessions are $curr_checker_sessions");
            logcomment("Remaining Sessions are $remaining_sessions");
        }

        if ($print_percentage != 100 && $zigzag == 1) {
            # Reduce transfers by 10%
            my $decreased_limit=int($max_sm_checker_sessions*($percentage - .10));
            logcomment("decreased limit is $decreased_limit");
            while ( (dp_limits::get_curr_sm_checker_sessions($node)) > $decreased_limit ) {
                logcomment("Reducing transfers back down below $decreased_limit (".($print_percentage-10)."% of max)");
                sleep 5;
            }
        }

        if ($print_percentage != 100 ) {
            # Need to run snapmirror update on all the snapmirror relationships.  If this
            # isn't done, than snapmirror check won't have a new snapshot to check.

            logcomment("SLEEPING 15 MINUTES, RUNNING UPDATES and MOVING ON TO NEXT PERCENTAGE INCREASE");
	    # Sleep 15 minutes so the background I/O changes the data around
	    sleep 900;

            my @update_subtests;
	    @array_of_vols = @$sm_info;
	    my $i=1000;
            for(my $x=0; $x<@array_of_vols;$x++) {
		for(my $y=0; $y < @{$array_of_vols[$x]{'destination_vols'}}; $y++) {
		    $i++;
		    push (@update_subtests, subtest sub {
		    	my $destination_node = $array_of_vols[$x]{'destination_node'};
			my $conn = connect_ssh_persist($i,$destination_node);
			my $source_path = $array_of_vols[$x]->{'cluster_src'} . "://" .
					  $array_of_vols[$x]->{'vserver_src'} ."/" .
					  $array_of_vols[$x]->{'source_vol'};
			my $destination_path = $array_of_vols[$x]->{'cluster_dst'} . "://" .
					  $array_of_vols[$x]->{'vserver_dst'} ."/" .
					  $array_of_vols[$x]->{'destination_vols'}[$y];

			my $return = "";
			if ($array_of_vols[$x]->{'sm_type'} =~ /ls/i) {
			    if ($y == 0 ) {
				logcomment("Updating LS destination mirrors");
				$return = snapmirror_update_cmode($conn,
					    'source_path' => $source_path,
					    'sm_type'     => $array_of_vols[$x]->{'sm_type'},
					    'foreground'  => "true",
					    'use_zapi'    => $use_zapi,
					    'timeout'     => 3600,
					    'filer'       => $node,
					    );
			    }
			    if ($return !~ /SnapMirror: done/i) {
				logcomment("source_path is $source_path");
				logcomment("destination_path is $destination_path");
				logcomment("sm_type is ". $array_of_vols[$x]->{'sm_type'});
				logresult("FATAL", "SnapMirror update Error is $return");
			    }
			} elsif ($array_of_vols[$x]->{'sm_type'} =~ /dp/i) {
			    logcomment("Updating DP destination mirrors");
			    $return = snapmirror_update_cmode($conn,
					'source_path' => $source_path,
					'destination_path' => $destination_path,
					'sm_type'     => $array_of_vols[$x]->{'sm_type'},
					'foreground'  => "true",
					'use_zapi'    => $use_zapi,
					'timeout'     => 3600,
					'filer'       => $node,
					);
			    if ($return !~ /SnapMirror: done/i) {
				logcomment("source_path is $source_path");
				logcomment("destination_path is $destination_path");
				logcomment("sm_type is ". $array_of_vols[$x]->{'sm_type'});
				logresult("FATAL", "SnapMirror update Error is $return");
			    }
			}

			$conn->destroy();
		    }, -bg, -runid, "updates_".$array_of_vols[$x]->{'source_vol'});
		}
            }

            foreach my $t (@update_subtests) {
                sleep 1 while ($t->isAlive());
            }
        }
    }

    logcomment("HIT LIMIT.  WAITING FOR ALL CHECKS TO FINISH");
    foreach my $t (@subtests) {
        sleep 1 while ($t->isAlive());
    }
}

sub add_data_to_vols ($%) {
    my($fc, %args) = @_;
    $Log->comment("add_data_to_vols args are " . Data::Dumper->Dump([ \%args ]));
    my $filer = $args{'host'} || die "host not defined";
    my $data_if = $args{'data_if'} || $args{'host'};
    my $fops_cmd = $args{'fops_cmd'} || '/usr/software/test/bin/fops';
    my $fops_dir_height = $args{'fops_dir_height'} || 3;
    my $fops_dir_width = $args{'fops_dir_width'} || 3;
    my $fops_files_per_dir = $args{'fops_files_per_dir'} || 50;
    my $fops_file_size = $args{'fops_file_size'} || '1000k';
    my $fops_ops = $args{'fops_ops'} || die "fops operations not defined";
    my $fops_script = $args{'fops_script'} || 'stress-scale/wafl/scripts/stress/workload/fops_nfs.thpl';
    my $max_vols = $args{'max_vols'} || die "maximum number of volumes not defined";
    my $linux_clients = $args{'linux_clients'} || die "linux clients not defined";
    my $vol_info = $args{'vol_info'} || die "vol_info not defined";
    my @array_of_vols_hosts = @$vol_info;
    my $batch_size = (exists($args{'batch_size'})) ? $args{'batch_size'} : undef;
    my $onexit_handler = exists($args{'onexit'}) ? $args{'onexit'} : undef;

    my @linux_clients = split(",", $linux_clients);
    my $total_linux_clients = @linux_clients;
    my $remainder_volumes = 0;
    my $num_volumes_per_client = int($max_vols/$total_linux_clients);
    $num_volumes_per_client = 1 if ($num_volumes_per_client == 0);
    if ($max_vols <  ($num_volumes_per_client*$total_linux_clients) ) {
        $total_linux_clients = $max_vols;
    } else {
        $remainder_volumes = $max_vols - ($num_volumes_per_client*$total_linux_clients);
    }

    if((not defined($batch_size)) or ($batch_size > $max_vols)){ 
	$batch_size = $max_vols; 
    } 

    my @subtests =();
    my $is_qtree = 0;
    my $totalqtrees = @array_of_vols_hosts;
    $Log->comment("Total qtrees is $totalqtrees");
    my $vol_to_use = $array_of_vols_hosts[0]{'vol_path'};
    $Log->comment("Top vol_to_use is ${vol_to_use}");
    if ($vol_to_use =~ /\/vol\/\w+\/\w+/) { #Qtree
        $is_qtree = 1;
    }

    logcomment("Total Linux Clients           = $total_linux_clients");
    logcomment("Number Vols/Qtrees per Client = $num_volumes_per_client");
    logcomment("Remainder Vols                = $remainder_volumes");
    logcomment("Batch Size                    = $batch_size");

    my $runid = "";
    my $abs_counter = 1;
    for(my $x=0; $x<$total_linux_clients; $x++) {
        my $counter=1;
        while($counter <= $num_volumes_per_client) {

            my $array_values = shift (@array_of_vols_hosts);
	    if(not defined($array_values)){ 
		$abs_counter++;
		$counter++;
	        next; 
	    } 
            # QSM on 6070 uses "DIR_HEIGHT=2", "DIR_WIDTH=2",
            # "FILES_PER_DIR=100", "FILE_SIZE=1000k",
            # VSM on 6070 uses "DIR_HEIGHT=3", "DIR_WIDTH=3",
            # "FILES_PER_DIR=50", "FILE_SIZE=1000k",
            # LA params: "DIR_HEIGHT=3", "DIR_WIDTH=3", 
            #"FILES_PER_DIR=6000", "FILE_SIZE=1500k", 

            push(@subtests, subtest(sub {
                my @fops_params = ( "FILER=${filer}", "MOUNT_CHMOD=777", 
                "DIR_HEIGHT=${fops_dir_height}", 
		"DIR_WIDTH=${fops_dir_width}", 
                "FILES_PER_DIR=${fops_files_per_dir}", 
		"FILE_SIZE=${fops_file_size}", 
		"MOUNT_CHMOD=777",
                "OPERATIONS=${fops_ops} ", "CLIENT=$linux_clients[$x]",
		"DATA_INTERFACE=${data_if}");

                if ($is_qtree) {
                    if(defined($array_values->{'vfiler'})) {
			if($array_values->{'vfiler'} =~ /$filer/){
		 	    # Newer or non-standard vfiler name	
			    $filer = $array_values->{'vfiler'};
			} else {
			    # Older naming convention
                            $filer=${filer}."-".$array_values->{'vfiler'};
 			}
                    }
                    $vol_to_use     = $array_values->{'vol_path'};
		    if(not defined($vol_to_use)){
		        return 0;
		    }
                    logcomment("vol_to_use is $vol_to_use");
                    my $qtree_to_use   = $array_values->{'vol_path'};
                    $qtree_to_use   =~ s#/vol/\w+/(\w+)#$1#;
		    if(not defined($qtree_to_use)){
		        return 0;
   		    }
                    $runid = $linux_clients[$x] . "_" . $qtree_to_use;
                    my $retry_mount = 1;
                    while ($retry_mount) {
			$Log->comment("Attempting to mount ${filer}:${vol_to_use} on $linux_clients[$x]");
                        my $mountpoint = mount ($filer, 
                                $vol_to_use,
                                client => "$linux_clients[$x]");
                        if ($mountpoint =~ /\//) {
                            push(@fops_params, "DIR_PATH=${mountpoint}");
                            $retry_mount = 0;
                        } else {
                            sleep 5;
                        }
                    }
                } else {
                    $vol_to_use     = $array_values->{'vol_path'};
		    if(not defined($vol_to_use)){
		        return 0;
		    }
                    $vol_to_use     =~ s#/vol/##;
                    $runid = $linux_clients[$x] . "_" . $vol_to_use;
                    push(@fops_params, "TEST_VOL=${vol_to_use}");
                }
                if(defined($array_values->{'vfiler'})) {
		    if($array_values->{'vfiler'} =~ /$filer/){
			push(@fops_params, "VFILER=" . $array_values->{'vfiler'});
		    } else {
                        push(@fops_params, "VFILER=${filer}-".$array_values->{'vfiler'});
		    }
                }
		if(defined($args{'say_timeout'})){
		    push(@fops_params, "SAY_TIMEOUT=" . $args{'say_timeout'});
		}
		push(@fops_params, 'FOPS_CMD=' . $fops_cmd); 
		if(defined($onexit_handler)){
		    push(@fops_params, -onexit => $onexit_handler);
		}
                subtest($fops_script, 
                        @fops_params, 
                        -runid, "$runid", 
                        -client, "$linux_clients[$x]");
            }, -bg, -runid, $runid));

            if(($abs_counter % $batch_size) == 0){
                # Wait for batch_size fops processes to complete
                foreach my $t (@subtests) {
                    sleep 1 while ($t->isAlive());
		}		
                @subtests = ();
            }
	    $abs_counter++;
            $counter++;
            sleep 1;
        }
    }

    if ($remainder_volumes > 0) {
        for (my $x=0; $x<$remainder_volumes; $x++) {

            my $array_values = shift (@array_of_vols_hosts);
	    if(not defined($array_values)){
		next;
	    }
#            push(@subtests, subtest(sub {
                my @fops_params = ( "FILER=$filer", "MOUNT_CHMOD=777", 
                                "DIR_HEIGHT=3", "DIR_WIDTH=3", 
                                "FILES_PER_DIR=50", "FILE_SIZE=1000k", 
                                "OPERATIONS=$fops_ops ", "CLIENT=$linux_clients[$x]",
				"DATA_INTERFACE=$data_if");
                if ($is_qtree) {
                    if(defined($array_values->{'vfiler'})) {
			if($array_values->{'vfiler'} =~ /$filer/){
			    $filer=$array_values->{'vfiler'};
			} else {
                            $filer=${filer}."-".$array_values->{'vfiler'};
			}
                    }
                    $vol_to_use     = $array_values->{'vol_path'};
		    if(not defined($vol_to_use)){
		        next;
		    }
                    my $qtree_to_use   = $array_values->{'vol_path'};
                    $qtree_to_use   =~ s#/vol/\w+/(\w+)#$1#;
                    $runid = $linux_clients[$x] . "_" . $qtree_to_use;
                    my $retry_mount = 1;
                    while ($retry_mount) {
                        my $mountpoint = mount ($filer, 
                                $vol_to_use,
                                client => "$linux_clients[$x]");
                        if ($mountpoint =~ /\//) {
                            push(@fops_params, "DIR_PATH=${mountpoint}");
                            $retry_mount = 0;
                        } else {
                            sleep 5;
                        }
                    }
                } else {
                    $vol_to_use     = $array_values->{'vol_path'};
		    if(not defined($vol_to_use)){
		        next;
		    }
                    $vol_to_use     =~ s#/vol/##;
                    $runid = $linux_clients[$x] . "_" . $vol_to_use;
                    push(@fops_params, "TEST_VOL=${vol_to_use}");
                }
                if(defined($array_values->{'vfiler'})) {
		    if($array_values->{'vfiler'} =~ /$filer/){
		        push(@fops_params, "VFILER=" . $array_values->{'vfiler'});
	            } else {	
                        push(@fops_params, "VFILER=${filer}-".$array_values->{'vfiler'});
		    }
                }
		if(defined($args{'say_timeout'})) {
		    push(@fops_params, "SAY_TIMEOUT=" . $args{'say_timeout'});
		}
		push(@fops_params, 'FOPS_CMD=' . $fops_cmd);
		if(defined($onexit_handler)){
		    push(@fops_params, -onexit => $onexit_handler);
 		}
                push(@subtests, subtest($fops_script, 
                        @fops_params, 
                        -runid, "$runid", 
                        -client => "$linux_clients[$x]",
			-runid => "creating_data",
			-bg));
            # }, -bg, -runid, "creating_data"));
            sleep 1;
        }
    }

    foreach my $t (@subtests) {
        sleep 1 while ($t->isAlive());
    }

    return 1;
}

sub add_data_to_vols_cmode ($%) {
    my ($node, %args) = @_;
    my $fops_params = $args{'fops_params'} || die "Missing fops_params";
    my $sm_info = $args{'sm_info'} || die "Missing sm_info";
    my $dataif = $args{'dataif'};
    my $max_vols = $args{'max_vols'} || die "Missing max_vols";
    my $iterations = $args{'iterations'} || die "Missing iterations";
    my $linux_clients = $args{'linux_clients'} || die "Linux Clients";
    my $io_tool = $args{'io_tool'};
    my $read_only_io = $args{'read_only_io'} || "0";
    my $num_fops_instances_per_client = $args{'num_fops_per_client'} || "1";
    my $client_io_sleep_time = $args{'client_io_sleep_time'} || 600;

    my @linux_clients = split(",", $linux_clients);
    my $max_parallel_iterations = $num_fops_instances_per_client *@linux_clients;
    my @array_of_vols = @$sm_info;
    my $total_linux_clients = @linux_clients;
    my $total_vols = @array_of_vols;
    logcomment("Total Linux Clients    = $total_linux_clients");
    logcomment("Total Number Vols      = $total_vols");

    my @subtests=();

    my $loop_count=0;
    for(my $x=0; $x<$total_vols; $x++) {
	# Allow no more than 1 instances per client
	if ($total_vols/$total_linux_clients > $num_fops_instances_per_client ) {
	    $loop_count++;
	    logcomment("Max parallel iterations is $max_parallel_iterations");
	    if ($loop_count == $max_parallel_iterations) {
		logcomment("Hit Max parallel iterations of $max_parallel_iterations");
		logcomment("Sleeping ${client_io_sleep_time}  minutes");
		sleep $client_io_sleep_time; 
		$loop_count = 0;
	    }
	}

        my $array_values = shift (@array_of_vols);
        if (@linux_clients == 0) {
            # Repopulate the list of linux clients to loop through
            @linux_clients = split(",", $linux_clients);
        }
        my $linux_client = shift (@linux_clients);
        push(@subtests, subtest sub {
	    my $path_to_use    = "/".$array_values->{'source_vol'};
	    my $runid = $linux_client . "_" . $array_values->{'source_vol'};
	    if(defined($array_values->{'qtree'})) {
		$path_to_use    = "$path_to_use/".$array_values->{'qtree'};
	    }
	    my $path_prefix="";
	    if($array_values->{'sm_type'} =~ /ls/i) {
                # LS volumes have to be mounted with /.admin in the path
	        $path_prefix="/.admin";
            }
	    $path_to_use = $path_prefix . $path_to_use;
	    if($array_values->{'source_ip'} ne '') {
	        $dataif = $array_values->{'source_ip'};
	    }
	    die "missing value for dataif" if ($dataif eq "");
            logcomment("Mounting $path_to_use from IP $dataif on client $linux_client");
            my $mntpnt;
			if ($read_only_io) {
	            $mntpnt = mount($dataif,
                            $path_to_use,
                            retries => 5,
                            proto   => "tcp",
                            vers    => 3,
                            client  => $linux_client,
                            mountopts => "ro,hard,intr,noac",
                            ) or die fail_result("mount failed: $@", stacktrace => 0);
			} else {
	            $mntpnt = mount($dataif,
                            $path_to_use,
                            retries => 5,
                            proto   => "tcp",
                            chmod   => "777",
                            vers    => 3,
                            client  => $linux_client,
                            mountopts => "rw,hard,intr,noac",
                            ) or die fail_result("mount failed: $@", stacktrace => 0);
			}
            subtest sub {
                my $loop_count=0;
                while (1) {
                    run_io($linux_client,
                        'io_tool'       => "$io_tool",
                        'mntpnt'        => "$mntpnt",
                        'fops_params'   => "$fops_params",
                        );
                    $loop_count++;
                    if ($loop_count == $iterations) {
                        last;
                    }
                }
            }, -runid, "$runid";
        }, -bg, -runid, "creating_data_".$linux_client
        );
        sleep 1;
    }

    foreach my $t (@subtests) {
        sleep 1 while ($t->isAlive());
    }

    return @subtests;
}


sub create_vols ($$%) {
    my ($fc, $filer_side, %args) = @_;

    my $sm_type = $args{'sm_type'} || "vsm";
    my $vol_type = $args{'vol_type'} || "flex";
    my $raid_type = $args{'raid_type'} || "raid4";
    my $vol_name_fmt = $args{'vol_name_fmt'} || "${sm_type}_${filer_side}_${vol_type}" . '_%d';
    my $vol_size = $args{'vol_size'} || "12g";
    my $enable_dedup = $args{'enable_dedup'} || 0;
    # Add $enable_compress someday
    my $export_vols;
    if(defined($args{'export_vols'})){
        $export_vols = $args{'export_vols'};
    } else { 
        $export_vols = 1;
    }
    my $filer = $args{'host'} || die "host not defined";
    my $max_vols = $args{'max_vols'} || die "max_vols is not defined";

    my @aggrs = (); 
    if($args{'aggrs'}){
        # New 'aggrs' list preempts single 'aggr'
        @aggrs = @{$args{'aggrs'}};
    } elsif ($args{'aggr'}){
        @aggrs = ( $args{'aggr'} );
    } else {
        @aggrs = ( 'test_aggr' );
    }
    $Log->comment('create_vols aggrs are: ' . join(' ', @aggrs));

    my @vol_list = (); # For return
   
    my $current_aggr;
    for(my $x=1; $x <= $max_vols; $x++) {
        my $aggr_num;
        if(scalar(@aggrs) > 0){
	    $aggr_num = $x % scalar(@aggrs);
	} else {
	    $Log->warn("SSQA::DPHelper::create_vols has empty aggrs list");
	    $aggr_num = 0;
        }
        my $aggr = $aggrs[$aggr_num];

	my $vol_name = sprintf($vol_name_fmt, $x);
	push(@vol_list, $vol_name);
	if ($vol_type eq "trad") {
	    if (!vol_exists($fc, $vol_name)) {
		vol_create($fc, $vol_name, 2, raidtype => $raid_type)
		    or die "\nFunction Failed: vol_create($fc, $vol_name, 2, raidtype => $raid_type)\n$@";
	    }
	} elsif ($vol_type eq "flex") {
	    if (!aggr_exists($fc, "$aggr")) {
		eval {
		    aggr_create($fc, "$aggr", '13');
 		};
		# Sometimes aggr_exists gets no info from ONTAP
		$Log->warn("\nFunction Failed: aggr_create($fc, $aggr, 13): $@");
	    }
	    if (!vol_exists($fc, $vol_name)) {
		vol_create_flex($fc, $vol_name, $aggr, $vol_size)
		    or die "\nFunction Failed: vol_create_flex($fc, $vol_name, $aggr, $vol_size)\n$@";
	    }
	    if ($enable_dedup && ($filer_side ne "dst")) {
		dense_toggle(host=>$filer, path_name=>"/vol/${vol_name}", state=>"on");
		dense_start(host=>$filer, path_name=>"/vol/${vol_name}");
	    }
	}
	# Turn off snapshot scheduler
	snapsched_volume($fc, $vol_name, (0, 0, 0, 0));
        if($export_vols){
	    # Export the volume
	    my $retry=1;
	    while ($retry) { 
	        eval {
		    logcomment("Exporting /vol/$vol_name");
		    ExportFS::export_path($fc,"/vol/${vol_name}", OPTIONS=>"sec=sys,rw,nosuid,anon=0");
	        };
	        if ($@) {
		    if ($@ =~ /already in progress/) {
		        logresult("WARN", "Error is $@");
		        sleep 5;
		        $retry=1;
		    } else {
		        logresult("FATAL", "Error is $@");
		        $retry=0;
		    } 
	        } else {
		    $retry=0;
	        }
	    } # END WHILE(retry) EXPORT VOL
        } # END IF(export_vols) 
    } # END FOR VOL 'x'

    return @vol_list;
}

sub create_vols_src ($%) {
    my ($fc, %args) = @_;

    my @vol_list = create_vols($fc, "src", %args);
    return @vol_list;
}

sub create_vols_dst ($%) {
    my ($fc, %args) = @_;

    my @vol_list = create_vols($fc, "dst", %args);
    return @vol_list;
}

sub create_qtrees ($$%) {
    my ($fc, $filer_side, %args) = @_;

    my $sm_type = $args{'sm_type'} || "vsm";
    my $filer = $args{'host'} || die "host not defined";
    my $qtrees_per_vol = $args{'qtrees_per_vol'} || die "qtrees_per_vol not defined";
    my $max_qtrees = $args{'max_qtrees'} || die "max_qtrees not defined";
    my $vols = $args{'vols'} || die "vols not defined";
    my @volume_list = @$vols; 
    my @qtree_list = ();

    $Log->comment("Creating $max_qtrees on $filer at $qtrees_per_vol qtrees per volume");

    my $qtree_counter = 0;
    my $vol = shift(@volume_list);
    $vol =~ s#/vol/##;
    for(my $x=1; $x <= $max_qtrees; $x++) {
	push(@qtree_list,"/vol/${vol}/${sm_type}_${filer_side}_qtree_${x}");
	if (!qtreePresent($fc, "/vol/${vol}/${sm_type}_${filer_side}_qtree_${x}")) {
	    logcomment("Creating Qtree /vol/${vol}/${sm_type}_${filer_side}_qtree_${x}");
	    qtreeCreate($fc, "/vol/${vol}/${sm_type}_${filer_side}_qtree_${x}")
		or logresult("FATAL",
	    	"Function Failed: qtreeCreate($fc, /vol/${vol}/${sm_type}_${filer_side}_qtree_${x})$@");
	}
	my $retry=1;
	while ($retry) { 
	    eval {
		ExportFS::export_path($fc,
		    "/vol/${vol}/${sm_type}_${filer_side}_qtree_${x}", 
		    OPTIONS=>"sec=sys,rw,nosuid,anon=0") or 
		    logresult("FATAL",
		    "Export Failed: export_path($fc,/vol/${vol}/${sm_type}_${filer_side}_qtree_${x})$@");
		    logcomment("qtree_counter is $qtree_counter, 
			qtrees_per_vol is $qtrees_per_vol, volume_list is ", @volume_list);
	    };
	    if ($@) {
		if ($@ =~ /already in progress/) {
		    logresult("WARN", "Error is $@");
		    sleep 5;
		    $retry=1;
		} else {
		    logresult("FATAL", "Error is $@");
		    $retry=0;
		} 
	    } else {
		$retry=0;
	    }
	}
	if ((++$qtree_counter == $qtrees_per_vol) && (@volume_list > 0)) {
	    $vol = shift(@volume_list);
	    $vol =~ s#/vol/##;
	    $qtree_counter = 0;
	}
    }
    return @qtree_list;
}

sub create_qtrees_src ($%) {
    my ($fc, %args) = @_;

    my @vol_list = create_qtrees($fc, "src", %args);
    return @vol_list;
}

sub create_qtrees_dst ($%) {
    my ($fc, %args) = @_;

    my @vol_list = create_qtrees($fc, "dst", %args);
    return @vol_list;
}

## create_vfilers
##
## Note: $args{'vfiler_name_fmt'} contains an sprintf format string with one 
##    '%d' which will be substituted with the number of each vfiler processed
sub create_vfilers ($$%) {
    my ($fc, $filer_side, %args) = @_;
   
    $Log->comment("create_vfilers args: " . Data::Dumper->Dump([ \%args ]));

    my $filer = $args{'host'} || die "host not defined";
    # my $max_vols = $args{'max_vols'} || die "max_vols is not defined";
    my $aggr = defined($args{'aggr'}) ? $args{'aggr'} : "test_aggr";
    my $enable_dedup = $args{'enable_dedup'} || 0;
    my $enable_compress = $args{'enable_compress'} || 0;
    if($enable_compress){ $enable_dedup = 1; }
    my $enable_snapmirror = $args{'enable_snapmirror'} || 1;
    my $enable_snapvault = $args{'enable_snapvault'} || 0;
    my $resources = $args{'resources'} || "";
    my @resource_list = ();
    if(ref($resources)){ @resource_list = @$resources; }
    my $vfiler_ips = $args{'vfiler_ips'} || logcomment "vfiler_ips not defined";
    my @vfiler_ip_addrs = ();
    if(ref($vfiler_ips)){ @vfiler_ip_addrs = @$vfiler_ips; }
    my $vfiler_info = $args{'vfiler_info'} || logcomment "vfiler_info not defined";
    my %vfiler_config_info = %$vfiler_info;
    my $vfiler_name_fmt = $args{'vfiler_name_fmt'} || 'vf%d';
    my $vfiler_rootvol_size = $args{'vfiler_rootvol_size'} || "1g";

    my $response; # APISet response object
    my $filer_ontap_version = get_ontap_version($fc->hostp());

    # Check @vfiler_ip_addrs against max vfilers, raise limit if necessary
    my $vfiler_limit = vfiler_limit($fc);
    my $vfiler_hard_limit = vfiler_limit_hard_limit($fc);
    if(@vfiler_ip_addrs > $vfiler_limit){
        if(@vfiler_ip_addrs < $vfiler_hard_limit){
             vfiler_limit_set($fc, @vfiler_ip_addrs);
        } else {
	     # Set vfilers to max, reduce @vfiler_ip_addrs to fit
	     logcomment("Reducing number vfiler ip addresses to hard limit");
	     vfiler_limit_set($fc, $vfiler_hard_limit);		
	     @vfiler_ip_addrs = @vfiler_ip_addrs[0 .. ($vfiler_hard_limit -1)];
	}
    }

    my $resource_vols_per_vfiler = int(@resource_list / @vfiler_ip_addrs);
    my $remainder_vols_per_vfiler = scalar(@resource_list) - ($resource_vols_per_vfiler * scalar(@vfiler_ip_addrs));
    my @vol_info = ();


    # Create vfilers and vfiler root vols on source and destination
    logcomment ("Create the Vfilers on $filer");
    my $host = Hostrec->new('id' => $filer);
    my $fc_telnet;
    do {
	sleep int(rand(5)); # A little randomized backoff
        eval {
	    $fc_telnet = Connectrec->new('conntype' => 'telnet',
				    'host' => $host, 
				    'template' => "${filer}#telnet", 
				    'timeout' => 30);
        };
	$Log->comment("Initial telnet connect returned error: " . $@) if ($@);
    } while ($@ =~ /Unexpected EOF/ && $fc_telnet->disconnect());
    if($@){
        die($@);
    }
    
    my $new_vol="";
    my $vol_path="";
    my $found_vol=0;
    for(my $x=1; $x <= @vfiler_ip_addrs ; $x++) {
	# Reestablish telnet session for each vfiler to avoid timeouts
	# Retry connection attempt while telnet session is locked out by
	#   other connections
	do {
	    sleep int(rand(5)); # A little randomized backoff
	    eval { 
	        $fc_telnet->connect();
            };
	    $Log->comment("Telnet connect returned error: " . $@) if($@);
        } while($@ =~ /Unexpected EOF/ && $fc_telnet->disconnect());
	if($@){
	    die($@);
        }

        my $vfiler_name = sprintf($vfiler_name_fmt, $x);
        my $vfiler_vol_name = $vfiler_name;
	$vfiler_vol_name =~ s/-/_/g;
       if (!vol_exists($fc, $vfiler_vol_name)) {
           vol_create_flex($fc, $vfiler_vol_name, $aggr, $vfiler_rootvol_size)
               or die "\nFunction Failed: vol_create_flex($fc, $vfiler_vol_name, $aggr, 1g)\n$@";
       }
       if ($enable_compress) {
	   filer_vol_option_set($fc, $vfiler_vol_name, 'compression', 'on');
	   # Remember: case this, because 8.0.x uses vol compress, 8.1.x doesn't
	   if(version_compare('8.0', $filer_ontap_version) > -1){
	       filer_vol_compress_start($fc, $vfiler_vol_name);
           } 
       }
       if ($enable_dedup) {
	   dense_toggle(host=>$filer, path_name=>"/vol/${vfiler_vol_name}", state=>"on");
	   dense_start(host=>$filer, path_name=>"/vol/${vfiler_vol_name}");
       }

        my %vfiler_status = vfiler_status($fc_telnet, $vfiler_name, option=>"-r");
        if (! exists $vfiler_status{$vfiler_name}) {
	    my $if_to_use = $vfiler_config_info{'if_to_use'} || die "if_to_use not defined";
	    my $netmask   = $vfiler_config_info{'netmask'}   || die "netmask not defined";
	    my $dnsdomain = $vfiler_config_info{'dnsdomain'} || die "dnsdomain not defined";
	    my $dnsserver = $vfiler_config_info{'dnsserver'} || die "dnsserver not defined";
	    my $nisdomain = $vfiler_config_info{'nisdomain'} || die "nisdomain not defined";
	    my $nisserver = $vfiler_config_info{'nisserver'} || die "nisserver not defined";
	    my $vfiler_password = $vfiler_config_info{'vfiler_password'} || die "vfiler_password not defined";
            vfiler_create($fc_telnet, $vfiler_name,
                'vfiler_password' => "$vfiler_password",
                'if_to_use' => "$if_to_use",
                'netmask' => "$netmask",
                'dnsdomain' => "$dnsdomain",
                'dnsserver' => "$dnsserver",
                'nisdomain' => "$nisdomain",
                'nisserver' => "$nisserver",
                'ipaddr_paths' => "-i $vfiler_ip_addrs[$x-1] /vol/${vfiler_vol_name}",
                'no_cifs' => "1",
                'count' => 1) or die "Vfiler not created on filer";

            %vfiler_status = vfiler_status($fc_telnet, $vfiler_name, option=>"-r");
            logcomment("vfiler ${vfiler_name} status hash is " . Data::Dumper->Dump([ \%vfiler_status ]));

        }
	push(@vol_info,{ 'vol_path' => "/vol/${vfiler_vol_name}", 
		         'vfiler' => $vfiler_name, 
			 'ip' => "$vfiler_ip_addrs[$x-1]"});

        logcomment("vfiler ${vfiler_name} status is ". $vfiler_status{$vfiler_name}{'status'});
        if ($vfiler_status{$vfiler_name}{'status'} != /running/) {
            logcomment("Starting vfiler ". $vfiler_status{$vfiler_name}{'status'});
            vfiler_start($fc_telnet, $vfiler_name) ||
                die "Failed to start vfiler ${vfiler_name}";
        }

	# Enable ZAPI,rsh on vFiler
        my $vf_cli_apiset = NACL::APISet->new(	'connection' => $fc,
						'category' => 'Node',
						'interface' => 'CLI',
						'set' => 'Vfiler',
						'context_definition' => {'name' => $vfiler_name }
						);
        $response = $vf_cli_apiset->options('httpd.admin.enable' => 'on');
	$response = $vf_cli_apiset->options('rsh.enable' => 'on');
	# $response = $vf_cli_apiset->options('rsh.access' => 'all');

        my $vf_zapi_apiset = NACL::APISet->new( 'connection' => $fc,
						'category' => 'Node',
						'interface' => 'ZAPI',
						'set' => 'Vfiler',
						'context_definition' => {'name' => $vfiler_name }
						);
	# Enable CIFS if requested
	if(defined($vfiler_config_info{'cifs_server_name'})){

	    $response = $vf_zapi_apiset->cifs_setup(
		'auth-type' => 'ad',
		'domain-name' => $vfiler_config_info{'cifs_domain'},
		'login-user' => $vfiler_config_info{'cifs_user'},
		'login-password' => $vfiler_config_info{'cifs_password'},
		'security-style' => $vfiler_config_info{'cifs_security_style'},
		'server-name' => $vfiler_config_info{'cifs_server_name'}
	    );

	    if(not(defined($response) and $response->{'execution_status'} eq 'passed')){
		die(Data::Dumper->Dump([ $response ]));
	    }

	    # Share rootvol
	    $response = $vf_zapi_apiset->cifs_share_add(
		'path' => '/vol/' . $vfiler_vol_name,
		'share-name' => $vfiler_vol_name
	    );
	}

	# Enable services/options on vFiler
        if($enable_snapmirror){ 
	    $response = $vf_cli_apiset->options('snapmirror.enable' => 'on');
            $response = $vf_cli_apiset->options('snapmirror.access' => 'all');
	    $response = $vf_cli_apiset->snapmirror_on();
        }
        if($enable_snapvault){
            $response = $vf_cli_apiset->options('snapvault.enable' => 'on');
            $response = $vf_cli_apiset->options('snapvault.access' => 'all');
        }

        my @vfiler_path;
        if(defined($vfiler_status{$vfiler_name}{'Path'})){
            @vfiler_path = @{$vfiler_status{$vfiler_name}{'Path'}};
        } else {
            @vfiler_path = ();
        }

	# Don't forget code to add remainder volumes to last vfiler
        if($x == @vfiler_ip_addrs){
	    $resource_vols_per_vfiler += @resource_list;
	}
	for(my $r = 0; $r < $resource_vols_per_vfiler; $r++){
            $vol_path = shift(@resource_list);
	    my $vol = $vol_path;
	    $vol =~ s#/vol/(\w+).*#$1#;
	    if($vol =~ /^$/){ next; }		
	    push(@vol_info,{ vol_path=>"$vol_path", vfiler=>$vfiler_name, ip=>"$vfiler_ip_addrs[$x-1]"});
	    if ("@vfiler_path" !~ /$vol_path/) {
			logcomment("Adding path /vol/$vol to vfiler ${vfiler_name}");
			vfiler_add($fc_telnet, $vfiler_name, path=>"/vol/$vol") ||
			    die "Failed to add path /vol/$vol to vfiler ${vfiler_name}";
			vfiler_run_vfiler_exportfs($fc_telnet, $vfiler_name, "/vol/$vol") ||
			    die "Failed to export path /vol/$vol on vfiler vf${x}";
	    }
	}
	$fc_telnet->disconnect();
    } # END FOR VFILERS

    return @vol_info;
}

## create_vfilers_qtree_roots
##
## Note: $args{'vfiler_name_fmt'} contains an sprintf format string with one 
##    '%d' which will be substituted with the number of each vfiler processed
sub create_vfilers_qtree_roots ($%) {
    my ($fc, %args) = @_;

    my $filer = $args{'host'} || die "host not defined";
    # my $max_vols = $args{'max_vols'} || die "max_vols is not defined";
    my $aggr = $args{'aggr'} || "test_aggr";
    my $enable_dedup = $args{'enable_dedup'} || 0;
    my $enable_compress = $args{'enable_compress'} || 0;
    if($enable_compress){ $enable_dedup = 1; }
    my $enable_snapmirror_sync = $args{'enable_snapmirror_sync'} || 0;
    my $enable_snapvault = $args{'enable_snapvault'} || 0;
    my $resources = $args{'resources'} || "";
    my @resource_list = ();
    if(ref($resources)){ @resource_list = @$resources; }
    my $vfiler_ips = $args{'vfiler_ips'} || logcomment "vfiler_ips not defined";
    my @vfiler_ip_addrs = ();
    if(ref($vfiler_ips)){ @vfiler_ip_addrs = @$vfiler_ips; }
    my $vfiler_info = $args{'vfiler_info'} || logcomment "vfiler_info not defined";
    my %vfiler_config_info = %$vfiler_info;
    my $vfiler_name_fmt = $args{'vfiler_name_fmt'} || 'vf%d';
    my $vfiler_root_vol = $args{'vfiler_root_vol'} || 'vol0';
    my $vfiler_root_vol_size = $args{'vfiler_root_vol_size'} || "1g";
    my $apiset = NACL::APISet->new('connection' => $fc,
					'category' => 'Node',
					'interface' => 'ZAPI',
					'set' => '7Mode');
    my $response; # APISet response object

    # Check @vfiler_ip_addrs against max vfilers, try to raise limit if needed. 
    # That might require a reboot, which is not attempted here.
    my $vfiler_limit = vfiler_limit($fc);
    my $vfiler_hard_limit = vfiler_limit_hard_limit($fc);
    if(@vfiler_ip_addrs > $vfiler_limit){
        if(@vfiler_ip_addrs < $vfiler_hard_limit){
             vfiler_limit_set($fc, @vfiler_ip_addrs);
        } else {
	     # Set vfilers to max, reduce @vfiler_ip_addrs to fit
	     logcomment("Reducing number vfiler ip addresses to hard limit");
	     vfiler_limit_set($fc, $vfiler_hard_limit);		
	     @vfiler_ip_addrs = @vfiler_ip_addrs[0 .. ($vfiler_hard_limit -1)];
	}
    }

    my $resource_vols_per_vfiler = int(@resource_list / @vfiler_ip_addrs);
    my $remainder_vols_per_vfiler = @resource_list - ($resource_vols_per_vfiler * @vfiler_ip_addrs);
    my @vol_info = ();


    # Create vfilers and vfiler root qtrees on source and destination
    logcomment ("Create the Vfilers on $filer");
    host ($filer);
    my $fc_telnet = connect("#telnet", -timeout, 10);
    my $new_vol="";
    my $vol_path="";
    my $found_vol=0;
    for(my $x=1; $x <= @vfiler_ip_addrs ; $x++) {
        my $vfiler_name = sprintf($vfiler_name_fmt, $x);
        my $vfiler_vol_name = $vfiler_root_vol;
	my $vfiler_qtree_name = $vfiler_name;
	$vfiler_qtree_name =~ s/-/_/g;
	if (!vol_exists($fc, $vfiler_vol_name)) {
           vol_create_flex($fc, $vfiler_vol_name, $aggr, $vfiler_root_vol_size)
               or die "\nFunction Failed: vol_create_flex($fc, $vfiler_vol_name, $aggr, $vfiler_root_vol_size)\n$@";
       }
       if ($enable_dedup) {
	   dense_toggle(host=>$filer, path_name=>"/vol/${vfiler_vol_name}", state=>"on");
	   dense_start(host=>$filer, path_name=>"/vol/${vfiler_vol_name}");
       }
       if ($enable_compress) {
	   filer_vol_option_set($fc, $vfiler_vol_name, 'compression', 'on');
	   # Remember: case this, because 8.0.x uses vol compress, 8.1.x doesn't
	   filer_vol_compress_start($fc, $vfiler_vol_name);
       }
       if(!qtreePresent($fc, "/vol/${vfiler_vol_name}/${vfiler_qtree_name}")){
	    qtreeCreate($fc, "/vol/${vfiler_vol_name}/${vfiler_qtree_name}")
		or die("Function Failed: qtree_create()\n$@");
	} 	

        my %vfiler_status = vfiler_status($fc_telnet, $vfiler_name, option=>"-r");
        if (! exists $vfiler_status{$vfiler_name}) {
	    my $if_to_use = $vfiler_config_info{if_to_use} || die "if_to_use not defined";
	    my $netmask   = $vfiler_config_info{netmask}   || die "netmask not defined";
	    my $dnsdomain = $vfiler_config_info{dnsdomain} || die "dnsdomain not defined";
	    my $dnsserver = $vfiler_config_info{dnsserver} || die "dnsserver not defined";
	    my $nisdomain = $vfiler_config_info{nisdomain} || die "nisdomain not defined";
	    my $nisserver = $vfiler_config_info{nisserver} || die "nisserver not defined";
	    my $vfiler_password = $vfiler_config_info{vfiler_password} || die "vfiler_password not defined";
            vfiler_create($fc_telnet, $vfiler_name,
                if_to_use=>"$if_to_use",
                netmask=>"$netmask",
                dnsdomain=>"$dnsdomain",
                dnsserver=>"$dnsserver",
                nisdomain=>"$nisdomain",
                nisserver=>"$nisserver",
                vfiler_password=>"$vfiler_password",
                ipaddr_paths=>"-i $vfiler_ip_addrs[$x-1] /vol/${vfiler_vol_name}/${vfiler_qtree_name}",
                no_cifs=> "1",
                count=>1) or die "Vfiler not created on filer";

            %vfiler_status = vfiler_status($fc_telnet, $vfiler_name, option=>"-r");
        }

        logcomment("vfiler ${vfiler_name} status is ". $vfiler_status{$vfiler_name}{'status'});
        if ($vfiler_status{$vfiler_name}{'status'} != /running/) {
            logcomment("Starting vfiler ". $vfiler_status{$vfiler_name}{'status'});
            vfiler_start($fc_telnet, $vfiler_name) ||
                die "Failed to start vfiler ${vfiler_name}";
        }
	# Enable ZAPI on vFiler (using ZAPI - strange isn't it?)
        my $apiset = NACL::APISet->new(	'connection' => $fc,
						'category' => 'Node',
						'interface' => 'CLI',
						'set' => 'Vfiler',
						'context_definition' => {'name' => $vfiler_name }
						);
	my $cur_vfiler = $apiset->get_vfiler();
	logcomment("Current vFiler is " . $cur_vfiler);
	if($cur_vfiler =~ /^$/){ 
	    die("Unable to set vFiler context for vfiler $vfiler_name"); 
        }
        $response = $apiset->options('httpd.admin.enable' => 'on');

        if($enable_snapvault){
            $response = $apiset->options('snapvault.enable' => 'on');
            $response = $apiset->options('snapvault.access' => 'all');
        }

        # if we're working with the last vfiler, we need to add the remainder
        # vols to it.
        # if ($x == @vfiler_ip_addrs) {
        #    $total_vols_per_vfiler+=$remainder_vols_per_vfiler;
        #}

        my @vfiler_path;
        if(defined($vfiler_status{$vfiler_name}{'Path'})){
            @vfiler_path = @{$vfiler_status{$vfiler_name}{'Path'}};
        } else {
            @vfiler_path = ();
        }

        for(my $y=0; $y <= $resource_vols_per_vfiler; $y++) {
            my $vol_path = shift(@resource_list);
	    logcomment("Resource vol path $y is $vol_path");
            push(@vol_info,{ vol_path=>"$vol_path", vfiler=>"vf${x}", ip=>"$vfiler_ip_addrs[$x-1]"});
            if ("@vfiler_path" !~ /$vol_path/) {
		if($vol_path =~ /^$/){ next; }
		logcomment("Adding path $vol_path to vfiler vf${x}");
                vfiler_add($fc_telnet, "vf${x}", path=>$vol_path) ||
                    die "Failed to add path $vol_path to vfiler vf${x}: ${@}";
                vfiler_run_vfiler_exportfs($fc_telnet, "vf${x}", "$vol_path") ||
                    die "Failed to export path $vol_path on vfiler vf${x}: ${@}";
            }
        }
    }

    disconnect ($fc_telnet);
    return @vol_info;
}

sub create_vfilers_qtrees ($$%) {
    my ($fc, $filer_side, %args) = @_;

    my $filer = $args{'host'} || die "host not defined";
    my $max_qtrees = $args{'max_qtrees'} || die "max_qtrees is not defined";
    my $max_vols = $args{'max_vols'} || die "max_vols is not defined";
    my $aggr = $args{'aggr'} || "test_aggr";
    my $enable_dedup = $args{'enable_dedup'} || 0;
    my $enable_compress = $args{'enable_compress'} || 0;
    if($enable_compress){ $enable_dedup = 1; }
    my $resources = $args{'resources'} || "";
    my @resource_list = ();
    if(ref($resources)){ @resource_list = @$resources; }
    my $vfiler_ips = $args{'vfiler_ips'} || logcomment "vfiler_ips not defined";
    my @vfiler_ip_addrs = @$vfiler_ips;
    my $vfiler_info = $args{'vfiler_info'} || logcomment "vfiler_info not defined";
    my $vfiler_rootvol_size = $args{'vol_size'} || "1g";
    my %vfiler_config_info = %$vfiler_info;

    my @vol_info = ();

    # Check @vfiler_ip_addrs against max vfilers, raise limit if necessary
    my $vfiler_limit = vfiler_limit($fc);
    my $vfiler_hard_limit = vfiler_limit_hard_limit($fc);
    if(@vfiler_ip_addrs > $vfiler_limit){
        if(@vfiler_ip_addrs < $vfiler_hard_limit){
             vfiler_limit_set($fc, @vfiler_ip_addrs);
        } else {
	     # Set vfilers to max, reduce @vfiler_ip_addrs to fit
	     logcomment("Reducing number vfiler ip addresses to hard limit");
	     vfiler_limit_set($fc, $vfiler_hard_limit);		
	     @vfiler_ip_addrs = @vfiler_ip_addrs[0 .. ($vfiler_hard_limit -1)];
	}
    }

    # Assert num_vfilers <= max_qtrees
    if(@vfiler_ip_addrs > $max_qtrees){
        die("Number of vFilers requested was greater than number of qtrees requested");
    }

    # Create vfilers, vols, and vfiler data qtrees on source and destination
    logcomment ("Create the Vfilers on $filer");
    host ($filer);
    my $fc_telnet = connect("#telnet", -timeout, 10);
    my $vol_name;
    my $vol_number=0;

    # Create vols
    for(my $x=1; $x <= $max_vols; $x++) {
        $vol_number = $x % @vfiler_ip_addrs;
        if($x <= @vfiler_ip_addrs){
	    # Create a root vol
	    $vol_name = "vf${vol_number}_root";
        } else {
 	    # Create a data vol
	    $vol_name = "vf${vol_number}_data${x}";
        }
        if (!vol_exists($fc, $vol_name)) {
           vol_create_flex($fc, $vol_name, $aggr, $vfiler_rootvol_size)
           or die "\nFunction Failed: vol_create_flex($fc, $vol_name, $aggr, 1g)\n$@";
        }
        if ($enable_dedup) {
	   dense_toggle(host=>$filer, path_name=>"/vol/${vol_name}", state=>"on");
	   dense_start(host=>$filer, path_name=>"/vol/${vol_name}");
        }
        if ($enable_compress) {
	   filer_vol_option_set($fc, $vol_name, 'compression', 'on');
	   filer_vol_compress_start($fc, $vol_name);
        }
    } # END FOR VOLS

    # Create qtrees and assign to vFilers
    my $vfiler_number;
    my $qtree_path;
    for(my $y=1; $y <= $max_qtrees; $y++){
        $vfiler_number = $y % @vfiler_ip_addrs; 
	$vol_number = $y % $max_vols;
        if($y <= @vfiler_ip_addrs){
 	    # Create vFiler root qtree on vFiler "root" vol
            $qtree_path = "/vol/vf${vfiler_number}_root/vf${vfiler_number}_qtree${y}";
            if (!qtreePresent($fc, $qtree_path)){
                qtreeCreate($fc, $qtree_path);        
            }
            # Create vFiler
            my %vfiler_status = vfiler_status($fc_telnet, "vf${vfiler_number}", option=>"-r");
            if (! exists $vfiler_status{"vf${vfiler_number}"}) {
	        my $if_to_use = $vfiler_config_info{if_to_use} || die "if_to_use not defined";
	        my $netmask   = $vfiler_config_info{netmask}   || die "netmask not defined";
	        my $dnsdomain = $vfiler_config_info{dnsdomain} || die "dnsdomain not defined";
	        my $dnsserver = $vfiler_config_info{dnsserver} || die "dnsserver not defined";
	        my $nisdomain = $vfiler_config_info{nisdomain} || die "nisdomain not defined";
	        my $nisserver = $vfiler_config_info{nisserver} || die "nisserver not defined";
	        my $vfiler_password = $vfiler_config_info{vfiler_password} || die "vfiler_password not defined";

                vfiler_create($fc_telnet, "vf${vfiler_number}",
                    if_to_use=>"$if_to_use",
                    netmask=>"$netmask",
                    dnsdomain=>"$dnsdomain",
                    dnsserver=>"$dnsserver",
                    nisdomain=>"$nisdomain",
                    nisserver=>"$nisserver",
                    vfiler_password=>"$vfiler_password",
                    ipaddr_paths=>"-i " . $vfiler_ip_addrs[$vfiler_number - 1] . " " . $qtree_path,
                    no_cifs=> "1",
                    count=>1) or die "Vfiler not created on filer: $@";

                    %vfiler_status = vfiler_status($fc_telnet, "vf${vfiler_number}", option=>"-r");

                    logcomment("vfiler vf${vfiler_number} status is ". $vfiler_status{"vf${vfiler_number}"}{'status'});
                    if ($vfiler_status{"vf${vfiler_number}"}{'status'} != /running/) {
                        logcomment("Starting vfiler ". $vfiler_status{"vf${vfiler_number}"}{'status'});
                        vfiler_start($fc_telnet, "vf${vfiler_number}") ||
                            die "Failed to start vfiler vf${vfiler_number}";
                    }

#                    vfiler_options_set($fc_telnet, "vf${vfiler_number}", 'snapmirror.enable', 'on');
#                    vfiler_options_set($fc_telnet, "vf${vfiler_number}", 'snapmirror.access', 'all');
                    vfiler_options_set($fc_telnet, "vf${vfiler_number}", 'httpd.admin.enable', 'on');
                } # End if not exists vfiler

	        push(@vol_info,{ vol_path => $qtree_path, 
				vfiler    => "vf${vfiler_number}", 
				ip        => "$vfiler_ip_addrs[$vfiler_number - 1]" });
              
        } else {

            # vFiler data qtree on data vol
	    $qtree_path = "/vol/vf${vfiler_number}_data${vol_number}/vf${vfiler_number}_qtree${y}";
            if (!qtreePresent($fc, $qtree_path)){
                qtreeCreate($fc, $qtree_path);        
            }

            # Add qtree to current vfiler
            vfiler_add($fc_telnet, "vf${vfiler_number}", {'path' => $qtree_path});
	    die($@) if $@;

	    push(@vol_info,{ vol_path => $qtree_path, 
			     vfiler   => "vf${vfiler_number}", 
			     ip       => "$vfiler_ip_addrs[$vfiler_number - 1]" });
        } # END IF(qtree_root)

    } # END FOR y = qtree

#    # Add "resource" paths to first vfiler (it's arbitrary)
#    foreach my $p (@resource_list){
#	$p =~ s#/vol/(\w+).*#$1#;
#        # If path is not already part of vFiler, add and export
#        # Get vfiler paths 
#        my %vf_status = vfiler_status($fc_telnet, "vf${vfiler_number}", option=>"-r");
#	        if ("@vfiler_path" !~ /$vol_path/) {
#	 	    logcomment("Adding path /vol/$vol to vfiler vf${x}");
#		    vfiler_add($fc_telnet, "vf${x}", path=>"/vol/$vol") ||
#		        die "Failed to add path /vol/$vol to vfiler vf${x}";
#		    vfiler_run_vfiler_exportfs($fc_telnet, "vf${x}", "/vol/$vol") ||
#		        die "Failed to export path /vol/$vol on vfiler vf${x}";
#	    push(@vol_info,{ vol_path=>"$vol_path", vfiler=>"vf${vfiler_number}", ip=> $vfiler_ip_addrs[$vfiler_number - 1] });
#        }
#     }

    disconnect ($fc_telnet);
    return @vol_info;
}

sub create_vfilers_qtrees_src ($%) {
    my ($fc, %args) = @_;

    my @vol_info = create_vfilers_qtrees($fc, "src", %args);
    return @vol_info;
}

sub create_vfilers_qtrees_dst ($%) {
    my ($fc, %args) = @_;

    my @vol_info = create_vfilers_qtrees($fc, "dst", %args);
    return @vol_info;
}

sub create_vfilers_src ($%) {
    my ($fc, %args) = @_;

    my @vol_info = create_vfilers($fc, "src", %args);
    return @vol_info;
}

sub create_vfilers_dst ($%) {
    my ($fc, %args) = @_;

    my @vol_info = create_vfilers($fc, "dst", %args);
    return @vol_info;
}

sub filer_aggr_exists($$){
    my $zapi_apiset = shift;
    my $aggr = shift;
    my $response = $zapi_apiset->aggr_list_info('aggregate' => $aggr);
    if(not(defined($response) and ($response->{'execution_status'} eq 'passed'))){
        $Log->comment("filer_aggr_exists: returning undef");
        return undef;
    }
    my $output_ref = $response->get_parsed_output();
    $Log->comment(Data::Dumper->Dump([ $output_ref->[0]{'aggregates'}[0]{'    aggr-info'} ]));
    my @aggr_info = @{$output_ref->[0]{'aggregates'}[0]{'aggr-info'}};
    foreach my $ai (@aggr_info){
        $Log->comment(Data::Dumper->Dump([ $ai ]));
        if($ai->{'name'} eq $aggr){
            $Log->comment("Aggr $aggr exists!");
            return 1;
        }
    }
    return 0;
}

sub filer_vol_exists($$){
    my $zapi_apiset = shift;
    my $vol = shift;
    my $response;
    eval {
        $response = $zapi_apiset->volume_list_info('volume' => $vol);
    };
    if(not(defined($response) and ($response->{'execution_status'} eq 'passed'))){
        $Log->comment("filer_vol_exists: returning undef");
        return undef;
    }
    my $output_ref = $response->get_parsed_output();
    $Log->comment(Data::Dumper->Dump([ $output_ref->[0]{'volumes'}[0]{'volume-info'} ]));
    my @vol_info = @{$output_ref->[0]{'volumes'}[0]{'volume-info'}};
    foreach my $vi (@vol_info){
        $Log->comment(Data::Dumper->Dump([ $vi ]));
        if($vi->{'name'} eq $vol){
            $Log->comment("Volume $vol exists!");
            return 1;
        }
    }
    return 0;
}

sub filer_service_is_licensed($$){
    my $zapi_apiset = shift;
    my $service = shift;
    my $response;
    my @license_info;
    $response = $zapi_apiset->license_list_info();
    if(not(defined($response) and $response->{'execution_status'} eq 'passed    ')){
        return undef;
    }
    my $output_ref = $response->get_parsed_output();
    # $Log->comment(Data::Dumper->Dump([ $output_ref->[0]{'licenses'}[0]{'li    cense-info'} ]));
    @license_info = @{$output_ref->[0]{'licenses'}[0]{'license-info'}};
    foreach my $li (@license_info){
        $Log->comment(Data::Dumper->Dump([ $li ]));
        if($li->{'service'} eq $service){
            if($li->{'is-licensed'} eq 'true'){
                return 1;
            }
        }
    }
    return 0;
}

sub snapmirror_initialize_destination_vfiler ($%) {
    my ($fc, %args) = @_;

    my $filer_src = $args{'host_src'} || die "host_src not defined";
    my $filer_dst = $args{'host_dst'} || die "host_dst not defined";
    my $timeout = $args{'timeout'} || 300;
    my $vol_info = $args{'vol_info'} || die "vols not defined";
    my @volume_list = @$vol_info;
    my %hash_of_vfiler_info=();
    my $is_qtree = 0;

    logcomment("Filer_src is $filer_src");
    logcomment("Filer_dst is $filer_dst");

    for(my $x=0; $x<@volume_list; $x++) {
	my $is_destination = 0;
	my $vfiler_dst = $volume_list[$x]{'vfiler'};
        my $vfiler_src = $vfiler_dst;
        if($vfiler_dst =~ /$filer_src/){
	    $vfiler_dst =~ s/$filer_src/$filer_dst/g;
        } else {
            $vfiler_src =~ s/$filer_dst/$filer_src/g;
        }
	my $vol = $volume_list[$x]{'vol_path'};
	my $src_vol;
	my $dst_vol;

	if ($vol =~ /\/vol\/\w+\/\w+/) { # Qtree
	    logcomment("$vol is qtree");
	    $src_vol = $vol;
	    $dst_vol = $vol;
	    $src_vol =~ s#dst#src#g;
	    $dst_vol =~ s#src#dst#g;
	    $is_qtree = 1;
	} else {
	    logcomment("$vol is volume");
	    $src_vol = $vol;
	    $dst_vol = $vol;
	    $src_vol =~ s#dst#src#;
	    $dst_vol =~ s#src#dst#;
	    $src_vol =~ s#/vol/##;
	    $dst_vol =~ s#/vol/##;
	}

	my (undef, undef, $sm_status_out) = say($fc, "vfiler run $vfiler_dst snapmirror status");
	foreach my $line (split(/\n/,$sm_status_out)) {
	    next if $line !~ /Source|Snapmirror/;
	    if ($line =~ /${vfiler_dst}:${dst_vol}\s+Snapmirrored/) {
		$is_destination = 1;
	    }
	}

	if ( ! $is_destination ) {
	    if (!$is_qtree) {
		vol_restrict($fc, "$dst_vol")
		    or die "\nFunction Failed: vol_restrict($fc, ${dst_vol}\n$@";
	    }

	    my $retry=1;
	    while ($retry) { 
		my (undef, undef, $sm_init_out) = say($fc, "vfiler run ${vfiler_dst} snapmirror initialize -w -S ${vfiler_src}:${src_vol} ${vfiler_dst}:${dst_vol}", -timeout => $timeout);
		if ($sm_init_out =~ /Transfer successful/) {
		    $retry = 0;
		} elsif ($sm_init_out =~ /busy/) {
		    logresult("WARN", "Busy Transfers: $sm_init_out", " Retrying");
		    sleep 5;
		} else {
		    logresult("FATAL", "Unable to initialize SnapMirror Destination.  Error is $sm_init_out");
		}
	    }

	    # Add to snapmirror.conf info
	    push(@{$hash_of_vfiler_info{"$vfiler_dst"}}, {
		    source_path             => "${vfiler_src}:${src_vol}",
		    destination_path        => "${vfiler_dst}:${dst_vol}",
		    });
	} else {
	    logcomment("${vfiler_dst}:${dst_vol} is already a snapmirror destination. Skipping initialization");

	    # Add to snapmirror.conf info
	    push(@{$hash_of_vfiler_info{"$vfiler_dst"}}, {
		    source_path             => "${vfiler_src}:${src_vol}",
		    destination_path        => "${vfiler_dst}:${dst_vol}",
		    });
	}
    }
    return %hash_of_vfiler_info;
}

sub snapmirror_initialize_destination ($%) {
    my ($fc, %args) = @_;

    my $filer_src = $args{'host_src'} || die "host_src not defined";
    my $filer_dst = $args{'host_dst'} || die "host_dst not defined";
    my $vol_info = $args{'vol_info'} || die "vols not defined";
    my $src_ip = $args{'src_ip'};
    my @volume_list = @$vol_info;
    my %hash_of_sm_info=();
    my $fh = host($filer_dst);
    my $srcfh = host($filer_src);
    my $is_qtree = 0;
    my $filer_newsrc;

    for(my $x=0; $x<@volume_list; $x++) {

	my $vol = $volume_list[$x]{'vol_path'};
	my $src_vol;
	if ($vol =~ /\/vol\/\w+\/\w+/) { # Qtree
	    logcomment("$vol is qtree");
	    $src_vol = $vol;
	    $src_vol =~ s#dst#src#g;
	    $is_qtree = 1;
	} else {
	    logcomment("$vol is volume");
	    $vol =~ s#/vol/##;
	    $src_vol = $vol;
	    $src_vol =~ s#dst#src#;
	}

	if ($src_ip) {
           $filer_newsrc=$src_ip;
        } else {
           $filer_newsrc=$filer_src;
        }


	my $init_status=0;
	if ($is_qtree) {
	    my @destinations = snapmirror_destinations(host=>$srcfh, 
			srcpath=>"${filer_newsrc}:$src_vol");
	    for (my $x=0; $x<@destinations; $x++) {
		for (my $y=0; $y<@{$destinations[$x]}; $y++) {
		    if ($destinations[$x][$y]{'destination-location'} =~ /$vol/) {
			$init_status=1;
		    }
		}
	    }
	} else {
	    my $check_status = snapmirror_status_vol(host=>$fh, vol=>"$vol");
	    if ( $check_status->{'is-destination'} =~ /true/) {
		$init_status=1
	    } else {
		vol_restrict($fc, "$vol")
		    or die "\nFunction Failed: vol_restrict($fc, $vol\n$@";
	    }
	}

	if (! $init_status) {
	    my $retry=1;

	    while ($retry == 1) {
		 eval {
			snapmirror_initialize(host=>$fh, 
				dst=>"${filer_dst}:$vol",
				src=>"${filer_newsrc}:$src_vol",
				parallel=>1, timeout=>900
				);
		      };

		if ($@) {
		    if ($@ =~ /transfer attempted for busy destination/) {
			logresult("WARN", "Error is $@");
			$retry=1;
		    } else {
			logresult("FATAL", "Error is $@");
			$retry=0;
		    } 
		} else {
		    $retry=0;
		}
	    } 
	    # Add to snapmirror.conf info
	    push(@{$hash_of_sm_info{"$filer_dst"}}, {
		    source_path             => "${filer_newsrc}:${src_vol}",
		    destination_path        => "${filer_dst}:${vol}",
		    });
	} else {
	    logcomment("$vol is already a snapmirror destination. Skipping initialization");
	    # Add to snapmirror.conf info
	    push(@{$hash_of_sm_info{"$filer_dst"}}, {
		    source_path             => "${filer_newsrc}:${src_vol}",
		    destination_path        => "${filer_dst}:${vol}",
		    });
	}
    }
    return %hash_of_sm_info;
}

sub snapmirror_create_cmode ($%) {
    my ($fc, %args) = @_;

    my $return = "";
    if($args{'use_zapi'} == 1) {
	$return = snapmirror_create_cmode_zapi($fc,%args)
    } else {
	$return = snapmirror_create_cmode_ssh($fc,%args)
    }

    return $return;
}

sub snapmirror_modify_cmode ($%) {
    my ($fc, %args) = @_;

    my $source_path = $args{'source_path'} || die "source_path not defined";
    my $destination_path = $args{'destination_path'} || "";
    my $schedule = $args{'schedule'} || "";
    my $output = $args{'output'} || "SnapMirror: done";
    my $timeout = $args{'timeout'} || "600";

    my $return = "";
    eval {
	    my @sm_args = ();
	    push(@sm_args, "-source-path $source_path");
	    push(@sm_args, "-destination-path $destination_path");
	    if($schedule ne "") {
		push(@sm_args, "-schedule $schedule");
	    }

	    say($fc,
		"snapmirror modify " .
		"@sm_args",
		$output,
		-timeout, $timeout);
    };

    if ($@) {
	return $@;
    } else {
	return "$output";
    }

}

sub snapmirror_create_cmode_ssh ($%) {
    my ($fc, %args) = @_;

    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $source_path = $args{'source_path'} || die "source_path not defined";
    my $schedule = $args{'schedule'} || "";
    my $output = $args{'output'} || "SnapMirror: done";
    my $destination_path = $args{'destination_path'} || "";
    my $timeout = $args{'timeout'} || "600";

    my $return = "";
    eval {
	    my @sm_args = ();
	    push(@sm_args, "-source-path $source_path");
	    push(@sm_args, "-destination-path $destination_path");
	    push(@sm_args, "-type $sm_type");
	    if($schedule ne "") {
		push(@sm_args, "-schedule $schedule");
	    }
	    say($fc,
		"snapmirror create " .
		"@sm_args",
		$output,
		-timeout, $timeout);
    };

    if ($@) {
	return $@;
    } else {
	return "$output";
    }

}

sub snapmirror_create_cmode_zapi ($%) {
    my ($fc, %args) = @_;

    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $filer = $args{'filer'} || die "filer not defined";
    my $output = $args{'output'} || "SnapMirror: done";
    my $source_path = $args{'source_path'} || die "source_path not defined";
    my $destination_path = $args{'destination_path'} || "";
    my $schedule = $args{'schedule'} || "";
    my $timeout = $args{'timeout'} || "600";
    my $zapi_user = $args{'zapi_user'} || "admin";
    my $zapi_password = $args{'zapi_password'} || "netapp1!";
    my $zapi_relationship_type;
    if ($sm_type =~ /dp/i) {
	$zapi_relationship_type="data_protection";
    } elsif ($sm_type =~ /ls/i) {
	$zapi_relationship_type="load_sharing";
    } else {
    	die "Bad value $sm_type for sm_type";
    }

    my $jobid=0;
    my $server = new ZapiTest($filer, $zapi_user, $zapi_password);
    $server->set_legacy_http_conn(0);
    die "Destination Path not defined" unless ($destination_path ne "");

    my $o;
    if ($schedule ne "") {
	logcomment("sending ontapi $filer $zapi_user $zapi_password snapmirror-create source-location $source_path destination-location $destination_path relationship-type $zapi_relationship_type schedule $schedule");
	$o = $server->Zapi::invoke_api(
	    'snapmirror-create', 
	    'source-location', "$source_path",
	    'destination-location', "$destination_path",
	    'schedule', "$schedule",
	    'relationship-type', "$zapi_relationship_type"
	    );
    } else {
	logcomment("sending ontapi $filer $zapi_user $zapi_password snapmirror-create source-location $source_path destination-location $destination_path relationship-type $zapi_relationship_type");
	$o = $server->Zapi::invoke_api(
	    'snapmirror-create', 
	    'source-location', "$source_path",
	    'destination-location', "$destination_path",
	    'relationship-type', "$zapi_relationship_type"
	    );
    }

    my $reason = $o->results_reason();
    my $return = "";
    if( $o->attr_get("status") ne "passed") {
	die("Zapi request failed to return valid XML (reason: $reason).");
    } else {
	$return = "SnapMirror: done";
    }


    #$jobid = $o->child_get_string("result-jobid");
    #logcomment("jobid is $jobid");
#
#    logcomment("Waiting for job to finish");
#    my $return = "";
#    while (1) {
#	my $conn = connect_ssh_persist("500001",$filer);
#    	my (undef, undef, $show_out) = say($conn, "set diag -confirmations off; rows 0; job show -id $jobid");
#	$conn->destroy();
#	foreach my $line (split(/\n/,$show_out)) {
#	    if ( $line =~ /Success/i) {
#		$return = "SnapMirror: done";
#		last;
#	    } elsif ($line =~ /Failure/i) {
#		$return = "FAILURE: $show_out";
#		last;
#	    } elsif ($line =~ /no entries matching/i) {
#		$return = "FAILURE: $show_out";
#		last;
#	    } else {
#		sleep 10;
#	    }
#	}
#    }

    #my $return = "";
    #while (1) {
#	my $o = $server->Zapi::invoke_api(
#	    'job-get',
#	    'job-id',
#	     $jobid 
#            );
#
#	my $reason = $o->results_reason();
#	if( $o->attr_get("status") ne "passed") {
#	    if ($reason eq "entry doesn't exist") {
#		logresult("WARN", "Job ID $jobid does not exist");
#		# Assume that it already falled off the list and passed
#		$return = "Success";
#		last;
#	    } elsif ($reason =~ /read timeout/) {
#		logresult("WARN", "Timeout checking job $jobid status through zapi");
#		next;
#	    } else {
#		die("Zapi request failed to return valid XML (reason: $reason).");
#	    }
#	}
#
#	my $job_attrs = $o->child_get("attributes");
#	my $job_info = $job_attrs->child_get("job-info");
#	my $jobstate = $job_info->child_get_string("job-state");
#	my $jobstatus = $job_info->child_get_string("job-status-code");
#	my $jobcompletion = $job_info->child_get_string("job-completion");
#	#logcomment("job-state is $jobstate");
#	if ($jobstate =~ /Success|Failure/i) {
#	    $return = "$jobcompletion";
#	    last;
#	} else {
#	    sleep 10;
#	}
#    }
#
    return $return;
}

sub snapmirror_update_cmode ($%) {
    my ($fc, %args) = @_;

    my $return = "";
    if($args{'use_zapi'} == 1) {
	$return = snapmirror_update_cmode_zapi($fc,%args)
    } else {
	$return = snapmirror_update_cmode_ssh($fc,%args)
    }

    return $return;
}

sub snapmirror_update_cmode_ssh ($%) {
    my ($fc, %args) = @_;

    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $source_path = $args{'source_path'} || die "source_path not defined";
    my $output = $args{'output'} || "SnapMirror: done";
    my $destination_path = $args{'destination_path'} || "";
    my $foreground = $args{'foreground'} || "true";
    my $timeout = $args{'timeout'} || "600";

    my $return = "";
    eval {
	if ($sm_type eq "LS") {
	    say($fc,
		"snapmirror update-ls-set " .
		"-source-path $source_path " .
		"-foreground $foreground ",
		$output,
		-timeout, $timeout);
	} else {
	    say($fc,
		"snapmirror update " .
		"-source-path $source_path " .
		"-destination-path $destination_path " .
		"-foreground $foreground ",
		$output,
		-timeout, $timeout);
	}
    };

    if ($@) {
	return $@;
    } else {
	return "$output";
    }

}

sub snapmirror_update_cmode_zapi ($%) {
    my ($fc, %args) = @_;

    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $filer = $args{'filer'} || die "filer not defined";
    my $output = $args{'output'} || "SnapMirror: done";
    my $source_path = $args{'source_path'} || die "source_path not defined";
    my $destination_path = $args{'destination_path'} || "";
    my $foreground = $args{'foreground'} || "true";
    my $timeout = $args{'timeout'} || "600";
    my $zapi_user = $args{'zapi_user'} || "admin";
    my $zapi_password = $args{'zapi_password'} || "netapp1!";
    my $conn_id = $args{'conn_id'} || 500000;

    my $jobid=0;
    my $server = new ZapiTest($filer, $zapi_user, $zapi_password);
    $server->set_legacy_http_conn(0);
    if ($sm_type eq "LS") {
	my $o = $server->Zapi::invoke_api(
	    'snapmirror-update-ls-set', 
            'source-location', $source_path,
            );
	my $reason = $o->results_reason();
	if( $o->attr_get("status") ne "passed") {
	    die("Zapi request failed to return valid XML (reason: $reason).");
	}
	$jobid = $o->child_get_string("result-jobid");
    } else {
	die "Destination Path not defined" unless ($destination_path ne "");
	UPDATE:
	my $o = $server->Zapi::invoke_api(
	    'snapmirror-update', 
            'source-location', "$source_path",
            'destination-location', "$destination_path"
            );
	my $reason = $o->results_reason();
	if( $o->attr_get("status") ne "passed") {
	    if ($reason =~ /read timeout/) {
		logresult("WARN", "Timeout updating snapmirror through zapi");
		sleep 5;
		goto UPDATE;
	    } else {
		die("Zapi request failed to return valid XML (reason: $reason).");
	    }
	}
	$jobid = $o->child_get_string("result-jobid");
    }

    logcomment("Waiting for job to finish");
    my $return = "";

    my $conn = connect_ssh_persist($conn_id,$filer);
    while ($return eq "") {
    	my (undef, undef, $show_out) = say($conn, "set diag -confirmations off; rows 0; job show $jobid");
	foreach my $line (split(/\n/,$show_out)) {
	    if ( $line =~ /Success/i) {
		logcomment("Job Succeeded");
		$return = "SnapMirror: done";
	    } elsif ($line =~ /Failure/i) {
		$return = "FAILURE: $show_out";
	    } elsif ($line =~ /no entries matching/i) {
		# Check history in case it fell off the queue.
		my (undef, undef, $history_show_out) = say($conn, "set diag -confirmations off; rows 0; job history show -id $jobid");
		foreach my $history_line (split(/\n/,$history_show_out)) {
		    if ( $history_line =~ /Succeeded/i) {
			logcomment("Job Succeeded");
			$return = "SnapMirror: done";
		    } else {
			$return = "FAILURE: $history_show_out";
		    }
		}
	    } else {
		sleep 10;
	    }
	}
    }
    $conn->destroy();


    # the job-get zapi was removed so this section was commented out in favor of ssh
    #while (1) {
#	my $o = $server->Zapi::invoke_api(
#	    'job-get',
#	    'job-id',
#	     $jobid 
#            );
#
#	my $reason = $o->results_reason();
#	if( $o->attr_get("status") ne "passed") {
#	    if ($reason eq "entry doesn't exist") {
#		logresult("WARN", "Job ID $jobid does not exist");
#		# Assume that it already falled off the list and passed
#		$return = "Success";
#		last;
#	    } elsif ($reason =~ /read timeout/) {
#		logresult("WARN", "Timeout checking job $jobid status through zapi");
#		next;
#	    } else {
#		die("Zapi request failed to return valid XML (reason: $reason).");
#	    }
#	}
#
#	my $job_attrs = $o->child_get("attributes");
#	my $job_info = $job_attrs->child_get("job-info");
#	my $jobstate = $job_info->child_get_string("job-state");
#	my $jobstatus = $job_info->child_get_string("job-status-code");
#	my $jobcompletion = $job_info->child_get_string("job-completion");
#	#logcomment("job-state is $jobstate");
#	if ($jobstate =~ /Success|Failure/i) {
#	    $return = "$jobcompletion";
#	    last;
#	} else {
#	    sleep 10;
#	}
#    }

    return $return;
}

sub snapmirror_check_cmode ($%) {
    my ($fc, %args) = @_;

    my $return = "";
    if($args{'use_zapi'} == 1) {
	$return = snapmirror_check_cmode_zapi($fc,%args)
    } else {
	$return = snapmirror_check_cmode_ssh($fc,%args)
    }

    return $return;
}

sub snapmirror_check_cmode_ssh ($%) {
    my ($fc, %args) = @_;

    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $source_path = $args{'source_path'} || die "source_path not defined";
    my $output = $args{'output'} || "SnapMirror: done";
    my $destination_path = $args{'destination_path'} || "";
    my $foreground = $args{'foreground'} || "true";
    my $timeout = $args{'timeout'} || "600";

    eval {
	say($fc,
	    "set diag -confirmations off; snapmirror check " .
	    "-source-path $source_path " .
	    "-destination-path $destination_path " .
	    "-foreground $foreground ",
	    $output,
	    -timeout, $timeout);
    };

    if ($@) {
	logcomment("Returning $@");
	return $@;
    } else {
	logcomment("Returning $output");
	return "$output";
    }
}

sub snapmirror_check_cmode_zapi ($%) {
    my ($fc, %args) = @_;

    my $sm_type = $args{'sm_type'} || die "sm_type not defined";
    my $filer = $args{'filer'} || die "filer not defined";
    my $source_path = $args{'source_path'} || die "source_path not defined";
    my $destination_path = $args{'destination_path'} || "";
    my $foreground = $args{'foreground'} || "true";
    my $timeout = $args{'timeout'} || "600";
    my $zapi_user = $args{'zapi_user'} || "admin";
    my $zapi_password = $args{'zapi_password'} || "netapp1!";

    my $jobid=0;
    my $server = new ZapiTest($filer, $zapi_user, $zapi_password);
    $server->set_legacy_http_conn(0);

    # Run zapi and get jobid if it passes
    my $o = $server->Zapi::invoke_api(
	'snapmirror-check', 
	'source-location', "$source_path",
	'destination-location', "$destination_path"
	);
    my $reason = $o->results_reason();
    if( $o->attr_get("status") ne "passed") {
	die("Zapi request failed to return valid XML (reason: $reason).");
    }
    $jobid = $o->child_get_string("result-jobid");

    # keep checking jobid for success or failure
    logcomment("Waiting for job to finish");
    my $return = "";
    while (1) {
	my $conn = connect_ssh_persist("500002",$filer);
    	my (undef, undef, $show_out) = say($conn, "set diag -confirmations off; rows 0; job show $jobid");
	$conn->destroy();
	foreach my $line (split(/\n/,$show_out)) {
	    if ( $line =~ /Success/i) {
		$return = "SnapMirror: done";
		last;
	    } elsif ($line =~ /Failure/i) {
		$return = "FAILURE: $show_out";
		last;
	    } elsif ($line =~ /no entries matching/i) {
		$return = "FAILURE: $show_out";
		last;
	    } else {
		sleep 10;
	    }
	}
    }

    #my $return;
    #while (1) {
#	my $o = $server->Zapi::invoke_api(
#	    'job-get',
#	    'job-id',
#	     $jobid 
#            );
#
#	my $reason = $o->results_reason();
#	if( $o->attr_get("status") ne "passed") {
#	    if ($reason eq "entry doesn't exist") {
#		logresult("FATAL", "Job ID $jobid does not exist");
#		# Assume that it already falled off the list and passed
#	    } elsif ($reason =~ /read timeout/) {
#		logresult("WARN", "Timeout checking job $jobid status through zapi");
#		next;
#	    } else {
#		die("Zapi request failed to return valid XML (reason: $reason).");
#	    }
#	    last;
#	}
#
#	my $job_attrs = $o->child_get("attributes");
#	my $job_info = $job_attrs->child_get("job-info");
#	my $jobstate = $job_info->child_get_string("job-state");
#	my $jobstatus = $job_info->child_get_string("job-status-code");
#	my $jobcompletion = $job_info->child_get_string("job-completion");
##	if ($jobstate =~ /Success|Failure/i) {
#	    $return = "$jobcompletion";
#	    last;
#	} else {
#	    sleep 10;
#	}
#    }
#
    return $return;

}

sub run_io ($%) {
    my ($client_to_use, %args) = @_;
    my $io_tool       = $args{'io_tool'};
    my $path          = $args{'mntpnt'};
    my $args          = $args{'fops_params'};

    my @cmd_args      = split(/\s+/, $args);
    push(@cmd_args, "-X 0");
    my $cmd_to_use = "";
    my $wait_str = "";

    if ($io_tool =~ /^\/.*fops/) {
        logcomment("Running FOPS");
        $cmd_to_use = $io_tool;
        $wait_str = "All operations completed successfully";
    } elsif ($io_tool =~ /fops/) {
        logcomment("Running FOPS");
        $cmd_to_use = "/usr/software/test/bin/fops";
        $wait_str = "All operations completed successfully";
    }

    my $cmd = join(" ", $cmd_to_use, $args, $path);


    logcomment("command is $cmd_to_use @cmd_args $path");
    my $result = subtest("$cmd_to_use", -client, $client_to_use, -iffatal, -fail, "$cmd_to_use failed", "--", @cmd_args, "$path");
    if ($result->results() != 0) {
        logresult("FATAL", "$cmd_to_use command failed");
    } else {
        logcomment("$cmd_to_use command passed");
    }

}

########################################################################
# Below is just notes on how to do things if I need to add to the script
# at some point
# To close connection handles to avoid the case where output from multiple
# sessions corrupt each other.  By doing the following, it will force a new
# connection each time.
#
#	foreach my $node_conn (@nodes) {
#		$server_obj->set_execution_server( -Server => $node_conn );
#		$server_obj->handle_connections( close => [qw(cli_connection)] );
#	}
########################################################################

sub add_vfilers_qtrees ($$%) {
    my ($fc, %args) = @_;

    my $aggr = $args{'aggr'} || 'test_aggr';
    my $debug = $args{'debug'} || 1;
    my $filer = $args{'host'} || die "host not defined";
    my $num_qtrees = $args{'num_qtrees'} || die "num_qtrees is not defined";
    my $volname_fmt = $args{'volname_fmt'} || 'vf%d_data';
    my $vol_size = $args{'vol_size'} || '1g';
    my $vfiler_ips = $args{'vfiler_ips'} || logcomment "vfiler_ips not defined";
    my @vfiler_ip_addrs = ();
    if(ref($vfiler_ips)){ @vfiler_ip_addrs = @$vfiler_ips; }
    my $num_vfilers = scalar(@vfiler_ip_addrs);
    my $num_vols = $args{'num_vols'} || $num_vfilers; # One data vol per vfiler
    my $vfiler_name_fmt = $args{'vfiler_name_fmt'} || 'vf%d';

    my @vol_info = ();

    # Create vfiler vols and data qtrees on source and destination
    host ($filer);
    my $fc_telnet = connect("#telnet", -timeout, 10);

    # Create one data vol per vfiler
    # Create (num_qtrees / vol) qtrees in each volume and add to vfiler 
    
    for(my $x=0; $x < $num_qtrees; $x++) {
	my $vfiler_num = ($x % $num_vfilers) + 1;
        my $vol_num = ($x % $num_vols) + 1; 
        my $qtr_num = int($x / $num_vols) + 1; 
	my $vfiler_name = sprintf($vfiler_name_fmt, $vfiler_num);
	my $vol_name = sprintf($volname_fmt, $vol_num);

        if($debug){ 
            logcomment("add_vfilers_qtrees: Num. Vols = $num_vols");
            logcomment("add_vfilers_qtrees: Num. Qtrees = $num_qtrees");
            logcomment("add_vfilers_qtrees: Cur. VFiler number = $vfiler_num");
            logcomment("add_vfilers_qtrees: Cur. Vol number = $vol_num");
            logcomment("add_vfilers_qtrees: Cur. Qtree number = $qtr_num");
        }

 	# Create a data vol
        if (!vol_exists($fc, $vol_name)) {
           vol_create_flex($fc, $vol_name, $aggr, $vol_size)
           or die "\nFunction Failed: vol_create_flex($fc, $vol_name, $aggr, $vol_size)\n$@";
        }

        # Create qtrees and assign to vFilers

        # vFiler data qtree on data vol
        my $qtree_path = "/vol/${vol_name}/${vfiler_name}_qtree${qtr_num}";
        if (!qtreePresent($fc, $qtree_path)){
            qtreeCreate($fc, $qtree_path);        
        }

        # Add qtree to current vfiler
        vfiler_add($fc_telnet, "vf${vol_num}", 'path' => $qtree_path);
        die($@) if $@;

        vfiler_run_vfiler_exportfs($fc, "vf${vol_num}", $qtree_path)
	    or die("Failed to export path ${qtree_path} on vfiler vf${vol_num}");

        push(@vol_info,{ vol_path => $qtree_path, 
			     vfiler   => "vf${vol_num}", 
			     ip       => "$vfiler_ip_addrs[$vol_num - 1]" 
			    });

    } # END FOR VOLS/VFILERS

    disconnect ($fc_telnet);
    return @vol_info;
}


sub add_vfilers_vols ($%) {
    my ($fc, %args) = @_;

    my $filer = $args{'host'} || die "host not defined";
    my $num_vols = $args{'num_vols'} || die "num_vols is not defined";
    $Log->comment("Args of aggr is " . $args{'aggr'});
    my $aggr = $args{'aggr'} || "test_aggr";
    my $debug = $args{'debug'} && 1;
    my $enable_dedup = $args{'enable_dedup'} || 0;
    my $enable_compress = $args{'enable_compress'} || 0;
    if($enable_compress){ $enable_dedup = 1; }
    my $export_vols;
    if(defined($args{'export_vols'}) and exists($args{'export_vols'})){
        $export_vols = $args{'export_vols'};
    } else {
        $export_vols = 1;
    }
    my $share_vols = $args{'share_vols'} || 0;
    my $vfiler_ips = $args{'vfiler_ips'} || logcomment "vfiler_ips not defined";
    my @vfiler_ip_addrs = ();
    if(ref($vfiler_ips)){ @vfiler_ip_addrs = @$vfiler_ips; }
    my $num_vfilers = scalar(@vfiler_ip_addrs);
    my $vfiler_name_fmt = $args{'vfiler_name_fmt'} || 'vf%d';
    my $vol_name_fmt = $args{'vol_name_fmt'} || 'vf%d_data';
    my $vol_size = $args{'vol_size'};
    my $vols_per_vfiler = int($num_vols / $num_vfilers);

    my @vol_info = ();

    # Create vfiler vols and data qtrees on source and destination
    my $fh_telnet = host ($filer);
    my $fc_telnet = connect("#telnet", -timeout, 10);
    my $filer_ontap_version = get_ontap_version($fh_telnet);
    my $vfiler_num = 0;
    my $vol_name;
    my $vol_number=0;

    for(my $vol_num=0; $vol_num < $num_vols; $vol_num++) {
        # $vol_num = ($x % $num_vols) + 1; 
	$vol_name = sprintf($vol_name_fmt, $vol_num);
 	$vfiler_num = ($vol_num % $vols_per_vfiler) + 1;

        if($debug){ 
            logcomment("add_vfilers_vols: Num. Vols = $num_vols");
  	    logcomment("add_vfilers_vols: Num. vFilers = $num_vfilers");
            logcomment("add_vfilers_vols: Cur. Vol number = $vol_num");
            # logcomment("add_vfilers_vols: Cur. vFiler number = $qtr_num");
        }

 	# Create a data vol
        if (!vol_exists($fc, $vol_name)) {
           vol_create_flex($fc, $vol_name, $aggr, $vol_size)
           or die "\nFunction Failed: vol_create_flex($fc, $vol_name, $aggr, $vol_size)\n$@";
        }
        if ($enable_compress) {
	   filer_vol_option_set($fc, $vol_name, 'compression', 'on');
	   # In BR, use 'vol compress', in RR use 'sis start'
 	   if(version_compare('8.0', $filer_ontap_version) > -1){
	       filer_vol_compress_start($fc, $vol_name);
	   }
        }
        if ($enable_dedup) {
	   dense_toggle(host=>$filer, path_name=>"/vol/${vol_name}", state=>"on");
	   dense_start(host=>$filer, path_name=>"/vol/${vol_name}");
        }

        # Add vol to current vfiler
        my $vfiler_name = sprintf($vfiler_name_fmt, $vol_num);
        vfiler_add($fc_telnet, $vfiler_name, 'path' => '/vol/' . $vol_name);
        die($@) if $@;
 
        if($export_vols){
            vfiler_run_vfiler_exportfs($fc, $vfiler_name, '/vol/' . $vol_name)
	    or die("Failed to export path /vol/${vol_name} on vfiler ${vfiler_name}");
        }

        if($share_vols){
            my $vf_zapi_apiset = NACL::APISet->new(
				'connection' => $fc,
				'category' => 'Node',
				'interface' => 'ZAPI',
				'set' => 'Vfiler',
				'context_definition' => {'name' => $vfiler_name }
            );
	    my $response = $vf_zapi_apiset->cifs_share_add(
					'path' => '/vol/' . $vol_name,
					'share-name' => $vol_name
 	    ); 
	    if(not(defined($response) and $response->{'execution_status'} eq 'passed')){
		die(Data::Dumper->Dump( $response ));
            }
	}
        push(@vol_info,{ vol_path => '/vol/' . $vol_name, 
			     vfiler   => $vfiler_name, 
			     ip       => "$vfiler_ip_addrs[$vol_num - 1]" 
			    });

    } # END FOR VOLS/VFILERS

    disconnect ($fc_telnet);
    return @vol_info;
}

sub _get_sm_state_for_location($$){
    my $sm_statii = shift;
    my $location = shift;
    for(my $i = 0; $i <= $#{$sm_statii}; $i++){
        if(($sm_statii->[$i]{'source-location'} eq $location) or
            ($sm_statii->[$i]{'destination-location'} eq $location)){
            return $sm_statii->[$i]{'state'};
        }
     }
    # Error: did not find location
    return undef;
}

1;
