## Stress Scalability License Library
##
## @author      dascenj@netapp.com
##
## @pod here

=head1 NAME

    SSQA::License - Stress Scalability License Library for ONTAP

=head1 SYNOPSIS

    A central place to handle simple licensing tasks

=head1 DESCRIPTION

    my $localResult = SSQA::License::add(command_interface => $node, service => "NFS");

    This library is based on NACL, and requires a NACL node object to be
    supplied. (See NACL docs for more information). The service parameter
    tells the library which service to add a license for. All licenses are
    maintained within this library for centralized storing.

=head1 TODO

    Needs to have the licenses filled in for 7mode.

=head1 BUGS

    Some probably exist but have not been listed yet.

=head1 STATUS

  Shared

=head1 AUTHOR

  Jd (dascenj@netapp.com)

=head1 COPYRIGHT

Copyright 2010, Network Appliance., All Rights Reserved


=cut

package SSQA::License;

use NATE::BaseException qw(:try);
use NACL::CS::License;
use NACL::STask::SystemLicense;
use Params::Validate qw(validate validate_with OBJECT SCALAR ARRAYREF HASHREF BOOLEAN UNDEF);
use Tharn qw ( $Log config_result );

use strict;
use warnings;

my $clusterLic = {
    '1-80-000011' => {
                     NFS => "BAKJZZGKWVMADF",
                     CIFS => "RQQQXZGKWVMADF",
                     GX_Base => "VMZIVZGKWVMADF",
                     GX_Mirror => "TBVMWZGKWVMADF",
                     GX_FlexVol_HPO => "FWSBXZGKWVMADF",
                     GX_CIFS => "RQQQXZGKWVMADF",
                     GX_SnapRestore => "DLOFYZGKWVMADF",
                     SnapMirror_DP => "NUHYZZGKWVMADF",
                     Striped_Volume => "LJDCBAHKWVMADF",
                     iSCSI => "ZOFNAAHKWVMADF",
                     FCP => "XDBRBAHKWVMADF",
                     },
    '1-80-000013' => {
                     NFS => "DPYLCMDPMZSEBG",
                     CIFS => "  TFFTAMDPMZSEBG",
                     GX_Base => "XBOLYLDPMZSEBG",
                     GX_Mirror => "VQJPZLDPMZSEBG",
                     GX_FlexVol_HPO => "HLHEAMDPMZSEBG",
                     GX_CIFS => "TFFTAMDPMZSEBG",
                     GX_SnapRestore => "FADIBMDPMZSEBG",
                     SnapMirror_DP => "PJWADMDPMZSEBG",
                     Striped_Volume => "NYREEMDPMZSEBG",
                     iSCSI => "BEUPDMDPMZSEBG",
                     FCP => "ZSPTEMDPMZSEBG",
                     },
    '1-80-000017' => {
                     NFS => "FMDPUSYLTDIFMA",
                     CIFS => "VCKWSSYLTDIFMA",
                     GX_Base => "ZYSOQSYLTDIFMA",
                     GX_Mirror => "XNOSRSYLTDIFMA",
                     GX_FlexVol_HPO => "JIMHSSYLTDIFMA",
                     GX_CIFS => "VCKWSSYLTDIFMA",
                     GX_SnapRestore => "HXHLTSYLTDIFMA",
                     SnapMirror_DP => "RGBEVSYLTDIFMA",
                     Striped_Volume => "PVWHWSYLTDIFMA",
                     iSCSI => "DBZSVSYLTDIFMA",
                     FCP => "BQUWWSYLTDIFMA",
                     },
           };

my $legacyLic = {};

=head2 add

    use SSQA::License;
    my $localResult = SSQA::License::add(command_interface => $node, service => "NFS");

    Call this routine with a NACL node object and a service name to
    add a license for said service on the node in question.

=cut


sub add {
    my (%params) =
        validate(@_, {
            command_interface => {
               type     => HASHREF,
               optional => 0,
            },
            service => { type => SCALAR,
                            optional => 0,
            }
        },);

    my $service = $params{service};
    my $node = $params{command_interface};

    # Default to 7 mode
    my $commandSet = "7Mode";

    my $Options = new Options;
    $Options->process();

    if ($node->hostrec()->{hosttype} =~ /-ng/i) {
        $commandSet = 'CMode';
    }

    # Get the APISet. We need it to get the version manager. This command is
    # different between BR and RR releases.
    my $apiSet = $node->apiset(category  => "Node",
                               interface => "CLI",
                               set       => $commandSet);

    $Log->comment("Installing $commandSet license for $service.");

    my $lic;
    if ($commandSet =~ /CMode/i) {
        my $cs = NACL::CS::License->fetch(command_interface => $node, requested_fields => [ qw(serial-number) ]);
        my $sn = $cs->serial_number();
        if (not defined($clusterLic->{$sn})) {
            die config_result("The installed cluster serial number of $sn is not a valid testing serial " .
                               "number. License correctness cannot be guaranteed.");
        };
        $lic = $clusterLic->{$sn};
    } else {
        $lic = $legacyLic;
    }

    if (not $lic->{$service}) {
        die config_result("The requested license for service $service in $commandSet is not implemented " .
                           "in this library yet.");
    }

    try {
        NACL::STask::SystemLicense->add(
            command_interface => $node,
            feature           => $service,
        );

    } catch NATE::BaseException with {
            my $ex = shift;
            die config_result("EXCEPTION: " . $ex->text() . "You may need to verify that the base " .
                               "license is from the correct group of internal licenses.");
    };

}
1;
