package SSQA::WaflHelper_tmp;
use strict;
use Exporter;
use Tharn;
use SayRetry;
use vars ('@ISA','@EXPORT');
use nfs::GenericExport qw(export_path);
use mount qw(mount umount);
@ISA = qw(Exporter);
@EXPORT = qw(filer_vols_get 
             aggr_free_space_get
             vol_snaps_get
             filer_qtrees_get
             check_mount
             get_vol_name_from_path
             get_max_used_files
             vol_size_get
             get_kbytes
             get_max_possible_files
             get_stats
             set_maxfiles
             filer_vol_freespace_get
             filer_vol_spaceinfo_get
             filer_vol_snap_freespace_get
             filer_vol_snap_totalspace_get
             filer_vol_snap_usedspace_get
             mount_vol_client_filer
             export_path);

## @name ::
## @usage 
## @synopsis
##    
## @arg
my $LARGE_MAX_FILE_NUM = "999999999999999999999999";
sub filer_vols_get($){

	my $fc = shift();
	my @vols;
	my $cmd = "vol status";
	my (undef,undef,$res) = say_retry($fc,$cmd);
	my @tmp = split(/\n/,$res);
	shift(@tmp);
	foreach my $val(@tmp){
		if(($val =~ /^\s*(\w+)\s+online|offline|restricted.*/)){
			push (@vols,$1);
		}
	}
        return(@vols);
}

sub aggr_free_space_get($$) {
	my $fc = shift();
	my $aggr = shift();
	my (undef,undef,$res) = say_retry($fc,"df -Am $aggr");
	my($freespace) = ($res =~/\d+\w\w\s+\d+\w\w\s+(\d+)\w\w/ );
	return($freespace);
}

sub vol_snaps_get($$){

	my $fc = shift();
        my $vol = shift();
	my $cmd = "snap list $vol";
	my (undef,undef,$res) = SayRetry::say_retry($fc,$cmd);
	my @snaps = split(/\n/,$res);
	splice(@snaps,0,5);
        return(@snaps);
}


sub filer_qtrees_get($$){
        my $fc = shift();
        my $vol=shift();
	my @qts;
	my (undef,undef,$out) = say_retry($fc,"qtree status $vol");
	my @tmp = split(/\n/,$out);
	shift(@tmp);
	shift(@tmp);
	foreach my $qt(@tmp){
		if($qt =~ /^\S+\s+(\S+)\s+\S+\s+\S+\s+\S+/){
			push(@qts,$1);
		}
	}
	return(@qts);
}


sub check_mount($$$$$) {
   my $filer=shift();
   my $vol = shift();
   my $fc = shift();
   my $cc = shift();
   my $mount_point=shift();
   # The mount library sometimes returns undefined or space when chmod fails
   if (!defined($mount_point) || $mount_point =~ /^\s+$/ || $mount_point =~ /^$/ || $mount_point  !~ /\w+/) {
      return 0;
   }
   my $mount_path .=  "/vol/" . $vol;
   my $cmd = "df $mount_point";
   my (undef,undef,$out) = say($cc, $cmd);
   my $match = $filer . ":" . $mount_path;
   logcomment("Out is $out");
   if ($out =~ /$match/ ) {
       return 1;
   }
   else {
      return 0;
   }

}
sub mount_vol_client_filer($$$$$$$$$) {
	my $filer=shift();
	my $vol_name = shift();
	my $fc = shift();
	my $client_conn = shift();
	my $mount_point=shift();
	my $client_param=shift();
	my $mount_path=shift();
	my $client=shift();
	my $mount_chmod=shift();

	#Check to see whether the mount is already there
	logcomment("prepare for checking mount, mountpoint is $mount_point");
        if (check_mount($filer,$vol_name,$fc,$client_conn,$mount_point) == 1 ) {
                logcomment("$mount_point is already mounted on $client_param");
        }else{
                #logcomment("Exporting $vol_name from $filer");
                #export_path($fc,$vol_name);
                #export_path($fc,$vol_name,OPTIONS=>"retry=20");
                #export_path($fc,$mount_path,OPTIONS=>"rw,anon=0,nosuid,sec=sys");
                logcomment("Mount volume $vol_name from $filer on $client_param");
                if ($mount_chmod != 0) {
                        $mount_point = mount( $filer, $mount_path,
                                             client => $client, 'chmod' => $mount_chmod,'timeout' => '300' );
                 }
                else {
                        $mount_point = mount( $filer, $mount_path,
                                             client => $client,'timeout' => '300' );
                }
	
                #$mountpoint = mount( $filer, $mount_path,
                #             client => $client, 'chmod' => '0777','timeout' => '300');
                #Check to make sure that the mount succeded
                if (check_mount($filer,$vol_name,$fc,$client_conn,$mount_point) == 0 ) {
                        logresult("FAIL", abort=>1, "Unable to mount the volume on the client");
                }
        }
}

sub get_max_used_files($$) {
   my $fc = shift();
   my $vol = shift();
   my $cmd = "priv set diag;maxfiles $vol";
   my (undef,undef,$out) = say_retry($fc,$cmd);
   my $maxfiles;
   my %ret=undef;
   if ($out =~ /maximum number of files is currently (\d+).*?(\d+)/){
        $ret{'max'} = $1;
        $ret{'used'} = $2;
   }
   return (%ret)
}

sub set_maxfiles($$$;$) {
   my $fc = shift();
   my $vol = shift();
   my $num = shift();
   my $mode = shift();
   if (!defined($mode)) {
      $mode = "diag";
   }
   my $cmd = "priv set  $mode " . ";maxfiles -f $vol $num";
   my (undef,undef,$out) = say_retry($fc,$cmd);
   if ($out =~ / maximum inode count cannot exceed|The value you entered is invalid/) {
      return (-1);
   }
   else {
      return (0);
   }
}
sub vol_size_get($$;$){

my $fc = shift();
my $vol = shift();
my $cmd = "vol size $vol";
my (undef,undef,$result)=say_retry($fc,$cmd);
  while( $result =~ /busy/){
     sleep(30);
     (undef,undef,$result)=say_retry($fc,$cmd);

  }
 if($result =~ /\s+\d+k\./){
          $result =~ /(\s+\d+k\.)/;
          my $size= $1;
          $size =~ /(\d+)/;
          $size= $1;
          return ($size);
  } 
  elsif($result =~ /\s+\d+m\./){
          $result =~ /(\s+\d+m\.)/;
          my $size= $1;
          $size =~ /(\d+)/;
          $size= $1;
          return (int($size*1024));
  }
  elsif($result =~ /\s+\d+g\./){
          $result =~ /(\s+\d+g\.)/;
          my $size= $1;
          $size =~ /(\d+)/;
          $size= $1;
          return (int($size*1024*1024));
  } 	
}

sub get_kbytes($){
my $val=shift();
my $return;
	if($val =~ /m|M$/){	#Amount of IO is in MB
		$val =~ s/m|M$//;
		$return = int(1024 * $val) ;
	}
	elsif($val =~ /g|G$/){	#Amount of IO is in GB
		$val=~ s/g|G$//;
		$return = int(1024 * 1024 * $val) ;

	}
	elsif($val =~ /t|T$/){	#Amount of IO is in TB
		$val=~ s/t|T$//;
		$return = int(1024 * 1024 * 1024 * $val) ;

	}
	else{
		$return = $val;
	}
return($return);

}

sub get_vol_name_from_path($) {
   my $vol=shift();
   my $vol_name;
   if ($vol =~ /^\/vol\/(\S+)\//){
	$vol_name = $1;
   }
   else
   {
	$vol_name = $vol; 
   }
  return($vol_name);
}

sub get_qtree_name_from_path($) {
   my $vol=shift();
   my $qt_name;
   if ($vol =~ /^\/vol\/.*?\/(\S+)/){
	$qt_name = $1;
   }
   else
   {
	$qt_name = $vol; 
   }
  return($qt_name);
}
sub get_max_possible_files($$;$) {
   my $fc=shift();
   my $vol = shift();
   my $mode=shift();
   if (!defined($mode)) {
      $mode = "diag";
   }
   my $cmd = "priv set $mode " . ";maxfiles $vol " . $LARGE_MAX_FILE_NUM;
   my $maxf=undef;
   my (undef,undef,$result)=say_retry($fc,$cmd);
   #maxfiles vol0 999999999999999999999999999999999999999999999999999999999999999999999999999999
   #maxfiles: The value you entered (-1) is invalid;
   #maximum inode count should be between 3727020 and 477060380
   if($result =~ /.*maximum inode count should be between\s+(\d+)\s+and\s+(\d+).*/s){
      $maxf=$2;
   }
   elsif ($result =~ /.*Maxfiles remains at\s+(\d+)/) {
       $maxf=$1;
   }
   return $maxf;
}

sub get_stats($) {
    my $times = shift;
    my $sum = 0;
    my $max = 0;
    my $min = 99999999999;
    my $cnt = scalar(@$times);

    # Return error is list is empty
    if ($cnt < 0) {
        return (0, 0, 0, 0);
    }
    $max = $times->[0];
    $min = $times->[0];
    # Step through list to find minimum and maximum
    # as well as calculating the total sum
    foreach my $time ( @$times ) {
        next if ($time ==0);
        $sum += $time;
        $max = $time if ($time > $max);
        $min = $time if ($time < $min);
    }
    # Return list statistics
    return (sprintf("%.2f", $sum/$cnt), sprintf("%.2f",$min), sprintf("%.2f",$max), sprintf("%.2f",$cnt));
}


sub filer_vol_spaceinfo_get($$){
   my $fc  = shift;
   my $vol  = shift;
   my $cmd = "df $vol";
   my (undef,undef,$result) = say_retry($fc,$cmd);
   my @tmprslt = split(/\n/,$result);   
   my @volout = split(/ +/,$tmprslt[1]);
   my @volsnapout = split(/ +/,$tmprslt[2]);
   my %dfoutput = (TOTAL => int($volout[1]), # total space on volume
           USED => int($volout[2]),  # Used space on the volume
           FREE => int($volout[3]),  # Free space on the volume
           SNAPTOTAL => int($volsnapout[1]), # total space in snapshots area on volume
           SNAPUSED => int($volsnapout[2]),  # used space in snapshots area on volume
           SNAPFREE => int($volsnapout[3])); # free space in snapshots area on volume    
   return (%dfoutput);
}


sub filer_vol_freespace_get ($$) {
   my $fc  = shift;
   my $vol = shift;

   my %outval = filer_vol_spaceinfo_get($fc,$vol);
   return($outval{"FREE"}* 1024);       
}


sub filer_vol_snap_freespace_get($$){
   my $fc  = shift;
   my $vol = filer_get_vol_name_from_path(shift);
   
   my %outval = filer_vol_spaceinfo_get($fc,$vol);
   return($outval{"SNAPFREE"}* 1024);       
}


sub filer_vol_snap_totalspace_get($$){
   my $fc  = shift;
   my $vol = shift;
   
   my %outval = filer_vol_spaceinfo_get($fc,$vol);
   return($outval{"SNAPTOTAL"}* 1024);       
}


sub filer_vol_snap_usedspace_get($$){
   my $fc  = shift;
   my $vol = filer_get_vol_name_from_path(shift);
   
   my %outval = filer_vol_spaceinfo_get($fc,$vol);
   return($outval{"SNAPUSED"}* 1024);       
}

sub export_path($$;%) {
   my $fc  = shift;
   my $vol = get_vol_name_from_path(shift);
   my $vol_path = "/vol/" . $vol;
   nfs::GenericExport::export_path($fc,
                                   $vol_path, 
                                   OPTIONS=>"rw,anon=0,nosuid,sec=sys", 
                                   PERSISTENT_OPT=>1,
                                   retry => 10) 
   or die "exportfs failed: $@";

}
1;
