# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/San/Application/Igroupconstant.pm#1 $
#
# Copyright (c) 2010 NetApp, Inc.
# All rights reserved.
#
##
## @summary constant definitions for igroup (cmode) testing
## @author SAN QA (c-dl-ssan-test@netapp.com)
## @status public

package San::Application::Igroupconstant;

use strict;
use Exporter;

our (@ISA, @EXPORT);

use Exporter;
@ISA    = qw(Exporter);
@EXPORT = qw(IGRP_ATTR_IGROUP_NAME
  IGRP_ATTR_VSERVER_NAME
  IGRP_ATTR_PROTOCOL
  IGRP_ATTR_OS_TYPE
  IGRP_ATTR_PORTSET_NAME
  IGRP_ATTR_INITIATOR
  IGRP_ATTR_INITIATOR_DETAILS
  IGRP_ATTR_ALUA
  IGRP_ATTR_UUID
  IGRP_ATTR_FORCE
  IGRP_ATTR_RENAME
  IGRP_ATTR_ATTRIBUTE
  IGRP_ATTR_VALUE
  IGRP_ATTR_ZAPI_NA


  IGRP_IGROUP_OBJ
  IGRP_VSERVER_OBJ
  IGRP_ATTR_INFO

  INITIATOR_NONE
  FCP_INITIATOR
  ISCSI_INITIATOR
  FCP_ALIAS_INITIATOR
  MIXED_INITIATOR
  MIXED_WITH_ALIAS_INITIATOR

  IGROUP_PROTOCOLS_SUPPORTED
  MAX_IGROUP_PER_NODE
  @ExpectedIgroupAttributes
  %IgroupAttributesDefault
  );

# MAX igroups per node
# soft value, it will allow to create more than 255
# refer http://wikid.netapp.com/w/Smokejumper/Bcom/Documents/BCOM_DS#C-Mode_RR.0_SAN_Limits
use constant MAX_IGROUP_PER_NODE => 256;

# object attribute name

use constant IGRP_IGROUP_OBJ  => 'igroup_obj';
use constant IGRP_VSERVER_OBJ => 'vserver_obj';
use constant IGRP_ATTR_INFO   => 'igrp_attr_info';

# igroup attributes (parameter) definitions
use constant IGRP_ATTR_IGROUP_NAME       => 'igroup';
use constant IGRP_ATTR_VSERVER_NAME      => 'vserver';
use constant IGRP_ATTR_PROTOCOL          => 'protocol';
use constant IGRP_ATTR_OS_TYPE           => 'ostype';
use constant IGRP_ATTR_PORTSET_NAME      => 'portset';
use constant IGRP_ATTR_INITIATOR         => 'initiator';
use constant IGRP_ATTR_INITIATOR_DETAILS => 'init_details';
use constant IGRP_ATTR_ALUA              => 'alua';
use constant IGRP_ATTR_UUID              => 'uuid';
use constant IGRP_ATTR_FORCE             => 'force';

use constant IGRP_ATTR_RENAME    => 'new_name';
use constant IGRP_ATTR_ATTRIBUTE => 'attribute';
use constant IGRP_ATTR_VALUE     => 'value';

# zapi output not applicable
use constant IGRP_ATTR_ZAPI_NA => 'zapi_na';

# portset attributes (parameter) definitions
#use constant PS_ATTR_PORTSET_NAME => 'portset';
#use constant PS_ATTR_VSERVER_NAME => 'vserver';
#use constant PS_ATTR_PROTOCOL     => 'protocol';
#use constant PS_ATTR_IGROUP       => 'igroups';
#use constant PS_ATTR_PORT_NAME    => 'port_name';
#use constant PS_ATTR_PORT_COUNT   => 'port_count';

# Bit definition
# Bit 0: NO INITIATOR - just default igroup
# Bit 1: FCP INITIATOR
# Bit 2: ISCSI INITIATOR
# Bit 3: FCP_ALIAS_INITIATOR
# Bit 4: NOT USED - for feature!!!
# Bit 5: NOT USED - for feature!!!
# Bit 6: NOT USED - for feature!!!
# Bit 7: NOT USED - for feature!!!

use constant INITIATOR_NONE             => 0b00000000;
use constant FCP_INITIATOR              => 0b00000001;
use constant ISCSI_INITIATOR            => 0b00000010;
use constant FCP_ALIAS_INITIATOR        => 0b00000100;
use constant MIXED_INITIATOR            => FCP_INITIATOR | ISCSI_INITIATOR;
use constant MIXED_WITH_ALIAS_INITIATOR => FCP_INITIATOR | ISCSI_INITIATOR | FCP_ALIAS_INITIATOR;

use constant IGROUP_PROTOCOLS_SUPPORTED => qw(iscsi fcp mixed);

# expected "igroup show" attributes
# since igroup attributes output is changing frequently (by developers)
# its defined here, so that once place to change....
our @ExpectedIgroupAttributes = (IGRP_ATTR_IGROUP_NAME,  IGRP_ATTR_VSERVER_NAME,
                                 IGRP_ATTR_PROTOCOL,     IGRP_ATTR_OS_TYPE,
                                 IGRP_ATTR_PORTSET_NAME, IGRP_ATTR_INITIATOR_DETAILS,
                                 IGRP_ATTR_INITIATOR,    IGRP_ATTR_ALUA,
                                 IGRP_ATTR_UUID);

# default value for igroup attributtes
our %IgroupAttributesDefault = ((IGRP_ATTR_IGROUP_NAME)       => '-',
                                (IGRP_ATTR_VSERVER_NAME)      => '-',
                                (IGRP_ATTR_PROTOCOL)          => 'mixed',
                                (IGRP_ATTR_OS_TYPE)           => '-',
                                (IGRP_ATTR_PORTSET_NAME)      => '-',
                                (IGRP_ATTR_INITIATOR)         => '-',
                                (IGRP_ATTR_INITIATOR_DETAILS) => '-',
                                (IGRP_ATTR_ALUA)              => 'true',
                                (IGRP_ATTR_UUID)              => '-',);

1;
