#!/usr/local/bin/perl
# $Id: //depot/prod/test/Rsierranevada/lib/San/AttrTable.pm#2 $
#
# Summary:
#   provides a list of attributes for CLI show commands so common parser can map ZAPI get-iter
#   output and build a returned hash that is homogeneous for CLI and ZAPI
#
# Description:
#   The table provides the cdef show commands with an array of expected returned attributes for CLI.
#   A common parser method will use this table to return a homogeneous output for either
#   CLI or ZAPI
#
# Dependcies:	Perl
# Status:	private
# Author:   korry@netapp.com, dl-ssan-automation

use strict;
use vars qw(%Show_cmds);

%Show_cmds = (
    # cdef show commands with a list of attributes
    'cluster_identity_show' => {
        'uuid' => {'zapi_field' => 'cluster-uuid', attr => 'cluster-identity-info'},
        'name' => {'zapi_field' => 'cluster-name', attr => 'cluster-identity-info'},
        'serialnumber' => {'zapi_field' => 'cluster-serial-number', attr => 'cluster-identity-info'},
        'location' => {'zapi_field' => 'cluster-location', attr => 'cluster-identity-info'},
        'contact' => {'zapi_field' => 'cluster-contact', attr => 'cluster-identity-info'},
        },
    'cluster_show' => {
        'node'        => {'zapi_field' => 'node',                      attr => 'node-details-info'},
        'node_uuid'   => {'zapi_field' => 'node-uuid',                 attr => 'node-details-info'},
        'uuid'        => {'zapi_field' => 'node-uuid',                 attr => 'node-details-info'},
        'epsilon'     => { 'zapi_field' => 'is-epsilon-node',          attr => 'node-details-info'},
        'eligibility' => { 'zapi_field' => 'is-node-cluster-eligible', attr => 'node-details-info'},
        'health'      => { 'zapi_field' => 'is-node-healthy',          attr => 'node-details-info'},
        },
    'system_node_show' => {
        'node' => {'zapi_field' => 'node', attr => 'node-details-info'},
        'owner' => {'zapi_field' => 'node-owner', attr => 'node-details-info'},
        'location' => {'zapi_field' => 'node-location', attr => 'node-details-info'},
        'model' => {'zapi_field' => 'node-model', attr => 'node-details-info'},
        'serialnumber' => {'zapi_field' => 'node-serial-number', attr => 'node-details-info'},
        'assettag' => { 'zapi_field' => 'not_exist', attr => ''},
        'uptime' => {'zapi_field' => 'node-uptime', attr => 'node-details-info'},
        'nvramid' => {'zapi_field' => 'node-nvram-id', attr => 'node-details-info'},
        'systemid' => {'zapi_field' => 'node-nvram-id', attr => 'node-details-info'},
        'vendor' => {'zapi_field' => 'node-vendor', attr => 'node-details-info'},
        'health' => { 'zapi_field' => 'not_exist', attr => ''},
        'eligibility' => { 'zapi_field' => 'not_exist', attr => ''},
        'epsilon' => { 'zapi_field' => 'not_exist', attr => ''},
        },
    'storage_disk_show' => {
        'uid' => {'zapi_field' => 'disk-uid', attr => 'disk-inventory-info'},
        'disk' => {'zapi_field' => 'disk-type', attr => 'storage-disk-info'},
        'type' => {'zapi_field' => 'disk-type', attr => 'disk-inventory-info'},
        'disk_type' => {'zapi_field' => 'disk-name', attr => 'disk-inventory-info'},
        'vendor' => {'zapi_field' => 'vendor', attr => 'disk-inventory-info'},
        'model' => {'zapi_field' => 'model', attr => 'disk-inventory-info'},
        'serialnumber' => {'zapi_field' => 'serial-number', attr => 'disk-inventory-info'},
        'revision' => {'zapi_field' => 'firmware-revision', attr => 'disk-inventory-info'},
        'rpm' => {'zapi_field' => 'rpm', attr => 'disk-inventory-info'},
        'checksum' => {'zapi_field' => 'not_exist', attr => ''},
        'bps' => {'zapi_field' => 'bytes-per-sector', attr => 'disk-inventory-info'},
        'capacity_sectors' => {'zapi_field' => 'capacity-sectors', attr => 'disk-inventory-info'},
        'sectorcount' => {'zapi_field' => 'capacity-sectors', attr => 'disk-inventory-info'},
        'capacity' => {'zapi_field' => 'not_exist', attr => ''},
        'rightsize_sectors' => {'zapi_field' => 'not_exist', attr => ''},
        'shelf' => {'zapi_field' => 'shelf', attr => 'disk-inventory-info'},
        'bay' => {'zapi_field' => 'shelf-bay', attr => 'disk-inventory-info'},
        'hostadapter' => {'zapi_field' => 'not_exist', attr => ''},
        'primaryport' => {'zapi_field' => 'not_exist', attr => ''},
        'secondaryport' => {'zapi_field' => 'not_exist', attr => ''},
        'secondaryname' => {'zapi_field' => 'not_exist', attr => ''},
        'is_bcs' => {'zapi_field' => 'is-block-checksum-compatible', attr => 'disk-inventory-info'},
        'is_dynamically_qualified' => {'zapi_field' => 'not_exist', attr => ''},
        'shm_timeinterval' => {'zapi_field' => 'not_exist', attr => ''},
        'grown_defect_list_count' => {'zapi_field' => 'grown-defect-list-count', attr => 'disk-inventory-info'},
        'scrub_last_done' => {'zapi_field' => 'not_exist', attr => ''},
        'scrub_last_done_seconds' => {'zapi_field' => 'not_exist', attr => ''},
        'scrub_count' => {'zapi_field' => 'not_exist', attr => ''},
        'percent_rated_life_used' => {'zapi_field' => 'not_exist', attr => ''},
        'percent_spares_consumed' => {'zapi_field' => 'not_exist', attr => ''},
        'percent_spares_consumed_limit' => {'zapi_field' => 'not_exist', attr => ''},
        'home' => {'zapi_field' => 'home-node-name', attr => 'disk-ownership-info'},
        'owner' => {'zapi_field' => 'owner-node-name', attr => 'disk-ownership-info'},
        'homeid' => {'zapi_field' => 'home-node-id', attr => 'disk-ownership-info'},
        'ownerid' => {'zapi_field' => 'owner-node-id', attr => 'disk-ownership-info'},
        'assigned_pool' => {'zapi_field' => 'pool', attr => 'disk-ownership-info'},
        'pool' => {'zapi_field' => 'pool', attr => 'disk-ownership-info'},
        'reserver' => {'zapi_field' => 'reserved-by-node-id', attr => 'disk-ownership-info'},
        'sanown_failed' => {'zapi_field' => 'is-failed', attr => 'disk-ownership-info'},
        'initiator' => {'zapi_field' => 'initiator-port', attr => 'disk-path-info'},
        'lun' => {'zapi_field' => 'lun-number', attr => 'disk-path-info'},
        'initiator_side_switch_port' => {'zapi_field' => 'initiator-side-switch-port', attr => 'disk-path-info'},
        'target_port_access_state' => {'zapi_field' => 'target-port-access-state', attr => 'disk-path-info'},
        'target_wwpn' => {'zapi_field' => 'target-wwpn', attr => 'disk-path-info'},
        'tpgn' => {'zapi_field' => 'tpgn', attr => 'disk-path-info'},
        'port_speed' => {'zapi_field' => 'initiator-port-speed', attr => 'disk-path-info'},
        'lun_io_kbps' => {'zapi_field' => 'lun-io-kbps', attr => 'disk-path-info'},
        'lun_iops' => {'zapi_field' => 'lun-iops', attr => 'disk-path-info'},
        'array_name' => {'zapi_field' => 'array-name', attr => 'disk-path-info'},
        'target_side_switch_port' => {'zapi_field' => 'target-side-switch-port', attr => 'disk-path-info'},
        'initiator_io_kbps' => {'zapi_field' => 'initiator-io-kbps', attr => 'disk-path-info'},
        'initiator_iops' => {'zapi_field' => 'initiator-iops', attr => 'disk-path-info'},
        'target_io_kbps' => {'zapi_field' => 'target-io-kbps', attr => 'disk-path-info'},
        'target_iops' => {'zapi_field' => 'target-iops', attr => 'disk-path-info'},
        'path_io_kbps' => {'zapi_field' => 'path-io-kbps', attr => 'disk-path-info'},
        'path_iops' => {'zapi_field' => 'path-iops', attr => 'disk-path-info'},
        'path_link_errors' => {'zapi_field' => 'path-link-errors', attr => 'disk-path-info'},
        'lun_path_use_state' => {'zapi_field' => 'lun-path-use-state', attr => 'disk-path-info'},
        'target_lun_in_use_count' => {'zapi_field' => 'target-lun-in-use-count', attr => 'disk-path-info'},
        'initiator_lun_in_use_count' => {'zapi_field' => 'initiator-lun-in-use-count', attr => 'disk-path-info'},
        'path_lun_in_use_count' => {'zapi_field' => 'path-lun-in-use-count', attr => 'disk-path-info'},
        'path_quality' => {'zapi_field' => 'path-quality', attr => 'disk-path-info'},
        'nodelist' => {'zapi_field' => 'node', attr => 'disk-path-info'},
        'diskpathnames' => {'zapi_field' => 'disk-name', attr => 'disk-path-info'},
        'container' => {'zapi_field' => 'container-type', attr => 'disk-raid-info'},
        'aggregate' => {'zapi_field' => 'aggregate-name', attr => 'disk-aggregate-info'},
        'checksum_type' => {'zapi_field' => 'checksum-type', attr => 'disk-aggregate-info'},
        'copydestination' => {'zapi_field' => 'copy-destination-name', attr => 'disk-aggregate-info'},
        'copypercent' => {'zapi_field' => 'copy-percent-complete', attr => 'disk-aggregate-info'},
        'is_offline' => {'zapi_field' => 'is-offline', attr => 'disk-aggregate-info'},
        'is_prefailed' => {'zapi_field' => 'is-prefailed', attr => 'disk-aggregate-info'},
        'prefailed' => {'zapi_field' => 'not_exist', attr => ''},
        'is_replacing' => {'zapi_field' => 'is-replacing', attr => 'disk-aggregate-info'},
        'replacing' => {'zapi_field' => 'not_exist', attr => ''},
        'is_reconstructing' => {'zapi_field' => 'is-reconstructing', attr => 'disk-aggregate-info'},
        'plex' => {'zapi_field' => 'plex-name', attr => 'disk-aggregate-info'},
        'position' => {'zapi_field' => 'position', attr => 'disk-aggregate-info'},
        'raidgroup' => {'zapi_field' => 'raid-group-name', attr => 'disk-aggregate-info'},
        'reconstructionpercent' => {'zapi_field' => 'reconstruct-percent-complete', attr => 'disk-aggregate-info'},
        'is_fdr' => {'zapi_field' => 'is-in-fdr', attr => 'disk-outage-info'},
        'outage_reason' => {'zapi_field' => 'is-in-fdr', attr => 'disk-outage-info'},
        'detailstate' => {'zapi_field' => 'not_exist', attr => ''},
        'is_scrubbing' => {'zapi_field' => 'is-media-scrubbing', attr => 'disk-aggregate-info'},
        'is_sparecore' => {'zapi_field' => 'is-sparecore', attr => 'disk-spare-info'},
        'is_zeroed' => {'zapi_field' => 'is-zeroed', attr => 'disk-aggregate-info'},
        'zeroed' => {'zapi_field' => 'is-zeroed', attr => 'disk-aggregate-info'},
        'is_zeroing' => {'zapi_field' => 'is-zeroing', attr => 'disk-aggregate-info'},
        'zeroingpercent' => {'zapi_field' => 'zeroing-percent-complete', attr => 'disk-aggregate-info'},
        'effective_type' => {'zapi_field' => 'effective-disk-type', attr => 'disk-raid-info'},
        'effective_disk_type' => {'zapi_field' => 'effective-disk-type', attr => 'disk-raid-info'},
        'physicalsize' => {'zapi_field' => 'not_exist', attr => ''},
        'physicalblocks' => {'zapi_field' => 'physical-blocks', attr => 'disk-raid-info'},
        'spare_pool' => {'zapi_field' => 'spare-pool', attr => 'disk-raid-info'},
        'usedsize' => {'zapi_field' => 'not_exist', attr => ''},
        'usedblocks' => {'zapi_field' => 'used-blocks', attr => 'disk-raid-info'},
        'poweron_hours' => {'zapi_field' => 'not_exist', attr => ''},
        'poweron_seconds' => {'zapi_field' => 'power-on-time-interval', attr => 'disk-stats-info'},
        'sectors_read' => {'zapi_field' => 'sectors-read', attr => 'disk-stats-info'},
        'blocksread' => {'zapi_field' => 'sectors-read', attr => 'disk-stats-info'},
        'sectors_written' => {'zapi_field' => 'sectors-written', attr => 'disk-stats-info'},
        'blockswritten' => {'zapi_field' => 'sectors-written', attr => 'disk-stats-info'},
        'physical_blocks' => {'zapi_field' => 'not_exist', attr => ''},
        'raidtype' => {'zapi_field' => 'position', attr => 'disk-aggregate-info'},
        'errors' => {'zapi_field' => 'not_exist', attr => ''},
        'count' => {'zapi_field' => 'not_exist', attr => ''},
        'state' => {'zapi_field' => 'not_exist', attr => ''},
        'owner_str' => {'zapi_field' => 'not_exist', attr => ''},
        'home_str' => {'zapi_field' => 'not_exist', attr => ''},
        },
    'storage_aggregate_show' => {
        'id_lost_write' => {'zapi_field' => 'raid-lost-write-state', attr => 'aggr-raid-attributes'},
        'raid_zoned_checksum' => {'zapi_field' => 'not_exist', attr => ''},
        'sis_metadata_space_used' => {'zapi_field' => 'not_exist', attr => ''},
        'uuid' => {'zapi_field' => 'aggregate-uuid', attr => 'aggr-attributes'},
        'state' => {'zapi_field' => 'state', attr => 'aggr-raid-attributes'},
        'ha_policy' => {'zapi_field' => 'ha-policy', attr => 'aggr-raid-attributes'},
        'free_space_realloc' => {'zapi_field' => 'free-space-realloc', attr => 'aggr-performance-attributes'},
        'raid_block_checksum' => {'zapi_field' => 'checksum-style', attr => 'aggr-raid-attributes'},
        'diskcount' => {'zapi_field' => 'disk-count', attr => 'aggr-raid-attributes'},
        'has_mroot' => {'zapi_field' => 'has-local-root', attr => 'aggr-raid-attributes'},
        'block_type' => {'zapi_field' => 'block-type', attr => 'aggr-fs-attributes'},
        'thorough_scrub' => {'zapi_field' => 'not_exist', attr => ''},
        'usedsize' => {'zapi_field' => 'size-used', attr => 'aggr-space-attributes'},
        'availsize' => {'zapi_field' => 'size-available', attr => 'aggr-space-attributes'},
        'hybrid_enabled' => {'zapi_field' => 'is-hybrid-enabled', attr => 'aggr-raid-attributes'},
        'inconsistent' => {'zapi_field' => 'is-inconsistent', attr => 'aggr-raid-attributes'},
        'snapshot_reserve' => {'zapi_field' => 'percent-inode-used-capacity', attr => 'aggr-snapshot-attributes'},
        'maxraidsize' => {'zapi_field' => 'raid-size', attr => 'aggr-raid-attributes'},
        'type' => {'zapi_field' => 'type', attr => 'aggr-fs-attributes'},
        'nodes' => {'zapi_field' => 'node-name', attr => 'nodes'},
        'raidgroups' => {'zapi_field' => 'raidgroup-name', attr => 'raidgroup-attributes'},
        'raid_lost_write_state' => {'zapi_field' => 'raid-lost-write-state', attr => 'aggr-raid-attributes'},
        'ignore_inconsistent' => {'zapi_field' => 'not_exist', attr => ''},
        'chksumenabled' => {'zapi_field' => 'is-checksum-enabled', attr => 'aggr-raid-attributes'},
        'disklist' => {'zapi_field' => 'not_exist', attr => ''},
        'plexes' => {'zapi_field' => 'plex-name', attr => 'plex-attributes'},
        'size' => {'zapi_field' => 'size-total', attr => 'aggr-space-attributes'},
        'voltype' => {'zapi_field' => 'not_exist', attr => ''},
        'percent_used' => {'zapi_field' => 'percent-used-capacity', attr => 'aggr-space-attributes'},
        'chksumstyle' => {'zapi_field' => 'checksum-style', attr => 'raidgroup-attributes'},
        'volcount' => {'zapi_field' => 'flexvol-count', attr => 'aggr-volume-count-attributes'},
        'aggregate' => {'zapi_field' => 'aggregate-name', attr => 'aggr-attributes'},
        'root' => {'zapi_field' => 'has-local-root', attr => 'aggr-raid-attributes'},
        'raidstatus' => {'zapi_field' => 'raid-status', attr => 'aggr-raid-attributes'},
        'has_partner_mroot' => {'zapi_field' => 'has-partner-root', attr => 'aggr-raid-attributes'},
        'volume_style' => {'zapi_field' => 'not_exist', attr => ''},
        'raidtype' => {'zapi_field' => 'raid-status', attr => 'aggr-raid-attributes'},
        'chksumstatus' => {'zapi_field' => 'checksum-status', attr => 'aggr-raid-attributes'},
        'snapshot_autodelete' => {'zapi_field' => 'not_exist', attr => ''},
        'hybrid' => {'zapi_field' => 'is-hybrid', attr => 'aggr-raid-attributes'},
        },
    'vserver_show' => {
        'vserver' => {'zapi_field' => 'vserver-name', attr => 'vserver-info'},
        'type' => {'zapi_field' => 'vserver-type', attr => 'vserver-info'},
        'uuid' => {'zapi_field' => 'uuid', attr => 'vserver-info'},
        'rootvolume' => {'zapi_field' => 'root-volume', attr => 'vserver-info'},
        'aggregate' => {'zapi_field' => 'root-volume-aggregate', attr => 'vserver-info'},
        'ns_switch' => {'zapi_field' => 'nsswitch', attr => 'name-server-switch'},
        'nm_switch' => {'zapi_field' => 'nmswitch', attr => 'name-mapping-switch'},
        'nisdomain' => {'zapi_field' => 'nis-domain', attr => 'vserver-info'},
        'rootvolume_security_style' => {'zapi_field' => 'root-volume-security-style', attr => 'vserver-info'},
        'ldap_client' => {'zapi_field' => 'ldap-domain', attr => 'vserver-info'},
        'language' => {'zapi_field' => 'language', attr => 'vserver-info'},
        'snapshot_policy' => {'zapi_field' => 'snapshot-policy', attr => 'vserver-info'},
        'comment' => {'zapi_field' => 'comment', attr => 'vserver-info'},
        'antivirus_on_access_policy' => {'zapi_field' => 'antivirus-on-access-policy', attr => 'vserver-info'},
        'quota_policy' => {'zapi_field' => 'quota-policy', attr => 'vserver-info'},
        'aggr_list' => {'zapi_field' => 'aggr-list', attr => 'vserver-info'},
        'max_volumes' => {'zapi_field' => 'max-volumes', attr => 'vserver-info'},
        'admin_state' => {'zapi_field' => 'state', attr => 'vserver-info'},
        'allowed_protocols' => {'zapi_field' => 'protocol', attr => 'allowed-protocols'},
        'disallowed_protocols' => {'zapi_field' => 'protocol', attr => 'disallowed-protocol'},
        'protocol_services_use_data_lifs' => {'zapi_field' => 'not_exist', attr => ''},
        },
    'network_fcp_adapter_show' => {
        'node' => {'zapi_field' => 'node', attr => 'fcp-config-adapter-info'},
        'adapter' => {'zapi_field' => 'adapter', attr => 'fcp-config-adapter-info'},
        'description' => {'zapi_field' => 'info-name', attr => 'fcp-config-adapter-info'},
        'physical_protocol' => {'zapi_field' => 'physical-protocol', attr => 'fcp-config-adapter-info'},
        'max_speed' => {'zapi_field' => 'max-speed', attr => 'fcp-config-adapter-info'},
        'status' => {'zapi_field' => 'state', attr => 'fcp-config-adapter-info'},
        'substatus' => {'zapi_field' => 'not_exist', attr => ''},
        'portaddr' => {'zapi_field' => 'port-address', attr => 'fcp-config-adapter-info'},
        'firmwareRev' => {'zapi_field' => 'firmware-rev', attr => 'fcp-config-adapter-info'},
        'pciBusWidth' => {'zapi_field' => 'pci-bus-width', attr => 'fcp-config-adapter-info'},
        'pciClockSpeed' => {'zapi_field' => 'pci-clock-speed', attr => 'fcp-config-adapter-info'},
        'cacheLineSize' => {'zapi_field' => 'cache-line-size', attr => 'fcp-config-adapter-info'},
        'packetSize' => {'zapi_field' => 'packet-size', attr => 'fcp-config-adapter-info'},
        'sramParityEnabled' => {'zapi_field' => 'sram-parity-enabled', attr => 'fcp-config-adapter-info'},
        'externalGbicEnabled' => {'zapi_field' => 'external-gbic-enabled', attr => 'fcp-config-adapter-info'},
        'dataLinkRate' => {'zapi_field' => 'data-link-rate', attr => 'fcp-config-adapter-info'},
        'fabricEstablished' => {'zapi_field' => 'fabric-established', attr => 'fcp-config-adapter-info'},
        'connEstablished' => {'zapi_field' => 'connection-established', attr => 'fcp-config-adapter-info'},
        'mediaType' => {'zapi_field' => 'media-type', attr => 'fcp-config-adapter-info'},
        'speed' => {'zapi_field' => 'speed', attr => 'fcp-config-adapter-info'},
        'state' => {'zapi_field' => 'not_exist', attr => ''},
        'lif_capable' => {'zapi_field' => 'not_exist', attr => ''},
        'tha_index' => {'zapi_field' => 'not_exist', attr => ''},
        'handle' => {'zapi_field' => 'not_exist', attr => ''},
        'pslot' => {'zapi_field' => 'not_exist', attr => ''},
        'orig_wwnn' => {'zapi_field' => 'not_exist', attr => ''},
        'fc_wwnn' => {'zapi_field' => 'node-name', attr => 'fcp-config-adapter-info'},
        'fc_wwpn' => {'zapi_field' => 'port-name', attr => 'fcp-config-adapter-info'},
        'queue_depth' => {'zapi_field' => 'not_exist', attr => ''},
        },
    'vserver_fcp_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'fcp-service-info'},
        'target_name' => {'zapi_field' => 'node-name', attr => 'fcp-service-info'},
        'status_admin' => {'zapi_field' => 'is-available', attr => 'fcp-service-info', 'true' => 'up', 'false' => 'down'},
        },
    'vserver_fcp_interface_show' => {
        'vserver'        => {'zapi_field' => 'vserver',        attr => 'fcp-interface-info'},
        'lif'            => {'zapi_field' => 'interface-name', attr => 'fcp-interface-info'},
        'wwpn'           => {'zapi_field' => 'port-name',      attr => 'fcp-interface-info'},
        'wwnn'           => {'zapi_field' => 'node-name',      attr => 'fcp-interface-info'},
        'status_admin'   => {'zapi_field' => 'not_exist', attr => ''},
        'status_oper'    => {'zapi_field' => 'not_exist', attr => ''},
        'status_extended'=> {'zapi_field' => 'not_exist', attr => ''},
        'port_address'   => {'zapi_field' => 'port-address',   attr => 'fcp-interface-info'},
        'curr_node'      => {'zapi_field' => 'current-node',   attr => 'fcp-interface-info'},
        'curr_port'      => {'zapi_field' => 'current-port',   attr => 'fcp-interface-info'},
        'is_home'        => {'zapi_field' => 'not_exist', attr => ''},
        'relative-port-id'=>{'zapi_field' => 'relative-port-id',attr => 'fcp-interface-info'},
        },
    # This is a NON_SHOW instruction, it does not return an attribute key, so
    # the attr key should have the value 'non_show' to be translatable to CLI.
    'vserver_fcp_status' => {
        'vserver'      => {'zapi_field' => 'not_exist', attr => ''},
        'status_admin' => {'zapi_field' => 'is-available', attr => 'non_show', 'true' => 'up', 'false' => 'down'},
        },
    'vserver_fcp_wwpn_alias_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'aliases-info'},
        'alias' => {'zapi_field' => 'aliases-alias', attr => 'aliases-info'},
        'wwpn' => {'zapi_field' => 'aliases-wwpn', attr => 'aliases-info'},
        },
    'vserver_iscsi_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'iscsi-service-info'},
        'target_name' => {'zapi_field' => 'node-name', attr => 'iscsi-service-info'},
        'target_alias' => {'zapi_field' => 'alias-name', attr => 'iscsi-service-info'},
        'status_admin' => {'zapi_field' => 'is-available', attr => 'iscsi-service-info', 'true' => 'up', 'false' => 'down'},
        'max_error_recovery_level' => {'zapi_field' => 'not_exist', attr => ''},
        'retain_timeout' => {'zapi_field' => 'not_exist', attr => ''},
        'login_timeout' => {'zapi_field' => 'not_exist', attr => ''},
        'max_conn_per_session' => {'zapi_field' => 'not_exist', attr => ''},
        'max_ios_per_session' => {'zapi_field' => 'not_exist', attr => ''},
        'tcp_window_size' => {'zapi_field' => 'not_exist', attr => ''},
        },
    'volume_show' => {
        'vserver' => {'zapi_field' => 'owning-vserver-name', attr => 'volume-id-attributes'},
        'volume' => {'zapi_field' => 'name', attr => 'volume-id-attributes'},
        'aggregate' => {'zapi_field' => 'containing-aggregate-name', attr => 'volume-id-attributes'},
        'size' => {'zapi_field' => 'size', attr => 'volume-space-attributes'},
        'name_ordinal' => {'zapi_field' => 'name-ordinal', attr => 'volume-id-attributes'},
        'dsid' => {'zapi_field' => 'dsid', attr => 'volume-id-attributes'},
        'msid' => {'zapi_field' => 'msid', attr => 'volume-id-attributes'},
        'state' => {'zapi_field' => 'state', attr => 'volume-state-attributes'},
        'type' => {'zapi_field' => 'type', attr => 'volume-id-attributes'},
        'volume_style' => {'zapi_field' => 'not_exist', attr => ''},
        'is_cluster_volume' => {'zapi_field' => 'is-cluster-volume', attr => 'volume-state-attributes'},
        'policy' => {'zapi_field' => 'pocily', attr => 'volume-export-attributes'},
        'user' => {'zapi_field' => 'user-id', attr => 'volume-security-unix-attributes'},
        'group' => {'zapi_field' => 'group-id', attr => 'volume-security-unix-attributes'},
        'security_style' => {'zapi_field' => 'style', attr => 'volume-security-attributes'},
        'unix_permissions' => {'zapi_field' => 'permissions', attr => 'volume-security-unix-attributes'},
        'junction_path' => {'zapi_field' => 'junction-path', attr => 'volume-id-attributes'},
        'junction_path_source' => {'zapi_field' => 'junction-path', attr => 'volume-id-attributes'},
        'junction_active' => {'zapi_field' => 'is-junction-active', attr => 'volume-state-attributes'},
        'parent' => {'zapi_field' => 'junction-parent-name', attr => 'volume-id-attributes'},
        'vsroot' => {'zapi_field' => 'is-vserver-root', attr => 'volume-state-attributes'},
        'comment' => {'zapi_field' => 'comment', attr => 'volume-id-attributes'},
        'available' => {'zapi_field' => 'size-available', attr => 'volume-space-attributes'},
        'total' => {'zapi_field' => 'size-total', attr => 'volume-space-attributes'},
        'used' => {'zapi_field' => 'size-used', attr => 'volume-space-attributes'},
        'percent_used' => {'zapi_field' => 'percentage-size-used', attr => 'volume-space-attributes'},
        'autosize' => {'zapi_field' => 'is-enabled', attr => 'volume-autosize-attributes'},
        'max_autosize' => {'zapi_field' => 'maximum-size', attr => 'volume-autosize-attributes'},
        'autosize_increment' => {'zapi_field' => 'increment-size', attr => 'volume-autosize-attributes'},
        'files' => {'zapi_field' => 'files-total', attr => 'volume-inode-attributes'},
        'files_used' => {'zapi_field' => 'files-used', attr => 'volume-inode-attributes'},
        'maxdir_size' => {'zapi_field' => 'max-dir-size', attr => 'volume-directory-attributes'},
        'space_guarantee' => {'zapi_field' => 'space-guarantee', attr => 'volume-space-attributes'},
        'space_guarantee_enabled' => {'zapi_field' => 'is-space-guarantee-enabled', attr => 'volume-space-attributes'},
        'min_readahead' => {'zapi_field' => 'minimal-read-ahead', attr => 'volume-performance-attributes'},
        'atime_update' => {'zapi_field' => 'is-atime-update-enabled', attr => 'volume-performance-attributes'},
        'snapdir_access' => {'zapi_field' => 'snapdir-access-enabled', attr => 'volume-snapshot-attributes'},
        'percent_snapshot_space' => {'zapi_field' => 'percentage-snapshot-reserved', attr => 'volume-space-attributes'},
        'snapshot_space_used' => {'zapi_field' => 'percentage-snapshot-reserve-used', attr => 'volume-space-attributes'},
        'snapshot_policy' => {'zapi_field' => 'snapshot-policy', attr => 'volume-snapshot-attributes'},
        'create_time' => {'zapi_field' => 'oem-character-set', attr => 'volume-language-attributes'},
        'language' => {'zapi_field' => 'language', attr => 'volume-language-attributes'},
        'stripe_count' => {'zapi_field' => 'not_exist', attr => ''},
        'stripe_width' => {'zapi_field' => 'stripe-width', attr => 'volume-striping-attributes'},
        'striping_epoch' => {'zapi_field' => 'not_exist', attr => ''},
        'properly_striped' => {'zapi_field' => 'not_exist', attr => ''},
        'concurrency' => {'zapi_field' => 'concurrency', attr => 'volume-striping-attributes'},
        'optimize' => {'zapi_field' => 'optimize', attr => 'volume-striping-attributes'},
        'clone_volume' => {'zapi_field' => 'not_exist', attr => ''},
        'antivirus_on_access_policy' => {'zapi_field' => 'on-access-policy', attr => 'volume-antivirus-attributes'},
        'flexcache_cache_policy' => {'zapi_field' => 'cache-policy', attr => 'volume-flexcache-attributes'},
        'flexcache_fill_policy' => {'zapi_field' => 'fill-policy', attr => 'volume-flexcache-attributes'},
        'uuid' => {'zapi_field' => 'uuid', attr => 'volume-id-attributes'},
        'nvfail' => {'zapi_field' => 'is-nvfail-enabled', attr => 'volume-state-attributes'},
        'in_nvfailed_state' => {'zapi_field' => 'in-nvfailed-state', attr => 'volume-state-attributes'},
        'filesys_size_fixed' => {'zapi_field' => 'is-filesys-size-fixed', attr => 'volume-space-attributes'},
        'overwrite_reserve' => {'zapi_field' => 'overwrite-reserve', attr => 'volume-space-attributes'},
        'fractional_reserve' => {'zapi_field' => 'percentage-fractional-reserve', attr => 'volume-space-attributes'},
        'auto_snapshots_enabled' => {'zapi_field' => 'auto-snapshots-enabled', attr => 'volume-snapshot-attributes'},
        'stripe_format' => {'zapi_field' => 'not_exist', attr => ''},
        'index_dir_enabled' => {'zapi_field' => 'index-dir-enabled', attr => 'volume-directory-attributes'},
        'load_sharing_source' => {'zapi_field' => 'not_exist', attr => ''},
        'move_target' => {'zapi_field' => 'not_exist', attr => ''},
        'physical_replica' => {'zapi_field' => 'is-replica-volume', attr => 'volume-mirror-attributes'},
        'max_write_alloc_blocks' => {'zapi_field' => 'max-write-alloc-blocks', attr => 'volume-performance-attributes'},
        'snapshot_clone_dependency' => {'zapi_field' => 'snapshot-clone-dependency-enabled', attr => 'volume-snapshot-attributes'},
        'create_ucode' => {'zapi_field' => 'is-create-ucode-enabled', attr => 'volume-language-attributes'},
        'convert_ucode' => {'zapi_field' => 'is-convert-ucode-enabled', attr => 'volume-language-attributes'},
        'space_mgmt_try_first' => {'zapi_field' => 'space-mgmt-option-try-first', attr => 'volume-space-attributes'},
        'sched_snap_name' => {'zapi_field' => 'not_exist', attr => ''},
        'is_inconsistent' => {'zapi_field' => 'is-inconsistent', attr => 'volume-state-attributes'},
        'is_quiesced_on_disk' => {'zapi_field' => 'is-quiesced-on-disk', attr => 'volume-state-attributes'},
        'is_quiesced_in_memory' => {'zapi_field' => 'is-quiesced-in-memory', attr => 'volume-state-attributes'},
        'transition_state' => {'zapi_field' => 'not_exist', attr => ''},
        'is_copied_for_transition' => {'zapi_field' => 'not_exist', attr => ''},
        'is_transitioned' => {'zapi_field' => 'not_exist', attr => ''},
        'is_sis_volume' => {'zapi_field' => 'is-sis-volume', attr => 'volume-sis-attributes'},
        'is_sis_logging_enabled' => {'zapi_field' => 'is-sis-logging-enabled', attr => 'volume-sis-attributes'},
        'sis_space_saved' => {'zapi_field' => 'total-space-saved', attr => 'volume-sis-attributes'},
        'sis_space_saved_percent' => {'zapi_field' => 'percentage-total-space-saved', attr => 'volume-sis-attributes'},
        'dedupe_space_saved' => {'zapi_field' => 'deduplication-space-saved', attr => 'volume-sis-attributes'},
        'dedupe_space_saved_percent' => {'zapi_field' => 'percentage-deduplication-space-saved', attr => 'volume-sis-attributes'},
        'compression_space_saved' => {'zapi_field' => 'compression-space-saved', attr => 'volume-sis-attributes'},
        'compression_space_saved_percent' => {'zapi_field' => 'percentage-compression-space-saved', attr => 'volume-sis-attributes'},
        'instance_uuid' => {'zapi_field' => 'instance-uuid', attr => 'volume-id-attributes'},
        'provenance_uuid' => {'zapi_field' => 'provenance-uuid', attr => 'volume-id-attributes'},
        'block_type' => {'zapi_field' => 'block-type', attr => 'volume-inode-attributes'},
        'flexcache_connection_status' => {'zapi_field' => 'not_exist', attr => ''},
        },
    'volume_qtree_show' => {
        'vserver'          => {'zapi_field' => 'vserver',        attr => 'qtree-info'},
        'path'             => {'zapi_field' => 'path',           attr => 'qtree-info'},
        'volume'           => {'zapi_field' => 'volume',         attr => 'qtree-info'},
        'qtree'            => {'zapi_field' => 'qtree',          attr => 'qtree-info'},
        'qtree-path'       => {'zapi_field' => 'not_exist',      attr => ''},
        'security-style'   => {'zapi_field' => 'security-style', attr => 'qtree-info'},
        'oplock-mode'      => {'zapi_field' => 'oplocks',        attr => 'qtree-info'},
        'status'           => {'zapi_field' => 'status',         attr => 'qtree-info'},
        'qtree-id'         => {'zapi_field' => 'qid',            attr => ''},
        'unix-permissions' => {'zapi_field' => 'not_exist',      attr => ''},
        },
    'lun_show' => {
        'vserver'          => {'zapi_field' => 'vserver',            attr => 'lun-info'},
        'path'             => {'zapi_field' => 'path',               attr => 'lun-info'},
        'qtree'            => {'zapi_field' => 'qtree',              attr => 'lun-info'},
        'volume'           => {'zapi_field' => 'volume',             attr => 'lun-info'},
        'lun'              => {'zapi_field' => 'not_exist',          attr => ''},
        'size'             => {'zapi_field' => 'size',               attr => 'lun-info'},
        'prefix_size'      => {'zapi_field' => 'prefix-size',        attr => 'lun-info'},
        'extent_size'      => {'zapi_field' => 'not_exist',          attr => ''},
        'suffix_size'      => {'zapi_field' => 'not_exist',          attr => ''},
        'serial'           => {'zapi_field' => 'serial-number',      attr => 'lun-info'},
        'comment'          => {'zapi_field' => 'not_exist',          attr => ''},
        'state'            => {'zapi_field' => 'online',             attr => 'lun-info', 'true' => 'online', 'false' => 'offline'},
        'uuid'             => {'zapi_field' => 'uuid',               attr => 'lun-info'},
        'mapped'           => {'zapi_field' => 'mapped',             attr => 'lun-info', 'true' =>  'mapped', 'false' => 'unmapped'},
        'vdiskId'          => {'zapi_field' => 'not_exist',          attr => ''},
        'nvfail'           => {'zapi_field' => 'not_exist',          attr => ''},
        'block_size'       => {'zapi_field' => 'block-size',         attr => 'lun-info'},
        'device_legacy_id' => {'zapi_field' => 'device-id',          attr => 'lun-info'},
        'device_binary_id' => {'zapi_field' => 'device-binary-id',   attr => 'lun-info'},
        'device_text_id'   => {'zapi_field' => 'device-text-id',     attr => 'lun-info'},
        'read_only'        => {'zapi_field' => 'read-only',          attr => 'lun-info'},
        'size_used'        => {'zapi_field' => 'sized-used',         attr => 'lun-info'},
        'ostype'           => {'zapi_field' => 'multiprotocol-type', attr => 'lun-info'},
        'inconsistent_filesystem' => {'zapi_field' => 'not_exist',   attr => ''},
        'inconsistent_blocks'     => {'zapi_field' => 'not_exist',   attr => ''},
        'creation_timestamp'      => {'zapi_field' => 'creation-timestamp',     attr => ''},
        'space_reserve_honored'   => {'zapi_field' => 'not_exist',              attr => ''},
        'space_reserve'    => {'zapi_field' => 'is-space-reservation-enabled',  attr => 'lun-info', 'true' => 'enabled', 'false' => 'disabled'},
        'space_allocation' => {'zapi_field' => 'is-space-alloc-enabled',        attr => 'lun-info', 'true' => 'enabled', 'false' => 'disabled'},
        },
    'lun_igroup_show' => {
        'vserver' => {'zapi_field' => 'vserver',                      attr => 'initiator-group-info'},
        'igroup'  => {'zapi_field' => 'initiator-group-name',         attr => 'initiator-group-info'},
        'protocol'=> {'zapi_field' => 'initiator-group-type',         attr => 'initiator-group-info'},
        'ostype'  => {'zapi_field' => 'initiator-group-os-type',      attr => 'initiator-group-info'},
        'portset' => {'zapi_field' => 'initiator-group-portset-name', attr => 'initiator-group-info'},
        'initiator'   => {'zapi_field' => 'initiator-name',              attr => 'initiator-info'},
        'uuid'        => {'zapi_field' => 'initiator-group-uuid',        attr => 'initiator-group-info'},
        'alua'        => {'zapi_field' => 'initiator-group-alua-enabled',attr => 'initiator-group-info'},
        'init_details'=> {'zapi_field' => 'not_exist', attr => ''},
        },
    'lun_mapped_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'lun-map-info'},
        'path' => {'zapi_field' => 'path', attr => 'lun-map-info'},
        'volume' => {'zapi_field' => 'not_exist', attr => ''},
        'qtree' => {'zapi_field' => 'not_exist', attr => ''},
        'lun' => {'zapi_field' => 'not_exist', attr => ''},
        'igroup' => {'zapi_field' => 'initiator-group', attr => 'lun-map-info'},
        'os' => {'zapi_field' => 'not_exist', attr => ''},
        'type' => {'zapi_field' => 'not_exist', attr => ''},
        'luno' => {'zapi_field' => 'lun-id', attr => 'lun-map-info'},
        'portset' => {'zapi_field' => 'not_exist', attr => ''},
        'alua' => {'zapi_field' => 'not_exist', attr => ''},
        'initiators' => {'zapi_field' => 'not_exist', attr => ''},
        },
    'lun_stats' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'lun-stats-info'},
        'volume' => {'zapi_field' => 'not_exist', attr => ''},
        'qtree' => {'zapi_field' => 'not_exist', attr => ''},
        'lun' => {'zapi_field' => 'not_exist', attr => ''},
        'errors' => {'zapi_field' => 'scsi-errors', attr => 'lun-stats-info'},
        },
    'lun_maxsize' => {
        'vserver' => {'zapi_field' => 'not_exist', attr => ''},
        'volume' => {'zapi_field' => 'not_exist', attr => ''},
        'qtree' => {'zapi_field' => 'not_exist', attr => ''},
        'lun' => {'zapi_field' => 'not_exist', attr => ''},
        'complete_ss_reserve' => {'zapi_field' => 'with-complete-snapshot-reserve', attr => 'non_show'},
        'ss_reserve' => {'zapi_field' => 'with-snapshot-reserve', attr => 'non_show'},
        'without_ss_reserve' => {'zapi_field' => 'without-snapshot-reserve', attr => 'non_show'},
        },
    'lun_geometry' => {
        'vserver' => {'zapi_field' => 'not_exist', attr => ''},
        'volume' => {'zapi_field' => 'not_exist', attr => ''},
        'qtree' => {'zapi_field' => 'not_exist', attr => ''},
        'lun' => {'zapi_field' => 'not_exist', attr => ''},
        'size' => {'zapi_field' => 'size', attr => 'non_show'},
        'bytes_per_sector' => {'zapi_field' => 'bytes-per-sector', attr => 'non_show'},
        'sectors_per_track' => {'zapi_field' => 'sectors-per-track', attr => 'non_show'},
        'tracks_per_cylinder' => {'zapi_field' => 'tracks-per-cylinder', attr => 'non_show'},
        'sectors_per_cylinder' => {'zapi_field' => 'not_exist', attr => ''},
        'cylinders' => {'zapi_field' => 'cylinders', attr => 'non_show'},
        'max_resize_size' => {'zapi_field' => 'max-resize-size', attr => 'non_show'},
        'sectors' => {'zapi_field' => 'not_exist', attr => ''},
        'cyl_size' => {'zapi_field' => 'not_exist', attr => ''},
        'dev_size' => {'zapi_field' => 'not_exist', attr => ''},
        },
    # FYI - This call is hard-coded to cli in VserverHelper.pm.  There are not
    # zapis for routing group cmds.  Note the attr listed below is for routes.
    # routing-group zapis may arrive in SN0.
    'network_routing_groups_show' => {
        'instance'     => {'zapi_field' => 'not_exist',    attr => ''},
        'vserver'      => {'zapi_field' => 'vserver',      attr => 'routing-group-route-info'},
        'routing_group'=> {'zapi_field' => 'routing-group',attr => 'routing-group-route-info'},
        'role'         => {'zapi_field' => 'not_exist',    attr => ''},
        'metric'       => {'zapi_field' => 'metric',       attr => 'routing-group-route-info'},
        },
    'network_routing_groups_route_show' => {
        'instance'     => {'zapi_field' => 'not_exist',          attr => ''},
        'vserver'      => {'zapi_field' => 'vserver',            attr => 'routing-group-route-info'},
        'routing_group'=> {'zapi_field' => 'routing-group',      attr => 'routing-group-route-info'},
        'destination'  => {'zapi_field' => 'destination-address',attr => 'routing-group-route-info'},
        'role'         => {'zapi_field' => 'not_exist',          attr => ''},
        'metric'       => {'zapi_field' => 'metric',             attr => 'routing-group-route-info'},
        },
    'network_interface_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'net-interface-info'},
        'server' => {'zapi_field' => 'vserver', attr => 'net-interface-info'},
        'lif' => {'zapi_field' => 'interface-name', attr => 'net-interface-info'},
        'vif' => {'zapi_field' => 'interface-name', attr => 'net-interface-info'},
        'role' => {'zapi_field' => 'role', attr => 'net-interface-info'},
        'data_protocol' => {'zapi_field' => 'data-protocol', attr => 'data-protocols'},
        'home_node' => {'zapi_field' => 'home-node', attr => 'net-interface-info'},
        'home_port' => {'zapi_field' => 'home-port', attr => 'net-interface-info'},
        'curr_node' => {'zapi_field' => 'current-node', attr => 'net-interface-info'},
        'curr_port' => {'zapi_field' => 'current-port', attr => 'net-interface-info'},
        'status_oper' => {'zapi_field' => 'operational-status', attr => 'net-interface-info'},
        'status_extended' => {'zapi_field' => 'not_exist', attr => ''},
        'numeric_id' => {'zapi_field' => 'not_exist', attr => ''},
        'is_home' => {'zapi_field' => 'is-home', attr => 'net-interface-info'},
        'address' => {'zapi_field' => 'address', attr => 'net-interface-info'},
        'address_family' => {'zapi_field' => 'address-family', attr => 'net-interface-info'},
        'netmask' => {'zapi_field' => 'netmask', attr => 'net-interface-info'},
        'auto' => {'zapi_field' => 'is-ipv4-link-local', attr => 'net-interface-info'},
        'netmask_length' => {'zapi_field' => 'netmask-length', attr => 'net-interface-info'},
        'routing_group' => {'zapi_field' => 'routing-group-name', attr => 'net-interface-info'},
        'status_admin' => {'zapi_field' => 'administrative-status', attr => 'net-interface-info'},
        'failover_policy' => {'zapi_field' => 'failover-policy', attr => 'net-interface-info'},
        'firewall_policy' => {'zapi_field' => 'firewall-policy', attr => 'net-interface-info'},
        'auto_revert' => {'zapi_field' => 'is-auto-revert', attr => 'net-interface-info'},
        'sticky' => {'zapi_field' => 'not_exist', attr => ''},
        'use_failover_group' => {'zapi_field' => 'use-failover-group', attr => 'net-interface-info'},
        'dns_zone' => {'zapi_field' => 'dns-domain-name', attr => 'net-interface-info'},
        'listen_for_dns_query' => {'zapi_field' => 'listen-for-dns-query', attr => 'net-interface-info'},
        'allow_lb_migrate' => {'zapi_field' => 'not_exist', attr => ''},
        'lb_weight' => {'zapi_field' => 'not_exist', attr => ''},
        'failover_group' => {'zapi_field' => 'failover-group', attr => 'net-interface-info'},
        'wwpn' => {'zapi_field' => 'not_exist', attr => ''},
        'comment' => {'zapi_field' => 'comment', attr => 'net-interface-info'},
        },
    'network_port_show' => {
        'node' => {'zapi_field' => 'node',       attr => 'net-port-info'},
        'port' => {'zapi_field' => 'port',       attr => 'net-port-info'},
        'role' => {'zapi_field' => 'role',       attr => 'net-port-info'},
        'link' => {'zapi_field' => 'link-status',attr => 'net-port-info', 'unknown' => 'off'},
        'mtu'  => {'zapi_field' => 'mtu',        attr => 'net-port-info'},
        'autonegotiate_admin'=> {'zapi_field' => 'is-administrative-auto-negotiate',attr => 'net-port-info'},
        'autonegotiate_oper' => {'zapi_field' => 'is-operational-auto-negotiate',   attr => 'net-port-info'},
        'duplex_admin'       => {'zapi_field' => 'administrative-duplex',           attr => 'net-port-info'},
        'duplex_oper'        => {'zapi_field' => 'operational-duplex',              attr => 'net-port-info'},
        'speed_admin'        => {'zapi_field' => 'administrative-speed',            attr => 'net-port-info'},
        'speed_oper'         => {'zapi_field' => 'operational-speed',               attr => 'net-port-info'},
        'flowcontrol_admin'  => {'zapi_field' => 'administrative-flowcontrol',      attr => 'net-port-info'},
        'flowcontrol_oper'   => {'zapi_field' => 'operational-flowcontrol',         attr => 'net-port-info'},
        'ifgrp_distr_func'   => {'zapi_field' => 'ifgrp-distribution-function',     attr => 'net-port-info'},
        'mac'        => {'zapi_field' => 'mac-address',         attr => 'net-port-info'},
        'up_admin'   => {'zapi_field' => 'is-administrative-up',attr => 'net-port-info'},
        'type'       => {'zapi_field' => 'port-type',           attr => 'net-port-info'},
        'ifgrp_node' => {'zapi_field' => 'ifgrp-node',          attr => 'net-port-info'},
        'ifgrp_port' => {'zapi_field' => 'ifgrp-node',          attr => 'net-port-info'},
        'ifgrp_mode' => {'zapi_field' => 'ifgrp-node',          attr => 'net-port-info'},
        'vlan_node'  => {'zapi_field' => 'vlan-node',           attr => 'net-port-info'},
        'vlan_port'  => {'zapi_field' => 'vlan-port',           attr => 'net-port-info'},
        'vlan_tag'   => {'zapi_field' => 'vlan-id',             attr => 'net-port-info'},
        },
    'lun_portset_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'portset-info'},
        'portset' => {'zapi_field' => 'portset-name', attr => 'portset-port-info'},
        'port_name' => {'zapi_field' => 'portset-port-name', attr => 'portset-info'},
        'protocol' => {'zapi_field' => 'portset-type', attr => 'portset-info'},
        'port_count' => {'zapi_field' => 'portset-port-total', attr => 'portset-info'},
        'igroups' => {'zapi_field' => 'initiator-group-name', attr => 'initiator-group-info'},
        },
    'vserver_iscsi_tpgroup_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'iscsi-tpgroup-list-entry-info'},
        'tpgroup' => {'zapi_field' => 'tpgroup-name', attr => 'iscsi-tpgroup-list-entry-info'},
        'tpgroup_tag' => {'zapi_field' => 'tpgroup-tag', attr => 'iscsi-tpgroup-list-entry-info'},
        'lif' => {'zapi_field' => 'interface-name', attr => 'interface-list-entry-info'},
        'is_user_defined' => {'zapi_field' => 'is-user-defined', attr => 'iscsi-tpgroup-list-entry-info'},
        },
    'vserver_iscsi_security_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'iscsi-security-entry-info'},
        'initiator_name' => {'zapi_field' => 'initiator', attr => 'iscsi-security-entry-info'},
        'auth_type' => {'zapi_field' => 'auth-type', attr => 'iscsi-security-entry-info'},
        'user_name' => {'zapi_field' => 'user-name', attr => 'iscsi-security-entry-info'},
        'outbound_user_name' => {'zapi_field' => 'outbound-user-name', attr => 'iscsi-security-entry-info'},
        'auth_chap_policy' => {'zapi_field' => 'auth-chap-policy', attr => 'iscsi-security-entry-info'},
        },
    'vserver_iscsi_interface_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'iscsi-interface-list-entry-info'},
        'lif' => {'zapi_field' => 'interface-name', attr => 'iscsi-interface-list-entry-info'},
        'status_admin' => {'zapi_field' => 'not_exist', attr => ''},
        'status_oper' => {'zapi_field' => 'not_exist', attr => ''},
        'enabled' => {'zapi_field' => 'is-interface-enabled', attr => 'iscsi-interface-list-entry-info'},
        'address' => {'zapi_field' => 'ip-address', attr => 'iscsi-interface-list-entry-info'},
        'ip_port' => {'zapi_field' => 'ip-port', attr => 'iscsi-interface-list-entry-info'},
        'curr_node' => {'zapi_field' => 'current-node', attr => 'iscsi-interface-list-entry-info'},
        'curr_port' => {'zapi_field' => 'current-port', attr => 'iscsi-interface-list-entry-info'},
        'is_home' => {'zapi_field' => 'not_exist', attr => ''},
        'tpgroup' => {'zapi_field' => 'tpgroup-name', attr => 'iscsi-interface-list-entry-info'},
        'tpgroup_tag' => {'zapi_field' => 'tpgroup-tag', attr => 'iscsi-interface-list-entry-info'},
        'relative_port_id' => {'zapi_field' => 'relative-port-id', attr => 'iscsi-interface-list-entry-info'},
        },
    'vserver_iscsi_portal_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'iscsi-interface-list-entry-info'},
        'lif' => {'zapi_field' => 'interface-name', attr => 'iscsi-interface-list-entry-info'},
        'status_admin' => {'zapi_field' => 'not_exist', attr => ''},
        'status_oper' => {'zapi_field' => 'not_exist', attr => ''},
        'enabled' => {'zapi_field' => 'is-interface-enabled', attr => 'iscsi-interface-list-entry-info'},
        'address' => {'zapi_field' => 'ip-address', attr => 'iscsi-interface-list-entry-info'},
        'ip_port' => {'zapi_field' => 'ip-port', attr => 'iscsi-interface-list-entry-info'},
        'curr_node' => {'zapi_field' => 'current-node', attr => 'iscsi-interface-list-entry-info'},
        'curr_port' => {'zapi_field' => 'current-port', attr => 'iscsi-interface-list-entry-info'},
        'is_home' => {'zapi_field' => 'not_exist', attr => ''},
        'tpgroup' => {'zapi_field' => 'tpgroup-name', attr => 'iscsi-interface-list-entry-info'},
        'tpgroup_tag' => {'zapi_field' => 'tpgroup-tag', attr => 'iscsi-interface-list-entry-info'},
        'relative_port_id' => {'zapi_field' => 'relative-port-id', attr => 'iscsi-interface-list-entry-info'},
        },
    # Note for iscsi connection show: Most of the items below marked 'not_exist'
    # can be fetched from the iscsi_session_show zapi. They are nested down in
    # iscsi-session-list-entry-info -> iscsi-session-connection-list-entry-info.
    # Possibly a future to-do?
    'vserver_iscsi_connection_show' => {
        'vserver'               => {'zapi_field' => 'vserver',           attr => 'iscsi-connection-list-entry-info'},
        'tpgroup'               => {'zapi_field' => 'tpgroup-name',      attr => 'iscsi-connection-list-entry-info'},
        'tsih'                  => {'zapi_field' => 'session-id',        attr => 'iscsi-connection-list-entry-info'},
        'connection_id'         => {'zapi_field' => 'connection-id',     attr => 'iscsi-connection-list-entry-info'},
        'connection_state'      => {'zapi_field' => 'connection-state',  attr => 'iscsi-connection-list-entry-info'},
        'has_session'           => {'zapi_field' => 'has-session',       attr => 'iscsi-connection-list-entry-info'},
        'lif'                   => {'zapi_field' => 'interface-name',    attr => 'iscsi-connection-list-entry-info'},
        'tpgroup_tag'           => {'zapi_field' => 'tpgroup-tag',       attr => 'iscsi-connection-list-entry-info'},
        'local_address'         => {'zapi_field' => 'local-ip-address',  attr => 'iscsi-connection-list-entry-info'},
        'local_ip_port'         => {'zapi_field' => 'local-ip-port',     attr => 'iscsi-connection-list-entry-info'},
        'authentication_method' => {'zapi_field' => 'not_exist',         attr => ''},
        'data_digest_enabled'   => {'zapi_field' => 'not_exist',         attr => ''},
        'header_digest_enabled' => {'zapi_field' => 'not_exist',         attr => ''},
        'rcv_window_size'       => {'zapi_field' => 'not_exist',         attr => ''},
        'initiator_mrdsl'       => {'zapi_field' => 'not_exist',         attr => ''},
        'remote_address'        => {'zapi_field' => 'remote-ip-address', attr => 'iscsi-connection-list-entry-info'},
        'remote_ip_port'        => {'zapi_field' => 'remote-ip-port',    attr => 'iscsi-connection-list-entry-info'},
        'target_mrdsl'          => {'zapi_field' => 'not_exist',         attr => ''},
        'node'                  => {'zapi_field' => 'not_exist',         attr => ''},
        },
    # Note for iscsi session show.  The cli returns a field called connection-ids.
    # The zapi does not return this explicitly.  However, the information is
    # availble if you dig down to:
    # iscsi-session-list-entry-info -> iscsi-session-connection-list-entry-info.
    # Possibly a future to-do
    'vserver_iscsi_session_show' => {
        'vserver'                => {'zapi_field' => 'vserver',                attr => 'iscsi-session-list-entry-info'},
        'tpgroup'                => {'zapi_field' => 'tpgroup-name',           attr => 'iscsi-session-list-entry-info'},
        'tsih'                   => {'zapi_field' => 'target-session-id',      attr => 'iscsi-session-list-entry-info'},
        'cmd_window_size'        => {'zapi_field' => 'cmd-window-size',        attr => 'iscsi-session-list-entry-info'},
        'data_pdu_in_order'      => {'zapi_field' => 'data-pdu-in-order',      attr => 'iscsi-session-list-entry-info'},
        'data_sequence_in_order' => {'zapi_field' => 'data-sequence-in-order', attr => 'iscsi-session-list-entry-info'},
        'default_time_to_retain' => {'zapi_field' => 'default-time-to-retain', attr => 'iscsi-session-list-entry-info'},
        'default_time_to_wait'   => {'zapi_field' => 'default-time-to-wait',   attr => 'iscsi-session-list-entry-info'},
        'error_recovery_level'   => {'zapi_field' => 'error-recovery-level',   attr => 'iscsi-session-list-entry-info'},
        'first_burst_length'     => {'zapi_field' => 'first-burst-length',     attr => 'iscsi-session-list-entry-info'},
        'immediate_data_enabled' => {'zapi_field' => 'immediate-data-enabled', attr => 'iscsi-session-list-entry-info'},
        'initiator_alias'        => {'zapi_field' => 'initiator-aliasname',    attr => 'iscsi-session-list-entry-info'},
        'initial_r2t_enabled'    => {'zapi_field' => 'initial-r2t-enabled',    attr => 'iscsi-session-list-entry-info'},
        'initiator_name'         => {'zapi_field' => 'initiator-nodename',     attr => 'iscsi-session-list-entry-info'},
        'isid'                   => {'zapi_field' => 'isid',                   attr => 'iscsi-session-list-entry-info'},
        'max_burst_length'       => {'zapi_field' => 'max-burst-length',       attr => 'iscsi-session-list-entry-info'},
        'max_connections '       => {'zapi_field' => 'max-connections',        attr => 'iscsi-session-list-entry-info'},
        'max_outstanding_r2t'    => {'zapi_field' => 'max-outstanding-r2t',    attr => 'iscsi-session-list-entry-info'},
        'session_type'           => {'zapi_field' => 'session-type',           attr => 'iscsi-session-list-entry-info'},
        'tpgroup_tag'            => {'zapi_field' => 'tpgroup-tag',            attr => 'iscsi-session-list-entry-info'},
        'node'                   => {'zapi_field' => 'not_exist',              attr => ''},
        'connection_ids'         => {'zapi_field' => 'not_exist',              attr => ''},
        },
    'vserver_iscsi_initiator_show' => {
        'vserver'         => {'zapi_field' => 'vserver',              attr => 'iscsi-initiator-list-entry-info'},
        'tpgroup'         => {'zapi_field' => 'tpgroup-name',         attr => 'iscsi-initiator-list-entry-info'},
        'tsih'            => {'zapi_field' => 'target-session-id',    attr => 'iscsi-initiator-list-entry-info'},
        'initiator_name'  => {'zapi_field' => 'initiator-nodename',   attr => 'iscsi-initiator-list-entry-info'},
        'initiator_alias' => {'zapi_field' => 'initiator-aliasname',  attr => 'iscsi-initiator-list-entry-info'},
        'tpgroup_tag'     => {'zapi_field' => 'tpgroup-tag',          attr => 'iscsi-initiator-list-entry-info'},
        'isid'            => {'zapi_field' => 'isid',                 attr => 'iscsi-initiator-list-entry-info'},
        'igroup'          => {'zapi_field' => 'initiator-group-name', attr => 'initiator-group-list-info'},
        },
    'vserver_iscsi_interface_accesslist_show' => {
        'vserver'         => {'zapi_field' => 'vserver',        attr => 'iscsi-accesslist-list-entry-info'},
        'initiator_name'  => {'zapi_field' => 'initiator',      attr => 'iscsi-accesslist-list-entry-info'},
        'lif'             => {'zapi_field' => 'interface-name', attr => 'iscsi-accesslist-list-entry-info'},
        },
    'vserver_iscsi_isns_show' => {
        'vserver'                => {'zapi_field' => 'vserver',                          attr => 'iscsi-isns-info'},
        'status_admin'           => {'zapi_field' => 'is-isns-enabled',                  attr => 'iscsi-isns-info'},
        'entity_id'              => {'zapi_field' => 'isns-entity-id',                   attr => 'iscsi-isns-info'},
        'address'                => {'zapi_field' => 'isns-ip-addr',                     attr => 'iscsi-isns-info'},
        'last_successful_update' => {'zapi_field' => 'last-successful-update-timestamp', attr => 'iscsi-isns-info'},
        'last_update_attempt'    => {'zapi_field' => 'last-update-attempt-timestamp',    attr => 'iscsi-isns-info'},
        'last_update_result'     => {'zapi_field' => 'last-update-result',               attr => 'iscsi-isns-info'},
        'last_processed'         => {'zapi_field' => 'not_exist',                        attr => ''},
        },
    # This is a NON_SHOW instruction, it does not return an attribute key, so
    # the attr key should have the value 'non_show' to be translatable to CLI.
    'vserver_iscsi_alias_get' => {
        'vserver'      => {'zapi_field' => 'not_exist',        attr => ''},
        'target_alias' => {'zapi_field' => 'alias-name',       attr => 'non_show'},
        'clear'        => {'zapi_field' => 'is-alias-assigned',attr => 'non_show'},
        },
    # This is a NON_SHOW instruction, it does not return an attribute key, so
    # the attr key should have the value 'non_show' to be translatable to CLI.
    'vserver_iscsi_nodename_get' => {
        'vserver'     => {'zapi_field' => 'not_exist', attr => ''},
        'target_name' => {'zapi_field' => 'node-name', attr => 'non_show'},
        },
    # This is a NON_SHOW instruction, it does not return an attribute key, so
    # the attr key should have the value 'non_show' to be translatable to CLI.
    'vserver_iscsi_status' => {
        'vserver'     => {'zapi_field' => 'not_exist',    attr => ''},
        'status_admin'=> {'zapi_field' => 'is-available', attr => 'non_show', 'true' => 'up', 'false' => 'down'},
        },
    'vserver_fcp_initiator_show' => {
        'vserver'     => {'zapi_field' => 'vserver', attr => 'fcp-adapter-initiators-info'},
        'lif'         => {'zapi_field' => 'adapter', attr => 'fcp-adapter-initiators-info'},
        'wwpn'        => {'zapi_field' => 'port-name',   attr => 'fcp-connected-initiator-info'},
        'port_address'=> {'zapi_field' => 'port-address',attr => 'fcp-connected-initiator-info'},
        'wwnn'        => {'zapi_field' => 'node-name',   attr => 'fcp-connected-initiator-info'},
        'alias'       => {'zapi_field' => 'portname-alias',       attr => 'portname-alias-name'},
        'igroup'      => {'zapi_field' => 'initiator-group-name', attr => 'initiator-group-list-info'},
        },
    'vserver_fcp_portname_show' => {
        'vserver' => {'zapi_field' => 'vserver', attr => 'fcp-port-name-info'},
        'lif' => {'zapi_field' => 'fcp-adapter', attr => 'fcp-port-name-info'},
        'is_used' => {'zapi_field' => 'is-used', attr => 'fcp-port-name-info'},
        'wwpn' => {'zapi_field' => 'port-name', attr => 'fcp-port-name-info'},
        },
    # This is a NON_SHOW instruction, it does not return an attribute key, so
    # the attr key should have the value 'non_show' to be translatable to CLI.
    'vserver_fcp_nodename_get' => {
        'vserver'     => {'zapi_field' => 'not_exist', attr => ''},
        'target_name' => {'zapi_field' => 'node-name', attr => 'non_show'},
        },
);

1;

#############################################################################
#                             POD Documentation
#############################################################################
## @pod here

=pod

=head1 Package Name

AttrTable.pm

=head1 Author

Korry Nguyen (dl-ssan-automation)

=head1 Description

AttrTable.pm provides cdef show commands with an array of expected returned attributes for CLI.
A common parser method will use this table to return a homogeneous output for either CLI or ZAPI.
If attribute does not exist from ZAPI output, it will fill in 'zapi_na'.

=head1 See Also

See CommonInterface.pm for further interface detail.

=head1 Usage

=over 4

=item Examples

  #
  # Get nodes info in a cluster via CLI interface
  #
  - Displaying nodes in cluster
	Name:      ssan-3170-23a
	Model:     FAS3170
	Release:   RollingRock__8.1.0
	Branch:    fsb_rr
	Build:     1215636
	SysId:     0151747370
	Health:    true
	Node uuid: 05ce02f2-edcc-11df-b609-79c8d92c265f

	Name:      ssan-3170-23b
	Model:     FAS3170
	Release:   RollingRock__8.1.0
	Branch:    fsb_rr
	Build:     1215636
	SysId:     0151747457
	Health:    true
	Node uuid: 0369e038-edcc-11df-8ee4-b73f657546dd

  -- PASS

  #
  # Get nodes info in a cluster via ZAPI interface
  #
  - Displaying nodes in cluster
	Name:      ssan-3170-23a
	Model:     FAS3170
	Release:   RollingRock__8.1.0
	Branch:    fsb_rr
	Build:     1215636
	SysId:     151747370
	Health:    zapi_na
	Node uuid: 05ce02f2-edcc-11df-b609-79c8d92c265f

	Name:      ssan-3170-23b
	Model:     FAS3170
	Release:   RollingRock__8.1.0
	Branch:    fsb_rr
	Build:     1215636
	SysId:     151747457
	Health:    zapi_na
	Node uuid: 0369e038-edcc-11df-8ee4-b73f657546dd

  -- PASS

=cut
