# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provides an interface to a Disk object
##
## @author korry@netapp.com, dl-ssan-test
## @status Shared

package San::ControllerCluster::Filer10M::Disk;

# compiler directives
use strict;

# inherit base class
use base qw(San::CommonInterface);

###############################################################################
# Method Name: new()
# Objective: Conrtuctor for a new Disk object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class, %args) = @_;

    my ($self);

    # initialize parameter
    $self = {};

    # bless instance into class
    bless $self, $class;

    # remove the dash '-' from input argument list which will be used to 
    # create 'get' accessor methods on the fly
    %args = &SanUtil::strip_dash(%args);

    # verify mandatory cmd argument
    unless (defined $args{filer} and length $args{filer}) {
        $self->_set_err("mandatory parameter \"-filer\" not specified ");
        return $self;
    }

    # call SUPER::_new from the base class (San::CommonInterface) 
    # to contruct the Disk object
    $self = $class->SUPER::_new(-name => $args{disk},
                                -filer => $args{filer},
                                %args);
    if ($self->error()) {
        $self->_set_err("failed to create a disk object", $self->errmsg());
        return $self;
    }

    # dump only first depth of hash if debug_level < 3
    # dump everything of debug_level => 3
    $self->sanlog(-msg => $self->dumper_get(\%args, $self->get_debug_level()),
                  -msg_level => 2);

    # set and cache all Disk elements
    foreach my $key (sort keys %args) {
        $self->{lc($key)} = $args{$key};
        push @{$self->{fields}}, lc($key);
    }

    # initialize object by creating and update it has not been initialized
    unless ($args{initialized}) {
        if ($self->{enable_cache}) {
            $self->refresh(%args);
            if ($self->error()) {
                $self->_set_err("failed to refresh disk object", $self->errmsg());
                return $self;
            }
        }
        $self->{initialized} = $args{initialized};
    } 
    return $self;
} ## end sub new

###############################################################################
# Method Name: refresh()
# Objective: Refresh object's data
# Details:   See POD documentation at the end of this file.
###############################################################################
sub refresh {
    my ($self, %args) = @_;

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    %args = (%args,
             cmd => "storage_disk_show",
             $self->_get_param_defaults());

    # call SUPER::refresh from base class (San::CommonInterface) to update
    # disk object data
    $self->SUPER::refresh(%args);
    if ($self->error()) {
        $self->_set_err("failed to refresh object's data", $self->errmsg());
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
} ## end sub refresh

################################################################################
# Method Name: is_zeroed()
# Objective:   Is disk zeroed
# Details:     See POD documentation at the end of this file
################################################################################
sub is_zeroed {
    my ($self, %args) = @_;

    $args{field} = "zeroed";

    # the _get_field() will handle error
    if ($self->_get_field(%args) =~ /true/i) {
        return $SanUtil::TRUE;
    } else {
        return $SanUtil::FALSE;
    }
}

################################################################################
# Method Name: get_bay()
# Objective:   Get disk bay
# Details:     See POD documentation at the end of this file
################################################################################
sub get_bay {
    my ($self, %args) = @_;

    $args{field} = "bay";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_checksum()
# Objective:   Get disk checksum
# Details:     See POD documentation at the end of this file
################################################################################
sub get_checksum {
    my ($self, %args) = @_;

    $args{field} = "checksum";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_recontructionpercent()
# Objective:   Get disk recontructionpercentage
# Details:     See POD documentation at the end of this file
################################################################################
sub get_recontructionpercent {
    my ($self, %args) = @_;

    $args{field} = "recontructionpercent";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_home()
# Objective:   Get disk home
# Details:     See POD documentation at the end of this file
################################################################################
sub get_home {
    my ($self, %args) = @_;

    $args{field} = "home";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_homeid()
# Objective:   Get disk homeid
# Details:     See POD documentation at the end of this file
################################################################################
sub get_homeid {
    my ($self, %args) = @_;

    $args{field} = "homeid";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_model()
# Objective:   Get disk model
# Details:     See POD documentation at the end of this file
################################################################################
sub get_model {
    my ($self, %args) = @_;

    $args{field} = "model";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_name()
# Objective:   Get disk name
# Details:     See POD documentation at the end of this file
################################################################################
sub get_name {
    my ($self, %args) = @_;

    $args{field} = "disk";

    # let's get from cache for name field
    $args{use_cache} = $SanUtil::TRUE;

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_owner()
# Objective:   Get disk owner
# Details:     See POD documentation at the end of this file
################################################################################
sub get_owner {
    my ($self, %args) = @_;

    $args{field} = "owner";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_ownerid()
# Objective:   Get disk ownerid
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ownerid {
    my ($self, %args) = @_;

    $args{field} = "ownerid";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_physical_blocks()
# Objective:   Get disk physical_blocks
# Details:     See POD documentation at the end of this file
################################################################################
sub get_physical_blocks {
    my ($self, %args) = @_;

    $args{field} = "physical_blocks";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_physicalsize()
# Objective:   Get disk physicalsize
# Details:     See POD documentation at the end of this file
################################################################################
sub get_physicalsize {
    my ($self, %args) = @_;

    $args{field} = "physicalsize";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_revision()
# Objective:   Get disk revision
# Details:     See POD documentation at the end of this file
################################################################################
sub get_revision {
    my ($self, %args) = @_;

    $args{field} = "revision";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_rpm()
# Objective:   Get disk rpm
# Details:     See POD documentation at the end of this file
################################################################################
sub get_rpm {
    my ($self, %args) = @_;

    $args{field} = "rpm";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_sectorcount()
# Objective:   Get disk sectorcount
# Details:     See POD documentation at the end of this file
################################################################################
sub get_sectorcount {
    my ($self, %args) = @_;

    $args{field} = "sectorcount";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_serialnumber()
# Objective:   Get disk serialnumber
# Details:     See POD documentation at the end of this file
################################################################################
sub get_serialnumber {
    my ($self, %args) = @_;

    $args{field} = "serialnumber";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_shelf()
# Objective:   Get disk shelf
# Details:     See POD documentation at the end of this file
################################################################################
sub get_shelf {
    my ($self, %args) = @_;

    $args{field} = "shelf";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_state()
# Objective:   Get disk state
# Details:     See POD documentation at the end of this file
################################################################################
sub get_state {
    my ($self, %args) = @_;

    $args{field} = "state";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_type()
# Objective:   Get disk type
# Details:     See POD documentation at the end of this file
################################################################################
sub get_type {
    my ($self, %args) = @_;

    $args{field} = "type";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_usedsize()
# Objective:   Get disk usedsize
# Details:     See POD documentation at the end of this file
################################################################################
sub get_usedsize {
    my ($self, %args) = @_;

    $args{field} = "usedsize";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_uuid()
# Objective:   Get disk uuid
# Details:     See POD documentation at the end of this file
################################################################################
sub get_uuid {
    my ($self, %args) = @_;

    $args{field} = "uuid";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_vendor()
# Objective:   Get disk vendor
# Details:     See POD documentation at the end of this file
################################################################################
sub get_vendor {
    my ($self, %args) = @_;

    $args{field} = "vendor";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_zeroingpercent()
# Objective:   Get disk zeroingpercent
# Details:     See POD documentation at the end of this file
################################################################################
sub get_zeroingpercent {
    my ($self, %args) = @_;

    $args{field} = "zeroingpercent";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Private methods
################################################################################

################################################################################
# Subroutine Name: _discover
# Objective:       create and create object. update object data and create
#                  accesser methods of each data
# Details:         See POD documentation at the end of this file
################################################################################
sub _discover {
    my ($class, %args) = @_;

    my ($self, $objs, @disk_objs);

    # parameter initialization
    $self = {};
    bless $self, $class;

    # remove the dash '-' from input argument list which will be used to
    # create 'get' accessor methods on the fly
    %args = &SanUtil::strip_dash(%args);

    # parent object is used to bubble up error if needed
    my $parent_obj = delete $args{parent_obj};

    # use common _new method from base class (San::CommonIterface)
    # to contruct disk object
    $self = $class->SUPER::_new(-name  => $args{disk},
                                -filer => $args{filer});
    if ($self->error()) {
        $parent_obj->_set_err("failed to construct disk object", $self->errmsg());
        return (wantarray()) ? () : [];
    }

    # dump only first depth of hash if debug_level < 3
    # dump everything of debug_level => 3
    $self->sanlog(-msg => $self->dumper_get(\%args, $self->get_debug_level()),
                  -msg_level => 2);

    # use common discover method from base class to update, set, and create
    # accessor methods for the object's data
    $objs = $self->SUPER::discover(-pkg  => __PACKAGE__,
                                   -cdef => "storage_disk_show",
                                   %args);
    if ($self->error()) {
        $parent_obj->_set_err("failed to discover disk objects", $self->errmsg());
        return (wantarray()) ? () : [];
    }

    # retrieve all the known state disk objects
    foreach my $obj (@$objs) {
        push @disk_objs, $obj unless ($obj->get_state() =~ /-/);
    }

    return (wantarray()) ? @disk_objs : \@disk_objs;
} ## end sub discover

################################################################################
# Method Name: _get_param_defaults
# Objective: Gets default disk parameters 
# Details:   See POD documentation at the end of this file. 
################################################################################
sub _get_param_defaults {
    my ($self) = @_;

    # get the default parameter of disk object
    my %defaults_params = (disk => $self->get_name(-use_cache => 1));

    return %defaults_params;
}

################################################################################
# Method Name: _get_field
# Objective: Get attribute from the object cache or the controller for Disk
################################################################################
sub _get_field {
    my ($self, %args) = @_;

    my $use_cache =  $SanUtil::FALSE;

    # remove the dash '-' from input argument list
    %args = &SanUtil::strip_dash(%args);

    # verify mandatory cmd argument
    unless (defined $args{field} and length $args{field}) {
        $self->_set_err("mandatory parameter \"-field\" not specified ");
        return "";
    }

    # set use_cache flag if exists
    if (exists $args{use_cache}) {
        $use_cache = delete $args{use_cache};
    }

    # get attribute from object cache or on the controller
    if ($use_cache == $SanUtil::TRUE) {
        # the get() will handle error
        return $self->get($args{field});
    } else {
        # the get_field_from_controller() will handle error
        return $self->get_field_from_controller(cmd => "storage_disk_show",
                                                field => $args{field},
                                                $self->_get_param_defaults());
    }
}

1;
