# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provides an interface to manage iSCSI Target object
##
## @author korry@netapp.com (dl-ssan-automation)
## @status public

package San::ControllerCluster::Filer10M::Vserver::Iscsitarget;

# compiler directives
use strict;

use Params::Validate qw( SCALAR ARRAYREF HASHREF OBJECT );

# inherit base class
use base qw( San::CommonInterface );

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Method Name: get_iscsi_obj()
# Objective:   Get iscsi_obj
# Details:     See POD documentation at the end of this file
################################################################################
sub get_iscsi_obj {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("iscsi_obj");
}

################################################################################
# Method Name: get_ip()
# Objective:   Get ip
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ip {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("ip");
}

################################################################################
# Method Name: get_ipv4()
# Objective:   Get ipv4
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv4 {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("ipv4");
}

################################################################################
# Method Name: get_ipv6()
# Objective:   Get ipv6
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv6 {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("ipv6");
}

################################################################################
# Method Name: get_ipv4_netmask()
# Objective:   Get ipv4_netmask
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv4_netmask {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("ipv4_netmask");
}

################################################################################
# Method Name: get_ipv6_netmask()
# Objective:   Get ipv6_netmask
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv6_netmask {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("ipv6_netmask");
}

################################################################################
# Method Name: get_ipv4_subnet()
# Objective:   Get ipv4_subnet
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv4_subnet {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("ipv4_subnet");
}

################################################################################
# Method Name: get_ipv6_subnet()
# Objective:   Get ipv6_subnet
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv6_subnet {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("ipv6_subnet");
}

################################################################################
# Method Name: get_mac()
# Objective:   Get mac
# Details:     See POD documentation at the end of this file
################################################################################
sub get_mac {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("mac");
}

################################################################################
# Method Name: get_nodename()
# Objective:   Get nodename
# Details:     See POD documentation at the end of this file
################################################################################
sub get_nodename {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("nodename");
}

################################################################################
# Method Name: get_parent_obj()
# Objective:   Get parent object
# Details:     See POD documentation at the end of this file
################################################################################
sub get_parent_obj {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("parent_obj");
}

################################################################################
# Method Name: get_state()
# Objective:   Get state
# Details:     See POD documentation at the end of this file
################################################################################
sub get_state {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("state");
}

################################################################################
# Method Name: get_subnet()
# Objective:   Get subnet
# Details:     See POD documentation at the end of this file
################################################################################
sub get_subnet {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("subnet");
}

################################################################################
# Method Name: get_tpgn()
# Objective:   Get tpgn
# Details:     See POD documentation at the end of this file
################################################################################
sub get_tpgn {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("tpgn");
}

################################################################################
# Method Name: get_tpgt()
# Objective:   Get tpgt
# Details:     See POD documentation at the end of this file
################################################################################
sub get_tpgt {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("tpgt");
}

################################################################################
# Method Name: is_mgmt_port
# Objective:   Returns 1 if this port is the management port, 0 if not
# Details:     See POD documentation at the end of this file
################################################################################
sub is_mgmt_port {
    my ($self) = @_;

    # the get() will handle error
    return $self->get("is_mgmt_port");
}

################################################################################
# Method Name: set_ip_mode
# Objective:   Set the working ip for this target to be the ipv4|ipv6 ip
# Details:     See POD documentation at the end of this file
################################################################################
sub set_ip_mode {
    my ($self, $mode) = @_;

    $self->_clear_err();
 
    if ($mode =~ /ipv4/i) {
        $self->{ip} = $self->{ipv4};
        $self->{subnet} = $self->{ipv4_subnet};
    } elsif ($mode =~ /ipv6/i) {
        $self->{ip} = $self->{ipv6};
        $self->{subnet} = $self->{ipv6_subnet};
    } else {
        $self->_set_err("Invalid IP mode: '$mode', must be ipv4 or ipv6");
        return $SanUtil::FALSE;
    }

    # if the working ip is empty then flag this as an error
    if ($self->{ip} eq "") {
        $self->_set_err("Invalid IP mode: '$mode', this target does not have an $mode ip address"); 
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
}



################################################################################
# P R I V A T E   M E T H O D S
################################################################################

###############################################################################
# Method Name: _new
# Objective: Create new iSCSI Target object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _new {
    my ($class, %args) = @_;

    my ($self, @inits);

    # parameter initialization
    $self = {};

    # bless instance into class
    bless $self, $class;

    # remove the dash '-' from input argument list which will be used to
    # create 'get' accessor methods on the fly
    %args = &SanUtil::strip_dash(%args);

    # verify mandatory cmd argument
    unless (defined $args{filer} and length $args{filer}) {
        if (defined $args{parent_obj}) {
            $args{filer} = $args{parent_obj}->get_filer();
        } else {
            $self->_set_err("missing filer parameters", $self->errmsg());
            return $self;
        }
    }

    if (exists $args{name}) {
        $args{iscsitarget} = $args{name};
    }

    # display argument hash if debug_level is high
    $self->sanlog(-msg => $self->dumper_get(\%args),
                  -msg_level => 3);

    # use base class (San::CommonInterface) via SUPER::_new method 
    # to contruct an iscsitarget object
    $self = $class->SUPER::_new(-name  => $args{iscsitarget},
                                -filer => $args{filer});

    if ($self->error()) {
        $self->_set_err("failed to create an iscsi target object", $self->errmsg());
        return $self;
    }

    # dump only first depth of hash if debug_level < 3
    # dump everything of debug_level => 3
    $self->sanlog(-msg => $self->dumper_get(\%args, $self->get_debug_level()),
                  -msg_level => 2);


    # store/initalize iscsi target elements
    foreach my $key (keys(%args)) {
        $self->{lc($key)} = $args{$key};
        push @{$self->{fields}}, lc($key);
    }

    # initialize ipv4 attributes
    if ($self->{ipv4} ne "not_set") {
        $self->{ipv4_subnet} = &SanUtil::get_ip_subnet(
                                       -ip_address   => $self->{ipv4},
                                       -ip_version   => "ipv4",
                                       -ipv4_netmask => $self->{ipv4_netmask});
    } else {
        $self->{ipv4_subnet} = "";
    }

    # default current subnet to ipv4_subnet
    $self->{subnet} = $self->{ipv4_subnet};

    # initialize ipv6 attributes
    if ($self->{ipv6} ne "not_set") {
        $self->{ipv6_subnet} = &SanUtil::get_ip_subnet(
                                       -ip_address      => $self->{ipv6},
                                       -ip_version      => "ipv6",
                                       -ipv6_prefix_length => $self->{ipv6_netmask_length});
    } else {
        $self->{ipv6_subnet} = "";
    }

    # add subnet, ipv4_subnet, and ipv6_subnet to the array fields
    push @{$self->{fields}}, ('subnet', 'ipv4_subnet', 'ipv6_subnet');

    # now we store and create accessor methods, delete the one we don't need
    delete $args{name};
    delete $args{parent_obj};

    # get initiators
    @inits = $self->_get_initiators();
    if ($self->error()) {
        $self->_set_err("failed to get initiators", $self->errmsg());
        return $self;
    }
    $self->{inits} = \@inits;
    $self->{initcnt} = scalar(@inits);

    return $self;
} ## end sub _new

################################################################################
# Method Name: _get_param_defaults
# Objective: Gets default igroup parameters 
# Details:   See POD documentation at the end of this file. 
################################################################################
sub _get_param_defaults {
    my ($self) = @_;

    # default hash parameters to be used in igroup object
    my %defaults_params = (vserver => $self->{parent_obj}->get_name(-use_cache => 1));

    return %defaults_params;
}

################################################################################
# Method Name: _get_initiators
# Objective: Gets initiators to into iscsi target object
# Details:   See POD documentation at the end of this file. 
################################################################################
sub _get_initiators {
    my ($self) = @_;

    my (@inits, @tmp_inits);
 
    @inits = ();

    # if tp group tag is not set, return empty list
    if ($self->get_tpgt() =~ /not_set/i) {
        return (wantarray()) ? () : [];
    }

    # we need to call initiator show to get all initiators that tags to 
    # target portal group
    if (defined $self->{iscsi_obj} and ref($self->{iscsi_obj}) =~ /iscsi/i) {
        # get an list of initiators and its key/value attributes
        @tmp_inits = $self->{iscsi_obj}->get_initiators();
        if ($self->{iscsi_obj}->error()) {
            $self->_set_err("failed to get initiators from iscsi object",
                            $self->{iscsi_obj}->errmsg());
            return (wantarray()) ? () : [];
        }

        # traverse through all and get the initiator name
        foreach my $init_hash (@tmp_inits) {
            if (defined $init_hash->{initiator} and $init_hash->{initiator} =~ /^iqn\.1991\-05/) {
                push @inits, $init_hash->{initiator};
            }
        }
    } else {
        $self->sanlog(-msg => "no iscsi object to get initiators");
        return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @inits : \@inits;
}

1;
