# Copyright (c) 2009 NetApp Inc.
# All rights reserved.
#
## @summary provides 7mode compatible methods for Lun class
##
## @author korry@netapp.com, dl-ssan-automation
## @status Shared

package San::ControllerCluster::Filer10M::Vserver::Volume::LunCompat;

# compiler directives
use strict;
use lib();

# imported modules;
use Params::Validate qw( SCALAR ARRAYREF HASHREF OBJECT );

################################################################################
# Method Name: list_info()
# Objective:   perform list_info on the lun object
# Details:     See POD documentation at the end of this file
################################################################################
sub list_info {
    my ($self, %args) = @_;

    $self->_clear_err();

    # now we get the vserver let's create an igroup object
    my $output_hash = $self->get_list_info();
    if ($self->error()) {
        $self->_set_err("fail to list lun info ");
        return {};
    }

    if (exists($output_hash->{lun_path})) {
        $output_hash->{path} = $output_hash->{lun_path};
    }
    # space reservation isn't supported yet, set it to enabled
    if (($output_hash->{space_resv}) = "-") {
        $output_hash->{space} = "enabled";
    }
    if (($output_hash->{state}) eq "online") {
        $output_hash->{online} = 'true';
    } else {
        $output_hash->{online} = 'false';
    }
    if (exists($output_hash->{"serial"})) {
        $output_hash->{"serial-number"} = $output_hash->{serial};
    }
    if (exists($output_hash->{ostype})) {
        $output_hash->{"multiprotocol-type"} = $output_hash->{ostype};
    }
    # hard code value for 7mode compatible, this is not currently supported
    $output_hash->{"read-only"} = "false";
    $output_hash->{"mapped"} = "false";
    $output_hash->{"share-state"} = "none";
    $output_hash->{"full-state"} = "(r/w, " . $output_hash->{state} . ")";

    return $output_hash;
} ## end sub list_info

################################################################################
# Method Name: get_filername()
# Objective:   get vserver name in Cmode. This method map one-to-one to 7-mode
# Details:     See POD documentation at the end of this file
################################################################################
sub get_filername {
    my ($self) = @_;

    return ($self->get_vserver());
}
1;

#############################################################################
#                             POD Documentation
#############################################################################
## @pod here

=pod

=head1 Package Name

LunCompat.pm

=head1 Author

Ann Harrold (dl-ssan-automation)

=head1 Description

LunCompat provides 7mode compatible and helper methods for Lun. This package maintains separately from Lun for maintainability purpose while preventing convoluting Lun object model.

=head1 See Also

LunCompat package is used as part of the base class by B<San::ControllerCluster::Filer10M::Vserver::Volume::Lun>. See these modules for further interface detail.

B<San::ControllerCluster::Filer10M.pm, San::CommonInterface.pm>

=head1 B<Public Methods>

=head2 B<list_info()>

Get detailed lun information.

=over 4

=item Inputs

None

=item Outputs

=item Examples

    my %info_hash = $lun_obj->list_info();
    if ( $lun_obj->error() ) {
        $Test->nlog( $lun_obj->errmsg() );
        return $TCD::FAIL;
    }


=back

=head2 B<get_filername()>

Get vserver name (7mode compatibility)

=over 4

=item Inputs

None

=item Outputs

Vserver name this lun resided in

=item Examples

    # display vserver in Cmode
    $Test->nlog("-- Filer_name =" . $lun_obj->get_filername());

=back
