#$Id: //depot/prod/test/Rsierranevada/lib/San/ControllerCluster/Filer10M/Vserver/Volume/Snapshot.pm#1 $

# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provides an interface to a Snapshot object
##
## @author tholland@netapp.com, dl-ssan-test
## @status Shared


package San::ControllerCluster::Filer10M::Vserver::Volume::Snapshot;

# compiler directives
use strict;

# imported modules
use San::ControllerCluster::Filer10M::Vserver::Volume::Snapshot;

# inherit base class
use base qw( San::CommonInterface);

###############################################################################
# Method Name: new
# Objective: Create new Snapshot object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class, %args) = @_;

    my ($self);

    # parameter initialization
    $self = {};

    # bless instance into class
    bless $self, $class;

    # remove the dash '-' from input argument list which will be used to
    # create 'get' accessor methods on the fly
    %args = &SanUtil::strip_dash(%args);

    if (exists $args{name}) {
        $args{snapshot} = delete $args{name};
    }

    # verify mandatory cmd argument
    unless (defined $args{snapshot} and length $args{snapshot}) {
        $self->_set_err("mandatory parameter \"-snapshot\" not specified ");
        return $self;
    }
    unless (defined $args{filer} and length $args{filer}) {
        $self->_set_err("mandatory parameter \"-filer\" not specified ");
        return $self;
    }

    $self->sanlog(-msg => $self->dumper_get(\%args, $self->get_debug_level()),
                  -msg_level => 2);

    # use base class (San::CommonInterface) via SUPER::_new method
    # to contruct snapshot object
    $self = $class->SUPER::_new(-name  => $args{snapshot},
                                -filer => $args{filer},
                                %args);
    if ($self->error()) {
        $self->_set_err("failed to construct snapshot object", $self->errmsg());
        return $self;
    }

    # dump only first depth of hash if debug_level < 3
    # dump everything of debug_level => 3
    $self->sanlog(-msg => $self->dumper_get(\%args, $self->get_debug_level()),
                  -msg_level => 2);

    # store Snapshot elements on a list
    foreach my $key (keys(%args)) {
        $self->{lc($key)} = $args{$key};
        push @{$self->{fields}}, lc($key);
    }

    # we can now remove params from the args list that we dont need
    delete $args{debug_level};

    # if the obj has been initialized, just refresh to update info
    # otherwise initialize it by create and update its object
    unless ($args{initialized}) {
        # snapshot has not been initialized/created, so we need to create it
        $self->_create_snapshot(%args);
        if ($self->error()) {
            # set error and bubble up
            $self->_set_err("failed to _create_snapshot() ", $self->errmsg());
            return $self;
        }
    }

    return $self;
} ## end sub new

###############################################################################
# Method Name: delete
# Objective: delete snapshot object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub delete {
    my ($self, %args) = @_;

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    # initialize cdef parameters
    %args = ($self->_get_param_defaults(%args),
             cmd => "volume_snapshot_delete");

    $self->SUPER::invoke_cmd_obj(%args);
        if ($self->error()) {
            $self->_set_err("failed to invoke_cmd_obj " .
                            $self->dumper_get(\%args, $self->get_debug_level()),
                            $self->errmsg());
            return $SanUtil::FALSE;
        }

    # set name to "_DELETE_" to indicate snapshot has been deleted
    $self->{name} = "_DELETE_";
    $self->{snapshot} = "_DELETE_";

    return $SanUtil::TRUE;
} ## end sub delete

###############################################################################
# Method Name: destroy
# Objective: destroy snapshot object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub destroy {
    my ($self, %args) = @_;

    return $self->delete(%args);
}

###############################################################################
# Method Name: modify
# Objective: modify snapshot attribute
# Details:   See POD documentation at the end of this file.
###############################################################################
sub modify {
    my ($self, %args) = @_;

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    # initialize cdef parameters
    %args = ($self->_get_param_defaults(%args),
             cmd => "volume_snapshot_modify");

    $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
} ## end sub modify

###############################################################################
# Method Name: refresh()
# Objective: Refresh object's data
# Details:   See POD documentation at the end of this file.
###############################################################################
sub refresh {
    my ($self, %args) = @_;

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    # initialize cdef parameters
    %args = ($self->_get_param_defaults(%args),
             cmd => "volume_snapshot_show");

    $self->sanlog(-msg => $self->dumper_get(\%args),
                  -msg_level => 3);

    # use base class (San::CommonInterface) refresh via SUPER::refresh
    # to update snapshot object data
    $self->SUPER::refresh(%args);
    if ($self->error()) {
        # error is already set, bubble up
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
} ## end sub refresh

###############################################################################
# Method Name: get_list_info()
# Objective: Get snapshot information
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_list_info {
    my ($self, %args) = @_;

    my (@output);

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    %args = (%args,
             cmd => "volume_snapshot_show",
             $self->_get_param_defaults());

    (@output) = $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @output : \@output;
} ## end sub get_list_info

###############################################################################
# Method Name: rename()
# Objective: Move a snapshot (basicaly this is a rename)
# Details:   See POD documentation at the end of this file.
###############################################################################
sub rename {
    my ($self, %args) = @_;
    $self->_clear_err();

    # remove the optional dash '-' prefix from the input args
    %args = &SanUtil::strip_dash(%args);

    # Check for the required path argument
    unless (defined $args{'new-name'} and length $args{'new-name'}) {
        $self->_set_err("Mandatory parameter \"new-name\" not specified ");
        return $SanUtil::FALSE;
    }
    my $new_name = $args{'new-name'};

    # Initialize cdef parameters
    %args = (%args,
             cmd          => "volume_snapshot_rename",
             "new-name"   => "$new_name",
             $self->_get_param_defaults() );

    $self->sanlog(-msg => "snapshot::rename() - rename a snapshot from $self->get_name() to $new_name",
                  -msg_level => 1);

    $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return $SanUtil::FALSE;
    }

    # Update our changed attributes after the rename and refresh
    $self->{snapshot}  = $new_name;
    $self->{name} = $new_name;
    $self->refresh();
    if ($self->error()) {
        $self->_set_err("Failed to refresh snapshot object:", $self->errmsg());
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
} ## end sub rename()

###############################################################################
# Method Name: restore()
# Objective: Restore a volume from a snapshot
# Details:   See POD documentation at the end of this file.
###############################################################################
sub restore {
    my ($self, %args) = @_;
    $self->_clear_err();

    # remove the optional dash '-' prefix from the input args
    %args = &SanUtil::strip_dash(%args);


    # Initialize cdef parameters
    %args = (%args,
             cmd => "volume_snapshot_restore",
             $self->_get_param_defaults() );

    $self->sanlog(-msg => "snapshot::restore() - Snapshot restore info:",
                  -msg_level => 1);

    $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
} ## end sub restore()

###############################################################################
# Method Name: restore_file()
# Objective: Restore a file from a snapshot
# Details:   See POD documentation at the end of this file.
###############################################################################
sub restore_file {
    my ($self, %args) = @_;
    $self->_clear_err();

    # remove the optional dash '-' prefix from the input args
    %args = &SanUtil::strip_dash(%args);

    # Check for the required snapshot_name and path arguments
    unless (defined $args{path} and length $args{path}) {
        $self->_set_err("Mandatory parameter \"path\" not specified ");
        return $SanUtil::FALSE;
    }
    my $path = $args{path};

    # Initialize cdef parameters
    %args = (%args,
             cmd             => "volume_snapshot_restore_file",
             'path'          => "$path",
             $self->_get_param_defaults() );

    $self->sanlog(-msg => "snapshot::restore_file() - Snapshot file restore info:",
                  -msg_level => 1);

    $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
} ## end sub restore_file()

###############################################################################
# Method Name: restore_file_info()
# Objective:   Info about snapshot file restores on a vserver.
# Details:     See POD documentation at the end of this file.
###############################################################################
sub restore_file_info {
    my ($self, %args) = @_;

    my ($output);

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    %args = (%args,
             cmd      => "volume_snapshot_restore_file_info",
             raw_data => "1",
             $self->_get_param_defaults());

    delete $args{volume} if (exists $args{volume});
    delete $args{snapshot} if (exists $args{snapshot});

    ($output) = $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return "";
    }

    return $output;
} ## end sub restore_file_info

###############################################################################
# Method Name: partial_restore_file()
# Objective:   Restore part of a file from a snapshot
# Details:     See POD documentation at the end of this file.
###############################################################################
sub partial_restore_file {
    my ($self, %args) = @_;
    $self->_clear_err();

    # remove the optional dash '-' prefix from the input args
    %args = &SanUtil::strip_dash(%args);

    # Check for the required arguments
    unless (defined $args{path} and length $args{path}) {
        $self->_set_err("Mandatory parameter \"path\" not specified ");
        return $SanUtil::FALSE;
    }
    unless (defined $args{'start-byte'} and length $args{'start-byte'}) {
        $self->_set_err("Mandatory parameter \"start-byte\" not specified ");
        return $SanUtil::FALSE;
    }
    unless (defined $args{'byte-count'} and length $args{'byte-count'}) {
        $self->_set_err("Mandatory parameter \"byte-count\" not specified ");
        return $SanUtil::FALSE;
    }
    my $snapshot_name = $args{'snapshot'};
    my $path = $args{path};
    my $start_byte = $args{'start-byte'};
    my $byte_count = $args{'byte-count'};

    # Initialize cdef parameters
    %args = (%args,
             cmd             => "volume_snapshot_partial_restore_file",
             'snapshot'      => "$snapshot_name",
             'path'          => "$path",
             'start-byte'    => "$start_byte",
             'byte-count'    => "$byte_count",
             $self->_get_param_defaults() );

    $self->sanlog(-msg => "snapshot::partial_restore_file() - partial_restore_file from snapshot $self->get_name() and path $path",
                  -msg_level => 1);

    $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
} ## end sub partial_restore_file()

###############################################################################
# Method Name: partial_restore_file_info()
# Objective:   Info about partial snapshot file restores on a Vserver.
# Details:     See POD documentation at the end of this file.
###############################################################################
sub partial_restore_file_info {
    my ($self, %args) = @_;

    my ($output);

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    %args = (%args,
             cmd => "volume_snapshot_partial_restore_file_list_info",
             raw_data => "1",
             $self->_get_param_defaults());
    delete $args{volume} if (exists $args{volume});
    delete $args{snapshot} if (exists $args{snapshot});

    ($output) = $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return "";
    }

    return $output;
} ## end sub partial_restore_file_info

###############################################################################
# Method Name: get_autodelete_list_info()
# Objective: Get snapshot autodelete information
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_autodelete_list_info {
    my ($self, %args) = @_;

    my (@output);

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    %args = (%args,
             cmd => "volume_snapshot_autodelete_show",
             raw_data => "1",
             $self->_get_param_defaults());
    delete $args{snapshot} if (exists $args{snapshot});

    (@output) = $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @output : \@output;
} ## get_autodelete_list_info

###############################################################################
# Method Name: autodelete_modify
# Objective: modify snapshot autodelete attributes
# Details:   See POD documentation at the end of this file.
###############################################################################
sub autodelete_modify {
    my ($self, %args) = @_;

    $self->_clear_err();

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    # initialize cdef parameters
    %args = ($self->_get_param_defaults(%args),
             cmd => "volume_snapshot_autodelete_modify");
    delete $args{snapshot} if (exists $args{snapshot});

    $self->SUPER::invoke_cmd_obj(%args);
    if ($self->error()) {
        $self->_set_err("failed to invoke_cmd_obj " .
                        $self->dumper_get(\%args, $self->get_debug_level()),
                        $self->errmsg());
        return $SanUtil::FALSE;
    }

    return $SanUtil::TRUE;
} ## end sub autodelete_modify

################################################################################
# A C C E S S O R   M E T H O D S
################################################################################

################################################################################
# Method Name: get_parent_obj()
# Objective:   Get snapshot parent object
# Details:     See POD documentation at the end of this file
################################################################################
sub get_parent_obj {
    my ($self, %args) = @_;

    $args{field} = "parent_obj";

    # let's get from cache for name field
    $args{use_cache} = $SanUtil::TRUE;

    # the _get_field() will handle error
    return $self->_get_field(%args);
} ## end sub get_parent_obj

################################################################################
# Method Name: get_name()
# Objective:   Get snapshot name
# Details:     See POD documentation at the end of this file
################################################################################
sub get_name {
    my ($self, %args) = @_;

    $args{field} = "snapshot";

    # let's get from cache for name field
    $args{use_cache} = $SanUtil::TRUE;

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_volume()
# Objective:   Get volume this snapshot created on
# Details:     See POD documentation at the end of this file
################################################################################
sub get_volume {
    my ($self, %args) = @_;

    $args{field} = "volume";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_vserver()
# Objective:   Get snapshot vserver
# Details:     See POD documentation at the end of this file
################################################################################
sub get_vserver {
    my ($self, %args) = @_;

    $args{field} = "vserver";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_comment()
# Objective:   Get snapshot comment
# Details:     See POD documentation at the end of this file
################################################################################
sub get_comment {
    my ($self, %args) = @_;

    $args{field} = "comment";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_creation_time()
# Objective:   Get snapshot creation time
# Details:     See POD documentation at the end of this file
################################################################################
sub get_creation_time {
    my ($self, %args) = @_;

    $args{field} = "create-time";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_busy()
# Objective:   Get whether snapshot is busy
# Details:     See POD documentation at the end of this file
################################################################################
sub get_busy {
    my ($self, %args) = @_;

    $args{field} = "busy";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_owners()
# Objective:   Get snapshot owners
# Details:     See POD documentation at the end of this file
################################################################################
sub get_owners {
    my ($self, %args) = @_;

    $args{field} = "owners";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_size()
# Objective:   Get snapshot size
# Details:     See POD documentation at the end of this file
################################################################################
sub get_size {
    my ($self, %args) = @_;

    $args{field} = "size";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_blocks()
# Objective:   Get snapshot blocks
# Details:     See POD documentation at the end of this file
################################################################################
sub get_blocks {
    my ($self, %args) = @_;

    $args{field} = "blocks";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_usedblocks()
# Objective:   Get snapshot usedblocks
# Details:     See POD documentation at the end of this file
################################################################################
sub get_usedblocks {
    my ($self, %args) = @_;

    $args{field} = "used";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# Method Name: get_is_7_mode()
# Objective:   Get whether snapshot is 7 mode
# Details:     See POD documentation at the end of this file
################################################################################
sub get_is_7_mode {
    my ($self, %args) = @_;

    $args{field} = "is-7-mode";

    # the _get_field() will handle error
    return $self->_get_field(%args);
}

################################################################################
# P R I V A T E   M E T H O D S
################################################################################

################################################################################
# Method Name: _get_param_defaults
# Objective: Gets default snapshot parameters
# Details:   See POD documentation at the end of this file.
################################################################################
sub _get_param_defaults {
    my ($self, %args) = @_;

    my %default_params = (%args, vserver => $self->get_vserver(-use_cache => 1));

    # if path is not provided, we need volume and snapshot params
    unless (exists $args{'snapshot-path'}) {
        %default_params = (%default_params,
                           snapshot => $self->get_name(-use_cache => 1),
                           volume   => $self->get_volume(-use_cache => 1));
    }

    return %default_params;
}

################################################################################
# Method Name: _get_field
# Objective: Get attribute from the object cache or the controller for snapshot
################################################################################
sub _get_field {
    my ($self, %args) = @_;

    my $use_cache =  $SanUtil::FALSE;

    # remove the dash '-' from input argument list
    %args = &SanUtil::strip_dash(%args);

    my $command = (defined $args{command}) ? $args{command} : "volume_snapshot_show";

    # verify mandatory cmd argument
    unless (defined $args{field} and length $args{field}) {
        $self->_set_err("mandatory parameter \"-field\" not specified ");
        return "";
    }

    # set use_cache flag if exists
    if (exists $args{use_cache}) {
        $use_cache = delete $args{use_cache};
    }

    # get attribute from object cache or on the controller
    if ($use_cache == $SanUtil::TRUE) {
        # the get() will handle error
        return $self->get($args{field});
    } else {
        # the get_field_from_controller() will handle error
        return $self->get_field_from_controller(cmd => $command,
                                                field => $args{field},
                                                $self->_get_param_defaults());
    }
}

################################################################################
# Method Name: _create_snapshot
# Objective: create snapshot
################################################################################
sub _create_snapshot {
    my ($self, %args) = @_;

    # append cmd for volume create to argument list
    %args = (%args, cmd => "volume_snapshot_create");
    delete $args{parent_obj} if (exists $args{parent_obj});

    $self->SUPER::invoke_cmd_obj(%args);
        if ($self->error()) {
            $self->_set_err("failed to invoke_cmd_obj " .
                            $self->dumper_get(\%args, $self->get_debug_level()),
                            $self->errmsg());
            return $self;
        }

    # call snapshot refresh to populate object if enable_cache is set
        if ($self->{enable_cache}) {
            $self->refresh();
            if ($self->error()) {
                # set error and bubble up
                $self->_set_err("failed to refresh " . $args{snapshot}, $self->errmsg());
                return $self;
            }
        }

    # set intitialized flag
    $self->{initialized} = $SanUtil::TRUE;

    return $SanUtil::TRUE;
}

1;

#############################################################################
#                             POD Documentation
#############################################################################
## @pod here

=pod

=head1 Package Name

Qtree.pm

=head1 Author

Trey Holland (dl-ssan-automation)

=head1 Description

Provides an interface to create and manage C-mode snapshots

=head1 See Also

Snapshot.pm inherits methods from B<San::CommonInterface>. See these
methods for further interface details.

=head1 Attributes

=head2 B<filer>

string = controller name

=head2 B<name>

string = name of snapshot object to be created

=head2 B<debug_level>

interger = 0 - 100 (default is 0)

=head2 B<create_time>

string = snapshot creation time

=head2 B<busy>

string = Snapshot busy (true|false)

=head2 B<owners>

string = list of owners

=head2 B<volume>

string = the name of the volume the snapshot is in

=head2 B<snapshot>

string = the name of the snapshot

=head2 B<size>

string = snapshot size

=head2 B<blocks>

integer = percentage of total blocks

=head2 B<usedblocks>

string = percentage of used blocks

=head2 B<comment>

integer = snapshot comment

=head2 B<is_7_mode>

integer = 7-mode snapshot (true|false)

=head1 B<Public Methods>

=head2 B<new(%args)>

Create a snapshot object

=over 4

=item Inputs

=item * I<-filer> => <controller name>

Controller name

=item * I<-snapshot> => <name of snapshot object>

Snapshot object name to be created

=item * I<-debug_level> => <level of debug> (optional)

Debug level from 0 - 100. With higher level, it will log more debug messages

=item * I<-volume> => <volume snapshot is on>

Volume snapshot is created on

=item * I<-comment> => <comment> (optional)

Snapshot Comment

=item * I<-foreground> => <true|false> (optional)

Foreground Process (default: true)

=item Outputs

Returns a snapshot object

=item Examples

 # ex. 1: Create snapshot directly, make sure you have a vserver and a volume

 use San::ControllerCluster::Filer10M::Vserver::Volume::Snapshot;

 my $vol = "vs0_vol";
 my $vserver = "vs0";
 my $snapshot = "snapshot_0";

 # Create filer object
 $filer_obj = new SanFiler();
 if ($filer->error()) {
    logcomment("Failed to create filer object" . $filer_obj->errmsg());
    return $TCD::FAIL;
 }

 # Get the vol object. Assuming the vol exists on that node.
 $vol_obj = $filer_obj->get_vol(volume  => $vol,
                                vserver => $vserver);
 if ($vol_obj->error()) {
    logcomment("Failed to get vol object" . $vol_obj->errmsg());
    return $TCD::FAIL;
 }

 # Create the snapshot
 my $snapshot_obj = new snapshot(-filer => $filer,
                           -name => $snapshot,
                           -vserver => $vserver,
                           -volume => $vol,
                           -debug_level => 0);
 if ($snapshot_obj->error()) {
    logcomment("Failed to create snapshot object" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

 # ex. 2: Create snapshot from a volume object
 my $snapshot_obj = $volume_obj->create_snapshot(...);
 if ($snapshot_obj->error()) {
    logcomment("Failed to create snapshot object" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<delete(%args)>

Deletes a snapshot object

=over 4

=item Inputs

=item * I<-foreground> => <true|false> (optional)

=item Outputs

Returns TRUE if the snapshot was deleted successfully. Returns FALSE and sets the
error flag if something failed.

=item Examples

 # Delete a snapshot
 my $snapshot_obj->delete();
 if ($snapshot_obj->error()) {
    logcomment("Failed to delete snapshot object" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<modify()>

Modifies snapshot attributes.

=over 4

=item Inputs

=item * I<-comment> => <comment>

Modifies the comment of the snapshot.

=item Outputs

Returns TRUE if the snapshot was modified successfully and FALSE otherwise. This
method will set an error flag if the command fails.

=item Examples

 # Modify state
 $snapshot_obj->modify(-comment => 'comment');
 if ($snapshot_obj->error()) {
    logcomment("Failed to modify snapshot object" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_list_info()>

Get snapshot information.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns a reference to a hash containing the output of "snapshot show" command. If
it fails, it will return a null and set the error flag.

=item Examples

 $snapshot_info = $snapshot_obj->get_list_info();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get snapshot information" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<rename()>

Rename a snapshot

=over 4

=item Inputs

=item * <-new-name> => <new-name>.

The new name of the snapshot.

=item Outputs

Returns TRUE if the snapshot was renamed successfully and FALSE otherwise. This
method will set an error flag if the command fails.

=item Examples

 # Rename snapshot
 $snapshot_obj->rename(new-name=> $new_snapshot_name);
 if ($snapshot_obj->error()) {
    logcomment("Failed to rename snapshot " . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<restore()>

Restore a volume from a snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns TRUE if the volume was restored successfully and FALSE otherwise. This
method will set an error flag if the command fails.

=item Examples

 # Restore volume from snapshot
 $snapshot_obj->restore();
 if ($snapshot_obj->error()) {
    logcomment("Failed to restore volume " . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<restore_file()>

Restore a file from a snapshot.

=over 4

=item Inputs

=item * <-path> => <path>.

Filepath of the file to be restored.

=item Outputs

Returns TRUE if the file was restored successfully and FALSE otherwise. This
method will set an error flag if the command fails.

=item Examples

 # Restore file from snapshot
 $snapshot_obj->restore_file(path=> $file_path);
 if ($snapshot_obj->error()) {
    logcomment("Failed to restore file " . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<restore_file_info()>

Get file restoration info.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns info string.

=item Examples

 # Get Restore File Info
 my $output = $snapshot_obj->restore_file_info();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get restore file info" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<partial_restore_file()>

Partially restore a file from a snapshot.

=over 4

=item Inputs

=item * <-path> => <path>.

Filepath of the file to be restored.

=item * <-byte-count> => <byte-count>.

Number of bytes to restore.

=item * <-start-byte> => <start-byte>.

Byte to being the restore process at.

=item Outputs

Returns TRUE if the file was restored successfully and FALSE otherwise. This
method will set an error flag if the command fails.

=item Examples

 # Partially restore file from snapshot
 $snapshot_obj->partial_restore_file(path         => $file_path,
                                     'start-byte' => "4096",
                                     'byte-count' => "4096",);
 if ($snapshot_obj->error()) {
    logcomment("Failed to restore file " . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<partial_restore_file_info()>

Get partial file restoration info.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns info string.

=item Examples

 # Get Partial Restore File Info
 my $output = $snapshot_obj->partial_restore_file_info();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get partial restore file info" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_autodelete_list_info()>

Get snapshot autodelete information.

=over 4

=item Inputs

=item * I<-enabled> => true|false (optional)

Enabled

=item * I<-commitment> => try|disrupt|distroy (optional)

Commitment

=item * I<-defer-delete> => scheduled|user_created|prefix|none (optional)

Defer Delete

=item * I<-delete-order> => newest_first|oldest_first (optional)

Delete Order

=item * I<-defer_delete_prefix> => <text (size 0..15)> (optional)

Defer Delete Prefix

=item * I<-target-free-space> => <percent> (optional)

Target Free Space

=item * I<-trigger> => volume|snap_reserve|space_reserve (optional)

Trigger

=item * I<-destroy-list> => <text> (optional)

Destroy List

=item Outputs

Returns a reference to a hash containing the output of "snapshot autodelete show" command. If
it fails, it will return a null and set the error flag.

=item Examples

 $snapshot_autodelete_info = $snapshot_obj->get_autodelete_list_info();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get snapshot autodelete information" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<autodelete_modify()>

Modifies snapshot autodelete attributes.

=over 4

=item Inputs

=item * I<-enabled> => true|false (optional)

Enabled

=item * I<-commitment> => try|disrupt|distroy (optional)

Commitment

=item * I<-defer-delete> => scheduled|user_created|prefix|none (optional)

Defer Delete

=item * I<-delete-order> => newest_first|oldest_first (optional)

Delete Order

=item * I<-defer_delete_prefix> => <text (size 0..15)> (optional)

Defer Delete Prefix

=item * I<-target-free-space> => <percent> (optional)

Target Free Space

=item * I<-trigger> => volume|snap_reserve|space_reserve (optional)

Trigger

=item * I<-destroy-list> => <text> (optional)

Destroy List

=item Outputs

Returns TRUE if the snapshot autodelete was modified successfully and FALSE otherwise. This
method will set an error flag if the command fails.

=item Examples

 # Modify state
 $snapshot_obj->autodelete_modify(-enabled => 'true');
 if ($snapshot_obj->error()) {
    logcomment("Failed toenable snapshot autodelete" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_name()>

Gets the name of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs
=item Outputs

Returns the name of this snapshot.

=item Examples

 # Get snapshot name
 my $snapshot = $snapshot_obj->get_name();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get name" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_vserver()>

Gets the vserver owning this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the vserver attribute

=item Examples

 # Get vserver
 my $vserver = $snapshot_obj->get_vserver();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get vserver" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_volume()>

Gets the volume of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the volume name

=item Examples

 # Get volume
 my $vole_name = $snapshot_obj->get_volume();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get volume" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_comment()>

Gets the comment of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the comment attribute

=item Examples

 # Get comment
 my $comment = $snapshot_obj->get_comment();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get comment" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_creation_time()>

Gets the creation time of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the create-time attribute

=item Examples

 # Get creation time
 my $creation_time = $snapshot_obj->get_creation_time();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get creation time" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_busy()>

Gets the busy attribute of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the busy attribute

=item Examples

 # Get busy
 my $busy = $snapshot_obj->get_busy();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get busy" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_owners()>

Gets the owners attribute of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the owners attribute

=item Examples

 # Get owners
 my $owners = $snapshot_obj->get_owners();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get owners" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_size()>

Gets the size attribute of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the size attribute

=item Examples

 # Get size
 my $size = $snapshot_obj->get_size();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get size" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_blocks()>

Gets the blocks attribute of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the blocks attribute

=item Examples

 # Get blocks
 my $blocks = $snapshot_obj->get_blocks();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get blocks" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_usedblocks()>

Gets the usedblocks attribute of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the usedblocks attribute

=item Examples

 # Get usedblocks
 my $usedblocks = $snapshot_obj->get_usedblocks();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get usedblocks" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

=head2 B<get_is-7-mode()>

Gets the is-7-mode attribute of this snapshot.

=over 4

=item Inputs

=item * None.

=item Outputs

Returns the is-7-mode attribute

=item Examples

 # Get is-7-mode
 my $7_mode = $snapshot_obj->get_is_7_mode();
 if ($snapshot_obj->error()) {
    logcomment("Failed to get is_7_mode" . $snapshot_obj->errmsg());
    return $TCD::FAIL;
 }

=back

