# $Id: /u/williamd/p4/test/lib/San/FIT/Misfit/Core/NotProcManager.pm $

#
# Copyright (c) 2011-2012 NetApp, Inc., All Rights Reserved.
#   Any use, modification, or distribution is prohibited
#   without prior written consent from NetApp, Inc.
#
# IMPORTANT:
#    This was knowingly and directly forked from NATE::ProcManager:
#        //depot/tools/nate/main/lib/NATE/ProcManager.pm#32
#    The intention is to push these changes BACK to NATE::ProcManager.
#    We do not want to keep this file, it is only temporary.

package San::FIT::Misfit::Core::NotProcManager;
#---------------------------------------------------------------------------

=head1 NAME

NATE::ProcManager - NATE Process Controller Module

=head1 SYNOPSIS

 use NATE::ProcManager;

=head1 DESCRIPTION

A C<ProcManager> object controls the execution of bunch of tests and codestrings.


IMPORTANT:
    This was knowingly and directly forked from NATE::ProcManager:
        //depot/tools/nate/main/lib/NATE/ProcManager.pm#32
    The intention is to push these changes BACK to NATE::ProcManager.
    We do not want to keep this file, it is only temporary.


=head1 METHODS

=cut

#---------------------------------------------------------------------------

use strict;
use warnings;
use Scalar::Util qw(weaken);
use autouse 'List::Util'     => qw(shuffle);
use autouse 'File::Path'     => qw(mkpath);
use autouse 'File::Basename' => qw(dirname);
use Params::Validate qw(validate BOOLEAN ARRAYREF UNDEF SCALAR HASHREF CODEREF);

use NATE::Log       qw(log_global);
use NATE::Events    qw(eventloop_drain timer_add timer_remove);
use NATE::Util      qw(guess_scriptname scriptspec_type redirect_stdout_to_file);
use NATE::STAF      qw(staf_global);
use NATE::ParamSet    ();
use NATE::Process     ();
use NATE::Child::Fork ();
use NATE::Database::Test ();

our $Unique_PMId = 0;

#---------------------------------------------------------------------------

=head2 new

This is the constructor method of NATE::ProcManager class that creates and
 manages multiple NATE::Process instances based on 'proc_info' specification.

Given process list is converted into a sequence of NATE:Process objects.
Key attributes of sub-units like background, params, runid are set by
ProcManager in this method.

=over

=item Synopsis

my $pm = NATE::ProcManager->new(
            proc_info => [
                {codespec => "path/to/test.thpl",
                params => {"KEY" => "PARAM1"}},
            ],
        );

=item Arguments

=over

=item C<< proc_info => \@test_hash_refs >>

(Required) This argument accepts a ARRAYREF containing NATE::Process objects
and process specifications - Refer L<C<< NATE::Process->new() >>|NATE::Process/new>
for valid arguments.

=item C<< logdir => $logdir_name >>

(Optional) This argument decides the logdir under which logfile of sub-units
will be placed.

Default: $LOGDIR

=item C<< runid => $runid >>

(Optional) This argument decides the runid with which this procManager will be
 referred to in logs. By default this is same as PM id.

Default: "PM*"

=item C<< client => $hostspec >>

(Optional) Host on which all sub-units will be executed. This method accepts Hostspec.

Default: 'localhost'

=item C<< runtime => $runtime >>

(Optional) This argument decides minimum time period all the tests in the
block are to be run. If all sub-tests complete before specified duration,
another iteration is started. Value to this argument must be in seconds.

Default: 0

=item C<< iterations => $iterations >>

(Optional) Number of iterations each of sub-units must go through.

Default: 1

=item C<< independent => $boolean >>

(Optional) This argument decides whether sub-units are independent. By default
test will be killed if parent exits.

Default: 0

=item C<< background => $boolean >>

(Optional) This argument decides whether sub-units will run in background (1)
or foreground (0);

Default: 1

=item C<< randomize => $boolean >>

(Optional) This argument decides whether sub-units will run in the same order (0)
as creation or in random order (1);

Default: 0

=back

=item Return values

This method returns a NATE::ProcManager instance.

=item Examples

    use NATE::ProcManager;
    use NATE::Process;
    my $proc = NATE::Process->new(
                    codespec => "path/to/test.thpl",
                    params => {"KEY" => "TESTNAME"})
                );

    my $pm = NATE::ProcManager->new(
                proc_info => [
                    {codespec => "path/to/test.thpl",
                    params => {"KEY" => "PARAM1"}},   #test1
                    {codespec => "validate/stests/verify_param_value.thpl",
                    params => {"KEY" => "PARAM2"}},   #test2
                    $proc,                            #test3
                ],
            );
    $pm->start;

=back

=cut

#---------------------------------------------------------------------------

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;

    my %opts = validate(@_, {
        proc_info       => { type => ARRAYREF},
        logdir          => { type => SCALAR|UNDEF, optional => 1},
        runid           => { type => SCALAR|UNDEF, optional => 1},
        preserve_runid  => { type => SCALAR|UNDEF, optional => 1},
        client          => { type => SCALAR|UNDEF, optional => 1},
        timers          => { type => HASHREF|UNDEF,optional => 1},
        iterations      => { type => SCALAR,       default  => 1},
        runtime         => { type => SCALAR,       optional => 1},
        independent     => { type => BOOLEAN,      default  => 0},
        ignore_results  => { type => BOOLEAN,      optional => 1},
        background      => { type => BOOLEAN,      default  => 1},
        randomize       => { type => BOOLEAN,      optional => 1},
        onexit          => { type => CODEREF,      optional => 1},
    });

    my $self = {};
    bless($self,$class);

    # Increment Unique_PMId irrespective of 'runid' user-defined or not
    $self->{runid}  = sprintf("PM%02d",$Unique_PMId++);
    $self->{runid}  = $opts{runid} if($opts{runid});
    $self->{logdir} = $opts{logdir} if($opts{logdir});
    $self->{last_child_id} = 0;
    $self->{last_fork_id} = 0;

    $self->{iterations}  = $opts{iterations};
    $self->{runtime}     = $opts{runtime} if($opts{runtime});
    $self->{timers}      = $opts{timers} if($opts{timers});
    $self->{independent} = $opts{independent};
    $self->{background}  = $opts{background};
    $self->{client}      = $opts{client} if($opts{client});
    $self->{onexit}      = $opts{onexit} if($opts{onexit});
    $self->{randomize}   = delete $opts{randomize} if($opts{randomize});
    $self->{preserve_runid} = delete $opts{preserve_runid} if($opts{preserve_runid});
    $self->{ignore_results} = delete $opts{ignore_results} if($opts{ignore_results});

    # Create a hash for the forked processes
    my %forks = ();
    tie(%forks, "Tie::IxHash");
    $self->{forks} = \%forks;

    # Create an array of sub-Process and sub-Stest::Block.
    my %children = ();
    tie(%children, "Tie::IxHash");
    $self->{children} = \%children;
    my $i = 0;
    foreach my $proc_info (@{$opts{proc_info}}) {
        $i++;
        if((ref $proc_info eq 'HASH') ) {
            my $child_id = sprintf("%02d",$i);
            my $proc     = $self->_create_child( proc_info => $proc_info, child_id => $child_id );
            $children{$i} = $proc;
        } elsif ( $proc_info->isa('NATE::Process') ) {
            # All sub-units of ProcManager will inherit 'background' and 'independent' properties
            $proc_info->{background} = $opts{background};
            $proc_info->{independent} = $opts{independent};
            $children{$i} = $proc_info;
        } elsif ( $proc_info->isa('San::FIT::Misfit::Core::NotProcManager') ) {
            $children{$i} = $proc_info;
        }
        else {
            NATE::Result::Script->new("Invalid proc_info specification!: Unknown format")->log;
        }
        $self->{last_child_id} = $i;
        $self->{preserve_runid} ? $children{$i}->parent() :  $children{$i}->parent($self->{runid});
    }

    return $self;
}

#---------------------------------------------------------------------------

=head2 add

This is the constructor method of NATE::ProcManager class that creates and
 manages multiple NATE::Process instances based on 'proc_info' specification.

Given process list is converted into a sequence of NATE:Process objects.
Key attributes of sub-units like background, params, runid are set by
ProcManager in this method.

If adding after starting, start must be issued again.

=over

=item Synopsis

$pm->add(
    proc_info => [
        {codespec => "path/to/test.thpl",
         params => {"KEY" => "PARAM1"}},
    ]
);

=item Return values

This method returns a NATE::ProcManager instance.

=item Examples

    use NATE::ProcManager;
    use NATE::Process;

    my $pm = NATE::ProcManager->new(
                proc_info => [
                    {codespec => "path/to/test.thpl",
                    params => {"KEY" => "PARAM1"}},   #test1
                    {codespec => "validate/stests/verify_param_value.thpl",
                    params => {"KEY" => "PARAM2"}},   #test2
                    $proc,                            #test3
                ],
            );
    $pm->start;

    $pm->add(
        proc_info => [
            {codespec => "path/to/anothertest.thpl",
             params => {"KEY" => "ANOTHERTESTNAME"}}, #test4
        ]
    );
    $pm->start;

=back

=cut

#---------------------------------------------------------------------------

sub add {
    my $self = shift;

    my %opts = validate(@_, {
        proc_info       => { type => ARRAYREF},
        client          => { type => SCALAR|UNDEF, optional => 1},
        timers          => { type => HASHREF|UNDEF,optional => 1},
        iterations      => { type => SCALAR,       default  => 1},
        runtime         => { type => SCALAR,       optional => 1},
        independent     => { type => BOOLEAN,      default  => 0},
        ignore_results  => { type => BOOLEAN,      optional => 1},
        background      => { type => BOOLEAN,      default  => 1},
        randomize       => { type => BOOLEAN,      optional => 1},
        onexit          => { type => CODEREF,      optional => 1},
    });

    $self->{iterations}  = $opts{iterations};
    $self->{runtime}     = $opts{runtime} if($opts{runtime});
    $self->{timers}      = $opts{timers} if($opts{timers});
    $self->{independent} = $opts{independent};
    $self->{background}  = $opts{background};
    $self->{client}      = $opts{client} if($opts{client});
    $self->{onexit}      = $opts{onexit} if($opts{onexit});
    $self->{randomize}   = delete $opts{randomize} if($opts{randomize});
    $self->{ignore_results} = delete $opts{ignore_results} if($opts{ignore_results});

    # Start ID at last child id to prevent overwriting previous children
    my $i = $self->{last_child_id};
    foreach my $proc_info (@{$opts{proc_info}}) {
        $i++;
        if((ref $proc_info eq 'HASH') ) {
            my $child_id = sprintf("%02d",$i);
            my $proc     = $self->_create_child( proc_info => $proc_info, child_id => $child_id );
            $self->{children}->{$i} = $proc;
        } elsif ( $proc_info->isa('NATE::Process') ) {
            # All sub-units of ProcManager will inherit 'background' and 'independent' properties
            $proc_info->{background} = $opts{background};
            $proc_info->{independent} = $opts{independent};
            $self->{children}->{$i} = $proc_info;
        } elsif ( $proc_info->isa('San::FIT::Misfit::Core::NotProcManager') ) {
            $self->{children}->{$i} = $proc_info;
        }
        else {
            NATE::Result::Script->new("Invalid proc_info specification!: Unknown format")->log;
        }
        $self->{last_child_id} = $i;
        $self->{preserve_runid} ? $self->{children}->{$i}->parent() :  $self->{children}->{$i}->parent($self->{runid});
    }

    return $self;
}

#=======================================================================
# parent
# This internal method that takes care of adoption of a NATE::ProcManager
# by another PM. Can/Should be called only once on a ProcManager instance
#=======================================================================

sub parent {
    my ($self,$parent_runid) = @_;
    die 'Cannot change parent! ProcManager '.$self->{runid}.' already adopted'
        if($self->{adopted});
    # modify runid
    $self->runid( prefix => $parent_runid ) if($parent_runid);
    $self->{adopted} = 1;
    return $self;
}

#=======================================================================
# _create_child
# This internal method converts given proc_info into a NATE::Process
# instance
#=======================================================================

sub _create_child {
    my $self = shift;
    my %opts = validate(@_, {
        proc_info => { type => HASHREF },
        child_id  => { type => SCALAR },
    });

    my $child_id  = $opts{child_id};
    my $proc_info = $opts{proc_info};
    die 'Invalid proc_info specification! "codespec" was not provided.'
            if!(exists $proc_info->{'codespec'});

    my $child_logdir = $self->logdir();
    my $runid        = choose_runid($proc_info->{'codespec'}, $child_id);
    my %proc_args           = ( codespec => $proc_info->{'codespec'},);
    $proc_args{params}      = \%{$proc_info->{'params'}} if(exists $proc_info->{'params'});
    $proc_args{runid}       = $proc_info->{runid} ? delete $proc_info->{runid} : $runid;
    $proc_args{independent} = $self->{independent};
    $proc_args{background}  = $self->{background};
    $proc_args{client}      = $proc_info->{client} ? delete $proc_info->{client} : $self->{client};
    delete $proc_info->{background} if($proc_info->{background});
    delete $proc_info->{independent} if($proc_info->{independent});
    delete $proc_info->{params} if($proc_info->{params});
    %proc_args = (%proc_args, %$proc_info);
    return NATE::Process->new( %proc_args );
}

#=======================================================================
# logdir
# This is an internal method to workout logdir logistics. If user
# has defined logdir, this method does nothing but simply return that.
# If not this method constructs logdir using TOP_LOGDIR, RUNID param
# and this PM's runid.
#=======================================================================

sub logdir {
    my $self = shift;
    my $paramset = NATE::ParamSet->get_global();
    return undef unless($paramset->get('LOGDIR')); #XXX This part needs to be tweaked depending on the remote OS

    # $self->{logdir} is set only if logdir is user-defined.
    return $self->{logdir} if($self->{logdir});
    my $parent_runid = $paramset->get('TOP_LOGDIR').'/'.$paramset->get('RUNID');
    return $parent_runid if($self->{preserve_runid});
    return ($parent_runid.'/'.$self->{runid});
}

#=======================================================================
# choose_runid
# This is an internal method used to choose appropriate runid for a fresh
# NATE::Process instance
#=======================================================================

sub choose_runid($$;) {
    my ($codespec, $stepid) = @_;
    my ($scriptname, $scriptname_short) = guess_scriptname($codespec,
                                                           scriptspec_type($codespec));
    my $runid;
    $runid = $stepid;
    $runid .= '_'.$scriptname_short if($scriptname_short);
    return $runid;
}

#=======================================================================
# runid
# This is an internal method used to set runid of sub-units - either
# prefix or entirely replace their runid.
#=======================================================================

sub runid {
    my $self = shift;
    my %opts = validate(@_, {
        prefix  => { type => SCALAR, optional => 1 },
    });

    return $self->{runid} if(!scalar keys %opts);
    my $runid = $opts{prefix} ? $opts{prefix}.'/'.$self->{runid} : undef;
    if($runid) {
        $self->{runid} = $runid;
        foreach my $child (values %{$self->{children}}) {
            $child->runid( %opts );
        }
    }

    return $self->{runid};
}

#=======================================================================
# set_param
# This is a params setter method that is used by ProcManager
#=======================================================================

sub set_param{
    my $self = shift;
    my %opts = validate(@_, {
        params    => { type => HASHREF},
        overwrite => { type => BOOLEAN, default => 1},
    });

    foreach my $child (values %{$self->{children}}) {
        $child->set_param(params    => $opts{'params'},
                          overwrite => $opts{'overwrite'});
    }
    return $self;
}

#---------------------------------------------------------------------------

=head2 start

This method is used to trigger start of all sub-process instances

=over

=item Synopsis

 my $procMan = NATE::ProcManager->new( proc_info => [ {codespec => "test.thpl"} ]);
 $procMan->start();

=item Arguments

None

=item Return values

None

=back

=cut

#---------------------------------------------------------------------------

sub start {
    my $self = shift;

    $self->{running} = {} unless(defined($self->{running}));
    $self->{finished_running} = {} unless(defined($self->{finished_running}));

    my $running_count = scalar keys %{$self->{running}};
    my $finished_count = scalar keys %{$self->{finished_running}};
    if (($running_count + $finished_count) == $self->{last_child_id}) {
        # No new tasks to start
        return $self;
    }
    my $fork_id = ++$self->{last_fork_id};
    my $runlength = $self->{iterations};
    # Cycle through all children IDs and assign to a new fork if unassigned
    foreach my $child_id (keys %{$self->{children}}){
        my $child = $self->{children}->{$child_id};
        if (defined($self->{running}->{$child_id})) {
            # Child already running in another fork
            next
        } elsif ($child->{iteration} > $runlength) {
            # Child already run max iterations
            next
        } else {
            $self->{children}->{$child_id}->{fork_id} = $fork_id;
            $self->{running}->{$child_id} = 1;
        }
    }
    $self->{proc} = NATE::Child::Fork->new
                    (call                 => [\&_start, $self, @_],
                     on_termination       => [\&_on_termination, $self],
                     kill_if_parent_exits => 1,
    );
    $self->{forks}->{$fork_id} = $self->{proc};
    $self->{proc} = undef;

    # fork() creates copies of children; so parent's references to children
    # need to be updated with correct 'iteration' value.
    $self->_increment_iterations();
    $self->wait() if(!$self->{background} && !$self->{adopted});
    return $self;
}

sub _increment_iterations {
    my $self = shift;
    foreach my $child_id (keys %{$self->{children}}) {
        my $child = $self->{children}->{$child_id};
        if ($child->{fork_id} != $self->{last_fork_id}) {
            # Child not in this fork
            next
        }
        if($child->isa('NATE::Process')) {
            $child->{iteration}++;
        } else {
            $child->_increment_iterations();
        }
    }
    return $self;
}

sub _start {
    my $self = shift;
    my %opts = @_;
    $NATE::Database::Test::Global->db_forked();
    staf_global()->handle();
    my $log = log_global();

    if(my $logfile = $self->{logfile}) {
        if(!-e $logfile) # Why does the file not exist?
        {
            my $logdir = dirname($logfile);
            mkpath($logdir) if(!-e $logdir);
        }
        # Redirect logging to process's own logfile, if there was one
        redirect_stdout_to_file($logfile);
        my $var = NATE::ParamSet->new( global => 1 );
        $log = $Tharn::Log = NATE::Log::reinitialize_log(logspec => $var->get('LOGSPEC'), store_results => 1);
    }

    $self->{running} = {} unless(defined($self->{running}));
    $self->{starttime} = time() unless($self->{starttime});
    my @order = keys %{$self->{children}};
    @order = shuffle(@order) if($self->{randomize});

    $self->set_timers() if($self->{timers});
    my $runlength = $self->{iterations};
    while($runlength-- && !$self->{terminated}) {
        foreach my $child_id (@order){
            last if($self->{terminated});
            my $child = $self->{children}->{$child_id};
            if (!($self->{children}->{$child_id}->{fork_id} == $self->{last_fork_id})) {
                # Child does not belong to this fork, remove it locally
                delete($self->{children}->{$child_id});
                delete($self->{running}->{$child_id});
                next;
            }
            $log->trace('Starting child '.$child->{runid});
            $child->start(%opts);
            $self->{running}->{$child_id} = 1;
            if(!$self->{background}) {
                $child->wait();
                next unless($child->isa('NATE::Process'));
                # Retrive param_results from finished child and pass on to other children
                # This need not be done if finished is a NATE::ProcManager, scripts inside
                # a block can't affect those outside or other children of super-parent ProcManager
                my @param_results = $child->results(result_type => 'PARAM');
                foreach my $param_result (@param_results) {
                    foreach my $assignment ($param_result->messages()) {
                        my ($name, $value) = $assignment =~ /^(\w+)=(.*)/s;
                        $opts{params}->{$name} = $value;
                    }
                }
            }
        }
        $self->wait();
        if($self->{runtime}) {
            my $runduration = time() - $self->{starttime};
            if($runduration >= $self->{runtime}) {
                $log->trace("Exiting! $self->{runid} completed expected runtime $runduration seconds");
                $self->{terminated} = 1;
            } else {
                $log->trace("Restarting! $self->{runid} completed: $runduration seconds. ".
                              "Expected runtime: $self->{runtime} seconds");
                $runlength++;
            }
        }
    }
    my $worst_result = $self->worst_result();
    exit( $worst_result ? $worst_result->exit_code() : 0 );
}


sub set_timers {
    my $self = shift;
    foreach my $timer_type (keys %{$self->{timers}}) {
        $self->{timers}->{$timer_type}->{setter}->($self);
    }
    return $self;
}

sub remove_timers {
    my $self = shift;
    foreach my $timer_type (keys %{$self->{timers}}) {
        my $timer = $self->{timers}->{$timer_type}->{timer};
        NATE::Events::timer_remove($timer) if($timer);
    }
    return $self;
}

sub _on_termination {
    my $child = shift;
    my $self  = shift;
    my $status = $child->exit_status();
    log_global()->trace("!! ProcManager $self->{runid} exited with $status");
    $self->{onexit}->($self) if($self->{onexit});
    $self->remove_timers() if($self->{timers});
    # Cycle through all children and finish/delete only those in this fork
    while (my @children = keys(%{$child->{children}})) {
        foreach my $child_id (@children) {
            if ($self->{children}->{$child_id}->{fork_id} == $child->{last_fork_id}) {
                $self->{finished_running}->{$child_id} = 1;
                delete($self->{running}->{$child_id});
            }
        }
    }
}

#---------------------------------------------------------------------------

=head2 wait

If sub-units are spawned in background this method can be used to wait until all of them terminate.

=over

=item Synopsis

  use NATE::ProcManager;
  my $procMan = NATE::ProcManager->new( proc_info => [ {codespec => "test.thpl"} ]);
  $procMan->start();
  $procMan->wait();

=item Arguments

None

=item Return values

None

=back

=cut

#---------------------------------------------------------------------------

sub wait {
    my $self = shift;
    return $self if( !($self->{running} && %{$self->{running}}) && !($self->{proc}));

    while(my @temp = keys(%{$self->{running}}))
    {
        foreach my $id (@temp)
        {
            next if($self->{children}->{$id}->{proc}->{alive});
            $self->{finished_running}->{$id} = 1;
            delete($self->{running}->{$id});
        }
        last if(!($self->{running} && %{$self->{running}}));
        eventloop_drain();
    }
    if (defined($self->{proc})) {
        eventloop_drain(timeout => 10) while($self->{proc} && $self->{alive});
        return $self;
    }
    foreach my $fork_id (keys %{$self->{forks}})
    {
        my $fork = $self->{forks}->{$fork_id};
        eventloop_drain() while($fork->{proc} && $fork->{alive});
    }
    return $self;
}

#---------------------------------------------------------------------------

=head2 get_live_children

This method returns an array of children that are currently in progress.

=over

=item Synopsis

  my $procMan = NATE::ProcManager->new( proc_info => [ $proc1, $proc2, $proc3 ]);
  $procMan->start();
  my @live_children = $procMan->get_live_children();

=item Arguments

None

=item Return values

 Array of children that are currently running.

=back

=cut

#---------------------------------------------------------------------------

sub get_live_children {
    my $self = shift;
    my @live;
    foreach my $child_id (keys %{$self->{running}}){
         push @live, $self->{children}->{$child_id} ;
    }
    return @live;
}

#---------------------------------------------------------------------------

=head2 stop

Stop all currenlty active children.

=over

=item Synopsis

  my $procMan = NATE::ProcManager->new( proc_info => [ $proc1, $proc2, $proc3 ]);
  $procMan->start();
  ... (do some other stuff) ...
  $procMan->stop();
  $procMan->destroy(); #Mandatory after final stop()

=item Arguments

None

=item Return values

None

=back

=cut

#---------------------------------------------------------------------------

sub stop {
    my $self = shift;
    my %opts = validate(@_, {
        message        => { type => ARRAYREF|UNDEF, optional => 1 },
        remote_message => { type => ARRAYREF|UNDEF, optional => 1 },
    });

    foreach my $child ($self->get_live_children){
        log_global()->trace('Stopping child '.$child->{runid});
        $child->stop(%opts);
    }
    foreach my $fork_id (keys %{$self->{forks}}){
        my $fork = $self->{forks}->{$fork_id};
        if (UNIVERSAL::isa($fork, 'NATE::Child::Fork')) {
            log_global()->trace('Stopping fork '.$fork_id);
            $fork->kill(0,$self->{kill_timeout}) if($fork);
            delete($self->{forks}->{$fork_id});
        }
    }
    return $self;
}

#---------------------------------------------------------------------------

=head2 destroy

Destroys all children. This method must be invoked when procManager
instance is no longer needed, in order to free up resources.

=over

=item Synopsis

  my $procMan = NATE::ProcManager->new( proc_info => [ $proc1, $proc2, $proc3 ]);
  $procMan->start();
  ... (do some other stuff) ...
  $procMan->stop();
  $procMan->start();
  ... (complete any number of iterations) ...
  $procMan->stop();
  $procMan->destroy();  #Mandatory after final stop()

=item Arguments

None

=item Return values

None

=back

=cut

#---------------------------------------------------------------------------

sub destroy {
    my $self = shift;
    foreach my $child (values %{$self->{children}}){
        log_global()->trace("Destroying child ".$child->{runid});
        $child->destroy;
    }
    return $self;
}

#---------------------------------------------------------------------------

=head2 cleanup_finished

If sub-units are spawned and completed in background this method can be used
to free the results and consumed memory.

=over

=item Synopsis

  use NATE::ProcManager;
  my $procMan = NATE::NATE::ProcManager->new(
      proc_info => [ {codespec => "test.thpl"} ]);
  $procMan->start();
  $procMan->cleanup_finished();

=item Arguments

None

=item Return values

None

=back

=cut

#---------------------------------------------------------------------------

sub cleanup_finished {
    my $self = shift;

    # Check to see if the child is active or finished
    foreach my $child_id (keys(%{$self->{children}})) {
        my $fork_id = $self->{children}->{$child_id}->{fork_id};
        if ($self->{forks}->{$fork_id}->{alive}) {
            # Child is alive, skip over it
            next;
        } else {
            # Child is finished, mark it for removal
            $self->{finished_running}->{$child_id} = 1;
            delete($self->{running}->{$child_id});
            $self->{children}->{$child_id}->stop();
        }
    }
    my @forks = keys %{$self->{forks}};
    foreach my $fork_id (@forks){
        my $fork = $self->{forks}->{$fork_id};
        if ($fork->{alive}) {
            # Fork still alive, check next
            next;
        } elsif ($fork->{exit_status}) {
            # Fork dead, kill it and delete it
            $fork->kill(0,$self->{kill_timeout});
            delete($self->{forks}->{$fork_id});
        }
    }
    return $self;
}

#---------------------------------------------------------------------------

=head2 is_independent

Getter for 'independent' attribute of NATE::ProcManager instance

=over

=item Synopsis

  ...
  my $independent = $procMan->is_independent();
  ...


=item Arguments

None

=item Return values

Boolean

=back

=cut

#---------------------------------------------------------------------------

sub is_independent {
    my $self = shift;
    return $self->{independent};
}

#---------------------------------------------------------------------------

=head2 is_running

Getter for 'running' attribute of NATE::ProcManager instance

=over

=item Synopsis

  ...
  my $running = $procMan->is_running();
  ...


=item Arguments

None

=item Return values

Boolean

=back

=cut

#---------------------------------------------------------------------------

sub is_running {
    my $self = shift;
    return (($self->{proc} || $self->get_live_children()) ? 1 : 0);
}

#---------------------------------------------------------------------------

=head2 own_results

Returns an array of all the result objects directly logged by sub-units, but not by any of their subordinate subtests.

=over

=item Synopsis

 @results = $pm->own_results();

=item Arguments

None

=item Return values

This method returns array of result objects

=back

=cut

#---------------------------------------------------------------------------

sub own_results {
    my $self = shift;
    return undef if($self->{ignore_results});
    my @own_results;
    foreach my $child (values %{$self->{children}}){
        log_global()->trace("Gathering own results for child ".$child->{runid});
        push @own_results, $child->own_results();
    }
    log_global()->trace("ProcManager $self->{runid} own results: ". join("\n",@own_results));
    return @own_results;
}

#---------------------------------------------------------------------------

=head2 results

Returns an array of all the result objects logged by sub-units, including
those generated by their subordinate subtests.

=over

=item Synopsis

 @results = $pm->results();

=item Arguments

None

=item Return values

This method returns array of result objects

=back

=cut

#---------------------------------------------------------------------------

sub results {
    my $self = shift;
    my %opts = @_;
    return undef if($self->{ignore_results});
    my @results;
    foreach my $child (values %{$self->{children}}){
        log_global()->trace("Gathering results for child ".$child->{runid});
        push @results, $child->results(%opts);
    }
    log_global()->trace("ProcManager $self->{runid} results: ". join("\n",@results));
    return @results;
}

sub result_summary {
    my $self = shift;
    return undef if($self->{ignore_results});
    my %result_summary;
    foreach my $child (values %{$self->{children}}){
        log_global()->trace("Gathering results for child ".$child->{runid});
        my $child_summary = $child->result_summary();
        %result_summary = (%result_summary, %{$child_summary}) if($child_summary && %$child_summary);
    }
    return \%result_summary;
}

#---------------------------------------------------------------------------

=head2 own_worst_result

Returns worst result among all the results logged by sub-units, excluding
those generated by their subordinate subtests.

=over

=item Synopsis

 $result = $pm->own_worst_result();

=item Arguments

None

=item Return values

This method returns single result object

=back

=cut

#---------------------------------------------------------------------------

sub own_worst_result {
    my $self = shift;
    return undef if($self->{ignore_results});
    my @own_results;
    my $own_worst_result;
    foreach my $child (values %{$self->{children}}){
        log_global()->trace("Gathering own results for child ".$child->{runid});
        push @own_results, $child->own_worst_result();
    }
    $own_worst_result = NATE::Result::worst(@own_results);
    log_global()->trace("ProcManager $self->{runid} own worst result: $own_worst_result") if($own_worst_result);
    return $own_worst_result;
}

#---------------------------------------------------------------------------

=head2 worst_result

Returns worst result among all the results logged by sub-units, including
those generated by their subordinate subtests.

=over

=item Synopsis

 $result = $pm->worst_result();

=item Arguments

None

=item Return values

This method returns single result object

=back

=cut

#---------------------------------------------------------------------------

sub worst_result {
    my $self = shift;
    return undef if($self->{ignore_results});
    my @results;
    my $worst_result;
    foreach my $child (values %{$self->{children}}){
        log_global()->trace("Gathering results for child ".$child->{runid});
        push @results, $child->worst_result();
    }
    $worst_result = NATE::Result::worst(@results);
    log_global()->trace("ProcManager $self->{runid} worst result: $worst_result") if($worst_result);
    return $worst_result;
}

=head1 AUTHOR/MAINTAINER

=over

=item NATE Development (dl-nate-dev@netapp.com)
=item Modified by SANFIT (dl-san-fit@netapp.com)

=back

 $Id: /u/williamd/p4/test/lib/San/FIT/Misfit/Core/NotProcManager.pm $
 Last revised $Date: 2012/07/31 $ by $Author: williamd $

=cut

1;
