
package San::FLI::FLIHelper;

#
# Module import.
#
use strict;
use warnings;
use ParamsMgr;
use Data::Compare;
use base 'NACL::C::Component::ONTAP';
use Params::Validate qw(validate_with SCALAR ARRAYREF SCALARREF);
use San::San;
use San::SanMgr;
use San::SanClient;
use San::ScsiTool;
use San::ScsiTool::ScsiUtil;
use san::scsi::scsit_wafl;
use San::ScsiTool::CDB;
use San::SanClient::ClientUtil;
use NACL::STask::Cluster;
use NACL::STask::Volume;
use NACL::STask::Lun;
use NACL::C::Node;
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NATE::BaseException qw(:try);
use Tharn qw($Log);
use File::Basename;
use Data::Dumper;
use Burt;
use IOMan;

sub new {
    my ($class, %args) = @_;
	my $self = {};
    bless($self, $class);
    my $Xml_File;
    my $Debug_Level;
	my $Node;
    foreach my $param (keys %args) {
        if ($param =~ m/file/ ) {
            unless ($Xml_File = $args{$param}) {
                $self->{'error'} =
                  'xml file is not passed to create SanMgr Object';
                return $self;
            }
        } elsif ($param =~ /debug_level$/ ) {
            $Debug_Level = $args{$param};
        } elsif ($Node =~ /node$/ ) {
            $Node = $args{$param};
        }
    } ## end foreach my $param (keys %args)
    $Log->comment(  'Creating SanMrg Object with xml file:'
                  . $Xml_File
                  . ' and Debug_Level:'
                  . $Debug_Level);
    $self->{'SanMgr_Obj'} = new San::SanMgr(-file => $Xml_File);
	$Log->comment(' SanMgr '. $self->{'SanMgr_Obj'} );
    if ($self->{'SanMgr_Obj'}->error()) {
        $Log->comment(
              "Failed in SanMgr constructor" . $self->{'SanMgr_Obj'}->errmsg());
        return 0;
    }
    $self->{'SanMgr_Obj'}->setup(-debug_level => $Debug_Level);
    if ($self->{'SanMgr_Obj'}->error()) {
        $self->{'error'} = 'xml file is not passed to create SanMgr Object';
        return $self;
    }
	
    # $Log->comment("Create XScsiT object");
    # $self->{'San_Scsit_Obj'} =
      # new San::SanScsiT::XScsiT(-san_obj         => $self->{'SanMgr_Obj'},
                                # -reboot_required => 0);
	 
    # if ($self->{'San_Scsit_Obj'}->error()) {
        # $Log->comment($self->{'San_Scsit_Obj'}->errmsg());
        # return $TCD::CONFIG;
    # }

    #my @Client_Objs = @{$self->{'San_Scsit_Obj'}->{client_objs}};
	
	my @Client_Objs = @{$self->{'SanMgr_Obj'}->get_objects(-type => "client")};
    
	# Pick one of the clients to perform this test
    if (scalar @Client_Objs > 0) {
        $self->{'Client_Obj'} = $Client_Objs[0];
    } else {
        $Log->comment("There is no client to perform the test");
        return 0;
    }

    my $Client_Name = $self->{'Client_Obj'}->hostname();

    #get the scsit tool obj
	$self->{'Scsi_Tool_Obj'} = ScsiTool->create_scsi_tool(
									-client => $self->{'Client_Obj'},
									-tool => 'spt',
                                    -debug_level => $Debug_Level);
    # Create ScsiUtil Object
    $self->{'Scsi_Util_Obj'} =
      new San::ScsiTool::ScsiUtil(-scsi_tool_obj => $self->{'Scsi_Tool_Obj'});
    if ($self->{'Scsi_Util_Obj'}->error()) {
        $Log->comment($self->{'Scsi_Util_Obj'}->error());
        return $TCD::CONFIG;
    }
    return $self;
} ## end sub new

## Write to the LUN with lun path passed from the host
# input param
#	1. device path
#	2. start_LBA
#	3. End_LBA
#	4. Pattern
# return :
#   1 if data is written successfully; otherwise 0
sub write_to_lun_at_lba {
    my ($self, %args) = @_;
    my $device_path;
    my $write_fail = 1;
    my $spt_obj    = $self->{'Scsi_Tool_Obj'};
    my $start_lba  = 3000;
    my $end_lba    = 8000;
    my $pattern;
    foreach my $param (keys %args) {
        if ($param =~ /^-?device_path$/) {
            unless ($device_path = $args{$param}) {
				$Log->comment('Device path is not passed');
				return 0;
			}
        } elsif ($param =~ /^-?start_lba$/) {
            $start_lba = $args{$param};
        } elsif ($param =~ /^-?end_lba$/) {
            $end_lba = $args{$param};
        } elsif ($param =~ /^-?pattern$/) {
            $pattern = $args{$param};
        }
    } ## end foreach my $param (keys %args)
	$Log->comment('Writing on device '. $device_path .' with pattern '. $pattern . ' from start lba '. $start_lba . ' still end lba '. $end_lba);
	if (!(defined $pattern)) {
		for (my $j = $start_lba ; $j <= $end_lba; $j=$j+8) {
			my $lba=$j;
			for ( $pattern = 1 ;  $pattern <=8 ; $pattern++){
				if ($spt_obj->read_or_write_lba(-dev        => $device_path,
												-start_lba  => $lba++,
												-pattern    => $pattern,
												-read_write => "write",
												-alloc_len  => 1) eq "FAIL"
				  ) {
					$Log->comment("Error: Write failed");
					$write_fail = 0;
					return $write_fail;
				} ## end if ($spt_obj->read_or_write_lba...
			}
		} ## end for (my $j = $start_lba...
	} else {
		for (my $j = $start_lba ; $j <=$end_lba; $j=$j+8) {
			if ($spt_obj->read_or_write_lba(-dev        => $device_path,
											-start_lba  => $j,
											-pattern    => $pattern,
											-read_write => "write",
											-alloc_len  => 8) eq "FAIL"
			  ) {
				$Log->comment("Error: Write failed");
				$write_fail = 0;
				return $write_fail;
			} ## end if ($spt_obj->read_or_write_lba...
		} ## end for (my $j = $start_lba...
	}
    return $write_fail;

} ## end sub write_to_lun_at_lba

## Write to the LUN with lun path passed from the host
# input param
#	1. spt tool obj
#	2. device path
#	3. start_LBA
#	4. End_LBA
#	5. Pattern
# return :
#   1 if data is verified successfully; otherwise 0
sub verify_data_lun_at_lba {
    my ($self, %args) = @_;
    my $read_fail = 1;
    my $device_path;
    my $ret_ref;
    my $spt_obj   = $self->{'Scsi_Tool_Obj'};
    my $start_lba = 3000;
    my $end_lba   = 8000;
    my $pattern;

    foreach my $param (keys %args) {
		if ($param =~ /^-?device_path$/) {
            unless ($device_path = $args{$param}) {
				$Log->comment('Device path is not passed');
				return 0;
			}
        } elsif ($param =~ /^-?start_lba$/) {
            $start_lba = $args{$param};
        } elsif ($param =~ /^-?end_lba$/) {
            $end_lba = $args{$param};
        } elsif ($param =~ /^-?pattern$/) {
            $pattern = $args{$param};
        }
    } ## end foreach my $param (keys %args)
	$Log->comment('verify on device '. $device_path .' with pattern '. $pattern . ' from start lba '. $start_lba . ' still end lba '.$end_lba);
	if (defined $pattern){
    for (my $j = $start_lba ; $j <=$end_lba; $j++) {
        $ret_ref =
          $spt_obj->read_or_write_lba(-dev        => $device_path,
                                      -start_lba  => $j,
                                      -read_write => "read",
                                      -alloc_len  => 1);
        if ($spt_obj->error()) {
            $Log->comment("Error: Read failed");
            return $read_fail;
        }
        $ret_ref =
          $spt_obj->check_pattern(-data_buf => $ret_ref->{SCSI_DATA},
                                  -pattern  => $pattern,);

        if ($spt_obj->error) {
            $Log->comment("Error : Verification of the Pattern has failed.");
            $Log->comment("FAIL : Error when " . $spt_obj->errmsg());
            $read_fail = 0;
            return $read_fail;
        }
		}
	} 
	else {
		for (my $j = $start_lba ; $j <=$end_lba; $j=$j+8) {
			my $lba=$j;
			for ( $pattern = 1 ;  $pattern <= 8 ; $pattern++) {
				$ret_ref =
				  $spt_obj->read_or_write_lba(-dev        => $device_path,
											  -start_lba  => $lba++,
											  -read_write => "read",
											  -alloc_len  => 1);
				if ($spt_obj->error()) {
					$Log->comment("Error: Read failed");
					return $read_fail;
				}
				$ret_ref =
				  $spt_obj->check_pattern(-data_buf => $ret_ref->{SCSI_DATA},
										  -pattern  => $pattern,);

				if ($spt_obj->error) {
					$Log->comment("Error : Verification of the Pattern has failed.");
					$Log->comment("FAIL : Error when " . $spt_obj->errmsg());
					$read_fail = 0;
					return $read_fail;
				}
			}
		}
	}
    return $read_fail;
} ## end sub Verify_Data_LUN_At_LBA

## Compare data of two LUNs
# input param
#	1. spt tool obj
#	2. device path1
#	3. device path2
#	4. start_LBA
#	5. End_LBA
# return :
#   1 if data is written successfully; otherwise 0
sub compare_data_luns_at_lba {
    my ($self, %args) = @_;
    my $read_fail = 1;
    my $spt_obj   = $self->{'Scsi_Tool_Obj'};
    my $device_path1;
    my $device_path2;
    my $start_lba=0;
    my $end_lba=64;
    my ($dev_ret_ref1, $dev_ret_ref2);
    foreach my $param (keys %args) {
        if ($param =~ /^-?device_path1$/) {
            unless ($device_path1 = $args{$param}) {
				$Log->comment('Failed:First Device path is not passed');
                return 0;
            }
        } elsif ($param =~ /^-?device_path2$/) {
            unless ($device_path2 = $args{$param}) {
				$Log->comment('Failed:Second Device path is not passed');
                return 0;
            }
        } elsif ($param =~ /^-?start_lba$/) {
            unless ($start_lba = $args{$param}) { 
				$Log->comment('Start LAB is not given, Using default value:0');
			}
        } elsif ($param =~ /^-?end_lba$/) {
            unless ($end_lba = $args{$param}) { 
				$Log->comment('End LAB is not givenl, Using default value:64');
			}
        } 
    } ## end foreach my $param (keys %args)
	$Log->comment('Compare the data on device '. $device_path1 . ' and ' .$device_path2);
	$Log->comment(' from start lba '. $start_lba . ' still end lba '.$end_lba);
    for (my $j = $start_lba ; $j <=$end_lba; $j++) {
        $Log->comment(
                   "Reading data on device " . $device_path1 . " at LBA " . $j);
        $dev_ret_ref1 =
          $spt_obj->read_or_write_lba(-dev        => $device_path1,
                                      -start_lba  => $j,
                                      -read_write => "read",
                                      -alloc_len  => 1);
        if ($spt_obj->error()) {
            $Log->comment("Error: Read failed");
            $read_fail = 0;
            return $read_fail;
        }
        $Log->comment(
                   "Reading data on device " . $device_path2 . " at LBA " . $j);
        $dev_ret_ref2 =
          $spt_obj->read_or_write_lba(-dev        => $device_path2,
                                      -start_lba  => $j,
                                      -read_write => "read",
                                      -alloc_len  => 1);
        if ($spt_obj->error()) {
            $Log->comment("Error: Read failed");
            $read_fail = 0;
            return $read_fail;
        }
        $Log->comment(  "Compare data on device "
                      . $device_path1 . " and "
                      . $device_path2
                      . "at LBA "
                      . $j);
        if (!(Compare(\$dev_ret_ref1->{SCSI_DATA}, \$dev_ret_ref1->{SCSI_DATA}))) {
            $Log->comment("Error : Verification of the Pattern has failed.");
            $read_fail = 0;
            return $read_fail;
        }
	} ## end for (my $j = $start_lba...
	
    return $read_fail;
} ## end sub Compare_Data_LUNs_At_LBA

## Discover the Net Lun mapped to host 
# 1. lun_path for which path need to find in singal or array ref
# 2. Type of lun path (primary|secondary)
#
# return : Netapp LUN path array of paths or single path
#
sub Get_NL_Device_Path{
   my ($self, %args) = @_;
   my $lun_path;
   my $device_path_type = 'primary';
   my $Scsi_util_obj = $self->{'Scsi_Util_Obj'};
   my @paths_data;my $paths_info;
    foreach my $param (keys %args) {
        if ($param =~ /^-?lun_path$/) {
            unless ($lun_path = $args{$param}) {
				$Log->comment("Lun path should be passed to dicovery the device path");
                return 0;
            }
        } elsif ($param =~ /^-?path_type$/) {
            $device_path_type = $args{$param};
		}
	}
	## discovering the LUN on client
	$self->{'Client_Obj'}->discover();
	$Log->comment('Lun path list : ' . Dumper $lun_path);
	if ($lun_path =~ /ARRAY/i){
		for (my $path=0; $path<(@{$lun_path}); $path++){ 
			$paths_info =
				  $Scsi_util_obj->get_paths_to_lun(-lun   => $lun_path->[$path]);
			$Log->comment('path list of lun : ' .$lun_path->[$path].' is Dumper '.  Dumper $paths_info);
			$Log->comment('Ref of path_info: '. ref $paths_info);
			if ($paths_info =~ /HASH/) {
				if ( $device_path_type =~ /pri/i ){
					if (defined $paths_info->{pri}){
						$paths_data[$path]->{$lun_path->[$path]} = ${$paths_info->{pri}}[0];
					} else {
						$Log->comment("Primary path is not available for the lun ".$lun_path->[$path] );
						$Log->comment("Taking secondry path " . ${$paths_info->{sec}}[0]);
						if (@{$paths_info->{sec}}){
							$paths_data[$path]->{$lun_path->[$path]} = ${$paths_info->{sec}}[0];
						}else {
							$Log->comment("No path found for LUN " . $lun_path->[$path]);
							return 0;
						}
					}
				} else {
					if (defined $paths_info->{sec}){
						$paths_data[$path]->{$lun_path->[$path]} = ${$paths_info->{sec}}[0] if (@{$paths_info->{sec}});
					} else {
						$Log->comment("Secondary path is not available for the lun ".$lun_path->[$path] );
						$Log->comment("Taking primary path " . ${$paths_info->{sec}}[0]);
						if (@{$paths_info->{pri}}){
						$paths_data[$path]->{$lun_path->[$path]} = ${$paths_info->{pri}}[0];
						} else {
							$Log->comment("No path found for LUN " . $lun_path->[$path]);
							return 0;
						}
					}
				}
			} else {
				$Log->comment("Unable to disCover the device path for lun " . Dumper $lun_path);
				$Log->comment("paths found for lun " . Dumper \@paths_data);
			}
		}
		$Log->comment("paths found for lun " . Dumper \@paths_data);
		return @paths_data;
	} else {
		$paths_info =
			  $Scsi_util_obj->get_paths_to_lun(-lun   => $lun_path,
										   -force => 1);
		if ($paths_info) {
				@paths_data = @{$paths_info->{pri}} if ( $device_path_type =~ /pri/i );
				@paths_data = @{$paths_info->{sec}} if ( $device_path_type =~ /sec/i );
		} else {
			$Log->comment("Unable to discover the device path for lun " . $lun_path);
		}
	}
	return @paths_data;
}

## Run IO (fill) Lun with data 
# 1. device path for host
# 2. start lba
# 3. end lba
#
# return : Pass-1 or Fail-0
#
sub run_IO{
my ($self, %args) = @_;
my $IOMan = new IOMan();
my $device_path;
my $start_lba=4096;
my $end_lba=100000;

foreach my $param (keys %args) {
	if ($param =~ /^-?device_path$/) {
		unless ($device_path = $args{$param}) {
			$Log->comment('Device path is not passed');
			return 0;
		}
	} elsif ($param =~ /^-?start_lba$/) {
		$start_lba = $args{$param};
	} elsif ($param =~ /^-?end_lba$/) {
		$end_lba = $args{$param};
	}
} ## end foreach my $param (keys %args)
	
if ($IOMan->error()) {
            $Log->comment($IOMan->errmsg());
            return 0;
     }
my $get_client_name = $self->{'Client_Obj'}->hostname();
my @io_records = $IOMan->create_io_records(
        -client  => $get_client_name,
        -devices => [$device_path],
        -io_tool => 'dt',
        -iotype  => 'sequential',
        -pattern => 'iot',
        -write     => '1',
        -bs        => "64k",
		-limit	  => $end_lba,
		-lba 		=> $start_lba,
    );
    if ($IOMan->error()) {
        $Log->comment($IOMan->errmsg());
        return 0;
    }

    # start I/O
    $IOMan->start(io_records => \@io_records);
    if ($IOMan->error()) {
        $Log->comment($IOMan->errmsg());
        return 0;
    }

    $IOMan->wait_finish(io_records => \@io_records);
    if ($IOMan->error()) {
        $Log->comment($IOMan->errmsg());
        return 0;
    }

    my $io_result = $IOMan->get_result(io_records => \@io_records);
    if ($IOMan->error()) {
        $Log->comment($IOMan->errmsg());
        return 0;
    }
    $Log->comment(Dumper($io_result));
    if($io_result->[0]->{error_count} > 0) {
        $Log->comment($io_result->[0]->{error});
        return 0;
    }
    return 1;
} ## end sub run_IO

## Load test cases xml scenarios
## 1. test cases xml file 
###########################
sub load_xml {
    my ($self, %args) = @_;
    ParamsMgr::process_args(%args);
    my @xml_files = ParamsMgr::get_arg(
        -arg      => "xml_files",
        -required => 1,
        -ref_type => "ARRAY"
    );
    my @schema_files = ParamsMgr::get_arg(
                -arg      => "schema_files",
                -required => 1,
                -ref_type => "ARRAY"
            );
    my $display_xml = &ParamsMgr::get_arg(
        -arg     => "display_xml",
        -default => 1
    );
    my $runid = &ParamsMgr::get_arg(
        -arg     => "display_xml_runid",
        -default => "XML-environment"
    );
    if (ParamsMgr::error()) {
        $Log->comment("Argument parsing failed \n" . ParamsMgr::get_error());
        return $TCD::FAIL;
    }
    # validate & load each xml_file into ParamsMgr
    foreach my $xml_file (@xml_files) {
        # Validate the XML file against the provided schema file list
        my $xml_string = &ParamsMgr::validate_and_process_xml(
            -schema_files => \@schema_files,
            -xml_file     => $xml_file,
            -root_element => "scenario",
        );
        if (ParamsMgr::error()) {
            $Log->comment("Failed to validate XML file: $xml_file\n"
                    . ParamsMgr::get_error());
            return $TCD::FAIL;
        }
       # load the xml string that was generated by validate_xml into paramsmgr
        &ParamsMgr::load_xml(
            -xml_string        => $xml_string,
            -display_xml       => $display_xml,
            -display_xml_runid => $runid
        );
        if (ParamsMgr::error()) {
            $Log->comment(
                "Failed to load XML String\n" . ParamsMgr::get_error());
            return $TCD::FAIL;
        }
    }
    return $TCD::PASS;
} ## end sub load_xml

## run_test  parameter
# cli_apiset, testcase name from testcase xml, arguement need for testcse
sub run_test {
    my ($self, $cli_apiset, $op_hash,$fd_serial_no,$os_type) = @_;
    my $pri_vs      = [];
    my $sec_vs      = [];
    my $vs_map      = {};
    my %translation = (
        igroup               => "LunIgroup",
        portset              => "LunPortset",
        lun                  => "Lun",
        volume               => "Volume",
        lif                  => "NetworkInterface",
        paths_of_luns_mapped => "LunMapped",
        iscsi_service        => "VserverIscsi",
        fcp_service          => "VserverFcp",
        accesslist_init      => "VserverIscsiInterfaceAccesslist",
    );
    my @thorough_validation_list
        = ("LunIgroup", "LunPortset", "Volume", "NetworkInterface",
           "VserverIscsi", "VserverFcp", "VserverIscsiInterfaceAccesslist",
          );
    my %filter_translation = (
        igroup               => "igroup",
        portset              => "portset",
        lun                  => "lun",
        volume               => "volume",
        lif                  => "lif",
        paths_of_luns_mapped => "paths",
        accesslist_init      => "initiator-name",
    );
    my $defaults = {
        "Vserver" => [
            'type',              'uuid',
            'allowed_protocols', 'disallowed_protocols',
            'protocol_services_use_data_lifs',
        ],
        "NetworkInterface" => [
            'lif',            'role',
            'address',        'netmask',
            'routing_group',  'auto_revert',
            'failover_group', 'data_protocol',
            'wwpn',           'failover_policy',
            'comment',
        ],
        "LunIgroup" =>
            ['protocol', 'initiator', 'portset', 'ostype', 'alua', 'igroup',],
        "LunPortset" =>
            ['protocol', 'port_name', 'port_count', 'igroups', 'portset',],
        "Lun"       => [],
        "Volume"    => [],
        "LunMapped" => [],
        "VserverIscsi" => ['vserver_uuid', 'target-name'],
        "VserverFcp" => ['vserver_uuid', 'target-name'],
        "VserverIscsiInterfaceAccesslist" => ["lif"],
    };
    my @steps     = keys(%$op_hash);
    my $num_steps = scalar(@steps);
    my $j         = 0;
    foreach my $step (1 .. $num_steps) {
        $Log->comment("Executing step: " . $steps[$j]);
        my $ops_hash = $op_hash->{$steps[$j++]};
        foreach my $key (keys(%$ops_hash)) {
			my $op_cli = $key;
			my %cmd_args;
			foreach my $param (keys(%{$ops_hash->{$key}})) {
				if ($param =~ /foreign/){
					$cmd_args{$param} = $ops_hash->{$key}->{$param} .' '. $fd_serial_no;
				} elsif ($param =~ /ostype/){
					$cmd_args{$param} = $ops_hash->{$key}->{$param} .' '. $os_type;
				} else {
				$cmd_args{$param} = $ops_hash->{$key}->{$param};
				}
			}
			my $response_obj = $cli_apiset->$op_cli(%cmd_args) ;
			my @parsed_output = $response_obj->get_parsed_output();
            $Log->comment("$key executed :"
                        . Dumper $response_obj->get_parsed_output())
                # my @splits = split("_", $key);
                # my $i;
                # my $lib;
                # for ($i = 0; $i < $#splits; $i++) {
                    # $splits[$i] =~ s/^(.)(.*)/uc($1).($2)/e;
                    # $lib .= $splits[$i];
                # }
                # my %cmd_args;
                # foreach my $param (keys(%{$ops_hash->{$key}})) {
                    # $cmd_args{$param} = $ops_hash->{$key}->{$param};
                # }
                # my $op      = $splits[$i];
                # my $cluster = delete $cmd_args{"cluster"};
                # $cmd_args{"command_interface"}
                    # = $cluster eq "A" ? $NodeA : $NodeB;
                # if ($op =~ /^show$/) {
                    # # my $cmd_int = $cluster eq "A" ? $NodeA : $NodeB;
                    # # "NACL::CS::$lib"->fetch(command_interface => $cmd_int,
                                            # # filter => \%cmd_args,
                                            # # allow_empty => 1
                                        # # );
                # } else {
                    # # $cmd_args{"command_interface"}
                        # # = $cluster eq "A" ? $NodeA : $NodeB;
                    # # "NACL::C::$lib"->$op(%cmd_args);
                # }
            }
        }
    return $TCD::PASS;
} ## end sub run_test

sub generate_ops_hash {
    my ($self,$input_ref) = @_;
    my $scenario_hash_ref = &ParamsMgr::search(-key => "TESTCASES");
    foreach my $tc (keys(%$scenario_hash_ref)) {
        my $tc_name = $scenario_hash_ref->{$tc}->{'NAME'};
        $scenario_hash_ref->{$tc_name} = delete($scenario_hash_ref->{$tc});
        delete($scenario_hash_ref->{$tc_name}->{'NAME'});
        foreach my $op (keys(%{$scenario_hash_ref->{$tc_name}})) {
            my $step = delete $scenario_hash_ref->{$tc_name}->{$op}->{'STEP'};
            $step =~ s/_//g;
            my $op_chop = lc $op;
            $op_chop =~ s/\d+//;
            $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                = delete $scenario_hash_ref->{$tc_name}->{$op};
            foreach my $param (
                keys(%{$scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}}))
            {
                $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{lc $param}
                    = delete $scenario_hash_ref->{$tc_name}->{$step}
                    ->{$op_chop}->{$param};
                delete $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{''}
                    if (
                    exists(
                        $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                            ->{''}
                    )
                    );
                delete $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{'_index'}
                    if (
                    exists(
                        $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                            ->{'_index'}
                    )
                    );
            }
            if(exists $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                ->{'pref_cmd_int'}){
                my $iface = delete $scenario_hash_ref->{$tc_name}->{$step}
                                   ->{$op_chop}->{'pref_cmd_int'};
                $scenario_hash_ref->{$tc_name}->{$step}
                    ->{$op_chop}->{'apiset_should'}->{'interface'} = $iface;
            }
            if ($op_chop =~ /network_interface/
                && exists $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                ->{'data-protocol'})
            {
                $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{'data-protocol'} = [
                    split(
                        ",",
                        $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                            ->{'data-protocol'}
                    )
                    ];
            } elsif ($op_chop =~ /igroup/
                && exists $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                ->{'initiator'})
            {
                $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{'initiator'} =~ s/_/-/g;
                $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{'initiator'} = [
                    split(
                        ",",
                        $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                            ->{'initiator'}
                    )
                    ];
            } elsif ($op_chop =~ /portset/
                && exists $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                ->{'port-name'})
            {
                $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{'port-name'} = [
                    split(
                        ",",
                        $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                            ->{'port-name'}
                    )
                    ];
            } elsif ($op_chop =~ /vserver/
                && exists $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                ->{'ns-switch'})
            {
                $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                    ->{'ns-switch'} = [
                    split(
                        ",",
                        $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                            ->{'ns-switch'}
                    )
                    ];
            } elsif ($op_chop =~ /accesslist/
                && exists $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                ->{'lif'})
            {
                $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                ->{'lif'} = [
                    split(
                        ",",
                        $scenario_hash_ref->{$tc_name}->{$step}->{$op_chop}
                            ->{'lif'}
                    )
                    ];
            }
        }
    }
    $$input_ref = $scenario_hash_ref;
    return $TCD::PASS;
} ## end sub generate_ops_hash

###############################################################################
########## function to find the max or last lba of lun  #######################
###############################################################################
# Input :
#       0. Device path
# Return :
#        Max number of blocks of lun in decimal.
# usage :
#       To find the max or last lba of lun
################################################################################
sub get_last_lba {
	my ($self, %args) = @_;
	my %cmd_construct;
    foreach my $param (keys %args) {
        if ($param =~ /^-?lun_path$/) {
            unless ($cmd_construct{'-device'} = $args{$param}) {
				$Log->comment("device path should be passed to get last lba");
                return 0;
            }
        } 
	}
    
	my $Scsi_Tool_Obj = $self->{'Scsi_Tool_Obj'};
    $cmd_construct{'-cdb'}        = "9e 10 00 00 00 00 00 00 00 00 00 00 02 00 00 00";
    $cmd_construct{'-repeat_cnt'} = 1;
    $cmd_construct{'-dir'}        = ScsiTool::READ;
    $cmd_construct{'-buf_len'}    = 1024;
	
    $Log->comment(
               "device under test for get_last_lba " .$cmd_construct{'-device'});
	my $max_lba_dec=0;
    # Send the command to test
    my $ret_ref = $Scsi_Tool_Obj->send_cmd(%cmd_construct);
    if ($Scsi_Tool_Obj->error) {
        $Log->comment("scsi command failed" . $Scsi_Tool_Obj->errmsg());
        return $max_lba_dec;
    }

    # Check the status of the command
    if (!( $Scsi_Tool_Obj->check_status(-send_cmd_output => $ret_ref,
                                        -scsi_status     => "GOOD"))
      ) {
        $Log->comment("scsi command didnt return correct status");
        return $max_lba_dec;
    }

    # Check the data length
    my $cmd_array   = $ret_ref->{SCSI_DATA};
    my $data_length = scalar @$cmd_array;
    if ($data_length < 1) {
        $Log->comment("Insufficient data to be processed -:" . $data_length);
        return $max_lba_dec;
    }
    # max_lba of the lun
    my $max_lba = join '', splice @$cmd_array, 0, 8;
    $max_lba_dec = hex $max_lba;

    unless (length $max_lba_dec and defined $max_lba_dec) {
        $Log->comment("Fail:unable to get maximum LBA of the LUN");
        return $max_lba_dec;
    }

    return $max_lba_dec;
} ## end sub _get_last_lba

###############################################################################
########## function to execute INQ PAGE 83h on device   #######################
###############################################################################
# Input :
#       0. Device path
# Return :
#        Serial Number found on INQ Page # 80h.
# usage :
#       To find the max or last lba of lun
################################################################################
sub get_inq_83_data_serial_no {
	my ($self, %args) = @_;
	my $test_device;
	my $Scsi_Tool_Obj = $self->{'Scsi_Tool_Obj'};
	my $Scsi_Util_Obj = $self->{'Scsi_Util_Obj'};
    foreach my $param (keys %args) {
        if ($param =~ /^-?lun_path$/) {
            unless ($test_device = $args{$param}) {
				$Log->comment("device path should be passed to get last lba");
                return 0;
            }
        } 
	}
	my $ret_val =
      $Scsi_Tool_Obj->test_scsi_cmd(-device      => $test_device,
                                    -cdb_name    => "inquiry_page_83h",
                                    -scsi_status => "GOOD",);
	
	if ($Scsi_Tool_Obj->error){
		$Log->comment('INQ 83h failed on device: '.$test_device );
		return 0;
	}
	
	my $scsi_data = $Scsi_Util_Obj->xlate_inq_pg_83h_bytes($ret_val->{SCSI_DATA});
	## Identifier type 3 and association 0 descriptor has Addressed logical unit number
	## this is used as serial number for disks
	my $lun_identifier;
	#$Log->comment('xlated data : ' . Dumper $scsi_data);
	for (my $i=0;$i<@{$scsi_data->{'ID_LIST'}};$i++){
		if ((defined $scsi_data->{'ID_LIST'}->[$i]->{'IDENTIFIER_TYPE'}) 
			and 
			$scsi_data->{'ID_LIST'}->[$i]->{'IDENTIFIER_TYPE'} == 3 
			and 
			$scsi_data->{'ID_LIST'}->[$i]->{'ASSOCIATION'} == 0 ){
				my $temp = unpack ("H*",$scsi_data->{'ID_LIST'}->[$i]->{'LUN_SERIAL_NUMBER_0_11'});
				$lun_identifier = $scsi_data->{'ID_LIST'}->[$i]->{'NETAPP_MAGIC_NUMBER'}.$temp;
		}
	}

	return $lun_identifier;
}

###############################################################################
# send raw spt build command - takes spt.last from the tools location
# Input parameter
# 0. spt_cmd
# 1. scsi_response or Pass-1|fail-0
# return  Pass|Fail or 
# Dump return  $VAR1 = {
# 'SENSE_KEY' => '5',
# 'HOST_STATUS' => '0',
# 'IOCTL_STATUS' => '-1',
# 'ASCNQ' => '26h/00h',
# 'DRIVER_STATUS' => '0',
# 'SCSI_STATUS' => '2',
# 'SCSI_DATA' => undef, ----> [1,2,3,4,5,6,7,8,9,10]
# };
############################################
sub send_spt_cmd {
	my $resp='0';my ($self,$spt_param);
	($self,$spt_param,$resp) = @_;
	my @scsi_data;
	my $client_obj  = $self->{'Client_Obj'};
    my $client_conn = $client_obj->get_connection();
	my $spt_loc;
    if($client_obj->hosttype() =~ /unix|vmware|linux/i) {
        $spt_loc = $client_obj->{TEST_BIN}."/spt.last" ;
    }
    elsif($client_obj->hosttype() =~ /windows/i) {
        $spt_loc = $client_obj->{TEST_BIN}. "spt.last";
    } 
	my %build_response;my ($spt_emit,$data_found,);
	my $emit = '"ioctl_status:%status,scsi_status:%scsi_status,host_status:%host_status,sense_code:%sense_code,bytes_transfered:%xfer,sense_key:%sense_key,asc:%asc,ascq:%ascq,ascnq:%asc/%ascq,resid:%resid"';
	my $spt_cmd = $spt_loc.' '.$spt_param .' emit='."$emit"; 
    my ($status, @output) = &ClientUtil::execute($client_conn, $spt_cmd);
	if ($status > 0) {
        $Log->comment( "Failure::send_spt_cmd(): " .
                              "Command ($spt_cmd) returned a bad status ($status).");
    }
	# $emit =~ s/"//g;
	# my @emit_str1 = split (/,/ ,$emit);
	# foreach (@emit_str1){
		# my ($k,$v) = split (/:/,$_);
		# $build_response{$k} = $v; 
	# }
	foreach my $ln (@output) {
        # take out any leading or trailing white space
        $ln =~ s/^\s*|\s*$//g;
        if($ln =~ /ioctl_status/i) {
            $data_found = 0;
            $ln =~ s/^(>\s)*//;
            $spt_emit = $ln;
        } elsif ( $ln =~ /Off(set)?\s+Data/ && !$data_found ) {
            # handle data here.....
            $data_found = 1;
            next;
        }
        if($data_found) {
            my ($hex, $bin) = split /"/, $ln;
            my @ln_data = split /\s+/, $hex;
            # the very first value will be the offset val
            # so take that off
            shift @ln_data;
            push @scsi_data, @ln_data;
        }
    }
	$build_response{SCSI_DATA} = \@scsi_data;
	
	my @emit_tokens = split /,/, $spt_emit;
    my ($str, $val);

    foreach my $ln (@emit_tokens) {
        ($str, $val) = split /:/, $ln;
		$build_response{$str} = $val;
    }
	if ($resp){
		($build_response{'ioctl_status'} == 0)? return 1:return 0;
	}else{
		return \%build_response;
	}
}

1;