# Copyright (c) 2008-2011 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of methods for the Common Interop Testplan thpls
## @author hrivera@netapp.com
## @status private

use strict;

# Package Name
##############
package SanCIT;

# Package Requirements
######################
use Tharn;
use San::SanUtil;
use San::SanFiler;
use San::SanClient::ClientUtil;
use TCD;
use ndisp::API;
use NATE::IPC::Socket;
use Data::Dumper;
use NACL::MTask::Group::SFOPair::CMode;

# Exceptions
use NATE::BaseException qw(:try);
use NATE::Exceptions::Argument qw(:try);
use NATE::ParamSet;
use NATE;

# Hardcoded parameters
use vars qw (
             $ENABLE_PER_CLIENT_PARAMS
             $RUNLENGTH
             $IO_ABORT_ON_ERROR
             $VALIDATE_ENVIRONMENT_IGNORE_TAKEOVER_OPTION
             $CF_TAKEOVER_DETECTION_TIMEOUT_OPTION
             $VALIDATE_ENVIRONMENT_ABORT_ON_ERROR
             $IO_DUMP_FILER_CONFIG_ON_ERROR
             $COLLECT_PERFSTATS
             $SLA_MEASUREMENT
             $FILER_SRAM_DUMP_CHECK
             $MULTIPATH
             $NTP_SYNC
             $FILER_CORE_REMOVAL
             $FILER_CONNECTION_TYPE
             $FILER_VOL_DIRTY_SNAP_TYPE
             $FILER_VOL_DIRTY_SNAP
             $FILER_VOL_DIRTY_SNAP_COUNT
             $FILER_VOL_DIRTY_SNAP_MAX_THREADS
             $FILER_CHECK_CORE
             $COLLECT_FILER_STATS
             $TUNE_IO_RESOURCE
             $TUNE_IO_CPU_MAX
             $TUNE_IO_CPU_MIN
             $TUNE_IO_DISK_MIN
             $TUNE_IO_SKIP_TUNING
             $IO_DT_SKIP_NO_PROG
             $IO_DT_OPTION_PATTERN
             $CLIENT_PATH_VALIDATION
             $CLIENT_PATH_VALIDATION_ON_TAKEOVER
             $IO_THRESHOLD_ABORT_ON_FAILURE
             $IO_THRESHOLD_SET_NONE
             $IO_THRESHOLD_SET_NO_THRESHOLD_LMT
             $IO_THRESHOLD_CHECK_KB
             $IO_THRESHOLD_CHECK_MAX_THRESHOLD_LMT
             $FAULTS
             $FAULT_CYCLE_COUNT
             $FAULT_DOWN_TIMEOUT
             $FILER_DOWN_TIMEOUT
             $FAULT_SERVICE_CYCLE_TIMEOUT
             $FAULT_DOWN_TIMEOUT_BETWEEN_PORTS
             $FAULT_SWITCH_PORT_SERIAL
             $FAULTS_ORDER_TYPE
             $FAULTS_ORDER_RANDOM_SEED
             $FAULT_SWITCH_ABORT_ON_ERROR
             $FAULT_SWITCH_ABORT_ON_CONFIG
             $FILER_ENABLE_DUMMY_VOLS
             $FILER_ENABLE_DUMMY_VOLS_FOR_NDU
             $FILER_DUMMY_VOL_MIN_SIZE
             $FILER_DUMMY_VOL_USE_MIN_VALUES
             $FILER_DUMMY_LUN_SIZE
             $FILER_VOL_NAME_UNIQUE
             $AGGR_MIN_CAPACITY
             $AGGR_MAX_CAPACITY
             $VM_CONFIG_REBOOT_LIST
             $VM_VMFS_TYPE
             $VM_VMFS_BLOCKSIZE
             $VM_RDM_TYPE
             $USE_EXTENDED_LUN_TYPES
             $FILER_ENABLE_PORTSETS_ACCESSLISTS
             $FILER_ENABLE_THIN_PROVISIONING
             $FAULT_INITIATOR_PORT_DOWN_TIME
             $FAULT_TARGET_PORT_DOWN_TIME
             $SKIP_CLEANUP
             $SKIP_CLIENT_CLEANUP
             $VM_SKIP_ESX_CLEANUP
             $SKIP_FILER_CLEANUP
             $CITRIXXS_SKIP_HOST_CLEANUP
             $FILER_LUN_MAX_COUNT
             $CONFIG_NFS_ALLOW_MGMT_PORT
             $NDVM_AGGRLIST
             $CIT_NDVM_VOL_NAMES
             $SLA_FOBENCH_GIVEBACK_TYPE
             $SLA_FOBENCH_TAKEOVER_TYPE
             $SLA_FOBENCH_CYCLES
             $SLA_FOBENCH_ENABLE_DEDUP
             $SLA_FOBENCH_ACTIVE_DEDUP_OPS
             $SLA_FOBENCH_ENABLE_COMPRESSION
             $SLA_FOBENCH_ACTIVE_COMPRESSION_OPS
             $SLA_FOBENCH_INLINE_COMPRESSION
             $SLA_FOBENCH_PARSE_PATH
             $SLA_FOBENCH_PANIC_THRESHOLD_MIN
             $SLA_FOBENCH_IO_DELAY
         );

# Mandatory (User configurable) parameters
use vars qw (
             $ALUA_SUPPORT
             $PROTOCOL
             $AGGR
             $CLIENT_VMS
             $FILERS
         );

# Discretionary parameters
use vars qw (
             $CFO_DETECTION
             $CLIENTS
             $NFS_CLIENTS
             $ETHERNET_SWITCHES
             $CIT_FCOE
             $VM_CLIENTS
             $KVM_CLIENTS
             $OVM_CLIENTS
             $CITRIXXS_CLIENTS
             $APCON_SWITCHES
             $FILER_VOL_LIST
             $CONFIG_ISCSI_TYPE
             $CONFIG_ISCSI_HARDWARE_VENDOR
             $CONFIG_ISCSI_IPV6_EXCLUDE_LINK_LOCAL
             $CONFIG_FCP_DRIVER_TYPE
             $CONFIG_FCP_AIX_ERROR_RECOVERY
             $CONFIG_FCP_AIX_NUM_CMD_ELEMS
             $CONFIG_FCP_AIX_DYNAMIC_TRACKING
             $FILER_PACKET_TRACE
             $FILER_PACKET_TRACE_PASS_THRU
             $FILER_SRAM_DUMP_CHECK_ALLOW_PANIC
             $DEBUG_OUTPUT
             $IO_PANIC_FILERS_ON_ERROR
             $RASTRACE_DUMP_ON_IO_ERROR
             $IO_ABORT_WAIT_TIME
             $CONFIG_ISCSI_DISCOVERY
             $NTP_TIME_SERVER
             $NTP_TIME_ZONE
             $FILER_VOL_DIRTY_SNAP_MAX_THREADS
             $FILER_VOL_SNAP_MAX_COUNT
             $FILER_VOL_MAX_COUNT
             $FILER_VOL_PER_CLIENT
             $FILER_ISCSI_ENABLED_INTERFACE_LIST
             $FILER_ISCSI_DISABLE_OTHER_INTERFACES
             $FILER_LUN_ENABLE_SPACE_ALLOC
             $CLIENT_MOUNT_OPTIONS_DISCARD
             $CLIENT_DATA_COLLECTION
             $CLIENT_DATA_COLLECTION_CMDS
             $CLIENT_POST_DATA_COLLECTION
             $CLIENT_POST_DATA_COLLECTION_CMDS
             $CLIENT_IO_BLOCK_SIZES
             $CLIENT_IO_FILE_SIZES
             $CLIENT_IO_THREADS
             $CLIENT_IO_TYPE
             $CLIENT_IO_ROTATE_USE_OF_BUFFER_CACHE
             $CLIENT_OFFLINE_DISCARD
             $CLIENT_OFFLINE_DISCARD_ABORT_ON_ERROR
             $CLIENT_VM_LAYOUT
             $CLIENT_VM_NUM_LVS
             $CLIENT_VM_VOL_SIZE
             $CLIENT_VM_STRIPE_SIZE
             $CLIENT_USE_DEVICE_ENCYPTION
             $CLIENT_DEVICE_ENCRYPTION_METHOD
             $CFO_DETECTION_TIMEOUT
             $PRE_CLEANUP
             $SKIP_SETUP
             $SKIP_IOF
             $SKIP_IO
             $SKIP_FILER_PRE_CLEANUP
             $SKIP_CLIENT_PRE_CLEANUP
             $SKIP_ESX_PRE_CLEANUP
             $SKIP_FILER_SETUP
             $VM_SKIP_FILER_SETUP
             $VM_SKIP_ESX_SETUP
             $SKIP_CLIENT_SETUP
             $CLIENT_DEV_LIST
             $CLIENT_IO_PROGRAM
             $CLIENT_IO_FS_TYPE
             $CLIENT_IO_PASS_THRU
             $TUNE_IO_MEASURE_DELAY
             $TUNE_IO_DELAY
             $TUNE_IO_STEP
             $TUNE_IO_ATTEMPTS
             $TUNE_IO_MEASURE_DELAY
             $TUNE_IO_DISK_MAX
             $TUNE_IO_DELAY_PERCENT
             $IO_DT_OPTION_TRIGGER_SCRIPT
             $IO_DT_OPTION_IOTYPE
             $IO_DT_OPTION_ALARM
             $IO_SIO_OPTION_IOTYPE
             $CLIENT_PATH_VALIDATION_INIT_STATE
             $CLIENT_PATH_VALIDATION_ISCSI_INIT_STATE
             $CLIENT_PATH_VALIDATION_FCP_INIT_STATE
             $CLIENT_PATH_VALIDATION_ABORT_SCRIPT
             $CLIENT_PATH_VALIDATION_TIMEOUT
             $CLIENT_PATH_VALIDATION_ABORT_ON_ERROR
             $CLIENT_PATH_VALIDATION_MIN_WAIT_TIME
             $CLIENT_PATH_VALIDATION_VERIFY_IO_SWITCHING
             $CLIENT_PATH_VALIDATION_VERIFY_IO_SWITCHING_ABORT_ON_ERROR
             $CLIENT_PATH_VALIDATION_PER_PORT
             $PATH_VALIDATION_BETWEEN_PORT_DISABLE_ENABLE
             $CLIENT_LINUX_ENABLE_SYSRQ_DUMP
             $CLIENT_LOSS_DETECTION
             $CLIENT_LOSS_DETECTION_TIMEOUT
             $CLIENT_ORPHANED_DEVICE_CLEANUP
             $CLIENT_VALIDATION
             $FAULT_SWITCH_PORT_FAILURE_SCRIPT
             $FAULT_SWITCH_PORT_RETRY_COUNT
             $FAULT_SWITCH_VALIDATE_OFFLINE_ON_DISABLE
             $FAULT_CYCLE_TIMEOUT
             $FAULT_START_DELAY
             $FAULT_CFO_RETRIES
             $FAULT_SWITCH_PORT_FCOE_ETHERNET_BLOCK
             $VALIDATE_ENVIRONMENT_RESTORE_TAKEOVER_OPTION
             $VALIDATE_AGGR_MIN_DISK_COUNT
             $ALLOW_THARNHOST
             $VM_CONTROLLER
             $VM_CONFIG_FOR_VMOTION
             $VM_SHUTDOWN_GUESTS_AFTER_CLEANUP
             $VM_REBOOT_ESX_AFTER_CLEANUP
             $VM_VMFS_FVL
             $VM_RDM_FVL
             $VM_DATASTORE_TYPE
             $VM_NUM_VMFS_VMDKS
             $VM_DONT_USE_CONFIG_MPATH
             $VM_VXDMP_IO_POLICY
             $FILER_PORTSET_EXCLUSIONS
             $FILER_ACCESSLIST_EXCLUSIONS
             $FILER_THIN_PROVISIONING_VOL_PERCENT
             $FILER_THIN_PROVISIONING_VOL_INCREMENT
             $USE_EXTENDED_LUN_TYPES
             $IO_DT_USE_PREFIX
             $IO_DT_OPTION_NOPROGTT
             $IO_DT_OPTION_NOPROGT
             $IO_DT_SOLARIS_VERSION_OVERRIDE
             $IO_DT_LINUX_VERSION_OVERRIDE
             $IO_DT_AIX_VERSION_OVERRIDE
             $IO_DT_HPUX_VERSION_OVERRIDE
             $IO_DT_WINDOWS_VERSION_OVERRIDE
             $IO_THRESHOLD_CHECKING
             $FILER_ENABLE_ASIS
             $FILER_ENABLE_COMPRESSION
             $FILER_ENABLE_INLINE_COMPRESSION
             $FILER_COMPRESSION_DELETE_CHECKPOINT
             $FILER_COMPRESSION_INCLUDE_SHARED_BLOCKS
             $FILER_COMPRESSION_INCLUDE_SNAPSHOT_BLOCKS
             $FILER_ASIS_SCHEDULE
             $FILER_USE_LUN_CLONE
             $FILER_USE_SIS_CLONE
             $FILER_USE_LUN_CLONE_SPLIT
             $FAULT_SWITCH_MISCOMPARE_ABORT_ON_ERROR
             $HYPERVISOR_TYPE
             $HV_CLIENTS
             $HV_VHD_FVL
             $HV_PT_FVL
             $HV_VHD_TYPE
             $HV_VHD_CONTROLLER_TYPE
             $HV_PT_CONTROLLER_TYPE
             $HV_NUM_VHD_DISKS_PER_LUN
             $HV_SKIP_FILER_SETUP
             $HV_SKIP_HYPERV_SETUP
             $HV_SKIP_HYPERV_CLEANUP
             $SKIP_HYPERV_PRE_CLEANUP
             $CITRIXXS_SKIP_FILER_SETUP
             $CITRIXXS_SKIP_HOST_SETUP
             $CITRIXXS_NUM_VDI_PER_LUN
             $CITRIXXS_VDI_FVL
             $SKIP_CITRIXXS_PRE_CLEANUP
             $TUNE_IO_NUM_MEASUREMENTS
             $FAULT_IGRP_THROTTLE_RESERVE
             $FAULT_IGRP_THROTTLE_PERIOD_FCP
             $FAULT_IGRP_THROTTLE_PERIOD_ISCSI
             $FAULT_VALIDATE_RUNLENGTH
             $FAULT_VALIDATE_CYCLE_TIME
             $FILER_KEEP_DUMMY_VOLS
             $FILER_DUMMY_ENABLE_ASIS
             $FILER_DUMMY_ENABLE_COMPRESSION
             $FILER_CNA_STATUS_CHECK
             $FILER_CNA_STATUS_CHECK_ABORT_ON_ERROR
             $VALIDATE_ENVIRONMENT_AGGR_CHECK_ABORT_ON_ERROR
             $USE_QA_ASUP_SERVER
             $SWITCHES
             $CIT_USE_WWPN_BLOCK_FAULT_TYPE_FOR_INITIATORS
             $CIT_USE_WWPN_BLOCK_FAULT_TYPE_FOR_TARGETS
             $CONFIG_NFS_SUBNET_MATCHING
             $CONFIG_NFS_IPVERSION
             $FILER_ROTATE_SYSLOGS
             $CLIENT_ROTATE_SYSLOGS
             $USE_SINGLE_CONTROLLER
             $CONFIG_REBOOT_CLIENTS
             $FILER_IGROUP_THROTTLE_RESERVE
             $FILER_IGROUP_THROTTLE
             $NDVM_AGGRLIST
             $CIT_NDVM_VOL_NAMES
             $SWITCH_ADMIN_DOMAIN
             $VALIDATE_ABORT_ON_ERROR
             $VM_VMA_HOST
             $SLA_FOBENCH_TITLE
             $SLA_FOBENCH_TYPE
             $SLA_FOBENCH_PREFIX
             $SLA_FOBENCH_IO_TYPE
             $SLA_FOBENCH_CLIENT_OS
             $SLA_FOBENCH_DESTINATION
             $SLA_FOBENCH_ONTAP_VERSION
             $CONFIG_DMMP_USE_USER_FRIENDLY_NAMES
             $CONFIG_DMMP_MAX_FDS
             $CONFIG_DMMP_QUEUE_WITHOUT_DAEMON
             $CONFIG_DMMP_FLUSH_ON_LAST_DEL
             $CONFIG_DMMP_PG_PRIO_CALC
             $CONFIG_DMMP_PATH_GROUPING_POLICY
             $CONFIG_DMMP_FEATURES
             $CONFIG_DMMP_PATH_CHECKER
             $CONFIG_DMMP_RR_WEIGHT
             $CONFIG_DMMP_RR_MIN_IO
             $CONFIG_DMMP_HARDWARE_HANDLER
             $CONFIG_DMMP_PRIO_CALLOUT
             $CONFIG_DMMP_PRIO
             $CONFIG_DMMP_FAILBACK
             $CONFIG_DMMP_GETUID_CALLOUT
             $CONFIG_DMMP_PATH_SELECTOR
             $CONFIG_DMMP_PRODUCT
             $VM_ADD_REMOVE_VMDK_STYLE
             $VM_ADD_REMOVE_VMDK_PARALLEL_LIMIT
             $VM_DELETE_NON_EMPTY_DATASTORES
             $FAULT_ESX_VXDMP_ROTATE_IO_POLICIES
             $FAULT_ESX_VXDMP_ROTATE_IO_POLICIES_ABORT_ON_ERROR
             $FAULT_ESX_VXDMP_ROTATE_IO_POLICY_STYLE
             $FAULT_ESX_VXDMP_IO_POLICY_LIST
             $FAULT_PORT_BLOCK_CYCLE_COUNT
         );

# Special parameters
use vars qw (
             $NDISPD_HOST
             $NDISPD_PORT
             $CIT_DONT_STORE_NDISP_DATA
             $CIT_REUSE_PREVIOUS_CONFIG
             $CIT_POLICY_HOST_TEAM
             $CIT_ONTAP_VERSION_OVERRIDE
             $CIT_NOT_FOR_SCORE
             $CIT_DIRECT_CONNECT
             $RRE
         );

# Global Vars
use vars qw (%no_tharnhost_hsh $cdl_displayed);

# Special load of not for score and RRE flags. Needs to be loaded
# on load of module to make decisions inside all methods
&param("CIT_NOT_FOR_SCORE", '-default', 0);
&param("RRE", '-default', 0);

# Exported Global Vars
$SanCIT::VM_IO_EXIT_STATUS = "fail";
$SanCIT::CLIENT_PARAM_HASH = {};

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: load_ha_list()
# Objective:       Load the HA_LIST parameter.
################################################################################
sub load_ha_list {
    my (%args) = @_;

    # Get user environment setting
    &discretionary_param("CIT_PARALLEL_HA_EVENTS",                  1);
    &discretionary_param("CIT_NUM_PARALLEL_HA_GROUPS_PER_CLUSTER", -1);
    &discretionary_param("CIT_PARALLEL_HA_EVENTS_DELAY_INTERVAL", 0);

    my $parallel_ha_events = &get_param("CIT_PARALLEL_HA_EVENTS");
    my $num_parallel_ha_groups_per_cluster = &get_param("CIT_NUM_PARALLEL_HA_GROUPS_PER_CLUSTER");
    my $parallel_interval = &get_param("CIT_PARALLEL_HA_EVENTS_DELAY_INTERVAL");
    my $ha_groups = "";
    my $ha_eventsets = "";
    my $ha_clusters = "";
    my $parallel_delays = "";
    my @nacl_nodes;
    # Get all the Nodes that are defined.
    @nacl_nodes = NACL::C::Node->find();

    my @nacl_cluster_objs_array;
    # Get all the unique cluster objects from the Nodes that are defined.
    @nacl_cluster_objs_array =
        NACL::STask::Cluster->find_unique_clusters(nodes => \@nacl_nodes);

    foreach my $cluster (@nacl_cluster_objs_array) {
        my @cluster_node_objs = $cluster->node_objs();
        # if user specified not to do parallel ha events OR
        #    user specified parallel groups per cluster is 0 or 1 OR
        #    user did not specify parallel groups per cluster and
        #        nodes in cluster less than 8
        # then create a serial list of nodes for this cluster
        if( $parallel_ha_events == 0 or
            $num_parallel_ha_groups_per_cluster == 0 or
            $num_parallel_ha_groups_per_cluster == 1 or
            ($num_parallel_ha_groups_per_cluster == -1 and
            scalar(@cluster_node_objs) < 8)) {
            my $nodes = join("','",$cluster->nodes());
            $ha_eventsets .= $cluster->name() . "_group:";
            $ha_clusters .= "['" . $cluster->name() . "']:";
            $ha_groups .= "['$nodes']:";
			$parallel_delays .= '0';
        } else {
            my @ha_lists = ();
            my $temp_num_parallel_ha_groups_per_cluster = $num_parallel_ha_groups_per_cluster;
            if ($temp_num_parallel_ha_groups_per_cluster == -1) {
                # Get how many parallel groups by putting 4 nodes per group.
                $temp_num_parallel_ha_groups_per_cluster = scalar(@cluster_node_objs)/4;
                # If there are leftovers then need to add another group for them.
                if (scalar(@cluster_node_objs)%4) {
                    $temp_num_parallel_ha_groups_per_cluster++;
                }
            }
            my @sfo_pairs = ();
            try {
                @sfo_pairs = NACL::MTask::Group::SFOPair::CMode->find(nodes => \@cluster_node_objs);
            } catch NATE::BaseException with {
                my $exception = shift;
                &SanUtil::nate_log('-comment',
                    "You need to define SFOPAIR in all nate_hosts files for all nodes in the testbed\n".
                    "SFOPAIR is defined by taking the two System IDs from the following\n\n".
                    "tb3170_05::> run -node your_node1 sysconfig\n".
                    "NetApp Release RsierranevadaN_130226_2330 Cluster-Mode: Tue Feb 26 23:42:55 PST 2013\n".
                    "System ID: 0987654321 (your-node1); partner ID: 1234567890 (your-node2)\n\n".
                    "seperated by a -\n".
                    "Here is an example SFOPAIR=0987654321-1234567890\n".
                    "The SFOPAIR parameter must be defined exactly the same for ".
                    "both nodes in each HA pair.");
                $exception->throw;
            };
            my $i = 0;
            foreach my $sfo_pair (@sfo_pairs) {
                push(@{$ha_lists[$i]},$sfo_pair->nodes());
                # Update the array index to point at the next place to add.
                # by doing i mod num parallel groups should wrap around when
                # we have the right number of groups in the array.
                $i = ($i+1)%$temp_num_parallel_ha_groups_per_cluster;
            }
            $i = 1;
            foreach my $list (@ha_lists) {
                $ha_groups .= "['" . join("','",@$list) . "']:";
                $ha_eventsets .= $cluster->name() . "_group$i:";
                $ha_clusters .= "['" . $cluster->name() . "']:";
                # Do i-1 so that we start at 0 delay
                $parallel_delays .= ($parallel_interval * ($i-1)) . ":";
                $i++;
            }
        }
    }

    # Remove the : that we put in at the end to seperate clusters
    $ha_groups =~ s/:$//;
    $ha_eventsets =~ s/:$//;
    $ha_clusters =~ s/:$//;
    $parallel_delays =~ s/:$//;
    # Convert all - to _ since - is not supported in the regex for
    # eventset name.
    $ha_eventsets =~ s/-/_/g;
    # Set HA_GROUPS and HA_EVENTSETS if not already set.
    &discretionary_param("CIT_HA_GROUPS",    $ha_groups);
    &discretionary_param("CIT_HA_EVENTSETS", $ha_eventsets);
    &discretionary_param("CIT_HA_CLUSTERS", $ha_clusters);
    &discretionary_param("CIT_PARALLEL_HA_EVENTS_DELAYS", $parallel_delays);

    return {nodes => \@nacl_nodes, clusters => \@nacl_cluster_objs_array};
}

################################################################################
# Subroutine Name: load_hardcode_params()
# Objective:       Load all hardcoded params with their hardcoded values.
################################################################################
sub load_hardcode_params {
    # Set hardcoded param overrides

    # General control settings
    &hardcode_param("ENABLE_PER_CLIENT_PARAMS",                    1);
    &hardcode_param("RUNLENGTH",                                   "iterations");
    &hardcode_param("IO_ABORT_ON_ERROR",                           1);
    &hardcode_param("VALIDATE_ENVIRONMENT_IGNORE_TAKEOVER_OPTION", 0);
    &hardcode_param("CF_TAKEOVER_DETECTION_TIMEOUT_OPTION",        "");
    &hardcode_param("VALIDATE_ENVIRONMENT_ABORT_ON_ERROR",         1);

    # Debug assistance
    &hardcode_param("IO_DUMP_FILER_CONFIG_ON_ERROR",        0);
    &hardcode_param("COLLECT_PERFSTATS",                    0);
    &hardcode_param("SLA_MEASUREMENT",                      0);
    &hardcode_param("FILER_SRAM_DUMP_CHECK",                1);

    # Filer/Host Configuration Settings
    &hardcode_param("NTP_SYNC",                             1);
    &hardcode_param("FILER_CORE_REMOVAL",                   1);
    &hardcode_param("FILER_CONNECTION_TYPE",                "#rsh");
    &hardcode_param("FILER_VOL_DIRTY_SNAP_TYPE",            "clean");
    &hardcode_param("FILER_VOL_DIRTY_SNAP",                 1);
    &hardcode_param("FILER_VOL_DIRTY_SNAP_COUNT",           2);
    &hardcode_param("FILER_VOL_NAME_UNIQUE",                1);
    &hardcode_param("FILER_CHECK_CORE",                     1);
    &hardcode_param("FILER_ENABLE_DUMMY_VOLS",              1);
    &hardcode_param("FILER_ENABLE_DUMMY_VOLS_FOR_NDU",      0);
    &hardcode_param("FILER_DUMMY_VOL_MIN_SIZE",             "200m");
    &hardcode_param("FILER_DUMMY_LUN_SIZE",                 "4m");
    &hardcode_param("FILER_DUMMY_VOL_USE_MIN_VALUES",       1);
    &hardcode_param("FILER_LUN_MAX_COUNT",                  "");
    &hardcode_param("AGGR_MIN_CAPACITY",                    "90%");
    &hardcode_param("AGGR_MAX_CAPACITY",                    "99%");
    &hardcode_param("FILER_ENABLE_THIN_PROVISIONING",       "1");
    &hardcode_param("FILER_ENABLE_PORTSETS_ACCESSLISTS",    "1");
    &hardcode_param("CONFIG_NFS_ALLOW_MGMT_PORT",           "1");
    &hardcode_param("CONFIG_ISCSI_USE_MGMT_PORT",           "0");

    # Data Collection
    &hardcode_param("COLLECT_FILER_STATS",                   1);

    # I/O Tuning
    &hardcode_param("TUNE_IO_RESOURCE",                      "disk");
    &hardcode_param("TUNE_IO_CPU_MIN",                       0);
    &hardcode_param("TUNE_IO_CPU_MAX",                       100);
    &hardcode_param("TUNE_IO_DISK_MIN",                      45);
    &hardcode_param("TUNE_IO_SKIP_TUNING",                   0);

    # DT Settings
    &hardcode_param("IO_DT_SKIP_NO_PROG",                    0);
    &hardcode_param("IO_DT_OPTION_PATTERN",                  "iot");

    # Path and I/O Validation
    &hardcode_param("CLIENT_PATH_VALIDATION",                1);
    &hardcode_param("CLIENT_PATH_VALIDATION_ON_TAKEOVER",    1);
    &hardcode_param("IO_THRESHOLD_ABORT_ON_FAILURE",         0);
    &hardcode_param("IO_THRESHOLD_SET_NONE",                 1);
    &hardcode_param("IO_THRESHOLD_SET_NO_THRESHOLD_LMT",     1);
    &hardcode_param("IO_THRESHOLD_CHECK_KB",                 "1000");
    &hardcode_param("IO_THRESHOLD_CHECK_MAX_THRESHOLD_LMT",  3);

    # Fault Injection
    &hardcode_param("FAULTS",                                "");
    &hardcode_param("FAULT_CYCLE_COUNT",                     1);
    &hardcode_param("FAULT_SERVICE_CYCLE_TIMEOUT",           60);
    &hardcode_param("FAULTS_ORDER_TYPE",                     "sequential");
    &hardcode_param("FAULTS_ORDER_RANDOM_SEED",              "");
    &hardcode_param("FAULT_INITIATOR_PORT_DOWN_TIME",        "");
    &hardcode_param("FAULT_TARGET_PORT_DOWN_TIME",           "");
    &hardcode_param("SKIP_CLEANUP",                          1);
    &hardcode_param("SKIP_CLIENT_CLEANUP",                   1);
    &hardcode_param("VM_SKIP_ESX_CLEANUP",                   1);
    &hardcode_param("HV_SKIP_HYPERV_CLEANUP",                1);
    &hardcode_param("CITRIXXS_SKIP_HOST_CLEANUP",            1);
    &hardcode_param("SKIP_FILER_CLEANUP",                    1);
    &hardcode_param("FAULT_SWITCH_ABORT_ON_ERROR",           1);
    &hardcode_param("FAULT_SWITCH_ABORT_ON_CONFIG",          1);

    # VMware parameters
    &hardcode_param("VM_CONFIG_REBOOT_LIST",                 "all");

    # SLA parameters
    &hardcode_param("SLA_FOBENCH_TAKEOVER_TYPE",             "nvpanic:graceful");
    &hardcode_param("SLA_FOBENCH_GIVEBACK_TYPE",             "forced:graceful");
    &hardcode_param("SLA_FOBENCH_CYCLES",                    "5");
    &hardcode_param("SLA_FOBENCH_PARSE_PATH",                "/x/eng/rtpperf/fo_perf/CFO-TO-GB-Measurements/CFO_DATA");
    &hardcode_param("SLA_FOBENCH_PANIC_THRESHOLD_MIN",       "95");
    &hardcode_param("SLA_CLIENT_AVAIL_CHECK_THRESHOLD",      "10");
    &hardcode_param("SLA_FOBENCH_ENABLE_DEDUP",              "0");
    &hardcode_param("SLA_FOBENCH_ACTIVE_DEDUP_OPS",          "0");
    &hardcode_param("SLA_FOBENCH_ENABLE_COMPRESSION",        "0");
    &hardcode_param("SLA_FOBENCH_ACTIVE_COMPRESSION_OPS",    "0");
    &hardcode_param("SLA_FOBENCH_INLINE_COMPRESSION",        "0");
    &hardcode_param("SLA_FOBENCH_IO_DELAY",                  "900");

    return;
}

################################################################################
# Subroutine Name: load_mandatory_params()
# Objective:       Load all mandatory params. These are required by the user.
################################################################################
sub load_mandatory_params {

    # call &mandatory_param() for all mandatory parameters,
    &mandatory_param("ALUA_SUPPORT",         "Required ALUA_SUPPORT");
    &mandatory_param("PROTOCOL",             "Required PROTOCOL");
    &mandatory_param("AGGR",                 "Required AGGR");
    &mandatory_param("CLIENT_VMS",           "Required CLIENT_VMS");
    &mandatory_param("FILERS",               "Required FILERS");
    &mandatory_param("CLIENT_IO_FILE_SIZES", "Required CLIENT_IO_FILE_SIZES");

    # load the per client param hash
    $SanCIT::CLIENT_PARAM_HASH = &get_per_client_params();

    # verify that config iscsi type is set properly if doing iscsi
    if (&any_iscsi_clients() and (&get_param("CONFIG_ISCSI_TYPE") eq "")) {
        &cit_error("-fatal", "Error: Missing Mandatory Parameter: CONFIG_ISCSI_TYPE " .
                             "must be specified (software or hardware) if PROTOCOL=iscsi");
    }

    # verify that config iscsi discovery is set properly if doing iscsi
    if (&any_iscsi_clients() and (&get_param("CONFIG_ISCSI_DISCOVERY") eq "")) {
        &cit_error('-fatal', "Error: Missing Mandatory Parameter: CONFIG_ISCSI_DISCOVERY " .
                             "must be specified (0 (off) or 1 (on)) if PROTOCOL=iscsi");
    }

    # verify that config iscsi discovery precleanup is set properly if doing iscsi
    if (&any_iscsi_clients() =~ /iscsi/ and (&get_param("CONFIG_ISCSI_DISCOVERY_PRECLEANUP") !~ /0|1|2/)) {
        &cit_error('-fatal', "Error: Missing Mandatory Parameter: CONFIG_ISCSI_DISCOVERY_PRECLEANUP " .
                             "must be specified [0 (off) | 1 (soft destroy) | 2 (hard destroy)] if PROTOCOL=iscsi");
    }

    return;
}

################################################################################
# Subroutine Name: load_discretionary_params()
# Objective:       Load all discretionary params. These are not required by
#                  the user and have default values.
################################################################################
sub load_discretionary_params {
    my (%args) = @_;

    # Detection Settings
    &discretionary_param("CLIENT_LOSS_DETECTION",                 1);

    # Discretionary parameters (Defaults)
    &discretionary_param("NFS_CLIENTS",                           "");
    &discretionary_param("SWITCHES",                              "");
    &discretionary_param("APCON_SWITCHES",                        "");
    &discretionary_param("ETHERNET_SWITCHES",                     "");
    &discretionary_param("CONFIG_FCP_DRIVER_TYPE",                "vendor");
    &discretionary_param("CONFIG_FCP_AIX_ERROR_RECOVERY",         "");
    &discretionary_param("CONFIG_FCP_AIX_DYNAMIC_TRACKING",       "");
    &discretionary_param("CONFIG_FCP_AIX_NUM_CMD_ELEMS",          "");
    &discretionary_param("FILER_PACKET_TRACE",                    0);
    &discretionary_param("FILER_PACKET_TRACE_PASS_THRU",          "");
    &discretionary_param("FILER_VOL_MAX_COUNT",                   100);
    &discretionary_param("FILER_VOL_PER_CLIENT",                  1);
    &discretionary_param("FILER_KEEP_DUMMY_VOLS",                 0);
    &discretionary_param("FILER_DUMMY_ENABLE_ASIS",               0);
    &discretionary_param("FILER_DUMMY_ENABLE_COMPRESSION",        0);
    &discretionary_param("FILER_CNA_STATUS_CHECK",                0);
    &discretionary_param("FILER_CNA_STATUS_CHECK_ABORT_ON_ERROR", 0);
    &discretionary_param("FILER_SRAM_DUMP_CHECK_ALLOW_PANIC",     0);
    &discretionary_param("DEBUG_OUTPUT",                          0);
    &discretionary_param("IO_PANIC_FILERS_ON_ERROR",              0);
    &discretionary_param("RASTRACE_DUMP_ON_IO_ERROR",             0);
    &discretionary_param("IO_ABORT_WAIT_TIME",                    0);
    &discretionary_param("NTP_TIME_SERVER",                       "ntp0.eng.netapp.com");
    &discretionary_param("NTP_TIME_ZONE",                         "EST5EDT");
    &discretionary_param("FILER_VOL_DIRTY_SNAP_MAX_THREADS",      2);
    &discretionary_param("FILER_VOL_SNAP_MAX_COUNT",              -1);
    &discretionary_param("FILER_ISCSI_ENABLED_INTERFACE_LIST",    "");
    &discretionary_param("FILER_ISCSI_DISABLE_OTHER_INTERFACES",  1);
    &discretionary_param("CLIENT_DATA_COLLECTION",                0);
    &discretionary_param("CLIENT_DATA_COLLECTION_CMDS",           "");
    &discretionary_param("CLIENT_POST_DATA_COLLECTION",           0);
    &discretionary_param("CLIENT_POST_DATA_COLLECTION_CMDS",      "");
    &discretionary_param("CONFIG_REBOOT_CLIENTS",                 0);
    &discretionary_param("CLIENT_OFFLINE_DISCARD",                0);
    &discretionary_param("CLIENT_OFFLINE_DISCARD_ABORT_ON_ERROR", 0);
    &discretionary_param("SWITCH_ADMIN_DOMAIN",                   "");
    &discretionary_param("HYPERVISOR_TYPE",                       "none");

    # iSCSI Configuration parameters
    &discretionary_param("CONFIG_ISCSI_TYPE",                   "");
    &discretionary_param("CONFIG_ISCSI_HARDWARE_VENDOR",        "");
    &discretionary_param("CONFIG_ISCSI_DISCOVERY",              "");
    &discretionary_param("CONFIG_ISCSI_DISCOVERY_TYPE",         "sendtargets");
    &discretionary_param("CONFIG_ISCSI_DISCOVERY_PRECLEANUP",   "");
    &discretionary_param("CONFIG_ISCSI_SUBNET_MATCHING",        1);
    &discretionary_param("CONFIG_ISCSI_CHAP_MODE",              "none");
    &discretionary_param("CONFIG_ISCSI_CHAP_USERNAME",          "");
    &discretionary_param("CONFIG_ISCSI_CHAP_IN_PASSWORD",       "NATE12345678");
    &discretionary_param("CONFIG_ISCSI_CHAP_OUT_PASSWORD",      "87654321ETAN");
    &discretionary_param("CONFIG_ISCSI_DATA_DIGEST",            0);
    &discretionary_param("CONFIG_ISCSI_HEADER_DIGEST",          0);
    &discretionary_param("CONFIG_ISCSI_ERL",                    0);
    &discretionary_param("CONFIG_ISCSI_ISNS_SERVER",            "");
    &discretionary_param("CONFIG_ISCSI_IPVERSION",              "ipv4");
    &discretionary_param("CONFIG_ISCSI_IPV6_EXCLUDE_LINK_LOCAL", 1);
    &discretionary_param("CONFIG_ISCSI_UDO_INITIAL_R2T",        0);
    &discretionary_param("CONFIG_ISCSI_UDO_IMMEDIATE_DATA",     0);
    &discretionary_param("CONFIG_ISCSI_DEFAULT_TIME2WAIT",      "");
    &discretionary_param("CONFIG_ISCSI_DEFAULT_TIME2RETAIN",    "");
    &discretionary_param("CONFIG_ISCSI_WIN_LBPOLICY",           0);
    &discretionary_param("CONFIG_ISCSI_WIN_NUM_SESSIONS",       1);
    &discretionary_param("CONFIG_ISCSI_WIN_MCS_NUM_CONNS_PER_SESSION", 1);
    &discretionary_param("CONFIG_ISCSI_WIN_MPIO",               1);
    &discretionary_param("CONFIG_ISCSI_WIN_PERSISTENT",         0);
    &discretionary_param("CONFIG_ISCSI_WIN_MAX_CONNECTIONS",    "");
    &discretionary_param("CONFIG_ISCSI_WIN_KEEP_TARGET_PORTALS",0);
    &discretionary_param("CONFIG_NFS_SUBNET_MATCHING",          1);
    &discretionary_param("CONFIG_NFS_IPVERSION",                "ipv4");
    &discretionary_param("CONFIG_WIN_LBPOLICY",                 0);

    # Client I/O and configuration Settings
    &discretionary_param("CLIENT_IO_BLOCK_SIZES",                 "variable_record_size_2k_256k");
    &discretionary_param("CLIENT_IO_THREADS",                     2);
    &discretionary_param("CLIENT_IO_TYPE",                        "fs");
    &discretionary_param("CLIENT_VM_LAYOUT",                      "striped");
    &discretionary_param("CLIENT_VM_NUM_LVS",                     4);
    &discretionary_param("CLIENT_VM_VOL_SIZE",                    "1g");
    &discretionary_param("CLIENT_VM_STRIPE_SIZE",                 "64k");

    # General VM_IO Control Flow
    &discretionary_param("PRE_CLEANUP",                           1);
    &discretionary_param("SKIP_SETUP",                            0);
    &discretionary_param("SKIP_IOF",                              0);
    &discretionary_param("SKIP_IO",                               0);
    &discretionary_param("SKIP_FILER_PRE_CLEANUP",                0);
    &discretionary_param("SKIP_CLIENT_PRE_CLEANUP",               0);
    &discretionary_param("SKIP_ESX_PRE_CLEANUP",                  0);
    &discretionary_param("SKIP_HYPERV_PRE_CLEANUP",               0);
    &discretionary_param("SKIP_FILER_SETUP",                      0);
    &discretionary_param("VM_SKIP_FILER_SETUP",                   0);
    &discretionary_param("VM_SKIP_ESX_SETUP",                     0);
    &discretionary_param("HV_SKIP_FILER_SETUP",                   "0");
    &discretionary_param("HV_SKIP_HYPERV_SETUP",                  "0");
    &discretionary_param("CITRIXXS_SKIP_FILER_SETUP",             0);
    &discretionary_param("CITRIXXS_SKIP_HOST_SETUP",              0);
    &discretionary_param("SKIP_CITRIXXS_PRE_CLEANUP",             0);
    &discretionary_param("SKIP_CLIENT_SETUP",                     0);
    &discretionary_param("CLIENT_DEV_LIST",                       "");

    &discretionary_param("CLIENT_IO_PROGRAM",                     "dt.stable");
    &discretionary_param("CLIENT_IO_FS_TYPE",                     "");
    &discretionary_param("CLIENT_IO_PASS_THRU",                   "");
    &discretionary_param("TUNE_IO_MEASURE_DELAY",                 120);
    &discretionary_param("TUNE_IO_DELAY",                         0);
    &discretionary_param("TUNE_IO_STEP",                          0);
    &discretionary_param("TUNE_IO_DISK_MAX",                      85);
    &discretionary_param("TUNE_IO_DELAY_PERCENT",                 100);
    &discretionary_param("IO_DT_USE_PREFIX",                      1);
    &discretionary_param("IO_DT_OPTION_TRIGGER_SCRIPT",           "");
    &discretionary_param("IO_DT_OPTION_IOTYPE",                   "random");
    &discretionary_param("IO_DT_OPTION_ALARM",                    "3s");
    &discretionary_param("IO_DT_OPTION_KEEPALIVET",               "300");
    &discretionary_param("IO_SIO_OPTION_IOTYPE",                  "random");
    &discretionary_param("CLIENT_PATH_VALIDATION_INIT_STATE",     "");
    &discretionary_param("CLIENT_PATH_VALIDATION_ISCSI_INIT_STATE", "");
    &discretionary_param("CLIENT_PATH_VALIDATION_FCP_INIT_STATE", "");
    &discretionary_param("CLIENT_PATH_VALIDATION_ABORT_SCRIPT",   "");
    &discretionary_param("CLIENT_PATH_VALIDATION_TIMEOUT",        240);
    &discretionary_param("CLIENT_LOSS_DETECTION_TIMEOUT",         300);
    &discretionary_param("CLIENT_PATH_VALIDATION_ABORT_ON_ERROR", 1);
    &discretionary_param("CLIENT_PATH_VALIDATION_MIN_WAIT_TIME",  180);
    &discretionary_param("CLIENT_PATH_VALIDATION_VERIFY_IO_SWITCHING", 0);
    &discretionary_param("CLIENT_PATH_VALIDATION_VERIFY_IO_SWITCHING_ABORT_ON_ERROR", 0);
    &discretionary_param("PATH_VALIDATION_BETWEEN_PORT_DISABLE_ENABLE", 0);
    &discretionary_param("CLIENT_PATH_VALIDATION_PER_PORT",       0);
    &discretionary_param("CLIENT_LINUX_ENABLE_SYSRQ_DUMP",        0);
    &discretionary_param("CLIENT_USE_DEVICE_ENCYPTION",           0);
    &discretionary_param("CLIENT_DEVICE_ENCRYPTION_METHOD",       "NOT_SET");
    &discretionary_param("CLIENT_ORPHANED_DEVICE_CLEANUP",        "1");
    &discretionary_param("CLIENT_VALIDATION",                     "1");
    &discretionary_param("FAULT_SWITCH_PORT_FAILURE_SCRIPT",      "");
    &discretionary_param("FAULT_SWITCH_PORT_RETRY_COUNT",         6);
    &discretionary_param("FAULT_DOWN_TIMEOUT_BETWEEN_PORTS",      60);
    &discretionary_param("FAULT_SWITCH_PORT_SERIAL",              1);
    &discretionary_param("FAULT_SWITCH_PORT_CYCLE_TIMEOUT",       60);
    &discretionary_param("FAULT_SWITCH_VALIDATE_OFFLINE_ON_DISABLE", 1);
    &discretionary_param("FAULT_CYCLE_TIMEOUT",                   180);
    &discretionary_param("FAULT_START_DELAY",                     0);
    &discretionary_param("FAULT_CFO_RETRIES",                     3);
    &discretionary_param("FAULT_SWITCH_PORT_FCOE_ETHERNET_BLOCK", 1);
    &discretionary_param("FAULT_SWITCH_PORT_DUMP_TOPOLOGY",       1);
    &discretionary_param("ALLOW_THARNHOST",                       0);
    &discretionary_param("CLIENTS",                               "");
    &discretionary_param("VM_CLIENTS",                            "");
    &discretionary_param("KVM_CLIENTS",                           "");
    &discretionary_param("OVM_CLIENTS",                           "");
    &discretionary_param("CITRIXXS_CLIENTS",                      "");
    &discretionary_param("FILER_VOL_LIST",                        "");
    &discretionary_param("VM_CONTROLLER",                         "lsilogic");
    &discretionary_param("VM_CONFIG_FOR_VMOTION",                 0);
    &discretionary_param("VM_VMA_HOST",                           "");
    &discretionary_param("VM_SHUTDOWN_GUESTS_AFTER_CLEANUP",      "none");
    &discretionary_param("VM_REBOOT_ESX_AFTER_CLEANUP",           0);
    &discretionary_param("VM_VMFS_FVL",                           "");
    &discretionary_param("VM_RDM_FVL",                            "");
    &discretionary_param("VM_DONT_USE_CONFIG_MPATH",              0);
    &discretionary_param("VM_VMFS_TYPE",                          "");
    &discretionary_param("VM_VMFS_BLOCKSIZE",                     "auto");
    &discretionary_param("VM_RDM_TYPE",                           "physical");
    &discretionary_param("VM_ADD_REMOVE_VMDK_STYLE",              "parallel");
    &discretionary_param("VM_ADD_REMOVE_VMDK_PARALLEL_LIMIT",     3);
    &discretionary_param("VM_DELETE_NON_EMPTY_DATASTORES",        0);
    &discretionary_param("VM_VXDMP_IO_POLICY",                    "");
    &discretionary_param("FILER_THIN_PROVISIONING_VOL_PERCENT",   "20");
    &discretionary_param("FILER_THIN_PROVISIONING_VOL_INCREMENT", "250m");
    &discretionary_param("USE_EXTENDED_LUN_TYPES",                "1");
    &discretionary_param("FILER_PORTSET_EXCLUSIONS",              "");
    &discretionary_param("FILER_ACCESSLIST_EXCLUSIONS",           "");
    &discretionary_param("FILER_USE_LUN_CLONE_SPLIT",             "0");
    &discretionary_param("FILER_USE_LUN_CLONE",                   "0");
    &discretionary_param("FILER_USE_SIS_CLONE",                   "0");
    &discretionary_param("VM_DATASTORE_TYPE",                     "one-to-one");
    &discretionary_param("VM_NUM_VMFS_VMDKS",                     "-1");
    &discretionary_param("VM_PSP_LIST",                           "");
    &discretionary_param("VM_PSP_LIST_RANDOM_SEED",               "");
    &discretionary_param("TUNE_IO_ATTEMPTS",                      10);
    &discretionary_param("IO_THRESHOLD_CHECKING",                 1);
    &discretionary_param("CLIENT_MOUNT_OPTIONS_DISCARD",          0);
    &discretionary_param("FILER_LUN_ENABLE_SPACE_ALLOC",          0);
    &discretionary_param("FILER_ENABLE_ASIS",                     "0");
    &discretionary_param("FILER_ENABLE_COMPRESSION",              "0");
    &discretionary_param("FILER_ENABLE_INLINE_COMPRESSION",       "0");
    &discretionary_param("FILER_COMPRESSION_DELETE_CHECKPOINT",   "0");
    &discretionary_param("FILER_COMPRESSION_INCLUDE_SHARED_BLOCKS",   "0");
    &discretionary_param("FILER_COMPRESSION_INCLUDE_SNAPSHOT_BLOCKS", "0");
    &discretionary_param("FILER_ASIS_SCHEDULE",                   "auto");
    &discretionary_param("FILER_IGROUP_THROTTLE_RESERVE",         "10");
    &discretionary_param("FILER_IGROUP_THROTTLE",                 "0");
    &discretionary_param("CFO_DETECTION",                         1);
    &discretionary_param("CFO_DETECTION_TIMEOUT",                 "");
    &discretionary_param("FAULT_SWITCH_MISCOMPARE_ABORT_ON_ERROR",1);
    &discretionary_param("TUNE_IO_NUM_MEASUREMENTS",              3);
    &discretionary_param("FAULT_IGRP_THROTTLE_RESERVE",           99);
    &discretionary_param("FAULT_IGRP_THROTTLE_PERIOD_FCP",        60);
    &discretionary_param("FAULT_IGRP_THROTTLE_PERIOD_ISCSI",      10);
    &discretionary_param("FAULT_VALIDATE_RUNLENGTH",            1800);
    &discretionary_param("FAULT_VALIDATE_CYCLE_TIME",           60);
    &discretionary_param("VALIDATE_ENVIRONMENT_RESTORE_TAKEOVER_OPTION",   0);
    &discretionary_param("VALIDATE_ENVIRONMENT_AGGR_CHECK_ABORT_ON_ERROR", 0);
    &discretionary_param("VALIDATE_AGGR_MIN_DISK_COUNT",                   0);
    &discretionary_param("USE_QA_ASUP_SERVER",                             0);
    &discretionary_param("CIT_USE_WWPN_BLOCK_FAULT_TYPE_FOR_INITIATORS",   0);
    &discretionary_param("CIT_USE_WWPN_BLOCK_FAULT_TYPE_FOR_TARGETS",      0);
    &discretionary_param("CIT_FCOE",                              0);
    &discretionary_param("CIT_DIRECT_CONNECT",                    0);
    &discretionary_param("FILER_ROTATE_SYSLOGS",                  0);
    &discretionary_param("CLIENT_ROTATE_SYSLOGS",                 0);
    &discretionary_param("USE_SINGLE_CONTROLLER",                 0);
    &discretionary_param("CLIENT_IO_ROTATE_USE_OF_BUFFER_CACHE",  1);

    # Linux DMMP configuration params
    &discretionary_param("CONFIG_DMMP_USE_USER_FRIENDLY_NAMES",   0);
    &discretionary_param("CONFIG_DMMP_MAX_FDS",                   "NOT_SET");
    &discretionary_param("CONFIG_DMMP_QUEUE_WITHOUT_DAEMON",      "NOT_SET");
    &discretionary_param("CONFIG_DMMP_FLUSH_ON_LAST_DEL",         "NOT_SET");
    &discretionary_param("CONFIG_DMMP_PG_PRIO_CALC",              "NOT_SET");
    &discretionary_param("CONFIG_DMMP_PATH_GROUPING_POLICY",      "NOT_SET");
    &discretionary_param("CONFIG_DMMP_FEATURES",                  "NOT_SET");
    &discretionary_param("CONFIG_DMMP_PATH_CHECKER",              "NOT_SET");
    &discretionary_param("CONFIG_DMMP_RR_WEIGHT",                 "NOT_SET");
    &discretionary_param("CONFIG_DMMP_RR_MIN_IO",                 "NOT_SET");
    &discretionary_param("CONFIG_DMMP_HARDWARE_HANDLER",          "NOT_SET");
    &discretionary_param("CONFIG_DMMP_PRIO_CALLOUT",              "NOT_SET");
    &discretionary_param("CONFIG_DMMP_PRIO",                      "NOT_SET");
    &discretionary_param("CONFIG_DMMP_FAILBACK",                  "NOT_SET");
    &discretionary_param("CONFIG_DMMP_GETUID_CALLOUT",            "NOT_SET");
    &discretionary_param("CONFIG_DMMP_PATH_SELECTOR",             "NOT_SET");
    &discretionary_param("CONFIG_DMMP_PRODUCT",                   "NOT_SET");

    # Hyper-V Params
    &discretionary_param("HV_CLIENTS",                            "");
    &discretionary_param("HV_NUM_VHD_DISKS_PER_LUN",              "1");
    &discretionary_param("HV_VHD_FVL",                            "");
    &discretionary_param("HV_PT_FVL",                             "");
    &discretionary_param("HV_VHD_CONTROLLER_TYPE",                "scsi");
    &discretionary_param("HV_PT_CONTROLLER_TYPE",                 "scsi");
    &discretionary_param("HV_VHD_TYPE",                           "fixed");

    # Citrix-XS params
    &discretionary_param("CITRIXXS_NUM_VDI_PER_LUN",              1);
    &discretionary_param("CITRIXXS_VDI_FVL",                      "");

    # DT params - individual teams need to be able to override
    &discretionary_param("IO_DT_OPTION_NOPROGTT",                 "125s");
    &discretionary_param("IO_DT_OPTION_NOPROGT",                  "15s");

    # DT version control based on platform type
    &discretionary_param("IO_DT_SOLARIS_VERSION_OVERRIDE",        "");
    &discretionary_param("IO_DT_LINUX_VERSION_OVERRIDE",          "");
    &discretionary_param("IO_DT_AIX_VERSION_OVERRIDE",            "");
    &discretionary_param("IO_DT_HPUX_VERSION_OVERRIDE",           "");
    &discretionary_param("IO_DT_WINDOWS_VERSION_OVERRIDE",        "");

    # NDVM Discretionary Parameters
    &discretionary_param("NDVM_AGGRLIST",                         "");
    &discretionary_param("CIT_NDVM_VOL_NAMED",                    "");

    # Client Validation Discretionary Parameters
    &discretionary_param("VALIDATE_ABORT_ON_ERROR",               "1");

    # ESX VxDMP Fault io policy cycling during faults
    &discretionary_param("FAULT_ESX_VXDMP_ROTATE_IO_POLICIES",    "0");
    &discretionary_param("FAULT_ESX_VXDMP_ROTATE_IO_POLICIES_ABORT_ON_ERROR",    "0");
    &discretionary_param("FAULT_ESX_VXDMP_ROTATE_IO_POLICY_STYLE","same");
    &discretionary_param("FAULT_ESX_VXDMP_IO_POLICY_LIST",        "minimumq,round-robin,adaptive");

    # SLA Parameters
    &discretionary_param("SLA_FOBENCH_IO_TYPE",                   "fcp");
    &discretionary_param("SLA_FOBENCH_DESTINATION",               "");
    &discretionary_param("SLA_FOBENCH_ONTAP_VERSION",             "");
    &discretionary_param("SLA_FOBENCH_TITLE",                     "SAN_Interop_60_Second_SLA");
    &discretionary_param("SLA_FOBENCH_TYPE",                      "SAN_Interop");
    &discretionary_param("SLA_FOBENCH_PREFIX",                    "nate_cit");
    &discretionary_param("SLA_FOBENCH_CLIENT_OS",                 "mixed");

    # Fault Injections
    &discretionary_param("FAULT_PORT_BLOCK_CYCLE_COUNT",          "1");
    &discretionary_param("FILER_DOWN_TIMEOUT",                    300);

    # bypass the fvl check if requested to
    if (exists($args{-bypass_fvl_check}) and $args{-bypass_fvl_check} == 1) {
        return;
    }

    # validate the filer vol list and clients parameters
    if (&get_param("CLIENTS") ne "" and &get_param("FILER_VOL_LIST") eq "") {
        &cit_error('-fatal', "SanCIT::load_discretionary_params(): FILER_VOL_LIST must be specified if CLIENTS is not empty.");
    }
    if (&get_param("VM_CLIENTS") ne "" and (&get_param("VM_VMFS_FVL") eq "" and &get_param("VM_RDM_FVL") eq "")) {
        &cit_error('-fatal', "SanCIT::load_discretionary_params(): VM_VMFS_FVL or VM_RDM_FVL must be specified if VM_CLIENTS is not empty.");
    }
    if (&get_param("HV_CLIENTS") ne "" and (&get_param("HV_VHD_FVL") eq "" and &get_param("HV_PT_FVL") eq "")) {
        &cit_error('-fatal', "SanCIT::load_discretionary_params(): HV_VHD_FVL or HV_PT_FVL must be specified if HV_CLIENTS is not empty.");
    }

    return;
}

################################################################################
# Subroutine Name: load_dynamic_params()
# Objective:       Load all dynamic params. Depends on the controller model
#                  and class.
################################################################################
sub load_dynamic_params {
    my (%args) = @_;

    my ($no_flexvol_limits) = 0;

    # parse arguments
    foreach (keys %args) {
        if (/-?no_flexvol_limits/) {
            if (not length($no_flexvol_limits = $args{$_})) {
                &cit_error('-script', "Error: Invalid value for -no_flexvol_limits");
                return;
            }
        }
    }

    # pull in the model and class into the environment
    my ($model, $class, $ontap_version) = &get_controller_model_class();

    # depending on the model, set specific parameters
    my $dynamic_params = {};

    ###########################################
    # Set dynamic params for the lowest class
    ###########################################
    $dynamic_params->{lowest}->{FILER_VOL_DIRTY_SNAP_MAX_THREADS} = 2;

    ###########################################
    # Set dynamic params for the low class
    ###########################################
    $dynamic_params->{low}->{FILER_VOL_DIRTY_SNAP_MAX_THREADS} = 2;

    ###########################################
    # Set dynamic params for the medium class
    ###########################################
    $dynamic_params->{medium}->{FILER_VOL_DIRTY_SNAP_MAX_THREADS} = 2;

    ###########################################
    # Set dynamic params for the high class
    ###########################################
    $dynamic_params->{high}->{FILER_VOL_DIRTY_SNAP_MAX_THREADS} = 2;

    ###########################################
    # Set dynamic params for the highest class
    ###########################################
    $dynamic_params->{highest}->{FILER_VOL_DIRTY_SNAP_MAX_THREADS} = 2;

    # set all the dynamic params for our controller class type
    foreach my $class_type (keys %{$dynamic_params}) {
        if ($class_type =~ /^$class$/) {
            foreach my $param_name (keys %{$dynamic_params->{$class_type}}) {
                &discretionary_param($param_name, $dynamic_params->{$class_type}->{$param_name});
            }
            last;
        }
    }

    # load the flexvol limits for FILER_VOL_MAX_COUNT unless specified not to
    if ($no_flexvol_limits == 0) {
        my ($status, $flex_vol_value) = &SanUtil::get_ontap_flexvol_limits(-ontap_version => $ontap_version,
                                                                           -controller_model => $model,
                                                                           -type => "minor");
        if ($status == $SanUtil::FALSE) {
            # at this point no match could be made. this should be an error.
            &cit_error('-config', "Error: Could not determine the FlexVol limits for the following configuration:\n" .
                                  " Controller Model  : $model\n" .
                                  " ONTAP Version     : $ontap_version\n");
            return;
        }

        &SanUtil::nate_log('-comment', "FlexVol Limits for this Configuration:\n" .
                                       " Controller Model  : $model\n" .
                                       " ONTAP Version     : $ontap_version\n" .
                                       " FlexVol Limit     : $flex_vol_value\n");

        # hardcode the filer vol max count value
        &hardcode_param("FILER_VOL_MAX_COUNT", $flex_vol_value);
    }

    # burt459106 if current ONTAP is >= 8.0.1 7mode, then enable Dedupe by
    # default
    if ($ontap_version =~ /8\.0\.[1-9]|8\.[1-3]|boilermaker|rollingrock|sierra|main|fsb/i) {
        &hardcode_param("FILER_ENABLE_ASIS",       1);
        &hardcode_param("FILER_DUMMY_ENABLE_ASIS", 1);
    }

    return;
}

################################################################################
# Subroutine Name: load_special_params()
# Objective:       Load all special params.
################################################################################
sub load_special_params {
    my ($unset_ndisp_params) = @_;

    &param("NDISPD_HOST",                             '-default', "");
    &param("NDISPD_PORT",                             '-default', "");
    &param("CIT_REUSE_PREVIOUS_CONFIG",               '-default', -1);
    &param("CIT_RESET_CLIENT_DEV_LIST_ON_RECONFIG",   '-default', 1);
    &param("CIT_DONT_STORE_NDISP_DATA",               '-default', 0);
    &param("NDISPD_PREVIOUS_EXIT_STATUS",             '-default', "");
    &param("CLIENT_DEV_LIST",                         '-default', "");
    &param("CIT_POLICY_HOST_TEAM",                    '-default', "default");

    # if the NDISPD host and NDISPD port are set then enable the REUSE PREVIOUS CONFIG feature
    if (&is_param_defined("NDISPD_HOST") and &is_param_defined("NDISPD_PORT")) {
        # open a connection to the ndisp server
        my $host = &get_param("NDISPD_HOST");
        my $port = &get_param("NDISPD_PORT");
        my $ndispd;

        # instantiate the ndispd object
        eval { $ndispd = ndisp::API->new("$host\:$port"); };
        if (not defined ($ndispd)) {
            &cit_error('-fatal', "Error: Could not instantiate a connection to the NDISPD server hosted at ${host}:${port}");
            return;
        }

        # if requested to disable ndisp parameters for CIT, do so here and return
        if ($unset_ndisp_params == 1) {

            &SanUtil::nate_log('-comment', "Disabling the CLIENT_DEV_LIST and resetting the previous exit status.");

            # disable the CLIENT_DEV_LIST and NDISPD_PREVIOUS_EXIT_STATUS from the NDISPD server
            &unset_ndispd_config_param($ndispd, "CLIENT_DEV_LIST", "ignore");

            # hardcode the client dev list to be null
            &hardcode_param("CLIENT_DEV_LIST", "");

            # remove the NDISP_PREVIOUS_EXIT_STATUS parameter from the ndisp server
            &unset_ndispd_config_param($ndispd, "NDISPD_PREVIOUS_EXIT_STATUS", "ignore");
            $ndispd = {};
            return;
        }

        # if The user wanted to reuse the previous config and the previous test failed.
        # then: set everything to reconfigure.
        # hardcode PRE_CLEANUP=1, SKIP_SETUP=0 CLIENT_DEV_LIST="", reset CLIENT_DEV_LIST
        # from NDISP if it exists.
        if (&get_param("CIT_REUSE_PREVIOUS_CONFIG") == 1 and
            &get_param("NDISPD_PREVIOUS_EXIT_STATUS") =~ /fail/i) {

            &SanUtil::nate_log('-comment', "Was requested to reuse the previous config but the last test failed. \n" .
                                           "Disabling the CLIENT_DEV_LIST and reconfiguring for this test.");

            # reset the control parameters
            &_reconfigure_and_reset_cdl($ndispd);

        } elsif (&get_param("CIT_REUSE_PREVIOUS_CONFIG") == 1 and
                 &get_param("NDISPD_PREVIOUS_EXIT_STATUS") =~ /pass|warn/i) {
            # In this case, the previous test did not fail and the user requested to reuse the config after non failures
            # Bypass Pre cleanup and setup and reuse the CLIENT_DEV_LIST from the previous test.
            # obtain the value from the NDISPD server, NOT the user.

            # Bypass pre cleanup and setup
            &hardcode_param("PRE_CLEANUP", 0);
            &hardcode_param("SKIP_SETUP",  1);

            # verify that CLIENT_DEV_LIST is currently set in the environment
            if (not &is_param_defined("CLIENT_DEV_LIST")) {
                # if it's not set, then to avoid problems with the queue, unset the previous exit status and fatal this current test.
                &unset_ndispd_config_param($ndispd, "NDISPD_PREVIOUS_EXIT_STATUS", "ignore");
                &cit_error('-fatal', "\nError: Unable to reuse the previous configuration because CLIENT_DEV_LIST is not\n" .
                                     "defined in neither the NATE environment nor the NDISP environment. \n" .
                                     "However, the previous test passed/warned. Re-run the test after doing either:" .
                                     "\n\t 1) ndispc -s ${host}:${port} set config CLIENT_DEV_LIST=<value>" .
                                     "\n or\n\t 2) insert the CLIENT_DEV_LIST parameter in your '.include' file.\n");
            } else {
                my $cdl = &get_param("CLIENT_DEV_LIST");
                # otherwise hardcode the client dev list with the value that was obtained from the ndisp server
                &SanUtil::nate_log('-comment', "The previous test passed. Bypassing precleanup and setup. Re-using the CLIENT_DEV_LIST listed below:\n " .
                                               "\tCLIENT_DEV_LIST=$cdl");
                return;
            }

        } elsif (&get_param("CIT_REUSE_PREVIOUS_CONFIG") == -1) {
            # This is the default case where the user did not choose to do any dynamic reconfiguration on their test
            &SanUtil::nate_log('-comment', "Was not requested to perform any dynamic reconfiguration. Will not override any control\n" .
                                           "settings set by the user (IE: SKIP_*). The default action is to run pre-cleanup \n ".
                                           "and setup unless specified otherwise in an include file.");

            # If told to reset the clientdevlist on reconfiguration and we are reconfiguring, unset it.
            if (&get_param("CIT_RESET_CLIENT_DEV_LIST_ON_RECONFIG") == 1 and &get_param("PRE_CLEANUP") == 1 and &get_param("SKIP_SETUP") == 0) {
                &unset_ndispd_config_param($ndispd, "CLIENT_DEV_LIST", "ignore");
                &hardcode_param("CLIENT_DEV_LIST", "");
                &SanUtil::nate_log('-comment', "The CLIENT_DEV_LIST has been reset because CIT_RESET_CLIENT_DEV_LIST_ON_RECONFIG=1. \n" .
                                               "Set this to '0' to keep the CLIENT_DEV_LIST intact.");
            }

        } elsif (&get_param("CIT_REUSE_PREVIOUS_CONFIG") == 0) {
            # not requested to reuse the previous config. Pre-cleanup and Setup are the default action to be performed here
            # but can be overriden by the user for finer control.
            &SanUtil::nate_log('-comment', "Was requested to always reconfigure the test. Disabling the\n" .
                                           "CLIENT_DEV_LIST and reconfiguring for this test.");

            # reset the control parameters
            &_reconfigure_and_reset_cdl($ndispd);

        } elsif (&get_param("CIT_REUSE_PREVIOUS_CONFIG") == 1) {
            # In this case there is no previous test data, we don't know if it failed or passed. Reconfigure in this case
            &SanUtil::nate_log('-comment', "Was requested to reuse the previous configuration but the history of the previous \n" .
                                           "test is not available. This probably happened because NDISPD_PREVIOUS_EXIT_STATUS \n" .
                                           "is not defined on the ndisp server. Disabling the CLIENT_DEV_LIST and reconfiguring for this test.");

            # reset the control parameters
            &_reconfigure_and_reset_cdl($ndispd);

        } elsif (&get_param("CIT_REUSE_PREVIOUS_CONFIG") == 2 and
                 not &is_param_defined("CLIENT_DEV_LIST")) {
            # in this case, the user requested to always use the previous config, but there is no client dev list,
            # so fail the test
            &SanCIT::cit_error('-config', "Was requested to always reuse the previous config but there is no record of a " .
                                          "CLIENT_DEV_LIST on the ndisp server or include file. Cannot proceed.");

        } elsif (&get_param("CIT_REUSE_PREVIOUS_CONFIG") == 2) {
            # In this case, the user requested to always use the previous config and a previous config exists.
            # force it to reuse the previous configuration

            # Bypass pre cleanup and setup
            &hardcode_param("PRE_CLEANUP", 0);
            &hardcode_param("SKIP_SETUP",  1);

            my $cdl = &get_param("CLIENT_DEV_LIST");
            &SanUtil::nate_log('-comment', "Was requested to always reuse the previous configuration when possible.\n" .
                                           "Re-using the CLIENT_DEV_LIST listed below:\n" .
                                           "\tCLIENT_DEV_LIST=$cdl");
        }

        # delete the ndisp object
        $ndispd = {};

    } else {
        &SanUtil::nate_log('-comment', "NDISP is not being used for this test.");
    }

    return;
}

################################################################################
# Subroutine Name: set_param()
# Objective:       Change the value of a parameter in the environment using
#                  Tharn.
################################################################################
sub set_param {
    my ($param, $value) = @_;

    # Given a parameter $param, this will update its value with $value
    &Tharn::param("$param", "-set", "$value");

    return;
}

################################################################################
# Subroutine Name: unset_param()
# Objective:       Delete the value of a tharn param in the environment
################################################################################
sub unset_param {
    my ($param) = @_;

    # if the parameter exists, unset it.
    if (&param_exists($param)) {
        &Tharn::param("$param", "-unset");
    }

    return;
}

################################################################################
# Subroutine Name: get_param()
# Objective:       Obtain the value of a parameter in the environment using
#                  Tharn. Ignore warnings (refine) around this to avoid
#                  collisions with Tharn version (always called package
#                  qualified)
################################################################################
no warnings qw(redefine);
sub get_param {
    my ($param) = @_;

    # verify the param exists
    if (not &is_param_defined($param)) {
        return "";
    }

    # get the value from the environment
    return &Tharn::param("$param");
}
use warnings qw(redefine);

################################################################################
# Subroutine Name: param_exists()
# Objective:       Determine if the specified parameter exists in Tharn
################################################################################
sub param_exists {
    my ($param) = @_;

    if (not defined(&Tharn::param("$param"))) {
        return 0;
    } else {
        return 1;
    }
}

################################################################################
# Subroutine Name: is_param_defined()
# Objective:       Determine if the specified parameter is defined already.
################################################################################
sub is_param_defined {
    my ($param) = @_;

    if (not &param_exists($param)) {
        # param is not in environment, is not defined
        return 0;
    } elsif (not defined(&Tharn::param("$param")) or &Tharn::param("$param") eq "") {
        # param exists but is not defined or param is empty
        return 0;
    } else {
        # param exists and is defined
        return 1;
    }
}

################################################################################
# Subroutine Name: hardcode_param()
# Objective:       Simplify the task of hardcoding a parameter in Tharn
################################################################################
sub hardcode_param {
    my ($param, $value, $force) = @_;

    # default value of forcce is 0
    if (not defined($force)) {
        $force = 0;
    }

    # If this is not for score mode, simply call discretionary param call
    # with the pass arguments.
    if (&get_param("CIT_NOT_FOR_SCORE") == 1 or &get_param("TESTNAME") =~ /SANInterop\/io\/sm_io\.thpl|SANInterop\/cit\/CIT_HA_custom\.thpl/) {
        # if the user really wants to hardcode regardless of
        # NOT_FOR_SCORE (like in case of "" value), allow this and
        # don't call discretionary_param, drop down and call original
        # logic.
        if ($force == 0) {
            &discretionary_param($param, "$value");
            return;
        }
    }

    # special case use for RRE to map certain flags to
    # discretionary (instead of hardcode for generic CIT
    # cases)
    if (&get_param("RRE") == 1) {
        if ($param eq "TUNE_IO_DISK_MIN") {
            &discretionary_param($param, "$value");
            return;
        }
    }

    # hardcode the value of the parameter
    &set_param($param, $value);

    # add param to the %no_tharnhost_hsh
    $no_tharnhost_hsh{$param} = $value;

    return;
}

################################################################################
# Subroutine Name: mandatory_param()
# Objective:       Verify that a parameter is set.
################################################################################
sub mandatory_param {
    my ($param, $failure_message) = @_;

    # in NATE 5.1.109 and later we need to do this slightly differently
    if ($NATE::VERSION >= 5.001109) {
        my $paramset = NATE::ParamSet->new(global => 1);

        my $origvalue = $paramset->get_initial_param($param);
        if (not defined($origvalue)) {
            # param was never defined by the user, bail out
            &cit_error('-config', "Error: Missing Mandatory Parameter '$param' : $failure_message");
        }
        # update the current value to be the original value which was provided by the user
        &set_param($param, $origvalue);
    } else {
        # if the parameter exists, set it to undefined
        if (&param_exists($param)) {
            &unset_param($param);
        }

        # load it in the environment using &param
        if (&get_param("$param") eq "") {
            &cit_error('-config', "Error: Missing Mandatory Parameter '$param' : $failure_message");
        }
    }

    # remove param from %no_tharnhost_hsh
    delete $no_tharnhost_hsh{$param};

    return;
}

################################################################################
# Subroutine Name: discretionary_param()
# Objective:       Load a discretionary param with a default value into the
#                  environment.
################################################################################
sub discretionary_param {
    my ($param, $default_value) = @_;

    # in NATE 5.1.109 and later we need to do this slightly differently
    if ($NATE::VERSION >= 5.001109) {
        my $paramset = NATE::ParamSet->new(global => 1);

        my $origvalue = $paramset->get_initial_param($param);
        if (not defined($origvalue)) {
            # param was never in the environment by the user, override the value to be the
            # new default
            &set_param("$param", "$default_value");
        } else {
            # ignore the default that was passed in, use the original value that was
            # in the environment
            &set_param("$param", "$origvalue");
        }
    } else {
        # if the parameter exists, set it to undefined
        if (&param_exists($param)) {
            &unset_param($param);
        }

        # load it in the environment using &param
        &param("$param", "-default", "$default_value");
    }

    # remove param from %no_tharnhost_hsh
    delete $no_tharnhost_hsh{$param};

    return;
}

################################################################################
# Subroutine Name: get_controller_model()
# Objective:       Query the controllers for model type and ontap version
################################################################################
sub get_controller_model_class {
    my (@filers) = split(/:/, &get_param("FILERS"));

    # special override for use when ONTAP version is not able
    # to be determined from sysconfig -v or version.
    &param("CIT_ONTAP_VERSION_OVERRIDE", '-default', "");

    if (scalar(@filers) < 1) {
        return (undef, undef, undef);
    }

    # determine the model type of one of the controllers
    my ($filer_a) = @filers;
    my ($output_str, $status, @output_arr) = &SanUtil::system_zapi($filer_a, "sysconfig -v");
    if ($status == SanUtil::ZAPI_FAILURE) {
        return (undef, undef, undef);
    }

    # controllers cannot be in a takeover state.
    if ($output_str =~ /This system has been taken over by|This system has taken over/i) {
        &cit_error('-config', "Error: The controllers (@filers) are in a takeover state. Please correct and try again.");
        return (undef, undef, undef);
    }

    # look for the model name
    my $model = "";
    my $ontap_version = "";
    foreach my $line (@output_arr) {
        # extract the model of the controller
        if ($line =~ /Model Name:\s+(\S+)/i) {
            $model = $1;
            # version should always be processed before model so it is ok
            # to break out at this point
            # we need to break out here because there can exist more "Model Name:" lines in @output_arr
            # its safe to assume that the controller model will be the first instance we see.
            last;
        }

        # NetApp Release MainNG__10.0.99 7-mode is returned on "main" builds
        # and on private builds
        # NetApp Release mainngN_090331_0100: Tue Mar 31 01:31:38 PDT 2009
        # Data ONTAP Release 7.3.1: Thu Jan  8 05:06:50 PST 2009 (IBM)
        # Style is returned on promoted or interim builds from rats.
        my $ontap_version_override = &get_param("CIT_ONTAP_VERSION_OVERRIDE");
        if ($ontap_version_override eq "") {
            if ($line =~ /(Data ONTAP|IBM|NetApp)\s+.*\s*Release\s+(.+?)(:|)\s+/i) {
                $ontap_version = $2;
                # if this release if not 6.x|7.x|8.x|9.x|10.x, then it's a branch
                # and go and get the major revision of the system
                if ($ontap_version !~ /(6\.|7\.|8\.|9\.|10\.|anchor|iron|boiler|gordon|rolling)/i) {
                    my $filer_obj = SanFiler->new(-filer => $filer_a,
                                                  -debug_level => 1,
                                                  -interface => "zapi",
                                                  -conn_type => "#rsh");
                    if ($filer_obj->error()) {
                        my @error = $filer_obj->errmsg();
                        &SanUtil::nate_log('-config',
                                           "SanCIT::get_controller_model_class Could not create filer object ($filer_a)\n" .
                                           "Error: " . join('\n', @error));
                        return (undef, undef, undef);
                    }
                    # get the controller version
                    $ontap_version = $filer_obj->get_version_major();
                    if ($filer_obj->error()) {
                        my @error = $filer_obj->errmsg();
                        &SanUtil::nate_log('-config',
                                           "SanCIT::get_controller_model_class Could not get ONTAP major revision from controller ($filer_a)",
                                           "Error: " . join('\n', @error));
                        return (undef, undef, undef);
                    }
                }
            }
        } else {
            $ontap_version = $ontap_version_override;
        }
    }

    # determine the class of the controller (lowest, low, med, high, highest)
    my $class_table = {};
    $class_table->{"lowest"}  = ["FAS250", "F825", "F825c", "F880", "F880c",
                                 "FAS270", "FAS270c", "FAS920", "N3700",
                                 "DataONTAP-vM100", "VSA1000", "VSA2000"];
    $class_table->{"low"}     = ["FAS2020", "FAS2050", "FAS2040", "FAS940", "FAS3020", "N3300",
                                 "N3400", "N3600", "N5200", "V2020", "V2050", "GF940", "V940",
                                 "V3020", "FAS2240-2", "N3220", "FAS2240-4", "N3240", "FAS2220"];
    $class_table->{"medium"}  = ["FAS2040", "FAS3140", "FAS3160", "FAS3040", "FAS3050", "FAS960",
                                 "N6040", "N6060", "N5500", "N5300", "V3140", "V3160",
                                 "V3040", "V3050", "GF960", "V960", "FAS3220", "FAS3250",
                                 "N6220", "N6250"];
    $class_table->{"high"}    = ["FAS980", "GF980", "V980"];
    $class_table->{"highest"} = ["FAS3170", "FAS3070", "FAS6030", "FAS6070", "FAS6040", "FAS6080",
                                 "N6070",   "N5600",   "N7600",   "N7800",   "N7700",   "N7900",
                                 "V3170",   "V3070",   "V6030",   "V6070",   "V6040",   "V6080",
                                 "FAS3210", "FAS3240", "FAS3270", "FAS6210", "FAS6240", "FAS6280",
                                 "N6210",   "N6240",   "N6270",   "N7550T",  "N7750T",  "N7950T",
                                 "V3210",   "V3240",   "V3270",   "V6210",   "V6240",   "V6280",
                                 "FAS8020", "V8020", "N8020", "FAS8040", "V8040", "N8040",
                                 "FAS8060", "V8060", "N8060", "FAS8080", "V8080", "N8080"];

    # match the class.
    my $class = "";
    foreach my $class_range (keys %{$class_table}) {
        if (grep(/^$model$/, @{$class_table->{$class_range}})) {
            $class = $class_range;
            last;
        }
    }

    # store the values in the FILER_* parameters
    &set_param("FILER_MODEL", $model);
    &set_param("FILER_CLASS", $class);
    &set_param("FILER_ONTAP_VERSION", $ontap_version);

    &SanUtil::nate_log('-comment', "Controller Info: \n" .
                                   " Controller model : $model\n" .
                                   " Class            : $class\n" .
                                   " ONTAP version    : $ontap_version");

    return ($model, $class, $ontap_version);
}

################################################################################
# Subroutine Name: print_environment_params()
# Objective:       Display all the environment parameters that have been set in
#                  the environment for this CIT test. Also, will return the
#                  contents of the params on a per client (hash) basis
################################################################################
sub print_environment_params {
    my ($runid, %args) = @_;

    my $unset_clients = delete($args{-unset_clients});

    # default the runid
    $runid = "CIT_Environment" if not defined($runid);

    # If this is not for score mode, print this out and already WARN
    # the test.
    if (&get_param("CIT_NOT_FOR_SCORE") == 1) {
        &SanUtil::nate_log('-warn', "CIT NOT FOR SCORE MODE\n" .
                           "#################################################################\n" .
                           "#################################################################\n" .
                           "#################################################################\n" .
                           "#####          ** CIT NOT FOR SCORE MODE **                ######\n" .
                           "#####          - Allowing user to override hardcoded       ######\n" .
                           "#####            params. Test will be allowed to run,      ######\n" .
                           "#####            but will be failed at the end of the      ######\n" .
                           "#####            CIT thpl.                                 ######\n" .
                           "#####          ** THESE RESULTS ARE NOT VALID FOR USE **   ######\n" .
                           "#####          ** IN STANDARD QUALIFICATION **             ######\n" .
                           "#################################################################\n" .
                           "#################################################################\n" .
                           "#################################################################\n" .
                           "\n");
    }

    my $subtest = subtest(
        sub {
            if ($unset_clients) {
                # unset the clients param if told to
                Tharn::param("CLIENTS", -unset);
            }

            my @print_stack = ();
            push(@print_stack, "The following are all the parameters that have been set for this test case.");

            # for each environment param, push its value onto a print stack
            my $param_set = NATE::ParamSet->new(global => 1);
            my @all_params = $param_set->walk();
            push(@print_stack, " Global ParamSet:");
            foreach my $paramset_entry (@all_params) {
                my $param = $paramset_entry->{key};
                my $value = $paramset_entry->{value};

                push(@print_stack, "  $param=$value");
            }

            push(@print_stack, "\n");
            push(@print_stack, " Tharn::Params Hash:");
            foreach my $param (sort keys %Tharn::Params) {
                my $value = &Tharn::param("$param");

                push(@print_stack, "  $param=$value");
            }

            # print out the print stack once it has been populated.
            my $print_string = join("\n", @print_stack);
            &SanUtil::nate_log('-comment', "$print_string");

            # validate that that the user has not violated the
            # "no tharnhost per client param" rules.
            &_validate_no_tharnhost_hsh();

        }, -runid, "$runid");

    return;
}

################################################################################
# Subroutine Name: get_tc_iteration_count()
# Objective:       Return to the user based on the platform group type
#                  what the iterations are for each of the test cases defined
#                  for CIT
################################################################################
sub get_tc_iteration_count {
    my ($test_case) = @_;

    my ($default, $tc_durations, $return);

    # if the group was not not passed or null then
    # we are to use the defined default values from CIT test plan
    my $group = &get_param("CIT_POLICY_HOST_TEAM");
    if (not defined($group) or $group eq "") {
        $group = "default";
    }

    # check to make sure the group is proper
    if ($group !~ /default|aix|hpux|windows|linux|esx|solaris/) {
        &SanUtil::nate_log('-comment', "SanCIT::get_tc_iteration_count: group (CIT_POLICY_HOST_TEAM=$group) is not valid. Must be one of (aix|hpux|windows|linux|esx|solaris) case sensitive");
        return undef;
    }

    # define TC duration counts on per group/platform basis

    # HA_CFO_take_give
    $default = 8;
    $tc_durations->{HA_CFO_take_give}->{default} = $default;
    $tc_durations->{HA_CFO_take_give}->{windows} = 4;
    $tc_durations->{HA_CFO_take_give}->{esx} = $default;
    $tc_durations->{HA_CFO_take_give}->{linux} = $default;
    $tc_durations->{HA_CFO_take_give}->{hpux} = $default;
    $tc_durations->{HA_CFO_take_give}->{solaris} = $default;
    $tc_durations->{HA_CFO_take_give}->{aix} = $default;

    # HA_CFO_forced_take_give
    $default = 8;
    $tc_durations->{HA_CFO_forced_take_give}->{default} = $default;
    $tc_durations->{HA_CFO_forced_take_give}->{windows} = 4;
    $tc_durations->{HA_CFO_forced_take_give}->{esx} = $default;
    $tc_durations->{HA_CFO_forced_take_give}->{linux} = $default;
    $tc_durations->{HA_CFO_forced_take_give}->{hpux} = $default;
    $tc_durations->{HA_CFO_forced_take_give}->{solaris} = $default;
    $tc_durations->{HA_CFO_forced_take_give}->{aix} = $default;

    # HA_CFO_panic_give
    $default = 8;
    $tc_durations->{HA_CFO_panic_give}->{default} = $default;
    $tc_durations->{HA_CFO_panic_give}->{windows} = 4;
    $tc_durations->{HA_CFO_panic_give}->{esx} = $default;
    $tc_durations->{HA_CFO_panic_give}->{linux} = $default;
    $tc_durations->{HA_CFO_panic_give}->{hpux} = $default;
    $tc_durations->{HA_CFO_panic_give}->{solaris} = $default;
    $tc_durations->{HA_CFO_panic_give}->{aix} = $default;

    # HA_CFO_reboot_give
    $default = 8;
    $tc_durations->{HA_CFO_reboot_give}->{default} = $default;
    $tc_durations->{HA_CFO_reboot_give}->{windows} = 4;
    $tc_durations->{HA_CFO_reboot_give}->{esx} = $default;
    $tc_durations->{HA_CFO_reboot_give}->{linux} = $default;
    $tc_durations->{HA_CFO_reboot_give}->{hpux} = $default;
    $tc_durations->{HA_CFO_reboot_give}->{solaris} = $default;
    $tc_durations->{HA_CFO_reboot_give}->{aix} = $default;

    # HA_CFO_poweroff_give
    $default = 8;
    $tc_durations->{HA_CFO_poweroff_give}->{default} = $default;
    $tc_durations->{HA_CFO_poweroff_give}->{windows} = 4;
    $tc_durations->{HA_CFO_poweroff_give}->{esx} = $default;
    $tc_durations->{HA_CFO_poweroff_give}->{linux} = $default;
    $tc_durations->{HA_CFO_poweroff_give}->{hpux} = $default;
    $tc_durations->{HA_CFO_poweroff_give}->{solaris} = $default;
    $tc_durations->{HA_CFO_poweroff_give}->{aix} = $default;

    # Host_stress_stop_start
    $default = 5;
    $tc_durations->{Host_stress_stop_start}->{default} = $default;
    $tc_durations->{Host_stress_stop_start}->{windows} = $default;
    $tc_durations->{Host_stress_stop_start}->{esx} = $default;
    $tc_durations->{Host_stress_stop_start}->{linux} = $default;
    $tc_durations->{Host_stress_stop_start}->{hpux} = $default;
    $tc_durations->{Host_stress_stop_start}->{solaris} = $default;
    $tc_durations->{Host_stress_stop_start}->{aix} = $default;

    # HA_IC_down_IO
    $default = 5;
    $tc_durations->{HA_IC_down_IO}->{default} = $default;
    $tc_durations->{HA_IC_down_IO}->{windows} = $default;
    $tc_durations->{HA_IC_down_IO}->{esx} = $default;
    $tc_durations->{HA_IC_down_IO}->{linux} = $default;
    $tc_durations->{HA_IC_down_IO}->{hpux} = $default;
    $tc_durations->{HA_IC_down_IO}->{solaris} = $default;
    $tc_durations->{HA_IC_down_IO}->{aix} = $default;

    # HA_cable_pull
    $default = 5;
    $tc_durations->{HA_cable_pull}->{default} = $default;
    $tc_durations->{HA_cable_pull}->{windows} = $default;
    $tc_durations->{HA_cable_pull}->{esx} = $default;
    $tc_durations->{HA_cable_pull}->{linux} = $default;
    $tc_durations->{HA_cable_pull}->{hpux} = $default;
    $tc_durations->{HA_cable_pull}->{solaris} = $default;
    $tc_durations->{HA_cable_pull}->{aix} = $default;

    # HA_port_blocks_host
    $default = 10;
    $tc_durations->{HA_port_blocks_host}->{default} = $default;
    $tc_durations->{HA_port_blocks_host}->{windows} = $default;
    $tc_durations->{HA_port_blocks_host}->{esx} = $default;
    $tc_durations->{HA_port_blocks_host}->{linux} = $default;
    $tc_durations->{HA_port_blocks_host}->{hpux} = $default;
    $tc_durations->{HA_port_blocks_host}->{solaris} = $default;
    $tc_durations->{HA_port_blocks_host}->{aix} = $default;

    # HA_port_blocks_controller
    $default = 20;
    $tc_durations->{HA_port_blocks_controller}->{default} = $default;
    $tc_durations->{HA_port_blocks_controller}->{windows} = $default;
    $tc_durations->{HA_port_blocks_controller}->{esx} = $default;
    $tc_durations->{HA_port_blocks_controller}->{linux} = $default;
    $tc_durations->{HA_port_blocks_controller}->{hpux} = $default;
    $tc_durations->{HA_port_blocks_controller}->{solaris} = $default;
    $tc_durations->{HA_port_blocks_controller}->{aix} = $default;

    # Host_stress_CPU
    $default = 5;
    $tc_durations->{Host_stress_CPU}->{default} = $default;
    $tc_durations->{Host_stress_CPU}->{windows} = $default;
    $tc_durations->{Host_stress_CPU}->{esx} = $default;
    $tc_durations->{Host_stress_CPU}->{linux} = $default;
    $tc_durations->{Host_stress_CPU}->{hpux} = $default;
    $tc_durations->{Host_stress_CPU}->{solaris} = $default;
    $tc_durations->{Host_stress_CPU}->{aix} = $default;

    # Controller_stress
    $default = 5;
    $tc_durations->{Controller_stress}->{default} = $default;
    $tc_durations->{Controller_stress}->{windows} = $default;
    $tc_durations->{Controller_stress}->{esx} = $default;
    $tc_durations->{Controller_stress}->{linux} = $default;
    $tc_durations->{Controller_stress}->{hpux} = $default;
    $tc_durations->{Controller_stress}->{solaris} = $default;
    $tc_durations->{Controller_stress}->{aix} = $default;

    # Host_max_paths
    $default = 4;
    $tc_durations->{Host_max_paths}->{default} = $default;
    $tc_durations->{Host_max_paths}->{windows} = 2;
    $tc_durations->{Host_max_paths}->{esx} = $default;
    $tc_durations->{Host_max_paths}->{linux} = $default;
    $tc_durations->{Host_max_paths}->{hpux} = $default;
    $tc_durations->{Host_max_paths}->{solaris} = $default;
    $tc_durations->{Host_max_paths}->{aix} = $default;

    # NDU_Host
    $default = 5;
    $tc_durations->{NDU_Host}->{default} = $default;
    $tc_durations->{NDU_Host}->{windows} = 2;
    $tc_durations->{NDU_Host}->{esx} = $default;
    $tc_durations->{NDU_Host}->{linux} = $default;
    $tc_durations->{NDU_Host}->{hpux} = $default;
    $tc_durations->{NDU_Host}->{solaris} = $default;
    $tc_durations->{NDU_Host}->{aix} = $default;

    # HA_dedupe
    $default = 3;
    $tc_durations->{HA_dedupe}->{default} = $default;
    $tc_durations->{HA_dedupe}->{windows} = $default;
    $tc_durations->{HA_dedupe}->{esx} = $default;
    $tc_durations->{HA_dedupe}->{linux} = $default;
    $tc_durations->{HA_dedupe}->{hpux} = $default;
    $tc_durations->{HA_dedupe}->{solaris} = $default;
    $tc_durations->{HA_dedupe}->{aix} = $default;

    # HA_Host_Cluster
    $default = 5;
    $tc_durations->{HA_Host_Cluster}->{default} = $default;
    $tc_durations->{HA_Host_Cluster}->{windows} = $default;
    $tc_durations->{HA_Host_Cluster}->{esx} = $default;
    $tc_durations->{HA_Host_Cluster}->{linux} = $default;
    $tc_durations->{HA_Host_Cluster}->{hpux} = $default;
    $tc_durations->{HA_Host_Cluster}->{solaris} = $default;
    $tc_durations->{HA_Host_Cluster}->{aix} = $default;

    # formulate return
    $return = $tc_durations->{$test_case}->{$group};
    &SanUtil::nate_log('-comment', "SanCIT::get_tc_iteration_count: $test_case iterations for group ($group) was determined to be $return");

    return($return);

}

################################################################################
# Subroutine Name: display_test_results()
# Objective:       Display the test results by parsing the results object
################################################################################
sub display_test_results {
    my ($results) = @_;

    return;
}

################################################################################
# Subroutine Name: run_vm_io()
# Objective:       Run the vm io infrastructure thpl
################################################################################
sub run_vm_io {
    my (%args) = @_;
    my $bg = 0;
    my $runid;
    my $vm_io_subtest;
    my @params = ();
    my $handler_ref;

    # parse arguments
    foreach (%args) {
        if (/-?bg/) {
            if (not length($bg = $args{$_})) {
                &SanUtil::nate_log('-script', "Error: SanCIT::run_vm_io() '-bg' value was empty");
                return {};
            }
        } elsif (/-?runid/) {
            $runid = $args{$_};
        } elsif (/-?params/) {
            @params = @{$args{$_}};
        } elsif (/-?handler/) {
            if (ref($handler_ref = $args{$_}) ne "CODE") {
                &SanUtil::nate_log('-script', "Error: SanCIT::run_vm_io() '-handler' value is not a valid code reference.");
            }
        }
    }

    unless (defined($runid)) {
        $runid = "vm_io";
    }

    # build out the arguments to the subtest invocation
    my @subtest_args = ();
    push(@subtest_args, "SANInterop/io/vm_io.thpl");
    push(@subtest_args, @params) if (scalar(@params));
    push(@subtest_args, "-runid", "$runid");

    # run vm_io in an eval block
    eval {
        # if running in the background, then we have to reap the subtest afterwords
        if ($bg == 1) {
            # run vm_io in the background with ipc comm
            $vm_io_subtest = subtest(@subtest_args, -bg, -ipc);

            # periodically check for messages from vm_io
            while ($vm_io_subtest->is_alive()) {
                if ($vm_io_subtest->ipc()->can_read()) {
                    my $vm_io_results = $vm_io_subtest->ipc()->read_obj();

                    # run the handler function with the results if we were passed in a valid handler
                    if (defined $handler_ref and ref($handler_ref) eq "CODE") {
                        &$handler_ref($vm_io_results);
                    }

                    # tell vm_io to continue
                    $vm_io_subtest->ipc()->write_message("continue");

                }
                &SanUtil::nate_sleep(30, -process_results => 0);
            }
        } else {
            # run vm_io in the foreground
            $vm_io_subtest = subtest(@subtest_args);
        }
    };

    # It may occur that vm_io has not completely exited, in that case, wait for it to finish.
    eval {
        $vm_io_subtest->wait_finish();
    };

    return $vm_io_subtest;
}

################################################################################
# Subroutine Name: ndisp_store_results()
# Objective:       Store vm_io_result parameters in the ndisp config file
#                  if ndisp is being used.
################################################################################
sub ndisp_store_results {
    my ($vm_io_results) = @_;

    # if requested to not store ndisp information from tests on the ndisp server
    # return
    if (&get_param("CIT_DONT_STORE_NDISP_DATA") == 1) {
        return;
    }

    # if ndisp host and port are set, the store the results on the server
    if (&is_param_defined("NDISPD_HOST") and
        &is_param_defined("NDISPD_PORT")) {

        # if nothing to do, return
        if (not defined($vm_io_results)) {
            &SanUtil::nate_log('-comment', "No parameters were available to store to the NDISP server.");
            return;
        }

        my ($ndispd_host) = &get_param("NDISPD_HOST");
        my ($ndispd_port) = &get_param("NDISPD_PORT");

        # create the ndisp object
        my $ndispd = ndisp::API->new("$ndispd_host\:$ndispd_port");
        if (not defined ($ndispd)) {
            &cit_error('-fatal', "Error: Could not instantiate an NDISP object at $ndispd_host\:$ndispd_port\n" .
                                 "No parameters were stored.");
            return;
        }

        # store all the params into the ndisp server
        foreach my $param (keys %{$vm_io_results->{params}}) {
            # pull out the value
            my $value = $vm_io_results->{params}->{$param};

            # store the parameter and value on the ndisp server
            &set_ndispd_config_param($ndispd, "$param", "$value");
        }

        # uninitialize the ndisp object
        $ndispd = {};
    } else {
        # otherwise user is not running ndisp, just continue
        &SanUtil::nate_log('-comment', "An NDISP server is not being used, continuing.");
    }

    return;
}

################################################################################
# Subroutine Name: get_tcd_results()
# Objective:       determine if the test passed/warned/failed
################################################################################
sub get_tcd_results {
    my ($subtest, $subtest_name) = @_;

    # default the subtest name to vm_io.thpl
    if ($subtest_name eq "") {
        $subtest_name = "vm_io.thpl";
    }

    $SanCIT::VM_IO_EXIT_STATUS = "fail";

    if (not defined ($subtest)) {
        # failure
        $SanCIT::VM_IO_EXIT_STATUS = "fail";
        return $TCD::FATAL;
    } else {
        # get results objects from all subtests
        # determine "worst" result
        my @results = $subtest->results();
        my $worst_result = NATE::Result::worst(@results);
        my $our_result_severity = $worst_result->severity();

        if ($our_result_severity == 0) {
            # pass result/severity
            $SanCIT::VM_IO_EXIT_STATUS = "pass";
        } elsif ($our_result_severity == 1) {
            # warn result/severity
            &SanUtil::nate_log('-warn', "$subtest_name issued a warning. Please " .
                                        "investigate the Warnings/Errors in the $subtest_name logs.");
            $SanCIT::VM_IO_EXIT_STATUS = "warn";
        } else {
            # failure result/severity
            $SanCIT::VM_IO_EXIT_STATUS = "fail";
        }

        # if not for score mode, put out a message stating this,
        # fail the test case (no abortive), but print out what the status would
        #  have been in the message.
        if (&get_param("CIT_NOT_FOR_SCORE") == 1) {
            &SanUtil::nate_log('-fail', "CIT NOT FOR SCORE MODE\n" .
                               "#################################################################\n" .
                               "#################################################################\n" .
                               "#################################################################\n" .
                               "#####          ** CIT NOT FOR SCORE MODE **                ######\n" .
                               "#####          - Failing test case because of this         ######\n" .
                               "#####          - Test status (if not NOT FOR SCORE) would  ######\n" .
                               "#####            have been ($SanCIT::VM_IO_EXIT_STATUS)    ######\n" .
                               "#####          ** THESE RESULTS ARE NOT VALID FOR USE **   ######\n" .
                               "#####          ** IN STANDARD QUALIFICATION **             ######\n" .
                               "#################################################################\n" .
                               "#################################################################\n" .
                               "#################################################################\n" .
                               "\n",
                               -abortive => 0);
            return $TCD::FATAL;
        }

        # return TCD status (no warning result)
        if ($SanCIT::VM_IO_EXIT_STATUS =~ /pass|warn/) {
            return $TCD::PASS;
        } else {
            return $TCD::FATAL;
        }
    }

    return;

}

################################################################################
# Subroutine Name: uninit_test()
# Objective:       Run a common routine to uninitialize the test
################################################################################
sub uninit_test {
    my ($test_exit_status) = @_;

    # store the test result on the ndisp server if it is running
    &_ndisp_store_test_result($test_exit_status);

    return;
}

################################################################################
# Subroutine Name: uninit_test_cmode()
# Objective:       Run a common routine to uninitialize the test for cmode
################################################################################
sub uninit_test_cmode {
    my (%args) = @_;

    my $worst_result = exists($args{-worst_result}) ? $args{-worst_result} : $TCD::PASS;
    my $eventman_obj = exists($args{-eventman_obj}) ? $args{-eventman_obj} : undef;
    my $graceful_abort = exists($args{-graceful_abort}) ? $args{-graceful_abort} : 0;

    # uninitialize the test with the worst result
    &SanCIT::uninit_test($worst_result);

    # if eventman is still running at this point, then abort it gracefully or non gracefully
    if (defined($eventman_obj) and $eventman_obj->is_running()) {
        # we will get into this state only if I/O Failed, tell eventman that it should abort
        # in this case.
        $eventman_obj->abort_events(-graceful => $graceful_abort);
        if ($eventman_obj->error()) {
            &cit_error('-warn', "Error: SanCIT::uninit_test_cmode: Failed to abort events\n", $eventman_obj->errmsg());
            return $TCD::FAIL;
        }
    }

    return $TCD::PASS;
}

################################################################################
# Subroutine Name: get_ndispd_config_param()
# Objective:       obtain the value of a parameter on the ndisp object
################################################################################
sub get_ndispd_config_param {
    my ($ndispd, $param, $ignore_error) = @_;

    my $output = $ndispd->call("list config", []);
    foreach my $key (keys %{$output}) {
        if ($key eq "$param") {
            return $output->{$key};
        }
    }

    return "";
}

################################################################################
# Subroutine Name: set_ndispd_config_param()
# Objective:       Set the value of a parameter on the ndisp object
################################################################################
sub set_ndispd_config_param {
    my ($ndispd, $param, $value, $ignore_error) = @_;

    my $display_value = $value;
    $display_value =~ s/^NDISPD;//;

    &SanUtil::nate_log('-comment', "ndispd: setting ${param}=${display_value}");

    if (not $ndispd->call("set config", {$param => $value})) {
        if ($ignore_error ne "ignore") {
            &cit_error('-fatal', "Error: SanCIT::set_ndispd_config_param(): Could not set config '$param' to '$value'.");
        } else {
            &SanUtil::nate_log('-warn', "Error: SanCIT::set_ndispd_config_param(): Could not set config '$param' to '$value'.");
            return 0; # fail
        }
    } else {
        return 1; # success
    }
}

################################################################################
# Subroutine Name: unset_ndispd_config_param()
# Objective:       Unset the value of a parameter on the ndisp object
################################################################################
sub unset_ndispd_config_param {
    my ($ndispd, $param, $ignore_error) = @_;

    &SanUtil::nate_log('-comment', "ndispd: unsetting $param");

    if (not $ndispd->call("unset config", ["$param"])) {
        return 0; # fail
    } else {
        return 1; # success
    }
}

################################################################################
# Subroutine Name: get_per_client_params()
# Objective:       This will get clients per-client pararms
################################################################################
sub get_per_client_params {
    my $clients = &get_param("CLIENTS");
    my $vm_clients = &get_param("VM_CLIENTS");
    my $hv_clients = &get_param("HV_CLIENTS");
    my $kvm_clients = &get_param("KVM_CLIENTS");
    my $ovm_clients = &get_param("OVM_CLIENTS");
    my $citrixxs_clients = &get_param("CITRIXXS_CLIENTS");

    my $v_hosts = "";
    my $ret_hash = {};

    # parse the vm_clients string, this will update $clients to include
    # virtual machines
    if ($vm_clients ne "" or $hv_clients ne "" or $kvm_clients ne "" or $ovm_clients ne "" or $citrixxs_clients ne "") {
        ($v_hosts, $clients) = &_parse_virtual_clients($vm_clients, $hv_clients, $kvm_clients, $ovm_clients,
                                                       $citrixxs_clients, $clients);
    }

    # build the entire clients array.
    my @all_clients = (split(/:/, $clients), split(/:/, $v_hosts));

    # loop through all the clients
    foreach my $client (sort @all_clients) {
        next if ($client =~ /^\s*$/);

        my $client_tharnhost_env = &ClientUtil::read_client_params($client, "PC_");
        $ret_hash->{$client} = $client_tharnhost_env;
    }

    return ($ret_hash);
}

################################################################################
# Subroutine Name: get_protocol_usage()
# Objective:       This will get return back which protocols are being used
#                  (globalprotocol, haveiscsi, havefcp)
################################################################################
sub get_protocol_usage {
    my $global_protocol = &get_param("PROTOCOL");
    my $have_iscsi_client = 0;
    my $have_fcp_client = 0;
    my (@fcp_clients, @iscsi_clients);

    foreach my $client (keys %{$SanCIT::CLIENT_PARAM_HASH}) {
        # if this client doesnt have a protocol defined on a per client level
        # assume the global protocol is its value
        if (!defined($SanCIT::CLIENT_PARAM_HASH->{$client}->{'PROTOCOL'})) {
            if ($global_protocol =~ /iscsi/i) {
                push(@iscsi_clients, $client);
            } elsif ($global_protocol =~ /fcp/i) {
                push(@fcp_clients, $client);
            }
            next;
        }

        my $pc_protocol = $SanCIT::CLIENT_PARAM_HASH->{$client}->{'PROTOCOL'};

        if ($pc_protocol =~ /iscsi/i) {
            push(@iscsi_clients, $client);
            $have_iscsi_client = 1;
        }

        if ($pc_protocol =~ /fcp/i) {
            push(@fcp_clients, $client);
            $have_fcp_client = 1;
        }
    }

    # store all the values in the protocol hash
    my $protocol_hash = {};
    $protocol_hash->{global_protocol} = $global_protocol;
    $protocol_hash->{have_iscsi_client} = $have_iscsi_client;
    $protocol_hash->{iscsi_clients} = \@iscsi_clients;
    $protocol_hash->{have_fcp_client} = $have_fcp_client;
    $protocol_hash->{fcp_clients} = \@fcp_clients;

    return $protocol_hash;
}

################################################################################
# Subroutine Name: any_iscsi_clients()
# Objective:       returns 1 if any clients are iscsi, 0 if not
################################################################################
sub any_iscsi_clients {
    my ($protocol_hash) = &get_protocol_usage();
    if ($protocol_hash->{global_protocol} =~ /iscsi/i or
        $protocol_hash->{have_iscsi_clients} == 1) {
        return 1;
    } else {
        return 0;
    }
}

################################################################################
# Subroutine Name: any_fcp_clients()
# Objective:       returns 1 if any clients are fcp, 0 if not
################################################################################
sub any_fcp_clients {
    my ($protocol_hash) = &get_protocol_usage();
    if ($protocol_hash->{global_protocol} =~ /fcp/i or
        $protocol_hash->{have_fcp_clients} == 1) {
        return 1;
    } else {
        return 0;
    }
}

################################################################################
# Subroutine Name: cit_error()
# Objective:       Kill the test with a warn/config|fatal error.
################################################################################
sub cit_error {
    my ($type, @msg) = @_;

    my $errmsg = join("\n", @msg);

    &SanUtil::nate_log('-warn', "$errmsg");

    if ($type ne "-warn") {
        &SanUtil::nate_log($type, "$errmsg");
    }

    return;
}

################################################################################
# Subroutine Name: get_p4_dir
# Objective: This function will dynamically obtain the p4 directory of the test
################################################################################
sub get_p4_dir {
    my (%args) = @_;

    my $fallback_dir = exists($args{-fallback_dir}) ? $args{-fallback_dir} : undef;

    my $p4_dir;

    # (1) try to determine from NATE_LIB/NATE_LIB_UNIX
    my @natelib;
    if (exists($ENV{"NATE_LIB"})) {
        @natelib = split(/;/, $ENV{"NATE_LIB"});
    }
    if (not scalar(@natelib) and exists($ENV{"NATE_LIB_UNIX"})) {
        @natelib = split(/;/, $ENV{"NATE_LIB_UNIX"});
    }

    # walk through all the natelib paths until we find one that works
    # all p4 clients that are using SANCIT should also have the EnGenX/DTDFiles
    foreach my $natelib_path (@natelib) {
        my $dtd_path = $natelib_path . "/lib/EnGenX/DTDFiles";
        if (-d $dtd_path) {
            return $natelib_path;
        }
    }

    # (2) try to determine the path based on the TESTNAME variable
    my $testname = $ENV{"TESTNAME"};
    if ($testname =~ /^(\/.+?\/p4\/.+?)\//) {
        $p4_dir = $1;

        # make sure this path exists, otherwise dont use it
        if (-d $p4_dir) {
            return $p4_dir;
        }
    }

    # (3) If we can't determine the P4 and $fallback_dir is defined, then return that.
    if (defined($fallback_dir)) {
        return $fallback_dir;
    }

    # (4) otherwise, default to sanrtest's version
    $p4_dir = "/u/sanrtest/p4/test";

    return $p4_dir;
}

################################################################################
# Subroutine Name: initialize_eventman
# Objective: Initialize EventMan inside SanMgr by loading/validating XML
################################################################################
sub initialize_eventman {
    my (%args) = @_;

    my $test_engine_obj = exists($args{-sanmgr_obj}) ? $args{-sanmgr_obj} : undef;
    my $xml_files = exists($args{-xml_files}) ? $args{-xml_files} : [];
    my $dtd_files = exists($args{-dtd_files}) ? $args{-dtd_files} : [];

    if (not defined($test_engine_obj)) {
        &cit_error('-fatal', "SanCIT::initialize_eventman: Missing or invalid value for -sanmgr_obj");
        return undef;
    }

    # if a set of xml files and dtd files were passed in, then
    # unset eventman from the paramset and reload the xml and dtd files into
    # SanMgr
    if (scalar(@{$xml_files})) {
        require ParamsMgr;
        &ParamsMgr::unset("EVENTMAN");

        # load in the Interop DTD File hierarchy
        my @dtd_list = &SanCIT::get_interop_dtd_file_list();
        unshift(@{$dtd_files}, @dtd_list);

        # load the xml/dtd for this testcase
        $test_engine_obj->load_xml(-xml_files => $xml_files,
                              -schema_files => $dtd_files,
                              -display_xml => 1,
                              -display_xml_runid => "XML-EventMan");
        if ($test_engine_obj->error()) {
            &cit_error('-fatal', "SanCIT::initialize_eventman: Error while loading XML for EventMan\n", $test_engine_obj->errmsg());
            return undef;
        }
    }

    # initialize EventMan inside sanmgr
    $test_engine_obj->initialize_eventman();
    if ($test_engine_obj->error()) {
        &cit_error('-fatal', "SanCIT::initialize_eventman: Error while initializing SanMgr EventMan\n", $test_engine_obj->errmsg());
        return undef;
    }

    my $eventman_obj = undef;
    my $err_msg = "";
    # extract the generated eventman object
    if ($test_engine_obj->isa("San::SanMgr")) {
        ($eventman_obj) = $test_engine_obj->get_objects(-type => "eventman");
        if ($test_engine_obj->error()) {
            &cit_error('-fatal', "SanCIT::initialize_eventman: Error while obtaining EventMan object from SanMgr\n" , $test_engine_obj->errmsg());
            return undef;
        }
    } elsif ($test_engine_obj->isa("EnGenX")) {
        try {
            ($eventman_obj) = $test_engine_obj->get_objects(type => "eventman");
        } otherwise {
            my $exception = shift;
            $err_msg = "EnGenX get_objects exception: " . $exception->text();
        };

        if ($err_msg ne "") {
            &cit_error('-fatal', "SanCIT::initialize_eventman: Error while obtaining EventMan object from EnGenX \n" , $err_msg);
            return undef;
        }
    } else {
        &cit_error('-fatal', "SanCIT::initialize_eventman: Error invalid test engine object. A " .
                   ref($test_engine_obj) . " was passed in.");
        return undef;
    }
    return $eventman_obj;
}

################################################################################
# Subroutine Name: initialize_cmode_iteration_counts
# Objective: Initialize cmode iteration count variables
################################################################################
sub initialize_cmode_iteration_counts {
    my $nodes = &SanCIT::get_param("NODES");
    my @nodes = split(",", $nodes);

    # iteration counts are defined by burt586970
    my $iter_count;

    if (&SanCIT::get_param("TESTNAME") =~ /SFO/) {
        # 8 node - iteration count = 2
        # 6 node - iteration count = 3
        # 4 node - iteration count = 4
        # 2 node - iteration count = 8
        if (scalar(@nodes) == 2) {
            $iter_count = 8;
        } elsif (scalar(@nodes) == 4) {
            $iter_count = 4;
        } elsif (scalar(@nodes) == 6) {
            $iter_count = 3;
        } elsif (scalar(@nodes) == 8) {
            $iter_count = 2;
        } else {
            $iter_count = 8;
        }

        &SanCIT::discretionary_param("CIT_CM_SFO_CYCLE_COUNT", $iter_count);
    } elsif (&SanCIT::get_param("TESTNAME") =~ /HA_regression/i) {
        # 8 node - iteration count = 2
        # 6 node - iteration count = 3
        # 4 node - iteration count = 4
        # 2 node - iteration count = 8
        if (scalar(@nodes) == 2) {
            $iter_count = 3;
        } elsif (scalar(@nodes) == 4) {
            $iter_count = 2;
        } elsif (scalar(@nodes) == 6) {
            $iter_count = 1;
        } elsif (scalar(@nodes) == 8) {
            $iter_count = 1;
        } else {
            $iter_count = 1;
        }

        &SanCIT::discretionary_param("CIT_CM_REGRESSION_CYCLE_COUNT", $iter_count);
    }

    return;
}

################################################################################
# Subroutine Name: initialize_sanmgr
# Objective: Initialize a SanMgr object for use
################################################################################
sub get_interop_dtd_file_list {
    my (@dtd_file_list) = ();

    # load in the Interop DTD File hierarchy if specified
    &SanCIT::discretionary_param("CIT_LOAD_INTEROP_DTD_FILES", 1);
    if (!&SanCIT::get_param("CIT_LOAD_INTEROP_DTD_FILES")) {
        return @dtd_file_list;
    }

    # First load in the Interop DTD file
    &SanCIT::hardcode_param("INTEROP_DTD_FILE", &SanCIT::get_p4_dir() . "/SANInterop/cit/cmode/dtd/interop.dtd");
    push(@dtd_file_list, &SanCIT::get_param("INTEROP_DTD_FILE"));

    # If we are running a CIT testcase, then load in the cit mode dtd file
    if (&SanCIT::get_param("TESTNAME") =~ /SANInterop\/cit\/cmode\/.*\.thpl|SANInterop\/cit\/fcswitch\/cmode/) {
        &SanCIT::hardcode_param("CIT_CMODE_DTD_FILE", &SanCIT::get_p4_dir() . "/SANInterop/cit/cmode/dtd/cit_cmode.dtd");
        push(@dtd_file_list, &SanCIT::get_param("CIT_CMODE_DTD_FILE"));
    } elsif (&SanCIT::get_param("TESTNAME") =~ /SANInterop\/io\/sm_io.thpl/) {
        &SanCIT::hardcode_param("SM_IO_DTD_FILE", &SanCIT::get_p4_dir() . "/SANInterop/cit/cmode/dtd/sm_io.dtd");
        push(@dtd_file_list, &SanCIT::get_param("SM_IO_DTD_FILE"));
    }
    # If the test case is a Netapp Proven testcase, load the NEAP
    # dtd file which contains settings common to all NEAP testcases
    if (&SanCIT::get_param("TESTNAME") =~ /SANInterop\/cit\/neap\/cmode\/.*\.thpl/) {
        &SanCIT::hardcode_param("NEAP_DTD_FILE", &SanCIT::get_p4_dir() . "/SANInterop/cit/neap/cmode/dtd/neap.dtd");
        push(@dtd_file_list, &SanCIT::get_param("NEAP_DTD_FILE"));
    }
    # If the test case is a fcswitch testcase, load the fcswitch
    # dtd file which contains settings common to all fcswitch testcases
    if (&SanCIT::get_param("TESTNAME") =~ /SANInterop\/cit\/fcswitch\/cmode\/.*\.thpl/) {
        &SanCIT::hardcode_param("FCSWITCH_DTD_FILE", &SanCIT::get_p4_dir() . "/SANInterop/cit/fcswitch/cmode/dtd/fcswitch.dtd");
        push(@dtd_file_list, &SanCIT::get_param("FCSWITCH_DTD_FILE"));
    }

    return @dtd_file_list;
}

################################################################################
# Subroutine Name: initialize_test_engine
# Objective: Initialize a test engine, either sanmgr or EnGenX
################################################################################
sub initialize_test_engine {
    my (%args) = @_;
    my $test_engine = exists($args{-test_engine}) ? $args{-test_engine} : "SANMGR";
    my $test_engine_obj;

    my %nodes_hash = %{&SanCIT::load_ha_list()};
    if ($test_engine =~ /ENGENX/i) {
       $test_engine_obj = &initialize_engenx(%args,%nodes_hash);
    } else {
       $test_engine_obj = &initialize_sanmgr(%args,%nodes_hash);
    }

    return $test_engine_obj;
}

################################################################################
# Subroutine Name: initialize_sanmgr
# Objective: Initialize a SanMgr object for use
################################################################################
sub initialize_sanmgr {
    my (%args) = @_;

    my $xml_file  = exists($args{-xml_file}) ? $args{-xml_file} : "";
    my $dtd_files = exists($args{-dtd_files}) ? $args{-dtd_files} : [];

    my %extra_vars = ();
    $extra_vars{clusters} = $args{clusters} if (exists($args{clusters}));
    $extra_vars{nodes} = $args{nodes} if (exists($args{nodes}));

    if ($xml_file eq "") {
        &cit_error('-fatal', "SanCIT::initialize_sanmgr: Missing or invalid value for -xml_file");
        return undef;
    }

    # load in all parameters relevant to CIT
    &SanCIT::load_discretionary_params(-bypass_fvl_check => 1);
    &SanCIT::load_hardcode_params();
    &SanCIT::hardcode_param("PRE_CLEANUP", 0);
    &SanCIT::load_special_params();

    # burt586970
    # Change default cycle counts for SFO testcases
    &SanCIT::initialize_cmode_iteration_counts();

    # load in the Interop DTD File hierarchy
    my @dtd_list = &SanCIT::get_interop_dtd_file_list();
    unshift(@{$dtd_files}, @dtd_list);

    # display the cit environment
    &SanCIT::print_environment_params("000_CIT-environment", -unset_clients => 1);

    require San::SanMgr;

    # create a SanMgr object
    my $sanmgr_obj = new San::SanMgr(-xml_file => $xml_file,
                                     -schema_files => $dtd_files,
                                     -load_resources => 1,
                                     -reconnect_client_objects_after_load => 0,
                                     -reconnect_switch_objects_after_load => 0,
                                     -reconnect_hypervisor_objects_after_load => 0,
                                     %extra_vars);
    if ($sanmgr_obj->error()) {
        &cit_error('-fatal', "SanCIT::initialize_sanmgr: " . $sanmgr_obj->errmsg());
        return undef;
    }

    return $sanmgr_obj;
}

################################################################################
# Subroutine Name: initialize_engenx
# Objective: Initialize an EnGenX object for use
################################################################################
sub initialize_engenx {
    my (%args) = @_;

    my $xml_file = exists($args{-xml_file}) ? $args{-xml_file} : "";
    my $dtd_files = exists($args{-dtd_files}) ? $args{-dtd_files} : [];

    my %extra_vars = ();
    $extra_vars{clusters} = $args{clusters} if (exists($args{clusters}));
    $extra_vars{nodes} = $args{nodes} if (exists($args{nodes}));

    if ($xml_file eq "") {
        &cit_error('-fatal', "SanCIT::initialize_engenx: Missing or invalid value for -xml_file");
        return undef;
    }

    # load in all parameters relevant to CIT
    &SanCIT::load_discretionary_params(-bypass_fvl_check => 1);
    &SanCIT::load_hardcode_params();
    &SanCIT::hardcode_param("PRE_CLEANUP", 0);
    &SanCIT::load_special_params();

    # burt586970
    # Change default cycle counts for SFO testcases
    &SanCIT::initialize_cmode_iteration_counts();

    # load in the Interop DTD File hierarchy
    my @dtd_list = &SanCIT::get_interop_dtd_file_list();
    unshift(@{$dtd_files}, @dtd_list);

    # display the cit environment
    &SanCIT::print_environment_params("000_CIT-environment", -unset_clients => 1);

    require EnGenX;
    my $exception = undef;
    my $engenx_obj = undef;
    try {
        $engenx_obj = new EnGenX(file => $xml_file,
                                 schema_files => $dtd_files,
                                 load_resources => 1,
                                 %extra_vars);
    } otherwise {
        $exception = shift;
    };

    if ($exception ne undef) {
        &cit_error('-fatal', "SanCIT::initialize_engenx: EnGenx::new ", $exception->text());
        return undef;
    }

    return $engenx_obj;
}

################################################################################
# Subroutine Name: initialize_neap
# Objective: Initialize NEAP
################################################################################
sub initialize_neap {
    # burt673374 (set min tuning level to 45 and make it discretionary)
    &SanCIT::discretionary_param("TUNE_IO_DISK_MIN", "45");
}

################################################################################
# Subroutine Name: initialize_fcswitch
# Objective: Initialize FCSwitch
################################################################################
sub initialize_fcswitch {
    # burt683675 (set min and max tuning level to discretionary)
    &SanCIT::discretionary_param("TUNE_IO_DISK_MIN", "45");
    &SanCIT::discretionary_param("TUNE_IO_DISK_MAX", "85");
}

################################################################################
# Subroutine Name: get_fault_port_block_count
# Objective: returns the number of ports to block based on the number of nodes
#            in the cluster. Return 0 will block all ports once.
################################################################################
sub get_fault_port_block_port_count {
    # burt695475 (determine port block count based on number of nodes)

    # get the number of nodes in the cluster
    my @nodes = split(',', &SanCIT::get_param("NODES"));
    my $node_count = @nodes;

    # if there are 4 or less nodes, block 20 ports.
    # otherwise return 0 to block all ports.
    return $node_count <= 4 ? 20 : 0;
}
################################################################################
################################################################################
sub run_host_io {
    my (%args) = @_;

    # parse out arguments
    my $test_engine_obj = exists($args{-sanmgr_obj}) ? $args{-sanmgr_obj} : undef;
    if (not defined($test_engine_obj)) {
        &cit_error('-fatal', "SanCIT::run_host_io: Missing or invalid value for -sanmgr_obj");
        return undef;
    }

    my $tcd_obj = exists($args{-tcd_obj}) ? $args{-tcd_obj} : undef;

    my $cdl = undef;
    # parse out the client dev list if it is stored on the sanmgr object
    try {
        $cdl = $test_engine_obj->get_client_dev_list(-cdl_format => 1);
    } otherwise {
        # No action required, client dev list is allowed to be undefined.
    };

    if ($test_engine_obj->error() or not defined($cdl)) {
        $test_engine_obj->_clear_err();
        # sanmgr didnt have a client dev list for us to work with, require that
        # the user specify it
        eval {
            &SanCIT::mandatory_param("CLIENT_DEV_LIST", "CLIENT_DEV_LIST must be specified");
        };
        if ($@) {
            return undef;
        }

        # remove any NDISPD; flags from the beginning of client_dev_list
        # and update the cdl value
        $cdl = &SanCIT::get_param("CLIENT_DEV_LIST");
        $cdl =~ s/^\s*NDISPD;//g;
        &SanCIT::set_param("CLIENT_DEV_LIST", $cdl);
    } else {
        # otherwise set the client dev list tharn param to be
        # discretionary with the default value being the
        # internal cdl stored in SanMgr
        &SanCIT::discretionary_param("CLIENT_DEV_LIST", $cdl);
        if (not $cdl_displayed) {
            $cdl_displayed = 1;
            &SanUtil::nate_log('-comment', "Setting CLIENT_DEV_LIST=$cdl");
            &SanUtil::nate_log('-comment', Dumper($cdl));
        }

        # check the client dev list, if it has the ndispd flag on it, then it originated from the ndisp server
        # If that is the case, then override it with sanmgr's CDL.
        if (&SanCIT::get_param("CLIENT_DEV_LIST") =~ /^\s*NDISPD;/) {
            # force the current CLIENT_DEV_LIST value to be that from sanmgr
            &SanCIT::set_param("CLIENT_DEV_LIST", $cdl);
        } else {
            # otherwise, the parameter was set either discretionally by us, or it was overridden by the user
        }
    }

    # store the client dev list on the ndisp server
    my $results;
    my $storable_cdl = &SanCIT::get_param("CLIENT_DEV_LIST");
    $storable_cdl =~ s/^\s*NDISPD;//g;
    $results->{params}->{CLIENT_DEV_LIST} = "NDISPD;" . $storable_cdl;
    &SanCIT::ndisp_store_results($results);

    # parse the client names from the client dev list
    my @client_dev_lists = split(/;/, $storable_cdl);
    my @clients = map {
        if (/^(\S+?)\,/) {
            $1;
        }
    } @client_dev_lists;


    # build the CLIENTS variable to pass into host_io.thpl
    my $client_names;
    foreach my $client (@clients) {
        $client_names .= $client . ":";
    }
    chop($client_names);

    my $cmode_cluster_names = "";
    my @cmode_cluster_objs;
    my $err_msg = "";
    if ($test_engine_obj->isa("San::SanMgr")) {
        @cmode_cluster_objs = $test_engine_obj->get_objects(-type => "cluster");
        if ($test_engine_obj->error()) {
            &cit_error('-fatal', "SanCIT::run_host_io: Error while obtaining Cluster object from SanMgr\n" , $test_engine_obj->errmsg());
            return undef;
        }
    } elsif ($test_engine_obj->isa("EnGenX")) {
        try {
            @cmode_cluster_objs = $test_engine_obj->get_objects(type => "cluster");
        } otherwise {
            my $exception = shift;
            $err_msg = "EnGenX get_objects exception: " . $exception->text();
        };

        if ($err_msg ne "") {
            &cit_error('-fatal', "SanCIT::run_host_io: Error while obtaining Cluster object from EnGenX \n" , $err_msg);
            return undef;
        }
    } else {
        &cit_error('-fatal', "SanCIT::run_host_io: Error invalid test engine object. A " .
                   ref($test_engine_obj) . " was passed in.");
        return undef;
    }

    foreach my $cluster_obj (@cmode_cluster_objs) {
        # Get the list of nodes within the cluster and grab one node to be the 'primary' node
        if ($cluster_obj->isa("NACL::C::Cluster")) {
            my @node_names = $cluster_obj->nodes();
            my ($node_name) = shift(@node_names);
            $cmode_cluster_names .= $node_name . ":";
        } else {
            my @node_objs = $cluster_obj->get_node_objs();
            my ($node) = shift(@node_objs);
            $cmode_cluster_names .= $node->get_name() . ":";
        }
    }
    chop($cmode_cluster_names);

    &SanUtil::nate_log('-comment', "Running I/O to CLIENTS=$client_names");
    &SanUtil::nate_log('-comment', "Run TUNING to CLUSTERS=$cmode_cluster_names");

    # force cmode tuning to be hardcoded on
    &SanCIT::hardcode_param("IO_ENABLE_CMODE_TUNING",      "1", 1);

    my $host_io_subtest;
    # hardcode the value in

    # initialize eventman in the background while we are starting I/O
    # this will get the test to do all the eventman initialization routines
    # right away
    if ($tcd_obj and $tcd_obj->get_next_testcase() eq 'start_events') {
        my $eventman_obj;
        my $error;
        try {
            ($eventman_obj) = $test_engine_obj->get_objects(type => "eventman");
            if (!$eventman_obj or $test_engine_obj->error()) {
                $error = $test_engine_obj->errmsg();
            }
        } otherwise {
            my $e = shift;
            $error = $e->text();
        };
        if ($error) {
            &cit_error('-fatal', "SanCIT::run_host_io: Could not get eventman object: $error");
        }

        $eventman_obj->initialize_events(-refresh_resources => 1);
        if ($eventman_obj->error()) {
            &cit_error('-fatal', "SanCIT::run_host_io: Could not Initialize Events on the EventMan object. ",
                       $eventman_obj->errmsg());
        }
    }

    eval {
        # invoke host io in background
        $host_io_subtest = &SanUtil::nate_subtest("SANInterop/io/host_io.thpl",
                                                  "IPC_COMM=1",
                                                  "CLIENTS=$client_names",
                                                  "IO_ENABLE_CMODE_TUNING=" . &SanCIT::get_param("IO_ENABLE_CMODE_TUNING"),
                                                  "CMODE_CLUSTERS=" . $cmode_cluster_names,
                                                  "FILERS=''",
                                                  -runid, "IO-" . &SanCIT::get_param("CLIENT_IO_TYPE"), -bg, -ipc);

        while (1) {
            my $count = 0;
            # wait for host io to start IO
            while (not $host_io_subtest->ipc()->can_read()) {
                if ($count % 12 == 0) {
                    my $time_waited = $count * 5;
                    &SanUtil::nate_log('-comment', "Waiting For Host I/O to start (waited=$time_waited sec)");
                }

                &SanUtil::nate_sleep(5);
                $count++;
            }

            my $message = $host_io_subtest->ipc()->read_message();
            if ($message eq "io_start") {
                &SanUtil::nate_log('-comment', "I/O has started");
                last;
            } else {
                redo;
            }

        }
    };
    if ($@) {
        # let the caller deal with the error
        return undef;
    }

    return $host_io_subtest;
}

################################################################################
# Subroutine Name: stop_host_io
# Objective: Tell SANInterop/io/host_io.thpl to stop io
################################################################################
sub stop_host_io {
    my ($host_io_subtest) = @_;

    if ($host_io_subtest->is_alive()) {
        $host_io_subtest->ipc()->write_message("io_stop");
    }

    # wait for Host IO to end
    my $count = 0;
    while ($host_io_subtest->is_alive()) {
        if ($count % 12 == 0) {
            &SanUtil::nate_log('-comment', "Waiting for Host I/O to stop");
        }
        &SanUtil::nate_sleep(5);
        $count++;
    }

    &SanUtil::nate_log('-comment', "I/O has stopped");

    return;
}

################################################################################
# Subroutine Nane: start_eventman
# Objective: Start events on the eventman object
################################################################################
sub start_eventman {
    my (%args) = @_;

    my $eventman_obj = exists($args{-eventman_obj}) ? $args{-eventman_obj} : undef;
    if (not defined($eventman_obj)) {
        &cit_error('-warn', "Error: Missing required EventMan object");
        return $TCD::FATAL;
    }

    foreach my $restype ("controllerclusters", "nacl_clusters_cm") {
        foreach my $cluster_name ($eventman_obj->get_resource_names_by_type(-type => "$restype")) {

            # obtain the regular vserver list from this cluster
            my @newlist = &get_vserver_names(-cluster_name => $cluster_name,
                                             -type => "regular");
            if ($newlist[0] == -1) {
                &cit_error('-warn', "Error: SanCIT::start_eventman: Failed to obtain 'regular' vserver list");
                return $TCD::FATAL;
            }

            my $eparams = $eventman_obj->get_resource_event_params(-name => $cluster_name,
                                                                   -type => "$restype");
            if ($eventman_obj->error()) {
                &cit_error('-warn', "Error: SanCIT::start_eventman: Failed to get res event params\n" , $eventman_obj->errmsg());
                return $TCD::FATAL;
            }
            $eparams->{vservers} = \@newlist;

            $eventman_obj->set_resource_event_params(-name => $cluster_name,
                                                     -type => "$restype",
                                                     -args => $eparams);
            if ($eventman_obj->error()) {
                &cit_error('-warn', "Error: SanCIT::start_eventman: Failed to set res event params\n" , $eventman_obj->errmsg());
                return $TCD::FATAL;
            }
        }
    }

    # make sure that if eventman was set to initialize that it is still running
    if ($eventman_obj->was_set_to_initialize() and not $eventman_obj->is_running()) {
        # something bad happened, abort
        &cit_error('-warn', "Error: SanCIT::start_eventman: EventMan failed to initialize\n", $eventman_obj->errmsg());
        return $TCD::FATAL;
    }

    # start events
    $eventman_obj->run_events(-async => 1,
                              -refresh_resources => 1);
    if ($eventman_obj->error()) {
        &cit_error('-warn', "Error: SanCIT::start_eventman: Failed to start events\n" , $eventman_obj->errmsg());
        return $TCD::FATAL;
    }

    return $TCD::PASS;
}

###############################################################################
# Method:    get_vserver_names
# Objective: obtain vserver names from ParamsMgr
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_vserver_names {
    my (%args) = @_;

    require ParamsMgr;

    &ParamsMgr::process_args(%args);

    my $cluster_name = &ParamsMgr::get_arg(-arg => "cluster_name",
                                           -required => 1);
    my $type = &ParamsMgr::get_arg(-arg      => "type",
                                   -required => 0,
                                   -default  => "all");
    if (&ParamsMgr::error()) {
        &cit_error('-warn', "Error: SanCIT::get_vserver_names: " . &ParamsMgr::get_error());
        return (-1);
    }

    my $clusters = &ParamsMgr::search(-search_path => [],
                                      -key => "CLUSTERS",
                                      -key_filter => qr/^CLUSTER\d*$/);
    if (&ParamsMgr::error()) {
        &cit_error('-warn', "Error: SanCIT::get_vserver_names: " . &ParamsMgr::get_error());
        return (-1);
    }

    my @vservers;
    # iterate over all clusters
    foreach my $cluster (values %{$clusters}) {
        next if ($cluster->{NAME} ne $cluster_name);

        $cluster->{_INDEX} = "" if (not defined($cluster->{_INDEX}));
        my $vservers = &ParamsMgr::search(-search_path => ["CLUSTERS"],
                                          -key => "CLUSTER" . $cluster->{_INDEX},
                                          -key_filter => qr/^VSERVER\d*$/);
        if (&ParamsMgr::error()) {
            &cit_error('-warn', "Error: SanCIT::get_vserver_names: " . &ParamsMgr::get_error());
            return (-1);
        }

        # iterate over all vservers
        foreach my $vserver (values %{$vservers}) {
            # filter vserver out if it doesnt match a specified type
            if (defined($type) and $type !~ /^all$|^\s*$/i) {
                # skip this vserver if the type doesnt match the requested type
                next if ($vserver->{VS_TYPE} !~ /^$type$/i);
            }

            # if this is a single vserver
            if ($vserver->{COUNT} == 1) {
                push(@vservers, $vserver->{NAME});
            } else {
                # handle multiple vservers that are related to this client
                foreach my $i (0..($vserver->{COUNT} - 1)) {
                    push(@vservers, $vserver->{NAME} . "_$i");
                }
            }
        }
    }

    return @vservers;
}

################################################################################
# Subroutine Nane: reap_eventman_and_host_io
# Objective: Reap the eventman and host io
################################################################################
sub reap_eventman_and_host_io {
    my (%args) = @_;

    my $host_io_subtest = exists($args{-host_io_subtest}) ? $args{-host_io_subtest} : undef;
    my $eventman_obj = exists($args{-eventman_obj}) ? $args{-eventman_obj} : undef;

    # while eventman and/or host io are still running
    while ((defined($eventman_obj) and $eventman_obj->is_running()) or (defined($host_io_subtest) and $host_io_subtest->is_alive())) {
        # if eventman hit an error, then stop io
        if (defined($eventman_obj) and $eventman_obj->error()) {
            &cit_error('-warn', "Error: SanCIT::reap_eventman_and_host_io: " .
                                "EventMan encountered an error\n" , $eventman_obj->errmsg());

            # stop IO
            $host_io_subtest->ipc()->write_message("io_stop") if (defined($host_io_subtest));

            # wait for host io to end
            while (defined($host_io_subtest) and $host_io_subtest->is_alive()) {
                &SanUtil::nate_sleep(5);
            }

            return $TCD::FATAL;
        }

        # if host io is not alive then stop events if eventman is alive
        if (defined($host_io_subtest) and not $host_io_subtest->is_alive()) {
            if (defined($eventman_obj)) {
                # stop eventman (this is a blocking call), it will return once eventman
                # is fully stopped
                $eventman_obj->stop_events();
                if ($eventman_obj->error()) {
                    &cit_error('-warn', "Error: SanCIT::reap_eventman_and_host_io: " .
                                   "Failed to stop EventMan\n" , $eventman_obj->errmsg());
                    return $TCD::FATAL;
                }
            } else {
                # otherwise I/O was running by itself, break out
                last;
            }
        }

        # if eventman is defined and not running then break out of the loop
        # this is the success case.
        if (defined($eventman_obj) and not $eventman_obj->is_running()) {
            last;
        }

        # if eventman is not defined and RUNLENGTH is set to iterations
        # then we need to return now
        if (not defined($eventman_obj) and &SanCIT::get_param("RUNLENGTH") =~ /iterations/i) {
            return $TCD::PASS;
        }

        &SanUtil::nate_sleep(10);
    }

    if (defined($eventman_obj) and $eventman_obj->error()) {
        &cit_error('-warn', "Error: SanCIT::reap_eventman_and_host_io: " .
                            "EventMan failed\n", $eventman_obj->errmsg());
        return $TCD::FATAL;
    }

    return $TCD::PASS;
}

################################################################################
# Subroutine Name: setup_sanmgr
# Objective: Setup SanMgr
################################################################################
sub setup_sanmgr {
    my (%args) = @_;

    my $sanmgr_obj = exists($args{-sanmgr_obj}) ? $args{-sanmgr_obj} : undef;
    if (not defined($sanmgr_obj)) {
        &cit_error('-warn', "Error: Missing required sanmgr object");
        return $TCD::FATAL;
    }

    # run sanmgr setup
    my $error = 0;
    try {
        $sanmgr_obj->setup();
    } otherwise {
        my $e = shift;
        &cit_error('-warn', "Error: EnGenX setup failed:\n", $e->text());
        $error = 1;
    };
    if ($error == 1) {
        return $TCD::FATAL;
    }
    if ($sanmgr_obj->error()) {
        &cit_error('-warn', "Error: SanMgr setup failed:\n", $sanmgr_obj->errmsg());
        return $TCD::FATAL;
    }

    # obtain the client dev list from sanmgr and print
    # it to the logs
    my $cdl = undef;
    try {
        $cdl = $sanmgr_obj->get_client_dev_list(-cdl_format => 1);
    } otherwise {
        # No action required, client dev list is allowed to be undefined.
    };

    if ( not defined($cdl) or not length($cdl)) {
        $sanmgr_obj->_clear_err();
        &SanUtil::nate_log('-comment', "No CLIENT_DEV_LIST was generated during " .
                           "SanMgr setup, probably because Client-setup was bypassed");
    } else {
        if (not $cdl_displayed) {
            $cdl_displayed = 1;
            &SanUtil::nate_log('-comment', "CLIENT_DEV_LIST=$cdl");
            &SanUtil::nate_log('-comment', Dumper($cdl));
        }

        # store the client dev list on the ndisp server
        # if one exists
        my $results;
        $results->{params}->{CLIENT_DEV_LIST} = "NDISPD;" . $cdl;
        &SanCIT::ndisp_store_results($results);
    }

    return $TCD::PASS;
}

################################################################################
# Subroutine Name: cleanup_sanmgr
# Objective: Cleanup SanMgr
################################################################################
sub cleanup_sanmgr {
    my (%args) = @_;

    my $sanmgr_obj = exists($args{-sanmgr_obj}) ? $args{-sanmgr_obj} : undef;
    if (not defined($sanmgr_obj)) {
        &cit_error('-warn', "Error: Missing required sanmgr object");
        return $TCD::FATAL;
    }

    # if an ndisp server is running for this test, unset the CLIENT DEV LIST
    # on the config list
    if (&is_param_defined("NDISPD_HOST") and
        &is_param_defined("NDISPD_PORT")) {

        my $host = &get_param("NDISPD_HOST");
        my $port = &get_param("NDISPD_PORT");
        my $ndispd;

        # instantiate the ndispd object (if there is one)
        eval { $ndispd = ndisp::API->new("$host\:$port"); };
        if (defined ($ndispd)) {
            # unset the ndispd config param
            # for CLIENT_DEV_LIST
            &SanCIT::unset_ndispd_config_param($ndispd, "CLIENT_DEV_LIST");
        }
    }

    # cleanup SanMgr
    my $error = 0;
    try {
        $sanmgr_obj->cleanup();
    } otherwise {
        my $e = shift;
        if ($e->isa("NATE::BaseException")) {
            &cit_error('-warn', "Error: EnGenX cleanup failed:\n", $e->text());
        } else {
            &cit_error('-warn', "Error: EnGenX cleanup failed:\n", Dumper($e));
        }
        $error = 1;
    };
    if ($error == 1) {
        return $TCD::FATAL;
    }
    if ($sanmgr_obj->error()) {
        &cit_error('-warn', "Error: SanMgr cleanup failed:\n", $sanmgr_obj->errmsg());
        return $TCD::FATAL;
    }

    return $TCD::PASS;
}

################################################################################
# P R I V A T E   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: _ndisp_store_test_result()
# Objective:       store the test result on the ndisp server
################################################################################
sub _ndisp_store_test_result {
    my ($test_exit_status) = @_;

    # it can happen that test_exit_status is a TCD result value,
    # translate it to either "pass" or "fail"
    if ($test_exit_status !~ /pass|warn|fail/i) {
        if ($test_exit_status == $TCD::PASS or $test_exit_status == $TCD::DISABLE) {
            $test_exit_status = "pass";
        } else {
            $test_exit_status = "fail";
        }
    }

    # if the host and port parameters are defined
    # then set the exit status of the test on the ndispd server
    if (&is_param_defined("NDISPD_HOST") and
        &is_param_defined("NDISPD_PORT")) {

        my $host = &get_param("NDISPD_HOST");
        my $port = &get_param("NDISPD_PORT");

        # create the ndisp object
        my $ndispd = ndisp::API->new("$host\:$port");
        if (not defined ($ndispd)) {
            &SanUtil::nate_log('-warn', "Could not instantiate a NDISP object at $host\:$port\n" .
                                        "Could not store the exit status on the server.");
            return;
        }

        # store the parameter and value on the ndisp server
        &set_ndispd_config_param($ndispd, "NDISPD_PREVIOUS_EXIT_STATUS", $test_exit_status);

        # uninitialize the ndisp object
        $ndispd = {};
    }

    return;
}

################################################################################
# Subroutine Name: _validate_no_tharnhost_hsh()
# Objective:       This will verify that the user has not overridden
#                  Any hardcoded parameters in the tharnhost files of all
#                  CLIENTS or VM_CLIENTS.
################################################################################
sub _validate_no_tharnhost_hsh {
    my $clients = &get_param("CLIENTS");
    my $vm_clients = &get_param("VM_CLIENTS");
    my $hv_clients = &get_param("HV_CLIENTS");
    my $kvm_clients = &get_param("KVM_CLIENTS");
    my $ovm_clients = &get_param("OVM_CLIENTS");
    my $citrixxs_clients = &get_param("CITRIXXS_CLIENTS");

    my $errors = {};
    my $v_hosts = "";
    my $total_errors = 0;

    # parse the vm_clients/hv_clients string, this will update $clients to include
    # virtual machines
    if ($vm_clients ne "" or $hv_clients ne "" or $kvm_clients ne "" or $ovm_clients ne "" or $citrixxs_clients ne "") {
        ($v_hosts, $clients) = &_parse_virtual_clients($vm_clients, $hv_clients, $kvm_clients, $ovm_clients,
                                                       $citrixxs_clients, $clients);
    }

    # build the entire clients array.
    my @all_clients = (split(/:/, $clients), split(/:/, $v_hosts));

    my @print_out_tharnhost_params = ();
    push(@print_out_tharnhost_params, "Tharnhost Per Client Parameters:\n");
    push(@print_out_tharnhost_params, "------------------------------------------\n");

    # loop through all the clients
    foreach my $client (sort @all_clients) {
        next if ($client =~ /^\s*$/);

        my $error_found = 0;
        my $client_tharnhost_env = &ClientUtil::read_client_params($client, "PC_");
        push(@print_out_tharnhost_params, "\n'$client':\n");

        # loop through each parameter that was overridden in the tharnhost file
        foreach my $th_param (sort keys %{$client_tharnhost_env}) {

            # match that with all the hardcoded parameters
            foreach my $hd_param (sort keys %no_tharnhost_hsh) {
                if ($th_param =~ /^$hd_param$/) {
                    # then this is an error, add this parameter to the error hash
                    # for this client
                    $errors->{$client}->{bad_params}->{$th_param} = 1;
                    $error_found = 1;
                    $total_errors++;
                }
            }

            # push the parameter and value onto the array so it can be printed nicely
            push(@print_out_tharnhost_params, "\t* ${th_param}=" . $client_tharnhost_env->{$th_param} . "\n");
        }

        if (scalar(keys %{$client_tharnhost_env}) == 0) {
            push(@print_out_tharnhost_params, "\t* (none)\n\n");
        } else {
            push(@print_out_tharnhost_params, "\n");
        }
        push(@print_out_tharnhost_params, "------------------------------------------\n");

        # mark the no error flag on the client if none were found.
        if ($error_found == 0) {
            $errors->{$client}->{no_errors} = 1;
        }
    }

    &SanUtil::nate_log('-comment', "@print_out_tharnhost_params");

    my @output = ();
    push(@output, "Tharnhost Per Client Parameter validation check:");

    # check each client in the errors hash.
    # and build out the output array to print to the user.
    foreach my $client (sort keys %{$errors}) {
        if ($errors->{$client}->{no_errors} == 1) {
            push(@output, "OK      <=> $client");
            next;
        } else {
            push(@output, "INVALID <=> $client");
            foreach my $param (sort keys %{$errors->{$client}->{bad_params}}) {
                push(@output, "\t---> $param");
            }
        }
    }

    my $output_str = join("\n", @output);

    # if any errors were found, issue a fatal
    if ($total_errors > 0) {
        &SanUtil::nate_log('-comment', "$output_str");
        &cit_error('-fatal', "SanCIT::_validate_no_tharnhost_hsh(): Tharnhost per client parameters are invalid.\n" .
                             "Please address the problems listed above.");
    } else {
        # otherwise issue a comment
        &SanUtil::nate_log('-comment', "$output_str");
        &SanUtil::nate_log('-comment', "Tharnhost per client parameters are valid.");
    }

    return;
}

################################################################################
# Subroutine Name: _parse_virtual_clients()
# Objective: This function will parse the VM|HV_CLIENTS parameter and return back
#            an array of strings: (esx_hosts,hyperv_hosts,clients)
#            where: esx_hosts = e1:e2:e3:...
#            where: hyperv_hosts = h1:h2:...
#            where: clients = $clients:g1:g2:g3:...
################################################################################
sub _parse_virtual_clients {
    my ($vm_clients, $hv_clients, $kvm_clients,
        $ovm_clients, $citrixxs_clients, $clients) = @_;

    $vm_clients =~ s/ //g; # get rid of spaces
    my (@vm_clients) = split(/:/, $vm_clients);
    $hv_clients =~ s/ //g; # get rid of spaces
    my (@hv_clients) = split(/:/, $hv_clients);
    $kvm_clients =~ s/ //g; # get rid of spaces
    my (@kvm_clients) = split(/:/, $kvm_clients);
    $ovm_clients =~ s/ //g; # get rid of spaces
    my (@ovm_clients) = split(/:/, $ovm_clients);
    $citrixxs_clients =~ s/ //g; # get rid of spaces
    my (@citrixxs_clients) = split(/:/, $citrixxs_clients);

    my @all_v_hosts = (@vm_clients, @hv_clients, @kvm_clients, @ovm_clients, @citrixxs_clients);

    my $v_hosts_string = "";
    my $clients_string = $clients;

    foreach my $v_host_and_clients (@all_v_hosts) {
        my ($v_host, $guests, @guests);

        # pull out the v_host name and guests name list
        if($v_host_and_clients =~ /(\S+?),\[(.+?)\]/) {
            $v_host = $1;
            $guests = $2;

            # turn all commas into colons
            $guests =~ s/\,/\:/g;
            # remove all vmfs/vhd and rdm/pt syntax that may be applied to guest names
            $guests =~ s/\(.+?\)//g;

            # if the v hosts string is empty dont append the colon.
            if ($v_hosts_string eq "") {
                $v_hosts_string .= $v_host;
            } else {
                $v_hosts_string .= ":" . $v_host;
            }

            # if the clients string is empty dont append the colon.
            if ($clients_string eq "") {
                $clients_string .= $guests;
            } else {
                $clients_string .= ":" . $guests;
            }
        } else {
            &cit_error('-fatal', "VM_CLIENTS|HV_CLIENTS|KVM_CLIENTS|OVM_CLIENTS|CITRIXXS_CLIENTS is not formatted correctly\n");
        }

    }

    return ($v_hosts_string, $clients_string);
}

################################################################################
# Subroutine Name: _reconfigure_and_reset_cdl
# Objective: This function will hardcode all the vm_io control parameters
# to run precleanup and setup and disable the CLIENT_DEV_LIST parameter
################################################################################
sub _reconfigure_and_reset_cdl {
    my ($ndispd) = @_;

    # prepare to redo precleanup and setup
    &hardcode_param("PRE_CLEANUP",             1);
    &hardcode_param("SKIP_CLIENT_PRE_CLEANUP", 0);
    &hardcode_param("SKIP_ESX_PRE_CLEANUP",    0);
    &hardcode_param("SKIP_HYPERV_PRE_CLEANUP", 0);
    &hardcode_param("SKIP_FILER_PRE_CLEANUP",  0);
    &hardcode_param("SKIP_SETUP",              0);
    &hardcode_param("SKIP_FILER_SETUP",        0);
    &hardcode_param("VM_SKIP_FILER_SETUP",     0);
    &hardcode_param("VM_SKIP_ESX_SETUP",       0);
    &hardcode_param("HV_SKIP_FILER_SETUP",     0);
    &hardcode_param("HV_SKIP_HYPERV_SETUP",    0);
    &hardcode_param("SKIP_CLIENT_SETUP",       0);

    # reset the client dev list parameter locally and on the ndisp server
    &hardcode_param("CLIENT_DEV_LIST", "", 1);
    &unset_ndispd_config_param($ndispd, "CLIENT_DEV_LIST", "ignore");

    return;
}

1;
