# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provides an interface to manage iSCSI Target object
##
## @author dl-san-automation
## @status public

package San::SanClient::Iscsitarget;

# compiler directives
use strict;
use Tharn qw(say param disconnect $Log);
use Params::Validate qw( SCALAR ARRAYREF HASHREF OBJECT );
use Data::Dumper;
use Error;
use San::SanLog;
use San::SanUtil;
use San::SanConsts;
use San::SanCache;

use vars qw(@ISA);
@ISA = qw( Error SanLog SanUtil SanCache SanConsts);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Method Name: get()
# Objective:   Get field from self
# Details:     See POD documentation at the end of this file
################################################################################
sub get {
    my ($self, $field) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->{$field};
} ## end sub get

################################################################################
# Method Name: get_ip()
# Objective:   Get ip
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ip {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("ip");
} ## end sub get_ip

################################################################################
# Method Name: get_ipv4()
# Objective:   Get ipv4
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv4 {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("ipv4");
} ## end sub get_ipv4

################################################################################
# Method Name: get_ipv6()
# Objective:   Get ipv6
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv6 {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("ipv6");
} ## end sub get_ipv6

################################################################################
# Method Name: get_ipv4_netmask()
# Objective:   Get ipv4_netmask
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv4_netmask {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("ipv4_netmask");
} ## end sub get_ipv4_netmask

################################################################################
# Method Name: get_ipv6_netmask()
# Objective:   Get ipv6_netmask
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv6_netmask {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("ipv6_netmask");
} ## end sub get_ipv6_netmask

################################################################################
# Method Name: get_ipv4_subnet()
# Objective:   Get ipv4_subnet
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv4_subnet {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("ipv4_subnet");
} ## end sub get_ipv4_subnet

################################################################################
# Method Name: get_ipv6_subnet()
# Objective:   Get ipv6_subnet
# Details:     See POD documentation at the end of this file
################################################################################
sub get_ipv6_subnet {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("ipv6_subnet");
} ## end sub get_ipv6_subnet

################################################################################
# Method Name: get_mac()
# Objective:   Get mac
# Details:     See POD documentation at the end of this file
################################################################################
sub get_mac {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("mac");
} ## end sub get_mac

################################################################################
# Method Name: get_nodename()
# Objective:   Get nodename
# Details:     See POD documentation at the end of this file
################################################################################
sub get_nodename {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("nodename");
} ## end sub get_nodename

################################################################################
# Method Name: get_state()
# Objective:   Get state
# Details:     See POD documentation at the end of this file
################################################################################
sub get_state {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("state");
} ## end sub get_state

################################################################################
# Method Name: get_subnet()
# Objective:   Get subnet
# Details:     See POD documentation at the end of this file
################################################################################
sub get_subnet {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("subnet");
} ## end sub get_subnet

################################################################################
# Method Name: get_tpgn()
# Objective:   Get tpgn
# Details:     See POD documentation at the end of this file
################################################################################
sub get_tpgn {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("tpgn");
} ## end sub get_tpgn

################################################################################
# Method Name: get_tpgt()
# Objective:   Get tpgt
# Details:     See POD documentation at the end of this file
################################################################################
sub get_tpgt {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("tpgt");
} ## end sub get_tpgt

################################################################################
# Method Name: get_vserver_name()
# Objective:   Get vserver_name
# Details:     See POD documentation at the end of this file
################################################################################
sub get_vserver_name {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("vserver_name");
} ## end sub get_vserver_name

################################################################################
# Method Name: get_cluster_name()
# Objective:   Get cluster_name
# Details:     See POD documentation at the end of this file
################################################################################
sub get_cluster_name {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("cluster_name");
} ## end sub get_cluster_name

################################################################################
# Method Name: get_mode()
# Objective:   Get mode
################################################################################
sub get_mode {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("mode");
} ## end sub get_mode

################################################################################
# Method Name: is_mgmt_port
# Objective:   Returns 1 if this port is the management port, 0 if not
# Details:     See POD documentation at the end of this file
################################################################################
sub is_mgmt_port {
    my ($self) = @_;
    $Log->enter();
    $Log->exit();
    # the get() will handle error
    return $self->get("is_mgmt_port");
} ## end sub is_mgmt_port

################################################################################
# Method Name: set_ip_mode
# Objective:   Set the working ip for this target to be the ipv4|ipv6 ip
# Details:     See POD documentation at the end of this file
################################################################################
sub set_ip_mode {
    my ($self, $mode) = @_;
    $Log->enter();
    $self->_clear_err();

    if ($mode =~ /ipv4/i) {
        $self->{ip}     = $self->{ipv4};
        $self->{subnet} = $self->{ipv4_subnet};
        $self->{mode}   = "ipv4";
    } elsif ($mode =~ /ipv6/i) {
        $self->{ip}     = $self->{ipv6};
        $self->{subnet} = $self->{ipv6_subnet};
        $self->{mode}   = "ipv6";
    } elsif ($mode =~ /auto/i) {
        if ($self->{ipv4} ne "not_set") {
            $self->{ip}     = $self->{ipv4};
            $self->{subnet} = $self->{ipv4_subnet};
            $self->{mode}   = "ipv4";
        } elsif ($self->{ipv6} ne "not_set") {
            $self->{ip}     = $self->{ipv6};
            $self->{subnet} = $self->{ipv6_subnet};
            $self->{mode}   = "ipv6";
        }
    } else {
        $self->_set_err("Invalid IP mode: '$mode', must be ipv4, ipv6, or auto");
        $Log->exit();
        return $SanUtil::FALSE;
    }

    # if the working ip is empty then flag this as an error
    if ($self->{ip} eq "") {
        $self->_set_err(
            "Invalid IP mode: '$mode', this target does not have an $mode ip address"
        );
        $Log->exit();
        return $SanUtil::FALSE;
    }
    $Log->exit();
    return $SanUtil::TRUE;
} ## end sub set_ip_mode

################################################################################
# P R I V A T E   M E T H O D S
################################################################################

###############################################################################
# Method Name: _new
# Objective: Create new iSCSI Target object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _new {
    my ($class, %args) = @_;

    my ($self, @inits);
    $Log->enter();
    # parameter initialization
    $self = {};

    # bless instance into class
    bless $self, $class;

    # remove the dash '-' from input argument list which will be used to
    # create 'get' accessor methods on the fly
    %args = &SanUtil::strip_dash(%args);

    if (exists $args{name}) {
        $args{iscsitarget} = $args{name};
    }

    # Set up the base arguments of the Object.
    $self->{name} = $args{iscsitarget};
    $self->{debug_level} = &Tharn::param('DEBUG_LEVEL', '-default', 0);

    # dump the argument hash
    $self->sanlog(
        -msg       => Dumper(\%args),
        -msg_level => 2
    );

    # store/initalize iscsi target elements
    foreach my $key (keys(%args)) {
        $self->{lc($key)} = $args{$key};
        push @{$self->{fields}}, lc($key);
    }

    # initialize ipv4 attributes
    if ($self->{ipv4} ne "not_set") {
        $self->{ipv4_subnet} = &SanUtil::get_ip_subnet(
            -ip_address   => $self->{ipv4},
            -ip_version   => "ipv4",
            -ipv4_netmask => $self->{ipv4_netmask}
        );
    } else {
        $self->{ipv4_subnet} = "";
    }

    # default current subnet to ipv4_subnet
    $self->{subnet} = $self->{ipv4_subnet};

    # initialize ipv6 attributes
    if ($self->{ipv6} ne "not_set") {
        $self->{ipv6_subnet} = &SanUtil::get_ip_subnet(
            -ip_address         => $self->{ipv6},
            -ip_version         => "ipv6",
            -ipv6_prefix_length => $self->{ipv6_netmask_length}
        );
    } else {
        $self->{ipv6_subnet} = "";
    }

    # add subnet, ipv4_subnet, and ipv6_subnet to the array fields
    push @{$self->{fields}}, ('subnet', 'ipv4_subnet', 'ipv6_subnet');
    $Log->exit();
    return $self;
} ## end sub _new

1;
