#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/SanClient/IsnsServer.pm#1 $

# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary  Creating an IsnsServer object
## @author skante@netapp.com
## @status private

package San::SanClient::IsnsServer;
use Tharn;
use Error;
use Exporter;
use strict;
use vars qw(@EXPORT @ISA);

@ISA = qw(Error);

###############################################################################
#                               Public Methods
###############################################################################

###############################################################################
# Method:   _new
# Objective: Creating an IsnsServer object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _new {
    my ($proto, %args) = @_;

    my ($self) = {};
    my ($class) = ref($proto) || $proto;
    my ($hosttype, $parent);
    bless($self, $class);

    foreach (keys %args) {
        if (/-?parent$/) {
            if (not length($parent = $args{$_})) {
                $self->_set_err("Invalid value for -parent: $args{$_}");
                return $self;
            }
            next;
        }
    }

    # check for existance of required -parent argument
    unless (ref($parent) =~ "SanClient") {
        $self->_set_err("Argument '-parent' must be a SanClient Object");
        return $self;
    }

    $hosttype = $parent->hosttype();
    if ($hosttype =~ /windows/) {
        require San::SanClient::IsnsServer::WinIsnsServer;
        $self =
          San::SanClient::IsnsServer::WinIsnsServer->_new(-parent => $parent);
    } else {
        $self->_set_err(
                    "Client OS \"$hosttype\" is not supported for ISNS server");
        return $self;
    }

    $self->{parent} = $parent;
    return $self;
} ## end sub _new
#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

IsnsServer.pm

=head1 Author

Srinivasarao Kante

=head1 Description

IsnsServer.pm provides an API interface to a host machine through the
use of the Telnet library.

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

B<Error>

=head1 Public Methods

=head2 B<_new(%args)>

Creates a Constructor for iSNS server object. Uses the SanClient object to get the type of host.

=over 4

=item Inputs

=item * -parent(required)

Sanclient Object

=item Outputs

iSNS Server Object if it exists

=item Examples

 $isns_server_obj = IsnsServer->_new(-parent => $self);

=back

=cut

