#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/SanClient/IsnsServer/WinIsnsServer.pm#1 $

# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary  Creating an WinIsnsServer object
## @author skante@netapp.com
## @status private

package San::SanClient::IsnsServer::WinIsnsServer;
use San::SanClient::ClientUtil;
use Tharn;
use Error;
use Data::Dumper;
use Exporter;
use strict;
use vars qw(@EXPORT @ISA);

@ISA = qw(Error IsnsServer SanClient);

###############################################################################
#                               Public Methods
###############################################################################

###############################################################################
# Method:   _new
# Objective: Creating an WinIsnsServer object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _new {
    my ($proto, %args) = @_;

    my ($self) = {};
    my ($class) = ref($proto) || $proto;
    bless($self, $class);
    my ($parent);
    $self->_clear_err();

    foreach (keys %args) {
        if (/-?parent$/) {
            if (not length($parent = $args{$_})) {
                $self->_set_err("Invalid value for -parent: $args{$_}");
                return $self;
            }
            next;
        }
    }

    # check for existance of required -parent argument
    unless (ref($parent) =~ "SanClient") {
        $self->_set_err("Argument '-parent' must be a SanClient Object");
        return $self;
    }

    $self->{conn} = $parent->get_connection();
    $self->isns_start();
    if ($self->error()) {
        $self->_set_err("Isns server start failed", $self->errmsg());
    }

    return $self;
} ## end sub _new

###############################################################################
# Method:   isns_start
# Objective: Start iSNS service on the iSNS server
# Details:   See POD documentation at the end of this file.
###############################################################################
sub isns_start {
    my ($self) = @_;

    $self->_clear_err();
    my $client_conn = $self->{conn};
    my $cmd1        = "sc start msisns";
    my $cmd2        = "sc query msisns";

    my ($status, @output) = &ClientUtil::execute($client_conn, $cmd1);

    if ($status != 0
        or not
        grep(/An instance of the service is already running.|START_PENDING/i,
             @output)
      ) {
        $self->_set_err("Isns Server is not started", $self->errmsg());
        return $ClientUtil::FALSE;
    }

    &SanUtil::nate_sleep(10);

    ($status, @output) = &ClientUtil::execute($client_conn, $cmd2);

    if ($status != 0 or not grep(/RUNNING/i, @output)) {
        $self->_set_err("Error: Isns server is not running", $self->errmsg());
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
} ## end sub isns_start

###############################################################################
# Method:    isns_stop
# Objective: Stop iSNS service on the iSNS server
# Details:   See POD documentation at the end of this file.
###############################################################################
sub isns_stop {
    my ($self) = @_;

    $self->_clear_err();
    my $client_conn = $self->{conn};
    my $cmd1        = "sc stop msisns";
    my $cmd2        = "sc query msisns";

    # Execute the 'sc stop msisns' command
    my ($status, @output) = &ClientUtil::execute($client_conn, $cmd1);
    if ($status != 0
        or not grep(/The service has not been started|STOP_PENDING/i, @output))
    {
        $self->_set_err("Isns Service not stopped, failed in 'sc stop msisns'",
                        $self->errmsg());
        return $ClientUtil::FALSE;
    }

    # sleep for 10 seconds
    &SanUtil::nate_sleep(10);

    # Execute the 'sc query msisns' command
    ($status, @output) = &ClientUtil::execute($client_conn, $cmd2);
    if ($status != 0 or not grep(/STOPPED/i, @output)) {
        $self->_set_err("Isns service not stopped, failed in 'sc query msisns'",
                        $self->errmsg());
        return $ClientUtil::FALSE;
    }
    return $ClientUtil::TRUE;
} ## end sub isns_stop

###############################################################################
# Method:    get_isns_info
#            NOTE: THIS IS 7-MODE VERSION OF THE METHOD!!!
# Objective: Gathers filer's iSNS registration info from the iSNS server.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_isns_info() {
    my ($self, %args) = @_;

    my ($iscsi_node_name, $cmd);
    my $output_hash = {};
    my $isns_info   = {};
    $self->_clear_err();
    $self->sanlog(-msg => "Start get_isns_info");

    #######################################################
    # THIS IS THE 7 MODE VERSION OF GET_ISNS_INFO!!!!     #
    #######################################################

    # Parse arguments
    foreach (keys %args) {
        if (/-?nodename/) {
            $iscsi_node_name = $args{$_};
            if (not length($iscsi_node_name)) {
                $self->_set_err("Invalid value for argument '-nodename'");
                return {};
            }
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $iscsi_node_name) {
        $self->_set_err("Missing required argument '-nodename'");
        return {};
    }

    $iscsi_node_name =~ s/^\s*//;
    $iscsi_node_name =~ s/\s*$//;    #Removing the trailing spaces if any.

    $cmd = "isnscli ln";

    # isnscli ln command - displays all the nodenames
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli ln
    #
    # Nodes:
    #
    # MSiSNSControlNode:0000168c
    # iqn.1992-08.com.netapp:sn.118041459-3070-5
    # iqn.1992-08.com.netapp:sn.118041459-3070-6
    # iqn.1992-08.com.netapp:sn.118041539-6030-6
    # iqn.1992-08.com.netapp:sn.118041539-6030-5
    # iqn.1991-05.com.microsoft:sanqa-isns-4
    # MSiSNSControlNode:00001b5c
    #
    # Success
    #
    my ($status, @output_isnscli_ln) =
      &ClientUtil::execute($self->{conn}, $cmd);

    if (($status != 0) or (not grep(/success/i, @output_isnscli_ln))) {
        $self->_set_err("Failed while executing isnscli ln command.");
        return {};
    }

    if (not grep(/$iscsi_node_name/i, @output_isnscli_ln)) {
        $self->sanlog(-msg =>"Nodename $iscsi_node_name: Not registered with "
                      . "isns server "
                      . $self->hostname());
        return {};
    }

    # Assigning Entity Status to output hash
    $isns_info->{"is-isns-enabled"} = "true";

    $cmd = "isnscli lnd $iscsi_node_name";

    # isnscli lnd nodename command - displays the details of specified node
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli lnd iqn.1992-08.com.netapp:sn.118041539-6030-5
    #
    # iqn.1992-08.com.netapp:sn.118041539-6030-5
    #        ---
    #        Entity Identifier: [isns:0000037b]
    #        Entity Registration Period: 300
    #        ---
    #        Portal IP address: 10.228.194.45
    #        Portal port: 3260
    #        ---
    #        Portal IP address: 10.228.128.45
    #        Portal port: 3260
    #        ---
    #        iSCSI Name: [iqn.1992-08.com.netapp:sn.118041539-6030-5]
    #        iSCSI node type: Target
    #        Alias: [NetApp]
    #        ---
    #        PG iSCSI Name: [iqn.1992-08.com.netapp:sn.118041539-6030-5]
    #        PG Portal IP address: 10.228.194.45
    #        PG Portal port: 3260
    #        PGT: 1000
    #        ---
    #        PG iSCSI Name: [iqn.1992-08.com.netapp:sn.118041539-6030-5]
    #        PG Portal IP address: 10.228.128.45
    #        PG Portal port: 3260
    #        PGT: 1001
    #
    #
    # Success
    #
    my ($stats, @output_isnscli_lnd) =
      &ClientUtil::execute($self->{conn}, $cmd);

    if (($stats != 0) or (not grep(/success/i, @output_isnscli_lnd))) {
        $self->_set_err("Failed while executing isnscli lnd command.");
        return {};
    }

    if (not grep(/iSCSI node type: Target/i, @output_isnscli_lnd)) {
        $self->_set_err(
                       "Nodename $iscsi_node_name: Not registered as a target");
        return {};
    }

    # Assigning the node name into the output hash
    $output_hash->{nodename} = $iscsi_node_name;

    # Getting Alias and Entity Id
    foreach my $temp (@output_isnscli_lnd) {
        if ($temp =~ /^\s*Entity Identifier:\s*\[(\S+)\]\s*$/) {
            $isns_info->{"isns-entity-id"} = $1;
        }
        if ($temp =~ /^\s*Alias:\s*\[(\S+)\]\s*$/) {
            $output_hash->{alias} = $1;
        }
    }

    # If there is no alias then assign nothing
    if (not $output_hash->{alias}) {
        $output_hash->{alias} = "";
    }

    # If there is no Entity Id then set_err
    if (not $isns_info->{"isns-entity-id"}) {
        $self->_set_err(
                  "Nodename $iscsi_node_name: Entity Identifier was not found");
        return {};
    }

    # Get portal_info(port number,portal Ip,tp group tag)
    my @array_port_info_node;
    my ($port_hashno, $portip_hashno, $tpgroup_hashno);
    $port_hashno = $portip_hashno = $tpgroup_hashno = 0;

    foreach my $tempport (@output_isnscli_lnd) {

        # Getting port number
        if ($tempport =~ /^\s*PG Portal port:\s*(\S+)\s*$/) {
            $array_port_info_node[$port_hashno]{"ip-port"} = $1;
            $port_hashno++;
        }

        # Getting port ip address
        if ($tempport =~ /^\s*PG Portal IP address:\s*(\S+)\s*$/) {
            $array_port_info_node[$portip_hashno]{"ip-address"} = $1;
            $portip_hashno++;
        }

        # Getting tp group tag
        if ($tempport =~ /^\s*PGT:\s*(\S+)\s*$/) {
            $array_port_info_node[$tpgroup_hashno]{"tpgroup-tag"} = $1;
            $tpgroup_hashno++;
        }
    } ## end foreach my $tempport (@output_isnscli_lnd)

    # Compare the hashcount is equal in all the above three parameters
    if (($port_hashno != $portip_hashno) or ($portip_hashno != $tpgroup_hashno))
    {
        $self->_set_err(
                   "Nodename $iscsi_node_name: Problem parsing target portals");
        return {};
    }

    # Assigning isns info hash to output hash
    $output_hash->{isns_info} = $isns_info;

    # Assigning port info hash to output hash
    $output_hash->{portal_info} = \@array_port_info_node;

    my $hashop = $self->{parent}->get_ip_info();
    if ($self->{parent}->error()) {
        $self->_set_err(
                    "isns server " . $self->hostname() . ": get_ip_info failed",
                    $self->{parent}->errmsg());
        return {};
    }

    # Assigning ip info to output hash
    $isns_info->{ip_address} = $hashop;

    return $output_hash;
} ## end sub get_isns_info()

###############################################################################
# Method:    get_isns_info_cmode
# Objective: Gathers filer's iSNS registration info from the iSNS server.
#            NOTE: THIS IS C-MODE VERSION OF THE METHOD!!!
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_isns_info_cmode() {
    my ($self, %args) = @_;

    my ($iscsi_node_name, $cmd);
    my $output_hash = {};
    my $deregistered = 0;

    $self->_clear_err();
    $self->sanlog(-msg => "Start get_isns_info_cmode");

    #######################################################
    # THIS IS THE C MODE VERSION OF GET_ISNS_INFO!!!!     #
    #######################################################

    # Parse arguments
    foreach (keys %args) {
        if (/-?nodename/) {
            $iscsi_node_name = $args{$_};
            if (not length($iscsi_node_name)) {
                $self->_set_err("Invalid value for argument '-nodename'");
                return {};
            }
        } elsif (/-?deregistered/) {
            $deregistered = $args{$_};
            if (not length($deregistered)) {
                $self->_set_err("Invalid value for argument '-deregistered'");
                return {};
            }
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (defined $iscsi_node_name) {
        $self->_set_err("Missing required argument '-nodename'");
        return {};
    }
    unless ($deregistered =~ /^(1|0)$/) {
        $$self->_set_err("deregistered must be 0 or 1");
        return {};
    }

    $iscsi_node_name =~ s/^\s*//;
    $iscsi_node_name =~ s/\s*$//;    #Removing the trailing spaces if any.

    $cmd = "isnscli ln";

    # isnscli ln command - displays all the nodenames
    # Expected output
    # C:\Users\Administrator.ssan-w2k8-64-60>isnscli ln

    # Nodes:
    #
    #  iqn.1992-08.com.netapp:sn.2d47a2070ed411e1b1f4123478563412:vs.3
    #  iqn.1992-08.com.netapp:sn.50b0ea59122c11e1a179123478563412:vs.5
    #  MSiSNSControlNode:00001760
    #
    # Success

    my ($status, @output_isnscli_ln) =
      &ClientUtil::execute($self->{conn}, $cmd);

    # If status is not 0, or we don't find 'success' on the last line, error.
    if (($status != 0) or ($output_isnscli_ln[-1] !~ /success/i)) {
        $self->_set_err("Failed while executing isnscli ln command.");
        return {};
    }

    # Scan each output line looking for an exact match of our nodename.
    my $found = 0;
    foreach my $line (@output_isnscli_ln) {
        if ($line =~ /^\s+$iscsi_node_name$/i) {
            $found = 1;
            last;
        }
    }

    # If nodename was not found, exit the method.
    if (!$found) {

        $self->sanlog(-msg => "Nodename $iscsi_node_name: Not registered with "
                      . "isns server "
                      . $self->hostname());
        return {};
    }

    # iSNS must be up since it's registered. (iSCSI too)
    $output_hash->{isns}->{"status-admin"}   = "up";
    $output_hash->{show}->{"status-admin"}   = "up";
    $output_hash->{status}->{"status-admin"} = "up";
    $output_hash->{status}->{"is-available"} = "true";

    # If the target was found by isnscli ln and the deregistered flag is set
    # to 1, let's go ahead and return here. (burt 560810). This avoids a corner
    # case where deregistration can occur after the isnscli ln and before the
    # isnscli lnd.
    if ($found and $deregistered) {
        return $output_hash;
    }

    # Get complete details for this nodename.
    $cmd = "isnscli lnd $iscsi_node_name";

    # Expected output
    # C:\Users\Administrator.ssan-w2k8-64-60>isnscli lnd iqn.1992-08.com.netapp:sn.50b0ea59122c11e1a179123478563412:vs.5
    #
    #  iqn.1992-08.com.netapp:sn.50b0ea59122c11e1a179123478563412:vs.5
    #         ---
    #         Entity Identifier: [isns:000001c4]
    #         Entity Registration Period: 300
    #         ---
    #         Portal IP address: 10.228.177.146
    #         Portal port: 3260
    #         ---
    #         iSCSI Name: [iqn.1992-08.com.netapp:sn.50b0ea59122c11e1a179123478563412:vs.5]
    #         iSCSI node type: Target
    #         Alias: [vs1]
    #         ---
    #         PG iSCSI Name: [iqn.1992-08.com.netapp:sn.50b0ea59122c11e1a179123478563412:vs.5]
    #         PG Portal IP address: 10.228.177.146
    #         PG Portal port: 3260
    #         PGT: 1030
    #
    #
    # Success
    my ($status2, @output_isnscli_lnd) =
      &ClientUtil::execute($self->{conn}, $cmd);

    # If status2 is not 0, or we don't find 'success' on the last line, error.
    if (($status2 != 0) or ($output_isnscli_lnd[-1] !~ /success/i)) {
        $self->_set_err("Failed while executing isnscli lnd command.");
        return {};
    }

    # Getting iscsi node type.  Should find node type of 'Target'.
    my $node_type;

    # Scan the array to find the node type line.
    foreach my $temp (@output_isnscli_lnd) {
        # If we find node type line, save the type and end the loop.
        if ($temp =~ /^\s*iSCSI node type:\s*(\S+)\s*$/) {
            $node_type = $1;
            last;
        }
    }

    # If node type was not set, just print message and return.  This
    # means we found the node with isnscli ln, but lnd did not reveal any
    # details.  Means registration has not completed yet.
    if (not length($node_type)) {
        $self->sanlog(-msg => "Nodename $iscsi_node_name: node type is empty.");
        return {};
    } elsif ($node_type =~ /^Initiator$/i) {
        # elsif type is initiator, raise an error.
        $self->set_err(
                      "Nodename $iscsi_node_name: registered as an initiator!");
        return {};
    } elsif ($node_type !~ /^Target$/i) {
        # Else if the type is not target, it means it is something we are not
        # expecting, so raise error.
        $self->set_err(
            "Nodename $iscsi_node_name: registered as $node_type, which is not expected."
        );
        return {};
    }

    # Assigning the node name into the output hash
    $output_hash->{nodename} = $iscsi_node_name;
    $output_hash->{show}->{"target-name"} = $iscsi_node_name;

    # Getting Alias and Entity Id
    foreach my $temp (@output_isnscli_lnd) {
        if ($temp =~ /^\s*Entity Identifier:\s*\[(\S+)\]\s*$/) {
            $output_hash->{isns}->{"entity-id"} = $1;
        } elsif ($temp =~ /^\s*Alias:\s*\[(\S+)\]\s*$/) {
            $output_hash->{alias} = $1;
            $output_hash->{show}->{"target-alias"} = $1;
        }
    }

    # If there is no alias then assign '-' to agree with ONTAP.
    if (not length($output_hash->{alias})) {
        $output_hash->{alias} = "-";
        $output_hash->{show}->{"target-alias"} = "-";
    }

    # If there is no Entity Id then set_err
    if (not $output_hash->{isns}->{"entity-id"}) {
        $self->_set_err(
                  "Nodename $iscsi_node_name: Entity Identifier was not found");
        return {};
    }

    # Scan all of the lines returned by isnscli lnd.  We need to capture
    # portal information.
    while (@output_isnscli_lnd) {

        # Is this the start of a PG block?  If not throw the line away
        # and continue.  Else we have a PG block and work to do.
        if ($output_isnscli_lnd[0] !~ /^\s+PG iSCSI Name:/) {
            shift(@output_isnscli_lnd);
            next;
        }

        # We have a PG block, let's process it.
        # Make hash to store tpgroup and interface data.
        my %tpgroup;
        my %interface;

        # Set some parms we know.
        # Interface is-home is always true for iscsi lifs.
        # Enabled is always true. (disabled lifs are not sent to isns)
        $interface{"is-home"} = "true";
        $interface{"enabled"} = "true";

        # Remove PG iSCSI Name: line
        shift(@output_isnscli_lnd);

        # Next line contains PG Portal IP address:, save it
        if ($output_isnscli_lnd[0] !~ /^\s+PG Portal IP address:\s+(\S+)/) {
            $self->_set_err( "While processing vserver $iscsi_node_name, "
                           . "did not find expected PG Portal IP address line");
            return {};
        }

        # We got an ip address, save it in the tpgroup and interface
        # structures.
        $interface{"address"} = $1;
        push(@{$tpgroup{"lif"}}, $1);

        # Remove PG Portal IP address: line
        shift(@output_isnscli_lnd);

        # Next line contains PG Portal port:, save it
        if ($output_isnscli_lnd[0] !~ /^\s+PG Portal port:\s+(\S+)/) {
            $self->_set_err(  "While processing vserver $iscsi_node_name, "
                            . "did not find expected PG Portal port line");
            return {};
        }

        # We got a port, save it in the interface structure.
        $interface{"ip-port"} = $1;

        # Remove PG Portal port: line
        shift(@output_isnscli_lnd);

        # Next line contains PGT:, save it. (tpgroup tag)
        if ($output_isnscli_lnd[0] !~ /^\s+PGT:\s+(\S+)/) {
            $self->_set_err(  "While processing vserver $iscsi_node_name, "
                            . "did not find expected PGT line");
            return {};
        }

        # We got PGT (tpgroup tag), save it in the tpgroup & interface structure.
        $tpgroup{"tpgroup-tag"}   = $1;
        $interface{"tpgroup-tag"} = $1;

        # Remove PGT: line
        shift(@output_isnscli_lnd);

        # Save the interface
        push(@{$output_hash->{"interface"}}, \%interface);

        # For tpgroup, we need to scan the current list of tpgroups.
        $found = 0;
        foreach my $saved_tpg (@{$output_hash->{tpgroup}}) {

            # If this tpgroup has already been saved, (Tpgroup exists with
            # the same tag), then just add the ip address to the existing
            # tpgroup's lif list.  Turn on the found flag and exit the loop.
            if ($saved_tpg->{"tpgroup-tag"} == $tpgroup{"tpgroup-tag"}) {
                push(@{$saved_tpg->{lif}}, ${$tpgroup{"lif"}}[0]);
                $found = 1;
                last;
            }
        } ## end foreach my $saved_tpg (@{$output_hash...

        # If we didn't find the tpgroup, this is a new tpgroup. Save it on
        # the tpgroup array.
        if (!$found) {
            push(@{$output_hash->{"tpgroup"}}, \%tpgroup);
        }

    } ## end while (@output_isnscli_lnd)

    # Finally, collect the ip info from the isns server.
    my $hashop = $self->{parent}->get_ip_info();
    if ($self->{parent}->error()) {
        $self->_set_err(
                    "isns server " . $self->hostname() . ": get_ip_info failed",
                    $self->{parent}->errmsg());
        return {};
    }

    # Assigning ip info to output hash
    $output_hash->{isns}->{address} = $hashop;

    return $output_hash;
} ## end sub get_isns_info_cmode()

#############################################################################
# Method:    create_discovery_domain
# Objective: Create discovery domain on iSNS server.
# Details:   See POD documentation at the end of this file.
#############################################################################
sub create_discovery_domain {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $name, $cmd);
    $client_conn = $self->{conn};

    $self->sanlog("Started create_discovery_domain");
    foreach (keys %args) {
        if (/-?name/) {
            $name = $args{$_};
            if (not defined($name)) {
                $self->_set_err("Required input '-name' missing");
                return "";
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (defined($name)) {
        $self->_set_err("Required input '-name' missing");
        return "";
    }

    # Expected output for command
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli createdd leela
    #
    # CreateDiscoveryDomain successful, new DD_ID: [29]
    #
    # Success
    #
    $cmd = "isnscli createdd $name";

    my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
    if ($status == 0 and grep(/Success/i, @output)) {
        foreach my $line (@output) {
            if ($line =~ /DD_ID:\s+\[(\d+)\]/) {
                return $1;
            }
        }
        $self->_set_err("Failed to get DD_ID");
        return "";
    } else {
        $self->_set_err("Failed to create Discovery domain on iSNS server");
        return "";
    }
} ## end sub create_discovery_domain

##############################################################################
# Method:    destroy_discovery_domain
# Objective: Destroy discovery domain on iSNS server.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub destroy_discovery_domain {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $dd_id, $cmd);
    $client_conn = $self->{conn};

    $self->sanlog("Started destroy_discovery_domain");
    foreach (keys %args) {
        if (/-?dd_id/) {
            $dd_id = $args{$_};
            if (not defined($dd_id)) {
                $self->_set_err("Required input '-dd_id' missing");
                return $ClientUtil::FALSE;
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (defined($dd_id)) {
        $self->_set_err("Required input '-dd_id' missing");
        return $ClientUtil::FALSE;
    }

    # Expected output for command
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli deletedd 29
    #
    # Success
    #
    $cmd = "isnscli deletedd $dd_id";

    my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
    if ($status != 0 or not grep(/Success/i, @output)) {
        $self->_set_err("Failed to destroy Discovery domain on iSNS server",
                        $self->errmsg());
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} ## end sub destroy_discovery_domain

##############################################################################
# Method:   create_discovery_domain_set
# Objective: create discovery domain set on iSNS server.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub create_discovery_domain_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $name, $cmd);
    $client_conn = $self->{conn};

    $self->sanlog("Started create_discovery_domain_set");
    foreach (keys %args) {
        if (/-?name/) {
            $name = $args{$_};
            if (not defined($name)) {
                $self->_set_err("Required input '-name' missing");
                return "";
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (defined($name)) {
        $self->_set_err("Required input '-name' missing");
        return "";
    }

    # Expected output for command
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli createdds leelav
    #
    # CreateDiscoveryDomainSet successful, new DDS_ID: [24]
    #
    # Success
    #
    $cmd = "isnscli createdds $name";

    my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
    if ($status == 0 and grep(/Success/i, @output)) {
        foreach my $line (@output) {
            if ($line =~ /DDS_ID:\s+\[(\d+)\]/) {
                return $1;
            }
        }
        $self->_set_err("Failed to get DDS_ID");
        return "";
    } else {
        $self->_set_err("Failed to create Discovery domain set on iSNS server");
        return "";
    }
} ## end sub create_discovery_domain_set

##############################################################################
# Method:   destroy_discovery_domain_set
# Objective: Destroy discovery domain set on iSNS server.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub destroy_discovery_domain_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $dds_id, $cmd);
    $client_conn = $self->{conn};

    $self->sanlog("Started destroy_discovery_domain_set");
    foreach (keys %args) {
        if (/-?dds_id/) {
            $dds_id = $args{$_};
            if (not defined($dds_id)) {
                $self->_set_err("Required input '-dds_id' missing");
                return $ClientUtil::FALSE;
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (defined($dds_id)) {
        $self->_set_err("Required input '-dds_id' missing");
        return $ClientUtil::FALSE;
    }

    # Expected output for command
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli deletedds 24
    #
    # Success
    #
    $cmd = "isnscli deletedds $dds_id";

    my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
    if ($status != 0 or not grep(/Success/i, @output)) {
        $self->_set_err("Failed to destroy Discovery domain set on iSNS server",
                        $self->errmsg());
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}    # end destroy_discovery_domain_set()

###############################################################################
# Method   : verify_isns_registration
# Objective: Verifies the filer's registration information on the iSNS server
#            matches what is reported by the filer
# Details  : See POD documentation at the end of this file
# Inputs   : -filer_obj - required - Filer object to verify iSNS registration
#            -force_isns_update - optional - For iscsi iSNS update
#                                 (Default is 1)
#            -retries - optional - Number of times to retry if filer not found
#                                  (Default is 20)
# Returns  : $ClientUtil::TRUE - Successful execution
#            $ClientUtil::FALSE & set_err - Execution failed
###############################################################################
sub verify_isns_registration {
    my ($self, %args) = @_;

    $self->_clear_err();

    my ($filer_obj, $force_isns_update, $retry_count, $node_name,
        $isns_hash_ref);

    $self->sanlog(-msg => "Start verify_isns_registration");

    # Parse arguments
    foreach (keys %args) {
        if (/-?-filer_obj/) {
            $filer_obj = $args{$_};
            if (not length($filer_obj)) {
                $self->_set_err(  "Invalid value passed in for -filer_obj"
                                . ", value must be a SanFiler Object");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?force_isns_update/) {
            $force_isns_update = $args{$_};
            if (   (not length($force_isns_update))
                or ($force_isns_update !~ /^(1|0)$/)) {
                $self->_set_err("Invalid value passed in for -force_isns_update"
                                . ", value must be 0 or 1");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?retries/) {
            $retry_count = $args{$_};
            if ((not length($retry_count)) or ($retry_count !~ /^\d+$/)) {
                $self->_set_err(
                          "Invalid value for -retries, value must be a number");
                return $ClientUtil::FALSE;
            }
        }
    } ## end foreach (keys %args)

    unless (ref($filer_obj) =~ /^San::SanFiler7M$/i) {
        $self->_set_err(  "Invalid value passed in for -filer_obj"
                        . ", value must be a SanFiler Object");
        return $ClientUtil::FALSE;
    }
    unless (defined $force_isns_update) {
        $force_isns_update = 1;
    }
    unless (defined $retry_count) {
        $retry_count = 20;
    }

    # Getting node name for the given filer.
    $node_name = $filer_obj->get_iscsi_nodename();
    if ($filer_obj->error()) {
        $self->_set_err("Error in getting iscsi node name",
                        $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    if ($force_isns_update == 1) {

        # If user specifies force_isns_update = 1, then call
        # SanFiler7M::iscsi_isns_update.
        $filer_obj->iscsi_isns_update();
        if ($filer_obj->error()) {
            $self->_set_err("iscsi_isns_update failed: Filer $node_name",
                            $filer_obj->errmsg());
            return $ClientUtil::FALSE;
        }

        # Get filer's iSNS registration information from the iSNS server
        # by calling get_isns_info.
        $isns_hash_ref = $self->get_isns_info(-nodename => $node_name);
        if ($self->error()) {
            $self->_set_err("get_isns_info failed: iSNS server="
                              . $self->hostname()
                              . ", Filer=$node_name",
                            $self->errmsg());
                return $ClientUtil::FALSE;
        }
        if (scalar(keys(%$isns_hash_ref)) == 0) {

                # Checking isns registration is happened or not.
                $self->_set_err( "WinIsnsServer::verify_isns_registration"
                      . ": Filer $node_name is not registered with iSNS server "
                      . $self->hostname()
                      . ".");
                  return $ClientUtil::FALSE;
        }
        # Call _process_isns_hash to compare information from iSNS server with
        # filer's iSNS registration information from the iSNS server.
        my $result =
          $self->_process_isns_hash(-filer_obj => $filer_obj,
                                    -isnshash  => $isns_hash_ref);
        if ($self->error()) {
            $self->_set_err("_process_isns_hash failed: iSNS server="
                              . $self->hostname()
                              . ", Filer=$node_name",
                            $self->errmsg());
            return $ClientUtil::FALSE;
        }

        if ($result == $ClientUtil::TRUE) {

            # If _process_isns_hash returns true, it means all of the
            # data matched. So we can return success.
            $self->sanlog(-msg_level => 1,
                          -msg       => "iSNS registration success!"
                            . ", iSNS server="
                            . $self->hostname()
                            . ", Filer=$node_name");
            return $ClientUtil::TRUE;
        } else {

            # Else _process_isns_hash must returns false, it means some of
            # the data is stale. So we can return failure.
            $self->_set_err(  "iSNS registration FAILED! iSNS server="
                            . $self->hostname()
                            . ", Filer=$node_name");
            return $ClientUtil::FALSE;
        }

    } else {
        # Else force_isns_update is zero.
        # Looping '$retry_count + 1' times, so there is an initial attempt
        # before start counting retries.
        for (my $i = 0 ; $i < $retry_count + 1 ; $i++) {

            # Get filer's iSNS registration information from the iSNS server
            # by calling get_isns_info.
            $isns_hash_ref = $self->get_isns_info(-nodename => $node_name);
            if ($self->error()) {
                $self->_set_err("get_isns_info failed: iSNS server="
                                  . $self->hostname()
                                  . ", Filer=$node_name",
                                $self->errmsg());
                return $ClientUtil::FALSE;
            }

            # If the node name is not registered with iSNS server.
            if (scalar(keys(%{$isns_hash_ref})) == 0) {

                # Wait for one minute.
                $self->sanlog(-msg => "WinIsnsServer::verify_isns_registration"
                      . ": Filer $node_name is not registered with iSNS server "
                      . $self->hostname()
                      . ".");

                # If we are NOT on the final attempt, sleep for a minute  before
                # trying again.
                # (If we are on the final attempt, don't want to waste time
                # sleeping.)
                if ($i != $retry_count) {
                    $self->sanlog(-msg => "Waiting for one minute...");
                    &SanUtil::nate_sleep(60);
                }
                if (($i == $retry_count ) && (scalar(keys(%{$isns_hash_ref})) == 0)) {
                # Checking isns registration is happened or not.
                $self->_set_err( "WinIsnsServer::verify_isns_registration"
                      . ": Filer $node_name is not registered with iSNS server "
                      . $self->hostname()
                      . ".");
                  return $ClientUtil::FALSE;

                }
                next;
            } ## end if (scalar(keys(%$isns_hash_ref...

            # Call _process_isns_hash to compare information from iSNS server with
            # filer's iSNS registration information from the iSNS server.
            my $process_var =
              $self->_process_isns_hash(-filer_obj => $filer_obj,
                                        -isnshash  => $isns_hash_ref);
            if ($self->error()) {
                $self->_set_err("_process_isns_hash failed: iSNS server="
                                  . $self->hostname()
                                  . ", Filer=$node_name",
                                $self->errmsg());
                return $ClientUtil::FALSE;
            }
            if ($process_var eq $ClientUtil::TRUE) {
                # If _process_isns_hash returns true, it means all of the
                # data matched. So we can return success.
                $self->sanlog(-msg_level => 1,
                              -msg       => "iSNS registration success"
                                . " after $i retries!"
                                . ", Filer=$node_name"
                                . ", iSNS server="
                                . $self->hostname());
                return $ClientUtil::TRUE;
            } else {

                # Else _process_isns_hash must returns false, it means some of
                # the data is stale. So wait for one minute.
                $self->sanlog(-msg => "WinIsnsServer::verify_isns_registration"
                          . ": filer $node_name is registered with isns server "
                          . $self->hostname()
                          . ", but some of the data is stale.");

                # If we are NOT on the final attempt, sleep for a minute  before
                # trying again.
                # (If we are on the final attempt, don't want to waste time
                # sleeping.)
                if ($i != $retry_count) {
                    $self->sanlog(-msg => "Waiting for one minute...");
                    &SanUtil::nate_sleep(60);
                }
            } ## end else [ if ($process_var eq $ClientUtil::TRUE)

        } ## end for (my $i = 0 ; $i < $retry_count...

    } ## end else [ if ($force_isns_update...

    # If you reach here, then all retries failed.
    $self->_set_err(  "iSNS registration FAILED after $retry_count retries!"
                    . " Filer=$node_name"
                    . ", iSNS server="
                    . $self->hostname());
    return $ClientUtil::FALSE;

} ## end sub verify_isns_registration

###############################################################################
# Method:    verify_isns_deregistration
# Objective: Verifying deregistration of the given Filer.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub verify_isns_deregistration {
    my ($self, %args) = @_;

    $self->_clear_err();

    my $retry_count = 20;    # Default value for retries.
    my ($filer_obj, $output_hash);

    $self->sanlog(-msg => "Start verify_isns_deregistration.");

    # Validating arguments.
    foreach (keys(%args)) {
        if (/-?filer_obj$/) {
            if (not length($filer_obj = $args{$_})) {
                $self->_set_err("Invalid value for -filer_obj: $args{$_}");
                return $ClientUtil::FALSE;
            }
        }
        if (/-?retries$/) {
            if (not length($retry_count = $args{$_})) {
                $self->_set_err("Invalid value for -retries: $args{$_}");
                return $ClientUtil::FALSE;
            }
        }
    } ## end foreach (keys(%args))

    # Checking Invalid value for the filer.
    unless (ref($filer_obj) =~ /^San::SanFiler7M$/i) {
        $self->_set_err(
            "Invalid value passed in for -filer_obj, value needs to be a SanFiler Object"
        );
        return $ClientUtil::FALSE;
    }

    #Get the filer's name.
    my $filer_name = $filer_obj->get_name();
    if ($filer_obj->error()) {
        $self->_set_err("Failed to get Filer's name from filer object.",
                        $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # Getting node name for the given filer.
    my $node_name = $filer_obj->get_iscsi_nodename();
    if ($filer_obj->error()) {
        $self->_set_err("Failed to get iscsi nodename for filer $filer_name",
                        $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # Loop for checking deregistration.
    # Looping 'retry_count + 1' times, so there is an initial attempt
    # before start counting retries.
    for (my $i = 0 ; $i < $retry_count + 1 ; $i++) {

        # Get isns information for the node name.
        $output_hash = $self->get_isns_info(-nodename => $node_name);
        if ($self->error) {
            $self->_set_err("get_isns_info failed: isns server="
                              . $self->hostname()
                              . ", filer=$node_name",
                            $self->errmsg);
            return $ClientUtil::FALSE;
        }
        if (scalar(keys(%$output_hash)) == 0) {

            # if the node name is no longer registered with isns server.
            $self->sanlog(-msg =>
                "WinIsnsServer::verify_isns_deregistration: Filer $node_name deregistered from isns server "
                . $self->hostname
                . " after $i retries.");
            return $ClientUtil::TRUE;
        } else {

            # Else still registered
            $self->sanlog(-msg =>
                "WinIsnsServer::verify_isns_deregistration: filer $node_name is still registered with isns server "
                . $self->hostname()
                . ".");

            # If we are NOT on the final attempt, sleep for a minute  before
            # trying again.
            # (If we are on the final attempt, don't want to waste time
            # sleeping.)
            if ($i != $retry_count) {
                $self->sanlog(-msg => "Waiting for one minute...");
                &SanUtil::nate_sleep(60);
            }
        } ## end else [ if (scalar(keys(%$output_hash...
    } ## end for (my $i = 0 ; $i < $retry_count...

    # if the node name registered with isns server
    $self->_set_err(  "Filer $node_name is still registered with isns server "
                    . $self->hostname()
                    . " after $retry_count retries.");
    return $ClientUtil::FALSE;
}    # End sub verify_isns_deregistration

#############################################################################
# Method:    add discovery domain to discovery domain set
# Objective: Adds discovery domain to discovery domain set on the iSNS server.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub add_discovery_domain_to_discovery_domain_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $cmd, $dds_id, @dd_ids);
    $client_conn = $self->{conn};

    $self->sanlog(
                -msg => "Started add_discovery_domain_to_discovery_domain_set");
    foreach (keys %args) {
        if (/-?dd_ids/) {
            if (not scalar(@dd_ids = @{$args{$_}})) {
                $self->_set_err("Required parameter '-dd_ids' missing");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?dds_id/) {
            $dds_id = $args{$_};
            if (not length($dds_id)) {
                $self->_set_err("Required parameter '-dds_id' missing");
                return $ClientUtil::FALSE;
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (scalar(@dd_ids)) {
        $self->_set_err("Required parameter '-dd_ids' missing");
        return $ClientUtil::FALSE;
    }
    unless (defined($dds_id)) {
        $self->_set_err("Required parameter '-dds_id' missing");
        return $ClientUtil::FALSE;
    }

    # add discovery domain to discovery domain set
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli addddtodds 5 9
    #
    # Success
    #

    foreach my $id (@dd_ids) {
        $cmd = "isnscli addddtodds $dds_id $id";
        my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
        if ($status != 0 or not grep(/Success/i, @output)) {
            $self->_set_err(
                "Failed to add dd_id $id to discovery domain set $dds_id on iSNS server",
                $self->errmsg());
            return $ClientUtil::FALSE;
        }
    } ## end foreach my $id (@dd_ids)

    return $ClientUtil::TRUE;
}    # end sub add_discovery_domain_to_discovery_domain_set

##############################################################################
# Method:    remove discovery domain from discovery domain set
# Objective: Remove discovery domain from discovery domain set on the iSNS server.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub remove_discovery_domain_from_discovery_domain_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $cmd, $dds_id, @dd_ids);
    $client_conn = $self->{conn};

    $self->sanlog(
           -msg => "Started remove_discovery_domain_from_discovery_domain_set");
    foreach (keys %args) {
        if (/-?dd_ids/) {
            if (not scalar(@dd_ids = @{$args{$_}})) {
                $self->_set_err("Required parameter '-dd_ids' missing");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?dds_id/) {
            $dds_id = $args{$_};
            if (not length($dds_id)) {
                $self->_set_err("Required parameter '-dds_id' missing");
                return $ClientUtil::FALSE;
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (scalar(@dd_ids)) {
        $self->_set_err("Required parameter '-dd_ids' missing");
        return $ClientUtil::FALSE;
    }
    unless (defined($dds_id)) {
        $self->_set_err("Required parameter '-dds_id' missing");
        return $ClientUtil::FALSE;
    }

    # remove discovery domain from discovery domain set
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli removeddfromdds 5 9
    #
    # Success
    #

    foreach my $id (@dd_ids) {
        $cmd = "isnscli removeddfromdds $dds_id $id";
        my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
        if ($status != 0 or not grep(/Success/i, @output)) {
            $self->_set_err(
                "Failed to remove dd_id $id from discovery domain set $dds_id on iSNS server",
                $self->errmsg());
            return $ClientUtil::FALSE;
        }
    } ## end foreach my $id (@dd_ids)

    return $ClientUtil::TRUE;
}    # end sub remove_discovery_domain_from_discovery_domain_set

##############################################################################
# Method:    enable_discovery_domain_set
# Objective: Enables a discovery domain set on the iSNS server.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub enable_discovery_domain_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $cmd, @dds_ids);
    $client_conn = $self->{conn};

    $self->sanlog(-msg => "Started enable_discovery_domain_set");
    foreach (keys %args) {
        if (/-?dds_ids/) {
            if (not scalar(@dds_ids = @{$args{$_}})) {
                $self->_set_err("Required parameter '-dds_ids' missing");
                return $ClientUtil::FALSE;
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (scalar(@dds_ids)) {
        $self->_set_err("Required parameter '-dds_ids' missing");
        return $ClientUtil::FALSE;
    }

    # enable discovery domain set
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli enabledds 5
    #
    # Success
    #

    foreach my $id (@dds_ids) {
        $cmd = "isnscli enabledds $id";
        my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
        if ($status != 0 or not grep(/Success/i, @output)) {
            $self->_set_err("Failed to enable discovery domain set $id",
                            $self->errmsg());
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}    # end sub enable_discovery_domain_set

##############################################################################
# Method:    disable_discovery_domain_set
# Objective: Disables a discovery domain set on the iSNS server.
# Details:   See POD documentation at the end of this file.
##############################################################################
sub disable_discovery_domain_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($client_conn, $cmd, @dds_ids);
    $client_conn = $self->{conn};

    $self->sanlog(-msg => "Started disable_discovery_domain_set");
    foreach (keys %args) {
        if (/-?dds_ids/) {
            if (not scalar(@dds_ids = @{$args{$_}})) {
                $self->_set_err("Required parameter '-dds_ids' missing");
                return $ClientUtil::FALSE;
            }
        }
    }    # end of foreach loop

    # Check for existence of required arguments
    unless (scalar(@dds_ids)) {
        $self->_set_err("Required parameter '-dds_ids' missing");
        return $ClientUtil::FALSE;
    }

    # disable discovery domain set
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli disabledds 5
    #
    # Success
    #

    foreach my $id (@dds_ids) {
        $cmd = "isnscli disabledds $id";
        my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
        if ($status != 0 or not grep(/Success/i, @output)) {
            $self->_set_err("Failed to disable discovery domain set $id",
                            $self->errmsg());
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}    # end sub disable_discovery_domain_set

###############################################################################
# Method:    add_iqn_to_discovery_domain
# Objective: Adds one or more iqns to a discovery domain on the iSNS sever
# Details:   See POD documentation at the end of this file.
###############################################################################
sub add_iqn_to_discovery_domain {
    my ($self, %args) = @_;

    $self->_clear_err();
    my (@iqnnodes, $dd_id, $connection, $cmd);
    $connection = $self->{conn};
    $self->sanlog(-msg => "Started add_iqn_to_discovery_domain");

    # Parse arguments.
    while (my ($key, $value) = each(%args)) {
        if ($key =~ /-?iqns/) {
            unless ($value) {
                $self->_set_err("Invalid value for \"$key\": " . $value);
                return $ClientUtil::FALSE;
            }
            @iqnnodes = @{$value};
        } elsif ($key =~ /-?dd_id/) {
            $dd_id = $value;
            if (not length($dd_id)) {
                $self->_set_err("Required input '-dd_id' missing");
                return $ClientUtil::FALSE;
            }
        }
    } ## end while (my ($key, $value) ...

    # check for existance of required arguments
    unless (scalar(@iqnnodes)) {
        $self->_set_err("Required input '-iqns' missing");
        return $ClientUtil::FALSE;
    }
    unless (defined($dd_id)) {
        $self->_set_err("Required input '-dd_id' missing");
        return $ClientUtil::FALSE;
    }

    # Add iqn to the discovery domain
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli addnodetodd 16 iqn.1992-08.com.netapp:sn.118041539-6030-6
    #
    # Success
    #
    foreach my $iqnnode (@iqnnodes) {
        $cmd = "isnscli addnodetodd $dd_id $iqnnode";
        my ($status, @output) = &ClientUtil::execute($connection, $cmd);
        if ($status != 0 or not grep(/Success/i, @output)) {
            $self->_set_err(  "Failed to add iqnnode: $iqnnode to"
                            . " discovery domain $dd_id");
            return $ClientUtil::FALSE;
        }
    }    # end of foreach loop
    return $ClientUtil::TRUE;
}    # end of add_iqn_to_discovery_domain

##############################################################################
# Method:    remove_iqn_from_discovery_domain
# Objective: removes one or more iqns from a discovery domain on the iSNS sever
# Details:   See POD documentation at the end of this file.
###############################################################################
sub remove_iqn_from_discovery_domain {
    my ($self, %args) = @_;

    $self->_clear_err();
    my (@iqnnodes, $all, $dd_id, $connection, $cmd);
    $connection = $self->{conn};
    $all        = 0;               # default value is set to 0.

    $self->sanlog(-msg => "Start remove_iqn_from_discovery_domain");

    # parsing the input arguments.
    while (my ($key, $value) = each(%args)) {
        if ($key =~ /-?iqns/) {
            unless ($value) {
                $self->_set_err("Invalid value for \"$key\": " . $value);
                return $ClientUtil::FALSE;
            }
            @iqnnodes = @{$value};
        } elsif ($key =~ /-?all/) {
            $all = $value;
            if ((not length($all)) or ($all !~ /^(1|0)$/)) {
                $self->_set_err(  "Invalid value for argument '-all', "
                                . "value must be 0 or 1");
                return $ClientUtil::FALSE;
            }
        } elsif ($key =~ /-?dd_id/) {
            $dd_id = $value;
            if (not length($dd_id)) {
                $self->_set_err("Invalid value for argument '-dd_id'");
                return $ClientUtil::FALSE;
            }
        }
    } ## end while (my ($key, $value) ...

    # check for existance of required arguments
    unless (defined($dd_id)) {
        $self->_set_err("Required input '-dd_id' missing");
        return $ClientUtil::FALSE;
    }

    # Either all=1 or iqnnodes to be provided, not both.
    if (($all) && scalar(@iqnnodes)) {
        $self->_set_err(  "Two parameters -iqns and -all:$all are provided,"
                        . " only one parameter has to be provided");
        return $ClientUtil::FALSE;
    }

    # Either specify all=1 or list some iqns.
    if ((!$all) && (not scalar(@iqnnodes))) {
        $self->_set_err(  "Method requires either -all=>1 or a list of"
                        . " -iqns to be provided.");
        return $ClientUtil::FALSE;
    }

    # If all=1, we need to grab the list of all iqns in the discovery domain
    if ($all) {
        @iqnnodes = $self->list_discovery_domain_node_members(-dd_id => $dd_id);
        if ($self->error()) {
            $self->_set_err("Failed while retrieving node members in discovery"
                              . " domain $dd_id",
                            $self->errmsg());
            return $ClientUtil::FALSE;
        }
    }

    # Remove iqn from the discovery domain
    # Expected output (@output)
    # C:\Users\Administrator.W2K8STD64-MASTE>isnscli removenodefromdd 4 iqn.1992-08.com.netapp:sn.118041459-ssan-6030-5
    #
    # Success
    #
    foreach my $iqnnode (@iqnnodes) {
        $cmd = "isnscli removenodefromdd $dd_id $iqnnode";
        my ($status, @output) = &ClientUtil::execute($connection, $cmd);
        if ($status != 0 or not grep(/Success/i, @output)) {
            $self->_set_err("Failed to remove iqnnode: $iqnnode from discovery"
                            . " domain $dd_id");
            return $ClientUtil::FALSE;
        }
    }    # end of foreach loop

    return $ClientUtil::TRUE;
} ## end sub remove_iqn_from_discovery_domain

###############################################################################
# Method   : list_discovery_domain_node_members
# Objective: To get all iqns in specified discovery domain on the iSNS sever
# Details  : See POD documentation at the end of this file
# Inputs   : -dd_id - required -  Discovery Domain ID
# Returns  : Array of all iqns in the specified discovery domain.
#            (Empty array if there are no members) - Successful execution
#            Return () & set_err - Execution failed
###############################################################################
sub list_discovery_domain_node_members {
    my ($self, %args) = @_;
    $self->_clear_err();

    my ($dd_id, $client_conn, $cmd, $status, @output, @dd_id_members);

    $self->sanlog(-msg => "Start list_discovery_domain_node_members");

    # Parse arguments
    foreach (keys %args) {
        if (/-?dd_id/) {
            $dd_id = $args{$_};
            if ((not length($dd_id)) or ($dd_id !~ /^\d+$/)) {
                $self->_set_err(  "Invalid value passed in for -dd_id"
                                . ", value must be a must be a number");
                return ();
            }
        }
    } ## end foreach (keys %args)

    unless (defined $dd_id) {
        $self->_set_err("Did not pass in a Discovery Domain ID for -dd_id");
        return ();
    }

    $client_conn = $self->{conn};
    $cmd         = "isnscli listdiscoverydomainnodemembers $dd_id";

    # Execute the command "isnscli listdiscoverydomainnodemembers $dd_id"
    ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
    if ($status != 0 or not grep(/Success/i, @output)) {
        $self->_set_err("Command execution failed", $self->errmsg());
        return ();
    }

    # Sample outputs of "isnscli lddnm <Discovery Domain ID>".

    #C:\Users\Administrator.W2K8STD64-MASTE>isnscli lddnm 1
    #
    # DD_ID: 1
    #        DD Member iSCSI Name: [iqn.1992-08.com.netapp:sn.118047048-6030-5]
    #        DD Member iSCSI Name: [iqn.1992-08.com.netapp:sn.118041is539-6030-5]
    #        DD Member iSCSI Name: [iqn.1991-05.com.microsoft:sanqa-isns-4]
    #        DD Member iSCSI Name: [iqn.1992-08.com.netapp:sn.118041459-ssan-6030-5]
    #
    #
    #Success

    #C:\Users\Administrator.W2K8STD64-MASTE>isnscli lddnm 3
    #
    # DD_ID: 3
    #
    #
    #Success

    foreach my $line (@output) {
        if ($line =~ /DD Member iSCSI Name:\s+\[(.*)\]/) {
            push(@dd_id_members, $1);
        }
    }
    return (wantarray()) ? @dd_id_members : \@dd_id_members;

} ## end sub list_discovery_domain_node_members

###############################################################################
# Method   : list_all_discovery_domain_sets_with_members
# Objective: To get all the discovery domain sets with members from the iSNS sever
# Details  : See POD documentation at the end of this file
# Inputs   : none
# Returns  : Hash of all the discovery domain sets with key as dds_id.
#            Hash will have dds_name, status and hash of Discovery Domains
#            that are members of the Discovery Domain Set.
#            Discovery Domain member hash will have keys as dd_id
#            and name of Discovery Domain as value. - Successful execution
#            Return {} & set_err - Execution failed
###############################################################################
sub list_all_discovery_domain_sets_with_members {
    my ($self) = @_;
    $self->_clear_err();

    my ($client_conn, $cmd, $status, @output, $dds_hash, $dds_domains, $dds_name, $line);

    my $all_dds_hash = {};

    $self->sanlog(-msg => "Start list_all_discovery_domain_sets_with_members");

    $client_conn = $self->{conn};
    $cmd         = "isnscli listalldiscoverydomainssetswithmembers";

    # Execute the command "isnscli listalldiscoverydomainssetswithmembers"
    ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
    if ($status != 0 or not grep(/Success/i, @output)) {
        $self->_set_err("Command execution failed", $self->errmsg());
        return {};
    }

    # Sample outputs of "isnscli ladds".
    #C:\Users\Administrator.ssan-w2k8-64-44>isnscli LADDS
    #
    #    iSNS Discovery Domain Sets:
    #
    #    DDS_ID: 1 [Default DDS]
    #    DDS Status: ENABLED
    #
    #           DDS Member DD_ID: 1 [Default DD]
    #	        DDS Member DD_ID: 2 [My DD]
    #
    #     DDS_ID: 9 [MyDDS]
    #     DDS Status: DISABLED
    #
    #            DDS Member DD_ID: 9 [MyDD]
    #
    #     DDS_ID: 5 [setiSNS]
    #     DDS Status: ENABLED
    #
    #            DDS Member DD_ID: 10 [domainiSNS]
    #
    #     DDS_ID: 9 [MyDDS]
    #     DDS Status: DISABLED
    #
    #     DDS Member DD_ID: 9 [MyDD]
    #
    #C:\Users\Administrator.ssan-w2k8-64-79>isnscli ladds
    #
    #iSNS Discovery Domain Sets:
    #
    # DDS_ID: 1 [Default DDS]
    # DDS Status: ENABLED
    #
    #        DDS Member DD_ID: 1 [Default DD]
    #
    #
    #Success

    foreach $line(@output) {
       if ($line =~ m/\s+DDS_ID:\s+(\d+)\s+\[(.*)\]/){
           $dds_hash = {};
           $dds_domains = {};
           $dds_hash->{domains} = $dds_domains;
           $all_dds_hash->{$1} = $dds_hash;
           $dds_hash->{dds_name} = $2;
       }

       if ($line =~ m/\s+DDS\s+Status:\s+(.*)/){
           $dds_hash->{status} = $1;
       }

       if ($line =~ m/\s+DDS\s+Member\s+DD_ID:\s+(\d+)\s+\[(.*)\]/){
           $dds_domains->{$1} = $2;
       }
    }

    # returns all the dds with their members
    return %$all_dds_hash;

}

###############################################################################
# Method   : create_basic_discovery_domain_and_set
# Objective: Creates one Discovery Domain and one Discovery Domain Set on
#            the iSNS sever
# Details  : See POD documentation at the end of this file
# Inputs   : -iqns - required - Reference to array of filer and
#               initiator iscsi nodenames to include in the Discovery Domain
#            -dd_name - optional - Name of Discovery Domain
#                     - default 'domain1'
#            -dds_name - optional - Name of Discovery Domain Set
#                      - default 'set1'
# Returns  : HASH, contains information about Discovery Domain ID and
#            Discovery Domain Set ID - Successful execution
#            return {} & set_err - Execution failed
###############################################################################
sub create_basic_discovery_domain_and_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($dd_name, $dds_name, $dd_id, $dds_id, @iqns, %discovery_domain_and_set);

    $self->sanlog(-msg => "Start create_basic_discovery_domain_and_set");

    # Parse arguments.
    while (my ($key, $value) = each(%args)) {
        if ($key =~ /-?iqns/) {
            unless ($value) {
                $self->_set_err("Invalid value for \"$key\": " . $value);
                return $SanUtil::FALSE;
            }
            @iqns = @{$value};
        } elsif ($key =~ /-?dd_name/) {
            $dd_name = $value;
            if (not length($dd_name)) {
                $self->_set_err("Invalid value for argument '-dd_name'");
                return {};
            }
        } elsif ($key =~ /-?dds_name/) {
            $dds_name = $value;
            if (not length($dds_name)) {
                $self->_set_err("Invalid value for argument '-dds_name'");
                return {};
            }
        }
    } ## end while (my ($key, $value) ...

    unless (scalar @iqns) {
        $self->_set_err("Did not pass in the value for argument '-iqns'");
        return {};
    }
    unless (defined $dd_name) {
        $dd_name = 'domain1';
    }
    unless (defined $dds_name) {
        $dds_name = 'set1';
    }

    # Create Discovery Domain.
    $dd_id = $self->create_discovery_domain(-name => $dd_name);
    if ($self->error()) {
        $self->_set_err("create_discovery_domain failed for $dd_name",
                        $self->errmsg());
        return {};
    }

    # Add 'filer and initiator iscsi nodenames' to Discovery Domain.
    $self->add_iqn_to_discovery_domain(-iqns  => \@iqns,
                                       -dd_id => $dd_id);
    if ($self->error()) {
        $self->_set_err("add_iqn_to_discovery_domain failed for domain $dd_id",
                        $self->errmsg());
        return {};
    }

    # Create Discovery Domain Set.
    $dds_id = $self->create_discovery_domain_set(-name => $dds_name);
    if ($self->error()) {
        $self->_set_err("create_discovery_domain_set failed for set $dds_name",
                        $self->errmsg());
        return {};
    }

    # Add Discovery Domain to Discovery Domain Set.
    $self->add_discovery_domain_to_discovery_domain_set(-dd_ids => [$dd_id],
                                                        -dds_id => $dds_id);
    if ($self->error()) {
        $self->_set_err(
                       "add_discovery_domain_to_discovery_domain_set failed for"
                         . " set=$dds_id and domain=$dd_id",
                       $self->errmsg());
        return {};
    }

    # Enable discovery domain set.
    $self->enable_discovery_domain_set(-dds_ids => [$dds_id]);
    if ($self->error()) {
        $self->_set_err("enable_discovery_domain_set failed for set $dds_id",
                        $self->errmsg());
        return {};
    }

    # If all the steps passed, then return %discovery_domain_and_set.
    $discovery_domain_and_set{dd_id}  = $dd_id;
    $discovery_domain_and_set{dds_id} = $dds_id;
    return %discovery_domain_and_set;

} ## end sub create_basic_discovery_domain_and_set

###############################################################################
# Method   : destroy_basic_discovery_domain_and_set
# Objective: Destroys a basic discovery domain and set on the iSNS sever
# Details  : See POD documentation at the end of this file
# Inputs   : -dd_id - required - ID of Discovery Domain
#            -dds_id - required - ID of Discovery Domain Set
# Returns  : $ClientUtil::TRUE - Successful execution
#            $ClientUtil::FALSE & set_err - Execution failed
###############################################################################
sub destroy_basic_discovery_domain_and_set {
    my ($self, %args) = @_;

    $self->_clear_err();
    my ($dd_id, $dds_id);

    $self->sanlog(-msg => "Start destroy_basic_discovery_domain_and_set");

    # Parse arguments.
    foreach (keys %args) {
        if (/-?dd_id/) {
            $dd_id = $args{$_};
            if (not length($dd_id)) {
                $self->_set_err("Invalid value for argument '-dd_id'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?dds_id/) {
            $dds_id = $args{$_};
            if (not length($dds_id)) {
                $self->_set_err("Invalid value for argument '-dds_id'");
                return $ClientUtil::FALSE;
            }
        }
    } ## end foreach (keys %args)

    unless (defined $dd_id) {
        $self->_set_err("Did not pass in the value for argument '-dd_id'");
        return $ClientUtil::FALSE;
    }
    unless (defined $dds_id) {
        $self->_set_err("Did not pass in the value for argument '-dds_id'");
        return $ClientUtil::FALSE;
    }

    # Remove 'filer and initiator iscsi nodenames' from Discovery Domain.
    $self->remove_iqn_from_discovery_domain(-all   => 1,
                                            -dd_id => $dd_id);
    if ($self->error()) {
        $self->_set_err("remove_iqn_from_discovery_domain failed."
                          . " dd_id=$dd_id, all=1",
                        $self->errmsg());
        return $ClientUtil::FALSE;
    }

    # Remove Discovery Domain from Discovery Domain Set.
    $self->remove_discovery_domain_from_discovery_domain_set(
                                                            -dd_ids => [$dd_id],
                                                            -dds_id => $dds_id);
    if ($self->error()) {
        $self->_set_err(
                     "remove_discovery_domain_from_discovery_domain_set failed."
                       . " set=$dds_id, domain=$dd_id",
                     $self->errmsg());
        return $ClientUtil::FALSE;
    }

    # Destroy Discovery Domain.
    $self->destroy_discovery_domain(-dd_id => $dd_id);
    if ($self->error()) {
        $self->_set_err("destroy_discovery_domain failed, domain=$dd_id",
                        $self->errmsg());
        return $ClientUtil::FALSE;
    }

    # Destroy Discovery Domain Set.
    $self->destroy_discovery_domain_set(-dds_id => $dds_id);
    if ($self->error()) {
        $self->_set_err("destroy_discovery_domain_set failed, set=$dds_id",
                        $self->errmsg());
        return $ClientUtil::FALSE;
    }

    # If all the steps passed, then return $SanUtil::TRUE.
    return $ClientUtil::TRUE;

} ## end sub destroy_basic_discovery_domain_and_set

###############################################################################
#                               Private Methods
###############################################################################

###############################################################################
# Method:    _process_isns_hash
# Objective: Compares hash returned by get_isns_info to what is reported by the
#            filer
# Details:   See POD documentation at the end of this file.  Note this method
#            is for 7-mode!!!
# Inputs   : -filer_obj - required - filer object
#            -isnshash  - required - ref to hash returned from from get_isns_info
# Returns  : $ClientUtil::TRUE - Successful execution
#            $ClientUtil::FALSE & set_err - Execution failed
###############################################################################
sub _process_isns_hash {
    my ($self, %args) = @_;

    my ($filer_obj, $isns_href, $verified);
    $self->_clear_err();

    # initialize the variables
    $verified = 0;    # Verification flag
    $self->sanlog(-msg => "Started _process_isns_hash");

    # Parse arguments
    foreach (keys %args) {
        if (/-?filer_obj/) {
            $filer_obj = $args{$_};
            if (not length($filer_obj)) {
                $self->_set_err("Invalid value for argument '-filer_obj'");
                return $ClientUtil::FALSE;
            }
        } elsif (/-?isnshash/) {
            $isns_href = $args{$_};
            if (not ref($isns_href) eq "HASH") {
                $self->_set_err("Invalid value for argument '-isnshash'");
                return $ClientUtil::FALSE;
            }
        }
    }    # end foreach arg

    # check for existance of required arguments
    unless (ref($filer_obj) =~ /^San::SanFiler7M$/i) {
        $self->_set_err(  "Invalid value passed in for -filer_obj,"
                        . " value needs to be a SanFiler Object");
        return $ClientUtil::FALSE;
    }
    if (scalar(keys(%{$isns_href})) == 0) {
        $self->_set_err("Missing required argument '-isnshash'");
        return $ClientUtil::FALSE;
    }

    # get the filer name for using in error messages
    my $filer_name = $filer_obj->get_name();
    if ($filer_obj->error()) {
        $self->_set_err("get_name method failed", $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # get the host name for using in error messages
    my $isns_name = $self->hostname();

    # Dump the isns hash for debugging purpose if needed
    my $dumpOut = Dumper($isns_href);
    $self->sanlog(-msg       => "Output isns-hash is:\n$dumpOut",
                  -msg_level => 3);

    # Grab isns show output to verify against the isns hash.
    my $showop = $filer_obj->iscsi_isns_show();
    if ($filer_obj->error()) {
        $self->_set_err("Filer $filer_name: iscsi_isns_show failed",
                        $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # Dump the iscsi isns show hash for debugging purpose if needed
    $dumpOut = Dumper($showop);
    $self->sanlog(-msg       => "Output iscsi-isns-show is:\n$dumpOut",
                  -msg_level => 3);

    # ipaddress stores ip from isns show command on filer.
    # isns_ip_add stores ip address from the hash on isns server.
    my $ipaddress  = $showop->{"isns-ip-addr"};
    my $isns_ipadd = $isns_href->{isns_info}->{ip_address};

    # Scan each ip address on the isns server to see if it matches the
    # isns ip address specified on the filer.
    foreach my $ip (keys %{$isns_ipadd}) {
        if (   ($ipaddress eq $isns_ipadd->{$ip}->{ipv4})
            or ($ipaddress eq $isns_ipadd->{$ip}->{ipv6})) {
            $verified = 1;
            last;
        }
    }
    # If filer ipaddress is not found in isns info hash, return error.
    if (!$verified) {
        $self->sanlog(-msg => "Filer $filer_name says isns ip address is "
                        . "$ipaddress.\nBut this ip address was not found "
                        . "on the server $isns_name!",
                      -msg_level => 1);
        return $ClientUtil::FALSE;
    }

    # reset the verification flag
    $verified = 0;

    # Make sure the entity id matches.
    if ($showop->{"isns-entity-id"} ne
        $isns_href->{isns_info}->{"isns-entity-id"}) {
        $self->sanlog(-msg => "Filer $filer_name says isns entity id is "
                        . $showop->{"isns-entity-id"}
                        . ".\nBut isns server $isns_name said it was "
                        . $isns_href->{isns_info}->{"isns-entity-id"},
                      -msg_level => 1);
        return $ClientUtil::FALSE;
    }

    # Make sure the is-isns-enabled id matches.
    if ($showop->{"is-isns-enabled"} ne
        $isns_href->{isns_info}->{"is-isns-enabled"}) {
        $self->sanlog(-msg => "Filer $filer_name say isns status is "
                        . $showop->{"is-isns-enabled"}
                        . ".\nBut isns server $isns_name says it is "
                        . $isns_href->{isns_info}->{"is-isns-enabled"},
                      -msg_level => 1);
        return $ClientUtil::FALSE;
    }

    # verify iscsi portal info fields ( ip-address, ip-port, tpgroup-tag )
    # Logic used here is to first take a unique field (in this case ip_address)
    # from isns hash to compare it to the ouput hash of iscsi-portal-show. At
    # the same time compare ip-port and tpgroup-tag if ip-address is matching.
    # Otherwise, set error and return ClientUtil::FALSE.
    my $portalop = $filer_obj->iscsi_portal_show();
    if ($filer_obj->error()) {
        $self->_set_err("Filer $filer_name: iscsi_portal_show failed",
                        $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # Dump the iscsi portal show hash for debugging purpose if needed
    $dumpOut = Dumper($portalop);
    $self->sanlog(-msg       => "Output iscsi-portal-show is:\n$dumpOut",
                  -msg_level => 3);

    # isns_portals are from the isns server hash
    # filer_portals are from the portal show cmd on filer.
    my $isns_portals = $isns_href->{portal_info};
    my $filer_portals =
      $portalop->{"iscsi-portal-list-entries"}
      ->{"iscsi-portal-list-entry-info"};
    # verify the isns portal hash count from both isns server and filer
    if (scalar(@$isns_portals) != scalar(@$filer_portals)) {
        $self->sanlog(
                -msg => "Filer $filer_name says there are "
              . scalar(@$filer_portals)
              . " portal(s).\nBut isns server $isns_name says there are "
              . scalar(@$isns_portals),
            -msg_level => 1);
        return $ClientUtil::FALSE;
    }

    # verify the ip-address, tcp_port, tpgroup_tag fields from isns hash with
    # data from filer.  Start by scanning each portal from the hash.
    foreach my $isns_portal (@$isns_portals) {

        # Scan each portal from filer, looking for a match.
        foreach my $filer_portal (@$filer_portals) {
            if($filer_portal->{'ip-address'} =~ /:/) {
               # Converting the ipv6 address "fd20:8b1e:b255:831e:2a0:98ff:fe08:4414 "
               # into standard ipv4 address "254.8.68.20" format. Since ISNS Server
               # isnscli lnd <nodaname> Eg :iqn.1992-08.com.netapp:ssan-6080-3 command
               # output is  return in ipv4 address format.
               my ($dig1, $dig2) = $filer_portal->{'ip-address'} =~ /(\w+):(\w+)$/;
               my ($d1, $d2) = $dig1 =~ /(.{2})(.{2})/;
               my ($d3, $d4) = $dig2 =~ /(.{2})(.{2})/;
               my  $dec1 = hex($d1);
               my  $dec2 = hex($d2);
               my  $dec3 = hex($d3);
               my  $dec4 = hex($d4);
               $filer_portal->{'ip-address'}
                      = "$dec1"."."."$dec2"."."."$dec3"."."."$dec4";
             }

            # If the ip address matches
            if ($isns_portal->{"ip-address"} eq $filer_portal->{"ip-address"}) {

                # verify for tcp_port field
                if ($isns_portal->{"ip-port"} ne $filer_portal->{"ip-port"}) {
                    $self->sanlog(-msg => "For ip address "
                                    . $isns_portal->{"ip-address"}
                                    . ":ip-port:\nisns server $isns_name says:"
                                    . $isns_portal->{"ip-port"}
                                    . " and filer $filer_name says:"
                                    . $filer_portal->{"ip-port"},
                                  -msg_level => 1);
                    return $ClientUtil::FALSE;
                } ## end if ($isns_portal->"ip-port"...

                # verify for tpgroup_tag field
                if ($isns_portal->{"tpgroup-tag"} ne
                    $filer_portal->{"tpgroup-tag"}) {
                    $self->sanlog(
                               -msg => "For ip address "
                                 . $isns_portal->{"ip-address"}
                                 . ":tpgroup-tag:\nisns server $isns_name says:"
                                 . $isns_portal->{"tpgroup-tag"}
                                 . " and filer $filer_name says:"
                                 . $filer_portal->{"tpgroup-tag"},
                               -msg_level => 1);
                    return $ClientUtil::FALSE;
                } ## end if ($isns_portal->"tpgroup-tag"...

                # if the code reaches here, it means we checked the ip-port
                # and tpgroup-tags for this ip address and they matched properly.
                # Use last to exist the inner for loop.
                $verified = 1;
                last;
            } ## end if ($isns_portal->"ip-address"...

        } ## end foreach my $filer_portal (@$filer_portals)

        # we scanned all of the filer portals and didn't find a match for
        # the portal reported by the isns server, then we need to exit.
        if (not $verified) {
            $self->sanlog(
                    -msg => "isns server $isns_name reported this portal:"
                      . $isns_portal->{"ip-address"}
                      . ".\nBut portal was not found on the filer $filer_name!",
                    -msg_level => 1);
            return $ClientUtil::FALSE;
        } else {
            $verified = 0;    # reset the verification flag
            $self->sanlog(-msg => "portal: "
                            . $isns_portal->{"ip-address"}
                            . " was verified.",
                          -msg_level => 3);
        }
    } ## end foreach my $isns_portal (@$isns_portals)

    # Verify  iscsi nodename
    my $iscsi_nodename = $filer_obj->get_iscsi_nodename();
    if ($filer_obj->error()) {
        $self->_set_err("Filer $filer_name: get_iscsi_nodename failed",
                        $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # If iscsi nodename does not match, raise error.
    if ($iscsi_nodename ne $isns_href->{nodename}) {
        $self->sanlog(-msg => "filer $filer_name says iscsi nodename is "
                        . "$iscsi_nodename.\nBut isns server $isns_name says:"
                        . $isns_href->{nodename},
                      -msg_level => 1);
        return $ClientUtil::FALSE;
    }

    # Verify iscsi alias
    my $iscsi_alias = $filer_obj->get_iscsi_alias();
    if ($filer_obj->error()) {
        $self->_set_err("Filer $filer_name: get_iscsi_alias failed",
                        $filer_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    # If iscsi alias does not match, raise error.
    if ($iscsi_alias ne $isns_href->{alias}) {
        $self->sanlog(-msg => "filer $filer_name says iscsi alias is "
                        . "$iscsi_alias.\nBut isns server $isns_name says:"
                        . $isns_href->{alias},
                      -msg_level => 1);
        return $ClientUtil::FALSE;
    }
    $self->sanlog(-msg => "Successfully completed _process_isns_hash");
    return $ClientUtil::TRUE;
}    # end of _process_isns_hash

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

WinIsnsServer.pm

=head1 Author

Srinivasarao Kante

=head1 Description

WinIsnsServer.pm provides an API interface to a host machine through the
use of the Telnet library.

=head1 See Also

Package inherits from the following modules. See these modules for further
interface detail.

B<San::SanClient,San::Sanlog,San::IsnsServer,San::SanClient::ClientUtil,Tharn, Error>

=head1 Public Methods

=head2 B<_new(%args)>

Creates constructor for Windows iSNS server object. Uses the IsnsServer object to start the isns service.

=over 4

=item Inputs

=item * -parent(required)

IsnsServer Object

=item Outputs

winisnsserver object

=item Examples

 $self = WinIsnsServer->_new(-parent => $self);

=back

=head2 B<isns_start()>

Start iSNS service on the iSNS server

=over 4

=item Inputs

none

=item Outputs

$ClientUtil::TRUE - iSNS started
$ClientUtil::FALSE & set_err - something went wrong

=item Examples

 $result = $self->isns_start();

=back

=head2 B<isns_stop()>

Stop iSNS service on the iSNS server

=over 4

=item Inputs

None

=item Outputs

$ClientUtil::TRUE - if iSNS stopped successfully
$ClientUtil::FALSE & set_err - something went wrong

=item Examples

 $result = $IsnsServer_Obj->isns_stop();

=back

=head2 B<create_discovery_domain(%args)>

Create discovery domain on isns server

=over 4

=item Inputs

=item * -name (required)

Required domain name

=item Outputs

Returns $DD_ID, if discovery domain created.

Returns "" and set_err - if discovery domain not created.

=item Examples

 $dd_id = $Isns_Obj->create_discovery_domain(-name => "mydomain");

=back

=head2 B<destroy_discovery_domain(%args)>

Destroy discovery domain on isns server

=over 4

=item Inputs

=item * -dd_id (required)

Required discovery domain ID

=item Outputs

Returns $ClientUtil::TRUE, if discovery domain destroyed

Returns $ClientUtil::FALSE and  set_err, if discovery domain not destroyed.

=item Examples

 $Isns_Obj->destroy_discovery_domain(-dd_id => 5);

=back

=head2 B<create_discovery_domain_set(%args)>

Create discovery domain set on isns server

=over 4

=item Inputs

=item * -name (required)

Required Domain set name

=item Outputs

Returns $DDS_ID, if discovery domain set created.

Returns "" and set_err, if discovery domain set not created.

=item Examples

 $Isns_Obj->create_discovery_domain_set(-name => "myset");

=back

=head2 B<destroy_discovery_domain_set(%args)>

Destroy discovery domain set on isns server

=over 4

=item Inputs

=item * -dds_id (required)

Required discovery domain set ID

=item Outputs

Returns $ClientUtil::TRUE, if discovery domain set destroyed.

Returns $ClientUtil::FALSE and set_err, if discovery domain set not destroyed.

=item Examples

 $Isns_Obj->destroy_discovery_domain_set(-dds_id => 4);

=back

=head2 B<verify_isns_registration(%args)>

Verifies the filer's registration information on the iSNS server matches what
is reported by the filer.

=over 4

=item Inputs

=item * -filer_obj (required)

Filer object to verify iSNS registration.

=item * I<-force_isns_update> (optional)

-force_isns_update = 1: To use "iscsi iSNS update" forcefully by calling iscsi_isns_update(),
-force_isns_update = 0: Not to use "iscsi iSNS update", (default is -force_isns_update = 1).

=item * I<-retries> (optional)

Number of times to retry if filer not found or registration information is stale.
Retries must be greater than or equal to 0 and there will be an initial attempt before start counting retries (default is -retries = 20).

Note: If the user specified 'force_isns_update' = 1, then the value of 'retries' will be ignored.

=item Outputs

$ClientUtil::TRUE, Filer's registration with iSNS server is verified.

$ClientUtil::FALSE & set_err, Filer did not register properly with iSNS Server or there was an execution error.

=item Supported Interfaces

cli, zapi

=item Examples

 my $var1 = $IsnsServerObj->verify_isns_registration(-filer_obj         => $SanFilerObj,
                                                     -force_isns_update => 1);

 my $var2 = $IsnsServerObj->verify_isns_registration(-filer_obj         => $SanFilerObj,
                                                     -force_isns_update => 0,
                                                     -retries           => 10);

=back

=head2 B<get_isns_info(%args)>

Gathers filer's iSNS registration info from the iSNS server.
NOTE: This is the 7 mode variation.

=over 4

=item Inputs

=item * -nodename (required)

Filer iscsi nodename for which you wish to retrieve registration details

=item Outputs

Reference to a hash of data containing: nodename, alias, portal_info, isns_info

NULL & set_err - Filer encountered some problem

NULL           - Target not registered

=item Examples

 $hashref = $self->get_isns_info(-nodename => "iqn.1992-08.com.netapp:sn.118041539-6030-6");
 The isns hash output is:
 $VAR1 = {
           'isns_info' => {
                            'isns-entity-id' => 'isns:00000002',
                            'is-isns-enabled' => 'true',
                            'ip_address' => {
                                              '0' => {
                                                       'interface_name' => 'Local Area Connection',
                                                       'ipv6' => 'fe80::f53f:332c:d2cb:8d11',
                                                       'ipv4' => '10.229.136.162',
                                                       'gateway' => 'fe80::5:73ff:fea0:3e%10',
                                                       'subnet_mask' => '255.255.248.0',
                                                       'mtusize' => '',
                                                       'ipv6_subnet' => 'fe80::',
                                                       'ipv6_prefix_len' => '64',
                                                       'ipv4_subnet' => '10.229.136',
                                                       'description' => 'Intel(R) PRO/1000 MT Network Connection',
                                                       'broadcast' => '',
                                                       'is_mgmt_interface' => 1,
                                                       'mac' => '0050569201AB'
                                                     }
                                            }
                          },
           'nodename' => 'iqn.1992-08.com.netapp:sn.1573744830',
           'portal_info' => [
                              {
                                'tpgroup-tag' => '1000',
                                'ip-address' => '10.228.194.133',
                                'ip-port' => '3260'
                              }
                            ],
           'alias' => 'mike'
         };

=back

=head2 B<get_isns_info_cmode(%args)>

Gathers filer's iSNS registration info from the iSNS server.
NOTE: This is the C mode variation.

=over 4

=item Inputs

=item * -nodename (required)

Filer iscsi nodename for which you wish to retrieve registration details

=item -deregistered (optional)

Set to 1 if checking if the target has de-registered from the iSNS server.
Set to 0 if checking if the target has registered with the iSNS server. (default)

=item Outputs

Reference to a hash of data containing: nodename, alias, interface, tpgroup,
isns, status, and show.

NULL & set_err - Filer encountered some problem

NULL           - Target not registered

=item Examples

 $hashref = $self->get_isns_info_cmode(-nodename => "iqn.1992-08.com.netapp:sn.d4b9d89619f111e193b9123478563412:vs.5");
 The isns hash output is:

 $VAR1 = {
           'show' => {
                       'status-admin' => 'up',
                       'target-name' => 'iqn.1992-08.com.netapp:sn.d4b9d89619f111e193b9123478563412:vs.5',
                       'target-alias' => 'vs3'
                     },
           'tpgroup' => [
                          {
                            'tpgroup-tag' => 1034,
                            'lif' => [
                                       '10.228.129.127'
                                     ]
                          },
                          {
                            'tpgroup-tag' => 1035,
                            'lif' => [
                                       '10.228.130.127'
                                     ]
                          }
                        ],
           'nodename' => 'iqn.1992-08.com.netapp:sn.d4b9d89619f111e193b9123478563412:vs.5',
           'status' => {
                         'status-admin' => 'up',
                         'is-available' => 'true'
                       },
           'interface' => [
                            {
                              'is-home' => 'true',
                              'tpgroup-tag' => '1034',
                              'address' => '10.228.129.127',
                              'ip-port' => '3260',
                              'enabled' => 'true'
                            },
                            {
                              'is-home' => 'true',
                              'tpgroup-tag' => '1035',
                              'address' => '10.228.130.127',
                              'ip-port' => '3260',
                              'enabled' => 'true'
                            }
                          ],
           'isns' => {
                       'entity-id' => 'isns:00000016',
                       'status-admin' => 'up',
                       'address' => {
                                      '1' => {
                                               'interface_name' => 'Local Area Connection',
                                               'ipv6' => 'fe80::1ce6:d923:e818:d7b8',
                                               'ipv4' => '10.229.136.188',
                                               'gateway' => 'fe80::5:73ff:fea0:3e%10',
                                               'subnet_mask' => '255.255.248.0',
                                               'mtusize' => '',
                                               'ipv6_subnet' => 'fe80::',
                                               'ipv6_prefix_len' => '64',
                                               'ipv4_subnet' => '10.229.136',
                                               'description' => 'Intel(R) PRO/1000 MT Network Connection',
                                               'broadcast' => '',
                                               'is_mgmt_interface' => 1,
                                               'mac' => '00505692013C'
                                             },
                                      '0' => {
                                               'interface_name' => 'Local Area Connection 2',
                                               'ipv6' => 'fe80::882a:b2e4:1062:643b',
                                               'ipv4' => '10.229.137.235',
                                               'gateway' => 'fe80::5:73ff:fea0:3e%12',
                                               'subnet_mask' => '255.255.248.0',
                                               'mtusize' => '',
                                               'ipv6_subnet' => 'fe80::',
                                               'ipv6_prefix_len' => '64',
                                               'ipv4_subnet' => '10.229.136',
                                               'description' => 'vmxnet3 Ethernet Adapter',
                                               'broadcast' => '',
                                               'is_mgmt_interface' => 0,
                                               'mac' => '005056AC06B1'
                                             }
                                    }
                     },
           'alias' => 'vs3'
         };

=back

=head2 B<verify_isns_deregistration(%args)>

Verifying the given node name is no longer registered with isns server.

=over 4

=item Inputs

=item * -filer_obj (required)

The Filer object.

=item * I<-retries> (optional)

Number of times to retry if filer is still registered.
There will be an initial attempt before start counting retries (default is -retries = 20).

=item Outputs

Returns $ClientUtil::TRUE, if the filer is NOT registered with isns server.

Returns $ClientUtil::FALSE and set_err means filer is still registered or there was another problem.
If an error occured the error flag and the errmsg is set.

=item Examples

 $result = $self->verify_isns_deregistration(-filer_obj => $filerobj, -retries => 2);

=back

=head2 B<add_discovery_domain_to_discovery_domain_set(%args)>

Add discovery domain to discovery domain set on isns server

=over 4

=item Inputs

=item * -dds_id (required)

Required discovery_domain_set_id

=item * -dd_ids (required)

Required reference to array of discovery_domain_ids

=item Outputs

Returns $ClientUtil::TRUE, if added dd_ids to dds_id.

Returns $ClientUtil::FALSE and set_err - if not added dd_ids to dds_id.

=item Examples

 $Isns_Obj->add_discovery_domain_to_discovery_domain_set(-dd_ids => \@dd_ids
                                                         -dds_id => $dds_id);

=back

=head2 B<remove_discovery_domain_from_discovery_domain_set(%args)>

Remove discovery domain from discovery domain set on isns server

=over 4

=item Inputs

=item * -dds_id (required)

Required discovery_domain_set_id

=item * -dd_ids (required)

Required reference to array of discovery_domain_ids

=item Outputs

Returns $ClientUtil::TRUE, if removed dd_ids from dds_id.

Returns $ClientUtil::FALSE and set_err - if not removed dd_ids from dds_id.

=item Examples

 $Isns_Obj->remove_discovery_domain_from_discovery_domain_set(-dd_ids => \@dd_ids
                                                              -dds_id => $dds_id);

=back

=head2 B<enable_discovery_domain_set(%args)>

Enable discovery domain set on isns server

=over 4

=item Inputs

=item * -dds_ids (required)

Required reference to array of discovery_domain_set_id

=item Outputs

Returns $ClientUtil::TRUE, if enable dds_id on iSNS server.

Returns $ClientUtil::FALSE and set_err - if fail to enable dds_id.

=item Examples

 $Isns_Obj->enable_discovery_domain_set(-dds_ids => \@dds_ids);

=back

=head2 B<disable_discovery_domain_set(%args)>

Disable discovery domain set on isns server

=over 4

=item Inputs

=item * -dds_ids (required)

Required reference to array of discovery_domain_set_id

=item Outputs

Returns $ClientUtil::TRUE, if disable dds_id on iSNS server.

Returns $ClientUtil::FALSE and set_err - if fail to disable dds_id.

=item Examples

 $Isns_Obj->disable_discovery_domain_set(-dds_ids => \@dds_ids);

=back

=head2 B<add_iqn_to_discovery_domain(%args)>

Adds one or more iqns to a discovery domain on the iSNS sever.

=over 4

=item Inputs

=item * -iqns (required)

Reference to an array of filer and initiator iscsi nodenames to add to the Discovery Domain.

=item * -dd_id (required)

Discovery Domain ID

=item Outputs

$ClientUtil::FALSE & set_err    - if something went wrong.

$ClientUtil::TRUE               - if all iqns added to discovery domain.

=item Examples

 $result = $self->add_iqn_to_discovery_domain(-iqns => \@iqnnodes,
                                              -dd_id => $dd_id);

=back

=head2 B<remove_iqn_from_discovery_domain(%args)>

Removes one or more iqns from a discovery domain on the iSNS sever. Or user may
specify -all flag to remove all iqns from the discovery domain.

=over 4

=item Inputs

One of the two optional parameters is required.

=item * I<-iqns> (optional)

Reference to an array of iscsi nodenames to remove from the Discovery Domain.

=item * I<-all> (optional)

If flag is set to 1, all iscsi nodenames will be removed from the discovery domain.
If flag is set to 0, provided list of -iqns by user will be removed from discovery domain.
(default = 0)

=item * -dd_id (required)

Discovery Domain ID

=item Outputs

$ClientUtil::FALSE & set_err    - if something went wrong.

$ClientUtil::TRUE               - if all iqns removed from discovery domain.

=item Examples

 $result = $self->remove_iqn_from_discovery_domain(-iqns => \@iqnnodes, # optional
                                                   -all => 1, # optional
                                                   -dd_id => $dd_id); # required

=back

=head2 B<list_discovery_domain_node_members(%args)>

Returns an array of all iqns in the specified discovery domain on the iSNS sever.

=over 4

=item Inputs

=item * -dd_id (required)

Discovery Domain ID.

=item Outputs

Array of all iqns in the specified discovery domain (Empty array if there are no members) - Successful execution.

Return () & set_err - Execution failed.

=item Examples

 @result = $IsnsServerObj->list_discovery_domain_node_members(-dd_id => 1);

=back

=head2 B<list_all_discovery_domain_sets_with_members()>

Returns an HASH of all Discovery Domain Sets with Discovery Domain members on the iSNS server.

=over 4

=item Inputs

None

=item Outputs

HASH, contains key as dds_id (Discovery Domain Set ID), value as dds_status, dds_name and
Hash of DD members with dd_id(Discovery Domain ID) as key and dd_name as value - Successful execution.

return {} & set_err - Execution failed.

=item Examples

 %result = $IsnsServerObj->list_all_discovery_domain_sets_with_members();
 resulting hash looks like:
 $VAR1 = {
    '35' => {
            'domains' => {'1' => 'Default DD'},
            'status'   => 'DISABLED',
            'dds_name' => 'DiscoDomain90guf9gu'
            },
    '1' =>  {'domains' => {'32' => 'DiscoDomain',
                          '1'  => 'Default DD'},
            'status'   => 'ENABLED',
            'dds_name' => 'Default DDS'
            },
    '32' => {'domains'  => {'32' => 'DiscoDomain'},
             'status'   => 'DISABLED',
             'dds_name' => 'DiscoDomainSet'
            },
    '33' => {'domains'  => {'32' => 'DiscoDomain'},
             'status'   => 'ENABLED',
             'dds_name' => 'DiscoDomainSet1'
            },
    '34' => {'domains'  => {},
             'status'   => 'ENABLED',
             'dds_name' => 'DiscoDomainSet007'
            },
    '36' => {'domains'  => {'32' => 'DiscoDomain'},
             'status'   => 'ENABLED',
             'dds_name' => 'DomainiSNS'
            }
};

=back

=head2 B<create_basic_discovery_domain_and_set(%args)>

Creates one Discovery Domain and one Discovery Domain Set on the iSNS sever.
The iqns are added to the discovery domain, the Discovery Domain is added to the Discovery Domain Set and the Set is enabled.
Returns an HASH, contains information about Discovery Domain ID and Discovery Domain Set ID.

=over 4

=item Inputs

=item * -iqns (required)

Reference to array of filer and initiator iscsi nodenames to include in the Discovery Domain.

=item * I<-dd_name> (optional)

Name of Discovery Domain - default 'domain1'

=item * I<-dds_name> (optional)

Name of Discovery Domain Set - default 'set1'

=item Outputs

HASH, contains value for dd_id (Discovery Domain ID) and dds_id (Discovery Domain Set ID) - Successful execution.

return {} & set_err - Execution failed.

=item Examples

 %result = $IsnsServerObj->create_basic_discovery_domain_and_set(-iqns     => \@Array_of_nodenames,
                                                                 -dd_name  => 'domainABC',
                                                                 -dds_name => 'setABC');

 %result = $IsnsServerObj->create_basic_discovery_domain_and_set(-iqns => ['iqn.1991-05.com.microsoft:sanqa-isns-4']);

=back

=head2 B<destroy_basic_discovery_domain_and_set(%args)>

Destroys a basic Discovery Domain and Set on the iSNS sever that was created by create_basic_discovery_domain_and_set.

=over 4

=item Inputs

=item * -dd_id (required)

ID of Discovery Domain.

=item * -dds_id (required)

ID of Discovery Domain Set.

=item Outputs

$ClientUtil::TRUE, Discovery Domain and Set destroyed.

$ClientUtil::FALSE & set_err, Discovery Domain and Set not destroyed or there was an execution error.

=item Examples

 $Status = $IsnsServerObj->destroy_basic_discovery_domain_and_set(-dd_id  => $dd_id,
                                                                  -dds_id => $dds_id);

 $Status = $IsnsServerObj->destroy_basic_discovery_domain_and_set(%Input_hash);

 Note: %Input_hash can be returned value of create_basic_discovery_domain_and_set(%args) of WinIsnsServer.pm.

=back

=head2 B<_process_isns_hash(%args)>

Compares hash returned by get_isns_info to what is reported by the filer.
Note this method is for 7 mode only.

=over 4

=item Inputs

=item * -filer_obj (required)

Filer Object

=item * -isnshash (required)

Hash reference returned by get_isns_info()

=item Outputs

$ClientUtil::FALSE & set_err    - if filer commands fails or other serious issue.

$ClientUtil::FALSE & NO set_err - the iSNS hash does not match the current filer state.

$ClientUtil::TRUE               - if everything matches.

=item Examples

 $result = $self->_process_isns_hash(-filer_obj => $SanFilerObj,
                                     -isnshash => $output_isnshash);

=back

=cut
