# Copyright (c) 2010,2011 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to MP software
## @author hrivera@netapp.com, arrance@netapp.com
## @status private
## @pod ../MP.pod

use strict;

# Package Name
##############
package San::SanClient::MP::ESXiMP;

# Package Requirements
######################
use San::SanClient::ClientUtil;
use San::SanLog;
use San::SanClient::MP;
use Error;
use Data::Dumper;
use vars qw(@ISA);

@ISA = qw(Error SanLog);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: new()
# Objective:       Create a new ESXMP object based on mptype specified
# Details:         See POD documentation at the end of this file
################################################################################
sub _new {
    my ($class, %args) = @_;

    my ($self, $parent, $mp_type);

    # bless
    $self = {};
    bless $self, $class;

    # parse arguments
    foreach (keys %args) {
        if (/-?mp_type/) {
            if (not length($mp_type = $args{$_})) {
                $self->_set_err("Invalid value for -mp_type: $args{$_}");
                return $self;
            }
            next;
        } elsif (/-?parent/) {
            if (not length($parent = delete $args{$_})) {
                $self->_set_err("Invalid value for -parent: $args{$_}");
                return $self;
            }
            next;
        }
    }

    $self->{parent} = $parent;

    # pull out the OS version
    my $os_version = $parent->{parent}->get_os_version();
    if ($mp_type =~ /native/i and $os_version =~ /^(4|5|6)\./) {
        # if doing ESX4 use the MP libs for ESX4
        require San::SanClient::MP::ESXiMP::NativeMP4;
        $self = San::SanClient::MP::ESXiMP::NativeMP4->_new(-parent => $self,
                                                            %args);
    } else {
        $self->_set_err("Invalid mp_type '$mp_type'");
        return $self;
    }

    return $self;
}

1;
