# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to MP software
## @author latif@netapp.com, rawat@netapp.com, thomasm@netapp.com
## @status private
## @pod ../MP.pod

use strict;

# Package Name
##############
package San::SanClient::MP::LinuxMP;

# Package Requirements
######################
use San::SanClient::ClientUtil;
use San::SanClient::MP;
use San::SanLog;
use Error;

use vars qw(@ISA);

@ISA = qw(Error SanLog);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: new()
# Objective:       Create a new LinuxMP object based on mptype specified
# Details:         See POD documentation at the end of this file
################################################################################
sub _new {
    my ($class, %args) = @_;

    my ($self, $parent, $mp_type);

    # bless
    $self = {};
    bless $self, $class;

    # parse arguments
    foreach (keys %args) {
        if (/-?mp_type/) {
            if (not length($mp_type = $args{$_})) {
                $self->_set_err("Invalid value for -mp_type: $args{$_}");
                return $self;
            }
            next;
        } elsif (/-?parent/) {
            if (not length($parent = delete $args{$_})) {
                $self->_set_err("Invalid value for -parent: $args{$_}");
                return $self;
            }
            next;
        }
    }

    $self->{parent} = $parent;
    $self->{conn}   = $parent->{parent}->get_connection();

    # Check different Multipathing types and require the correct module.
    if ($mp_type =~ /dmmp/i) {
        require San::SanClient::MP::LinuxMP::DMMP;
        $self = San::SanClient::MP::LinuxMP::DMMP->_new(-parent => $self,
                                                        %args);
    } elsif ($mp_type =~ /dmp/i) {
        require San::SanClient::MP::LinuxMP::DMP;
        $self = San::SanClient::MP::LinuxMP::DMP->_new(-parent => $self,
                                                       %args);
    } else {
        $self->_set_err("Invalid mp_type ($mp_type)");
        return $self;
    }

    return ($self);

}

1;
