use strict;
use warnings;

use File::Basename;
use San::SanClient::MPDevMgr::MPDevPath;

package San::SanClient::MPDevMgr::LinuxMPDevMgr::LinuxMPDev;

sub _new {
    my ($self, %args) = @_;

    return $self;
}

sub _refresh {
    my ($self, %args) = @_;

    my $sc_obj = $self->{sc_obj};

    if ($self->mp_type() =~ /dmmp/i) {
        $self->_refresh_dmmp_device(%args);
        return $self;
    } else {
        $self->_set_err("Invalid mp_type");
        return $self;
    }

}

sub _refresh_dmmp_device {
    my ($self) = @_;

    my $mp_device_name = $self->mp_device_name();
    my $sc_obj = $self->{sc_obj};

    require San::ScsiTool;
    require San::ScsiTool::ScsiUtil;

    my $scsi_tool_obj = ScsiTool->create_scsi_tool(-client => $sc_obj,
                                                   -tool   => "spt");
    if ($scsi_tool_obj->error()) {
        $self->_set_err($scsi_tool_obj->errmsg());
        return $self;
    }

    my $scsi_util_obj = San::ScsiTool::ScsiUtil->new(-scsi_tool_obj => $scsi_tool_obj);
    if ($scsi_util_obj->error()) {
        $self->_set_err($scsi_util_obj->errmsg());
        return $self;
    }

    my $cmd = "multipath -ll $mp_device_name";
    my ($status2, @output2) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
    if ($status2 != 0) {
        $self->_set_err("Failed");
        return $self;
    }

    my $device_vendor;
    my $device_model;
    my $device_revision;
    my $device_size;
    my $lun_id;
    my $lun_paths;
    my $scsi_level;
    my $protocol;
    my $protocol_path_counts;

    foreach my $line (@output2) {
        # non user friendly names
        if ($line =~ /${mp_device_name}\s+dm\-\d+\s+(NETAPP),(LUN(?: C\-Mode)?\b)/ or
            $line =~ /${mp_device_name}\s+\(360\S+?\)\s+dm\-\d+\s+(NETAPP),(LUN(?: C\-Mode)?\b)/) {
            $device_vendor = $1;
            $device_model = $2;
            next;
        }

        if ($line =~ /\[?size=(\S+?)]?/) {
            $device_size = $1;
            next;
        }

        if ($line =~ /(?:_|\-)\s+(\d+):(\d+:\d+):(\d+)\s+(\S+)\s+\S+\s+\[?(active|ready|faulty|failed)\]?\s*\[?(active|ready|faulty|failed)\]?/) {
            my $lun_path;
            my $this_paths_protocol;
            my $path_id = $1 . ":" . $2 . ":" . $3;
            my $target_id = $1 . ":" . $2;
            $lun_id = $3;
            my $raw_scsi_device_name = "/dev/" . $4;
            my $path_state = "[$5][$6]";
            $path_state = "dead" if ($path_state =~ /faulty|failed/i);
            $path_state = "active" if ($path_state =~ /active|ready/i);
            my $alua_state;

            # verify that this file exists
            # /x/eng/localtest/arch/i386-redhat-linux8/bin/
            &ClientUtil::execute($sc_obj->{conn}, "touch /x/eng/localtest/arch/");

            # determine the alua state of this path
            my $ret_val = $scsi_tool_obj->test_scsi_cmd(-device => $raw_scsi_device_name,
                                                        -cdb_name => "inquiry_page_C1h");
            if (ref($ret_val) eq "HASH" and exists($ret_val->{SCSI_DATA})) {
                my $s_data = $scsi_util_obj->scsi_bytes_to_strings(-scsi_data_ref => $ret_val->{SCSI_DATA},
                                                                   -scsi_cmd_name => "inquiry_page_C1h");
                if (ref($s_data) eq "HASH") {
                    if ($s_data->{PROXY} == 1) {
                        $alua_state = "ANO";
                    } elsif ($s_data->{PROXY} == 0) {
                        $alua_state = "AO";
                    }
                } else {
                    $alua_state = "DOWN";
                }
            } else {
                $alua_state = "UNKNOWN";
            }

            # determine scsi level and device revision
            # cat /sys/class/scsi_device/${path_id}/device/rev
            # cat /sys/class/scsi_device/${path_id}/device/scsi_level
            $cmd = "cat /sys/class/scsi_device/${path_id}/device/rev";
            my ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
            if ($status3 != 0) {
                $self->_set_err("Failed");
                return $self;
            }
            $device_revision = $output3[0];

            $cmd = "cat /sys/class/scsi_device/${path_id}/device/scsi_level";
            ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
            if ($status3 != 0) {
                $self->_set_err("Failed");
                return $self;
            }
            $scsi_level = $output3[0];

            # determine the protocol of this path
            # look at: ls -l /sys/class/scsi_device/$path_id | grep device
            # fibrechannel =~ /(rport-\S+?)\/target${target_id}\/${path_id}/
            # store $rport_id
            # iscsi =~ /session(\d+)\/target${target_id}\/${path_id}/
            # store $session_id = session$1 and $session_id_num = $1
            my $rport_id;
            my $session_id;
            my $session_id_num;
            my $hba_id;

            $cmd = "ls -l /sys/class/scsi_device/${path_id}";
            ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
            if ($status3 != 0) {
                $self->_set_err("Failed");
                return $self;
            }
            @output3 = grep(/devices\/platform\/|devices\/pci/, @output3);

            if ($output3[0] =~ /(host\d+)\/(rport-\S+?)\/target${target_id}\/${path_id}/) {
                $hba_id = $1;
                $rport_id = $2;

                $protocol_path_counts->{fcp}++;
                $this_paths_protocol = "fcp";
            } elsif ($output3[0] =~ /(host\d+)\/session(\d+)\/target${target_id}\/${path_id}/) {
                $hba_id = $1;
                $session_id = "session" . $2;
                $session_id_num = $2;

                $protocol_path_counts->{iscsi}++;
                $this_paths_protocol = "iscsi";
            }

            my $mp_dev_path_obj;

            # determine the target iqn node name and tpgt
            # iscsi:
            # cat /sys/class/iscsi_session/${session_id}/targetname
            # cat /sys/class/iscsi_session/${session_id}/tpgt
            if ($this_paths_protocol eq "iscsi") {
                $cmd = "cat /sys/class/iscsi_session/${session_id}/targetname";
                ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                if ($status3 != 0) {
                    $self->_set_err("Failed");
                    return $self;
                }
                my $target_iqn_name = $output3[0];

                $cmd = "cat /sys/class/iscsi_session/${session_id}/tpgt";
                ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                if ($status3 != 0) {
                    $self->_set_err("Failed");
                    return $self;
                }
                my $tpgt = $output3[0];

                # determine the ip address of target session if iscsi
                # cat /sys/class/iscsi_connection/connection${session_id_num}:0/address
                # cat /sys/class/iscsi_connection/connection${session_id_num}:0/port
                $cmd = "cat /sys/class/iscsi_connection/connection${session_id_num}:0/address";
                ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                if ($status3 != 0) {
                    $self->_set_err("Failed");
                    return $self;
                }
                my $target_ip_address = $output3[0];

                $cmd = "cat /sys/class/iscsi_connection/connection${session_id_num}:0/port";
                ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                if ($status3 != 0) {
                    $self->_set_err("Failed");
                    return $self;
                }
                my $target_ip_port = $output3[0];

                my $initiator_iqn;

                # parse the initiator nodename
                if (defined($hba_id)) {
                    $cmd = "cat /sys/class/iscsi_host/${hba_id}/initiatorname";
                    ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                    if ($status3 != 0) {
                        $self->_set_err("Failed");
                        return $self;
                    }
                    $initiator_iqn = $output3[0];
                    if ($initiator_iqn =~ /NULL/i) {
                        # then it's probably sw iscsi, grab it from the init list
                        my @inits = $sc_obj->get_initiators();
                        foreach my $init (@inits) {
                            if ($init->{firmware} eq "software") {
                                $initiator_iqn = $init->get_nodename();
                                last;
                            }
                        }
                    }
                }

                $mp_dev_path_obj = new San::SanClient::MPDevMgr::MPDevPath(-path_name => $raw_scsi_device_name,
                                                                           -lun_id => $lun_id,
                                                                           -path_state => $path_state,
                                                                           -initiator_iqn => $initiator_iqn,
                                                                           -target_iqn => $target_iqn_name,
                                                                           -target_iscsi_alias => "",
                                                                           -target_iscsi_session => "",
                                                                           -target_ip_address => $target_ip_address,
                                                                           -target_ip_port => $target_ip_port,
                                                                           -tpgt => $tpgt,
                                                                           -alua_state => $alua_state,
                                                                           -protocol => "iscsi");
            }

            # determine the wwpn and wwnn of the target
            # fcp:
            # cat /sys/class/fc_remote_ports/${rport_id}/node_name
            # cat /sys/class/fc_remote_ports/${rport_id}/port_name
            if ($this_paths_protocol eq "fcp") {
                $cmd = "cat /sys/class/fc_remote_ports/${rport_id}/node_name";
                ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                if ($status3 != 0) {
                    $self->_set_err("Failed");
                    return $self;
                }
                my $target_node_name = $output3[0];
                $target_node_name =~ s/^0x//;

                $cmd = "cat /sys/class/fc_remote_ports/${rport_id}/port_name";
                ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                if ($status3 != 0) {
                    $self->_set_err("Failed");
                    return $self;
                }
                my $target_port_name = $output3[0];
                $target_port_name =~ s/^0x//;

                # parse out the wwpn and wwnn of the initiator used
                # cat /sys/class/fc_host/host{$host_id}/node_name
                # cat /sys/class/fc_host/host{$host_id}/port_name
                my $initiator_port_name;
                my $initiator_node_name;

                if (defined($hba_id)) {
                    $cmd = "cat /sys/class/fc_host/${hba_id}/port_name";
                    ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                    if ($status3 != 0) {
                        $self->_set_err("Failed");
                        return $self;
                    }
                    $initiator_port_name = $output3[0];
                    $initiator_port_name =~ s/^0x//;

                    $cmd = "cat /sys/class/fc_host/${hba_id}/node_name";
                    ($status3, @output3) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
                    if ($status3 != 0) {
                        $self->_set_err("Failed");
                        return $self;
                    }
                    $initiator_node_name = $output3[0];
                    $initiator_node_name =~ s/^0x//;
                }

                $mp_dev_path_obj = new San::SanClient::MPDevMgr::MPDevPath(-path_name => $raw_scsi_device_name,
                                                                           -lun_id => $lun_id,
                                                                           -path_state => $path_state,
                                                                           -initiator_wwnn => $initiator_node_name,
                                                                           -initiator_wwpn => $initiator_port_name,
                                                                           -target_wwnn => $target_node_name,
                                                                           -target_wwpn => $target_port_name,
                                                                           -alua_state => $alua_state,
                                                                           -protocol => "fcp");
            }

            $protocol = $this_paths_protocol;
            $lun_paths->{$raw_scsi_device_name} = $mp_dev_path_obj;
        }
    }

    # note that the protocol for this mp device is mixed if it contains more
    # than one protocol
    $protocol = "mixed" if (scalar(keys %{$protocol_path_counts}) > 1);

    # assign all the devices
    $self->device_size($device_size);
    $self->protocol($protocol);
    $self->mp_device_path("/dev/mapper/" . $mp_device_name);
    $self->device_vendor($device_vendor);
    $self->device_model($device_model);
    $self->device_revision($device_revision);
    $self->device_scsi_level($scsi_level);
    $self->lun_id($lun_id);
    $self->alua("enabled");

    # add the newly discovered path state for this device
    # convert hash to array of path objects first
    my @path_objs = values %{$lun_paths};
    $self->_add_path_state(-mp_device_path_objs => \@path_objs);

    return $self;
}

sub _refresh_paths {
    my ($self, %args) = @_;

    # obtain the current path state,
    # for each path in in the current path state
    # clone each path object, and refresh the alua state, and
    # refresh the path state of the path.
    # after all paths have been refreshed, create a new path state

    my $curr_path_state_obj = $self->get_current_path_state();
    my @curr_path_objs = $curr_path_state_obj->get_path_objs();
    my @new_path_objs;
    my $mp_device_name = $self->mp_device_name();
    my $sc_obj = $self->{sc_obj};

    require San::ScsiTool;
    require San::ScsiTool::ScsiUtil;

    my $scsi_tool_obj = ScsiTool->create_scsi_tool(-client => $sc_obj,
                                                   -tool   => "spt");
    if ($scsi_tool_obj->error()) {
        $self->_set_err($scsi_tool_obj->errmsg());
        return $self;
    }

    my $scsi_util_obj = San::ScsiTool::ScsiUtil->new(-scsi_tool_obj => $scsi_tool_obj);
    if ($scsi_util_obj->error()) {
        $self->_set_err($scsi_util_obj->errmsg());
        return $self;
    }

    my $cmd = "multipath -ll $mp_device_name";
    my ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
    if ($status != 0) {
        $self->_set_err("Failed");
        return $self;
    }

    foreach my $path_obj (@curr_path_objs) {
        # build a new path object based on the input of
        my $new_path_obj = new San::SanClient::MPDevMgr::MPDevPath(%{$path_obj});

        # refresh the alua state and path state for this path
        my $raw_scsi_device_name = $new_path_obj->path_name();
        require File::Basename;
        my $raw_scsi_device_basename = &File::Basename::basename($raw_scsi_device_name);

        foreach my $line (@output) {
            if ($line =~ /(?:_|\-)\s+\d+:\d+:\d+:\d+\s+${raw_scsi_device_basename}\s+\S+\s+\[?(active|ready|faulty|failed)\]?\s*\[?(active|ready|faulty|failed)\]?/) {
                my $path_state = "[$1][$2]";
                $path_state = "dead" if ($path_state =~ /faulty|failed/i);
                $path_state = "active" if ($path_state =~ /active|ready/i);

                my $alua_state;

                # verify that this file exists
                # /x/eng/localtest/arch/i386-redhat-linux8/bin/
                &ClientUtil::execute($sc_obj->{conn}, "touch /x/eng/localtest/arch/");

                # determine the alua state of this path
                my $ret_val = $scsi_tool_obj->test_scsi_cmd(-device => $raw_scsi_device_name,
                                                            -cdb_name => "inquiry_page_C1h");
                if (ref($ret_val) eq "HASH" and exists($ret_val->{SCSI_DATA})) {
                    my $s_data = $scsi_util_obj->scsi_bytes_to_strings(-scsi_data_ref => $ret_val->{SCSI_DATA},
                                                                       -scsi_cmd_name => "inquiry_page_C1h");
                    if (ref($s_data) eq "HASH") {
                        if ($s_data->{PROXY} == 1) {
                            $alua_state = "ANO";
                        } elsif ($s_data->{PROXY} == 0) {
                            $alua_state = "AO";
                        }
                    } else {
                        $alua_state = "DOWN";
                    }
                } else {
                    $alua_state = "UNKNOWN";
                }

                # update the states
                $new_path_obj->path_state($path_state);
                $new_path_obj->alua_state($alua_state);
                last;
            }
        }

        push(@new_path_objs, $new_path_obj);
    }

    $self->_add_path_state(-mp_device_path_objs => \@new_path_objs);

    return;
}

sub _load {
    my ($self) = @_;

    # run the mount command and figure out where this device is mounted
    # and what filesystem it has

    my ($cmd, $status, @output);
    my $sc_obj = $self->{sc_obj};

    my $mount_device = $self->mp_device_path();

    # check if the device is partitioned
    $cmd = "fdisk -l $mount_device";
    ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
    if ($status > 0) {
        # the device definitely isnt partitioned
        $self->is_partitioned(0);
    } else {
        my @partitions;
        foreach my $line (@output) {
            if ($line =~ /^(${mount_device}\d+)\s+/) {
                push(@partitions, $1);
            }
        }

        if (scalar(@partitions)) {
            # add each partition to the partition list for this device
            $self->mp_device_partition_list(\@partitions);
            $self->is_partitioned(1);
        } else {
            # the device definitely isnt partitioned
            $self->is_partitioned(0);
        }
    }

    if ($self->is_partitioned()) {
        # grab the first partition from the list
        # and update the mount device
        $mount_device = $self->mp_device_partition_list()->[0];
    }

    # check to see if this mount device is mounted
    $cmd = "mount | grep '${mount_device} on'";
    ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
    if ($status > 0) {
        # mount point does not exist
        $self->is_mounted(0);

        # but it might have a filesystem, try to find out
        $cmd = "blkid ${mount_device}";
        ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
        if ($status == 0 and $output[0] =~ /\s+TYPE="(\S+)"/i) {
            # it has a filesystem
            $self->fs_type($1);
            $self->has_filesystem(1);
        } else {
            # no filesystem
            $self->fs_type("");
            $self->has_filesystem(0);
        }
    } else {
        # device is mounted, parse the output
        my $line = $output[0];
        if ($line =~ /^${mount_device} on (.+?) type (\S+)(?:\((.+?)\))?/) {
            my $mount_point = $1;
            my $fs_type = $2;
            my $mount_options = $3;

            # update the object with the mount data
            $self->mount_point($mount_point);
            $self->fs_type($fs_type);
            if (defined($mount_options)) {
                $self->set_os_specific_data(-mount_options => $mount_options);
            }

            # if it's mounted, it must have a filesystem
            $self->has_filesystem(1);
        }
    }

    return;
}

sub _mount {
    my ($self, %args) = @_;

    my ($cmd, $status, @output);
    my $sc_obj = $self->{sc_obj};

    # parse the arguments
    my $mount_point = exists($args{-mount_point}) ? $args{-mount_point} : undef;
    if (not defined($mount_point)) {
        $self->_set_err("Missing required -mount_point parameter");
        return "";
    }

    my $create_dir = exists($args{-create_dir}) ? $args{-create_dir} : 1;

    # if the device is partitioned, then grab the partition and mount it, otherwise mount
    # the raw device path
    my $mount_device = $self->mp_device_path();
    if ($self->is_partitioned()) {
        # grab the first partition from the list
        my $partition_list = $self->mp_device_partition_list();
        $mount_device = $partition_list->[0];
    }

    # check if requested to create the mount point directory
    if ($create_dir) {
        $cmd = "mkdir -p $mount_point";
        ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
        if ($status > 0) {
            $self->_set_er("Failed to create mount point directory $mount_point");
            return "";
        }
    }

    # mount the device
    $cmd = "mount ${mount_device} ${mount_point}";
    ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
    if ($status > 0) {
        $self->_set_er("Failed to mount ${mount_device} to ${mount_point}.");
        return "";
    }

    return $mount_point;
}

sub _unmount {
    my ($self) = @_;

    my ($cmd, $status, @output);
    my $sc_obj = $self->{sc_obj};

    my $mp_device_path = $self->mp_device_path();

    if ($self->is_mounted()) {
        my $unmount_dev = $mp_device_path;

        # if this device has a mount point, key off that instead
        if (defined($self->mount_point())) {
            $unmount_dev = $self->mount_point();
        } elsif (defined($self->is_partitioned())) {
            # if this device is partitioned, then grab the first partition and unmount it
            $unmount_dev = $self->mp_device_partition_list()->[0];
        }

        # unmount this device
        $cmd = "umount ${unmount_dev}";
        ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
        if ($status > 0) {
            $self->_set_er("Failed to unmount $mp_device_path (${unmount_dev})");
            return;
        }
    } else {
        $self->_set_err("Could not unmount '$mp_device_path', it is not currently mounted");
        return;
    }

    return;
}

sub _mkfs {
    my ($self, %args) = @_;

    my ($cmd, $status, @output);
    my $sc_obj = $self->{sc_obj};

    # parse the arguments
    my $fs_type = exists($args{-fs_type}) ? $args{-fs_type} : undef;
    if (not defined($fs_type)) {
        $self->_set_err("Missing required -fs_type parameter");
        return;
    }

    # by default mkfs the entire device
    my $mp_device_path = $self->mp_device_path();
    my $mkfs_device = $mp_device_path;

    # if this device is partitioned, then grab the first partition
    # to make the filesystem on
    if (defined($self->is_partitioned())) {
        $mkfs_device = $self->mp_device_partition_list()->[0];
    }

    # mkfs this device if it doesnt have a filesystem laid down
    if (not $self->has_filesystem()) {
        $cmd = "mkfs -t ${fs_type} ${mkfs_device}}";
        ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
        if ($status > 0) {
            $self->_set_er("Failed to mkfs ${mkfs_device}");
            return;
        }
    } else {
        $self->_set_err("Could not make filesystem on ${mkfs_device}, it already has a filesystem");
        return;
    }

    return;
}

sub _destroy_partition {
    my ($self) = @_;

    my $mp_device_path = $self->mp_device_path();
    my $sc_obj = $self->{sc_obj};

    my $cmd = "echo 'd\n1\nw\n' | fdisk ${mp_device_path}";
    my ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
    if ($status > 0 ) {
        $self->_set_err("Could not fdisk ${mp_device_path}");
        return [];
    }

    return;
}

sub _create_partition {
    my ($self) = @_;

    my $mp_device_path = $self->mp_device_path();
    my $sc_obj = $self->{sc_obj};

    my $cmd = "echo 'd\n1\nn\np\n1\n\n\nw\n' | fdisk ${mp_device_path}";
    my ($status, @output) = &ClientUtil::execute($sc_obj->{conn}, $cmd);
    if ($status > 0 ) {
        $self->_set_err("Could not fdisk ${mp_device_path}");
        return [];
    }

    # return the list of partitions made
    return ["${mp_device_path}1"];
}

###############################################################################
# Method:    online_device()
# Objective: online this device
###############################################################################
sub _online_device {
    my ($self, %args) = @_;

    # NOOP

    return;
}

###############################################################################
# Method:    offline_device()
# Objective: offline this device
###############################################################################
sub _offline_device {
    my ($self, %args) = @_;

    # NOOP

    return;
}

1;
