# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to sanlun utility
## @author hrivera@netapp.com
## @status public
## @pod ../Sanlun.pm

# Package Name
##############
package San::SanClient::Sanlun::VWinSanlun;

use strict;

# Package Requirements
######################
use San::SanClient::Sanlun;
use San::SanClient::ClientUtil;
use San::SanClient::Sanlun::WinSanlun;

use Error;
use San::SanLog;

use vars qw( @ISA );

@ISA = qw(San::SanClient::Sanlun::WinSanlun Error SanLog Sanlun);

################################################################################
# P U B L I C   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: display_adapters()
# Objective:       NOOP for virtual Windows machine
# Details:         See POD documentation at the end of this file
################################################################################
sub display_adapters {
    my ($self, $protocol) = @_;

    $self->_clear_err();

    return (wantarray()) ? () : [];
} 

################################################################################
# P R I V A T E   M E T H O D S
################################################################################

################################################################################
# Subroutine Name: _new()
# Objective:       Create a new sanlun object
# Details:         See POD documentation at the end of this file
################################################################################
sub _new {
    my ($class, %args) = @_;
    my $build;
    my $winclient;
    my $self;
    my $parent;

    $self={};

    bless $self, $class;

    #
    # parse arguments
    #
    foreach (keys %args) {
        if (/-?WinClient/) {
            $winclient = $args{$_};
            if (not (ref($winclient) eq "WinClient::Host")) {
                $self->_set_err("Error: WinSanlun::_new() Invalid value for argument '-WinClient'");
                return $self;
            }        
            next;
        } elsif (/-?parent/) {
            if (not length($parent = $args{$_})) {
                $self->_set_err("Error: WinSanlun::_new() Invalid value for argument '-parent'");
                return $self; 
            }        
            next;
        }
    }     
    
    #
    # check for existance of required -WinClient argument
    #
    unless (defined $winclient and length $winclient)
    {
        $self->_set_err("Error: WinSanlun::_new() ".
                        "Missing required argument '-WinClient'");
        # TBD - this error comes back as "3" not the actual string.
        return $self; 
    }
    $self->{WinClient} = $winclient;
    
    #
    # check for existance of required -parent argument
    #
    if (not defined($parent)) {
        $self->_set_err("Error: WinSanlun::_new()",
                        "Missing required argument '-parent'");
        return $self;
    }
    
    #
    # initilize the object members
    #
    $self->{WinClient} = $winclient;
    $self->{conn} = $parent->get_connection();
    $self->{protocol} = $parent->get_protocol();
    $self->{parent} = $parent;

    return $self;
}

1;
