# Copyright (c) 2007-2009 NetApp Inc.
# All rights reserved.
#
## @summary Provides common cluster methods
## @author latif@netapp.com
## @status private
## @pod here

use strict;

# Package Name
##############
package ClientClusterUtil;

# Package Requirements
######################
use Tharn;
use San::SanUtil;

################################################################################
# Subroutine Name: parse_cluster_param()
# Objective:       Provided CLIENT_CLUSTERS, returns hash of clusters and
#                  attributes.
# Details:         See POD documentation at the end of this file
################################################################################
sub parse_cluster_param {
    my (%args) = @_;

    my $cluster = "";
    my $cluster_hash;

    # parse out input arguments
    foreach (keys %args) {
        if (/-?cluster$/) {
            $cluster = $args{$_};
        }
    }

    # Get rid of any white space
    $cluster =~ s/ //g;

    # Split up the clusters on colon
    # Example: CLIENT_CLUSTERS="cluster1,[node1,node2]:cluster2[node3,node4]"
    my (@clusters) = split(/:/, $cluster);

    # Go through each cluster in the array and pull out cluster specific data
    # like nodes and any other stuff passed in.
    foreach my $cluster (@clusters) {
        my ($cluster_info, $clients);

        # Pull out the nodes
        if ($cluster =~ /(\S+?),\[(.+?)\]/) {
            $cluster_info = $1;
            $clients = $2;

            # Build out the hash values for the cluster in the cluster hash
            $cluster_hash->{$1}->{name} = $1;
            @{$cluster_hash->{$1}->{clients}} = split(/\,/, $clients);
        }
    }

    # Return the hash
    return $cluster_hash;
}

1;
