# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to Cluster by creating a SanClientClusterObject
## @author  gowtham@netapp.com
## @status
## @pod     here

package San::SanClientCluster::HPUX;

# Compiler directives.
use strict;

# Module import.
use Tharn;
use San::SanLog;
use San::SanClientCluster::ClientClusterUtil;
use Error;
use Exporter;
use vars qw(@EXPORT @ISA);

@ISA = qw(Error SanLog);

################################################################################
# Subroutine Name: _new()
# Objective:       Call libraries to create a new SanClientClusterObject
# Details:         Not much more to say about this really
################################################################################
sub _new {
    my ($class, %args) = @_;

    my ($self);
    $self = {};

    # Bless the self reference to the package class
    bless ($self, $class);

    # Validate and create the appropriate cluster object if the required parameter clustertype is available
    if (grep (/^-cluster_type$/, (keys %args))) {
        if ($args{-cluster_type} =~ /hpux-mcsg/i) {
            require San::SanClientCluster::HPUX::MCSG;
            $self = San::SanClientCluster::HPUX::MCSG->_new(%args);
        } elsif ($args{-cluster_type} =~ /vcs/i) {
            require San::SanClientCluster::HPUX::VCS;
            $self = San::SanClientCluster::HPUX::VCS->_new(%args);
        } else {
            $self->_set_err("Error: San::SanClientCluster::HPUX::_new() Cluster '" . $args{-cluster_type} . "'" . " type is invalid or not currently supported with this library");
            return $self;
        }
    } else {
        $self->_set_err("Error: San::SanClientCluster::HPUX::_new() Missing Cluster type (ex: rhcs, ocfs, or vcs)");
        return $self;
    }
}

1;
