# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to Cluster
## @author  rawat@netapp.com
## @status  Private
## @pod     here

package San::SanClientCluster::Linux;

use strict;

use Error;

use vars qw(@ISA);

@ISA = qw(Error SanLog);

sub _new {
    my ($class, %args) = @_;
    my ($self, $hosttype, $type, $cluster_ip, $cluster_name, $cluster_subnet, $client);

    $self = {};

    # bless the object upfront in case an error occurs.
    bless $self, $class;

    foreach (keys %args) {
        if (/-?client/) {
            if (not length($client = $args{$_})) {
                $self->_set_err("Invalid value for argument '-client'");
                return $self;
            }
            next;
        } elsif (/-?cluster_type/) {
            if (not length($type = $args{$_})) {
                $self->_set_err("Invalid value for argument '-cluster_type'");
                return $self;
            }
            next;
        } elsif (/-?cluster_ip/) {
            if (not length($cluster_ip = $args{$_})) {
                $self->_set_err("Invalid value for argument '-cluster_ip'");
                return $self;
            }
            next;
        } elsif (/-?cluster_subnet/) {
            if (not length($cluster_subnet = $args{$_})) {
                $self->_set_err("Invalid value for argument '-cluster_subnet'");
                return $self;
            }
            next;
        } elsif (/-?cluster_name/) {
            if (not length($cluster_name = $args{$_})) {
                $self->_set_err("Invalid value for argument '-cluster_name'");
                return $self;
            }
            next;
        }
    }
    # check for existance of required -cluster_type argument
    if (grep (/^-cluster_type$/, (keys %args))) {
        unless ($args{-cluster_type} =~ /vcs|ocfs|rhcs/i) {
            $self->_set_err("Unsupported or invalid Cluster type for argument '-cluster_type' \(" . $args{-cluster_type} . "\)");
            return $self;
        }
    } else {
        $self->_set_err("Missing Cluster type (ex: rhcs, ocfs, or vcs)");
        return $self;
    }

    # load the appropriate Cluster for Linux
    if ($type =~ /rhcs/i) {
        require San::SanClientCluster::Linux::RHCS;
        $self = San::SanClientCluster::Linux::RHCS->_new(%args);
    } elsif ($type =~ /vcs/i) {
        require San::SanClientCluster::Linux::VCS;
        $self = San::SanClientCluster::Linux::VCS->_new(%args);
    } elsif ($type =~ /ocfs/i) {
        require San::SanClientCluster::Linux::OCFS;
        $self = San::SanClientCluster::Linux::OCFS->_new(%args);
    } else {
        $self->_set_err("Cluster type'" . $type . "'not currently supported with this library");
        return $self;
    }

    return $self if ($self->error);

    $self->{parent} = $args{-parent};

    #
    # By returning $self here we are returning a parent class such
    # as RHCS, not a Cluster object.  This is OK because
    # the OS specific Cluster::XXXCluster classes add Cluster in @ISA
    # so Cluster is the base class for the parent
    #
    return $self;

}

1;

