#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/SanConfig.pm#1 $

# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to san configuration
## @author knopp@netapp.com
## @status private
## @pod here

package SanConfig;

use strict;
use lib();
use Data::Dumper;
use Exporter;
use vars qw (@ISA);
use San::SanClient::ClientUtil;
our @EXPORT = qw(
                 new
                 lun_cleanup
                 lun_display
                 lun_setup
                 lvm_cleanup
                 lvm_setup
                 rescan_if_needed


                 );

@ISA=qw(Exporter Error);


################################################################################
## P U B L I C    F U N C T I O N S  
################################################################################

################################################################################
# Subroutine Name: new()
# Objective:       Create a new Util object
# Details:         See POD documentation at the end of this file
################################################################################
sub new {
    my ($class, %args) = @_;
    my (
        $client,        
        $configpath,
        $devmap_refresh,       
        $log,
        $lvm,   
        $self,
        );

    $self={};

    bless $self,$class;    
    
    $configpath=0;

    # parse arguments
    foreach (keys %args) {
        if (/^-?client/i) {
            if (not length($client = $args{$_})) {
                $self->_set_err("Error: SanConfig::new()",
                                "Invalid value for argument '-client'");
                return $self;
            }
            next;
        }
        elsif (/^-?lvm/i) {
            if (not length($lvm = $args{$_})) {
                $self->_set_err("Error: SanConfig::new()",
                                "Invalid value for argument '-lvm'");
                return $self;
            }
            next;
        }
        elsif (/^-?devmap_refresh/i) {
            if (not length($devmap_refresh = $args{$_})) {
                $self->_set_err("Error: SanConfig::new()",
                                "Invalid value for argument '-devmap_refresh'");
                return $self;
            }
            next;
        }
        elsif (/^-?config/i) {
            if (not length($configpath = $args{$_})) {
                $self->_set_err("Error: SanConfig::new()",
                                "Invalid value for argument '-configpath'");
                return $self;
            }
            next;
        }
        elsif (/-?loghdl/i) {
            $log=$args{$_} if (defined $args{$_} and length($args{$_}));                        
            next;
        } 
        else {
            $self->_set_err("Error: SanConfig::new()",
                            "Invalid parameter specified \'$_\'");
            return $self;                            
        }
    }  
    
    ### check for existance of required -client argument
    unless (defined $client and ref($client) =~ "SanClient" ) {
        $self->_set_err("Error: SanConfig::new()",
                        "Missing or invalid argument '-clientObj'");
        return $self; 
    }

    if (defined $devmap_refresh and $devmap_refresh=~/1|0/){
        $self->{devmap_refresh}=$devmap_refresh;
    }

    # assume system does not require a rescan.
    # However this bit will be set whenever this
    # module removes a lun.
    # $self->_rescan(0);

    # initilize the object members
    $self->{client}=$client;
    $self->{lvm}=$lvm if defined($lvm);
    $self->{configpath}=$configpath;
    
    
    ### Allow CLI commands to be logged;
    $self->{log}=$log if (defined $log);

    return $self;

}


################################################################################
#Subroutine Name: lun_cleanup
#Objective:       destroy a named lun object
#Details:         See POD documentation at the end of this file
################################################################################
sub lun_cleanup {
    my ($self,%args) = @_;
    my (
        $fvol_obj,
        $prefix,
        $fail_cnt,
        $lun,
        $lun_cnt,
        @lun_objs,
        @err,
        @luns
        );      

    $self->_clear_err();


    $fail_cnt=0;

    ### parse arguments
    foreach (keys %args) {
        if (/^-?fvol|^-?filer_vol/i) {
            if (not length($fvol_obj = $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-fvol_obj'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?prefix/i) {
            if (not length($prefix = $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-prefix'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?lun/i) {
            if (not length($lun = $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-lun'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        else {
            $self->_set_err("Error: SanConfig::lun_setup()",
                            "Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;                            
        }
    }  

    ### check for existance of required -filer_vol (fvol) argument
    unless (defined $fvol_obj and ref($fvol_obj) eq "Volume" ) {
        $self->_set_err("Error: SanConfig::new()",
                        "Missing or invalid argument '-fvol_obj'");
        return $ClientUtil::FALSE; 
    } 

    unless (defined $prefix) {
        $self->_set_err("Error: SanConfig::lun_setup()",
                        "Missing required argument '-prefix'");
        return $ClientUtil::FALSE;
    }

    
    ### Delete the lun object and update the parent lun list.
    $self->_log("-- Unmap/destroy filer luns with prefix '".$prefix."' on ".
                $fvol_obj->get_parent()->get_name()."(".$fvol_obj->get_name().")");

    ### Refresh the filer object so that it has the latest list 
    ### of lun objects
    $fvol_obj->refresh();

    if ($lun) {
        @lun_objs=$fvol_obj->get_lun($lun);
        if ($fvol_obj->error()) {
            push @err,$fvol_obj->errmsg();
            $fail_cnt++;
        }
    } else {
        @lun_objs=$fvol_obj->get_luns();
        if ($fvol_obj->error()) {
            push @err,$fvol_obj->errmsg();
            $fail_cnt++;
        }
    }
    
    
    foreach my $lun_obj (@lun_objs){ 
        if ($lun_obj->get_name() =~ /^$prefix/) {

            while ($lun_obj->is_mapped()) {

                $lun_obj->unmap();
                if ($lun_obj->error) {
                    push @err,$lun_obj->errmsg();
                }
            }

            $lun_obj->destroy();
            if ($lun_obj->error) {
                push @err,$lun_obj->errmsg();
            }
            else {
                $self->_log("   - Lun '".$lun_obj->get_name()."' destroyed");
            }
            ### If luns are destroyed system will need a rescan to clear 
            ### device links
            $self->_rescan(1);
        } 
        
    }


    if (@err) {
        $self->_set_err(@err);
    }

    
    return(@err or $fail_cnt>0) ? $ClientUtil::FALSE : $ClientUtil::TRUE; 
}

################################################################################
#Subroutine Name: lun_display
#Objective:       verify_luncnt
#Details:         See POD documentation at the end of this file
################################################################################
sub lun_display{
    my ($self,%args)=@_;
    my (
        $bHeader,
        $fvol_obj,
        $prefix,
        $lun_cnt,
        $fail_cnt,
        $header,
        $summary,
        @buffer,
        @fvol_objs,
        );

    ### initialize variables
    
    $fail_cnt=0;
    $bHeader=1;
    
    ### parse arguments
    foreach (keys %args) {
        if (/^-?fvol_obj/i) {
            if (length($args{$_})) {
                push @fvol_objs, (ref($args{$_}) eq "ARRAY") ? @{$args{$_}} : $args{$_};
            }
            else {
                $self->_set_err("Error: SanConfig::lun_display()",
                                "Invalid value for argument '-fvol_objs'");
                return (wantarray()) ? () : [];
            }
            next;
        }
        elsif (/^-?prefix/i) {
            if (not length($prefix = $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_display()",
                                "Invalid value for argument '-prefix'");
                return (wantarray()) ? () : [];
            }
            next;
        }
        else {
            $self->_set_err("Error: SanConfig::lun_display()",
                            "Invalid parameter specified \'$_\'");
            return (wantarray()) ? () : [];                           
        }
    }  

    ### check for existance of required -fvolObj argument
    if (@fvol_objs){
        foreach my $fvol_obj (@fvol_objs){
            unless (ref ($fvol_obj) eq "Volume"){
                $self->_set_err("Error: SanConfig::lun_display()",
                                "Invalid value for -fvol_objs.");
                return (wantarray()) ? () : []; 
            }
        }
    }
    else {
        $self->_set_err("Error: SanConfig::lun_display()",
                        "Required argument missing -fvol_objs");
        return (wantarray()) ? () : [];  
    }


    ### Refresh the filer object
    foreach my $fvol_obj (@fvol_objs){
        
        $fvol_obj->refresh();

        foreach my $lun_obj ($fvol_obj->get_luns()){
            
            ### Only display luns with prefix
            if ($lun_obj->get_name()=~/^$prefix/i) {

                ### display header 
                if ($bHeader) {
                    
                    push @buffer,sprintf("Lun Detail:\n".
                                         "%-16.16s %-12.12s %-25.25s %-8.8s %-6.6s %-7.7s %-6.6s %-3.3s %-30.30s\n".
                                         "%-16.16s %-12.12s %-25.25s %-8.8s %-6.6s %-7.7s %-6.6s %-3.3s %-30.30s",
                                         "FILERNAME","VOLUME","NAME","SIZE","ACCESS","STATE","MAPPED","ID","IGROUP",
                                         "="x16,"="x12,"="x25,"="x8,"="x6,"="x7,"="x6,"="x3,"="x12);
                    
                    $bHeader=0;
                }
                
                push @buffer, sprintf("%-16.16s %-12.12s %-25.25s %-8.8s %-6.6s %-7.7s %-6.6s %-3.3s %-50s",
                                      $fvol_obj->{parent}->get_name(), 
                                      $fvol_obj->get_name(),
                                      $lun_obj->get_name(),
                                      $lun_obj->get_size(),
                                      $lun_obj->get_access(),
                                      $lun_obj->get_state(),
                                      ($lun_obj->is_mapped()) ? "Yes" : "No",
                                      $lun_obj->get_id(),
                                      $lun_obj->get_igroup());
                
                
                ### Display any additional igroups the lun is mapped too.
                my $primary_igroup=$lun_obj->get_igroup();
                my @maps=$lun_obj->get_maps();
                foreach my $map (@maps){
                    ### Skip over first igroup
                    next if ($map=~/^$primary_igroup$/i);
                    push @buffer,
                    sprintf("%-16.16s %-12.12s %-25.25s %-8.8s %-6.6s %-7.7s %-6.6s %-3.3s %-50s",
                            "",
                            "",
                            "",
                            "",
                            "",
                            "",
                            "",
                            $lun_obj->get_mapid($map),
                            $map
                            );

                }

            }

        }
    }

    push @buffer,"\n\n" if (@buffer);


    return (wantarray()) ? @buffer : [@buffer];
}


###############################################################################
# Method:    lun_setup
# Objective: create and map filer luns
# Details:   See POD documentation at the end of this file.
###############################################################################
sub lun_setup {
    my ($self,%args)=@_;
    my (
        $nbr,
        $size,
        $type,
        $fvol_obj,
        $igrp_obj,
        $igrp_name,
        $bHeader,
        $fail_cnt,
        $header,
        $prefix,
        $summary,
        $type_values,
        @buffer,
        @fvol_objs,
        );

    ### initialize variables 
    $fail_cnt=0;
    $prefix=undef;
    $type_values="solaris|hpux|aix|linux|vld|netware|vmware";
   
    ### parse arguments
    foreach (keys %args) {
        if (/^-?nbr/i) {
            if (not length($nbr = $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-nbr'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?fvol_obj/i) {
            if (length($args{$_})) {
                push @fvol_objs, (ref($args{$_}) eq "ARRAY") ? @{$args{$_}} : $args{$_};
            }
            else {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-fvol_objs'");
                return $self;
            }
            next;
        }
        elsif (/^-?igrp/i) {
            if (not length($igrp_name= $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-igrp_name'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?prefix/i) {
            if (not length($prefix = $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-prefix'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?size/i) {
            if (not length($size = $args{$_})) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-size'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?type/i) {
            if (not length($type = $args{$_}) and $type !~ /$type_values/i) {
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for argument '-type'",
                                "Valid values: ".join(",",split("|",$type_values)));
                return $ClientUtil::FALSE;
            }
            next;
        }
        else {
            $self->_set_err("Error: SanConfig::lun_setup()",
                            "Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;                            
        }
    }  

    ### check for existance of required -lvm argument
    ### check for existance of required -fvolObj argument
    if (@fvol_objs){
        foreach my $fvol_obj (@fvol_objs){
            unless (ref ($fvol_obj) eq "Volume"){
                $self->_set_err("Error: SanConfig::lun_setup()",
                                "Invalid value for -fvol_objs.");
                return $ClientUtil::FALSE;  
            }
        }
    }
    else {
        $self->_set_err("Error: SanConfig::lun_setup()",
                        "Required argument missing -fvol_objs");
        return $ClientUtil::FALSE;   
    }

    unless (defined $nbr) {
        $self->_set_err("Error: SanConfig::lun_setup()",
                        "Missing required argument '-nbr'");
        return $ClientUtil::FALSE;
    }

    unless (defined $size) {
        $self->_set_err("Error: SanConfig::lun_setup()",
                        "Missing required argument '-size'");
        return $ClientUtil::FALSE;
    }

    unless (defined $prefix) {
        $self->_set_err("Error: SanConfig::lun_setup()",
                        "Missing required argument '-prefix'");
        return $ClientUtil::FALSE;
    }

    ### If lun type is not supplied then lets default to the
    ### host platform type
    unless (defined $type) {
        ### Group linux flavors under same type 
        if ($self->{client}->hosttype()=~/linux/) {
            $type="linux";
        }
        else { 
            $type=$self->{client}->hosttype();
            $type=~s/unix-//g;
        }
    }


    foreach my $fvol_obj (@fvol_objs){

        if (defined $igrp_name){
            
            my $filer_obj=$fvol_obj->get_parent();
            $igrp_obj=$filer_obj->get_igroup(-name=>$igrp_name);
            if ($filer_obj->error) {
                $self->_set_err($filer_obj->errmsg);
                return $ClientUtil::FALSE;
            }
        }        
        foreach my $id (0..($nbr-1)) {
            my $lun_obj=$fvol_obj->create_lun(-name=>$prefix.$id,-size=>$size,-type=>$type);
            if ($fvol_obj->error) {
                $self->_set_err($fvol_obj->errmsg);
                $fail_cnt++;
            }
            else {
                ### Only map lun if igrp object was passed in
                if (defined $igrp_obj){
                    $lun_obj->map($igrp_obj->get_name());
                    if ($lun_obj->error) {
                        $self->_set_err($lun_obj->errmsg);
                        $fail_cnt++;
                    }
                }
            }
        }
    }

    return($fail_cnt==0) ? $ClientUtil::TRUE : $ClientUtil::FALSE;

}


########## CLEANUP#################################################

sub lvm_cleanup{
    my ($self,@vgnames)=@_;
    my (
        $devObj,
        @devObjs,
        @dtObjs,
        @errmsg,
        $vgObj,
        @vgObjs,
        @volObjs
        );

    $self->_clear_err();


    ### check for existance of required -lvm argument
    unless (defined $self->{lvm}) {
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        "A Lvm object was not passed into SanConfig::new() using parameter -lvm'");
        return $ClientUtil::FALSE; 
    }

    $self->_log("-- Retrieving existing disk groups from host");


    if (defined @vgnames and scalar(@vgnames)>0) {
        foreach my $vgname (@vgnames){
            my $vgObj=$self->{lvm}->get_vg($vgname);
            unless ($self->{lvm}->error){
                push @vgObjs,$vgObj;
            }
        }
    }
    else {
        @vgObjs=$self->{lvm}->get_vgs();
        if ($self->{lvm}->error){
            $self->_set_err($self->{lvm}->errmsg());
            return $ClientUtil::FALSE;
        }
    }

    foreach $vgObj (@vgObjs){
        $self->_log("\n-- Retrieving existing volumes in disk group '".$vgObj->get_name()."'");         
        @volObjs=$vgObj->get_vols();            
        foreach my $volObj (@volObjs){
            ### Unmount filesystem
            $self->_log("   - Unmount ".$volObj->get_name()." if mounted");             
            if ($volObj->is_mounted()){
                $volObj->unmount(-force=>1);
                if ($volObj->error){
                    my @err = $volObj->errmsg();
                    $err[0] =~ s/Error:/Warning:/;
                    push @errmsg, @err;
                }
            }

            $self->_log("   - Deleting volume '".$volObj->get_name()."'");
            $volObj->destroy();
            if ($volObj->error){
                push @errmsg, $volObj->errmsg;
            }

            # check for @errmsg, if populated, return
            if (@errmsg) {
                $self->_set_err(@errmsg);
                return $ClientUtil::FALSE;
            }
        }

        $self->_log("   - Deleting volume group '".$vgObj->get_name()."'");
        $vgObj->destroy();
        if ($vgObj->error){
            $self->_set_err($vgObj->errmsg);
            return $ClientUtil::FALSE;
        }               
    }

    return $ClientUtil::TRUE;

}

########## setup ###################################################
sub lvm_setup {
    my ($self,%args)=@_;
    my (
        $device_cnt,
        $devices,
        $devmap_refresh,
        $directio,
        $dgids, 
        $disk_fmts,
        $dg_prefix, 
        $fs_types,
        $filer_vol,
        $layouts,
        $lun_prefix,
        $mount_pt,
        $volids,
        $vol_sizes,
        $unique,
        $vol_cnts,
        @devices,
        @disk_fmts,
        @devs,
        @dgs,
        @fvol_objs,
        @fs,
        @layout,
        @luns,
        @mount,
        @size,
        @vols,
        @vol_cnts,
        %mnts,
        );
    
    $self->_clear_err();

    ### Clear Volume object cache
    #$self->{lvm}->clear_cache();

    #initialize variables.

    $disk_fmts='';
    $fs_types='raw';
    $devices=1;
    $dg_prefix='dg';
    $layouts="stripe";
    $mount_pt="/mnt/qa";
    $unique=0;
    
    ### refresh dev map on device discovery
    $devmap_refresh=(defined $self->{devmap_refresh})
        ? $self->{devmap_refresh}
    : 0;

    ### filer_vol to use
    #$filer_vol=(defined $self->{filer_vol})
    # ? $self->{filer_vol}
    # : "vol1";

    ### lun_prefix to use
    $lun_prefix=$self->{lun_prefix} if (defined $self->{lun_prefix});
    

    # parse arguments
    foreach (keys %args) {
        if (/^-?fvol_objs/i) {
            if (length($args{$_})) {
                push @fvol_objs, (ref($args{$_}) eq "ARRAY") ? @{$args{$_}} : $args{$_};
            }
            else {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-fvol_objs'");
                return $self;
            }
            next;
        }
        elsif (/^-?dg_devices/i) {
            if (not length($devices = $args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-dg_devices'");
                return $ClientUtil::FALSE;
            }
            else {
                @devices=$self->_get_ids($devices);

                unless (@devices){
                    $self->_set_err("Error: SanConfig::lvm_setup()",
                                    "Invalid value '$devices' for argument '-dg_devices'");
                    return $ClientUtil::FALSE;
                }
            }         
            next;
        }
        elsif (/^-?dg_ids/i) {
            if (not length($dgids = $args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-dg_ids'");
                return $ClientUtil::FALSE;
            }
            else {
                @dgs=$self->_get_ids($dgids);
                unless (@dgs){
                    $self->_set_err("Error: SanConfig::lvm_setup()",
                                    "Invalid value '$dgids' for argument '-dg_ids'");
                    return $ClientUtil::FALSE;
                }
            }
            next;
        }
        elsif (/^-?directio/i) {

            if (not length($directio = $args{$_}) or  $args{$_} !~ /1|0/) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-directio'");
                return $ClientUtil::FALSE;
            }
           
            next;
        }
        elsif (/^-?vol_cnts/i) {
            if (not defined $args{$_}) {
                $vol_cnts=0;
            }
            else {
                if (not length($vol_cnts = $args{$_})) {
                    $self->_set_err("Error: SanConfig::lvm_setup()",
                                    "Invalid value for argument '-vol_cnts'");
                    return $ClientUtil::FALSE;
                }
            }
            next;
        }
        elsif (/^-?vol_layouts/i) {
            if (not length($layouts = $args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-vol_layouts'");
                return $ClientUtil::FALSE;
            }
            next;
        } 
        elsif (/^-?vol_sizes/i) {
            if (not length($vol_sizes = $args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-vol_sizes'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?fs_type/i) {
            if (not defined $args{$_}) {
                $fs_types="raw" 
                } 
            else {
                if (not length($fs_types = $args{$_})) {
                    $self->_set_err("Error: SanConfig::lvm_setup()",
                                    "Invalid value for argument '-fs_types'");
                    return $ClientUtil::FALSE;
                }
            }
            next;
        }
        elsif (/^-?dg_prefix/i) {
            if (not length($dg_prefix=$args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-dg_prefix'");
                return $ClientUtil::FALSE;
            }        
            next;
        }
        elsif (/^-?lun_prefix/i) {
            if (not length($lun_prefix=$args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-lun_prefix'");
                return $ClientUtil::FALSE;
            }        
            next;
        }
        elsif (/^-?disk_fmts/i) {
            $disk_fmts = $args{$_};
            next;
        }
        elsif (/^-?dg_unique/i) {
            if (not length($unique = $args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-dg_unique'");
                return $ClientUtil::FALSE;
            }
            else {
                $unique=($unique=~/1|y|t|yes|true|o|on/i) ? "_$$" : "";
            }
            next;
        }
        elsif (/^-?mount_pt/i) {
            if (not length($mount_pt=$args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-mount_pt'");
                return $ClientUtil::FALSE;
            }        
            next;
        }
        elsif (/^-?devmap_refresh/i) {
            if (not length($devmap_refresh=$args{$_})) {
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for argument '-devmap_refresh'");
                return $ClientUtil::FALSE;
            }        
            next;
        }
        else {
            $self->_set_err("Error: SanConfig::lvm_setup()",
                            "Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;                            
        }

    }  


    ### check for existance of required -lvm argument
    unless (defined $self->{lvm}) {
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        "A Lvm object was not passed into SanConfig::new() using parameter -lvm'");
        return $ClientUtil::FALSE; 
    }

    unless (@dgs){
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        "Required argument missing -dg_ids");
        return $ClientUtil::FALSE;       
    }
    
    if (@fvol_objs){
        foreach my $fvol_obj (@fvol_objs){
            unless (ref ($fvol_obj) eq "Volume"){
                $self->_set_err("Error: SanConfig::lvm_setup()",
                                "Invalid value for -fvol_objs.");
                return $ClientUtil::FALSE;  
            }
        }
    }
    else {
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        "Required argument missing -fvol_objs");
        return $ClientUtil::FALSE;   
    }

    unless (defined $vol_cnts and length $vol_cnts){
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        "Required argument missing -vol_cnts");
        return $ClientUtil::FALSE;       
    }
    
    unless (defined $vol_sizes and length $vol_sizes){
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        "Required argument missing -vol_sizes");
        return $ClientUtil::FALSE;       
    }

    unless ($devmap_refresh=~/1|0/){
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        "Invalid value for -devmap_refresh");
        return $ClientUtil::FALSE;       
    }
    
    $self->_log("-- Discover host devices");
    $self->{client}->discover();
    if ($self->{client}->error) {
        $self->_log($self->{client}->errmsg());
        return $ClientUtil::FALSE;
    }

    # Sometimes it takes a while for Linux to see the LUNs
    # sleep (5) if ($self->{client}->hosttype()=~/lin/); 

    $self->_log("-- Rescan Lvm disk list ");
    $self->{lvm}->rescan();
    if ($self->{lvm}->error){
        $self->_set_err("Error: SanConfig::lvm_setup()",
                        $self->{lvm}->errmsg());
        return $ClientUtil::FALSE;
    }


    ### Repeat parameter values to match the number of 
    ### volume groups to create
    @devices=$self->_fill_array(scalar(@dgs),"@devices"); 
    @disk_fmts=$self->_fill_array(scalar(@dgs),$disk_fmts);
    @vol_cnts=$self->_fill_array(scalar(@dgs),$vol_cnts);

    ### init counters into parameter arrays
    my $device_idx=0;
    my $dg_idx=0;
    my $fvol_cnt=0;
    my $filer_cnt=0;

    #
    # Get a list of existing volume groups so we can check to see
    # if the current volume group exists.
    #
    my @all_dg_names = $self->{lvm}->get_vgnames();
    foreach my $dg_name (@all_dg_names)
    {
        $self->_log("Found Existing Volume Group " . $dg_name);
    }

    foreach my $dg_id (@dgs){
        #
        # Check to see if the disk group already exists.  If it does, we
        # do nothing (note this assumes that the matching name makes it
        # have the same contents i.e. the same volumes).
        #
        my $dg_found = 0;
        foreach my $dg_name (@all_dg_names)
        {
            if ($dg_name eq $dg_prefix.$dg_id.$unique)
            {
                $dg_idx++;
                $dg_found = 1;
            }
        } # end foreach dg_name

        if ($dg_found)
        {
            $self->_log("Using existing Volume Group: " . 
                        $dg_prefix.$dg_id.$unique);
            next;
        } else
        {
            $self->_log("Creating a new Volume Group: " .
                        $dg_prefix.$dg_id.$unique);
        }

        $device_cnt=$devices[$device_idx++];
        #my $fvol=$fvols[$fvol_cnt++];

        my $plural = ($device_cnt > 1) ? "s" : "";
        $self->_log("-- Get $device_cnt free device".$plural);
        
        ## initialize variables 
        @devs=();
        my $idx=0; 

        ## Retrieve requested free devices from each filer volume.
        ## Retrieve extra odd number device from first filer volume
        my $fvol_cnt=scalar(@fvol_objs);
        my @devcnt;

        
        ### Retrieve 1 device from each fvol_obj in the list supplied 
        ### until device requirment is satisfied.  Wrap around list if
        ### more devices than fvol_objs exist.

        my %hsh;
        my $cnt=1;
        while ($cnt <= $device_cnt){
            foreach my $fvol_obj (@fvol_objs){
                $hsh{$fvol_obj->get_parent()->get_name()}{$fvol_obj->get_name()}+=1;                     
                last if ($cnt++==$device_cnt);
            }            
        }

        ### IMPORTANT: Retrieval of free devices for any one filer volume should happen
        ### with one get_free_devices() call unless used immeidately.  Otherwise, 
        ### successive get_free_devices() on the same filer volume will return the 
        ### same devices since they have not been used yet.

        foreach my $filername (keys %hsh) {
            foreach my $fvolname (keys %{$hsh{$filername}}) {
                my $devcnt=$hsh{$filername}{$fvolname};

                $self->_log("-- Retrieve ".$devcnt." free devices from filer ".$filername."(".$fvolname.")");
                
                ## Retrieve free devices
                my %hDevParams=(-cnt=>$devcnt,-filer=>$filername,-vol=>$fvolname,-refresh=>$devmap_refresh);
                
                ## determine if user only wants to look for free luns 
                ## starting with a specific name.
                $hDevParams{-lun}=$lun_prefix if (defined $lun_prefix); 

                my @free_devices=$self->{lvm}->get_free_devices(%hDevParams);
                
                if ($self->{lvm}->error){
            $self->_set_err("Error: SanConfig::lvm_setup()",
                                    " Unable to locate free devices. ",
                            $self->{lvm}->errmsg());        
                    return $ClientUtil::FALSE;
                }
                
                ## Store array ref of device onto an array
                push @devs,[@free_devices];
            }
        }
        
        ## merge and alternate filers devices into single array 
        @devs=$self->_array_merge(\@devs);
        $self->_log("-- Device Order: @devs");   

        $self->_log("-- Initialize device".$plural." under lvm control");
        if ($disk_fmts[$dg_idx]) {
            $self->{lvm}->initialize(-devices=>\@devs,-format=> $disk_fmts[$dg_idx]);
        }
        else {
            $self->{lvm}->initialize(-devices=>\@devs);
        }
        if ($self->{lvm}->error){
            $self->_set_err("Error: SanConfig::lvm_setup()",
                                    " Unable to initialize device $plural. ",
                            $self->{lvm}->errmsg());        
            return $ClientUtil::FALSE;
        }
        
        
        $self->_log((defined $unique and length $unique)                     
                    ? "-- Create disk group '$dg_prefix".$dg_id.$unique."'"
                    : "-- Create disk group '$dg_prefix".$dg_id."'" );
        

        my $dgObj;
        if (exists $args{-no_add}) {
            $dgObj= $self->{lvm}->create_vg(-name=>$dg_prefix.$dg_id.$unique,-devices=>\@devs,-no_add);
        }
        else {
            $dgObj= $self->{lvm}->create_vg(-name=>$dg_prefix.$dg_id.$unique,-devices=>\@devs);
        }
        if ($self->{lvm}->error){
            $self->_set_err("Error: SanConfig::lvm_setup()",
                                    " Unable to create volume group. ",
                            $self->{lvm}->errmsg());        
            $self->_set_err($self->{lvm}->errmsg());    
            return $ClientUtil::FALSE;
        }
        
        ### Repeat values based on total number of volumes
        @layout=$self->_value_repeat($vol_cnts[$dg_idx],$layouts);
        @fs=$self->_value_repeat($vol_cnts[$dg_idx],$fs_types);
        @size=$self->_value_repeat($vol_cnts[$dg_idx],$vol_sizes);
        @mount=$self->_value_repeat($vol_cnts[$dg_idx],$mount_pt);

        my $vol_idx=0;
        foreach my $vol_id (1..$vol_cnts[$dg_idx]) {
            my $fs_type=$fs[$vol_idx];
            my $vol_size=$size[$vol_idx];
            my $layout=$layout[$vol_idx];
            my $mount=$mount[$vol_idx];
            $vol_idx++;
            $self->_log("   - Creating volume 'vol$vol_id'");
            my $volObj=$dgObj->create_vol(-name=>"vol$vol_id",-size=>$vol_size,-layout=>$layout);
            if ($dgObj->error){
            $self->_set_err("Error: SanConfig::lvm_setup()",
                            $dgObj->errmsg());      
                return $ClientUtil::FALSE;
            }
            
            ### Check if raw or filesystem
            if ($fs_type !~ /raw/i){
                $self->_log("   - Creating filesystem on volume vol$vol_id");
                $volObj->create_fs(-type=>$fs_type);
                if ($volObj->error){
                    $self->_set_err("Error: SanConfig::lvm_setup()",
                                    $volObj->errmsg()); 
                    return $ClientUtil::FALSE;
                }

                $self->_log("   - Mounting filesystem ");

                if (defined $mount and length $mount){
                    ### Remove trailing slash
                    $mount =~ s/\/$//;
                    $mnts{-path}=$mount if (length $mount);
                }
                
                $mnts{-directio}=$directio if (defined $directio);
                $volObj->mount(%mnts);
                if ($volObj->error){
                    $self->_set_err("Error: SanConfig::lvm_setup()",
                                    $volObj->errmsg()); 
                    return $ClientUtil::FALSE;
                }
            }
            else {
                $self->_log("   - volume vol$vol_id specified as raw");
            }
        }

        $dg_idx++;
    } # end foreach disk group


    ### Lets not make this user selectable.
    ### Run it every time so we don't run 
    ### into path misconfig issues
    $self->{client}->config_paths();
    if ($self->{client}->error()){
        $self->_set_err($self->{client}->errmsg());
        return $ClientUtil::FALSE;
    }
    
    return $ClientUtil::TRUE;
}

################################################################################
#Subroutine Name: host();
#Objective:       determine if host rescan is required
#Details:         See POD documentation at the end of this file
################################################################################
sub host_rescan {
    my ($self)=@_;

    $self->_clear_err();

    if ($self->_rescan()){
        $self->_discover();
    }

}

###############################################################################
## P R I V A T E    F U N C T I O N S  
###############################################################################
###############################################################################
# Method:    _array_compare
# Objective: Perform comparison of values between arrays
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _array_compare {
    my ($self,$a1,$a2)=@_;
    my (
        $count,
        $item,
        %count,
        %isect,
        %union,
        @diff,
        @isect,
        @union,
        );

    @union = @isect = @diff = ();
    %union = %isect = ();
    %count = ();

    @isect = @diff = @union = ();

    foreach $item (@$a1, @$a2){$count{$item}++;};

    foreach $item (keys %count) {
        push(@union, $item);
        push @{ $count{$item} == 2 ? \@isect : \@diff }, $item;
    }

    return (@diff) ? "1" : 0;

}
################################################################################
# Subroutine Name: _array_merge
# Objective:       return merge array of alternateing array elements
# Details:         See POD documentation at the end of this file
################################################################################
sub _array_merge {
    my ($self,$arr)=@_;
    my (
        $max,
        $size,
        $total_arrays,
        @c,
        );

    $max=0;

    ### Determine number of arrays
    $total_arrays=scalar(@{$arr});   
    
    ### Determine larget array
    foreach (0..($total_arrays-1)){
        $size=scalar(@{$arr->[0]});
        $max=($size>$max)?$size:$max;
    }    

    for(my $x=0;$x<$max;$x++){
        foreach my $idx (0..($total_arrays-1)){
            push @c,@$arr->[$idx][$x] if (defined @$arr->[$idx][$x]);
        }
    }
    
    return @c;
}

################################################################################
#Subroutine Name: _discover
#Objective:       discover devices on host
#Details:         See POD documentation at the end of this file
################################################################################
sub _discover {
    my ($self)=@_;
    my (
        $fail_cnt,
        $state,
        @err
        );
    
    $self->_clear_err();
    

    ### initialize device list
    $fail_cnt=0;
    
    
    ### Need to discover the new luns 
    $self->_log("-- Discover device list");
    $self->{client}->discover();        
    if ($self->{client}->error) {
        push @err,$self->{client}->errmsg();
        $fail_cnt++;
    }

    ### For HP-UX, setup pathing order
    $self->{client}->config_paths();
    if ($self->{client}->error) {
        push @err,$self->{client}->errmsg();
        $fail_cnt++;
    }    

    ### If LVM is involved then perform a lvm
    ### rescan as well so it picks up the device
    ### changes as well
    if (exists $self->{lvm}){
        $self->_log("-- Rescan Lvm device list ");
        $self->{lvm}->rescan();
        if ($self->{lvm}->error){
            push @err,$self->{lvm}->errmsg();
            $fail_cnt++;
        }   
    }

    if (@err) {
        $self->_set_err(@err);
    }
    else {
        $self->_rescan(0);
    }
    

    return(scalar(@err)==0 and $fail_cnt==0) ? $ClientUtil::TRUE : $ClientUtil::FALSE; 
}


###############################################################################
# Method:    _fill_array
# Objective: Generate array of repeated values to match cnt 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _fill_array {
    my ($self,$cnt,$pattern)= @_;
    my (
        $idx,
        @arr,
        @buf,
        );

    $idx=0;

    $pattern=~s/,|:|;|\\|\// /g;

    @arr=split(/\s+/,$pattern);

    foreach (1..$cnt){   
        if($idx ne scalar(@arr)){
            push @buf, $arr[$idx];
        }
        else {
            $idx=0;
            push @buf, $arr[$idx];
        }
        $idx++;
    }
    
    return (wantarray()) ? @buf : \@buf;
}


###############################################################################
# Method:    _get_ids
# Objective: Convert scalar fmt of ids ("1..3,6,15..12") into an array of ids
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _get_ids {
    my ($self,$value)=@_;
    my (
        $start,
        $end,
        @buf,
        @val,
        );
    
    @buf=split(/,/,$value); 
    foreach my $line (@buf){
        $start=$end=undef;

        if ($line=~/^\d+\.\.\d+$/){  ### Index List specified
            ($start,$end)=split(/\.\./,$line);
            if ($start<=$end){  
                while($start<=$end){                    
                    push @val,$start; 
                    $start++;           
                }
            }
            else {
                while($start>=$end){
                    push @val,$start;
                    $start--;
                }
            }
            
        }
        elsif ($line=~/^[0-9]{1,3}$/){ ### Index specified
            push @val,$line;
        }
        else {};
    }
    return @val;
}

###############################################################################
# Method:    _log
# Objective: Log test output to stdout and file
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _log{
    my ($self,@line) = @_;
    my (
        $bFirst,
        $dash,
        );
    
    $bFirst=1;
    
    if (defined $self->{log}){

        if ($line[0] eq "") {
            print {${$self->{log}}} "\n";
            return;
        }

        foreach my $line (@line){
        

            #$dash = ($bFirst == 1) ? "- " : "  ";
            #$bFirst=0;

            print {${$self->{log}}} "   ".$dash.$line."\n";
        }


    }

    Tharn::tstlog(-comment,@line);
}

################################################################################
#Subroutine Name: _rescan
#Objective:       set state of rescan if needed
#Details:         See POD documentation at the end of this file
################################################################################

sub _rescan {
    my ($self,$state)=@_;       

    if (defined $state and $state=~/0|1/i){
        $self->{rescan}=$state;
    }

    return $self->{rescan};
    
}

###############################################################################
# Method:    _value_repeat
# Objective: Generate array of repeated values to match pattern cnts 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _value_repeat {
    my ($self,$pattern,$values)= @_;
    my (
        $idx,
        @values,
        @pattern,
        @buf,
        );

    $idx=0;

    $pattern=~s/,|:|;|-|\\|\// /g;
    @pattern=split(/\s+/,$pattern);

    $values=~s/,|:|;|-/ /g;
    @values=split(/\s+/,$values);

    foreach my $pat (@pattern){
        $idx=0;
        foreach (1..$pat){      
            if($idx ne scalar(@values)){
                push @buf, $values[$idx];
            }
            else {
                $idx=0;
                push @buf, $values[$idx];
            }
            $idx++;
        }
    }
    
    return (wantarray()) ? @buf : \@buf;
}



1;

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

SanConfig.pm

=head1 Author

=begin html 

Gary Knopp

=end html

=head1 Description

SanConfig provides general functions for performing LVM configuration

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error.pm>

=head1 Attributes

=head2 B<client>

SanClient Object = A handle to the a Client object

=head2 B<lvm>

Lvm Object = A handle to the an Lvm object

=head2 B<configpath>

boolean = <1 | 0> Determines whether path configuration is automatically
performed for hosts that require it.

=head2 B<log>

Filehandle = Filehandle to log output to.

=head2 B<devmap_refresh>

boolean = <1 | 0> Determines whether cache of availble luns is refreshed prior
to each request for a free device.

=head2 B<lun_prefix>

string = A string you want luns to begine with (example: mylun)

=head2 B<rescan>

boolean = <1 | 0> Set if rescan needed

=head1 Public Methods

=head2 B<new(I<%args>)>

Creates a new SanConfig object. 

=over 4

=item Inputs

=item * -clientObj 

A handle to a Client object

=item * -lvmObj 

A handle to a Lvm object

=item * I<-configpath> (default: 0)

Determine whether path configuration is automatically performed 
for hosts that require it

=item * I<-devmap_refresh> (default: 0)

Determines whether cache of availble luns is refreshed prior
to each request for a free device.  This is automatically performed
on the first request for a free device.  However, if new luns are
being added by the test it should be set to 1 so these devices are
added to the lun cache and canbe used by the test.

=item * I<-loghdl> 

Filehandle to log output to.

=item Outputs

Returns a SanConfig object.  If an error occured the error flag is set, 
errmsg is set to the failure reason, and 0 is passed back 
as the return value.

=item Examples

   # Create a SanConfig 
   $Cfg=new SanConfig(
        -mtdObj=>$Test,
        -clientObj=>$Client,
        -lvmObj=>$Lvm,       
        -loghdl=>$Test->get_loghdl(),        
        );

    if ($Cfg->error){
        $Test->log($Cfg->errmsg());
        return $MTD::FAIL;
    }

=back

=head2 B<lun_cleanup(I<%args>)>

Destroy luns on a filer volume of a give name prefix

=over 4

=item Inputs

=item * -fvol_obj

Filer volume object to use for lun deletion

=item * -prefix

Lun name prefix

=item Outputs

Returns an scalar value of 1 if successful.  If an error occured the error 
flag is set, errmsg is set to the failure reason, and 0 is passed back as 
the return value.

=item Example


    ### Delete luns starting with Mylun"
    $Cfg->lun_cleanup(-fvol_obj=>$Fvol,-prefix=>"Mylun");
    if ($Cfg->error){ 
        $Test->nlog($Cfg->errmsg());
        return $MTD::FAIL;
    }   


=back

=head2 B<lun_display(I<%args>)>

Display luns on a filer volume starting with name prefix

=over 4

=item Inputs

=item * -fvol_obj

Filer volume object to use 

=item * -prefix

Lun name prefix

=item Outputs

Returns an scalar value of 1 if successful.  If an error occured the error 
flag is set, errmsg is set to the failure reason, and 0 is passed back as 
the return value.

=item Example


    ### dislay luns starting with Mylun"
    $Cfg->lun_display(-fvol_obj=>$fvol_obj,-prefix=>"Mylun");
    if ($Cfg->error){ 
        $Test->nlog($Cfg->errmsg());
        return $MTD::FAIL;
    }   


=back

=head2 B<lun_setup(I<%args>)>

Creates luns on a filer volumes

=over 4

=item Inputs

=item * -nbr 

The number of luns to create

=item * -fvol_obj

Filer volume object to use for lun creation

=item * I<-igrp_obj>  

Igroup name to use for mapping luns to host.  If not 
included luns will not be mapped.

=item * -prefix

Prefix to use for generating each lun name

=item * -size

Size to use for each lun.  A size unit should also 
be included as part of the size value.  For example '1g'.

=item Outputs

Returns an scalar value of 1 if successful.  If an error occured the error 
flag is set, errmsg is set to the failure reason, and 0 is passed back as 
the return value.

=item Example


    ### Create 5 1g luns with names my_lun0..my_lun4
    $Cfg->lun_setup(-fvol_obj=>$fvol_obj,-igrp_name=>"my_igp",-prefix=>"my_lun", -nbr=>5,-size=>"1g");
    if ($Cfg->error){ 
        $Test->nlog($Cfg->errmsg());
        return $MTD::FAIL;
    }   

=back

=head2 B<lvm_cleanup(@vgnames)>

Remove Lvm config off client machine

=over 4

=item Inputs

=item * I<@vgnames>

Array of volume group names to remove.  If no values are 
provided then all existing volume groups on the system will 
be removed.

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set,
errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Examples

    $Test->log("Remove volume content");
    if (not $Cfg->cleanup()){
        $Test->log($Cfg->errmsg);
    }

=back

=head2 B<lvm_setup(I<%args>)>

Creates a lvm configuration on the client

=over 4

=item Inputs

=item * I<-devmap_refresh> (default: 0)

Determines whether cache of availble luns is refreshed prior
to each request for a free device.  This is automatically performed
on the first request for a free device.  However, if new luns are
being added by the test it should be set to 1 so these devices are
added to the lun cache and canbe used by the test. This value can be
set globally in the 'new()' method.

=item * I<-dg_devices> (default: 1)

The number of devices to use for each diskgroup.  This value can be provided as 
a scalar (1) or scalar list (1..3,7).  Each disk group will be created with the 
next number of devices in the list.  In the scalar list example, the first dg has 
one device, the second 2 devices, the third 3 devices, and the fourth 7 devices.
The list will wrap around if there are more dgs than devices items.  A fifth dg
would be created with 1 device.

=item * I<-dg_prefix> (default is 'dg')

The prefix to use when creating a new disk group name.

=item * -dg_ids 

The ids to use when creating new disk groups.  This value can be a scalar or scalar
list (1..3,75).  This will create diskgroups with the name 'dg1','dg2','dg3','dg75'.

=item * -fvol_objs 

The filer volume object to use for locating free luns (devices) for disk group creation. This value 
is provided as an array reference of filer volume object ($fvol_obj1,$fvol_obj2). Each disk group device
will be  created on the next filer volume in the list.  The list will wrap around if there are more disk
group devices than filer volume objects.  From the example, if a disk group with 3 devices was created,
the first would use filer volume object $fvol_obj1, the second $fvol_obj2, and the third $fvol_obj1.

=item * I<-lun_prefix> (default is to use any free lun)

Select only lun names beginning with a specific prefix.

=item * -vol_cnts

The number of volumes to create for a disk group.  This value is provided as a scalar value
containing comma delimited values ('2,3')  This will create 2  volumes for the first
diskgroup and 3 volumes for the second diskgroup.  The list will wrap around if there are
more dikgroups than volume counts.  From the above example, a third diskgroup would be 
created with 2 volumes.

=item * I<-vol_layouts> (default: 'stripe')

The layout to use when creating new volumes.  This value is provided as a scalar value
containing comma delimited values ('stripe,concat'). Each volume will be created with 
the next layout in the list. The list will wrap around if there are more volumes 
than layout values.  From the example, if 3 volumes were created, the first layout would
be 'stripe', the second 'concat', and the third a 'stripe'.

=item * -vol_sizes (Note: include units 'k,m,g');

The size to use when creating new volumes. This value is provided as a scalar value
containing comma delimited values ('500m,1g,2g'). Each volume will be created with 
the next size in the list. The list will wrap around if there are more volumes 
than size values.  From the example, if 3 volumes were created, the first would 
be '500m', the second '1g', and the third '2g'.

=item * I<-fs_vals> (default: 'raw');

The filesystem to use when creating new volumes. This value is provided as a scalar value
containing comma delimited values ('vxfs,raw,raw'). Each volume will be created with 
the next filesystem in the list. The list will wrap around if there are more volumes 
than filesystem values.  From the example, if 4 volumes were created, the first would 
be 'vxfs', the second 'raw', the third 'raw', and fourth 'vxfs'.

=item * I<-dg_unique> (default: 0);

The process id will be appended to all disk groups created ('dg1_3483') 

=item * I<-dsk_fmts> (default: 'sliced' for Veritas 3.5 'cdsdisk' for Veritas 4.0);

The disk format (Veritas Only) to use when initiailizing new disks for a disk group
under veritas control. This value is provided as a scalar value containing comma delimited
values ('sliced,cdsdisk'). Each disk group will be created with the next disk format in the list.
The list will wrap around if there are more disk groups than disk format values.  From the example,
if 4 disk groups were created, the first would be 'sliced', the second 'cdsdisk', the third 'sliced',
and fourth 'cdsdisk'.  Volume managers that dont support this option will ignore it.

=item Outputs

Returns an scalar value of 1 if successful.  If an error occured the error 
flag is set, errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Examples


       ### Create 2 dgs with 3 striped 200m vxfs volumes 
       $rtn=$Cfg->setup(
            -lun_prefix=>"mylun",
            -dg_unique=>0,
            -dg_devices=>"2",
            -dg_ids=>"1..2",
            -vol_cnts=>"2,3",
            -vol_layouts=>"stripe",
            -vol_sizes=>"200m",
            -fvol_objs=[@fvols_objs],
            -fs_vals=>'vxfs',
            -dsk_fmts="sliced,cdsdisk"
       );

       if ($Cfg->error()){
           $Test->log($Cfg->errmsg());
           return $MTD::FAIL;
       }

=back

=head2 B<host_rescan()>

Rescan host if filer luns have been deleted

=over 4

=item Inputs

None

=item Outputs

Rediscovers host devices if any luns have been removed since the last
device discovery.  If an error occured the error flag is set, errmsg is
set to the failure reason.

=item Example

    ### Delete luns starting with Mylun"
    $Cfg->host_rescan()
    if ($Cfg->error){ 
        $Test->nlog($Cfg->errmsg());
        return $MTD::FAIL;
    }   


=back

=cut
