# Copyright (c) 2007-2011 NetApp Inc.
# All rights reserved.
#
## @summary Provides common methods that
## should be inherited by other object oriented
## packages
##
## @author jcoca@netapp.com
## @status Shared
## @pod here

package SanConsts;

use lib();

#
# Compiler directives.
#
use strict;

# module import
#
use Data::Dumper;
use San::SanLog;
use CmdsDriver;
use Error;

# Default Serial Number
use constant DEFAULT_SERIAL_NUMBER=>"1-80-000011";

# Serial Number and Licenses Codes constants.
use constant RollingRock_Licenses => {
                        '1-80-123456' =>{
                                         'base'       => "NWUZZJPJYBFDAA",
                                         'flexvol_hpo'=> "HPHLDKPJYBFDAA",
                                         'cifs'       => "JAMHCKPJYBFDAA",
                                         'snaprestore'=> "VUJWCKPJYBFDAA",
                                         'nfs'        => "TJFAEKPJYBFDAA",
                                         'snapmirror' => "FEDPEKPJYBFDAA",
                                         'striped_volume' => "DTYSFKPJYBFDAA",
                                         'iscsi'    => "RYAEFKPJYBFDAA",
                                         'fcp'      => "PNWHGKPJYBFDAA",
                                         'flexclone'=> "HPHLDKPJYBFDAA"
                                        },
                        '1-80-000011' =>{
                                          'base'       => "VMZIVZGKWVMADF",
                                          'mirror'     => "TBVMWZGKWVMADF",
                                          'flexvol_hpo'=> "FWSBXZGKWVMADF",
                                          'cifs'       => "RQQQXZGKWVMADF",
                                          'snaprestore'=> "DLOFYZGKWVMADF",
                                          'gx_base'       => "VMZIVZGKWVMADF",
                                          'gx_mirror'     => "TBVMWZGKWVMADF",
                                          'gx_flexvol_hpo'=> "FWSBXZGKWVMADF",
                                          'gx_cifs'       => "RQQQXZGKWVMADF",
                                          'gx_snaprestore'=> "DLOFYZGKWVMADF",
                                          'nfs'          => "BAKJZZGKWVMADF",
                                          'snapmirror'   => "NUHYZZGKWVMADF",
                                          'snapmirror_dp'=> "NUHYZZGKWVMADF",
                                          'striped_volume'=>"LJDCBAHKWVMADF",
                                          'iscsi'    => "ZOFNAAHKWVMADF",
                                          'fcp'      => "XDBRBAHKWVMADF",
                                          'flexclone'=> "PFMUYZGKWVMADF"
                                        },
                        '1-80-000013' =>{
                                          'base'       => "XBOLYLDPMZSEBG",
                                          'mirror'     => "VQJPZLDPMZSEBG",
                                          'flexvol_hpo'=> "HLHEAMDPMZSEBG",
                                          'cifs'       => "TFFTAMDPMZSEBG",
                                          'snaprestore'=> "FADIBMDPMZSEBG",
                                          'gx_base'       => "XBOLYLDPMZSEBG",
                                          'gx_mirror'     => "VQJPZLDPMZSEBG",
                                          'gx_flexvol_hpo'=> "HLHEAMDPMZSEBG",
                                          'gx_cifs'       => "TFFTAMDPMZSEBG",
                                          'gx_snaprestore'=> "FADIBMDPMZSEBG",
                                          'nfs'           => "DPYLCMDPMZSEBG",
                                          'snapmirror'    => "PJWADMDPMZSEBG",
                                          'snapmirror_dp' => "PJWADMDPMZSEBG",
                                          'striped_volume'=> "NYREEMDPMZSEBG",
                                          'iscsi'    => "BEUPDMDPMZSEBG",
                                          'fcp'      => "ZSPTEMDPMZSEBG",
                                          'flexclone'=> "RUAXBMDPMZSEBG"
                                        },
                        '1-80-000017'=> {
                                         'base'       => "ZYSOQSYLTDIFMA",
                                         'mirror'     => "XNOSRSYLTDIFMA",
                                         'flexvol_hpo'=> "JIMHSSYLTDIFMA",
                                         'cifs'       => "VCKWSSYLTDIFMA",
                                         'snaprestore'=> "HXHLTSYLTDIFMA",
                                         'gx_base'       => "ZYSOQSYLTDIFMA",
                                         'gx_mirror'     => "XNOSRSYLTDIFMA",
                                         'gx_flexvol_hpo'=> "JIMHSSYLTDIFMA",
                                         'gx_cifs'       => "VCKWSSYLTDIFMA",
                                         'gx_snaprestore'=> "HXHLTSYLTDIFMA",
                                         'nfs'           => "FMDPUSYLTDIFMA",
                                         'snapmirror'    => "RGBEVSYLTDIFMA",
                                         'snapmirror_dp' => "RGBEVSYLTDIFMA",
                                         'striped_volume'=> "PVWHWSYLTDIFMA",
                                         'iscsi'    => "DBZSVSYLTDIFMA",
                                         'fcp'      => "BQUWWSYLTDIFMA",
                                         'flexClone'=> "TRFAUSYLTDIFMA"
                                        }
                        };


use constant SierraNevada_Licenses => {
                        '1-80-000011'=> {
                                         'base'=> "ETYYFLXUQUMADFAAAAAAAAAAAAAA",
                                         'flexclone'=> "GCKYSWDUSAFDAAAAAAAAAAAAAAAA",
                                         'snapmirror' => "UHMJSWDUSAFDAAAAAAAAAAAAAAAA",
                                         'snaprestore'=> "INOURWDUSAFDAAAAAAAAAAAAAAAA",
                                        },
                        '1-80-000013'=> {
                                         'base'=> "GINBJXTZGYSEBGAAAAAAAAAAAAAA",
                                         'flexclone'=> "GCKYSWDUSAFDAAAAAAAAAAAAAAAA",
                                         'snapmirror' => "UHMJSWDUSAFDAAAAAAAAAAAAAAAA",
                                         'snaprestore'=> "INOURWDUSAFDAAAAAAAAAAAAAAAA",
                                        },
                        '1-80-000017' =>{
                                         'base'=> "IFSEBEPWNCIFMAAAAAAAAAAAAAAA",
                                         'flexclone'=> "GCKYSWDUSAFDAAAAAAAAAAAAAAAA",
                                         'snapmirror' => "UHMJSWDUSAFDAAAAAAAAAAAAAAAA",
                                         'snaprestore'=> "INOURWDUSAFDAAAAAAAAAAAAAAAA",
                                        },
                        '1-81-000000' => {
                                          'cdmi' => "WSQFRWDUSAFDAAAAAAAAAAAAAAAA",
                                          'cifs' => "MJXMPWDUSAFDAAAAAAAAAAAAAAAA",
                                          'fcp'      => "KYSQQWDUSAFDAAAAAAAAAAAAAAAA",
                                          'flexclone'=> "GCKYSWDUSAFDAAAAAAAAAAAAAAAA",
                                          'iscsi' => "YDVBQWDUSAFDAAAAAAAAAAAAAAAA",
                                          'nfs'   => "APZXOWDUSAFDAAAAAAAAAAAAAAAA",
                                          'snaplock'   => "ERFCUWDUSAFDAAAAAAAAAAAAAAAA",
                                          'snamanager' => "QLDRUWDUSAFDAAAAAAAAAAAAAAAA",
                                          'snapmirror' => "UHMJSWDUSAFDAAAAAAAAAAAAAAAA",
                                          'snapprotect'=> "CGBGVWDUSAFDAAAAAAAAAAAAAAAA",
                                          'snaprestore'=> "INOURWDUSAFDAAAAAAAAAAAAAAAA",
                                          'snapvault'  => "SWHNTWDUSAFDAAAAAAAAAAAAAAAA",
                                        }
                    };


# This hash hold the name of the attributes (as a key) and the nested hash key
#(as value) of the special cases that return the array of values inside of a nested
# attribute, using ZAPI interface.
use constant zapi_nested_attributes =>('fcp-connected-initiators' => "fcp-connected-initiator-info");

use constant ntp_ip_addresses_by_site=> ('svl' => "10.56.32.33,10.56.32.34,10.56.32.35",
                                         'gdl' => "10.56.32.33,10.56.32.34,10.56.32.35",
                                         'ptc' => "10.98.128.24,10.98.18.43,10.98.17.43",
                                         'rtp' => "10.57.158.104",
                                         'nane'=> "10.98.128.24,10.97.0.11,172.19.4.20",
                                         'nb_dc'  => "10.72.8.34,10.72.8.35,10.72.8.36",
                                         'nb_lab' => "10.72.144.134,10.72.144.135,10.72.144.136",
                                        );


1;
