#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/SanFiler/Fcp.pm#1 $

# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary provides an Fcp object with generic methods that related to fcp commands
##          (Currently, this module is only support 'fcp-wwpnalias' 
## @author korry@netapp.com
## @status in_progress
## @pod here

package Fcp;

#
# Compiler directives.
#
use strict;

#
# Module import.
#
use San::SanLog;
use San::SanUtil;
use Error;
use CmdsDriver;
use Exporter;
use Tharn;
use Tie::IxHash;

use vars qw(      
          @EXPORT 
          @ISA
          );

#
# Inherit from the Error, SanLog, and SanUtil classes
#
@ISA = qw(Error SanLog SanUtil);

########################################################################
#                            Public Methods
########################################################################

########################################################################
# Method:    fcp_new
# Objective: Create new Fcp object.
# Details:   See POD documentation at the end of this file.
########################################################################
sub fcp_new
{

    my($proto, %args) = @_;
    my $class = ref($proto) || $proto;
    my (             
        $files_xml,
        $debug_level,
        );

    #
    # Create a hash to store attributes and bless the hash to create an object.
    #
    my $self = {}; 
    bless $self, $class;


    #
    #  default attributes
    #
    $self->{log}            = \*ntest::NTEST_STDOUT;
    $self->{filer_version}  = 0;
    $files_xml              = "fcp";

    #
    # Error message is not initially set, this will set the
    # following inherited attributes:
    #   $self->{error}  = 0;
    #   $self->{errmsg} = '';
    #
    $self->_clear_err();

    #  check for overrides args
    #
    foreach (keys %args)
    {
        if (/-?debug_level$/) {
            length($debug_level = $args{$_}) or
            do {
                $self->_set_err(ref($self)."::fcp_new() invalid value for \"-debug_level\":",
                                "$args{$_}");
                return $self;
            };    
            $self->set_debug_level($debug_level);
        }
        elsif (/-?log$/) {
            length($self->{log} = $args{$_}) or
            do {
                $self->_set_err(ref($self)."::fcp_new() invalid value for \"-log\":",
                                "$args{$_}");
                return $self;
            };
        }
        elsif (/-?filer_version$/) {
            if (not length($self->{filer_version} = $args{$_}) or $args{$_} !~ /\d+/) {
                $self->_set_err(ref($self)."::fcp_new() invalid value for \"-filer_version\":",
                                "$args{$_}");
                return $self;
            };
        }
        elsif (/-?filer$/) {
            length($self->{name} = $args{$_}) or
            do {
                $self->_set_err(ref($self)."::fcp_new() invalid value for \"-filer\":",
                                "$args{$_}");
                return $self;
            };
        }
        elsif (/-?filer_conn$/) {
            if (not length($self->{connection} = $args{$_}) or ref($args{$_}) !~ /HASH/) {
                $self->_set_err(ref($self)."::fcp_new() invalid value for \"-filer_conn\":",
                                "$args{$_}");
                return $self;
            };
        }
        elsif (/-?cmds_driver_obj$/) {
            if (not length($self->{cmd_obj}= $args{$_}) or ref($args{$_}) !~ /HASH/) {
                $self->_set_err(ref($self)."::fcp_new() invalid value for \"-cmds_driver_obj\":",
                                "$args{$_}");
                return $self;
            };
        }
        elsif (/-?cmds$/) {
            length($files_xml = $args{$_}) or
            do {
                $self->_set_err(ref($self)."::fcp_new() invalid value for \"-cmds\":",
                                "$args{$_}");
                return $self;
            };
        }
        else {
            $self->_set_err(ref($self)."::fcp_new() invalid parameter specified: $_");
            return $self;
        }
    } # end foreach arg

    #
    # Check for required parameters
    # require either filer or filer_conn
    unless (defined($self->{name}) or defined($self->{connection})) {
        $self->_set_err("Error: ".ref($self)."::fcp_new() ",
                        "Require either '-filer' or '-filer_conn'");
        return $self;
    }

    # make connection if filer_conn is not passed in
    unless (defined($self->{connection})) {
        $self->{connection} = $self->filer_connect();
        return $self if ($self->error);
    }

    # set filer name as well
    unless (defined($self->{name})) {
        $self->{name} = $self->{connection}->hostp->hostname;
    }

    #
    #  create new cmd obj if one is not passed in
    #
    unless (defined($self->{cmd_obj})) {
        $self->{cmd_obj} = new CmdsDriver(-log         => $self->{log}, 
                                          -cmds        => $files_xml,
                                          -debug_level => $debug_level
                                         );
        # check error
        if ($self->error()) {
            $self->_set_err(ref($self)."::new() Failed to instantiate CmdsDriver object: " .
                             $self->{cmd_driver_obj}->errmsg());
            return $self;
        }

        # set default privilege, vfiler, partner, and etc. if create cmd_driver_obj
        $self->{cmd_obj}->connection($self->{connection});
        $self->{cmd_obj}->filer($self->{name});
    }

    # if filer version is zero, discover version
    # otherwise set filer version 
    $self->sanlog(-msg => ref($self)."::".                   
                          "fcp_new() filer_verion = ".
                          $self->{filer_version},
                  -msg_level => 2);

    ($self->{filer_version} eq 0) ?
        $self->{cmd_obj}->discover_filer_version($self->{name}) :
            $self->{cmd_obj}->filer_version($self->{filer_version});

    return $self;
}  # end of new method                              

###############################################################################
# Method:    fcp_wwpnalias_exists
# Objective: check if specified alias was exist
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_wwpnalias_exists
{
    my($self, %args) = @_;
    my(
        $alias,
        $wwpn,
        $rv_ref,
        $cmdsDriverObj,
        );

    $self->_clear_err();
    foreach (keys %args) {
        if (/-?alias/) {
            if (not length($alias = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_exists() ",
                                "Invalid value for argument '-alias'");
                return $SanUtil::FAIL;
            }
            next;
        }else {
            $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_exists() ",
                            "Invalid parameter specified \'$_\'");
            return $SanUtil::FAIL;
        }
    }

    # can not have both alias and wwpn params
    unless (defined($alias)) {
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_exists() ",
                        "-alias was not specified");
        return $SanUtil::FAIL;
    }

    $rv_ref = $self->fcp_wwpnalias_get(-alias=>$alias);
    foreach my $key (keys %{$rv_ref}) {
        return $SanUtil::TRUE if ($key =~ /$alias/);
    }
    return $SanUtil::FALSE;

}

###############################################################################
# Method:    fcp_wwpnalias_get
# Objective: getting alias(es) for wwpn
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_wwpnalias_get
{
    my($self, %args) = @_;
    my(
        $alias,
        $wwpn,
        $rv_ref,
        $cmdsDriverObj,
        );

    $self->_clear_err();
    foreach (keys %args) {
        if (/-?alias/) {
            if (not length($alias = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_get() ",
                                "Invalid value for argument '-alias'");
                return undef;
            }
            next;
        }elsif (/-?wwpn/) {
            if (not length($wwpn = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_get() ",
                                "Invalid value for argument '-wwpn'");
                return undef;
            }
            next;
        }else {
            $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_get() ",
                            "Invalid parameter specified \'$_\'");
            return undef;
        }
    }

    # can not have both alias and wwpn params
    if (defined($wwpn) and defined($alias)) {
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_get() ",
                        "Both -alias and -wwpn were specified");
        return undef;
    }

    # get cmdsDriver obj
    if ($self->cmds_driver_exist()) {
        $cmdsDriverObj = $self->cmds_driver_get();
    } else {
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_get() ",
                        "cmds driver object does not exist");
        return undef;
    }

    # build cmd args
    my @api_args = ("-cmd", "fcp-wwpnalias-get-alias-info");
    push (@api_args, "-wwpn", $wwpn) if (defined($wwpn));
    push (@api_args, "-alias", $alias) if (defined($alias));
    # take what ever was given and let cmds_driver_obj execute it
    $rv_ref = $cmdsDriverObj->execute_cmd(@api_args);
    # for zapi, get alias failed if alias was not set
    # ignore if zapi return this error
    if (($cmdsDriverObj->error() or $rv_ref->{result}) 
            and $rv_ref->{reason} !~ /WWPN Aliases not set/) {
        # parse cli output
        my $errmsg=$rv_ref->{output};
        if ("cli" eq ($cmdsDriverObj->interface())) {
            $errmsg =~ s/.*: //;
        } else {
            $errmsg = $rv_ref->{reason};
        }
        $self->sanlog(-msg => "Error: ".ref($self)."::fcp_wwpnalias_get() ". $errmsg,
                      -msg_level => 2) ;
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_get() ". $errmsg);
        foreach my $key(keys %{$rv_ref})
        {
            my $value;
            if ($key !~ /(commands|data_files)/ and ref($rv_ref->{$key}) =~ /(HASH|ARRAY|GLOB)/) {
                $value = $self->dumper_get($rv_ref->{$key});
            } else {
                $value = $rv_ref->{$key};
            }
            $self->sanlog(-msg => " -->  $key => $value", -msg_level => 2);
        }
        return undef;
    }

    # parse XML output
    # create xml parser object
    tie my %alias_hash, 'Tie::IxHash';
    if ($cmdsDriverObj->{interface} eq 'zapi') {
        if (defined($rv_ref->{output})) {
            my $out_hash = $self->xml2hash($rv_ref->{output});
            my $alias_info = $out_hash->{'aliases'}->{'aliases-info'};
            $self->sanlog(-msg => " -->  alias_info = $alias_info", -msg_level => 2);
            my $out = $self->dumper_get($alias_info);
            $self->sanlog(-msg => " -->  dump output = $out", -msg_level => 2);
            if (ref($alias_info) =~ /HASH/) {
                # single hash
                $alias_hash{$alias_info->{'aliases-alias'}} = $alias_info->{'aliases-wwpn'};
            } elsif (ref($alias_info) =~ /ARRAY/) {
                # array of hash
                foreach my $pair (@{$alias_info}) {
                    $alias_hash{$pair->{'aliases-alias'}} = $pair->{'aliases-wwpn'};
                }
            }
        } 
    } else {
        # parse for cli output
        my @info = split(/\n/,$rv_ref->{output});
        my @alias_info;
        foreach my $line (@info) {
            my ($wwpn_name, $alias_name);
            # we need to handle both format for show -w wwpn and -a alias
            # for CLI command
            # for -w wwpn output
            if ($line =~ /^\d+:.*/) {
                ($wwpn_name, $alias_name) = split(/\t/, $line, 2);
                $self->sanlog(-msg => " -->  $alias_name => $wwpn_name");
                $wwpn_name =~ s/^\s+//;
                $alias_name =~ s/^\s+//;
                $alias_name =~ s/\s+$//;
                # add key=value pair to array
                $alias_hash{$alias_name} = $wwpn_name;
            }
            # for -a alias output
            elsif ($line =~ /WWPN Alias\(es\):\s+(.*)/) {
                my @aliases = split(/, /,$1);
                foreach (@aliases) {
                    $alias_hash{$_} = $wwpn;
                }
            }
        }
    }

    return \%alias_hash;

}

###############################################################################
# Method:    fcp_wwpnalias_remove
# Objective: remove the mapping between alias(es) and corresponding wwpn(s)
#            or all aliases for a given (single) wwpn.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_wwpnalias_remove
{
    my($self, %args) = @_;
    my(
        $alias,
        $wwpn,
        $rv_ref,
        $cmdsDriverObj,
        );

    $self->_clear_err();
    foreach (keys %args) {
        if (/-?alias/) {
            if (not length($alias = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_remove() ",
                                "Invalid value for argument '-alias'");
                return $SanUtil::FAIL;
            }
            next;
        }elsif (/-?wwpn/) {
            if (not length($wwpn = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_remove() ",
                                "Invalid value for argument '-wwpn'");
                return $SanUtil::FAIL;
            }
            next;
        }else {
            $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_remove() ",
                            "Invalid parameter specified \'$_\'");
            return $SanUtil::FAIL;
        }
    }

    # can not have both alias and wwpn params
    if (defined($wwpn) and defined($alias)) {
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_remove() ",
                        "Both -alias and -wwpn were specified");
        return $SanUtil::FAIL;
    }

    # get cmdsDriver obj
    if ($self->cmds_driver_exist()) {
        $cmdsDriverObj = $self->cmds_driver_get();
    } else {
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_remove() ". 
                        "cmds driver object does not exist");
        return $SanUtil::FAIL;
    }

    # build cmd args
    my @api_args = ("-cmd", "fcp-wwpnalias-remove");
    push (@api_args, "-wwpn", $wwpn) if (defined($wwpn));
    push (@api_args, "-alias", $alias) if (defined($alias));
    # take what ever was given and let cmds_driver_obj execute it
    $rv_ref = $cmdsDriverObj->execute_cmd(@api_args);
    # 
    if ($cmdsDriverObj->error() or $rv_ref->{result}) {
        # parse cli output
        my $errmsg=$rv_ref->{output};
        if ("cli" eq ($cmdsDriverObj->interface())) {
            $errmsg =~ s/.*: //;
        } else {
            $errmsg = $rv_ref->{reason};
        }
        $self->sanlog(-msg => "Error: ".ref($self)."::fcp_wwpnalias_remove() ". $errmsg,
                      -msg_level => 2) ;
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_remove() ". $errmsg);
        foreach my $key(keys %{$rv_ref})
        {
            my $value;
            if ($key !~ /(commands|data_files)/ and ref($rv_ref->{$key}) =~ /(HASH|ARRAY|GLOB)/) {
                $value = $self->dumper_get($rv_ref->{$key});
            } else {
                $value = $rv_ref->{$key};
            }
            $self->sanlog(-msg => " -->  $key => $value", -msg_level => 2);
        }
        return $SanUtil::FAIL;
    }

    return $SanUtil::PASS;

}

###############################################################################
# Method:    fcp_wwpnalias_set
# Objective: setting alias for wwpn
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fcp_wwpnalias_set
{
    my($self, %args) = @_;
    my(
        $alias,
        $wwpn,
        $force,
        $force_option,
        $rv_ref,
        $cmdsDriverObj,
        );

    $self->_clear_err();
    foreach (keys %args) {
        if (/-?alias/) {
            if (not length($alias = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_set() ",
                                "Invalid value for argument '-alias'");
                return $SanUtil::FAIL;
            }
            next;
        }elsif (/-?wwpn/) {
            if (not length($wwpn = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_set() ",
                                "Invalid value for argument '-wwpn'");
                return $SanUtil::FAIL;
            }
            next;
        }elsif (/-?force/) {
            if (not length($force = $args{$_})) {
                $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_set() ",
                                "Invalid value for argument '-force'");
                return $SanUtil::FAIL;
            }
            next;
        }else {
            $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_set() ",
                            "Invalid parameter specified \'$_\'");
            return $SanUtil::FAIL;
        }
    }
    # must have both alias and wwpn params
    if (not defined($wwpn) or not defined($alias)) {
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_set() ",
                        "Both -alias and -wwpm must be specified");
        return $SanUtil::FAIL;
    }

    # get cmdsDriver obj
    if ($self->cmds_driver_exist()) {
        $cmdsDriverObj = $self->cmds_driver_get();
    } else {
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_set() ",
                        "cmds driver object does not exist");
        return $SanUtil::FAIL;
    }

    $force_option = ("cli" eq ($cmdsDriverObj->interface())) ? "" : $force;
    # build cmd args
    my @api_args = ("-cmd", "fcp-wwpnalias-set");
    push (@api_args, "-alias", $alias) if (defined($alias));
    push (@api_args, "-wwpn", $wwpn) if (defined($wwpn));
    push (@api_args, "-force", $force_option) if (defined($force));

    # take what ever was given and let cmds_driver_obj execute it
    $rv_ref = $cmdsDriverObj->execute_cmd(@api_args);
    # 
    if ($cmdsDriverObj->error() or $rv_ref->{result}) {
        # parse cli output
        my $errmsg=$rv_ref->{output};
        if ("cli" eq ($cmdsDriverObj->interface())) {
            $errmsg =~ s/.*: //;
        } else {
            $errmsg = $rv_ref->{reason};
        }
        $self->sanlog(-msg => "Error: ".ref($self)."::fcp_wwpnalias_set() ". $errmsg,
                      -msg_level => 2) ;
        $self->_set_err("Error: ".ref($self)."::fcp_wwpnalias_set() ". $errmsg);
        foreach my $key(keys %{$rv_ref}) {
            my $value;
            if ($key !~ /(commands|data_files)/ and ref($rv_ref->{$key}) =~ /(HASH|ARRAY|GLOB)/) {
                $value = $self->dumper_get($rv_ref->{$key});
            } else {
                $value = $rv_ref->{$key};
            }
            $self->sanlog(-msg => " -->  $key => $value", -msg_level => 2);
        }
        return $SanUtil::FAIL;
    }

    return $SanUtil::PASS;

} # end fcp_wwpnalias_set

###############################################################################
#                               Accessor Methods
###############################################################################

###############################################################################
#                               Private Methods
###############################################################################

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

Fcp.pm

=head1 Author

Korry Nguyen

=head1 Description

Fcp.pm provides an object for executing fcp commands on the filer
(Currently, this module only supports fcp wwpn-alias)

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<San::Sanlog>

B<San::SanUtil>

B<Error>

B<CmdsDriver>

B<Tharn>

=head1 Attributes

=head2 B<debug_level>

integer = Current debug level (higher level increases verbosity
and >100 sprints call stacks with comments).  Default is 0.

=head2 B<name>

string = filer name

=head2 B<connection>

object = filer connection

=head2 B<cmd_obj>

object = commands driver object

=head2 B<filer_version>

integer = ONTAP version number (defaulted to 0 will trigger the discover of filer version)

=head1 Public Methods

=head2 B<fcp_new()>

Creates a new fcp object.  Give default values to all supported attributes.

=over 4

=item Inputs

=item * I<-debug_level> (default is 0)

integer = current debug level (higher level increases verbosity
and >100 prints call stacks with comments and errors).

=item * I<-filer> (required if filer_conn does not pass in)

string = filer name

=item * I<-filer_conn> (optional)

object = filer connection object

=item * I<-cmds_driver_obj> (optional)

object = commands driver object

=item * I<-filer_version> (optional)

integer = ONTAP version number

=item Outputs

Returns a fcp object.

=item Examples

    #
    # Create a fcp object
    #
    $fcp_obj = fcp_new Fcp(
                       -filer          => "rtpqa-fas2020-3",
                       -filer_version  => "73",
                       -debug_level    => $debug_level);
    if ($fcp_obj->error())
    {
       my $errmsg = join("\n", $fcp_obj->errmsg());
       $Test->nlog($errmsg);
       return $TCD::FATAL;
    }


=back

=head2 B<fcp_wwpnalias_get(%args)>

Get WWPN alias

=over 4

=item Inputs

=item * -alias (optional)

Getting WWPN that mapped to the specified alias

=item * -wwpn (optional)

Get alias(es) that associated with specified wwpn

=item Outputs

Returns a hash. The hash contains {alias_name => wwpn_name}. 
Failure or no existing alias will return undef.

  single alias output:
    alias_name => 10:00:00:00:c9:48:ce:9f

  multi aliases output:
    alias_name1 => 10:00:00:00:c9:48:ce:9f
    alias_name2 => 10:00:00:00:c9:48:ce:9f
    alias_name3 => 10:00:00:00:c9:48:ce:9f
    alias_name4 => 10:00:00:00:c9:48:ce:9f
    alias_name5 => 10:00:00:00:c9:48:ce:9f

=item Examples

    #
    # Get alias(es) that mapped to a wwpn
    #
    $hash_ref = $fcp_obj->fcp_wwpnalias_get(-wwpn=>$wwpn_name);
    if ($fcp_obj->error())
    {
       my $errmsg = join("\n", $fcp_obj->errmsg());
       $Test->nlog($errmsg);
       return $TCD::FAIL;
    }

=back

=head2 B<fcp_wwpnalias_remove(%args)>

If alias was given, remove that alias which mapped to a WWPN
If wwpn was given, remove alias(es) that mapped to the given WWPN

=over 4

=item Inputs

=item * -alias  or -wwpn (required either)

Name of an alias or wwpn

=item Outputs

 TRUE  = SUCCESS = 1
 FALSE = FAILURE = 0

=item Examples

    #
    # Remove given alias that mapped to a wwpn
    #
    if (not $fcp_obj->fcp_wwpnalias_remove(-alias=>$alias_name)) 
    {
       my $errmsg = join("\n", $fcp_obj->errmsg());
       $Test->nlog($errmsg);
       return $TCD::FAIL;
    }

=back

=head2 B<fcp_wwpnalias_set(%args)>

Create an alias and map to the given wwpn

=over 4

=item Inputs

=item * -alias  (required)

Name of a alias.

=item * -wwpn  (required)

Name of a WWPN.

=item Outputs

 TRUE  = SUCCESS = 1
 FALSE = FAILURE = 0

=item Examples

    #
    # Create an alias and map to a given wwpn
    #
    if (not $fcp_obj->fcp_wwpnalias_set(-alias=>$alias_name,
                                    -wwpn=>$wwpn_name)) 
    {
       my $errmsg = join("\n", $fcp_obj->errmsg());
       $Test->nlog($errmsg);
       return $TCD::FAIL;
    }
    
=back

=cut

1;
