#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/SanFiler/IGroup.pm#2 $

# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to Filer IGroups
## @author knopp@netapp.com
## @status private
## @pod here


# Package Name
##############
package IGroup;

use strict;

# Package Requirements
######################
use File::Basename;
use File::Copy;
use POSIX;

use San::SanClient::ClientUtil;
use San::SanFiler::IGroup;
use San::SanLog;
use Error;

use vars qw( @ISA );

@ISA=qw(Error SanLog);

################################################################################
# P U B L I C   M E T H O D S
################################################################################


################################################################################
# Subroutine Name: destroy
# Objective:       destroy a igroup
# Details:         See POD documentation at the end of this file
################################################################################
sub destroy
{
    my ($self,%args)=@_;
    my $cmd_obj = $self->{parent}->{cmd_obj};
    my(
       $cmd,
       $force,
       $name,
       $option,
       @igrpObjs,
       @output,
       );

    $self->_clear_err();

    # parse arguments
    foreach (keys %args) {
        if (/-?force/) {
            if (not length($force= $args{$_})) {
                $self->_set_err("Invalid value for argument '-force'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }

    # Get the igroup name
    $name=$self->get_name();

    if($self->{parent}->{interface} eq "cli") {
        $option=(defined $force) ? "-f" : "";
        $cmd = "igroup destroy $option $name";
        @output = $self->{parent}->execute($cmd,-hide=>1);

        if ($output[0]=~/maps.*exist/i) {
            $self->_set_err("Unable to destroy igroup.",@output);
            return $ClientUtil::FALSE;
        }
    } else {
        my $filer=$self->{parent}->get_name();
        my @cmds_args = ("-cmd", "igroup-destroy");

        push (@cmds_args, "-cmd_filer", $filer);
        push (@cmds_args, "-check_result", "1");
        push (@cmds_args, "-initiator-group-name", $name);

        if (defined $force) { push(@cmds_args, "-force", 1);}

        my $resultsHash_ref = $cmd_obj->execute_cmd(@cmds_args);

        if($cmd_obj->error()) {
           my $errmsg = join "\n", $cmd_obj->errmsg();
           $self->_set_err("Unable to destroy Igroup: $errmsg");
           return $ClientUtil::FALSE;
        }

        #
        # Since we checked the result, lets see if our check was OK
        #
        if(!$resultsHash_ref->{result_check}) {
            $self->_set_err("Expected result did not match: " .
                            $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }

        #
        # Get the output
        #
        my $output = $resultsHash_ref->{output};

        my $output_hash = $cmd_obj->_xml_to_hash($output);

        my $status = $output_hash->{status};
        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Unable to destroy Igroup. $errmsg");
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: display()
# Objective:       Display volume information
# Details:         See POD documentation at the end of this file
################################################################################
sub display
{
    my ($self)=@_;
    my (
        $cnt,
        $name,
        $header,
        $summary,
        @output,
        @buffer,
        );

    $self->_clear_err();

    $name=$self->{name};

    $cnt=0;

    $header =
        sprintf("IGroup Detail:\n".
                "%-25.25s %-8.8s %-10.10s %-8.8s\n".
                "%25.25s %8.8s %10.10s %-8.8s",
                "NAME","PROTOCOL","OSTYPE","NODE_CNT",
                "="x25,"="x8,"="x10,"="x8);

    push @buffer,$header;


    $summary =
        sprintf("%-25.25s %-8.8s %-10.10s %-8.8s\n%54.54s",
                $self->get_name(),
                $self->get_protocol(),
                $self->get_ostype(),
                $self->get_node_cnt(),
                "="x54
                );

    push @buffer, $summary;


    foreach my $node (@{$self->{nodes}}){
        $summary = sprintf(" %d) %-40.40s",++$cnt,$node);
        push @buffer, $summary;
    }


    push @buffer, "\n\n";

    return (wantarray()) ? @buffer : \@buffer;

}

###############################################################################
# Subroutine Name:  add_nodes()
# Objective:        Add nodes to an igroup
# Details:          See POD documentation at the end of this file
###############################################################################
sub add_nodes()
{
    my ($self, %args) = @_;
    my (
        $igroup_name,
        $igroup_nodename_ref,
        $force,
       );

    my $cmd_obj = $self->{parent}->cmd_obj();

    $self->sanlog(-msg => "add nodes");
    $self->_clear_err();

    $igroup_name = $self->get_name();

    # parse arguments
    foreach (keys %args)
    {
        if (/^-?igroup_nodes$/)
        {
            if (not length($igroup_nodename_ref = $args{$_}))
            {
                $self->_set_err("Invalid value for argument " .
                                "'-igroup_nodes'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        elsif (/^-?force$/)
        {
            if (not length($force = $args{$_}))
            {
                $self->_set_err("Invalid value for argument " .
                                "'-force'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        else
        {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Verify mandatory parameters
    #
    if (!defined ($igroup_nodename_ref))
    {
        $self->_set_err("Igroup nodes(igroup_nodes) is required argument");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    #
    # Commands must be handled differently for each interface
    #
    if($self->{parent}->{interface} =~ m/zapi/i)
    {
        foreach my $node (@$igroup_nodename_ref)
        {
            my %cmd_args = (
                    "-cmd"                  => "igroup-add",
                    "-cmd_filer"            => $self->{parent}->get_name(),
                    "-check_result"         => 1,
                    "-initiator-group-name" => $igroup_name,
                    "-initiator"            => $node);

            if( defined($force) && $force =~ /(yes|true|1)/i ) {
                $cmd_args{"-force"} = 1;
            }

            $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
            if ($cmd_obj->error())
            {
                my $errmsg = "Failed calling execute_cmd for 'igroup-add' " .
                             "using $self->{parent}->{interface}:\n".
                             "Reason: " .
                             join("\n", $cmd_obj->errmsg());
                $self->_set_err($errmsg);
                return $ClientUtil::FALSE;
            }

            #
            # Check that the result and output aren't wrong
            #
            if (!$resultsHash_ref->{result_check})
            {
                $self->_set_err("Expected result did not match: " .
                            $resultsHash_ref->{output});
                return $ClientUtil::FALSE;
            }
        }
    }
    else
    {
        my $nodes = join(' ', @$igroup_nodename_ref) if (defined($igroup_nodename_ref));
        my @cmds_args = ("-cmd", "igroup-add");
        push (@cmds_args, "-initiator-group-name", $igroup_name);
        push (@cmds_args, "-initiator", $nodes);

        if( defined($force) && $force =~ /(yes|true|1)/i )
        {
            push (@cmds_args, "-force", "");
        }

        push (@cmds_args, "-check_result", "1");

        $resultsHash_ref = $cmd_obj->execute_cmd(@cmds_args);

        if ($cmd_obj->error())
        {
            my $errmsg = "Failed calling execute_cmd for 'igroup-add' " .
                         "using $self->{parent}->{interface}:\n".
                         "Reason: " . join("\n", $cmd_obj->errmsg());
            $self->_set_err($errmsg);
            return $ClientUtil::FALSE;
        }

        #
        # Check that the result and output aren't wrong
        #
        if (!$resultsHash_ref->{result_check})
        {
            $self->_set_err("Expected result did not match: " .
                            $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Subroutine Name:  bind_portset()
# Objective:        Bind an igroup
# Details:          There is no POD documentation
###############################################################################
sub bind_portset()
{
    my ($self, %args) = @_;
    my (
        $portset_name,
        $igroup_name,
       );
    my $cmd_obj = $self->{parent}->cmd_obj();

    $self->sanlog(-msg => "bind portset");
    $self->_clear_err();
    $igroup_name = $self->get_name();

    # parse arguments
    foreach (keys %args)
    {
        if (/^-?portset_name$/)
        {
            if (not length($portset_name = $args{$_}))
            {
                $self->_set_err("Invalid value for argument " .
                                "'-portset_name'");
                return $ClientUtil::FALSE;
            }
            next;
        }
        else
        {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Verify mandatory parameters
    #
    if (!defined ($portset_name))
    {
        $self->_set_err("Portset name(portset_name) is required argument");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref =
             $cmd_obj->execute_cmd(
                        -cmd                    => "igroup-bind-portset",
                        -cmd_filer              => $self->{parent}->get_name(),
                        -check_result           => 1,
                        "-initiator-group-name" => $igroup_name,
                        "-portset-name"         => $portset_name);

    if ($cmd_obj->error())
    {
        my $errmsg = "Failed calling execute_cmd for 'igroup-bind-portset' " .
                     "using $self->{parent}->{interface}:\n".
                     "Reason: " . join("\n", $cmd_obj->errmsg());
        $self->_set_err($errmsg);
        return $ClientUtil::FALSE;
    }

    #
    # Check that the result and output aren't wrong
    #
    if (!$resultsHash_ref->{result_check})
    {
        $self->_set_err("Expected result did not match: " .
                        $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

###############################################################################
# Subroutine Name:  unbind_portset()
# Objective:        Unbind an igroup
# Details:          There is no POD documentation
###############################################################################
sub unbind_portset()
{
    my ($self) = @_;
    my $igroup_name;
    my $cmd_obj = $self->{parent}->cmd_obj();

    $self->sanlog(-msg => "unbind portset");
    $self->_clear_err();

    $igroup_name = $self->get_name();

    my $resultsHash_ref =
             $cmd_obj->execute_cmd(
                        -cmd                    => "igroup-unbind-portset",
                        -cmd_filer              => $self->{parent}->get_name(),
                        -check_result           => 1,
                        "-initiator-group-name" => $igroup_name);

    if ($cmd_obj->error())
    {
        my $errmsg = "Failed calling execute_cmd for 'igroup-unbind-portset'" .
                     " using $self->{parent}->{interface}:\n".
                     "Reason: " . join("\n", $cmd_obj->errmsg());
        $self->_set_err($errmsg);
        return $ClientUtil::FALSE;
    }

    #
    # Check that the result and output aren't wrong
    #
    if (!$resultsHash_ref->{result_check})
    {
        $self->_set_err("Expected result did not match: " .
                        $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: get_name
# Objective:       Return Volumes name
# Details:         See POD documentation at the end of this file
################################################################################
sub get_name
{
    my ($self) = @_;

    $self->_clear_err();


    if (defined $self->{name}){
        return $self->{name};
    } else {
        $self->_set_err("IGroup object is missing name");
        return "";
    }
}

################################################################################
# Subroutine Name: get_protocol
# Objective:       Return protocol type
# Details:         See POD documentation at the end of this file
################################################################################
sub get_protocol
{
    my ($self) = @_;

    $self->_clear_err();


    if (defined $self->{protocol}){
        return $self->{protocol};
    } else {
        $self->_set_err("IGroup object is missing protocol type");
        return "";
    }
}

################################################################################
# Subroutine Name: get_ostype
# Objective:       Return os type
# Details:         See POD documentation at the end of this file
################################################################################
sub get_ostype
{
    my ($self) = @_;

    $self->_clear_err();


    if (defined $self->{ostype}){
        return $self->{ostype};
    } else {
        $self->_set_err("IGroup object is missing os type");
        return "";
    }
}


################################################################################
# Subroutine Name: get_nodes
# Objective:       return array of nodes
# Details:         See POD documentation at the end of this file
################################################################################
sub get_nodes
{
    my ($self) = @_;
    my (
        @nodes,
        );

    $self->_clear_err();

    if (defined $self->{nodes}) {
        @nodes = @{$self->{nodes}};
    } else {
        $self->_set_err("IGroup object has no nodes");
        return (wantarray()) ? () : [];
    }

    return (wantarray()) ? @nodes : \@nodes;

}

################################################################################
# Subroutine Name: get_node_cnt
# Objective:       return number of igroup nodes
# Details:         See POD documentation at the end of this file
################################################################################
sub get_node_cnt
{
    my ($self) = @_;

    $self->_clear_err();
    if (defined $self->{nodes}) {
        return scalar(@{$self->{nodes}});
    } else {
        $self->_set_err("IGroup object has no node count");
        return 0;
    }

    return 0;
}

################################################################################
# Subroutine Name: get_vfiler
# Objective:       return the igroup's vfiler
# Details:         See POD documentation at the end of this file
################################################################################
sub get_vfiler
{
    my ($self) = @_;

    $self->_clear_err();

    #
    # If we have a vfiler name, return it.  If not just return the default.
    #
    if (defined $self->{vfiler}) {
        return ($self->{vfiler});
    } else {
        return "vfiler0";
    }
}

###############################################################################
# Subroutine Name: get_status
# Objective:       return alua status, whether logged through vtic alone or not
# Details:         See POD documentation at the end of this file
################################################################################
sub get_status {
    my ($self) = @_;
    # the subroutine will check whether the igroup is logged in with only vtic
    # or not

    my ($igroup_status, $arr, @ports, @output);
    my $igroup_name = $self->get_name();
    my $cmd_obj = $self->{parent}->cmd_obj();
    $self->_clear_err();

    # Execute igroup list info command
    my $resultsHash_ref = $cmd_obj->execute_cmd(
                            -cmd                    => "igroup-list-info",
                            "-initiator-group-name" => $igroup_name,
                            -cmd_interface          => "cli",
                            -verbose                => "",
                        );

    # Check for any errors
    if ($cmd_obj->error()) {
        my $errmsg = "igroup-list-info: " . join "\n", $cmd_obj->errmsg();
        $self->_set_err("Failed calling execute_cmd: " . $errmsg);
        return $ClientUtil::FAIL;
    } # end if ($cmd_obj...

    # get the ouput from the output hash
    my $lines = $resultsHash_ref->{output};
    @output = split("\n", $lines);
    # set the alua status
    # Go through the output.
    foreach my $line (@output) {
        next unless ($line =~ /ALUA:/);
        # ALUA: Yes
        if ($line =~ /ALUA:\s+(\w+)/) {
            if ($1 =~ /Yes/i) {
                $igroup_status->{alua} = "TRUE";
            } else {
                $igroup_status->{alua} = "FALSE";
            } # end if ($1 =~...
        }  # end if ($line...
    } # end foreach...

    # if ALUA is not set then it wont enter into the For loop.
    if ($igroup_status->{alua} !~ /TRUE/)  {
        $igroup_status->{alua} = "FALSE";
    } # end if (!$igroup_status...

    # igroup1 (FCP):
        # OS Type: windows
        # Host Multipathing Software: Required
        # Member: 10:00:00:00:c9:4a:e8:90 (logged in on: vtic, 1a)
        # Member: 10:00:00:00:c9:4a:e9:35 (logged in on: 1b)
        # UUID: 51c5639f-9a65-11e1-bfd4-123478563412
        # ALUA: Yes
        # Report SCSI Name in Inquiry Descriptor: No

    foreach my $line (@output) {
        next unless ($line =~ /Member/);
        if ($line =~ /logged\s+in\s+on\:\s+(vtic)/) {
            if ($1 =~ /vtic/) {
                $igroup_status->{'vtic_only'} = "TRUE";
            } else {
                $igroup_status->{'vtic_only'} = "FALSE";
            } # end if ($1...
        } elsif (($arr) = $line =~ /logged in on: (.+)\)/) {
            push (@ports, split(",", $arr));
            $igroup_status->{'ports'} = \@ports;
            $igroup_status->{'vtic_only'} = "FALSE";
        } # end if ($line =...
    } # end foreach my...

    return $igroup_status;

} ## end sub get_alua_status()
################################################################################
# Subroutine Name: refresh
# Objective:       refresh content of igroup.
# Details:         See POD documentation at the end of this file
################################################################################
sub refresh
{
    my ($self)=@_;

    # Remove existing nodes cache
    $self->{nodes}=undef;

    $self->sanlog(-msg => "refreshing igroup '".$self->get_name()."'");

    $self->_load_igroup($self->get_name());
    return $self if ($self->error);
}

################################################################################
# Subroutine Name: set
# Objective:       interface to igroup set
# Details:         See POD documentation at the end of this file
################################################################################
sub set
{
    my ($self, %args) = @_;
    my (
        $cmd,
        @output,
        $force,
        $attr,
        $value,
        $option,
        );

    # parse arguments
    foreach (keys %args) {
        if (/-?force/) {
            if (not length($force = $args{$_})) {
                $self->_set_err("Invalid value for argument '-force'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?attr/) {
            if (not length($attr = $args{$_})) {
                $self->_set_err("Invalid value for argument '-attr'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/-?value/) {
            if (not length($value = $args{$_})) {
                $self->_set_err("Invalid value for argument '-force'");
                return $ClientUtil::FALSE;
            }
        } else {
            $self->_set_err("Invalid parameter specified: '$_'");
            return $ClientUtil::FALSE;
        }
    }

    # get force information
    $option = (defined $force) ? "-f" : "";

    # create command
    $cmd = "igroup set $option $self->{name} $attr $value";
    @output = $self->{parent}->execute($cmd,-hide=>1);
    if (grep(/No such|invalid|not supported/i, @output)) {
        $self->_set_err("Unable execute igroup set command".
                        $cmd ."\n". @output);
        return $ClientUtil::FALSE;
    }

}

###############################################################################
# Subroutine Name:  remove_nodes()
# Objective:        Remove nodes from an igroup
# Details:          See POD documentation at the end of this file
###############################################################################
sub remove_nodes() {
    my ($self, %args) = @_;
    my $igroup_name = $self->get_name();
    my $igroup_nodename_ref;
    my $force = 0;

    my $cmd_obj = $self->{parent}->cmd_obj();

    $self->sanlog(-msg => "remove nodes");
    $self->_clear_err();

    # parse arguments
    foreach (keys %args) {
        if (/^-?igroup_nodes$/) {
            if (not length($igroup_nodename_ref = $args{$_})) {
                $self->_set_err(
                             "Invalid value for argument " . "'-igroup_nodes'");
                return $ClientUtil::FALSE;
            }
            next;
        } elsif (/^-?force$/) {
            if (not length($force = $args{$_})) {
                $self->_set_err("Invalid value for argument " . "'-force'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified \'$_\'");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Verify mandatory parameters
    #
    if (!defined($igroup_nodename_ref)) {
        $self->_set_err("Igroup nodes(igroup_nodes) is required argument");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref;

    #
    # Commands must be handled differently for each interface
    #
    if ($self->{parent}->{interface} =~ m/zapi/i) {
        foreach my $node (@$igroup_nodename_ref) {
            my %cmd_args = (
                    "-cmd"                  => "igroup-remove",
                    "-cmd_filer"            => $self->{parent}->get_name(),
                    "-check_result"         => 1,
                    "-initiator-group-name" => $igroup_name,
                    "-initiator"            => $node);

            if( defined($force) && $force =~ /(yes|true|1)/i ) {
                $cmd_args{"-force"} = 1;
            }

            $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
            if ($cmd_obj->error()) {
                my $errmsg =
                    "Failed calling execute_cmd for 'igroup-remove' ".
                    "using $self->{parent}->{interface}:\n".
                    "Reason: ".
                    join("\n", $cmd_obj->errmsg());
                $self->_set_err($errmsg);
                return $ClientUtil::FALSE;
            }

            #
            # Check that the result and output aren't wrong
            #
            if (!$resultsHash_ref->{result_check}) {
                $self->_set_err("Expected result did not match: "
                                . $resultsHash_ref->{output});
                return $ClientUtil::FALSE;
            }
        } #end of foreach
    } else {
      #
      # CLI interface
      #
        my $nodes = join(' ', @$igroup_nodename_ref)
            if (defined($igroup_nodename_ref));
        my @cmds_args = ("-cmd", "igroup-remove");
        push(@cmds_args, "-initiator-group-name", $igroup_name);
        push(@cmds_args, "-initiator",            $nodes);
        if (defined($force) && $force =~ /(yes|true|1)/i) {
            push(@cmds_args, "-force", "");
        }

        push(@cmds_args, "-check_result", "1");
        $resultsHash_ref = $cmd_obj->execute_cmd(@cmds_args);

        if ($cmd_obj->error()) {
            my $errmsg =
                "Failed calling execute_cmd for 'igroup-remove' ".
                "using $self->{parent}->{interface}:\n".
                "Reason: ".
                join("\n", $cmd_obj->errmsg());
            $self->_set_err($errmsg);
            return $ClientUtil::FALSE;
        }

        #
        # Check that the result and output aren't wrong
        #
        if (!$resultsHash_ref->{result_check}) {
            $self->_set_err(
                "Expected result did not match: " . $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
}

################################################################################
# Subroutine Name: rename
# Objective:       rename an existing igroup
# Details:         No POD documentation
################################################################################
sub rename {
    my ($self, %args) = @_;
    my $igroup_name = $self->get_name();
    my $cmd;
    my $igroup_new_name;
    my @output;

    my $cmd_obj = $self->{parent}->cmd_obj();
    $self->sanlog(-msg => "rename igroup");
    $self->_clear_err();

    # parse arguments
    foreach (keys %args) {
        if (/-?new-name/) {
            if (not length($igroup_new_name = $args{$_})) {
                $self->_set_err("Invalid value for argument '-new-name'");
                return $ClientUtil::FALSE;
            }
            next;
        } else {
            $self->_set_err("Invalid parameter specified: '$_'");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Verify mandatory parameters
    #
    unless (defined($igroup_new_name)) {
        $self->_set_err("Igroup new name is required argument");
        return $ClientUtil::FALSE;
    }

    my $resultsHash_ref =
      $cmd_obj->execute_cmd("-cmd"                      => "igroup-rename",
                            "-cmd_filer"                => $self->{parent}->get_name(),
                            "-check_result"             => 1,
                            "-initiator-group-name"     => $igroup_name,
                            "-initiator-group-new-name" => $igroup_new_name);
    if ($cmd_obj->error()) {
        my $errmsg =
            "Failed calling execute_cmd for 'igroup-renames' " .
            "using $self->{parent}->{interface}:\n" .
            "Reason: ".
            join("\n", $cmd_obj->errmsg());
        $self->_set_err($errmsg);
        return $ClientUtil::FALSE;
    }

    $self->{parent}->log_cmd_results($resultsHash_ref);

    #
    # Check that the result and output aren't wrong
    #
    if (!$resultsHash_ref->{result_check}) {
        $self->_set_err(
                "Expected result did not match: " . $resultsHash_ref->{output});
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
}

################################################################################
# P R I V A T E   M E T H O D S
################################################################################
################################################################################
# Subroutine Name: _new()
# Objective:       Create a new igroup object
# Details:         See POD documentation at the end of this file
################################################################################
sub _new
{
    my ($class, %args) = @_;
    my (
         $conn,
         $log,
         $name,
         $parent,
         $clientObj,
         );

    my $self     = {};
    my $initObjs = [];
    my $_exist   = 0;

    bless $self,$class;

    # protocol and nodes are only used if we don't get a client or any initObjs
    my $protocol = "fcp";
    my $hosttype = "solaris";
    my $nodes    = "";

    # parse arguments
    foreach (keys %args) {

        if (/-?name/) {
            if (not length($name = $args{$_})) {
                $self->_set_err("Invalid value for argument '-name'");
                return $self;
            }
            next;
        }
        elsif (/-?_exist/) {
            if (not length($_exist = $args{$_}) and $_exist!~/1|0/) {
                $self->_set_err("Invalid value for argument '-_exist'");
                return $self;
            }
            next;
        }
        elsif (/-?init/i) {
            if (length($args{$_}) and ref($args{$_}) eq "ARRAY"){
                foreach my $obj (@{$args{$_}}){
                    unless (ref($obj) eq "Initiator"){
                        $self->_set_err("Invalid value for argument '-initObjs'");
                        return $self;
                    }
                }

                $initObjs=$args{$_};
            }
            elsif (length($args{$_}) and ref($args{$_}) eq "Initiator"){
                push my @ig,$args{$_};
                $initObjs=\@ig;
            }
            else {
                $self->_set_err("Invalid value for argument '-initObjs'");
                return $self;
            }
            next;
        }
        elsif (/-?parent/) {
            if (not length($parent = $args{$_})) {
                $self->_set_err("Invalid value for argument '-parent'");
                return $self;
            }
            next;
        }
        elsif (/-?client/) {
            unless (length($args{$_}) and ref($args{$_}) =~ "SanClient") {
                $self->_set_err("Invalid value for argument '-client'");
                return $self;
            }
            $self->{client} = $args{$_};
            next;
        }
        elsif (/-?protocol/) {
            unless (length($args{$_})) {
                $self->_set_err("Invalid value for argument '-protocol'");
                return $self;
            }
            $protocol = $args{$_};
            next;
        }
        elsif (/-?hosttype/) {
            unless (length($args{$_})) {
                $self->_set_err("Invalid value for argument '-hosttype'");
                return $self;
            }
            $hosttype = $args{$_};
            next;
        }
        elsif (/-?nodes/) {
            $nodes = $args{$_};
            next;
        }
        elsif (/-?loghdl/) {
            if (not length($log = $args{$_})) {
                $self->_set_err("Invalid value for argument '-loghdl'");
                return $self;
            }
            next;
        }
        else {
            $self->_set_err("Invalid parameter specified $_");
            return $self;
        }
    }
    $self->{log}=$log if (defined $log);

    # check for filer parent object (filer)
    unless (defined $parent and length $parent) {
        $self->_set_err("Required parameter parent object not given");
        return $self;
    }

    $self->{parent}  = $parent;
    $self->{client_or_init} = 1;

    # Create a new igroup if not exist
    if ($_exist eq 0) {

        # check that at least 1 initiator group or client object was passed in.
        unless (scalar(@$initObjs)>0 or defined $self->{client}) {
            $self->{client_or_init} = 0;
        }

        if (scalar(@$initObjs)>0) {
            $self->{initObjs}=$initObjs;

            # use the client handle from the first initiator group to determine
            # client specific info.

            $clientObj=$initObjs->[0]->{parent};


            # check for valid client object
            unless (defined $clientObj and length $clientObj) {
                $self->_set_err("Invalid or missing value for '-clientObj'");
                return $self;
            }

            $self->{client}=$clientObj;

            # use the hostname as the default igroup name
            # if a name is not given.
            unless (defined $name and length $name) {
                $name=$clientObj->get_name();
            }

        }

        #
        # Figure out the IGroup name, protocol and hostype.  If we were given
        # a client or initObj we can use a client obj, otherwise the user will
        # have to have given us a name, protocol and hosttype will either given
        # or defaulted.
        #
        if ($self->{client_or_init}) {
            # use the hostname as the default igroup name
            # if a name is not given.
            unless (defined $name and length $name) {
                $name=$self->{client}->get_name();
            }

            # determine protocol and hosttype to use from client object
            $self->{protocol}=($self->{client}->get_protocol() =~ /fcp/i) ? "fcp" : "iscsi";
            $self->{hosttype}=$self->{client}->hosttype();
        } else {
            #
            # If we get here we expected the user gave us a name a protocol
            # and the nodes by hand instead of using -client and/or -init
            #
            unless (defined $name and length $name) {
                $self->_set_err(
                    "Invalid or missing value. "                    .
                    "You must either supply a name using '-name', " .
                    "a SanClient object using '-client' or at "     .
                    "least 1 Initiator Object using '-init'");
                return $self;
            }
            $self->{node_list} = $nodes;
            $self->{protocol}  = $protocol;
            $self->{hosttype}  = $hosttype;
        }

        $self->{name} = $name;
        if ($self->_exist()) {
            $self->_set_err("Igroup '$name' already exists");
            return $self;
        }

        # Create igroup using attributes stored above
        $self->_create_igroup();
    }
    else {
        # Igroup must already exist so load it
        $self->_load_igroup($name);
    }

    #
    # Add a vfiler attribute to the object.  Basically, we will get the
    # vfiler contxt of the parent (SanFiler) object and save that as the
    # vfiler context.
    #
    my $vfiler = $self->{parent}->get_vfiler_context();
    unless ((defined($vfiler)) and (length($vfiler))) {
        $vfiler = "vfiler0";
    }
    $self->{vfiler}  = $vfiler;
    return $self;
} # end _new()

################################################################################
# Subroutine Name: _load_igroup
# Objective:       load existing igroup objects
# Details:         See POD documentation at the end of this file
################################################################################
sub _load_igroup
{
    my($self,$name)=@_;
    my (
        $cmd,
        $igroup,
        $protocol,
        @output,
        );


    $cmd="igroup show -v $name";

    # Query all available igroups
    @output = $self->{parent}->execute($cmd,-hide=>1);
    if ($output[0] =~ /No such initiator/i){
        $self->_set_err("Igroup '$name' does not exist",@output);
        return $ClientUtil::FALSE;
    }

    my @nodes=();
    foreach my $line(@output){
        if ($line=~/\((?:FCP|iSCSI)\)/i){
        # This won't work for the type of output where
        # client name also contains 'iscsi'
        #if ($line=~/FCP|iSCSI/i){
            ($igroup,$protocol)=$line=~/\s+(\S+)\s+\((\w+)\)/;
            $self->{name}=$igroup;
            $self->{protocol}=$protocol;
        }
        elsif($line=~/OS Type:/i){
            my($ostype)=$line=~/:\s+(\w+)/;
            $self->{ostype}=$ostype;
            $self->{nodes}=@_;
        }
        elsif ($line=~/Member:/i){
            my($node)=$line=~/Member:\s+(\S+)/i;
            if ($protocol=~/FCP/) {
                # store nodename unformatted; but not for iSCSI
                $node=~s/://g;
            }
            push @nodes,$node;
        }elsif($line=~/Throttle Reserve:/i){
            my ($throt_r) = $line=~/:\s+(\d+)\%/;
            $self->{throt_res} = $throt_r;
        }elsif($line=~/Throttle Borrow:/i){
            my ($throt_b) = $line=~/:\s+(\w+)/;
            $self->{throt_borrow} = $throt_b;
        }elsif($line=~/ALUA:/i){
            my ($alua) = $line=~/:\s+(\w+)/;
            $self->{alua} = $alua;
        }

    }

    $self->{nodes}=\@nodes;
    return $self;
}


################################################################################
# Subroutine Name: _create_igroup
# Objective:       create igroup object
# Details:         See POD documentation at the end of this file
################################################################################
sub _create_igroup
{
    my ($self)=@_;
    my (
        $cmd,
        $proto,
        @nodes,
        @output,
        );



    my $cmd_obj = $self->{parent}->{cmd_obj};
    my $hosttype = $self->{hosttype};
    my $protocol = $self->{protocol};

    # Get protocol (it's different for each interface, zapi and cli)
    if($self->{parent}->interface() eq "cli")
    {
        $proto=($protocol =~ /fcp/i) ? "-f" : "-i";
    }
    else
    {
        $proto = $protocol;
    }

    # Get ostype from hosttype
    if ($hosttype=~/sun|sol/i){
        $self->{ostype}="solaris";
    }
    elsif ($hosttype=~/hpux/i){
        $self->{ostype}="hpux";
    }
    elsif ($hosttype=~/aix/i){
        $self->{ostype}="aix";
    }
    elsif ($hosttype=~ /vmware/i) {
        $self->{ostype}="vmware";
    }
    elsif ($hosttype=~/linux/i){
        $self->{ostype}="linux";
    }
    elsif ($hosttype=~/sanblaze/i) {
        $self->{ostype}="linux";
    }
    elsif ($hosttype =~ /hyper-v/i) {
        # the hosttype for hyper-v must be before windows because it is denoted by
        # 'windows-hyper-v'
        # igroup type of 'hyper_v' is only available 7.3.1 and later,
        # default to 'windows' if on an earlier ONTAP

        # Note: we have to get the ONTAP version and in this case we need to
        # look at the minor revision. Because we do not have a good way to
        # get the minor revision, this check could always choose "hyper_v"
        # for builds that do not return standard version strings (like private builds)
        my $version = $self->{parent}->get_version();
        if ($version =~ /7\.[012]|7\.3:|anchor|tsing|gordon/i) {
            $self->{ostype} = "windows";
        } else {
            $self->{ostype} = "hyper_v";
        }
    }
    elsif ($hosttype=~/windows/i){
        $self->{ostype}="windows";
    }
    elsif ($hosttype=~/xen/i)
    {
        $self->{ostype}="xen";
    }
    else {
         $self->{ostype}=$hosttype;
         $self->sanlog(-msg => "creating igroup using " .
                       $hosttype . " hosttype");
    }

    # Retrieve the iscsi or fcp client nodes
    if ($self->{client_or_init}) {
        if ($protocol=~/fcp/i){
            foreach my $initObj (@{$self->{initObjs}}){
                push @nodes,$initObj->get_portname();

            }

        }
        elsif ($protocol=~/iscsi/i) {
            foreach my $initObj (@{$self->{initObjs}}){
                push @nodes,$initObj->get_nodename();

            }
        }
    }

    # Get the nodes from the node list
    # protocol and nodes are only used if we don't get a client or any initObjs
    if ($self->{node_list}) {
        foreach my $node (split(/\s+/,$self->{node_list})) {
                push @nodes, $node;
        }
    }

    $self->{nodes}=\@nodes;

    if($self->{parent}->interface() eq "zapi") {
        my $filer=$self->{parent}->get_name();
        my $name=$self->get_name();
        my %cmd_args = (
                    "-cmd", "igroup-create",
                    "-cmd_filer", $filer,
                    "-check_result", 1,
                    "-initiator-group-name", $name,
                    "-initiator-group-type", $proto,
                    "os-type", $self->{ostype},
                    "-node", join(" ",@nodes),
                   );
        # Execyte zapi command
        my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

        if($cmd_obj->error()) {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("Unable to create Igroup: $errmsg");
            return $ClientUtil::FALSE;
        }

        $self->{parent}->log_cmd_results($resultsHash_ref);

        if (!$resultsHash_ref->{result_check}) {
            $self->_set_err("Expected result did not match: " .
                            $resultsHash_ref->{output});
            return $ClientUtil::FALSE;
        }

        my $output = $resultsHash_ref->{output};

        # If the interface is zapi we have to convert the output from xml
        # to a string.
        my $output_hash = $self->{parent}->xml2hash($output);

        my $status = $output_hash->{status};
        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("Unable to create igroup '".$self->{name}."'",
                            $errmsg);
            return $ClientUtil::FALSE;
        }

        if( ($#nodes+1) >= 1 ) {
            my $node;
            my @nodearray;
            my $add_nodes_result;

            # add each node one at a time
            foreach $node (@nodes) {
                push @nodearray,$node;

                $add_nodes_result=$self->add_nodes(-igroup_nodes => \@nodearray);

                # Check that the result and output aren't wrong
                if (!$add_nodes_result) {
                    return $ClientUtil::FALSE;
                }

                @nodearray=();
            }
       }

    } else {
        # Execyte cli command
        $cmd = "igroup create ".$proto." -t ".$self->{ostype}." ".$self->{name}. " @{$self->{nodes}}";

        @output = $self->{parent}->execute($cmd,-hide=>1);
        if ("@output" =~ /already exists|illegal/i) {
            $self->_set_err("Unable to create igroup '".$self->{name}."'",
                            @output);
            return $ClientUtil::FALSE;
        }
    }

    return $ClientUtil::TRUE;
} # end _create_igroup()

################################################################################
# Subroutine Name: _exist()
# Objective:       Determine if an igroup exists
# Details:         See POD documentation at the end of this file
################################################################################
sub _exist
{
    my ($self)=@_;
    my (
        $cmd,
        $name,
        @output,
        );

    $name=$self->{name};

    $cmd = "igroup show ".$name;

    @output = $self->{parent}->execute($cmd,-hide=>1);
    if ($self->{parent}->error()) {
        $self->_set_err($self->{parent}->errmsg());
        return $ClientUtil::FALSE;
    }

    if ("@output" =~ /No such initiator/i) {
        return $ClientUtil::FALSE;
    }

    return $ClientUtil::TRUE;
} # end _exist()

1;


#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

IGroup.pm

=head1 Author

=begin html

Gary Knopp

=end html

=head1 Description

Igroup  provides an API interface to a filer igroup through the use of rsh.  The package can
be instantiated through any of the following SanFiler2 package methods;
create_igroup(), get_igroup(), get_igroups().

=head1 Attributes

=head2 B<parent>

reference = holds the parent object

=head2 B<name>

string = Name of the IGroup

=head2 B<nodes>

Array = Array of node names

=head2 B<protocol>

string = protocol type

=head2 B<ostype>

string = ostype type

=head2 B<client>

reference = holds the client object

=head2 B<initObjs>

reference = holds the array of initiator objects

=head1 Public Methods

=head2 B<destroy(%args)>

Destroy a filer igroup

=over 4

=item Inputs

=item * I<-force> (default: 0)

The option will destroy the igroup even if its currently mapped to luns.

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set, errmsg
is set to the failure reason, and 0 is passed back as the return value.

=item Examples

    $Test->nlog("-- Destroy igroup");

    $igrpObj->destroy(-force=>1);
    if ($igrpObj->error()){
        $Test->nlog($igrpObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<display()>

Display detailed igroup information

=over 4

=item Inputs

None

=item Outputs

Returns an array or array reference if successful.  If an error occured the error
flag is set, errmsg is set to the failure reason, and 0 is passed back as the return value.


=item Examples

    $Test->log("-- Retrieving igroup information");

    $Test->log($igrpObj->display());
    if ($igrpObj->error){
        $Test->log($igrpObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<error()>

This method allows you to determine if any errors occured in the
previous method call.  Note that the error flag is reset at the
beginning of each method, so it should be checked after B<every>
major method call.  If you do not check for errors, your calling
program may fail at some point and you will not know why.

=over 4

=item Inputs

None

=item Outputs

Returns 1 if an error occured in the previous call, 0 otherwise.

=item Examples

 # Determine if error occurred...
 if ($igrpObj->error) {
    $Test->log("Error encountered in previous method call",$igrpObj->errmsg());
 }

=back

=head2 B<errmsg()>

This method allows you to get the text of the error message if an error
occurred on the previous method call.

=over 4

=item Inputs

None

=item Outputs

Returns the text of the last error message set.  If no error message is
set, an empty string will be returned.

=item Examples

 # Check for errors.
 if ($igrpObj->error) {
   $Test->nlog($igrpObj->errmsg());
 }

=back

=head2 B<get_name()>

Name of filer igroup

=over 4

=item Inputs

none

=item Outputs

Returns a scalar containing the name of the igroup.  If an error occurs the error flag
is set, errmsg is set to the failure reason, and "" is returned.

=item Examples

    $Test->nlog("The igroup name is '".$igrpObj->get_name()."'");

=back

=head2 B<get_nodes()>

List of igroup nodes

=over 4

=item Inputs

none

=item Outputs

Returns an array or array reference of initiator nodes.  If an error
occurs the error flag is set, errmsg is set to the failure reason, and an empty array
or array refereced is returned.

=item Examples

    my @nodes=$igrpObj->get_nodes();

    if ($igrpObj->error){
        $Test->nlog($igrpObj->errmsg());
    }
    else {
        $Test->nlog("Initiator nodes:",@nodes);
    }


=back

=head2 B<get_node_cnt()>

Total number of igroup nodes

=over 4

=item Inputs

none

=item Outputs

Returns a scalar containining the number of igroup nodes.  If an error
occurs the error flag is set, errmsg is set to the failure reason, and 0 is passed
back as the return value;

=item Examples

    my $cnt=$igrpObj->get_node_cnt();

    $Test->nlog($igrpObj->get_name()." has a total of $cnt initiator nodes.");

=back

=head2 B<get_protocol()>

Retrieve the igroup protocol

=over 4

=item Inputs

none

=item Outputs

Returns a scalar containing the initiator protocol.  If an error occurs the error flag
is set, errmsg is set to the failure reason, and "" is returned.

=item Examples

    $Test->nlog("The igroup protocol is ".$iGrpObj->get_protocol());


=back

=head2 B<get_ostype()>

Retrieve the igroup OW type

=over 4

=item Inputs

none

=item Outputs

Returns a scalar containing the igroup OS type  If an error occurs the error flag
is set, errmsg is set to the failure reason, and "" is returned.

=item Examples

    $Test->nlog("The igroup ostype is ".$iGrpObj->get_ostype());

=back

=head2 B<get_vfiler()>

Retrieve the igroup vfiler context

=over 4

=item Inputs

none

=item Outputs

Returns a scalar containing the igroup vfiler context  If an error occurs the error flag
is set, errmsg is set to the failure reason, and "" is returned.

=item Examples

    $Test->nlog("The igroup vfiler context is ".$iGrpObj->get_vfiler());

=back

=head2 B<refresh()>

Refresh igroup object.  This method will rebuild node cache.

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set,
errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Examples

    $Test->log("Refresh igroup");
    if (not $igrpObj->refresh()) {
    if ($igrpObj->error){
                $Test->log($igrpObj->errmsg());
    }

=back

=head2 B<remove_nodes()>

Remove nodes from an igroup.

=over 4

=item Inputs

=item * -igroup_nodes

This option is used to pass the nodes that will be removed.
Severel nodes can be passed as an array reference.

=item * I<-force>

Removes the nodes from the igroup forcefully, even if it is currently mapped to luns.
Valid values are 1, yes or true.  The parameter defaults to 0.

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set,
errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Supported Interfaces

cli, zapi

=item Examples

    $igrpObj->remove_nodes("-igroup_nodes" => \@nodenames);
    if ($igrpObj->error()) {
        my $error_msg = join "\n", $igrpObj->errmsg();
        $Test->nlog("Failed to remove nodes from igroup " .
                    ":\nReason: $error_msg");
        return $TCD::FAIL;
    }
    $igrpObj->refresh();

=back

=head2 B<rename()>

Rename an existing igroup.

=over 4

=item Inputs

=item * -new-name

The new name for the igroup.

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set,
errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Supported Interfaces

cli, zapi

=item Examples

    $igrpObj->rename("-new-name" => $new_name);
    if ($igrpObj->error()) {
        my $error_msg = join "\n", $igrpObj->errmsg();
        $Test->nlog("Failed to rename igroup " .
                        ":\nReason: $error_msg");
        return $TCD::FAIL;
    }

=back

=head2 B<add_nodes()>

Add nodes to an igroup.

=over 4

=item Inputs

=item * -igroup_nodes

This option is used to pass the nodes that will be added.
Severel nodes can be passed as an array reference.

=item * I<-force>

Add the nodes to the igroup forcefully, even if it is currently mapped to luns.
Valid values are 1, yes or true.  The parameter defaults to 0.

=item Outputs

Returns a scalar value of 1 if successful.  If an error occured the error flag is set,
errmsg is set to the failure reason, and 0 is passed back as the return value.

=item Supported Interfaces

cli, zapi

=item Examples

    $igrpObj->add_nodes("-igroup_nodes" => \@nodenames);
    if ($igrpObj->error()) {
        my $error_msg = join "\n", $igrpObj->errmsg();
        $Test->nlog("Failed to add nodes to igroup " .
                    ":\nReason: $error_msg");
        return $TCD::FAIL;
    }
    $igrpObj->refresh();

=back

=head2 B<get_status()>

Retrieve the igroup alua status and ports name,

=over 4

=item Inputs

none

=item Outputs

Returns a hash containing the igroup alua status, whether vtic alone logged

in or not, number of ports  If an error occurs the error flag

is set, errmsg is set to the failure reason, and "" is returned.

=item Examples

    my $igroup_status = $iGrpObj->get_status();

=back

=cut