#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/SanFiler/Vfiler.pm#1 $

# Copyright (c) 2007-2008 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to a Vfiler
## @author pressley@netapp.com
## @status public
## @pod here

package San::SanFiler::Vfiler;

#
# Compiler directives.
#
use strict;

#
# Module import.
#
use San::SanLog;
use Error;
use Exporter;
use Data::Dumper;

use vars qw(      
          @EXPORT 
          @ISA
          );

#
# Inherit from the Error SanLog classes
#
@ISA = qw(Error SanLog);

########################################################################
# Sample hash from vfiler-list-info
########################################################################
#
# 'vfilers' => {
#     'vfiler-info' => {
#         'vfiler0' => {
#             'dns-info' => {
#                 'dns-servers' => {
#                     'server' => [
#                                  '10.60.248.10',
#                                  '10.60.132.40'
#                                 ]
#                 },
#                 'dns-domain-name' => 'rtp.netapp.com'
#             },
#             'uuid' => '00000000-0000-0000-0000-000000000000',
#             'vfstores' => {
#                 'vfstore-info' => {
#                     'status' => 'online',
#                     'path' => '/',
#                     'is-etc' => 'true'
#                 }
#             },
#             'vfnets' => {
#                 'vfnet-info' => [
#                                 {
#                                     'interface' => 'e0b',
#                                     'netmask' => '255.255.255.0',
#                                     'ipaddress' => '192.168.1.11'
#                                 },
#                                 {
#                                     'interface' => 'e1a',
#                                     'netmask' => '255.255.255.0',
#                                     'ipaddress' => '10.60.133.199'
#                                 },
#                                 {
#                                     'interface' => 'e1b',
#                                     'netmask' => '255.255.255.0',
#                                     'ipaddress' => '10.60.133.201'
#                                 }
#                                ]
#            },
#            'nis-info' => {
#                 'nis-domain-name' => 'netapp.com',
#                 'nis-servers' => {
#                     'server' => '10.60.132.13'
#                 }
#            },
#            'ipspace' => 'default-ipspace',
#            'admin-host' => {}
#         },
#         'fas3170-3-vf1' => {
#             'dns-info' => {
#                 'dns-servers' => {
#                     'server' => '10.60.132.40'
#                 },
#                 'dns-domain-name' => 'hq.netapp.com'
#             },
#             'uuid' => '51deafd4-e4d2-11dd-8cf5-00a0980c070e',
#             'vfstores' => {
#                 'vfstore-info' => {
#                     'status' => 'online',
#                     'path' => '/vol/fas3170_3_vf1',
#                     'is-etc' => 'true'
#                 }
#             },
#             'vfnets' => {
#                 'vfnet-info' => {
#                     'interface' => 'e1b',
#                     'netmask' => '255.255.255.0',
#                     'ipaddress' => '10.60.133.196'
#                 }
#             },
#             'nis-info' => {
#                 'nis-domain-name' => 'netapp.com',
#                 'nis-servers' => {
#                     'server' => 'y0'
#                 }
#             },
#             'ipspace' => 'default-ipspace',
#             'admin-host' => '10.60.133.199'
#         }
#     }
# }
#
########################################################################

########################################################################
#                            Public Methods
########################################################################

########################################################################
# Method:    get_name
# Objective: Return the name of the vfiler
# 
# Details:   See POD documentation at the end of this file.
########################################################################
sub get_name()
{
    my ($self) = @_;
    
    if (defined $self->{name}) {
        return ($self->{name});
    }
    
    $self->_set_err("Unable to determine vfiler name");
    return $ClientUtil::FALSE;
}

########################################################################
# Method:    get_ipaddress
# Objective: Return an IP address for the vfiler.  Let the user send
#            in an interface (i.e e0a) optionally and return the IP for
#            that interface if one is given.  If one is not given, we
#            can return the first one found or a space separated list
# 
# Details:   See POD documentation at the end of this file.
########################################################################
sub get_ipaddress()
{
    my ($self, %args) = @_;
    my $interface;
    my $requested_ip;
    my $ipaddress;

    $self->_clear_err();

    foreach (keys %args) {  
        if (/^-?interface$/) {
            unless(length($interface = $args{$_})) {
                $self->_set_err("Invalid value for \"interface\"");
            }
        } elsif (/^-?ipaddress$/) {
            unless(length($requested_ip = $args{$_})) {
                $self->_set_err("Invalid value for \"ipaddress\"");
            }
        }
    }
    
    #
    # We are expecting one of the two following formats for the
    # interface data:  Either there are multiple interfaces and
    # $self->{vfnets}{"vfnet-info"} is an array of hash, or there is
    # only one interface and the vfnet-info is a single hash.  Here's
    # and example of multiple interfaces:
    #
    # 'vfnets' => {
    #     'vfnet-info' => [
    #                     {
    #                         'interface' => 'e0b',
    #                         'netmask' => '255.255.255.0',
    #                         'ipaddress' => '192.168.1.11'
    #                     },
    #                     {
    #                         'interface' => 'e1a',
    #                         'netmask' => '255.255.255.0',
    #                         'ipaddress' => '10.60.133.199'
    #                     },
    #                     {
    #                         'interface' => 'e1b',
    #                         'netmask' => '255.255.255.0',
    #                         'ipaddress' => '10.60.133.201'
    #                     }
    #                    ]
    #  }
    #
    unless (defined $self->{vfnets}{"vfnet-info"}) {
        $self->_set_err("Unable to locate interface data in attributes");
        return $ClientUtil::FALSE;
    }
    
    my $vfnet;
    my $info = $self->{vfnets}{"vfnet-info"};
    
    #
    # info will be a hash if there is only one interface defined
    #
    if (ref($info) =~ /HASH/) {
        $vfnet = $info;
        if (defined($interface) and
            $interface ne $vfnet->{interface}) {
            $self->_set_err("Given interface does not match");
            return $ClientUtil::FALSE;
        }
        if (defined($requested_ip) and
            $requested_ip ne $vfnet->{ipaddress}) {
            $self->_set_err("Given ipaddress does not match");
            return $ClientUtil::FALSE;
        }
    } else {
        #
        # Mulitple interfaces with IP's will be in array
        #
        foreach my $current_vfnet (@{$info}) {
            if (defined($interface) and
                $interface eq $current_vfnet->{interface}) {
                $vfnet = $current_vfnet;
                last;
                
            } elsif (defined ($requested_ip) and
                      $requested_ip eq $vfnet->{ipaddress}) {
                $vfnet = $current_vfnet;
                last;
            } elsif (!defined($interface) and !defined($requested_ip)) {
                #
                # If we get here, the user hasn't told us which ip to
                # return, grab the first one (TBD)
                #
                $vfnet = $current_vfnet;
                last;
            }
        } # end foreach vfnet
    }
    
    #
    # If vfnet isn't defined then we didn't find one for the given
    # arguments.
    #
    unless (defined($vfnet)) {
        my $errmsg = "Unable to determine IP address";
        $errmsg .= " for the $interface interface" if (defined($interface));
        if (defined($requested_ip)) {
            $errmsg .= "and " if (defined($interface));
            $errmsg .= " for the $requested_ip ipaddress";
        }
        $self->_set_err($errmsg);
        return $ClientUtil::FALSE;
    }
    
    #
    # If we made it here, we should have a valid vfnet-info hash that
    # contains the ipaddress
    #
    if (defined $vfnet->{ipaddress}) {
        return($vfnet->{ipaddress});
    } else {
        $self->_set_err("Unable to determinte IP address, vfnet-info invalid");
        return $ClientUtil::FALSE;
    }
} # end get_ipaddress()

########################################################################
# Method:    get_paths
# Objective: get the path information for the vfiler
# Details:   See POD documentation at the end of this file.
########################################################################
sub get_paths()
{
    my ($self, %args) = @_;
    my ($first_found, $status, $is_etc, $path);

    $self->_clear_err();
    
    foreach (keys %args) {
        if (/^-?first-found$/) {
            unless(length($first_found = $args{$_})) {
                $self->_set_err("Invalid value for \"first-found\"");
            }
        } elsif (/^-?status$/) {
            unless(length($status = $args{$_})) {
                $self->_set_err("Invalid value for \"status\"");
            }
        } elsif (/^-?is-etc$/) {
            unless(length($is_etc = $args{$_})) {
                $self->_set_err("Invalid value for \"is-etc\"");
            }
        }
    }

    my $vfiler_hash = $self->list_info();

    #
    # We are expecting the following format. Here's
    # and example of multiple paths:
    #
    #                    <vfstores>
    #                            <vfstore-info>
    #                                    <path>/vol/vol1</path>
    #                                    <status>online</status>
    #                                    <is-etc>true</is-etc>
    #                            </vfstore-info>
    #                            <vfstore-info>
    #                                    <path>/vol/vol2</path>
    #                                    <status>online</status>
    #                                    <is-etc>false</is-etc>
    #                            </vfstore-info>
    #                    </vfstores>
    
    unless (defined $vfiler_hash->{"vfstores"}->{"vfstore-info"}) {
        $self->_set_err("Unable to locate path data in attributes");
        return $ClientUtil::FALSE;
    }

    my $vfstore_info;
    my $info = $vfiler_hash->{vfstores};

    if (ref($info) =~ /HASH/) {
        $vfstore_info = $info->{"vfstore-info"};
    } else {
        foreach my $current_vfstore (@{$info}) {
            if (!defined($status) and !defined($is_etc)) {
                #
                # If we get here, the user hasn't told us which path to
                # return, grab the first one (TBD)
                #
                $vfstore_info = $current_vfstore->{"vfstore-info"};
                last;
            }
        } # end foreach vfnet
    }

    #
    # If we made it here, we should have a valid vfstore-info hash that
    # contains the path
    #
    if (defined $vfstore_info->{"path"}) {
        return($vfstore_info->{"path"});
    } else {
        $self->sanlog(-msg       => "Unable to determine path");
        return $ClientUtil::FALSE;
    }

} # end get_paths()


################################################################################
# Subroutine Name: list_info()
# Objective:       Get info from filer and put it in a hash
# Details:         See POD documentation at the end of this file
################################################################################
sub list_info
{
    my ($self, %args) = @_;

    my $cmd_obj = $self->{parent}->cmd_obj();
    my $interface = $self->{parent}->interface();
    my $info_hash;

    $self->_clear_err();

    #
    # Create the list of arguments for the CmdsDriver call.
    # Note that -verbose is really for the cli and should just
    # be ignored for the zapi
    #
    my %cmd_args = ("-cmd"            => "vfiler-list-info",
                    "-cmd_filer"      => $self->{parent}->get_name(),
                    "-check_result"   => 1,
                    "-cmd_interface"  => $interface,
                    "-check_output"   => 0,
                    "-verbose"        => "",
                    "-vfiler"         => $self->{name},
                   );

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if($cmd_obj->error()) {
        $self->_set_err("CmdsDriver failed: ", $cmd_obj->errmsg());
        $self->sanlog(-msg => join ("\n", $cmd_obj->errmsg()) );
        return $ClientUtil::FALSE;
    }

    $self->{parent}->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("Result check failed: $output");
        return $ClientUtil::FALSE;
    }

    #
    # If we get here, we have results back from the filer.  If it's a zapi
    # we want to set the info_hash to the lun-info hash in the luns hash.
    # If it's cli, we want to parse the data line by line and fill in the
    # fields of the info_hash matching the field names from the zapi
    #
    if ($interface eq "zapi") {
        my $output_hash = $self->{parent}->xml2hash($output);
        unless (defined $output_hash->{"vfilers"}->{"vfiler-info"}) {
            $self->_set_err("Unable to parse output: {vfilers}->{vfiler-info} "
                            . "does not exist");
            return $ClientUtil::FALSE;
        }
        $info_hash = $output_hash->{"vfilers"}->{"vfiler-info"};
    } else {
        #
        # CLI case
        # We are expecting output that looks like this:
        # rtpqa-fas3050-23> vfiler status -a
        # vfiler0                          running
        #   ipspace: default-ipspace
        #   IP address: 10.60.169.134 [e0a]
        #   Path: / [/etc]
        #   UUID: 00000000-0000-0000-0000-000000000000
        #   Protocols allowed: 7
        #   Allowed: proto=rsh
        #   Allowed: proto=ssh
        #   Allowed: proto=nfs
        #   Allowed: proto=cifs
        #   Allowed: proto=iscsi
        #   Allowed: proto=ftp
        #   Allowed: proto=http
        #   Protocols disallowed: 0
        #
        my @output_array = split(/\n/, $output);
        my $vfiler_hash = {};
        foreach my $line (@output_array) {
            $line =~ s/^\s+//; # remove leading whitespace
            $line =~ s/\s+$//; # remove trailing whitespace
            if ($line =~ /running/) {
                #
                # Regex below parses the first line shown above in the example.
                # Using that example, for now just get the running vfilers...
                # this is incomplete...
                # vfiler0                          running
                #
                $line =~ /\s*(\S+)\s+\S*/;
                my $name = $1;
                #
                # Load the vfiler hash with what we know
                #
                $vfiler_hash->{name}        = $name;
                $self->sanlog(-msg  => "vfiler name is $name",
                              -msg_level => 7);

            } elsif ($line =~ /ipspace/) {
                $line =~ s/ipspace:\s+//; # strip off the prefix
                $vfiler_hash->{"ipspace"} = $line;
            } elsif ($line =~ /^UUID:/) {
                $line =~ s/UUID:\s+//;
                $vfiler_hash->{"uuid"} = $line;
            } elsif ($line =~ /^Path/) {
                #   Path: / [/etc]
                $line =~ /Path:\s+(\S+)\s+\S+/;
                $vfiler_hash->{"vfstores"}->{"vfstore-info"}->{"path"} = $1;
            } elsif ($line =~ /^IP address/) {
                # Should do something here...like build a vfnet hash...
            }
        } # end foreach line
        $info_hash = $vfiler_hash;
    } # end if CLI
    return $info_hash;
} # end list_info()

########################################################################
# Method:    start
# Objective: Call the vfiler-list-info ZAPI to refresh the data for
#            this vfiler.
# Details:   See POD documentation at the end of this file.
########################################################################
sub start()
{
    my ($self, %args) = @_;
    my $cmd_obj   = $self->{parent}->cmd_obj();
    my $filer     = $self->{parent}->get_name();
    my $interface = $self->{parent}->interface();

    $self->_clear_err();

    #
    # Start the vfiler
    #
    my $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "vfiler-start",
                        -cmd_filer              => $filer,
                        -check_result           => 0,
                        -vfiler                 => $self->{name});

    if($cmd_obj->error()) {
        my $errmsg = $cmd_obj->errmsg();
        $self->_set_err("Unable to start vfiler $self->{name}: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->{parent}->log_cmd_results($resultsHash_ref);
    
    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($interface eq "zapi") {
        my $output_hash = $self->{parent}->xml2hash($output);
        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("$errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if ($output =~ /does not exist/i) {
            my $errmsg = $output;
            $self->_set_err("could not start the vfiler $self->{name}: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

} # end start()

########################################################################
# Method:    stop
# Objective: Call the vfiler-list-info ZAPI to refresh the data for
#            this vfiler.
# Details:   See POD documentation at the end of this file.
########################################################################
sub stop()
{
    my ($self, %args) = @_;
    my $cmd_obj   = $self->{parent}->cmd_obj();
    my $filer     = $self->{parent}->get_name();
    my $interface = $self->{parent}->interface();

    $self->_clear_err();

    #
    # Stop the vfiler
    #
    my $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "vfiler-stop",
                        -cmd_filer              => $filer,
                        -check_result           => 0,
                        -vfiler                 => $self->{name});

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to stop vfiler $self->{name}: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->{parent}->log_cmd_results($resultsHash_ref);
    
    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($interface eq "zapi") {
        my $output_hash = $self->{parent}->xml2hash($output);
        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("$errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if ($output =~ /does not exist/i) {
            my $errmsg = $output;
            $self->_set_err("could not stop the vfiler $self->{name}: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

} # end stop()

########################################################################
# Method:    add
# Objective: Call the vfiler-add-storage ZAPI to add the given
#            volume/qtree to the vfiler
# Details:   See POD documentation at the end of this file.
########################################################################
sub add()
{
    my ($self, %args) = @_;
    my $cmd_obj   = $self->{parent}->cmd_obj();
    my $filer     = $self->{parent}->get_name();
    my $interface = $self->{parent}->interface();
    my $path;

    foreach (keys %args) {  
        if (/^-?path$/) {
            unless(length($path = $args{$_})) {
                $self->_set_err("Invalid value for \"path\"");
            }
        } else {
            $self->sanlog(-msg => "Vfiler::add() ignoring argument $_");
        }
    }

    #
    # Check for required path argument
    #
    unless (defined($path) and length($path)) {
        $self->_set_err("Missing required argument '-path'");
        return $ClientUtil::FALSE;
    }
    
    $self->sanlog(-msg => "Vfiler::add() Moving $filer:$path to vfiler "
                         . $self->get_name(),
                  -msg_level => 1);
    
    #
    # If the path doesn't start with /vol/, add it
    #
    unless ($path =~ /^\/vol\//) {
        $path = "/vol/$path";
    }
        
    #
    # Build the command line and execute the command
    #
    my %cmd_args = ("-cmd"              => "vfiler-add-storage",
                    "-cmd_filer"        => $filer,
                    "-check_result"     => 1,
                    "-vfiler"           => $self->get_name(),
                    "-storage-path"     => $path,
                   );
    $cmd_args{"-force"} = ""  if ($interface eq "cli");
    
    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        $self->_set_err("Calling execute_cmd: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->{parent}->log_cmd_results($resultsHash_ref);

    return $ClientUtil::TRUE;
} # end add()

########################################################################
# Method:    remove
# Objective: Call the vfiler-list-info ZAPI to refresh the data for
#            this vfiler.
# Details:   See POD documentation at the end of this file.
########################################################################
sub remove()
{
    my ($self, %args) = @_;
    my $cmd_obj   = $self->{parent}->cmd_obj();
    my $filer     = $self->{parent}->get_name();
    my $interface = $self->{parent}->interface();
    my $path;

    foreach (keys %args) {
        if (/^-?path$/) {
            unless(length($path = $args{$_})) {
                $self->_set_err("Invalid value for \"path\"");
            }
        } else {
            $self->sanlog(-msg => "Vfiler::remove() ignoring argument $_");
        }
    }

    #
    # Check for required path argument
    #
    unless (defined($path) and length($path)) {
        $self->_set_err("Missing required argument '-path'");
        return $ClientUtil::FALSE;
    }

    $self->sanlog(-msg => "Vfiler::remove() Removing $filer:$path from vfiler "
                         . $self->get_name(),
                  -msg_level => 1);

    #
    # If the path doesn't start with /vol/, add it
    #
    unless ($path =~ /^\/vol\//) {
        $path = "/vol/$path";
    }

    #
    # Build the command line and execute the command
    #
    my %cmd_args = ("-cmd"              => "vfiler-remove-storage",
                    "-cmd_filer"        => $filer,
                    "-check_result"     => 1,
                    "-vfiler"           => $self->get_name(),
                    "-storage-path"     => $path,
                   );
    $cmd_args{"-force"} = ""  if ($interface eq "cli");

    my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);
    if ($cmd_obj->error()) {
        $self->_set_err("Calling execute_cmd: ", $cmd_obj->errmsg());
        return $ClientUtil::FALSE;
    }

    $self->{parent}->log_cmd_results($resultsHash_ref);

    return $ClientUtil::TRUE;
} # end remove()

########################################################################
# Method:    refresh
# Objective: Call the vfiler-list-info ZAPI to refresh the data for
#            this vfiler.
# Details:   See POD documentation at the end of this file.
########################################################################
sub refresh()
{
    return 1;
} # end refresh()

########################################################################
# Method:    destroy
# Objective: Destroy the vfiler on the filer and set $self to an
#            empty hash.
#
# Details:   See POD documentation at the end of this file.
########################################################################
sub destroy()
{
    my ($self, %args) = @_;
    my $cmd_obj   = $self->{parent}->cmd_obj();
    my $filer     = $self->{parent}->get_name();
    my $interface = $self->{parent}->interface();
    my $resultsHash_ref;

    $self->_clear_err();
    
    #
    # Destroy the vfiler
    #
    $resultsHash_ref =
        $cmd_obj->execute_cmd(
                        -cmd                    => "vfiler-destroy",
                        -cmd_filer              => $filer,
                        -check_result           => 1,
                        -vfiler                 => $self->{name},
                        -force                  => "");

    if($cmd_obj->error()) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to destroy vfiler $self->{name}: $errmsg");
        return $ClientUtil::FALSE;
    }

    $self->{parent}->log_cmd_results($resultsHash_ref);

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If the interface is zapi we have to convert the output from xml
    # to a string.
    #
    if($interface eq "zapi") {
        my $output_hash = $self->{parent}->xml2hash($output);
        my $status = $output_hash->{status};

        if($status eq "failed") {
            my $errmsg=$output_hash->{reason};
            $self->_set_err("$errmsg");
            return $ClientUtil::FALSE;
        }
    } else {
        if ($output =~ /'"'.$self->{name}.'" does not exist.'/i) {
            my $errmsg = $output;
            $self->_set_err("could not destroy the vfiler $self->{name}: $errmsg");
            return $ClientUtil::FALSE;
        }
    }

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ($resultsHash_ref->{result_check}) {
        $self->_set_err("results check failed");
        return $ClientUtil::FALSE;
    } else {
        return $ClientUtil::TRUE;
    }
} # end destroy()

###############################################################################
#                               Private Methods
###############################################################################

########################################################################
# Method:    _new
# Objective: Create new Vfiler object.
# Details:   This constructor will not actually create a vfiler on
#            the filer.  It will bless the hash given as the object.
#            As such, we expect to be given a hash from the
#            vfiler-list-info ZAPI (with -parent added).
#
#            There may be portions of that hash that we need to check
#            and reformat.  For example, parts of the hash may be
#            a hash for 1 entry and an array of hash for more than
#            one entry.  TBD, reformat the hash here OR just make sure
#            the accessor methods know what to do.
########################################################################
sub _new
{
    my ($proto, %vfiler_hash) = @_;
    my $class = ref($proto) || $proto;

    #
    # Create an object from the hash given
    #
    my $self = \%vfiler_hash;
    bless $self, $class;

    $self->_clear_err();
    return $self;
}  # end of _new method

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

San::SanFiler::Vfiler

=head1 Author

Ann Harrold

=head1 Description

Vfiler.pm provides an API interface to a Vfiler.  Vfiler objects are created
by the SanFiler::create_vfiler() and SanFiler::_load_vfilers() methods

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error.pm>
B<SanLog.pm>

=head1 Attributes

=head2 B<name>

Name of the vfiler

=head2 B<dns-info>

Hash of dns information inclusing hashes for dns-servers

=head2 B<uuid>

Universal Unique Identifier

= head2 B<vfstores>

Hash of data for the vfiler stores (volumes)

= head2 B<vfnets>

Hash of data for the vfiler network (intefrace, ip, etc)

=head2 B<nis-info>

Hash of NIS data (domain name, servers, etc)

=head2 B<ipspace>

Name of the vfiler ipspace

=head2 B<admin-host>

IP address of the admin host

=head1 Public Methods

=back

=head2 B<get_name()>

Access the name attribute

=over 4

=item Inputs

None

=item Outputs

Returns the vfiler name if defined. If it is not defined, the
error flag is set, errmsg is set to the failure reason, and FALSE is
passed back as the return value.

=item Supported Interfaces

All

=item Examples

    my $vfiler_name = $vfiler_obj->get_name();

=back

=head2 B<get_ipaddress(%args)>

Return an IP address for the vfiler.  If the user sent in an interface
(i.e e0a), return the IP for that interface.  If the user sent in an
ip address, we will return that same IP address if it's found.  Sending
in an ip address is really a way of checking the validity of a given
ip address for the vfiler, this is used by SanFiler::set_vfiler_context().
If an error is found, the error flag is set, errmsg is set to the failure
reason, and FALSE is passed back as the return value.

=over 4

=item Inputs

=item * I<-interface>

Vfiler interface to get IP address for (i.e. e0a)

=item * I<-ipaddress>

Requested IP address

=item Outputs

Returns a TRUE if successful. If an error occured the error flag is set,
errmsg is set to the failure reason, and FALSE is returned.

=item Supported Interfaces

All

=item Examples

    my $vfiler_ip = $vfiler_obj->get_ipaddress(-interface => "e0b");

=back

=head2 B<get_paths(%args)>

Return the path for the vfiler.  If the user sent in is_etc, return the
path that corresponds to /etc for this vfiler (TBD).  If the user sent in
first_found, we will return the first path found (TBD).  If the user sent in
status, we will return the paths corresponding to that status - offline/
online.  If an error is found, the error flag is set, errmsg is set to the
failure reason, and FALSE is passed back as the return value.

=over 4

=item Inputs

=item * I<-first_found>

Whether to return the first path found

=item * I<-status>

Whether to return online/offline volumes

=item * I<-is-etc>

Whether to return the /etc device for this vfiler

=item Outputs

Returns a volume path if successful. If an error occured the error flag is set,
errmsg is set to the failure reason, and FALSE is returned.

=item Supported Interfaces

All

=item Examples

        my $path = $vfilerobj->get_paths();

=back

=head2 B<list_info()>

Returns a hash with information about the lun. For the ZAPI interface,
we simply translate the XML returned into the hash and return the
data that came back.  For the CLI, we parse the data that came back
and the hash will contain fields that match those from the ZAPI.
If a failure occurs, the error flag is set, errmsg is set to
the failure reason, and 0 is passed back as the return value.

=item Supported Interfaces:

cli zapi

=over 4

=item Outputs

Returns a volume path if successful. If an error occured the error flag is set,
errmsg is set to the failure reason, and FALSE is returned.

=item Supported Interfaces

All

=item Examples

    my $vfiler_hash = $vfiler_obj->list_info();
    if ($vfiler_obj->error()) {
        $Test->nlog("Error getting vfiler info: " . $vfiler_obj->errmsg())
    }

=back

=head2 B<add(%args)>

Add the given volume/qtree to the volume

=over 4

=item Inputs

=item * -path

Path of storage to add

=item Outputs

Returns a TRUE if successful. If an error occured the error flag is set,
errmsg is set to the failure reason, and FALSE is returned.

=item Supported Interfaces

cli, zapi

=item Examples

    $vfiler_obj->add(-path => "/vol/vf_vol");

=back

=head2 B<start()>

Start the vfiler - TBD - not yet implemented

=back

=head2 B<stop()>

Stop the vfiler - TBD - not yet implemented

=back

=head2 B<remove()>

Remove storate (volume) from the vfiler - TBD - not yet implemented

=back

=head2 B<destroy()>

Destroy the vfiler - TBD - not yet implemented

=back

=head2 B<refresh()>

Update the Vfiler objects attributes - TBD - not yet implemented

=back

=cut

1;
