#
# Copyright (c) 2010 NetApp Inc.
# All rights reserved.
#
## @summary Provide a common set of libraries to work on resource validation
## @author latif@netapp.com
## @status private
## @pod here

package San::SanMgr::Client::validate;

# Compiler directives.
use strict;

# Module import.
use Tharn;
use Error;
use San::SanLog;
use ParamsMgr;
use San::SanUtil;

use Data::Dumper;

use vars qw(@ISA);
@ISA = qw(Error SanLog);

###############################################################################
# Method:    validate_single
# Objective: Setup run against a single resource
# Details:   See POD documentation at the end of this file.
###############################################################################
sub validate_single {
    my ($self, %args) = @_;

    &ParamsMgr::process_args(%args);
    my $resource = &ParamsMgr::get_arg(-arg => "resource", -required => 1, -default => "");
    if (&ParamsMgr::error()) {
        &SubIPC::fail(-msg => &ParamsMgr::get_error());
    }

    # $self is a Client object
    # $resource is a hash of parameters for a particular resource to work on
    $self->sanlog(-msg => "Resource parameters/data:\n" . Dumper($resource),
                  -msg_level => 0);

    # Get object for resource supplied
    my $client_object = $self->{parent}->get_objects(-type => "client", -name => $resource->{NAME});
    if ($self->error()) {
        $self->sanlog(-msg => $self->get_error());
        &SubIPC::fail(-msg => $self->get_error());
    }

    $self->sanlog(-msg => "Client object is:\n" . Dumper($client_object) . "\n",
                  -msg_level => 10);

    # Example of message write to parent with type of misc
    &SubIPC::write(-msg => "Writing a message", -type => "misc");
    if (&SubIPC::error()) {
        $self->_set_err(&SubIPC::get_error());
        &SubIPC::fail(-msg => $self->get_error());
    }

    # Sleep x seconds to just sleep. . .
    &SanUtil::nate_sleep(10);

    # Example of read from parent
    #my $message = &SubIPC::read(-block => 1);
    #if ($message ne undef) {
    #    $self->sanlog(-msg => "Received: " . Dumper($message) . " from parent");
    #}

    # Example of subtest fail and pass
    #&SubIPC::fail(-msg => "Had a failure");
    &SubIPC::pass(-msg => "Subtest completed");

    return $SanMgr::TRUE;
}

1;
