# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/San/SanNdu.pm#1 $
#
# Copyright (c) 2008 Network Appliance, Inc.
# All rights reserved.

## @summary
## San NDU specific functions and variables. The library has three functions: 
## NDU, revert and upgrade to handle the upgrade or revert
## process using SANInterop's ndu.thpl and revert.thpl code
##
## @description
## This package contains some of the useful commands that are used by ndu/upgrade/revert test
##
## @author yuj@netapp.com
##
##
## @param $filers, $dest_release, $ndu_type
package San::SanNdu;

#
# Compiler directives.
#
use strict;

# Module import.
use Exporter;
use San::SanLog;
use Error;
use San::SanUtil;
use vars qw(@ISA);
use Tharn;

#
# Inherit from the Error, SanLog, SanUtil, Sanfilerand SanFiler7M classes
#
@ISA = qw(Exporter Error SanLog SanUtil);

###############################################################################
#                               Public Methods
###############################################################################

###############################################################################
# Method:    new
# Objective: Create new SanNDU object. initialize default hash values 
# if not passed in
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class, %args) = @_;
    my $self = bless({}, $class);
    $self->sanlog(-msg => "SanNdu::new() - creating new SanNdu object");
    $self->{ontap_build_type} = "pbo";       #build type: standard,debug,pbo
    $self->{oem_rebrand}      = "netapp";    #OEM rebrand: IBM, netapp
    $self->{revert_ignore_version_mismatch} =
      "1"; #Whether to ignore version mismatch on filers: 1-not ignore, 0-ignore
    $self->{revert_sis_timeout} = "10800";
    $self->{ndu_force_giveback} = "1";
    $self->{time_out}           = "86400";
    $self->{FILER_DOWN_TIMEOUT} = "1200";
    $self->_clear_err();

    foreach (keys %args) {
        if (/^-?ontap_build_type?/) {
            length($self->{ontap_build_type} = $args{$_})
              or do {
                $self->_set_err(
                     "Error: invalid value for \"-ontap_build_type\": $args{$_}"
                );
                return 0;
              };
        } elsif (/^-?oem_rebrand?/) {
            length($self->{oem_rebrand} = $args{$_})
              or do {
                $self->_set_err(
                        "Error: invalid value for \"-oem_rebrand\": $args{$_}");
                return 0;
              };
        } elsif (/^-?revert_ignore_version_mismatch?/) {
            length($self->{revert_ignore_version_mismatch} = $args{$_})
              or do {
                $self->_set_err(
                    "Error: invalid value for \"-revert_ignore_version_mismatch\": $args{$_}"
                );
                return 0;
              };
        } elsif (/^-?ndu_force_giveback?/) {
            length($self->{force_giveback} = $args{$_})
              or do {
                $self->_set_err(
                    "Error: invalid value for \"-ndu_force_giveback\": $args{$_}"
                );
                return 0;
              };
        } elsif (/^-?revert_sis_timeout?/) {
            length($self->{revert_sis_timeout} = $args{$_})
              or do {
                $self->_set_err(
                    "Error: invalid value for \"-revert_sis_timeout\": $args{$_}"
                );
                return 0;
              };
        } elsif (/^-?time_out?/) {
            length($self->{time_out} = $args{$_})
              or do {
                $self->_set_err(
                           "Error: invalid value for \"-time_out\": $args{$_}");
                return 0;
              };

        } else {
            $self->_set_err("Error: invalid parameter specified: $_");
            return 0;
        }
    } ## end foreach (keys %args)

    return $self;
} ## end sub new
###############################
#Method: setup
###############################


###############################################################################
# Method:    ndu
# Objective: use all passed in parameters and call SANInterop/fault/ndu.thpl
# for a minor or major NDU
# Details:   See POD documentation at the end of this file.
###############################################################################
sub ndu {
    my ($self, %args) = @_;

    # init default ndu_type as major
    my $ndu_type = $args{-ndu_type};
    my $filers = $args{-filers};
    my $dest_release = $args{-dest_release};
    
    $self->_clear_err();
    # Process arguments.  If an error occurs during the arg processing,
    # The error flag and errmsg values are set and the object is returned.
    # It is up to the caller to check for errors after the new call.

    #Verify mandatory parameters.
    unless (defined($filers)) {
        $self->_set_err("Error: '$filers' is required parameter for filer information");
        return 0;
    }
    unless (defined($dest_release)) {
        $self->_set_err(
            "Error:  'dest_release' is reqired parameter for the destination release"
        );
        return 0;
    }
    unless (defined($dest_release)){
        $self->_set_err(
            "Error:  'ndu_type' is reqired parameter for the NDU type"
        );
        return 0;
    }
    
    
    
    my $ndu_ret = $self->_ndu_upgrade($filers,$dest_release,$ndu_type);
    return ($ndu_ret == 0) ? 1 : 0;
} ## end sub ndu
###############################################################################
# Method:    upgrade
# Objective: use all passed in parameters and call SANInterop/fault/ndu.thpl
# for an upgrade, which is disruptive
# Details:   See POD documentation at the end of this file.
###############################################################################
sub upgrade {
    my ($self, %args) = @_;
    my $ndu_type = "disruptive";
    my $filers = $args{-filers};
    my $dest_release = $args{-dest_release};
    
    $self->_clear_err();
    # Process arguments.  If an error occurs during the arg processing,
    # The error flag and errmsg values are set and the object is returned.
    # It is up to the caller to check for errors after the new call.

    #Verify mandatory parameters.
    unless (defined($filers)) {
        $self->_set_err("Error: '$filers' is required parameter for filer information");
        return 0;
    }
    unless (defined($dest_release)) {
        $self->_set_err(
            "Error:  'dest_release' is reqired parameter for the destination release"
        );
        return 0;
    }

    $self->_ndu_upgrade($filers,$dest_release,$ndu_type);
    
    if ($self->error()) {
        my $error_msg = $self->errmsg();
        $self->_set_err("--upgrade test failed with message: $error_msg");
        return 0;
    }
        
    return 1;
} ## end sub upgrade
###############################################################################
# Method:    revert
# Objective: use all passed in parameters and call SANInterop/fault/revert.thpl
# for a revert test, which is disruptive
# Details:   See POD documentation at the end of this file.
###############################################################################
sub revert {
    my ($self, %args) = @_;
    my $fail_cnt = 0;
    my $filers       = $args{-filers};
    my $dest_release = $args{-dest_release};

    $self->_clear_err();
    # Process arguments.  If an error occurs during the arg processing,
    # The error flag and errmsg values are set and the object is returned.
    # It is up to the caller to check for errors after the new call.

    #Verify mandatory parameters.
    unless (defined($filers)) {
        $self->_set_err("Error: '$filers' is required parameter for filer information");
        return 0;
    }
    unless (defined($dest_release)) {
        $self->_set_err(
            "Error:  'dest_release' is reqired parameter for the destination release"
        );
        return 0;
    }

    $self->sanlog(
             -msg => "-- Subtest revert.thpl will be run to start revert test");

    #call subtest ndu.thpl code from SANInterop to perform NDU
    my $revert_sb_obj = subtest(
        "SANInterop/fault/revert.thpl",
        # thpl script parameters
        "REVERT_ONTAP_DEST=$dest_release",
        "ONTAP_BUILD_TYPE=$self->{ontap_build_type}",
        "OEM_REBRAND=$self->{oem_rebrand}",
        "REVERT_SIS_TIMEOUT=$self->{revert_sis_timeout}",
        "REVERT_IGNORE_VERSION_MISMATCH=$self->{revert_ignore_version_mismatch}",
        "FILERS=$filers",
        # Subtest specific options
        -nobg,
        -runid =>"san_revert",
        # Fail if the test takes longer than time_out value
        -timeout =>$self->{time_out});

    # Since this last subtest was run in the foreground, it'll not return quickly.
    #   we need to sit around and wait for it to finish so we can get the
    #   results

    $self->sanlog(-msg => "Waiting on subtest revert.thpl to finish");

    $revert_sb_obj->wait_finish();

    $self->sanlog(
           -msg => "   - subtest " . $revert_sb_obj->runid() . " is completed");

    $self->sanlog(-msg => "-- Verifying subtest results");

    my @ret_results = $revert_sb_obj->rslts();

    foreach my $result (@ret_results) {
        if ($result->type() =~ /pass/) {
            $fail_cnt = 0;
            $self->sanlog(-msg => "upgrade has completed successfully");
        } elsif ($result->type() =~ /FATAL|WARN|FAIL|SCRIPT|CONFIG/i) {
            $self->_set_err($result);
            $self->sanlog(-msg => "   - "
                            . $result->type() . "("
                            . $result->severity
                            . ") RESULT for "
                            . $revert_sb_obj->runid(),
                          "   - msg: " . $result);
            $fail_cnt++;
        } ## end elsif ($result->type() =~...

    } ## end foreach my $result (@ret_results)

    return ($fail_cnt == 0) ? 1 : 0;
} ## end sub revert
###############################################################################
#private funtion
# Method:    _ndu_upgrade
# Objective: use all passed in parameters and call SANInterop/fault/ndu.thpl
# for an upgrade or ndu
# Details:   See POD documentation at the end of this file.
###############################################################################

sub _ndu_upgrade {
    my ($self, $filers, $dest_release, $ndu_type) = @_;
    my $error_msg;
    my $fail_cnt = 0;    
    
    #call subtest ndu.thpl code from SANInterop to perform NDU
    $self->sanlog(-msg => "SanNdu::_ndu_upgrade() -start upgrade subtest");
    my $upg_sb_obj = subtest(
        "SANInterop/fault/ndu.thpl",
        # thpl script parameters
        "NDU_ONTAP_DEST=$dest_release",
        "NDU_TYPE=$ndu_type",
        "NDU_FORCE_GIVEBACK=$self->{ndu_force_giveback}",
        "ONTAP_BUILD_TYPE=$self->{ontap_build_type}",
        "OEM_REBRAND=$self->{oem_rebrand}",
        "FILER_DOWN_TIMEOUT=$self->{FILER_DOWN_TIMEOUT}",
        "NDU_POLICY_FILE=http://eng-web.rtp.netapp.com/iaq/bin/share/ndupolicy_73.conf",
        "FILERS=$filers",
        # Subtest specific options
        -nobg,
        -runid =>"san_ndu_upgrade",
        # Fail if the test takes longer than 24 hours
        -timeout =>$self->{time_out});

    $self->sanlog(-msg => "Waiting on subtest ndu.thpl to finish");

    $upg_sb_obj->wait_finish();

    $self->sanlog(
              -msg => "   - subtest " . $upg_sb_obj->runid() . " is completed");

    $self->sanlog(-msg => "-- Verifying subtest results");
    my @upg_result = $upg_sb_obj->rslts();
    foreach my $result (@upg_result) {
        if ($result->type() =~ /pass/) {
            $fail_cnt = 0;
            $self->sanlog(-msg => "upgrade has completed successfully");
        } elsif ($result->type() =~ /FATAL|FAIL|SCRIPT|CONFIG/i) {
            $self->_set_err($result);
            $self->sanlog(-msg => "   - "
                            . $result->type() . "("
                            . $result->severity
                            . ") RESULT for "
                            . $upg_sb_obj->runid(),
                          "   - msg: " . $result);
            $fail_cnt=1;
        } ## end elsif ($result->type() =~...
    } ## end foreach my $result (@upg_result)

    return ($fail_cnt == 0) ? 1 : 0;

} ## end sub _ndu_upgrade

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

SanNdu.pm

=head1 Author

Jiangping Yu

=head1 Description

SanNdu.pm provides an API interface for NDU/upgrade/revert test for San QA team 

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error.pm>

=head1 Attributes

=head2 B<san_obj>

scalar = san object

=head2 B<filers>

array = list of filer objects

=head2 B<dest_release>

string = destination release

=head2 B<ontap_build_type>

string = <standard | debug | pbo> (default pbo)ontap build type

=head2 B<ndu_type>

string =  <minor | major | disruptive>  NDU type

=head2 B<oem_rebrand>

string = <IBM | netapp>(default netapp) OEM rebrand type

=head2 B<revert_ignore_version_mismatch>

boolean = <1 | 0> to ginore version mismatch or not
    
=head1 Public Methods

=head2 B<new()>

Creates a new SanNdu object. 

=over 4

=item Inputs

None

=item Outputs

Returns a SanNdu object.

=item Examples

# Create a SanNDU object

my $NduObj = new San::SanNdu;

=back

=head2 B<san_ndu_setup(%args)>

Get required parameters and set defaults for SanNDU onject. 

=over 4

=item Inputs

=item * -san_obj

pass in the san_obj object

=item * -filers

get the filers information

=item * -dest_release

get destination release information

=item Outputs

Returns a scalar value of 1 if successful.If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

    $NduObj->san_ndu_setup(-san_obj      => $san_obj,
                           -filers       => $filers,
                           -dest_release => "mainngN",
                                                    );

=back

=head2 B<ndu()>

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if successful.If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

    #
    # Instantiate SanNDU Object
    #
    my $NduObj = new San::SanNdu;

    $NduObj->san_ndu_setup(-san_obj      => $san_obj,
                           -filers       => $filers,
                           -dest_release => "mainngN",
                           -ndu_type     => "major");
    #run ndu test                       
    my $ret = $NduObj->ndu();                  

=back

=head2 B<upgrade()>

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if successful.If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

    my $NduObj = new San::SanNdu;
    
    $NduObj->san_ndu_setup(-san_obj      => $san_obj,
                           -filers       => $filers,
                           -dest_release => "7.3",
                                                    );

    my $ret = $NduObj->upgrade();
    if $NduObj->error(){
        my $error_msg = $NduObj->errmsg();
        $Test->nlog("Error: failed to upgrade: $error_msg");
        return $TCD::FAIL;
    }

=back

=head2 B<revert()>

=over 4

=item Inputs

none

=item Outputs

Returns a scalar value of 1 if successful.If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

    my $NduObj = new San::SanNdu;
    
    $NduObj->san_ndu_setup(-san_obj      => $san_obj,
                           -filers       => $filers,
                           -dest_release => "7.3");

    my $ret = $NduObj->revert();

=back


1;
