# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/San/SanXfun/SanXfun.pm#1 $
#
# Copyright (c) 2009 Network Appliance, Inc.
# All rights reserved.
## @summary Provide an interface to a SanXfun test suite. This class just a wrapper to San.
## It provides addtional setup for SnapMirror and some usefull function to test with 
## cross functional usecases
##
## How to use : http://wikid.netapp.com/w/SAN_QA/SAN_QA_Feature_Team/Cross_Functional_Project_Specification
##
## @author Muthukumaran Govindarajan (muthug@netapp.com)
## @status public
## @pod here

# Class : SanXfun

package SanXfun;

use strict;

#
# Inherit from the following classes (DO NOT CHANGE THE ORDER)
#     San::San (SUPER class)
#     San::SanXfun::SanXfunIoUtil
#     San::SanXfun::SanXfunSMBase
#

BEGIN {
    use San::San;
    use San::SanXfun::SanXfunIoUtil;
    use San::SanXfun::SanXfunSMBase;

    use base qw{San SanXfunIoUtil SanXfunSMBase};
}

use Tharn;    # for connect, disconnect, subtest, logcomment
use File::Basename;
use File::Copy;
use Admin::License qw(license);
use Wafliron;    # for wafliron_start
use Leak;        # for mem_monitor and mem_check
use Data::Dumper;

##############################################################################
###   Function:                  - setup - To create SanXfun environmnet for
###                              testing cross functional use cases. This is
###                              wrap for San::setup.
###   Parameters:                - See Pod more details
###   Returns:
###      on success              - 1
###      on failure              - 0
##############################################################################
sub setup() {
    my ( $self, %args ) = @_;

    $self->sanlog( -msg => "SanXfun setup" );
    $self->_clear_err();

    #
    # check for existance of required arguments: filers
    #
    unless ( defined $args{-filers} and length $args{-filers} ) {
        $self->_set_err("Missing required argument '-filers'");
        return $SanUtil::FALSE;
    }

    #
    # check for SanXfun environment variables
    # validate the param and set it to default if specified wrongly.
    # Remove the options from the args list, which is not
    # supported by San::setup
    #

    # SanXfun Environment variable
    if ( defined $args{-san_xfun_env} ) {
        $self->{san_xfun_env} =
          ( $args{-san_xfun_env} =~ /default|snapmirror|vfiler|snapvault/i )
          ? lc( $args{-san_xfun_env} )
          : "default";
        delete $args{-san_xfun_env};
    }
    else {
        $self->{san_xfun_env} = "default";
    }

    #
    # if SanXfun environment is snapmirror, we need the filer_vol_list
    # check for existance of required arguments: filer_vol_list
    #
    if ( $self->{san_xfun_env} =~ /snapmirror/i ) {
        unless ( defined $args{-filer_vol_list}
            and length $args{-filer_vol_list} )
        {
            $self->_set_err( "Missing required argument '-filer_vol_list'"
                  . " for snapmirror environment" );
            return $SanUtil::FALSE;
        }
    }

    # SnapMirror sync type, default : async
    if ( defined $args{-sm_sync_type} ) {
        $self->{sm_sync_type} =
          ( lc( $args{-sm_sync_type} ) eq "sync" ) ? "sync" : "async";
        delete $args{-sm_sync_type};
    }
    else {
        $self->{sm_sync_type} = "async";
    }

    # SnapMirror type, default : vsm, volume snapmirror
    if ( defined $args{-sm_replicate_type} ) {
        $self->{sm_replicate_type} =
          ( lc( $args{-sm_replicate_type} ) eq "qsm" ) ? "qsm" : "vsm";
        delete $args{-sm_replicate_type};
    }
    else {
        $self->{sm_replicate_type} = "vsm";
    }

    # Interface, default : cli
    if ( defined $args{-interface} ) {
        $self->{interface} = ( $args{-interface} =~ /zapi/i ) ? "zapi" : "cli";
        delete $args{-interface};
    }
    else {
        $self->{interface} = "cli";
    }

    # protocol, default : fcp
    if ( defined $args{-protocol} ) {
        $self->{protocol} = ( $args{-protocol} =~ /iscsi/i ) ? "iscsi" : "fcp";
    }
    else {
        $self->{protocol} = "fcp";
    }

    # San debug level
    if ( defined $args{-debug_level} ) {
        $self->{debug_level} = $args{-debug_level};
        delete $args{-debug_level};
    }
    else {
        $self->{debug_level} = 1;
    }

    # To skip cleanup, default : no, cleanup
    if ( defined $args{-skip_cleanup} ) {
        $self->{skip_cleanup} =
          ( $args{-skip_cleanup} =~ /1|yes|true|on/i ) ? 1 : 0;
        delete $args{-skip_cleanup};
    }
    else {
        $self->{skip_cleanup} = 0;
    }

    # Wafl iron validation, default : no, run wafl iron
    if ( defined $args{-skip_wafl_check} ) {
        $self->{skip_wafl_check} =
          ( $args{-skip_wafl_check} =~ /1|yes|true|on/i ) ? 1 : 0;
        delete $args{-skip_wafl_check};
    }
    else {
        $self->{skip_wafl_check} = 0;
    }

    # Additional info can be log as setup param
    if ( defined $args{-param_log_info} ) {
        $self->{param_log_info} = $args{-param_log_info};
        delete $args{-param_log_info};
    }
    else {
        $self->{param_log_info} = "";
    }

    #
    # form the string for necessary license -
    #
    my @license_str = qw(fcp iscsi);
    if ( $self->{san_xfun_env} =~ /snapmirror/i ) {
        push( @license_str, "snapmirror" );
        if ( $self->{sm_sync_type} eq "sync" ) {
            push( @license_str, "snapmirror_sync" );
        }
    }

    #
    # check user requested to enable asis on the volume
    #
    if ( defined( $args{-enable_asis} ) && length( $args{-enable_asis} ) ) {

    #
    # A-SIS is supported on the following platforms: 2000, 3000, 
    # 6000 series and R200. # On 2000, 3000, 6000 you need 
    # to add license # nearstore_option first:  SAXCNCG, 
    # after this add # a_sis license :  OSFYRVH
    #
    #
    # TODO : need to check if the vol type is flex, since 
    # asis works only for flex vol.
    # if this is hanndle by San:setup we can remove this
    #  if( $args{-enable_asis} =~ /on|true|yes|1/i 
    #                       && $vol_type =~ /flex/i) {
    #
        if ( $args{-enable_asis} =~ /on|true|yes|1/i ) {
            push( @license_str, "nearstore_option" );
            push( @license_str, "a_sis" );
        }
    }

    #
    # now add the user request license keys
    #
    if ( defined $args{-license_check} ) {
        push( @license_str, @{ $args{-license_check} } );
        delete $args{-license_check};
    }
    @{ $self->{ontap_license} } = @license_str;

    #
    # Get the filer list.
    #
    my @filers_array = split( /:/, $args{-filers} );

    #
    # get OnTap version and check/set necessary license on the filer
    #
    my $verion_found = 0;
    foreach my $filer (@filers_array) {

        #
        # Connect to filer
        #
        my $fh = host($filer);
        my $fc = connect("#rsh-socket");

        #
        # Check for necessary license
        #
        $self->sanlog( -msg => "Checking for necessary "
              . "license(@license_str) on '$filer'.........." );
        if ( !Admin::License::license( $fc, \@license_str ) ) {
            $self->_set_err(
                "Could not license services @license_str for '$filer': $@");
            return $SanUtil::FALSE;
        }

        #
        # detect onTap version from the filer,
        # filer_ontap_version_get is in SanUtil.pm
        # ASSUMPTION: both filers in cluster has same version
        #
        if ( $verion_found == 0 ) {
            $self->sanlog( -msg => "Detecting OnTap version.........." );
            $self->{ontap_version} =
              $self->filer_ontap_version_get( -filer => $filer );
            if ( !$self->{ontap_version} ) {
                $self->_set_err( "OnTap version detection failed on "
                      . "'$filer': $self->errmsg()" );
                return $SanUtil::FALSE;
            }
            $verion_found = 1;
            $self->sanlog(
                -msg => "OnTap version detected: $self->{ontap_version}" );
        }
        disconnect($fc);
    }

    #
    # check if we have OnTap version
    #
    if ( $verion_found == 0 ) {
        $self->_set_err("Unable to detect the OnTap version");
        return $SanUtil::FALSE;
    }

    #
    # Set the Debug Level for the San Object
    #
    $self->set_debug_level( $self->{debug_level} );

    #
    # get the client name; we always wrok with first client
    #
    $self->{client_name} = $args{-clients};

    #
    # If a comma separated list of clients are passed, 
    # accept the first one and strip of the rest.
    #
    $self->{client_name} =~ s/\,.*//;

    #
    # Start the memory Monitor
    #
    $self->{-memory_check} = $SanUtil::FALSE;
    if ( defined $args{-memory_check} ) {
        if ( $args{-memory_check} =~ /1|yes|true|on/i ) {
            Leak::mem_monitor();
            $self->{-memory_check} = $SanUtil::TRUE;
        }
        delete $args{-memory_check};
    }

    #
    # create san setup based on environment
    #
    if ( $self->{san_xfun_env} =~ /default/i ) {
        $self->San::setup(%args);
        if ( $self->error() ) {
            $self->_set_err( $self->errmsg() );
            return $SanUtil::FALSE;
        }
    }
    elsif ( $self->{san_xfun_env} =~ /snapmirror/i ) {

        #
        # setup snapmirror environment
        #
        $self->_setup_sanxfun_snapmirror(%args);
        if ( $self->error() ) {
            $self->_set_err( $self->errmsg() );
            return $SanUtil::FALSE;
        }
    }
    elsif ( $self->{san_xfun_env} =~ /vfiler|snapvault/i ) {
        $self->_set_err("$self->{san_xfun_env} not yet supported..........");
        return $SanUtil::FALSE;
    }
    else {
        $self->_set_err( "$self->{san_xfun_env} not supported"
              . " in SanXfun Environment.........." );
        return $SanUtil::FALSE;
    }

    #
    # ASSUMPTION: san setup should complete by this time
    #

    #
    # store filers array and the filer attributes hash
    #
    @{ $self->{'filer_array'} } = @filers_array;
    $self->{'filer_attr_hash'} = {};

    #
    # get the san resources created by san::setup,
    # this(_get_san_resources) function call _set_err() if any error
    #

    if ( !$self->_get_san_resources() ) {
        return $SanUtil::FALSE;
    }

    #
    # set/get necessary attributes on San, SanFiler object
    #
    foreach my $filer_name ( @{ $self->{'filer_array'} } ) {
        my $filer_obj = $self->get_filer($filer_name);
        if ( $self->error() ) {
            $self->_set_err( $self->errmsg() );
            return $SanUtil::FALSE;
        }

        #
        # set OnTap vesion and zapi log for CmdDriver for filer
        #
        $filer_obj->filer_version( $self->{ontap_version} );
        $filer_obj->interface( $self->{interface} );
        if ( $self->{interface} =~ /zapi/i ) {
            $filer_obj->cmd_obj()->log_output(1);
        }

        #
        # store the lun attributes
        #

        my %lun_attr = $self->get_filer_lun_list_info(
            -filer_name => $filer_name,
            -interface  => "zapi"
        );
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err($error_msg);
            return $SanUtil::FALSE;
        }
        $self->{'filer_attr_hash'}->{$filer_name}->{lun_attr} = \%lun_attr;
    }

    #
    # Check for unique serials
    #
    if (!$self->verify_lun_unique_serial())
    {
        return $SanUtil::FALSE;
    }

    # TODO
    #
    # Need to run IO to all luns ??? (if snapmirror run io to all source lun).
    #

    my $param_msg =
        "\n=================================================================\n"
      . "           SanXfun: '$self->{san_xfun_env}' Environment Parameters: \n"
      . "=================================================================\n";
    if ( $self->{san_xfun_env} =~ /snapmirror/i ) {
        $param_msg .=
            "----- Snap mirror related parameters -------------\n"
          . "   SM SYNC TYPE                  = $self->{sm_sync_type}\n"
          . "   SM_REPLICATION_TYPE           = $self->{sm_replicate_type}\n"
          . "   MIRROR_X_TYPE                 = $self->{sm_xfer_type}\n"
          . "   SM Source Path                = $self->{src_snapmirror_path}\n"
          . "   SM Destination Path           = $self->{dst_snapmirror_path}\n";
    }
    $param_msg .=
        "----- Controller realated parameters -------------\n"
      . "   Controller(s)                 = @{$self->{'filer_array'}}\n"
      . "   License to check              = @{$self->{ontap_license}}\n"
      .

# #         "   Vol options to enable         = @{$self->{vol_options}}\n"       .
      "   OnTap version detected        = $self->{ontap_version}\n"
      . "----- Client realated parameters -------------\n"
      . "   Client(s) Name                = $self->{client_name}\n"
      . "   Client Protocol               = $self->{protocol}\n"
      . "----- Testing realated parameters -------------\n"
      . "   Interface                     = $self->{interface}\n"
      . "---------- Other parameters ---------------------\n"
      . "   debug_level                   = $self->{debug_level}\n"
      . "   skip_cleanup                  = $self->{skip_cleanup}\n"
      . "   skip_wafl_check               = $self->{skip_wafl_check}\n"
      . $self->{param_log_info};
    $self->sanlog( -msg => $param_msg );

    #
    # log the params and other values, if debug level is 3 or more
    #
    my $filer_attr_hash = $self->{'filer_attr_hash'};

    my $dumpOut = Dumper($filer_attr_hash);
    $self->sanlog( -msg => "Output Hash is:\n$dumpOut", -msg_level => 3 );

    return $SanUtil::TRUE;
}
##############################################################################
###   Function:                  - To cleanup what created by this setup
###   Parameters:                - $cleanup_reused (0|1|noaggr)
###   Returns:
###      on success              - 1
###      on failure              - 0 and sanxfun object errmsg() will 
###                                contains error message
##############################################################################
sub cleanup() {
    my ( $self, $cleanup_reused ) = @_;

    $self->sanlog( -msg => "SanXfun cleanup" );
    $self->_clear_err();

    my $error_msg = "";

    if ( !defined($cleanup_reused) || !length($cleanup_reused) ) {
        $cleanup_reused = 0;
    }

    #
    # break SnapMirror relationship, if it is established by setup
    #
    $self->_cleanup_sanxfun_snapmirror();

    #
    # do wafl check (wafl iron on vol/aggr)
    #
    if ( !$self->{skip_wafl_check} ) {
        if ( $self->_check_wafliron() == $SanUtil::FALSE ) {
            $error_msg .= "WAFL Iron Failed: \n" . $self->errmsg();
        }
    }

    #
    # call san::cleanup
    #
    $self->San::cleanup( $self->{skip_cleanup}, $cleanup_reused );
    if ( $self->error() ) {
        $error_msg .= "San::cleanup failed:\nReason: " . $self->errmsg();
    }

    #
    # Stop the memory Monitor
    #
    if ( $self->{-memory_check} == $SanUtil::TRUE ) {
        Leak::mem_check();
    }

    if ( $error_msg ne "" ) {
        $self->_set_err($error_msg);
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
}
##############################################################################
###   Function:                  - get_sanxfun_attributes
###                                get the attributtes of SanXfun objects
###   Parameters:                - None
###   Returns:
###      on success              - refrence to hash
###      on failure              - 0 and sanxfun object -> errmsg() will contains error message
##############################################################################
sub get_sanxfun_attributes() {
    my ($self) = @_;
    $self->_clear_err();

    my $sanXfun_attr    = {};
    my $filers_res_hash = $self->{'filer_attr_hash'};

    $sanXfun_attr->{'san_xfun_env'} = $self->{'san_xfun_env'};
    $sanXfun_attr->{'filers'}       = $self->{'filer_array'};    # refernce to array
    $sanXfun_attr->{'protocol'}     = $self->{'protocol'};
    $sanXfun_attr->{'client_name'}  = $self->{'client_name'};

    foreach my $filer_name ( @{ $sanXfun_attr->{'filers'} } ) {

        # get the filer object of all controllers
        my $filer_obj = $self->get_filer($filer_name);
        if ( $self->error() ) {
            $self->_set_err( $self->errmsg() );
            return $SanUtil::FALSE;
        }
        @{ $sanXfun_attr->{$filer_name}->{'aggr_names'} } =
          @{ $filers_res_hash->{$filer_name}->{aggr_name} };
        @{ $sanXfun_attr->{$filer_name}->{'igroup_names'} } =
          @{ $filers_res_hash->{$filer_name}->{igroup} };
        @{ $sanXfun_attr->{$filer_name}->{'vol_names'} } =
          @{ $filers_res_hash->{$filer_name}->{vol_name} };

        foreach
          my $vol_name ( @{ $sanXfun_attr->{$filer_name}->{'vol_names'} } )
        {
            my $vol_obj = $filer_obj->get_vol($vol_name);
            if ( $filer_obj->error() ) {
                $self->_set_err( $filer_obj->errmsg() );
                return $SanUtil::FALSE;
            }
            @{ $sanXfun_attr->{$filer_name}->{$vol_name}->{'lun_names'} } =
              @{ $filers_res_hash->{$filer_name}->{$vol_name}->{luns} };
        }
    }
    if ( $self->{san_xfun_env} =~ /snapmirror/i ) {
        $sanXfun_attr->{'snapmirror_location'} =
          $self->{sm_xfer_type};    # same filer "local" or two filers "remote"
        $sanXfun_attr->{'src_filer_name'}      = $self->{src_filer_name};
        $sanXfun_attr->{'dst_filer_name'}      = $self->{dst_filer_name};
        $sanXfun_attr->{'src_vol_name'}        = $self->{src_vol_name};
        $sanXfun_attr->{'dst_vol_name'}        = $self->{dst_vol_name};
        $sanXfun_attr->{'src_snapmirror_path'} = $self->{src_snapmirror_path};
        $sanXfun_attr->{'dst_snapmirror_path'} = $self->{dst_snapmirror_path};
    }
    return $sanXfun_attr;
}
##############################################################################
###   Function:                  - get_sanxfun_environment
###                                get the SanXfun environment
###   Parameters:                - None
###   Returns:
###      on success              - string - (default|snapmirror|snapvault|vfiler)
###      on failure              - None
##############################################################################
sub get_sanxfun_environment() {
   my ($self) = @_;

   return $self->{'san_xfun_env'};
}
##############################################################################
###   Function:                  - get_controllers_names
###                                get the name(s) of controllers
###   Parameters:                - None
###   Returns:
###      on success              - Array of controllers name
###      on failure              - None
##############################################################################
sub get_controllers_names() {
   my ($self) = @_;

   return @{$self->{'filer_array'}};
}
##############################################################################
###   Function:                  - get_san_client
###                                get the SanClient Object
###   Parameters:                - None
###   Returns:
###      on success              - refrence to SanClinet Object
###      on failure              - 0 and sanxfun object -> errmsg() will 
###                                contains error message
##############################################################################
sub get_san_client() {
   my ($self) = @_;

   $self->_clear_err();

   my $sanClient = $self->get_client($self->{'client_name'});
   if( $self->error){
      $self->_set_err( $self->errmsg() );
      return $SanUtil::FALSE;
   }
   return $sanClient;
}
##############################################################################
###   Function:            - verify_lun_unique_serial
###                        Verifies the lun serial numbers are unique.
###                        This function is useful after doing any lun creation
###                        operation like lun create, clone, SFSR etc.,, to
###                        verify lun serial numbers are unique
###                        across the filer(s).
###
###   Parameters:
###        -lun_attr_array - array of lun attributtes hash refernce
###
###   Returns:
###      on success        - TRUE, if duplicate serial number NOT found.
###      on failure        - 0 and sanxfun object -> errmsg() will
###                         contains error message
##############################################################################
sub verify_lun_unique_serial() {
    my ( $self, %args ) = @_;

    $self->_clear_err();
                         
    my @lun_attr_array;
    
    foreach my $filer_name ( @{ $self->{'filer_array'} } ) {
        my $filer_obj = $self->get_filer($filer_name);
        if ( $self->error() ) {
            $self->_set_err( $self->errmsg() );
            return $SanUtil::FALSE;
        }
        my %lun_attr = $self->get_filer_lun_list_info(
            -filer_name => $filer_name,
            -interface  => "zapi"
        );
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err($error_msg);
            return $SanUtil::FALSE;
        }
        #
        # Store the lun attr. in local variable to verify later...
        #
        push( @lun_attr_array, \%lun_attr);
    }

    my @lun_serial_num = ();
    foreach my $lun_attr (@lun_attr_array) {
        for my $key ( keys %{$lun_attr} ) {
            my $lun_serial = $lun_attr->{$key}->{'serial-number'};
            $self->sanlog(
                -msg => "Unique Serial Verification: " .
                            "Lun: $key, Serial Number: $lun_serial",
                -msg_level => 3
            );
            if ( !grep { /^$lun_serial/ } @lun_serial_num ) {
                push( @lun_serial_num, $lun_serial );
            }
            else {
                $self->_set_err(
                    "Duplicate serial number($lun_serial) found. Lun: $key");
                return $SanUtil::FALSE;
            }
        }
    }
    return $SanUtil::TRUE;
}

##############################################################################
###   Function:                  - verify_lun_attributes_are_same
###                                verifies the lun attributes are same,
###   Parameters:
###      -org_lun_attr_hash_ref  - hash reference to orginal lun attributes
###      -cur_lun_attr_hash_ref  - hash reference to current lun attributes
###      -not_to_verify_attr     - array to not to verify list, Must match 
###                                the Keys value
###   Returns:
###      on success              - 1
###      on failure              - 0 and sanxfun object errmsg() 
###                                will contains error message
##############################################################################
sub verify_lun_attributes_are_same() {
    my ( $self, %args ) = @_;

    $self->_clear_err();

    #
    # check for existance of required arguments: org_lun_attr_hash_ref
    #
    if (   !defined( $args{-org_lun_attr_hash_ref} )
        || !length( $args{-org_lun_attr_hash_ref} ) )
    {
        $self->_set_err("Missing required argument '-org_lun_attr_hash_ref'");
        return $SanUtil::FALSE;
    }
    my $org_lun_attr = $args{-org_lun_attr_hash_ref};

    # check for existance of required arguments: cur_lun_attr_hash_ref
    #
    if (   !defined( $args{-cur_lun_attr_hash_ref} )
        || !length( $args{-cur_lun_attr_hash_ref} ) )
    {
        $self->_set_err("Missing required argument '-cur_lun_attr_hash_ref'");
        return $SanUtil::FALSE;
    }
    my $cur_lun_attr = $args{-cur_lun_attr_hash_ref};

    my @not_to_verify_attr = $args{-not_to_verify_attr};

    my @err_str = ("Lun attributes does not match");
    my $err_cnt = 0;

    for my $lun_path ( keys %$org_lun_attr ) {
        push( @err_str, "Lun: $lun_path" );
        my $lun_err_flag = 0;
        foreach my $key ( keys %{ $org_lun_attr->{$lun_path} } ) {
            if ( !grep { /^$key/ } @not_to_verify_attr ) {
                if ( $org_lun_attr->{$lun_path}->{$key} ne
                    $cur_lun_attr->{$lun_path}->{$key} )
                {
                    $lun_err_flag = 1;
                    $err_cnt++;
                    push( @err_str, "\t$key => Org.: " .
                          "'$org_lun_attr->{$lun_path}->{$key}' <-> " .
                          "Cur.: '$cur_lun_attr->{$lun_path}->{$key}'");
                }
            }
        }
        if ( $lun_err_flag == 0 ) {
            pop(@err_str);
        }
    }
    if ($err_cnt) {
        my $text = join( "\n", @err_str );
        $self->_set_err($text);
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
}

##############################################################################
###   Function:                  - get_filer_lun_list_info
###                                To get the lun attributes for the 
###                                given controller
###   Parameters:
###      -filer_name            - name of the controller
###      -interface             - the method to get lun attributes, 
###                               zapi or cli, default cli
###   Returns:
###      on success              - hash, if succesful
###      on failure              - 0 and sanxfun object errmsg() 
###                                will contains error message
##############################################################################
sub get_filer_lun_list_info() {
    my ( $self, %args ) = @_;

    $self->_clear_err();

    #
    # check for existance of required arguments: filer_name
    #
    unless ( defined $args{-filer_name} and length $args{-filer_name} ) {
        $self->_set_err("Missing required argument '-filer_name'");
        return $SanUtil::FALSE;
    }

    my $filer_name = $args{-filer_name};
    my $interface = ( $args{-interface} =~ /zapi/i ) ? "zapi" : "cli";

    my $san_filer_obj = $self->get_filer($filer_name);
    if ( $self->error() ) {
        my $error_msg = $self->errmsg();
        $self->_set_err(
            "Failed to get san filer object for $filer_name: $error_msg");
        return $SanUtil::FALSE;
    }

    my $cmd_obj = $san_filer_obj->cmd_obj();
    my $resultsHash_ref;
    my $info_hash;
    my $entry_hash;
    my $lun_hash = {};    # create empty hash

    #
    # Create the list of arguments for the CmdsDriver call.
    # Note that -verbose is really for the cli and should just
    # be ignored for the zapi
    #
    my %cmd_args = (
        "-cmd"           => "lun-list-info",
        "-check_result"  => 1,
        "-cmd_interface" => $interface,
        "-check_output"  => 0,
        "-verbose"       => "",
    );

    $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

    #
    # Check for CmdsDriver errors (not command failure)
    #
    if ( $cmd_obj->error() ) {
        my $errmsg = join "\n", $cmd_obj->errmsg();
        $self->_set_err("Unable to list lun info: $errmsg");
        return {};
    }

    $self->log_cmd_results($resultsHash_ref);

    #
    # Since we checked the result, lets see if our check was OK
    #
    unless ( $resultsHash_ref->{result_check} ) {
        $self->_set_err("Unable to get lun info: result check failed");
        return {};
    }

    #
    # Get the output
    #
    my $output = $resultsHash_ref->{output};

    #
    # If we get here, we have results back from the filer.  If it's a zapi
    # we want to make the XML a hash and look at the "luns" entry -
    #   if there are no luns, there will be no lun-info entry
    #   if there is one lun, lun-info will be a hash
    #   if there are >1 luns, lun-info will be an array
    #
    # If it's cli, we want to parse the data line by line and fill in the
    # fields of the entry_hash matching the field names from the zapi
    #
    if ( $interface eq "zapi" ) {
        my $output_hash = $self->xml2hash($output);

        #
        # if there are no luns, there will be no lun-info entry
        # return an empty hash
        #
        unless ( defined $output_hash->{"luns"}->{"lun-info"} ) {
            return ();
        }

        #
        # if there is one lun, lun-info will be a hash
        # if there are >1 luns, lun-info will be an array
        #
        if ( ref( $output_hash->{"luns"}->{"lun-info"} ) eq "HASH" ) {
            $entry_hash = $output_hash->{"luns"}->{"lun-info"};
            $entry_hash->{name} = basename( $entry_hash->{path} );
            $lun_hash->{ $entry_hash->{path} } = $entry_hash;
        }
        else {
            foreach $entry_hash ( @{ $output_hash->{'luns'}->{'lun-info'} } ) {
                $entry_hash->{name} = basename( $entry_hash->{path} );
                $lun_hash->{ $entry_hash->{path} } = $entry_hash;
            }
        }
    }
    else {

        #
        # CLI case
        # We are expecting output that looks like this:
        # /vol/blocks_fvt/cloned_lun 8m (8388608) (r/w, offline, mapped, shared)
        #       Serial#: hofBSJMIuVXV
        #       Backed by: /vol/blocks_fvt/.snapshot/clonesnap/mappedonline_lun
        #       Share: none
        #       Space Reservation: enabled
        #       Multiprotocol Type: solaris
        #
        my @output_array = split( /\n/, $output );
        $entry_hash = {};
        my $lun_path = "";
        foreach my $line (@output_array) {
            $line =~ s/^\s+//;    # remove leading whitespace
            $line =~ s/\s+$//;    # remove trailing whitespace

            #
            # If this line doesn't pertain to this volume, skip it
            #
            if (    ( $line !~ /^\/vol/ )
                and ( !defined( $lun_hash->{$lun_path} ) ) )
            {
                next;
            }

            #
            # This will be the first line in a new lun
            #
            if ( $line =~ /^\/vol/ ) {

                #
                # Regex below parses the first line shown above in the example.
                # Using that example:
                # $new_path = /vol/blocks_fvt/cloned_lun
                # $size = 8m, $access = r/w, $state = offline, mapped, shared
                #
                # my ($new_path,$size,$unit, $access,$state)=
                my ( $new_path, $size, $unit, $bytes, $access, $state ) =
                  $line =~ /^\s*(.*)\s+(\d+.?\d{0,})(\w)\s+\((.*)\)\s+\((\w+\/\w+),\s+(.*)\)/;

                $new_path =~ s/\s+$//;    # remove trailing whitespace
                $self->sanlog(
                    -msg => "lun_list_info(): $line\n"
                      . "new path is $new_path,\n"
                      . "size     is $size$unit,\n"
                      . "bytes    is $bytes,\n"
                      . "access   is $access,\n"
                      . "state    is $state\n",
                    -msg_level => 3
                );

                $lun_path = $new_path;

                #
                # Check the $state for mapped, online
                # 1 for mapped, otherwise 0.
                #
                my $mapped   = ( $state  =~ /mapped/ ) ? "true"  : "false";
                my $online   = ( $state  =~ /online/ ) ? "true"  : "false";
                my $readonly = ( $access =~ /w/ )      ? "false" : "true";

                #
                # Load the lun hash with what we know
                #
                $lun_hash->{$lun_path}->{path}        = $new_path;
                $lun_hash->{$lun_path}->{name}        = basename($new_path);
                $lun_hash->{$lun_path}->{size}        = $bytes;
                $lun_hash->{$lun_path}->{online}      = $online;
                $lun_hash->{$lun_path}->{mapped}      = $mapped;
                $lun_hash->{$lun_path}->{"read-only"} = $readonly;
            }
            elsif ( $line =~ /^Serial/ ) {
                $line =~ s/Serial#:\s+//;    # strip off the prefix
                $lun_hash->{$lun_path}->{"serial-number"} = $line;
            }
            elsif ( $line =~ /^Share:/ ) {
                $line =~ s/Share:\s+//;
                $lun_hash->{$lun_path}->{"share-state"} = $line;
            }
            elsif ( $line =~ /^Space/ ) {

                #
                # If the line contains "enabled" then use true
                #
                $lun_hash->{$lun_path}->{"is-space-reservation-enabled"} =
                  ( $line =~ /enabled/ ) ? "true" : "false";
            }
            elsif ( $line =~ /^Multiprotocol/ ) {
                $line =~ s/Multiprotocol\s+Type:\s+//;
                $lun_hash->{$lun_path}->{"multiprotocol-type"} = $line;
            }
            elsif ( $line =~ /^Backed/ ) {
                $line =~ s/Backed\s+by:\s+//;
                $lun_hash->{$lun_path}->{"backing-snapshot"} = $line;
            }
            elsif ( $line =~ /^Maps/ ) {
                $line =~ s/Maps:\s+//;
                $lun_hash->{$lun_path}->{maps} = $line;
            }
        }    # end foreach line
    }    # end if CLI

    #
    # get lun map info for all the luns,
    # just igroup name and the lind ID,
    # if we need other info, we will fill it later
    #
    for my $lun_path ( keys %{$lun_hash} ) {

        #
        # if lun is not mapped skip to next lun
        #
        if ( $lun_hash->{$lun_path}->{'mapped'} ne 'true' ) {
            next;
        }
        my %cmd_args = (
            "-cmd"           => "lun-map-list-info",
            "-check_result"  => 1,
            "-cmd_interface" => "zapi",
            "-check_output"  => 0,
            "-path"          => $lun_path,
            "-verbose"       => "",
        );
        my $resultsHash_ref = $cmd_obj->execute_cmd(%cmd_args);

        #
        # Check for CmdsDriver errors (not command failure)
        #
        if ( $cmd_obj->error() ) {
            my $errmsg = join "\n", $cmd_obj->errmsg();
            $self->_set_err("Unable to list lun map info: $errmsg");
            return {};
        }

        $self->log_cmd_results($resultsHash_ref);

        #
        # Since we checked the result, lets see if our check was OK
        #
        unless ( $resultsHash_ref->{result_check} ) {
            $self->_set_err("Unable to get lun info: result check failed");
            return {};
        }

        #
        # Get the output
        #
        my $output      = $resultsHash_ref->{output};
        my $output_hash = $self->xml2hash($output);

        #
        # if there are no luns, there will be no lun-info entry
        # return an empty hash
        #
        unless (
            defined $output_hash->{"initiator-groups"}
            ->{"initiator-group-info"} )
        {
            next;
        }

     #
     # if there is one map info for the lun, initiator-group-info will be a hash
     # if there are > 1 map infos, initiator-group-info will be an array
     #
        if (
            ref( $output_hash->{"initiator-groups"}->{"initiator-group-info"} )
            eq "HASH" )
        {
            $entry_hash =
              $output_hash->{"initiator-groups"}->{"initiator-group-info"};
            my $igroup_name = $entry_hash->{"initiator-group-name"};
            $lun_hash->{$lun_path}->{$igroup_name}->{'initiator-group-name'} =
              $entry_hash->{'initiator-group-name'};
            $lun_hash->{$lun_path}->{$igroup_name}->{'lun-id'} =
              $entry_hash->{'lun-id'};

            #
            # will be filled all other info if we need. - TODO
            #
            # merge the existing lun attributes with this one
            # my %lun_attr_merge = (%{$lun_hash->{$lun_path}}, %{$entry_hash});
            # $lun_hash->{$lun_path} = \%lun_attr_merge;
        }
        else {
            foreach $entry_hash (
                @{
                    $output_hash->{"initiator-groups"}->{"initiator-group-info"}
                }
              )
            {
                my $igroup_name = $entry_hash->{"initiator-group-name"};
                foreach my $key ( keys %$entry_hash ) {
                    $lun_hash->{$lun_path}->{$igroup_name}
                      ->{'initiator-group-name'} =
                      $entry_hash->{'initiator-group-name'};
                    $lun_hash->{$lun_path}->{$igroup_name}->{'lun-id'} =
                      $entry_hash->{'lun-id'};

                }

       #
       # will be filled all other info if we need. - TODO
       #
       # form it with igroup name:
       # foreach my $key (keys %$entry_hash) {
       #   $lun_hash->{$lun_path}->{$igroup_name}->{$key} = $entry_hash->{$key};
       # }
            }
        }
    }
    return ( %{$lun_hash} );
}    # end lun_list_info()

##############################################################################
###   Function:         - get_sanxfun_lun_objs
###                     To get all the lun(s) obj, created/reuse by San.
###                     if environment is snapmirror, it will return only luns
###                     from the soruce volume.
###   Parameters:       - None
###   Returns:
###      on success     - array of lun objects.
###      on failure     - 0, and the text contains error message is set
##############################################################################
sub get_sanxfun_lun_objs() {
    my ($self) = @_;

    $self->_clear_err();

    my @lun_objs = ();

    my $filers_res_hash = $self->{'filer_attr_hash'};
    if ( $self->{'san_xfun_env'} =~ /snapmirror/i ) {

        # get lun objects from source volume lun only
        my $filer_name = $self->{'src_filer_name'};
        my $filer_obj  = $self->get_filer($filer_name);
        if ( $self->error() ) {
            $self->_set_err( $self->errmsg() );
            return $SanUtil::FALSE;
        }
        my $vol_name = $self->{'src_vol_name'};
        my $vol_obj  = $filer_obj->get_vol($vol_name);
        if ( $filer_obj->error() ) {
            $self->_set_err( $filer_obj->errmsg() );
            return $SanUtil::FALSE;
        }
        foreach my $lun_name (
            @{ $filers_res_hash->{$filer_name}->{$vol_name}->{luns} } )
        {
            my $lun_obj = $vol_obj->get_lun($lun_name);
            if ( $vol_obj->error() ) {
                $self->_set_err( $vol_obj->errmsg() );
                return $SanUtil::FALSE;
            }
            push( @lun_objs, $lun_obj );
        }
    }
    else {

        #
        # return all the luns. first with first filer, first vol
        # and its lun(s), then second filer...
        #
        foreach my $filer_name ( @{ $self->{'filer_array'} } ) {

            # get the filer object of all controllers
            my $filer_obj = $self->get_filer($filer_name);
            if ( $self->error() ) {
                $self->_set_err( $self->errmsg() );
                return $SanUtil::FALSE;
            }
            foreach
              my $vol_name ( @{ $filers_res_hash->{$filer_name}->{vol_name} } )
            {
                my $vol_obj = $filer_obj->get_vol($vol_name);
                if ( $filer_obj->error() ) {
                    $self->_set_err( $filer_obj->errmsg() );
                    return $SanUtil::FALSE;
                }
                foreach my $lun_name (
                    @{ $filers_res_hash->{$filer_name}->{$vol_name}->{luns} } )
                {
                    my $lun_obj = $vol_obj->get_lun($lun_name);
                    if ( $vol_obj->error() ) {
                        $self->_set_err( $vol_obj->errmsg() );
                        return $SanUtil::FALSE;
                    }
                    push( @lun_objs, $lun_obj );
                }
            }
        }
    }
    return @lun_objs;
}

##############################################################################
###   Function:         - get_controller_lun_objs
###                     To get all the lun(s) obj from the specified contoller,
###                     created/reuse by San.
###   Parameters:       - -filer_name : Name of the controller 
###                       to get the lun objs.
###   Returns:
###      on success     - array of lun objects.
###      on failure     - 0, and the text contains error message is set
##############################################################################
sub get_controller_lun_objs() {
   my ( $self, %args ) = @_;

   $self->_clear_err();

   #
   # check for existance of required arguments: filer_name
   #
   unless ( defined $args{-filer_name} and length $args{-filer_name} ) {
       $self->_set_err("Missing required argument '-filer_name'");
       return $SanUtil::FALSE;
   }
   my $filer_name;
   #
   # make sure we have the controller requested in the list
   #
   
   foreach my  $controller_name ( @{ $self->{'filer_array'} } ) {
      if($args{-filer_name} eq $controller_name){
         $filer_name = $controller_name;
         last;
      }
   }
   if( !defined($filer_name) and !length()){
      $self->_set_err("Requested controller not exist: $args{-filer_name}");
      return $SanUtil::FALSE;
   }
   my @lun_objs = ();
   
   my $filers_res_hash = $self->{'filer_attr_hash'};
   #
   # return all the luns from the requested controller
   # first with first vol its lun(s), then second volume...
   #
   # get the filer object of all controllers
   my $filer_obj = $self->get_filer($filer_name);
   if ( $self->error() ) {
       $self->_set_err( $self->errmsg() );
       return $SanUtil::FALSE;
   }
   foreach my $vol_name ( @{ $filers_res_hash->{$filer_name}->{vol_name} } )
   {
       my $vol_obj = $filer_obj->get_vol($vol_name);
       if ( $filer_obj->error() ) {
           $self->_set_err( $filer_obj->errmsg() );
           return $SanUtil::FALSE;
       }
       foreach my $lun_name (
           @{ $filers_res_hash->{$filer_name}->{$vol_name}->{luns} } )
       {
           my $lun_obj = $vol_obj->get_lun($lun_name);
           if ( $vol_obj->error() ) {
               $self->_set_err( $vol_obj->errmsg() );
               return $SanUtil::FALSE;
           }
           push( @lun_objs, $lun_obj );
       }
   }
    return @lun_objs;
}
##############################################################################
#	                   P R I V A T E   M E T H O D S                     #
##############################################################################

##############################################################################
###   Function:   - _get_san_resources
###                  get the san resources attributes, 
###                   created or reused by San::setup
###
###   Parameters: - None
###   Returns:    - 1 for success, or 0, this function will set the error
###   TBD         - need to check for qsm volume, need to add hash for vol/lun
##############################################################################
sub _get_san_resources() {
    my ($self) = @_;

    my $filers_res_hash = $self->{'filer_attr_hash'};

    foreach my $filer_name ( @{ $self->{'filer_array'} } ) {

        #
        # get resource : aggr_name
        #
        my @filer_resources = $self->get_resources(
            -type => "filer",
            -name => $filer_name,
            -cmd  => "aggr_create"
        );
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err(
                "Failed to get san resources from for aggr_create: $error_msg");
            return $SanUtil::FALSE;
        }
        foreach my $r_hash (@filer_resources) {
            if ( $r_hash->{-cmd} =~ /aggr_create/i ) {
                push(
                    @{ $filers_res_hash->{$filer_name}->{aggr_name} },
                    $r_hash->{-cmd_args}
                );
            }
        }    # end foreach

        #
        # get the resources : vol
        #
        @filer_resources = $self->get_resources(
            -type => "filer",
            -name => $filer_name,
            -cmd  => "vol_create"
        );
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err(
                "Failed to get san resources from for vol_create: $error_msg");
            return $SanUtil::FALSE;
        }
        foreach my $r_hash (@filer_resources) {
            if ( $r_hash->{-cmd} =~ /vol_create/i ) {
                my $san_vol_name = $r_hash->{-cmd_args};
                $san_vol_name =~ s/^\/vol\///;    # remove /vol/
                $san_vol_name =~ s/\/$//;         # remove trailing "/"
                push(
                    @{ $filers_res_hash->{$filer_name}->{vol_name} },
                    $san_vol_name
                );
            }
        }    # end foreach
        #
        # get the resources : igroup
        #
        @filer_resources = $self->get_resources(
            -type => "filer",
            -name => $filer_name,
            -cmd  => "igroup_create"
        );
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err(
                "Failed to get san resources from for igroup_create: $error_msg"
            );
            return $SanUtil::FALSE;
        }
        foreach my $r_hash (@filer_resources) {
            if ( $r_hash->{-cmd} =~ /igroup_create/i ) {
                push(
                    @{ $filers_res_hash->{$filer_name}->{igroup} },
                    $r_hash->{-cmd_args}
                );
            }
        }    # end foreach

        #
        # get the resources : lun
        #
        @filer_resources = $self->get_resources(
            -type => "filer",
            -name => $filer_name,
            -cmd  => "lun_create"
        );
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err(
                "Failed to get san resources from for igroup_create: $error_msg"
            );
            return $SanUtil::FALSE;
        }
        foreach my $r_hash (@filer_resources) {
            if ( $r_hash->{-cmd} =~ /lun_create/i ) {

                #
                # remove volume just take the lun name
                # -cmd_args' => '/vol/io_lvm1 rtpqa-sun280r-2_lun1'
                #
                my ( $vol_name, $lun_name ) =
                  split( " ", $r_hash->{-cmd_args} );
                $vol_name =~ s/^\/vol\///;    # remove /vol/
                $vol_name =~ s/\/$//;         # remove trailing "/"
                push(
                    @{ $filers_res_hash->{$filer_name}->{$vol_name}->{luns} },
                    $lun_name
                );
            }
        }    # end foreach
    }    # end of foreach filers
    return $SanUtil::TRUE;
}
##############################################################################
###   Function:                  - _check_wafliron
###                                Run wafl iron on aggr if it is flex vol.
###                                if trad do wafl iron on that volume
###   Parameters:                - None
###   Returns:
###      on success              - 1
###      on failure              - 0, and the text contains error message
##############################################################################
sub _check_wafliron() {
    my ($self) = @_;

    my $filers_res_hash = $self->{'filer_attr_hash'};
    my @wi_subtest_obj  = ();
    foreach my $filer_name ( @{ $self->{'filer_array'} } ) {

        #
        # Get filer Object
        #
        my $san_filer_obj = $self->get_filer($filer_name);
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err(
                "Failed to get san filer object for $filer_name: $error_msg");
            return $SanUtil::FALSE;
        }

        #
        # get filer connection
        #
        my $fc = $san_filer_obj->connection();

        #
        # get the aggr_name, if aggr_name is not present then
        # it must be trad vol, so run on trad vol
        #
        my @aggrs      = @{ $filers_res_hash->{$filer_name}->{aggr_name} };
        my $no_of_aggr = @aggrs;
        if ( $no_of_aggr >= 1 ) {
            foreach my $aggr_name (@aggrs) {
                $self->sanlog(
                    -msg => "Starting wafl iron on filer($filer_name) "
                      . "using aggr($aggr_name).........." );

                my $sio_subtest_id = "wafliron_start_". 
                                     $filer_name . "_" . $aggr_name;

                my ($sec,$min,$hour,$mday,$mon,
                      $year,$wday,$yday,$isdst)=localtime(time);
                $sio_subtest_id=sprintf("%s_%04d%02d%02d_%02d%02d%02d",
                   $sio_subtest_id,$year+1900,$mon+1,$mday,$hour,$min,$sec);
                my $sub_test_obj = subtest(
                    \&_wafliron_subtest,
                    '-bg',
                    '-runid',
                    $sio_subtest_id,
                    '-onexit',
                    sub { },
                    '--',
                    $fc,
                    $aggr_name,
                    $filer_name
                );
                push( @wi_subtest_obj, $sub_test_obj );
            }
        }
        else {

            #
            # it should be trad vol, so run on trad vol
            #
            my @vol_list = @{ $filers_res_hash->{$filer_name}->{vol_name} };
            foreach my $vol_name (@vol_list) {
                $self->sanlog(
                    -msg => "Starting wafl iron on filer($filer_name) "
                      . " using trad vol($vol_name).........." );

                my $sio_subtest_id = "wafliron_start_". 
                                     $filer_name . "_" . $vol_name;
                my ($sec,$min,$hour,$mday,$mon,
                      $year,$wday,$yday,$isdst)=localtime(time);
                $sio_subtest_id=sprintf("%s_%04d%02d%02d_%02d%02d%02d",
                   $sio_subtest_id,$year+1900,$mon+1,$mday,$hour,$min,$sec);
                my $sub_test_obj = subtest(
                    \&_wafliron_subtest,
                    '-bg',
                    '-runid',
                    $sio_subtest_id,
                    '-onexit',
                    sub { },
                    '--',
                    $fc,
                    $vol_name,
                    $filer_name
                );
                push( @wi_subtest_obj, $sub_test_obj );
            }
        }
    }
    $self->sanlog( -msg => "Waiting for WAFL iron to complete..." );
    my $wafl_iron_err = "";
    foreach my $st_obj (@wi_subtest_obj) {
        if ( $st_obj->is_alive() == 1 ) {
            $st_obj->wait_finish();
        }
        my @results = $st_obj->results();
        if ( "@results" !~ /SanXfun WaflIron Passed/ ) {

            #
            # get the error from subtest run
            #
            foreach my $line (@results) {
                if ( $line =~
                    qr/wafl Iron failed on filer\(\s+\) using aggr\/trad vol\(\s+\):/
                  )
                {
                    if ( $wafl_iron_err eq "" ) {
                        $wafl_iron_err = $line;
                    }
                    else {
                        $wafl_iron_err .= "\n" . $line;
                    }
                    last;
                }
            }
        }
    }
    if ( $wafl_iron_err ne "" ) {
        $self->_set_err($wafl_iron_err);
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
}
##############################################################################
###   Function:                  - _wafliron_subtest
###                                subtest routine to start wafl iron
###   Parameters:                - filer connection, aggr/volume name,
###                                filer_name
###   Returns:
###      on success              - 1
###      on failure              - 0
###
###      for wack use the following, lib avaliable @ /p4/test/lib/wack.pm
###
###                              use wack;
###                              use boot;
###                              my $h = host($FILER);
###                              my $res = wack($h,args=&gt;"$FILER_VOL");
###                              tstlog('-fatal',$@) if (!$res);
###               tstlog('-fail',"vol $FILER_VOL is not supposed to be dirty")
###                                      if ($res ne WACK_CLEAN);
###                              boot($h) or tstlog('-script',$@);
##############################################################################
sub _wafliron_subtest() {
    my ( $fc, $aggr_vol_name, $filer_name ) = @_;

    my ( $ret_code, $text ) = Wafliron::wafliron_start( $fc, $aggr_vol_name );
    if ($ret_code) {
        logresult( "FAIL",
                "wafl Iron failed on filer($filer_name) "
              . "using aggr/trad vol($aggr_vol_name): $text" );
        return $SanUtil::FALSE;
    }
    logresult( "PASS", "SanXfun WaflIron Passed" );
    return $SanUtil::TRUE;
}

#############################################################################
#                             POD Documentation
#############################################################################

=pod

=head1 Package Name

SanXfun.pm

=head1 Author

Muthukumaran Govindarajan (muthug@netapp.com)

=head1 Description

SanXfun.pm provides an API interface to a SanXfun with the main intent to create
environment for testing cross functional use cases. It is wrapper to San Object,and inhertis
its methods, along with SanSnapMirror.pm and SanXfunUitls.pm. All the methods for,m those module 
can be acceess directly fron SanXfun Object.


=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<San.pm>
B<SanXfunIoUtil.pm>
B<SanSnapMirror.pm>


=head1 Attributes

=head2 B<Refer San.pm POD for all attributes>

=head1 Public Methods

Also refer SanXfunIoUtil.pm and SanSnapMirror.pm for addtional Public methods, which can 
be access throguh SanXfun object.

=head2 B<new()>

See San::new() for more detials.

=over 4

=item Inputs

=item * log (defaults to \*ntest::NTEST_STDOUT)

File descriptor for a log file.

=item Outputs

Returns a SanXfun object, actually it is blessed by San Object.

=item Examples

   #
   # Create a SanXfun object
   #
   my $san_xfun_obj = new SanXfun(\*ntest::NTEST_STDOUT);
   if ($san_xfun_obj->error) {
      my $err_mag = $san_xfun_obj->errmsg();
      $Test->nlog("Failed to instantiate SanXfun object:\nReason: $err_mag");
      return $TCD::FAIL;
   }

=back

=head2 B<setup(%args)>

Create necessary resources on filers and clients using San library.  It takes same parameters as San::setup.
In addition it takes following parameters. Refer San Pod for San::setup parameters.

=over 4

=item Inputs

=item * I<all inputs that San::setup takes>

=item * I<-san_xfun_env> <default | snapmirror | vfiler | snapvault> (default: default)

NOTE: Currently "default" and "snapmirror" is implemented.

Determine the environment to create for cross functional testing.
Valid value for this argument (default|snapmirror|vfiler|snapvault).
If invalid value passed it will default to "default" environment.
"default" environment is nothing but using San::setup.

If SanXfun enviroment is SnapMirror, 

* Setup Notes
   * Creates SnapMirror relationship with one source volume to one Destination Volume.
      * Destination volume should be equal or grater size.
      * For Trad volume, we create volume with no of disk, if Destination become less, script will fail
   
   * Local Transfer ( Source & destination volume on same filer)
      * create the volume on source with the given name, and append "_dst" for the destination volume name
      * SYNC mirror is not supported on local filer
   
   * Setup creates source & destination volume separately and add those into one San Object.
   
   * Setup create up to volume (both source & destination), and lun(s) only on the source volume
      * After source volume completed, it will do SnapMirror Initialize, so that all the luns will be 
        replicated to destination volume as read-only-luns, and setup will map those destination volume 
        luns to same host(igroup) as source.
      
   * Need "filer_vol_list" to setup the controllers.
      
* Qtree SnapMirror
   * Creates the qtree on the source volume and lun(s) on the qtree
   * On SnapMirror initialize it will replicate to destination volume
      * On destination Qtree should not exist, if setup try to re-use it and if happens qtree exist on destination, 
        SnapMirror initialize will fail, clear the qtree on destination volume(or remove the volume itself) and re-run the script.

* SYNC SnapMirror      
      * Need two filers, local transfer does not supported
      * Need non cluster testbed, HA pair is not supported, if testbed is cluster, script will "disable cf"
      * Only Volume(VSM) replication, not Qtrees(QSM, quota trees)


=item * I<-sm_sync_type> <sync | async> (default: async)                    

If the san xfun environment is set it to "snapmirror", this parameter is needed to 
determine the transfer type of snapmirror.

=item * I<-sm_replicate_type> <vsm | qsm> (default: vsm)      

If the san xfun environment is set it to "snapmirror", this parameter is needed to 
determine the snapmirror type. vsm for volume based snapmirror, qsm for Qtree based snapmirror.

=item * I<-license_check> <reference to perl array> (no default) 

Perl array reference to validate the license necessary for testing.
This is check for existing license, if not it will license the controllers.


=item * I<-interface> <cli | zapi> (default: cli)

Interface to test the main test cases and also set the san object CmdDriver attribute to this interface.

=item * I<-skip_cleanup> <yes|1 or no|0> (default: no)

To skip the cleanup created by San::setup. Default is set to 'no", means it will cleanup when SanXfun::cleanup called.

=item * I<-skip_wafl_check> <yes|1 or no|0> (default: 0)

To skip the Wafl Iron check on Aggr/volume. Default is set to 'no", means when SanXfun::cleanup called 
it will check for wafliron at end of the test before calling San::cleanup.

=item * I<-param_log_info> <string> (no default)

To log addtional information during the setup


=item Outputs

Returns a scalar value of 1 if successful. If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      my @license_key_check = qw(snaprestore);
              
      #
      # log other test related parameters
      #
      my $param_log_info =     "----- Lun clone resize related parameters -------------\n"   .
                              "   Max lun Clone depth           = $max_lun_clone_depth\n";
      
      #
      # Instantiate SanXfun Object
      #
      
      $SanXfun_obj = new SanXfun(\*ntest::NTEST_STDOUT);
      if ($SanXfun_obj->error) {
         my $err_mag = $SanXfun_obj->errmsg();
         $Test->nlog("Failed to instantiate SanXfun object:\nReason: $err_mag");
         return $TCD::FAIL;
      }
      #
      # Call setup to create environment to testing
      #
      $SanXfun_obj->setup(
            -san_xfun_env        => "snapmirror",
            -filers              => $filers,
            -clients             => $client_name,
            -protocol            => $protocol,
            -aggr                => $aggr_name,
            -num_disks           => $no_of_disks,
            -filer_vol_list      => $filer_vol_list,
            -interface           => $interface,
            -debug_level         => $debug_level,
            -sm_sync_type        => "async",
            -sm_replicate_type   => "qsm",
            -license_check       => \@license_key_check,
            -skip_cleanup        => $skip_cleanup,
            -skip_wafl_check     => $skip_wafl_check,
            -param_log_info      => $param_log_info
      );
      if ($SanXfun_obj->error) {
         $Test->nlog($SanXfun_obj->errmsg());
         return $TCD::FAIL;
      }

=back

=head2 B<cleanup(<cleanup_reused>)>

Before calling San::cleanup, it will cleanup the snap mirror realtionship, if it is sanpmirror environment.
And run WaflIron on Aggr/volume based on -skip_wafl_check setting throuh setup() function. Then it calls the
San::cleanup() to destroy created resources on filers and clients by San::setup.  This includes aggrs,
vols, igroups, luns on the filer and devices, vg, lv, fs, file on the
client.

=over 4

=item Inputs

=item * I<cleanup_reused> <0|1|noaggr> (default to 0)

This parameter is used by San::cleanup.
If this is set, the cleanup script will cleanup reused resources as well
as resources that have just been created.  If noaggr is used we will
cleanup everything bug aggrs.

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      $cleanup_reused = $Test->get_param("CLEANUP_REUSED", 0);
      $SanXfun_obj->cleanup($cleanup_reused);
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("cleanup failed:\nReason: $error_msg");
         return $TCD::FAIL;
      }

=back

=head2 B<get_sanxfun_attributes()>

To get the SanXfun resources. Return refrence to perl tpe hash.
Refer the example below or the function itself for the hash key names

=over 4

=item Inputs

None

=item Outputs

Returns a refernce to perl hash if successful. If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      my $sanXfun_attr = $SanXfun_obj->get_sanxfun_attributes();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun attribute failed:\nReason: $error_msg");
         return 0;
      }
      print "SanXfun Environment = " . $sanXfun_attr->{'san_xfun_env'};
      # filers list, note it is refernce to array.
      my @filers = @{ $sanXfun_attr->{'filers'};
      print "Filer(s) used = @filers";
      print "Protocol used = " . $sanXfun_attr->{'protocol'} ;
      
      foreach my $filer_name ( @{ $sanXfun_attr->{'filers'} } ) {
         # to get the filer object of all controllers
         my $filer_obj = $sanXfun_attr->{$filer_name}->{'san_filer_obj'};
         my @aggr_names = @{$sanXfun_attr->{$filer_name}->{'aggr_names'}};
         my @igroup_names = @{$sanXfun_attr->{$filer_name}->{'igroup_names'}};
         foreach my $vol_name (@{$sanXfun_attr->{$filer_name}->{'vol_names'}}) {
            my $vol_obj = $sanXfun_attr->{$filer_name}->{$vol_name}->{'vol_obj'};
            my @lun_names = @{$sanXfun_attr->{$filer_name}->{$vol_name}->{'lun_names'}};
         }
      }
      if( $sanXfun_attr->{'san_xfun_env'} =~ /snapmirror/i) {
         my $src_sm_path = $sanXfun_attr->{'src_snapmirror_path'};
         my $dst_sm_path = $sanXfun_attr->{'dst_snapmirror_path'};
      }

=back

=head2 B<get_sanxfun_environment()>

To get the SanXfun environment

=over 4

=item Inputs

None

=item Outputs

Returns a string of the current environment

=item Examples

      my $sanxfun_env = $SanXfun_obj->get_sanxfun_environment();
      print $sanxfun_env;

=back

=head2 B<get_controllers_names()>

To get the name(s) of controllers

=over 4

=item Inputs

None

=item Outputs

Returns an array of controllers name

=item Examples

      my @filer_name = $SanXfun_obj->get_controllers_names();
      print @filer_name;

=back

=head2 B<get_san_client()>

To get the SanClient Object

=over 4

=item Inputs

None

=item Outputs

Returns a Sanclinet Object if successful. If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      my $sanXfun_attr = $SanXfun_obj->get_san_client();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("Unable to get Sanclinet object:\nReason: $error_msg");
         return 0;
      }

=back

=head2 B<verify_lun_unique_serial()>

To verify the lun serial numbers are uniquie across the contorllers.

=over 4

=item Inputs

None

=item Outputs

Returns a scalar value of 1 if no duplicate serial number found. If duplicate serial number found, the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # Check for unique serials
      #
      if ( !$SanXfun_obj->verify_lun_unique_serial() ) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog($error_msg);
         return 0;
      }

=back

=head2 B<verify_lun_attributes_are_same(%args)>

To verify the lun atrributtes matches with orginal and current, optionaly you can pass the key names which can be ignored.
NOTE: if current lun list has more attributte than orginal, it will skip those. May be feature enahancement.

=over 4

=item Inputs

=item * I<-org_lun_attr_hash_ref>

Refernce to orginal lun attributes.
See below example for more details

=item * I<-cur_lun_attr_hash_ref>

Refernce to current lun attributes
See below example for more details

=item * I<-not_to_verify_attr> (optional)

Array of key names (lun attributes name, refer function get_filer_lun_list_info for key names).
See below example for more details

=item Outputs

Returns a scalar value of 1 if all lun attributes matches. If lun attrbuttes does not match, the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # get the SanXfun resources
      #
      my $sanXfun_attr = $SanXfun_obj->get_sanxfun_attributes();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun attribute failed:\nReason: $error_msg");
         return 0;
      }
      #
      # store the lun attributes for each controller, during setup or testing
      #
      
      my @org_lun_attr_ref;
      foreach my $filer_name ( @{ $sanXfun_attr->{'filers'} } ) {
         my %lun_attr = $SanXfun_obj->get_filer_lun_list_info(
                                 -filer_name => $filer_name);
         if ( $SanXfun_obj->error() ) {
            my $error_msg = $SanXfun_obj->errmsg();
            $Test->nlog($error_msg);
            return 0;
         }
         push(@org_lun_attr_ref, \%lun_attr);
      }
      
      #
      # do testing
      #
      ...
      ...
      ...
      ...
      #
      # get the current lun attributes for each controller
      #
      my @cur_lun_attr_ref;
      foreach my $filer_name ( @{ $sanXfun_attr->{'filers'} } ) {
         my %lun_attr = $SanXfun_obj->get_filer_lun_list_info(
                                 -filer_name => $filer_name);
         if ( $SanXfun_obj->error() ) {
            my $error_msg = $SanXfun_obj->errmsg();
            $Test->nlog($error_msg);
            return 0;
         }
         push(@cur_lun_attr_ref, \%lun_attr);
      }
      #
      # prepare array for not to verify
      #
      my @not_to_verify = ("read-only", "serial-number");
      my $lun_ref_count = @org_lun_attr_ref;
      
      for (my $i = 0; $i < $lun_ref_count; $i++) {
         my $org_lun_ref = $org_lun_attr_ref[$i];
         my $cur_lun_ref = $cur_lun_attr_ref[$i];
         my %lun_attr = $SanXfun_obj->verify_lun_attributes_are_same(
                                       -org_lun_attr_hash_ref => $org_lun_ref, 
                                       -cur_lun_attr_hash_ref => $cur_lun_ref,
                                       -not_to_verify_attr    => @not_to_verify); 
         if ( $SanXfun_obj->error() ) {
            my $error_msg = $SanXfun_obj->errmsg();
            $Test->nlog($error_msg);
            return 0;
         }
      }

=back

=head2 B<get_filer_lun_list_info(%args)>

To get the all the lun attributtes for the given controller.
Once it gets the info from controller, it will form the hash and return it.

=over 4

=item Inputs

=item * I<-filer_name>

Name of the controller to return the lun attributtes, It will return all the luns availble on the controller.
See below example for more details

=item * I<-interface> (cli | zapi) (default : zapi)  (optional)

Interface to use for fetching the lun attributes. Reference to current lun attributes
See below example for more details

=item Outputs

Returns a hash value with all lun attributes on the given controller.

NOTE: If controller does not have any lun, it will return empty hash.
If it fails to execute command, the error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # get the SanXfun resources
      #
      my $sanXfun_attr = $SanXfun_obj->get_sanxfun_attributes();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun attribute failed:\nReason: $error_msg");
         return 0;
      }
      #
      # get the lun attributes for each controller
      #
      my @org_lun_attr_ref;
      foreach my $filer_name ( @{ $sanXfun_attr->{'filers'} } ) {
         my %lun_attr = $SanXfun_obj->get_filer_lun_list_info(
                                 -filer_name => $filer_name);
         if ( $SanXfun_obj->error() ) {
            my $error_msg = $SanXfun_obj->errmsg();
            $Test->nlog($error_msg);
            return 0;
         }
         push(@org_lun_attr_ref, \%lun_attr);
      }

=back


=head2 B<get_sanxfun_lun_objs()>

To get all the lun(s) obj, created/reuse by San.
If environment is snapmirror, it will return only luns source volume lun objects.

=over 4

=item Inputs

None

=item Outputs

Returns a array of lun objects.

=item Examples

      #
      # get the SanXfun resources
      #
      my @lunObjs = $SanXfun_obj->get_sanxfun_lun_objs();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun Lun Objects failed:\nReason: $error_msg");
         return 0;
      }
      #
      # get the first lun from the list and do a lun clone
      #
      my $lun_obj = $lunObjs[0];
      #
      # create lun clone from parent lun
      #
      my $src_path     = $lun_obj->get_path();
      my $src_lun_name = $lun_obj->get_name();
      my $clone_path   = $src_path;
      my $lc_lun_name  = $src_lun_name . "_clone_" . $i;
      $clone_path =~ s/$src_lun_name/$lc_lun_name/;
      
      # create lun clone
      my $lun_clone_obj = $lun_obj->clone_create(
         -snap_name => $snapname,
         -path      => $clone_path
      );
      if ( $lun_obj->error ) {
         my $msg = $lun_obj->errmsg();
         $ret_err = "Failed to create the lun $clone_path:\nErorr: " . $msg;
         return 0;
      }

=back


=head2 B<get_controller_lun_objs()>

To get all the lun(s) obj from the specified contoller,
created/reuse by San.

=over 4

=item * I<-filer_name>

Name of the controller to return the lun objects, It will return all the luns availble on the controller.
See below example for more details

=item Outputs

Returns a array of lun objects.

=item Examples

      #
      # get the Lun objects from the specified controller
      #
      my @lunObjs = $SanXfun_obj->get_controller_lun_objs(-filer_name => "rtpqa-fas960-1");
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun Lun Objects failed:\nReason: $error_msg");
         return 0;
      }
      #
      # get the first lun from the list and do a lun clone
      #
      my $lun_obj = $lunObjs[0];
      #
      # create lun clone from parent lun
      #
      my $src_path     = $lun_obj->get_path();
      my $src_lun_name = $lun_obj->get_name();
      my $clone_path   = $src_path;
      my $lc_lun_name  = $src_lun_name . "_clone_" . $i;
      $clone_path =~ s/$src_lun_name/$lc_lun_name/;
      
      # create lun clone
      my $lun_clone_obj = $lun_obj->clone_create(
         -snap_name => $snapname,
         -path      => $clone_path
      );
      if ( $lun_obj->error ) {
         my $msg = $lun_obj->errmsg();
         $ret_err = "Failed to create the lun $clone_path:\nErorr: " . $msg;
         return 0;
      }

=back

=head2 B<write_data_to_lun(%args)>

To write known data on given one lun. It uses Sio with fillonce options.
Block Io size can be specified and should follow sio syntax.

For more details, refer the fucntion in San::SanXfun::SanXfunIoUtil.pm

=over 4

=item Inputs

=item * I<-lun_objs>

refernce to array of Lun Object(s), to write. Lun Should already Mapped and accessible from client.

=item * I<-wait_flag> <1|0> (default: 1)

1 for wait, 0 for not to wait and return the SanIo Object.

=item * I<-io_block_size> (default: 4096)

Block Size for read/write. SIO syntax need to follow.

=item * I<-data_pattern> (default: "G")

Character to fill the lun.

=item * I<-sio_num_threads> (default: 10)

Number of sio threads to create, by default it will create 10 threads

=item * I<-sio_subtest_id> (default: "sio_write_verify_data")

PID name - SanIo start Sio using Subtest, The name will be used as PID name and useful to search in logs.
It will append the date with the given sutest id. For example it will append date with following format
sio_write_verify_data_20090410_171352

=item Outputs

Returns a scalar value of 1 if wait flag is set and SanIO succeeds.
If wait flag is not set and SanIO started successfully, then it returns the SanIO Object.
If SanIo fails to start/write, the error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # get the SanXfun resources
      #
      my @lunObjs = $SanXfun_obj->get_sanxfun_lun_objs();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun Lun Objects failed:\nReason: $error_msg");
         return 0;
      }
      #
      # get the first lun from the list and write on that lun
      #
      my $lun_obj = $lunObjs[0];
      my $sanIo_obj = $SanXfun_obj->write_data_to_lun(
        -lun_objs       => [$lunObj],                  # for single lun
        -io_block_size  => $io_block_size,
        -data_pattern   => $data_pattern,
        -wait_flag      => 1,
        -sio_subtest_id => "fill_org_lun_write_" . $lun_name
     );
     if ( $SanXfun_obj->error() ) {
         my $errmsg = $SanXfun_obj->errmsg();
         $self->_set_err($errmsg);
         return 0;
     }

=back

=head2 B<verify_data_on_lun(%args)>

To read and verify with known data on given one lun. It uses Sio reads approximately 20 mints.
Block Io size can be specified and should follow sio syntax.

For more details, refer the fucntion in San::SanXfun::SanXfunIoUtil.pm

=over 4

=item Inputs

=item * I<-lun_objs>

refernce to array of Lun Object(s), to read/verify. Lun Should already Mapped and accessible from client.

=item * I<-wait_flag> <1|0> (default: 1)

1 for wait, 0 for not to wait and return the SanIo Object.

=item * I<-io_block_size> (default: 4096)

Block Size for read/write. SIO syntax need to follow.

=item * I<-data_pattern> (default: "G")

Character to fill the lun.

=item * I<-sio_num_threads> (default: 10)

Number of sio threads to create, by default it will create 10 threads

=item * I<-sio_read_runtime> (default: 600)

Number of seconds for reading the datapattern. This nuber is used when reading data from lun(s)
Defaulted to 600 secs. i.e. 10 mints.
                             
=item * I<-sio_subtest_id> (default: "sio_read_verify_data")

PID name - SanIo start Sio using Subtest, The name will be used as PID name and useful to search in logs.
It will append the date with the given sutest id. For example it will append date with following format
sio_read_verify_data_20090410_171352

=item Outputs

Returns a scalar value of 1 if wait flag is set and SanIO succeeds.
If wait flag is not set and SanIO started successfully, then it returns the SanIO Object.
If SanIo fails to start/write, the error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # get the SanXfun resources
      #
      my @lunObjs = $SanXfun_obj->get_sanxfun_lun_objs();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun Lun Objects failed:\nReason: $error_msg");
         return 0;
      }
      #
      # get the first lun from the list
      #
      my $lun_obj = $lunObjs[0];
      my $sanIo_obj = $SanXfun_obj->verify_data_on_lun(
        -lun_obj        => [$lunObj],
        -io_block_size  => $io_block_size,
        -data_pattern   => $data_pattern,
        -wait_flag      => 1,
        -sio_subtest_id => "fill_org_lun_read_" . $lun_name
     );
     if ( $SanXfun_obj->error() ) {
         my $errmsg = $SanXfun_obj->errmsg();
         $self->_set_err($errmsg);
         return 0;
     }

=back

=head2 B<get_lun_devices_info(%args)>

To get the device inforamtion with client lun size(client user space) 

For more details, refer the fucntion in San::SanXfun::SanXfunIoUtil.pm

=over 4

=item Inputs

=item * I<-lun_objs>

refernce to array of Lun Object(s). Lun Should already Mapped and accessible from client.

=item Outputs

Returns a refernce to perl hash if successful. If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # get the SanXfun resources
      #
      my @lunObjs = $SanXfun_obj->get_sanxfun_lun_objs();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun Lun Objects failed:\nReason: $error_msg");
         return 0;
      }
      #
      # get the first lun from the list
      #
      my $lun_obj = $lunObjs[0];
      # get the client device name and size for the lun
      my %lun_device_info = $SanXfun_obj->get_lun_devices_info( -lun_objs => $lun_objs );
      if ($SanXfun_obj->error()) {
          my $errmsg = $SanXfun_obj->errmsg();
          $SanXfun_obj->_set_err( "get_lun_devices_info failed: $errmsg");
          return 0;
      }
      my @device_array;
      my $filesize;
      #
      # Assume all luns are same size
      #
      for my $device_name (keys %lun_device_info) {
         $SanXfun_obj->sanlog(-msg => "get_lun_devices_info:\n" .
                  "Lun Path:         " . $lun_device_info{$device_name}{'lun_path'} . "\n" .
                  "Filer Name:       " . $lun_device_info{$device_name}{'filer_name'} . "\n" .
                  "Volume Name:      " . $lun_device_info{$device_name}{'vol_name'} . "\n" .
                  "Device Name:      " . $lun_device_info{$device_name}{'device_name'} . "\n" . 
                  "Full Device Name: " . $lun_device_info{$device_name}{'full_device_name'} . "\n" .
                  "Lun Size:         " . $lun_device_info{$device_name}{'size'} ."\n");
         push (@device_array, $lun_device_info{$device_name}{'full_device_name'} );
         $filesize = $lun_device_info{$device_name}{'size'};
      }

=back

=head2 B<update_device_geometry(%args)>

To update the Solaris client device geometry information. This will be helpful if lun size changed after
the lun is presented to Solaris host. Lun resize wont be recognized by host until it updates the geometry information.
This function will update the Solaris geometry and next rescan will get the lun resize information.
see burt:344816 for more information.

For more details, refer the fucntion in San::SanXfun::SanXfunIoUtil.pm

=over 4

=item Inputs

=item * I<-lun_objs>

refernce to array of Lun Object(s). Lun Should already Mapped and accessible from client.

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # get the SanXfun resources
      #
      my @lunObjs = $SanXfun_obj->get_sanxfun_lun_objs();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun Lun Objects failed:\nReason: $error_msg");
         return 0;
      }
      ...
      ...
      ...
      #
      # resize the lun 
      #
      ...
      ...
      ...
      #
      # Solaris is not detecting the new size, method to update partition infomation
      #
      $SanXfun_obj->update_device_geometry( -lun_objs => \@$lunObjs );
      if ($SanXfun_obj->error()) {
          my $errmsg = $SanXfun_obj->errmsg();
          $ret_err =  "Failed to update the lun clone devices: $errmsg";
          goto _test_lun_clone_resize_cleanup;
      }

=back

=head2 B<rescan_client_scsi_device()>

Rescan the scsi device of the the client setup during the San::Setup.
Useful when you map the lun after Setup, it will re-discover those lun(s).

For more details, refer the fucntion in San::SanXfun::SanXfunIoUtil.pm

=over 4

=item Inputs

None

=item Outputs

Returns a scalar value of 1 if successful. If an error occured the
error flag is set, errmsg is set to the failure reason, and 0 is
passed back as the return value.

=item Examples

      #
      # get the SanXfun resources
      #
      my @lunObjs = $SanXfun_obj->get_sanxfun_lun_objs();
      if ($SanXfun_obj->error) {
         my $error_msg = $SanXfun_obj->errmsg();
         $Test->nlog("get SanXfun Lun Objects failed:\nReason: $error_msg");
         return 0;
      }
      #
      # get the first lun from the list and do a lun clone
      #
      my $lun_obj = $lunObjs[0];
      #
      # create lun clone from parent lun
      #
      my $src_path     = $lun_obj->get_path();
      my $src_lun_name = $lun_obj->get_name();
      my $clone_path   = $src_path;
      my $lc_lun_name  = $src_lun_name . "_clone_" . $i;
      $clone_path =~ s/$src_lun_name/$lc_lun_name/;
      
      # create lun clone
      my $lun_clone_obj = $lun_obj->clone_create(
         -snap_name => $snapname,
         -path      => $clone_path
      );
      if ( $lun_obj->error ) {
         my $msg = $lun_obj->errmsg();
         $ret_err = "Failed to create the lun $clone_path:\nErorr: " . $msg;
         return 0;
      }
      #
      # map the cloned lun
      #
      $lun_clone_obj->map($igroup_name);
      if ( $lun_clone_obj->error ) {
         $ret_err = "Failed to map igroup " . $lun_clone_obj->errmsg();
         return 0;
      }
      #
      # rescan the scsi device for the newly created clone lun(s)
      #
      $SanXfun_obj->rescan_client_scsi_device();
      if ( $SanXfun_obj->error() ) {
        my $ret_err = $SanXfun_obj->errmsg();
         return 0;
      }

=back

=cut

1;

