# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/San/SanXfun/SanXfunIoUtil.pm#1 $
#
# Copyright (c) 2009 NetApp, Inc.
# All rights reserved.
#
# lib/San/SanXfun/SanXfunIoUtil.pm
##
## @summary Support Utility functions to test with SanXfun suite
##
## @description
##   This utility functions in addtion to SanXfun functions, it has
##   IO utility functions
##
## @author Muthukumaran Govindaraj (muthug@netapp.com)
## @status public
## @change 2008-10-16 muthu: Original template created
## @pod SanXfun.pm

# Class : SanXfunIoUtil

package SanXfunIoUtil;

use strict;

use Tharn;                    # for subtest, logcomment, timeout2time
use TestEnv::MinWin qw(:all); # minwin_compatible 
use File::Basename;


##############################################################################
###   Function:         - write_data_to_lun
###                       write to the one specfic lun with given data pattern,
###                       block size. Function will wait for write to complete
###                       if wait flag is set, else return the SanIO obeject.
###
###   Parameters:       - see pod for details in SanXfun.pm
###
###   Returns:
###      on success     - 1, if wait_flag is not set, it will returns
###                       SanIo Object
###
###      on failure     - 0 and sanxfun object errmsg() will
###                       contains error message
##############################################################################
sub write_data_to_lun() {
    my ( $self, %args ) = @_;

    $args{'-write_flag'} = 1;
    return $self->_write_verify_data(%args);
}
##############################################################################
###   Function:         - verify_data_on_lun
###                       read and verify from one specfic lun with given
###                       data pattern, block size. Function will wait for
###                       read to complete, if wait flag is set,
###                       else return the SanIO obeject.
###
###   Parameters:       - see pod for details in SanXfun.pm
###
###   Returns:
###      on success     - 1, if wait_flag is not set, it will returns
###                       SanIo Object
###
###      on failure     - 0 and sanxfun object errmsg() will
###                       contains error message
##############################################################################
sub verify_data_on_lun() {
    my ( $self, %args ) = @_;

    $args{'-write_flag'} = 0;
    return $self->_write_verify_data(%args);
}
##############################################################################
###   Function:         - rescan_client_scsi_device
###                     rescan the scsi device - useful for windows host
###                     if lun changes(like remap), or lun clone mapped to host
###
###   Parameters:       - none
###
###   Returns:
###      on success     - 1
###      on failure     - 0 and sanxfun object errmsg() will
###                       contains error message
##############################################################################
sub rescan_client_scsi_device() {
    my ( $self ) = @_;

    $self->_clear_err();

    my $clientObj = $self->get_client( $self->{client_name} );
    if ( $self->error() ) {
        my $errmsg = $self->errmsg();
        $self->_set_err(
            "Failed to get client($self->{client_name}) object: $errmsg");
        return $SanUtil::FALSE;
    }
    $self->sanlog( -msg => "Scanning the scsi device on client "
          . "'$self->{client_name}' in progress..." );

    #
    # Discover the scsi devices on the client
    #
    $clientObj->discover();
    if ( $clientObj->error() ) {
        my $error_msg = $clientObj->errmsg();
        $self->_set_err( "Failure occured during Scanning the scsi "
              . "device on client '$self->{client_name}': $error_msg" );
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
}
##############################################################################
###   Function:        - get_byte_size
###                    return the byte size of given lun size
###                    might not work for real numbers (1.5g, 1.2m like that)
###   Parameters:      - size - e.g. ( 100, 1000m, 1.2g, 1.0t)
###   Returns:
###      on success    - integer value of the given size in bytes
##############################################################################
sub get_byte_size() {
    my ( $self, $size ) = @_;

    my $multiplier = 1;

    if ( $size =~ /c$/ ) {
        $multiplier = 1;
    }
    elsif ( $size =~ /w$/ ) {
        $multiplier = 2;
    }
    elsif ( $size =~ /b$/ ) {
        $multiplier = 512;
    }
    elsif ( $size =~ /k$/ ) {
        $multiplier = 1024;
    }
    elsif ( $size =~ /m$/ ) {
        $multiplier = 1024 * 1024;
    }
    elsif ( $size =~ /g$/ ) {
        $multiplier = 1024 * 1024 * 1024;
    }
    elsif ( $size =~ /t$/ ) {
        $multiplier = 1024 * 1024 * 1024 * 1024;
    }

    $size =~ s/(\d+).*/$1/;
    return int( $size * $multiplier );
}

###############################################################################
# Method:    get_lun_devices_info
# Objective:  To get the device name for the given lun Object(s).
# Inputs:    -lun_objs - array refernce to Lun objects.
# Outputs:   FALSE(0) with error message, if error occured. or array of 
#            device name
###############################################################################
sub get_lun_devices_info()
{
   my ( $self, %args ) = @_;

   $self->_clear_err();

   if( $args{'-lun_objs'} !~ /ARRAY/){
      $self->_set_err("Need to pass ARRAY refernce of lun Objects");
      return $SanUtil::FALSE;
   }

   my @lunObjRef = @{$args{'-lun_objs'}};
   
   my $client_obj = $self->get_san_client();
   if( $self->error){
      return $SanUtil::FALSE;
   }

   #
   # Get the host name
   #
   my $client_name = $client_obj->hostname();
   if ($client_obj->error)
   {
       my $errmsg = $client_obj->errmsg();
       $self->_set_err("Call to hostname failed: ",
                       "$errmsg");
       return $SanUtil::FALSE;                                          
   }
   #
   # Get the host type
   #
   my $client_type = $client_obj->hosttype();
   if ($client_obj->error) {
       my $errmsg = $client_obj->errmsg();
       $self->_set_err("Call to hosttype failed: ",
                       "$errmsg");
       return $SanUtil::FALSE;                                          
   }
   $client_type =~ s/unix-solaris/solaris/; # strip off unix-
   
   #
   # Create device manager object
   #
   $self->sanlog(-msg => "SanXfun::get_lun_devices_info: ". 
                     "Creating '$client_type' device manager object....");
   my $devmgr_obj = $client_obj->create_devmgr(-parent=>$client_obj);
   if ($client_obj->error) {
      $self->_set_err("Failed to createDevMgr object: " .
                    $client_obj->errmsg());
      return $SanUtil::FALSE;
   }

   my %devarg = ();
   $devarg{-refresh} = 1;
   my %device_info = ();

   foreach my $lunObj (@lunObjRef) {
      if( ref($lunObj) !~ /Lun/){
         $self->_set_err("Is not Lun Object refernce");
         return $SanUtil::FALSE;
      }
      #
      # Get full path for matching for the San::get_devices()
      #
      my $lun_name_path = $lunObj->get_path();
      my $full_lun_path = $lun_name_path;
      if ( $lunObj->error() ) {
          my $errmsg = $lunObj->errmsg();
          $self->_set_err("Unable to get the lun path from "
                . "the lun object for reason $errmsg" );
          return $SanUtil::FALSE;
      }
      #
      # Stripe out the /vol/vol_name from the path, 
      # get <qtree_name>/lun name
      #
      $lun_name_path =~ s#^/vol/\w+/(.*)#$1#;
      # get the client device list for the lun
      my $filer_name = $lunObj->get_filername();
      if ( $lunObj->error() ) {
          my $errmsg = join "\n", $lunObj->errmsg();
          $self->_set_err("Unable to get the filer name from "
                . "the lun object for reason $errmsg" );
          return $SanUtil::FALSE;
      }

      # get the volume name associated with the lun object
      my $vol_name = $lunObj->get_volname();
      if ( $lunObj->error() ) {
          my $errmsg = join "\n", $lunObj->errmsg();
          $self->_set_err("Unable to get the volume name from "
                . "the lun object for reason $errmsg" );
          return $SanUtil::FALSE;
      }

      $devarg{-filer} = $filer_name;
      $devarg{-vol} = $vol_name;
      $devarg{-lun} = "^$lun_name_path\$"; # match exact lun name

      $self->sanlog(-msg => "fetching device name for filer: $filer_name, " .
              "Volume: $vol_name, Lun: $lun_name_path, client: $client_name");

      my @devices = $devmgr_obj->get_free_devices(%devarg);
      if ( $devmgr_obj->error() ) {
          my $errmsg = $devmgr_obj->errmsg();
          $self->_set_err("Get_devices failed for device($lun_name_path): $errmsg");
          return $SanUtil::FALSE;
      }
      if ( @devices <= 0 ) {
          $self->_set_err("Device (filer: $filer_name, Volume: $vol_name, Lun: $lun_name_path, client: $client_name) not found");
          return $SanUtil::FALSE;
      }
      #
      # Now that we have the devices, lets push them on our array.
      #
      foreach my $device (@devices) {
         if ($client_type =~ /linux/ or
             $client_type =~ /hpux/  or
             $client_type =~ /aix/ ) {
             #
             # remove the "/dev" if it's there.
             #
             if ($device =~ /^\/dev\//) {
                $device = basename($device);
             }
         }
         #
         # for windows and soloaris it works as it is
         #
         $self->sanlog(-msg => "Querying the lun size for the device '$device' in client '$client_type'"); 
         my $dev_obj = $devmgr_obj->get_device($device);
         if($devmgr_obj->error){
            my $errmsg = $devmgr_obj->errmsg();
            $self->_set_err("Unable to get device object for " .
                            "$device, " .
                            "$errmsg");
            return $SanUtil::FALSE;
         }
         my $device_lun_size = $dev_obj->get_size();
         my $device_lun_name = $dev_obj->get_lun();
         my $device_device_name = $dev_obj->get_name();

         my $device_full_name = $self->_get_full_device_string($device, $client_type);
         my $size;

         if ( $client_type =~ /solaris/ ) {
            $size = $dev_obj->get_partition_size(6);
         } elsif ( $client_type =~ /linux/ ) {
            if ($dev_obj->is_partitioned()) {
                $size = $dev_obj->get_partition_size(1);
            } else {
                $size = $dev_obj->get_size();
            }
         }else{
            $size = $dev_obj->get_partition_size();
         }
         if($dev_obj->error){
            my $errmsg = $dev_obj->errmsg();
            $self->_set_err("Unable to get partition size for " .
                            "$device: $errmsg");
            return $SanUtil::FALSE;
         }
         $device_info{$device}{'filer_name'}  = $devarg{-filer};
         $device_info{$device}{'vol_name'}  = $devarg{-vol};
         $device_info{$device}{'lun_path'}    = $full_lun_path;
         $device_info{$device}{'device_name'} = $device;
         $device_info{$device}{'full_device_name'} = $device_full_name;
         $device_info{$device}{'size'} = $size;

         $self->sanlog(-msg_level => 3, 
            -msg => "Lun Path:         " . $device_info{$device}{'lun_path'} . "\n" .
                    "Filer Name:       " . $device_info{$device}{'filer_name'} . "\n" .
                    "Volume Name:      " . $device_info{$device}{'vol_name'} . "\n" .
                    "Device Name:      " . $device_info{$device}{'device_name'} . "\n" . 
                    "Full Device Name: " . $device_info{$device}{'full_device_name'} . "\n" .
                    "Lun Size:         " . $device_info{$device}{'size'} ."\n".
                    "Device infos : LunSize: $device_lun_size, LunName: $device_lun_name, DeviceName: $device_device_name\n" .
                    "Client Type:      $client_type");
      } # end foreach device
      #
      # We only want to refresh (call sanlun one time)
      #
      $devarg{-refresh} = 0;
   }
   #
   # Return the devices found
   #
   if( keys (%device_info) ){
      return %device_info;
   }else{
      $self->_set_err("Unable to find the any lun device(s) info");
      return $SanUtil::FALSE;
   }
}

sub update_device_geometry() 
{
   my ( $self, %args ) = @_;

   $self->_clear_err();

   if( $args{'-lun_objs'} !~ /ARRAY/){
      $self->_set_err("Need to pass ARRAY refernce of lun Objects");
      return $SanUtil::FALSE;
   }

   my @lunObjRef = @{$args{'-lun_objs'}};
   
   my $client_obj = $self->get_san_client();
   if( $self->error){
      return $SanUtil::FALSE;
   }

   #
   # Get the host name
   #
   my $client_name = $client_obj->hostname();
   if ($client_obj->error)
   {
       my $errmsg = $client_obj->errmsg();
       $self->_set_err("Call to hostname failed: ",
                       "$errmsg");
       return $SanUtil::FALSE;                                          
   }
   #
   # Get the host type
   #
   my $client_type = $client_obj->hosttype();
   if ($client_obj->error) {
       my $errmsg = $client_obj->errmsg();
       $self->_set_err("Call to hosttype failed: ",
                       "$errmsg");
       return $SanUtil::FALSE;                                          
   }
   $client_type =~ s/unix-solaris/solaris/; # strip off unix-

   #
   # This function is only usefull for slorais, so 
   # if other os simpley return true.
   #
   if($client_type !~ /solaris/){
      return $SanUtil::TRUE;
   }

   #
   # Create device manager object
   #
   $self->sanlog(-msg => "SanXfun::update_device_geometry: ". 
                     "Creating '$client_type' device manager object....");
   my $devmgr_obj = $client_obj->create_devmgr(-parent=>$client_obj);
   if ($client_obj->error) {
      $self->_set_err("Failed to createDevMgr object: " .
                    $client_obj->errmsg());
      return $SanUtil::FALSE;
   }

   my %devarg = ();
   $devarg{-refresh} = 1;
   my %device_info = ();

   foreach my $lunObj (@lunObjRef) {
      if( ref($lunObj) !~ /Lun/){
         $self->_set_err("Is not Lun Object refernce");
         return $SanUtil::FALSE;
      }
      #
      # Get full path for matching for the San::get_devices()
      #
      my $lun_name_path = $lunObj->get_path();
      my $full_lun_path = $lun_name_path;
      if ( $lunObj->error() ) {
          my $errmsg = $lunObj->errmsg();
          $self->_set_err("Unable to get the lun path from "
                . "the lun object for reason $errmsg" );
          return $SanUtil::FALSE;
      }
      #
      # Stripe out the /vol/vol_name from the path, 
      # get <qtree_name>/lun name
      #
      $lun_name_path =~ s#^/vol/\w+/(.*)#$1#;
      # get the client device list for the lun
      my $filer_name = $lunObj->get_filername();
      if ( $lunObj->error() ) {
          my $errmsg = join "\n", $lunObj->errmsg();
          $self->_set_err("Unable to get the filer name from "
                . "the lun object for reason $errmsg" );
          return $SanUtil::FALSE;
      }

      # get the volume name associated with the lun object
      my $vol_name = $lunObj->get_volname();
      if ( $lunObj->error() ) {
          my $errmsg = join "\n", $lunObj->errmsg();
          $self->_set_err("Unable to get the volume name from "
                . "the lun object for reason $errmsg" );
          return $SanUtil::FALSE;
      }

      $devarg{-filer} = $filer_name;
      $devarg{-vol} = $vol_name;
      $devarg{-lun} = "^$lun_name_path\$"; # match exact lun name

      $self->sanlog(-msg => "refreshing the geomentry info for device on filer: $filer_name, " .
              "Volume: $vol_name, Lun: $lun_name_path, client: $client_name");

      my @devices = $devmgr_obj->get_free_devices(%devarg);
      if ( $devmgr_obj->error() ) {
          my $errmsg = $devmgr_obj->errmsg();
          $self->_set_err("Get_devices failed for device($lun_name_path): $errmsg");
          return $SanUtil::FALSE;
      }
      if ( @devices <= 0 ) {
          $self->_set_err("Device (filer: $filer_name, Volume: $vol_name, Lun: $lun_name_path, client: $client_name) not found");
          return $SanUtil::FALSE;
      }
      #
      # Now that we have the devices, lets push them on our array.
      #
      foreach my $device (@devices) {
         my $dev_obj = $devmgr_obj->get_device($device);
         if($devmgr_obj->error){
            my $errmsg = $devmgr_obj->errmsg();
            $self->_set_err("Unable to get device object for " .
                            "$device, " .
                            "$errmsg");
            return $SanUtil::FALSE;
         }

         #
         # Upate the geomentry
         #
         if($client_type =~ /solaris/){
            $dev_obj->refresh_geometry();
            if($dev_obj->error){
               my $errmsg = $dev_obj->errmsg();
               $self->_set_err("Unable to refresh the geomentry for " .
                               "$device: $errmsg");
               return $SanUtil::FALSE;
            }
          }
      } # end foreach device
      #
      # We only want to refresh (call sanlun one time)
      #
      $devarg{-refresh} = 0;
   }
   return $SanUtil::TRUE;
}

##############################################################################
#	                   P R I V A T E   M E T H O D S                     #
##############################################################################





###############################################################################
# Method:    _get_full_device_string (only for raw devices)
# Objective: Format a devices so user can run I/O immediately.
# Inputs:    device       - the device in question
#            host_type    - type of client (solaris|hpux|aix|...)
#
# Outputs:   updated device name
###############################################################################
sub _get_full_device_string() {
    my ($self, $device, $host_type) = @_;

    if ($host_type =~ /solaris/) {
        #
        # If we have a raw device, that's been formatted, lets
        # make sure that it writes to slice 6 instead of slice 2.
        #
        $device =~ s/s2/s6/g;
        unless ($device =~ /^\/dev\/rdsk\//) {
            $device = "/dev/rdsk/$device";
        }
    }
    elsif ($host_type =~ /linux/) {
        #
        # Add the "/dev" if it's not there.
        #
        unless ($device =~ /^\/dev\//) {
            $device = "/dev/$device";
        }
    }
    elsif ($host_type =~ /hpux/) {
        #
        # Add the "/dev/rdsk/" if it's not there.
        #
        unless ($device =~ /^\/dev\//) {
            $device = "/dev/rdsk/$device";
        }
    }
    elsif ($host_type =~ /aix/) {
        #
        # Add the "/dev/rdsk/" if it's not there.
        #
        unless ($device =~ /^\/dev\//) {
            $device = "/dev/r" . $device;
        }
    }
    return $device;
} 
##############################################################################
###   Function:           - wait_verify_sioObj
###                       wait for SanIO to complete and verify the run
###   Parameters:         - see function below
###   Returns:
###      on success       - 1, if no error found
###
###      on failure     - 0 and sanxfun object errmsg() will
###                       contains error message
##############################################################################
sub wait_verify_sioObj() {
   my ( $self, $SioObjarrayref ) = @_;
   
   $self->_clear_err();
   my $ret_err = "";

   if( ref($SioObjarrayref) !~ /ARRAY/){
      $self->_set_err("Need to pass array reference to subtest obeject(s)");
      return $SanUtil::FALSE;
   }

   $self->sanlog( -msg => "Waiting for one or more I/O to complete......" );

   foreach my $subtest (@{$SioObjarrayref}) {
      $subtest->wait_finish();
      #
      # check out the result return from the subtest
      #
      my %results = $self->get_sio_result(-sio_subtest_obj => $subtest);
      if($self->error()){ 
         $ret_err .= "subtest: '". $subtest->runid() .    
                                  "' failed:\nReason: " . $self->errmsg() . "\n";
      }
   }
   if($ret_err ne ""){ 
      $self->_set_err($ret_err);
      return $SanUtil::FALSE;
   }
   return $SanUtil::TRUE;
}

##############################################################################
###   Function:           - stop_sioObj
###                       To Stop the IO abruptly
###   Parameters:         - see function below
###   Returns:            - None
##############################################################################
sub stop_sioObj() {
   my ( $self, $SioObjarrayref ) = @_;
   
   $self->_clear_err();

   foreach my $subtest (@{$SioObjarrayref}) {
      if ($subtest->is_alive() == 1) {
         $subtest->kill(message=> ["Pass", "error_count\n0\nStoped by user request"]);
         $subtest->wait_finish();
      }
   }
}

##############################################################################
###   Function:        - _get_lun_size
###                    return the byte size of given lun size, based on ostype
###   Parameters:
###         -ostype    - valid os type name. Ex;. (solaris|windows|linux)
###         -lun_size  - string/integer ex.: ( 100, 1000m, 1.2g, 1.0t
###   Returns:
###      on success    - integer value of the given size in bytes
###      on failure    - 0
##############################################################################
sub _get_lun_size() {
    my ( $self, %args ) = @_;

    $self->_clear_err();

    my ( $ostype,  $lun_size );
    my ( $src_ref, $dst_ref );
    foreach ( keys %args ) {
        if (/^-?lun_size$/) {
            length( $lun_size = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-lun_size\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-?ostype$/) {
            length( $ostype = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-ostype\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        else {
            $self->_set_err("Invalid parameter specified: $_");
            return $SanUtil::FALSE;
        }
    }
    unless ( defined($lun_size) ) {
        $self->_set_err("Mandatory parameter '-lun_size' not specified");
        return 0;
    }
    unless ( defined($ostype) ) {
        $self->_set_err("Mandatory parameter '-ostype' not specified");
        return $SanUtil::FALSE;
    }

    my $actual_size;

    my $ONE_MEG     = 1048576;
    my $WIN_BLKSIZE = 512;

    if ( $ostype =~ m/solaris/i ) {
        # s6 has 1021MB (1M for prefix and 2M for suffix)
        $actual_size = $self->get_byte_size($lun_size) - ( 3 * $ONE_MEG );
    }
    elsif ( $ostype =~ m/windows/i ) {

        # windows lun has 1020MB (probably windows prefix and suffix)
        $actual_size =
          $self->get_byte_size($lun_size) - ( 4 * $ONE_MEG ) - $WIN_BLKSIZE;
    }
    else {
        $actual_size = $self->get_byte_size($lun_size);    # linux
    }
    return $actual_size;
}
##############################################################################
###   Function:         - _write_verify_data
###                 read/write to the one specfic lun with given data pattern
###
###   Parameters:       - see pod for details in SanXfun.pm
###
###   -lun_obj          - San Lun Object reference, to read and write. Lun
###                       Should already Mapped and accesed to the client.
###
###   -io_block_size    - Block Size to read/write. (sio syntax).
###      (Opt. Def: 4096)
###
###   -data_pattern     - Character to write on the Blocks. ("G", "a", etc)
###      (Opt. Def: "G")
###
###   -wait_flag        - 1 for wait, 0 for not to wait and return the
###      (Opt. Def: 1)    SanIo Object.
###
###   -sio_subtest_id   - PID name - SanIo Starts Sio using Subtest
###      (Opt. Def: "_write_verify_data")
###
###   -write_flag       - 1 to write, 0 - read/verify
###
###   Returns:
###      on success     - 1, if wait_flag is not set, it will returns
###                       SanIo Object
###
###      on failure     - 0 and sanxfun object errmsg() will
###                       contains error message
##############################################################################
sub _write_verify_data() {
    my ( $self, %args ) = @_;

    $self->_clear_err();

    my $lun_objs;
    my $io_block_size = 4096;    # 4K
    my $data_pattern  = "G";     # read/write G to the lun
    my $wait_flag      = 1;                      # to wait
    my $sio_num_threads = 10;
    my $sio_read_runtime = 600; # (in seconds) 600 sec = 10 minutes
    my $first_byte     = 0;
    my $last_byte      = 0;
    my $write_one_dev  = 0;

    my $write_flag;
    my $sio_subtest_id;
 
    foreach ( keys %args ) {
         if (/-lun_objs/) {
           $lun_objs = $args{$_};
           if( ref($lun_objs) !~ /ARRAY/ ){
              $self->_set_err("Invalid value for \"-lun_objs\": $lun_objs" .
                              " Need to pass ARRAY refernce of lun Objects");
              return $SanUtil::FALSE;
           }
         }
        elsif (/-io_block_size/) {
            length( $io_block_size = $args{$_} )
              or do {
                $self->_set_err(
                    "Invalid value for \"-io_block_size\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-data_pattern/) {
            length( $data_pattern = $args{$_} )
              or do {
                $self->_set_err(
                    "Invalid value for \"-data_pattern\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-wait_flag/) {
            length( $wait_flag = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-wait_flag\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-sio_subtest_id/) {
            length( $sio_subtest_id = $args{$_} )
              or do {
                $self->_set_err(
                    "Invalid value for \"-sio_subtest_id\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-write_flag/) {
            length( $write_flag = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-write_flag\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-sio_num_threads/) {
            length( $sio_num_threads = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-sio_num_threads\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-sio_read_runtime/) {
            length( $sio_read_runtime = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-sio_read_runtime\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-first_byte/) {
            (length( $first_byte = $args{$_} ) && ($first_byte =~ /^\s*[0-9]+(k|m|g|t)?\s*$/i) )
              or do {
                $self->_set_err("Invalid value for \"-first_byte\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-last_byte/) {
            (length( $last_byte = $args{$_} ) && ($last_byte =~ /^\s*[0-9]+(k|m|g|t)?\s*$/i) )
              or do {
                $self->_set_err("Invalid value for \"-last_byte\": $args{$_}");
                return $SanUtil::FALSE;
              };
        }
        elsif (/-write_one_dev/) {
            length( $write_one_dev = $args{$_} && ($write_one_dev =~ /yes|no|1|0/i) ) 
              or do {
                $self->_set_err("Invalid value for \"-write_one_dev\": $args{$_}");
                return $SanUtil::FALSE;
              };
            $write_one_dev = 1 if ($write_one_dev =~ /yes/i);
            $write_one_dev = 0 if ($write_one_dev =~ /no/i);
        } 
        else {
            $self->_set_err("Invalid parameter specified: $_");
            return $SanUtil::FALSE;
        }
    }

    #
    # check for read_write_flag
    #
    if ( !length($write_flag) ) {
        $self->_set_err( "Need to sepcifiy the value for "
              . "\"-write_flag\": $write_flag" );
        return $SanUtil::FALSE;
    }
    if( ref($lun_objs) !~ /ARRAY/ ){
       $self->_set_err("Invalid value for \"-lun_objs\": $lun_objs" .
                       " Need to pass ARRAY refernce of lun Objects");
       return $SanUtil::FALSE;
    }
    if( !defined($sio_subtest_id) or !length($sio_subtest_id) ){
       if($write_flag == 1){
          $sio_subtest_id = "sio_write_verify_data";
       }else{
          $sio_subtest_id = "sio_read_verify_data";
       }
      
    }
    #
    # get the required info from the lun Object
    #
    my $client_name = $self->{client_name};
    my $protocol    = $self->{protocol};

    $self->sanlog( -msg => "getting device information....");

    # get the client device name and size for the lun
    my %lun_device_info = $self->get_lun_devices_info( -lun_objs => $lun_objs );
    if ($self->error()) {
        my $errmsg = $self->errmsg();
        $self->_set_err( "get_lun_devices_info failed: $errmsg");
        return $SanUtil::FALSE;
    }
    my @device_array;
    my $filesize;
    #
    # Assume all luns are same size
    #
    for my $device_name (keys %lun_device_info) {
       $self->sanlog(-msg_level => 1, -msg => "get_lun_devices_info:\n" .
                  "Lun Path:         " . $lun_device_info{$device_name}{'lun_path'} . "\n" .
                  "Filer Name:       " . $lun_device_info{$device_name}{'filer_name'} . "\n" .
                  "Volume Name:      " . $lun_device_info{$device_name}{'vol_name'} . "\n" .
                  "Device Name:      " . $lun_device_info{$device_name}{'device_name'} . "\n" . 
                  "Full Device Name: " . $lun_device_info{$device_name}{'full_device_name'} . "\n" .
                  "Lun Size:         " . $lun_device_info{$device_name}{'size'} ."\n");

       push (@device_array, $lun_device_info{$device_name}{'full_device_name'} );
       $filesize = $lun_device_info{$device_name}{'size'};
       
    }
    # check whether user has provided any valid value to '-last_byte', 
    # if not then wrtie on complete lun
    $last_byte = $filesize if ($last_byte == 0);
    
    my $client_io_devs = join " ", @device_array;

    # Check whether user has provided any valid value to '-write_one_dev'. 
    # Make sure reads/writes happen on one LUN device path in case 'sanlun show' displays 
    # more than one path and '-write_one_dev' value is 'yes' or 1.
    if ( $write_one_dev && $#device_array > 0 ) {
       $self->sanlog( -msg_level => 3, -msg => "parameter -write_one_dev is set to 1...");
       $client_io_devs =  $device_array[0];
    }
    #
    # Instantiate a Sio Object
    #
    my $blksize = $io_block_size;
    my $pattern = ord($data_pattern);
    my $numops  = int( $filesize / $blksize );

    my $read_pct = ( $write_flag == 1 ) ? 0 : 100;
    my $run_time = ( $write_flag == 1 ) ? 0 : $sio_read_runtime; 

    my $pass_thru =
      ( $write_flag == 1 )
      ? "-fillonce -fixedfill -$pattern -direct -nosysinfo"
      : "-fixedfill -$pattern -verify -break_on_dc -direct -nosysinfo";

    $self->sanlog( -msg => "Waiting for I/O to start......" );

    my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst)=localtime(time);
    $sio_subtest_id=sprintf("sio_%s_%04d%02d%02d_%02d%02d%02d",
            $sio_subtest_id,$year+1900,$mon+1,$mday,$hour,$min,$sec  );

    my $io_subTest = subtest (sub {_run_sio(
                 $client_name,                  # client name        
                 $client_io_devs,               # client_io_dev 
                 $read_pct,                     # read_pct  - $CLIENT_IO_READ_PCT     
                 0,                             # random_pct - $CLIENT_IO_RANDOM_PCT   
                 $blksize,                      # block_size - $CLIENT_IO_BLOCK_SIZES    
                 $first_byte,                   # first_byte - $CLIENT_IO_DEV_OFFSET_BEGIN    
                 $last_byte,                    # last_byte - $CLIENT_IO_DEV_OFFSET_END     
                 $run_time,                     # time in seconds
                 $sio_num_threads,              # num_threads - $CLIENT_IO_NUM_THREADS  
                 $pass_thru)},                  # sio_switches - $CLIENT_IO_PASS_THRU
                 -runid  => $sio_subtest_id,
                 '-ifscript', "-eval_error",
                 '-iffatal' , "-eval_error",
                 -bg, -onexit =>  sub{});
    #
    # if wait flag is set, wait for IO to complete
    #
    if ( $wait_flag == 1 ) {
        $self->wait_verify_sioObj([$io_subTest]);
        if($self->error()){
            $self->_set_err($self->errmsg()); 
            return $SanUtil::FALSE;
        }
    }
    else{
        # sleep for 30 secs to I/O to start
        my $start_time = time();
        my $max_wait_time = 30 + $start_time; 
        # Poll every 15 seconds to check the IO subtest is alive for max ~1 mint
        SanUtil::nate_sleep(15) until ((!$io_subTest->isAlive()) or ((time()) > $max_wait_time));

        if (!$io_subTest->isAlive()) {
           #
           # if not alive, check io completes without error
           # if error return failure
           #
           $self->get_sio_result(-sio_subtest_obj => $io_subTest);
           if($self->error()){ 
               $self->_set_err("subtest is not alive: " . $self->errmsg());
               return $SanUtil::FALSE;
           }
        }
        return $io_subTest;
    }
    return $SanUtil::TRUE;
}

sub get_sio_result() {
   my ( $self, %args ) = @_;
   
   $self->_clear_err();
   my $sio_subtest_obj;
   foreach ( keys %args ) {
        if (/^-?sio_subtest_obj$/) {
          $sio_subtest_obj = $args{$_};
          if( !length($sio_subtest_obj) ){
             $self->_set_err("Invalid value for '-sio_subtest_obj'" .
                             " Need to pass subtest object reference");
             return $SanUtil::FALSE;
          }
        }
        else {
            $self->_set_err("Invalid parameter specified: $_");
            return $SanUtil::FALSE;
        }
   }

   if( !defined($sio_subtest_obj) or !length($sio_subtest_obj)) {
      $self->_set_err(" Need to pass subtest object ". 
                      "reference for '-sio_subtest_obj'");
      return $SanUtil::FALSE;
   }
   my $subtest_id = $sio_subtest_obj->runid();

   if ($sio_subtest_obj->is_alive() == 1) {
      $self->_set_err("Sio subtest($subtest_id) is still running");
      return $SanUtil::FALSE;
   }
   
   #
   # check out the result return from the subtest
   #
   my %results = 0;
   $results{'error_cont'} = 0;
   $results{'error_msg'} = "";
   my @ret_results = $sio_subtest_obj->rslts();
   my @subtest_messages = $sio_subtest_obj->results();
   my @temp_result = split("\n", $subtest_messages[0]);

   foreach my $result (@ret_results) {
      my $result_type = $result->type();
      my $no_msg = $result->messages();
      $self->sanlog(-msg => "RESULT TYPE = $result_type", -msg_level => 7);
      $self->sanlog(-msg => "No of Messages = $no_msg", -msg_level => 7);
      if($result_type =~ /PASS/){ 
         #
         # if subtest passes, subtest prints the hash value thorugh logcomment
         # output will be: first line <key> and next line <value>
         # 
         for(my $i=0; $i < @temp_result; $i += 2){
            $results{$temp_result[$i]} = $temp_result[$i+1];
         }
      }else {
         if( $result_type =~ /FAIL/){
            #
            # index 0 will have key = 'error_count'
            # index 1 will have value for the key='error_count'
            # index 2 will have key = 'error_msg';
            # rest will have the error message;
            $results{$temp_result[0]} = $temp_result[1];
            $results{'error_msg'} = $temp_result[3];
            for(my $i=4; $i < $no_msg; $i++){
               $results{'error_msg'} .= "\n" . $temp_result[$i];
            }
         } else{
            $results{'error_cont'}++;
            $results{'error_msg'} .= $result . "\n";
         }
      }
   } # end foreach my $result (@ret_results)
   
   $self->sanlog(-msg => "get_io_result(): "                .
    "**********RESULT =\n"                                  .
    "error_count        = $results{'error_count'}\n"        .
    "error message      = $results{'error_msg'}\n"          .
    "iops               = $results{'iops'}\n"               .
    "tput               = $results{'tput'}\n"               .
    "latency_calculated = $results{'latency_calculated'}\n" .
    "latency_actual     = $results{'latency_actual'}\n"     .
    "read               = $results{'read'}\n"               .
    "random             = $results{'random'}\n"             .
    "ios                = $results{'ios'}\n"                .
    "seconds            = $results{'seconds'}\n"            .
    "threads            = $results{'threads'}\n"            .
    "blksize            = $results{'blksize'}\n"            .
    "csw_io             = $results{'csw_io'}\n"             .
    "int_io             = $results{'int_io'}\n"             .
    "sys                = $results{'sys'}\n"                .
    "usr                = $results{'usr'}\n"                .
    "wt                 = $results{'wt'}\n"                 .
    "idl                = $results{'idl'}\n"                .
    "uio                = $results{'uio'}\n"                .
    "ukb                = $results{'ukb'}\n"                .
    "uuio               = $results{'uuio'}\n"               .
    "tuio               = $results{'tuio'}\n",
    -msg_level => 7);

   # interpret the result and set the success flag
   if( $results{'error_count'} > 0 ) {
      $self->_set_err( $results{'error_msg'});
      return $SanUtil::FALSE;
   }
   return %results;
}

sub _run_sio() 
{
   my ($client_name, $client_io_dev, $read_pct, $random_pct, 
       $block_size, $first_byte, $last_byte, $run_time, $num_threads, $other_sio_params) = @_;

   logcomment("Starting I/O, Sio paramters\n" .
              "Client         : $client_name\n" .
              "Device(s)      : $client_io_dev\n" . 
              "Read Pct       : $read_pct\n" . 
              "Random Pct     : $random_pct\n" . 
              "Block Size     : $block_size\n" . 
              "First byte     : $first_byte\n" . 
              "Last  byte     : $last_byte\n" . 
              "No. of Threads : $num_threads\n" . 
              "Run Time       : $run_time\n" . 
              "Pass Through   : $other_sio_params" );

   my $chost = host($client_name);
   my $client_conn;

    if ( $chost->{hosttype} =~ /windows/ ) {
       $client_conn = "#rsh";
    } elsif (defined($chost->{connection_fields}->{ssh})) {
        $client_conn = "#ssh";
    } elsif (defined($chost->{connection_fields}->{rsh})) {
        $client_conn = "#rsh";
    } elsif (defined($chost->{connection_fields}->{telnet})) {
        $client_conn = "#telnet";
    } else {
        $client_conn = "#telnet";
    }

   my $client_connection = connect ($client_name.$client_conn.";inherit=0");
   return $ClientUtil::FALSE  unless ($client_connection);

	
   my %results;
   $results{'error_count'} = 0;
   
   # Format the list of devices.
   $client_io_dev = _format_sio_device("$client_io_dev");

   # Create the system independent program invocation.
   my $cmd  = "sio $read_pct $random_pct"
                  . " $block_size $first_byte $last_byte" 
                  . " $run_time" 
                  . " $num_threads" 
                  . " $client_io_dev"
                  . " $other_sio_params";
   
   if( $chost->{hosttype} =~ /windows/) {
      # Establish MinWin mapped drive
      if ( minwin_compatible($chost->id()) ) {
          $cmd = 'X:\\bin\\'.$cmd;   
      }
   }

   my $byte_count;
   $byte_count = $last_byte;
   $byte_count =~ s/(\d+).*/$1/;
   if ($last_byte =~ m/k/i) {
      $byte_count = $byte_count * 1024;
   } elsif ($last_byte =~ m/m/i) {
      $byte_count = $byte_count * 1024 * 1024;
   } elsif ($last_byte =~ m/g/i) {
      $byte_count = $byte_count * 1024 * 1024 * 1024;
   } elsif ($last_byte =~ m/t/i) {
      $byte_count = $byte_count * 1024 * 1024 * 1024 * 1024;
   }
   
   # Send the command to the client and get sio output.
   #
   # Allow some additional time for sio to return
   # after it completes the specified $runlength
   # yet, timeout in case there is a hang condition
   my $timeout = $run_time + 1800;
   my $one_meg = 1024 * 1024;
   # because fillonce, use last_byte and blksize to determine timeout.
   if ($run_time == 0) {
      $timeout = (($byte_count / $one_meg) + 1800);
   }
   
   my ($status, @output) = &ClientUtil::execute($client_connection, $cmd, $timeout);
   if ($status gt 0) {
      $results{'error_count'} = 1;
      $results{'error_msg'} = "Error executing command on device \'$client_name\' \nCommand = " . $cmd . "\nOutput = " . join "\n", @output;
      logresult("FAIL", %results);
      return $ClientUtil::FALSE;      
   } 
   
   my @unexpected_output;
   my $error_message = "Unrecognized sio output on device \'$client_name\'";
   my $last_label = '';

   foreach my $line (@output) {
      if ($line =~ /^Read:\s+\d+\s+Rand:\s+/) {
         # this is the display of params info
      } elsif  ($line =~ /^(\d+),\s*(\d+),\s*([\.\d]+),?\[([\.\d]+)\],\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*([\.\d]+),\s*([\.\d]+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*([\.\d]+),\s*(\d+),\s*(\d+)/) {
         $results{'iops'} = $1;    # I/O operations per second
         $results{'tput'} = $2;    # KB per second
         $results{'latency_calculated'} = $3;     # latency in milliseconds
         $results{'latency_actual'} = $4;     # latency in milliseconds
         $results{'read'} = $5;    # read percentage
         $results{'random'} = $6;  # random percentage
         $results{'ios'} = $7;     # total I/O operations
         $results{'seconds'} = $8; # runlength in seconds
         $results{'threads'} = $9; # threads
         $results{'blksize'} = $10;   # block size
         $results{'csw_io'} = $11;   # 
         $results{'int_io'} = $12;  # 
         $results{'sys'} = $13;  # 
         $results{'usr'} = $14;  # 
         $results{'wt'} = $15;  # 
         $results{'idl'} = $16;  # 
         $results{'uio'} = $17;  # 
         $results{'ukb'} = $18;  # 
         $results{'uuio'} = $19;  # 
         $results{'tuio'} = $20;  # 
      } elsif ($line =~ /^(\d+),\s*(\d+),\s*([\.\d]+),?\[([\.\d]+)\],\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*([\.\d]+),\s*([\.\d]+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*([\.\d]+)/) {
         # Older versions only sent back 18 values
         $results{'iops'} = $1;    # I/O operations per second
         $results{'tput'} = $2;    # KB per second
         $results{'latency_calculated'} = $3;     # latency in milliseconds
         $results{'latency_actual'} = $4;     # latency in milliseconds
         $results{'read'} = $5;    # read percentage
         $results{'random'} = $6;  # random percentage
         $results{'ios'} = $7;     # total I/O operations
         $results{'seconds'} = $8; # runlength in seconds
         $results{'threads'} = $9; # threads
         $results{'blksize'} = $10;   # block size
         $results{'csw_io'} = $11;   # 
         $results{'int_io'} = $12;  # 
         $results{'sys'} = $13;  # 
         $results{'usr'} = $14;  # 
         $results{'wt'} = $15;  # 
         $results{'idl'} = $16;  # 
         $results{'uio'} = $17;  # 
         $results{'ukb'} = $18;  # 
      } elsif ($line =~ /^(\d+),\s*(\d+),\s*([\.\d]+),?\[([\.\d]+)\],\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+),\s*(\d+)/) {
         # Older versions only sent back 18 values
         $results{'iops'} = $1;    # I/O operations per second
         $results{'tput'} = $2;    # KB per second
         $results{'latency_calculated'} = $3;     # latency in milliseconds
         $results{'latency_actual'} = $4;     # latency in milliseconds
         $results{'read'} = $5;    # read percentage
         $results{'random'} = $6;  # random percentage
         $results{'ios'} = $7;     # total I/O operations
         $results{'seconds'} = $8; # runlength in seconds
         $results{'threads'} = $9; # threads
         $results{'blksize'} = $10;   # block size
      } elsif ($line =~ /^0,0,/) {
         # No I/O was performed!
         $results{'iops'} = 0;    # I/O operations per second
         $results{'tput'} = 0;    # KB per second
         logresult('FAIL',"No I/O performed by sio");
      } elsif ($line =~ /^\s*$|^Terminating threads .../) {
         # Normal output lines with no data to return
      } elsif ($line =~ /^sio\s/) {
         # echo of sio input command
      } elsif ($line =~ /^IOPS,\s+TPUT|^IOPS,\s*TPUT\(KB\/s\)/) {
         # Normal output lines with no data to return
      } elsif ($line =~ /I\/O completed successfully|PASS/) {
         # If random options are used, this string is printed in
         # lieu of stats.
      } elsif ($line =~ /Version:\s*(\S+)/) {
         $results{'version'} = $1;   
      } elsif ($line =~ /Maximum latency observed:\s*(\d+)/) {
         $results{'max_latency'} = $1;  
      } elsif ($line =~ m/Execution less than expected/ &&
             $other_sio_params =~ m/-numops/ ) {
             # because runlength=0, sec is default to 1073741823
             # use -numops to read
             # we should ignore
             next;
      # ignore extra output messages from verion 6.22
      } elsif ($line =~ /^(global_|Computed)/) {
         next;     
      # Configuration Errors
      } elsif ($line =~ /host or user not listed in rhosts/) {
         $results{'error_count'}++;
         $results{'error_msg'} = "CLIENT $client_name needs to enable rsh access";
         logresult("FAIL", %results);
         return $ClientUtil::FALSE;      
      #  ExitConfigError("CLIENT $client needs to enable rsh access");
      } elsif ($line =~ / sio:  not found/) {
         $results{'error_count'}++;
         $results{'error_msg'} = "CLIENT $client_name needs access to the sio program";
         logresult("FAIL", %results);
         return $ClientUtil::FALSE;      
      #  ExitConfigError("CLIENT $client needs access to the sio program");
      # Script Errors
      } elsif ($line =~ /Unknown size type /) {
         $results{'error_count'} = 1;
         $results{'error_msg'} = "Invalid block size parameter";
         logresult("FAIL", %results);
         return $ClientUtil::FALSE;      
      #  ExitScriptError("Invalid block size parameter");
      # Other Errors    
      } elsif ($line =~ /File open error|CreateFile fails with The system cannot find the file specified/) {
         $error_message = "SIO CLIENT $client_name must be configured with: $client_io_dev";
         $results{'error_count'}++;
      } elsif ($line =~ m/sio:\s+(.*)/) {
         $error_message = "SIO reported an error on CLIENT $client_name: $1";
         $results{'error_count'}++;
      #Version: 6.06\r\n
      #Read: 50 Rand: 50 BlkSz: 1024 BegnBlk: 0 EndBlk: 100 Secs: 240 Threads: 1 Devs: 1  /dev/rnate_lvlog
      #open: No such file or directory
      #File: /dev/rnate_lvlog
      #
      #sio: File open error
      } elsif ($line =~ m/(open: No such file or directory)/) {
         $error_message = "SIO reported an error on CLIENT $client_name: $line";  
         $results{'error_count'}++;
      } elsif ($line =~ m/File: (\S+)/) {
         if (0 == $results{'error_count'}) {
             # Previous message should have been seen first,
             # don't count this unless it is first
             $error_message = "SIO reported an error on CLIENT $client_name with file: $1";
             $results{'error_count'}++;
         }
      # write fails 
      } elsif ($line =~ m/error accessing file|WriteFile fails|Filename:/) {
         $error_message = "SIO reported write error on CLIENT $client_name: $line";  
         $results{'error_count'}++;
      } elsif ($line =~ m/error reading:/) {
         $error_message = "SIO reported write error on CLIENT $client_name: $line";  
         $results{'error_count'}++;
      } elsif ($line =~ m/connect to address|Connection refused|Trying\s+\S+\s+rsh/) {  
         # connect to address 10.72.237.39 port 544: Connection refused\r\n
         # Trying krb4 rsh...\r\n
         # connect to address 10.72.237.39 port 544: Connection refused\r\n
         # trying normal rsh (/usr/bin/rsh)\r\n
         # just ignore these lines....
         next;
      }
      else {
         push(@unexpected_output,$line);
         $results{'error_count'}++;
      }   #
    } # end of main foreach loop

    if (@unexpected_output or $results{'error_count'} > 0 ) {
       $results{'error_msg'} = "$error_message" . join "\n", @unexpected_output;
       logresult("FAIL", %results);
       return $ClientUtil::FALSE;      
    }
    logresult("PASS", %results);
    return $ClientUtil::TRUE;

} # end of _run_sio

# ######################################################################
## @name ::_format_sio_device
## @summary Return a formatted device name from something like "e:"
##
## @arg in_device Required: Device string to format.
##
# ######################################################################
sub _format_sio_device($) 
{
   my $in_device = shift;
   my $out_device = $in_device;
   
   # Match any number of preceeding characters that are "\" or "."
   # followed by any letter and ":" such as "e:"
   # followed by anything.
   if ($out_device =~ m/[\S]*[a-zA-Z]\:.*/i)
   {
      # Remove anything that preceeds any letter followed by a colon.
      $out_device =~ s/\S+([a-zA-Z]\:\S*)/$1/g;
      
      # Replace "e:" with "\\\\.\\e:".
      $out_device =~ s/([a-zA-Z]\:)/\\\\\.\\$1/g;
   }
   
   return $out_device;
} # end _format_sio_device



1;
