# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/San/SanXfun/SanXfunSMBase.pm#1 $
#
# Copyright (c) 2009 NetApp, Inc.
# All rights reserved.
#
# lib/San/SanXfun/SanXfunSMBase.pm
##
## @summary Wrapper to SanSnapMirror library to use with
##  SanXfun Setup for SnapMirror environment.
##
## @description
##   This utility functions used by SanXfun setup for SnapMIrror environment.
##
## @author Muthukumaran Govindaraj (muthug@netapp.com)
## @status public
## @change 2009-02-02 muthu: Original template created
## @pod here

# Class : SanXfunSMBase

package SanXfunSMBase;

use strict;

#
# Inherit from the following classes (DO NOT CHANGE THE ORDER)
#     San::SanXfun::SanSnapMirror (SUPER class)
#

BEGIN {
    use San::SanXfun::SanSnapMirror;
    use base qw{SanSnapMirror};
}

##############################################################################
#	                   P R I V A T E   M E T H O D S                         #
##############################################################################

##############################################################################
###   Function:       - _setup_sanxfun_snapmirror -
###                   To create SnapMirror Environmnet for Sanxfun
###   Parameters:     - See bleow
###   Returns:
###      on success              - 1
###      on failure              - 0, and sets the error message
##############################################################################
sub _setup_sanxfun_snapmirror() {
    my ( $self, %args ) = @_;

    $self->_clear_err();
    $self->{snapmirror_initialized} = 0;

    #
    # check SanXfun is snapmirror
    #
    if ( $self->{san_xfun_env} !~ /snapmirror/i ) {
        $self->_set_err( "SanXfun Environment($self->{san_xfun_env}) "
              . "is not snapmirror, can not call this function." );
        return $SanUtil::FALSE;
    }

    #
    # SYNC mirror does not support QSM
    #
    if (   $self->{sm_sync_type} eq "sync"
        && $self->{sm_replicate_type} =~ /qsm/i )
    {
        $self->_set_err("SYNC (rsm) snapmirror does not support qsm transfer");
        return $SanUtil::FALSE;
    }

    #
    # get filers list to detrmine whether it is local or remote transfer
    #
    ( $self->{src_filer_name}, $self->{dst_filer_name} ) =
      split( /:/, $args{-filers} );

    if (   !defined( $self->{src_filer_name} )
        || !length( $self->{src_filer_name} ) )
    {
        $self->_set_err(
            "Source filer name not found, must pass -filers: $args{-filers}");
        return $SanUtil::FALSE;
    }

    #
    # remove -filers enrty from arg, since we are going to setup
    # indiviually for each filer
    #
    delete $args{-filers};

    #
    # parse the filer_vol_list, if sm_replicate_type = qsm or local tranfer
    #
    my $src_filer_vol_list = $args{-filer_vol_list};

    #
    # Trim off any space toward the end of the string, so that match
    # below will work for when there is no comma after the vol_prefix.
    #

    $src_filer_vol_list =~ s/\s+$//g;
    my $dst_filer_vol_list = $src_filer_vol_list;

    #
    # remove -filer_vol_list enrty from arg,
    # since we are going to setup indiviually for each filer
    #
    delete $args{-filer_vol_list};

    #
    # Append _dst for the destination volume name, if it is local sm transfer
    #
    if (   !defined( $self->{dst_filer_name} )
        || !length( $self->{dst_filer_name} ) )
    {
        $self->{sm_xfer_type}   = "local";
        $self->{dst_filer_name} = $self->{src_filer_name};

        #
        # snapmirror sync does not work in same filer or in HA pair,
        # if it is cluster testbed we need to disable the cf.
        # refer burt 110792, 132274 for more details -
        #
        if ( $self->{sm_sync_type} eq "sync" ) {
            $self->_set_err(
                "SYNC (rsm) snapmirror does not support local filer");
            return $SanUtil::FALSE;
        }

        # check for the type and vol_prefix parameter passed, if not just 
        # default the vol type to flex and vol name to san_vol_dst. 
        # if defined get the volume name from the user and 
        # append _dst to the volume name.
        #
        if ( $dst_filer_vol_list =~ /\d+,\d+\w*,\d+,\d+\w*,\w+,(.*?)(,|$)/ ) {
            my $org_vol_prefix;
            my $qtree_prefix;
            my $vol_prefix = $org_vol_prefix = $1;
            ( $vol_prefix, $qtree_prefix ) = split( /\//, $vol_prefix );
            $vol_prefix =
                $qtree_prefix
              ? $vol_prefix . "_dst/$qtree_prefix"
              : $vol_prefix . "_dst";
            $dst_filer_vol_list =~ s/$org_vol_prefix/$vol_prefix/;
        }
        else {
            if ( $dst_filer_vol_list =~ /\d+\w*,(\w+)/ ) {
                $dst_filer_vol_list .= ",san_vol_dst";
            }
            else {
                $dst_filer_vol_list .= ",flex,san_vol_dst";
            }
        }
    }
    else {
        $self->{sm_xfer_type} = "remote";
    }

    #
    # remove the destination volume qtree prefix from the volume_prefix,
    # since QSM does not initialize with the existing qtree,
    # which san::setup creates
    #
    if ( $self->{sm_replicate_type} =~ /qsm/i ) {

        # since it is qsm trafsfer, user should defintaly pass the type
        # and volume perfix with qsm name so remove the QSM name from 
        # the volume prefix. expected format for filer vol list :
        #                      1,50g,5,1g,flex,ems_vol/Qsm,test_aggr
        #   
        if ( $dst_filer_vol_list =~ /\d+,\d+\w*,\d+,\d+\w*,\w+,(.*?)(,|$)/ ) {
            my $org_vol_prefix;
            my $qtree_prefix;
            my $vol_prefix = $org_vol_prefix = $1;
            ( $vol_prefix, $qtree_prefix ) = split( /\//, $vol_prefix );
            if (   !defined($qtree_prefix)
                || !length($qtree_prefix) )
            {
                $self->_set_err(
                    "qtree prefix is not defined in the filer_vol_list");
                return $SanUtil::FALSE;
            }
            $self->{qtree_prefix} = $qtree_prefix;
            $dst_filer_vol_list =~ s/$org_vol_prefix/$vol_prefix/;
        }
        else {
            $self->_set_err("volume/qtree prefix is not defined " .
                                "in the filer_vol_list for QSM transfer" );
            return $SanUtil::FALSE;
        }
    }

    #
    # do san setup, first on source(filer)
    # san::setup for the source (filer) volume
    #
    $self->San::setup(
        -filers         => $self->{src_filer_name},
        -filer_vol_list => $src_filer_vol_list,
        %args
    );

    if ( $self->error() ) {
        $self->_set_err( $self->errmsg() );
        return $SanUtil::FALSE;
    }

    #
    # get the resources from source(filer): vol
    #
    my @filer_resources = $self->get_resources(
        -type => "filer",
        -name => $self->{src_filer_name},
        -cmd  => "vol_create"
    );

    if ( $self->error() ) {
        my $error_msg = $self->errmsg();
        $self->_set_err(
            "Failed to get san resources from for vol_create: $error_msg");
        return $SanUtil::FALSE;
    }

    my $san_vol_name = "";
    foreach my $r_hash (@filer_resources) {
        if ( $r_hash->{-cmd} =~ /vol_create/i ) {
            $san_vol_name = $r_hash->{-cmd_args};
            $san_vol_name =~ s/^\/vol\///;    # remove /vol/
            $san_vol_name =~ s/\/$//;         # remove trailing "/"
            last;
        }
    }    # end foreach
    if ( $san_vol_name eq "" ) {
        $self->_set_err("Failed to get source volume name");
        return $SanUtil::FALSE;
    }
    $self->{src_vol_name} = $san_vol_name;

    #
    # if it 2 controller, sync does not support HA pair, so disable cf
    #
    if ( $self->{src_filer_name} ne $self->{dst_filer_name} ) {
        if ( $self->{sm_sync_type} eq "sync" ) {

            # disable cf  - make sure we are eanbleing back in the cleanup
            my $filer_obj = $self->get_filer( $self->{src_filer_name} );
            if ( $self->error() ) {
                my $errmsg = $self->errmsg();
                $self->_set_err(
                    "Unable to get the filer object for the source: $errmsg");
                return $SanUtil::FALSE;
            }
            $filer_obj->cf_disable();
            if ( $filer_obj->error() ) {
                my $errmsg = $filer_obj->errmsg();
                $self->_set_err("Unable to disable cluster pair: $errmsg");
                return $SanUtil::FALSE;
            }
        }
    }

    #
    # if it is different filer, i.e remote snapmirror, not local
    #
    my $san_obj_dst = "";

    #
    # do san setup, on destination(filer)
    # here we need to stop after aggr/vol - create.
    #
    $san_obj_dst = new San( $self->{log} );
    if ( $san_obj_dst->error() ) {
        $self->_set_err( $san_obj_dst->errmsg() );
        return $SanUtil::FALSE;
    }
    $san_obj_dst->set_debug_level( $self->{debug_level} );

    #
    # san::setup for the destiantion (filer) volume
    #
    $san_obj_dst->setup(
        -filers         => $self->{dst_filer_name},
        -setup_luns     => 0,
        -filer_vol_list => $dst_filer_vol_list,
        %args
    );

    if ( $san_obj_dst->error() ) {
        $self->_set_err( $san_obj_dst->errmsg() );
        return $SanUtil::FALSE;
    }

    #
    # get the resources from destination(filer): vol
    #
    my @filer_resources = $san_obj_dst->get_resources(
        -type => "filer",
        -name => $self->{dst_filer_name},
        -cmd  => "vol_create"
    );
    if ( $san_obj_dst->error() ) {
        my $error_msg = join "\n", $san_obj_dst->errmsg();
        $self->_set_err(
            "Failed to get san resources from for vol_create: $error_msg");
        return $SanUtil::FALSE;
    }

    my $san_vol_name = "";
    foreach my $r_hash (@filer_resources) {
        if ( $r_hash->{-cmd} =~ /vol_create/i ) {
            $san_vol_name = $r_hash->{-cmd_args};
            $san_vol_name =~ s/^\/vol\///;    # remove /vol/
            $san_vol_name =~ s/\/$//;         # remove trailing "/"
            last;
        }
    }    # end foreach
    if ( $san_vol_name eq "" ) {
        $self->_set_err("Failed to get destination volume name");
        return $SanUtil::FALSE;
    }
    $self->{dst_vol_name} = $san_vol_name;

    #
    # get the resources from destination(filer): igroup
    #
    my @filer_resources = $san_obj_dst->get_resources(
        -type => "filer",
        -name => $self->{dst_filer_name},
        -cmd  => "igroup_create"
    );
    if ( $san_obj_dst->error() ) {
        my $error_msg = join "\n", $san_obj_dst->errmsg();
        $self->_set_err(
            "Failed to get san resources from for igroup_create: $error_msg");
        return $SanUtil::FALSE;
    }
    my $igrp_name = "";
    foreach my $r_hash (@filer_resources) {
        if ( $r_hash->{-cmd} =~ /igroup_create/i ) {
            $igrp_name = $r_hash->{-cmd_args};
            last;
        }
    }    # end foreach
    if ( $igrp_name eq "" ) {
        $self->_set_err("Failed to get destination igroup name");
        return $SanUtil::FALSE;
    }
    $self->{dst_igroup_name} = $igrp_name;

    #
    # NOTE: SM Path path should be <volname>
    # if it is Qtree then </vol/volname/qtreename>
    #
    if ( $self->{sm_replicate_type} =~ /vsm/i ) {
        $self->{src_snapmirror_path} = $self->{src_vol_name};
        $self->{dst_snapmirror_path} = $self->{dst_vol_name};
    }
    else {
        $self->{src_snapmirror_path} =
          "/vol/" . $self->{src_vol_name} . "/$self->{qtree_prefix}";

        $self->{dst_snapmirror_path} =
          "/vol/" . $self->{dst_vol_name} . "/$self->{qtree_prefix}";
    }

    #
    # TODO - check destination volume is equal or grather than Source volume
    #

    my @sm_agr;
    push( @sm_agr, "-sm_replicate_type",   $self->{sm_replicate_type} );
    push( @sm_agr, "-sm_sync_type",        $self->{sm_sync_type} );
    push( @sm_agr, "-src_filer_name",      $self->{src_filer_name} );
    push( @sm_agr, "-dst_filer_name",      $self->{dst_filer_name} );
    push( @sm_agr, "-src_snapmirror_path", $self->{src_snapmirror_path} );
    push( @sm_agr, "-dst_snapmirror_path", $self->{dst_snapmirror_path} );

    $self->SanSnapMirror::setup(@sm_agr);
    if ( $self->error() ) {
        my $errmsg = $self->errmsg();
        $self->_set_err("Failed to setup SnapMirror object: $errmsg");
        return $SanUtil::FALSE;
    }

    #
    # create snapmirror realtionship
    #
    $self->san_sm_initialize();
    if ( $self->error() ) {
        my $errmsg = $self->errmsg();
        $self->_set_err("Failed to establish SnapMirror relationship: $errmsg");
        return $SanUtil::FALSE;
    }
    $self->{snapmirror_initialized} = 1;

    #
    # Map the destination luns
    #

    my $filer_obj = $san_obj_dst->get_filer( $self->{dst_filer_name} );
    if ( $san_obj_dst->error() ) {
        my $errmsg = $san_obj_dst->errmsg();
        $self->_set_err("Unable to get the filer object: $errmsg");
        return $SanUtil::FALSE;
    }

    #
    # get the volume object from destination
    #
    my $vol_obj = $filer_obj->get_vol( $self->{dst_vol_name} );
    if ( $filer_obj->error() ) {
        my $errmsg = $filer_obj->errmsg();
        $self->_set_err("Unable to get the dst. volume object: $errmsg");
        return $SanUtil::FALSE;
    }

    #
    # refersh the volume to get all the luns
    #
    $vol_obj->refresh();
    my @lun_objs = $vol_obj->get_luns();
    foreach my $lun_obj (@lun_objs) {
        my $lun_name = $lun_obj->get_name();

        #
        # add this into dest. cleanup stack as newly created luns
        # since it is created by snapmirror init.
        #
        $san_obj_dst->cleanup_add(
            -type     => "filer",
            -name     => $self->{dst_filer_name},
            -cmd      => "lun_destroy",
            -cmd_args => "$self->{dst_vol_name} $lun_name"
        );
        if ( $san_obj_dst->error() ) {
            my $error_msg = join "\n", $san_obj_dst->errmsg();
            $self->_set_err(
                "Unable to add lun destroy cleanup stack: $error_msg");
            return $SanUtil::FALSE;
        }

        #
        # map the lun and add it to cleanup stack
        #
        $lun_obj->map( $self->{dst_igroup_name} );
        if ( $lun_obj->error() ) {
            my $error_msg = join "\n", $lun_obj->errmsg();
            $self->_set_err( "Unable to map the lun($lun_name) to "
                  . "igroup($self->{dst_igroup_name}): $error_msg" );
            return $SanUtil::FALSE;
        }
        $san_obj_dst->cleanup_add(
            -type => "filer",
            -name => $self->{dst_filer_name},
            -cmd  => "lun_unmap",
            -cmd_args =>
              "$self->{dst_vol_name} $lun_name $self->{dst_igroup_name}"
        );
        if ( $san_obj_dst->error() ) {
            my $error_msg = join "\n", $san_obj_dst->errmsg();
            $self->_set_err(
                "Unable to add lun unmap cleanup stack: $error_msg");
            return $SanUtil::FALSE;
        }
    }

    #
    # add this filer to the original san object
    #
    if ( $self->{src_filer_name} ne $self->{dst_filer_name} ) {
        my $filer_obj = $san_obj_dst->get_filer( $self->{dst_filer_name} );
        if ( $san_obj_dst->error() ) {
            my $errmsg = $san_obj_dst->errmsg();
            $self->_set_err("Unable to get the filer object: $errmsg");
            return $SanUtil::FALSE;
        }
        $self->add_filer( -filer => $filer_obj );
        if ( $self->error() ) {
            my $errmsg = $self->errmsg();
            $self->_set_err("Unable to add filer to San Object: $errmsg");
            return $SanUtil::FALSE;
        }
    }

    #
    # build the cleanup stack for the destination volume
    #
    if ( !$self->_add_cleanup_stack($san_obj_dst) ) {
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
}

##############################################################################
###   Function:                  - _cleanup_sanxfun_snapmirror -
###                              To cleanup the sampmirror relationship
###   Parameters:                - None
###   Returns:
###      on success              - 1
###      on failure              - 0, and the text contains error message
##############################################################################
sub _cleanup_sanxfun_snapmirror() {
    my ($self) = @_;

    #
    # uninitialized snapmirror relationship
    #
    if ( $self->{snapmirror_initialized} == 1 ) {
        $self->san_sm_cleanup();
    }

    #
    # enable the cf if we are testing with SYNC sanpmirror
    #
    if ( $self->{src_filer_name} ne $self->{dst_filer_name} ) {
        if ( $self->{sm_sync_type} eq "sync" ) {
            my $filer_obj = $self->get_filer( $self->{src_filer_name} );
            if ( $self->error() ) {
                my $errmsg = $self->errmsg();
                $self->_set_err(
                    "Unable to get the filer object for the source: $errmsg");
                return $SanUtil::FALSE;
            }

            #
            # enable cf back
            #
            $filer_obj->cf_enable();
            if ( $filer_obj->error() ) {
                my $errmsg = $filer_obj->errmsg();
                $self->_set_err("Unable to disable cluster pair: $errmsg");
                return $SanUtil::FALSE;
            }
        }
    }
    return $SanUtil::TRUE;
}

##############################################################################
###   Function:                  - _add_cleanup_stack -
###                                To add clenaup stack for SM
###   Parameters:                - TBD
###   Returns:
###      on success              - 1
###      on failure              - 0, and the text contains error message
##############################################################################
sub _add_cleanup_stack() {
    my ( $self, $san_obj_dst ) = @_;

    $self->_clear_err();

    if ( ref($san_obj_dst) !~ /San/ ) {
        $self->_set_err("Need to pass San Object");
        return $SanUtil::FALSE;
    }

    #
    # get all the resources from dst san object and
    # add it to original san object
    #
    my @filer_resources = $san_obj_dst->get_resources();
    if ( $san_obj_dst->error() ) {
        my $error_msg = join "\n", $san_obj_dst->errmsg();
        $self->_set_err(
            "Failed to get all san resources from dstination: $error_msg");
        return $SanUtil::FALSE;
    }
    foreach my $cleanup_hash (@filer_resources) {

        #
        # Translate the command from a cleanup type to a setup action
        #
        my $cmd_temp = $cleanup_hash->{-cmd};
        $cmd_temp =~ s/aggr_create/aggr_destroy/g;
        $cmd_temp =~ s/vol_create/vol_destroy/g;
        $cmd_temp =~ s/lun_create/lun_destroy/g;
        $cmd_temp =~ s/igroup_create/igroup_destroy/g;
        $cmd_temp =~ s/lun_map/lun_unmap/g;
        $cleanup_hash->{-cmd} = $cmd_temp;

        #
        # add it to the original san object
        #
        $self->cleanup_add(%$cleanup_hash);
        if ( $self->error() ) {
            my $error_msg = $self->errmsg();
            $self->_set_err(
                "Unable to add cleanup stack to original san object: $error_msg"
            );
            return $SanUtil::FALSE;
        }
    }    # end foreach
    return $SanUtil::TRUE;
}

1;

