# Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/San/SanXfun/UsecaseLunClone.pm#1 $
#
# Copyright (c) 2008 NetApp, Inc.
# All rights reserved.
#
# lib/San/SanXfun/UsecaseLunClone.pm
##
## @summary Lun clone Utility to test with San xfun suite
## @description
##   This utility to provide lun clone sub command function
##
## @description Objective:
## This utility to provide lun clone sub command function.
## @author Muthukumaran Govindarajan (muthug@netapp.com)
## @status public
## @change 2008-10-16 muthu: Original template created

package UsecaseLunClone;

use strict;

use Exporter;
use San::SanXfun::SanXfun;
use San::SanLog;
use San::SanUtil;
use Error;

use vars qw( @EXPORT @ISA );

#
# Inherit from the Error, SanLog, and SanUtil classes
#
@ISA = qw(Exporter Error SanLog SanUtil);

#
# constant values
#

# to decrease the lun clone size from original
use constant LUN_RESIZE_DECREASE => 0; 
# to increase the lun clone size from original  
use constant LUN_RESIZE_INCREASE => 1;    

use constant IO_WAIT    => 1;
use constant IO_NO_WAIT => 0;

use constant BREAK_SM_RELATIONSHIP      => 1;
use constant DONT_BREAK_SM_RELATIONSHIP => 0;

use constant NO_IO_TEST => 0;
use constant IO_TEST => 1;


#
# constructor
#
sub new {
    my ( $class, %args ) = @_;
    my $self = bless( {}, $class );

    $self->{log}          = \*ntest::NTEST_STDOUT;
    $self->{'sanxfun'}    = undef;

    $self->sanlog( -msg => "UsecaseLunClone new" );
    $self->_clear_err();

    foreach ( keys %args ) {
        if (/^-?sanxfun$/) {
            $self->{'sanxfun'} = $args{$_};
            if ( ref( $self->{'sanxfun'} ) !~ /SanXfun/ ) {
                $self->_set_err("Invalid value for \"-sanxfun\": " 
                               . "$args{$_}, need a SanXfun reference");
                $self->{'sanxfun'} = undef;
                return $self;
            }
        }
        elsif (/^-?log$/) {
            $self->{log} = $args{$_};
            if( !length($self->{log})) {
               $self->_set_err("Invalid value for \"-log\": $args{$_}");
               $self->{'sanxfun'} = undef;
               return $self;
            }
        }
        else {
            $self->_set_err("Invalid parameter specified: $_");
            $self->{'sanxfun'} = undef;
            return $self;
        }
    }
    #
    # Check required paramters
    #
    if ( ref( $self->{'sanxfun'} ) !~ /SanXfun/ ) {
        $self->_set_err("Need a Xfun reference for '-sanxfun'");
        $self->{'sanxfun'} = undef;
        return $self;
    }
    return $self;
}
##########################################################################################################
###   Function:                  - lun_clone_resize
###                               test lun clone resize after snapshot taken with partial write to
###                               extended(resize) lun
###                              INCREASE by 500m
###                              DECREASE by 200m
###                              use case for Burt 317575
###                              Steps invole (resize invole increase and decrease of clone lun)
###                              1) create snapshot
###                              2) create lun clone
###                              3) resize AFS clone
###                              4) create another snapshot
###                              5) create clone of clone (created in 2 and resized in 3).
###                              6) resize AFS clone of clone
###                              7) do splits
###
###   Restriction
###                              1) Lun size should at least bigger enough to do decrease 
###                              2) Need to have enough space in the volume to do lun clone(s)
###                                 Calculate lun size * max_clone_depth + some space in vol for snapshot
###                              3) Check the host conectivty before using the script
###                              4) if testing with Snapmirror, pass the source lun obj
###                                 if it is qsm, pass source lun object which reside in qtree.
###
###   Parameters:                - see the function
###   Returns:
###      on success              - 1
###      on failure              - 0
##########################################################################################################
sub lun_clone_resize() {
    my ( $self, %args ) = @_;

    $self->_clear_err();

    if ( ref( $self->{'sanxfun'} ) !~ /SanXfun/ ) {
        $self->_set_err("-sanxfun is not initialize. "
                   . "Instantiate UsecaseLunClone object before "
                   . "calling this method(lun_clone_resize)");
        return $SanUtil::FALSE;
    }

    my ( $lunObj, $clone_depth, $resize_flag, $sm_break, $test_io );

    $clone_depth = 3;
    $resize_flag = LUN_RESIZE_INCREASE;
    $sm_break = DONT_BREAK_SM_RELATIONSHIP;
#    $test_io = NO_IO_TEST;
    $test_io = IO_TEST;



    foreach ( keys %args ) {
        if (/^-?lun_obj$/) {
            $lunObj = $args{$_};
            if ( ref($lunObj) !~ /Lun/ ) {
                do {
                    $self->_set_err( "Invalid value for \"-lun_obj\": "
                                   . "$args{$_}, need a Lun reference" );
                    return $SanUtil::FALSE;
                };
            }
        }
        elsif (/^-?resize_flag$/) {
            length( $resize_flag = $args{$_} )
              or do {
                $self->_set_err( "Invalid value for \"-resize_flag\": "
                               . "$args{$_}" );
                return $SanUtil::FALSE;
              };
        }
        elsif (/^-?max_clone_depth$/) {
            length( $clone_depth = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-max_clone_depth\": "
                              . "$args{$_}" );
                return $SanUtil::FALSE;
              };
        }
        elsif (/^-?sm_break$/) {
            length( $sm_break = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-sm_break\": "
                              . "$args{$_}" );
                return $SanUtil::FALSE;
              };
        }
        elsif (/^-?test_io$/) {
            length( $test_io = $args{$_} )
              or do {
                $self->_set_err("Invalid value for \"-test_io\": "
                              . "$args{$_}" );
                return $SanUtil::FALSE;
              };
        }
        else {
            $self->_set_err( "Invalid parameter specified: $_" );
            return $SanUtil::FALSE;
        }
    }

    #
    # check for required parameter
    #
    $sm_break = ($sm_break =~ /1|yes|true|on/i ) ? 
                  BREAK_SM_RELATIONSHIP : DONT_BREAK_SM_RELATIONSHIP;

    $test_io = ($test_io =~ /1|yes|true|on/i ) ? 
                  IO_TEST : NO_IO_TEST;
    
    my $san_xfun_obj = $self->{'sanxfun'};


    # if lunObj is not specified then work with First Lun avaible
    my $volObj;
    my $lun_name;
    my $sanXfun_attr = $san_xfun_obj->get_sanxfun_attributes();
    my $sanXfun_env = $san_xfun_obj->get_sanxfun_environment();

    #
    # If user do not pass the lun object, we will take the first lun object in the list.
    #
    if ( ref($lunObj) !~ /Lun/ ) {
       # 
       # get the first filer name assume it is source 
       #
       my @lun_objs = $san_xfun_obj->get_sanxfun_lun_objs();
       if ( $san_xfun_obj->error() ) {
          my $errmsg = $san_xfun_obj->errmsg();
          $self->_set_err($errmsg);
          return $SanUtil::FALSE;
       }
       $lunObj = $lun_objs[0];
    }

    #
    # determine the volume object based on the given lun object
    #
    $volObj = $lunObj->get_parent();

    #
    # makse sure this lun is mapped, if not map it to igroup
    #

    my ( $ret_code, $ret_err );


    my $lun_name = $lunObj->get_name();
    my $igroup_name;
    $igroup_name = $lunObj->get_igroup()
      if ( !defined($igroup_name) or $igroup_name eq '' );

    #
    # write a known data on to the orignial lun
    #
    my $io_block_size = 4096;
    my $data_pattern  = "G";

    if( $test_io == IO_TEST) {
       #
       # make sure we have clean scsi device at host end(useful for windows host)
       #
       $san_xfun_obj->rescan_client_scsi_device();
       if ( $san_xfun_obj->error() ) {
           my $errmsg = $san_xfun_obj->errmsg();
           $self->_set_err($errmsg);
           return $SanUtil::FALSE;
       }
       $san_xfun_obj->write_data_to_lun(
           -lun_objs       => [$lunObj],
           -io_block_size  => $io_block_size,
           -data_pattern   => $data_pattern,
           -wait_flag      => IO_WAIT,
           -sio_subtest_id => "fill_org_lun_write_" . $lun_name
       );
       if ( $san_xfun_obj->error() ) {
           my $errmsg = $san_xfun_obj->errmsg();
           $self->_set_err($errmsg);
           return $SanUtil::FALSE;
       }
    }

    if ($sanXfun_env =~ /snapmirror/ and $sm_break == BREAK_SM_RELATIONSHIP) {
       $san_xfun_obj->san_sm_break();
    }

    #
    # we need to disable the snaphot auto schedule if any, since
    # this test case might take more time, if any auot snapshot create 
    # will be back by this snapshot and can not be removed, unitl remove the auto
    # schedule snapshot.
    #
    $volObj->options(nosnap => "on");
    if ( $volObj->error() ) {
        my $errmsg = $volObj->errmsg();
        $self->_set_err("Unable to set vol option nosnap to on: $errmsg");
        return $SanUtil::FALSE;
    }
    #
    # create lun clone form parent lun and verify the data
    #
    my $org_lunObj     = $lunObj;
    my @SioObj         = ();
    my @lcObjs         = ();
    my @snapshot_names = ();

    $ret_err = "";
    for ( my $i = 1 ; $i <= $clone_depth ; $i++ ) {

        #
        # create snapshot
        #
        my $snapname = "lcresize_sh_" . $i;

        # delete the snapshot if already exists
        $volObj->snap_delete( -name => $snapname );
        $self->sanlog( -msg => "creating snapshot - " . $snapname );
        if ( not $volObj->snap_create( -name => $snapname ) ) {
            $ret_err =
              "Failed to create a snapshot $snapname " . $volObj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }
        push( @snapshot_names, $snapname );

        #
        # create lun clone from parent lun
        #
        my $src_path     = $org_lunObj->get_path();
        my $src_lun_name = $org_lunObj->get_name();
        my $clone_path   = $src_path;
        my $lc_lun_name  = $src_lun_name . "_c" . $i;
        $clone_path =~ s/$src_lun_name/$lc_lun_name/;

        # make sure the lun clone is not exist, if so destroy it
        my $luncloneObj = $volObj->get_lun($lc_lun_name);
        if ( ref($luncloneObj) eq 'Lun' ) {
            $luncloneObj->destroy( -force => 1 );
        }

        # create lun clone
        my $luncloneObj = $org_lunObj->clone_create(
            -snap_name => $snapname,
            -path      => $clone_path
        );
        if ( $org_lunObj->error ) {
            my $msg = $org_lunObj->errmsg();
            $ret_err = "Failed to create the lun $clone_path:\nErorr: " . $msg;
            goto _test_lun_clone_resize_cleanup;
        }

        #
        # map the cloned lun
        #
        $luncloneObj->map($igroup_name);
        if ( $luncloneObj->error ) {
            $ret_err = "Failed to map igroup " . $luncloneObj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }

        # change the lun object to newly created clone object
        push( @lcObjs, $luncloneObj );
        $org_lunObj = $luncloneObj;
    }
    # 
    # get the Snap Usage 
    #
    foreach my $sh_name(@snapshot_names) {
       $self->_get_lun_snap_usage(-san_vol_obj => $volObj,
                                  -snap_name =>  $sh_name);
       if ( $self->error() ) {
           $ret_err = $self->errmsg();
           goto _test_lun_clone_resize_cleanup;
       }
    }

    if( $test_io == IO_TEST) {
       #
       # rescan the scsi device for the newly created clone lun(s)
       #
       $san_xfun_obj->rescan_client_scsi_device();
       if ( $san_xfun_obj->error() ) {
           $ret_err = $san_xfun_obj->errmsg();
           goto _test_lun_clone_resize_cleanup;
       }
       #
       # verify the data after creating lun clone
       #
        $ret_code = $san_xfun_obj->verify_data_on_lun(
            -lun_objs       => \@lcObjs,
            -io_block_size  => $io_block_size,
            -data_pattern   => $data_pattern,
            -wait_flag      => IO_WAIT,
            -sio_subtest_id => "verify_after_lun_clone"
        );
        if ( $san_xfun_obj->error() ) {
            $ret_err = $san_xfun_obj->errmsg();
            # need to stop the subtest if we started earlier,,, before cleanup....
            $san_xfun_obj->stop_sioObj(\@SioObj);
            goto _test_lun_clone_resize_cleanup;
        }
    }
    # 
    # if testing with SnapMirror and we did not break SM, 
    # update the destination so that lun clone is sync with source.
    #
    if ($sanXfun_env =~ /snapmirror/ and 
                  $sm_break == DONT_BREAK_SM_RELATIONSHIP) {
       $san_xfun_obj->san_sm_update();
       #
       # map the cloned lun on destination 
       #
       my $san_obj = $san_xfun_obj->get_filer($sanXfun_attr->{'dst_filer_name'});
       if ( $san_xfun_obj->error() ) {
           $ret_err = $san_xfun_obj->errmsg();
           goto _test_lun_clone_resize_cleanup;
       }
       my $vol_obj = $san_obj->get_vol($sanXfun_attr->{'dst_vol_name'});
       if ( $san_obj->error() ) {
           $ret_err = $san_obj->errmsg();
           goto _test_lun_clone_resize_cleanup;
       }
       #
       # refresh the destination volume object to get the update lun object(s)
       #
       $vol_obj->refresh();
       #
       # get lun object of source and with the same name
       # get the lun object of destination
       #
       foreach my $luncloneObj (@lcObjs) {
           if ( ref($luncloneObj) eq 'Lun' ) {
              my $lc_lun_name = $luncloneObj->get_name();
              my $dst_lun_obj = $vol_obj->get_lun($lc_lun_name);
              if ( $vol_obj->error() ) {
                  $ret_err = $vol_obj->errmsg();
                  goto _test_lun_clone_resize_cleanup;
              }
              # map it
              $dst_lun_obj->map($igroup_name);
              if ( $dst_lun_obj->error ) {
                  $ret_err = "Failed to map igroup " . $dst_lun_obj->errmsg();
                  goto _test_lun_clone_resize_cleanup;
              }
           }
       }
    }
    #
    # To make sure lun serial are unique on controller(s)
    # after creating lun clone and possible with snapmirror update
    #
    $san_xfun_obj->verify_lun_unique_serial();
    if ( $san_xfun_obj->error() ) {
        $ret_err = $san_xfun_obj->errmsg();
        goto _test_lun_clone_resize_cleanup;
    }
    #
    # resize the lun clone in AFS
    #
    $ret_err = "";

    # get lun geometry
    my $geo = $lunObj->get_geometry();
    if ( $lunObj->error() ) {
        $ret_err = $lunObj->errmsg();
        goto _test_lun_clone_resize_cleanup;
    }

    #
    # $geo->{device} is 'device size', need to change the geomentry function.
    my $lun_resize_to_size = $geo->{device};

    for ( my $i = 0 ; $i < @lcObjs ; $i++ ) {
        my $luncloneObj = $lcObjs[$i];
        my $lc_lun_name = $luncloneObj->get_name();
        my $clone_path  = $luncloneObj->get_path();

        my $geo = $luncloneObj->get_geometry();
        if ( $luncloneObj->error() ) {
            $ret_err = "$lc_lun_name : " . $luncloneObj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }
        if ( $resize_flag == LUN_RESIZE_INCREASE ) {
            $lun_resize_to_size =
              $lun_resize_to_size + $san_xfun_obj->get_byte_size("500m");
            $self->sanlog( -msg => 
                "lun clone($lc_lun_name) size before resize=$geo->{device},"
              . " requested byte size to increase = $lun_resize_to_size" );
        }
        else {
            $lun_resize_to_size =
              $lun_resize_to_size - $san_xfun_obj->get_byte_size("200m");
            $self->sanlog( -msg =>
                "lun clone($lc_lun_name) size before resize=$geo->{device},"
              . " requested byte size to descrease = $lun_resize_to_size");
        }

        #
        # resize the AFS clone lun
        #
        $luncloneObj->resize( -size => $lun_resize_to_size, -force => 'yes' );
        if ( $luncloneObj->error ) {
            $ret_err = $luncloneObj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }
        $ret_err = "";
        $volObj->refresh();
        $luncloneObj = $volObj->get_lun($lc_lun_name);
        $lcObjs[$i] = $luncloneObj;
        my $lun_size = $luncloneObj->get_size();
        my $lc_geo   = $luncloneObj->get_geometry();
        if ( $luncloneObj->error() ) {
            $ret_err = "$lc_lun_name : " . $luncloneObj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }
        $self->sanlog( -msg => "lun clone($lc_lun_name) size after "
                             . "resize=$lun_size($lc_geo->{device})" );
    }

    # 
    # if testing with SnapMirror and we did not break SM, 
    # update the destination so that lun clone resize is sync with source.
    #
    if ($sanXfun_env =~ /snapmirror/ and 
                  $sm_break == DONT_BREAK_SM_RELATIONSHIP) {
       $san_xfun_obj->san_sm_update();
    }

    #
    # rescan the scsi device to make sure new lun size(resized) is detected
    #
    $san_xfun_obj->rescan_client_scsi_device();
    if ( $san_xfun_obj->error() ) {
        my $errmsg = $san_xfun_obj->errmsg();
        $ret_err =  "Failed to update the lun clone devices: $errmsg";
        goto _test_lun_clone_resize_cleanup;
    }
    #
    # Solaris is not detecting the new size, method to update partition infomation
    # see burt:344816 for more information. This function could corrupt the data
    # since we dont bother about data it is ok.
    #
    $san_xfun_obj->update_device_geometry( -lun_objs => \@lcObjs );
    if ($san_xfun_obj->error()) {
        my $errmsg = $san_xfun_obj->errmsg();
        $ret_err =  "Failed to update the lun clone devices: $errmsg";
        goto _test_lun_clone_resize_cleanup;
    }
    #
    # after resize the clone lun, do partial write 
    # (i.e., less than 4k wafl size)
    #
    $ret_err = "";
    @SioObj  = ();
    foreach my $luncloneObj (@lcObjs) {
        my $lc_lun_name = $luncloneObj->get_name();

        #
        # after resize the clone lun, 
        # do partial write (i.e., less than 4k wafl size)
        #
        $io_block_size = 512;
        $data_pattern  = "M";

        $self->sanlog( -msg => "Writing partial block after resize......" );

        $ret_code = $san_xfun_obj->write_data_to_lun(
            -lun_objs       => [$luncloneObj],
            -io_block_size  => $io_block_size,
            -data_pattern   => $data_pattern,
            -wait_flag      => IO_NO_WAIT,
            -sio_subtest_id => "lun_resize_write_" . $lc_lun_name
        );
        if ( $san_xfun_obj->error() ) {
            $ret_err = $san_xfun_obj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }
        else {
            push( @SioObj, $ret_code );
        }
    }

    #
    # wait for all the IO object to complete
    #
    $san_xfun_obj->wait_verify_sioObj( \@SioObj );
    if ( $san_xfun_obj->error() ) {
        $ret_err = $san_xfun_obj->errmsg();
        goto _test_lun_clone_resize_cleanup;
    }

    # 
    # if testing with SnapMirror and we did not break SM, 
    # update the destination so that partial write is sync with source.
    #
    if ($sanXfun_env =~ /snapmirror/ and 
                  $sm_break == DONT_BREAK_SM_RELATIONSHIP) {
       $san_xfun_obj->san_sm_update();
    }

    #
    # split the clone and verify the data interity
    #
    $ret_err = "";
    foreach my $luncloneObj (@lcObjs) {

        # Split the clone
        $luncloneObj->clone_split_start();
        if ( $luncloneObj->error() ) {
            $ret_err = $luncloneObj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }

        # wait for split to complete
        $luncloneObj->clone_split_wait();
        if ( $luncloneObj->error() ) {
            $ret_err = $luncloneObj->errmsg();
            goto _test_lun_clone_resize_cleanup;
        }
    }

    # 
    # if testing with SnapMirror and we did not break SM, 
    # update the destination so that lun clone resize is synced with source.
    #
    if ($sanXfun_env =~ /snapmirror/ and 
                  $sm_break == DONT_BREAK_SM_RELATIONSHIP) {
       $san_xfun_obj->san_sm_update();
    }

    if( $test_io == IO_TEST) {
       #
       # check data integrity, after lun clone split
       #
       $ret_err = "";
       @SioObj  = ();
       foreach my $luncloneObj (@lcObjs) {
           my $lc_lun_name = $luncloneObj->get_name();
           $io_block_size = 512;
           $data_pattern  = "M";
   
           $self->sanlog(
               -msg => "Verifying the data after split of the lun clone......" );
   
           $ret_code = $san_xfun_obj->verify_data_on_lun(
               -lun_objs       => [$luncloneObj],
               -io_block_size  => $io_block_size,
               -data_pattern   => $data_pattern,
               -wait_flag      => IO_NO_WAIT,
               -sio_subtest_id => "lun_clone_split_verify_" . $lc_lun_name
           );
           if ( $san_xfun_obj->error() ) {
               $ret_err = $san_xfun_obj->errmsg();
               goto _test_lun_clone_resize_cleanup;
           }
           else {
               push( @SioObj, $ret_code );
           }
       }
   
       #
       # wait for all the IO object completes
       #
       $ret_err = "";
       $san_xfun_obj->wait_verify_sioObj( \@SioObj );
       if ( $san_xfun_obj->error() ) {
           $ret_err = $san_xfun_obj->errmsg();
       }
    }
  _test_lun_clone_resize_cleanup:

     # 
     # before cleanup get the Snap Usage, to make sure
     # none of the auto snap schedule snapshot backing these snapshot
     #
     foreach my $sh_name(@snapshot_names) {
        $self->_get_lun_snap_usage(-san_vol_obj => $volObj,
                                   -snap_name =>  $sh_name);
     }

    #
    # Cleanup: remove snapshot and the lun clone
    #
    # remove the clone luns first
    foreach my $luncloneObj (@lcObjs) {
        if ( ref($luncloneObj) eq 'Lun' ) {
            $luncloneObj->destroy( -force => 1 );
        }
    }

    # remove in reverse order
    my $noofsnaps = @snapshot_names;
    for ( my $i = 0 ; $i < $noofsnaps ; $i++ ) {
        my $snapname = pop(@snapshot_names);

        # remove the snapshot, now it can be removed
        $volObj->snap_delete( -name => $snapname );
    }
    # set vol option nosnap to off.
    $volObj->options(nosnap => "off");

    # 
    # if testing with SnapMirror and we did not break SM, 
    # update the destination. to make sure the status of testSuite
    # is retian after this use case.
    #
   if ($sanXfun_env =~ /snapmirror/) {
      if( $sm_break == DONT_BREAK_SM_RELATIONSHIP) {
         $san_xfun_obj->san_sm_update();
      }elsif($sm_break == BREAK_SM_RELATIONSHIP){
         $san_xfun_obj->san_sm_resync();
      }
      #
      # need to refresh the destination volume object 
      # since it might hold the lun clone objects 
      #
      my $san_obj = $san_xfun_obj->get_filer($sanXfun_attr->{'dst_filer_name'});
      if ( !$san_xfun_obj->error() ) {
         my $vol_obj = $san_obj->get_vol($sanXfun_attr->{'dst_vol_name'});
         if ( !$san_obj->error() ) {
            $vol_obj->refresh();
         }
      }
   }
    
    if ( $ret_err ne "" ) {
        $self->_set_err($ret_err);
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
}
##########################################################################################################
###   Function:                  - lun_clone_recycle_snap
###                               test when max snap created and recyle it should not panic
###                              use case for Burt 349265
###                              - vol options vol1 snapshot_clone_dependency on
###                              - lun create -s 4m -t linux /vol/vol1/lun1
###                              - snap create vol1 snap1 
###                              - lun clone create /vol/vol1/clone -b /vol/vol1/lun1 snap1
###                              - snap create vol1 snap2
###                              - lun destroy /vol/vol1/clone
###                              - snap delete vol1 snap1
###                              
###                              Now create and destroy 253 snapshots to recycle snap id 1. After this:
###                              
###                              - lun clone create /vol/vol1/clone -b /vol/vol1/lun1 snap2
###                              - snap create vol1 snap1
###                              - vol clone create vol2 -b vol1  (if snapmirror update teh destination)
###                              
###                              After following these steps, the system shouldn't panic.
###
###   Restriction
###                              1) Volume should have enough sapce to create snapshot
###
###   Parameters:                - see the function
###   Returns:
###      on success              - 1
###      on failure              - 0
##########################################################################################################
sub lun_clone_recycle_snap() {
    my ( $self, %args ) = @_;

    $self->_clear_err();

    if ( ref( $self->{'sanxfun'} ) !~ /SanXfun/ ) {
        $self->_set_err("-sanxfun is not initialize. "
                   . "Instantiate UsecaseLunClone object before "
                   . "calling this method(lun_clone_resize)");
        return $SanUtil::FALSE;
    }
    return $SanUtil::TRUE;
}

##########################################################################################################
#	                               P R I V A T E   M E T H O D S                                     #
##########################################################################################################
sub _get_lun_snap_usage()
{
   my ($self, %args) = @_;

   my $san_vol_obj;
   my $volume_name;
   my $snap_name;
   my $return_output = 0; # 1 - to get the actuall output for the command: 0 - to get the hash output

   foreach (keys %args) 
   {
       if (/^-?san_vol_obj$/) 
       {
          $san_vol_obj = $args{$_};
           if (not length($san_vol_obj) or ref($san_vol_obj) !~ /Volume/) 
           {
              $self->_set_err("Invalid value for argument '-san_vol_obj'");
              return $SanUtil::FALSE;
           }
       } 
       elsif (/^-?snap_name$/) 
       {
           if (not length($snap_name= $args{$_})) 
           {
              $self->_set_err("Invalid value for argument '-snap_name'");
              return $SanUtil::FALSE;
           }
       }
       elsif (/^-?return_output$/) 
       {
           if (not length($return_output= $args{$_})) 
           {
              $self->_set_err("Invalid value for argument '-return_output'");
              return $SanUtil::FALSE;
           }
       }
       else 
       {
           $self->_set_err("Invalid parameter specified \'$_\'");
           return $SanUtil::FALSE;
       }
   } # end foreach

   my $resultsHash_ref;
   my $cmdObj;

   if( ref($san_vol_obj) !~ /Volume/){
      $self->_set_err("Need SanFiler reference for argument '-san_vol_obj'");
      return $SanUtil::FALSE;
   }

   if( not length ($snap_name)){
      $self->_set_err("Need snapshot name for argument '-snap_name'");
      return $SanUtil::FALSE;
   }

   my $cmdObj = $san_vol_obj->get_parent()->cmd_obj();
   my $interface = $san_vol_obj->get_parent()->interface();
   $volume_name = $san_vol_obj->get_name();

   $resultsHash_ref = $cmdObj->execute_cmd(
                        -cmd             => "lun-snap-usage-list-info",
                        -volume          => $volume_name,
                        -snapshot        => $snap_name,
                        -check_output    => "yes",
                        -check_result    => "yes",
                        );

   if($cmdObj->error()) {
       my $errmsg = $cmdObj->errmsg();
       $self->_set_err("cmd driver execution error: $errmsg");
       return $SanUtil::FALSE;
   }
   #
   # Get the output
   #
   my $output = $resultsHash_ref->{output};

   #
   # Since we checked the result, lets see if our check was OK
   #
   if($resultsHash_ref->{result_check} == 0) {
      $self->_set_err("cmd driver result check failed: $output");
      return $SanUtil::FALSE;
   }
   #
   # if return output is set, return the command output.
   #
   if ($return_output == 1) {
      return ($output);
   }

   my $lc_hash = {};
   
   if($interface eq "zapi")
   {
      my $output_hash = $san_vol_obj->get_parent()->xml2hash($output);
      # my $dumpOut = Dumper($output_hash);
      # $self->sanlog(-msg      => "Zapi(lun-snap-usage-list-info) Output Hash is:\n$dumpOut", msg_level => 7);
      my $status = $output_hash->{status};
      if($status eq "failed")
      {
          my $errmsg=$output_hash->{reason};
          $self->_set_err("Unable to get lun snap usage status: $errmsg");
          return $SanUtil::FALSE;
      }

      my $entry_hash;
      if(ref($output_hash->{'lun-snap-usage-luns'}->{'lun-snap-usage-lun-info'}) ne "ARRAY")
      {
          $entry_hash = $output_hash->{'lun-snap-usage-luns'}->{'lun-snap-usage-lun-info'};
          my $id;
          if( $entry_hash->{'snapshot'} eq "Active"){
             $id = $snap_name;
          } else {
             $id = $entry_hash->{'snapshot'};
          }
          $lc_hash->{$id} = $entry_hash;
      } else{
         foreach $entry_hash (@{$output_hash->{'lun-snap-usage-luns'}->{'lun-snap-usage-lun-info'}}) {
            my $id;
            if( $entry_hash->{'snapshot'} eq "Active"){
               $id = $snap_name;
            } else {
               $id = $entry_hash->{'snapshot'};
            }
            $lc_hash->{$id} = $entry_hash;
         }
      }
   } else {
      my @out = split('\n', $output);
      my $id;
      my $value;

      foreach my $line (@out) 
      {
          if($line =~ /Active:/)
          {
             $id = $snap_name;
             $lc_hash->{$id}->{'snapshot'} = "Active";
          }
          elsif($line =~ /Snapshot/)
          {
              (undef, $id) = split(/- \s*/, $line);
              $lc_hash->{$id}->{'snapshot'} = $id;
          }
          elsif($line =~ /LUN:/)
          {
              (undef, $value) = split(/:\s*/, $line);
              $lc_hash->{$id}->{'path'} = $value;
          }
          elsif($line =~ /Backed By:/)
          {
              (undef, $value) = split(/:\s*/ , $line);
              $lc_hash->{$id}->{'backing-store'} = $value;
          }
      }
   }
   return ($lc_hash);
}


1;

