#$Id: //depot/prod/test/Rsierranevada/lib/San/ScsiTool/SanBlaze.pm#1 $

## @summary Provide an interface to the IESB tool
## @author evan@netapp.com
## @status public
## @pod ../ScsiTool.pm


#
# Package Name
#
package San::ScsiTool::SanBlaze;

use strict;

#
# Package Requirements
#
use San::ScsiTool;
use San::SanClient::ClientUtil;

use Error;
use San::SanLog;

use vars qw(@ISA $ScsiTool);

@ISA = qw(Error SanLog ScsiTool);

########################################################################
# P U B L I C   M E T H O D S
########################################################################

########################################################################
# Subroutine Name: send_cmd()
# Objective:       Execute the SanBlaze tool to send a SCSI Command
# Details:         See POD documentation in ScsiTool.pm
########################################################################
sub send_cmd
{
    my ($self, %args) = @_;
    my ($sc_ref, $er_ref, $client_conn);
    my (@cdb_bytes);
    my ($port, $filename, $cmdname, @cmds, $status, @output);
    my (@scsi_data, $asc, $ascq);

    $self->_clear_err();

    $self->handle_send_cmd_params(%args);
    return $self if($self->error);
    
    $self->build_exp_rsp_hash();
    return $self if($self->error);

    $sc_ref = $self->get_send_cmd_params();
    $er_ref = $self->get_exp_rsp_params();

    # Set error handling to stop on error so we see the error if any
    $port = $sc_ref->{DEVICE};
    $port =~ s/target\d+lun\d+/port/;
    push (@cmds, "echo 'OnError=StopTest' > $port");

    # Build the CDB on the device and execute it
    $cmdname = "nate$$";
    if ($sc_ref->{DIR} == ScsiTool::READ) {
        $filename = "/virtualun/commands/usercdb/read/$cmdname";
        push (@cmds, "echo '//Write=0' > $filename");
    } else {
        $filename = "/virtualun/commands/usercdb/write/$cmdname";
        push (@cmds, "echo '//Write=1' > $filename");
    }
    
    @cdb_bytes = split(/\s+/, $sc_ref->{CDB});
    push (@cmds, "echo '//CDBLen=" . scalar(@cdb_bytes) . "' >> $filename");
    push (@cmds, "echo '//Sequential=0' >> $filename");

    foreach my $byte (@cdb_bytes) {
        push (@cmds, "echo '$byte' >> $filename");
    }
    push (@cmds, "cat $filename");

    my $iocmd = "/usr/bin/io $sc_ref->{DEVICE} $cmdname -txlen $sc_ref->{BUF_LEN} -count $sc_ref->{REPEAT_CNT} -a $self->{initiator}";
    if ($self->{verbose}) {
	$iocmd .= " -v";
    }
    push (@cmds, $iocmd);
    
    my $client_conn = $self->{client}->get_connection();

    foreach my $command (@cmds) {
        ($status, @output) = &ClientUtil::execute($client_conn, $command, $sc_ref->{REPEAT_CNT} < 600 ? undef : $sc_ref->{REPEAT_CNT});
        if ($status > 0) {
            $self->_set_err("Error: ScsiTool::send_cmd(): " .
                            "Command ($command) returned a bad status ($status).");
        }
    }
    # Save command status into the response hash
    $er_ref->{HOST_STATUS} = $status;

    # We no longer need the CDB file so let's remove it
    ($status, undef) = &ClientUtil::execute($client_conn, "rm -f $filename");
    if ($status > 0) {
        $self->_set_err("Error: ScsiTool::send_cmd(): " .
                        "Command (rm -f $filename) returned a bad status ($status).");
    }
    $self->sanlog(-msg => "Client hostname command returned " .
                          join("\n", @output));

    # Format output into response hash
    $er_ref->{TOOLNAME} = "sanblaze";

    # Successes look like this:
    #Issuing CDB=c0 00 02 0a 98 0a ff 02 00 00
    #Allocate read buffer of 200 bytes
    #0000000000000000: 00 00 01 f4 0a 98 0a 02 00 00 00 0c 69 67 6e 61                igna
    #0000000000000010: 72 00 00 00 01 00 00 0c 00 00 00 01 0a 3c 97 d3    r            <
    #0000000000000020: 10 00 00 24 2f 76 6f 6c 2f 73 61 6e 5f 76 6f 6c       $/vol/san_vol
    #0000000000000030: 31 2f 72 74 70 2d 73 61 6e 62 6c 61 7a 65 5f 6c    1/rtp-sanblaze_l
    #0000000000000040: 75 6e 31 00 11 00 00 14 07 79 11 5d 73 61 6e 5f    un1      y ]san_
    #0000000000000050: 76 6f 6c 31 00 00 00 00 12 00 00 18 00 00 00 00    vol1
    #0000000000000060: 2f 76 6f 6c 2f 73 61 6e 5f 76 6f 6c 31 00 00 00    /vol/san_vol1
    #0000000000000070: 13 00 00 08 00 00 00 00 14 00 00 08 00 00 00 00
    #0000000000000080: 15 00 00 1c 00 00 00 02 80 00 00 00 00 00 00 00
    #0000000000000090: 00 00 00 00 00 00 00 00 00 00 00 00 16 00 00 1c
    #00000000000000a0: 00 00 00 01 72 74 70 2d 73 61 6e 62 6c 61 7a 65        rtp-sanblaze
    #00000000000000b0: 5f 66 63 70 00 00 00 00 17 00 00 08 00 00 00 00    _fcp
    #00000000000000c0: 18 00 00 0c 6c 69 6e 75 78 00 00 00 20 01 00 18        linux
    #00000000000000d0: 00 00 00 07 50 0a 09 80 8a 22 ad c2 50 0a 09 81        P    "  P
    #00000000000000e0: 9a 22 ad c2 21 00 00 0c 76 2e 30 61 00 00 00 00     "  !   v.0a
    #00000000000000f0: 24 00 00 10 00 00 01 00 76 74 69 63 00 00 00 00    $       vtic
    #0000000000000100: 25 00 00 08 00 00 00 08 22 00 00 4c 20 2f 78 2f    %       "  L /x/
    #0000000000000110: 65 6e 67 2f 72 74 70 62 6c 64 30 33 2f 62 75 69    eng/rtpbld03/bui
    #0000000000000120: 6c 64 2f 00 30 30 34 31 30 37 30 30 31 38 00 00    ld/ 0041070018
    #0000000000000130: 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
    #0000000000000140: 46 41 53 32 30 35 30 00 00 00 00 00 00 00 00 7d    FAS2050        }
    #0000000000000150: 00 00 00 00 23 03 00 44 02 00 10 00 30 30 34 31        #  D    0041
    #0000000000000160: 30 37 30 30 31 38 00 00 00 00 00 00 00 00 00 00    070018
    #0000000000000170: 00 00 00 00 00 00 00 00 46 41 53 32 30 35 30 00            FAS2050
    #0000000000000180: 00 00 00 00 00 00 00 02 50 0a 09 80 8a 22 ad c2            P    "
    #0000000000000190: 50 0a 09 81 9a 22 ad c2 26 00 00 18 05 00 01 01    P    "  &
    #00000000000001a0: 01 01 01 01 00 00 00 00 00 00 00 00 00 00 01 eb
    #00000000000001b0: 27 00 00 10 01 01 00 00 00 00 00 01 6d 6f 6d 00    '           mom
    #00000000000001c0: 32 00 00 34 00 00 01 00 00 00 00 65 35 69 65 53    2  4       e5ieS
    #00000000000001d0: 5a 48 6b 46 59 2d 59 00 00 00 00 00 00 00 00 78    ZHkFY-Y        x
    #00000000000001e0: 6e 06 48 50 85 20 2d 00 00 00 00 00 00 00 00 00    n HP  -
    #00000000000001f0: 00 00 00 00
    #Successful Read

    # Failures look like this:
    #Issuing CDB=00 00 00 00 00 00
    #Allocate read buffer of 200 bytes
    #TestUnitReady failed
    #TestUnitReady[0x 0] 1 blocks at lba=0 b1:t4:l0 failed.  SCSIStatus=02 CheckCondition.  IOCStatus=00 Success.
    #
    #Byte[ 0] Response Code=f0
    #Byte[ 2]           Key=06       Unit Attention
    #Byte[ c]           ASC=29
    #Byte[ d]          ASCQ=00
    # 29/00  DTLPWROMAEBKVF  POWER ON, RESET, OR BUS DEVICE RESET OCCURRED
    #
    #
    #Raw Sense Data:
    #f0  0  6  0  0  0  0  e  0  0  0  0 29  0  0  0  0  0

    for (my $i=0; $i<@output; $i++) {
        if ($output[$i] =~ /^\d+:/) {
            push (@scsi_data, $output[$i]);
        }
        if ($output[$i] =~ /Successful/i) {
            $er_ref->{SCSI_STATUS} = 0;
            $er_ref->{IOCTL_STATUS} = 0;
        }
        if ($output[$i] =~ /SCSIStatus=(\d+)/i) {
            $er_ref->{SCSI_STATUS} = hex($1);
        }
        if ($output[$i] =~ /IOCStatus=(\d+)/i) {
            $er_ref->{IOCTL_STATUS} = hex($1);
        } 
        if ($output[$i] =~ /Key=(\d+)/i) {
            $er_ref->{SENSE_KEY} = hex($1);
        } 
        if ($output[$i] =~ /ASC=(\d+)/i) {
            $asc = $1;
        } 
        if ($output[$i] =~ /ASCQ=(\d+)/i) {
            $ascq = $1;
        } 
        if ($output[$i] =~ /Raw Sense Data:/i) {
            $er_ref->{SENSE_DATA} = $output[$i+1];
        }
    }
    $er_ref->{SCSI_DATA} = \@scsi_data if ($er_ref->{SCSI_STATUS} == 0);
    $er_ref->{ASCNQ} = "${asc}h/${ascq}h" if ($er_ref->{SCSI_STATUS} != 0);

    #$er_ref->{TOOL_LIB_STATUS}
    #$er_ref->{RESIDUAL_BYTES}
    #$er_ref->{RESID_OVER}
    #$er_ref->{RESID_UNDER}
    #$er_ref->{BYTES_XFERD}

    #
    # before we send the response hash back to the testcase
    # make sure that all the required params are were not
    # updated are set to undef
    #
    $self->handle_unsup_responses();

    #
    # now we are ready to reply back to the testcase.
    #
    return $self->get_exp_rsp_params();
} # end send_cmd()

sub verbose {
    my ($self, $value) = @_;
    if (defined($value)) {
	$self->{verbose} = $value;
    }

    return $self->{verbose};
}

sub initiator {
    my ($self, $value) = @_;
    if (defined($value) && $value <= $self->{aliases}) {
	$self->{initiator} = $value;
    }

    return $self->{initiator};
}

########################################################################
# P R I V A T E   M E T H O D S
########################################################################

########################################################################
# Subroutine Name: _new()
# Objective:       Create a new sanlun object
# Details:         See POD documentation at the end of this file
########################################################################
sub _new
{
    my ($caller, %args) = @_;
    my $class = ref($caller) || $caller;
    my $self = $class->_base_new(%args);

    $self->{verbose} = 1;
    $self->{initiator} = 0;
    return $self;

} # end _new()

1;
