#$Id: //depot/prod/test/Rsierranevada/lib/San/ScsiTool/ScsiPR.pm#1 $
# Copyright (c) 2009 NetApp Inc.
# All rights reserved.
#
## @summary Provide helper routines for persistent reservations
## @author dsharma@netapp.com

package San::ScsiTool::ScsiPR;

use strict;
use warnings;
use San::ScsiTool::CDB;
use Data::Dumper;
use San::SanLog;
use Error;
require Exporter;

use vars qw(@ISA);

#
# Inherit from the Error, SanLog classes
#
our @ISA = qw(Error SanLog Exporter);


our @EXPORT = qw(SCSI_PR_IN_READ_KEYS
		 SCSI_PR_IN_READ_RESERVATION
		 SCSI_PR_IN_REPORT_CAP
		 SCSI_PR_OUT_REGISTER
		 SCSI_PR_OUT_RESERVE
		 SCSI_PR_OUT_RELEASE
		 SCSI_PR_OUT_CLEAR
		 SCSI_PR_OUT_PREEMPT
		 SCSI_PR_OUT_PREEMPT_ABORT
		 SCSI_PR_OUT_REGISTER_IGNORE
		 SCSI_PR_OUT_REGISTER_MOVE
		 SCSI_PR_OUT_READ_SHARED
		 SCSI_PR_OUT_WRITE_EXCL
		 SCSI_PR_OUT_READ_EXCL
		 SCSI_PR_OUT_EXCL_ACCESS
		 SCSI_PR_OUT_SHARED_ACCESS
		 SCSI_PR_OUT_WRITE_EXCL_REG
		 SCSI_PR_OUT_EXCL_ACCESS_REG
		 SCSI_PR_OUT_WRITE_EXCL_ALL
		 SCSI_PR_OUT_EXCL_ACCESS_ALL
		 SCSI_PR_OUT_CDB_LENGTH
		 UNIT_ATTENTION
		 CHECK_CONDITION
		 RESERVATION_CONFLICT
		 ILLEGAL_REQUEST
		 NOT_READY
	   );


use constant {SCSI_PR_IN_READ_KEYS        => 0,
	      SCSI_PR_IN_READ_RESERVATION => 1,
	      SCSI_PR_IN_REPORT_CAP       => 2,
	      SCSI_PR_OUT_REGISTER        => 0,
              SCSI_PR_OUT_RESERVE         => 1,
	      SCSI_PR_OUT_RELEASE         => 2,
	      SCSI_PR_OUT_CLEAR           => 3,
	      SCSI_PR_OUT_PREEMPT         => 4,
              SCSI_PR_OUT_PREEMPT_ABORT   => 5,
	      SCSI_PR_OUT_REGISTER_IGNORE => 6,
	      SCSI_PR_OUT_REGISTER_MOVE   => 7,
	      SCSI_PR_OUT_READ_SHARED     => 0,
              SCSI_PR_OUT_WRITE_EXCL      => 1,
	      SCSI_PR_OUT_READ_EXCL       => 2,
	      SCSI_PR_OUT_EXCL_ACCESS     => 3,
	      SCSI_PR_OUT_SHARED_ACCESS   => 4,
              SCSI_PR_OUT_WRITE_EXCL_REG  => 5,
	      SCSI_PR_OUT_EXCL_ACCESS_REG => 6,
	      SCSI_PR_OUT_WRITE_EXCL_ALL  => 7,
	      SCSI_PR_OUT_EXCL_ACCESS_ALL => 8,
              SCSI_PR_OUT_CDB_LENGTH      => 10,
	      UNIT_ATTENTION              => 6,
	      CHECK_CONDITION             => 2,
	      RESERVATION_CONFLICT        => 18,
	      ILLEGAL_REQUEST             => 5,
	      NOT_READY			  => 2,};

sub new {
    my ($caller) = @_;
    my $class = ref($caller) || $caller;

    #
    # Create a hash to store attributes and bless the hash to create an object.
    #
    my $self = {};
    bless $self, $class;

    # add the scsi tool obj debug level.
    #
    $self->set_debug_level(3);

    return $self;
}

####################	scsi_pr_in			    ###################
## @step Objective: Execute the PR IN SCSI command
###############################################################################

sub scsi_pr_in {

    my ($self, %args) = @_;
    my ($scsi_tool_obj, $scsi_util_obj, $device, $type, $expected, $sense_key,
        $ascnq, $buf_len);

    $self->_clear_err();

    my $result = 1;

    $sense_key = "undef";
    $ascnq     = "undef";
    $buf_len   = 1024;

    foreach my $param (keys %args) {
        if ($param =~ /^-?util$/) {    # Utility object
            $scsi_util_obj = $args{$param};
            unless ($scsi_util_obj) {
                $self->_set_err(  "set_pr_in(): "
                                . "Invalid value passed for the '-util' "
                                . "argument: $args{$param}");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?device$/) {    # Device ITL
            $device = $args{$param};
            unless ($device) {
                $self->_set_err(  "set_pr_in(): "
                                . "Invalid value passed for the '-device' "
                                . "argument: $args{$param}");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?pr_in_type$/) {    # PR type
            $type = $args{$param};
            if ($type !~ /^[0-3]/) {
                $self->_set_err(  "set_pr_in(): "
                                . "Invalid value passed for the '-pr_in_type' "
                                . "argument: $param");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?expected_result$/) {    # Expected return value
            $expected = $args{$param};
            unless ($expected) {
                $self->_set_err("set_pr_in(): "
                            . "Invalid value passed for the '-expected_result' "
                            . "argument: $param");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?buf_len$/) {            # Buffer Length
            $buf_len = $args{$param};
            unless (length($buf_len)) {
                $self->_set_err(  "set_pr_in(): "
                                . "Invalid value passed for the '-buf_len' "
                                . "argument: $param");
                return $self;
            }

            next;
        } else {
            $self->_set_err("set_pr_in(): " . "Invalid Argument");
            return $self;
        }
    }

    unless ($device) {
        $self->_set_err(  "set_pr_in(): "
                        . "Invalid value passed for the '-device' "
                        . "argument");
        return $self;
    }

    unless ($scsi_util_obj) {
        $self->_set_err(  "set_pr_in(): "
                        . "Invalid value passed for the '-util' "
                        . "argument");
        return $self;
    }

    if ($type == SCSI_PR_IN_READ_KEYS) {
        $result =
          $self->_scsi_pr_in_read_keys($scsi_util_obj, $device,
                                       $expected,      $buf_len);
    } elsif ($type == SCSI_PR_IN_READ_RESERVATION) {
        $result =
          $self->_scsi_pr_in_read_reservation($scsi_util_obj, $device,
                                              $expected, $buf_len);
    } elsif ($type == SCSI_PR_IN_REPORT_CAP) {
        $result =
          $self->_scsi_pr_in_report_capabilities($scsi_util_obj, $device,
                                                 $expected, $buf_len);
    }

    return $result;
}

####################	_scsi_pr_in_read_keys   	        ###################
## @step Objective: Execute the PR IN SCSI command
## Input parameters: $device - ITL nexus information over which the command has to
## be executed
###################################################################################

sub _scsi_pr_in_read_keys {
    my ($self, $scsi_util_obj, $device, $expected, $buf_len) = @_;
    my $result = 1;
    my $errmsg;
    my $cdb = "scsi_pr_in_read_keys";
    my $h_ref;

    $self->sanlog(-msg       => "PR IN: Read Keys",
                  -msg_level => 3);

    my $scsi_tool_obj = $scsi_util_obj->{s_t_obj};

    my $return =
      $scsi_tool_obj->test_scsi_cmd(-device      => $device,
                                    -cdb_name    => $cdb,
                                    -buf_len     => $buf_len,
                                    -scsi_status => $expected,);

    if ($scsi_tool_obj->error) {
        $self->_set_err($scsi_tool_obj->errmsg());
        return $return;
    }

    if ($return->{SCSI_DATA}) {
        $h_ref =
          $scsi_util_obj->scsi_bytes_to_strings(
                                         -scsi_cmd_name => $cdb,
                                         -scsi_data_ref => $return->{SCSI_DATA},
          );

        if ($scsi_util_obj->error) {
            $self->_set_err($scsi_util_obj->errmsg());
            return $return;
        } else {
            $self->sanlog(-msg       => "Success:",
                          -msg_level => 3);
        }

        # Display the scsi data
        $self->_display_data($h_ref);

        return $h_ref;
    }

    return $return;
}

#####################		_display_data			#################
# Params:  $href: hash values - scsi data to be displayed
# Returns: None
#################################################################################

sub _display_data {
    my ($self, $h_ref) = @_;

    $self->sanlog(-msg       => "GENERATION     : $h_ref->{GENERATION}",
                  -msg_level => 3);
    $self->sanlog(-msg       => "ADD_LENGTH     : $h_ref->{ADD_LENGTH}",
                  -msg_level => 3);

    my @key = @{$h_ref->{KEYS}};

    foreach my $key (@key) {
        $self->sanlog(-msg       => "Registered Keys     : $key",
                      -msg_level => 3);
    }
}

####################      _scsi_pr_in_read_reservation          ###################
## @step Objective: Execute the PR IN SCSI command
## Input parameters: $device - ITL nexus information over which the command has to
## be executed
###################################################################################

sub _scsi_pr_in_read_reservation {
    my ($self, $scsi_util_obj, $device, $expected, $buf_len) = @_;
    my $result = 1;
    my $errmsg;
    my $cdb = "scsi_pr_in_read_resv";

    $self->sanlog(-msg       => "PR IN: Read Reservation",
                  -msg_level => 3);

    my $scsi_tool_obj = $scsi_util_obj->{s_t_obj};

    my $return =
      $scsi_tool_obj->test_scsi_cmd(-device      => $device,
                                    -cdb_name    => $cdb,
                                    -buf_len     => $buf_len,
                                    -scsi_status => $expected,);

    if ($scsi_tool_obj->error) {
        $self->_set_err($scsi_tool_obj->errmsg());
        return $return;
    }

    my $h_ref =
      $scsi_util_obj->scsi_bytes_to_strings(
                                         -scsi_cmd_name => $cdb,
                                         -scsi_data_ref => $return->{SCSI_DATA},
      );

    if ($scsi_util_obj->error) {
        $self->_set_err($scsi_util_obj->errmsg());
         return $return;
    } else {
        $self->sanlog(-msg       => "Success:",
                      -msg_level => 3);
    }

    # Display the scsi data
    $self->_display_scsi_pr_in_read_resv($h_ref);

    return $h_ref;
}

####################        _display_scsi_pr_in_read_resv       ###################
# Params:  $href: hash values - scsi data to be displayed
# Returns: None
###################################################################################

sub _display_scsi_pr_in_read_resv {
    my ($self, $h_ref) = @_;

    $self->sanlog(-msg       => "GENERATION     : $h_ref->{GENERATION}",
                  -msg_level => 3);

    $self->sanlog(-msg       => "ADD_LENGTH     : $h_ref->{ADD_LENGTH}",
                  -msg_level => 3);

    if ($h_ref->{ADD_LENGTH} == 16) {
        $self->sanlog(-msg => "KEY       : "
                        . $h_ref->{KEYS}
                        . "\nADDRESS : "
                        . $h_ref->{ADDRESS}
                        . "\nRESERVED: "
                        . $h_ref->{RESERVED}
                        . "\nTYPE    : "
                        . $h_ref->{TYPE},
                      -msg_level => 3);
    }

}

####################	_scsi_pr_in_report_capabilities         ###################
## @step Objective: Execute the PR IN SCSI command
## Input parameters: $device - ITL nexus information over which the command has to
## be executed
###################################################################################

sub _scsi_pr_in_report_capabilities {
    my ($self, $scsi_util_obj, $device, $expected, $buf_len) = @_;
    my $result = 1;
    my $errmsg;
    my $cdb = "scsi_pr_in_rep_cap";

    $self->sanlog(-msg       => "PR IN: Report Capabilities",
                  -msg_level => 3);

    my $scsi_tool_obj = $scsi_util_obj->{s_t_obj};

    my $return =
      $scsi_tool_obj->test_scsi_cmd(-device      => $device,
                                    -cdb_name    => $cdb,
                                    -buf_len     => $buf_len,
                                    -scsi_status => $expected,);

    if ($scsi_tool_obj->error) {
        $self->_set_err($scsi_tool_obj->errmsg());
        return $return;
    }

    my $h_ref =
      $scsi_util_obj->scsi_bytes_to_strings(
                                         -scsi_cmd_name => $cdb,
                                         -scsi_data_ref => $return->{SCSI_DATA},
      );

    if ($scsi_util_obj->error) {
        $self->_set_err($scsi_util_obj->errmsg());
        return $return;
    } else {
        $self->sanlog(-msg       => "Success:",
                      -msg_level => 3);
    }

    # Display the scsi data
    $self->_display_scsi_pr_in_rep_cap($h_ref);

    return $h_ref;
}

####################       _display_scsi_pr_in_read_resv        ###################
# Params:  $href: hash values - scsi data to be displayed
# Returns: None
###################################################################################

sub _display_scsi_pr_in_rep_cap {
    my ($self, $h_ref) = @_;

    $self->sanlog(-msg => "LENGTH     :"
                    . $h_ref->{LENGTH}
                    . "\nPTPL_C     :"
                    . $h_ref->{PTPL_C}
                     . "\nATP_C     :"
                    . $h_ref->{ATP_C}
                    . "\nSIP_C      :"
                    . $h_ref->{SIP_C}
                    . "\nCRH        :"
                    . $h_ref->{CRH}
                    . "\nPTPL_A     :"
                    . $h_ref->{PTPL_A}
                    . "\nTMV        :"
                    . $h_ref->{TMV}
                    . "\nMASK       :"
                    . $h_ref->{MASK}
                    . "\nRES        :"
                    . $h_ref->{RES},
                  -msg_level => 3);
}

####################	          scsi_pr_out		       ###################
## @step Objective: Execute the PR IN SCSI command
## Input parameters:
## $device             - ITL nexus information over which the command has to be
##			 executed
## $service_action     - specifies the particular service (i.e. reserve, release,
##			 register etc.) being requested by the Initiator
## $pr_type            - defines the characteristics of the persistent reservation
##			 being established
## $reservation_key    - 8-Byte reservation key that is already registered for the
##			 I_T nexus on which the command was received
## $service_action_key - new 8-Byte reservation key to be registered
## $aptpl              - persist through power loss on/off
###################################################################################

sub scsi_pr_out {
    my ($self, %args) = @_;
    my $result = 1;

    my ($service_action,     $pr_type,  $reservation_key,
        $service_action_key, $aptpl,    $device,
        $errmsg,             $expected, $buf_len,
        $scsi_util_obj,      $scsi_tool_obj);

    $self->sanlog(-msg       => "PR OUT..",
                  -msg_level => 3);

    $self->_clear_err();

    # set the default values for the keys, action, type and the aptpl bit
    $service_action     = 0;
    $pr_type            = 0;
    $reservation_key    = "00:00:00:00:00:00:00:00";
    $service_action_key = "00:00:00:00:00:00:00:00";
    $expected           = "GOOD";
    $aptpl              = 0x00;
    $buf_len            = 1024;

    #
    # Parse the arguments sent in the args hash
    #

    foreach my $param (keys %args) {
        if ($param =~ /^-?util$/) {    # Utility object
            $scsi_util_obj = $args{$param};
            $scsi_tool_obj = $scsi_util_obj->{s_t_obj};
            unless ($scsi_util_obj) {
                $self->_set_err(  "set_pr_out(): "
                                . "Invalid value passed for the '-util' "
                                . "argument: $args{$param}");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?device$/) {    # Device ITL
            $device = $args{$param};
            unless ($device) {
                $self->_set_err(  "set_pr_out(): "
                                . "Invalid value passed for the '-device' "
                                . "argument: $args{$param}");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?service_action$/) {    # Service Action
            $service_action = $args{$param};
            unless ($service_action =~ /^[0-7]/) {
                $self->_set_err("set_pr_out(): "
                             . "Invalid value passed for the '-service_action' "
                             . "argument: "
                             . $args{$param});
                return $self;
            }

            next;
        } elsif ($param =~ /^-?pr_type$/) {           # PR type
            $pr_type = $args{$param};
            if ($pr_type !~ /^[0-8]/) {
                $self->_set_err(  "set_pr_out(): "
                                . "Invalid value passed for the '-pr_type' "
                                . "argument: $param");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?reservation_key$/) {    # Reservation key
            $reservation_key = $args{$param};
            unless (length($reservation_key)) {
                $self->_set_err("set_pr_out(): "
                            . "Invalid value passed for the '-reservation_key' "
                            . "argument: $param");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?sa_reserve_key$/)
        {    # Service Action reservation key
            $service_action_key = $args{$param};
            unless (length($service_action_key)) {
                $self->_set_err("set_pr_out(): "
                             . "Invalid value passed for the '-sa_reserve_key' "
                             . "argument: $param");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?aptpl$/) {    # APTPL
            $aptpl = $args{$param};
            unless (length($aptpl)) {
                $self->_set_err(  "set_pr_out(): "
                                . "Invalid value passed for the '-aptpl' "
                                . "argument: $param");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?buf_len$/) {    # Buffer Length
            $buf_len = $args{$param};
            unless (length($buf_len)) {
                $self->_set_err(  "set_pr_out(): "
                                . "Invalid value passed for the '-buf_len' "
                                . "argument: $param");
                return $self;
            }

            next;
        } elsif ($param =~ /^-?expected_result$/) {    # Expected return value
            $expected = $args{$param};
            unless ($expected) {
                $self->_set_err(  "set_pr_out(): "
                                . "Invalid value passed for the '-expected' "
                                . "argument: $args{$param}");
                return $self;
            }

            next;
        } else {
            $self->_set_err("set_pr_out(): " . "Invalid Argument");
            return $self;
        }
    }

    unless ($device) {
        $self->_set_err(  "set_pr_out(): "
                        . "Invalid value passed for the '-device' "
                        . "argument");
        return $self;
    }

    unless ($scsi_util_obj) {
        $self->_set_err(  "set_pr_out(): "
                        . "Invalid value passed for the '-util' "
                        . "argument");
        return $self;
    }

    # Get the CDB string for scsi_pr_out based on service action
    # and the persistent reservation type

    my $cdb_ref = new CDB(SCSI_PR_OUT_CDB_LENGTH);

    $cdb_ref->set_pr_out_cdb(-service_action => $service_action,
                             -pr_type        => $pr_type,);

    my $cdb_str = join ' ', $cdb_ref->get_cdb_string("with_spaces");

    # Get the command parameter list string

    $cdb_ref->set_pr_out_cmd_param_list(-reservation_key => $reservation_key,
                                        -sa_key          => $service_action_key,
                                        -aptpl           => $aptpl);

    my $cdb_param_str = join ' ', $cdb_ref->get_cdb_param_string("with_spaces");

    $result =
      $scsi_tool_obj->test_scsi_cmd(-device      => $device,
                                    -cdb_name    => $cdb_str,
                                    -cdb_param   => $cdb_param_str,
                                    -scsi_status => $expected,
                                    -buf_len     => $buf_len);

    if ($scsi_tool_obj->error) {
        $self->_set_err($scsi_tool_obj->errmsg());
    }

    return $result;
}

1;
