#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/ScsiTool/Uscsi.pm#1 $
#$DateTime: 

## @summary Provide an interface to the USCSI tool
## @author bhanup@netapp.com
## @status public
## @pod ../ScsiTool.pm


#
# Package Name
#
package San::ScsiTool::Uscsi;

use strict;

#
# Package Requirements
#
use San::ScsiTool;
use San::SanClient::ClientUtil;

use Error;
use San::SanLog;

use vars qw( @ISA $ScsiTool $tool_loc);

@ISA = qw(Error SanLog ScsiTool);

#$tool_loc = "/u/bhanup/p4/san_host/test/solaris/any_scsi";

########################################################################
# P U B L I C   M E T H O D S
########################################################################

########################################################################
# Subroutine Name: send_cmd()
# Objective:       Execute the Uscsi tool to send a SCSI Command
# Details:         See POD documentation in ScsiTool.pm
########################################################################
sub send_cmd
{
    my ($self, %args)   = @_;
    
    $self->_clear_err();

    $self->handle_send_cmd_params(%args);
    # just return here no need to join, the caller will do that
    return $self if($self->error);

    #
    # build the cmd string that SPT expects for execution
    #
    $self->build_cmd_string();
    # just return here no need to join, the caller will do that
    return $self if($self->error);

    my $cmd = $self->{sc_cmd};
    
    $self->sanlog(-msg          => ref($self)."::send_cmd()\n"   . 
                                   "Cmd  String  => $cmd\n",
                  -msg_level    => 3
                 );

    #
    # execute the cmd on the client
    #
    my $client_obj  = $self->{client};
    my $client_conn = $client_obj->get_connection();
    
    my ($status, @output) = &ClientUtil::execute($client_conn, $cmd);
    
    $self->sanlog(-msg          => "SCSI command returned: " .
                                    join("\n", @output),
                  -msg_level    => 3
                 );    
    
    #
    # Now here comes the power of expected_responses api call.
    # 
    $self->update_rsp_hash(\@output);
    
    if ($status > 0) {
        $self->sanlog(-msg => "Warning: ".ref($self)."::send_cmd(): " .
                              "Command ($cmd) returned a bad status ($status).");
    }
    
    #
    # before we send the response hash back to the testcase
    # make sure that all the required params are were not 
    # updated are set to undef
    #
    $self->handle_unsup_responses();
    
    #
    # now we are ready to reply back to the testcase.
    #
    return $self->get_exp_rsp_params();
    
} # end send_cmd()

########################################################################
# P R I V A T E   M E T H O D S
########################################################################

########################################################################
# Subroutine Name: update_rsp_hash()
# Objective:       Take the exp resp bitmask and fill in the keys
#                  of the response hash that sub-class have to address
#                  This should be done when the constructor is executed first.
# Details:         See POD documentation in ScsiTool.pm
# /u/bhanup/p4/san_host/test/solaris/any_scsi /dev/rdsk/c7t60A9800056716935533448486E4C3953d0s2 32 6 8 0 0 0 1 0

# SCSI Status:                            GOOD

        # 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00
        # 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00 00

#/u/bhanup/p4/san_host/test/solaris/any_scsi /dev/rdsk/c7t60A9800056716935533448486E4C3953d0s2 32 6 8 0 0 0 1 0

# SCSI Status:                            Check Condition

# Sense key:                              Unit Attention
# Addional Sense len:                     0x0E
# Addional Sense Code(ASC):               0x3F
# Addional Sense Code Qualifier(ASCQ):    0x0E
# USCSI ioctl failed: I/O error
#
########################################################################
sub update_rsp_hash 
{
    my ($self, $outp_ref)   = @_;

    $self->_clear_err();
    
    # since the exp_rsp_params hash is used to convey results to
    # the testcase.....
    # make sure to reset all the expected responses params to defaults.
    # so the next call to update_rsp_hash will always return data
    # that is valid for this execution of send_cmd and not the 
    # previous invocation.
    #
    $self->build_exp_rsp_hash();
    return $self if($self->error);
    
    # first get the exp_rsp_hash ref
    my $er_ref = $self->get_exp_rsp_params();

    # make sure to set the IOCTL_STATUS to fail and let the 
    # parsing of the output override with success
    $self->get_exp_rsp_params()->{IOCTL_STATUS} = "fail";
    
    my $data = 0;
    my @scsi_data;
    my ($str, $ss, $sk, $asc, $ascq, $cmd_status);
    
    foreach my $ln (@$outp_ref) {
        # if the line has only spaces or is empty ignore it.
        next if ($ln =~ /^\s*$/);
        
        # take out any leading or trailing white space
        $ln =~ s/^\s*|\s*$//;

        if($ln =~ /SCSI Status:/) {
            ($str, $ss) = split /:/, $ln;
            # take out any leading or trailing white space
            $ss =~ s/^\s*|\s*$//;
            if($ss =~ /GOOD/){
                $data = 1;
                $self->get_exp_rsp_params()->{IOCTL_STATUS} = "Success";
            }
            $self->get_exp_rsp_params()->{SCSI_STATUS} = uc($ss);
            next;
        } elsif($ln =~ /ioctl failed:(.+)/i) {
            # handle data here.....
            $cmd_status = $1;
            $self->get_exp_rsp_params()->{IOCTL_STATUS} = $cmd_status;
            $data = 0;
            next;
        } elsif($ln =~ /sense key/i){
            ($str, $sk) = split /:/, $ln;
            # take out any leading or trailing white space
            $sk =~ s/^\s*|\s*$//;
            $self->get_exp_rsp_params()->{SENSE_KEY} = uc($sk);
            $data = 0;
            next;
        } elsif($ln =~ /Addional Sense Code\(ASC\):/) {
            ($str, $asc) = split /:/, $ln;
            # take out any leading or trailing white space
            # and any 0x indicators
            $asc =~ s/^\s*(0x)?|\s*$//;
            $self->get_exp_rsp_params()->{ASCNQ} = $asc."h/";
            $data = 0;
            next;        
        } elsif($ln =~ /Addional Sense Code Qualifier\(ASCQ\):/) {
            ($str, $ascq) = split /:/, $ln;
            # take out any leading or trailing white space
            $ascq =~ s/^\s*(0x)?|\s*$//;
            $self->get_exp_rsp_params()->{ASCNQ} .= $ascq."h";
            $data = 0;
            next;        
        }
        if($data && $ln =~ /(..\s)+/) {
            my @ln_data = split /\s/, $ln;
            push @scsi_data, @ln_data;
        }
    }
    
    $er_ref->{SCSI_DATA}  = \@scsi_data if(@scsi_data);
    
    #
    # set the toolname to any_scsi
    $self->get_exp_rsp_params()->{TOOLNAME} = "any_scsi";
    
    return 1;
}   


########################################################################
# Subroutine Name: build_cmd_string()
# Objective:       For the send_cmd params build the string that 
#                  SPT uses to specify which cmd to execute.
#                  This should be done when ever send_cmd is invoked.
# Details:         See POD documentation in ScsiTool.pm
########################################################################
sub build_cmd_string 
{
    my $self = shift @_;

    $self->_clear_err();
# /u/bhanup/p4/san_host/test/solaris/any_scsi
# usage: /u/bhanup/p4/san_host/test/solaris/any_scsi <device> <buf_len> <bytes> [byte0] [byte1]...
#

    my $sc_ref          = $self->get_send_cmd_params();

    my $dev             = $sc_ref->{DEVICE};
    my $cdb             = $sc_ref->{CDB};
    my $buf_len         = $sc_ref->{BUF_LEN};
    my $timeout         = $sc_ref->{TIMEOUT};
    
    my $opcode          = $sc_ref->{OPCODE};

    if($dev eq ScsiTool::REQ_RESPONSE) {
        $self->_set_err("Error: ".ref($self)."::build_cmd_string(): " .
                        "Valid device name is required to send SCSI cmd.");
        return $self;        
    }

    # figure out how many byte cdb is the being processed.
    #

    my @cdb_bytes = split /\s/, $cdb;
    chomp(@cdb_bytes);
    s/^\s*|\s*$// for(@cdb_bytes);

#    my $c_len   = scalar @cdb_bytes;

    if($opcode != ScsiTool::SCSI_CDB) {        
        if($opcode & ScsiTool::NACA_BIT || 
           $opcode & ScsiTool::LINK_BIT) {
            # (REDUNDANT option)
            # the op string is not set here but the cdb that goes into
            # the cmd string is manipulated to enable NACA and/or LINK bits
            # in the control byte of the cdb. 
            $cdb_bytes[$#cdb_bytes] |= 0x04 if($opcode & ScsiTool::NACA_BIT);
            $cdb_bytes[$#cdb_bytes] |= 0x01 if($opcode & ScsiTool::LINK_BIT);
        }
        else {
            $self->_set_err("Error: ".ref($self)."::build_cmd_string():" .
                            "Unsupported OP for USCSI: $opcode");
            $self->{TOOL_LIB_STATUS} |= ScsiTool::OP_UNSUP;
            return $self;
        }
    }
    
    #
    # Make sure the the CDB bytes are pre-pended with 0x to
    # indicate hex values for the any_scsi_sanqa tool
    my $x;
    for ($x=0;$x <= $#cdb_bytes;$x++) {
        if($cdb_bytes[$x] =~ /^..$/) {
            $cdb_bytes[$x] = "0x".$cdb_bytes[$x];
        }elsif($cdb_bytes[$x] !~ /^0x/) {
            $self->_set_err("Error: ".ref($self)."::build_cmd_string():" .
                            "Non-hex value for a CDB byte detected: ".
                            join ' ', @cdb_bytes);
            return $self;
        }        
    }
    
    #
    # Now that hex notation is complete we can go ahead and 
    # create the cdb string to be sent
    #
    $cdb = join ' ', @cdb_bytes;
    
    #
    # now that we have processed what the opcode is go ahead
    # and update the cdb in the cmd string
    #
    
    my $cmd     = $tool_loc."\\\n ".$dev."\\\n ".$buf_len."\\\n ".
                  ($#cdb_bytes + 1)."\\\n ".$cdb;
    
    unless (length $cmd) {
        $self->_set_err("Error: ".ref($self)."::build_cmd_string(): ".
                        "Could not build USCSI cmd!");
        return $self;
    }
    
    #
    # store the $cmd in $self.
    #    
    $self->{sc_cmd} = $cmd;
    
    return 1;
}


########################################################################
# Subroutine Name: _new()
# Objective:       Create a new Uscsi object
# Details:         See POD documentation at the end of this file
########################################################################
sub _new
{
    my ($caller, %args) = @_;
    my $class = ref($caller) || $caller;
    my $self = $class->_base_new(%args);
    
    if($self->{client}->hosttype() =~ /solaris/i) {
        $tool_loc = $self->{client}->{TEST_BIN}."/any_scsi_sanqa" ;
    } else {
        $self->_set_err("Error: ".ref($self)."::new() Invalid host type: ".
                        $self->{client}->hosttype());
    }
    
    return $self;
} # end _new()


1;
