#Hey emacs, this is basically -*- perl -*-
##$Id: //depot/prod/test/Rsierranevada/lib/San/ScsiTool/Xcopy.pm#1 $
#
### @summary Provide an interface to several client side SCSI tools
### @author chadwell@netapp.com
### @status public
### @pod here
#

package Xcopy;

use strict;
use vars qw(@ISA);
use San::SanLog;
use Error;
use Tharn;
use Tie::IxHash;
use List::MoreUtils qw( each_array );

@ISA = qw(Error SanLog);

########################################################################################
## Method:    xcopy_build_cmd
## Objective: Build the Xcopy parameters hash so they can be changed per test
## Details:   See http://wikid/w/SAN_QA/Projects/Protocol_Intern/Xcopy_Functional_spec
#########################################################################################
sub xcopy_build_cmd {
    my ($self, %args) = @_;
    my ($src_lun, $dst_lun,        $buf_len);
    my ($ret_ref, $third_lun_path, $bd);
    my $xcopy_params = ();

    foreach my $param (keys %args) {

        # src lun for xcopy
        if ($param =~ /^-?src_lun/) {
            $src_lun = $args{$param};
        }

        # dest lun for xcopy
        if ($param =~ /^-?dest_lun/) {
            $dst_lun = $args{$param};
        }

        # third lun, in case one more lun is reuired in xcopy cdb
        if ($param =~ /^-?third_lun/) {
            $third_lun_path = $args{$param};
        }

        # segment descriptor's parameter of Source Block Device LBA, Destination Block Device LBA and Number of LBAs
        if ($param =~ /^-?bd/) {
            $bd = $args{$param};
        }
    } ## end foreach my $param (keys %args)
    my %xcopy_cdb_bytes = (
        desc_code =>
          "e4",  # byte 0 : Descriptor code (fixed value) -value at bytes in hex
        luid_type => "00", # byte 1 bit 7-6 : LU ID Type -value at bit in binary
        nul       => "0",  # byte 1 bit 5 : NUL -value at bit in binary
        per_dev_t => "00000"
        ,    # byte 1 bit 4-0 : peripheral device type -value at bit in binary
        rel_init_port_id => "00 00"
        ,  # byte 2-3 : Relative Initiator Port Identifer -value at bytes in hex
        reserve4  => "0000", # byte 4 bit 7-4 : Reserved -value at bit in binary
        code_set  => "0001", # byte 4 bit 3-0 : Code set -value at bit in binary
        reserved5 => "00",   # byte 5 bit 7-6 : Reserved -value at bit in binary
        assoc     => "00",   # byte 5 bit 5-4 : Reserved -value at bit in binary
        des_type =>
          "0011",  # byte 5 bit 3-0 : Designator type 3h -value at bit in binary
        reserved6  => "00",  # byte 6 : Reserved -value at bit in binary
        des_len    => "10",  # byte 7 : Designator Length -value at bytes in hex
        designator => ""
        , # byte 8-23 : place holder for src lun serial number -value at bytes in hex
        reserved24_27 =>
          "00 00 00 00",    # byte 24-27 : Reserved -value at bytes in hex
        dev_type_params => "00 00 00 00"
        ,  # byte 28-31 : Device type specific parameters -value at bytes in hex
                          );

    # source lun TD
    $xcopy_params->{src_target_desc} = {%xcopy_cdb_bytes};

    # Destination lun TD
    $xcopy_params->{dest_target_desc} = {%xcopy_cdb_bytes};

    $xcopy_params->{block_seg_desc} = {
        desc_code =>
          "02",       # byte 0 : Descriptor type code 02h -value at bytes in hex
        reserved1 =>
          "00000",     # byte 1 bit 7-3 : Reserved -value at bit in binary
        full => "1",   # byte 1 bit 2 : FULL -value at bit in binary
        dc   => "0",   # byte 1 bit 1 : DC -value at bit in binary
        cat  => "0",   # byte 1 bit 0 : CAT -value at bit in binary
        desc_len =>
          "00 18",     # byte 2-3 : Descriptor Length -value at bytes in hex
        src_index => "00 00"
        ,    # byte 4-5 : Source Tagert Descriptor Index -value at bytes in hex
        dest_index => "00 01"
        , # byte 6-7 : Destination Tagert Descriptor Index-value at bytes in hex
        reserved8_9 => "00 00",    # byte 8-9 : Reserved -value at bytes in hex
             # It has total 27 bytes: Number of LBAs (byte 10-11),
          # Source Block Device LBA (byte 12-19), Target Block Device LBA (byte 20-27)
          # These byte values are taken as input argument from the test case and stored in variable $bd
          # which is in format of source lba , target lba(dest lba) and no. of lbas.
                                      };
    $xcopy_params->{param_list} = {
        list_id => "01"
        ,    # byte 0 : list Identifier set it default 01 -value at bytes in hex
        reserved1 => "00",   # byte 1 bit 7-6 : Reserved -value at bit in binary
        str       => "0",    # byte 1 bit 5 : STR -value at bit in binary
        nrcr      => "0",    # byte 1 bit 4 : NRCR -value at bit in binary
        nlid      => "0",    # byte 1 bit 3 : NLID -value at bit in binary
        priority  => "000",  # byte 1 bit 2-0 : Priority -value at bit in binary
        target_list_len => "00 40"
        , # byte 2-3 : Target Descriptor List Length for now there are always only 3 -value at bytes in hex
        reserved4_7 =>
          "00 00 00 00",    # byte 4-7 : Reserved -value at bytes in hex
        seg_list_len => "00 00 00 1c"
        , # byte 8-11 : Segment descriptor list length param length computed -value at bytes in hex
        inline_data_len => "00 00 00 00"
        ,    # byte 12-15 : Can totally leave this blank -value at bytes in hex
                                  };
    $xcopy_params->{dest_lun} = $dst_lun;   # dest lun, target for xcopy command
    $xcopy_params->{buf_len} = 108;   # lenght of the parameter in xcopy command
    $xcopy_params->{xcopy_cdb} =
      "83 00 00 00 00 00 00 00 00 00 00 00 00 6c 00 00";    # xcopy cdb
    $xcopy_params->{bd_1} =
      $bd;    # segment descriptor to store src lba,dest lba, and no of lba
    $xcopy_params->{src_lun} = $src_lun;    # Source Device for xcopy command

    #
    #Constructing the third lun target descriptor if third lun path is available
    #
    if (length($third_lun_path)
        and (   $third_lun_path =~ m#/dev/rdsk/#
             or $third_lun_path =~ m#\\.\physicaldrive#
             or $third_lun_path =~ m#/dev/sd#)
      ) {
        $xcopy_params->{third_target_desc} = {%xcopy_cdb_bytes};
    }

    # Get the serial of lUNs
    my @devices =
      ($third_lun_path
        ? ($src_lun, $dst_lun, $third_lun_path)
        : ($src_lun, $dst_lun));
    my @descriptors = (
         $third_lun_path
         ? ($xcopy_params->{src_target_desc},
            $xcopy_params->{dest_target_desc},
            $xcopy_params->{third_target_desc})
         : ($xcopy_params->{src_target_desc}, $xcopy_params->{dest_target_desc})
    );

    # Loop through both arrays at the same time.
    my $it = each_array(@devices, @descriptors);
    while (my ($device, $descriptor) = $it->()) {

        # Log that you are getting inquiry data for the lun
        $self->sanlog(
                 -msg => "Get the inquiry data of the lun : " . $device . "\n");

        # Get the inquiry data
        $ret_ref = $self->send_cmd(-device => $device,
                                   -cdb    => 'inquiry_page_83h',
                                   -dir    => 'read');

        # Store the appropriate inquiry data in the designator
        $descriptor->{designator} =
            $ret_ref->{SCSI_DATA}[44] . " "
          . $ret_ref->{SCSI_DATA}[45] . " "
          . $ret_ref->{SCSI_DATA}[46] . " "
          . $ret_ref->{SCSI_DATA}[47] . " "
          . $ret_ref->{SCSI_DATA}[48] . " "
          . $ret_ref->{SCSI_DATA}[49] . " "
          . $ret_ref->{SCSI_DATA}[50] . " "
          . $ret_ref->{SCSI_DATA}[51] . " "
          . $ret_ref->{SCSI_DATA}[52] . " "
          . $ret_ref->{SCSI_DATA}[53] . " "
          . $ret_ref->{SCSI_DATA}[54] . " "
          . $ret_ref->{SCSI_DATA}[55] . " "
          . $ret_ref->{SCSI_DATA}[56] . " "
          . $ret_ref->{SCSI_DATA}[57] . " "
          . $ret_ref->{SCSI_DATA}[58] . " "
          . $ret_ref->{SCSI_DATA}[59];

        # Check for error reading inquiry
        if (length($descriptor->{designator}) <= 15) {
            $self->_set_err(
                "Error: San::ScsiTool::Xcopy",
                "INQUIRY EVPD page 83h Identification Descriptor.Possible error?"
            );
            $self->sanlog(-msg       => "Device path may not be valid.",
                          -msg_level => 3);
            $self->sanlog(-msg       => "Device Path Passed:" . $device,
                          -msg_level => 3);
            return $self;
        } else {

            # log the serial number
            $self->sanlog(-msg => "lun serial number is "
                            . $descriptor->{designator} . "\n",
                          -msg_level => 3);
        }

    } ## end while (my ($device, $descriptor...
    return $xcopy_params;
}    #end sub

#########################################################################################
### Method:    xcopy_send_cmd
### Objective: Build the Xcopy CDB and parameter list, actually sends command
### Details:   See http://wikid/w/SAN_QA/Projects/Protocol_Intern/Xcopy_Functional_spec
#########################################################################################
sub xcopy_send_cmd {
    my ($self, $xcopy_params) = @_;
    my ($luid_type, $nul,    $per_dev_t, $byte_1,   $reserve4, $code_set,
        $reserve5,  $byte_4, $assoc,     $des_type, $reserve);
    my ($str, $nrcr, $nlid, $priority, $byte_5, $seg_desc, $src_lun,
        $third_lun);
    my ($dest_lun, $param_header, @Xcopy_return, $full, $dc, $cat,
        $param_list_byte_1);
    my ($src_addr, $dest_addr, $block_num);
    my ($no_of_blocks, $return_ref);

    #
    #Creating Target Descriptor for Destination LUN
    #luid_type, nul, per_dev_t all make up one byte in the data.
    #So taking these as binary value and compile them
    #as the hex string.
    #

    $luid_type = $xcopy_params->{dest_target_desc}->{luid_type};
    $nul       = $xcopy_params->{dest_target_desc}->{nul};
    $per_dev_t = $xcopy_params->{dest_target_desc}->{per_dev_t};
    $byte_1    = sprintf("%08u", $luid_type . $nul . $per_dev_t);
    $byte_1    = unpack("H*", pack("B*", $byte_1));

    #
    #The reserved,and code_set values make up one byte in the data.
    #So taking these as binary value and compile them
    #as the hex string.
    #

    $reserve4 = $xcopy_params->{dest_target_desc}->{reserve4};
    $code_set = $xcopy_params->{dest_target_desc}->{code_set};
    $byte_4   = sprintf("%08u", $reserve4 . $code_set);
    $byte_4   = unpack("H*", pack("B*", $byte_4));

    #
    #The reserved, assoc and des_type values make up one byte in the data.
    #So taking these as binary value and compile them
    #as the hex string.
    #

    $reserve5 = $xcopy_params->{dest_target_desc}->{reserved5};
    $assoc    = $xcopy_params->{dest_target_desc}->{assoc};
    $des_type = $xcopy_params->{dest_target_desc}->{des_type};
    $byte_5   = sprintf("%08u", $reserve5 . $assoc . $des_type);
    $byte_5   = unpack("H*", pack("B*", $byte_5));

    #
    #actually building the destination target descripor block
    #
    $dest_lun =
        $xcopy_params->{dest_target_desc}->{desc_code} . " "
      . $byte_1 . " "
      . $xcopy_params->{dest_target_desc}->{rel_init_port_id} . " "
      . $byte_4 . " "
      . $byte_5 . " "
      . $xcopy_params->{dest_target_desc}->{reserved6} . " "
      . $xcopy_params->{dest_target_desc}->{des_len} . " "
      . $xcopy_params->{dest_target_desc}->{designator} . " "
      . $xcopy_params->{dest_target_desc}->{reserved24_27} . " "
      . $xcopy_params->{dest_target_desc}->{dev_type_params};
    $self->sanlog(
           -msg => "Destination target descriptor block is " . $dest_lun . "\n",
           -msg_level => 3);

    #
    #Creating Target Descriptor for Source LUN
    #luid_type, nul, per_dev_t all make up one byte in the data.
    #So taking these as binary value and compile them
    #as the hex string.
    #

    $luid_type = $xcopy_params->{src_target_desc}->{luid_type};
    $nul       = $xcopy_params->{src_target_desc}->{nul};
    $per_dev_t = $xcopy_params->{src_target_desc}->{per_dev_t};
    $byte_1    = sprintf("%08u", $luid_type . $nul . $per_dev_t);
    $byte_1    = unpack("H*", pack("B*", $byte_1));

    #
    #The reserved,and code_set values make up one byte in the data.
    #So taking these as binary value and compile them
    #as the hex string.
    #

    $reserve4 = $xcopy_params->{src_target_desc}->{reserve4};
    $code_set = $xcopy_params->{src_target_desc}->{code_set};
    $byte_4   = sprintf("%08u", $reserve4 . $code_set);
    $byte_4   = unpack("H*", pack("B*", $byte_4));

    #
    #The reserved, assoc and des_type values make up one byte in the data.
    #So taking these as binary value and compile them
    #as the hex string.
    #

    $reserve5 = $xcopy_params->{src_target_desc}->{reserved5};
    $assoc    = $xcopy_params->{src_target_desc}->{assoc};
    $des_type = $xcopy_params->{src_target_desc}->{des_type};
    $byte_5   = sprintf("%08u", $reserve . $assoc . $des_type);
    $byte_5   = unpack("H*", pack("B*", $byte_5));

    #
    #Actually building the source lun descriptor block.
    #same format as above, but just the designator differs.
    #

    $src_lun =
        $xcopy_params->{src_target_desc}->{desc_code} . " "
      . $byte_1 . " "
      . $xcopy_params->{src_target_desc}->{rel_init_port_id} . " "
      . $byte_4 . " "
      . $byte_5 . " "
      . $xcopy_params->{src_target_desc}->{reserved6} . " "
      . $xcopy_params->{src_target_desc}->{des_len} . " "
      . $xcopy_params->{src_target_desc}->{designator} . " "
      . $xcopy_params->{src_target_desc}->{reserved24_27} . " "
      . $xcopy_params->{src_target_desc}->{dev_type_params};

    $self->sanlog(
                 -msg => "Source target descriptor block is " . $src_lun . "\n",
                 -msg_level => 3);

    #
    #if third Lun is going to be used
    #
    if (!($xcopy_params->{third_target_desc}->{designator} =~ /60/)) {
        $third_lun = "";
    } else {
        $luid_type = $xcopy_params->{third_target_desc}->{luid_type};
        $nul       = $xcopy_params->{third_target_desc}->{nul};
        $per_dev_t = $xcopy_params->{third_target_desc}->{per_dev_t};
        $byte_1    = sprintf("%08u", $luid_type . $nul . $per_dev_t);
        $byte_1    = unpack("H*", pack("B*", $byte_1));

        $reserve4 = $xcopy_params->{third_target_desc}->{reserve4};
        $code_set = $xcopy_params->{third_target_desc}->{code_set};
        $byte_4   = sprintf("%08u", $reserve4 . $code_set);
        $byte_4   = unpack("H*", pack("B*", $byte_4));

        $reserve5 = $xcopy_params->{third_target_desc}->{reserved5};
        $assoc    = $xcopy_params->{third_target_desc}->{assoc};
        $des_type = $xcopy_params->{third_target_desc}->{des_type};
        $byte_5   = sprintf("%08u", $reserve5 . $assoc . $des_type);
        $byte_5   = unpack("H*", pack("B*", $byte_5));

        $third_lun =
            $xcopy_params->{third_target_desc}->{desc_code} . " "
          . $byte_1 . " "
          . $xcopy_params->{third_target_desc}->{rel_init_port_id} . " "
          . $byte_4 . " "
          . $byte_5 . " "
          . $xcopy_params->{third_target_desc}->{reserved6} . " "
          . $xcopy_params->{third_target_desc}->{des_len} . " "
          . $xcopy_params->{third_target_desc}->{designator} . " "
          . $xcopy_params->{third_target_desc}->{reserved24_27} . " "
          . $xcopy_params->{third_target_desc}->{dev_type_params} . " ";

        $self->sanlog("third target descriptor block is " . $third_lun . "\n");
    } ## end else [ if (!($xcopy_params->{...

    #
    #The Parameter list Descriptor's 1st byte is compose of reserved (bit 7-6),
    #STR (bit 5), NRCR(bit 4), NLID (bit 3), Priority (bit 2-0).
    #Below is code to construct this from the binary values of these components.
    #

    $reserve  = $xcopy_params->{param_list}->{reserved1};
    $str      = $xcopy_params->{param_list}->{str};
    $nrcr     = $xcopy_params->{param_list}->{nrcr};
    $nlid     = $xcopy_params->{param_list}->{nlid};
    $priority = $xcopy_params->{param_list}->{priority};
    $param_list_byte_1 =
      sprintf("%08u", $reserve . $str . $nrcr . $nlid . $priority);
    $param_list_byte_1 = unpack("H*", pack("B*", $param_list_byte_1));

    $param_header =
        $xcopy_params->{param_list}->{list_id} . " "
      . $param_list_byte_1 . " "
      . $xcopy_params->{param_list}->{target_list_len} . " "
      . $xcopy_params->{param_list}->{reserved4_7} . " "
      . $xcopy_params->{param_list}->{seg_list_len} . " "
      . $xcopy_params->{param_list}->{inline_data_len};

    #
    #Making byte 1 hex out of reserved (0), full, DC, and cat
    #
    $reserve = $xcopy_params->{block_seg_desc}->{reserved1};
    $full    = $xcopy_params->{block_seg_desc}->{full};
    $dc      = $xcopy_params->{block_seg_desc}->{dc};
    $cat     = $xcopy_params->{block_seg_desc}->{cat};
    $byte_1  = sprintf("%08u", $reserve . $full . $dc . $cat);
    $byte_1  = unpack("H*", pack("B*", $byte_1));

    #
    # sending an xcopy command per transfer tuple in array
    # which describes where to move information between luns
    #

    # traversing hash ref of xcopy_info
    foreach my $param (keys %$xcopy_params) {
        # in case "bd" key is available, contains array ref of (src lba, dest lba, no. of lba)
        if ($param =~ /bd/) {
            $src_addr = sprintf("%016x", $xcopy_params->{$param}[0]);
            $src_addr =~
                  s/(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})/$1 $2 $3 $4 $5 $6 $7 $8/;

            $dest_addr = sprintf("%016x", $xcopy_params->{$param}[1]);
            $dest_addr =~
                  s/(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})/$1 $2 $3 $4 $5 $6 $7 $8/;

            $no_of_blocks = sprintf("%04x", $xcopy_params->{$param}[2]);
            $no_of_blocks =~ s/(.{2})(.{2})/$1 $2/;
            $seg_desc =
                   $xcopy_params->{block_seg_desc}->{desc_code} . " "
                  . $byte_1 . " "
                  . $xcopy_params->{block_seg_desc}->{desc_len} . " "
                  . $xcopy_params->{block_seg_desc}->{src_index} . " "
                  . $xcopy_params->{block_seg_desc}->{dest_index} . " "
                  . $xcopy_params->{block_seg_desc}->{reserved8_9} . " "
                  . $no_of_blocks . " "
                  . $src_addr . " "
                  . $dest_addr . " ";

            $self->sanlog(-msg => "\nSource Device : " . $xcopy_params->{src_lun}
                                 ."\nDestination Device : ". $xcopy_params->{dest_lun}
                                 ."\nSource LBA on Source device : ". $xcopy_params->{$param}[0]
                                 ."\nDestination LBA on Destionation device : ". $xcopy_params->{$param}[1]
                                 ."\nNumber of block : ". $xcopy_params->{$param}[2]);

            $self->sanlog(-msg => "\n xcopy CDB : "
                                . $xcopy_params->{xcopy_cdb}
                                . "\n xcopy param header : "
                                . $param_header
                                . "\n src lun desgn: "
                                . $src_lun
                                . "\n dst lun desgn: "
                                . $dest_lun
                                . "\n seg desc: "
                                . $seg_desc,
                              -msg_level => 3);

            if ($third_lun) {
                    $self->sanlog(
                            -msg => "\nthird lun desgn: " . $third_lun . "\n");
                }

            ## Executing xcopy cdb
            $return_ref =
                  $self->send_cmd(-device    => $xcopy_params->{dest_lun},
                                  -cdb       => $xcopy_params->{xcopy_cdb},
                                  -cdb_param => $param_header . " "
                                    . $src_lun . " "
                                    . $dest_lun . " "
                                    . $third_lun
                                    . $seg_desc,
                                  -buf_len => $xcopy_params->{buf_len},
                                  -dir     => 'write');
            if (!($self->error)){
                    @Xcopy_return = (@Xcopy_return, $return_ref);
            }else{
	                $self->sanlog(-msg => "Xcopy is not executed");
	                $self->_set_err(
	                   "Error: San::ScsiTool::Xcopy",
	                    "Fail to execute Xcopy command");
	                return $self
           }
        }
    } ## end foreach my $param (keys %$xcopy_params)
    return @Xcopy_return;
}    #end sub
#########################################################################################
#                                   POD Documentation                                   #
#########################################################################################

=pod

=head1 Package Name

Xcopy.pm

=head1 Author

Jaikumar Periyannan, Chandan Kumar.

=head1 Description

Xcopy.pm provide subrotine to construct the xcopy cdb parameters
and execute it in 7 mode filer.

=head1 See Also

Package is inherited by the Package ScsiTool.pm

=head1 Attributes

=head1 Public Methods

=head2 B<xcopy_build_cmd(%args)>

Build the Xcopy parameters hash so they can be changed per testcases.
It takes device path from testcase, construct Target Descriptor,
 construct Segmenet Descriptor (block_seg_desc) and
construct Parameter List Descriptor (param_list).
Any value of this can be modifed in the testcase.
To construct Designator in Target Descriptor INQUERY SCSI command
is send to page 83h on each LUN and byte 44-59 of the INQUIRY EVPD
page 83h Identification Descriptor is extracted.


=over 4

=item Inputs

=item *I<-src_lun>

Device path of the source lun. This device will be used as source for
the xcopy cdb. Source LBA on this will be passed from the testcase.

=item * I<-dest_lun>

Device path of the Destination lun. This device will be used as
destination for the xcopy cdb. Destination LBA on this will be
passed from the testcase.

=item * I<-third_lun>

Device path of the one extra lun. This device will be used as
as per testcase requirement.

=item * I<-bd>

Segment Descriptor's parameter which has Source Block Device LBA,
Destination Block Device LBA and Number of LBAs.
It can be constructed in the testcase in array reference
used in subroutine xcopy_send_cmd().
Its value will be passed from the testcase in format of src lun's source lba,
dest lba(dest lun) and number of lbas.

=item Outputs

Returns a hash ref of xcopy command. If an error occured the error flag
is set, errmsg is set to the failure reason.
This hash reference can be used to modify any value in testcase.

=item Examples
  Invoke in Testcase
    # below is source lba, dest lba, number of lba
    my @a = (8192,8192,8);
    my $xcopy_cdb_info =
        $scsi_tool_obj->xcopy_build_cmd(  -src_lun=>  $def_src_dev_pri[0],
                                          -dest_lun=> $def_dst_dev_pri[0],
                                          -bd_1=>     \@a
                                       );
    # other three luns
    $xcopy_cdb_info =
        $scsi_tool_obj->xcopy_build_cmd(  -src_lun=>  $def_src_dev_pri[0],
                                          -dest_lun=> $def_dst_dev_pri[0],
                                          -third_lun => $third_lun_path,
                                          -bd_1=>     \@a
                                        );

 Modify the any value in the testcase :
    # set the Designator Length byte in the Src Target Descriptor
    $xcopy_cdb_info->{src_target_desc}->{des_len} = "ff";

    # Xcopy cdb parameter length will change to 8c
    $xcopy_cdb_info->{xcopy_cdb} = "83 00 00 00 00 00 00 00 00 00 00 00 00 8c 00 00";

    # length of parameter in spt tool will change to 140
    $xcopy_cdb_info->{buf_len} = "140";

    # length of parameter in xcopy to 60h
    $xcopy_cdb_info->{param_list}->{target_list_len}="00 60";

    $Test->nlog("Setting the src target index as 0 and dest target index as 1");
    $xcopy_cdb_info->{block_seg_desc}->{src_index} = "00 00";
    $xcopy_cdb_info->{block_seg_desc}->{dest_index} = "00 01";


=back

=head2 B<xcopy_send_cmd($self,$xcopy_cdb_info)>

Build the Xcopy CDB and parameter list, actually sends command

=over 4

=item Inputs

=item * $self

Object of ScsiTool.pm

=item * I<$xcopy_cdb_info>

Hash reference created in subroutine xcopy_build_cmd()
which has all parameters of xcopy cdb to execute.

=item Outputs

Returns an array which has hash reference, contains
the Scsi Status return after xcopy is executed.
If an error occured the error flag
is set, errmsg is set to the failure reason.

=item Examples

    my @return_array = $scsi_tool_obj->xcopy_send_cmd($xcopy_cdb_info);
    foreach my $scsicmd_return (@return_array){
        if ($scsicmd_return->{'IOCTL_STATUS'} == 0 ){
            if($scsicmd_return->{'SCSI_STATUS'} == 0){
                # validate the SCSI status, sense key and ASC/ASCQ
                if (!($scsi_tool_obj->check_status(-send_cmd_output => $scsicmd_return,
                            -sense_key     => $exp_sense_key,
                            -scsi_status => $exp_scsi_status,
                            -ascnq => $exp_ascnq))){
                  return $TCD::FAIL;
                 }
            }
        }
    }

=back

=cut

1;
