# -*- perl -*-
# $Id: $
#
# Copyright (c) 2010 NetApp, Inc.
# All rights reserved.
#
##
## @summary DeviceTypeSpecificParams Object
## @author SAN QA (c-dl-ssan-test@netapp.com)
## @status public
## @pod here

package San::ScsiTool::Xcopy_T10::TargetDescriptor::DeviceTypeSpecificParams;

# compiler directives
use strict;

# imports
use Carp;

################################################################################
# Method Name: get_peripheral_device_type
# Objective:   abstract - Must be implemented by inheriting class
# Details:     See POD documentation at the end of this file for usage
################################################################################
sub get_peripheral_device_type() {
    my ($self) = @_;
    my $class = ref($self);
    croak "someone used an (abstract) DeviceTypeSpecificParams object"
        if $class eq
            "San::ScsiTool::Xcopy_T10::TargetDescriptor::DeviceTypeSpecificParams";
    croak "call to abstract method ${class}::get_peripheral_device_type";
}

################################################################################
# Method Name: to_hex_string
# Objective:   Converts object to a scalar hex string representing the relevant
#              bytes of the Xcopy Device Type Specific Params.
# Details:     The returned string is formatted as space separated string of
#              sense hexadecimal bytes.  ie.   "00 00 00 00 00 00 00 00"
################################################################################
sub to_hex_string() {
    my ($self) = @_;
    my $class = ref($self);
    croak "someone used an (abstract) DeviceTypeSpecificParams object"
        if $class eq
            "San::ScsiTool::Xcopy_T10::TargetDescriptor::DeviceTypeSpecificParams";
    croak "call to abstract method ${class}::to_hex_string";
}

1;

###############################################################################
#                             POD Documentation
###############################################################################

=pod

=head1 Package Name

San::ScsiTool::Xcopy_T10::TargetDescriptor::DeviceTypeSpecificParams

=head1 Author

Chris Schroll (schroll@netapp.com)

=head1 Description

DeviceTypeSpecificParams.pm

This is an abstract class that defines the interface that all
DeviceTypeSpecificParams child classes must implement.

=head1 See Also

None

=head1 Attributes

None

=head1 Revision History

12-22-2011 schroll@netapp.com: Created Interface

=head1 Public Methods

=head2 B<get_peripheral_device_type()>

Abstract method.  Child class must override this and return the peripheral
device type that is relevant for the object.

=over 4

=item Inputs

=item Outputs

=item Examples

Child class must override this method.

=back

=head2 B<get_peripheral_device_type()>

Abstract method.  Child class must override to_hex_string to return the hex bytes
associated with DeviceTypeSpecificParams.

=over 4

=item Inputs

=item Outputs

=item Examples

Child class must override this method.

=back

=cut
