#Hey emacs, this is basically -*- perl -*-
##$Id: //depot/prod/test/Rsierranevada/lib/San/ScsiTool/Xcopyorg.pm#1 $
#
### @summary Provide an interface to several client side SCSI tools
### @author chadwell@netapp.com
### @status public
### @pod here
#

use strict;
package Xcopy;

########################################################################
## Method:    Xcopy_build_cmd 
## Objective: Build the Xcopy parameters hash so they can be changed per test
## Details:   See http://wikid/w/SAN_QA/Projects/Protocol_Intern/Xcopy_Functional_spec
#########################################################################

sub Xcopy_build_cmd{
    my ($self, %args) = @_;
    my ($src_lun, @ip_bytes);

    my $Xcopy_params = { 
                         src_ip_desc     => { 
                                              desc_code        => "e5",
                                              luid_type        => "0",
                                              nul              => "0",
                                              per_dev_t        => "0",
                                              rel_init_port_id => "00 00", 
                                              lu_id            => "00 00 00 00 00 00 00 00",
                                              ipv4_addr        => "",                               #source IP
                                              reserve16_21     => "00 00 00 00 00 00",
                                              port_num         => "00 00",
                                              reserve24_25     => "00 00",
                                              it_prot_num      => "00 00",
                                              dev_type_params  => "00 00 00 00"
                                            },
                         src_target_desc => {
                                              desc_code        => "e4",
                                              luid_type        => "0",
                                              nul              => "0",
                                              per_dev_t        => "0",
                                              rel_init_port_id => "00 00", 
                                              reserve4         => "0",
                                              code_set         => "1",
                                              reserved5        => "0",
                                              assoc            => "0",
                                              des_type         => "3",
                                              reserve6         => "00",
                                              des_len          => "10",
                                              designator       => "",      #This is src lun serial number
                                              reserved24_27    => "00 00 00 00",
                                              dev_type_params  => "00 00 00 00",
                                            },
                         dest_target_desc=> {
                                              desc_code        => "e4",
                                              luid_type        => "0",
                                              nul              => "0",
                                              per_dev_t        => "0",
                                              rel_init_port_id => "00 00", 
                                              reserve4         => "0",
                                              code_set         => "1",
                                              reserved5        => "0",
                                              assoc            => "0",
                                              des_type         => "3",
                                              reserve6         => "00",
                                              des_len          => "10",
                                              designator       => "",      #This is dest lun serial number
                                              reserved24_27    => "00 00 00 00",
                                              dev_type_params  => "00 00 00 00",
                                            },
                         block_seg_desc  => {
                                              desc_code        => "02",
                                              reserved1        => "0",
                                              full             => "1",
                                              dc               => "0",
                                              cat              => "0",
                                              desc_len         => "00 18",
                                              src_index        => "00 01",
                                              dest_index       => "00 02",
                                              reserved8_9      => "00 00",
                                            },
                         param_list  =>     {
                                              list_id          => "00",                 #should this be randomly generated?
                                              reserved1        => "0",
                                              str              => "0",
                                              nrcr             => "0",
                                              nlid             => "0",
                                              priority         => "0",
                                              target_list_len  => "00 60",              #for now there are always only 3
                                              reserved4_7      => "00 00 00 00",
                                              seg_list_len     => "00 00 00 1c",        #for now there are always only 1
                                              inline_data_len  => "00 00 00 00",        #Can totally leave this blank
                                            },
                         dest_lun   =>''          ,
                         xcopy_cdb  => "83 00 00 00 00 00 00 00 00 00 00 00 00 8c 00 00"   
                     };


    foreach my $param (keys %args) {
        if ($param =~ /^-?src_lun/){
            $src_lun=$args{$param};
        }

        if ($param =~ /^-?dest_lun/){
            $Xcopy_params->{dest_lun}=$args{$param};
        }

        if ($param =~ /^-?bd/){
            $Xcopy_params->{$param}=$args{$param};
        }
    }


    #
    #Getting IP address for source lun
    #
    my $ret_ref = $self->send_cmd(-device      => $src_lun,
                                  -cdb         => 'inquiry_page_85h'
                                 );
    
    #
    #8th byte is length of ip address, form "http:/IP/"
    #
    my $ip_length = hex($ret_ref->{SCSI_DATA}[7]);
    for(my $i=0;$i<$ip_length;$i++){
        push @ip_bytes,$ret_ref->{SCSI_DATA}[8+$i];
    }
    my $ip_addr = pack("H*", join '', @ip_bytes);
    $ip_addr =~ s/http\:\/\/(\d+)\.(\d+)\.(\d+)\.(\d+)\//$1 $2 $3 $4/;
    
    $Xcopy_params->{src_ip_desc}->{ipv4_addr}= sprintf("%02x %02x %02x %02x",split / /,$ip_addr);

    unless ($Xcopy_params->{src_ip_desc}->{ipv4_addr}){
        $self->_set_err("Problem parsing source IP address for Xcopy command.  IP address not found");
    }
    $self->sanlog(-msg => "Source IP address is ".$Xcopy_params->{src_ip_desc}->{ipv4_addr}."\n",
                  -msg_level => 3);
    
    #
    #Getting serial number for source lun
    #
    $ret_ref = $self->send_cmd(-device   => $src_lun,
                               -cdb      => 'inquiry_page_83h'
                              );
    $Xcopy_params->{src_target_desc}->{designator}=$ret_ref->{SCSI_DATA}[44]." ".$ret_ref->{SCSI_DATA}[45]." ".
                                                   $ret_ref->{SCSI_DATA}[46]." ".$ret_ref->{SCSI_DATA}[47]." ".
                                                   $ret_ref->{SCSI_DATA}[48]." ".$ret_ref->{SCSI_DATA}[49]." ".
                                                   $ret_ref->{SCSI_DATA}[50]." ".$ret_ref->{SCSI_DATA}[51]." ".
                                                   $ret_ref->{SCSI_DATA}[52]." ".$ret_ref->{SCSI_DATA}[53]." ".
                                                   $ret_ref->{SCSI_DATA}[54]." ".$ret_ref->{SCSI_DATA}[55]." ".
                                                   $ret_ref->{SCSI_DATA}[56]." ".$ret_ref->{SCSI_DATA}[57]." ".
                                                   $ret_ref->{SCSI_DATA}[58]." ".$ret_ref->{SCSI_DATA}[59];

    unless ($Xcopy_params->{src_target_desc}->{designator}){
        $self->_set_err("Error:  page 83 inquiry to source lun did not return data. Possible error?");
    }
    $self->sanlog(-msg => "Source lun serial number is ".$Xcopy_params->{src_target_desc}->{designator}."\n",
                  -msg_level => 3);

    #
    #Getting serial number for destination lun
    #        
    $ret_ref = $self->send_cmd(-device   => $Xcopy_params->{dest_lun},
                               -cdb      => 'inquiry_page_83h'
                              );
    $Xcopy_params->{dest_target_desc}->{designator}=$ret_ref->{SCSI_DATA}[44]." ".$ret_ref->{SCSI_DATA}[45]." ".
                                                    $ret_ref->{SCSI_DATA}[46]." ".$ret_ref->{SCSI_DATA}[47]." ".
                                                    $ret_ref->{SCSI_DATA}[48]." ".$ret_ref->{SCSI_DATA}[49]." ".
                                                    $ret_ref->{SCSI_DATA}[50]." ".$ret_ref->{SCSI_DATA}[51]." ".
                                                    $ret_ref->{SCSI_DATA}[52]." ".$ret_ref->{SCSI_DATA}[53]." ".
                                                    $ret_ref->{SCSI_DATA}[54]." ".$ret_ref->{SCSI_DATA}[55]." ".
                                                    $ret_ref->{SCSI_DATA}[56]." ".$ret_ref->{SCSI_DATA}[57]." ".
                                                    $ret_ref->{SCSI_DATA}[58]." ".$ret_ref->{SCSI_DATA}[59];

     unless ($Xcopy_params->{src_target_desc}->{designator}){
                 $self->_set_err("Error:  page 83 inquiry to destination lun did not return data. Possible error?");
     }
     $self->sanlog(-msg => "Destination lun serial number is ".$Xcopy_params->{dest_target__desc}->{designator}."\n",
                   -msg_level => 3);
                     
    return $Xcopy_params;
}#end sub 




########################################################################
### Method:    Xcopy_send_cmd
### Objective: Build the Xcopy CDB and parameter list, actually sends command
### Details:   See http://wikid/w/SAN_QA/Projects/Protocol_Intern/Xcopy_Functional_spec
##########################################################################

sub Xcopy_send_cmd
{
    my ($self, $Xcopy_params) = @_;
    my ($luid_type, $nul, $per_dev_id, $byte_1, $reserve, $assoc);
    my ($str, $nrcr, $nlid, $priority, $byte_5, $seg_desc, $src_lun);
    my ($dest_lun, $src_ip, $param_header, @Xcopy_return, $full, $dc, $cat);
    my ($src_addr, $dest_addr, $block_num);
    
    #
    #Creating ID Target Descriptor for Destination LUN
    #luid_type, nul, per_dev_id all make up one byte in the data, but
    #are changed as decimal number inputs.  So we have to break those
    #down into binary and then recompile them as the hex string we want
    #
    $luid_type = $Xcopy_params->{dest_target_desc}->{luid_type};
    $luid_type = unpack("B32", pack("N", $luid_type));
    $luid_type =~ s/^0+(?=\d)//;

    $nul = $Xcopy_params->{dest_target_desc}->{nul};
    $nul = unpack("B32", pack("N", $nul));
    $nul =~ s/^0+(?=\d)//;

    $per_dev_id = $Xcopy_params->{dest_target_desc}->{per_dev_id};
    $per_dev_id = unpack("B32", pack("N", $per_dev_id));
    $per_dev_id =~ s/^0+(?=\d)//;
    
    $byte_1=unpack("H*", pack ("B*", $luid_type.$nul.$per_dev_id));

    #
    #The reserved and assoc values make up one byte in the data, but
    #are changed as decimal number inputs.  We have to break these
    #down into binary and then recompile them as the hex string we want
    #
    $reserve = $Xcopy_params->{dest_target_desc}->{reserved5};
    $reserve = unpack("B32", pack("N", $reserve));
    $reserve =~ s/^0+(?=\d)//;
 
    $assoc = $Xcopy_params->{dest_target_desc}->{assoc};
    $assoc = unpack("B32", pack("N", $assoc));
    $assoc =~ s/^0+(?=\d)//;

    $byte_5 = unpack("H*", pack ("B*", $reserve.$assoc));
    $byte_5 =~ s/(\d)(\d)/$2/;

    
    #
    #actually building the destination target descripor block
    #    
    $dest_lun=$Xcopy_params->{dest_target_desc}->{desc_code}." ".
                     $byte_1." ".
                     $Xcopy_params->{dest_target_desc}->{rel_init_port_id}." ".
                     $Xcopy_params->{dest_target_desc}->{reserve4}.
                     $Xcopy_params->{dest_target_desc}->{code_set}." ".
                     $byte_5.$Xcopy_params->{dest_target_desc}->{des_type}." ".
                     $Xcopy_params->{dest_target_desc}->{reserve6}." ".
                     $Xcopy_params->{dest_target_desc}->{des_len}." ".
                     $Xcopy_params->{dest_target_desc}->{designator}." ".
                     $Xcopy_params->{dest_target_desc}->{reserved24_27}." ".
                     $Xcopy_params->{dest_target_desc}->{dev_type_params};

    $self->sanlog(-msg=>"Destination target descriptor block is ".$dest_lun."\n",
                  -msg_level=> 3);
                     

    #
    #Creating ID Target Descriptor for Source LUN
    #luid_type, nul, per_dev_id all make up one byte in the data, but
    #are changed as decimal number inputs.  So we have to break those
    #down into binary and then recompile them as the hex string we want
    #    
    $luid_type = $Xcopy_params->{src_target_desc}->{luid_type};
    $luid_type = unpack("B32", pack("N", $luid_type));
    $luid_type =~ s/^0+(?=\d)//;

    $nul = $Xcopy_params->{src_target_desc}->{nul};
    $nul = unpack("B32", pack("N", $nul));
    $nul =~ s/^0+(?=\d)//;

    $per_dev_id = $Xcopy_params->{src_target_desc}->{per_dev_id};
    $per_dev_id = unpack("B32", pack("N", $per_dev_id));
    $per_dev_id =~ s/^0+(?=\d)//;
    
    $byte_1=unpack("H*", pack ("B*", $luid_type.$nul.$per_dev_id));

    #
    #The reserved and assoc values make up one byte in the data, but
    #are changed as decimal number inputs.  We have to break these
    #down into binary and then recompile them as the hex string we want
    #
    $reserve = $Xcopy_params->{src_target_desc}->{reserved5};
    $reserve = unpack("B32", pack("N", $reserve));
    $reserve =~ s/^0+(?=\d)//;
 
    $assoc = $Xcopy_params->{src_target_desc}->{assoc};
    $assoc = unpack("B32", pack("N", $assoc));
    $assoc =~ s/^0+(?=\d)//;

    $byte_5 = unpack("H*", pack ("B*", $reserve.$assoc));
    $byte_5 =~ s/(\d)(\d)/$2/;

    #
    #Actually building the source lun descriptor block.
    #same format as above, but different information
    #    
    $src_lun= $Xcopy_params->{src_target_desc}->{desc_code}." ".
                     $byte_1." ".
                     $Xcopy_params->{src_target_desc}->{rel_init_port_id}." ".
                     $Xcopy_params->{src_target_desc}->{reserve4}.
                     $Xcopy_params->{src_target_desc}->{code_set}." ".
                     $byte_5.$Xcopy_params->{src_target_desc}->{des_type}." ".
                     $Xcopy_params->{src_target_desc}->{reserve6}." ".
                     $Xcopy_params->{src_target_desc}->{des_len}." ".
                     $Xcopy_params->{src_target_desc}->{designator}." ".
                     $Xcopy_params->{src_target_desc}->{reserved24_27}." ".
                     $Xcopy_params->{src_target_desc}->{dev_type_params};

    $self->sanlog(-msg=> "Source target descriptor block is ".$src_lun."\n",
                  -msg_level=> 3);
                     
                     
    #
    #Creating IPv4 target Descriptor for Souce Copy Manager
    #Same type of breaking down and reassembling done here.
    #
    
    $luid_type = $Xcopy_params->{src_ip_desc}->{luid_type};
    $luid_type = unpack("B32", pack("N", $luid_type));
    $luid_type =~ s/^0+(?=\d)//;

    $nul = $Xcopy_params->{src_ip_desc}->{nul};
    $nul = unpack("B32", pack("N", $nul));
    $nul =~ s/^0+(?=\d)//;

    $per_dev_id = $Xcopy_params->{src_ip_desc}->{per_dev_id};
    $per_dev_id = unpack("B32", pack("N", $per_dev_id));
    $per_dev_id =~ s/^0+(?=\d)//;

    $byte_1 = unpack("H*", pack ("B*", $luid_type.$nul.$per_dev_id));

    #
    #actually building the source ip target descriptor block
    #
    $src_ip=$Xcopy_params->{src_ip_desc}->{desc_code}." ".
                   $byte_1." ".
                   $Xcopy_params->{src_ip_desc}->{rel_init_port_id}." ".
                   $Xcopy_params->{src_ip_desc}->{lu_id}." ".
                   $Xcopy_params->{src_ip_desc}->{ipv4_addr}." ".
                   $Xcopy_params->{src_ip_desc}->{reserve16_21}." ".
                   $Xcopy_params->{src_ip_desc}->{port_num}." ".
                   $Xcopy_params->{src_ip_desc}->{reserve24_25}." ".
                   $Xcopy_params->{src_ip_desc}->{it_prot_num}." ".
                   $Xcopy_params->{src_ip_desc}->{dev_type_params};

    $self->sanlog(-msg=>"IPv4 target descriptor block is ".$src_ip."\n",
                  -msg_level=> 3);

    #
    # Loop for multiple Segment Descriptors as defined in xfer_array
    # IMPORTANT: I AM MAKING THE ASSUMPTION THAT THE TARGET DESCRIPTOR ORDER IS
    # ALWAYS SOURCE IP, SOURCE LUN, DEST LUN
    # Note:  the header is always the same, so I make it before the loop
    #

    $reserve = $Xcopy_params->{param_list}->{reserved1};
    $reserve = unpack("B32", pack("N", $reserve));
    $reserve =~ s/^0+(?=\d)//;

    $str = $Xcopy_params->{param_list}->{str};
    $str = unpack("B32", pack("N", $str));
    $str =~ s/^0+(?=\d)//;

    $nrcr = $Xcopy_params->{param_list}->{nrcr};
    $nrcr = unpack("B32", pack("N", $nrcr));
    $nrcr =~ s/^0+(?=\d)//;

    $nlid = $Xcopy_params->{param_list}->{nlid};
    $nlid = unpack("B32", pack("N", $nlid));
    $nlid =~ s/^0+(?=\d)//;

    $priority = $Xcopy_params->{param_list}->{priorty};
    $priority = unpack("B32", pack("N", $priority));
    $priority =~ s/^0+(?=\d)//;

    $param_header = $Xcopy_params->{param_list}->{list_id}." ".
                           unpack("H*", pack ("B*", $reserve.$str.$nrcr.$nlid.$priority))." ".
                           $Xcopy_params->{param_list}->{target_list_len}." ".
                           $Xcopy_params->{param_list}->{reserved4_7}." ".
                           $Xcopy_params->{param_list}->{seg_list_len}." ".
                           $Xcopy_params->{param_list}->{inline_data_len};

    #
    #Making byte 1 hex out of reserved (0), full, DC, and cat
    #
    $reserve = $Xcopy_params->{block_seg_desc}->{reserved1};
    $reserve = unpack("B32", pack("N", $reserve));
    $reserve =~ s/^0+(?=\d)//;

    $full = $Xcopy_params->{block_seg_desc}->{full};
    $full = unpack("B32", pack("N", $full));
    $full =~ s/^0+(?=\d)//;

    $dc = $Xcopy_params->{block_seg_desc}->{dc};
    $dc = unpack("B32", pack("N", $dc));
    $dc =~ s/^0+(?=\d)//;
 
    $cat = $Xcopy_params->{block_seg_desc}->{reserved1};
    $cat = unpack("B32", pack("N", $cat));
    $cat =~ s/^0+(?=\d)//;

    $byte_1 = sprintf("%08u",$reserve.$full.$dc.$cat);
    $byte_1 = unpack("H*", pack ("B*", $byte_1));

    #
    #sending an xcopy command per transfer tuple in array
    #hich describes where to move information between luns
    #
    foreach my $param (keys %$Xcopy_params){
        if($param =~ /bd/){

            $block_num = sprintf("%04x",$Xcopy_params->{$param}[2]);
            $block_num =~ s/(.{2})(.{2})/$1 $2/;

            $src_addr = sprintf("%016x",$Xcopy_params->{$param}[0]);
            $src_addr =~ s/(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})/$1 $2 $3 $4 $5 $6 $7 $8/; 

            $dest_addr = sprintf("%016x",$Xcopy_params->{$param}[1]);
            $dest_addr =~ s/(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})(.{2})/$1 $2 $3 $4 $5 $6 $7 $8/; 

            $seg_desc = $Xcopy_params->{block_seg_desc}->{desc_code}." ".
                                                             $byte_1." ".
                         $Xcopy_params->{block_seg_desc}->{desc_len}." ".
                        $Xcopy_params->{block_seg_desc}->{src_index}." ".
                       $Xcopy_params->{block_seg_desc}->{dest_index}." ".
                      $Xcopy_params->{block_seg_desc}->{reserved8_9}." ".
                             $block_num." ".$src_addr." ".$dest_addr." ";

            $self->sanlog(-msg=> " Xcopy CDB: ".$Xcopy_params->{xcopy_cdb}."\n".
                                 " Xcopy data out: ".$param_header." ".$src_ip." ".$src_lun." ".$dest_lun." ".$seg_desc."\n".
                                 " Destination lun recieving command: ".$Xcopy_params->{dest_lun}."\n",
                          -msg_level => 3);

            my $return_ref = $self->send_cmd(-device             =>$Xcopy_params->{dest_lun},
                                                 -cdb            =>$Xcopy_params->{xcopy_cdb},
                                                 -cdb_param      =>$param_header." ".$src_ip." ".$src_lun." ".$dest_lun." ".$seg_desc,
                                            );

            @Xcopy_return=(@Xcopy_return, $return_ref);
        }
    }

    #
    #is an array of return hashes from send_cmd()
    #
    return @Xcopy_return;
} #end sub
1;
