#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/San/ScsitSimulator.pm#1 $
# Copyright (c) 2009 NetApp Inc.
# All rights reserved.
#
## @summary Provide an interface to a HOST machine
## @author chadwell@netapp.com
## @status private
## @pod here

package San::ScsitSimulator;

# Compiler directives.
use strict;

# Module import.
use File::Basename;
use File::Compare;
use Time::HiRes qw(gettimeofday);
use San::SanFiler::Volume;
use San::SanFiler::IGroup;
use San::SanFiler::Target;
use San::SanFiler::ISCSITarget;
use CmdsDriver;
use XML::Simple;
use Data::Dumper;
use San::SanFiler::Cmds::Portset;
use San::SanFiler::Cmds::Fcp;

use Tharn;
use San::CommonInterface;
use San::SanUtil;
use San::SanLog;
use San::SanCache;
use San::SanClient::Lvm;
use San::SanClient::DevMgr;
use San::SanClient::Sanlun;
use San::SanClient::ClientUtil;
use San::SanClient::Initiator;
use San::SanClient::MP;
use San::SanClient::HU;
use WinClient;
use Utils qw(make_host make_conn);
use Error;
use Exporter;

use vars qw($_CLIENT_CNT
            $_FILER
            @EXPORT 
            @ISA
            $AUTOLOAD);

use constant DUMPER_DEBUG_LEVEL => 10;


 
@ISA = qw(Error SanLog San::SanClient);


###############################################################################
#                               Public Methods
###############################################################################


################################################################################
# Subroutine Name: get_protocol
# Objective:       Return protocol type
# Details:         See POD documentation at the end of this file
################################################################################
sub get_protocol {
    my ($self) = @_;

    $self->_clear_err();

    if (defined $self->{protocol}) {
        return $self->{protocol};
    } else {
        $self->_set_err("Error: SanClient::get_protocol() SanClient".
                        "object is missing protocol type");
        return "";
    }
}

###############################################################################
# Method:    get_name()
# Objective: Get filer name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_name
{
    my ($self) = @_;

    $self->_clear_err();

    return $self->{filer} if (defined $self->{filer});    

    $self->_set_err("Error: SanFiler::get_name(): Unable to get filer name");
    return "";
}


###############################################################################
# Method:    hosttype()
# Objective: Get client host type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub hosttype {
    my ($self) = @_;

    return $self->{conn}{hostp}{hosttype} if ($self->{conn}{hostp}{hosttype});

    $self->_set_err("Can not determine host type");
    return "";

}


###############################################################################
# Method:    execute
# Objective: Issue arbitrary command to host and get results.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute
{
    my($self, %args) = @_;
    if (ref($self)=~ /Simulator/){
        $self->execute_as_filer(-cmd => $args{-cmd});
    }
    elsif ( ref($self)=~"SanClient") {
       ClientUtil::execute($self->{connection}, %args, $self->{timeout});
    }

} # end sub execute
                                                                                                    

###############################################################################
# Method:    get_connection()
# Objective: Get nate connection object
# Details:   See POD documentation at the end of this file.
###############################################################################
sub get_connection {
    my ($self) = @_;

    return $self->{connection};
}

###############################################################################
## Method:    timeout
## Objective: set or get telnet timeout
## Details:   See POD documentation at the end of this file.
################################################################################
sub timeout {
    my ($self, $timeout) = @_;

        if (defined $timeout and length $timeout) {
             $self->{timeout} = $timeout;
        } else {
             return $self->{timeout};
        }
}

###############################################################################
### Method:    get_scsit_devices 
### Objective: returns device array for simulator 
### Details:   none.
#################################################################################
sub get_scsit_devices{
    my ($self) = @_;
    
    #
    # Sending command to linux host to print IT nexus file.
    # This will return the pipes list used to send commands
    # to the scsit simulator
    #
    my @output = $self->execute(-cmd=>"cat iflist");
    while($output[0] !~ /^pipes/){
        #
        # this cuts out the top row of array @output
        #
        splice(@output,0,1);
    }
    return (wantarray()) ? @output : join("\n", @output);
}

###############################################################################
### Method:    get_filer 
### Objective: returns box name 
### Details:   none.
#################################################################################
sub get_filer{
    my ($self) = @_;

    return $self->{filer};
}

###############################################################################
### Method:    get_cmd_interface 
### Objective: returns command interface 
### Details:   none.
#################################################################################
sub get_cmd_interface{
    my ($self) = @_;

    return $self->{interface};
}

###############################################################################
# Method:    AUTOLOAD()
# Objective: Mention no-op routines and add debugging level information on
#             routines that should be added.  AUTOLOAD picks up all the filer 
#             commands that are not implemented with the scsit simulator and attempts
#             to send information back that keeps the tests and interacting modules happy.
# Details:   No Pod Documentation
###############################################################################
sub AUTOLOAD{
    my ($self)=@_;
    
    $self->sanlog(-msg => $AUTOLOAD. " is a no-op with the scsit_simulator".
                                     "...Returning happiness and success.\n".
                                     "$AUTOLOAD wants". wantarray()."\n"
                  -msg_level=> 3);
                                    
    my $obj={};

    if ($AUTOLOAD =~ /DESTROY|destroy/) {
        return $ClientUtil::TRUE;

    #
    # If the calling program is calling get_luns, it is going
    # to attempt to make further calls through the object returned
    # by get_luns.  I return a blessed object of the ScsitSimulator type
    # so that those calls will also get filtered through this autoloader
    #
    }elsif ($AUTOLOAD =~ /get_luns/) {
        if (wantarray()) {
            return bless $obj, "San::ScsitSimulator";
        }
        $obj=[bless $self, "San::ScsitSimulator"];
        return bless $obj, "San::ScsitSimulator";
    }else{
        return wantarray() ? (): bless $obj, "San::ScsitSimulator";
    }
}


###############################################################################
#                               Private Methods
###############################################################################

###############################################################################
# Method:    _new
# Objective: Create new Host object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _new{
    my($class, %args) = @_;

    my $self={};
    $class = ref($class) || $class;
    bless $self, $class;
    
    if ($args{-filer} =~ /sanqa/) {
        $self->_new_scsit_filer(%args)
    }
        #elsif($args{-filer} !~ /sanqa/)
        #{$self->_new_scsit_client(%args)}
    else {
        $self->_set_err("-type contains neither filer or client or -type is wrong argument");
    }
}

###############################################################################
# Method:    _new_scsit_sanclient
# Objective: Create new fake san client object.
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new_scsit_client {
    my ($proto, %args) = @_;

    my ($subr, $execute, $host, $connection, $conn_type, $log, $passwd);
    my ($protocol, $rc, $show_cmd, $timeout, $user,   $multipath, $debug_level);
    my ($winclient_obj, $device_type, $sanblaze_ports, $initiator_count);
    my ($alua_support, $mp_type, $load_inits, $test_bin_dir, $pnate_environment);
    my ($connection_retries, @protocols, %hArgs, $manufacturer);

    my $self = {}; 
    my $sanlun_obj = {};

    my $class = ref($proto) || $proto;
    bless ($self, $class); 

    # init variables
    $protocol           = "nfs";
    $multipath          = 1;
    $debug_level        = 0;
    $device_type        = "raw";
    $timeout            = 1800;
    $mp_type            = "none";
    $load_inits         = 1;
    $test_bin_dir       = "NOT_SET";
    $pnate_environment  = 0;
    $connection_retries = 5;
    
    # Process arguments.  If an error occurs during the arg processing,
    # The error flag and errmsg values are set and the object is returned.
    # It is up to the caller to check for errors after the new call.
    foreach (keys %args) {
        if (/-?host$/) {
            length($host = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"host\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/^-?connection$/) {
            length($connection = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"connection\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/^-?conn_type$/) {
            length($conn_type = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"conn_type\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/^-?user$/) {
            length($user = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"user\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/^-?passwd$/) {
            length($passwd = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"passwd\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/-?timeout$/) {
            length($timeout = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"timeout\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/-?conn_retries$/) {
            length($connection_retries = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"conn_retries\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/-?debug_level$/) {
            length($debug_level = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"debug_level\":",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/-?device_type$/) {   
            if (not length($device_type = $args{$_}) or $device_type !~ /raw|lv|fs/i) {
                $self->_set_err("SanClient::new invalid value for \"device_type\":".$args{$_});
                return $self;
                
            }
            next;
        } elsif (/-?mp_type$/) {
            if (not length($mp_type = $args{$_})) {
                $self->_set_err("SanClient::new invalid value for \"mp_type\":".$args{$_});
            }
            next;
        } elsif (/-?load_inits$/) {
            if (not length($load_inits = $args{$_}) or $load_inits !~ /1|0/) {
                $self->_set_err("SanClient::new invalid value for \"load_inits\":".$args{$_});
            }
            next;
        } elsif (/-?pnate_environment$/) {
            if (not length($pnate_environment = $args{$_}) or $pnate_environment !~ /1|0/) {
                $self->_set_err("SanClient::new invalid value for \"pnate_environment\":".$args{$_});
            }
            next;
        }
        elsif (/-?test_bin_dir$/) {
            if (not length($test_bin_dir = $args{$_})) {
                $self->_set_err("SanClient::new invalid value for \"test_bin_dir\":" . $args{$_});
            }
            next;
        } elsif (/-?protocol$/) {
            # Default to nfs if exist and not user defined 
            # (Note: Undefined get_parameter returns 0)
            $protocol = $args{$_};
            if (exists $args{$_} and $args{$_} =~ /0/) {
                $protocol = "nfs";
            } else {
                unless (exists $args{$_} and $args{$_} =~ /^nfs$|^fcp$|^iscsi$/i) {
                    $self->_set_err("SanClient::new invalid value for \"protocol\":" . $args{$_});
                    return $self;
                    
                }
            }
        } elsif (/-?show_cmd$/) {
            length($show_cmd = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"show_cmd\"",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/-?loghdl$/) {
            length($log = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for",
                                    "\"log\" $args{$_}");
                    return $self;
                };
        } elsif (/-?multipath$/) {
            if (not length($multipath = $args{$_}) or $multipath !~ /no|yes|1|0|off|on/i) {
                $self->_set_err("SanClient::new invalid value for",
                                "\"multipath\" $args{$_}");
                return $self;
            } else {
                if ($multipath =~ /yes|1|on/) {
                    $multipath=1;
                } else {
                    $multipath=0;
                }
            }
            next;
        } elsif (/-?_obsolete|-?parent/) {
            next;
            
        } elsif (/-?sanblaze_ports/) {
            length($sanblaze_ports = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"sanblaze_ports\"",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/-?initiator_count/) {
            length($initiator_count = $args{$_}) or
                do {
                    $self->_set_err("SanClient::new invalid value for \"initiator_count\"",
                                    "$args{$_}");
                    return $self;
                };
        } elsif (/-?alua_support$/) {
            if (not length($alua_support = $args{$_}) or $alua_support !~ /no|yes|1|0|off|on/i) {
                $self->_set_err("SanClient::new invalid value for",
                                "\"alua_support\" $args{$_}");
                return $self;
            } else {
                if ($alua_support =~ /yes|1|on/) {
                    $alua_support=1;
                } else {
                    $alua_support=0;
                }
            }
            next;
        }
    }
    
    # Verify mandatory parameters.
    unless (defined $host and length $host) {
        $self->_set_err("SanClient::new mandatory parameter \"host\"",
                        "not specified");
        return $self;
    }

    # Set the timeout if one is given.
    $self->timeout($timeout) if (defined $timeout);

    # Set the debug_level if one is given.
    $self->set_debug_level($debug_level);

    # initialize the number of connect retries on the object
    $self->{connection_retries} = $connection_retries;
    
    # Determine if a connection type has been providede
    if (!defined($connection)) {
        # Connect to the host and store the connection obj
        $connection = $self->_connect($host, $user, $passwd, $timeout, $conn_type);
        if ($self->error) {
            # Error message is already set, return object so caller
            # can get the error.
            return $self;
        }
    }
    
    $self->{conn} = $connection;
    
    return $self if ($self->error);
    
    # determine the right platform and require/import/call the appropriate
    # OS object's new call.
    my $hosttype = $self->hosttype();  
   
   if ($hosttype =~ /unix-linux/i) {
        if ($manufacturer =~ /VMware|HyperV|Microsoft/i) {
            require San::SanClient::Host::VLinuxHost;
            $self = San::SanClient::Host::VLinuxHost->_new(-conn => $self->get_connection());
        } else { 
            require San::SanClient::Host::LinuxHost;
            $self = San::SanClient::Host::LinuxHost->_new(-conn => $self->get_connection(), %args);
        }
   }
    # check if there were any errors from per platform constructor
    if ($self->error()) {
        my @error = $self->errmsg();
        my $errmsg  = join('\n', @error);
        $self->sanlog(-msg => "Error: SanClient::new() Error from per-platform ($hosttype) SanClient constructor\n" .
                              $errmsg);
        return $self;
    }

    if (not defined($self->{arch})) {
        $self->sanlog(-msg => "SanClient::new() Host Architecture is not set");
    }

    # Store connection parameters in object.
    $self->{host} =  $host;
    $self->{user} =  $user;
    $self->{pass} =  $passwd;
    
    # Initially connected.
    $self->{connected} = 1;

    # save value of conn_type in host object
    $self->{conn_type} = $conn_type;
    
    # save the connection again
    $self->{conn} = $connection;
    
    # Show command mode.  By default when printing, the command
    # issued will be displayed.  This can be turned off by the user.
    $self->{show_cmd} = defined $show_cmd ? $show_cmd : 1;

    $self->{show_data} = 1;
    $self->{protocol}  = $protocol;
    $self->{multipath} = $multipath;
    
    # set client device type; default is "raw"
    $self->{device_type} = $device_type;

    # Allow CLI commands to be formed, but not executed.
    $self->{execute} = defined $execute ? $execute : 1;

    # Allow CLI commands to be logged;
    $self->{log} = $log if (defined $log);

    # Set the timeout if one is given.
    $self->timeout($timeout) if (defined $timeout);
    
    # Set the test_bin_dir if provided
    if ($test_bin_dir ne "NOT_SET") {
        $self->{TEST_BIN} = $test_bin_dir;
    }

    # Set the pnate_environment flag
    $self->{pnate_environment} = $pnate_environment;

    # initialize the connect retries on the object
    $self->{connection_retries} = $connection_retries;

    # Setting the multipathing type if given, check to see if it's already
    # set (possibly by platform specific _new())
    if (not defined ($self->{mp_type})) {
        $self->{mp_type} = $mp_type;
    } else {
        # if the default mp_type was set in the host contructor and
        # it is not the same as what was passed (and it not "none")
        # use what was passed in (instead of the default set in the
        # contructor).
        if (($self->{mp_type} ne $mp_type) and ($mp_type ne "none")) {
            $self->{mp_type} = $mp_type;
        }
    }
    
    # Set the debug_level if one is given.
    $self->set_debug_level($debug_level);

    # Set the alua_support if one is given, else default to off (0)
    if (defined($alua_support)) {
        $self->{alua_support} = $alua_support;
    } else {
        $self->{alua_support} = 0;
    }
   
    # Load configuration params for iSCSI, fcp, and hu
    $self->_load_config_params();
    
    # Check if client supports user specified protocol.
    # Retrieve list of supported protocols and compare 
    # to what was  passed in
    $self->sanlog(-msg => "Obtaining support protocols");
    @protocols = $self->get_supported_protocols();
    $self->sanlog(-msg => "Obtaining the specified protocol");
    $protocol = $self->get_protocol();
    $self->sanlog(-msg => "Protocol used here is $protocol");

    unless ("@protocols" =~ /$protocol/i) {
        $self->_set_err("Error: SanClient::new() $protocol protocol on " .
                        $self->hosttype() .
                        " is currently not supported by SanClient object");
        return $self;
    }
    $self->{load_inits} = $load_inits;
    if ($protocol =~ /fcp|iscsi/i && $self->{load_inits}) {
        $hArgs{-loghdl} = $self->{log} if (defined $self->{log});

        # Create a Sanlun object with the parent being passed to get protocol and conn
        $sanlun_obj = Sanlun->new(-parent => $self,
                                  %hArgs);
        $self->sanlog(-msg => "Sanlun object created");
        if ($sanlun_obj->error) {
            $self->_set_err($sanlun_obj->errmsg());
            return $self;                                    
        }

        $sanlun_obj->set_debug_level($self->get_debug_level());
        $self->{sanlun} = $sanlun_obj;

        # Load initiators
        $self->sanlog(-msg => "SanClient::new() Loading Initiators");
        $self->_load_initiators($self->get_protocol());
        return $self if ($self->error);
    } else {
        $self->sanlog(-msg => "SanClient::new() Load Initiators skipped due to (-load_inits=>" .
                              $self->{load_inits} . ") being defined");
    }

    # Create the multipathing object if mp_type is specified
    if ($self->{mp_type} ne "" && $self->{mp_type} !~ /^none$/i) {
        $self->sanlog(-msg => "Creating MultiPathing (MP) Object");
        $self->{mp} = MP->new(-mp_type => $self->{mp_type}, -hosttype => $self->hosttype(), -parent => $self);
        if ($self->{mp}->error) {
            $self->sanlog(-msg => "Error: SanClient::new() unable to create MP object");
            $self->_set_err($self->{mp}->errmsg());
            return $self;
        }
        $self->sanlog(-msg => "MultiPathing (MP) Object Created (type = $self->{mp_type})");
    }
    $self->{mp}->{mp_type} = $self->{mp_type};

    return $self;
}  #end sub _new_scsit_sanclient

###############################################################################
# Method:    _new_scsit_filer
# Objective: Create new SanFiler7M object
# Details:   See POD documentation at the end of this file.
###############################################################################

sub _new_scsit_filer
{
    my($class, %args) = @_;
    my(
       $execute,
       $filer,
       $log,
       $passwd,
       $rc,
       $self,
       $show_cmd,
       $timeout,
       $debug_level,
       $user,
       $connection,
       $conn_type,
       $cmd_obj,
       $filer_version,
       $interface,
       $vfiler,
       $vfiler_ip,
       $partner_mode,
       $iscsi_alias,
       $type,
       );

    # Create CLI object.  Create it before processing args so
    # we can perform full error reporting during the arg
    # processing stage.
    $self = {};
    bless $self;

    $debug_level    = 1;
    $interface      = "cli";     # cli, zapi, zapi_cli
    $connection     = undef;
    $conn_type      = "#rsh";

    # Process arguments.  If an error occurs during the arg processing,
    # The error flag and errmsg values are set and the object is returned.
    # It is up to the caller to check for errors after the new call.
    no strict 'refs';
    foreach (keys %args) {
         if (/^-?filer$/) {
            length($filer = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"filer\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?timeout$/) {
            length($timeout = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"timeout\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?show_cmd$/) {
            length($show_cmd = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"show_cmd\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?debug_level$/) {
            length($debug_level = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"debug_level\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?loghdl$/) {

            length($log = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"loghdl\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?connection$/) {

            length($connection = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"connection\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?conn_type$/) {
            length($conn_type = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"conn_type\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?interface$/) {
            length($interface = $args{$_}) or
                do {
                    $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                                    "value for \"interface\": $args{$_}");
                    return $self;
                };
        }
        elsif (/^-?type$/) {
            length($type = $args{$_}) or
                do {
                    self ->_set_err("Error: " . ref($self) > "::_new() invalid ",
                                    "value for \"type\": $args{$_}");
                }
        }
        else {
            $self->_set_err("Error: " . ref($self) . "::_new() invalid ",
                            "parameter specified: $_");
            return $self;
        }
    }

    #set the filer name and interface
    $self->{filer}     = $filer;
    $self->{interface} = $interface;
    use strict 'refs';
    # Initially unconnected.
    $self->{connected} = 0;

    # Show command mode.  By default when printing, the command
    # issued will be displayed.  This can be turned off by the user.
    $self->{show_cmd} = defined $show_cmd ? $show_cmd : 1;

    # Debug level, higher gives more data, > 100 gives call stack
    $self->set_debug_level($debug_level);

    $self->{show_data}=1;
    $self->{debug}=0;

    # Allow CLI commands to be formed, but not executed.
    $self->{execute} = defined $execute ? $execute : 1;

    # Allow CLI commands to be logged;
    $self->{log}=$log if (defined $log);

    # Determine timeout to use
    $self->{timeout}=(defined $timeout) ? $timeout : 60;
       
    # If we were given a connection, store it.  Otherwise we should
    # make a connection to the filer.
    unless (defined $connection) 
    {
        $self->{$conn_type}= $conn_type;
        $self->{connection}= $self->_connect($filer, $timeout, $conn_type);
        return $self if ($self->error);
        $self->sanlog(-msg => ref($self) . "::_new(): Created a connection");
    }
    else 
    {
        $self->{$conn_type}= $conn_type; #$connection;
        $self->{connection}= $connection;
        $self->sanlog(-msg => ref($self) . "::_new(): Using provided connection");
    }
    
    #Create a CmdsDriver object
       $cmd_obj = new CmdsDriver(-log          => \*ntest::NTEST_STDOUT,
                                 -debug_level  => $debug_level);

       if ($cmd_obj->error())
       {
          my $errmsg = join "\n", $cmd_obj->errmsg();
          $self->_set_err("Failed to instantiate CmdsDriver object " . $errmsg);
           return $self;
       }

      # Set CmdsDriver connection
     $cmd_obj->connection($self->{connection});
    
    
   #
    # Only create a connection for cli 
    #
    if ($self->{interface} eq "cli") {
        #
        # TBD, until we have the ability to rely fully on the zapi interface
        # we'll always make a connection.
        #
        # Set SanFiler connection
        # $self->{connection}=$self->_connect($filer, $timeout);
        # return $self if($self->error);
    }

    # Load filer iscsi targets
    $self->{iscsi_targets} = [];
    $self->_load_iscsi_targets();

    return $self;
} # end sub _new_scsit_filer

###############################################################################
# Method:    execute_as_filer
# Objective: execute command and return results as an array of hash records
# Details:   See POD documentation at the end of this file.
###############################################################################
sub execute_as_filer {
    my ($self, %args) = @_;

    my ($buffer, $cmd, $conn, $connMgr_obj, $errmsg, @records, $raw_data);
    my ($timeout, $sec_str);

    $self->_clear_err();

    # set default value
    $raw_data = $SanUtil::FALSE;
    $timeout  = 1800;
    $sec_str  = "second";

    # remove the dash '-' from input args if exist
    # SanUtil::strip_dash
    %args = &SanUtil::strip_dash(%args);

    # process parameters
    while (my ($key, $value) =  each(%args)) { 
        if ($key =~ /-?cmd$/) {
            if (! length($cmd = $value)) {
                $self->_set_err("invalid value for \"-$key\": " . $value);
                return (wantarray()) ? () : [];
            }
        } elsif ($key =~ /-?raw_data$/) {
            if (! length($raw_data = $value) || $value !~ /$SanUtil::TRUE|$SanUtil::FALSE/) {
                $self->_set_err("invalid value for \"-$key\": " . $value);
                return (wantarray()) ? () : [];
            }
        } elsif ($key =~ /-?timeout$/) {
            if (! length($timeout = $value) || $value !~ /\d+/) {
                $self->_set_err("invalid value for \"-$key\": " . $value);
                return (wantarray()) ? () : [];
            }
        }
    }

    # verify mandatory cmd argument
    unless (defined $cmd and length $cmd) {
        $self->_set_err("mandatory parameter \"-cmd\" not specified ");
        return (wantarray()) ? () : [];
    }

    # retrieve handle to connectionMgr object
    # this should be get_conn_obj()
    #$connMgr_obj = $self->{conn_obj} if (defined $self->{conn_obj});
    $conn = $self->get_connection(-filer     => $self->get_filer(),
                                  -cmd_interface => $self->get_cmd_interface());

    # place '-->' before each command execution
    $self->sanlog(-msg => "--> (cmd: $cmd)");

    # set start times
    my $start_time   = [Time::HiRes::gettimeofday()];

    # issue command and get results.
    eval {
        (undef, undef, $buffer, undef) = Tharn::say($conn, $cmd, -timeout => $timeout);
        $buffer =~ s/\cM//g;
    };
    if ($@) {
        $self->_set_err("failed to execute: " . $cmd . $@);
        return (wantarray()) ? () : [];
    }

    # get elapsed times
    my $elapsed =  Time::HiRes::tv_interval($start_time);    

    # determine singular or plural second
    unless ($elapsed =~ /^(0|1)\./) {
        $sec_str .= "s";
    }    

    # place '<--' after each command execution
    $self->sanlog(-msg => "<-- (elapsed: $elapsed $sec_str)");

    # determine if output contains error
    if ($buffer =~ /ERROR:/i) {

        # extract only error message and set it
        ($buffer) = $buffer =~ /ERROR: (.*)/s;
        $self->_set_err($buffer);
        return (wantarray()) ? ($buffer) : [$buffer];
    }
    
    return (wantarray()) ? split(/\n/, $buffer): $buffer;
} ## end sub execute_as_filer


################################################################################
## Subroutine Name: _get_preferred_conn_types
## Objective:       get the preferred list of connection types to use on a per
##                  platform basis. Cannot be in platform host objects,
##                  because _connect is called before per-platform contructors
#################################################################################
sub _get_preferred_conn_types {
   my ($self, $host) = @_;

    my ($host_type);

    $host_type = $host->{hosttype};
    $self->sanlog(-msg=> "At _get_preferred_conn_types, \"host\" is ". $host->{hosttype}."\n");
    if ($host_type =~ /aix/i) {
        return("telnet", "ssh", "rsh");
     } elsif ($host_type =~ /linux-vmware/i) {
        return("ssh");
     } elsif ($host_type =~ /unix-linux/i) {
        return("ssh", "rsh", "telnet");
     } elsif ($host_type =~ /hpux/i) {
        return("rsh", "telnet");
     } elsif ($host_type =~ /solaris/i) {
        return("rsh", "telnet", "ssh");
     } elsif ($host_type =~ /windows/i) {
        return("rsh");
     } else {
        return ("rsh");
     }
 }

##############################################################################
# Method:    _connect
# Objective: Create telnet connection to host
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _connect {
    my ($self, $hostname, $user, $passwd, $timeout, $conn_type) = @_;

    my ($connect, $conn_str, $errmsg, $host, $id);
    my ($hostInfo, $retries, @preferred_conns);

    # Variable init.
    $self->{connected} = 0;

    $id = $SanClient::_CLIENT_CNT++;
    
    $host = Utils::make_host($hostname); 
    my $max_retries = $self->{connection_retries} || 5;

    if ($conn_type eq "THARNHOST") {
        $conn_type = "";

        # get list of preferred connection types for this
        # platforms. No error to check in this context.
        @preferred_conns = $self->_get_preferred_conn_types($host);

        # Check to see if any connection types are defined, if not return the default value
        if (!defined($host->{connection_fields}) or 
            scalar(keys(%{$host->{connection_fields}})) == 0) {
            # Set the $conn_type to the default of telnet
            $conn_type = "telnet";
            if ($host->hosttype() =~ /windows/) {
                $conn_type = "rsh";
            }
        } elsif (scalar(@preferred_conns) > 0) {
            # If we find preferred connection types in the 
            # for this platform, try each of those to see if they
            # are defined, if one is, use it.
            foreach my $type (@preferred_conns) {
                if (defined($host->{connection_fields}->{$type})) {
                    $conn_type = $type;
                    last;
                }
            }
        }
    }

        # Go through each remainder field and set $conn_type
        # IF rsh is defined, set that. This means that if the
        # conn type of rsh is defined in the tharnhost file,
        # we set $conn_type to that. To not use rsh, commen
        # out the conn type of rsh from the tharnhost file.
        # This will then check to see if the next available
        # conn type is defined in the thatnhost file.
        # The order of connection type is:
        # 1) rsh
        # 2) ssh
        # 3) telnet
        # 4) rlogin
        # 5) console
        if ($conn_type eq "") {
            if (defined($host->{connection_fields}->{rsh})) {
                $conn_type = "rsh";
            } elsif (defined($host->{connection_fields}->{"rsh-socket"})) {
                $conn_type = "rsh-socket";
            } elsif (defined($host->{connection_fields}->{ssh})) {
                $conn_type = "ssh";
            } elsif (defined($host->{connection_fields}->{telnet})) {
                $conn_type = "telnet";
            } elsif (defined($host->{connection_fields}->{rlogin})) {
                $conn_type = "rlogin";
            } elsif (defined($host->{connection_fields}->{console})) {
                $conn_type = "console";
            }
        } 

        # If conn type is still not defined, default.
        if ($conn_type eq "") {
            $conn_type = "telnet";
            if ($host->hosttype() =~ /windows/) {
                $conn_type = "rsh";
            }
    } elsif ($conn_type eq "") {
        $conn_type = "telnet";
        if ($host->hosttype() =~ /windows/) {
            $conn_type = "rsh";
        }
    } else {
        # Default $conn_type to the specified value user passed in
    }

    unless (defined($timeout) and length($timeout)) {
        $self->timeout($host->{connection_fields}{$conn_type}{timeout});
    }
    
    $conn_str = $hostname . "#" . $conn_type . ";" .
        "username=" . $host->{connection_fields}{$conn_type}{username} . ";" .
        "password=" . $host->{connection_fields}{$conn_type}{password} . ";" .
        "timeout="  . $self->{timeout} . ";inherit=0;";

    $self->sanlog(-msg_level => 3, 
                  -msg => "Connection string is $conn_str\n".
                          "retries will attempt connection $max_retries times.\n"); 
    while (!defined($connect) and $retries < $max_retries) {
        $connect = &Tharn::connect(
            $conn_str,
            -eval=>sub {
                if (@_) {
                    $retries++;
                    $connect = undef;

                    $errmsg = join("", @_);
                    $errmsg =~ s/\n//g;
                                      
                    # Display an error in case user doesn't include error 
                    # handler on object instantiation.  
                    $self->sanlog(-msg => "WARNING: SanClient::_connect() " .
                                          "unable to connect to client '" .
                                          $hostname."': $errmsg\n" . 
                                          "Attempt $retries of $max_retries");
                    &Tharn::disconnect($conn_str, '-ok');
                    &SanUtil::nate_sleep(30);
                }
            });
    }

    # if retry count has been exhausted, set error and return
    if ($retries == $max_retries) {
        $self->_set_err($errmsg);
        return {};
    }
    
    return $connect;

}
