# -*- perl -*-
#  $Id: $
## @summary Library to facilitate retry of say calls. 
## Built for handling stress testing in SSQA
##
## @description
##    This package creates a wrapper subroutine above the  Tharn say command to facilitate 
##    retries in case of timeout of the say command and in case connection is refused because 
##    all rsh sockets/ports are currently in use.
##
## @author boppana@netapp.com
## @status public

package SayRetry;
use strict;
use Exporter;
use Tharn;
use Time::HiRes qw(gettimeofday tv_interval);
use vars ('@ISA','@EXPORT');
@ISA = qw(Exporter);
@EXPORT = qw(say_retry);

## @name ::say_retry
## @usage say_retry(@say_params,,"-retry",3,"-iter_sleep",2,"-cmds_time","local")
## @synopsis
##    Retries the say command until either the call reurns or the retry limit is reached.
##    Adds two extra arguments to teh existing say call: iter_sleep and  retry_limit
##    Retries in case of timeout of the say command and in case connection is refused because 
##    all rsh sockets/ports are currently in use.
##    It does not retry if the prompt string appears and then the connection times out
##    It only retries in cases where the connection does not see a prompt string but times out
##    because the filer is too busy to respond.
##    It also adds a cmds_time flag to teh say call. This call can be used to calculate the
##    amount of time taken to run a particular command. This flag takes three values: filer, local and both
##    Default is filer
##    If filer is specified, it use the time command on the filer to find the time taken to execute a command
##    If the command is a ; seperated list, it will find the time taken to execute each individual command
##    If local is specified, it will use the time on the local machine to find the time taken to execute the command
##    If both is specified, it wil luse bothteh abobe specified methods
##    The times taken to excute command or a ; seperated list of commands 
##    Used if running against a filer under heavy load which takes along time to respond 
##    for the say call
##    Returns the outout of say in an array  (@out)
## @usage 	
##    my(undef,undef,$result)=say_retry($fc,$cmd,"-wait","done","-retry",3,"-iter_sleep",2 );
##    The above call will retry the calling teh say function 3 times 
##    with sleep of 2 seconds between each retry
#######################my $retry_string = 'Err=TIMEOUT waiting for waitstrings\/prompts|Connection refused|Unable to create rsh i\/o socket|could not establish connection';
sub say_retry(@) {

   my @args = @_ ;
   my $retry =10;
   my $sleep_time=90;
   my $cmds_time=0;
   my $cmds_time_args="filer";
   my @splice_ind;
   my @splice_flags;
   my $is_wait=0;
   my @times;
   my $out_index=2;
   my $stime;
   my $etime;
   for (my $i=0;$i <= scalar(@args); $i++){
           if ($args[$i] =~ /retry/) {
                   $retry = $args[$i+1];
                   splice(@args,$i,2);
           }
           if ($args[$i] =~ /iter_sleep/) {
                   $sleep_time = $args[$i+1];
                   splice(@args,$i,2);
           }
           if ($args[$i] =~ /cmds_time/) {
                   $cmds_time = 1;
                   splice(@args,$i,1);
                   if ($args[$i] =~ /filer|local/) {
                      $cmds_time_args=$args[$i];
                      splice(@args,$i,1);
                   }
           }
           if ($args[$i] =~ /wait/) {
                   $is_wait = 1;
           }


   }
   my @out;
   my $i=0;
   if($cmds_time ==1 && $cmds_time_args =~ /filer|both/){
      my $cmd_str = $args[1];
      $cmd_str =~ s/;/;time /g;
      $cmd_str = "priv set diag; time " . $cmd_str;
      $args[1]=$cmd_str;
   }
   while(1){
      eval { 
            $stime = [gettimeofday];
            @out = say(@args);
            $etime = [gettimeofday];

         };
      $i++;
      if($@){
            
           if(($@ =~ /Err=TIMEOUT waiting for waitstrings\/prompts/i) ||($@ =~ /Connection refused/) || ($@ =~ /Unable to create rsh i\/o socket/) || ($@ =~ /could not establish connection/)){
                 if($i < ($retry)){
                    sleep($sleep_time);
                    logcomment("Retry No: $i");
                 }
                 else{
                   # If the retry fails on last retry cycle too then we return what ever say returns to us
                   sleep($sleep_time);
                   logcomment("Retry No: $i - Last Retry");
                   @out = say(@args);
                   last;
                 }
            }
            else {
               logresult("FAIL", abort=>1,"Say returned a non-retry error:\n $@");
               last;   
            }
      }
      else{
            if ($is_wait==1) {
               $out_index=1;
            }
            else {
               $out_index=2;
            }
            if($cmds_time ==1){
               if ($cmds_time_args =~ /filer|both/) {
                  @times = ($out[$out_index] =~ /\d+ms/g) ;
               }
               elsif($cmds_time_args =~ /local|both/)  {
                  push(@times,(tv_interval($stime, $etime) * 1000));
               }
               push(@out,\@times);
            }
            last;
      }
   }
   return(@out);
}

1;
