#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/ServerUtil.pm#1 $

## @summary WAFL utility library
## @author lippe@netapp.com
## @status public
## @pod here

# Package Name
##############
package ServerUtil;

use strict;
use Error;
use Exporter;

use Server;
use San::SanClient::ClientUtil;

use vars qw (
	     @ISA
	     @MIRROR
	     );

@ISA = qw(Error);

###############################################################################
# Method:    new
# Objective: Create new ServerUtil object
# Details:   See POD documentation at the end of this file
###############################################################################
sub new {
    my ($class, %args) = @_;
    my ($log,
	@reply,
	$self,
	$server);

    $self = {};

    bless $self, $class;

    foreach (keys %args) {
	if (/-?server$/) {
	    length($server = $args{$_}) or do {
		$self->_set_err("ServerUtil::new: invalid value for \"server\": $args{$_}");
		return $self;
	    };
	}
	elsif (/^-?loghdl$/) {
	    length($log = $args{$_}) or do {
		$self->_set_err("ServerUtil::new: invalid value for \"loghdl\": $args{$_}");
		return $self;
	    };
	}
	else {
	    $self->_set_err("ServerUtil::new: Invalid parameter specified: $_");
	    return $self;
	}
    }
    
    # check for existance of required -server argument
    unless (grep (/^-server$/, (keys %args))) {
	$self->_set_err("Error: ServerUtil::new(): Missing required argument '-server'");
	return $self; 
    }
	
    # the server param must be a server object
    unless ($$server->isa("Server")) {
	$self->_set_err("ServerUtil::new: \"server\" must be an object of type \"Server\"");
	return $self; 
    }

    # check for existance of required -loghdl argument
    unless (grep (/^-loghdl$/, (keys %args))) {
	$self->_set_err("Error: ServerUtil::new(): Missing required argument '-server'");
	return $self; 
    }

    $self->{server} = $server;
    $self->{log} = $log;

    # determine system model type
    eval {
	@reply = ${$self->{server}}->node_show();
    };

    if ($@) {
	$self->_set_err("Error: ServerUtil::new(): Unable to determine system model: $@");
	return $self; 
    }

    $self->{platform} = $reply[0]->{model};

    # determine drive type
    eval {
	@reply = ${$self->{server}}->storage_disk_show();
        @reply = ${$self->{server}}->storage_disk_show(disk => $reply[0]->{disk});
    };

    if ($@) {
	$self->_set_err("Error: ServerUtil::new(): Unable to determine drive type: $@");
	return $self; 
    }

    $self->{drive_type} = $reply[0]->{type};
    $self->{drive_size} = $reply[0]->{size};

    bless $self, $class;

    return $self;
} 

###############################################################################
# Method:    create_aggregate
# Objective: Create a single or group of aggregates on the filer
#
#               Required(3 Options):
#               1)	-aggr_list: Array of aggregate disk counts 
#	        2)	-maxnum: Create the max number of aggregates based on existing
#				 number of available disks. 
#			-node: The node name which to create the aggregates.
#	        3)	-num: Number of aggregates to create
#			-diskcount: Number of disks to assign to the aggregate.
#
#	        Optional:
#			-prefix: Prefix name of the volume(s). Default is "wafl_vol"
#			-raidtype: Aggregate raid type. Valid values are "raid_dp" or
#				   "raid4". Default is "raid_dp". 
#			-maxsize: Create a 16TB aggregate. Not supported with maxnum or 
#				  maxconfig
#			-maxconfig: Create the max number of 16TB aggregates. Not supported 
#				    with maxnum.
#	        			Platform           Drive Type         Max # 16Tb Aggregates
#					-------------      ----------------   ----------------------
#					Excelsior          FC                 18
#					Excelsior          ATA                31
#					Deux               FC                 6
#					Deux               ATA                10   				
#
#
###############################################################################
sub create_aggregate {
    my ($self, %args) = @_;
    my ($aggr_list,
	$aggr_name,
	$diskcount,
	$found,
	$i,
	$raidtype,	
	$loopcount,
	$maxconfig,
	$maxsize,
	$maxaggrsize,
	$maxnum,
	$node,
	$maxaggrs,
	$num,
	$freedisks,
	$currentaggrs,
	$possibleaggrs,
	$prefix,
	$reply);

    $self->_clear_err();

    foreach (keys %args) { 	
    	
	if (/-?prefix$/) {
	    length($prefix = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_aggregate: invalid value for \"prefix\": $args{$_}");
		return $self;
	    };
	}
	elsif (/^-?num$/) {
	    length($num = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_aggregate: invalid value for \"num\": $args{$_}");
		return $self;
	    };
	}
	elsif (/^-?diskcount$/) {
	    length($diskcount = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_aggregate: invalid value for \"diskcount\": $args{$_}");
		return $self;
	    };
	}
	elsif (/-?raidtype$/) {
	    length($raidtype = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_aggregate: invalid value for \"raidtype\": $args{$_}");
		return $self;
	    };
	}
	elsif (/^-?node$/) {            
	    length($node = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_aggregate: invalid value for \"node\": $args{$_}");
		return $self;
	    };            
	}
	elsif (/^-?maxsize$/) {
	    # create the largest supported aggregate
	    $maxsize = $ClientUtil::TRUE;	    
	}
	elsif (/^-?maxconfig$/) {
	    # maxconfig depends on the system configuration
	    $maxconfig = $ClientUtil::TRUE;	    
	}
	elsif (/^-?maxnum$/) {
	    # maxnum depends on the system configuration
	    $maxnum = $ClientUtil::TRUE;
	}
	elsif (/^-?aggr_list$/) {
	    # the user can specify an array of sizes defining the aggregates to create	   
	    scalar($aggr_list = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_aggregate: invalid value for \"aggr_list\": $args{$_}");
		return $self;
	    };
	}
	else {
	    $self->_set_err("ServerUtil::create_aggregate: Invalid parameter specified: $_");
	    return $self;
	}
    }

    # set defaults
    $prefix = "new_aggr" unless defined($prefix);   
    $maxaggrs = 100;      				#Max number of aggregates per node.
    #$maxaggrsize = 16*1024*1024;			#Max size of an aggregate is 16TB.
    $maxaggrsize = 200*1024;
    $loopcount = 5;
    $raidtype = "raid_dp" unless defined($raidtype);
    
    # Determine number of required disks to use for the initial aggregate create    
    if ($maxsize eq $ClientUtil::TRUE) {    	
    	$num = 1;
    	if($raidtype eq "raid_dp"){
            $diskcount = 3;
    	}
    	else{
            $diskcount = 2;
    	}   	
    }   	
    # Determine number of required disks for the max configuration based on platform,
    # and drive type.	
    if ($maxconfig eq $ClientUtil::TRUE) {	
    	
	$aggr_list = undef;
	if ($self->{platform} =~ /3050/) {		
	    if ($self->{drive_type} =~ /FC/) {
		for (1 .. 6) {
		    # 16 TB max assumed
		    push @$aggr_list, int((16*1024*1024)/$self->{drive_size});		    
		} 
	    } elsif ($self->{drive_type} =~ /ATA/) {
		for (1 .. 10) {
		    # 16 TB max assumed
		    push @$aggr_list, int((16*1024*1024)/$self->{drive_size});		    
		} 
	    } else {
		$self->_set_err("ServerUtil::create_aggregate: Unsupported drive type: $self->{drive_type}");
		return $self;
	    }
	} elsif ($self->{platform} =~ /6070/) {		
		push @$aggr_list, int((200*1024)/$self->{drive_size});
	    if ($self->{drive_type} =~ /FC/) {
		for (1 .. 18) {
		    # 16 TB max assumed
		    push @$aggr_list, int((16*1024*1024)/$self->{drive_size});
		} 
	    } elsif ($self->{drive_type} =~ /ATA/) {
		for (1 .. 31) {
		    # 16 TB max assumed
		    push @$aggr_list, int((16*1024*1024)/$self->{drive_size});
		} 
	    } else {
		$self->_set_err("ServerUtil::create_aggregate: Unsupported drive type: $self->{drive_type}");
		return $self;
	    }
	} else {
	    $self->_set_err("ServerUtil::create_aggregate: Unsupported system platform: $self->{platform}");
	    return $self;
	}
    }

	# set the number of aggregates to create based on the number of existing aggregates and spare disks.	
	if ($maxnum eq $ClientUtil::TRUE){
		$diskcount = 3;
                # aggregate name is mandatory
                unless (defined($node)) {
                    $self->_set_err('ServerUtil::create_aggregatee: -node must be defined when using -maxnum');
                    return $self;
                }
                
		$reply = ${$self->{server}}->storage_disk_show(state=>'spare', owner=>$node); 
		$freedisks = scalar(@$reply);
		$reply = ${$self->{server}}->storage_aggregate_show(nodes=>$node); 
		$currentaggrs = scalar(@$reply);
				
		if ($raidtype eq "raid4"){
			$diskcount = 2;
		}
					
		$possibleaggrs = $freedisks/$diskcount;
		
		$num = $maxaggrs - $currentaggrs;
			
		if($num > $possibleaggrs){
			$num = int($possibleaggrs);
		}			    							
	}	

    # the user must pass in either the array of aggregates to create or the number+diskcount
    if (defined($aggr_list)) {    	
	for ($i=0; $i<scalar(@$aggr_list); $i++) {		
	    $aggr_name = $prefix . "_$i";
	    
	    eval {
		${$self->{server}}->storage_aggregate_new(aggregate=>$aggr_name,
		diskcount=>$$aggr_list[$i],
		raidtype=>$raidtype);
	    };

	    while ($loopcount--) {
		eval {
		  $reply = ${$self->{server}}->storage_aggregate_show(aggregate=>$aggr_name);  
		};
		if ($@) {
                    last;
		}
		
		if ($reply->[0]->{state} =~ /creating/i) {
                    $self->_log("ServerUtil::create_aggregate: Verifying aggregate creation");
                    sleep 5;
                    next;
		} else {
                    $found = 1;
                    last;
		}
	    }

	    unless ($found) {
			my @buf = $@;

			# remove all aggregates created
			$self->delete_aggregate(
					-prefix => $prefix
			);

			$self->_set_err("ServerUtil::create_aggregate: Error creating aggregate: @buf");
			last;
	    }
    } 
    } elsif (defined($num) || ($maxnum eq $ClientUtil::TRUE) || ($maxsize eq $ClientUtil::TRUE)) {
        unless ((defined($diskcount)) || ($maxnum eq $ClientUtil::TRUE)) {
	    $self->_set_err('ServerUtil::create_aggregate: -diskcount must be defined with -num');
	    return $self;
        }                
		for ($i=0; $i<$num; $i++) {
		    $aggr_name = $prefix . "_$i";
		    $self->_log("ServerUtil::create_aggregate: Creating aggregate $aggr_name");
                    if (defined($node)){
                        eval {                            
                            ${$self->{server}}->storage_aggregate_new(aggregate=>$aggr_name,
                            raidtype=>$raidtype,nodes=>$node,diskcount=>$diskcount);
                        };
                    }else{
                        eval {                            
                            ${$self->{server}}->storage_aggregate_new(aggregate=>$aggr_name,
                            raidtype=>$raidtype,
                            diskcount=>$diskcount);
                        };
                    }
                    
		    while ($loopcount--) {
				eval {
				    $reply = ${$self->{server}}->storage_aggregate_show(aggregate=>$aggr_name);  
				};
				if ($@) {
				    last;
				}
			
				if ($reply->[0]->{state} =~ /creating/i) {
				    $self->_log("ServerUtil::create_aggregate: Verifying aggregate creation");
				    sleep 5;
				    next;
				} else {
				    $found = 1;
				    last;
				}
		    }
                    
		    unless ($found) { 
			# remove all aggregates created
			$self->delete_aggregate(
						-prefix => $prefix
						);

			$self->_set_err("ServerUtil::create_aggregate: Error creating aggregate: $@");
			last;
                        
	    	}
        }
        if ($maxsize eq $ClientUtil::TRUE){        	
        	while($ClientUtil::TRUE){ 
                    # Add more disks to the aggregate until either there are no more disks to add or the aggregate
                    # meets the max allowed size.
                    eval {
                        ${$self->{server}}->storage_aggregate_add(aggregate=>$aggr_name, diskcount=>1);  				    
                    };
                    if ($@) {
                        last;
                    }else{      
			while ($loopcount--) {
                            eval {
                                $reply = ${$self->{server}}->storage_aggregate_show(aggregate=>$aggr_name);  
                            };
                            if ($@) {
                                last;
                            }
				
                            # Check to see if the disk add has completed
                            if($reply->[0]->{diskcount} eq $diskcount+1){
				$diskcount +=1;
				last;
                            }
                            else{
				sleep 5;
				next;
                            }			
		    	}
                    }		 		
        	}
        	
        	# Compare aggregate size to max aggregate size(16TB)
        	$reply = ${$self->{server}}->storage_aggregate_show(aggregate=>$aggr_name);   
        	if(($reply->[0]->{size}) < $maxaggrsize){ 
                    $self->_set_err("ServerUtil::create_aggregate: Not enough disks to create a $maxaggrsize byte aggregate.\n
Aggregate Size: $reply->[0]->{size} bytes.");
                    return $self;
        	}        	  
        }
         
    } else {
	$self->_set_err('ServerUtil::create_aggregate: -aggr_list or -num must be defined');
	return $self;
    }   
    
    return $self;
} 

###############################################################################
# Method:    delete_aggregate
# Objective: Remove a single or group of aggregates on the filer
# Details:   See POD documentation at the end of this file
###############################################################################
sub delete_aggregate {
    my ($self, %args) = @_;
    my ($aggr_name,
	$name,
	$prefix);

    $self->_clear_err();

    foreach (keys %args) {
	if (/-?name$/) {
	    length($name = $args{$_}) or do {
		$self->_set_err("ServerUtil::delete_aggregate: invalid value for \"name\": $args{$_}");
		return $self;
	    };
	}	
	elsif (/-?prefix$/) {
	    length($prefix = $args{$_}) or do {
		$self->_set_err("ServerUtil::delete_aggregate: invalid value for \"prefix\": $args{$_}");
		return $self;
	    };
	}
	else {
	    $self->_set_err("ServerUtil::delete_aggregate: Invalid parameter specified: $_");
	    return $self;
	}
    }

    # check for existance of required -name argument
    unless (defined($prefix) or defined($name)) {
	$self->_set_err("Error: ServerUtil::delete_aggregate(): Argument '-name' or '-prefix' required");
	return $self; 
    }

    if (defined($prefix)) {
	$aggr_name = $prefix . '*';
    } else {
	$aggr_name = $name;
    }

    $self->_log("ServerUtil::delete_aggregate: Deleting aggregate $aggr_name");
    
    eval {
	${$self->{server}}->storage_aggregate_delete(aggregate => $aggr_name);
    };

    if ($@) {
	$self->_set_err("Error: ServerUtil::delete_aggregate(): Error deleting aggregate: $@");
	return $self; 
    }

    return $self;
}

###############################################################################
# Method:    volume_mirror_create
# Objective: Create an specified number of volume mirrors
#            
#
#               Required (3 Options):                                  
#	        1)	-num: Number of mirrors to create
#			-volume: Name of source volume
#                       -aggr_name: Name of the destination aggregate.			
#	        2)	-maxnum: Number of aggregates to create
#			-volume: Name of source volume
#                       -aggr_name: Name of the destination aggregate.
#               3)      -vol_list: Array of unique source volume names. 
#
#	        Optional:
#			-prefix: Prefix name of the mirror(s).
#                                Default is "<volume>_mirror"                       
#                       -parallel: Create mirrors in parallel (Not tested)
#                       -replicate: Replicate the mirror(s)
#                       -type: Mirror type. DR or LS. Default is DR.
#
#		vol_list array fields. Each are required fields except 'type':
#				vol_list[0]=>'volume'
#				vol_list[0]=>'aggr_name'
#                               vol_list[0]=>'type'
# 
###############################################################################
sub volume_mirror_create {
    my ($self, %args) = @_;
    my ($aggr_name,
        $vserver,
	@buf,
	@cmd,
        @cmd2,
	$mirror,
	$m_hash,
	$timeout,
	@v_hash,	
        $i,
        $num,
        $loopcnt,
        $maxmirrors,
        $replicate,
        $maxnum,
        $parallel,
        $prefix,
        $type,
	$volume,
        $vol_list);

    $self->_clear_err();

    # timeout = 5 min (30*10sec)
    $timeout = 30;

    foreach (keys %args) {
	if (/-?vol_list$/) {
	    scalar($vol_list = $args{$_}) or do {
		$self->_set_err("ServerUtil::volume_mirror_create: invalid value for \"vol_list\": $args{$_}");
		return $self;
	    };
	}
        elsif (/^-?num$/) {
	    length($num = $args{$_}) or do {
		$self->_set_err("ServerUtil::volume_mirror_create: invalid value for \"num\": $args{$_}");
		return $self;
	    };
	}
        elsif (/^-?volume$/) {
	    length($volume = $args{$_}) or do {
		$self->_set_err("ServerUtil::volume_mirror_create: invalid value for \"volume\": $args{$_}");
		return $self;
	    };
	}
        elsif (/^-?aggr_name$/) {
	    length($aggr_name = $args{$_}) or do {
		$self->_set_err("ServerUtil::volume_mirror_create: invalid value for \"aggr_name\": $args{$_}");
		return $self;
	    };
	}
        elsif (/^-?type$/) {
	    length($type = $args{$_}) or do {
		$self->_set_err("ServerUtil::volume_mirror_create: invalid value for \"type\": $args{$_}");
		return $self;
	    };
	}
        elsif (/^-?maxnum$/) {
	    # create the max number of mirrors
	    $maxnum = $ClientUtil::TRUE;	    
	}
        elsif (/^-?parallel$/) {
	    # create the mirros in parallel
	    $parallel = $ClientUtil::TRUE;	    
	}
        elsif (/^-?replicate$/) {
	    # replicate the created mirror(s)
	    $replicate = $ClientUtil::TRUE;	    
	}
	else {
	    $self->_set_err("ServerUtil::volume_mirror_create: Invalid parameter specified: $_");
	    return $self;
	}
    }

    # set defaults      
    $maxmirrors = 32;					#Max number of mirrors per volume.

    # iterate through volume array issuing volume mirror create commands
    if (defined($vol_list)){
        $loopcnt = scalar(@$vol_list);        
    } elsif (defined($num) || ($maxnum eq $ClientUtil::TRUE)){
        $loopcnt = 1;
    }else {
            $self->_set_err('ServerUtil::volume_mirror_create: -vol_list or -num must be defined');
            return $self;
    }
    
             
    for ($i=0; $i<$loopcnt; $i++) {            
        if (defined($vol_list)){
            # volume name is mandatory
            unless (defined($$vol_list[$i]{'volume'})) {
                $self->_set_err('ServerUtil::volume_mirror_create: -volume name must be defined for each mirror');
                return $self;
            }
        
            # aggregate name is mandatory
            unless (defined($$vol_list[$i]{'aggr_name'})) {
                $self->_set_err('ServerUtil::volume_mirror_create: -aggr_name must be defined for each mirror');
                return $self;
            }
                          
            $volume = $$vol_list[$i]{'volume'};
            $aggr_name = $$vol_list[$i]{'aggr_name'};
            $type = "DR" unless defined($$vol_list[$i]{'type'});
            $prefix = $volume unless defined($prefix); 
            $mirror = $prefix . "_mirror";  
            $num = 1;
        }elsif (defined($num) || ($maxnum eq $ClientUtil::TRUE)){
            # volume name is mandatory
            unless (defined($volume)) {
                $self->_set_err('ServerUtil::volume_mirror_create: -volume name must be defined for each mirror');
                return $self;
            }
        
            # aggregate name is mandatory
            unless (defined($aggr_name)) {
                $self->_set_err('ServerUtil::volume_mirror_create: -aggr_name must be defined for each mirror');
                return $self;
            }
                        
            # set defaults
            $type = "DR" unless defined($type);
            $prefix = $volume unless defined($prefix);
            
            if ($maxnum eq $ClientUtil::TRUE){
                if($type eq "DR"){
                    $num = $maxmirrors;
                }
                else{
                    $num = 1;
                }                
            }elsif(($num > $maxmirrors)||($num < 0)){
               $self->_set_err("Error: ServerUtil::volume_mirror_create(): Invalid value for num: $num");
               return $self; 
            }elsif($num != 1 && $type eq "LS"){
               $self->_set_err("Error: ServerUtil::volume_mirror_create(): The max LS mirrors per aggregate is 1");
               return $self;
            }            
        }else {
            $self->_set_err('ServerUtil::volume_mirror_create: -vol_list or -num must be defined');
            return $self;
        }            
        
    	# verify that the volume exists
        eval {
            @v_hash = ${$self->{server}}->volume_show(volume => $volume);
        };
	
        if ($@) {
            $self->_set_err("Error: ServerUtil::volume_mirror_create(): Error locating source volume: $volume");
            return $self; 
        }

        $vserver = $v_hash[0]->{vserver};

        $self->_log("ServerUtil::volume_mirror_create: Creating mirror $mirror");
            
        if ($parallel eq $ClientUtil::TRUE){
            push @cmd, "ngsh -c \'volume mirror create -vserver $vserver -source $volume -mirror $mirror -aggregate $aggr_name -type $type\' < /dev/null 2>&1 &";
            
            $m_hash = {
                VOL => $volume,
                MIRROR => $mirror,
                VSERVER => $vserver            
            };	
            push @MIRROR, $m_hash;
        }
        else{
            for ($i=0; $i<$num; $i++) {                
                if (!(defined($vol_list))) {
                    $mirror = $prefix . "_mirror_$i";
                }
                
                eval {
                    ${$self->{server}}->volume_mirror_new(vserver=>$vserver,
                                                          aggregate=>$aggr_name,
                                                          source=>$volume,
                                                          mirror=>$mirror,
                                                          type=>$type);                   
                };                               
                
                eval {
                   # wait on all jobs
                   while ($timeout--) {
                       @buf = ${$self->{server}}->job_show();
	    
                       if (grep(/Mirror\s+Create.+(Queued|Running)/i, @buf)) {
                           $self->_log("ServerUtil::volume_mirror_create: Mirror Create queued (10s sleep)");	
                           sleep 10;
                       } elsif (grep(/Mirror\s+Create.+Failure/i, @buf)) {
                           $self->_set_err("ServerUtil::volume_mirror_create(): Error creating mirror: @buf");
                           last;
                       } else {
                           last;
                        }
                    }
                };
    
                if ($@) {
                    $self->_set_err("Error: ServerUtil::volume_mirror_create(): Error creating mirror: $@");
	
                    # delete all vol mirrors potentially created
                    $self->volume_mirror_delete();
	
                    return $self; 
                }
                
                if($replicate eq $ClientUtil::TRUE){
                    eval {
                        ${$self->{server}}->volume_mirror_replicate(source=>$volume,
                                                                    vserver=>$vserver,
                                                                    foreground=>"false",
                                                                    mirror=>$mirror); 
                    };                
                
                    if ($@) {
                        $self->_set_err("Error: ServerUtil::volume_mirror_create(): Error replicating mirror($mirror): $@");
                        return $self; 
                    }
                }
                $m_hash = {
                    VOL => $volume,
                    MIRROR => $mirror,
                    VSERVER => $vserver            
                };	
                push @MIRROR, $m_hash;                
            }
        }       	
    }    

    if ($parallel eq $ClientUtil::TRUE) {
        $loopcnt=0;
        eval {
            # spawning these off in a tight loop will provide parallelism
            foreach (@cmd) {
                ${$self->{server}}->shell(cmd => $_);
            }
        };
        
        while($loopcnt<2){
            eval {
                # wait on all jobs
                while ($timeout--) {
                    @buf = ${$self->{server}}->shell(cmd => 'ngsh -c \'job show\'');
	    
                    if (grep(/Mirror\s+Create.+(Queued|Running)/i, @buf)) {
                        $self->_log("ServerUtil::volume_mirror_create: Mirror Create or Replicate queued (10s sleep)");	
                        sleep 10;
                    } elsif (grep(/Mirror\s+Create.+Failure/i, @buf)) {
                        $self->_set_err("ServerUtil::volume_mirror_create(): Error creating or replicating mirror: @buf");
                        last;
                    } else {
                        last;
                    }
                }
            };
    
            if ($@) {
                $self->_set_err("Error: ServerUtil::volume_mirror_create(): Error creating or replicating mirror: $@");
    	
                # delete all vol mirrors potentially created
                $self->volume_mirror_delete();
            
                return $self; 
            }
            if(($replicate eq $ClientUtil::TRUE) && $loopcnt eq 0){
                eval {
                   # spawning these off in a tight loop will provide parallelism
                   $self->volume_mirror_replicate();                
                };
                ++$loopcnt;
            } else{            
                last;
            }
        }
    }
    
    return $self;
}

###############################################################################
# Method:    volume_mirror_delete
# Objective: Delete volume mirror(s) created by volume_mirror_create
#
#	        Optional:
#                       -parallel: Deletes the mirror(s) in parallel (Not tested)
#
#               Assumptions:
#               The mirror(s) to be replicated have been created using
#               the volume_mirror_create function and are contained in the global
#               @MIRROR.
#
###############################################################################
sub volume_mirror_delete {
    my ($self, %args) = @_;
    my (@buf,
	@cmd,
        $parallel,
	$timeout);
    
    $self->_clear_err();

    # timeout = 5 min (30*10sec)
    $timeout = 30;

    foreach (keys %args) {
	if (/^-?parallel$/) {
	    # delete the mirror(s) in parallel
	    $parallel = $ClientUtil::TRUE;
	} 
	else {
	    $self->_set_err("ServerUtil::volume_mirror_delete: Invalid parameter specified: $_");
	    return $self;
	}
    }
    
    if ($parallel eq $ClientUtil::TRUE) {
        # remove all mirrors known in @MIRROR global    
        foreach (@MIRROR) {
            $self->_log("ServerUtil::volume_mirror_delete: Deleting mirror $_->{MIRROR}");
            push @cmd, "ngsh -c \'volume mirror delete -vserver $_->{VSERVER} -source $_->{VOL} -mirror $_->{MIRROR}\' < /dev/null 2>&1 &";
        }

        eval {
            foreach (@cmd) {
                # delete in parallel
                ${$self->{server}}->shell(cmd => $_);
            }
        };

        eval {
            # wait on all jobs
            while ($timeout--) {
                @buf = ${$self->{server}}->shell(cmd => 'ngsh -c \'job show\'');
	    
                # look for queued jobs in buffer
                if (grep(/Mirror\s+Delete.+(Queued|Running)/i, @buf)) {
                    $self->_log("ServerUtil::volume_mirror_delete: Mirror Delete queued (10s sleep)");		
                    sleep 10;
                } elsif (grep(/Mirror\s+Delete.+Failure/i, @buf)) {
                    $self->_set_err("Error: ServerUtil::volume_mirror_delete(): Error deleting mirror: @buf");
                    last;
                } else {
                    last;
                }
            }
        };
    
        if ($@) {
            $self->_set_err("Error: ServerUtil::volume_mirror_delete(): Error deleting mirror: $@");
            return $self; 
        }
    }else {
        foreach(@MIRROR){            
            eval {
                ${$self->{server}}->volume_mirror_delete(vserver=>$_->{VSERVER},                                                          
                                                         source=>$_->{VOL},
                                                         mirror=>$_->{MIRROR});                   
                };                               
                
            eval {
                # wait on all jobs
                while ($timeout--) {
                    @buf = ${$self->{server}}->job_show();
	    
                    if (grep(/Mirror\s+Delete.+(Queued|Running)/i, @buf)) {
                        $self->_log("ServerUtil::volume_mirror_delete: Mirror Delete queued (10s sleep)");	
                        sleep 10;
                    } elsif (grep(/Mirror\s+Delete.+Failure/i, @buf)) {
                        $self->_set_err("ServerUtil::volume_mirror_delete(): Error deleting mirror: @buf");
                        last;
                    } else {
                        last;
                    }
                }
            };
    
            if ($@) {
                $self->_set_err("Error: ServerUtil::volume_mirror_replicate(): Error deleting mirror: $@");
                return $self; 
            }
        }        
    }
    
    return $self;
}

###############################################################################
# Method:    volume_mirror_replicate
# Objective: Replicate an arbitrary number of volume mirrors
#
#	        Optional:
#                       -parallel: Replicate the mirrors in parallel (Not tested)
#
#               Assumptions:
#               The mirror(s) to be replicated have been created using
#               the volume_mirror_create function and are contained in the global
#               @MIRROR.
#
###############################################################################
sub volume_mirror_replicate {
    my ($self, %args) = @_;
    my ($agg,
	@buf,
	@cmd,
	$mirror,
        $parallel,
	$m_hash,
	$timeout,
	@v_hash,
	$volume);

    $self->_clear_err();

    # timeout = 5 min (30*10sec)
    $timeout = 30;
    
    foreach (keys %args) {
	if (/^-?parallel$/) {
	    # replicate the mirror(s) in parallel
	    $parallel = $ClientUtil::TRUE;
	} 
	else {
	    $self->_set_err("ServerUtil::volume_mirror_replicate: Invalid parameter specified: $_");
	    return $self;
	}
    }

    if ($parallel eq $ClientUtil::TRUE) {
       # iterate through volume array issuing volume mirror replicate commands
        foreach (@MIRROR) { 
            $self->_log("ServerUtil::volume_mirror_replicate: Replicating mirror $_->{MIRROR}");
            push @cmd, "ngsh -c \'volume mirror replicate -vserver $_->{VSERVER} -source $_->{VOL} -mirror $_->{MIRROR} -foreground false\' < /dev/null 2>&1 &";
        }
    
        eval {
            # spawning these off in a tight loop will provide parallelism
            foreach (@cmd) {
                ${$self->{server}}->shell(cmd => $_);
            }
        };
    
        eval {
            # wait on all jobs
            while ($timeout--) {
                @buf = ${$self->{server}}->shell(cmd => 'ngsh -c \'job show\'');
	    
        	if (grep(/Mirror\s+Repl.+(Queued|Running)/i, @buf)) {
                    $self->_log("ServerUtil::volume_mirror_replicate: Mirror Replicate queued (10s sleep)");
                    #$self->_log(@buf);
                    sleep 10;
                } elsif (grep(/Mirror\s+Repl.+Failure/i, @buf)) {
                    $self->_set_err("ServerUtil::volume_mirror_replicate(): Error replicating mirror: @buf");
                    last;
                } else {
                    last;
                }
            }
        };

        if ($@) {
            $self->_set_err("Error: ServerUtil::volume_mirror_replicate(): Error replicating mirror: $@");
        }
    } else {        
        foreach(@MIRROR){            
            eval {
                ${$self->{server}}->volume_mirror_replicate(vserver=>$_->{VSERVER},                                                          
                                                            source=>$_->{VOL},
                                                            foreground=>"false",
                                                            mirror=>$_->{MIRROR});                   
                };                               
            sleep 10;    
            eval {
                # wait on all jobs
                while ($timeout--) {
                    @buf = ${$self->{server}}->job_show();
	    
                    if (grep(/Mirror\s+Repl.+(Queued|Running)/i, @buf)) {
                        $self->_log("ServerUtil::volume_mirror_replicate: Mirror Replicate queued (10s sleep)");	
                        sleep 10;
                    } elsif (grep(/Mirror\s+Repl.+Failure/i, @buf)) {
                        $self->_set_err("ServerUtil::volume_mirror_replicate(): Error replicating mirror: @buf");
                        last;
                    } else {
                        last;
                    }
                }
            };
    
            if ($@) {
                $self->_set_err("Error: ServerUtil::volume_mirror_replicate(): Error replicating mirror: $@");
	
                # delete all vol mirrors potentially created
                $self->volume_mirror_delete();
	
                return $self; 
            }
        }
    }

    return $self;
}

###############################################################################
# Method:    volume_mirror_promote (Not Tested)
# Objective: Promote an arbitrary number of volume mirrors in parallel 
# Details:   See POD documentation at the end of this file
###############################################################################
sub volume_mirror_promote {
    my ($self, %args) = @_;
    my ($agg,
	@buf,
	@cmd,
	$mirror,
	$m_hash,
	$timeout,
	@v_hash,
	$volume);

    $self->_clear_err();

    # timeout = 5 min (30*10sec)
    $timeout = 30;

    # iterate through volume array issuing volume mirror promote commands
    foreach (@MIRROR) {
	$self->_log("ServerUtil::volume_mirror_promote: Replicating mirror $_->{MIRROR}");
	push @cmd, "ngsh -c \'volume mirror promote -vserver vs0 -source $_->{VOL} -mirror $_->{MIRROR}\' < /dev/null 2>&1 &";
    }
    
    eval {
	# spawning these off in a tight loop will provide parallelism
	foreach (@cmd) {
	    ${$self->{server}}->shell(cmd => $_);
	}
    };
    
    eval {
	# wait on all jobs
	while ($timeout--) {
	    @buf = ${$self->{server}}->shell(cmd => 'ngsh -c \'job show\'');
	    
	    if (grep /Mirror\s+Prom.+(Queued|Run)/i, @buf) {
		$self->_log("ServerUtil::volume_mirror_promote: Mirror Replicate queued (10s sleep)");
		#$self->_log(@buf);
		sleep 10;
	    } elsif (grep /Mirror\s+Prom.+Fail/i, @buf) {
		$self->_set_err("ServerUtil::volume_mirror_promote(): Error replicating mirror: @buf");
		last;
	    } else {
		last;
	    }
        }
    };

    if ($@) {
	$self->_set_err("Error: ServerUtil::volume_mirror_promote(): Error replicating mirror: $@");
    }

    return $self;
}

###############################################################################
# Method:    create_volume
# Objective: Create a single or group of volumes on the filer
#
#	        Required(2 Options):
#	        1)	-vserver: Vserver name.
#			-aggr_name: Aggregate name.
#	        2)	-vol_list: Array of hashes for volumes to be created. No other 
#				   parameters are used if a vol_list is set. See content 
#				   requirements as listed below. 
#
#	        Optional Parameters for Option #1:
#			-prefix: Prefix name of the volume(s). Default is "wafl_vol"
#			-num: Number of volumes to create. Default is 1.
#			-size: Size of the volume(s). Default is 20MB.
#			-junction_path: Junction path for mounting the volume(s).
#					Not supported if num > 1. Default is "/$vol_name".
#			-space_guarantee: Volume space guarantee format. Default is 
#					  "volume".
#			-maxnum: Create the max number of volumes. 
#			-maxsize: Create the largest possible volume based on the size of the
#                                 given aggregate. Not supported with maxnum or
#				  maxconfig.
#			-maxconfig: Create the max number of 16TB volumes. Not supported 
#				    with maxnum or maxsize.
#	        		    Platform           Drive Type         Max # 16Tb Volumes
#				    -------------      ----------------   -------------------
#				    Excelsior          FC                 18
#				    Excelsior          ATA                31
#				    Deux               FC                 6
#				    Deux               ATA                10   				
#
#
#			 vol_list array fields. vserver and aggr_name are required fields.:
#				vol_list[0]=>'vserver'
#				vol_list[0]=>'aggr_name'
#				vol_list[0]=>'prefix'
#				vol_list[0]=>'size'
#				vol_list[0]=>'space_guarantee'
#				vol_list[0]=>'junction_path'  Default is "/$vol_name".
#
###############################################################################
sub create_volume {
    my ($self, %args) = @_;
    my ($vol_list,
	$vol_name,
	$size,
        $size_value,
	$found,
	$i,
	$loopcount,	
	$aggr_count,
	$vol_count,
	$space_guarantee,
	$vserver,
	$aggr_name,
	$node,
	$junction_path,
	$maxsize,
        $maxconfig,
	$maxnum,
	$max_vvols,
	$prefix,
	$num,
	$reply);

    $self->_clear_err();
    foreach (keys %args) {
    # volume name	
	if (/-?prefix$/) {
	    length($prefix = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"prefix\": $args{$_}");
		return $self;
	    };
	}
	# number of volumes to create
	elsif (/^-?num$/) {
	    length($num = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"num\": $args{$_}");
		return $self;
	    };
	}
	# size for each volume
	elsif (/^-?size$/) {
	    length($size = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"size\": $args{$_}");
		return $self;
	    };
	}
	# space guarantee is none or volume. default is volume.
	elsif (/^-?space_guarantee$/) {
	    length($space_guarantee = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"space_guarantee\": $args{$_}");
		return $self;
	    };
	}
	# vserver name
	elsif (/^-?vserver$/) {
	    length($vserver = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"vserver\": $args{$_}");
		return $self;
	    };
	}
	# aggregate name
	elsif (/^-?aggr_name$/) {
	    length($aggr_name = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"aggr_name\": $args{$_}");
		return $self;
	    };
	}	
	# junction path to mount volume
	elsif (/^-?junction_path$/) {
	    length($junction_path = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"junction_path\": $args{$_}");
		return $self;
	    };
	}
	# generates largest volume based on the aggregate size
	elsif (/^-?maxsize$/) {
	    # max depends on the size of the aggregate
	    $maxsize = $ClientUtil::TRUE;
	}
        # generates max number of 16TB volumes
	elsif (/^-?maxconfig$/) {
	    # max depends on the system configuration
	    $maxconfig = $ClientUtil::TRUE;
	}
	# generates max number of volumes
	elsif (/^-?maxnum$/) {
	    # max depends on the system configuration
	    $maxnum = $ClientUtil::TRUE;
	}
	# array of volumes and their attributes
	elsif (/^-?vol_list$/) {
	    # the user can specify an array of sizes defining the volumes to create	   
	    scalar($vol_list = $args{$_}) or do {
		$self->_set_err("ServerUtil::create_volume: invalid value for \"vol_list\": $args{$_}");
		return $self;
	    };
	}
	else {
	    $self->_set_err("ServerUtil::create_volume: Invalid parameter specified: $_");
	    return $self;
	}
    }
    
    # set defaults  
    $prefix = "new_vol" unless defined($prefix);
    $num = 1 unless defined($num);
    $max_vvols = 5;  #500
    $loopcount = 5;
    
    # set up maxnum configuration. not supported if vol_list is defined. 
    if (!(defined(@$vol_list))){
    if ($maxnum eq $ClientUtil::TRUE) {    
    	$vol_count = 0;
    	
    	# aggregate name is mandatory
	unless (defined($aggr_name)) {
 	   $self->_set_err('ServerUtil::create_volume: -aggr_name must be defined for the volume');
	    return $self;
        }
        
        # find the node name by using the aggregate name
        eval{
	    	$reply = ${$self->{server}}->storage_aggregate_show(aggregate=>$aggr_name);
	    };
	    if ($@) {
			$self->_set_err("ServerUtil::create_volume: Cannot find aggregate: $aggr_name");
			return $self;
		}	
	    $node = $reply->[0]->{nodes};          	    	 
    	
    	$reply = ${$self->{server}}->storage_aggregate_show(nodes=>$node);    	
    	$aggr_count = scalar(@$reply);   	
    	
    	for( ;$i<$aggr_count;$i++){
    		$vol_count += $reply->[$i]->{volcount};    		
    	}            
    	
    	$num = $max_vvols - $vol_count;    	
    }        
    elsif ($maxsize eq $ClientUtil::TRUE){
        $reply = ${$self->{server}}->storage_aggregate_show(aggregate=>$aggr_name);
        
        $size = $reply->[0]->{size};        
        $size =~ s/(\d*.\d)/$1/;
        $size_value = $1;
        
        #Create a volume that uses 95% of the aggregate space
        $size = $size_value * .95;  
        if ($reply->[0]->{size} =~ /tb/i) {
            #Terabyte Size
            $size = $size ."TB";
        }elsif ($reply->[0]->{size} =~ /gb/i){
            #Gigabyte Size
            $size = $size ."GB";
        }else{
            #Megabyte Size            
            $size = $size ."MB";
        }        
    }
    # set up maxsize configuration. not supported if vol_list is defined.
    elsif ($maxconfig eq $ClientUtil::TRUE) {
	if ($self->{platform} =~ /3050/) {
	    if ($self->{drive_type} =~ /FC/) {
			$num = 6;
    		$size = "16TB"; 
	    } elsif ($self->{drive_type} =~ /ATA/) {
			$num = 10;
    		$size = "16TB";
	    } else {
		$self->_set_err("ServerUtil::create_volume: Unsupported drive type: $self->{drive_type}");
		return $self;
	    }
	} elsif ($self->{platform} =~ /6070/) {
	    if ($self->{drive_type} =~ /FC/) {
			$num = 18;
    		$size = "16TB"; 
	    } elsif ($self->{drive_type} =~ /ATA/) {
			$num = 31;
    		$size = "16TB"; 
	    } else {
		$self->_set_err("ServerUtil::create_volume: Unsupported drive type: $self->{drive_type}");
		return $self;
	    }
	} else {
	    $self->_set_err("ServerUtil::create_volume: Unsupported system platform: $self->{platform}");
	    return $self;
	}
    }    
    }
    
    # the user must pass in either the array of volumes to create or the volume sizes    
    if (defined(@$vol_list)){
        $self->_set_err('ServerUtil::create_volume: ANDREW incorrect');
	for ($i=0; $i<scalar(@$vol_list); $i++) {		
		# vserver name is mandatory
		unless (defined($$vol_list[$i]{'vserver'})) {
	    $self->_set_err('ServerUtil::create_volume: -vserver must be defined for each volume');
	    return $self;
        }
        
		# aggregate name is mandatory
		unless (defined($$vol_list[$i]{'aggr_name'})) {
	    $self->_set_err('ServerUtil::create_volume: -aggr_name must be defined for each volume');
	    return $self;
        }
        
        # set vserver and aggregate name
        $vserver = $$vol_list[$i]{'vserver'};
        $aggr_name = $$vol_list[$i]{'aggr_name'};
        
        # set prefix
        if (defined($$vol_list[$i]{'prefix'})){
        	$prefix = $$vol_list[$i]{'prefix'};
        }
        else{
        	$prefix = "new_vol";
        }
        		
        # set volume name		
	$vol_name = $prefix . "_$i";	    
	    
	# set size
        if (defined($$vol_list[$i]{'size'})){
        	$size = $$vol_list[$i]{'size'};
        }
        else{
        	$size = "20MB";
        }
        
        # set space_guarantee
        if (defined($$vol_list[$i]{'space_guarantee'})){
        	$space_guarantee = $$vol_list[$i]{'space_guarantee'};
        }
        else{
        	$space_guarantee = "volume";
        }
        
        # set junction_path
        if (defined($$vol_list[$i]{'junction_path'})){
        	$junction_path = $$vol_list[$i]{'junction_path'};
        }
        else{
        	$junction_path = "/$vol_name";
        }     
	    
	    eval {
		${$self->{server}}->volume_new(vserver=>$vserver,
						  		   aggregate=>$aggr_name,
	    							   volume=>$vol_name,
	    							   size=>$size,
	    							   "space-guarantee"=>$space_guarantee,
	    							   "junction-path"=>$junction_path);
	    };

	    while ($loopcount--) {
		eval {
		    $reply = ${$self->{server}}->volume_show(aggregate=>$aggr_name,
		    										 vserver=>$vserver,
		    										 volume=>$vol_name);  
		};
		if ($@) {
		    last;
		}
		
		if ($reply->[0]->{state} =~ /creating/i) {
		    $self->_log("ServerUtil::create_volume: Verifying volume creation");
		    sleep 5;
		    next;
		} else {
		    $found = 1;
		    last;
		}
	    }

	    unless ($found) {
		# remove all volumes created
		$self->delete_volume(
					 -name=>$vol_name,
					 -vserver=>$vserver,
					 -aggr_name=>$aggr_name
					 );

		$self->_set_err("ServerUtil::create_volume: Error creating volume: $@");
		last;
	    }
        } 
    } else {
        
        # vserver name is mandatory
	unless (defined($vserver)) {
	    $self->_set_err('ServerUtil::create_volume: -vserver must be defined for the volume');
	    return $self;
        }
        
	# aggregate name is mandatory
	unless (defined($aggr_name)) {
	    $self->_set_err('ServerUtil::create_volume: -aggr_name must be defined for the volume');
	    return $self;
        }
        $num = 5; #andrew remove
        $size = "20MB" unless defined($size);
    	$space_guarantee = "volume" unless defined($space_guarantee);
        $self->_set_err('ServerUtil::create_volume: ANDREW correct');
    	for ($i=0; $i<$num; $i++) {
            $self->_log("ServerUtil::create_volume: ANDREW: number: $num");
            $self->_log("ServerUtil::create_volume: ANDREW: i: $i");
	    $vol_name = $prefix . "_$i";
            $self->_log("ServerUtil::create_volume: ANDREW: vol_name: $vol_name");
            if($num eq 1){
                $junction_path = "/$vol_name" unless defined($junction_path);
            }
            else{
                $junction_path = "/$vol_name";
            }
		    
	    eval {
	    ${$self->{server}}->volume_new(vserver=>$vserver,
					   aggregate=>$aggr_name,
	    				   volume=>$vol_name,
	    				   size=>$size,
	    				   "space-guarantee"=>$space_guarantee,
	    				   "junction-path"=>$junction_path);	
		
	    };

	    while ($loopcount--) {
		eval {
		    $reply = ${$self->{server}}->volume_show(aggregate=>$aggr_name,
		    										 vserver=>$vserver,
		    										 volume=>$vol_name);  
		};
		if ($@) {						
		    last;
		}
		
		if ($reply->[0]->{state} =~ /creating/i) {
		    $self->_log("ServerUtil::create_volume: Verifying volume creation");
		    sleep 5;
		    next;
		} else {
		    $found = 1;
		    last;
		}
	    }

	#    unless ($found) {
	#	# remove all volumes created
	#	$self->delete_volume(   -prefix=>$vol_name,
	#				-vserver=>$vserver
	#			    );
	#
	#	$self->_set_err("ServerUtil::create_volume: Error creating volume: $@");
	#	last;
	#    }
        } 
    } 

	return $self;
}

###############################################################################
# Method:    delete_volume
# Objective: Remove a single or group of volumes on the filer
# Details:   See POD documentation at the end of this file
#
#			 Required:
#				-vserver: Vserver name.
#
#			 Only One Required:
#				-prefix: Prefix of the volume(s). "prefix*" volume(s) will be
#						 deleted.
#				-name: Name of the volume to delete. 
#
###############################################################################
sub delete_volume {
    my ($self, %args) = @_;
    my ($vol_name,
    	$vserver,    
        $name,
        $prefix);

    $self->_clear_err();

    foreach (keys %args) {    	
        if (/-?name$/) {
            length($name = $args{$_}) or do {
                $self->_set_err("ServerUtil::delete_volume: invalid value for \"name\": $args{$_}");
                return $self;
            };
        }
        elsif (/-?prefix$/) {
            length($prefix = $args{$_}) or do {
                $self->_set_err("ServerUtil::delete_volume: invalid value for \"prefix\": $args{$_}");
                return $self;
            };            
        }
        elsif (/-?vserver$/) {
            length($vserver = $args{$_}) or do {
                $self->_set_err("ServerUtil::delete_volume: invalid value for \"vserver\": $args{$_}");
                return $self;
            };            
        }        
        else {
            $self->_set_err("ServerUtil::delete_volume: Invalid parameter specified: $_");
            return $self;
        }
    }
	
    # check for existance of required arguments
    unless (defined($prefix) or defined($name)) {
        $self->_set_err("Error: ServerUtil::delete_volume(): Argument '-name' or '-prefix' required");
        return $self;
    }
    
    unless (defined($vserver)) {
        $self->_set_err("Error: ServerUtil::delete_volume(): Argument '-vserver' required");
        return $self;
    }    
	
    if (defined($prefix)) {
        $vol_name = $prefix . '*';
    } else {
        $vol_name = $name;
    }

    $self->_log("ServerUtil::delete_volume: Deleting volume $vol_name");

    eval {
        ${$self->{server}}->volume_delete(volume => $vol_name,
        								  vserver => $vserver);
    };

    if ($@) {
        $self->_set_err("Error: ServerUtil::delete_volume(): Error deleting volume: $@");
        return $self;
    }

    return $self;
}

###############################################################################
# Method:    volume_snapshot_create
# Objective: Create a single or group of snapshots on the filer
# Details:   See POD documentation at the end of this file
#
#               Required (2 Options):  
#		1)	-vol_name: Volume name.
#               2)      -vol_list: Array of hashs for volumes to be created. No other 
#				   parameters are used if a vol_list is set. See content 
#				   requirements as listed below. (Not Tested)
#
#	        Optional:
#			-prefix: Prefix name of the snapshot(s). Default is "new_vol_ss".
#			-num: Number of snapshots to create. Default is 1.
#			-datasize: Size of the dummy data to write onto the mounted volume
#				   before each snapshot creation. Not supported if -datafill
#				   is used.(Not Tested)
#			-datafill: Writes 50k of dummy data onto the mounted volume before 
#				   each snapshot creation. Not supported if -datasize is used.(Not Tested)
#			-loghdl: Log handle. Used for IOMgr. Required if datasize or 
#		                 datafill are used(Not Tested)
#			-mntobj: Mount Object. Used for datatest. Required if datasize
#				 or datafill are used.(Not Tested)
#			-maxnum: Create the max number of snapshots.
#
#			 Volume array should contain the following fields:
#				vol_list[0]=>'vserver'
#				vol_list[0]=>'vol_name'
#				vol_list[0]=>'prefix'
#				vol_list[0]=>'num'
#				vol_list[0]=>'datasize'
#				vol_list[0]=>'datafill'
#				vol_list[0]=>'loghdl'
#				vol_list[0]=>'mntobj'
#				vol_list[0]=>'maxnum'
#
###############################################################################
sub volume_snapshot_create {
    my ($self, %args) = @_;
    my (@vol_list,
    	$vol_name,
    	$ss_name,
    	$ss_current_count,
    	$adj_max_vvol_ss,
    	$max_vvol_ss,
    	$vserver,
    	$datafill,
    	$datasize,
    	$maxnum,
    	$num,
        @buf,
        $timeout,
        $a,
        $new_a,
    	$loopcount,
    	$outerloop,
    	$mntobj,   
    	$ioObj,
    	$found,   
    	$iomgr, 
    	$loghdl,
    	$reply,
        $prefix);

    $self->_clear_err();
    
    foreach (keys %args) {        
        if (/-?prefix$/) {
            length($prefix = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_create: invalid value for \"prefix\": $args{$_}");
                return $self;
            };
        }
        elsif (/-?maxnum$/) {            
            $maxnum = $ClientUtil::TRUE;                        
    	}
        elsif (/-?vol_name$/) {
            length($vol_name = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_create: invalid value for \"vol_name\": $args{$_}");
                return $self;
            };
        }
        elsif (/-?vserver$/) {
            length($vserver = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_create: invalid value for \"vserver\": $args{$_}");
                return $self;
            };
        }
        elsif (/-?datasize$/) {
            length($datasize = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_create: invalid value for \"datasize\": $args{$_}");
                return $self;
            };            
        }
        elsif (/-?datafill$/) {
            $datafill = $ClientUtil::TRUE;            
        }
        elsif (/-?num$/) {
            length($num = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_create: invalid value for \"num\": $args{$_}");
                return $self;
            };
        }  
        elsif (/-?loghdl$/) {
            length($loghdl = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_create: invalid value for \"loghdl\": $args{$_}");
                return $self;
            };
        }    
        elsif (/-?mntobj$/) {
            length($mntobj = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_create: invalid value for \"mntobj\": $args{$_}");
                return $self;
            };
        }       	        
        else {
            $self->_set_err("ServerUtil::volume_snapshot_create: Invalid parameter specified: $_");
            return $self;
        }
    }
    
    # set defaults          
    $max_vvol_ss = 255;    
    $loopcount = 5;
    
    # timeout = 5 min (30*10sec)
    $timeout = 30;    
    
    #adjust the code below so that it is not duplicated for the non-vol_list path    
    # the user must pass in either an array of volumes or the volume name
    if (scalar(@vol_list)) {
    	$outerloop = scalar(@vol_list);
    }else {
    	$outerloop = 1;
    }
    
    for (my $i=0; $i<$outerloop; $i++) {
		
	# Vol_list variable settings
	if (scalar(@vol_list)) {
            # vserver name is mandatory
            unless (defined(($vol_list[$i]{'vserver'}))) {
                $self->_set_err('ServerUtil::volume_snapshot_create: -vserver must be defined for each volume');
                return $self;
            }
        
	    # volume name is mandatory
            unless (defined(($vol_list[$i]{'vol_name'}))) {
                $self->_set_err('ServerUtil::volume_snapshot_create: -vol_name must be defined for each volume');
                return $self;
            }       
		
            $vol_name = ($vol_list[$i]{'vol_name'});	     		       
            $vserver = $vol_list[$i]{'vserver'};
            $prefix = "wafl_vol_ss" unless defined ($vol_list[$i]{'prefix'});
            $num = 1 unless defined($vol_list[$i]{'num'});        
        
            if(defined ($vol_list[$i]{'datasize'})){
        	$datafill = $ClientUtil::TRUE;
      		$datasize="($vol_list[$i]{'datasize'})";
            }elsif (defined ($vol_list[$i]{'datafill'})){
      		$datafill = $ClientUtil::TRUE;
      		$datasize="50k";
            }
        
            if($datafill eq $ClientUtil::TRUE){
        	if(defined(($vol_list[$i]{'mntobj'}))){
                    $mntobj = $vol_list[$i]{'mntobj'};
        	}else{
                    $self->_set_err('ServerUtil::volume_snapshot_create: -mntobj must be defined when using the datafill or datasize options');
		    return $self;
        	} 
        	if(defined(($vol_list[$i]{'loghdl'}))){
                    $loghdl = $vol_list[$i]{'loghdl'};
        	}else{
                    $self->_set_err('ServerUtil::volume_snapshot_create: -loghdl must be defined when using the datafill or datasize options');
		    return $self;
        	}  
        	
        	# setup IOMgr for creating data
        	$iomgr = new IOMgr(-loghdl{$loghdl});
  		if ($iomgr->error) {
                    $self->_set_err("ServerUtil::volume_snapshot_create: Error creating IOMgr: $iomgr->errmsg()");
		    return $self;
    		}     	
            }        
	    
	    # set up maxnum configuration. not supported if vol_list is defined.
	    if (defined(($vol_list[$i]{'max_num'}))) {    	
   	 	$reply = ${$self->{server}}->volume_snapshot_show(volume=>$vol_name,
   		 													  vserver=>$vserver); 
   	 	$num = scalar(@$reply);
    		$num = $max_vvol_ss - $num;  
    		$adj_max_vvol_ss = $num; 
    		$maxnum = $ClientUtil::TRUE; 	
            }else{
  		$num = 1 unless defined($vol_list[$i]{'num'}); 
  		$maxnum = $ClientUtil::FALSE;
            }
        }		
	# Non-vol_list variable settings
	else{        
            # volume name is mandatory
            unless (defined($vol_name)) {
                $self->_set_err('ServerUtil::volume_snapshot_create: -vol_name must be defined for each volume');
                return $self;
            }          
        
            # set defaults
            $num = 1 unless defined($num);    
            $prefix = "new_vol_ss" unless defined($prefix);
       	    
            eval{
	    	$reply = ${$self->{server}}->volume_show(volume=>$vol_name);
	    };
	    if ($@) {
		$self->_set_err("ServerUtil::volume_snapshot_create: Cannot find volume: $vol_name");
		return $self;
            }	
	    $vserver = $reply->[0]->{vserver}; 
                        
	    # set up maxnum configuration. not supported if vol_list is defined.
	    if ($maxnum eq $ClientUtil::TRUE){            
                eval{
                    $reply = ${$self->{server}}->volume_snapshot_show(volume=>$vol_name,
                                                                      vserver=>$vserver);
                };
                if ($@) {
                    $num = $max_vvol_ss;  
                    $adj_max_vvol_ss = $max_vvol_ss;                    
                }else {
                    $num = scalar(@$reply);                    
                    $num = $max_vvol_ss - $num;  
                    $adj_max_vvol_ss = $num;                    
                    
                }
            }
  		
            if(defined ($datasize)){
  		$datafill = $ClientUtil::TRUE;
            }elsif ($datafill eq $ClientUtil::TRUE){
  		$datasize="50k";
            }
  		
            if($datafill eq $ClientUtil::TRUE){
        	unless (defined($mntobj)){        		
                    $self->_set_err("ServerUtil::volume_snapshot_create: -mntobj must be defined when using the datafill or datasize options");
		    return $self;
        	}   
        	unless (defined($loghdl)){        		
                    $self->_set_err("ServerUtil::volume_snapshot_create: -loghdl must be defined when using the datafill or datasize options");
		    return $self;
        	}     	
            }   		
	}
  		
  	# Create the correct number of snapshots for each volume in the array
	for($a=0;$a<$num;$a++){
            $ss_name = $prefix . "_$a"; 	    
	    
	   eval {
		${$self->{server}}->volume_snapshot_new(vserver=>$vserver,
                  			  		snapshot=>$ss_name,
	    						volume=>$vol_name);
	    };
            if ($@) {
                $self->_set_err("ServerUtil::volume_snapshot_create(): Error creating snapshot: @buf");
                last;
	    }
	
            while ($timeout--) {
                @buf = ${$self->{server}}->job_show();
	    
                if (grep(/Snap\s+Create.+(Queued|Running)/i, @buf)) {
                    $self->_log("ServerUtil::volume_snapshot_create: Snapshot Create queued (10s sleep)");	
                    sleep 10;
                } elsif (grep(/Snap\s+Create.+Failure/i, @buf)) {
                    $self->_set_err("ServerUtil::volume_snapshot_create(): Error creating snapshot: @buf");
                    last;
                } else {
                    if ($datafill eq $ClientUtil::TRUE){					
                        # Add data into the mounted directory
       			$ioObj = $iomgr->datatest(-mntobj=>$mntobj,
                        		          -limit=>$datasize,
                                		  -dispose=>"false");
		       	if ($ioObj->error) {
                           $self->_set_err("ServerUtil::volume_snapshot_create: Error adding data to mounted directory: $ioObj->errmsg()");           			
       			}
       
       			# Wait until IO is finished
       			while ($ioObj->is_alive) {
                            sleep 5;
			}		
		    }
                    last;
                }
            }			    
		    
	    # Check if Max_Snapshots needs to be adjusted due to a scheduled snapshot being executed.
	    if ($maxnum eq $ClientUtil::TRUE){		    		    	
                $reply = ${$self->{server}}->volume_snapshot_show(vserver=>$vserver,
					 			  volume=>$vol_name);  								
	    	$ss_current_count = scalar(@$reply);
                $new_a = $a + 1;        #Make $a value 1-based 
		   	
	    	if(($adj_max_vvol_ss-$new_a) != ($max_vvol_ss - $ss_current_count)) {				
                    $num = $max_vvol_ss - ($ss_current_count - $new_a);
	    	}
            }
	}	    
    }       
    return $self;
}

###############################################################################
# Method:    volume_snapshot_delete
# Objective: Remove a single or group of snapshots on the filer
# Details:   See POD documentation at the end of this file
#
#			 Required:
#				-vserver: Vserver name.
#				-vol_name: Volume name.
#
#			 Only One Required:
#				-prefix: Prefix of the snapshot(s). "prefix*" snapshot(s) will be
#						 deleted.
#				-name: Name of the snapshot to delete. 
#
###############################################################################
sub volume_snapshot_delete {
    my ($self, %args) = @_;
    my ($vol_name,
    	$vserver, 
    	$ss_name,   	
        $name,
        $prefix);

    $self->_clear_err();

    foreach (keys %args) {
        if (/-?name$/) {
            length($name = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_delete: invalid value for \"name\": $args{$_}");
                return $self;
            };
        }
        elsif (/-?prefix$/) {
            length($prefix = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_delete: invalid value for \"prefix\": $args{$_}");
                return $self;
            };
        }
        elsif (/-?vol_name$/) {
            length($prefix = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_delete: invalid value for \"vol_name\": $args{$_}");
                return $self;
            };
        }
        elsif (/-?vserver$/) {
            length($vserver = $args{$_}) or do {
                $self->_set_err("ServerUtil::volume_snapshot_delete: invalid value for \"vserver\": $args{$_}");
                return $self;
            };
        }        
        else {
            $self->_set_err("ServerUtil::volume_snapshot_delete: Invalid parameter specified: $_");
            return $self;
        }
    }

    # check for existance of required arguments
    unless (defined($prefix) or defined($name)) {
        $self->_set_err("Error: ServerUtil::volume_snapshot_delete(): Argument '-name' or '-prefix' required");
        return $self;
    }
    
    unless (defined($vserver)) {
        $self->_set_err("Error: ServerUtil::volume_snapshot_delete(): Argument '-vserver' required");
        return $self;
    }
    
    unless (defined($vol_name)) {
        $self->_set_err("Error: ServerUtil::volume_snapshot_delete(): Argument '-vol_name' required");
        return $self;
    }
	
    if (defined($prefix)) {
        $ss_name = $prefix . '*';
    } else {
        $ss_name = $name;
    }

    $self->_log("ServerUtil::volume_snapshot_delete: Deleting snapshot $ss_name");

    eval {
        ${$self->{server}}->snapshot_delete(volume => $vol_name,
        									snapshot => $ss_name,	
        								  	vserver => $vserver);
    };

    if ($@) {
        $self->_set_err("Error: ServerUtil::volume_snapshot_delete(): Error deleting snapshot($ss_name): $@");
        return $self;
    }

    return $self;
}

###############################################################################
# Method:    _log
# Objective: Log test output to stdout and file
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _log{
    my ($self,@line) = @_;
    my (
        $bFirst,
        $dash,
	);
    
    $bFirst=1;
    
    if (defined $self->{log}){

        foreach my $line (@line){
            $dash = ($bFirst == 1) ? "- " : "  ";
            $bFirst=0;

            print {${$self->{log}}} "   ".$dash.$line."\n";
        }

        Tharn::tstlog(-comment,@line);
    }
}

###############################################################################
# Method:    get_vol_uuid
# Objective: Return the UUID of the volume specified.
# Details:   UUID = See POD documentation at the end of this file.
###############################################################################

## @usage ::get_vol_uuid::
##              This routine obtains the UUID of the named volume. 
##              The method for obtaining this information is release 
##		dependent. For Tricky it is extracted from the value 
##		returned by the dbladecli df command. For Boilermaker
##		you can use the value returned by volume show -instance.
##   
##   $vol_uuid = get_vol_uuid($server, $volume);
##   Returns -1 if Volume is not found or if an error occurs
##
sub get_vol_uuid {

    my ($Server, $Volume) = @_;
    my ($fail_cnt,
        $uuid,
        $zapi_uuid,
        $output,
        @datarray,
        @matches,
        $match,
        @items,
        $item);

    $fail_cnt = 0;
    $zapi_uuid = -1;

    $output = $Server->get_release(format => "cooked");

    if ($output =~ "Tricky") {	

        eval {
    	        $output = $Server->dblade(cmd => "df -V -s");
    	        # returns /vol/UUID_b5756e0d_2eb7_11db_abe8_000423c205d4_vol18/         72          0           0%
        };

	    if ($@) {
       		$fail_cnt++;
    	} else {
		    # Look thru the array of returned values and see if any match 
		    # what you are looking for (UUID_..._$Volume)
     	    @datarray = split(" ",$output);
		    @matches = grep /$Volume/, (@datarray);
		    if (scalar(@matches) > 0) {
		        for ($match=0; $match < scalar(@matches); $match++) {  
		    	    @items=split(" ",@matches[$match]);
		      	    for ($item=0; $item < scalar(@items); $item++) {  
		                if (( @items[$item] =~ "UUID" ) && ( @items[$item] =~ $Volume )){
				            # extract field 2, which is just the uuid string
            			    $uuid = (split("/",@items[$item]))[2];
		                    # This gives you the wafl scan UUID not the ZAPI UUID
				            $zapi_uuid = cut ("_",2,6,$uuid); # cut out the UUID field
		                    $zapi_uuid =~ tr/_/-/; # and convert to the correct format
            			
		                }
		            }
		        }
     	    } else {
       	        $fail_cnt++;
       	  	}
     	}
    } else {
        # Assume that volume show will return the uuid for releases other than
        # tricky
        eval {
            @datarray = $Server->volume_show(volume=>$Volume);
            # returns {uuid} = b5756e0d-2eb7-11db-abe8-000423c205d4
        };
	    if ($@) {
       	    $fail_cnt++;
    	} else {
            if ( exists $datarray[0]{uuid} ) {
	            $zapi_uuid = $datarray[0]{uuid};
            } else {
       	        $fail_cnt++;
            }
    	}
    }    
    return ($fail_cnt == 0) ? $zapi_uuid : -1;
}

sub cut {

	# There are more elegant ways to do a cut
	# but this is simple, and it works.
	
	my $f1=shift;    # separator
	my $f2=shift;    # first field
	my $f3=shift;    # last field
	my $input=shift; # input string
	my $output="";   # output string
	my @items;
	
	@items = split($f1,$input);
	$f2--; $f3--; # adjust array indexes

	for ( ; $f2 <= $f3; $f2++) {
	    $output = $output.@items[$f2];
	    if ($f2 < $f3) { $output = $output.$f1; } 
	}
	return $output;
}


1;


