#Hey emacs, this is basically -*- perl -*-
#$Id: //depot/prod/test/Rsierranevada/lib/SfoEvents.pm#1 $
## @summary SfoEvents Interface
##
## @author knopp@netapp.com
## @status private
## @pod here
package SfoEvents;

use strict;
use Carp qw(croak);
use Error;
use Params::Validate qw(validate SCALAR BOOLEAN OBJECT ARRAYREF);
use Module::Common qw(validate_mixed);
use Tharn qw($Log host);

use vars qw(@ISA $Server);
@ISA = qw(Error);

###############################################################################
# Method:    new
# Objective: create new sfo helper object 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub new {
    my ($class,%args) = @_;
    my (
      $code,  
      $client,
      $errmsg,
      $self,
      $tharnhost,
      $uid,   
    );
    
    $self = bless {}, $class;
    
    ### Clear error buffer
    $self->_clear_err();
    
    ### Determine err handle upfront in case validation method
    ### fails before error handler is selected
    $self->{on_err}=(defined $args{on_err} and $args{on_err}=~/die|rtn/)
        ? $args{on_err}
        : "die";
    
    ### Validate input arguments
    eval {
        ($class,%args) = validate_mixed (
            @_,
            [ { type => SCALAR} ],
            {
                node     => {type => SCALAR, optional=>1 },
                on_err  =>  {type => SCALAR, optional=>1, default=>"die",
                             callbacks=>{
                                'Legal Values'=>sub{shift()=~/die|rtn/i
                            }}},  
                server  =>  {type => OBJECT,  required => 1 ,isa=>'Server', ,
                             callbacks=>{
                                'Type Of Filer is GX'=>sub{
                                   Tharn::host(($args{server}->servers())[0])
                                   ->hosttype()=~/ng/i;              
                            }}},
                sleep    => {type => SCALAR, default=>1,
                          callbacks=>{
                            'Is An Integer>0'=>sub{shift()>0}}},
                timeout  => {type => SCALAR, default=>300,
                          callbacks=>{
                            'Is An Integer>0'=>sub{shift()>0}}},
            }
        );
        
        $self->{on_err}=$args{on_err};
        $self->{server}=$args{server};
        $self->{sleep}=$args{sleep};
        $self->{timeout}=$args{timeout};
        
        if (defined $args{node}){
            $self->{server}->set_execution_server(-Server => $args{node});    
        }
        else {
            $args{node}=$self->{server}->get_execution_server();
        }
        
        $self->{node_name}=$args{node};
        
        
    };
    if ($@){
       $Log->comment("ERROR OCCURRED!!!!!!". $@); 
       ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
       return $self;
    }
    
    return $self;
}


###############################################################################
# Method:    aggrs_location_save
# Objective: save location of aggrs 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub aggrs_location_save {
    my ($self) = @_;
    my (
        $aggr,
        $node,
        $nodes,
        @aggrs_of,
        %map_of,
    );

    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node name object represents
    $node=$self->node_name();
    
    ### Set execution state
    $self->{server}->set_execution_server(-Server=>$node);
    
    eval {
        @aggrs_of=$self->{server}->storage_aggregate_show();
        foreach my $aggr_of (@aggrs_of){
            my $aggr=$aggr_of->{aggregate};
            my $nodes=(ref($aggr_of->{nodes}) eq "ARRAY")
                    ? join (",", sort @{$aggr_of->{nodes}}) 
                    : $aggr_of->{nodes};
            
            $map_of{$aggr}=$nodes;              
        }
    };
    if ($@){
        ($self->on_err()=~/die/)
            ? die $@
            : $self->_set_err($@);
            
            return {};
    }
    
    $self->{aggrs_location}=\%map_of;   
       
    return $self->{aggrs_location};  
}

###############################################################################
# Method:    aggrs_sendhome_wait
# Objective: Wait on aggregate to be sent home
# Details:   See POD documentation at the end of this file.
###############################################################################
sub aggrs_sendhome_wait {
    my ($self,%args)=@_;
    my (
        $errmsg,
        $node,
        @outputs,
        @aggrs,
        );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node name object represents
    $node=$self->node_name();
    
    ### Set execution state
    $self->{server}->set_execution_server(-Server=>$node);
    
    ### Check if test ran the instance's aggrs_location_save() prior
    ### to takeover.  If not there no location data to verify that
    ### aggregates are back home.  Fail if location data unavailable. 
    unless (defined $self->{aggrs_location}){
        $errmsg=
            "The original aggregate node locations were not saved during\n".
            "test.  Run the instance method aggrs_location_save() prior\n".
            "to performing a takeover operation.";
                
        if ($self->on_err()=~/die/) {
            die $errmsg
        }
        $self->_set_err($@);
        
        return 0;
    }
    
    ### Validate input arguments
    eval {
    ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            
            aggrs  =>   { type => SCALAR|ARRAYREF, optional=>1,
                          callbacks=>{
                            'aggrs exist'=>sub{
                                push @aggrs, (ref($args{aggrs}) eq "ARRAY")
                                    ? @{$args{aggrs}}
                                    : $args{aggrs};
                                
                                foreach my $aggr (@aggrs){
                                    unless (scalar(grep{/^$aggr$/}
                                        keys %{$self->{aggrs_location}})){
                                        return 0;
                                    }
                                }
                                1;
                            }}},       
           
        })
    };
    if ($@){
        ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
        
        return 0;
       
    }
    
    ### Determine what aggregates to monitor. By default we'll wait
    ### for all of them to be sent home.  However, if the user supplies
    ### either a scalar or arrayref of aggregate names then this method
    ### will return after those aggregates have been sent home
    if (not defined $args{aggrs}) {
        push @aggrs, sort keys %{$self->{aggrs_location}}; 
    }
    
    ### Remove the 'aggrs' parameter from the args list
    delete $args{aggrs};
        
    ### Iterate through each aggregate waiting until its been
    ### sent home.
    foreach my $aggr (@aggrs){
        
        ### skip if not one to be monitored
        next if (not scalar(grep{/^$aggr$/}
                (keys %{$self->{aggrs_location}})));                    
        
        ### Initialize method event parameters
        $args{_records}  = [$aggr];                 
        $args{_method} =
            '$self->{server}->storage_aggregate_show(aggregate=>$record)';
        $args{_param}  = "nodes";
        $args{_value}  = $self->{aggrs_location}{$aggr};
        
        ### Forward the request off
        eval {
            $self->_wait_on_event(%args);
        };
        if ($@){
            ($self->on_err()=~/die/)
                ? die $@
                : $self->_set_err($@);
                
                return 0;
        }
        
    }
    
     
    return 1;
}

###############################################################################
# Method:    cluster_health_wait
# Objective: Wait on cluster to be healthy 
# Details:   See POD documentation at the end of this file.
###############################################################################
sub cluster_health_wait {
    my ($self,%args)=@_;
    my (
        $node,
        );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node name object represents
    $node=$self->node_name();
    
    ### Set execution state
    $self->{server}->set_execution_server(-Server=>$node);
    
    ### Initialize method event parameters
    $args{_records}  = [$self->{server}->servers()];
    $args{_method} = '$self->{server}->cluster_show(node=>$record)';
    $args{_param}  = "health";
    $args{_value}  = "true";
    
    eval {
        $self->_wait_on_event(%args);
    };
    if ($@){
        ($self->on_err()=~/die/)
            ? die $@
            : $self->_set_err($@);
            
            return 0;
    }
    
    return 1;
}

###############################################################################
# Method:    failover_options
# Objective: Set failover options
# Details:   See POD documentation at the end of this file.
###############################################################################
sub failover_options {
    my ($self,%args)=@_;
    my (
        $cmd,
        $node,
        );
    
    $self->_clear_err();
    
    ### Retrieve node object represents
    $node=$self->node_name();
    
    ### Validate input arguments
    eval {
    ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            node     => { type => SCALAR, default=>$node,
                          callbacks=>{
                            'node exists'=>sub{
                                scalar(grep{/^$args{node}$/}
                                       $self->{server}->servers())           
                            }}},
            'auto-giveback'
                     => { type => SCALAR, optional => 1,
                          callbacks=>{
                            'legal values'=>sub{
                                shift()=~/true|false/i}}},
            timeout  =>  {type => SCALAR, default=>30,
                          callbacks=>{
                            'Is An Integer>0'=>sub{shift()>0}}},
                
        })
    };
    if ($@){
        ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
        
        return 0;
    }
    
    ### Make sure to set what node is executing command
    $self->{server}->set_execution_server(-Server=>$self->node_name());
    
    ### Separate out commands
    eval {
        
        ### Separate out special options that require 
        ### confirmation from the user.
        if (defined $args{"auto-giveback"}) {
            
            $cmd="storage failover modify ".
                 "-node ".$args{node}. " ".
                 "-auto-giveback ".$args{'auto-giveback'};
        
            ### Execute the method
            $self->{server}->cli(
                cmd => $cmd,
                    additional_prompts_timeout => $args{timeout},
                    additional_prompts => {
                        "Do you want to disable auto-giveback\?" => "y",
                        "::.?>" => "FINAL_PROMPT",
                    },
                );
            
            delete $args{"auto-giveback"};
            delete $args{"timeout"};
        };
        
        ### If there are other options remaining, process them.
        ### NOTE: There should be at least one key (node) that
        ### is always in this hash.  So look for something that
        ### has more than one key
        if (scalar(keys %args) > 1 ) {
          $self->{server}->storage_failover_modify(%args);
        }
        
    };
    if ($@){
        ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
        return 0;
    }
    
    return 1;
}


###############################################################################
# Method:    fav_location
# Objective: Retrieve fav location
# Details:   See POD documentation at the end of this file.
###############################################################################
sub fav_location {
    my ($self,%args)=@_;
    my (
        $cmd,
        $dsid,
        $loc,
        $node,
        $raw,
        @lines,
    );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node name object represents
    $node=$self->node_name();
    
    ### Set execution state
    $self->{server}->set_execution_server(-Server=>$node);
    
    ### Validate input arguments
    eval {
    ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {

            file_path => {type => SCALAR },
            volume    => { type => SCALAR },
                        
        })
    };
    if ($@){
        ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
    }
    
    eval {
        $dsid = $self->_get_fav_dsid(%args);              
        $loc  = $self->_get_dsid_location(dsid=>$dsid);
    };
    if ($@){
        ($self->on_err()=~/die/)
            ? die $@
            : $self->_set_err($@);
            
        return "";
    }
    
    return $loc; 
    
}


###############################################################################
# Method:    giveback
# Objective: giveback
# Details:   See POD documentation at the end of this file.
###############################################################################
sub giveback {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $node,
        );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node object represents
    $node=$self->node_name();
    
    ### Validate input arguments
    eval {
        ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            node     => { type => SCALAR, default=>$node,
                          callbacks=>{
                            'node exists'=>sub{
                                scalar(grep{/^shift()$/}
                                       $self->{server}->servers())           
                            }}},
            override_vetoes =>
                        { type => SCALAR, default => "false" ,
                           callbacks=>{
                             'Legal Values'=>
                                sub{shift()=~/true|false/i }}},
        });
    
        ### Make sure its set to the node object represents.
        ### Important to check since Server object is a singleton
        $self->{server}->set_execution_server(-Server=>$node);
        
        ### Execute the method
        $self->{server}->storage_failover_giveback(
            fromnode          => $args{node},
            "override-vetoes" => $args{override_vetoes},
        );
    };
    if ($@){
        $errmsg="ERROR: Unable to perform giveback";
        ($self->on_err()=~/die/)
            ? die $errmsg." ".$@
            : $self->_set_err($errmsg." ".$@);
            
        return 0;
    }
    
    return 1;
}

###############################################################################
# Method:    giveback_wait
# Objective: wait until giveback of other node storage is complete
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub giveback_wait {
    my ($self,%args)=@_;
    
    ### Reset error state
    $self->_clear_err();
    
    $args{msg}="is up";
    
    eval {
        $self->_wait(%args);
    };
    if ($@){
        ($self->on_err()=~/die/)
            ? die $@
            : $self->_set_err($@);
            
        return 0;
    }
    
    return 1;
}

###############################################################################
# Method:    node_name
# Objective: Return node_name
# Details:   See POD documentation at the end of this file.
###############################################################################
sub node_name {
    my ($self,%args)=@_;
    
    return $self->{node_name};
}

###############################################################################
# Method:    sync_disable
# Objective: Disable nvram checkpointing
# Details:   See POD documentation at the end of this file.
###############################################################################
sub sync_disable {
    my ($self,%args)=@_;
    my (
        $cmd,
        $node,
        );
    
    $self->_clear_err();
    
    ### Retrieve node object represents
    $node=$self->node_name();
    
    ### Validate input arguments
    eval {
    ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            node     => { type => SCALAR, default=>$node,
                          callbacks=>{
                            'node exists'=>sub{
                                scalar(grep{/^$args{node}$/}
                                       $self->{server}->servers())           
                            }}},
        })
    };
    if ($@){
        ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
       
        return 0;
    }
    
    ### Construct the command and execute it.
    eval {
        
        $cmd="node run -node ".$args{node}." -command ".
             "\"priv set test;sync disable\"";
        
        ### Make sure its set to the node object represents.
        ### Important to check since Server object is a singleton
        $self->{server}->set_execution_server(-Server=>$node);
        
        unless ($self->_get_build_type() =~ /debug/i){
            die "Disabling checkpointing requires an ONTAP Debug build";
        }
            
        ### Execute the method
        $self->{server}->cli(cmd => $cmd);
        
    };
    if ($@){
        ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
        
        return 0;
       
    }
    
    return 1;
}

###############################################################################
# Method:    on_err
# Objective: Return errhdlr_type
# Details:   See POD documentation at the end of this file.
###############################################################################
sub on_err {
    my ($self,$type)=@_; 
    
    if (defined $type) {
        $self->{on_err}=$type;
    }
    
    return $self->{on_err}; 
}

###############################################################################
# Method:    takeover
# Objective: perform storage takeover
# Details:   See POD documentation at the end of this file.
###############################################################################  
sub takeover {
    my ($self,%args)=@_;
    my (
        $cmd,
        $errmsg,
        $node,
        );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node object represents
    $node=$self->node_name();
    
    ### Validate input arguments
    eval{
        ($self, %args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            node     => { type => SCALAR, default=>$node,
                          callbacks=>{
                            'node exists'=>sub{
                                scalar(grep{/^shift()$/}
                                       $self->{server}->servers())           
                            }}},
            option    => { type => SCALAR,
                           callbacks=>{
                            'legal values'=>sub{
                                shift()=~/normal|immediate/i}}},
            
        });
        
        ### Make sure its set to the node object represents.
        ### Important to check since Server object is a singleton
        $self->{server}->set_execution_server(-Server=>$node);
        
        ### Execute the method
        $self->{server}->storage_failover_takeover(
            bynode=>$args{node},
            option=>$args{option}
        );
    };
    if ($@){
        $errmsg="ERROR: Unable to perform takeover";
        ($self->on_err()=~/die/)
            ? die $errmsg." ".$@
            : $self->_set_err($errmsg." ".$@);
            
        return 0;
    }      
    
    return 1;
}

###############################################################################
# Method:    takeover_wait
# Objective: wait until takeover of other node storage is complete
# Details:   See POD documentation at the end of this file.
############################################################################### 
sub takeover_wait {
    my ($self,%args)=@_;
    
    ### Reset error state
    $self->_clear_err();
    
    $args{msg}="ready for giveback";
    
    eval {
        $self->_wait(%args);
    };
    if ($@){
        ($self->on_err()=~/die/)
            ? die $@
            : $self->_set_err($@);
            
        return 0;
    }
    
    return 1;
}

###############################################################################
# Method:    vols_online_wait
# Objective: Wait on volume to be online
# Details:   See POD documentation at the end of this file.
###############################################################################
sub vols_online_wait {
    my ($self,%args)=@_;
    my (
        $node,
        @outputs,
        @volumes,
        );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node name object represents
    $node=$self->node_name();
    
    ### Set execution state
    $self->{server}->set_execution_server(-Server=>$node);
    
    ### Check for volumes passed in otherwise query
    ### for existing offline volumes to monitor
    unless (defined $args{vols}){
        eval {
            @outputs=$self->{server}->volume_show(state=>"offline"); 
            foreach my $record(@outputs){
                push @volumes,$record->{volume};
            }
        };
        if ($@){
            ### None exist - don't return error.
            return $self if ($@=~/no entries matching/i);
            
            ### Retrun error
            ($self->on_err()=~/die/)
                ? die $@
                : $self->_set_err($@);
                
            return 0;
        }
    }
    
    ### Initialize method event parameters 
    $args{_records}  = (defined $args{vols})
                        ? $args{vols}
                        : [@volumes];
    $args{_method} = '$self->{server}->volume_show(volume=>$record)';
    $args{_param}  = "state";
    $args{_value}  = "online";
    
    ### Remove the 'volumes' parameter from the args list
    ### whose purpose was to set the $args{_records} value
    delete $args{vols};
    
    ### Forward the request off
    eval {
        $self->_wait_on_event(%args);
    };
    if ($@){
        ($self->on_err()=~/die/)
            ? die $@
            : $self->_set_err($@);
        return 0;
    }
    
    return 1;
}



###############################################################################
#  P R I V A T E   F U N C T I O N S
###############################################################################
# Method: _get_fav_dsid
# Objective: Retrieve fav assiged to volume file
###############################################################################
sub _get_fav_dsid {
    my ($self,%args)=@_;
    my (
        $cmd,
        $dsid,
        $path,
        $raw,
        @lines,
    );
    
    ### Initialize variables
    $path=($args{file_path} !~ /^\//)
        ? "/".$args{file_path}
        : $args{file_path};
    
    $cmd=
        "wex inode ".$args{volume}.".".$path." ".
        "-list dsid,fav" ;
    
    ### Execute command to retrieve FAV dsid
    $raw=$self->{server}->shell(cmd=>$cmd);
    $raw=~s/\r\r\n/\n/g;
    @lines=split(/\n/,$raw);
    $Log->comment(@lines);

    ### Retrieve volume record containing fav and
    ### parse out dsid
    foreach my $line (@lines) {
        if ($line=~/fav/) {
            ($dsid)=$line=~/^\s+(\d+)/;
        }
    }
    
    unless (defined $dsid) {
        croak "FAV dsid not found";
    }
    
    return $dsid;    
}

###############################################################################
# Method:    _get_build_type
# Objective: Retrieve build type 
############################################################################### 
sub _get_build_type {
    my ($self,%args) = @_;
    my ($build_type);
    
    eval {
    $build_type = 
        $self->{server}->shell(
            cmd=>"cat /BUILD | grep 'TYPE:' | awk '{print \$2}'");
    };
    if ($@) {
        croak "ERROR: Retrieving build type failed.".$@;
    }
    
    return $build_type;
}

###############################################################################
# Method: _get_dsid_location
# Objective: Retrieve node name associated with dsid
###############################################################################
sub _get_dsid_location {
    my ($self,%args)=@_;
    my (
        $cmd,
        $loc,
        $raw,
        @lines,
    );
    
    ### Initialize variables
    $cmd="wex volume ".$args{volume}." -list dsid,location";
    
    ### Execute command to retrieve FAV dsid
    $raw=$self->{server}->shell(cmd=>$cmd);

    $raw=~s/\r\r\n/\n/g;
    @lines=split(/\n/,$raw);
    $Log->comment(@lines);
    
    ### Retrieve volume record name containing dsis
    foreach my $line (@lines) {
        if ($line=~/\s+$args{dsid}\s+/) {
            ($loc)=$line=~/\d+\s+(\S+)/;
        }
    }
    
    unless (defined $loc) {
        croak "convert dsid to location failed";
    }
    
    return $loc;
    
}



###############################################################################
# Method:    _wait
# Objective: wait until failover operation is complete
###############################################################################
sub _wait {
    my ($self,%args)=@_;
        my (
        $cmd,
        $node,
        $raw,
       );
    
    ### Retrieve node object represents
    $node=$self->node_name();
    
    ### Validate input arguments
    eval{
        ($self,%args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            msg      => { type => SCALAR },
            node     => { type => SCALAR, default=>$node,
                          callbacks=>{
                            'node exists'=>sub{
                                scalar(grep{/^shift()$/}
                                       @{$self->{server}->servers()})           
                            }}},
            sleep    => {type => SCALAR, default=>$self->{sleep},
                          callbacks=>{
                            'Is An Integer>0'=>sub{shift()>0}}},
            timeout  => {type => SCALAR, default=>$self->{timeout},
                          callbacks=>{
                            'Is An Integer>0'=>sub{shift()>0}}},
        })
    };
    if ($@){
        die $@;
    }
    
    ### Create command
    $cmd="node run -node ".$args{node}." -command \"cf status\"";
    
    ### Execute command in loop until state or
    ### timeout met
    eval {
        local $SIG{ALRM} = sub { die "command timeout (".$args{timeout}."s)"; }; 
        alarm $args{timeout};
        while (1) {
            eval {
                $raw=$self->{server}->cli(cmd=>$cmd);
            };
            if ($@){
                next if ($@=~/error sending/i);
                die $@;
            }
            last if ($raw=~/$args{msg}/i);
            
            Tharn::snooze($args{sleep});
        }
        alarm 0;
    };
    if ($@){
        die $@;
    }
    
    return $self;
}

###############################################################################
# Method:    _wait_on_event
# Objective: Wait on event
# Details:   See POD documentation at the end of this file.
###############################################################################
sub _wait_on_event {
    my ($self,%args)=@_;
    my (
        $method,
        $node,
        $param,
        $records,
        $value,
        );
    
    ### Reset error state
    $self->_clear_err();
    
    ### Retrieve node object represents
    $node=$args{node};
  
    ### Validate input arguments
    eval{
        ($self,%args) = validate_mixed(
        @_,
        [ { type => OBJECT } ],
        {
            
            sleep    => {type => SCALAR, default=>$self->{sleep},
                          callbacks=>{
                            'Is An Integer>0'=>sub{shift()>0}}},
            timeout  => {type => SCALAR, default=>$self->{timeout},
                          callbacks=>{
                            'Is An Integer>0'=>sub{shift()>0}}},
            _records => {type => ARRAYREF},
            _method  => {type => SCALAR},
            _param   => {type => SCALAR},
            _value   => {type => SCALAR},
        })
    };
    if ($@){
       ($self->on_err()=~/die/)
        ? die $@
        : $self->_set_err($@);
    }
    
    ### Initialize variables
    $records = $args{_records};
    $method  = $args{_method}; 
    $param   = $args{_param};
    $value   = $args{_value};
    
    ### timeout met
    eval {
        local $SIG{ALRM} = sub { die "command timeout (".$args{timeout}."s)";}; 
        alarm $args{timeout};
        foreach my $record (@$records){
            while (1){
                eval {
                    while (1) {
                        my @outputs=eval "$method";
                        
                        my $attrib = (ref ($outputs[0]->{$param}) eq "ARRAY")
                            ? join (",",@{$outputs[0]->{$param}})
                            : $outputs[0]->{$param};
                        
                        last if ($attrib =~ /$value/i);
                        
                        Tharn::snooze($args{sleep});
                    }
                };
                if ($@){
                    next if ($@=~/error sending/i);
                    die $@;
                }
                
                last;
            }
        }
        alarm 0;
    };
    if ($@){
        die $@;
    }
    
    return $self;
    
}

############################################################################
#  P O D
############################################################################

=pod

=head1 Package Name

SfoEvents.pm

=head1 Author

dl-striping-qa@netapp.com

=head1 Description

SfoEvents.pm provides an API interface for issuing storage failover requests.

=head1 See Also

Package inherits from the following modules. See these modules for further 
interface detail.

B<Error.pm>

=head1 Public Methods

=head2 B<new( B<server>, I<on_err>, I<node>, I<sleep>, I<timeout> )>

Creates a new SfoEvents object

=over 4

=item Inputs

=over 4

=item * B<server>  

Server object created to manage cluster configuration

=item * I<sleep> (default: 1)

Time to sleep in seconds between monitoring checks for state change

=item * I<timeout> (default: 300)

Maximum amount of time to wait in seconds for the command to complete before exiting with error

=item * I<on_err> (default: "die")    

Error handler to use for returning errors back to the calling script.

=begin html

<ul>
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=item * I<node> (default: $Server->get_execution_server())

Name of node to use for issuing CLI command requests.

=back

=item Outputs

Returns SfoEvents object handle.   

=item Examples

    # Example: Create  SfoEvents object
    $Test->nlog("- Creating SfoEvents object);
    my $sfoEventsObj = new sfoEvents(server=>$server,on_err=>'rtn');
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<aggrs_location_save( I<aggrs>)>

Save current node location(s) of aggregates prior to a takeover event.

=over 4

=item Inputs

=over 4

=item * I<aggregates>  (default: all)

Array reference of aggregate names

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples

    # Example: Save aggregate locations
    $Test->nlog("- Save away current aggregate locations");
    $sfoEventsObj->aggrs_location_save();
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<aggrs_sendhome_wait( I<aggrs>, I<sleep>, I<timeout> )>

Wait for aggregates to be sent home to original node(s) location 
prior to their takeover

=over 4

=item Inputs

=over 4

=item * I<aggregates>  (default: aggregates saved via aggrs_location_save())

Aggregates to monitor send home state

=item * I<sleep> (default: sleep value passed in during object creation (new()))

Time to sleep in seconds between monitoring checks for state change

=item * I<timeout> (default: timeout value passed in during object creation (new()))

Maximum amount of time to wait in seconds for the command to complete before exiting with error

=back

=item Outputs

Returns SfoEvents object handle 

=item Examples

    # Example: Wait for aggregates to be sent home
    $Test->nlog("- Wait for aggregates send home");
    $sfoEventsObj->aggrs_sendhome_wait();
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<cluster_health_wait( I<node>, I<sleep>, I<timeout> )>

Wait for cluster to be in a healthy state

=over 4

=item Inputs

=over 4

=item * I<node>  (default: node specified during object creation)

Name of the node to use to monitor the state of the cluster until its in a healthy state

=item * I<sleep> (default: sleep value passed in during object creation (new()))

Time to sleep in seconds between monitoring checks for state change

=item * I<timeout> (default: timeout value passed in during object creation (new()))

Maximum amount of time to wait in seconds for the command to complete before exiting with error


=back

=item Outputs

Returns 1 on success, 0 on failure 

=item Examples

    # Example: Wait for cluster to be healthy
    $Test->nlog("- Wait for cluster to be healthy");
    $sfoEventsObj->cluster_health_wait();
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<failover_options( I<node>, I<timeout>, I<auto-giveback> )>

Configure storage failover options on node

=over 4

=item Inputs

=over 4

=item * I<node> (default: node specified during object creation)   

Name of node to set the state of auto-giveback for.   

=item * I<timeout> (default: timeout value passed in during object creation (new()))

Maximum amount of time to wait in seconds for the command to complete before exiting with error
   
=item * I<auto-giveback> (values: true|false)

Turn auto-giveback on or off

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples

    # Example: Turn auto-giveback off
    $Test->nlog("- Disable auto-giveback");
    $sfoEventsObj->failover_options('auto-giveback'=>"false");
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<giveback( I<node>, I<override_vetoes> )>

Issues a storage giveback 

=over 4

=item Inputs

=over 4

=item * I<node>  (default: node specified during object creation)   

Name of node returning the storage

=item * I<override_vetoes> (default: "false")   

Option to override-vetoes that would otherwise prevent giveback from occuring

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples
 
    # Example: Perfom storage givback
    $Test->nlog("- Perfom storage giveback");
    $sfoEventsObj->giveback(override_vetoes=>"true");
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<giveback_wait( I<node>, I<sleep>, I<timeout> )>

Waits on storage giveback to complete

=over 4

=item Inputs

=over 4

=item * I<node>  (default: node specified during object creation)   

Name of node returning the storage

=item * I<sleep> (default: sleep value passed in during object creation (new()))

Time to sleep in seconds between monitoring checks for state change

=item * I<timeout> (default: timeout value passed in during object creation (new()))

Maximum amount of time to wait in seconds for the command to complete before exiting with error

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples
 
    # Example 1: Wait on giveback
    $Test->nlog("- Wait on storage giveback to complete");
    $sfoEventsObj->giveback_wait(timeout=>180);
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

    # Example 2: Issue giveback and wait
    $Test->nlog("- Execute giveback and wait to complete");
    eval {
       $sfoEventsObj->giveback()->giveback_wait(timeout=>180);
    };
    if ($@) {
       $Test->nlog($@);
    }

=back

=head2 B<node_name()>

Node name being used by object instance to issue cluster requests.

=over 4

=item Outputs

Returns a scalar value containing the node name

=item Examples 

 # Get nodename
 $nodename= $sfoEventsObj->node_name();
 $Test->nlog("Node" ". $nodename);

=back

=head2 B<on_err( I<$type> )>

Defines what error handling mechanism to use on SfoEvents object instance

=over 4

=item Inputs

=over 4

=item * I<$type>

Indicates what errhandler type to use for all  SfoEvents object methods
calls.  If no errorhandle is provided, it returns the current handler type 

=begin html

<ul>
<li><b>die</b>: All errors encountered through object are thrown by die 
<li><b>rtn</b>: All errors encountered are stored in the object.  It is the
responsibility of the caller to analyze errors through additional method calls.
See module <b>ERROR</b> for detail.
</ul><p>

=end html

=back

=item Outputs

Returns error handle type  on success, throws error on failure

=item Examples

 # Example 1: Set the defualt error handle from 'die' to 'rtn'   
 $hdlr=$sfoEventsObj->on_err('rtn');
 $Test->nlog("Error handler changed to '".$hdlr."'");

=back

=head2 B<sync_disable( I<node> )>

Disables nvram checkpointing on selected node.  Once disabled it will not  
be enabled until the node is panicked or powercycled to restart. Anything 
syncing the file system will hang.  Reboot or halt will panic.  

This method only works with an ONTAP debug build.  It will fail if executed
on non-debug.

=over 4

=item Inputs

=over 4

=item * I<node>  (default: node specified during object creation)  

Name of the node to disable checkpointing

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples

 # Example 1: disable checkpointing 
 $sfoEventsObj->sync_disable(node=>'gi-3040-2');

=back

=head2 B<takeover( I<node>, B<option> )>

Issues a storage takeover 

=over 4

=item Inputs

=over 4

=item * I<node>  (default: node specified during object creation)   

Name of node performing the takeover

=item * B<option> (values: normal|immediate)   

Takeover the storage after either gracefully or immediatly shutting it down

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples
 
    # Example: Perfom storage takeover
    $Test->nlog("- Perfom storage takeover");
    $sfoEventsObj->takeover(option=>"immediate");
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<takeover_wait( I<node>, I<sleep>, I<timeout> )>

Wait on storage takeover to complete

=over 4

=item Inputs

=over 4

=item * I<node>  (default: node specified during object creation)   

Name of node taking over the storage

=item * I<sleep> (default: sleep value passed in during object creation (new()))

Time to sleep in seconds between monitoring checks for state change

=item * I<timeout> (default: timeout value passed in during object creation (new()))

Maximum amount of time to wait in seconds for the command to complete before exiting with error

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples
 
    # Example 1: Wait on takeover
    $Test->nlog("- Wait on storage takeover to complete");
    $sfoEventsObj->takeover_wait(timeout=>180);
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=head2 B<vols_online_wait( I<vols>, I<sleep>, I<timeout> )>

Wait for volumes to come back online

=over 4

=item Inputs

=over 4

=item * I<vols> (default: all existing offline volumes)    

A scalar or array reference of volume names to monitor

=item * I<sleep> (default: 1)

Time to sleep in seconds between monitoring checks for state change

=item * I<timeout> (default: 300)

Maximum amount of time to wait in seconds for the command to complete before exiting with error

=back

=item Outputs

Returns 1 on success, 0 on failure

=item Examples

    # Example: Wait for particular volume to be online
    $Test->nlog("- Wait for volume to be on line");
    $sfoEventsObj->vols_online_wait(vols=>"p_acro_1");
    if ($sfoEventsObj->error()){
        $Test->nlog($sfoEventsObj->errmsg());
        return $TCD::FAIL;
    }

=back

=cut


1;
