# $Id: //depot/prod/test/Rsierranevada/lib/SfoUtil.pm#1 $
## @ SfoUtil.pm - small utilities for sfo tests
## @author Thiem Pho
## @Date created: Sept 20 2007
##  Completed: Dec 04 2007
##

use strict;
use warnings;

#
# Libraries to use.
#
use Tharn;
use Server;
use Options;
use Client;
use TracePkg;
use VSeries::client_object;
use Data::Dumper;

my $Server = new Server();
my $Trace = new TracePkg;
my $client_obj = VSeries::client_object->new();
my $options = new Options();

######## run_dt ################################################################
sub run_dt
{
   my $client           = shift;        # hostname of test client running i/o
   my $testdir          = shift;        # test directory - equals to $nfs_mnt 
   my $vif_mnt          = shift;        # filer data port (VIF)
   my $runtime          = shift;        # time to run IO
   my $blocksize        = shift;        # Blocksize to write
   my $procs            = shift;        # number of threads
   my $limit            = shift;        # filesize
   my $wait             = shift;        # waiting to finish?
   my $ref_Logfiles     = shift;        # Pointer to arrays of logfiles

   my $hostrec_object;  # switch host object 
   my $connectrec_object;  # connectrec object 
   my $dt_bin="/usr/bin/dt";
   my $status = 0;
   my $rc = 0;
   my @testdirs;
   my $getmountHash;
   my $filesys="";
   my %mountHash;
   my @tmparray=();
   my $args = "";
   my @pids=();
   my $pid;
   my $of;
   my $file_size;
   my $return_code;
   my @tmparr;
   my $tmp;
   my $logfiles;
   my $dt_testfile = "dt_file";
   my $dt_ok = 14;              ### dt PASS return code
   my $pattern="0xdeadbeef";
   my $logfile;

   #
   # Connect to client.
   #
   $hostrec_object = Tharn::host($client);
   $connectrec_object = Tharn::connect("#rlogin");

   $client_obj->set_internal_variables(

                     hostname      => $client,
                     default_ip    => $hostrec_object->default_ip(),
                     conntype      => $connectrec_object->conntype(),
                     connport      => $connectrec_object->connport(),
                     username      => $connectrec_object->username(),
                     password      => $connectrec_object->password(),
                     conn          => $connectrec_object,
    );

   $Trace->comment(msg=> "clientname = $client");
   $Trace->comment(msg=> "IO method  = dt");

   #
   # Create test directory
   #
   $testdirs[0]=$testdir;

   $rc =$client_obj->create_dir(@testdirs);

   if ($rc) {
       $Trace->fatal(msg=>"Create directory $testdir failed",result=>"FAIL");
       return (1);
   } else {
       $Trace->comment(msg=>"Create directory $testdir succeeded.");
   }

   #
   # Mount the NFS mount point
   #
   $args="-t nfs $vif_mnt $testdir";
   $rc =$client_obj->set_mount($args);
   if ($rc) {
       $Trace->fatal(msg=>"mount $args failed",result=>"FAIL");
       return (1);
   } else {
       $Trace->comment(msg=>"mount $args succeeded.");
   }

   $getmountHash =$client_obj->get_mount();
   %mountHash = %$getmountHash;

   foreach $filesys ( sort keys %mountHash ) {
       $Trace->debug(msg=> "stats for $filesys: @{$mountHash{$filesys}})");
   }

   $args = $testdir."/"."*";
    
   $Trace->comment(msg=> "Delete old test files $args");
   @tmparray = $client_obj->send_client_cmd("rm -f $args");

   #
   # cleanup log files prior test
   #
   $logfiles = $testdir."/"."*"."."."log";
   $Trace->comment(msg=> "Delete old log files $logfiles");
   @tmparray = $client_obj->send_client_cmd("rm -f $logfiles");
    
   #
   # Test starts here. 
   #
   if ($procs eq "1") {
      $file_size = $limit;
   }
   else {
      $tmp = $limit / $procs;
      $file_size = convert_num ($tmp, "F");   ### remove floating point
   }

   #
   # Construct output file for dt to generate.
   #
   $of = $testdir."/".$dt_testfile;
   $logfile=$testdir."."."dt"."."."log";
   push (@${ref_Logfiles}, $logfile);   ## save logfile for later error checking

   if ($procs eq "1") {
      $args = "of=$of bs=$blocksize pattern=$pattern dispose=keep enable=compare,lbdata runtime=$runtime limit=$file_size log=$logfile";
   }
   else {      ### multithreaded 
      $args = "of=$of bs=$blocksize pattern=$pattern dispose=keep procs=$procs enable=compare,lbdata runtime=$runtime limit=$file_size log=$logfile";
   }

   # 
   # Launch dt background
   # 
   @pids = $client_obj->run_io("$dt_bin", "$args");
   if (scalar(@pids) == 0) {
      $Trace->fatal(msg=>"Subtest $dt_bin $args failed",result=>"FAIL");
      return (1);
   }
   $Trace->comment(msg=>"Executed \"$dt_bin $args\" in background, pids = @pids");
   if (!$wait) {
      $Trace->comment(msg=>"No waiting requested. Returning...");
      return @pids;    ### for later error check.
   }
   else {
      $Trace->comment(msg=>"Sleep $runtime secs for subtest(s) to finish...");
      sleep $runtime;
   }

   #
   # Checking subtest(s) to return.
   #
   foreach my $PID (@pids) {
      $Trace->comment(msg=> "Checking pid=$PID...");
      $rc = wait_pid($PID,"10");
      if ($rc != 0) {
         $Trace->fatal(msg=>"Failed: subtest pid=$PID timed out",result=>"FAIL",);
         $status = 1;
      }
   }

   #
   # Now check for error.
   # 
   if ($procs eq "1") {
      $return_code = dt_check ($pids[0],$logfile,"0");
      if ($return_code ne $dt_ok) {
         $Trace->fatal(msg=>"Subtest return error = $return_code. Check log file $logfile on client.",result=>"FAIL",);
         $status = 1;
      }
      else {
         $Trace->comment(msg=>"Subtest passed");
      }

   }   ### if ($procs eq "1")
   else {
      #
      # Note $i=1 => skip the 1st pid which is the parent process ID.
      #
      $Trace->comment(msg=>"All pids '@pids'");
      for (my $i=1; $i < scalar(@pids);$i++) {
         $Trace->comment(msg=> "Checking the returned result of pid=$pids[$i]...");
         $return_code = dt_check ($pids[$i],$logfile,"1");
         if ($return_code ne $dt_ok) {
            $Trace->fatal(msg=>"Subtest pid=$pids[$i] failed with returned code=$return_code",result=>"FAIL",);
         }
         else {
            $Trace->comment(msg=>"Subtest pid=$pids[$i] passed");
         }
      }   ### for (my $i=1; y <=scalar(@pids);$i++) 
   }

   #
   # No clean up if test failed.
   #
   if ($status) {
      $Trace->fatal(msg=>"Test failed. No clean up perform.");
   }
   else {
      $rc = cleanup($of,$testdir);
      if ($rc) {
         $Trace->comment(msg=>"Clean up failed. Manual clean up is required!");
      }
   }

   return ($status);

} # end run_dt

######## run_hammer ################################################################
sub run_hammer
{
   my $client           = shift;        # hostname of test client running i/o
   my $testdir          = shift;        # test directory 
   my $vif_mnt          = shift;        # filer data port
   my $runtime          = shift;        # time to run IO
   my $blocksize        = shift;        # Blocksize to write
   my $wait             = shift;        # waiting for finish? 
   my $ref_Logfiles     = shift;        # Pointer to arrays of logfiles

   my $hostrec_object;  # switch host object 
   my $connectrec_object;  # connectrec object 
   my $hammer_bin="/usr/local/bin/hammer";
   my $status = 0;
   my $rc = 0;
   my @testdirs;
   my $getmountHash;
   my $filesys="";
   my %mountHash;
   my @tmparray=();
   my $args = "";
   my @pids=();
   my $pid;
   my $of;
   my @tmparr;
   my $tmp;
   my $logfiles;
   my $logfile;

   #
   # Connect to client
   #
   $hostrec_object = Tharn::host($client);
   $connectrec_object = Tharn::connect("#rlogin");

   $client_obj->set_internal_variables(

                     hostname      => $client,
                     default_ip    => $hostrec_object->default_ip(),
                     conntype      => $connectrec_object->conntype(),
                     connport      => $connectrec_object->connport(),
                     username      => $connectrec_object->username(),
                     password      => $connectrec_object->password(),
                     conn          => $connectrec_object,
    );

   $Trace->comment(msg=> "clientname = $client");
   $Trace->comment(msg=> "IO method  = hammer");

   #
   # Create test directory
   #
   $testdirs[0]=$testdir;

   $rc =$client_obj->create_dir(@testdirs);

   if ($rc) {
       $Trace->fatal(msg=>"Create directory $testdir failed",result=>"FAIL");
       return (1);
   } else {
       $Trace->comment(msg=>"Create directory $testdir succeeded.");
   }

   #
   # Mount NFS mount point
   #
   $args="-t nfs $vif_mnt $testdir";
   $rc =$client_obj->set_mount($args);
   if ($rc) {
       $Trace->fatal(msg=>"mount $args failed",result=>"FAIL");
       return (1);
   } else {
       $Trace->comment(msg=>"mount $args succeeded.");
   }

   $getmountHash =$client_obj->get_mount();
   %mountHash = %$getmountHash;

   foreach $filesys ( sort keys %mountHash ) {
      $Trace->debug(msg=> "stats for $filesys: @{$mountHash{$filesys}})");
   }

   $args = $testdir."/"."*";
    
   $Trace->comment(msg=> "Delete old test files $args");
   @tmparray = $client_obj->send_client_cmd("rm -f $args");

   #
   # cleanup log files prior test
   #
   $logfiles = $testdir."/"."*"."."."log";
   $Trace->comment(msg=> "Delete old log files $logfiles");
   @tmparray = $client_obj->send_client_cmd("rm -f $logfiles");
   
   #
   # Preparation
   #
   if ($blocksize eq "") {            ### no passed in
      $blocksize = "16k";             ### use the default
   }
   $tmp = $testdir."/"."core";
   $logfile=$testdir."."."hammer"."."."log";
   push (@${ref_Logfiles}, $logfile);   ## save logfile for later checking

   #
   # hammer IO starts here. 
   # 
   $args = "$testdir -filercore $tmp -maxfsize 32K -runtime $runtime -blocksize $blocksize -logfile $logfile -bg";
   @pids = $client_obj->run_io("$hammer_bin", "$args");
   $pid = $pids[0];
   $Trace->comment(msg=>"Executed \"$hammer_bin $args\" in background, pid=$pid");
   if (!$wait) {
      $Trace->comment(msg=>"No waiting requested. Returning...");
      return ($pid);
   }
   else {
      $Trace->comment(msg=>"Sleep $runtime seconds for subtest to finish...");
      sleep $runtime;
   }

   $Trace->comment(msg=>"Checking pid=$pid...");
   $rc = wait_pid($pid,"10");
   if ($rc != 0) {
      $Trace->fatal(msg=>"Failed: subtest pid $pid timed out",result=>"FAIL",);
      $status = 1;
   }

   #
   # Now check for error.
   #
   my $errmsg = $client_obj->send_client_cmd("/bin/grep -i error $logfile");
   if ($errmsg ne "0") {
      $Trace->fatal(msg=>"Subtest return error = $errmsg",result=>"FAIL",);
      $status = 1;
   }
   else {
      $Trace->comment(msg=>"Subtest '$hammer_bin $args' passed");
      $of = $testdir."/"."*"."."."ham";
      $rc = cleanup($of,$testdir);
      if ($rc) {
         $Trace->comment(msg=>"Clean up failed. Manual clean up is required!");
      }
   }

   return ($status);

} # end run_hammer

sub cleanup {
   my $test_file = $_[0];
   my $test_dir  = $_[1];
   my $rc = 0;
   my $test_files;

   $Trace->comment(msg=>"Clean up...");

   $test_files = $test_file."*";
   $Trace->comment(msg=>"Delete test file(s) $test_files");
   $client_obj->send_client_cmd("rm -f $test_files");
   
   $rc = $client_obj->set_unmount($test_dir);
   if ($rc) {
       $Trace->comment(msg=>"umount $test_dir failed");
   } else {
       $Trace->comment(msg=>"umount $test_dir succeeded");
   }

   $rc =$client_obj->remove_dir($test_dir);
   if ($rc) {
       $Trace->comment(msg=>"rmdir $test_dir failed");
   } else {
       $Trace->comment(msg=>"rmdir $test_dir succeeded");
   }

   return ($rc);

}

sub convert_num {
   my $num = $_[0];
   my $type = $_[1];
   my $dot=".";
   my @tmp;
   my $kilo=1024;
   my $mega=($kilo * $kilo);
   my $giga=($mega * $mega);
   my $whole;
   my $a1;
   my $a2;

   if ($type eq "MB") {    ### type pass in already in mega byte format
      $a1 = substr($num,0,-2);   ### remove 'MB' at the end.
      #
      # remove the floating point '.'
      #
      if (grep /$dot/, $a1) {
         @tmp = split('\.',$a1);
         $whole = $tmp[0];
      }
      else {     ### no dot
         $whole = $a1;
      }
   }
   elsif ($type eq "G") {       ### in Giga format, convert to MByte
      $a1 = substr($num,0,-1);   ### remove 'G' at the end.
      $a2 = ($a1 * $kilo);
      if (grep /$dot/, $a2) {
         @tmp = split('\.',$a2);
         $whole = $tmp[0];
      }
      else {     ### no dot
         $whole = $a2;
      }
   }
   elsif ($type eq "B") {       ### in integer format, convert to MByte
      $a1 = ($num / $mega);
      if (grep /$dot/, $a1) {
         @tmp = split('\.',$a1);
         $whole = $tmp[0];
      }
      else {     ### no dot
         $whole = $a1;
      }
   }
   elsif ($type eq "F") {    ### remove the floating point only
      if (grep /$dot/, $num) {
         @tmp = split('\.',$num);
         $whole = $tmp[0];
      }
      else {     ### no dot
         $whole = $num;
      }
   }

   return $whole;
}

sub wait_pid {
   my $pid = $_[0];
   my $sleep_time = $_[1];
   my $no_pid = 0;
   my $status = 0;
   my $line;
   my $loop_cnt=1;
   my $count = 20;
   my @tmparray;

   while ( ! $no_pid ) {
      $Trace->comment(msg=> "Looking for pid=$pid...");
      @tmparray = $client_obj->send_client_cmd("ps ax | grep \"$pid\"");
      foreach $line (@tmparray) {
         if ($line =~ /^ *?${pid} / ) { 
             $Trace->comment(msg=>"sleep $sleep_time sec and check pid=$pid. Count $loop_cnt");
             sleep $sleep_time; 
             last;
         }
         else { 
             $Trace->comment(msg=>"pid $pid has returned");
             $no_pid=1;
         }
      }
      if ($loop_cnt == $count) {
         $Trace->fatal(msg=>"Timed out waiting for pid='$pid' to return",result=>"FAIL");
         $status = 1;
         last;
      }
   $loop_cnt++;
   }

   return $status;
}

sub dt_check {
   my $pid     = $_[0];
   my $logfile = $_[1];
   my $multi_threads = $_[2];
   my $dt_msg = "Terminate code";
   my @tmp1;
   my @tmp2;
   my $tmp3;
   my $rc=0;

   if ($multi_threads eq "0")    {    ### single thread, no need to look at pid
      @tmp1 = $client_obj->send_client_cmd("/bin/grep -i \"$dt_msg\" $logfile");
      @tmp2 = split(' ',$tmp1[0]);
      chomp($tmp2[3]);  ### get rid of ^M (\r\n)
      $rc = $tmp2[3];
   }
   else {     ### multithreaded 
      @tmp1 = $client_obj->send_client_cmd("/bin/grep -i \"$dt_msg\" $logfile");
      foreach my $ret (@tmp1) {
         if ($ret =~ /$pid/) {
            @tmp2 = split(' ',$ret);
            chomp($tmp2[4]);
            $rc = $tmp2[4];
            last;
         }
      } 
   } 

   return $rc;
}

sub create_vol {
   my $VS = $_[0];
   my $vol = $_[1];
   my $Vol_size = $_[2];
   my $server = $_[3];
   my $rc = 0;
   my $Hash;
   my $hash;
   my $Aggr;
   my $aggr = "none";

   $Hash = $Server->storage_aggregate_show(nodes => $server);

   for (my $i=0; $i<=(scalar(@{$Hash}) - 1); $i++) {
     $Aggr = $$Hash[$i]{aggregate};
     $hash = $Server->storage_aggregate_show(
                                            nodes => $server,
                                        aggregate => $Aggr);
#    $Trace->comment(msg=>"Dumper of \$hash:\n".Dumper($hash));
#    print "m=$$hash[0]{'has-mroot'}\n";
     if ($$hash[0]{'has-mroot'} ne "true") {
         $aggr = $$Hash[$i]{aggregate};
     }
   }

   if ($aggr eq "none") {
      $Trace->fatal(msg=>"Could not find any aggregate on $server to create volume $vol",result=>"FAIL",);
      return (1);
   }

   eval {
         $Server->volume_new(vserver => $VS,
                              volume => $vol,
                           aggregate => $aggr,
                               state => "online",
                              parent => $vol,
                              size   => $Vol_size,
                   'junction-active' => "true",
                   'junction-path'   => "/".$vol,
                   );
         };

   if ($@) {
       $Trace->fatal(msg=>"Error from creating volume $vol",result=>"FAIL",);
       $rc = 1;
   }
   else {
       $Trace->comment(msg => "Volume $vol, size=$Vol_size successfully created");
   }

   return $rc;
}

sub vol_delete {
   my $vol = $_[0];
   my $VS =  $_[1];
   my $rc = 0;
   my $params = $options->get_hash();

   eval {
      $Server->volume_delete(vserver=>$VS,
                          volume=>$vol,);
   };
   if ($@) {
      $Trace->fatal(msg=>"Failed to delete volume $vol",result=>"FAIL",);
      $rc = 1;
   }
   else {
      $Trace->comment(msg=>"Sucessfuly delete volume $vol");
   }
   return $rc;
}

sub vols_delete {
   my $aggregate = $_[0];
   my $dump;
   my $vol;
   my $Hash;
   my $i;
   my $params = $options->get_hash();

   eval {
      $Hash = $Server->volume_show(vserver=> $$params{VS},
                               volume=> $$params{VOLUME}[0]."_".$aggregate."\*",
                               aggregate=> $aggregate, );
   };

   if ($@ =~ /There are no entries matching your query/) {
      $Trace->comment(msg=>"No volumes found from agrregate $aggregate to be deleted.");
      return;
   }
   else {
      $Trace->comment(msg=>"All volumes from agrregate $aggregate will be deleted.");
   }

   for ($i=0; $i<=(scalar(@{$Hash}) - 1); $i++) {
      $Trace->comment(msg=>"Deleting volume $$Hash[$i]{volume}");
      $Server->volume_delete(vserver=>$$params{VS},
                                   volume=>$$Hash[$i]{volume},);
   }
}

sub run_ptar {
   my $client = shift; 
   my $vol = shift;
   my $vif = shift; 
   my $duration = shift; 
   my $ptar_count = shift; 
   my $ref_Subtest_handles = shift; 

   my $vif_mnt;
   my $nfs_mnt;
   my $Client_handle;
   my $Client_connection;
   my $rc=0;
   my $Failure;
   my $Message="Test";
   my $params = $options->get_hash();

   $vif_mnt=$vif.":/".$vol;
   $nfs_mnt="/nfs/sfo/".$client."/".$vol;
   $Trace->comment(msg=> "Mounting $vif_mnt on $nfs_mnt ..."); 

  $Client_handle = host($client);
   if (!defined($$Client_handle{CLIENT_CONN})) {
      $Client_connection = connect('#rlogin');
   } else {
      $Client_connection = connect($$Client_handle{CLIENT_CONN});
   }

	  say($Client_connection, "sudo mkdir $nfs_mnt", "-fatal", "Timed out waiting for prompt after mkdir $nfs_mnt on client $client");
	  say($Client_connection, "sudo mount $vif_mnt $nfs_mnt", "-fatal", "Timed out waiting for prompt after mount $vif_mnt $nfs_mnt on client $client");
	  say($Client_connection, "sudo chmod 777 $nfs_mnt", "-fatal", "Timed out waiting for prompt after chmod of $nfs_mnt on client $client");
	  my $handle = undef;
	  $options->set(TESTCASE=>undef);
     $options->set(DURATION=>undef);
     $options->set(IRATE=>undef);
     eval {
               $handle = subtest ("cluster/fs/ptar/ptar.thpl",
                                   '-bg',
                                   '-client',"$client",
                                   '-runid',"ptar_sfo_takeover_sendhome_".$client."_".$vol,
                                   "testdir=$nfs_mnt",
                                   "DURATION=$duration",
                                   "count=$ptar_count",
 				   "pathdiff_options=-only Checksum,Mode -noscan -fulldebug",
                                   "PTAR=original",
                                   "TEST_MODE=classic",
                                   "SKIP_NUFFO=1",
                                  );
            };
    if ($@) {
       $Failure="$Message: An unexpected error occured in subtest, handle '$handle'";
       $Trace->fatal(msg=>"$Message: mount_vol() failed",result=>"FATAL",);
       $rc++;
    }
    else {
       push(@${ref_Subtest_handles}, $handle);
    }

    return $rc;
}

sub check_subtest {
   my $ref_pids = shift;
   my $ref_logs = shift;
   my $subtest  = shift;
   my $rc = 0;
   my $res = 0;
   my $dt_ok = 14;
   my $i = 0;

   print "a=@${ref_pids}-b=@${ref_logs}-c=$subtest\n";
   foreach my $PID (@${ref_pids}) {
      $Trace->comment(msg=> "Checking pid=$PID...");
      $rc = wait_pid($PID,"10");
      if ($rc != 0) {
         $Trace->fatal(msg=>"Failed: pid=$PID timed out",result=>"FAIL",);
         last;
      }
      else { ### check for error.
         if ($subtest eq "run_dt") {
            $res = dt_check ($PID,$ref_logs->[$i],"0");
            if ($res ne $dt_ok) {
               $Trace->fatal(msg=>"Failed: subtest with pid=$PID, check log file $ref_logs->[$i] for more detail",result=>"FAIL",); 
            }
            else {
               $Trace->comment(msg => "subtest with pid $PID passed");
            }
         }
         else {     ### hammer
            my $errmsg = $client_obj->send_client_cmd("/bin/grep -i error $ref_logs->[$i]");
            if ($errmsg ne "0") {
              $Trace->fatal(msg=>"Subtest pid $PID error = $errmsg",result=>"FAIL",);
              $Trace->comment(msg=>"Check $ref_logs->[$i] for detail");
              $rc = 1;
            }
            else {
              $Trace->comment(msg => "subtest with pid $PID passed");
            }
         }
      }
      $i++;
   }
   return $rc;
}

sub connect_client {
   my $client = $_[0];

   my $hostrec_object = Tharn::host($client);
   my $connectrec_object = Tharn::connect("#rlogin");

   $client_obj->set_internal_variables(

                     hostname      => $client,
                     default_ip    => $hostrec_object->default_ip(),
                     conntype      => $connectrec_object->conntype(),
                     connport      => $connectrec_object->connport(),
                     username      => $connectrec_object->username(),
                     password      => $connectrec_object->password(),
                     conn          => $connectrec_object,
    );
}

sub get_data_ports {
   my $filer = shift;
   my $ref_data_ports = shift;
   my $VS = shift;
   my $Hash;
   my $port;

   $Hash = $Server->network_interfaces_show(server=>$VS,
                                            role => "data",
                                            'home-node' => $filer,
                                            );
        if ($@ =~ /There are no entries matching your query/) {
            $Trace->fatal(msg=>"No data ports for $filer. Exiting...");
        }
        else {
            $Trace->comment(msg=>"data ports for $filer");
             for (my $i=0; $i<=(scalar(@{$Hash}) - 1); $i++) {
                 $port = $$Hash[$i]{'curr-port'};
                 $Trace->comment(msg=>"port = $port");
                 push (@${ref_data_ports}, $port);
             }
        }
}

sub get_port_info {
   my $filer = shift;
   my $ref_data_ports = shift;
   my $ref_IP_address = shift;
   my $ref_data_mask = shift;
   my $option_ref;
   my $Hash;
   my $ip1;
   my $ip2;
   my $port1;
   my $port2;
   my $mask1;
   my $mask2;
   
   $option_ref = $options->get_hash();
   $Hash  = $option_ref->{$filer} = host($filer);
   $port1 = $option_ref->{$filer}->{DATA_PORT1};  ### e1a
   $port2 = $option_ref->{$filer}->{DATA_PORT2};  ### e1b
   $ip1   = $option_ref->{$filer}->{DATA_IP1};    ### 172.16.12.1
   $ip2   = $option_ref->{$filer}->{DATA_IP2};    ### 172.16.12.2
   $mask1   = $option_ref->{$filer}->{DATA_MASK1};    ### 255.255.0.0
   $mask2   = $option_ref->{$filer}->{DATA_MASK2};    ### 255.255.0.0 

   if ($port1 ne "") {
      push (@${ref_data_ports}, $port1);
   }
   else {
      $Trace->warn(msg=>"port $port1 has no value, blank");
   }
   if ($port2 ne "") {
      push (@${ref_data_ports}, $port2);
   }
   else {
      $Trace->warn(msg=>"port $port2 has no value, blank");
   }
   if ($ip1 ne "") {
      push (@${ref_IP_address}, $ip1);
   }
   else {
      $Trace->warn(msg=>"IP address $ip1 has no value, blank");
   }
   if ($ip2 ne "") {
      push (@${ref_IP_address}, $ip2);
   }
   else {
      $Trace->warn(msg=>"IP address $ip2 has no value, blank");
   }
   if ($mask1 ne "") {
      push (@${ref_data_mask}, $mask1);
   }
   else {
      $Trace->warn(msg=>"data mask $mask1 has no value, blank");
   }
   if ($mask2 ne "") {
      push (@${ref_data_mask}, $mask2);
   }
   else {
      $Trace->warn(msg=>"data mask $mask2 has no value, blank");
   }

}

sub vif_create {
   my $filer = shift;
   my $VS = shift;
   my $vif_ports = shift;
   my $vif_address = shift;
   my $net_mask = shift;
   my $ref_Vif_create = shift;
   my $policy = shift;
   my $status_admin = shift;
   my @data_ports;
   my @vif_IPs;

   @data_ports = split(/,/,$vif_ports); 
   @vif_IPs = split(/,/,$vif_address); 

   for (my $i=0,my $j=1; $i<=(scalar(@data_ports) - 1); $i++,$j++) {
      my $vif = $filer."_data".$j;
      $Trace->comment(msg=>"Creating VIF $vif for $filer");
      $Server->network_interfaces_new(server => $VS,
                                      vif => $vif,
                                      'home-filer' => $filer,
                                      'home-port' => $data_ports[$i],
                                      address => $vif_IPs[$i],
                                      netmask => $net_mask,
                                      role => "data",
                                      'failover-policy' => $policy,
                                      'status-admin' => $status_admin,
                                      );

      push (@${ref_Vif_create}, $vif);
   }
}

sub startTraffic {
  my $junction_path = shift;
  my $mount_path = shift;
  my $client_name = shift;
  my $run_time = shift;
  my $handle;
 
  my $Client = new Client(-host=>$client_name,
           -timeout=>300,
           -protocol=>"nfs"
  );
 
  if ($Client->error)
  {
      $Trace->comment(msg=>$Client->errmsg);
  }
 
  $Trace->comment(msg=>"Traffic generator: hammer");
  $Trace->comment(msg=>"Junctions Path: $junction_path");
  $Trace->comment(msg=>"Mount Path: $mount_path");
 
  # NFS share to mount
 
  my $mntObj = $Client->create_mnt(-share=>$junction_path,
                                -path=>$mount_path);
  if ($mntObj->error) {
      $Trace->comment(msg=>$mntObj->errmsg());
  }
 
     eval
     {
       $handle = subtest("/usr/local/bin/sudo",
           '-client',$client_name,
           '-bg',
           '--',
           "/usr/local/bin/hammer", "$mount_path",
           "-bg","-nofilercore",
           "-runtime","$run_time",
           "-blocksize","random",
           "-logfile","/var/hammer.log",
           "-fill","-nocleanup",
       );
     };
 
  return $handle;
}
