#Hey emacs, this is basically -*- perl -*-
# $Id$
#***********************************************************************
#
# NAME
#   Shoc::CompareResult.pm 
#
# SYNOPSIS
#   Shoc (show-output-compare) Compare result object.
#
# DESCRIPTION
#   This library provides a simple result object for comparisons.
#
# AUTHOR
#   dl-coati-core
#***********************************************************************

=pod

=head1 Library Name

Shoc::CompareResult.pm

=head1 Author

dl-coati-core

=head1 Description

This class defines an object consisting of a Boolean good(1)/bad(0) status
as well as a text string describing the comparison.

Though the messages do not need to be in any specific format, for comparison
data to print out nicely in shoc_diff.thpl output, it is recommended that
comparisons are formatted like so:

[key1:data1;key2;data2;...keyN:dataN]{fieldx} cs1_value1 (==|!=|=~|!~) cs2_value1

Where key[1..N] are the primary keys of the ComponentState, fieldx is the
ComponentState field that we are comparing, and cs1_value1 and cs2_value1 are the 
values of the first and second ComponentStates respectively.

In the case where you are comparing multiple fields at once (like in Shoc::CompareSpec::cmp_lifs())
the format should be:

[key1:data1;key2;data2;...keyN:dataN]{field1;field2;..fieldY} cs1_value1;cs1_value2;..cs1_valueY (==|!=|=~|!~) cs2_value1;cs2_value2;...cs2_valueY

=head1 See Also

See these modules and scripts for further detail.

tools/shoc_diff.thpl
tools/shoc_save.thpl

lib/Shoc.pm
lib/Shoc/CompareResult.pm
lib/Shoc/CompareSpec.pm
lib/Shoc/CompareSpecTable.pm
lib/DataStructureStore.pm

=cut

package Shoc::CompareResult;

use Tharn;

# -----------------------------------------------
# Function: new
#   constructor
#
# Usage:
#   Shoc::CompareResult->new( status => 0, message => "field x \"true\" != \"false\"")
#
# Parameters:
#       status    - Boolean, true(1)/false(0)
#       message   - string,  information regarding the compare
#
# Return: blessed reference
# -----------------------------------------------
sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;

    my $self     = {
        status  => 0,
        message => "",
        @_,
    };
    bless($self, $class);
}

1;
