#!/usr/bin/perl
=head1 NAME

SnapFilePath - Generate alternative paths via .snapshot folders.

=head1 SYNOPSIS
use SnapFilePath;

my $s = new SnapFilePath(file_path => "/usr/path/to/snap/in/the/vol/file",
                         vol_path => "/usr/path/to/snap",
                         snapshot_name => "hourly.2009-03-30");
print Dumper($s->getPathList());
print Dumper($s->getRandomPath());
=head1 DESCRIPTION

=cut 


package SnapFilePath;
use Data::Dumper;

sub new {
    my $proto = shift;
    my $class = ref($proto) || $proto;
    my (%opts) = @_;
    my $self = {};

    if ($opts{file_path}) {
        $self->{file_path} = $opts{file_path};
    }
    if ($opts{vol_path}) {
        $self->{vol_path} = $opts{vol_path};
    }
    if ($opts{snapshot_name}) {
        $self->{snapshot_name} = $opts{snapshot_name};
    }
    _generatePathList($self);

    bless($self, $class);
    return $self;
     
}

sub _generatePathList {
    my $self = shift;

    $self->{file_path};
    $self->{vol_path};
    my @pathList;

    if ($self->{file_path} =~ m#^($self->{vol_path})(/)*(.*)#) {
        my ($scrub_base,$scrub_file) = ($1,$3);
        #print "$scrub_file\n";
        my @file_split = split '/', $scrub_file;

        #print Dumper(\@file_split);

        for (my $i = 0; $i <= $#file_split; $i++) {
            #print "pre: ".Dumper(@file_split[0..$i]);
            #print "post: ".Dumper(@file_split[$i..$#file_split]);
            my @pre = @file_split[0..$i];
            my @post = @file_split[$i..$#file_split];

            pop @pre;
            
            my $path = join ('/',($scrub_base,@pre,".snapshot",$self->{snapshot_name},@post));
            #print "Whole: $path\n"; 
            push @pathList,"/$path";
        }
        
    }
    $self->{path_list} = \@pathList;
}

sub getPathList {
    my $self = shift;
    return @{$self->{path_list}};
}

sub getRandomPath {
    my $self = shift;
    my @list = @{$self->{path_list}};

    return $list[rand($#list+1)];
}


1;
