# Hey emacs, this is basically -*- perl -*-
#
#Copyright (c) 2007 Network Appliance, Inc.
#All rights reserved.
# $Id: $
#
## @summary  Snaplock Helper Library Library
##
## @description
##
##
## These functions help in verifying the WORM properties of WORM Volumes and WORM Files 
##
##
## @usage
##    use SnapLock::Helper;
##
## @keywords snaplock worm WORM  
##
## @dependencies
##    lib/NOTE
##
## @author dl-snaplock-qa (DRG QA)
##
## @status review
##
##

=head1 NAME

Snaplock Functionality Helper  Module

=head1 SYNOPSIS
    use SnapLock::Helper;

=head1 DESCRIPTION
    Subroutines that will help to create/verify the snaplock functionality 

=head1 ROUTINES
read_configuration_file
create_worm_files
create_expired_worm_files
worm_files_delete
expired_worm_files_delete
verify_worm_aggregate_online
verify_worm_aggregate_destroy
verify_worm_volume_destroy
worm_volume_rename
worm_files_rename
worm_volume_options_modify
wait_till_expiry
=cut

package SnapLock::Helper;
use strict;
use base qw(Exporter);
use Cwd qw(getcwd);
use Carp qw(croak);
use Tharn;
use boot;
use filer;
use SPARSE::Util::Logger qw( logStep logError logWarn logDbg);
use NOTE;
use Volume::Copy qw(volcopy_start volcopy_status);
use Dense::Utils qw(dense_parse_dumpblock);
use Data::Dumper;
use SnapLock::Snaplock_constants;
use Date::Manip;
use Time::ParseDate;
use Time::Local;
use snaplock;	

BEGIN
{
  our @EXPORT = qw(
    read_version_matrix
    read_configuration_file
    create_worm_files
    create_expired_worm_files
    worm_files_delete
    verify_worm_aggregate_online
    verify_worm_aggregate_destroy
    verify_worm_volume_destroy
    verify_nfs_verifier_flag_on_inode
    worm_volume_rename
    worm_files_rename
    worm_volume_options_modify
    wait_till_expiry
    setup_vsm
    setup_volcopy
    setup_volclone
    setup_qsm
    setup_sv
    setup_ndmpcopy
    clone_split
    generate_qt_name   
    create_src_dst_qt
    set_lv_relation
    is_dst_qt_files_worm	
    qt_worm_data_compare	
    verify_sv_start_log_file_info	
  );
}

sub read_version_matrix($)
{
    my ($infile) = @_;

    tstlog ('-comment', "Inputfile is : $infile");
    open(INFILE,$infile) || die ("Couldn't open file!!");

    my @data = <INFILE>;
    close(INFILE);
    my %input;
    my $entry_count=1;

    ### Read each line from the version matrix file and store the values in a hash
    ### Return the hash
    ### release   ( AS.7/AS.sl/AS.spare/AS.fs )
    ### version   ( /x/eng/build2/scratch/milena/milena_testgb_696591/ | /x/eng/build9/scratch/milena/milena_testic_737292)
    ### original_fs_version  ( 57/72)
    ### containment_fs_version ( 4096/4112 )
    ### slc_containment_version   ( 0/1/2 )
    ### sle_containment_version   ( 0/1/2 )
    ### spare1_containment_version   ( 0/1/2 )
    ### spare2_containment_version   ( 0/1/2 )

    foreach my $line (@data){
        chop($line);
        if($line =~/^#/ || $line eq ''  ) {
            #skip the comments & blank lines
        }
        ##Process the valid entry
        else {
            my @entries = split(/\t+/,$line);
            if( $#entries >= 7) {
                my %entry = (
                             "version"=>$entries[0],
                             "release"=>$entries[1],
                             "original_fs_version"=>$entries[2],
                             "containment_fs_version"=>$entries[3],
                             "slc_containment_version"=>chop($entries[4]),
                             "sle_containment_version"=>chop($entries[5]),
                             "spare1_containment_version"=>chop($entries[6]),
                             "spare2_containment_version"=>chomp($entries[7]),
                            );
                $input{$entry_count}=\%entry;
                $entry_count++;
            }
        }
    }
    ## Print the Hash Values
    if($::DEBUG==1)
    {
        foreach my $key (keys %input) {
            my $inhash = $input{$key};
            foreach my $inkey (keys %$inhash ){
                tstlog ('-comment', "Hash : $key ->Value : $inkey : $inhash->{$inkey}");
                tstlog('-comment',"Inside special case shud appear only once")
            }
        }
    }
    return \%input;

}


=head2 read_configuration_file
    my $input_file = /u/kasasrik/config.dat
    my $hash_reference = read_configuration_file($input_file);

This routine reads the configuration file and stores the values into hash

hash{100}->{"from_release"}
hash{100}->{"to_release"}
hash{100}->{"from_release_status"}
hash{100}->{"to_release_status"}
hash{100}->{"next_good_release"}

returns hash reference 


=over

=item input_file

(Required) : /u/kasasrik/config.dat

=back

=for html <hr>

=cut



## @name ::read_configuration_file
##
## @description
##  get the entries from the configuration file and return the nested hash
##  Hash contains the following members
##  Inputfile{100}->{"from_release"}
##  Inputfile{100}->{"to_release"}
##  Inputfile{100}->{"from_release_status"}
##  Inputfile{100}->{"to_release_status"}
##  Inputfile{100}->{"next_good_release"}
##
## @synopsis
##  read_configuration_file($)
##
## @example
##  my $input_hash = read_configuration_file($)
##

sub  read_configuration_file($)
{
    my ($infile) = @_;

    tstlog ('-comment', "Inputfile is : $infile");
    open(INFILE,$infile) || die ("Couldn't open file!!");

    my @data = <INFILE>;
    close(INFILE);
    my %input;
    my $entry_count=100;

    ### Read each line from the configuration file and store the values in a hash
    ### Return the hash 
    ### from_release   ( 7.3 or RironcityN )
    ### to_release     ( 7.1 or RtsingtaoN)
    ### to_release_status ( Good/Bad)
    ### from_release_status ( Good/Bad)
    ### next_good_release   ( 7.3 or RironcityN )
    ### 

    foreach my $line (@data){
        chop($line);
        if($line =~/^#/ || $line eq ''  ) {
            #skip the comments & blank lines
        }
        ##Process the valid entry
        else {
            my @entries = split(/\t/,$line);
            if( $#entries >= 3) {
                my %entry = (
                             "from_release"=>$entries[0],
                             "to_release"=>$entries[1],
                             "from_release_status"=>$entries[2],
                             "to_release_status"=>$entries[3],
			     "reg_vol_status"=>$entries[4],
                             "next_good_release"=>$entries[5],
			     "is_cleanup_required"=>$entries[6],
                            );
                $input{$entry_count}=\%entry;
                $entry_count++;
            }
        }
    }

    ## Print the Hash Values
    if($::DEBUG==1)
    {
        foreach my $key (keys %input) {
            my $inhash = $input{$key};
            foreach my $inkey (keys %$inhash ){
                tstlog ('-comment', "Hash : $key ->Value : $inkey : $inhash->{$inkey}");
                tstlog('-comment',"Inside special case shud appear only once") 
                if(exists $inhash->{next_good_release} && $inhash->{next_good_release} ne '' ) ;
            }
        }
    } 
    return \%input;

}


=head2 create_worm_files
    my $array_of_file_objects = create_worm_files($object,$count,$retention_period);

This routine uses NOTE library routine and 
creates WORM files with the Specified retention period

returns array reference containing file objects


=over

=item root_object

(Required) : Volume or Qtree Note Object

=item count

(Required) : Number of WORM files to create 

=item retention_period

(Required) : WORM file retention period (3600s/1h/1d/1m/1y)

=back

=for html <hr>

=cut 

##
## @ name :: create_worm_files
## @ Arguments - NOTE container Object ( volume or qtree object )
## @             Count - Number of WORM files to be created
## @             Retention - Retention Period ( 10s,1h,1d)
##   
## @ Create WORM Files with Specific Retention
##

sub create_worm_files($$$;$)
{
    my ($root,$count,$retention,$size) = @_;
    my $file_size;
    if (!defined $size ) {
        $file_size = "1m";
    } else {
        $file_size = $size;
    }
    NOTE::logcomment("Inside Creating WORM Files");
    my @files;
    while ($count) {
        my $file = $root->nfs
                        ->create
                        ->fill(max=>$file_size)
                        ->commit(retention =>"$retention", force=>1);
        push(@files,$file);
        $count--;
    }
    return \@files;
}


=head2 create_expired_worm_files
    my $array_of_file_objects = create_expired_worm_files($object,$count);

This routine uses NOTE library routine and 
creates WORM files with 5 seconds retention time
wait until the files gets expired

returns array reference containing file objects


=over

=item root_object

(Required) : Volume or Qtree Note Object

=item count

(Required) : Number of Expired WORM files to create 


=back

=for html <hr>

=cut



##
##
## @ name ::create_expired_worm_files
## @ Arguments - Note Container Object ( volume or qtree object)
## @             Count - Number of WORM Files to be created
##
## @ Create WORM Files and wait until the Files get Expired 
##
##

sub create_expired_worm_files($$)
{
    my ($root,$count) = @_;

    my @files;
    while ($count) {
        my $file = $root->nfs
                        ->create
                        ->fill(max=>"1m")
                        ->commit(retention =>"5s");
        push(@files,$file);
        $count--;
    }
    wait_till_expiry(\@files);
    return \@files
}

##
## @ name ::wait_till_expiry
## @ Argument : array reference of file objects 
## @ Description : waits until the worm file expiration and returns
##
sub wait_till_expiry($)
{
    my ( $files ) = @_;
    foreach my $file (@$files){
        $file->expire;
    }
}



=head2 verify_worm_aggregate_online
    verify_worm_aggregate_online($aggregate_object);

This routine uses NOTE library routine and 
tries to online the Aggregate for that volume 

returns 1 if aggregate can not be brought be online
dies if the aggregate becomes online 


=over

=item volume_object

(Required) : Volume Note Object


=back

=for html <hr>

=cut


##
## @ name ::
## @ Verifies whether the WORM Volume/Aggregate can be brought online on a Bad release
## @ verifies whether the error message is appropriate
## @ returns success if volume/aggregate can't be made online
## @ croaks if the volume/aggregate can be brought online
##
##
sub verify_worm_aggregate_online($)
{
    my ($vol) = @_;
    if ( $vol->snaplock eq 'compliance' || $vol->snaplock eq 'enterprise') {

        ## Make the Aggregate online
        eval { 
            my $aggr ;
		if( $vol->type ne 'trad' ) {
			$aggr = $vol->Aggregate;
            		$aggr->online; 
		} else {
			$aggr = $vol;
			$aggr->online;
		}
        };
        
        ## Check for the Error message and pass/fail 
        if($@ =~ /ONTAP does not recognize the filesystem/) {
            NOTE::logcomment("PASS : As expected failed to online the Snaplock Compliance aggregate \n\n $@");
            return 1;
        }
	elsif ( $@ =~ /Install a version of Data ONTAP that is supported with SnapLock/) {
                NOTE::logcomment("PASS : As expected failed to online the Snaplock Compliance aggregate \n\n $@");
                    return 1;
        }else {
            croak "FAILED : Able to make the Snaplock Compliance Volume online on a filer with previous release ONTAP";
        }
    }
}




=head2 verify_worm_aggregate_destroy
    verify_worm_aggregate_online($aggregate_object);

This routine uses NOTE library routine and 
tries to destroy the Aggregate for that volume 

returns 1 if aggregate can not be destroyed
dies if the aggregate is destroyed


=over

=item volume_object

(Required) : Volume Note Object


=back

=for html <hr>

=cut 

##
## @ name :: 
##
##

sub verify_worm_aggregate_destroy($)
{
    my ($vol) = @_;
    if ( $vol->snaplock eq 'compliance') {
        eval { 
            my $aggr = $vol->Aggregate;
            $aggr->destroy; 
        };
        if($@ =~ /it contains unexpired WORM files/ ) {
            NOTE::logcomment("PASS : As expected failed to destroy the Snaplock Compliance Aggregate \n\n $@");
            return 1;
        } else {
            NOTE::logcomment("FAILED : Able to destroy the Snaplock Compliance volume online on a Filer".
                             "with Previous release ONTAP  ");
            croak "FAILED : Able to destroy the Snaplock Compliance Aggregate on a filer with previous release ONTAP";
        }
    }
}


=head2 verify_worm_volume_destroy
    verify_worm_volume_destroy($volume_object);

This routine uses NOTE library routine and 
tries to destroy the volume 

returns 1 if volume can not be destroyed
dies if the volume is destroyed


=over

=item volume_object

(Required) : Volume Note Object


=back

=for html <hr>

=cut 

sub verify_worm_volume_destroy($)
{
    my ($vol) = @_;
    if ( $vol->snaplock eq 'compliance') {
        eval { 
            $vol->purge;
        };
        if($@ =~ /it contains unexpired WORM files/ ) {
            NOTE::logcomment("PASS : As expected failed to destroy the Snaplock Compliance volume \n\n $@");
            eval { $vol->online;};
            return 1;
        }
        else {
            NOTE::logcomment("FAILED : Able to destroy the Snaplock Compliance volume online on a Filer with Previous release ONTAP  ");
            croak "FAILED : Able to destroy the Snaplock Compliance Volume on a filer with previous release ONTAP";
        }
    }
}

=head2 worm_volume_rename
    worm_volume_rename($volume_object);

This routine uses NOTE library routine and 
tries to rename the volume 

returns 1 if Snaplock Compliance volume can not be destroyed
dies if the Snaplock Compliance volume can be renamed


=over

=item volume_object

(Required) : Volume Note Object


=back

=for html <hr>

=cut 

sub worm_volume_rename($)
{
    my ($vol) = @_;

    eval{$vol->rename;};
    my $vol_type = $vol->snaplock;
    if($vol_type eq 'compliance'){
        if($@ =~ /cannot be renamed because it is a SnapLock Compliance Volume/i){
            NOTE::logcomment("PASS : As expected failed to rename Snaplock Compliance volume \n\n $@");
        } else{ 
            croak $@;
        }
    }

}

=head2 worm_files_delete

    my %hash = ( file_name => "WORM File Name");
    $hash{file_name}{obj}= $file_object;

    worm_files_delete(\%$hash);

This routine uses NOTE library routine and 
tries to delete the worm files which are not expired  

returns 1 if WORM files can't be deleted
dies if the worm file can be deleted

=over

=item hash

(Required) : hash with file objects

=back

=for html <hr>

=cut 

sub worm_files_delete
{
    my ($files) = @_;

    foreach my $file_name (keys %$files) {
      my $file = $files->{$file_name}->{obj};
       my $path = $file->path;
       eval { $file->rm; } ;

       if($@) {
           NOTE::logcomment("PASS : As expected not able to delete the WORM Files \n\n $@ ");
           return 1;
       } else {
           croak "FAILED : Able to delete WORM Files : $@";
       }
    }
}



=head2 worm_files_rename

    my %hash = ( file_name => "WORM File Name");
    $hash{file_name}{obj}= $file_object;

    worm_files_rename(\%$hash);

This routine uses NOTE library routine and 
tries to rename the worm files 

returns 1 if WORM files can't be renamed
dies if the worm file can be renamed

=over

=item hash

(Required) : hash with file objects

=back

=for html <hr>

=cut 

sub worm_files_rename($)
{
    my ($files) = @_;

    foreach my $file_name (keys %$files) {
        my $file = $files->{$file_name}->{obj};
        my $path = $file->path;
        my $vol = $file->Volume;
        eval {$vol->nfs->rename($path,"$path.name"); };
        my $msg = $@;
        if ($msg =~ "Read-only file") {
            NOTE::logcomment("PASS, As expected not able to rename the WORM Files \n\n $msg ");
        } else{
            NOTE::logcomment("FAILED : Able to Rename WORM Files look into it : $msg");
            croak "Failed, Able to rename WORM files - look into it";
        }
     }
}


=head2 worm_volume_options_modify

    worm_volume_options_modify($volume_object)

This routine uses NOTE library routine and 
modifies the WORM Volume options
snaplock_default_period
snaplock_maximum_period
snaplock_minimum_period

returns 1 if options can be modified 
dies incase of failure

resets back the original values after the test

=over

=item volume_object

(Required) : Note Volume Object

=back

=for html <hr>

=cut 


##
## @ name :: worm_volume_options_modify
## @ Argument :: NOTE Volume Object
## @ Modifies the snaplock_default_period
## @              snaplock_minimum_period
## @              snaplock_maximum_period 
## @          to 20s 
##

sub worm_volume_options_modify
{
    my ($vol) = @_;

    NOTE::logcomment("Trying to Modify the Volume options");
    if($vol->snaplock) {
        my $default = $vol->options("snaplock_default_period");
        $vol->options("snaplock_default_period"=>"max");
         
        my $min = $vol->options("snaplock_minimum_period");
        $vol->options("snaplock_minimum_period"=>"0y");

        my $max = $vol->options("snaplock_maximum_period");
        $vol->options("snaplock_maximum_period"=>"20s");
        NOTE::logcomment("Modified the Volume options");

        ## Reset back to the old values
        $vol->options("snaplock_maximum_period"=>$max);
        $vol->options("snaplock_minimum_period"=>$min);
        $vol->options("snaplock_default_period"=>$default);


    }
}

##
## @name :: verify_nfs_verifier_flag_on_inode
## @Argument :: filer connection object
## @Argument :: volume name
## @Argument :: Array of inode numbers present in the volume
## @returns  :: Returns 0 if NFS_VERIFIER_FLAG doesn't present on inode.
##		Returns count if NFS_VERIFIER_FLAG present on inode.

sub verify_nfs_verifier_flag_on_inode($$$) {
    my ($filer_conn, $vol , $inums ) = @_;
 
    ## Flush all the data to the disk. 
    my $cmd = "priv set test; wafl flush cache";
    my ( undef, undef , $out ) = say($filer_conn , $cmd);
    sleep 20;
 
    my $count = 0; 
    foreach my $inum (@$inums) {
	my $ishow = Dense::Utils::dense_parse_dumpblock(conn => $filer_conn , vol => $vol , inum => $inum );
	my $disk_flag = $ishow->{dflags};
	logcomment("Disk flag = $disk_flag");
	if ( ( hex $ishow->{dflags} == hex NFS_VERIFIER_ON_WORM ) || ( hex $ishow->{dflags} == hex NFS_VERIFIER_ON_REGULAR_FILE ) || ( hex $ishow->{dflags} == hex NFS_VERIFIER_ON_WORM_APPEND) ) {
		$count++;
	}
    }
    if ( ( scalar @$inums ) == $count )  {
	return $count;
    }else {
	return 0;
    }
}

=head2 ::setup_vsm

=cut

sub setup_vsm($$) {
    my ( $vsm_src, $vsm_dst ) = @_;

    my $sm;
    eval {

       $sm = NOTE::SnapMirror->initialize(
                    src => $vsm_src,
                    dst => $vsm_dst,
                );
    };
    if ( $@ ) {
            NOTE::logcomment("Snapmirror Initialize Failed\n");
            return $@;
    } else {
            return $sm;
    }
}

=head2 ::setup_qsm

=cut

sub setup_qsm($$) {
    my ( $srcvol, $dstvol ) = @_;

    my ($name,%qt, $src,$dst);
    my $set = "SET1";

    ##  @step Create qtree on soruce volume
    my $qt_src = NOTE::Qtree->create( 
                        Volume => $srcvol,
    );

    # Destination qtree object definition,
    # the qtree is not actually created on the filer
    # since it will be created with snapmirror
    my $qt_dst = NOTE::Qtree->new(
                        Volume => $dstvol,
    );

    my $sm;
    eval {
       $sm =  NOTE::SnapMirror->initialize(
                        src => $qt_src,
                        dst => $qt_dst,
            );
   };
   if ( !$sm ) {
        NOTE::logcomment("Snapmirror Initialize Failed\n");
        return $@;
    } else {
        return $sm;
    }
}

=head2 ::setup_volcopy

=cut
sub setup_volcopy($$) {
    my ( $volcopy_src, $volcopy_dst ) = @_;

    my $vc;
    eval {
        $vc = NOTE::Volume::Copy->start(
		src => $volcopy_src,
		dst => $volcopy_dst,
        );
    };
    if ( !$vc ) {
	NOTE::logcomment("Vol copy start Failed : $@\n");
	return ;
    }
    sleep 10;
    return $vc;
}

sub setup_volclone($$) {

    my ( $vol, $filer ) = @_;

    if ( $vol->type eq 'flex') {

        my ($vol_clone_name, $cmd, $result, $vol_clone, $clone_data);
        $vol_clone_name = "clone_".$vol->name;
        $cmd = "vol clone create ".$vol_clone_name." -b ".$vol->name;
        $filer->license('flex_clone');
        $result = $filer->say($cmd);

        if($result =~ /The source volume is a strict worm volume/i || $result =~ /The source volume is a SnapLock Compliance volume/i) {
            $cmd = "vol clone create ".$vol_clone_name." -f -b ".$vol->name;
	    $filer->say($cmd);
        }
        $vol_clone = NOTE::Volume->attach(Filer => $filer,
				name => $vol_clone_name, );
	return $vol_clone;
   }
   return ;

}

sub clone_split($$) {

    my ( $clone_vol , $filer ) = @_;

    my $cmd = "vol clone split start ".$clone_vol->name;
    eval {
        $filer->say($cmd);
    };

}

sub setup_sv($$$) {

    my ( $srcvol , $dstvol , $logvol ) = @_;

    ##  @step Create qtree on soruce volume
    my $qt_src = NOTE::Qtree->create( 
                        Volume => $srcvol,
    );
    # Destination qtree object definition,
    # the qtree is not actually created on the filer
    # since it will be created with snapmirror
    my $qt_dst = NOTE::Qtree->new(
                        Volume => $dstvol,
    );
    my $lv;
    my %opts;
    $opts{src} = $qt_src;
    $opts{dst} = $qt_dst;
    $opts{log} = $logvol;
    $opts{snaplock_minimum_period} = "300s";
    ## Setup compliance license
    if ( $logvol->snaplock eq 'compliance' ) {
        $logvol->Filer->license('snaplock');
    }elsif ( $logvol->snaplock eq 'enterprise' ) {
        $logvol->Filer->license('snaplock_enterprise');
    }
        
    eval {
    	$lv = NOTE::LockVault->start(%opts);
    };
    if ( !$lv ) {
        NOTE::logcomment("Snapvault start Failed\n");
        return $@;
    }
    return $lv;
}



sub set_lv_relation{
        my ($src_qt_obj,$dst_qt_obj,$log_vol_obj) = @_;

        my $log_vol_name = $log_vol_obj->name;
        my $cmnd = 'options snapvault.lockvault_log_volume '
                   . $log_vol_name ;
        $log_vol_obj->Filer->say ($cmnd);
        my $lv = eval { NOTE::SnapVault->start(
                        src => $src_qt_obj,
                        dst => $dst_qt_obj,
                        snaplock_maximum_period => "30s",
                      );
                  };

        $lv = NOTE::LockVault->attach(
                         src=>$src_qt_obj,
                         dst=>$dst_qt_obj,
                         log=>$log_vol_obj
                     );
        return $lv;
}


sub create_src_dst_qt {

 my ( $src_vol_obj, $dst_vol_obj,$qt_name) = @_;
 my %qt ;

        $qt{'src_qt_obj'} = NOTE::Qtree->create (
                                    Volume => $src_vol_obj,
                                    name => $qt_name,
                               );
        $qt{'dst_qt_obj'} = NOTE::Qtree->new(
                                Volume => $dst_vol_obj,
                                name => $qt_name,
                                #tag    => 'destination'
                           );
      return \%qt;
}

sub generate_qt_name {
       my ($src_vol_type,$dst_vol_type,$run_id) = @_;
       my $qt_name = $src_vol_type. '_' . $dst_vol_type . '_' . $run_id;
       return $qt_name;
}

sub qt_worm_data_compare {
        my ($src_qt_obj,$restore_qt_obj) = @_;

        my @src_qt_file_objs = $src_qt_obj->nfs->children;
        my @restore_qt_file_objs = $restore_qt_obj->nfs->children;
        my $count = 0;
        foreach my $src_qt_file_obj (@src_qt_file_objs){
                if ( $src_qt_file_obj->name !~ /snapshot/){
                        my $src_qt_file_name = $src_qt_file_obj->name ;
                        logcomment ("File $src_qt_file_name Exists in SRC QT, This to be Tested");
                        foreach my $restore_qt_file_obj (@restore_qt_file_objs){
                                if ( $restore_qt_file_obj->name !~ /snapshot/){
                                        if ( $src_qt_file_obj->name eq $restore_qt_file_obj->name &&
                                             $src_qt_file_obj->atime == $restore_qt_file_obj->atime){
                                                $count++;
                                                my $restore_qt_file_name = $restore_qt_file_obj->name;
                                                logcomment ("File $restore_qt_file_name Exists in Restore QT");
                                                my $src_atime = $src_qt_file_obj->atime;
                                                my $restore_atime = $restore_qt_file_obj->atime;
                                                logcomment ("Src_File_Atime = $src_atime Restore_File_Atime = $restore_atime");
                                        }
                                }

                        }
                }

        }
        my $file_count = @src_qt_file_objs - 1; #Excluding .Snapshot
        if ($count == $file_count){
                return 1;
        }else{
                return 0;
        }
}

sub is_dst_qt_files_worm {

        my ($hash_ref) = @_;
        my %param = %{$hash_ref};

        my ( $src_qt_obj , $dst_vol_obj , $dst_qt_name )
                = (
                        $param{src_qt_obj} ,
                        $param{dst_vol_obj} ,
                        $param{dst_qt_name} ,
                  );

        my @src_qt_file_objs = $src_qt_obj->nfs->children;
        my @src_qt_file_names ;
        foreach my $file_obj (@src_qt_file_objs){
                my $file_name = $file_obj->name;
                if ($file_name !~ /snapshot/){
                        push ( @src_qt_file_names , $file_name );
                }
        }

        my @dst_vol_childs = $dst_vol_obj->nfs->children;
        my $qt_dir_obj;
        foreach my $file_obj (@dst_vol_childs){
                my $file_name = $file_obj->name ;
                if ($file_name =~ /$dst_qt_name/){
                        $qt_dir_obj = $file_obj;
                        last;
                }
        }

        my @ls_dir = $qt_dir_obj->ls('-l');
        logcomment ("ls -l o/p of Qtree Directory used in Snapvault Destination");
        logcomment (@ls_dir);
        my $worm_files_count = 0;

        foreach my $file_name (@src_qt_file_names){
                foreach my $line (@ls_dir){
                        if ($line =~ /$file_name/){
                                logcomment ("In DST QT File $file_name Found");
                                my @words = split '\s+', $line;
                                if ($words[0] !~ /w|x/){
                                        logcomment ("In DST QT File $file_name is WORM");
                                        logcomment ("ls OutPut  # $line");
                                        $worm_files_count++;
                                }else {
                                        logcomment ("In DST QT File $file_name is NOT set to WORM");
                                        logcomment ("ls OutPut  # $line");
                                }

                        }else {
                                logcomment ("In DST QT File $file_name NOT Found");
                        }
                }
         }

        if ( $worm_files_count == @src_qt_file_names){
                return 1;
        }else{
                return 0;
        }
}

sub verify_sv_start_log_file_info {

        my ($src_qt_obj, $dst_qt_obj, $log_vol_obj ) = @_;

        my $src_vol_obj = $src_qt_obj->Volume;
        my $dst_vol_obj = $dst_qt_obj->Volume;
        #my $src_vcc = snaplock_get_system_compliance_clock ($src_vol_obj->Filer->conn);
        #my $src_vcc = $src_vol_obj->compliance_clock; 
	my $src_vcc = get_filer_epoc_time ($src_vol_obj->Filer);
	my $dst_vcc = get_filer_epoc_time ($dst_vol_obj->Filer);
        #my $dst_vcc = $dst_vol_obj->compliance_clock;
        my @src_qt_file_objs = $src_qt_obj->nfs->children;
        my @src_qt_file_names ;

        foreach my $file_obj (@src_qt_file_objs){
                if ($file_obj->name !~ /snapshot/){
                        my $file_name = $file_obj->name;
                        push (@src_qt_file_names,$file_name);
                }

        }

        my %sv_log_info_hash = get_sv_log_file_info ($log_vol_obj);

        my $sv_start_time = $sv_log_info_hash{start_time};
        $sv_start_time = &UnixDate($sv_start_time, "%s");
        my $sv_src_time_stamp = $sv_log_info_hash{source_snapshot_timestamp};
        $sv_src_time_stamp = &UnixDate($sv_src_time_stamp, "%s");
        my $ref_files = $sv_log_info_hash{files_contained};
        my @files_in_sv_log = @{$ref_files};

        logcomment ("Files in the SRC QT");
        logcomment (@src_qt_file_names);
        logcomment ("Files in the SV LOG File");
        logcomment (@files_in_sv_log);
        logcomment ("sv_start_time # $sv_start_time  **** DST_VCC # $dst_vcc");
        logcomment ("sv_src_time_stamp # $sv_src_time_stamp *** SRC_VCC $src_vcc");

        my $pass_count = 0;

        if (abs ($src_vcc - $sv_src_time_stamp) < 60){
                $pass_count++;
        }else {
                logcomment ("DEBUG # SV_LOG_TEST_FUNC # FAILED #sv_src_time_stamp and SRC_VCC Diff FAILED");
        }

        if (abs ($dst_vcc - $sv_start_time ) < 60){
                $pass_count++;
        }else {
                logcomment ("DEBUG # SV_LOG_TEST_FUNC # FAILED #sv_start_time  and DST_VCC  Diff FAILED");
        }

        my $common_file_count = 0;

        foreach my $src_file (@src_qt_file_names) {
                if (grep $src_file eq $_, @files_in_sv_log) {
                        $common_file_count ++;
                }
        }
        my $src_qt_file_count = @src_qt_file_names;

        if ($src_qt_file_count == $common_file_count ){
                $pass_count++;
        }else{
                logcomment ("DEBUG # SV_LOG_TEST_FUNC # FAILED # All files of SRC QT Not In SV LOG FILE ");
        }

        if ($pass_count == 3){
                return 1;
        }else{
                return 0;
        }

}


sub get_sv_log_file_info {
        my ($vol_obj) = @_;
        my $sv_log_file_obj;

                my @file_objs = $vol_obj->nfs->children;
                foreach my $file_objL_1 (@file_objs ){
                        if ($file_objL_1->name =~ /etc/){
                                logcomment ("SV_DEBUG_1 # in etc Dir");
                                my @file_objs = $file_objL_1->children;
                                foreach my $file_objL_2 (@file_objs ){
                                        if ($file_objL_2->name =~ /log/){
                                                logcomment ("SV_DEBUG_2 # in log Dir");
                                                 my @file_objs = $file_objL_2->children;
                                                 foreach my $file_objL_3(@file_objs ){
                                                        if ($file_objL_3->is_dir){
                                                                my $dir_name = $file_objL_3->name;
                                                                logcomment ("SV_DEBUG_3 # in $dir_name Dir");
                                                                $sv_log_file_obj = recure_get_sv_log_file_info ($file_objL_3);
                                                                last;
                                                        }
                                                 }
                                        }
                                }
                        }
                }

                if ($sv_log_file_obj){
                        my $name = $sv_log_file_obj->name;
                        my $path = $sv_log_file_obj->fullpath;
                        logcomment ("In get_sv_log_file_info SV_Log_File Name # $name  path # $path");
                        my $MY_SV_LOG_FL;
                        open ($MY_SV_LOG_FL , "< $path");
                        my @lines ;
                        while (<$MY_SV_LOG_FL>){
                                chomp ;
                                push (@lines , $_);
                        };
                        logcomment (@lines);
                        close $MY_SV_LOG_FL;
                        my %sv_hash = convert_sv_log_file_to_hash (\@lines , $name );
                        return %sv_hash ;
                }else {
                        die 'Unable to find Sv_log File';
                }

}

sub convert_sv_log_file_to_hash {

        my ($ref_lines , $name ) = @_ ;
        my @lines = @{$ref_lines};
        my %sv_log_hash ;
        my @file_names;
        foreach my $line (@lines){

                if ($line =~ /Transfer type:(.*)/){
                        $sv_log_hash{'transfer_type'} =  $1;
                }elsif ( $line =~ /From:(.*)/){
                        $sv_log_hash{'sv_from'} = $1;
                }elsif ( $line =~ /To:(.*)/){
                        $sv_log_hash{'sv_to'} = $1;
                }elsif ( $line =~ /Start time:(.*)/){
                        $sv_log_hash{'start_time'} = $1;
                }elsif ( $line =~ /Source snapshot timestamp:(.*)/){
                        $sv_log_hash{'source_snapshot_timestamp'} = $1;
                }elsif ($line =~ /Create File/){
                        $line =~ /\.\/(.*)/;
                        my $file_name = $1;
                        $file_name  =~ s/^\s+//; $file_name  =~ s/\s+$//;
                        push (@file_names, $file_name);
                }
        }
        foreach my $key (keys (%sv_log_hash)){
                my $str = $sv_log_hash{$key};
                $str =~ s/^\s+//;
                $str =~ s/\s+$//;
                $sv_log_hash{$key} = $str;
        }
        $sv_log_hash{'sv_log_file_name'} = $name;
        $sv_log_hash{'files_contained'} = \@file_names;
        print Dumper (\%sv_log_hash);
        return %sv_log_hash ;
}


sub recure_get_sv_log_file_info {
        my ($child) = @_;

        logcomment ("SV_REC # IN recure_get_sv_log_file_info FUNC ");
        my @file_objs = $child->children;
                foreach my $child (@file_objs){
                        if ($child->is_dir && $child->name !~ /snapshot/){
                                my $dir_name = $child->name ;
                                logcomment ("DEBUG # In recure_get_sv_log_file_info # in Dir $dir_name");
                                my $sv_log_obj =  recure_get_sv_log_file_info($child);
                                return $sv_log_obj;
                        }else {
                                my $logfile_name = $child->name ;
                                logcomment ("DEBUG # In recure_get_sv_log_file_info # SV File Name $logfile_name ");
                                return $child;
                        }
                }
}

sub get_filer_epoc_time {
	my ($filer_obj) = @_;
	my $filer_time = $filer_obj->date;
	$filer_time =  &UnixDate($filer_time, "%s");
	return $filer_time;

}

1;


