# -*- perl -*-
#
# $Id: //depot/prod/test/main/lib/SnapLock/Raid/Constants.pm#1 $
#
## @summary Constants for SnapLock Raid functions
##
## @author sureshb@netapp.com 
## @status Private 
## @pod here

package SnapLock::Raid::Constants;

use strict;
use NATE;
use Tharn;

# Constant definitions
# required for the various snaplock raid functionality tests
use constant TRUE => 1;
use constant FALSE => 0;

# basic constants being abstracted here
#
# license
use constant SNAPLOCK_COMPLIANCE => "snaplock";
use constant SNAPLOCK_ENTERPRISE => "snaplock_enterprise";

## volume types
use constant COMPLIANCE => "compliance";
use constant ENTERPRISE => "enterprise";
use constant REGULAR => "non-sl";

## disk label types
use constant VERSION1 => "v1";
use constant VERSION2 => "v2";
use constant BOTH => "v1v2";
use constant SPARES => "spare";

## for DD command
use constant DD_COUNT => 1;

# other miscellaneous constants
use constant FILER_MODE => "MAINT";
use constant CONNTYPE => "#console";
use constant MAINT => "MAINT";

use constant SNAPLOCK_TYPE_BAD_CORRUPT_DISK => COMPLIANCE;
use constant SNAPLOCK_TYPE => COMPLIANCE;

# configurable variants
use constant AGGR_SLC_CORRUPT => "slc_corrupt";
use constant AGGR_BADVERSION => "slc_higher_ver";
use constant AGGR_NORMAL => "normal";
use constant AGGR_SLC => "slc";

##EDITABLE:
## basic aggr size
use constant RAID4_AGGR_TYPE => "raid4";
use constant RAID4_NUM_DISK => "2";
use constant RAID_DP_NUM_DISK => "3";
use constant AGGR_SLC_DISK_UNFAIL => "slc_disk_unfail";

use constant DEFAULT_AGGR_TYPE => RAID4_AGGR_TYPE;
use constant DEFAULT_AGGR_NUM_DISK => RAID4_NUM_DISK;

#used for label buildtree
use constant DEFAULT_AGGR_BUILDTREE => "buildtree";
use constant DEFAULT_AGGR_WORM_TEMP => "dflt_worm_aggr";
use constant DEFAULT_AGGR_NONWORM_TEMP => "dflt_nonworm_aggr";

##EDITABLE:
# this should match the number of disk required for corrupt
# refer to %CORRUPT_DISKS
use constant CORRUPT_DISK_RAIDGROUP_SIZE => "5";

##EDITABLE:
# this flag will indicate the type of worm aggregate that each script will work on
# this should be retained as a constant value for one complete run 
# possible values "SNAPLOCK_COMPLIANCE" and "SNAPLOCK_ENTERPRISE"
use constant WORM_AGGR_TYPE_UNDER_TEST => "SNAPLOCK_COMPLIANCE";

use constant WAIT_TIME_FOR_DISK_ZERO_COMPLETE => 0;

# ############################################
# currently not being used, for future use
# start here
# ############################################
our %SLC_CORRUPT_DISKS = 
(
	v1 => 1,
	v2 => 1,
	v1v2 => 3
);

our %NORMAL_CORRUPT_DISKS = 
(
	v1 => 1,
	v2 => 1,
	v1v2 => 2
);

our %SL_AGGR = (
	badversion => {
		raidtype => "raid4",
		num_disks => 2,
		name => "badversion",
		reuse => TRUE,
	},

	slc => {
		raidtype => "raid_dp",
		num_disks => 4,
		name => "slc",
		reuse => TRUE,
	},

	normal => {
		raidtype => "raid_dp",
		num_disks => 3,
		name => "normal",
		reuse => TRUE,
	},
);

# experimental to include all libraries in ds
my @includes = (
"use raid_label", 
"use SnapLock::Raid::Constants;",
"use LabelUtils;",
"use SnapLock::Raid::Utils;"
);

use constant INCLUDES => "@includes";
# ############################################
# end here
# ############################################

our %DISK_STATES =
(
	good => "good", 
	corrupt => "corrupted", 
	badversion => "bad_version", 
	regular => "non-sl"
);

# this type will let different disk types to be used by the scripts
# one or more disk types, will let the same test be run on various disk types
# caution: not completely tested
# use only one disk type for now
#
#our @DISK_TYPES = ("DATA", "DPARITY", "PARITY");
our @DISK_TYPES = ("DATA");

# this is list of array definitions for the different types of label fields 
# to be worked on
#
## library will iterate this array to corrupt all fields
our @LABEL_FIELDS_TO_CORRUPT = 
(
	["magic", "0x100"]
);

##EDITABLE:
## fields that will be updated to make disk label a bad-version (not-supported version)
## this field may need to be edited depending on the release that will higher version
## TODO: make this as a function to retrieve the range of allowable label-version and 
##       determine the higher/lower version to make it badversion

our %LABEL_FIELDS_TO_BADVERSION = (

	"7.1" => ["label-version", "8"],
	"7.2" => ["label-version", "9"],
	"7.3" => ["label-version", "10"],
        "8.0" => ["label-version", "11"],

);
##EDITABLE:
## fields that will be updated to make disk v1 label a bad-version (not-supported version)

our %LABEL_FIELDS_V1_TO_BADVERSION = (

	"7.1" => ["raid_version", "8"],
	"7.2" => ["raid_version", "9"],
	"7.3" => ["raid_version", "10"],
	"8.0" => ["raid_version", "11"],

);
# this is very important to keep in mind
# error/output message can be different for each release
# This DS gives all common output messages for all releases
# *do not* include any release specific output message here
our %MESSAGE = 
(

	LABEL_WIPEV1_CORRUPT_NONWORM => "",

	LABEL_COPY_BADVERSION_WORM_TO_GOOD_NONWORM => "Error: Cannot copy label because disk <disk> has a bad RAID version.",

	LABEL_COPY_BADVERSION_NONWORM_TO_GOOD_WORM => "Error: Cannot copy label because disk <disk> has a bad RAID version.",

	LABEL_COPY_CORRUPT_NONWORM_TO_GOOD_WORM => "is part of a SnapLock Compliance volume.",

	LABEL_COPY_CORRUPT_WORM_TO_GOOD_NONWORM => "Labels copied",

	LABEL_EDITV1_GOOD_WORM_TO_CORRUPT => "Error: label edit not permitted as the disk is part of a SnapLock Compliance volume.",

	LABEL_EDIT_GOOD_WORM_TO_CORRUPT => "Error: label edit not permitted as the disk is part of a SnapLock Compliance volume.",

	LABEL_EDITV1_GOOD_NONWORM_TO_CORRUPT => "",

	LABEL_EDIT_GOOD_NONWORM_TO_CORRUPT => "",

	LABEL_EDITV1_GOOD_WORM_TO_BADVERSION => "Error: label edit not permitted as the disk is part of a SnapLock Compliance volume.",

	LABEL_EDIT_GOOD_WORM_TO_BADVERSION => "Error: label edit not permitted as the disk is part of a SnapLock Compliance volume.",

	LABEL_EDITV1_GOOD_NONWORM_TO_BADVERSION => "Warning: Unable to write old",

	LABEL_EDIT_GOOD_NONWORM_TO_BADVERSION => "Error writing edited label!",

	LABEL_WIPE_GOOD_WORM => "Error: label wipe not permitted as the disk is part of a SnapLock Compliance volume.",

	LABEL_WIPEV1_GOOD_WORM => "Error: label wipev1 not permitted as the disk is part of a SnapLock Compliance volume.",

	LABEL_WIPE_BADVERSION_WORM => "Error: label wipe not permitted as the disk has a bad RAID version.",

	LABEL_WIPEV1_BADVERSION_WORM => "Error: label wipev1 not permitted as the disk has a bad RAID version.",

	LABEL_WIPE_CORRUPT_WORM => "",

	LABEL_WIPE_CORRUPT_NONWORM => "",

	LABEL_WIPEV1_CORRUPT_WORM => "",

	DD_GOOD_NONWORM_GOOD_WORM => "Disk <disk> is part of a SnapLock Compliance volume. It will not be overwritten or formatted",

	DD_GOOD_NONWORM_CORRUPT_WORM => "",

	DD_GOOD_NONWORM_BADVERSION_WORM => "has a bad raid label version. It will not be overwritten or formatted",

	DD_GOOD_WORM_BADVERSION_GOOD_WORM => "is part of a SnapLock Compliance volume. It will not be overwritten or formatted",

	LABEL_COPY_GOOD_NONWORM_TO_WORM => "is part of a SnapLock Compliance volume.",

        LABEL_COPY_GOOD_WORM_TO_GOOD_NONWORM =>  "is part of a SnapLock Compliance volume.",

	LABEL_MAKERAID_GOOD_WORM => "Unable to automatically patch disk <disk> into raidtree",

	LABEL_COPY_GOOD_WORM_RECONSTRUCT_TO_WORM => "Error: Cannot copy label because disk <disk> is part of a SnapLock Compliance volume.",

        SCSI_FORMAT_GOOD_WORM => "Un-Formatted. Snaplock Compliance Disk",

        SCSI_FORMAT_GOOD_NONWORM => "Formatted",

        SCSI_FORMAT_CORRUPT_WORM => "Formatted",

        SCSI_FORMAT_CORRUPT_NONWORM => "Formatted",

	LABEL_BUILDTREE_GOOD_WORM => "Error: label buildtree not permitted as the disk is part of a SnapLock Compliance volume.",

 	LABEL_MAKESPARE_BADVERSION_WORM_TO_SPARE => "Error: label makespare not permitted",

        LABEL_MAKESPARE_GOOD_WORM_TO_SPARE => "Error: label makespare not permitted",

	LABEL_MAKESPARE_BADVERSION_WORM_TO_SPARE => "Error: label makespare not permitted",

	LABEL_MAKESPARE_GOOD_WORM_TO_SPARE => "Error: label makespare not permitted",
       
        LABEL_MAKESPARE_CORRUPT_NONWORM_TO_SPARE => "forced to be a SPARE disk",

 	DISK_COPY_GOOD_NONWORM_TO_GOOD_WORM => "diskcopy: Disk <disk> is part of a SnapLock Compliance volume. It will not be overwritten or formatted",

	DISK_COPY_GOOD_WORM_TO_BADVERSION_WORM => "diskcopy: Disk <disk> has a bad raid label version. It will not be overwritten ",

	DISK_COPY_GOOD_NONWORM_TO_BADVERSION_WORM => "diskcopy: Disk <disk> has a bad raid label version. It will not be overwritten ",

	DISK_COPY_CORRUPT_WORM_TO_GOOD_NONWORM => "",

        AGGR_UNDESTROY_BADVERSION_WORM => "No matching disk found",
	
	AGGR_UNDESTROY_CORRUPT_WORM => "No matching disk found",

	AGGR_UNDESTROY_GOOD_WORM => "part of a SnapLock Compliance volume and cannot be used for aggr undestroy",

	##Vfiler Messages
	VFILER_MOVE_VOL_PRIVDEL_ENABLE => "Cannot move storage unit. Given storage unit <path> belongs to volume with privilege delete on.Turn privilege delete off before moving volume.",
	VFILER_MOVE_QTREE_PRIVDEL_ENABLE => "Cannot move storage unit. Given storage unit <path> belongs to volume with privilege delete on.Turn privilege delete off before moving volume.",
	VFILER_MOVE_VOL_PRIVDEL_OFF => "WARNING: reassigning storage to another vfiler does not change the security information on that storage. If the security domains are not identical, unwanted access may be permitted, and wanted access may be denied.",
	ENABLE_PRIVDEL_VFILER_VOL => "snaplock priv_delete: cannot set option: Vfiler does not completely own the given volume",
	VFILER_SET_WORM_LOG => "Cannot set the SnapLock log volume. SnapLock log volume <path> should be completely owned by default vfiler.",
	VFILER_MOVE_PARTIAL_WORM_LOG => "Cannot move storage unit. Given storage unit <path> is on an active SnapLock log volume.",
	PRIVDEL_STATUS_VFILER_VOLUME => "snaplock priv_delete: cannot get option: Vfiler does not completely own the given volume",

        ##snaplock clock messages.
        NONSL_COMPLIANCE_CLOCK => "Error: Volume regvol1 neither a SnapLock volume nor is a root volume.",

        ## Options snaplock.log.maximum_size messages.
        LOG_MINIMUM_VALUE => "Minimum permissible value for snaplock.log.maximum_size is 100k",
        LOG_MAXIMUM_VALUE => "Maximum permissible value for snaplock.log.maximum_size is (4t- 1) bytes",
        LOG_USAGE_MESSAGE => "Usage: options snaplock.log.maximum_size (<count>[|k|m|g|t])",
	LOG_FILE_CLOSE_FAIL => "Failed to close SnapLock log file",
	LOG_STATUS_FAIL => "snaplock log status failed: SnapLock log files on volume are inconsistent",
	LOG_VOL_CHANGE_FAIL => "Cannot change the SnapLock log volume",
	LOG_ARCHIVE_FAIL => "snaplock log archive failed: Failed to write SnapLock log file",
        LOG_APPEND_FAIL => "Failed to write SnapLock log file",
 	LOG_FILE_CREATE_FAIL =>"Failed to create SnapLock log file",


);

# Output messages specific to IC.0 version
# include only those output messages that are specific to this version
# all common messages should be included in base messages %MESSAGE
# any message that is not found here will be searched in base messages %MESSAGE
our %IC_MESSAGE = 
(

# TBD
#	LABEL_EDITV1_GOOD_WORM_TO_BADVERSION => "Error: Cannot edit labels of disks in a SnapLock Compliance volume.",

#	LABEL_EDIT_GOOD_WORM_TO_BADVERSION => "Error: Cannot edit labels of disks in a SnapLock Compliance volume.",


	LABEL_WIPEV1_BADVERSION_WORM => "Error: label wipev1 not permitted as the disk has a bad RAID version.",
);


# Output messages specific to GB version
# include only those output messages that are specific to this version
# all common messages should be included in base messages %MESSAGE
# any message that is not found here will be searched in base messages %MESSAGE
our %GB_MESSAGE = 
(
	LABEL_EDITV1_GOOD_NONWORM_TO_BADVERSION => "Warning: Unable to write old",
);

# Output messages specific to TT version
# include only those output messages that are specific to this version
# all common messages should be included in base messages %MESSAGE
# any message that is not found here will be searched in base messages %MESSAGE
our %TT_MESSAGE = 
(
);

# Output messages specific to AS version
# include only those output messages that are specific to this version
# all common messages should be included in base messages %MESSAGE
# any message that is not found here will be searched in base messages %MESSAGE
our %AS_MESSAGE = 
(
);

# this is required when filer is not able to auto-boot
# need to be fine-tuned as different platform need different boot string like "boot_primary", "boot <drive>" etc
use constant TEST_LOADER_BOOT_IMAGE => "boot_primary";


use vars ('@ISA', '@EXPORT', '@EXPORT_OK');
require Exporter;

@ISA = qw(Exporter);

@EXPORT = qw(
		TRUE
		FALSE
		COMPLIANCE
		ENTERPRISE
		REGULAR
		SPARES
		VERSION1
		VERSION2
		BOTH
		DD_COUNT
		CONNTYPE
		@DISK_TYPES
		%LABEL_FIELDS_TO_BADVERSION
		%LABEL_FIELDS_V1_TO_BADVERSION
		@LABEL_FIELDS_TO_CORRUPT
		%MESSAGE
		%IC_MESSAGE
		%GB_MESSAGE
		%TT_MESSAGE
		%AS_MESSAGE
		%DISK_STATES
		TEST_LOADER_BOOT_IMAGE
		MAINT
		SNAPLOCK_TYPE_BAD_CORRUPT_DISK 
		SNAPLOCK_TYPE
		AGGR_SLC_CORRUPT
		AGGR_NORMAL
		AGGR_BADVERSION
		AGGR_SLC
		RAID4_AGGR_TYPE
		RAID4_NUM_DISK
		RAID_DP_NUM_DISK
		CORRUPT_DISK_RAIDGROUP_SIZE
		%SLC_CORRUPT_DISKS
		%NORMAL_CORRUPT_DISKS
		AGGR_SLC_DISK_UNFAIL
		SNAPLOCK_COMPLIANCE
		SNAPLOCK_ENTERPRISE
		WORM_AGGR_TYPE_UNDER_TEST
		WAIT_TIME_FOR_DISK_ZERO_COMPLETE
		DEFAULT_AGGR_TYPE
		DEFAULT_AGGR_NUM_DISK
		DEFAULT_AGGR_WORM_TEMP
		DEFAULT_AGGR_NONWORM_TEMP
	);

1;
