# -*- perl -*-

# $id //depot/prod/test/main/lib/snaplock/Raid/LabelUtils.pm#1 $

## @description
##   Contains function to get the disk of desired type. 
##   	Eg: get_disk_type( host => $fh, state => "good", snaplock => "non-sl", type => "DATA");
##          get_disk_type( host => $fh, state => "corrupted", snaplock => "compliance", type => "PARITY");
##
## @author kbala@netapp.com
## @status private

package SnapLock::Raid::LabelUtils;

use strict;
use warnings;

use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF BOOLEAN);
use Data::Dumper;
use Storable;
use Aggregate qw(aggr_root aggr_status);
use Storage::DiskUtils qw(get_disk_list_maint);

use RaidSqa;
use Raid::Label qw(label_examine label_examinev1);

$Storable::canonical = 1;

BEGIN {
Params::Validate::validation_options(
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
}

use vars ('@ISA', '@EXPORT', '@EXPORT_OK');
require Exporter;

@ISA = qw(Exporter);

@EXPORT = qw(
        get_disk_type
);

=head2 get_disk_type 

get_disk_type(host=>$host, conntype=>$conntype, state=>$state, snaplock=>$snaplock, version=>$version, type=>$type)

Get the disks of specified type from the filer

It returns an array of disks if present

The following arguments are accepted:

=over

=item host

(Required): Hostspec to filer

=item conntype

(Optional): Connection type to use. Default is "#rsh". For filermode "MAINT" it is "#console" by default.

=item state 

(Optional) : State can be one of the following type corrupted/bad_version/good disk. Default is "good" disk

=item snaplock

(Optional) : Snaplock can be one of the following type compliance/enterprise/non-sl. Default is "non-sl" disk

=item version

(Optional) : Label version can be v1/v2/v1v2. default version is null

=item type

(Optional) : Disk type can be either DATA/PARITY/DPARITY. Default type is "DATA"

=back

=cut

#---------------------------------------------------------------------
# get_disk_type
#---------------------------------------------------------------------

sub get_disk_type {
       my (%opts) = validate( @_, {
                                 host => { type=>OBJECT, isa=>'Hostrec' },
                                 conntype => {type=>SCALAR, default=>"#console", optional=>1},
				 state => {type=>SCALAR, default=>"good", optional=>1, regex => qr/[corrupted|bad_version|good]/ },
                                 snaplock => {type=>SCALAR, default=>"non-sl", optional=>1, regex => qr/[compliance|enterprise|non-sl|spare]/ },
				 version => {type=>SCALAR, default=>"v1v2", optional=>1, regex => qr/[v1|v2|v1v2]/ },
                                 type => {type=>SCALAR, default=>"", optional=>1, regex => qr/[DATA|PARITY|DPARITY]/ },
                                 force => {type=>SCALAR, default=> 1, optional=>1, regex => qr/[0|1]/ },
                                 strict => {type=>BOOLEAN, default=> 1, optional=>1 },
                                }
                            );

      my $host = $opts{host};
      my $state = $opts{state};
      my $snaplock = $opts{snaplock};
      my $version = $opts{version};
      my $type = $opts{type};
      my ($obj, $disk_ref, $count, @disks);
      $count = 0;

      #die "Please specify state as [ corrupted/bad_version/good ]" if ( $state !~ /corrupted|bad_version|good/ );
      #die "Please specify snaplock type as [ compliance/enterprise/nonsl/spare ]" if ( $snaplock !~ /compliance|enterprise|non-sl|spare/ );
      #die "Please specify type as [ DATA/PARITY/DPARITY ]" if ( $type !~ /DATA|PARITY|DPARITY|ANY/ );

      logcomment("Request to find disk [state=$state, version=$version, snaplock=$snaplock, type=$type]");

     our %ENV;
#     if(defined $ENV{all_disks} and $ENV{all_disks} ne "")
#     {
	     $obj = $ENV{all_disk};
	     logcomment("Re-using the discovered disks = $obj");
#     }
#     else
#     {

      $obj = _get_disks_info( force => $opts{force}, host => $host, 
			#	state => $state, snaplock => $snaplock, 
			#	version => $version, type => $type, strict => $opts{strict}
			);
#     }

	my $ref = ref($obj);

        print Dumper($obj);
        print("\n");
	logcomment("Reference=$ref, obj=$obj\n");
      if ( ( $version =~ /v1|v2|v1v2/ ) && ( $state =~ /corrupted/ ) ) 
      { 
          $disk_ref = $obj->{"$state"}->{"$version"}->{"$snaplock"}->{"$type"};
          print(" disk_ref = $disk_ref \n");
	      my $disk_ref_count = scalar(@$disk_ref);
	      logcomment("scalar for disk_ref=$disk_ref_count, count=$#{$disk_ref}");
	      if("$#{$disk_ref}" < 0) 
	      {
		      @{$disk_ref} = (); 
		      push @{$disk_ref}, @{$obj->{"$state"}->{"$version"}->{"$snaplock"}->{"DATA"}}; 
		      push @{$disk_ref}, @{$obj->{"$state"}->{"$version"}->{"$snaplock"}->{"PARITY"}}; 
		      push @{$disk_ref}, @{$obj->{"$state"}->{"$version"}->{"$snaplock"}->{"DPARITY"}}; 
		      logcomment("disk_ref=$#{$disk_ref}, disks=@{$disk_ref}");
	      }
      } 
      elsif ( ( $state =~ "good" ) || ( $state =~ /bad_version/ ) ) 
      {
	      $disk_ref = ($snaplock eq "spare")? $obj->{"$snaplock"} : $obj->{"$state"}->{"$snaplock"}->{"$type"};
	      my $disk_ref_count = scalar(@$disk_ref);
	      logcomment("scalar for disk_ref=$disk_ref_count, count=$#{$disk_ref}");
	      if($#{$disk_ref} < 0 and $snaplock ne "spare")
	      {
		      @{$disk_ref} = (); 
	            push @{$disk_ref}, @{$obj->{"$state"}->{"$snaplock"}->{"DATA"}}; 
	            push @{$disk_ref}, @{$obj->{"$state"}->{"$snaplock"}->{"PARITY"}}; 
		    push @{$disk_ref}, @{$obj->{"$state"}->{"$snaplock"}->{"DPARITY"}}; 

		      logcomment("disk_ref=$#{$disk_ref}, disks=@{$disk_ref}");
	      }
      } else {
	      #die "Invalid label state";
	      logcomment("No disk qualified the specified type or invalid disk type specified");
      }

      logcomment(@{$disk_ref});
      $count = scalar(@$disk_ref);

      #die "Disk of type [$state $version $snaplock $type] does not exist" if ($count == 0);
      if ($count == 0)
      { 
	      #logcomment("Disk of type [state=$state, version=$version, snaplock=$snaplock, type=$type] does not exist"); 
	      print Dumper($obj);
	      die "Disk of type [state=$state, version=$version, snaplock=$snaplock, type=$type] does not exist";
      }

      foreach my $i (0..($count-1)) {
 	 push( @disks, $disk_ref->[$i] );
      }
     
     logcomment("Saving it to env all_disks");
     $ENV{all_disks} = $obj;
     logcomment("verify saved: $ENV{all_disks}");

     logcomment("returning ...");
    logcomment(@disks); 
      return (@disks);
}

sub analyze_one_disktype
{
	my %opts = @_;

	my @disks;
	( ref($opts{disks}) eq "ARRAY")?  @disks = $opts{disks} : push @disks, $opts{disks};

	my $host = $opts{host};
	my $conntype = $opts{conntype};
	my ($fc, $output, $output1, $lbl_version);
	my (%disk, $disk, $version, $label, $aggr, $raid_type, $sl);

	my ($version2, $version1);

	my $ds = $opts{store};
	$lbl_version = $ds->{root_label_version};

	foreach my $disk (@disks) 
	{
		$version2 = label_examine(
						   host => $host,
						   disk => $disk,
						  );

		$version1 = label_examinev1(
						  host => $host,
						  disk => $disk,
						 );

	 #	next if ( ($version2 =~ /Disk $disk does not exist/) || ($version2 =~ /root/) );

	 	if($version2 =~ /lbl $lbl_version/i)
		{
			$label = "bad_version";
		}
	 	elsif($version2 =~ /Invalid Label/i and $version2 =~ /Invalid Label/i)
		{
			$label = "corrupted";
			$version = "v1v2";
		}
		elsif( $version2 =~ /Invalid Label/i and $version2 !~ /Invalid Label/i)
		{
			$label = "corrupted";
			$version = "v2";
		}
		elsif( $version2 !~ /Invalid Label/i and $version2 =~ /Invalid Label/i)
		{
			$label = "corrupted";
			$version = "v1";
		}
		else
		{
			$label = "good";
		}

		if ($version2 =~ /\|WORM\|WORM admin/) {
		    $sl = "enterprise";
		} elsif ($version2 =~ /\|WORM/) {
		    $sl = "compliance";
		} elsif ($version2 =~ /(.*)Disk type\sSPARE/) {
		    $sl = "spare";
		} else {
		    $sl = "non-sl";
		}

		$aggr = $2 if ($version1 =~ /(.*)Volume Name:\s(.*)/);

		$raid_type = $2 if ($version1 =~ /(.*)Disk type\s(.*)/);
		chop($raid_type);
		chop($raid_type);
		chomp($raid_type);

		logcomment("Disk: label=$label, version=$version, sl=$sl, raid_type=$raid_type");

		my $disk_array = $opts{store}->{$opts{type}}{$label}{$version}{$sl}{$raid_type};
		push @$disk_array, $disk;

		print Dumper(\%{$opts{store}});
	}
}

sub disk_gatherer
{
	my %opts = @_;
	my %root_disks = get_root_label_version (conn => $opts{conn}, prompt => $opts{prompt});

	analyze_one_disktype(type=> "root", disks => \$root_disks{root}{"good"}{"non-sl"}{"DATA"});

	my %all_disks = collect_disks_by_type (conn => $opts{conn});

	foreach my $key (keys %all_disks)
	{
		foreach my $ele (@{$all_disks{$key}})
		{
			analyze_one_disktype(type => $key, disks => $ele);
		}
	}
}
sub collect_disks_by_type
{
	my %opts = @_;
	my (%disks, @aggr_list, @adisks);

	foreach my $aggrtype ("snaplock_compliance", "snaplock_enterprise", "root", "normal", "spare" )
	{
		@aggr_list = ();
		@aggr_list = get_aggr_by_status(conn => $opts{conn}, prompt => $opts{prompt}, aggrtype => $aggrtype) if($aggrtype ne "spare");

		logcomment("aggr list = @aggr_list");

		$disks{$aggrtype} = ();
		@adisks = ();
		if($aggrtype eq "spare")
		{
			@adisks = get_aggr_disks (conn => $opts{conn}, prompt => $opts{prompt}, aggropt => "-s", prompt => $opts{prompt});
		}
		else
		{
			foreach my $aggr (@aggr_list)
			{
				logcomment("working on aggr=$aggr");
				@adisks = get_aggr_disks (conn => $opts{conn}, prompt => $opts{prompt}, aggr => $aggr, prompt => $opts{prompt});
				foreach my $a (@adisks)
				{
					push @{$disks{$aggrtype}}, $a;
				}

			}
		}
	}

	logcomment("Collected Disks are=@adisks");
	foreach my $key (keys %disks)
	{
		logcomment("$key=$disks{$key}");
	}
	return %disks;
}
sub get_aggr_by_status
{
	my %opts = @_;
	my ($output, $prompt);
	(undef, undef, $output, $prompt) = say($opts{conn}, "aggr status $opts{aggr}", '-prompt', $opts{prompt});

	my $match = ( defined $opts{aggrtype} )? $opts{aggrtype} : '(\w+)?';

	logcomment("Looking for aggr of type $match");
	my @aggrs = ();
	foreach my $line (split /\n/, $output)
	{
		if($line =~ /(\w+)\s+\w+\s+\w+(, \w+)?\s+$match/)
		#if($line =~ /(\w+)\s+\w+\s+\w+(, \w+)?\s+root/)
		{
			push @aggrs, "$1";
			logcomment("Line=aggr=$1");
			last if($match eq "root");
		}
	}
	return @aggrs;
}
sub get_aggr_disks
{
	my %opts = @_;
	my ($output, $prompt);
	my @disks = ();

	(undef, undef, $output, $prompt) = say($opts{conn}, "aggr status $opts{aggr} $opts{aggropt}", '-prompt', $opts{prompt});
	#my @aggrs = get_aggr_by_status(conn => $opts{conn}, prompt => $opts{prompt}, aggr => $root_aggr);
	foreach my $line (split /\n/, $output)
	{
#		logcomment("Line=$line");
		if($line =~ /[data|parity|dparity]\s+(\w+(\.\w+)+)*\s+/)
		{
			push @disks, "$1";
			logcomment("Line=disk=$line, 1=$1");
		}
	}

	return @disks;
}
sub get_root_label_version
{
	my %opts = @_;
	my (@root_aggr, @root_disks);

	@root_disks = ();
	my ($output, $prompt);
	logcomment("Finding root aggr");
	@root_aggr = get_aggr_by_status (conn => $opts{conn}, prompt => $opts{prompt}, aggrtype => "root");

	logcomment("Root aggr=$root_aggr[0]");

	push @root_disks, $root_aggr[0];

	@root_disks = get_aggr_disks (conn => $opts{conn}, prompt => $opts{prompt}, aggr => $root_aggr[0], prompt => $opts{prompt});
	logcomment("Root aggr disks=@root_disks");

	my %disk_types = ();
	
	analyze_one_disktype( type => "root", disks => $root_disks[0], store => \%disk_types);

	return %disk_types;
}
#---------------------------------------------------------------------
# _get_disks_info
#---------------------------------------------------------------------

sub _get_disks_info {
       my (%opts) = validate( @_, {
                                 host => { type=>OBJECT, isa=>'Hostrec' },
                                 conntype => {type=>SCALAR, default=>"#console"},
                                 force => {type=>SCALAR, default=> 1, optional=>1, regex => qr/[0|1]/ },
				#strict => { type => BOOLEAN, default => 1, optional => 1 },
                                }
                            );

    my $host = $opts{host};
    my $conntype = $opts{conntype};
    my ($fc, $output, $output1, $lbl_version, @disks);
    my (%disk, $disk, $version, $label, $aggr, $raid_type, $sl);

    $fc = make_conn($host, $conntype);
    @disks = get_disk_list_maint($fc);

    my (@arr1, @arr2, @arr3, @arr4, @arr5, @arr6, @arr7, @arr8, @arr9, @arr10, @arr11, @arr12, @arr13, @arr14, @arr15);
    my (@arr16, @arr17, @arr18, @arr19, @arr20, @arr21, @arr22, @arr23, @arr24, @arr25, @arr26, @arr27, @arr28, @arr29, @arr30,);
    my (@arr31, @arr32, @arr33, @arr34, @arr35, @arr36, @arr37, @arr38, @arr39, @arr40, @arr41, @arr42, @arr43, @arr44, @arr45);
    my (@arr46);

    foreach my $disk (@disks) {
        $output = Label::label_examine(
                                           host => $host,
                                           disk => $disk,
					   filermode => 'maint',
					   conntype => '#console',
                                          );
        if ($output =~ /root/) {
            $lbl_version = $2 if ($output =~ /(.*)lbl\s(.*)/);
            my @lbl_version = split(/,/, $lbl_version);
            $lbl_version = $lbl_version[0];
	    print  $lbl_version;
	    logcomment("Root disk label version:  $lbl_version");
            last;
        }
    }
#    $lbl_version = get_root_label_version(@disks);

    foreach my $disk (@disks) {
        $output = Label::label_examine(
                                           host => $host,
                                           disk => $disk,
                                           filermode => 'maint',
                                           conntype => '#console',

                                          );

        next if ( ($output =~ /Disk $disk does not exist/) || ($output =~ /root/) );

        $output1 = Label::label_examinev1(
                                                host => $host,
                                                disk => $disk,
                                                filermode => 'maint',
                                                conntype => '#console',
                                              );

        if ($output !~ /$lbl_version/) {
            $label = "bad_version";
	} else {
 
            if($output =~ /Invalid Label/) {
                if($output1 =~ /Labels appear to be corrupted/) {
                    $version = "v1v2";
                    $label = "corrupted";
                } else {
                    $version = "v2";
                    $label = "corrupted";
                }

            } elsif ($output !~ /Invalid Label/) {
                $output1 = Label::label_examinev1(
                                                        host => $host,
                                                        disk => $disk,
                                               filermode => 'maint',
                                                conntype => '#console',

                                                       );

                  if($output1 =~ /Labels appear to be corrupted/) {
                      $version = "v1";
                      $label = "corrupted"; 
                  } else {
                      $label = "good";
                  }
             }
        }

        if ($output =~ /WORM\|WORM admin/) {
            $sl = "enterprise";
        } elsif ($output =~ /WORM/) {
            $sl = "compliance";
        } elsif ($output =~ /Disk\s+type\s+SPARE/) {
            $sl = "spare";
        } else {
            $sl = "non-sl";
        }

        $aggr = $2 if ($output1 =~ /(.*)Volume Name:\s(.*)/);

        $raid_type = $2 if ($output1 =~ /(.*)Disk type\s(.*)/);

        chop($raid_type);
        chop($raid_type);
        chomp($raid_type);

	if($raid_type =~ /SPARE/i)
	{
		logcomment("Changing label type from [$label] to [good] since [raid_type] is [$raid_type]");
	    $label = "good";
            $sl = "spare";
	    $version = "v1v2";
	}

	logcomment("DISK: [$disk, label=$label, version=$version, SL=$sl, disktype=$raid_type] aggr=$aggr");

        if ($label eq "corrupted") {
            if ($sl eq "compliance") {
                push (@arr1, $disk) if ( ($version eq "v1") && ($raid_type eq "DATA") );
                push (@arr2, $disk) if ( ($version eq "v1") && ($raid_type eq "PARITY") );
                push (@arr3, $disk) if ( ($version eq "v1") && ($raid_type eq "DPARITY") );
                push (@arr4, $disk) if ( ($version eq "v2") && ($raid_type eq "DATA") );
                push (@arr5, $disk) if ( ($version eq "v2") && ($raid_type eq "PARITY") );
                push (@arr6, $disk) if ( ($version eq "v2") && ($raid_type eq "DPARITY") );
                push (@arr7, $disk) if ( ($version eq "v1v2") && ($raid_type eq "DATA") );
                push (@arr8, $disk) if ( ($version eq "v1v2") && ($raid_type eq "PARITY") );
                push (@arr9, $disk) if ( ($version eq "v1v2") && ($raid_type eq "DPARITY") );
            } elsif ($sl eq "enterprise") {
                push (@arr10, $disk) if ( ($version eq "v1") && ($raid_type eq "DATA") );
                push (@arr11, $disk) if ( ($version eq "v1") && ($raid_type eq "PARITY") );
                push (@arr12, $disk) if ( ($version eq "v1") && ($raid_type eq "DPARITY") );
                push (@arr13, $disk) if ( ($version eq "v2") && ($raid_type eq "DATA") );
                push (@arr14, $disk) if ( ($version eq "v2") && ($raid_type eq "PARITY") );
                push (@arr15, $disk) if ( ($version eq "v2") && ($raid_type eq "DPARITY") );
                push (@arr16, $disk) if ( ($version eq "v1v2") && ($raid_type eq "DATA") );
                push (@arr17, $disk) if ( ($version eq "v1v2") && ($raid_type eq "PARITY") );
                push (@arr18, $disk) if ( ($version eq "v1v2") && ($raid_type eq "DPARITY") );
            } elsif ($sl eq "non-sl") {
                push (@arr19, $disk) if ( ($version eq "v1") && ($raid_type eq "DATA") );
                push (@arr20, $disk) if ( ($version eq "v1") && ($raid_type eq "PARITY") );
                push (@arr21, $disk) if ( ($version eq "v1") && ($raid_type eq "DPARITY") );
                push (@arr22, $disk) if ( ($version eq "v2") && ($raid_type eq "DATA") );
                push (@arr23, $disk) if ( ($version eq "v2") && ($raid_type eq "PARITY") );
                push (@arr24, $disk) if ( ($version eq "v2") && ($raid_type eq "DPARITY") );
                push (@arr25, $disk) if ( ($version eq "v1v2") && ($raid_type eq "DATA") );
                push (@arr26, $disk) if ( ($version eq "v1v2") && ($raid_type eq "PARITY") );
                push (@arr27, $disk) if ( ($version eq "v1v2") && ($raid_type eq "DPARITY") );
           } elsif ($label eq "spare") {
        	push (@arr46, $disk);
            }
        } elsif ($label eq "good") {
            if ($sl eq "compliance") {
                push (@arr28, $disk) if ( ($raid_type eq "DATA") );
                push (@arr29, $disk) if ( ($raid_type eq "PARITY") );
                push (@arr30, $disk) if ( ($raid_type eq "DPARITY") );
            } elsif ($sl eq "enterprise") {
                push (@arr31, $disk) if ( ($raid_type eq "DATA") );
                push (@arr32, $disk) if ( ($raid_type eq "PARITY") );
                push (@arr33, $disk) if ( ($raid_type eq "DPARITY") );
            } elsif ($sl eq "non-sl") {
                push (@arr34, $disk) if ( ($raid_type eq "DATA") );
                push (@arr35, $disk) if ( ($raid_type eq "PARITY") );
                push (@arr36, $disk) if ( ($raid_type eq "DPARITY") );
            }elsif ($sl eq "spare") {
            	push (@arr46, $disk);
            }
        } elsif ($label eq "bad_version") {
            if ($sl eq "compliance") {
                push (@arr37, $disk) if ( ($raid_type eq "DATA") );
                push (@arr38, $disk) if ( ($raid_type eq "PARITY") );
                push (@arr39, $disk) if ( ($raid_type eq "DPARITY") );
            } elsif ($sl eq "enterprise") {
                push (@arr40, $disk) if ( ($raid_type eq "DATA") );
                push (@arr41, $disk) if ( ($raid_type eq "PARITY") );
                push (@arr42, $disk) if ( ($raid_type eq "DPARITY") );
            } elsif ($sl eq "non-sl") {
                push (@arr43, $disk) if ( ($raid_type eq "DATA") );
                push (@arr44, $disk) if ( ($raid_type eq "PARITY") );
                push (@arr45, $disk) if ( ($raid_type eq "DPARITY") );
            }elsif ($sl eq "spare") {
            	push (@arr46, $disk);
            }
        } elsif($label eq "spare")
	{
            	push (@arr46, $disk);
	}
    }

    $disk{"corrupted"}{"v1"}{"compliance"}{"DATA"} = \@arr1;
    $disk{"corrupted"}{"v1"}{"compliance"}{"PARITY"} = \@arr2;
    $disk{"corrupted"}{"v1"}{"compliance"}{"DPARITY"} = \@arr3;
    $disk{"corrupted"}{"v2"}{"compliance"}{"DATA"} = \@arr4;
    $disk{"corrupted"}{"v2"}{"compliance"}{"PARITY"} = \@arr5;
    $disk{"corrupted"}{"v2"}{"compliance"}{"DPARITY"} = \@arr6;
    $disk{"corrupted"}{"v1v2"}{"compliance"}{"DATA"} = \@arr7;
    $disk{"corrupted"}{"v1v2"}{"compliance"}{"PARITY"} = \@arr8;
    $disk{"corrupted"}{"v1v2"}{"compliance"}{"DPARITY"} = \@arr9;
    $disk{"corrupted"}{"v1"}{"enterprise"}{"DATA"} = \@arr10;
    $disk{"corrupted"}{"v1"}{"enterprise"}{"PARITY"} = \@arr11;
    $disk{"corrupted"}{"v1"}{"enterprise"}{"DPARITY"} = \@arr12;
    $disk{"corrupted"}{"v2"}{"enterprise"}{"DATA"} = \@arr13;
    $disk{"corrupted"}{"v2"}{"enterprise"}{"PARITY"} = \@arr14;
    $disk{"corrupted"}{"v2"}{"enterprise"}{"DPARITY"} = \@arr15;
    $disk{"corrupted"}{"v1v2"}{"enterprise"}{"DATA"} = \@arr16;
    $disk{"corrupted"}{"v1v2"}{"enterprise"}{"PARITY"} = \@arr17;
    $disk{"corrupted"}{"v1v2"}{"enterprise"}{"DPARITY"} = \@arr18;
    $disk{"corrupted"}{"v1"}{"non-sl"}{"DATA"} = \@arr19;
    $disk{"corrupted"}{"v1"}{"non-sl"}{"PARITY"} = \@arr20;
    $disk{"corrupted"}{"v1"}{"non-sl"}{"DPARITY"} = \@arr21;
    $disk{"corrupted"}{"v2"}{"non-sl"}{"DATA"} = \@arr22;
    $disk{"corrupted"}{"v2"}{"non-sl"}{"PARITY"} = \@arr23;
    $disk{"corrupted"}{"v2"}{"non-sl"}{"DPARITY"} = \@arr24;
    $disk{"corrupted"}{"v1v2"}{"non-sl"}{"DATA"} = \@arr25;
    $disk{"corrupted"}{"v1v2"}{"non-sl"}{"PARITY"} = \@arr26;
    $disk{"corrupted"}{"v1v2"}{"non-sl"}{"DPARITY"} = \@arr27;
    $disk{"good"}{"compliance"}{"DATA"} = \@arr28;
    $disk{"good"}{"compliance"}{"PARITY"} = \@arr29;
    $disk{"good"}{"compliance"}{"DPARITY"} = \@arr30;
    $disk{"good"}{"enterprise"}{"DATA"} = \@arr31;
    $disk{"good"}{"enterprise"}{"PARITY"} = \@arr32;
    $disk{"good"}{"enterprise"}{"DPARITY"} = \@arr33;
    $disk{"good"}{"non-sl"}{"DATA"} = \@arr34;
    $disk{"good"}{"non-sl"}{"PARITY"} = \@arr35;
    $disk{"good"}{"non-sl"}{"DPARITY"} = \@arr36;
    $disk{"bad_version"}{"compliance"}{"DATA"} = \@arr37;
    $disk{"bad_version"}{"compliance"}{"PARITY"} = \@arr38;
    $disk{"bad_version"}{"compliance"}{"DPARITY"} = \@arr39;
    $disk{"bad_version"}{"enterprise"}{"DATA"} = \@arr40;
    $disk{"bad_version"}{"enterprise"}{"PARITY"} = \@arr41;
    $disk{"bad_version"}{"enterprise"}{"DPARITY"} = \@arr42;
    $disk{"bad_version"}{"non-sl"}{"DATA"} = \@arr43;
    $disk{"bad_version"}{"non-sl"}{"PARITY"} = \@arr44;
    $disk{"bad_version"}{"non-sl"}{"DPARITY"} = \@arr45;
    $disk{"spare"} = \@arr46;

    print Dumper(\%disk);
    if($opts{force})
    {
	    my $fh;
	    my $file = "DB_FILE";
	    #open($fh, ">$file") or warn "Could not write to file $file";
	    #print $fh Dumper(\%disk);
	    #close($fh);

	    my $arr = eval(%disk);
	    store(\$arr, $file); 
    }
    return \%disk;

}

1;
