# -*- perl -*-
#
# $Id: //depot/prod/test/main/lib/SnapLock/Raid/Utils.pm#1 $
#
## @summary supporting functions for SnapLock Raid functions
##
## @author sureshb@netapp.com
## @status Private
## @pod here

package SnapLock::Raid::Utils;

use Switch;
# base libaries
use NATE;
use Tharn;
use Utils qw(make_conn make_host);
use filer;
use boot qw(transit_to boot_state);
use Filer::Version qw(get_ontap_version);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF UNDEF);
use Aggregate qw(aggr_status aggr_list aggr_create aggr_disks);

# snaplock-raid specific libraries
use SnapLock::Raid::Constants;


print "Using this from ". __FILE__;

	
use vars ('@ISA', '@EXPORT', '@EXPORT_OK');
require Exporter;

@ISA = qw(Exporter);

@EXPORT = qw( 
	verify_labels 
	Get_Expected_Message 
	filer_required_state 
	launch_subtest_setup 
	get_aggregate_type 
	get_spare_pool_status
    	verify_test_disks_exist

);

#@EXPORT_OK = qw( 
#	verify_labels 
#	Get_Expected_Message 
#	filer_required_state 
#	launch_subtest_setup 
#	get_aggregate_type 
#);


=head2 Get_Expected_Message (%)

Retrieve/Generate the expected output/error message based on the disk and message tag

Arguments

=over

=item host
(Required) : host object to filer

=item conntype
(Optional) : connection object to filer

=item disk
(Conditional) : Required depending on the output message string
		Majority of the output message require disk name to be specified

=item disk1
(Conditional) : Some of the output messages require two disk names to be specified

=item path
(Conditional) : Some of the output messages require volume or qtree path (ex : vfiler move)

=item msg
(Required) : Message code to identify the type of output message being constructed

=back

Returns
success, output/error message based on the message tag

on failure, returns the error from eval

=for html <hr />
=cut

sub Get_Expected_Message
{
	my %opts = validate( @_, {
					host => { type=>OBJECT, isa=>'Hostrec' },
					conntype => {type=>SCALAR, default=>"#console", optional => 1},
#					conn => {type=>OBJECT, isa => 'Connectrec', optional => 1 },
					disk => { type => SCALAR, optional => 1 },
					disk1 => { type => SCALAR, optional => 1 },
					path => { type => SCALAR, optional => 1 },
					msg =>  { type => SCALAR, default => "" },
				}
			);

	# Get the filer up and running in normal mode
	#filer_required_state(host => $opts{host}, state => "UP");
	my $conn = make_conn($opts{host},$opts{conntype});
	transit_to($conn, "UP");

	#my $conn = make_conn($host, $conntype);
	my $ver;
     
        # make the connection to filer, if reqd
	# retrieve the version of the filer
	# currently filer_version_get will return valid version string
	# but will not work for private builds, need to find other viable solutions	
        eval {	
		my $host = make_host($opts{host});
		my $conn = make_conn($opts{conntype});
		$ver = filer_version_get($conn);
		logcomment("Ver=[$ver]");
	};

	if($@)
	{
		logcomment("Utils: Could not get version info using filer_version_get");
	}

	eval {
		logcomment("Utils: Trying get_ontap_version");
		$ver = get_ontap_version($opts{host});
	};

	if($@)
	{
		logcomment("Utils: Could not get version info using get_ontap_version");
	}

	logcomment("Trying to find message for [ver=$ver, code=$opts{msg}]");
	$opts{msg} =~ s/\s+//g;
	# this logic can be improved to adopt a better approach
	# leaving it for now
	my $ret_msg = "";
	if ($ver =~ /^7.3/ or $ver =~ /RironcityN/)
	{
		logcomment("IC_MESSAGE=$IC_MESSAGE{$opts{msg}}");
		logcomment("DFLT_MESSAGE=$MESSAGE{$opts{msg}}");
		$ret_msg = (defined $IC_MESSAGE{$opts{msg}})?  $IC_MESSAGE{$opts{msg}} : $MESSAGE{$opts{msg}};
	}
	elsif( $ver =~ /^7.2/ or $ver =~ /RgordonbierschN/)
	{
		logcomment("GB_MESSAGE=$GB_MESSAGE{$opts{msg}}");
		logcomment("DFLT_MESSAGE=$MESSAGE{$opts{msg}}");
		$ret_msg = (defined $GB_MESSAGE{$opts{msg}})?  $GB_MESSAGE{$opts{msg}} : $MESSAGE{$opts{msg}};
	}
	elsif( $ver =~ /^7.1/ or $ver =~ /RttN/)
	{
		logcomment("TT_MESSAGE=$TT_MESSAGE{$opts{msg}}");
		logcomment("DFLT_MESSAGE=$MESSAGE{$opts{msg}}");
		$ret_msg = (defined $TT_MESSAGE{$opts{msg}})?  $TT_MESSAGE{$opts{msg}} : $MESSAGE{$opts{msg}};
	}
	elsif( $ver =~ /^7.0/ or $ver =~ /RanchorsteamN/)
	{
		logcomment("AS_MESSAGE=$AS_MESSAGE{$opts{msg}}");
		logcomment("DFLT_MESSAGE=$MESSAGE{$opts{msg}}");
		$ret_msg = (defined $AS_MESSAGE{$opts{msg}})?  $AS_MESSAGE{$opts{msg}} : $MESSAGE{$opts{msg}};
	}
	else
	{
		logcomment("No release information found ($ver), setting default expected message");
		$ret_msg = $MESSAGE{$opts{msg}};
	}

	#if($ret_msg eq "")
	#{
		#logcomment("ver=$ver, msg=$ret_msg, code=$opts{msg}, setting default expected message");
	#	$ret_msg = $MESSAGE{$opts{msg}};
	#}

	# Processing tags in the message
	# disk,disk1 with disk information
	$ret_msg =~ s/<disk>/$opts{disk}/g;
	$ret_msg =~ s/<disk1>/$opts{disk1}/g;
	$ret_msg =~ s/<path>/$opts{path}/g;

	# throwing the message for debugging purpose
	logcomment("[ver=$ver, MESG = $ret_msg, code=$opts{msg}]");

	return $ret_msg;
}

=head2 verify_labels (%)

This function is not yet implemented, but will be good to have.

Verify disk labels for the operation performed on one/more disks

Arguments

=over

=item host
(Required) : host object to filer

=item conntype
(Optional) : connection object to filer

=item oper
(Required) : type of operation being performed

=item disk
(Conditional) : Required depending on the output message string
		Majority of the output message require disk name to be specified

=item disk1
(Conditional) : Some of the output messages require two disk names to be specified

=item disk_list
(Conditional) : List of disks to work with

=item dstate
(Required) : State of the disk used for the operation

=item dtype
(Required) : type of the disk used for the operation

=item aggrtype
(Required) : type of the aggregate under test

=item vol
(Optional) : volume under test

=back

Returns
success, output/error message based on the message tag

on failure, returns the error from eval

=for html <hr />
=cut

sub verify_labels 
{
	my %opts = validate( @_, {
					disk => { type => SCALAR, optional => 1 },
					disk1 => { type => SCALAR, optional => 1 },
					disk_list => { type => ARRAYREF, optional => 1 },
					oper => { type => SCALAR, regex => qr/[edit|editv1|wipe|wipev1|labelcopy|makespare|buildtree|makeraid|dd|diskcopy]/i },
					dtype => { type => SCALAR, regex => qr/[DATA|PARITY|DPARITY]/i },
					aggrtype => { type => SCALAR, regex => qr/[snaplock|enterprise|regular|non-sl]/i },
					dstate => { type => SCALAR, regex => qr/[good|corrupt|badversion|spare]/i },
					vol => { type => SCALAR , optional => 1 },
				}
			);

	logcomment("params: @_");
	logcomment("SnapLock::Raid::Utils::verify_labels: Not Yet Implemented, returning TRUE for now");

	#should return 1/0
	#
	return 1;
}

=head2 filer_required_state (%)

A helper function
This function can be used to get the filer to a required state (maintenance, normal boot)
This function is limited by the boot::transit_to function for all the filer states

Arguments

=over

=item host
(Required) : host object to filer

=item conn
(Optional) : connection object to filer

=item conntype
(Required) : type of the connection to connect to filer

=item boot_image
(Optional) : boot image at loader prompt as part of netboot sequence

=item state
(Optional) : required state of the filer

=back

Returns
success

on failure, returns the error from transit_to

=for html <hr />
=cut

sub filer_required_state
{
	my %opts = validate( @_, {
				host => { type=>OBJECT, isa=>'Hostrec' },
				conntype => {type=>SCALAR, default=>"#console", optional => 1},
		#		conn => {type=>OBJECT,  optional => 1 },
				boot_image => { type=>SCALAR, optional => 1  },
				state => { type=>SCALAR, regex => qr/[MAINT|UP]/, default => "MAINT", optional => 1  },
				}
			);

	logcomment("Required filer state ($opts{state}), host=$opts{host}, conntype=$opts{conntype}");

	logcomment("Taking filer to ($opts{state}) state");

	return 
	transit_to($opts{host}, "$opts{state}", bootimage => $opts{boot_image});
}

# this is experimental function to launch setup as subtest from the script
# currently *not used*
sub launch_subtest_setup
{
	my %opts = @_;

	my $date = localtime();
	my $runid = "setup_";
	$date =~ s/ /_/g;
	$date =~ s/:/_/g;
	$runid .= "$date";

	subtest(
		"drg/snaplock/raid_cmds/setup.thpl",
		'-client', $opts{client},
		'-runid', "$runid",
		"--",
	       	"HIGHER_VERSION=$opts{HIGHER_VERSION}", "TEST_VERSION=$opts{TEST_VERSION}", "PRIVATE_KERNEL=$opts{PRIVATE_KERNEL}", 
		"CORRUPTED_LABEL_DISKS=$opts{CORRUPTED_LABEL_DISKS}", "GOOD_DISKS=$opts{GOOD_DISKS}", "BAD_VERSION_DISKS}=$opts{BAD_VERSION_DISKS}", 
	);
}

# experimental function to retreive disks and labels from the aggregate rather than from the "disk_list" command
# currently *not used*
sub get_aggregate
{
	my %opts = validate( @_, {
				host => { type => OBJECT, isa => 'Hostrec' },
				conntype => {type => SCALAR, default => "#console", optional => 1},
				type => { type => SCALAR, default => "snaplock", optional => 1 },
				aggr => { type => SCALAR },
			} );

	my (@aggr_list, @disks);

	if( !defined $opts{aggr} or $opts{aggr} ne "")
	{
		@aggr_list = aggr_list( $opts{host} );
		foreach my $aggr (@aggr_list)
		{
			my ($state, $status, $vol, %options) = aggr_status( $opts{host}, $aggr );
			if($options{snaplock})
			{
				@disks = aggr_disks( $opts{host}, $aggr );
				$opts{aggr} = $aggr;
				last;
			}
		}
	}

	if($#disks < 1 and $opts{aggr} ne "" )
	{
		aggr_create( $opts{host}, $opts{aggr} );
		return $opts{aggr};
	}

	return "";
}


sub get_aggregate_type
{
	my (%opts) = @_;

	my ($conn, @aggrs, $status, @ret);

	$conn = $opts{conn} || make_conn("$fh,$opts{conntype}");

	@aggrs = aggr_list($conn);

	# Determine aggr status for each of the aggregates
	foreach my $ag (@agggrs)
	{
		$status = aggr_status($conn, $ag);

		my %options = $opts{options};
		#check each of the conditions in the options flag
		foreach my $key (keys %options)
		{
			switch ($key)
			{
				case (type) { 
						push @ret, $options{$key} 
							if($status->type() eq $options{$key});
					}
			}
		}
	}

	die "Could not find aggregate of specified type" if($#ret < 0);
	logcomment("returning aggrs [@ret]");
	return @ret;
}

=head2 ::get_spare_pool_status

Get the number of spare pool disks containing in pool0 and pool1
The format is as follows
        get_spare_pool_status ($FILER_OBJ)

=item Arguments

B<$FILER> => I<Filer object>

I<Required.>
A Filer NOTE object.

=item Examples

=back

    get_spare_pool_status ($Filer_obj);

=head2 ::status

Returns hash with keys pool0 pool1 and values with number of disks
correspondingly .

=over

=cut

sub get_spare_pool_status {

         my ($FILER_OBJ) = @_;
         my (@pool0_disks,@pool1_disks);
         my $conn = $FILER_OBJ->conn;
         my $zcmd = new NaElement("disk-list-info");
         my $zapi = Zapi->new(
                                     $conn->hostp()->default_ip(),
                                     "root",
                                     $conn->hostp()->default_password()
                             );
         $zapi->{fail} = \&_fail_default;
         my $zapi_exec = $zapi->invoke_elem($zcmd, "any");
         my $zapi_op  = $zapi_exec->toString();
         my $xml = XML::Simple->new;
         my $zapi_hash_ref = $xml->XMLin($zapi_op);
         #print Dumper($zapi_hash_ref);
         my %disk_hash = %{$zapi_hash_ref->{'disk-details'}{'disk-detail-info'}};
         foreach my $disk ( keys %disk_hash ){
                if ($disk_hash{$disk}{'raid-state'} =~ /spare/){
                        if ($disk_hash{$disk}{'pool'} =~ /Pool0/){
                                push (@pool0_disks,$disk);
                        }
                        if ($disk_hash{$disk}{'pool'} =~ /Pool1/){
                                push (@pool1_disks,$disk);
                        }
                }
         }

         my %spare_pool_status = (
                                  'pool0' => \@pool0_disks,
                                  'pool1' => \@pool1_disks,
                                );
         return %spare_pool_status;
}

=head2 ::verify_test_disks_exist

It is to verify specific number of spare disks exist or not.
The format is as follows
        verify_test_disks_exist ($FILER_OBJ,$disks_needed)

=item Arguments

B<$FILER> => I<Filer object>

I<Required.>
A Filer NOTE object.

B<$disks> => I<number>

I<Required.>
A numeric.

=item Examples

=back

    verify_test_disks_exist ($FILER_OBJ,$disks_needed);

=head2 ::status

Returns 1 if exists other wise 0.

=over

=cut

sub verify_test_disks_exist {

        my ( $filer_obj , $disks_needed ) = @_;
        my %spare_disk_hash = get_spare_pool_status($filer_obj);
        my (@pool1_disks,@pool0_disks);
        @pool1_disks = @{$spare_disk_hash{pool1}};
        @pool0_disks = @{$spare_disk_hash{pool0}};
        my @disks_exists = (@pool1_disks,@pool0_disks);

        if (@disks_exists >= $disks_needed){
                return 1;
        }else{
                return 0;
        }
}

1;






1;
