
package SnapLock::Snaplock_auto_commit;


BEGIN {
   our @EXPORT = qw(
                 	set_vol_auto_commit
                 	set_filer_auto_commit
                 	get_filer_auto_commit
                   	get_vol_auto_commit
     			is_vol_auto_commit_scan_on
			get_count_possible_vol_inodes
			set_vol_for_max_file
			fill_vol_by_zerobyte_file
                        get_count_inode_commited
                        get_count_vol_worm_files
                        auto_commit_scanner_state
                        get_dst_qt_file_objs
                        get_dst_vol_file_objs
			set_sl_vol_retension
			is_qt_files_worm
			is_vol_files_worm
			set_vol_auto_commit_by_zapi
			get_vol_auto_commit_by_zapi
			revert_verify_snaplock
			set_autoCommit_existing_vol
			create_sl_test_vols
			local_filer_ping
			local_create_files
			wait_till_inode_commited
			license_delete_snaplock_compliance
			license_delete_snaplock_enterprise
			license_add_snaplock_compliance
			license_add_snaplock_enterprise
			get_file_by_name
                   );
}


use strict;
use NOTE;
use Tharn;
use TracePkg;
use SnapLock::Snaplock_datamanip;
use SnapLock::Helper;
use SnapLock::WORMLogging;
use Zapi;
use ZapiErrno;
use NaElement;
use XML::Simple;
use Data::Dumper;
use Filer::Version qw(get_wafl_version WV_ROLLINGROCK_MIN);
use config_base qw(host_is_alive);
use base qw(Exporter);



=head2 ::set_vol_auto_commit

Set the the volume auto-commit to a values.

The format is as follows
    set_vol_auto_commit ($vol_obj,$auto_commit_period)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

B<auto_commit_period>

I<Required>
A string

=item Examples

=back

    set_vol_auto_commit ($vol_obj,$auto_commit_period)

=head2 ::status

Returns 0 on success and return output on Failure.

=over

=cut

sub set_vol_auto_commit {

     my ($vol_obj,$auto_val,$exec_type) = @_;	
     my $vol_name = $vol_obj->name;
     my $filer_version = get_wafl_version($vol_obj->Filer->conn->{hostp});	

     if ($filer_version >= WV_ROLLINGROCK_MIN ){      
 	
	   if (!defined ($exec_type) || $exec_type eq 'CLI'){ 	
		my $cmd = "vol options"." $vol_name"." snaplock_autocommit_period"." $auto_val";
     		my $responce = $vol_obj->Filer->say ($cmd);
     		if ($responce){
        		return $responce;
     		}else {
        	 	return 0;
     		}
	  }
	  if ($exec_type eq 'ZAPI'){
		my $conn = $vol_obj->Filer->conn;
        	logcomment ("Setting Auto-commit Value using ZAPI");
        	my $zcmd = new NaElement("volume-set-option");
        	$zcmd->child_add(new NaElement("volume","$vol_name"));
        	$zcmd->child_add(new NaElement("option-name","snaplock_autocommit_period"));
        	$zcmd->child_add(new NaElement("option-value","$auto_val"));
        	$zcmd->child_add(new NaElement("volume","$vol_name"));
        	my $zapi = Zapi->new(
                                $conn->hostp()->default_ip(),
                                "root",
                                $conn->hostp()->default_password()
                             );
        	$zapi->{fail} = \&_fail_default;
        	my $zapi_exec = $zapi->invoke_elem($zcmd, "any");
        	sleep 5; #Wait for Auto-commit scanner to kick-off
		my $zapi_op  = $zapi_exec->toString();
                return $zapi_op;
	  }
      }else {
	
		my $cmd = "options snaplock.autocommit_period " . "$auto_val";
		my $responce = $vol_obj->Filer->say ($cmd);
		if ($responce){
                        return $responce;
                }else {
                        return 0;
                }
      }	
 
}



=head2 ::set_filer_auto_commit

Set the the filer auto-commit to a values.

The format is as follows
    set_filer_auto_commit ($vol_obj,$auto_commit_period)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

B<auto_commit_period> => I<string>

I<Required>
A string

=item Examples

=back

    set_filer_auto_commit ($vol_obj,$auto_commit_period)

=head2 ::status

Returns 0 on success and return output on Failure.

=over

=cut


sub set_filer_auto_commit {

     my ($vol_obj,$auto_commit_period,$exec_type) = @_;

     if (  !defined ($exec_type) || 
	   ( $exec_type  eq 'CLI')
     ){
	 my $cmd = "options snaplock.autocommit_period "
               . "$auto_commit_period";
     	 my $responce = $vol_obj->Filer->say ($cmd);

     	if ($responce){
        	  return $responce;
     	}else {
        	  return 0;
     	}
     }
     if ( $exec_type  eq 'ZAPI'){
	my $conn = $vol_obj->Filer->conn;
	logcomment ("Setting Filer Auto-commit Value using ZAPI");
     	my $zcmd = new NaElement("options-set");
	$zcmd->child_add(new NaElement("name","snaplock.autocommit_period"));
	$zcmd->child_add(new NaElement("value","$auto_commit_period"));
	my $zapi = Zapi->new(
                                $conn->hostp()->default_ip(),
                                "root",
                                $conn->hostp()->default_password()
                             );
                $zapi->{fail} = \&_fail_default;
                my $zapi_exec = $zapi->invoke_elem($zcmd, "any");
                sleep 5; #Wait for Auto-commit scanner to kick-off
		my $zapi_op  = $zapi_exec->toString();
                return $zapi_op;
     }			
	
}

=head2 ::get_vol_auto_commit

Get the the volume auto-commit to a value.

The format is as follows
    get_vol_auto_commit ($vol_obj)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

=item Examples

=back

    get_vol_auto_commit ($vol_obj)

=head2 ::status

Returns 0 on Failure and auto-commit value on pass.

=over

=cut


sub get_vol_auto_commit {

        my ($vol_obj,$exec_type) = @_;

	if (!defined ($exec_type) || $exec_type eq 'CLI'){ 	
		my $opts =  $vol_obj->options;
        	if ( defined ($opts->{'snaplock_autocommit_period'})){
              		return $opts->{'snaplock_autocommit_period'};
        	}else {
            		return 0;
        	}
	}
	
	if ( $exec_type eq 'ZAPI'){
	        my $conn = $vol_obj->Filer->conn;
        	my $vol_name = $vol_obj->name;
        	my $zcmd = new NaElement("volume-options-list-info");
        	$zcmd->child_add(new NaElement("volume","$vol_name"));
        	my $zapi = Zapi->new(
                                $conn->hostp()->default_ip(),
                                "root",
                                $conn->hostp()->default_password()
                             );
        	$zapi->{fail} = \&_fail_default;
        	my $zapi_exec = $zapi->invoke_elem($zcmd, "any");
        	my $zapi_op = $zapi_exec->toString();
        	my $xml = XML::Simple->new;
        	my $zapi_hash_ref = $xml->XMLin($zapi_op);
        	print Dumper($zapi_hash_ref);
        	logcomment ("Getting Auto-commit Value by ZAPI, when Vol is Created %s",
                	        $zapi_hash_ref->{'options'}{'volume-option-info'}{'snaplock_autocommit_period'}{'value'}
                    	);
        	my $auto_val = $zapi_hash_ref->{'options'}{'volume-option-info'}{'snaplock_autocommit_period'}{'value'};
        	return $auto_val;
	}
}

=head2 ::is_vol_auto_commit_scan_on

Check if the auto-commit scanner is running on the volume.

The format is as follows
    is_vol_auto_commit_scan_on($vol_obj)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

=item Examples

=back

    is_vol_auto_commit_scan_on ($vol_obj)

=head2 ::status

Returns 0 on Failure and 1 on pass.

=over

=cut

sub is_vol_auto_commit_scan_on {

        my ($vol_obj) = @_;
        my $cmd = "priv set diag ;wafl scan status -S -V ";
        my $vol_name = $vol_obj->name;
        $cmd = $cmd.$vol_name;

	foreach my $iteration (0..9){
        	my $responce = $vol_obj->Filer->say($cmd);
        	my @message_lines = split /\n/, $responce;
        	foreach my $line (@message_lines){
          		if ($line =~ /autocommit volume scan/){
            			return 1;
          		}
        	}
		logcomment ("Iteration - $iteration For checking  Auto commit scanner");
		sleep 1;
	}
	
        return 0;
}


=head2 ::get_count_possible_vol_inodes

Get the number of inode can be set on the volume.

The format is as follows
    get_count_possible_vol_inodes($vol_obj)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

=item Examples

=back

    get_count_possible_vol_inodes ($vol_obj)

=head2 ::status

Returns  number of inode.

=over

=cut

sub get_count_possible_vol_inodes {
   my ($vol_obj) = @_;

   my $cmd = "priv set diag ;maxfiles ";
   my $vol_name = $vol_obj->name;
   my $huge_number = "999999999999";
   my $max_inode_index = 8;
      $cmd  = $cmd . "$vol_name " . "$huge_number";
   my $responce = $vol_obj->Filer->say($cmd);
   my @message_lines = split /\n/, $responce;

   foreach my $line (@message_lines){
     if ($line =~ /maximum inode count/){
        my @strings = split / /,$line;
        return $strings[$max_inode_index];
     }
   }
}


=head2 ::set_vol_for_max_file

Set the volume to create max files on it by tuning the number of inode to max.

The format is as follows
    set_vol_for_max_file($vol_obj)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

=item Examples

=back

    set_vol_for_max_file ($vol_obj)

=head2 ::status

Returns  number of inode.

=over

=cut


sub set_vol_for_max_file {
  my ($vol_obj) = @_;
  my $cmd = "priv set diag ;maxfiles -f ";
  my $vol_name = $vol_obj->name;
  my $vol_i_count = get_count_possible_vol_inodes ($vol_obj);
      $cmd  = $cmd . "$vol_name " . "$vol_i_count";
  my $responce = $vol_obj->Filer->say($cmd);

  if ($responce =~ /new maximum number of files will be/){
      return 1;
  }else {
          return 0;
  }

}


=head2 ::fill_vol_by_zerobyte_file

Fill the vol with number of zero byte files.

The format is as follows
    fill_vol_by_zerobyte_file ($vol_obj,$file_count)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

B<file_count> => I<Number>

I<Required>
A Number.

=item Examples

=back

    fill_vol_by_zerobyte_file ($vol_obj,$file_count)

=head2 ::status

Returns  number of inode.

=over

=cut

sub fill_vol_by_zerobyte_file{

      my ($vol_obj,$file_count) = @_;
      my $cmd = "priv set diag ;mkfile 0 /vol/";
      my $vol_name = $vol_obj->name;
      my $file_name = '/zero_byte';
      $cmd = $cmd.$vol_name.$file_name." $file_count";
      my $responce = $vol_obj->Filer->say($cmd);

      if ($responce){
          return $responce;
      }else {
          return 0;
      }
}

#################################

sub get_count_inode_commited {

        my ($vol_obj) = @_;
        my $cmd = "priv set diag ;wafl scan status -S -V ";
        my $vol_name = $vol_obj->name;
        $cmd = $cmd.$vol_name;
        my $responce = $vol_obj->Filer->say($cmd);
        my @message_lines = split /\n/, $responce;

        foreach my $line (@message_lines){
          if ($line =~ /autocommit volume scan/){
                $line =~ /autocommit volume scan(.*)inodes converted/;
                my $commited_inodes_count = $1;
                $commited_inodes_count =~ s/^\s+//;
                $commited_inodes_count  =~ s/\s+$//;
            	return $commited_inodes_count;
          }
        }
        return -1;

}


sub get_count_vol_worm_files {

      my ($vol_obj) = @_;
      my $count = 0;

      $vol_obj->unexportfs;
      $vol_obj->exportfs( 'rw,nosuid,anon=0');
      my @file_objs = $vol_obj->nfs->children;
      foreach my $file_obj (@file_objs){
        	my $file_name = $file_obj->name;
        	if ($file_name !~ /\.snapshot/){
	   		my $response = $file_obj->ls('-l');
           		if ($response =~ /-r--r--r--/){
            			$file_obj->ls('-l');
            			$count++;
          		}
        	}
      }
      return $count;
}

sub auto_commit_scanner_state {
    my ($vol_obj,$file_count) =  @_;
    my $files_commited = get_count_inode_commited ($vol_obj);

    if ( ($file_count - $files_commited) < 10 ){
        return 'AUTO_COMMIT_FINISHED';
    }else {
        return $file_count - $files_commited;
    }
}


###################################

sub get_dst_qt_file_objs {
    my ($qt_obj,$ref_file_objs) = @_;
    my @dst_file_objs;
    my $vol_obj = $qt_obj->Volume;
    $vol_obj->unexportfs;
    $qt_obj->exportfs( 'rw,nosuid,anon=0');
    my @dst_qt_file_objs = $qt_obj->nfs->children;
    my @src_qt_file_objs = @{$ref_file_objs};
    foreach my $src_file_obj (@src_qt_file_objs){
        my $src_file_name = $src_file_obj->name;
        foreach my $dst_file_obj (@dst_qt_file_objs){
                my $dst_file_name = $dst_file_obj->name;
                if ( $dst_file_name =~ /$src_file_name/){
                  push (@dst_file_objs ,$dst_file_obj);
                }
        }
    }
        return @dst_file_objs;
}


sub set_sl_vol_retension {
   my ($vol_obj,$retension) = @_;
   my $vol_name = $vol_obj->name;

        if ($vol_obj->snaplock =~ /enterprise/){
                $vol_obj->Filer->say ("vol options $vol_name snaplock_minimum_period $retension");
        }

        if ( $vol_obj->snaplock =~ /compliance/){
                $vol_obj->Filer->say ("vol options $vol_name snaplock_maximum_period $retension");
        }
}


sub is_qt_files_worm {
        my (@file_objs)  = @_;
        my $count = 0;
        my $count_files = @file_objs;
	foreach my $iteration (0..9){
        	foreach my $file_obj (@file_objs){
           		$file_obj->ls('-l');
           		my $response = $file_obj->ls('-l');
           		if ($response =~ /-r--r--r--/){
             			$count++;
           		}
        	}
       		if ( $count_files == $count){
          		return 1;
       		}else {
          		return 0;
       		}
	  logcomment ("Waiting For Files to get Worm\n");		
	  sleep 2;
	}
}


sub is_vol_files_worm {

       my (@file_objs)  = @_;
       my $count = 0;
       my $count_files = @file_objs;
       foreach my $file_obj (@file_objs){
           my $file_name = $file_obj->name;
           if ($file_obj->is_worm == 0 ){
             $file_obj->ls('-l');
             $count++;
           }
        }
       if ( $count_files == $count){
          return 1;
       }else {
          return 0;
       }
}

sub get_dst_vol_file_objs {
    my ($vol_obj,$ref_file_objs) = @_;
    my @dst_file_objs;
    $vol_obj->unexportfs;
    $vol_obj->exportfs( 'rw,nosuid,anon=0');
    my @dst_vol_file_objs = $vol_obj->nfs->children;
    my @src_vol_file_objs = @{$ref_file_objs};
    foreach my $src_file_obj (@src_vol_file_objs){
        my $src_file_name = $src_file_obj->name;
        foreach my $dst_file_obj (@dst_vol_file_objs){
                my $dst_file_name = $dst_file_obj->name;
                if ( $dst_file_name =~ /$src_file_name/){
                  push (@dst_file_objs ,$dst_file_obj);
                }
        }
    }
        return @dst_file_objs;
}

sub get_filer_auto_commit {
        my ($vol_obj,$exec_type) = @_;
	my $index_autocommit_period = 4 ;

        if ( !defined ($exec_type) ||
              ($exec_type eq 'CLI')
        ){
                my $cmd = "options snaplock";
                my $responce = $vol_obj->Filer->say ($cmd);
                my @message_lines = split /\n/, $responce;
                foreach my $line (@message_lines){
                        if ($line =~ /snaplock.autocommit_period/){
                                my @strings = split / /,$line;
                                return $strings[3];
                        }
                }
        }
        if ( $exec_type eq 'ZAPI'){ 
		my $vol_name = $vol_obj->name; 
		my $conn = $vol_obj->Filer->conn;
		my $zcmd = new NaElement("options-get");
		$zcmd->child_add(new NaElement("name","snaplock.autocommit_period"));
		my $zapi = Zapi->new(
                                $conn->hostp()->default_ip(),
                                "root",
                                $conn->hostp()->default_password()
                             );
                $zapi->{fail} = \&_fail_default;
		my $zapi_exec = $zapi->invoke_elem($zcmd, "any");
                my $zapi_op = $zapi_exec->toString();
                my $xml = XML::Simple->new;
                my $zapi_hash_ref = $xml->XMLin($zapi_op);
                print Dumper($zapi_hash_ref);
		return $zapi_hash_ref->{'value'};
        }

}


sub set_autoCommit_existing_vol{

        my ($FILER_OBJ,$set_ac_val) = @_;
        my @vol_objs = $FILER_OBJ->Volumes;
        my $SL_VOL_EXIST = 0;

        foreach my $vol_obj (@vol_objs){

                if ($vol_obj->snaplock =~ /enterprise|compliance/){

                        set_vol_auto_commit ($vol_obj,$set_ac_val);
                        my $auto_val = get_vol_auto_commit($vol_obj);

                        if ($auto_val !~ /$set_ac_val/ ){
                                logcomment ("Unable to Set auto-commit value to $set_ac_val");
                                logresult ("FATAL","Cannot Proceed Further , Test requires all vol ac val to $set_ac_val");
                        }

                        $SL_VOL_EXIST = 1;
                }
        }

        return $SL_VOL_EXIST;
}

sub revert_verify_snaplock {

        my ($FILER_OBJ, $pre_rr_version, $verify_string) = @_;
        my $Revert_Query1 = '\(Yes\/No\)\?';
        my $Revert_Query2 = '\[yes\/no\]\?';
        my $Revert_Query3 = '\(y\/n\)\?';
        my $conn = $FILER_OBJ->conn;

        my (undef, $before, undef)
                 = say($conn, "revert_to -f $pre_rr_version",
                        qr/$Revert_Query1|$Revert_Query2|$Revert_Query3/,
                        -swallow,  -eval_error, -noprompt
                   );
        say($conn, "no", -eval_error, -noprompt);
        if ($before =~ /$verify_string/) {
                return 1;
        }else {
                return 0;
        }
}

sub create_sl_test_vols {

         my ( $FILER_OBJ, $sl_vol_type, $num_vols ) = @_;
         my @vol_objs;
         my %vol_params = (
                       Filer => $FILER_OBJ,
                       size => '30m',
                       snaplock => $sl_vol_type,
                   );

        foreach my $iteration (0..$num_vols){

                logcomment ("Creating the Vol $sl_vol_type ** $iteration");
                $vol_objs[$iteration] = create_volume(%vol_params);
        }

        return @vol_objs;
}

sub local_filer_ping {
        my ( $FILER_OBJ ) = @_;
        my $filer_conn_host =  $FILER_OBJ->conn->{hostp};
        if (!host_is_alive($filer_conn_host->default_ip(), 10)){
                logcomment("Filer is NOT Responding");
                return 1;
        }else {
                logcomment("Filer is Responding");
                return 0;
        }
}

sub local_create_files {

        my ($vol_obj,$file_prefix,$file_count,$file_size) = @_;
        my $vol_name = $vol_obj->name ;
        my $version = $vol_obj->Filer->version;
        my $TIMEOUT = 3000;
        my $response = say($vol_obj->Filer->conn,
                                "priv set diag; mkfile $file_size /vol/$vol_name/$file_prefix $file_count",
                                -timeout=>$TIMEOUT);
        sleep 4;
        return $response;

}


=head2 ::wait_till_inode_commited

Wait till specified count of inodes are commited.

The format is as follows
    wait_till_inode_commited ($vol_obj,$inode_count,$max_wait_time)

=item Arguments

B<vol_obj> => I<Volume object>

I<Required>
A Volume NOTE object.

B<inode_count> => I<Number>

I<Required>
A Number.

B<max_wait_time> => I<Number>

I<Required>
A Number in seconds, max timeout for all inode to commit
=item Examples

=back

    wait_till_inode_commited ($vol_obj,$inode_count,$max_wait_time);

=head2 ::status

Returns  1 if specified inodes are commited else returns 0.

=over

=cut

sub wait_till_inode_commited {
        my ($vol_obj,$i_count,$max_wait) = @_;

        my $iteration = 0;
        my $committed_files = 0;
        my $max_iteration = $max_wait / 3;
        do {
                $committed_files = get_count_inode_commited ($vol_obj);
                if ( $iteration == $max_iteration ){
                        last;
                }
                sleep 3;
                $iteration++;
        }while ($committed_files < $i_count);

        if ($iteration == $max_iteration ){
                return 0;
        }else {
                return 1;
        }
}

sub license_delete_snaplock_compliance {
        my ($FILER_OBJ) = @_;
        my $responce = $FILER_OBJ->say ("license delete snaplock");
        return $responce;
}

sub license_delete_snaplock_enterprise {
        my ($FILER_OBJ) = @_;
        my $responce = $FILER_OBJ->say ("license delete snaplock_enterprise");
        return $responce;
}

sub license_add_snaplock_compliance {
        my ($FILER_OBJ) = @_;
        my $responce = $FILER_OBJ->say ("license add KJHHIJE");
        return $responce;
}

sub license_add_snaplock_enterprise {
        my ($FILER_OBJ) = @_;
        my $responce = $FILER_OBJ->say ("license add AOXRXFF");
        return $responce;
}

sub get_file_by_name {

        my ($vol_obj , $file_name) = @_;

        my @file_objs = $vol_obj->nfs->children;
        foreach my $file_obj (@file_objs){
                if ($file_obj->name =~ /$file_name/){
                        return  $file_obj ;
                }
        }
        return 0;
}

1;
