# Hey emacs, this is basically -*- perl -*-
#
#Copyright (c) 2007 Network Appliance, Inc.
#All rights reserved.
#
# $Id: $
## @summary  Upgrade/Revert Library 
##
## @description
##     Upgrade/Revert library,determines whether its upgrade/revert by comparing the current ontap version
##     and the desired ontap version and calls appropriate upgrade/revert routine 
##     Includes extra functionality during revert which is not handled by the Rervert Library
##         - Deletes aggregate snapshots
##         - Deletes snaplock_enterprise license 
##         - Disables Snapmirror/Snapvault
##
## @usage
##    use ubs;
##    pass the Hash by reference with appropriate to_release and Filer to do_transform routine 
##
## @keywords snaplock compliance clock
##
## @dependencies
##    lib/Filer/Transform.pm
##    lib/Filer/Upgrade.pm
##    lib/Filer/Revert.pm
##
##
## @author dl-snaplock-qa (DRG QA)
## @author kasasrik
##
## @status review
## @pod here
##

=head1 NAME

Filer Upgrade/Revert Library Module

=head1 SYNOPSIS
    use SnapLock::UpgradeRevert;
    my %hash = (FILER=>$FILER,
                to_release=>"RironcityN_071031_2230"
               );
    do_transform(\%hash);

or 

    my %hash = (FILER=>$FILER,
                from_release=>'7.2.4.X8',
                to_release=>"RironcityN_071031_2230"
               );
    get_filer_to_desired_release(\%hash);
    do_transform(\%hash);

=head1 DESCRIPTION
    Upgrade/Revert library,determines whether its upgrade/revert by comparing the current ontap version
    and the desired ontap version and calls appropriate upgrade/revert routine 
    Includes extra functionality during revert which is not handled by the Rervert Library
        - Deletes aggregate snapshots
        - Deletes snaplock_enterprise license 
        - Disables Snapmirror/Snapvault

=head1 ROUTINES
do_transform
get_filer_to_desired_release

=cut


package SnapLock::UpgradeRevert;
use strict;
use base qw(Exporter);
use Cwd qw(getcwd);
use Carp qw(croak);
use Tharn;
use Filer::Version qw(codename_to_version);
use Filer::Upgrade;
use Filer::Revert;
use boot;
use filer;
use SPARSE::Util::Logger qw( logStep logError logWarn logDbg);
use NOTE;
use constant DOWNLOAD_TIMEOUT => 3600;

BEGIN
{
  our @EXPORT = qw(
    do_transform
    get_filer_to_desired_release
  );
}


=head2 compare_versions
    my $install = compare_versions($from_release,$to_release);

this routine  compares two version strings and compares the major versions 

returns whether its Upgrade/Revert/NOT Supported strings

Not supported if we try to compare 6.5 and 7.0

=over

=item from_release

(Required) : Version string (7.2.4X8 or RironcityN_071031_2230)

=item to_release

(Required) : Version string (7.2.4X8 or RironcityN_071031_2230)

=back

=for html <hr>

=cut


##
## @ Compare the ONTAP Major version number and  
## @ Convert the release names to ONTAP Version using codename_to_version routine 
## @ return UPGRADE / REVERT/ NOT_SUPPORTED
##
##

sub compare ($$)
{
    my ($from_release,$to_release) = @_;

    my ($from_release_short, $to_release_short);

    

    if($from_release !~ /milena/) {
        $from_release_short = codename_to_version($from_release);
        if ($from_release_short =~ /^(\d+\.\d+)\.*/){
            $from_release_short = $1;
        }
    } else {
          if($from_release =~ /ic/i){ $from_release_short='7.3'; }
          elsif($from_release =~ /gb/i){ $from_release_short='7.2'; }
          elsif($from_release =~ /7.2/i){ $from_release_short='7.2'; }
          elsif($from_release =~ /tt/i){ $from_release_short='7.1'; }
          elsif($from_release =~ /as/i){ $from_release_short='7.0'; }
    }


    if ($to_release !~ /milena/) {
       $to_release_short = codename_to_version($to_release);
       if ($to_release_short =~ /^(\d+\.\d+)\.*/){
           $to_release_short = $1;
       }
    }else {
          if($to_release =~ /ic/i){ $to_release_short='7.3'; }
          elsif($to_release =~ /gb/i){ $to_release_short='7.2'; }
          elsif($from_release =~ /7.2/i){ $from_release_short='7.2'; }
          elsif($to_release =~ /tt/i){ $to_release_short='7.1'; }
          elsif($to_release =~ /as/i){ $to_release_short='7.0'; }
    }

    my @from_release_split = split(/\./,$from_release_short);
    my @to_release_split = split(/\./,$to_release_short);

    #if($::DEBUG==1) 
    #{
        tstlog('-comment',"Original Code Name From_release::To_release => $from_release :: $to_release");
        tstlog('-comment',"Version From_release::To_release => @from_release_split :: @to_release_split");
    #}
    
    if ($from_release_split[0] =~ /$to_release_split[0]/ || 
                                  $to_release_split[0] =~ /$from_release_split[0]/) 
    {
        print "IN 1\n";
        if($from_release_split[1] <= $to_release_split[1]) { 
            return 'UPGRADE';
        } else { 
            return "REVERT"; 
        }
    }elsif ( $from_release_split[0] <= $to_release_split[0] )  { 
        print "IN 2\n";
	## Upgrade to BR and above release
	return 'UPGRADE';
    } elsif ( $from_release_split[0] >= $to_release_split[0] )  {
        print "IN 3\n";
	## Revert to lower release from BR.
	return "REVERT";
    } else {
        print "IN 4\n";
	return 'NOT_SUPPORTED' ;
    }
}



=head2 do_transform
    my %hash = (FILER => $FILER,
                from_release => '7.2.4X8',
                to_release   => 'RironcityN_071031_2230'
               );
    my $install = do_transform(\%$hash);

This routine gets the current filer version and 
if the current version of the filer and the to_release version are same then returns 1  

else, 
calls the compare_versions subroutine and based on the return value 

calls do_upgrade routine if return value is UPGRADE 
calls do_reverte routine if return value is REVERT 
dies if return value is NOT SUPPOURTED

returns 1 on sucess or dies on failure  

Supported for ONTAP 7.0 and above

=over

=item hash_reference

(Required) : 
    my %hash = (FILER => $FILER,
                from_release => '7.2.4X8',
                to_release   => 'RironcityN_071031_2230'
               );

=back

=for html <hr>

=cut


##
## @ Compares the From and To_release  
## @ Calls appropriate subroutine for upgrade/revert
## @ Deletes the aggregate snapshots before revert
##

sub do_transform ($)
{
    my ($input) = @_;

    my ($fh,$fc,$fc_console,$current_release,$to_release,$install);

    my $to_release = $input->{"to_release"};

    ## Construct the Filer Connection 
    $fh = host($input->{FILER});
    $fc = connect($::FILER_CONN);
    $fc_console = connect('#console', -timeout, $::TIMEOUT);
    $fc_console->timeout($::CMDTIMEOUT);

    if ( $fh->hosttype eq 'filer-simulator' ) {
        die "cannot run on simulator";
    }

    $input->{fc} = $fc_console;
    my $current_release = _get_filer_version($input->{fc});
    $input->{"from_release"} = $current_release;

    ## Compare the Current Version on Filer and to_release 
    if( $current_release ne $to_release ) {
        logStep("Filer Current ONTAP release : $current_release");
        logStep("Trying to Upgrade/Revert the Filer from : $current_release ====> $to_release");
	
        $install = compare($current_release,$to_release);
  
        ## Enable NFS incase it was turned off during failed previous runs
        _enable_nfs($input->{FILER});
        if ($install eq "UPGRADE") { 
            do_upgrade($input); 
        } elsif( $install eq "REVERT") {
             do_revert($input); 
        } else { 
            die "Unsupported Upgrade/Revert $to_release \n";
        }

        ## Compare the current version on filer and to_release
        ## croak if the versions are different
        $current_release = _get_filer_version($input->{fc});
        if($current_release ne $to_release) {
            croak "$install Failed \n The filer should have been booted with : $to_release but\n the".
                  "filer current ontap version is : $current_release";
        } else {
            logStep("$install Successful - Filer Current Ontap Version : $current_release ");
        } 
        
    } else {
        logStep("Filer is running the same version as to_release : $to_release");
    }

}



=head2 do_upgrade
    my %hash = (FILER => $FILER,
                from_release => '7.2.4X8',
                to_release   => 'RironcityN_071031_2230',
                fc           => $filer_connection_object
               );
    do_upgrade(\%$hash);

This routine uses Filer/Upgrade.pm 
Creates Upgrade object 
downloads the kernel
reboot the filer

returns 1 on sucess or dies on failure  

Supported for ONTAP 7.0 and above
Not Supported for x86-64 platforms

=over

=item hash_reference

(Required) : 
    my %hash = (FILER => $FILER,
                from_release => '7.2.4X8',
                to_release   => 'RironcityN_071031_2230',
                fc           => $filer_connection_object
               );

=back

=for html <hr>

=cut


##
##
## @ Upgrade subroutine
## @ Creates an Object for Filer::Upgrade with necessary parameters
## @ Call the Upgrade 
## @ boot the filer 
##
##
sub do_upgrade ()
{
    my ($data) = @_;

    my ($fc_console,$upgrade_version,$filer,$old_filer,$upgrade);

    $fc_console = $data->{'fc'};
    $upgrade_version = $data->{"to_release"};
    $filer = $data->{'FILER'};

    tstlog('-comment',"Inside Upgrade");

    ## Set the Global Param $::FILER to the filer we are using : 
    $old_filer = $::FILER;
    $::FILER = $filer;
   
    # Create Upgrade Object 
    $upgrade = Filer::Upgrade->new($fc_console,
                                      to_version => $upgrade_version,
                                      FILER => $filer,
                                     ) or die "cannot create upgrade object";
    $upgrade->do_upgrade(timeout => DOWNLOAD_TIMEOUT ) or die "upgrade failed";
    transit_to($fc_console, UP);
    logStep("Filer is rebooted after upgrade");
    $::FILER = $old_filer; 
}


=head2 do_revert
    my %hash = (FILER => $FILER,
                from_release => '7.2.4X8',
                to_release   => 'RironcityN_071031_2230',
                fc           => $filer_connection_object
               );
    do_revert(\%$hash);

This routine uses NOTE & Filer/Revert.pm Libraries

Aggregate snapshots are deleted
snaplock enterprise license will be deleted 
snapmirror and snapvault will be disabled

Creates Revert object 
downloads the kernel
execute revert_to
reboot the filer

returns 1 on sucess or dies on failure  

Supported for ONTAP 7.0 and above
Not Supported for x86-64 platforms

=over

=item hash_reference

(Required) : 
    my %hash = (FILER => $FILER,
                from_release => '7.2.4X8',
                to_release   => 'RironcityN_071031_2230',
                fc           => $filer_connection_object
               );

=back

=for html <hr>

=cut

##
## @ Revert Subroutine
## @ Creates and Object for Filer::Revert with necessary parameters
## @ Check whether revert is allowed ( revert is allowed to only two previous major releases )
## @ Install the ONTAP Kernel
## @ Do the Revert_to 
## @ booth the Filer
## @ Validate whether REVERT is succesfull
##
##

sub do_revert ()
{
    my ($data) = @_;

    my ($fc_console,$revert_to,$filer,$revert);
 
    $fc_console = $data->{'fc'};
    $revert_to = $data->{"to_release"};
    $filer = $data->{'FILER'};

    tstlog('-comment',"Inside REVERT, Reverting to : $revert_to");
    ## Delete the aggregate snapshots 
    _aggregate_snapshot_delete($data->{FILER});

    ## @step SETUP :
    ## Create a revert object;
    my $revert = Filer::Revert->new($fc_console, to_version => $revert_to,
                                    FILER => $filer 
                                   )  or die "Cannot create revert object";

    ## Check whether revert is allowed - Only two previous releases are allowed
    if ( $revert->allowed() ) {
        logStep("$revert->to_version is okay");
    } else {
        die "$revert->to_version is not okay";
    }

    ## Downgrade the filer to a supported older version and reboot the filer
    $revert->install_release or die "Could not initialize revert object";
    $revert->do() or die "could not perform the revert";
    transit_to($fc_console, UP);
    logStep("Filer is rebooted after down grade");

    if ( $revert->validate()) {
        logStep("Revert successful");
    } else {
       die "Revert unsuccessful";
    }
}


##
## @ Enable nfs on the filer before doing any Upgrade/Revert 
## @ Argument : Filer Name 
##
sub _enable_nfs($)
{
    my ($filer) = @_;
	
    my $filer_obj = NOTE::Filer->connect(host=>$filer);
    $filer_obj->say("nfs on");
    return;
}



##
##
## @ Function Name : _aggregate_snapshot_delete
## @ Argument : FilerName
## @ Connect to the Filer
## @ Get the Aggregates list 
## @ For each aggregate, delete all the snapshots
## @ Disable snapmirror/snapvault
## @ Delete the snaplock_enterprise license as ONTAP 7.0 supports only one license at a time
##
sub _aggregate_snapshot_delete($)
{
    my ($filer_name) = @_;

    my ($filer,@aggr_objs,$cmd,$full_cmd);
    	
    $filer = NOTE::Filer->connect(host=>$filer_name);
    eval {
        $filer->say("nfs on");
        $filer->say("license delete snaplock_enterprise");
        $filer->say("options snapmirror.enable off");
        $filer->say("options snapvault.enable off");
    };

    @aggr_objs = $filer->Aggregates;
    $cmd = 'snap delete -A -a -f '; 
    eval {	
        foreach my $aggr (@aggr_objs) {
            $aggr->online;
            $full_cmd = $cmd.$aggr->name;
            $filer->say($full_cmd);  
        }
    };
    die "Failed while deleting the Aggregate Snapshots $@" if($@) ;
}










##
## @ name :: get_filer_to_desired_release 
## @ Arguments : Hash Containing 'from_release'
## 
## @ Needs to be Called before starting the Upgrade/Revert test
## @ Calls do_transform to get the filer to desired 'from_release' 

sub get_filer_to_desired_release($)
{
    my ($data) = @_;
    
    NOTE::logcomment("Inside get_filer_to_desired_release");    
    my $desired_release = $data->{'from_release'};   
    
    logStep("Desired Filer ONTAP version before performing Upgrade/Revert is : $desired_release ");
    my %inhash = ( 
		  'to_release' => $desired_release,
		  'FILER'=> $data->{'FILER'},
                 );
    # Call do_transform with the the constructed hash
    do_transform(\%inhash);
    return 1;
}

##
## @ Get the Current Filer Version
## @ Argument : Filer Connection
##
sub _get_filer_version($)
{
    my ($fc) = @_;
    my $version = filer_version_get($fc);
    
    my $current_version_short = codename_to_version($version);
    if ( $version =~ m/(.*)<.*>/ ) {
        $version = $1;    # strip last <xx> charactesr
        return $version;
    }
}

##
## @ Convert the ONTAP release code name to version string
## @ Argument : ONTAP release code name
##
sub _filer_version_short($)
{
    my ($version1) = $_;
    my $version = $version1;
    my $version_short = codename_to_version($version);
    if ( $version =~ m/(.*)<.*>/ ) {
        $version = $1;    # strip last <xx> charactesr
        return $version;
    }
    croak "Not able to resolve the Codename to Version String\n";
}

1;


