#$Id:$

## @summary  File Fingerprint library
## @pod      here
## @status   Public
## @author   Abhishek Dubey ( abhishek.dubey@netapp.com, dl-snaplock-qa(DRG QA))

=head1 NAME

SnapLock::fingerprint

=head1 VERSION

$Id:

=head1 SYNOPSIS

use SnapLock::fingerprint

=head1 DESCRIPTION

These Library functions help in verifying the fingerprint
functionality.

=head1 METHODS

=cut



package SnapLock::fingerprint;




require Exporter;
our @ISA = qw(Exporter);
our @EXPORT =  qw(verify_fp_checksum
		  create_fp_options
		  get_file_paths_from_data_hash
		  execute_fingerprint
		  verify_fingerprint
		  compare_fingerprint
		  volume_fsid
		  file_id 
		  volume_uuid
		  create_setup_fingerprint
		  get_filer_version
		  wafl_scan_fold
		  file_reservation_enable
		  file_reservation_disable		  
		  sis_clone
		  get_data
		  asis_start
		  asis_stop
		  setup_ndmp
		 );

use strict;
use SayWrap;
use Tharn;
use NOTE;
use Zapi;
use ZapiUtil;
use ZapiErrno;
use NaElement;
use XML::Simple;
use MIME::Base64;
use Options;
use Data::Dumper;
use Tharn qw(say logcomment logresult);
use Tharn qw(host connect);
use Utils qw(make_host make_conn);
use TracePkg;
use constant DEFAULT_USER => "root";
use filer;
use Volume qw(vol_online);
use SnapLock::WORMLogging qw(create_volume) ;
use SnapLock::Snaplock_datamanip;
use Filer::Version qw(codename_to_version get_wafl_version WV_ROLLINGROCK_MIN) ;
use snaplock qw(compliance_clock_status);


use vars qw($VOL_TYPE);
param("VOL_TYPE", "-default", 'flex');



=head2 ::create_setup_fingerprint

Create the SLC,SLE and Regular aggregates and volumes.

=over

=item Arguments

B<Filer> => I<host spec>

I<Required.>
The filer on which to create the volume.

B<name> => I<string>

I<Required.>
specifies which type of snaplock volume to create.

I<optional.>
The size to make the volume.
Only valid for flexible volumes.

B<type> => C<flex> | C<trad>

I<Optional.>
The type of volume to create.
The default is C<flex>.

B<trad> => C<{> I<::create options> c<}>

=cut

sub create_setup_fingerprint($%) {

        my ($filer,$voltype) = @_;
        ##Create a hash for storing the volume details
        my %vol_data = (filer    => $filer,
                type     => 'flex',
                size     => '200m',
                snaplock => 'compliance',
        );

        NOTE::logcomment("Inside create_setup, Creating volumes");
	my $filer_conn = NOTE::Filer->connect(
		host => $filer,
		mode => '#rsh',
		status => 0
		);

	snaplock::snaplock_enterprise_license($filer_conn->conn);
	snaplock::compliance_clock_initialize($filer_conn->conn, strict=>0, reinit_ok=>1);

        my @volume_array;
        if ( $voltype eq 'reg' ) {
                logcomment("Creating Regular volumes");
                my @setup_array;

                ## Create flexible regular volume
                $vol_data{snaplock}='regular';
                logcomment("Creating flexible volume of type [$vol_data{snaplock}]");
                $vol_data{type}='flex';
                my $vol_setup_flex = create_volume(%vol_data);

                ## Create traditional regular volume
                logcomment("Creating traditional volume of type [$vol_data{snaplock}]");
                $vol_data{type}='trad';
                my $vol_setup_trad = create_volume(%vol_data);

                push(@setup_array, $vol_setup_flex);
                push(@setup_array, $vol_setup_trad);

                return (@setup_array);

        } elsif($::VOL_TYPE eq 'flex' ) {

                ## Create flexible enterprise volume
                $vol_data{snaplock}='enterprise';
                logcomment("Creating flexible volume of type [$vol_data{snaplock}]");
                $vol_data{type}='flex';
                my $sle_volume_flex =  create_volume(%vol_data);

                ## Create flexible regular volume
                $vol_data{snaplock}='regular';
                logcomment("Creating flexible volume of type [$vol_data{snaplock}]");
                $vol_data{type}='flex';
                my $reg_volume_flex = create_volume(%vol_data);

                ## Create flexible compliance volume
                $vol_data{snaplock}='compliance';
                logcomment("Creating flexible volume of type [$vol_data{snaplock}]");
                $vol_data{type}='flex';
                my $slc_volume_flex = create_volume(%vol_data);

                push(@volume_array, $sle_volume_flex);
                push(@volume_array, $slc_volume_flex);
                push(@volume_array, $reg_volume_flex);
                return (@volume_array);

        }elsif ( $::VOL_TYPE eq 'trad' ) {

                ## Create traditional enterprise volume
                $vol_data{snaplock}='enterprise';
                logcomment("Creating traditional volume of type [$vol_data{snaplock}]");
                $vol_data{type}='trad';
                my $sle_volume_trad =  create_volume(%vol_data);

                ## Create traditional regular volume
                $vol_data{snaplock}='regular';
                logcomment("Creating traditional volume of type [$vol_data{snaplock}]");
                $vol_data{type}='trad';
                my $reg_volume_trad = create_volume(%vol_data);


                ## Create traditional compliance volume
                logcomment("Creating traditional volume of type [$vol_data{snaplock}]");
                $vol_data{snaplock}='compliance';
                $vol_data{type}='trad';
                my $slc_volume_trad =  create_volume(%vol_data);

		push(@volume_array, $sle_volume_trad);
		push(@volume_array, $slc_volume_trad);
		push(@volume_array, $reg_volume_trad); 
		return (@volume_array);
        }else {
		logresult('FATAL', "Invalid Volume type : $::VOl_TYPE");
        }
}



=head2 ::get_file_paths_from_data_hash

Returns an array of file attributes from created file object

=item Arguments

B<DataHash> => I<DataHash>

I<Required.>

Array of the file objects

B<Volume> => I<Volume>

I<Required.>

Volume object

=over

=cut


sub get_file_paths_from_data_hash($$;$) {

	my ($DataHash, $vol, $qtree_path) = @_;
	my $Volume = $vol->name;
	my $path_qtree = $qtree_path;
	my $filer = $vol->Filer->conn;

	my @FilePaths;
	my $FilePath;
	my $volume_uuid;

	foreach my $Type ( keys %$DataHash ) {
		my $Data = $DataHash->{$Type};
		foreach my $File ( keys %$Data) {
			my $FileObj = $Data->{$File}->{obj};
			if (defined ($path_qtree)) {
				$FilePath = "/vol/".$path_qtree.$FileObj->path;
			} else {
				$FilePath = "/vol/".$Volume.$FileObj->path;
			}

			my $volume_uuid = volume_uuid($filer, $FilePath, $Volume);
			my $fsid = volume_fsid($filer , $FilePath);

			my %Hash = (
					file_name => $FileObj->path,
					file_path => $FilePath,
					file_atime => $FileObj->atime,
					file_mtime => $FileObj->mtime,
					file_ctime => $FileObj->ctime,
					volume_name => $Volume,
					file_obj => $FileObj,
					volume_fsid => $fsid,
					volume_uuid => $volume_uuid,	
				   );
			push(@FilePaths , \%Hash);
		}
	}
	return @FilePaths;
}

=head2 ::get_filer_version

Returns the filer version 

=item Auguments.

B<filer_conn> => I<filer_conn>

I<Required.>
Filer connection to execute commands on the filer.

=over

=cut

sub get_filer_version($) {
	my ($fc) = @_;
	my $fp_version;
	my $version = filer_version_get($fc);
	my $current_version_short = codename_to_version($version);
	if ( $current_version_short =~ /7.3/ ) {
		$fp_version = 1;
	}elsif ( $current_version_short > 7.3 ) {
		$fp_version = 2;
	}
	return ($fp_version);
}


=head2 ::execute_fingerprint

Execute the fingerprint using ZAPI interface or CLI interface.

The format is as follows
	execute_fingerprint( $filer , $path , $fp_options, %options );

=item Arguments

B<filer> => I<host spec>

I<Required.>
Filer on which test should run.

B<path> => I<path>

I<Required.>
Path of the file on which fingerprint has to be taken

B<fp_options> => I<fp_options>

I<Required.>
The fingerprint option 

B<options> => C<zapi>
Option whether fingerprint should be executed over CLI or ZAPI

=over

=cut


sub execute_fingerprint($$$%) {
	my ( $filer , $path , $fp_options, %options ) = @_;
        ## Get volume name from file path.
        my $ver = get_wafl_version($filer->{hostp});
        my @vol_split = split(/\// , $path);
        my $volume = $vol_split[2];
	if ( $options{use_zapi} ) {
		my $zcmd = new NaElement("file-get-fingerprint");
		$zcmd->child_add(new NaElement("path", $path));
		my $zapi = Zapi->new($filer->hostp()->default_ip(),
				DEFAULT_USER,
				$filer->hostp()->default_password());
		my $fp_output = $zapi->invoke_elem($zcmd,"any");
		if ( ! $fp_output->results_ok() ) {
			my $error  = "ZAPI file-get-fingerprint " . $fp_output->toString();
			return $error;
		}
  		if ( $ver >= WV_ROLLINGROCK_MIN ) {
                    my $vcc = compliance_clock_status($filer, $volume);
                    $fp_output->{volume_cc} = $vcc;
                    my $scc = compliance_clock_status($filer);
                    $fp_output->{system_cc} = $scc;
                }
		return ($fp_output);
	}
	return eval {

		my $data = sayEOF($filer, "file fingerprint $fp_options $path",{timeout=>7200});
		if ( $data =~ /<fingerprint><fingerprint-info>/ ) {
			my $XML_obj = new XML::Simple;
			my $fp_output = $XML_obj->XMLin($data);
  		        if ( $ver >= WV_ROLLINGROCK_MIN ) {
                             my $vcc = compliance_clock_status($filer, $volume);
                             $fp_output->{volume_cc} = $vcc;
                        }
			return ($fp_output);
		} else {
			return ($data);
		}
	}
}


=head2 ::volume_fsid

Returns the fsid of the volume

=item Auguments.

B<filer_conn> => I<filer_conn>

I<Required.>
Filer connection to execute commands on the filer.

B<file_path> => I<File_Path>

I<Required.>
Path of the file 

=over

=cut

sub volume_fsid ($$) {
	my ( $filer , $path ) = @_;
	my %filerfh = filer_showfh($filer, $path);
	my $fs_id = hex ($filerfh{"fsid"});
	return ($fs_id);
}	

=head2 ::file_id

Returns the file id / inode number of the file

=item Auguments.

B<filer_conn> => I<filer_conn>

I<Required.>
Filer connection to execute commands on the filer.

B<file_path> => I<File_Path>

I<Required.>
Path of the file

=over

=cut

sub file_id ($$) {
	my ( $filer , $path ) = @_;
	my %filerfh1 = filer_showfh($filer, $path);
	my $file_id = hex ($filerfh1{"fileid"});
	return ($file_id);
}

=head2 ::volume_uuid

Returns the volume uuid of the given volume.

=item Auguments.

B<filer_conn> => I<filer_conn>

I<Required.>
Filer connection to execute commands on the filer.

B<file_path> => I<File_Path>

I<Required.>
Path of the file

B<volume name> => I<volume name>

I<Required.>
Volume name 

=over

=cut


sub volume_uuid ($$$) {
	my ( $filer , $path , $volume_name) = @_;
	my $file_id_fp = file_id($filer , $path );
	my $uuid_vol = sayEOF($filer, "priv set -q advanced;inodepath -v $volume_name $file_id_fp");
	my @uuid_vol_1 = split(/\n/, $uuid_vol);
	foreach my $line ( @uuid_vol_1 ) {
		if ( $line =~ /UUID/ ) {
			logcomment ("line = $line");
			my @uuid_vol_1 = split(/ /, $line);
			my $uuid_vol_output = pop(@uuid_vol_1);
			return ($uuid_vol_output);
		}
	}
}

=head2 ::setup_ndmp

Executes the NDMP copy between two filers, and returns the failure/suceess

=item Arguments

B<Source volume>> => I<src_vol>

I<Required.>
Source volume object 

B<Destination volume> => I<dst_vol>

I<Required.>
Destination  volume object

=over

=cut


sub setup_ndmp($$) {

        my ($src_vol, $dst_vol) = @_;

	my $TIMEOUT = 86400;
	my $response;
        logcomment("Running NDMP copy");
	my $source_filer = $src_vol->Filer->conn->{hostp}->hostname;
	my $dest_filer = $dst_vol->Filer->conn->{hostp}->hostname;
	my $source_volume = $src_vol->name;
	my $dest_volume = $dst_vol->name;
        my $cmd = "ndmpcopy $source_filer:/vol/$source_volume $dest_filer:/vol/$dest_volume";
        (undef, undef, $response) = say($src_vol->Filer->conn, $cmd, -timeout, "$TIMEOUT");
        logresult('FATAL', "NDMP copy failed") unless ($response =~ /Transfer successful/);

        sleep 20;
}


=head2 ::asis_start

Enable and executes the SIS operation on the volume

=item Arguments

B<Filer> => I<host spec>.

I<Required.>
Filer on which test will run

B<volume_name> => I<volume_name>

I<Required.>
Volume on which SIS has to be executed

=over

=cut


sub asis_start ($$) {
	my ( $filer ,$volume_name) = @_;
	my $vol_path = "/vol/$volume_name";
	logcomment("Enabling ASIS on the volume $volume_name");
	my $sis_on_cmd = "sis on $vol_path";
	my ($return_text, $result_code) = filer_cli_say($filer, $sis_on_cmd);
        # No errror is return
        if( $result_code eq cmdOkReturn){
		logcomment ("ASIS is Enabled on the volume $volume_name successfully");
                #return $return_text;
        }else{
                die "$return_text\n";
        }

	sleep(10);
	logcomment("Starting SIS operation on the volume $volume_name");
	my $sis_start_cmd = "sis start $vol_path";
	my ($return_text, $result_code) = filer_cli_say($filer, $sis_start_cmd);
	# No errror is return
	if( $result_code eq cmdOkReturn){
		return $return_text;
	}else{
		die "$return_text\n";
	}
	sleep(10);
}

=head2 ::asis_stop

Disable the SIS and undo the already deduped files

=item Arguments

B<Filer> => I<host spec>.

I<Required.>
Filer on which test will run

B<volume_name> => I<volume_name>

I<Required.>
Volume on which SIS has to be disabled

=over

=cut


sub asis_stop ($$) {
	my ( $filer ,$volume_name) = @_;
	my $vol_path = "/vol/$volume_name";
	logcomment("Disabling ASIS on the volume $volume_name");
	my $sis_off_cmd = "sis off $vol_path";
        my ($return_text, $result_code) = filer_cli_say($filer, $sis_off_cmd);
        # No errror is return
        if( $result_code eq cmdOkReturn){
                #return $return_text;
		logcomment ("ASIS is Disabled on the volume $volume_name successfully");
        }else{
                die "$return_text\n";
        }

	sleep(10);
	logcomment("Undo SIS operation on the volume $volume_name");
	my $sis_undo_cmd = "priv set -q diag; sis undo $vol_path";
        my $result_code = say($filer,"$sis_undo_cmd" );
        # No errror is return
	$result_code eq "" or croak("$sis_undo_cmd failed: $result_code");
        sleep(10);
}


=head2 ::get_data

Returns an array of file attributes like path,atime.mtime etc from all created file object in the volume

=item Arguments

B<vol_obj> => I<volume object>

I<Required.>

The created volume object

=over

=cut


sub get_data($) {
    my ($vol_obj) = @_;

    my $volume_name = $vol_obj->name;
    my @files = $vol_obj->nfs->children;
    my $filer = $vol_obj->Filer->conn;
    my @file_paths = ();
    foreach my $file (@files) {
        if ( $file->name !~ /snapshot/ ) {
            if ( ! $file->is_dir ) {
		my $path_file = "/vol/".$volume_name.$file->path;
                my $new_file_path = $vol_obj->mount_path.$file->path;
		my $fsid = volume_fsid($filer , $path_file);
		my $volume_uuid = volume_uuid($filer, $path_file, $volume_name);
                my %hash = (
                        file_path => $new_file_path,
                        file_size => $file->size,
                        file_atime => $file->atime,
                        file_ctime => $file->ctime,
                        file_mtime => $file->mtime,
                        file_obj   => $file,
			volume_fsid => $fsid,
			volume_uuid => $volume_uuid,
                        );
                push(@file_paths,\%hash);
            }
        }
    }
    return @file_paths;
}

=head2 ::file_reservation_enable

Enables the space reservation on the file

=item Arguments

B<Filer> => I<host spec>.

I<Required.>
Filer on which test will run

B<File name> => I<File name>

I<Required.>
File path of the file

=over

=cut

sub file_reservation_enable ($$) {
	my ( $filer, $filename ) = @_;
	my $cmd = "file reservation $filename enable";
	my $text = say($filer, "priv set -q diag; $cmd");
	$text eq "" or croak("$cmd failed: $text");
	sleep(20);
}

=head2 ::file_reservation_disable

Disable the space reservation on the file

=item Arguments

B<Filer> => I<host spec>.

I<Required.>
Filer on which test will run

B<File name> => I<File name>

I<Required.>
File path of the file

=over

=cut

sub file_reservation_disable ($$) {
	my ( $filer, $filename ) = @_;
	my $cmd = "file reservation $filename disable";
	my $text = say($filer, "priv set -q diag; $cmd");
	$text eq "" or croak("$cmd failed: $text");
	sleep(20);
}

=head2 ::wafl_scan_fold

Run the wafl scan scanner on the file

=item Arguments

B<Filer> => I<host spec>.

I<Required.>
Filer on which test will run

B<File name> => I<File name>

I<Required.>
File path of the file

=over

=cut

sub wafl_scan_fold($$) {

	my ( $filer ,$filename) = @_;
	my $cmd = "wafl scan fold $filename";
	my $text = say($filer, "priv set -q diag; $cmd");
        $text eq "" or croak("$cmd failed: $text");
        sleep(30);
}

=head2 ::sis_clone

Executes the sis clone on the file from regular volume

=item Arguments

B<Filer> => I<host spec>.

I<Required.>
Filer on which test will run

B<File name> => I<File name>

I<Required.>
File name 

B<Vol>  => I<Vol>

I<Required.>
Volume object 

=over

=cut


sub sis_clone($$$) {

	my ( $vol, $filer ,$filename) = @_;
	my $clone_file = $filename."_clone";
	my %file_hash;
	my $cmd = "clone start $filename $clone_file";
	my $text = say($filer, "priv set -q diag; $cmd");
	$text eq "" or croak("$cmd failed: $text");
	sleep(30);
	return ($clone_file);
	my @files_in_volume = get_data($vol);
	foreach my $existing_file ( @files_in_volume ) {
		if ( $existing_file->{file_path} eq $clone_file ) {
			my %file_hash = (
					file_path => $clone_file,
					file_atime => $existing_file->atime,
					file_ctime => $existing_file->ctime,
					file_mtime => $existing_file->mtime,
					);
		}
		return (\%file_hash);
	}
}


=head2 ::create_fp_options

Create different options for fingerprint CLI.
Returns a hash with all the fingerprint options contained in it.

=item Arguments

B<args> => I<fingerprint option argument>

I<optional.>

The option for the fingerprint CLI
If absent the fingerprint cli will not take any option and will show 
both data and metadata fingerprint in the output

=over

=cut

sub create_fp_options(;%) {
	my (%args ) = @_;

	my %fp_options;

	if (!defined($args{options})) {
		%fp_options = (
				option_xml_fp => '-x',
				option_sha_xml => '-a sha-256 -x',
				option_md5_algo => '-a md5 -x',
			      );
	} elsif  ( $args{options} eq "-d" ) {
		%fp_options = (
				option_xml_fp => '-d -x',
				option_sha_xml => '-d -a sha-256 -x',
				option_md5_algo => '-d -a md5 -x',
			      );
	} elsif  ( $args{options} eq "-m" ) {
		%fp_options = (
				option_xml_fp => '-m -x',
				option_sha_xml => '-m -a sha-256 -x',
				option_md5_algo => '-m -a md5 -x',
			      );
	}

	return (\%fp_options);
}


=head2 ::verify_fp_checksum

Verifies that fingerprint generated by ONTAP is same as the standard md5 algo generated fingerprint.

the format is as follows
	verify_fp_checksum( $filer_src1, $client_conn, $volname, $filename , $output_md5)

=item Arguments

B<Filer> => I<host spec>

I<Required.>
The filer on which to create the volume.

B<client> => I<client host spec>

I<Required.>
Hostspec of the unix client

B<volume> => I<Volume>
The volume name on which the file exists

B<File> => I<File>
The file name on which fingerprint has to be calculated

B<fp_output> => I<fp_output>

I<Required.>
The fingerprint output in a hash.

=over

=cut


sub verify_fp_checksum ($$$$$) {
	my ( $filer_src1, $client_conn, $volname, $filename , $output_md5) = @_;
	my $op;
	return eval {
		my $path = "/vol/$volname";
		#my $filename = $file->{file_name};
		ExportFS::export_path($filer_src1, $path,
				OPTIONS => "rw,anon=0,nosuid",
				WRITE_ETC => 1);
		my $mpt = mount::mount($filer_src1->hostp()->default_ip(),
				"/vol/$volname",
				mountopts => $::MOUNT_OPTS,
				chmod => "0777");
		my $file_path = "$mpt"."$filename";
		my $md5sum = sayEOF($client_conn, " md5sum $file_path");
		my $md5_binary = sayEOF($client_conn, "openssl md5 -binary $file_path > file_output");
		my $md5_base64 = sayEOF($client_conn, "openssl base64 -in file_output");
		logcomment("Linux md5sum updated output for fingerprint in base 64 is $md5_base64");	
		if ( $output_md5->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"data-fingerprint"} eq $md5_base64 )  {
			logcomment("ONTAP generated fingerprint matched with the standard md5 algorithm generated fingerprint");
		} else {
			logcomment('FAIL', "ONTAP generated fingerprint doesn't match with the standard md5 algorithm generated fingerprint");
		}
	}
}

=head2 ::verify_fingerprint

Verifies the ZAPI and CLI fingerprint output.

the format is as follows
        verify_fingerprint( $filer, $fp_output , $file, %options);

=item Arguments

B<Filer> => I<host spec>

I<Required.>
The filer on which to create the volume.

B<fp_output> => I<fp_output>

I<Required.>
The fingerprint output in a hash.

B<options> => C<zapi>
Option whether fingerprint should be executed over CLI or ZAPI

=over

=cut


sub verify_fingerprint ($$$%) {

	my ( $filer, $fp_output , $file, %options) = @_;
	my  $uuid_vol_output;
	if ( $options{use_zapi} ) {

		## ZAPI verification
		NOTE::logcomment ("Verifying the Zapi output of fingerprint");
 		my $fp_output_z1 = $fp_output->child_get("fingerprint");
		my $fp_output_z2 = $fp_output_z1->child_get("fingerprint-info");
		my $fp_output_z3 = $fp_output_z2->child_get("metadata-files");
		my $fp_output_z4 = $fp_output_z3->child_get("file-fingerprint-info");

		my $filer_name =  sayEOF($filer, "hostname");
		my $IC_fp_version = get_filer_version($filer);

		if ($fp_output_z2->child_get("filer-name")->{content} eq $filer_name) {
			logcomment("Filername in fingerprint output is same as the filer name");
		}else {
			logresult('FAIL', "Filer name in fingerprint output is not same as the filer name");
		}
		if ($fp_output_z2->child_get("fingerprint-version")->{content} eq $IC_fp_version ) {
			logcomment("Fingerprint version is correct in the output");
		}else {
			logresult('FAIL', "Fingerprint version is not correct in the output");
		}
		if ($fp_output_z2->child_get("fingerprint-input-path")->{content} eq $file->{file_path} ) {
			logcomment("File path printed in fingerprint outout is correct");
		}else {
			logresult('FAIL', "File path printed in fingerprint outout is not correct");
		}
		if ($fp_output_z4->child_get("fsid")->{content} eq volume_fsid($filer , $file->{file_path} ) ) {
			logcomment("File fsid in fingerprint output is correct");
		}else {
			logresult('FAIL', "File fsid in fingerprint output is not correct");
		}
		my $volume_name = $fp_output_z2->child_get("volume-name")->{content};
		if ($fp_output_z2->child_get("volume-uuid")->{content} eq volume_uuid($filer, $file->{file_path}, $volume_name) ) {
			logcomment ("Volume UUId in fingerprint output is same as volume uuid ");
		}else {
			logresult('FAIL', "Volume UUId in fingerprint output is not same as volume uuid");
		}
		if ($fp_output_z4->child_get("modified-time")->{content}  eq $file->{file_mtime}) {
			logcomment("File modified-time in fingerprint is same as file mtime");
		}else {
			logresult('FAIL', "File modified-time in fingerprint is not same as file mtime");
		}
		if ($fp_output_z4->child_get("changed-time")->{content}  eq $file->{file_ctime}) {
			logcomment("File Changed time in fingerprint is same as file ctime");
		}else {
			logresult('FAIL', "File Changed time in fingerprint is not same as file ctime");
		}
		my $file_state = "regular";
		if ($fp_output_z4->child_get("file-type")->{content}  eq "$file_state") {
			if ($fp_output_z4->child_get("access-time")->{content}  eq $file->{file_atime}) {
				logcomment("File access time in fingerprint is same as file atime");
			}else {
				logresult('FAIL', "File access time in fingerprint is not same as file atime");
			}
		}else {
			## "retention-time" & "access-time" time entry will not be there if the file retention is "infinite". So Including "formatted-retention-time" to verify the infinite retention.
			if (($fp_output_z4->child_get("retention-time")->{content}  eq $file->{file_atime}) or ($fp_output_z4->child_get("formatted-retention-time")->{content} eq $file->{file_atime})) {
				logcomment("File retention time in fingerprint is same as file atime");
			}else {
				logresult('FAIL', "File retention time in fingerprint is not same as file atime");
			}
		}

  	        my $ver = get_wafl_version($filer->{hostp});
                if ( $ver >= WV_ROLLINGROCK_MIN ) {
                    my $vcc_diff = $fp_output->{volume_cc} - $fp_output_z2->child_get("snaplock-volume-compliance-clock")->{content} ;
		    ## increasing the vcc diff to 20 seconds to verify the difference on all the situations. ( different platforms , filer under load, rsh delays etc... )
                    logcomment("VCC diff = $vcc_diff");
                    if ( $vcc_diff == 0 or $vcc_diff < 20 ) {
                        logcomment("VCC on fingerprint output matches with VCC taken from 'snaplock clock status' command");
                    } else {
                        logresult('FAIL',"VCC on fingerprint output does not match with VCC taken from 'snaplock clock status' command");
                    }
                    my $scc_diff = $fp_output->{system_cc} - Date::Manip::UnixDate($fp_output_z2->child_get("formatted-snaplock-volume-compliance-clock")->{content} , "%s" );
                    if ( $scc_diff == 0 or $scc_diff < 20 ) {
                        logcomment("SCC on fingerprint output matches with SCC taken from 'snaplock clock status' command");
                    } else {
                        logresult('FAIL',"SCC on fingerprint output does not match with SCC taken from 'snaplock clock status' command");
                    }

                }


                
	}else {

		## CLI verification
		NOTE::logcomment ("Verifying the CLI output of fingerprint");
		my $filer_name =  sayEOF($filer, "hostname");
		if ( $fp_output->{"fingerprint-info"}->{"filer-name"} eq $filer_name ) {
			logcomment("Filername in fingerprint output is same as the filer name");
		}else {
			logresult('FAIL', "Filer name in fingerprint output is not same as the filer name");
		}
		if ( $fp_output->{"fingerprint-info"}->{"fingerprint-version"} eq get_filer_version($filer) ) {
			logcomment("Fingerprint version is correct in the output");
		}else {
			logresult('FAIL', "Fingerprint version is not correct in the output");
		}
		if ( $fp_output->{"fingerprint-info"}->{"fingerprint-input-path"} eq $file->{file_path} ) {
			logcomment("File path printed in fingerprint outout is correct");
		}else {
			logresult('FAIL', "File path printed in fingerprint outout is not correct");
		}
		if ( $fp_output->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"fsid"} eq  $file->{volume_fsid}) {
			logcomment("File fsid in fingerprint output is correct");
		}else {
			logresult('FAIL', "File fsid in fingerprint output is not correct");
		}
		my $volume_name = $fp_output->{"fingerprint-info"}->{"volume-name"};
		if ( $fp_output->{"fingerprint-info"}->{"volume-uuid"} eq $file->{volume_uuid}) {
			logcomment ("Volume UUId in fingerprint output is same as volume uuid ");
		}else {
			logresult('FAIL', "Volume UUId in fingerprint output is not same as volume uuid");
		}

		if ( $fp_output->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"modified-time"} eq $file->{file_mtime} )  {
			logcomment("File modified-time in fingerprint is same as file mtime");
		}else {
			logresult('FAIL', "File modified-time in fingerprint is not same as file mtime");
		}
		if ( $fp_output->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"changed-time"} eq $file->{file_ctime} )  {
			logcomment("File Changed time in fingerprint is same as file ctime");
		}else {
			logresult('FAIL', "File Changed time in fingerprint is not same as file ctime");
		}
		## "retention-time" & "access-time" time entry will not be there if the file retention is "infinite". So Including "formatted-retention-time" to verify the infinite retention.
		if ( $fp_output->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"file-type"} ne "regular") {
			if (( $fp_output->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"retention-time"} eq $file->{file_atime} ) || ( $fp_output->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"access-time"} eq $file->{file_atime} ) or ( $fp_output->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"formatted-retention-time"} eq $file->{file_atime} ) ) {
				logcomment("File retention/access time in fingerprint is same as file atime");
			}else {
				logresult('FAIL', "File retention/access time in fingerprint is not same as file atime");
			}
		} 
		my $ver = get_wafl_version($filer->{hostp});
   		if ( $ver >= WV_ROLLINGROCK_MIN ) {
                    my $vcc_diff = $fp_output->{volume_cc} - $fp_output->{"fingerprint-info"}->{"snaplock-volume-compliance-clock"} ;
		    logcomment("VCC diff = $vcc_diff");
		    ## increasing the vcc diff to 20 seconds to verify the difference on all the situations. ( different platforms , filer under load, rsh delays etc... )
    		    if ( $vcc_diff == 0 or $vcc_diff < 20 ) {
			logcomment("VCC on fingerprint output matches with VCC taken from 'snaplock clock status' command");
		    } else {
			logresult('FAIL',"VCC on fingerprint output does not match with VCC taken from 'snaplock clock status' command");
		    }
		    my $scc_diff = $fp_output->{system_cc} - Date::Manip::UnixDate($fp_output->{"fingerprint-info"}->{"formatted-snaplock-volume-compliance-clock"} , "%s" );
                    if ( $scc_diff == 0 or $scc_diff < 20 ) {
			logcomment("SCC on fingerprint output matches with SCC taken from 'snaplock clock status' command");
		    } else { 
			logresult('FAIL',"SCC on fingerprint output does not match with SCC taken from 'snaplock clock status' command");
		    }

                    logresult('FAIL' , "snaplock-compliance-clock and formatted-snaplock-compliance-clock should not exists in fingerprint taken from RR Release" ) if (defined $fp_output->{"fingerprint-info"}->{"snaplock-compliance-clock"} or defined $fp_output->{"fingerprint-info"}->{"formatted-snaplock-compliance-clock"} );
		}
	}
}


=head2 ::compare_fingerprint

Compares the fingerprint outputs and returns the array of field name which
did not match in the metadata fingerprint comparison

The format is as follows
compare_fingerprint ( Fingerprint output before ,Fingerprint output after );

=item Arguments

B<fp_output_before> => I<fp_output_before>

I<Required.>
The fingerprint output taken initially

B<fp_output_after> => I<fp_output_after>

I<Required.>
The fingerprint output taken after operation.

=over

=cut


sub compare_fingerprint ($$) {

        my ( $fp_output_before , $fp_output_after ) = @_;
        my @changed_field;

        NOTE::logcomment ("Comparing two fingerprint outputs");

        if (defined ($fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"data-fingerprint"} )) {
                if ( $fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"data-fingerprint"} eq $fp_output_after->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"data-fingerprint"} ) {
                        logcomment ("Data fingerprint matched in both the fingerprint outputs");
                }else {
                        logcomment("Data fingerprint is not same in both the fingerprint outputs ");
                }
        }

        if (defined ($fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"metadata-fingerprint"} )) {
                if ( $fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"metadata-fingerprint"} eq $fp_output_after->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"metadata-fingerprint"} ) {
                        logcomment ("Metadata fingerprint matched in both the fingerprint outputs");
                }else {
                        if ( $fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"modified-time"} ne $fp_output_after->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"modified-time"} ) {
                                my $changed_mtime = "modified-time";
                                push (@changed_field , $changed_mtime);
                        }
                        if ( $fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"changed-time"} ne $fp_output_after->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"changed-time"}  ) {
                                my $changed_ctime = "changed-time";
                                push (@changed_field , $changed_ctime);
                        }
                        if ( ( $fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"retention-time"} ne $fp_output_after->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"retention-time"} ) || ( $fp_output_before->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"access-time"} ne $fp_output_after->{"fingerprint-info"}->{"metadata-files"}->{"file-fingerprint-info"}->{"access-time"}  ) ) {
                                my $changed_atime = "access/retention time";
                                push (@changed_field , $changed_atime);
                        }
                        logcomment("Metadata fingerprint is not same in both the fingerprint outputs");
                }
        }
        return (@changed_field);
}


