# -*-perl-*x
# $Id: //depot/prod/test/Rsierranevada/lib/Space_count_blocks.pm#1 $
#
#
# Copyright (c) 2008 Network Appliance, Inc.
# All rights reserved.
#
# This is C-Mode Library for Space Management which uses Server.pm
#
## @summary
##     Library functions for counting number of blocks by providing file size
##
## @description
##
##     count_file_blocks - finds out number of blocks (Level 0 and Level 1) based on file size (fs_size)
##                         and aggregate block type (32-bit or 64-bit)
##
## @usage
##     use Space_count_blocks;
##
## @author Hardikkumar Darji (darji@netapp.com)
##
## @status public

package Space_count_blocks;

use Carp qw(croak);
use Tharn;
use Params::Validate qw (validate validate_pos SCALAR OBJECT ARRAYREF);
use Server;
use POSIX; # for ceil()
use Data::Dumper;

use vars ('@ISA','@EXPORT');
@ISA = qw(Exporter);
@EXPORT = qw(count_file_blocks count_file_blocks_basic);

## @name ::count_file_blocks(server=>$server,fs_size=> $file_size,aggregate=>$FILER_AGGR)
##
## @summary
#i     calculates number of blocks considering fs_size (file size) and aggregate block_type
## @description
##
##
## @arg server Required: filer connection object.
## @arg fs_size Required: size of file.
## @arg aggregate Required: name of aggregate (to find out agregate block_type).
##
##
## @return value
##      Returns errcode = 0 - Success
##                      < 0 - Failure
##              result = On Success, number of blocks.
##                     = On Failure, error message.
##
sub count_file_blocks(%)
{
     my %opts = validate(
  @_,
  {
    server => { type => OBJECT },
    aggregate => {type => SCALAR},
    fs_size => {type => SCALAR}

  }
    );


    my $server = $opts{server};
    my $aggregate= $opts{aggregate};
    my $filesize = $opts{fs_size};
    my $result;
    my $errcode = 0;
    my $aggr_type;


    my $res;
    eval
    {
        $res = $server->storage_aggregate_show(aggregate=>$aggregate);
    };
    if($@)
    {
        $errcode=-1;

        return ($errcode,"aggregate_show was not successful: $@")
    }
    else
    {
     logcomment("====== aggregate show output is:\n" . Dumper(\$res));

     ## get the aggregate_type for calculating blocks
          foreach (keys %$res)
          {
            if($_ eq "block-type")
             {

                $aggr_type=$res->[0]->{$_};
             }

          }
      ## calculating blocks based on block_type (32-bit or 64-bit) of aggregate

          my ($blocks,$count_blocks);
          logcomment("starting block count with $filesize bytes");
          my $l0_blocks=($filesize-($filesize%4096))/4096;
          if ( $filesize%4096 > 0 ) {
              $l0_blocks=$l0_blocks+1;
          }
          $blocks = $count_blocks = $l0_blocks;
          if ( $aggr_type eq "32-bit") {
              while ( $blocks > 8 ) {
                  $count_blocks=$count_blocks+ceil($blocks/510);
                  $blocks=ceil($blocks/510);
              }
          }
          if ( $aggr_type eq "64-bit") {
              while ( $blocks > 5 ) {
                  $count_blocks=$count_blocks+ceil($blocks/255);
                  $blocks=ceil($blocks/255);
              }
          }
          $errcode=0;
          return ($errcode,$count_blocks);

    }

}

## @usage: count_file_blocks_basic(fs_size=> $file_size,aggr_type=>$aggregate_type)
##
## @summary: calculates number of blocks WAFL will use given $fs_size and $aggr_type
##
## @description
## This lib function returns buff tree lvl for given size. Calling function needs to
## indicate which block type to use ('64' or '32' bit types) for given calculation.
##
## @arg fs_size Required: size of file.
## @arg aggregate_type Required: agregate block_type.
##
##
## @return value
##                     = number of blocks & buff tree lvl.
##

sub count_file_blocks_basic (%)
{

    my %opts = validate(
    @_,
    {
    fs_size => {type => SCALAR},
    aggr_type => {type => SCALAR}
    }
    );

    my $filesize=$opts{fs_size};
    my $aggr_type=$opts{aggr_type};
    my ($blocks,$count_blocks);
    ## for smaller files why we add 1 block
    logcomment("starting block count with $filesize bytes");
    my $l0_blocks=($filesize-($filesize%4096))/4096;
    my $lvl = 0;
    if ( $filesize%4096 > 0 ) {
        $l0_blocks=$l0_blocks+1;
    }
    $blocks = $count_blocks = $l0_blocks;
    if ( $aggr_type == "32") {
        while ( $blocks > 8 ) {
            $count_blocks=$count_blocks+ceil($blocks/510);
            $blocks=ceil($blocks/510);
            $lvl = $lvl + 1;
        }
    }
    if ( $aggr_type == "64") {
        while ( $blocks > 5 ) {
            $count_blocks=$count_blocks+ceil($blocks/255);
            $blocks=ceil($blocks/255);
            $lvl = $lvl + 1;
        }
    }
    return ($count_blocks,$lvl);
}