# -*-perl-*x
# $Id: //depot/prod/test/Rsierranevada/lib/Space_stats.pm#1 $
# 
#
# Copyright (c) 2008 Network Appliance, Inc.
# All rights reserved.
#
# This is C-Mode Library for Space Management
#
## @summary
##     Library functions for parsing "stats show" output
##
## @description
##
##     stats_show - Calls 'stats show' to check the value of space counters for volume,aggregate or quota.
##     
##
## @usage
##     use Space_stats;
##
## @author Hardikkumar Darji (darji@netapp.com) 
##
## @status public

package Space_stats;

use Carp qw(croak);
use Tharn;
use Params::Validate qw (validate validate_pos SCALAR OBJECT ARRAYREF);
use Server;
use Data::Dumper;

use vars ('@ISA','@EXPORT');
@ISA = qw(Exporter);
@EXPORT = qw(stats_show);

## @name ::stats_show($server,$type,$instance,$counter)
##
## @summary
#i     Calls statistics_show to check the value of space counters for volume,aggregate or quota
## @description
##     
##
## @arg $server Required: filer connection object.
## @arg $type Required: volume,aggregate or quota.
## @arg $instance Required: name of volume,aggregate or quota.
## @arg $counter Required: name of counter.
##
## @return value
##      Returns errcode = 0 - Success
##                      < 0 - Failure
##              result = On Success, the value of the counter.
##                     = On Failure, error message.
##                          
sub stats_show(%)
{
     my %opts = validate(
	@_,
	{
		server => { type => OBJECT },
		type => {type => SCALAR, default=>'volume'},
		instance => {type => SCALAR},
                counter => {type => SCALAR}
	}
    );
     
     
    my $server = $opts{server};
    my $type= $opts{type}; 
    my $instance = $opts{instance};
    my $counter = $opts{counter};
    my $node = $server->get_execution_server();
    
    my $result;
    my $errcode = 0;
        
    ## check for invalid type of object 
    if($type ne 'volume' && $type ne 'aggregate' && $type ne 'quota')
     {
          $errcode= -1;
          $result= "Object type is not correct";
          return ($errcode, $result);
     }
     
    ## set privilege level to "test" 
    $server->set(privilege=>"test");
    
    ## execution of stats show command
    my $res;
    eval
    {
        $res = $server->statistics_show(node=>$node,category=>$type,object=>$instance,counter=>$counter);
    };
    if($@)
    {
        $errcode=-1;
        
        return ($errcode,"Error: statistics_show was not successful:$@")
    }
    else
    {
     logcomment("====== statistics show output is:\n" . Dumper(\$res));
      foreach (keys %$res)
      {
        if($_ eq "value")
         {
            
            return($errcode,$res->[0]->{$_});
         }
   	
       }  
    }
}

1;



