#!/usr/local/bin/perl
#--------------------------------------------------------------------------
# $Id: //depot/prod/test/Rsierranevada/lib/Step/sutils.pm#1 $
#
# Copyright (c) 2005 Network Appliance, Inc.
# All rights reserved.
##
## @summary 	library functions to be used by NATE tests by STeP 
## @status	public      return 1; 
## @author 	dl-step-internal@netapp.com, dl-sar-qa@netapp.com
#--------------------------------------------------------------------------
require "tharnit_lib.pl";

use strict;
use warnings;

#print "My \@INC is @INC\n";
#
# Export subroutine calls for external use
#
use Exporter;
our (@EXPORT, @ISA);
@ISA = qw(Exporter);
@EXPORT = qw(
	     sutils_cf_disks_get
	     sutils_check_log_for_message
	     sutils_disk_offline
	     sutils_disk_online
	     sutils_get_month
	     sutils_get_timestamp	
	     sutils_is_simulator
	     sutils_log_msg
	     sutils_mount_volume
	     sutils_disks_get
	     sutils_shutdown
	     sutils_spares_filter
	     );
		
#
# use statements 
#
use Tharn;
use boot; 

use vars qw ($FILER
	     $FILERA
	     $FILERB
	     $CLIENT
	     $TEST_MODE
	     $VERBOSE
	    );
use lib "/rlse/p4/ontap/mainN/test/";
#
# ATI base modules
#
use ATI::ati_ontapi qw(ATI_SUCCESS
		       ATI_ONTAPI_PRIVILEGE_DIAG
		       ati_ontapi_create_context
		       ati_ontapi_call_api
		       ati_ontapi_call_cli);

#define constants
use constant SAY_TIMEOUT => 240;

#
# parameters
#
my $myhost = host($FILER);
my $filer = $myhost->hostname();
my @out;
#
# Create an ATI context for the filer connection with explicitly
# specified parameters (rather than taking the defaults)
#
my $context = ati_ontapi_create_context(filer => $filer);

sub sutils_client_saycmd($$) {
    my $ch = host($_[0], -fatal, "saycmd failed to allocate $_[0]");
    my $cc = connect("#rsh", -fatal, "connection failed");  
    say $cc, $_[1], -fatal, "$_[1]: failed"; 
}

## @name ::sutils_disk_offline
## @summary Offline the specified disk 
## @description
## sutils_disk_offline takes the specified disk offline.
## This function validates the specified argument before
## issuing a disk offline command. If the argument is blank
## it returns a fail status=0.   
## This function returns a successful status=1 if the return
## message has "successfully offlined". 
## @arg 
##   $fc -- connection oject 
##   $disk_name (Required) a valid disk name
## @returns 0 -- failed; 1 -- success
## @status public 
##
sub sutils_disk_offline($$) {
	my($fc, $disk_name) = @_;
	
	if ($disk_name =~ /^\s*$/) {
            tstlog -comment, "sutils_disk_offline: Invalid disk name.\n";
            return 0; 
        }

	# enable disk offline for raid4
	#
	say($fc, "priv set -q $TEST_MODE; options raid.disk.offline.raid4.enable on"); 
	say($fc, "priv set -q $TEST_MODE; options raid.disk.offline.enable on"); 
	say($fc, "priv set -q $TEST_MODE; setflag raid_disk_offline_relaxed 1"); 
	my(undef, undef, $output) = say($fc, "priv set -q $TEST_MODE; disk offline $disk_name");
	
        # Return a successful status
	if ($output =~ "successfully offlined") {
	  tstlog -comment, "sutils_disk_offline: successfully offlined disk: <$disk_name>\n"; 
	  return 1; 
	}
		       
	# Output the sysconfig for debugging
	# 
	if ($VERBOSE) {
	    say($fc, "sysconfig -r");
	}
	# Return failed if the specified disk cannot be offline
	tstlog -comment, "sutils_disk_offline: failed to offline disk: <$disk_name>\n"; 
	return 0;
} #sutils_disk_offline

## @name ::sutils_disk_online
## @summary Online the specified disk 
## @description
## sutils_disk_online takes the specified disk online.
## This function validates the disk_name argument before
## issuing a disk online command. If the argument is blank
## it returns a fail status=0.   
## This function returns a successful status=1 if the return
## message has "successfully offlined". 
## @arg 
## $fc - (Required) connection object 
## $disk_name - (Required) specify the name disk to be online
## @returns 0 -- failed; 1 -- success
## @status public 
##
sub sutils_disk_online($$) {
    my($fc, $disk_name) = @_;
    
    if ($disk_name =~ /^ *$/) {
	tstlog -comment, "sutils_disk_online: Invalid disk name.\n";
    }
    
    my(undef, undef, $output) = say($fc, "priv set -q $TEST_MODE; disk online $disk_name");
    
    tstlog(-comment, "output=<$output>"); 

    # return a successful status
    if ($output =~ "successfully onlined") {
	tstlog -comment, "sutils_disk_online: successfully onlined disk: <$disk_name>\n"; 
	return 1;
    }
    
    # return failed if the specified disk cannot be online
    tstlog -comment, "sutils_disk_online: failed to online disk: <$disk_name>\n"; 
    return 0;
} #sutils_disk_online

#
# Function to search for a particular string in /etc/messages file under the root
# directory of the simulator
# @param Name of the Filer
# @param String we are searching for in /etc/messages
# @param array whose elements are month, day, hour, minute and second corresponding to the
# timestamp when the test volume was created
# @return 1 if string is found and 0 if it is not
#
sub sutils_check_log_for_message($$@) {
	my ($FILERNAME, $search_string, @date) = @_;
	my ($line, @tmp, @tmp2);

	my $FLAG = 0;

	if(!open(FH, "/t/$FILERNAME/root/etc/messages")) {
		tstlog -comment, "Cannot open file /t/$FILERNAME/root/etc/messages\n";
		return $FLAG;
	}

	while ($line = <FH>) {
		$line =~ tr/G/:/;
		@tmp = split(/:/,$line);
		@tmp2 = split(/\s+/, $tmp[0]);
		my $log_mon = sutils_get_month($tmp2[1]);
		my $log_day = $tmp2[2];
		my $log_hr = $tmp2[3];
		my $log_min = $tmp[1];
		my $log_sec = $tmp[2];

		if(($log_mon < $date[0]) || (($log_mon == $date[0]) && ($log_day < $date[1])) || (($log_mon == $date[0]) && ($log_day == $date[1]) && ($log_hr < $date[2])) || (($log_mon == $date[0]) && ($log_day == $date[1]) && ($log_hr == $date[2]) && ($log_min < $date[3])) || (($log_mon == $date[0]) && ($log_day == $date[1]) && ($log_hr == $date[2]) && ($log_min == $date[3]) && ($log_sec < $date[4]))) {
			next;
		}

		if($line =~ /$search_string/) {
			$FLAG = 1;
			tstlog -comment, "\$line = $line\n";
			last;
		}
	}
	return $FLAG;
}

#
# Function to determine if it running on simulator or filer
#
sub sutils_is_simulator() {
	@out = ati_ontapi_call_api(context => $context,
				   api     => 'system-get-info');
	($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
	my $o = $out[1]->{'system-info'}{'system-model'};
	tstlog -comment, "System model: $o\n";
	if ($o =~ "simulator") {
		return 1;
	}
	return 0;
}

#
# Function to get the integer equivalent of month that is passed in 
# as a 3-character string. 
# @param  Month represented as a 3-character string
# @return Month represented as an integer
#
sub sutils_get_month($) {
	my (%MONTHS, $intMonth);
	%MONTHS = (
                "Jan" => 1,
                "Feb" => 2,
                "Mar" => 3,
                "Apr" => 4,
                "May" => 5,
                "Jun" => 6,
                "Jul" => 7,
                "Aug" => 8,
                "Sep" => 9,
                "Oct" => 10,
                "Nov" => 11,
                "Dec" => 12
          );
	$intMonth = $MONTHS{$_[0]};
	return $intMonth; 
}

#
# Function to determine the date.
# @return array whose elements are month, day, hour, minute and second
#
sub sutils_get_timestamp() {
	my (@tmp0, @tmp1, @date);
	@out = ati_ontapi_call_cli(context   => $context,
				   privilege => ATI_ONTAPI_PRIVILEGE_DIAG,
				   command   => 'date');
	($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
	$out[1] =~ tr/G/:/;
#	tstlog -comment, "DATE: $out[1]\n";
	@tmp0 = split(/:/,$out[1]);
	@tmp1 = split(/\s+/, $tmp0[0]);
	$date[0] = sutils_get_month($tmp1[1]);
	$date[1] = $tmp1[2];
	$date[2] = $tmp1[3];
	$date[3] = $tmp0[1];
	$date[4] = $tmp0[2];

	return @date;
}

#
# Function to clear any stale mounts and then mount the Filer and the volume.
# @param Name of the Filer
# @param Name of the volume
#
sub sutils_mount_volume($$) {
	my $FILERNAME = $_[0];
	my $FILER_VOL = $_[1];

	# - - - - - - - - - - - - - - - - - - - -
	# Check the list of available licenses
	# - - - - - - - - - - - - - - - - - - - -
#	tstlog -comment, "Check list of available licenses...\n";
	@out = ati_ontapi_call_api(context => $context,
				   api     => 'license-list-info');
	($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
	foreach my $license (@{$out[1]->{licenses}}) {
		if ($license->{'service'} =~ "nfs") {
			if ($license->{'is-licensed'} eq "not licensed"){
				# - - - - - - - - - - - - - - - - - - - -
				# License NFS 
				# - - - - - - - - - - - - - - - - - - - -
				ati_ontapi_call_api(context => $context,
						    api     => 'license-add',
						    args    => ['code' => 'TZBGCKE']);
			} else {
				tstlog -comment, "$license->{'service'} = $license->{'is-licensed'}\n";
			}	
		}
	}
	# - - - - - - - - - - - - - - - - - - - -
	# Unmount to clear stale mounts
	# - - - - - - - - - - - - - - - - - - - -
#	tstlog -comment, "Unmount the filer $FILERNAME\n";
#	tstlog -comment, "$CLIENT\n";
	sutils_client_saycmd($CLIENT, "r umount -f /t/$FILERNAME/$FILER_VOL");
	sutils_client_saycmd($CLIENT, "r umount -f /t/$FILERNAME/root");
	
	# - - - - - - - - - - - - - - - - - - - -
	# Mount the Filer
	# - - - - - - - - - - - - - - - - - - - -
	tstlog -comment, "Mount the filer $FILERNAME\n";
	@out = ati_ontapi_call_cli(context   => $context,
				   privilege => ATI_ONTAPI_PRIVILEGE_DIAG,
				   command   => 'exportfs -p sec=sys,rw,anon=0 /vol/$FILER_VOL');
	($out[0] == ATI_SUCCESS) || die "Error $out[2]: $out[3]\n";
	sutils_client_saycmd($CLIENT, "r sim_mount $FILERNAME");
	sutils_client_saycmd($CLIENT, "r sim_mount $FILERNAME $FILER_VOL");
}

=head1 NAME 
    sutils_shutdown -- This routine will send a shutdown, panic, halt, or other
    to the filer/simulator depends on the shutdown argument.
    Return 0 - FAILED and 1 -- SUCCESS
    The list of arguments are: 
=over
=head1 SYNOPSIS
    my $ret = sutils_shutdown($fc, $shutdown_type); 

=over
=item $fc
    (Required) : filer connection object
= item $shutdown_type
    (Required) : shutdown type: "reboot", "panic", or "halt". 
=back 
=over

=cut
sub sutils_shutdown($$) {
    my $co = shift;
    my $shutdown = shift; 

    my ($result, $state) = transit_to($co, CTRL_C, haltcmd=> "reboot -t 0"); 

    if ($shutdown =~ /reboot/i) {
      #	if (reboot($co)) {
      #	    tstlog -comment, "Successfully rebooted the system"; 
      #	    return 1; 
      #	}
      #	tstlog -comment, "Failed to reboot the system";
      #        return 0;
      #    
      
      my( $res , $state);
      ($res , $state) = transit_to($co, CTRL_C,haltcmd=>"reboot -t 0") ;
      if (!$res) {
	tstlog -comment, "res=$res"; 
	tstlog -comment, "state=$state"; 
	tstlog -comment, "Failed to reboot the system"; 
	return 1; 
      }
      transit_to($co, UP , from=>$state) or tstlog('-fatal' , $@);
      return 1; 
    }
    
    if ($shutdown =~ /panic/i) {
	my $panic_str = "panic"; #since panic will be deprecated soon 
	my (undef, undef, $output) = say($co, "version", "-noprompt"); 
	if ($output =~ /<td>/) {
	    $panic_str = "panic -D"; 
	}
	(undef, undef, $output) = say($co, "priv set -q $TEST_MODE; $panic_str", "-noprompt"); 
	if ($output =~ /PANIC: panic without core/g  || $output =~ /Wait...Filer will panic at the end of the next/g) {
	    tstlog -comment, "Successfully panicked the system";
	    return 1;
	}
	# ------------------------------------------------------
	# Tom Doan -- we won't be able to match the result of 
	# panic on the real filer, let make it less sensitive
	# on matching the output here. 
	# ------------------------------------------------------
	tstlog -comment, "Unable to determine if <$shutdown> the system is success or not";
	return 1; 
    }

    if ($shutdown =~ /halt/i) {
        if (halt($co)) {
            tstlog -comment, "Successfully halted the system";
            return 1;
        }
        tstlog -comment, "Failed to halt the system";
        return 0;
    }

    # Any new shutdown command should go here
    tstlog -comment, "Unrecognized command <$shutdown>"; 
    return 0; 
} #sutils_shutdown

# -----------------------------------------
# name::_get_key - return the persistent key
# outmsg - out message text
# retrun the key string
# -----------------------------------------
sub _get_key($){
  my ($outmsg) = @_;
  my $key; 

  my (@lines) = split(/\n/, $outmsg); 
  foreach my $line (@lines) {
    printf("line=<%s>\n", $line); 
    if ($line =~ /key/) {
      my (@buff) = split(/ +/, $line); 
      $key = $buff[2];
      printf("key=<%s>\n", $key); 
    }
  }
  return $key; 
} #_get_key

## ------------------------------------------------------------------
## @name - sutils_disks_get- get persistent disks and partner's
## persistent disks.
## 
## Persistent disks are neither of the SES nor the broken list. 
## 
## @arg - 
##  aconn -- connection object
##  spares_r -- spare array reference
##  partner_spares_r -- partner spare array reference
## 
## @return - FAILURE or SUCCESS
## 
## --------------------------------------------------------------------
sub sutils_disks_get($$$){
  my ($aconn) = shift; 
  my ($spares_r, $partner_spares_r) = @_;
 
  my($priv) = say($aconn, "priv set  -q $TEST_MODE ;", -timeout, SAY_TIMEOUT);
  my(@info);
  my($line);
  my($tree_count);
  my $output;
  my %info_disk_htable = (); 
  my %spares_ht = (); 

  use constant SES_BAY => 2; 
  
  say($aconn, "priv set -q $TEST_MODE;", -timeout, SAY_TIMEOUT);
  (undef, undef, $output) = say($aconn, "priv set -q $TEST_MODE; raid_config info listdisk" , -timeout, SAY_TIMEOUT);

  #tstlog('-comment', "output=$output"); 

  @info = split(/\r?\n/, $output);
  my(%count) = ();

  foreach $line (@info) {
    #chop $line;
    my(@disk_info) = split(",", $line);
    my($entry);
    my(%entrydb) = ();
    
    # Load the entry
    foreach $entry (@disk_info) {
      if ($entry =~ /(.*)=(.*)/) {
	$entrydb{$1} = $2;
      }
    }
    
    # Now, go back and get the disk name
    my($name) = $entrydb{"name"};
    if (!$name) {
      next;
    }

    # skip the disks that are not in the specified bay
    my ($bay) = $entrydb{"bay"}; 
    if ($bay < SES_BAY) {
      next; 
    }

    # -----------------------------
    # skip those in the POOL:broken 
    # -----------------------------
    if ($entrydb{"tree"} eq "POOL:broken") {
	next; 
    }

    # --------------------------------------------------   
    # skip broken disks which can only be checked as the 
    # avl_blks=0 from  the partner node
    # --------------------------------------------------
    if ($entrydb{"avl_blks"} <= 0) {
      next; 
    }
    my($tree) = $entrydb{"tree"};

    # --------------------------------------------------
    # Remained disks could go to either the POOL:partner
    # or one common POOL and let called it POOL:common
    # --------------------------------------------------
    if ($tree eq "POOL:partner") {
	if ($count{$tree}++ > 0) {
	    $info_disk_htable{$tree} .= ",";
	}
	$info_disk_htable{$tree} .= $name ;
    } else {
	if ($count{"POOL:common"}++ > 0) {
	    $info_disk_htable{"POOL:common"} .= ","; 
	}
	$info_disk_htable{"POOL:common"} .= $name; 
    }
    
  } #for
  
  #
  # sort all available disks
  #
  my @spares1 = sort split(",", $info_disk_htable{"POOL:common"}); 

  # get a list of spares from the partner
  # 
  my @partner_spares = sort split(",", $info_disk_htable{"POOL:partner"}); 
  
  if ($VERBOSE) {
    if ($#spares1 < 0) {
      tstlog('-comment', "The spare list is empty"); 
    } else {
	print "\nspares= "; 
	print "@spares1" . "\n"; 
    }
  }
  
  if ($VERBOSE) {
    if ($#partner_spares < 0) {
      tstlog('-comment', "The partner spare is empty");
    } else {
	printf("\npartner_spares= "); 
	print "@partner_spares\n"; 
    }
  }

  @$spares_r = @spares1; 
  @$partner_spares_r = @partner_spares; 
  return 1;
} #sutils_disks_get


sub _is_match($$) {

    my ($disk, $pdisk) = @_; 
    
    my ($da, $did) = split(/\./, $disk); 
    my ($dap, $dpid) = split(/\./, $pdisk); 

    if ($did eq $dpid) {
	return 1; 
    } 
	
    return 0; # they don't match 
}

sub _match_two_list(\@\@) {
    my ($r1, $r2 ) = @_; 

    my @t1 = @$r1; 
    my @t2 = @$r2; 

    my @tbd = (); 
    my @tbpd = (); 

    print "\nBEFORE\n" . "$#t1 " . "a1= " . "@t1" . "\n"; 
    print "$#t2 " . "a2= " . "@t2\n"; 

    for (my $i =0; $i<= $#t1; $i++) {
#	print $i . " ".  @t1[$i] . "\n"; 
#	print $i . " ".  @t2[$i]  . "\n"; 
	my $j = 0; 
	
	my $match = 0; 
	for (my $j = 0; $j<=$#t2; $j++) {
	    if ( _is_match($t1[$i], $t2[$j])) {
#		print "** found a match <$t1[$i]> <-> <$t2[$j]> **"; 
		$match = 1; 
		push(@tbd, $t1[$i]); 
		push(@tbpd, $t2[$j]); 
		last; 
	    }
	} #for 
    } #for

    print "RESULT" . "$#tbd= " . "@tbd" . "\n"; 
    print "RESULT" . "$#tbpd= " . "@tbpd" . "\n\n"; 
    @$r1 = @tbd; 
    @$r2 = @tbpd; 
} #_match_two_list

## --------------------------------------------------------------------
## sutils_cf_disks_get - get disks from FILERA and the corresponding 
## disks from the FILERB. The routine passes
## 2 array list with the first is the spare list and the second is
## the spare list from its partner.
##
## If the two list are mismatched in terms of number of spares or
## name of the spare, then it returns a FAIL. 
## 
## If there is no spare or there is no corresponding spares from the 
## parter then return a FAIL status. 
## 
## @arg - 
##  aconn -- filera connection object
##  bconn -- filerb connection object
##  $disks_r -- get a list of PR disks
##  $ap
##              EX: "10a.23 11b.23 10a.34 11b.23 .. "  
## 
## @return - FAILURE 0 
##           SUCCESS 1 
## --------------------------------------------------------------------
sub sutils_cf_disks_get($$$$){
  my $accon = shift;
  my $bccon = shift;
  my ($disks_r, $partner_disks_r) = @_;
	
  my (@adisks, @bdisks, @apartner_disks, @bpartner_disks); 
  sutils_disks_get($accon, \@adisks, \@bpartner_disks); 
  sutils_disks_get($bccon, \@bdisks, \@apartner_disks); 

  if ($VERBOSE) {
      print "sutils_cf_disks_get:\n";
      print @adisks."" . " aspares= " . "@adisks\n"; 
      print @apartner_disks."" . " apartner_spares= " . "@apartner_disks\n\n";  
      print @bdisks."" . " bspares= " . "@bdisks\n"; 
      print @bpartner_disks."" . " bpartner_spares= " . "@bpartner_disks\n";  
  }

  # matching the 2 two list and eliminate those that don't match
  # 
  _match_two_list(@adisks, @apartner_disks); 

  # -----------------------------------------------------------
  # check for the number of spares should be the same
  # if they are not, then there is an issue and we need to stop.
  # -----------------------------------------------------------
  if ($#adisks != $#apartner_disks) {
      # dump the list of spares and what can be seen from its partners
      # 
      tstlog("-config", "the <$#adisks> spares don't match with its partner <$#apartner_disks>"); 
      return 0; 
  }
  
  @$disks_r = @adisks; 
  @$partner_disks_r = @apartner_disks; 
  return 1;
} #sutils_cf_disks_get


=head2 sutils_is_cluster

This routine shall determine if the cluster is UP and running
and return 1 for SUCCESS or 
Return 0 - FAILED 
The list of arguments are: 
=filera
  (Required) : FILERA connection object
=filerb
 (Required) : FILERB connection object
=cut 
sub sutils_is_cluster($$) {
    my ($fa, $fb) = @_; 
    my $TIMEOUT = 240; 

    if ($fa eq "") {
	tstlog(-comment, "found undefined cluster <$fa>"); 
	return 0; 
    }
    my (undef, undef, $output) = say($fa, "priv set -q $TEST_MODE; cf status", '-timeout', $TIMEOUT); 
    if ($output =~ /Cluster enabled/ || 
	$output =~ /Cluster disabled/ || 
	$output =~ /has disabled takeover by/ || 
	$output =~ /attempting takeover/ || 
	$output =~ /giveback in progress/ ) {
      	tstlog(-comment, "found an enable cluster"); 
    } else {
     	 tstlog(-config, "the FILERA is not part of a cluster"); 
    }

    if ($fb eq "") {
	tstlog(-comment, "found undefined cluster <$fb>"); 
	return 0; 
    }
    (undef, undef, $output) = say($fb, "priv set -q $TEST_MODE; cf status", '-timeout', $TIMEOUT); 
    if ($output =~ /Cluster enabled/ || 
	$output =~ /Cluster disabled/ || 
	$output =~ /has disabled takeover by/ || 
	$output =~ /attempting takeover/ || 
	$output =~ /giveback in progress/ ) {
      	tstlog(-comment, "found an enable cluster"); 
	return 1; 
    } else {
	tstlog(-comment, "looks like this is not a clustering system");
    	return 0;
    }
} #sutils_is_cluster

=head2 sutils_log_msg
  my $status = sutils_log_msg (msg=>$message)

This function invokes the tstlog function with the passing
message only when VERBOSE is set. 

=over
=item msg
 (Required) : the message content
=back
=cut

sub sutils_log_msg($) {
   my ($msg) = @_;
   if ($VERBOSE) {
 	tstlog('-comment', $msg); 
   }
   return 1; 
}

=head2 sutils_spares_filter

my $status = sutils_spares_filter(disks_r=>$disks, pdisk_r=>$pdisks)

This function compares the two specified disk lists, filters, and 
returns only the spares one from the partner disks. Normally, the 
partner should see a full list of disks whether or not they are 
spare while on the filer there are only spares. 

=item @disks - disks of a filer
=item @pdisks - disks of a filer which seen on the partner
=item $pdisks_r - return the equivalent spares seen from the partner
=return 1 SUCCESS
=cut 
sub sutils_spares_filter($$$) {
    my (@disks, @pdisks, $pdisks_r) = @_; 
    my @tmp = ();  
    my $i = 0; 
    my $pi = 0; 
    my $not_found = 1; 

    # for each spare on the filer we need to find a match 
    # from its partner

    for ($i=0; $i<$#disks; $i++) {
	my ($shelf, $did) = split(/./, $disks[$i]); 
	while ($not_found && $pi < $#pdisks) {	
	   my ($pshelf, $pdid) = split(/\./, $pdisks[$pi]);
	
	   sutils_log_msg("pshelf=" . $pshelf . "pdid=" . $pdid); 
	   # compared the disk id from both list and if they
	   # match then we found the right spare. 
	   if ($did eq $pdid) {
	       $not_found = 0;
		sutils_log_msg("$disks[$i] match $pdisks[$pi]");
           }
	   $pi++; 
	} #while

	# ============================
	# save the match partner disk
	# ============================
	if ($not_found == 0) {
	    $tmp[$i] = $pdisks[$pi];   	
	}
	$not_found = 1; 
     }#for

  # At this point both the list must have the same number of spares
  if ($#disks != $#tmp) {
	tstlog(-comment, "sutils_spares_filter: unable to filter spares"); 
	print "spares= " . "@disks" . "\n"; 
	print "partner_spares= " . "@tmp" . "\n"; 
	@$pdisks_r = @tmp; 
	return 0; 
   }
} #sutils_spares_filter

#
# End of sutils package
#
1;

