#$Id:
# 
## @summary ACP & EXP results
##
## @author nivedita@netapp.com
## Given two EBOD systems,for paralell testing, the command on the first EBOD should be passed to the the 
## gem_cmd_api along with the "misc" parameter set to "Parallel"
## During Stress cycle tests, the gem_cmd_api should be called with its "misc" parameter set to "Stress"


package Storage::ACP_EXP_results;

use strict;
use warnings;
use Tharn qw(say script_result host logcomment logresult param);
use Storage::Sapphire qw(gem_cmd_api);
use Storage::SASUtils qw(
    filer_cmd
    storage_logresult
);

use Storage::ACP_EXP qw(transit_expcon_expdiag transit_acp_toexp _acp_login);

use base qw(Exporter);
our @EXPORT_OK = qw(test_acp
    test_acpdig
    test_exp
    test_expdig);


##Test cases to be tested on acp console
my @acpcon_test = ("fwstatus",
    "fwstatus A",
    "fwstatus B");

##Test cases to be tested on acp loader prompt
my @acp_test = ("printenv",
    "acp ver",
    "acp i2c",
    "acp we",
    "acp mid",
    "acp cntl last",
    "acp cntl rs",
    "acp cntl lrebod",
    "acp cntl  lrst",
    "acp cntl led",
    "acp cntl  id",
    "acp cntl ac",
    "acp ser",
    "acp mem",
    "acp post",
    "acp reset enc_off",
    "acp volt 1",
    "acp reset",
    "acp volt 2",
    "acp reset enc_on",
    "acp volt 3",
    "acpdig"

);

##Test cases to be testes on acp diag mode
my @acpdig_test = (
    "a 0",   #U-boot Image CRC Test",
    "a 1",   #DDR Walk Test",
    "a 2",   #DDR Address Test
    "a 3",   #DDR Background Test,
    "a 4",   #DDR March C- Test,
    "a 5",   #NAND Rewrite Quick Test,
    "a 6",   #NAND Rewrite Exhaustive Test,
    "a 7",   #Memory to Memory DMA Test,
    "a 8",   #NOR Flash to CPLD DMA Test                                         
    #"a 9",   #Stress Cycle 1 (no checking)                                       
    #"a A",   #Stress Cycle 2 (no enet or checking)                               
    #"a B",   #Stress Test                                                        
    #"a C",   #Stress Test (no enet)                                              
    "a D",   #Enet MII test                                                      
    "a E",   #Enet SEEPROM reload test                                           
    #"a F",   #Enet PAR LED test                                                  
    "a 10",  #Enet VTC diag                                                      
    "a 11",  #Enet interrupt test                                                
    "a 12",  #Enet file check test                                               
    "a 13",  #Ethernet SEEPROM CRC Test                                          
    "a 14",  #Ethernet SEEPROM WE Test                                           
    "a 15",  #Ethernet SEEPROM R/W Test                                          
    "a 16",  #Uart Loop Test                                                     
    "a 17",  #Uart Fifo Test                                                     
    "a 18",  #Uart Expander Test                                                 
    "a 19",  #ECC Memory Test                                                    
    #"a 1A",  #Watch Dog Interrupt Test                                           
    #"a 1B",  #Watch Dog Reset Test                                               
    "a 1C",  #ACP CPLD Test                                                      
    "a 1D",  #ACP CPLD Register Test                                             
    #"a 1E",  #ACP CPLD LED Test                                                  
    "a 1F",  #ACP CPLD Heartbeat Test                                            
    "a 20",  #ACP CPLD Reset Test                                                
    "a 21",  #ACP CPLD Status Test                                               
    "a 22",  #ACP CPLD Post Register Test                                        
    "a 23",  #ACP CPLD Magic Register Test                                       
    "a 24",  #ACP CPLD I2C Select Test                                           
    "a 25",  #EBOD CPLD JTAG Size Test                                           
    "a 26",  #EBOD CPLD JTAG ID Test                                             
    "a 27",  #Read the EBOD UES register                                         
    "a 28",  #ACP CPLD JTAG Size Test                                            
    "a 29",  #ACP CPLD JTAG ID Test                                              
    "a 2A",  #Power Supply Voltage Test                                          
    "a 2B",  #Mid Plane CPLD JTAG ID Test                                        
    "a 2C",  #Mid Plane CPLD JTAG Size Test                                      
    "a 2D"   #Mid Plane CPLD JTAG enable Test 
);


##Test case to be tested on expander diag prompt
my @exp_test = ("ver",
    "ebod",
    "i2c temp",
    "mid sw 2",
    "mid temp",
    "mid sw 1",
    "mid temp",
    "mid vpd 1 0x00 8",
    "mid vpd 2 0x00 8",
    "i2c 1 pr",
    "i2c 2 pr",
    "ebod fix",
    "ebod ver",
    "ebod volt",
    "sgpio version",
    "sgpio init",
    "i2c 2 pr",
    "ebod fix",
    "ebod ver",
    "ebod volt",
    "sgpio version",
    "sgpio init",
    "sgpio drive",
    "sgpio psw",
    "sgpio owner",
    "qsfp sel 1",
    "qsfp present",
    "qsfp sel 2",
    "qsfp present"
);

##test cases to be tested on expander diag mode
##that requires "a" prefixed to the start of the
##test command
my @expdig_test = (
    "a 01", #SRAM Address Test                          
    "a 02", #SRAM March C Test                          
    #"a 03", #Flash Log memory test (non-destructive)    
    #"a 04", #PCRL memory test (non-destructive)         
    #"a 05", #Image A flash memory test (non-destructive)
    #"a 06", #Image B flash memory test (non-destructive)
    "a 07", #I2C 3 controller reg Test                  
    "a 08", #I2c 3 probe Test                           
    "a 09", #Temp ID Test                               
    "a 0A", #Temp reg Test                              
    "a 0B", #VPD data path test
    "a 0C", #VPD memory write enable test
    "a 0D", #VPD memory test 
    "a 0E", #VPD Check sum Test    
    "a 0F", #QSFP enables Test     
    "a 10", #QSFP register Test    
    "a 11", #QSFP Memory Test      
    "a 12", #QSFP check sum Test    
    #"a 13", #QSFP cable check      
    "a 14", #QSFP signal Test      
    "a 15", #QSFP selection Test   
    "a 16", #CPLD version check    
    "a 17", #CPLD register test    
    #"a 18", #Fault LED test        
    "a 19", #Heart beat check      
    #"a 1A", #CPLD reset test       
    "a 1B", #CPLD voltage check    
    "a 1C", #CPLD status check     
    "a 1D", #ACP post register
    "a 1E", #CPLD Module reset magic register test     
    #"a 1F", #ACP reset test         
    #"a 20", #ebod reset test        
    #"a 21", #Watch dog count test   
    #"a 22", #Watch dog reset test   
    "a 23", #JTAG enable test       
    "a 24", #JTAG chain size test   
    "a 25", #JTAG ID test           
    "a 26", #Mid plane probe test   
    #"a 27", #SGPIO Display test     
    #"a 28", #SGPIO drive active led test     
    #"a 29", #SGPIO LED test                  
    "a 2A", #Mid plane VPD test              
    "a 2B", #Mid plane VPD write protect test
    #"a 2C", #Drive fault test
    "a 2D", #Drive status test
    "a 2E", #I2c 3 Interrupt test
    "a 2F" #I2c 3 reset test
);


##Test cases that reset the EBOD
my @expdig_reset = (
 
    "a 20", #ebod reset test
    "a 1A"  #CPLD reset test
);

##Key(command)- Value(result expected) pair of all the acp commands

my %hash_acp = (
    "fwstatus"=>" ",
    "fwstatus A"=>'Code Image A \(running\)',
    "fwstatus B"=>'Code Image B \(running\)',
    "printenv"=>" ",

    "acp ver"=>"ACP version is 0x12\nACP Fix register is 0x5a\nACP reports this is a IOM6 board",

    "acp i2c"=>'Current I2C mux select is set \(enet\)',

    "acp we"=>"Enet EEPROM Write Enable is not set",

    "acp mid"=>"Mid Plane JTAG Enable is not set",

    "acp cntl last"=>"ACP Last Reset TRST bit = 0x0",

    "acp cntl rs"=>"Reset State bit is set",

    "acp cntl lrebod"=>"ACP Last Reset EBOD bit = 0x0",

    "acp cntl  lrst"=>"ACP Last Reset Register bit = 0x0",

    "acp cntl led"=>"Fault LED is not set",

    "acp cntl  id"=>"ACP SBB Slot ID bit = 0x1",

    "acp cntl ac"=>"ACP SBB AC Good bit = 0x0",

    "acp ser"=>"Serial bus update is set",

    "acp mem"=>"Nor Flash is ready\nNand Flash is ready",

    "acp post"=>"The ACP Post Register is 0x8000",

    "acp reset enc_off"=>"",

    "acp volt 1"=>"ACP hot swap is good\n3.3 ACP voltage is good\n2.5 ACP voltage is good\n1.8 ACP voltage is good\n1.5 ACP voltage is good\n1.2 ACP voltage is good\nEBOD hot swap is bad\n1.0 EBOD voltage is bad\n5.0 EBOD voltage is bad\n4.0 EBOD voltage is bad\n3.3 EBOD voltage is bad\n1.2 EBOD voltage is bad",

    "acp reset"=>"ACP Power Control Shadow register = 0x0\nPower Control Active register = 0x1",

    "acp volt 2"=>"ACP hot swap is good\n3.3 ACP voltage is good\n2.5 ACP voltage is good\n1.8 ACP voltage is good\n1.5 ACP voltage is good\n1.2 ACP voltage is good\nEBOD hot swap is bad\n1.0 EBOD voltage is bad\n5.0 EBOD voltage is bad\n4.0 EBOD voltage is bad\n3.3 EBOD voltage is bad\n1.2 EBOD voltage is bad",

    "acp reset enc_on"=>"",

    "acp volt 3"=>"ACP hot swap is good\n3.3 ACP voltage is good\n2.5 ACP voltage is good\n1.8 ACP voltage is good\n1.5 ACP voltage is good\n1.2 ACP voltage is good\nEBOD hot swap is good\n1.0 EBOD voltage is good\n5.0 EBOD voltage is good\n4.0 EBOD voltage is good\n3.3 EBOD voltage is good\n1.2 EBOD voltage is good",

    "acpdig"=>" ",
    "a 0"=>" ",
    "a 1"=>"PASS",
    "a 2"=>"PASS",
    "a 3"=>"PASS",
    "a 4"=>"PASS",
    "a 5"=>"PASS",
    "a 6"=>"PASS",
    "a 7"=>"PASS",
    "a 8"=>"PASS",
    "a 9"=>"PASS",
    "a A"=>"PASS",
    "a B"=>"PASS",
    "a C"=>"PASS",
    "a D"=>"PASS",
    "a E"=>"PASS",
    "a F"=>"PASS",
    "a 10"=>"PASS",
    "a 11"=>"PASS",
    "a 12"=>"PASS",
    "a 13"=>"PASS",
    "a 14"=>"PASS",
    "a 15"=>"PASS",
    "a 16"=>"PASS",
    "a 17"=>"PASS",
    "a 18"=>"PASS",
    "a 19"=>"PASS",
    "a 1A"=>"PASS",
    "a 1B"=>"PASS",
    "a 1C"=>"PASS",
    "a 1D"=>"PASS",
    "a 1E"=>"PASS",
    "a 1F"=>"PASS",
    "a 20"=>"PASS",
    "a 21"=>"PASS",
    "a 22"=>"PASS",
    "a 23"=>"PASS",
    "a 24"=>"PASS",
    "a 25"=>"PASS",
    "a 26"=>"PASS",
    "a 27"=>"PASS",
    "a 28"=>"PASS",
    "a 29"=>"PASS",
    "a 2A"=>"PASS",
    "a 2B"=>"PASS",
    "a 2C"=>"PASS",
    "a 2D"=>"PASS",

);

##Key(command)- Value(result expected) of all the EBOD commands

my %hash_exp = (
    "ver"=>"EBOD Expander Embedded Diagnostics 2.62",
    "ebod"=>" ",
    "i2c temp"=>" ",
    "mid sw 2"=>" ",
    "mid temp"=>" ",
    "mid sw 1"=>" ",
    "mid temp"=>" ",
    "mid vpd 1 0x00 8"=>" ",
    "mid vpd 2 0x00 8"=>" ",
    "i2c 1 pr"=>" ",
    "i2c 2 pr"=>" ",
    "ebod fix"=>" ",
    "ebod ver"=>" ",
    "ebod volt"=>" ",
    "sgpio version"=>" ",
    "sgpio init"=>"",
    "i2c 2 pr"=>" ",
    "ebod fix"=>" ",
    "ebod ver"=>" ",
    "ebod volt"=>" ",
    "sgpio version"=>" ",
    "sgpio init"=>" ",
    "sgpio drive"=>" ",
    "sgpio psw"=>" ",
    "sgpio owner"=>" ",
    "qsfp sel 1"=>"",
    "qsfp present"=>"The QSFP 1 is present"."\r"."The QSFP 2 is present",
    "qsfp sel 2"=>"",
    "a 01"=>"PASS",
    "a 02"=>"PASS",
    "a 03"=>"PASS", 
    "a 04"=>"PASS", 
    "a 05"=>"PASS", 
    "a 06"=>"PASS", 
    "a 07"=>"PASS", 
    "a 08"=>"PASS", 
    "a 09"=>"PASS", 
    "a 0A"=>"PASS", 
    "a 0B"=>"PASS",
    "a 0C"=>"PASS", 
    "a 0D"=>"PASS", 
    "a 0E"=>"PASS", 
    "a 0F"=>"PASS", 
    "a 10"=>"PASS", 
    "a 11"=>"PASS", 
    "a 12"=>"PASS", 
    "a 13"=>"PASS", 
    "a 14"=>"PASS", 
    "a 15"=>"PASS", 
    "a 16"=>"PASS", 
    "a 17"=>"PASS", 
    "a 18"=>"PASS", 
    "a 19"=>"PASS", 
    "a 1A"=>"PASS", 
    "a 1B"=>"PASS", 
    "a 1C"=>"PASS", 
    "a 1D"=>"PASS", 
    "a 1E"=>"PASS",
    "a 1F"=>"PASS", 
    "a 20"=>"PASS", 
    "a 21"=>"PASS", 
    "a 22"=>"PASS", 
    "a 23"=>"PASS", 
    "a 24"=>"PASS", 
    "a 25"=>"PASS", 
    "a 26"=>"PASS", 
    "a 27"=>"PASS", 
    "a 28"=>"PASS", 
    "a 29"=>"PASS", 
    "a 2A"=>"PASS", 
    "a 2B"=>"PASS", 
    "a 2C"=>"PASS", 
    "a 2D"=>"PASS", 
    "a 2E"=>"PASS", 
    "a 2F"=>"PASS",
    );

##Testing the ACP commands

sub test_acp {
    my ($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE) = @_;
    my $j=0;
    my $i=0;
    my $output;
    while($acp_test[$i]) {
        $$acp_ctrlConn[0]->discardbuff(timeout=>10);
        $$acp_ctrlConn[1]->discardbuff(timeout=>10);
        while($$acpCon[$j]){
            #logcomment("Testing in acp $$acpCon[$j], $acp_test[$i]");
            if($acp_test[$i] =~ "acp volt"){
                if($$acpCon[$j+1]){ 
                    $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], misc=>"Parallel", cmd=>"acp volt",timeout=>$TIMEOUT,connmode=>$CONNMODE);
                    $j++;
                    next;
                }else {
                    $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"acp volt",timeout=>$TIMEOUT,connmode=>$CONNMODE);
                }
            }else {
                if($$acpCon[$j+1]){ 
                    $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], misc=>"Parallel", cmd=>$acp_test[$i], timeout=>$TIMEOUT,connmode=>$CONNMODE);
                    $j++;
                    next;
                }else {
                    $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>$acp_test[$i],timeout=>$TIMEOUT,connmode=>$CONNMODE);
                }

            }
            if(!$$acpCon[$j+1]){ 
                if($output =~ /$hash_acp{$acp_test[$i]}/ || !$hash_acp{$acp_test[$i]}){
                    logresult("PASS", "ON ACP$j, $acp_test[$i]\n\n$output");
                }else {
                    logresult("FAIL", "ON ACP$j, $acp_test[$i]\n\n$output");
                    logcomment("Expected output \n$hash_acp{$acp_test[$i]}");
                }
            }
            $j ++;
        }
        $j = 0;
        $i++;
    }
}

##Testing the ACP diag commands

sub test_acpdig {
    my ($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE) = @_;
    my $j=0;
    my $i=0;
    my $output;
    #logcomment("acp diag Stress cycle\n");
    #$output = gem_cmd_api(host=>$$acpCon[0],conn=>$$acp_ctrlConn[0], cmd=>"o",timeout=>$TIMEOUT,connmode=>$CONNMODE);
    #gem_cmd_api(host=>$$acpCon[0], conn=>$$acp_ctrlConn[0], cmd=>"a 9", timeout=>$TIMEOUT,connmode=>$CONNMODE);
    #gem_cmd_api(host=>$$acpCon[0], conn=>$$acp_ctrlConn[0], cmd=>"special r",timeout=>$TIMEOUT,connmode=>$CONNMODE);
    #logcomment("$output");
    while($acpdig_test[$i]) {
        $$acp_ctrlConn[0]->discardbuff(timeout=>10);
        $$acp_ctrlConn[1]->discardbuff(timeout=>10);
        while($$acpCon[$j]){
            $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"o",timeout=>$TIMEOUT,connmode=>$CONNMODE);
            logcomment("testing in acp diag $$acpCon[$j], $acpdig_test[$i]");
            gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>$acpdig_test[$i],timeout=>$TIMEOUT,connmode=>$CONNMODE);
            $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"r",timeout=>$TIMEOUT,connmode=>$CONNMODE);
            if($output =~ $hash_acp{$acpdig_test[$i]} || !$hash_acp{$acpdig_test[$i]}){
                logresult("PASS", "ON ACP$j, $acpdig_test[$i]\n\n\n\n$output");
            }else {
                logresult("FAIL", "ON ACP$j, $acpdig_test[$i]\n\n\n\n$output");
            }
            $j ++;
        }
        $j = 0;
        $i++;
    }
}


##Testing the expander commands part 1

sub test_exp {
    my ($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE) = @_;
    my $j=0;
    my $i=0;
    my $output;
    while($exp_test[$i]) {
        while($$acpCon[$j]){
            logcomment("Testing in Expander of $j, $exp_test[$i]");
            $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>$exp_test[$i],timeout=>$TIMEOUT,connmode=>$CONNMODE);
            if($output =~ $hash_exp{$exp_test[$i]} || !$hash_exp{$exp_test[$i]}){
                logresult("PASS", "ON ACP$j, $exp_test[$i]\n\n\n\n$output");
            }else {
                logresult("FAIL", "ON ACP$j, $exp_test[$i]\n\n\n\n$output");
                logcomment("Expected output \n $hash_exp{$exp_test[$i]}");
            }
            $j ++;
        }
        $j = 0;
        $i++;
    }
}




##Testing the expander diag commands part 2, where commands require a prefix "a" before execution

sub test_expdig {
    my ($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE) = @_;
    my $j=0;
    my $i=0;
    my $output;
    while($expdig_test[$i]) {
        while($$acpCon[$j]){
            $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"o",timeout=>$TIMEOUT,connmode=>$CONNMODE);
            logcomment("Testing in expander diag of $j, $expdig_test[$i]");
            gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>$expdig_test[$i],timeout=>$TIMEOUT,connmode=>$CONNMODE);
            $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"r",timeout=>$TIMEOUT,connmode=>$CONNMODE);
            if($output =~ $hash_exp{$expdig_test[$i]} || !$hash_exp{$expdig_test[$i]}){
                logresult("PASS", "ON ACP$j, $expdig_test[$i]\n\n\n\n$output");
            }else {
                logresult("FAIL", "ON ACP$j, $expdig_test[$i]\n\n\n\n$output");
            }
            $j ++;
        }
        $j = 0;
        $i++;
    }
    reset_expdig($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE);
    reset_acpdig($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE);
}

##Resets the EBOD
sub reset_expdig {
    my ($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE) = @_;
    my $j=0;
    my $i=0;
    my $output;
    while($expdig_reset[$i]) {
        $$acp_ctrlConn[0]->discardbuff(timeout=>10);
        $$acp_ctrlConn[1]->discardbuff(timeout=>10);
        while($$acpCon[$j]){
            $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"o",timeout=>$TIMEOUT,connmode=>$CONNMODE);
            logcomment("Testing in expander diag of $j, $expdig_reset[$i]");
            gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>$expdig_reset[$i],timeout=>$TIMEOUT,connmode=>$CONNMODE);
            $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"r",timeout=>$TIMEOUT,connmode=>$CONNMODE);

            $j ++;
        }
        transit_expcon_expdiag($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE);
        $j = 0;
        $i++;
    }
}


##Resets ACP 
sub reset_acpdig {
    my ($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE) = @_;
    my $j=0;
    my $output;
    while($$acpCon[$j]){
        $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"o",timeout=>$TIMEOUT,connmode=>$CONNMODE);
        logcomment("Testing in expander diag of $j, ACP Reset");
        gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"a 1F",timeout=>$TIMEOUT,connmode=>$CONNMODE);
        $output = gem_cmd_api(host=>$$acpCon[$j], conn=>$$acp_ctrlConn[$j], cmd=>"r",timeout=>$TIMEOUT,connmode=>$CONNMODE);
        _acp_login($$acp_ctrlConn[$j]);
        $j ++;
    }
    transit_acp_toexp($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE);
    transit_expcon_expdiag($acpCon, $acp_ctrlConn, $TIMEOUT, $CONNMODE);
    $j = 0;
}




1;
