#$Id: //depot/prod/test/Rsierranevada/lib/Storage/Acorn.pm#1 $
## @summary Acorn.pm - library module for Acorn specific commands
##
## @status in-progress
##
## @author dl-storage-automation,anishv
##
## @pod here

=head1 NAME

Storage::Acorn

=head1 SYNOPSIS

    use Acorn qw(acorn_get_status);

    my $host = host($FILER);
    my $conn = connect($FILER);
    my $hash = acorn_get_status(host=>$host,....);


=head1 DESCRIPTION

    This is a library module for Acorn(BSAS) command

=head1 ROUTINES

=cut




package Storage::Acorn;

use strict;
use warnings;

BEGIN {
    use base qw(Exporter);
    our @EXPORT_OK = qw(
            acorn_get_status
            acorn_sata_identify_device
            acorn_set_features
            acorn_get_alta_log
            acorn_get_alta_crash_dump
            acorn_get_vega_crash_dump
            );

    Params::Validate::validation_options(
            on_fail => sub {my @msgs = @_; chomp(@msgs);
            Carp::confess(Tharn::script_result(@msgs))}
            );
}

use Carp qw(confess croak);
use Params::Validate qw(validate validate_pos OBJECT SCALAR HASHREF);
use Storage::Misc qw(filer_cmd);
use Storage::Storage qw(storage_show_disk);


use Tharn qw(say script_result host logcomment logresult param);
use Utils qw(make_conn);
use Data::Dumper qw(Dumper);
use ATI::ati_ontapi qw(ATI_SUCCESS
		       ATI_ONTAPI_PRIVILEGE_DIAG
		       ati_ontapi_create_context
		       ati_ontapi_call_api);
#use filer;
use vars qw(@ISA @EXPORT
	$FILER_TEST_UI
);

=head2 acorn_get_status

    my $hash = acorn_get_status(host=>$host, disk_id=>"diskid");

This routine returns a pointer to hash table of acorn get_status  command
It croaks on failure.
Following is an example of the hash table returned by the function:

$VAR1 = {
          'LosCount0' => '0',
          'BootLoaderVer' => 'BTLDR_01_02',
          'InvalidDwordCount' => '0',
          'DisparityErrCount0' => '0',
          'LinkRate1' => '9',
          'FWVer' => '02_00_02_00',
          'PCBRevision' => undef,
          'DisparityErrorCount' => '0',
          'SerialNumber' => undef,
          'InvalidDWordCount1' => '0',
          'InvalidDWordCount0' => '0',
          'LossCount' => '0',
          'ManufDate' => undef,
          'LinkDownCount' => '0',
          'SasPortId0' => '5006056b00035ddd',
          'ChipIdentifier' => '2520',
          'Phy1' => '0',
          'LosCount1' => '0',
          'ManufLocation' => undef,
          'PhyResetProbCnt1' => '0',
          'Phy0' => '0',
          'SataWWN' => '5006056b00035ddc',
          'LinkRate0' => '9',
          'PhyId0' => '20',
          'SasPortId1' => '5006056b00035dde',
          'PhyId1' => '20',
          'PhyResetProbCnt0' => '0',
          'SASAddress1' => '5006056b00035dde',
          'SpeedNegErrorCount' => '0',
          'ConfigDataVer' => '10C.2008',
          'DisparityErrCount1' => '0',
          'SASAddress0' => '5006056b00035ddd',
          'ChipRevision' => 'AB00',
          'XmitErrorCount' => '0',
          'PCBPartNumber' => undef,
          'ReceiveErrorCount' => '0',
          'OEMName' => undef
        };

=over

=item host

(Required) : Filer connnection to use.


=item filermode

(Optional) : The mode of filer: "normal" or "maint". Default is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to 
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item disk_id

(Required) : Disk ID to run acorn get_status for

=back 

=cut



sub acorn_get_status{
	my (%opts) = validate(@_, {
				host => { type=>OBJECT, isa=>'Hostrec' },
				filermode => { type=>SCALAR, default=>"normal", optional=>1 },
				conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
				timeout => {type=>SCALAR,default=>300,optional=>1},
				disk_id => {type=>SCALAR},
                     }
                );

	my $hash;
	my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"acorn get_status $opts{disk_id}");
	print "output is : $output \n";
	if ($output !~ /SAS LINK INFO/){
            croak $output;
       	}
	my @status = split(/\r*\n/,$output);
    foreach my $status (@status){
        my @element = split(/\s+/,$status);
        if($status =~ /^PHY:/){
            $hash->{Phy0} = $element[1];
            $hash->{Phy1} = $element[2];
        }
        elsif($status =~ /^Negotiated Link Rate:/){
            $hash->{LinkRate0} = $element[3];
            $hash->{LinkRate1} = $element[4];
        }
        elsif($status =~ /^SAS address:/){
            $hash->{SASAddress0} = $element[2];
            $hash->{SASAddress1} = $element[3];
        }
        elsif($status =~ /^Attached PHY ID:/){
            $hash->{PhyId0} = $element[3];
            $hash->{PhyId1} = $element[4];
        }
        elsif($status =~ /^Invalid DWORD count:/){
            $hash->{InvalidDWordCount0} = $element[3];
            $hash->{InvalidDWordCount1} = $element[4];
        }
        elsif($status =~ /^Disparity Error count:/){
            $hash->{DisparityErrCount0} = $element[3];
            $hash->{DisparityErrCount1} = $element[4];
        }
        elsif($status =~ /^LOS count:/){
            $hash->{LosCount0} = $element[2];
            $hash->{LosCount1} = $element[3];
        }
        elsif($status =~ /^PHY Reset Problem count:/){
            $hash->{PhyResetProbCnt0} = $element[4];
            $hash->{PhyResetProbCnt1} = $element[5];
        }
        elsif($status =~ /^RECEIVE ERROR COUNT:/){
            $hash->{ReceiveErrorCount} = $element[3];
        }
        elsif($status =~ /^TRANSMIT ERROR COUNT:/){
            $hash->{XmitErrorCount} = $element[3];
        }
        elsif($status =~ /^LINK DOWN COUNT:/){
            $hash->{LinkDownCount} = $element[3];
        }
        elsif($status =~ /^DISPARITY ERROR COUNT:/){
            $hash->{DisparityErrorCount} = $element[3];
        }
        elsif($status =~ /^INVALID DWORD COUNT:/){
            $hash->{InvalidDwordCount} = $element[3];
        }
        elsif($status =~ /^LOS COUNT:/){
            $hash->{LossCount} = $element[2];
        }
        elsif($status =~ /^SPEED NEG/){
            $hash->{SpeedNegErrorCount} = $element[4];
        }
        elsif($status =~ /^CHIP IDENTIFIER/){
            $hash->{ChipIdentifier} = $element[2];
        }
        elsif($status =~ /^CHIP REVISION/){
            $hash->{ChipRevision} = $element[2];
        }
        elsif($status =~ /^SERIAL NUMBER:/){
            if(defined($element[2])){
                $hash->{SerialNumber} = $element[2];
            }
            else{
                $hash->{SerialNumber} = undef;
            }
        }
        elsif($status =~ /^SATA WWN:/){
            $hash->{SataWWN} = $element[2];
        }
        elsif($status =~ /^SAS PORT 0 ID:/){
            $hash->{SasPortId0} = $element[4];
        }
        elsif($status =~ /^SAS PORT 1 ID:/){
            $hash->{SasPortId1} = $element[4];
        }
        elsif($status =~ /^PCB PART NUMBER:/){
            if(defined($element[3])){
                $hash->{PCBPartNumber} = $element[3];
            }
            else{
                $hash->{PCBPartNumber} = undef;
            }
        }
        elsif($status =~ /^OEM NAME:/){
            if(defined($element[2])){
                $hash->{OEMName} = $element[2];
            }
            else{
                $hash->{OEMName} = undef;
            }
        }
        elsif($status =~ /^PCB REVISION:/){
            if(defined($element[2])){
                $hash->{PCBRevision} = $element[2];
            }
            else{
                $hash->{PCBRevision} = undef;
            }
        }
        elsif($status =~ /^MANUF\. DATE:/){
            if(defined($element[2])){
                $hash->{ManufDate} = $element[2];
            }
            else{
                $hash->{ManufDate} = undef;
            }
        }
        elsif($status =~ /^MANUF\. LOCATION:/){
            if(defined($element[2])){
                $hash->{ManufLocation} = $element[2];
            }
            else{
                $hash->{ManufLocation} = undef;
            }
        }
        elsif($status =~ /FIRMWARE REVISION:/){
                $hash->{FWVer} = $element[2];
        }
        elsif($status =~ /BOOT LOADER VERSION:/){
                $hash->{BootLoaderVer} = $element[3];
        }
        elsif($status =~ /^CONFIG DATA VERSION:/){
                $hash->{ConfigDataVer} = $element[3];
        }
    }

    return $hash;

}


=head2 acorn_sata_identify_device

    my $hash = acorn_sata_identify_device(host=>$host, disk_id=>"diskid");

This routine returns a string  of acorn sata_identify_device  command
It croaks on failure.
Following is an example of the output returned by the function:

acorn identify_device: done for 2d.03.12

0000: 045a 3fff c837 0010 0000 0000 003f 0000  --- .Z?..7.......?..
0008: 0000 0000 5a41 4757 3254 3346 2020 2020  --- ....ZAGW2T3F    
0016: 2020 2020 2020 2020 0003 79ef 0034 4b32  ---         ..y..4K2
0024: 414f 4147 3041 4849 5441 4348 4920 4844  --- AOAG0AHITACHI HD
0032: 5337 3235 3035 304b 4c41 3336 3020 2020  --- S725050KLA360   
0040: 2020 2020 2020 2020 2020 2020 2020 8010  ---               ..
0048: 0000 2f00 4000 0200 0200 0007 3fff 0010  --- ../.@.......?...
0056: 003f fc10 00fb 0100 ffff 0fff 0000 0407  --- .?..............
0064: 0003 0078 0078 00f0 0078 0000 0000 0000  --- ...x.x...x......
0072: 0000 0000 0000 001f 0306 0000 005e 0040  --- .............^.@
0080: 00fc 001a 346b 7fe9 4163 3449 3c01 4163  --- ....4k..Ac4I<.Ac
0088: 007f 0078 0000 0000 fffe 0000 80fe 0000  --- ...x............
0096: 0000 0000 0000 0000 6030 3a38 0000 0000  --- ........`0:8....
0104: 0000 0000 0000 5a87 5000 cca2 0dcc 5072  --- ......Z.P.....Pr
0112: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0120: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0128: 0001 000a 0000 0000 2882 0cb1 fe20 0001  --- ........(.... ..
0136: 4000 0000 0000 0000 0000 d1b5 45c5 342a  --- @...........E.4*
0144: 0600 0280 3f7f 00c0 0040 4800 8000 0000  --- ....?....@H.....
0152: 4141 4347 0000 000a 0000 0000 0000 0000  --- AACG............
0160: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0168: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0176: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0184: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0192: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0200: 0000 0000 0000 0000 0000 0000 003f 0000  --- .............?..
0208: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0216: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0224: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0232: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0240: 0000 0000 0000 0000 0000 0000 0000 0000  --- ................
0248: 0000 0000 0000 0000 0000 0000 0000 91a5  --- ................



=over

=item host

(Required) : Filer connnection to use.


=item filermode

(Optional) : The mode of filer: "normal" or "maint". Default is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to 
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item disk_id

(Required) : Disk ID 

=back 

=cut




sub acorn_sata_identify_device {
	my (%opts) = validate(@_, {
				host => { type=>OBJECT, isa=>'Hostrec' },
				filermode => { type=>SCALAR, default=>"normal", optional=>1 },
				conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
				timeout => {type=>SCALAR,default=>300,optional=>1},
				disk_id => {type=>SCALAR},
                     }
                );

	my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"acorn sata_identify_device $opts{disk_id}");
	#print "output is : $output \n";
	if ($output !~ /0000:/){
            croak $output;
       	}
        return $output;
}

=head2 acorn_set_features

    my $hash = acorn_set_features(host=>$host, disk_id=>"diskid");

This routine issues acorn set_features  command
It croaks on failure.

=over

=item host

(Required) : Filer connnection to use.


=item filermode

(Optional) : The mode of filer: "normal" or "maint". Default is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to 
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item disk_id

(Required) : Disk ID 

=item cmd_opts

(Required) : Command options

=back 

=cut



sub acorn_set_features {
	my (%opts) = validate(@_, {
				host => { type=>OBJECT, isa=>'Hostrec' },
				filermode => { type=>SCALAR, default=>"normal", optional=>1 },
				conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
				timeout => {type=>SCALAR,default=>300,optional=>1},
				disk_id => {type=>SCALAR},
				cmd_opts => {type=>SCALAR},
                     }
                );

	my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"acorn set_features $opts{cmd_opts} $opts{disk_id}");
	print "output : $output \n";
	if ($output !~ /acorn set features: complete/){
            croak $output;
       	}
        return 1;
}


=head2 acorn_get_alta_log

    my $hash = acorn_get_alta_log(host=>$host, disk_id=>"diskid");

This routine issues acorn get_alta_log  command
It croaks on failure.

=over

=item host

(Required) : Filer connnection to use.


=item filermode

(Optional) : The mode of filer: "normal" or "maint". Default is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to 
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item disk_id

(Required) : Disk ID 

=back 

=cut





sub acorn_get_alta_log {
	my (%opts) = validate(@_, {
				host => { type=>OBJECT, isa=>'Hostrec' },
				filermode => { type=>SCALAR, default=>"normal", optional=>1 },
				conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
				timeout => {type=>SCALAR,default=>300,optional=>1},
				disk_id => {type=>SCALAR},
                     }
                );

	my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"acorn get_alta_log $opts{disk_id}");
	print "output : $output \n";
	if ($output !~ /Writing device/){
            croak $output;
       	}
        if($opts{filermode} eq "maint"){
            return $output; # We print logs on the console in maintenance mode
        }
        else{
            return 1; # Logs go into /etc/crash directory
        }
}


=head2 acorn_get_alta_crash_dump

    my $hash = acorn_get_alta_crash_dump(host=>$host, disk_id=>"diskid");

This routine issues acorn get_alta_crash_dump  command
It croaks on failure.

=over

=item host

(Required) : Filer connnection to use.


=item filermode

(Optional) : The mode of filer: "normal" or "maint". Default is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to 
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item disk_id

(Required) : Disk ID 

=back 

=cut








sub acorn_get_alta_crash_dump {
	my (%opts) = validate(@_, {
				host => { type=>OBJECT, isa=>'Hostrec' },
				filermode => { type=>SCALAR, default=>"normal", optional=>1 },
				conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
				timeout => {type=>SCALAR,default=>300,optional=>1},
				disk_id => {type=>SCALAR},
                     }
                );

	my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"acorn get_alta_crash_dump $opts{disk_id}");
	print "output : $output \n";
	if ($output !~ /Writing device/){
            croak $output;
       	}
        if($opts{filermode} eq "maint"){
            return $output; # We print logs on the console in maintenance mode
        }
        else{
            return 1; # Logs go into /etc/crash directory
        }
}


=head2 acorn_get_vega_crash_dump

    my $hash = acorn_get_vega_crash_dump(host=>$host, disk_id=>"diskid");

This routine issues acorn get_vega_crash_dump  command
It croaks on failure.

=over

=item host

(Required) : Filer connnection to use.


=item filermode

(Optional) : The mode of filer: "normal" or "maint". Default is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to 
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item disk_id

(Required) : Disk ID 

=back 

=cut


sub acorn_get_vega_crash_dump {
	my (%opts) = validate(@_, {
				host => { type=>OBJECT, isa=>'Hostrec' },
				filermode => { type=>SCALAR, default=>"normal", optional=>1 },
				conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
				timeout => {type=>SCALAR,default=>300,optional=>1},
				disk_id => {type=>SCALAR},
                     }
                );
	my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"acorn get_vega_crash_dump $opts{disk_id}");
	print "output : $output \n";
	if ($output !~ /Writing device/){
            croak $output;
       	}
        if($opts{filermode} eq "maint"){
            return $output; # We print logs on the console in maintenance mode
        }
        else{
            return 1; # Logs go into /etc/crash directory
        }
}


=head1 SEE ALSO

    Where additional information can be found.


=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = acorn

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    anishv@netapp.com,dl-storage-automation@netapp.com

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut



1;
