# $Id: //depot/prod/test/Rsierranevada/lib/Storage/DiskTest.pm#1 $
#
# 

## @summary Library of routines to wrap the disktest command
## @author hansen
## @pod here

=head1 NAME

Storage::DiskTest

=head1 SYNOPSIS

use Storage::DiskTest qw(
    disktest
);

my $results = disktest( 
    filer => $hostrec,
    # all below are optional
    test => $test_type, # R, W, A, WV, V, T
    b_port => $b_port, # 1|0 use B port
    adapter => $adapterlist,
    disk_list => $disks,
    shelf_list => $shelves,
    IO_size => $num_sectors, # x or x-y for T tests 
    runtime => $runtime, # minutes
    IO_timeout => $io_timeout
    que_depth => $que_depth
    pass_count => $pass_count # applys to T tests
    literals => $opt_string,
    timeout => $term_time, # time to let continuous runs complet
);

=cut

package Storage::DiskTest;

use strict;
use warnings;

use Tharn;

use Data::Dumper;

use base qw(
    Exporter
);
our @EXPORT_OK = qw(
    disktest
);

use boot qw(
    transit_to
);

use Params::Validate qw(
    validate 
    validate_pos 
    OBJECT 
    SCALAR
);

BEGIN { 
    Params::Validate::validation_options(
        on_fail => sub {
            my @msgs = @_; 
            chomp(@msgs);
            Carp::confess(Tharn::script_result(@msgs))
        }
   );
}


sub disktest { 
    my (%opts) = validate(
        @_, 
        { 
            filer       =>  { type => OBJECT, isa => 'Hostrec'                 }, 
            test        =>  { type => SCALAR, default => undef, optional => 1, }, 
            b_port      =>  { type => SCALAR, default => 0,     optional => 1, },
            adapter     =>  { type => SCALAR, default => undef, optional => 1, },
            disk_list   =>  { type => SCALAR, default => undef, optional => 1, },
            shelf_list  =>  { type => SCALAR, default => undef, optional => 1, },
            IO_size     =>  { type => SCALAR, default => undef, optional => 1, },
            IO_timeout  =>  { type => SCALAR, default => undef, optional => 1, },
            runtime     =>  { type => SCALAR, default => 2,     optional => 1, },
            que_depth   =>  { type => SCALAR, default => undef, optional => 1, },
            pass_count  =>  { type => SCALAR, default => undef, optional => 1, },
            literals    =>  { type => SCALAR, default => undef, optional => 1, },
            timeout     =>  { type => SCALAR, default => 30,    optional => 1, },
        }
    ); 

    my $filer = $opts{filer};

    my $timeout = ($opts{timeout} * 60) + 900;

    my $cmd_string = "disktest ";

    if (defined $opts{test}) {
        $cmd_string .= " -$opts{test} ";
    }

    $cmd_string .= " -v -y -t $opts{runtime} ";

    if ($opts{b_port}) {
        $cmd_string .= " -B ";
    }
    
    if (defined $opts{IO_size}) {
        if ($opts{IO_size} =~ /(\d+)-(\d+)/) {
            $cmd_string .= " -m $1 -n $2 ";
        } else {
            $cmd_string .= " -n $opts{IO_size} ";
        }
    }

    if (defined $opts{IO_timeout}) {
        $cmd_string .= " -i $opts{IO_timeout} ";
    }

    if (defined $opts{que_depth}) {
        $cmd_string .= " -q $opts{que_depth} ";
    }

    if (defined $opts{pass_count}) {
        $cmd_string .= " -p $opts{pass_count} ";
    }

    if (defined $opts{literals}) {
        $cmd_string .= " $opts{literals} ";
    }

    if (defined $opts{adapter}) {
        $cmd_string .= " -a $opts{adapter} ";
    } 
    

    if (defined $opts{disk_list}) {
        $cmd_string .= " -d $opts{disk_list} ";
    }
    
    if (defined $opts{shelf_list}) {
        $cmd_string .= " -s $opts{shelf_list} ";
    }

    logcomment("disktest command: $cmd_string" );

    my $state = transit_to( $filer, 'MAINT' );

    if (!defined $state) {
        logcomment( "transit_to faile: ", "$@" );
        die ($@);
    }
    
    logcomment("State after transit_to: ", $state);

    $filer = host( $filer );
    
    my $filer_conn = connect("#console");

    my ($match, $before, $after, $prompt) = say( 
        $filer_conn, 
        $cmd_string, 
        '-noprompt',
        '-prompt', "^\\*>",
        '-timeout', $timeout, 
        '-ok' 
    );

    my ($a, $b, $c);

    if ($prompt !~ />/) { 
        # We did not see the end prompt, so we are going to send CTL-C
        logcomment( "Sending CTL-C" );
        ($a, $b, $c) = say(
            $filer_conn,
            "\cC",
            '-nocr',
            '-noprompt',
            '-prompt', "^\\*>",
            '-timeout', $timeout, 
        );
    }

    $after .= "$b $a $c";

	my (undef, undef, $check) = say(
        $filer_conn,
        "\r",
        '-nocr',
        '-noprompt',
        '-prompt', "^\\*>"
    );

	sleep 10;
	$check = " $after " . " $check ";
    my $result_struct = _commandcheck2($check);

    logcomment( Dumper( $result_struct ) );
    
    return $result_struct;
}

###### Subroutine to check for command output.
sub _commandcheck {
    my $text = shift(@_);
    my $result = "";
	if($text !~ /Confidence/){
        tstlog('-warn',"disktest  didnot run at all! \n $text");
	} else {
        my @confidence = split(/hard\s+SCSI\r*\n/,$text);
        my @confidence1 = split(/\r*\n/,$confidence[1]);
        my $confidence1;
		foreach $confidence1 (@confidence1) {
			my @confidence2 = split(/\s+/,$confidence1);
			if ( defined($confidence2[0]) && defined($confidence2[8]) ) {
                logcomment( "$confidence2[0]", "$confidence2[8]" );
                $result .= "$confidence2[0]:$confidence2[8]\n";
                if($confidence2[8] !~ /1\.00/) {
                    tstlog(
                        '-warn',
                        "Confidence factor for adapter $confidence2[0] is $confidence2[8]. Should be 1.00. Probably some drive or loop problem"
                    );
                }
			}
		}
	}
    return $result;
}


sub _commandcheck2 {

    my $output = shift(@_);

    my @c = split(/\r*\n/,$output);
	my $struct = {};
	my $sas = 0;
    my $sas_total = 0;
	my $fcal = 0;
	my $scsi = 0;
	foreach my $d (@c){

		my @arr = split(/\s+/,$d);

        # if the array is a single element skip to next line

        if ($#arr == 0) {
            if ( ($sas == 1) && ($arr[0] =~ /=+/) ) {
                $sas = 0;
                $sas_total = 1;
            }
            next;
        }

		if(defined($arr[$#arr])) {
            if ($arr[$#arr] =~ /FC-AL/) {
                $fcal = 1;
                $sas = 0;
                $scsi = 0;
                $sas_total = 0;
            }
            elsif($arr[$#arr] =~ /SAS/){
                $sas = 1;
                $fcal = 0;
                $scsi = 0;
                $sas_total = 0;
            }
            elsif($arr[$#arr] =~ /SCSI/){
                $sas = 0;
                $fcal = 0;
                $scsi = 1;
                $sas_total = 0;
            } 
            
            logcomment( "fcal: $fcal", "sas: $sas", "sas_total: $sas_total" , "scsi: $scsi", @arr );
            
            if($fcal){ 
                if(($arr[1] =~ /^\d+$/) && ($arr[2] =~ /^\d+$/)){
                    $struct->{$arr[0]}->{LinkFailureCount} = $arr[1];
                    $struct->{$arr[0]}->{TransportErrorCount} = $arr[2];
                    $struct->{$arr[0]}->{LossOfSyncCount} = $arr[3];
                    $struct->{$arr[0]}->{InvalidCRCCount} = $arr[4];
                    $struct->{$arr[0]}->{FrameInCount} = $arr[5];
                    $struct->{$arr[0]}->{FrameOutCount} = $arr[6];
                    $struct->{$arr[0]}->{ConfidenceFactor} = $arr[7];
                }
            }
            
            if($sas){
                if(($arr[2] =~ /^\d+$/) && ($arr[3] =~ /^\d+$/)){
                    $struct->{$arr[1]}->{InvDWrdCnt} = $arr[2];
                    $struct->{$arr[1]}->{DispErrCnt} = $arr[3];
                    $struct->{$arr[1]}->{LossSyncCnt} = $arr[4];
                    $struct->{$arr[1]}->{PhyResProb} = $arr[5];
                    $struct->{$arr[1]}->{PhyChngCtH} = $arr[6];
                    $struct->{$arr[1]}->{PhyChngCtS} = $arr[7];
                    $struct->{$arr[1]}->{DevUdrnErrs} = $arr[8];
                    $struct->{$arr[1]}->{DevOvrnErrs} = $arr[9];
                    $struct->{$arr[1]}->{TrnsErrcnt} = $arr[10];
                    $struct->{$arr[1]}->{CRCErrCnt} = $arr[11]; 
                    $struct->{$arr[1]}->{DevQsceCnt} = $arr[12];
                    $struct->{$arr[1]}->{FrmsInCnt} = $arr[13];
                    $struct->{$arr[1]}->{FrmsOutCnt} = $arr[14];
                    
                    if(defined($arr[15])){
                        $struct->{$arr[1]}->{ConfidenceFactor} = $arr[15];
                    }
                }
            }
            
            if($sas_total){
                # format of total is odd for sas. the device name is something
                # like 0c. 89 the space needs to be extracted.
                if (($arr[1] =~ /\.$/) && (($arr[2] =~ /^\d+$/) && ($arr[3] =~ /^\d+$/))) {
                    $arr[1] = $arr[1] . $arr[2];
                    $struct->{$arr[1]}->{InvDWrdCnt} = $arr[3];
                    $struct->{$arr[1]}->{DispErrCnt} = $arr[4];
                    $struct->{$arr[1]}->{LossSyncCnt} = $arr[5];
                    $struct->{$arr[1]}->{PhyResProb} = $arr[6];
                    $struct->{$arr[1]}->{PhyChngCtH} = $arr[7];
                    $struct->{$arr[1]}->{PhyChngCtS} = $arr[8];
                    $struct->{$arr[1]}->{DevUdrnErrs} = $arr[9];
                    $struct->{$arr[1]}->{DevOvrnErrs} = $arr[10];
                    $struct->{$arr[1]}->{TrnsErrcnt} = $arr[11];
                    $struct->{$arr[1]}->{CRCErrCnt} = $arr[12]; 
                    $struct->{$arr[1]}->{DevQsceCnt} = $arr[13];
                    $struct->{$arr[1]}->{FrmsInCnt} = $arr[14];
                    $struct->{$arr[1]}->{FrmsOutCnt} = $arr[15];
                    
                    if(defined($arr[16])){
                        $struct->{$arr[1]}->{ConfidenceFactor} = $arr[16];
                    }
                }
            }

            if($scsi){
                if(($arr[1] =~ /^\d+$/) && ($arr[2] =~ /^\d+$/)){
                    $struct->{$arr[0]}->{WriteCount} = $arr[1];
                    $struct->{$arr[0]}->{ReadCount} = $arr[2];
                    $struct->{$arr[0]}->{IOPS} = $arr[3];
                    $struct->{$arr[0]}->{DataRate} = $arr[4];
                    $struct->{$arr[0]}->{DataXferSize} = $arr[5];

                    my $serror = $arr[6];
                    
                    $serror =~ s/\///;

                    $struct->{$arr[0]}->{DeviceErrorsS} = $serror;
                    $struct->{$arr[0]}->{DeviceErrorsH} = $arr[7];
                    $struct->{$arr[0]}->{ConfidenceFactorSCSI} = $arr[8]; 
                }
            } 
        }
	}
	return $struct ;
}

    


=cut

disktest -B -R -n 8 -t 2 -v -y -s 1d:1 1d:2 1d:3
SEQUENTIAL_READ using [4kb I/O - timeout: 300s queuedepth: 3 ] -- Max LBA Range 6MB

disktest: will use both A and B loops for FC disks.
Disks selected: 1d.60 1d.33 1d.35 1d.32 1d.37 1d.38 1d.42 1d.41 1d.44 1d.45 1d.18 1d.19 1d.20 1d.25 1d.28 1d.51 1d.48 1d.52 1d.54 1d.59 1d.57 
This test will take about 2 minutes to complete.

disktest performs reads and writes to the savecore area of
the selected disk(s). Ensure savecore has been run prior
to executing disktest to avoid losing a corefile.


Running disk test ... (^C to stop)
DBG: cmd_disktest_disk_wait_for_notifications: Timer expired .. no notifications
Loop on FC Adapter 0a open ... skipping.
Loop on FC Adapter 0b open ... skipping.
Loop on FC Adapter 1a open ... skipping.
Loop on FC Adapter 1b open ... skipping.
Loop on FC Adapter 1c open ... skipping.
Time limit reached - stopping I/O


-----------------------------------[Cumulative RCA Errors]------------------------------
                 Link  Transport   Loss of   Invalid    Frame In   Frame Out  Confidence
Loop          Failure      error      sync       CRC       count       count      Factor
  ID            count      count     count     count                               FC-AL
----------------------------------------------------------------------------------------
1d.51               0          0         0         0      131290      393866
1d.48               0          0         0         0      131318      393950
1d.52               0          0         0         0      131308      393920
1d.54               0          0         0         0      131319      393953
1d.59               0          0         0         0      131319      393953
1d.57               0          0         0         0      131306      393914
1d.60               0          0         0         0      131288      393860
1d.33               0          0         0         0      131288      393860
1d.35               0          0         0         0      131293      393875
1d.32               0          0         0         0      131309      393923
1d.37               0          0         0         0      131304      393908
1d.38               0          0         0         0      131281      393839
1d.42               0          0         0         0      131310      393926
1d.41               0          0         0         0      131301      393899
1d.44               0          0         0         0      131301      393899
1d.45               0          0         0         0      131301      393899
1d.18               0          0         0         0      131307      393917
1d.19               0          0         0         0      131311      393929
1d.20               0          0         0         0      131283      393845
1d.25               0          0         0         0      131289      393863
1d.28               0          0         0         0      131286      393854
======================================================================================
1d                  0          0         0         0     8271852     2757312        1.00 [actual: 0 / BER allowed: 0]


---------------------------------------------------------------------------------------
Adapter           Write      Read      IOPS     Data        Data     Device  Confidence
 ID               count     count               Rate        xfer     errors      Factor
                                                MB/S        size   soft/hard       SCSI
---------------------------------------------------------------------------------------
1d.60                 0    131287      1060      4.2          4k      0/   0
1d.33                 0    131287      1062      4.2          4k      0/   0
1d.35                 0    131292      1057      4.1          4k      0/   0
1d.32                 0    131308      1069      4.2          4k      0/   0
1d.37                 0    131303      1053      4.1          4k      0/   0
1d.38                 0    131280      1067      4.2          4k      0/   0
1d.42                 0    131309      1066      4.2          4k      0/   0
1d.41                 0    131300      1059      4.2          4k      0/   0
1d.44                 0    131300      1063      4.2          4k      0/   0
1d.45                 0    131300      1061      4.2          4k      0/   0
1d.18                 0    131306      1056      4.1          4k      0/   0
1d.19                 0    131310      1055      4.1          4k      0/   0
1d.20                 0    131282      1064      4.2          4k      0/   0
1d.25                 0    131288      1065      4.2          4k      0/   0
1d.28                 0    131285      1052      4.1          4k      0/   0
1d.51                 0    131289      1068      4.2          4k      0/   0
1d.48                 0    131317      1070      4.2          4k      0/   0
1d.52                 0    131307      1059      4.2          4k      0/   0
1d.54                 0    131318      1058      4.1          4k      0/   0
1d.59                 0    131318      1051      4.1          4k      0/   0
1d.57                 0    131305      1054      4.1          4k      0/   0
=====================================================================================
1d                    0   2757291     22083     86.2          4k      0/   0       1.00



disktest -y -t 45

-----------------------------------[Cumulative RCA Errors]------------------------------
                 Link  Transport   Loss of   Invalid    Frame In   Frame Out  Confidence
Loop          Failure      error      sync       CRC       count       count      Factor
  ID            count      count     count     count                               FC-AL
----------------------------------------------------------------------------------------
0a                  0          0         0         0    59933979    59940219        1.00 [actual: 0 / BER allowed: 0]
0b                  0          0         0         0    74551038    74559878        1.00 [actual: 0 / BER allowed: 0]


-----------------------------------[Cumulative RCA Errors]------------------------------
          Inv. Disp Loss  Phy  Phy  Phy  Dev  Dev Trns  CRC  Dev  Frms    Frms Confidnce
Shelf /   DWrd  Err Sync  Res Chng Chng Udrn Ovrn  Err  Err Qsce    In     Out    Factor
Disk ID    Cnt  Cnt  Cnt Prob Ct-H Ct-S Errs Errs  Cnt  Cnt  Cnt   Cnt     Cnt       SAS
--------  ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ---- ------- ------- -------
 0c.90.0      0    0    0    0   48    0    0    0    0    0    0  4880614  4880103
 0c.90.1      0    0    0    0   48    0    0    0    0    0    0  4833945  4833562
 0c.90.2      0    0    0    0   52    0    0    0    0    0    0  5051695  5051312
 0c.90.3      0    0    0    0   50    0    0    0    0    0    0  5027514  5027003
 0c.90.4      0    0    0    0   52    0    0    0    0    0    0  4995404  4994893
 0c.90.5      0    0    0    0   60    0    0    0    0    0    0  5020364  5019853
 0c.90.6      0    0    0    0   53    0    0    0    0    0    0      154      154
 0c.90.7      0    0    0    0   54    0    0    0    0    0    0  4854354  4853843
 0c.90.8      0    0    0    0   54    0    0    0    0    0    0  4928975  4928592
 0c.90.9      0    0    0    0   47    0    0    0    0    0    0      154      154
 0c.90.10     0    0    0    0   49    0    0    0    0    0    0      154      154
 0c.90.11     0    0    0    0   49    0    0    0    0    0    0      154      154
 0c.90.12     0    0    0    0   49    0    0    0    0    0    0      154      154
 0c.90.13     0    0    0    0   51    0    0    0    0    0    0      154      154
 0c.90.14     0    0    0    0   49    0    0    0    0    0    0      154      154
 0c.90.15     0    0    0    0   49    0    0    0    0    0    0      154      154
 0c.90.16     0    0    0    0   47    0    0    0    0    0    0      154      154
 0c.90.17     0    0    0    0   46    0    0    0    0    0    0      154      154
 0c.90.18     0    0    0    0   46    0    0    0    0    0    0      154      154
 0c.90.19     0    0    0    0   59    0    0    0    0    0    0  4840574  4840063
 0c.  90     0    0    0    0 XX      0    0    0    0    0    0 44435133 44430918  1.00
 [total: 0 / BER allowed: 0]

---------------------------------------------------------------------------------------
Adapter           Write      Read      IOPS     Data        Data     Device  Confidence
 ID               count     count               Rate        xfer     errors      Factor
                                                MB/S        size   soft/hard       SCSI
---------------------------------------------------------------------------------------
0a               114822    114822        84     84.0       1024k      0/   0       1.00
0b               142828    142828       105    105.0       1024k      0/   0       1.00
0c               341770    341770       252    252.0       1024k      0/   0       1.00


command for testing: 

ntest -hw -cf -console NATE_LIB+=/u/hansen/p4/ontap/test TESTNO=1 /u/hansen/p4/ontap/test/storage/disk/disktest.thpl

=cut

1;
