#$Id: //depot/prod/test/Rsierranevada/lib/Storage/Ems.pm#3 $
##
## @summary Ems - Library module for ems command
## @author sajan@netapp.com,dl-storage-automation
## @status testme
## @change 10-31-2006 anishv: Added ems_error_monitor function
## @change 08-01-2007 anishv: Added ignore_error_values option to ems_error_monitor function
## @change 09-25-2007 anishv: Added key_value_errors option to ems_error_monitor function
## @change 02-20-2008 sreedhar: Removed unwanted console messages from the command ems log dump 1s command output
## @change 11-25-2008 anishv: Added ems.autosuppress.enable on
## @change 08-05-2009 hansen: Changed {type} key to {event_type} to avoid
## @change 09-25-2009 deepak1:Changed the say command in ems_log_get_mark function  corruption of field for certain ems message (burt 370377)
## @change 01-08-2010 anishv: Replaced say call with filer_cmd function which should handle both 7-mode and 10-mode nodescope connections
## @change 01-08-2010 anishv: Added two new functions: ems_log_dump() and ems_log_search()
## @change 01-08-2010 anishv: Fixed ems_event_status function( OnTAP output changed)
## @change 01-08-2010 anishv: Used set_filer_options/get_filer_options in the get_mark function
## @change 01-08-2010 anishv: Using ems_log_dump() function in ems_error_monitor() and ems_log_check() functions.Previously these functions were issuing ems log dump command twice which was causing unnecessary delays and filling extra logs. Now it issues ems log dump command only once.
##
## @change 04-15-2010 anishv: fix for burt: 407445 
# @change 09-25-2009 deepak1: Changed the say command in ems_log_get_mark function  corruption of field for certain ems message (burt 370377)
# @change 04-04-2013 villas: added sub ems_log_rotate to rotate ems logs
#                            added code to rotate ems log in the begining of the script based on param EMSROTATE
## @pod here

=head1 NAME

Storage::Ems

=head1 DESCRIPTION

This library defines a set of interfaces to interrogate ems subsystem 
and return (pointer to hash) ems log data

=head1 SYNOPSIS

use Storage::Ems qw(
    ems_log_get_mark 
    ems_log_dump_by_type 
    ems_log_dump_by_id 
    ems_event_status 
    ems_get_value_by_key
);
   

my $host = host($FILER);
my $mark = ems_log_get_mark(host=>$host);
$event = $event_by_type->{"dfu.firmwareUpToDate"};
if ($event eq "") {
    croak "Disk firmware download didn't complete in given time";
}

OR 

my $host = host($FILER);
my $mark = ems_log_get_mark(host=>$host);
# Check if adpater 6a has been offlined or not
my $ret = ems_log_check(
    host=>$host, 
    eventtype=>"fci.adapter.offline", 
    key=>"adapterName" 
    value=>"6a", 
    timestamp=>$mark
);

if ($ret == 0) {
    logcomment("The specified event is not found in the ems log");
} else {
    logcomment("Found the specified ems event $ret times in the ems log");
}

=head1 ROUTINES

=cut

package Storage::Ems;

use strict;
use warnings;

BEGIN {

    use base qw(Exporter);
    our @EXPORT_OK = qw(
            ems_log_get_mark 
            ems_log_dump 
            ems_log_dump_by_type 
            ems_log_dump_by_id 
            ems_event_status
            ems_log_check
            ems_error_monitor
            ems_get_value_by_key
            ems_log_search
            ems_build_error_hash
            ems_log_rotate
            );

    Params::Validate::validation_options(
            on_fail => sub {my @msgs = @_; chomp(@msgs);
            Carp::confess(Tharn::script_result(@msgs))}
            );
}

use Carp qw(confess croak);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF );
use Tharn qw(say script_result host logcomment logresult param);
use Storage::Misc qw(filer_cmd client_cmd);
use Storage::SASUtils qw(set_filer_options get_filer_options);

use Utils qw(make_conn);
use Data::Dumper;
use NACL::C::Node;
use vars qw ( $EMSROTATE );

#Rotates the ems log in based on param EMSROTATE in the beginig of the script
#By default EMSROTATE is zero
my $is_emsrotate = Tharn::param('EMSROTATE', '-default', 0);
if($is_emsrotate){
    logcomment('Clearing the ems logs');
    my @nodes = NACL::C::Node->find();
    foreach (@nodes){
        my $apiset_obj = $_->get_7m_or_nodescope_apiset();
        ems_log_rotate(host => $_->hostrec, apiset_obj => $apiset_obj);
    }
}

#----------------------------------------------------------------------
# The exported rotines are coded in the following section
#----------------------------------------------------------------------
=head2 ems_log_rotate

This routine clear the ems logs

my $host = host($FILER);
ems_log_rotate(host=>$host);
my $apiset_obj =  $node_obj->get_7m_or_nodescope_apiset();
ems_log_rotate(host=>$host, apiset_obj => $apiset_obj);

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item apiset_obj

(Optional) : 7mode or nodescope object in case cmode

=back

=cut

sub ems_log_rotate {
    my (%opts) = validate(@_, { host => { type=>OBJECT, isa=>'Hostrec', },
            apiset_obj => { type=>OBJECT, optional=>1 },
            });
    my $apiset_obj = $opts{apiset_obj};
    if(!defined($apiset_obj)){
        my $filer_node = NACL::C::Node->new( hostrec => $opts{host} );
        $apiset_obj = $filer_node->get_7m_or_nodescope_apiset();
    }
    $apiset_obj->execute_command(command => 'priv set diag; ems rotate');
}


=head2 ems_get_merk

This routine returns the current mark(timestamp in format <n>t) of the event 
log

my $host = host($FILER);
my $ret = ems_get_mark(host=>$host);
my $ret = ems_get_mark(host=>$host, timeout=>60);

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item timeout

(Optional) : Timeout value for the say. Default : 300

=back

=cut

sub ems_log_get_mark {
    my (%opts) = validate(@_, { host => { type=>OBJECT, isa=>'Hostrec',default=>"",optional=>1 },
            timeout => { type=>SCALAR, default=>1200, optional=>1 },
            say_timeout => { type=>SCALAR, default=>300, optional=>1 },
            ems_file => {type=>SCALAR,default=>"none",optional=>1},
            client => {type=>SCALAR,default=>undef,optional=>1},
            });

    if($opts{ems_file} eq "none"){

#my $conn = make_conn($opts{host});
# To Disable ems duplicate events
#my $option = get_filer_options(host=>$opts{host},options=>"ems.log.dupsuppress.enable",timeout=>$opts{timeout});
#if($option->{'ems.log.dupsuppress.enable'} ne "off"){
#    eval {set_filer_options(host=>$opts{host},options=>"ems.log.dupsuppress.enable",value=>"off",checkoptions=>0,timeout=>$opts{say_timeout});};
#}
#$option = get_filer_options(host=>$opts{host},options=>"ems.autosuppress.enable",timeout=>$opts{timeout});
#if($option->{'ems.autosuppress.enable'} ne "off"){
    set_filer_options(host=>$opts{host},options=>"ems.autosuppress.enable",value=>"off",checkoptions=>0,timeout=>$opts{say_timeout});
#}

#say($conn, "options ems.log.dupsuppress.enable off", "-timeout", $opts{timeout});
#say($conn, "options ems.autosuppress.enable off", "-timeout", $opts{timeout});
#sleep 2;
    my $cmd = "ems log dump 1s";
#my ($match, undef, $after) =  say($conn, "$cmd", "^<NR", "-timeout", $opts{timeout});
#my $output = $match . $after;
    my $output = filer_cmd(host=>$opts{host},timeout=>$opts{timeout},cmd=>$cmd);
    my(@lines) = split(/\r*\n/, $output);
# removing rsh related console messages that were reported from few clients,from the command output
    @lines = grep /<NR/, @lines;
#Use header (NR) only to get the timestamp
    my %emslog_nr = emslog_parseNR($lines[0]);
    return ($emslog_nr{t} . "t");
    }
    else{
        my @lines;
        my $found = 0;
        my @reversed;
        open(FILE,"< $opts{ems_file}") or die "Could not open file $opts{ems_file} for reading: $!\n";
        while(<FILE>){
            push @lines,$_;
        }
        @reversed = reverse(@lines);
        foreach my $line (@reversed){
            if($line =~ /^\<LR.+t=\"(\d+)\"/){
                $found = 1;
                return ($1 . "t");
            }
        }
        close(FILE);
        unless($found){
            croak "Did not find any line in file $opts{ems_file} starting with \<LR and having t= timestamp";
        }
    }
}









=head2 ems_log_dump_by_type

This routine returns pointer to hash table having event name as key and 
array of event ids as value.

my $host = host($FILER);
my $ret = ems_log_dump_by_type(host=>$host);
OR
my $mark = ems_log_get_mark(host=>$host);
my $event_by_type = ems_log_dump_by_type(host=>$host, timestamp=>$mark);
$event = $event_by_type->{"dfu.firmwareDownloading"};
if ($event eq "") {
    croak "Disk firmware download is not started yet";
}

$event = $event_by_type->{"dfu.firmwareUpToDate"};
if ($event eq "") {
    croak "Disk firmware download didn't complete in given time";
}

The output looks like:
$VAR1 = {
          'fci.adapter.offlining' => [
                                       '1133838353/531',
                                       '1133838353/539'
                                     ],
          'fci.adapter.offline' => [
                                     '1133838353/532',
                                     '1133838353/540'
                                   ],
          'fci.adapter.onlining' => [
                                      '1133838353/536',
                                      '1133838353/543'
                                    ],
          'unowned.disk.reminder' => [
                                       '1133838353/529'
                                     ],
          'monitor.shelf.fault' => [
                                     '1133838353/527',
                                     '1133838353/528'
                                   ],
          'fci.adapter.online' => [
                                    '1133838353/537',
                                    '1133838353/544'
                                  ],
          'kern.syslog.msg' => [
                                 '1133838353/530',
                                 '1133838353/533',
                                 '1133838353/534',
                                 '1133838353/535',
                                 '1133838353/538',
                                 '1133838353/542'
                               ],
          'config.NotMultiPath' => [
                                     '1133838353/541'
                                   ],
          'kern.uptime.filer' => [
                                   '1133838353/526'
                                 ]
        };

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item timestamp

(Optional) : Get the event from timestamp Default : ""(means only header)

Timestamp semantic can be : 

<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional) : The logfile to get events from. Default : "" (means current)

=item timeout

(Optional) : Timeout value for the say. Default : 1200

=back

=cut

sub ems_log_dump_by_type {
    my (%opts) = validate(@_, { host => { type=>OBJECT, isa=>'Hostrec',default=>"",optional=>1 },
            timestamp => { type=>SCALAR,default=>"", optional=>1 },
            log => { type=>SCALAR,default=>"", optional=>1 },
            verbose => { type=>SCALAR,default=>"yes", optional=>1 },
            timeout => { type=>SCALAR, default=>1200, optional=>1 },
            ems_file => {type=>SCALAR,default=>"none",optional=>1},
            });

    my $emslog_entry;
    my @emslog_events;
    my @emslog_event_lines;
    my %emslog_event_by_id;
    my %emslog_event_by_type;
    my %emslog_nr;
    my @lines;
    my $verbose = " ";
    if($opts{verbose} eq "no"){
        $verbose = "-noverbose";
    }
    if ($opts{ems_file} eq "none"){
#	my $conn = make_conn($opts{host});
        my $cmd = "ems log dump $opts{log} $opts{timestamp}";
        logcomment("Issuing $cmd  $verbose");
#my (undef, undef, $output) = say($conn, "$cmd","$verbose", "-timeout", $opts{timeout});
        my $output = filer_cmd(host=>$opts{host},timeout=>$opts{timeout},cmd=>$cmd);
        @lines = split(/\r*\n/, $output);
    }
    else{
        open(FILE,"< $opts{ems_file}") or die "Could not open file $opts{ems_file} for reading: $!\n";
        while(<FILE>){
            push @lines,$_;
        }
    }

    foreach my $line (@lines) {
        if (!emslog_parse($line, \%emslog_nr, \$emslog_entry)) {
            next;
        }
        emslog_process_line($emslog_entry, \@emslog_events, 
                \@emslog_event_lines, 
                \%emslog_event_by_id, 
                \%emslog_event_by_type);
        $emslog_entry = "";
    }
    return(\%emslog_event_by_type);
}

=head2 ems_log_dump_by_id

This routine returns pointer to hash table having event info by id

my $ret = ems_log_dump_by_id(host=>$host);
my $ret = ems_log_dump_by_id(host=>$host, timeout=>300, timestamp=>23456788t);

The output looks like:
$VAR1 = {
          '1133838353/530' => {
                                'vfilerName' => '',
                                'header' => {
                                              'n' => 'raid-u23',
                                              'o' => 'rc',
                                              'p' => 6,
                                              's' => 'OK',
                                              't' => '1134007568',
                                              'd' => '08Dec2005 02:06:08',
                                              'vf' => '',
                                              'id' => '1133838353/530'
                                            },
                                'msg' => 'Taking adapter 6a offline at administrator&apos;s request.&nl;',
                                'event_type' => 'kern.syslog.msg',
                                'attr' => 'msg,vfilerName'
                              },
          '1133838353/531' => {
                                'header' => {
                                              'n' => 'raid-u23',
                                              'o' => 'rc',
                                              'p' => 6,
                                              's' => 'OK',
                                              't' => '1134007568',
                                              'd' => '08Dec2005 02:06:08',
                                              'vf' => '',
                                              'id' => '1133838353/531'
                                            },
                                'adapterName' => '6a',
                                'event_type' => 'fci.adapter.offlining',
                                'attr' => 'adapterName'
                              },
          .
          .
          .
}


The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item timestamp

(Optional) : Get the event from this timestamp Default : ""(means only header)

Timestamp semantic can be : 

<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional) : The logfile to get events from. Default : "" (means current)

=item timeout

(Optional) : Timeout value for the say. Default : 1200: Increasing the default
value as the log file might be quite big and can take longer to dump the file 

=back

=cut

sub ems_log_dump_by_id {
    my (%opts) = validate(@_, { host => { type=>OBJECT, isa=>'Hostrec',default=>"",optional=>1 },
            timestamp => { type=>SCALAR,default=>"", optional=>1 },
            log => { type=>SCALAR,default=>"", optional=>1 },
            verbose => { type=>SCALAR,default=>"yes", optional=>1 },
            timeout => { type=>SCALAR, default=>1200, optional=>1 },
            ems_file => {type=>SCALAR,default=>"none",optional=>1},
            });

    my $emslog_entry;
    my @emslog_events;
    my @emslog_event_lines;
    my %emslog_event_by_id;
    my %emslog_event_by_type;
    my %emslog_nr;
    my @lines;
    my $verbose = " ";
    if($opts{verbose} eq "no"){
        $verbose = "-noverbose";
    }
    if ($opts{ems_file} eq "none"){
#my $conn = make_conn($opts{host});
        my $cmd = "ems log dump $opts{log} $opts{timestamp}";
        logcomment("Issuing $cmd  $verbose");
#my (undef, undef, $output) = say($conn, "$cmd","$verbose", "-timeout", $opts{timeout});
        my $output = filer_cmd(host=>$opts{host},timeout=>$opts{timeout},cmd=>$cmd);
        @lines = split(/\r*\n/, $output);
    }
    else{
        open(FILE,"< $opts{ems_file}") or die "Could not open file $opts{ems_file} for reading: $!\n";
        while(<FILE>){
            push @lines,$_;
        }
    }
    foreach my $line (@lines) {
        if (!emslog_parse($line, \%emslog_nr, \$emslog_entry)) {
            next;
        }
        emslog_process_line($emslog_entry, \@emslog_events, 
                \@emslog_event_lines, 
                \%emslog_event_by_id, 
                \%emslog_event_by_type);
        $emslog_entry = "";
    }
    return(\%emslog_event_by_id);
}

=head2 ems_event_status

This routine returns pointer to hash table having event summery info return 
by the command "ems event status"

my $ret = ems_event_status(host=>$host);

The output looks like:

$VAR1 = {
          'monitor.globalStatus.ok' => {
                                         'AutoSuppr' => '0',
                                         'Drops' => '2',
                                         'LastTime' => '08Jan2010 14:16:00',
                                         'DupSuppr' => '0',
                                         'TimerSuppr' => '0',
                                         'Indications' => '',
                                         'Priority' => 'INFO'
                                       },
          'monitor.globalStatus.critical' => {
                                               'AutoSuppr' => '0',
                                               'Drops' => '3',
                                               'LastTime' => '08Jan2010 14:07:00',
                                               'DupSuppr' => '0',
                                               'TimerSuppr' => '0',
                                               'Indications' => '',
                                               'Priority' => 'CRIT'
                                             },
          'netif.linkUp' => {
                              'AutoSuppr' => '0',
                              'Drops' => '3',
                              'LastTime' => '07Jan2010 11:54:31',
                              'DupSuppr' => '0',
                              'TimerSuppr' => '0',
                              'Indications' => '',
                              'Priority' => 'INFO'
                            },
...}

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item timeout

(Optional) : Timeout value for the say. Default : 300

=back

=cut

sub ems_event_status {
    my (%opts) = validate(@_, { host => { type=>OBJECT, isa=>'Hostrec' },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            });

    my %ems_event_status;

    my $cmd = "ems event status";
    my $event = "";

    my $output = filer_cmd(host=>$opts{host},timeout=>$opts{timeout},cmd=>"$cmd");
    my (@lines) = split(/\r*\n/, $output);
    foreach my $line (@lines) {
        if ($line =~ /^\s*(\w+)\./) {    
            my @event_info = split(/\s+/, $line); 
            my @event = split(/:/, $event_info[0]); 
            $event = $event[0];
            $ems_event_status{$event}{'Priority'} = $event[1];
            if(defined($event_info[2])){
                $ems_event_status{$event}{'LastTime'} = "$event_info[1]" . " $event_info[2]";
            }
            else{
                $ems_event_status{$event}{'LastTime'} = $event_info[1];
            }

        }
        elsif(($line =~ /^\s*\d+\s+/) && ($event =~ /\./)){
            my @event_info = split(/\s+/, $line); 
            $ems_event_status{$event}{'Indications'} = $event_info[0];
            $ems_event_status{$event}{'Drops'} = $event_info[1];
            $ems_event_status{$event}{'DupSuppr'} = $event_info[2];
            $ems_event_status{$event}{'TimerSuppr'} = $event_info[3];
            $ems_event_status{$event}{'AutoSuppr'} = $event_info[4];
        }
        else{
            $event = "";
        }
    }
    return(\%ems_event_status);
}

=head2 ems_log_check

This routine returns 0 if a given event on given objectname is not found from a specified timestamp otherwise returns <n> for the n number of instances found in the log.

my $host = host($FILER);
my $mark = ems_log_get_mark(host=>$host);

my $ret = ems_log_check(
    host=>$host, 
    eventtype=>"fci.adapter.offline", 
    key=>"adapterName" 
    value=>"6a", 
    timestamp=>$mark
);

if ($ret == 0) {
    logcomment("The specified event is not found in the ems log");
} else {
    logcomment("Found the specified ems event $ret times in the ems log");
}

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item eventtype

(Required) : The type of events to look into, like "fci_adapter_offline"...

=item key

(Required) : What is the key (the objectname for a given event type) to look into the event 
message. For example, in the following ems message key is "adapterName":

<LR d="07Dec2005 18:45:31" n="raid-u23" t="1133981131" id="1133838353/404" p="6" s="OK" o="ispfc_init_2" vf="">
<fci_adapter_offline_1
        adapterName="6a"/>
</LR>

=item value

(Required) : What is the value (the objectname for a given event type) to look into for the 
given event type and key. To check for the above ems message the value would be "6a".

=item timestamp

(Required) : Get the event from this timestamp.

Timestamp semantic can be : 

<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional) : The logfile to get events from. Default : "" (means current)

=item timeout

(Optional) : Timeout value for the say. Default : 1200: Increasing the default
value as the log file might be quite big and can take longer to dump the file 

=back

=cut

sub ems_log_check {
    my (%opts) = validate(
            @_, 
            { 
            host => { type=>OBJECT, isa=>'Hostrec' }, 
            eventtype => { type=>SCALAR },
            key => { type=>SCALAR },
            value => { type=>SCALAR },
            timestamp => { type=>SCALAR },
            log => { type=>SCALAR, default=>"", optional=>1 },
            timeout => { type=>SCALAR, default=>1200, optional=>1 },
            patternmatch => { type=>SCALAR, default=>0, optional=>1 },
            }
            );

    my $event_by_type;
    my $event_by_id;
    my $hash;

    if (defined($opts{log})) {
        $hash = ems_log_dump(
                host=>$opts{host}, 
                timestamp=>$opts{timestamp}, 
                timeout=>$opts{timeout}, 
                log=>$opts{log}
                );
    } else {
        $hash = ems_log_dump(
                host=>$opts{host}, 
                timestamp=>$opts{timestamp}, 
                timeout=>$opts{timeout}
                );
    }
    my $event = $hash->{by_type}->{"$opts{eventtype}"};
    if(!defined($event)) {
        return (0);
    } else {
        my $found;
        $found = 0;
        my $n = 0;
        my %errfound;
        my @vals = split(/\&/,$opts{value});
        foreach my $val (@vals){
            $errfound{$val} = $n;
        }
        my $num = 0;
        if($opts{patternmatch}){
            foreach my $id (@{$event}) {
#my @vals = split(/\&/,$opts{value});
                foreach my $val (@vals){
                    if ($hash->{by_id}->{$id}{$opts{key}} =~ /$val/){
                        $n++;
                        $errfound{$val} = $n;
                    }
                }
            }
            my($k,$v);
            while (($k,$v) = each(%errfound)){
                if($v == 0){
                    logcomment("Event $k not seen");
                }
            }
            my @val1 = sort(values(%errfound));
            return $val1[0];

        }
        else{
            foreach my $id (@{$event}) {
                if ($hash->{by_id}->{$id}{$opts{key}} eq "$opts{value}") {
                    ++$found; 
                }
            }

        }
        return($found);
    }
}



=head2 ems_error_monitor
	my @ret = ems_error_monitor(host=>$host, mark=>$mark, filermode=>$filermode,monitor_time=>$monitor_time,sleep_time=>$sleep_time,error_values=>\@errors,fatal_on_failure=>"yes");
	
This subrutine checks for ems errors 

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item mark

(Required) : Time marker from where to start checking the errors

=item monitor_time 

(Optional): Monitor time. Default is 1 sec.

=item sleep_time

(Optional): Sleep time between error checks. Default is 1 sec.

=item error_values

(Optional): Reference to an Array with a list of errors to be checked. By default it will use a list defined in the library

=item additional_error_values

(Optional): Reference to an Array with a list of additional errors to be checked. The list of additional_error_values will be appended to the list of errors already defined in the function. By default it will use the  list defined in the library without additional errors.

=item key_value_errors

(Optional): Reference to a hash with a list of additional errors with key value pairs to be checked. The list of additional_error_values will be appended to the list of errors already defined in the function. By default it will use the  list defined in the library without additional errors. For example if we want to check for the following message:

<LR d="26Sep2007 02:40:44" n="stor-u104" pn="stor-u105" t="1190774444" id="1190710933/2316" p="5" s="C=1U" o="cf_main" vf="">
<cf_fsm_takeoverOfPartnerDisabled_1
        partner="stor-u105"
        reason="status of backup mailbox is uncertain"/>

we can use :
%err = ("cf.fsm.takeoverofPartnerDisabled" => "reason=status of backup mailbox is uncertain");
ems_error_monitor(host=>$host,key_value_errors=>\%err);

=item ignore_error_values

(Optional): Reference to an Array with a list of  errors to be ignored. The list of additional_error_values will be removed to the list of errors already defined in the function. By default it will use the  list defined in the library without removing errors.

=item fatal_on_failure

(Optional): To stop test on seeing errors? Default is yes.

=item timeout

(Optional) : Timeout value for the say. Default : 1200: Increasing the default
value as the log file might be quite big and can take longer to dump the file 

=item logresults

(Optional) : Allows users of this routine to prevent the routine from
logging test results. If set to 'no' then an error string is returned.
Default : yes.

=back

=cut

sub ems_error_monitor {
	my (%opts) = validate(@_,{
				host => {type=>OBJECT, isa=>'Hostrec'},
				mark => {type=>SCALAR},
				monitor_time => { type=>SCALAR,default=>1,optional=>1 },
				sleep_time => {type=>SCALAR,default=>1,optional=>1},
				error_values => {type=>ARRAYREF, optional =>1 },
				additional_error_values => {type=>ARRAYREF, optional =>1 },
				ignore_error_values => {type=>ARRAYREF, optional =>1 },
				key_value_errors => {type=>HASHREF, optional =>1 },
				fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
           			timeout => { type=>SCALAR, default=>3600, optional=>1 },
           			logresults => { type=>SCALAR, default=>'yes', optional=>1 },
	}
	);
        my $errmsg = '';
	my @error_values;
	my $mark;
	my $k;
	my $v;
	my $filer = $opts{host}->{'hostname'};
	print "FILER IS: $filer \n";
	#sleep 20;
	

	if(!defined($opts{error_values})){
		my %ems_error_messages=(#"01"=>"monitor.shelf.fault",
                    "02"=>"raid.disk.missing",
                    "03"=>"raid.config.spare.disk.missing",
                    "04"=>"cf.fsm.backupMailboxError",
		    "05"=>"fci.adapter.parity",
		    "06"=>"fci.adapter.bad",
		    "07"=>"sas.adapter.bad",
		    "08"=>"fci.adapter.failed",
		    "09"=>"sas.adapter.failed",
		    "10"=>"fci.adapter.nvram.invalid",
		    "11"=>"sas.adapter.nvram.invalid",
		    "12"=>"fci.device.invalid.wwn",
		    "13"=>"sas.device.invalid.wwn",
		    "14"=>"fci.device.invalidate.soft.address",
		    "15"=>"sas.device.invalidate.soft.address",
		    "16"=>"fci.adapter.firmware.fault",
		    "17"=>"sas.adapter.firmware.fault",
		    "18"=>"fci.adapter.firmware.pause",
		    "19"=>"sas.adapter.firmware.update.failed",
		    "20"=>"fci.adapter.not.ready",
		    "21"=>"sas.adapter.not.ready",
		    "22"=>"sas.adapter.offline",
		    "23"=>"fci.device.login.failure",
		    "24"=>"fci.adapter.unexpected.status",
		    "25"=>"sas.adapter.unexpected.status",
		    "26"=>"disk.write.failure",
		    "27"=>"diskown.errorReadingOwnership",
		    "28"=>"disk.senseError",
		    "29"=>"disk.readReservationFailed",
		    "30"=>"scsi.cmd.adapterFatal",
		    "31"=>"scsi.cmd.adapterHardwareError",
		    "32"=>"scsi.cmd.adapterResources",
		    "33"=>"scsi.cmd.contingentAllegiance",
		    "33"=>"scsi.cmd.noMorePaths",
		    "34"=>"raid.disk.unload.done",
		    "35"=>"raid.rg.recons.missing",
		    "35"=>"raid.disk.maint.start",
		    

                   );
		 @error_values=values(%ems_error_messages);
	}
	else{
		 @error_values = @{$opts{error_values}};
	}

	if(defined($opts{additional_error_values})){
		 push @error_values,@{$opts{additional_error_values}};
	 }

	if(defined($opts{ignore_error_values})){
		@error_values = _get_unique_elements($opts{ignore_error_values},\@error_values);
	 }


  my $loop_count = int($opts{monitor_time} / $opts{sleep_time});

   $mark = $opts{mark};

  my $i = 1;
  my $fail = "no";
  my $hash;

	while ($i <= $loop_count) {
		my $mark1;
		if($loop_count != 1){
      		$mark1 = ems_log_get_mark(host=>$opts{host},timeout=>$opts{timeout});
		}
      		$hash = ems_log_dump(host=>$opts{host}, timestamp=>$mark,timeout=>$opts{timeout});
      		foreach my $msg (@error_values) {
          		my $event = $hash->{by_type}->{"$msg"};
          		if (defined($event)) {
		  		#if($opts{fatal_on_failure} eq "yes"){
              			#	logresult('FATAL',"Failed because of: $msg");
		  		#}
		  		#else{
              				logcomment("FAILED because of: $msg");
					$fail = "yes";
		  		#}

          		}
      		}
			if(defined($opts{key_value_errors})){
				#print "Defined \n";
      		#		my $event_by_id = ems_log_dump_by_id(host=>$opts{host}, timestamp=>$mark,timeout=>$opts{timeout});
				#print "-----------------------------------\n";
				#print Dumper $opts{key_value_errors};
				#print Dumper $event_by_id;
				#print Dumper $event_by_type;
				#print "-----------------------------------\n";
				while(($k,$v) = each %{$opts{key_value_errors}}){
					#print "$k => $v \n";
    					my $event = $hash->{by_type}->{"$k"};
					#print Dumper ($event);
					if(defined($event)) {
        					foreach my $id (@{$event}) {
							my @key_val = split(/=/,$v);
								#print "KEY is $key_val[0] VALUE TO CHECK: $key_val[1] \n";
            						if(defined($hash->{by_id}->{$id}{$key_val[0]})){
            							my $val = $hash->{by_id}->{$id}{$key_val[0]} ;
								my $value_to_check = $key_val[1];
								if($val =~ /$value_to_check/){
              									logcomment("FAILED: Saw unexpected ems event on filer $filer : $k => $v : Error: $val");
										$fail = "yes";
								}
							}
						}
					}
				}
			}
							


		if($loop_count != 1){
      			$mark = $mark1;
		}
      		sleep $opts{sleep_time};
      		++$i;
  	}

	if ($opts{logresults} eq 'yes') {
		# Didn't find any error in the given time
		if($fail eq "no"){
			logcomment("No recognizable errors found.");
		} else {
			if($opts{fatal_on_failure} eq "yes"){
				logresult("fatal","Unexpected EMS errors found. Please check the logs above for details");
			} else {
				logresult("fail","Unexpected EMS errors found. Please check the logs above for details");
			}
		}
	} else {
		# Didn't find any error in the given time
		if($fail eq "yes"){
			$errmsg = "Unexpected EMS errors found. Please check the logs above for details";
		}
		return ($errmsg);
	}
}


=head2 ems_get_value_by_key
	my @ret = ems_get_value_by_key(host=>$host, eventtype=>$eventtype, key=>$key,timestamp=>$timestamp,log=>$log,timeout=>$timeout);
	
This subrutine returns the value of a given key.

The following arguments are accepted:

=over

=item host
(Required): Hostspec to filer

=item eventtype

(Required) : Ems type. ex: "raid.rg.recons.aborted"

=item key 

(Required): The key which the requested value for. ex:dbn 

=item timestamp
(Required) : the event from this timestamp.
Timestamp semantic can be :
<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional): 

=item timeout

(Optional) : Timeout value for the say. Default : 1200: Increasing the default
value as the log file might be quite big and can take longer to dump the file 

=back

=cut

sub ems_get_value_by_key {

    my (%opts) = validate(@_, { host => { type=>OBJECT, isa=>'Hostrec' },
                     eventtype => { type=>SCALAR },
                     key => { type=>SCALAR },
                     timestamp => { type=>SCALAR },
                     log => { type=>SCALAR,default=>"", optional=>1 },
                     timeout => { type=>SCALAR, default=>1200, optional=>1 },
                 });

    my $event_by_type;
    my $event_by_id;

    if (defined($opts{log})) {
        $event_by_type = ems_log_dump_by_type(host=>$opts{host}, timestamp=>$opts{timestamp}, timeout=>$opts{timeout}, log=>$opts{log});
    } else {
        $event_by_type = ems_log_dump_by_type(host=>$opts{host}, timestamp=>$opts{timestamp}, timeout=>$opts{timeout});
    }

    if (defined($opts{log})) {
        $event_by_id = ems_log_dump_by_id(host=>$opts{host}, timestamp=>$opts{timestamp}, timeout=>$opts{timeout}, log=>$opts{log});
    } else {
        $event_by_id = ems_log_dump_by_id(host=>$opts{host}, timestamp=>$opts{timestamp}, timeout=>$opts{timeout});
    }

    my $event = $event_by_type->{"$opts{eventtype}"};
    if(!defined($event)) {
        return (0);
    } else {
        foreach my $id (@{$event}) {
            return $event_by_id->{$id}{$opts{key}} ;
        }
        return(undef);
    }
}

=head2 ems_log_dump

This routine returns pointer to hash table having  both event name and event type as key. 

my $host = host($FILER);
my $mark = ems_log_get_mark(host=>$host);
my $hash = ems_log_dump(host=>$host, timestamp=>$mark);

The output looks like:

$VAR1 = {
          'by_id' => {
                       '1262865202/227' => {
                                             'debug_string' => 'PHY UP [2] RATE 0x2 PS 0x1, NPIP 3 :: DEVTYPE 0x20, TGT 0x2, INIT 0x2, ADDR 50050cc10210127f',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/227',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_intrd_1'
                                                         }
                                           },
                       '1262865202/231' => {
                                             'debug_string' => 'Asyncd device scan done -- SATA 0/0, SATA reserved 0/0.',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/231',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_asyncd_1'
                                                         }
                                           },
                       '1262865202/228' => {
                                             'debug_string' => 'PHY UP [3] RATE 0x2 PS 0x1, NPIP 4 :: DEVTYPE 0x20, TGT 0x2, INIT 0x2, ADDR 50050cc10210127f',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/228',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_intrd_1'
                                                         }
                                           },
                       '1262865202/226' => {
                                             'debug_string' => 'PHY UP [1] RATE 0x2 PS 0x1, NPIP 2 :: DEVTYPE 0x20, TGT 0x2, INIT 0x2, ADDR 50050cc10210127f',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/226',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_intrd_1'
                                                         }
                                           },
                       '1262865202/229' => {
                                             'debug_string' => 'Port 0: disabled 0, up 4, down 0: old state 3 --&gt; new state 3',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/229',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_asyncd_1'
                                                         }
                                           },
                       '1262865202/224' => {
                                             'debug_string' => 'PHY UP [0] RATE 0x2 PS 0x1, NPIP 1 :: DEVTYPE 0x20, TGT 0x2, INIT 0x2, ADDR 50050cc10210127f',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/224',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_intrd_1'
                                                         }
                                           },
                       '1262865202/223' => {
                                             'event_type' => 'sas.adapter.reset',
                                             'attr' => 'adapterName',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:41',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/223',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889461',
                                                           'o' => 'pmcsas_asyncd_1'
                                                         }
                                           },
                       '1262865202/225' => {
                                             'debug_string' => 'PORT UP -- 2a',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/225',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_intrd_1'
                                                         }
                                           },
                       '1262865202/230' => {
                                             'debug_string' => 'Port 1: disabled 4, up 0, down 0: old state 1 --&gt; new state 1',
                                             'event_type' => 'sas.adapter.debug',
                                             'attr' => 'adapterName,debug_string',
                                             'adapterName' => '2a',
                                             'header' => {
                                                           'vf' => '',
                                                           'n' => 'svt-u23',
                                                           'd' => '07Jan2010 18:37:42',
                                                           's' => 'C=1U',
                                                           'p' => '6',
                                                           'id' => '1262865202/230',
                                                           'pn' => 'svt-u24',
                                                           't' => '1262889462',
                                                           'o' => 'pmcsas_asyncd_1'
                                                         }
                                           }
                     },
          'by_type' => {
                         'sas.adapter.debug' => [
                                                  '1262865202/224',
                                                  '1262865202/225',
                                                  '1262865202/226',
                                                  '1262865202/227',
                                                  '1262865202/228',
                                                  '1262865202/229',
                                                  '1262865202/230',
                                                  '1262865202/231'
                                                ],
                         'sas.adapter.reset' => [
                                                  '1262865202/223'
                                                ]
                       }
        };



The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item timestamp

(Optional) : Get the event from timestamp Default : ""(means only header)

Timestamp semantic can be : 

<n>s    Last n seconds
<n>m    Last n minutes
<n>h    Last n hours
<n>d    Last n days
<n>t    Since timestamp <n>

=item log

(Optional) : The logfile to get events from. Default : "" (means current)

=item timeout

(Optional) : Timeout value for the say. Default : 1200

=back

=cut




sub ems_log_dump {
    my (%opts) = validate(@_, { host => { type=>OBJECT, isa=>'Hostrec',default=>"",optional=>1 },
            timestamp => { type=>SCALAR,default=>"", optional=>1 },
            log => { type=>SCALAR,default=>"", optional=>1 },
            verbose => { type=>SCALAR,default=>"yes", optional=>1 },
            timeout => { type=>SCALAR, default=>1200, optional=>1 },
            ems_file => {type=>SCALAR,default=>"none",optional=>1},
            });

    my $emslog_entry;
    my @emslog_events;
    my @emslog_event_lines;
    my %emslog_event_by_id;
    my %emslog_event_by_type;
    my %emslog_nr;
    my $ems_hash;
    my @lines;
    my $verbose = " ";
    if($opts{verbose} eq "no"){
        $verbose = "-noverbose";
    }
    if ($opts{ems_file} eq "none"){
#my $conn = make_conn($opts{host});
        my $cmd = "ems log dump $opts{log} $opts{timestamp}";
        logcomment("Issuing $cmd  $verbose");
        my $output = filer_cmd(host=>$opts{host},timeout=>$opts{timeout},cmd=>"$cmd");
#my (undef, undef, $output) = say($conn, "$cmd","$verbose", "-timeout", $opts{timeout});
        @lines = split(/\r*\n/, $output);
    }
    else{
        my @ems_files = split(/\s+|,/,$opts{ems_file});
        my $found = 0;
        foreach my $ems_file (@ems_files){
            open(FILE,"< $ems_file") or die "Could not open file $ems_file for reading: $!\n";
            while(<FILE>){
                my $line = $_;
                $line =~ s/\r*\n//g;
                if($opts{timestamp} =~ /\d+t/){
                    my $mark = $opts{timestamp};
                    $mark =~ s/t$//g;
                    if(($line =~ /t=\"$mark\"/) && ($found == 0)){
                        print "Mark $mark found \n";
                        $found = 1;
                    }
                }
                else{
                    $found = 1;
                }
                if($found){
                    push @lines,$line;
                }
            }
            close(FILE);
        }
    }

    foreach my $line (@lines) {
        if (!emslog_parse($line, \%emslog_nr, \$emslog_entry)) {
            next;
        }
        emslog_process_line($emslog_entry, \@emslog_events, 
                \@emslog_event_lines, 
                \%emslog_event_by_id, 
                \%emslog_event_by_type);
        $emslog_entry = "";
    }
    $ems_hash = {
        'by_type' => \%emslog_event_by_type,
        'by_id' => \%emslog_event_by_id,
    };


    return($ems_hash);
}

=head2 ems_log_search

This routine returns 0 if a given event on given objectname is not found from a specified timestamp otherwise returns <n> for the n number of instances found in the log.

my $host = host($FILER);
my $mark = ems_log_get_mark(host=>$host);
my $hash = ems_log_dump(host=>$host,timestamp=>$mark);

my $ret = ems_log_search(
    eventtype=>"fci.adapter.offline", 
    key=>"adapterName" 
    value=>"6a", 
    hash=>$hash
);

if ($ret == 0) {
    logcomment("The specified event is not found in the ems log");
} else {
    logcomment("Found the specified ems event $ret times in the ems log");
}

The following named arguments are accepted,

=over

=item hash

(Required) : Hash reference returned by ems_log_dump command

=item eventtype

(Optional) : The type of events to look into, like "fci_adapter_offline"...

=item key

(Optional) : What is the key (the objectname for a given event type) to look into the event 
message. For example, in the following ems message key is "adapterName":

<LR d="07Dec2005 18:45:31" n="raid-u23" t="1133981131" id="1133838353/404" p="6" s="OK" o="ispfc_init_2" vf="">
<fci_adapter_offline_1
        adapterName="6a"/>
</LR>

=item value

(Optional) : What is the value (the objectname for a given event type) to look into for the 
given event type and key. To check for the above ems message the value would be "6a".


=item severity

(Optional) : Severity level of the messages to check. We can provide a "," or " " seperated list of severities.  Currently ontap supports level 0 to 7.

=item err_hash

(Optional) : Hash of errors to look for. Hash format is returned by ems_build_err_hash() function.

=item ignore_events

(Optional) : Reference of array of events to be ignored.

=item print_table

(Optional) : Print table of matching events found. Default 1.


=back

=cut




sub ems_log_search{
    my (%opts) = validate(@_,{
            hash => {type=>HASHREF},
            severity => {type=>SCALAR,default=>undef,optional=>1},
            eventtype => {type=>SCALAR,default=>undef,optional=>1},
            key => {type=>SCALAR,default=>undef,optional=>1},
            value => {type=>SCALAR,default=>undef,optional=>1},
            patternmatch => {type=>SCALAR,default=>0,optional=>1},
            verbose => {type=>SCALAR,default=>0,optional=>1},
            err_hash => {type=>HASHREF,default=>undef,optional=>1},
            ignore_events => {type=>ARRAYREF,default=>undef,optional=>1},
            print_table => {type=>HASHREF,default=>1,optional=>1},
            });
    my $hash = $opts{hash};
    my $seen = 0;
    my @sev;
    my $struct;

    if(defined($opts{severity})){
        @sev = split(/\s+|,/,$opts{severity});
    }
    else{
        @sev = (0 .. 7);
    }

    my %severity = (
                    '0' => "EMERGENCY",
                    '1' => "ALERT",
                    '2' => "CRITICAL",
                    '3' => "ERROR",
                    '4' => "WARNING",
                    '5' => "NOTICE",
                    '6' => "INFO",
                    '7' => "DEBUG",
    );
    my @ids = keys %{$hash->{'by_id'}};
    if(!defined($opts{err_hash})){
        foreach my $id (@ids){
            foreach my $sev (@sev){
                if($hash->{'by_id'}->{$id}->{header}->{p} eq $sev){
                    my $event = $hash->{'by_id'}->{$id}->{event_type};
                    if(defined($opts{ignore_events})){
                        my @ignored_events = @{$opts{ignore_events}};
                        if(grep /^$event$/,@ignored_events){
                            print ("Ignoring event $event \n") if($opts{verbose});
                            goto LOG_SEARCH_LABEL1;
                        }
                    }

                    if(defined($opts{eventtype})){
                        if($event =~ /^$opts{eventtype}/){
                            if(defined($opts{key}) && defined($opts{value})){
                                if($opts{patternmatch}){
                                    if($hash->{'by_id'}->{$id}->{$opts{key}} =~ /$opts{value}/){
                                        my $val  = $hash->{'by_id'}->{$id}->{$opts{key}};
                                        $struct->{$event}->{Sev} = $sev; 
                                        $struct->{$event}->{Count} = scalar(@{$hash->{by_type}->{$event}});
                                        print("Found matching EVENT:  $event  ID:  $id  KEY: $opts{key} VALUE: \"$val\"  SEVERITY: $sev \n") if($opts{verbose});
                                        $seen++;
                                    }
                                }
                                else{
                                    if($hash->{'by_id'}->{$id}->{$opts{key}} eq  $opts{value}){
                                        my $val  = $hash->{'by_id'}->{$id}->{$opts{key}};
                                        $struct->{$event}->{Sev} = $sev; 
                                        $struct->{$event}->{Count} = scalar(@{$hash->{by_type}->{$event}});
                                        print("Found matching EVENT:  $event  ID:  $id  KEY: $opts{key} VALUE: \"$val\"  SEVERITY: $sev \n") if($opts{verbose});
                                        $seen++;
                                    }
                                }
                            }
                            else{
                                $struct->{$event}->{Sev} = $sev; 
                                $struct->{$event}->{Count} = scalar(@{$hash->{by_type}->{$event}});
                                print("Found matching event $event id $id with severity $sev \n") if($opts{verbose});
                                $seen++;
                            }

                        }
                    }
                    else{
                        $struct->{$event}->{Sev} = $sev; 
                        $struct->{$event}->{Count} = scalar(@{$hash->{by_type}->{$event}});
                        print("Found matching event $event id $id with severity $sev \n") if($opts{verbose});
                        $seen++;
                    }
LOG_SEARCH_LABEL1:;
                }

            }
        }
    }
    else{
        my $err_hash = $opts{err_hash};
        my @err_ids = keys %{$err_hash};
        foreach my $id (@ids){
            my $event = $hash->{'by_id'}->{$id}->{event_type};
            my $sev = $hash->{'by_id'}->{$id}->{header}->{p};
            foreach my $sev1 (@sev){
                if($hash->{'by_id'}->{$id}->{header}->{p} eq $sev1){
                    foreach my $err_id (@err_ids){
                        my $err_type = $err_hash->{$err_id}->{EventType};
                        if($event =~ /^$err_type/){
                            if(defined($opts{ignore_events})){
                                my @ignored_events = @{$opts{ignore_events}};
                                if(grep /^$event$/,@ignored_events){
                                    print ("Ignoring event $event \n") if($opts{verbose});
                                    goto LOG_SEARCH_LABEL2;
                                }
                            }
                            if(defined($err_hash->{$err_id}->{Key}) && defined($err_hash->{$err_id}->{Value})){
                                my $err_key = $err_hash->{$err_id}->{Key};
                                my $err_value = $err_hash->{$err_id}->{Value};
                                if($opts{patternmatch}){
                                    if(defined($hash->{'by_id'}->{$id}->{$err_key})){
                                        if($hash->{'by_id'}->{$id}->{$err_key} =~ /$err_value/){
                                            my $val  = $hash->{'by_id'}->{$id}->{$opts{key}};
                                            $struct->{$event}->{Sev} = $sev; 
                                            $struct->{$event}->{Count} = scalar(@{$hash->{by_type}->{$event}});
                                            print("Found matching EVENT:  $event  ID:  $id  KEY: $err_key VALUE: \"$err_value\"\n") if($opts{verbose});
                                            $seen++;
                                        }
                                    }
                                }
                                else{
                                    if(defined($hash->{'by_id'}->{$id}->{$err_key})){
                                        if($hash->{'by_id'}->{$id}->{$err_key} eq "$err_value"){
                                            my $val  = $hash->{'by_id'}->{$id}->{$opts{key}};
                                            $struct->{$event}->{Sev} = $sev; 
                                            $struct->{$event}->{Count} = scalar(@{$hash->{by_type}->{$event}});
                                            print("Found matching EVENT:  $event  ID:  $id  KEY: $err_key VALUE: \"$err_value\" \n ") if($opts{verbose});
                                            $seen++;
                                        }
                                    }
                                }
                            }
                            else{
                                $struct->{$event}->{Sev} = $sev; 
                                $struct->{$event}->{Count} = scalar(@{$hash->{by_type}->{$event}});
                                print("Found matching event $event id $id \n") if($opts{verbose});
                                $seen++;
                            }

                        }
LOG_SEARCH_LABEL2:;
                    }

                }
            }
        }
    }

    if($opts{print_table}){
        my @types = keys %{$hash->{'by_type'}};
        foreach my $sev (@sev){
            my $in = 1;
            foreach my $type (@types){
                if(defined($struct->{$type}->{Sev})){
                    if($struct->{$type}->{Sev} eq "$sev"){
                        if($in){
                            print "-----------------------------------------------------------\n";
                            printf '%-50s %-8s',$severity{$sev},"Count" ;
                            print "\n";
#print $severity{$sev},  " events \t\t\t\t Count \n";
                            print "-----------------------------------------------------------\n";
                        }
                        printf '%-50s %-8s',$type, $struct->{$type}->{Count};
                        print "\n";
                        $in =0;
                    }
                }
            }
        }
        print "\n";
    }

    return $seen;
}


=head2 ems_build_error_hash

This routine parses ems_error_file and returns a hash with format:

$VAR1 = {
          '6' => {
                   'Value' => 'efg',
                   'EventType' => 'sas.dd.ee',
                   'Key' => 'abc'
                 },
          '4' => {
                   'Value' => '',
                   'EventType' => 'sas.adapter.debug',
                   'Key' => 'debug'
                 },
          '1' => {
                   'EventType' => 'shm'
                 },
          '3' => {
                   'Value' => 'SMP error',
                   'EventType' => 'sas.adapter.debug',
                   'Key' => 'debug_string'
                 },
          '0' => {
                   'EventType' => 'config'
                 },
          '2' => {
                   'EventType' => 'disk.'
                 },
          '5' => {
                   'EventType' => 'raid.rg.readerr.bad.file.block'
                 }
        };


The hash returned is used by ems_log_search() function to look for errors defined in the error file.

=over

=item err_file

(Required) : Error file to parse

=back

=cut


sub ems_build_error_hash{
    my (%opts) = validate(@_,{
            err_file => {type=>SCALAR},
            });


    my $struct;
    my $i = 0;
    my @errors;

    open(FILE,"< $opts{err_file}") or die "Could not open file $opts{err_file} for reading: $!\n";
    while(<FILE>){
        if(($_ !~ /^#/) && ($_ =~ /\w+/)){
            push @errors,$_;
        }
    }
    foreach my $er (@errors){
        $er =~ s/\s*\r*\n$//g;
        if($er =~ /(^\w+\S+)\s*/){
#print "Eventtype is $1 \n";
            $struct->{$i}->{EventType} = $1;
        }
        if($er =~ /\s+(\S+=\".*\"$)/){
        #if($er =~ /\s+(\S+=\".*\")/){
            print "ERRRRRRRR: $er \n";
            my $key = $1;
        my $value = $1;
        $key =~ s/=.*//g;
        print "Key is $key \n";
        $value =~ s/.*=//g;
        $value =~ s/\"//g;
        print "Value is $value \n";
        $struct->{$i}->{Key} = $key;
        $struct->{$i}->{Value} = $value;

    }
    $i++;
    }
    close(FILE);

    return $struct;
}






  
#----------------------------------------------------------------------
# The local routines are coded in the following section
#----------------------------------------------------------------------

sub emslog_parse {
    my($line, $emslog_nr, $emslog_entry) = @_;

    if ($line =~ /^<NR/) {
        %{$emslog_nr} = emslog_parseNR($line);
        return 0;
    }

    if ($line =~ /^<LR/) {
        ${$emslog_entry} = $line;
        return 0;
    }
	
    if ($line =~ /^<\/LR/) {
        ${$emslog_entry} = ${$emslog_entry} . $line;
        return 1;
    }

    if (!defined(${$emslog_entry})) {
        ${$emslog_entry} = " $line";
    } else {
        ${$emslog_entry} = ${$emslog_entry} . " $line";
    }
    return 0;
}

sub emslog_process_line {
    my($line, $emslog_events, $emslog_event_lines, $emslog_event_by_id, 
       $emslog_event_by_type) = @_;

    my($header, $event, $id, $i);
    my(@headerlist, @eventlist);
    my($type, $version);
	
    if ($line =~ /^\s*<LR (.*)>\s*<(.*)\/>\s*<\/LR>\s*$/) {
        #print "MATCH: $line \n";
        $header = "$1";
        $event = $2;
    } else {
        logcomment("emslog_process_line: mismatch\n $line");
        return;
    }
	
    #
    # Get the id.
    #
    if ($header =~ /\s*id=\"([^\"]+)\"/) {
        $id = $1;
    }
	
    push @{$emslog_events}, $id;
    push @{$emslog_event_lines}, $line;
	

    #
    # Parse the header (LR) section of the event record
    #	
    while ($header ne "") {
        if ($header =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/) {
            $emslog_event_by_id->{$id}{header}{$1} = $2;
            $header = $3;
        } else {
            $header = "";
        }
    }
	
    if ($event =~ /^\s*(\S+)\s*(.*)/) {
        $type = $1;
        $event = $2;
    }
	
    if ($type =~ /^(\S+)_([0-9]+)$/) {
        $type = $1;
        $type =~ s/_/\./g;
    }
	
    emslog_add_by_type($type, $id, $emslog_event_by_type);
    $emslog_event_by_id->{$id}{event_type} = $type;

    #
    # Parse the per-event specific part of the event record
    #
    while ($event ne "") {
        if ($event =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/) {
            $emslog_event_by_id->{$id}{$1} = $2;
            if (!exists($emslog_event_by_id->{$id}{attr})) {
                $emslog_event_by_id->{$id}{attr} = $1;
            } else {
                $emslog_event_by_id->{$id}{attr} .= ",$1";
            }
            $event = $3;
        } else {
            $event = "";
        }
    }
}

sub emslog_parseNR {
    my($line) = @_;
    my %emslog_nr = ();
    my($header);
    if ($line =~ /^\s*<NR (.*)\/>\s*$/) {
        $header = "$1";
    } else {
        logcomment("parseNR mismatch: '$line'\n");
        croak $line;
    }
	
    #
    # Parse the header (NR) section of the event record
    #	
    while ($header ne "") {
        if ($header =~ /^\s*([^=]+)=\"([^\"]*)\"\s*(.*)$/) {
            $emslog_nr{$1} = $2;
            $header = $3;
        } else {
            $header = "";
        }
    }
    return (%emslog_nr);
}

sub emslog_add_by_type {
    my($type, $id, $emslog_event_by_type) = @_;
	
    #if ($emslog_event_by_type->{$type} eq "") {
    #    $emslog_event_by_type->{$type} = $id;
    #} else {
    #    $emslog_event_by_type->{$type} .= ",$id";
    #}
    push(@{$emslog_event_by_type->{$type}}, $id);
}

#----------------------------------------------------------------------
# The testme routine to test the exported functions of this library
#
# This routine can be invoked in the following manner
#
# ntest -hw FILER=raid-u24 PKGNAME=Storage::Ems util/test_pkg (test all functions)
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_log_get_mark PKGNAME=Storage::Ems util/test_pkg
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_log_dump_by_type PKGNAME=Storage::Ems util/test_pkg
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_log_dump_by_type TIMESTAMP=1133355221t PKGNAME=Storage::Ems util/test_pkg
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_log_dump_by_id TIMESTAMP=2s PKGNAME=Storage::Ems util/test_pkg
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_log_dump_by_id PKGNAME=Storage::Ems util/test_pkg
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_log_dump_by_id TIMESTAMP=1133355221t PKGNAME=Storage::Ems util/test_pkg
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_event_status TIMESTAMP=1133355221t PKGNAME=Storage::Ems util/test_pkg
#
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=ems_log_check TIMESTAMP=30s PKGNAME=Storage::Ems util/test_pkg
#
#----------------------------------------------------------------------

sub testme {

    package main;
    our ($FILER);
    our ($FUNCTION_NAME);
    our ($TIMESTAMP);
    package Storage::Ems;
    param('FILER',       '-mesg',    "Filer to test");
    param('FUNCTION_NAME',       '-default',    "all");
    param('TIMESTAMP',       '-default',    "1s");

    logcomment("Testing on FILER $FILER");
    logcomment("Testing $FUNCTION_NAME");
    logcomment("Testing $TIMESTAMP");

    my $host = host($FILER);
    my $ret;

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "ems_log_get_mark") {
        eval {
            $ret = ems_log_get_mark(host=>$host);
        };

        if ($@) {
            logresult('fail', "function ems_log_get_mark $@");
        } else { 
            logcomment("The marker is: $ret\n");
        }

        logresult('pass', "Got expected result from function ems_log_get_mark");
    }

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "ems_log_dump_by_type") {
        eval {
            $ret = ems_log_dump_by_type(host=>$host, timestamp=>$TIMESTAMP);
        };

        if ($@) {
            logresult('fail', "function ems_log_dump_by_type $@");
        } else { 
            print Dumper($ret);
        }

        logresult('pass', "Got expected result from function ems_log_dump_by_type");
    }

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "ems_log_dump_by_id") {
        eval {
            $ret = ems_log_dump_by_id(host=>$host, timestamp=>$TIMESTAMP);
        };

        if ($@) {
            logresult('fail', " function ems_log_dump_by_id $@");
        } else { 
            print Dumper($ret);
        }

        logresult('pass', "Got expected result from function ems_log_dump_by_id");
    }

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "ems_event_status") {
        eval {
            $ret = ems_event_status(host=>$host);
        };

        if ($@) {
            logresult('fail', " function ems_event_status $@");
        } else { 
            print Dumper($ret);
        }

        logresult('pass', "Got expected result from function ems_event_status");
    }

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "ems_log_check") {
        eval {
            $ret = ems_log_check(host=>$host, eventtype=>"fci.adapter.offline", key=>"adapterName", value=>"6a", timestamp=>$TIMESTAMP);
        };

        if ($ret != 0) {
            logresult('fail', " function ems_log_check: $ret");
        } else { 
            logresult('pass', "Got expected result from function ems_log_check");
        }
    }
}

#-------------------------------------------------
# Local subroutine to get unique elements which are in array2 but not in array1
#-------------------------------------------------
sub _get_unique_elements {
	my ($array1,$array2) = @_;
	my @disks = ();
	my $item;
	my %seen = ();
	foreach $item (@$array1) {
		$seen{$item} = 1;
	}
	foreach $item (@$array2) {
		unless ($seen{$item}){
			push (@disks, $item) ;
		}
	}
	return @disks;
}

=head1 SEE ALSO

    Where additional information can be found.

=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Testme

=head1 ATTRIBUTES

    ONTAP_COMMANDS = ems

    ONTAP_ZAPI_APIS = none 

=head1 AUTHOR

    Sajan Agrawal (sajan@netapp.com), dl-storage-automation

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;

