# _*_ emacs _*_
# $Id: //depot/prod/test/Rsierranevada/lib/Storage/Environ.pm#1 $

## @summary Environ - the library provides routines to give 
## the information of environment 
## @author yingz@netapp.com 
## @change faslur@netapp.com changed parsing line with space' ' to '\s+' at line 639 and 1063
## @change anosh@netapp.com changed say to filer_cmd
=head1 NAME

Environ - Environ  Commands Routines

=head1 SYNOPSIS

use Storage::Environ qw(environ_chassis, environ_status, environ_shelf); 

$s = environ_chassis(%input);
    
=head1 DESCRIPTION

The library supports routines to give the information of the library.  

The defined routines should work on both filers and "simulators".
  
Here is the list of provided routines are listed 
in alphabetical order. Please refer to each routine
description for details information.  

=head1 FUNCTION PROTOTYPES

$s = environ_chassis(%input)
 
=cut 

=head1 ROUTINES
 
=cut

package Storage::Environ;

use strict;
use warnings;
BEGIN {
    use base qw(Exporter);
    our @EXPORT_OK = qw(
            environ_chassis
            environ_status
            environ_shelf
            ); 
    Params::Validate::validation_options(
            on_fail => sub {my @msgs = @_; chomp(@msgs);
            Carp::confess(Tharn::script_result(@msgs))}
            );
}

use Carp qw(confess croak);
use Params::Validate qw(validate validate_pos OBJECT SCALAR ARRAYREF);
use Utils qw(make_conn make_conn_and_complain); 
use Tharn; 
use Data::Dumper;
use RaidInfo;
use Storage::Misc qw(filer_cmd);

use vars qw(@ISA); 


our $MAINT_PROMPT = "\\*\>";

# ======================= DEFINE ROUTINES ===========================

=head1 environ_chassis(%input);

The routine collect the chassis information by using environ
command

=item Return

Return the hash of the chassis information

=item host
(Required) : Host spec to the FILER

=item conntype
(Optional) : Connection type to the Filer. Default is rsh.

=cut

=back

=item Example: 
    
    $ret = environ_chassis(%input);

=item SEE ALSO
   None

=back

=cut

sub environ_chassis(%) {
    my(%input) = validate ( @_, {
	      host => {type=>OBJECT, isa=>'Hostrec'},
	      conntype => {type=>SCALAR, default=>"#rsh", optional=>1},
	      filermode => {type=>SCALAR,default=>"normal",optional=>1},
	      timeout => {type=>SCALAR, default=>300, optional=>1},
	  }
			   );
    if ($input{filermode} ne "normal") { 
        $input{conntype} = "#console";
    }
    my(%chassis_status, @info, @status, $line, $status_key, $outputinfo, $key,$value);
    my $conn = make_conn($input{host},$input{conntype});
    my $count = 0;
    if ($input{filermode} ne "normal") {
        my $cmd = "environment chassis"; 
#		(undef, undef, $outputinfo) = say($conn, "$cmd", -prompt, "$MAINT_PROMPT", "-timeout", $input{timeout});
		$outputinfo = filer_cmd(host=>$input{host},filermode=>$input{filermode},conntype=>$input{conntype},timeout=>$input{timeout},cmd=>$cmd);
    } else {
        my $cmd = "priv set -q diag; environment chassis";
#        (undef, undef, $outputinfo) = say($conn, "$cmd", "-timeout", $input{timeout});
	$outputinfo = filer_cmd(host=>$input{host},filermode=>$input{filermode},conntype=>$input{conntype},timeout=>$input{timeout},cmd=>$cmd);
    }
    if(($outputinfo !~ /Fan ok/) && ($outputinfo !~ /Fans ok/ && ($outputinfo !~ /Voltage ok/) ) && ($outputinfo !~ /Power ok/)) {
	croak $outputinfo;
    } else {
	@info = split(/\r?\n/, $outputinfo);
	foreach $line (@info) {
	    if ($line =~ /Power supply/) {
		@status = split(/\s+/, $line);
		$status_key = "Powersupply$status[2]";
		$chassis_status{$status_key} = $line;
	    } elsif ($line=~ /RTC/) {
		@status = split(/\s+/, $line);
		$status_key = ($status[0].$status[1]);
		$chassis_status{$status_key} = $status[2];
	    } elsif ($line =~ /Power Supply:/){ 
		@status = split(/\s+/, $line);
		++$count;
		$status_key = "Powersupply$count";
		$chassis_status{$status_key} = $line;
	    } elsif ($line=~ /Power Supply/) {
		@status = split(/\s+/, $line);
		$status_key = "powersupply";
		$chassis_status{$status_key} = $status[2];
	    }  elsif ($line=~ /Temp:/) {
		@status = split (/:/, $line);
		$chassis_status{$status[0]} = $status[1];
            } elsif ($line=~ /NVRAM Battery:/) {
                @status = split (/:/, $line);
                $chassis_status{$status[0]} = $status[1];
	    } else {
		@status = split(/\s+/, $line);
		$chassis_status{$status[0]} = $status[1];
	    }
	}
    }
    return (\%chassis_status);
}
 

=head2 environ_status(%input);

The routine collect the environment information of system by using environ
command

The output looks like:
$VAR1 = {
          8 => {
                 '0' => {
                          'shelfmapping' => [
                                              6,
                                              5,
                                              4,
                                              3,
                                              2,
                                              1,
                                              '0'
                                            ],
                          'Appversion' => '3.1',
                          'Controlattempted' => '6;',
                          'shelfbays' => '	  6, 5, 4, 3, 2, 1, 0
',
                          'backplaneType' => '	 Backplane Type : Single Fibre Channel Backplane 
',
                          'backplanebyte' => '	 backplane byte=0xe	 cabinet id=0x0
',
                          'SESPATH' => '8.4',
                          'coolingelements' => '	Cooling Element installed element list: 1, 2, 3; with error: none
',
                          'Statusattempted' => '279;',
                          'sesproductrevision' => '0x02050301',
                          'sesvendor' => 'EuroLogc',
                          'Status' => 'normal',
                          'sesproductidentification' => 'XL500',
                          'vendorserial' => '02411355',
                          'kenerlversion' => '2.5',
                          'tempthreshlow1' => '	      Low critical:  0C (32 F); low warning 10 C (50 F)
',
                          'Controlfailure' => '0',
                          'Statusfailure' => '0',
                          'seslogical' => '0x3003040000000000',
                          'backplaneFunction' => '	 Backplane Function : Storage System 
',
                          'MODULE' => 'VEM;',
                          'ESreporting' => '	ES Electronics reporting element: 1
',
                          'sesconfigid' => 4,
                          'shelftemp1' => '	  [1] 29 C (84 F)  Normal temperature range
',
                          'shelftemp2' => '	  [1] High critical: 55 C (131 F); high warning 50 C (122 F)
',
                          'MONITORING' => 'active',
                          'ESinstalled' => '	ES Electronics installed element list: 1; with error: none
'
                        },
                 'Shelvesfail' => 'no',
                 1 => {
                        'shelfmapping' => [
                                            14,
                                            13,
                                            12,
                                            11,
                                            10,
                                            9,
                                            8
                                          ],
                        'Appversion' => '3.1',
                        'Controlattempted' => '6;',
                        'shelfbays' => '	  6, 5, 4, 3, 2, 1, 0
',
                        'backplaneType' => '	 Backplane Type : Single Fibre Channel Backplane 
',
                        'backplanebyte' => '	 backplane byte=0xe	 cabinet id=0x1
',
                        'SESPATH' => '8.12',
                        'coolingelements' => '	Cooling Element installed element list: 1, 2, 3; with error: none
',
                        'Statusattempted' => '279;',
                        'sesproductrevision' => '0x02050301',
                        'sesvendor' => 'EuroLogc',
                        'Status' => 'normal',
                        'sesproductidentification' => 'XL500',
                        'vendorserial' => '02394069',
                        'kenerlversion' => '2.5',
                        'tempthreshlow1' => '	      Low critical:  0C (32 F); low warning 10 C (50 F)
',
                        'Controlfailure' => '0',
                        'Statusfailure' => '0',
                        'seslogical' => '0x300b0c0000010000',
                        'backplaneFunction' => '	 Backplane Function : Storage System 
',
                        'MODULE' => 'VEM;',
                        'ESreporting' => '	ES Electronics reporting element: 1
',
                        'sesconfigid' => 12,
                        'shelftemp1' => '	  [1] 30 C (86 F)  Normal temperature range
',
                        'shelftemp2' => '	  [1] High critical: 55 C (131 F); high warning 50 C (122 F)
',
                        'MONITORING' => 'active',
                        'ESinstalled' => '	ES Electronics installed element list: 1; with error: none
'
                      },
                 3 => {
                        'shelfmapping' => [
                                            61,
                                            60,
                                            59,
                                            58,
                                            57,
                                            56,
                                            55,
                                            54,
                                            53,
                                            52,
                                            51,
                                            50,
                                            49,
                                            48
                                          ],
                        'Controlattempted' => '4;',
                        'shelfbays' => '	  13, 12, 11, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0
',
                        'SESPATH' => '8.49',
                        'coolingelements' => '	Cooling Element installed element list: 1, 2; with error: none
',
                        'Statusattempted' => '279;',
                        'sesproductrevision' => 1111,
                        'sesvendor' => 'XYRATEX',
                        'Status' => 'normal',
                        'sesproductidentification' => 'DiskShelf14',
                        'vendorserial' => 'OPS315880000048',
                        'tempthreshlow1' => '	      Low critical:  0C (32 F); low warning 10 C (50 F)
',
                        'tempthreshlow2' => '	      Low critical:  0C (32 F); low warning 10 C (50 F)
',
                        'tempthreshlow3' => '	      Low critical:  0C (32 F); low warning 10 C (50 F)
',
                        'Statusfailure' => '0',
                        'Controlfailure' => '0',
                        'seslogical' => '0x50050cc002000036',
                        'MODULE' => 'LRC;',
                        'ESreporting' => '	ES Electronics reporting element: 1
',
                        'sesconfigid' => 49,
                        'shelftemp1' => '	  [1] 29 C (84 F) (ambient)  Normal temperature range
',
                        'shelftemp2' => '	  [2] 33 C (91 F)  Normal temperature range
',
                        'shelftemp3' => '	  [3] 32 C (89 F)  Normal temperature range
',
                        'shelftemp4' => '	  [1] High critical: 50 C (122 F); high warning 40 C (104 F)
',
                        'shelftemp5' => '	  [2] High critical: 63 C (145 F); high warning 53 C (127 F)
',
                        'MONITORING' => 'active',
                        'shelftemp6' => '	  [3] High critical: 63 C (145 F); high warning 53 C (127 F)
',
                        'ESinstalled' => '	ES Electronics installed element list: 1, 2; with error: none
'
                      },
                 'Shelfmode' => 'yes',
                 'Shelfnumber' => 3
               },

=item host
(Required) : Host spec to the FILER

=item conntype
(Optional) : Connection type to the Filer. Default is rsh.

=item filermode
(Optional): the state of the filer. Default is normal.

=item Return

Return the hash of the environment information

=cut

=back

=item Example: 
    
    $ret = environ_status(%input);

=item SEE ALSO
   None

=back

=cut
sub environ_status {
    my(%input) = validate ( @_, {
	      host => {type=>OBJECT, isa=>'Hostrec'},
	      conntype => {type=>SCALAR, default=>"#rsh", optional=>1},
	      filermode => {type=>SCALAR,default=>"normal",optional=>1},
	      timeout => {type=>SCALAR, default=>300, optional=>1},
	  }
			   );
    if ($input{filermode} ne "normal") {
	$input{conntype} = "#console";
    }
    my($outputinfo, @info, %environinfo, $cmd, @status, $channelname, $shelfname, $shelf1,$specialchannel, $specialshelf, $diskid, $toggle);
    my %HOH;
    my $line = 0;
    my $filer = $input{host} ->hostname(); 
    my $conn = make_conn($input{host},$input{conntype});
    my $powersupp = 0;
    my $ses_flag = 0;
    my $vendor_flag = 0;
    my $shelfbay_flag = 0;
    my $power_flag = 0;
    my $temp_flag = 0;
    my $thresh_flag = 0;
    my $es_flag = 0;
    my $temp_count = 0;
    my $thresh_high = 0;
    my $thresh_low = 0;
    my $esserialcount = 0;
    my $shelf_flag = 0;
    my $powercount = 0;
    my @shelf_fields;
    my $channel_flag = 0;
    my ($keyinfo, $keyinfo_1);
    my $tcount = 0;
    my $fan_flag = 0;
    my $connect_flag = 0;
    my $connectcount = 0;
    my $acp_flag = 0;
    my $acpcount = 0;
    my $fancount = 0;
    my $voltage_flag = 0;
    my $voltagecount = 0;
    my $current_flag = 0;
    my $currentcount = 0;
    my $powerversioncount = 0;
    my $powertypecount = 0;
    if ($input{filermode} ne "normal") {
        my $cmd = "environment status"; 
#	(undef, undef, $outputinfo) = say($conn, "$cmd", -prompt, "$MAINT_PROMPT", "-timeout", $input{timeout});
	$outputinfo = filer_cmd(host=>$input{host},filermode=>$input{filermode},conntype=>$input{conntype},timeout=>$input{timeout},cmd=>$cmd);
    } else {
        my $cmd = "priv set -q diag; environment status";
#	(undef, undef, $outputinfo) = say($conn, "$cmd", "-timeout", $input{timeout});
	$outputinfo = filer_cmd(host=>$input{host},filermode=>$input{filermode},conntype=>$input{conntype},timeout=>$input{timeout},cmd=>$cmd);
    }
    #if(($outputinfo !~ /Fan ok/) && ($outputinfo !~ /Fans ok/) && ($outputinfo !~ /Voltage ok/) && ($outputinfo !~ /Power ok/)) {
    if($outputinfo !~ /Environment for/){
	    croak $outputinfo;
    } 
    @info = split(/\r?\n/, $outputinfo);
    foreach $line (@info) {
#	printf "Now print $line \n";
	@status = split(/\s+|=|;/, $line);
#	printf "Now the status 1 is $status[1].\n";
	# Get the general environment status of the FILER
	if (defined ($line)) {
	    if ($line =~ /Voltage/) {
		if (!$channel_flag){
		    $HOH{$filer}{'Voltage'} = $status[1];	
		}
	    } elsif ($line =~ /^Power Supply/) {
		if (!$channel_flag){
		    $HOH{$filer}{'Powersupply'} = $line;	
		}
	    } elsif($line =~ /^Power/) {
		if (!$channel_flag){
		    $HOH{$filer}{'Power'} = $status[1];	
		}
	    } elsif ($line =~ /^Power Supply:/) {
		if (!$channel_flag){
		    ++$powersupp;
		    $keyinfo = "powerused$powersupp";
		    $HOH{$filer}{$keyinfo} = $line;
		}
	    } elsif ($line =~ /RTC/) {
		$HOH{$filer}{'RTCBattery'} = $status[2]; 
	    } elsif ($line =~ /Power supply/) {
		if (!$channel_flag) {
		    $keyinfo = "power$status[2]";
		    $HOH{$filer}{$keyinfo} = $status[5];
		}
	    } elsif ($line =~ /^Temp/) {
		if (!$channel_flag){
		    $keyinfo = $status[0];
		    $HOH{$filer}{$keyinfo} = $line;
		}
	    }elsif ($line =~ /Battery/) {
		if (!$channel_flag){
		    $keyinfo = $status[0];
		    $HOH{$filer}{$keyinfo} = $line;
		}
	    }elsif ($line =~ /^Temperature/) {
		if (!$channel_flag) {
		    $HOH{$filer}{'Temperature'} = $status[1];
		}
	    }
	    elsif ($line =~ /^Channel/) {
	   	logcomment ("Now there are ATA shelves on the FILER.");
	   	$tcount = 0;
	   	$specialchannel = $status[1];
	   	$specialshelf = $status[3];
	   	$HOH{$specialchannel}{$specialshelf}{'Moduletype'} = $status[6];
	    }
	   #elsif ($line =~ /^Shelf/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'Shelfserial'} = $status[3];
	   # } elsif ($line =~ /^Module/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'Moduleserial'} = $status[4];
	   # } elsif ($line =~ /link_up/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'Linkup'} = $status[2];
	   # } elsif ($line =~ /link_down/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'Linkdown'} = $status[2]; 
	   # } elsif ($line =~ /link_toggle/) {
	   #	$toggle = "linktoggle$tcount";
	   #	$HOH{$specialchannel}{$specialshelf}{$toggle} = $status[2];
	   #	++$tcount;
	   # } elsif ($line =~ /watchdog/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'watchdogboot'} = $status[2]; 
	   # } elsif ($line =~ /warm/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'warmboot'} = $status[2]; 
	   # } elsif ($line =~ /current_soft/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'currentsoft'} = $status[1];
	   # } elsif ($line =~ /current_hard/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'currenthard'} = $status[1];
	   # } elsif ($line =~ /total_soft/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'totalsoft'} = $status[2];
	   # } elsif ($line =~ /total_hard/) {
	   #	$HOH{$specialchannel}{$specialshelf}{'totalhard'} = $status[2];
	   #  } elsif ($status[0] =~ /\./) {
	   #	$diskid = $status[0];
	   #	$HOH{$specialchannel}{$specialshelf}{$diskid}{'lossofsingal'} = $status[1];
	   #	$HOH{$specialchannel}{$specialshelf}{$diskid}{'invalidtxcrc'} = $status[2];
	   #	$HOH{$specialchannel}{$specialshelf}{$diskid}{'invalidrxcrc'} = $status[3];
	   #	$HOH{$specialchannel}{$specialshelf}{$diskid}{'invalidtxword'} = $status[4];
	     else { 
		if ($line =~ /Environment for/){
		    $channel_flag = 1;
		    $channelname = $status[4];
		} elsif ($line =~ /Number of shelves/) {
		    $HOH{$channelname}{'Shelfnumber'} = $status[5];
		    $HOH{$channelname}{'Shelfmode'} = $status[7];
		} elsif($line =~ /Environmental failure/) {
		    $HOH{$channelname}{'Shelvesfail'} = $status[8];
		} elsif($line =~ /Shelf:/) {
		    $shelfname = $status[2];
		    $powersupp = 0;
		    $powerversioncount = 0;
		    $powertypecount = 0;
		    $ses_flag = 0;
		    $vendor_flag = 0;
		    $shelfbay_flag = 0;
		    $power_flag = 0;
		    $temp_flag = 0;
		    $thresh_flag = 0;
		    $es_flag = 0;
		    $temp_count = 0;
		    $thresh_high = 0;
		    $thresh_low = 0;
		    $esserialcount = 0;
		    $shelf_flag = 0;
		    $powercount = 0;
		    $fan_flag = 0;
		    $voltage_flag = 0;
		    $current_flag = 0;
		    $fancount = 0;
		    $voltagecount = 0;
		    $currentcount = 0;
		    $connect_flag = 0;
		    $connectcount = 0;
		    $acp_flag = 0;
		    $acpcount = 0;
		} elsif($line =~ /SES device/) {
		    $HOH{$channelname}{$shelfname}{'SESPATH'} = $status[6];
		} elsif($line =~ /Module type/) {
		    $HOH{$channelname}{$shelfname}{'MODULE'} = $status[3];
		    $HOH{$channelname}{$shelfname}{'MONITORING'} = $status[7];
		} elsif($line =~ /Shelf status/){
		    $HOH{$channelname}{$shelfname}{'ShelfStatus'} = $status[3];
		} elsif($line =~ /SES Configuration,/) {
		    $ses_flag = 1;
		    $HOH{$channelname}{$shelfname}{'sesconfigid'} = $line;
		} elsif((($line =~ /logical/)||($line =~ /vendor/)) && (!$power_flag)) {
		    if ($ses_flag) {
			$keyinfo = "ses$status[1]";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $status[3];
		    }
		} elsif($line =~ /product identification/) {
		    if ($ses_flag) {
			$keyinfo = "sesproduct$status[2]";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $status[3];
		    }
		} elsif($line =~ /product revision/) {
		    if ($ses_flag) {
			$keyinfo = "sesproduct$status[2]";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $status[4];
		    }
		}elsif($line =~ /Vendor-specific/) {
		    $vendor_flag = 1;
		} elsif(($line =~ /backplane/)||($line =~ /Backplane/)){
		    if ($vendor_flag) {
			$keyinfo = "backplane$status[2]";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    }
		} elsif ($line =~ /Serial Num/) {
		    if ($vendor_flag) {
			$HOH{$channelname}{$shelfname}{'vendorserial'} = $status[4];
		    }
		} elsif ($line =~ /Kernel/) {
		    if ($vendor_flag) {
			$HOH{$channelname}{$shelfname}{'kenerlversion'} = $status[4];
			$HOH{$channelname}{$shelfname}{'Appversion'} = $status[8];
		    }
		} elsif (($line =~ /Status reads/)||($line =~ /Control writes/)) {
		    $keyinfo = "$status[1]attempted";
		    $keyinfo_1 = "$status[1]failure";
		    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[4];
		    $HOH{$channelname}{$shelfname}{$keyinfo_1} = $status[7];
		} elsif ($line =~ /Shelf bays/){
		    $shelfbay_flag = 1;
		} elsif(($shelfbay_flag) && ($status[1] =~ /\d*,/ ) && (!$power_flag)) {
		    $HOH{$channelname}{$shelfname}{'shelfbays'} = $line; 
		} elsif (($shelfbay_flag) && ($line =~ /Power Supply installed/)) {
		    $HOH{$channelname}{$shelfname}{'powererr'} = $status[11];
		}  elsif ($line =~ /Shelf voltages/) {
		    $voltage_flag = 1;
		} elsif($status[1] =~ /\[\d+\]/){
		    if (($voltage_flag)&&(!$current_flag)) {
			++$voltagecount;
			$keyinfo = "voltage$voltagecount";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    } elsif (($voltage_flag) && ($current_flag) && (!$fan_flag)) {
			++$currentcount;
			$keyinfo = "current$currentcount";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    } elsif (($power_flag) && (!$voltage_flag) && (!$temp_flag)) {
			++$powercount;
			$keyinfo = "powerserial$powercount";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    } elsif (($current_flag) && ($fan_flag) && (!$temp_flag)) {
			++$fancount;
			$keyinfo = "fanelement$fancount";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    } elsif (($temp_flag) && (!$thresh_flag) && (!$es_flag)) {
			++$temp_count;
			$keyinfo = "shelftemp$temp_count";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    } elsif (($temp_flag) && ($thresh_flag)&&(!$es_flag)) {
			++$thresh_high;
			$keyinfo = "tempthreshhigh$thresh_high";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    } elsif (($temp_flag) && ($thresh_flag) && ($es_flag) && (!$connect_flag)) {
			++$esserialcount;
			$keyinfo = "ESserial$esserialcount";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		    } elsif (($es_flag) && ($connect_flag) && (!$acp_flag)) {
		        logcomment("====================Now to get cable information");
			++$connectcount;
			$keyinfo = "SASconnector$connectcount";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
			
		    } elsif (($es_flag) && ($connect_flag) && ($acp_flag) && (!$shelf_flag)) {
			++$acpcount;
			$keyinfo = "ACPelement$acpcount";
			$HOH{$channelname}{$shelfname}{$keyinfo} = $line;
			
		    }
		} elsif ($line =~ /Shelf currents/) {
		    $current_flag = 1;		
		} elsif ($line =~ /Power Supply information/){
		    $power_flag = 1;
		    $powercount = 0;
		} elsif (($line =~ /Type/) && ($power_flag) && (!$connect_flag)){
		    ++$powertypecount;
		    $keyinfo = "powersupplytype$powertypecount";
		     $HOH{$channelname}{$shelfname}{$keyinfo} = $line; 
		} elsif (($line =~ /Firmware version:/) && ($power_flag) && (!$connect_flag)){
		    ++$powerversioncount;
		    $keyinfo = "powersupplyversion$powerversioncount";
		    $HOH{$channelname}{$shelfname}{$keyinfo} = $line; 
		} elsif($line =~ /Cooling Unit installed/){
		    $HOH{$channelname}{$shelfname}{'coolingelements'} = $line; 
		}elsif ($line =~ /Cooling Units by/) {
		    $fan_flag = 1; 
		} elsif($line =~ /Temperature Sensor/) {
		    $HOH{$channelname}{$shelfname}{'Temperaturesensor'} = $line;
		} elsif($line =~ /Shelf temperature/) {
		    $temp_flag =1;
		}  elsif($line =~ /Temperature thresholds/) {
		    $thresh_flag = 1;
		} elsif($line =~ /Low/){
		    ++$thresh_low;
		    $keyinfo = "tempthreshlow$thresh_low";
		    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
		} elsif($line =~ /ES Electronics installed/){
		    $HOH{$channelname}{$shelfname}{'ESinstalled'} = $line;
		} elsif($line =~ /ES Electronics reporting/){
		    $HOH{$channelname}{$shelfname}{'ESreporting'} = $line;
		} elsif($line =~ /ES Electronics information/){
		    $es_flag = 1;
		} elsif($line =~ /SAS connector attached/) {
		    $HOH{$channelname}{$shelfname}{'Connectorattached'} = $line;
		} elsif($line =~ /SAS cable information/) {
		    print "Now turn on the cable flag.\n";
		    $connect_flag = 1;
		    $connectcount = 0;   
		} elsif($line =~ /ACP installed/) {
		    $HOH{$channelname}{$shelfname}{'ACPinstalled'} = $line;
		} elsif($line =~ /ACP information by/) {
		    print "Now turn on the ACP flag.\n";
		    $acp_flag = 1;
		    $acpcount = 0;
		} elsif ($line =~ /SAS Expander/) {
		    $HOH{$channelname}{$shelfname}{'Expandermodule'} = $line;
		} elsif($line =~ /Shelf mapping/){
		    $shelf_flag = 1;
		} elsif($line =~ /Shelf\s+(\d+):\s+(.*)/){
		    if ($shelf_flag) {
			$shelf1 = $1;
			@shelf_fields = split(/\s+/, $2);
			$HOH{$channelname}{$shelf1}{'shelfmapping'} = [@shelf_fields];
		    }
		}
	    }
	}    
    }
    return \%HOH;
}



=head2 environ_shelf(%input);

The routine collect the specified adapter environment information of system by using environ
command

=item host
(Required) : Host spec to the FILER

=item adapt
(Required) : Adapter of the FILER

=item conntype
(Optional) : Connection type to the Filer. Default is rsh.

=item filermode
(Optional): the state of the filer. Default is normal.


=item Return

Return the hash of the environment information

=cut

=back

=item Example: 
    
    $ret = environ_status(%input);

=item SEE ALSO
   None

=back

=cut

sub environ_shelf {

    my (%input) = validate(
        @_,
        {
            host     => { type => OBJECT, isa     => 'Hostrec' },
            conntype => { type => SCALAR, default => "#rsh", optional => 1 },
            adapt     => { type => SCALAR },
            filermode => { type => SCALAR, default => "normal", optional => 1 },
            timeout   => { type => SCALAR, default => 300, optional => 1 },
        }
    );
    if ( $input{filermode} ne "normal" ) {
        $input{conntype} = "#console";
    }

    my (
        $outputinfo,     @info,         %environinfo, $cmd,
        @status,         $channelname,  $shelfname,   $shelf1,
        $specialchannel, $specialshelf, $diskid,      $toggle
    );
    my %HOH;
    my $line  = undef;
    my $filer = $input{host} ->hostname();
    my $conn  = make_conn($input{host},$input{conntype});
    my $adapter_name  = $input{adapt};
    my $powersupp     = 0;
    my $ses_flag      = 0;
    my $vendor_flag   = 0;
    my $shelfbay_flag = 0;
    my $power_flag    = 0;
    my $temp_flag     = 0;
    my $thresh_flag   = 0;
    my $es_flag       = 0;
    my $temp_count    = 0;
    my $thresh_high   = 0;
    my $thresh_low    = 0;
    my $esserialcount = 0;
    my $shelf_flag    = 0;
    my $powercount    = 0;
    my @shelf_fields;
    my $channel_flag = 0;
    my ( $keyinfo, $keyinfo_1 );
    my $tcount       = 0;
    my $fan_flag     = 0;
    my $fancount     = 0;
    my $voltage_flag = 0;
    my $voltagecount = 0;
    my $current_flag = 0;
    my $currentcount = 0;

    if ( $input{filermode} ne "normal" ) {
        my $cmd = "environment shelf $adapter_name";
        # ( undef, undef, $outputinfo ) = say( $conn, "$cmd", -prompt, "$MAINT_PROMPT", "-timeout", $input{timeout} );
	$outputinfo = filer_cmd(host=>$input{host},filermode=>$input{filermode},conntype=>$input{conntype},timeout=>$input{timeout},cmd=>$cmd);
    }
    else {
        my $cmd = "priv set -q diag; environment shelf $adapter_name";
        # ( undef, undef, $outputinfo ) = say( $conn, "$cmd", "-timeout", $input{timeout} );
	$outputinfo = filer_cmd(host=>$input{host},filermode=>$input{filermode},conntype=>$input{conntype},timeout=>$input{timeout},cmd=>$cmd);
    }
    if ( $outputinfo !~ /Environment for channel/ ) {
        croak $outputinfo;
    }

    @info = split( /\r?\n/, $outputinfo );
    foreach $line (@info) {

        # skip blank lines
        if ( $line eq '' ) { next; }

        @status = split( /\s+|=|;/, $line );
        if ( defined($line) ) {

            if ( $line =~ /^Channel/ ) {
                logcomment("Now there are ATA shelves on the FILER.");
                $tcount                                            = 0;
                $specialchannel                                    = $status[1];
                $specialshelf                                      = $status[3];
                $HOH{$specialchannel}{$specialshelf}{'Moduletype'} = $status[6];
            }
            elsif ( $line =~ /^Shelf/ ) {
                $HOH{$specialchannel}{$specialshelf}{'Shelfserial'} =
                  $status[3];
            }
            elsif ( $line =~ /^Module/ ) {
                $HOH{$specialchannel}{$specialshelf}{'Moduleserial'} =
                  $status[4];
            }
            elsif ( $line =~ /link_up/ ) {
                $HOH{$specialchannel}{$specialshelf}{'Linkup'} = $status[2];
            }
            elsif ( $line =~ /link_down/ ) {
                $HOH{$specialchannel}{$specialshelf}{'Linkdown'} = $status[2];
            }
            elsif ( $line =~ /link_toggle/ ) {
                $toggle = "linktoggle$tcount";
                $HOH{$specialchannel}{$specialshelf}{$toggle} = $status[2];
                ++$tcount;
            }
            elsif ( $line =~ /watchdog/ ) {
                $HOH{$specialchannel}{$specialshelf}{'watchdogboot'} =
                  $status[2];
            }
            elsif ( $line =~ /warm/ ) {
                $HOH{$specialchannel}{$specialshelf}{'warmboot'} = $status[2];
            }
            elsif ( $line =~ /current_soft/ ) {
                $HOH{$specialchannel}{$specialshelf}{'currentsoft'} =
                  $status[1];
            }
            elsif ( $line =~ /current_hard/ ) {
                $HOH{$specialchannel}{$specialshelf}{'currenthard'} =
                  $status[1];
            }
            elsif ( $line =~ /total_soft/ ) {
                $HOH{$specialchannel}{$specialshelf}{'totalsoft'} = $status[2];
            }
            elsif ( $line =~ /total_hard/ ) {
                $HOH{$specialchannel}{$specialshelf}{'totalhard'} = $status[2];
            }
            elsif ( $status[0] =~ /\./ ) {
                $diskid = $status[0];
                $HOH{$specialchannel}{$specialshelf}{$diskid}{'lossofsingal'} =
                  $status[1];
                $HOH{$specialchannel}{$specialshelf}{$diskid}{'invalidtxcrc'} =
                  $status[2];
                $HOH{$specialchannel}{$specialshelf}{$diskid}{'invalidrxcrc'} =
                  $status[3];
                $HOH{$specialchannel}{$specialshelf}{$diskid}{'invalidtxword'} =
                  $status[4];
            }
            else {

                if ( $line =~ /Environment for/ ) {
                    $channel_flag = 1;
                    $channelname  = $status[4];
                }
                elsif ( $line =~ /Number of shelves/ ) {
                    $HOH{$channelname}{'Shelfnumber'} = $status[5];
                    $HOH{$channelname}{'Shelfmode'}   = $status[7];
                }
                elsif ( $line =~ /Environmental failure/ ) {
                    $HOH{$channelname}{'Shelvesfail'} = $status[8];
                }
                elsif ( $line =~ /Shelf:/ ) {
                    $shelfname     = $status[2];
                    $powersupp     = 0;
                    $ses_flag      = 0;
                    $vendor_flag   = 0;
                    $shelfbay_flag = 0;
                    $power_flag    = 0;
                    $temp_flag     = 0;
                    $thresh_flag   = 0;
                    $es_flag       = 0;
                    $temp_count    = 0;
                    $thresh_high   = 0;
                    $thresh_low    = 0;
                    $esserialcount = 0;
                    $shelf_flag    = 0;
                    $powercount    = 0;
                    $fan_flag      = 0;
                    $voltage_flag  = 0;
                    $current_flag  = 0;
                    $fancount      = 0;
                    $voltagecount  = 0;
                    $currentcount  = 0;
                }
                elsif ( $line =~ /SES device/ ) {

                    $HOH{$channelname}{$shelfname}{'SESPATH'} = $status[6];
                }
                elsif ( $line =~ /Module type/ ) {
                    $HOH{$channelname}{$shelfname}{'MODULE'}     = $status[3];
                    $HOH{$channelname}{$shelfname}{'MONITORING'} = $status[7];
                }
                elsif ( $line =~ /Shelf status/ ) {
                    $HOH{$channelname}{$shelfname}{'ShelfStatus'} = $status[3];
                }
                elsif ( $line =~ /SES Configuration,/ ) {

                    $ses_flag = 1;
                    $HOH{$channelname}{$shelfname}{'sesconfigid'} = $line;
                }
                elsif ( ( $line =~ /logical/ ) && ($ses_flag) ) {
                    $keyinfo = "seslogical";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[3];
                }
                elsif (( $line =~ /vendor/ )
                    && ($ses_flag) )
                {
                    $keyinfo = "sesvendor";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[3];
                }
                elsif (( $line =~ /product identification/ )
                    && ($ses_flag) )
                {
                    $keyinfo = "sesproductidentification";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[3];
                }
                elsif (( $line =~ /product revision/ )
                    && ($ses_flag) )
                {
                    $keyinfo = "sesproductrevision";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[4];
                    $ses_flag = 0;
                }
                elsif ( $line =~ /Vendor/ ) {
                    $vendor_flag = 1;
                    $ses_flag    = 0;
                }
                elsif (( ( $line =~ /backplane/ ) || ( $line =~ /Backplane/ ) )
                    && ($vendor_flag) )
                {
                    $keyinfo = "backplane$status[2]";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
                }
                elsif ( ( $line =~ /Serial Num/ ) && ($vendor_flag) ) {
                    $HOH{$channelname}{$shelfname}{'vendorserial'} = $status[4];
                }
                elsif (( $line =~ /Kernel/ )
                    && ($vendor_flag) )
                {
                    $HOH{$channelname}{$shelfname}{'kenerlversion'} =
                      $status[4];
                    $HOH{$channelname}{$shelfname}{'Appversion'} = $status[8];
                }
                elsif (( $line =~ /Optional Settings/ )
                    && ($vendor_flag) )
                {
                    $HOH{$channelname}{$shelfname}{'optional'} = $status[5];
                }
                elsif (( $line =~ /Status reads/ )
                    || ( $line =~ /Control writes/ ) )
                {
                    $vendor_flag = 0;
                    $keyinfo     = "$status[1]attempted";
                    $keyinfo_1   = "$status[1]failure";
                    $HOH{$channelname}{$shelfname}{$keyinfo}   = $status[4];
                    $HOH{$channelname}{$shelfname}{$keyinfo_1} = $status[7];
                }
                elsif ( $line =~ /Shelf bays/ ) {
                    $shelfbay_flag = 1;
                }
                elsif (($shelfbay_flag)
                    && ( $status[1] =~ /\d*,/ )
                    && ( !$power_flag ) )
                {
                    $HOH{$channelname}{$shelfname}{'shelfbays'} = $line;
                }
                elsif ( ($shelfbay_flag) && ( $line =~ /with error:/ ) ) {
                    $HOH{$channelname}{$shelfname}{'shelfbay_error'} =
                      $status[3];
                    $shelfbay_flag = 0;
                }
                elsif ( $line =~ /Power Supply installed/ ) {
                    $HOH{$channelname}{$shelfname}{'powererr'} = $status[11];
                }
                elsif ( $line =~ /Power Supply information/ ) {
                    $power_flag = 1;
                    $powercount = 0;
                }

                # power supply info
                elsif ( ( $line =~ /Serial number/ ) && ($power_flag) ) {
                    ++$powercount;
                    $keyinfo = "powerserial$powercount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[4];
                }
                elsif ( ( $line =~ /Type/ ) && ($power_flag) ) {
                    $keyinfo = "powertype$powercount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[2];
                }
                elsif ( ( $line =~ /Firmware version/ ) && ($power_flag) ) {
                    $keyinfo = "powerfwver$powercount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[3];
                }

                elsif ( ( $line =~ /Auto power reset/ ) && ($power_flag) ) {
                    $HOH{$channelname}{$shelfname}{'NDRauto'} = $status[5];
                }

                elsif ( $line =~ /Voltage Sensor/ ) {
                    $power_flag = 0;
                }

                elsif ( $line =~ /Shelf voltages/ ) {
                    $voltage_flag = 1;
                }
                elsif ( ( $status[1] =~ /\[\d\]/ ) && ($voltage_flag) ) {
                    ++$voltagecount;
                    $keyinfo = "voltage$voltagecount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
                }
                elsif ( $line =~ /Shelf currents/ ) {
                    $current_flag = 1;
                    $voltage_flag = 0;
                    $currentcount = 0;
                }
                elsif ( ( $status[1] =~ /\[\d\]/ ) && ($current_flag) ) {

                    ++$currentcount;
                    $keyinfo = "current$currentcount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
                }
                elsif ( $line =~ /Cooling Unit installed/ ) {
                    $power_flag                                       = 0;
                    $current_flag                                     = 0;
                    $HOH{$channelname}{$shelfname}{'coolingelements'} = $line;
                }
                elsif ( $line =~ /Cooling Units by/ ) {
                    $fan_flag = 1;
                }
                elsif ( ( $status[1] =~ /\[\d\]/ ) && ($fan_flag) ) {
                    ++$fancount;
                    $keyinfo = "fanelement$fancount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
                }
                elsif ( $line =~ /Temperature Sensor/ ) {
                    $HOH{$channelname}{$shelfname}{'Temperaturesensor'} = $line;
                }
                elsif ( $line =~ /Shelf temperature/ ) {
                    $temp_flag = 1;
                    $fan_flag  = 0;
                }
                elsif ( ( $status[1] =~ /\[\d\]/ ) && ($temp_flag) ) {
                    ++$temp_count;
                    $keyinfo = "shelftemp$temp_count";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
                }

                elsif ( $line =~ /Temperature thresholds/ ) {
                    $thresh_flag = 1;
                    $temp_flag   = 0;
                }
                elsif ( ( $status[1] =~ /\[\d\]/ ) && ($thresh_flag) ) {
                    ++$thresh_high;
                    $keyinfo = "tempthreshhigh$thresh_high";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
                }
                elsif ( ( $line =~ /Low/ ) && ($thresh_flag) ) {
                    ++$thresh_low;
                    $keyinfo = "tempthreshlow$thresh_low";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $line;
                }
                elsif ( $line =~ /ES Electronics installed/ ) {
                    $HOH{$channelname}{$shelfname}{'ESinstalled'} = $line;
                }
                elsif ( $line =~ /ES Electronics reporting/ ) {
                    $HOH{$channelname}{$shelfname}{'ESreporting'} = $line;
                }
                elsif ( $line =~ /ES Electronics info/ ) {
                    $es_flag     = 1;
                    $thresh_flag = 0;
                }

                elsif ( ( $status[1] =~ /\[\d\]/ ) && ($es_flag) ) {
                    ++$esserialcount;
                    $keyinfo = "ESserial$esserialcount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[4];
                }
                elsif ( ( $line =~ /CPLD/ ) && ($es_flag) ) {
                    $keyinfo = "EScpldver$esserialcount";
                    $HOH{$channelname}{$shelfname}{$keyinfo} = $status[3];
                }
                elsif ( $line =~ /SAS Expander/ ) {
                    $HOH{$channelname}{$shelfname}{'Expandermodule'} = $line;
                }
                elsif ( $line =~ /Shelf mapping/ ) {
                    $shelf_flag = 1;
                }
                elsif ( $line =~ /Shelf\s+(\d+):\s+(.*)/ ) {
                    if ($shelf_flag) {
                        $shelf1 = $1;
                        @shelf_fields = split( /\s+/, $2 );
                        $HOH{$channelname}{$shelf1}{'shelfmapping'} =
                          [@shelf_fields];
                    }
                }
            }
        }
    }
    return \%HOH;
}






#----------------------------------------------------------------------
# The testme routine to test the exported functions of this library
#
# This routine can be invoked in the following manner
#
# ntest -hw FILER=raid-u24 PKGNAME=Storage::Environ util/test_pkg (test all functions in normal filer mode)
#
# ntest -hw FILER=raid-u24 FILERMODE=maint PKGNAME=Storage::Environ util/test_pkg (test all functions in maintenance mode)
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=environ_status PKGNAME=Storage::Environ util/test_pkg (test function environ_status in normal mode)
#
# ntest -hw FILER=raid-u24 FILERMODE=maint FUNCTION_NAME=enviorn_status PKGNAME=Storage::Environ util/test_pkg (test function environ_status in maint mode)
#
# ntest -hw FILER=raid-u24 FUNCTION_NAME=environ_chassis PKGNAME=Storage::Environ util/test_pkg (test function environ_chassis in normal mode)
#
# ntest -hw FILER=raid-u24 FILERMODE=maint FUNCTION_NAME=environ_chassis PKGNAME=Storage::Environ util/test_pkg (test function environ_chassis in maint mode)
#----------------------------------------------------------------------------
sub testme {

    package main;
    our ($FILER);
    our ($FUNCTION_NAME);
    our ($FILERMODE);
    our ($ADAPTER);
    package Storage::Environ;
    param('FILER',       '-mesg',    "Filer to test");
    param('FUNCTION_NAME',       '-default',    "all");
    param('FILERMODE',       '-default',    "normal");

    logcomment("Testing on FILER $FILER");
    logcomment("Testing Function $FUNCTION_NAME");
    logcomment("Testing Filermode $FILERMODE");

    my $host = host($FILER);
    my $ret;

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "environ_status") {
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = environ_status(host=>$host, filermode=>$FILERMODE);
            };

            if ($@) {
                logresult('fail', "function environ_status fail: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_status");
            }
	    #test console connection + normal mode
            eval {
                $ret = environ_status(host=>$host, filermode=>$FILERMODE, conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function  environ_status failed: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_status");
            }
           

	}else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = environ_status(host=>$host, filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function environ_status failed: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_status");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = environ_status(host=>$host, filermode=>"maint", conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function environ_status failed: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_status");
            }

	}
    }
    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "environ_chassis") {
        if ($FILERMODE eq "normal") {
            #test rsh connection + normal mode
            eval {
                $ret = environ_chassis(host=>$host, filermode=>$FILERMODE);
            };

            if ($@) {
                logresult('fail', "function environ_chassis fail: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_chassis");
            }

            #test console connection + normal mode
            eval {
                $ret = environ_chassis(host=>$host, filermode=>$FILERMODE, conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function environ_chassis failed: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_chassis");
            }

	}else {     #maintenance mode testing
            #test implicit console connection + maint mode
            eval {
                $ret = environ_chassis(host=>$host, filermode=>"maint");
            };

            if ($@) {
                logresult('fail', "function environ_chassis failed: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_chassis");
            }

            #test explicit console connection + maint mode
            eval {
                $ret = environ_chassis(host=>$host, filermode=>"maint", conntype=>"#console");
            };

            if ($@) {
                logresult('fail', "function environ_chassis failed: $@");
            } else { 
                print Dumper($ret);
                logresult('pass', "Got expected result from function environ_chassis");
            }

	}
    }
}
=head1 SEE ALSO

    None

=head1 BUGS

    None

=head1 STATUS

    Testing

=head1 ATTRIBUTES

    ONTAP_COMMANDS = disk

=head1 AUTHOR

    Ying Zhang (yingz@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Storage/Environ.pm#1 $
    Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;

