# $Id: //depot/prod/test/Rsierranevada/lib/Storage/Fcadmin.pm#5 $

## @summary Fcadmin - library module for the fcadmin command
## @author dl-storsub-qa
##         hp@netapp.com
## @change 8-27-2008 hansen: modify fcadmin_device_map to correctly handle
## ShelfMapping of multiple shelves
## @change 4-04-2006 erickson: add fcadmin_bridge_cmd_cemi function
## @change 8-19-2011 gurus: Added C-mode support to fcadmin_zombie_adapter() function.
## @status testme
## @pod here
## @change 27-01-2012 gururajc: Added filermode option to the sub fcadmin_bridge_cmd_cemi.
## @change 16-08-2013 jolie: added in support for filer command "would you like to zombify those adapaters too?" if filers isn't host type filer-ng
## @change 21-08-2013 nandeesh: Provided support for handling prompts in sub fcadmin_zombie_adapter() for Cmode and 7Mode: Burt #695754

=head1 NAME

Storage::Fcadmin

=head1 SYNOPSIS

    use Fcadmin qw(fcadmin_link_stats fcadmin_fcal_stats fcadmin_link_state fcadmin_device_map fcadmin_channels fcadmin_offline fcadmin online fcadmin_reset_adapter fcadmin_reset_dev fcadmin_rescan fcadmin_lip fcadmin_hub_pctrl fcadmin_bridge_cmd_cemi);

    my $host = host($FILER);

    my %hash = fcadmin_link_stats(host=>$host,...);
    my %hash = fcadmin_fcal_stats(host=>$host,...);
    my %hash = fcadmin_link_state(host=>$host,...);
    my %hash = fcadmin_device_map(host=>$host,...);
    my %hash = fcadmin_channels(host=>$host,...);
    my %hash = fcadmin_offline(host=>$host,...);
    my %hash = fcadmin_online(host=>$host,...);
    my %hash = fcadmin_reset_adapter(host=>$host,...);
    my %hash = fcadmin_reset_dev(host=>$host,...);
    my %hash = fcadmin_rescan(host=>$host,...);
    my %hash = fcadmin_lip(host=>$host,...);
    my %hash = fcadmin_hub_pctrl(host=>$host,...);
        if (fcadmin_bridge_cmd_cemi(host=>$host,...)){ ...


=head1 DESCRIPTION

This is a library module for the fcadmin command

=head1 ROUTINES

=cut

package Storage::Fcadmin;

use strict;
use warnings;

BEGIN {
    use base qw(Exporter);
    our @EXPORT_OK = qw(fcadmin_link_stats fcadmin_fcal_stats fcadmin_link_state fcadmin_device_map fcadmin_offline fcadmin_online fcadmin_channels fcadmin_reset_adapter fcadmin_reset_dev fcadmin_rescan fcadmin_lip fcadmin_hub_pctrl fcadmin_bridge_cmd_cemi fcadmin_zombie_adapter  fcadmin_inject testme);
    Params::Validate::validation_options(
            on_fail => sub {my @msgs = @_; chomp(@msgs);
            Carp::confess(Tharn::script_result(@msgs))}
            );
}
use Carp qw(confess croak);
use Params::Validate qw(validate OBJECT SCALAR);
use Tharn qw(say script_result host logcomment logresult param);

use Utils qw(make_conn);
use Data::Dumper;
use filer qw(filer_kernel_is_debug);
use Filer::Version qw(get_wafl_version);

use Storage::Ems qw(ems_log_get_mark ems_log_dump_by_type ems_log_check);
#use Storage::Misc qw(filer_cmd) ;
use Storage::SASUtils qw(filer_cmd);

our $MAINT_MODE_PROMPT = "\\*\>";

=head2 fcadmin_link_stats

    my $ret = fcadmin_link_stats(host=>$host, channel_name=>$channel_name);

This routine returns a pointer to hash table of information for each channel in the filer on success and dies on failure. Here is an example of how the hash table looks like:
$VAR1 = '10b.20';
$VAR2 = {
          'LossOfSyncCount' => 16616,
          'LinkFailureCount' => 1,
          'FrameOutCount' => 387501,
          'InvalidCRCCount' => '0',
          'FrameInCount' => 387207,
          'TransportErrorCount' => '0'
        };
$VAR13 = '10a';
$VAR14 = {
           'State' => 'DOWN'
         };
$VAR15 = '10b';
$VAR16 = {
           'LossOfSyncCount' => '0',
           'LinkFailureCount' => '0',
           'FrameOutCount' => 7073067,
           'InvalidCRCCount' => '0',
           'FrameInCount' => 9461467,
           'TransportErrorCount' => '0',
           'State' => 'UP'
         };

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer

=item channel_name

(Optional) : Specific channel for which the information is required.
If not specified then the information for all the channels will be supplied.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=back

=cut

sub fcadmin_link_stats($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                channel_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                }
            );

    my $cmd = "fcadmin link_stats $opts{channel_name}";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
    if ($output =~ /fcadmin link_stats: No such channel/) {
        croak $output;
    }

    my %HoH =();

    my @lines = split(/\r*\n/, $output);
    foreach (@lines) {
        if ($_ =~ /Cannot complete operation on channel (\w+); link is DOWN/) {
            $HoH{$1}{'State'} = "DOWN";
        }
        elsif($_ =~ /(\w+)\./) {
            my @fields = split(/\s+/, $_);
            my $device_name = $fields[0];
            if($device_name =~ /(\w+)\.ha/) {
                if((!exists $HoH{$1}{'State'}) && (!defined $HoH{$1}{'State'})) {
                    $device_name = $1;
                    $HoH{$device_name}{'State'} = "UP";
                }
            }
            $HoH{$device_name}{'LinkFailureCount'} = $fields[1];
            $HoH{$device_name}{'TransportErrorCount'} = $fields[2];
            $HoH{$device_name}{'LossOfSyncCount'} = $fields[3];
            $HoH{$device_name}{'InvalidCRCCount'} = $fields[4];
            $HoH{$device_name}{'FrameInCount'} = $fields[5];
            $HoH{$device_name}{'FrameOutCount'} = $fields[6];
        }
    }
    return \%HoH;
}

=head2 fcadmin_fcal_stats

    my $ret = fcadmin_fcal_stats(host=>$host, channel_name=>$channel_name);

This routine returns a pointer to hash table of information for each channel in the filer on success and dies on failure. Here is an example of how the hash table looks like:
$VAR1 = '10b.0x38';
$VAR2 = {
          'mailbox_timedout' => '0',
          'mailbox_failed' => '0',
          'mailbox_issued' => 1
        };
$VAR3 = '10b.16';
$VAR4 = {
          'device_timer_histogram: 1' => '0',
          'device_victim_abort_cnt' => '0',
          'device_link_fail_cnt' => '0',
          'device_timer_histogram:10' => '0',
          'device_timer_histogram: 2' => '0',
          'device_timer_histogram:11' => '0',
          'device_timer_histogram: 3' => '0',
          'device_timer_histogram:12' => '0',
          'device_timer_histogram: 4' => '0',
          'device_quiesce_cnt' => '0',
          'device_timer_histogram: 5' => '0',
          'device_timer_histogram: 6' => '0',
          'device_timer_histogram: 7' => '0',
          'device_timer_histogram: 8' => '0',
          'device_timer_histogram: 9' => '0',
          'device_deferred_login_cnt:0' => '0',
          'device_logged_out_cnt' => '0',
          'device_deferred_login_cnt:1' => '0',
          'device_deferred_login_cnt:2' => '0',
          'device_incomplete_xfer_cnt' => '0',
          'device_transport_error_cnt' => '0',
          'device_deferred_login_cnt:3' => '0',
          'device_deferred_login_cnt:4' => '0',
          'device_deferred_login_cnt:5' => '0',
          'device_deferred_login_cnt:6' => '0',
          'device_overrun_cnt' => '0',
          'device_unavail_cnt' => '0',
          'device_abort_cnt' => '0',
          'device_forced_login_cnt' => '0',
          'device_timeout_cnt: 0' => '0',
          'device_underrun_cnt' => '0',
          'device_io_holdoff_cnt' => '0',
          'device_timeout_cnt: 1' => '0',
          'device_timeout_cnt: 2' => '0',
          'device_timeout_cnt: 3' => '0',
          'device_timeout_cnt: 4' => '0',
          'device_timeout_cnt: 5' => '0',
          'device_reset_cnt' => '0'
        };
$VAR71 = '10a';
$VAR72 = {
           'device_timer_histogram: 4_total' => '0',
           'device_io_holdoff_cnt_total' => '0',
           'device_overrun_cnt_total' => '0',
           'device_timer_histogram: 9_total' => '0',
           'device_deferred_login_cnt:4_total' => '0',
           'device_link_fail_cnt_total' => '0',
           'adap_sram_parity_error_cnt' => '0',
           'device_timeout_cnt: 3_total' => '0',
           'adap_reinit_force_lipreset_cnt' => '0',
           'device_timer_histogram: 1_total' => '0',
           'device_logged_out_cnt_total' => '0',
           'adap_loop_open_cnt' => 3,
           'device_timer_histogram: 6_total' => '0',
           'device_deferred_login_cnt:1_total' => '0',
           'adap_invalid_response_cnt' => '0',
           'device_timeout_cnt: 0_total' => '0',
           'device_incomplete_xfer_cnt_total' => '0',
           'device_deferred_login_cnt:6_total' => '0',
           'device_timeout_cnt: 5_total' => '0',
           'adap_spurious_interrupt_cnt' => '0',
           'adap_internally_generated_lip_cnt' => '0',
           'device_timer_histogram:11_total' => '0',
           'adap_unexpected_risc_stall_cnt' => '0',
           'device_abort_cnt_total' => '0',
           'adap_dropped_drives_cnt' => '0',
           'device_reset_cnt_total' => '0',
           'device_timer_histogram: 3_total' => '0',
           'adap_reinit_retry_cnt' => 2,
           'adap_max_lips_per_period_cnt' => '0',
           'device_timer_histogram: 8_total' => '0',
           'device_deferred_login_cnt:3_total' => '0',
           'device_underrun_cnt_total' => '0',
           'device_timeout_cnt: 2_total' => '0',
           'adap_bad_handle_cnt' => '0',
           'device_victim_abort_cnt_total' => '0',
           'device_unavail_cnt_total' => '0',
           'adap_p_to_p_link_reestablished_cnt' => '0',
           'adap_reset_cnt' => '0',
           'adap_fatal_error_cnt' => '0',
           'device_timer_histogram: 5_total' => '0',
           'device_deferred_login_cnt:0_total' => '0',
           'adap_status_continuation_cnt' => '0',
           'device_quiesce_cnt_total' => '0',
           'device_deferred_login_cnt:5_total' => '0',
           'device_forced_login_cnt_total' => '0',
           'device_timeout_cnt: 4_total' => '0',
           'device_timer_histogram:10_total' => '0',
           'adap_ct_timeout_cnt' => '0',
           'adap_revisited_response_cnt' => '0',
           'adap_externally_generated_lip_cnt' => '0',
           'device_timer_histogram: 2_total' => '0',
           'adap_link_down_cnt' => '0',
           'device_timer_histogram: 7_total' => '0',
           'device_deferred_login_cnt:2_total' => '0',
           'device_timeout_cnt: 1_total' => '0',
           'adap_dma_timeout_cnt' => '0',
           'adap_bad_response_cnt' => '0',
           'adap_warped_response_index_cnt' => '0',
           'device_timer_histogram:12_total' => '0',
           'device_transport_error_cnt_total' => '0'
         };

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item channel_name

(Optional) : Specific channel for which the information is required.
If not specified then the information for all the channels will be supplied.

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=item timeout

(Optional) : Timeout value for the say. Default : 300

=back

=cut

sub fcadmin_fcal_stats($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                channel_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                }
            );

    my $cmd = "fcadmin fcal_stats $opts{channel_name}";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if ($output =~ /fcadmin fcal_stats: No such channel/) {
        croak $output;
    }

    my %HoH =();

    my @lines = split(/\r*\n/, $output);
    my ($slot_name, @mailbox_fields, @device_fields);
    foreach (@lines) {
        #Get the slot
        if($_ =~ /driver statistics for slot (\w+)/) {
            $slot_name = $1;
        }
        #Get adapter-related information
        elsif($_ =~ /adap_/) {
            $_ =~ s/^\s+//;
            my @adap_fields = split(/:\s*/, $_);
            $adap_fields[1] =~ s/\s+//;

            $HoH{$slot_name}{$adap_fields[0]} = $adap_fields[1];
        }
        #Get mailbox-related information
        elsif($_ =~ /mailbox status: (.*)/) {
            @mailbox_fields = split(/\s+/, $1);
        }
        elsif(($_ =~ /(issued)(.*)/) || ($_ =~ /(failed)(.*)/) || ($_ =~ /(timed out)(.*)/)) {
            my $mailbox_status = "mailbox_".$1;
            my $mailbox_status_fields = $2;
            $mailbox_status =~ s/\s+//;
            $mailbox_status_fields =~ s/\s+//;
            my @fields = split(/\s+/, $mailbox_status_fields);

            my $count;
            for($count = 0 ; $count < @mailbox_fields ; $count++) {
                my $mailbox_name = $slot_name."\.".$mailbox_fields[$count];

                $HoH{$mailbox_name}{$mailbox_status} = $fields[$count];
            }
        }
        #Get device-related information
        elsif($_ =~ /device status:\s+(.*)/) {
            @device_fields = split(/\s+/, $1);
        }
        elsif($_ =~ /device_/) {
            my ($device_status, @fields);
            $_ =~ s/^\s+//;
            if($_ =~ /^\w+:/) {
                $_ =~ /(\w+:?\s*\d*)\s+(.*)/;
                $device_status = $1;
                @fields = split(/\s+/, $2);
            }
            else {
                $_ =~ /(\w+)\s+(.*)/;
                $device_status = $1;
                @fields = split(/\s+/, $2);
            }
            my $count = 0;
            for($count = 0 ; $count < @device_fields ; $count++) {
                my $device_name = $slot_name."\.".$device_fields[$count];

                $HoH{$device_name}{$device_status} = $fields[$count];
            }
            my $total = $device_status."_total";
            $HoH{$slot_name}{$total} = $fields[$count+1];
        }
    }
    return \%HoH;
}

=head2 fcadmin_link_state

    my $ret = fcadmin_link_state(host=>$host, adapter_name=>$adapter_name);

This routine returns a pointer to hash table of information for each adapter in the filer on success and dies on failure. Here is an example of how the hash table looks like:
$VAR1 = '10a';
$VAR2 = {
          'SwitchPort' => undef,
          'StateType' => 'physical',
          'PortType' => undef,
          'SwitchWWN' => undef,
          'Switch' => undef,
          'State' => 'OFFLINE'
        };
$VAR3 = '10b';
$VAR4 = {
          'SwitchPort' => undef,
          'StateType' => '-',
          'PortType' => 'L-port',
          'SwitchWWN' => undef,
          'Switch' => undef,
          'State' => 'UP'
        };

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific adapter for which the information is required.
If not specified then the information for all the adapters will be supplied.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=back

=cut

sub fcadmin_link_state($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                adapter_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                }
            );

    my $cmd = "fcadmin link_state $opts{adapter_name}";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);


    if ($output =~ /fcadmin link_state: No such channel/) {
        croak $output;
    }

    my $struct = {};

    my @lines = split(/\r*\n/, $output);
    foreach my $line (@lines) {
        if($line !~ /fcadmin link_state/) {
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
            my @fields = split(/\s+/, $line);

            my $adapter_name = $fields[5];
            $adapter_name =~ /(\w+):/;
            $adapter_name = $1;
            $struct->{$adapter_name}->{State} = $fields[6];
            my $state_type = $fields[7];
            $state_type =~ s/^\(//;
            $state_type =~ s/\)$//;
            $struct->{$adapter_name}->{StateType} = $state_type;
            $struct->{$adapter_name}->{PortType} = $fields[8];
            my $switch_name = $fields[14];
            $switch_name =~ s/^\'//;
            $switch_name =~ s/\'$//;
            $struct->{$adapter_name}->{Switch} = $switch_name;
            $struct->{$adapter_name}->{SwitchPort} = $fields[11];
            my $switch_wwn = $fields[15];
            $switch_wwn =~ s/^\(//;
            $switch_wwn =~ s/\)$//;
            $struct->{$adapter_name}->{SwitchWWN} = $switch_wwn;
        }
    }
    return $struct;
}

=head2 fcadmin_device_map

    my $ret = fcadmin_device_map(host=>$host, channel_name=>$channel_name);

This routine returns a pointer to hash table of information for each channel in the filer on success
and dies on failure.
Here is an example of how the hash table looks like:

$VAR1 = {
          '5d' => {
                    'PortCount' => '34',
                    '1' => {
                             'TargetSES' => [
                                              '14',
                                              '15'
                                            ],
                             'ShelfMapping' => [
                                                 '29',
                                                 '28',
                                                 '27',
                                                 '26',
                                                 '25',
                                                 '24',
                                                 '23',
                                                 '22',
                                                 '21',
                                                 '20',
                                                 '19',
                                                 '18',
                                                 '17',
                                                 '16'
                                               ]
                           },
                    '2' => {
                             'TargetSES' => [
                                              '30',
                                              '31'
                                            ],
                             'ShelfMapping' => [
                                                 '45',
                                                 '44',
                                                 '43',
                                                 '42',
                                                 '41',
                                                 '40',
                                                 '39',
                                                 '38',
                                                 '37',
                                                 '36',
                                                 '35',
                                                 '34',
                                                 '33',
                                                 '32'
                                               ]
                           },
                    'State' => 'UP',
                    'TranslatedMap' => [
                                         '0',
                                         '7',
                                         '14',
                                         '15',
                                         '16',
                                         '17',
                                         '18',
                                         '19',
                                         '20',
                                         '21',
                                         '22',
                                         '23',
                                         '24',
                                         '25',
                                         '26',
                                         '27',
                                         '28',
                                         '29',
                                         '30',
                                         '31',
                                         '32',
                                         '33',
                                         '34',
                                         '35',
                                         '36',
                                         '37',
                                         '38',
                                         '39',
                                         '40',
                                         '41',
                                         '42',
                                         '43',
                                         '44',
                                         '45'
                                       ]
                  },
          '0b' => {
                    'State' => 'DOWN'
                  },
          '0g' => {
                    'PortCount' => '30',
                    '1' => {
                             'ShelfMapping' => [
                                                 '29',
                                                 '28',
                                                 '27',
                                                 '26',
                                                 '25',
                                                 '24',
                                                 '23',
                                                 '22',
                                                 '21',
                                                 '20',
                                                 '19',
                                                 '18',
                                                 '17',
                                                 '16'
                                               ]
                           },
                    '2' => {
                             'ShelfMapping' => [
                                                 '45',
                                                 '44',
                                                 '43',
                                                 '42',
                                                 '41',
                                                 '40',
                                                 '39',
                                                 '38',
                                                 '37',
                                                 '36',
                                                 '35',
                                                 '34',
                                                 '33',
                                                 '32'
                                               ]
                           },
                    'State' => 'UP',
                     'TranslatedMap' => [
                                         '7',
                                         '0',
                                         '29',
                                         '28',
                                         '27',
                                         '25',
                                         '26',
                                         '23',
                                         '22',
                                         '21',
                                         '20',
                                         '16',
                                         '19',
                                         '18',
                                         '17',
                                         '24',
                                         '45',
                                         '44',
                                         '43',
                                         '41',
                                         '42',
                                         '39',
                                         '38',
                                         '37',
                                         '36',
                                         '32',
                                         '35',
                                         '34',
                                         '33',
                                         '40'
                                       ]
                  },
         };

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item channel_name

(Optional) : Specific channel for which the information is required.
If not specified then the information for all the channels will be supplied.

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=item timeout

(Optional) : Timeout value for the say. Default : 300

=back

=cut

sub fcadmin_device_map($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                channel_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                }
            );

    my $cmd = "fcadmin device_map $opts{channel_name}";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if ($output =~ /fcadmin device_map: No such channel/) {
        croak $output;
    }

    my %HoH =();

    my @lines = split(/\r*\n/, $output);
    my ($channel, $shelf, $translated_map_flag, @translated_map, $shelf_map_flag, @shelf_map, $target_ses_flag, @target_ses_map);
    $translated_map_flag = $shelf_map_flag = $target_ses_flag = 0;
    $shelf = '';
    foreach (@lines) {
        $_ =~ s/^\s+//;
        $_ =~ s/\s+$//;
        if ($_ =~ /Cannot complete operation on channel (\w+); link is DOWN/) {
            $HoH{$1}{'State'} = "DOWN";
        }
        elsif($_ =~ /Loop Map for channel (\S+):$/) {
            my @channel = split(/\s+/,$_);
            $channel = $channel[4];
            chop($channel);
            #$channel = $1;
            $HoH{$channel}{'State'} = "UP";
        }
        elsif($_ =~ /Translated Map: Port Count (\d+)/) {
            $HoH{$channel}{'PortCount'} = $1;
            $translated_map_flag = 1;
        }
        elsif(($translated_map_flag) && ($_ !~ /Shelf mapping:/)) {
            my @fields1 = split(/\s+/, $_);
            push(@translated_map, @fields1);
        }
        elsif($_ =~ /Shelf mapping:/) {
            $shelf_map_flag = 1;
            $HoH{$channel}{'TranslatedMap'} = [@translated_map];
            # clear map
            @translated_map = ();
            $translated_map_flag = 0;
        }
        elsif($_ =~ /Target SES devices on this loop:/) {
            $target_ses_flag = 1;
            $shelf_map_flag = 0;
        }
        elsif ($_ =~ /^Shelf (\d+):\s+(.*)/) {
            $shelf = $1;

            # Die if the shelf configuration is not valid
            if ($shelf eq "Unknown") {
                die "Configuration issue in loop: Loop $channel not properly configured";
            }

            my @fields = split(/\s+/, $2);
            if($shelf_map_flag) {
                push(@shelf_map, @fields);
                print "\nShelf Map : @shelf_map";
                $HoH{$channel}{$shelf}{'ShelfMapping'} = [@shelf_map];
                @shelf_map = ();
            }
            elsif($target_ses_flag) {
                push(@target_ses_map, @fields);
                $HoH{$channel}{$shelf}{'TargetSES'} = [@target_ses_map];
                @target_ses_map = ();
            }
        }
    }
    return \%HoH;
}

=head2 fcadmin_channels

    my $ret = fcadmin_channels(host=>$host, channel_name=>$channel_name);

This routine returns a pointer to hash table of information for each channel in the filer on success and dies on failure. Here is an example of how the hash table looks like:
$VAR1 = '10a';
$VAR2 = {
          'WWN' => '200000e08b1cfed7',
          'FA' => undef,
          'PortType' => 'L-port',
          'PhysPort' => undef,
          'PeerWWN' => undef,
          'SwitchWWN' => undef,
          'GIHAID' => 1,
          'LFA' => undef
        };
$VAR3 = '10b';
$VAR4 = {
          'WWN' => '200100e08b3cfed7',
          'FA' => undef,
          'PortType' => 'L-port',
          'PhysPort' => undef,
          'PeerWWN' => undef,
          'SwitchWWN' => undef,
          'GIHAID' => 2,
          'LFA' => undef
        };

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item channel_name

(Optional) : Specific channel for which the information is required.
If not specified then the information for all the channels will be supplied.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=back

=cut

sub fcadmin_channels($%) {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
            channel_name => { type=>SCALAR, optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            }
            );

    my $cmd = "fcadmin channels $opts{channel_name}";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if ($output =~ /fcadmin channels: No such channel/) {
        croak $output;
    }

    my %HoH =();

    my @lines = split(/\r*\n/, $output);
    my $channel;
    foreach (@lines) {
        $_ =~ s/^\s+//;
        $_ =~ s/\s+$//;
        my @fields = split(/\s+/, $_);
        $channel = $fields[1];
        $HoH{$channel}{'WWN'} = $fields[0];
        $HoH{$channel}{'PortType'} = $fields[2];
        if($_ =~ /GIHAID:\s+(\w+)[\,\)]/) {
            $HoH{$channel}{'GIHAID'} = $1;
        }
        else {
            $HoH{$channel}{'GIHAID'} = undef;
        }
        if($_ =~ /Peer WWN:\s+(\w+)[\,\)]/) {
            $HoH{$channel}{'PeerWWN'} = $1;
        }
        else {
            $HoH{$channel}{'PeerWWN'} = undef;
        }
        if($_ =~ /Phys Port:\s+(\w+)[\,\)]/) {
            $HoH{$channel}{'PhysPort'} = $1;
        }
        else {
            $HoH{$channel}{'PhysPort'} = undef;
        }
        if($_ =~ /FA:\s+(\w+)[\,\)]/) {
            $HoH{$channel}{'FA'} = $1;
        }
        else {
            $HoH{$channel}{'FA'} = undef;
        }
        if($_ =~ /LFA:\s+(\w+)[\,\)]/) {
            $HoH{$channel}{'LFA'} = $1;
        }
        else {
            $HoH{$channel}{'LFA'} = undef;
        }
        if($_ =~ /Switch WWN:\s+(\w+)[\,\)]/) {
            $HoH{$channel}{'SwitchWWN'} = $1;
        }
        else {
            $HoH{$channel}{'SwitchWWN'} = undef;
        }
    }
    return \%HoH;
}

=head2 fcadmin_offline

    my $ret = fcadmin_offline(host=>$host, adapter_name=>$adapter_name);

This routine offlines the specified adapter. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific adapter which is to be offlined.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item ems_verify

(Optional) : Indicates whether error-checking is to be performed via EMS or not. Default : yes

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=back

=cut

sub fcadmin_offline($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                adapter_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                ems_verify => { type=>SCALAR, default=>"yes", optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                }
            );

    # If the filer is in maintenance mode, console connection is required. Also,
    # EMS messages are not generated by events.
    if($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
        $opts{ems_verify} = "no";
    }

    # no zapi so make connection
    my ($cmd, $mark);
    my $host = $opts{host};
    $cmd = "fcadmin offline $opts{adapter_name}";

    if($opts{ems_verify} eq "yes") {
        $mark = ems_log_get_mark(host=>$host);
    }

     my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if($output =~ /fcadmin offline: No such channel $opts{adapter_name}/) {
        croak $output;
    }

    if($opts{ems_verify} eq "yes") {
        my $ret = ems_log_check(host=>$host, eventtype=>"fci.adapter.offline", key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
        if($ret == 0) {
            croak "Adapter $opts{adapter_name} was not offlined";
        }
        elsif($ret > 1) {
            croak "Adapter $opts{adapter_name} was offlined more than once";
        }
    }
    return 1;
}

=head2 fcadmin_online

    my $ret = fcadmin_online(host=>$host, adapter_name=>$adapter_name);

This routine onlines the specified adapter. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific adapter which is to be onlined.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item ems_verify

(Optional) : Indicates whether error-checking is to be performed via EMS or not. Default : yes
=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=back

=cut

sub fcadmin_online($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                adapter_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                ems_verify => { type=>SCALAR, default=>"yes", optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                }
            );

    # If the filer is in maintenance mode, console connection is required. Also,
    # EMS messages are not generated by events.
    if($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
        $opts{ems_verify} = "no";
    }

    # no zapi so make connection
    my ($cmd, $mark);
    my $host = $opts{host};
    $cmd = "fcadmin online $opts{adapter_name}";
    if($opts{ems_verify} eq "yes") {
        $mark = ems_log_get_mark(host=>$host);
    }
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if($output =~ /fcadmin online: No such channel $opts{adapter_name}/) {
        croak $output;
    }

    if($opts{ems_verify} eq "yes") {
        my $ret = ems_log_check(host=>$host, eventtype=>"fci.adapter.onlining", key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
        if($ret == 0) {
            croak "Adapter $opts{adapter_name} was not onlined";
        }
        elsif($ret > 1) {
            croak "Adapter $opts{adapter_name} was onlined more than once";
        }
    }
    return 1;
}

=head2 fcadmin_reset_adapter

    my $ret = fcadmin_reset_adapter(host=>$host, adapter_name=>$adapter_name);

This routine resets the specified adapter. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific adapter which is to be reset.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=item ems_verify

(Optional) : Indicates whether error-checking is to be performed via EMS or not. Default : yes

=back

=cut

sub fcadmin_reset_adapter($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                adapter_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                ems_verify => { type=>SCALAR, default=>"no", optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                }
            );

    # If the filer is in maintenance mode, console connection is required. Also,
# EMS messages are not generated by events.
    if($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
        $opts{ems_verify} = "no";
    }
    my  $host = $opts{host};

    my ($cmd, $mark);
    $cmd = "fcadmin reset_adapter $opts{adapter_name}";

    if($opts{ems_verify} eq "yes") {
        $mark = ems_log_get_mark(host=>$opts{host});
    }

    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);



    if($output =~ /fcadmin reset_adapter: No such channel $opts{adapter_name}/) {
        croak $output;
    }

    if($opts{ems_verify} eq "yes") {
        sleep 30; # It can take upto 30 secs to flush ems message to logs
        my $ret = ems_log_check(host=>$host, eventtype=>"fci.adapter.reset", key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
        if($ret == 0) {
            croak "Adapter $opts{adapter_name} was not reset";
        }
        elsif($ret > 1) {
            croak "Adapter $opts{adapter_name} was reset more than once";
        }
    }
    return 1;
}

=head2 fcadmin_reset_dev

    my $ret = fcadmin_reset_dev(host=>$host, adp_or_dev_name=>$adp_or_dev_name);

This routine resets the specified adapter or device. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adp_or_dev_name

(Optional) : Specific adapter or device which is to be reset.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=item ems_verify

(Optional) : Indicates whether error-checking is to be performed via EMS or not. Default : yes

=back

=cut

sub fcadmin_reset_dev($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                adp_or_dev_name => { type=>SCALAR,default=>undef, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                ems_verify => { type=>SCALAR, default=>"no", optional=>1 },
                }
            );

    # If the filer is in maintenance mode, console connection is required. Also,
    # EMS messages are not generated by events.
    if($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
        $opts{ems_verify} = "no";
    }

    my $host = $opts{host};
    my @adp_list;
    if(!defined($opts{adp_or_dev_name}) && ($opts{ems_verify} eq "yes")) {
        my $ret = fcadmin_link_state(host=>$host, conntype=>$opts{conntype}, filermode=>$opts{filermode});
        foreach my $adapter_name (keys %$ret) {
            if($ret->{$adapter_name}->{"State"} eq "UP"){
                push(@adp_list, $adapter_name);
            }
        }
    }

    my ($cmd, $mark);
    $cmd = "fcadmin reset_dev $opts{adp_or_dev_name}";

    if($opts{ems_verify} eq "yes") {
        $mark = ems_log_get_mark(host=>$host);
    }

    my $output = "";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if($output =~ /fcadmin reset_dev: No such channel $opts{adp_or_dev_name}/) {
        croak $output;
    }

    if(($opts{ems_verify} eq "yes")  && (!defined($opts{adp_or_dev_name}))) {
        sleep 30;
        foreach my $adapter_name (@adp_list) {
            logcomment("\nChecking logs for $adapter_name");
            my $ret = ems_log_check(host=>$host, eventtype=>"fci.channel.resetting", key=>"adapterName", value=>"$adapter_name", timestamp=>$mark);
            if($ret == 0) {
                croak "Adapter $adapter_name was not reset";
            }
            elsif($ret > 1) {
                croak "Adapter $adapter_name was reset more than once";
            }
        }
    }
    elsif(($opts{ems_verify} eq "yes")  && ($opts{adp_or_dev_name} =~ /\./)) {
        sleep 30;
        #Device name passed in as parameter
        my $ret = ems_log_check(host=>$host, eventtype=>"fci.device.resetting", key=>"deviceName", value=>"$opts{adp_or_dev_name}", timestamp=>$mark,patternmatch=>1);
        if($ret == 0) {
            croak "Device $opts{adp_or_dev_name} was not reset";
        }
        elsif($ret > 1) {
            croak "Device $opts{adp_or_dev_name} was reset more than once";
        }
    }
    elsif(($opts{ems_verify} eq "yes")  && ($opts{adp_or_dev_name} !~ /\./)) {
        #Device name passed in as parameter
        my $ret = ems_log_check(host=>$host, eventtype=>"fci.channel.resetting", key=>"adapterName", value=>"$opts{adp_or_dev_name}", timestamp=>$mark);
        if($ret == 0) {
            croak "Adapter $opts{adp_or_dev_name} was not reset";
        }
        elsif($ret > 1) {
            croak "Adapter $opts{adp_or_dev_name} was reset more than once";
        }
    }
    return 1;
}

=head2 fcadmin_rescan

    my $ret = fcadmin_rescan(host=>$host, adapter_name=>$adapter_name);

This routine rescans the specified adapter. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific adapter which is to be rescanned.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=back

=cut

sub fcadmin_rescan($%) {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
            adapter_name => { type=>SCALAR, optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            }
            );

    my $cmd = "fcadmin rescan $opts{adapter_name}";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

    if ($output =~ /fcadmin rescan: No such channel/) {
        croak $output;
    }

    return 1;
}

=head2 fcadmin_lip

    my $ret = fcadmin_lip(host=>$host, adapter_name=>$adapter_name);

This routine issues a LIP on the specified adapter. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific adapter on which the LIP is to be issued.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=back

=cut

sub fcadmin_lip($%) {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
            adapter_name => { type=>SCALAR, optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            filermode => { type=>SCALAR, default=>"normal", optional=>1 },
            }
            );


    my $cmd = "fcadmin lip $opts{adapter_name}";
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);


    if (defined($output) && ($output =~ /fcadmin lip: No such channel/)) {
        croak $output;
    }

    return 1;
}

=head2 fcadmin_zombie_adapter

    my $ret = fcadmin_zombie_adapter(host=>$host, adapter_name=>$adapter_name);

This routine resets the specified adapter. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific adapter which is to be reset.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=item ems_verify

(Optional) : Indicates whether error-checking is to be performed via EMS or not. Default : yes

=back

=cut

sub fcadmin_zombie_adapter($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                adapter_name => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                #noprompt => {type=>SCALAR, default=>"0", optional=>1},
                }
            );

    # If the filer is in maintenance mode, console connection is required. Also,
    # EMS messages are not generated by events.
    if(($opts{filermode} ne "normal") || ($opts{host}->{hosttype} =~ /filer-ng/)) {
        $opts{conntype} = "#console";
    }
    my $conn = make_conn($opts{host},$opts{conntype});
    my ($cmd, $mark);
    my $host = $opts{host};
    if($opts{filermode} eq "normal") {
        if($opts{host}->{hosttype} =~ /filer-ng/) {
            $cmd = "run local \"priv set -q test; fcadmin zombie_adapter $opts{adapter_name}\"";
        } else {
            $cmd = "priv set -q test; fcadmin zombie_adapter $opts{adapter_name}";
        }
    }
    else {
        $cmd = "fcadmin zombie_adapter $opts{adapter_name}";
    }
    my $output;
    my $prompt=@{$host->{prompt_norm}};
    if ($opts{filermode} eq "normal"){
        eval {
            $output = $conn->execute(command => $cmd,
                match_table => [(".*Would you like to zombify those adapters too*") => "n",
                ],
                timeout => $opts{timeout},
                );
        };
        if($@){
            logresult('fail', "$@");
        }
        if($output) {
            print "Executed output is $output\n";
        }
    }
    elsif($opts{conntype} eq "#console") {
        (undef, undef, $output) = say($conn, "$cmd", '-wait', "$opts{adapter_name} is now online", "-timeout", $opts{timeout}, -noprompt);
    }
    else {
        (undef, undef, $output) = say($conn, "$cmd", -prompt, "$MAINT_MODE_PROMPT", "-timeout", $opts{timeout});
    }

    if($output =~ /"fcadmin zombie_adapter: No such channel $opts{adapter_name}"|"Usage:"|"Invalid option"/) {
        croak $output;
    }
    return 1;
}



=head2 fcadmin_hub_pctrl

    my $ret = fcadmin_hub_pctrl(host=>$host, adapter_name=>$adapter_name);

This routine issues a control request to a specific ESH. It returns 1 on success and dies on failure.
The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item wwn

(Required) : Separated value as reported by storage show hub.

=item port

(Required) : Value 0-15 to bypass on shelf.

=item value

(Required) : 0x01 - Auto Mode
             0x02 - Force Bypass
             0x03 - Force Insert
             0x04 - Cascade to Tree
             0x05 - Cascade to String
             0xa0 - Clock Delta Fault bypass
             0xa1 - CRC fault bypass
             0xa2 - WRD fault bypass
             0xa3 - Increment CRC error +1
             0xa4 - Increment CRC error +2
             0xa5 - Increment WRD error +1000
             ----------------------------------
             0xf0 - Force Bypass **ALL**
             0xf1 - Force Insert **ALL**
             0xf2 - Clock Delta Fault Bypass **ALL**
             0xf3 - CRC fault bypass **ALL**
             0xf4 - WRD fault bypass **ALL**
             NOTE: 0xf0-0xf4 affect only drive ports 2-15

=item loop

(Required) : A or B loop of the shelf.

=item timeout

(Optional) : Timeout value for the say. Default : 300

=item filermode

(Optional) : Indicates whether filer is in normal or maintenance mode. Default : normal

=back

=cut

sub fcadmin_hub_pctrl($%) {
    my (%opts) = validate(@_, {
                host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                wwn => { type=>SCALAR, optional=>1 },
                port => { type=>SCALAR, optional=>1 },
                value => { type=>SCALAR, optional=>1 },
                loop => { type=>SCALAR, optional=>1 },
                timeout => { type=>SCALAR, default=>300, optional=>1 },
                filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                ems_verify => { type=>SCALAR, default=>"yes", optional=>1 },
                }
            );

    if($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
        $opts{ems_verify} = "no";
    }

    my ($cmd, $mark);
    my $host = $opts{host};
    $cmd = "fcadmin hub pctrl -w $opts{wwn} -p $opts{port} -v $opts{value} -l $opts{loop}";
    if($opts{ems_verify} eq "yes") {
        $mark = ems_log_get_mark(host=>$host);
    }
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);


    if($output =~ /Usage/) {
        croak $output;
    }
    if($opts{ems_verify} eq "yes") {
        sleep 30;
        my $event_by_type = ems_log_dump_by_type(host=>$host, timestamp=>$mark);
        my $event = $event_by_type->{"shelf.test.in.progress"};
        if ($event eq "") {
            croak "Control request was not issued";
        }
    }

    return 1;
}

=head2 fcadmin_bridge_cmd_cemi

        my $ret = fcadmin_bridge_cmd_cemi(host=>$host, disk_id=>$disk_id, action=>'pup',
                                                conntyp=>#rsh, timeout=>100);

This routine returns 1 on success or croaks on failure.  It calls
fcadmin bridge cmd_cemi -d <ha.id> -c "<action>"  where action = pdown, pup, or pcycle

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item disk_id

(Required) : Device id of disk to be powered

=item action

(Required) : command string to be issued

=item conntype

(Optional) : Connection type for filer. Default : #rsh

=item timeout

(Optional) : Timeout value for the say. Default : 30

=back

=cut

sub fcadmin_bridge_cmd_cemi(%) {
        my (%opts) = validate(@_, {
                                host => { type=>OBJECT, isa=>'Hostrec' },
                                disk_id => { type=>SCALAR, regex => qr/^(\w+\.\w+)$/},
                                action => { type=>SCALAR, regex => qr/^(pup|pdown|pcycle)$/},
                                conntype => { type=>SCALAR,default=>"#rsh",optional=>1 },
                                timeout => { type=>SCALAR, default=>30, optional=>1 },
                                filermode=>{type=>SCALAR,default=>"normal",optional=>1},
                                }
                        );

        # no zapi so make connection
        my $cmd = "priv set -q test; fcadmin bridge cmd_cemi -d $opts{disk_id} -c $opts{action}";
        my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if ($output =~ /fcadmin bridge: failed|syntax error:/) {
                croak $output;
        }

        return 1;
}


=head2 fcadmin_inject

    my $ret = fcadmin_inject(host => $host, adapter_name => $adapter_name,timeout => $timeout,command=>"abort",period=>$period);

This routine injects various sasadmin inject errors for a specific period of time
It returns 1 on success and croaks on failure.

The following named arguments are accepted:

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Optional) : Specific channel for which the information is
 required. This is optional if device_name is defined otherwise it's a required option.

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item device_name

(Optional) : Device name on which the errors are to be injected. This is optional if adapter_name is defined otherwise it's a required option.

=item command

(Required) :  Command options to inject. Options are : "dev_reset", "lip", "adapter_reset", "abort", "timeout", "show", "disable".

=item period

(Optional) : Period how long we want to run the error injections. Default is 300 secs.

=back



=cut

sub fcadmin_inject($@) {
    my (%opts) = validate(@_,{
            host => {type=>OBJECT, isa=>'Hostrec'},
            adapter_name => {type=>SCALAR,default=>undef,optional=>1},
            device_name => {type=>SCALAR,default=>undef,optional=>1},
            period => {type=>SCALAR,default=>300,optional=>1},
            command => {type=>SCALAR},
            timeout => {type=>SCALAR,default=>300,optional=>1},
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            ems_verify => {type=>SCALAR,default=>"yes",optional=>1},
            }
            );
    my $output = "";
    my $adapter_name;
    my $a = "";
    my $b = "";
    my $mark;
    my $cmd_option;
    my $cmd1;
    my $cmd;
    my $MAINT_PROMPT = "\\*\>";

    $cmd1 = "fcadmin inject";
    if($opts{command} =~ /^dev_reset$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 dev_reset enable -t $opts{period} -r -c $opts{adapter_name}";
        }
        elsif(defined($opts{device_name})){
            $cmd = "$cmd1 dev_reset enable -t $opts{period} -d $opts{device_name}";
        }
        else{
            logresult('script', "adapter_name or device_name are the required parameters");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

        if($output =~ /adapter|failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;
    }
    elsif($opts{command} =~ /^abort$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 abort enable -t $opts{period} -r -c $opts{adapter_name}";
        }
        elsif(defined($opts{device_name})){
            $cmd = "$cmd1 abort enable -t $opts{period} -d $opts{device_name}";
        }
        else{
            logresult('script', "adapter_name or device_name are the required parameters");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

        if($output =~ /failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;
    }
    elsif($opts{command} =~ /^timeout$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 timeout enable -t $opts{period} -r -c $opts{adapter_name}";
        }
        elsif(defined($opts{device_name})){
            $cmd = "$cmd1 timeout enable -t $opts{period} -d $opts{device_name}";
        }
        else{
            logresult('script', "adapter_name or device_name are the required parameters");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;

    }
    elsif($opts{command} =~ /^adapter_reset$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 adapter_reset enable -t $opts{period}  -c $opts{adapter_name}";
        }
        else{
            logresult('script', "adapter_name is the required parameter");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;
    }
    elsif($opts{command} =~ /^lip$/){
        if(defined($opts{adapter_name})){
            $cmd = "$cmd1 lip enable -t $opts{period}  -c $opts{adapter_name}";
        }
        else{
            logresult('script', "adapter_name is the required parameter");
        }
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

        if($output =~ /failed|No such adapter|Usage/){
            croak $output;
        }
        return 1;

    }

    elsif($opts{command} =~ /^disable$/){
        $cmd = "$cmd1 disable";

        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if($output =~ /failed/){
            croak $output;
        }
        return 1;

    }
    elsif($opts{command} =~ /^show$/){
        $cmd = "$cmd1 show";
        $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

        if($output =~ /failed|No such adapter|Usage/){
            croak $output;
        }
        return $output;

    }
    else{
        logresult('script', "Invalid command:  %opts{command}");
    }

}




# self test routine for this library
# This routine can be invoked in the following manner
#
# ntest PKGNAME=Storage::Fcadmin util/test_pkg "FUNCTION_NAME=fcadmin_link_stats"
sub testme {

    package main;
    our ($FILER);
    our ($FUNCTION_NAME, $FILERMODE, $ADP_OR_DEV_NAME, $DEVICE);
    package Storage::Fcadmin;
    param('FILER',       '-mesg',    "Filer to test");
    param('FUNCTION_NAME',       '-default',    "all");
    param('FILERMODE',       '-default',    "normal");
    param('ADP_OR_DEV_NAME',    '-default', undef);

    logcomment("Testing on FILER $FILER");
    logcomment("Testing $FUNCTION_NAME");

    my $host = host($FILER);

    my $ret;

    if(($FUNCTION_NAME eq "fcadmin_link_stats") || ($FUNCTION_NAME eq "all")) {
        #Testing with default rsh connection
        eval {
            $ret = fcadmin_link_stats(host=>$host, filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        #Testing with console connection
        eval {
            $ret = fcadmin_link_stats(host=>$host, conntype=>"#console", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        # test for nonexistant channel
        eval {
            fcadmin_link_stats(host=>$host, channel_name=>50, filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin link_stats: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_fcal_stats") || ($FUNCTION_NAME eq "all")) {
        #Testing with default rsh connection
        eval {
            $ret = fcadmin_fcal_stats(host=>$host, filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        #Testing with console connection
        eval {
            $ret = fcadmin_fcal_stats(host=>$host, conntype=>"#console", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        # test for nonexistant channel
        eval {
            fcadmin_fcal_stats(host=>$host, channel_name=>50, filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin fcal_stats: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_link_state") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_link_state(host=>$host, conntype=>"#console", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper($ret);
        }

        # test for nonexistant channel
        eval {
            fcadmin_link_state(host=>$host, adapter_name=>50, filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin link_state: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_device_map") || ($FUNCTION_NAME eq "all")) {
        #Testing with default rsh connection
        eval {
            $ret = fcadmin_device_map(host=>$host, filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        #Testing with console connection
        eval {
            $ret = fcadmin_device_map(host=>$host, conntype=>"#console", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        # test for nonexistant channel
        eval {
            fcadmin_device_map(host=>$host, channel_name=>50, filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin device_map: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_channels") || ($FUNCTION_NAME eq "all")) {
        #Testing with default rsh connection
        eval {
            $ret = fcadmin_channels(host=>$host, filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        #Testing with console connection
        eval {
            $ret = fcadmin_channels(host=>$host, conntype=>"#console", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            print Dumper(%$ret);
        }

        # test for nonexistant channel
        eval {
            fcadmin_channels(host=>$host, channel_name=>50, filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin channels: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_offline") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_offline(host=>$host, conntype=>"#console", adapter_name=>$ADP_OR_DEV_NAME, timeout=>1200, ems_verify=>"yes", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            logresult('pass', "Adapter successfully offlined");
        }

        # test for nonexistant adapter
        eval {
            fcadmin_offline(host=>$host, adapter_name=>50, ems_verify=>"no", filermode=>$FILERMODE, timeout=>1200);
        };
        print "\nOutput : $@";
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin offline: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_online") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_online(host=>$host, conntype=>"#console", adapter_name=>$ADP_OR_DEV_NAME, timeout=>1200, ems_verify=>"yes", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            logresult('pass', "Adapter successfully offlined");
        }

        # test for nonexistant adapter
        eval {
            fcadmin_online(host=>$host, adapter_name=>50, ems_verify=>"no", filermode=>$FILERMODE, timeout=>1200);
        };
        print "\nOutput : $@";
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin online: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_reset_adapter") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_reset_adapter(host=>$host, conntype=>"#console", adapter_name=>$ADP_OR_DEV_NAME, timeout=>1200, ems_verify=>"yes", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            logresult('pass', "Adapter successfully reset");
        }

        # test for nonexistant adapter
        eval {
            fcadmin_reset_adapter(host=>$host, adapter_name=>50, ems_verify=>"no", filermode=>$FILERMODE, timeout=>1200);
        };
        print "\nOutput : $@";
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin reset_adapter: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_reset_dev") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_reset_dev(host=>$host, conntype=>"#console", adp_or_dev_name=>$ADP_OR_DEV_NAME, timeout=>1200, ems_verify=>"yes", filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            logresult('pass', "Adapter/Device successfully reset");
        }

        # test for nonexistant adapter
        eval {
            fcadmin_reset_dev(host=>$host, adp_or_dev_name=>50, ems_verify=>"no", filermode=>$FILERMODE, timeout=>1200);
        };
        print "\nOutput : $@";
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin reset_dev: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_rescan") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_rescan(host=>$host, conntype=>"#console", adapter_name=>$ADP_OR_DEV_NAME, timeout=>1200, filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            logresult('pass', "Adapter successfully rescanned");
        }

        # test for nonexistant adapter
        eval {
            fcadmin_rescan(host=>$host, adapter_name=>50, filermode=>$FILERMODE, timeout=>300);
        };
        print "\nOutput : $@";
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin rescan: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_lip") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_lip(host=>$host, conntype=>"#console", adapter_name=>$ADP_OR_DEV_NAME, timeout=>1200, filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            logresult('pass', "LIP successfully issued on adapter");
        }

        # test for nonexistant adapter
        eval {
            fcadmin_lip(host=>$host, adapter_name=>50, filermode=>$FILERMODE, timeout=>300);
        };
        print "\nOutput : $@";
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /fcadmin lip: No such channel/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_hub_pctrl") || ($FUNCTION_NAME eq "all")) {
        #Testing with console connection
        eval {
            $ret = fcadmin_hub_pctrl(host=>$host, conntype=>"#console", wwn=>"50:05:0c:c0:02:10:8d:4b", port=>"4", value=>"0x01", loop=>"A", timeout=>1200, filermode=>$FILERMODE);
        };
        if ($@) {
            logresult('fail', "$@");
        }
        else {
            logresult('pass', "Control request successfully issued");
        }

        # test for nonexistant adapter
        eval {
            $ret = fcadmin_hub_pctrl(host=>$host, conntype=>"#console", wwn=>"50:05:0c:c0:02:10:8d:4b", port=>"4", value=>"0x01", loop=>"C", timeout=>1200, filermode=>$FILERMODE);
        };
        print "\nOutput : $@";
        if (!$@) {
            logresult('fail',"Expected the routine to fail");
        }
        elsif ($@ !~ /Usage/) {
            logresult('fail',"Did not get expected failure message");
        }
        else {
            logresult('pass', "Got expected result");
        }
    }

    if(($FUNCTION_NAME eq "fcadmin_bridge_cmd_cemi") || ($FUNCTION_NAME eq "all")) {
                # test for missing required params action and disk_id
                eval {
                    $ret = fcadmin_bridge_cmd_cemi(host=>$host);
                };
                if (!$@) {
                    print Dumper(%$ret);
                    logresult('fail', "Expected the routine to fail");
                }

                # test for malformed disk_id
                eval {
                    fcadmin_bridge_cmd_cemi(host=>$host, disk_id=>50, action=>"pup");
                };
                if (!$@) {
                    logresult('fail',"Expected the routine to fail");
                }

                # test for as yet unsupported action
                eval {
                    fcadmin_bridge_cmd_cemi(host=>$host, disk_id=>"0a.16", action=>"pstatus");
                };
                if (!$@) {
                    logresult('fail',"Expected the routine to fail");
                } else {
                    logresult('pass', "Got expected result");
                }
        }

}

=head1 SEE ALSO

    Where additional information can be found.

=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = fcadmin

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    Haripriya (hp@netapp.com), dl-storsub-qa@netapp.com
    $Id: //depot/prod/test/Rsierranevada/lib/Storage/Fcadmin.pm#5 $
    Last revised $Date: 2013/08/26 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;
