# this is basically -*- perl -*-
#!/usr/local/bin/perl

# TODO: 1 - cleanup the use of variables, esp. globals
#       2 - add more comments and a POD with examples
# 
## This library assumes you used gx_boot & deux_init to install and configure
## your system. If you did not you can use max_aggrs to create aggregates for
## your test to use. If you have no striped aggregates you can force one to be
## created by specifying VOLUME_CLASS=striped and 
## MAX_AGGRS=number_of_non_root_aggrs_that_already_exist+1

#package Storage::HA::HA_FIT_Utils;  commented out 08/29/2008 -tpho

# Compiler directives.
use strict;

use vars qw(@ISA @EXPORT);
require Exporter;
@ISA = qw(Exporter);
@EXPORT = qw(
             ha_fit_setup 
             ha_fit_cleanup 
             reset_volumes 
             reset_aggregates 
             reset_vservers
             reestablish_connection
             get_port_info
             sfo_enable
             create_failover_rules
             data_vif
             takeover_option_enable
             ha_fiji
             revert_vif
             revert_lif
             startTraffic
             licenses_add
             get_build_root
             );
   
# Lib location
use Data::Dumper;

# Module import.
use POSIX qw(strftime); 
use TCD;
use Server;
use TracePkg;
use Client;
use Tharn;
use Options;

# Global Variables.
use vars qw($Test 
	    $Server
	    $FILERA
	    $FILERB
	    @FILERS
	    $_filers
	    $Nodes
	    $Vol 
	    $Aggr
	    @Aggregates
	    @DeleteAggr
	    $delay
	    $pause
	    $Suffix
	    $_aggregates
	    $_volumes
	    @Vservers
	    $Vserver
	    $_vservers
	    @Voltypes
	    $VolType 
	    $_voltypes
	    $vols_per_disk
	    @vols_before_reboot
	    $_volsize
	    @Volumes
	    $CLIENT1
	    $Client
	    $MntObj
	    $Share
	    $Data_IP
	    $reboot   
	    %Aggregate_Hash 
    	    $Raid
    	    $RaidType
    	    $Traffic
    	    $Traffic_Gen
    	    @Clients
    	    $_per_test
    	    $_clients
    	    @Tasks
	    );        

use vars qw($USE_EXISTING
	    $USE_EXISTING_VS
	    $USE_EXISTING_AGGRS
	    $USE_EXISTING_VOLS 
	    @Aggregate_Data
	    @Vserver_Data
	    @Volume_Data
	    @Disk_Data
	    $Vserver_Count
	    $Aggregate_Count
	    $Volume_Count
	    $Disk_Count
	    $aggrs_per_node
	    $vols_per_aggr
	    $create_volumes
	    $create_vservers
	    $create_aggregates
	    $vols_per_node
	    $vols_remainder
	    $Root_Vol
	    $Mod
	    $aggrs_remainder
	    $VserverName
	    $_diskcount
	    );        

my $Trace = new TracePkg;

# Find the name of the script running this code
my $Testname = $Tharn::Scriptspec;
$Testname =~ s/.+\///; # just get the name of the test
$Testname =~ s/\.thpl//; # strip off the .thpl extension

my $Options = new Options();
$Options->parameters(definitions=>
        [
         {
          name => "MAX_VS",
          description => "Maximum number of vservers to use during the test",
          default => "1",
         },
         {
          name => "MAX_AGGRS",
          description => "Maximum number of aggregates to use during the test",
          default => "1",
         },
         {
          name => "MAX_VOLS",
          description => "Maximum number of volumes to use during the test",
          default => "1",
         },
         {	# This variable is initially defined in Volume.pm
          name => "VOL_SIZE",
          description => "Volume size for volumes to be created in GB",
          spec => 'e',
          default => 4,
         },
         {
          name => "SUFFIX",
          description => "Suffix to be used when creating vservers, aggregates or volumes",
          default => "HA",
         },
         {
          name => "RAIDTYPE",
          description => "Raid type to be used when creating aggregates",
          valid_values => [qw(raid4 raid_dp)],
          default => "raid4",
         },
         {
          name => "DISKCOUNT",
          description => "Disk count to be used when creating aggregates",
          default => "2",
         },
         {
          name => "TRAFFIC_GEN",
          description => "Indicates which traffic generation to use",
          valid_values => [qw(dt hammer sio)],
          default => "dt",
         },
         {
          name => "NUM_CLIENTS",
          description => "Indicates number of clients to use for traffic generation",
          default => "1",
         },
         {	# This variable is initially defined in Common.pm 
          name => "CLIENT",
          description => "List of clients to use for traffic generation",
          spec => 's@',
         },
         {	# This variable is initially defined in Volume.pm 
          name => "VOLUME_CLASS",
          description => "List of volume types to be used when creating volumes",
          valid_values => [qw(flex striped acro)],
          default => "flex",
          spec => 's@',
         },
         {
          name => "USE_EXISTING",
          description => "Use all existing vservers, aggregates and volumes",
          valid_values => [qw(true false)],
          default => "false",
         },
         {
          name => "USE_EXISTING_VS",
          description => "Use all existing vservers",
          valid_values => [qw(true false)],
          default => "false",
         },
         {
          name => "USE_EXISTING_AGGR",
          description => "Use all existing aggregates",
          valid_values => [qw(true false)],
          default => "false",
         },
         {
          name => "USE_EXISTING_VOLS",
          description => "Use all existing volumes",
          valid_values => [qw(true false)],
          default => "false",
         },
         {
          name => "PAUSE",
          description => "Value in seconds of Short Timeout",
          default => "30",
         },
         {
          name => "DELAY",
          description => "Value in seconds of Long Timeout",
          default => "600",
         },
        ],
       );
	          
# SETUP #############################################################
sub ha_fit_setup { 

# Currently this routine does the aggregate, vserver and volume setup 
# in the same routine. Eventually it will be in setup_aggregates,
# setup_volumes and setup_vservers, although the ha_fit_setup routine
# will still call all 3 functions.

     my ($fail_cnt,
         @VolTypes,
         @_voltypes,
         $vserver_index,
         $voltype_index,
         $volsize,
         $volumes,
         @volume,
         @aggregate,
         $filer_index,
         $Disks,
         @aggrs,
         @disklist,
         $allowable,
         $aggrs_per_filer,
         $i, $x);

    my @Volumes_Created;
    my @Aggregates_Created;
    my @Vservers_Created;

    my $Server = shift;
    my $Options = shift;
    my $options = shift;

    $fail_cnt = 0;

    # Merge the parameters passed into the complete set of Options
 
    $Trace->trace(msg=>"SETUP: determine existing topology and cleanup if necessary");

    if (defined ( $options->{ 'DEBUG' } )) { $Options->print(); }
    $options = validate_setup_parameters($options);
    if (defined ( $options->{ 'DEBUG' } )) { $Options->print(); }

    @FILERS = @{$options->{ 'NODES' }};
    
    $Trace->trace(msg=>"SETUP: determine existing topology and cleanup if necessary");
  
    if ($options->{ 'USE_EXISTING_VOLS' } eq "false") { reset_volumes($Server); }
    eval { @Volume_Data = $Server->volume_show(vsroot=>"false", "junction-active"=>"true"); }; 
    $Volume_Count = scalar(@Volume_Data);

    if ($options->{ 'USE_EXISTING_VS' } eq "false") { reset_vservers($Server); }
    eval { @Vserver_Data = $Server->vserver_show(type=>"storage"); };
    $Vserver_Count = scalar(@Vserver_Data);

    if ($options->{ 'USE_EXISTING_AGGRS' } eq "false") { reset_aggregates($Server); } 
    eval { @Aggregate_Data = $Server->storage_aggregate_show(root=>"false"); };
    $Aggregate_Count = scalar(@Aggregate_Data);

    # Determine how many disks are available as spares 
    @Disk_Data=$Server->storage_disk_show(state=>"spare");

    # Determine what size volumes to create
    $volsize = $options->{ 'VOL_SIZE' } . "G";
    $voltype_index = 0;

    @Voltypes = split(",",$options->{ 'VOLUME_CLASS' });  
    $_voltypes = scalar(@Voltypes);		

    # Determine how many aggregates to create per node. 
    # If you specify more aggregates than nodes they will get evenly distributed. 
    # If the number is not evenly divisible the extra aggrs will get created on 
    # the first node. 
    if ($Aggregate_Count < $options->{ 'MAX_AGGRS' }) {
        $create_aggregates = $options->{ 'MAX_AGGRS' } - $Aggregate_Count;
        $aggrs_per_node = int($create_aggregates / $options->{ 'NUM_FILERS' });
        $aggrs_remainder = $create_aggregates % $options->{ 'NUM_FILERS' };
    } else { 
        $create_aggregates = $aggrs_per_node = $aggrs_remainder = 0;
    } 

    $Trace->trace(msg=>"SETUP: determine what topology needs to be created");

    # Determine how many volumes to create per aggregate. If you specify more
    # aggregates than volumes we will create at least one  volume per aggregate.
    # If the number is not evenly divisible the extra vols will get created on 
    # the first aggregate.
    # Note: MAX_VOLS usually applies to the total number of volumes across all
    # aggregates. BUT, if MAX_VOLS = 1 and there is 1 existing volume on any
    # aggregate the if is true and no other volumes get created. I did this on
    # purpose, so that I didn't have to create a new volume on each aggregate
    # every time I ran a test. If you must have that number of volumes per 
    # aggregate then make sure you have USE_EXISTING_VOLS = false, which will 
    # delete any extra volumes it finds.
        
    if ($Volume_Count < $options->{ 'MAX_VOLS' }) {
        $create_volumes = $options->{ 'MAX_VOLS' } - $Volume_Count; 
        $vols_per_aggr = int($create_volumes / $options->{ 'MAX_AGGRS' });
        $vols_remainder = $create_volumes % $options->{ 'MAX_AGGRS' };
        if ( $vols_per_aggr == 0 ) { $vols_per_aggr = 1; $vols_remainder = 0; }
    } else { 
        $create_volumes = $vols_per_aggr = $vols_remainder = 0;
    } 

    # Determine how many vservers to create. 
    if ($Vserver_Count < $options->{ 'MAX_VS' }) {
        $create_vservers   = $options->{ 'MAX_VS' } - $Vserver_Count; 
    } else { 
        $create_vservers = 0;
    } 

    # Determine how many disks to allocate based on Raid Type
    $RaidType = $options->{ 'RAIDTYPE' };
    if ( $RaidType eq "raid4" ) {
	$Raid = $options->{ 'DISKCOUNT' };
    } else {
	$Raid = $options->{ 'DISKCOUNT' } + 1;
    }  

    # Create aggregates and volumes ONLY IF they do not exist. 
    # The default is to create flex aggregates and put them all 
    # on node one, but based on the options passed the volume 
    # type and node can be changed for every aggregate create.
    # Also, for Coral the number of disks is per node. 

    $Trace->trace(msg=>"SETUP: create necessary aggregates for each node");

    # For each node to be used  
    for ($filer_index=0; $filer_index < $options->{ 'NUM_FILERS' }; $filer_index++) {

        $Trace->trace(msg=>"For Node: " . @FILERS[$filer_index]);
        # As noted above, if the numebr of aggregates to be created is 
        # not evenly divisible create the extra on the first node.
        if ($filer_index == 0) { 
        	$create_aggregates = $aggrs_per_node + $aggrs_remainder; 
        } else {
        	$create_aggregates = $aggrs_per_node; 
	}        
        # For each aggregate being created on that node 
        for ($i=0; $i<$create_aggregates; $i++) {
    
    	    $Aggr    = @FILERS[$filer_index] . "_" . $options->{ 'SUFFIX' } . "_" . $i;
    	    $Aggr    =~ tr/-/_/;   # Convert - to _

            $Trace->trace(msg=>"Creating Aggregate: " . $Aggr);

            # Keep cycling through the voltype list
            $voltype_index  = ($voltype_index %= $_voltypes);
            $VolType=@Voltypes[$voltype_index];
            $voltype_index++;

    	    # if doing Coral the default values are a little different
    	    if ( ($VolType eq "striped" ) || ( $VolType eq "acro") ) {
		$Disks = $Raid * scalar(@FILERS);
         	$Nodes=join(',',@FILERS);
    	    } else {
		$Disks = $Raid;
         	$Nodes=@FILERS[$filer_index];
    	    }  
	    $Trace->trace(msg=>"node $Nodes aggregate $Aggr voltype $VolType disks $Disks");

	    $Trace->trace(msg=>"Check to see if aggregate $Aggr already exists");
    	    eval {
		$Server->storage_aggregate_show(aggregate=>$Aggr);
    	    };

	    if ($@) {	  
                @Disk_Data=$Server->storage_disk_show(state=>"spare", owner=>$Nodes);
                $Disk_Count = scalar(@Disk_Data);
	        if ($Disks <= $Disk_Count) {	  
		    # Only create the aggregate if it does not already exist
		    # and only then if there are enough disks to do it.
    	
	            $Trace->trace(msg=>"Creating $VolType aggregate $Aggr on $Nodes using $Disks disks");
                    eval {
	  		$Server->storage_aggregate_new(aggregate=>$Aggr,
							diskcount=>$Disks,
							nodes=>$Nodes,
							"volume-style"=>$VolType,
							-Timeout=>$delay,
							raidtype=>$RaidType,
							-Verification=>"");
	 	    }; 
	 	    if ($@) {
		        $Trace->comment(msg=>"ERROR: aggregate create Failed: $@");
       		        return $TCD::FAIL;
	     	    }
                    eval {
	  		$Server->verify_aggregate_state(aggregate=>$Aggr,
							nodes=>$Nodes,
							"wait_for"=>$delay,
							state=>"online");
	 	    }; 
	 	    if ($@) {
		        $Trace->comment(msg=>"ERROR: verify aggregate state Failed: $@");
       		        return $TCD::FAIL;
	     	    } else {
    		        # need to keep track of what you create
        	        push @Aggregates_Created, $Aggr;
	            } 
		} else {
		    $Trace->trace(msg=>"WARN: not enough disks available to create $Aggr. Found $Disk_Count. Needed $Disks.");
		}
	    } else {
	        # If it already exists make sure it is online
                eval {
	  	    $Server->verify_aggregate_state(aggregate=>$Aggr,
							state=>"online");
	        }; 
	        if ($@) {
		    $Trace->comment(msg=>"ERROR: verify aggregate state Failed: $@");
       		    return $TCD::FAIL;
    	        }
	    }
        }
    }
    
    # At this point all the aggregates have been created, so
    # update the array of hashes with the @Aggregate_Data.
    @Aggregate_Data = $Server->storage_aggregate_show(root=>"false");
    @{$options->{AGGREGATES}} = @Aggregate_Data;

    $Trace->trace(msg=>"SETUP: create necessary vservers");

    # Now determine if you need to create vservers
    for ($x=0;$x<$create_vservers;$x++) {
  	  
  	$VserverName  = "vserver_". ($x+1);
  	$Root_Vol = "root_" . $VserverName;

        my $y=0;
  	until ($Aggregate_Data[$y]{'volume-style'} eq "flex") { $y++; }
  	$Aggr = $Aggregate_Data[$y]{'aggregate'}; # Basically select a random aggregate
  	 
        eval {
            $Trace->trace(msg=>"Creating vserver $Vserver on aggregate $Aggr");
            $Server->vserver_new(vserver=>$VserverName,
		    		rootvolume=>$Root_Vol,
		    		aggregate=>$Aggr,
		    		"ns-switch"=>"file",
		    		"rootvolume-security-style"=>"mixed");
    	    };
    	    if ($@) {
        	$Trace->comment(msg=>"ERROR: Vserver Create Failed: $@");
       		return $TCD::FAIL;
    	    } else {
    		    # need to keep track of what you create
        	    push @Vservers_Created, $VserverName;
	    }  
    }

    # At this point all the vservers have been created, so
    # update the array of hashes with the @Vserver_Data.
    @Vserver_Data = $Server->vserver_show(type=>"cluster");
    @{$options->{VSERVERS}} = @Vserver_Data;
    $Mod = ($options->{ 'MAX_AGGRS' } % $options->{ 'MAX_VS' });

    $Trace->trace(msg=>"SETUP: create necessary volumes");
    
    for my $x ( 0 .. $#Aggregate_Data ) {

  	$Aggr = $Aggregate_Data[$x]{'aggregate'}; 
  	$Vserver = $Vserver_Data[$Mod]{'vserver'}; 

        for (my $y=0;$y<$vols_per_aggr;$y++) {

    		$Vol    = $Aggr . "_vol". $y;
         	eval {
        	    $Trace->trace(msg=>"Creating volume $Vol on aggregate $Aggr with vserver $Vserver");
        	    $Server->volume_new(vserver=>$Vserver,
		    				volume=>$Vol,
		    				aggregate=>$Aggr,
		    				"junction-path"=>"/$Vol",
		    				"security-style"=>"mixed",
		    				size=>$volsize);
    		};
    		if ($@) {
    		   if ($@ =~ "Insufficient space") {
        		    $Trace->trace(msg=>"WARNING: Volume out of space.");
        		    # Cause processing of the aggregate to be complete
        		    $y = $vols_per_aggr;
		   } elsif ($@ =~ "duplicate key") {        
        		    $Trace->trace(msg=>"WARNING: Volume already exists.");
       	 	   } else {
        		    $Trace->comment(msg=>"ERROR: Error creating volume: $@");
       			    return $TCD::FAIL;
    		   }
    		} else {
    		    # need to keep track of what you create
        	    push @Volumes_Created, $Vol;
		} 
        } # End volume creation for this aggregate
    } 

    # At this point all the volumes have been created, so update the
    # array of hashes with the @Volume_Data. Need to eval, as if
    # no volumes but root_vs0 exist it returns error. Also, only
    # count volumes that have junction paths.
    eval { 
        @Volume_Data = $Server->volume_show(vsroot=>"false",
        				    "junction-active"=>"true");
    };
    @{$options->{VOLUMES}} = @Volume_Data;

    # Save the list of objects you created so you can clean up correctly,
    # since the complete list may include objects that already existed.
    @{$options->{CREATED_VOLUMES}}    = @Volumes_Created;
    @{$options->{CREATED_AGGREGATES}} = @Aggregates_Created;
    @{$options->{CREATED_VSERVERS}}   = @Vservers_Created;

    $Trace->trace(msg=>"SETUP: required topology created");

   return ($fail_cnt == 0) ? $TCD::PASS : $TCD::FAIL;

}

# CLEANUP ###########################################################
sub ha_fit_cleanup { 

    my $fail_cnt = 0;

    my $Server = shift;
    my $Options = shift;
    my $options = shift;
   
    # The order of deletion here is important. It must be:
    # 1 - non-vsroot volumes
    # 2 - vsroot volumes
    # 3 - vservers
    # 4 - aggregates

    # Save the list of objects you created so you can clean up correctly,
    # since the complete list may include objects that already existed.

    my @Volumes_Created    = @{$options->{CREATED_VOLUMES}};
    my @Vservers_Created   = @{$options->{CREATED_VSERVERS}};
    my @Aggregates_Created = @{$options->{CREATED_AGGREGATES}};
   
    if (scalar(@Volumes_Created) > 0) {
      for my $x ( 0 .. $#Volumes_Created ) {
 	  # $Test->nlog ("Delete Volume: @Volumes_Created[$x]");
 	  eval {
 	      $Server->volume_delete(volume=>$Volumes_Created[$x],
 	    	    		       vserver=>"*",
 	    	    		       foreground=>"true");
 	  };
 	  if ($@) {
	    $Trace->trace(msg=>"ERROR: volume delete Failed: $@");
	    $fail_cnt++;
    } }   }

    if (scalar(@Vservers_Created) > 0) {
      for my $x ( 0 .. $#Vservers_Created ) {
 	  $Trace->trace(msg=>"Delete Vserver: $Vservers_Created[$x]");
	  eval {
 	      $Server->vserver_delete(vserver=>$Vservers_Created[$x]);
 	  };
 	  if ($@) {
	      $Trace->trace(msg=>"ERROR: vserver delete Failed: $@");
	      $fail_cnt++;
    } }   }

    if (scalar(@Aggregates_Created) > 0) {
      for my $x ( 0 .. $#Aggregates_Created ) {
 	  $Trace->trace(msg=>"Delete Aggregate: $Aggregates_Created[$x]");
 	  eval {
 	      $Server->storage_aggregate_delete(aggregate=>$Aggregates_Created[$x]);
 	  };
 	  if ($@) {
	      $Trace->trace(msg=>"ERROR: aggregate delete Failed: $@");
	      $fail_cnt++;
    } }   }
    
    return ($fail_cnt == 0) ? $TCD::PASS : $TCD::FAIL;
}

# RESET ###########################################################
sub reset_vservers { 

    my $fail_cnt = 0;
    my $Server = shift;
    my @Vserver_Data;
   
    $Trace->trace(msg=>"RESET: reset_vservers\n");

    eval { @Vserver_Data = $Server->vserver_show(type=>"cluster"); };
    if (scalar(@Vserver_Data) > 0) {
      for my $x ( 0 .. $#Vserver_Data ) {
 	$Trace->trace(msg=>"Vserver: $Vserver_Data[$x]{'vserver'}");
 	if (($Vserver_Data[$x]{'vserver'}) ne "vs0") {
	  eval {
 	      $Server->vserver_delete(vserver=>$Vserver_Data[$x]{'vserver'});
 	  };
 	  if ($@) {
	      $Trace->comment(msg=>"ERROR: vserver delete Failed: $@");
    } } } }

    return ();
}

sub reset_aggregates { 

    my $fail_cnt = 0;
    my $Server = shift;
    my @Aggregate_Data;

    $Trace->trace(msg=>"RESET: reset_aggregates\n");

    eval { @Aggregate_Data = $Server->storage_aggregate_show(root=>"false"); };
    if (scalar(@Aggregate_Data) > 0) {
      for my $x ( 0 .. $#Aggregate_Data ) {
        if ( ($Aggregate_Data[$x]{'aggregate'} !~ "_aggr0") && 
 	         ($Aggregate_Data[$x]{'aggregate'} !~ "_aggr1") &&
 	         ($Aggregate_Data[$x]{'aggregate'} ne "stripedaggr_0") ) {
 	  $Trace->trace(msg=>"Delete Aggregate: $Aggregate_Data[$x]{'aggregate'}");
 	  eval {
 	      $Server->storage_aggregate_delete(aggregate=>$Aggregate_Data[$x]{'aggregate'});
 	  };
 	  if ($@) {
	      $Trace->trace(msg=>"ERROR: aggregate delete Failed: $@");
    } } } }

    return ();
}

sub reset_volumes { 

    my $fail_cnt = 0;
    my $Server = shift;
    my @Volume_Data;
    
    $Trace->trace(msg=>"RESET: reset_volumes\n");
      
    eval { @Volume_Data = $Server->volume_show(vsroot=>"false", "junction-active"=>"true"); }; 
    if (scalar(@Volume_Data) > 0) {
      for my $x ( 0 .. $#Volume_Data ) {
 	  # $Test->nlog ("Delete Volume: $Volume_Data[$x]{'volume'}");
 	  eval {
 	    $Server->volume_delete(volume=>$Volume_Data[$x]{'volume'},
 	    	    		       vserver=>$Volume_Data[$x]{'vserver'},
 	    	    		       foreground=>"true");
 	  };
 	  if ($@) {
	    $Trace->trace(msg=>"ERROR: volume delete Failed: $@");
    } }   }
            
    return ();
}

sub reestablish_connection {

    # There are those times when you need to re-establish the connection to the filer
    # If using the console I don't know, but you do.
	
    my $Server = shift;
    my $FILERA = shift;

    eval {
	$Server->set_execution_server(-Server => $FILERA);
	$Server->system_show(-timeout=>30);
        };
    if ($@) {
	eval {
	    $Server->handle_connections(close => [qw(console_connection shell_connection cli_connection)]);
        };
	sleep 10;
	eval {
	    $Server->handle_connections(open => [qw(console_connection shell_connection cli_connection)]);
        };
	eval {
	    $Server->set_execution_server(-Server => $FILERA);
        };
    }
    return ($@);
}

sub validate_setup_parameters {

    # This routine checks the values in the parameter hash and inserts default values
    # for fields not defined. Options does something like this and if I can figure out
    # how to make it work here I will use it instead.

    # These variables control how many vservers, aggregates and volumes to use.
    # Since deux_init always creates a vserver and a set of aggregates, the test 
    # assumes these can be used and attempts to do so. However, the root 
    # aggregate (aggr0) and vserver root volumes for each node are never used 
    # during the test, and are not counted as part of the calculations.
    #
    # For the other aggregates and volumes, whether these are created or existing 
    # ones are used is controlled by the USE_EXISTING parameters. For example, if 
    # you define MAX_AGGR=3 and USE_EXISTING_AGGRS is TRUE, the 3 existing non-root 
    # aggregates, aggr1 on both nodes and the striped aggr created by deux_init, 
    # will be used. If you define MAX_AGGR greater than 3, or there are not 3 
    # aggregates available, then they will be created based on the VOLTYPES parameter 
    # below and the disk size calculation made later on. MAX_VOLS is interpreted as 
    # total volumes distributed equally across all the aggregates.
    #
    # SPECIAL NOTE: Defining MAX_AGGRS or MAX_VOLS = 0 has a special meaning. 
    # If MAX_AGGRS=0 setup attempts to create as many aggregates as possible based 
    # on the number of disks attached, the VOLTYPES defined and the RAIDTYPE. 
    # IF MAX_VOLS=0 setup attempts to create as many volumes PER aggregate as
    # possible based on the VOL_SIZE. 
    #
    # RANDOM NOTES: The total number of volumes to be created must be evenly divisible
    # by the total number of aggregates or we will round down to make it so. Same
    # for number of aggregates divided by number of filers. 
    #
    # IMPORTANT NOTE: If you happen to specify more aggregates or volumes than can
    # be created on the filer then setup will create as many as it can and continue
    # on with the test. It will NOT FAIL because you screwed up and can't count.
  
    my $hash_ref  = shift;

    $hash_ref->{ 'MAX_VS' }    = $hash_ref->{ 'MAX_VS' } ? $hash_ref->{ 'MAX_VS' } : 1 ;	
    $hash_ref->{ 'MAX_AGGRS' } = $hash_ref->{ 'MAX_AGGRS' } ? $hash_ref->{ 'MAX_AGGRS' } : 1 ;	
    $hash_ref->{ 'MAX_VOLS' }  = $hash_ref->{ 'MAX_VOLS' } ? $hash_ref->{ 'MAX_VOLS' } : 1 ; 
    $hash_ref->{ 'VOL_SIZE' }  = $hash_ref->{ 'VOL_SIZE' } ? $hash_ref->{ 'VOL_SIZE' } : 4 ; 
    $hash_ref->{ 'SUFFIX' }    = $hash_ref->{ 'SUFFIX' } ? $hash_ref->{ 'SUFFIX' } : "HA" ;
    $hash_ref->{ 'RAIDTYPE' }  = $hash_ref->{ 'RAIDTYPE' } ? $hash_ref->{ 'RAIDTYPE' } : "raid4" ;
    $hash_ref->{ 'DISKCOUNT' } = $hash_ref->{ 'DISKCOUNT' } ? $hash_ref->{ 'DISKCOUNT' } : "2" ;

    $hash_ref->{ 'IO' }	 = $hash_ref->{ 'IO' } ? $hash_ref->{ 'IO' } : "0" ;
    $hash_ref->{ 'IO_SCRIPT' } = $hash_ref->{ 'IO_SCRIPT' } ? $hash_ref->{ 'IO_SCRIPT' } : "hammer" ;
    $hash_ref->{ 'TRAFFIC' } = $hash_ref->{ 'TRAFFIC' } ? $hash_ref->{ 'TRAFFIC' } : "true" ;
    $hash_ref->{ 'TRAFFIC_GEN' } = $hash_ref->{ 'TRAFFIC_GEN' } ? $hash_ref->{ 'TRAFFIC_GEN' } : "dt" ;
    $hash_ref->{ 'NUM_CLIENTS' } = $hash_ref->{ 'NUM_CLIENTS' } ? $hash_ref->{ 'NUM_CLIENTS' } : "1" ;

    if (defined ( $hash_ref->{ 'CLIENT' } )) {
        my $_clients = scalar(@{$hash_ref->{ 'CLIENT' }});		
        if ($_clients == 0) { $hash_ref->{ 'IO' } = "0"; $hash_ref->{ 'TRAFFIC' } = "false";}
    } 

    if (defined ( $hash_ref->{ 'VOLUME_CLASS' } )) {
        my @Voltypes = @{$hash_ref->{ 'VOLUME_CLASS' }};  
        my $_voltypes = scalar(@Voltypes);		
        if ($_voltypes == 0) { ${$hash_ref->{ 'VOLUME_CLASS' }}[0] = "flex"; }
    } else {
        ${$hash_ref->{ 'VOLUME_CLASS' }}[0] = "flex";
    }

    $hash_ref->{ 'NUM_FILERS' } = scalar(@{$hash_ref->{ 'NODES' }});

    # These variables control whether to reuse existing vservers, aggregates and volumes
    # The default is False, which causes the Setup routine to delete any extraneous
    # vservers, aggregates or volumes, in an attempt to reset the state to what it looked
    # like after deux_init was done. If set True then anything created by previous tests 
    # is not cleaned up, but used in the test instead.

    $hash_ref->{ 'USE_EXISTING' } = $hash_ref->{ 'USE_EXISTING' } ? $hash_ref->{ 'USE_EXISTING' } : "false" ; 
    if ( $hash_ref->{ 'USE_EXISTING' } eq "true" ) {
    	$hash_ref->{ 'USE_EXISTING_VS' } = $hash_ref->{ 'USE_EXISTING_AGGRS' } =  $hash_ref->{ 'USE_EXISTING_VOLS'} = "true"; 
    } else {
        $hash_ref->{ 'USE_EXISTING_VS' }    = $hash_ref->{ 'USE_EXISTING_VS' }    ? $hash_ref->{ 'USE_EXISTING_VS' }    : "false" ;  
        $hash_ref->{ 'USE_EXISTING_AGGRS' } = $hash_ref->{ 'USE_EXISTING_AGGRS' } ? $hash_ref->{ 'USE_EXISTING_AGGRS' } : "false" ; 
        $hash_ref->{ 'USE_EXISTING_VOLS'}   = $hash_ref->{ 'USE_EXISTING_VOLS' }  ? $hash_ref->{ 'USE_EXISTING_VOLS' }  : "false" ; 
    }
    
    $pause  = $hash_ref->{ 'PAUSE' } ? $hash_ref->{ 'PAUSE' } : 30 ; 
    $delay  = $hash_ref->{ 'DELAY' } ? $hash_ref->{ 'DELAY' } : 300 ;

    return ($hash_ref);
    
}

#
# This function enable options panic|reboot for a takeover if it is not yet. 
#
sub takeover_option_enable {
   my $Server = shift;
   my $Options = shift;
   my $Params = shift;
   my $opt = shift;
   my $res = "false";
   my $Output;
   my @Nodes;
   my $Hash;

   @Nodes = @{$Params->{ 'NODES' }};
   $Hash = $Server->storage_failover_show(-options,);
   for (my $i=0; $i<=(scalar(@{$Hash}) - 1); $i++) {
      if ($$Hash[$i]{$opt} eq "true") {
         $Trace->comment(msg=>"takeover $opt is already enabled");
         $res = "true";
         last;
      }
      else { ### enable it
         $Trace->comment(msg=>"takeover $opt is not enabled. Enable it.");
         $Server->storage_failover_modify(filer=> $$Hash[$i]{node},
                                           $opt => "true", );
         eval {         
              $Output = $Server->storage_failover_show(
                                           node => $Nodes[0],
                                           $opt => "true", );
              };
         if ($@ =~ /There are no entries matching your query/) {
            $Trace->fatal(msg=>"Failed to enable takeover $opt",result=>"FAIL",); 
         }
         elsif ($@) {
            $Trace->fatal(msg=>"An error occured from the query $opt",result=>"FAIL",);
         }
         else {
            $Trace->comment(msg=>"Successfully enabled takeover $opt");
            $res = "true";
         }
      }
   }

   return ($res);
}

#
# This function creates data VIFs that bound to the v-server
# for each filer, if not existed.
#

sub data_vif {
   my $Server = shift;
   my $Options = shift;
   my $Params = shift;
   my $vserver = shift;
   my $failover_policy = shift;
   my $status_admin = "up";
   my @Vif_create = ();
   my @Nodes = ();
   my %VIFS = ();
   my $Failure;
   my $Output;
   my $Hash;
   my $vif_cnt = 0;

   @Nodes = @{$Params->{ 'NODES' }};

   foreach my $filer (@Nodes) {
      my @data_ports = ();
      my @IP_address = ();
      my @data_mask = ();
      my $vif;
      my $rc = 0;
      my $ref_data_ports = \@data_ports;
      my $ref_IP_address = \@IP_address;
      my $ref_data_mask = \@data_mask;

      ### Get filer info from tharnhost file
      $rc = get_port_info($filer,$Params,$ref_data_ports,
                          $ref_IP_address,$ref_data_mask); 
      if (!$rc) {
         $Trace->comment(msg=>"Could not retrieve lifs data for $filer.");
      }

      eval {
         $Server->network_interface_show(server=>$vserver,
                                   role        => "data",
                                  'home-node'  => $filer,
	  		                             output   => \$Output,
                     	    );
            };
      if ($@ =~ /There are no entries matching your query/) {
         $Trace->comment(msg=>"No data VIFs from $filer bound to $vserver.");	
          for (my $i=0,my $j=1; $i <=(scalar(@data_ports)-1); $j++,$i++) {
             $vif = $filer."_data".$j;
             $Trace->comment(msg=>"Creating VIF $vif...");
             eval {
                  $Server->network_interface_new(server=>$vserver,
                                      vif=>$vif,
                             'home-filer'=> $filer,
                              'home-port'=>$data_ports[$i],
                                  address=>$IP_address[$i],
                                  netmask=>$data_mask[$i],
                                     role=>"data",
                        'failover-policy'=>$failover_policy,
                           'status-admin'=>$status_admin,
                                   output=>\$Output,
                             );
             };
             if ($@ =~ /ERROR: command failed: A lif with the same name already exists/) {
                $Trace->comment(msg=>"Data $vif exists. Continue");
             }
             elsif ($@) {
                $Trace->fatal(msg=>"Error creating data $vif",
                                                   result=>"FAIL",);
             }
             else {
                my $value = $vif.":".$IP_address[$i].":".$filer;
                push(@Vif_create, $value);
                $Trace->comment(msg=>"VIF $vif on filer $filer bound to $vserver now exists");
                $vif_cnt++;
             }
           }  ### for (my $i=0,$j=1; $j <= scalar(@data_ports); $j++,$i++)
       } elsif ($@) {
	    $Failure="An unexpected error occured querying for the vif\n$Output\n";
            $Trace->fatal(msg=>"An unexpected error occured querying for the vif\n$Output\n", result=>"FAIL",);
       } else {
            $Trace->comment(msg=>"There are data VIFs on $filer bound to $vserver already exists");
     }
  } ###  foreach my $filer (@Filers)

  #
  # Obtain array of data VIFs that are both bound to the filers
  # the arrays looks like nqa-f3050-7_data1:172.16.7.1:nqa-f3050-7
  #                       nqa-f3050-7_data2:172.16.7.2:nqa-f3050-7
  #                       nqa-f3050-8_data1:172.17.7.1:nqa-f3050-8
  #                       nqa-f3050-8_data2:172.17.7.2:nqa-f3050-8
  #
  if ($vif_cnt == 0) {  ### @Vif_create did not get populated
  foreach my $node (@Nodes) {
     $Hash = $Server->network_interface_show(server => $vserver,
                                        role          => "data",
                                        'home-node'  => $node,
                                        'status-oper' => "up",
                     );
     for (my $i=0; $i<=(scalar(@{$Hash})-1); $i++) {
         my $value1 = $$Hash[$i]{vif}.":".$$Hash[$i]{address};
         my $value2 = $value1.":".$$Hash[$i]{'home-node'}; 
         push(@Vif_create,$value2);
     }
    }
  }
   $Trace->comment(msg=>"vifs=@Vif_create");

   return (@Vif_create);
}

#
# This function creates failover rules if not existed.
#
sub create_failover_rules {
   my $Server = shift;
   my $Options = shift;
   my $Params = shift;
   my $vserver = shift;
   my $rulesetname = shift;
   my $status = 0;
   my $Output;
   my $ret;
   my $Hash;
   my @Nodes = ();
   my $FO_rule = undef;

   @Nodes = @{$Params->{ 'NODES' }};

   $Trace->comment(msg=> "SFO pair: $Nodes[0] $Nodes[1]");

   eval {
         $Server->protocol_exportruleset_show(vserver=>$vserver,
                                              rulesetname=>$rulesetname,
                                              output=>\$Output,);
        };

   if ($@ =~ /There are no entries matching your query/) {
     $Trace->comment(msg=>"no exportrulexist for $vserver. Creating $rulesetname");
     $Server->protocol_exportruleset_new(vserver=>$vserver,
                                     rulesetname=>$rulesetname,
                                     );
   }
   else {
      $Trace->comment(msg=>"exportrulexist for $vserver existed.");
   }

   ########################################################################
   # Ensure export rule within the ruleset. Create new export rule if needed
   ########################################################################
   eval {
       $Server->protocol_exportruleset_rule_show(vserver=> $vserver,
                                             rulesetname=> $rulesetname,
                                                    anon=> 0,
                                                protocol=> "any",
                                             clientmatch=> "0.0.0.0/0",
                                                  rorule=> "any",
                                                  rwrule=> "any",
                                               superuser=> "any",
                                                  output=>\$Output,
                          );
        };
   if ($@ =~ /There are no entries matching your query/) {
      $Trace->comment(msg=>"discovered that no rule exists in the ruleset $rulesetname for $vserver. Creating...");
      $Server->protocol_exportruleset_rule_new(vserver=> $vserver,
                                       rulesetname=> $rulesetname,
                                              anon=> 0,
                                          protocol=> "any",
                                       clientmatch=> "0.0.0.0/0",
                                            rorule=> "any",
                                            rwrule=> "any",
                                         superuser=> "any",
                                        ruleindex => 666,
                                               );
      $Trace->comment(msg=>"created rule for $rulesetname vserver $vserver");
   }

   # Ensure failover rules are setup for the VIF between the two filers.
   # Create them if necessary
   # First, get all VIFS bound to the VS

   ### Need to work here if no data VIF or the command below fail.
   ### some thing like There are no entries matching your query.
   $Hash = $Server->network_interface_show(server=> $vserver);
   $FO_rule = undef;

   for (my $i=0; $i<=(scalar(@{$Hash}) - 1); $i++) {
      my @data_ports = ();
      my @IP_address = ();
      my @data_mask = ();
      my $ref_data_ports = \@data_ports;
      my $ref_IP_address = \@IP_address;
      my $ref_data_mask = \@data_mask;
      foreach my $filer (@Nodes) {
      ### Get filer info from tharnhost file
         get_port_info($filer,$Params,$ref_data_ports,$ref_IP_address,$ref_data_mask); 
         if (@data_ports eq "" || @IP_address eq "") {
            $Trace->fatal(msg=>"Missing data from $filer tharnhost file"); 
         }
         foreach my $port (@data_ports) {
	    eval {
                 $Server->network_interface_failover_show(server => $vserver,
                                                              vif => $$Hash[$i]{vif},
                                                      filer => $filer,
                                                      port => $port,
                                                      output =>\$Output,);
	    		};
    	   if ($@ =~ /There are no entries matching your query/) {
              $Trace->comment(msg=>"Failover rule using filer $filer and port $port does not exist for $$Hash[$i]{vif}. Creating...");
  	      $FO_rule=$Server->network_interface_failover_show(server=> $vserver,
	                                                         vif=> $$Hash[$i]{vif},
                        	                                              	);
	   ########################################################################
	   # Calling scalar will return number of elements in the existing FO rule
	   # Since indexs start at 0, this number will be the priority to use 
	   # on the new command 
	   ########################################################################
	   my $priority = scalar(@{$FO_rule});
	   $Server->network_interface_failover_new(server=> $vserver,
                                                    vif=> $$Hash[$i]{vif},
                                                    priority=> $priority,
                                                    filer=> $filer,
                                                    port=> $port,
    					       	    );
            $Trace->comment(msg=>"Created failover rule at priority $priority using filer $filer, port $port, priority $priority for vif $$Hash[$i]{vif}");
       	} elsif ($@) {
            $Trace->fatal(msg=>"An unexpected error occured querying for the failover rules\n$Output\n", result=>"FAIL",);
       	} else {
       	   $Trace->comment(msg=>"A failover rule exists for $$Hash[$i]{vif} using filer $filer and port $port");
       	}
      }  ### foreach my $port (@data_ports) 
   }  ### foreach my $filer
 } ### for (my $i=0; $i<=(scalar(@{$Hash}) - 1); $i++)
}  ### end of create_failover_rules()

#
# This function collects info from the tharnhost file for a filer
#
sub get_port_info {

   my $filer = shift;
   my $Params = shift;
   my $ref_data_ports = shift;
   my $ref_IP_address = shift;
   my $ref_data_mask = shift;
   my $Hash;
   my $ip1;
   my $ip2;
   my $port1;
   my $port2;
   my $mask1;
   my $mask2;
   
   $Hash  = $Params->{$filer} = host($filer);
   $port1 = $Params->{$filer}->{DATA_PORT1};  ### e1a
   $port2 = $Params->{$filer}->{DATA_PORT2};  ### e1b
   $ip1   = $Params->{$filer}->{DATA_IP1};    ### 172.16.12.1
   $ip2   = $Params->{$filer}->{DATA_IP2};    ### 172.16.12.2
   $mask1   = $Params->{$filer}->{DATA_MASK1};    ### 255.255.0.0
   $mask2   = $Params->{$filer}->{DATA_MASK2};    ### 255.255.0.0 

   if ($port1 ne "") {
      push (@${ref_data_ports}, $port1);
   }
   else {
      $Trace->comment(msg=>"could not retrieve value for data port DATA_PORT1");
      return 1;
   }
   if ($port2 ne "") {
      push (@${ref_data_ports}, $port2);
   }
   else {
      $Trace->comment(msg=>"could not retrieve value for data port DATA_PORT2");
      return 1;
   }
   if ($ip1 ne "") {
      push (@${ref_IP_address}, $ip1);
   }
   else {
      $Trace->comment(msg=>"could not retrieve value for IP DATA_IP1");
      return 1;
   }
   if ($ip2 ne "") {
      push (@${ref_IP_address}, $ip2);
   }
   else {
      $Trace->comment(msg=>"could not retrieve value for IP DATA_IP2");
      return 1;
   }
   if ($mask1 ne "") {
      push (@${ref_data_mask}, $mask1);
   }
   else {
      $Trace->comment(msg=>"could not retrieve value for DATA_MASK1");
      return 1;
   }
   if ($mask2 ne "") {
      push (@${ref_data_mask}, $mask2);
   }
   else {
      $Trace->comment(msg=>"could not retrieve value for DATA_MASK2");
      return 1;
   }

return 0;
}  ### end of get_port_info()

#
# This function will enable SFO if it's not already enable.
#
sub sfo_enable {

   my $Server = shift;
   my $Options = shift;
   my $Params = shift;
   my $res = "true";
   my $SFO_enable = 0;
   my $Hash;
   my $Failure;
   my $found = 0;
   my @Nodes = ();

   #
   # Enable SFO on 2 filers-pair if it is disabled
   #
   @Nodes = @{$Params->{ 'NODES' }};
   $Hash = $Server->storage_failover_show(-Timeout=>100, -SkipINFO=>1);
   for (my $i=0; $i<=(scalar(@{$Hash}) - 1); $i++) {
      if ( grep($_ eq $$Hash[$i]{filer},@Nodes)) {
         if ($$Hash[$i]{enabled} eq "false") {
            if (!$found) {
               $Trace->comment(msg=>"Enable SFO on $$Hash[$i]{filer} ...");
               $Server->storage_failover_modify(filer=> $$Hash[$i]{filer},
                                                     enabled => "true", );
               $SFO_enable = 1;
               $Hash = $Server->storage_failover_show(-Timeout=>100,-SkipINFO=>1);
               if ($$Hash[$i]{enabled} ne "true") {
                  $Trace->fatal(msg=>"Failed to enable SFO on $$Hash[$i]{filer}",result=>"FATAL", );
               } else {
                  $found++;
               }
         } else {  ### $$Hash[$i]{enabled} eq "true"
            if ($SFO_enable) {
               $res = "false";
               $Failure= "Enabled SFO on one filer, but not on the other";
               $Trace->fatal(msg=>"$Failure", result=>"FATAL",);
            }
            else {
               $res = "false";
               $Failure = "SFO previously enabled on one filer, not on the other";
               $Trace->fatal(msg=>"$Failure", result=>"FATAL",);
            }
         }
      } else {
          $Trace->comment(msg=>"SFO on $$Hash[$i]{filer} is already enabled");
      }
    }
  }
  return ($res);
}  ### end of sfo_enable()

# FIJI ###########################################################
sub ha_fiji { 

    # This routine handles processing standard testpoints (tp) and fiji testpoints (fiji).
    # It enables priv set test, then uses the dblade or system node run to perform
    # the desired action, usually setting a testpoint or doing a show/list. Since system
    # node run does not pass back the error code, the error checking is based on what
    # type of command it was. 
    # Also, it is designed to handle tricky or boilermaker, so there are two different
    # ways of setting privilege mode and executing a dblade command.
    
    my $fail_cnt = 0;

    my $Server = shift;
    my $Cmd = shift;
    my $Node = shift;
    my $output;
    
    my $Release = $Server->get_release(format => "cooked");
    $Trace->trace(msg=>"Setting testpoint on $Node:\n $Cmd");
       	
    if ($Release =~ /Tricky/) {
        eval { 
            $Server->dblade_set_privilege(-Server=>"$Node", level=> "test", quiet=> "off");
        };
        if ($@) {
	    $Trace->warn(msg=>"ERROR: could not set privilege mode test: $@");
       	    return  $TCD::FAIL;
       	}
        eval { 
            $output = $Server->dblade(-Server=>"$Node", cmd =>"$Cmd");
        };
        if ($@) {
	    $Trace->warn(msg=>"ERROR: Command $Cmd failed: $@");
       	    return  $TCD::FAIL;
       	}

    } else {  
        eval { 
            $Server->set(privilege => "test" ); 
        };
        if ($@) {
	    $Test->nlog("ERROR: Could not set privilege mode test: $@");
            return  $TCD::FAIL;
        }    
        eval { 
            $output = $Server->system_node_run(-Server => "$Node", node => "$Node", command => "$Cmd" );
        };
        if ($@) {
	    $Trace->warn(msg=>"ERROR: Command $Cmd failed: $@");
       	    return  $TCD::FAIL;
       	}
    }   
    # Check for setting a fiji testpoint
    if (($Cmd =~ "fiji filter") && ($output !~ "filter set")) {
	    $Trace->warn(msg=>"ERROR: set fiji testpoint failed");
       	    $fail_cnt++;
    } 
    # Check for showing fiji testpoint
    if (($Cmd =~ "fiji show") && ($output =~ "no filter set")) {
	    $Trace->warn(msg=>"ERROR: fiji testpoint not found");
       	    $fail_cnt++;
    } 
    # Check for setting a testpoint
    if ((($Cmd =~ "tp error") || ($Cmd =~ "tp panic")) && ($output =~ "not found")) {
	    $Trace->warn(msg=>"ERROR: set testpoint failed");
       	    $fail_cnt++;
    }     
    # Check for showing testpoint
    if (($Cmd =~ "tp list") && ($output !~ "enabled")) {
	    $Trace->warn(msg=>"ERROR: testpoint not found");
       	    $fail_cnt++;
    } 
   
    return ($fail_cnt == 0) ? 1 : 0; # This makes the return values TRUE and FALSE
}

sub revert_lif {
   my $Server = shift;
   my $vserver = shift;

   $Trace->comment(msg=>"Reverting lifs...");
   $Server->network_interface_revert(server => $vserver, lif => "*");

}

sub revert_vif {
   my $Server = shift;
   my $vserver = shift;
   my $node    = shift;
   my $Hash = "";
   my $Output = "";

   $Trace->comment(msg=>"Entered revert_vif(): $vserver $node");
   $Hash = $Server->network_interface_show(
                'role'        => "data",
                'server'      => $vserver,
                'home-node'   => $node,
                'status-oper' => 'up',
    );

    #
    # The network_interface_show will return 2 vifs:
    # $$Hash[0]{vif} and $$Hash[1]{vif}
    # Example: $$Hash[0]{vif} = nqa-f3050-16_data1,
    # $$Hash[1]{vif} = nqa-f3050-16_data2
    # Need to know which one not-home then revert it.
    #
    for (my $i=0; $i < 2; $i++) {
       eval {
           $Server->network_interface_show (
                    'server'      => $vserver,
                    'home-node'   => $node,
                    'vif'         => $$Hash[$i]{vif},
                    'is-home'     => 'true',
                    output        => \$Output,
            );
       };

       if ($@ && $Output =~ /no entries matching your query/) {
          $Trace->comment(msg=>"reverting vif $$Hash[$i]{vif} back home...");
          eval {
               my $output1 = $Server->shell(cmd => "ngsh -c 'revert -server $vserver -lif $$Hash[$i]{vif}'");
          };

          if ($@) {
             $Trace->fatal(msg => "Command \"revert\" returned error",
                                    Error => $@,result => "FAIL",);
          }
       }
    } ### for (my $i=0; $i < 2; $i++)
}

sub licenses_add {
   my $Server = shift;
   my $Options = shift;
   my $Params = shift;
   my $GX_Mirror = "LLQDBKPJYBFDAA";
   my $SnapMirror_DP = "FEDPEKPJYBFDAA";
   my $SnapMirror_LS = "RYAEFKPJYBFDAA";
   my $Striped_Volume = "DTYSFKPJYBFDAA";
   my $GX_CIFS = "JAMHCKPJYBFDAA";
   my @Nodes = ();

   @Nodes = @{$Params->{ 'NODES' }};

   #
   # Add 2 licenses for now.
   #
   foreach my $filer (@Nodes) {
      $Server->set_execution_server(-Server => $filer);
      my $msg = "add license on $filer: "; 
      $Trace->comment(msg=>"$msg SnapMirror_DP '$SnapMirror_DP'");
      $Server->shell(cmd => "ngsh -c 'license add $SnapMirror_DP'");
      $Trace->comment(msg=>"$msg SnapMirror_LS '$SnapMirror_LS'");
      $Server->shell(cmd => "ngsh -c 'license add $SnapMirror_LS'");
   }
   $Server->shell(cmd => "ngsh -c 'set -privilege advanced;license show'");
}

#
# This function verifies a debug build.
#
sub get_build_root {
    my $Server = shift;
    my $cmd;
    my $ret = "none";
    my $output;

    $cmd = "ident /boot/modules/maytag.ko | grep -i debug";

    eval {
        $output=$Server->shell(cmd=>$cmd);
    };
    if ($output !~ /debug\s+\$$/){   # $NtapBuildOpts: x86.debug $
        $Trace->fatal(msg=>"Filer not running debug build $output",
                    result=>"CONFIG",);
    }
    else {
       $ret = "debug";
    }

    return $ret;
}

sub startTraffic {
  my $io_tool = shift;
  my $junction_path = shift;
  my $mount_path = shift;
  my $client_name = shift;
  my $handle;
 
  my $Client = new Client(-host=>$client_name,
           -timeout=>100,
           -protocol=>"nfs"
  );
 
  if ($Client->error)
  {
      $Trace->comment(msg=>$Client->errmsg);
  }
 
  $Trace->comment(msg=>"Traffic generator: $io_tool");
  $Trace->comment(msg=>"Junctions Path: $junction_path");
  $Trace->comment(msg=>"Mount Path: $mount_path");
 
  # NFS share to mount
 
  my $mntObj = $Client->create_mnt(-share=>$junction_path,
                                -path=>$mount_path);
  if ($mntObj->error) {
      $Trace->comment(msg=>$mntObj->errmsg());
  }
 
  if ($io_tool eq "hammer") {
     eval
     {
       $handle = subtest("/usr/local/bin/sudo",
           '-client',$client_name,
           '-bg',
           '--',
           "/usr/local/bin/hammer", "$mount_path",
           "-bg","-nofilercore",
           "-runtime","600",
           "-blocksize","random",
           "-logfile","/var/hammer.log",
           "-fill","-nocleanup",
       );
     };
   }
   else {   ### dt only
     my $dt_file = $mount_path."/"."dtfile";
     eval
     {
       $handle = subtest("/usr/local/bin/sudo",
           '-client',$client_name,
           '-bg',
           '--',
           "/usr/bin/dt",
           "of=$dt_file",
           "limit=2g",
           "procs=5",
           "log=/var/dt.log",
           "runtime=100",
       );
     };
   }
 
  return $handle;
}
