#
# $Id: //depot/prod/test/Rsierranevada/lib/Storage/HDDQualUtils.pm#1 $
#
# Library of utility functions for HDD qualification tests
#
#
## @summary Library of routines to help simply HDD qualfication test development
##
## @synopsis
## use Storage::HDDQualUtils;
##
##
## @author dl-sar-qa@netapp.com, dl-step@netapp.com
## Modified failparitydrive
## 
## Burt no: 357713 
##   Title: bootitgood function in Storage/HDDQualutils Library should check for failure to boot the filer.
##   Author: maji
##   Date: 05-06-2009 
##   Description:Extra checks implemented for bootitgood function mentioned below:
##                1> Check FILER state before sending any command.
##                2> Check, expected FILER prompt before sending any command
##
##            old issues:
##                Waiting for unlimited time as expected filer prompt does not appear.
##                This situation occurs when it try to boot the filer with
##                wrong FIRMWARE/TFTP SERVER IP address/hosttype. It even sends commands
##                if the filer is not in stable state (booting with 4/4a option) or not booted properly (panic).
##         
####################################################################


package Storage::HDDQualUtils;
use strict;
require Exporter;
use vars ('@ISA','@EXPORT');
@ISA = ('Exporter');
@EXPORT = qw(
              sendpanic
		disk_list
		storage_show_disk_a
		disk_qual_start_t12
		scsi_inquiry
		verify_unique_wwn
		get_poh
		clearsavecore
		checkfc
		rundiskqual
		waitfordqend
		run_ndst
		getfcchannels
		disktest_test
		getsysstat
		waitfordisktestend
		rundisktest
		rundqscripts
		run_dq_short
		waitfordqscriptend
		printdiskqual
		bootitgood
		setup_autosupport
		send_autosupport_message
		disable_autosupport
		logitall
		getdate
		getsys
		getstorage
		failparitydrive
		countsparedrives
		waitreconstructend
		unfailthedrive
		config_volumes
		start_scrub
		waitforscrubend
		getparitydrive
		checkfcchannels
		checkvolumes
		checkdisks
		enable_nfs
		start_fw_update
		checkfwupdate
		startio
		stopio
		run_dq_perf_1
		run_dq_perf_2
		run_dq_perf_3
		run_dq_perf_4
		run_dq_perf_5
		run_dq_perf_6
		run_dq_perf_7
		run_dq_perf_8
		run_dq_perf_9
		run_dq_perf_a
		run_dq_perf_b
		run_dq_perf_c
		run_dq_perf_d
		run_dq_perf_e
		);
use Tharn;
use install;
use boot;
use NATE::BaseException;

#### Function to get the parameters of the disks in a filer through
#### command disk_list.
## @name ::disk_list
## @summary Returns the disk structure
## @synopsis
##	disk_list($filer)
##
## @arg $filer Required: Filer connnection to use
## @arg $priv Optional: Privilege level to use
sub disk_list
{
	my ($c,$priv) = @_;

	my $after;
	if (!defined($priv)) {
		(undef, undef, $after, undef) = say($c, "disk_list");
	} else {
		(undef, undef, $after, undef) = say($c, "priv set -q $priv;disk_list");
	}

	my $result = {};
	my @lines = split(/\n/, $after);
	my $n = 0;
	# skip to the start of table contents
	while ($n <= $#lines) {
		last if ($lines[$n] =~ /^-----/);
		$n++;
	}
	$n++;
	
	foreach my $line (@lines[$n..$#lines]) {
		my (
			$disk, $chan, $ven, $prodid, $rev, $serialno, $hw, $blocks, $bps
		) = (
			$line =~ /(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)/);

		$result->{$disk} = {
			'DISK' => $disk,
			'CHAN' => $chan,
			'VENDOR' => $ven,
			'PRODUCT ID' => $prodid,
			'PRODUCT_ID' => $prodid,
			'REV' => $rev,
			'SERIAL' => $serialno,
			'HW' => $hw,
			'BLOCKS' => $blocks,
			'BPS' => $bps
		}
	}

	return $result;
}

#### Function to get the structure of the disks in a filer
#### through command storage_show_disk_a.
## @name ::storage_show_disk_a
## @summary Returns the disk structure
## @synopsis
##	storage_show_disk_a($filer)
##
## @arg $filer Required: Filer connnection to use
sub storage_show_disk_a
{
	my ($c) = @_;

	my (undef, undef, $after, undef) = say($c, "storage show disk -a");

	my $result = {};
	my @lines = split(/\n/, $after);
	my $n = 0;
	my $cur_disk = "";
	foreach my $line (@lines) {
		if ($line =~ /^Disk:\s+(\S+)/) {
			$cur_disk = $1;
			$result->{$cur_disk} = {};
		}
		elsif ($line =~ /(\S+):\s+(\S+)/) {
			next if (!defined($result->{$cur_disk}));
			my $propname = $1;
			my $propval = $2;
			$result->{$cur_disk}->{$propname} = $propval;
		}
	}

	return $result;
}

#### Function to start to PRINT LOG SENSE DATA on all disks.
## @name ::disk_qual_start_t12
## @summary Returns the number of failed and passwd disk in the filer
## @synopsis
##	disk_qual_start_t12($filer,$diskstr)
##
## @arg $filer Required: Filer connnection to use
## @arg $diskstr Required: Diskid to issue the command 
sub disk_qual_start_t12
{
	my ($c, $diskstr) = @_;

	my $result = {
		'n_passed' => 0,
		'n_failed' => 0,
		'passed_disks' => [],
		'failed_disks' => [],
	};

	my $timeout = 1000;
       my (undef, undef, $after, undef) = say($c, "disk_qual start -t 12 -d $diskstr", -timeout, $timeout);

	my @lines = split(/\n/, $after);
	foreach my $line (@lines) {
		my ($disk, $msg) = ($line =~ /\[(\S+)\]: (.*)$/);
		if (defined($disk)) {
			$result->{$disk} = [] if (!defined($result->{$disk}));
			push @{$result->{$disk}}, $msg;

			if ($msg =~ /Passed/i) {
				push @{$result->{'passed_disks'}}, $disk;
				$result->{'n_passed'}++;
			}
			if ($msg =~ /Failed/i) {
				push @{$result->{'failed_disks'}}, $disk;
				$result->{'n_failed'}++;
			}
		}
	}

	return $result;
}

#### Function to inquire the device's vendor and firmware information.
## @name ::scsi_inquiry
## @summary Returns the device information
## @synopsis
##      scsi_inquiry($filer,$diskid)
##
## @arg $filer Required: Filer connnection to use
## @arg $diskstr Required: Diskid to send scsi inquiry 
sub scsi_inquiry
{
	my ($c, $disk) = @_;

	my $result = {};
	my $match = say($c, "scsi inquiry $disk",
		"type \\d+ support \\d+.*\\n"
		. "vendor.*\\n" 
		. "product.*\\n" 
		. "revision.*\\n" 
		. "vendor unique.*\\n",
		-timeout, 10);

	foreach my $line (split(/\n/, $match)) {
		if ($line =~ /type\s+(\d+)\s+support\s+(\d+)/) {
			$result->{'type'} = $1;
			$result->{'support'} = $2;
		}
		elsif ($line =~ /product\s+(\S+)/) {
			$result->{'product'} =$1;
		}
		elsif ($line =~ /revision\s+(\S+)/) {
			$result->{'revision'} =$1;
		}
		# NOTE: since "vendor" and "vendor unique" are similar, must put
		#  "vendor unique" scanning ahead of "vendor"
		elsif ($line =~ /vendor unique\s+(\S+)/) {
			$result->{'vendor_unique'} =$1;
		}
		elsif ($line =~ /vendor\s+(\S+)/) {
			$result->{'vendor'} = $1;
		}
	}

	return $result;
}

#### Function to verify whether each device has unique wwn.
## @name ::verify_unique_wwn
## @summary return 1 when the wwn of all the devices are unique 
## @synopsis
##      verify_unique_wwn($filer)
##
## @arg $filer Required: Filer connnection to use 
sub verify_unique_wwn
{
	my ($c) = @_;

	my $is_unique = 1;
	my $diskinfo = storage_show_disk_a($c);
	my %wwns = ();
	foreach my $drive (keys %$diskinfo) {
		my $wwn = $diskinfo->{$drive}->{'WWN'};
		if (defined($wwns{$wwn})) {
			my $conflictdrive = $wwns{$wwn};
			logresult("WARN", "Drive $drive has identical WWN($wwn) with drive $conflictdrive");
			$is_unique = 0;
		}
		else {
			$wwns{$wwn} = $drive;
		}
	}

	return $is_unique;
}

#### Function to start disk_qual on all the disks
## @name ::get_poh
## @summary No returns on this function
## @synopsis
##	get_poh($filer)
##
## @arg $filer Required: Filer connnection to use
sub get_poh
{
	my ($c) = @_;
	#say($c, "disk_qual remarks -d all -o 'get Log Sense data'", -prompt, '\*>');
	say($c, "disk_qual remarks -d all -o 'get Log Sense data'", -swallow);
	my $qual12_res = disk_qual_start_t12($c, "all");
}

#### Function to send panic to the filer
## @name ::sendpanic
## @summary No returns on this function
## @synopsis
##	sendpanic($filer)
##
## @arg $filer Required: Filer connnection to use
sub sendpanic
{
	my ($c) = @_;

	say($c, "priv set -q advanced; panic", "Warm reboot", -noprompt,
            -timeout, 120, -ok);
}

#### Function to clear all core on the filer
## @name ::clearsavecore
## @summary No returns on this function
## @synopsis
##	clearsavecore($filer)
##
## @arg $filer Required: Filer connnection to use
sub clearsavecore
{
	my ($c) = @_;

	say($c, "savecore -k");
	sleep 20;
}

#### Function to get the FILER system information
## @name ::checkfc
## @summary No returns on this function
## @synopsis
##	checkfc($filer)
##
## @arg $filer Required: Filer connnection to use
sub checkfc
{
	my ($c) = @_;
	say($c, "version");
	say($c, "disk_list");
	sleep 10;
	#say($c, " ");
	say($c, "fcadmin device_map");
	sleep 10;
	#say($c, " ");
	say($c, "fcadmin link_stats");
	sleep 10;
	#say($c, " ");
	say($c, "fcadmin fcal_stats");
	sleep 10;
	#say($c, " ");
}

#### Function to send panic to the filer
## @name ::rundiskqual
## @summary No returns on this function
## @synopsis
##	rundiskqual($filer,$tn,$qd,$lc,$pc,$fs,$ls,$ns,$nov)
##
## @arg $filer Required: Filer connnection to use
## @arg $tn Required: Test number
## @arg $qd Required: Number of outstanding I/O's per disk
## @arg $lc Required: Loop count
## @arg $pc Required: Pass count
## @arg $fs Required: First Sector to test
## @arg $ls Required: Last Sector to test
## @arg $ns Required: number of sectors to read/write per I/O
## @arg $nov Required : whether need to verify or not
sub rundiskqual
{
	my ($c, $tn, $qd, $lc, $pc, $fs, $ls, $ns, $nov) = @_;

	say($c, "disk_qual remarks -d all -o 'testing $tn'");
	if ($nov) {
		say($c, "disk_qual start -t $tn -q $qd -L $lc -p $pc -f $fs -l $ls -n $ns -d all -o noverify");
	}
	else {
		say($c, "disk_qual start -t $tn -q $qd -L $lc -p $pc -f $fs -l $ls -n $ns -d all");
	}
}


#### Function to wait for disk_qual finish
## @name ::waitfordqend
## @summary No returns on this function
## @synopsis
##	waitfordqend($filer)
##
## @arg $filer Required: Filer connnection to use
sub waitfordqend
{
	my ($c, $timeout) = @_;

	my $match = say($c, "", "Passed|PANIC", -nocr, -noprompt, -timeout, $timeout, 
		'-eval', 
		sub {
			say($c, "disk_qual stop -d all", -swallow);
		}
	);

	if ($match =~ /PANIC/) {
		sleep 300;
		logresult("FATAL", "Filer panic while waiting for disk_qual end.");
	}
}

#### Function to run disk self test(test number 20)
## @name ::run_ndst
## @summary No returns on this function
## @synopsis
##	run_ndst($filer)
##
## @arg $filer Required: Filer connnection to use
sub run_ndst
{
	my ($c, $opt) = @_;
	my $timeout = 60;
	my $test_ok = 1;
	
	say($c, "disk_qual remarks -d all -o 'testing NDST'");
	my (undef, undef, $after, undef) = 
		say($c, "disk_qual start -t 20 -d all -o $opt");
	if ($after =~ /NETAPP DRIVE SELF TEST (NDST) Failed/) {
		$test_ok = 0;
	}
	elsif ($after =~ /wait for .* test to finish/) {
		$test_ok = 0;
	}

	return $test_ok;
}

#### Function to clear all core on the filer
## @name ::getfcchannels
## @summary Return the channel count and channel on the filer
## @synopsis
##	getfcchannels($filer)
##
## @arg $filer Required: Filer connnection to use
sub getfcchannels
{
	my ($c) = @_;
	my (undef, undef, $after, undef) = say($c, "storage show adapter -a");
	my @lines = split(/\n/, $after);
	my $channelcount = 0;
	my $channel = [];
	foreach my $line (@lines) {
		if ($line =~ /Fibre Channel Host Adapter (\S+)/) {
			$channel->[$channelcount] = $1;
			$channelcount++;
		}
	}

	return ($channelcount, $channel);
}


#### Function to run disktest on the filer
## @name ::disktest_test
## @summary No returns on the function
## @synopsis
##	disktest_test($filer,$TS,$BS,$TM)
## @arg $filer Required: Filer connnection to use
## @arg $TS??????????????
## @arg $BS Required: max # of sectors for -T, -R, -W and -A cmds to use
## @arg $TM Required: duration in minutes (default is 5 minutes)
sub disktest_test
{
	my ($c, $TS, $BS, $TM) = @_;
	say($c, "disktest -i 30 -B -$TS -n $BS -t $TM -v -y", "Running disk test", -noprompt);
}

#### Function to get io information on the filer
## @name ::getsysstat
## @summary No returns on the function
## @synopsis
##	getsysstat($filer)
## @arg $filer Required: Filer connnection to use
sub getsysstat
{
	my ($c, $time) = @_;
	
	say($c, "sysstat 1", -blind);
	sleep $time;
	$c->send_intr();
}

#### Function to wait for disktest end
## @name ::waitfordisktestend
## @summary No returns on the function
## @synopsis
##	waitfordisktestend($filer)
## @arg $filer Required: Filer connnection to use
sub waitfordisktestend
{
	my ($c) = @_;
	my $timeout = 18050;
	logcomment("waitfordisktestend: *** Waiting for disktest end ***");
	my $match = say($c, "", "Cumulative .* Errors|PANIC", -noprompt, -nocr, -timeout, $timeout, -eval_error);
	if ($@) {
		$c->send_intr();
	}
	sleep 5;
}

#### Function to run disktest with different options
## @name ::rundisktest
## @summary No returns on the function
## @synopsis
##	rundisktest($filer)
## @arg $filer Required: Filer connnection to use
## @arg $testid Required: option for disktest
## @arg $
sub rundisktest
{
	my ($c, $testid, $channels) = @_;
	my $timeout = 18050;

	my $cmd;
	if ($testid == 1) {
		logcomment("rundisktest: **** Doing Write test");
		$cmd = "disktest -W -B -t 30 -n 1023 -v";
	}
	elsif ($testid == 2) {
		logcomment("rundisktest: **** Doing Read test");
		$cmd = "disktest -R -B -t 30 -n 1023 -v";
	}
	elsif ($testid == 3) {
		logcomment("rundisktest: **** Doing Alternating test");
		$cmd = "disktest -A -B -t 30 -n 1029 -v";
	}
	elsif ($testid == 4) {
		logcomment("rundisktest: **** Doing Alternating test with large I/O");
		$cmd = "disktest -A -B -t 30 -n 4096 -v";
	}
	elsif ($testid == 5) {
		logcomment("rundisktest: **** Doing Write Verify test");
		$cmd = "disktest -V -B -t 30 -v";
	}
	elsif ($testid == 6) {
		logcomment("rundisktest: **** Doing RCA test");
		$cmd = "disktest -T -t 60 -v " . $channels->[0];
	}
	elsif ($testid == 7 && (scalar(@$channels) > 1)) {
		logcomment("rundisktest: **** Doing RCA test");
		$cmd = "disktest -T -B -t 60 -v " . $channels->[1];
	}
	else {
		return;
	}

	say($c, $cmd, -noprompt, -prompt, "Do you wish to continue?", -timeout, 60);
	say($c, "y", "Running disk test", -noprompt, -timeout, 60);


	my $match = say($c, "", "PANIC|Cumulative .* Errors", -nocr, -noprompt, -timeout, $timeout, 
		'-eval', 
		sub {
			$c->send_intr();
		}
	);
	
	if ($match =~ /PANIC/) {
		logresult("FATAL", "Filer panic while running disktest.");
	}
}



sub rundqscripts
{
	my ($c) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'run disk_qual script'");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual runscript -d all");

	sleep $timeout;
}


sub run_dq_short
{
	my ($c) = @_;

	say($c, "disk_qual remarks -d all -o 'run short disk_qual script'");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 5 -L -1 -p 1 -f 0 -l 200000 -q 3 -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 1 -L -1 -p 1 -f 0 -l 200000 -q 3 -n 8 -d all");
	say($c, "disk_qual script2 -t 99 -L 3 -d all");
	say($c, "disk_qual script3 -t 3 -L -1 -p 1 -f 0 -l 200000 -q 4 -n 8 -d all");
	say($c, "disk_qual script4 -t 4 -L -1 -p 1 -f 0 -l 200000 -q 5 -n 8 -d all");
	say($c, "disk_qual script5 -t 2 -L -1 -p 1 -f 0 -l 200000 -q 6 -n 8 -d all");
	say($c, "disk_qual script6 -t 1 -L -1 -p 1 -f 0 -l 200000 -q 2 -n 8 -d all");
	say($c, "disk_qual script7 -t 5 -L -1 -p 1 -f 300000 -l 350000 -q 2 -n 32 -d all");
	say($c, "disk_qual script8 -t 6 -L -1 -p 1 -f 300000 -l 350000 -q 3 -n 32 -d all -o noverify");
	say($c, "disk_qual script9 -t 1 -L -1 -p 2 -f 300000 -l 350000 -q 3 -n 32 -d all");
	say($c, "disk_qual script10 -t 7 -L -1 -p 3 -f 300000 -l 350000 -q 3 -n 32 -d all -o noverify");
	say($c, "disk_qual script11 -t 8 -L -1 -p 1 -f 300000 -l 350000 -q 3 -n 64 -d all -o noverify");
	say($c, "disk_qual script12 -t 1 -L -1 -p 1 -f 300000 -l 350000 -q 3 -n 64 -d all");
	say($c, "disk_qual script13 -t 13 -L -1 -p 3 -f 300000 -l 350000 -q 3 -n 64 -d all");
	say($c, "disk_qual script14 -t 5 -L -1 -p 1 -f 500000 -l 1000000 -q 6 -n 16 -d all -o noverify");
	say($c, "disk_qual script15 -t 5 -L -1 -p 1 -f 500000 -l 1000000 -q 6 -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 3 -L -1 -p 3 -f 500000 -l 1000000 -q 6 -n 128 -d all");
	say($c, "disk_qual script17 -t 4 -L -1 -p 1 -f 500000 -l 1000000 -q 6 -n 128 -d all");
	say($c, "disk_qual script18 -t 2 -L -1 -p 1 -f 500000 -l 1000000 -q 6 -n 128 -d all");
	say($c, "disk_qual script19 -t 1 -L -1 -p 1 -f 500000 -l 1000000 -q 6 -n 128 -d all");
	say($c, "disk_qual script20 -t 12 -d all");
	say($c, "disk_qual script21 -t -1 -d all");
	say($c, "disk_qual runscript -d all");
	sleep 120;
}


sub waitfordqscriptend
{
	my ($c, $testtime) = @_;

	my $match = say($c, "", "Ending script execution|PANIC", -nocr, -noprompt, -timeout, $testtime, 
		'-eval', 
		sub {
			say($c, "disk_qual stop -d all");
		}
	);

	if ($match =~ /PANIC/) {
		sleep 300;  # allow some time for console log the panic msg
		logresult("FATAL", "Filer panic while waiting for disk_qual script end.");
	}
}


sub printdiskqual
{
	my ($c) = @_;

	say($c, "disk_qual summary -d all", -swallow);
}


sub bootitgood
{
	my ($c, $bootup, $bootmode, $numberofvols, $spares, 
		$numraidgroups, $filer, $filerip, $gateway, $netmask) = @_;

	my $filer_state = "";

	$filer_state = boot_state($c);
	if (!($filer_state =~ /^(UP|MAINT|FIRMWARE|HALTED|BOOT_MENU|CFE_BANNER|LOADER_BANNERFIRMWARE)/)) {
	    logresult('FATAL',"Not supported filer prompt. Filer is not in stable state, exiting from booting the filer");
	}

    #Restore the original boot Request if Supported prompts are available
    transit_to($c, "FIRMWARE");
    transit_to($c, "BOOT_MENU");


    $filer_state = boot_state($c);
    if (!($filer_state =~ /^(BOOT_MENU)/)) {
        logresult('FATAL',"Expected filer prompt not appreared, exiting from booting the filer");
    }

	if ($bootmode eq "1") {
		transit_to($c, "UP");
	}
	elsif ($bootmode eq "4" || $bootmode eq "4a") {
		say($c, "$bootmode", -blind);
		say($c, "", -nocr, -prompt, "Zero disks and install a new file system?", -timeout, 10);
		say($c, "y", -prompt, "This will erase all the data on the disks, are you sure?", -timeout, 10);
		my $match = say($c, "y", "Zeroing disks takes about \\d+ minutes", -noprompt, -timeout, 30);
		my $zerotime;
		if ($match =~ /about (\d+) minutes/) {
			$zerotime = $1;
		}
		$zerotime =  ($zerotime * 60) + 24000;
		say($c, "", "Ctrl-C to skip DHCP search", -nocr, -noprompt, -timeout, $zerotime);
		$c->send_intr();

		my $setup = {
			"HOSTNAME" => $filer,
			"IFC_e0" => 1,
			"IP_e0" => $filerip,
			"NETMASK_e0" => $netmask,
			"MEDIATYPE_e0" => "auto",
			"GATEWAY" => $gateway,
		};
		basic_setup($c, setup=>$setup);
		transit_to($c, "UP",, netboot => 1, tftphost_ip => '172.23.8.23');
		logcomment("bootitgood: Basic setup finished.");
	}
	elsif ($bootmode eq "5") {
		transit_to($c, "MAINT");
		$c->prompts($c->prompts(), '\*>');
	}

	$filer_state = boot_state($c);
        if (!($filer_state =~ /^(UP|MAINT)/)) {
                 logresult('FATAL',"Not able to boot the filer on expected MODE, exiting from booting the filer");
        }


	if ($bootmode eq "4" || $bootmode eq "4a" || $bootmode eq "1") {
		$c->prompts($c->prompts(), "$filer>");
		say($c, "priv set -q test", -prompt, "$filer*>");
	}
}


#sub bootitgood
#{
#	my ($c, $bootup, $bootmode, $numberofvols, $spares, 
#		$numraidgroups, $filer, $filerip, $gateway, $netmask) = @_;
#
#	my @qna = (
#		"ok " => "$bootup\r",
#		"Selection (1-5)\\?" => "$bootmode\r",
#		"designated as spares" => "$spares\r",
#		"Zero disks and install a new file system\\?" => "y\r",
#		"This will erase all the data on the disks, are you sure\\?" => "y\r",
#		"For initial volume creation , you can choose to create traditional volumes" => "y\r",
#		"file system without zeroing disks\\?" => "y\r",
#		"How many volumes\\?" => "$numberofvols\r",
#		"Number of RAID groups per volume" => "$numraidgroups\r",
#		"Ctrl-C to skip DHCP search" => "\003",
#		"Please enter the new hostname" => "$filer\r",
#		"Do you want to configure virtual network interfaces\\?" => "\r",
#		"Please enter the IP address for Network Interface e0" => "$filerip\r",
#		"Please enter the netmask for Network Interface e0" => "$netmask\r",
#		"Please enter media type for e0 {100tx-fd, tp-fd, 100tx, tp, auto}" => "\r",
#		"Please enter the IP address for Network Interface e\\?\\?" => "\r",
#		"IP address for FC Network Interface" => "\r",
#		"a partner IP address during failover?" => "\r",
#		"Would you like to continue setup through the web interface\\?" => "\r",
#		"Please enter the name or IP address of the default gateway" => "$gateway\r",
#		"Please enter the name or IP address of the administration host:" => "\r",
#		"Where is the filer located\\?" => "\r",
#		"Do you want to run DNS resolver\\?" => "\r",
#		"Do you want to run NIS client\\?" => "\r",
#		"Would you like to configure the RMC LAN interface" => "n\r",
#		"Press the return key to continue." => "\r",
#		#"$filer>" => "priv set -q test\r",
#		#"$filer\\*>" => "priv set -q test\r",
#	);
#
#
#	prompt_loop($c, \@qna, $c->{prompt}, 100, 72000);
#	say($c, "priv set -q test");
#}




sub setup_autosupport
{
	my ($c, $filer) = @_;

	logcomment("Setting up Autosupport");
	say($c, "options autosupport.enable ON", -swallow);
	say($c, "options autosupport.content minimal");
	say($c, "options autosupport.from $filer");
	say($c, "options autosupport.mailhost 10.56.10.24");
	say($c, "options autosupport.noteto jchen\@netapp.com");
	say($c, "options autosupport.support.enable OFF");
	say($c, "options autosupport.to jchen\@netapp.com");
	logcomment("Autosupport Setup Complete");
}

sub send_autosupport_message
{
	my ($c, $msg) = @_;
	logcomment("Sending Autosupport message: $msg");
	say($c, "options autosupport.doit '$msg'", -swallow);
}


sub disable_autosupport
{
	my ($c) = @_;
	logcomment("Sending Autosupport message: Test Complete");
	say($c, "options autosupport.doit 'Test Complete'", -swallow);
	sleep 10;
	say($c, "options autosupport.enable OFF", -swallow);
}


sub logitall
{
	my ($c) = @_;
	say($c, "\r", -nocr, -noprompt, -swallow);
	say($c, "wrfile /etc/syslog.conf\r", -nocr, -noprompt);
	say($c, "# \$Id: //depot/prod/test/Rsierranevada/lib/Storage/HDDQualUtils.pm#1 $\r", -nocr, -noprompt);
	say($c, "# Copyright (c) 1994-2004 Network Appliance.\r", -nocr, -noprompt);
	say($c, "# All rights reserved.\r", -nocr, -noprompt);
	say($c, "# Sample syslog.conf file.  Copy to /etc/syslog.conf to use.\r", -nocr, -noprompt);
	say($c, "# You must use TABS for separators between fields.\r", -nocr, -noprompt);
	say($c, "\r", -nocr, -noprompt);
	say($c, "# Log messages of priority info or higher to the console and to /etc/messages\r", -nocr, -noprompt);
	say($c, "*.debug 					 /dev/console\r", -nocr, -noprompt);
	say($c, "*.debug 					 /etc/messages\r", -nocr, -noprompt);
	say($c, "\r", -nocr, -noprompt);
	say($c, "# Edit and uncomment following line to log all messages of priority\r", -nocr, -noprompt);
	say($c, "# err or higher and all kernel messages to a remote host, e.g. adminhost\r", -nocr, -noprompt);
	say($c, "# *.err;kern.*                          \@adminhost\r", -nocr, -noprompt);
	$c->send_intr();
	sleep 30;
	say($c, " ", -swallow);
	#say($c, " ", -swallow);
	#say($c, " ", -swallow);
}


sub getdate
{
	my ($min, $hr, $d, $m, $y) = (localtime)[1..5];
	return ($y . $m . $d . $hr . $min);
}


sub getsys
{
	my ($c, $opt) = @_;
	say($c, "sysconfig $opt", -swallow);
	#say($c, " ");
}

sub getstorage
{
	my ($c) = @_;

	say($c, "storage show", -swallow);
	sleep 10;
	#say($c, " ");
	say($c, "storage show hub", -swallow);
	sleep 10;
	#say($c, " ");
}


sub getparitydrive
{
	my ($c) = @_;

	my @paritydrive = ();
	my $paritycount = 0;

	my (undef, undef, $after, undef) = say($c, "sysconfig -r", -swallow);

	foreach my $line (split(/\n/, $after)) {
		if ($line =~ /parity\s+(\S+)/) {
			push @paritydrive, $1;
			$paritycount++;
		}
	}
	
	logcomment("getparitydrive: SYSOP SYSOP! number of parity drive is: $paritycount");
	return $paritycount;
}

sub failparitydrive
{
	my ($c) = @_;

	my @paritydrive = ();
	my $paritycount = 0;

	my (undef, undef, $after, undef) = say($c, "sysconfig -r");

	foreach my $line (split(/\n/, $after)) {
		if ($line =~ /parity\s+(\S+)/) {
			push @paritydrive, $1;
			$paritycount++;
		}
	}
	
	logcomment("failparitydrive: SYSOP SYSOP! number of parity drive is: $paritycount");

	for (my $i=0; $i<$paritycount; $i++) {
		sleep 2;
		my $parity = $paritydrive[$i];
		logcomment("failparitydrive: SYSOP SYSOP! failing disk $parity");
		say($c, "disk fail $parity", 
			-prompt, "Really fail disk", 
			"Really prefail disk");
		say($c, "y");
	}

	return $paritycount;
}


sub waitreconstructend
{
	my ($c, $numfaildr) = @_;

	my $timeout = 36000;
	my $drcompleted = 0;
	my $curtime = time;
	my $endtime = $curtime + $timeout;
	
	while (($drcompleted < $numfaildr) && ($curtime < $endtime)) {
		say($c, "", "reconstruction completed for| completed in", -nocr, 
			-timeout, 60, -eval_error);
		if (!$@) {
			$drcompleted++;
		}
		$curtime = time;
	}

	if ($drcompleted < $numfaildr) {
		logcomment("waitreconstructend: SYSOP SYSOP! RAID reconstruction timeout!! Only $drcompleted of the estimated total of $numfaildr reconstructions were completed.");
	}
}


sub unfailthedrive
{
	my ($c) = @_;

	my $failcount = 0;
	my @faildrive = ();

	my (undef, undef, $after, undef) = say($c, "sysconfig -r");

	foreach my $line (split(/\n/, $after)) {
		if ($line =~ /failed\s+(\S+)/) {
			push @faildrive, $1;
			$failcount++;
		}
	}

	logcomment("unfailthedrive: SYSOP SYSOP! number of failed drive is: $failcount");

	for (my $i=0; $i < $failcount; $i++) {
		my $disk = $faildrive[$i];
		sleep 2;
		logcomment("unfailthedrive: SYSOP SYSOP! unfail disk $disk");
		say($c, "disk unfail $disk");
	}
}


sub countsparedrives
{
	my ($c) = @_;

	my (undef, undef, $after, undef) = say($c, "sysconfig -r");
	my $n = 0;
	foreach my $line (split(/\n/, $after)) {
		$n++ if ($line =~ /spare/);
	}

	return $n;
}


sub config_volumes
{
	my ($c, $numberofvols, $drivepervol) = @_;

	if (($numberofvols == 0) || ($drivepervol == 0)) {
		my $numspares = countsparedrives($c);
		my $spares;
		if ($numspares <= 16) {
			($numberofvols, $spares) = (1, 0);
		}
		elsif ($numspares <= 33) {
			($numberofvols, $spares) = (2, 1);
		}
		elsif ($numspares <= 50) {
			($numberofvols, $spares) = (3, 2);
		}
		elsif ($numspares <= 67) {
			($numberofvols, $spares) = (4, 3);
		}
		else {
			($numberofvols, $spares) = (5, 4);
		}

		$drivepervol = int ($numspares - $spares) / $numberofvols;
		$numspares = 2 + $numspares - ($drivepervol * $numberofvols);
		logcomment("config_volumes: Total spare drive = $numspares");
		logcomment("config_volumes: drive per volume = $drivepervol");
	}
	for (my $total = 0; $total < $numberofvols; $total++) {
              logcomment("Value of total $total");
		if ($total == 0) {
			my $numdrive = $drivepervol - 2;
			if ($numdrive > 0) {
                           logcomment("add volume");
				say($c, "vol add vol0 $numdrive",
					-prompt, "Are you sure you want to continue with vol add?|Continue?");
				say($c, "yes", "has completed successfully", -timeout, 7200);
			}
		}
		else {
			say($c, "vol create vol$total $drivepervol",
				-prompt, "Continue?");
			say($c, "y", "has completed successfully", -timeout, 7200);
		}
	}
	sleep 10;
	my $spares = countsparedrives($c);
	logcomment("config_volumes: spares = $spares");
}


sub start_scrub
{
	my ($c) = @_;
	my $match = say($c, "disk scrub start", "starting scrub|unable to start scrubbing", -noprompt, -timeout, 100, -ok);
	if ($match =~ /starting/) {
		logcomment("scrub process started successfully");
              return 1;
	} else {
		logcomment("RAID group is in degraded mode, is reconstructing, is recomputing parity, is copying disk, or is resyncing");
              return 0;
	}

}

sub waitforscrubend
{
	my ($c, $numdr) = @_;

	my $drcompleted = 0;
	logcomment("waitforscrubend: \$numdr = $numdr\t\$drcompleted = $drcompleted");
	while ($drcompleted < $numdr) {
		my $match = say($c, "", "scrub completed|PANIC", -nocr, -noprompt, -timeout, 600, -ok);

		if ($match =~ /PANIC/) {
			sleep 300;
			logresult("FATAL", "Filer panic while waiting for scrub end.");
		}
		elsif ($match =~ /scrub completed/) {
			$drcompleted++;
		}

		logcomment("waitforscrubend: \$numdr = $numdr\t\$drcompleted = $drcompleted");
	}
}


sub checkfcchannels
{
	my ($c, $testid) = @_;

	my $channelcount = 0;
	my $channel = [];
	my $timeout = 9;

	my ($FCchannelcount, $channel) = getfcchannels($c);
	for (my $total=0; $total < $FCchannelcount; $total++) {
		if ($testid == 1) {
			logcomment("checkfcchannels: Injecting dev_reset to channel $channel->[$total]");
			say($c, "fcadmin inject dev_reset enable -t 90 -c $channel->[$total]");
		}
		elsif ($testid == 2) {
			logcomment("checkfcchannels: Injecting adapter_reset to channel $channel->[$total]");
			say($c, "fcadmin inject adapter_reset enable -t 90 -c $channel->[$total]");
		}
		elsif ($testid == 3) {
			logcomment("checkfcchannels: Injecting LIP to channel $channel->[$total]");
			say($c, "fcadmin inject lip enable -t 90 -c $channel->[$total]");
		}
		elsif ($testid == 4) {
			logcomment("checkfcchannels: Injecting abort task set to channel $channel->[$total]");
			say($c, "fcadmin inject abort enable -t 90 -r -c $channel->[$total]");
		}
		elsif ($testid == 5) {
			logcomment("checkfcchannels: Injecting command timeout to channel $channel->[$total]");
			say($c, "fcadmin inject timeout enable -t 90 -r -c $channel->[$total]");
		}
		sleep 15;
	}

	return ($FCchannelcount, $channel);
}


sub checkvolumes
{
	my ($c) = @_;

	my $numvols = 0;
	my (undef, undef, $after, undef) = say($c, "vol status");
	foreach my $line (split(/\n/, $after)) {
		if ($line =~ /online/) {
			$numvols++;
		}
	}
	return $numvols;
}


sub checkdisks
{
	my ($c) = @_;

	my $diskcount = 0;
	my (undef, undef, $after, undef) = say($c, "disk_list");
#       logcomment("in after what is there $after");
#       my @lines = split(/\n/, $after);
 #      logcomment("line value is");

	foreach my $line (split(/\n/, $after)) {
              if ($line =~ /\d\w?:\d+|FC:[AB]/) {
			$diskcount++;
		}
	}
       logcomment("value of $diskcount");
	return $diskcount;
}


sub enable_nfs
{
	my ($c) = @_;
	say($c, "license add uytcncg", "nfs enabled");
}


sub start_fw_update
{
	my ($c, $opt) = @_;
	say($c, "disk_fw_update $opt", -prompt, "Are you sure you want to ");
	say($c, "y", 
		"Firmware is up-to-date on all disk drives|Disk drive firmware updated", 
		-timeout, 30, -ok);
}


sub checkfwupdate
{
	my ($c, $testtime) = @_;

	say($c, "", "Unable to download firmware", -timeout, $testtime, -eval_error);
	# in this case we fail the test if there is NOT a $@, ie. we did get the error msg
	if (!$@) {
		logresult("FATAL", "SYSOP! Unable to download firmware, please check your configuration!!!!");
	}
}


sub startio
{
	my ($c, $numthreadspervolume, $numvols, $typeofhammerthread) = @_;

	my $threadcount = 0;

	# numthreadspervolume should be multiples of 2
	for (my $i=0; $i < $numvols; $i++) {
		for (my $h=0; $h < $numthreadspervolume; $h++) {
			if ($typeofhammerthread eq "small") {
				say($c, "hammer -f -1 /vol/vol$i/h$h 123", -prompt, "Do you still want to run Hammer?");
				say($c, "y");
				$threadcount++;
			}
			elsif ($typeofhammerthread eq "large") {
				say($c, "hammer -f -1 /vol/vol$i/h$h 12345678", -prompt, "Do you still want to run Hammer?");
				say($c, "y");
				$threadcount++;
			}
			elsif ($typeofhammerthread eq "mixed") {
				say($c, "hammer -f -1 /vol/vol$i/h$h 1234", -prompt, "Do you still want to run Hammer?");
				say($c, "y");
				$threadcount++;
				say($c, "hammer -f -1 /vol/vol$i/h$h 12345678", -prompt, "Do you still want to run Hammer?");
				$threadcount++;
				$h++; 		# increment $h once more
			}
		}
	}
}


sub stopio
{
	my ($c) = @_;
	say($c, "hammer abort");
}


sub run_dq_perf_1
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T1; SEQUENTIAL READ PERF; OD/MD/ID; 1 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 1 -q 1 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 1 -q 1 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 1 -q 1 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 1 -q 1 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 1 -q 1 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 1 -q 1 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 1 -q 1 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 1 -q 1 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 1 -q 1 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 1 -q 1 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 1 -q 1 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 1 -q 1 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 1 -q 1 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 1 -q 1 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 1 -q 1 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 1 -q 1 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 1 -q 1 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 1 -q 1 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 1 -q 1 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 1 -q 1 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 1 -q 1 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 1 -q 1 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 1 -q 1 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 1 -q 1 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 1 -q 1 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 1 -q 1 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 1 -q 1 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}

sub run_dq_perf_2
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T2; RANDOM READ PERF; OD/MD/ID; 1 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 3 -q 1 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 3 -q 1 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 3 -q 1 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 3 -q 1 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 3 -q 1 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 3 -q 1 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 3 -q 1 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 3 -q 1 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 3 -q 1 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 3 -q 1 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 3 -q 1 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 3 -q 1 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 3 -q 1 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 3 -q 1 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 3 -q 1 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 3 -q 1 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 3 -q 1 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 3 -q 1 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 3 -q 1 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 3 -q 1 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 3 -q 1 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 3 -q 1 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 3 -q 1 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 3 -q 1 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 3 -q 1 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 3 -q 1 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 3 -q 1 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_3
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T3; SEQUENTIAL READ PERF; OD/MD/ID; 3 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 1 -q 3 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 1 -q 3 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 1 -q 3 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 1 -q 3 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 1 -q 3 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 1 -q 3 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 1 -q 3 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 1 -q 3 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 1 -q 3 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 1 -q 3 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 1 -q 3 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 1 -q 3 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 1 -q 3 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 1 -q 3 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 1 -q 3 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 1 -q 3 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 1 -q 3 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 1 -q 3 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 1 -q 3 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 1 -q 3 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 1 -q 3 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 1 -q 3 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 1 -q 3 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 1 -q 3 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 1 -q 3 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 1 -q 3 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 1 -q 3 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_4
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T4; RANDOM READ PERF; OD/MD/ID; 3 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 3 -q 3 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 3 -q 3 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 3 -q 3 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 3 -q 3 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 3 -q 3 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 3 -q 3 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 3 -q 3 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 3 -q 3 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 3 -q 3 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 3 -q 3 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 3 -q 3 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 3 -q 3 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 3 -q 3 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 3 -q 3 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 3 -q 3 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 3 -q 3 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 3 -q 3 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 3 -q 3 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 3 -q 3 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 3 -q 3 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 3 -q 3 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 3 -q 3 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 3 -q 3 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 3 -q 3 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 3 -q 3 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 3 -q 3 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 3 -q 3 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_5
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T5; SEQUENTIAL READ PERF; OD/MD/ID; 6 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 1 -q 6 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 1 -q 6 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 1 -q 6 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 1 -q 6 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 1 -q 6 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 1 -q 6 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 1 -q 6 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 1 -q 6 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 1 -q 6 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 1 -q 6 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 1 -q 6 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 1 -q 6 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 1 -q 6 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 1 -q 6 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 1 -q 6 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 1 -q 6 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 1 -q 6 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 1 -q 6 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 1 -q 6 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 1 -q 6 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 1 -q 6 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 1 -q 6 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 1 -q 6 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 1 -q 6 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 1 -q 6 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 1 -q 6 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 1 -q 6 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_6
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T6; RANDOM READ PERF; OD/MD/ID; 6 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 3 -q 6 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 3 -q 6 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 3 -q 6 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 3 -q 6 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 3 -q 6 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 3 -q 6 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 3 -q 6 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 3 -q 6 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 3 -q 6 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 3 -q 6 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 3 -q 6 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 3 -q 6 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 3 -q 6 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 3 -q 6 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 3 -q 6 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 3 -q 6 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 3 -q 6 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 3 -q 6 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 3 -q 6 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 3 -q 6 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 3 -q 6 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 3 -q 6 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 3 -q 6 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 3 -q 6 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 3 -q 6 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 3 -q 6 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 3 -q 6 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_7
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T7; SEQUENTIAL WRITE PERF; OD/MD/ID; 1 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 5 -q 1 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 5 -q 1 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 5 -q 1 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 5 -q 1 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 5 -q 1 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 5 -q 1 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 5 -q 1 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 5 -q 1 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 5 -q 1 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 5 -q 1 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 5 -q 1 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 5 -q 1 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 5 -q 1 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 5 -q 1 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 5 -q 1 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 5 -q 1 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 5 -q 1 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 5 -q 1 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 5 -q 1 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 5 -q 1 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 5 -q 1 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 5 -q 1 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 5 -q 1 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 5 -q 1 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 5 -q 1 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 5 -q 1 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 5 -q 1 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_8
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T8; RANDOM WRITE PERF; OD/MD/ID; 1 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 7 -q 1 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 7 -q 1 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 7 -q 1 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 7 -q 1 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 7 -q 1 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 7 -q 1 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 7 -q 1 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 7 -q 1 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 7 -q 1 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 7 -q 1 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 7 -q 1 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 7 -q 1 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 7 -q 1 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 7 -q 1 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 7 -q 1 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 7 -q 1 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 7 -q 1 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 7 -q 1 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 7 -q 1 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 7 -q 1 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 7 -q 1 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 7 -q 1 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 7 -q 1 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 7 -q 1 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 7 -q 1 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 7 -q 1 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 7 -q 1 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_9
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'T9; SEQUENTIAL WRITE PERF; OD/MD/ID; 3 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 5 -q 3 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 5 -q 3 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 5 -q 3 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 5 -q 3 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 5 -q 3 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 5 -q 3 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 5 -q 3 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 5 -q 3 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 5 -q 3 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 5 -q 3 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 5 -q 3 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 5 -q 3 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 5 -q 3 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 5 -q 3 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 5 -q 3 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 5 -q 3 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 5 -q 3 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 5 -q 3 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 5 -q 3 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 5 -q 3 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 5 -q 3 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 5 -q 3 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 5 -q 3 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 5 -q 3 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 5 -q 3 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 5 -q 3 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 5 -q 3 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_a
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'TA; RANDOM WRITE PERF; OD/MD/ID; 3 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 7 -q 3 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 7 -q 3 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 7 -q 3 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 7 -q 3 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 7 -q 3 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 7 -q 3 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 7 -q 3 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 7 -q 3 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 7 -q 3 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 7 -q 3 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 7 -q 3 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 7 -q 3 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 7 -q 3 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 7 -q 3 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 7 -q 3 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 7 -q 3 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 7 -q 3 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 7 -q 3 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 7 -q 3 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 7 -q 3 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 7 -q 3 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 7 -q 3 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 7 -q 3 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 7 -q 3 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 7 -q 3 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 7 -q 3 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 7 -q 3 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_b
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'TB; SEQUENTIAL WRITE PERF; OD/MD/ID; 6 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 5 -q 6 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 5 -q 6 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 5 -q 6 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 5 -q 6 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 5 -q 6 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 5 -q 6 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 5 -q 6 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 5 -q 6 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 5 -q 6 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 5 -q 6 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 5 -q 6 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 5 -q 6 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 5 -q 6 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 5 -q 6 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 5 -q 6 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 5 -q 6 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 5 -q 6 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 5 -q 6 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 5 -q 6 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 5 -q 6 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 5 -q 6 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 5 -q 6 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 5 -q 6 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 5 -q 6 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 5 -q 6 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 5 -q 6 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 5 -q 6 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_c
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'TC; RANDOM WRITE PERF; OD/MD/ID; 6 QUE; 4-256K Transfers; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	say($c, "disk_qual script0 -t 7 -q 6 -L 4000 -p 1 -f od -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 7 -q 6 -L 4000 -p 1 -f od -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 7 -q 6 -L 4000 -p 1 -f od -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 7 -q 6 -L 4000 -p 1 -f od -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 7 -q 6 -L 4000 -p 1 -f od -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 7 -q 6 -L 4000 -p 1 -f od -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 7 -q 6 -L 4000 -p 1 -f od -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 7 -q 6 -L 4000 -p 1 -f od -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 7 -q 6 -L 4000 -p 1 -f od -n 512 -d all -o noverify");
	## MD SEQUENTIAL READ ##
	say($c, "disk_qual script9 -t 7 -q 6 -L 4000 -p 1 -f md -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 7 -q 6 -L 4000 -p 1 -f md -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 7 -q 6 -L 4000 -p 1 -f md -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 7 -q 6 -L 4000 -p 1 -f md -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 7 -q 6 -L 4000 -p 1 -f md -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 7 -q 6 -L 4000 -p 1 -f md -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 7 -q 6 -L 4000 -p 1 -f md -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 7 -q 6 -L 4000 -p 1 -f md -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 7 -q 6 -L 4000 -p 1 -f md -n 512 -d all -o noverify");
	## ID SEQUENTIAL READ ##
	say($c, "disk_qual script18 -t 7 -q 6 -L 4000 -p 1 -f id -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 7 -q 6 -L 4000 -p 1 -f id -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 7 -q 6 -L 4000 -p 1 -f id -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 7 -q 6 -L 4000 -p 1 -f id -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 7 -q 6 -L 4000 -p 1 -f id -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 7 -q 6 -L 4000 -p 1 -f id -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 7 -q 6 -L 4000 -p 1 -f id -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 7 -q 6 -L 4000 -p 1 -f id -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 7 -q 6 -L 4000 -p 1 -f id -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_d
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'TD; FULL DISK RANDOM READ PERF; 1-3-6 QUEs; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	## FULL DISK RANDOM READ, -q 1 ##
	say($c, "disk_qual script0 -t 3 -q 1 -L 10000 -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 3 -q 1 -L 10000 -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 3 -q 1 -L 10000 -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 3 -q 1 -L 10000 -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 3 -q 1 -L 10000 -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 3 -q 1 -L 10000 -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 3 -q 1 -L 10000 -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 3 -q 1 -L 10000 -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 3 -q 1 -L 10000 -n 512 -d all -o noverify");
	## FULL DISK RANDOM READ, -q 3 ##
	say($c, "disk_qual script9 -t 3 -q 3 -L 10000 -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 3 -q 3 -L 10000 -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 3 -q 3 -L 10000 -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 3 -q 3 -L 10000 -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 3 -q 3 -L 10000 -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 3 -q 3 -L 10000 -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 3 -q 3 -L 10000 -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 3 -q 3 -L 10000 -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 3 -q 3 -L 10000 -n 512 -d all -o noverify");
	## FULL DISK RANDOM READ, -q 6 ##
	say($c, "disk_qual script18 -t 3 -q 6 -L 10000 -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 3 -q 6 -L 10000 -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 3 -q 6 -L 10000 -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 3 -q 6 -L 10000 -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 3 -q 6 -L 10000 -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 3 -q 6 -L 10000 -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 3 -q 6 -L 10000 -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 3 -q 6 -L 10000 -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 3 -q 6 -L 10000 -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


sub run_dq_perf_e
{
	my ($c, $fw, $drid) = @_;
	my $timeout = 60;

	say($c, "disk_qual remarks -d all -o 'TE; FULL DISK MIXED PERF; 1-3-6 QUEs; $drid drive; $fw FW");
	say($c, "disk_qual setparam SyncScripting 5");
	say($c, "disk_qual setparam ContinueOnError 1");
	say($c, "disk_qual setparam Relocate03Error 1");
	## FULL DISK RANDOM READ, -q 1 ##
	say($c, "disk_qual script0 -t 16 -q 1 -L 10000 -n 8 -d all -o noverify");
	say($c, "disk_qual script1 -t 16 -q 1 -L 10000 -n 16 -d all -o noverify");
	say($c, "disk_qual script2 -t 16 -q 1 -L 10000 -n 24 -d all -o noverify");
	say($c, "disk_qual script3 -t 16 -q 1 -L 10000 -n 32 -d all -o noverify");
	say($c, "disk_qual script4 -t 16 -q 1 -L 10000 -n 48 -d all -o noverify");
	say($c, "disk_qual script5 -t 16 -q 1 -L 10000 -n 64 -d all -o noverify");
	say($c, "disk_qual script6 -t 16 -q 1 -L 10000 -n 128 -d all -o noverify");
	say($c, "disk_qual script7 -t 16 -q 1 -L 10000 -n 256 -d all -o noverify");
	say($c, "disk_qual script8 -t 16 -q 1 -L 10000 -n 512 -d all -o noverify");
	## FULL DISK RANDOM READ, -q 3 ##
	say($c, "disk_qual script9 -t 16 -q 3 -L 10000 -n 8 -d all -o noverify");
	say($c, "disk_qual script10 -t 16 -q 3 -L 10000 -n 16 -d all -o noverify");
	say($c, "disk_qual script11 -t 16 -q 3 -L 10000 -n 24 -d all -o noverify");
	say($c, "disk_qual script12 -t 16 -q 3 -L 10000 -n 32 -d all -o noverify");
	say($c, "disk_qual script13 -t 16 -q 3 -L 10000 -n 48 -d all -o noverify");
	say($c, "disk_qual script14 -t 16 -q 3 -L 10000 -n 64 -d all -o noverify");
	say($c, "disk_qual script15 -t 16 -q 3 -L 10000 -n 128 -d all -o noverify");
	say($c, "disk_qual script16 -t 16 -q 3 -L 10000 -n 256 -d all -o noverify");
	say($c, "disk_qual script17 -t 16 -q 3 -L 10000 -n 512 -d all -o noverify");
	## FULL DISK RANDOM READ, -q 6 ##
	say($c, "disk_qual script18 -t 16 -q 6 -L 10000 -n 8 -d all -o noverify");
	say($c, "disk_qual script19 -t 16 -q 6 -L 10000 -n 16 -d all -o noverify");
	say($c, "disk_qual script20 -t 16 -q 6 -L 10000 -n 24 -d all -o noverify");
	say($c, "disk_qual script21 -t 16 -q 6 -L 10000 -n 32 -d all -o noverify");
	say($c, "disk_qual script22 -t 16 -q 6 -L 10000 -n 48 -d all -o noverify");
	say($c, "disk_qual script23 -t 16 -q 6 -L 10000 -n 64 -d all -o noverify");
	say($c, "disk_qual script24 -t 16 -q 6 -L 10000 -n 128 -d all -o noverify");
	say($c, "disk_qual script25 -t 16 -q 6 -L 10000 -n 256 -d all -o noverify");
	say($c, "disk_qual script26 -t 16 -q 6 -L 10000 -n 512 -d all -o noverify");
	say($c, "disk_qual script27 -t -1 -d all");
	say($c, "disk_qual runscript -d $drid");

	#say($c, "", -nocr, -prompt,
	#"Starting .* test. Using sector interval .*", 
	#"wait for .* test to finish",
	#-timeout, $timeout, -ok);
	sleep 20;
}


1;
