#$Id: //depot/prod/test/Rsierranevada/lib/Storage/Quarch.pm#1 $
##
## @summary quarch - Library module for quarch command
## @author phamj@netapp.com,dl-storage-verification

=head1 NAME

Storage::Quarch

=head1 DESCRIPTION

This library defines a set of interfaces to interact with quarch controller and
quarch drive card 

=head1 SYNOPSIS

use Storage::Quarch qw(
    quarch_power_cycle
    quarch_init
    quarch_set_iicfault
    quarch_set_iic1_fault
    quarch_set_iic2_fault
    quarch_set_iic3_fault
    quarch_clear_iicfault
    quarch_set_sgpiofault
    quarch_clear_sgpiofault
    quarch_plugin
    quarch_mate_e
    quarch_slot_ids
    quarch_drv_power_cycle
    quarch_drv_init
    quarch_drv_short_3V
    quarch_drv_short_5V
    quarch_drv_short_12V
    quarch_drv_iso_3V
    quarch_drv_iso_5V
    quarch_drv_iso_12V
    quarch_drv_disable_5V
    quarch_drv_disable_12V
    quarch_drv_plugin
    quarch_drv_mate_disabled
    quarch_drv_mate_e
    expectoutput
);
   
=head1 ROUTINES

=cut

package Storage::Quarch;
use strict;
use warnings;


BEGIN {

    use base qw(Exporter);
    our @EXPORT_OK = qw(
	quarch_power_cycle
        quarch_init
        quarch_set_iicfault
        quarch_set_iic1_fault
        quarch_set_iic2_fault
        quarch_set_iic3_fault
        quarch_clear_iicfault
        quarch_set_sgpiofault
        quarch_clear_sgpiofault
        quarch_plugin
        quarch_mate_e
        quarch_slot_ids
        quarch_drv_power_cycle
        quarch_drv_init
        quarch_drv_short_3V
        quarch_drv_short_5V
        quarch_drv_short_12V
        quarch_drv_disable_5V
        quarch_drv_disable_12V
        quarch_drv_iso_3V
        quarch_drv_iso_5V
        quarch_drv_iso_12V
        quarch_drv_plugin
        quarch_drv_mate_disabled
        quarch_drv_mate_e
	expectoutput
            );
}

use Carp qw(confess croak);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF HASHREF );
use Tharn qw(say script_result host logcomment logresult param);
use Utils qw(make_conn);
use Data::Dumper;

#----------------------------------------------------------------------
# The exported rotines are coded in the following section
#----------------------------------------------------------------------

=head1 FUNCTION NAME

quarch_power_cycle

=head1 FUNCTION DESCRIPTION

The library function support power cycle quarch card

=head1 FUNCTION PROTOTYPES

quarch_power_cycle($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_power_cycle {

      my ($qc) = @_;
      logcomment(" Quarch Power Cycle \n");
      say($qc, "power down");
      say($qc, "power up");

}

=head1 FUNCTION NAME

quarch_init

=head1 FUNCTION DESCRIPTION

The library function provideds initialization quarch card

=head1 FUNCTION PROTOTYPES

quarch_init($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_init {

       my ($qc) = @_;
       logcomment(" Quarch Initialize \n");
       my $start_dec = 109;
       my $end_dec = 208;
       my $start = "0x00";

       say($qc, "write 0x00 0x01");
       while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x77");
                $start_dec = $start_dec + 1;
       }
       $start_dec = 1;
       $end_dec = 108;
       while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x00");
                $start_dec = $start_dec + 1;
       }
       say($qc, "write 0x00 0x01");
}

=head1 FUNCTION NAME

quarch_set_iicfault

=head1 FUNCTION DESCRIPTION

The library function set iic faults to all iic buses of the module

=head1 FUNCTION PROTOTYPES

quarch_set_iicfault($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_set_iicfault {

        my ($qc) = @_;
       logcomment(" Quarch Set IICFAULT \n");
	say($qc, "write 0x75 0x00");
        say($qc, "write 0x76 0x00");
        say($qc, "write 0x77 0x00");
	
}

=head1 FUNCTION NAME

quarch_set_iic1_fault

=head1 FUNCTION DESCRIPTION

The library function set iic bus 1 faults can be cleared by function quarch_clear_iicfault

=head1 FUNCTION PROTOTYPES

quarch_set_iic1_fault($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_set_iic1_fault {

        my ($qc) = @_;
       logcomment(" Quarch Set IIC1 FAULT \n");
	say($qc, "write 0x75 0x00");
}

	
=head1 FUNCTION NAME

quarch_set_iic2_fault

=head1 FUNCTION DESCRIPTION

The library function set iic bus 2 faults can be clear by quarch_clear_iicfault.

=head1 FUNCTION PROTOTYPES

quarch_set_iic2_fault($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_set_iic2_fault {

        my ($qc) = @_;
       logcomment(" Quarch Set IIC2 FAULT \n");
	say($qc, "write 0x76 0x00");
}

	
=head1 FUNCTION NAME

quarch_set_iic3_fault

=head1 FUNCTION DESCRIPTION

The library function set iic bus 3 faults can be cleared by quarch_clear_iicfault

=head1 FUNCTION PROTOTYPES

quarch_set_iic3_fault($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_set_iic3_fault {

        my ($qc) = @_;
       logcomment(" Quarch Set IIC3 FAULT \n");
	say($qc, "write 0x77 0x00");
}

	
=head1 FUNCTION NAME

quarch_clear_iicfault

=head1 FUNCTION DESCRIPTION

The library function reset iic to all iic buses of the module

=head1 FUNCTION PROTOTYPES

quarch_clear_iicfault($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_clear_iicfault {

        my ($qc) = @_;
        logcomment(" Quarch Clear IICFAULT \n");
	say($qc, "write 0x75 0x77");
        say($qc, "write 0x76 0x77");
        say($qc, "write 0x77 0x77");
	
}


=head1 FUNCTION NAME

quarch_set_sgpiofault

=head1 FUNCTION DESCRIPTION

The library function set sgpio faults to module

=head1 FUNCTION PROTOTYPES

quarch_set_sgpiofault($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_set_sgpiofault {

        my ($qc) = @_;
        logcomment(" Quarch Set SGPIOFAULT \n");
	say($qc, "write 0x7E 0x00");
        say($qc, "write 0x7F 0x00");
	
}

=head1 FUNCTION NAME

quarch_clear_sgpiofault

=head1 FUNCTION DESCRIPTION

The library function reset sgpio signal

=head1 FUNCTION PROTOTYPES

quarch_clear_sgpiofault($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_clear_sgpiofault {

        my ($qc) = @_;
        logcomment(" Quarch Clear SGPIOFAULT \n");
	say($qc, "write 0x7E 0x77");
        say($qc, "write 0x7F 0x77");
	
}

=head1 FUNCTION NAME

quarch_plugin

=head1 FUNCTION DESCRIPTION

The library function simulate a plugin module to the mid-plane

=head1 FUNCTION PROTOTYPES

quarch_plugin(%input);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=item s1_delay
s1_delay provide the delay of power source 1 to the module

=item s2_delay
s2_delay provide the delay of power source 1 to the module

=cut

sub quarch_plugin {

        my ($qc, $s1_delay, $s2_delay) = @_;
        logcomment(" Quarch PlugIn \n");
        my $start_dec = 109;
        my $end_dec = 208;
        my $start = "0x00";

        my $s1x = 0x00;
        my $s2x = 0x00;
        $start_dec = 1;
        $end_dec = 208;
        while ($start_dec <= $end_dec){
               $start = sprintf("%02x",$start_dec);
               say($qc, "write 0x$start 0x00");
               $start_dec = $start_dec + 1;
        }
        #set up source S1 and S2
        #
        #enable Control Source S1 and S2
        say($qc, "write 0x04 0x11");
        #S1 Configure
        #
        # Set Initial Delay Period for S1 to s1_delay
        $s1x = sprintf("%02x",$s1_delay);
        say($qc, "write 0x07 0x$s1x");
        #S2 Configure
        #
        #Set initial delay period for S2 to s2_delay
        $s2x = sprintf("%02x",$s2_delay);
        say($qc, "write 0x18 0x$s2x");
        #Set Oscillation Period for S2 to 0.01ms
        say($qc, "write 0x19 0x01");
        #Set Oscillation time for S2 to 1ms
        say($qc, "write 0x1A 0x01");
        #Set Oscillation Duty Cycle for S2 to 50%
        say($qc, "write 0x1b 0x32");
        #Assign GND_PIN_M3_G6 to Source 2
        say($qc, "write 0xd0 0x02");
        #other Signals
        #Assign to Source 1
        $start_dec = 109;
        $end_dec = 207;
        while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x11");
                $start_dec = $start_dec + 1;
        }
}


=head1 FUNCTION NAME

quarch_mate_e

=head1 FUNCTION DESCRIPTION

The library function simulate a plugin module to the mid-plane with mate signal had been delayed

=head1 FUNCTION PROTOTYPES

quarch_plugin(%input);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_mate_e {

        my ($qc) = @_;
        logcomment(" Quarch Mate_E \n");
        my $start_dec = 1;
        my $end_dec = 208;
        my $start = "0x00";

        $start_dec = 1;
        $end_dec = 208;
        while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x00");
                $start_dec = $start_dec + 1;
        }
        #set up source S1,S2 and S3
        say($qc, "write 0x04 0x11");
        say($qc, "write 0x05 0x01");
        #S1 Configure
        #
        # Set Initial Delay Period for S1 to 20ms
        say($qc, "write 0x07 0x14");
        #S2 Configure
        #
        #Set initial delay period for S2 to 10ms
        say($qc, "write 0x18 0x0a");
        #S3 Configure
        #
        #Set initial delay period for S3 to 21ms
        say($qc, "write 0x29 0x15");
        #signal assignment
        #Assign 12V_A ON &12V_B to ON
        say($qc, "write 0x6D 0x77");
        #Assign STANDBY_PWR to source 3 and POWER_OFF_L to source 3
        say($qc, "write 0x6e 0x33");
        #Assign SLOT_ID to source 3 & MATED_L to source 1
        say($qc, "write 0x6F 0x31");
        #Assign GND_PIN_M3_G6 to Source 2
        say($qc, "write 0xd0 0x02");
        #other Signals
        #Assign to Source 3
        $start_dec = 112;
        $end_dec = 207;
        while ($start_dec <= $end_dec){
               $start = sprintf("%02x",$start_dec);
               say($qc, "write 0x$start 0x33");
               $start_dec = $start_dec + 1;
        }
}

=head1 FUNCTION NAME

quarch_slot_ids

=head1 FUNCTION DESCRIPTION

The library function simulate a plugin module to the mid-plane with slot ids signal had been delayed

=head1 FUNCTION PROTOTYPES

quarch_plugin(%input);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_slot_ids {

        my ($qc) = @_;
        logcomment(" Quarch Slot_IDS \n");
        my $start_dec = 1;
        my $end_dec = 208;
        my $start = "0x00";

        $start_dec = 1;
        $end_dec = 208;
        while ($start_dec <= $end_dec){
               $start = sprintf("%02x",$start_dec);
               say($qc, "write 0x$start 0x00");
               $start_dec = $start_dec + 1;
        }
        #set up source S1,S2 and S3
        say($qc, "write 0x04 0x11");
        say($qc, "write 0x05 0x01");
        #S1 Configure
        #
        # Set Initial Delay Period for S1 to 20ms
        say($qc, "write 0x07 0x14");
        #S2 Configure
        #
        #Set initial delay period for S2 to 10ms
        say($qc, "write 0x18 0x0A");
        #S3 Configure
        #
        #Set initial delay period for S3 to 2lms
        say($qc, "write 0x29 0x15");
        #signal assignment
        #Assign 12V_A ON &12V_B to ON
        say($qc, "write 0x6D 0x77");
        #Assign STANDBY_PWR to source 1 and POWER_OFF_L to source 1
        say($qc, "write 0x6e 0x11");
        #Assign SLOT_ID to source 3 & MATED_L to source 1
        say($qc, "write 0x6F 0x31");
        #Assign GND_PIN_M3_G6 to Source 2
        say($qc, "write 0xd0 0x02");
        #other Signals
        #Assign to Source 1
        $start_dec = 112;
        $end_dec = 207;
        while ($start_dec <= $end_dec){
              $start = sprintf("%02x",$start_dec);
              say($qc, "write 0x$start 0x11");
              $start_dec = $start_dec + 1;
       }
}


=head1 FUNCTION NAME

quarch_drv_power_cycle

=head1 FUNCTION DESCRIPTION

The library function support power cycle quarch drive card

=head1 FUNCTION PROTOTYPES

quarch_drv_power_cycle($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch drive card

=cut

sub quarch_drv_power_cycle {

      my ($qc) = @_;
      logcomment(" Quarch Drive Power Cycle \n");
      say($qc, "power down");
      say($qc, "power up");

}


=head1 FUNCTION NAME

quarch_drv_init

=head1 FUNCTION DESCRIPTION

The library function provideds initialization quarch drive card

=head1 FUNCTION PROTOTYPES

quarch_drv_init($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_init {

       my ($qc) = @_;
       logcomment(" Quarch Drive Initialize \n");
       my $start_dec = 109;
       my $end_dec = 111;
       my $start = "0x00";

       while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x77");
                $start_dec = $start_dec + 1;
       }
       $start_dec = 1;
       $end_dec = 108;
       while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x00");
                $start_dec = $start_dec + 1;

       }
       say($qc, "write 0x70 0x77");
       say($qc, "write 0x00 0x01");
       say($qc, "write 0x00 0x01");
}

=head1 FUNCTION NAME

quarch_drv_short_3V

=head1 FUNCTION DESCRIPTION

The library function set short 3V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_short_3V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_short_3V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Short 3V \n");
        say($qc, "write 0x01 0x13");

}


=head1 FUNCTION NAME

quarch_drv_short_5V

=head1 FUNCTION DESCRIPTION

The library function set short 5V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_short_5V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_short_5V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Short 5V \n");
        say($qc, "write 0x01 0x25");

}


=head1 FUNCTION NAME

quarch_drv_short_12V

=head1 FUNCTION DESCRIPTION

The library function set short 12V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_short_12V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_short_12V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Short 12V \n");
        say($qc, "write 0x01 0x49");

}



=head1 FUNCTION NAME

quarch_drv_iso_3V

=head1 FUNCTION DESCRIPTION

The library function Isolated 3V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_iso_3V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_iso_3V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Isolate 3V \n");
        say($qc, "write 0x01 0x11");

}


=head1 FUNCTION NAME

quarch_drv_iso_5V

=head1 FUNCTION DESCRIPTION

The library function Isolate 5V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_iso_5V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_iso_5V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Isolate 5V \n");
        say($qc, "write 0x01 0x21");

}


=head1 FUNCTION NAME

quarch_drv_iso_12V

=head1 FUNCTION DESCRIPTION

The library function Isolate 12V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_iso_12V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_iso_12V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Isolate 12V \n");
        say($qc, "write 0x01 0x41");

}


=head1 FUNCTION NAME

quarch_drv_disable_5V

=head1 FUNCTION DESCRIPTION

The library function disable 5V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_disable_5V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_disable_5V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Disable 5V \n");
        say($qc, "write 0x6E 0x00");

}


=head1 FUNCTION NAME

quarch_drv_disable_12V

=head1 FUNCTION DESCRIPTION

The library function disable 12V to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_disable_12V($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_disable_12V{

        my ($qc) = @_;
       logcomment(" Quarch Drive Disable 12V \n");
        say($qc, "write 0x6F 0x00");

}


=head1 FUNCTION NAME

quarch_drv_mate_disabled

=head1 FUNCTION DESCRIPTION

The library function disable mate pin to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_mate_disabled($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_mate_disabled{

        my ($qc) = @_;
       logcomment(" Quarch Drive Disable Mate Pin\n");
        say($qc, "write 0x70 0x00");

}


=head1 FUNCTION NAME

quarch_drv_mate_enabled

=head1 FUNCTION DESCRIPTION

The library function enable mate pin to the drive

=head1 FUNCTION PROTOTYPES

quarch_drv_mate_enabled($quarch_name);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_mate_enabled{

        my ($qc) = @_;
       logcomment(" Quarch Drive Enable Mate Pin\n");
        say($qc, "write 0x70 0x77");

}


=head1 FUNCTION NAME

quarch_drv_plugin

=head1 FUNCTION DESCRIPTION

The library function simulate a plugin  or insert a drive

=head1 FUNCTION PROTOTYPES

quarch_drv_plugin(%input);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=item s1_delay
s1_delay provide the delay of power source 1 to the module

=item s2_delay
s2_delay provide the delay of power source 1 to the module

=cut

sub quarch_drv_plugin {

        my ($qc, $s1_delay, $s2_delay) = @_;
        logcomment(" Quarch Drive PlugIn \n");
        my $start_dec = 1;
        my $end_dec = 111;
        my $start = "0x00";

        my $s1x = 0x00;
        my $s2x = 0x00;
        $start_dec = 1;
        $end_dec = 111;
        while ($start_dec <= $end_dec){
               $start = sprintf("%02x",$start_dec);
               say($qc, "write 0x$start 0x00");
               $start_dec = $start_dec + 1;
        }
        #set up source S1 and S2
        #
        #enable Control Source S1 and S2
        say($qc, "write 0x04 0x11");
        #S1 Configure
        #
        # Set Initial Delay Period for S1 to s1_delay
        $s1x = sprintf("%02x",$s1_delay);
        say($qc, "write 0x07 0x$s1x");
        #S2 Configure
        #
        #Set initial delay period for S2 to s2_delay
        $s2x = sprintf("%02x",$s2_delay);
        say($qc, "write 0x18 0x$s2x");
        #Set Oscillation Period for S2 to 0.01ms
        say($qc, "write 0x19 0x01");
        #Set Oscillation time for S2 to 1ms
        say($qc, "write 0x1A 0x01");
        #Set Oscillation Duty Cycle for S2 to 50%
        say($qc, "write 0x1b 0x32");
        #Assign GND_PIN_M3_G6 to Source 2
        say($qc, "write 0x70 0x02");
        #other Signals
        #Assign to Source 1
        $start_dec = 109;
        $end_dec = 111;
        while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x11");
                $start_dec = $start_dec + 1;
        }
}


=head1 FUNCTION NAME

quarch_drv_mate_e

=head1 FUNCTION DESCRIPTION

The library function simulate a plugin module to the mid-plane with mate signal had been delayed

=head1 FUNCTION PROTOTYPES

quarch_drv_mate_e(%input);

=item quarch_name
quarch_name provide the serial port connected to quarch card

=cut

sub quarch_drv_mate_e {

        my ($qc) = @_;
        logcomment(" Quarch Drive Mate_E \n");
        my $start_dec = 1;
        my $end_dec = 111;
        my $start = "0x00";

        $start_dec = 1;
        $end_dec = 111;
        while ($start_dec <= $end_dec){
                $start = sprintf("%02x",$start_dec);
                say($qc, "write 0x$start 0x00");
                $start_dec = $start_dec + 1;
        }
        #set up source S1,S2 and S3
        say($qc, "write 0x04 0x11");
        say($qc, "write 0x05 0x01");
        #S1 Configure

        #
        # Set Initial Delay Period for S1 to 20ms
        say($qc, "write 0x07 0x14");
        #S2 Configure
        #
        #Set initial delay period for S2 to 10ms
        say($qc, "write 0x18 0x0a");
        #S3 Configure
        #
        #Set initial delay period for S3 to 21ms
        say($qc, "write 0x29 0x15");
        #signal assignment
        #Assign 12V_A ON &12V_B to ON
        say($qc, "write 0x6D 0x77");
        #Assign STANDBY_PWR to source 3 and POWER_OFF_L to source 3
        say($qc, "write 0x6e 0x33");
        #Assign SLOT_ID to source 3 & MATED_L to source 1
        say($qc, "write 0x6F 0x31");
        #Assign GND_PIN_M3_G6 to Source 2
        say($qc, "write 0x70 0x02");
        #other Signals
        #Assign to Source 3
        $start_dec = 109;
        $end_dec = 111;
        while ($start_dec <= $end_dec){
               $start = sprintf("%02x",$start_dec);
               say($qc, "write 0x$start 0x33");
               $start_dec = $start_dec + 1;
        }
#               Set MATED to follow source 1 and 2
        say($qc, "write 0x70 0x33");

}



=head1 FUNCTION NAME

expectoutput

=head1 FUNCTION DESCRIPTION

The library function check whether a expected text is present in a output

=head1 FUNCTION PROTOTYPES

expectoutput($$$);

=item output_str expected_str err_str
output_str provide the output from the test
expected_str provide the expected string in the output
err_str provide the message displayed if the expected string cannot be found in the output string

=cut

sub expectoutput { 

        my ($output_str, $expected_str, $err_str) = @_;
	if ($output_str =~ m/$expected_str/) { 
	    	logresult('PASS', "expected text ", $expected_str, " found");
	}
	else
	{
		logresult('FAIL', $err_str);
	}

}

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;

