#$Id: //depot/prod/test/Rsierranevada/lib/Storage/SESPage.pm#1 $
#
## @summary SES Page status checking library 
#
## @status TESTME
##
## @author vickie@netapp.ocm
##
#
## @pod here

=head1 NAME

SESPage

=head1 SYNOPSIS

	use SESPage qw(function_name);

	my $host = host($SHELF);

	my $hash = function_name(host=>$host,...);

=head1 DESCRIPTION
	This is a library module for the SES status page checking

=head1 ROUTINES

=cut

package Storage::SESPage;

use strict;
use warnings;

use Carp qw(confess croak);
use Params::Validate qw(validate validate_pos SCALAR OBJECT ARRAYREF);
BEGIN {
Params::Validate::validation_options(
	on_fail => sub {my @msgs = @_; chomp(@msgs);
	Carp::confess(Tharn::script_result(@msgs))}
	);
}
use Tharn qw(say script_result host logcomment logresult param);

use base qw(Exporter);
use Utils qw(make_conn);
use Data::Dumper;
use Storage::Sapphire qw(phy_report expctrl_tp exp_report ses_statuspage get_midplanevpd get_drive_status ver get_psu_status get_fan_status get_temp_status ddump_general phydump ddump_drvmgr dumpdrives qsfpinfo cablemgr_tp ddump_expmgr ddump_acpmgr ses_report_page2 gem_cmd);

our  @EXPORT_OK = qw(
	sesPage0_status
	sesPage1_status
	sesPage2_status
	sesPage5_status
	sesPage7_status
	sesPage0a_status
	sesPage8081_status
	sesPage8283_status
);

our $MAINT_PROMPT = "\\*\>";

=head2 sesPage0_status
	my $ret = sesPage0_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

	This routine checks each bytes of page 0 and compare data with shelf firmware commands to check the consistency and correctness for the page.

	Example of returning status:
	my $status = sesPage0_status();
	print "Page 0 status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage0_status {
    my (%opts) = validate(@_,{
        host => {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef},
        AorB => {type=>SCALAR,default=>"A"},
        sespage0_data_file => {type=>SCALAR, default=>undef},
        }
    );
    my $pagenum = 0;
    my %hash = ();
    my $output;
    my $c;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$pagenum);

    logcomment("Basic SES Page 0 Test");

    my $read_in = `cat $opts{sespage0_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $status = 0;
    my $err = 0;
my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];
                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage 0 has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0 Data File."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage 0 has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0 Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage 0 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0 Data File."); 
                    $err = 1;
                }
            }
        }
    } # end of foreach 

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage 1 \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    } 
    return $err;
}

=head2 sesPage1_status
    my $ret = sesPage1_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

    This routine checks each bytes of page 1 and compare data with shelf firmware commands to check the consistency and correctness for the page.

    Example of returning status:
    my $status = sesPage1_status();
    print "Page 1 status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage1_status {
    my (%opts) = validate(@_,{
        host => {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef,optional=>1},
        AorB => {type=>SCALAR,default=>"A"},
        sespage1_data_file => {type=>SCALAR, default=>undef},
        }
    );
    my $pagenum = 1;
    my %hash = ();
    my $output;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$pagenum);

    logcomment("Basic SES Page 1 Test");

    my $read_in = `cat $opts{sespage1_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $wwn;
    my $fw_ver;
    my $shelf_id;
    my $bp_serial_no;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $err = 0;
    my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];
                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage 1 has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 1 Data File."); 
                } elsif ($bytes->{$tt[0]}->{value} eq "x") {
        	    if ($bytes->{$tt[0]}->{info} =~ /Enclosure Logical ID/) {             
                        logcomment("Enclosure Logical ID (WWN) byte $tt[0]");
                        $wwn = $wwn.$output->{$tt[0]};
                    } elsif ($bytes->{$tt[0]}->{info} =~ /Controller FW version/) {             
                        logcomment("Canister Firmware version byte $tt[0]");
                        my $ver_tmp = chr (hex $output->{$tt[0]});
                        $fw_ver = $fw_ver.$ver_tmp;
                        $fw_ver =~ tr/A-Z/a-z/;
                    } elsif ($bytes->{$tt[0]}->{info} =~ /Shelf ID/) {
                        logcomment("Shelf ID byte $tt[0]");
                        $shelf_id = $output->{$tt[0]};
                    } elsif ($bytes->{$tt[0]}->{info} =~ /Backplane Serial Number/) {
                        logcomment("Backplane Serial Number byte $tt[0]");
                        my $sn_tmp = chr (hex $output->{$tt[0]});
                        $bp_serial_no = $bp_serial_no.$sn_tmp;
                    }
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage 1 has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 1 Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage 1 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 1 Data File."); 
                    $err = 1;
                }
            }
        }
    }

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage 1 \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    } else {
        my $info = get_midplanevpd(host=>$opts{host},timeout=>$opts{timeout},gemconn=>$opts{gemconn},connmode=>$opts{connmode}, adapter_id=>$opts{adapter_id}, shelf_id=>$opts{shelf_id}, AorB=>$opts{AorB});
    
        # check Enclosure Logical ID: WWN 
        if ($wwn eq $info->{enclWWN_1}) {
            print "Page $pagenum has right shelf wwn: $wwn\n";
        } else {
            logcomment("FAIL: Page $pagenum has wrong shelf wwn: $wwn, should be $info->{enclWWN_1}");
            $err = 1;
        }
    
        # check shelf id
        if ((hex $shelf_id) == $info->{enclID_1}) {
            print "Page $pagenum has right Shelf ID: $shelf_id\n";
        } else {
            logcomment("FAIL: Page $pagenum has wrong Shelf ID: $shelf_id, should be $info->{enclID_1}");
            $err = 1;
        }
    
        # check backplane serial number
        if ((int $bp_serial_no) eq (int $info->{chasSerialNum_1})) {
            print "Page $pagenum has right Backplane Serial Number: $bp_serial_no\n";
        } else {
            logcomment("FAIL: Page $pagenum has wrong Backplane Serial Number: $bp_serial_no, should be $info->{chasSerialNum_1}");
           $err = 1;
        }
    
        # check firmware version
        my $version = ver(host=>$opts{host},timeout=>$opts{timeout},gemconn=>$opts{gemconn},connmode=>$opts{connmode},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB});
        my $temp_ver = $version->{CanisterFW};
        $temp_ver =~ s/\.//g; 
        if ($fw_ver == $temp_ver) {
            print "Page $pagenum has right firmware revision: $fw_ver\n";
        } else {
            logcomment("FAIL: Page $pagenum has wrong firmware revision: $fw_ver, should be $temp_ver");
            $err = 1;
        }
    }
    return $err;
}

=head2 sesPage2_status
	my $ret = sesPage2_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

	This routine checks each bytes of page 2 and compare data with shelf firmware commands to check the consistency and correctness for the page.

	Example of returning status:
	my $status = sesPage2_status();
	print "Page 2 status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=item shelf_type

(Required) : shelf type to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage2_status {
    my (%opts) = validate(@_,{
        host => {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef,optional=>1},
        AorB => {type=>SCALAR,default=>"A"},
        sespage2_data_file => {type=>SCALAR, default=>undef},
        }
    );

    my $pagenum = 2;
    my %hash = ();
    my $output;
    my $c;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$pagenum);

    logcomment("Basic SES Page 2 Test library routine");

    my $read_in = `cat $opts{sespage2_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $wwn;
    my $fw_ver;
    my $shelf_id;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $err = 0;
my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];

                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage 2 has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 2 Data File."); 
                } elsif ($bytes->{$tt[0]}->{value} eq "x") {
                    logcomment("byte $tt[0] from ses_statuspage 2 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 2 Data File. This byte can be any reasonable value, please check the values manually. This mismatch won't cause test script to fail."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Status 1st/) { 
                    if ($output->{$tt[0]} =~ /05/) {
                        logcomment("byte $tt[0] from ses_statuspage 2 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 2 Data File. This indicates the element is Not Installed, please check the element condition manually. This mismatch won't cause test script to fail."); 
                    } else {
                        logcomment("FAIL: byte $tt[0] from ses_statuspage 2 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 2 Data File. It does not indicate the element is Not Installed neither, please check the element condition manually. This mismatch will cause test script to fail."); 
                        $err = 1;
                    }
                } elsif ($bytes->{$tt[0]}->{info} =~ /Summary Status/) {
                    logcomment("byte $tt[0] from ses_statuspage 2 has Summary Status $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 2 Data File. The INVOP, INFO, NON-CRIT, CRIT, and UNRECOV bits returned in normal status pages are mandatory. Please check shelf health condition."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage 2 has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 2 Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } elsif ($bytes->{$tt[0]}->{info} =~ /1st byte/) { 
                    if ($bytes->{$tt[0]}->{info} =~ /Overall/) { 
                        logcomment("FAIL: Overall Status byte $tt[0] \($bytes->{$tt[0]}->{info}\) should be 0, but ses_statuspage 2 has value $output->{$tt[0]}."); 
                        $err = 1;
                    } elsif ($bytes->{$tt[0]}->{info} =~ /Generation Code/) { 
                        logcomment("FAIL: Generation Code 1st byte $tt[0] \($bytes->{$tt[0]}->{info}\) should be 0, but ses_statuspage 2 has value $output->{$tt[0]}."); 
                        $err = 1;
                    } elsif (($bytes->{$tt[0]}->{info} =~ /Ethernet/) && ($output->{$tt[0]} == 0))  { 
                        logcomment("Ethernet 1st byte $tt[0] \($bytes->{$tt[0]}->{info}\) should be 01, but ses_statuspage 2 has value 0 which means Ethernet element is not supported. This is fine for IOM6 module"); 
                    } elsif (($bytes->{$tt[0]}->{info} =~ /Ethernet/) && ($output->{$tt[0]} == 1))  { 
                        logcomment("Ethernet 1st byte $tt[0] \($bytes->{$tt[0]}->{info}\) is 01, it means Ethernet element is supported. This is a IOM6E module"); 
                    } else {
                        logcomment("FAIL: 1st byte $tt[0] \($bytes->{$tt[0]}->{info}\) should be $bytes->{$tt[0]}->{value}, but ses_statuspage 2 has value $output->{$tt[0]}. Check this element's health condition."); 
                        $err = 1;   
                    }
                } elsif ($bytes->{$tt[0]}->{info} =~ /2nd byte/) { 
                    if (($bytes->{$tt[0]}->{info} =~ /SAS Expander/) && ($output->{$tt[0]} == 8)) {
                        logcomment("Element \($bytes->{$tt[0]}->{info}\) 2nd byte $tt[0] is 08, this is Master Module."); 
                    } elsif (($bytes->{$tt[0]}->{info} =~ /SAS Expander/) && ($output->{$tt[0]} == 00)) {
                        logcomment("Element \($bytes->{$tt[0]}->{info}\) 2nd byte $tt[0] is 00, this is Slave Module."); 
                    } elsif (($bytes->{$tt[0]}->{info} =~ /Ethernet/) && ($output->{$tt[0]} == 0))  { 
                        logcomment("Ethernet 2nd byte $tt[0] \($bytes->{$tt[0]}->{info}\) should be 01, but ses_statuspage 2 has value 0 which means Ethernet Lane Enabled is not enabled. This is fine for IOM6 module"); 
                    } elsif (($bytes->{$tt[0]}->{info} =~ /Ethernet/) && ($output->{$tt[0]} == 1))  { 
                        logcomment("Ethernet 2nd byte $tt[0] \($bytes->{$tt[0]}->{info}\) is 01, it means Ethernet Lane Enabled is enabled. This is a IOM6E module"); 
                    }
                } elsif ($bytes->{$tt[0]}->{info} =~ /4th byte/) { 
                    if (($bytes->{$tt[0]}->{info} =~ /SAS Expander/) && ($output->{$tt[0]} == 0)) {
                        logcomment("Element \($bytes->{$tt[0]}->{info}\) 4th byte $tt[0] is 00, there is no event logging data available."); 
                    } elsif (($bytes->{$tt[0]}->{info} =~ /SAS Expander/) && ($output->{$tt[0]} == 80)) {
                        logcomment("Element \($bytes->{$tt[0]}->{info}\) 4th byte $tt[0] is 80, there is event logging data available."); 
                    }
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage 2 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 2 Data File."); 
                    $err = 1;
                }
            }
        }
    }

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage 2 \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    }
    return $err;
}
    
=head2 sesPage5_status
    my $ret = sesPage5_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

    This routine checks each bytes of page 5 and compare data with shelf firmware commands to check the consistency and correctness for the page.

    Example of returning status:
    my $status = sesPage5_status();
    print "Page 5 status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=item shelf_type

(Required) : shelf type to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage5_status {
    my (%opts) = validate(@_,{
        host => {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_type => {type=>SCALAR, default=>undef},
        AorB => {type=>SCALAR,default=>"A"},
        sespage5_data_file => {type=>SCALAR, default=>undef},
        }
    );

    my $pagenum = 5;
    my %hash = ();
    my $output;
    my $c;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$pagenum);

    logcomment("Basic SES Page 5 Test library routine");

    my $read_in = `cat $opts{sespage5_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $wwn;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $err = 0;
my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];

                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage 5 has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 5 Data File."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Summary Status/) {
                    logcomment("byte $tt[0] from ses_statuspage 5 has Summary Status $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 5 Data File. The INVOP, INFO, NON-CRIT, CRIT, and UNRECOV bits returned in normal status pages are mandatory. Please check shelf health condition."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage 5 has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 5 Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } elsif (($bytes->{$tt[0]}->{info} =~ /1st byte/) && 
                    ($bytes->{$tt[0]}->{info} =~ /Generation Code/)) { 
                    logcomment("FAIL: Generation Code 1st byte $tt[0] \($bytes->{$tt[0]}->{info}\) should be 0, but ses_statuspage 5 has value $output->{$tt[0]}."); 
                    $err = 1;
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage 5 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 5 Data File. Please check this element's health condition."); 
                    $err = 1;
                }
            }
        }
    }

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage 5 \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    }
    return $err;
}

=head2 sesPage7_status
    my $ret = sesPage7_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

    This routine checks each bytes of page 7 and compare data with shelf firmware commands to check the consistency and correctness for the page.

    Example of returning status:
    my $status = sesPage7_status();
    print "Page 7 status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage7_status {
    my (%opts) = validate(@_,{
        host=> {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef,optional=>1},
        AorB => {type=>SCALAR,default=>"A"},
        sespage7_data_file => {type=>SCALAR, default=>undef},
        }
    );
    my $pagenum = 7;
    my %hash = ();
    my $output;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$pagenum);

    logcomment("Basic SES Page 7 Test");

    my $read_in = `cat $opts{sespage7_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $wwn;
    my $fw_ver;
    my $shelf_id;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $err = 0;
my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];
                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage 7 has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 1 Data File."); 
                } elsif ($bytes->{$tt[0]}->{value} eq "x") {
                    logcomment("byte $tt[0] from ses_statuspage 7 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 7 Data File. Please check the element value manually."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage 7 has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 7 Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage 7 has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 7 Data File."); 
                    $err = 1;
                }
            }
        }
    }

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage 7 \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    }
    return $err;
}

=head2 sesPage0a_status
    my $ret = sesPage0a_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

    This routine checks each bytes of page 0a and compare data with shelf firmware commands to check the consistency and correctness for the page.

    Example of returning status:
    my $status = sesPage0a_status();
    print "Page 0a status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage0a_status {
    my (%opts) = validate(@_,{
        host => {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef,optional=>1},
        AorB => {type=>SCALAR,default=>"A"},
        sespage0a_data_file => {type=>SCALAR, default=>undef},
        }
    );
    my $pagenum = 0x0a;
    my %hash = ();
    my $output;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$pagenum);

    logcomment("Basic SES Page 0a Test");

    my $read_in = `cat $opts{sespage0a_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $wwn;
    my $fw_ver;
    my $shelf_id;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $err = 0;
my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];
                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage 0a has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 1 Data File."); 
                } elsif ($bytes->{$tt[0]}->{value} eq "x") {
                    logcomment("byte $tt[0] from ses_statuspage 0a has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0a Data File. Please check the element value manually."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage 0a has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0a Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } elsif ($bytes->{$tt[0]}->{info} =~ /Invalid\+Reserved\+EIP\+Protocol ID/) {
                    if ($output->{$tt[0]} =~ /96/) {
                        logcomment("byte $tt[0] from ses_statuspage 0a has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0a Data File. This is due to the device is Not Installed and not considered a failure. Please check the device manually."); 
                    }
                } elsif ($bytes->{$tt[0]}->{info} =~ /Reserved\+Attached Device Type\+Reserved/) {
                    if ($output->{$tt[0]} =~ /00/) {
                        logcomment("byte $tt[0] from ses_statuspage 0a has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0a Data File. This is due to the device is Not Installed and not considered a failure. Please check the device manually."); 
                    }
                } elsif ($bytes->{$tt[0]}->{info} =~ /SATA Port Selector\+Reserved\+SSP Target Port/) {
                    if ($output->{$tt[0]} =~ /00/) {
                        logcomment("byte $tt[0] from ses_statuspage 0a has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0a Data File. This is due to the device is Not Installed and not considered a failure. Please check the device manually."); 
                    }
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage 0a has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 0a Data File. Please check the element value manually."); 
                    $err = 1;
                }
            }
        }
    }

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage 0a \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    }
    return $err;
}

=head2 sesPage8081_status
    my $ret = sesPage8081_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

    This routine checks each bytes of page 8081 and compare data with shelf firmware commands to check the consistency and correctness for the page.

    Example of returning status:
    my $status = sesPage8081_status();
    print "Page 8081 status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage8081_status {
    my (%opts) = validate(@_,{
        host => {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef,optional=>1},
        AorB => {type=>SCALAR,default=>"A"},
        sespage8081_data_file => {type=>SCALAR, default=>undef},
        pagenum => {type=>SCALAR, default=>undef}
        }
    );
    my %hash = ();
    my $output;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$opts{pagenum});

    logcomment("Basic SES Page $opts{pagenum} Test");

    my $read_in = `cat $opts{sespage8081_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $wwn;
    my $fw_ver;
    my $shelf_id;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $err = 0;
my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];
                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage $opts{pagenum} has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 1 Data File."); 
                } elsif ($bytes->{$tt[0]}->{value} eq "x") {
                    logcomment("byte $tt[0] from ses_statuspage $opts{pagenum} has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. Please check the element value manually."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage $opts{pagenum}  has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } elsif ($bytes->{$tt[0]}->{info} =~ /Phy Status/) {
                    if ($output->{$tt[0]} =~ /21|60/) {
                        logcomment("byte $tt[0] from ses_statuspage $opts{pagenum} has phy status $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. This may be caused by the device is Not Installed or not Used, please check the device condition manually. This is not considered a failure"); 
                    } else {
                        logcomment("FAIL: byte $tt[0] from ses_statuspage $opts{pagenum} has phy status $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. This is not caused by the device is Not Installed neither, please check the device condition manually."); 
                        $err = 1;
                    }
                } elsif ($bytes->{$tt[0]}->{info} =~ /Negotiated Phy Link Rate/) {
                    if ($output->{$tt[0]} =~ /a7|97|07/) {
                        logcomment("byte $tt[0] from ses_statuspage $opts{pagenum} has phy status $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. This may be caused by the device is 6GB/3GB or Not Installed/Used, please check the device condition manually. This is not considered a failure"); 
                    } else {
                        logcomment("FAIL: byte $tt[0] from ses_statuspage $opts{pagenum} has phy status $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. This is not caused by the device is Not Installed neither, please check the device condition manually."); 
                        $err = 1;
                    }
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage $opts{pagenum} has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. Please check the error count manually."); 
                    $err = 1;
                }
            }
        }
    }

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage $opts{pagenum} \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    }
    return $err;
}

=head2 sesPage8283_status
    my $ret = sesPage8283_status(host =>$host, conmode=>"shelfcli",timeout => $timeout);

    This routine checks each bytes of page 8283 and compare data with shelf firmware commands to check the consistency and correctness for the page.

    Example of returning status:
    my $status = sesPage8283_status();
    print "Page 8283 status return is $status\n";

The following arguments are accepted:

=over

=item host

(Required) : Hostspec to shelf or filer.

=item conntype

(Optional) : connection type. Default "#rsh" if connmode is "inband" and "#console" if connmode is "shelfcli" or "gemtest"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item connmode 

(Optional) : Connection Mode. "shelfcli" if we are connecting directly through shelf cli. "inband" if we are issuing inband command via ontap. "gemtest" if we are using "gremtest" windows utility. Default is "shelfcli".

=item filermode

(Optional) : Filermode. Maintenance Mode or Normal Mode. Default is "normal".

=item adapter_id

(Optional) : Adapter id to which the shelf is attached to. Required only when connmode=>"inband"

=item shelf_id

(Optional) : shelf id to which we want to issue command to. Required only when connmode=>"inband"

=back

=cut

sub sesPage8283_status {
    my (%opts) = validate(@_,{
        host => {type=>OBJECT,isa=>'Hostrec'},
        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
        connmode => {type=>SCALAR,default=>"shelfcli",optional=>1},
        filermode => {type=>SCALAR,default=>"normal",optional=>1},
        gemconn => {type=>OBJECT,default=>undef,isa=>'Connectrec',optional=>1},
        timeout => {type=>SCALAR, default=>300,optional=>1},
        adapter_id => {type=>SCALAR, default=>undef,optional=>1},
        shelf_id => {type=>SCALAR, default=>undef,optional=>1},
        AorB => {type=>SCALAR,default=>"A"},
        sespage8283_data_file => {type=>SCALAR, default=>undef},
        pagenum => {type=>SCALAR, default=>undef}
        }
    );
    my %hash = ();
    my $output;
    my $cmd;
    my $id;

    $output = ses_statuspage(host=>$opts{host},conntype=>$opts{conntype},connmode=>$opts{connmode},timeout=>$opts{timeout},adapter_id=>$opts{adapter_id},shelf_id=>$opts{shelf_id},AorB=>$opts{AorB},filermode=>$opts{filermode},page_num=>$opts{pagenum});

    logcomment("Basic SES Page $opts{pagenum} Test");

    my $read_in = `cat $opts{sespage8283_data_file}`;
    my @bytes_temp = split('\n', $read_in);
    my $bytes;
    my @tt;
    my $wwn;
    my $fw_ver;
    my $shelf_id;
    my $page_len_from_file;
    my $page_len_from_shelf;
    my $page_len_err = 0;
    my $err = 0;
    my $byte = 0;
    foreach my $line (@bytes_temp) {
        if (($line =~ /^#/) || ($line =~ /^\n+|\r+/)) {
            print "skip comment line\n";
        } else {
            my @lines = split(/;|\r*\n/, $line);
            foreach my $line (@lines) {
                @tt = split(/:/,$line);
                $tt[0] = $byte;
                $byte++;
                $bytes->{$tt[0]}->{value} = $tt[1];
                $bytes->{$tt[0]}->{info} = $tt[2];
                if ($bytes->{$tt[0]}->{value} eq "$output->{$tt[0]}") {
                    logcomment("byte $tt[0] from ses_statuspage $opts{pagenum} has value $output->{$tt[0]} which is the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page 1 Data File."); 
                } elsif ($bytes->{$tt[0]}->{value} eq "x") {
                    logcomment("byte $tt[0] from ses_statuspage $opts{pagenum} has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. Please check the element value manually."); 
                } elsif ($bytes->{$tt[0]}->{info} =~ /Page Length/) {
                    logcomment("byte $tt[0] from ses_statuspage $opts{pagenum}  has page length value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File."); 
                    $page_len_from_file = $page_len_from_file.$bytes->{$tt[0]}->{value};
                    $page_len_from_shelf = $page_len_from_shelf.$output->{$tt[0]};
                    $page_len_err = 1;
                } else {
                    logcomment("FAIL: byte $tt[0] from ses_statuspage $opts{pagenum} has value $output->{$tt[0]} which is not the same as the value $bytes->{$tt[0]}->{value} \($bytes->{$tt[0]}->{info}\) from SES Page $opts{pagenum} Data File. Please check the error count manually."); 
                    $err = 1;
                }
            }
        }
    }

    if ($page_len_err == 1) {
        logcomment("FAIL: Page Length \($page_len_from_file\) from Data file is different from the ses_statuspage $opts{pagenum} \($page_len_from_shelf\), exit the rest of page checking...");
        $err = 1;
    }
    return $err;
}


=head1 SEE ALSO

    Where additional information can be found.

=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = none

    ONTAP_ZAPI_APIS = none

=head1 AUTHOR

    Vickie Yang (vickie@netapp.com)
    Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

    Copyright 2012, Network Appliance Inc., All Rights Reserved.

=cut

1;
