# Library with storage ESH related functions.
## @summary Library of routines for ESH related tests.
##
## @description
##  Library of routines for ESH related tests.
##
## @synopsis
##
## use Storage::ShelfUtils;
##
## @status TESTME
##
## @author dl-sar-qa@netapp.com, dl-step@netapp.com
##
####################

package Storage::ShelfUtils;
use strict;
require Exporter;
use vars ('@ISA', '@EXPORT_OK');
use NACL::APISet;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::STask::Node;
@ISA = ('Exporter');
@EXPORT_OK = qw ( diskbypass diskbypass_maint get_esh_disks get_esh_nonses_disks get_bypassdisklist get_activesesdisks get_spares get_all_disks get_shelfbay get_shelftype get_ses_disks get_ses_spares get_ses_inuse );
use Tharn;
my $timeout = 600;


########## Subroutine to bypass/unbypass a disk
## @name ::diskbypass
## @summary Bypasses/unbypasses an ESH drive port.
##
## @synopsis
##   diskbypass($filer,$error,$disk_id,$portAorB)
## 
## @arg $filer Required: Filer connection to use
## @arg $error Required: Error type to inject for bypassing drives.Following types of errors can be injected:
## 			 "0x02" => "FORCE BYPASS". 
##		  	 "0xa0" => "Clock Delta Fault Bypass".
##		  	 "0xa1" => "CRC Fault Bypass".
##		  	 "0xa2" => "WRD Fault Bypass".
##		   	 "0xa3" => "Increment CRC error +1".
##		  	 "0xa4" => "Increment CRC error +2".
##		  	 "0xa5" => "Increment WRD error +1000".
##
## @arg $disk_id Required: disk id to be bypassed.
## @arg $portAorB optional: To bypass port A or port B of the drive. "A" or "B", if not defined that will bypass/unbypass both ports
## @status TESTME
## @change 03-07-2005 anishv: Add ESH2 support for diskbypass
## @change 04-29-2005 skg: Added support to bypass/unbypass both port and removed debug build check
## @change 06-02-06 erickson: modified diskbypass to accept a list of diskid's and loop through them
## @change 19-03-2013 villas: modified sub diskbypass get_esh_disks get_esh_nonses_disks get_bypassdisklist get_activesesdisks get_ses_spares
##                             to support cmode as well as 7mode

sub diskbypass {

	my ( $a , $b , $c , $filer, $error , $diskidraw_list, $portAorB , $i , $diskwwn, %wwnmap, %portid);

	($filer ,$error,$diskidraw_list,$portAorB)= @_ ;

#	#### Make sure that it's debug build
#	say($filer, "", -swallow);
#	sleep 5;
#	($a,$b,$c) = say($filer, "version",-timeout ,$timeout);
#	my @version = split(/\r*\n/,$c);
#	if ( $version[0]  !~ /d/) {
#		tstlog ("-config" ,"Disks can be bypassed only on debug builds");
#		}
        #Using api set obj to support cmode also
        my $host = $filer->hostp();
        my $mode = '7Mode';
        $mode = 'Nodescope' if ($host->{hosttype} =~ /filer\-ng/);
        my $api_set_obj = NACL::APISet->new(
            hostobj   => $host,
            category  => 'Node',
            interface => 'CLI',
            set       => $mode,
        );
        $api_set_obj->set_timeout('connectrec-timeout' => $timeout);
        $c = $api_set_obj->execute_command(command => "priv set -q advanced;storage show hub",);
        sleep 30;
	my @hub = split(/\r?\n/,$c);

	#### First create an array of adapters connected to hub

	my @channels = grep /Channel/, @hub;
	my @channel1;
	my $n =0;
	while ( $n <= $#channels) {
		my @hub1 = split(/\s+/,$channels[$n]);
	 	$channel1[$n] = $hub1[1];
		$n++;
	}
        sleep 30;
	#### Create an array of  shelf ids of disks

	$n = 0;
	my( @shelf1 , @shelves, @shelf );
	@shelves = grep /Shelf id:/, @hub;
	while ($n <= $#shelves) {
		@shelf = split(/\s+/,$shelves[$n]);
		$shelf1[$n] = $shelf[2];
		$n++
	}
        sleep 30;
	#### Create an array of wwn nos.

	$n =0;
	my ( @wwn , @wwn2 , @wwn1);
	@wwn = grep /UID/ , @hub;
	while ( $n <= $#wwn) {
		@wwn2 = split(/\s+/,$wwn[$n]);
		$wwn1[$n] = $wwn2[2];
		$n++;
	}
        sleep 30;
	#### Create a hash table of adapters , shelves and wwns

	$i =0;
	my ( %wwnmap1 ,$adshelf ,@wwnmap); 
	while ( $i <= $#channel1 ) {
		$adshelf = $channel1[$i].".".$shelf1[$i] ;
		%wwnmap = ($adshelf, $wwn1[$i]);
		%wwnmap1 = (%wwnmap1,%wwnmap);
		$i++;
	}

        sleep 30;
    foreach my $diskidraw (split " ", $diskidraw_list){
	my ( @diskid , $diskid , $adapid ,$shelfid);
	@diskid = split(/\./ , $diskidraw);
	$diskid = $diskid[1];
	$adapid = $diskid[0];
	if ( $diskid <= 13) {
		$shelfid = 0;
		}
	elsif( $diskid >=16 && $diskid <=29) {
		$shelfid =1;
		}
	elsif( $diskid >=32 && $diskid <= 45) {
		$shelfid = 2;
		}
	elsif( $diskid >=48 && $diskid <=61 ) {
		$shelfid = 3;
		}
	elsif( $diskid >=64 && $diskid <=77) {
		$shelfid = 4;
		}
	elsif( $diskid >=80 && $diskid <=93 ) {
		$shelfid = 5;
		}
	elsif( $diskid >=96 && $diskid <=109) {
		$shelfid = 6;
		}
	else { $shelfid = 7;}

	my $diskkey = "$adapid" . "." . "$shelfid";
        sleep 30;
	### Check for ESH or ESH2
	

        $c = $api_set_obj->execute_command(command => "priv set -q advanced;environ shelf $adapid",);
        sleep 30;
	my @eshtype = split(/\r?\n/,$c);
	my $tt = "no";
	my $eshtype;
	my $eshtype1;
	foreach $eshtype (@eshtype){
		if($eshtype =~  /Shelf:\s+$shelfid/){
			$tt = "yes";
		}
		if(($eshtype  =~ /Module type/) && ($tt eq "yes")){
			$eshtype1 = "$eshtype";
#			goto LABEL ;
                        last;
		}
	}




#        LABEL :	my $diskwwn = 0;
        $diskwwn = 0;



	$diskwwn = $wwnmap1{$diskkey};

 	my  $j ;

	for ( $i = 0 ; $i <=13 ; $i++) {
		$j = $i;
		while ( $j <= 112 + $i){

			if($eshtype1 =~ /ESH;/){
				$portid{$j} = $i + 2;
			}

			if($eshtype1 =~ /ESH2/){
				$portid{$j} = $i + 4;
			}
#MTK_EDIT: Added the below code for ESH4 model type shelfs
                        if($eshtype1 =~ /ESH4/){
                                $portid{$j} = $i + 4;
                        }

			$j = $j + 16;

			}
		}
	my $port = $portid{$diskid};
        sleep 30;
	if ( $diskwwn eq '') {
	#	tstlog("-script","printing here: $diskidraw is not a hub disk");
return 0;
		}
	else {

	#print "WWN for disk: $diskidraw is $diskwwn and it's port no is 

	   if ($error eq "0x02") {
#MTK_EDIT: Chaned the prompt string from "Manually bypassed" to "DBG4: PORT CONTROL request port: $port"
#MTK_EDIT: Chaned the prompt string from "" to "fcadmin: hub request in progress to shelf UID: 50:05:0c:c0:02:10:60:ff"
		if ( $portAorB eq "") {
		        $c = $api_set_obj->execute_command(command => "priv set -q advanced;fcadmin hub pctrl -w $diskwwn -p $port -v $error -l A",);
                        sleep 30;
                        die 'Not getting expected string "fcadmin: hub request in progress to shelf"' if ($c !~ /fcadmin: hub request in progress to shelf UID: $diskwwn/);
                        $c = $api_set_obj->execute_command(command => "priv set -q advanced;fcadmin hub pctrl -w $diskwwn -p $port -v $error -l B",);
                        sleep 30;
                        die 'Not getting expected string "fcadmin: hub request in progress to shelf"' if ($c !~ /fcadmin: hub request in progress to shelf UID: $diskwwn/);
		} else {
                        $c = $api_set_obj->execute_command(command => "priv set -q advanced;fcadmin hub pctrl -w $diskwwn -p $port -v $error -l $portAorB",);
                        sleep 30;
                        die 'Not getting expected string "fcadmin: hub request in progress to shelf"' if ($c !~ /fcadmin: hub request in progress to shelf UID: $diskwwn/);
		}

	   } else {
		if ( $portAorB eq "") {
                        $c = $api_set_obj->execute_command(command => "priv set -q advanced;fcadmin hub pctrl -w $diskwwn -p $port -v $error -l A",);
                        sleep 30;
                        $c = $api_set_obj->execute_command(command => "priv set -q advanced;fcadmin hub pctrl -w $diskwwn -p $port -v $error -l B",);
                } else {
                        say($filer,"priv set -q advanced;fcadmin hub pctrl -w $diskwwn -p $port -v $error -l $portAorB" , '-wait', '\n',"-timeout" ,"$timeout");
                        $c = $api_set_obj->execute_command(command => "priv set -q advanced;fcadmin hub pctrl -w $diskwwn -p $port -v $error -l $portAorB",);
                }
	   }
	}
	sleep 30;
    }
    return (1);

}

########## Subroutine to bypass/unbypass a disk in maintenance mode
## @name ::diskbypass_maint
## @summary Bypasses/unbypass an ESH drive port.
##
## @synopsis
##   diskbypass_maint($filer,$error,$disk_id,$portAorB)
## 
## @arg $filer Required: Filer connection to use
## @arg $error Required: Error type to inject for bypassing drives.Following types of errors can be injected:
## 			 "0x02" => "FORCE BYPASS". 
##		  	 "0xa0" => "Clock Delta Fault Bypass".
##		  	 "0xa1" => "CRC Fault Bypass".
##		  	 "0xa2" => "WRD Fault Bypass".
##		   	 "0xa3" => "Increment CRC error +1".
##		  	 "0xa4" => "Increment CRC error +2".
##		  	 "0xa5" => "Increment WRD error +1000".
##
## @arg $disk_id Required: disk id to be bypassed.
## @arg $portAorB optional: To bypass port A or port B of the drive. "A" or "B", if not defined will bypass both port
## @status TESTME
 


sub diskbypass_maint {

	my ( $a , $b , $c , $filer, $error , $diskidraw, $portAorB , $i , %wwnmap, %portid);

	($filer, $error, $diskidraw, $portAorB)= @_ ;

#	#### Make sure that it's debug build
#	say($filer, "", -swallow);
#	sleep 5;
#	($a,$b,$c) = say($filer, "version",-timeout ,$timeout);
#	my @version = split(/\r*\n/,$c);
#	if ( $version[0]  !~ /d/) {
#		tstlog ("-config" ,"Disks can be bypassed only on debug builds");
#		}

#MTK_EDIT: Added the timeout parameter to below say function
	($a, $b, $c)=	say($filer, "storage show hub",'-prompt', "\\*>", '-wait', '\n', -timeout , $timeout);
	my @hub = split(/\r?\n/,$c);

	#### First create an array of adapters connected to hub

	my @channels = grep /Channel:/, @hub;
	my @channel1;
	my $n =0;
	while ( $n <= $#channels) {
		my @hub1 = split(/\s+/,$channels[$n]);
	 	$channel1[$n] = $hub1[1];
		$n++;
	}
	print "here is the list of channels: @channel1 \n";

	#### Create an array of  shelf ids of disks

	$n = 0;
	my( @shelf1 , @shelves, @shelf );
	@shelves = grep /Shelf id:/, @hub;
	while ($n <= $#shelves) {
		@shelf = split(/\s+/,$shelves[$n]);
		$shelf1[$n] = $shelf[2];
		$n++
	}
	print "here is the list of shelves: @shelf1 \n";

	#### Create an array of wwn nos.

	$n =0;
	my ( @wwn , @wwn2 , @wwn1);
	@wwn = grep /UID/ , @hub;
	while ( $n <= $#wwn) {
		@wwn2 = split(/\s+/,$wwn[$n]);
		$wwn1[$n] = $wwn2[2];
		$n++;
	}
	print "here is the list of wwn: @wwn1 \n";

	#### Create a hash table of adapters , shelves and wwns

	$i =0;
	my ( %wwnmap1 ,$adshelf ,@wwnmap); 
	while ( $i <= $#channel1 ) {
		$adshelf = $channel1[$i].".".$shelf1[$i] ;
		%wwnmap = ($adshelf, $wwn1[$i]);
		%wwnmap1 = (%wwnmap1,%wwnmap);
		$i++;
	}


	my ( @diskid , $diskid , $adapid ,$shelfid);
	@diskid = split(/\./ , $diskidraw);
	$diskid = $diskid[1];
	$adapid = $diskid[0];
	if ( $diskid <= 13) {
		$shelfid = 0;
		}
	elsif( $diskid >=16 && $diskid <=29) {
		$shelfid =1;
		}
	elsif( $diskid >=32 && $diskid <= 45) {
		$shelfid = 2;
		}
	elsif( $diskid >=48 && $diskid <=61 ) {
		$shelfid = 3;
		}
	elsif( $diskid >=64 && $diskid <=77) {
		$shelfid = 4;
		}
	elsif( $diskid >=80 && $diskid <=93 ) {
		$shelfid = 5;
		}
	elsif( $diskid >=96 && $diskid <=109) {
		$shelfid = 6;
		}
	else { $shelfid = 7;}
	print "here is adapter id: $adapid \n";
	print "here is shelf id: $shelfid \n";

	my $diskkey = "$adapid" . "." . "$shelfid";

	### Check for ESH or ESH2
	

	(undef,undef,$c) =say($filer,"environ shelf $adapid",'-prompt', "\\*>", '-wait', '\n');
	my @eshtype = split(/\r?\n/,$c);
	my $tt = "no";
	my $eshtype;
	my $eshtype1;
	foreach $eshtype (@eshtype){
		if($eshtype =~  /Shelf:\s+$shelfid/){
			$tt = "yes";
		}
		if(($eshtype  =~ /Module type/) && ($tt eq "yes")){
			$eshtype1 = "$eshtype";
			goto LABEL ;
		}
	}
	print "here is esh type : $eshtype1 \n";




        LABEL :	my $diskwwn = 0;

	$diskwwn = $wwnmap1{$diskkey};

 	my  $i ;
 	my  $j ;

	for ( $i = 0 ; $i <=13 ; $i++) {
		$j = $i;
		while ( $j <= 112 + $i){

			if($eshtype1 =~ /ESH;/){
				$portid{$j} = $i + 2;
			}

			if($eshtype1 =~ /ESH2/){
				$portid{$j} = $i + 4;
			}
                        if($eshtype1 =~ /ESH4/){
                                $portid{$j} = $i + 4;
                        }

			$j = $j + 16;

			}
		}

	if ( $diskwwn eq '') {
		tstlog("-script","$diskidraw is not a hub disk");
		}
	else {

	#print "WWN for disk: $diskidraw is $diskwwn and it's port no is 

		if ( $portAorB eq "") {
			say($filer,"fcadmin hub pctrl -w $diskwwn -p $portid{$diskid} -v $error -l A" ,'-prompt', "\\*>", '-wait', '\n');
			sleep 30;
			say($filer,"fcadmin hub pctrl -w $diskwwn -p $portid{$diskid} -v $error -l B" ,'-prompt', "\\*>", '-wait', '\n');

		} else {
			say($filer,"fcadmin hub pctrl -w $diskwwn -p $portid{$diskid} -v $error -l $portAorB" ,'-prompt', "\\*>", '-wait', '\n');
		}
	}
	return (1);

}

########### End of subroutine


#### Subroutine to get a list of hub disks
## @name ::get_esh_disks
## @summary Get a list of drives in the ESH shelves.
## @synopsis
##	get_esh_disks($filer,$raidtype)
##
## @arg $filer Required: Filer conenction to use
## @arg $raidtype Optional: By default returns all the esh drives. 
##			  Returns spare esh drives if $raidtype is "spare".
##			  Returns data/parity esh drives if $raidtype is "data".
##			  Returns failed esh drives if $raidtype is "failed".

##
## @status TESTME

sub get_esh_disks {

	my ( $a , $b , $c , $filer, $diskidraw ,$i , $j ,  $raidtype , @diskarray);
	($filer , $raidtype)= @_ ;
#$diskidraw = "bes5:5.17";
	unless (defined $raidtype) { $raidtype = "all";}
        my $host = $filer->hostp();
        my $mode = '7Mode';
        $mode = 'Nodescope' if ($host->{hosttype} =~ /filer\-ng/);
        my $api_set_obj = NACL::APISet->new(
            hostobj   => $host,
            category  => 'Node',
            interface => 'CLI',
            set       => $mode,
        );
        $api_set_obj->set_timeout('connectrec-timeout' => $timeout);
        $c = $api_set_obj->execute_command(command => "vol status -r",);
	my @raiddisks = split(/\r*\n/,$c);
	my @sparedisks1 = grep /spare/ , @raiddisks;
	my @datadisks1 = grep /data|parity/ , @raiddisks;
	my @faileddisks1 = grep /failed|admin failed|admin removed/ , @raiddisks;
	my @partnerdisks1 = grep /partner/ , @raiddisks;
	my @raiddisks1;


	##### Get an array of Spare disks 
	if (($raidtype eq "spare") || ($raidtype eq "all")) {

		$j =0;
		$i =0;
		my @sparedisks;
		while ($j <= $#sparedisks1) {
	 		@raiddisks1 = split(/\s+/,$sparedisks1[$j]) ;
           		if ( $raiddisks1[1] !~ /^spare$/) {
	   			$sparedisks[$i] = $raiddisks1[1];
				$i++;
				}
			$j++;
			}
		#print "\n\n\n SPARES : @sparedisks \n\n\n";
		push( @diskarray , @sparedisks);

	}

	if (($raidtype eq "data") || ($raidtype eq "all")) {
		 
	##### Get an array of Data or Parity disks

		$j =0;
		my @datadisks;
		while ($j <= $#datadisks1) {
	 		@raiddisks1 = split(/\s+/,$datadisks1[$j]) ;
	   		$datadisks[$j] = $raiddisks1[2];
			$j++;
		}
	#print "\n\n\n DATA : @datadisks \n\n\n";
		push ( @diskarray , @datadisks);
	}

	if(($raidtype eq "failed") || ($raidtype eq "all")) {

	##### Get an array of failed  disks

	$j =0;
	my @faileddisks;
	while ($j <= $#faileddisks1) {
		 @raiddisks1 = split(/\s+/,$faileddisks1[$j]) ;
	   	if ($raiddisks1[0] =~ /^failed$/) {
	   		$faileddisks[$j] = $raiddisks1[1];
		}
	   	else {
	  		 $faileddisks[$j] = $raiddisks1[2];
		}
			
		$j++;
	}
	#print "\n\n\n FAILED : @faileddisks \n\n\n";
		push ( @diskarray , @faileddisks);
	}

##### Get an array of partner  disks

	if (($raidtype eq "partner") || ($raidtype eq "all")) {

		$j =0;
		my @partnerdisks;
		while ($j <= $#partnerdisks1) {
			 @raiddisks1 = split(/\s+/,$partnerdisks1[$j]) ;
	   		 $partnerdisks[$j] = $raiddisks1[1];
		 	$j++;
		}
	#print "\n\n\n PARTNER : @partnerdisks \n\n\n";
		push(@diskarray , @partnerdisks);
	}

	#print "\n\n\n\n TOTAL @diskarray \n\n\n\n";
        $c = $api_set_obj->execute_command(command => "priv set -q advanced;storage show hub",);
	my @hub = split(/\r*\n/,$c);

	#print "\n\n\n @hub \n\n\n";

	#### First create an array of adapters connected to hub

	my @channels = grep /Channel/, @hub;
	my @channel1;
	my $n =0;
	while ( $n <= $#channels) {
		my @hub1 = split(/\s+/,$channels[$n]);
	 	$channel1[$n] = $hub1[1];
		$n++;
	}

	#### Create an array of  shelf ids of disks

	$n = 0;
	my( @shelf1 , @shelves, @shelf );
	@shelves = grep /Shelf id:/, @hub;
	while ($n <= $#shelves) {
		@shelf = split(/\s+/,$shelves[$n]);
		$shelf1[$n] = $shelf[2];
		$n++
	}

	#### Create an array of wwn nos.

	$n =0;
	my ( @wwn , @wwn2 , @wwn1);
	@wwn = grep /UID/ , @hub;
	while ( $n <= $#wwn) {
		@wwn2 = split(/\s+/,$wwn[$n]);
		$wwn1[$n] = $wwn2[2];
		$n++;
	}

	#### Create a hash table of adapters , shelves and wwns

	$i =0;
	my ( %wwnmap1 ,$adshelf ,@wwnmap , %wwnmap, %portid); 
	#%wwnmap1 = {};
	while ( $i <= $#channel1 ) {
		$adshelf = $channel1[$i].".".$shelf1[$i] ;
		%wwnmap = ($adshelf, $wwn1[$i]);
		%wwnmap1 = (%wwnmap1,%wwnmap);
		$i++;
	}


	my ( @diskid , $diskid , $adapid ,$shelfid , $k , @eshdisks);
	foreach $diskidraw (@diskarray) {
		@diskid = split(/\./ , $diskidraw);
		$diskid = $diskid[1];
		$adapid = $diskid[0];
		if ( $diskid <= 13) {
			$shelfid = 0;
		}
		elsif( $diskid >=16 && $diskid <=29) {
			$shelfid =1;
		}
		elsif( $diskid >=32 && $diskid <= 45) {
			$shelfid = 2;
		}
		elsif( $diskid >=48 && $diskid <=61 ) {
			$shelfid = 3;
		}
		elsif( $diskid >=64 && $diskid <=77) {
			$shelfid = 4;
		}
		elsif( $diskid >=80 && $diskid <=93 ) {
			$shelfid = 5;
		}
		elsif( $diskid >=96 && $diskid <=109) {
			$shelfid = 6;
		}
		else { $shelfid = 7;}

		my $diskkey = "$adapid" . "." . "$shelfid";
		my $diskwwn = 0;

		$diskwwn = $wwnmap1{$diskkey};

 		my  $j ;

		for ( $i = 0 ; $i <=13 ; $i++) {
			$j = $i;
			while ( $j <= 112 + $i){
				$portid{$j} = $i + 2;
				$j = $j + 16;
			}
		}

		unless ( defined($diskwwn))  {
		#	tstlog("-script","$diskidraw is not a hub disk");
		}
		else {
			$eshdisks[$k++] = $diskidraw ;
		}

	}
	#print "\n\n\n ESHDISKS : @eshdisks \n TOTAL disks : $#eshdisks \n\n";
	return (@eshdisks);


}
## @name ::get_esh_nonses_disks
## @summary To get an array of non-ses drives in the  ESH shelves.
## @synopsis
##	get_esh_nonses_disks($filer);
##
## @arg $filer Required: Filer connection to use
## @status TESTME
sub get_esh_nonses_disks {

	my ($filer,$ses,$raidtype) = @_ ;
	unless (defined($ses)) { $ses = "nonses";} 
	unless (defined $raidtype) { $raidtype = "all";}
	my @eshdisks = get_esh_disks($filer,"$raidtype"); 

	#### First create an array of adapters connected to hub
	my($a , $b , $c);
        my $host = $filer->hostp();
        my $mode = '7Mode';
        $mode = 'Nodescope' if ($host->{hosttype} =~ /filer\-ng/);
        my $api_set_obj = NACL::APISet->new(
            hostobj   => $host,
            category  => 'Node',
            interface => 'CLI',
            set       => $mode,
        );
        $api_set_obj->set_timeout('connectrec-timeout' => $timeout);
        $c = $api_set_obj->execute_command(command => "priv set -q advanced;storage show hub",);
	my @hub = split(/\r*\n/,$c);
	my @channels = grep /Channel/, @hub;
	my @channel1;
	my $n =0;
	my @hub1;
	while ( $n <= $#channels) {
       		  @hub1 = split(/\s+/,$channels[$n]);
         	  $channel1[$n] = $hub1[1];
        	  $n++;
        }

	my $i =0;
	my @sesdrives;
	my @sesdrives1;

	####Get an array of  possible SES drives.
	$i = 0;
	while ( $i <= $#channel1){

		@sesdrives = ("$channel1[$i]". "." . "0" , "$channel1[$i]". "." . "1" , "$channel1[$i]". "." . "16" , "$channel1[$i]". "." . "17" , "$channel1[$i]". "." . "32" , "$channel1[$i]". "." . "33" , "$channel1[$i]". "." . "48" , "$channel1[$i]". "." . "49" , "$channel1[$i]". "." . "64" , "$channel1[$i]". "." . "65" , "$channel1[$i]". "." . "80" , "$channel1[$i]". "." . "81" , "$channel1[$i]". "." . "96" , "$channel1[$i]". "." . "97", "$channel1[$i]". "." . "112" , "$channel1[$i]". "." . "113"); 

		$i++;
		push @sesdrives1 , @sesdrives ;
	}

	$i =0;
	my $hspare;
	my @esh_ses_disks;

	if ( $ses eq "ses") {

		foreach $hspare (@eshdisks) {
			if ( grep  /^$hspare$/, @sesdrives1) {
				$esh_ses_disks[$i] = $hspare;
				$i++;
			}
		}
	}
	else {

		foreach $hspare (@eshdisks) {
			unless ( grep  /^$hspare$/, @sesdrives1) {
				$esh_ses_disks[$i] = $hspare;
				$i++;
			}
		}
	}
	return (@esh_ses_disks);

}

## @name ::get_bypassdisklist
## @summary Returns an array of bypassed ESH drives in the system.
## @synopsis
##	get_bypassdisklist($filer);
## @arg $filer Required: Filer connection to use
## @status TESTME
	
sub get_bypassdisklist {

	my ($filer, $c ,@adap1 , $findad , @adap2 , @adap3 , $abcd , $i, @adap  , $keys);
	$filer = $_[0];
        my $host = $filer->hostp();
        my $mode = '7Mode';
        $mode = 'Nodescope' if ($host->{hosttype} =~ /filer\-ng/);
        my $api_set_obj = NACL::APISet->new(
            hostobj   => $host,
            category  => 'Node',
            interface => 'CLI',
            set       => $mode,
        );
        $api_set_obj->set_timeout('connectrec-timeout' => $timeout);
        $c = $api_set_obj->execute_command(command => "fcadmin device_map",);
	@adap1 = split(/\r*\n/ , $c);
	$i =0;
	foreach $findad (@adap1){
		if ( !($findad =~ /failed/) &&   ($findad =~ /channel|adapter/)){
			$adap2[$i] = $findad;
			$i++;
                }
        }
     	   $i =0;
           foreach $abcd (@adap2){
                        @adap3 = split(/\s+/,$abcd);
                        $adap[$i] = $adap3[4];
                        $adap[$i] =~ s/:$//;
                        $i++;
           }
        # print " Active adapters in the system : @adap \n";
	
	my ($adapter , $adapter1 , @loop_map , %diskmap , $diskmap , $shelf , $j);	
	foreach $adapter (@adap) {

                $c = $api_set_obj->execute_command(command => "fcadmin device_map $adapter",);
		@adap1 = split(/\r*\n/ , $c);

		foreach $adapter1 (@adap1) {
			if( $adapter1 =~ /.Shelf/) {
				@loop_map = split(" ",$adapter1);
				chop($loop_map[1]);
				#print "\n\n\n LOOP MAP : $loop_map[1] \n\n\n";
				$shelf = $loop_map[1];	
				#@loop_map = reverse @loop_map;
				
#				print "\n\n\n LOOP MAP1 : @loop_map \n\n\n";
				$j = $#loop_map;
				for ($i = 0; $i <= 13 ; $i++) {
					$diskmap = $adapter . "." . (($shelf * 16) + $i);
					#print "\n\n\n DISKMAP : $diskmap \n\n\n";
					$diskmap{$diskmap} = $loop_map[$j--];
					}
				}
		}
	}	
	#my $diskdisk;
	#foreach $diskdisk (keys %diskmap){
	#	print "\n\n $diskdisk => $diskmap{$diskdisk} \n\n";
	#		}
	return (%diskmap);
}	

## @name ::get_activesesdisks
## @summary Returns a list of active SES disks in the system.
## @synopsis
##	get_activesesdisks($filer, $adapter)
##
## @arg $filer Required: Filer connection to use
## @arg $adapter Optional: Adapter for which to get active SES disks
## @status TESTME

sub get_activesesdisks($;$){

	my ($adapter, $c, @c , $d, @d , $filer, @activeses, $i);
	$filer = $_[0];
        $adapter = $_[1];
        my $host = $filer->hostp();
        my $mode = '7Mode';
        $mode = 'Nodescope' if ($host->{hosttype} =~ /filer\-ng/);
        my $api_set_obj = NACL::APISet->new(
            hostobj   => $host,
            category  => 'Node',
            interface => 'CLI',
            set       => $mode,
        );
        $api_set_obj->set_timeout('connectrec-timeout' => $timeout);
        if (defined($adapter)) {
            $c = $api_set_obj->execute_command(command => "environ shelf $adapter",);
        } else {
            $c = $api_set_obj->execute_command(command => "environ shelf",);
        }
        sleep 30;
	@c = split(/\r*\n/,$c);
	foreach $d (@c) {
		if ( $d =~ /SES device path/){
			@d = split(/\s+/,$d);
			#if(@activeses !~ /^$d[6]$/){
				$activeses[$i++] = $d[6];
			#	}
	
			}

		}
#print "\n\n\n ACTIVE SES DISKS => @activeses  $#activeses\n\n\n";
return (@activeses);
}

## @name ::get_shelftype
## @summary Returns a list of shelves and the types
## @synopsis
##      get_shelftype($filer)
##
## @arg $filer Required: Filer connection to use
## @status TESTME
sub get_shelftype($) {

       my ($c, @c, $current_shelf, $d, @d, $elem, $filer, %shelf) ;
       $filer = $_[0];
       %shelf = ();
       (undef, undef, $c) = say($filer, "environ shelf",-timeout ,$timeout);
       @c = split(/\r*\n/,$c);
       foreach $d (@c) {
           if ($d =~ /Shelf:\s+(.*)$/) {
               $current_shelf = $1;
           } elsif ($d =~ /Module\s+type\:\s+(.*);/) {
               if (!defined($current_shelf)) {
                   croak("Module type came before shelf id");
               }
               $shelf{$current_shelf} = { 'type' => $1 };
           } elsif ($d =~ /Shelf\s+status\:/) {
               # Defensive incase the output lines are moved around
               $current_shelf = undef;
           }
       }
       return (\%shelf);      
}

## @name ::get_spares()

## @summary Get an array of all spares on the Filer

## @synopsis
##   get_spares($spec)

## @arg spec Required: hostspec of the filer

## @status in-progress

sub get_spares( $$% ) {
    my ($conn, %args) = @_;
    my $raid_object = new RaidInfo($conn);
    my %spare_disk_hash = $raid_object->spare_disks();
    my @spare_disk_list = keys(%spare_disk_hash);

    return @spare_disk_list; 
}


## @name ::get_all_disks()

## @summary Get an array of all disks on the Filer

## @synopsis
##   get_all_disks($spec)

## @arg spec Required: hostspec of the filer

## @status in-progress

sub get_all_disks( $$% ) {
    my ($conn, %args) = @_;
    my $raid_object = new RaidInfo($conn);
    my %all_disks_hash = $raid_object->all_disks();
    my @all_disks_list = keys(%all_disks_hash);

    return @all_disks_list;
}



## @name ::get_shelfbay()

## @summary Get an array of the Shelf Bay Locations of all disks

## @synopsis
##   get_shelfbay($spec)

## @arg spec Required: hostspec of the filer
## @status in-progress

sub get_shelfbay( $$% ) {
    my ($conn, %args) = @_;
    my $raid_object = new RaidInfo($conn);
    my %all_disks_hash = $raid_object->all_disks();
    my @all_disks_list = keys(%all_disks_hash);
    my $tot_num_disks = scalar( @all_disks_list );
    my @disk_bay_param;
    my $disk_object;
    my $count = 0;

        for( $count = 0 ; $count < $tot_num_disks ; ++$count ) {
            $disk_object = $raid_object->disk( "$all_disks_list[$count]" );
            $disk_bay_param[$count] = $disk_object->parameter( "bay" );
        }
    return @disk_bay_param;
}


## @name ::get_ses_disks()

## @summary Get an array of all SES  disks on the Filer (returns any disk in Shelf bay 0 || 1 )

## @synopsis
##   get_ses_disks($spec)

## @arg spec Required: hostspec of the filer
## @status in-progress

sub get_ses_disks {
    my ($conn) = @_;
    my $raid_object = new RaidInfo($conn);
    my %all_disks_hash = $raid_object->all_disks();
    my @all_disks_list = keys(%all_disks_hash);
    my $tot_num_disks = scalar( @all_disks_list );
    my @disk_bay_param;
    my $disk_object;
    my @ses_disks_list;
    my $ses_count = 0;
    my $count = 0;

    for( $count = 0 ; $count < $tot_num_disks ; ++$count ) {
        $disk_object = $raid_object->disk( "$all_disks_list[$count]" );
        $disk_bay_param[$count] = $disk_object->parameter( "bay" );
#        logcomment( "Disk: $all_disks_list[$count] is in bay: $disk_bay_param[$count]");
               
        if ( ($disk_bay_param[$count] == 0) || ($disk_bay_param[$count] == 1) ) {
	    $ses_disks_list[$ses_count] = $all_disks_list[$count];
            ++$ses_count;
        }
    }

    return @ses_disks_list;


}

## @name ::get_ses_spares()

## @summary Get an array of all SES disks which are spares

## @synopsis
##   get_ses_spares($spec)

## @arg spec Required: hostspec of the filer
## @status in-progress

sub get_ses_spares {
    my ($conn) = @_;
    my $host = $conn->hostp();
    my $mode = '7Mode';
    $mode = 'CMode' if ($host->{hosttype} =~ /filer-ng/);
    my $api_set_obj = NACL::APISet->new(
             hostobj   => $host,
             category  => 'Node',
             interface => 'CLI',
             set       => $mode,
    );
    my $raid_object = new RaidInfo($conn, 'api_set_obj' => $api_set_obj);
    my %all_disks_hash = $raid_object->all_disks();
    my @all_disks_list = keys(%all_disks_hash);
    my $tot_num_disks = scalar( @all_disks_list );
    my @disk_bay_param;
    my $disk_object;
    my @ses_disks_list;
    my $ses_count = 0;
    my $count = 0;

    for( $count = 0 ; $count < $tot_num_disks ; ++$count ) {
        $disk_object = $raid_object->disk( "$all_disks_list[$count]" );
        $disk_bay_param[$count] = $disk_object->parameter( "bay" );

        if ( ($disk_bay_param[$count] == 0) || ($disk_bay_param[$count] == 1) ) {
	    $ses_disks_list[$ses_count] = $all_disks_list[$count];
            ++$ses_count;
        }
    }
    my @ses_spares_list;
    my $ses_spares_count = 0;
    my $tot_num_sesdisks = scalar( @ses_disks_list );
    my @disk_spare_param;
    
    for( $count = 0 ; $count < $tot_num_sesdisks ; ++$count ) {
        $disk_object = $raid_object->disk( "$ses_disks_list[$count]" );
        $disk_spare_param[$count] = $disk_object->parameter( "spare" );
      
        if ( $disk_spare_param[$count] == 1)  {
	    $ses_spares_list[$ses_spares_count] = $ses_disks_list[$count];
            ++$ses_spares_count;
        }
    }

    return @ses_spares_list;
}


# ***  Get an array of all SES disks which are IN_USE

## @name ::get_ses_inuse()

## @summary Get an array of all SES disks which are IN_USE

## @synopsis
##   get_ses_inuse($specs)

## @arg spec Required: hostspec of the filer

## @status in-progress

sub get_ses_inuse {
    my ($conn) = @_;
    my $raid_object = new RaidInfo($conn);
    my %all_disks_hash = $raid_object->all_disks();
    my @all_disks_list = keys(%all_disks_hash);
    my $tot_num_disks = scalar( @all_disks_list );
    my @disk_bay_param;
    my $disk_object;
    my @ses_disks_list;
    my $ses_count = 0;
    my $count = 0;

    for( $count = 0 ; $count < $tot_num_disks ; ++$count ) {
        $disk_object = $raid_object->disk( "$all_disks_list[$count]" );
        $disk_bay_param[$count] = $disk_object->parameter( "bay" );

        if ( ($disk_bay_param[$count] == 0) || ($disk_bay_param[$count] == 1) ) {
	    $ses_disks_list[$ses_count] = $all_disks_list[$count];
            ++$ses_count;
        }
    }
    my @ses_inuse_list;
    my $ses_inuse_count = 0;
    my $tot_num_sesdisks = scalar( @ses_disks_list );
    my @disk_inuse_param;
    
    for( $count = 0 ; $count < $tot_num_sesdisks ; ++$count ) {
        $disk_object = $raid_object->disk( "$ses_disks_list[$count]" );
        $disk_inuse_param[$count] = $disk_object->parameter( "spare" );
      
        if ( $disk_inuse_param[$count] == 0)  {
	    $ses_inuse_list[$ses_inuse_count] = $ses_disks_list[$count];
            ++$ses_inuse_count;
        }
    }

    return @ses_inuse_list;
}

1;
