# $Id: //depot/prod/test/Rsierranevada/lib/Storage/Storage.pm#7 $

## @summary Storage - storage library module
## @author vivienne@netapp.com
## @status testme
## @param FILER_TEST_UI
##  Optional: default is CLI, other value recognized is 'ZAPI'
## @pod here
## @change 19-05-2010 faslur: edited storage_download_shelf_new() function
## @change 18-02-2011 gurus: Modified storage_download_shelf_new() function to accept 'y' for storage download shelf -f -S command.
## @change 19-08-2011 gurus: Modified storage_show_disk() function to rectify some uninitialised variable messages.
## @change 20-01-2012 gururajc: Given C-Mode support to storage_power_cycle_shelf subroutine.
## @change 04-03-2012 rpradeep: Modified storage_show_disk subroutine to support "Dongle Revision" field.
## @change 08-31-2012 amal: Modified storage_show_disk() $disk_name assign. refer to 626739
## @change 01-23-2013 ssrinath: Added CMode support to storage_download_shelf_new() function
## @change 2013-06-02 gururajc: Given C-Mode support to storage_download_shelf subroutine.
## @change 01-03-2013 gururajc: Given C-Mode support to storage_download_acp subroutine.
## @change 26-06-2013 gururajc: Given interactive session  to storage_download_acp subroutine.
## @change 31-08-2013 gururajc: Replaced say with execute function in storage_power_cycle_shelf subroutine.
## @change 07-12-2013 nandeesh: Fixed the issue found in the burt 753731 in the sub storage_disable_adapter.

=head1 NAME

Storage::Storage

=head1 SYNOPSIS

    use Storage qw(storage_show_adapter storage_show_disk storage_show_hub storage_show_initiators storage_disable_adapter storage_enable_adapter storage_download_shelf storage_show_acp);

    my $host = host($FILER);
    my $conn = connect($FILER);
    my $hash = storage_show_adapter(host=>$host,....);


=head1 DESCRIPTION

    This is a library module for the ONTAP storage commands.

=head1 ROUTINES

=cut

package Storage::Storage;

use strict;
use warnings;
use Time::Local qw( timelocal );
use boot qw( reboot );

#Added following 4 libraries for ATTO related subroutines - Divyesh
#use Tharn;
#use Hostrec;
use NATE::Log;
use Net::FTP;

#use Switch;
BEGIN{
    use base qw(Exporter);
    our @EXPORT_OK = qw(
            storage_show_acp
            storage_show_adapter
            storage_show_disk
            storage_show_fault
            storage_show_hub
            storage_show_bridge
            run_atto_cmd
            get_atto_core_file_name
            download_atto_firmware
            retrieve_atto_core
            storage_show_initiators
            storage_disable_adapter
            storage_enable_adapter
            storage_download_shelf
            storage_download_shelf_new
            storage_show_expander
            storage_download_acp
            monitor_storage_download_acp
            storage_power_cycle_shelf
            testme
            );
};
#use Storage::SASUtils qw(filer_cmd);

use Carp qw(confess croak);
use Params::Validate qw(validate validate_pos OBJECT SCALAR ARRAYREF UNDEF);
BEGIN {
Params::Validate::validation_options(
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
}
use Tharn qw(say script_result host logcomment logresult param disconnect);
use Utils qw(make_conn);
use Data::Dumper qw(Dumper);
use Storage::Ems qw(ems_log_get_mark ems_log_dump_by_type ems_log_check ems_error_monitor ems_log_dump);
use Storage::SASUtils qw(filer_cmd storage_logresult);
use Storage::ACP qw(get_active_acp_ids get_acp_ids);

use ATI::ati_ontapi qw(ATI_SUCCESS
               ATI_ONTAPI_PRIVILEGE_DIAG
               ati_ontapi_create_context
               ati_ontapi_call_api);
use filer;
#use vars qw(@ISA @EXPORT
#   $FILER_TEST_UI
#);
our $MAINT_PROMPT = "\\*\>";

#param('FILER_TEST_UI','-default','CLI');   # Or 'ZAPI'

=head2 storage_show_adapter

    my $hash = storage_show_adapter(host=>$host, adapter_name=>$adapter_name);

This routine returns a pointer to hash table of storage adapters information in the filer
It dies on failure.
Following is an example of the hash table returned by the function:
$VAR1 = {
          7 => {
                 'Description' => 'IDE Host Adapter 7'
               },
      '10a' => {
                     'Description' => 'FibreChannelHostAdapter10a(Dual-channel,Q
Logic2312rev.2)',
                     'PCIClockSpeed' => '33MHz',
                     'LinkDataRate' => '2Gbit',
                     'FirmwareRev' => '3.3.14',
                     'CachelineSize' => 16,
                     'FCNodeName' => '2:000:00e08b:1cf4d7',
                     'PCIBusWidth' => '64-bit',
                     'SRAMParity' => 'Yes',
                     'Redundant' => 'Yes',
                     'External GBIC' => 'No',
                     'FCPacketSize' => 2048,
                     'InUse' => 'Yes',
                     'State' => 'Enabled'
                   }
        };

The following named arguments are accepted,

=over

=item host

(Required) : Filer connnection to use.

=item adapter_name

(Optional) : Specific adapter for which the information is required.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_show_adapter($%) {
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                         adapter_name => { type=>SCALAR, optional=>1 },
                             timeout => { type=>SCALAR, default=>300, optional=>1 },
                             cmd_opt => { type=>SCALAR, default=> "-a", optional=>1 },
                            }
                     );
    my $adapter_struct = {};
    my ($cmd, $output, $line, $adapter_name, @adap, @adapter_list, $phy);


        if (defined($opts{adapter_name})) {
            $cmd = "storage show adapter $opts{cmd_opt} $opts{adapter_name}";
        } else {
            $cmd = "storage show adapter $opts{cmd_opt}";
        }

        $output = filer_cmd (host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);

            # Check for command failure either through ems or one of the
            # possible values
            if (!($output =~ /Slot:/)) {
            croak $output;
            }

            # make an array for list of adapters in the output and call it @adapter_list
        @adapter_list = split (/\r*\n/,$output);
        @adapter_list =  grep /\S+/, @adapter_list;
        foreach $line (@adapter_list) {
            @adap = split(/:\s+|\r\n/ ,$line);
            if (!defined($adap[1])) {
                #print "Undefined info : $adap[0] \n";
                 $adap[1] = "undef";
            }
            if ( $line =~ /^Slot:/){
                $adapter_name = $adap[1];
                $adapter_name =~ s/\s+//g;
                goto LABEL;
            }
            if ($adap[0] =~ /Description/){
                $adapter_struct->{$adapter_name}->{Description} = $adap[1];
            } elsif ($adap[0] =~ /Firmware Rev/){
                $adapter_struct->{$adapter_name}->{FirmwareRev} = $adap[1];
            } elsif ($adap[0] =~ /PCI Bus Width/){
                $adapter_struct->{$adapter_name}->{PCIBusWidth} = $adap[1];
            } elsif ($adap[0] =~ /PCI Clock Speed/){
                $adapter_struct->{$adapter_name}->{PCIClockSpeed} = $adap[1];
            } elsif ($adap[0] =~ /FC Node Name/){
                $adapter_struct->{$adapter_name}->{FCNodeName} = $adap[1];
            } elsif ($adap[0] =~ /Cacheline Size/){
                $adapter_struct->{$adapter_name}->{CachelineSize} = $adap[1];
            } elsif ($adap[0] =~ /FC Packet Size/){
                $adapter_struct->{$adapter_name}->{FCPacketSize} = $adap[1];
            } elsif ($adap[0] =~ /Link Data Rate/){
                $adapter_struct->{$adapter_name}->{LinkDataRate} = $adap[1];
            } elsif ($adap[0] =~ /SRAM Parity/){
                $adapter_struct->{$adapter_name}->{SRAMParity} = $adap[1];
            } elsif ($adap[0] =~ /^State/){
                $adapter_struct->{$adapter_name}->{State} = $adap[1];
            } elsif ($adap[0] =~ /In Use/){
                $adapter_struct->{$adapter_name}->{InUse} = $adap[1];
            } elsif ($adap[0] =~ /Redundant/){
                $adapter_struct->{$adapter_name}->{Redundant} = $adap[1];
                        } elsif ($adap[0] =~ /Base WWN/){
                                $adapter_struct->{$adapter_name}->{BaseWWN} = $adap[1];
            } elsif ($adap[0] =~ /Phy State/){
                $phy = 0;
                $adap[1] =~ s/\[0\]\s//;
                                if ($adap[1] =~ /,/) {
                                        my @tmp = split (/,/,$adap[1]);
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $tmp[0];
                                        $tmp[1] =~ s/^\s*//;
                                        $tmp[1] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhySpeed} = $tmp[1];
                                } else {
                                        $adap[1] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $adap[1];
                                }
            } elsif ($adap[0] =~ /\[1\]/) {
                $phy = 1;
                $adap[0] =~ s/\s+\[1\]\s//;
                                if ($adap[0] =~ /,/) {
                                        my @tmp = split (/,/,$adap[0]);
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $tmp[0];
                                        $tmp[1] =~ s/^\s*//;
                                        $tmp[1] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhySpeed} = $tmp[1];
                                } else {
                                        $adap[0] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $adap[0];
                                }
            } elsif ($adap[0] =~ /\[2\]/) {
                $phy = 2;
                $adap[0] =~ s/\s+\[2\]\s//;
                                if ($adap[0] =~ /,/) {
                                        my @tmp = split (/,/,$adap[0]);
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $tmp[0];
                                        $tmp[1] =~ s/^\s*//;
                                        $tmp[1] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhySpeed} = $tmp[1];
                                } else {
                                        $adap[0] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $adap[0];
                                }
            } elsif ($adap[0] =~ /\[3\]/) {
                $phy = 3;
                $adap[0] =~ s/\s+\[3\]\s//;
                                if ($adap[0] =~ /,/) {
                                        my @tmp = split (/,/,$adap[0]);
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $tmp[0];
                                        $tmp[1] =~ s/^\s*//;
                                        $tmp[1] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhySpeed} = $tmp[1];
                                } else {
                                        $adap[0] =~ s/\s\(.\)$//;
                                        $adapter_struct->{$adapter_name}->{$phy}->{PhyState} = $adap[0];
                                }
            }
            LABEL: ;
        }
        #print Dumper($adapter_struct);
        return $adapter_struct;


}

=head2 storage_show_disk

    my $hash = storage_show_disk(host=>$host, cmd_opt=>$cmd_opt, disk_name=>$disk_name);
This routine returns a pointer to hash table that list all disks, with current name/path, physical locations and identifying info.

Following is an example of the hash table returned by the function:
$VAR1 = {
          '10b.20' => {
                        'Scrubcount' => 179,
                        'PriPort' => 'B',
                        'Vendor' => 'NETAPP',
                        'PowerOnHours' => 5479,
                        'WWN' => '2:000:0011c6:53eca5',
                        'Bay' => 4,
                        'Scrublastdone' => '00:00:19',
                        'RPM' => 10000,
                        'SecPort' => 'A',
                        'Downrev' => 'no',
                        'Model' => 'X274_SCHT6146F10',
                        'Serial' => '3HY9RYWW00007525JCKG',
                        'GlistCount' => '0',
                        'Dynamicallyqualified' => 'No',
                        'BlkWritten' => 112,
                        'SecPath' => '10a.20',
                        'Shelf' => 1,
                        'Timeinterval' => '05:04:27',
                        'BlkRead' => 4913,
                        'Rev' => 'NA07'
                      }
    };

The following named arguments are accepted,

=over

=item host

(Required) : Filer connnection to use.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item disk_name

(Optional) : Specific disk for which the information is required.

=item cmd_opt (future)

(optional) : default to "-a";
    -a option displays all information about disks
    -p option displays the primary and secondary paths to a disk device     -x option displays the disk specific information including serial number, vendor name and model

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_show_disk {
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                         disk_name => { type=>SCALAR, optional=>1 },
                             timeout => { type=>SCALAR, default=>300, optional=>1 },
                             cmd_opt => { type=>SCALAR, default=>"-a", optional=>1 },
                            }
                     );
    my $disk_struct = {};
    my ($cmd, $output, $disk_name, $disklist, @disk, $diskserial);


    if (defined $opts{disk_name}) {
        $cmd = "storage show disk $opts{cmd_opt} $opts{disk_name}";
    } else {
        $cmd = "storage show disk $opts{cmd_opt}";
    }
          $output = filer_cmd (host=>$opts{host}, filermode=>$opts{filermode}, conntype=>$opts{conntype}, timeout=>$opts{timeout}, cmd=>$cmd);

        # Check for command failure either through ems or one of the
        # possible values
        if (!($output =~ /Disk:/)) {
            croak $output;
        }

    my @disklist = split(/\r*\n/,$output);
    @disklist = grep /\S+/,@disklist;
    foreach $disklist (@disklist) {
        @disk = split (/:\s+|\r\n/,$disklist);
        if (defined($disk[1])) {
            $disk[1] =~ s/\s+//g;
        } else {
            #print "Undefined info : $disk[0] \n";
            $disk[1] = "undef";
        }
                if (defined $opts{disk_name}){
                        $disk_name = $opts{disk_name};
                } elsif ( $disklist =~ /^Disk:/){
            $disk_name = $disk[1];
            $disk_name =~ s/\s+//g;
            goto LABEL;
        }
        if ($disk[0] =~ /Shelf/){
            $disk_struct->{$disk_name}->{Shelf} = $disk[1];
        } elsif ($disk[0] =~ /Bay/){
            $disk_struct->{$disk_name}->{Bay} = $disk[1];
        } elsif ($disk[0] =~ /Vendor/){
            $disk_struct->{$disk_name}->{Vendor} = $disk[1];
        } elsif ($disk[0] =~ /Model/){
            $disk_struct->{$disk_name}->{Model} = $disk[1];
        } elsif ($disk[0] =~ /Dongle Revision/){
            $disk_struct->{$disk_name}->{DongleRevision} = $disk[1];
        } elsif ($disk[0] =~ /RPM/){
            $disk_struct->{$disk_name}->{RPM} = $disk[1];
        } elsif ($disk[0] =~ /WWN/){
            $disk_struct->{$disk_name}->{WWN} = $disk[1];
                } elsif ($disk[0] =~ /UID/){
                        $disk_struct->{$disk_name}->{UID} = $disk[1];
        } elsif ($disk[0] =~ /Downrev/){
            $disk_struct->{$disk_name}->{Downrev} = $disk[1];
                } elsif ($disk[0] =~ /Pri Port/){
                        $disk_struct->{$disk_name}->{PriPort} = $disk[1];
                } elsif ($disk[0] =~ /Sec Name/){
                        $disk_struct->{$disk_name}->{SecPath} = $disk[1];
                } elsif ($disk[0] =~ /Sec Port/){
                        $disk_struct->{$disk_name}->{SecPort} = $disk[1];
                } elsif ($disk[0] =~ /Power-on Hours/){
                        $disk_struct->{$disk_name}->{PowerOnHours} = $disk[1];
                } elsif ($disk[0] =~ /Blocks read/){
                        $disk_struct->{$disk_name}->{BlkRead} = $disk[1];
                } elsif ($disk[0] =~ /Blocks written/){
                        $disk_struct->{$disk_name}->{BlkWritten} = $disk[1];
        } elsif ($disk[0] =~ /Time interval/){
            $disk_struct->{$disk_name}->{Timeinterval} = $disk[1];
                } elsif ($disk[0] =~ /Glist count/){
                        $disk_struct->{$disk_name}->{Glistcount} = $disk[1];
        } elsif ($disk[0] =~ /Scrub last done/){
            $disk_struct->{$disk_name}->{Scrublastdone} = $disk[1];
        } elsif ($disk[0] =~ /Scrub count/){
            $disk_struct->{$disk_name}->{Scrubcount} = $disk[1];
                } elsif ($disk[0] =~ /LIP count/){
                        $disk_struct->{$disk_name}->{LIPcount} = $disk[1];
        } elsif ($disk[0] =~ /Dynamically qualified/){
            $disk_struct->{$disk_name}->{Dynamicallyqualified} = $disk[1];
                } elsif ($disk[0] =~ /Current owner/){
                        $disk_struct->{$disk_name}->{CurrentOwner} = $disk[1];
                } elsif ($disk[0] =~ /Home owner/){
                        $disk_struct->{$disk_name}->{HomeOwner} = $disk[1];
                } elsif ($disk[0] =~ /Reservation owner/){
                        $disk_struct->{$disk_name}->{ReservationOwner} = $disk[1];
        } elsif ($disk[0] =~ /Rev/){
                        $disk_struct->{$disk_name}->{Rev} = $disk[1];
        } elsif ($disk[0] =~ /Serial/){
            $diskserial = "$disk[1]" if (defined($disk[1]));
            if($diskserial =~ /undef/) {
                                logcomment("Couldn't get serial number of $disk_name.");
                                #$diskserial = $disk_name; #Why its done this way? Shouldn't this be logged? - Divyesh
            }
            $disk_struct->{$disk_name}->{Serial} = $diskserial;
        } else {
            if($disk_name =~ /\./){# If disk_name doesnot contain a "." then we can assume that it's not an output of a disk.
                $disk_struct->{$disk_name}->{$disk[0]} = $disk[1]; #This is needed to populate attributes NOT defined above. Divyesh.
            }
        }
        LABEL: ;
    }
    #print Dumper($disk_struct);
    return ($disk_struct);
}

=head2 storage_show_hub

    my $hash = storage_show_hub(host=>$host, hub_name=>$hub_name);

This routine returns a pointer to hash table of storage hubs information in the filer.
It dies on failure.
The output can contain:
NOTE: ESH A on channels 4a/4b shelf id 1 is reporting
      an error condition. The following port information may not be accurate.
But in phase I of the project, This will be ignored.

Following is an example of the hash table returned by the function:
$VAR1 = {
          '10b.shelf2' => {
                            'IN' => {
                                      'StallCount' => '0',
                                      'LoopupCount' => '0',
                                      'DiskBay' => 'undef',
                                      'InsertCount' => '0',
                                      'InvalidCRCCount' => '0',
                                      'InvalidWordCount' => '0',
                                      'PortState' => 'OK',
                                      'ClockDelta' => -8,
                                      'Util%' => '0'
                                    },
                            'Shelfstate' => 'ONLINE',
                            'ShelfUID' => '50:05:0c:c0:02:10:d6:36',
                            'OUT' => {
                                       'StallCount' => '0',
                                       'LoopupCount' => '0',
                                       'DiskBay' => 'undef',
                                       'InsertCount' => '0',
                                       'InvalidCRCCount' => '0',
                                       'InvalidWordCount' => '0',
                                       'PortState' => 'OK',
                                       'ClockDelta' => 16,
                                       'Util%' => '0'
                                     },
                            'Termswitch' => 'N/A',
                            'ESHstate' => 'OK',
                            'Channel' => '10b',
                            'Loop' => 'B',
                            'Shelfid' => 2,
                            32 => {
                                    'StallCount' => '0',
                                    'LoopupCount' => '0',
                                    'DiskBay' => '0',
                                    'InsertCount' => '0',
                                    'InvalidCRCCount' => '0',
                                    'InvalidWordCount' => '0',
                                    'PortState' => 'OK',
                                    'ClockDelta' => 40,
                                    'Util%' => '0'
                                  },

                ........ # 33 ~ 44

                            45 => {
                                    'StallCount' => '0',
                                    'LoopupCount' => '0',
                                    'DiskBay' => 13,
                                    'InsertCount' => '0',
                                    'InvalidCRCCount' => '0',
                                    'InvalidWordCount' => '0',
                                    'PortState' => 'OK',
                                    'ClockDelta' => 32,
                                    'Util%' => '0'
                                  }
                          },
          '10a.shelf2' => {
                            'IN' => {
                                      'StallCount' => '0',
                                      'LoopupCount' => '0',
                                      'DiskBay' => 'undef',
                                      'InsertCount' => '0',
                                      'InvalidCRCCount' => '0',
                                      'InvalidWordCount' => '0',
                                      'PortState' => 'OK',
                                      'ClockDelta' => 8,
                                      'Util%' => '0'
                                    },
                            'Shelfstate' => 'ONLINE',
                            'ShelfUID' => '50:05:0c:c0:02:10:d6:36',
                            'OUT' => {
                                       'StallCount' => '0',
                                       'LoopupCount' => '0',
                                       'DiskBay' => 'undef',
                                       'InsertCount' => '0',
                                       'InvalidCRCCount' => '0',
                                       'InvalidWordCount' => '0',
                                       'PortState' => 'TERM',
                                       'ClockDelta' => '0',
                                       'Util%' => '0'
                                     },
                            'Termswitch' => 'N/A',
                            'ESHstate' => 'OK',
                            'Channel' => '10a',
                            'Loop' => 'A',
                            'Shelfid' => 2,
                            32 => {
                                    'StallCount' => '0',
                                    'LoopupCount' => '0',
                                    'DiskBay' => '0',
                                    'InsertCount' => '0',
                                    'InvalidCRCCount' => '0',
                                    'InvalidWordCount' => '0',
                                    'PortState' => 'OK',
                                    'ClockDelta' => 32,
                                    'Util%' => '0'
                                  },

                ........ # 33 ~ 44

                            45 => {
                                    'StallCount' => '0',
                                    'LoopupCount' => '0',
                                    'DiskBay' => 13,
                                    'InsertCount' => '0',
                                    'InvalidCRCCount' => '0',
                                    'InvalidWordCount' => '0',
                                    'PortState' => 'OK',
                                    'ClockDelta' => 32,
                                    'Util%' => '0'
                                  }
                          }
        };

=over

=item host

(Required) : Filer connnection to use.

=item hub_name

(Optional) : Specific hub for which the information is required.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_show_hub {
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                         hub_name => { type=>SCALAR, optional=>1 },
                             timeout => { type=>SCALAR, default=>300, optional=>1 },
                             cmd_opt => { type=>SCALAR, default=>"-a", optional=>1 },
                            }
                     );
    my $hub_struct = {};
    my ($cmd, $output, $hub_name, $hublist, @hub, @fields, $Diskid);

    # no zapi so make connection
    # check if filer in maintenance mode
    $cmd = "storage show hub $opts{cmd_opt} $opts{hub_name}";
       $output = filer_cmd (host=>$opts{host}, filermode=>$opts{filermode}, conntype=>$opts{conntype}, timeout=>$opts{timeout}, cmd=>$cmd);

        # Check for command failure either through ems or one of the
        # possible values
        if (!($output =~ /Shelf name:/)) {
                croak $output;
        }

    my @hublist = split(/\r*\n/,$output);
    @hublist = grep /\S+/,@hublist;
    foreach $hublist (@hublist) {
        @hub = split (/:\s+|\r\n/,$hublist);
        if (defined($hub[1])) {
            $hub[1] =~ s/\s+//g;
        } else {
               $hub[1] = "undef";
        }
                      if ( $hublist =~ /^Shelf name:/){
                        $hub_name = $hub[1];
                        $hub_name =~ s/\s+//g;
                        goto LABEL;
                }
        if ($hub[0] =~ /Channel/){
                $hub_struct->{$hub_name}->{Channel} = $hub[1];
        } elsif ($hub[0] =~ /Loop$/){
            $hub_struct->{$hub_name}->{Loop} = $hub[1];
                } elsif ($hub[0] =~ /Module:/){
                        $hub_struct->{$hub_name}->{Module} = $hub[1];
        } elsif ($hub[0] =~ /Shelf id/){
            $hub_struct->{$hub_name}->{Shelfid} = $hub[1];
        } elsif ($hub[0] =~ /Shelf UID/){
            $hub_struct->{$hub_name}->{ShelfUID} = $hub[1];
                } elsif ($hub[0] =~ /Shelf S\/N:/){
                        $hub_struct->{$hub_name}->{ShelfSN} = $hub[1];
        } elsif ($hub[0] =~ /Term switch/){
            $hub_struct->{$hub_name}->{Termswitch} = $hub[1];
        } elsif ($hub[0] =~ /Shelf state/){
            $hub_struct->{$hub_name}->{Shelfstate} = $hub[1];
        } elsif ($hub[0] =~ /ESH state/){
            $hub_struct->{$hub_name}->{ESHstate} = $hub[1];
                } elsif ($hub[0] =~ /Module state:/){
                        $hub_struct->{$hub_name}->{ModuleState} = $hub[1];
                } elsif ($hub[0] =~ /Module uptime:/){
                        $hub_struct->{$hub_name}->{ModuleUptime} = $hub[1];
        } elsif ($hub[0] =~ /\[/){
            @fields = split(/\s+/, $hub[0]);
            if ($fields[1] =~ /^\]/) {
                $Diskid = $fields[0];
                $Diskid =~ s/\[//;
                $hub_struct->{$hub_name}->{$Diskid}->{DiskBay} = "undef";
                $hub_struct->{$hub_name}->{$Diskid}->{PortState} = $fields[2];
                $hub_struct->{$hub_name}->{$Diskid}->{LoopupCount} = $fields[3];
                $hub_struct->{$hub_name}->{$Diskid}->{InvalidCRCCount} = $fields[4];
                $hub_struct->{$hub_name}->{$Diskid}->{InvalidWordCount} = $fields[5];
                $hub_struct->{$hub_name}->{$Diskid}->{ClockDelta} = $fields[6];
                $hub_struct->{$hub_name}->{$Diskid}->{InsertCount} = $fields[7];
                $hub_struct->{$hub_name}->{$Diskid}->{StallCount} = $fields[8];
                $hub_struct->{$hub_name}->{$Diskid}->{'Util%'} = $fields[9];
                                $hub_struct->{$hub_name}->{$Diskid}->{LIPCount} = $fields[10];
            } else{
                $Diskid = $fields[1];
                $Diskid =~ s/\]//;
                $hub_struct->{$hub_name}->{$Diskid}->{DiskBay} = $fields[2];
                $hub_struct->{$hub_name}->{$Diskid}->{PortState} = $fields[3];
                $hub_struct->{$hub_name}->{$Diskid}->{LoopupCount} = $fields[4];
                $hub_struct->{$hub_name}->{$Diskid}->{InvalidCRCCount} = $fields[5];
                $hub_struct->{$hub_name}->{$Diskid}->{InvalidWordCount} = $fields[6];
                $hub_struct->{$hub_name}->{$Diskid}->{ClockDelta} = $fields[7];
                $hub_struct->{$hub_name}->{$Diskid}->{InsertCount} = $fields[8];
                $hub_struct->{$hub_name}->{$Diskid}->{StallCount} = $fields[9];
                $hub_struct->{$hub_name}->{$Diskid}->{'Util%'} = $fields[10];
                                $hub_struct->{$hub_name}->{$Diskid}->{LIPCount} = $fields[11];
            }
        }
        LABEL: ;
    }
    #print Dumper($hub_struct);
    return ($hub_struct);
}

=head2 storage_show_bridge

    my $hash = storage_show_bridge(host=>$host, bridge_name=>$bridge_name);

This routine returns a pointer to hash table of storage bridges information in the filer.
It dies on failure.

Following is an example of the hash table returned by the function:

$VAR1 = {
          'svt-300-1:9.126L0' => {
                                   'FC Port 2' => {
                                                    'LIPCount' => '0',
                                                    'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                    'LinkFailureCount' => '1',
                                                    'Port' => '2',
                                                    'State' => 'down',
                                                    'SFPPartNumber' => 'AFBR-57D7APZ',
                                                    'Topology' => 'unknown',
                                                    'Speed' => 'unknown',
                                                    'LossofSyncCount' => '1',
                                                    'FramesOut' => '0',
                                                    'FramesIn' => '0',
                                                    'CRCErrorCount' => '0'
                                                  },
                                   'FC Port 1' => {
                                                    'LIPCount' => '0',
                                                    'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                    'LinkFailureCount' => '0',
                                                    'Port' => '1',
                                                    'State' => 'up',
                                                    'SFPPartNumber' => 'AFBR-57D7APZ',
                                                    'Topology' => 'point-to-point',
                                                    'Speed' => '8 Gb/s',
                                                    'LossofSyncCount' => '0',
                                                    'FramesOut' => '26821432923',
                                                    'FramesIn' => '12900372683',
                                                    'CRCErrorCount' => '0'
                                                  },
                                   'SAS Port' => {
                                                   'QSFPPartNumber' => '112-00178+A0    ',
                                                   'PHYS' => {
                                                               '1' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '15228',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '14754',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '1',
                                                                        'CRCErrorCount' => '0'
                                                                      },
                                                               '3' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '14124',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '13001',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '3',
                                                                        'CRCErrorCount' => '0'
                                                                      },
                                                               '0' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '22114',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '18418',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '2',
                                                                        'CRCErrorCount' => '0'
                                                                      },
                                                               '2' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '21157',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '20046',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '26',
                                                                        'CRCErrorCount' => '0'
                                                                      }
                                                             },
                                                   'QSFPType' => 'Passive Copper 5m ID:00',
                                                   'Port' => 'A'
                                                 },
                                   'Vendor' => 'AVAGO',
                                   'Model' => 'FibreBridge6500N',
                                   'Serial' => 'AD1044A00KE',
                                   'Rev' => '1.11'
                                 },
          'svt-5100-2:1.126L0' => {
                                    'FC Port 2' => {
                                                     'LIPCount' => '0',
                                                     'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                     'LinkFailureCount' => '1',
                                                     'Port' => '2',
                                                     'State' => 'down',
                                                     'SFPPartNumber' => 'FTLF8528P2BCV',
                                                     'Topology' => 'unknown',
                                                     'Speed' => 'unknown',
                                                     'LossofSyncCount' => '1',
                                                     'FramesOut' => '0',
                                                     'FramesIn' => '0',
                                                     'CRCErrorCount' => '0'
                                                   },
                                    'FC Port 1' => {
                                                     'LIPCount' => '0',
                                                     'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                     'LinkFailureCount' => '0',
                                                     'Port' => '1',
                                                     'State' => 'up',
                                                     'SFPPartNumber' => 'FTLF8528P2BCV',
                                                     'Topology' => 'point-to-point',
                                                     'Speed' => '8 Gb/s',
                                                     'LossofSyncCount' => '0',
                                                     'FramesOut' => '25247990064',
                                                     'FramesIn' => '9836266540',
                                                     'CRCErrorCount' => '0'
                                                   },
                                    'SAS Port' => {
                                                    'QSFPPartNumber' => '112-00178+20    ',
                                                    'PHYS' => {
                                                                '1' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '7429',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '5794',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '17',
                                                                         'CRCErrorCount' => '0'
                                                                       },
                                                                '3' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '29965',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '29601',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '3',
                                                                         'CRCErrorCount' => '0'
                                                                       },
                                                                '0' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '13276',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '11522',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '33',
                                                                         'CRCErrorCount' => '0'
                                                                       },
                                                                '2' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '18351',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '15995',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '29',
                                                                         'CRCErrorCount' => '0'
                                                                       }
                                                              },
                                                    'QSFPType' => 'Passive Copper 5m ID:00',
                                                    'Port' => 'A'
                                                  },
                                    'Vendor' => 'FINISAR CORP.',
                                    'Model' => 'FibreBridge6500N',
                                    'Serial' => 'PH41B9Z',
                                    'Rev' => '1.11'
                                  },
          'svt-300-2:1.126L0' => {
                                   'FC Port 2' => {
                                                    'LIPCount' => '0',
                                                    'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                    'LinkFailureCount' => '1',
                                                    'Port' => '2',
                                                    'State' => 'down',
                                                    'SFPPartNumber' => 'FTLF8528P2BCV',
                                                    'Topology' => 'unknown',
                                                    'Speed' => 'unknown',
                                                    'LossofSyncCount' => '1',
                                                    'FramesOut' => '0',
                                                    'FramesIn' => '0',
                                                    'CRCErrorCount' => '0'
                                                  },
                                   'FC Port 1' => {
                                                    'LIPCount' => '0',
                                                    'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                    'LinkFailureCount' => '105',
                                                    'Port' => '1',
                                                    'State' => 'up',
                                                    'SFPPartNumber' => 'FTLF8528P2BCV',
                                                    'Topology' => 'point-to-point',
                                                    'Speed' => '8 Gb/s',
                                                    'LossofSyncCount' => '14',
                                                    'FramesOut' => '32467204837',
                                                    'FramesIn' => '14636838057',
                                                    'CRCErrorCount' => '0'
                                                  },
                                   'SAS Port' => {
                                                   'QSFPPartNumber' => '112-00178+A0    ',
                                                   'PHYS' => {
                                                               '1' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '9630',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '7381',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '6',
                                                                        'CRCErrorCount' => '0'
                                                                      },
                                                               '3' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '28951',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '27136',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '2',
                                                                        'CRCErrorCount' => '0'
                                                                      },
                                                               '0' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '21419',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '19379',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '16',
                                                                        'CRCErrorCount' => '0'
                                                                      },
                                                               '2' => {
                                                                        'PhyChangeCount' => '1',
                                                                        'InvalidDWordCount' => '14486',
                                                                        'State' => 'up',
                                                                        'PhyResetProb' => '0',
                                                                        'DisprtErrorCount' => '13377',
                                                                        'Speed' => '3.0 Gb/s',
                                                                        'LossSyncCount' => '1',
                                                                        'CRCErrorCount' => '0'
                                                                      }
                                                             },
                                                   'QSFPType' => 'Passive Copper 5m ID:00',
                                                   'Port' => 'A'
                                                 },
                                   'Vendor' => 'FINISAR CORP.',
                                   'Model' => 'FibreBridge6500N',
                                   'Serial' => 'PH408R3',
                                   'Rev' => '1.11'
                                 },
          'svt-5100-1:9.126L0' => {
                                    'FC Port 2' => {
                                                     'LIPCount' => '0',
                                                     'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                     'LinkFailureCount' => '1',
                                                     'Port' => '2',
                                                     'State' => 'down',
                                                     'SFPPartNumber' => 'FTLF8528P2BCV',
                                                     'Topology' => 'unknown',
                                                     'Speed' => 'unknown',
                                                     'LossofSyncCount' => '1',
                                                     'FramesOut' => '0',
                                                     'FramesIn' => '0',
                                                     'CRCErrorCount' => '0'
                                                   },
                                    'FC Port 1' => {
                                                     'LIPCount' => '0',
                                                     'SFPCapabilities' => '2, 4, 8 Gb/s',
                                                     'LinkFailureCount' => '0',
                                                     'Port' => '1',
                                                     'State' => 'up',
                                                     'SFPPartNumber' => 'FTLF8528P2BCV',
                                                     'Topology' => 'point-to-point',
                                                     'Speed' => '8 Gb/s',
                                                     'LossofSyncCount' => '0',
                                                     'FramesOut' => '33156937506',
                                                     'FramesIn' => '13225012132',
                                                     'CRCErrorCount' => '0'
                                                   },
                                    'SAS Port' => {
                                                    'QSFPPartNumber' => '112-00177+A0    ',
                                                    'PHYS' => {
                                                                '1' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '271',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '226',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '1',
                                                                         'CRCErrorCount' => '0'
                                                                       },
                                                                '3' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '29483',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '28048',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '3',
                                                                         'CRCErrorCount' => '0'
                                                                       },
                                                                '0' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '13697',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '12229',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '2',
                                                                         'CRCErrorCount' => '0'
                                                                       },
                                                                '2' => {
                                                                         'PhyChangeCount' => '3',
                                                                         'InvalidDWordCount' => '9199',
                                                                         'State' => 'up',
                                                                         'PhyResetProb' => '0',
                                                                         'DisprtErrorCount' => '7086',
                                                                         'Speed' => '3.0 Gb/s',
                                                                         'LossSyncCount' => '2',
                                                                         'CRCErrorCount' => '0'
                                                                       }
                                                              },
                                                    'QSFPType' => 'Passive Copper 2m ID:00',
                                                    'Port' => 'A'
                                                  },
                                    'Vendor' => 'FINISAR CORP.',
                                    'Model' => 'FibreBridge6500N',
                                    'Serial' => 'PH40X41',
                                    'Rev' => '1.11'
                                  }
        };
=over

=item host

(Required) : Filer connnection to use.

=item bridge_name

(Optional) : Specific bridge for which the information is required.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_show_bridge {
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                         bridge_name => { type=>SCALAR, optional=>1 },
                             timeout => { type=>SCALAR, default=>300, optional=>1 },
                             cmd_opt => { type=>SCALAR, default=>"", optional=>1 },
                            }
                     );
    my $bridge_struct = {};
    my ($cmd, $output, $bridge_name, $bridgelist, @bridge, @phys, $phy, $phy_id, $SASPort, $FCPort);

    if ($opts{filermode} ne "normal") {
            $opts{conntype} = "#console";
    }

        $cmd = "storage show bridge $opts{cmd_opt} $opts{bridge_name}";

        $output = filer_cmd (host=>$opts{host}, filermode=>$opts{filermode}, conntype=>$opts{conntype}, timeout=>$opts{timeout}, cmd=>$cmd);

        if ($output =~ /No bridges found/) {
                return $output;
        } elsif (!($output =~ /Bridge:/)) {
                croak $output;
        } else {

    my @bridgelist = split(/\r*\n/,$output);
    @bridgelist = grep /\S+/,@bridgelist; # Remove spaced line(s) (by greping ALL lines containing non-white space characters), if any.
    foreach $bridgelist (@bridgelist) {
        @bridge = split (/:\s+|\r\n/,$bridgelist);
        if (defined($bridge[1])) {
            $bridge[1] =~ s/^\s+//g; #Remove only pre-space(s) from values.
        } else {
            #print "Undefined info : $hub[0] \n"; Some output line like SAS Port A: do not have any value...
            $bridge[1] = "undef";
        }

                if ( $bridgelist =~ /^Bridge:/){
                        $bridge_name = $bridge[1];
                        $bridge_name =~ s/\s+//g; # Remove any space(s) from name.
                        goto LABEL;
                }

        if ($bridge[0] =~ /^Serial/){ # This has to be 1st character as SAS port stats header has State defined.
            $bridge_struct->{$bridge_name}->{Serial} = $bridge[1];
        } elsif ($bridge[0] =~ /^Vendor/){
            $bridge_struct->{$bridge_name}->{Vendor} = $bridge[1];
                } elsif ($bridge[0] =~ /^Model/){
                        $bridge_struct->{$bridge_name}->{Model} = $bridge[1];
                } elsif ($bridge[0] =~ /^Rev/){
                        $bridge_struct->{$bridge_name}->{Rev} = $bridge[1];
                } elsif ($bridge[0] =~ /^SAS Port A/){
                        $SASPort = "SAS Port"; #We don't need to look for B port.
                        $bridge[1] = "A";
                        $bridge_struct->{$bridge_name}->{$SASPort}->{Port} = $bridge[1];
                }elsif ($bridge[0] =~ /^QSFP Vendor/){
                        $bridge_struct->{$bridge_name}->{$SASPort}->{QSFPVendor} = $bridge[1];
                }elsif ($bridge[0] =~ /^QSFP Part Number/){
                        $bridge_struct->{$bridge_name}->{$SASPort}->{QSFPPartNumber} = $bridge[1];
                }elsif ($bridge[0] =~ /^QSFP Type/){
                        $bridge_struct->{$bridge_name}->{$SASPort}->{QSFPType} = $bridge[1];
                }elsif ($bridge[0] =~ /^QSFP Serial Number/){
                        $bridge_struct->{$bridge_name}->{$SASPort}->{QSFPSerialNumber} = $bridge[1];
                }elsif ($bridge[0] =~ /0 Gb\/s/){
                        $phy="PHYS";
                        @phys=split(/\s+/,$bridge[0]);
                        #$phys[0] =~ s/^\s+//g; #Is this needed? - Divyesh.
                        shift @phys; #Remove 1st element from the array as it would be white spave character.
                        $phy_id=$phys[0];
                        #$bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{Phy} = $phys[0];
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{State} = $phys[1];
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{Speed} = "$phys[2]"." "."$phys[3]";
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{InvalidDWordCount} = $phys[4];
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{DisprtErrorCount} = $phys[5];
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{LossSyncCount} = $phys[6];
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{PhyResetProb} = $phys[7];
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{PhyChangeCount} = $phys[8];
                        $bridge_struct->{$bridge_name}->{$SASPort}->{$phy}->{$phy_id}->{CRCErrorCount} = $phys[9];
                }elsif ($bridge[0] =~ /^FC Port 1/) {
                        $FCPort = "FC Port 1";
                        $bridge[1] = "1";
                        $bridge_struct->{$bridge_name}->{$FCPort}->{Port} = $bridge[1];
                }elsif ($bridge[0] =~ /^FC Port 2/) {
                        $FCPort = "FC Port 2";
                        $bridge[1] = "2";
                        $bridge_struct->{$bridge_name}->{$FCPort}->{Port} = $bridge[1];
                }elsif ($bridge[0] =~ /^State/){ # This has to be 1st character as SAS port stats header has State defined.
                        $bridge_struct->{$bridge_name}->{$FCPort}->{State} = $bridge[1];
                }elsif ($bridge[0] =~ /^Speed/){ # Here it need not be 1st character BUT putting it anyway.
                        $bridge_struct->{$bridge_name}->{$FCPort}->{Speed} = $bridge[1];
                }elsif ($bridge[0] =~ /^Topology/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{Topology} = $bridge[1];
                }elsif ($bridge[0] =~ /^Link Failure Count/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{LinkFailureCount} = $bridge[1];
                }elsif ($bridge[0] =~ /^Loss of Sync Count/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{LossofSyncCount} = $bridge[1];
                }elsif ($bridge[0] =~ /^CRC Error Count/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{CRCErrorCount} = $bridge[1];
                }elsif ($bridge[0] =~ /^LIP Count/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{LIPCount} = $bridge[1];
                }elsif ($bridge[0] =~ /^Frames In/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{FramesIn} = $bridge[1];
                }elsif ($bridge[0] =~ /^Frames Out/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{FramesOut} = $bridge[1];
                }elsif ($bridge[0] =~ /^SFP Vendor/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{SFPVendor} = $bridge[1];
                }elsif ($bridge[0] =~ /^SFP Part Number/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{SFPPartNumber} = $bridge[1];
                }elsif ($bridge[0] =~ /^SFP Serial Number/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{SFPSerialNumber} = $bridge[1];
                }elsif ($bridge[0] =~ /^SFP Capabilities/){
                        $bridge_struct->{$bridge_name}->{$FCPort}->{SFPCapabilities} = $bridge[1];
                } else {
                        next;
                }
        LABEL: ;
    }
    #print Dumper($bridge_struct);
    return ($bridge_struct);
    }
}

#Adding atto specific subroutines here - Divyesh.

sub run_atto_cmd{
    my (%opts) = validate(@_, {
                         bridge_name => {type=>OBJECT, isa=>'Hostrec'},
                         bridge_cmd => {type=>SCALAR,default=>"info",optional=>1},
                             raw_output => {type=>SCALAR, default=>0, optional=>1 },
                 conntype => {type=>SCALAR,default=>"#telnet",optional=>1},
                 user => {type=>SCALAR,default=>"root",optional=>1},
                 password => {type=>SCALAR,default=>"Password",optional=>1},
                             timeout => {type=>SCALAR, default=>90, optional=>1 },
                            }
                       );
     my $br = $opts{bridge_name};
     my $cmd = $opts{bridge_cmd};
     my $user = $opts{user};
     my $passwd = $opts{password};

     my $conn;
     if ($opts{conntype} eq "#telnet") {
             $conn = make_conn($opts{bridge_name},"#telnet");
     } else {
             $conn = make_conn($opts{bridge_name},"#console");
     }

     #my $output = filer_cmd (host=>$opts{bridge_name}, conntype=>$opts{conntype}, prompt_check=>"Ready.", cmd=>$cmd); #Not going to work as filer_cmd cannot check for prompt string by itself.#

     (undef, undef, my $output) = say ($conn, "$opts{bridge_cmd}", -prompt, "Ready.", "-timeout", $opts{timeout});
           sleep 10;
           disconnect($conn);
           sleep 30;

   if (!$opts{raw_output}) {
     logcomment("Defined structured output requested for $cmd command, if available.");
     if ($opts{bridge_cmd} eq "info") {
           my $bridge_info = {};

           if (!($output =~ /Device/)) {
                croak $output;   ### We should NOT be croaking out here - Divyesh.
           }

           my @info = split(/\r*\n/,$output);
           @info = grep /\S+/,@info; #Remove spaced line(s), if any.
           foreach my $line (@info) {
                 my @fields = split (/=/,$line);
                 $fields[1] =~ s/^\s+//; #Remove white space ONLY from beginning of the value. No need to remove trailing white spaces from key.
                 $fields[1] =~ s/\s+$//; #Remove white space ONLY from ending of the value. No need to remove trailing white spaces from key.

                 if ($fields[0] eq $opts{bridge_cmd}) {
                         next;
                 } elsif ($fields[0] =~ /^Device/) {
                        $bridge_info->{Device} = $fields[1];
                 } elsif ($fields[0] =~ /^Serial Number/) {
                        $bridge_info->{SerialNumber} = $fields[1];
                 } elsif ($fields[0] =~ /^Device Version/) {
                        $bridge_info->{DeviceVersion} = $fields[1];
                 } elsif ($fields[0] =~ /^Build Number/) {
                        $bridge_info->{BuildNumber} = $fields[1];
                 } elsif ($fields[0] =~ /^Build Date/) {
                        $bridge_info->{BuildDate} = $fields[1];
                 } elsif ($fields[0] =~ /^Flash Revision/) {
                        $bridge_info->{FlashRevision} = $fields[1];
                 } elsif ($fields[0] =~ /^CLI Revision/) {
                        $bridge_info->{CLIRevision} = $fields[1];
                 } elsif ($fields[0] =~ /^Base version/) {
                        $bridge_info->{Baseversion} = $fields[1];
                 } elsif ($fields[0] =~ /^Version Number/) {
                        $bridge_info->{VersionNumber} = $fields[1];
                 } elsif ($fields[0] =~ /^User-defined name/) {
                        $bridge_info->{Userdefinedname} = $fields[1];
                 } elsif ($fields[0] =~ /^World Wide Name/) {
                        $fields[1] =~ s/\s/:/g; # Getting it standardized.
                        $bridge_info->{WorldWideName} = $fields[1];
                 } elsif ($fields[0] =~ /^FC1 Node Name/) {
                        $fields[1] =~ s/\s/:/g; # Getting it standardized.
                        $bridge_info->{FC1NodeName} = $fields[1];
                 } elsif ($fields[0] =~ /^FC1 Port Name/) {
                        $fields[1] =~ s/\s/:/g; # Getting it standardized.
                        $bridge_info->{FC1PortName} = $fields[1];
                 } elsif ($fields[0] =~ /^FC1 Data Rate/) {
                        $bridge_info->{FC1DataRate} = $fields[1];
                 } elsif ($fields[0] =~ /^FC1 Connection Mode/) {
                        $bridge_info->{FC1ConnectionMode} = $fields[1];
                 } elsif ($fields[0] =~ /^FC2 Node Name/) {
                        $fields[1] =~ s/\s/:/g; # Getting it standardized.
                        $bridge_info->{FC2NodeName} = $fields[1];
                 } elsif ($fields[0] =~ /^FC2 Port Name/) {
                        $fields[1] =~ s/\s/:/g; # Getting it standardized.
                        $bridge_info->{FC2PortName} = $fields[1];
                 } elsif ($fields[0] =~ /^FC2 Data Rate/) {
                        $bridge_info->{FC2DataRate} = $fields[1];
                 } elsif ($fields[0] =~ /^FC2 Connection Mode/) {
                        $bridge_info->{FC2ConnectionMode} = $fields[1];
                 } elsif ($fields[0] =~ /^MP1 MAC Address/) {
                        $fields[1] =~ s/\s/:/g; # Getting it standardized.
                        $bridge_info->{MP1MACAddress} = $fields[1];
                 } elsif ($fields[0] =~ /^MP1 IP Address/) {
                        $bridge_info->{MP1IPAddress} = $fields[1];
                 } elsif ($fields[0] =~ /^MP1 IP Subnet Mask/) {
                        $bridge_info->{MP1IPSubnetMask} = $fields[1];
                 } elsif ($fields[0] =~ /^MP1 IP Gateway/) {
                        $bridge_info->{MP1IPGateway} = $fields[1];
                 } elsif ($fields[0] =~ /^MP1 IP DHCP/) {
                        $bridge_info->{MP1IPDHCP} = $fields[1];
                 } elsif ($fields[0] =~ /^MP2 MAC Address/) {
                        $fields[1] =~ s/\s/:/g; # Getting it standardized.
                        $bridge_info->{MP2MACAddress} = $fields[1];
                 } elsif ($fields[0] =~ /^MP2 IP Address/) {
                        $bridge_info->{MP2IPAddress} = $fields[1];
                 } elsif ($fields[0] =~ /^MP2 IP Subnet Mask/) {
                        $bridge_info->{MP2IPSubnetMask} = $fields[1];
                 } elsif ($fields[0] =~ /^MP2 IP Gateway/) {
                        $bridge_info->{MP2IPGateway} = $fields[1];
                 } elsif ($fields[0] =~ /^MP2 IP DHCP/) {
                        $bridge_info->{MP2IPDHCP} = $fields[1];
                 } elsif ($fields[0] =~ /^Active Configuration/) {
                        $bridge_info->{ActiveConfiguration} = $fields[1];
                 } else {
                        print "$fields[0] --- $fields[1] \n";
                        logresult ("FAIL","We should not be here unless new info is added");
                 }
           }
           return ($bridge_info);
     } elsif ($opts{bridge_cmd} eq "coredumpinfo") {
           if ($output =~ /No core dump found/) {
                #Populate Dummy Hash.
                my $bridge_core_info = {};
                $bridge_core_info->{CoreDumpVersion} = "00000101";
                $bridge_core_info->{BuildNumber} = "063Y";
                $bridge_core_info->{DateTime} = "01/01/2000 00:00:00";
                $bridge_core_info->{Size} = "11111111";
                $bridge_core_info->{Reason} = "FATAL: Dummy interrupt occurred:";
                return ($bridge_core_info);
           } else {
                my $bridge_core_info = {};

                my @core_info = split(/\r*\n/,$output);
                @core_info = grep /\S+/,@core_info; #Remove spaced line(s), if any.
                foreach my $core_line (@core_info) {
                     my @core_fields = split (/=/,$core_line);
                     $core_fields[1] =~ s/^\s+//; #Remove white space ONLY from beginning of the value. No need to remove trailing white spaces from key.
                     $core_fields[1] =~ s/\s+$//; #Remove white space ONLY from ending of the value. No need to remove trailing white spaces from key.

                     if ($core_fields[0] eq $opts{bridge_cmd}) {
                         next;
                     } elsif ($core_fields[0] =~ /^Core Dump Version/) {
                         $bridge_core_info->{CoreDumpVersion} = $core_fields[1];
                     } elsif ($core_fields[0] =~ /^Build Number/) {
                         $bridge_core_info->{BuildNumber} = $core_fields[1];
                     } elsif ($core_fields[0] =~ /^Date/) {
                         $bridge_core_info->{DateTime} = $core_fields[1];
                     } elsif ($core_fields[0] =~ /^Size/) {
                         $bridge_core_info->{Size} = $core_fields[1];
                     } elsif ($core_fields[0] =~ /^Reason/) {
                         $bridge_core_info->{Reason} = $core_fields[1];
                     } else {
                         print "$core_fields[0] --- $core_fields[1] \n";
                         logresult ("FAIL","We should not be here unless new info is added");
                     }
                 }
                 return ($bridge_core_info);
            }
     } else {
            logcomment ("This subroutine do not have defined formatted output for $cmd command - Returning the raw output.");
            return $output;
     }
  } else {
            return $output;
  }
}
sub get_atto_core_file_name{
        my (%opts) = validate(@_, {
                             bridge_name => {type=>OBJECT, isa=>'Hostrec'},
                             conntype => {type=>SCALAR,default=>"#telnet",optional=>1},
                             user => {type=>SCALAR,default=>"root",optional=>1},
                             password => {type=>SCALAR,default=>"Password",optional=>1},
                             timeout => {type=>SCALAR, default=>600, optional=>1 },
                            }
                            );
        my $output = run_atto_cmd(bridge_name=>$opts{bridge_name},bridge_cmd=>"get serialnumber", conntype=>$opts{conntype}, raw_output=>1);
        my @seq_info = split (/\r*\n/, $output);
        @seq_info = grep /^Part Identifier/,@seq_info; #Remove all other lines.
        my @seq_info2 = split (/:/, $seq_info[0]);
        $seq_info2[2] =~ s/^\s+//;
        my $core_file_name = "0N" . $seq_info2[2] . ".mem";
        return $core_file_name;
}

sub download_atto_firmware{
    my (%opts) = validate(@_, {
                         bridge_name => { type=>OBJECT, isa=>'Hostrec' },
                             fw_file => { type=>SCALAR },
                 port => {type=>SCALAR,default=>"mp1",optional=>1},
                 conntype => {type=>SCALAR,default=>"#telnet",optional=>1},
                 user => {type=>SCALAR,default=>"root",optional=>1},
                 password => {type=>SCALAR,default=>"Password",optional=>1},
                            }
                            );
     my $br = $opts{bridge_name};
     my $fw_file = $opts{fw_file};
     my $port = $opts{port};
     my $user = $opts{user};
     my $passwd = $opts{password};

     my $bridge_ip = run_atto_cmd(bridge_name=>$opts{bridge_name},bridge_cmd=>"get ipaddress $opts{port}", conntype=>$opts{conntype}, raw_output=>1);
     my @port_info = split (/\r*\n/, $bridge_ip);
     @port_info = grep /^IPAddress/,@port_info; #Remove all other lines.
     my @port_info2 = split (/=/, $port_info[0]);
     $port_info2[1] =~ s/^\s+//;
     $port_info2[1] =~ s/\s+$//;

     my $ftp_conn = Net::FTP->new($port_info2[1], Debug => 3) or die "Cannot connected to the atto bridge. (Pl. check if nate_host file has proper IP address defined and atto bridge is reachable. \n";
     $ftp_conn->login($user,$passwd) or die "Cannot login";
     $ftp_conn->binary;
     logcomment("Downloading $fw_file on ATTO Bridge");
     $ftp_conn->put($fw_file) or die "Firmware download failed ",$ftp_conn->message;
     logcomment("Firmware download complete");
     $ftp_conn->quit;
     return 1;
}


sub retrieve_atto_core{
    my (%opts) = validate(@_, {
                         bridge_name => { type=>OBJECT, isa=>'Hostrec' },
                 port => {type=>SCALAR,default=>"mp1",optional=>1},
                 conntype => {type=>SCALAR,default=>"#telnet",optional=>1},
                 user => {type=>SCALAR,default=>"root",optional=>1},
                 password => {type=>SCALAR,default=>"Password",optional=>1},
                            }
                            );
     my $br = $opts{bridge_name};
     my $user = $opts{user};
     my $passwd = $opts{password};

     my $core_file = get_atto_core_file_name(bridge_name=>$opts{bridge_name});

     my $bridge_ip = run_atto_cmd(bridge_name=>$opts{bridge_name},bridge_cmd=>"get ipaddress $opts{port}", conntype=>$opts{conntype}, raw_output=>1);
     my @port_info = split (/\r*\n/, $bridge_ip);
     @port_info = grep /^IPAddress/,@port_info; #Remove all other lines.
     my @port_info2 = split (/=/, $port_info[0]);
     $port_info2[1] =~ s/^\s+//;
     $port_info2[1] =~ s/\s+$//;

     my $ftp_conn = Net::FTP->new($port_info2[1], Debug => 3) or die "Cannot connected to the atto bridge. (Pl. check if nate_host file has proper IP address defined and atto bridge is reachable. \n";
     $ftp_conn->login($user,$passwd) or die "Cannot login";
     $ftp_conn->binary;
     logcomment("Retrieving $core_file from ATTO Bridge");
     $ftp_conn->get($core_file) or die "Core retrieval failed ",$ftp_conn->message;
     logcomment("Core $core_file retrieved successfully");
     $ftp_conn->quit;
     return 1;
}

=head2 storage_show_initiators

    my $hash = storage_show_initiators(host=>$host);

This routine returns a pointer to hash table of storage initiatorss information in the filer. It dies on failure.
It displays the host name and system id of other filers in a Shared Storage configuration. The local host is denoted by the word self after its system ID.

Example of hash returned by the function:
$VAR1 = {
          'step-u16' => {
                          'SystemInfo' => 'not member of shared backend.',
                          'SystemID' => '0050413978'
                        },
          'step-u17' => {
                          'SystemInfo' => 'self -- not member of shared backend.',
                          'SystemID' => '0050414395'
                        }
        };

=over

=item host

(Required) : Filer connnection to use.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_show_initiators {
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                             timeout => { type=>SCALAR, default=>300, optional=>1 },
                             cmd_opt => {type=>SCALAR,default=>"-a",optional=>1},
                            }
                     );
    my $init_struct = {};
    my @fields = ();
    my ($cmd, $output, $initlist, @initlist, @initiator, $hostname);



        $cmd = "storage show initiators $opts{cmd_opt}";



        $output = filer_cmd (host=>$opts{host}, filermode=>$opts{filermode}, conntype=>$opts{conntype}, timeout=>$opts{timeout}, cmd=>$cmd);

        # Check for command failure either through ems or one of the
        # possible values
        if (!($output =~ /SHARED STORAGE HOSTNAME/)) {
            croak $output;
        }
    @initlist = split(/\r*\n/,$output);
    foreach $initlist (@initlist) {
        if ($initlist =~ /SHARED STORAGE HOSTNAME|-------------/) {
            goto LABEL;
        }
        @fields = split(/\s\s+|\s\(|\s\-\-\s/, $initlist);
        $fields[2] =~ s/\)/ / if(defined $fields[2]);
        $hostname = $fields[0];
        $init_struct->{$hostname}->{SystemID} = $fields[1];
        $init_struct->{$hostname}->{SystemInfo} = $fields[2];
        LABEL: ;
    }
    #print Dumper($init_struct);
    return ($init_struct);
}

=head2  storage_disable_adapter
    my $ret = storage_disable_adapter(host=>$host, adapter_name => $adapter_name,timeout => $timeout);

This routine disables and stops I/O traffic on adapters and switches
It returns 1 on success and dies on failure.

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required. Default is all.

=item cmd_opt

(optional) : default to "";
    -f option forces the adapter to be taken offline

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_disable_adapter($@) {
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                         adapter_name => { type=>SCALAR },
                             cmd_opt => { type=>SCALAR, default=>" ", optional=>1 },
                 timeout => { type=>SCALAR, default=>300, optional=>1 },
                 ems_check => { type=>SCALAR, default=>"no" , optional=>1 },
                             exp_result => { type=>SCALAR, default=>"pass" , optional=>1 },
                            }
                     );

    my $cmd = "storage disable $opts{cmd_opt} adapter $opts{adapter_name}";


        my $mark;
        if($opts{ems_check} eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
        }

        my $output = filer_cmd (host=>$opts{host}, filermode=>$opts{filermode}, conntype=>$opts{conntype}, timeout=>$opts{timeout}, cmd=>$cmd);
        if($opts{exp_result} eq "pass") {
                if($output !~ /Host adapter $opts{adapter_name} disable succeeded|Disabling adapter $opts{adapter_name}.../){
                        croak $output;
                }
                if($opts{ems_check} eq "yes") {
                        sleep 30; # Wait for EMS flush or CP? - Divyesh.
                                my $ret = ems_log_check(host=>$opts{host}, eventtype=>"fci.adapter.offline", key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
                        if ($ret == 0) {
                                logcomment("The 'fci.adapter.offline' event is not found in the ems log");
                                croak $output;
                        } else {
                                logcomment("Found the 'fci.adapter.offline' ems event $ret times in the ems log");
                        }
                }
        } else {
                return $output;
        }

    #return 1;
}

=head2  storage_enable_adapter
    my $ret = storage_enable_adapter(host=>$host, adapter_name => $adapter_name,timeout => $timeout);

This routine enables and starts I/O traffic on an adapter
It returns 1 on success and dies on failure.

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item adapter_name

(Required) : Specific channel for which the information is
 required. Default is all.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_enable_adapter($@) {
    my (%opts) = validate(@_, {
                         host => { type=>OBJECT, isa=>'Hostrec' },
                             filermode => {type=>SCALAR,default=>"normal",optional=>1 },
                 conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                         adapter_name => { type=>SCALAR },
                 timeout => { type=>SCALAR, default=>300, optional=>1 },
                 ems_check => { type=>SCALAR, default=>"no", optional=>1 },
                             exp_result => { type=>SCALAR, default=>"pass" , optional=>1 },
                            }
                     );


    my $cmd = "storage enable adapter $opts{adapter_name}";


        my $mark;
        if($opts{ems_check} eq "yes"){
                $mark = ems_log_get_mark(host=>$opts{host});
        }

        my $output = filer_cmd (host=>$opts{host}, filermode=>$opts{filermode}, conntype=>$opts{conntype}, timeout=>$opts{timeout}, cmd=>$cmd);

        if($opts{exp_result} eq "pass") {
                if($output !~ /Host adapter $opts{adapter_name} enable succeeded/){
                        croak $output;
                }
                if($opts{ems_check} eq "yes") {
                        sleep 10; # Wait for EMS flush or CP? - Divyesh.
                                my $ret = ems_log_check(host=>$opts{host}, eventtype=>"fci.adapter.online", key=>"adapterName", value=>"$opts{adapter_name}", timestamp=>$mark);
                        if ($ret == 0) {
                                logcomment("The 'fci.adapter.online' event is not found in the ems log");
                                croak $output;
                        } else {
                                logcomment("Found the 'fci.adapter.online' ems event $ret times in the ems log");
                        }
                }
        } else {
                return $output;
        }
    #return 1;
}

=head2  storage_download_shelf
    my $ret = storage_download_shelf(host=>$host, channel_name => $channel_name, shelf_name => $shelf_name, file_name => $file_name, timeout => $timeout);

This routine Downloads new firmware to disk shelves attached to adapter
It returns 1 on success and dies on failure.

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item channel_name

(Optional) : Specific channel for which the new firmware will be updated to its attached shelves. Default to "".

=item shelf_name

(Optional) : Specific shelf for which the new firmware will be updated. Default to "".

=item cmd_opt

(optional) : default to "";
    -Q option starts download qualification test.
    -S option stops download qualification test.
    -R option allows the firmware to be reverted to the version shipped with the current Data ONTAP release.
    -F <filename> option forces download of specific firmware files to matching modules

=item force

(optional) : force the cmd to execute without prompt for question to go further, can be ""; Default to "-f";

=item file_name

(optional) : specific firmware file to be downloaded. Default to "";

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item wait
(Optional) : 1 - wait for shelf fw download to finish. Default is 1.
         0 - will not wait for shelf fw download to finish.

=back

=cut

sub storage_download_shelf ($@) {
    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                        channel_name => { type=>SCALAR, optional=>1 },
                        shelf_name => { type=>SCALAR, optional=>1 },
                    timeout => { type=>SCALAR, default=>300, optional=>1 },
                        cmd_opt => { type=>SCALAR, optional=>1 },
                        force => { type=>SCALAR, default=>"-f", optional=>1 },
                        file_name => { type=>SCALAR, optional=>1 },
                            wait=> { type => SCALAR,default=>1, optional => 1},
                }
                     );
    my ($host) = $opts{host};
    my ($conntype) = $opts{conntype};
    my ($wait) = $opts{wait};
    my ($timeout) = $opts{timeout};
    my $cmd = "";

    if (!defined($opts{cmd_opt})) {
        # Issue storage download shelf without options
        $cmd = "storage download shelf $opts{force} $opts{channel_name} $opts{shelf_name}";
        _shelffwcommand($host,$cmd,$conntype,$wait,$timeout);
    } elsif ($opts{cmd_opt} eq "-Q"){
        # Issue storage download shelf -Q for qulification test
        $cmd = "storage download shelf $opts{force} -Q";
        _shelffwcommand($host,$cmd,$conntype,$wait,$timeout);
    } elsif ($opts{cmd_opt} eq "-S") {
        # Issue storage download shelf -S to stop qulification test
        $cmd = "storage download shelf $opts{force} -S";
        _shelffwcommand($host,$cmd,$conntype,$wait,$timeout);
    } elsif ($opts{cmd_opt} eq "-R") {
        # Issue storage download shelf -R to revert firmware to
        # the version shipped with the current Data ONTAP version
        $cmd = "storage download shelf $opts{force} -R";
        _shelffwcommand($host,$cmd,$conntype,$wait,$timeout);
    } elsif ($opts{cmd_opt} eq "-F") {
        # Issue storage download shelf -F <filename> to force
        # download of specific firmware files to matching modules
        $cmd = "storage download shelf $opts{force} -F $opts{file_name}";
        _shelffwcommand($host,$cmd,$conntype,$wait,$timeout);
    } else {
            croak "Invalid $opts{cmd_opt} option";
    }

    return 1;
}

sub _shelffwcommand {
    my($host, $cmd, $conntype, $wait ,$timeout)=@_;
    my($output1, $output2);
    my($mark, $event_by_type, $event, $ret);
    my $conn = make_conn($host,$conntype); # will default to rsh connection

    $mark = ems_log_get_mark(host=>$host);
    $output1 = filer_cmd (host=>$host,conntype=>$conntype,timeout=>$timeout,cmd=>$cmd);
    print "Now the mark1 is $mark.\n";
    if ($output1 =~ /No shelves eligible for update|usage:/) {
        croak $output1;
    } elsif ($output1 =~ /still in progress/) {
        if ($cmd =~ /-S/) {
            sleep 120;
            $wait = 0;
            for (my $i = 0; $i < 20; $i++) {
                $ret = ems_log_check(host=>$host, eventtype=>"kern.syslog.msg", key=>"msg", value=>"DBG: ==== shelf_fw_update Qualification run STOPPED ====", timestamp=>$mark);
                if ($ret == 0) {
                    logcomment("The specified event is not yet found in the ems log");
                    sleep 60;
                } else {
                    logcomment("Found the specified ems event $ret times in the ems log");
                    last;
                }
            }
            if ($ret == 0) {
                logcomment("Did not get expect output");
                croak $output1;
            }
        } else {
            logcomment("FW update is still in progress");
            croak $output1;
        }
    } elsif ($output1 =~ /Error:/) {
        logcomment("Please check channel/shelf specified in the command or the option specified may not be supported in the non-debug build");
        croak $output1;

    } else {
        logcomment("Sleeping for 30 secs. before checking for sfu.downloadStarted event in the EMS logs");
        sleep 30;
        #print "Now the mark2 is $mark.\n";
        $event_by_type = ems_log_dump_by_type(host=>$host, timestamp=>$mark);
        printf "Now go into the ELSE.Output of $event_by_type\n";
        $event = $event_by_type->{"sfu.downloadStarted"};
        if ($event eq "") {
        logcomment ("Issuing $cmd failed.");
        croak "Shelf firmware download is not started yet";
        } else {
        logcomment ("Shelf firmware download started");
        }
    }
    # Decide whether it is needed to wait for shelf fw download to finish
    if ($wait != 0) {
        # Check ems log every minute up to 60 minutes
        for (my $i = 0; $i < 60; $i++) {
            $event_by_type = ems_log_dump_by_type(host=>$host, timestamp=>$mark);
            $event = $event_by_type->{"sfu.downloadSummary"};
            if ($event eq "") {
                sleep 60;
            } else {
                last;
            }
        }
        if ($event eq "") {
            croak "Shelf firmware download did not complete in given time";
        }
    }
}

sub storage_download_shelf_new ($@) {
    my (%opts) = validate(@_, {
    host => { type=>OBJECT, isa=>'Hostrec' },
    hostB => { type=>OBJECT, isa=>'Hostrec', default=>undef, optional=>1},
    conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
    timeout => { type=>SCALAR, default=>300, optional=>1 },
    channel_name => { type=>SCALAR | UNDEF , default=>undef, optional=>1 },
    encl_type => { type=>SCALAR | UNDEF , default=>undef, optional=>1 },
    encl_id => { type=>SCALAR | UNDEF, default=>undef, optional=>1 },
        #cmd_opt => { type=>SCALAR, default=>" ", optional=>1 },
    #cmd_opt => { type=>SCALAR, default=>"0", optional=>1, regex=>qr/^(0|-Q|-S|-R|-F|-Q -F|-M)$/ },
    cmd_opt => { type=>SCALAR | UNDEF, default=>undef, optional=>1 },
    file_name => { type=>SCALAR | UNDEF, default=>undef,optional=>1 },
    force => { type=>SCALAR, default=>"-f", optional=>1 },
    boot => { type=>SCALAR, default=>0, optional=>1 },
    reboot_timeout => { type=>SCALAR, default=>300, optional=>1 },
    takegive => { type=>SCALAR, default=>0, optional=>1 },
    wait_for_start => { type => SCALAR, default=>1, optional => 1},
    wait_for_summary => { type => SCALAR, default=>1, optional => 1},
    ems_mark => { type => SCALAR, default=>undef, optional => 1},
    ems_markB => { type => SCALAR, default=>undef, optional => 1},
    ems_check => { type => SCALAR, default=>1, optional => 1},
    additional_error_values => { type=>ARRAYREF, default=>undef, optional => 1},
    ignore_error_values => { type=>ARRAYREF, default=>undef, optional => 1},
    fatal_on_failure => { type => SCALAR, default=>"yes", optional => 1},
    }
     );

    # Setting up the command.

    my $cmd = "storage download shelf $opts{force}";

   if ( ($opts{cmd_opt} =~ /F/)  && (!defined $opts{file_name}) ) {
        logcomment("Filename must be specified with -F cmd_opt. \n");
        croak;
    }

    if (defined $opts{cmd_opt}) {
    $cmd .= " $opts{cmd_opt}";
    if ($opts{file_name}) {
        $cmd .= " $opts{file_name}";
    }
    }

    if (defined $opts{channel_name}) {
    $cmd .= " -H $opts{channel_name}";
    }

    if (defined $opts{encl_type}) {
        $cmd .= " -T $opts{encl_type}";
    }

    if (defined $opts{encl_id}) {
        $cmd .= " -E $opts{encl_id}";
    }

    # Defining default ems error strings.
    my %ems_error_messages;
    my @ems_error_values;

    if ($opts{ems_check}) {
        %ems_error_messages=("00"=>"sfu.downloadSummaryErrors",
                    "01"=>"sfu.statusCheckFailure",
                    "02"=>"sfu.partnerInaccessible",
                    "03"=>"sfu.partnerRefusedUpdate",
                    "04"=>"sfu.partnerNotResponding",
                    "05"=>"sfu.firmwareUpToDate",
                    "06"=>"sfu.downloadFailed",
                    "07"=>"sfu.downloadError",
                    "08"=>"ses.fw.emptyFile",
                    "09"=>"ses.fw.resourceNotAvailable",
                    "10"=>"ses.shelf.ctrlFailErr",
                    "11"=>"ses.shelf.em.ctrlFailErr",
                    "12"=>"ses.status.ATFCXError",
                    "13"=>"ses.access.noShelfSES",
                    "14"=>"ses.access.sesUnavailable",
                    "15"=>"ses.access.noEnclServ",
                    "16"=>"ses.access.noMoreValidPaths",
                    "17"=>"ses.status.ESHError",
                    "18"=>"ses.status.displayError",
                    "19"=>"ses.status.upsError",
                    "20"=>"ses.status.electronicsError",
                    "21"=>"ses.status.temperatureError",
                    "22"=>"ses.status.fanError",
                    "23"=>"ses.status.psError",
                    "24"=>"esh.bypass.err.disk"
                    );
    @ems_error_values=values(%ems_error_messages);
    # It makes sense to add other strings here though commenting it.
    #if (defined $opts{additional_error_values}) {
    #####my @additional_error_values = split(/\s+|,/,$opts{additional_error_values});
    #     push @ems_error_values,@opts{additional_error_values};
    #}
    }
    my $ret;
    my $event_by_type;
    my $event;
    # Setting up EMS log markers.
    my $mark;
    my $markB;

    if (defined $opts{ems_mark}) {
        $mark = $opts{ems_mark};
    } else {
        $mark = ems_log_get_mark(host=>$opts{host});
    }

    if (defined $opts{hostB}) {
        if (defined $opts{ems_markB}) {
            $markB = $opts{ems_markB};
        } else {
            $markB = ems_log_get_mark(host=>$opts{hostB});
        }
    }

    my $boot = "$opts{boot}";
    my $takegive = "$opts{takegive}";
    my $reboot_timeout=$opts{reboot_timeout};
    if ($boot) {
        logcomment("Rebooting to kick off FW download.");
        my $time1 = timelocal( (localtime));
    my ($result, $state)=reboot($opts{host},timeout=>$reboot_timeout);
    my $time2 = timelocal( (localtime));
    my $time3 = $time2 - $time1;
    if($result)                  {
              logcomment("Rebooted the filer sucessfully. It took $time3 seconds to reboot");
    } else {
            storage_logresult(fatal_on_failure => $opts{fatal_on_failure}, string => "Failed to reboot the filer");
        }

 #  filer_cmd (host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},noprompt=>1,cmd=>"reboot"); # Need to replace by boot.pm's reboot function.
      #  sleep 180; # Assumption: FW download during boot will not hold up filer's boot process or result into unexpected takeover.This is why this need to be replaced by boot.pm's reboot function.
    } elsif($takegive) {
        if ($opts{wait_for_summary} == 0) {
             filer_cmd (host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
             sleep 30; # Wait for EMS flush.
        } else {
             logcomment("Checking for ems errors after successful takegive during FW download.");
        }
    } else {
        #my $output = Storage::SASUtils::filer_cmd (host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
        if ($opts{cmd_opt} =~ /S/){
            if($opts{host}->{hosttype} =~ /filer-ng/) {
                $cmd = "run local $cmd";
            }
                my $console_conn = make_conn($opts{host},"#console");
                my $output;
                $output = $console_conn->execute(
                    command => "$cmd",
                    timeout => 300,
                    match_table => [(".*Are you sure.*") => "y",]
                );
                sleep 30;
                if ($opts{wait_for_summary}) {
                    for (my $i = 0; $i < 20; $i++) {
                        $ret = ems_log_check(host=>$opts{host}, eventtype=>"kern.syslog.msg", key=>"msg", value=>"DBG: ==== shelf_fw_update Qualification run STOPPED ====&#x0A;", timestamp=>$mark);
                        if ($ret == 0) {
                            logcomment("The specified event is not yet found in the ems log");
                            sleep 60;
                        } else {
                            logcomment("Found the specified ems event $ret times in the ems log");
                            last;
                        }
                    }
                    if ($ret == 0) {
                        logcomment("Did not get expect output");
                        croak $output;
                    }
                } else {
                    logcomment("Suceesfully issued QUAL mode STOP command. Please allow sufficient time for its complete STOP before issuing the next FW download command. \n");
                }
                return 1;

        } else {
            my $output = filer_cmd (host=>$opts{host},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
            if ($output =~ /No shelves eligible for update|usage:/) {
                croak $output;
            } elsif ($output =~ /Error:/) {
                logcomment("Please check channel/shelf specified in the command or the option specified may not be supported in the non-debug build");
                croak $output;
            } elsif ($output =~ /still in progress/) {
                logcomment("FW update is still in progress");
                croak $output;
            } else {
                 sleep 60; # FW download command is successfully issued. Waiting for EMS flush. Observed the failure upto 30 seconds for EMS logging of start message. No need to rush for checking "sfu.downloadStarted" and hence bumping to 60.  - Divyesh.
            }
        }

    }
    # Verifying EMS event for start of FW download.
    if($opts{wait_for_start}) {
                # Lets verify if FW download is started or not. This is unconditional EMS check.
        $event_by_type = ems_log_dump_by_type(host=>$opts{host}, timestamp=>$mark);
        $event = $event_by_type->{"sfu.downloadStarted"};
        if ($event eq "") {
            logcomment ("FW update didn't kick off during boot OR it failed to issue $cmd failed.");
            croak "Shelf firmware download is not started yet";
        } else {
            logcomment ("Shelf firmware download started");
                }
    }

    # Verifying EMS event for summary of FW download.
    if ($opts{wait_for_summary} != 0) {
        # Lets wait upfront for couple of minutes for FW download to complete.
        sleep 120;
        # Check ems log every minute up to 30 minutes
    for (my $i = 0; $i < 31; $i++) {
        $event_by_type = ems_log_dump_by_type(host=>$opts{host}, timestamp=>$mark);
            $event = $event_by_type->{"sfu.downloadSummary"};
        if ($event eq "") {
        sleep 60;
                if ($i == 30) {
                croak "Shelf firmware download did not complete in given time";
                }
        } else {
        logcomment ("Shelf firmware download completed.");
                last;
            }
        }
    }

    # Verify if FW download encountered any errors or not.
    if($opts{ems_check}) {
        #Storage::Ems::ems_error_monitor(host=>$opts{host},mark=>$mark,error_values=>\@ems_error_values,additional_error_values=>\@{$opts{additional_error_values}},ignore_error_values=>\@{$opts{ignore_error_values}},fatal_on_failure=>$opts{fatal_on_failure});
        ems_error_monitor(host=>$opts{host},mark=>$mark,error_values=>\@ems_error_values,additional_error_values=>\@{$opts{additional_error_values}},ignore_error_values=>\@{$opts{ignore_error_values}},fatal_on_failure=>$opts{fatal_on_failure});
        if(defined $opts{hostB}) {
            ems_error_monitor(host=>$opts{hostB},mark=>$markB,error_values=>\@ems_error_values,additional_error_values=>\@{$opts{additional_error_values}},ignore_error_values=>\@{$opts{ignore_error_values}},fatal_on_failure=>$opts{fatal_on_failure});
        }
    }
    return 1;
}
=head2 storage_show_expander

    my $hash = storage_show_hub(host=>$host, expander_name=>$expander_name);

This routine returns a pointer to hash table of storage show expander information in the filer.
It dies on failure.
The output can contain:

Following is an example of the hash table returned by the function:

$VAR1 = {
          '50:0c:0f:f0:0a:45:15:3c' => {
                                         'A' => {
                                                  '11' => {
                                                            'InvalidWordCount' => '67',
                                                            'PartialPathTimeout' => '7',
                                                            'PhyChangeCount' => '20',
                                                            'RunningDisparityCount' => '66',
                                                            'LinkRate' => '3',
                                                            'LossDwordCount' => '5',
                                                            'PortState' => 'OK',
                                                            'PhyResetProblem' => '0',
                                                            'CRCErrorCount' => '0'
                                                          },
                                                  'OUT0' => {
                                                              'InvalidWordCount' => '0',
                                                              'PartialPathTimeout' => '0',
                                                              'PhyChangeCount' => '0',
                                                              'RunningDisparityCount' => '0',
                                                              'LinkRate' => 'NA',
                                                              'LossDwordCount' => '0',
                                                              'PortState' => 'UNUSED',
                                                              'PhyResetProblem' => '0',
                                                              'CRCErrorCount' => '0'
                                                            },
                                                  'IOMstate' => 'OK',
                                                  'IN2' => {
                                                             'InvalidWordCount' => '68',
                                                             'PartialPathTimeout' => '7',
                                                             'PhyChangeCount' => '11',
                                                             'RunningDisparityCount' => '67',
                                                             'LinkRate' => '3',
                                                             'LossDwordCount' => '5',
                                                             'PortState' => 'OK',
                                                             'PhyResetProblem' => '0',
                                                             'CRCErrorCount' => '0'
                                                           },
                                                  'Shelfstate' => 'ONLINE',
                                                  'Shelfname' => 'PARTNER.shelf99',
                                                  'Shelfid' => '99',
                                                  'IN3' => {
                                                             'InvalidWordCount' => '72',
                                                             'PartialPathTimeout' => '7',
                                                             'PhyChangeCount' => '11',
                                                             'RunningDisparityCount' => '71',
                                                             'LinkRate' => '3',
                                                             'LossDwordCount' => '5',
                                                             'PortState' => 'OK',
                                                             'PhyResetProblem' => '0',
                                                             'CRCErrorCount' => '0'
                                                           },
                                                  'Channel' => 'PARTNER',
                                                },
                                         'B' => {
                                                  '11' => {
                                                            'InvalidWordCount' => '91',
                                                            'PartialPathTimeout' => '7',
                                                            'PhyChangeCount' => '22',
                                                            'RunningDisparityCount' => '90',
                                                            'LinkRate' => '3',
                                                            'LossDwordCount' => '7',
                                                            'PortState' => 'OK',
                                                            'PhyResetProblem' => '0',
                                                            'CRCErrorCount' => '0'
                                                          },
                              .........
                                                          }
                              .......
                                                }
                                       }
        };


=over

=item host

(Required) : Filer connnection to use.

=item expander_name

(Optional) : Specific hub for which the information is required.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_show_expander {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            shelf_name => { type=>SCALAR, optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );
    my $iom_struct = {};
    my ($cmd, $output, $shelf_name, $hublist, @hub, @fields, $Diskid);

# no zapi so make connection
# check if filer in maintenance mode
    $cmd = "storage show expander $opts{shelf_name}";
    $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>$cmd);
# Check for command failure either through ems or one of the
# possible values
    if (!($output =~ /Shelf name:/)) {
        croak $output;
    }

    my @shelflist = split(/\r*\n/,$output);
    my $shelflist;
    my $shelfname;
    my @shelfline;
    my $channel;
    my $module;
    my $shelfid;
    my $shelfuid;
    my $shelfstate;
    my $phy;
    my $iomstate;
    my $shelfserial;

    foreach  $shelflist (@shelflist){
        @shelfline = split(/\s+/,$shelflist);
        if($shelflist =~ /^Shelf name:/){
            $shelfname = $shelfline[2];
            print "SHELF: $shelfname \n";
        }
        elsif($shelflist =~ /^Channel:/){

            $channel = $shelfline[1];
        }
        elsif($shelflist =~ /^Module:/){
            $module = $shelfline[1];
        }
        elsif($shelflist =~ /^Shelf id:/){
            $shelfid = $shelfline[2];
        }
        elsif($shelflist =~ /^Shelf UID:/){
            $shelfuid = $shelfline[2];
        }
        elsif($shelflist =~ /^Shelf S\/N:/){
            $shelfserial = $shelfline[2];
        }
        elsif($shelflist =~ /^Shelf state:/){
            $shelfstate = $shelfline[2];
        }
        elsif($shelflist =~ /^Module state:/){
            $iomstate = $shelfline[2];
            $iom_struct->{$shelfuid}->{$module}->{Shelfname} = $channel;
            $iom_struct->{$shelfuid}->{$module}->{Channel} = $channel;
#   $iom_struct->{$shelfuid}->{$module}->{Shelfid} = $shelfid;
            $iom_struct->{$shelfuid}->{$module}->{Shelfid} = "$channel" . "." . "$shelfid";
            $iom_struct->{$shelfuid}->{$module}->{Serial} = $shelfserial;
            $iom_struct->{$shelfuid}->{$module}->{Shelfstate} = $shelfstate;
            $iom_struct->{$shelfuid}->{$module}->{IOMstate} = $iomstate;
        }
        elsif($shelflist =~ /^\[IN/){

            $phy = $shelfline[0];
            $phy =~ s/\[//g;
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PortState} = $shelfline[2];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PartialPathTimeout} = $shelfline[3];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LinkRate} = $shelfline[4];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{InvalidWordCount} = $shelfline[5];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{RunningDisparityCount} = $shelfline[6];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LossDwordCount} = $shelfline[7];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyResetProblem} = $shelfline[8];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{CRCErrorCount} = $shelfline[9];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyChangeCount} = $shelfline[10];
        }
        elsif($shelflist =~ /^\[OUT/){

            $phy = $shelfline[0];
            $phy =~ s/\[|\]//g;
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PortState} = $shelfline[1];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PartialPathTimeout} = $shelfline[2];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LinkRate} = $shelfline[3];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{InvalidWordCount} = $shelfline[4];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{RunningDisparityCount} = $shelfline[5];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LossDwordCount} = $shelfline[6];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyResetProblem} = $shelfline[7];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{CRCErrorCount} = $shelfline[8];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyChangeCount} = $shelfline[9];
        }
        elsif($shelflist =~ /^\[SQR|^\[CIR|^\[SIL/){

            $phy = $shelfline[0];
            $phy =~ s/\[|\]//g;
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PortState} = $shelfline[1];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PartialPathTimeout} = $shelfline[2];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LinkRate} = $shelfline[3];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{InvalidWordCount} = $shelfline[4];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{RunningDisparityCount} = $shelfline[5];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LossDwordCount} = $shelfline[6];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyResetProblem} = $shelfline[7];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{CRCErrorCount} = $shelfline[8];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyChangeCount} = $shelfline[9];
        }

        elsif($shelflist =~ /^\[\s+\d+/){

            $phy = $shelfline[1];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PortState} = $shelfline[3];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PartialPathTimeout} = $shelfline[4];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LinkRate} = $shelfline[5];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{InvalidWordCount} = $shelfline[6];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{RunningDisparityCount} = $shelfline[7];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{LossDwordCount} = $shelfline[8];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyResetProblem} = $shelfline[9];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{CRCErrorCount} = $shelfline[10];
            $iom_struct->{$shelfuid}->{$module}->{$phy}->{PhyChangeCount} = $shelfline[11];
        }
    }
    #print Dumper($iom_struct);
    return ($iom_struct);
}






=head2 storage_show_acp

    my $hash = storage_show_acp(host=>$host);

This routine returns a pointer to hash table of storage show acp command
It croaks on failure.
Following is an example of the hash table returned by the function:

$VAR1 = {

          'IPAddress' => '198.15.1.148',

          'Status' => 'Active',

          '4d.22.B' => {

                         'Status' => 'active',

                         'IPAddress' => '198.15.1.146',

                         'FWVersion' => '05.00',

                         'ResetCnt' => '000'

                       },

          'Domain' => '198.15.1.0',

          '4d.02.A' => {

                         'Status' => 'active',

                         'IPAddress' => '198.15.1.180',

                         'FWVersion' => '05.00',

                         'ResetCnt' => '000'

                       },

          '4d.12.B' => {

                         'Status' => 'active',

                         'IPAddress' => '198.15.1.16',

                         'FWVersion' => '05.00',

                         'ResetCnt' => '000'

                       },

          'Interface' => 'e0d',

          '4d.22.A' => {

                         'Status' => 'active',

                         'IPAddress' => '198.15.1.223',

                         'FWVersion' => '05.00',

                         'ResetCnt' => '000'
                       },


          '4d.02.B' => {

                         'Status' => 'active',

                         'IPAddress' => '198.15.1.224',

                         'FWVersion' => '05.00',

                         'ResetCnt' => '000'

                       },

          'ACPStatus' => 'Enabled',

          'Netmask' => '255.255.255.0',

          'Connectivity' => 'Full Connectivity',

          '4d.12.A' => {

                         'Status' => 'active',

                         'IPAddress' => '198.15.1.244',

                         'FWVersion' => '05.00',

                         'ResetCnt' => '000'

                       }

    };


=over

=item host

(Required) : Filer connnection to use.

=item options

(Optional) : Options to use. Default is "-a".

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Default is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Default is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=back

=cut

sub storage_show_acp {
    my (%opts) = validate(@_, {
            host => { type=>OBJECT, isa=>'Hostrec' },
            filermode => {type=>SCALAR,default=>"normal",optional=>1 },
            conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
            options => { type=>SCALAR,default=>"-a", optional=>1 },
            timeout => { type=>SCALAR, default=>300, optional=>1 },
            }
            );

    my $hash;
    my $output = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},conntype=>$opts{conntype},timeout=>$opts{timeout},cmd=>"storage show acp $opts{options}");
#print "output is : $output \n";
    if ($output !~ /Alternate Control Path:/){
        croak $output;
    }
    my @acplist = split(/\r*\n/,$output);
    foreach my $line ( @acplist){
        if($line =~ /:\s+/){
            $line =~ s/^\s+//;
            $line =~ s/\s+$//;
            my @info = split(/:\s+/,$line);
            if($info[0] =~ /Alternate Control/){
                $hash->{ACPStatus} = $info[1];
            }
            elsif($info[0] =~ /Ethernet Interface/){
                $hash->{Interface} = $info[1];
            }
            elsif($info[0] =~ /ACP Status/){
                $hash->{Status} = $info[1];
            }
            elsif($info[0] =~ /ACP domain|ACP Domain|ACP Subnet/){
                $hash->{Domain} = $info[1];
            }
            elsif($info[0] =~ /netmask|Netmask/){
                $hash->{Netmask} = $info[1];
            }
            elsif($info[0] =~ /ACP Connectivity Status/){
                $hash->{Connectivity} = $info[1];
            }
            elsif($info[0] =~ /ACP Partner Connectivity Status/){
                $hash->{PartnerConnectivity} = $info[1];
            }
            elsif($info[0] =~ /ACP IP address|ACP IP Address/){
                $hash->{IPAddress} = $info[1];
            }
            else{
                logcomment("Unknown line : $line");
            }

        }
        elsif($line =~ /\d+\.\d+\.\d+\.\d+/){
            my @info = split(/\s+/,$line);
            $hash->{$info[0]}->{ResetCnt} = $info[1];
            $hash->{$info[0]}->{IPAddress} = $info[2];
            $hash->{$info[0]}->{FWVersion} = $info[3];
            if($output =~ /Module Type/){
                $hash->{$info[0]}->{ModuleType} = $info[4];
                $hash->{$info[0]}->{Status} = $info[5];
            }
            else{
                $hash->{$info[0]}->{Status} = $info[4];
            }
        }
    }
    return $hash;
}


=head2 storage_show_fault(%opts);

The routine returns the information that can be retrieved via
"storage show fault [-a] [-v] shelf_name" command.

The output looks like:

$VAR1 = {
          '4a' => {
                    '22' => {
                              'FAN' => {
                                         '6' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '6',
                                                  'StatusBytes' => '01,00,F7,A1'
                                                },
                                         '3' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '3',
                                                  'StatusBytes' => '01,01,1D,A1'
                                                },
                                         '7' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '7',
                                                  'StatusBytes' => '01,01,24,A1'
                                                },
                                         '2' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '2',
                                                  'StatusBytes' => '01,01,15,A1'
                                                },
                                         '8' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '8',
                                                  'StatusBytes' => '01,01,06,A1'
                                                },
                                         '1' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '1',
                                                  'StatusBytes' => '01,01,24,A1'
                                                },
                                         '4' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '4',
                                                  'StatusBytes' => '01,01,24,A1'
                                                },
                                         '5' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '5',
                                                  'StatusBytes' => '01,01,2C,A1'
                                                }
                                       },
                              'VenUniIOM83' => {
                                                 '1' => {
                                                          'StatusDesc' => 'MASTER',
                                                          'Status' => 'OK',
                                                          'Element' => '1',
                                                          'StatusBytes' => '01,08,00,00'
                                                        },
                                                 '2' => {
                                                          'StatusDesc' => undef,
                                                          'Status' => 'OK',
                                                          'Element' => '2',
                                                          'StatusBytes' => '01,00,00,00'
                                                        }
                                               },
                              'EnclElect' => {
                                               '1' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '1',
                                                        'StatusBytes' => '01,00,00,80'
                                                      },
                                               '2' => {
                                                        'StatusDesc' => 'REPORT',
                                                        'Status' => 'OK',
                                                        'Element' => '2',
                                                        'StatusBytes' => '01,00,01,80'
                                                      }
                                             },
                              'OPS' => {
                                         '1' => {
                                                  'StatusDesc' => undef,
                                                  'Status' => 'OK',
                                                  'Element' => '1',
                                                  'StatusBytes' => '01,00,00,00'
                                                }
                                       },
                              'CurSensor' => {
                                               '6' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '6',
                                                        'StatusBytes' => '01,00,A5,01'
                                                      },
                                               '3' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '3',
                                                        'StatusBytes' => '01,00,FA,00'
                                                      },
                                               '7' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '7',
                                                        'StatusBytes' => '01,00,E6,00'
                                                      },
                                               '2' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '2',
                                                        'StatusBytes' => '01,00,3E,02'
                                                      },
                                               '8' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '8',
                                                        'StatusBytes' => '01,00,6B,01'
                                                      },
                                               '1' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '1',
                                                        'StatusBytes' => '01,00,21,01'
                                                      },
                                               '4' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '4',
                                                        'StatusBytes' => '01,00,7A,01'
                                                      },
                                               '5' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '5',
                                                        'StatusBytes' => '01,00,01,01'
                                                      }
                                             },
                              'VolSensor' => {
                                               '6' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '6',
                                                        'StatusBytes' => '01,00,C6,04'
                                                      },
                                               '3' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '3',
                                                        'StatusBytes' => '01,00,FB,01'
                                                      },
                                               '7' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '7',
                                                        'StatusBytes' => '01,00,FB,01'
                                                      },
                                               '2' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '2',
                                                        'StatusBytes' => '01,00,B7,04'
                                                      },
                                               '8' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '8',
                                                        'StatusBytes' => '01,00,BE,04'
                                                      },
                                               '1' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '1',
                                                        'StatusBytes' => '01,00,FB,01'
                                                      },
                                               '4' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '4',
                                                        'StatusBytes' => '01,00,C6,04'
                                                      },
                                               '5' => {
                                                        'StatusDesc' => undef,
                                                        'Status' => 'OK',
                                                        'Element' => '5',
                                                        'StatusBytes' => '01,00,FB,01'
                                                      }
                                             },
                              'ProductSerialNumber' => 'SHX0931422G000D',
                              'TempSensor' => {
                                                '6' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '6',
                                                         'StatusBytes' => '01,00,4A,00'
                                                       },
                                                '11' => {
                                                          'StatusDesc' => undef,
                                                          'Status' => 'OK',
                                                          'Element' => '11',
                                                          'StatusBytes' => '01,00,3B,00'
                                                        },
                                                '3' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '3',
                                                         'StatusBytes' => '01,00,30,00'
                                                       },
                                                '7' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '7',
                                                         'StatusBytes' => '01,00,3D,00'
                                                       },
                                                '9' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '9',
                                                         'StatusBytes' => '01,00,3B,00'
                                                       },
                                                '12' => {
                                                          'StatusDesc' => undef,
                                                          'Status' => 'OK',
                                                          'Element' => '12',
                                                          'StatusBytes' => '01,00,40,00'
                                                        },
                                                '2' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '2',
                                                         'StatusBytes' => '01,00,39,00'
                                                       },
                                                '8' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '8',
                                                         'StatusBytes' => '01,00,4A,00'
                                                       },
                                                '1' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '1',
                                                         'StatusBytes' => '01,00,2F,00'
                                                       },
                                                '4' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '4',
                                                         'StatusBytes' => '01,00,3C,00'
                                                       },
                                                '10' => {
                                                          'StatusDesc' => undef,
                                                          'Status' => 'OK',
                                                          'Element' => '10',
                                                          'StatusBytes' => '01,00,50,00'
                                                        },
                                                '5' => {
                                                         'StatusDesc' => undef,
                                                         'Status' => 'OK',
                                                         'Element' => '5',
                                                         'StatusBytes' => '01,00,3B,00'
                                                       }
                                              },
                              'EnclosureStatus' => 'normal',
                              'PSU' => {
                                         '4' => {
                                                  'StatusDesc' => 'RQSTED',
                                                  'Status' => 'OK',
                                                  'Element' => '4',
                                                  'StatusBytes' => '01,00,00,A0'
                                                },
                                         '1' => {
                                                  'StatusDesc' => 'RQSTED',
                                                  'Status' => 'OK',
                                                  'Element' => '1',
                                                  'StatusBytes' => '01,00,00,A0'
                                                },
                                         '3' => {
                                                  'StatusDesc' => 'RQSTED',
                                                  'Status' => 'OK',
                                                  'Element' => '3',
                                                  'StatusBytes' => '01,00,00,A0'
                                                },
                                         '2' => {
                                                  'StatusDesc' => 'RQSTED',
                                                  'Status' => 'OK',
                                                  'Element' => '2',
                                                  'StatusBytes' => '01,00,00,A0'
                                                }
                                       },
                              'Disk' => {
                                          '11' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '11',
                                                    'StatusBytes' => '01,0B,00,00'
                                                  },
                                          '21' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '21',
                                                    'StatusBytes' => '01,15,00,00'
                                                  },
                                          '7' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '7',
                                                   'StatusBytes' => '01,07,00,00'
                                                 },
                                          '17' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '17',
                                                    'StatusBytes' => '01,11,00,00'
                                                  },
                                          '2' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '2',
                                                   'StatusBytes' => '01,02,00,00'
                                                 },
                                          '22' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '22',
                                                    'StatusBytes' => '01,16,00,00'
                                                  },
                                          '1' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '1',
                                                   'StatusBytes' => '01,01,00,00'
                                                 },
                                          '18' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '18',
                                                    'StatusBytes' => '01,12,00,00'
                                                  },
                                          '0' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '0',
                                                   'StatusBytes' => '01,00,00,00'
                                                 },
                                          '23' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '23',
                                                    'StatusBytes' => '01,17,00,00'
                                                  },
                                          '16' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '16',
                                                    'StatusBytes' => '01,10,00,00'
                                                  },
                                          '13' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '13',
                                                    'StatusBytes' => '01,0D,00,00'
                                                  },
                                          '6' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '6',
                                                   'StatusBytes' => '01,06,00,00'
                                                 },
                                          '3' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'NOT INSTALLED',
                                                   'Bay' => '3',
                                                   'StatusBytes' => '05,03,00,00'
                                                 },
                                          '9' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '9',
                                                   'StatusBytes' => '01,09,00,00'
                                                 },
                                          '12' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '12',
                                                    'StatusBytes' => '01,0C,00,00'
                                                  },
                                          '20' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '20',
                                                    'StatusBytes' => '01,14,00,00'
                                                  },
                                          '14' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '14',
                                                    'StatusBytes' => '01,0E,00,00'
                                                  },
                                          '15' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '15',
                                                    'StatusBytes' => '01,0F,00,00'
                                                  },
                                          '8' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '8',
                                                   'StatusBytes' => '01,08,00,00'
                                                 },
                                          '4' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '4',
                                                   'StatusBytes' => '01,04,00,00'
                                                 },
                                          '19' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '19',
                                                    'StatusBytes' => '01,13,00,00'
                                                  },
                                          '10' => {
                                                    'StatusDesc' => undef,
                                                    'Status' => 'OK',
                                                    'Bay' => '10',
                                                    'StatusBytes' => '01,0A,00,00'
                                                  },
                                          '5' => {
                                                   'StatusDesc' => undef,
                                                   'Status' => 'OK',
                                                   'Bay' => '5',
                                                   'StatusBytes' => '01,05,00,00'
                                                 }
                                        },
                              'ShelfType' => 'DS4243',
                              'VenUniIOM85' => {
                                                 '1' => {
                                                          'StatusDesc' => undef,
                                                          'Status' => 'OK',
                                                          'Element' => '1',
                                                          'StatusBytes' => '01,00,00,00'
                                                        },
                                                 '2' => {
                                                          'StatusDesc' => undef,
                                                          'Status' => 'OK',
                                                          'Element' => '2',
                                                          'StatusBytes' => '01,00,00,00'
                                                        }
                                               },
                              'ModuleType' => 'IOM3',
                              'SasConnector' => {
                                                  '4' => {
                                                           'StatusDesc' => undef,
                                                           'Status' => 'OK',
                                                           'Element' => '4',
                                                           'StatusBytes' => '01,03,FF,00'
                                                         },
                                                  '1' => {
                                                           'StatusDesc' => undef,
                                                           'Status' => 'OK',
                                                           'Element' => '1',
                                                           'StatusBytes' => '01,03,FF,00'
                                                         },
                                                  '3' => {
                                                           'StatusDesc' => undef,
                                                           'Status' => 'OK',
                                                           'Element' => '3',
                                                           'StatusBytes' => '01,03,FF,00'
                                                         },
                                                  '2' => {
                                                           'StatusDesc' => undef,
                                                           'Status' => 'OK',
                                                           'Element' => '2',
                                                           'StatusBytes' => '01,03,FF,00'
                                                         }
                                                }
                            },
                    '12' =>

=over

=item host
(Required) : Host spec to the FILER

=item adapt
(Required) : Adapter of the FILER

=item conntype
(Optional) : Connection type to the Filer. Default is rsh.

=item filermode
(Optional): the state of the filer. Default is normal.


=item Return

Return the hash of "storage show fault" command output.

=item Example:

    $ret = storage_show_fault(%opts);

=item SEE ALSO
   None

=back

=cut

sub storage_show_fault {

    my (%opts) = validate(
            @_,
            {
            host     => { type => OBJECT, isa     => 'Hostrec' },
            filermode => { type => SCALAR, default => "normal", optional => 1 },
            #conntype => { type => SCALAR, default => "#rsh", optional => 1 },
            #timeout   => { type => SCALAR, default => 300, optional => 1 },
            cmd_opt     => { type => SCALAR, default=>"-v", optional=>1 },
            #option     => { type => SCALAR, default=>" ", optional=>1 },
            shelf_name     => { type => SCALAR, default=>" ", optional=>1 },
            }
            );

    #if ( $opts{filermode} ne "normal" ) {
    #   $opts{conntype} = "#console";
    #}

    my (
            $cmd, $outputinfo, @info, $line, @status,
                        $element_type, $enclosure_status,
                        $specialchannel, $specialshelf, $diskid
       );

    my %HOH;
    $outputinfo = filer_cmd(host=>$opts{host},filermode=>$opts{filermode},cmd=>$cmd);

    if ( $outputinfo =~ /Illegal option|Invalid shelf name/ ) {
        croak $outputinfo;
    }

    if ($opts{cmd_opt} eq "-v") {
        if ( $outputinfo !~ /Enclosure Status/ ) {
            croak $outputinfo;
        }
    } else {
        return $outputinfo;
    }

    @info = split( /\r?\n/, $outputinfo );
    foreach $line (@info) {

                # skip blank lines
        if ( $line eq '' ) {
            next;
        }

        $line =~ s/^\s*//; # Remove the white space(s) from the beginning of the line.

        @status = split( /\s+/, $line ); # Start processing each line.

        if ( defined($line) ) {      #### I am not sure why this check was done. No output at all?

            if ( $line =~ /^Enclosure Status:/ ) {
                $element_type = " "; # Reset the Element Type before starting processing of next shelf data.
                $enclosure_status = $status[2];
            }
            elsif ( $line =~ /^Channel:/ ) {
                $specialchannel = $status[1];
            }
            elsif ( $line =~ /^Shelf:/ ) {
                $specialshelf = $status[1];
            }
            elsif ( $line =~ /^Shelf Type:/ ) {
                $HOH{$specialchannel}{$specialshelf}{'ShelfType'} = $status[2];
                $HOH{$specialchannel}{$specialshelf}{'EnclosureStatus'} = $enclosure_status;
            }
            elsif ( $line =~ /^Product Serial Number:/ ) {
                $HOH{$specialchannel}{$specialshelf}{'ProductSerialNumber'} = $status[3];
            }
            elsif ( $line =~ /^Module Type:/ ) {
                $HOH{$specialchannel}{$specialshelf}{'ModuleType'} = $status[2];
            }
            elsif ( $line =~ /\[Bay/ ) {
                $element_type = "Disk";
                $diskid = $status[0];
                $status[2] =~ s/]://;
                if ($status[4] !~ /,/) {
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'Bay'} = $status[2];
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'Status'} = "$status[3] $status[4]";
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'StatusBytes'} = $status[5];
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'StatusDesc'} = $status[6];
                } else {
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'Bay'} = $status[2];
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'Status'} = $status[3];
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'StatusBytes'} = $status[4];
                    $HOH{$specialchannel}{$specialshelf}{$element_type}{$diskid}{'StatusDesc'} = $status[5];
                }
            }
            elsif ( $line =~ /^Power Supplies:/ ) {
                $element_type = "PSU";
            }
            elsif ( $line =~ /^Fans:/ ) {
                $element_type = "FAN";
            }
            elsif ( $line =~ /^Temperature Sensors:/ ) {
                $element_type = "TempSensor";
            }
            elsif ( $line =~ /^Enclosure Electronics:/ ) {
                $element_type = "EnclElect";
            }
            elsif ( $line =~ /^OPS Panel:/ ) {
                $element_type = "OPS";
            }
            elsif ( $line =~ /^Voltage Sensors:/ ) {
                $element_type = "VolSensor";
            }
            elsif ( $line =~ /^Current Sensors:/ ) {
                $element_type = "CurSensor";
            }
            elsif ( $line =~ /^SAS Connectors:/ ) {
                $element_type = "SasConnector";
            }
            elsif ( $line =~ /^Vendor Unique Element 83-IOM3:/ ) {
                $element_type = "VenUniIOM83";
            }
            elsif ( $line =~ /^Vendor Unique Element 85-IOM3:/ ) {
                $element_type = "VenUniIOM85";
            }
            elsif ( $line =~ /^Vendor Unique Element 82-AT-FCX:/ ) {
                $element_type = "VenUniATFCX82";
            }
            elsif ( $line =~ /^Vendor Unique Element 80-ESH4:/ ) {
                $element_type = "VenUniESH480";
            }
            elsif ( $line =~ /^Vendor Unique Element 84-ESH4:/ ) {
                $element_type = "VenUniESH484";
            }
            elsif ( $line =~ /^Audible Alarm:/ ) {
                $element_type = "AudAlarm";
            }
            elsif ( $line =~ /\d/ ) {
                if ($element_type =~ /PSU|FAN|TempSensor|OPS|VolSensor|CurSensor|AudAlarm|VenUniESH484/) {
                    $status[0] =~ s/://;
                    if ($status[2] !~ /,/) {
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Element'} = $status[0];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Status'} = "$status[1] $status[2]";
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusBytes'} = $status[3];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusDesc'} = $status[4];
                    } else {
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Element'} = $status[0];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Status'} = $status[1];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusBytes'} = $status[2];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusDesc'} = $status[3];
                    }
                } elsif ($element_type =~ /VenUniESH480|VenUniATFCX82|VenUniIOM85|VenUniIOM83|SasConnector|EnclElect/){
                    $status[0] =~ s/://;
                    if ($status[5] !~ /,/) {
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Element'} = $status[0];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Status'} = "$status[4] $status[5]";
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusBytes'} = $status[6];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusDesc'} = $status[7];
                    } else {
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Element'} = $status[0];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'Status'} = $status[4];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusBytes'} = $status[5];
                        $HOH{$specialchannel}{$specialshelf}{$element_type}{$status[0]}{'StatusDesc'} = $status[6];
                    }

                }
            }
        }
    }
    return \%HOH;
}


#----------------------------------------------------------------------
# The testme routine to test the exported functions of this library
#
# ntest PKGNAME=Storage::Storage SUBNAME=testme util/test_pkg.thpl
#
# The routine can also be invoked with many optional parameters, such as:
#
# ntest PKGNAME=Storage::Storage SUBNAME=testme FUNCTION_NAME=storage_download_shelf CHANNEL_NAME=10a util/test_pkg.thpl
#
# ntest PKGNAME=Storage::Storage SUBNAME=testme FUNCTION_NAME=storage_disable_adapter FILERMODE=maint util/test_pkg.thpl
#
# ...
#----------------------------------------------------------------------
sub testme {

    package main;
    our ($FILER,$FUNCTION_NAME,$FILERMODE,$FILE_NAME,$CHANNEL_NAME,$WAIT);
    package Storage::Storage;
    param('FILER',       '-mesg',    "Filer to test");
    param('FUNCTION_NAME',   '-default',    "all");
    param('FILERMODE',       '-default',    "normal");
    param('FILE_NAME',       '-default',    "ESH2.1430.SFW");
    param('CHANNEL_NAME',    '-default',    "");
    param('WAIT',       '-default',    1);

    logcomment("Testing on FILER $FILER, Function $FUNCTION_NAME");
    logcomment("Testing Filermode $FILERMODE");
    my $host = host($FILER);
    my $ret;

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "storage_show_adapter"){
            # test rsh connection + normal mode
        # OR rsh connection + maint mode
        # OR implicit console connection + maint mode depend on FILERMODE
        eval {
                $ret = storage_show_adapter(host=>$host, filermode=>$FILERMODE);
        };
        if ($@ =~ /Connection timed out/) {
            logresult('pass', "Expected fail. Check filer mode and connection type");
        } elsif ($@) {
            logresult('fail', "$@");
        } else {
            print Dumper($ret);
                logresult('pass', "Got expected result");
        }
        # test console connection + normal mode
        eval {
                $ret = storage_show_adapter(host=>$host, conntype=>"#console");
        };
        if ($@ =~ "Err=TIMEOUT waiting for waitstrings/prompts" ) {
            logresult('pass', "Expected fail. Check filer mode and connection type");
        } elsif ($@) {
            logresult('fail', "$@");
        } else {
            print Dumper($ret);
                logresult('pass', "Got expected result");
        }

        # test for nonexistant adapter
        eval {
                $ret = storage_show_adapter(host=>$host, adapter_name=>"abc", filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail', "Expected the routine to fail");
        } elsif ($@ !~ /Connection timed out|Invalid host adapter/) {
                logresult('fail',"Did not get expected failure message");
        } else {
                logresult('pass', "Got expected fail result");
        }
    }

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "storage_show_disk") {
        eval {
                $ret = storage_show_disk(host=>$host, filermode=>$FILERMODE);
        };
        if ($@ =~ /Connection timed out/) {
            logresult('pass', "Expected fail. Check filer mode and connection type");
        } elsif ($@) {
            logresult('fail', "$@");
        } else {
            print Dumper($ret);
                logresult('pass', "Got expected result");
        }
        # test for nonexistant disk
        eval {
                $ret = storage_show_disk(host=>$host, disk_name=>"abc", filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail', "Expected the routine to fail");
        } elsif ($@ !~ /Connection timed out|does not exist/) {
                logresult('fail',"Did not get expected failure message");
            } else {
                logresult('pass', "Got expected fail result");
            }
    }

    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "storage_show_hub") {
        eval {
                $ret = storage_show_hub(host=>$host, filermode=>$FILERMODE);
        };
        if ($@ =~ /Connection timed out/) {
            logresult('pass', "Expected fail. Check filer mode and connection type");
        } elsif ($@) {
            logresult('fail', "$@");
        } else {
            print Dumper($ret);
                logresult('pass', "Got expected result");
        }
        # test for nonexistant hub
        eval {
                $ret = storage_show_hub(host=>$host, hub_name=>"abc", filermode=>$FILERMODE);
        };
        if (!$@) {
            logresult('fail', "Expected the routine to fail");
        } elsif ($@ !~ /Connection timed out|Invalid hub name|Invalid host adapter/) {
                logresult('fail',"Did not get expected failure message");
            } else {
                logresult('pass', "Got expected fail result");
            }
    }
    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "storage_show_initiators") {
        eval {
                $ret = storage_show_initiators(host=>$host, filermode=>$FILERMODE);
        };
        if ($@ =~ /Connection timed out/) {
            logresult('pass', "Expected fail. Check filer mode and connection type");
        } elsif ($@) {
            logresult('fail', "$@");
        } else {
            print Dumper($ret);
                logresult('pass', "Got expected result");
        }
    }

    # test storage_disable_adapter and storage_enable_adapter
    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "storage_disable_adapter") {
        my $adap_info;
        eval{
            $adap_info = storage_show_adapter(host=>$host, filermode =>$FILERMODE);
        };
        foreach my $adap (keys %$adap_info) {
            # First check that loops are redundant before proceeding.
            if ( $adap_info->{$adap}->{'Redundant'} !~ /Yes/){
                logcomment("Adapter $adap is not redundant");
                next;
            } else {
                logcomment("Adapter $adap is redundant");
                # Start disabling adapter.
                eval {
                        $ret = storage_disable_adapter(host=>$host, adapter_name => $adap, filermode=>$FILERMODE);
                };
                if ($@) {
                    logresult('fail', "$@");
                } else {
                    #print Dumper($ret);
                        logresult('pass', "Adapter $adap is now offline");
                }
                # Start enabling adapter.
                eval {
                        $ret = storage_enable_adapter(host=>$host, adapter_name => $adap, filermode=>$FILERMODE);
                };
                if ($@) {
                    logresult('fail', "$@");
                } else {
                    #print Dumper($ret);
                        logresult('pass', "Adapter $adap is now online");
                }
                sleep 10;
            }
        }
    }

    # test storage_download_shelf
    if ($FUNCTION_NAME eq "all" || $FUNCTION_NAME eq "storage_download_shelf") {

        eval {
                $ret = storage_download_shelf(host=>$host, cmd_opt => "-Q", wait=>0);
        };
        if ($@) {
            if ($@ =~ /No shelves eligible for update/) {
                logresult('pass', "Got expected result");
            } elsif ($@ =~ /usage:/) {
                logresult('pass', "Expected Cmd failed, not work in maint mode");
            } else {
                logresult('fail', "$@");
            }
        } else {
                logresult('pass', "Got expected result");
        }
        eval {
                $ret = storage_download_shelf(host=>$host, cmd_opt => "-S");
        };
        if ($@) {
            if ($@ =~ /No shelves eligible for update/) {
                logresult('pass', "Got expected result");
            } elsif ($@ =~ /usage:/) {
                logresult('pass', "Expected Cmd failed, not work in maint mode");
            } else {
                logresult('fail', "$@");
            }
        } else {
                logresult('pass', "Got expected result");
        }
        eval {
                $ret = storage_download_shelf(host=>$host, cmd_opt => "-F", file_name=>$FILE_NAME);
        };
        if ($@) {
            if ($@ =~ /No shelves eligible for update/) {
                logresult('pass', "Got expected result");
            } elsif ($@ =~ /usage:/) {
                logresult('pass', "Expected Cmd failed, not work in maint mode");
            } else {
                logresult('fail', "$@");
            }
        } else {
                logresult('pass', "Got expected result");
        }
        eval {
                $ret = storage_download_shelf(host=>$host, cmd_opt => "-R");
        };
        if ($@) {
            if ($@ =~ /No shelves eligible for update/) {
                logresult('pass', "Got expected result");
            } elsif ($@ =~ /usage:/) {
                logresult('pass', "Expected Cmd failed, not work in maint mode");
            } else {
                logresult('fail', "$@");
            }
        } else {
                logresult('pass', "Got expected result");
        }
    }
}



=head2  storage_download_acp
        my $ret = storage_download_acp(host=>$host, expander_name => $expander_name, cmd_opt => $cmd_opt, file_name => $file_name, timeout => $timeout);

This routine downloads the ACP firmware to the attaches shelves
It returns 1 on success and dies on failure.

The following named arguments are accepted,

=over

=item host

(Required) : Hostspec to filer.

=item expander_name

(Required) : The expander to which the ACP firmware needs to be downloaded.

=item cmd_opt

(optional) : default to "";
        -Q option starts download qualification test.
        -S option stops download qualification test.
        -R option allows the firmware to be reverted to the version shipped with the current Data ONTAP release.
        -F <filename> option forces download of specific firmware files to matching modules

=item file_name

(Optional) : specific firmware file to be downloaded. Default to "";

=item conntype

(Optional) : The connection type to be created by this function to talk to the filer.  Default "#rsh"

=item timeout

(Optional) : timeout value. Default is 300 secs.

=item wait
(Optional) : 1 - wait for acp fw download to complete. Default is 1.
             0 - do not wait for fw download to complete

=back

=cut

sub storage_download_acp ($@) {
    my (%opts) = validate(@_, {
                        host => { type=>OBJECT, isa=>'Hostrec' },
                conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                        expander_name => { type=>SCALAR, optional=>1 },
                    timeout => { type=>SCALAR, default=>120, optional=>1 },
                        cmd_opt => { type=>SCALAR, optional=>1 },
                        file_name => { type=>SCALAR, optional=>1 },
                    monitor_time=> { type=>SCALAR, default=>5, optional=>1 },
                            wait=> { type => SCALAR,default=>1, optional => 1},
                }
                     );
    my ($host) = $opts{host};
    my ($conntype) = $opts{conntype};
    my ($wait) = $opts{wait};
    my ($timeout) = $opts{timeout};
    my $cmd = "";

    if (!defined($opts{cmd_opt})) {
        # Issue storage download acp without options
        if(defined($opts{expander_name})){
            $cmd = "storage download acp $opts{expander_name}";
                        $cmd = "run local ". $cmd if ($host->{hosttype} =~/filer-ng/);
        }
        else{
            $cmd = "storage download acp";
            $cmd = "run local ". $cmd if ($host->{hosttype} =~/filer-ng/);
        }

    } elsif ($opts{cmd_opt} eq "-Q"){
        # Issue storage download acp -Q for qulification test
        $cmd = "storage download acp -Q";
        $cmd = "run local ". $cmd if ($host->{hosttype} =~/filer-ng/);

    } elsif ($opts{cmd_opt} eq "-S") {
        # Issue storage download acp -S to stop qulification test
        $cmd = "storage download acp -S";
        $cmd = "run local ". $cmd if ($host->{hosttype} =~/filer-ng/);

    } elsif ($opts{cmd_opt} eq "-R") {
        # Issue storage download acp -R to revert firmware to
        # the version shipped with the current Data ONTAP version
        $cmd = "storage download acp -R $opts{expander_name}";
        $cmd = "run local ". $cmd if ($host->{hosttype} =~/filer-ng/);

    } elsif ($opts{cmd_opt} eq "-F") {
        # Issue storage download acp -F <filename> to force
        # download of specific firmware files to matching modules
        if(defined($opts{expander_name})){
            $cmd = "storage download acp -F $opts{file_name} $opts{expander_name}";
            $cmd = "run local ". $cmd if ($host->{hosttype} =~/filer-ng/);
        }
        else{
            $cmd = "storage download acp -F $opts{file_name}";
            $cmd = "run local ". $cmd if ($host->{hosttype} =~/filer-ng/);
        }

    } else {
            croak "Invalid $opts{cmd_opt} option";
    }

    return _acppfwcommand($host,$cmd,$conntype,$wait,$timeout,\%opts);

}

sub _acppfwcommand {
    my($host, $cmd, $conntype, $wait ,$timeout,$opts)=@_;
    my ($output, $exp_names);

    my $err = 0;

    my $conn = make_conn($host,$conntype); # will default to rsh connection

    # get all expander names
    #if ($cmd ~= "-F" && !defined($opts{expander_name}) ) {

    my $msg1 = "it is not active";
        my $msg2 = "No ACP module is eligible to download firmware";
        my $msg3 = "not eligible for update";
        my $msg4 = "usage:";
        my $msg5 = "Error:";
        my $msg6 = 'Are you sure you want to continue with ACP processor firmware update\?';
        my $msg7 = "file does not exist";
    my $msg8 = "ACP module\s+(\S+)\s+is not eligible for update\.";
    my $pmt = '\n*.*>\s';

                 $output= $conn->execute(command => "$cmd",
                                timeout => 360,
                                match_table => [(".*$msg6.*") => "y",
                                ],
                                );


    if ($output =~ /$msg1|$msg2|$msg3|$msg4|$msg7|$msg8/) {
        logcomment("Unable to download ACP firmware. Message: $output.");
        return ++$err;
    }
    elsif ($output =~ /Error:/) {
        logcomment("Hit error: $output.");
        return ++$err;
    }
    else {
    }

    $err += monitor_storage_download_acp($host, $opts->{expander_name}, $opts->{monitor_time} );

    return $err;

}


sub monitor_storage_download_acp {

    my($host,$exp_name, $monitor_time)=@_;

    my $i = 0;
    my $done = 0;
    my $err = 0;
    my ( $acp, @allacpp, @activeacpp);
    my ($mark, $event_by_type, $event);

    $mark = ems_log_get_mark(host=>$host);
    print "Now the mark1 is $mark.\n";

    while ($i < $monitor_time && ! $done ) {

        my $events = ems_log_dump(host=>$host, timestamp=>$mark);

        if(defined $exp_name) {

            foreach my $event ( @{$events->{'by_type'}->{'acp.update.failed'}}) {

                my $event_inband_id = lc $events->{'by_id'}->{$event}->{'inband_id'};

                if($event_inband_id eq $exp_name)   {

                    logcomment("ACP firmware download failed for $exp_name");
                    $err++;
                }
            }
        }
        else {
            if(defined $events->{'by_type'}->{'acp.update.failed'}) {
                logcomment("ACP firmware download failed");
                $err++;
            }
        }

        $acp = storage_show_acp(host=>$host);
                @allacpp = get_acp_ids(hash=>$acp);
                @activeacpp = get_active_acp_ids(hash=>$acp);

        if ($#allacpp eq $#activeacpp ) {
            $done = 1;
        }

        $i++;
        if ( $i eq $monitor_time && ! $done ) {
            logcomment("ACP firmware download did not complete in given time.");
            return $err;
        }

        sleep 60;
    }

    return $err;
}

=head2 storage_power_cycle_shelf

    my $result = storage_power_cycle_shelf(
        host=>$host,
        adapter=>$hba_port,
        shelf=>$shelf_number,

    );

    Description of function and behavior goes here

=over

=item host

(Required) : Filer host object to use.

=item adapter

(Required) : Channel to run power_cycle on.

=item shelf

(Optional) : Shelf on adapter to power_cycle.

=item command

(Optional) : One of "help", "start" or "completed". Default: start.

=item force

(Optional) : Either 1 or 0, if 1 appends -f flag to command.  Default: 1.

=item filermode

(Optional) : The mode of filer: "normal" or "maint". Defualt is "normal".
If filer in maintenance mode, the connection type will be "#console".

=item conntype

(Optional) : The connection type to be created by this function to talk to
the filer. Defualt is "#rsh".

=item timeout

(optional) : timeout value. Default is 300 secs.

=item assert1

(Optional) : Either y or n, This is used to assert first prompt during manual power_cycle.  Default: y.

=item assert2

(Optional) : Either y or n, This is used to assert second prompt during manual power_cycle.  Default: y.

=back
=cut

sub storage_power_cycle_shelf {

        my (%opts) = validate(
        @_,
        {
            host        => { type=>OBJECT, isa=>'Hostrec'                 },
            adapter     => { type=>SCALAR, default=>undef,    optional=>1 },
            shelf       => { type=>SCALAR, default=>undef,    optional=>1 },
            force       => { type=>SCALAR, default=>1,        optional=>1 },
            command     => { type=>SCALAR, default=>'start',  optional=>1 },
            assert1     => { type=>SCALAR, default=>'y',      optional=>1 },
            assert2     => { type=>SCALAR, default=>'y',      optional=>1 },
            filermode   => { type=>SCALAR, default=>"normal", optional=>1 },
            conntype    => { type=>SCALAR, default=>"#rsh-socket",   optional=>1 },
            timeout     => { type=>SCALAR, default=>300,      optional=>1 },
        }
    );

    my $cmd = "storage power_cycle shelf ";

    my $ems_start = ems_log_get_mark( host=> $opts{'host'} );

    my $prompt = '';

    if (($opts{assert1} ne "y") && ($opts{assert1} ne "n") && ($opts{assert2} ne "y") && ($opts{assert2} ne "n")) {
    logcomment("provide proper assertion value. either yes|y or no|n");
    return 0;
    }

    if ($opts{'command'} eq 'completed') {
        # completed has no further arguments
        $cmd .= "completed";
    } elsif ($opts{'command'} eq 'help') {
        $cmd .= "-h";
    } else {
        # start has other options and arguments
        $cmd .= "start ";

        if ($opts{force}) {
            # add force flag
            $cmd .= "-f ";
        } else {
            $opts{conntype} = "#console";
            $prompt = 'Are you sure you want to continue with shelf power-cycle? (yes/no):'
        }

        # add channel flag with adapter name
        $cmd .= "-c $opts{'adapter'} ";

        if (defined($opts{shelf})) {
            # add shelf flag (last option)
            $cmd .= "-s $opts{shelf}";
        }
    }
    # $cmd is now complete and ready to run
    # make connection

    if ($opts{filermode} ne "normal") {
        $opts{conntype} = "#console";
    }
    else {
        $cmd = "priv set -q advanced;$cmd";
    }
    $opts{conntype} = "#console";
    my $conn = make_conn($opts{host},$opts{conntype}); # will default to rsh connection
    if($opts{host}->{hosttype} =~ /filer-ng/){
        $cmd = "run local  \"$cmd\"";
              }
             else{
              $cmd = "$cmd";
              }

    my $output = $conn->execute(command => "$cmd",
                           match_table => [(".*Are you sure you want to continue with shelf power.*") => "y",
                           (".*Would you like to prepare the system for a manual power-cycle of shelf.*") => "y",
                           ],
                           timeout => 300,
                           );

    # At this point we have run the command and collected output.  The output
    # will contain information on the result.

    if (( $output =~ /Support for automatic power-cycling shelf/)
    && ($opts{assert2} =~ /n/) && ($opts{force} eq "0")) {
        croak ("Shelves on adapter: do not support power_cycle");
    } elsif (( $output =~ /Support for automatic power-cycling shelf/)
    && ($opts{force} eq "1")) {
        croak ("Shelves on adapter: do not support power_cycle. But System is ready for the manual power-cycle of shelf");
    } elsif ($output =~ /Support for automatic power-cycling is not available for the following shelves on channel/) {
    croak ("Support for automatic power-cycling is not available for the following shelves on channel");
    } elsif ($output =~ /System is ready for the manual power-cycle of shelf/) {
        # return success manual

        return "OK: Ready for manual power cycle";
    } elsif ($output =~ /Manual shelf power-cycle process complete./) {
        # return success completed

        return "OK: Manual shelf power-cycle process completed";

    } elsif ($output =~ /power_cycle shelf start [-f] -c <channel_name> [-s <shelf_id>]/) {
        return "OK: storage power_cycle shelf -h command was successful.";
    } elsif ( $output =~ /-h    display this help and exit/) {
        return "OK: storage power_cycle shelf -h command was successful";

    } elsif ( $output =~ /Automatic power-cycling has been initiated on shelf/
         || $output =~ /The following shelves have been initiated with the automatic power-cycle on channel/
         || $output =~ /The following shelves have been initiated with the auto/ && $opts{force} eq 0)  {
         # MTK added third condition to handle manual power_cycle success for all the shelves.

        # power cycle has started.  Check ems until done.
        sleep 180; # sleep 3 minutes to give time for power cycle to complete.

        # power_cycle has started.
        # monitor ems for completion messages
        # <LR d="03Sep2009 21:22:28" n="stor-u108" pn="stor-u109" t="1252012948" id="1248469822/34006" p="6" s="C=1U" o="ses_admin" vf="">
        # <ses_shelf_reset_request_good_1
        #   shelfType="DS14-Mk2-AT"
        #   channelName="0a"
        #   shelfId="4"
        #   serialNumber="OPS652002196DCA"/>
        # </LR>

    my $ems_found=0;
    my $max_timout=time + 1200; #20 mins from now
        my $sleep_time=5;
       for( ; time < $max_timout ; ) {
        my $check_state = ems_log_check(
            host        => $opts{'host'},
            eventtype   => "ses.shelf.reset.request.good",
            key         => "shelfId",
            value       => $opts{'shelf'},
            timestamp   => $ems_start,
        );

        if ($check_state > 0) {
            $ems_found=1;
            last;
        } else {
            logcomment("Couldn't find the ems for completion of automatic storage power_cycle shelf of"." $opts{'adapter'}".".$opts{shelf}"." Let's check after $sleep_time sec.");
            sleep $sleep_time;
        }
    } # end of for
    if($ems_found) {
         return "OK: automatic storage power_cycle shelf successfully completed on". " $opts{'adapter'}".".$opts{shelf}" ;
    }else {
         croak ("automatic storage power_cycle shelf failed to complete on". " $opts{'adapter'}".".$opts{shelf}");
    }

    } elsif ( $output =~ /power_cycle shelf completed/ ) {
         return "OK: automatic storage power_cycle shelf successfully completed";
    }
    #We may want to add more checks here before failing if the filer output changes with the diffrent version-Faslu
    elsif ( $output =~ /Support for automatic power-cycling is not available for the/ ) {
        croak ( "Shelves on adapter: $opts{adapter} do not support power_cycle" );
    } elsif ( $output =~ /Could not power-cycle the shelf/ || $output =~ /Could not power-cycle the shelves/ ) {
        croak ( "Power-Cycle failed: $output" );
    } elsif ( $output =~ /Manual shelf power-cycle process complete/ ) {
        return ( "OK: Manual shelf power-cycle process complete" );
    } elsif ( ($opts{'command'} eq 'completed') && ($output !~/\S+/) ) {
        return ( "OK: Manual shelf power-cycle process complete" );
    } else {
        croak ( "Unexpected output from storage power_cycle command: $output" );
    }




    # This prompt appears when -s is used on shelves that do not support NDR:
    #
    # Would you like to prepare the system for a manual power-cycle of shelf 6a.1? (yes/no):
    # Support for automatic power-cycling is not available for the
    # following shelves on channel 6a
    #
    # shelf_id: 1
    # shelf_id: 2
    #
    # One or more shelves in channel 6a do not have the support for automatic power-cycling.
    # Use "power_cycle shelf start -c 6a -s <shelf_id>", for each of the shelves that do not
    # have the automatic power-cycling support and follow the instructions.
    #
    # $prompt = "Are you sure you want to continue with shelf power-cycle? (yes/no):"
    #
    # This output occurs with power cycle completed: Manual shelf power-cycle process complete.
    #
    # Manual failure during shelf fw update:
    # Error preparing the system for manual reset of shelf 0a.1.[Error=14. Reason=Shelf f/w update in progress]
    #
    #

}


=head1 SEE ALSO

    Where additional information can be found.

=head1 BUGS

    Description of bugs or what the library does not do and why

=head1 STATUS

    Shared

=head1 ATTRIBUTES

    ONTAP_COMMANDS = volume, df, useradmin

    ONTAP_ZAPI_APIS = storage-adapter-get-adapter-list, storage-adapter-get-adapter-info

=head1 AUTHOR

    Vivienne (vivienne@netapp.com)
    $Id: //depot/prod/test/Rsierranevada/lib/Storage/Storage.pm#7 $
    Last revised $Date: 2013/12/07 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

1;


