# this is basically -*-perl-*-
# $Id: //depot/prod/test/Rsierranevada/lib/Storage/Trace.pm#1 $
#
## @summary	Trace.pm - Storage Trace Library
## @author	dl-storsub-qa@netapp.com
## @status	private
## @notes	Created by EP Network Storage Performance Lab, Inc.
## @pod		here

=head1 NAME

Trace - Storage Trace Library

=head1 SYNOPSIS

  use Storage::Trace;

  # Get the storage trace info
  $info_h = storage_trace_info(conn => $conn);

  # Get the storage trace info for all the adapters
  $info_h = storage_trace_info(conn => $conn, adapter => 'all');

  # Get the storage trace info for adapter '0b'
  $info_h = storage_trace_info(conn => $conn, adapter => '0b');

  # Get the storage trace info for all the devices
  $info_h = storage_trace_info(conn => $conn, device => 'all');

  # Get the storage trace info for device '0b.20'
  $info_h = storage_trace_info(conn => $conn, device => '0b.20');

  # Get the storage trace info for all the shelves
  $info_h = storage_trace_info(conn => $conn, shelf => 'all');

  # Get the storage trace info for shelf '0b.shelf1'
  $info_h = storage_trace_info(conn => $conn, shelf => '0b.shelf1');

  # Get the storage trace replay info for all the adapters
  $list = storage_trace_replay(conn => $conn, adapter => 'all');

  # Get the storage trace replay info for adapter '0b'
  $list = storage_trace_replay(conn => $conn, adapter => '0b');

  # Get the storage trace replay info for all the devices
  $list = storage_trace_replay(conn => $conn, device => 'all');

  # Get the storage trace replay info for device '0b.20'
  $list = storage_trace_replay(conn => $conn, device => '0b.20');

  # Get the storage trace replay info for all the shelves
  $list = storage_trace_replay(conn => $conn, shelf => 'all');

  # Get the storage trace replay info for shelf '0b.shelf1'
  $list = storage_trace_replay(conn => $conn, shelf => '0b.shelf1');

  # Get the storage trace filter info for adapter '0b'
  $list = storage_trace_filter(conn => $conn, adapter => '0b');

  # Get the storage trace filter info for all the devices
  $list = storage_trace_filter(conn => $conn, device => 'all');

  # Get the storage trace filter info for device '0b.20'
  $list = storage_trace_filter(conn => $conn, device => '0b.20');

  # Get the storage trace filter info for all the shelves
  $list = storage_trace_filter(conn => $conn, shelf => 'all');

  # Get the storage trace filter info for shelf '0b.shelf1'
  $list = storage_trace_filter(conn => $conn, shelf => '0b.shelf1');

  # Clear all previous trace buffers
  $return = storage_trace_clear(conn => $conn);

  # Clear all previous trace buffers for all adapters
  $return = storage_trace_clear(conn => $conn, adapter => 'all');

  # Clear all previous trace buffers for adapter '0b'
  $return = storage_trace_clear(conn => $conn, adapter => '0b');

  # Clear all previous trace buffers for all the devices
  $return = storage_trace_clear(conn => $conn, device => 'all');

  # Clear all previous trace buffers for device '0b.20'
  $return = storage_trace_clear(conn => $conn, device => '0b.20');

  # Clear all previous trace buffers for all the shelves
  $return = storage_trace_clear(conn => $conn, shelf => 'all');

  # Clear all previous trace buffers for shelf '0b.shelf1'
  $return = storage_trace_clear(conn => $conn, shelf => '0b.shelf1');

  # Disable all traces
  $return = storage_trace_disable(conn => $conn);

  # Disable all traces for all adapters
  $return = storage_trace_disable(conn => $conn, adapter => 'all');

  # Disable all traces for adapter '0b'
  $return = storage_trace_disable(conn => $conn, adapter => '0b');

  # Disable all traces for all the devices
  $return = storage_trace_disable(conn => $conn, device => 'all');

  # Disable all traces for device '0b.20'
  $return = storage_trace_disable(conn => $conn, device => '0b.20');

  # Disable all traces for all the shelves
  $return = storage_trace_disable(conn => $conn, shelf => 'all');

  # Disable all traces for shelf '0b.shelf1'
  $return = storage_trace_disable(conn => $conn, shelf => '0b.shelf1');

  # Enable all traces
  $return = storage_trace_enable(conn => $conn);

  # Enable all traces for all adapters
  $return = storage_trace_enable(conn => $conn, adapter => 'all');

  # Enable all traces for adapter '0b'
  $return = storage_trace_enable(conn => $conn, adapter => '0b');

  # Enable all traces for all the devices
  $return = storage_trace_enable(conn => $conn, device => 'all');

  # Enable all traces for device '0b.20'
  $return = storage_trace_enable(conn => $conn, device => '0b.20');

  # Enable all traces for all the shelves
  $return = storage_trace_enable(conn => $conn, shelf => 'all');

  # Enable all traces for shelf '0b.shelf1'
  $return = storage_trace_enable(conn => $conn, shelf => '0b.shelf1');

  # Dump all traces into file <file_name>
  $return = storage_trace_dump(conn => $conn, file => <file_name>);

  # Dump all traces for all adapters into file <file_name>
  $return = storage_trace_dump(conn    => $conn,
                               adapter => 'all',
                               file    => <file_name>);

  # Dump all traces for adapter '0b' into file <file_name>
  $return = storage_trace_dump(conn    => $conn,
                               adapter => '0b',
                               file    => <file_name>);

  # Dump all traces for all the devices into file <file_name>
  $return = storage_trace_dump(conn   => $conn,
                               device => 'all',
                               file   => <file_name>);

  # Dump all traces for device '0b.20' into file <file_name>
  $return = storage_trace_dump(conn   => $conn,
                               device => '0b.20',
                               file   => <file_name>);

  # Dump all traces for all the shelves into file <file_name>
  $return = storage_trace_dump(conn  => $conn,
                               shelf => 'all',
                               file  => <file_name>);

  # Dump all traces for shelf '0b.shelf1' into file <file_name>
  $return = storage_trace_dump(conn  => $conn,
                               shelf => '0b.shelf1',
                               file  => <file_name>);

  # Extract trace replays from file $file
  $traces = storage_trace_replay_trace_file(conn =>$conn, file => $file);

  # Disable all FC adapter trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   adapter => 'FC',
                                   action  => 'off',
                                   list    => <trace_id_list>);

  # Disable all SAS adapter trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   adapter => 'SAS',
                                   action  => 'off',
                                   list    => <trace_id_list>);

  # Disable all SCSI adapter trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   adapter => 'SCSI',
                                   action  => 'off',
                                   list    => <trace_id_list>);

  # Disable all device trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   device => 'all',
                                   action  => 'off',
                                   list    => <trace_id_list>);

  # Disable all disk_shelf trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   disk_shelf => 'all',
                                   action  => 'off',
                                   list    => <trace_id_list>);

  # Enable all FC adapter trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   adapter => 'FC',
                                   action  => 'on',
                                   list    => <trace_id_list>);

  # Enable all SAS adapter trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   adapter => 'SAS',
                                   action  => 'on',
                                   list    => <trace_id_list>);

  # Enable all SCSI adapter trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   adapter => 'SCSI',
                                   action  => 'on',
                                   list    => <trace_id_list>);

  # Enable all device trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   device => 'all',
                                   action  => 'on',
                                   list    => <trace_id_list>);

  # Enable all disk_shelf trace_ids in list <trace_id_list>
  $return = storage_trace_trace_id(conn => $conn,
                                   disk_shelf => 'all',
                                   action  => 'on',
                                   list    => <trace_id_list>);

  # Set trace filter type less than trace level 3 for all FC adapters
  $return = storage_trace_set_trace_filter(conn => $conn,
                                           adapter => 'FC',
                                           set => 'trace_filter',
                                           filter_type => 'less_than',
                                           level => 3);

  # Set trace filter type greater than trace level 3 for all devices
  $return = storage_trace_set_trace_filter(conn => $conn,
                                           device => 'all',
                                           set => 'trace_filter',
                                           filter_type => 'greater_than',
                                           level => 3);

  # Set trace category flags to 0x1111 for all shelves
  $return = storage_trace_set_trace_filter(conn => $conn,
                                           shelf => 'all',
                                           set => 'cat',
                                           cat_flags => '0x1111');

  # Set trace level 3 on  all FC adapter trace_ids in list <trace_id_list>
  $return = storage_trace_set_trace_level(conn => $conn,
                                          adapter => 'FC',
                                          level  => 3,
                                          list    => <trace_id_list>);

  # Set trace level 3 on  all device trace_ids in list <trace_id_list>
  $return = storage_trace_set_trace_level(conn => $conn,
                                          device => 'all',
                                          level  => 3,
                                          list    => <trace_id_list>);

  # Set trace level 3 on  all disk_shelf trace_ids in list <trace_id_list>
  $return = storage_trace_set_trace_level(conn => $conn,
                                          shelf => 'all',
                                          level  => 3,
                                          list    => <trace_id_list>);

  # Set trace level buffer size 1000 for adapter 5a
  $return = storage_trace_set_buffer_size(conn    => $conn,
                                          adapter => '5a',
                                          size    => 1000);

  # Set trace level buffer size 500 for device 0b.20
  $return = storage_trace_set_buffer_size(conn   => $conn,
                                          device => '0b.20',
                                          size   => 500);

  # Set trace level buffer size 674 for disk_shelf 0a.shelf4
  $return = storage_trace_set_buffer_size(conn  => $conn,
                                          shelf => '0a.shelf4',
                                          size  => 674);

=head1 DESCRIPTION

Library of storage trace libraries.

=head1 STATUS

  Private

=head1 AUTHOR

  dl-storsub-qa@netapp.com
  $Id: //depot/prod/test/Rsierranevada/lib/Storage/Trace.pm#1 $
  Last revised $Date: 2013/01/22 $

head1 COPYRIGHT

  Copyright 2009 Network Appliance Inc., All Rights Reserved.

=head1 ROUTINES

=cut

package Storage::Trace;

use strict;
use warnings;

use Storage::Misc qw(filer_cmd);

use Carp qw(croak);
use SayWrap qw(sayEOF);
use Params::Validate qw(validate OBJECT SCALAR ARRAYREF);
use base qw(Exporter);

our @EXPORT_OK = qw(
    storage_trace_info
    storage_trace_replay
    storage_trace_filter
    storage_trace_clear
    storage_trace_disable
    storage_trace_enable
    storage_trace_dump
    storage_trace_replay_trace_file
    storage_trace_trace_id
    storage_trace_set_trace_filter
    storage_trace_set_trace_level
    storage_trace_set_buffer_size
);

#-------------------------------------------------------------------------------
# Version for package
#-------------------------------------------------------------------------------
our $VERSION = '1.0';

BEGIN {
    # Die with a script error when an
    # input parameter fails validation
    Params::Validate::validation_options(
        on_fail => sub { my @msgs = @_;
                         chomp(@msgs);
                         Carp::confess(Tharn::script_result(@msgs)) }
    );
};

#-------------------------------------------------------------------------------
# Function prototypes
#-------------------------------------------------------------------------------
sub storage_trace_info(%;);
sub storage_trace_replay(%;);
sub storage_trace_filter(%;);
sub storage_trace_clear(%;);
sub storage_trace_disable(%;);
sub storage_trace_enable(%;);
sub storage_trace_dump(%;);
sub storage_trace_replay_trace_file(%;);
sub storage_trace_trace_id(%;);
sub storage_trace_set_trace_filter(%;);
sub storage_trace_set_trace_level(%;);
sub storage_trace_set_buffer_size(%;);
sub _parse_info($;);
sub _parse_replay($;);
sub _parse_replay_info($;);

#-------------------------------------------------------------------------------

=head2 storage_trace_info

    $info_h = storage_trace_info(conn => $conn
                                 [, adapter => $adapter|'all' ]
                                 [, device => $device|'all' ]
                                 [, shelf => $shelf|'all' ]);

Display the storage trace info for the given adapter, device or shelf.
If no options are given the info for all the adapters, devices and shelves
is retrieved. These options are mutually exclusive.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name or 'all'.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=back

=item Returns

Reference to a hash.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_info(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
        }
    );
    my (%ret, @list, $info, $trace, $type, $out, $i);
    my $cmd = 'priv set -q diag; storage trace info';
    my $nargs = 0;

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if (!$nargs) {
        # No specific type is given so display all
        $cmd .= " all";
    } elsif ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}" if $opts{$type} ne 'all';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);
    # Remove any \r from the output
    $out =~ s/\r+//g;

    # Split output into sections
    @list = split /\n\n/, $out;

    # Output should contain sections in pairs
    croak $out if @list % 2;

    # Process output sections
    for ($i=0; $i<@list; $i+=2) {
        # Get section type (adapter, device or shelf)
        $list[$i] =~ /^(\S+)/;
        $type = lc($1);

        # Parse the info and trace sections for given type
        $info  = _parse_info($list[$i]);
        $trace = _parse_info($list[$i+1]);

        # Save type info into hash
        $ret{$type} = {
            info  => $info,
            trace => $trace,
        };
    }

    if ($nargs == 1) {
        return $ret{$type};
    } else {
        return \%ret;
    }
}

#-------------------------------------------------------------------------------

=head2 storage_trace_replay

    $list = storage_trace_replay(conn => $conn
                                 [, adapter => $adapter|'all' ]
                                 [, device => $device|'all' ]
                                 [, shelf => $shelf|'all' ]
                                 [, count => $count ]);

Display the storage trace replay info for the given adapter, device or shelf.
These options are mutually exclusive.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name or 'all'.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=item count

(Optional) : Limit the replay count to this number.

=back

=item Returns

Reference to an array.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_replay(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
            count   => { type => SCALAR, optional => 1 },
        }
    );
    my (@ret, @traces, $trace, $replay, $type, $out);
    my $cmd = 'priv set -q diag; storage trace replay';
    my $nargs = 0;

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}" if $opts{$type} ne 'all';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } elsif ($nargs == 0) {
        croak "At least on the following options must be present: " .
              "'adapter', 'device' or 'shelf'";
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    # Add the count parameter to the command
    $cmd .= " -n $opts{count}" if defined $opts{count};

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);

    # Remove any \r from the output
    $out =~ s/\r+//g;

    # Fix spelling errors
    $out =~ s/rertry/retry/g;

    # Split output into sections
    @traces = split /\n=+\s*E[NM]D OF TRACE\s*=+\n?/, $out;

    # Process each section
    foreach $trace (@traces) {
        $replay = _parse_replay($trace);
        push @ret, $replay if defined $replay;
    }

    # Check for errors
    croak $out if $out && !@ret && !@traces;

    return \@ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_filter

    $list = storage_trace_filter(conn => $conn
                                 [, adapter => $adapter|'all' ]
                                 [, device => $device|'all' ]
                                 [, shelf => $shelf|'all' ]);

Display the storage trace filter info for the given adapter, device or shelf.
These options are mutually exclusive.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=back

=item Returns

Reference to an array.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_filter(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
        }
    );
    my ($ret, $type, $module_id, $entry, $out);
    my $cmd = 'priv set -q diag; storage trace display_trace_filter';
    my $nargs = 0;

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}"
                  if $opts{$type} ne 'all' || $type eq 'adapter';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } elsif ($nargs == 0) {
        croak "At least on the following options must be present: " .
              "'adapter', 'device' or 'shelf'";
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);

    # Remove any \r from the output
    $out =~ s/\r+//g;

    # Get module id
    $out =~ s/^Trace filter settings for module ID:\s+(\S+)\s*\n//;
    $module_id = $1;

    # Check for errors
    croak $out if $out !~ /^Name /;

    # Parse the info
    $ret = _parse_info($out);

    if (defined $module_id) {
        foreach $entry (@$ret) {
            $entry->{module_id} = $module_id;
        }
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_clear

    $return = storage_trace_clear(conn => $conn
                                 [, adapter => $adapter|'all' ]
                                 [, device => $device|'all' ]
                                 [, shelf => $shelf|'all' ]);

Clears all previous trace buffers for the given adapter, device or shelf.
These options are mutually exclusive.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=back

=item Returns

True if successful.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_clear(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
        }
    );
    my ($type, $out);
    my $cmd = 'priv set -q diag; storage trace clear';
    my $nargs = 0;
    my $ret = 1;

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if (!$nargs) {
        # No specific type is given so clear all
        $cmd .= " all";
    } elsif ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}" if $opts{$type} ne 'all';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);

    if (defined($out) && ($out =~ /Usage: / || $out =~ /Invalid / ||
        $out =~ /Cannot find shelf name /)) {
        $ret = 0;
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_disable

    $return = storage_trace_disable(conn => $conn
                                 [, adapter => $adapter|'all' ]
                                 [, device => $device|'all' ]
                                 [, shelf => $shelf|'all' ]);

Disables traces for the given adapter, device or shelf.
These options are mutually exclusive.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=back

=item Returns

True if successful.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_disable(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
        }
    );
    my ($type, $out);
    my $cmd = 'priv set -q diag; storage trace disable';
    my $nargs = 0;
    my $ret = 1;

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if (!$nargs) {
        # No specific type is given so disable all
        $cmd .= " all";
    } elsif ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}" if $opts{$type} ne 'all';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);

    if (defined($out) && ($out =~ /Usage: / || $out =~ /Invalid / ||
        $out =~ /Cannot find shelf name /)) {
        $ret = 0;
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_enable

    $return = storage_trace_enable(conn => $conn
                                 [, adapter => $adapter|'all' ]
                                 [, device => $device|'all' ]
                                 [, shelf => $shelf|'all' ]);

Enables traces for the given adapter, device or shelf.
These options are mutually exclusive.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=back

=item Returns

True if successful.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_enable(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
        }
    );
    my ($type, $out);
    my $cmd = 'priv set -q diag; storage trace enable';
    my $nargs = 0;
    my $ret = 1;

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if (!$nargs) {
        # No specific type is given so enable all
        $cmd .= " all";
    } elsif ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}" if $opts{$type} ne 'all';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);

    if (defined($out) && ($out =~ /Usage: / || $out =~ /Invalid / ||
        $out =~ /Cannot find shelf name /)) {
        $ret = 0;
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_dump

    $return = storage_trace_dump(conn => $conn,
                                 file => $file
                                 [, adapter => $adapter|'all' ]
                                 [, device => $device|'all' ]
                                 [, shelf => $shelf|'all' ]);

Dumps all traces for the given adapter, device or shelf into file $file.
These options are mutually exclusive.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item file

(Required) : File path where to dump traces

=item adapter

(Optional) : Adapter name.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=back

=item Returns

True if successful.
Croaks otherwise.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_dump(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            file    => { type => SCALAR },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
        }
    );
    my ($type, $out);
    my $cmd = 'priv set -q diag; storage trace dump';
    my $nargs = 0;
    my $ret = 1;
    my $file = $opts{'file'};
    if (!defined($file)) {
        croak "No file name specified";
    }

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if (!$nargs) {
        # No specific type is given so dump all
        $cmd .= " all";
    } elsif ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}" if $opts{$type} ne 'all';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    # Send the command
    $cmd .= " -f $file";
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);
    if (defined($out) && $out =~ /Error open trace file/) {
        croak "Invalid file name: $file";
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_replay_trace_file

    $traces = storage_trace_replay_trace_file(conn => $conn, file => $file);

Extracts all traces from file $file.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item file

(Required) : File path where to dump traces

=item Returns

Extracted traces in scalar $traces
Croaks otherwise.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_replay_trace_file(%;) {
    my (%opts) = validate (
        @_,
        {
            conn => { type => OBJECT },
            file => { type => SCALAR },
        }
    );
    my ($traces);
    my $cmd = 'priv set -q diag; storage trace replay_trace_file';
    my $file = $opts{'file'};
    if (!defined($file)) {
        croak "No file name specified";
    } else {
        $cmd .= " $file";
    }

    # Send the command
    #$traces = sayEOF($opts{conn}, $cmd);
    $traces = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);
    if (defined($traces) && $traces =~ /Error opening trace file/) {
        croak "Invalid file name: $file";
    #} elsif (defined($traces) && $traces !~ /Trace version: /) {
    #    croak "Unexpected output found";
    }

    return $traces;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_trace_id

    $return = storage_trace_trace_id(conn   => $conn,
                                     [, adapter => $adapter|'all' ]
                                     [, device => $device|'all' ]
                                     [, shelf => $shelf|'all' ]
                                     action => on|off,
                                     list   => <trace_id_list>);

Enables or disables all trace_ids in <trace_id_list> for the given adapter,
device or shelf based on action.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name or 'FC' for all FC type adapters.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=item action

(Required) : Action to take: 'on' if enable, 'off' if disable.

=item list

(Required) : Array reference to list of trace_ids to enable|disable.

=back

=item Returns

True if successful.
Croaks otherwise.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_trace_id(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
            action  => { type => SCALAR },
            list    => { type => ARRAYREF },
        }
    );
    my ($action, $item, @list, $type, $out);
    my $cmd = 'priv set -q diag; storage trace trace_id';
    my $ret = 1;
    my $nargs = 0;
    $action = $opts{'action'};

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}"
                  if $opts{$type} ne 'all' || $type eq 'adapter';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } elsif ($nargs == 0) {
        croak "At least one the following options must be present: " .
              "'adapter', 'device' or 'shelf'";
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    if ($action ne 'on' && $action ne 'off') {
        # Error if option is something other than 'on' or 'off'
        croak "Option 'action' must be 'on' or 'off'";
    } else {
        $cmd .= " $action";
    }
    @list = @{$opts{'list'}};
    foreach $item (@list) {
        $cmd .= " $item";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);
    if (defined($out) && ($out =~ /Invalid trace ID/ || $out =~ /Trace ID/)) {
        $ret = 0;
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_set_trace_filter

  $return = storage_trace_set_trace_filter(conn => $conn,
                                           [, adapter => $adapter|'all' ]
                                           [, device => $device|'all' ]
                                           [, shelf => $shelf|'all' ]
                                           set  => trace_filter|cat,
                                           [ filter_type => less_than|equal|greater_than, ]
                                           [ level     => $level, ]
                                           [ cat_flags => $cat ]);

Sets trace filter or category flags for given adapter, device or shelf using
the given filter_type and level for trace filter or given category for trace
category

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name or 'FC' for all FC type adapters.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=item set

(Required) : Option to set: 'cat' for cat flag | 'trace_filter' for trace filter

=item filter_type

(Optional) : Filter type to set: less_than|equal|greater_than.
             Required if set = trace_filter.

=item level

(Optional) : Filter trace level to set. Required if set = trace.

=item cat_flags

(Optional) : Category flags to set. Required if set = cat.

=back

=item Returns

True if successful.
Croaks otherwise.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_set_trace_filter(%;){
    my (%opts) = validate (
        @_,
        {
            conn        => { type => OBJECT },
            adapter     => { type => SCALAR, optional => 1 },
            device      => { type => SCALAR, optional => 1 },
            shelf       => { type => SCALAR, optional => 1 },
            set         => { type => SCALAR },
            filter_type => { type => SCALAR, optional => 1 },
            level       => { type => SCALAR, optional => 1 },
            cat_flags   => { type => SCALAR, optional => 1 },
        }
    );

    my ($cat, $fil, $lev, $ret, $type, $set, $out);
    my $cmd = 'priv set -q diag; storage trace set_trace_filter';
    my $nargs = 0;
    $ret = 1;
    $set = $opts{'set'};
    $cat = $opts{'cat_flags'};
    $fil = $opts{'filter_type'};
    $lev = $opts{'level'};

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}"
                  if $opts{$type} ne 'all' || $type eq 'adapter';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } elsif ($nargs == 0) {
        croak "At least one the following options must be present: " .
              "'adapter', 'device' or 'shelf'";
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    if (defined($set) && $set eq 'trace_filter') {
        # Set a trace filter
        $cmd .= " -t";
        if (!defined($fil)) {
            croak "Missing option 'filter_type'";
        } else {
            $cmd .= " $fil";
            if(!defined($lev)) {
                croak "Missing option 'level'";
            } else {
                $cmd .= " $lev";
            }
        }
    } elsif(defined($set) && $set eq 'cat') {
        # Set category flags
        $cmd .= " -c";
        if (!defined($cat)) {
            croak "Missing option 'cat_flags'";
        } else {
            $cmd .= " $cat";
        }
    } else {
        croak "At least one the following options must be present: " .
              "'trace_filter' or 'cat'";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);
    if ($out !~ /Trace filter settings for module ID:/) {
        $ret = 0;
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_set_trace_level

    $return = storage_trace_set_trace_level(conn   => $conn,
                                            [, adapter => $adapter|'all' ]
                                            [, device => $device|'all' ]
                                            [, shelf => $shelf|'all' ]
                                            level => $level,
                                            list   => <trace_id_list>);

Set trace level $level on all trace_ids in <trace_id_list> for the given
adapter, device or shelf.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name or 'FC' for all FC type adapters.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=item level

(Required) : Level to set.

=item list

(Required) : Array reference to list of trace_ids to set level on.

=back

=item Returns

True if successful.
Croaks otherwise.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_set_trace_level(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
            level   => { type => SCALAR },
            list    => { type => ARRAYREF },
        }
    );
    my ($level, $item, @list, $type, $out);
    my $cmd = 'priv set -q diag; storage trace set_trace_level';
    my $ret = 1;
    my $nargs = 0;
    $level = $opts{'level'};

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}"
                  if $opts{$type} ne 'all' || $type eq 'adapter';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } elsif ($nargs == 0) {
        croak "At least one the following options must be present: " .
              "'adapter', 'device' or 'shelf'";
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    $cmd .= " $level";
    @list = @{$opts{'list'}};
    foreach $item (@list) {
        $cmd .= " $item";
    }

    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    eval{
      $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);
    };

    if ($out !~ /TRACE ID NAME/) {
        $ret = 0;
    }

    return $ret;
}

#-------------------------------------------------------------------------------

=head2 storage_trace_set_buffer_size

    $return = storage_trace_set_buffer_size(conn   => $conn,
                                            [, adapter => $adapter|'all' ]
                                            [, device => $device|'all' ]
                                            [, shelf => $shelf|'all' ]
                                            size => $size);

Set trace level buffer size $size on given adapter, device or shelf.

The following named arguments are accepted:

=over

=item Arguments

=over

=item conn

(Required) : Connectrec object of filer.

=item adapter

(Optional) : Adapter name or 'FC' for all FC type adapters.

=item device

(Optional) : Device name or 'all'.

=item shelf

(Optional) : Shelf name or 'all'.

=item size

(Required) : size to set.

=back

=item Returns

True if successful.
Croaks otherwise.

=back

=for html <hr />

=cut

#-------------------------------------------------------------------------------
sub storage_trace_set_buffer_size(%;) {
    my (%opts) = validate (
        @_,
        {
            conn    => { type => OBJECT },
            adapter => { type => SCALAR, optional => 1 },
            device  => { type => SCALAR, optional => 1 },
            shelf   => { type => SCALAR, optional => 1 },
            size    => { type => SCALAR },
        }
    );
    my ($size, $type, $out);
    my $cmd = 'priv set -q diag; storage trace set_buffer_size';
    my $ret = 1;
    my $nargs = 0;
    $size = $opts{'size'};

    # Check mutually exclusive options
    $opts{adapter} && $nargs++;
    $opts{device}  && $nargs++;
    $opts{shelf}   && $nargs++;

    if ($nargs == 1) {
        # Only one type is given
        foreach $type (qw(adapter device shelf)) {
            if (defined $opts{$type}) {
                $cmd .= " $type";
                $cmd .= " $opts{$type}"
                  if $opts{$type} ne 'all' || $type eq 'adapter';
                last;
            }
        }
        # Rename shelf option to correct filer option
        $cmd =~ s/ shelf/ disk_shelf/;
    } elsif ($nargs == 0) {
        croak "At least one the following options must be present: " .
              "'adapter', 'device' or 'shelf'";
    } else {
        # Error if more than one option is specified
        croak "The options 'adapter', 'device' and 'shelf' " .
              "are mutually exclusive";
    }

    $cmd .= " $size";
    # Send the command
    #$out = sayEOF($opts{conn}, $cmd);
    $out = filer_cmd(host=>$opts{conn}->hostp(), cmd=>$cmd);
    if ($out !~ /$size entries allocated for module instance/) {
        $ret = 0;
    }

    return $ret;
}

#-------------------------------------------------------------------------------
# Internal function
# Parse output given it has the following format:
#   a. Column headers and data are separated by a line of '---'
#   b. The column headers MUST be separated by at least two spaces so a column
#      header could have spaces
#   c. Data in each column MUST not have any spaces and MUST exist
#
# The following output:
#   Name         modinst   trace level   trace cat   filter param
#   -------------------------------------------------------------
#   0b.22              1             0       ffffh   TRACE_LEVEL_FILTER_EQUAL
#   0b.23              2             0       ffffh   TRACE_LEVEL_FILTER_EQUAL
#
# will return the following:
#   [
#       {
#           'name'         => '0b.22',
#           'modinst'      => '1',
#           'trace_level'  => '0'
#           'trace_cat'    => 'ffffh',
#           'filter_param' => 'TRACE_LEVEL_FILTER_EQUAL',
#       },
#       {
#           'name'         => '0b.23',
#           'modinst'      => '2',
#           'trace_level'  => '0'
#           'trace_cat'    => 'ffffh',
#           'filter_param' => 'TRACE_LEVEL_FILTER_EQUAL',
#       },
#   ]
#-------------------------------------------------------------------------------
sub _parse_info($;) {
    my $info = shift;
    my (@ret, $header, $text, $line, @headers);

    # Nothing to do if there is no info
    if (!defined $info || $info =~ /^\s*$/) {
        return [];
    }

    # Split the header from the text
    ($header, $text) = split /\n-+\n/, $info;

    # Get the headers
    @headers = map {s/\s+/_/g; s/\?//g; lc} split /\s\s+/, $header;

    # Check for valid output
    croak $info if !defined $text;

    # Process info
    foreach $line (split /\n/, $text) {
        my %item;
        @item{@headers} = ($line =~ /(\S+)\s*/g);
        push @ret, \%item;
    }

    return \@ret;
}

#-------------------------------------------------------------------------------
# Internal function
# Parse the replay output
#-------------------------------------------------------------------------------
sub _parse_replay($;) {
    my $trace = shift;
    my (%ret, $item);

    # Get the name, id and instance
    if ($trace !~ s/=+\s+Module device name (\S+),\s+module id (\S+),\s+module instance (\S+)\s+=+\n\+*\s*//) {
        return;
    }

    # Save top level info
    %ret = (
        name     => $1,
        id       => $2,
        instance => $3,
    );

    # Process each trace
    foreach $item (split /\++\s+/, $trace) {
        ($item =~ s/^(\d+)\s+(\S+)\s+Trace id ([^\(]+)\((\d+)\),?\s*//);
        push @{$ret{replays}}, {
            id            => $1,
            tstamp        => $2,
            trace_id_name => $3,
            trace_id      => $4,
            raw_info      => $item,
            info          => _parse_replay_info($item),
        }
    }

    return \%ret;
}

#-------------------------------------------------------------------------------
# Internal function
# Parse output for a single trace given it has the following format:
#   a. key/value pairs are separated by either a ',' or a new line
#   b. key/value separators could either be a ':', '=', '()' or a space
#
# Output is a hash having all the key/value pairs found in the input string
#-------------------------------------------------------------------------------
sub _parse_replay_info($;) {
    my $info = shift;
    my (%ret, $key, $value);

    # Process each key/value pair
    foreach (split /[,\n]\s*/, $info) {
        # Replay info have four (4) different formats:
        #  a. key and value separated by a ':'
        #     location: ../driver/scsi/scsi.c:4855
        #     led_flash: on = 0x0  off = 0x800  status = 0x0
        #     Current Map(port count = 15):  198 197 195 188 186 185
        #  b. key and value separated by an '='
        #     old primary = 0x0
        #  c. key and value separated by '()'
        #     state(0x1)
        #  d. key and value separated by a space
        #     subentries 3
        #     byte count 62
        if (/([^:]+):\s*(.+)/ || /(.+)\s*=\s*(.+)/
         || /(\S+)\(([^\)]+)\)/ || /(.+)\s+(\S+)/) {
            # Change key to lower case
            $key = lc($1);
            $value = $2;

            # Remove unnecessary characters for the key
            $key =~ s/[:#]//g;

            # Convert all spaces to '-' on the key
            $key =~ s/\s+/_/g;

            if ($key =~ /current_map\(.*\)/) {
                # Change value of this key to an array
                # Current Map(port count = 15):  198 197 195 188 186 185
                # current_map => [ 198, 197, 195, 188, 186, 185 ]
                $key = 'current_map';
                $value = [ split ' ', $value ];
            }
            if ($key =~ /^led_(flash|error)$/) {
                # Change value of this key to a hash
                # led_flash: on = 0x0  off = 0x800  status = 0x0
                # led_flash => { on => 0x0, off => 0x800, status => 0x0 }
                $value = { $value =~ /(\S+)\s+=\s+(\S+)/g };
            }

            # Save key/value on the return hash
            $ret{$key} = $value;
        }
    }

    return \%ret;
}

1;

