# -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Storage/persistent_reservation.pm#2 $
## @summary libary provides test routines for Persistent Reservation
## @author tomdoan@netapp.com, dl-step@netapp.com, dl-sar-qa@netapp.com
## @description
##
## The library provides routines to test and verify disk persistent
## reservation characteristics.
##
## For details information regarding disk persistent reservation,
## refer to the on line functional spec:
## http://web.netapp.com/~tomdoan/storage/Persistent_Reservations/
## @status public
##
=head1 NAME

    persistent_reservation - library provides routines to verify persistent reservation of disk drives.

=cut

=head1 SYNOPSIS

    use Storage::persistent_reservation;

=cut

=head1 DESCRIPTION

    The library provides routines to verify and test disk drive persistent reservation.

=head1 ROUTINES

=cut

use strict; 
use warnings; 

use Exporter;
use boot; 
use cluster; 
use Sys::Hostname; 
use RaidInfo;
use Storage::Misc qw(filer_cmd);
use Volume qw(vol_create vol_destroy vol_exists vol_add vol_offline); 
#use Step::sutils(); 

our (@EXPORT, @ISA); 
#use vars ('@ISA', '@EXPORT'); 
@ISA = ('Exporter'); 

# -----------------------
# define extern functions
# -----------------------
@EXPORT = qw(pr_disk_resv_compare
	     pr_disk_get_resv_info
	     pr_display_hash
	     pr_get_resv_types
	     pr_get_selected_disks
	     pr_init
	     pr_load_create
	     pr_load_destroy
	     pr_read_keys
	     pr_register_keys
	     pr_validate_parameters
	     pr_verify_generation
	     pr_verify_resv
	     pr_verify_takeover
	     ); 

use vars qw($CLIENT
	    $FILER
            $FILERA
	    $FILERB
	    $FILER_VOL
	    $ITERATIONS
	    $NUM_DISK
	    $SHUTDOWN
	    $VERBOSE
	    ); 

# ---------------- 
# define Constants
# ---------------- 
use constant SAY_TIMEOUT => 240;

# ======================
# define local functions
# Since PR is so unpredictable, the third argument 3 (try) is there
# for how many times to execute the command until no more errors
# ======================
sub _pr_say($$$) {
    my ($fc, $cmd, $try) = @_; 
    
    # -----------------------------
    # set the appropriate test mode
    # -----------------------------
    my $say_cmd = "priv set -q test; " . $cmd; 
    #my (undef, undef, $output) =  say($fc, $say_cmd, -ignore, SAY_TIMEOUT);
    my $output = filer_cmd(host=>$fc->hostp(), cmd=>$say_cmd);

    # return immediately if try == 0
    if ($output !~ /error:/ || $try == 0) {
       return $output; 
    }
    # ----------------------------------------
    # attempt to read multiple times due to PR
    # is very unpredictable
    # ----------------------------------------
    my $cnt = 0; 
    while ($cnt++ <= $try && $output =~ /error:/) {
        #(undef, undef, $output) =  say($fc, $say_cmd, -ignore, SAY_TIMEOUT);
        my $output = filer_cmd(host=>$fc->hostp(), cmd=>$say_cmd);
        if ($output !~ /error:/) {
           return $output; 
        }
        _pr_debug("warning: attempt to run scsi again\n");
        sleep(0);
    }
    return $output; 
} #_pr_say

# ---------------------
# mount the root volume 
# ---------------------
sub _mount_root_vol($) {
    my ($filername) = @_;
    
    print "_mount_root_vol: filername=<$filername>\n"; 
    my $fh = host($filername); 
    my $fc = connect("#rsh"); 
#    _pr_say($fc, "license add TZBGCKE", 0);

    # ---------------------------
    # clear the mount point first
    # ---------------------------
    my $ch = host($CLIENT); 
    my $cc = connect("#rsh"); 
    say($cc, "r umount -f /t/$filername/root", -fatal, "unable to umount /t/$filername");

    # -------------------------    
    # let mount the root volume
    # -------------------------
    say($cc, "r sim_mount $filername", -fatal, "unable to mount /t/$filername/root");
} #_mount_root_vol

# ---------------------
# mount the root volume 
# ---------------------
sub _umount_root_vol($) {
    my $filername = $_[0];
    
    # ---------------------------
    # clear the mount point first
    # ---------------------------
    my $ch = host($CLIENT); 
    my $cc = connect("#rsh"); 
    say($cc, "r umount -f /t/$filername/root", -fatal, "unable to umount /t/$filername");

} #_umount_root_vol

# ------------------------------------------
# get current index of the ems messages file
# ------------------------------------------
sub _get_msg_current_index($) {
  my $filername = $_[0];

  _mount_root_vol($filername);
  print("_get_msg_current_index: filername=<$filername>\n"); 
  my $ems_file = "/t/$filername/root/etc/messages"; 

  open(FH, $ems_file) || die "Unable to open the message log <$ems_file>"; 

  # --------------------------------
  # get the current index of the msg log 
  # --------------------------------
  seek(FH, 0, 2);
  my $index = tell(FH); 
  close(FH);
  print "The current index of msg log = <$index>\n"; 
  return $index; 
}

sub _disk_unfail($$) {
  my ($fc, $disk) = @_; 

  my $output = _pr_say($fc, "disk unfail -s $disk", 0); 
  if ($output =~ /disk unfail: unfailing disk $disk/) {
    tstlog(-comment, "Successfully unfail disk <$disk>"); 
    return 1; 
  }

  if ($output =~ /disk unfail: Disk is not currently failed/) {
    tstlog(-warn, "Look like disk <$disk> is not in failed state"); 
    return 0;
  }

  $output = _pr_say($fc, "disk unfail $disk", 0); 
  if ($output =~ /disk unfail: unfailing disk $disk/) {
    tstlog(-comment, "Successfully unfail disk <$disk>"); 
    return 1; 
  }

  tstlog(-fail, "Unable to unfail disk <$disk>"); 
  return 0; 
} #_disk_unfail 

# ---------------------------------------------------
# remove the disk temporarily. Wait until the disk is
# being moved to the broken pool. 
# ---------------------------------------------------
sub _disk_remove($$) {
  my ($fc, $disk) = @_; 

  my $output = _pr_say($fc, "disk remove -f $disk", 0); 
  if ($output =~ /Disk $disk is already in broken pool/) {
      # -----------------------------      
      # verify from the disk show CLI
      # -----------------------------
      $output = _pr_say($fc, "disk show -v $disk", 0); 
      if ($output =~ /FAILED/) {
	  tstlog -comment, "Successfully removed disk <$disk>";
	  return 1; 
      }
      tstlog -fatal, "Unable to remove to fail disk <$disk>"; 
  }
  
  # ------------------------------------------------
  # wait until the disk showed up in the broken pool 
  # ------------------------------------------------
  my $max_cnt = 10; 
  my $cnt = 0; 
  while ($cnt++ <= $max_cnt) {
    $output = _pr_say($fc, "vol status -f", 0); 
      foreach my $line (split(/\n/, $output)) {
	if ($output !~ /admin removed/) {
	  next; 
	}
	
	if ($line =~ /admin removed\t$disk/) {
	  tstlog(-comment, "Disk <$disk> had been moved successfully to the broken pool"); 
	  # -------------------------------------      
	  # one more check from the disk show CLI
	  # -------------------------------------
	  $output = _pr_say($fc, "disk show -v $disk", 0); 
	  if ($output =~ /FAILED/) {
	      tstlog -comment, "Successfully removed disk <$disk>";
	      return 1; 
	  }
	  
	  tstlog -fatal, "Unable to remove to fail disk <$disk>"; 
	}
	sleep(2); 
      }
  }
  tstlog(-fail, "Unable to remove <$disk>"); 
  return 0; 
} #_disk_remove

# ------------------------------------------
# dump new messages from the specified index
# of the ems messages. 
# Dismount both of the root volumes. 
# ------------------------------------------
sub _dump_msg_log($) {
  my ($rHoH) = @_; 


  if (!$VERBOSE) {
    return; 
  }
  my $filera = $rHoH->{FA}->{FNAME}; 
  my $filera_idx = $rHoH->{FA}->{MLOG_INDEX};

  my $filerb = $rHoH->{FB}->{FNAME}; 
  my $filerb_idx = $rHoH->{FB}->{MLOG_INDEX};

  print "FilerA index: <$filera_idx> FilerB index: <$filerb_idx>\n"; 
  # --------------------------------------
  # open and dump the latest message log 
  # --------------------------------------
  my $ems_file = "/t/$filera/root/etc/messages"; 
  open(FH, $ems_file) || die "Unable to open the message log <$ems_file>";
  seek(FH, $filera_idx, 0); 
  tstlog(-comment, "message log from filer <$filera>"); 
  while (<FH>) {
    print $_; 
  }
  close(FH); 

  # -------------------------------------------------
  # open and dump the latest message log from filerB 
  # -------------------------------------------------
  $ems_file = "/t/$filerb/root/etc/messages"; 
  open(FH, $ems_file) || die "Unable to open the message log <$ems_file>";

  seek(FH, $filerb_idx, 0); 
  tstlog(-comment, "message log from filer <$filerb>"); 
  while (<FH>) {
    print $_; 
  }
  
  print "\n\n"; 
  close(FH); 
  _umount_root_vol($filera); 
  _umount_root_vol($filerb); 
} #_dump_msg_log; 

# ---------------------------------------
# define routine to output debug messages
# ---------------------------------------
sub _pr_debug($) {
    my ($msg) = @_; 

    if (! $VERBOSE) {
	return;
    }
    print("DEBUG: " . $msg); 
}
# ----------------------------------------------------------
# the message log wrapper check for the message state and if 
# it is a '-fatal' then it perform cleanup before calling
# the tstlog with an '-fatal'. 
# arguments: 
# $mstate (required) - '-fatal', '-pass', or '-fail'
# $msg - the message content to be output
# $mdisk (optional) - the disk name 
# 
# ---------------------------------------------------------
sub _pr_log($$;$) {
    my ($mstate, $msg, $mdisk) = @_; 
    
    # ---------------------------------------------------------
    # to restore the disk ownership and output disk reservation
    # information regarding the specified disk 
    # ---------------------------------------------------------
    if ($mstate eq "-fatal") {
	#restore disk ownership ***
	
	#display detail reservation information regarding the failed disk
	# _pr_cleanup($mdisk); 
	tstlog($mstate, $msg); 
    }

    # ------------------------------------------------    
    # display the message in either PASS or FAIL state
    # ------------------------------------------------

    if (defined $mdisk ) {
	tstlog($mstate, $msg . "<$mdisk>"); 
    } else {
	tstlog($mstate, $msg); 
    }
} #_pr_log

# -----------------------------------------------------------
# sub _get_prkey - retrieves the key from the specified output 
# from the CLI pr_read_key
# -----------------------------------------------------------
sub _get_prkey($){
    my ($outmsg) = @_;
    my $key;
    
    my (@lines) = split(/\n/, $outmsg);
    foreach my $line (@lines) {
	_pr_debug("line=" . $line . "\n");
	if ($line =~ /key/) {
	    my (@buff) = split(/ +/, $line);
	    $key = $buff[2];
	    _pr_debug("key=" . $key . "\n");
	}
    }
    return $key;
} #_get_prkey


# -----------------------------------------------------------
# sub _pr_preempt_retry - retry PREEMPT again just in case
# the scsi preempt failed for some reason. 
# -----------------------------------------------------------
sub _pr_preempt_retry($$$$$$$){
  my ($rHoH, $fac, $fbc, $fad, $fbd, $fak, $fbk) = @_; 

  _pr_debug("** Running pr_preempt_retry **\n"); 
  my $o = _pr_clear($fac, $fbc, $fad, $fbd); 
  pr_register_keys($rHoH, $fac, $fbc, $fad, $fbd, $fak, $fbk); 
  $o = _pr_clear($fac, $fbc, $fad, $fbd); 
  pr_register_keys($rHoH, $fac, $fbc, $fad, $fbd, $fak, $fbk); 

} #_pr_preempt_retry

# -----------------------------------------------------
# retrieve the reservation key and its reservation type
# from the specified disk 
# -----------------------------------------------------
sub _pr_disk_get_resv_type($$$$) {
    my ($fc, $disk, $rkey, $rt) = @_; 

    # ------------------------------------------------------
    # Try to read a few times so we can get the right result
    # ------------------------------------------------------
    my $output = _pr_say($fc, "scsi pr_read_reservation $disk", 3);

    foreach my $line (split('\n', $output)) {
	if ($line =~ /resv_key/) {
	    my @buf = split(/ +/, $line); 
	    $rkey = $buf[1];
	    _pr_debug("resv_key = $rkey\n"); 
	    next; 
	}

	if ($line =~ /type/) {
	    my @buf = split(/ +/, $line); 
	    $rt = $buf[1]; 
	    _pr_debug("resv_type = $rt\n"); 
	    last; 
	}
    }
} #_pr_disk_get_resv_type

=head2 pr_get_resv_types

The routine looks up the model of the specified disk and comes
up with an array of reserveration types. 

Currently, there are 6 different resv types: 

"we" , "0x1", 
"ea" , "0x3",
"wero" , "0x5", 
"earo" , "0x6", 
"wear" , "0x7", 
"eaar" , "0x8"

Some may not work for all drives. Such as "wear" and "eaar" are
current not supported on Seagate drives. Therefore for Seagate
drives, this routine will return an array with only the first
4 reservation types: "we", "ea", "wero", and "earo". 

=over

=item Arguments

=over

=item rHoH

(Required) : the PR hash table

=back

=item Notes
This function should be updated from time to time to support different
type of disk drives. 

=back 

=item Returns

The routine returns an array of reservation types.  

=back

=cut 

sub pr_get_resv_types($$) {
    my ($rHoH, $disk) = @_; 

    my $disk_model; 
    if (defined $rHoH->{FA}->{DISKS}->{$disk}) {
	$disk_model = $rHoH->{FA}->{DISKS}->{$disk}[3]; 
    } else { 
	$disk_model = $rHoH->{FB}->{DISKS}->{$disk}[3]; 
    }
    
#    my %default_tb =  ("we" , "0x1", "ea" , "0x3","wero" , "0x5", "earo" , "0x6", "wear", "0x7", "eaar", "0x8");    
    my %default_tb =  ("we" , "0x1", "ea" , "0x3","wero" , "0x5", "earo" , "0x6");    

    if ($disk_model =~ /_S/) {
	my %tb = ("we" , "0x1", "ea" , "0x3","wero" , "0x5", "earo" , "0x6"); 
	return %tb;
    } else { 
	tstlog(-fail, "Unable to determine the reservation types for the disk model <$disk_model>. You need to modify the function 'pr_get_resv_types in the library persistent_reservation.pm to provide the appropriate reservation types."); 
	return %default_tb; 
    }
    
} # pr_get_resv_types


=head2 pr_get_selected_disks

The routine returns a list of selected disks of the requested 
filer. The result is an array. 

=over

=item Arguments

=over

=item rHoH

(Required) : the PR hash table

=item filername

(Required) : name of the specified filer

=back 

=item Returns

An array of selected disks

=back

=cut 

sub pr_get_selected_disks(\%$) {
    my ($rHoH, $filername) = @_; 

    if ($rHoH->{FA}->{FNAME} eq $filername) {
	return $rHoH->{FA}->{SEL_DISKS};
    }
    if ($rHoH->{FB}->{FNAME} eq $filername) {
	return $rHoH->{FB}->{SEL_DISKS};
    }
    
    _pr_abort("Unable to retrieve the selected disks due to unknown filername <$filername>"); 
} # pr_get_selected_disks


=head2 pr_display_hash

The routine takes the passing reference pointer of the PR hash table
and displays all elements.

=over

=item Arguments

=over

=item rHoH

(Required) : the PR hash table

=back

=item Returns

None 

=back

=cut 

sub pr_display_hash(\%) {
    my ($rHoH) = @_; 

    # -------------------------------------
    # display only when verbose mode is set
    # -------------------------------------
    if (!$VERBOSE) {
	return; 
    }
    # ---------------------------
    # display the table of models
    # ---------------------------
    print("FA_KEY=" , $rHoH->{FA}->{PRKEY}, "\n"); 
    
    print("FilerA PRE_SELECTED_DISKS= "); 
    for (my $i=0; $i <= $#{$rHoH->{FA}->{SEL_DISKS}}; $i++) {
	print($rHoH->{FA}->{SEL_DISKS}[$i] . "\t"); 
    }
    print("\n"); 

    # ---------------------------				 
    # display the table of filerA
    # ---------------------------
    print("\nFILERA INFO:\n"); 

    print("\nDISK\tSHELF\tBAY\tSerial Number\t\tMODEL\t\tOWNER\t\tPOOL\tGENERATION\tRESV_KEY\tRESV_TYPE\tSTATE\n"); 
    foreach my $key (sort keys (%{$rHoH->{FA}->{DISKS}})) {
	print("$key\t");
	
	for (my $i=0; $i <= $#{$rHoH->{FA}->{DISKS}->{$key}}; $i++) {
	    print($rHoH->{FA}->{DISKS}->{$key}[$i] ."\t");
	}
	print("\n"); 
    } #foreach 

    # ------------------------
    # dump the table of FILERB
    # ------------------------
    print("\n\n\t\tFILERB INFO:\n"); 
    print("FB_KEY=" , $rHoH->{FB}->{PRKEY}, "\n"); 

    # display the selected disks from filerB
    # 
    print("FilerB SELECTED_DISKS= "); 
    for (my $i=0; $i <= $#{$rHoH->{FB}->{SEL_DISKS}}; $i++) {
	print($rHoH->{FB}->{SEL_DISKS}[$i] . "\t"); 
    }
    print("\n"); 

    # -------------------
    # dump FILERB content
    # -------------------
    print("DISK\tSHELF\tBAY\tSerial Number\t\tMODEL\t\tOWNER\t\tPOOL\tGENERATION\tRESV_KEY\tRESV_TYPE\tSTATE\n"); 
    foreach my $key (sort keys (%{$rHoH->{FB}->{DISKS}})) {
	print("$key\t");
	
	for (my $i=0; $i <= $#{$rHoH->{FB}->{DISKS}->{$key}}; $i++) {
	    print($rHoH->{FB}->{DISKS}->{$key}[$i] ."\t");
	}
	print("\n"); 
    } #foreach 
    

    print("\n\n DISK MODELS:\n"); 
    print("\nDISK_MODEL\tDISK\tSHELF\tBAY\tSerial_Num\n"); 
    foreach my $model (sort keys %{$rHoH->{MODELS}}) {
	print("$model \t"); 
	for (my $i=0; $i <= $#{$rHoH->{MODELS}->{$model}}; $i++) {
	    print($rHoH->{MODELS}->{$model}[$i] . "\t"); 
	}
	print("\n"); 
    }

    print("\t-------------------\n"); 

    # ---------------------------
    # display a list of bad disks
    # ---------------------------
    my $fn = $rHoH->{FA}->{FNAME}; 
    print("\n\n BAD DISKS of $fn\n"); 
    print("\nDISK\tPOOL\tSerial_Num\n"); 
    foreach my $disk (sort keys %{$rHoH->{BADDISK}}) {
	for (my $i=0; $i <= $#{$rHoH->{$fn}->{BADDISK}->{$disk}}; $i++) {
	    print($rHoH->{$fn}->{BADDISK}->{$disk}[$i] . "\t"); 
	}
	print("\n"); 
    }
    $fn = $rHoH->{FB}->{FNAME}; 
    print("\n\n BAD DISKS of $fn\n"); 
    print("\nDISK\tPOOL\tSerial_Num\n"); 
    foreach my $disk (sort keys %{$rHoH->{BADDISK}}) {
	for (my $i=0; $i <= $#{$rHoH->{$fn}->{BADDISK}->{$disk}}; $i++) {
	    print($rHoH->{$fn}->{BADDISK}->{$disk}[$i] . "\t"); 
	}
	print("\n"); 
    }

    print("\t-------------------\n"); 
#    exit(0);
} # pr_display_hash


# ---------------------------------------------------------
# define the cleanup routine to be used for cleaning up the 
# test environment before exiting the test script 
# ---------------------------------------------------------
sub _pr_cleanup($$$) {


} #_pr_cleanup

# --------------------------------------------------
# display the selected disks to be tested against PR
# --------------------------------------------------
sub _pr_report($) {
    my ($rHoH) = @_; 
    
    print("\t =========== Selected disks ============\n\n"); 
    print("FILER\tDISK\tSHELF\tBAY\tSerial Number\t\tMODEL\t\tOWNER\t\tPOOL\n");
    print("-----\t----\t-----\t---\t-------------\t\t-----\t\t-----\t\t----\n");
 

    # ------------------------------------
    # display the pre-selected disks from FILERA
    # ------------------------------------
    for (my $i=0; $i<= $#{$rHoH->{FA}->{SEL_DISKS}}; $i++) {
	my $fad = $rHoH->{FA}->{SEL_DISKS}[$i]; 

	# -------------------	
	# write the disk name
	# -------------------
	print($rHoH->{FA}->{FNAME} . $fad . "\t"); 
	for (my $j=0; $j <= $#{$rHoH->{FA}->{DISKS}->{$fad}}; $j++) {
	    print($rHoH->{FA}->{DISKS}->{$fad}[$j] . "\t"); 
	}
	print("\n");
    } #foreach 
    

    # ------------------------------------
    # display a pre-selected disks from FILERB
    # ------------------------------------
    for (my $i=0; $i<= $#{$rHoH->{FB}->{SEL_DISKS}}; $i++) {
	my $fbd = $rHoH->{FB}->{SEL_DISKS}[$i]; 

	# -------------------	
	# write the disk name
	# -------------------
	print($rHoH->{FB}->{FNAME} . $fbd . "\t"); 
	for (my $j=0; $j <= $#{$rHoH->{FB}->{DISKS}->{$fbd}}; $j++) {
	    print($rHoH->{FB}->{DISKS}->{$fbd}[$j] . "\t"); 
	}
	print("\n");
    } #foreach 
    
    print("\t==========================================\n"); 


    if ($VERBOSE) {
      print("\n\n\t\tDumping the message log:\n"); 
      # _dump_msg_log($rHoH); 
    }

    # clean up the test volume if they exist
    pr_load_destroy($rHoH); 
} # _pr_report

# --------------------------------------------------------------------------
#    _pr_clear(fa_conn=>$fac, fb_conn=>$fbc, fa_dev=>$fadev, fb_dev=>$fbdev) 
#The routine clears the reservation of the specified disk. 
#
#Arguments
#
#=over
#
#=item fa_conn
#
#(Required) : FILERA connection object
#
#=item fb_conn
#
#(Required) : FILERB connection object
#
#=item fa_dev
# 
#(Required) : specify a disk from FILERA
#
#=item fb_dev
#
#(Required) : specify the same disk that seen from FILERB
#
#=back
#
#=item Returns
#
#None 

sub _pr_clear($$$$) {
    my ($fac, $fbc, $fadev, $fbdev) = @_;  
    my (undef, $output, $cnt); 
    use constant MAX => 2; 
    
    _pr_debug("\nPR_CLEAR: clear the persistent reservation on both filers\n"); 
    
    # -------------------------
    # clear filera reservation 
    # -------------------------
    $cnt = 0;
    
    while ($cnt++ < MAX) {
	$output = _pr_say($fac," scsi pr_register $fadev", 2);
	$output = _pr_say($fac," scsi pr_clear $fadev", 2);
	if ($output =~ /action succeeded/) {
	    last;
	}
    } #while

    if ($output =~ /action succeeded/) {
	tstlog('-comment', "successfully clear <$fadev>"); 
    } else {
	tstlog('-warn', "unable to clear <$fadev>"); 
    }
    
    # --------------------
    # register with filerb
    # --------------------
    $cnt = 0; 
    while ( $cnt++ < MAX) {
	$output = _pr_say($fbc,"scsi pr_register $fbdev", 2);
	$output = _pr_say($fbc,"scsi pr_clear $fbdev", 2);
	if ($output =~ /action succeeded/) {
	    last;
	}
    } #while 
  
    if ($output =~ /action succeeded/) {
	tstlog('-comment', "successfully clear <$fbdev>"); 
    } else {
	tstlog('-warn', "unable to clear <$fbdev>"); 
    }

    # display reservation information on the disk
    _pr_say($fac, "scsi pr_read_keys $fadev; scsi pr_read_reservation $fadev", 2);
} #_pr_clear

# ------------------------------------------------------------
#    _pr_disk_rm_ownership(hash_info=>$rHoH, fa_conn=>$fac, fb_conn=>$fbc, fa_dev=>$fadev, fb_dev=>$fbdev) 
#
#removes the disk ownership of the specified disk. 
#At the end of the test, the ownership shall be restored back. 
#
#=over
#
#=item Arguments
#
#=over
#
#=item fa_conn
#
#(Required) : FILERA connection object
#
#=item fb_conn
#
#(Required) : FILERB connection object
#
#=item fa_dev
# 
#(Required) : specify a disk from FILERA
#
#=item fb_dev
#
#(Required) : specify the same disk that seen from FILERB
#
#=back
#
#=item return
#None

sub _pr_disk_rm_ownership($$$) {
    my ($rHoH, $frsh, $disk) = @_;  

    # ----------------------------------
    #remove the specified disk ownership
    # ----------------------------------
    my $re_msg = "Volumes must be taken offline. Are all impacted volumes offline\\(y/n\\)\\?\\?"; 
    
    say($frsh, "priv set -q test; disk remove_ownership $disk" , '-blind'); 
    while (1) {
	my $prompt; 
	$frsh->expect(match => [$re_msg], 
		      matching_regex => \$prompt); 
	# EOF
	last if ($@ && $@ =~ /EOF/); 
	
	# Watch out for the time out
	die $@ if $@; 
	if ($prompt eq $re_msg) {
	    $frsh->send("y\r"); 
	}
    } #while 

# -------------------------------------------
# verify that the specified disk has no owner
# -------------------------------------------
    my $output = _pr_say($frsh, "disk show -n $disk", 0); 
    if ($output =~ /disk show: No disks match/) {
	_pr_abort($rHoH, "Unable to remove the ownership of disk <$disk>", $disk); 
	return 0; #failure
    }
    
    foreach my $line (split(/\n/, $output)) {
	if ($line =~ /DISK/ || $line =~ /----------/) {
	    next; 
	}
	my ($disk, $not, $owner, $pool, $serial_num) = split(/ +/, $line); 
	if ($disk ne $disk) {
	    _pr_abort($rHoH, "Unable to remove the ownership of disk <$disk>", $disk); 
	    return 0; #failure
	}
    } # foreach 

    tstlog(-comment, "Successfully remove the ownership of disk <$disk>"); 
    return 1; #success
} #_pr_disk_rm_ownership


#=head2 _pr_disk_verify_resv
#
#    _pr_disk_verify_resv(ITable=>$rHoH, fa_conn=>$fac, fa_dev=>$fadev, resv_key=>$resv_key, resv_type=>$resv_type) 
#
#The routine verifies that the reservation key must match the specified
#key and the disk reservation type must match the specified 
#    
#The routine will exit with a fatal message if the key does not match or the 
#reservation type has not been set corrected. 
#
#=over
#
#=item Arguments
#
#=over
#
#=item fa_conn
#
#(Required) : FILERA connection object
#
#=item fa_dev
# 
#(Required) : specify a disk to be checked for the required access
#
#=item access
#
#(Required) : specify the access types of the specified disks
#
#=back
#
#=item return
#None 

sub _pr_disk_verify_resv($$$$$) {
    my ($rHoH, $frsh, $disk, $resv_key, $resv_type) = @_;

    my $resv;
    
    _pr_debug("\nPR_DISK_VERIFY_RESV: disk verify reservation\n");     
    print "key= " . $resv_key . "\n"; 
    # -----------------------------
    # Identify the reservation type
    # -----------------------------
    if ($resv_type eq "we" ) {
	$resv = "0x1"; 
    } elsif ($resv_type eq "ea") {
	$resv = "0x3";
    } elsif ($resv_type eq "wero"){
	$resv = "0x5"; 
    } elsif ($resv_type eq "earo") {
	$resv = "0x6";
    } elsif ($resv_type eq "wear") {
	$resv = "0x7";
    } elsif ($resv_type eq "eaar") {
	$resv = "0x8";
    } else { 
	_pr_abort($rHoH, "Unable to identify the reservation type of <$resv_type>"); 
	return 0; #failure

    }
    
    my $output = _pr_say($frsh, "scsi pr_read_reservation $disk", 2);

    if ($output !~ /generation/) {
	_pr_abort($rHoH, "Unable to read the disk reservation of <$disk>", $disk); 
	return 0; #failure
    }

    # -------------------------------------------------------
    # parses and looks for the both the resv_key and its type
    # -------------------------------------------------------
    foreach  my $line (split(/\n/, $output)) {
      my ($key, $value) = split(/:/, $line); 
      if ($key =~ /resv_key/) {
	if ($value =~ $resv_key) {
	  tstlog(-pass, "the reservation key <$resv_key> matched"); 
	} else {
	  _pr_abort($rHoH, "the reservation key <$resv_key> does not match with <$value>", $disk); 
	}
	next; 
      }
      
      if ($key =~ /type/) {
	if ($value =~ $resv) {
	  tstlog(-pass, "the reservation type <$resv_type> matched"); 
	} else {
	  _pr_abort($rHoH, "the reservation type <$resv_type> does not match <$value>", $disk); 
	}
	last; 
      }
    } #foreach 
    return 1; #success
} #_pr_disk_verify_resv

#=head2 _pr_disk_verify_rw
#
#    _pr_disk_verify_rw(ITable=>$rHoH, fa_conn=>$fac, fa_dev=>$fadev, res_type=>$res_t, owner_type=>$owner_t) 
#
#The routine verifies that the disk on the filer has the right access based
#on whether or not it is the disk owner (or just the partner) and what type of reservation it has on the specified disk. 
#
#    The reservation could be one of the 6 types: we, ea, wero, earo, wear, and eaar. The access type can be divided into 4 groups based on the res_type of: 
#
#    RESV_TYPE  OWNER       PARTNER      OWNER(PREEMPT)  PARTNER(PREEMPT)
#    ========  =====       =======      ==============  ================
#       we     read/write   read        read/write      read
#       ea     read/write   None        read/write      None
#       wero   read/write   read        read/write      read
#       earo   read/write   read/write  read/write      None
#
#
#   The routine uses the passing argument $op variable to determine what
#type of permission should be verified. The argument $owner_t should be passed
#with the following string: 
#     owner - the specified filer is the disk owner
#     partner - the specified filer is the partner
#     ope - the specified filer is the owner by preempted. 
#     ppe - the specified filer is the partner by preempted. 
#
#=over
#
#=item Arguments
#
#=over
#
#=item fa_conn
#
#(Required) : FILERA connection object
#
#=item fa_dev
# 
#(Required) : specify a disk to be checked for the required access
#
#=item res_t
#
#(Required) : specify the reservation type 
#
#=item owner_t
#
#(Required) : specify the owner type which could be one of the 4 types:
# "owner", "partner", "ope", and "ppe". 
#
#=back
#
#=item return
#
#None 
#
#=back
#
#=cut 

sub _pr_disk_verify_rw($$$$$) {
    my ($rHoH, $frsh, $disk, $res_t, $owner_t) = @_;  

    my $output; 
    my $access; 
    my $MAX_ATTEMPT = 3; 
    my $cnt; 

    # -----------------------------
    # check for write exclusive (we)
    # -----------------------------
    if ($res_t eq "we") {
	if ($owner_t eq "owner") {
	    $access = "rw"; #read/write permission
	} elsif ($owner_t eq "partner") {
	    $access = "rW";  # read permission
	} elsif ($owner_t eq "ope") {
	    $access = "rw"; # read/write permission
	} elsif ($owner_t eq "ppe") {
	    $access = "rW"; # read only
	}
    } elsif ($res_t eq "ea") {
	if ($owner_t eq "owner") {
	    $access = "rw"; #read/write permission
	} elsif ($owner_t eq "partner") {
	    $access = "RW";  # No read/write permission
	} elsif ($owner_t eq "ope") {
	    $access = "rw"; # read/write permission
	} elsif ($owner_t eq "ppe") {
	    $access = "RW"; # No read write
	}
	
    } elsif ($res_t eq "wero") {
	if ($owner_t eq "owner") {
	    $access = "rw"; #read/write permission
	} elsif ($owner_t eq "partner") {
	    $access = "rw";  # No read/write permission
	} elsif ($owner_t eq "ope") {
	    $access = "rw"; # read/write permission
	} elsif ($owner_t eq "ppe") {
	    $access = "rW"; # No read write
	}
    } elsif ($res_t eq "earo") {
	if ($owner_t eq "owner") {
	    $access = "rw"; #read/write permission
	} elsif ($owner_t eq "partner") {
	    $access = "rw";  # No read/write permission
	} elsif ($owner_t eq "ope") {
	    $access = "rw"; # read/write permission
	} elsif ($owner_t eq "ppe") {
	    $access = "RW"; # No read write
	}
    }

    # ----------------------------------
    # verifies that the disk is readable
    # ----------------------------------
    if ($access =~ 'r') {
      $cnt = 0; 
      while ($cnt++ >= $MAX_ATTEMPT) {
	$output = _pr_say($frsh,"scsi dump_lba -l 0 -n 1 -o /etc/dump_lba.log $disk", 2);
	if ($output !~ /reservation conflict/) {
	  tstlog(-pass, "verified the drive <$disk> is readable"); 
	  last; 
	}
	if ($cnt >= $MAX_ATTEMPT) {
	    _pr_abort($rHoH, "disk <$disk> is NOT READABLE", $disk); 
	} 
	tstlog(-comment, "WARNING: disk <$disk> is NOT readable yet"); 
	sleep(3);
      } #while
    } #if access is readable

    # ----------------------------------
    # verifies that the disk is writable
    # ----------------------------------
    if ($access =~ 'w') {
      $cnt = 0; 
      while ($cnt++ >= $MAX_ATTEMPT) {
	$output = _pr_say($frsh,"scsi test $disk", 2); 
	if ($cnt >= $MAX_ATTEMPT && $output =~ /reservation conflict/) {
	  _pr_abort($rHoH, "disk <$disk> is NOT WRITEABLE", $disk); 
	}
	if ($output !~ /reservation conflict/) {
	  tstlog(-pass, "verified the drive <$disk> is writable"); 
	  last; 
	}
	tstlog(-comment, "WARNING: disk <$disk> is NOT writable yet"); 
	sleep(3); 
      } #while 
    } #if access W

    # --------------------------------------
    # verifies that the disk is not readable 
    # --------------------------------------
    if ($access =~ 'R') {
      $cnt = 0; 
      while ($cnt++ >= $MAX_ATTEMPT) {
	$output = _pr_say($frsh,"scsi dump_lba -l 0 -n 1 -o /etc/dump_lba.log $disk", 2); 
	if ($output =~ /reservation conflict/) {
	  tstlog(-pass, "verifies that the drive <$disk> is not readable"); 
	  last; 
	} 
	tstlog(-comment, "WARNING: drive <$disk> supposed NOT be be readable yet"); 
	if ($cnt >= $MAX_ATTEMPT) {
	  _pr_abort($rHoH, "drive <$disk> supposed NOT TO BE READABLE", $disk); 
	}
	sleep(3); 
      } #while 
    } #if 

    # --------------------------------------
    # verifies that the disk is not writable
    # to avoid intermittent problem, let try 
    # 3 times before giving up 
    # --------------------------------------
    if ($access =~ 'W') {
      $cnt = 0; 
      while ($cnt++ >= $MAX_ATTEMPT) {
	$output = _pr_say($frsh,"scsi test $disk", 2); 
	if ($output =~ /reservation conflict/) {
	  tstlog(-pass, "verifies that the disk <$disk> is NOT WRITEABLE"); 
	  last; 
	} 
	tstlog(-comment, "WARNING: drive <$disk> supposed NOT to be writable yet"); 
	if ($cnt >= $MAX_ATTEMPT) {
	  _pr_abort($rHoH, "drive <$disk> supposed NOT TO BE WRITABLE under <$res_t>", $disk); 
	} 
	sleep(3); 
      } #while try a few times 

    } #if 
    return 1; #success
} #_pr_disk_verify_rw

#=head2 _pr_disk_verify_rw_all
#
#    _pr_disk_verify_rw_all(ITable=>$rHoH, fa_conn=>$farsh, fb_conn=>$fbrsh, fa_disk=>$fadev, fb_disk=>$fbdev) 
#
#The routine verifies that both nodes should have full access to the disk
#before reservation taking place. 
#
#=over
#
#=item Arguments
#
#=over
#
#=item fa_conn
#
#(Required) : FILERA connection object via rsh 
#
#=item fb_conn
# 
#(Required) : FILERB connection object via rsh
#
#=item adev
#
#(Required) : disk name from filerA
#
#=back
#
#=item bdev
#(Required) : disk name from filerB
#
#None 
#
#=back
#
#=cut 

sub _pr_disk_verify_rw_all($$$$$) {
    my ($rHoH, $farsh, $fbrsh, $fadev, $fbdev) = @_;  

    _pr_debug("PR_DISK_VERIFY_RW_ALL:\n"); 
    _pr_debug("Verify disk is accessible from both nodes before reservation started\n"); 
    # ------------------------------------------------------------
    # verifies that the disk is readable and writeable from filerA
    # ------------------------------------------------------------
    my $output = _pr_say($farsh,"scsi dump_lba -l 0 -n 1 -o /etc/dump_lba.log $fadev", 2); 
    if ($output =~ /reservation conflict/) {

	_pr_say($farsh, "scsi pr_read_keys $fadev; scsi pr_read_reservation $fadev", 2); 
	_pr_abort($rHoH, "disk <$fadev> is NOT READABLE", $fadev); 
	return 0; 
    } else {
	_pr_log('-pass',"verifies the drive <$fadev> is readable", $fadev);
    }

    $output = _pr_say($farsh,"scsi test $fadev", 2); 
    if ($output =~ /reservation conflict/) {
	_pr_say($farsh, "scsi pr_read_keys $fadev; scsi pr_read_reservation $fadev", 2); 
	_pr_abort($rHoH, "disk <$fadev> is NOT WRITEABLE", $fadev); 
    } else {
	_pr_log('-pass', "verifies the drive <$fadev> is writable", $fadev); 
    }

    # ------------------------------------------------------------
    # verifies that the disk is readable and writeable from filerB
    # ------------------------------------------------------------
    $output = _pr_say($fbrsh,"scsi dump_lba -l 0 -n 1 -o /etc/dump_lba.log $fbdev", 2); 
    if ($output =~ /reservation conflict/) {
	_pr_abort($rHoH, "disk <$fbdev> is NOT READABLE", $fbdev); 
	return 0; 
      } else {
	_pr_log(-pass, "verifies the drive <$fbdev> is readable", ); 
    }

    $output = _pr_say($fbrsh, "scsi test $fbdev", 0); 
    if ($output =~ /reservation conflict/) {
      _pr_abort($rHoH, "disk <$fbdev> is NOT WRITEABLE", $fbdev); 
    } else {
	_pr_log('-pass', "verifies the drive <$fbdev> is writable"); 
    }

    return 1; #success
} #_pr_disk_verify_rw_all

=head2 pr_disk_get_resv_info

    pr_disk_get_resv_info(filer_conn=>$fco, disk=>$disk, generation_no=>$gen_no, resv_key=>$resv_key, resv_type=>$resv_type) 

The routine retrieves the generation number, the reservation type, and the
reservation key of the specified disk. 

=item Arguments

=over

=item $filer_conn

(Required) : connection object of the specified filer

=item $disk

(Required) : specify a disk name

=item $gen_no

(Required) : value of the reservation generation number

=item $resv_key

(Required) : the reservation key 

=item $resv_type

(Required) : reservation type of the specified disk

=back

=item Returns

returns 0 if the disk does not exist. 

=cut 

sub pr_disk_get_resv_info($$$$$) {
    my ($fc, $disk, $generation_r, $resv_key_r, $resv_type_r) = @_;  

    my $cnt = 0; 
    my $max_cnt = 10; 
    _pr_debug("PR_DISK_GET_RESERVATION_INFO:\n");
    $$generation_r = "UNKNOWN"; 
    $$resv_key_r = "UNKNOWN"; 
    $$resv_type_r = "UNKNOWN"; 
    # ------------------------------------------------------
    # try to read the reservation a few times before give up
    # This could cause by latency between cluster or we hit 
    # at the right update cycle 
    # ------------------------------------------------------
    my $output; 
    $output = _pr_say($fc, "scsi pr_read_reservation $disk", 2); 

	# ---------------------------------------------------
	# could be latency issue or right on the update cycle
	# ---------------------------------------------------
	if ($output =~ /error: check condition/) {
	    tstlog(-comment, "Unable to read reservation info of <$disk>; will try again"); 
	}

	if ($output =~ /not found/) {
	    tstlog(-fail, "Unable to get reservation information of disk <$disk>"); 
	    return 0; 
	}


    foreach my $line (split(/\n/, $output)) {
	my ($key, $val) = split(/: /, $line); 
	if ($key =~ /generation/) {
	    _pr_debug("generation=<$val>\n");
	    $$generation_r = $val; 
	    next; 
	}

	if ($key =~ /resv_key/) {
	    _pr_debug("resv_key=<$val>\n");
	    $$resv_key_r = $val; 
	    next; 
	}

	if ($key =~ /type/) {
	    _pr_debug("resv_type=<$val>\n");
	    $$resv_type_r = $val; 
	    return 1; 
	}

    } #each line
    return 0; #failed 
} #_pr_disk_get_resv_info

=head2 pr_load_destroy

The routine stops all hammer activities and then 
destroys the volume. The test needs the parameter
FILER_VOL. 

=over

=item Arguments

=over

=item rHoH

(Required) : The reference pointer of the hash table

=back

=item Returns

None

=cut 
sub pr_load_destroy($) {
    my ($rHoH) = @_;  

    # ----------------------------------------------
    # if the test does not create a vol then skip it
    # ----------------------------------------------
    if (!defined $FILER_VOL) {
	return;
    }

    _pr_debug("calling pr_load_destroy routine\n"); 
    if (defined $rHoH->{FB}) {
	if (vol_exists($rHoH->{FB}->{FCO}, $FILER_VOL)) {
	    _pr_say($rHoH->{FB}->{FCO}, "hammer abort", 0); 
	    sleep(10); 
	    vol_offline($rHoH->{FB}->{FCO}, $FILER_VOL);
	    sleep(10); 
	    vol_destroy($rHoH->{FB}->{FCO}, $FILER_VOL); 
	}
    }

    if (vol_exists($rHoH->{FA}->{FCO}, $FILER_VOL)) {
	_pr_say($rHoH->{FA}->{FCO}, "hammer abort", 0); 
	sleep(10); 
	vol_offline($rHoH->{FA}->{FCO}, $FILER_VOL); 
	sleep(10); 
	vol_destroy($rHoH->{FA}->{FCO}, $FILER_VOL); 
    }
} #pr_load_destroy 

=head2 pr_load_create

The routine creates a test volumes, waits until the volume
is online and then creates some load using hammer. 

In the future, we need to add NFS traffic to this. 
=over

=item Arguments

=over

=item rHoH

(Required) : The reference pointer of the hash table

=item vol_name

(Required) : name of the created test volume 

=back

=cut 
sub pr_load_create(\%%) {
    my ($rHoH, $vol_name) = @_;  
    my $filename = "/vol/$vol_name/hammer_file"; 

    # ------------------------------------------------
    # create and wait until the volume becomes online
    # if it already exists using it
    # ------------------------------------------------
    if (!vol_exists(($rHoH->{FA}->{FCO}, $vol_name))) {
	vol_create($rHoH->{FA}->{FCO}, $vol_name, 2); 
    }
    # start some hammer
    _pr_say($rHoH->{FA}->{FCO}, "hammer -f -1 $filename 1000000", 0);
    sleep(20); 
    
    if (!defined $rHoH->{FB}) {
	return 1; 
    }

    if (!vol_exists(($rHoH->{FB}->{FCO}, $vol_name))) {
	vol_create($rHoH->{FB}->{FCO}, $vol_name, 2);
    }
    _pr_say($rHoH->{FB}->{FCO}, "hammer -f -1 $filename 1000000", 0);
    sleep(20); 

} #pr_load_create 


=head2 pr_disk_resv_compare

pr_disk_resv_compare(ht_ref=>$rh_before, ht_ref=>$rh_after) 

The routine compares the reservation information including: 
generation, reservation type, and reservation key for each of
the selected disk to make sure they match the list before and after
reboot/panic/halt. 

Reboot could take some time and the duration varies from system
to system. To accommodate all cases, the routine will try 3 times
and if they all fail, then the result is conclusive. 

=over

=item Arguments

=over

=item ITable

(Required) : The reference pointer of the hash table

=back

=item Returns

None

=cut 

sub pr_disk_resv_compare(\%\%) {
    my ($rh_before, $rh_after) = @_;  

    _pr_debug("pr_disk_resv_compare:\n"); 

    my $cnt = 0; 
    my $max_cnt = 2; 
    my $sleep_time = 300; 
    my $fc = $rh_after->{FA}->{FCO}; 

    # --------------------------------------------------
    # will try 3 times to verify if there is a mismatch
    # will wait for about 2 minutes before retry again. 
    # ---------------------------------------------------
    pr_display_hash(%$rh_before); 
    pr_display_hash(%$rh_after); 
    my @mis_match_disks;

    while ($cnt++ < $max_cnt) {
	@mis_match_disks = (); 
	# ---------------------------------------------------    
	# compare resv type, key, and generation of each disk
	# before and after the system had been reboot
	# ---------------------------------------------------
	foreach my $disk (reverse keys %{$rh_before->{FA}->{DISKS}}) {    
	    _pr_debug("comparing disk <$disk>\n"); 

	    # --------------------------------------------------------	    
	    # To deal with a real BAD disk we just skip it altoghether
	    # since there is not much we can find out about PR for a
	    # BAD disk. How do we know if the disk is "BAD"? The answer
	    # is a BAD disk does not a serial number to start with. 
	    # --------------------------------------------------------
	    if ($rh_after->{FA}->{DISKS}->{$disk}[2] eq "UNKNOWN") {
	      _pr_debug("skip a BAD disk <$disk>\n"); 
	      next; 
	    }

	    # ----------------------------------------------------
	    # verify that resv generation number must be increased 
	    # ----------------------------------------------------
	    if ($rh_after->{FA}->{DISKS}->{$disk}[6] < $rh_before->{FA}->{DISKS}->{$disk}[6]) {
		push(@mis_match_disks, $disk); 
		_pr_debug(" ** disk <$disk> reservation generation before is <$rh_before->{FA}->{DISKS}->{$disk}[6]> and reservation generation number after is <$rh_after->{FA}->{DISKS}->{$disk}[6]>\n"); 
		next; # jump to the next disk 
	    } 
	    # verify that resv key must match
	    if ($rh_after->{FA}->{DISKS}->{$disk}[7] ne $rh_before->{FA}->{DISKS}->{$disk}[7]) {
		_pr_debug(" ** disk <$disk> reservation key before reboot is <$rh_before->{FA}->{DISKS}->{$disk}[7] and after reboot is <$rh_after->{FA}->{DISKS}->{$disk}[7]>\n"); 

		# ------------------------------------		
		# Try to get the reservation key again
		# before really mark the disk resv is 
		# different from before reboot
		# ------------------------------------
		my ($generation, $resv_key, $resv_type); 
		pr_disk_get_resv_info($fc, $disk, \$generation, \$resv_key, \$resv_type); 
		if ($rh_before->{FA}->{DISKS}->{$disk}[7] ne $resv_key) {
		  _pr_debug("The reservation key didn't match after second attempt\n"); 
		  push(@mis_match_disks, $disk); 	    
		}
		else {
		  _pr_debug("The reservation key matched after second attempt\n"); 
		  # update the reservation record
		  $rh_after->{FA}->{DISKS}->{$disk}[7] = $resv_key;  
		  $rh_after->{FA}->{DISKS}->{$disk}[8] = $resv_type; 
		}
	    } 

	    # ---------------------------------------	    
	    # verify that reservation type must match
	    # ---------------------------------------
	    if ($rh_after->{FA}->{DISKS}->{$disk}[8] ne $rh_before->{FA}->{DISKS}->{$disk}[8]) {
		push(@mis_match_disks, $disk); 	    
		_pr_debug(" ** disk <$disk> reservation type before reboot is <$rh_after->{FA}->{DISKS}->{$disk}[8] and after reboot is <$rh_before->{FA}->{DISKS}->{$disk}[8]>\n"); 
	    } 
	} #each disk 
	
       # -----------------------------------------
       # If there is no mis match then we are done
       # -----------------------------------------
       if ($#mis_match_disks <= 0) {
	   my $total_time = $cnt * $sleep_time; 
	   tstlog(-comment, "All disks reservation information are the same before and after reboot after $total_time seconds"); 
	   return 1; #success
       } else {
	   tstlog(-comment, "There are several mismatch disks: " . @mis_match_disks); 
       }
       
       tstlog(-comment, "Waiting $sleep_time for reservation information on all disks to be updated"); 
       sleep($sleep_time); 
       
       # -----------------------------------------------
       # reload all disks with the latest PR information
       # -----------------------------------------------
       my $bayid = 1; 
       my $filera = $rh_after->{FA}->{FNAME}; 
       my $filerb = $rh_after->{FB}->{FNAME}; 
       
       _pr_disk_load_info($rh_after, $filera, $bayid); 
       _pr_disk_load_info($rh_after, $filerb, $bayid); 
   } # loop at least 3 times before give up
    
    tstlog(-fatal, "Found a list of mismatch disks: " . @mis_match_disks); 
    return 1; #success
} #pr_disk_resv_compare

#=head2 _pr_disk_restore_ownership
#
#    _pr_disk_restore_ownership(Itable=>$rHoH, fname=>$fname, specify_disk=>$disk) 
#    
#Define the routine to restore the disk ownership of the specified disk.
#When an ownership is established, the owner should have write exclusive
#access to the disk. We need to make sure the ownership is established
#and the write exclusive is validated. 
#
#
#=over
#
#=item Arguments
#
#=over
#
#=item fname
#
#(Required) : the name of the filer
#
#=item disk
#
#(Required) : specify the same disk that needed to be restore the owner
#
#=back
#
#=item return
#
#None 
#
#=back
#
#=cut 

sub _pr_disk_restore_ownership($$$) {
    my ($rHoH, $filer, $pdisk) = @_;  
    
    my $fc = host($filer); 
    my $frsh = connect("#rsh"); 
    
    _pr_debug("PR_DISK_RESTORE_OWNERSHIP:\n"); 

    # ------------------------------------
    # restore the specified disk ownership
    # ------------------------------------
    _pr_say($frsh, "disk assign $pdisk -o $filer", 0); 

    # ----------------------------------------------    
    # verify that the specified should have an owner
    # ----------------------------------------------
    my $output = _pr_say($frsh, "disk show $pdisk", 0); 
    
    foreach my $line (split(/\n/, $output)) {
	if ($line =~ /DISK/ || $line =~ /--------/) {
	    next; 
	}
	my ($disk, $owner, $pool, $serial_num) = split(/ +/, $line); 
	if ($disk ne $disk) {
	    _pr_abort($rHoH, "Unable to restore the ownership of <$disk>", $disk); 
	    return 0; #failure

      } 
    } # foreach 

    # ----------------------------------------------------
    # verify the ownership and write exclusive on the disk
    # ----------------------------------------------------
    _pr_disk_verify_rw($rHoH, $frsh, $pdisk, "we", "owner"); 
    
    tstlog(-comment, "Successfully restore the ownership of <$pdisk>"); 
    return 1; #success
} #_pr_disk_restore_ownership


#=head2 _pr_disk_display_resv
#    
#    _pr_disk_display_resv(filer_conn=>$fc, disk_name=>$disk) 
#    
#    The routine simply queries and displays the reservation information
#    of the specify disk.
#
#=over
#
#=item Arguments
#
#=over
#
#=item filer_conn 
#
#(Required) : specify the filer connection object
#
#=item disk_name
# 
#(Required) : name of the display to be displayed 
#
#=back
#
#=item Returns
#
#None
#
#=back
#
#=cut
 
sub _pr_disk_display_resv($$) {
    my ($fc, $disk) = @_;  

    _pr_say($fc, "scsi pr_read_keys $disk", 2); 
    _pr_say($fc, "scsi pr_read_reservation $disk", 2); 

} #_pr_disk_display_resv


#=head2 _pr_disk_load_info
#    
#    _pr_disk_load_info(ITable=>$rHoH, filer_name=>$fname, bay_id=>$bayid) 
#    
#    The routine  storage commands to retrieve information regarding
#    all disks on the cluster. The hash table will be loaded with a list
#    of disks based on key=disk_name and value=disk_serial_number. If
#    the bay_id is specified, then only disks with BAY > $bayid will
#    be retrieved. 
#
#    To create a complete list of disks, the routine shall temporarily
#    restore the ownership of disk without an owner and then remove
#    their ownership after gathering the model number and its pool ID.  
#    
#    The routine return 1 on success and 0 on failure
#    
#=over
#
#=item Arguments
#
#=over
#
#=item filer_conn
#
#(Required) : specify the filer connection object
#
#=item list_of_disks
# 
#(Required) : specify the hash table by reference of the list of disks
#
#=back
#
#=item Returns
#
#1 - success
#0 - fail - unable to connect to the specify filer or unable to retrieve
#    a list of disk from the specify filer.  
#
#=back
#
#=cut 

sub _pr_disk_load_info(\%$;$) {
    my ($rHoH, $fname, $bayid) = @_;  
    my ($fc, $fn, $disks_without_owner);
    
    # --------------------------------------------    
    # save the filername and its connection object
    # --------------------------------------------
    if ($rHoH->{FA}->{FNAME} eq $fname) {
	$fc = $rHoH->{FA}->{FCO};
	$fn = "FA"; 
    } else {
	$fc = $rHoH->{FB}->{FCO}; 
	$fn = "FB"; 
    }

    # ---------------------------------------------------------
    # load the RaidInfo object so we could get a list of spares
    # ---------------------------------------------------------
    my $fa_obj = new RaidInfo($rHoH->{FA}->{FCO}); 
    my $fb_obj = new RaidInfo($rHoH->{FB}->{FCO}); 
    my %fa_spares = $fa_obj->spare_disks(); 
    my %fb_spares = $fb_obj->spare_disks(); 
    my %fa_fails  = $fa_obj->failed_disks(); 
    my %fb_fails  = $fb_obj->failed_disks(); 
    # ---------------------------------------------------------------
    # restore disks ownership temporarily so we could retrieve detail
    # information about the disk its Pool ID, owner, and model number
    # In case of NON SANOWN, we just skip right over this. 
    # 06/01/06 - Tom Doan - let not worry about this without any owner
    # since they could be bad disks and we don't want to try out PR
    # on bad disks
    # ---------------------------------------------------------------

    if ($rHoH->{SANOWN_STATE} == 1) {
      $disks_without_owner = "No disks match"; 
#	$disks_without_owner = _pr_say($fc, "disk show -n"); 
#	
#	if ($disks_without_owner !~ /No disks match/) {
#	    foreach my $line (split(/\n/, $disks_without_owner)) {
#		if ($line =~ /DISK/ || $line =~ /---------/) {
#		    next; 
#		}
#		my ($disk, $f1, $f2, $pool, $serialN) = split(/ +/, $line); 
#		_pr_disk_restore_ownership($rHoH, $fname, $disk); 
#	    } #foreach 
#	}
    } #if 
    
    # ----------------------------
    # retrieve the available disks 
    # ----------------------------
    my $output = _pr_say($fc, "storage show disk ",0);  
    my @storage_disks = split(/\n/, $output); 
    
    # ---------------------------------------------------
    # build the hash table based with key=<disk_name>
    # and value=<disk_serial>. If there is a disk without
    # a serial number, then probably the disk is in the
    # FAIL state. Only disks with bayid greater than the 
    # specified bayID will be added to the list
    # ---------------------------------------------------
    foreach my $line (@storage_disks) {
	
	# ---------------------      
	# skip the header block
	# ---------------------
	if ($line =~ "DISK" || $line =~ "---------") {
	    next; 
	}
	
	# ------------------------------
	# We should NOT try PR on: bad disk, mailbox disks, and
        # we only look for spare disks 
	# ------------------------------
	my ($disk, $shelf, $bay, $serialN, $vendor, $model, $rev) = split (/ +/, $line); 
	# --------------------------------	
	# retrieve the disk PR information 
	# --------------------------------
	my ($generation, $resv_key, $resv_type); 
	pr_disk_get_resv_info($fc, $disk, \$generation, \$resv_key, \$resv_type); 
	tstlog(-comment, "generation=<$generation>  resv_key=<$resv_key>  resv_type=<$resv_type>"); 

	# -----------------------------------------------------
	# determine the state of the disk which could be spare, 
	# fail, or a system disk 
	# -----------------------------------------------------
	my $state; 
	if (defined($fa_spares{$disk}) || defined($fb_spares{$disk})) {
	  $state = "SPARE"; 
	}
	elsif (defined($fa_fails{$disk}) || defined($fb_fails{$disk})) {
	  $state = "FAIL"; 
	}
	else {
	  $state = "USED";
	}

	# -----------------------
	# try to catch a bad disk
	# -----------------------
	if ($serialN eq "NETAPP" || $rev eq "") {
	    $rHoH->{$fn}->{DISKS}->{$disk} =  [ $shelf, $bay, "UNKNOWN", $model , "NOT_OWNED", "UNKNOWN", $generation, $resv_key, $resv_type, $state];
	} 
	else {
	    $rHoH->{$fn}->{DISKS}->{$disk} =  [ $shelf, $bay, $serialN, $model , "NOT_OWNED", "UNKNOWN", $generation, $resv_key, $resv_type, $state];
	  }

	# -------------------------------
	# pre-build a list of disk models
	# with one of the spare disks 
	# -------------------------------
	if ($rHoH->{$fn}->{DISKS}->{$disk}[9] ne "SPARE") {
	  next; 
	}
	
	if (!defined $rHoH->{MODELS}->{$model} && $bay > 1) {
	  $rHoH->{MODELS}->{$model} = [ $disk, $shelf, $bay, $serialN]; 
	}
    } #foreach 
    
    # -----------------------------------------------------------
    # now retrieve a list of disks with owner, model, and pool ID 
    # This should include disk with and without the owner. 
    # -----------------------------------------------------------
    if ($rHoH->{SANOWN_STATE} == 1) {
	my $disk_shows = _pr_say($fc, "disk show", 2); 
	foreach my $disk_line (split(/\n/, $disk_shows)) {
	    if ($disk_line =~ "DISK" || $disk_line =~ /---------/) {
		next; 
	    }
	    
	    my ($disk, $owner, $ownerid, $pool, $serialN) = split(/ +/, $disk_line); 
	    
	    if (defined $rHoH->{$fn}->{DISKS}->{$disk}) {
		$rHoH->{$fn}->{DISKS}->{$disk}[4] = $owner; 
		$rHoH->{$fn}->{DISKS}->{$disk}[5] = $pool; 
	    } else {
		_pr_abort($rHoH, "Found unidentified disk <$disk>", $disk);
		return 0; #failure
	    }
	    
	} #foreach 
    } #if 

    # ------------------------------------------------------------
    # identifies which disks are spare. We need to have at least 3 
    # spares in order to conduct persistent reservation testing. 
    # For non SANOWN this is not required. 
    # ------------------------------------------------------------
    if ($rHoH->{SANOWN_STATE} == 1) {
      my $disk_spares = _pr_say($fc, "vol status -s", 0); 
      foreach my $disk_line (split(/\n/, $disk_spares)) {
	
	# skip blank lines
	if ($disk_line !~ /^spare/) {
	  next; 
	}

	my ($rdisk, $disk, @buf) = split(/ +/, $disk_line); 
	  
	if ($rdisk ne /spare/) {
	  next; 
	}
	
	if (defined $rHoH->{$fn}->{DISKS}->{$disk}) {
	  $rHoH->{$fn}->{DISKS}->{$disk}[5] = "spare"; 
	} else {
	  _pr_abort($rHoH, "Found unidentified disk <$disk>", $disk);
	  return 0; #failure
	}
	
      } #foreach 
    } #if 
    
    # -------------------------------------
    # now restore those disks without owner
    # back to the state "NOT_OWNED"
    # -------------------------------------
    if ($rHoH->{SANOWN_STATE} == 1) {
      if ($disks_without_owner =~ /No disks match/) {
	    return 1; 
	  }
      foreach my $line (split(/\n/, $disks_without_owner)) {
	if ($line =~ /DISK/ || $line =~ /---------/) {
	  next; 
	}
	
	my ($disk, $f1, $f2, $pool, $serialN) = split(/ +/, $line); 
	_pr_disk_rm_ownership($rHoH, $fc, $disk); 
	
	# make the disk as NOT OWNED
	if (defined $rHoH->{$fn}->{DISKS}->{$disk}) {
	  $rHoH->{$fn}->{DISKS}->{$disk}[4] = "NOT_OWNED"; 
	}
	    
      } #foreach 
    } #if 

    return 1; #success    
} #_pr_disk_load_info


#=head2 _pr_get_fail_disks
#    
#    _pr_get_fail_disks(ITable=>$rHoH, filer_name=>$fname) 
#    
#    The routine retrieve a list of FAIL disks which are not the right
#    candidate for running any PR test. 

#    The routine return 1 on success and 0 on failure
#    
#=over
#
#=item Arguments
#
#=over
#
#=item ITable
#
#(Required) : specify the hash table by reference of the list of disks
#
#=back
#
#=item Returns
#
#1 - success
#0 - fail - unable to connect to the specify filer or unable to retrieve
#    a list of disk from the specify filer.  
#
#=back
#
#=cut 

sub _pr_get_fail_disks(\%$;$) {
    my ($rHoH, $fa, $fb) = @_;  
    my ($fc, $fn, $disks_list);
    
    # --------------------------------------------    
    # retrieve the filerA connection 
    # --------------------------------------------
    if ($rHoH->{FA}->{FNAME} eq $fa) {
	$fc = $rHoH->{FA}->{FCO};
	$fn = $fa; 
    }

    $disks_list = _pr_say($fc, "disk show -v", 0); 
    foreach my $line (split(/\n/, $disks_list)) {
      if ($line =~ /DISK/ || $line =~ /---------/) {
	next; 
      }
      my ($disk, $owner, $ownerid, $pool, $serialN) = split(/ +/, $line); 

      # -----------------------------------------------------------      
      # select a bad disks based on status FAILED or SERIAL NUMBER
      # is BLANK
      # -----------------------------------------------------------
      if (!defined($pool) || !defined($serialN) ) {
	$rHoH->{$fn}->{BADDISKS}->{$disk} = [ $disk, $pool, $serialN]; 
      }
    } #foreach 

    # --------------------------------------------    
    # retrieve the filerB connection 
    # --------------------------------------------
    if ($rHoH->{FB}->{FNAME} eq $fb) {
	$fc = $rHoH->{FB}->{FCO};
	$fn = $fb; 
    }

    $disks_list = _pr_say($fc, "disk show -v", 0); 
    foreach my $line (split(/\n/, $disks_list)) {
      if ($line =~ /DISK/ || $line =~ /---------/) {
	next; 
      }
      my ($disk, $owner, $ownerid, $pool, $serialN) = split(/ +/, $line); 
      
      # -----------------------------------------------------------      
      # select a bad disks based on status FAILED or SERIAL NUMBER
      # is BLANK
      # -----------------------------------------------------------
      if (!defined($pool) || !defined($serialN)) {
	$rHoH->{$fn}->{BADDISKS}->{$disk} = [ $disk, $pool, $serialN]; 
      }
    } #foreach 

    return 1; #success    
} #_pr_get_fail_disks


#=head2 _pr_disk_get_generation
#
#my $generation = _pr_disk_get_generation(fa_conn=>$fac, fb_conn=>$fbc, fa_dev=>$fadev, fb_dev=>$fbdev) 
#
#my $generation = _pr_disk_get_generation($fac, $fbc, 
#Retrieve the persistent reservation generation information of the specified disk from the cluster. The generation information should be in the following format: "generation: nnnnn". 
#
#=over
#
#=item Arguments
#
#=over
#
#=item $fac
#
#(Required) : FILERA connection object
#
#=item $fbc
#
#(Required) : FILERB connection object
#
#=item $fadev
# 
#(Required) : specify a disk from FILERA
#
#=item $fbdev
#
#(Required) : specify the same disk that seen from FILERB
#
#=back
#
#=item Returns
#
#Returns the persistent reservation generation sequence of the specified disk.
#
#=back
#
#=cut 

sub _pr_disk_get_generation($$$$) {
    my ($fac, $fbc, $fadev, $fbdev) = @_; 

    my $aoutput = _pr_say($fac,"scsi pr_read_keys $fadev", 2); 
    my $boutput = _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);
    
    my (@ageneration) = split(/\n/, $aoutput); 
    my (@bgeneration) = split(/\n/, $boutput); 

    if ($ageneration[0]  =~ $bgeneration[0]){
	return $ageneration[0]; 
    }
    return "";
} #_pr_disk_get_generation

=head2 pr_verify_generation

The function compares if the persistent reservation generation on the disk and the specified generation are the same. If the generation is the same then returns 1. 

=over

=item Arguments

=over

=item $fac

(Required) : FILERA connection object

=item $fbc

(Required) : FILERB connection object

=item $fadev

(Required) : specify a disk from FILERA

=item $fbdev

(Required) : specify the same disk that seen from FILERB

=item $generation

(Required) : specify the persistent reserations generation info to be compared with. 

=back

=item Returns

Returns 1 if the PR generation is the same.

Returns 0 if the PR generation is different.

=back

=cut 

# pr_verify_generation
sub pr_verify_generation($$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $generation) = @_; 
    
    my ( $aoutput) = _pr_say($fac,"scsi pr_read_keys $fadev",2); 
    my ($boutput) = _pr_say($fbc,"scsi pr_read_keys $fbdev", 2); 
    
    # ----------------------------------------------
    # compare the specified generation number with 
    # the current generation, return 1 if they match
    # else return 0; 
    # ----------------------------------------------
    if ( $aoutput =~ $generation && $boutput =~ $generation) {
	tstlog(-comment, "persistent reservation generation remains the same"); 
	return 1; #success
    } 
    
    _pr_debug("pr_mis_match: $aoutput"); 
    _pr_debug("pr_mis_match: $boutput"); 
    _pr_abort($rHoH, "persistent reservation generation has been altered.");
    return 0; #failure
} #pr_mis_match

#=head2 _pr_filer_get_key
#    
#The routine returns the PR key of the specified filer.
#    
#The PR key is the hex value of the unique System ID of each node. 
#The PR key has been derived from the value of the NVRAM. 
#      
#=over
#
#=item Arguments
#
#=over
#
#=item $fc
#
#(Required) : FILER connection object
#
#=item $fakey_r
#(Required) : reference of the filerA PR key 
#
#=item $fbkey_r
#(Required) : reference of the filerB PR key 
#
#=item Returns
#
#Returns 0 if there is a problem of retrieving the PR key. 
#Returns 1 if there is no problem in retrieving both PR keys.
#Returns 2 if there is a problem in retrieving the partner key. 
#
#=back
#
#=cut 

sub _pr_filer_get_key($$$$) {
  my ($rHoH, $fc, $fakey_r, $fbkey_r ) = @_; 
  my $display = 0;

    my $output = _pr_say($fc, "sysconfig", 0); 
    my @lines = split("\n", $output); 
    foreach my $line (@lines) {
	if ($line =~ /System ID:/) {
	    # ------------------------------------
	    # Convert the System ID to a hex value
	    # ------------------------------------
	    my @fields = split(" ", $line); 
	    $$fakey_r = sprintf("0x%x", $fields[2]);

	    # Convert the Partner ID as well
	    if ($line =~ /partner ID:/) {
	      $$fbkey_r = sprintf("0x%x", $fields[6]); 
	    } else {
	      tstlog('-warn', 'The system is a stand alone node'); 
	      return 2; 
	    }
	    return 1; #successful 
	}
    } #foreach 
    
    # ------------------------------------------------
    # Unable to retrieve the PR key from the System ID
    # ------------------------------------------------
    _pr_abort($rHoH, "unable to retrieve the System ID; therefore the value of PR key could not be determined at this time"); 
    return 0; #failure
} #_pr_filer_get_key


# _pr_mode_enable
sub _pr_mode_enable($$) {
    my ($fac, $fbc) = @_; 
    
    printf("\nPR_MODE_ENABLE: enable the persistent reservation test mode\n"); 
    
    _pr_say($fac,"scsi pr_test_mode -t enable",0); 
    _pr_say($fbc,"scsi pr_test_mode -t enable", 0); 
    _pr_say($fac,"vol media_scrub stop", 0); 
    _pr_say($fbc,"vol media_scrub stop", 0); 
    return 1; #success
} #pr_mode_enable

# ---------------------------
# _pr_mode_disable
# $fac -- the filer connection object
# ---------------------------
sub _pr_mode_disable($$) {
  my ($fac, $fbc) = @_; 

  _pr_debug("\nPR_MODE_DISABLE: disable the persistent reservation test mode\n"); 

  _pr_say($fac,"scsi pr_test_mode -t disable", 0); 
  _pr_say($fbc,"scsi pr_test_mode -t disable", 0 ); 
  _pr_say($fac,"vol media_scrub start", 0); 
  _pr_say($fbc,"vol media_scrub start", 0); 
  return 1; #success
} #_pr_mode_disable

# ------------------------------------------------
# pr_read_keys
# $fac -- connection object of the primary 
# $fbc -- connection object of the secondary
# $fadev - specify a disk on FILERA 
# $fbdev - specifiy a disk on FILERB
# ------------------------------------------------
sub pr_read_keys($$$$) {
    my ($fac, $fbc, $fadev, $fbdev) = @_;
    
    _pr_debug("\nPR_READ_KEYS: read keys from both filers\n"); 
    
    _pr_say($fac,"scsi pr_read_keys $fadev", 0); 
    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2); 
    return 1; #success
}

# ------------------------------------------------
# pr_register_keys
# $fac -- connection object
# $dev   -- device name
# ------------------------------------------------
sub pr_register_keys($$$$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey) = @_;
    
    _pr_debug("\nPR_REGISTER_KEYS: register keys on both filers\n"); 
    
    _pr_say($fac,"scsi pr_register $fadev", 2); 
    my $output = _pr_say($fac,"scsi pr_read_keys $fadev", 2); 
    if ($output !~ $fakey) {
	_pr_abort($rHoH, "the registered key <$fakey> does not match"); 
	return 0; #failure
    } else {
	tstlog(-pass, "the registered key <$fakey> matched"); 
    }
    
    _pr_say($fbc,"scsi pr_register $fbdev", 2); 
    $output = _pr_say($fbc,"scsi pr_read_keys $fbdev", 2); 
    if ($output !~ $fbkey) {
	_pr_abort($rHoH, "the registered key <$fbkey> does not match"); 
	return 0; #failure
    } else {
	tstlog(-pass, "the registered key <$fbkey> matched"); 
    }

    return 1; #success
} #pr_register_keys

=head2 pr_verify_resv

    For each of the selected disks, tries to establish PR on the disk
    and verifies whether or not the following information is correct:
    1. persistent key must match the owner key
    2. reservation key must match the specified reservation type
    3. read and write permission must be enable or disable partially or
       completely. 
    4. For SANOWN, the ownership will be removed temporarily for the disks and then restored back before the test exits. 

=over

=item Arguments

=item $rHoH 

(Required) - the PR hash table reference

=over

=back

=item Returns

None 

=cut 
sub pr_verify_resv(\%) {  
    my ($rHoH) = @_; 
  
    my $farsh  = $rHoH->{FA}->{FCO}; 
    my $fbrsh  = $rHoH->{FB}->{FCO}; 
    my $fakey  = $rHoH->{FA}->{PRKEY}; 
print "fakey=" . $fakey . "\n"; 
    my $fbkey  = $rHoH->{FB}->{PRKEY}; 
    my $filera = $rHoH->{FA}->{FNAME};
    my $filerb = $rHoH->{FB}->{FNAME};
    
    # ------------------------
    # enable test PR test mode
    # ------------------------
    _pr_mode_enable($farsh, $fbrsh);
    
    # determine how many disks to run the test with
    if ($#{$rHoH->{FA}->{SEL_DISKS}} < 0) {
	_pr_abort($rHoH, "Unable to find any disks for PR testing due there is no spare with the specified type for testing"); 
	return 0; #failure
    }

    # ------------------------------------------------------------
    # run and verify PR functionalities against the selected disks
    # ------------------------------------------------------------
    for (my $i=0; $i <= $#{$rHoH->{FA}->{SEL_DISKS}}; $i++) {
	my $fad = $rHoH->{FA}->{SEL_DISKS}[$i]; 
	my $fbd = $rHoH->{FB}->{SEL_DISKS}[$i]; 
	
	# ----------------------------------------------
	# get the current index of the message logs
	# ----------------------------------------------
	#$rHoH->{FA}->{MLOG_INDEX} = _get_msg_current_index($filera); 
	#$rHoH->{FB}->{MLOG_INDEX} = _get_msg_current_index($filerb); 

	_pr_debug("\t\t ******* testing disk <$fad> and <$fbd> ******\n"); 
	
	#--------------------------------
	# determine the owner of the disk
	# in order to fail the disk accordingly
	#--------------------------------
	my $disk_owner = $rHoH->{FA}->{DISKS}->{$fad}[4]; 
	if ($disk_owner eq $filera) {
	    _disk_remove($farsh, $fad); 
	} elsif ($disk_owner eq $filerb) {
	    _disk_remove($fbrsh, $fbd);
	} else {
	  tstlog(-comment, "disk <$fad> has no ownership"); 
	}

	_pr_clear($farsh, $fbrsh, $fad, $fbd); 
	pr_read_keys($farsh, $fbrsh, $fad, $fbd);
	

	my %resv_tb = pr_get_resv_types($rHoH, $fad); 
	# ----------------------------------------------------
	# run and verify the test for each type of reservation
	# ----------------------------------------------------
	foreach my $restype (keys %resv_tb) { 
	    # verify PR write exclusively
	    pr_register_keys($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey);
	    _pr_exclusive($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey, $restype); 
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);

	    # ------------------------------------------	    
	    # verify PR preempt write exclusively
	    # ------------------------------------------
	    _pr_disk_verify_rw_all($rHoH, $farsh, $fbrsh, $fad, $fbd); 
	    _pr_preempt_write_exclusive($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey, $restype);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);
	    pr_register_keys($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey); 
	    _pr_clear($farsh, $fbrsh, $fad, $fbd); 

	    # ------------------------------------------	
	    # verify PR preempt write exclusively ignore
	    # ------------------------------------------
	    _pr_disk_verify_rw_all($rHoH, $farsh, $fbrsh, $fad, $fbd); 
	    _pr_preempt_write_exclusive_ignore($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey, $restype);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);
	    pr_register_keys($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd); 

	    # ------------------------------------------	    
	    # verify PR preempt exclusively access
	    # ------------------------------------------
	    _pr_disk_verify_rw_all($rHoH, $farsh, $fbrsh, $fad, $fbd); 
	    _pr_preempt_exclusive_access($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey, $restype);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);
	    pr_register_keys($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd); 
	  
	    # ------------------------------------------
	    # verify PR preempt write exclusively local
	    # ------------------------------------------
	    _pr_disk_verify_rw_all($rHoH, $farsh, $fbrsh, $fad, $fbd); 
	    _pr_preempt_write_exclusive_local($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey, $restype);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);
	    pr_register_keys($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd); 
	  
	    # ------------------------------------------
	    # verify PR preempt exclusive access local  
	    # ------------------------------------------
	    _pr_disk_verify_rw_all($rHoH, $farsh, $fbrsh, $fad, $fbd); 
	    _pr_preempt_exclusive_access_local($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey, $restype);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);
	    pr_register_keys($rHoH, $farsh, $fbrsh, $fad, $fbd, $fakey, $fbkey);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd); 
	    
	    # clear reservation one more time 
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);

	    # ----------------------------	    
	    # run abnormal test cases
	    # ----------------------------
	    _pr_run_abnormal_tests($rHoH, $farsh, $fbrsh, $fad, $fbd);
	    _pr_clear($farsh, $fbrsh, $fad, $fbd); 
	    # clear reservation one more time 
	    _pr_clear($farsh, $fbrsh, $fad, $fbd);

	} #foreach reservation type 

	#-----------------------
	# restore the disk back 
	#-----------------------
	$disk_owner = $rHoH->{FA}->{DISKS}->{$fad}[4]; 
	if ($disk_owner eq $filera) {
	    _disk_unfail($farsh, $fad); 
	} elsif ($disk_owner eq $filerb) {
	    _disk_unfail($fbrsh, $fbd);
	} else {
	  tstlog(-comment, "disk <$fad> has no ownership"); 
	}
    } #foreach 
    _pr_mode_disable($farsh, $fbrsh);  

    _pr_report($rHoH); 
    return 1; #success
} # pr_verify_resv

# -----------------------------------------------------------------
# _pr_exclusive - reserve with either
# access mode exclusive access "ea" or write exclusive "we"
# $fac -- connection object of filera
# $fbc -- connection object of the partner
# $fadev  -- device name of filera
# $fbdev  -- device name of filerb
# $fakey -- filerA reservation key
# $fbkey -- filerB reservation key 
# $res_t  -- "we -- write exclusive" "wero", "eaar", "wear', and  
#           "ea -- exclusive access"
# -----------------------------------------------------------------
sub _pr_exclusive($$$$$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey, $res_t) = @_;
    
    _pr_debug("\nPR_EXCLUSIVE: write exclusive reserve test on filera\n");  
    my $output = _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2); 
    if ($output !~ /action succeeded/) {
      _pr_abort($rHoH, "Unable to reserve disk <$fadev> with mode <$res_t>", $fadev); 
    } else {
      tstlog(-pass, "Successfully reserved disk <$fadev> with mode <$res_t>");
    }

    # ----------------------------------------------------    
    # verify access permission from both owner and partner
    # ----------------------------------------------------
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "owner"); 
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "partner"); 
    _pr_disk_verify_resv($rHoH, $fac, $fadev, $fakey, $res_t); 

    # --------------------------------------------------    
    # verify that an attempt to reserve the disk by the 
    # other node should fail 
    # --------------------------------------------------
    $output = _pr_say($fbc,"scsi pr_reserve -t $res_t $fbdev", 2); 
    if ($output =~ /reservation conflict/) {
	tstlog(-pass, "the disk <$fbdev> cannot be reserved by other node"); 
    } else {
	_pr_abort($rHoH, "the disk <$fbdev> should not be reserved again", $fbdev); 
	return 0; #failure
    }

    # -------------------------------------------------------
    # verify that access and reservation should remain intact
    # after the partner node failed to reserve the disk
    # -------------------------------------------------------
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "owner");
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "partner");  
    _pr_disk_verify_resv($rHoH, $fac, $fadev, $fakey, $res_t); 

    return 1; #success
} #_pr_exclusive

# -----------------------------------------------
# _pr_preempt_write_exclusive
# $fac -- connection object
# $fadev   -- device name
# $fbc -- filerb connection object
# $fbdev - name of the driver
# $fakey -- the filerA reservation key
# $fbkey -- the filerB reservation key
# $res_t -- reservation type
# ------------------------------------------------
sub _pr_preempt_write_exclusive($$$$$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey, $res_t) = @_; 
    my $key; 

    _pr_debug("\n_PR_PREEMPTWRITE_EXCLUSIVE: preemption of write exclusive on filera\n"); 

    _pr_say($fac,"scsi pr_register $fadev",0 ); 
    my $output = _pr_say($fac, "scsi pr_read_keys $fadev", 2);
    $key = _get_prkey($output); 
    
    _pr_say($fbc,"scsi pr_register $fbdev", 2); 
    
    $output = _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2);
    if ($output !~ /action succeeded/) {
	_pr_abort($rHoH, "Unable to reserve disk <$fadev> with mode <$res_t>", $fadev); 
	return 0; #failure
    } else {
	tstlog(-pass, "Successfully reserved disk <$fadev> with mode <$res_t>");
    }
    
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "owner");

    _pr_disk_verify_resv($rHoH, $fac, $fadev, $fakey, $res_t); 
    
    # ----------------------------------------------
    # verify what the partner node can or cannot do
    # ----------------------------------------------
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "partner"); 
#    _pr_disk_verify_resv($rHoH, $fbc, $fbdev, $fbkey, $res_t); 
#    _pr_say($fbc,"scsi pr_read_reservation $fbdev");   

    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);

    # -----------------------------------------------
    # FilerB is going to take over the disk ownership
    # -----------------------------------------------
    my $try = 3; 
    my $cnt = 0; 
    my $o = _pr_say($fbc,"scsi pr_preempt -t $res_t -s $key  $fbdev", 2);   
    while ($o !~ /action succeeded/ && $cnt++ < $try) { #handling preempt error for any reasons 
      _pr_preempt_retry($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey); 
      $o = _pr_say($fbc,"scsi pr_preempt -t $res_t -s $key  $fbdev", 2);   
    }
    # ----------------------------------------------------------
    # the disk should belong to nodeB and verify reservation key
    # must be the filerB key
    # ----------------------------------------------------------
    _pr_disk_verify_resv($rHoH, $fbc, $fbdev, $fbkey, $res_t); 

    # ----------------------------------------------------    
    # verify the filerB access privileges after preemption
    # ----------------------------------------------------
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "ope"); 

    # node A now should only have limited access
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "ppe"); 

    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);   
    _pr_disk_verify_resv($rHoH, $fbc, $fbdev, $fbkey, $res_t); 
    _pr_say($fbc,"scsi pr_clear $fbdev", 2);

    return 1; #success
} #_pr_preempt_write_exclusive

# -----------------------------------------------
# _pr_preempt_write_exclusive_ignore
# The routine utilize 
# $fac -- connection object
# $dev   -- device name
# $fakey -- filerA key
# $fbkey -- filerB key
# ------------------------------------------------
sub _pr_preempt_write_exclusive_ignore($$$$$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey, $res_t) = @_; 
    my $key; 
    
    _pr_debug("\nPR_PREEMPT_WRITE_EXCLUSIVE_IGNORE: preemption of write exclusive ignore on filera\n"); 

    # ----------------------------------------------------
    # Right after the disk had been reserved exclusive access (ea)
    # the disk PR got cleared out by the system update (not from the
    # test) for some reason which cause the test to fail. To work
    # around this problem, we need to try to reserve the disk again
    # for 3 times before giving up. 
    # ----------------------------------------------------
    my $attempt = 0; 
    while ($attempt++ <= 3) { 
      _pr_say($fac,"scsi pr_register $fadev", 2); 
      _pr_say($fac,"scsi pr_read_keys $fadev", 2); 
      my $output = _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2); 
      if ($output !~ /action succeeded/) {
	_pr_abort($rHoH, "Unable to reserve disk <$fadev> with mode <$res_t>", $fadev); 
      } else {
	tstlog(-pass, "Successfully reserved disk <$fadev> with mode <$res_t>");
      }

      # ---------------------------------------------  
      # we might have to read again and again until
      # we have the right key. This is a real pain
      # since PR could be very unpredictable sometime.
      # ----------------------------------------------
      my $cnt = 0; 
      while (!defined($key)) {
	$output = _pr_say($fac,"scsi pr_read_keys $fadev",2); 
	$key = _get_prkey($output);
	if ($cnt++ > 4 && $attempt >= 3) {
          _pr_abort($rHoH, "Unable to get the prkey for <$fadev> after <$attempt> attempts"); 
	}
	if (!defined($key)) {
	  _pr_debug("** unable to retrieve $fadev key after $cnt attempt**\n");
	}
	sleep(1); 
      }

      # ---------------------------------
      # get the defined key, then we done
      # ---------------------------------
      if (defined($key)) {
	last; 
      }
      # ----------------------------------------------------------
      # Turn on the message if the normal run failed (attempt == 1) 
      # ----------------------------------------------------------
      if ($attempt > 1) {
	_pr_debug("ATTEMPT: try <$attempt>\n");  
      }
    } # attempt reservation 3 times 

    # -----------------------------------------------
    # register ignore key as many time as we want and
    # there should not be any error 
    # -----------------------------------------------
    my $output = _pr_say($fbc,"scsi pr_register_ignore_key $fbdev", 2); 
    if ($output =~ /reservation conflict/) {
	_pr_abort($rHoH, "register ignore key does not work after the 3 try"); 
	return 0; #failure	
    }
    
#    _pr_say($fac,"scsi test $fadev", 2); 
#    _pr_say($fac,"scsi dump_lba -l 0 -n 1 -o /etc/dump_lba.log $fadev", 2); 
    # -----------------------------------------    
    # verify that owner should have full access 
    # -----------------------------------------
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "owner"); 
    _pr_say($fac,"scsi pr_read_reservation $fadev", 2); 

    # ---------------------------------------------    
    # verify the access privileges from the partner
    # ---------------------------------------------
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "partner"); 

    _pr_say($fbc,"scsi pr_read_reservation $fbdev", 2);   
    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);   

    _pr_say($fbc,"scsi pr_preempt -t $res_t -s $key $fbdev", 2);   

    # ----------------------------------------------------------
    # the disk should belong to nodeB and verify reservation key
    # must be the filerB key
    # ----------------------------------------------------------
    _pr_disk_verify_resv($rHoH, $fbc, $fbdev, $fbkey, $res_t); 

    _pr_say($fbc,"scsi pr_read_reservation $fbdev", 2);   
    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);   

    # ---------------------------------------------
    # verify access priv from both owner and partner
    # ---------------------------------------------
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "ope"); 
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "ppe"); 

    #verify the filerA shall not be able to clear PR
    $output = _pr_say($fac, "scsi pr_clear $fadev", 2); 
    if ($output !~ /reservation conflict/ ) {
	_pr_abort($rHoH, "filerA was able to clear PR of <$fadev>; however, it should NOT"); 
	return 0; #failure
    }
    _pr_say($fbc,"scsi pr_clear $fbdev", 2);   
    return 1; #success
} #_pr_preempt_write_exclusive_ignore

# -----------------------------------------------
# _pr_preempt_write_exclusive_local
# $fac -- connection object
# $fbc -- filerb connection object
# $fadev   -- filera's device name
# $fbdev   -- filerb's device name
# $res_t  -- reservation type 
# ------------------------------------------------
sub _pr_preempt_write_exclusive_local($$$$$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey, $res_t) = @_; 
    my $key; 
    
    _pr_debug("\nPR_PREEMPT_WRITE_EXCLUSIVE_LOCAL: preemption of write exclusive on filera locally\n"); 
    
    _pr_say($fac,"scsi pr_register $fadev", 2); 
    my $output = _pr_say($fac,"scsi pr_read_keys $fadev",2); 
    $key = _get_prkey($output); 

    $output =  _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2);
    if ($output !~ /action succeeded/) {
      _pr_abort($rHoH, "Unable to reserve disk <$fadev> with mode <$res_t>", $fadev); 
    } else {
      tstlog(-pass, "Successfully reserved disk <$fadev> with mode <$res_t>");
    }
    
    _pr_say($fac,"scsi pr_read_reservation $fadev", 2);
    _pr_say($fac,"scsi pr_preempt -t $res_t -s $key $fadev", 2); 
    if ($output !~ /action succeeded/) {
      _pr_abort($rHoH, "Unable to pr_preempt <$fadev> in mode <$res_t>", $fadev); 
    } else {
      tstlog(-pass, "Successfully reserved disk <$fadev> with mode <$res_t>");
    }
    
    # -----------------------------------------------------------
    # verify the result of preemption by checking the key and its 
    # reservation type
    # -----------------------------------------------------------
    _pr_disk_verify_resv($rHoH, $fac, $fadev, $fakey, $res_t); 

    # -----------------------------------------
    # verify access permission after preemption 
    # -----------------------------------------
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "ope"); 
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "ppe"); 

    # -------------------------------    
    # Read keys from the second filer
    # -------------------------------
    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);
    _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2);

    # -------------------------------------
    # filerB should not be able to clear PR
    # -------------------------------------
    $output = _pr_say($fbc, "scsi pr_clear $fbdev", 0); 
    if ($output !~ /reservation conflict/) {
	_pr_abort($rHoH, "filerB was able to clear PR; however, it should NOT");
	return 0; #failure
    }
    # --------------------------------------
    # only filerA should be able to clear PR
    # --------------------------------------
    _pr_say($fac,"scsi pr_clear $fadev", 2); 
    return 1; #success
} #_pr_preempt_write_exclusive_local

# -----------------------------------------------
# _pr_preempt_exclusive_access_local
# $fac -- connection object
# $fbc -- filerb connection object
# $fadev  -- filera's device name
# $fbdev  -- filerb's device name
# $fakey  -- filera key
# $fbkey  -- filerB key
# $res_t  -- reservation type 
# ------------------------------------------------
sub _pr_preempt_exclusive_access_local($$$$$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey, $res_t) = @_; 
    my $key; 
    
    _pr_debug("\nPR_PREEMPT_EXCLUSIVE_ACCESS_LOCAL:: preemption of exclusive access on filera\n"); 
    
    _pr_say($fac,"scsi pr_register $fadev", 2); 
    my $output = _pr_say($fac,"scsi pr_read_keys $fadev", 2); 
    $key = _get_prkey($output); 

    _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2);
    _pr_say($fac,"scsi pr_read_reservation  $fadev", 2);
    _pr_say($fac,"scsi pr_preempt -t $res_t -s $key $fadev", 2);

    # ------------------------------------------------------------
    # verify the result of preemption by checking the key and its 
    # reservation type
    # ------------------------------------------------------------
    _pr_disk_verify_resv($rHoH, $fac, $fadev, $fakey, $res_t); 

    # -----------------------------------------
    # verify access permission after preemption 
    # -----------------------------------------
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "ope"); 
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "ppe"); 

    # Read keys from the second filer
    # 
    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);   
    _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2);
 
    # -------------------------------------
    # filerB should not be able to clear PR
    # -------------------------------------
    $output = _pr_say($fbc, "scsi pr_clear $fbdev", 0); 
    if ($output !~ /reservation conflict/) {
	_pr_abort($rHoH, "filerB was able to clear PR; however, it should NOT");
	return 0; #failure
    }

    # --------------------------------------
    # only filerA should be able to clear PR
    # --------------------------------------
    _pr_say($fac,"scsi pr_clear $fadev", 2);   
    return 1; #success    
} #_pr_preempt_exclusive_access_local

# -----------------------------------------------
# _pr_run_abnormal_tests
# $rHoH -- reference of the PR hash table
# $fac -- connection object
# $fbc -- filerb connection object
# $fadev  -- filera's device name
# $fbdev  -- filerb's device name
# ------------------------------------------------
sub _pr_run_abnormal_tests($$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev) = @_; 
    my $remote_key = "0x1111111111"; 
    
    _pr_debug("\nPR_RUN_ABNORMAL_TESTS\n"); 
    
    # register a remote key
    _pr_say($fac,"scsi pr_register -s $remote_key $fadev", 2); 

    # verify the remote key register is fine
    my $output = _pr_say($fac,"scsi pr_read_keys $fadev", 2);
    if ($output =~ /$remote_key/) {
	tstlog(-pass, "Successfully register via a FAKE key <$remote_key>"); 
    } else {
	_pr_abort($rHoH, "Successfully register via a FAKE key <$remote_key>"); 
    }

    # make a reservation under the remote key
    _pr_say($fac,"scsi pr_reserve -k $remote_key -t we $fadev", 2);
    $output = _pr_say($fac,"scsi pr_read_reservation $fadev", 2);
    if ($output =~ /$remote_key/ && $output =~ /type: 0x1/) {
	tstlog(-pass, "Successfully reserve using we type under remote key <$remote_key>"); 
    } else {
	_pr_abort($rHoH, "Failed to reserve under we type using remote key <$remote_key>"); 
    }

    # --------------------------------------------------    
    # make a reservation using a wrong reservation type
    # --------------------------------------------------
    my $dummy_type = "foo"; 
    $output = _pr_say($fac,"scsi pr_reserve -s $remote_key -t $dummy_type $fadev", 0);
    if ($output =~ /Invalid option/) {
	tstlog(-pass, "Not supposed to register with a bad type <$dummy_type>"); 
    } else {
	_pr_abort($rHoH, "Reservation accepts invalid type <$dummy_type>"); 
    }

    # --------------------------------------
    # let clear the invalid reservation type
    # --------------------------------------
    $output = _pr_say($fac," scsi pr_clear -k $remote_key $fadev", 0);
    if ($output =~ /action succeeded/) {
	tstlog(-pass, "Successfully cleared resv <$remote_key");
    } else {
	$output = _pr_say($fac," scsi pr_clear -k $remote_key $fadev", 0);
    }
    return 1; #success    
} #_pr_run_abnormal_tests

# -----------------------------------------------
# _pr_preempt_exclusive_access
# $fac -- connection object
# $fbc -- filerb connection object
# $fadev  -- filera's device name
# $fbdev  -- filerb's device name
# $fakey  -- filerA key
# $fbkey  -- filerB key
# $res_t  -- reservation type 
# ------------------------------------------------
sub _pr_preempt_exclusive_access($$$$$$$$) {
    my ($rHoH, $fac, $fbc, $fadev, $fbdev, $fakey, $fbkey, $res_t) = @_; 
    my $key; 
    
    _pr_debug("\nPR_EXCLUSIVE_ACCESS_LOCAL: preemption exclusive access on filera\n"); 
    _pr_say($fac,"scsi pr_register $fadev", 2); 
    my $output = _pr_say($fac,"scsi pr_read_keys $fadev", 2); 
    $key = _get_prkey($output); 
    
    # Read keys from the second filer
    # 
    _pr_say($fbc,"scsi pr_register $fbdev", 2);   
    _pr_say($fac,"scsi pr_reserve -t $res_t $fadev", 2);

    # verify key and the reservation type
    _pr_disk_verify_resv($rHoH, $fac, $fadev, $fakey, $res_t); 

    # ------------------------------------
    # verify access from owner and partner
    # ------------------------------------
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "owner"); 
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "partner");
    
    _pr_say($fbc,"scsi dump_lba -l 0 -n 1 -o /etc/dump_lba.log $fbdev", 2);
    _pr_say($fbc,"scsi pr_read_reservation $fbdev", 2);
    _pr_say($fbc,"scsi pr_register $fbdev", 2);   
    _pr_say($fbc,"scsi pr_read_keys $fbdev", 2);
    
    # -----------------------------------------------------    
    # verify that filerB should have full read/write access
    # -----------------------------------------------------
    _pr_disk_verify_rw($rHoH, $fbc, $fbdev, $res_t, "ope"); 
    _pr_disk_verify_rw($rHoH, $fac, $fadev, $res_t, "ppe");
    
    _pr_say($fbc,"scsi dump_lba -l 0 -n 1 -o /etc/dump_lba.log $fbdev", 2);
    _pr_say($fbc,"scsi pr_clear $fbdev", 2);
    return 1; #success
} #_pr_preempt_exclusive_access
    
=head2 pr_init    

pr_init($$$$)
The routine carries out the initialization of the test 
in the following steps: 

    * starts the filers online if they are offline 
    * returns with a CONFIG if the kernel is not a debug kernel 
    * determines whether or not software-based disk ownership is on or off
    * validates the specified list of disks 
    * determines the PR key based on System ID. 
    * build a list of disks on both filers 

The assumption here is both filerA & filerB must be specified.    

=over

=item Arguments

=over

=item $rHoH

(Required) : specify the table of information passing by reference

=item $filera

(Required) : specify the filername of the FILERA

=item $filerb

(Required) : specify the filername of the FILERB

=back

=item Returns

None 

=cut 
sub pr_init(\%) {
    my ($rHoH) = @_; 

    my ($filera, $filerb, $ffiler); 

    # ----------------------------    
    # load the PR reservation type
    # ----------------------------
#    %{$rHoH->{RESV_TYPE}} = ("we" , "0x1", "ea" , "0x3","wero" , "0x5", "earo" , "0x6", "wear" , "0x7", "eaar" , "0x8"); 
    
    %{$rHoH->{RESV_TYPE}} =  ("we" , "0x1", "ea" , "0x3","wero" , "0x5", "earo" , "0x6");
    
    # ----------------------
    # Verify the CLIENT node
    # ----------------------
    my $nhost = hostname(); 
    print "hostname is <$nhost> and client = <$CLIENT>\n"; 
    if ($CLIENT !~ /$nhost/) {
      tstlog -comment, "The specified client is <$CLIENT> which does not match with the client where to launch your <$nhost>"; 
      $CLIENT = $nhost; 
    }
    
    # ---------------------------------------
    # and get the current message indexes 
    # ---------------------------------------
    #$rHoH->{FA}->{MLOG_INDEX} = _get_msg_current_index($FILERA); 
    #$rHoH->{FB}->{MLOG_INDEX} = _get_msg_current_index($FILERB); 
    
    # ----------------------------------------
    # Start the filers if they are not running
    # ----------------------------------------
#    my $fo = host($filer);
#    my $fc = connect("#console");
#    my $status = boot($fc);
    
    my $fa = host($FILERA);
    $filera =  $fa->hostname();
    my $farsh = connect("#rsh");
    $rHoH->{FA}{FCO} = $farsh;
    $rHoH->{FA}{FNAME} = $filera;
    $rHoH->{FA}{MLOG_INDEX} = 0; 

    my $fb = host($FILERB); 
    my $fbrsh = connect("#rsh"); 
    $filerb = $fb->hostname(); 
    $rHoH->{FB}{FCO} = $fbrsh;
    $rHoH->{FB}{FNAME} = $filerb;
    $rHoH->{FB}{MLOG_INDEX} = 0;

    # ------------------------
    # enable test PR test mode
    # ------------------------
    _pr_mode_enable($farsh, $fbrsh);

    # ---------------------- 
    # or just a single FILER
    # ----------------------
    my $ff = host($FILERA); 
    my $ffrsh = connect("#rsh"); 
    $ffiler = $ff->hostname(); 
    $rHoH->{FF}{FCO} = $ffrsh;
    $rHoH->{FF}{FNAME} = $ffiler; 
    $rHoH->{FF}{MLOG_INDEX} = 0;
 
    # ---------------------------------------
    # If the cluster is currently taking over
    # then try to giveback. If it still failed
    # then we need to exit the test with a config
    # problem in the cluster. 
    # ---------------------------------------
    my $max_cnt = 20; 
    my $cnt = 0; 
    if (get_cf_state($filera) =~ /TAKEOVER/ || get_cf_state($filera) =~ /DEAD/) {
      cf_giveback($filera);
      # wait until FILERB is up
      while ($cnt++ < $max_cnt) {
	if (get_cf_state($filerb) eq "UP") {
	  last; 
	}
	sleep(10); 
      }
    }
    
#    if (get_cf_state($FILERB) ne "UP") {
#      my $state = get_cf_state($FILERB); 
#      tstlog-comment, "Filer <$FILERB> state is <$state>"; 
#      tstlog -config, "Unable to GIVEBACK the filer <$FILERB>"; 
#    }

    $cnt = 0; 
    if (get_cf_state($filerb) =~ /TAKEOVER/ || get_cf_state($filera) =~ /DEAD/) {
      cf_giveback($filerb);
      # wait until FILERB is up
      while ($cnt++ < $max_cnt) {
	if (get_cf_state($filera) eq "UP") {
	  last; 
	}
	sleep(10); 
      }
    }

    if (get_cf_state($filera) eq /TAKEOVER/ || 
        get_cf_state($filera) eq /DEAD/ || 
        get_cf_state($filerb) eq /TAKEOVER/ || 
        get_cf_state($filerb) eq /DEAD/) {
      tstlog -config, "Unable to GIVEBACK on the cluster"; 
    }

    # --------------------------------
    # Validate if it is a debug kernel
    # --------------------------------
    my $output = _pr_say($farsh, "version", 2); 
    my @bufstr = split(" ", $output); 
    if ($bufstr[9] !~ /d/) {
	_pr_abort($rHoH, "Must run with the DEBUG kernel instead of $output"); 
	return 0; #failure
    } else {
	tstlog(-comment, "Version=" . $output);
    }
    
    $output = _pr_say($fbrsh, "version", 2); 
    @bufstr = split(" ", $output); 
    if ($bufstr[9] !~ /d/) {
	_pr_abort($rHoH, "Must run with the DEBUG kernel instead of $output"); 
	return 0; #failure
    } else {
	tstlog(-comment, "Version=" . $output);
    }
    
    # --------------------------------------------
    # get the PR key which based on the system Id 
    # which will be converted into hex. 
    # --------------------------------------------
    my ($fakey, $fbkey); 
    my $status = _pr_filer_get_key($rHoH, $farsh, \$fakey, \$fbkey); 
    $rHoH->{FA}->{PRKEY} = $fakey;    
    $status = _pr_filer_get_key($rHoH, $fbrsh, \$fbkey, \$fakey); 
    $rHoH->{FB}->{PRKEY} = $fbkey;    
    
    # ------------------------------------------------------
    # determine if the software based ownership is on or off
    # for both nodes
    # ------------------------------------------------------
    my $aoutput = _pr_say($farsh, "storage show initiators", 0); 
    my $boutput = _pr_say($fbrsh, "storage show initiators", 0); 
    my $sanown_state; 
    if ($aoutput =~ /SANOWN not enabled/ && $boutput =~ /SANOWN not enabled/) {
	$sanown_state = 0; 
	tstlog('-comment', "SANOWN is not enable from both nodes"); 
    } elsif ($aoutput !~ /SANOWN not enabled/ && $boutput !~ /SANOWN not enabled/) {
	tstlog('-comment', "SANOWN is enable");
	$sanown_state = 1; 
    } else {
	_pr_abort($rHoH, "SANOWN is not enable from one but not the other node."); 
	return 0; #failure
    }

    # ---------------------------------------------------------
    # create a table of disks that are broken since 
    # they are not the right candidates for running and PR tests
    # ---------------------------------------------------------
    _pr_get_fail_disks(%$rHoH, $filera, $filerb); 

    # ------------------------------------------------
    # build a list of disks for both filerA and FILERB
    # ------------------------------------------------
    my $bayid = 1;
    $rHoH->{SANOWN_STATE} = $sanown_state;
    _pr_disk_load_info(%$rHoH, $filera, $bayid); 
    _pr_disk_load_info(%$rHoH, $filerb, $bayid); 
    
    return 1; #success
} #pr_init

# ----------------------------------------------------------- 
# the routine creates a situation of disk reservation conflict
# by clearing the reservation and then change the ownership
# of the specified disk to its partner. 
# 
# $fowner - filename of the disk owner
# $fpartner - partner that is going to assume the ownership
# $odisk - the disk see by the owner
# $pdisk - the disk see by the partner 
# ----------------------------------------------------------- 
sub _pr_takeover($$$$) {
    my ($fowner, $fpartner, $odisk, $pdisk) = @_; 
    my $rt = "we"; 
    my $cnt;
    use constant MAX_CNT => 30; 
    my $foc = host($fowner); 
    my $forsh = connect("#rsh");  # owner rsh object
    
    my $fpc = host($fpartner); 
    my $fprsh = connect("#rsh");  # partner rsh object
    my $output; 

    # ----------------------------------------------------
    # clear and then  start a new registration on the disk
    # ----------------------------------------------------
    $output = _pr_say($forsh, "scsi pr_clear $odisk", 2); 
    if ($output !~ /action succeeded/) {
	_pr_say($forsh, "scsi pr_clear $odisk", 2); 
    }

    $output = _pr_say($fprsh, "scsi pr_register $pdisk", 2); 
    if ($output !~ /action succeeded/) {
	_pr_say($fprsh, "scsi pr_register $pdisk", 2); 
    }
    
    # ----------------------------------------------
    # within 30 seconds TAKEOVER should taking place
    # ----------------------------------------------
    $output = _pr_say($fprsh,"scsi pr_reserve -t $rt $pdisk",0 ); 
    if ($output !~ /action succeeded/ ) {
	tstlog -fail, "Unable to reserve disk <$pdisk> with reserve type <$rt> from $fpartner"; 
	return 0; 
    }

    # ----------------------------------------
    # display the latest reservation on disk
    # ----------------------------------------
    _pr_say($fprsh, "scsi pr_read_reservation $pdisk", 2); 

    # ---------------------------------------------------    
    # wait for about 120 seconds which is much more than 
    # the periodic reassertion which is at 30 secs
    # ---------------------------------------------------
    $cnt = 0; 
    while ($cnt++ < MAX_CNT && get_cf_state($fowner) !~ /TAKEOVER/) {
	sleep(10); 
    }
	
    if (get_cf_state($fpartner) !~ /TAKEOVER/) {
	tstlog -fail, "Unable to create a reservation conflict on disk <$odisk>"; 
	return 0; #failure
    }

    # ---------------------------------------------------	
    # display reservation of the disk during the takeover
    # ---------------------------------------------------
    tstlog(-comment, "Display the disk reservation information during takeover"); 
    _pr_disk_display_resv($fprsh, $pdisk); 

    tstlog -comment, "Successfully created a reservation conflict on disk <$odisk>"; 
    
    # giveback now 
    cf_giveback($fpartner); 

    # ---------------------------------
    # wait until the cluster is full UP
    # or ran out of time (300 seconds)
    # ---------------------------------
    $cnt = 0; 
    while (get_cf_state($fowner) !~ "UP" && $cnt++ < 60) {
	tstlog -comment, "waiting for the filer to be UP"; 
	sleep(5); 
    }

    if (get_cf_state($fowner) ne "UP") {
	tstlog -fail, "Unable to bring the cluster UP after 5 minutes"; 
	return 0; 
    }
	
    return 1; #success
} #pr_takeover


sub pr_verify_takeover(\%) {  
    my ($rHoH) = @_; 
  
    my $output; 

    my $farsh = $rHoH->{FA}->{FCO}; 
    my $fbrsh = $rHoH->{FB}->{FCO}; 
    my $fakey = $rHoH->{FA}->{PRKEY}; 
    print "fakey=" . $fakey . "\n"; 
    my $fbkey = $rHoH->{FB}->{PRKEY}; 
    my $filera = $rHoH->{FA}->{FNAME};
    my $filerb = $rHoH->{FB}->{FNAME};

    # ---------------------------------------
    # Enable cluster takeover from both nodes
    # ---------------------------------------
    _pr_say($farsh, "cf enable", 0); 
    _pr_say($fbrsh, "cf enbale", 0); 

    # ------------------------
    # enable test PR test mode
    # ------------------------
    # _pr_mode_enable($farsh, $fbrsh);
    
    # determine how many disks to run the test with
    if ($#{$rHoH->{FA}->{SEL_DISKS}} < 0) {
	_pr_abort($rHoH, "Could not find any selected disk."); 
	return 0; #failure
    }
    
    # --------------------------------------------------
    # run and verify takeover against the selected disks
    # --------------------------------------------------
    for (my $i=0; $i <= $#{$rHoH->{FA}->{SEL_DISKS}}; $i++) {
	my $fad = $rHoH->{FA}->{SEL_DISKS}[$i]; 
	my $fbd = $rHoH->{FB}->{SEL_DISKS}[$i]; 
	
	_pr_debug("** Starting running reservation conflict on disk <$fad> and <$fbd>\n"); 

	# -----------------------------------------------------------------	
	# add the disk to the test volume to make our test more interesting
	# -----------------------------------------------------------------
	# vol_add($rHoH->{FA}->{FCO}, $FILER_VOL, $fad);  # this routine does not work
	_pr_say($rHoH->{FA}->{FCO}, "vol add $FILER_VOL -d $fad", 0); 
	sleep(10); 
	
	#----------------------------------------------------------
	# To create a reservation conflict by changing the ownership
	# from one node to another. 
	#----------------------------------------------------------
	my $disk_owner = $rHoH->{FA}->{DISKS}->{$fad}[4];

	# ---------------------------------------------------
	# determine who the owner is and try to take over the 
	# ownership from the other node. 
	# Due to the timing issue (30 second interval could be
	# a factor here) we need to try this a few times before 
	# we can cause a reservation conflict to happen. 
        #
	# if the disk owner is FILERA then use FILERB to assume 
	# the ownership
	# -----------------------------------------------------
	my $status; 
	if ($disk_owner eq $filera) {

	    my $reserv_type; 
	    my $reserv_key; 

	    # get the reservation information of the disk before taking over
	    _pr_disk_get_resv_type($farsh, $fad, \$reserv_type, \$reserv_key); 

	    # -------------------------------------------------
	    # create a situation of takeing over a disk due to 
	    # changing the ownership of the disk 
	    # -------------------------------------------------
	    $status = _pr_takeover($filera, $filerb, $fad, $fbd); 
	    if ($status eq 0) {
		tstlog -fail, "Unable to create a reservation conflict."; 
		return 0; #failure
	    }

	    # --------------------------------------------------
	    # afer takeover and give back took place
	    # verify the ownership should be restored on the disk
	    # --------------------------------------------------
	    my $reserv_type_after; 
	    my $reserv_key_after; 
	    _pr_disk_get_resv_type($farsh, $fad, \$reserv_type_after, \$reserv_key_after); 
	    _pr_debug("reserv_type: <$reserv_type> reserv_type_after: <$reserv_type_after\n"); 
	    if ($reserv_type ne $reserv_type_after) {
		_pr_abort("the resv type before <$reserv_type> does not match with the resv type after <$reserv_type_after> of disk $fad"); 
	    }
	    _pr_debug("reserv_key: <$reserv_key> reserv_key_after: <$reserv_key_after\n"); 	    
	    if ($reserv_key ne $reserv_key_after) {
		_pr_abort("the resv type before <$reserv_type> does not match with the resv type after <$reserv_type_after> of disk $fad"); 
	    }
	} else {
	    $status = _pr_takeover($filerb, $filera, $fbd, $fad); 
	}
	
	# ---------------------------------------------------------------
	# verify the original ownership shall be reshore on the specified
	# disk
	# ---------------------------------------------------------------
	
    } #for each selected disk
    _pr_report($rHoH); 

    return 1; #success
} # pr_verify_takeover

#=head2 _pr_abort
#
#    _pr_abort(Itable=>$rHoH, Message=>$msg, Disk=>$disk) 
#    
#    the routine handles any fatal or fail conditions that could
#    happen during the test. 
#  
#    The routine displays sysconfig -r and dump the latest content
#    of the message log. 
#    
#    If the disk is specified then displays the current reservation
#    of the specified disk.
#
#=over
#  
#=item None
#
#=over
#    
#=item Returns
#
#None 
#
#=back
#
#=cut 

sub _pr_abort($$;$) {
    my ($rHoH, $msg, $disk) = @_; 
    
    my $fac = $rHoH->{FA}->{FCO}; 
    my $faname = $rHoH->{FA}->{FNAME}; 
    my $fbc = $rHoH->{FB}->{FCO};
    my $fbname = $rHoH->{FB}->{FNAME}; 
    if ($VERBOSE) {
	tstlog(-comment, "entering _pr_abort routine"); 
      #my $output = _pr_say($fac, "version; sysconfig -r", 0);
      #$output = _pr_say($fbc, "version; sysconfig -r", 0); 
	
      #tstlog(-comment, "run disk show -v from filer <$faname>"); 
      #_pr_say($fac, "disk show -v"); 
#	_pr_say($fac, "vol status -f"); 
#	tstlog(-comment, "run disk show -v from filer <$fbname>"); 
#	_pr_say($fbc, "disk show -v"); 
#	_pr_say($fbc, "vol status -f"); 
    }
    
    # --------------------	
    # disable pr test mode
    # --------------------
    _pr_mode_disable($fac, $fbc); 

    # ---------------------    
    # dump the message log 
    # ---------------------
#    if ($VERBOSE) {
#      _dump_msg_log($rHoH); 
#    }

    # ----------------------------
    # destroy the load for testing
    # ----------------------------
    pr_load_destroy($rHoH); 

    # unfail the disk
    _disk_unfail($fac, $disk); 
    _disk_unfail($fbc, $disk); 

    # --------------------------
    # print the message and quit
    # --------------------------
    if (defined $disk) {
	if (defined $rHoH->{FA}->{DISKS}->{$disk}) {
	    my $frsh = $rHoH->{FA}->{FCO};
	    _pr_say($frsh, "scsi pr_read_keys $disk; scsi pr_read_reservation $disk",0); 
	} else {
	    tstlog('-fail', "Found a bad disk <$disk>"); 
	}
	
	if (defined $rHoH->{FB}->{DISKS}->{$disk}) {
	    my $frsh = $rHoH->{FB}->{FCO};
	    _pr_say($frsh, "scsi pr_read_keys $disk; scsi pr_read_reservation $disk",2);
	}
    }
    tstlog('-fatal', $msg); 
} # _pr_abort

=head2 pr_validate_parameters

The routine validates a number of parameters of the PR test scripts.
The criteria for selecting disks are based on the following assumption:
1. the drive model matches the specify model number
2. the drive should be one of the spares. If there is no more spare
   then returns a CONFIG problem

  FILERA_DISKS - verifies that the specified disks are valid. If there is
an error returns a CONFIG. 
  FILERB_DISKS - verifies that the specified disks are valid. If there is
an error returns a CONFIG. 
  ITERATIONS - should not be zero
  DISKS_MODEL - if the specified models don't match, returns a CONFIG error.

=over

=item Arguments

=over

=item $fahash_r

(Required) : Reference of the filerA hash table

=item $fbhash_r

(Required) :  Reference of the filerB hashtable 

=item $fadisks 

(Required) :  List of disks specified for the filerA 

=item $fbdisks

(Required) : List of disks specified for the filerB

=item $disks_model

(Required) : Specify a list of disk mode type separated by a blank space

=back

=item Returns

None 

=cut 
sub pr_validate_parameters(\%$$$$) {
    my ($rHoH, $par_fadisks, $par_fbdisks, $dmodel, $disks_action) = @_;
  
    my ($fadisks, $fbdisks, @fa_sel_disks, $fb_sel_disks); 
    my $disks_model = ""; 
    
    foreach my $dm (split(/ +/, $dmodel)) {
	if ($dm =~ /seagate/i) {
	    $disks_model = $disks_model . " _S"; 
	} elsif ($dm =~ /hitachi/i) {
	    $disks_model = $disks_model . " _H"; 
	} elsif ($dm =~ /maxtor/i) {
	    $disks_model = $disks_model . " _M"; 
	} else {
	    $disks_model = $disks_model . " $dm"; 
	} 
    } #foreach 

    # ---------------------------------------------------------
    # load the RaidInfo object so we could get a list of spares
    # ---------------------------------------------------------
    my $fa_obj = new RaidInfo($rHoH->{FA}->{FCO}); 
    my $fb_obj = new RaidInfo($rHoH->{FB}->{FCO}); 
    my %fa_spares = $fa_obj->spare_disks(); 
    my %fb_spares = $fb_obj->spare_disks(); 

    # --------------------------
    # process ALL_SPARES options
    # --------------------------
    if ($par_fadisks eq "ALL_SPARES") {
        _pr_debug("Filera selected ALL_SPARES\n"); 
        $fadisks = ""; 
        foreach my $disk (keys %fa_spares) {
           $fadisks = $fadisks . $disk . " "; 
        }
        _pr_debug("Filera selected ALL_SPARES <$fadisks>\n"); 
    } 
    else {
        $fadisks = $par_fadisks; 
    }
    # ----------------------------------------------
    # validate a list of specified disks from filerA
    # and then from filerB. 
    # The user choice could be spare or non-spare
    # If they are not spares then we won't be able to
    # create a test volume to create any I/O situation
    # ------------------------------------------------
    if ($fadisks ne "") {
	foreach my $disk (split(/\s+/, $fadisks)) {
	    if ( ! defined $rHoH->{FA}->{DISKS}->{$disk}) {
		tstlog(-config, "Could not find the specified disk name: <$disk> on the <$rHoH->{FA}->{FNAME}>"); 
		next; 
	    }
	    
	    # ---------------------------------------------------------
	    # skip the disk with BAYID is (<= 1) on a non SANOWN system
	    # ---------------------------------------------------------
	    if ($rHoH->{FA}->{DISKS}->{$disk}[1] <= 1) {
		tstlog(-comment, "WARNING: skip disk <$disk> since its BAYID is 1 or 0"); 
		next; 
	    }

	    # -------------------------------------------
	    # we try to lookup for the disk serial number
	    # -------------------------------------------
	    my $sel_disk_serial_num = $rHoH->{FA}->{DISKS}->{$disk}[2]; 
	    _pr_debug("disk <$disk> serial no: <$sel_disk_serial_num>\n");

	    # ---------------------------------------------------
	    # determine the equivalent name of the disk on filerB
	    # ---------------------------------------------------
	    foreach my $bdisk (keys %{$rHoH->{FB}->{DISKS}}) {

		if (! defined($rHoH->{FB}->{DISKS}->{$bdisk}[2])) {
		    next; 
		}

		# -----------------------------------------------------
		# add the selected disk to the both filerA & FilerB
		# -----------------------------------------------------
		if ($rHoH->{FB}->{DISKS}->{$bdisk}[2] eq $sel_disk_serial_num) {
		    push(@{$rHoH->{FA}->{SEL_DISKS}}, $disk);
                    push(@{$rHoH->{FB}->{SEL_DISKS}}, $bdisk); 
		    last;
		}
	    } #foreach 
	} 
    } #if the filera disk list is NOT empty
    
    # ----------------------------------------------
    # validate a list of specified disks from FILERB
    # ----------------------------------------------
    # --------------------------
    # process ALL_SPARES options
    # --------------------------
    if ($par_fbdisks eq "ALL_SPARES") {
        _pr_debug("Filerb selected ALL_SPARES\n"); 
        $fbdisks = ""; 
        foreach my $disk (keys %fb_spares) {
           $fbdisks = $fbdisks . " " . $disk; 
        }
    } 
    else {
        $fbdisks = $par_fbdisks; 
    }

    if ($fbdisks ne "") {
	foreach my $disk (split(/\s+/, $fbdisks)) {
	    if (! $rHoH->{FB}->{DISKS}->{$disk}) {
		tstlog(-config, "Could not find a specified disk name: <$disk> from the <filerB>"); 
		next; 
	    }
	    
	    # ---------------------------------------------------------
	    # skip the disk with BAYID is (<= 1) on a non SANOWN system
	    # ---------------------------------------------------------
	    if ($rHoH->{FB}->{DISKS}->{$disk}[1] <= 1) {
		tstlog(-comment, "WARNING: skip disk <$disk> since its BAYID is 1 or 0"); 
		next; 
	    }

	    # --------------------------	  
	    # get the disk serial number
	    # --------------------------
	    my $sel_disk_serial_num = $rHoH->{FB}->{DISKS}->{$disk}[2]; 
	    _pr_debug("disk <$disk> serial no: <$sel_disk_serial_num>\n");
	    
	    # -----------------------------------------------
	    # determine the equivalent name of disk on filerA
	    # -----------------------------------------------
	    foreach my $adisk (keys %{$rHoH->{FA}->{DISKS}}) {
		if (! defined($rHoH->{FA}->{DISKS}->{$adisk}[2])) {
		    next; 
		}
		if ($rHoH->{FA}->{DISKS}->{$adisk}[2] eq $sel_disk_serial_num) {
		    _pr_debug("found the corresponding name of <$disk> is <$adisk>\n"); 
		    push(@{$rHoH->{FB}->{SEL_DISKS}}, $disk); 
		    push(@{$rHoH->{FA}->{SEL_DISKS}}, $adisk); 
		    last; 
		}
	    } #foreach 
	}
    } #if the filerb disk list is not empty 

  
    # ---------------------------------------------------------
    # Without any input from the user, the script shall select
    # one of each disk model and then make sure they are spare
    # . If no spare could be found then use the preselected one
    # which is one of the disk from the MODEL list. 
    # ---------------------------------------------------------
    if ($disks_model eq "" && $fadisks eq "" && $fbdisks eq "") {
	
	# ----------------------------------------    
	# select a disk that should be seen from 
	# both filers and it should not be in FAILED state
	# in spare state with the right model number 
	# ----------------------------------------
	my $current_model; 
	foreach my $model (keys %{$rHoH->{MODELS}}) {
	    my $sel_disk = ""; 
	    my $model_disk = $rHoH->{MODELS}->{$model}[0]; 
	    
	    # ---------------------------------------------------
	    # if the model disk does not belong to the spare list
	    # then try to locate a spare to replace it
	    # ----------------------------------------------------
	    if (!defined $fa_spares{$model_disk} && !defined $fb_spares{$model_disk}) {
		_pr_debug("disk <$model_disk> is not a spare\n");
		foreach my $ldisk (keys %fa_spares) {
		    if ($rHoH->{FA}->{DISKS}->{$ldisk}[3] eq $model) {
			$sel_disk = $ldisk;
			_pr_debug("found a spare with the right model <$sel_disk>"); 
			last; 
		    }
		} # each spare

		# it must be in filerB 
		if ($sel_disk eq "") {
		    foreach my $ldisk (keys %fb_spares ) {
_pr_debug("Model: $rHoH->{FB}->{DISKS}->{$ldisk}[3] == $model\n"); 
			if ($rHoH->{FB}->{DISKS}->{$ldisk}[3] eq $model) {
			    $sel_disk = $ldisk;
			    _pr_debug("found a spare with the right model <$sel_disk> from filer <$rHoH->{FB}->{FNAME}>\n"); 
			    last; 
			}
		    } # each spare
		} #if 

		# -------------------------------------
		# we CANNOT continue with PR testing since
		# the test disk must be one of the spares
		# -------------------------------------
		if ($sel_disk eq "") {
		    $sel_disk = $model_disk; 
		    _pr_debug("the selected disk <$sel_disk> is not spare\n"); 
		    
		}
	    } else {
		# at this point the model disk is one of the spares
		$sel_disk = $model_disk; 
		_pr_debug("the model disk <$sel_disk> is also a spare\n"); 
	    }

	    
	    # -----------------------------------------------------      
	    # if the disk belongs to filerA then find its corresponding
	    # from filerB and save both
	    # -----------------------------------------------------
	    if (defined $rHoH->{FA}->{DISKS}->{$sel_disk}) {
		_pr_debug("selected a disk <$sel_disk>\n"); 

		push(@{$rHoH->{FA}->{SEL_DISKS}}, $sel_disk); 

		# find the corresponding disk from filerB
		foreach my $diskb (reverse keys %{$rHoH->{FB}->{DISKS}}) {
		    
		    #find a match serial number
		    if ($rHoH->{FB}->{DISKS}->{$diskb}[2] eq $rHoH->{FA}->{DISKS}->{$sel_disk}[2] ) {
			push(@{$rHoH->{FB}->{SEL_DISKS}}, $diskb); 
			last; 
		    }  
		}
		next; 
	    }
	    
	    # -----------------------------------------------------      
	    # At this point the disk must belong to filerB
	    # -----------------------------------------------------
	    if (defined $rHoH->{FB}->{DISKS}->{$sel_disk}) {
		push(@{$rHoH->{FB}->{SEL_DISKS}}, $sel_disk); 

		# find the corresponding disk from filerA
		foreach my $diska (reverse keys %{$rHoH->{FA}->{DISKS}}) {
		  #find a match serial number
		    if ($rHoH->{FA}->{DISKS}->{$diska}[2] eq $rHoH->{FB}->{DISKS}->{$sel_disk}[2]) {
		      push(@{$rHoH->{FA}->{SEL_DISKS}} , $diska); 
		      last; 
		  }
		}
		next; 
	    }
	    
	    # -----------------------------------------------
	    # At this point the disk does not belong to any node
	    # -----------------------------------------------
	    _pr_abort($rHoH, "Unable to identify disk <$sel_disk>", $sel_disk); 
	    return 0; #failure
	    
	}#foreach 
	return 1; #success
	# select one of each disk models of filerB
    } # select disks by the script
    
    
    # ---------------------------------------
    # to select all disks (DISKS_ACTION=all)
    # and check for the specified disk models
    # Let try to select only those that are spares
    # ---------------------------------------
    _pr_debug("** before disks_action\n"); 
    
    if ($disks_action =~ /all/i) {
	
	# clear both selected disks from both filerA &filerB
	$rHoH->{FA}->{SEL_DISKS} = []; 
	$rHoH->{FB}->{SEL_DISKS} = []; 

	# skip non-disk entries
	# 
	foreach my $diska (reverse keys %{$rHoH->{FA}->{DISKS}}) {
	    # ---------------------
	    # skip non-disk entries
	    # ---------------------
	    if ($diska =~ '=') { 
		next; 
	    }
	    my $sel_disk_serial = $rHoH->{FA}->{DISKS}->{$diska}[2]; 
            my $diskb = "";
            foreach my $dk (keys %{$rHoH->{FB}->{DISKS}}) {
               if ($rHoH->{FB}->{DISKS}->{$dk}[2] eq $sel_disk_serial) {
                  $diskb = $dk;
                  last;
                }
            }

	    # ---------------------------------------------------------
	    # skip the disk with BAYID is (<= 1) on a non SANOWN system
            # and non-spares
	    # ---------------------------------------------------------
	    if ($rHoH->{FA}->{DISKS}->{$diska}[1] <= 1) {
		tstlog(-comment, "WARNING: skip disk <$diska> since its BAYID <= 1"); 
		next; 
	    }
	    if ($rHoH->{FA}->{DISKS}->{$diska}[9] ne "SPARE") {
		tstlog(-comment, "WARNING: skip <$diska> it is not a SPARE");
		next; 
	    }

	    if ($disks_model ne "") {
		foreach my $model (split(/ +/, $disks_model)) {
		    if ($rHoH->{FA}->{DISKS}->{$diska}[3] =~ $model) {
			push(@{$rHoH->{FA}->{SEL_DISKS}}, $diska);  
			# -------------------------------------------------
			# we need to find the corresponding disk for FILERB
			# by checking against its serial number 
			# -------------------------------------------------
			foreach my $diskb (keys %{$rHoH->{FB}->{DISKS}}) {
			    if ($rHoH->{FB}->{DISKS}->{$diskb}[2] eq $rHoH->{FA}->{DISKS}->{$diska}[2]) {
				push(@{$rHoH->{FB}->{SEL_DISKS}}, $diskb); 
			    }
			}
		    }
		} #foreach 
	    } else {
		push(@{$rHoH->{FA}->{SEL_DISKS}}, $diska); 
		# -------------------------------------------------
		# we need to find the corresponding disk for FILERB
		# by checking against its serial number 
		# -------------------------------------------------
		foreach my $diskb (keys %{$rHoH->{FB}->{DISKS}}) {
		    if ($rHoH->{FB}->{DISKS}->{$diskb}[2] eq $rHoH->{FA}->{DISKS}->{$diska}[2]) {
			push(@{$rHoH->{FB}->{SEL_DISKS}}, $diskb);
		    }
		}
	    }
	} #foreach
	return 1; #success
    } #if

    # --------------------------------    
    # select the specified disks model
    # only use spare disk for PR testing
    # --------------------------------
    foreach my $dsm (split(/ +/, $disks_model)) {
	foreach my $da (reverse keys %{$rHoH->{FA}->{DISKS}}) {

	    # ---------------------------------------------------------
	    # skip the disk with BAYID is (<= 1) on a non SANOWN system
	    # ---------------------------------------------------------
	    if ($rHoH->{FA}->{DISKS}->{$da}[1] <= 1) {
		tstlog(-comment, "WARNING: skip disk <$da> since its BAYID is 1 or 0"); 
		next; 
	    }
	    # -----------------------------
	    # skip non-spares disks
	    # -----------------------------
	    if (!defined($fa_spares{$da})) {
	      _pr_debug("skipping non-spare disk <$da>"); 
	      next; 
	    }

	    if ($rHoH->{FA}->{DISKS}->{$da}[3] =~ $dsm) {
		push(@{$rHoH->{FA}->{SEL_DISKS}}, $da);
		# -------------------------------------------------
		# we need to find the corresponding disk for FILERB
		# by checking against its serial number 
		# -------------------------------------------------
		foreach my $db (keys %{$rHoH->{FB}->{DISKS}}) {
		    if ($rHoH->{FB}->{DISKS}->{$db}[2] eq $rHoH->{FA}->{DISKS}->{$da}[2]) {
			push(@{$rHoH->{FB}->{SEL_DISKS}}, $db);
		    }
		}
		last; 
	    }
	} #foreach 
    } #foreach 
    return 1; #success
} # pr_validate_parameters

=head1 SEE ALSO

  Step::sutils library for additional common routines to retrieve
  disks from both filers. 

=head1 BUGS

    None

=head1 STATUS

    Public

=head1 ATTRIBUTES

    ONTAP_COMMANDS = scsi pr

    CLIENT_OS = Solaris, Windows

    ONTAP_VERSIONS = Main, Anchorsteam, Scrimshaw

=head1 AUTHOR

    tomdoan@netapp.com
    $Id: //depot/prod/test/Rsierranevada/lib/Storage/persistent_reservation.pm#2 $
    Last revised $Date: 2013/02/28 $

=head1 COPYRIGHT

    Copyright 2005, Network Appliance Inc., All Rights Reserved.

=cut

# end Step::persistent_reservation
#
1; 
