## @summary Persistent.pm - library module for SCSI Persistent_Reservation commands
##
## @status TESTME
##
## @author vijaykumar.javaraiah@netapp.com
##
## 
=head1 NAME

Persistent.pm

=head1 SYNOPSIS


       	use Persistent.pm qw(scsi_pr);

=head1 DESCRIPTION
	
	This is a library module for various SCSI Persistent Reservation commands

=head1 ROUTINES


=cut



#use strict; 
#use warnings; 

use Exporter;
our (@EXPORT_OK, @ISA); 
@ISA = ('Exporter'); 



@EXPORT_OK = qw(scsi_pr scsi_pr_read_reservation get_partner_disk_name storage_release_disks get_SYS_ID); 

use Tharn qw(say script_result host logcomment logresult param);
use Storage::SASUtils qw(filer_cmd);
use Utils qw(make_conn);
use Data::Dumper;
use boot;

use Carp qw(confess croak);
use Params::Validate qw(validate validate_pos ARRAYREF OBJECT SCALAR HASHREF );

our $MAINT_PROMPT = "\\*\>";


BEGIN {
Params::Validate::validation_options(
    on_fail => sub {my @msgs = @_; chomp(@msgs);
        Carp::confess(Tharn::script_result(@msgs))}
   );
}

=head5 scsi_pr_read_reservation

        my %output = scsi_pr_read_reservation($host,$adapter_port,$device_name,$filermode);

This routine returns the hash with the following key value pairs for a given disk from the specified path.

		'generation'=>"$GEN",
                'resv_key'=>"$KEY",
                'additional'=>"$ADDLEN",
                'extent_length'=>"$EXTENTLEN",
                'type'=>"$TYPE"

=cut


#scsi_pr_read_reservation
sub scsi_pr_read_reservation {



	my (%opts) = validate(@_, {
        		                host => { type=>OBJECT, isa=>'Hostrec' },
                		        timeout => { type=>SCALAR, default=>300, optional=>1 },
                        		filermode => { type=>SCALAR, default=>"normal", optional=>1 },
	                        	conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
					adapter_name => {type=>SCALAR},
					disk_name => {type=>SCALAR},
		                        fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
	           		  });
                          
	if ($opts{filermode} ne "normal")
	{ 
	       $opts{conntype} = "#console";
	}

		    my $host = $opts{host};
		    my $disk_name = $opts{disk_name};
		    my $conntype = $opts{conntype};
	 	    my $timeout = $opts{timeout};
		    my $conn = make_conn($host, $conntype);
		    my $filermode = $opts{filermode};
		    my $adapter_name = $opts {adapter_name};
		    my $output="";
	
	 if ($filermode eq "MAINT")
	 {
		 my $cmd = "scsi pr_read_reservation -h $adapter_name $disk_name";
		 do{
		 (undef, undef, $output) = say($conn,"$cmd",-prompt,"$MAINT_PROMPT","-timeout",$opts{timeout});
                  } while ($output eq "error: check condition  key: 0x6  code: 0x2a  qual: 0x1");

		 
	 }
	 else
	 {	
		do {
		 my $cmd = "priv set test;scsi pr_read_reservation -h $adapter_name $disk_name";	
                 #(undef, undef, $output) = say($conn,"$cmd","-timeout",$opts{timeout});
		 $output = filer_cmd(host => $conn->hostp(), cmd => $cmd, timeout=>$opts{timeout});
		   } while ($output eq "error: check condition  key: 0x6  code: 0x2a  qual: 0x1");
	 }

 	
	my @output = split (/\r*\n/,$output);


#To get generation count

my @gen = grep /generation/,@output;
my $gen="@gen";
my @generation= split(/:/,$gen);
my $GEN=$generation[1];

#To get reservation key

my @key = grep /resv_key/,@output;
my $key="@key";
my @reservationkey= split(/:/,$key);
my $KEY=$reservationkey[1];

#To get additional length

my @len = grep /additional/,@output;
my $len="@len";
my @additionallen= split(/:/,$len);
my $additionallen=$additionallen[1];
   $additionallen=~s/^\s//;
my $ADDLEN=$additionallen;


#To get extent_lenght

@len = grep /extent_length/,@output;
$len="@len";
my @extentlen= split(/:/,$len);
my $EXTENTLEN=$extentlen[1];

#To get type

my @type = grep /type/,@output;
my $type="@type";
@type= split(/:/,$type);
my $TYPE=$type[1];

my %RESULT = (  'generation'=>"$GEN",
		'resv_key'=>"$KEY",
		'additional'=>"$ADDLEN",
		'extent_length'=>"$EXTENTLEN",
		'type'=>"$TYPE" );





return %RESULT;
 


}#scsi_pr_read_reservation


=head8 scsi_pr 

		For establishing the reservation.
		$Output=scsi_pr(host=>$host,timeout=>$TIMEOUT,adapter_name=>$HBA_port,resev_key =>$key,resev_type=>"we",disk_name=>$disk,command=>"reserve",fatal_on_failure=>$fatal);

This routine accepts the following inputs and forms the commands based on the inputs and executes them.

		host => hostrec of the filer,
                timeout => Value in ms.
                filermode => Filer mode default=>"normal". for MAINT mode its "MAINT".
                conntype => Filer connection tyupe default=>"#rsh".
                adapter_name => "<2a>".adapter name required for the command.
	        resev_key => Reservation key value.
                resev_type => Reservation type value.
                disk_name => Disk name.
                new_resev_key => New Reservation Key value.
                command => "reserve" or "release" or "preempt" or "clear".based on the commnd it accepts the necessary inputs for the command and executes.
                fatal_on_failure => if set to "yes" then the script terminates if it encounters any command failures.

=cut


#scsi_pr

sub scsi_pr_read_keys
 {

        my (%opts) = validate(@_, {
                                        host => { type=>OBJECT, isa=>'Hostrec' },
					disk_name => {type=>SCALAR,optional=>1},
					timeout => { type=>SCALAR, default=>300, optional=>1 },
                                        filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                                        adapter_name => {type=>SCALAR,optional=>1},
					fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},

				  });





                        if ($opts{filermode} ne "normal")
                        {
                               $opts{conntype} = "#console";
                        }


                    my $host = $opts{host};
                    my $disk_name = $opts{disk_name};
                    my $conntype = $opts{conntype};
                    my $timeout = $opts{timeout};
		    my $conn = make_conn($host, $conntype);
                    my $filermode = $opts{filermode};
                    my $adapter_name = $opts {adapter_name};
		    my $fatal_on_failure =$opts {fatal_on_failure};
		    my $cmd="scsi pr_read_keys -h $adapter_name $disk_name";		


	if ($filermode eq "MAINT")
         {
                 do{
                         (undef, undef, $output) = say($conn,"$cmd",-prompt,"$MAINT_PROMPT","-timeout",$opts{timeout});
                    }while ($output eq "error: check condition  key: 0x6  code: 0x2a  qual: 0x1");


                 if ($fatal_on_failure eq "yes")
                 {
                         if ($output =~ /usage:/)
                         {
                                logresult('fatal',"Command scsi_pr $cmd Failed to execute");
                         }
                         else
                         {
                               #return $output;
                         }
                }
                else
                {
                        if ($output =~ /usage:/)
                         {
                                logcomment("Command scsi_pr $cmd Failed to execute");
                         }
                         else
                         {
                                #return $output;
                         }

                }

         }
         else
         {
                 my $cmd1 = "priv set test;$cmd";
                 do {
                   #(undef, undef, $output) = say($conn,"$cmd1","-timeout",$opts{timeout});
                   $output = filer_cmd(host => $conn->hostp(), cmd => $cmd1, timeout=>$opts{timeout});
                    } while ($output eq "error: check condition  key: 0x6  code: 0x2a  qual: 0x1");

                 if ($fatal_on_failure eq "yes")
                 {
                         if ($output =~ /usage:/)
                         {
                                logresult('fatal',"Command '$cmd' Failed to execute");
                         }
                         else
                         {
                                #return $output;
                         }
                }
                else
                {
                        if ($output =~ /usage:/)
                         {
                                logcomment("Command '$cmd' Failed to execute");
                         }
                	 else
			 {
                                #return $output;
                         }
                }

         }

	
	 my @output = split (/\r*\n/,$output);


#To get generation count

my @gen = grep /generation/,@output;
my $gen="@gen";
my @generation= split(/:/,$gen);
my $GEN=$generation[1];

#To get key 0

@key = grep /key   0/,@output;
$key="@key";
@reservationkey= split(/:/,$key);
my $KEY0=$reservationkey[1];


#To get key 1

@key = grep /key   1/,@output;
$key="@key";
@reservationkey= split(/:/,$key);
my $KEY1=$reservationkey[1];


#To get additional length

my @len = grep /additional/,@output;
my $len="@len";
my @additionallen= split(/:/,$len);
my $additionallen=$additionallen[1];
   $additionallen=~s/^\s//;
my $ADDLEN=$additionallen;


my %RESULT = (  'generation'=>"$GEN",
                'key0'=>"$KEY0",
		'key1'=>"$KEY1",
                'additional'=>"$ADDLEN",);

return %RESULT;


        #return $output;

 }#scsi_pr_read_keys

sub scsi_pr 
 {

	my (%opts) = validate(@_, {
                                        host => { type=>OBJECT, isa=>'Hostrec' },
                                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                                        filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                                        adapter_name => {type=>SCALAR,optional=>1},
					resev_key => {type=>SCALAR,optional=>1},
					resev_type => {type=>SCALAR,optional=>1},
                                        disk_name => {type=>SCALAR,optional=>1},
					new_resev_key => {type=>SCALAR,optional=>1},		
					command => {type=>SCALAR,optional=>1},
                                        fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
                                  });





		        if ($opts{filermode} ne "normal")
		        {
		               $opts{conntype} = "#console";
		        }


                    my $host = $opts{host};
                    my $disk_name = $opts{disk_name};
                    my $conntype = $opts{conntype};
                    my $timeout = $opts{timeout};
                    my $conn = make_conn($host, $conntype);
                    my $filermode = $opts{filermode};
                    my $adapter_name = $opts {adapter_name};
		    my $resev_key = $opts {resev_key};
		    my $resev_type = $opts {resev_type};
                    my $output="";
		    my $fatal_on_failure =$opts {fatal_on_failure};
		    my $new_resev_key =$opts {new_resev_key};
		    my $command =$opts {command};
		    my $cmd;

	if ($command eq "reserve")
	{
		$cmd = "scsi pr_reserve -h $adapter_name -k $resev_key -t $resev_type $disk_name";

	}elsif($command eq "release")
	{
		$cmd = "scsi pr_release -h $adapter_name -k $resev_key -t $resev_type $disk_name";
	}elsif($command eq "clear")
	{
		$cmd = "scsi pr_clear -h $adapter_name -k $resev_key $disk_name";		
	}elsif($command eq "preempt")
	{
		$cmd = "scsi pr_preempt -h $adapter_name -k $resev_key -t $resev_type -s $new_resev_key $disk_name";
	}elsif($command eq "register")
	{
		$cmd = "scsi pr_register -h $adapter_name -s $resev_key $disk_name";
	}elsif($command eq "registerignorekey")
    {
        $cmd = "scsi pr_register_ignore_key -h $adapter_name -s $resev_key $disk_name";
    }


         if ($filermode eq "MAINT")
         {	
		 do {
	                 (undef, undef, $output) = say($conn,"$cmd",-prompt,"$MAINT_PROMPT","-timeout",$opts{timeout});
		    }while ($output eq "error: check condition  key: 0x6  code: 0x2a  qual: 0x1");


		 if ($fatal_on_failure eq "yes")
                 {
                         if ($output =~ /usage:/)
                         {
                                logresult('fatal',"Command $cmd Failed to execute");
                         }
                         else
                         {
                                return $output;
                         }
                }
                else
                {
                        if ($output =~ /usage:/)
                         {
                                logcomment("Command $cmd Failed to execute");
                         }
                         else
                         {
                                return $output;
                         }

                }


         }
         else
         {
                 my $cmd1 = "priv set test;$cmd";
		 do {
                   #(undef, undef, $output) = say($conn,"$cmd1","-timeout",$opts{timeout});
		 $output = filer_cmd(host => $conn->hostp(), cmd => $cmd1, timeout=>$opts{timeout});
		    } while ($output eq "error: check condition  key: 0x6  code: 0x2a  qual: 0x1");

		 if ($fatal_on_failure eq "yes")
		 {
	              	 if ($output =~ /usage:/)
			 {
	       			#croak $output;
                                logresult('fatal',"Command $cmd Failed to execute"); 
			 }
			 else
			 {
			        return $output;
		         }
		}
		else
		{
			if ($output =~ /usage:/)
                         {
                                logcomment("Command $cmd Failed to execute");
                         }
                         else
                         {
				return $output;
                         }

		}

         }

	#return $output;

 }#scsi_pr

#This routine accepts  disk name as read from the first filer and returns the disk name of the same disk as read from its partner.

=head9
	 my $Pdisk=get_partner_disk_name(host => $hostB,timeout=>$TIMEOUT,filermode=>$filermode,conntype=>$FILER_CONN,disk_name=>$disk);
	 	

=cut

sub get_partner_disk_name 
{

        my (%opts) = validate(@_, {
                                        host => { type=>OBJECT, isa=>'Hostrec' },
                                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                                        filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                                        disk_name => {type=>SCALAR},
                                        fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
                                  });

        if ($opts{filermode} ne "normal")
        {
               $opts{conntype} = "#console";
        }

                    my $host = $opts{host};
                    my $disk_name = $opts{disk_name};
                    my $conntype = $opts{conntype};
                    my $timeout = $opts{timeout};
                    my $conn = make_conn($host, $conntype);
                    my $filermode = $opts{filermode};
                    my $output="";

		my $SSD = storage_show_disk(host=>$host,conntype=>$conntype,filermode=>$filermode,timeout=>"500");
	        my  @ARRAY=keys(%$SSD);

                foreach my $disk (@ARRAY)
                        {
                                my @disk1= split(/\./,$disk_name,2);
                                my $disk1= $disk1[1];
                                if ($disk =~ m/($disk1)$/)
                                {
                                return $disk;
                                }

                        }



}#END of get_partner_disk_name


#Storage release_disks
sub storage_release_disks {


        my (%opts) = validate(@_, {
                                        host => { type=>OBJECT, isa=>'Hostrec' },
                                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                                        filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                                        fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
                                  });

        if ($opts{filermode} ne "normal")
        {
               $opts{conntype} = "#console";
        }

                    my $host = $opts{host};
                    my $conntype = $opts{conntype};
                    my $timeout = $opts{timeout};
                    my $conn = make_conn($host, $conntype);
                    my $filermode = $opts{filermode};
		    my $output="";
         if ($filermode eq "maint")
         {
                 my $cmd = "storage release disks";
                 (undef, undef, $output) = say($conn,"$cmd",-prompt,"$MAINT_PROMPT","-timeout",$opts{timeout});

         }
         else
         {
                 my $cmd = "priv set test;storage release disks";
                 (undef, undef, $output) = say($conn,"$cmd","-timeout",$opts{timeout});
         }
}#END of Storage _release_disks





#-----------------------------------------------------------------------
# Subroutine to get the system ID and Partner ID
#-----------------------------------------------------------------------

sub get_SYS_ID
{

	my (%opts) = validate(@_, {
                                        host => { type=>OBJECT, isa=>'Hostrec' },
                                        timeout => { type=>SCALAR, default=>300, optional=>1 },
                                        filermode => { type=>SCALAR, default=>"normal", optional=>1 },
                                        conntype => {type=>SCALAR,default=>"#rsh",optional=>1},
                                        fatal_on_failure => {type=>SCALAR,default=>"yes",optional=>1},
                                  });

        if ($opts{filermode} ne "normal")
        {
               $opts{conntype} = "#console";
        }

                    my $host = $opts{host};
                    my $conntype = $opts{conntype};
                    my $timeout = $opts{timeout};
                    my $conn = make_conn($host, $conntype);
                    my $filermode = $opts{filermode};
                    my $output="";
         if ($filermode eq "maint")
         {
                 my $cmd = "sysconfig -v";
                 (undef, undef, $Output) = say($conn,"$cmd",-prompt,"$MAINT_PROMPT","-timeout",$opts{timeout});

         }
         else
         {
                 my $cmd = "priv set test;sysconfig -v";
                 #(undef, undef, $Output) = say($conn,"$cmd","-timeout",$opts{timeout});
		 $Output = filer_cmd(host => $conn->hostp(), cmd => $cmd, timeout=>$opts{timeout});
         }

	my @Output=split(/\r*\n/,$Output);
        my @ID;
        for(my $i=0;$i<=37;$i++)
        {
                if (grep/System\s+ID/,$Output[$i])
                {
                        my $Output=$Output[$i];
                        $Output =~s/\s*//g;
                        my @Output=split(/;/,$Output);
                        my @SYSID= split(/:/,$Output[0]);
                        @SYSID=split(/\(/,$SYSID[1]);
                        my @PARTNERID=split(/:/,$Output[1]);
                        @PARTNERID=split(/\(/,$PARTNERID[1]);
                        @ID=($SYSID[0],$PARTNERID[0]);
                }
        }
        #returns the SYS_ID and PARTNER SYS_ID
	print "SYSTEM IDS aRE :@ID\n\n\n";
        return @ID;


} #End of get_SYS_ID routine

