#Hey emacs, this is basically -*- perl -*-
# $Id: //depot/prod/test/Rsierranevada/lib/Sync.pm#1 $

## @summary     Package to handle syncing script execution
## @author      bryans@netapp.com
## @pod here

package Sync;

use Tharn;
use TracePkg;
use Exporter ();
use IO::Select;
use Data::Dumper;

@ISA = qw(Exporter);
@EXPORT = qw(sync wait_for_sync);

my $Trace = new TracePkg;


sub convertmsg {
    my $msg = shift;
    my %retval;

    foreach my $kv (split(/\s+/, $msg)){
	my ($k, $v) = split(/=/, $kv);
	$retval{$k} = $v;
    }
    return \%retval;
}

sub sync
{
    my %params = (
		  syncwait => 15,
		  @_,
	      );

    my @retval;
    $Trace->entry();
    if (!exists $params{subtests}){
	my @runnning_pids = Subtest::get_live();
	$params{subtests} = \@running_pids;
    }
    $Trace->debug(msg => "There are " . (scalar @{$params{subtests}}) . " pids");
    if (!@{$params{subtests}}){
	$Trace->exit(msg => "No subtests");
	return 0;
    }
    my $start = time();
    foreach my $pid (@{$params{subtests}}){
	$Trace->debug(msg => "Sending sync message to $pid");
	# Should check to make sure that ipc() is valid and if not remove it from the list
	$pid->ipc()->write_message("cmd=sync time=" . time());
    }
    my $count = 0;
    my $selectgroup = new IO::Select;
    my %sock_to_pid;
    my %fileno_to_handle;
    foreach my $pid (@{$params{subtests}}){
	my $ipc = $pid->ipc();
	my $fileno = fileno($ipc);
	$selectgroup->add($fileno);
	$sock_to_pid{$fileno} = $pid;
	$fileno_to_handle{$fileno} = $ipc;
    }
    while ($selectgroup->count()){
	if (my @handles = $selectgroup->can_read(0.01)){
	  handleloop:
	    foreach my $handle (@handles){
		my $cmsg;
		$Trace->debug(msg => "Got message from $sock_to_pid{$handle}->{host} on handle $handle");
		{
		    local $SIG{ALRM} = sub { die "Got tired of waiting"};
		    eval {
			alarm 1;
			$cmsg = $fileno_to_handle{$handle}->read_message();
			alarm 0;
		    };
		    if ($@ =~ /Got tired of waiting/){
			print "Timed out\n";
			next handleloop;
		    }
		}
		$Trace->debug(msg => "Received message(".length($cmsg)."): " . Dumper(convertmsg($cmsg)));
		push @retval, $sock_to_pid{$handle};
		next handleloop unless length($cmsg);
		$selectgroup->remove($handle);
		$count++;
            }
        }
    }

    # Let's assume that getting all of the messages out to the clients takes as long as it took
    # us to send and receive the initial sync message.  This will help ensure that we have an
    # accurate time since the writing of the messages could be queued such that it looks like
    # all of the writes happen instantly.  Reading the responses to the messages gives us a
    # better sense of the round trip time.
    my $runtime = time() - $start;
    $Trace->debug(msg => "runtime = $runtime");
    # Got them all, so calculate the sync time
    my $synctime = time() + $runtime + $params{syncwait};
    foreach my $pid (@retval){
	$Trace->debug(msg => "Sending go_at message to " . $pid->ipc());
	$pid->ipc()->write_message("cmd=go_at time=$synctime");
    }
    $Trace->exit(msg => "Returning " . join(',', @retval));
    @retval;
}

sub wait_for_sync
{
    my %params = (
		  timeout => 300,
		  @_,
	      );

    $Trace->entry();
    my $socket = Subtest::ipc();
    my $state = 0;
    my $count = 0;
    my $selectgroup = new IO::Select($socket);
    my $sock_to_pid;
    while ($selectgroup->count()){
	if (my @handles = $selectgroup->can_read(0.01)){
	    $Trace->comment(msg => "Have " . (scalar @handles) . " to read");
	    foreach my $handle (@handles){
                my $msg = $handle->read_message();
                my $received_message_at = time();
                my $msgh = convertmsg($msg);
                $Trace->comment(msg => "Received message at " . (scalar localtime $received_message_at) . Dumper($msgh));
                my $delta = 0;
                if ($msgh->{cmd} =~ /sync/){
                    my $host = `/bin/hostname`;
                    chomp $host;

                    # Since we were waiting for the message the synchronizer latency to us should be relatively small,
                    # so calculate our delta so that we can figure out the synchronized start time
                    $delta = $received_message_at - $msgh->{time};

		    $Trace->comment(msg => "Sending message back to controller");
                    # Send back our "OK we're waiting message
                    $handle->write_message("host=$host delta=$delta");
		    $Trace->comment(msg => "Message sent back to controller");
                    $state++; # 1 => Waiting for the sync time message
                } elsif ($msgh->{cmd} =~ /go_at/){
                    # We got a go_at message.  This means that all the other clients reported back that they were ready
                    # (or the synchonizer gave up on them) and we've been instructed to go at a particular time.
                    # Adjust the time by our synchronizer time delta, calculate how long we need to sleep before that
                    # time arrives and then sleep that long.
                    $msgh->{time} += $delta;
                    my $sleeptime = $msgh->{time} - time();
                    if ($sleeptime > 0){
                        $Trace->comment(msg => "Go_At time is $sleeptime seconds away");
                        sleep($sleeptime);
                    } else {
                        $Trace->comment(msg => "Got a negative sleeptime of $sleeptime.  Sigh");
                    }
                    $Trace->comment(msg => "Synced.  Running");
		    $selectgroup->remove($handle);
                    last;
		} else {
		    $Trace->comment(msg => "Could not handle message: ". Dumper($msgh));
		}
	    }
	}
    }
    $Trace->exit();
    return 1;
}

1;
