# $Id: //depot/prod/test/Rsierranevada/lib/Sysctl.pm#1 $
## @summary Perl module for access to the Tricky 'sysctl' command
## @author Bill Snider
## @pod here

=head1 NAME

Sysctl.pm - Library of common GX sysctl commands

=head1 DESCRIPTION

C<Sysctl> provides common routines used to access the sysctl command

=head1 SYNOPSIS

    use Sysctl;
    my $sys = new Sysctl;
    my $value = $sys->get(oid=>"variable");

=head1 ROUTINES

=cut

my $LibraryLoaded;
if ($ENV{OVERRIDESYSCTL}) {
    my $newModule = $ENV{OVERRIDESYSCTL};
    delete $ENV{OVERRIDESYSCTL};
    print "The main Sysctl module will be replaced with $newModule\n";
    require($newModule);
    $LibraryLoaded = 1;
}
return 1 if ($LibraryLoaded);

package Sysctl;

use strict;
use warnings;
use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF);

use TracePkg;
my $Trace = new TracePkg;

use Server;

=head2 new

    use Sysctl;
    my $sys = new Sysctl;

C<new> is the invocant for the Sysctl class. It will create it's
own L<Server> object so that it can call 'sysctl' directly.

=cut

sub new {
    my $invocant = shift;
    my $class = ref($invocant) || $invocant;
    my $self = {
		@_, # overlay defaults with what was explicitly passed
		};
    bless($self, $class);
    $Trace->entry();
    $self->{server} = new Server;
    $Trace->exit();
    return $self;
}

=head2 get

    my $value = $sys->get(oid=>"variable");

C<get> provides a method to get the value of an oid.

=over

=item oid

The name of the oid (kernel parameter).

=back

=cut

sub get {
    my $self = shift;
    my (%params) = validate_mixed(
				  @_,
				  [ ],
				  {
				      oid => {type => SCALAR},
				  }
				  );
    $Trace->entry(msg=>"oid=>$params{oid}");
    my $value;
    
    my $output = $self->{server}->shell(cmd=>"sysctl $params{oid}");
    if ($output =~ /$params{oid}: (\S+)/) {
        $value = $1;
    } else {
        $Trace->fatal(msg=>"Could not parse output of 'sysctl $params{oid}': $output");
    }
    $Trace->exit(msg=>"value=$value");
    return $value;
}

=head1 PARAMS

=over

=back

=head1 SEE ALSO

L<Server>

=head1 BUGS


=head1 STATUS

  Shared

=head1 AUTHOR

  Bill Snider (snider@netapp.com)
  $Id: //depot/prod/test/Rsierranevada/lib/Sysctl.pm#1 $
  Last revised $Date: 2013/01/22 $

=head1 COPYRIGHT

Copyright 2006, Network Appliance., All Rights Reserved

=cut


1;
