package SystestCommon;

sub MergeHashes {
    # Merge multiple hashes, with preference being given to the first hash.
    my %retval, $key;

    foreach my $hr (@_){
	if (ref($hr) ne "HASH"){
#	    warn "$hr not a hash reference.  Skipping";
	    next;
	}
	foreach $key (keys %{$hr}){
#	    print "$hr:$key\n";
	    $retval{$key} = $$hr{$key};
	}
    }

    \%retval;
}

#***********************************************************************
#
# Routine:     CopyFile
#
# Description: Copy a file.
#
# Arguments:   from specifies the source file
#              to specifies the destination directory or file
#
# Returns:     undef on failure
#              1 if the file was updated
#              2 if the file was the same 
#
#***********************************************************************


sub CopyFile {
    my ($from, $to) = @_;
    my ($srcbuf, $dstbuf);
    my ($sbytes, $dbytes);
    my $old = undef;
    my $new = "$to.new";
    my $different = undef;

    if (!$from) {
	print "No Src File provided to CopyFile\n";
	return undef;
    }
    if (!$to) {
	print "No Dest File provided to CopyFile\n";
	return undef;
    }
    if (! -f $from) {
	print "$from does not exist\n";
	return undef;
    }
    if (-d $to) {
	my @path = split(/\//, $from);
	my $file = pop(@path);
	$to .= "/$file";
    }
    
    if (-l $to) {
	if (!unlink($to)) {
	    print "Cannot unlink $to\n";
	    return undef;
	}
    }
    if (!open(SRC, $from)) {
	print "Could not open $from file to Copy\n";
	return undef;
    }
    if (!open(NEW, ">$new")) {
	print "Could not open $new for Copy\n";
	return undef;
    }
    if (-e $to) {
	open(DST, $to);
	$old = "$to.old";
    }
    while ($sbytes = read(SRC, $srcbuf, 4096)) {
	if ($old) {
	    $dbytes = read(DST, $dstbuf, 4096);
	    if (!$different) {
		if ($sbytes != $dbytes || $srcbuf ne $dstbuf) {
		    $different = 1;  
		} 
	    }
	}
	print NEW $srcbuf; 
    }
    if (!$old || $different) {
	if ($old) {
	    unlink($old);
	    rename($to, $old);
	}
	if (!rename($new, $to)) {
	    if ($! eq "Text file busy") {
		rename($to, "$to.busy");
		if (!rename($new, $to)) {
		    print "Could not install new version of $to\n";
		    return undef;
		}
	    }
	}
    } else {
	# already copied
	unlink($new);
	return 2;
    }
    return 1;
}

sub SymLink {
    my ($symlink, $file) = @_;
    
    if (!$file) {
	print "No file provided to Symlink\n";
	return undef;
    }
    if (!$symlink) {
	print "No symlink provided to Symlink\n";
	return undef;
    }
    if (-l $file) {
	if (readlink($file) eq $symlink) {
	    return 1;
	} else {
	    unlink($file);
	}
    }
    if (!symlink($symlink, $file)) {
	print "Could not create symbolic link from $file to $symlink\n";
	return undef;
    }
    return 1;
}

1;
