package Testinfo;

use strict;
use Options;
use Tharn;
use Data::Dumper;

#***********************************************************************
#
# NAME
#   Testinfo.pm - 
#
# DESCRIPTION
#   The one sub routine in this module was copied and pasted in 3 
#   separate thpls.  Thought it was time to consolidate.
#
# SUB ROUTINE
#   dir() - upon success, dir returns the testinfo directory, and 
#           sets the NATE param DESTDIR.
#
#   chmod_whole_dir() - chmod -R 777 the whole testInfo directory.
#
#   make_open_dir( <dir name> ) - create a directory and chmod it to
#                                 fully open (777)
#
# Author: mgarrett
# 
#***********************************************************************


require Exporter;
our @ISA = qw(Exporter);
our @EXPORT = qw();
our @EXPORT_OK = qw();

#Setup some package global variables											
my $Options = Options->new();
my $Trace = TracePkg->new();

#Grab these parameters passed in from either Nate or the Testbed
$Options->parameters(definitions=>
	[
		   {
		       name => "DESTDIR",
		       description => "Destination directory for log output.",
		   },
	]
);

$Options->process();
my $Dest_dir = $Options->get(option => "DESTDIR");



# upon success, dir returns the testinfo directory, and sets the NATE 
# PARAM 'DESTDIR'.
sub dir() {

    #This clip of code was stolen from gsi.thpl
    #If no other script passed us the DESTDIR param, we
    #determine what it should be, and pass it on.
    if ( ! $Dest_dir ) {
        $Dest_dir = Tharn::param("LOGDIR","-mesg","foo");
        if ( $Dest_dir eq ""  ) { 
            $Dest_dir = "/x/eng/spin-cores/$ENV{LOGNAME}";
        }
        my ($sec,$min,$hr,$day,$mon,$yr,$dow,$doy,$isdst) = localtime(time);
        $yr += 1900; $mon += 1;
        my $now = sprintf("%04d%02d%02d.%02d%02d%02d", $yr,$mon,$day,$hr,$min,$sec);
        $Dest_dir .= "/$now.testInfo";
        logresult('PARAM',"DESTDIR=$Dest_dir");
    }

    #Now that we know what the DESTDIR is, we now may need to create
    #it.
    if (-d $Dest_dir) { #Check to see if the dir is already there
        $Trace->comment(msg=>"$Dest_dir already exists.  No create needed.");
        return $Dest_dir;
    } else {
        $Trace->comment(msg=>"$Dest_dir doesn't exist.  Creating now...");
        make_open_dir($Dest_dir);

        #Now verify that it actually worked.
        if (-d $Dest_dir) {
            return $Dest_dir;
        } else {
            $Trace->warn(msg=>"Error: I tried to create and chmod this dir '$Dest_dir', but it still doesn't exist.");
            return undef;
        }
    }
}

#Convinience routine to run one or many system calls after using trace to comment on the call I'm about to make.
sub comment_and_system {
    for (@_) {
        $Trace->comment(msg=>"running system call: '$_'");
        system "$_";
    }
}

sub chmod_whole_testinfo_dir {
    my $dir=dir();
    comment_and_system("sudo chmod -R 777 $dir","hostname","find $dir ! -perm 777 -exec ls -ld '{}' \\; | grep -v autosupport","find $dir -exec ls -ld '{}' \\;");
}

sub make_open_dir {
    my $dir=shift;

    if (-d $dir) {
        $Trace->comment(msg=>"I was asked to create directory '$dir', but it already exists.");
    } else {
        if (mkdir "$dir") {
            $Trace->comment(msg=>"chmod -R 777 $dir");
            system "chmod -R 777 $dir";
        } else {
            $Trace->exit(msg=>"Error: I tried to create directory '$dir', but it failed with this error:  '$!'", result=>"FAIL");
        }
    }
}

1;
