#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     CIFS Stress Scalability Topology Library
##
## @author      bellizzi@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::CIFS:;Parameters - CIFS Stress Scalability test topology library

=head1 DESCRIPTION

=head1 TODO

=head1 SEE ALSO

=head1 BUGS

=head1 STATUS

  Shared

=head1 AUTHOR

  David Bellizzi (gvg@netapp.com)
  $Id: $
  Last revised $Date: 2008/05/16 $
    
=head1 COPYRIGHT

Copyright 2008, Network Appliance., All Rights Reserved

=cut

package Topology::NAS::CIFS::Parameters;

use strict;
use warnings;
use Tharn qw($Log script_result);
$Log->comment("Starting " . __FILE__);
use Options;
use Params::Validate qw(
    SCALAR
    HASHREF
    ARRAYREF
    OBJECT
    UNDEF
    CODEREF);

my $Options = new Options;
$Options->parameters(
    definitions => [
        {   name        => 'ANTIVIRUS',
            spec        => 's',
            default     => undef,
            description => 'Enable AV',
        },
        {   name        => 'APISET_TIMEOUT',
            spec        => 's',
            default     => 300,
            description => 'The default APISET timeout',
        },
        {   name        => 'AV_VENDOR',
            spec        => 's',
            default     => 'sophos',
            description => 'The antivirus vendor to use',
        },
        {   name        => 'AV_PROTOCOLS',
            spec        => 's',
            default     => 'cifs',
            description => 'The desired protocols to scan',
        },
        {   name        => 'AV_CIFS_SHARE_REGEX',
            spec        => 's',
            default     => '"^.*$"',
            description => 'The shares the policy should scan',
        },
        {   name        => 'AV_CIFS_FILE_REGEX',
            spec        => 's',
            default     => '\"^.*$\"',
            description => 'The files the policy should scan',
        },
        {   name        => 'AGGREGATES',
            spec        => 's',
            default     => undef,
            description => 'The aggregates to use for this test',
        },
        {   name    => 'AGGR_REGEX',
            spec    => 's',
            default => '.*',
            description =>
                'One ore more comma separated regex to match aggregates '
                . 'where load is applied',
        },
        {   name    => 'ALT_LOGDIR',
            spec    => 's',
            default => undef,
            description =>
                'Write a copy of the csv file to this dir if defined',
        },
        {   name        => 'AUTOSUPPORT',
            spec        => 's',
            default     => undef,
            description => 'Enables Autosupport feature on a node',   
        },
        {   name        => 'AUTOSUPPORT_MAILHOSTS',
            spec        => 's@',
            default     => 'mailhost.netapp.com, mailhost.eng.netapp.com',
            description => 'The mailhost used to send ASUPs',
        },
        {   name        => 'AUTOSUPPORT_TO',
            spec        => 's@',
            default     => 'dl-qa-autosupport@netapp.com',
            description => 'The address for sending an ASUP to so that it goes to our internal ASUP servers',
        },
        {   name        => 'BRANCH_CACHE',
            spec        => 's',
            default     => undef,
            description => 'Enables the Branch Cache feature'
        },
        {   name    => 'CAPACITY',
            spec    => 's',
            default => '99',
            description =>
                'The capacity threshold where we fail for lack of diskspace',
        },
        {   name        => 'CDEF_PATH',
            spec        => 's',
            default     => undef,
            description => 'The original CDEF path',
        },
        {   name => 'CIFS_SERVER_ACCESS_MODE',
            description =>
                'Determines what the server will be in \\server\share',
            spec         => 's',
            default      => 'ip',
            valid_values => ['', 'ip', 'name', 'fqdn'],
        },
        {   name        => 'CIFS_DC',
            description => 'The domain controller to use',
            spec        => 's',
        },
        {   name        => 'CIFS_HOMEDIR',
            description => 'Sets up I/O to be run against a users homedir',
            default     => undef,
        },
        {   name        => 'CIFS_SECURITY_STYLE',
            default     => undef,
            spec        => 's',
            description => 'The security style used with cifs setup',
        },
        {   name        => 'CIFS_SERVER_BASENAME',
            description => 'The base name to use when creating CIFS server '
                . 'names on multiple vservers',
            spec => 's',
        },
        {   name    => 'CLEANUP_TASKS',
            spec    => 's@',
            default => 'processes,mappings,shares,data',
            description =>
                'Controls what gets cleaned up durng the cleanup phase',
        },
        {   name        => 'CLIENT_ASSIGNMENT',
            spec        => 's',
            default     => undef,
            description => 'How we assign clients',
        },
        {   name         => 'CLIENT_CONNECT_METHOD',
            spec         => 's',
            default      => '#staf',
            valid_values => ['', '#rsh', '#staf', '#telnet'],
            description => 'Use STAF, RSH or telnet to connect to the client',
        },
        {   name        => 'CLIENT_ETC_HOSTS_CLEAN_UP',
            spec        => 's',
            default     => 1,
            description => 'Cleans up client hosts file',
        },
        {   name        => 'CLIENT_HOSTTYPE_FILTER',
            spec        => 's',
            default     => undef,
            description => 'Deprecated, use CLIENT_HOSTTYPE_REGEX',
        },
        {   name        => 'CLIENT_HOSTTYPE_REGEX',
            spec        => 's',
            default     => '.*',
            description => 'How we filter clients on hosttype',
        },
        {   name    => 'CLIENT_STARTUP_DELAY',
            spec    => 's@',
            default => '1',
            description =>
                'How long to wait between configuring clients the next client',
        },
        {   name    => 'CSV_DIR',
            spec    => 's',
            default => 'csv',
            description =>
                'The directory in TOP_LOGDIR where we write csv files',
        },
        {   name    => 'DATA_LIFS_PER_NODE',
            spec    => 's',
            default => 1,
            description =>
                'The number of data lifs to create per node per vserver',
        },
        {   name        => 'DC_CONNECT_METHOD',
            spec        => 's',
            default     =>  '#staf',
            valid_values => ['', '#rsh', '#staf', '#telnet'],
            description => 'Use STAF, RSH or telnet to connect to the client',
        },
        {   name        => 'DEATH',
            spec        => 's',
            default     => undef,
            description => 'how workers die',
        },
        {   name        => 'DEFAULT_UNIX_USER',
            spec        => 's',
            default     => 'nobody',
            description => 'Just what it says',
        },
        {   name        => 'DNS_SERVERS',
            spec        => 's',
            default     => undef,
            description => 'list of namerserves',
        },
        {   name        => 'DNS_SEARCH',
            spec        => 's@',
            default     => undef,
            description => 'DNS search path',
        },
        {   name        => 'DESTIP',
            spec        => 's',
            default     => undef,
            description => 'IP ranges for the DUTs '
                . '7M:<node index>;<interface>;<ip range name>,<node index>;<interface>;<ip range name> '
                . 'CM:<node index>;<interface>;<ip range name>,<node index>;<interface>;<ip range name>',
        },
        {   name         => 'DESTIP_VERSION',
            spec         => 's',
            default      => 4,
            description  => 'The IP version you want to use for load',
            valid_values => ['', 4, 6],
        },
        {   name        => 'FREE_CLIENTS',
            spec        => 's',
            default     => 0,
            description => 'Free client reservations',
        },
        {   name    => 'FREE_UNUSED_CLIENTS',
            spec    => 's',
            default => 0,
            description =>
                'Free reservations for clients that are not needed',
        },
        {   name        => 'GB_REQ_PER_CLIENT',
            spec        => 's',
            default     => '10',
            description => 'Define the number of GBs per client required',
        },
        {   name    => 'IO_CHECK_FOR_LOG_ERRORS',
            spec    => 's',
            default => 1,
            description =>
                'Check the binary logs for know error regulare expressions',
        },
        {   name        => 'IO_KILL_ALL_TOOLS_DELAY',
            spec        => 's',
            default     => 180,
            description => 'How long we wait after killing all tools',
        },
        {   name        => 'IO_MAP_SHARES',
            spec        => 's',
            default     => 1,
            description => 'Map the shares for all tools',
        },
        {   name        => 'IO_REMOVE_DATA',
            spec        => 's',
            default     => 1,
            description => 'Remove client generated data',
        },
        {   name        => 'IO_REMOVE_DATA_DELAY',
            spec        => 's',
            default     => 120,
            description => 'How long we wait after removing data',
        },
        {   name        => 'IO_REMOVE_MAPS_DELAY',
            spec        => 's',
            default     => 120,
            description => 'How long we wait after removing maps',
        },
        {   name        => 'IO_TOOLS',
            spec        => 's',
            default     => undef,
            description => 'Used internally to access the tools hash',
        },
        {   name        => 'IO_TOOLS_FILTER',
            spec        => 's',
            default     => undef,
            description => 'Deprecated, use IO_TOOLS_REGEX',
        },
        {   name    => 'IO_MAX_ATTEMPTS',
            spec    => 's',
            default => 10,
            description =>
                'How many time we try leanup operations before we fail',
        },
        {   name        => 'IO_RETRY_DELAY',
            spec        => 's',
            default     => 60,
            description => 'How long we wait before cleanup attempts',
        },
        {   name        => 'IO_TOOLS_CONFIG',
            spec        => 's',
            default     => '',
            description => 'The file containing the tools hash',
        },
        {   name        => 'IO_LOAD_CONFIG',
            spec        => 's',
            default     => undef,
            description => 'The file containing the load configuration',
        },
        {   name        => 'IO_TOOLS_OVERRIDES',
            spec        => 's',
            default     => undef,
            description => 'A comma separated list of tool overrrides '
                . 'toolname-instance-PARAMETER=value',
        },
        {   name        => 'IO_TOOLS_REGEX',
            spec        => 's',
            default     => undef,
            description => 'A regex for limiting the tools that get executed',
        },
        {   name        => 'INSTANCES_PER_CLIENT',
            spec        => 's',
            default     => '1',
            description => 'Define the number of instances per client',
        },
        {   name        => 'KERBEROS',
            spec        => 's',
            default     => '1',
            description => 'Enables and disables the use of kerberos',
        },
        {   name        => 'LAB',
            spec        => 's',
            description => 'Import LAB from NATE params',
        },
        {   name        => 'LOCAL_GROUP_COUNT',
            spec        => 's',
            default     => undef,
            description => 'The number of local groups to create clusterwide',
        },
        {   name        => 'LOCAL_GROUP_MEMBERS',
            spec        => 's',
            default     => undef,
            description => 'Comma separated list of users in the form of '
                . 'basename:start:end or username',
        },
        {   name        => 'LOCAL_UNIX_USERS',
            description => 'List of local unix users to create.  Each user '
                . 'should be in the format of name:uid:gid:full_name',
            default => 'root:0:0:"Local Root",nobody:65534:65534:nobody',

            # intentionally a string because Options splits on , or :
            spec => 's',
        },
        {   name        => 'LOCAL_UNIX_GROUPS',
            description => 'List of local unix groups to create.  Each group '
                . 'should be in the format of name:gid:users',
            default => 'root:0:root,nobody:65534:nobody',

            # intentionally a string because Options splits on , or :
            spec => 's',
        },
        {   name        => 'LOGDIR_WINDOWS',
            spec        => 's',
            description => 'Import LOGDIR_WINDOWS from NATE params',
        },
        {   name        => 'LOGSPEC',
            spec        => 's',
            description => 'Import LOGSPEC from NATE params',
        },
        {   name    => 'MAX_AGGREGATE_USAGE',
            spec    => 's',
            default => 90,
            description =>
                'The percentage of the avaiable aggr space to use, once '
                . 'exceeded the aggregate will be marked as full nad not '
                . 'volumes will be assigned to it',
        },
        {   name        => 'MAX_CLIENTS',
            spec        => 's',
            description => 'One ore more (comma separated) values that '
                . 'specify the max clients to use',
        },
        {   name        => 'MAX_CLIENT_LOSS',
            spec        => 's',
            default     => 10,
            description => 'The max percentage of processes that can '
                . 'become inactive before test fails',
        },
        {   name => 'MAX_SHARE_ACLS',
            description =>
                'The maximum number of share ACLs to create per share',
            default => 255,
        },
        {   name        => 'NIS_DOMAINNAME',
            spec        => 's',
            description => 'The NIS domainname',
        },
        {   name        => 'NIS_SERVERS',
            spec        => 's',
            description => 'The NIS servers',
        },
        {   name        => 'NODES',
            spec        => 's@',
            description => 'Import from NATE params',
        },
        {   name        => 'NODE_REGEX',
            spec        => 's',
            default     => '.*',
            description => 'One ore more (comma separated) regex to match '
                . 'node names where load is applied',
        },
        {   name        => 'ORIGIN_VOLUME_PERCENT',
            spec        => 's',
            default     => undef,
            description => 'The approximate percentage of data volumes that '
                . 'will be cache origin volumes',
        },
        {   name         => 'PRE_TEST_CLEANUP',
            spec         => 's',
            default      => undef,
            valid_values => ['', 'defined', 'discovered', 'disabled'],
            description =>
                'cleanup everything or just what the current test defines',
        },
        {   name        => 'POST_FAILURE_CLEANUP',
            spec        => 's',
            default     => 0,
            description => 'Cleanup on failures',
        },
        {   name        => 'PREFERRED_DOMAIN_CONTROLLERS',
            spec        => 's@',
            default     => undef,
            description => 'Comma spererated list of AD servers IP addresses',
        },
        {   name        => 'QUOTAS',
            spec        => 's',
            default     => undef,
            description => 'Enable and define quotas',
        },
        {   name => 'RESERVATION_REASON',
            spec => 's',
            description =>
                'The string to be added to the reason when reserving '
                . 'clients via NDISP',
        },
        {   name        => 'ROOT_VOLUME_SIZE',
            spec        => 's',
            default     => 100,
            description => 'The size of all root volumes in MB',
        },
        {   name        => 'SMB2',
            spec        => 's',
            default     => undef,
            description => 'Enable SMB2 on a vfiler',
        },
        {   name    => 'SCRIPT_INDEX',
            spec    => 's',
            default => 0,
            description =>
                'Index to append to file where used clients are stored',
        },
        {   name        => 'SHARE_COUNT',
            spec        => 's',
            default     => undef,
            description => 'The number of additional shares to create',
        },
        {   name        => 'SHARE_PROPERTIES',
            spec        => 's',
            default     => 'oplocks,browsable,changenotify',
            description => 'The number of shares to create per volume,  '
                . 'calculated if undef',
        },
        {   name        => 'SHARES_PER_DIRECTORY',
            spec        => 's',
            default     => 1,
            description => 'The number of shares to create per directory',
        },
        {   name    => 'SHARES_PER_VOLUME',
            spec    => 's',
            default => undef,
            description =>
                'The number of shares to create per volume, calculated if undef',
        },
        {   name         => 'SHARE_PATH_CONFIG',
            spec         => 's',
            default      => 'multiple',
            valid_values => ['', 'multiple', 'single'],
            description  => 'The number of shares to create per directory',
        },
        {   name        => 'SHARE_ACL_PERMISSIONS',
            description => 'List of share ACL permissions to use',
            spec        => 's@',
            default     => 'No Access,Read,Change,Full Control',
        },
        {   name        => 'SHARE_REGEX',
            description => 'Regex to choose the shares we modify',
            default     => undef,
        },
        {   name        => 'SHARE_DATA_VOLUMES',
            spec        => 's',
            default     => undef,
            description => 'Create data volume shares',
        },
        {   name        => 'SHARE_ROOT_VOLUMES',
            spec        => 's',
            default     => 1,
            description => 'Create vserver root volume shares',
        },
        {   name    => 'SHARED_VFILERS',
            spec    => 's',
            default => 0,
            description =>
                'Vfilers are shared so don\'t count the root volumes',
        },
        {   name        => 'SITE_CGIBIN',
            spec        => 's',
            description => 'Import SITE_CGIBIN from NATE params',
        },
        {   name        => 'SITE_ROOT',
            spec        => 's',
            description => 'Import ROOT from NATE params',
        },
        {   name        => 'SITE_WEBROOT',
            spec        => 's',
            description => 'Import ROOT from NATE params',
        },
        {   name        => 'SMB_SIGNING',
            spec        => 's',
            default     => undef,
            description => 'Enables cifs signing',
        },
        {   name         => 'STRIPE_FORMAT',
            spec         => 's',
            default      => 'data',
            valid_values => ['', 'data', 'full'],
            description  => 'The striped volume type',
        },
        {   name        => 'THARN_VERBOSE',
            spec        => 's',
            default     => undef,
            description => 'Set to one or greater to see ZAPI debugs',
        },
        {   name    => 'TO_DESTIPS',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name    => 'TO_PROCESS_NAMES',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name    => 'TO_NODES',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name    => 'TO_VFILERS',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name    => 'TO_VSERVERS',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name    => 'TO_SERVERS',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name    => 'TO_VOLUMES',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name    => 'TO_SHARES',
            spec    => 's',
            default => undef,
            description =>
                'Comma separated list passed to each client instance',
        },
        {   name        => 'TERMFILE',
            spec        => 's',
            default     => undef,
            description => 'File to write TERMSTRINGS to',
        },
        {   name        => 'TERMSTRING',
            spec        => 's',
            default     => undef,
            description => 'string to write in TERMFILE when complete',
        },
        {   name    => 'TERMSTRINGS',
            spec    => 's@',
            default => undef,
            description =>
                'A comma seperated list of strings to look for in TERMFILE',
        },
        {   name        => 'TEST_FILE_BASENAME',
            description => 'The basename used to create the test files',
            default     => undef,
        },
        {   name        => 'TEST_FILE_COUNT',
            description => 'The number of test files to use',
            default     => undef,
        },
        {   name        => 'TEST_MAIN_DURATION',
            spec        => 's',
            default     => undef,
            description => 'The duration value used in test main',
        },
        {   name        => 'TEST_MAIN_SLEEP',
            spec        => 's',
            default     => undef,
            description => 'The SLEEP value used in test main',
        },
        {   name        => 'TESTBED',
            spec        => 's',
            description => 'Import TESTBED from NATE params',
        },
        {   name        => 'TESTBED_WEBROOT',
            spec        => 's',
            description => 'Import TESTBED_WEBROOT from NATE params',
        },
        {   name => 'TIMESTAMP',
            spec => 's',
            description =>
                'Used to create unique subtest runids and log names',
        },
        {   name        => 'TEST_GROUP_BASENAME',
            description => 'The basename of all the domain groups used',
            default     => undef,
        },
        {   name        => 'TEST_USER_BASENAME',
            description => 'The basename of all the domain users used',
            default     => undef,
        },
        {   name        => 'TEST_USER_COUNT',
            description => 'The number of test users to use',
            default     => undef,
        },
        {   name => 'TIME_ZONE',
            spec => 's',
            description =>
                'The timezone to use when configuring time on nodes',
        },
        {   name        => 'TIME_SERVER',
            spec        => 's',
            description => 'The NTP server to use',
        },
        {   name        => 'TOP_LOGDIR',
            spec        => 's',
            description => 'Import TOP_LOGDIR from NATE params',
        },
        {   name        => 'TOP_LOGDIR_WINDOWS',
            spec        => 's',
            description => 'Import TOP_LOGDIR_WINDOWS from NATE params',
        },
        {   name        => 'RA',
            spec        => "s",
            default     => 0,
            description => 'Enable or disable resource agitation',
        },
        {   name    => 'RA_IGNORE',
            spec    => "s@",
            default => undef,
            description =>
                'A comma separeted list of ra subsystems to ignore',
        },
        {   name        => 'RA_CONFIG',
            spec        => "s",
            default     => 'stress-scale/cifs/resource_agitation.yml',
            description => 'The YAML file containing the ra config',
        },
        {   name        => 'VFILER_COUNT',
            spec        => 's',
            default     => undef,
            description => 'The number of vfilers to use',
        },
        {   name        => 'VOLUME_ASSIGNMENT',
            spec        => 's',
            default     => 'round_robin',
            description => 'How volumes are assigned to aggregates',
        },
        {   name        => 'VOLUME_COUNT',
            spec        => 's',
            default     => undef,
            description => 'The number of volumes to use/configure',
        },
        {   name        => 'VOLUME_SIZE',
            spec        => 's',
            default     => '1g',
            description => 'Specifies size for each volume',
        },
        {   name        => 'VOLUMES_PER_VFILER',
            spec        => 's',
            default     => "sym",
            description => 'Specifies the allocation method and otionally '
                . 'number of volumes per vfiler rnd, sym, rnd:n or sym:n',
        },
        {   name        => 'VSERVER_COUNT',
            spec        => 's',
            default     => 1,
            description => 'The number of vservers to use',
        },
        {   name    => 'VFILER_REGEX',
            spec    => 's',
            default => undef,
            description =>
                'The regular expression used to match vfiler names',
        },
        {   name    => 'VSERVER_REGEX',
            spec    => 's',
            default => undef,
            description =>
                'The regular expression used to match vserver names',
        },
        {   name        => 'WAIT_FOR_FILE',
            spec        => 's',
            default     => undef,
            description => 'Halt program operation until this file is found',
        },
        {   name        => 'WAIT_FOR_REPLAY',
            spec        => 's',
            default     => '10,600',
            description => 'Wait this long for replays',
        },
        {   name        => 'USER_OR_GROUP_BASENAME',
            description => 'The basename of all the domain groups used',
            default     => undef,
        },
        {   name        => 'WINS_SERVERS',
            spec        => 's@',
            default     => undef,
            description => 'Comma seperated list of WINS server IP addresess',
        },
        {   name    => 'WAFL_DEFAULT_SECURITY_STYLE',
            spec    => 's',
            default => undef,
            description =>
                'The value for the option wafl.default_security_style',
        }
    ],
);
$Options->process();

my $Options_hash = $Options->get_hash();

if ($Options_hash->{'CLIENT_HOSTTYPE_FILTER'}) {
    die script_result(
        'CLIENT_HOSTTYPE_FILTER is no longer supported, please use CLIENT_HOSTTYPE_REGEX'
    );
}

if ($Options_hash->{'IO_NODE_FILTER'}) {
    die script_result(
        'IO_NODE_FILTER is no longer supported,  please use NODE_REGEX');
}

if ($Options_hash->{'IO_TOOLS_FILTER'}) {
    die script_result(
        'IO_TOOLS_FILTER is no longer supported, please use IO_TOOLS_REGEX');
}

if ($Options_hash->{'VFILER_ADDRESSES'}) {
    die script_result(
        'VFILER_ADDRESSES is no longer supported, please use DESTIP');
}

if ($Options_hash->{'VSERVER_ADDRESSES'}) {
    die script_result(
        'VSERVER_ADDRESSES is no longer supported, please use DESTIP');
}
1;
