#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NAS Protocol Topology for ConfigBroker
##
## @author      Jd dascenj@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::ConfigBroker - NAS Protocol Topology for ConfigBroker

=head1 SYNOPSIS

    See Topology.pm

=head1 DESCRIPTION

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Probably. 

=head1 STATUS

  Awesome

=head1 AUTHOR

  Jd (dascenj@netapp.com)
  $Id: $
  Last revised $Date: 2011 Mar 26 $
    
=head1 COPYRIGHT

Copyright 2011, Network Appliance., All Rights Reserved


=cut

package Topology::NAS::ConfigBrokerTopo;
use base qw(Topology);

use strict;

use ConfigBroker;
use Module::Common qw(validate_mixed);
use Params::Validate
    qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use SayWrap qw(sayEOF);
use Module::Load qw(load);
use Tharn qw($Log $Staf subtest config_result script_result fail_result host);
use XML::Simple;
use Sort::Key::Natural qw(natsort natkeysort);
use File::Basename;
use Data::Dumper qw(Dumper);
use Options;
use Hash::Merge qw(merge);

#use Array::Shuffle qw(shuffle_array);

my $Options;
my $Options_hash;
my @nodes;

###################################################################

$Options = new Options;
$Options->parameters(
    definitions => [
        {   name => "CB_DEFAULT_LU_PASSWD",
            spec => "s",
            description =>
                "The default user password for discovered local users",
            default => 'field:full_name'
        },
        {   name => "CB_DISCOVER",
            spec => "s",
            description =>
                "Delete the resources out of the CB database, and rediscovers",
            default => 0,
        },
        {   name        => "CB_POPULATE",
            spec        => "s",
            description => "Populates the appropriate cifs and nfs tables",
            default     => 0,
        },
        {   name        => "CB_PROTOCOLS",
            spec        => "s@",
            description => "The determines the correct protocols to use",
            default     => 'cifs,nfs',
        },
        {   name        => "CB_USERS",
            spec        => "s",
            description => "The key of the desired user key",
        },
        {   name        => "CB_USER_CONFIG",
            spec        => "s",
            description => "The file contaning user info to read into CB",
        },
        {   name => "CIFS_RG_CONFIG",
            spec => "s",
            description =>
                'Full or relative path to the file the defines the resource group layout',
            default =>
                'system_test/workload/client/cifs/default_cm_rg_config.xml',
        },
        {   name        => "CIFS_RG_FILTER",
            spec        => "s",
            description => 'Override resource group filters',
            default     => undef
        },
        {   name        => "CIFS_WORKLOAD_FILE",
            spec        => "s",
            description => 'Full or relative path to the CIFS workload file',
            default =>
                'system_test/workload/client/cifs/default_workload.xml',
        },
        {   name        => "COMM_LOG_LEVEL",
            spec        => "s",
            description => 'Controls how verbose NATE is',
            default     => 'debug'
        },
        {   name => "FAIL_NO_CB_INFO",
            spec => "s",
            description =>
                'Die if no CB infor is defined for any resource group',
        },
        {   name => "TML_MGR_START_TIME",
            spec => "s",
            description => 'The epoch time when define_topology has run',
        },
        {   name => "NFS_RG_CONFIG",
            spec => "s",
            description =>
                'Relative path to the file the defines the resource group layout',
            default =>
                'system_test/workload/client/nfs/default_cm_rg_config.xml',
        },
        {   name        => "NFS_RG_FILTER",
            spec        => "s",
            description => 'Override resource group filters',
            default     => undef
        },
        {   name        => "NFS_WORKLOAD_FILE",
            spec        => "s",
            description => 'Relative path to the CIFS workload file',
            default =>
                'system_test/workload/client/nfs/SN_public_workload.xml',
        },
        {   name => "SAN_RG_CONFIG",
            spec => "s",
            description =>
                'Relative path to the file the defines the resource group layout',
            default =>
                'system_test/workload/client/san/default_cm_rg_config.xml',
        },
        {   name        => "SAN_RG_FILTER",
            spec        => "s",
            description => 'Override resource group filters',
            default     => undef
        },
        {   name        => "SAN_WORKLOAD_FILE",
            spec        => "s",
            description => 'No user servicable parts inside',
            default     => undef,
        },
        {   name => "SAN_WORKLOAD_FILE_UNIX",
            spec => "s",
            description =>
                'Relative path to the SAN workload file for UNIX systems',
            default =>
                'system_test/workload/client/san/default_workload_unix.xml',
        },
        {   name => "SAN_WORKLOAD_FILE_WINDOWS",
            spec => "s",
            description =>
                'Relative path to the SAN workload file for Windows systems',
            default =>
                'system_test/workload/client/san/default_workload_windows.xml',
        },
        {   name => "SKTRACE_OPTIONS",
            spec => "s@",
            description =>
                "A list of values to be modified with the sysctl command. "
                . "Example: SKTRACE_OPTIONS=sysvar.sktrace.clear=1,sysvar.sktrace.Nfs4Lock_enable=63"
        },
        {   name => "TESTUSER_FILE",
            spec => "s",
            description =>
                "file containing the list of testusers who test run as",
            file_tests => "fr",
            subroutine => [\&read_testuser_file],
        },
        {   name        => "TESTUSER_PASSWORD",
            spec        => "s",
            description => "The password to use with TESTUSER",
            default     => undef,
        },
        {   name        => "TESTUSER",
            spec        => "s@",
            description => "List of test users to use for the testing",
            default     => undef,
        },
        {   name => "TML_WORKER_COUNT",
            spec => "s",
            description =>
                "The number of workers TML will create. Worker will "
                . "correspond to the thpl called (usually io.thpl)",
            default => 1,
        },
        {   name        => "TML_WORKER_INSTANCE",
            spec        => "s",
            description => 'The instance number passed to the worker',
            default     => undef,
        },
        {   name => "WORKLOAD_PARSER_LIB",
            spec => "s",
            description =>
                "The package name of the parser lib to use to load the "
                . "*_WORKLOAD_FILE. (the .pm file will need to be found in \@inc",
            default => 'SystemTest::WorkloadParser',
        }
    ]
);

sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;
    my $self     = {@_,};
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
} ## end sub new

sub define_topology {
    my $self = shift;
    $Log->enter();

    $Options_hash = $Options->get_hash();

    $Log->comment("Options: \n" . Dumper($Options_hash));

    # Obtain the node object
    require NACL::C::Node;
    @nodes = NACL::C::Node->find();

    $self->{cb}->{object}->{configbroker} = new ConfigBroker;
    my $cb = $self->get_cb_object('configbroker');

    # Keep the names of the filers around.
    foreach my $n (@nodes) {
        push(@{$self->{nodes}}, $n->hostrec->id());
    }

    $self->{clients} = $Options_hash->{'CLIENT'};

    my $node_name = $self->{nodes}->[0];
    $Log->comment(
        "The topology will utilize these filers: " . Dumper($self->{nodes}));

    $self->{cb}->{config} = {
        cifs => {
            'tables' => {
                activity => {
                    primarykey => ['starttime', 'name'],
                    schema     => [
                        'starttime',      'destip',
                        'cifs_server',    'volume',
                        'vserver',        'aggregate',
                        'aggregate_node', 'stoptime',
                        'client',         'credentials',
                        'workitem',       'result',
                        'sync_master',    'iteration',
                        'status'
                    ],
                    table => 'cifs_activity',
                },
                clients => {
                    primarykey => ['name'],
                    schema     => ['hosttype', 'os_ver', 'type'],
                    table      => 'cifs_clients'
                },
                resources => {
                    primarykey => ['name'],
                    schema     => [
                        'type',             'destip',
                        'destip_home',      'cifs_server',
                        'share',            'path',
                        'share_properties', 'vserver',
                        'auth_style',       'volume',
                        'aggregate',        'aggregate_node',
                        'lif_local_to_vol'
                    ],
                    table => 'cifspath'
                },
                users => {
                    primarykey => ['name'],
                    schema     => [
                        'id',      'user_name', 'password', 'ad',
                        'nis',     'host',      'hosttype', 'homedir',
                        'vserver', 'protocol'
                    ],
                    table => 'cifs_users'
                },
                workloads => {
                    primarykey => ['name'],
                    schema     => ['workload'],
                    table      => 'cifs_workloads'
                },
            },
            vstype => 'vservercifs',
        },
        nfs => {
            'tables' => {
                activity => {
                    primarykey => ['starttime', 'name'],
                    schema     => [
                        'starttime',   'destip',
                        'volume',      'vserver',
                        'aggregate',   'aggregate_node',
                        'stoptime',    'client',
                        'credentials', 'workitem',
                        'result',      'sync_master',
                        'iteration',   'status'
                    ],
                    table => 'nfs_activity',
                },
                clients => {
                    primarykey => ['name'],
                    schema     => ['hosttype', 'os_ver', 'type'],
                    table      => 'nfs_clients'
                },
                resources => {
                    primarykey => ['name'],
                    schema     => [
                        'type',           'junction_path',
                        'vserver',        'destip',
                        'volume',         'v3',
                        'v4',             'v4_1',
                        'udp',            'tcp',
                        'kerberos',       'aggregate',
                        'destpath',       'destip_home',
                        'aggregate_node', 'lif_local_to_vol',
                    ],
                    table => 'nfspath'
                },
                users => {
                    primarykey => ['name'],
                    schema     => [
                        'id',      'user_name', 'password', 'ad',
                        'nis',     'host',      'hosttype', 'homedir',
                        'vserver', 'protocol'
                    ],
                    table => 'nfs_users'
                },
                workloads => {
                    primarykey => ['name'],
                    schema     => ['workload'],
                    table      => 'nfs_workloads'
                },
            },
            vstype => 'vservernfs',
        },
        san => {
            'tables' => {
                activity => {
                    primarykey => ['starttime', 'name'],
                    schema     => [
                        'starttime',      'disk',
                        'device',         'uid',
                        'interface',      'partition',
                        'drive',          'destip',
                        'cifs_server',    'volume',
                        'vserver',        'aggregate',
                        'aggregate_node', 'stoptime',
                        'client',         'workitem',
                        'result',         'sync_master',
                        'iteration',      'status'
                    ],
                    table => 'san_activity',
                },
                clients => {
                    primarykey => ['name'],
                    schema     => ['hosttype', 'os_ver', 'type'],
                    table      => 'san_clients'
                },
                resources => {
                    primarykey => ['name'],
                    schema     => [
                        'type',        'client',
                        'disk',        'device',
                        'uid',         'interface',
                        'partition',   'drive',
                        'destip',      'destip_home',
                        'cifs_server', 'share',
                        'path',        'lun',
                        'vserver',     'volume',
                        'aggregate',   'aggregate_node'
                    ],
                    table => 'sanpath'
                },
                workloads => {
                    primarykey => ['name'],
                    schema     => ['workload', 'hosttype'],
                    table      => 'san_workloads'
                },
            },
            vstype => 'vserver',
        },
    };

    $self->{cb_protocols} = $Options_hash->{'CB_PROTOCOLS'};

    # Do the required groups exist? Do discovery if not.
    $Log->comment("Getting all resource groups");
    my @rgs = eval { $cb->get_all_resource_groups(); };

    # Repopulate the data the CIFS and NFS path resouce group
    if ($Options_hash->{CB_POPULATE}) {

        # Rediscover if there is no cluster resource group or CB_DISCOVER=1
        my $clus_rg;
        if ($Options_hash->{CB_DISCOVER} || (!grep { $_ eq 'cluster' } @rgs))
        {

            # This is the discovery phase
            $Log->comment("Deleting resource groups from ConfigBroker "
                    . "to prepare for discovery\n"
                    . Dumper(\@rgs));

            # We are going to rediscover, get rid of all the resources in CB,
            # as well as the resourcegroups
            foreach my $rg (@rgs) {
                eval { $rg->delete_resources_from_db(); };
            }

            eval { $cb->delete_all_resource_groups(); };

            # Rediscover the cluster and put into a single rg
            # Shutup the disco output.
            $Log->comment("Executing ConfigBroker discovery");
            local $NATE::Log::CommEvent::CommLogLevel
                = $Options_hash->{COMM_LOG_LEVEL};
            $clus_rg = $cb->discover_and_create_resource_group(
                groupname     => 'cluster',
                discoverusers => 1
            );
            local $NATE::Log::CommEvent::CommLogLevel = 'debug';
        } else {
            $Log->comment("Using the existing ConfigBroker database, "
                    . "skipping discovery");
            $clus_rg = $cb->get_resource_group(name => 'cluster');
        }
        $self->{cb}->{object}->{cluster} = $clus_rg;

        # Create the tables here becasue ConfigBroker sucks at squelching
        # DBI errors when a table does not exist. =/
        foreach my $proto ('cifs', 'nfs', 'san') {

            # Create the current protocol's tables
            foreach my $rgtype ($self->get_cb_tables($proto)) {

                # Get the relevant information for the current proto and rg
                # type
                my $rtschema = $self->get_cb_rtschema($proto, $rgtype);
                my $rtname = $self->get_cb_rtname($proto, $rgtype);
                my $rtpkey = $self->get_cb_rt_primarykey($proto, $rgtype);

                # The condition exists where you get duplicated entries
                # because of discovery. That is, if you have 10 resources that
                # are part of the the resourcegroup JD, and then you discover,
                # you will have the same 10 resources added to your table
                # (for a total of 20) in the default resource group. To get
                # around this, drop all the tables we are about to create.
                # This is likely okay, because we already don't trust CB. The
                # filer configuration may have changed, so these tables are
                # not realiable anyway.
                eval {
                    $Log->comment("Removing $rgtype table $rtname");
                    my $result = $cb->delete_resource_type(name => $rtname);
                };

                # Create the resource table
                $Log->comment("Creating $rgtype table $rtname");
                $cb->create_resource_type(
                    name       => $rtname,
                    primarykey => $rtpkey,
                    tabledefn  => $rtschema
                );

            }
        }

        # Read the resource group config
        my $vservers = {};
        foreach my $proto (@{$self->{cb_protocols}}) {

            # Determine the right resource group config to use
            my ($rgconf_files, $rg_filter);
            if ($proto eq 'cifs') {
                $rgconf_files = $Options_hash->{'CIFS_RG_CONFIG'};
                $rg_filter    = $Options_hash->{'CIFS_RG_FILTER'};
            } elsif ($proto eq 'nfs') {
                $rgconf_files = $Options_hash->{'NFS_RG_CONFIG'};
                $rg_filter    = $Options_hash->{'NFS_RG_FILTER'};
            } elsif ($proto eq 'san') {
                $rgconf_files = $Options_hash->{'SAN_RG_CONFIG'};
                $rg_filter    = $Options_hash->{'SAN_RG_FILTER'};
            } else {
                die script_result("We don't have a resource group "
                        . "config file for the $proto protocol");
            }

            # Take a comma separated list of resource group config and
            # combine then in to one hashref
            my $rgconf = {};

            # Create the XML::Simple object
            my $xml = new XML::Simple;

            # Split apart the values. Values can be in the form of
            # system_test/workload/client/cifs/rgfile.xml
            # system_test/workload/client/cifs:rgfile1.xml:rgfile2.xml
            my @files;
            my @values = split(/,/, $rgconf_files);
            foreach my $value (@values) {
                if ($value =~ /:/) {
                    my @fields = split(/:/, $value);
                    if (@fields < 2) {
                        die script_result("Invalid format for "
                                . uc($proto)
                                . "_RG_CONF. When specifying multiple files "
                                . "per path please specify "
                                . "relative/path:file1:file2 ")->log();
                    }
                    for (my $i = 1; $i < @fields; $i++) {
                        push(@files, "$fields[0]/$fields[$i]");
                    }
                } else {
                    push(@files, $value);
                }
            }

            # Pass #1 Get all the included files
            my $includes = {};
            for (my $f = 0; $f < @files; $f++) {
                my $relative_path = $files[$f];

                # Use NATE to find the real path
                $Log->comment(
                    "Searching for resource group config $relative_path");
                my $full_path = NATE::Inc::find_file($relative_path);
                if (!$full_path) {
                    die script_result("Cannot find $relative_path");
                }

                # Convert the XML file into a hashref
                my $h = $xml->XMLin("$full_path");

                # Store any includes for later processing
                $Log->comment("Processing $relative_path for includes");
                if (defined($h->{include})) {
                    for (my $i = 0; $i < @{$h->{include}}; $i++) {
                        $Log->comment("Including $h->{include}->[$i]");
                        push(
                            @{$includes->{$relative_path}},
                            $h->{include}->[$i]
                        );
                    }
                }
            }
            $Log->debug(Dumper($includes));

            # Sort all the files so we process them in the order that was
	    # given
            my @all_files;
            for (my $f = 0; $f < @files; $f++) {
                my $relative_path = $files[$f];
                push(@all_files, $relative_path);
                if (defined($includes->{$relative_path})) {
                    my @inc = @{$includes->{$relative_path}};
                    for (my $i = 0; $i < @inc; $i++) {
                        push(@all_files, $inc[$i]);
                    }
                }
            }
            $Log->debug(Dumper(\@all_files));

            # Pass #2 Merge all files
            for (my $f = 0; $f < @all_files; $f++) {
                my $relative_path = $all_files[$f];

                # Use NATE to find the real path
                $Log->comment(
                    "Searching for resource group config $relative_path");
                my $full_path = NATE::Inc::find_file($relative_path);
                if (!$full_path) {
                    die script_result("Cannot find $relative_path");
                } else {
                    $Log->comment("Creating "
                            . uc($proto)
                            . " resource groups from $relative_path");
                }

                # Convert the XML file into a hashref
                my $h = $xml->XMLin("$full_path");

                # Default to the left hash having precedence unless
                # otherwise specified
                my $precedence = 'LEFT_PRECEDENT';
                if (defined($h->{precedence})) {
                    $precedence = $h->{precedence};
                }

                # Merge the hashref with the resource group configuration
                # hashref
                my $merge = Hash::Merge->new($precedence);
                my $merged_hash = $merge->merge($rgconf, $h);
                $rgconf = $merged_hash;
            }

            # Delete crontrol keys that would be interpreted as resource
            # groups later on
            if (defined($rgconf->{include})) { delete($rgconf->{include}); }
            if (defined($rgconf->{precedence})) {
                delete($rgconf->{precedence});
            }

            # Here we allow people to pass in node names and client ranges per
            # resource group so they don't have to hard code them in
            # (CIFS|NFS)_RG_CONFIG
            my $format
                = '<name>:'
                . '<aggregate owner regex>:'
                . '<lif owner regex>:'
                . '<resource range>:'
                . '<workload regex>:'
                . '<client range>';

            if ($rg_filter) {
                my @filters = split(/,/, $rg_filter);
                foreach my $filter (@filters) {
                    my @fields = split(/:/, $filter);
                    if (@fields < 1 || @fields > 6) {
                        die script_result(
                                  "Invalid RG_FILTER format. Expected a "
                                . "minimum of 2 values in 6 fields, received "
                                . "$filter.\n Valid format: $format");
                    } ## end

                    my ($rgname, $aggr_owner, $lif_owner, $resource_range,
                        $workload_regex, $client_range)
                        = split(/:/, $filter);
                    if ($aggr_owner) {
                        $rgconf->{$rgname}->{path_filters}->{aggregate_node}
                            ->{match} = 'true';
                        $rgconf->{$rgname}->{path_filters}->{aggregate_node}
                            ->{regex} = $aggr_owner;
                    }
                    if ($lif_owner) {
                        $rgconf->{$rgname}->{path_filters}->{destip_home}
                            ->{match} = 'true';
                        $rgconf->{$rgname}->{path_filters}->{destip_home}
                            ->{regex} = $lif_owner;
                    }
                    if ($resource_range) {
                        $rgconf->{$rgname}->{resource_range}
                            = $resource_range;
                    }
                    if ($workload_regex) {
                        $rgconf->{$rgname}->{workload_regex}
                            = $workload_regex;
                    }
                    if ($client_range) {
                        $rgconf->{$rgname}->{client_range} = $client_range;
                    }
                }
            }
            $Log->comment(
                uc($proto) . " resource group config\n" . Dumper($rgconf));

            my @resource_groups = keys(%{$rgconf});
            foreach my $key (natsort(@resource_groups)) {
                $Log->comment("Working on resource group $key");
                $cb->create_resource_group(name => $key);

                # Create the resource group(s). The order of the steps is
                # dependent on the protocol. For SAN we get the info from the
                # clients and then match it up to the info from ONTAP.

                # Die if the current resource group does not have at least
                # one client, one resource and one workload and
                # FAIL_NO_CB_INFO is true
                my $workload_count     = 0;
                my $filtered_clients   = [];
                my $filtered_resources = [];
                if ($proto ne 'san') {

                    # Populate the <proto>path table with <proto> resources
                    # for this resource group
                    $filtered_resources
                        = $self->populate_cb_resources_table($proto, $key,
                        $rgconf);

                    if (@{$filtered_resources}) {

                        # Populate the <proto>_workload table with <proto>
                        # workloads for this resource group
                        $workload_count
                            = $self->populate_cb_workloads_table($proto, $key,
                            $rgconf);

                        # Die if there are no workloads for this resource
                        # group and FAIL_NO_CB_INFO is true
                        if (  !$workload_count
                            && $Options_hash->{FAIL_NO_CB_INFO}) {
                            my $msg
                                = "There were no workloads assigned to the "
                                . "$key resource group. Please check your "
                                . uc($proto)
                                . " resource group configuraton and filters";
                            die script_result($msg);
                        }

                        # Populate the <proto>_clients table with <proto>
                        # clients for this resource group
                        $filtered_clients
                            = $self->populate_cb_clients_table($proto, $key,
                            $rgconf);

                        # Die if there are no clients for this resource
                        # group and FAIL_NO_CB_INFO is true
                        if (!scalar(@{$filtered_clients})
                            && $Options_hash->{FAIL_NO_CB_INFO}) {
                            my $msg
                                = "There were no clients assigned to the "
                                . "$key resource group. Please check your "
                                . uc($proto)
                                . " resource group configuraton and filters";
                            die script_result($msg);
                        }
                    } else {

                        # Die if there are no resources for this resource
                        # group and FAIL_NO_CB_INFO is true
                        if ($Options_hash->{FAIL_NO_CB_INFO}) {
                            my $msg
                                = "There were no resources assigned to the "
                                . "$key resource group. Please check your "
                                . uc($proto)
                                . " resource group configuraton and filters";
                            die script_result($msg);
                        }
                    }
                } else {

                    # Populate the <proto>_clients table with <proto> clients
                    # for this resource group
                    $filtered_clients
                        = $self->populate_cb_clients_table($proto, $key,
                        $rgconf);

                    # Die if there are no clients for this resource group
                    # and FAIL_NO_CB_INFO is true
                    if (!scalar(@{$filtered_clients})
                        && $Options_hash->{FAIL_NO_CB_INFO}) {
                        my $msg
                            = "There were no clients assigned to the $key "
                            . "resource group. Please check your "
                            . uc($proto)
                            . " resource group configuraton and filters";
                        die script_result($msg);
                    }

                    # Populate the <proto>path table with <proto> resources
                    # for this resource group
                    $filtered_resources
                        = $self->populate_san_cb_resources_table($key,
                        $rgconf, $filtered_clients);

                    # Die if there are no resources for this resource group
                    # and FAIL_NO_CB_INFO is true
                    if (!scalar(@{$filtered_resources})
                        && $Options_hash->{FAIL_NO_CB_INFO}) {
                        my $msg
                            = "There were no resources assigned to the $key "
                            . "resource group. Please check your "
                            . uc($proto)
                            . " resource group configuraton and filters";
                        die script_result($msg);
                    }

                    # Populate the <proto>_workloads table with <proto>
                    # workloads for this resource group
                    $workload_count
                        = $self->populate_cb_workloads_table($proto, $key,
                        $rgconf);

                    # Die if there are no workloads for this resource group
                    # and FAIL_NO_CB_INFO is true
                    if (!$workload_count && $Options_hash->{FAIL_NO_CB_INFO})
                    {
                        my $msg
                            = "There were no workloads assigned to the $key "
                            . "resource group. Please check your "
                            . uc($proto)
                            . " resource group configuraton and filters";
                        die script_result($msg);
                    }
                }

                # Putting this here in case some one wants to enable users
                # for SAN
                if (   $Options_hash->{CB_USER_CONFIG}
                    && $Options_hash->{CB_USERS}
                    && $proto =~ /cifs|nfs/) {

                    # Populate the <proto>_clients table with <proto>
                    # clients for this resource group
                    $self->populate_cb_users_table($proto, $key, $rgconf);
                }
            }
        }
    }

    foreach my $proto (@{$self->{cb_protocols}}) {

        my $activitytablename = $proto . '_activity';
        $Log->comment("Creating resource group for $activitytablename");
        my $rg = $cb->create_resource_group(
            name   => $activitytablename,
            share  => 'SHARE',
            status => 'READY',
            reuse  => 'REUSE'
        );

        # We have to enter a row to make the resourcegroup stick around
        my $row;
        if ($proto eq 'cifs') {
            $row = {
                aggregate      => 'TheBeginning',
                aggregate_node => 'TheBeginning',
                cifs_server    => 'TheBeginning',
                client         => 'TheBeginning',
                cluster        => 'TheBeginning',
                credentials    => 'TheBeginning',
                destip         => 'TheBeginning',
                iteration      => 'TheBeginning',
                name           => 'ConfigBrokerTopo' . uc($proto),
                result         => '',
                starttime      => time(),
                status         => 'TheBeginning',
                stoptime       => time(),
                sync_master    => 'TheBeginning',
                volume         => 'TheBeginning',
                vserver        => 'TheBeginning',
                workitem       => 'TheBeginning',
            };
        } elsif ($proto eq 'nfs') {
            $row = {
                aggregate      => 'TheBeginning',
                aggregate_node => 'TheBeginning',
                cifs_server    => 'TheBeginning',
                client         => 'TheBeginning',
                cluster        => 'TheBeginning',
                credentials    => 'TheBeginning',
                destip         => 'TheBeginning',
                iteration      => 'TheBeginning',
                name           => 'ConfigBrokerTopo' . uc($proto),
                result         => '',
                starttime      => time(),
                status         => 'TheBeginning',
                stoptime       => time(),
                sync_master    => 'TheBeginning',
                volume         => 'TheBeginning',
                vserver        => 'TheBeginning',
                workitem       => 'TheBeginning',
            };
        } else {
            $row = {
                aggregate      => 'TheBeginning',
                aggregate_node => 'TheBeginning',
                cifs_server    => 'TheBeginning',
                client         => 'TheBeginning',
                cluster        => 'TheBeginning',
                credentials    => 'TheBeginning',
                destip         => 'TheBeginning',
                device         => 'TheBeginning',
                disk           => 'TheBeginning',
                drive          => 'TheBeginning',
                interface      => 'TheBeginning',
                iteration      => 'TheBeginning',
                name           => 'ConfigBrokerTopo' . uc($proto),
                partition      => 'TheBeginning',
                result         => '',
                starttime      => time(),
                status         => 'TheBeginning',
                stoptime       => time(),
                sync_master    => 'TheBeginning',
                volume         => 'TheBeginning',
                vserver        => 'TheBeginning',
                workitem       => 'TheBeginning',
            };
        }

        $rg->add_resources(
            resource_type    => $activitytablename,
            resourcearrayref => [$row],
        );

        # Create the client_setup hash
        $self->create_client_setup($proto);

    }

    # Setup the instances
    $Log->debug(Dumper($self->{clients}));
    $Log->debug(Dumper($self->{clients_status}));
    $self->configure_instances();

    # Create the TML test options
    $self->create_test_options();

    # Get creative about logging all our information
    my $top_logdir = Tharn::param('TOP_LOGDIR');
    my $runid      = Tharn::param('RUNID');
    my $lonely_log = $top_logdir . '/' . $runid . '_config_info.log';
    if ($Log->may_debug()) {
        $Log->comment("define_topology Topology: \n" . Dumper($self),
            path => $lonely_log);

    } else {
        $Log->comment("filer_setup:" . Dumper($self->{filer_setup}),
            path => $lonely_log);
        $Log->comment("instances:" . Dumper($self->{instances}),
            path => $lonely_log);
        $Log->comment("client_config:" . Dumper($self->{client_config}),
            path => $lonely_log);
        $Log->comment("test_options:" . Dumper($self->{test_options}),
            path => $lonely_log);
    }

    # Configure the sktracing if it is applicable. Normally something like
    # this should be placed in filer setup. However, I have decided to stick
    # it here in define_topology because I want to be sure that sktracing is
    # set correctly even when the topology is run with SKIPSTAGE=filer_setup
    # to use an existing setup
    local $NATE::Log::CommEvent::CommLogLevel
        = $Options_hash->{COMM_LOG_LEVEL};
    $self->sktrace_setup();
    local $NATE::Log::CommEvent::CommLogLevel = 'debug';

    # Create the TOP_LOGDIR/clients dir so the clients don't reset each other
    # permissions to it
    mkdir("$top_logdir/clients");
    Tharn::sleep 1;
    chmod(0777, "$top_logdir/clients");
} ## end sub define_topology

sub get_cb_tables {
    my ($self, $proto) = @_;
    return keys(%{$self->{cb}->{config}->{$proto}->{tables}});
}

sub get_cb_object {
    my ($self, $name, $type) = @_;
    if ($name eq 'cluster' || $name eq 'configbroker') {
        if (defined($self->{cb}->{object}->{$name})) {
            return $self->{cb}->{object}->{$name};
        }
    } elsif (defined($self->{cb}->{object}->{$name}->{$type})) {
        return $self->{cb}->{object}->{$name}->{$type};
    } else {
        return;
    }
} ## end sub get_cb_object

sub get_cb_rtname {
    my ($self, $proto, $type) = @_;
    return $self->{cb}->{config}->{$proto}->{tables}->{$type}->{'table'};
}

sub get_cb_rtschema {
    my ($self, $proto, $type) = @_;
    return $self->{cb}->{config}->{$proto}->{tables}->{$type}->{'schema'};
}

sub get_cb_rt_primarykey {
    my ($self, $proto, $type) = @_;
    my $pk
        = $self->{cb}->{config}->{$proto}->{tables}->{$type}->{'primarykey'};

    # set a default
    $pk = ["name", "resourcegroup"] if (not $pk);

    return $pk;
} ## end sub get_cb_rt_primarykey

sub get_cb_vstype {
    my ($self, $proto) = @_;
    return $self->{cb}->{config}->{$proto}->{vstype};
}

sub populate_cb_clients_table {
    my ($self, $proto, $rgname, $rgconf) = @_;

    my $cb     = $self->get_cb_object('configbroker');
    my $rg     = $cb->get_resource_group(name => $rgname);
    my $rtname = $self->get_cb_rtname($proto, 'clients');
    my @clients;

    foreach my $client ($self->clients()) {
        my $h = eval { host($client); };
        if ($@) { next; }
        my $hosttype = $h->hosttype();

        if ($proto eq 'cifs' && $hosttype !~ /win/) { next; }
        if ($proto eq 'nfs' && $hosttype =~ /win/) { next; }
        push(@clients, $client);
    }

    my $range = $rgconf->{$rgname}->{client_range};
    if (!$range) { $range = '1-100%'; }
    my $filter = '.*';
    if ($range =~ /^(\w+):(\d+.+)/) {
        $filter = $1;
        $range  = $2;
    }
    my ($start, $end)
        = SystemTest::Populate::parse_range($range, scalar(@clients));
    $Log->comment(
        "Limiting client range to $range($start-$end) of clients matching /$filter/"
    );

    # Walk the list of resources and filter by range
    my $client_count = 0;
    my @filtered_clients;
    foreach my $client (@clients) {
        my $h = eval { host($client); };
        if ($@) { next; }

        my $hosttype = $h->hosttype();
        if ($hosttype !~ /$filter/) { next; }

        my %fields = $h->param_fields();
        my $os_ver = $fields{OS_VER};

        $client_count++;
        if (   $client_count < $start
            || $client_count > $end) {
            next;
        }

        my $row = {
            name     => $client,
            hosttype => $hosttype,
            type     => $rtname,
            os_ver   => $os_ver,
        };

        $rg->add_resources(
            resource_type    => $rtname,
            resourcearrayref => [$row],
        );
        push(@filtered_clients, $client);
    }
    return \@filtered_clients;
} ## end sub populate_cb_clients_table

sub populate_cb_resources_table {

    my ($self, $proto, $rgname, $rgconf) = @_;
    my @objects;
    $Log->comment(
        "Populating resources for " . uc($proto) . " resource group $rgname");

    # Get the cluster resource group. (Which actually contains all clusters)
    my $clus_rg = $self->get_cb_object('cluster');

    $Log->comment("Creating resource group for $rgname");
    my $cb = $self->get_cb_object('configbroker');
    my $rg = $cb->create_resource_group(name => $rgname);

    # Get all vservers configured for the current protocol, and on the
    # correct cluster
    # Query ConfigBroker for the the vservers using the supplied filters
    my $args = {
        type          => 'vserver',
        match_filters => {type => 'data'}
    };
    my $vservers = $clus_rg->get_resources($args);

    # Warn if no vservers
    if (!defined($vservers)) {
        $Log->warn("There are no " . uc($proto) . " vservers available");
        return;
    }
    $Log->comment("Found " . @{$vservers} . " vservers");

    foreach my $vserver (@{$vservers}) {

        # Get The name of the current vserver
        my $vserver_name = $vserver->{name};

        # Populate the current protocols obj
        my $obj_vserver;
        my $rtschema = $self->get_cb_rtschema($proto, 'resources');
        my $rtname = $self->get_cb_rtname($proto, 'resources');
        foreach my $key (@$rtschema) {
            $obj_vserver->{$key} = "";
        }
        $obj_vserver->{type}    = $rtname;
        $obj_vserver->{vserver} = $vserver_name;

        # Add vserver specific obj_vserver info
        if ($proto eq 'nfs') {
            my $proto_vserver = $clus_rg->get_resources(
                type          => $self->get_cb_vstype($proto),
                match_filters => {vserver => $vserver_name},
            );

            $obj_vserver->{v3}               = $proto_vserver->[0]->{v3};
            $obj_vserver->{v4}               = $proto_vserver->[0]->{v4};
            $obj_vserver->{v4_1}             = $proto_vserver->[0]->{v4_1};
            $obj_vserver->{tcp}              = $proto_vserver->[0]->{tcp};
            $obj_vserver->{udp}              = $proto_vserver->[0]->{udp};
            $obj_vserver->{lif_local_to_vol} = 0;
        } else {

            # Get the disacovered CIFS info so we can know the cifs_server
            my $cifs_info = $clus_rg->get_resources(
                type          => 'vservercifs',
                match_filters => {vserver => $vserver_name}
            );
            $obj_vserver->{auth_style}   = $cifs_info->[0]->{auth_style};
            $obj_vserver->{cifs_server}  = $cifs_info->[0]->{cifs_server};
            $obj_vserver->{default_site} = $cifs_info->[0]->{default_site};
            $obj_vserver->{domain}       = $cifs_info->[0]->{domain};
            $obj_vserver->{domain_workgroup}
                = $cifs_info->[0]->{domain_workgroup};

        }

        # Find all the addresses associated with this vserver
        $Log->debug(
            "Finding all addresses associated with vserver $vserver_name");

        # Query ConfigBroker for the LIFs using the supplied filters
        $args                             = {};
        $args                             = {type => 'networkinterface'};
        $args->{match_filters}->{vserver} = $vserver_name;
        $args->{match_filters}->{role}    = 'data';
        SystemTest::Populate::get_resource_filters($rgconf, $rgname, 'lif',
            $args);

        my $lifs = $clus_rg->get_resources($args);
        if (!@{$lifs}) {
            my $filters = $rgconf->{$rgname}->{resource_filters}->{lif};
            $Log->warn("No LIFs found for vserver $vserver_name "
                    . "using the following filters\n"
                    . Dumper($filters));
            next;
        } else {
            $Log->comment(
                "Found " . @{$lifs} . " LIFs for vserver $vserver_name");
        }

        # Find the KRB config for all LIFS of this vserver
        $Log->debug("Finding the kerberos config of vserver $vserver_name");

       # Query ConfigBroker for the kerberos-config using the supplied filters
        $args = {};
        $args = {type => 'vservernfskerberosconfig'};
        $args->{match_filters}->{vserver} = $vserver_name;
        my $krb_config = $clus_rg->get_resources($args);
        $Log->debug(Dumper($krb_config));

        if (!@{$krb_config}) {
            $Log->comment(
                "No kerberos-config found for vserver $vserver_name");
            $krb_config = [];
        }

        # We want the junction path values from all volumes
        $Log->debug("Finding volumes associated wth vserver $vserver_name");

        # Query ConfigBroker for the volumes using the supplied filters
        $args                             = {};
        $args                             = {type => 'volume'};
        $args->{match_filters}->{vserver} = $vserver_name;
        SystemTest::Populate::get_resource_filters($rgconf, $rgname, 'volume',
            $args);
        my $filtered_volumes = $clus_rg->get_resources($args);

        if (!@{$filtered_volumes}) {
            my $filters = $rgconf->{$rgname}->{resource_filters}->{'volume'};
            $Log->warn("No volumes found for vserver $vserver_name "
                    . "using the following filters\n"
                    . Dumper($filters));
            next;
        } else {
            $Log->comment("Found "
                    . @{$filtered_volumes}
                    . " volumes for vserver $vserver_name");
        }

        # Query ConfigBroker for all volumes
        $args                             = {};
        $args                             = {type => 'volume'};
        $args->{match_filters}->{vserver} = $vserver_name;
        my $volumes = $clus_rg->get_resources($args);

        # Query ConfigBroker for the audit paths
        $Log->debug("Finding audit paths for vserver $vserver_name");
        $args                             = {};
        $args                             = {type => 'vserveraudit'};
        $args->{match_filters}->{vserver} = $vserver_name;
        my $audit = $clus_rg->get_resources($args);
        $Log->debug(Dumper($audit));

        if (!defined($audit)) {
            $Log->comment("Audit not configured for vserver $vserver_name");
        }

        # Create a blacklist of volumes to ignore
        my @volume_blacklist;
        foreach my $a (@{$audit}) {
            my $matches = SystemTest::Populate::get_volume_from_path(
                $a->{destination}, $volumes);
            if (defined($matches->[0]->{volume})) {
                push(@volume_blacklist, $matches->[0]->{volume});
            }
        }

        # Query ConfigBroker for the homedir paths
        $Log->debug("Finding homedir paths for vserver $vserver_name");
        $args = {};
        $args = {type => 'vservercifshomedirectorysearchpath'};
        $args->{match_filters}->{vserver} = $vserver_name;
        my $homedirs = $clus_rg->get_resources($args);
        $Log->debug(Dumper($homedirs));

        if (!defined($homedirs)) {
            $Log->comemnt(
                "Home directories not configured for vserver $vserver_name");
        }

        foreach my $h (@{$homedirs}) {
            my $matches
                = SystemTest::Populate::get_volume_from_path($h->{path},
                $volumes);
            if (defined($matches->[0]->{volume})) {
                push(@volume_blacklist, $matches->[0]->{volume});
            }
        }

        my @storage_objects;
        if ($proto eq 'cifs') {
            $Log->debug(
                "Finding shares associated with vserver $vserver_name");

            # Query ConfigBroker for the volumes using the supplied filters
            $args                             = {};
            $args                             = {type => 'vservercifsshare'};
            $args->{match_filters}->{vserver} = $vserver_name;
            SystemTest::Populate::get_resource_filters($rgconf, $rgname,
                'share', $args);
            my $shares = $clus_rg->get_resources($args);

            if (!@{$shares}) {
                my $filters
                    = $rgconf->{$rgname}->{resource_filters}->{'share'};
                $Log->warn("No shares found for vserver $vserver_name "
                        . "using the following filters\n"
                        . Dumper($filters));
                next;
            } else {
                $Log->comment("Found "
                        . @{$shares}
                        . " shares on vserver $vserver_name");
            }
            @storage_objects = @{$shares};
        } else {

            # Process the filtered volumes
            foreach my $v (@{$filtered_volumes}) {

                # Ignore blacklisted volumes
                if (grep { $_ eq $v->{volume} } @volume_blacklist) {
                    $Log->comment("Skipping blacklisted volume $v->{volume}");
                    next;
                }

                # Store in the generalized storage objects array
                push(@storage_objects, $v);

                # Print the destpath if debug
                $Log->debug("$v->{volume}:$v->{junction_path}");
            }
        }

        bless $obj_vserver;

        # Add volume specific obj_vserver info
        foreach my $object (@storage_objects) {

            $Log->debug("Working on " . Dumper($object));
            if ($proto eq 'nfs' && !$object->{junction_path}) {

                # Skip volumes without a junction path
                $Log->warn("volume has no junction-path definition");
                next;
            }

            if ($proto eq 'cifs' && !$object->{path}) {

                # Skip share without a path
                $Log->warn("share has no path definition");
                next;
            }

            # Copy the obj_vserver
            my $obj_storage = Storable::dclone $obj_vserver;

            # If the obj contains junction_path
            if ($proto eq 'nfs' && defined($obj_storage->{junction_path})) {
                $obj_storage->{junction_path} = $object->{junction_path};
            }

            if ($proto eq 'nfs' && defined($obj_storage->{volume})) {
                $obj_storage->{volume} = $object->{volume};
            }

            if ($proto eq 'nfs' && defined($obj_storage->{aggregate})) {
                $obj_storage->{aggregate} = $object->{aggregate};
            }

            if ($proto eq 'cifs' && defined($obj_storage->{share})) {
                $obj_storage->{share} = $object->{name};
            }

            if ($proto eq 'cifs' && defined($obj_storage->{path})) {
                $obj_storage->{path} = $object->{path};
            }

            if ($proto eq 'cifs'
                && defined($obj_storage->{share_properties})) {
                $obj_storage->{share_properties}
                    = $object->{share_properties};
            }

            # Ignore homedir shares since these could be on any volume.
            my $populate_storage = 1;
            if (   $proto eq 'cifs'
                && defined($obj_storage->{share})
                && $obj_storage->{share} =~ /%/) {
                $populate_storage = 0;
            }

            # For CIFS shares get the volume that hosts the share
            # path
            if (   $proto eq 'cifs'
                && defined($obj_storage->{share})
                && $populate_storage) {
                my $matches = SystemTest::Populate::get_volume_from_path(
                    $obj_storage->{path}, $filtered_volumes);
                if (defined($matches->[0]->{volume})) {
                    if (@{$matches} > 1) {
                        my %matched;
                        foreach my $m (@{$matches}) {
                            $matched{$m->{volume}} = $m->{junction_path};
                        }
                        $Log->warn("The path for $obj_storage->{share} "
                                . "matches multiple junction paths, using "
                                . "the first we find\n"
                                . Dumper(\%matched));
                    }
                    $obj_storage->{volume} = $matches->[0]->{volume};
                } else {
                    $Log->warn("Skipping the share $obj_storage->{share} "
                            . "because it\'s path did not match\nany "
                            . "juntion paths in the list of volumes found "
                            . "by your volume\nresource filters. This could "
                            . "be by design for the current test\ncase or "
                            . "signify the need to review your resource\n"
                            . "filters.");

                    $Log->debug(
                              "Searching for matches to $obj_storage->{path} "
                            . "in all junction paths");
                    $matches = SystemTest::Populate::get_volume_from_path(
                        $obj_storage->{path}, $volumes);
                    if (@{$matches}) {
                        my %matched;
                        foreach my $m (@{$matches}) {
                            $matched{$m->{volume}} = $m->{junction_path};
                        }
                        $Log->warn("The path for $obj_storage->{share} "
                                . "matches the following junction paths\n"
                                . "not found by the current resource "
                                . "filters\n"
                                . Dumper(\%matched));
                    }
                    next;
                }
            }

            # If we are dealing with a homedir share, don't get aggregate
            # since we won't know that until we know the user
            if (defined($obj_storage->{aggregate}) && $populate_storage) {
                foreach my $vol (@{$volumes}) {
                    if ($vol->{volume} eq $obj_storage->{volume}) {
                        $obj_storage->{aggregate} = $vol->{aggregate};
                        last;
                    }
                }
            }

            if ($populate_storage) {

                # Skip this entry if there is no volume information
                if (!$obj_storage->{volume}) { next; }

                # Get aggregate information if the current volume is
                # not an IV
                my @matches
                    = grep { $_->{'volume'} eq $obj_storage->{volume} }
                    @{$volumes};
                my $volume_style = $matches[0]->{volume_style};

                if ($volume_style !~ /infi/) {
                    if (!$obj_storage->{aggregate}) { next; }

                    # To figure out the node where the volume lives, we need
                    # to look at the aggregate. Go get it from CB.
                    my $aggregates = $clus_rg->get_resources(
                        type          => 'aggregate',
                        match_filters => {name => $obj_storage->{aggregate},}
                    );

                    if (!@{$aggregates}) {
                        $Log->warn("ConfigBroker has no aggregate named "
                                . "$obj_storage->{aggregate}, "
                                . "hosting volume $object->{volume}");
                    }

                    if (@{$aggregates} > 1) {
                        $Log->warn("Weird. Found "
                                . @{$aggregates}
                                . " aggregates named $obj_storage->{aggregate}."
                        );
                    }

                    if (defined($obj_storage->{aggregate_node})) {
                        $obj_storage->{aggregate_node}
                            = $aggregates->[0]->{nodes};
                    }
                }
            }

            bless $obj_storage;
            foreach my $lif (@{$lifs}) {

                # must have address
                if (!$lif->{address}) {
                    $Log->warn("LIF has no address defined");
                    next;
                }

                # Copy once more, all the data above is still valid.
                #require Storable;
                my $obj_lif = Storable::dclone $obj_storage;

                # What do you know now?
                if (defined($obj_lif->{destip})) {
                    $obj_lif->{destip} = $lif->{address};
                }

                if (defined($obj_lif->{destip_home})) {
                    $obj_lif->{destip_home} = $lif->{home_node};
                }

                # Do some figgerin, but use facts instead of advice
                # from celebrities...
                if (defined($obj_lif->{destpath})) {
                    $obj_lif->{destpath} = $obj_lif->{destip} . ':'
                        . $obj_lif->{junction_path};
                }

                #  The DB needs a name as the primary key
                if ($obj_lif->{destip_home}
                    =~ /^$obj_lif->{aggregate_node}$/i) {
                    $obj_lif->{lif_local_to_vol} = 1;
                }

                if ($proto eq 'cifs') {
                    $obj_lif->{name}
                        = $obj_lif->{destip} . ':'
                        . $obj_lif->{vserver} . ':'
                        . $obj_lif->{cifs_server} . ':'
                        . $obj_lif->{share} . ':'
                        . $obj_lif->{aggregate};
                }

                if ($proto eq 'nfs') {
                    $obj_lif->{name}
                        = $obj_lif->{destip} . ':'
                        . $obj_lif->{vserver} . ':'
                        . $obj_lif->{volume} . ':'
                        . $obj_lif->{aggregate};

                    # If our LIF has kerberos enabled set kerberos = enabled
                    # and store the SPN

                    if (grep {
                                   $_->{lif} eq $lif->{lif}
                                && $_->{kerberos} eq 'enabled'
                        } @{$krb_config}
                        ) {
                        $obj_lif->{kerberos} = 'enabled';
                    }
                }

                # Check to make sure all fields on this object are
                # defined. For NBAliases support we can have the scenario
                # where destip and destip are undef
                my @undef;
                foreach my $key (@$rtschema) {

                    # Don't check this key because it might be zero
                    next if ($key eq 'lif_local_to_vol');

                    # Don't die if the value for aggregate_node is undef.
                    # Infinite volumes do not have a aggregate
                    next if ($key eq 'aggregate_node');

                    # Don't die if the value for kerberos is undef.
                    next if ($key eq 'kerberos');

                    # Don't die if the value for destip is undef
                    # but the value for cifs_server is defined
                    next
                        if ($key eq 'destip'
                        && !$obj_lif->{destip}
                        && $obj_lif->{cifs_server});

                    # Don't die if the value for destip_home is undef
                    # but the value for cifs_server is defined
                    next
                        if ($key eq 'destip_home'
                        && !$obj_lif->{destip_home}
                        && $obj_lif->{cifs_server});

                    # Don't die if the value for storage fields are
                    # undef but the value for share is a homedir share
                    next
                        if ($key =~ /aggregate|volume/
                        && $obj_lif->{share} =~ /%/);

                    if (!$obj_lif->{$key}) { push(@undef, $key); }
                }

                # Warn if some of the fields are undef
                if (@undef) {
                    $Log->warn("Path not complete! Udefined Keys: @undef");
                }

                $Log->debug("Filtering:\n" . Dumper($obj_lif));

                # Filter out path table entries. If there are path filters
                # and one of them does not evaulate to true, skip the resource
                my $skip         = 0;
                my $path_filters = $rgconf->{$rgname}->{path_filters};
                foreach my $key (keys(%{$path_filters})) {
                    my $match
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{match};
                    my $regex
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{regex};
                    if ($match eq 'true' && $obj_lif->{$key} !~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $obj_lif->{name} $key($obj_lif->{$key}) !~ /$regex/"
                        );
                        last;
                    }
                    if ($match eq 'false' && $obj_lif->{$key} =~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $obj_lif->{name} $key($obj_lif->{$key}) =~ /$regex/"
                        );
                        last;
                    }
                }

                if ($skip && keys(%{$path_filters})) { next; }
                push(@objects, $obj_lif);
            }
        }
    }

    my $range = $rgconf->{$rgname}->{resource_range};
    if (!$range) { $range = '1-100%'; }
    my ($start, $end)
        = SystemTest::Populate::parse_range($range, scalar(@objects));

    # Walk the list of resources and filter by range
    my $resource_count     = 0;
    my $filtered_resources = [];
    foreach my $obj (@objects) {
        $resource_count++;
        if (   $resource_count >= $start
            && $resource_count <= $end) {
            $Log->debug("Adding resource $resource_count "
                    . "$obj->{name} to resource group $rgname");
            push(@{$filtered_resources}, $obj);
        }
    }

    my $rtname = $self->get_cb_rtname($proto, 'resources');
    if (@{$filtered_resources} > 0) {
        $Log->comment("Adding "
                . @{$filtered_resources}
                . " $rtname objects to ConfigBroker");
        $rg->add_resources(
            resource_type    => $rtname,
            resourcearrayref => $filtered_resources
        );
    } else {
        $Log->comment("No $rtname objects found");
    }
    return $filtered_resources;
} ## end sub populate_cb_resources_table

sub populate_cb_workloads_table {
    my ($self, $proto, $rgname, $rgconf) = @_;

    my $cb             = $self->get_cb_object('configbroker');
    my $rg             = $cb->get_resource_group(name => $rgname);
    my $rtname         = $self->get_cb_rtname($proto, 'workloads');
    my $workload_regex = $rgconf->{$rgname}->{workload_regex};

    my ($file, @files, @files_hosttype);
    if ($proto eq 'cifs') {
        $file = $Options->get(option => "CIFS_WORKLOAD_FILE");
        push(@files, $file);
    }
    if ($proto eq 'nfs') {
        $file = $Options->get(option => "NFS_WORKLOAD_FILE");
        push(@files, $file);
    }
    if ($proto eq 'san') {
        $file = $Options->get(option => "SAN_WORKLOAD_FILE_WINDOWS");
        if ($file) {
            push(@files,          $file);
            push(@files_hosttype, $file . "/wind");
        }
        $file = $Options->get(option => "SAN_WORKLOAD_FILE_UNIX");
        if ($file) {
            push(@files,          $file);
            push(@files_hosttype, $file . "/unix");
        }
    }

    my $workload_count = 0;
    my $file_count     = 0;
    foreach $file (@files) {
        my $filename = NATE::Inc::find_file($file);
        if (!$filename) {
            die script_result("Cannot file workload file $file");
        }

        my $wfpo = $Options->get(option => "WORKLOAD_PARSER_LIB");

        # No additional error checking is needed, the load will suicide
        # if it doesn't work
        load $wfpo;

        my $parser = $wfpo->new(filename => $filename);

        foreach my $wname (@{$parser->workloads()}) {
            my $rows;
            if ($workload_regex && $wname !~ /$workload_regex/) {
                next;
            }
            if ($proto eq 'san') {
                if ($files_hosttype[$file_count] =~ "win") {
                    $rows = {
                        name     => $wname,
                        workload => $wname,
                        hosttype => "windows"
                    };
                } else {
                    $rows = {
                        name     => $wname,
                        workload => $wname,
                        hosttype => "unix"
                    };
                }
            } else {
                $rows = {name => $wname, workload => $wname};
            }

            $rg->add_resources(
                resource_type    => $rtname,
                resourcearrayref => [$rows]
            );
            $workload_count++;
        }
        $file_count++;
    }
    return $workload_count;
} ## end sub populate_cb_workloads_table

sub filer_setup {

    # Setup is handled by ConfigBroker and setup engines such as engenx or CCL
}

sub filer_cleanup {

    #same as above...topology no longer handles this.
}

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    $Log->exit();
    return;
} ## end sub client_setup

sub DESTROY {
    my $self = shift;
    local ($@);

    $self->client_cleanup();
} ## end sub DESTROY

sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);

    $Log->exit();
} ## end sub client_cleanup

######################### Internal Routines ##################

my $File_read = undef;

sub read_testuser_file {
    my $tfile = shift;
    my $file
        = NATE::Inc::find_file($tfile)
        ? NATE::Inc::find_file($tfile)
        : $tfile;
    $Log->enter("file=>$file");
    if ($File_read) {
        $Log->trace("file already processed");
        goto done;
    }
    my $testuser = $Options->get(option => "TESTUSER");

    if (open(IF, $file)) {
        while (<IF>) {
            next if (/^\s*\#/);
            chomp;
            push(@{$testuser}, $_);
        }
        close(IF);
    } else {
        die "Could not open TESTUSER_FILE: $tfile ($file) : $! / $@";
    }

    $File_read = 1;

done:
    $Log->exit();
} ## end sub read_testuser_file

sub configure_instances {
    $Log->enter();
    my ($self) = @_;

    # Create the client_setup hash its just a very small static hash of a few
    # options that the supervisor will use. client_config contains the
    # relational setup of the clients <-> instances and everything else. I
    # wanted to use the client_setup area for this, but the way that the i
    # topology.pm interacts with that hash makes that too risky.
    my @clients;
    my $cb_protocols = $Options->get(option => "CB_PROTOCOLS");
    $Log->comment(
        "Creating client instances for the protocols: (@{$cb_protocols})");
    foreach my $client ($self->clients()) {
        my $host = Hostrec->new(id => $client);
        my $hosttype = $host->hosttype();
        if (grep(/cifs|san/i, @{$cb_protocols})
            && $hosttype =~ /win/) {
            $Log->comment("Adding $hosttype client $client");
            push(@clients, $client);
        }
        if (grep(/nfs|san/i, @{$cb_protocols}) && $hosttype !~ /win/) {
            $Log->comment("Adding $hosttype client $client");
            push(@clients, $client);
        }
    }

    my $instancesPerClient = $Options->get(option => "TML_WORKER_COUNT");
    my $testUsers          = $Options->get(option => "TESTUSER");
    my $test_password      = $Options->get(option => "TESTUSER_PASSWORD");

    # Only look at the IP addresses if this is being called as part
    # of the virtual topology. Use the number of supplied addresses
    # as the default number of synthetic clients to create.
    my $numAddrs;
    my ($package, $filename, $line) = caller;

    my $numberOfInstances = scalar(@clients) * $instancesPerClient;
    $Log->comment(
        "Setting the total number of client processes to $numberOfInstances."
    );
    $Log->comment("Creating $instancesPerClient process for each of the ("
            . scalar(@clients)
            . ") clients.");

    die config_result(
        "I was asked to create zero instances per client. Review configuration"
    ) if ($numberOfInstances == 0);

    $self->{_num_instances} = $numberOfInstances;

    # Spread all the instances among the clients.
    # Spread test users across test instances if given
    my $clientIndex = 0;
    my $userIndex   = 0;
    foreach my $instanceNumber (1 .. $numberOfInstances) {
        my $client       = $self->{client_config}->{$clients[$clientIndex]};
        my $host         = host($client->{name});
        my $instanceHash = {
            client       => $client,
            client_index => $clientIndex,
            name         => $instanceNumber
        };

       # Add the user ID's if we are going to run processes as different users
       # Cycle through the available user IDs
        if ($testUsers->[0]) {
            $instanceHash->{userName} = $testUsers->[$userIndex]
                if ($testUsers->[$userIndex]);
            $userIndex = 0 if (++$userIndex > scalar(@{$testUsers}));
            if (grep(/cifs/i, @{$cb_protocols})
                && $host->hosttype() =~ /win/) {
                $instanceHash->{password} = $test_password;
            }
        }

        # Keep all instances in a single place
        $self->{instances}->{$instanceNumber} = $instanceHash;

        # Link each instance to its physical client
        $self->{client_config}->{$clients[$clientIndex]}->{instances}
            ->{$instanceNumber} = $self->{instances}->{$instanceNumber};

        # Loop through the clients
        $clientIndex = 0 if (++$clientIndex > (scalar(@clients) - 1));

    }

    $Log->exit();
    return 1;
} ## end sub configure_instances

sub get_instances {
    my ($self) = @_;

    my @return = ();

    # Go through every instance, and find the ones using the current
    # usable volume
    foreach my $instance_name (keys(%{$self->{instances}})) {
        push(@return, $self->{instances}->{$instance_name});

    }
    return @return;
} ## end sub get_instances

sub create_client_setup {
    my ($self, $proto) = @_;

    my $addr_type = $Options_hash->{ADDR_TYPE};
    my $device    = $Options_hash->{DEVICE};
    foreach my $client (@{$self->{clients}}) {
        $self->{client_setup}->{$client}->{ADDR_TYPE}  = $addr_type;
        $self->{client_setup}->{$client}->{DEVICE}     = $device;
        $self->{client_setup}->{$client}->{CLIENTNAME} = $client;

        # All the relational stuff is in client_config
        $self->{client_config}->{$client}->{name} = $client;
    }

    return 1;
} ## end sub create_client_setup

#
# Just let the topology decide when this is a
# a good idea.
#
sub start_ip_manager {
    $Log->enter();
    my ($self) = @_;

    my $ipManager;
    my %argsForNew;

    # Because CMode_Virtual is derived from CMode,
    # The potential to start the IPManager twice exists.
    # Don't do that.
    if ($self->{ipManager}) {
        $ipManager = $self->{ipManager};

    } else {
        my $mode = $Options->get(option => "P_EXE_MODE");
        $argsForNew{start_messenger} = 0;

        $Log->trace(
            "Creating a new IPManager. The TestManagerLite mode is $mode.");

        # The IPManager should only be communicating with
        # the IPManager server when it is in manager mode.
        # supervisors get their information via Staph.
        if ($mode =~ /mgr/i) {
            $argsForNew{start_messenger} = 1;
        }

        # Start the client IP manager software
        $ipManager = Utils::IPManager::start_ip_manager(%argsForNew);
    }

    $Log->exit();
    return $ipManager;
} ## end sub start_ip_manager

sub sktrace_setup {
    my ($self) = shift;
    my %skHash;
    my @ops = @{$Options->get(option => "SKTRACE_OPTIONS")};
    my $enable = (scalar(@ops)) ? 1 : 0;

    $Log->comment("SKTRACE_OPTIONS: " . Dumper(\@ops));

    # set this on all nodes in use
    foreach my $node (@nodes) {
        my $bsd = NACL::APISet->new(
            category  => 'Node',
            interface => 'CLI',
            set       => 'Systemshell',
            hostobj   => $node->hostrec()
        );

        # The CDEFS for sysctl are less than awesome. Use the
        # execute_raw_command instead. We just always run cleanup to make sure
        # we are only getting the options that we want, just in case cleanup
        # fails or is skipped
        $Log->comment(
            "Cleaning up the sktrace options on " . $node->hostrec()->id());

        # This is cleanup, start with a dump to finish everything off

        # This little gem resets all your sktrace counters to zero. Don't
        # reset the Nfs4Info counters. They are on by default, leave them
        # that way.
        my $coolCommand
            = 'sudo sysctl sysvar.sktrace | '
            . 'grep _enable: | grep -vi Nfs4Info | '
            . 'grep -v \': 0\' | sed \'s/:.*//\' | '
            . 'xargs -I SYSCTL sudo sysctl SYSCTL=0';
        $bsd->execute_raw_command(command => $coolCommand);

        if ($enable) {
            $Log->comment("Setting up the sktrace options on "
                    . $node->hostrec()->id());

            # We are enabling. Start with a cleari. BURT i
            # http://web.netapp.com/~burt/views/498/498012.html
            # $bsd->execute_raw_command(command =>
            # "sudo sysctl sysvar.sktrace.clear=1");

            foreach my $op (@ops) {
                $bsd->execute_raw_command(command => "sudo sysctl " . $op);
            }
        }
    }
} ## end sub sktrace_setup

sub create_test_options {
    my ($self) = @_;

    # TML will die if these options are are null. Thanks medmonds! :)
    # use the test options to pass the name of the resource group we
    # created to the clients.
    foreach my $instance ($self->get_instances()) {
        my $c        = $instance->{client}->{name};
        my $i        = $instance->{name};
        my $username = $instance->{userName};
        my $password = $instance->{password};
        $self->{test_options}->{$c}->{$i}->{WORKLOAD_PARSER_LIB}
            = $Options->get(option => "WORKLOAD_PARSER_LIB");
        if (host($c)->hosttype() =~ /win/) {
            $self->{test_options}->{$c}->{$i}->{TESTUSER_PASSWORD}
                = $password;
            $self->{test_options}->{$c}->{$i}->{CIFS_WORKLOAD_FILE}
                = $Options->get(option => "CIFS_WORKLOAD_FILE");
            $self->{test_options}->{$c}->{$i}->{SAN_WORKLOAD_FILE}
                = $Options->get(option => "SAN_WORKLOAD_FILE_WINDOWS");
        }
        if (host($c)->hosttype() !~ /win/) {
            $self->{test_options}->{$c}->{$i}->{NFS_WORKLOAD_FILE}
                = $Options->get(option => "NFS_WORKLOAD_FILE");
            $self->{test_options}->{$c}->{$i}->{SAN_WORKLOAD_FILE}
                = $Options->get(option => "SAN_WORKLOAD_FILE_UNIX");

        }

        $self->{test_options}->{$c}->{$i}->{CLIENTNAME}          = $c;
        $self->{test_options}->{$c}->{$i}->{TESTUSER}            = $username;
        $self->{test_options}->{$c}->{$i}->{TML_WORKER_INSTANCE} = $i;

    }
} ## end sub create_test_options

sub get_nacl_nodes {
    my ($self) = @_;

    return \@nodes;
}

sub populate_san_cb_resources_table {
    my ($self, $rgname, $rgconf, $clients) = @_;

    my $cb      = $self->get_cb_object('configbroker');
    my $clus_rg = $self->get_cb_object('cluster');
    my $rg      = $cb->get_resource_group(name => $rgname);
    my $rtname  = $self->get_cb_rtname('san', 'resources');
    my $volumes = $clus_rg->get_resources(type => 'volume');

    # Walk the list of resources and filter by range
    my $client_count = 0;
    my $paths;
    my @resources;
    foreach my $client (@{$clients}) {
        my $h = eval { host($client); };
        if ($@) { next; }

        my $hosttype = $h->hosttype();
        my %fields   = $h->param_fields();
        my @objects;
        my $os_ver = $fields{OS_VER};

        if ($hosttype =~ /win/i) {

            my $host = Hostrec->new(id => $client);
            my $conn = $host->connect(
                conntype => "rsh-persay",
                id       => $client
            );

            my $disk_info
                = SystemTest::Populate::get_disk_info($hosttype, $conn,
                $client);
            $Log->debug("Disk Info\n" . Dumper($disk_info));
            my $hypervisor
                = SystemTest::Populate::get_hypervisor($hosttype, $conn,
                $client);
            $Log->debug("Hypervisor: $hypervisor");
            my $disk_info
                = SystemTest::Populate::get_disk_info($hosttype, $conn,
                $client, $hypervisor);
            $Log->debug("Disk Info\n" . Dumper($disk_info));
            my $hv_info;
            my @path_info;

            # We found a hypervisor. Lets process the paths returned for the
            # current client.
            if ($hypervisor) {
                $hypervisor =~ s/(^[^\.]+).+/$1/g;
                $hv_info = SystemTest::Populate::get_host_info($hypervisor,
                    $client);
                $Log->debug("VM Paths:\n" . Dumper($hv_info));
                foreach my $info (@{$hv_info}) {
                    my ($path,        $vserver,   $volume,
                        $server,      $share,     $destip,
                        $destip_home, $aggregate, $aggregate_node,
                        $lun
                    );

                    # Paths can be UNC or drive letters. Cluster Storage
                    # Volumes are not supported
                    #
                    # IDE \\10.226.184.71\s_vol_base100\vm_n...
                    # SCSI e:\\Win8-9200-Gold_iscsi_d.vhdx
                    if ($info->{'path'} =~ /^[\\]+([^\\]+)[\\]+([^\\]+)/) {
                        $server = $1;
                        $share  = $2;
                        my $share_info = $clus_rg->get_resources(
                            type          => 'vservercifsshare',
                            match_filters => {share_name => $share}
                        );
                        $vserver = $share_info->[0]{vserver};
                        $path    = $share_info->[0]{path};

                        if ($server =~ /\d+\.|:|literal/i) {
                            if ($server =~ /ipv6-literal\.net/i) {
                                $Log->debug(
                                    "Converting $server to IPv6 address");
                                $server =~ s/\.ipv6-literal\.net//i;
                                $server =~ s/-/:/g;
                            }

                            $destip = $server;

                            my $lif_info = $clus_rg->get_resources(
                                type          => 'networkinterface',
                                match_filters => {
                                    vserver => $vserver,
                                    address => $destip
                                }
                            );
                            $destip_home = $lif_info->[0]{home_node};
                        }

                        my $matches
                            = SystemTest::Populate::get_volume_from_path(
                            $path, $volumes);
                        if (defined($matches->[0]->{volume})) {
                            if (@{$matches} > 1) {
                                my %matched;
                                foreach my $m (@{$matches}) {
                                    $matched{$m->{volume}}
                                        = $m->{junction_path};
                                }
                                $Log->warn(
                                    "The path for $share matches multiple "
                                        . "junction paths, using the first we find"
                                        . Dumper(\%matched));
                            }
                            $volume = $matches->[0]->{volume};
                        } else {
                            $Log->warn(
                                "Skipping the share $share because it\'s "
                                    . "path did not match any juntion paths in "
                                    . "int the list of volumes found by your "
                                    . "volume resource filters.\nThis could be "
                                    . "by design for the current test case or "
                                    . "may signify the need to review your "
                                    . "resource filters.");

                            $Log->debug("Searching for matches to $path "
                                    . "in all junction paths");
                            $matches
                                = SystemTest::Populate::get_volume_from_path(
                                $path, $volumes);
                            if (@{$matches}) {
                                my %matched;
                                foreach my $m (@{$matches}) {
                                    $matched{$m->{volume}}
                                        = $m->{junction_path};
                                }
                                $Log->warn("The path for $share "
                                        . "matches the following junction paths\n"
                                        . "not found by the current resource "
                                        . "filters\n"
                                        . Dumper(\%matched));
                            }
                            next;
                        }
                    } elsif ($info->{'lun'}) {
                        $lun = $info->{lun};
                        my $lun_info = $clus_rg->get_resources(
                            type          => 'lun',
                            match_filters => {lun => $lun}
                        );
                        $vserver = $lun_info->[0]->{vserver};
                        $path    = $lun_info->[0]->{path};
                        $volume  = $lun_info->[0]->{volume};

                    }

                    foreach my $vol (@{$volumes}) {
                        if ($vol->{volume} eq $volume) {
                            $aggregate = $vol->{aggregate};
                            last;
                        }
                    }

                    # Get aggregate information if the current volume is not
                    # an IV
                    my @matches
                        = grep { $_->{'volume'} eq $volume } @{$volumes};
                    my $volume_style = $matches[0]->{volume_style};

                    if ($volume_style !~ /infi/) {

                        # To figure out the node where the volume lives, we
                        # need to look at the aggregate. Go get it from CB.
                        my $aggregates = $clus_rg->get_resources(
                            type          => 'aggregate',
                            match_filters => {name => $aggregate}
                        );

                        $aggregate_node = $aggregates->[0]->{nodes};
                    }
                    push(
                        @path_info,
                        {   type           => $info->{type},
                            destip         => $destip,
                            destip_home    => $destip_home,
                            share          => $share,
                            path           => $path,
                            lun            => $lun,
                            vserver        => $vserver,
                            volume         => $volume,
                            aggregate      => $aggregate,
                            aggregate_node => $aggregate_node
                        }
                    );
                }
                $Log->debug("Path Info:\n" . Dumper(\@path_info));
            }

            # This is for the Windows - Non Hypervisor (plain)
            else {
                my @keys_in_hash   = keys %$disk_info;
                my $number_of_keys = scalar(@keys_in_hash);
                my ($aggregate, $aggregate_node);
                for (my $looper = 0; $looper < $number_of_keys; $looper++) {
                    my $volume      = $disk_info->{$looper}->{volume};
                    my $vol_vserver = $disk_info->{$looper}->{vserver};
                    foreach my $vol (@{$volumes}) {

                        # The code here is slightly different from that in 
			# Windows Hypervisor.  This is because we can have 
			# a case where the Volume name may be the same. 
			# However the pair (Vserver, Volume_name) form a 
			# unique pair and hence this is used. This change 
			# has to be made for windows hypervisor also.
                        if (   ($vol->{volume} eq $volume)
                            && ($vol->{vserver} eq $vol_vserver)) {
                            $aggregate = $vol->{aggregate};

                            my $aggregates = $clus_rg->get_resources(
                                type          => 'aggregate',
                                match_filters => {name => $aggregate}
                            );
                            $aggregate_node = $aggregates->[0]->{nodes};
                            $disk_info->{$looper}->{aggregate} = $aggregate;
                            $disk_info->{$looper}->{aggregate_node}
                                = $aggregate_node;
                            last;
                        }
                        push(
                            @path_info,
                            {   type        => "",
                                destip      => "",
                                destip_home => "",
                                share       => "",
                                path        => $disk_info->{$looper}->{path},
                                lun         => $disk_info->{$looper}->{lun},
                                vserver => $disk_info->{$looper}->{vserver},
                                volume  => $disk_info->{$looper}->{volume},
                                aggregate =>
                                    $disk_info->{$looper}->{aggregate},
                                drive => $disk_info->{$looper}->{drive},
                                disk  => $disk_info->{$looper}->{disk},
                                proto => $disk_info->{$looper}->{proto},
                                aggregate_node =>
                                    $disk_info->{$looper}->{aggregate_node},
                            }
                        );
                    }

                }
                $Log->debug("Disk Info \n" . Dumper($disk_info));
            }

            my $rtschema = $self->get_cb_rtschema('san', 'resources');
            for (my $i = 0; $i < @path_info; $i++) {

                # If there are multiple VMs with the same name HyperV will
                # return multiple path entries. If there is no corresponding
                # disk info we should not add the row becasue there will be
                # missing info in the sanpath table
                if (!defined($disk_info->{$i})) { next; }

                my $row;
                foreach my $key (@$rtschema) {
                    $row->{$key} = '';
                }
                $row->{client}         = $client;
                $row->{disk}           = $disk_info->{$i}->{disk};
                $row->{device}         = $disk_info->{$i}->{id};
                $row->{interface}      = $disk_info->{$i}->{type};
                $row->{partition}      = $disk_info->{$i}->{partition};
                $row->{drive}          = $disk_info->{$i}->{drive};
                $row->{volume}         = $disk_info->{$i}->{volume};
                $row->{aggregate}      = $disk_info->{$i}->{aggregate};
                $row->{aggregate_node} = $disk_info->{$i}->{aggregate_node};
                $row->{vserver}        = $disk_info->{$i}->{vserver};
                $row->{path}           = $disk_info->{$i}->{path};
                $row->{interface}      = $disk_info->{$i}->{proto};
                $row->{lun}            = $disk_info->{$i}->{lun};
                $row->{name}
                    = $client . ':'
                    . $disk_info->{$i}->{disk} . ':'
                    . $disk_info->{$i}->{partition};

                if (defined($disk_info->{$i}->{vserver})) {
                    $row->{name} . ':' . $disk_info->{$i}->{vserver};
                }

                if (defined($disk_info->{$i}->{volume})) {
                    $row->{name} . ':' . $disk_info->{$i}->{volume};
                }

                if ($disk_info->{$i}->{type} eq $path_info[$i]->{type}) {
                    foreach my $key (@$rtschema) {
                        if (defined($path_info[$i]->{$key})) {
                            $row->{$key} = $path_info[$i]->{$key};
                        }
                    }
                }

                # Filter out path table entries. If there are path filters
                # and one of them does not evaulate to true, skip the resource
                my $skip         = 0;
                my $path_filters = $rgconf->{$rgname}->{path_filters};
                foreach my $key (keys(%{$path_filters})) {
                    my $match
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{match};
                    my $regex
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{regex};
                    if ($match eq 'true' && $row->{$key} !~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $row->{name} $key($row->{$key}) !~ /$regex/"
                        );
                        last;
                    }
                    if ($match eq 'false' && $row->{$key} =~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $row->{name} $key($row->{$key}) =~ /$regex/"
                        );
                        last;
                    }
                }

                if ($skip && keys(%{$path_filters})) { next; }
                $Log->debug("Row:\n" . Dumper($row));
                push(@objects, $row);

            }
        }

        # For Linux (Non-Hypervisor)
        elsif ($hosttype !~ /win/i) {
            my $host = Hostrec->new(id => $client);
            my $conn = $host->connect(
                conntype => "rsh-persay",
                id       => $client
            );

            my $disk_info
                = SystemTest::Populate::get_disk_info($hosttype, $conn,
                $client, "");
            $Log->debug("Disk Info\n" . Dumper($disk_info));
            my @populate_array;
            my @keys_in_hash   = keys %$disk_info;
            my $number_of_keys = scalar(@keys_in_hash);
            my $looper         = 0;
            my $volume;
            my $aggregate;
            my $aggregate_node;

            for ($looper = 0; $looper < $number_of_keys; $looper++) {
                my $volume      = $disk_info->{$looper}->{volume_id};
                my $vol_vserver = $disk_info->{$looper}->{vserver};
                foreach my $vol (@{$volumes}) {
                    my $lif_info = $clus_rg->get_resources(
                        type          => 'networkinterface',
                        match_filters => {
                            vserver => $disk_info->{$looper}->{vserver},
                            address => $disk_info->{$looper}->{ip_address}
                        }
                    );

                    # The code here is slightly different from that in Windows 
		    # Hypervisor.  This is because we can have a case where 
		    # the Volume name may be the same. However the pair 
		    # (Vserver, Volume_name) form a unique pair and hence this
                    # is used. This change has to be made for windows also.
                    if (   ($vol->{volume} eq $volume)
                        && ($vol->{vserver} eq $vol_vserver)) {
                        $aggregate = $vol->{aggregate};

                        my $aggregates = $clus_rg->get_resources(
                            type          => 'aggregate',
                            match_filters => {name => $aggregate}
                        );
                        my $lif_info = $clus_rg->get_resources(
                            type          => 'networkinterface',
                            match_filters => {
                                vserver => $disk_info->{$looper}->{vserver},
                                address =>
                                    $disk_info->{$looper}->{ip_address},
                            }
                        );

                        $aggregate_node = $aggregates->[0]->{nodes};
                        $disk_info->{$looper}->{aggregate} = $aggregate;
                        $disk_info->{$looper}->{aggregate_node}
                            = $aggregate_node;
                        my $destIPHome = $lif_info->[0]{home_node};
                        $disk_info->{$looper}->{ip_address_home}
                            = $lif_info->[0]{home_node};
                        last;
                    }
                }
            }
            $Log->debug("Disk Info\n" . Dumper($disk_info));
            my $rtschema = $self->get_cb_rtschema('san', 'resources');
            for (my $i = 0; $i < $number_of_keys; $i++) {

                if (!defined($disk_info->{$i})) { next; }

                my $row;
                foreach my $key (@$rtschema) {
                    $row->{$key} = '';
                }
                if ($disk_info->{$i}->{aggregate} ne "") {
                    $row->{client}    = $client;
                    $row->{disk}      = "";
                    $row->{device}    = $disk_info->{$i}->{physical_drive};
                    $row->{uid}       = $disk_info->{$i}->{uid};
                    $row->{interface} = $disk_info->{$i}->{interface};
                    $row->{partition} = "";
                    $row->{drive}     = "";
                    $row->{volume}    = $disk_info->{$i}->{volume_id};
                    $row->{vserver}   = $disk_info->{$i}->{vserver};
                    $row->{aggregate} = $disk_info->{$i}->{aggregate};
                    $row->{aggregate_node}
                        = $disk_info->{$i}->{aggregate_node};
                    $row->{lun}         = $disk_info->{$i}->{lun_id};
                    $row->{destip}      = $disk_info->{$i}->{ip_address};
                    $row->{destip_home} = $disk_info->{$i}->{ip_address_home};
                    $row->{path}        = $disk_info->{$i}->{lun_pathname};
                    $row->{name}
                        = $client . ':' . $disk_info->{$i}->{physical_drive};
                }

                # Filter out path table entries. If there are path filters
                # and one of them does not evaulate to true, skip the resource
                my $skip         = 0;
                my $path_filters = $rgconf->{$rgname}->{path_filters};
                foreach my $key (keys(%{$path_filters})) {
                    my $match
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{match};
                    my $regex
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{regex};
                    if ($match eq 'true' && $row->{$key} !~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $row->{name} $key($row->{$key}) !~ /$regex/"
                        );
                        last;
                    }
                    if ($match eq 'false' && $row->{$key} =~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $row->{name} $key($row->{$key}) =~ /$regex/"
                        );
                        last;
                    }
                }

                if ($skip && keys(%{$path_filters})) { next; }
                $Log->debug("Row:\n" . Dumper($row));
                push(@objects, $row);

            }
        }

        my $range = $rgconf->{$rgname}->{resource_range};
        if (!$range) { $range = '1-100%'; }
        my ($start, $end)
            = SystemTest::Populate::parse_range($range, scalar(@objects));

        # Walk the list of resources and filter by range
        my $resource_count     = 0;
        my $filtered_resources = [];
        foreach my $obj (@objects) {
            $resource_count++;
            if (   $resource_count >= $start
                && $resource_count <= $end) {
                $Log->debug("Adding resource $resource_count "
                        . "$obj->{name} to resource group $rgname");
                push(@{$filtered_resources}, $obj);
                push(@resources,             $obj);
            }
        }
        if (@{$filtered_resources} > 0) {
            $Log->comment("Adding "
                    . @{$filtered_resources}
                    . " $rtname objects to ConfigBroker for client $client");
            $rg->add_resources(
                resource_type    => $rtname,
                resourcearrayref => $filtered_resources
            );
        } else {
            $Log->comment("No $rtname objects found on $client");
        }
    }
    return \@resources;
} ## end sub populate_san_cb_resources_table

sub populate_cb_users_table {

    my ($self, $proto, $rgname, $rgconf) = @_;
    my @objects;
    $Log->comment(
        "Populating users for " . uc($proto) . " resource group $rgname");

    my $cb            = $self->get_cb_object('configbroker');
    my $clus_rg       = $self->get_cb_object('cluster');
    my $clus_rg_users = $clus_rg->get_resources(type => 'user');

    if (!@{$clus_rg_users}) {
        $Log->warn("No users found");
        next;
    } else {
        $Log->comment("Found " . @{$clus_rg_users} . " users");
    }

    # Sort users by id because ConfigBroker does not consistently maintain
    # sort order for CB.users
    my @sorted_users = natkeysort { $_->{id} } @{$clus_rg_users};

    $Log->comment("Creating resource group for $rgname");
    my $rg = $cb->create_resource_group(name => $rgname);
    my $rtschema = $self->get_cb_rtschema($proto, 'users');
    my $rtname = $self->get_cb_rtname($proto, 'users');

    foreach my $row (@sorted_users) {

        # Filter out user table entries. If there are user filters
        # and one of them does not evaulate to true, skip the resource
        my $skip         = 0;
        my $user_filters = $rgconf->{$rgname}->{user_filters};
        foreach my $key (keys(%{$user_filters})) {
            my $match = $rgconf->{$rgname}->{user_filters}->{$key}->{match};
            my $regex = $rgconf->{$rgname}->{user_filters}->{$key}->{regex};
            if ($match eq 'true' && $row->{$key} !~ /$regex/) {
                $skip = 1;
                $Log->debug(
                    "Skipping $row->{name} $key($row->{$key}) !~ /$regex/");
                last;
            }
            if ($match eq 'false' && $row->{$key} =~ /$regex/) {
                $skip = 1;
                $Log->debug(
                    "Skipping $row->{name} $key($row->{$key}) =~ /$regex/");
                last;
            }
        }

        if ($skip && keys(%{$user_filters})) { next; }
        push(@objects, $row);
    }

    my $range = $rgconf->{$rgname}->{user_range};
    if (!$range) { $range = '1-100%'; }
    my ($start, $end)
        = SystemTest::Populate::parse_range($range, scalar(@objects));

    # Walk the list of users and filter by range
    my $user_count     = 0;
    my $filtered_users = [];
    foreach my $obj (@objects) {
        $user_count++;
        if (   $user_count >= $start
            && $user_count <= $end) {
            $Log->debug("Adding user $user_count "
                    . "$obj->{name} to user group $rgname");
            push(@{$filtered_users}, $obj);
        }
    }

    my $rtname = $self->get_cb_rtname($proto, 'users');
    if (@{$filtered_users} > 0) {

        #shuffle_array(@{$filtered_users});
        $Log->comment("Adding "
                . @{$filtered_users}
                . " $rtname objects to ConfigBroker");
        $rg->add_resources(
            resource_type    => $rtname,
            resourcearrayref => $filtered_users
        );
    } else {
        $Log->comment("No $rtname objects found");
    }
    return $filtered_users;
} ## end sub populate_cb_users_table
1;
