#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NAS Protocol Topology for ConfigBroker
##
## @author      Jd dascenj@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::ConfigBroker - NAS Protocol Topology for ConfigBroker

=head1 SYNOPSIS

    See Topology.pm

=head1 DESCRIPTION

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Probably. 

=head1 STATUS

  Awesome

=head1 AUTHOR

  Jd (dascenj@netapp.com)
  $Id: $
  Last revised $Date: 2011 Mar 26 $
    
=head1 COPYRIGHT

Copyright 2011, Network Appliance., All Rights Reserved


=cut

package Topology::NAS::ConfigBrokerTopo7M;
use base qw(Topology);

use strict;

use ConfigBroker;
use Module::Common qw(validate_mixed);
use Params::Validate
    qw(validate SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use SayWrap qw(sayEOF);
use Module::Load qw(load);
use Tharn qw($Log $Staf subtest config_result script_result fail_result host);
use XML::Simple;
use Sort::Key::Natural qw(natsort natkeysort);
use File::Basename;
use Data::Dumper qw(Dumper);
use Options;

my $Options;
my $Options_hash;
my @nodes;

###################################################################

$Options = new Options;
$Options->parameters(
    definitions => [
        {   name => "CB_DISCOVER",
            spec => "s",
            description =>
                "Delete the resources out of the CB database, and rediscovers",
            default => 0,
        },
        {   name        => "CB_POPULATE",
            spec        => "s",
            description => "Populates the appropriate cifs and nfs tables",
            default     => 0,
        },
        {   name        => "CB_PROTOCOLS",
            spec        => "s@",
            description => "The determines the correct protocols to use",
            default     => 'cifs,nfs,san',
        },
        {   name        => "CB_USERS",
            spec        => "s",
            description => "The key of the desired user key",
        },
        {   name        => "CB_USER_CONFIG",
            spec        => "s",
            description => "The file contaning user info to read into CB",
        },
        {   name => "CIFS_RG_CONFIG",
            spec => "s",
            description =>
                'Relative path to the file the defines the resource group layout',
            default =>
                'system_test/workload/client/cifs/default_7m_rg_config.xml',
        },
        {   name        => "CIFS_RG_FILTER",
            spec        => "s",
            description => 'Override resource group filters',
            default     => undef
        },
        {   name        => "CIFS_WORKLOAD_FILE",
            spec        => "s",
            description => 'Relative path to the CIFS workload file',
            default =>
                'system_test/workload/client/cifs/default_workload.xml',
        },
        {   name        => "COMM_LOG_LEVEL",
            spec        => "s",
            description => 'Controls how verbose NATE is',
            default     => 'debug'
        },
        {   name => "NFS_RG_CONFIG",
            spec => "s",
            description =>
                'Relative path to the file the defines the resource group layout',
            default =>
                'system_test/workload/client/nfs/default_7m_rg_config.xml',
        },
        {   name        => "NFS_RG_FILTER",
            spec        => "s",
            description => 'Override resource group filters',
            default     => undef
        },
        {   name        => "NFS_WORKLOAD_FILE",
            spec        => "s",
            description => 'Relative path to the CIFS workload file',
            default =>
                'system_test/workload/client/nfs/SN_public_workload.xml',
        },
        {   name => "SAN_RG_CONFIG",
            spec => "s",
            description =>
                'Relative path to the file the defines the resource group layout',
            default =>
                'system_test/workload/client/san/default_7m_rg_config.xml',
        },
        {   name        => "SAN_RG_FILTER",
            spec        => "s",
            description => 'Override resource group filters',
            default     => undef
        },
        {   name        => "SAN_WORKLOAD_FILE",
            spec        => "s",
            description => 'No user servicable parts inside',
            default     => undef,
        },
        {   name        => "SAN_WORKLOAD_FILE_UNIX",
            spec        => "s",
            description => 'Relative path to the SAN workload file for Unix',
            default =>
                'system_test/workload/client/san/default_workload_unix.xml',
        },
        {   name => "SAN_WORKLOAD_FILE_WINDOWS",
            spec => "s",
            description =>
                'Relative path to the SAN workload file for Windows',
            default =>
                'system_test/workload/client/san/default_workload_windows.xml',
        },
        {   name => "SKTRACE_OPTIONS",
            spec => "s@",
            description =>
                "A list of values to be modified with the sysctl command. "
                . "Example: SKTRACE_OPTIONS=sysvar.sktrace.clear=1,sysvar.sktrace.Nfs4Lock_enable=63"
        },
        {   name => "TESTUSER_FILE",
            spec => "s",
            description =>
                "file containing the list of testusers who test run as",
            file_tests => "fr",
            subroutine => [\&read_testuser_file],
        },
        {   name        => "TESTUSER_PASSWORD",
            spec        => "s",
            description => "The password to use with TESTUSER",
            default     => undef,
        },
        {   name        => "TESTUSER",
            spec        => "s@",
            description => "List of test users to use for the testing",
            default     => undef,
        },
        {   name => "TML_WORKER_COUNT",
            spec => "s",
            description =>
                "The number of workers TML will create. Worker will "
                . "correspond to the thpl called (usually io.thpl)",
            default => 1,
        },
        {   name        => "TML_WORKER_INSTANCE",
            spec        => "s",
            description => 'The instance number passed to the worker',
            default     => undef,
        },
        {   name => "WORKLOAD_PARSER_LIB",
            spec => "s",
            description =>
                "The package name of the parser lib to use to load the "
                . "*_WORKLOAD_FILE. (the .pm file will need to be found in \@inc",
            default => 'SystemTest::WorkloadParser',
        }
    ]
);

sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;
    my $self     = {@_,};
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
} ## end sub new

sub define_topology {
    my $self = shift;
    $Log->enter();

    $Options_hash = $Options->get_hash();

    $Log->comment("Options: \n" . Dumper($Options_hash));

    # Obtain the node object
    require NACL::C::Node;
    @nodes = NACL::C::Node->find();

    $self->{cb}->{object}->{configbroker} = new ConfigBroker;
    my $cb = $self->get_cb_object('configbroker');

    # Keep the names of the filers around.
    foreach my $n (@nodes) {
        push(@{$self->{nodes}}, $n->hostrec->id());
    }

    $self->{clients} = $Options_hash->{'CLIENT'};

    my $node_name = $self->{nodes}->[0];
    $Log->comment(
        "The topology will utilize these filers: " . Dumper($self->{nodes}));

    $self->{cb}->{config} = {
        cifs => {
            'tables' => {
                activity => {
                    primarykey => ['starttime', 'resourcename'],
                    schema     => [
                        'starttime',   'resourcename',
                        'destip',      'cifs_server',
                        'volume',      'vserver',
                        'aggregate',   'aggregate_node',
                        'stoptime',    'client',
                        'credentials', 'workitem',
                        'result',      'sync_master',
                        'iteration',   'status'
                    ],
                    table => 'cifs_activity',
                },
                clients => {
                    primarykey => ['name'],
                    schema     => ['hosttype', 'os_ver', 'type'],
                    table      => 'cifs_clients'
                },
                resources => {
                    primarykey => ['name'],
                    schema     => [
                        'type',             'destip',
                        'destip_home',      'cifs_server',
                        'share',            'path',
                        'share_properties', 'vserver',
                        'auth_style',       'volume',
                        'aggregate',        'aggregate_node',
                        'lif_local_to_vol'
                    ],
                    table => 'cifspath'
                },
                users => {
                    primarykey => ['name'],
                    schema     => [
                        'id',      'user_name', 'password', 'ad',
                        'nis',     'host',      'hosttype', 'homedir',
                        'vserver', 'protocol'
                    ],
                    table => 'cifs_users'
                },
                workloads => {
                    primarykey => ['name'],
                    schema     => ['workload'],
                    table      => 'cifs_workloads'
                },
            },
        },
        nfs => {
            'tables' => {
                activity => {
                    primarykey => ['starttime', 'resourcename'],
                    schema     => [
                        'starttime',      'resourcename',
                        'destip',         'volume',
                        'vserver',        'aggregate',
                        'aggregate_node', 'stoptime',
                        'client',         'credentials',
                        'workitem',       'result',
                        'sync_master',    'iteration',
                        'status'
                    ],
                    table => 'nfs_activity',
                },
                clients => {
                    primarykey => ['name'],
                    schema     => ['hosttype', 'os_ver', 'type'],
                    table      => 'nfs_clients'
                },
                resources => {
                    primarykey => ['name'],
                    schema     => [
                        'type',     'junction_path',
                        'vserver',  'destip',
                        'volume',   'v3',
                        'v4',       'v4_1',
                        'udp',      'tcp',
                        'kerberos', 'aggregate',
                        'destpath', 'destip_home',
                        'lif_local_to_vol',
                    ],
                    table => 'nfspath'
                },
                users => {
                    primarykey => ['name'],
                    schema     => [
                        'id',      'user_name', 'password', 'ad',
                        'nis',     'host',      'hosttype', 'homedir',
                        'vserver', 'protocol'
                    ],
                    table => 'nfs_users'
                },
                workloads => {
                    primarykey => ['name'],
                    schema     => ['workload'],
                    table      => 'nfs_workloads'
                },
            },
        },
        san => {
            'tables' => {
                activity => {
                    primarykey => ['starttime', 'resourcename'],
                    schema     => [
                        'starttime', 'resourcename',
                        'disk',      'device',
                        'interface', 'partition',
                        'drive',     'destip',
                        'volume',    'vserver',
                        'aggregate', 'aggregate_node',
                        'stoptime',  'client',
                        'workitem',  'result',
                    ],
                    table => 'san_activity',
                },
                clients => {
                    primarykey => ['name'],
                    schema     => ['hosttype', 'os_ver', 'type'],
                    table      => 'san_clients'
                },
                resources => {
                    primarykey => ['name'],
                    schema     => [
                        'type',        'client',
                        'disk',        'device',
                        'interface',   'partition',
                        'drive',       'destip',
                        'destip_home', 'share',
                        'path',        'lun',
                        'vserver',     'volume',
                        'aggregate',   'aggregate_node'
                    ],
                    table => 'sanpath'
                },
                workloads => {
                    primarykey => ['name'],
                    schema     => ['workload'],
                    table      => 'san_workloads'
                },
            },
            vstype => 'vserver',
        },
    };

    $self->{cb_protocols} = $Options_hash->{'CB_PROTOCOLS'};

    # Do the required groups exist? Do discovery if not.
    $Log->comment("Getting all resource groups");
    my @rgs = eval { $cb->get_all_resource_groups(); };

    # Repopulate the data the CIFS and NFS path resouce group
    if ($Options_hash->{CB_POPULATE}) {

        # Rediscover if there is no cluster resource group or CB_DISCOVER=1
        my $clus_rg;
        if ($Options_hash->{CB_DISCOVER} || (!grep { $_ eq 'cluster' } @rgs))
        {

            # This is the discovery phase
            $Log->comment("Deleting resource groups from ConfigBroker "
                    . "to prepare for discovery\n"
                    . Dumper(\@rgs));

            # We are going to rediscover, get rid of all the resources in CB,
            # as well as the resourcegroups
            foreach my $rg (@rgs) {
                eval { $rg->delete_resources_from_db(); };
            }

            eval { $cb->delete_all_resource_groups(); };

            # Rediscover the cluster and put into a single rg
            # Shutup the disco output.
            $Log->comment("Executing ConfigBroker discovery");
            local $NATE::Log::CommEvent::CommLogLevel
                = $Options_hash->{COMM_LOG_LEVEL};
            $clus_rg = $cb->discover_and_create_resource_group(
                groupname     => 'cluster',
                is7mode       => 1,
                discoverusers => 1
            );
            local $NATE::Log::CommEvent::CommLogLevel = 'comment';
        } else {
            $Log->comment("Using the existing ConfigBroker database, "
                    . "skipping discovery");
            $clus_rg = $cb->get_resource_group(name => 'cluster');
        }
        $self->{cb}->{object}->{cluster} = $clus_rg;

        # Create the resource tables here becasue ConfigBroker sucks at
        # squelching DBI errors when a table does not exist. =/
        foreach my $proto ('cifs', 'nfs', 'san') {

            # Create the current protocol's tables
            foreach my $rgtype ($self->get_cb_tables($proto)) {

                # Get the relevant information for the current proto and rg
                # type
                my $rtschema = $self->get_cb_rtschema($proto, $rgtype);
                my $rtname = $self->get_cb_rtname($proto, $rgtype);
                my $pkey = $self->get_cb_rt_primarykey($proto, $rgtype,);

                # The condition exists where you get duplicated entries
                # because of discovery. That is, if you have 10 resources that
                # are part of the the resourcegroup JD, and then you discover,
                # you will have the same 10 resources added to your table
                # (for a total of 20) in the default resource group. To get
                # around this, drop all the tables we are about to create.
                # This is likely okay, because we already don't trust CB. The
                # filer configuration may have changed, so these tables are
                # not realiable anyway.
                eval {
                    my $result = $cb->delete_resource_type(name => $rtname);
                };

                # Create the resource table
                $Log->comment("Creating resource table for $rtname");
                $cb->create_resource_type(
                    name       => $rtname,
                    primarykey => $pkey,
                    tabledefn  => $rtschema
                );

            }
        }

        # Read the resource group config
        foreach my $proto (@{$self->{cb_protocols}}) {

            # Determine the right resource group config to use
            my ($rgconf_files, $rg_filter);
            if ($proto eq 'cifs') {
                $rgconf_files = $Options_hash->{'CIFS_RG_CONFIG'};
                $rg_filter    = $Options_hash->{'CIFS_RG_FILTER'};
            } elsif ($proto eq 'nfs') {
                $rgconf_files = $Options_hash->{'NFS_RG_CONFIG'};
                $rg_filter    = $Options_hash->{'NFS_RG_FILTER'};
            } elsif ($proto eq 'san') {
                $rgconf_files = $Options_hash->{'SAN_RG_CONFIG'};
                $rg_filter    = $Options_hash->{'SAN_RG_FILTER'};
            } else {
                die script_result("We don't have a resource group "
                        . "config file for the $proto protocol");
            }

            # Take a comma separated list of resource group config and
            # combine then in to one hashref
            my $rgconf = {};

            # Create the XML::Simple object
            my $xml = new XML::Simple;

            # Split apart the values. Values can be in the form of
            # system_test/workload/client/cifs/rgfile.xml
            # system_test/workload/client/cifs:rgfile1.xml:rgfile2.xml
	    my @files;
            my @values = split(/,/, $rgconf_files);
            foreach my $value (@values) {
                if ($value =~ /:/) {
                    my @fields = split(/:/, $value);
                    if (@fields < 2) {
                        die script_result("Invalid format for "
                                . uc($proto)
                                . "_RG_CONF. When specifying multiple files "
                                . "per path please specify "
                                . "relative/path:file1:file2 ")->log();
                    }
                    for (my $i = 1; $i < @fields; $i++) {
                        push(@files, "$fields[0]/$fields[$i]");
                    }
                } else {
                    push(@files, $value);
                }
            }

            # Loop through each file path
            foreach my $relative_path (@files) {

                # Use NATE to fine the real path
                $Log->comment(
                    "Searching for resource group config $relative_path");
                my $full_path = NATE::Inc::find_file($relative_path);
                if (!$full_path) {
                    die script_result("Cannot find $relative_path");
                } else {
                    $Log->comment("Creating "
                            . uc($proto)
                            . " resource groups from $relative_path");
                }

                # Convert the XML file into a hashref
                my $h = $xml->XMLin("$full_path");

                # Copy all keys to the main hashref. Warn if there are
                # duplicate keys
                foreach my $key (keys(%{$h})) {
                    if (defined($rgconf->{$key})) {
                        $Log->warn("Duplicate key detected when adding keys "
                                . "from $full_path! The key $key has "
                                . "already been added. Please check your "
                                . "${proto}_RG_CONFIG parameter to make sure "
                                . "you are not passing the same file in "
                                . "multiple times or you are not using the "
                                . "same resource group name in multiple files"
                        );
                    }
                    $rgconf->{$key} = $h->{$key};
                }
            }

            # Here we allow people to pass in node names and client ranges per
            # resource group so they don't have to hard code them in
            # (CIFS|NFS)_RG_CONFIG
            my $format
                = '<name>:'
                . '<vfnet node regex>:'
                . '<vfnet interface regex>:'
                . '<resource range>:'
                . '<workload regex>:'
                . '<client range>';

            if ($rg_filter) {
                my @filters = split(/,/, $rg_filter);
                foreach my $filter (@filters) {
                    my @fields = split(/:/, $filter);
                    if (@fields < 1 || @fields > 6) {
                        die script_result(
                                  "Invalid RG_FILTER format. Expected a "
                                . "minimum of 2 values in 6 fields, received "
                                . "$filter.\n Valid format: $format");
                    } ## end

                    my ($rgname, $vfnet_node, $vfnet_interface,
                        $resource_range, $workload_regex, $client_range)
                        = split(/:/, $filter);
                    if ($vfnet_node) {
                        $rgconf->{$rgname}->{resource_filters}->{vfnet}
                            ->{regex}->{node} = $vfnet_node;
                    }
                    if ($vfnet_interface) {
                        $rgconf->{$rgname}->{resource_filters}->{vfnet}
                            ->{regex}->{interface} = $vfnet_interface;
                    }
                    if ($resource_range) {
                        $rgconf->{$rgname}->{resource_range}
                            = $resource_range;
                    }
                    if ($workload_regex) {
                        $rgconf->{$rgname}->{workload_regex}
                            = $workload_regex;
                    }
                    if ($client_range) {
                        $rgconf->{$rgname}->{client_range} = $client_range;
                    }
                }
            }
            $Log->comment(
                uc($proto) . " resource group config\n" . Dumper($rgconf));

            my @resource_groups = keys(%{$rgconf});
            foreach my $key (natsort(@resource_groups)) {
                $Log->comment("Working on resource group $key");

                # Create the resource group(s). The order of the steps is
                # dependent on the protocol. For SAN we get the info from the
                # clients and then match it up to the info from ONTAP.
                if ($proto ne 'san') {
                    my $resource_count
                        = $self->populate_cb_resources_table($proto, $key,
                        $rgconf);

                    if (@{$resource_count}) {
                        $self->populate_cb_workloads_table($proto, $key,
                            $rgconf);
                        $self->populate_cb_clients_table($proto, $key,
                            $rgconf);
                    }
                } else {
                    $cb->create_resource_group(name => $key);
                    my $san_clients
                        = $self->populate_cb_clients_table($proto, $key,
                        $rgconf);
                    $self->populate_san_cb_resources_table($key, $san_clients,
                        $rgconf);
                    $self->populate_cb_workloads_table($proto, $key, $rgconf);
                }

                # Putting this here in case some one wants to enable users
                # for SAN
                if (   $Options_hash->{CB_USER_CONFIG}
                    && $Options_hash->{CB_USERS}
                    && $proto =~ /cifs|nfs/) {

                    # Populate the <proto>_clients table with <proto>
                    # clients for this resource group
                    $self->populate_cb_users_table($proto, $key, $rgconf);
                }
            }
        }
    }

    foreach my $proto (@{$self->{cb_protocols}}) {

        # Want to reset the actitivity table for every run, regardless of CB
        # discovery settings. Need to do this all down here, so that the CB
        # rediscover doesn't blow out all the RG's and tables.

        # Add the activity table for each table. This is where clients will
        # log their activities to show which resources they are acting on
        my $activitytablename = $proto . '_activity';

        $Log->comment("Creating resource group for $activitytablename");
        my $rg = $cb->create_resource_group(
            name   => $activitytablename,
            share  => 'SHARE',
            status => 'READY',
            reuse  => 'REUSE'
        );

        # We have to enter a row to make the resourcegroup stick around
        my $row;
        if ($proto =~ /cifs|nfs/) {
            $row = {
                aggregate      => 'TheBeginning',
                aggregate_node => 'TheBeginning',
                cifs_server    => 'TheBeginning',
                client         => 'TheBeginning',
                credentials    => 'TheBeginning',
                destip         => 'TheBeginning',
                iteration      => 'TheBeginning',
                name           => 'ConfigBrokerTopo' . uc($proto),
                result         => '',
                starttime      => time(),
                status         => 'TheBeginning',
                stoptime       => time(),
                sync_master    => 'TheBeginning',
                volume         => 'TheBeginning',
                vserver        => 'TheBeginning',
                workitem       => 'TheBeginning',
            };
        } else {
            $row = {
                aggregate      => 'TheBeginning',
                aggregate_node => 'TheBeginning',
                cifs_server    => 'TheBeginning',
                client         => 'TheBeginning',
                credentials    => 'TheBeginning',
                destip         => 'TheBeginning',
                iteration      => 'TheBeginning',
                name           => 'ConfigBrokerTopo' . uc($proto),
                result         => '',
                starttime      => time(),
                status         => 'TheBeginning',
                stoptime       => time(),
                sync_master    => 'TheBeginning',
                volume         => 'TheBeginning',
                vserver        => 'TheBeginning',
                workitem       => 'TheBeginning',
            };
        }

        $rg->add_resources(
            resource_type    => $activitytablename,
            resourcearrayref => [$row],
        );

        # Create the client_setup hash
        $self->create_client_setup($proto);

    }

    # Setup the instances
    $Log->debug(Dumper($self->{clients}));
    $Log->debug(Dumper($self->{clients_status}));
    $self->configure_instances();

    # Create the TML test options
    $self->create_test_options();

    # Get creative about logging all our information
    my $top_logdir = Tharn::param('TOP_LOGDIR');
    my $runid      = Tharn::param('RUNID');
    my $lonely_log = $top_logdir . '/' . $runid . '_config_info.log';
    if ($Log->may_debug()) {
        $Log->comment("define_topology Topology: \n" . Dumper($self),
            path => $lonely_log);

    } else {
        $Log->comment("filer_setup:" . Dumper($self->{filer_setup}),
            path => $lonely_log);
        $Log->comment("instances:" . Dumper($self->{instances}),
            path => $lonely_log);
        $Log->comment("client_config:" . Dumper($self->{client_config}),
            path => $lonely_log);
        $Log->comment("test_options:" . Dumper($self->{test_options}),
            path => $lonely_log);
    }

    # Configure the sktracing if it is applicable. Normally something like
    # this should be placed in filer setup. However, I have decided to stick
    # it here in define_topology because I want to be sure that sktracing is
    # set correctly even when the topology is run with SKIPSTAGE=filer_setup
    # to use an existing setup
    local $NATE::Log::CommEvent::CommLogLevel
        = $Options_hash->{COMM_LOG_LEVEL};
    $self->sktrace_setup();
    local $NATE::Log::CommEvent::CommLogLevel = 'comment';

    # Create the TOP_LOGDIR/clients dir so the clients don't reset each other
    # permissions to it
    mkdir("$top_logdir/clients");
    Tharn::sleep 1;
    chmod(0777, "$top_logdir/clients");
} ## end sub define_topology

sub get_cb_tables {
    my ($self, $proto) = @_;
    return keys(%{$self->{cb}->{config}->{$proto}->{tables}});
}

sub get_cb_object {
    my ($self, $name, $type) = @_;
    if ($name eq 'cluster' || $name eq 'configbroker') {
        if (defined($self->{cb}->{object}->{$name})) {
            return $self->{cb}->{object}->{$name};
        }
    } elsif (defined($self->{cb}->{object}->{$name}->{$type})) {
        return $self->{cb}->{object}->{$name}->{$type};
    } else {
        return;
    }
} ## end sub get_cb_object

sub get_cb_rtname {
    my ($self, $proto, $type) = @_;
    return $self->{cb}->{config}->{$proto}->{tables}->{$type}->{'table'};
}

sub get_cb_rtschema {
    my ($self, $proto, $type) = @_;
    return $self->{cb}->{config}->{$proto}->{tables}->{$type}->{'schema'};
}

sub get_cb_rt_primarykey {
    my ($self, $proto, $type) = @_;
    my $pk
        = $self->{cb}->{config}->{$proto}->{tables}->{$type}->{'primarykey'};

    # set a default
    $pk = ["name"] if (not $pk);

    return $pk;
} ## end sub get_cb_rt_primarykey

sub get_cb_vstype {
    my ($self, $proto) = @_;
    return $self->{cb}->{config}->{$proto}->{vstype};
}

sub populate_cb_clients_table {
    my ($self, $proto, $rgname, $rgconf) = @_;

    my $cb     = $self->get_cb_object('configbroker');
    my $rg     = $cb->get_resource_group(name => $rgname);
    my $rtname = $self->get_cb_rtname($proto, 'clients');
    my @clients;
    foreach my $client ($self->clients()) {
        my $h = eval { host($client); };
        if ($@) { next; }
        my $hosttype = $h->hosttype();

        if ($proto eq 'cifs' && $hosttype !~ /win/) { next; }
        if ($proto eq 'nfs' && $hosttype =~ /win/) { next; }
        push(@clients, $client);
    }

    my $range = $rgconf->{$rgname}->{client_range};
    if (!$range) { $range = '1-100%'; }
    my $filter = '.*';
    if ($range =~ /^([^:]+):(\d+.+)/) {
        $filter = $1;
        $range  = $2;
    }
    my ($start, $end) = $self->_parse_range($range, scalar(@clients));

    $Log->comment(
        "Limiting client range to $range($start-$end) of clients matching /$filter/"
    );

    # Walk the list of resources and filter by range
    my $client_count = 0;
    my @filtered_clients;
    foreach my $client (@clients) {
        my $h = eval { host($client); };
        if ($@) { next; }

        my $hosttype = $h->hosttype();
        if (defined($filter) && $hosttype !~ /$filter/) {
            next;
        }

        my %fields = $h->param_fields();
        my $os_ver = $fields{OS_VER};

        $client_count++;
        if (   $client_count < $start
            || $client_count > $end) {
            next;
        }

        push(@filtered_clients, $client);

        my $row = {
            name     => $client,
            hosttype => $hosttype,
            type     => $rtname,
            os_ver   => $os_ver,
        };

        $rg->add_resources(
            resource_type    => $rtname,
            resourcearrayref => [$row],
        );
    }
    return \@filtered_clients;
} ## end sub populate_cb_clients_table

sub populate_cb_resources_table {

    my ($self, $proto, $rgname, $rgconf) = @_;
    my @objects;
    $Log->comment(
        "Populating resources for " . uc($proto) . " resource group $rgname");

    # Get all vservers configured for the current protocol
    $Log->comment("Getting all " . uc($proto) . " vfilers");
    my $clus_rg = $self->get_cb_object('cluster');
    my $cifs_vfilers = $clus_rg->get_resources(type => 'cifs');

    $Log->comment("Creating resource group for $rgname");
    my $cb = $self->get_cb_object('configbroker');
    my $rg = $cb->create_resource_group(name => $rgname);
    foreach my $node (@{$self->{nodes}}) {

        # Query ConfigBroker for the the vfilers using the supplied filters
        my $args = {type => 'vfiler'};
        $args->{match_filters}->{node} = $node;
        $self->get_resource_filters($rgconf, $rgname, 'vfiler', $args);
        my $vfilers = $clus_rg->get_resources($args);

        foreach my $vfiler (@{$vfilers}) {

            # Get The name of the current vfiler
            my $vfiler_name = $vfiler->{name};
            if ($proto eq 'cifs' && !grep { $_->{vfiler} eq $vfiler_name }
                @{$cifs_vfilers}) {
                $Log->debug("Skipping $vfiler_name");
                next;
            }

            # Populate the current protocols obj
            my $obj_vfiler;
            my $rtschema = $self->get_cb_rtschema($proto, 'resources');
            my $rtname = $self->get_cb_rtname($proto, 'resources');
            foreach my $key (@$rtschema) {
                $obj_vfiler->{$key} = "";
            }
            $obj_vfiler->{type}    = $rtname;
            $obj_vfiler->{vserver} = $vfiler_name;

            # Add vserver specific obj_vfiler info
            if ($proto eq 'nfs') {
                my $nfs_options = $clus_rg->get_resources(
                    type          => 'options7m',
                    match_filters => {vfiler => $vfiler_name, node => $node},
                    regex_filters => {name => 'nfs'}
                );

                my %nfs = (
                    v3  => 'nfs.v3.enable',
                    v4  => 'nfs.v4.enable',
                    tcp => 'nfs.tcp.enable',
                    udp => 'nfs.udp.enable'
                );

                $obj_vfiler->{v4_1} = '';

                foreach my $key (keys(%nfs)) {
                    foreach my $o (@{$nfs_options}) {
                        if ($nfs{$key} eq $o->{name} && $o->{value} eq 'on') {
                            $obj_vfiler->{$key} = 'enabled';
                        }
                    }
                }

                $obj_vfiler->{lif_local_to_vol} = 0;
            } else {
                my $cifs_info = $clus_rg->get_resources(
                    type          => 'cifs',
                    match_filters => {vfiler => $vfiler_name, node => $node},
                );

                if ($cifs_info->[0]->{auth_type} eq 'ad') {
                    $obj_vfiler->{auth_style} = 'domain';
                } else {
                    $obj_vfiler->{auth_style} = 'workgroup';
                }
                $obj_vfiler->{cifs_server}
                    = $cifs_info->[0]->{netbios_servername};
                $obj_vfiler->{default_site} = $cifs_info->[0]->{ad_site};
                $obj_vfiler->{domain}
                    = $cifs_info->[0]->{netbios_domainname};
                $obj_vfiler->{domain_workgroup}
                    = $cifs_info->[0]->{workgroup};
            }

            # Find all the addresses associated with this vfiler
            $Log->debug(
                "Finding all addresses associated with vfiler $vfiler_name");

            # Query ConfigBroker for the vfiler IP addresses using the
            # supplied filters
            $args                            = {};
            $args                            = {type => 'vfnet'};
            $args->{match_filters}->{vfiler} = $vfiler_name;
            $args->{match_filters}->{node}   = $node;

            $self->get_resource_filters($rgconf, $rgname, 'vfnet', $args);
            $Log->debug("vfnet resource filters\n" . Dumper($args));

            my $vfnets = $clus_rg->get_resources($args);
            $Log->debug("vfnet resources\n" . Dumper($vfnets));

            if (!@{$vfnets}) {
                my $filters = $rgconf->{$rgname}->{resource_filters}->{vfnet};
                $Log->warn(
                          "No network addresses found on vfiler $vfiler_name "
                        . "using the following filters\n"
                        . Dumper($filters));
                next;
            } else {
                $Log->comment("Found "
                        . @{$vfnets}
                        . " network addresses on vfiler $vfiler_name");
            }

            # Query ConfigBroker for the vfstores
            $args                            = {};
            $args                            = {type => 'vfstore'};
            $args->{match_filters}->{vfiler} = $vfiler_name;
            $args->{match_filters}->{node}   = $node;
            $self->get_resource_filters($rgconf, $rgname, 'vfstore', $args);
            my $vfstores = $clus_rg->get_resources($args);

            $Log->debug(
                "Finding volumes associated with vfiler $vfiler_name");

            # Query ConfigBroker for the volumes using the supplied filters
            $args                                   = {};
            $args                                   = {type => 'volume'};
            $args->{match_filters}->{owning_vfiler} = $vfiler_name;
            $args->{match_filters}->{node}          = $node;

            $self->get_resource_filters($rgconf, $rgname, 'volume', $args);

            my $filtered_volumes = $clus_rg->get_resources($args);

            if (!@{$filtered_volumes}) {
                my $filters
                    = $rgconf->{$rgname}->{resource_filters}->{'volume'};
                $Log->warn("No volumes found on vfiler $vfiler_name "
                        . "using the following filters\n"
                        . Dumper($filters));
                next;
            } else {
                $Log->comment("Found "
                        . @{$filtered_volumes}
                        . " volumes on vfiler $vfiler_name");
            }

            # Query ConfigBroker for all volumes
            $args                                   = {};
            $args                                   = {type => 'volume'};
            $args->{match_filters}->{owning_vfiler} = $vfiler_name;

            my $volumes = $clus_rg->get_resources($args);

            my @storage_objects;
            if ($proto eq 'cifs') {
                $Log->debug(
                    "Finding shares associated with vfiler $vfiler_name");

               # Query ConfigBroker for the volumes using the supplied filters
                $args                            = {};
                $args                            = {type => 'shares'};
                $args->{match_filters}->{vfiler} = $vfiler_name;
                $args->{match_filters}->{node}   = $node;

                $self->get_resource_filters($rgconf, $rgname, 'shares',
                    $args);

                my $shares = $clus_rg->get_resources($args);

                $Log->comment("Found "
                        . @{$shares}
                        . " shares on vfiler $vfiler_name.");
                @storage_objects = @{$shares};
            } else {
                @storage_objects = @{$filtered_volumes};
            }

            bless $obj_vfiler;

            # Add volume specific obj_vfiler info
            foreach my $object (@storage_objects) {
                if ($proto eq 'cifs') {
                    my $vols
                        = $self->get_volume_from_path($object->{mount_point},
                        $vfstores);
                    if (!@{$vols} && $object->{vfiler} ne 'vfiler0') {
                        $Log->comment(
                                  "Skipping $object->{share_name} because "
                                . "its mount point does not match any "
                                . "vfstore paths");
                        next;
                    }
                } else {
                    if (!grep(/\/vol\/$object->{volume}$/, @{$vfstores})
                        && $object->{owning_vfiler} ne 'vfiler0') {
                        $Log->comment("Skipping $object->{volume} because "
                                . "its path not match any vfstore paths");
                        next;
                    }
                }

                # Copy the obj_vfiler
                my $obj_storage = Storable::dclone $obj_vfiler;

                if ($proto eq 'nfs' && defined($obj_storage->{volume})) {
                    $obj_storage->{volume} = $object->{volume};
                }

                if ($proto eq 'nfs' && defined($obj_storage->{aggregate})) {
                    $obj_storage->{aggregate} = $object->{aggregate};
                }

                if ($proto eq 'cifs' && defined($obj_storage->{share})) {
                    $obj_storage->{share} = $object->{share_name};
                }

                if ($proto eq 'cifs' && defined($obj_storage->{path})) {
                    $obj_storage->{path} = $object->{mount_point};
                }

                if ($proto eq 'cifs' && defined($obj_storage->{share})) {

                    my $volume;
                    if ($object->{mount_point} =~ /\/vol\/([^\/]+)/) {
                        $volume = $1;
                    } else {
                        next;
                    }

                    $obj_storage->{volume} = $volume;
                }

                if (defined($obj_storage->{aggregate})) {
                    foreach my $vol (@{$volumes}) {
                        if ($vol->{volume} eq $obj_storage->{volume}) {
                            $obj_storage->{aggregate} = $vol->{aggregate};
                            last;
                        }
                    }
                }

                # Skip this entry if there is no volume information
                if (!$obj_storage->{volume}) { next; }
                $obj_storage->{aggregate_node} = $node;

                bless $obj_storage;
                foreach my $vfnet (@{$vfnets}) {

                    # Copy once more, all the data above is still valid.
                    #require Storable;
                    my $obj_vfnet = Storable::dclone $obj_storage;

                    # What do you know now?
                    if (defined($obj_vfnet->{destip})) {
                        $obj_vfnet->{destip} = $vfnet->{ipaddress};
                    }

                    if (defined($obj_vfnet->{destip_home})) {

                        $obj_vfnet->{destip_home} = $node;
                    }

                    # Do some figgerin, but use facts instead of advice
                    # from celebrities...
                    if (defined($obj_vfnet->{destpath})) {
                        $obj_vfnet->{destpath}
                            = $obj_vfnet->{destip} 
                            . ':/vol/'
                            . $obj_vfnet->{volume};
                    }

                    if (defined($obj_vfnet->{junction_path})) {
                        $obj_vfnet->{junction_path}
                            = '/vol/' . $obj_vfnet->{volume};
                    }

                    #  The DB needs a name as the primary key
                    $obj_vfnet->{lif_local_to_vol} = 1;

                    if ($proto eq 'cifs') {
                        $obj_vfnet->{name} = $obj_vfnet->{destip} . ':'
                            . $obj_vfnet->{share};
                    }
                    if ($proto eq 'nfs') {
                        $obj_vfnet->{name} = $obj_vfnet->{destpath};
                    }

                    $Log->debug("Filtering:\n" . Dumper($obj_vfnet));

                    # Filter out path table entries. If there are path filters
                    # and one of them does not evaulate to true, skip the
                    # resource
                    my $skip         = 0;
                    my $path_filters = $rgconf->{$rgname}->{path_filters};
                    foreach my $key (keys(%{$path_filters})) {
                        my $match = $rgconf->{$rgname}->{path_filters}->{$key}
                            ->{match};
                        my $regex = $rgconf->{$rgname}->{path_filters}->{$key}
                            ->{regex};
                        if (   $match eq 'true'
                            && $obj_vfnet->{$key} !~ /$regex/) {
                            $skip = 1;
                            $Log->comment(
                                "Skipping $obj_vfnet->{name} $key($obj_vfnet->{$key}) !~ /$regex/"
                            );
                            last;
                        }
                        if (   $match eq 'false'
                            && $obj_vfnet->{$key} =~ /$regex/) {
                            $skip = 1;
                            $Log->comment(
                                "Skipping $obj_vfnet->{name} $key($obj_vfnet->{$key}) =~ /$regex/"
                            );
                            last;
                        }
                    }

                    if ($skip && keys(%{$path_filters})) { next; }

                    $Log->debug("Adding the $rgname object to CB: "
                            . Dumper($obj_vfnet));
                    push(@objects, $obj_vfnet);
                }
            }
        }
    }

    my $range = $rgconf->{$rgname}->{resource_range};
    if (!$range) { $range = '1-100%'; }
    my ($start, $end) = $self->_parse_range($range, scalar(@objects));

    # Walk the list of resources and filter by range
    my $resource_count     = 0;
    my $filtered_resources = [];
    foreach my $obj (@objects) {
        $resource_count++;
        if (   $resource_count >= $start
            && $resource_count <= $end) {
            $Log->debug("Adding resource $resource_count "
                    . "$obj->{name} to resource group $rgname");
            push(@{$filtered_resources}, $obj);
        }
    }

    my $rtname = $self->get_cb_rtname($proto, 'resources');
    if (@{$filtered_resources} > 0) {
        $Log->comment("Adding "
                . @{$filtered_resources}
                . " $rtname objects to ConfigBroker");
        $rg->add_resources(
            resource_type    => $rtname,
            resourcearrayref => $filtered_resources
        );
    } else {
        $Log->comment("No $rtname objects found");
    }
    return $filtered_resources;
} ## end sub populate_cb_resources_table

sub populate_cb_workloads_table {
    my ($self, $proto, $rgname, $rgconf) = @_;

    my $cb             = $self->get_cb_object('configbroker');
    my $rg             = $cb->get_resource_group(name => $rgname);
    my $rtname         = $self->get_cb_rtname($proto, 'workloads');
    my $workload_regex = $rgconf->{$rgname}->{workload_regex};

    my ($file, @files);
    if ($proto eq 'cifs') {
        $file = $Options->get(option => "CIFS_WORKLOAD_FILE");
        push(@files, $file);
    }
    if ($proto eq 'nfs') {
        $file = $Options->get(option => "NFS_WORKLOAD_FILE");
        push(@files, $file);
    }
    if ($proto eq 'san') {
        $file = $Options->get(option => "SAN_WORKLOAD_FILE_UNIX");
        if ($file) { push(@files, $file); }
        $file = $Options->get(option => "SAN_WORKLOAD_FILE_WINDOWS");
        if ($file) { push(@files, $file); }
    }

    foreach $file (@files) {
        my $filename = NATE::Inc::find_file($file);
        if (!$filename) {
            die script_result("Cannot file workload file $file");
        }

        my $wfpo = $Options->get(option => "WORKLOAD_PARSER_LIB");

        # No additional error checking is needed, the load will suicide
        # if it doesn't work
        load $wfpo;

        my $parser = $wfpo->new(filename => $filename);

        foreach my $wname (@{$parser->workloads()}) {
            if ($workload_regex && $wname !~ /$workload_regex/) {
                next;
            }
            my $rows = {name => $wname, workload => $wname};
            $rg->add_resources(
                resource_type    => $rtname,
                resourcearrayref => [$rows]
            );
        }
    }
} ## end sub populate_cb_workloads_table

sub filer_setup {

    # Setup is handled by ConfigBroker and setup engines such as engenx or CCL
}

sub filer_cleanup {

    #same as above...topology no longer handles this.
}

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    $Log->exit();
    return;
} ## end sub client_setup

sub DESTROY {
    my $self = shift;
    local ($@);

    $self->client_cleanup();
} ## end sub DESTROY

sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);

    $Log->exit();
} ## end sub client_cleanup

######################### Internal Routines ##################

my $File_read = undef;

sub read_testuser_file {
    my $tfile = shift;
    my $file
        = NATE::Inc::find_file($tfile)
        ? NATE::Inc::find_file($tfile)
        : $tfile;
    $Log->enter("file=>$file");
    if ($File_read) {
        $Log->trace("file already processed");
        goto done;
    }
    my $testuser = $Options->get(option => "TESTUSER");

    if (open(IF, $file)) {
        while (<IF>) {
            next if (/^\s*\#/);
            chomp;
            push(@{$testuser}, $_);
        }
        close(IF);
    } else {
        die "Could not open TESTUSER_FILE: $tfile ($file) : $! / $@";
    }

    $File_read = 1;

done:
    $Log->exit();
} ## end sub read_testuser_file

sub configure_instances {
    $Log->enter();
    my ($self) = @_;

    # Create the client_setup hash its just a very small static hash of a few
    # options that the supervisor will use. client_config contains the
    # relational setup of the clients <-> instances and everything else. I
    # wanted to use the client_setup area for this, but the way that the i
    # topology.pm interacts with that hash makes that too risky.
    my @clients;
    my $cb_protocols = $Options->get(option => "CB_PROTOCOLS");
    $Log->comment(
        "Creating client instances for the protocols: (@{$cb_protocols})");
    foreach my $client ($self->clients()) {
        my $host = Hostrec->new(id => $client);
        my $hosttype = $host->hosttype();
        if (grep(/cifs|san/i, @{$cb_protocols})
            && $hosttype =~ /win/) {
            $Log->comment("Adding $hosttype client $client");
            push(@clients, $client);
        }
        if (grep(/nfs|san/i, @{$cb_protocols}) && $hosttype !~ /win/) {
            $Log->comment("Adding $hosttype client $client");
            push(@clients, $client);
        }
    }

    my $instancesPerClient = $Options->get(option => "TML_WORKER_COUNT");
    my $testUsers          = $Options->get(option => "TESTUSER");
    my $test_password      = $Options->get(option => "TESTPASSWORD");

    # Only look at the IP addresses if this is being called as part
    # of the virtual topology. Use the number of supplied addresses
    # as the default number of synthetic clients to create.
    my $numAddrs;
    my ($package, $filename, $line) = caller;

    my $numberOfInstances = scalar(@clients) * $instancesPerClient;
    $Log->comment(
        "Setting the total number of client processes to $numberOfInstances."
    );
    $Log->comment("Creating $instancesPerClient process for each of the ("
            . scalar(@clients)
            . ") clients.");

    die config_result(
        "I was asked to create zero instances per client. Review configuration"
    ) if ($numberOfInstances == 0);

    $self->{_num_instances} = $numberOfInstances;

    # Spread all the instances among the clients.
    # Spread test users across test instances if given
    my $clientIndex = 0;
    my $userIndex   = 0;
    foreach my $instanceNumber (1 .. $numberOfInstances) {
        my $client       = $self->{client_config}->{$clients[$clientIndex]};
        my $host         = host($client->{name});
        my $instanceHash = {
            client       => $client,
            client_index => $clientIndex,
            name         => $instanceNumber
        };

       # Add the user ID's if we are going to run processes as different users
       # Cycle through the available user IDs
        if ($testUsers->[0]) {
            $instanceHash->{userName} = $testUsers->[$userIndex]
                if ($testUsers->[$userIndex]);
            $userIndex = 0 if (++$userIndex > scalar(@{$testUsers}));
            if (grep(/cifs/i, @{$cb_protocols})
                && $host->hosttype() =~ /win/) {
                $instanceHash->{password} = $test_password;
            }
        }

        # Keep all instances in a single place
        $self->{instances}->{$instanceNumber} = $instanceHash;

        # Link each instance to its physical client
        $self->{client_config}->{$clients[$clientIndex]}->{instances}
            ->{$instanceNumber} = $self->{instances}->{$instanceNumber};

        # Loop through the clients
        $clientIndex = 0 if (++$clientIndex > (scalar(@clients) - 1));

    }

    $Log->exit();
    return 1;
} ## end sub configure_instances

sub get_instances {
    my ($self) = @_;

    my @return = ();

    # Go through every instance, and find the ones using the current
    # usable volume
    foreach my $instance_name (keys(%{$self->{instances}})) {
        push(@return, $self->{instances}->{$instance_name});

    }
    return @return;
} ## end sub get_instances

sub create_client_setup {
    my ($self, $proto) = @_;

    my $addr_type = $Options_hash->{ADDR_TYPE};
    my $device    = $Options_hash->{DEVICE};
    foreach my $client (@{$self->{clients}}) {
        $self->{client_setup}->{$client}->{ADDR_TYPE}  = $addr_type;
        $self->{client_setup}->{$client}->{DEVICE}     = $device;
        $self->{client_setup}->{$client}->{CLIENTNAME} = $client;

        # All the relational stuff is in client_config
        $self->{client_config}->{$client}->{name} = $client;
    }

    return 1;
} ## end sub create_client_setup

#
# Just let the topology decide when this is a
# a good idea.
#
sub start_ip_manager {
    $Log->enter();
    my ($self) = @_;

    my $ipManager;
    my %argsForNew;

    # Because CMode_Virtual is derived from CMode,
    # The potential to start the IPManager twice exists.
    # Don't do that.
    if ($self->{ipManager}) {
        $ipManager = $self->{ipManager};

    } else {
        my $mode = $Options->get(option => "P_EXE_MODE");
        $argsForNew{start_messenger} = 0;

        $Log->trace(
            "Creating a new IPManager. The TestManagerLite mode is $mode.");

        # The IPManager should only be communicating with
        # the IPManager server when it is in manager mode.
        # supervisors get their information via Staph.
        if ($mode =~ /mgr/i) {
            $argsForNew{start_messenger} = 1;
        }

        # Start the client IP manager software
        $ipManager = Utils::IPManager::start_ip_manager(%argsForNew);
    }

    $Log->exit();
    return $ipManager;
} ## end sub start_ip_manager

sub sktrace_setup {
    my ($self) = shift;
    my %skHash;
    my @ops = @{$Options->get(option => "SKTRACE_OPTIONS")};
    my $enable = (scalar(@ops)) ? 1 : 0;

    $Log->comment("SKTRACE_OPTIONS: " . Dumper(\@ops));

    # set this on all nodes in use
    foreach my $node (@nodes) {
        my $bsd = NACL::APISet->new(
            category  => 'Node',
            interface => 'CLI',
            set       => 'Systemshell',
            hostobj   => $node->hostrec()
        );

        # The CDEFS for sysctl are less than awesome. Use the
        # execute_raw_command instead. We just always run cleanup to make sure
        # we are only getting the options that we want, just in case cleanup
        # fails or is skipped
        $Log->comment(
            "Cleaning up the sktrace options on " . $node->hostrec()->id());

        # This is cleanup, start with a dump to finish everything off

        # This little gem resets all your sktrace counters to zero. Don't
        # reset the Nfs4Info counters. They are on by default, leave them
        # that way.
        my $coolCommand
            = 'sudo sysctl sysvar.sktrace | '
            . 'grep _enable: | grep -vi Nfs4Info | '
            . 'grep -v \': 0\' | sed \'s/:.*//\' | '
            . 'xargs -I SYSCTL sudo sysctl SYSCTL=0';
        $bsd->execute_raw_command(command => $coolCommand);

        if ($enable) {
            $Log->comment("Setting up the sktrace options on "
                    . $node->hostrec()->id());

            # We are enabling. Start with a cleari. BURT i
            # http://web.netapp.com/~burt/views/498/498012.html
            # $bsd->execute_raw_command(command =>
            # "sudo sysctl sysvar.sktrace.clear=1");

            foreach my $op (@ops) {
                $bsd->execute_raw_command(command => "sudo sysctl " . $op);
            }
        }
    }
} ## end sub sktrace_setup

sub create_test_options {
    my ($self) = @_;

    # TML will die if these options are are null. Thanks medmonds! :)
    # use the test options to pass the name of the resource group we
    # created to the clients.
    foreach my $instance ($self->get_instances()) {
        my $c        = $instance->{client}->{name};
        my $i        = $instance->{name};
        my $username = $instance->{userName};
        my $password = $instance->{password};
        $self->{test_options}->{$c}->{$i}->{WORKLOAD_PARSER_LIB}
            = $Options->get(option => "WORKLOAD_PARSER_LIB");
        if (host($c)->hosttype() =~ /win/) {
            $self->{test_options}->{$c}->{$i}->{TESTUSER_PASSWORD}
                = $password;
            $self->{test_options}->{$c}->{$i}->{CIFS_WORKLOAD_FILE}
                = $Options->get(option => "CIFS_WORKLOAD_FILE");
            $self->{test_options}->{$c}->{$i}->{SAN_WORKLOAD_FILE}
                = $Options->get(option => "SAN_WORKLOAD_FILE_WINDOWS");
        }
        if (host($c)->hosttype() !~ /win/) {
            $self->{test_options}->{$c}->{$i}->{NFS_WORKLOAD_FILE}
                = $Options->get(option => "NFS_WORKLOAD_FILE");
            $self->{test_options}->{$c}->{$i}->{SAN_WORKLOAD_FILE}
                = $Options->get(option => "SAN_WORKLOAD_FILE_UNIX");

        }

        $self->{test_options}->{$c}->{$i}->{CLIENTNAME}          = $c;
        $self->{test_options}->{$c}->{$i}->{TESTUSER}            = $username;
        $self->{test_options}->{$c}->{$i}->{TML_WORKER_INSTANCE} = $i;

    }
} ## end sub create_test_options

sub get_nacl_nodes {
    my ($self) = @_;

    return \@nodes;
}

sub create_nfs_rg {
}

sub _parse_range {
    my ($self, $range, $total) = @_;
    my ($start, $end);
    if ($range =~ /%/) {

        # If % is specified, if 'start %' has values greater than 1,
        # substract by 1, so you will not miss out any element. But
        # increment the real value of 'start', so that the same element
        # will not be reused. This is based on the assumption, that user
        # does not repeat the numbers. If he does, then it's most likely
        # that's what he wants. E.g "1-30%,30%-40" makes 1% of the lines
        # in 30% range will be redundant.
        $range =~ s/%//;
        my ($startp, $endp) = split("-", $range);
        if ($startp <= 1) {
            $startp = 0;
        } else {
            $startp = $startp - 1;
        }
        $start = int($startp * $total / 100);
        $end   = int($endp * $total / 100);
        if ($endp != 0 && $end == 0) {

            # Set at least one element, if 0 is not intended.
            $end = 1;
        }
        $start = $start + 1;
    } else {

        # No % given, so straight forward to just play with the numbers
        ($start, $end) = split("-", $range);
    }
    return ($start, $end);
} ## end sub _parse_range

# This is a hack to get somthing into the sanpath table. I'm going on the
# assumption that fixed disks in windows are most likely going to be SAN
# devices.
# TODO: Leverage CofigBroker SAN client discovery
sub populate_san_cb_resources_table {
    my ($self, $rgname, $clients, $rgconf) = @_;

    my $cb      = $self->get_cb_object('configbroker');
    my $clus_rg = $self->get_cb_object('cluster');
    my $rg      = $cb->get_resource_group(name => $rgname);
    my $rtname  = $self->get_cb_rtname('san', 'resources');
    my $volumes = $clus_rg->get_resources(type => 'volume');

    # Walk the list of resources and filter by range
    my $client_count = 0;
    my $paths;
    foreach my $client (@{$clients}) {
        my $h = eval { host($client); };
        if ($@) { next; }

        my $hosttype = $h->hosttype();
        my %fields   = $h->param_fields();
        my $os_ver   = $fields{OS_VER};

        if ($hosttype =~ /win/i) {
            my $host = Hostrec->new(id => $client);
            my $conn = $host->connect(
                conntype => "rsh-persay",
                id       => $client
            );

            # TODO: Figure out how to map back to ONTAP
            my $disk_info = $self->get_disk_info($hosttype, $conn);
            my $hv_info;

            my $rtschema = $self->get_cb_rtschema('san', 'resources');
            for (my $i = 0; $i < @{$disk_info}; $i++) {
                my $row;
                foreach my $key (@$rtschema) {
                    $row->{$key} = '';
                }
                $row->{client}    = $client;
                $row->{disk}      = $disk_info->[$i]->{disk};
                $row->{device}    = $disk_info->[$i]->{id};
                $row->{interface} = $disk_info->[$i]->{type};
                $row->{partition} = $disk_info->[$i]->{partition};
                $row->{drive}     = $disk_info->[$i]->{drive};
                $row->{name}
                    = $client . ':'
                    . $disk_info->[$i]->{disk} . ':'
                    . $disk_info->[$i]->{partition};

                # Filter out path table entries. If there are path filters
                # and one of them does not evaulate to true, skip the resource
                my $skip         = 0;
                my $path_filters = $rgconf->{$rgname}->{path_filters};
                foreach my $key (keys(%{$path_filters})) {
                    my $match
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{match};
                    my $regex
                        = $rgconf->{$rgname}->{path_filters}->{$key}->{regex};
                    if ($match eq 'true' && $row->{$key} !~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $row->{name} $key($row->{$key}) !~ /$regex/"
                        );
                        last;
                    }
                    if ($match eq 'false' && $row->{$key} =~ /$regex/) {
                        $skip = 1;
                        $Log->comment(
                            "Skipping $row->{name} $key($row->{$key}) =~ /$regex/"
                        );
                        last;
                    }
                }

                if ($skip && keys(%{$path_filters})) { next; }

                $rg->add_resources(
                    resource_type    => $rtname,
                    resourcearrayref => [$row],
                );
            }
        }
    }

} ## end sub populate_san_cb_resources_table

sub get_hypervisor {
    my ($self, $hosttype, $conn) = @_;
    my $host;
    if ($hosttype =~ /win/i) {
        my $output;
        my $cmd
            = 'reg.exe query "HKLM\SOFTWARE\Microsoft\Virtual Machine\Guest\Parameters" /v HostName';

        eval {
            $output = $conn->execute(
                command          => $cmd,
                input_log_level  => 'debug',
                output_log_level => 'debug',
                timeout          => 60
            );
        };
        foreach my $line (split(/\n/, $output)) {
            if ($line =~ /\s+HostName\s+REG_SZ\s+(\S+)/) {
                $host = $1;
                last;
            }
        }
    }

    return $host;
} ## end sub get_hypervisor

sub get_disk_info {
    my ($self, $hosttype, $conn) = @_;

    my @disk_info;
    if ($hosttype =~ /win/i) {

        my $output;
        my $cmd = 'cscript /nologo x:\\bin\\disk_info.vbs';
        eval {
            $output = $conn->execute(
                command          => $cmd,
                input_log_level  => 'debug',
                output_log_level => 'debug',
                timeout          => 60
            );
        };

        foreach my $line (split(/\n/, $output)) {
            if ($line =~ /(\d+)\s+(\S+)\s+(\S+)\s+(\w+)/) {
                my @fields = split(/\s+/, $line);
                $fields[1] =~ s/(\\|\.)//g;
                $fields[2] =~ s/(\\|\.)//g;
                my $drive;
                if (defined($fields[5])) {
                    $drive = $fields[5];
                    $drive =~ s/://g;
                }
                push(
                    @disk_info,
                    {   disk      => $fields[0],
                        id        => $fields[1],
                        name      => $fields[2],
                        type      => $fields[3],
                        partition => $fields[4],
                        drive     => $drive
                    }
                );
            }
        }
    }

    return \@disk_info;
} ## end sub get_disk_info

sub get_host_info {
    my ($self, $hypervisor, $client) = @_;

    $Log->comment(
        "Getting path information from hypervisor $hypervisor for $client");
    my $host = Hostrec->new(id => $hypervisor);
    my $conn = $host->connect(
        conntype => "rsh-persay",
        id       => $hypervisor
    );
    my (@paths);
    if ($host->hosttype() =~ /win/i) {
        my $output;
        my $cmd
            = 'powershell (get-vm -Name ' 
            . $client
            . ').HardDrives | Format-Table -AutoSize -Property ControllerType,Path | Out-String -Width 4096';
        eval {
            $output = $conn->execute(
                command          => $cmd,
                input_log_level  => 'debug',
                output_log_level => 'debug',
                timeout          => 60
            );
        };
        foreach my $line (split(/\n/, $output)) {
            if ($line =~ /^\w|-/) { next; }
            if ($line =~ /^(\w+)\s+(\S+)/) {
                push(@paths, {type => $1, path => $2});
            }
        }
    }

    return (\@paths);
} ## end sub get_host_info

sub get_resource_filters {
    my ($self, $rgconf, $rgname, $resource, $args) = @_;
    my $match_filters
        = $rgconf->{$rgname}->{resource_filters}->{$resource}->{match};
    foreach my $key (keys(%{$match_filters})) {
        $args->{match_filters}->{$key} = $match_filters->{$key};
    }

    my $regex_filters
        = $rgconf->{$rgname}->{resource_filters}->{$resource}->{regex};
    foreach my $key (keys(%{$regex_filters})) {
        $args->{regex_filters}->{$key} = $regex_filters->{$key};
    }
    $Log->debug("Match Filters:\n" . Dumper($args->{match_filters}));
    $Log->debug("Regex Filters:\n" . Dumper($args->{regex_filters}));
} ## end sub get_resource_filters

sub populate_cb_users_table {

    my ($self, $proto, $rgname, $rgconf) = @_;
    my @objects;
    $Log->comment(
        "Populating users for " . uc($proto) . " resource group $rgname");

    my $cb            = $self->get_cb_object('configbroker');
    my $clus_rg       = $self->get_cb_object('cluster');
    my $clus_rg_users = $clus_rg->get_resources(type => 'user');

    if (!@{$clus_rg_users}) {
        $Log->warn("No users found");
        next;
    } else {
        $Log->comment("Found " . @{$clus_rg_users} . " users");
    }

    # Sort users by id because ConfigBroker does not consistently maintain
    # sort order for CB.users
    my @sorted_users = natkeysort { $_->{id} } @{$clus_rg_users};

    $Log->comment("Creating resource group for $rgname");
    my $rg = $cb->create_resource_group(name => $rgname);
    my $rtschema = $self->get_cb_rtschema($proto, 'users');
    my $rtname = $self->get_cb_rtname($proto, 'users');

    foreach my $row (@sorted_users) {

        # Filter out user table entries. If there are user filters
        # and one of them does not evaulate to true, skip the resource
        my $skip         = 0;
        my $user_filters = $rgconf->{$rgname}->{user_filters};
        foreach my $key (keys(%{$user_filters})) {
            my $match = $rgconf->{$rgname}->{user_filters}->{$key}->{match};
            my $regex = $rgconf->{$rgname}->{user_filters}->{$key}->{regex};
            if ($match eq 'true' && $row->{$key} !~ /$regex/) {
                $skip = 1;
                $Log->debug(
                    "Skipping $row->{name} $key($row->{$key}) !~ /$regex/");
                last;
            }
            if ($match eq 'false' && $row->{$key} =~ /$regex/) {
                $skip = 1;
                $Log->debug(
                    "Skipping $row->{name} $key($row->{$key}) =~ /$regex/");
                last;
            }
        }

        if ($skip && keys(%{$user_filters})) { next; }
        push(@objects, $row);
    }

    my $range = $rgconf->{$rgname}->{user_range};
    if (!$range) { $range = '1-100%'; }
    my ($start, $end) = $self->_parse_range($range, scalar(@objects));

    # Walk the list of users and filter by range
    my $user_count     = 0;
    my $filtered_users = [];
    foreach my $obj (@objects) {
        $user_count++;
        if (   $user_count >= $start
            && $user_count <= $end) {
            $Log->debug("Adding user $user_count "
                    . "$obj->{name} to user group $rgname");
            push(@{$filtered_users}, $obj);
        }
    }

    my $rtname = $self->get_cb_rtname($proto, 'users');
    if (@{$filtered_users} > 0) {
        $Log->comment("Adding "
                . @{$filtered_users}
                . " $rtname objects to ConfigBroker");
        $rg->add_resources(
            resource_type    => $rtname,
            resourcearrayref => $filtered_users
        );
    } else {
        $Log->comment("No $rtname objects found");
    }
    return $filtered_users;
} ## end sub populate_cb_users_table

sub get_volume_from_path {

    $Log->enter();
    my ($self, $path, $vfstores) = @_;

    $Log->debug("Getting volume from path $path");

    # Try the exact match
    $Log->debug("Searching vfstore paths for an exact match");
    my @matches = grep { $_->{path} eq $path } @{$vfstores};

    # Keep deconstructing the path and comparing it against known
    # junction-paths until we find a matching volume. I'm assuming
    # that junction paths are unique
    if (!@matches) {
        $Log->debug("No exact path match found for "
                . "the given path, deconstructing");
        my @elements = split(/\//, $path);
        if (@elements) {
            while (@elements) {
                pop(@elements);
                my $current_path = join('/', @elements);

                # If the current path is undef, this means we are down to the
                # two elements / and <dir>. Reset the current path to be /
                # so we match the root volume.
                if (!$current_path) { $current_path = '/'; }
                $Log->debug("Checking for paths matching $current_path");
                @matches = grep { $_->{path} eq $current_path } @{$vfstores};
                if (@matches) { last; }
            }
        }
    }

    $Log->debug("Matches:");
    foreach my $m (@matches) {
        $Log->debug("$m->{path}");
    }

    if (@matches > 1) {
        $Log->debug("The mount_point $path matches multiple "
                . "paths, Using the first one we found");
    }

    $Log->exit();
    return \@matches;
} ## end sub get_volume_from_path
1;
