#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NFS Stress Scalability 10Mode Topology Library
##
## @author      gvg@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::NFS::10M - NFS Stress Scalability test Topology Library for 10Mode ONTAP

=head1 SYNOPSIS

    See Topology.pm

=head1 DESCRIPTION

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Some probably exist but have not been listed yet.

=head1 STATUS

  Shared

=head1 AUTHOR

  GV Govindasamy (gvg@netapp.com)
  $Id: $
  Last revised $Date: 2008/09/30 $
    
=head1 COPYRIGHT

Copyright 2008, Network Appliance., All Rights Reserved

=cut

package Topology::NAS::NFS::10M;
use base qw(Topology::NAS::NFS);

use strict;
use warnings;

use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Server;
use SayWrap qw(sayEOF);
use Tharn qw($Log config_result);

use Data::Dumper qw(Dumper);

use Options;
my $Options;
my $Options_hash;

my $Server;

###################################################################

$Options = new Options;
$Options->parameters(
    definitions => [
        {name         => "VOL_TYPE",
         spec         => "s",
         description  => "The type of volume to be created for testing",
         valid_values => ["flex", "striped", "mixed"],
         default      => undef,},
        {name         => "VOL_STRIPE_FORMAT",
         spec         => "s",
         description  => "The format type used for striped volumes",
         valid_values => ["full", "data"],
         default      => undef,},
        {name        => "TESTUSER",
         spec        => "s@",
         description => "List of test users to use for the testing",
         default     => undef,},
        {name        => "TESTUSER_FILE",
         spec        => "s",
         description => "file containing the list of testusers who test run as",
         file_tests  => "fr",
         subroutine  => [\&_read_testuser_file],},
        {name => "NUM_EXPORT_RULES",
         spec => "s",
         description =>
           "The number of export rules per export policy to be configured",
         default => 1,},
        {name        => "VSERVER",
         spec        => "s",
         description => "VSERVER to use",
         required => "Enter the VSERVER test should run against on the cluster",
         default  => "vs0",},
        {name        => "MNTPNT_MODE",
         description => "The mode to set the client mountpoints to.",
         default     => "777",},
        {name => "VOLPATHS_FILE",
         spec => "s",
         description =>
           "file containing the list of volume paths for clients to mount to",
         file_tests => "fr",
         subroutine => [\&_read_volpaths_file],},]);

sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;
    my $self     = {@_,};
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
} ## end sub new

sub define_topology {
    my $self = shift;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::define_topology(@_);

    $Options_hash = $Options->get_hash();

    # Options used here
    my $num_exports          = $Options_hash->{NUM_EXPORTS};
    my $instances_per_client = $Options_hash->{INSTANCES_PER_CLIENT};
    my $vol_size =
      $Options_hash->{VOL_SIZE};    # VOL_SIZE is considered per instance
    my @volumes = $self->volumes();

    my $max_nfs_clients = $Options_hash->{MAX_NFS_CLIENTS};
    $Log->debug("Options: \n" . Dumper($Options_hash));

    #
    # Spread Test Instances across available physical clients
    #

    my @clients = $self->clients();
    my $total_instances;

    my $instances;
    my $remainders;
    if ($max_nfs_clients) {
        if ($instances_per_client * scalar(@clients) < $max_nfs_clients) {
            die config_result(
                    "INSTANCES_PER_CLIENT($instances_per_client) * CLIENT("
                  . scalar(@clients)
                  . ") < MAX_NFS_CLIENTS($max_nfs_clients). Please adjust these variables and run the test."
                  );
        } else {
            $instances  = int($max_nfs_clients / scalar(@clients));
            $remainders = $max_nfs_clients % scalar(@clients);
            $Log->comment(
                "INSTANCES_PER_CLIENT will be adjusted from $instances_per_client to $instances(+1) "
                );
        }
    } else {
        $instances  = $instances_per_client;
        $remainders = 0;
    }
    foreach my $client (@clients) {
        my $inst = 1;
        for (; $inst <= $instances ; $inst++) {

            # Get instances in here, so we can fill in the values in the next block
            $self->{test_options}{$client}{$inst}{VOLPATH} = "";
            $total_instances++;
        }
        if ($remainders) {
            $self->{test_options}{$client}{$inst}{VOLPATH} = "";
            $remainders--;
            $total_instances++;
        }
    } ## end foreach my $client (@clients)

    #
    # Spread volume/dir layout across test instances
    #

    my $size = $self->size_in_mb(size => $vol_size);
    $size =
      ($size) ? $size : 30;    # Set 30MB per instance, in case if none is set.

    my $instances_per_volume = int($total_instances / scalar(@volumes));
    my $remainder_vols       = $total_instances % scalar(@volumes);
    if ($instances_per_volume < 1) {
        die config_result(
            "The number of test instances $total_instances are given lesser than the volumes "
              . scalar(@volumes)
              . ". Please verify your options");
    }

    if ((!$remainder_vols) && ($instances_per_volume == 1)) {
        $Log->comment(
                    "number of volumes and number of test instances are equal");

        my @vols = @volumes;
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                my $volume = pop @vols;
                $self->{filer_setup}{volume}{$volume}{name}    = $volume;
                $self->{filer_setup}{volume}{$volume}{size}    = $size;
                $self->{test_options}{$client}{$inst}{VOLPATH} = "/" . $volume;
            }
        }
    } else {
        $Log->comment(
                     "we have more than one instances sharing the same volume");

        my $volume;
        my $instances = 0;
        my @vols      = @volumes;
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                if (!$instances) {
                    if (!($volume = pop @vols)) {
                        die config_result(
                            "${client}_{$inst} couldn't find volume to run on. Some config/logic issue"
                            );
                    }
                    $instances = $instances_per_volume;
                    if ($remainder_vols) {
                        $instances++;
                        $remainder_vols--;
                    }
                    $self->{filer_setup}{volume}{$volume}{name} = $volume;
                    $self->{filer_setup}{volume}{$volume}{size} =
                      $size * $instances;
                } ## end if (!$instances)

                my $dir = "${client}_${inst}";
                push @{$self->{filer_setup}{dirs}{$volume}}, $dir;
                $self->{test_options}{$client}{$inst}{VOLPATH} =
                  "/${volume}/${dir}";

                $instances--;
            } ## end foreach my $inst (keys %{$self...
        } ## end foreach my $client (@clients)
    } ## end else [ if ((!$remainder_vols)...

    #
    # Export Layout for all test instances
    #
    if ($num_exports > scalar(@volumes)) {
        die config_result("NUM_EXPORTS > NUM_VOLS. Failed topology");
    }

    my @rulesets;
    for (my $i = 0 ; $i < $num_exports ; $i++) {
        if ($Options_hash->{EXPORT_OPTIONS}) {
            my %hash_opt = eval "$Options_hash->{EXPORT_OPTIONS}";

            $hash_opt{rulesetname} = $self->testcase_string()."_custom_" . $i
              if (!$hash_opt{rulesetname});

            push(@{$self->{filer_setup}{exportruleset}},
                 {vserver     => $Options_hash->{VSERVER},
                  rulesetname => $hash_opt{rulesetname},});

            push(@{$self->{filer_setup}{exportruleset_rules}}, {%hash_opt,});

            push @rulesets, $hash_opt{rulesetname};
        } else {
            if ($i == 0) {

                # This rule is added later
                push @rulesets, $self->testcase_string()."_allow_all";
                next;
            }

            push(@{$self->{filer_setup}{exportruleset}},
                 {vserver     => $Options_hash->{VSERVER},
                  rulesetname => $self->testcase_string()."_allow_all_" . $i,});

            push(@{$self->{filer_setup}{exportruleset_rules}},
                 {ruleindex   => 1,
                  vserver     => $Options_hash->{VSERVER},
                  rulesetname => $self->testcase_string()."_allow_all_" . $i,
                  anon        => 0,
                  protocol    => "any",
                  clientmatch => "0.0.0.0/0",
                  rorule      => "any",
                  rwrule      => "any",
                  superuser   => "any",});

            push @rulesets, $self->testcase_string()."_allow_all_" . $i;
        } ## end else [ if ($Options_hash->{EXPORT_OPTIONS...
    } ## end for (my $i = 0 ; $i < $num_exports...

    my @rules;
    foreach my $volume (@volumes) {
        @rules = @rulesets if (!@rules);
        my $ruleset = pop @rules;
        my $opts = {vserver => $Options_hash->{VSERVER},
                    policy  => $ruleset,
                    volume  => $volume};
        $self->{filer_setup}{exports}{$volume} = $opts;
    }

    # Create export rule $self->testcase_string()."_allow_all" created.
    push(@{$self->{filer_setup}{exportruleset}},
         {vserver     => $Options_hash->{VSERVER},
          rulesetname => $self->testcase_string()."_allow_all",});

    push(@{$self->{filer_setup}{exportruleset_rules}},
         {ruleindex   => 1,
          vserver     => $Options_hash->{VSERVER},
          rulesetname => $self->testcase_string()."_allow_all",
          anon        => 0,
          protocol    => "any",
          clientmatch => "0.0.0.0/0",
          rorule      => "any",
          rwrule      => "any",
          superuser   => "any",});

    #
    # Spread test users across test instances if given
    #
    my $testusers = $Options_hash->{TESTUSER};
    my $index     = 0;
    if (my $num_users = scalar(@$testusers)) {
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                $self->{test_options}{$client}{$inst}{TESTUSER} =
                  $testusers->[$index];
                $index = 0 if (++$index >= $num_users);
            }
        }
    }

    $self->{_num_instances} = $total_instances;

    $Log->debug(Dumper($self));
    $Log->exit();
} ## end sub define_topology

sub filer_setup {
    my ($self, %options) = @_;

    $Options_hash = $Options->get_hash();

    # Execute the one from the parent
    $self->SUPER::filer_setup(%options);

    $Server = new Server;

    # Define the base list of command nodes to discover in the cluster
    my @discovery_nodes = qw(aggregate vserver network_interfaces);

    $self->{discovered_topology} =
      $Server->discover_topology(cmd_nodes => \@discovery_nodes);
    $Log->debug("Discovered Topology: " . Dumper($self->{discovered_topology}));

    my @discovered_vifs =
      @{$self->{discovered_topology}{cluster}{network_interfaces}};

    my @data_vifs;
    foreach my $vif (@discovered_vifs) {
        if ($vif->{role} eq "data" && $vif->{"status-admin"} eq "up") {
            $Log->debug("Adding vif '$vif->{address}' to interfaces list");
            push(@data_vifs, $vif->{address});
        }
    }
    $Log->comment("Available data interfaces: ".join(",", @data_vifs));

    if (!@{$Options_hash->{DATA_INTERFACES}}) {
        $Options->set(DATA_INTERFACES => \@data_vifs);
        $Options_hash = $Options->get_hash();
    }
    $Log->comment("Chosen DATA_INTERFACES are: ".join(",", @{$Options_hash->{DATA_INTERFACES}}));

    my $test_topology = $self->_update_test_topology();
    $Server->create_topology(print_node => undef, topology => $test_topology);

    my @clients = $self->clients();

    # Enable Dedupe for Volumes
    # ???

    # Create required directories
    my $dirsref = $self->{filer_setup}{dirs};
    if (keys %$dirsref) {
        $Log->comment("Directories need to be created on volumes");
        foreach my $volume (keys %$dirsref) {

            # If volume already exists and exported with restricted permission
            # create_topology() didn't seem to change anything, hence we need
            # override here so dir creation can succeed. This shouldn't be a
            # problem, since we will reexport with test required export policy
            # anyways.
            $Server->volume_modify(vserver => $Options_hash->{VSERVER},
                                   volume  => $volume,
				   -Timeout => 120,
                                   policy  => $self->testcase_string()."_allow_all");

        } ## end foreach my $volume (keys %$dirsref)

        $self->create_dirs(@{$Options_hash->{DATA_INTERFACES}}[0],
                           \@clients, $dirsref);
    } else {
        $Log->comment(
                "Directories weren't required to be created for this Topology");
    }

    # Setup exports appropriately

    foreach my $export (keys %{$self->{filer_setup}{exports}}) {
        my $opts = $self->{filer_setup}{exports}{$export};
        $Server->volume_modify(%$opts, -Timeout => 120);
    }

    # Set this option, so clients using non-previlaged ports also will have access
    # ????

    $Log->exit();

} ## end sub filer_setup

sub filer_cleanup {
    my ($self, %options) = @_;

    my $test_topology = $self->{test_topology};

    # create a list of all node resources in the test_topology
    my @resources;
    foreach my $resource_ref (values %{$test_topology->{cluster}}) {
        push(@resources, @{$resource_ref})
          if (ref($resource_ref) eq "ARRAY");
    }
    foreach my $node (keys %{$test_topology->{node}}) {
        foreach my $res_ref (values %{$test_topology->{node}{$node}}) {
            push(@resources, @{$res_ref}) if (ref($res_ref) eq "ARRAY");
        }
    }

    # check all of the resources for a comment that indicates that the resource
    # was previously created by TestManager. Mark this resource for cleanup
    foreach my $resource (@resources) {
        next if (ref($resource) ne "HASH");
        if (   $resource->{comment}
            && $resource->{comment} =~ "created by TestManagerLite") {
            $resource->{Delete_Resource} = 1;
        } elsif ($resource->{rulesetname}) {
            $resource->{Delete_Resource} = 1;
        }
    }

    # In case, test did SFO TO/GB stuff, or reboot the nodes, we may have lost the
    # connections to the node. Now, it's time to remind Server to handle the connections
    # correctly so that we will be successful cleaning up the resources.
    $Server->handle_connections(close => [qw(console_connection shell_connection)]);

    $Server->delete_topology(print_node => undef, topology => $test_topology);

    # Execute the one from the parent
    $self->SUPER::filer_cleanup(%options);
} ## end sub filer_cleanup

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    $Log->exit();
}

sub DESTROY {
    my $self = shift;
    local ($@);

    $self->client_cleanup();
}

sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);

    $Log->exit();
}

sub _update_test_topology {
    my $self = shift;

    $Log->enter();
    $Options_hash = $Options->get_hash();

    my $test_topology;

    # Make sure given vserver exists on the cluster
    my $vserver_exists = 0;
    foreach my $vserver (@{$self->{discovered_topology}{cluster}{vserver}}) {
        if ($vserver->{vserver} eq $Options_hash->{VSERVER}) {
            $vserver_exists = 1;
        }
    }
    die config_result(
          "Given vserver $Options_hash->{VSERVER} doesn't exist on the cluster")
      if (!$vserver_exists);

    # Find the minimum size required for an aggr using min vol size
    my $min_size = 300;
    foreach my $vol (keys %{$self->{filer_setup}{volume}}) {
        if ($min_size < $self->{filer_setup}{volume}{$vol}{size}) {
            $min_size = $self->{filer_setup}{volume}{$vol}{size};
        }
    }

    my %args = (state      => "online",
                avail_size => "$min_size");
    $args{voltype} = $Options_hash->{VOL_TYPE}
      if (defined($Options_hash->{VOL_TYPE}));
    my @Aggrs = $Server->pick_aggregates(%args);
    $Log->debug("Auto Picked Aggrs: " . Dumper(\@Aggrs));

    my @Aggregates;
    if (@{$Options_hash->{aggregate}}) {

        # If aggrs are user-specified, then make sure they exist, otherwise fail
        foreach my $aggr (@{$Options_hash->{aggregate}}) {
            my $found = 0;
            foreach my $ag_hash (@Aggrs) {
                if ($ag_hash->{aggregate} eq $aggr) {
                    $found = 1;
                    push @Aggregates, $ag_hash;
                }
            }
            die config_result(
                    "Specified Aggr $aggr doesn't exist with size $min_size MB")
              if (!$found);
        } ## end foreach my $aggr (@{$Options_hash...
    } else {
        push @Aggregates, @Aggrs;
    }
    $Log->debug("Usable Aggr hash: " . Dumper(\@Aggregates));

    my $aggrs = $self->{discovered_topology}{cluster}{aggregate};

    my $orig_server;
    if (my @clustered = $Server->get_clustered_nodes()) {
        $orig_server = $Server->set_execution_server(-Server => $clustered[0]);
    }

    my $volume_number = 0;
    foreach my $volume (keys %{$self->{filer_setup}{volume}}) {
        my $vol_hash;
        $vol_hash->{volume} = $volume;
        my $vol_size = $self->{filer_setup}{volume}{$volume}{size};
        if ($vol_size < 300) {
            $Log->comment(
                "Vol size is adjusted from $vol_size to 300 to keep min size of 300MB per volume"
                );
            $vol_size = 300;
        }
        $vol_hash->{size} = $vol_size . "MB";

        # see if the volume already exists in the cluster
        my $existing_volume;
        eval { $existing_volume = $Server->volume_show(volume => $volume, -Timeout => 120) };

        # make sure that we have a volume array and that the volume hash is a real one.
        # For 7G, a volume_hash gets generated and is not complete....
        if ($existing_volume && $existing_volume->[0]{volume}) {
            $existing_volume = $existing_volume->[0];

            # find info on the hosting aggregate
            my ($aggr_hash) =
              grep($_->{aggregate} eq $existing_volume->{aggregate},
                   @Aggregates);
            if (ref($aggr_hash) ne "HASH") {
                $Log->comment(Dumper $existing_volume);
                $Log->comment(Dumper \@Aggregates);
                die config_result(
                    "Could not find the aggregate hosting volume '$existing_volume->{volume}'"
                    );
            }
            foreach my $key (keys %{$existing_volume}) {

                # Don't cache volume params that are "-" or blank
                $vol_hash->{$key} = $existing_volume->{$key}
                  if ($existing_volume->{$key} !~ /^-$|^$/);
            }
            next;
        } ## end if ($existing_volume &&...

        # fill in missing aggregate information
        my @usable_aggrs;
        foreach my $agr (@Aggregates) {
            if ($agr->{availsize} >= $vol_size) {
                push(@usable_aggrs, $agr);
            }
        }

        my $aggr_info;
        map($aggr_info .= "$_->{aggregate} => $_->{availsize}\n",
            @usable_aggrs);
        $Log->trace($aggr_info);

        if (!scalar(@usable_aggrs)) {
            die config_result(
                           "No aggregates with sufficient space are available, "
                             . $vol_hash->{size}
                             . " needed.");
        }

        # pick aggregates in a round-robin fashion rather than random
        my $agr_hash = $usable_aggrs[$volume_number++ % (scalar @usable_aggrs)];
        $agr_hash->{availsize} -= $vol_size;
        $vol_hash->{aggregate} = $agr_hash->{aggregate}
          if (!$vol_hash->{aggregate});

        $Log->trace(
            "placed volume '$vol_hash->{volume}' on aggregate '$vol_hash->{aggregate}': its size is now $agr_hash->{availsize}"
            );

        $vol_hash->{vserver} = $Options_hash->{VSERVER};

	# Set volume stripe format
	if (defined($Options_hash->{VOL_STRIPE_FORMAT})) {
	    $vol_hash->{'stripe-format'} = $Options_hash->{VOL_STRIPE_FORMAT};
	}
	
        # fill in missing security_style information. It defaults to "unix" unless specified.
        if (!$vol_hash->{'security-style'}) {
            $vol_hash->{'security-style'} = $$Options_hash{VOL_SECURITY_STYLE}
              || "unix";
        }

        # fill in user/group information
        if (   $$Options_hash{VOLUME_OWNER}
            && $$Options_hash{VOLUME_OWNER} eq 'self') {
            $vol_hash->{user} = $ENV{LOGNAME};
            (undef(),            undef(), undef(),
             $vol_hash->{group}, undef(), undef(),
             undef(),            undef(), undef()) = getpwnam($ENV{LOGNAME});
        } else {
            $vol_hash->{user}  = 0 if (!$vol_hash->{user});
            $vol_hash->{group} = 0 if (!$vol_hash->{group});
        }

        # fill in junction-path information
        $vol_hash->{'junction-path'} = "/$volume";
        $vol_hash->{ruleset} = $self->testcase_string()."_allow_all";

        $vol_hash->{'unix-permissions'} = $Options_hash->{MNTPNT_MODE};

        # Indicate that it was created by TestManagerLite
        $vol_hash->{comment} = '"created by TestManagerLite"';

        push @{$test_topology->{cluster}{volume}}, $vol_hash;
    } ## end foreach my $volume (keys %{...

    # Push exportrule and rules required to test_topology
    push(@{$test_topology->{cluster}{exportruleset}},
         @{$self->{filer_setup}{exportruleset}});

    push(@{$test_topology->{cluster}{exportruleset_rules}},
         @{$self->{filer_setup}{exportruleset_rules}});

    $self->{test_topology} = $test_topology;

    $Log->debug("Test Topology: \n" . Dumper($test_topology));

    $Log->exit();
    return $test_topology;
} ## end sub _update_test_topology

######################### Internal Routines ##################

my $File_read = undef;

sub _read_testuser_file {
    my $tfile = shift;
    my $file =
      NATE::Inc::find_file($tfile) ? NATE::Inc::find_file($tfile) : $tfile;
    $Log->enter("file=>$file");
    if ($File_read) {
        $Log->trace("file already processed");
        goto done;
    }
    my $testuser = $Options->get(option => "TESTUSER");

    if (open(IF, $file)) {
        while (<IF>) {
            next if (/^\s*\#/);
            chomp;
            push(@{$testuser}, $_);
        }
        close(IF);
    } else {
        die "Could not open TESTUSER_FILE: $tfile ($file) : $! / $@";
    }

    $File_read = 1;

  done:
    $Log->exit();
} ## end sub _read_testuser_file

1;
