#Hey emacs, this is basically -*- perl -*-
# $Id: $

## @summary     NFS Stress Scalability 7Mode Topology Library
##
## @author      gvg@netapp.com
##
## @pod here

=head1 NAME

Topology::NAS::NFS::7M - NFS Stress Scalability test Topology Library for 7Mode ONTAP

=head1 SYNOPSIS

    use qw(Topology);
    my $topology = new Topology(topology=>"Topology::NAS::NFS::7M");

=head1 DESCRIPTION

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/TestManagerLite

More detailed documentation will be available at
http://wikid.netapp.com/w/Stress-Scalability_QA/Topology

=head1 TODO

=head1 SEE ALSO

Options TestManagerLite Topology

=head1 BUGS

Some probably exist but have not been listed yet.

=head1 STATUS

  Shared

=head1 AUTHOR

  GV Govindasamy (gvg@netapp.com)
  $Id: $
  Last revised $Date: 2008/05/23 $
    
=head1 COPYRIGHT

Copyright 2008, Network Appliance., All Rights Reserved

=cut

package Topology::NAS::NFS::7M;
use base qw(Topology::NAS::NFS);

use strict;
use warnings;

use Module::Common qw(validate_mixed);
use Params::Validate qw(SCALAR HASHREF ARRAYREF OBJECT UNDEF CODEREF);
use Server;
use nfs::GenericExport qw(export_path);
use Utils qw(make_conn);
use SayWrap qw(sayEOF);
use Tharn qw($Log config_result snooze host);
use filer qw(filer_options_set);
use Dense qw(dense_start dense_toggle);
use Data::Dumper qw(Dumper);
use Network::Vlan qw(vlan_create vlan_delete);
use Network::Utils qw(do_ifconfig);
use Filer::Version qw(get_wafl_version WV_BOILERMAKER_MIN);
use Options;
my $Options;
my $Options_hash;
my @vlan_addrs;
my $last_vlan_id;
my @filers;
use RemoteExecution;
my $Remote = new RemoteExecution;
my ($Options_hash, %apiset, $Response, %conn);
use NACL::APISet;
use NACL::C::Volume;
use NACL::C::Node;
use NACL::APISet::Exceptions::ResponseException qw(:try);
use NACL::APISet::Exceptions::FeatureNotLicensedException qw(:try);

my $Server;

###################################################################

$Options = new Options;
$Options->parameters(
    definitions => [
        {name        => "TESTUSER",
         spec        => "s@",
         description => "List of test users to use for the testing",
         default     => undef,},
        {name        => "TESTUSER_FILE",
         spec        => "s",
         description => "file containing the list of testusers who test run as",
         file_tests  => "fr",
         subroutine  => [\&_read_testuser_file],},
        {name        => "SIS_ENABLE",
         spec        => "s",
         description => "Param that is used to enable the sis volumes",
         default     => "0",},
        {name => "QUOTA_VOLUME_ENABLE",
         spec => "boolean",
         description =>
           "Enable a volume quota for each user that will utilize said volume",
         default => 0,},
        {name => "QUOTAS",
         spec => "s",
         description =>
           "Param that contains target:type:disk:files:thold:sdisk:sfile",},
        {name        => "MAX_VLAN_ENABLE",
         spec        => "boolean",
         description => "Enable max vlan for each controller",
         default     => 0,},
        {name => "VLAN_ADDRS",
         spec => "s",
         description =>
           "Starting IP and number of IPs to be assigned to vlan separated by '/'",
         default => undef,},
        {name        => "VLAN_IDS",
         spec        => "s",
         description => "Starting Vlan ID",
         default     => undef,},
        {name        => "VLAN_NETMASK",
         spec        => "s",
         description => "Netmask that uses for vlan setup",
         default     => "255.255.192.0",},
        {name        => "VLAN_INTERFACE",
         spec        => "s",
         description => "The interface that is used for creating vlans",
         default     => undef,},
        {name => "SKTRACE_OPTIONS",
         spec => "s@",
         description =>
           "A list of values to be modified with the sysctl command. Example: SKTRACE_OPTIONS=sysvar.sktrace.clear=1,sysvar.sktrace.Nfs4Lock_enable=63"
        },
        {name => "VOLPATHS_FILE",
         spec => "s",
         description =>
           "file containing the list of volume paths for clients to mount to",
         file_tests => "fr",
         subroutine => [\&_read_volpaths_file],},]);

sub new {
    my $invocant = shift;
    my $class    = ref($invocant) || $invocant;
    my $self     = {@_,};
    $Log->enter();
    bless($self, $class);

    $Log->exit();
    return $self;
} ## end sub new

sub define_topology {
    my $self = shift;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::define_topology(@_);

    $Options_hash = $Options->get_hash();

    @filers = $self->filers();
    if (scalar(@filers) > 1) {
        die config_result(
             "This Topology supports only one FILER specified. Given: @filers");
    }

    # Make 7Mode and ZAPI APISet connections for all nodes and store
    # for reuse
    foreach my $node (@filers) {

        $Log->comment("Creating CLI connection object for $node");
        my $Host = Tharn::host("$node");
        $apiset{'cli'}{$node} =
          NACL::APISet->new(hostobj   => $Host,
                            category  => "Node",
                            interface => "CLI",
                            set       => "7Mode",
                            cdef_path => $Options_hash->{'CDEF_PATH'});
        $Log->comment("Creating ZAPI connection object for $node");
        $apiset{'zapi'}{$node} =
          NACL::APISet->new(hostobj   => $Host,
                            category  => "Node",
                            interface => "ZAPI",
                            set       => "7Mode",
                            cdef_path => $Options_hash->{'CDEF_PATH'});
    } ## end foreach my $node (@filers)

    # Get specified vlan addrs
    @vlan_addrs = $self->_create_range($Options->get(option => "VLAN_ADDRS"));

    # Options used here
    my $num_exports          = $Options_hash->{NUM_EXPORTS};
    my $instances_per_client = $Options_hash->{INSTANCES_PER_CLIENT};
    my $dir_per_instance     = $Options_hash->{DIR_PER_INSTANCE};
    my $vol_size =
      $Options_hash->{VOL_SIZE};    # VOL_SIZE is considered per instance
    my @volumes = $self->volumes();
    my $export_options =
      ($Options_hash->{EXPORT_OPTIONS})
      ? $Options_hash->{EXPORT_OPTIONS}
      : "rw,anon=0";
    my $max_nfs_clients = $Options_hash->{MAX_NFS_CLIENTS};
    $Log->comment("Options: \n" . Dumper($Options_hash));

    #
    # Spread Test Instances across available physical clients
    #

    my @clients = $self->clients();
    my $total_instances;

    my $instances;
    my $remainders;
    if ($max_nfs_clients) {
        if ($instances_per_client * scalar(@clients) < $max_nfs_clients) {
            die config_result(
                    "INSTANCES_PER_CLIENT($instances_per_client) * CLIENT("
                  . scalar(@clients)
                  . ") < MAX_NFS_CLIENTS($max_nfs_clients). Please adjust these variables and run the test."
                  );
        } else {
            $instances  = int($max_nfs_clients / scalar(@clients));
            $remainders = $max_nfs_clients % scalar(@clients);
            $Log->comment(
                "INSTANCES_PER_CLIENT will be adjusted from $instances_per_client to $instances(+1) "
                );
        }
    } else {
        $instances  = $instances_per_client;
        $remainders = 0;
    }
    foreach my $client (@clients) {
        my $inst = 0;
        for (; $inst < $instances ; $inst++) {

            # Get instances in here, so we can fill in the values in the next block
            $self->{test_options}{$client}{$inst}{VOLPATH} = "";
            $total_instances++;
        }
        if ($remainders) {
            $self->{test_options}{$client}{$inst}{VOLPATH} = "";
            $remainders--;
            $total_instances++;
        }
    } ## end foreach my $client (@clients)

    #
    # Spread volume/dir layout across test instances
    #

    my $size = $self->size_in_mb(size => $vol_size);
    $size =
      ($size) ? $size : 30;    # Set 30MB per instance, in case if none is set.

    my $instances_per_volume = int($total_instances / scalar(@volumes));
    my $remainder_vols       = $total_instances % scalar(@volumes);
    if ($instances_per_volume < 1) {

        $Log->warn(
            "Number of test instances $total_instances is lesser than the number of volumes scalar(@volumes) "
            );

        # Should clients go through a selected volumes
        my @vols = @volumes;
        my $volume;
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                $volume = pop @vols;
                $self->{filer_setup}{volume}{$volume}{name} = $volume;
                $self->{filer_setup}{volume}{$volume}{size} =
                  $size * $instances;
                if ($dir_per_instance) {
                    my $dir = "${client}_${inst}";
                    push @{$self->{filer_setup}{dirs}{$volume}}, $dir;
                    $self->{test_options}{$client}{$inst}{VOLPATH} =
                      "/vol/${volume}/${dir}";
                } else {
                    $self->{test_options}{$client}{$inst}{VOLPATH} =
                      "/vol/${volume}";
                }
                $self->{test_options}{$client}{$inst}{VOLUME} = $volume;
            } ## end foreach my $inst (keys %{$self...
        } ## end foreach my $client (@clients)

        # Fill in left over volume for creating and exporting purposes
        if (@vols) {
            foreach my $volume (@vols) {
                $self->{filer_setup}{volume}{$volume}{name} = $volume;
                $self->{filer_setup}{volume}{$volume}{size} = $size;
            }
        }
    } elsif ($instances_per_volume >= 1) {
        my @vols = @volumes;
        $Log->debug("Setting up vol options for : @vols");
        my $volume;
        $instances = 0;

        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                if (!$instances && !($volume = pop @vols)) {
                    die config_result(
                        "${client}_{$inst} couldn't find volume to run on. Some config/logic issue"
                        );
                }

                if (!$instances) {
                    $instances = $instances_per_volume;
                    if ($remainder_vols) {
                        $instances++;
                        $remainder_vols--;
                    }
                    $self->{filer_setup}{volume}{$volume}{name} = $volume;
                    $self->{filer_setup}{volume}{$volume}{size} =
                      $size * $instances;
                } ## end if (!$instances)

                if ($dir_per_instance) {
                    my $dir = "${client}_${inst}";
                    push @{$self->{filer_setup}{dirs}{$volume}}, $dir;
                    $self->{test_options}{$client}{$inst}{VOLPATH} =
                      "/vol/${volume}/${dir}";
                } else {
                    $self->{test_options}{$client}{$inst}{VOLPATH} =
                      "/vol/${volume}";
                }
                $self->{test_options}{$client}{$inst}{VOLUME} = $volume;
                $instances--;
            } ## end foreach my $inst (keys %{$self...
        } ## end foreach my $client (@clients)
    } ## end elsif ($instances_per_volume...

    #
    # Export Layout for all test instances
    #
    if ($num_exports == $total_instances) {

        # Number of test instances == number of exports. This means each instance will run
        # against exclusive directory/volume with exclusive export
        $Log->comment("An export per test instance detected");

        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                my $volpath = $self->{test_options}{$client}{$inst}{VOLPATH};
                $self->{filer_setup}{exports}{$volpath}{options} =
                  $export_options;
            }
        }
    } elsif ($num_exports < $total_instances) {
        if ($num_exports == scalar(@volumes)) {

            # Export each volume separately
            $Log->comment(
                "An export per volume is detected with export_options $export_options"
                );
            foreach my $volume (@volumes) {
                my $export = "/vol/" . $volume;
                $self->{filer_setup}{exports}{$export}{options} =
                  $export_options;
            }
        } elsif ($num_exports > scalar(@volumes)) {

            # ??? - GV
            # Not sure if there are any other better ways to handle this. For now, export all volumes first
            # to make sure all are usable, then export sub directories until reach $num_exports

            my $exports = $num_exports;
            $Log->comment("exports $num_exports > volumes "
                . scalar(@volumes)
                . " detected. Exporting volumes first and then export directories"
                );
            foreach my $volume (@volumes) {
                my $export = "/vol/" . $volume;
                $self->{filer_setup}{exports}{$export}{options} =
                  $export_options;
                $exports--;
            }
            foreach my $client (@clients) {
                foreach my $inst (keys %{$self->{test_options}{$client}}) {
                    my $volpath =
                      $self->{test_options}{$client}{$inst}{VOLPATH};
                    $self->{filer_setup}{exports}{$volpath}{options} =
                      $export_options;
                    $exports--;
                    last if (!$exports);
                }
                last if (!$exports);
            } ## end foreach my $client (@clients)
        } else {
            die config_result(  "This doesn't make sense. NUM_VOLS("
                              . scalar(@volumes)
                              . ") && NUM_EXPORTS ($num_exports)");
        }
    } else {
        die config_result("This doesn't make sense. NUM_VOLS("
             . scalar(@volumes)
             . ") && NUM_EXPORTS ($num_exports) && INSTANCES ($total_instances)"
             );
    }

    # Spread test users across test instances if given
    my $testusers = $Options_hash->{TESTUSER};
    my $index     = 0;
    if (my $num_users = scalar(@$testusers)) {
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                $self->{test_options}{$client}{$inst}{TESTUSER} =
                  $testusers->[$index];
                $index = 0 if (++$index >= $num_users);
            }
        }
    }

    $self->{_num_instances} = $total_instances;

    # Configure the sktracing if it is applicable. Normally something like this
    # should be placed in filer setup. However, I have decided to stick it here in
    # define_topology because I want to be sure that sktracing is set correctly even
    # when the topology is run with SKIPSTAGE=filer_setup to use an existing setup
    if ($Options->get(option => "SKTRACE_OPTIONS")) {
        $self->sktrace_setup(enable => 1);
    }

    $Log->comment(Dumper($self));
    $Log->exit();

} ## end sub define_topology

sub filer_setup {
    my ($self, %options) = @_;

    my $filer = ($self->filers())[0];

    # Execute the one from the parent
    $self->SUPER::filer_setup(%options);

    $Server = new Server;

    # Create conn object for the filer
    my $conn = make_conn($filer);

    # Define the base list of command nodes to discover in the cluster
    my @discovery_nodes = qw(aggregate vserver network_interfaces);

    $self->{discovered_topology} =
      $Server->discover_topology(cmd_nodes => \@discovery_nodes);
    $Log->debug("Discovered Topology: " . Dumper($self->{discovered_topology}));

    my $test_topology = $self->_update_test_topology();
    $Server->create_topology(print_node => undef, topology => $test_topology);

    my @clients = $self->clients();
    my $data_int;

    #Enable Dedupe for Volumes
    if ($Options_hash->{SIS_ENABLE} != 0) {
        my @volumes = $self->volumes();
        foreach my $dedup_vol (@volumes) {
            dense_toggle(host      => ($self->filers())[0],
                         path_name => "/vol/$dedup_vol",
                         state     => "on");
            dense_start(host      => ($self->filers())[0],
                        path_name => "/vol/$dedup_vol");
        }
    } ## end if ($Options_hash->{SIS_ENABLE...

    # Create Vlans and assign ips to them
    if ($Options_hash->{MAX_VLAN_ENABLE} != 0) {
        Tharn::host("$filer");
        my $fc      = Tharn::connect("#telnet");
        my $vlan_id = $Options_hash->{VLAN_IDS};
        my $out;
        foreach my $num (0 .. (scalar(@vlan_addrs) - 1)) {
            eval {
                $out = vlan_create(fc      => $fc,
                                   gvrp    => 'on',
                                   if_name => $Options_hash->{VLAN_INTERFACE},
                                   vlanid  => $vlan_id);
            };
            if ($@) {
                if ($@ =~ m/"Exceeded the maximum allowable interfaces"/) {
                    $Log->comment("WARN", "Vlan create failed  -  $@");
                    next;
                }
            } else {

                do_ifconfig($fc, "$Options_hash->{VLAN_INTERFACE}\-$vlan_id",
                            ip_addr => $vlan_addrs[$num],
                            state   => 'up',
                            mtusize => '1500',
                            netmask => $Options_hash->{VLAN_NETMASK});

                # IO traffic should go through this IP
                # There are two data vlans (3948 and 3949) that our gdl system currently can use to communicate with clients
                if ($vlan_id == "3948") { $data_int = $vlan_addrs[$num] }
                $vlan_id++;
            } ## end else [ if ($@)
        } ## end foreach my $num (0 .. (scalar...
        $last_vlan_id = $vlan_id;
    } ## end if ($Options_hash->{MAX_VLAN_ENABLE...

    # Create the volume quotas
    if ($Options_hash->{QUOTA_VOLUME_ENABLE} != 0) {
        $self->configure_quotas('node' => $filer);
    }

    # Reset DATA_INTERFACES for max vlan case
    if ($data_int) {
        my @data_interfaces = @{$Options->get(option => "DATA_INTERFACES")};
        push(@data_interfaces, $data_int);
        $Options->set(DATA_INTERFACES => \@data_interfaces);
    }

    # Create required directories
    if (keys %{$self->{filer_setup}{dirs}}) {
        $Log->comment("Directories need to be created on volumes");

        my $Remote     = new RemoteExecution;
        my $filerip    = @{$Options_hash->{DATA_INTERFACES}}[0];
        my $client_idx = 0;
        my ($client, $mntpnt);
        my $num_subs = 0;
        foreach my $volume (keys %{$self->{filer_setup}{dirs}}) {
            my @directories = @{$self->{filer_setup}{dirs}{$volume}};

            # If volume already exists and exported with restricted permission
            # create_topology() didn't seem to change anything, hence we need
            # override here so dir creation can succeed. This shouldn't be a
            # problem, since we will reexport them anyways.
            export_path($conn, "/vol/$volume", OPTIONS => "rw,anon=0")
              or die config_result("export_path of /vol/$volume failed: $@");

            # Pick a number 500 directories per client as optimal value
            while (my @dirs = splice(@directories, 0, 500)) {
                if ($client_idx >= scalar(@clients)) {
                    $client_idx = 0;
                }
                $client = $clients[$client_idx];

                $mntpnt =
                  "/t/${filerip}_${volume}_${client}_${client_idx}_filer_setup";

                # create directories on the client
                my $params = {
                    DIRS => join(',', @dirs),
                    MOUNT_LIST => "${filerip};/vol/${volume};${mntpnt};rw;777",
                    };
                $Log->comment(
                    "Run tools/TestManager/filer_directory_setup.thpl on $volume from $client to create dirs: "
                      . join(",", @dirs));
                $Remote->execute(
                       host        => $client,
                       nate_params => $params,
                       cmd   => "tools/TestManager/filer_directory_setup.thpl",
                       runid => sprintf("%03d_%s_%s",
                                        $num_subs, "filer_setup_dirs", $client),
                       handler => sub { });
                $client_idx++;
                $num_subs++;
            } ## end while (my @dirs = splice(...
        } ## end foreach my $volume (keys %{...
        my $fail_result = $Remote->wait(timeout => 300);
        die config_result("dirs creation failed: $fail_result")
          if ($fail_result);
    } else {
        $Log->comment(
                "Directories weren't required to be created for this Topology");
    }

    # Setup exports appropriately
    # If one wants to export max exports allowed (10240), then we have to unexport
    # everything for max exports to succeed.
    if ((keys %{$self->{filer_setup}{exports}}) >= 10240) {
        unexport_all($conn);
    }

    foreach my $export (keys %{$self->{filer_setup}{exports}}) {
        my $options = $self->{filer_setup}{exports}{$export}{options};

        export_path($conn, "$export", OPTIONS => $options)
          or die config_result("export_path of $export failed: $@");
    }

    if ($Options_hash->{PERSISTENT_EXPORT}) {
        my $output;
        eval {
            $output = sayEOF($conn, "exportfs -w /etc/exports", timeout => 180);
        };
        die config_result("export -w: $output: $@") if ($@);
        $Log->comment("export -w: $output");
    }

    # Set this option, so clients using non-previlaged ports also will have access
    filer::filer_options_set($conn, "nfs.mount_rootonly", "off");

    $Log->exit();

} ## end sub filer_setup

sub filer_cleanup {
    my ($self, %options) = @_;

    # Stop the sktracing if it was enabled.
    if ($Options->get(option => "SKTRACE_OPTIONS")) {
        $self->sktrace_setup(disable => 1);
    }

    # Remove Vlans
    my $filer = ($self->filers())[0];
    Tharn::host("$filer");
    my $fc      = Tharn::connect("#rsh");
    my $vlan_id = $Options_hash->{VLAN_IDS};
    foreach my $num ($vlan_id .. ($last_vlan_id - 1)) {
        vlan_delete(fc      => $fc,
                    if_name => $Options_hash->{VLAN_INTERFACE},
                    vlanid  => '$num');
    }

    #Clean up quota
    if ($Options_hash->{QUOTA_VOLUME_ENABLE} != 0) {

        my $Zapi    = $apiset{'zapi'}{$filer};
        my @volumes = $self->volumes();
        my @clients = $self->clients();
        my $quota   = $Options_hash->{QUOTAS};
        foreach my $client (@clients) {
            foreach my $inst (keys %{$self->{test_options}{$client}}) {
                my $user   = $self->{test_options}{$client}{$inst}{TESTUSER};
                my $volume = $self->{test_options}{$client}{$inst}{VOLUME};
                $Log->comment("Checking quota status on $volume");
                my $Status;
                try {
                    $Response = $Zapi->quota_status('volume' => $volume);
                    $Status = $Response->get_parsed_output();
                }
                catch NACL::APISet::Exceptions::ResponseException with {
                    my $Exception = shift;
                    die config_result($Exception->text());
                };

                if (grep { $_ eq 'on' } @{$Status->[0]->{'status'}}) {

                    $Log->comment("Stopping quotas on $volume");

                    try {

                        $Response = $Zapi->quota_off('volume' => $volume);
                    }
                    catch NACL::APISet::Exceptions::ResponseException with {
                        my $Exception = shift;
                        die config_result($Exception->text());
                    };
                } ## end if (grep { $_ eq 'on' ...
                my @fields = split(/:/, $quota);
                $fields[0] = $user;
                $fields[1] =~ s/\@volume//g;
                try {
                    $Response =
                      $Zapi->quota_delete_entry('quota-target' => $fields[0],
                                                'quota-type'   => $fields[1],
                                                'volume'       => $volume,
                                                'qtree'        => '');
                }
                catch NACL::APISet::Exceptions::ResponseException with {
                    my $Exception = shift;
                    die config_result($Exception->text());
                };
            } ## end foreach my $inst (keys %{$self...
        } ## end foreach my $client (@clients)
    } ## end if ($Options_hash->{QUOTA_VOLUME_ENABLE...
    my $test_topology = $self->{test_topology};

    # create a list of all node resources in the test_topology
    my @resources;
    foreach my $resource_ref (values %{$test_topology->{cluster}}) {
        push(@resources, @{$resource_ref})
          if (ref($resource_ref) eq "ARRAY");
    }
    foreach my $node (keys %{$test_topology->{node}}) {
        foreach my $res_ref (values %{$test_topology->{node}{$node}}) {
            push(@resources, @{$res_ref}) if (ref($res_ref) eq "ARRAY");
        }
    }

    # check all of the resources for a comment that indicates that the resource
    # was previously created by TestManager. Mark this resource for cleanup
    foreach my $resource (@resources) {
        next if (ref($resource) ne "HASH");
        if ($resource->{comment} =~ "created by TestManagerLite") {
            $resource->{Delete_Resource} = 1;
        } elsif ($resource->{rulesetname}) {
            $resource->{Delete_Resource} = 1;
        }
    }

    $Server->delete_topology(print_node => undef, topology => $test_topology);

    # Execute the one from the parent
    $self->SUPER::filer_cleanup(%options);
} ## end sub filer_cleanup

sub client_setup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_setup(%options);

    $Log->exit();
}

sub DESTROY {
    my $self = shift;
    local ($@);

    $self->client_cleanup();
}

sub client_cleanup {
    my ($self, %options) = @_;
    $Log->enter();

    # Execute the one from the parent
    $self->SUPER::client_cleanup(%options);

    $Log->exit();
}

sub _update_test_topology {
    my $self = shift;

    $Log->enter();
    $Options_hash = $Options->get_hash();

    my $test_topology;

    # Find the minimum size required for an aggr using min vol size
    my $min_size = 300;
    foreach my $vol (keys %{$self->{filer_setup}{volume}}) {
        if ($min_size < $self->{filer_setup}{volume}{$vol}{size}) {
            $min_size = $self->{filer_setup}{volume}{$vol}{size};
        }
    }

    my @Aggrs = $Server->pick_aggregates(state      => "online",
                                         avail_size => "$min_size");
    $Log->debug("Auto Picked Aggrs: " . Dumper(\@Aggrs));

    my @Aggregates;
    if (@{$Options_hash->{aggregate}}) {

        # If aggrs are user-specified, then make sure they exist, otherwise fail
        foreach my $aggr (@{$Options_hash->{aggregate}}) {
            my $found = 0;
            foreach my $ag_hash (@Aggrs) {
                if ($ag_hash->{aggregate} eq $aggr) {
                    $found = 1;
                    push @Aggregates, $ag_hash;
                }
            }
            die config_result(
                    "Specified Aggr $aggr doesn't exist with size $min_size MB")
              if (!$found);
        } ## end foreach my $aggr (@{$Options_hash...
    } else {
        push @Aggregates, @Aggrs;
    }
    $Log->debug("Usable Aggr hash: " . Dumper(\@Aggregates));

    my $aggrs = $self->{discovered_topology}{cluster}{aggregate};

    my $orig_server;
    if (my @clustered = $Server->get_clustered_nodes()) {
        $orig_server = $Server->set_execution_server(-Server => $clustered[0]);
    }

    my $volume_number = 0;
    foreach my $volume (keys %{$self->{filer_setup}{volume}}) {
        my $vol_hash;
        $vol_hash->{volume} = $volume;
        my $vol_size = $self->{filer_setup}{volume}{$volume}{size};
        if ($vol_size < 300) {
            $Log->comment(
                "Vol size is adjusted from $vol_size to 300 to keep min size of 300MB per volume"
                );
            $vol_size = 300;
        }
        $vol_hash->{size} = $vol_size . "MB";

        # see if the volume already exists in the cluster
        my $existing_volume;
        eval { $existing_volume = $Server->volume_show(volume => $volume) };

        # make sure that we have a volume array and that the volume hash is a real one.
        # For 7G, a volume_hash gets generated and is not complete....
        if ($existing_volume && $existing_volume->[0]{volume}) {
            $existing_volume = $existing_volume->[0];

            # find info on the hosting aggregate
            my ($aggr_hash) =
              grep($_->{aggregate} eq $existing_volume->{aggregate},
                   @Aggregates);
            if (ref($aggr_hash) ne "HASH") {
                $Log->comment(Dumper $existing_volume);
                $Log->comment(Dumper \@Aggregates);
                die config_result(
                    "Could not find the aggregate hosting volume '$existing_volume->{volume}'"
                    );
            }
            foreach my $key (keys %{$existing_volume}) {

                #Don't cache volume params that are "-" or blank
                $vol_hash->{$key} = $existing_volume->{$key}
                  if ($existing_volume->{$key} !~ /^-$|^$/);
                $vol_hash->{-Filer} = $aggr_hash->{filers};
            }
            next;
        } ## end if ($existing_volume &&...

        # fill in missing aggregate information
        my @usable_aggrs;
        foreach my $agr (@Aggregates) {
            if ($agr->{availsize} >= $vol_size) {
                push(@usable_aggrs, $agr);
            }
        }

        my $aggr_info;
        map($aggr_info .= "$_->{aggregate} => $_->{availsize}\n",
            @usable_aggrs);
        $Log->trace($aggr_info);

        if (!scalar(@usable_aggrs)) {
            die config_result(
                           "No aggregates with sufficient space are available, "
                             . $vol_hash->{size}
                             . " needed.");
        }

        # pick aggregates in a round-robin fashion rather than random
        my $agr_hash = $usable_aggrs[$volume_number++ % (scalar @usable_aggrs)];
        $agr_hash->{availsize} -= $vol_size;
        $vol_hash->{aggregate} = $agr_hash->{aggregate}
          if (!$vol_hash->{aggregate});

        $Log->trace(
            "placed volume '$vol_hash->{volume}' on aggregate '$vol_hash->{aggregate}': its size is now $agr_hash->{availsize}"
            );

        # fill in vserver information - Should we make Server.pm happy?
        $vol_hash->{vserver} = 'N/A';

        # fill in missing security_style information. It defaults to "unix" unless specified.
        if (!$vol_hash->{'security-style'}) {
            $vol_hash->{'security-style'} = $$Options_hash{VOL_SECURITY_STYLE}
              || "unix";
        }

        # fill in user/group information
        if ($$Options_hash{VOLUME_OWNER} eq 'self') {
            $vol_hash->{user} = $ENV{LOGNAME};
            (undef(),            undef(), undef(),
             $vol_hash->{group}, undef(), undef(),
             undef(),            undef(), undef()) = getpwnam($ENV{LOGNAME});
        } else {
            $vol_hash->{user}  = 0 if (!$vol_hash->{user});
            $vol_hash->{group} = 0 if (!$vol_hash->{group});
        }

        # fill in junction-path information - Server.pm happy?
        $vol_hash->{'junction-path'} = "/$volume";
        $vol_hash->{ruleset} = "allow_all";

        # Classic needs to know which filer the aggr/volume are on
        $vol_hash->{-Filer} = $agr_hash->{filers};

        # Indicate that it was created by TestManager
        $vol_hash->{comment} = '"created by TestManagerLite"';

        push @{$test_topology->{cluster}{volume}}, $vol_hash;
    } ## end foreach my $volume (keys %{...

    push(@{$test_topology->{cluster}{exportruleset}},
         {vserver     => 'N/A',
          rulesetname => "allow_all",});
    push(@{$test_topology->{cluster}{exportruleset_rules}},
         {ruleindex   => 1,
          vserver     => 'N/A',
          rulesetname => "allow_all",
          anon        => 0,
          protocol    => "any",
          clientmatch => "0.0.0.0/0",
          rorule      => "any",
          rwrule      => "any",
          superuser   => "any",});

    $self->{test_topology} = $test_topology;

    $Log->debug("Test Topology: \n" . Dumper($test_topology));

    $Log->exit();
    return $test_topology;
} ## end sub _update_test_topology

######################### Internal Routines ##################

sub _create_range {
    my ($self, $ip) = @_;
    my ($begin, $range);
    if ($ip =~ /(.+)\/(\d+)/) {
        ($begin, $range) = ($1, $2);
    } elsif (!defined($ip)) {
        return ();
    } else {
        $begin = $ip;
        $range = 1;
    }
    $range--;

    my @range;
    ## This works for both IPv4 and IPv6
    my $ipobj = new Net::IP("$begin + $range")
      or die "Failed to create ip object: $@";

    do {
        push(@range, $ipobj->ip());
    } while (++$ipobj);

    return (wantarray) ? @range : $range[$#range];
} ## end sub _create_range

sub unexport_all {
    my ($conn) = @_;
    my $output = sayEOF($conn, "exportfs -ua", timeout => 180);

    $Log->comment("unexport all: $output");
    return $output;
}

sub export_all {
    my ($conn) = @_;
    my $output = sayEOF($conn, "exportfs -a", timeout => 180);

    $Log->comment("export all: $output");
    return $output;
}

my $File_read = undef;

sub configure_quotas {

    $Log->enter();

    my ($self, %options) = @_;

    my $node             = $options{'node'};
    my $Zapi_Api_Set_Obj = $apiset{'zapi'}{$node};
    $Log->comment("Configuring quotas for Node: $node");

    my $path = '/vol/vol0/etc/quotas';
    my $file_info;
    eval {
        $Response = $Zapi_Api_Set_Obj->file_get_file_info('path' => $path);
        $file_info = $Response->get_parsed_output();
    };

    # if the file does not exist create it
    if (!$file_info) {
        $self->write_file('ascii' => "\n",
                          'node'  => $node,
                          'path'  => $path);
    }

    #target:type:disk:files:thold:sdisk:sfile
    #NFS_USER:user@volume:-:-:700M:700M:5K
    my @volumes = $self->volumes();
    my @clients = $self->clients();
    my $quota   = $Options_hash->{QUOTAS};
    my $vol;

    foreach my $client (@clients) {
        foreach my $inst (keys %{$self->{test_options}{$client}}) {

            if ($quota !~ /\@volume/) { next; }
            my $user = $self->{test_options}{$client}{$inst}{TESTUSER};
            $vol = $self->{test_options}{$client}{$inst}{VOLUME};
            my @fields = split(/:/, $quota);
            $fields[0] = $user;
            $fields[1] =~ s/\@volume//g;
            my $Entry;
            eval {
                $Response =
                  $Zapi_Api_Set_Obj->quota_get_entry(
                                                   'quota-target' => $fields[0],
                                                   'quota-type'   => $fields[1],
                                                   'volume'       => $vol,
                                                   'qtree'        => '');
                $Entry = $Response->get_parsed_output();

            };
            if (!$Entry) {
                $Log->comment("Adding quota to $vol\n");
                try {

                    $Response =
                      $Zapi_Api_Set_Obj->quota_add_entry(
                                                'quota-target'    => $fields[0],
                                                'quota-type'      => $fields[1],
                                                'disk-limit'      => $fields[2],
                                                'file-limit'      => $fields[3],
                                                'threshold '      => $fields[4],
                                                'soft-disk-limit' => $fields[5],
                                                'soft-file-limit' => $fields[6],
                                                'volume'          => $vol,
                                                'qtree'           => '');
                } ## end try
                catch NACL::APISet::Exceptions::ResponseException with {
                    my $Exception = shift;
                    die config_result($Exception->text());
                };
            }    # foreach my $inst (keys %

            push @{$self->{filer_setup}{volume}{$vol}{quota}}, $quota;

            # Turn on the quota for vols
            try {

                $Response = $Zapi_Api_Set_Obj->quota_on('volume' => $vol,);
            }
            catch NACL::APISet::Exceptions::ResponseException with {
                my $Exception = shift;
                die config_result($Exception->text());
            };
            snooze 1;

        }    # foreach my $inst
    }    # foreach my $client
} ## end sub configure_quotas

sub write_file {

    $Log->enter();

    my ($self, %options) = @_;

    my $ascii     = $options{'ascii'};
    my $node      = $options{'node'};
    my $path      = $options{'path'};
    my $offset    = $options{'offset'} || 0;
    my $overwrite = $options{'overwrite'} || 'true';

    my $Zapi = $apiset{'zapi'}{$node};
    my $Cli  = $apiset{'cli'}{$node};

    # Set the vfiler context for the next ZAPI calls
    try {
        $Response = $Zapi->set_vfiler('vfiler-name' => 'vfiler0');
    }
    catch NACL::APISet::Exceptions::ResponseException with {
        my $Exception = shift;
        die config_result($Exception->text());
    };

    $Log->comment("Writing to file $path");
    try {

        $Response =
          $Zapi->file_write_file('data'      => unpack("H*", $ascii),
                                 'offset'    => $offset,
                                 'overwrite' => $overwrite,
                                 'path'      => $path);

    }
    catch NACL::APISet::Exceptions::ResponseException with {
        eval {
            $self->truncate_file('node'   => $node,
                                 'vfiler' => 'vfiler0',
                                 'path'   => $path);

            snooze 1;

            foreach my $line (split(/\n/, $ascii)) {
                $Cli->wrfile(append    => 1,
                             'content' => "$line",
                             'file'    => $path,);
            }
        };
        if ($@) { die config_result($@); }

    };

    $Log->exit();
} ## end sub write_file

sub _read_testuser_file {
    my $tfile = shift;
    my $file =
      NATE::Inc::find_file($tfile) ? NATE::Inc::find_file($tfile) : $tfile;
    $Log->enter("file=>$file");
    if ($File_read) {
        $Log->trace("file already processed");
        goto done;
    }
    my $testuser = $Options->get(option => "TESTUSER");

    if (open(IF, $file)) {
        while (<IF>) {
            next if (/^\s*\#/);
            chomp;
            push(@{$testuser}, $_);
        }
        close(IF);
    } else {
        die "Could not open TESTUSER_FILE: $tfile ($file) : $! / $@";
    }

    $File_read = 1;

  done:
    $Log->exit();
} ## end sub _read_testuser_file

sub sktrace_setup {

    # Check if ONTAP version supports sktraces or not

    my $ver = get_wafl_version(host($filers[0]));
    if ($ver < WV_BOILERMAKER_MIN) {
        $Log->comment("ONTAP version on the filer doesn't support sktraces ");
    }

    else {

        my ($self, %params) =
          validate_mixed(@_,
                         [{type => OBJECT}],
                         {enable => {type     => UNDEF | SCALAR,
                                     optional => 1,
                                     default  => 1},
                          disable => {type     => UNDEF | SCALAR,
                                      optional => 1,
                                      default  => 0},});

        my %skHash;
        my $enable = $params{enable};
        my @ops = @{$Options->get(option => "SKTRACE_OPTIONS")};
        $Log->comment("SKTRACE_OPTIONS: " . Dumper(\@ops));

        # set this on all nodes in use
        foreach my $node (@filers) {
            my $hostobj = host($node);
            my $bsd = NACL::APISet->new(category  => 'Node',
                                        interface => 'CLI',
                                        set       => 'Systemshell',
                                        hostobj   => $hostobj);

            # the CDEFS for sysctl are less than awesome. Use the
            # execute_raw_command instead. Default to enable
            if ((not $enable) or ($params{disable})) {
                $Log->comment("Cleaning up the sktrace options on " . $node);

                # This is cleanup, start with a dump to finish everything off
                $bsd->execute_raw_command(
                                command => "sudo sysctl sysvar.sktrace.dump=1");

                foreach my $op (@ops) {

                    # reset all the given values to zero
                    if ($op =~ /(.+)=\d+/) {
                        $op = $1;
                        $op = $op . '=0';
                    }
                    $bsd->execute_raw_command(command => "sudo sysctl " . $op);
                }

            } else {
                $Log->comment("Setting up the sktrace options on " . $node);

                # We are enabling. Start with a clear
                $bsd->execute_raw_command(
                               command => "sudo sysctl sysvar.sktrace.clear=1");

                # Check if other modules are being traced by default. If found, set them to zero
                my $out =
                  $bsd->execute_raw_command(command =>
                     "sudo sysctl sysvar.sktrace | grep enable | grep -v ': 0' "
                     );
                my @out = split(/\n/, $out);
                foreach my $module (@out) {
                    $module =~ s/:\s\d+/=0/g;
                    $bsd->execute_raw_command(command => "sudo sysctl $module");
                }
                foreach my $op (@ops) {
                    $bsd->execute_raw_command(command => "sudo sysctl " . $op);
                }
            } ## end else [ if ((not $enable) or (...
        } ## end foreach my $node (@filers)
    } ## end else [ if ($ver < WV_BOILERMAKER_MIN)
    $Log->exit();
} ## end sub sktrace_setup

1;
